/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include <wx/wx.h>
#include <wx/menu.h>
#include <wx/file.h>
#include <wx/mstream.h>
#include <wx/dir.h>
#include <wx/textfile.h>
#include <wx/tokenzr.h>

#include "wdfmain-musiceditor.hpp"

#include "drw/musicedt.hpp"
#include "smr/smr.hpp"
#include "smr/dlg/settings.hpp"

IMPLEMENT_APP(SMREditorApp)

BEGIN_EVENT_TABLE(MyScoreEditor, wxMEScoreFrame)
	EVT_MENU(5000,MyScoreEditor::OnNew)
	EVT_MENU(5001,MyScoreEditor::OnLoad)
	EVT_MENU(5002,MyScoreEditor::OnSave)
	EVT_MENU(5003,MyScoreEditor::OnSaveAs)
	EVT_MENU(5004,MyScoreEditor::OnSettings)
	//
	EVT_MENU(5005, MyScoreEditor::OnEncodeSMXF)
	EVT_MENU(5006, MyScoreEditor::OnDecodeBITS)
	EVT_MENU(5007, MyScoreEditor::OnSaveAsMP4)
	EVT_MENU(5008, MyScoreEditor::OnLoadFromMP4)
	EVT_MENU(5009, MyScoreEditor::OnAddSMFL)
	EVT_MENU(5010, MyScoreEditor::OnAddFont)
	EVT_MENU(5011, MyScoreEditor::OnAddMIDI)
	EVT_MENU(5012, MyScoreEditor::OnBatchSaveAsMP4)
	//
	EVT_MENU_RANGE(6000,6199,MyScoreEditor::OnView)
END_EVENT_TABLE()


MyScoreEditor::MyScoreEditor(wxWindow* parent,const wxString& tit,const wxPoint& pos, const wxSize& siz)
	: wxMEScoreFrame(parent, tit, pos, siz)
{
	wxMenu *fileMenu=GetfileMenu();

	if(SMRApp::GetApp().GetTopWindow()==this)
	{
		fileMenu->Insert(0,new wxMenuItem(fileMenu, 5000, "New..."));
		fileMenu->Insert(1,new wxMenuItem(fileMenu, 5001, "Load..."));
		fileMenu->Insert(2,new wxMenuItem(fileMenu, 5008, "Load from MP4..."));
		fileMenu->Insert(3,new wxMenuItem(fileMenu, 5002, "Save"));
		fileMenu->Insert(4,new wxMenuItem(fileMenu, 5003, "Save As..."));
		fileMenu->Insert(5,new wxMenuItem(fileMenu, 5007, "Save As MP4..."));
		fileMenu->Insert(6,new wxMenuItem(fileMenu, 5005, "Save As SMR Bitstream..."));
		fileMenu->Insert(7,new wxMenuItem(fileMenu, 5012, "Batch Save As MP4..."));
		fileMenu->Insert(8,new wxMenuItem(fileMenu)); // SEPARATOR
		fileMenu->Insert(9,new wxMenuItem(fileMenu, 5009, "Add SMFL file..."));
		fileMenu->Insert(10,new wxMenuItem(fileMenu, 5010, "Add Font file..."));
		fileMenu->Insert(11,new wxMenuItem(fileMenu, 5011, "Add MIDI file..."));
		fileMenu->Insert(12,new wxMenuItem(fileMenu)); // SEPARATOR
		fileMenu->Insert(13,new wxMenuItem(fileMenu, 5004, "Settings..."));
		fileMenu->Insert(14,new wxMenuItem(fileMenu));

		GetMenuBar()->Insert(1,CreatePartsMenu(),"Parts");
	}
}

void MyScoreEditor::OnNew(wxCommandEvent&)
{
	if(scorecanvas->GetNScore()!=0 || !scorecanvas->GetSimbImg())
		return;
	scorecanvas->Prepare();
	wxClientDC scoreDC(scorecanvas);
	Scr->SetMainDC(&scoreDC);
	
	MusicEditor->New();
	
	wxMenu *oldMenu=GetMenuBar()->Replace(1,CreatePartsMenu(),"Parts");
	delete oldMenu;
}

void MyScoreEditor::OnLoad(wxCommandEvent&)
{
	if(scorecanvas->GetNScore()!=0 || !scorecanvas->GetSimbImg())
		return;

	wxString smrDir=smrgetenv("SMR_FILES_PATH");
	if (smrDir.Last()=='\\' || smrDir.Last()=='/') smrDir.RemoveLast();

	wxDirDialog dialog(this, "Load SMRObject", smrDir);
	if (dialog.ShowModal()==wxID_OK)
	{
		MusicEditor->Load(dialog.GetPath());

		wxMenu *oldMenu=GetMenuBar()->Replace(1,CreatePartsMenu(),"Parts");
		delete oldMenu;
	}

	wxString partName = "MainScore";
	SetTitle(MusicEditor->smrf->GetTitle()+" - SMR Music Editor - ["+partName+"]");
}

void MyScoreEditor::OnLoadFromMP4(wxCommandEvent&)
{
	if(scorecanvas->GetNScore()!=0 || !scorecanvas->GetSimbImg())
		return;
	wxClientDC scoreDC(scorecanvas);
	Scr->SetMainDC(&scoreDC);

	wxFileDialog dialog(NULL, "Load from MP4", "", "", "*.mp4", wxOPEN);
	if (dialog.ShowModal()==wxID_OK)
	{
		if(MusicEditor->LoadMP4(dialog.GetPath()))
		{
			wxMenu *oldMenu=GetMenuBar()->Replace(1,CreatePartsMenu(),"Parts");
			delete oldMenu;
		}
	}

	wxString partName = "MainScore";
	SetTitle(MusicEditor->smrf->GetTitle()+" - SMR Music Editor - ["+partName+"]");
}

void MyScoreEditor::OnSave(wxCommandEvent&)
{
	MusicEditor->smrf->Save();
	wxString partName = "MainScore";
	SetTitle(MusicEditor->smrf->GetTitle()+" - SMR Music Editor - ["+partName+"]");
}

void MyScoreEditor::OnSaveAs(wxCommandEvent&)
{
    wxTextEntryDialog dialog(NULL,"Insert SMR Object Name:","Save As","",
							 wxOK |  wxCANCEL |  wxCENTRE, wxDefaultPosition);
	if (dialog.ShowModal()==wxID_OK)
	{
		wxString name=dialog.GetValue();
		MusicEditor->smrf->SaveAs(name);
	}
	wxString partName = "MainScore";
	SetTitle(MusicEditor->smrf->GetTitle()+" - SMR Music Editor - ["+partName+"]");
}

void MyScoreEditor::OnSaveAsMP4(wxCommandEvent&)
{
	SMRFile *pWF = MusicEditor->smrf;
	wxString title = pWF->GetTitle();
	if (title == "")
	{
		wxMessageBox("Open a SMR Object first!","Warning!",wxOK | wxCENTRE | wxICON_EXCLAMATION) ; 
		return;
	}

	wxFileDialog dialog(NULL, "Save As MP4", "", title+".mp4", "*.mp4", wxSAVE);
	if (dialog.ShowModal()==wxID_OK)
	{
		wxString srcDirName = dialog.GetPath();
		wxString codings[]={ "plain XML", "GZIP", "BiM"};
		Coding_Type ct;

		int pos=wxGetSingleChoiceIndex("Coding type", "Select coding type", 3, codings);
		switch(pos)
		{
		case 0:
			ct = XMLF;
			break;
		case 1:
			ct = GZIP;
			break;
		case 2:
			ct = BIMF;
			break;
		}
		
		if(pos!=-1)
			pWF->SaveToMP4(dialog.GetPath(), ct);

		//if (err != 0)
		//	wxMessageBox("Save to MP4 Fatal Error.","Error!",wxOK | wxCENTRE | wxICON_EXCLAMATION); 
	}		
}

void MyScoreEditor::OnBatchSaveAsMP4(wxCommandEvent&)
{
	wxFileDialog dialog(NULL, "Load batch", "", "", "*.smr-batch", wxOPEN);
	if (dialog.ShowModal()==wxID_OK)
	{
		wxString batchFilename = dialog.GetPath();
		wxTextFile batch;
		batch.Open(batchFilename);
		if(batch.IsOpened())
		{
			wxString line;
			wxString path=batchFilename.BeforeLast('\\');

			int curLine=0;
			int lastRepeat=-1;
			wxString lastRepeatVar;
			wxArrayString lastRepeatValues;
			int lastRepeatValue;

			while ( curLine < batch.GetLineCount() )
			{
				line=batch[curLine];
				wxArrayString l;
				wxStringTokenizer tkz(line);
				while ( tkz.HasMoreTokens() )
				{
					l.Add(tkz.GetNextToken());
				}
				if(l[0].StartsWith("#"))
				{
				}
				else if(l[0]=="REPEAT" && l.Count()>=3 && lastRepeat==-1)
				{
					lastRepeatVar = l[1];
					lastRepeat=curLine;
					lastRepeatValue=0;
					lastRepeatValues.Clear();
					for(int i=2; i<l.Count(); i++)
						lastRepeatValues.Add(l[i]);
				}
				else if(l[0]=="ENDREPEAT" && lastRepeat>=0)
				{
					lastRepeatValue++;
					if(lastRepeatValue<lastRepeatValues.Count())
					{
						curLine=lastRepeat;
					}
					else
						lastRepeat=-1;
				}
				else if(l[0]=="LOAD" && l.Count()>=2)
				{
					wxString file=l[1];
					if(lastRepeat>=0)
						file.Replace("$("+lastRepeatVar+")",lastRepeatValues[lastRepeatValue]);
					MusicEditor->Load(path+"\\"+file);
					wxMenu *oldMenu=GetMenuBar()->Replace(1,CreatePartsMenu(),"Parts");
					delete oldMenu;
					SetTitle(MusicEditor->smrf->GetTitle()+" - SMR Music Editor - [Main Score]");
					wxYield();
				}
				else if(l[0]=="SAVEMP4" && l.Count()>=3)
				{
					wxString file=l[1];
					if(lastRepeat>=0)
						file.Replace("$("+lastRepeatVar+")",lastRepeatValues[lastRepeatValue]);

					SMRFile *pWF = MusicEditor->smrf;
					Coding_Type ct=XMLF;
					if(l[2]=="XML")
						ct = XMLF;
					else if(l[2]=="GZIP")
						ct = GZIP;
					else if(l[2]=="BIM")
						ct = BIMF;
					
					pWF->SaveToMP4(path+"\\"+file, ct);
				}
				curLine++;
			}
			/*for ( line = batch.GetFirstLine(); !batch.Eof(); line = batch.GetNextLine() )
			{
				wxArrayString l;
				wxStringTokenizer tkz(line);
				while ( tkz.HasMoreTokens() )
				{
					l.Add(tkz.GetNextToken());
				}
				if(l[0].StartsWith("#"))
					continue;
				if(l[0]=="LOAD" && l.Count()>=2)
				{
					MusicEditor->Load(path+"\\"+l[1]);
					wxMenu *oldMenu=GetMenuBar()->Replace(1,CreatePartsMenu(),"Parts");
					delete oldMenu;
				}
				else if(l[0]=="SAVEMP4" && l.Count()>=3)
				{
					SMRFile *pWF = MusicEditor->smrf;
					Coding_Type ct=XMLF;
					if(l[2]=="XML")
						ct = XMLF;
					else if(l[2]=="GZIP")
						ct = GZIP;
					else if(l[2]=="BIM")
						ct = BIMF;
					
					pWF->SaveToMP4(path+"\\"+l[1], ct);
				}
			} */
		}
	}

}

void MyScoreEditor::OnSettings(wxCommandEvent&)
{
	SMREnvSettingsDlg dlg(SMRApp::GetApp().settings);
	dlg.ShowDlg();
}

//
void MyScoreEditor::OnEncodeSMXF(wxCommandEvent&)
{	
	SMRFile *pWF = MusicEditor->smrf;
	wxString title = pWF->GetTitle();
	if (title == "")
	{
		wxMessageBox("Open a WEDELOBJECT first!","Warning!",wxOK | wxCENTRE | wxICON_EXCLAMATION) ; 
		return;
	}

	wxString smrDir=smrgetenv("SMR_FILES_PATH");
	wxFileDialog dialog(NULL, "Encode SMXF Object", smrDir, title+".smr", "*.smr", wxSAVE);
	if (dialog.ShowModal()==wxID_OK)
	{
		wxString codings[]={ "plain XML", "GZIP", "BiM" };
		Coding_Type ct;

		int pos=wxGetSingleChoiceIndex("Coding type", "Select coding type", 3, codings);
		switch(pos)
		{
		case 0:
			ct = XMLF;
			break;
		case 1:
			ct = GZIP;
			break;
		case 2:
			ct = BIMF;
			break;
		}
		
		if(pos!=-1)
		{
			int err = pWF->EncodeSMXF(dialog.GetPath(), ct);
			if (err != 0)
				wxMessageBox("SMXF Encoder Fatal Error.","Error!",wxOK | wxCENTRE | wxICON_EXCLAMATION); 
		}
	}		
}

void MyScoreEditor::OnDecodeBITS(wxCommandEvent&)
{
/*
	SMR smr;

	while(1)
	{
		wxFileDialog dialog(NULL, "Load SMXF Part", "", "", "*.*", wxOPEN);
		if (dialog.ShowModal()==wxID_OK)
		{

			smr.LoadSMXFPart(dialog.GetPath());
		}
		else break;
	}
	wxFileDialog dialog(NULL, "Load SMXF Main", "", "", "*.*", wxOPEN);
	if (dialog.ShowModal()==wxID_OK)
	{
		smr.LoadSMXFMain(dialog.GetPath());
	}
	else return;



	wxString msg;

	msg= wxString::Format("Measures: %d\n", smr.getMeasureCount());

	for(int i=1 ; i<=smr.getPartCount(); i++)
	{
		msg += wxString::Format("Part %d: %s\n", i, smr.getPartName(i));
	}

	wxMessageBox(msg);

	smr.SaveMP4("c:\\smr.mp4");
*/
/*
	SMRFile *pWF = MusicEditor->smrf;
	
	wxString smrDir=smrgetenv("SMR_FILES_PATH");
	wxFileDialog dialog(NULL, "Decode Bit Stream", smrDir, "", "*.smr", wxOPEN);
	if (dialog.ShowModal()==wxID_OK)
	{
		wxString srcDirName = dialog.GetPath();
		int err = pWF->DecodeBITS(dialog.GetPath());
		if (err != 0)
			wxMessageBox("SMXF Encoder Fatal Error.","Error!",wxOK | wxCENTRE | wxICON_EXCLAMATION); 
*/
/*
		wxFile smr;
		smr.Open(dialog.GetPath());
		int length=smr.Length();
		char * buffer=new char[length];
		smr.Read(buffer,length);
		smr.Close();

		wxMemoryInputStream *mem=new wxMemoryInputStream(buffer,length);
		pWF->DecodeBITS(mem,"test.wdf");
	}		
*/
}

//

wxMenu* MyScoreEditor::CreatePartsMenu()
{
	wxMenu *viewMenu=new wxMenu();
	SMRFile *smrf=MusicEditor->smrf;

	int nSym=smrf->CountItems(SMR_Symbolic);
	for(int i=0; i< nSym; i++)
	{
		viewMenu->Append(6000+i,wxString(i==0? "Symbolic: " : "       ")+smrf->GetSymItem(i)->GetItemDescription());
	}
	if(nSym==0)
	{
		int nSpa=smrf->GetMainScore()->GetNumSpa();
		viewMenu->Append(6000,"Symbolic: Main Score");
		for(int i=0; i< nSpa; i++)
		{
			wxString instr=smrf->GetMainScore()->GetSpartNum(i)->GetStrumEsec();
			viewMenu->Append(6001+i,wxString::Format("       part %d - ",i+1)+instr);
		}
	}

	return viewMenu;
}

void MyScoreEditor::OnView(wxCommandEvent& event)
{
	if(scorecanvas->GetNScore()!=0 || !scorecanvas->GetSimbImg())
		return;
	scorecanvas->Prepare();

	wxClientDC scoreDC(scorecanvas);
	Scr->SetMainDC(&scoreDC);
	
	int nScore=event.GetId()-6000;
	
	MusicEditor->smrf->OpenSymbolicScore(nScore);
	
}

void MyScoreEditor::OnAddSMFL(wxCommandEvent&)
{
	wxString smrDir=smrgetenv("SMR_FILES_PATH");
	wxFileDialog dialog(NULL, "Load SMFL", smrDir, "", "*.*", wxOPEN);
	if (dialog.ShowModal()==wxID_OK)
	{
		MusicEditor->smrf->AddSMFL(dialog.GetPath());
	}		
}

void MyScoreEditor::OnAddFont(wxCommandEvent&)
{
	wxString smrDir=smrgetenv("SMR_FILES_PATH");
	wxFileDialog dialog(this, "Load Font", smrDir, "", "*.*", wxOPEN);
	if (dialog.ShowModal()==wxID_OK)
	{
		MusicEditor->smrf->AddFont(dialog.GetPath());
	}		
}

void MyScoreEditor::OnAddMIDI(wxCommandEvent&)
{
	wxString smrDir=smrgetenv("SMR_FILES_PATH");
	wxFileDialog dialog(this, "Load MIDI", smrDir, "", "*.mid", wxOPEN);
	if (dialog.ShowModal()==wxID_OK)
	{
		scorecanvas->Prepare();
		if(MusicEditor->smrf->AddMIDI(dialog.GetPath()))
			scorecanvas->Refresh();
	}		
}

int WelcomeOpen(int duration, wxWindow* parent);

void SMREditorApp::OpenTopWindow(const char* fileToLoad)
{
	smrf.New("");
	smrf.OpenSymbolicScore(0);

	if (fileToLoad && wxDir::Exists(fileToLoad))
	{
		//MusicEditor->Load(fileToLoad);

		//wxString partName = "MainScore";
		//SetTitle(MusicEditor->smrf->GetTitle()+" - SMR Music Editor - ["+partName+"]");
	}

	WelcomeOpen(5, GetTopWindow());
}

