/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include <wx/wx.h>
#include <wx/menu.h>
#include <wx/textdlg.h>
#include <wx/dir.h>
#include <wx/textfile.h>
#include <wx/listctrl.h>
#include <wx/splitter.h>

#include "wdfmain-musicviewer.hpp"

#include "liooext.hpp"
#include "drw/scorecan.hpp"

#include "smr/smr.hpp"
#include "smr/workspac.hpp"

extern wxDC *currDC;
extern wxDC *currMainDC;
extern wxMEScoreCanvas *ScoreInExecution;

IMPLEMENT_APP(SMRViewerApp)

#define MUSICVIEWER_NEW			1000
#define MUSICVIEWER_LOAD		1001
#define MUSICVIEWER_LOADMP4		1002
#define MUSICVIEWER_LOADPLAYLIST		1006
#define MUSICVIEWER_IMPORTMIDI	1003
#define MUSICVIEWER_TRANSPOSE	1004
#define MUSICVIEWER_EXIT		1005

#define MUSICVIEWER_PLAYMIDI	1010
#define MUSICVIEWER_PAUSE		1011
#define MUSICVIEWER_STOP		1012

#define MUSICVIEWER_TOP			1100
#define MUSICVIEWER_FORWARD		1101
#define MUSICVIEWER_FORWARD5	1102
#define MUSICVIEWER_BACKWARD	1103
#define MUSICVIEWER_BACKWARD5	1104
#define MUSICVIEWER_BOTTOM		1105
#define MUSICVIEWER_JUMP		1106
#define MUSICVIEWER_PLAYLIST	1107
#define MUSICVIEWER_SCROLL		1108
#define MUSICVIEWER_SELECTION	1109

#define MUSICVIEWER_VIEW_SYM	2000
#define MUSICVIEWER_VIEW_IMG	2100

class SMRMusicViewer : public wxFrame
{
public:
	SMRMusicViewer(wxWindow* parent,const wxString& tit,const wxPoint& pos, const wxSize& siz);
    ~SMRMusicViewer();

	bool Load(const wxString& file, bool symbolicOrImage=true, int nScore=0);
	bool LoadMP4(const wxString& mp4file);
	bool LoadPlayList(const wxString& playListFile);
	bool ImportMIDI(const wxString& file);

	int PrepareMusicViewer(bool SimbImg, int NScore);
	
    void OnSize(wxSizeEvent& event);
	void OnIdle(wxIdleEvent& event);
	void OnClose(wxCloseEvent& event)
	{
		Destroy();
	}

	void OnNewViewer(wxCommandEvent& event)
	{
		int x=-1,y=-1,w=500,h=300;
		wxClientDC dc(scoreCanvas);
		scoreCanvas->Prepare(&dc);

		SMRMusicViewer *viewer=new SMRMusicViewer(NULL,"MPEG SMR Music Viewer",wxPoint(x,y), wxSize(w,h));
		viewer->Show();
	}

	void OnLoadMP4(wxCommandEvent& event)
	{
		wxClientDC dc(scoreCanvas);
		scoreCanvas->Prepare(&dc);

		wxString cwd=wxGetCwd();
		wxString filepath=wxFileSelector("Open MP4 File","","","","mp4 files (*.mp4)|*.mp4");
		wxSetWorkingDirectory(cwd);
		if (filepath!="")
		{
			LoadMP4(filepath);
		}
	}

	void OnLoadPlayList(wxCommandEvent& event)
	{
		wxString cwd=wxGetCwd();
		wxString filepath=wxFileSelector("Open Playlist File","","","","SMR playlist files (*.smrpl)|*.smrpl");
		wxSetWorkingDirectory(cwd);
		if (filepath!="")
		{
			LoadPlayList(filepath);
		}
	}

	void OnLoad(wxCommandEvent& event)
	{
		wxClientDC dc(scoreCanvas);
		scoreCanvas->Prepare(&dc);
		wxString smrDir=smrgetenv("SMR_FILES_PATH");
		if (smrDir.Last()=='\\' || smrDir.Last()=='/') smrDir.RemoveLast();

		wxDirDialog dialog(this,"Load SMRObject",smrDir);
		if (dialog.ShowModal()==wxID_OK)
		{
			Load(dialog.GetPath());
		}
	}

	void OnImportMIDI(wxCommandEvent& event)
	{
		wxString cwd=wxGetCwd();
		wxString filepath=wxFileSelector("Open Source File","","","","Midi files (*.mid)|*.mid");
		wxSetWorkingDirectory(cwd);
		if (filepath!="")
		{
			ImportMIDI(filepath);
		}
	}

	void OnTransposeClef(wxCommandEvent& event)
	{
		wxString clefs[7];
		clefs[0]="Treble";
		clefs[1]="Bass";
		clefs[2]="Baritone";
		clefs[3]="Tenor";
		clefs[4]="Alto";
		clefs[5]="Mezzo Soprano";
		clefs[6]="Soprano";

		int nParts=smrf.CountItems(SMR_Symbolic)-1;
		int part = wxGetNumberFromUser( "", "part to be transposed (0=main score):", "Part", 0, 0, nParts , this);
		int choice=wxGetSingleChoiceIndex("transposition clef:","Transposition",7,clefs, this);

		if(choice>=0)
			scoreCanvas->Transpose(1,0,part,choice,-1,-1,0,0,0,0);
	}

	void OnExit(wxCommandEvent& event)
	{
		exit(1);
	}

	void OnPlayMIDI(wxCommandEvent& event) 
	{
		wxClientDC dc(scoreCanvas);
		scoreCanvas->Prepare(&dc);
		if(scoreCanvas->InitMIDIExecution())
			scoreCanvas->Start(); 
	};
	void OnPause(wxCommandEvent& event) { scoreCanvas->Pause(); };
	void OnStop(wxCommandEvent& event) { scoreCanvas->Stop(); };

	void OnTop(wxCommandEvent& event) { scoreCanvas->GoTop(); };
	void OnBottom(wxCommandEvent& event) { scoreCanvas->GoBottom(); };
	void OnForward(wxCommandEvent& event) { scoreCanvas->GoForward(); };
	void OnBackward(wxCommandEvent& event) { scoreCanvas->GoBackward(); };
	void OnForward5(wxCommandEvent& event) { scoreCanvas->GoForward5(); };
	void OnBackward5(wxCommandEvent& event) { scoreCanvas->GoBackward5(); };
	void OnScroll(wxCommandEvent& event)
	{
		int measure = hscroll->GetThumbPosition()+1;
		scoreCanvas->Jump(measure);
	}

	void OnJump(wxCommandEvent& event) 
	{
		int nMeasure = wxGetNumberFromUser( "", "measure:", "Jump", 1, 1, scoreCanvas->GetNMeasures() , this);
		if(nMeasure>=1)
			scoreCanvas->Jump(nMeasure);
	}

	void OnSelection(wxCommandEvent& event) 
	{
		wxClientDC dc(scoreCanvas);
		scoreCanvas->Prepare(&dc);
		wxArrayString ids=scoreCanvas->GetSelectionIds();

		if(ids.Count() > 0)
		{
			wxString* sids=new wxString[ids.Count()];
			for(int i=0; i<ids.Count(); i++)
			{
				SMRAnnotation ann=scoreCanvas->GetSelectionAnnotation(ids[i]);
				if(ann.shortText.Count()>0)
					sids[i]=ann.shortText[0];
				else
					sids[i]=ids[i];
			}
			
			int sel=wxGetSingleChoiceIndex("Selection:","Selections", ids.Count(), sids, this);
			if(sel>=0)
			{
				if(activeSelectionId!="")
					scoreCanvas->Select(activeSelectionId, false);
				scoreCanvas->Select(ids[sel]);
				activeSelectionId=ids[sel];
			}
			delete[] sids;
		}
		else
			wxMessageBox("No selections are present");
	};

	void OnPlayListDClick(wxListEvent& event)
	{
		wxClientDC dc(scoreCanvas);
		scoreCanvas->Prepare(&dc);
		wxString file = playList[event.GetIndex()];
		if(file.AfterLast('.')=="mp4")
			LoadMP4(file);
		else
			Load(file);
	}

	void OnView(wxCommandEvent& event)
	{
		wxClientDC dc(scoreCanvas);
		scoreCanvas->Prepare(&dc);

		bool symImg=true;
		int nScore=event.GetId()-MUSICVIEWER_VIEW_SYM;
		if(event.GetId()>=MUSICVIEWER_VIEW_IMG)
		{
			symImg=false;
			nScore=event.GetId()-MUSICVIEWER_VIEW_IMG;
		}
		PrepareMusicViewer(symImg,nScore);
	}

	DECLARE_EVENT_TABLE()
protected:
	SMRFile smrf;
    wxMEScoreCanvas *scoreCanvas;
	bool resized;
	wxArrayString playList;
	wxListCtrl *playListCtrl;
	wxSplitterWindow* splitWin;
	wxScrollBar* hscroll;
	wxString activeSelectionId;
};

BEGIN_EVENT_TABLE(SMRMusicViewer, wxFrame)
    EVT_SIZE(SMRMusicViewer::OnSize)
	EVT_IDLE(SMRMusicViewer::OnIdle)
	EVT_CLOSE(SMRMusicViewer::OnClose)

	EVT_MENU(MUSICVIEWER_NEW,SMRMusicViewer::OnNewViewer)
	EVT_MENU(MUSICVIEWER_LOAD,SMRMusicViewer::OnLoad)
	EVT_MENU(MUSICVIEWER_LOADMP4,SMRMusicViewer::OnLoadMP4)
	EVT_MENU(MUSICVIEWER_LOADPLAYLIST,SMRMusicViewer::OnLoadPlayList)
	EVT_MENU(MUSICVIEWER_IMPORTMIDI,SMRMusicViewer::OnImportMIDI)
	EVT_MENU(MUSICVIEWER_TRANSPOSE,SMRMusicViewer::OnTransposeClef)
	EVT_MENU(MUSICVIEWER_EXIT,SMRMusicViewer::OnExit)

	EVT_MENU(MUSICVIEWER_PLAYMIDI,SMRMusicViewer::OnPlayMIDI)
	EVT_MENU(MUSICVIEWER_PAUSE,SMRMusicViewer::OnPause)
	EVT_MENU(MUSICVIEWER_STOP,SMRMusicViewer::OnStop)

	EVT_MENU(MUSICVIEWER_TOP,SMRMusicViewer::OnTop)
	EVT_MENU(MUSICVIEWER_FORWARD,SMRMusicViewer::OnForward)
	EVT_MENU(MUSICVIEWER_FORWARD5,SMRMusicViewer::OnForward5)
	EVT_MENU(MUSICVIEWER_BACKWARD,SMRMusicViewer::OnBackward)
	EVT_MENU(MUSICVIEWER_BACKWARD5,SMRMusicViewer::OnBackward5)
	EVT_MENU(MUSICVIEWER_BOTTOM,SMRMusicViewer::OnBottom)
	EVT_MENU(MUSICVIEWER_JUMP,SMRMusicViewer::OnJump)
	EVT_MENU(MUSICVIEWER_SELECTION,SMRMusicViewer::OnSelection)

	EVT_MENU_RANGE(MUSICVIEWER_VIEW_SYM,MUSICVIEWER_VIEW_IMG+99,SMRMusicViewer::OnView)

	EVT_COMMAND_SCROLL(MUSICVIEWER_SCROLL, SMRMusicViewer::OnScroll)  
	EVT_LIST_ITEM_ACTIVATED(MUSICVIEWER_PLAYLIST, SMRMusicViewer::OnPlayListDClick)
END_EVENT_TABLE()

SMRMusicViewer::SMRMusicViewer(wxWindow *parent,const wxString& tit,const wxPoint& pos, const wxSize& siz) 
   :wxFrame(parent,-1, tit,pos,siz,wxDEFAULT_FRAME_STYLE) 
{
	int w,h;
	GetClientSize(&w, &h);
	SetIcon(wxICON(wdfmed));

	splitWin = new wxSplitterWindow(this,-1);
	wxPanel* scorePanel = new wxPanel(splitWin, -1);
	scoreCanvas = new wxMEScoreCanvas(scorePanel, wxPoint(0,0), wxSize(w, h));
	scoreCanvas->Prepare();
	resized = false;
	smrf.New("");
	hscroll = new wxScrollBar(scorePanel, MUSICVIEWER_SCROLL);

	wxBoxSizer* sizer = new wxBoxSizer(wxVERTICAL);
	sizer->Add(scoreCanvas,1,wxGROW);
	sizer->Add(hscroll,0,wxGROW);

	scorePanel->SetSizer(sizer);
	scorePanel->SetAutoLayout(TRUE);

	//playListCtrl = new wxListBox(splitWin,MUSICVIEWER_PLAYLIST, wxDefaultPosition, wxDefaultSize, 0, NULL, wxLB_SINGLE);
	playListCtrl = new wxListCtrl(splitWin,MUSICVIEWER_PLAYLIST, wxDefaultPosition, wxDefaultSize, wxLC_REPORT);
	playListCtrl->InsertColumn(0, "File");
	playListCtrl->InsertColumn(1, "Description");
	splitWin->SplitHorizontally(scorePanel,playListCtrl, h-100);

	wxMenuBar* menubar = new wxMenuBar();

	wxMenu* fileMenu=new wxMenu();
	fileMenu->Append(MUSICVIEWER_NEW, "&New Viewer...");
	fileMenu->Append(MUSICVIEWER_LOAD, "&Load...");
	fileMenu->Append(MUSICVIEWER_LOADMP4, "&Load MP4...");
	fileMenu->Append(MUSICVIEWER_LOADPLAYLIST, "&Load Playlist...");
	fileMenu->AppendSeparator();
	fileMenu->Append(MUSICVIEWER_IMPORTMIDI, "&Import MIDI...");
	fileMenu->Append(MUSICVIEWER_TRANSPOSE, "&Key change...");
	fileMenu->AppendSeparator();
	fileMenu->Append(MUSICVIEWER_SELECTION, "&View selections...\tCtrl-S");
	fileMenu->AppendSeparator();
	fileMenu->Append(MUSICVIEWER_EXIT, "&Exit");
	menubar->Append(fileMenu,"&File");

	wxMenu* viewMenu=new wxMenu();
	menubar->Append(viewMenu, "&View");

	wxMenu* moveMenu=new wxMenu();
	moveMenu->Append(MUSICVIEWER_TOP, "&Top\tCtrl-T");
	moveMenu->Append(MUSICVIEWER_FORWARD, "&Forward 1 measure\tCtrl-F");
	moveMenu->Append(MUSICVIEWER_FORWARD5, "&Forward 5 measures\tCtrl-Shift-F");
	moveMenu->Append(MUSICVIEWER_BACKWARD, "&Backward 1 measure\tCtrl-B");
	moveMenu->Append(MUSICVIEWER_BACKWARD5, "&Backward 5 measures\tCtrl-Shift-B");
	moveMenu->Append(MUSICVIEWER_BOTTOM, "Botto&m\tCtrl-M");
	moveMenu->AppendSeparator();
	moveMenu->Append(MUSICVIEWER_JUMP, "&Jump\tCtrl-J");
	menubar->Append(moveMenu,"&Move");

	wxMenu* playMenu=new wxMenu();
	playMenu->Append(MUSICVIEWER_PLAYMIDI, "&Play MIDI\tCtrl-P");
	playMenu->Append(MUSICVIEWER_PAUSE, "Pa&use\tCtrl-U");
	playMenu->Append(MUSICVIEWER_STOP, "&Stop\tCtrl-S");
	menubar->Append(playMenu, "&Play");

	SetMenuBar(menubar);
}


SMRMusicViewer::~SMRMusicViewer()
{
	delete scoreCanvas; 
}

void SMRMusicViewer::OnSize(wxSizeEvent& event)
{
	int w,h;
	GetClientSize(&w, &h);
/*
	if (scoreCanvas)
	{
		scoreCanvas->SetSize(0, 0, w, h); 
	}
*/
	splitWin->SetSize(0,0,w,h);
	resized=true;
}

void SMRMusicViewer::OnIdle(wxIdleEvent& event)
{ 
	if(resized && scoreCanvas->GetMusicWin()!=NULL && 
		!scoreCanvas->GetMusicWin()->GetModEsecuzione() && !scoreCanvas->GetSimbImg())
	{
		wxClientDC maindc(scoreCanvas);
		wxDC *prevDC=currDC;
		wxDC *prevMainDC=currMainDC;
		LiooWindow* prevMusicWin=pMusicWin;
		
		Scr->SetMainDC(&maindc);
		scoreCanvas->Prepare(); 
		Scr->GSetMainDrawArea();
		
		resized=false;
		currDC=prevDC;
		currMainDC=prevMainDC;
		pMusicWin=prevMusicWin;
	} 
}

int SMRMusicViewer::PrepareMusicViewer(bool symImg, int nScore)
{
	scoreCanvas->SetMusicScore(&smrf, symImg, nScore);

	wxString partName = "Sym: ";
	if(smrf.GetSymItem(nScore)!=NULL)
		partName += smrf.GetSymItem(nScore)->GetItemDescription();

	SetTitle(smrf.GetTitle()+" - SMR Music Viewer - ["+partName+"]");

	return 1;
}

bool SMRMusicViewer::Load(const wxString& file, bool symbolicOrImage, int nScore)
{
	if(!smrf.Load(file))
		return false;

	PrepareMusicViewer(symbolicOrImage,nScore);

	//builds the view menu with the symbolic/imagescore components
	wxMenu *viewMenu=new wxMenu();
	int nSym=smrf.CountItems(SMR_Symbolic);
	for(int i=0; i< nSym; i++)
	{
		viewMenu->Append(MUSICVIEWER_VIEW_SYM+i,wxString(i==0? "Symbolic: " : "       ")+smrf.GetSymItem(i)->GetItemDescription());
	}

	wxMenu *oldViewMenu=GetMenuBar()->Replace(1, viewMenu, "View");
	delete oldViewMenu;
	hscroll->SetScrollbar(0,1,smrf.GetMeasureCount(), 2);

	return true;
}

bool SMRMusicViewer::LoadMP4(const wxString& file)
{
	if(smrf.LoadFromMP4(file)!=0)
		return false;

	PrepareMusicViewer(true,0);

	//builds the view menu with the symbolic/imagescore components
	wxMenu *viewMenu=new wxMenu();
	int nSym=smrf.CountItems(SMR_Symbolic);
	for(int i=0; i< nSym; i++)
	{
		viewMenu->Append(MUSICVIEWER_VIEW_SYM+i,wxString(i==0? "Symbolic: " : "       ")+smrf.GetSymItem(i)->GetItemDescription());
	}

	wxMenu *oldViewMenu=GetMenuBar()->Replace(1, viewMenu, "View");
	delete oldViewMenu;

	hscroll->SetScrollbar(0,1,smrf.GetMeasureCount(), 2);

	return true;
}

bool SMRMusicViewer::LoadPlayList(const wxString& file)
{
	wxTextFile playlist;

	playlist.Open(file);
	if(!playlist.IsOpened())
		return false;
	wxString path = file.BeforeLast('\\')+"\\";

	playList.Clear();
	playListCtrl->DeleteAllItems();
	for(int l=0; l<playlist.GetLineCount(); l++)
	{
		wxString file = playlist[l].BeforeFirst('|');
		wxString descr = playlist[l].AfterFirst('|');
		file=file.Trim().Trim(FALSE);
		if(!wxDirExists(file) && !wxFileExists(file))
		{
			file=path+file;
			if(!wxDirExists(file) && !wxFileExists(file))
				continue;
		}
		playList.Add(file);
		long index=playListCtrl->InsertItem(playList.Count()-1, file);
		playListCtrl->SetItem(index,1, descr);
	}

	return true;
}

bool SMRMusicViewer::ImportMIDI(const wxString& file)
{
	PrepareMusicViewer(true,0);

	scoreCanvas->ImportMIDI(file);

	//builds the view menu with the symbolic/imagescore components
	wxMenu *viewMenu=new wxMenu();
	int nSym=smrf.CountItems(SMR_Symbolic);
	for(int i=0; i< nSym; i++)
	{
		viewMenu->Append(MUSICVIEWER_VIEW_SYM+i,wxString(i==0? "Symbolic: " : "       ")+smrf.GetSymItem(i)->GetItemDescription());
	}

	wxMenu *oldViewMenu=GetMenuBar()->Replace(1, viewMenu, "View");
	delete oldViewMenu;

	return true;
}

int WelcomeOpen(int duration, wxWindow* parent);

void SMRViewerApp::OpenTopWindow(const char* fileToLoad)
{
	int x=-1,y=-1,w=500,h=300;
	workspace->GetWindowConfig("MusicEditor", x, y, w, h);

	SMRMusicViewer *viewer=new SMRMusicViewer(NULL,"MPEG SMR Music Viewer",wxPoint(x,y), wxSize(w,h));

	if (fileToLoad && wxDir::Exists(fileToLoad)) 	{
		viewer->Load(fileToLoad);
	}

	viewer->Show();

	WelcomeOpen(2, viewer); //two seconds
}