/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include <xercesc/util/PlatformUtils.hpp>

#include "wx/wx.h"
#include "wx/mstream.h"

#include "smr/smrmain.hpp"
#include "liooext.hpp"
#include "smr/smr.hpp"
//#include "wdf/lyrwf.hpp"
#include "par/partitur.hpp"
#include "smr/dlg/settings.hpp"
#include "gui/masaewin.hpp"
#include "drw/musicedt.hpp"
#include "drw/scorefrm.hpp"
#include "wx/image.h"

#include "SMRDecoderLib.h"
#include "SMRcursor.h"

class SMRDecoderApp : public SMRApp
{
public:
	SMRDecoderApp()
	{
		argv=new char*[1];
		argv[0]="e:\\a_dummy.exe";
		OnInit();
	}

	bool IsInteractive() { return false; }
	
	void OpenTopWindow(const char* fileToLoad=NULL) {};
};

class SMRDecoderGScreen : public GScreen
{
public:
	SMRDecoderGScreen(int ww, int hh) { w=ww; h=hh;}
	int GetMaxPixY() { return h; }
	int GetMaxPixX() { return w; }
	void SetMaxPix(int ww, int hh) { w=ww; h=hh;}

protected:
	int w,h;
};

extern wxMemoryDC *currMDC;

static SMRDecoderApp *app=NULL;
static char* commands[] = { "ADD_TEXT_ANNOTATION", "ADD_LABEL", "ADD_NOTE", "ADD_REST", "ADD_SYMBOL", "ADD_MEASURE", 
							"CHANGE_CLEF", "CHANGE_KEYSIGNATURE", "CHANGE_TIME", "SET_ALTERATION", "SET_METRONOME",
							"SET_DOTS", "DEL_MEASURE", "DELETE", "TRANSPOSE", 0};
static char* views[] = { "CWMN", "CWMN_PRINT", 0};

// ERROR CODES
#define OUT_OF_BOUNDS	-1
#define PART_NOT_SET	-2
#define MEASURE_NOT_SET	-3
#define VOICE_NOT_SET	-4
#define EVENT_NOT_SET	-5
#define WRONG_COMMAND	-6

// EVENT TYPES
enum {
		UNKNOWN_EVT = 0,
		NOTE_EVT,
		REST_EVT,
		CHORD_EVT,
		REFRAIN_EVT,
		KEY_EVT,
		CLEF_EVT,
		
		//		LABEL2_EVT,		//necessario : le label sono di 2 tipi : lettere e riferimenti
		
}eventType;


SMRInternalDecoder::SMRInternalDecoder(char* decoderInfo, int decoderInfoSize, char* buffer, int size, int nscore, int battuta,int width,int height)
{
	if(app==NULL)
	{
		app=new SMRDecoderApp();
		app->Initialize();
	}

	lyrics = NULL;
	smrf=new SMRFile;

	scoreWidth=width;
	scoreHeight=height;
	Scr=new SMRDecoderGScreen(width,height);
	Scr->SetMouse(new Mouse);

	wxInputStream* accessUnitStream=new wxMemoryInputStream(buffer, size);
	wxInputStream* decoderInfoStream=new wxMemoryInputStream(decoderInfo, decoderInfoSize);
	
	if(smrf->DecodeBITS(decoderInfoStream, accessUnitStream, "smr-decoded.xsm")!=0)
	{
		delete accessUnitStream;
		delete decoderInfoStream;
	}

	smrf->Load("smr-decoded.xsm");
	partitura = smrf->GetMainScore();

	partitura->SetCurrentScore(nscore);

	
	pixmap = new LPixmap(width,height);	//
	Scr->GSetPixmap((LPixmap*)pixmap);			//
	
	if(nscore==0)
		pMusicWin=new MasaeWindow;
	else
		pMusicWin=new LiooWindow;
	pMusicWin->WinInit(Point(0,0),width,height,partitura);	//

	if(nscore==0)
	{
		tipoLeggio=LEGGIO_MASAE;
		Leg=false;
	}
	else
	{
		tipoLeggio=LEGGIO_DLIOO;
		Leg=true;
	}
	SimbImg=1;
	AutoLineBreak=1;
	DebugNormalMillaPos=0;
	currentViewType=0;
	Scr->SetFont(0);	
	
	if(partitura==NULL)
		return;

	if(battuta==0)
		partitura->GoTop();
	else
		partitura->GoBatt(battuta);

	LoadSynch("smr-decoded.xsm");

	extractLyrics();

	updated = true;

	smrCursor = new SMRcursor(partitura);
	hitLevel = 0;

//	oldColor = new Colore();
	highlighted = false;
	onebarshown = false;
}

SMRInternalDecoder::SMRInternalDecoder(char* path, int nscore, int battuta,int width,int height)
{
	if(app==NULL)
	{
		app=new SMRDecoderApp();
		app->Initialize();
	}

	smrf=new SMRFile;
	
	smrf->Load(path);

	partitura = smrf->GetMainScore();
	partitura->SetCurrentScore(nscore);

	scoreWidth=width;
	scoreHeight=height;
	Scr=new SMRDecoderGScreen(width,height);
	Scr->SetMouse(new Mouse);
	
	pixmap = new LPixmap(width,height);	//
	Scr->GSetPixmap((LPixmap*)pixmap);			//
	
	if(nscore==0)
		pMusicWin=new MasaeWindow;
	else
		pMusicWin=new LiooWindow;
	pMusicWin->WinInit(Point(0,0),width,height,partitura);	//

	if(nscore==0)
	{
		tipoLeggio=LEGGIO_MASAE;
		Leg=false;
	}
	else
	{
		tipoLeggio=LEGGIO_DLIOO;
		Leg=true;
	}
	SimbImg=1;
	AutoLineBreak=1;
	DebugNormalMillaPos=0;
	currentViewType=0;
	Scr->SetFont(0);	
	
	if(partitura==NULL)
		return;

	if(battuta==0)
		partitura->GoTop();
	else
		partitura->GoBatt(battuta);

	LoadSynch(path);
	updated = true;

	smrCursor = new SMRcursor(partitura);
	hitLevel = 0;

//	oldColor = new Colore();
	highlighted = false;
	onebarshown = false;

	lyrics = NULL;
}
void SetMusicEditorSMRF(SMRFile*);
SMRInternalDecoder::~SMRInternalDecoder()
{
	delete smrf;
	SetMusicEditorSMRF(NULL);
	//delete partitura;

	delete (LPixmap*)pixmap;
	delete smrCursor;

	//-- delete lyrics --//
	if(lyrics){
		char** ptr = lyrics;
		while(*ptr) {
			free(*ptr);
			ptr++;
		}
		free(lyrics);
		lyrics = NULL;
	}
}

SMRInternalDecoder::SMRInternalDecoder(int width,int height, int nscore, char* title)	//Called by IMED
{
	if(app==NULL)
	{
		app=new SMRDecoderApp();
		app->Initialize();
	}

	smrf			=			NULL;
	partitura		=			NULL;

	SetMusicEditorSMRF(NULL);

	scoreWidth=width;
	scoreHeight=height;

	Scr			=			new SMRDecoderGScreen(width,height);

	Scr->SetMouse(new Mouse);
	
	pixmap		=			new LPixmap(width,height);

	Scr->GSetPixmap((LPixmap*)pixmap);			//
	

	if(title)
	{
		smrf=new SMRFile(); 
		pMusicWin = NULL;
		smrf->New("");

		partitura			= smrf->GetMainScore();

//		updated				=	true;

		if(partitura!=NULL)
		{
			partitura->SetCurrentScore(nscore);
			partitura->GoTop();
		}
	}

	if(nscore==0)
		pMusicWin=new MasaeWindow;
	else
		pMusicWin=new LiooWindow;

	pMusicWin->PPartit	=	partitura;
	pMusicWin->WinInit(Point(0,0),width,height,partitura);	//

	if(nscore==0)
	{
		tipoLeggio=			LEGGIO_MASAE;
		Leg		=			false;
	}
	else
	{
		tipoLeggio=			LEGGIO_DLIOO;
		Leg		=			true;
	}

	SimbImg		=			1;
	AutoLineBreak=			1;
	DebugNormalMillaPos=	0;
	currentViewType	=		0;

	Scr->SetFont(0);	

	//	SMR MODEL ACCESS for Score follow
	smrCursor = new SMRcursor(partitura);
	hitLevel = 0;

//	oldColor = new Colore();
	highlighted = false;
	onebarshown = false;

	updated = true; //false;

	lyrics = NULL;
}

int SMRInternalDecoder::NewSMR(char* title, int nscore)
{
	delete smrf;
	//delete partitura;
	partitura = NULL;

	SetMusicEditorSMRF(NULL);

	smrf	=	new SMRFile();
	pMusicWin = NULL;
	smrf->New(title);

	if(nscore==0)
		pMusicWin=new MasaeWindow;
	else
		pMusicWin=new LiooWindow;

	partitura			= smrf->GetMainScore();

	updated				=	true;
	
	if(partitura==NULL)
		return 0;

	partitura->SetCurrentScore(nscore);

	pMusicWin->PPartit	=	partitura;	
	pMusicWin->WinInit(Point(0,0),0,0,partitura);	//reset ATTENZIONE width=height=0 forza il resize.

	//	SMR MODEL ACCESS for Score follow
	smrCursor->SetScore(partitura);

	//partitura->GoTop();

	//LoadSynch(path);

	onebarshown			= !onebarshown;
	ShowOneBar(!onebarshown);

	return 1;
}


int SMRInternalDecoder::LoadSMR(char* path, int nscore)
{
	SMRFile *pF = new SMRFile;

	if(!pF->Load(path))
	{
		delete pF;
		return 0;
	}

	delete smrf;
	
	//partitura = NULL;

	SetMusicEditorSMRF(NULL);

	smrf					=	pF;
	
	if(pixmap==NULL)
		return 0;

	Scr->GSetPixmap((LPixmap*)pixmap);

	if(nscore==0)
		pMusicWin=new MasaeWindow;
	else
		pMusicWin=new LiooWindow;

	partitura			= smrf->GetMainScore();

	updated				=	true;

	if(partitura==NULL)
		return 0;
	
	partitura->SetCurrentScore(nscore);
	
	pMusicWin->PPartit	=	partitura;	
	pMusicWin->WinInit(Point(0,0),0,0,partitura);	//reset ATTENZIONE width=height=0 forza il resize.
	
	partitura->GoTop();

	extractLyrics();

	//	SMR MODEL ACCESS for Score follow
	smrCursor->SetScore(partitura);

	LoadSynch(path);

	onebarshown			= !onebarshown;
	ShowOneBar(!onebarshown);

	return 1;
}

const char* SMRInternalDecoder::GetSMRDir()
{
	return SMRApp::GetApp().settings->GetFilesPath().c_str();
}

int SMRInternalDecoder::SaveSMR()
{
	if (partitura == NULL)
		return 0;

	//HilightOff();
	return smrf->Save();
}

int SMRInternalDecoder::SaveAsSMR(char* title)
{
	if (partitura == NULL)
		return 0;

	//HilightOff();
	return smrf->SaveAs(wxString(title));
}

void SMRInternalDecoder::Decode(void* videobuffer,int width,int height)
{
	delete ((LPixmap*)pixmap);
	
	((SMRDecoderGScreen*)Scr)->SetMaxPix(width, height);						
	
	pixmap = new LPixmap(width,height);
	Scr->GSetPixmap((LPixmap*)pixmap);
	
	pMusicWin->Resize(VUnit(Scr->GetMaxPixX()),VUnit(Scr->GetMaxPixY()));

	if (partitura)
	{	
		partitura->Redraw();

		//clean some dust... on the left corner
		Scr->SetColor(Colore(BIANCO)); //white;
		Scr->Rect(Rectangle(Point(0,0),Point(4,height)),TRUE);
	}

	wxBitmap* bitmap=(wxBitmap*)((LPixmap*)pixmap)->GetPixmap();
	wxImage img(*bitmap);
	void *rawdata=img.GetData();
	if (rawdata)
		memcpy(videobuffer, rawdata, width*height*3);
}

void SMRInternalDecoder::JumpToMeasure(int measure)
{
	if(partitura!=NULL)
	{
		if(measure<=GetMeasureCount())
		{
			partitura->GoBatt(measure);
			updated=true;
		}
	}
}

void SMRInternalDecoder::JumpToLabel(char* labelName)
{
	if(partitura!=NULL)
	{
		NumCode measureID=partitura->GetBattEtichetta(labelName[0]);
		if(measureID!=0)
		{
			partitura->GoBattIfOut(measureID);
			updated=true;
		}
	}
}

int SMRInternalDecoder::GetMeasureCount()
{
	if(partitura!=NULL)
		return partitura->GetNumeroBattute();
	return 0;
}

char ** SMRInternalDecoder::GetAvailableCommands()
{
	return commands;
}

char** SMRInternalDecoder::GetAvailableLyrics()
{
	return lyrics;
}

void SMRInternalDecoder::extractLyrics()
{
	// delete old values
	if(lyrics){
		char** ptr = lyrics;
		while(*ptr) {
			free(*ptr);
			ptr++;
		}
		free(lyrics);
		lyrics = NULL;
	}

	//------------------

	if(partitura==NULL)
		return;

	SMRItemList LyrList;
	
	smrf->GetItems(SMR_Lyrics,LyrList);
	
	int nLyr=LyrList.GetCount();
	if(nLyr==0)
		return;
	
	int numParts = GetPartCount();

	lyrics				=	(char**) malloc ( (1 + numParts) * sizeof(char*) );
	lyrics[numParts]	=	NULL;

    SMRLyricsItem *lwf;
	
	for(int part=0 ; part<numParts ; part++)
	{
		int n=0;
		wxString lyr_acr;
		wxSMRItemListNode *node = LyrList.GetFirst();
		while (node)
		{
			lwf= (SMRLyricsItem*)node->GetData();
			wxString lyric;
			
			if( lwf->GetPartID() == partitura->GetSpartNum(part)->GetNumericCode())
			{
				lyric		=	lwf->GetItemDescription();
				lyr_acr		+=	lyric.Mid(0,3) + " ";
				n++;
			}
			
			node = node->GetNext();
		}
		lyr_acr.RemoveLast();
		if (n==0)
			lyr_acr = "\"" + lyr_acr + "\"";

		char* partlyr = (char*) malloc ( (1 + lyr_acr.Len()) * sizeof(char) );
		partlyr[lyr_acr.Len()]	=	NULL;

		strcpy(partlyr, lyr_acr.c_str());
		lyrics[part] = partlyr;
	}
}

char ** SMRInternalDecoder::GetAvailableViewTypes()
{
	return views;
}

int SMRInternalDecoder::GetPartCount()
{
	if(partitura!=NULL)
		return partitura->GetNumSpa();
	return 0;
}

char* SMRInternalDecoder::GetPartName(int part)
{
	if(partitura!=NULL)
	{
		if(part>=0 && part<GetPartCount())
		{
			const char* partName=partitura->GetSpartNum(part)->GetNome();
			if(partName[0]!=0)
				return (char*)partName;
			else
			{
				static char tmpPartName[100];
				sprintf(tmpPartName,"Part %d", part+1);
				return tmpPartName;
			}
		}
	}
	return NULL;
}

char** SMRInternalDecoder::GetPartLyrics()
{
	if(partitura==NULL)
		return NULL;

	int numParts = GetPartCount();
	char** strList  =	(char**) malloc ( numParts * sizeof(char*) );
	strList[numParts]	=	NULL;

	for(int part=0 ; part<numParts ; part++)
	{
		SMRLyricsItem *lwf = partitura->GetSpartNum(part)->GetCurrentLyrWF();
		wxString lang;
		if (lwf != NULL)
		{
			lang = lwf->GetItemDescription();
			lang = lang.Mid(0,3);
		}
		else
			lang = "\"\"";

		strList[part] = (char*) malloc (4 * sizeof(char));
		strList[part][4] = NULL;

		strcpy(strList[part],lang.c_str());
	}
	return strList;
}

void SMRInternalDecoder::SetPartLyrics(char** lyr)
{
	if(partitura==NULL || lyr == NULL)
		return;
	
	SMRItemList LyrList;	
	smrf->GetItems(SMR_Lyrics,LyrList);
	wxString text,lyrFile;
	SMRLyricsItem *lwf;


	int part = 0;
	while(lyr[part] != NULL)
	{
		text = lyr[part];
		partitura->GetSpartNum(part)->RemoveLyr();
		wxSMRItemListNode *node = LyrList.GetFirst();
		while (node)
		{
			lwf= (SMRLyricsItem*)node->GetData();			
			if(lwf->GetPartID() == partitura->GetSpartNum(part)->GetNumericCode() && text==lwf->GetItemDescription())
			{
				lyrFile = smrf->GetPath()+PATH_SEP+lwf->GetFilename();
				partitura->GetSpartNum(part)->LoadLyr((char *)lyrFile.c_str());
			}
			node = node->GetNext();
		}
		part++;
	}
	updated=true;
}


int SMRInternalDecoder::GetLabelCount()
{
	if(partitura!=NULL)
	{
		return partitura->GetNumEtichette();
	}
	return 0;
}

char* SMRInternalDecoder::GetLabelName(int i)
{
	if(partitura!=NULL && i>=0 && i<partitura->GetNumEtichette() )
	{
		static char etichetta[2];
		etichetta[0]=partitura->GetEtichetta(i);
		etichetta[1]=0;
		return etichetta;
	}
	return "";
}

int SMRInternalDecoder::GetFirstVisibleMeasure()
{
	if(partitura!=NULL)
	{
		int first,last;
		partitura->GetPageMeasures(first,last);
		return first;
	}
	return 0;
}

int SMRInternalDecoder::GetLastVisibleMeasure()
{
	if(partitura!=NULL)
	{
		int first,last;
		partitura->GetPageMeasures(first,last);
		return last;
	}
	return 0;
}

/*===============================================================================*/

void SMRInternalDecoder::JumpToTime(int time)
{
	bool end;
	int sinfo=TimeToSynchPos(time, end);
	if(sinfo<0)
		return;
	int measure=synchInfo[sinfo].measure;
	if(measure>=1)
	{
		JumpToMeasure(measure);
		updated = true;
	}
}

void SMRInternalDecoder::JumpToTime(int time, int& xx, int& yy, bool& reachedEnd)
{
	int sinfo=TimeToSynchPos(time, reachedEnd);
	if(sinfo<0)
	{
		xx=0;
		return;
	}
	int measure=synchInfo[sinfo].measure;
	
	if(measure >=1 && (measure<GetFirstVisibleMeasure() || measure >GetLastVisibleMeasure()))
	{
		JumpToMeasure(measure);
	}

	int measureDurate = synchInfo[sinfo].durate;
	Battuta* pMeasure=NULL;

	if(!Leg)
	{
		pMeasure=partitura->GetFirstSpa()->GetBattNumProg(measure);
	}
	else
	{
		int nScore=partitura->GetCurrentScore();
		pMeasure=((Spartito*)partitura->GetListaSpa()->AtPos(nScore-1))->GetBattNumProg(measure);
	}
	if(pMeasure)
	{
		int measureX=pMeasure->GetAbsPos().x;
		int measureWidth=pMeasure->GetWidth();

		int startTime = synchInfo[sinfo].startTime;

		xx = measureX+(int)((measureWidth*(time-startTime)/measureDurate));
		if(xx > measureX + measureWidth)
			xx = measureX + measureWidth;

		yy = scoreHeight - (pMeasure->GetAbsPos().y+pMeasure->GetVU2Up());
	}
}

void SMRInternalDecoder::GetDefaultHighlightPos(int& xx, int& yy)
{
	int measure=GetFirstVisibleMeasure();
	if(measure<=0)
	{
		xx = yy = 0;
		return;
	}
	Battuta* pMeasure=NULL;

	if(!Leg)
	{
		pMeasure=partitura->GetFirstSpa()->GetBattNumProg(measure);
	}
	else
	{
		int nScore=partitura->GetCurrentScore();
		pMeasure=((Spartito*)partitura->GetListaSpa()->AtPos(nScore-1))->GetBattNumProg(measure);
	}
	if(pMeasure)
	{
		xx = pMeasure->GetAbsPos().x;
		yy = scoreHeight - (pMeasure->GetAbsPos().y+pMeasure->GetVU2Up());
	}
}

char* SMRInternalDecoder::Link(int x, int y)
{
	char* stringa;
	if(partitura!=NULL)
	{
		DrawObject* drwobj=NULL;
		SymPath path;

		Point mousePos(x, y);
		if(partitura->Hit(mousePos,FIGURA_SEL, drwobj, path))
		{
			Figura* fig=(Figura*)drwobj;
			wxString link=fig->Get_Link();
			if(link=="")
				link="No Link";

			int lunghezza = link.length();
			stringa = new char[lunghezza+1];

			strcpy(stringa, link);
	
			return stringa;
		}
		return 0;
	}
	return 0;
}

/*===============================================================================*/

void SMRInternalDecoder::ShowOneBar(bool value)
{
	if (onebarshown != value)
	{
		onebarshown = value;

		if(partitura==NULL)
			return;
		
		Spartito	*currSpa;
		Battuta		*currBat;
		//ListaSpartiti *listaSpa = partitura->GetListaSpa();
		currSpa = partitura->GetFirstSpa();
		while (currSpa != NULL)
		{
			currBat = currSpa->GetBattNum(0);
			while (currBat != NULL)
			{
				currBat->SetMainScoreLineBreak(0, value);
				currBat->SetPartLineBreak(0, value);

				currBat = currSpa->GetNextBat(currBat);
			}

			currSpa = partitura->GetNextSpa(currSpa);
		}
		
		updated = true;
	}

}

void SMRInternalDecoder::EnableAutoMagnification(bool status)
{
	Scr->EnableAutoMagnification(status);
	updated = true;
}

float SMRInternalDecoder::GetMagnification()
{
	if(partitura)
		return partitura->GetComputerViewParams()->GetMagnification();

	return 1.0;
}

void SMRInternalDecoder::SetMagnification(float value)
{
	if(partitura)
	{
		partitura->GetComputerViewParams()->SetMagnification(value);
		updated = true;
	}
}

void SMRInternalDecoder::ShowParts(int part[], int n, bool useMulti)
{
	if (partitura == NULL)
		return;
/*	if (n == 0)
		n = GetPartCount();
	{	
		partitura->ShowAllParts();
		return;
	}*/

	if(n==1 && !useMulti)
	{
		if(!Leg)
		{
			//delete pMusicWin;
			pMusicWin=new LiooWindow;
			Leg=true;
			tipoLeggio=	LEGGIO_DLIOO;
		}
		smrf->GetScores(part[0],partitura);
		partitura->SetCurrentScore(part[0]);
		pMusicWin->SetInView(1);
		DrawObject::SetCurrentView(1);
		pMusicWin->WinInit(Point(0,0),VUnit(Scr->GetMaxPixX()),VUnit(Scr->GetMaxPixY()),partitura);
		((LoMas*)partitura)->SetNPentagrammi(4);
		pMusicWin->GoTop();
	}
	else
	{
		if(Leg)
		{
			//delete pMusicWin;
			pMusicWin=new MasaeWindow;
			Leg=false;
			tipoLeggio=	LEGGIO_MASAE;
		}
		DrawObject::SetCurrentView(0);
		smrf->GetScores(0,partitura);
		partitura->SetCurrentScore(0);
		pMusicWin->WinInit(Point(0,0),scoreWidth,scoreHeight,partitura);
		
		bool match;
		int i,count = 0; 
		Spartito *currSpa, *firstVis, *lastVis;
		ListaSpartiti *listaSpa = partitura->GetListaSpa();
		currSpa = partitura->GetFirstSpa();
		while (currSpa != NULL)
		{
			count++;
			if(n==0)
				match=true;
			else
			{
				match = false;
				for (i = 0; i < n; i++)
				{	
					if (count == part[i] || n == GetPartCount())
						match = true;
				}
			}
			
			if (match && !currSpa->IsVisible())
			{
				currSpa->SetVisible(true);
				currSpa->SetTipo(SPART_GENERICO);
				//smrf->ShowSymPart(listaSpa->GetPos(currSpa)+1);
				
			}
			else if (!match && currSpa->IsVisible())
			{
				currSpa->SetVisible(false);
				currSpa->SetTipo(SPART_GENERICO);
				//smrf->HideSymPart(listaSpa->GetPos(currSpa)+1);
			}
			currSpa=partitura->GetNextSpa(currSpa);
		}
		
		firstVis=listaSpa->GetFirstVisible();
		lastVis=listaSpa->GetLastVisible();
		if(firstVis && lastVis)
		{
			if(firstVis==lastVis)
				firstVis->SetTipo(SPART_UNICO);
			else
			{
				firstVis->SetTipo(SPART_SUPERIORE);
				lastVis->SetTipo(SPART_INFERIORE);
			}
		}
		
		partitura->Redraw();
	}
	updated = true;

}

void SMRInternalDecoder::SetViewType(char* viewType)
{
}


bool SMRInternalDecoder::SetCommand(char* command, int argCount, const char** arguments)
{
	if(partitura==NULL)
		return false;

	wxString cmd(command); 

	int layer=1;

	if (cmd == "ADD_TEXT_ANNOTATION" && argCount >= 1)
	{
		pMusicWin->SetCommand(CMD_PARTITURA_INSERTTEXT,IMMEDIATE_SEL,ANY_SEL,0,0,0,0,0,(char*)arguments[0]);
	}
	else if (cmd == "ADD_LABEL" && argCount >= 1)
	{
		wxString lab_str(arguments[0]);

		if (lab_str == "SEGNO")
			pMusicWin->SetCommand(CMD_BATTUTA_SEGNO,IMMEDIATE_SEL,BATTUTA_SEL);
		else if (lab_str == "CODA")
			pMusicWin->SetCommand(CMD_BATTUTA_CODA,IMMEDIATE_SEL,BATTUTA_SEL);
		else
			pMusicWin->SetCommand(CMD_BATTUTA_TSALTO,IMMEDIATE_SEL,BATTUTA_SEL,0,0,0,0,0,(char*)lab_str.c_str());
	}
	else if (cmd == "ADD_NOTE" && argCount >= 3)
	{
		layer = atoi(arguments[0]);

		wxString dur_str(arguments[1]);
		CmdID dur_cmd;

		if (dur_str == "D1")
			dur_cmd = CMD_BATTUTA_INS_NSEMIBREVE;
		else if (dur_str == "D1_2")
			dur_cmd = CMD_BATTUTA_INS_NMINIMA;
		else if (dur_str == "D1_4")
			dur_cmd = CMD_BATTUTA_INS_NSEMIMINIMA;
		else if (dur_str == "D1_8")
			dur_cmd = CMD_BATTUTA_INS_NCROMA;
		else if (dur_str == "D1_16")
			dur_cmd = CMD_BATTUTA_INS_NSEMICROMA;
		else if (dur_str == "D1_32")
			dur_cmd = CMD_BATTUTA_INS_NBISCROMA;
		else if (dur_str == "D1_64")
			dur_cmd = CMD_BATTUTA_INS_NSEMIBISCROMA;
		else return false;

		wxString head_str(arguments[2]);
		NoteHead_ID head;

		if (head_str == "CLASSIC")
			head = HEAD_CLASSIC;
		else if (head_str == "RHYTHMIC")
			head = HEAD_RHYTHMIC;
		else if (head_str == "X" || head_str == "X_HEAD")
			head = HEAD_X;
		else if (head_str == "PLUS" || head_str == "PLUS_HEAD")
			head = HEAD_PLUS;
		else if (head_str == "DDIESIS" || head_str == "HSHARP")
			head = HEAD_DDIESIS;
		else if (head_str == "DIAMOND")
			head = HEAD_DIAMOND;
		else if (head_str == "CIRCLEX")
			head = HEAD_CIRCLEX;
		else if (head_str == "MOON")
			head = HEAD_MOON;
		else return false;

		pMusicWin->SetCommand(dur_cmd,INSERT_SEL,BATTUTA_SEL,
			0,layer,AUTOMATICO,FALSE,0,"",0,0,0,head);
	}
	else if (cmd == "ADD_REST" && argCount >= 2)
	{
		layer = atoi(arguments[0]);

		wxString dur_str(arguments[1]);
		CmdID dur_cmd;

		if (dur_str == "D1")
			dur_cmd = CMD_BATTUTA_INS_PSEMIBREVE;
		else if (dur_str == "D1_2")
			dur_cmd = CMD_BATTUTA_INS_PMINIMA;
		else if (dur_str == "D1_4")
			dur_cmd = CMD_BATTUTA_INS_PSEMIMINIMA;
		else if (dur_str == "D1_8")
			dur_cmd = CMD_BATTUTA_INS_PCROMA;
		else if (dur_str == "D1_16")
			dur_cmd = CMD_BATTUTA_INS_PSEMICROMA;
		else if (dur_str == "D1_32")
			dur_cmd = CMD_BATTUTA_INS_PBISCROMA;
		else if (dur_str == "D1_64")
			dur_cmd = CMD_BATTUTA_INS_PSEMIBISCROMA;
		else return false;
		
		pMusicWin->SetCommand(dur_cmd,INSERT_SEL,BATTUTA_SEL,
			0,layer,AUTOMATICO,FALSE);
	}
	else if (cmd == "ADD_SYMBOL" && argCount >= 1)
	{
		wxString sym_str(arguments[0]);
		CmdID sym_cmd;
	
		if (sym_str == "TENUTO")
			sym_cmd = CMD_NOTA_TENUTO;
		else if (sym_str == "SFORZATO")
			sym_cmd = CMD_NOTA_SFORZATO;
		else if (sym_str == "ACCENTO")
			sym_cmd = CMD_NOTA_ACCENTO;
		else if (sym_str == "STACCATO")
			sym_cmd = CMD_NOTA_STACCATO;
		else if (sym_str == "MARTDOLCE")
			sym_cmd = CMD_NOTA_MARTDOLCE;
		else if (sym_str == "PUNTOALLUNGATO")
			sym_cmd = CMD_NOTA_PUNTOALL;
		else if (sym_str == "MARTELLATO")
			sym_cmd = CMD_NOTA_MARTELLATO;
		else if (sym_str == "ARCO")
			sym_cmd = CMD_NOTA_ARCO;
		else if (sym_str == "PONTICELLO")
			sym_cmd = CMD_ARCO_PONTICELLO;
		else if (sym_str == "TASTIERA")
			sym_cmd = CMD_ARCO_TASTIERA;
		else if (sym_str == "PUNTA")
			sym_cmd = CMD_ARCO_PUNTA;
		else if (sym_str == "TALLONE")
			sym_cmd = CMD_ARCO_TALLONE;
		else if (sym_str == "BOWUP")
			sym_cmd = CMD_ARCO_ARCOSU;
		else if (sym_str == "BOWDOWN")
			sym_cmd = CMD_ARCO_ARCOGIU;
		else if (sym_str == "PIZZ")
			sym_cmd = CMD_ARCO_CORDA;
		else return false;

		pMusicWin->SetCommand(sym_cmd,IMMEDIATE_SEL,NOTA_SEL,AUTOMATICO);
	}
	else if (cmd == "ADD_MEASURE" && argCount >= 1)
	{
		wxString bat_str(arguments[0]);

		if (bat_str == "BEFORE")
			pMusicWin->SetCommand(CMD_PARTITURA_ADD_BATTUTA,IMMEDIATE_SEL,BATPART_SEL,0,FALSE);//prima
		else if (bat_str == "AFTER")
			pMusicWin->SetCommand(CMD_PARTITURA_ADD_BATTUTA,IMMEDIATE_SEL,BATPART_SEL,0,TRUE);//dopo
		/*else if (bat_str == "APPEND")  // e qui come si fa?
			bat_cmd = ;*/
		else return false;
	}
	else if (cmd == "DEL_MEASURE"/* && argCount >= 1*/) //a che serve l'argomento?
	{
		pMusicWin->SetCommand(CMD_PARTITURA_DEL_BATTUTA,IMMEDIATE_SEL,BATPART_SEL); 
	}
	else if (cmd == "CHANGE_CLEF" && argCount >= 1)
	{
		wxString clf_str(arguments[0]);
		CmdID clf_cmd;		
		/*
		BARITONE
		BASS
		ALTO
		MEZZOSOPRANO
		SOPRANO
		TENOR
		TREBLE
		EMPTY
		PERCUSBOX

		BASSOLD
		TREBLE8
		8TREBLE
		TENOR8
		BASS8
		8BASS
		PERCUS2LINES
		TAB
		
		*/
		if (clf_str == "TREBLE")
			clf_cmd = CMD_BATTUTA_CVIOLINO;
		else if (clf_str == "BASS")
			clf_cmd = CMD_BATTUTA_CBASSO;
		else if (clf_str == "BARITONE")
			clf_cmd = CMD_BATTUTA_CBARITONO;
		else if (clf_str == "ALTO")
			clf_cmd = CMD_BATTUTA_CCONTRALTO;
		else if (clf_str == "SOPRANO")
			clf_cmd = CMD_BATTUTA_CSOPRANO;
		else if (clf_str == "PERCUSBOX")
			clf_cmd = CMD_BATTUTA_CPERCUSBOX;
		else if (clf_str == "MEZZOSOPRANO")
			clf_cmd = CMD_BATTUTA_CMEZZOSOPRANO;
		else if (clf_str == "TENOR")
			clf_cmd = CMD_BATTUTA_CTENORE;
		else if (clf_str == "EMPTY")
			clf_cmd = CMD_BATTUTA_CVUOTA;
		/*
		else if (clf_str == "8TREBLE")
			clf_cmd = CMD_BATTUTA_CVIOLINO8SOPRA;
		else if (clf_str == "TREBLE8")
			clf_cmd = CMD_BATTUTA_CVIOLINO8;
		else if (clf_str == "BASS8")
			clf_cmd = CMD_BATTUTA_CBASSO8;
		else if (clf_str == "8BASS")
			clf_cmd = CMD_BATTUTA_CBASSO8SOPRA;
		else if (clf_str == "TENOR8")
			clf_cmd = CMD_BATTUTA_CTENORE8;
		else if (clf_str == "PERCUS2LINES")
			clf_cmd = CMD_BATTUTA_CPERCUS2LINES;
		else if (clf_str == "TAB")
			clf_cmd = CMD_BATTUTA_CTAB;*/
		else return false;

		pMusicWin->SetCommand(clf_cmd,IMMEDIATE_SEL,BATTUTA_SEL,0,layer);
	}
	else if (cmd == "CHANGE_KEYSIGNATURE" && argCount >= 1)
	{
		wxString key_str(arguments[0]);
		CmdID key_cmd;		
		/*
		DOdM	LAdm
		FAdM	REdm 
		SIM	SOLdm
		MIM	DOdm
		LAM	FAdm
		REM	SIm
		SOLM	MIm
		DOM	Lam
		FAM	REm
		SIbM	SOLm
		MIbM	Dom
		LAbM	FAm
		REbM	SIbm
		SOLbM	MIbm
		DObM	LAbm
		*/
		if (key_str == "MIM")
			key_cmd = CMD_BATTUTA_ARMCHV_4D;
		else if (key_str == "LAM")
			key_cmd = CMD_BATTUTA_ARMCHV_3D;
		else if (key_str == "REM")
			key_cmd = CMD_BATTUTA_ARMCHV_2D;
		else if (key_str == "SOLM")
			key_cmd = CMD_BATTUTA_ARMCHV_1D;
		else if (key_str == "DOM")
			key_cmd = CMD_BATTUTA_ARMCHV_0D;
		else if (key_str == "FAM")
			key_cmd = CMD_BATTUTA_ARMCHV_1B;
		else if (key_str == "SIbM")
			key_cmd = CMD_BATTUTA_ARMCHV_2B;
		else if (key_str == "MIbM")
			key_cmd = CMD_BATTUTA_ARMCHV_3B;
		else if (key_str == "LAbM")
			key_cmd = CMD_BATTUTA_ARMCHV_4B;

		else if (key_str == "DOdm")
			key_cmd = CMD_BATTUTA_ARMCHV_4Dm;
		else if (key_str == "FAdm")
			key_cmd = CMD_BATTUTA_ARMCHV_3Dm;
		else if (key_str == "SIm")
			key_cmd = CMD_BATTUTA_ARMCHV_2Dm;
		else if (key_str == "MIm")
			key_cmd = CMD_BATTUTA_ARMCHV_1Dm;
		else if (key_str == "LAm")
			key_cmd = CMD_BATTUTA_ARMCHV_0Dm;
		else if (key_str == "REm")
			key_cmd = CMD_BATTUTA_ARMCHV_1Bm;
		else if (key_str == "SOLm")
			key_cmd = CMD_BATTUTA_ARMCHV_2Bm;
		else if (key_str == "DOm")
			key_cmd = CMD_BATTUTA_ARMCHV_3Bm;
		else if (key_str == "FAm")
			key_cmd = CMD_BATTUTA_ARMCHV_4Bm;

		else return false;
 
		pMusicWin->SetCommand(key_cmd,IMMEDIATE_SEL,BATTUTA_SEL);
	}
	else if (cmd == "CHANGE_TIME" && argCount >= 1)
	{
		wxString time_str(arguments[0]);
		if (time_str == "C" || time_str == "C/" || time_str == "4/4" || time_str == "3/4" || time_str == "2/4"
							|| time_str == "3/8" || time_str == "6/8" || time_str == "9/8" || time_str == "12/8")
			pMusicWin->SetCommand(CMD_BATTUTA_TEMPO,IMMEDIATE_SEL,BATTUTA_SEL,0,0,0,0,0,(char*)time_str.c_str());
		else return false;
	}
	else if (cmd == "SET_ALTERATION" && argCount >= 1)
	{
		wxString alt_str(arguments[0]);
		CmdID alt_cmd;
		/*
		NATURALUP	 
		NATURAL	 
		NATURALDOWN	 
		SHARP1Q	 
		SHARP	 
		SHARP3Q	 
		FLAT1Q	 
		FLAT 	 
		FLAT3Q	 
		DOUBLESHARP	 
		DOUBLEFLAT	   
		*/
		if (alt_str == "NATURAL")
			alt_cmd = CMD_NOTA_BEQUADRO;
		else if (alt_str == "SHARP")
			alt_cmd = CMD_NOTA_DIESIS;
		else if (alt_str == "DSHARP")
			alt_cmd = CMD_NOTA_2DIESIS;
		else if (alt_str == "FLAT")
			alt_cmd = CMD_NOTA_BEMOLLE;
		else if (alt_str == "DFLAT")
			alt_cmd = CMD_NOTA_2BEMOLLE;
		else return false;

		pMusicWin->SetCommand(alt_cmd,IMMEDIATE_SEL,NOTASING_SEL,AUTOMATICO);
	}
	else if (cmd == "SET_DOTS" && argCount >= 1)
	{
		int punti = atoi(arguments[0]);
		/*if (punti == 0)		//e qui come si fa?
*/
		if (punti == 1)
			pMusicWin->SetCommand(CMD_FIGURA_1PUNTO,IMMEDIATE_SEL,FIGURA_SEL);
		if (punti == 2)
			pMusicWin->SetCommand(CMD_FIGURA_2PUNTI,IMMEDIATE_SEL,FIGURA_SEL);
		else return false;
	}
	else if (cmd == "SET_METRONOME" && argCount >= 3)
	{
		int metro = atoi(arguments[2]);
		
		wxString dot_str(arguments[1]);
		bool has_dot;
		
		if (dot_str == "TRUE")
			has_dot = TRUE;
		else if (dot_str == "FALSE")
			has_dot = FALSE;
		else return false;

		wxString dur_str(arguments[0]);
		ClassID dur_cmd;

		if (dur_str == "D1")
			dur_cmd = CL_NSEMIBREVE;
		else if (dur_str == "D1_2")
			dur_cmd = CL_NMINIMA;
		else if (dur_str == "D1_4")
			dur_cmd = CL_NSEMIMINIMA;
		else if (dur_str == "D1_8")
			dur_cmd = CL_NCROMA;
		else if (dur_str == "D1_16")
			dur_cmd = CL_NSEMICROMA;
		else if (dur_str == "D1_32")
			dur_cmd = CL_NBISCROMA;
		else if (dur_str == "D1_64")
			dur_cmd = CL_NSEMIBISCROMA;
		else return false;

		pMusicWin->SetCommand(CMD_BATTUTA_MOVIMENTO,IMMEDIATE_SEL,BATTUTA_SEL,dur_cmd,has_dot,metro,0,0,"");

	}
	else if (cmd == "DELETE")
	{
		pMusicWin->SetCommand(CMD_DEL_SYMBOL,IMMEDIATE_SEL,ANY_SEL);
	}								
	else if (cmd == "TRANSPOSE" && argCount >= 4 )
	{
		//pMusicWin->SetCommand(CMD_PARTITURA_TRANSPOSE,NO_SEL,ANY_SEL);
		//pMusicWin->DoCommand();
		int part=atoi(arguments[0]);
		if(part<0)
			part=0;
		if(part>pMusicWin->PPartit->GetNumSpa())
			part=pMusicWin->PPartit->GetNumSpa();

		int fromMeasure=atoi(arguments[1]);
		if(fromMeasure<1)
			fromMeasure=1;
		
		int toMeasure=atoi(arguments[2]);
		if(toMeasure < 0 || toMeasure > pMusicWin->PPartit->GetNumeroBattute())
			toMeasure=0; //last measure;
		
		int interval=atoi(arguments[3]);
		int up=1;
		int numStaff=0;
		
		if(interval<0)
		{
			up=0;
			interval=-interval;
		}

		pMusicWin->PPartit->GetListaSpa()->Transpose(fromMeasure, toMeasure, part, -1,0,interval, up, numStaff, TRUE, TRUE );
		updated =true;
		return false; //caso particolare
	}								
	else 
		return false;




/*
Table 68 - REPETITION types

MEASURE	 
TWOMEASURE	 
BEAT	 
TWOBEAT	 


Table 75 - ARPEGGIO types

UP
DOWN 
NONE

	
Table 77 - BARLINE types

SINGLE	 
DOUBLE
START_REFRAIN	 
END_REFRAIN 
START_END_REFRAIN
END
INVISIBLE
	

*/
	return true;
}

void SMRInternalDecoder::DoCommand(char* command, int x, int y, int argCount, const char** arguments)
{
	if(partitura==NULL)
		return;

	if( SetCommand(command, argCount, arguments) )
	{
		Point mousePos(x, Scr->GetMaxPixY() - y);
		partitura->DoCommand(mousePos);
		updated=true;
	}
}

bool SMRInternalDecoder::DoNetCommand(char* stringa)
{
	if(partitura==NULL)
		return false;
	
	Network rete;
	SymCmd appoggio;
	
	rete.String2Sym(stringa,appoggio);

	if( appoggio.CommandId == CMD_DEL_SYMBOL || 
		appoggio.CommandId == CMD_PARTITURA_DEL_BATTUTA || 
		appoggio.CommandId == CMD_DEL_COL_FIGURE || 
		appoggio.CommandId == CMD_PARTITURA_DEL_SPARTITO)
	{
		if(smrCursor){
			smrCursor->Deselect();
			smrCursor->Reset();
		}
	}

	if (!partitura->DoNetCommand(appoggio))
		return false;

	updated=true;
	return true;
}

bool SMRInternalDecoder::CreateNetCommandFromCursor(char* command, int argCount, const char** arguments, char* cmd, char* anti){

	if(smrCursor == NULL)
		return false;

	Point p;
	switch(smrCursor->GetDepth()){
		case D_NOTE:
			p = smrCursor->GetNote()->GetAbsPos();
			break;
		case D_FIGURE:
			p = smrCursor->GetFigure()->GetAbsPos();
			break;
		case D_VOICE:
			p = smrCursor->GetVoice()->GetFirstFig()->GetAbsPos(); //TODO controllare !!!
			break;
		case D_MEASURE:
			p = smrCursor->GetMeasure()->GetAbsPos();
			break;
		case D_PART:
			p = smrCursor->GetPart()->GetAbsPos();
			break;
		default:
			return false;
	}

	if( SetCommand(command, argCount, arguments) )
	{
		//Point mousePos(x, Scr->GetMaxPixY() - y);
		return partitura->CreateNetCommand(p, cmd, anti);
	}

	return false;
}

bool SMRInternalDecoder::CreateNetCommand(char* command, int x, int y, int argCount, const char** arguments, char* cmd, char* anti)
{
	if(partitura == NULL)
		return false;
	
	if( SetCommand(command, argCount, arguments) )
	{
		Point mousePos(x, Scr->GetMaxPixY() - y);
		return partitura->CreateNetCommand(mousePos, cmd, anti);
	}

	return false;
}
int SMRInternalDecoder::TimeToSynchPos(int time, bool& reachedEnd)
{
	int temp = 0;
	int count = 0;

	while (count < synchInfoCount && time >= synchInfo[count].startTime)
	{
		count++;
	}
	if(count==synchInfoCount && time>= synchInfo[count-1].startTime+synchInfo[count-1].durate)
		reachedEnd=true;
	else
		reachedEnd=false;

	return count-1;
}

void SMRInternalDecoder::LoadSynch(char* path)
{
	int rows=0;
	wxString SyncPath(path);
	wxString SyncFiles("\\performance1.ett");

	SyncFiles=SyncPath+SyncFiles;

	FILE *file = fopen(SyncFiles,"r");
	if (file)	///FFF
	{
		synchInfoCount = 0;
		fscanf(file,"%d", &synchInfoCount);
		
		synchInfo=new SynchInfo[synchInfoCount];
		
		synchInfo[0].startTime = 0;
		for(int i=0;i<synchInfoCount;i++)
		{
			fscanf(file,"%d %d", &synchInfo[i].measure, &synchInfo[i].durate);
			if(i>0)
				synchInfo[i].startTime = synchInfo[i-1].startTime + synchInfo[i-1].durate;
		}
		fclose(file);
	}
	else
	{
		partitura->InitTimeExec();
		synchInfoCount = partitura->GetNumeroBattute();
		synchInfo=new SynchInfo[synchInfoCount];
		
		synchInfo[0].startTime = 0;
		for(int i=0;i<synchInfoCount;i++)
		{
			synchInfo[i].measure=i+1;
			synchInfo[i].durate=partitura->GetTimeExec(i+1);
			if(i>0)
				synchInfo[i].startTime = synchInfo[i-1].startTime + synchInfo[i-1].durate;
		}
	}
}

void SMRInternalDecoder::GoTop()		
{ 
	if(partitura) {
		partitura->GoTop();
		updated=true;}
}

void SMRInternalDecoder::GoBottom()		
{ 
	if(partitura) {
		partitura->GoBottom();
		updated=true;}
}

void SMRInternalDecoder::GoBackward()	
{ 
	if(partitura) {
		partitura->GoBackward();
		updated=true;}
}

void SMRInternalDecoder::GoForward()	
{ 
	if(partitura) {
		partitura->GoForward();
		updated=true;}
}

const unsigned int SMRInternalDecoder::GetHitLevel()
{
	return hitLevel;
}

const unsigned int SMRInternalDecoder::getCurrentDepth()
{
	if(!hitLevel || hitLevel > smrCursor->GetDepth())
		return smrCursor->GetDepth();
	else
		return hitLevel;
}

void SMRInternalDecoder::SetHitLevel(const unsigned int & level)
{
	hitLevel = level;

	if(!level)
		return;

	smrCursor->Deselect();

//SE NON CE' nulla a quel livello provare ad inserire il primo puntatore della lista(se non e' vuota)
	if (!smrCursor->GetDepth())
		return;
	
	if (smrCursor->GetDepth() < hitLevel)//TODO: controllare!!
	{
		switch(smrCursor->GetDepth())
		{
		//case 0 :
		//	smrCursor->Reset();
		//	break;
		case D_PART :
			if(SetCurrentMeasure(GetFirstVisibleMeasure()) !=0 )
				hitLevel = D_PART;
			if (smrCursor->GetDepth() == hitLevel) 
				break;
		case D_MEASURE :
			if(SetCurrentVoice(1) !=0 )
				hitLevel = D_MEASURE;
			if (smrCursor->GetDepth() == hitLevel) 
				break;
		case D_VOICE :
			if(SetCurrentEvent(1) !=0 )
				hitLevel = D_VOICE;
			if (smrCursor->GetDepth() == hitLevel) 
				break;
		case D_FIGURE :
			if(SetChordNote(1) !=0 )
				/*hitLevel = D_FIGURE*/;
			break;
		}		
	}
	

	if (hitLevel == D_NOTE && smrCursor->GetDepth() == D_FIGURE)
		selectCursorAtDepth(D_FIGURE);
	//else if (hitLevel==D_EMPTY)
	//	smrCursor->Select(smrCursor->GetDepth());
	else
		selectCursorAtDepth(getCurrentDepth());	

	updated=true;
}

//PER IL treeCursor
bool SMRInternalDecoder::Hit(int x, int y)
{
	if (partitura == NULL)
		return false;

	DrawObject *pObject = NULL;
	SymPath		Path;

	float scale = GetMagnification();
	int xS, yS;
	xS = x/scale;
	yS = y/scale;

	Point mousePos(xS, Scr->GetMaxPixY() - yS);

	smrCursor->Deselect();
	smrCursor->Reset();


/*		
MY_SEL:
0	ANY
1		SPA
2		SPA + BAT
3		SPA + BAT + LAY
4		SPA + BAT + LAY + FIG1
5		SPA + BAT + LAY + FIG1 + FIG2
6		SPA + BAT + LAY + FIG1 + FIG2 + NOTE
7		SPA + BAT + LAY + FIG1 + FIG2 + NOTE + ATTR
?		COLONNA DI BATTUTE?
*/



/*	partitura->Hit(mousePos, ANY_SEL, pObject, Path);
	if(pObject)
	{
		DrawObject *pParentObj;
		partitura->Hit(mousePos, FIGURA_SEL, pParentObj, Path);
		if(pObject != pParentObj)
			smrCursor->SetSymbol(pObject);
		// e il parent deve essere nota/figura
	}*/
							
	//partitura->Hit(mousePos, NOTASING_SEL, pObject, Path);
	//if (pObject)
	//	smrCursor->SetNote((Nota*)pObject);

	//partitura->Hit(mousePos, NOTA_SEL, pObject, Path);						
	//if (pObject)
	//	smrCursor->SetChordOrNote((Nota*)pObject);

	//partitura->Hit(mousePos, GRUPPONOTE_SEL, pObject, Path);
	//if (!pObject)
	
	partitura->Hit(mousePos, FIGURA_SEL, pObject, Path);
	if (pObject)
	{
		smrCursor->SetFigure((Figura*)pObject);

		smrCursor->SetVoice( ((Figura*)pObject)->GetPBattuta()->GetPLayer(Path.Array[LAYER_PATH_LEVEL] - 1));

		if (pObject->GetID() == CL_ACCORDO)
		{
			partitura->Hit(mousePos, NOTASING_SEL, pObject, Path);
			if (pObject)
				smrCursor->SetNote((Nota*)pObject);
		}
	}

	partitura->Hit(mousePos, BATTUTA_SEL, pObject, Path);
	if (pObject)
		smrCursor->SetMeasure((Battuta*)pObject);

	partitura->Hit(mousePos, SPARTITO_SEL, pObject, Path);
	if (pObject)
		smrCursor->SetPart((Spartito*)pObject);

	updated=true;

	//curDepth depth;
	//if (!hitLevel)
	//	depth = smrCursor->GetDepth();
	//else
	//	depth = (curDepth)hitLevel;

	if(!smrCursor->GetDepth() ||  hitLevel > smrCursor->GetDepth())
	{
		if(smrCursor->GetDepth() < D_FIGURE || !smrCursor->GetFigure()->GetDurata())
		{
			smrCursor->Reset(); //TODO: controllare!!!
			return false;
		}
		else
			selectCursorAtDepth(D_FIGURE);
	}
	else
		selectCursorAtDepth(getCurrentDepth());
	

	InitDrag(x,y);


/*
ANY_SEL,        // seleziona qualsiasi oggetto
SPARTITO_SEL,   // seleziona uno Spartito
BATTUTA_SEL,    // seleziona una Battuta
BATPART_SEL,    // seleziona una colonna di battute della partitura
FIGURA_SEL,     // seleziona una Figura (Nota,Pausa o Accordo)
NOTA_SEL,       // seleziona una Nota   (Nota o Accordo)
NOTASING_SEL,   // seleziona una Nota
GRUPPONOTE_SEL  // seleziona un GruppoNote
*/


	//Hilight(pObjectPrev);

	return true;
}

//void SMRInternalDecoder::HilightOn() 
//{ 
//	hilighted = true; 
//	Hilight();
//	//if (evtCursor.evtType == CHORD_EVT)
//	//	Hilight(evtCursor.pChordNote);
//	//else
//	//	Hilight(evtCursor.objRef);
//	updated=true;
//}
//void SMRInternalDecoder::HilightOff()
//{ 
//	hilighted = false; 
//	Hilight();
//	updated=true;
//}

void SMRInternalDecoder::selectCursorAtDepth(const unsigned int depth){
	if(!depth || highlighted)
		smrCursor->Select((curDepth)depth);
	//else if(highlighted)
	//	smrCursor->Select((curDepth)depth);
}


void SMRInternalDecoder::Highlight(bool val)
{
	if(highlighted == val)
		return;

	highlighted = val;

	if(val)
		selectCursorAtDepth(getCurrentDepth());
	else
		selectCursorAtDepth(0);

	updated=true;

	//if (hilighted)
	//{
	//	if (pObj==NULL)
	//	{
	//		if (oldObj!=NULL)
	//			oldObj->Deselect();//oldObj->SetColor(*oldColor);//ripristino
	//		updated=true;
	//	}	
	//	if (pObj!=oldObj)
	//	{
	//		if (oldObj!=NULL)
	//			oldObj->Deselect();//oldObj->SetColor(*oldColor);//ripristino
	//		
	//		if(pObj/* && oldColor*/)
	//		{
	//			//*oldColor = pObj->GetColor();
	//			pObj->Select();//pObj->SetColor(oldColor->Red()+127, oldColor->Green()+127, oldColor->Blue()+127);//aggiorna
	//		}
	//		oldObj = pObj;
	//		updated=true;
	//	}
	//}
	//else
	//{
	//	if (oldObj!=NULL)
	//	{
	//		oldObj->Deselect();//oldObj->SetColor(*oldColor);//ripristino
	//		oldObj=NULL;
	//		updated=true;
	//	}
	//}
}


//-- SCORE FOLLOWER SPECIFIC COMMANDS

void SMRInternalDecoder::GetTreeCursor(treecursor & cur)
{
	//reset
	for(int i=0; i<5; i++)
		cur[i] = 0;
	
	if (!partitura)
		return;
	
	if(smrCursor->GetDepth() == 0)
		return;

	Spartito* s = partitura->GetFirstSpa();
	while(s)
	{
		cur[0]++;
		if(s == smrCursor->GetPart())
		{
			if(smrCursor->GetDepth() == 1 || hitLevel == 1)
				return;

			Battuta* b = s->GetFirstBat();
			while(b)
			{
				cur[1]++;
				if(b == smrCursor->GetMeasure())
				{			
					if(smrCursor->GetDepth() == 2 || hitLevel == 2)
						return;

					for(int i=0; i < NUM_MAX_LAYER; i++)
					{
						Layer* v = b->GetPLayer(i);
						if(v && v->GetNumeroFig())
						{
							cur[2]++;
							if(v == smrCursor->GetVoice())
							{
								if(smrCursor->GetDepth() == 3 || hitLevel == 3)
									return;

								Figura* f = v->GetFirstFig();
								while(f)
								{
									cur[3]++;
									if(f == smrCursor->GetFigure())
									{
										if(f->GetID() == CL_ACCORDO)
										{
											if(smrCursor->GetDepth() == 4 || hitLevel == 4)
												return;

											Nota* n = (Nota*)((Accordo*)f)->GetFirstF();
											while(f)
											{
												cur[4]++;
												if(n == smrCursor->GetNote())
													break;

												n = (Nota*)((Accordo*)f)->GetNextF(n);
											}
										}
										break;
									}
									f = v->GetNextFig();
								}

								break;
							}

						}
					}

					if(i == NUM_MAX_LAYER)
					{
						cur[2] = 0;
						break;
					}

					break;
				}
				b = s->GetNextBat(b);
			}

			break;
		}
		s = partitura->GetNextSpa(s);
	}
	
}

void SMRInternalDecoder::SetTreeCursor(treecursor & cur)
{
	if(!partitura)
		return;

	smrCursor->Deselect();
	smrCursor->Reset();				//controllare

	if(SetCurrentPart(cur[0]) == 0){	
		if(SetCurrentMeasure(cur[1]) == 0){	
			if(SetCurrentVoice(cur[2]) == 0){
				if(SetCurrentEvent(cur[3]) == 0){
					if(SetChordNote(cur[4]) == 0){
					}
				}
			}
		}
	}

	if(smrCursor->GetDepth())
	{
		hitLevel = smrCursor->GetDepth();
		selectCursorAtDepth(getCurrentDepth());
	}
	updated=true;
}

//#define CLIP(a, lo, hi) ( (a)>(lo)?( (a)<(hi)?(a):(hi) ):(lo) )

void SMRInternalDecoder::MoveTreeCursor(const treecursorpos & tcp)
{
	if(!partitura/* || !smrCursor->GetDepth()*/)
		return;

	// bool page_turn;= false;

	curDepth depth = (curDepth)getCurrentDepth();

	//if (hitLevel==D_EMPTY)
	//	depth = smrCursor->GetDepth();
	//else
	//	depth = (curDepth)hitLevel;

	//-- HIT LEVEL--
	if(tcp ==POS_IN) {
		if(depth == D_FIGURE && smrCursor->GetFigure() && smrCursor->GetFigure()->GetID() != CL_ACCORDO)
			;
		else if(depth < D_NOTE) {
			//smrCursor->GoIn(depth)
			SetHitLevel(depth+1);
		}
		return;
	}
	else if(tcp ==POS_OUT) {
		if(depth == D_NOTE && smrCursor->GetFigure() && smrCursor->GetFigure()->GetID() != CL_ACCORDO)
			//smrCursor->GoOut(D_FIGURE);
			SetHitLevel(D_VOICE);
		else if(depth > D_PART) {
			//smrCursor->GoOut(depth)
			SetHitLevel(depth-1);
		}
		return;
	}
	//------------------

	smrCursor->Deselect();

	switch(tcp)
	{
	case POS_PREV:
		smrCursor->GoPrev(depth);
		if(!smrCursor->GetDepth())
			smrCursor->GoFirst(depth);
		break;
	case POS_NEXT:
		smrCursor->GoNext(depth);
		if(!smrCursor->GetDepth())
			smrCursor->GoLast(depth);
		break;
	case POS_BEGIN:
		//depth = (curDepth)hitLevel;
		smrCursor->GoFirst((curDepth)hitLevel);
		depth = (curDepth)getCurrentDepth();
	    break;
	case POS_END:
		//depth = (curDepth)hitLevel;
		smrCursor->GoLast((curDepth)hitLevel);
		depth = (curDepth)getCurrentDepth();
	    break;
	case POS_LEFT:
//		graph_mooving = true;
		{
			int first = GetFirstVisibleMeasure();
			smrCursor->GoLeft(depth);
			unsigned int pos = smrCursor->GetPosition(D_MEASURE);
			if(pos == smrCursor->GetCount(D_MEASURE))
				GoBottom();
			else if(pos < first)
				GoBackward();
		}
		break;
	case POS_RIGHT:
//		graph_mooving = true;
		{
			int last = GetLastVisibleMeasure();
			smrCursor->GoRight(depth);
			unsigned int pos = smrCursor->GetPosition(D_MEASURE);
			if(pos == 1)
				GoTop();
			else if(pos > last)
				GoForward();
		}
	    break;
	case POS_UP:
//		graph_mooving = true;
		smrCursor->GoUp(depth);
		break;
	case POS_DOWN:
//		graph_mooving = true;
		smrCursor->GoDown(depth);
		break;	
	//case POS_IN:
	//	smrCursor->GoIn(depth);
	//	break;
	//case POS_OUT:
	//	smrCursor->GoOut(depth);
	//	break;
	}

	//if (graph_mooving){
	//	if(smrCursor->GetPosition(D_MEASURE) == 1)
	//		GoTop();
	//	else if(smrCursor->GetPosition(D_MEASURE) > GetLastVisibleMeasure())
	//		GoForward();
	//	else if(smrCursor->GetPosition(D_MEASURE) == smrCursor->GetCount(D_MEASURE))
	//		GoBottom();
	//	else if(smrCursor->GetPosition(D_MEASURE) < GetFirstVisibleMeasure())
	//		GoBackward();
	//}

	if (depth == D_NOTE && smrCursor->GetDepth() == D_FIGURE)
		selectCursorAtDepth(D_FIGURE);
	else
		selectCursorAtDepth(depth);	

	updated=true;
}


int SMRInternalDecoder::SetCurrentPart(int part)
{
	if (partitura == NULL)
		return OUT_OF_BOUNDS;
	else if (part < 1 || part > GetPartCount())
		return OUT_OF_BOUNDS;

	smrCursor->SetPart(partitura->GetSpartNum(part-1));

	//evtCursor.staffNum	= -1;
	smrCursor->SetMeasure(NULL);
	smrCursor->SetVoice(NULL);
	smrCursor->SetFigure(NULL);
	smrCursor->SetNote(NULL);
/*
	evtCursor.objRef	= NULL;
	evtCursor.pChordNote = NULL;
	evtCursor.evtType	= UNKNOWN_EVT;

	// default position
	SetCurrentStaff(1);
	SetCurrentMeasure(1);
*/
	return 0;
}

int SMRInternalDecoder::SetCurrentStaff(int staff)
{
/*	if (partitura == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetPart() == NULL)
		return OUT_OF_BOUNDS;
	else if (staff < 1 || staff > GetStaffCount())
		return OUT_OF_BOUNDS;

	evtCursor.staffNum = staff-1;
*/
	return 0;
}

int SMRInternalDecoder::SetCurrentMeasure(int measure)
{
	if (partitura == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetPart() == NULL)
		return OUT_OF_BOUNDS;
	else if (measure < 1 || measure > GetMeasureCount())
		return OUT_OF_BOUNDS;

	smrCursor->SetMeasure(smrCursor->GetPart()->GetBattNum(measure-1));

	smrCursor->SetVoice(NULL);
	smrCursor->SetFigure(NULL);
	smrCursor->SetNote(NULL);
//	evtCursor.evtType	= UNKNOWN_EVT;

	// default position
	//SetCurrentVoice(1);

	return 0;
/////
}

int SMRInternalDecoder::SetCurrentVoice(int voice)
{
	if (partitura == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetPart() == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetMeasure() == NULL)
		return OUT_OF_BOUNDS;
	else if (voice < 1 || voice > GetVoiceCount())
		return OUT_OF_BOUNDS;

	// setting layerID
	Battuta* batt = smrCursor->GetMeasure();
	
	int s=0;
	for (int j=0; j<NUM_MAX_LAYER; j++)
	{
		if (batt->GetPLayer(j) != NULL && batt->GetPLayer(j)->GetNumeroFig())
			s++;
		if (s == voice)
		{
			smrCursor->SetVoice(batt->GetPLayer(j));
			break;
		}
	}
	
	smrCursor->SetFigure(NULL);
	smrCursor->SetNote(NULL);
	//evtCursor.evtType	= UNKNOWN_EVT;

	// default position
	//SetCurrentEvent(1);

	return 0;
/////
}


int SMRInternalDecoder::SetCurrentEvent(int event)
{
	if (partitura == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetPart() == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetMeasure() == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetVoice() == NULL)
		return OUT_OF_BOUNDS;
	else if (event < 1 || event > GetEventCount())
		return OUT_OF_BOUNDS;

	smrCursor->SetFigure(smrCursor->GetVoice()->AtPosFig(event-1));

	smrCursor->SetNote(NULL);
//	evtCursor.evtType	= UNKNOWN_EVT;
	
	//Hilight(evtCursor.objRef);//DA RIMETTERE

	return 0;
}

int SMRInternalDecoder::SetChordNote(int event)
{
	if (partitura == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetPart() == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetMeasure() == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetVoice() == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetFigure() == NULL)
		return OUT_OF_BOUNDS;
	else if (event < 1 || event > GetChordNoteCount())
		return OUT_OF_BOUNDS;

	smrCursor->SetNote((Nota*)((Accordo*)smrCursor->GetFigure())->AtPosFig(event-1));
	
	//Hilight(evtCursor.pChordNote);//DA RIMETTERE

	return 0;
}


char* SMRInternalDecoder::GetPartName()
{
	if (partitura == NULL || smrCursor->GetPart() == NULL)
		return NULL;	
	return (char*)smrCursor->GetPart()->GetNome();
}

int SMRInternalDecoder::GetStaffCount()
{
	if (partitura == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetPart() == NULL)
		return PART_NOT_SET;

	return smrCursor->GetPart()->GetNumberOfStaff();
}

int SMRInternalDecoder::GetMeasCount()
{
	if (partitura == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetPart() == NULL)
		return PART_NOT_SET;

	return smrCursor->GetPart()->GetNumeroBattute();
}

int SMRInternalDecoder::GetVoiceCount()
{
	if (partitura == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetPart() == NULL)
		return PART_NOT_SET;
	else if (smrCursor->GetMeasure() == NULL)
		return MEASURE_NOT_SET;
	
	Battuta* batt = smrCursor->GetMeasure();
	
	int s=0;
	for (int j=0; j<NUM_MAX_LAYER; j++)
	{
		if (batt->GetPLayer(j) != NULL && batt->GetPLayer(j)->Calc_Fig_Layer())
			s++;
	}
		
	return s;
}

int SMRInternalDecoder::GetEventCount()
{
	if (partitura == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetPart() == NULL)
		return PART_NOT_SET;
	else if (smrCursor->GetMeasure() == NULL)
		return MEASURE_NOT_SET;
	else if (smrCursor->GetVoice() == NULL)
		return VOICE_NOT_SET;
	
	return smrCursor->GetVoice()->Calc_Fig_Layer();
}

int SMRInternalDecoder::GetEventType()
{
	if (partitura == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetPart() == NULL)
		return PART_NOT_SET;
	else if (smrCursor->GetMeasure() == NULL)
		return MEASURE_NOT_SET;
	else if (smrCursor->GetVoice() == NULL)
		return VOICE_NOT_SET;
	else if (smrCursor->GetFigure() == NULL)
		return EVENT_NOT_SET;

	Figura *pFig = smrCursor->GetFigure();
	if (pFig->is(CL_ACCORDO))
	{
		return CHORD_EVT;

		//default position
		//SetChordNote(1);
	}
	else if (pFig->IsANote())
		return NOTE_EVT;
	else if (pFig->IsARest())
		return REST_EVT;
	else if (pFig->is(CL_ARMATURACHIAVE))
		return KEY_EVT;

	else switch (pFig->GetID())
	{
		case CL_CBARITONO:
		case CL_CBASSO:
		case CL_CCONTRALTO:
		case CL_CMEZZOSOPRANO:
		case CL_CSOPRANO:
		case CL_CTENORE:
		case CL_CTENORE8:
		case CL_CVIOLINO:
		case CL_CVIOLINO8:
		case CL_CVIOLINO8SOPRA:
		case CL_CBASSO8:
		case CL_CBASSO8SOPRA:
		case CL_CBASSOOLD:
		case CL_CVUOTA:
		case CL_CTAB:
		case CL_CPERCUSBOX:
		case CL_CPERCUS2LINES:
			return CLEF_EVT;
			break;
	}
	//SPAZIO
	//REPEATS
	return UNKNOWN_EVT;//.evtType;
}

int SMRInternalDecoder::GetChordNoteCount()
{
	if (partitura == NULL)
		return OUT_OF_BOUNDS;
	else if (smrCursor->GetPart() == NULL)
		return PART_NOT_SET;
	else if (smrCursor->GetMeasure() == NULL)
		return MEASURE_NOT_SET;
	else if (smrCursor->GetVoice() == NULL)
		return VOICE_NOT_SET;
	else if (smrCursor->GetFigure() == NULL)
		return EVENT_NOT_SET;
	else if (GetEventType() != CHORD_EVT)
		return WRONG_COMMAND;

	return ((Accordo*)smrCursor->GetFigure())->GetNumObj();
}

// MEASURE INFO /////////////////////////////////////////////////////////////////////////////////////////////

int SMRInternalDecoder::GetBarType()
{
	if(smrCursor->GetMeasure() == NULL)
		return WRONG_COMMAND;

	return smrCursor->GetMeasure()->GetBarra()->GetTipoBarra();
}

int SMRInternalDecoder::GetMetroBPM()
{
	if(smrCursor->GetMeasure() == NULL)
		return WRONG_COMMAND;

	Metronomo *metro;
	Movimento *mov;
	Battuta *pBat = smrCursor->GetMeasure();
	while (pBat)
	{
		mov = pBat->GetPMovimento();
		if (mov != NULL)
		{
			metro = mov->GetMetronomo();
			if (metro != NULL)
				return metro->GetTNumerico()->GetNumero();
		}

		pBat = smrCursor->GetPart()->GetPrevBat(pBat);
	}
	return WRONG_COMMAND;
}

int SMRInternalDecoder::GetMetroRef()
{
	if(smrCursor->GetMeasure() == NULL)
		return WRONG_COMMAND;

	Metronomo *metro;
	Movimento *mov;
	Battuta *pBat = smrCursor->GetMeasure();
	while (pBat)
	{
		mov = pBat->GetPMovimento();
		if (mov != NULL)
		{
			metro = mov->GetMetronomo();
			if (metro != NULL)
				return 4096 * metro->GetValNota();
		}

		pBat = smrCursor->GetPart()->GetPrevBat(pBat);
	}
	return WRONG_COMMAND;
}

bool SMRInternalDecoder::IsUpdated()
{ 
	bool upd=updated; 
	updated=false; 
	return upd; 
}
	
char* SMRInternalDecoder::GetLabel()
{
	if(smrCursor->GetMeasure() == NULL)
		return "";

	Lettera *letter = smrCursor->GetMeasure()->GetPLettera();
	if (letter != NULL)
		return (char*)letter->GetTxt();
	
	Riferimento *rif = smrCursor->GetMeasure()->GetPRiferimento();
	if (rif != NULL)
	{
		if (rif->GetID() == CL_SEGNO)
			return "SEGNO";
		if (rif->GetID() == CL_CODA)
			return "CODA";
	}
//		return (char*)rif->GetTxt();

	TSalto *salto = smrCursor->GetMeasure()->GetPTSalto();
	if (salto != NULL)
	{
		char *label = (char*)salto->GetTxt();
		if(strcmp(label,"D.C. al Fine") == 0)
			return "DCAF";
		if(strcmp(label,"D.C.") == 0)
			return "DC";
		if(strcmp(label,"D.S.") == 0)
			return "DS";
		if(strcmp(label,"D.S. al Fine") == 0)
			return "DSAF";
		if(strcmp(label,"D.C. al Segno") == 0)
			return "DCAS";
		if(strcmp(label,"D.S. al Coda") == 0)
			return "DSAC";
		if(strcmp(label,"Fine") == 0)
			return "FINE";
	}
//		return (char*)salto->GetTxt();
	
	return "";
}

int SMRInternalDecoder::GetTimeNum()
{
	if(smrCursor->GetMeasure() == NULL)
		return WRONG_COMMAND;

	return smrCursor->GetMeasure()->GetTempo().GetNumeratore();
}

int SMRInternalDecoder::GetTimeDen()
{
	if(smrCursor->GetMeasure() == NULL)
		return WRONG_COMMAND;

	return smrCursor->GetMeasure()->GetTempo().GetDenominatore();
}

// EVENT INFO /////////////////////////////////////////////////////////////////////////////////////////////

char* SMRInternalDecoder::GetPitch()
{
	char* msg = (char*) malloc(4 * sizeof(char));
	memset(msg, 0, 4 * sizeof(char));

	if(getCurrentDepth() >= D_FIGURE) {
		switch (GetEventType())
		{
		case NOTE_EVT:
			//note = (Nota*)evtCursor.objRef;
			//strcpy( msg, note->CreatePitchStr().c_str());
			strcpy( msg, ((Nota*)smrCursor->GetFigure())->CreatePitchStr().c_str());
			break;
		case CHORD_EVT:
			if (hitLevel == D_NOTE && smrCursor->GetNote())
			{
				//note = evtCursor.pChordNote;
				strcpy( msg, ((Nota*)smrCursor->GetNote())->CreatePitchStr().c_str());
			}
			break;
		}
	}
	return msg;
}

void SMRInternalDecoder::InitDrag(int x, int y)
{
	dragTemp = 0;
}

void SMRInternalDecoder::Drag(int dx, int dy)
{
	if(!smrCursor->GetDepth())
		return;

	float norm = (dy - dragTemp)/GetMagnification();
	int dH = norm/3.0;

	if(!dH) return;

	dragTemp = dy;

	switch(hitLevel)
	{
	case 1:
		break;
	case 2:
		break;
	case 3:
		if(smrCursor->GetVoice()) smrCursor->GetVoice()->Raise(dH);
		break;
	case 4:
		if(smrCursor->GetFigure()) smrCursor->GetFigure()->Raise(dH);
		break;
	//case 5:
	//	if(smrCursor->GetChordOrNote()) smrCursor->GetChordOrNote()->Raise(dH);
	//	break;
	case 5:
		if(smrCursor->GetNote()) smrCursor->GetNote()->Raise(dH);
		break;
	}

	updated = true;
}

void SMRInternalDecoder::EndDrag(int dx, int dy)
{
	//updated = true;
}

int SMRInternalDecoder::GetHeight()
{
	if(getCurrentDepth() >= D_FIGURE) {
		switch (GetEventType())
		{
		case NOTE_EVT:
		case REST_EVT:
			return smrCursor->GetFigure()->GetAltezza();
			break;
		case CHORD_EVT:
			if (hitLevel == D_NOTE && smrCursor->GetNote())
				return smrCursor->GetNote()->GetAltezza();
			else
				return WRONG_COMMAND;
			break;
		}
	}
	return WRONG_COMMAND;
}

void SMRInternalDecoder::SetHeight(int value)
{
/*	if(evtCursor.objRef == NULL)
		return //WRONG_COMMAND;
	switch (evtCursor.evtType)
	{
	case NOTE_EVT:
	case REST_EVT:
	case CHORD_EVT:
		evtCursor.objRef->SetAltezza(value);
		updated = true;
		break;
	default:
		//return WRONG_COMMAND;
		break;
	}*/
}

int SMRInternalDecoder::GetDuration()
{
	if(getCurrentDepth() < D_FIGURE)
		return WRONG_COMMAND;

	switch (GetEventType())
	{
	case NOTE_EVT:
	case REST_EVT:
	case CHORD_EVT:
		return 4096 * smrCursor->GetFigure()->GetDurata();
		break;
		//return 4096 * smrCursor->GetNote()->GetDurata();
		//break;
	}

	return WRONG_COMMAND;
}

int SMRInternalDecoder::SetDuration(int value)
{
/*	if(evtCursor.objRef == NULL)
		return WRONG_COMMAND;
	switch (evtCursor.evtType)
	{
	case NOTE_EVT:
	case REST_EVT:
	case CHORD_EVT:
		evtCursor.objRef->SetDurata( value/(float)4096 );
		updated = true;
		return value;
		//break;
	default:
		return WRONG_COMMAND;
		break;
	}*/
	return WRONG_COMMAND;
}

int SMRInternalDecoder::isTied()
{
	switch(getCurrentDepth())
	{
	case D_FIGURE:
		if(smrCursor->GetFigure()->IsANote())
			return ((Nota*)smrCursor->GetFigure())->GetTie();
		break;
	case D_NOTE:
		return smrCursor->GetNote()->GetTie();
	    break;
	}

	return WRONG_COMMAND;
}

int SMRInternalDecoder::GetClef(bool isInternal)
{
	if(getCurrentDepth() < D_MEASURE || smrCursor->GetMeasure() == NULL)
		return WRONG_COMMAND;

	ClassID clefID;

	if (isInternal)
	{
		//if(evtCursor.objRef == NULL)
		//	return WRONG_COMMAND;
		if(GetEventType() != CLEF_EVT)
			return WRONG_COMMAND;
		
		clefID = smrCursor->GetFigure()->GetID();
	}
	else
		clefID = smrCursor->GetMeasure()->GetChiave(0/*smrCursor->GetFigure()->GetNStaff()*/)->GetID(); //TODO: aggiustare Staff!

	switch (clefID)
	{
		case CL_CBARITONO:
			return 0;
			break;
		case CL_CBASSO:
			return 1;
			break;
		case CL_CBASSOOLD:
			return 2;
			break;
		case CL_CCONTRALTO:
			return 3;
			break;
		case CL_CMEZZOSOPRANO:
			return 4;
			break;
		case CL_CSOPRANO:
			return 5;
			break;
		case CL_CTENORE:
			return 6;
			break;
		case CL_CTENORE8:
			return 7;
			break;
		case CL_CVIOLINO:
			return 8;
			break;
		case CL_CVIOLINO8:
			return 9;
			break;
		case CL_CVIOLINO8SOPRA:
			return 10;
			break;
		case CL_CBASSO8:
			return 11;
			break;
		case CL_CBASSO8SOPRA:
			return 12;
			break;
		case CL_CVUOTA:
			return 13;
			break;
		case CL_CTAB:
			return 14;
			break;
		case CL_CPERCUSBOX:
			return 15;
			break;
		case CL_CPERCUS2LINES:
			return 16;
			break;
		default:
			return WRONG_COMMAND;
			break;
	}
	return WRONG_COMMAND;
}

int SMRInternalDecoder::GetKey(bool isInternal)
{
	if(getCurrentDepth() < D_MEASURE || smrCursor->GetMeasure() == NULL)
		return WRONG_COMMAND;

	if (isInternal)
	{
		//if(evtCursor.objRef == NULL)
		//	return WRONG_COMMAND;
		if(GetEventType() != KEY_EVT)
			return WRONG_COMMAND;
		
		return ((ArmaturaChiave*)smrCursor->GetFigure())->GetArmatura();
	}
	else
		return smrCursor->GetMeasure()->GetArmatura(0/*smrCursor->GetFigure()->GetNStaff()*/);//TODO: aggiustare Staff!
	return WRONG_COMMAND;
}

void SMRInternalDecoder::SetColor(const unsigned char red, const unsigned char green, const unsigned char blue){
	smrCursor->SetColor(Colore(red, green, blue), (curDepth)getCurrentDepth());
	updated = true;
}
