
#include "SMRcursor.h"
#include "par/partitur.hpp"

SMRcursor::SMRcursor(Partitura* const & s ){
	SetScore(s);

//	highlighted = false;
}

void SMRcursor::SetScore(Partitura* const & s){
	pScore = s;
	Reset();
}
//SMRcursor::~SMRcursor(void ){
//
//}

void SMRcursor::Reset() {
	pPart		= NULL;
	pMeasure	= NULL;
	pVoice		= NULL;
	pFigure		= NULL;
	pNote		= NULL;
}

void SMRcursor::Reset(const curDepth &level) {
	switch (level){
		case D_EMPTY :
			return;
		case D_PART :
			if(pPart) pPart->Deselect();
			pPart		= NULL;
		case D_MEASURE :
			if(pMeasure) pMeasure->Deselect();
			pMeasure	= NULL;
		case D_VOICE :
			if(pVoice) pVoice->Deselect(/*pMeasure->GetAbsPos() + pMeasure->GetVU2Rgt()/2*/);
			pVoice		= NULL;
		case D_FIGURE :
			if(pFigure) pFigure->Deselect();
			pFigure		= NULL;
		case D_NOTE :
			if(pNote) pNote->Deselect();
			pNote		= NULL;
	}
}

void SMRcursor::Deselect(){
	if(pPart)		pPart->Deselect();
	if(pMeasure)	pMeasure->Deselect();
	if(pVoice)		pVoice->Deselect();
	if(pFigure)		pFigure->Deselect();
	if(pNote)		pNote->Deselect();
}

void SMRcursor::Select(const curDepth &level){

	if(!level)
		Deselect();

	if(level > GetDepth())
		return;

	switch (level){
		case D_PART :
			if(pPart) pPart->Select();
			break;
		case D_MEASURE :
			if(pMeasure) pMeasure->Select();
			break;
		case D_VOICE :
			if(pVoice) pVoice->Select(/*pMeasure->GetAbsPos() + pMeasure->GetVU2Rgt()/2*/);
			break;
		case D_FIGURE :
			if(pFigure) pFigure->Select();
			break;
		case D_NOTE :
			if(pNote) pNote->Select();
			break;
	}
}

const curDepth SMRcursor::GetDepth() {
	if(pNote)
		return D_NOTE;
	if(pFigure)
		return D_FIGURE;
	if(pVoice)
		return D_VOICE;
	if(pMeasure)
		return D_MEASURE;
	if(pPart)
		return D_PART;

	return D_EMPTY;
}

Partitura* const & SMRcursor::GetScore(){
	return pScore;
}

Spartito* const & SMRcursor::GetPart() {
	return pPart;
}

Battuta* const & SMRcursor::GetMeasure() {
	return pMeasure;
}

Layer* const & SMRcursor::GetVoice() {
	return pVoice;
}

Figura* const & SMRcursor::GetFigure() {
	return pFigure;
}

Nota* const & SMRcursor::GetNote() {
	return pNote;
}

void SMRcursor::SetPart(Spartito* const & p) {
	pPart = p;
}

void SMRcursor::SetMeasure(Battuta* const & m) {
	pMeasure = m;
}

void SMRcursor::SetVoice(Layer* const & v) {
	pVoice = v;
}

void SMRcursor::SetFigure(Figura* const & f) {
	pFigure = f;
}

void SMRcursor::SetNote(Nota* const & n) {
	pNote = n;
}

void SMRcursor::GoFirst(const curDepth &level){
	if(!level || !pScore)
		return;

	if(level == D_PART || (!pPart && level > D_PART)) {
		pPart = pScore->GetFirstSpa();
		Reset(D_MEASURE);
	}

	if(level == D_MEASURE || (!pMeasure && level > D_MEASURE)) {
		pMeasure = pPart->GetFirstBat();
		if (!pMeasure) {
			GoNext(D_PART);
			pMeasure = pPart->GetFirstBat();
		}		
		Reset(D_VOICE);
	}

	if(level == D_VOICE || (!pVoice && level > D_VOICE)) {
		pVoice = pMeasure->GetFirstLayer();//pVoice = pMeasure->GetTopLayer(); //
		while(!pVoice)/*if(!pVoice)*/{
			GoNext(D_MEASURE);
			pVoice = pMeasure->GetFirstLayer();//pVoice = pMeasure->GetTopLayer(); //
			if(pMeasure == pPart->GetLastBat())
				break;
		}

		if(!pVoice){ //spartito vuoto
			Reset();
			return;
		}

		Reset(D_FIGURE);
	}

	if(level == D_FIGURE || (!pFigure  && level > D_FIGURE)) {
		pFigure = pVoice->GetFirstFig();
		if(!pFigure){
			GoNext(D_VOICE);
			pFigure = pVoice->GetFirstFig();
		}
		Reset(D_NOTE);
	}

	if(level == D_NOTE || (!pNote && level > D_NOTE)) {
		if(pFigure->GetID() == CL_ACCORDO)
			pNote = ((Accordo*)pFigure)->GetFirstNote();

		if(!pNote){
			GoNext(D_FIGURE);
			if(pFigure->GetID() == CL_ACCORDO)
				pNote = ((Accordo*)pFigure)->GetFirstNote();
		}
	}


	if(GetDepth() != level)
		Reset();
}

void SMRcursor::GoLast(const curDepth &level){
	if(!level || !pScore)
		return;

	if(level == D_PART || (!pPart && level > D_PART)) {
		pPart = pScore->GetLastSpa();
		Reset(D_MEASURE);
	}

	if(level == D_MEASURE || (!pMeasure && level > D_MEASURE)) {
		pMeasure = pPart->GetLastBat();
		if (!pMeasure) {
			GoPrev(D_PART);
			pMeasure = pPart->GetLastBat();
		}		
		Reset(D_VOICE);
	}

	if(level == D_VOICE || (!pVoice && level > D_VOICE)) {
		pVoice = pMeasure->GetLastLayer();//pVoice = pMeasure->GetBottomLayer(); //
		/*if(!pVoice)*/while(!pVoice){
			GoPrev(D_MEASURE);
			pVoice = pMeasure->GetLastLayer();//pVoice = pMeasure->GetBottomLayer(); //
			if(pMeasure == pPart->GetFirstBat())
				break;
		}

		if(!pVoice){ //spartito vuoto
			Reset();
			return;
		}

		Reset(D_FIGURE);
	}

	if(level == D_FIGURE || (!pFigure  && level > D_FIGURE)) {
		pFigure = pVoice->GetLastFig();
		if(!pFigure){
			GoPrev(D_VOICE);
			pFigure = pVoice->GetLastFig();
		}
		Reset(D_NOTE);
	}

	if(level == D_NOTE || (!pNote && level > D_NOTE)) {
		if(pFigure->GetID() == CL_ACCORDO)
			pNote = ((Accordo*)pFigure)->GetLastNote();

		if(!pNote){
			GoPrev(D_FIGURE);
			if(pFigure->GetID() == CL_ACCORDO)
				pNote = ((Accordo*)pFigure)->GetLastNote();
		}
	}


	if(GetDepth() != level)
		Reset();
}

void SMRcursor::GoNext(const curDepth &level){
	if(!level || level > GetDepth())
		return;

	//Spartito *	tmpPart			=pPart;
	//Battuta *	tmpMeasure		=pMeasure;
	//Layer *		tmpVoice		=pVoice;
	//Figura *	tmpFigure		=pFigure;
	//Nota *		tmpNote			=pNote;

	switch (level){
		case D_NOTE :
			if(pFigure && pFigure->GetID() == CL_ACCORDO){
				pNote = ((Accordo*)pFigure)->GetNextNote(pNote);
				if (pNote)
					break;

				GoNext(D_FIGURE);
				while(pFigure){
					if(pFigure->GetID() == CL_ACCORDO){
						GoFirst(D_NOTE);	
						break;
					}
					GoNext(D_FIGURE);
				}
			}
			break;
		case D_FIGURE :
			Reset(D_NOTE);
			if(pVoice){
				pFigure = pVoice->GetNextFig(pFigure);
				if (pFigure)
					break;

				GoNext(D_VOICE);
				while(pVoice){
					if(pVoice->Calc_Fig_Layer()){
						GoFirst(D_FIGURE);	
						break;
					}
					GoNext(D_VOICE);
				}				
			}
			break;
		case D_VOICE :
			Reset(D_FIGURE);
			if(pMeasure){
				pVoice = pMeasure->GetNextLayer(pVoice);//pVoice = pMeasure->GetMinorLayer(pVoice); //
				if (pVoice)
					break;

				GoNext(D_MEASURE);
				while (pMeasure){
					if(pMeasure->GetNUsedLayers()){
						GoFirst(D_VOICE);
						break;
					}
					GoNext(D_MEASURE);
				}
			}
			break;
		case D_MEASURE :
			Reset(D_VOICE);
			if(pPart){
				pMeasure = pPart->GetNextBat(pMeasure);
				if (pMeasure)
					break;

				GoNext(D_PART);
				while (pPart){
					if(pPart->GetNumeroBattuteVere()){
						GoFirst(D_MEASURE);
						break;

					}
					GoNext(D_PART);
				}			
			}
			break;
		case D_PART :
			Reset(D_MEASURE);
			if(pScore) 
				pPart = pScore->GetNextSpa(pPart);
			break;
	}
}

void SMRcursor::GoPrev(const curDepth &level){
	if(!level || level > GetDepth())
		return;


	switch (level){
		case D_NOTE :
			if(pFigure && pFigure->GetID() == CL_ACCORDO){
				pNote = ((Accordo*)pFigure)->GetPrevNote(pNote);
				if (pNote)
					break;

				GoPrev(D_FIGURE);
				while(pFigure){
					if(pFigure->GetID() == CL_ACCORDO){
						GoLast(D_NOTE);	
						break;
					}
					GoPrev(D_FIGURE);
				}
			}
			break;
		case D_FIGURE :
			Reset(D_NOTE);
			if(pVoice){
				pFigure = pVoice->GetPrevFig(pFigure);
				if (pFigure)
					break;

				GoPrev(D_VOICE);
				while(pVoice){
					if(pVoice->Calc_Fig_Layer()){
						GoLast(D_FIGURE);	
						break;
					}
					GoPrev(D_VOICE);
				}				
			}
			break;
		case D_VOICE :
			Reset(D_FIGURE);
			if(pMeasure){
				pVoice = pMeasure->GetPrevLayer(pVoice);//pVoice = pMeasure->GetMajorLayer(pVoice); //
				if (pVoice)
					break;

				GoPrev(D_MEASURE);
				while (pMeasure){
					if(pMeasure->GetNUsedLayers()){
						GoLast(D_VOICE);
						break;
					}
					GoPrev(D_MEASURE);
				}
			}
			break;
		case D_MEASURE :
			Reset(D_VOICE);
			if(pPart){
				pMeasure = pPart->GetPrevBat(pMeasure);
				if (pMeasure)
					break;

				GoPrev(D_PART);
				while (pPart){
					if(pPart->GetNumeroBattuteVere()){
						GoLast(D_MEASURE);
						break;

					}
					GoPrev(D_PART);
				}			
			}
			break;
		case D_PART :
			Reset(D_MEASURE);
			if(pScore) 
				pPart = pScore->GetPrevSpa(pPart);
			break;
	}
}


void SMRcursor::GoLeft(const curDepth &level){
	if(!GetDepth())
	{
		GoFirst(level);
		return;
	}

	if(!level || (level > GetDepth() && GetDepth() < D_FIGURE))
		return;

	unsigned int pos;
	
	switch (level){
		case D_NOTE :
			pos = GetPosition(D_NOTE);
			GoLeft(D_FIGURE);
			while(!pFigure->GetDurata())
				GoLeft(D_FIGURE);
			
			if(pFigure->GetID() == CL_ACCORDO) {
				if(pos && pos <= ((Accordo*)pFigure)->GetNumObj())
					SetPosition(D_NOTE, pos);
				else
					SetPosition(D_NOTE, ((Accordo*)pFigure)->GetNumObj());
			}
			break;
		case D_FIGURE :
			if(GetPosition(D_FIGURE) != 1)
				GoPrev(D_FIGURE);
			else //if (GetPosition(D_MEASURE) > 1)
			{
				GoLeft(D_VOICE);
				GoLast(D_FIGURE);
			}
			break;
		case D_VOICE :	//TODO: controllare che il Layer ci sia
			if(pos = GetPosition(D_VOICE)){
				GoLeft(D_MEASURE);
				while(!SetPosition(D_VOICE, pos)){//TODO potrebbe andare in loop infinito
					GoLeft(D_MEASURE);//GoRight(D_MEASURE);
					//SetPosition(D_VOICE, pos);
				}
			}
			break;
		case D_MEASURE :
			if (GetPosition(D_MEASURE) > 1)
				GoPrev(D_MEASURE);
			else
				GoLast(D_MEASURE);
			break;
		case D_PART :
			GoPrev(D_PART);
			break;
	}
}

void SMRcursor::GoRight(const curDepth &level){
	if(!GetDepth())
	{
		GoLast(level);
		return;
	}

	if(!level || (level > GetDepth() && GetDepth() < D_FIGURE))
		return;

	unsigned int pos;

	switch (level){
		case D_NOTE :
			pos = GetPosition(D_NOTE);
			GoRight(D_FIGURE);
			while(!pFigure->GetDurata())
				GoRight(D_FIGURE);

			if(pFigure->GetID() == CL_ACCORDO) {
				if(pos && pos <= ((Accordo*)pFigure)->GetNumObj())
					SetPosition(D_NOTE, pos);
				else
					SetPosition(D_NOTE, ((Accordo*)pFigure)->GetNumObj());
			}
			break;
		case D_FIGURE :
			if(GetPosition(D_FIGURE) != GetCount(D_FIGURE))
				GoNext(D_FIGURE);
			else //if (GetPosition(D_MEASURE) < GetCount(D_MEASURE))
			{
				GoRight(D_VOICE);
				GoFirst(D_FIGURE);
			}
			break;
		case D_VOICE :
			if(pos = GetPosition(D_VOICE)){
				GoRight(D_MEASURE);
				while(!SetPosition(D_VOICE, pos)){ //TODO potrebbe andare in loop infinito
					GoRight(D_MEASURE);//GoLeft(D_MEASURE);
					//SetPosition(D_VOICE, pos);
				}
			}
			break;
		case D_MEASURE :
			if (GetPosition(D_MEASURE) < GetCount(D_MEASURE))
				GoNext(D_MEASURE);
			else
				GoFirst(D_MEASURE);
			break;
		case D_PART :
			GoNext(D_PART);
			break;
	}
}

void SMRcursor::GoUp(const curDepth &level){
	if(!GetDepth())
	{
		GoFirst(level);
		return;
	}

	if(!level || (level > GetDepth() && GetDepth() < D_FIGURE))
		return;

	unsigned int pos;
	Figura* pFig;
	float dur = 0.0;

	switch (level){
		case D_NOTE :
			if(pNote && pNote != ((Accordo*)pFigure)->GetLastNote())
				pNote = ((Accordo*)pFigure)->GetNextNote(pNote);
			else
			{
				GoUp(D_FIGURE);
				if(pFigure->GetID() == CL_ACCORDO)
					pNote = ((Accordo*)pFigure)->GetFirstNote();
			}
			break;
		case D_FIGURE :
			pFig = pVoice->GetFirstFig();
			while (pFig)
			{
				if(pFig == pFigure)
					break;
				dur += pFig->GetDurata();
				pFig = pVoice->GetNextFig(pFig);
			}
			
			GoUp(D_VOICE);

			pFig = pVoice->GetFirstFig();
			while (pFig)
			{
				dur -= pFig->GetDurata();
				if(dur </*=*/ 0.0)
					break;
				pFig = pVoice->GetNextFig(pFig);
			}
			if(!pFig)
				pFig = pVoice->GetLastFig();

			SetFigure(pFig);

			break;
		case D_VOICE :
			if(GetVoiceGraphicPosition() > 1) {//GoPrev(D_VOICE);
				Reset(D_FIGURE);
				if(pMeasure){
					pVoice = pMeasure->GetMajorLayer(pVoice); //pVoice = pMeasure->GetPrevLayer(pVoice);
					if (pVoice)
						break;

					GoPrev(D_MEASURE);
					while (pMeasure){
						if(pMeasure->GetNUsedLayers()){
							GoLast(D_VOICE);
							break;
						}
						GoPrev(D_MEASURE);
					}
				}
			}		
			else //if (GetPosition(D_PART) > 1)
			{
				GoUp(D_MEASURE);
				//GoLast(D_VOICE);
				pVoice = pMeasure->GetBottomLayer();
				if(!pVoice){
					GoPrev(D_MEASURE);
					pVoice = pMeasure->GetBottomLayer();
				}
				Reset(D_FIGURE);
			}
			break;
		case D_MEASURE :
			if(pos = GetPosition(D_MEASURE)) {
				GoUp(D_PART);
				SetPosition(D_MEASURE, pos);
			}
			break;
		case D_PART :
			GoPrev(D_PART);
			if(!GetDepth())
				GoLast(D_PART);
			break;
	}
}

void SMRcursor::GoDown(const curDepth &level){
	if(!GetDepth())
	{
		GoLast(level);
		return;
	}

	if(!level || (level > GetDepth() && GetDepth() < D_FIGURE))
		return;

	unsigned int pos;
	Figura* pFig;
	float dur = 0.0;

	switch (level){
		case D_NOTE :
			if(pNote && pNote != ((Accordo*)pFigure)->GetFirstNote())
				pNote = ((Accordo*)pFigure)->GetPrevNote(pNote);
			else
			{
				GoDown(D_FIGURE);
				if(pFigure->GetID() == CL_ACCORDO)
					pNote = ((Accordo*)pFigure)->GetLastNote();
			}
			break;
		case D_FIGURE :
			pFig = pVoice->GetFirstFig();
			while (pFig)
			{
				if(pFig == pFigure)
					break;
				dur += pFig->GetDurata();
				pFig = pVoice->GetNextFig(pFig);
			}

			GoDown(D_VOICE);

			pFig = pVoice->GetFirstFig();
			while (pFig)
			{
				dur -= pFig->GetDurata();
				if(dur </*=*/ 0.0)
					break;
				pFig = pVoice->GetNextFig(pFig);
			}
			if(!pFig)
				pFig = pVoice->GetLastFig();

			SetFigure(pFig);
			break;
		case D_VOICE :
			if(GetVoiceGraphicPosition() != GetCount(D_VOICE)) {//GoNext(D_VOICE);
				Reset(D_FIGURE);
				if(pMeasure){
					pVoice = pMeasure->GetMinorLayer(pVoice);
					if (pVoice)
						break;

					GoNext(D_MEASURE);
					while (pMeasure){
						if(pMeasure->GetNUsedLayers()){
							GoFirst(D_VOICE);
							break;
						}
						GoNext(D_MEASURE);
					}
				}
			}		
			else //if (GetPosition(D_PART) < GetCount(D_PART))
			{
				GoDown(D_MEASURE);
				//GoFirst(D_VOICE);
				pVoice = pMeasure->GetTopLayer();
				if(!pVoice){
					GoNext(D_MEASURE);
					pVoice = pMeasure->GetTopLayer();
				}
				Reset(D_FIGURE);
			}
			break;
		case D_MEASURE :
			if(pos = GetPosition(D_MEASURE)) {
				GoDown(D_PART);
				SetPosition(D_MEASURE, pos);
			}
			break;
		case D_PART :
			GoNext(D_PART);
			if(!GetDepth())
				GoFirst(D_PART);
			break;
	}
}

unsigned int SMRcursor::GetPosition(const curDepth &level){
	if(!level || level > GetDepth())
		return 0;

	unsigned int index = 0;
	void* p_ob;

	switch (level){
		case D_NOTE :
			p_ob = ((Accordo*)pFigure)->GetFirstNote();
			while(p_ob){
				index++;
				if( ((Nota*)p_ob) == pNote)
					return index;
				p_ob = ((Accordo*)pFigure)->GetNextNote((Nota*)p_ob);
			}
			break;

		case D_FIGURE :
			p_ob = pVoice->GetFirstFig();
			while(p_ob){
				index++;
				if( ((Figura*)p_ob) == pFigure)
					return index;
				p_ob = pVoice->GetNextFig((Figura*)p_ob);
			}
			break;

		case D_VOICE :
			p_ob = pMeasure->GetFirstLayer();
			while(p_ob){
				index++;
				if( ((Layer*)p_ob) == pVoice)
					return index;
				p_ob = pMeasure->GetNextLayer((Layer*)p_ob);
			}
			break;

		case D_MEASURE :
			p_ob = pPart->GetFirstBat();
			while(p_ob){
				index++;
				if( ((Battuta*)p_ob) == pMeasure)
					return index;		
				p_ob = pPart->GetNextBat((Battuta*)p_ob);
			}
			break;

		case D_PART :
			p_ob = pScore->GetFirstSpa();
			while(p_ob){
				index++;
				if( ((Spartito*)p_ob) == pPart)
					return index;
				p_ob = pScore->GetNextSpa((Spartito*)p_ob);
			}
			break;
	}

	return 0;
}

unsigned int SMRcursor::GetVoiceGraphicPosition(){
	if(D_VOICE > GetDepth())
		return 0;

	unsigned int index = 0;
	Layer* p_ob;

	p_ob = pMeasure->GetTopLayer(); //pMeasure->GetFirstLayer();
	while(p_ob){
		index++;
		if(p_ob == pVoice)
			break;
		p_ob = pMeasure->GetMinorLayer(p_ob); //pMeasure->GetNextLayer((Layer*)p_ob);
	}
	return index;
}

const int SMRcursor::SetPosition(const curDepth &level, const unsigned int & pos) {

	if(level > GetDepth() + 1)
		return 0;

	if (pos < 1 || pos > GetCount(level))
		return 0;

	Figura* pFig;
	Layer* pLay;
	Nota* pNot;
	int count;

	switch(level)
	{
	case D_EMPTY:
		Reset(D_PART);
		break;
	case D_PART:
		SetPart(pScore->GetSpartNum(pos-1));
		Reset(D_MEASURE);
		break;
	case D_MEASURE:
		SetMeasure(pPart->GetBattNum(pos-1));
		Reset(D_VOICE);
		break;
	case D_VOICE:
		count = 0;
		pLay = pMeasure->GetFirstLayer();
		while (pLay)
		{
			count++;
			if(pos == count)
			{
				SetVoice(pLay);
				break;
			}
			pLay = pMeasure->GetNextLayer(pLay);
		}	
		Reset(D_FIGURE);
		break;
	case D_FIGURE:
		count = 0;
		pFig = pVoice->GetFirstFig();
		while(pFig){
			count++;
			if(pos == count)
			{
				SetFigure(pFig);
				break;
			}
			pFig = pVoice->GetNextFig(pFig);
		}
		Reset(D_NOTE);
		break;
	case D_NOTE:
		count = 0;
		if(pFigure->GetID()!= CL_ACCORDO)
			return 0;
		pNot = ((Accordo*)pFigure)->GetFirstNote();
		while(pNot){
			count++;
			if(pos == count)
			{
				SetNote(pNot);
				break;
			}
			pNot = ((Accordo*)pFigure)->GetNextNote(pNot);
		}
		break;
	}

	return 1;
}



const int SMRcursor::SetVoiceGraphicPosition(const unsigned int & pos){
	if(D_VOICE > GetDepth() + 1)
		return 0;

	if (pos < 1 || pos > GetCount(D_VOICE))
		return 0;

	int count = 0;

	Layer* pLay = pMeasure->GetTopLayer(); //pMeasure->GetFirstLayer();
	while (pLay)
	{
		count++;
		if(pos == count)
		{
			SetVoice(pLay);
			break;
		}
		pLay = pMeasure->GetMinorLayer(pLay); //pMeasure->GetNextLayer(pLay);
	}	
	Reset(D_FIGURE);
	return 1;
}
const int SMRcursor::GetCount(const curDepth &level) {

	if(!level || level > GetDepth() + 1)
		return 0;

	Figura* pFig;
	int fig_count;

	switch(level)
	{
	case D_PART:
		return pScore->GetNumSpa();
	case D_MEASURE:
		return pPart->GetNumeroBattute();
	case D_VOICE:
		return pMeasure->GetNUsedLayers();
	case D_FIGURE:
		fig_count = 0;
		pFig = pVoice->GetFirstFig();
		while(pFig)
		{
			fig_count++;
			pFig = pVoice->GetNextFig(pFig);
		}
	    return fig_count;
	case D_NOTE:
		return ((Accordo*)pFigure)->GetNumObj();
	}

	return 0;
}

//----------------------------------------------------------//

//void Highlight(bool val){
//	highlighted = val;
//}

void SMRcursor::SetColor(const Colore & color, const curDepth &level) {
	if(!level || (level > GetDepth() && GetDepth() < D_FIGURE))
		return;

	bool opaque = !(color.GetCode() == "FFFFFF");

	switch(level)
	{
	case D_PART:
		(opaque ? pPart->SetColor(color) : pPart->UnsetColor());
		return;
	case D_MEASURE:
		(opaque ? pMeasure->SetColor(color) : pMeasure->UnsetColor());
		return;
	case D_VOICE:
		(opaque ? pVoice->SetColor(color) : pVoice->UnsetColor());
		return;
	case D_FIGURE:
		(opaque ? pFigure->SetColor(color) : pFigure->UnsetColor());
		return;
	case D_NOTE:
		if(pNote)
			(opaque ? pNote->SetColor(color) : pNote->UnsetColor());
		else
			(opaque ? pFigure->SetColor(color) : pFigure->UnsetColor());
		return;
	}
}

//void SMRcursor::GoIn(curDepth &level){
//	if(level < D_NOTE) {
//		if(level == D_FIGURE && !pNote)
//			return;
//		level = (curDepth)(level + 1);
//		GoFirst(level);
//	}
//}
//
//void SMRcursor::GoOut(curDepth &level){
//	if(level > D_PART) {
//		Reset(level);
//		level = (curDepth)(level - 1);
//	}
//}
