/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: abbellim.cpp,v 1.1 2005/07/25 10:09:01 bellinpo Exp $ */

#include "abb/abbellim.hpp"
#include "liooext.hpp"
#include "gui/dialogs.hpp" // per finestra debug milla

extern int DebugNormalMilla;
extern int DebugNormalMillaPos;
extern wxString msgdebug;


Abbellimento::Abbellimento()
{ aboveNota=TRUE;
PAltSopra=NULL;
PAltSotto=NULL;
Doppio=FALSE;
}

Abbellimento::~Abbellimento()
{
	delete PAltSopra;
	delete PAltSotto;
} 

Bool Abbellimento::GetAboveNota()
{ return aboveNota; };

void Abbellimento::SetAboveNota(Bool above)
{ aboveNota=above;};

void Abbellimento::SetAboveNotaAuto(TipoInserimento tpIns,Nota* pNota,Battuta* pBattuta)
{
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		
		
		msgdebug.sprintf("");
		
	}	
	
	
	switch(tpIns)
	{
	case SOPRA:
		aboveNota=TRUE;
		break;
	case SOTTO:
		aboveNota=FALSE;
		break;
	case AUTOMATICO:
		aboveNota = immissione.ImmRule(GetID(), pNota, pBattuta, NULL, 0); // lo zero e'il current_layer //GB
		break;
	}
	
	
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		posizionam.TestTokenCond(pNota, NULL, pBattuta, (int)NULL, 0);
		
		WDebugMilla(); 
	}
	
}


Bool Abbellimento::is(ClassID CLId)
{
	if (CLId==CL_ABBELLIMENTO) return TRUE;
	return DrawObject::is(CLId);
}

void Abbellimento::SetDoppio(Bool doppio)
{
	Doppio = doppio;
};

Bool Abbellimento::GetDoppio()
{
	return Doppio;
}

Alterazione* Abbellimento::GetPAltSopra()
{ return PAltSopra; }

Alterazione* Abbellimento::GetPAltSotto()
{ return PAltSotto; }

void Abbellimento::SetPAltSopra(Alterazione *p)
{ 
	if(p!=NULL) 
		PAltSopra = p;
	else
		PAltSopra = p;
}

void Abbellimento::SetPAltSotto(Alterazione *p)
{ PAltSotto = p; }


CmdResult Abbellimento::SymCommand(SymCmd& SCmd, Battuta *pBattuta)
{
	int Quale;
	// Quale contiene :
	//	   0 se il comando e' fallito
	//    1 se l'alterazione e' stata inserita nel primo puntatore
	//    2 se l'alterazione e' stata inserita nel secondo puntatore
	
	switch(SCmd.CommandId)
	{
	case CMD_NOTA_DIESIS: 
		Quale=CmdAbbDiesis((TipoInserimento) SCmd.ArgV[0]); break;
	case CMD_NOTA_DIESIS1Q: 
		Quale=CmdAbbDiesis1Q((TipoInserimento) SCmd.ArgV[0]); break;
	case CMD_NOTA_DIESIS3Q: 
		Quale=CmdAbbDiesis3Q((TipoInserimento) SCmd.ArgV[0]); break;
	case CMD_NOTA_2DIESIS: 
		Quale=CmdAbb2Diesis((TipoInserimento) SCmd.ArgV[0]); break;
	case CMD_NOTA_BEMOLLE: 
		Quale=CmdAbbBemolle((TipoInserimento) SCmd.ArgV[0]); break;
	case CMD_NOTA_BEMOLLE1Q: 
		Quale=CmdAbbBemolle1Q((TipoInserimento) SCmd.ArgV[0]); break;
	case CMD_NOTA_BEMOLLE3Q: 
		Quale=CmdAbbBemolle3Q((TipoInserimento) SCmd.ArgV[0]); break;
	case CMD_NOTA_2BEMOLLE: 
		Quale=CmdAbb2Bemolle((TipoInserimento) SCmd.ArgV[0]); break;
	case CMD_NOTA_BEQUADRO: 
		Quale=CmdAbbBequadro((TipoInserimento) SCmd.ArgV[0]); break;   
	case CMD_NOTA_BEQUADRO1Q: 
		Quale=CmdAbbBequadro1Q((TipoInserimento) SCmd.ArgV[0]); break;   
	case CMD_NOTA_BEQUADRO3Q: 
		Quale=CmdAbbBequadro3Q((TipoInserimento) SCmd.ArgV[0]); break;   
	}
	SCmd.ArgV[6]=Quale;
	if (Quale==0) return(CMD_UNKNOWN);    
	return CMD_OK;
} 

int Abbellimento::CmdAbbDiesis(TipoInserimento AboveAbb)
{
	return InserisciAlt(CL_DIESIS,AboveAbb);
}

int Abbellimento::CmdAbbDiesis1Q(TipoInserimento AboveAbb)
{
	return InserisciAlt(CL_DIESIS1Q,AboveAbb);
}

int Abbellimento::CmdAbbDiesis3Q(TipoInserimento AboveAbb)
{
	return InserisciAlt(CL_DIESIS3Q,AboveAbb);
}

int Abbellimento::CmdAbb2Diesis(TipoInserimento AboveAbb)
{
	return InserisciAlt(CL_DOPPIODIESIS,AboveAbb);
}

int Abbellimento::CmdAbbBemolle(TipoInserimento AboveAbb)
{
	return InserisciAlt(CL_BEMOLLE,AboveAbb);
}

int Abbellimento::CmdAbbBemolle1Q(TipoInserimento AboveAbb)
{
	return InserisciAlt(CL_BEMOLLE1Q,AboveAbb);
}

int Abbellimento::CmdAbbBemolle3Q(TipoInserimento AboveAbb)
{
	return InserisciAlt(CL_BEMOLLE3Q,AboveAbb);
}

int Abbellimento::CmdAbb2Bemolle(TipoInserimento AboveAbb)
{
	return InserisciAlt(CL_DOPPIOBEMOLLE,AboveAbb);
}

int Abbellimento::CmdAbbBequadro(TipoInserimento AboveAbb)
{
	return InserisciAlt(CL_BEQUADRO,AboveAbb);
}

int Abbellimento::CmdAbbBequadro1Q(TipoInserimento AboveAbb)
{
	return InserisciAlt(CL_BEQUADRO1Q,AboveAbb);
}

int Abbellimento::CmdAbbBequadro3Q(TipoInserimento AboveAbb)
{
	return InserisciAlt(CL_BEQUADRO3Q,AboveAbb);
}

Bool Abbellimento::Hit(const Point& p, SelObj selObj, DrawObject*& drwObjSel,
					   SymPath& symPath)
{
	if(PAltSopra!=NULL &&
		PAltSopra->Hit(p,selObj,drwObjSel,symPath))
		return TRUE;
	if(PAltSotto!=NULL &&
		PAltSotto->Hit(p,selObj,drwObjSel,symPath))
		return TRUE;
	if(DrawObject::Hit(p,selObj,drwObjSel,symPath))
		return TRUE;  
	return FALSE;
}   	       

Bool Abbellimento::SymbolicHit(SymPath& symPath, SelObj selObj, 
							   DrawObject*& drwObjSel)
{
	if(PAltSopra!=NULL &&
		PAltSopra->SymbolicHit(symPath, selObj, drwObjSel))
		return TRUE;
	if(PAltSotto!=NULL &&
		PAltSotto->SymbolicHit(symPath, selObj, drwObjSel))
		return TRUE;
	if(DrawObject::SymbolicHit(symPath, selObj, drwObjSel))
		return TRUE; 
	return FALSE;
} 	    


Bool Abbellimento::DeleteSym(DrawObject *sym)
{
	if(EliminaAlt(sym))
		return TRUE; 
	else return FALSE; 
}       	   

DrawObject * Abbellimento::GetAlterazione(SymCmd *MyCmd)
{
	// ricerco tra le dua alterazioni quella effetivamente inserita dal comando simbolico
	// ArgV[6] contiene
	//		1 - Sopra
	//     2 - Sotto
	if (MyCmd->ArgV[6]==1) return PAltSopra;
	else return PAltSotto;
}

void Abbellimento::Edit(Battuta *pBat)
{
    EditingMarkersDlg(this);
}  
