/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: abbellim.hpp,v 1.1 2005/07/25 10:09:01 bellinpo Exp $ */

#ifndef ABBELLIMENTO_HPP
#define ABBELLIMENTO_HPP

#include "drw/drawobje.hpp"
#include "alt/alterazi.hpp"
#include "alt/diesis.hpp"
#include "alt/diesis1q.hpp"
#include "alt/diesis3q.hpp"
#include "alt/doppiodi.hpp"
#include "alt/bemolle.hpp"
#include "alt/bemoll1q.hpp"
#include "alt/bemoll3q.hpp"
#include "alt/doppiobe.hpp"
#include "alt/bequadro.hpp"
#include "alt/bequadro1q.hpp"
#include "alt/bequadro3q.hpp"

class Nota;
class Battuta;

class Abbellimento : public DrawObject

{ 
private:
    int CmdAbbDiesis(TipoInserimento AboveAbb);
	//Assegna all'abbellimento l'alterazione diesis.
    int CmdAbbDiesis1Q(TipoInserimento AboveAbb);
	//Assegna all'abbellimento l'alterazione diesis1Q.
    int CmdAbbDiesis3Q(TipoInserimento AboveAbb);
	//Assegna all'abbellimento l'alterazione diesis3Q.
    int CmdAbb2Diesis(TipoInserimento AboveAbb);
	//Assegna all'abbellimento l'alterazione doppiodiesis.      
    int CmdAbbBemolle(TipoInserimento AboveAbb);
	//Assegna all'abbellimento l'alterazione bemolle.
    int CmdAbbBemolle1Q(TipoInserimento AboveAbb);
	//Assegna all'abbellimento l'alterazione bemolle1Q.
    int CmdAbbBemolle3Q(TipoInserimento AboveAbb);
	//Assegna all'abbellimento l'alterazione bemolle3Q.
    int CmdAbb2Bemolle(TipoInserimento AboveAbb);
	//Assegna all'abbellimento l'alterazione doppiobemolle.  
    int CmdAbbBequadro(TipoInserimento AboveAbb);
	//Assegna all'abbellimento l'alterazione bequadro.
    int CmdAbbBequadro1Q(TipoInserimento AboveAbb);
	//Assegna all'abbellimento l'alterazione bequadro1Q.
    int CmdAbbBequadro3Q(TipoInserimento AboveAbb);
	//Assegna all'abbellimento l'alterazione bequadro3Q.
	
protected:  
    Bool aboveNota;
    Bool Doppio;
	//Attributo per indicare il tipo di Mordente:
	//TRUE se e' doppio, FALSE altrimenti.  
    Alterazione *PAltSopra;
	//Puntatore all'eventuale alterazione sopra all'abbellimento.
    Alterazione *PAltSotto;
	//Puntatore all'eventuale alterazione sotto all'abbellimento.  
	
	
public: 
    Abbellimento();
    ~Abbellimento();
    
    virtual int InserisciAlt(ClassID,TipoInserimento)
    { return FALSE; };
    virtual Bool EliminaAlt(DrawObject*)
    { return FALSE; };
	
	DrawObject * GetAlterazione(SymCmd *MyCmd);
    
    Bool GetAboveNota();
    void SetAboveNota(Bool);
	
	void SetAboveNotaAuto(TipoInserimento tpIns, Nota* pNota, Battuta* pBattuta);
    Bool is(ClassID);
    // dice se l'oggetto puntato e' un abbellimento              
	
    
    void SetDoppio(Bool);
    Bool GetDoppio();
    
    Alterazione *GetPAltSopra();       
    Alterazione *GetPAltSotto();
    void SetPAltSopra(Alterazione *);
    void SetPAltSotto(Alterazione *);
	//Metodi per accedere a PAltSopra e a PAltSotto da Nota.    
    
    void Draw(){};
    
    CmdResult SymCommand(SymCmd&, Battuta *pb=NULL);
    
    Bool Hit(const Point&, SelObj, DrawObject*&, SymPath&);
    Bool SymbolicHit(SymPath&, SelObj, DrawObject*&);
    
    Bool DeleteSym(DrawObject *sym);
	
	virtual void Edit(Battuta *pBat);

	int SaveXML(FILE *fp, char *type, int num=-1);
	virtual int SaveXML(FILE *fp)=0;
};


#endif //ABBELLIMENTO_HPP
