/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: glissato.cpp,v 1.1 2005/07/25 10:09:02 bellinpo Exp $ */


#include "abb/glissato.hpp"
#include "liooext.hpp"

Glissato::Glissato()
{ ID=CL_GLISSATO; Glissonda=FALSE; }

Glissato::Glissato(Bool b)
{ ID=CL_GLISSATO; SetGlissonda(b); }

void Glissato::SetPos(const Point& PLft,const Point& PRgt)
{
	VUnit h2;
	
	if(PLft.x >= PRgt.x)
	{
		PointTo = PLft;
		GetAbsPos() = PRgt;
	}
	else
	{
		PointTo = PRgt;
		GetAbsPos() = PLft;
	}
	
	
	h2 = (VUnit)(ceil((float)(3)));
	if(GetAbsPos().y < PointTo.y)
	{
		VU2Up =  PointTo.y - GetAbsPos().y + h2;
		VU2Dwn = h2;
	}
	else
	{
		VU2Dwn = - PointTo.y + GetAbsPos().y + h2;
		VU2Up = h2;
	}   
	
}


VUnit Glissato::GetVU2Up()
{
	return VU2Up;
};

VUnit Glissato::GetVU2Dwn()
{
	return VU2Dwn;
};

VUnit Glissato::GetVU2Lft()
{
	return 0;
};

VUnit Glissato::GetVU2Rgt()
{
	return PointTo.x-GetAbsPos().x;
};

Bool Glissato::GetGlissonda() {return Glissonda;};

void Glissato::SetGlissonda(Bool b)
{
	Glissonda = b;
};


void Glissato::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	if (GetGlissonda())
	{
		if (PointTo==GetAbsPos())
			return;
		VUnit ax = GetAbsPos().x ;
		VUnit ay = GetAbsPos().y ;
		VUnit bx = PointTo.x;
		VUnit by = PointTo.y;
		int T = (bx - ax)/2;
		VUnit asc[100];
		VUnit ord[100];
		for (int i=0;i<=bx-ax;i++)
		{
			asc[i] = i;
			ord[i] = (3*sin(2*3.14*asc[i]/T) + (by-ay)*asc[i]/(bx-ax));
		}
		for (i=0;i<bx-ax;i++)
		{
			Scr->BigLine(GetAbsPos() + Point(asc[i],ord[i]) + Point(DELTAXGLISSATOONDA,DELTAYGLISSATOONDA), 
				GetAbsPos() + Point(asc[i+1],ord[i+1]) + Point(DELTAXGLISSATOONDA,DELTAYGLISSATOONDA), 2);
		}
	}
	else
	{	
		Scr->BigLine(GetAbsPos() + Point(DELTAXGLISSATO,DELTAYGLISSATO), PointTo + Point(DELTAXGLISSATO,DELTAYGLISSATO), 2); 
	}
	Scr->SetColor(oldcol);
};  


