/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: mordente.cpp,v 1.1 2005/07/25 10:09:04 bellinpo Exp $ */


#include "abb/mordente.hpp"
#include "drw/gscreen.hpp"


int Mordente::InserisciAlt(ClassID CL_ALT, TipoInserimento Ssa)
{
	Bool AboveAbb;
	
	switch(Ssa)
	{
	case SOPRA:
	case AUTOMATICO:
		AboveAbb=TRUE;
		break;
	case SOTTO:
		AboveAbb=FALSE;
		break;
	}
	
	if(AboveAbb)
	{
		//L'alterazione va sopra al mordente.
		if((PAltSopra==NULL) && (PAltSotto==NULL))
			switch (CL_ALT)
		{                             
	case CL_DIESIS:
		PAltSopra=new Diesis;	  
		return 1; 
	case CL_DIESIS1Q:
		PAltSopra=new Diesis1Q;	  
		return 1; 
	case CL_DIESIS3Q:
		PAltSopra=new Diesis3Q;	  
		return 1; 
	case CL_DOPPIODIESIS:
		PAltSopra=new DoppioDiesis;	  
		return 1;       
	case CL_BEMOLLE:
		PAltSopra=new Bemolle;
		return 1;
	case CL_BEMOLLE1Q:
		PAltSopra=new Bemolle1Q;
		return 1;
	case CL_BEMOLLE3Q:
		PAltSopra=new Bemolle3Q;
		return 1;
	case CL_DOPPIOBEMOLLE:
		PAltSopra=new DoppioBemolle;
		return 1;
	case CL_BEQUADRO:
		PAltSopra=new Bequadro;
		return 1;	  
		}
	}    
	else                                
		//L'alterazione va sotto al mordente. 
		if((PAltSotto==NULL) && (PAltSopra==NULL))
			switch (CL_ALT)
		{                         
	case CL_DIESIS:
		PAltSotto=new Diesis;	  
		return 2; 
	case CL_DIESIS1Q:
		PAltSotto=new Diesis1Q;	  
		return 2; 
	case CL_DIESIS3Q:
		PAltSotto=new Diesis3Q;	  
		return 2; 
	case CL_DOPPIODIESIS:
		PAltSotto=new DoppioDiesis;	  
		return 2;       
	case CL_BEMOLLE:
		PAltSotto=new Bemolle;
		return 2;
	case CL_BEMOLLE1Q:
		PAltSotto=new Bemolle1Q;
		return 2;
	case CL_BEMOLLE3Q:
		PAltSotto=new Bemolle3Q;
		return 2;
	case CL_DOPPIOBEMOLLE:
		PAltSotto=new DoppioBemolle;
		return 2;
	case CL_BEQUADRO:
		PAltSotto=new Bequadro;
		return 2; 	  
		}                                  
		return 0;     
}

Bool Mordente::EliminaAlt(DrawObject *sym)
{
	if(sym==PAltSopra)
	{ 
		delete PAltSopra;
		PAltSopra=NULL;
		return TRUE;    
	}  
	if(sym==PAltSotto) 
	{
		delete PAltSotto;
		PAltSotto=NULL;
		return TRUE;
	}
	return FALSE;
}

void Mordente::Draw(){}

Bool Mordente::is(ClassID CLId)
{
	if (CLId==CL_MORDENTE) return TRUE;
	return Abbellimento::is(CLId);
}
