/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: tremolo.cpp,v 1.1 2005/07/25 10:09:04 bellinpo Exp $ */


#include "abb/tremolo.hpp"
#include "liooext.hpp"

Tremolo::Tremolo()
{
	ID=CL_TREMOLO;
	SetNumBarre(0);
}

void Tremolo::SetNumBarre(short nb)
{
	NBarre = nb;
}

short Tremolo::GetNumBarre()
{
	return NBarre;
}


void Tremolo::SetPos(const Point& PLft,const Point& PRgt)
{
	VUnit h2;
	
	if(PLft.x >= PRgt.x)
	{
		PointTo = PLft;
		GetAbsPos() = PRgt;
	}
	else
	{
		PointTo = PRgt;
		GetAbsPos() = PLft;
	}
	
	
	h2 = (VUnit)(ceil((float)(NBarre*3+(NBarre-1)*
		(7-3))/2));
	if(GetAbsPos().y < PointTo.y)
	{
		VU2Up =  PointTo.y - GetAbsPos().y + h2;
		VU2Dwn = h2;
	}
	else
	{
		VU2Dwn = - PointTo.y + GetAbsPos().y + h2;
		VU2Up = h2;
	}
	
	if ((NBarre % 2) == 0)
	{
		VU2Dwn += - 3;
		VU2Up += 3 + 1;
	}
	{
		VU2Up += 7 + 1;
		VU2Dwn += - 7 - 3;
	}
    
	
}

VUnit Tremolo::GetVU2Lft()
{
	return 0;
}
VUnit Tremolo::GetVU2Dwn()
{
	return VU2Dwn;
}

VUnit Tremolo::GetVU2Rgt()
{
	return  PointTo.x-GetAbsPos().x;
}

VUnit Tremolo::GetVU2Up()
{
	return VU2Up;
}

void Tremolo::Draw()
{
	int i;
	Point pr=GetAbsPos()+Point(DELTAXTREMOLO,DELTAYTREMOLO);
	Point prf=PointTo+Point(DELTAXTREMOLO,DELTAYTREMOLO);
	
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	
	for(i=0;i<NBarre;i++)                 
	{
		switch (i)
		{
		default: 
		case 0:
			Scr->BigLine(pr + Point(0,1*7), prf + Point(0,1*7), 3);
			break;
		case 1:
			Scr->BigLine(pr +Point(0,2*7), prf + Point(0,2*7), 3);
			break;
		case 2:
			Scr->BigLine(pr +Point(0,0*7), 	prf + Point(0,0*7), 3);
			break;
		case 3:
			Scr->BigLine(pr +Point(0,3*7), 	prf + Point(0,3*7), 3);
			break;
		case 4:
			Scr->BigLine(pr +Point(0,-1*7), prf + Point(0,-1*7), 3);
			break;
		case 5:
			Scr->BigLine(pr +Point(0,4*7), prf + Point(0,4*7), 3);
			break;
		case 6:
			Scr->BigLine(pr +Point(0,-2*7), prf + Point(0,-2*7), 3);
			break;
		}  
	}

	Scr->SetColor(oldcol);

}