/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: altecomp.cpp,v 1.1 2005/07/25 10:43:12 bellinpo Exp $ */


#include "alt/altecomp.hpp"

#include "alt/bemolle.hpp"
#include "alt/bemoll1q.hpp"
#include "alt/bemoll3q.hpp"
#include "alt/bequadro.hpp"
#include "alt/diesis.hpp"
#include "alt/diesis1q.hpp"
#include "alt/diesis3q.hpp"
#include "alt/doppiobe.hpp"
#include "alt/doppiodi.hpp"
#include "alt/bequadro1q.hpp"
#include "alt/bequadro3q.hpp"

AlterazComposta::AlterazComposta()
{
	int i;
	n_alterazioni=0;
	for(i=1; i<=N_ALT_MAX; i++)
		alterazioni[i-1]=NULL;
}


AlterazComposta::~AlterazComposta()
{
	int i;
	for(i=1; i<=N_ALT_MAX; i++)
		delete alterazioni[i-1];
}


int AlterazComposta::Getn_alterazioni()
{
	return n_alterazioni;
}


Bool AlterazComposta::InserisciAlt(ClassID CL_ALT, int parentesi)
{
	int i;
	if (n_alterazioni<N_ALT_MAX)       
	{
		extern Bool Get_Alterazione_Con_Parentesi();
		
		//controllo se l'alterazione da inserire c'e' gia'
		for (i=1; i<=n_alterazioni; i++)
			if (alterazioni[i-1]->GetID()==CL_ALT) break;
			if (i<=n_alterazioni)  //L'alterazione c'era gia'.
				return FALSE;
			else 
			{
				switch (CL_ALT)
				{
				case CL_DOPPIODIESIS:
					alterazioni[i-1]=(new DoppioDiesis);
					break;
				case CL_DOPPIOBEMOLLE:
					alterazioni[i-1]=(new DoppioBemolle);
					break;
				case CL_DIESIS:
					alterazioni[i-1]=(new Diesis);
					break;
				case CL_DIESIS1Q:
					alterazioni[i-1]=(new Diesis1Q);
					break;  
				case CL_DIESIS3Q:
					alterazioni[i-1]=(new Diesis3Q);
					break;  
				case CL_BEMOLLE:
					alterazioni[i-1]=(new Bemolle);
					break;  
				case CL_BEMOLLE1Q:
					alterazioni[i-1]=(new Bemolle1Q);
					break;  
				case CL_BEMOLLE3Q:
					alterazioni[i-1]=(new Bemolle3Q);
					break; 
				case CL_BEQUADRO:
					alterazioni[i-1]=(new Bequadro);
					break;
				case CL_BEQUADRO1Q:
					alterazioni[i-1]=(new Bequadro1Q);
					break;  
				case CL_BEQUADRO3Q:
					alterazioni[i-1]=(new Bequadro3Q);
					break;  
				}
				n_alterazioni++;
				if(parentesi==-1)
				{
					if (Get_Alterazione_Con_Parentesi())
						alterazioni[i-1]->SetParentesi(TRUE);
				}
				else
					alterazioni[i-1]->SetParentesi(parentesi);
				return TRUE;
			}
	}
	else return FALSE;
}        	  


Alterazione *AlterazComposta::GetPAlterazione(int i)
{
	return alterazioni[i-1];
};

Alterazione *AlterazComposta::GetPAlterazione(CmdID MyId)
{
	ClassID MyAlt;
	
	int i;
	switch (MyId) {
	case CMD_NOTA_DIESIS: MyAlt=CL_DIESIS; break;
	case CMD_NOTA_DIESIS1Q: MyAlt=CL_DIESIS1Q; break;
	case CMD_NOTA_DIESIS3Q: MyAlt=CL_DIESIS3Q; break;
	case CMD_NOTA_BEMOLLE: MyAlt=CL_BEMOLLE; break;
	case CMD_NOTA_BEMOLLE1Q: MyAlt=CL_BEMOLLE1Q; break;
	case CMD_NOTA_BEMOLLE3Q: MyAlt=CL_BEMOLLE3Q; break;
	case CMD_NOTA_BEQUADRO: MyAlt=CL_BEQUADRO; break;
	case CMD_NOTA_2DIESIS: MyAlt=CL_DOPPIODIESIS; break;
	case CMD_NOTA_2BEMOLLE: MyAlt=CL_DOPPIOBEMOLLE; break;
	case CMD_NOTA_BEQUADRO1Q: MyAlt=CL_BEQUADRO1Q; break;
	case CMD_NOTA_BEQUADRO3Q: MyAlt=CL_BEQUADRO3Q; break;
	default: return NULL;
	}
	
	for(i=1; i<=n_alterazioni; i++) 
		if (alterazioni[i-1]->GetID()==MyAlt) return alterazioni[i-1];
		
		return NULL;
}

Alterazione *AlterazComposta::GetPAlterazione(ClassID MyAlt)
{
	int i;
	for(i=1; i<=n_alterazioni; i++) 
		if (alterazioni[i-1]->GetID()==MyAlt) 
			return alterazioni[i-1];	
	return NULL;
}

Bool AlterazComposta::DeleteSym(DrawObject *sym)
{        
	int i;
	for(i=1; i<=n_alterazioni; i++)
		if(sym==alterazioni[i-1])
		{
			delete alterazioni[i-1];
			while (i<n_alterazioni)
			{
				alterazioni[i-1]=alterazioni[i];
				i++;
			}
			alterazioni[i-1]=NULL; 
			n_alterazioni--;
			return TRUE;
		}
		return FALSE;
}      


Bool AlterazComposta::Hit(const Point& p, SelObj selObj, DrawObject*& drwObjSel,
						  SymPath& symPath)
{
	int i;
	for(i=1; i<=n_alterazioni; i++)
		if(alterazioni[i-1]->Hit(p,selObj,drwObjSel,symPath))
			return TRUE;
		return FALSE;
}


Bool AlterazComposta::SymbolicHit(SymPath& symPath, SelObj selObj, 
								  DrawObject*& drwObjSel)
{
	int i;
	for(i=1; i<=n_alterazioni; i++)
		if (alterazioni[i-1]->SymbolicHit(symPath, selObj, drwObjSel))
			return TRUE;
		return FALSE;
}


void AlterazComposta::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	int i;
	for(i=1; i<=n_alterazioni; i++)
		alterazioni[i-1]->Draw();

	Scr->SetColor(oldcol);

}        


VUnit AlterazComposta::GetVU2Up()
{ return 0; };

VUnit AlterazComposta::GetVU2Dwn()
{ return 0; };

VUnit AlterazComposta::GetVU2Lft()
{ return 0; };

VUnit AlterazComposta::GetVU2Rgt()
{ return 0; };

void AlterazComposta::SetPAlterazione(int i,Alterazione* alt)
{
 alterazioni[i-1]=alt;
};
