/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#ifndef AUDIOTHREAD_HPP
#define AUDIOTHREAD_HPP

#include <wx/thread.h>

#include "aud/audiomanager.hpp"


#define FROM_BEGIN -2
#define TO_END -3



#define WAVE_BLOCK_LENGTH 10000// lunghezza in campioni del buffer

#define WAVE_MICRO_GRAIN_LENGTH 100// lunghezza del pezzettino di segnale che viene ripetuto o annullato,
//	esso determina lo step percentuale di variazione della velocit, 
//	ad esempio 100/10000 da' un uno per cento di step
#define WAVE_EARING_THRESHOLD 20// numero massimo di pezzettini che possono essere aggiunti o tolti,
//  esso determina la percentuale massima di alterazione della velocit,
//	se lo step  uno per cento e il numero massimo  20 allora la percentuale massima  20%. 



class MpegMultiRatePlayer;
class WaveMultiRatePlayerOnThread;

class AudioThread : public wxThread

{
public:
	AudioThread( const wxString&, int beginBlk, int endBlk, int startTime, int endTime, bool play );
	virtual void *Entry() {return NULL;};
	virtual void OnExit() {};
	virtual void Stop() {};
	virtual void Play() {};
	virtual void Pause() {};
	virtual bool InPause() { return true; };
	virtual void JumpTo(int) {};
	virtual int  Slower() {return 0;};
	virtual int  Faster() {return 0;};
	virtual void NormalSpeed() {};
	virtual int   GetDuration() {return 0;};
	virtual int   GetPosition() {return 0;};
	virtual float GetSpeed() {return 0;};
	virtual wxString GetInformation() {return " ";};
	virtual int   GetNBlocks() {return 0;};
	virtual int GetCurrentBlock()  {return 0;};
	virtual void SetCurrentBlock(int,int endBlock=-2 )  { };
	virtual void DeclickON( ) {};
	virtual void DeclickOFF( ) {};
	virtual int GetDurationOf(int nBlocks) { return 0; };
	virtual void SetStartBlock(int) {};
	virtual void SetEndBlock(int) {};

	bool WaitAPlayer(int count=3,int sleepTime=100);

protected:

	wxString audioFile;
	int beginBlock;
	int endBlock;
	int startingTime;
	int endingTime;
	MpegMultiRatePlayer *mpegPlayer;
	WaveMultiRatePlayerOnThread *wavePlayer;
	bool playOnStart;
};


///////////////////////////////////////////////////////////////////////////////////////////////////


class WaveThread : public AudioThread
{
public:
	WaveThread( const wxString& UaudioFile, int beginBlk, int endBlk, int startTime, int endTime, bool play );
	WaveThread( wxInputStream *, int beginBlk, int endBlk, int startTime, int endTime, bool play );
    virtual void *Entry( );
	virtual void OnExit();
	void Stop();
	void Play();
	void Pause();
	int  Slower();
	int  Faster();
	void NormalSpeed();
	int  GetDuration();
	int  GetPosition();
	wxString GetInformation();
	float GetSpeed();
	void JumpTo(int);
	int  GetNBlocks();
	int  GetCurrentBlock();
	void SetCurrentBlock(int,int endBlock=-2 );
	void DeclickON( );
	void DeclickOFF( );
	bool InPause();
	int GetDurationOf(int nBlocks);
	void SetStartBlock(int) {};
	void SetEndBlock(int);
private:
	wxInputStream * inputStream;
};


/////////////////////////////////////////////////////////////////////////////////////////////////////


class MpegThread : public AudioThread
{
public:
	MpegThread( const wxString&, int beginBlk, int endBlk, int startTime, int endTime, bool play);
	MpegThread( wxInputStream *, int beginBlk, int endBlk, int startTime, int endTime, bool play);
    virtual void *Entry( );
	virtual void OnExit();
	void Stop();
	void Play();
	void Pause();
	int  Slower();
	int  Faster();
	void NormalSpeed();
	int  GetDuration();
	int  GetPosition();
	float  GetSpeed();
	void JumpTo(int);
	wxString GetInformation();
	int  GetNBlocks();
	int  GetCurrentBlock();
	void SetCurrentBlock(int,int endBlock=-2 );
	void DeclickON( );
	void DeclickOFF( );
	bool InPause();
	int GetDurationOf(int nBlocks);
	void SetStartBlock(int);
	void SetEndBlock(int);
private:
	wxInputStream * inputStream;
};

#endif