/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: armatura.cpp,v 1.1 2005/07/25 10:50:58 bellinpo Exp $ */


#include "bat/armatura.hpp"

ArmaturaChiave::ArmaturaChiave()
{
	ID = CL_ARMATURACHIAVE;
	tipoArm = tipoArmPrec = DO_maggiore;
	tipoAlt = FALSE; 
	numeroDiesis=0;
	numeroBemolle=0;
	numeroBequadroD=0;
	numeroBequadroB=0;
	//SetColor((ColorIcon)NORMAL_COLOR);
}  

void ArmaturaChiave::SetVU2Up(VUnit vu) 
{ARMATURA2Up=vu;};  

void ArmaturaChiave::SetVU2Dwn(VUnit vu) 
{ARMATURA2Dwn=vu;};  

void ArmaturaChiave::SetVU2Lft(VUnit vu) 
{ARMATURA2Lft=vu;};  

void ArmaturaChiave::SetVU2Rgt(VUnit vu) 
{ARMATURA2Rgt=vu;};  

VUnit ArmaturaChiave::GetVU2Up() 
{return ARMATURA2Up;};

VUnit ArmaturaChiave::GetVU2Dwn() 
{return ARMATURA2Dwn;};

VUnit ArmaturaChiave::GetVU2Lft() 
{return ARMATURA2Lft;};

VUnit ArmaturaChiave::GetVU2Rgt() 
{return ARMATURA2Rgt;};

void ArmaturaChiave::SetArmatura(tipoArmaturaChiave tparm) 
{tipoArm=tparm;};

tipoArmaturaChiave ArmaturaChiave::GetArmatura() 
{
    return tipoArm; 
};


void ArmaturaChiave::SetVU(int shift)
{
	numeroDiesis=0;
	numeroBemolle=0;
	numeroBequadroD=0;
	numeroBequadroB=0;
	
	SetVU2Up(ARMATURA_DEPTH);
	SetVU2Dwn(0);
	
	SetVU2Lft(ARMATURA2LFT);  
    // Margini armatura in relazione allo scostamento shift introdotto 
    // dalla chiave attuale.
	
	if(tipoAlt)
	{
		//tramite indice della armatura decido quanti diesis
		//e bemolle devo impostare
		int index = (tipoArm % 15);
		if (index==0)
		{
			SetVU2Rgt(0);
			SetVU2Lft(0);  	
		}
		else
		{
			if (index<=7)
			{	
				numeroDiesis+=index;
				SetVU2Rgt((DIST_ALTERAZIONI_ARMAT * (numeroDiesis))); //era -1
			}
			else
			{
				numeroBemolle+=(index-7);
				SetVU2Rgt((DIST_ALTERAZIONI_ARMAT * (numeroBemolle))); 
			}
		}
	}  
	else
	{
		int indexprec = (tipoArmPrec % 15);
		int index = (tipoArm % 15);

		//check sulla precedente armatura per settare i bequadro
		if (indexprec==0)
		{
			SetVU2Rgt(0);
			SetVU2Lft(0);  	
		}
		else
		{	
			if (indexprec<=7)
			{
				if (index > 7)
					numeroBequadroD+=indexprec;
				else
					numeroBequadroD+=(indexprec-index)*(indexprec-index > 0);
			}
			else
			{
				if (index <= 7)
					numeroBequadroB+=(indexprec-7); 
				else
					numeroBequadroB+=(indexprec-index)*(indexprec-index > 0);
			}
		}
		
		//check sulla nuova armatura per diesis e bemolle		
		if (index==0)
		{
			SetVU2Rgt((DIST_ALTERAZIONI_ARMAT * (0+numeroBequadroD+numeroBequadroB-1)));
			SetVU2Lft(0);  	
		}
		else
		{
			if (index<=7)
			{	
				numeroDiesis+=index;
				SetVU2Rgt((DIST_ALTERAZIONI_ARMAT * (numeroDiesis+numeroBequadroD+numeroBequadroB-1))); 
			}
			else
			{
				numeroBemolle+=(index-7);
				SetVU2Rgt((DIST_ALTERAZIONI_ARMAT * (numeroBemolle+numeroBequadroD+numeroBequadroB-1))); 			  
			}
		}
		
	}  
}


void ArmaturaChiave::SetPos(DrawObject *drwobj,const Point& p)
{
	DrawObject::SetPos(drwobj,p);
	
	int altezzaDiesis[7]={ALTEZZA_ARM_DIESIS1,ALTEZZA_ARM_DIESIS2,ALTEZZA_ARM_DIESIS3,
		ALTEZZA_ARM_DIESIS4,ALTEZZA_ARM_DIESIS5,ALTEZZA_ARM_DIESIS6,
		ALTEZZA_ARM_DIESIS7};
	
	int deltaxDiesis[7]={DELTAX_ARM_DIESIS1,DELTAX_ARM_DIESIS2,DELTAX_ARM_DIESIS3,
		DELTAX_ARM_DIESIS4,DELTAX_ARM_DIESIS5,DELTAX_ARM_DIESIS6,
		DELTAX_ARM_DIESIS7};
	
	int altezzaBemolle[7]={ALTEZZA_ARM_BEMOLLE1,ALTEZZA_ARM_BEMOLLE2,ALTEZZA_ARM_BEMOLLE3,
		ALTEZZA_ARM_BEMOLLE4,ALTEZZA_ARM_BEMOLLE5,ALTEZZA_ARM_BEMOLLE6,
		ALTEZZA_ARM_BEMOLLE7};
	
	int deltaxBemolle[7]={DELTAX_ARM_BEMOLLE1,DELTAX_ARM_BEMOLLE2,DELTAX_ARM_BEMOLLE3,
		DELTAX_ARM_BEMOLLE4,DELTAX_ARM_BEMOLLE5,DELTAX_ARM_BEMOLLE6,
		DELTAX_ARM_BEMOLLE7};
	

	int startArm = (tipoArmPrec % 15);
	startArm = (startArm > 7) ? startArm-8 : startArm-1;

	bool isAltTypeChanged = (numeroBequadroD && numeroBemolle) || (numeroBequadroB && numeroDiesis);

	int cc;

	int b1 =0;
	for(cc=startArm-numeroBequadroD+1;cc<=startArm;cc++)
	{
		if((altezzaDiesis[cc]-shiftChiave+altezzaChiave)<-1)
		{
			bqdr[b1].SetAltezza(altezzaDiesis[cc]+7);
		}
		else if((altezzaDiesis[cc]-shiftChiave+altezzaChiave)>9)
		{
			bqdr[b1].SetAltezza(altezzaDiesis[cc]-7);
		}
		else
		{
			bqdr[b1].SetAltezza(altezzaDiesis[cc]);
		}
		
		if (isAltTypeChanged)
			bqdr[b1].SetPos(this,Point(deltaxDiesis[b1],0));
		else
			bqdr[b1].SetPos(this,Point(deltaxDiesis[b1] + (DIST_ALTERAZIONI_ARMAT * numeroDiesis),0));

		b1++;
	}  

	
	int b2 = 0;
	for(cc=startArm-numeroBequadroB+1;cc<=startArm;cc++)
	{
		if((altezzaBemolle[cc]-shiftChiave+altezzaChiave)<-1)
		{
			bqdr[b2].SetAltezza(altezzaBemolle[cc]+7);
		}
		else if((altezzaBemolle[b2]-shiftChiave+altezzaChiave)>9)
		{
			bqdr[b2].SetAltezza(altezzaBemolle[cc]-7);
		}
		else
		{
			bqdr[b2].SetAltezza(altezzaBemolle[cc]);
		}

		if (isAltTypeChanged)
			bqdr[b2].SetPos(this,Point(deltaxBemolle[b2],0));
		else
			bqdr[b2].SetPos(this,Point(deltaxBemolle[b2] + (DIST_ALTERAZIONI_ARMAT * numeroBemolle),0));

		b2++;
	}  

	
	for(int i=0;i<numeroDiesis;i++)
	{
		if((altezzaDiesis[i]-shiftChiave+altezzaChiave)<-1)
		{
			dss[i].SetAltezza(altezzaDiesis[i]+7);
		}
		else if((altezzaDiesis[i]-shiftChiave+altezzaChiave)>9)
		{
			dss[i].SetAltezza(altezzaDiesis[i]-7);
		}
		else
		{
			dss[i].SetAltezza(altezzaDiesis[i]);
		}
		
		if (isAltTypeChanged)
		dss[i].SetPos(this,Point(deltaxDiesis[i]+(DIST_ALTERAZIONI_ARMAT * 
			(numeroBequadroD+numeroBequadroB)),0));
		else
			dss[i].SetPos(this,Point(deltaxDiesis[i],0));
	}  
	
	for(int j=0;j<numeroBemolle;j++)
	{
		if((altezzaBemolle[j]-shiftChiave+altezzaChiave)<-1)
		{
			bmll[j].SetAltezza(altezzaBemolle[j]+7);
		}
		else if((altezzaBemolle[j]-shiftChiave+altezzaChiave)>9)
		{
			bmll[j].SetAltezza(altezzaBemolle[j]-7);
		}
		else
		{
			bmll[j].SetAltezza(altezzaBemolle[j]);
		}
		
		if (isAltTypeChanged)
		bmll[j].SetPos(this,Point(deltaxBemolle[j]+(DIST_ALTERAZIONI_ARMAT * 
			(numeroBequadroD+numeroBequadroB)),0));
		else
			bmll[j].SetPos(this,Point(deltaxBemolle[j],0));
	}  
}


void ArmaturaChiave::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());
	
	for(int k1=0;k1<numeroBequadroD;k1++)
	{
//FFF		bqdr[k1].SetColor(GetColor());
		bqdr[k1].Draw();
	}  
	
	for(int k2=0;k2<numeroBequadroB;k2++)
	{
//FFF		bqdr[k2].SetColor(GetColor());
		bqdr[k2].Draw();
	}  
	
	
	for(int i=0;i<numeroDiesis;i++)
	{
//FFF		dss[i].SetColor(GetColor());
		dss[i].Draw();
	}  
	
	for(int j=0;j<numeroBemolle;j++)
	{
//FFF		bmll[j].SetColor(GetColor());
		bmll[j].Draw();
	}  

	if(isSelected)
		DrawContour();

	Scr->SetColor(oldcol);
};


UL ArmaturaChiave::GetSp2NextFig()
{
	UL d=Figura::GetSp2NextFig();
	
	if(d==0)
	{
		d=int(VU2UL(GetWidth())*1.2);
		if ((tipoLeggio==LEGGIO_MASAE) || (tipoLeggio==LEGGIO_MASE))
			SetSpG(d);
		if ((tipoLeggio==LEGGIO_DLIOO) || (tipoLeggio==LEGGIO_PDLOO)) 
			SetSpOr(d);
	}
	return d;
}

void ArmaturaChiave::Adjust(Bool primaleg, Battuta *pBattuta, short current_layer, short current_staff)
{         
	SetVU2UpF(ARMATURA2Up);	//FFF
	SetVU2DwnF(ARMATURA2Dwn);
	SetVU2LftF(ARMATURA2Lft);
	SetVU2RgtF(ARMATURA2Rgt);	

	Figura::Adjust(primaleg, pBattuta, current_layer, current_staff);
}




