/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: armatura.hpp,v 1.1 2005/07/25 10:50:58 bellinpo Exp $ */

#ifndef ARMATURACHIAVE_HPP
#define ARMATURACHIAVE_HPP

#include "fig/spazio.hpp"
#include "alt/diesis.hpp" 
#include "alt/bemolle.hpp"
#include "alt/bequadro.hpp"

#include "drw/gscreen.hpp"

class ArmaturaChiave:public Spazio
{
protected:  
    VUnit ARMATURA2Up,ARMATURA2Dwn,ARMATURA2Lft,ARMATURA2Rgt;
	// Distanze da AbsPos dei punti superiore, inferiore, sinistro e
	// destro del rettangolo contenente l'armatura di chiave.
    Diesis dss[7];
	// Diesis dell'armatura.
    Bemolle bmll[7];
	// Bemolle dell'armatura.  
    Bequadro bqdr[7];
	// Bemolle dell'armatura.  
    int numeroDiesis;
	// Numero di diesis dell'armatura.  
    int numeroBemolle;
	// Numero di bemolle dell'armatura.  
    int numeroBequadroD;
	// Numero di bequadro dell'armatura.  
    int numeroBequadroB;
	// Numero di bequadro dell'armatura.  
    tipoArmaturaChiave tipoArm;
	// Tipo di armatura.  
    
public:
    int shiftChiave;
	// Scostamento dell'armatura dalla chiave.  
    int altezzaChiave;
	// Altezza della chiave.  
    Bool tipoAlt;
	// tipo di armatura (diesis o bemolle) a cui deve fare riferimento,
	// se esiste, l'armatura dei bequadri.  
    tipoArmaturaChiave tipoArmPrec;
	
    ArmaturaChiave();
    
    void SetVU2Up(VUnit vu);  
    void SetVU2Dwn(VUnit vu);  
    void SetVU2Lft(VUnit vu);  
    void SetVU2Rgt(VUnit vu);  
	
    VUnit GetVU2Up();
    VUnit GetVU2Dwn();
    VUnit GetVU2Lft();
    VUnit GetVU2Rgt();
	
    void SetVU(int);
	// Imposta le dimensioni dell'armatura di chiave.
    void SetPos(DrawObject *drwobj,const Point& p);
	// Imposta la posizione dell'armatura.
    void SetArmatura(tipoArmaturaChiave tparm);
	// Imposta il tipo di armatura di chiave di chiave.
    tipoArmaturaChiave GetArmatura();
	// Restituisce il tipo di armatura di chiave dell'intestazione.
    void Draw();  
	// Disegna l'armatura di chiave                                                           
	
    Bool HasSameDesc(ArmaturaChiave*);
	// TRUE se l'oggetto ha la stessa descrizione di quello fornito.
	
    UL GetSp2NextFig();
	
	void Adjust(Bool primaleg, Battuta *pBattuta, short current_layer, short current_staff);

	int SaveXML(FILE *fp);
};  

#endif //ARMATURACHIAVE_HPP
