/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: battuta.cpp,v 1.1 2005/07/25 10:50:59 bellinpo Exp $ */


#include <math.h>
#include "bat/giustif.hpp" 
#include "gui/dialogs.hpp" // finestra debug milla
#include "gui/lioowin.hpp"


extern int DebugNormalMilla;
extern int DebugNormalMillaPos;

#include "wx/wx.h"
extern wxString msgdebug;
extern wxString msgtoken;

#include "liooext.hpp"
#include "bat/battuta.hpp" 
#include "fig/nota.hpp"
#include "drw/frmanage.hpp" 
#include "gui/dlg/editmeasuredlg.hpp"

#include "fig/nbreve.hpp"
#include "fig/nsemibre.hpp"  
#include "fig/nminima.hpp"
#include "fig/nsemimin.hpp"
#include "fig/ncroma.hpp"
#include "fig/nsemicro.hpp"
#include "fig/nbiscrom.hpp"
#include "fig/nsemibis.hpp"
#include "fig/nfusa.hpp"

#include "abb/tremolo.hpp"
#include "abb/glissato.hpp"

#include "fig/psemibre.hpp"  
#include "fig/pminima.hpp"
#include "fig/psemimin.hpp"
#include "fig/pcroma.hpp"
#include "fig/psemicro.hpp"
#include "fig/pbiscrom.hpp"
#include "fig/psemibis.hpp"
#include "fig/pfusa.hpp"
#include "fig/pduebatt.hpp"
#include "fig/pquabatt.hpp"
#include "fig/pgeneric.hpp"
#include "fig/spazio.hpp"
#include "fig/gruppono.hpp" 
#include "fig/accordo.hpp"
#include "fig/ripet.hpp"
#include "fig/riptempo.hpp"
#include "fig/ripbatt.hpp"
#include "fig/ripmbat.hpp"

#include "bat/bsingola.hpp"
#include "bat/bfinale.hpp"
#include "bat/bdoppia.hpp"
#include "bat/bfinerit.hpp"
#include "bat/binizior.hpp"
#include "bat/binifine.hpp"
#include "bat/binvisibile.hpp"
#include "bat/bdashed.hpp"


#include "bat/cviolino.hpp"
#include "bat/cviolin8.hpp"
#include "bat/cviolino8sopra.hpp"
#include "bat/cbasso.hpp"
#include "bat/cbasso8.hpp"
#include "bat/cbariton.hpp"
#include "bat/ctenore.hpp"
#include "bat/ctenore8.hpp"
#include "bat/ccontral.hpp"
#include "bat/cmezzoso.hpp"
#include "bat/csoprano.hpp"
//
#include "bat/cbasso8sopra.hpp"
#include "bat/ctab.hpp"
#include "bat/cpercusbox.hpp"
#include "bat/cpercus2lines.hpp"
#include "bat/cvuota.hpp"
#include "bat/cbassoold.hpp"
//

#include "int/legatqua.hpp"

#define max(x,y) ((x)>(y)? x : y)

#define DIST_1FIG 15

bool Battuta::AutoBeaming=true;

Battuta::Battuta()
{
	int i,j;
	tipoBat = GENERICA;
	ID=CL_BATTUTA;
	NumPagDLIOO = 0;
	NumPagMASAE = 0;
	NumPagMASE = 0;
	
	NumMultiRest = 0;
	IsMultiRest = FALSE;
	pMultiRest = NULL;
	
	for(i=0;i<2;i++)
	{
		mainScoreLineBreak[i]=false;
		partLineBreak[i]=false;
	}
	
	// 
	
	distanzaFig=MIN_DIST_FIG; 
	
	for(i=0;i<NUM_MAX_LAYER;i++)
	{
		for(j=0;j<2;j++)
		{
			DirSp2FirstFig[i][j]=0;
		}
	}
	for(i=0;i<NUM_MAX_LAYER;i++)
	{
		for(j=0;j<2;j++)
		{
			OrcSp2FirstFig[i][j]=0;
		}
	}
	for(i=0;i<NUM_MAX_LAYER;i++)
	{
		for(j=0;j<2;j++)
		{
			DirSp2FirstFigUt[i][j]=0;
		}
	}
	for(i=0;i<NUM_MAX_LAYER;i++)
	{
		for(j=0;j<2;j++)
		{
			OrcSp2FirstFigUt[i][j]=0;
		}
	}
	for(i=0;i<NUM_MAX_LAYER;i++)
	{
		for(j=0;j<2;j++)
		{
			DirSp2FirstFigLine[i][j]=0;
		}
	}
	for(i=0;i<NUM_MAX_LAYER;i++)
	{
		for(j=0;j<2;j++)
		{
			OrcSp2FirstFigLine[i][j]=0;
		}
	}
	for(i=0;i<NUM_MAX_LAYER;i++){numeroFig[i]=0;}
	
    
	EverJustified[0]=EverJustified[1]=EverJustified[2]=EverJustified[3]=NO;
	
	xBarra=0;
	NumProgress=1;
	ptrBarra=NULL;
	ptrBarraPrec=NULL;
	ptrNumBattuta=NULL;  
	ptrNumGrande = NULL;
	ptrScansione = NULL;
	ptrLettera = NULL;
	ptrTSalto = NULL;
	ptrMovimento = NULL;
	ptrRiferimento = NULL;
    
	VU2Up = 50;
	VU2Dwn = 50;
	VU2Figure = 0;
	// costruttore di battuta, inizializzo i valori
	// della giustificazione, riguardare
	kgiustificazione[0]=kgiustificazione[1]=0;
	kgiustificazioneparte[0]=kgiustificazioneparte[1]=0;
   	tipogiust[0]=tipogiust[1]=G_NONE;
   	tipogiustparte[0]=tipogiustparte[1]=G_NONE;
	NStaffLine=5;
    parentScore=NULL;
	
	LayerVector[0]=new Layer;
	LayerVector[1]=new Layer;
	
	for(i=2;i<NUM_MAX_LAYER; i++)
		LayerVector[i]=NULL;
	
	intest[0]=new Intestazione;
	intest[1]=NULL;
	intest[2]=NULL;
	intest_end[0]=NULL;
	intest_end[1]=NULL;
	intest_end[2]=NULL;
	NumberOfStaff=1;
    
}; 

Battuta::~Battuta() 
{
	Free();
}

Battuta::Battuta(int Nrighi)
{
	int i,j;
	tipoBat = GENERICA;
	ID=CL_BATTUTA;
	NumPagDLIOO = 0;
	NumPagMASAE = 0;
	NumPagMASE = 0;
	
	NumMultiRest = 0;
	IsMultiRest = FALSE;
	pMultiRest = NULL;
	
	for(i=0;i<2;i++)
	{
		mainScoreLineBreak[i]=false;
		partLineBreak[i]=false;
	}
	
	// 
	
	distanzaFig=MIN_DIST_FIG; 
	
	for(i=0;i<NUM_MAX_LAYER;i++)
	{
		for(j=0;j<2;j++)
		{
			DirSp2FirstFig[i][j]=0;
		}
	}
	for(i=0;i<NUM_MAX_LAYER;i++)
	{
		for(j=0;j<2;j++)
		{
			OrcSp2FirstFig[i][j]=0;
		}
	}
	for(i=0;i<NUM_MAX_LAYER;i++)
	{
		for(j=0;j<2;j++)
		{
			DirSp2FirstFigUt[i][j]=0;
		}
	}
	for(i=0;i<NUM_MAX_LAYER;i++)
	{
		for(j=0;j<2;j++)
		{
			OrcSp2FirstFigUt[i][j]=0;
		}
	}
	for(i=0;i<NUM_MAX_LAYER;i++)
	{
		for(j=0;j<2;j++)
		{
			DirSp2FirstFigLine[i][j]=0;
		}
	}
	for(i=0;i<NUM_MAX_LAYER;i++)
	{
		for(j=0;j<2;j++)
		{
			OrcSp2FirstFigLine[i][j]=0;
		}
	}
	for(i=0;i<NUM_MAX_LAYER;i++){numeroFig[i]=0;}
	
	EverJustified[0]=EverJustified[1]=EverJustified[2]=EverJustified[3]=NO;
	
	
	
	xBarra=0;
	NumProgress=1;
	ptrBarra=NULL;
	ptrBarraPrec=NULL;
	ptrNumBattuta=NULL;  
	ptrNumGrande = NULL;
	ptrScansione = NULL;
	ptrLettera = NULL;
	ptrTSalto = NULL;
	ptrRiferimento = NULL;
	
	ptrMovimento = NULL;
	
	VU2Up = 50;
	VU2Dwn = 50;
	VU2Figure = 0;
	// costruttore di battuta, inizializzo i valori
	// della giustificazione, riguardare
	kgiustificazione[0]=kgiustificazione[1]=0;
	kgiustificazioneparte[0]=kgiustificazioneparte[1]=0;
	tipogiust[0]=tipogiust[1]=G_NONE;
	tipogiustparte[0]=tipogiustparte[1]=G_NONE;
	
	NStaffLine=Nrighi;
	parentScore=NULL; 
	
	LayerVector[0]=new Layer;
	LayerVector[1]=new Layer;
	
	for(i=2;i<NUM_MAX_LAYER; i++)
		LayerVector[i]=NULL;
	
	intest[0]=new Intestazione;
	intest[1]=NULL;
	intest[2]=NULL;
	intest_end[0]=NULL;
	intest_end[1]=NULL;
	intest_end[2]=NULL;
	NumberOfStaff=1;
}

int Battuta::GetEverJustified(int part, int view)
{
   	if ((DIR==part)&&(COMPUTERVIEW==view)) 
		return EverJustified[0];
	if ((ORC==part)&&(COMPUTERVIEW==view)) 
		return EverJustified[1];
	if ((DIR==part)&&(PRINTVIEW==view)) 
		return EverJustified[2];
	if ((ORC==part)&&(PRINTVIEW==view)) 
		return EverJustified[3];
	
	return(NO);
}


void Battuta::SetEverJustified(int part, int view, int value) 
{
	if ((DIR==part)&&(COMPUTERVIEW==view)) 
		EverJustified[0]=value;
	if ((ORC==part)&&(COMPUTERVIEW==view)) 
		EverJustified[1]=value;
	if ((DIR==part)&&(PRINTVIEW==view)) 
		EverJustified[2]=value;
	if ((ORC==part)&&(PRINTVIEW==view)) 
		EverJustified[3]=value;
	
}

double Battuta::Getkgiustificazione() 
{return kgiustificazione[currentViewType];}

double Battuta::Getkgiustificazione(int a) 
{return kgiustificazione[a];}

void Battuta::Setkgiustificazione(double k) 
{kgiustificazione[currentViewType]=k;}

void Battuta::Setkgiustificazione(double k, int dove) 
{kgiustificazione[dove]=k;}

double Battuta::Getkgiustificazioneparte() {return kgiustificazioneparte[currentViewType];}

double Battuta::Getkgiustificazioneparte(int a) 
{return kgiustificazioneparte[a];}

void Battuta::Setkgiustificazioneparte(double k) {kgiustificazioneparte[currentViewType]=k;}

void Battuta::Setkgiustificazioneparte(double k, int dove) 
{kgiustificazioneparte[dove]=k;}

TipoGiust Battuta::Gettipogiust() {return tipogiust[currentViewType];}

TipoGiust Battuta::Gettipogiust(int a) {return tipogiust[a];}

void Battuta::Settipogiust(TipoGiust t) {tipogiust[currentViewType]=t;}

void Battuta::Settipogiust(TipoGiust t, int a) {tipogiust[a]=t;}

TipoGiust Battuta::Gettipogiustparte() {return tipogiustparte[currentViewType];}

TipoGiust Battuta::Gettipogiustparte(int a) {return tipogiustparte[a];}

void Battuta::Settipogiustparte(TipoGiust t) {tipogiustparte[currentViewType]=t;}

void Battuta::Settipogiustparte(TipoGiust t, int a) {tipogiustparte[a]=t;}

UL Battuta::GetDirSp2FirstFig(short current_layer) {return DirSp2FirstFig[current_layer][currentViewType];}
UL Battuta::GetOrcSp2FirstFig(short current_layer) {return OrcSp2FirstFig[current_layer][currentViewType];}
UL Battuta::GetDirSp2FirstFigLine(short current_layer) {return DirSp2FirstFigLine[current_layer][currentViewType];}
UL Battuta::GetOrcSp2FirstFigLine(short current_layer) {return OrcSp2FirstFigLine[current_layer][currentViewType];}
UL Battuta::GetDirSp2FirstFigUt(short current_layer) {return DirSp2FirstFigUt[current_layer][currentViewType];}
UL Battuta::GetOrcSp2FirstFigUt(short current_layer) {return OrcSp2FirstFigUt[current_layer][currentViewType];}

void Battuta::SetDirSp2FirstFig(short current_layer,UL par) {DirSp2FirstFig[current_layer][currentViewType]=par;}
void Battuta::SetOrcSp2FirstFig(short current_layer,UL par) {OrcSp2FirstFig[current_layer][currentViewType]=par;}
void Battuta::SetDirSp2FirstFigLine(short current_layer,UL par) {DirSp2FirstFigLine[current_layer][currentViewType]=par;}
void Battuta::SetOrcSp2FirstFigLine(short current_layer,UL par) {OrcSp2FirstFigLine[current_layer][currentViewType]=par;}
void Battuta::SetDirSp2FirstFigUt(short current_layer,UL par) {DirSp2FirstFigUt[current_layer][currentViewType]=par;}
void Battuta::SetOrcSp2FirstFigUt(short current_layer,UL par) {OrcSp2FirstFigUt[current_layer][currentViewType]=par;}

UL Battuta::GetDirSp2FirstFig(short current_layer,int a) {return DirSp2FirstFig[current_layer][a];}
UL Battuta::GetOrcSp2FirstFig(short current_layer,int a) {return OrcSp2FirstFig[current_layer][a];}
UL Battuta::GetDirSp2FirstFigLine(short current_layer,int a) {return DirSp2FirstFigLine[current_layer][a];}
UL Battuta::GetOrcSp2FirstFigLine(short current_layer,int a) {return OrcSp2FirstFigLine[current_layer][a];}
UL Battuta::GetDirSp2FirstFigUt(short current_layer,int a) {return DirSp2FirstFigUt[current_layer][a];}
UL Battuta::GetOrcSp2FirstFigUt(short current_layer,int a) {return OrcSp2FirstFigUt[current_layer][a];}

void Battuta::SetDirSp2FirstFig(short current_layer,UL par, int a) {DirSp2FirstFig[current_layer][a]=par;}
void Battuta::SetOrcSp2FirstFig(short current_layer,UL par, int a) {OrcSp2FirstFig[current_layer][a]=par;}
void Battuta::SetDirSp2FirstFigLine(short current_layer,UL par, int a) {DirSp2FirstFigLine[current_layer][a]=par;}
void Battuta::SetOrcSp2FirstFigLine(short current_layer,UL par, int a) {OrcSp2FirstFigLine[current_layer][a]=par;}
void Battuta::SetDirSp2FirstFigUt(short current_layer,UL par, int a) {DirSp2FirstFigUt[current_layer][a]=par;}
void Battuta::SetOrcSp2FirstFigUt(short current_layer,UL par, int a) {OrcSp2FirstFigUt[current_layer][a]=par;}

Layer* Battuta::GetPLayer(short current_layer)   // GB
{
	return LayerVector[current_layer];	
}


void Battuta::SetNumMultiRest(int nb)
{ NumMultiRest=nb; }

int Battuta::GetNumMultiRest()
{ return NumMultiRest; }

void Battuta::SetIsMultiRest(Bool ismrest)
{ IsMultiRest=ismrest; }

Bool Battuta::GetIsMultiRest()
{ return IsMultiRest; }

void Battuta::SetPMultiRest(Battuta *pb)
{ pMultiRest=pb;}

Intestazione* Battuta::GetIntestEnd(short staff) 
{ 
	Layer *pLayer;
	Figura *fig=NULL;
	ClassID id, intern_ClefID; 
	tipoArmaturaChiave intern_KeySignID;
	Chiave *ptrChiave=NULL;
	bool intern_clef=true;
	bool modify_Clef=false;
	bool modify_KeySign=false;
	
	if(intern_clef)	
	{
		for(int i=0; i<NUM_MAX_LAYER; i++)
		{
			pLayer=GetPLayer(i);
			if(pLayer==NULL)
				continue;
			
			fig=pLayer->GetFirstFig();
			while(fig!=NULL)
			{
				id=fig->GetID();
				//Si controlla se la figura corrente e' una chiave nel caso lo sia si arresta il procedimento
				//Se l'armatura esistente coincide con la modifica va eliminata la figura
				//che la rappresenta temporaneamente.
				if (fig->GetNStaff()==staff && ((id >= CL_CVIOLINO) && (id <= CL_CSOPRANO))||((id >= CL_CTENORE8) && (id <= CL_CBASSOOLD)))
				{
					intern_ClefID=id;
					modify_Clef=true;
				}
				if (fig->GetNStaff()==staff && id == CL_ARMATURACHIAVE) 
				{
					intern_KeySignID=((ArmaturaChiave*)fig)->GetArmatura();
					modify_KeySign=true;
				}
				
				fig=pLayer->GetNextFig();
			} 
		}
		
		if(intest_end[staff]==NULL)
			intest_end[staff]=new Intestazione;
		
		intest_end[staff]->Copy(intest[staff]);
		if(modify_Clef)
		{	
			switch(intern_ClefID)
			{
			case CL_CVIOLINO:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CViolino;
				break;
			case CL_CBASSO8SOPRA:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CBasso8Sopra;
				break;
			case CL_CTAB:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CTab;
				break;
			case CL_CPERCUSBOX:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CPercusBox;
				break;
			case CL_CPERCUS2LINES:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CPercus2Lines;
				break;
			case CL_CVUOTA:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CVuota;
				break;
			case CL_CBASSOOLD:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CBassoOld;
				break;
			case CL_CVIOLINO8:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CViolino8;
				break;
			case CL_CVIOLINO8SOPRA:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CViolino8Sopra;
				break;
			case CL_CBASSO:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CBasso;
				break;
			case CL_CBASSO8:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CBasso8;
				break;
			case CL_CBARITONO:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CBaritono;
				break;
			case CL_CTENORE:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CTenore;
				break;
			case CL_CTENORE8:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CTenore8;
				break;
			case CL_CCONTRALTO:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CContralto;
				break;
			case CL_CMEZZOSOPRANO:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CMezzoSoprano;
				break;
			case CL_CSOPRANO:
				if(ptrChiave!=NULL)
					delete ptrChiave;
				ptrChiave=new CSoprano;
				break;
			}
			if(ptrChiave!=NULL)
				intest_end[staff]->SetChiave(ptrChiave);			
		}
		if(modify_KeySign)
			intest_end[staff]->SetArmatura(intern_KeySignID);
		
		return intest_end[staff];
	}
	return intest[staff]; 
}

Intestazione** Battuta::GetAllIntestEnd() 
{
	for(int i=0; i<NumberOfStaff; i++)
		GetIntestEnd(i);
	return intest_end; 
}

/****************/


void Battuta::SetBarra(tipoBarra tbar)
{
	tpBarra = tbar;
	SetptrBarra(tbar);
	SetptrBarraPrec(tbar, TRUE);
}


void Battuta::SetptrBarra(tipoBarra tbar)
{
	if (ptrBarra != NULL)
		delete ptrBarra;
	
	switch(tbar)
	{
    case SINGOLA: 
		ptrBarra= new BSingola;   
		ptrBarra->SetNStaffLine(NStaffLine);	
		
		break;
    case DOPPIA:
		ptrBarra= new BDoppia;
		ptrBarra->SetNStaffLine(NStaffLine);	
		
		break;
    case FINALE:                
		ptrBarra= new BFinale;
		ptrBarra->SetNStaffLine(NStaffLine);	
		
		break;
    case INIZIORIT:                
		ptrBarra= new BInizioRit;
		ptrBarra->SetNStaffLine(NStaffLine);	
		
		break;    
    case FINERIT:                
		ptrBarra= new BFineRit;
		ptrBarra->SetNStaffLine(NStaffLine);	
		
		break;
    case INIZIOFINERIT:
		ptrBarra= new BInizioFine;
		ptrBarra->SetNStaffLine(NStaffLine);	
		
		break;
	case INVISIBILE: 
		ptrBarra= new BInvisibile;      
		ptrBarra->SetNStaffLine(NStaffLine);	
		
		break;
    default:
		ptrBarra= new BSingola;
		ptrBarra->SetNStaffLine(NStaffLine);	
		
		break;
	}
	
	if (!ptrBarra)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","SetBarra()");
	
}; 


void Battuta::SetptrBarraPrec(tipoBarra tbar, Bool IsNull)
{
	
	if (ptrBarraPrec != NULL)
		delete ptrBarraPrec;
    
	if (IsNull) 
	{
		ptrBarraPrec = NULL;
		return;
	}
	
	switch(tbar)
	{
    case SINGOLA: 
		ptrBarraPrec= new BSingola;  
		ptrBarraPrec->SetNStaffLine(NStaffLine);	
		
		break;
    case DOPPIA:
		ptrBarraPrec= new BDoppia;
		ptrBarraPrec->SetNStaffLine(NStaffLine);	
		
		break;
    case FINALE:                
		ptrBarraPrec= new BFinale;
		ptrBarraPrec->SetNStaffLine(NStaffLine);	
		
		break;
    case INIZIORIT:                
		ptrBarraPrec= new BInizioRit;
		ptrBarraPrec->SetNStaffLine(NStaffLine);	
		
		break;    
    case FINERIT:                
		ptrBarraPrec= new BFineRit;
		ptrBarraPrec->SetNStaffLine(NStaffLine);	
		
		break;
    case INIZIOFINERIT:
		ptrBarraPrec= new BInizioFine;
		ptrBarraPrec->SetNStaffLine(NStaffLine);	
		
		break;
	case INVISIBILE: 
		ptrBarra= new BInvisibile;      
		ptrBarra->SetNStaffLine(NStaffLine);	
		
		break;
    default:
		ptrBarraPrec= new BSingola;
		ptrBarraPrec->SetNStaffLine(NStaffLine);	
		
		break;
	}
	
	if (!ptrBarraPrec)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","SetBarra()");
	
}; 


Barra *Battuta::GetBarra()
{
	return ptrBarra;
}

Battuta* Battuta::GetPMultiRest()
{ return pMultiRest;}

void Battuta::SetPNumBattuta(NumBattuta *p)
{ ptrNumBattuta=p; }

void Battuta::SetPNumGrande(NumGrande *p)
{ ptrNumGrande=p; }

NumGrande *Battuta::GetPNumGrande()
{ return ptrNumGrande; }

void Battuta::SetPScansione(Scansione *p)
{ ptrScansione=p; }

void Battuta::SetPLettera(Lettera *p)
{ ptrLettera=p; }

Lettera *Battuta::GetPLettera()
{ return ptrLettera; }

void Battuta::SetPTSalto(TSalto *p)
{ ptrTSalto=p; }

TSalto *Battuta::GetPTSalto()
{ return ptrTSalto; }

void Battuta::SetPRiferimento(Riferimento *p)
{ ptrRiferimento=p; }

Riferimento *Battuta::GetPRiferimento()
{ return ptrRiferimento; }

void Battuta::SetNumProgress(int num)
{NumProgress=num;}

int Battuta::GetNumProgress()
{return NumProgress;}

void Battuta::SetVU2Lft(VUnit vu) 
{BATTUTA2Lft=vu;};  

void Battuta::SetVU2Rgt(VUnit vu) 
{BATTUTA2Rgt=vu;};  

VUnit Battuta::GetVU2Up() 
{return VU2Up;};

VUnit Battuta::GetVU2Dwn() 
{return VU2Dwn;};

VUnit Battuta::GetVU2Lft() 
{return BATTUTA2Lft;};

VUnit Battuta::GetVU2Rgt() 
{return BATTUTA2Rgt;}; 

void Battuta::SetDistanzaFig(VUnit df) 
{ distanzaFig=df; };

void Battuta::SetVU2Up(VUnit vu) 
{ VU2Up = vu; };

void Battuta::SetVU2Dwn(VUnit vu) 
{ VU2Dwn=vu; };

void Battuta::SetxBarra(VUnit xpb) 
{ xBarra=xpb; };

void Battuta::SetNumericCode(NumCode nc)
{ NumericCode=nc; }

NumCode Battuta::GetNumericCode()
{ return NumericCode; }

void Battuta::SetNumPagDLIOO(int num)
{ NumPagDLIOO = (NumCode)num; }

NumCode Battuta::GetNumPagDLIOO(void)
{ return NumPagDLIOO; }

void Battuta::SetNumPagMASAE(int num)
{ NumPagMASAE = (NumCode)num; }

NumCode Battuta::GetNumPagMASAE(void)
{ return NumPagMASAE; }

void Battuta::SetNumPagMASE(int num)
{ NumPagMASE = (NumCode)num; }

NumCode Battuta::GetNumPagMASE(void)
{ return NumPagMASE; }

unsigned long int Battuta::GetNumeroFig() 
{	
	unsigned long int maxFig = 0, tempnumfig;
	
	
	for(int i = 0; i < NUM_MAX_LAYER; i ++)
	{
		if (LayerVector[i]!=NULL)
		{
			tempnumfig = LayerVector[i]->GetNumeroFig();
			if (tempnumfig > maxFig) maxFig = tempnumfig;
		}
	}
	
	return maxFig;
	
}

Intestazione* Battuta::GetIntest(short current_staff) 
{
	return intest[current_staff];
}

Intestazione** Battuta::GetAllIntest() 
{
	return (Intestazione**)&intest;
}

void Battuta::SetBattuta(tipoBattuta tpbat,Intestazione **intst,tipoSpartito tsp) // GB

{
	Bool DrawNumero;
	
	tipoBat = tpbat; 
	
	for(int current_staff=0; current_staff<NumberOfStaff; current_staff++)
	{
		Intestazione *pIntest=NULL;
		if(intst!=NULL)
			pIntest=intst[current_staff];
		intest[current_staff]->SetVU(tpbat,pIntest); 
	}
	
	// Non fa disegnare il numero ne' ad inizio brano, ne' all'interno del rigo.
	DrawNumero = tpbat==INIZIO_PAGINA || tpbat==INIZIO_PENTAGRAMMA
		|| ((tpbat==INIZIO_BRANO) && (GetNumProgress()>1));
	// Nella partitura solo il pentagramma superiore riporta il numero.
	if ((tipoLeggio==LEGGIO_MASAE) || (tipoLeggio==LEGGIO_MASE))
		DrawNumero = DrawNumero && (tsp==SPART_SUPERIORE || tsp==SPART_UNICO); 
	if(DrawNumero)
	{
		if(ptrNumBattuta==NULL)
		{
			ptrNumBattuta = new NumBattuta(GetNumProgress());
			if(!ptrNumBattuta )
				EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","SetBattuta()");
		}  
		else ptrNumBattuta->SetNumero(GetNumProgress());
	}
	else
	{
		if(ptrNumBattuta!=NULL)
		{
			delete ptrNumBattuta;
			ptrNumBattuta=NULL;
		}
	}
};

tipoBattuta Battuta::GetBattuta() 
{return tipoBat;};

void Battuta::SetChiave(short current_staff,Chiave *chv) 
{intest[current_staff]->SetChiave(chv);};

void Battuta::SetArmatura(short current_staff ,tipoArmaturaChiave arm) 
{intest[current_staff]->SetArmatura(arm);};

Chiave *Battuta::GetChiave(short current_staff) 
{return intest[current_staff]->GetChiave();};

tipoArmaturaChiave Battuta::GetArmatura(short current_staff) 
{return intest[current_staff]->GetTipoArmatura();};

void Battuta::SetTempo(char tmp[]) 
{intest[0]->SetTempo(tmp); 
if (intest[1]!=NULL) intest[1]->SetTempo(tmp); 
if (intest[2]!=NULL) intest[2]->SetTempo(tmp); 
};

Tempo Battuta::GetTempo() 
{ return intest[0]->GetTempo(); }

void Battuta::CopyIntest(short current_staff, Intestazione* intes) 
{ intest[current_staff]->Copy(intes); }

void Battuta::CopyAllIntest(Intestazione** intes) 
{ 
	for(int staff=0;staff<NumberOfStaff;staff++)
		intest[staff]->Copy(intes[staff]);
}

void Battuta::Setup(VUnit vf,VUnit MaxWdt) // Calcola il numero di Figure di Battuta,
{
	
	VU2Figure = vf;
	
	for(short i = 0; i<NUM_MAX_LAYER; i++)
	{
		if (LayerVector[i]!=NULL) LayerVector[i]->Calc_Fig_Layer();
		
	}
	
	SetVU(MaxWdt);  
};  


void Battuta::SetVU(VUnit MaxWdt)
{
	VUnit vf, v2r, sp; 
	int spmax = 0;
	
	if(VU2Figure>0)
		vf = VU2Figure;
	else
	{
		vf=0;
		for(int s=0; s<GetNumberOfStaff(); s++)
		{
			VUnit tmp = intest[s]->GetWidth(); 
			if(tmp>vf)
				vf=tmp;
		}
	}
	
	if ((tipoLeggio==LEGGIO_MASAE) || (tipoLeggio==LEGGIO_MASE))
	{
		if (AutoLineBreak==0)
		{
			for(int i = 0; i< NUM_MAX_LAYER; i++)
			{
				
				if (LayerVector[i]!=NULL && (LayerVector[i]->GetFirstF()) != NULL)
				{
					sp=UL2VU(LayerVector[i]->GetSpTotale()+DirSp2FirstFig[i][currentViewType]+DirSp2FirstFigUt[i][currentViewType]);
					if (sp > spmax) spmax = sp; 
				}
			}
		}
		else 
		{
			for(int i = 0; i< NUM_MAX_LAYER; i++)
			{
				if (LayerVector[i]!=NULL && (LayerVector[i]->GetFirstF())!=NULL)
				{
					sp=UL2VU(LayerVector[i]->GetSpTotale()+DirSp2FirstFigLine[i][currentViewType]+DirSp2FirstFigUt[i][currentViewType]);	
					if (sp > spmax) spmax = sp;
				}
			}		
			
			
		}
	}
	
	if ((tipoLeggio==LEGGIO_DLIOO) || (tipoLeggio==LEGGIO_PDLOO)) 
	{
		if (AutoLineBreak==0)
		{
			for(int i = 0; i< NUM_MAX_LAYER; i++)
			{
				if (LayerVector[i]!=NULL && (LayerVector[i]->GetFirstF())!=NULL)
				{
					sp=UL2VU(LayerVector[i]->GetSpTotale()+OrcSp2FirstFig[i][currentViewType]+OrcSp2FirstFigUt[i][currentViewType]);
					if (sp > spmax) spmax = sp;
				}
			}
		}
		else
		{
			for(int i = 0; i< NUM_MAX_LAYER; i++)
			{
				if (LayerVector[i]!=NULL && (LayerVector[i]->GetFirstF())!=NULL)
				{
					sp=UL2VU(LayerVector[i]->GetSpTotale()+OrcSp2FirstFigLine[i][currentViewType]+OrcSp2FirstFigUt[i][currentViewType]);
					if (sp > spmax) spmax = sp;
				}
			}
		}
	}
	
	
	SetVU2Lft(intest[0]->GetVU2Lft());
	
	
	if (ptrBarraPrec!=NULL)
		v2r=DIST_BATT_INTEST + vf 
		+ spmax+DIST_1FIG
		+ xBarra + ptrBarraPrec->GetVU2Rgt() + ptrBarra->GetVU2Rgt();
	else
		v2r=DIST_BATT_INTEST + vf 
		+ spmax+DIST_1FIG
		+ xBarra + ptrBarra->GetVU2Rgt();
	
	if (MaxWdt!=0)        
	{
		xBarra+=MaxWdt-v2r;
		SetVU2Rgt(MaxWdt);
	}
	else SetVU2Rgt(v2r);
}


NumCode Battuta::InsNota(int Altezza, Bool SegnoAltezza, ClassID figID, short current_layer,short current_staff,
						 TipoInserimento GamboUp, Bool inAccordo, NumCode figCode,
						 Bool InsInTesta, NoteHead_ID ht,NumCode thisCode)
{
	short AltSpazio=0;
	Bool found=TRUE;
	
	if(LayerVector[current_layer]==NULL)  
		LayerVector[current_layer]=new Layer;
	
	NumCode ret=LayerVector[current_layer]->InsNota(this,Altezza, SegnoAltezza, figID,current_layer,current_staff, GamboUp, inAccordo, figCode, InsInTesta, ht);
	if(ret!=0 && Battuta::GetAutoBeaming())
		BeamingAuto();
	
	
	return ret;
}



/* Quando crea un nuovo accordo, sostituisce la figura precedente col nuovo accordo
e assegna al nuovo accordo il nc della figura precedente (che mantiene a sua volta lo stesso nc).
Se aggiungo una nota ad un accordo gli assegno un nuovo nc (secondo il CodeCounter della battuta).

*/


Bool Battuta::InsInAccordo(Nota *pNota, Figura *pFig, Layer *pLayer, 
						   TipoInserimento GamboUp, short current_layer,short current_staff)
{
	ClassID FigID=pFig->GetID();
	DrawObject *drw;
	drw = pNota; 
	
	if(LayerVector[current_layer]==NULL)  
		LayerVector[current_layer]=new Layer;
	
	NumCode ret=LayerVector[current_layer]->InsInAccordo(this,pNota,pFig, GamboUp,current_layer,current_staff);
	return ret;
}

NumCode Battuta::InsPausa(int Altezza, Bool SegnoAltezza, ClassID figID, short current_layer,short current_staff,
						  TipoInserimento AltVariabile, Bool inAccordo, NumCode figCode,
						  Bool InsInTesta,NumCode thisCode)
{
	short AltSpazio=0;
	
	if(LayerVector[current_layer]==NULL)  
		LayerVector[current_layer]=new Layer;
	
	NumCode ret=LayerVector[current_layer]->InsPausa(this,Altezza, SegnoAltezza, figID, current_layer,current_staff, AltVariabile, inAccordo, figCode, InsInTesta,thisCode);
	if(ret!=0 && Battuta::GetAutoBeaming())
		BeamingAuto();
	
	return ret;
}    

NumCode Battuta::InsSpazio(short current_layer, NumCode figCode, Bool InsInTesta, short CodeCommand,short current_staff,NumCode thisCode) 
{	
	if(LayerVector[current_layer]==NULL)  
		LayerVector[current_layer]=new Layer;
	GiustificaBattuta(Gettipogiust(),Getkgiustificazione());
	
	return LayerVector[current_layer]->InsSpazio(this,current_layer, figCode, InsInTesta,CodeCommand, current_staff,positionrelat,thisCode);
}

NumCode Battuta::Note2GruppoNote(short current_layer, TipoInserimento tpIns, NumCode figCode1, NumCode figCode2)
{
	
	if(LayerVector[current_layer]==NULL)  
		return 0;
	return LayerVector[current_layer]->Note2GruppoNote(this, current_layer, tpIns, figCode1, figCode2);
}


Bool Battuta::GruppoNote2Note(GruppoNote* gruppo) 
{
	Layer *LayerFig; 
	
	if(gruppo==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_GRUPPO_NON_PERMESSO","Battuta","Note2GruppoNote()");
		return FALSE;
	}
	
	for(int current_layer=0; current_layer<NUM_MAX_LAYER; current_layer++)
	{
		LayerFig=LayerVector[current_layer];
		if(LayerFig!=NULL && LayerFig->FindFig(gruppo) && LayerFig->GruppoNote2Note(gruppo,this,current_layer))
			break;
	}
	
	return TRUE;
	
}


void Battuta::EliminaGruppi()
{
	for (short current_layer=0;current_layer<NUM_MAX_LAYER;current_layer++)
	{
		if (LayerVector[current_layer]!= NULL)
			LayerVector[current_layer]->EliminaGruppi(this,current_layer);
	}
	
}

void Battuta::EliminaGruppi(int nLayer)
{
	if (LayerVector[nLayer]!= NULL)
		LayerVector[nLayer]->EliminaGruppi(this,nLayer);
}

void Battuta::Select()
{
	DrawObject::Select();
	//Scr->SetColor((ColorIcon)XOR_SELECTED_COLOR);
	//Scr->SetDrwMode(XORMode);
	//Scr->Rect(class Rectangle(*this));
	//Scr->SetDrwMode(DefMode);	
}

void Battuta::Deselect()
{
	DrawObject::Deselect();
	/*Select();*/
}


void Battuta::EraseBattuta()
{
	short nLayer=1;
	short AltSpazio = 8;
	
	for (short i=0;i<NUM_MAX_LAYER;i++)
	{
		if(LayerVector[i]!=NULL)
			LayerVector[i]->ClearDeep();
	}
	
}


Bool Battuta::DeleteSym(DrawObject *sym)
{
	Figura *pFig;
	short nSpazi;
	Layer *pLayer; 
	short current_staff;
	short nLayer = 0; 
	short AltSpazio = 8;
	
	pLayer = LayerVector[0]; 
	
	while(nLayer<NUM_MAX_LAYER) 
	{
        if (pLayer==NULL){
			nLayer++;
			pLayer=LayerVector[nLayer];
			continue;
        }
        pFig = pLayer->GetFirstFig();
        while(pFig!=NULL)
        {
			if(pFig==sym)
			{
				if(pLayer->GetLastGruppoNote()==NULL)
				{
					current_staff= pFig->GetNStaff();
					pLayer->DelPtrFig(pFig, this, nLayer,current_staff); 
					if ((pFig->GetID()==CL_PGENERICA) && (ptrNumGrande!=NULL)) 
					{
						delete ptrNumGrande;
						ptrNumGrande=NULL;
					}
					delete pFig;
					if(Battuta::GetAutoBeaming())
						BeamingAuto();
					return TRUE;
				}
				else
				{
					// se e' una pausa viene sostiuita con uno spazio lasciando il gruppo note.
					switch(pFig->GetID())
					{
					case CL_PSEMIBREVE:
					case CL_PMINIMA:
					case CL_PSEMIMINIMA:
					case CL_PCROMA:
					case CL_PSEMICROMA:
					case CL_PBISCROMA:
					case CL_PSEMIBISCROMA:
					case CL_PFUSA:
						pLayer->GetLastGruppoNote()->DelPtr(pFig);
						delete pFig;
						if(Battuta::GetAutoBeaming())
							BeamingAuto();
						return(TRUE);
					case CL_PGENERICA:
						pLayer->GetLastGruppoNote()->DelPtr(pFig);
						delete pFig;
						// cancello anche il numero !
						if (ptrNumGrande!=NULL) delete ptrNumGrande;
						ptrNumGrande=NULL;
						if(Battuta::GetAutoBeaming())
							BeamingAuto();
						return(TRUE);
					}
					// se invece e' una nota del gruppo note viene "sbimato" e quindi sostituita la nota
					// con lo spazio.
					GruppoNote *lastGruppoNote=pLayer->GetLastGruppoNote();
					Figura *fig=lastGruppoNote->ExtractFigura(this, nLayer);
					while(fig!=NULL)
					{
						pLayer->InsertAfterF(lastGruppoNote,fig);
						fig=lastGruppoNote->ExtractFigura(this, nLayer);
					}
					pLayer->DelPtr(lastGruppoNote);
					
					pLayer->DelPtr(pFig);
					delete pFig;
					if(Battuta::GetAutoBeaming())
						BeamingAuto();
					return TRUE;
				}
			}
			else if(pLayer->GetLastGruppoNote()==sym)
			{
				nSpazi = pFig->GetNumSpaces();
				// Il gruppo viene sostituito con spazi.
				pLayer->DelPtr(pFig);
				delete pFig;
				if(Battuta::GetAutoBeaming())
					BeamingAuto();
				return TRUE;
			}
			else
			{
				
				// Chiede alla figura se contiene il simbolo.
				Bool puntoval = sym->GetID()==CL_PUNTOVALORE;
				if(pFig->DeleteSym(sym))
				{
					if(pFig->GetID()==CL_ACCORDO && ((Accordo*)pFig)->GetNumObj()==1)
					{
						Accordo* pAcc=(Accordo*)pFig;
						Nota *pnota=(Nota*)pAcc->GetFirstF();
						ClassID nID=pnota->GetID();
						pnota->TransSymb((Nota*)pFig);
						pnota->SetCoda(TRUE);
						pnota->SetNumericCode(pFig->GetNumericCode());
						
						if(pLayer->GetLastGruppoNote()==NULL)
						{
							ListaIntEst* pListaIE = GetParentScore()->GetListaIntervalli();
							pListaIE->ReplaceIntEstFig(pFig, pnota);
							pLayer->ChangeF(pFig,pnota);
							if (immissione.ImmRule(nID, pnota, this, NULL, nLayer))  
							{ 
								if (pnota->GetNotina())
									pnota->SetGamboUp();
								else
								{ 
									Bool gamboupdwn = TRUE;
									int lgambo = int(posizionam.PosRule(pnota, pnota, this,
										gamboupdwn, nLayer));
									pnota->SetLGambo(lgambo);
								}
							}	  
							else 
							{  
								if (pnota->GetNotina())
									pnota->SetGamboDwn();
								else
								{ 
									Bool gamboupdwn = FALSE;
									int lgambo = int(posizionam.PosRule(pnota, pnota, this,
										gamboupdwn, nLayer));  
									pnota->SetLGambo(-lgambo);
								}
							}
						}
						else
						{
							GruppoNote *pGrp=pLayer->GetLastGruppoNote();
							if(pGrp->GetFirstNota()==pFig)
								pGrp->SetFirstNota(pnota);
							if(pGrp->GetLastNota()==pFig)
								pGrp->SetLastNota(pnota);
							pGrp->ChangeF(pFig,pnota);
						}
						
						//delete pFig;
					}
					if(puntoval && Battuta::GetAutoBeaming())   BeamingAuto();
					return TRUE;
				}  
			}
			pFig = pLayer->GetNextFig();
        }
        // Imposta Layer 2 per rieseguire il ciclo.
        nLayer++;
        pLayer = LayerVector[nLayer];
        AltSpazio = 0;
	}
	
	
	if(sym==ptrNumGrande)
	{
        delete ptrNumGrande;
        ptrNumGrande=NULL;
        return TRUE;
	}
	if(sym==ptrScansione)
	{
        delete ptrScansione;
        ptrScansione=NULL;
        return TRUE;
	}
	if(sym==ptrLettera)
	{
        delete ptrLettera;
        ptrLettera=NULL;
        return TRUE;
	}
	if(sym==ptrTSalto)
	{
        delete ptrTSalto;
        ptrTSalto=NULL;
        return TRUE;
		
    }
    if(sym==ptrRiferimento)
    {
		delete ptrRiferimento;
		ptrRiferimento=NULL;
		return TRUE;
    }
    if(sym==ptrMovimento)
    {
		delete ptrMovimento;
		ptrMovimento=NULL;
		return TRUE;
    }
	
	return FALSE;
}


Bool Battuta::InsertHit(const Point& posiz, short current_layer, Bool inAccordo,
						Bool& questaBat, NumCode& Altezza, Bool& SegnoAltezza,
						NumCode& FigCode,NumCode& InsInTesta, short& nStaff)
{  
	
	Figura *pf;
	short dy=-1;
	short nfig=0; // Numero della figura da cercare nel layer.
	Bool found;
	NumCode FigCodePrev;
	Intestazione *pintest;
	
	pintest = intest[0];
	
	int ascissahit, ascissa, ascissaPrev;
	int tmp1=0;
	
	// Da` per scontato che la battuta sia stata "colpita" nella selezione
	// con BATPART.
	
	
	if(class Rectangle(*this).hasIn(posiz))
		questaBat=TRUE;    
	else 
		questaBat=FALSE;
	
	InsInTesta = FALSE;
	
	
	VUnit pentHeight;
	VUnit spc = GetVU2Up()-((GetParentScore()->GetNCorde(0)-1)*VU_PER_ROW); 
	
	nStaff=GetNumberOfStaff()-1;
	Altezza=0;
	int dist=0,space =0;
	int selspace = ((GetAbsPos().y+GetParentScore()->GetBaseVU2Up(0)+spc)-posiz.y);
	for(int i=0;i<GetNumberOfStaff();i++)
	{
		pentHeight=(GetParentScore()->GetNCorde(i)-1)*VU_PER_ROW;
		space += pentHeight+2*spc;
		if(space >= selspace)
		{
			nStaff = i;
			break;
		}
	}
	
	if(nStaff==0)
		dist = 0;
	else
	{
		if((nStaff==1)||(nStaff==2))
			for(int i=0;i<nStaff;i++)
				dist += GetParentScore()->GetBaseVU2Dwn(i)+2*spc+GetParentScore()->GetBaseVU2Up(i+1);
	}
	
	Altezza = ((posiz.y-(GetAbsPos().y-dist)+VU_PER_ROW/4)*2)/VU_PER_ROW;
	if(posiz.y-(GetAbsPos().y-dist)<-1)
		Altezza--;
	
	
	
	if (Altezza > 0) SegnoAltezza = TRUE;
	else
	{
		Altezza = - Altezza; // sarebbe a dire Altezza = abs(Altezza);
		SegnoAltezza = FALSE;    
	}
	
	
	VUnit vf=(VU2Figure>0 ? VU2Figure : pintest->GetWidth());
	
	Layer *pLayer; 
	
	
	// Esamina solo il layer indicato negli argomenti.
	pLayer = LayerVector[current_layer]; 
	
/*	if (pLayer==NULL) 
		pLayer = LayerVector[current_layer]=new Layer;
	
	
	pf = pLayer->GetFirstFig(); */

	//FFF
	if (pLayer==NULL)
		pf = NULL;
	else
		pf = pLayer->GetFirstFig();
	//FFF

	found = FALSE;
	
	// due diversi scorrimenti per inaccordo e non?
	//// scorri la lista figure     
	ascissahit=posiz.x; // ascissa dello hit 
	// dentro la battuta
	ascissa=(GetAbsPos().x + vf); // valore che incremento all'interno 
	// della battuta
	
	if(inAccordo)
	{
		// Inserimento in accordo.
		if(pf==NULL) 
			return FALSE;
		else
		{
			// scorri la lista delle figure
			FigCodePrev = FigCode;
			ascissaPrev=0;
			while(pf!=NULL && !found)
			{
				// questa e'la posizione del secondo
				ascissa=pf->GetAbsPos().x;
				FigCode = pf->GetNumericCode();
				if (ascissahit<ascissa)  
				{
					found = TRUE;
					// verifica a quale e'piu'vicino
					if (abs(ascissa-ascissahit)>abs(ascissahit-ascissaPrev))  
						FigCode = FigCodePrev;
				}
				else
				{
					ascissaPrev=ascissa;
					FigCodePrev = FigCode;
				}
				pf=pLayer->GetNextFig();
			}
		}
		return TRUE;
	}
	else
	{
		// Inserimento fra le figure.
		if(pf==NULL)
		{
			InsInTesta = TRUE;
			positionrelat=0;
			return TRUE;
		}
		else
		{
			ascissaPrev=ascissa;
			FigCodePrev=0;
			while(!found && pf!=NULL)
			{
				FigCode=pf->GetNumericCode();
				if(pf->GetID()!=CL_SPAZIO)
				{
					ascissa=pf->GetAbsPos().x;
					if(ascissahit<ascissa)
					{
						FigCode=FigCodePrev;
						found = TRUE;
						if(FigCode==0)
							InsInTesta = TRUE;
						
						// se  uno spazio mi interessa ascissaprec e ascissa(next) per calcolare 
						//     relpos da scrivere nello spazio
						positionrelat=int((((float)ascissahit-(float)ascissaPrev)/((float)ascissa-(float)ascissaPrev)*100)); 
					}
					ascissaPrev=ascissa;
				}
				FigCodePrev=FigCode;
				pf=pLayer->GetNextFig();
			}
			//inserimento in coda
			if (!found)
			{
				if(ptrBarra!=NULL)
				{
					ascissa=ptrBarra->GetAbsPos().x;
					positionrelat=int((((float)ascissahit-(float)ascissaPrev)/((float)ascissa-(float)ascissaPrev)*100)); 
				}
				else
					positionrelat=100;						
			}
			// N.B. Anche se la figura non e` stata trovata, restituisce l'ultima
			// figura del layer.
			return TRUE;
		}
	}
	return FALSE;   
}		  

void Battuta::AddFig(Figura* fig,short current_layer) 
{ 
	if(LayerVector[current_layer]==NULL)
		LayerVector[current_layer]=new Layer;
	
	fig->SetPBattuta(this);
	LayerVector[current_layer]->InsertEnd(fig); 
}

void Battuta::SetNumberOfStaff(short i) //  Gli staff vanno da 1 a 3
{
	if(i<1 || i>3) NumberOfStaff = 1;
	NumberOfStaff = i;
	
	for(int k=1; k<NumberOfStaff; k++)
		if(intest[k]==NULL)
		{
			intest[k]=new Intestazione;
			if(k==1)
			{
				intest[k]->SetChiave(new CBasso);
				intest[k]->SetArmatura(DO_maggiore);
			}
			else if(k==2)
			{
				intest[k]->SetChiave(new CMezzoSoprano);
				intest[k]->SetArmatura(DO_maggiore);
			}
			*intest[k]->GetPtrTemp()=intest[0]->GetTempo();
		}
}

short Battuta::GetNumberOfStaff(void)
{
	return (NumberOfStaff);
}

void Battuta::SetupGambi() 
{
	for (int i=0; i<NUM_MAX_LAYER;i++)
	{ 
		if (LayerVector[i]!=NULL)
			LayerVector[i]->SetupGambi(this,i);
	}
}


void Battuta::GiustificaBattuta(TipoGiust tp,double kGiust, bool clearSpUt)
{ 
	// dichiaro un oggetto di classe giustificazione
	Giustificazione g;
	int nfig=GetNumeroFig();
	
	if(nfig==0)
		return;
	
	SetupDurata();
	
	g.Costruisci(2*nfig,GetNUsedLayers());
	g.Inserimento(this);
	
	g.Sincronizza();
	switch(tp)
	{
	case G_LOG:
		
		g.GiustificaLog(kGiust);
		
		break;
	case G_LIN:
		
		g.GiustificaLin(kGiust);
		
		break;
	}
	
	if ((tipoLeggio==LEGGIO_MASAE) || (tipoLeggio==LEGGIO_MASE)) g.SetUpSpG(clearSpUt);
	if ((tipoLeggio==LEGGIO_DLIOO) || (tipoLeggio==LEGGIO_PDLOO)) g.SetUpSpOr(clearSpUt);
	
	if ((tipoLeggio==LEGGIO_DLIOO) || (tipoLeggio==LEGGIO_PDLOO))
	{
		Settipogiustparte(tp);
		Setkgiustificazioneparte(kGiust);	 
	}
}

void Battuta::SetPos(Pentagramma *pentagr,const Point& posizione)
{ 
	Figura *fig,*figtmp=NULL; 
	Layer *pLayer; 
	VUnit vf, cy;
	VUnit MultiStaffDist;
	VUnit xFigPos[NUM_MAX_LAYER]; 
	Bool flag;
	VUnit prova[NUM_MAX_LAYER], anchorpos[NUM_MAX_LAYER]; 
	int i;
	
	for(i=0;i<NUM_MAX_LAYER;i++) 
		prova[i]=0;
	for(i=0;i<NUM_MAX_LAYER;i++) 
		anchorpos[i]=0;
	
	
	
	/////////////////////////////////////////////////////////////////////////
	// La spaziatura non si basa piu' sulla stessa distanzaFig uguale per tutte le
	// figure, ma sara' personalizzata
	
	DrawObject::SetPos(pentagr,posizione);
	
	int MaxInt=0;
	for(int staff=0;staff<NumberOfStaff;staff++)
	{
		if(pentagr!=NULL)
		{
			intest[staff]->SetPos(&pentagr[staff],Point(DIST_BATT_INTEST+posizione.x,0)); 
			pentagr[staff].SetIsUsed(true);
		}
		else
			intest[staff]->SetPos(NULL,Point(DIST_BATT_INTEST+posizione.x,0)); 
		
		if(intest[staff]->GetWidth()>MaxInt) MaxInt=intest[staff]->GetWidth();
	}
	vf = (VU2Figure>0 ? VU2Figure : MaxInt); 
	
	
	if(ptrBarraPrec!=NULL)
	{
		ptrBarraPrec->SetPos(this,Point(vf + xBarra,0));
		MultiStaffDist=GetParentScore()->GetBaseVU2Dwn()-GetParentScore()->GetBaseVU2Dwn(0);  // DB
		ptrBarraPrec->SetProlDwnMultiStaff(MultiStaffDist); 
		vf += ptrBarraPrec->GetVU2Rgt();
	}
	
	if ((tipoLeggio==LEGGIO_MASAE) || (tipoLeggio==LEGGIO_MASE))
	{
		if (AutoLineBreak==0) // line break disabilitato
		{  
			for(int i=0;i<NUM_MAX_LAYER;i++)
			{
				xFigPos[i] = DIST_BATT_INTEST + vf + DIST_1FIG + UL2VU(DirSp2FirstFig[i][currentViewType]+DirSp2FirstFigUt[i][currentViewType]);
			}
			
		}
		if (AutoLineBreak==1)
		{
			for(int i=0;i<NUM_MAX_LAYER;i++)
			{
				xFigPos[i] = DIST_BATT_INTEST + vf + DIST_1FIG + UL2VU(DirSp2FirstFigLine[i][currentViewType]+DirSp2FirstFigUt[i][currentViewType]); 
			}
		}
	}	
	else if ((tipoLeggio==LEGGIO_DLIOO) || (tipoLeggio==LEGGIO_PDLOO)) 
	{
		if (AutoLineBreak==0)// line break disabilitato 			
		{
			for(int i=0;i<NUM_MAX_LAYER;i++)
			{
				xFigPos[i] = DIST_BATT_INTEST + vf + DIST_1FIG + UL2VU(OrcSp2FirstFig[i][currentViewType]+OrcSp2FirstFigUt[i][currentViewType]);
			}
		}
		if (AutoLineBreak==1)
		{
			for(int i=0;i<NUM_MAX_LAYER;i++)
			{
				xFigPos[i] = DIST_BATT_INTEST + vf + DIST_1FIG + UL2VU(OrcSp2FirstFigLine[i][currentViewType]+OrcSp2FirstFigUt[i][currentViewType]); 
			}
		}
	}
	
	for(int layer=0;layer<NUM_MAX_LAYER; layer++)
	{
		if(LayerVector[layer]!=NULL)
		{
			LayerVector[layer]->SetPosLayer(pentagr,this,xFigPos[layer],layer); 
		}
	}
	
	if(ptrBarra!=NULL)
	{
		ptrBarra->SetPos(this,Point(GetWidth()-ptrBarra->GetVU2Rgt(),0));
		MultiStaffDist=GetParentScore()->GetBaseVU2Dwn()-GetParentScore()->GetBaseVU2Dwn(0);	// DB
		ptrBarra->SetProlDwnMultiStaff(MultiStaffDist); // GB
	}
	
	if(ptrNumBattuta!=NULL)
	{
		// Suppone che ptrNumBattuta->SetNumero sia gia` stato eseguito
		// dal metodo SetBattuta.
		// ptrNumBattuta->SetNumero(GetNumProgress());
		ptrNumBattuta->SetPos(this,Point(0, VU_PER_ROW*6));
	}
	
	if (ptrNumGrande!=NULL)
	{
		if (ptrNumGrande->GetID() == CL_NUMPAUSA)
		{ 
			flag=TRUE;
			
			for(int layer=0;layer<NUM_MAX_LAYER; layer++)
			{	
				pLayer = GetPLayer(layer);	
				if (pLayer!=NULL)
				{	
					fig=pLayer->GetFirstF();
					while((fig!=NULL) && (flag))
					{
						if (fig->GetID() == CL_PGENERICA)
						{
							ptrNumGrande->SetPos(fig,Point(0, VU_PER_ROW*4));
							flag=FALSE;
						}
						fig=pLayer->GetNextF(fig);
					}
				}
			}
		}
		else
		{
			if (ptrNumGrande->GetID() == CL_TNUMERICO)
			{ 
				flag=TRUE;
				for(int layer=0;layer<NUM_MAX_LAYER; layer++)
				{		
					pLayer = GetPLayer(layer);
					if (pLayer!=NULL)
					{
						fig=pLayer->GetFirstF();
						while((fig!=NULL) && (flag))
						{
							if (fig->GetID() == CL_RIPBATTUTA)
							{
								ptrNumGrande->SetPos(fig,Point(0, VU_PER_ROW*4));
								flag=FALSE;
							}
							fig=pLayer->GetNextF(fig);
						}
					}
				}
			}
			else
			{
				ptrNumGrande->SetPos(this,Point(GetWidth()/2, VU_PER_ROW*4));
			}
		}
	}
	
	cy = VU_PER_ROW*8;
	
	if (FirstColonna || tipoLeggio==LEGGIO_DLIOO)
		if (ptrMovimento!=NULL)
		{
			ptrMovimento->SetPos(this,Point(0, cy));
			cy += ptrMovimento->GetVU2Up() + VU_PER_ROW;
		}
		
		if (ptrScansione!=NULL)
		{
			ptrScansione->SetPos(this,Point(0, cy));
			cy += ptrScansione->GetVU2Up() + VU_PER_ROW;
		}
		
		if (ptrTSalto!=NULL)
		{
			VUnit scarto_x = ptrTSalto->GetWidth();
			VUnit scarto_y = ptrTSalto->GetHeight();
			ptrTSalto->SetPos(this,Point(GetWidth()- scarto_x, -scarto_y));
		}
		if (ptrRiferimento!=NULL)
		{
			ptrRiferimento->SetPos(this,Point(0, cy));
			cy += ptrRiferimento->GetVU2Up() + VU_PER_ROW;
		}
		if (FirstColonna || tipoLeggio==LEGGIO_DLIOO)
			if (ptrLettera!=NULL)
			{
				ptrLettera->SetPos(this,Point(0, cy));
				cy += ptrLettera->GetVU2Up() + VU_PER_ROW;
			}
			
}

void Battuta::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());
	
	Layer *pLayer; 
	Intestazione *pIntest; 
	
	for(short inte=0; inte < NumberOfStaff ; inte++)
	{
		pIntest = intest[inte];
		pIntest->Draw();
	}
	
	for(short i=0; i < NUM_MAX_LAYER; i ++)
	{
		pLayer = LayerVector[i];
		if(pLayer!=NULL)
            pLayer->Draw();
	}
	
	
	if(ptrBarra!=NULL) ptrBarra->Draw();
	
	if(ptrBarraPrec!=NULL) ptrBarraPrec->Draw();
	
	if(ptrNumBattuta!=NULL) ptrNumBattuta->Draw();
	
	if(ptrNumGrande!=NULL) ptrNumGrande->Draw();
	if(ptrScansione!=NULL) ptrScansione->Draw();
	if(ptrTSalto!=NULL) ptrTSalto->Draw();
	
	if(ptrRiferimento!=NULL) ptrRiferimento->Draw();       
	if (FirstColonna || tipoLeggio==LEGGIO_DLIOO)
	{
		if(ptrMovimento!=NULL) ptrMovimento->Draw();
		if(ptrLettera!=NULL) ptrLettera->Draw();
	}

	if(isSelected)	{
		DrawContour();
	}

	Scr->SetColor(oldcol);
}


Bool Battuta::Hit(const Point& posizione,SelObj selObj,DrawObject *& drw,
				  SymPath& symPath)
{
	Figura *fig;
	Layer *pLayer=NULL;
	
	
	drw=NULL;
	
	
	if(selObj==BATPART_SEL)
	{
		if ((tipoLeggio==LEGGIO_MASAE) || (tipoLeggio==LEGGIO_MASE))
		{
            if(posizione.x>=(GetAbsPos().x - GetVU2Lft()) &&
				posizione.x<=(GetAbsPos().x + GetVU2Rgt()))
            {
				drw=this;
				symPath.Array[BATTUTA_PATH_LEVEL]=GetNumericCode();
				return TRUE;
            }
            return FALSE;
		}
		else
		{
            if(posizione.x>=(GetAbsPos().x - GetVU2Lft()) &&
				posizione.x<=(GetAbsPos().x + GetVU2Rgt()) &&
				posizione.y>=(GetAbsPos().y - GetVU2Dwn()) &&
				posizione.y<=(GetAbsPos().y + GetVU2Up()))
            {
				drw=this;
				symPath.Array[BATTUTA_PATH_LEVEL]=GetNumericCode();
				return TRUE;
            }
            return FALSE;
		}
	}
	
	if(selObj==BATTUTA_SEL)
	{
		if(class Rectangle(*this).hasIn(posizione))
		{
            drw=this;
            symPath.Array[BATTUTA_PATH_LEVEL]=GetNumericCode();
            int nStaff;
            // determina nStaff...
			VUnit pentHeight=(GetNStaffLine()-1)*VU_PER_ROW;
            VUnit spc = GetVU2Up()-pentHeight; // GB
            nStaff = ((GetAbsPos().y+pentHeight+spc)-posizione.y)/(pentHeight + 2*spc); //GB
            
            symPath.Array[STAFF_PATH_LEVEL]=nStaff;
            return TRUE;
		}
		return FALSE;
	}
	
	// Imposta il livello piu' alto per le figure.
	symPath.Level = FIGURA1_PATH_LEVEL;
	
	
	for(int layer=0;layer<NUM_MAX_LAYER;layer++)
	{
		pLayer=GetPLayer(layer);
		if(pLayer!=NULL)
		{
            fig = pLayer -> GetFirstF();
            while(fig!=NULL)
            {
				if( fig->Hit(posizione,selObj,drw,symPath) )
				{
					symPath.Array[BATTUTA_PATH_LEVEL]=GetNumericCode();
					symPath.Array[LAYER_PATH_LEVEL]=layer + 1; //+1?
					return TRUE;
				}  
				fig = pLayer -> GetNextF(fig);
            }
		}
	}
	
	
	
	if(drw==NULL && selObj==ANY_SEL)
	{
		if(ptrNumGrande!=NULL &&
            ptrNumGrande->Hit(posizione,selObj,drw,symPath))
		{ 
            symPath.Array[BATTUTA_PATH_LEVEL]=GetNumericCode();
            symPath.Array[symPath.Level]=GetNumericCode();
            return(TRUE);
		}
		
		if(ptrScansione!=NULL &&
            ptrScansione->Hit(posizione,selObj,drw,symPath))
		{ 
            symPath.Array[BATTUTA_PATH_LEVEL]=GetNumericCode();
            symPath.Array[symPath.Level]=GetNumericCode();
            return(TRUE);
		}
		
		
		if(ptrTSalto!=NULL &&
            ptrTSalto->Hit(posizione,selObj,drw,symPath))
		{ 
            symPath.Array[BATTUTA_PATH_LEVEL]=GetNumericCode();
            symPath.Array[symPath.Level]=GetNumericCode();
            return(TRUE);
		}
		
		if(ptrRiferimento!=NULL &&
            ptrRiferimento->Hit(posizione,selObj,drw,symPath))
		{ 
            symPath.Array[BATTUTA_PATH_LEVEL]=GetNumericCode();
            symPath.Array[symPath.Level]=GetNumericCode();
            return(TRUE);
		}
		
		if (FirstColonna)
		{
            if(ptrLettera!=NULL &&
				ptrLettera->Hit(posizione,selObj,drw,symPath))
            { 
				symPath.Array[BATTUTA_PATH_LEVEL]=GetNumericCode();
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
            }
            
            if(ptrMovimento!=NULL &&
				ptrMovimento->Hit(posizione,selObj,drw,symPath))
            { 
				symPath.Array[BATTUTA_PATH_LEVEL]=GetNumericCode();
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
            }
            
		}
		if(class Rectangle(*this).hasIn(posizione))
		{
			drw=this;
			symPath.Array[BATTUTA_PATH_LEVEL]=GetNumericCode();
			return TRUE;
		}
	}
	return FALSE;
}



Bool Battuta::DeleteHit(const Point& posiz, NumCode& FigCode, 
						NumCode& InsInTesta)
{
	Bool b;
	short sh;
	Bool sgnAlt;
	short staff;
	
	return InsertHit(posiz, 1, TRUE, b, sh, sgnAlt, FigCode, InsInTesta,staff);
}


Bool Battuta::Hit(const class Rectangle& r,SelObj selObj,short current_layer,
				  SymPath& sp1, SymPath& sp2)
{
	Figura *pFig;
	Layer *pLayer;
	Bool found=FALSE;
	
	pLayer = LayerVector[current_layer];
	
	if(pLayer==NULL)
		return FALSE;
	
	
	// Imposta il livello piu' alto per le figure.
	sp1.Level = FIGURA1_PATH_LEVEL;
	sp2.Level = FIGURA1_PATH_LEVEL;
	
	pFig=pLayer->GetFirstF();
	while(pFig!=NULL)
	{
		if(pFig->Hit(r,selObj,sp1,sp2))
		{
			found = TRUE;
		}
		else if(found)
		{
			// In questo caso il rettangolo e` gia` stato ispezionanto al
			// completo, percio` e` inutile proseguire.
			break;
		}
		pFig = pLayer->GetNextF(pFig);
	}
	if(found)
	{
		if(sp1.Array[BATTUTA_PATH_LEVEL]==0)
		{
			sp1.Array[BATTUTA_PATH_LEVEL] = GetNumericCode();
			sp1.Array[LAYER_PATH_LEVEL]= current_layer+1;//GB
		}
		sp2.Array[BATTUTA_PATH_LEVEL] = GetNumericCode();
		sp2.Array[LAYER_PATH_LEVEL]= current_layer+1;//GB
		return TRUE;
	}
	return FALSE;
}


Bool Battuta::SymbolicHit(SymPath& symPath, SelObj selObj,DrawObject *& drw)
{
	Figura *fig;
	Layer *pLayer; 
	
	
	drw=NULL;
	
	if(GetNumericCode()!=symPath.Array[BATTUTA_PATH_LEVEL])
		return FALSE;
	
	// Produce sullo slave gli stessi effetti che InsertHit ha prodotto
	// sul master.
	
	
	if((selObj==BATTUTA_SEL) || (selObj==BATPART_SEL) || symPath.Level==BATTUTA_PATH_LEVEL)
	{
		drw=this;
		return TRUE;
	}
	
	// Imposta il livello piu' alto per le figure.
	symPath.Level = FIGURA1_PATH_LEVEL;
	
	int current_layer = symPath.Array[LAYER_PATH_LEVEL]-1; 

	if (current_layer >= 0) 
	{
		pLayer = LayerVector[current_layer]; 
		
		if (pLayer==NULL) return FALSE;   
			
		fig=pLayer->GetFirstF();
		while(fig!=NULL)
		{
			if( fig->SymbolicHit(symPath,selObj,drw) )
				return TRUE;
			fig=pLayer->GetNextF(fig);
		}
	}
	
	
	
	if(drw==NULL && selObj==ANY_SEL)
	{
		
		if(ptrNumGrande!=NULL)
			if( ptrNumGrande->SymbolicHit(symPath,selObj,drw) )
				return TRUE;
			
			if(ptrScansione!=NULL)
				if( ptrScansione->SymbolicHit(symPath,selObj,drw) )
					return TRUE;
				
				if(ptrLettera!=NULL)
					if( ptrLettera->SymbolicHit(symPath,selObj,drw) )
						return TRUE;
					
					if(ptrTSalto!=NULL)
						if( ptrTSalto->SymbolicHit(symPath,selObj,drw) )
							return TRUE;
						
						if(ptrRiferimento!=NULL)
							if( ptrRiferimento->SymbolicHit(symPath,selObj,drw) )
								return TRUE;
							
							if(ptrMovimento!=NULL)
								if( ptrMovimento->SymbolicHit(symPath,selObj,drw) )
									return TRUE;
	}
	return FALSE;
}  

Figura* Battuta::GetFig(short current_layer, NumCode nc)
{
	if(current_layer<0 || current_layer>=NUM_MAX_LAYER) current_layer = 0;
	if(LayerVector[current_layer]==NULL) return NULL;
	return (Figura*)(LayerVector[current_layer]->GetFig(nc));
	
}

Figura* Battuta::GetFigTotal(short current_layer, NumCode nc)
{
	
	if(LayerVector[current_layer]==NULL) return NULL;
	return (Figura*)(LayerVector[current_layer]->GetFigTotal(nc));
}

CmdResult Battuta::SymCommand(SymCmd& MyCmd, Battuta *pBattuta)
{ 
	DrawObject *pobj;
	NumCode Appo, Appo2;
	SymCmd SCmd;
	CopiaCommandStruct(&SCmd,&MyCmd);
	
	switch(SCmd.CommandId)
	{
	case CMD_BATTUTA_BSINGOLA:
		if(ptrBarra!=NULL)
			delete ptrBarra;
		ptrBarra=new BSingola;
		ptrBarra->SetLineType((LineType)SCmd.ArgV[0]);	//linetype
		if(ptrBarra==NULL)       
		{     
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","SymCommand()");   
			return CMD_FAIL;
		};
		ptrBarra->SetNStaffLine(NStaffLine);	
		return CMD_OK;
		break;
	case CMD_BATTUTA_BSINGOLA_RIP2B:
		if(ptrBarra!=NULL)
			delete ptrBarra;
		ptrBarra=new BSingola(TRUE);
		if(ptrBarra==NULL)       
		{     
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","SymCommand()");   
			return CMD_FAIL;
		};
		ptrBarra->SetNStaffLine(NStaffLine);	
		return CMD_OK;
		break;
	case CMD_BATTUTA_BDOPPIA:
		if(ptrBarra!=NULL)
			delete ptrBarra;
		ptrBarra=new BDoppia;
		if(ptrBarra==NULL)       
		{     
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","SymCommand()");   
			return CMD_FAIL;
		};
		ptrBarra->SetNStaffLine(NStaffLine);	
		return CMD_OK;
		break;
	case CMD_BATTUTA_BFINALE:
		if(ptrBarra!=NULL)
			delete ptrBarra;
		ptrBarra=new BFinale;
		if(ptrBarra==NULL)       
		{     
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","SymCommand()");   
			return CMD_FAIL;
		};
		ptrBarra->SetNStaffLine(NStaffLine);	
		return CMD_OK;
		break;
	case CMD_BATTUTA_BINIZIORIT:
		if(ptrBarra!=NULL)
			delete ptrBarra;
		ptrBarra=new BInizioRit;
		if(ptrBarra==NULL)       
		{     
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","SymCommand()");   
			return CMD_FAIL;
		};
		ptrBarra->SetNStaffLine(NStaffLine);	
		return CMD_OK;
		break;
	case CMD_BATTUTA_BFINERIT:
		if(ptrBarra!=NULL)
			delete ptrBarra;
		ptrBarra=new BFineRit;
		if(ptrBarra==NULL)       
		{     
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","SymCommand()");   
			return CMD_FAIL;
		}; 
		ptrBarra->SetNStaffLine(NStaffLine);	
		return CMD_OK;
		break;
	case CMD_BATTUTA_BINIZIOFINERIT:
		if(ptrBarra!=NULL)
			delete ptrBarra;
		ptrBarra=new BInizioFine;
		if(ptrBarra==NULL)       
		{     
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","SymCommand()");   
			return CMD_FAIL;
		};  
		ptrBarra->SetNStaffLine(NStaffLine);	
		return CMD_OK;
		break;
	case CMD_BATTUTA_BINVISIBILE:
		if(ptrBarra!=NULL)
			delete ptrBarra;
		ptrBarra=new BInvisibile;
		if(ptrBarra==NULL)       
		{     
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","SymCommand()");   
			return CMD_FAIL;
		};
		ptrBarra->SetNStaffLine(NStaffLine);	
		return CMD_OK;
		break;
	case CMD_BATTUTA_INS_NBREVE:
		Appo=InsNota((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_NBREVE,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode&)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NoteHead_ID)SCmd.ArgV[8],(NumCode&)SCmd.ArgV[6]);
		if(Appo!=0) { MyCmd.ArgV[6]=Appo; MyCmd.ArgV[4]=SCmd.ArgV[4];
		return CMD_OK; }
		return CMD_FAIL;
		
	case CMD_BATTUTA_INS_NSEMIBREVE:
		Appo=InsNota((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_NSEMIBREVE,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode&)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NoteHead_ID)SCmd.ArgV[8],(NumCode&)SCmd.ArgV[6]);
		if(Appo!=0) { MyCmd.ArgV[6]=Appo; MyCmd.ArgV[4]=SCmd.ArgV[4];
		return CMD_OK; }
		return CMD_FAIL;
		
	case CMD_BATTUTA_INS_NMINIMA:
		Appo=InsNota((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_NMINIMA,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode&)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NoteHead_ID)SCmd.ArgV[8],(NumCode&)SCmd.ArgV[6]);
		if(Appo!=0) { MyCmd.ArgV[6]=Appo; MyCmd.ArgV[4]=SCmd.ArgV[4];
		return CMD_OK; }
		return CMD_FAIL;
		
	case CMD_BATTUTA_INS_NSEMIMINIMA:
		Appo=InsNota((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_NSEMIMINIMA,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode&)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NoteHead_ID)SCmd.ArgV[8],(NumCode&)SCmd.ArgV[6]);
		if(Appo!=0) { MyCmd.ArgV[6]=Appo; MyCmd.ArgV[4]=SCmd.ArgV[4];
		return CMD_OK; }
		return CMD_FAIL;
		
	case CMD_BATTUTA_INS_NCROMA:
		Appo=InsNota((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_NCROMA,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode&)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NoteHead_ID)SCmd.ArgV[8],(NumCode&)SCmd.ArgV[6]);
		if(Appo!=0) {  MyCmd.ArgV[6]=Appo; MyCmd.ArgV[4]=SCmd.ArgV[4];
		return CMD_OK; }
		return CMD_FAIL;
		
	case CMD_BATTUTA_INS_NSEMICROMA:
		Appo=InsNota((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_NSEMICROMA,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode&)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NoteHead_ID)SCmd.ArgV[8],(NumCode&)SCmd.ArgV[6]);
		if(Appo!=0) { MyCmd.ArgV[6]=Appo; MyCmd.ArgV[4]=SCmd.ArgV[4];
		return CMD_OK; }
		return CMD_FAIL;
		
	case CMD_BATTUTA_INS_NBISCROMA:
		Appo=InsNota((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_NBISCROMA,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode&)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NoteHead_ID)SCmd.ArgV[8],(NumCode&)SCmd.ArgV[6]);
		if(Appo!=0) { MyCmd.ArgV[6]=Appo; MyCmd.ArgV[4]=SCmd.ArgV[4];
		return CMD_OK; }
		return CMD_FAIL;
		
	case CMD_BATTUTA_INS_NSEMIBISCROMA:
		Appo=InsNota((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_NSEMIBISCROMA,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode&)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NoteHead_ID)SCmd.ArgV[8],(NumCode&)SCmd.ArgV[6]);
		if(Appo!=0) { MyCmd.ArgV[6]=Appo; MyCmd.ArgV[4]=SCmd.ArgV[4];
		return CMD_OK; }
		return CMD_FAIL;
		
	case CMD_BATTUTA_INS_NFUSA:
		Appo=InsNota((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_NFUSA,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode&)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NoteHead_ID)SCmd.ArgV[8],(NumCode&)SCmd.ArgV[6]);
		if(Appo!=0) { MyCmd.ArgV[6]=Appo; MyCmd.ArgV[4]=SCmd.ArgV[4];
		return CMD_OK; }
		return CMD_FAIL;
		
		
	case CMD_BATTUTA_INS_SPAZIO:
		Appo=InsSpazio((short)SCmd.ArgV[1]-1,(NumCode)SCmd.ArgV[4], (Bool)SCmd.ArgV[5],
			(short)SCmd.ArgV[2],(short)SCmd.ArgV[9],(NumCode&)SCmd.ArgV[6]);
		if (Appo!=0) { MyCmd.ArgV[6]=Appo;

		return CMD_OK; }
		return CMD_FAIL;
		
	case CMD_BATTUTA_INS_PSEMIBREVE:
		Appo=InsPausa((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_PSEMIBREVE,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NumCode&)SCmd.ArgV[6]);
		if (Appo!=0) { MyCmd.ArgV[6]=Appo;
		return CMD_OK; }
		return CMD_FAIL; 
		
	case CMD_BATTUTA_INS_PMINIMA:
		Appo=InsPausa((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_PMINIMA,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NumCode&)SCmd.ArgV[6]);
		if (Appo!=0) { MyCmd.ArgV[6]=Appo;
		return CMD_OK; }
		return CMD_FAIL; 
		
	case CMD_BATTUTA_INS_PSEMIMINIMA:
		Appo=InsPausa((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_PSEMIMINIMA,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NumCode&)SCmd.ArgV[6]);
		if (Appo!=0) { MyCmd.ArgV[6]=Appo;
		return CMD_OK; }
		return CMD_FAIL; 
		
	case CMD_BATTUTA_INS_PCROMA:
		Appo=InsPausa((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_PCROMA,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NumCode&)SCmd.ArgV[6]);
		if (Appo!=0) { MyCmd.ArgV[6]=Appo;
		return CMD_OK; }
		return CMD_FAIL; 
		
	case CMD_BATTUTA_INS_PSEMICROMA:
		Appo=InsPausa((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_PSEMICROMA,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NumCode&)SCmd.ArgV[6]);
		if (Appo!=0) { MyCmd.ArgV[6]=Appo;
		return CMD_OK; }
		return CMD_FAIL; 
		
	case CMD_BATTUTA_INS_PBISCROMA:
		Appo=InsPausa((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_PBISCROMA,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NumCode&)SCmd.ArgV[6]);
		if (Appo!=0) { MyCmd.ArgV[6]=Appo;
		return CMD_OK; }
		return CMD_FAIL; 
		
	case CMD_BATTUTA_INS_PSEMIBISCROMA:
		Appo=InsPausa((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_PSEMIBISCROMA,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NumCode&)SCmd.ArgV[6]);
		if (Appo!=0) { MyCmd.ArgV[6]=Appo;
		return CMD_OK; }
		return CMD_FAIL; 
		
	case CMD_BATTUTA_INS_PFUSA:
		Appo=InsPausa((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_PFUSA,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NumCode&)SCmd.ArgV[6]);
		if (Appo!=0) { MyCmd.ArgV[6]=Appo;
		return CMD_OK; }
		return CMD_FAIL; 
		
	case CMD_BATTUTA_INS_PDUEBATT:
		Appo=InsPausa((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_PDUEBATTUTE,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NumCode&)SCmd.ArgV[6]);
		if (Appo!=0) { MyCmd.ArgV[6]=Appo;
		return CMD_OK; }
		return CMD_FAIL; 
		
	case CMD_BATTUTA_INS_PQUABATT:
		Appo=InsPausa((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_PQUATTROBATTUTE,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NumCode&)SCmd.ArgV[6]);
		if (Appo!=0) { MyCmd.ArgV[6]=Appo;
		return CMD_OK; }
		return CMD_FAIL; 
		
	case CMD_BATTUTA_INS_PGENERICA:
		Appo2 = (int)SCmd.ArgV[6];
		Appo=InsPausa((int)SCmd.ArgV[0],(Bool)SCmd.ArgV[7],CL_PGENERICA,(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9],
			(TipoInserimento)SCmd.ArgV[2],(Bool)SCmd.ArgV[3],(NumCode)SCmd.ArgV[4],
			(Bool)SCmd.ArgV[5],(NumCode&)SCmd.ArgV[6]);
		if (Appo!=0) 
		{ 
			
			if(InsNumPausa((unsigned char*)&(SCmd.Text[0])))
			{
				MyCmd.ArgV[6]=Appo;
				return CMD_OK; 
			}
		}
		return CMD_FAIL; 
		
		
	case CMD_BATTUTA_INS_RIPTEMPO:
		Appo=InsRipetizione(CL_RIPTEMPO, (NumCode)SCmd.ArgV[4], (Bool)SCmd.ArgV[5],
			(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9]);
		if (Appo==0) return CMD_FAIL;
		MyCmd.ArgV[6]=Appo;
		return CMD_OK;
		
	case CMD_BATTUTA_INS_RIPBATTUTA:
		Appo=InsRipetizione(CL_RIPBATTUTA, (NumCode)SCmd.ArgV[4], (Bool)SCmd.ArgV[5],
			(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9]);
		if (Appo==0) 
			return CMD_FAIL;
		
		if (ptrNumGrande!=NULL) delete(ptrNumGrande);
		ptrNumGrande = new NumGrande;
		ptrNumGrande->SetFont(nMusicFont);
		if (ptrNumGrande==NULL)
		{
			return CMD_FAIL;
		};   
		if (ptrNumGrande->SetTxt((unsigned char*)&(SCmd.Text[0])))
		{
			return CMD_OK;
		}
		else
		{
			delete ptrNumGrande;
			return CMD_FAIL;
		}		
		
	case CMD_BATTUTA_INS_RIPMEZZABATTUTA:
		Appo=InsRipetizione(CL_RIPMEZZABATTUTA, (NumCode)SCmd.ArgV[4], (Bool)SCmd.ArgV[5],
			(short)SCmd.ArgV[1]-1,(short)SCmd.ArgV[9]);
		if (Appo==0) return CMD_FAIL;
		MyCmd.ArgV[6]=Appo;
		return CMD_OK;
		
	case CMD_SPARTITO_ADD_BEAM:
	case CMD_BATTUTA_NOTE2GRUPPONOTE:
		Appo=Note2GruppoNote((short)SCmd.ArgV[0]-1,(TipoInserimento)SCmd.ArgV[3],(NumCode)SCmd.ArgV[1],(NumCode)SCmd.ArgV[2]);          
		if (Appo==0) return CMD_FAIL;
		MyCmd.ArgV[6]=Appo;
		return CMD_OK;
		
	case CMD_BATTUTA_GRUPPONOTE2NOTE:
		if (SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj))
		{ if(GruppoNote2Note((GruppoNote*)pobj))
		return CMD_OK;
		}
		return CMD_FAIL;
	case CMD_BATTUTA_ELIMINAGRUPPI:
		EliminaGruppi();
		return CMD_OK;
		
	case CMD_BATTUTA_NUGUALE:
		if(InsNumUguale((unsigned char*)SCmd.Text))
			return CMD_OK;
		
	case CMD_BATTUTA_SCANSIONE:
		if(InsScansione((short)SCmd.ArgV[1]))
			return CMD_OK;
		
	case CMD_BATTUTA_TSALTO:
		if(InsTSalto((unsigned char*)SCmd.Text))
			return CMD_OK;
		
	case CMD_BATTUTA_SEGNO:
		if(InsRiferimento(CMD_BATTUTA_SEGNO))
			return CMD_OK;
	case CMD_BATTUTA_CODA:
		if(InsRiferimento(CMD_BATTUTA_CODA))
			return CMD_OK;
		
	case CMD_BATTUTA_LETTERA:
		if(InsLettera((unsigned char*)SCmd.Text))
			return CMD_OK;
		
	case CMD_BATTUTA_MOVIMENTO:
		if(InsMovimento( (unsigned char*)SCmd.Text, (ClassID)SCmd.ArgV[0], 
			(Bool)SCmd.ArgV[1], (int)SCmd.ArgV[2]))
			return CMD_OK;
		
		
	case CMD_BATTUTA_ERASE:
		EraseBattuta();
		return CMD_OK;
		// spazi introdotti dall'utente
	case CMD_BATTUTA_SPACE_ADD:
		{
			NumCode FigCode=SCmd.ArgV[4];
			NumCode InsInTesta=SCmd.ArgV[5];
			int current_layer=SCmd.ArgV[1]-1; 
			
			// se figcode e' uno spazio assegna a figcode la figura prima (!= da spazio)
			// o al limite poni a TRUE insintesta
			
			if (!InsInTesta)
			{
				Figura *pf1;
				pf1=GetFigTotal(current_layer,FigCode);
				if (pf1->GetID()==CL_SPAZIO)
				{
					Layer *pLayer=NULL; 
					// Esamina solo il layer indicato negli argomenti.
					
					pLayer=LayerVector[current_layer];
					// ritorna la prima figura non ancoraggio prima di questo ancoraggio
					// oppure torna true a insintesta
					
					while (pf1!=NULL && pf1->GetID()==CL_SPAZIO)
					{
						pf1=pLayer->GetPrevF(pf1);
						if (pf1==NULL)
						{ 
							InsInTesta=TRUE;
							break;
						}
						if (pf1->GetID()==CL_ACCORDO)
							break;
					}
					if (!InsInTesta)
						FigCode = pf1->GetNumericCode();
				}
			}
			
			/**** fine modifiche per gli ancoraggi ****/
			
			if(!InsInTesta)
			{
				Figura* pFig;
				
				pFig=GetFigTotal(current_layer,FigCode);
				
				if ((tipoLeggio==LEGGIO_MASAE) || (tipoLeggio==LEGGIO_MASE))
					pFig->SetSpDirUt(pFig->GetSpDirUt()+300);
				if ((tipoLeggio==LEGGIO_DLIOO) || (tipoLeggio==LEGGIO_PDLOO)) 
					pFig->SetSpOrcUt(pFig->GetSpOrcUt()+300);
			}
			else
			{
				if ((tipoLeggio==LEGGIO_MASAE) || (tipoLeggio==LEGGIO_MASE))
				{ 
					DirSp2FirstFigUt[current_layer][currentViewType]+=300; //GB
				}  
				if ((tipoLeggio==LEGGIO_DLIOO) || (tipoLeggio==LEGGIO_PDLOO)) 
				{
					OrcSp2FirstFigUt[current_layer][currentViewType]+=300;
				}
			}
		}
		return CMD_OK;
	case CMD_BATTUTA_SPACE_SUB:
		{
			NumCode FigCode=SCmd.ArgV[4];
			NumCode InsInTesta=SCmd.ArgV[5];
			int current_layer=SCmd.ArgV[1]-1; 
			if (!InsInTesta)
			{         
				Figura *pf1;
				pf1=GetFigTotal(current_layer,FigCode);
				if (pf1->GetID()==CL_SPAZIO) 	
				{ 		 
					Layer *pLayer=NULL;  
					// Esamina solo il layer indicato negli argomenti. 
					pLayer=LayerVector[current_layer]; 	
				}           
				// ritorna la prima figura non ancoraggio prima di questo ancoraggio 
				// oppure torna true a insintesta    
				Layer *pLayer=NULL;  
				pLayer=LayerVector[current_layer];
				while (pf1!=NULL && pf1->GetID()==CL_SPAZIO)   
				{ 		  
					pf1=pLayer->GetPrevF(pf1); 
					if (pf1->GetID()==CL_ACCORDO) 
						break;    
					// domanda: e se fosse una notina o un gruppo di notine? da verificare 
					if (pf1==NULL) 		
						InsInTesta=TRUE; 		
				} 		
				if (!InsInTesta) 	
					// assegna il FigCode 
					FigCode = pf1->GetNumericCode(); 
			}  
			
			/**** fine modifiche per gli ancoraggi  ***/
			
			if(!InsInTesta)
			{
				Figura* pFig;
				NumCode FigCode=SCmd.ArgV[4];
				int current_layer=SCmd.ArgV[1]-1; 
				pFig=GetFigTotal(current_layer,FigCode);
				
				if ((tipoLeggio==LEGGIO_MASAE) || (tipoLeggio==LEGGIO_MASE))
					pFig->SetSpDirUt(pFig->GetSpDirUt()-300);
				if ((tipoLeggio==LEGGIO_DLIOO) || (tipoLeggio==LEGGIO_PDLOO))
					pFig->SetSpOrcUt(pFig->GetSpOrcUt()-300);
			}
			else
			{
				int current_layer=SCmd.ArgV[1]-1;
				if ((tipoLeggio==LEGGIO_MASAE) || (tipoLeggio==LEGGIO_MASE))
				{ 
					DirSp2FirstFigUt[current_layer][currentViewType]-=300; //GB
				}  
				if ((tipoLeggio==LEGGIO_DLIOO) || (tipoLeggio==LEGGIO_PDLOO)) 
				{
					OrcSp2FirstFigUt[current_layer][currentViewType]-=300;
				}
			}
		}
		return CMD_OK;
	default:
		short current_staff = SCmd.Path1.Array[STAFF_PATH_LEVEL]; 
		if (SCmd.CommandId>=CMD_BATTUTA_ARMCHV_0D && SCmd.CommandId<=CMD_BATTUTA_ARMCHV_7Bm)//GB
		{
			for(int staff=0;staff<GetNumberOfStaff();staff++)
			{
				intest[staff]->SymCommand(SCmd);
			}
			return CMD_OK;
		}
		if (SCmd.CommandId!=CMD_BATTUTA_TEMPO)
			return intest[current_staff]->SymCommand(SCmd); 
		else if (SCmd.CommandId==CMD_BATTUTA_TEMPO) 
		{ 
			for(int staff=0;staff<GetNumberOfStaff();staff++)
			{
				intest[staff]->SymCommand(SCmd);
			}
			return CMD_OK;
		}
			  }
			  return CMD_UNKNOWN;
}



//
//  Metodi Privati.
//




NumCode Battuta::InsRipetizione(ClassID figID, NumCode figCode, Bool InsInTesta, short current_layer,short current_staff)
{
	
	short AltSpazio=0;
	
	if(LayerVector[current_layer]==NULL)  
		LayerVector[current_layer]=new Layer;
	
	return LayerVector[current_layer]->InsRipetizione(this,figID, figCode, InsInTesta, current_layer,current_staff);
	
	
}

void Battuta::AdjustTremolo(Figura *fig, short current_layer)
{
	Figura *Fig2=NULL;
	short fl;
	int Iden;
	
	Iden = fig->GetID();
	
	if ((Iden != CL_NBREVE) && (Iden != CL_NSEMIBREVE) && 
		(Iden != CL_NMINIMA) && (Iden != CL_NSEMIMINIMA) && 
		(Iden != CL_NCROMA) && (Iden != CL_NSEMICROMA) && 
		(Iden != CL_NBISCROMA) && (Iden != CL_NSEMIBISCROMA) && 
		(Iden != CL_NFUSA) && (Iden != CL_ACCORDO))
		return ;
	
	if (((Nota*)fig)->GetPAbbellimento()==NULL) return;
	if (((Nota*)fig)->GetPAbbellimento()->GetID()!=CL_TREMOLO) return;
	
	Fig2 = GetPLayer(current_layer)->GetNextFigura(fig);
	fl=false;
	while (Fig2!=NULL && !fl)
	{
		if (Fig2->GetID()==CL_SPAZIO)
		{
			Fig2 = GetPLayer(current_layer)->GetNextFigura(Fig2);
		}
		else
		{
			Iden = Fig2->GetID();
			if ((Iden != CL_NBREVE) && (Iden != CL_NSEMIBREVE) && 
				(Iden != CL_NMINIMA) && (Iden != CL_NSEMIMINIMA) && 
				(Iden != CL_NCROMA) && (Iden != CL_NSEMICROMA) && 
				(Iden != CL_NBISCROMA) && (Iden != CL_NSEMIBISCROMA) && 
				(Iden != CL_NFUSA) && (Iden != CL_ACCORDO))
				Fig2=NULL;
			else
				fl=true;
		}
	}
	
	if (Fig2!=NULL)
		((Tremolo*)(((Nota*)fig)->GetPAbbellimento()))->
		SetPos(fig->GetAbsPos() + Point(fig->GetVU2Rgt()+4,0),
		Fig2->GetAbsPos() + Point(-Fig2->GetVU2Lft()-4,0));
	else
	{
		EM.Error(MUSICAL_ERR_AREA, "EC_FIG_NOTFOUND","Battuta","AdjustTremolo()");
		delete ((Nota*)fig)->GetPAbbellimento();
		((Nota*)fig)->SetPAbbellimento(NULL);
	}
	
	
	return;
	
}

void Battuta::AdjustGlissato(Figura *fig, short current_layer) 
{
	Figura *Fig2=NULL;
	short fl;
	int Iden;
	
	Iden = fig->GetID();
	
	if ((Iden != CL_NBREVE) && (Iden != CL_NSEMIBREVE) && 
		(Iden != CL_NMINIMA) && (Iden != CL_NSEMIMINIMA) && 
		(Iden != CL_NCROMA) && (Iden != CL_NSEMICROMA) && 
		(Iden != CL_NBISCROMA) && (Iden != CL_NSEMIBISCROMA) && 
		(Iden != CL_NFUSA) && (Iden != CL_ACCORDO))
		return ;
	
	if (((Nota*)fig)->GetPAbbellimento()==NULL) return;
	if (((Nota*)fig)->GetPAbbellimento()->GetID()!=CL_GLISSATO) return;
	
	Fig2 = GetPLayer(current_layer)->GetNextFigura(fig);
	fl=false;
	
	while (Fig2!=NULL && !fl)
	{
		if (Fig2->GetID()==CL_SPAZIO)
		{
			Fig2 = GetPLayer(current_layer)->GetNextFigura(Fig2);
		}
		else
		{
			Iden = Fig2->GetID();
			if ((Iden != CL_NBREVE) && (Iden != CL_NSEMIBREVE) && 
				(Iden != CL_NMINIMA) && (Iden != CL_NSEMIMINIMA) && 
				(Iden != CL_NCROMA) && (Iden != CL_NSEMICROMA) && 
				(Iden != CL_NBISCROMA) && (Iden != CL_NSEMIBISCROMA) && 
				(Iden != CL_NFUSA) && (Iden != CL_ACCORDO))
				Fig2=NULL;
			else
				fl=true;
		}
	}
	
	if (Fig2!=NULL)  
		((Glissato*)(((Nota*)fig)->GetPAbbellimento()))->
		SetPos(fig->GetAbsPos() + Point(fig->GetVU2Rgt()+4,0),
		Fig2->GetAbsPos() + Point(-Fig2->GetVU2Lft()-4,0));
	else
	{
		EM.Error(MUSICAL_ERR_AREA, "EC_FIG_NOTFOUND","Battuta","AdjustGlissato()");
		delete ((Nota*)fig)->GetPAbbellimento();
		((Nota*)fig)->SetPAbbellimento(NULL);
	}
	
	return;
}


Bool Battuta::FigPtr2SymPath(SymPath& symPath, Figura *figptr)
{
	Bool trovato=FALSE;
	
	symPath.Array[BATTUTA_PATH_LEVEL]=GetNumericCode();
	
	for(int i = 0; i <NUM_MAX_LAYER; i++)
	{
		if (LayerVector[i]!=NULL)
		{	
			trovato = LayerVector[i]->FigPtr2SymPath(symPath,figptr);
			if(trovato)
			{
				symPath.Array[LAYER_PATH_LEVEL]=i+1;
				break;
			}
		}
	}
	return trovato;
}

void Battuta::SetCodeCounterFig(short current_layer,NumCode nc) 
{
	LayerVector[current_layer]->SetCodeCounter(nc);
}



NumCode Battuta::GetCodeCounterFig(short current_layer) 
{
	return LayerVector[current_layer]->GetCodeCounter();
}

float Battuta::GetTempoTimeExec()
{
	float res;
	Intestazione *pIntest;
	pIntest = intest[0];	
	
	res = (float) ((pIntest->GetTempo()).GetNumeratore());
	res /= (float) ((pIntest->GetTempo()).GetDenominatore());
	
	
	return res;
}


NumCode Battuta::GetPos(short current_layer, Figura *ptr)
{
	if(current_layer<0) current_layer = 0;
	return (NumCode)LayerVector[current_layer]->GetPos((DrawObject*)ptr);
	
}

DrawObject * Battuta::GetObject(SymCmd *MyCmd,short current_staff) 
{
	switch(MyCmd->CommandId)
	{
	case CMD_BATTUTA_CVIOLINO:
	case CMD_BATTUTA_CVIOLINO8:
	case CMD_BATTUTA_CVIOLINO8SOPRA:
	case CMD_BATTUTA_CBASSO:
	case CMD_BATTUTA_CBASSO8:
	case CMD_BATTUTA_CBASSO8SOPRA:
	case CMD_BATTUTA_CBARITONO:
	case CMD_BATTUTA_CTENORE:
	case CMD_BATTUTA_CTENORE8:
	case CMD_BATTUTA_CCONTRALTO:
	case CMD_BATTUTA_CMEZZOSOPRANO:
	case CMD_BATTUTA_CSOPRANO:
	case CMD_BATTUTA_CPERCUSBOX:
	case CMD_BATTUTA_CPERCUS2LINES:
	case CMD_BATTUTA_CTAB:
	case CMD_BATTUTA_CVUOTA:
		// Se ArgV[2]==CommandId allora e' nella battuta, altrimenti nella intestazione
		if (MyCmd->ArgV[2]!=MyCmd->CommandId) 
			return ((DrawObject *)(intest[current_staff]->GetChiave())); 
		
		return ((DrawObject *)(GetFig(0,MyCmd->ArgV[6]))); 
		
		break;
		
		
		
		
		
	case CMD_BATTUTA_ARMCHV_0D:
	case CMD_BATTUTA_ARMCHV_1D:
	case CMD_BATTUTA_ARMCHV_2D:
	case CMD_BATTUTA_ARMCHV_3D:
	case CMD_BATTUTA_ARMCHV_4D:
	case CMD_BATTUTA_ARMCHV_5D:
	case CMD_BATTUTA_ARMCHV_6D:
	case CMD_BATTUTA_ARMCHV_7D:
	case CMD_BATTUTA_ARMCHV_1B:
	case CMD_BATTUTA_ARMCHV_2B:
	case CMD_BATTUTA_ARMCHV_3B:
	case CMD_BATTUTA_ARMCHV_4B:
	case CMD_BATTUTA_ARMCHV_5B:
	case CMD_BATTUTA_ARMCHV_6B:
	case CMD_BATTUTA_ARMCHV_7B: 
		//Riferite alle armature minori
	case CMD_BATTUTA_ARMCHV_0Dm:
	case CMD_BATTUTA_ARMCHV_1Dm:
	case CMD_BATTUTA_ARMCHV_2Dm:
	case CMD_BATTUTA_ARMCHV_3Dm:
	case CMD_BATTUTA_ARMCHV_4Dm:
	case CMD_BATTUTA_ARMCHV_5Dm:
	case CMD_BATTUTA_ARMCHV_6Dm:
	case CMD_BATTUTA_ARMCHV_7Dm:
	case CMD_BATTUTA_ARMCHV_1Bm:
	case CMD_BATTUTA_ARMCHV_2Bm:
	case CMD_BATTUTA_ARMCHV_3Bm:
	case CMD_BATTUTA_ARMCHV_4Bm:
	case CMD_BATTUTA_ARMCHV_5Bm:
	case CMD_BATTUTA_ARMCHV_6Bm:
	case CMD_BATTUTA_ARMCHV_7Bm: 
		// Se ArgV[2]==CommandId allora e' nella battuta, altrimenti nella intestazione
		if (MyCmd->ArgV[2]!=MyCmd->CommandId) 
			return ((DrawObject *)(intest[current_staff]->GetPtrArmatura())); 
		
		return ((DrawObject *)(GetFig(0,MyCmd->ArgV[6]))); 
		
		break;
		
	case CMD_BATTUTA_TEMPO:   
		return ((DrawObject *)(intest[0]->GetPtrTemp()));; break;
	case CMD_BATTUTA_BSINGOLA: 
	case CMD_BATTUTA_BDOPPIA: 
	case CMD_BATTUTA_BFINALE:
	case CMD_BATTUTA_BINIZIORIT:
	case CMD_BATTUTA_BFINERIT:
	case CMD_BATTUTA_BSINGOLA_RIP2B:		
	case CMD_BATTUTA_BINVISIBILE:
	case CMD_BATTUTA_BINIZIOFINERIT: return ((DrawObject *)(ptrBarra)); break;
	case CMD_BATTUTA_INS_RIPTEMPO:
	case CMD_BATTUTA_INS_RIPBATTUTA:
	case CMD_BATTUTA_INS_RIPMEZZABATTUTA: 
		return ((DrawObject *)(GetFig(0,MyCmd->ArgV[4]))); break; 
	case CMD_SPARTITO_ADD_BEAM:
	case CMD_BATTUTA_NOTE2GRUPPONOTE:
		return ((DrawObject *)(GetFig(MyCmd->ArgV[0]-1,MyCmd->ArgV[6])));
		break;
	case CMD_BATTUTA_GRUPPONOTE2NOTE:
		return ((DrawObject *)(GetFig(MyCmd->ArgV[0]-1,MyCmd->ArgV[1])));
		break;
	case CMD_BATTUTA_NUGUALE:
		return ((DrawObject *)(ptrNumGrande)); break;
	case CMD_BATTUTA_SCANSIONE:		
		return ((DrawObject *)(ptrScansione)); break;
	case CMD_BATTUTA_TSALTO:			
		return ((DrawObject *)(ptrTSalto)); break;
		
	case CMD_BATTUTA_RIFERIMENTO:
	case CMD_BATTUTA_SEGNO:
	case CMD_BATTUTA_CODA:
		return ((DrawObject *)(ptrRiferimento)); break;
	case CMD_BATTUTA_LETTERA:
		return ((DrawObject *)(ptrLettera)); break;
	case CMD_BATTUTA_MOVIMENTO:		
		return ((DrawObject *)(ptrMovimento)); break;
		
	case CMD_DEL_COL_FIGURE:  			
	case CMD_BATTUTA_ERASE:			
	case CMD_BATTUTA_ELIMINAGRUPPI:
	default: return NULL; break;
        }
		return NULL;
}

Figura* Battuta::GetFigAll(short current_layer, NumCode nc)
{
	if(LayerVector[current_layer]==NULL) return NULL;
	return (Figura*)(LayerVector[current_layer]->GetFigAll(nc));
	
	
}

Bool Battuta::GetMultiVoice()
{
	Figura *pFig;
	Bool fl2;
	
	fl2 = FALSE;
	
	if(LayerVector[1]!=NULL)
	{
		pFig = LayerVector[1]->GetFirstF(); 
		
		
		while (pFig!=NULL)
		{
			if (pFig->GetID() != CL_SPAZIO)
			{
				fl2 = TRUE;
				pFig = NULL;
			}
			else
				// GB
				pFig = LayerVector[1]->GetNextF(pFig);
		}
		
	}
	
	return (fl2);
	
}

int Battuta::GetVoice(Figura *pFig)
{
	for (int i=0; i < NUM_MAX_LAYER;i++)
	{
		
		if (LayerVector[i]!=NULL && LayerVector[i]->FindFig(pFig))
			return i;
	}    
	return -1;
	
	
}

void Battuta::SetupSimboli(Bool beforeLegature)  
{
	for(int layer=0;layer<NUM_MAX_LAYER;layer++)
	{
		if(LayerVector[layer]!=NULL)
			LayerVector[layer]->SetupSimboli(beforeLegature,this,layer);
	}
}


Bool Battuta::GetChiaveFig(short current_layer, Figura *pfig) 
{
	int pos;
	
	pos = GetPos(0,pfig);
	if (pos<0)
	{
		for(int i=0; i<NUM_MAX_LAYER; i++)
		{
			pos = GetPos(i,pfig);
			if(pos>0) break;
		}
		return FALSE;
	}
	
	pos += 1;
	
	LayerVector[current_layer]->GetChiaveLayer(pos);
	return FALSE;
}

void Battuta::SetupNumeroFig() 
{
	for(int i=0;i<NUM_MAX_LAYER;i++)
	{
		if(LayerVector[i]!=NULL)
			LayerVector[i]->Calc_Fig_Layer();
	}
}

int Battuta::GetNUsedLayers()
{
	int n=0;
	for(int i=0;i<NUM_MAX_LAYER;i++)
	{
		if(LayerVector[i]!=NULL && LayerVector[i]->GetNumObj()>0)
			n++;
	}
	return n;
}

void Battuta::SetupDurata()
{
	Tempo tempo;
	float num, den;
	Figura *pfig;
	LegatQuadra *pleg=NULL;
	Figura *plegend=NULL;
	int i;
	
	//ricavo il tempo in chiave   
	
	tempo=GetTempo();
	num=float(tempo.GetNumeratore());
	den=float(tempo.GetDenominatore());
	
	for(i=0; i<NUM_MAX_LAYER; i++)
	{
		if(LayerVector[i]!=NULL)
		{
			pfig=LayerVector[i]->GetFirstFig();
			pleg=NULL;
			plegend=NULL;
			while (pfig!=NULL)
				
			{
				if(pfig->GetID()==CL_PSEMIBREVE)
				{
					pfig->SetDurata(num/den);
				}
				if(pfig->GetDurata()!=0.)
				{
					if(pleg==NULL && pfig->GetTerzina()!=NULL)
					{
						pleg=pfig->GetTerzina();
						plegend=pleg->GetPFigEnd();
					}
					if(pleg!=NULL)
					{
						pfig->SetTerzina(pleg);
					}
					if(pfig==plegend)
						pleg=NULL; 
				}
				pfig=LayerVector[i]->GetNextFig();
			}
		}
	}
}   


void Battuta::BeamingAuto(short current_staff, short current_layer)
{ 
	if(CheckBatt())
	{
		for(int i=0;i<NUM_MAX_LAYER;i++)
		{
			if(LayerVector[i]!=NULL)
				LayerVector[i]->BeamingAuto(this,0,i); 
		}
	}
	else 
	{
		EliminaGruppi();
	}    	
}

void Battuta::BeamingAuto()// questo sara il beamigauto di della Battuta
{
	bool layerOK[NUM_MAX_LAYER];
	
	CheckBatt(layerOK);
	
	for(int i=0;i<NUM_MAX_LAYER;i++)
	{
		if(LayerVector[i]!=NULL)
		{
			if(layerOK[i])
				LayerVector[i]->BeamingAuto(this,0,i); // GB
			else
				EliminaGruppi(i);
		}
	}
}


/* Metodo che verifica se una battuta e' costituita da sole pause */

int Battuta::CheckBattPause(Battuta *pbat)
{
	// scandisco la battuta e verifico che ogni figura che ne fa parte 
	// una pausa (infatti la battuta non deve essere vuota...)
	// anzi il valore complessivo della pausa deve corrispondere al
	// tempo in chiave.
	
	// ritorna 1 se la battuta  di sole pause
	// ritorna 0 se nella battuta sono presenti delle note
	// ritorna 2 se la battuta  di sole pause ma con tempo diverso dalla prima
	// o con una barra diversa 
	
	Tempo tempo;
	float num, den;
	
	Figura *pfig;
	
	
	float temp;
	Bool layer2;
	
	Bool allrest;
	
	Bool tmp;
	Bool barra;
	Bool movimento;
	
	int i;
	
	//ricavo il tempo in chiave   
	
	tempo=GetTempo();
	num=float(tempo.GetNumeratore());
	den=float(tempo.GetDenominatore());
	
	temp=num/den;
	
	layer2=FALSE;
	
	// ciclo per effettuare la verifica di consistenza sui due layer
	
	allrest=TRUE;
	
	for(i=0; i<NUM_MAX_LAYER; i++)                             
	{
		if(LayerVector[i]!=NULL)
		{
			pfig=LayerVector[i]->GetFirstFig();
			while (pfig!=NULL)
			{
				if(pfig->IsANote())
				{
					allrest=FALSE; 
					break;
				} 	
				pfig=LayerVector[i]->GetNextFig();
			}
		}
	}
	
	if(!allrest) return 0;
	
	if(pbat==NULL) return 1; 
	else 
	{
		
		
		if(GetTempo()!=pbat->GetTempo())
			tmp=FALSE;
		else
			tmp=TRUE;  
		if(GetBarra()->GetTipoBarra()!=pbat->GetBarra()->GetTipoBarra()) 
			barra=FALSE;
		else
			barra=TRUE;
		
		
		
		if(GetPMovimento()!=NULL)
			movimento=FALSE;
		else
			movimento=TRUE;
		
		
		
		if(!tmp || !movimento) return 2;
		else if(!barra) return 3;
		else return 1;
		
	}
}  

void Battuta::MakeBattMultiRest(Battuta* pbat1, Battuta *pbat2, int nb)
{
	int nump, nump2, num;
	unsigned char str[10];
	NumCode nc;
	
	short current_staff=0;
	
	SetNumericCode(pbat1->GetNumericCode());
	CopyIntest(current_staff,pbat1->GetIntest(current_staff));
	
	
	if(pbat1->GetPMovimento()!=NULL)
	{
		ptrMovimento=new Movimento(pbat1->GetPMovimento());
	}
	
	SetNumMultiRest(nb);
	
	// Imposta il numero progressivo alla nuova battuta ed alle seguenti.
	nump = pbat1->GetNumProgress();
	nump2 = pbat2->GetNumProgress();
	
	if(nump<nump2) num=nump;
	else num=nump2;
	
	SetNumProgress(num);
	
	sprintf((char*)str,"%d",nb);
	
	InsNumPausa(str);
	nc=InsSpazio(1,0,1,0,current_staff);
	nc=InsPausa(0,1,CL_PGENERICA,1,current_staff,AUTOMATICO,0,nc,0);
	nc=InsSpazio(1,nc,0,0,current_staff);
	InsSpazio(1,nc,0,0,current_staff);
	
	SetBarra(pbat2->GetBarra()->GetTipoBarra());
}

void Battuta::ResetIntRefs()
{
	Figura *pFig;
	Layer *pLayer; 
	
	for(int lay=0;lay < NUM_MAX_LAYER; lay++) 
	{
		pLayer=GetPLayer(lay);
		if(pLayer!=NULL)
		{
			
			pFig=pLayer->GetFirstF();
			while(pFig!=NULL)
			{
				pFig->DelIntRefs();
				pFig=pLayer->GetNextF(pFig);
			}  
		}  
	}
}


/**            METODO AGGIUNTO PER LIRICA                 **/

Figura *Battuta::Symbolic(SymPath& symPath)
{
	// Questo metodo verifica dato un indirizzo  //
	// simbolico di una figura se questa fa parte//
	// della battuta corrente,restituendo in tal //
	// caso il puntatore attuale alla figura.Se  //
	// la figura non fa parte della battuta rende//
	// NULL.                                     //
	
	Figura *fig; 
	// Variabili temporanee contenenti l'indirizzo di
	// memoria attuale dove  memorizzata la figura
	// associata alla sillaba.
	
	
	if(GetNumericCode()!=symPath.Array[BATTUTA_PATH_LEVEL])
		// Se la battuta corrente non  quella in cui 
		// presente la figura a cui  associata la sillaba........
		return NULL;
	// Ritorna restituendo NULL.
	
	//.... Altrimenti procede ad esaminate le figure
	// della battuta.
	
	symPath.Level = FIGURA1_PATH_LEVEL;
	// Imposta il livello che permette di
	// identificare la figura cerceta in base al suo
	// NumeriCode.	
	
	
	Layer* layer=LayerVector[symPath.Array[LAYER_PATH_LEVEL]-1];
	
	int nc=symPath.Array[FIGURA1_PATH_LEVEL];
	if(symPath.Array[FIGURA2_PATH_LEVEL]!=0)
		nc=symPath.Array[FIGURA2_PATH_LEVEL];
	
	if(layer!=NULL)
	{
		fig=layer->GetFirstFig();
		// Prende la prima figura della lista
		// appartenente alla battuta corrente.
		
		while(fig!=NULL)
		{			
			if(fig->GetNumericCode() == nc)
				return fig;
			
			fig=layer->GetNextFig();
			//..altrimenti passa ad esaminare la successiva.
		};
	}
	
	return NULL;
	// Se la figura non fa parte della battuta
	// corrente restituisce NULL.	
}


void Battuta::Free()
{	
	if(ptrBarra!=NULL) delete ptrBarra;
	if(ptrBarraPrec!=NULL) delete ptrBarraPrec;
	if(ptrNumBattuta!=NULL) delete ptrNumBattuta;
	if(ptrNumGrande!=NULL) delete ptrNumGrande;
	if(ptrScansione!=NULL) delete ptrScansione;
	if(ptrLettera!=NULL) delete ptrLettera;
	if(ptrTSalto!=NULL) delete ptrTSalto;
	if(ptrRiferimento!=NULL) delete ptrRiferimento;
	if(ptrMovimento!=NULL) delete ptrMovimento;
	
	// cancella layer
	for(int i=0; i<NUM_MAX_LAYER; i++)
	{
		if(LayerVector[i]!=NULL)
		{
			delete LayerVector[i];
			LayerVector[i]=NULL;
		}
	}
	
	// cancella intestazioni
	for(i=0; i<NUM_MAX_INTEST; i++)
	{
		if(intest[i]!=NULL)
			delete intest[i];
		if(intest_end[i]!=NULL)
			delete intest_end[i];
	}
}

//
//  Metodi Privati.
//

Bool Battuta::InsNumPausa(unsigned char* s)
{
	if (ptrNumGrande!=NULL) delete(ptrNumGrande);
	ptrNumGrande = new NumPausa;
	if (ptrNumGrande==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","InsNumPausa()");
		return FALSE;
	};   
	if (ptrNumGrande->SetTxt(s))
		return TRUE;
	else
	{
		EM.Error(GENERAL_ERR_AREA, "EC_CMD_FAULT","Battuta","InsNumPausa()");
		delete ptrNumGrande;
		return FALSE;
	}
}

Bool Battuta::InsNumRipBattuta(unsigned char* s)
{
	if (ptrNumGrande!=NULL) delete(ptrNumGrande);
	ptrNumGrande = new NumGrande;
	if (ptrNumGrande==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","InsNumPausa()");
		wxMessageBox("Il puntatore a NumGrande e' NULL");
		return FALSE;
	};   
	if (ptrNumGrande->SetTxt(s))
		return TRUE;
	else
	{
		EM.Error(GENERAL_ERR_AREA, "EC_CMD_FAULT","Battuta","InsNumPausa()");
		wxMessageBox("Non riesce il NumGrande::SetTxt()");
		delete ptrNumGrande;
		return FALSE;
	}
}


Bool Battuta::InsNumUguale(unsigned char* s)
{
	if (ptrNumGrande!=NULL) delete(ptrNumGrande);
	ptrNumGrande = new NumUguale;
	if (ptrNumGrande==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","InsNumUguale()");   
		return FALSE;
	};   
	if (ptrNumGrande->SetTxt(s))
		return TRUE;
	else
	{
		EM.Error(GENERAL_ERR_AREA, "EC_CMD_FAULT","Battuta","InsNumUguale()");
		delete ptrNumGrande;
		return FALSE;
	}
}

Bool Battuta::InsScansione(short n)
{
	if (ptrScansione!=NULL) delete(ptrScansione);
	ptrScansione = new Scansione;
	if (ptrScansione==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","InsScansione()");   
		return FALSE;
	};   
	ptrScansione->SetNumScan(n);
	return TRUE;
}

Bool Battuta::InsTSalto(unsigned char* s)
{
	if (ptrTSalto!=NULL) delete(ptrTSalto);
	ptrTSalto = new TSalto;
	if (ptrTSalto==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","InsTSalto()");   
		return FALSE;
	};
	if (ptrTSalto->SetTxt(s))
		return TRUE;
	else
	{
		EM.Error(GENERAL_ERR_AREA, "EC_CMD_FAULT","Battuta","InsTSalto()");
		delete ptrTSalto;
		return FALSE;
	}
}

Bool Battuta::InsRiferimento(short CodeCommand)
{
	switch (CodeCommand) {
	case (CMD_BATTUTA_SEGNO):
		{
			ptrRiferimento=new Segno;
			break;
		}
	case (CMD_BATTUTA_CODA):
		{
			ptrRiferimento=new Coda;
			break;
		}
	default:
		return 0;
	}
	return TRUE;
}

Bool Battuta::InsLettera(unsigned char* s)
{
	if (ptrLettera!=NULL) delete(ptrLettera);
	ptrLettera = new Lettera;
	if (ptrLettera==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","InsLettera()");   
		return FALSE;
	};
	if (ptrLettera->SetTxt(s))
		return TRUE;
	else
	{
		EM.Error(GENERAL_ERR_AREA, "EC_CMD_FAULT","Battuta","InsLettera()");
		delete ptrLettera;
		return FALSE;
	}
}

Bool Battuta::InsMovimento(unsigned char *p, ClassID ident, Bool punt, int s)
{
	
	if (ptrMovimento!=NULL) delete(ptrMovimento);
	ptrMovimento = new Movimento;
	if (ptrMovimento==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Battuta","InsMovimento()");
		return FALSE;
	};
	
	if (ident != 0 && s!=0)
	{
		if (!(ptrMovimento->SetMetronomo()))
		{
			EM.Error(GENERAL_ERR_AREA, "EC_CMD_FAULT","Battuta","InsMovimento()");
			delete ptrMovimento;
			return FALSE;
		}
	}
	
	
	if (ptrMovimento->MovSetTxt(p, ident, punt, s))
		return TRUE;
	else
	{
		EM.Error(GENERAL_ERR_AREA, "EC_CMD_FAULT","Battuta","InsMovimento()");
		delete ptrMovimento;
		return FALSE;
	}
}

float Battuta::GetMetronomoTimeExec()
{
	if (ptrMovimento==NULL)
		return 0;
	else
		return ptrMovimento->GetTimeExec();
}


void Battuta::DrawBall()
{
	Point p1 = GetAbsPos() + Point(20,30),
        p2 = GetAbsPos() + Point(40,50);
	
	class Rectangle rr( p1, p2);
	
	Colore oldcol = Scr->GetColor();
	Scr->SetColor((ColorIcon)XOR_SELECTED_COLOR);
	Scr->SetDrwMode(XORMode);
	
	Scr->Arc(rr, 0, 360, TRUE);
	p1 = p1 + Point(-2,-2),
		Scr->BigLine(p1, p2, 3);
	
	Scr->SetDrwMode(DefMode);
	Scr->SetColor(oldcol);
}

void Battuta::SetPMovimento(Movimento *p)
{ ptrMovimento=p; }

Movimento *Battuta::GetPMovimento()
{ return ptrMovimento; }

void Battuta::SetFirsColonna(Bool f)
{
	FirstColonna = f;
}

void Battuta::CheckBattuta()
{
	return;
}


Bool Battuta::CheckBatt(bool *layer, float *Layer_value)//, 
{
	Tempo tempo;
	float num, den;
	float temp;
	int i;
	Layer* pLayer;
	int mylayer[NUM_MAX_LAYER];
	//ricavo il tempo in chiave   
	
	tempo=GetTempo();
	num=float(tempo.GetNumeratore());
	den=float(tempo.GetDenominatore());
	
	temp=num/den;
	SetupDurata();
	bool onetime = false;
	
	for(i=0; i<NUM_MAX_LAYER; i++)
	{
		mylayer[i]=-1;
		pLayer=LayerVector[i];
		if(pLayer==NULL)
			continue;
		if(!pLayer->isEmpty())		
		{
			onetime=true;
			float layerDuration=pLayer->GetTotalDuration(num,den);
			if (fabs(layerDuration-temp)<0.001)  
			{
				mylayer[i]=1; 
				if(Layer_value!=NULL)
					Layer_value[i]=0;
			}   
			else
			{
				mylayer[i]=0; 
				if(Layer_value!=NULL)
					Layer_value[i]=(layerDuration-temp)*den;
			}
		}
		
		
		
	}
	
	bool result = true;
	if(!onetime)
	{
		float layerDuration=LayerVector[0]->GetTotalDuration(num,den);
		mylayer[0]=0; 
		if(Layer_value!=NULL)
			Layer_value[0]=(layerDuration-temp)*den;
	}
	
	
	for(i=0; i<NUM_MAX_LAYER; i++)
	{
		if(i==0 || mylayer[i]!=-1)
			result = result && mylayer[i]==1; 
		if(layer!=NULL)
			layer[i]=mylayer[i];
	}
	
	return result; 
}   

float Battuta::CheckLayer(int nLayer)
{
	Tempo tempo;
	float num, den;
	float temp;
	Layer* pLayer;
	
	tempo=GetTempo();
	num=float(tempo.GetNumeratore());
	den=float(tempo.GetDenominatore());
	
	temp=num/den;
	
	SetupDurata();
	
	pLayer=LayerVector[nLayer];
	if(pLayer==NULL)
		return 0.;
	
	float layerDuration=pLayer->GetTotalDuration(num,den);
	if (fabs(layerDuration-temp)<0.001)  
	{
		return 0.;
	}   
	else
	{
		return (layerDuration-temp)*den;
	}
}   
/**                        FINE                            **/


static int ConvertSignature2Int(tipoArmaturaChiave start_signature)
{
	int keysign[15]={7,8,9,10,11,12,13,14,6,5,4,3,2,1,0};
	
	return keysign[start_signature];
}
//Funzione inversa della precedente
static tipoArmaturaChiave ConvertSignature2Enum(int index)
{
	tipoArmaturaChiave key[15]={DOb_maggiore,
		SOLb_maggiore,
		REb_maggiore,
		LAb_maggiore,
		MIb_maggiore,
		SIb_maggiore,
		FA_maggiore,
		DO_maggiore,
		SOL_maggiore,
		RE_maggiore,
		LA_maggiore,
		MI_maggiore,
		SI_maggiore,
		FAd_maggiore,
		DOd_maggiore};
	return key[index];
}

//Funzione per il calcolo dello shift sulla tavola del Ciclo delle Quinte
//Gli intervalli in gioco sono: Unisono, seconda min, seconda magg, terza min,
//terza magg, quarta perfetta, quarta ecced, quinta perfetta, sesta min, sesta magg,
//settima min, settima magg, ottava. Presi cosi' nell'ordine sono elencati in ordine
//crescente rispetto al numero di semitoni ad essi associati, es: unisono=0, seconda
//min =1, seconda magg=2, ecc.
static int CalculateShifting(int interval)
{
	int intervals_shift_down[13]={0,-5,2,-3,4,-1,6,1,-4,3,-2,5,12};
	return intervals_shift_down[interval];
}

//Metodo di gestione delle trasposizioni
Bool Battuta::Transpose(int clef,int translation,int interval,int up, int numofstaff, Bool sharps, Bool adjust)
{
	
	if(translation!=0) 
		Translation(translation, numofstaff);
	
	if(interval!=-1) 
		if(!TransposeInterval(interval,up, numofstaff, sharps, adjust))
			return FALSE;
		
		if(clef!=-1)
			if(!TransposeClef(clef, numofstaff))
				return FALSE;
			
			return TRUE;
			
}    


int Battuta::CalculateOffsetFromClefs(ClassID start,ClassID target)
{
	
	int i=0;
	int start_value=0, target_value=0;
	int offset=0;
	ClassID clefID;
	
	for(i=1;i<=2; i++)
	{
		switch(i)
		{
		case 1:
			clefID=start;
			break;
		case 2:
			clefID=target;
			break;
		}    
		
		switch(clefID)
		{
		case CL_CVIOLINO:
			switch(i)
			{
			case 1:
				start_value= -6;
				break;
			case 2:
				target_value= -6;;
				break;
			}  
			break;
			
			case CL_CBASSO:
				switch(i)
				{
				case 1:
					start_value= 6;
					break;
				case 2:
					target_value= 6;
					break;
				}  
				break;
				
				case CL_CBARITONO:
					switch(i)
					{
					case 1:
						start_value= 4;
						break;
					case 2:
						target_value= 4;
						break;
					}  
					break;
					case CL_CTENORE:
						switch(i)
						{
						case 1:
							start_value= 2;
							break;
						case 2:
							target_value= 2;
							break;
						}  
						break;
						case CL_CCONTRALTO:
							switch(i)
							{
							case 1:
								start_value= 0;
								break;
							case 2:
								target_value= 0;
								break;
							}  
							break;
							case CL_CMEZZOSOPRANO:
								switch(i)
								{
								case 1:
									start_value= -2;
									break;
								case 2:
									target_value= -2;
									break;
								}  
								break;
								case CL_CSOPRANO:
									switch(i)
									{
									case 1:
										start_value= -4;
										break;
									case 2:
										target_value= -4;
										break;
									}  
									break;
		}
	}
	return (target_value - start_value);
}

Bool Battuta::TransposeClef(int clef, int numofstaff)
{
	
	Layer *pLayer;  
	int offset=0;
	ClassID id;
	Accordo *chord;
	Figura *fig_chord;
	Chiave chiave;
	Bool exit=TRUE;
	Bool exittmp=TRUE;
	int start, end;
	
	int numStaff;
	
	ClassID clefvectorID[]={CL_CVIOLINO,
		CL_CBASSO,
		CL_CBARITONO,
		CL_CTENORE,
		CL_CCONTRALTO,
		CL_CMEZZOSOPRANO,
		CL_CSOPRANO };
	
	if(numofstaff==0 && GetNumberOfStaff()>1)
	{
		start=0;
		end=GetNumberOfStaff()-1;
	}
	else if(numofstaff==0 && GetNumberOfStaff()==1)
	{
		start=end=0;
	}
	else if(numofstaff!=0 && GetNumberOfStaff()>1)
		start=end=numofstaff-1;
				else
					return FALSE;
				
				for(numStaff=start; numStaff<=end; numStaff++)
				{
					for(int l=0;l<NUM_MAX_LAYER;l++)
					{
						pLayer = GetPLayer(l); 
						if(pLayer==NULL)
							continue;
						
						offset=CalculateOffsetFromClefs(GetChiave(numStaff)->GetID(),clefvectorID[clef]);
						switch(clefvectorID[clef])
						{
						case CL_CVIOLINO:
							SetChiave(numStaff,new CViolino);
							break;
						case CL_CBASSO:
							SetChiave(numStaff,new CBasso);
							break;
						case CL_CBARITONO:
							SetChiave(numStaff,new CBaritono);
							break;
						case CL_CTENORE:
							SetChiave(numStaff,new CTenore);
							break;
						case CL_CCONTRALTO:
							SetChiave(numStaff,new CContralto);
							break;
						case CL_CMEZZOSOPRANO:
							SetChiave(numStaff,new CMezzoSoprano);
							break;
						case CL_CSOPRANO: 
							SetChiave(numStaff,new CSoprano);
							break;
						}
						Figura *fig=pLayer->GetFirstFig();
						int i=0;
						while(fig!=NULL)
						{
							
							id=fig->GetID();
							//Si controlla se la figura corrente e' una chiave nel caso lo sia si arresta il procedimento
							//Se l'armatura esistente coincide con la modifica va eliminata la figura
							//che la rappresenta temporaneamente.
							if ((id >= CL_CVIOLINO) && (id <= CL_CSOPRANO))
							{
								if(fig->GetNStaff()==numStaff)
								{
									if(id==GetChiave(numStaff)->GetID())
										pLayer->DelPtrFig(fig, this, 1,0);
									exittmp=FALSE;
								}
								break;
							}
							else
							{
								switch(id)
								{
								case CL_ACCORDO:
									{
										chord = (Accordo *) fig;
										fig_chord = chord->GetFirstF();
										while(fig_chord!=NULL)
										{
											if(fig_chord->GetNStaff()==numStaff)	
												fig_chord->SetAltezza(fig_chord->GetAltezza()+offset);
											
											fig_chord=chord->GetNextF(fig_chord);
										}
									}
									break;
								default:
									if(fig->GetNStaff()==numStaff)
									{	
										if ((id >= CL_NBREVE) && (id <= CL_NFUSA))
											fig->SetAltezza(fig->GetAltezza()+offset);
									}	
									break;
								}
								fig=pLayer->GetNextFig();
							} 
							
						}
					}
					exit=exit & exittmp;
				}
				return exit;
}



Bool Battuta::TransposeInterval(int interval, int up, int numofstaff, Bool sharps, Bool adjust)
{
	Layer *pLayer; 
	ClassID id, temp1_accidental[7], temp2_accidental[7];
	Accordo *chord;
	Figura *fig_chord;
	Nota *fig_nota;
	int adj=0;
	int tone_fin=0;
	int tone_in=0;
	int shifting=0;
	int i=0;
	int pitch=0;
	tipoArmaturaChiave start_signature, target_signature;
	Bool prefer_sharps;
	Bool exittmp=TRUE;
	Bool exit=TRUE;
	int start, end;
	
	int numStaff;
	
	prefer_sharps=sharps;
	
	if(numofstaff==0 && GetNumberOfStaff()>1)
	{
		start=0;
		end=GetNumberOfStaff()-1;
	}
	else if(numofstaff==0 && GetNumberOfStaff()==1)
	{
		start=end=0;
	}
	else if(numofstaff!=0 && GetNumberOfStaff()>1)
		start=end=numofstaff-1;
				else
					return FALSE;
				
				for(numStaff=start; numStaff<=end; numStaff++)
				{	
					
					start_signature=GetArmatura(numStaff);
					tone_in = ConvertSignature2Int(start_signature);
					
					//Conversione intervallo di shifting
					if(up==1)
					{
						shifting=CalculateShifting(interval);
					}
					else
					{
						shifting=-CalculateShifting(interval);
					}
					
					//Calcolo della nuova tonalita'
					if((shifting!=0)&&(shifting!=12)&&(shifting!=-12))
					{
						tone_fin=(tone_in+shifting);
						
						if(tone_fin<0)
							tone_fin=12+(tone_fin%15);
						if(tone_fin>14)
							tone_fin=3+(tone_fin%15);
						
						if((prefer_sharps)&&(tone_fin<3))
							tone_fin=tone_fin+12;
						
						target_signature=ConvertSignature2Enum(tone_fin);
					}
					
					for(int l=0;l<NUM_MAX_LAYER; l++)
					{
						pLayer = GetPLayer(l); 
						if(pLayer==NULL)
							continue;
						
						if(l==0) //Set dell'armatura in chiave solo escluso il caso di trasposizione all'unisono e di ottava
						{
							if((shifting!=0)&&(shifting!=12)&&(shifting!=-12))
								SetArmatura(numStaff,target_signature);
						}
						
						for(i=0;i<7;i++)
						{
							temp1_accidental[i]=CL_NATURAL;
							temp2_accidental[i]=CL_NATURAL;
						}
						
						Figura *fig=pLayer->GetFirstFig();
						int i=0;
						while(fig!=NULL)
						{
							id=fig->GetID();
							if (id == CL_ARMATURACHIAVE) //Si controlla se la figura corrente e' un'armatura nel caso lo sia si arresta il procedimento
							{
								if(fig->GetNStaff()==numStaff)
								{
									//Se l'armatura esistente coincide con la modifica va eliminata la figura
									//che la rappresenta temporaneamente.
									if(((ArmaturaChiave *)fig)->GetArmatura()==target_signature)
										pLayer->DelPtrFig(fig, this, 1,0);
									exittmp=FALSE;
								}
								break;
							}
							else
							{
								switch(id)
								{
								case CL_ACCORDO:
									{
										chord = (Accordo *) fig;
										fig_chord = chord->GetFirstF();
										while(fig_chord!=NULL)
										{
											if(fig_chord->GetNStaff()==numStaff)
											{
												fig_nota=(Nota*) fig_chord;
												fig_nota->TransposeNoteByInterv(this,shifting,temp1_accidental,start_signature, temp2_accidental, up);
											}
											
											fig_chord=chord->GetNextF(fig_chord);
										}
									}
									break;
								default:
									if(fig->GetNStaff()==numStaff)
									{
										if ((id >= CL_NBREVE) && (id <= CL_NFUSA))
										{
											fig_nota=(Nota*) fig;
											fig_nota->TransposeNoteByInterv(this,shifting,temp1_accidental,start_signature, temp2_accidental, up);
										}
									}
									break;
								}
								fig=pLayer->GetNextFig();
							}
						}
					}
					exit=exit & exittmp;
	}
	return exit;
}


void Battuta::Translation(int translation, int numofstaff)
{
	int offset=0;
	Layer *pLayer; 
	Accordo *chord;
	Figura *fig_chord;
	ClassID id;
	
	offset=translation;
	
	int start, end;
	int numStaff;
	
	
	if(numofstaff==0 && GetNumberOfStaff()>1)
	{
		start=0;
		end=GetNumberOfStaff()-1;
	}
	else if(numofstaff==0 && GetNumberOfStaff()==1)
	{
		start=end=0;
	}
	else if(numofstaff!=0 && GetNumberOfStaff()>1)
		start=end=numofstaff-1;
				
	
	for(numStaff=start; numStaff<=end; numStaff++)
	{	
		
		for(int l=0;l<NUM_MAX_LAYER; l++) 
		{
			pLayer = GetPLayer(l); 
			if(pLayer==NULL)
				continue;
			
			Figura *fig=pLayer->GetFirstFig();
			while(fig!=NULL)
			{
				id=fig->GetID();
				switch(id)
				{
				case CL_ACCORDO:
					{
						chord = (Accordo *) fig;
						fig_chord = chord->GetFirstF();
						while(fig_chord!=NULL)
						{
							if(fig_chord->GetNStaff()==numStaff)
							{
								fig_chord->SetAltezza(fig_chord->GetAltezza()+offset);
							}
							fig_chord=chord->GetNextF(fig_chord);
						}
					}
					break;
				default:
					if(fig->GetNStaff()==numStaff)
					{					
						if ((id >= CL_NBREVE) && (id <= CL_NFUSA))
							fig->SetAltezza(fig->GetAltezza()+offset);
					}
					break;
				}
				fig=pLayer->GetNextFig();
			} 
		}
	}
}

//Funzione di conversione della tonalita' in valore intero rispetto al tipo
//enumerato tipoArmaturaChiave. Viene eseguito un ordinamento rispetto al Tavola
//del Ciclo delle Quinte.



/********************************************************************/
void Battuta::View_Link(bool view)
{
	ListaFigure *pLayer;
	int num_layer;
	for(num_layer=0;num_layer<NUM_MAX_LAYER;num_layer++)
	{	
		pLayer=GetPLayer(num_layer);
		if(pLayer==NULL)
			continue;
		
		Figura *fig=pLayer->GetFirstFig();
		while(fig!=NULL)
		{
			if (!(fig->Get_Link().IsEmpty()))	
			{		
				if(fig->GetID()==CL_ACCORDO)
				{
					Accordo *acc=(Accordo*)fig;
					Figura* tmp=acc->GetFirstF();
					while(tmp!=NULL)
					{
						if (view)
							tmp->SetColor((ColorIcon)BLU);
						else 
							tmp->SetColor((ColorIcon)black);
						tmp=acc->GetNextF(tmp);
					}			
				}
				else
				{
					if (view)
						fig->SetColor((ColorIcon)BLU);
					else 
						fig->SetColor((ColorIcon)black);
				}
			}	
			else 
				fig->SetColor((ColorIcon)black);
			
			fig=pLayer->GetNextFig();
		} 
	} 
}

/********************************************************************/
Figura* Battuta::ChangeFigDuration(Figura *pFig, ClassID Target_ID)
{
	Figura *ptr_prev=NULL;
	Figura *ptr_newfig=NULL;
	Figura *ptr_tmpfig=NULL;
	ListaFigure *pLayer=NULL;
	GruppoNote *currGruppoNote=NULL;
	Figura *ptr_fig=NULL;
	Accordo *tmpacc=NULL;
	ClassID figID; 
	bool stop=false;
	
	if(pFig->GetParentFig())
	{
		ptr_fig=pFig->GetParentFig();
		if(ptr_fig->GetID()==CL_ACCORDO)
			figID=CL_ACCORDO;
		else
		{
			figID= pFig->GetID();
			ptr_fig=pFig;
		}
	}
	else
	{
		ptr_fig=pFig;
		figID= pFig->GetID();
	}
	
	for(int i=0; i<NUM_MAX_LAYER; i++)
	{
		pLayer=GetPLayer(i);
		if(pLayer==NULL)
			continue;
		
		ptr_tmpfig=pLayer->GetFirstFig();
		while (ptr_tmpfig!=NULL)
		{
			if (ptr_tmpfig==ptr_fig)
			{	
				currGruppoNote=pLayer->GetLastGruppoNote();
				
				stop=true;
				break;
			}
			
			
			ptr_tmpfig=pLayer->GetNextFig();
		}
		if(stop)
			break;
	}
	
	//Distinzione fra note e pause.
	if((figID>CL_PAUSA)&&(figID<CL_SPAZIO))
	{
		switch (Target_ID)
		{
		case CL_PQUATTROBATTUTE:
			{
				PQuattroBattute *ptrPausa = new PQuattroBattute;
				ptr_newfig= (Figura *) ptrPausa;
			}
			break;
		case  CL_PDUEBATTUTE:
			{
				PDueBattute *ptrPausa = new PDueBattute;
				ptr_newfig= (Figura *) ptrPausa;
			}
			break;
		case  CL_PSEMIBREVE:
			{
				PSemibreve *ptrPausa = new PSemibreve;
				ptr_newfig= (Figura *) ptrPausa;
			}
			break;
		case  CL_PMINIMA:
			{
				PMinima *ptrPausa = new PMinima;
				ptr_newfig= (Figura *) ptrPausa;
			}
			break;
		case  CL_PSEMIMINIMA:
			{
				PSemiminima *ptrPausa = new PSemiminima;
				ptr_newfig= (Figura *) ptrPausa;
			}
			break;
		case  CL_PCROMA:
			{
				PCroma *ptrPausa = new PCroma;
				ptr_newfig= (Figura *) ptrPausa;
			}
			break;
		case  CL_PSEMICROMA:
			{
				PSemicroma *ptrPausa = new PSemicroma;
				ptr_newfig= (Figura *) ptrPausa;
			}
			break;
		case  CL_PBISCROMA:
			{
				PBiscroma *ptrPausa = new PBiscroma;
				ptr_newfig= (Figura *) ptrPausa;
			}
			break;
		case  CL_PSEMIBISCROMA:
			{
				PSemibiscroma *ptrPausa = new PSemibiscroma;
				ptr_newfig= (Figura *) ptrPausa;
			}
			break;
		case  CL_PFUSA:
			{
				PFusa *ptrPausa = new PFusa;
				ptr_newfig= (Figura *) ptrPausa;
			}
			break;
		case  CL_PGENERICA:
			{
				PGenerica *ptrPausa = new PGenerica;
				ptr_newfig= (Figura *) ptrPausa;
			}
			break;
		}
	}
	else if((figID>CL_NOTA)&&(figID<CL_GRUPPONOTE))
	{
		ptr_newfig=CreaNota(Target_ID);			
		((Nota *) ptr_newfig)->TransSymb((Nota *)ptr_tmpfig);
	}
	
	if(figID==CL_ACCORDO)
	{	
		tmpacc=new Accordo;
		Accordo *oldacc=(Accordo *)ptr_tmpfig;
		Figura *nota_acc=oldacc->GetFirstF();
		while(nota_acc!=NULL)
		{
			Figura *tmpNota=CreaNota(Target_ID);
			((Nota *)tmpNota)->SetAltezza(((Nota*)nota_acc)->GetAltezza());
			((Nota *)tmpNota)->SetHeadType(((Nota*)nota_acc)->GetHeadType());
			((Nota *)tmpNota)->SetColor(((Nota*)nota_acc)->GetColor());
			((Nota *)tmpNota)->SetNStaff(((Nota*)nota_acc)->GetNStaff());
			tmpacc->AddNota((Nota *)tmpNota);
			if(nota_acc==pFig)
				ptr_fig=tmpNota;
			nota_acc=oldacc->GetNextF(nota_acc);
		}
		ptr_newfig=(Figura *)tmpacc;
		
	}
	if(currGruppoNote)
	{	
		GruppoNote2Note(currGruppoNote);
	}
	ptr_prev=pLayer->GetPrevF(ptr_tmpfig);
	
	//Inserimento nella lista figure.
	if((ptr_tmpfig!=NULL)&&(ptr_prev!=NULL))
		pLayer->InsertAfterF(ptr_prev, ptr_newfig);
	
	if((ptr_tmpfig!=NULL)&&(ptr_prev==NULL))
		pLayer->InsertTop((DrawObject *) ptr_newfig);
	
	//Cancellazione della vecchia figura.
	if(ptr_tmpfig!=NULL)
	{
		DeleteSym(ptr_tmpfig);
	}
	
	
	
	
	if(figID==CL_ACCORDO)
		return ptr_fig;
	else
		return ptr_newfig;
	
}

Figura * Battuta::CreaNota(ClassID targetIDnota)
{
	Figura *ptr_newfig=NULL;
	
	switch (targetIDnota)
	{
	case  CL_NBREVE:
		{
			NBreve *ptrNota = new NBreve;
			ptr_newfig= (Figura *) ptrNota;
		}
		break;
	case  CL_NSEMIBREVE:
		{
			NSemibreve *ptrNota = new NSemibreve;
			ptr_newfig= (Figura *) ptrNota;
		}
		break;
	case  CL_NMINIMA:
		{
			NMinima *ptrNota = new NMinima;
			ptr_newfig= (Figura *) ptrNota;
		}
		break;
	case  CL_NSEMIMINIMA:
		{
			NSemiminima *ptrNota = new NSemiminima;
			ptr_newfig= (Figura *) ptrNota;
		}
		break;
	case  CL_NCROMA:
		{
			NCroma *ptrNota = new NCroma;
			ptr_newfig= (Figura *) ptrNota;
		}
		break;
	case  CL_NSEMICROMA:
		{
			NSemicroma *ptrNota = new NSemicroma;
			ptr_newfig= (Figura *) ptrNota;
		}
		break;
	case  CL_NBISCROMA:
		{
			NBiscroma *ptrNota = new NBiscroma;
			ptr_newfig= (Figura *) ptrNota;
		}
		break;
	case  CL_NSEMIBISCROMA:
		{
			NSemibiscroma *ptrNota = new NSemibiscroma;
			ptr_newfig= (Figura *) ptrNota;
		}
		break;
	case  CL_NFUSA:
		{
			NFusa *ptrNota = new NFusa;
			ptr_newfig= (Figura *) ptrNota;
		}
		break;
		
	}
	
	return ptr_newfig;
}

void Battuta::View_Layer(bool view, int num_layer)
{
	ListaFigure *pLayer;
	int color,layerColors[NUM_MAX_LAYER]={BLU,ROSSO,GIALLO,VERDE,VERDEC,VERDEA,CELESTE,VIOLA,MAGENTA,ARANCIO,MARRONE,GIALLOC};
	
	pLayer=GetPLayer(num_layer);
	color=layerColors[num_layer];
	
	if(pLayer!=NULL)
	{
		Figura *fig=pLayer->GetFirstFig(); 
		while(fig!=NULL)
		{
			if(fig->GetID()==CL_ACCORDO)
			{
				Accordo *acc=(Accordo*)fig;
				Figura* tmp=acc->GetFirstF();
				while(tmp!=NULL)
				{
					if (view)
						tmp->SetColor((ColorIcon)color);
					else 
						tmp->SetColor((ColorIcon)black);
					tmp=acc->GetNextF(tmp);
				}
				
			}
			else
			{
				if (view)
					fig->SetColor((ColorIcon)color);
				else 
					fig->SetColor((ColorIcon)black);
			}
			
			fig=pLayer->GetNextFig();
		} 
	}	 
}

void Battuta::Transfer_Layer(Battuta *fromBat, int layerFrom, short layer)
{
	ListaFigure *layerFromBat;
	Figura *fig;
	
	layerFromBat=fromBat->GetPLayer(layerFrom);
	
	if(layerFromBat!=NULL)
	{
		fig=layerFromBat->GetFirstF();
		while(fig!=NULL)
		{
			switch(fig->GetID())
			{
			case CL_ACCORDO:
				{
					Accordo *acc=(Accordo*)fig;
					Figura* tmp=acc->GetFirstF();
					while(tmp!=NULL)
					{
						tmp->SetPBattuta(this);
						tmp=acc->GetNextF(tmp);
					}
					
				}
				break;
			case CL_GRUPPONOTE:
				{
					GruppoNote *grp=(GruppoNote *)fig;
					Figura* tmp=grp->GetFirstF();
					while(tmp!=NULL)
					{
						tmp->SetPBattuta(this);
						tmp=grp->GetNextF(tmp);
					}
					
				}
				break;
			default:
				break;
			}
			AddFig(fig,layer);
			fig=layerFromBat->GetNextF(fig);
		}
		layerFromBat->Clear();
	}	 
}

void Battuta::Transfer_Layer_MergeParts(Battuta *fromBat)
{
	for(int lay=0; lay<2; lay++)
		Transfer_Layer(fromBat,lay,lay+2);
}

bool Battuta::CompareIntest(Battuta *ext_bat)
{
	Intestazione *pIntest=NULL;
	Intestazione *pIntest2compare=NULL;
	
	Intestazione *pIntestFin=NULL;
	Intestazione *pIntestFin2compare=NULL;
	
	
	if(intest!=NULL && ext_bat->GetIntest()!=NULL)
	{
		pIntest=intest[0];
		pIntest2compare = ext_bat->GetIntest();
	}
	else
		return false;
	
	bool intestIniz = pIntest->GetChiave()->GetID() == pIntest2compare->GetChiave()->GetID() &&
					   pIntest->GetArmatura().GetArmatura() == pIntest2compare->GetArmatura().GetArmatura() &&
					   pIntest->GetTempo().GetDenominatore() == pIntest2compare->GetTempo().GetDenominatore() &&
					   pIntest->GetTempo().GetNumeratore() == pIntest2compare->GetTempo().GetNumeratore();
	
	
	return (intestIniz); 
}

void Battuta::SwitchLayer(int from, int to)
{
	Layer *tmp, *empty;
	tmp = LayerVector[from];
	//Si deve gestire il problema che per default i layer 1 e 2 devono esistere anche se vuoti
	//pertanto se il layer di provenienza non esiste perche' il ptr  NULL occorre creare un
	//layer vuoto da agganciare al posto del layer che stiamo spostando
	if((from == 0 || from == 1) && LayerVector[to]==NULL)
	{
		empty = new Layer;
		LayerVector[from]=empty;
	}
	else
		LayerVector[from]=LayerVector[to];
	LayerVector[to]=tmp;
}



void Battuta::Multi2Single_Transfer_Layer(Battuta *fromBat, int staffnum)
{
	ListaFigure *layerFromBat;
	Figura *fig;
	short i, lay;
	
	lay=0;
	for(i=0; i<NUM_MAX_LAYER; i++)
	{
		layerFromBat=fromBat->GetPLayer(i);
		if(layerFromBat!=NULL)
		{
			
			fig=layerFromBat->GetFirstF();
			
			while(fig!=NULL)
			{
				short minStaff,maxStaff;
				fig->GetMinMaxStaff(minStaff,maxStaff);
				if(minStaff==staffnum && maxStaff==staffnum)
				{
					switch(fig->GetID())
					{
					case CL_ACCORDO:
						{
							Accordo *acc=(Accordo*)fig;
							Figura* tmp=acc->GetFirstF();
							while(tmp!=NULL)
							{
								tmp->SetNStaff(0);
								tmp->SetPBattuta(this);
								tmp=acc->GetNextF(tmp);
							}
							fig->SetNStaff(0);
						}
						break;
					case CL_GRUPPONOTE:
						{
							GruppoNote *grp=(GruppoNote *)fig;
							Figura* tmp=grp->GetFirstF();
							while(tmp!=NULL)
							{
								tmp->SetNStaff(0);
								tmp->SetPBattuta(this);
								tmp=grp->GetNextF(tmp);
							}
							fig->SetNStaff(0);
						}
						break;
					default:
						fig->SetNStaff(0);
						break;
					}
					AddFig(fig,lay);
				}
				fig=layerFromBat->GetNextF(fig);
			}
			
			if(GetPLayer(lay)!=NULL && !GetPLayer(lay)->isEmpty())
			{
				lay++;
				layerFromBat->Clear();
			}
			
		}
	}
}

void Battuta::Single2Multi_Transfer_Layer(Battuta *fromBat, int staffnum)
{
	ListaFigure *layerFromBat;
	Figura *fig;
	short i, lay;
	
	lay=0;
	for(i=4*staffnum; i<4*(staffnum+1); i++)
	{
		layerFromBat=fromBat->GetPLayer(lay);
		if(layerFromBat!=NULL)
		{
			
			fig=layerFromBat->GetFirstF();
			if(fig!=NULL)
			{
				
				
				while(fig!=NULL)
				{
					
					switch(fig->GetID())
					{
					case CL_ACCORDO:
						{
							Accordo *acc=(Accordo*)fig;
							Figura* tmp=acc->GetFirstF();
							while(tmp!=NULL)
							{
								tmp->SetNStaff((short)staffnum);
								tmp->SetPBattuta(this);
								tmp=acc->GetNextF(tmp);
							}
							fig->SetNStaff((short)staffnum);
						}
						break;
					case CL_GRUPPONOTE:
						{
							GruppoNote *grp=(GruppoNote *)fig;
							Figura* tmp=grp->GetFirstF();
							while(tmp!=NULL)
							{
								tmp->SetNStaff((short)staffnum);
								tmp->SetPBattuta(this);
								tmp=grp->GetNextF(tmp);
							}
							fig->SetNStaff((short)staffnum);
						}
						break;
					default:
						fig->SetNStaff((short)staffnum);
						break;
					}
					AddFig(fig,i);
					fig=layerFromBat->GetNextF(fig);
				}
				layerFromBat->Clear();				
			}
			
		}
		lay++;
	}
}

void Battuta::Delete_Layer(int layer)
{
	ListaFigure *currLayer;
	currLayer=GetPLayer(layer);
	
	if(currLayer!=NULL)
		currLayer->ClearDeep();
}


void Battuta::UpDownAuto()
{
	ListaFigure *currLayer;
	ClassID figID;
	Figura *tmpfig;
	Bool gamboupdwn;
	int lgambo;
	Nota *pNota;
	int layer;
	
	for ( layer=NUM_MAX_LAYER-1; layer>=0; layer--)
	{
		currLayer=GetPLayer(layer);
		if(currLayer==NULL)
			continue;
		
		tmpfig=currLayer->GetFirstFig();
		while(tmpfig!=NULL)
		{
			figID=tmpfig->GetID();
			if((figID>=CL_NOTA)&&(figID<=CL_ACCORDO))
			{
				pNota=(Nota *) tmpfig;
				if(immissione.ImmRule(figID, pNota, this, NULL, layer))
				{
					if (pNota->GetNotina())
					{ 
						pNota->SetGamboUp();
					}
					else
					{
						gamboupdwn = TRUE;
						lgambo = int(posizionam.PosRule((DrawObject*)pNota, pNota, this,
							gamboupdwn, layer));
						pNota->SetLGambo(lgambo);
					}
				}	  
				else 
				{ 
					if (pNota->GetNotina())
					{ 
						pNota->SetGamboDwn();
					}
					else
					{
						gamboupdwn = FALSE;
						lgambo = int(posizionam.PosRule((DrawObject*)pNota, pNota, this,
							gamboupdwn, layer));  
						pNota->SetLGambo(-lgambo);
					}
				}
				pNota->UpDownSymb(this);
			}
			tmpfig=currLayer->GetNextFig();
		}
		
	}
	
}

void Battuta::Edit(Battuta *pBat)
{
	EditMeasureDlg *dialog=new EditMeasureDlg();
	dialog->SetPsp(parentScore);
	dialog->SetBat(pBat);
	dialog->Apply();
	dialog->Centre();
	dialog->Show(true);
}

bool Battuta::IsDisjoinable()
{
	short lay, minStaff, maxStaff;
	Layer *laytmp;
	
	for(lay=0; lay<NUM_MAX_LAYER; lay++)
	{
		laytmp=GetPLayer(lay);
		if(laytmp!=NULL && !laytmp->isEmpty())
		{
			laytmp->GetMinMaxStaff(minStaff, maxStaff);
			if(minStaff!=maxStaff)
				return false;
		}
	}
	return true;
}

double Battuta::GetSpazioIniziale(){

	double tmp;
	tmp=this->distanzaFig;
	tmp=tmp + Getkgiustificazione();
	

	return tmp;
}

//*BIAN-VERR*
Figura* Battuta::GetFiguraInLayer(short current_layer,NumCode FigRif)
{
	if (current_layer < 1) return NULL;

	Figura *fig;
	fig=LayerVector[current_layer]->GetFigura(FigRif);
	return fig;
}
//*BIAN-VERR*

//*BIAN-VERR*
CmdResult Battuta::GetSymCode(SymCmd& MyCmd, Battuta *pBattuta)
{   
	NumCode Appo;
	SymCmd SCmd;
	CopiaCommandStruct(&SCmd,&MyCmd);
/*  	switch(SCmd.CommandId)
	{  
	case CMD_BATTUTA_INS_NBREVE:		
	case CMD_BATTUTA_INS_NSEMIBREVE:		
	case CMD_BATTUTA_INS_NMINIMA:	
	case CMD_BATTUTA_INS_NSEMIMINIMA:			
	case CMD_BATTUTA_INS_NCROMA:	
	case CMD_BATTUTA_INS_NSEMICROMA:
	case CMD_BATTUTA_INS_NFUSA:	
	case CMD_BATTUTA_INS_NBISCROMA:	
	case CMD_BATTUTA_INS_NSEMIBISCROMA:
	case CMD_BATTUTA_INS_PQUABATT:
    case CMD_BATTUTA_INS_PDUEBATT:
    case CMD_BATTUTA_INS_PSEMIBREVE:
    case CMD_BATTUTA_INS_PMINIMA:
    case CMD_BATTUTA_INS_PSEMIMINIMA:
    case CMD_BATTUTA_INS_PCROMA:
    case CMD_BATTUTA_INS_PSEMICROMA:
    case CMD_BATTUTA_INS_PBISCROMA:
    case CMD_BATTUTA_INS_PSEMIBISCROMA:
    case CMD_BATTUTA_INS_PFUSA:
    case CMD_BATTUTA_INS_PGENERICA:
*/		if(LayerVector[SCmd.ArgV[1]-1]!=NULL)  
		{	
			Appo=LayerVector[SCmd.ArgV[1]-1]->GetCodeCounter();
			if(Appo!=0) 
			{ 
				MyCmd.ArgV[6]=Appo; MyCmd.ArgV[4]=SCmd.ArgV[4];
				return CMD_OK; 
			}
		}
		return CMD_FAIL;	
	
	}/*
}*/
//*BIAN-VERR*

Layer* Battuta::GetFirstLayer()
{
	for (int i=0;i<NUM_MAX_LAYER;i++)
	{
		if(LayerVector[i] && LayerVector[i]->GetNumObj()>0)
			return LayerVector[i];
	}
	return NULL;
}

Layer* Battuta::GetNextLayer(Layer* pL)
{
	if(!pL) return NULL;

	int i=0;
	for (;i<NUM_MAX_LAYER;i++)
	{
		if(LayerVector[i] == pL)
			break;
	}

	if(i==NUM_MAX_LAYER-1)
		return NULL;

	for (i++;i<NUM_MAX_LAYER;i++)
	{
		if(LayerVector[i] && LayerVector[i]->GetNumObj()>0)
			return LayerVector[i];
	}

	return NULL;
}

Layer* Battuta::GetPrevLayer(Layer* pL)
{
	if(!pL) return NULL;

	int i=NUM_MAX_LAYER-1;
	for (;i>=0;i--)
	{
		if(LayerVector[i] == pL)
			break;
	}

	if(i==0)
		return NULL;

	for (i--;i>=0;i--)
	{
		if(LayerVector[i] && LayerVector[i]->GetNumObj()>0)
			return LayerVector[i];
	}

	return NULL;
}

Layer* Battuta::GetLastLayer()
{
	for (int i=NUM_MAX_LAYER-1;i>=0;i--)
	{
		if(LayerVector[i] && LayerVector[i]->GetNumObj()>0)
			return LayerVector[i];
	}
	return NULL;
}

//-- Graphic Layer Ordering Top_Bottom --//

Layer* Battuta::GetBottomLayer(){
	float avg = 10000.0;
	float cur;

	Layer* pLay = GetFirstLayer();
	Layer* pMinLay = NULL;
	while (pLay)
	{
		cur = pLay->GetAltezzaMedia();
		if(cur < avg)
		{
			avg = cur;
			pMinLay = pLay;
		}
		pLay = GetNextLayer(pLay);
	}
	return pMinLay;
}

Layer* Battuta::GetMajorLayer(Layer* pL){
	float avg = 10000.0;
	float inf = pL->GetAltezzaMedia();
	float cur;

	Layer* pLay = GetFirstLayer();
	Layer* pMinLay = NULL;
	while (pLay)
	{
		cur = pLay->GetAltezzaMedia();
		if(cur > inf && cur < avg)
		{
			avg = cur;
			pMinLay = pLay;
		}
		pLay = GetNextLayer(pLay);
	}
	return pMinLay;
}

Layer* Battuta::GetMinorLayer(Layer* pL){
	float avg = -10000.0;
	float sup = pL->GetAltezzaMedia();
	float cur;

	Layer* pLay = GetLastLayer();
	Layer* pMaxLay = NULL;
	while (pLay)
	{
		cur = pLay->GetAltezzaMedia();
		if(cur < sup && cur > avg)
		{
			avg = cur;
			pMaxLay = pLay;
		}
		pLay = GetPrevLayer(pLay);
	}
	return pMaxLay;
}

Layer* Battuta::GetTopLayer(){
	float avg = -10000.0;
	float cur;

	Layer* pLay = GetLastLayer();
	Layer* pMaxLay = NULL;
	while (pLay)
	{
		cur = pLay->GetAltezzaMedia();
		if(cur > avg)
		{
			avg = cur;
			pMaxLay = pLay;
		}
		pLay = GetPrevLayer(pLay);
	}
	return pMaxLay;
}