/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#ifndef BATTUTA_HPP
#define BATTUTA_HPP

#include "drw/drawobje.hpp" 
#include "bat/listafig.hpp"
#include "bat/layer.hpp"  
#include "bat/def_bat.hpp" 
#include "spa/def_spa.hpp" 
#include "spa/pentagra.hpp" 
#include "bat/intestaz.hpp" 
#include "bat/barra.hpp"
#include "bat/chiave.hpp"
#include "bat/tempo.hpp"
#include "sca/numbatt.hpp"
#include "sca/numgrand.hpp"
#include "sca/numpausa.hpp"
#include "sca/numugual.hpp"
#include "sca/scansion.hpp"
#include "sca/lettera.hpp"
#include "sca/tsalto.hpp"
#include "sca/riferimento.hpp"
#include "sca/segno.hpp"
#include "sca/coda.hpp"

#include "sca/moviment.hpp"
#include "fig/gruppono.hpp"

class MoodsMidiWrite;

class Nota;

class Spartito;

class Battuta: public DrawObject
{
private:           
	NumCode Note2GruppoNote(short current_layer, TipoInserimento tpIns, NumCode figCode1, NumCode figCode2);
	// Crea un gruppo note sul relativo layer.
	
	Bool GruppoNote2Note(GruppoNote*);
	// Scioglie il gruppo note reinserendo le note o accordi singoli nella lista. 
	
	
	NumCode InsNota(int Altezza, Bool SegnoAltezza, ClassID figID, short current_layer,short current_staff,
		TipoInserimento GamboUp, Bool inAccordo, NumCode figCode, 
		Bool InsInTesta, NoteHead_ID ht, NumCode thisCode = 0);
	Bool InsInAccordo(Nota *pNota, Figura *pFig, Layer *pLayer,
		TipoInserimento GamboUp, short current_layer,short current_staff);
	// Se pFig punta un accordo e la nota puntata da pNota e` compatibile
	// con l'accordo, viene inserita.
	// Se pFig punta uno spazio, la nota va a sostituire lo spazio.
	NumCode InsPausa(int Altezza, Bool SegnoAltezza, ClassID figID, short current_layer,short current_staff,
		TipoInserimento AltVariabile, Bool inAccordo, NumCode figCode,
		Bool InsInTesta, NumCode thisCode = 0);
	
	NumCode InsSpazio(short current_layer, NumCode figCode, Bool InsInTesta, short CodeCommand, short current_staff, NumCode thisCode = 0);
	
	
	NumCode InsRipetizione(ClassID figID, NumCode figCode, Bool InsInTesta,short current_layer,short current_staff);
	// Inserisce un simbolo di Ripetizione.
	// Ritorna FALSE se fallisce.
	
	Bool InsNumPausa(unsigned char* s);
	// Inserisce un simbolo di NumPausa.
	
	
	// Ritorna FALSE se fallisce.
	Bool InsNumRipBattuta(unsigned char* s);
	// Inserisce un simbolo di NumPausa.
	// Ritorna FALSE se fallisce.
    
	
    Bool InsNumUguale(unsigned char* s);
	// Inserisce un simbolo di NumUguale.
   	// Ritorna FALSE se fallisce.
	
	
    Bool InsScansione(short n);
    // Inserisce un simbolo di Scansione.
	// Ritorna FALSE se fallisce.
	Bool InsTSalto(unsigned char* s);
	// Inserisce un simbolo di TSalto.
	// Ritorna FALSE se fallisce.
	Bool InsRiferimento(short CodeCommand);
	// Inserisce un simbolo di Riferimento.
	// Ritorna FALSE se fallisce.
    
	Bool InsLettera(unsigned char* s);
	// Inserisce un simbolo di Lettera.
	// Ritorna FALSE se fallisce.
	
	Bool InsMovimento(unsigned char *p, ClassID ident, Bool punt, int s);
	// Inserisce un simbolo di Movimento: un testo piu' il metronomo. 
	// Indica il l'andamento del brano. Ritorna FALSE se fallisce.
	// p e' il testo che verra' stampato; ident e' il tipo di nota di
	// riferimento, punt indica se la nota e' puntata oppure no e s e' 
	// il numero associato alla nota che da l'indicazione della velocita' 
	// di esecuzione del brano.
	
	//void AdjustTremolo(Figura *fig, short current_layer);
	//void AdjustGlissato(Figura *fig, short current_layer);
	
protected:
	tipoBattuta tipoBat;
	// Tipo di battuta
	
	int NumProgress;
	// Numero progressivo della battuta nello spartito.
	
	NumCode NumericCode;
	// codice numerico della battuta
	
	NumCode NumPagDLIOO;
	// Indica se una battuta del DLIOO e' di inizio pagina(valore != 0) oppure no
	// (valore nullo). Se diverso da zero indica il numero di pagina.
	
	NumCode NumPagMASAE;
	// Indica se una battuta del MASAE o del MASE e' di inizio pagina(valore != 0) oppure no
	// (valore nullo). Se diverso da zero indica il numero di pagina.
	
	NumCode NumPagMASE;
	// Indica se una battuta del MASAE o del MASE e' di inizio pagina(valore != 0) oppure no
	// (valore nullo). Se diverso da zero indica il numero di pagina.
	
	Bool FirstColonna;
	// Indica se la battuta e' in testa alla colonna di battute	
	
	Intestazione *intest[NUM_MAX_INTEST]; 
	// Vettore di intestazioni
	// Gli indici vanno da 0 a 2 compresi.
	// Gli indici di staff invece, vanno da 1 a 3
	
	Intestazione *intest_end[NUM_MAX_INTEST];
	// Intestazione finale della battuta.
	
	short NumberOfStaff; // numero dei pentagrammi
	
	Barra *ptrBarra;      // Barra della battuta.
	
	Barra *ptrBarraPrec;  // Barra della battuta precedente.
	
	tipoBarra tpBarra;    // Tipo della Barra della battuta attuale.
	
	NumBattuta *ptrNumBattuta; // Puntatore al numero della battuta.
	
	NumGrande *ptrNumGrande;   // Puntatore a NumGrande.
	
	Scansione *ptrScansione;   // Puntatore a Scansione.
	
	Lettera *ptrLettera;  // Puntatore a Lettera.
	
	TSalto *ptrTSalto;   // Puntatore a TSalto.
	
	Movimento *ptrMovimento; // Puntatore a Movimento.
	
	Riferimento *ptrRiferimento; 	// Puntatore a Riferimento
	
	int NStaffLine;	// Numero delle righe del pentagramma 
	
	Layer *LayerVector[NUM_MAX_LAYER];  
	// Vettore dei layer. Max fino ad 12
	// Gli indici vanno da 0 a 11.
	
	int numeroFig[NUM_MAX_LAYER]; 
	// vettore contenente il numero di
	// figure del layer i-esimo.
	
	VUnit VU2Figure;   // Distanza dalla posizione della battuta alla x della prima figura.
	
	VUnit distanzaFig; // Distanza tra due figure successive nei due layers.
	
	
	VUnit BATTUTA2Lft,BATTUTA2Rgt;
	// Distanze da AbsPos dei punti sinistro e destro del rettangolo 
	// contenente la battuta.
	
	VUnit xBarra;
	// Distanza aggiuntiva della barra dall'ultima figura
	// per eventuali aggiustamenti a fine pentagramma.
	
	VUnit VU2Up, VU2Dwn;
	
	int NumMultiRest;  // indica il numero di battute "contenute" nella multirest
	
	Bool IsMultiRest;       // se TRUE la battuta fa parte di una multirest 
	
	Battuta *pMultiRest;
	// puntatore della prima battuta di pausa di una
	// multirest. Punta alla battuta multirest. 
	static bool AutoBeaming;
	
	bool mainScoreLineBreak[2];
	bool partLineBreak[2];
	
public:  
	Battuta();
	~Battuta();
	
	double GetSpazioIniziale();
	Spartito * parentScore; 
	
	Battuta(int Nrighi);	
	
	
	// tipo giustificazione
	TipoGiust tipogiust[2], tipogiustparte[2];
	// parametro
	double kgiustificazione[2], kgiustificazioneparte[2];
	// spazi fino alla prima figura
	int EverJustified[4];
	// mi dice se la battuta e' mai stata giustificata
	// 1 - dir computer view 2 - orc computer view view
	// 3 - dir print view 4 - orc print view
	// 
	
	UL DirSp2FirstFig[NUM_MAX_LAYER][2];
 	  UL OrcSp2FirstFig[NUM_MAX_LAYER][2];
	  UL DirSp2FirstFigLine[NUM_MAX_LAYER][2];
	  UL OrcSp2FirstFigLine[NUM_MAX_LAYER][2];
	  UL DirSp2FirstFigUt[NUM_MAX_LAYER][2];
	  UL OrcSp2FirstFigUt[NUM_MAX_LAYER][2];
	  // spazi utente prima della 1. figura
	  
	  
	  int positionrelat;
	  // posizione relativa dell'ancoraggio, da passare a Spazio
	  void Suona(MoodsMidiWrite *);

	  Layer *GetPLayer(short current_layer); 
	  // Riceve come parametro il layer
	  // Restituisce il puntatore al Layer i-esimo
	  int GetNUsedLayers();
	  
	  unsigned long start_ticks;
	  
	  int GetNStaffLine() {return NStaffLine;}
	  void SetNStaffLine(int nstaffline)
	  {
		  
		  NStaffLine=nstaffline;
		  if(ptrBarra!=NULL)
			  ptrBarra->SetNStaffLine(NStaffLine);
	  }
	  
	  Spartito* GetParentScore() {return parentScore;}
	  void SetParentScore(Spartito *parentscore) {parentScore=parentscore;}	
	  
	  VUnit AskWidth(int DirOrOrc, int LineBreakOrNot);
	  
	  int GetEverJustified(int part, int view);
	  void SetEverJustified(int part, int view, int value); 
	  
	  inline int Getpositionrelat() {return positionrelat;}
	  inline void Setpositionrelat(int pos) {positionrelat=pos;}
	  
	  double Getkgiustificazione();
	  double Getkgiustificazione(int a);
	  void Setkgiustificazione(double k);
	  void Setkgiustificazione(double k, int a);
	  
	  double Getkgiustificazioneparte();
	  double Getkgiustificazioneparte(int a);
	  void Setkgiustificazioneparte(double k);
	  void Setkgiustificazioneparte(double k, int a);
	  
	  TipoGiust Gettipogiust();
	  TipoGiust Gettipogiust(int a);
	  void Settipogiust(TipoGiust t);
	  void Settipogiust(TipoGiust t, int a);
	  
	  TipoGiust Gettipogiustparte();
	  TipoGiust Gettipogiustparte(int a);
	  void Settipogiustparte(TipoGiust t);
	  void Settipogiustparte(TipoGiust t, int a);
	  
	  
	  UL GetDirSp2FirstFig(short current_layer);
	  UL GetOrcSp2FirstFig(short current_layer);
	  UL GetDirSp2FirstFigLine(short current_layer);
	  UL GetOrcSp2FirstFigLine(short current_layer);
	  UL GetDirSp2FirstFigUt(short current_layer);
	  UL GetOrcSp2FirstFigUt(short current_layer);
	  
	  
	  void SetDirSp2FirstFig(short current_layer,UL par);
	  void SetOrcSp2FirstFig(short current_layer,UL par);
	  void SetDirSp2FirstFigLine(short current_layer,UL par);
	  void SetOrcSp2FirstFigLine(short current_layer,UL par);
	  void SetDirSp2FirstFigUt(short current_layer,UL par);
	  void SetOrcSp2FirstFigUt(short current_layer,UL par);
	  
	  
	  
	  UL GetDirSp2FirstFig(short current_layer,int a);
	  UL GetOrcSp2FirstFig(short current_layer,int a);
	  UL GetDirSp2FirstFigLine(short current_layer,int a);
	  UL GetOrcSp2FirstFigLine(short current_layer,int a);
	  UL GetDirSp2FirstFigUt(short current_layer,int a);
	  UL GetOrcSp2FirstFigUt(short current_layer,int a);
	  
	  
	  
	  void SetDirSp2FirstFig(short current_layer,UL par, int a);
	  void SetOrcSp2FirstFig(short current_layer,UL par, int a);
	  void SetDirSp2FirstFigLine(short current_layer,UL par, int a);
	  void SetOrcSp2FirstFigLine(short current_layer,UL par, int a);
	  void SetDirSp2FirstFigUt(short current_layer,UL par, int a);
	  void SetOrcSp2FirstFigUt(short current_layer,UL par, int a);
	  
	  
	  DrawObject * GetObject(SymCmd *MyCmd,short current_staff);
	  // Ottengo il puntatore di un oggetto colpito da un comando simbolico !
	  // Funziona per comandi che riguardano la battuta come armature, chiavi, etc.
	  
	  void SetNumMultiRest(int nb);
	  int GetNumMultiRest();
	  
	  void SetPMultiRest(Battuta *pb);
	  Battuta* GetPMultiRest();
	  
	  void SetIsMultiRest(Bool ismrest=TRUE);
	  Bool GetIsMultiRest();
	  
	  
	  
	  // Qui si setta il numero di pentagrammi
	  // per ogni battuta.
	  // Lo staff inteso come numero di
	  // pentagrammi va da 1 a 3 compresi.
	  //
	  // Per Intestazione* intest[NUM_MAX_INTEST]
	  // gli indici vanno da 0 a 2,
	  
	  void SetNumberOfStaff(short i); 
	  short GetNumberOfStaff(void); 
	  
	  
	  void SetPNumBattuta(NumBattuta *);
	  void SetPNumGrande(NumGrande *p);
	  NumGrande *GetPNumGrande();
	  void SetPScansione(Scansione *);
	  void SetPLettera(Lettera *);
	  Lettera *GetPLettera();
	  
	  
	  void SetPTSalto(TSalto *);
	  TSalto *GetPTSalto();

	  void SetPRiferimento(Riferimento *);
	  Riferimento *GetPRiferimento();
	  
	  void SetPMovimento(Movimento *);
	  Movimento *GetPMovimento();
	  
	  
	  void SetNumProgress(int num);
	  int GetNumProgress(void);
	  
	  void SetNumPagDLIOO(int num);
	  NumCode GetNumPagDLIOO(void);
	  void SetNumPagMASAE(int num);
	  NumCode GetNumPagMASAE(void);
	  void SetNumPagMASE(int num);
	  NumCode GetNumPagMASE(void);
	  
	  float GetMetronomoTimeExec();
	  float GetTempoTimeExec();
	  void DrawBall();
	  
	  void SetVU2Lft(VUnit vu);  
	  void SetVU2Rgt(VUnit vu);  
	  
	  VUnit GetVU2Up();
	  VUnit GetVU2Dwn();
	  VUnit GetVU2Lft();
	  VUnit GetVU2Rgt(); 
	  
	  void SetVU2Up(VUnit vu);
	  void SetVU2Dwn(VUnit vu);
	  
	  void SetDistanzaFig(VUnit df); // Imposta la distanza tra le figure
	  
	  void SetxBarra(VUnit xpb=0);  // Imposta la distanza aggiuntiva xPosBarra.
	  
	  void SetNumericCode(NumCode nc);
	  NumCode GetNumericCode();
	  
      void Setup(VUnit vf,VUnit MaxWdt=0);
	  	  
	  
	  void SetVU(VUnit MaxWdt=0);
	  
	  
	  void SetupNumeroFig();
	  
	  unsigned long int GetNumeroFig();
	  // Restituisce il numero massimo di figure contenute nella battuta
	  // (che  il il numero di figure del layer che ne contiene di pi).
	  
	  Intestazione* GetIntest(short current_staff=0);
	  Intestazione** GetAllIntest();
	  // Restituisce il puntatore all'current_staff.     
	  
	  Intestazione* GetIntestEnd(short current_staff=0);
	  Intestazione** GetAllIntestEnd();
	  // Restituisce il puntatore all'intestazione finale: permette di considerare
	  // la presenza dei cambi chiave interni alla battuta con propagazione alle successive
	  
	  
	  void SetBattuta(tipoBattuta tpbat=GENERICA, Intestazione **intst=NULL,
		  tipoSpartito tsp=SPART_GENERICO);
	  // Imposta il tipo di battuta (di default la battuta  GENERICA). 
	  
	  
	  tipoBattuta GetBattuta(); // Restituisce il tipo di battuta.  
	  
	  void SetChiave(short current_staff, Chiave *chv);  // Imposta il tipo di chiave. 
	  
	  void SetArmatura(short current_staff, tipoArmaturaChiave arm);
	  // Imposta il tipo di armatura (di default l'armatura  DO_maggiore). 
	  
	  Chiave *GetChiave(short current_staff);    // Restituisce la chiave della battuta.  
	  
	  tipoArmaturaChiave GetArmatura(short current_staff);  // Restituisce l'armatura della battuta.  
	  
	  void SetTempo(char tmp[]);   // Imposta il tipo di tempo. 
	  
	  void CopyIntest(short current_staff, Intestazione* intes);   // copia l'intestazione fornita sull'intestazione della battuta.
	  void CopyAllIntest(Intestazione** intes);   // copia tutte le intestazione fornite sull'intestazione della battuta.
	  
	  void SetPos(Pentagramma*, const Point&); // Imposta la posizione della battuta e di tutti i suoi elementi.
	  
	  void Draw();  //Disegna battuta.
	  
	  void Select();
	  // seleziona l'oggetto impostando il colore di selezione 
	  // e ridisegnando l'oggetto.
	  
	  void Deselect();
	  // deseleziona l'oggetto impostando il colore normale 
	  // ridisegnando l'oggetto.
	  
	  Bool Hit(const Point&,SelObj,DrawObject*&,SymPath&); 
	  // restituisce il puntatore all'oggetto stesso o al sottoggetto
	  // che  stato colpito (l'oggetto contiene il Point). 
	  // Se vale NULL o non si  colpito niente  oppure l'oggetto 
	  // che  stato colpito non  del tipo SelObj.
	  
	  Bool InsertHit(const Point& posiz, short current_layer, Bool inAccordo,
		  Bool& questaBat, NumCode& Altezza, Bool& SegnoAltezza, 
		  NumCode& FigCode, NumCode& InsInTesta, short& staff);
	  // Metodo da usare per le battute selezionate con BATPART_SEL.
	  
	  Bool DeleteHit(const Point& posiz, NumCode& FigCode, NumCode& InsInTesta);
	  // Fornisce il codice della figura del layer 1 "colpita" da p.
	  
	  Bool Hit(const class Rectangle& r,SelObj selObj,short current_layer,SymPath&,SymPath&);
	  // restituisce una lista degli oggetti di tipo selObj contenuti
	  // nel rettangolo di selezione Rectangle. 
	  // Se vale NULL o il rettangolo non contiene niente oppure gli oggetti 
	  // contenuti nel rettangolo non sono del tipo SelObj.
	  
	  void AdjustTremolo(Figura *fig, short current_layer);
	  void AdjustGlissato(Figura *fig, short current_layer);
	  
	  Bool SymbolicHit(SymPath& , SelObj selObj,DrawObject *& drw);
	  // Decodifica un path.
	  
	  void SetBarra(tipoBarra tbar);
	  void SetptrBarra(tipoBarra tbar);
	  
	  Figura *Symbolic(SymPath&);
	  // Questo metodo verifica dato un indirizzo
	  // simbolico di una figura se questa fa parte
	  // della battuta corrente,restituendo in tal
	  // caso il puntatore attuale alla figura.Se
	  // la figura non fa parte della battuta rende
	  // NULL.
	  	  
	  void SetptrBarraPrec(tipoBarra tbar, Bool IsNull);
	  
	  Barra* GetBarra();
	  // ritorna il puntatore alla barra della battuta.
	  
	  
	  void EliminaGruppi();
	  // Converte in note singole tutti i gruppi della battuta.
	  void EliminaGruppi(int nLayer);
	  // Converte in note singole tutti i gruppi del layer.
	  
	//*BIAN-VERR*
	Figura* GetFiguraInLayer(short current_layer,NumCode FigRif);
	//*BIAN-VERR*

	//*BIAN-VERR*
	CmdResult GetSymCode(SymCmd&, Battuta *pb=NULL);
	//Inserisce nell'UsrSymCmd il NC senza istaziare la nota (al momento)
	//*BIAN-VERR*

	  CmdResult SymCommand(SymCmd&, Battuta *pb=NULL);
	  // Esegue il comando indicato.
	  // Ritorna CMD_OK se il comando ha avuto successo, CMD_FAIL se il comando
	  // non e' stato eseguito, CMD_UNKNOWN se il comando non lo riguarda.
	  
	  Bool DeleteSym(DrawObject*);
	  // "Cancella" il simbolo specificato. Ritorna TRUE se e' stato cancellato.
	  
	  void AddFig(Figura* fig,short current_layer);  
	  
	  
	  Figura* GetFig(short current_layer, NumCode nc);
	  // Ritorna il puntatore alla figura nel layer lay con NumericCode nc.
	  
	  Bool FigPtr2SymPath(SymPath&, Figura*);
	  // ricostruisce il PATH di una figura a partire dal puntatore.
	  
	  void Edit(Battuta *pBat);	
	  
	  void EraseBattuta();
	  // Converte in spazi tutte le figure della battuta.
	  
	  void SetCodeCounterFig(short current_layer,NumCode nc); 
	  
	  NumCode GetCodeCounterFig(short current_layer); 
	  	  
	  void SetFirsColonna(Bool);
	  
	  Bool GetMultiVoice(); 
	  
	  int GetVoice(Figura *pFig);  
	  
	  void SetupSimboli(Bool beforeLegature=FALSE); 
	  
	  void SetupGambi(); 
	  
	  void Free();
	  // Dealloca tutti i puntatori connessi alla battuta.
	  
	  NumCode GetPos(short current_layer, Figura *ptr);
	  
	  Figura* GetFigTotal(short current_layer, NumCode nc);
	  // Dealloca tutti i puntatori connessi alla battuta.
	  
	  
	  Figura* GetFigAll(short current_layer, NumCode nc);
	  
	  Bool GetChiaveFig(short current_layer, Figura *pfig);
	  
	  void CheckBattuta();
	  
	  Bool CheckBatt(bool *layer=NULL, float *Layer_value=NULL); 
	  float CheckLayer(int nLayer);
	  
	  void SetupDurata();
	  
	  Tempo GetTempo(); 
	  
	  void BeamingAuto(short current_staff,short current_layer);
	  void BeamingAuto();
	  
	  bool GetMainScoreLineBreak(int i)				{return mainScoreLineBreak[i];};
	  void SetMainScoreLineBreak(int i, bool state) {mainScoreLineBreak[i]=state;}; 
	  bool GetPartLineBreak(int i)					{return partLineBreak[i];};
	  void SetPartLineBreak(int i, bool state)		{partLineBreak[i]=state;}; 
	  
	  
	  Figura* ChangeFigDuration(Figura *pFig, ClassID Target_ID);
	  Figura *CreaNota(ClassID targetIDnota);
	  
	  
	  
	  
	  int CheckBattPause(Battuta *pbat);
	  void MakeBattMultiRest(Battuta* pbat1, Battuta *pbat2, int nb);
	  void UpDownAuto();
	  
	  
	  void GiustificaBattuta(TipoGiust,double kGiust, bool clearSpUt=false);
	  
	  
	  Bool Transpose(int clef,int translation,int interval,int up, int numofstaff, Bool sharps, Bool adjust);
	  Bool TransposeClef(int clef, int numofstaff);
	  Bool TransposeInterval(int interval, int up, int numofstaff, Bool sharps, Bool adjust);
	  int  CalculateOffsetFromClefs(ClassID start,ClassID target);
	  void Translation(int translation, int numofstaff);
	  bool IsDisjoinable();
	  void Multi2Single_Transfer_Layer(Battuta *fromBat, int staffnum);
	  void Single2Multi_Transfer_Layer(Battuta *fromBat, int staffnum);
	  void Transfer_Layer_MergeParts(Battuta *fromBat);
	  bool CompareIntest(Battuta *ext_bat);
      void SwitchLayer(int from, int to);
	  static bool GetAutoBeaming() { return AutoBeaming;};
	  static void SetAutoBeaming(bool autobeaming) { AutoBeaming=autobeaming;};
	  
	  
	  void ResetIntRefs();
	  
	  void View_Link(bool);
	  void View_Layer(bool view, int num);
	  void Transfer_Layer(Battuta *fromBat, int layerFrom, short layer=0);
	  void Delete_Layer(int layer);
	  
	  
	  
	  void Set_start_ticks(unsigned long ticks){start_ticks=ticks;}
	  unsigned long Get_start_ticks(){return start_ticks;}

	  int SaveXML(FILE *);

	  Layer* GetFirstLayer();
	  Layer* GetPrevLayer(Layer* pL);
	  Layer* GetNextLayer(Layer* pL);
	  Layer* GetLastLayer();

	  //-- Graphic layers ordering --//
	  Layer* GetBottomLayer();
	  Layer* GetMinorLayer(Layer* pL);
	  Layer* GetMajorLayer(Layer* pL);
	  Layer* GetTopLayer();
};

#endif //BATTUTA_HPP
