/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: cbasso8.cpp,v 1.1 2005/07/25 10:51:03 bellinpo Exp $ */



#include "bat/cbasso8.hpp"

VUnit CBasso8::GetVU2Up()  
{return ((GetChiavina())? CBASSO82UP_CHIAVINA: CBASSO82UP);};

VUnit CBasso8::GetVU2Dwn() 
{return ((GetChiavina())? CBASSO82DWN_CHIAVINA: CBASSO82DWN);};

VUnit CBasso8::GetVU2Lft() 
{return ((GetChiavina())? CBASSO82LFT_CHIAVINA: CBASSO82LFT);};

VUnit CBasso8::GetVU2Rgt() 
{return ((GetChiavina())? CBASSO82RGT_CHIAVINA: CBASSO82RGT);};

int CBasso8::GetShift() 
{return CBASSO8SHIFT; };

int CBasso8::GetAltezza() 
{
	if (GetChiavina())
		return ALTEZZA_CBASSO8_INA;
	else 
		return ALTEZZA_CBASSO8;
};


void CBasso8::SetPos(DrawObject *d,const Point& p)
{
	DrawObject::SetPos(d,p);
	if (GetChiavina())
		GetAbsPos().y+=ALTEZZA_CBASSO8_INA*VU_PER_ROW/2; 
	else 
		GetAbsPos().y+=ALTEZZA_CBASSO8*VU_PER_ROW/2; 
}; 


void CBasso8::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	char code[2]={0,0};
	code[0]=((GetChiavina())? CODE_CBASSO8_CHIAVINA: CODE_CBASSO8);
	
	if (GetChiavina())
	{
		Scr->TextTo(GetAbsPos()+Point(DELTAXCBASSO8_CHIAVINA,DELTAYCBASSO8_CHIAVINA));
	}
	else
	{
		Scr->TextTo(GetAbsPos()+Point(DELTAXCBASSO8,DELTAYCBASSO8));
	}
	
	Scr->OutText((unsigned char *)code);
	
	Chiave::Draw();

	Scr->SetColor(oldcol);
};  

void CBasso8::Adjust(Bool primaleg, Battuta *pBattuta, short current_layer, short current_staff) 
{
	if(GetChiavina())
	{
		SetVU2UpF(CBASSO82UP_CHIAVINA);
		SetVU2DwnF(CBASSO82DWN_CHIAVINA);
		SetVU2LftF(CBASSO82LFT_CHIAVINA);
		SetVU2RgtF(CBASSO82RGT_CHIAVINA);
	}
	else
	{
		SetVU2DwnF(CBASSO82UP);
		SetVU2DwnF(CBASSO82DWN);
		SetVU2LftF(CBASSO82LFT);
		SetVU2RgtF(CBASSO82RGT);
	}
	
	Figura::Adjust(primaleg, pBattuta, current_layer, current_staff);
}