/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: csoprano.cpp,v 1.1 2005/07/25 10:51:05 bellinpo Exp $ */

#include "bat/csoprano.hpp"

VUnit CSoprano::GetVU2Up()  
{return ((GetChiavina())? CSOPRANO2UP_CHIAVINA: CSOPRANO2UP);};

VUnit CSoprano::GetVU2Dwn() 
{return ((GetChiavina())? CSOPRANO2DWN_CHIAVINA: CSOPRANO2DWN);};

VUnit CSoprano::GetVU2Lft() 
{return ((GetChiavina())? CSOPRANO2LFT_CHIAVINA: CSOPRANO2LFT);};

VUnit CSoprano::GetVU2Rgt() 
{return ((GetChiavina())? CSOPRANO2RGT_CHIAVINA: CSOPRANO2RGT);};

int CSoprano::GetShift() 
{return CSOPRANOSHIFT; };

int CSoprano::GetAltezza() 
{
	if (GetChiavina())
		return ALTEZZA_CSOPRANO_INA;
	else 
		return ALTEZZA_CSOPRANO;
};


void CSoprano::SetPos(DrawObject *d,const Point& p)
{
	DrawObject::SetPos(d,p);
	if (GetChiavina())
		GetAbsPos().y+=ALTEZZA_CSOPRANO_INA*VU_PER_ROW/2;
	else 
		GetAbsPos().y+=ALTEZZA_CSOPRANO*VU_PER_ROW/2; 
}; 


void CSoprano::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	char code[2]={0,0};
	code[0]=((GetChiavina())? CODE_CSOPRANO_CHIAVINA: CODE_CSOPRANO);

	if (GetChiavina())
		Scr->TextTo(GetAbsPos()+Point(DELTAXCSOPRANO_CHIAVINA,DELTAYCSOPRANO_CHIAVINA));
	else
		Scr->TextTo(GetAbsPos()+Point(DELTAXCSOPRANO,DELTAYCSOPRANO));
	Scr->OutText((unsigned char *)code);

	Chiave::Draw();

	Scr->SetColor(oldcol);
};  

void CSoprano::Adjust(Bool primaleg, Battuta *pBattuta, short current_layer, short current_staff) 
{
	if(GetChiavina())
	{
		SetVU2UpF(CSOPRANO2UP_CHIAVINA);
		SetVU2DwnF(CSOPRANO2DWN_CHIAVINA);
		SetVU2LftF(CSOPRANO2LFT_CHIAVINA);
		SetVU2RgtF(CSOPRANO2RGT_CHIAVINA);
	}
	else
	{
		SetVU2DwnF(CSOPRANO2UP);
		SetVU2DwnF(CSOPRANO2DWN);
		SetVU2LftF(CSOPRANO2LFT);
		SetVU2RgtF(CSOPRANO2RGT);
	}
	
	Figura::Adjust(primaleg, pBattuta, current_layer, current_staff);
}