/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: ctab.cpp,v 1.1 2005/07/25 10:51:06 bellinpo Exp $ */


#include "bat/ctab.hpp"

VUnit CTab::GetVU2Up()  
{return ((GetChiavina())? CTAB2UP_CHIAVINA: CTAB2UP);};

VUnit CTab::GetVU2Dwn() 
{return ((GetChiavina())? CTAB2DWN_CHIAVINA: CTAB2DWN);};

VUnit CTab::GetVU2Lft() 
{return ((GetChiavina())? CTAB2LFT_CHIAVINA: CTAB2LFT);};

VUnit CTab::GetVU2Rgt() 
{return ((GetChiavina())? CTAB2RGT_CHIAVINA: CTAB2RGT);};

int CTab::GetShift() 
{return CTABSHIFT; };

int CTab::GetAltezza() 
{
	if (GetChiavina())
		return ALTEZZA_CTAB_INA;
	else 
		return ALTEZZA_CTAB;
};


void CTab::SetPos(DrawObject *d,const Point& p)
{
	DrawObject::SetPos(d,p);
	if (GetChiavina())
		GetAbsPos().y+=ALTEZZA_CTAB_INA*VU_PER_ROW/2; 
	else 
		GetAbsPos().y+=ALTEZZA_CTAB*VU_PER_ROW/2; 
}; 


void CTab::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	char code[2]={0,0};
	code[0]=((GetChiavina())? CODE_CTAB_CHIAVINA: CODE_CTAB);

	if (GetChiavina())
		Scr->TextTo(GetAbsPos()+Point(DELTAXCTAB_CHIAVINA,DELTAYCTAB_CHIAVINA));
	else
		Scr->TextTo(GetAbsPos()+Point(DELTAXCTAB,DELTAYCTAB));
	
	Scr->OutText((unsigned char *)code);
	
	Chiave::Draw();

	Scr->SetColor(oldcol);
	
};  

void CTab::Adjust(Bool primaleg, Battuta *pBattuta, short current_layer, short current_staff) 
{
	if(GetChiavina())
	{
		SetVU2UpF(CTAB2UP_CHIAVINA);
		SetVU2DwnF(CTAB2DWN_CHIAVINA);
		SetVU2LftF(CTAB2LFT_CHIAVINA);
		SetVU2RgtF(CTAB2RGT_CHIAVINA);
	}
	else
	{
		SetVU2DwnF(CTAB2UP);
		SetVU2DwnF(CTAB2DWN);
		SetVU2LftF(CTAB2LFT);
		SetVU2RgtF(CTAB2RGT);
	}
	
	Figura::Adjust(primaleg, pBattuta, current_layer, current_staff);
}
