/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: ctenore.cpp,v 1.1 2005/07/25 10:51:06 bellinpo Exp $ */


#include "bat/ctenore.hpp"

VUnit CTenore::GetVU2Up()  
{return ((GetChiavina())? CTENORE2UP_CHIAVINA: CTENORE2UP);};

VUnit CTenore::GetVU2Dwn() 
{return ((GetChiavina())? CTENORE2DWN_CHIAVINA: CTENORE2DWN);};

VUnit CTenore::GetVU2Lft() 
{return ((GetChiavina())? CTENORE2LFT_CHIAVINA: CTENORE2LFT);};

VUnit CTenore::GetVU2Rgt() 
{return ((GetChiavina())? CTENORE2RGT_CHIAVINA: CTENORE2RGT);};

int CTenore::GetShift() 
{return CTENORESHIFT; };

int CTenore::GetAltezza() 
{
	if (GetChiavina())
		return ALTEZZA_CTENORE_INA;
	else 
		return ALTEZZA_CTENORE;
};


void CTenore::SetPos(DrawObject *d,const Point& p)
{
	DrawObject::SetPos(d,p);
	if (GetChiavina())
		GetAbsPos().y+=ALTEZZA_CTENORE_INA*VU_PER_ROW/2; 
	else 
		GetAbsPos().y+=ALTEZZA_CTENORE*VU_PER_ROW/2; 
}; 


void CTenore::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	char code[2]={0,0};
	code[0]=((GetChiavina())? CODE_CTENORE_CHIAVINA: CODE_CTENORE);

	if (GetChiavina())
		Scr->TextTo(GetAbsPos()+Point(DELTAXCTENORE_CHIAVINA,DELTAYCTENORE_CHIAVINA));
	else
		Scr->TextTo(GetAbsPos()+Point(DELTAXCTENORE,DELTAYCTENORE));
	Scr->OutText((unsigned char *)code);
	
	Chiave::Draw();

	Scr->SetColor(oldcol);
};  

void CTenore::Adjust(Bool primaleg, Battuta *pBattuta, short current_layer, short current_staff) 
{
	if(GetChiavina())
	{
		SetVU2UpF(CTENORE2UP_CHIAVINA);
		SetVU2DwnF(CTENORE2DWN_CHIAVINA);
		SetVU2LftF(CTENORE2LFT_CHIAVINA);
		SetVU2RgtF(CTENORE2RGT_CHIAVINA);
	}
	else
	{
		SetVU2DwnF(CTENORE2UP);
		SetVU2DwnF(CTENORE2DWN);
		SetVU2LftF(CTENORE2LFT);
		SetVU2RgtF(CTENORE2RGT);
	}
	
	Figura::Adjust(primaleg, pBattuta, current_layer, current_staff);
}
