/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

// IMPLEMENTAZIONE DEI METODI DI GIUSTIFICAZIONE

/* $Id: giustif.cpp,v 1.1 2005/07/25 10:51:08 bellinpo Exp $ */


// Per poter fare la giustificazione occorre fare un preprocessing 
// in cui si costruisce una struttura tabellare che contiene le
// informazioni su una fila verticale di battute (al limite una sola)
// e su tutte le figure di ogni battuta.
// Ogni colonna corrisponde a una voce del sistema,...quindi una battuta
// in polifonia ha due colonne. Ogni riga corrisponde ad un beatline, ovvero
// un momento musicale in cui e' suonata almeno una nota. Le voci che non
// suonano ad un certo beatline sono marchiate in corrispondenza di tale riga
// con NOP (No Operation) che puo' essere un flag.

#include "stdio.h"
#include <stdlib.h>
#include <math.h>

#include "bat/giustif.hpp"
#include "bat/battuta.hpp"

extern TipoLeggio tipoLeggio;

// se sono presenti ancoraggi prima della prima figura effettiva non modificare 
// lo spazio fino alla prima figura
int modificareprimospazio=YES;


Giustificazione::~Giustificazione()
{
	if(Sims!=NULL)
	{
		for (int i=0; i<nrighe; i++)
		{
			delete[] Sims[i];
		}
		delete Sims;
		Sims=NULL;
	}
	if(Bats!=NULL)
	{
		delete Bats;
		Bats=NULL;
	}
}


void Giustificazione::Costruisci(int nr, int nc)
{
	
	nrighe=nr;
	nmaxcolonne=nc;
	ncolonne=0;
	
	int i=0, j=0;
	
	// alloco una matrice di nmaxcolonne colonne e nrighe righe 
	if(nrighe==0)
		return;
	
	Bats=new Battuta* [nmaxcolonne];
	
	Sims=new Gelemento* [nrighe];
	for (i=0; i<nrighe; i++)
	{
		Sims[i]=new Gelemento[nmaxcolonne];
	}
	// inizializzo i valori della tabella
	for (j=0; j<nmaxcolonne; j++)
	{
		Bats[j]=0;   // puntatore alla battuta inserita in quella colonna 
	}
	for (i=0; i<nrighe; i++)
		for (j=0; j<nmaxcolonne; j++)
		{
			Sims[i][j].beat=-2;  // per distinguere i layer vuoti dalle caselle NOP
			Sims[i][j].fig=NULL;
			Sims[i][j].durata=-1;
			Sims[i][j].space=0; 
			Sims[i][j].fraz=0.; 
			Sims[i][j].measure=NULL; 
			Sims[i][j].layer=0; 
			Sims[i][j].end=0; 
			Sims[i][j].woutduration=0; 
			
			
		}   
		
}  

void Giustificazione::Inserimento(Battuta *pbat)
{
	// inserisco una nuova battuta nella tabella
	ListaFigure *pLayer;
	
	den=float(pbat->GetTempo().GetDenominatore());
	num=float(pbat->GetTempo().GetNumeratore());
	
	for(int l=0; l<NUM_MAX_LAYER; l++)
	{
		pLayer=pbat->GetPLayer(l);
		if(pLayer==NULL)
			continue;
		
		Figura *fig=pLayer->GetFirstFig();
		int i=0;
		while(fig!=NULL)
		{
			
			while (fig!=NULL && fig->GetID()==CL_SPAZIO)
			{
				fig=pLayer->GetNextFig();
				modificareprimospazio=NO;
			}
			if(fig!=NULL) {
				Sims[i][ncolonne].fig=fig;
                // per debug
				if ( Sims[i][ncolonne].fig->GetStatus()==GRACED || Sims[i][ncolonne].fig->GetStatus()==GHOSTED )  
					Sims[i][ncolonne].durata=0;
				else Sims[i][ncolonne].durata=Sims[i][ncolonne].fig->GetDurata(den, num);
                Sims[i][ncolonne].measure=pbat;
                Sims[i][ncolonne].layer=l;
            }
			i++;
			fig=pLayer->GetNextFig();
		}
		
		
		if(i!=0)
		{
			Bats[ncolonne]=pbat; 
			ncolonne++;
		}
	}
	
}    

void Giustificazione::Sincronizza()  
{
	
	// sulle righe ci stanno i layer
	// sulle colonne ci stanno le simultaneita'	
	
	// sincronizza la struttura: inserisci dove necessario caselle NOP 
	//                           aggiorna il campo beat 
	int i=0, it=0, j=0, risalgo=0; // i=righe j=colonne risalgo=riga
	float NextBeat=0.;
	float Beatline=0.;
	UL figsdspace=1;
	UL figsdspacetmp=0;
	
	if (Sims==NULL || ncolonne==0) return;
	
	
	i=0;
	// SENZA DURATA
	// Visito orizzontalmente la struttura e se ci sono delle figure SENZA DURATA
	// aggiungo nop su tutta la linea meno che li',e setto tutti i
	// campi space e beatline e vado alla riga successiva
	// sono all'inizio della struttura
	while (figsdspace!=0)  {
		figsdspace=0;
		for (j=0; j<ncolonne; j++) {
			// cerca se ci sono figure senza durata e prendi la spaziatura
			// massima
			if (Sims[i][j].fig!=NULL)
				if (Sims[i][j].durata==0 )  {
					figsdspacetmp=int(VU2UL(Sims[i][j].fig->GetWidth())*1.2);
					if (figsdspacetmp>figsdspace) figsdspace=figsdspacetmp;    
				}
		}
		if (figsdspace!=0)  {
			// scorri la struttura orizzontalmente 
			// mettendo nop per le figure con durata
			// e mettendo space=figsdspace su tutta la riga  
			for (j=0; j<ncolonne; j++) {
				if (Sims[i][j].fig!=NULL)  
					if (Sims[i][j].durata==0)  {
						Sims[i][j].space=figsdspace;
						Sims[i][j].woutduration=1;
						Sims[i][j].beat=Beatline;  // 0.
						Sims[i][j].fraz=1.;
					}
					else  {
						// scorri
						risalgo=nrighe-2;
						while (risalgo>=i) {
							Sims[risalgo+1][j].fig=Sims[risalgo][j].fig;
							Sims[risalgo+1][j].durata=Sims[risalgo][j].durata;
							risalgo--;
						}  
						// inserisco NOP
						Sims[i][j].fig=NULL;
                        Sims[i][j].durata=-1;
						Sims[i][j].space=figsdspace;
						Sims[i][j].beat=Beatline;
						Sims[i][j].fraz=1.;						
					}          
			}
			i++;
		}    
	}
	
	// inizializzo il NextBeat
	for (j=0; j<ncolonne; j++) {   
		
		if (Sims[i][j].fig!=NULL)  
			NextBeat=Sims[i][j].durata;
		
        break;
	}
	
	
	
	Sims[i][0].beat=0;
	// cerco il minimo (NextBeat) sulla prima riga 
	// e inizializzo il primo beatline a 0
	
	for (j=0; j<ncolonne; j++) {   
		if (Sims[i][j].fig!=NULL)
			
			if (Sims[i][j].durata<NextBeat) NextBeat=Sims[i][j].durata;
			// il beatline della prima riga e' 0
			Sims[i][j].beat=0;
			
	}
	
	
	it=i;
	if (it==0) it=1;
	
	// Visita la struttura verticalmente
	for (i=it; i<nrighe; i++)  {
		// setto il beatline corrente
		Beatline=NextBeat;
		
		// 
		// SENZA DURATA
		// Visito orizzontalmente la struttura e se ci sono delle figure SENZA DURATA
		// aggiungo nop su tutta la linea meno che li',e setto tutti i
		// campi space e beatline e vado alla riga successiva
		figsdspace=1;
		while (figsdspace!=0)  {
			figsdspace=0;
			for (j=0; j<ncolonne; j++) {
				// cerca se ci sono figure senza durata e prendi la spaziatura
				// massima
				if (Sims[i][j].fig!=NULL)
					if ( Sims[i][j].durata==0 )  {
						
						figsdspacetmp=int(VU2UL(Sims[i][j].fig->GetWidth())*1.2);
						if (figsdspacetmp>figsdspace) {
							figsdspace=figsdspacetmp;
						}
					}
			}
			if (figsdspace!=0)  {
				// scorri la struttura orizzontalmente 
				// mettendo nop per le figure con durata
				// e mettendo space=figsdspace su tutta la riga  
				for (j=0; j<ncolonne; j++) {
					if (Sims[i][j].fig!=NULL && (Sims[i][j].durata==0)) {  
						Sims[i][j].space=figsdspace;
						
						Sims[i][j].woutduration=1;
						Sims[i][j].beat=Beatline;
						Sims[i][j].fraz=1.;
						
					}
					else  {
						// scorri
						risalgo=nrighe-2;
						while (risalgo>=i) {
							Sims[risalgo+1][j].fig=Sims[risalgo][j].fig;
							Sims[risalgo+1][j].durata=Sims[risalgo][j].durata;
							risalgo--;
						}  
						// inserisco NOP
						Sims[i][j].fig=NULL;
						Sims[i][j].durata=-1;
						Sims[i][j].space=figsdspace;
						Sims[i][j].beat=Beatline;
						Sims[i][j].fraz=1.;
						
					}          
				}
				i++;
			}    
		}  
		
		// Visito orizzontalmente la struttura aggiusto i NOP sul beatline corrente
		int consistente, m, n, fine; 
		for (j=0; j<ncolonne; j++) {
			
			// cerco la prima figura prima una figura prima?
			risalgo=i-1;
			while (Sims[risalgo][j].fig==NULL || 
				(Sims[risalgo][j].fig!=NULL && Sims[risalgo][j].durata==0 ) ) 
			{
				if (risalgo>0)
					risalgo--;
				else break;
			}
			
			if (Sims[risalgo][j].fig!=NULL) {
				consistente=0;
				m=0, n=0;
				fine=0;
				
				for (m=0; m<ncolonne; m++) {
					fine=0;						
					if (m!=j) {
						for (n=i; n<nrighe; n++) {
							if (Sims[n][m].fig!=NULL)
							{  consistente=1;
							fine=1;
							
							}
							if (1==consistente)
								break;
						}
					}
				}
				
				
				if ((fabs((Sims[risalgo][j].beat+Sims[risalgo][j].durata)-Beatline)<0.001)||(consistente==0)) 
					Sims[i][j].beat=Beatline;
				else if ((Sims[risalgo][j].beat+Sims[risalgo][j].durata)>Beatline) 
				{   
					risalgo=nrighe-2;
					while (risalgo>=i) {
						Sims[risalgo+1][j].fig=Sims[risalgo][j].fig;
						Sims[risalgo+1][j].durata=Sims[risalgo][j].durata;
						
						Sims[risalgo+1][j].beat=Sims[risalgo][j].beat;
						Sims[risalgo+1][j].space=Sims[risalgo][j].space;
						Sims[risalgo+1][j].end=Sims[risalgo][j].end;
						Sims[risalgo+1][j].woutduration=Sims[risalgo][j].woutduration;
						
						risalgo--;
					}  
					// inserisco NOP
					Sims[i][j].fig=NULL;
					Sims[i][j].durata=-1;
					
					Sims[i][j].beat=Beatline;
					Sims[i][j].space=0;
					Sims[i][j].end=0;
					Sims[i][j].woutduration=0;					
				}
			}
			
		}
		
		
        int paperino=0;
		for (int pluto=0; pluto<ncolonne; pluto++) { 
			
			int pippo=0, fine=0;
			
			risalgo=i;
			// se in questa riga c'e'una figura senza durata...
			while (Sims[risalgo][pluto].fig==NULL || 
				(Sims[risalgo][pluto].fig!=NULL && Sims[risalgo][pluto].durata==0 )) 
			{
				if (risalgo>0) 
					risalgo--;
				else break;
				
			}
			
			fine=0;	
			for (pippo=risalgo+1; pippo<nrighe; pippo++) {
				if (Sims[pippo][pluto].fig!=NULL)
				{
					fine=1;
					break;
				}
				
			}
			if (1==fine)  
			{
				if (Sims[risalgo][pluto].fig!=NULL && Sims[risalgo][pluto].durata!=0) {
					NextBeat=float(Sims[risalgo][pluto].beat+Sims[risalgo][pluto].durata);
					paperino=1;
				}
				break;
				
			}
			
		}
		
		
		// sono alla frutta, lo inizializzo comunque sulla prima voce
        if (0==paperino)
		{
			// inizializzo comunque sulla prima voce
			// lo inizializzo sulla prima colonna
			risalgo=i;
			// se in questa riga c'e'una figura senza durata...
			while (Sims[risalgo][0].fig==NULL || 
				(Sims[risalgo][0].fig!=NULL && Sims[risalgo][0].durata==0 )) 
			{
				if (risalgo>0) 
					risalgo--;
				else break;
				
			}
			
			if (Sims[risalgo][0].fig!=NULL && Sims[risalgo][0].durata!=0)
				NextBeat=float(Sims[risalgo][0].beat+Sims[risalgo][0].durata);
			
		}
		
		for (j=0; j<ncolonne; j++) { 
			risalgo=i;
			
			while ((Sims[risalgo][j].fig==NULL || 
				(Sims[risalgo][j].fig!=NULL && Sims[risalgo][j].durata==0 ))) 
			{
				if (risalgo>0) 
					risalgo--; 
				else 
					break;
			}
			
			
			int pippo=0, fine=0;
			
			for (pippo=risalgo+1; pippo<nrighe; pippo++) {
				if (Sims[pippo][j].fig!=NULL)
				{
					fine=1;
					break;
				}
				
			}
			if (0==fine)  
				for (pippo=risalgo; pippo<nrighe; pippo++)
					Sims[pippo][j].end=1;
				
				
				if (Sims[risalgo][j].fig!=NULL && Sims[risalgo][j].durata!=0) {
					double txy=Sims[risalgo][j].beat;
					double txx=Sims[risalgo][j].durata;
					if (((txy+txx)<NextBeat) && (Sims[risalgo][j].end!=1) )
						NextBeat=float(txy+txx);
				}	 
		}
		
		
	}											
	
	// scorri verticalmente
	// scorri orizzontalmente
	// se trovi una fsd scorri per tutte le fsd che trovi su
	// quella voce fino all'ultima fsd memorizza da dove parto
	// scorri le nop fino alla prossima fcd subito prima
	
	int pfsd, ffsd, psc, jtemp, flag, itemp;
	UL spacetemp;
	// scorri tutte le voci 
	for (j=0; j<ncolonne; j++) { 
		
		// controlla sulla voce se ci sono delle fsd
		i=0;
		it=0;
		while (i<nrighe)  {
			if  (Sims[i][j].fig!=NULL && 
				Sims[i][j].durata==0 )  {
				// ho trovato una fsd
				// fisso il punto di partenza delle fsd
				pfsd=i;
				// scorro le fsd
				while (i<nrighe && Sims[i][j].fig!=NULL && 
					Sims[i][j].durata==0 ) i++;
				// fisso l'ultima fsd
				ffsd=i-1;
				
				// scorro i nop dopo le fsd
				while (i<nrighe && Sims[i][j].fig==NULL) 
					i++;
				if (i==nrighe)  {
					// ultima
					risalgo=ffsd;
					flag=0;
					while (flag=0) {
						for (jtemp=0; jtemp<ncolonne; jtemp++) {
							if (Sims[risalgo][jtemp].fig!=NULL) 
								flag=1;
						}   
						if (flag=1) 
						{ risalgo++;
						flag=0;
						}
						else break;
					}   
					// punto dove inserire la fsd che e'anche lultima figura dellavoce
					psc=risalgo;  
					
				}
				// punto dove inserire la fsd
				else psc=i-1;
				
				// questi casi particolari ci devo stare attento 
				if ( ffsd<nrighe && psc>ffsd ) {
					
					// ripeti per le fsd a paertire dall'ultima indietro 
					for (itemp=ffsd; itemp>=pfsd; itemp--)  {
						// scorri le colonne
						// guarda se in j=punto di scorrimento 
						// c' una fsd in tal caso permuta semplicemente 
						// il campo fig e cambia il campo space nel caso
						// che sia maggiore
						flag=0;
						spacetemp=0;
						for (jtemp=0; jtemp<ncolonne; jtemp++) {
							if (Sims[psc][jtemp].fig!=NULL && 
								Sims[psc][jtemp].durata==0) {
								if (spacetemp<Sims[psc][jtemp].space)
									spacetemp=Sims[psc][jtemp].space;
								flag=1;
							}
						}
						// c'era una fsd in psc
						if (flag==1)  {
							Sims[psc][j].fig=Sims[itemp][j].fig;
							Sims[itemp][j].fig=NULL;
							for (jtemp=0; jtemp<ncolonne; jtemp++) {
								Sims[psc][jtemp].space=spacetemp;
							}
							
							psc--;
						}
						// altrimenti inserisci nop dopo psc e setta fig di fsd
						// e lo space di tutto il beatline
						else  {
							// scorri in giu'
							for (jtemp=0; jtemp<ncolonne; jtemp++) {
								risalgo=nrighe-2;
								while (risalgo>psc) {
									Sims[risalgo+1][jtemp].fig=Sims[risalgo][jtemp].fig;
									Sims[risalgo+1][jtemp].durata=Sims[risalgo][jtemp].durata;
									
									Sims[risalgo+1][jtemp].beat=Sims[risalgo][jtemp].beat;
									Sims[risalgo+1][jtemp].space=Sims[risalgo][jtemp].space;
									Sims[risalgo+1][jtemp].woutduration=Sims[risalgo][jtemp].woutduration;
									risalgo--;
								}  
								// inserisco NOP
								Sims[psc+1][jtemp].fig=NULL;
								Sims[psc+1][jtemp].space=0;
								Sims[psc+1][jtemp].woutduration=0;
								
							}
							// setta la fsd in psc+1
							Sims[psc+1][j].fig=Sims[itemp][j].fig;
							Sims[psc+1][j].durata=Sims[itemp][j].durata;
							
							Sims[itemp][j].fig=NULL;
                            Sims[itemp][j].durata=-1;
							
							spacetemp=Sims[itemp][j].space;
							for (jtemp=0; jtemp<ncolonne; jtemp++) {
								Sims[psc+1][jtemp].space=spacetemp;
							}
						}
					}  
				}
				
				// chiudo l'if ho trovato una fsd
			}
			// se non trovo una fsd
			else i++;
			// chiudo while i< nrighe
    }
	// chiudo for j < ncolonne
  }
  
  
  
  // adesso devo eliminare le colonne vuote
  for (i=0; i<nrighe; i++) { 
	  flag=0;
	  // guarda se c'e'una colonna di nop
	  for (j=0; j<ncolonne; j++) { 
		  if (Sims[i][j].fig!=NULL) flag=1;
	  }
	  if (flag==0)  {
		  // tira su tutto quello che c'e' dopo i di 1
		  for (j=0; j<ncolonne; j++) { 
			  risalgo=i;
			  while (risalgo<nrighe-1) {
				  Sims[risalgo][j].fig=Sims[risalgo+1][j].fig;
				  Sims[risalgo][j].durata=Sims[risalgo+1][j].durata;
				  
				  Sims[risalgo][j].beat=Sims[risalgo+1][j].beat;
				  Sims[risalgo][j].space=Sims[risalgo+1][j].space;
				  Sims[risalgo][j].woutduration=Sims[risalgo+1][j].woutduration;
				  risalgo++;
			  }  
		  }         
	  }
  }
  
}

void Giustificazione::CleanTable()  
{
	int i=0, j=0, nonpulisci=0;
	for (i=0; i<nrighe; i++) { 
		nonpulisci=0;
		for (j=0; j<ncolonne; j++) { 
			if (NULL!=Sims[i][j].fig && 0==Sims[i][j].durata)
				
				nonpulisci=1;
		}
		if (0==nonpulisci){
			for (j=0; j<ncolonne; j++) { 
				Sims[i][j].space=0;
				Sims[i][j].fraz=0;
			}
		}
		
		
	}
	
}


void Giustificazione::dump()  

{
	// passo il puntatore alla tabella al metodo dump che la stampa su file
	FILE *pFileDati;
	int controllo=0, i=0, j=0;
	// Tabella= durata (-1 se null); beat corrente; spazio fino alla prossima; fi
	
	if ((pFileDati=fopen ("dellasantabasta.txt","w"))==NULL)
		printf ("Errore nell'apertura del file tabsim.dat\n");
	else {
		
		fprintf(pFileDati,"\nTabella righe=%d colonne=%d:",nrighe,ncolonne);
		for (i=0; i<nrighe; i++) {
			fprintf(pFileDati,"\n[durata, beatline, spazio, fraz, end, woutduration, SpG]\n");
			for (j=0; j<ncolonne; j++) {
				fprintf(pFileDati,"[ %f,  %f,  %d,  %f, %d, %d, %d] ",Sims[i][j].fig==NULL? -1
					:Sims[i][j].durata, Sims[i][j].beat, Sims[i][j].space, Sims[i][j].fraz, Sims[i][j].end, Sims[i][j].woutduration, Sims[i][j].fig==NULL ? -1 : Sims[i][j].fig->GetSpG());
			}
			fprintf(pFileDati,"\n");
		}
		controllo=fclose (pFileDati);
	}
}

void Giustificazione::dumpclean()  

{
	// passo il puntatore alla tabella al metodo dump che la stampa su file
	FILE *pFileDati;
	int controllo=0, i=0, j=0;
	// Tabella= durata (-1 se null); beat corrente; spazio fino alla prossima; fi
	
	if ((pFileDati=fopen ("conferenzadumpclean.txt","w"))==NULL)
		printf ("Errore nell'apertura del file tabsim.dat\n");
	else {	
		
		fprintf(pFileDati,"\nTabella righe=%d colonne=%d:",nrighe,ncolonne);
		for (i=0; i<nrighe; i++) {
			fprintf(pFileDati,"\n[durata, beatline, spazio, fraz, end, woutduration, SpDLine]\n");
			for (j=0; j<ncolonne; j++) {
				fprintf(pFileDati,"[ %f,  %f,  %d,  %f, %d, %d, %d] ",Sims[i][j].fig==NULL? -1
					:Sims[i][j].fig->GetDurata(den, num), Sims[i][j].beat, Sims[i][j].space, Sims[i][j].fraz, Sims[i][j].end, Sims[i][j].woutduration, Sims[i][j].fig==NULL ? -1 : Sims[i][j].fig->GetSpDLine());
			}
			fprintf(pFileDati,"\n");
		}
		controllo=fclose (pFileDati);
	}
}


void Giustificazione::GiustificaLog(double kGiust)  

{
	// scorre la struttura creata e setta i campi space di ogni nodo
	// (spazi UL fino al beatline successivo)
	int i=0, j=0, risalgo=0, it=0; // i=righe j=colonne risalgo=riga
	int p=0, z=0, totw=0, y=0, d=0;
	UL figsdspace=0;
	UL figsdspacetmp=0;
	double totfi=0., tm=0.;
	double m=1./8.;   
	double k=0., k1=0.;
	double Beatline=0.;
	double dmin=0;
	double fi;
	double width;
	int flag=0;
	double log2=log(2.0);
	
	
	
	
	if(ncolonne==0) return;
	
	
	// cerco la durata minore della battuta
	for (i=0; i<nrighe; i++) {
		for (j=0; j<ncolonne; j++) {
			if (Sims[i][j].fig!=NULL) 
				if ((Sims[i][j].durata!=0) && 
					(Sims[i][j].durata<m))  
					m=Sims[i][j].durata;
		}
	}
	
	k=kGiust-(log(m)/log2);
	
	//////////////////////////////////////////////
	// scorri le simultaneita' della battuta,  
	// Per ogni simultaneita' trova di, la durata inferiore tra le note
	// inizianti o continuanti nell'istante della simultaneita'. Tale di
	// definisce lo spazio usato da tutte le note simultanee.
	// Calcola fi, la frazione della nota piu' corta che ci sara' fino
	// alla prossima sim, calcolata con la formula 
	// fi=(beat successivo-beat corrente)/durata minore
	//////////////////////////////////////////////
	
	// se e'una riga dove c'una figura senza durata
	// incrementa i
	
	i=0;
	while (Sims[i][0].space!=0) i++;
	it=i;	  
	
	for (i=it; i<nrighe; i++) {   // le righe della struttura identificano
		// le simultaneita'
		
		// se e'una riga dove c'una figura senza durata
		// incrementa i
		// ok
		while (Sims[i][0].space!=0) i++;
		
		// cerca di della simultaneita'
		// la inizializza sulla prima colonna
		risalgo=i;
		
		while (risalgo>0 && (Sims[risalgo][0].fig==NULL || 
			(Sims[risalgo][0].fig!=NULL && Sims[risalgo][0].durata==0 ))) risalgo--;
		if (Sims[risalgo][0].fig!=NULL)
			dmin=Sims[risalgo][0].durata;
		// battute inconsistenti
		else dmin=0;
		
		if (i<nrighe-1) 
			if (dmin!=0)
				fi=(Sims[i+1][0].beat-Sims[i][0].beat)/dmin;
			else fi=1;
			for (j=1; j<ncolonne; j++) {
				risalgo=i;
				while (risalgo>0 && (Sims[risalgo][j].fig==NULL || 
					(Sims[risalgo][j].fig!=NULL && Sims[risalgo][j].durata==0 ))) {
					risalgo--;
				}
				
				if (Sims[risalgo][j].fig!=NULL)
					if (((Sims[risalgo][j].durata<dmin) && (Sims[risalgo][j].end!=1)) 
						|| (dmin==0))  {
						dmin=Sims[risalgo][j].durata; 
						if (i<nrighe-1) 
							if (dmin!=0)
								fi=(Sims[i+1][j].beat-Sims[i][j].beat)/dmin; 
							else fi=1;
					}	
			}     
			// calcola larghezza ideale della sim: wi=fi(log2(di)+k)
			// log2(x)=ln(x)/ln(2)  
			// 720 fattore di conversione da nhw a UL
			if(i==nrighe-1)  {
				fi=0;
				for(j=0;j<ncolonne;j++)
					if(Sims[i][j].fig!=NULL) fi=1;
			}
			
			flag=0;
			for(j=0;j<ncolonne;j++) {
				// se c'e'qualcosa flag =1
				if (Sims[i][j].fig!=NULL) 
					flag=1;
				
			}
			
			// e' l'ultima figura della battuta?
			int sonoinfondo=1;
			for(j=0;j<ncolonne;j++) {
				// se non sono in fondo
				if (Sims[i][j].end==0) 
					sonoinfondo=0;
				
			}
			
			// per battute inconsistenti
			if (flag==1)
				if (fi==0)
					fi=1;
				
				if (fi<0)
					fi=1;
				if (fi>1)
					fi=1;
				
				// setto il campo fraz
				for(j=0;j<ncolonne;j++) {
					Sims[i][j].fraz=fi;
				}
				
				
				width=(fi*(log(dmin)/log2+k)*720.0);
				
				
				for (j=0; j<ncolonne; j++)
					if (Sims[i][j].fig!=NULL)
						if (((Sims[i][j].fig->GetVU2Rgt()+DIST_MIN)*100)>width)
							width=(Sims[i][j].fig->GetVU2Rgt()+DIST_MIN)*100;
						
						
						
						// aggiorna tutti i campi space della sim corrente col valore di width
						for (j=0; j<ncolonne; j++) Sims[i][j].space=(int)width;
						
						
	} 
	///////////////////////////////////////////////
	// Inserisco i controlli sulla larghezza di blocco
	// la struttura e' gia' sincronizzata
	
	
	// bug delle chiavine sovrapposte
	int senzadurata=0;
	
	for(i=1;i<nrighe;i++) {
		senzadurata=0;
		for(j=0;j<ncolonne;j++)  {
			
			// se e' una figura senza durata senzadurata =1
			if (Sims[i][j].fig!=NULL && Sims[i][j].durata==0) 
				senzadurata=1;
			
		}
		if (senzadurata==1){
			for(j=0;j<ncolonne;j++)  
				Sims[i][j].fraz=1;
		}
		
	}
	
	
	for(i=1;i<nrighe;i++) {
		for(j=0;j<ncolonne;j++)  {
			if (Sims[i][j].fig!=NULL)  { 
				p=i-1;
				while (p>=0 && Sims[p][j].fig==NULL) p--;
				// *100 perche' VU2UL
				if (p>=0) { 
					
					z=(Sims[p][j].fig->GetVU2Rgt()+Sims[i][j].fig->GetVU2Lft()+1)*100;
					
					totw=0;
					totfi=0;
					for(y=p;y<=(i-1);y++) {
						totw+=Sims[y][j].space;
						totfi+=Sims[y][j].fraz;
					}
					k1=(z-totw);
					if (k1>0 && totfi!=0) {
						for (y=p;y<=(i-1);y++) {
							tm=(k1*Sims[y][j].fraz)/totfi;
							for (d=0;d<ncolonne;d++) Sims[y][d].space+=int(tm);
						}
					}  
					
				}
			}	
		}
	}	
}


void Giustificazione::SetUpSpG(bool clearSpUt)
{
	// la struttura Sims per la battuta e' stata creata e giustificata
	// adesso si tratta di andare a settare il parametro SpG per ogni
	// figura della battuta
	int i=0, j=0, scendo=0; // i=righe j=colonne scendo=riga
	UL spazio;
	
	if (Sims==NULL) return;
	
	// trattazione dello spazio fino alla prima figura
	// caso di acciaccature  fare il conto per settare 
	// anche questi parametri (di battuta)
	
	//resetto i valori
	for (j=0; j<ncolonne; j++) {
		
        // controlli perche' se e' giustificata al caricamento
		// si deve ricordare anche gli spazi utente
		if(Sims[0][j].measure!=NULL)
		{
			if (clearSpUt) //YES==Sims[0][j].measure->GetEverJustified(DIR, currentViewType))
			{
				if (YES==modificareprimospazio)
					Sims[0][j].measure->SetDirSp2FirstFigUt(Sims[0][j].layer,0);
			}		
			
			if (YES==modificareprimospazio)
				Sims[0][j].measure->SetDirSp2FirstFig(Sims[0][j].layer,0);
		}
	}
    
	int spaziotmp=0;
	//////////////////////////////////////
    // guardo la sx di ogni prima figura
    for (j=0; j<ncolonne; j++) {
		// accidentals
		if (Sims[0][j].fig!=NULL) {
			if ((Sims[0][j].fig->GetVU2Lft()-2)*100>spaziotmp) 
				spaziotmp = (Sims[0][j].fig->GetVU2Lft()-2)*100;  // 100/
		}
	}

	/////////////////////
	
	for (j=0; j<ncolonne; j++) {
		// spazio fino alla prima figura (di battuta)
		i=0;
		
        // prime sono notine   		
		while ((i<nrighe) && (Sims[i][j].fig==NULL))  {
			spazio=Sims[i][j].space;
			scendo=i+1;
			while ((scendo<nrighe) && (Sims[scendo][j].fig==NULL)) {
				spazio+=Sims[scendo][j].space;
				scendo++;
			}

			// setto lo spazio fino alla prima figura
			if(Sims[0][j].measure!=NULL)
				Sims[0][j].measure->SetDirSp2FirstFig(Sims[0][j].layer,spazio);
			i=scendo;
		}
		
		// se le prime sono notine ma quello dopo prende piu'spazio (es accordo + accidentals)
		if ((i<nrighe) && (NULL!=Sims[i][j].fig))
		{
			if ((Sims[i][j].fig->GetVU2Lft()-2)*100>spaziotmp) 
				spaziotmp = (Sims[i][j].fig->GetVU2Lft()-2)*100;
		}
						
		// scorro tutta la lista
		while ((i<nrighe) && (Sims[i][j].fig!=NULL))  {
			// scorri tutte le voci e calcola per ogni figura
			spazio=Sims[i][j].space;
			scendo=i+1;
			while ((scendo<nrighe) && (Sims[scendo][j].fig==NULL)) {
				spazio+=Sims[scendo][j].space;
				scendo++;
			}
			Sims[i][j].fig->SetSpG(spazio);
			
			if (clearSpUt) //YES==Bats[j]->GetEverJustified(DIR, currentViewType))
				Sims[i][j].fig->SetSpDirUt(0);				
			i=scendo;
		}
	}
	
    int passaggio=0;
    for (j=0; j<ncolonne; j++) {
		// spazio fino alla prima figura (di battuta)
		// aggiungo quello che ho calcolato all'inizio
		i=0;
		
		if(Sims[0][j].measure!=NULL)
		{
			passaggio=Sims[0][j].measure->GetDirSp2FirstFig(Sims[0][j].layer);
			Sims[0][j].measure->SetDirSp2FirstFig(Sims[0][j].layer,spaziotmp+passaggio);
		}			
	}
}


void Giustificazione::SetUpSpDLine()
{
	// la struttura Sims per la battuta e' stata creata e giustificata
	// adesso si tratta di andare a settare il parametro SpDLine per ogni
	// figura della battuta
	int i=0, j=0, scendo=0; // i=righe j=colonne scendo=riga
	UL spazio;
	
	if (Sims==NULL) return;
	
	////////////////////////////////////
	
	// trattazione dello spazio fino alla prima figura
	// caso di acciaccature  fare il conto per settare 
	// anche questi parametri (di battuta)
	
	//resetto i valori
	for (j=0; j<ncolonne; j++) {
		
        // controlli perche' se e' giustificata al caricamento
		// si deve ricordare anche gli spazi utente
		if(Sims[0][j].measure!=NULL)
			Sims[0][j].measure->SetDirSp2FirstFigLine(Sims[0][j].layer,0);			
	}
	
	int spaziotmp=0;
    // guardo la sx di ogni prima figura
    for (j=0; j<ncolonne; j++) {
		
		// accidentals
		if (Sims[0][j].fig!=NULL) { 
			
			if ((Sims[0][j].fig->GetVU2Lft()-2)*100>spaziotmp) 
				spaziotmp = (Sims[0][j].fig->GetVU2Lft()-2)*100;  // 100/2
			
		} 
		
	}
	
    for (j=0; j<ncolonne; j++) {
		// spazio fino alla prima figura (di battuta)
		i=0;
		
        // prime sono notine   		
		while ((i<nrighe) && (Sims[i][j].fig==NULL))  {
			spazio=Sims[i][j].space;
			scendo=i+1;
			while ((scendo<nrighe) && (Sims[scendo][j].fig==NULL)) {
				spazio+=Sims[scendo][j].space;
				scendo++;
			}
					
			// setto lo spazio fino alla prima figura
			if(Sims[0][j].measure!=NULL)
				Sims[0][j].measure->SetDirSp2FirstFigLine(Sims[0][j].layer,spazio);
			
			i=scendo;
		}
		
		// se le prime sono notine ma quello dopo prende piu'spazio (es accordo + accidentals)
		if ((i<nrighe) && (NULL!=Sims[i][j].fig))
			if ((Sims[i][j].fig->GetVU2Lft()-2)*100>spaziotmp) 
				spaziotmp = (Sims[i][j].fig->GetVU2Lft()-2)*100;
						
			// scorro tutta la lista
			while ((i<nrighe) && (Sims[i][j].fig!=NULL))  {
				// scorri tutte le voci e calcola per ogni figura
				spazio=Sims[i][j].space;
				scendo=i+1;
				while ((scendo<nrighe) && (Sims[scendo][j].fig==NULL)) {
					spazio+=Sims[scendo][j].space;
					scendo++;
				}
				Sims[i][j].fig->SetSpDLine(spazio);	
				i=scendo;
			}
	}
	
    int passaggio=0;
    for (j=0; j<ncolonne; j++) {
		// spazio fino alla prima figura (di battuta)
		// aggiungo quello che ho calcolato all'inizio
		i=0;
		
		if(Sims[0][j].measure!=NULL)
		{
			passaggio=Sims[0][j].measure->GetDirSp2FirstFigLine(Sims[0][j].layer);
			Sims[0][j].measure->SetDirSp2FirstFigLine(Sims[0][j].layer,spaziotmp+passaggio);
		}
	}
}

void Giustificazione::SetUpSpOLine()
{
	// la struttura Sims per la battuta e' stata creata e giustificata
	// adesso si tratta di andare a settare il parametro SpOLine per ogni
	// figura della battuta
	int i=0, j=0, scendo=0; // i=righe j=colonne scendo=riga
	UL spazio;
				
	if (Sims==NULL) return;
				
	//////////////////////////////////////////////
	// trattazione dello spazio fino alla prima figura
	// caso di acciaccature  fare il conto per settare 
	// anche questi parametri (di battuta)
				
	//resetto i valori
	for (j=0; j<ncolonne; j++) {		
		// controlli perche' se e' giustificata al caricamento
		// si deve ricordare anche gli spazi utente
					
		if(Sims[0][j].measure!=NULL)
			Sims[0][j].measure->SetOrcSp2FirstFigLine(Sims[0][j].layer,0);
	}
				
	int spaziotmp=0;
	// guardo la sx di ogni prima figura
	for (j=0; j<ncolonne; j++) {
		// accidentals
		if (Sims[0][j].fig!=NULL) { 						
			if ((Sims[0][j].fig->GetVU2Lft()-2)*100>spaziotmp) 
				spaziotmp = (Sims[0][j].fig->GetVU2Lft()-2)*100;  // 100/2				
		} 					
	}
		
	for (j=0; j<ncolonne; j++) {
		// spazio fino alla prima figura (di battuta)
		i=0;
					
		// prime sono notine   		
		while ((i<nrighe) && (Sims[i][j].fig==NULL))  {
			spazio=Sims[i][j].space;
			scendo=i+1;
			while ((scendo<nrighe) && (Sims[scendo][j].fig==NULL)) {
				spazio+=Sims[scendo][j].space;
				scendo++;
			}
						
			// setto lo spazio fino alla prima figura
			if(Sims[0][j].measure!=NULL)
				Sims[0][j].measure->SetOrcSp2FirstFigLine(Sims[0][j].layer,spazio);
						
			i=scendo;
		}
					
		// se le prime sono notine ma quello dopo prende piu'spazio (es accordo + accidentals)
		if ((i<nrighe) && (NULL!=Sims[i][j].fig))
			if ((Sims[i][j].fig->GetVU2Lft()-2)*100>spaziotmp) 
				spaziotmp = (Sims[i][j].fig->GetVU2Lft()-2)*100;
						
		// scorro tutta la lista
		while ((i<nrighe) && (Sims[i][j].fig!=NULL))  {
			// scorri tutte le voci e calcola per ogni figura
			spazio=Sims[i][j].space;
			scendo=i+1;
			while ((scendo<nrighe) && (Sims[scendo][j].fig==NULL)) {
				spazio+=Sims[scendo][j].space;
				scendo++;
			}
			Sims[i][j].fig->SetSpOLine(spazio);
			i=scendo;
		}
	}
				
	int passaggio=0;
	for (j=0; j<ncolonne; j++) {
		// spazio fino alla prima figura (di battuta)
		// aggiungo quello che ho calcolato all'inizio
		i=0;		
		if(Sims[0][j].measure!=NULL)
		{	   
			passaggio=Sims[0][j].measure->GetOrcSp2FirstFigLine(Sims[0][j].layer);
			Sims[0][j].measure->SetOrcSp2FirstFigLine(Sims[0][j].layer,spaziotmp+passaggio);
		}						
	}
}


// per ora questa non e'utilizzata
// verra'chiamata quando sara'l'orchestrale a
// chiamare la giustificazione
// DA RIGUARDARE
void Giustificazione::SetUpSpOr(bool clearSpUt)
{
	// la struttura Sims per la battuta e' stata creata e giustificata
	// adesso si tratta di andare a settare il parametro SpOr per ogni
	// figura della battuta
	int i=0, j=0, scendo=0; // i=righe j=colonne scendo=riga
	UL spazio;
	
	if (Sims==NULL) return;
	
	
    
	// trattazione dello spazio fino alla prima figura
	// caso di acciaccature  fare il conto per settare 
	// anche questi parametri (di battuta)
	
	//resetto i valori
	for (j=0; j<ncolonne; j++) {
		
        // controlli perche' se e' giustificata al caricamento
		// si deve ricordare anche gli spazi utente
		if(Sims[0][j].measure!=NULL)
		{
			if (clearSpUt) //YES==Sims[0][j].measure->GetEverJustified(ORC, currentViewType))
			{
				Sims[0][j].measure->SetOrcSp2FirstFigUt(Sims[0][j].layer,0);
			}		
			
			Sims[0][j].measure->SetOrcSp2FirstFig(Sims[0][j].layer,0);
		}
	}

	int spaziotmp=0;
	//////////////////////////////////////
    // guardo la sx di ogni prima figura
    for (j=0; j<ncolonne; j++) {
		
		// accidentals
		if (Sims[0][j].fig!=NULL) { 
			
			if ((Sims[0][j].fig->GetVU2Lft()-2)*100>spaziotmp) 
				spaziotmp = (Sims[0][j].fig->GetVU2Lft()-2)*100;  // 100/2
		} 		
	}
	/////////////////////
	for (j=0; j<ncolonne; j++) {
		// spazio fino alla prima figura (di battuta)
		i=0;
		
        // prime sono notine   		
		while ((i<nrighe) && (Sims[i][j].fig==NULL))  {
			spazio=Sims[i][j].space;
			scendo=i+1;
			while ((scendo<nrighe) && (Sims[scendo][j].fig==NULL)) {
				spazio+=Sims[scendo][j].space;
				scendo++;
			}
			
			// setto lo spazio fino alla prima figura
			if(Sims[0][j].measure!=NULL)
			{
				Sims[0][j].measure->SetOrcSp2FirstFig(Sims[0][j].layer,spazio);
			}
			
			i=scendo;
		}
		
		
		// se le prime sono notine ma quello dopo prende piu'spazio (es accordo + accidentals)
		if ((i<nrighe) && (NULL!=Sims[i][j].fig))
			if ((Sims[i][j].fig->GetVU2Lft()-2)*100>spaziotmp) 
				spaziotmp = (Sims[i][j].fig->GetVU2Lft()-2)*100;
			
			
			// scorro tutta la lista
			while ((i<nrighe) && (Sims[i][j].fig!=NULL))  {
				// scorri tutte le voci e calcola per ogni figura
				spazio=Sims[i][j].space;
				scendo=i+1;
				while ((scendo<nrighe) && (Sims[scendo][j].fig==NULL)) {
					spazio+=Sims[scendo][j].space;
					scendo++;
				}
				Sims[i][j].fig->SetSpOr(spazio);
				
				if (clearSpUt) //YES==Bats[j]->GetEverJustified(ORC, currentViewType))
					Sims[i][j].fig->SetSpOrcUt(0);
				
				i=scendo;
			}
	}
	
    int passaggio=0;
    for (j=0; j<ncolonne; j++) {
		// spazio fino alla prima figura (di battuta)
		// aggiungo quello che ho calcolato all'inizio
		i=0;
		
		if(Sims[0][j].measure!=NULL)
		{
			passaggio=Sims[0][j].measure->GetOrcSp2FirstFig(Sims[0][j].layer);
			Sims[0][j].measure->SetOrcSp2FirstFig(Sims[0][j].layer,spaziotmp+passaggio);
		}			
	}
}


void Giustificazione::GiustificaLin(double kGiust)  
{
	// scorre la struttura creata e setta gli attributi di g. di ogni figura 
	// in base
	// alla sua durata
	// (spazi UL fino al beatline successivo)
	
	int i=0, j=0, risalgo=0, it=0; // i=righe j=colonne risalgo=riga
	int p=0, z=0, totw=0, y=0, d=0;
	UL figsdspace=0;
	UL figsdspacetmp=0;
	double totfi=0., tm=0.;
	double m=1./8.;   
	double fc=0., k1=0.;
	double Beatline=0.;
	double dmin=0;
	double fi;
	double width;
	int flag=0;
	
	if(ncolonne==0) return;
	
	// cerco la durata minore della battuta
	for (i=0; i<nrighe; i++) {
		for (j=0; j<ncolonne; j++) {
			if (Sims[i][j].fig!=NULL) 
				if ((Sims[i][j].durata!=0) && 
					(Sims[i][j].durata<m))  
					m=Sims[i][j].durata;
				
		}
	}
	
	
	// setto la spaziatura minima a seconda della durata minima nella battuta
	//  if (m==0.125) fc=12000;        // 1/8
	//  if (m==0.0625) fc=24000;       // 1/16
	//  if (m==0.03125) fc=48000;      // 1/32 
	//  if (m==0.015625) fc=96000;     // 1/64 
	//  if (m==0.0078125) fc=192000;   // 1/128
	fc=kGiust/(m*8); 
	// per passargli un parametro mettere kGiust al posto di 12000
	// 
	
	
	//////////////////////////////////////////////
	// scorri le simultaneita' della battuta,  
	// Per ogni simultaneita' trova di, la durata inferiore tra le note
	// inizianti o continuanti nell'istante della simultaneita'. Tale di
	// definisce lo spazio usato da tutte le note simultanee.
	// Calcola fi, la frazione della nota piu' corta che ci sara' fino
	// alla prossima sim, calcolata con la formula 
	// fi=(beat successivo-beat corrente)/durata minore
	//////////////////////////////////////////////
	
	// se e'una riga dove c'una figura senza durata
	// incrementa i
	
	i=0;
	while (Sims[i][0].space!=0) i++;
	it=i;	  
	
	
	for (i=it; i<nrighe; i++) {   // le righe della struttura identificano
		// le simultaneita'
		
		// se e'una riga dove c'una figura senza durata
		// incrementa i
		
		while (Sims[i][0].space!=0) i++;
		
		// cerca di della simultaneita'
		// la inizializza sulla prima colonna
		risalgo=i;
		while (risalgo>0 && (Sims[risalgo][0].fig==NULL || 
			(Sims[risalgo][0].fig!=NULL && Sims[risalgo][0].durata==0 ))) risalgo--;
		
		if (Sims[risalgo][0].fig!=NULL)
			dmin=Sims[risalgo][0].durata;
		else dmin=0;
		
		if (i<nrighe-1) 
			if (dmin!=0)
				fi=(Sims[i+1][0].beat-Sims[i][0].beat)/dmin; 
			else fi=1;
			
			for (j=1; j<ncolonne; j++) {
				risalgo=i;
				while (risalgo>0 && (Sims[risalgo][j].fig==NULL || 
					(Sims[risalgo][j].fig!=NULL && Sims[risalgo][j].durata==0 ))) 
					risalgo--;
				
				if (Sims[risalgo][j].fig!=NULL)
					if (((Sims[risalgo][j].durata<dmin)  && (Sims[risalgo][j].end!=1))
						|| (dmin==0)) {
						dmin=Sims[risalgo][j].durata; 
						if (i<nrighe-1) 
							if (dmin!=0)
								fi=(Sims[i+1][j].beat-Sims[i][j].beat)/dmin; 
							else fi=1;
					}	
			}     
			// 720 fattore di conversione da nhw a UL
			if(i==nrighe-1)
			{
				fi=0;
				for(j=0;j<ncolonne;j++)
					if(Sims[i][j].fig!=NULL) fi=1;
			}
			
			
			flag=0;
			for(j=0;j<ncolonne;j++) {
				if (Sims[i][j].fig!=NULL) flag=1;
			}
			
			// per battute inconsistenti
			if (flag==1)
				if (fi==0)
					fi=1;
				
				if (fi<0)
					fi=1;
				if (fi>1)
					fi=1;
				
				
				// setto il campo fraz
				for(j=0;j<ncolonne;j++) {
					Sims[i][j].fraz=fi;
				}
				
				
				width=(fi*dmin*fc);
				
				for (j=0; j<ncolonne; j++)
					if (Sims[i][j].fig!=NULL)
						if (((Sims[i][j].fig->GetVU2Rgt()+DIST_MIN)*100)>width)
							width=(Sims[i][j].fig->GetVU2Rgt()+DIST_MIN)*100;					
						
						// aggiorna tutti i campi space della sim corrente col valore di width
						for (j=0; j<ncolonne; j++) Sims[i][j].space=(int)width;
						
	}
	
	
	
	int senzadurata=0;
	
	for(i=1;i<nrighe;i++) {
		senzadurata=0;
		for(j=0;j<ncolonne;j++)  {
			
			// se e' una figura senza durata senzadurata =1
			if (Sims[i][j].fig!=NULL && Sims[i][j].durata==0) 
				senzadurata=1;
			
		}
		if (senzadurata==1){
			for(j=0;j<ncolonne;j++)  
				Sims[i][j].fraz=1;
		}
		
	}
	
	
	///////////////////////////////////////////////
	// Inserisco i controlli sulla larghezza di blocco
	// la struttura e' gia' sincronizzata
	for(i=1;i<nrighe;i++) {
		for(j=0;j<ncolonne;j++)  {
			if (Sims[i][j].fig!=NULL)  { 
				p=i-1;
				while (p>=0 && Sims[p][j].fig==NULL) p--;
				// *100 perche' VU2UL
				if (p>=0) { 
					z=(Sims[p][j].fig->GetVU2Rgt()+Sims[i][j].fig->GetVU2Lft()+1)*100;
					
					totw=0;
					totfi=0;
					for(y=p;y<=(i-1);y++) {
						totw+=Sims[y][j].space;
						totfi+=Sims[y][j].fraz;
					}
					k1=(z-totw);
					if (k1>0 && totfi!=0) {
						for (y=p;y<=(i-1);y++) {
							tm=(k1*Sims[y][j].fraz)/totfi;
							for (d=0;d<ncolonne;d++) Sims[y][d].space+=int(tm);
						}
					}  
				}
			}	
		}
	} 
}

