/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: intestaz.cpp,v 1.1 2005/07/25 10:51:09 bellinpo Exp $ */

#include "bat/intestaz.hpp" 

#include "bat/cviolino.hpp"
#include "bat/cviolin8.hpp"
#include "bat/cviolino8sopra.hpp"
#include "bat/cbasso.hpp"
#include "bat/cbasso8.hpp"
#include "bat/cbariton.hpp"
#include "bat/ctenore.hpp"
#include "bat/ctenore8.hpp"
#include "bat/ccontral.hpp"
#include "bat/cmezzoso.hpp"
#include "bat/csoprano.hpp"
//
#include "bat/cbasso8sopra.hpp"
#include "bat/ctab.hpp"
#include "bat/cpercusbox.hpp"
#include "bat/cpercus2lines.hpp"
#include "bat/cvuota.hpp"
#include "bat/cbassoold.hpp"
//

void Intestazione::SetVU2Up(VUnit vu) 
{INTESTAZIONE2Up=vu;};  

void Intestazione::SetVU2Dwn(VUnit vu) 
{INTESTAZIONE2Dwn=vu;};  

void Intestazione::SetVU2Lft(VUnit vu) 
{INTESTAZIONE2Lft=vu;};  

void Intestazione::SetVU2Rgt(VUnit vu) 
{INTESTAZIONE2Rgt=vu;};  

VUnit Intestazione::GetVU2Up() 
{return INTESTAZIONE2Up;};

VUnit Intestazione::GetVU2Dwn() 
{return INTESTAZIONE2Dwn;};

VUnit Intestazione::GetVU2Lft() 
{return INTESTAZIONE2Lft;};

VUnit Intestazione::GetVU2Rgt() 
{return INTESTAZIONE2Rgt;};

void Intestazione::SetChiave(Chiave *chv)
{ ptrChiave=chv; };

Chiave *Intestazione::GetChiave() 
{ return ptrChiave; };

void Intestazione::SetArmatura(tipoArmaturaChiave arm)
{ armChiave.SetArmatura(arm); };

tipoArmaturaChiave Intestazione::GetTipoArmatura() 
{   
    return armChiave.GetArmatura();
};

ArmaturaChiave * Intestazione::GetPtrArmatura()
{ return &armChiave; };

ArmaturaChiave Intestazione::GetArmatura() 
{ return armChiave;};


void Intestazione::SetTempo(char tmp[])
{ tempo.SetTempo(tmp); };

Tempo Intestazione::GetTempo() 
{ return tempo;};

Tempo * Intestazione::GetPtrTemp(void)
{ return &tempo; };



void Intestazione::SetVU(tipoBattuta tipoBat, Intestazione *intest)
{
	VUnit wdt=0;
	armChiave.shiftChiave = (ptrChiave->GetShift());
	armChiave.altezzaChiave = (ptrChiave->GetAltezza());
	if(intest != NULL)      
	{
		tipoArmaturaChiave tmpArmChiave = tipoArmaturaChiave(GetTipoArmatura() % 15);
		tipoArmaturaChiave tmpIntArmChiavePrec = tipoArmaturaChiave(intest->GetTipoArmatura() % 15);
				
		armChiave.tipoAlt = FALSE;
		armChiave.tipoArmPrec = intest->GetTipoArmatura();
        		
	}
	
	switch(tipoBat)
	{
    case INIZIO_BRANO:
		drwChiave=drwArmChv=drwTempo=TRUE;
		break;
    case INIZIO_PENTAGRAMMA:
    case INIZIO_PAGINA:
		drwChiave=drwArmChv=TRUE;
		drwTempo=FALSE;
		break;
    case GENERICA:
		drwChiave=drwArmChv=drwTempo=FALSE;
		break;
	}
	if(intest!=NULL)
	{
		if(GetChiave()->GetID()!=intest->GetChiave()->GetID())
			drwChiave=TRUE;
		if(GetTipoArmatura()!=intest->GetTipoArmatura())
			drwArmChv=TRUE;
		if(GetTempo()!=intest->GetTempo())
			drwTempo=TRUE;
	}
	
	switch(timeSignatureStatus)
	{
	case TimeSignature_HIDDEN:
		drwTempo=FALSE;
		break;
	case TimeSignature_ALWAYSVISIBLE:
		drwTempo=TRUE;
		break;
	}
	SetVU2Up(INTESTAZIONE2UP );
	SetVU2Dwn(INTESTAZIONE2DWN );
	SetVU2Lft(INTESTAZIONE2LFT );
	
	if(drwChiave)
		wdt+=DIST_INTEST_CHIAVE+ptrChiave->GetWidth();
	if(drwArmChv)
	{
		armChiave.SetVU(ptrChiave->GetShift());
		wdt+=DIST_CHIAVE_ARMCHIAVE+armChiave.GetWidth();
	}
	if(drwTempo)
	{
		tempo.SetVU();
		wdt+=DIST_ARMCHIAVE_TEMPO+tempo.GetWidth();
	}
	SetVU2Rgt(wdt);
}

void Intestazione::SetPos(DrawObject *drwobj,const Point& p)
{ 
	VUnit x=0;
		
	DrawObject::SetPos(drwobj,p);
	
	if(drwChiave)
	{
		ptrChiave->SetPos(this,Point(DIST_INTEST_CHIAVE+ptrChiave->GetVU2Lft(),0));
 		x+=DIST_INTEST_CHIAVE+ptrChiave->GetWidth();
	}
	else if(drwArmChv)
		ptrChiave->SetPos(this,Point(-ptrChiave->GetVU2Rgt(),0));
    
	if(drwArmChv)
	{
		armChiave.SetPos(ptrChiave,
			Point(DIST_CHIAVE_ARMCHIAVE+ptrChiave->GetVU2Rgt(),
			-ptrChiave->GetShift()*VU_PER_ROW/2));
		x+=DIST_CHIAVE_ARMCHIAVE+armChiave.GetWidth();
	}
	if(drwTempo)
	{
		tempo.SetPos(this,Point(x+DIST_ARMCHIAVE_TEMPO+tempo.GetVU2Lft(),0));
	}
}

void Intestazione::Draw()
{ 
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	if(drwChiave)
		ptrChiave->Draw();
	if(drwArmChv)
		armChiave.Draw();
	if(drwTempo)
		tempo.Draw();    

	Scr->SetColor(oldcol);
} 

Bool Intestazione::Hit(const Point& p,SelObj selObj,DrawObject *& drw,
					   SymPath& symPath)
{
	if(ptrChiave->Hit(p,selObj,drw,symPath))
		return TRUE;
	if(armChiave.Hit(p,selObj,drw,symPath))
		return TRUE;
	if(tempo.Hit(p,selObj,drw,symPath))
		return TRUE;
	return FALSE;
} 

void Intestazione::Copy(Intestazione* intes)
{
	switch(intes->GetChiave()->GetID())
	{
    case CL_CVIOLINO:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CViolino;
		break;
		//
    case CL_CBASSO8SOPRA:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CBasso8Sopra;
		break;
    case CL_CTAB:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CTab;
		break;
    case CL_CPERCUSBOX:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CPercusBox;
		break;
    case CL_CPERCUS2LINES:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CPercus2Lines;
		break;
    case CL_CVUOTA:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CVuota;
		break;
    case CL_CBASSOOLD:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CBassoOld;
		break;
		//
    case CL_CVIOLINO8:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CViolino8;
		break;
    case CL_CVIOLINO8SOPRA:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CViolino8Sopra;
		break;
    case CL_CBASSO:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CBasso;
		break;
    case CL_CBASSO8:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CBasso8;
		break;
    case CL_CBARITONO:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CBaritono;
		break;
    case CL_CTENORE:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CTenore;
		break;
    case CL_CTENORE8:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CTenore8;
		break;
    case CL_CCONTRALTO:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CContralto;
		break;
    case CL_CMEZZOSOPRANO:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CMezzoSoprano;
		break;
    case CL_CSOPRANO:
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CSoprano;
		break;
	}
	if(!ptrChiave)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT"," Intestazione","Copy  ()");
	
	SetArmatura(intes->GetTipoArmatura());
	tempo=intes->GetTempo();
}

void Intestazione::Free()
{
	if(ptrChiave!=NULL)
		delete ptrChiave;
}


CmdResult Intestazione::SymCommand(SymCmd& SCmd, Battuta *pBattuta) // GB
{
	switch(SCmd.CommandId)
	{
    case CMD_BATTUTA_CVIOLINO:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CVIOLINO))
			return CMD_FAIL;
		
		if (ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CViolino;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;
		
    case CMD_BATTUTA_CBASSO8SOPRA:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CBASSO8SOPRA))
			return CMD_FAIL;
		
		if (ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CBasso8Sopra;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;
		
    case CMD_BATTUTA_CTAB:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CTAB))
			return CMD_FAIL;
		
		if (ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CTab;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;
		
    case CMD_BATTUTA_CPERCUSBOX:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CPERCUSBOX))
			return CMD_FAIL;
		
		if (ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CPercusBox;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;
		
    case CMD_BATTUTA_CPERCUS2LINES:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CPERCUS2LINES))
			return CMD_FAIL;
		
		if (ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CPercus2Lines;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;
		
    case CMD_BATTUTA_CVUOTA:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CVUOTA))
			return CMD_FAIL;
		
		if (ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CVuota;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;
		
    case CMD_BATTUTA_CBASSOOLD:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CBASSOOLD))
			return CMD_FAIL;
		
		if (ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CBassoOld;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;
		
		
    case CMD_BATTUTA_CVIOLINO8SOPRA:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CVIOLINO8SOPRA))
			return CMD_FAIL;
		
		if (ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CViolino8Sopra;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;
		
    case CMD_BATTUTA_CBASSO:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CBASSO))
			return CMD_FAIL;
		
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CBasso;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;
		
    case CMD_BATTUTA_CVIOLINO8:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CVIOLINO8))
			return CMD_FAIL;
		
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CViolino8;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;
		
    case CMD_BATTUTA_CBASSO8:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CBASSO8))
			return CMD_FAIL;
		
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CBasso8;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;
		
    case CMD_BATTUTA_CBARITONO:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CBARITONO))
			return CMD_FAIL;
		
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CBaritono;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;
		
    case CMD_BATTUTA_CTENORE:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CTENORE))
			return CMD_FAIL;
		
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CTenore;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;
		
    case CMD_BATTUTA_CTENORE8:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CTENORE8))
			return CMD_FAIL;
		
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CTenore8;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;
		
	case CMD_BATTUTA_CCONTRALTO:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CCONTRALTO))
			return CMD_FAIL;
		
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CContralto;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;
		
    case CMD_BATTUTA_CMEZZOSOPRANO:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CMEZZOSOPRANO))
			return CMD_FAIL;
		
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CMezzoSoprano;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;
		
    case CMD_BATTUTA_CSOPRANO:
		if ((ptrChiave!=NULL) && (ptrChiave->GetID()==CL_CSOPRANO))
			return CMD_FAIL;
		
		if(ptrChiave!=NULL)
			delete ptrChiave;
		ptrChiave=new CSoprano;
		if(ptrChiave==NULL)
		{      
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT"," Intestazione","SymCommand()");
			return CMD_FAIL;
		};
		return CMD_OK;  
		
    case CMD_BATTUTA_ARMCHV_0D:
		if (GetTipoArmatura() == DO_maggiore)
			return CMD_FAIL;
		SetArmatura(DO_maggiore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_1D:
		if (GetTipoArmatura() == SOL_maggiore)
			return CMD_FAIL;
		SetArmatura(SOL_maggiore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_2D:
		if (GetTipoArmatura() == RE_maggiore)
			return CMD_FAIL;
		SetArmatura(RE_maggiore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_3D:
		if (GetTipoArmatura() == LA_maggiore)
			return CMD_FAIL;
		SetArmatura(LA_maggiore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_4D:
		if (GetTipoArmatura() == MI_maggiore)
			return CMD_FAIL;
		SetArmatura(MI_maggiore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_5D:
		if (GetTipoArmatura() == SI_maggiore)
			return CMD_FAIL;
		SetArmatura(SI_maggiore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_6D:
		if (GetTipoArmatura() == FAd_maggiore)
			return CMD_FAIL;
		SetArmatura(FAd_maggiore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_7D:
		if (GetTipoArmatura() == DOd_maggiore)
			return CMD_FAIL;
		SetArmatura(DOd_maggiore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_1B:
		if (GetTipoArmatura() == FA_maggiore)
			return CMD_FAIL;
		SetArmatura(FA_maggiore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_2B:
		if (GetTipoArmatura() == SIb_maggiore)
			return CMD_FAIL;
		SetArmatura(SIb_maggiore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_3B:
		if (GetTipoArmatura() == MIb_maggiore)
			return CMD_FAIL;
		SetArmatura(MIb_maggiore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_4B:
		if (GetTipoArmatura() == LAb_maggiore)
			return CMD_FAIL;
		SetArmatura(LAb_maggiore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_5B:
		if (GetTipoArmatura() == REb_maggiore)
			return CMD_FAIL;
		SetArmatura(REb_maggiore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_6B:
		if (GetTipoArmatura() == SOLb_maggiore)
			return CMD_FAIL;
		SetArmatura(SOLb_maggiore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_7B:
		if (GetTipoArmatura() == DOb_maggiore)
			return CMD_FAIL;
		SetArmatura(DOb_maggiore);
		return CMD_OK;
		break;
		//Per le armature minori
    case CMD_BATTUTA_ARMCHV_0Dm:
		if (GetTipoArmatura() == LA_minore)
			return CMD_FAIL;
		SetArmatura(LA_minore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_1Dm:
		if (GetTipoArmatura() == MI_minore)
			return CMD_FAIL;
		SetArmatura(MI_minore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_2Dm:
		if (GetTipoArmatura() == SI_minore)
			return CMD_FAIL;
		SetArmatura(SI_minore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_3Dm:
		if (GetTipoArmatura() == FAd_minore)
			return CMD_FAIL;
		SetArmatura(FAd_minore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_4Dm:
		if (GetTipoArmatura() == DOd_minore)
			return CMD_FAIL;
		SetArmatura(DOd_minore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_5Dm:
		if (GetTipoArmatura() == SOLd_minore)
			return CMD_FAIL;
		SetArmatura(SOLd_minore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_6Dm:
		if (GetTipoArmatura() == REd_minore)
			return CMD_FAIL;
		SetArmatura(REd_minore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_7Dm:
		if (GetTipoArmatura() == LAd_minore)
			return CMD_FAIL;
		SetArmatura(LAd_minore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_1Bm:
		if (GetTipoArmatura() == RE_minore)
			return CMD_FAIL;
		SetArmatura(RE_minore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_2Bm:
		if (GetTipoArmatura() == SOL_minore)
			return CMD_FAIL;
		SetArmatura(SOL_minore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_3Bm:
		if (GetTipoArmatura() == DO_minore)
			return CMD_FAIL;
		SetArmatura(DO_minore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_4Bm:
		if (GetTipoArmatura() == FA_minore)
			return CMD_FAIL;
		SetArmatura(FA_minore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_5Bm:
		if (GetTipoArmatura() == SIb_minore)
			return CMD_FAIL;
		SetArmatura(SIb_minore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_6Bm:
		if (GetTipoArmatura() == MIb_minore)
			return CMD_FAIL;
		SetArmatura(MIb_minore);
		return CMD_OK;
		break;
    case CMD_BATTUTA_ARMCHV_7Bm:
		if (GetTipoArmatura() == LAb_minore)
			return CMD_FAIL;
		SetArmatura(LAb_minore);
		return CMD_OK;
		break;
		
    case CMD_BATTUTA_TEMPO:
		if (tempo.HasSameDesc((char*)SCmd.Text))
			return CMD_FAIL;
		SetTempo((char*)SCmd.Text);
		return CMD_OK;
		break;
  }
  return CMD_UNKNOWN;
}

