/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: intestaz.hpp,v 1.1 2005/07/25 10:51:09 bellinpo Exp $ */



#ifndef INTESTAZIONE_HPP
#define INTESTAZIONE_HPP

#include "drw/drawobje.hpp"
#include "bat/chiave.hpp"
#include "bat/armatura.hpp"
#include "bat/tempo.hpp"

class Intestazione:public DrawObject
{
protected:
    ArmaturaChiave armChiave,armChiaveAttuale;
    Tempo tempo;
    Chiave *ptrChiave;
    Bool drwChiave,drwArmChv,drwTempo;
    VUnit INTESTAZIONE2Up,INTESTAZIONE2Dwn,INTESTAZIONE2Lft,INTESTAZIONE2Rgt;
	// Distanze da AbsPos dei punti superiore, inferiore, sinistro e
	// destro del rettangolo contenente l'intestazione.
	TimeSignatureStatus timeSignatureStatus;
	
public:
    Intestazione() { ptrChiave=NULL;timeSignatureStatus=TimeSignature_NORMAL; };
    ~Intestazione() { if (ptrChiave!=NULL) delete ptrChiave; };
	// tipo di armatura (diesis o bemolle) a cui deve fare riferimento,
	// se esiste, l'armatura dei bequadri.  
    void SetVU2Up(VUnit vu);  
    void SetVU2Dwn(VUnit vu);  
    void SetVU2Lft(VUnit vu);  
    void SetVU2Rgt(VUnit vu);  
    
    VUnit GetVU2Up();
    VUnit GetVU2Dwn();
    VUnit GetVU2Lft();
    VUnit GetVU2Rgt();
    
    void SetVU(tipoBattuta,Intestazione*);
	// Imposta la dimensioni dell'intestazione in relazione al tipo di battuta attuale
	// e all'intestazione della battuta precedente.
    void SetPos(DrawObject *d,const Point& p);
	// Imposta la posizione delle parti dell'intestazione presenti
	// in relazione al tipo di battuta attuale.
    void SetChiave(Chiave *chv);
	// Imposta la chiave dell'intestazione.
    Chiave *GetChiave(); //GB
	// Restituisce la chiave dell'intestazione.
    void SetArmatura(tipoArmaturaChiave arm);
	// Imposta l'armatura di chiave dell'intestazione.
    tipoArmaturaChiave GetTipoArmatura();
	// Restituisce il tipo di armatura di chiave dell'intestazione.
    ArmaturaChiave GetArmatura();
    ArmaturaChiave * GetPtrArmatura();
	// Restituisce l'armatura dell'intestazione.
	
    void SetTempo(char tmp[]);
	// Imposta l'Tempo di chiave dell'intestazione.
    Tempo GetTempo();
    Tempo * GetPtrTemp();
	// Restituisce il Tempo dell'intestazione.
    void Copy(Intestazione*);
	// Copia l'intestazione attuale.
    void Draw();                           
	// Necessario per la definizione VIRTUAL di Draw() in DrawObject.
    Bool Hit(const Point&,SelObj,DrawObject*&,SymPath&);
	// restituisce il puntatore all'oggetto stesso o al sottoggetto
	// che  stato colpito (l'oggetto contiene il Point). 
	// Se vale NULL o non si  colpito niente  oppure l'oggetto 
	// che  stato colpito non  del tipo SelObj.
    CmdResult SymCommand(SymCmd&, Battuta *pb=NULL);
    void Free();
	// Dealloca tutti i puntatori connessi all'intestazione.
	
	TimeSignatureStatus GetTimeSignatureStatus() { return timeSignatureStatus;};
	void SetTimeSignatureStatus(TimeSignatureStatus tss) { timeSignatureStatus=tss;};
	
	int SaveXML(FILE *fp);
};

#endif //INTESTAZIONE_HPP
