/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include "bat/layer.hpp"


#include "liooext.hpp"
#include "bat/battuta.hpp" 
#include "fig/nota.hpp"

#include "fig/nbreve.hpp"
#include "fig/nsemibre.hpp"  
#include "fig/nminima.hpp"
#include "fig/nsemimin.hpp"
#include "fig/ncroma.hpp"
#include "fig/nsemicro.hpp"
#include "fig/nbiscrom.hpp"
#include "fig/nsemibis.hpp"
#include "fig/nfusa.hpp"

#include "abb/tremolo.hpp"
#include "abb/glissato.hpp"

#include "fig/psemibre.hpp"  
#include "fig/pminima.hpp"
#include "fig/psemimin.hpp"
#include "fig/pcroma.hpp"
#include "fig/psemicro.hpp"
#include "fig/pbiscrom.hpp"
#include "fig/psemibis.hpp"
#include "fig/pfusa.hpp"
#include "fig/pduebatt.hpp"
#include "fig/pquabatt.hpp"
#include "fig/pgeneric.hpp"
#include "fig/spazio.hpp"
#include "fig/gruppono.hpp" 
#include "fig/accordo.hpp"
#include "fig/ripet.hpp"
#include "fig/riptempo.hpp"
#include "fig/ripbatt.hpp"
#include "fig/ripmbat.hpp"

#include "bat/bsingola.hpp"
#include "bat/bfinale.hpp"
#include "bat/bdoppia.hpp"
#include "bat/bfinerit.hpp"
#include "bat/binizior.hpp"
#include "bat/binifine.hpp"


#include "bat/cviolino.hpp"
#include "bat/cviolin8.hpp"
#include "bat/cviolino8sopra.hpp"
#include "bat/cbasso.hpp"
#include "bat/cbasso8.hpp"
#include "bat/cbariton.hpp"
#include "bat/ctenore.hpp"
#include "bat/ctenore8.hpp"
#include "bat/ccontral.hpp"
#include "bat/cmezzoso.hpp"
#include "bat/csoprano.hpp"
//
#include "bat/cbasso8sopra.hpp"
#include "bat/ctab.hpp"
#include "bat/cpercusbox.hpp"
#include "bat/cpercus2lines.hpp"
#include "bat/cvuota.hpp"
#include "bat/cbassoold.hpp"
#include "gui/dialogs.hpp" // finestra debug milla
//

#include "int/legatqua.hpp"


Layer::Layer()
{
	Lista::Lista();
	CodeCounter=1;
	numeroFig=0;
	isSelected = FALSE;
	Opaque = false;
}

Layer::~Layer()
{
	
#ifdef DEBUG_FREE
	if (tipoLeggio!=LEGGIO_MASAE)
	{
		FILE *pippo;
		
		pippo = fopen("DEAL_NET.txt","a");
		fprintf(pippo,"\n\n\n ---- non masae   DEALLOCO Layer  ----------------\n\n\n");
		fclose(pippo);
	}
#endif
	
	Free();
}

void Layer::SetNumeroFig(unsigned long int numFig)
{
	numeroFig = numFig;	
}

unsigned long int Layer::GetNumeroFig()
{
	return numeroFig;
}


NumCode Layer::Note2GruppoNote(Battuta* pbat,short current_layer, TipoInserimento tpIns, NumCode figCode1, NumCode figCode2)
{
	
	GruppoNote *newGruppo;
	Figura *fig;
	Bool giusto=TRUE, found1=FALSE, found2=FALSE;
	// Non sostituire la seguente lista con una lista di figure, altrimenti
	// i NumericCode delle figure vengono cambiati!
	Lista selList;  
	short current_staff;
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		msgdebug.sprintf("");
	}
	
	
	newGruppo=new GruppoNote;
	if(! newGruppo)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Layer","Note2GruppoNote()");
	
	
	// Scandisce la battuta.
	fig = GetFirstF();
	current_staff=fig->GetNStaff();
	while(fig!=NULL && giusto && !found2)
	{
		if(fig->GetNumericCode()==figCode1 && !found1)
			found1 = TRUE;
		if(found1)
		{
			if(fig->GetNumericCode()==figCode2)
				found2 = TRUE;
			giusto = newGruppo->AddFigura(fig);
		}
		fig = GetNextF(fig);
	}
	
	if(newGruppo->GetNumNote()<=1)
	{
		do
		fig = newGruppo->ExtractFigura(pbat, current_layer);
		while(fig!=NULL);
		delete newGruppo;
		newGruppo=NULL; 
		EM.Error(MUSICAL_ERR_AREA,
			"EC_GRUPPO_NON_PERMESSO","Layer","Note2GruppoNote()");
		return 0;
	}
	
	fig = newGruppo->GetFirstF();
	ChangeF(fig, newGruppo);
	
	while(fig!=NULL)
	{
		// Toglie dal layer le figure selezionate eccetto la prima.
		DelPtr(fig);
		fig = newGruppo->GetNextF(fig);
	}
	
	// metto qui la chiamata a ImmRule perche' credo che qui il gruppo
	// di note sia gia' formato!  
	
	switch(tpIns)
	{
    case AUTOMATICO:
		switch (immissione.ImmRule(CL_GRUPPONOTE,(Nota*)newGruppo, pbat, NULL, current_layer))
		{
		case TRUE:
			newGruppo->SetBeamPosition(GruppoNote::BEAM_UP);
			break;
		case FALSE:
			newGruppo->SetBeamPosition(GruppoNote::BEAM_DWN);
			break;
		case 2:
			newGruppo->SetBeamPosition(GruppoNote::BEAM_12);
			break;
		case 3:
			newGruppo->SetBeamPosition(GruppoNote::BEAM_23);
			break;
		default:
			newGruppo->SetGamboUp();
		}   
		break; 
		case SOPRA:
			newGruppo->SetGamboUp();
			break;
		case SOTTO:
			newGruppo->SetGamboDwn();
			break; 
		case MID12:
			newGruppo->SetBeamPosition(GruppoNote::BEAM_12);
			break;
		case MID23:
			newGruppo->SetBeamPosition(GruppoNote::BEAM_23);
			break; 
	} 
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		wxMessageBox(msgdebug);
	}
	
	return newGruppo->GetNumericCode();
}


NumCode Layer::InsNota(Battuta *pbat,int Altezza, Bool SegnoAltezza, ClassID figID, short current_layer,
					   short current_staff, TipoInserimento GamboUp, Bool inAccordo, NumCode figCode,
					   Bool InsInTesta, NoteHead_ID ht, NumCode thisCode)
{
	Nota *pNota;
	Figura *pFig;
	short AltSpazio=0;
	int lgambo;
	Bool gamboupdwn;
	Bool found=TRUE;
	
    if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		msgdebug.sprintf("");
	}
	
	
	switch(figID)
	{
    case CL_NBREVE:
		pNota=new NBreve;
		break;
    case CL_NSEMIBREVE:
		pNota=new NSemibreve;
		break; 
    case CL_NMINIMA:
		pNota=new NMinima;
		break;
    case CL_NSEMIMINIMA:
		pNota=new NSemiminima;
		break;
    case CL_NCROMA:
		pNota=new NCroma;
		break;
    case CL_NSEMICROMA:
		pNota=new NSemicroma;
		break;
    case CL_NBISCROMA:
		pNota=new NBiscroma;
		break;
    case CL_NSEMIBISCROMA:
		pNota=new NSemibiscroma;
		break;
    case CL_NFUSA:
		pNota=new NFusa;
		break;
    default:
		EM.Error(MUSICAL_ERR_AREA, "EC_COD_FIG_UNKNOW","Layer","InsNota()");
		return 0;
	}
	if(pNota==NULL)
	{ 
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Layer","InsNota()");
		return 0;
	}
	// Imposta la nota.
	pNota->SetNumericCode(thisCode);

	if (!SegnoAltezza) Altezza = - Altezza;
	pNota->SetAltezza(Altezza);
	
	pNota->SetNStaff(current_staff); // 
	
	pNota->SetCoda(TRUE);
	
	pNota->SetHeadType(ht);
	switch(ht)
	{
    case HEAD_ALPHANUM:
    case HEAD_ALPHANUM_SQUARE:
    case HEAD_ALPHANUM_REVERSE:
		{
			char headVal[10];
			
			headVal[0]=0;
			if(InputString("Head value (max 2 chars):",headVal,10,"Head type"))
			{
				headVal[2]=0;
				pNota->SetHeadCode((unsigned char*)headVal);
			}
			else
				return 0;
		}
	}
    
	if(!inAccordo)
	{
		// INSERIMENTO FRA LE FIGURE ESISTENTI.
		if((pbat->GetNumeroFig())>=MAX_FIGURE)
		{
			EM.Error(MUSICAL_ERR_AREA, "EC_BATFULL","Layer","InsNota()");
			return 0;
		}
		if(InsInTesta)
		{
			// Inserimento in testa.
			InsertTop(pNota);
			
			switch(GamboUp)
			{
			case AUTOMATICO:
				if (immissione.ImmRule(figID, pNota, pbat, NULL, current_layer))  
				{ 
					if (pNota->GetNotina())
					{ 
						pNota->SetGamboUp();
					} 
					else
					{ 
						gamboupdwn = TRUE;
						lgambo = posizionam.PosRule((DrawObject*)pNota, pNota, pbat,gamboupdwn, current_layer,current_staff);
						pNota->SetLGambo(lgambo);
					}
				}	  
				else 
				{  
					if (pNota->GetNotina())
					{ 
						pNota->SetGamboDwn();
					} 
					else
					{ 
						gamboupdwn = FALSE;
						lgambo = posizionam.PosRule((DrawObject*)pNota, pNota, pbat,gamboupdwn, current_layer,current_staff);
						pNota->SetLGambo(-lgambo);
					}
				}	 
				break;  
			case SOPRA:
				{ 
					if (pNota->GetNotina())
					{ 
						pNota->SetGamboUp();
					} 
					else
					{ 
						gamboupdwn = TRUE;
						lgambo = posizionam.PosRule((DrawObject*)pNota, pNota, pbat,gamboupdwn, current_layer,current_staff);
						pNota->SetLGambo(lgambo);
					}
				}
				break;
			case SOTTO:
				{ 
					if (pNota->GetNotina())
					{ 
						pNota->SetGamboDwn();
					} 
					else
					{ 
						gamboupdwn = FALSE;
						lgambo = posizionam.PosRule((DrawObject*)pNota, pNota, pbat,gamboupdwn, current_layer,current_staff);
						pNota->SetLGambo(-lgambo);
					}
				}	
				break;
			} 
			
			
		}
		else
		{
			// Inserimento dopo qualche figura.
			pFig = GetFirstFig(); 
			found = FALSE;
			while(pFig!=NULL && !found)
			{
				if(pFig->GetNumericCode()==figCode)
					found = TRUE;
				else
				{
					pFig = GetNextFig(); 
				}
			}
			switch(GamboUp)
			{
			case AUTOMATICO:
				if (immissione.ImmRule(figID, pNota, pbat, NULL, current_layer))  
				{
					if (pNota->GetNotina())
					{ 
						pNota->SetGamboUp();
					}
					else
					{
						gamboupdwn = TRUE;
						lgambo = posizionam.PosRule((DrawObject*)pNota, pNota, pbat,gamboupdwn, current_layer,current_staff);
						pNota->SetLGambo(lgambo);
					}
				}	  
				else 
				{ 
					if (pNota->GetNotina())
					{ 
						pNota->SetGamboDwn();
					}
					else
					{
						gamboupdwn = FALSE;
						lgambo = posizionam.PosRule((DrawObject*)pNota, pNota, pbat,gamboupdwn, current_layer,current_staff);  
						pNota->SetLGambo(-lgambo);
					}
				}	 
				break;  
			case SOPRA:  
				{
					if (pNota->GetNotina())
					{ 
						pNota->SetGamboUp();
					}
					else
					{
						gamboupdwn = TRUE;
						lgambo = posizionam.PosRule((DrawObject*)pNota, pNota, pbat,gamboupdwn, current_layer,current_staff);
						pNota->SetLGambo(lgambo);
					}
				}	
				break;
			case SOTTO:
				{ 
					if (pNota->GetNotina())
					{ 
						pNota->SetGamboDwn();
					}
					else
					{
						gamboupdwn = FALSE;
						lgambo = posizionam.PosRule((DrawObject*)pNota, pNota, pbat,gamboupdwn, current_layer,current_staff);  
						pNota->SetLGambo(-lgambo);
					}
				}
				break;
			}	
			
			if(found)
			{
				InsertAfterFig(pFig, pNota, pbat, current_layer,current_staff); 
				
			}
			else
			{
				EM.Error(MUSICAL_ERR_AREA, "EC_FIG_NOTFOUND","Layer","InsNota()");
				delete pNota;
				pNota=NULL; 
				return 0;
			}
		}  
  }
  else
  {
	  // INSERIMENTO IN ACCORDO.
	  // Cerca la figura nella quale inserire.
	  pFig = GetFirstF();
	  found = FALSE;
	  while(pFig!=NULL && !found)
	  {
		  if(pFig->GetNumericCode()==figCode)
			  found = TRUE;
		  else
			  pFig = GetNextF(pFig);
	  }
	  if(found)
	  {
		  if(InsInAccordo(pbat,pNota, pFig, GamboUp, current_layer,current_staff))
		  {
			  figCode=pFig->GetNumericCode();
		  }
		  else
			  return 0;
	  }
  }
  if(!found)
  {
	  EM.Error(MUSICAL_ERR_AREA, "EC_FIG_NOTFOUND","Layer","InsNota()");
	  delete pNota;
	  pNota=NULL; 
	  return 0;
  }
  
  if (DebugNormalMilla || DebugNormalMillaPos)
  { 
	  posizionam.TestTokenCond(pNota, (DrawObject*)pNota, pbat, gamboupdwn, current_layer);
	  
	  WDebugMilla();   
  }
  
  return pNota->GetNumericCode();
}  


Bool Layer::InsInAccordo(Battuta *pbat,Nota *pNota, Figura *pFig, TipoInserimento GamboUp, short current_layer,short current_staff)
{
	Accordo *pAcc;
	ClassID FigID=pFig->GetID();
	Bool gamboupdwn;
	int lgambo;
	DrawObject *drw;
	drw = pNota; 
	
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		msgdebug.sprintf("");
	}
	
	
	
	if(FigID == pNota->GetID())
	{
		// Crea l'accordo.
		pAcc = new Accordo;
		if(!pAcc)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Layer","InsInAccordo()");
		
		pAcc->SetNumericCode(pFig->GetNumericCode());
		pAcc->SetColor(pFig->GetColor());
		pAcc->SetNStaff(pFig->GetNStaff()); 
		pAcc->SetPBattuta(pbat);
		pAcc->SetHeadCode(((Nota *)pFig)->GetHeadCode());
		
		if(!(pAcc->AddNota((Nota*)pFig)) || !(pAcc->AddNota(pNota)))
		{
			EM.Error(MUSICAL_ERR_AREA, "EC_ACCORDO_NON_PERMESSO","Layer","InsInAccordo()");
			// Non distrugge l'accordo, altrimenti distruggerebbe anche le note.
			return FALSE;
		}
		pAcc->SetCoda(TRUE);
		
		switch(GamboUp)
		{
		case AUTOMATICO:
			if (immissione.ImmRule(FigID, pAcc, pbat, NULL, current_layer))  
			{
				if (pAcc->GetNotina())
				{
					pAcc->SetGamboUp();
				} 
				else
				{  
					gamboupdwn = TRUE;
					lgambo = int(posizionam.PosRule((DrawObject*)pNota, pAcc, pbat, gamboupdwn, current_layer,pFig->GetNStaff()));
					pAcc->SetLGambo(lgambo);
				}
			} 
			else
			{
				if (pAcc->GetNotina())
				{
					pAcc->SetGamboDwn();
				} 
				else
				{  
					gamboupdwn = FALSE;
					lgambo = int(posizionam.PosRule((DrawObject*)pNota, pAcc, pbat, gamboupdwn, current_layer,pFig->GetNStaff()));
					pAcc->SetLGambo(-lgambo);
				}
			}	 
			break;  
		case SOPRA:
			{
				if (pAcc->GetNotina())
				{
					pAcc->SetGamboUp();
				} 
				else
				{  
					gamboupdwn = TRUE;
					lgambo = int(posizionam.PosRule((DrawObject*)pNota, pAcc, pbat, gamboupdwn, current_layer,pFig->GetNStaff()));
					pAcc->SetLGambo(lgambo);
				} 
			} 
			break;
		case SOTTO:
			{ 
				if (pAcc->GetNotina())
				{
					pAcc->SetGamboDwn();
				} 
				else
				{  
					gamboupdwn = FALSE;
					lgambo = int(posizionam.PosRule((DrawObject*)pNota, pAcc, pbat, gamboupdwn, current_layer,pFig->GetNStaff()));
					pAcc->SetLGambo(-lgambo);
				}
			}
			break;
		}
		
		pNota->SetNumericCode(GetCodeCounter());
		SetCodeCounter(GetCodeCounter()+1);
		ChangeFSimple(pFig, pAcc);
		
		ListaIntEst* pLista;
		pLista = pFig->GetPBattuta()->GetParentScore()->GetListaIntervalli();
		pLista->ReplaceIntEstFig(pFig, pAcc);

		return TRUE;
	}
	else if(FigID==CL_ACCORDO)
	{
		// Aggiunge la nota all'accordo.
		pAcc = (Accordo*)pFig;
		
		if(!pAcc->AddNota(pNota))
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_ACCORDO_NON_PERMESSO","Layer","InsInAccordo()");
			return FALSE;
		}
		
		switch(GamboUp)
		{
		case AUTOMATICO:
			if (immissione.ImmRule(FigID, pAcc, pbat, NULL, current_layer))  
			{
				if (pAcc->GetNotina())
				{
					pAcc->SetGamboUp();
				}  
				else 
				{ 
					gamboupdwn = TRUE;
					lgambo = int(posizionam.PosRule((DrawObject*)pNota, pAcc, pbat, gamboupdwn, current_layer,pFig->GetNStaff()));
					pAcc->SetLGambo(lgambo);
				}
			}  
			else 
			{ 
				if (pAcc->GetNotina())
				{
					pAcc->SetGamboDwn();
				}  
				else 
				{ 
					gamboupdwn = FALSE;
					lgambo = int(posizionam.PosRule((DrawObject*)pNota, pAcc, pbat, gamboupdwn, current_layer,pFig->GetNStaff()));
					pAcc->SetLGambo(-lgambo);
				}
			}	 
			break;  
		case SOPRA:
			{
				if (pAcc->GetNotina())
				{
					pAcc->SetGamboUp();
				}  
				else 
				{ 
					gamboupdwn = TRUE;
					lgambo = int(posizionam.PosRule((DrawObject*)pNota, pAcc, pbat, gamboupdwn, current_layer,pFig->GetNStaff()));
					pAcc->SetLGambo(lgambo);
				} 
			} 
			break;
		case SOTTO:
			{ 
				if (pAcc->GetNotina())
				{
					pAcc->SetGamboDwn();
				}  
				else 
				{ 
					gamboupdwn = FALSE;
					lgambo = int(posizionam.PosRule((DrawObject*)pNota, pAcc, pbat, gamboupdwn, current_layer,pFig->GetNStaff()));
					pAcc->SetLGambo(-lgambo);
				}	
			}	 
			break;
		}
		
		pNota->SetNumericCode(GetCodeCounter());
		SetCodeCounter(GetCodeCounter()+1);
		
		return TRUE;
	}
	else if(FigID==CL_SPAZIO)
	{
		FigID=pNota->GetID();
		ChangeF(pFig, pNota);
		
		switch(GamboUp)
		{
		case AUTOMATICO:
			if (immissione.ImmRule(FigID, pNota, pbat, NULL, current_layer))  
			{ 
				if (pNota->GetNotina())
				{ 
					pNota->SetGamboUp();
				}	  
				else
				{
					gamboupdwn = TRUE;
					lgambo = int(posizionam.PosRule((DrawObject*)pNota, pNota, pbat, gamboupdwn, current_layer,pFig->GetNStaff()));
					pNota->SetLGambo(lgambo);
				}
			}	  
			else
			{
				if (pNota->GetNotina())
				{ 
					pNota->SetGamboDwn();
				}	  
				else
				{
					gamboupdwn = FALSE;
					lgambo = int(posizionam.PosRule((DrawObject*)pNota, pNota, pbat, gamboupdwn, current_layer,pFig->GetNStaff()));
					pNota->SetLGambo(-lgambo);
				}
			}	 
			break;  
		case SOPRA:
			{  
				if (pNota->GetNotina())
				{ 
					pNota->SetGamboUp();
				}	  
				else
				{
					gamboupdwn = TRUE;
					lgambo = int(posizionam.PosRule((DrawObject*)pNota, pNota, pbat, gamboupdwn, current_layer,pFig->GetNStaff()));
					pNota->SetLGambo(lgambo);
				}
			}	 
			break;
		case SOTTO:
			{
				if (pNota->GetNotina())
				{ 
					pNota->SetGamboDwn();
				}	  
				else
				{
					gamboupdwn = FALSE;
					lgambo = int(posizionam.PosRule((DrawObject*)pNota, pNota, pbat, gamboupdwn, current_layer,pFig->GetNStaff()));
					pNota->SetLGambo(-lgambo);
				}	
			}	
			break;
		}
		
		
		if (DebugNormalMilla || DebugNormalMillaPos)
		{ 
			wxMessageBox(msgdebug);
			posizionam.TestTokenCond(pNota, (DrawObject*)pNota, pbat, gamboupdwn, current_layer);
			
			WDebugMilla(); 
		}
		
		
		return TRUE;
	}
	
	EM.Error(MUSICAL_ERR_AREA,"EC_ACCORDO_NON_PERMESSO","Layer","InsInAccordo()");
	return FALSE;
}



NumCode Layer::InsPausa(Battuta *pbat,int Altezza, Bool SegnoAltezza, ClassID figID, short current_layer,short current_staff,
						TipoInserimento AltVariabile, Bool inAccordo, NumCode figCode,
						Bool InsInTesta, NumCode thisCode)
{
	Pausa *pPausa;
	
	Figura *pFig;
	short AltSpazio=0;
	Bool found=TRUE;
	
	if (!SegnoAltezza) Altezza = - Altezza;
	
	switch(figID)
	{
    case CL_PSEMIBREVE:
		pPausa=new PSemibreve;
		if(!pPausa)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Layer","InsPausa()");
		if(AltVariabile==AUTOMATICO)
			pPausa->SetAltezza(6);
		else
			pPausa->SetAltezza(Altezza);
		break; 
    case CL_PMINIMA:
		pPausa=new PMinima;
		if(!pPausa)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Layer","InsPausa()");
		if(AltVariabile==AUTOMATICO)
			pPausa->SetAltezza(4);
		else
			pPausa->SetAltezza(Altezza);
		break;
    case CL_PSEMIMINIMA:
		pPausa=new PSemiminima;
		if(!pPausa)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Layer","InsPausa()");
		if(AltVariabile==AUTOMATICO)
			pPausa->SetAltezza(1);
		else
			pPausa->SetAltezza(Altezza);
		break;
    case CL_PCROMA:
		pPausa=new PCroma;
		if(!pPausa)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Layer","InsPausa()");
		if(AltVariabile==AUTOMATICO)
			pPausa->SetAltezza(2);
		else
			pPausa->SetAltezza(Altezza);
		break;
    case CL_PSEMICROMA:
		pPausa=new PSemicroma;
		if(!pPausa)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Layer","InsPausa()");
		if(AltVariabile==AUTOMATICO)
			pPausa->SetAltezza(0);
		else
			pPausa->SetAltezza(Altezza);
		break;
    case CL_PBISCROMA:
		pPausa=new PBiscroma;
		if(!pPausa)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Layer","InsPausa()");
		if(AltVariabile==AUTOMATICO)
			pPausa->SetAltezza(0);
		else
			pPausa->SetAltezza(Altezza);
		break;
    case CL_PSEMIBISCROMA:
		pPausa=new PSemibiscroma;
		if(!pPausa)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Layer","InsPausa()");
		if(AltVariabile==AUTOMATICO)
			pPausa->SetAltezza(0);
		else
			pPausa->SetAltezza(Altezza);
		break;
    case CL_PFUSA:
		pPausa=new PFusa;
		if(!pPausa)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Layer","InsPausa()");
		if(AltVariabile==AUTOMATICO)
			pPausa->SetAltezza(0);
		else
			pPausa->SetAltezza(Altezza);
		break;
    case CL_PDUEBATTUTE:
		pPausa=new PDueBattute;
		if(!pPausa)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Layer","InsPausa()");                       
		if(AltVariabile==AUTOMATICO)
			pPausa->SetAltezza(5);
		else
			pPausa->SetAltezza(Altezza);
		break;
    case CL_PQUATTROBATTUTE:
		pPausa=new PQuattroBattute;
		if(!pPausa)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Layer","InsPausa()");               
		if(AltVariabile==AUTOMATICO)
			pPausa->SetAltezza(5);
		else
			pPausa->SetAltezza(Altezza);
		break;
    case CL_PGENERICA:
		pPausa=new PGenerica(pbat);
		if(!pPausa)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Layer","InsPausa()");               
		if(AltVariabile==AUTOMATICO)
			pPausa->SetAltezza(5);
		else
			pPausa->SetAltezza(Altezza);
		break;
    default:
        EM.Error(MUSICAL_ERR_AREA, "EC_COD_FIG_UNKNOW","Layer","InsPausa()");
        return 0;
  }
  pPausa->SetNStaff(current_staff);
  if(pPausa==NULL)
  {
	  return 0;
  }
  
	pPausa->SetNumericCode(thisCode);

  if(!inAccordo)
  {
	  // INSERIMENTO FRA LE FIGURE ESISTENTI.
	  if((pbat->GetNumeroFig())>=MAX_FIGURE)
	  {
		  EM.Error(MUSICAL_ERR_AREA, "EC_BATFULL","Layer","InsPausa()");
		  delete pPausa;
		  pPausa=NULL; 
		  return 0;
	  }
	  
	  if(InsInTesta)
	  {
		  // Inserimento in testa.
		  InsertTop(pPausa);
	  }
	  else
	  {
		  // Inserimento dopo qualche figura.
		  pFig = GetFirstFig(); 
		  found = FALSE;
		  while(pFig!=NULL && !found)
		  {
			  if(pFig->GetNumericCode()==figCode)
				  found = TRUE;
			  else
			  {
				  pFig = GetNextFig(); 
			  }
		  }
		  if(found)
		  {
			  InsertAfterFig(pFig, pPausa, pbat, current_layer,current_staff); 
		  }
		  else
		  {
			  EM.Error(MUSICAL_ERR_AREA, "EC_FIG_NOTFOUND","Layer","InsPausa()");
			  delete pPausa;
			  pPausa=NULL; 
			  return 0;
			  
		  }
	  }  
  }
  else
  {
	  // INSERIMENTO IN SOSTITUZIONE DI UNO SPAZIO.
	  // Cerca lo spazio da sostituire.
	  pFig = GetFirstF();
	  found = FALSE;
	  while(pFig!=NULL && !found)
	  {
		  if(pFig->GetNumericCode()==figCode && pFig->GetID()==CL_SPAZIO)
			  found = TRUE;
		  else
			  pFig = GetNextF(pFig);
	  }
	  if(found)
	  {
		  ChangeF(pFig, pPausa);
	  }
  }
  if (!found)
  {
	  EM.Error(MUSICAL_ERR_AREA, "EC_FIG_NOTFOUND","Layer","InsPausa()");
	  delete pPausa;
	  pPausa=NULL; 
	  return 0;
  }
  
  return pPausa->GetNumericCode();
}    


NumCode Layer::InsSpazio(Battuta *pbat,short current_layer, NumCode figCode,Bool InsInTesta, short CodeCommand,
						 short current_staff,int positionrelat, NumCode thisCode)
{
	Figura *pFig;
	short AltSpazio=0, AltAltroSpazio=0;
	Bool found;
	
	Spazio *pSpazio;
	switch (CodeCommand)
	{
    case 0:
		pSpazio=new Spazio;
		pSpazio->Setrelpos(positionrelat);
		positionrelat=0;
		break;
		
    case CMD_BATTUTA_CVIOLINO:
		pSpazio=new CViolino;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
		//
    case CMD_BATTUTA_CBASSO8SOPRA:
		pSpazio=new CBasso8Sopra;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
    case CMD_BATTUTA_CTAB:
		pSpazio=new CTab;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
    case CMD_BATTUTA_CPERCUSBOX:
		pSpazio=new CPercusBox;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
    case CMD_BATTUTA_CPERCUS2LINES:
		pSpazio=new CPercus2Lines;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
    case CMD_BATTUTA_CVUOTA:
		pSpazio=new CVuota;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
    case CMD_BATTUTA_CBASSOOLD:
		pSpazio=new CBassoOld;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
		//
    case CMD_BATTUTA_CVIOLINO8SOPRA:
		pSpazio=new CViolino8Sopra;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
    case CMD_BATTUTA_CBASSO:
		pSpazio=new CBasso;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
    case CMD_BATTUTA_CVIOLINO8:
		pSpazio=new CViolino8;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
    case CMD_BATTUTA_CBASSO8:
		pSpazio=new CBasso8;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
    case CMD_BATTUTA_CBARITONO:
		pSpazio=new CBaritono;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
    case CMD_BATTUTA_CTENORE:
		pSpazio=new CTenore;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
    case CMD_BATTUTA_CTENORE8:
		pSpazio=new CTenore8;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
    case CMD_BATTUTA_CCONTRALTO:
		pSpazio=new CContralto;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
    case CMD_BATTUTA_CMEZZOSOPRANO:
		pSpazio=new CMezzoSoprano;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
    case CMD_BATTUTA_CSOPRANO:
		pSpazio=new CSoprano;
		((Chiave*)pSpazio)->SetChiavina(TRUE);
		break;
		
		////////////////////////////////////////
		
    case CMD_BATTUTA_ARMCHV_0D:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(DO_maggiore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_1D:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(SOL_maggiore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_2D:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(RE_maggiore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_3D:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(LA_maggiore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_4D:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(MI_maggiore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_5D:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(SI_maggiore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_6D:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(FAd_maggiore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_7D:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(DOd_maggiore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_1B:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(FA_maggiore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_2B:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(SIb_maggiore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_3B:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(MIb_maggiore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_4B:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(LAb_maggiore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_5B:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(REb_maggiore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_6B:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(SOLb_maggiore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_7B:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(DOb_maggiore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
		//Riferite alle armature minori
    case CMD_BATTUTA_ARMCHV_0Dm:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(LA_minore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_1Dm:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(MI_minore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_2Dm:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(SI_minore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_3Dm:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(FAd_minore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_4Dm:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(DOd_minore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_5Dm:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(SOLd_minore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_6Dm:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(REd_minore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_7Dm:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(LAd_minore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_1Bm:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(RE_minore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_2Bm:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(SOL_minore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_3Bm:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(DO_minore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_4Bm:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(FA_minore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_5Bm:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(SIb_minore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_6Bm:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(MIb_minore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    case CMD_BATTUTA_ARMCHV_7Bm:
		pSpazio=new ArmaturaChiave;
		((ArmaturaChiave*)pSpazio)->SetArmatura(LAb_minore);
		((ArmaturaChiave*)pSpazio)->SetVU(0);
		break;
    default:
		return 0;
  }
  
  if(!pSpazio)
	  EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Layer","InsSpazio()");    
  
  if((pbat->GetNumeroFig())>=MAX_FIGURE)
  {
	  EM.Error(MUSICAL_ERR_AREA, "EC_BATFULL","Layer","InsSpazio()");
	  return 0;
  }
  
  pSpazio->SetNumericCode(thisCode);
  
  pSpazio->SetAltezza(AltSpazio);
  pSpazio->SetNStaff(current_staff);
  
  // INSERIMENTO FRA LE FIGURE ESISTENTI.
  if(InsInTesta)
  {
	  // Inserimento in testa.
	  InsertTop(pSpazio);
	  return pSpazio->GetNumericCode();
  }
  else
  {
	  // Inserimento dopo qualche figura.
	  pFig = GetFirstFig(); 
	  found = FALSE;
	  while(pFig!=NULL && !found)
	  {
		  if(pFig->GetNumericCode()==figCode)
			  found = TRUE;
		  else
		  {
			  pFig = GetNextFig(); 
		  }
	  }
	  if(found)
	  {
		  InsertAfterFig(pFig, pSpazio, pbat, current_layer,current_staff); 
		  return pSpazio->GetNumericCode();
	  }
  }
  EM.Error(MUSICAL_ERR_AREA, "EC_FIG_NOTFOUND","Layer","InsSpazio()");
  delete pSpazio;
  pSpazio=NULL; 
  return 0;
}


NumCode Layer::InsRipetizione(Battuta *pbat,ClassID figID, NumCode figCode, Bool InsInTesta, short current_layer,short current_staff)
{
	Ripetizione *pRip;
	Figura *pFig;
	short AltSpazio=0;
	Bool found;
	
	
	switch(figID)
	{
    case CL_RIPTEMPO:
		pRip = new RipTempo;
		break;
    case CL_RIPBATTUTA:
		pRip = new RipBattuta;
		break;
    case CL_RIPMEZZABATTUTA:
		pRip = new RipMezzaBattuta;     
		break;
    default:
		EM.Error(MUSICAL_ERR_AREA, "EC_COD_FIG_UNKNOW","Layer","InsRipetizione()");
		return 0;
	}
	
	if(pRip==NULL)
	{     
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Layer","InsRipetizione()");   
		return 0;
	};
	
	pRip->SetNStaff(current_staff);
	
    
	// INSERIMENTO FRA LE FIGURE ESISTENTI.
	if((pbat->GetNumeroFig())>=MAX_FIGURE)
	{
		EM.Error(MUSICAL_ERR_AREA, "EC_BATFULL","Layer","InsRipetizione()");
		return 0;
	}
	
	if(InsInTesta)
	{
		// Inserimento in testa.
		InsertTop(pRip);
		return pRip->GetNumericCode();
	}
	else
	{
		// Inserimento dopo qualche figura.
		pFig = GetFirstFig();
		found = FALSE;
		while(pFig!=NULL && !found)
		{
			if(pFig->GetNumericCode()==figCode)
				found = TRUE;
			else
			{
				pFig = GetNextFig();
			}
		}
		if(found)
		{
			InsertAfterFig(pFig, pRip, pbat, current_layer,current_staff);
			return pRip->GetNumericCode();
		}
	}  
	EM.Error(MUSICAL_ERR_AREA, "EC_FIG_NOTFOUND","Layer","InsRipetizione()");
	delete pRip;
	pRip=NULL; 
	return 0;
	
}


Bool Layer::GruppoNote2Note(GruppoNote* gruppo, Battuta *pbat, short current_layer) // e' OK
{						
	
	Figura *fig;
	
	fig=gruppo->ExtractFigura(pbat, current_layer);
	while(fig!=NULL)
	{
		InsertAfterF(gruppo,fig);
		fig=gruppo->ExtractFigura(pbat,current_layer);
	}
	DelPtr(gruppo);
	return TRUE;
}


void Layer::EliminaGruppi(Battuta *pbat,short current_layer) // e' OK
{
	Figura *pFig, *pNextFig;
    
	pFig = GetFirstF();
	while(pFig!=NULL)
	{
		pNextFig = GetNextF(pFig);
		if(pFig->GetID()==CL_GRUPPONOTE)
			GruppoNote2Note((GruppoNote*)pFig,pbat, current_layer);
		pFig = pNextFig;
	}
}


unsigned long int Layer::Calc_Fig_Layer() // Calcola il numero di Figure del Layer
{
	Figura *fig;
	numeroFig=0;
    
	fig = GetFirstF();
	while(fig!=NULL)
	{
		numeroFig += fig->GetNumSpaces();
		fig = GetNextF(fig);
	}
	return numeroFig;
}


Bool Layer::GetChiaveLayer(unsigned long pos)
{
	Figura *pfig;
	ClassID id;
	
	pfig = AtPosF(pos);
	while(pfig!=NULL)
	{
		id = pfig->GetID();
		if ((id >= CL_CVIOLINO && id <= CL_CSOPRANO) || (id >= CL_CTENORE8 && id <= CL_CBASSOOLD))
			return TRUE;
		pfig = GetNextF(pfig);
	}
	return FALSE;
}



void Layer::SetupGambi(Battuta *pbat, short current_layer)
{
	Figura *fig;
	Bool gamboupdwn;
	int lgambo;
	int current_staff;
	
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		msgdebug.sprintf("");
	}
	
	
	fig = GetFirstF();
	while(fig!=NULL)
	{
		if((fig->GetID()==CL_NMINIMA)        |
			(fig->GetID()==CL_NSEMIMINIMA)   |
			(fig->GetID()==CL_NCROMA)        |
			(fig->GetID()==CL_NSEMICROMA)    |
			(fig->GetID()==CL_NBISCROMA)     |
			(fig->GetID()==CL_NSEMIBISCROMA) |
			(fig->GetID()==CL_NFUSA)         |
			(fig->GetID()==CL_NSEMIBREVE)    |
			(fig->GetID()==CL_ACCORDO))
		{
			if (((Nota*)fig)->GetLGambo()>0)
			{
				if(((Nota*)fig)->GetNotina())
				{
					((Nota*)fig)->SetGamboUp();
				}
				else
				{
					gamboupdwn = TRUE; 
					current_staff = fig->GetNStaff();
					
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						msgdebug.sprintf("");
					}
					
					
					lgambo = posizionam.PosRule((DrawObject*)fig,(Nota*)fig,pbat,gamboupdwn,current_layer,current_staff);// eliminato ultimo parametro 
					((Nota*)fig)->SetLGambo(lgambo);
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (fig->GetTraceMilla()) {
							posizionam.TestTokenCond((Nota*)fig, (DrawObject*)fig, pbat, gamboupdwn, current_layer);
							
							WDebugMilla(); 
							fig->SetTraceMilla(FALSE);
						}
					}
					
					
				}
			}  
			else 
			{
				if(((Nota*)fig)->GetNotina())
				{
					((Nota*)fig)->SetGamboDwn();
				}
				else
				{
					gamboupdwn = FALSE;
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						msgdebug.sprintf("");
					}
					
					
					current_staff = fig->GetNStaff();
					lgambo = posizionam.PosRule((DrawObject*)fig,(Nota*)fig,pbat,gamboupdwn,current_layer,current_staff);
					((Nota*)fig)->SetLGambo(-lgambo);
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (fig->GetTraceMilla()) {
							posizionam.TestTokenCond((Nota*)fig, (DrawObject*)fig, pbat, gamboupdwn, current_layer);
							
							WDebugMilla(); 
							fig->SetTraceMilla(FALSE);
						}
					}
					
				}	
			}
		} 
		fig = GetNextF(fig);
		
	}
	
} 

void Layer::SetupSimboli(Bool beforeLegature,Battuta *pbat,short current_layer)
{
	Figura *fig;
				
	fig = GetFirstF();
	while(fig!=NULL)
	{
		fig -> Adjust(beforeLegature,pbat,current_layer,fig->GetNStaff());
		fig = GetNextF(fig);
	}
	
}

void Layer::Select()
{
	isSelected = TRUE;
	Draw();
}

void Layer::Deselect()
{
	isSelected = FALSE;
	Draw();
}

void Layer::Raise(int delta_altezza)
{
	Figura* pF=GetFirstF();
	while(pF)
	{
		pF->Raise(delta_altezza);
		pF = GetNextF(pF);
	}
}

void Layer::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(Opaque)
		Scr->SetColor(GetColor());

	Figura *fig;

	Colore oldCol;

	fig = GetFirstF();
	while(fig != NULL)
	{
		fig ->Draw();
		fig = GetNextF(fig);
	}

	if(isSelected)
		DrawContour();

	Scr->SetColor(oldcol);
}

void Layer::DrawContour()
{
	Figura *fig;

	Point LeftTop(10000,0);
	Point RighBot(0,10000);
	//Point center;
	//VUnit top,bot,lft,rgt;
	class Rectangle bounds;

	fig = GetFirstF();
	if(!fig)
		return;

	Scr->SetColor((ColorIcon)SELECTED_COLOR);

	while(fig != NULL) {
		bounds = (class Rectangle)*fig;
		if (bounds.PUpLft.x < LeftTop.x)
			LeftTop.x = bounds.PUpLft.x;
		if (bounds.PUpLft.y > LeftTop.y)
			LeftTop.y = bounds.PUpLft.y;
		if (bounds.PDwnRgt.x > RighBot.x)
			RighBot.x = bounds.PDwnRgt.x;
		if (bounds.PDwnRgt.y < RighBot.y)
			RighBot.y = bounds.PDwnRgt.y;

		fig = GetNextF(fig);
	}

//	Scr->Rect(class Rectangle(LeftTop + Point(-3*SELECTBOX_MARGIN, 3*SELECTBOX_MARGIN), RighBot + Point(3*SELECTBOX_MARGIN, -3*SELECTBOX_MARGIN)));
	Scr->BigRect(class Rectangle(LeftTop + Point(-3*SELECTBOX_MARGIN, 3*SELECTBOX_MARGIN), RighBot + Point(3*SELECTBOX_MARGIN, -3*SELECTBOX_MARGIN)), 3);
}

void Layer::SetPosLayer(Pentagramma *pentagr,Battuta *pBat,int xFigPos,short current_layer)
{
	Figura *fig,*figtmp=NULL;
	short current_staff;
	VUnit prova=0, anchorpos;
	Spazio *figsp;
	VUnit pentY=0;
	
	fig = GetFirstF();
	prova=xFigPos;
	while(fig!=NULL)
	{
		if (fig->GetID()!=CL_SPAZIO )
		{
			prova=xFigPos;
			
			if(fig->GetID()==CL_ARMATURACHIAVE)
			{
				Chiave *ptrStartClef = pBat->GetIntest(fig->GetNStaff())->GetChiave();
				Chiave *ptrClef=NULL;
				ptrClef=ptrStartClef;
				
				tipoArmaturaChiave tmpArmChiave = tipoArmaturaChiave(((ArmaturaChiave*)fig)->GetArmatura() % 15);
				tipoArmaturaChiave tmpIntArmChiavePrec = tipoArmaturaChiave(pBat->GetIntest(fig->GetNStaff())->GetTipoArmatura() % 15);
				
				if(((tmpArmChiave >= SOL_maggiore && tmpArmChiave <= DOd_maggiore) &&
					(tmpIntArmChiavePrec >= FA_maggiore && tmpIntArmChiavePrec <= DOb_maggiore)) ||
					((tmpIntArmChiavePrec >= SOL_maggiore &&tmpIntArmChiavePrec <= DOd_maggiore) &&
					(tmpArmChiave >= FA_maggiore && tmpArmChiave <= DOb_maggiore)))
				{
					((ArmaturaChiave*)fig)->tipoAlt = FALSE;
					((ArmaturaChiave*)fig)->tipoArmPrec = tmpIntArmChiavePrec;
				}
				else
				{
					((ArmaturaChiave*)fig)->tipoAlt = TRUE;
					((ArmaturaChiave*)fig)->tipoArmPrec = tmpArmChiave;
				}
				
				((ArmaturaChiave*)fig)->shiftChiave = ptrClef->GetShift();
				((ArmaturaChiave*)fig)->altezzaChiave = ptrClef->GetAltezza();
				
				((ArmaturaChiave*)fig)->SetVU(ptrClef->GetShift());
				fig->SetPos(pBat,Point(xFigPos,(ptrClef->GetAltezza()-ptrClef->GetShift())*VU_PER_ROW/2));
				
				xFigPos+=UL2VU(fig->GetSp2NextFig());
			} // fine armatura chiave
			else //non nullo e non armatura chiave
			{
				fig->SetPBattuta(pBat); //  Aggiunto per avere un aggancio da parte della figura alla battuta
				current_staff=fig->GetNStaff();
				if(current_staff<0) current_staff = pBat->GetNumberOfStaff()-1;
				if(pentagr!=NULL)
					pentY=pentagr[current_staff].GetAbsPos().y;
				fig->SetPos(NULL,Point(pBat->GetAbsPos().x,pentY)+Point(xFigPos,0));
			}
			fig->SetPBattuta(pBat); //  Aggiunto per avere un aggancio da parte della figura alla battuta
			fig->Adjust(TRUE, pBat, current_layer,current_staff);
		}
		else  // gestione spazio
		{
			if (figtmp!=NULL && figtmp->GetID()==CL_GRUPPONOTE)
			{		
				figsp=(Spazio *)fig;
				anchorpos=UL2VU((((GruppoNote*)figtmp)->GetLastNota()->GetSp2NextFig())*
					figsp->Getrelpos())/100;
				anchorpos+=((GruppoNote*)figtmp)->GetLastNota()->GetAbsPos().x-25;
				current_staff=fig->GetNStaff();
				if(current_staff<0) current_staff = pBat->GetNumberOfStaff()-1;
				if(pentagr!=NULL)
					pentY=pentagr[current_staff].GetAbsPos().y;
				fig->SetPos(NULL,Point(pBat->GetAbsPos().x,pentY)+Point(anchorpos,0));
				fig->Adjust(TRUE, pBat, current_layer,current_staff);
			}
			else
			{
				figsp=(Spazio *)fig;
				anchorpos=((xFigPos-prova)*figsp->Getrelpos())/100;
				anchorpos+=prova;
				current_staff=fig->GetNStaff();
				if(current_staff<0) current_staff = pBat->GetNumberOfStaff()-1;
				if(pentagr!=NULL)
					pentY=pentagr[current_staff].GetAbsPos().y;
				fig->SetPos(NULL,Point(pBat->GetAbsPos().x,pentY)+Point(anchorpos,0));
				fig->Adjust(TRUE, pBat, current_layer,current_staff);
			}
		} // fine gestione spazio
		
		if (fig->GetID()!=CL_SPAZIO)
			figtmp=fig;
		
		// Controllo: se la figura e' una notina vai a vedere la figura
		// seguente e conta le notine finche' non trovi una nota:
		// poi posiziona la notina a xFigPos1-(n.ro notine*costante)
		// se le notine sono beamate e' un problema che mi pongo dopo
		
		if(fig->GetStatus()==GRACED || fig->GetStatus()==GHOSTED )
		{
			VUnit spnotine=2;
			
			Figura *figtmp=fig;
			
			figtmp=GetNextF(figtmp);
			
            // scorri le notine se piu' di una
            while ( figtmp!=NULL && (figtmp->GetStatus()==GRACED || figtmp->GetStatus()==GHOSTED ) ) 
				
			{
				
				if (figtmp!=NULL && (figtmp->GetStatus()==GRACED || figtmp->GetStatus()==GHOSTED ))
					spnotine+=figtmp->GetWidth()/2;
				
				figtmp=GetNextF(figtmp);
				
				
			}
			
			
			
			if(fig->GetID()==CL_GRUPPONOTE)
			{
				GruppoNote *gn=(GruppoNote*)fig;
				if (gn->GetFirstF()!=NULL)
					current_staff=gn->GetFirstF()->GetNStaff();
				if (current_staff<0) 
					current_staff = pBat->GetNumberOfStaff()-1;
				gn->SetPBattuta(pBat);
			}
			else // per gli accordi graced casca qui
			{
				current_staff=fig->GetNStaff();
				if(current_staff<0) current_staff = pBat->GetNumberOfStaff()-1;
			}
			if(pentagr!=NULL)
				pentY=pentagr[current_staff].GetAbsPos().y;
			
			fig->SetPBattuta(pBat); //  Aggiunto per avere un aggancio da parte della figura alla battuta
			fig->SetPos(NULL,Point(pBat->GetAbsPos().x,pentY)+Point(xFigPos-spnotine,0));
			fig->Adjust(TRUE, pBat, current_layer,current_staff);
			xFigPos+=UL2VU(fig->GetSp2NextFig());
			
			fig = GetNextF(fig);
		}  
		else if(fig->GetID()!=CL_SPAZIO && fig->GetID()!=CL_ARMATURACHIAVE)
		{   
			if(fig->GetID()==CL_GRUPPONOTE)
			{
				GruppoNote *gn=(GruppoNote*)fig;
				current_staff=gn->GetFirstF()->GetNStaff();
				if(current_staff<0) current_staff = pBat->GetNumberOfStaff()-1;
				gn->SetPBattuta(pBat);
			}
			else
			{
				current_staff=fig->GetNStaff();
				if(current_staff<0) current_staff = pBat->GetNumberOfStaff()-1;
			}
			fig->SetPBattuta(pBat); //  Aggiunto per avere un aggancio da parte della figura alla battuta
			if(pentagr!=NULL)
				pentY=pentagr[current_staff].GetAbsPos().y;
			fig->SetPos(NULL,Point(pBat->GetAbsPos().x,pentY)+Point(xFigPos,0));
			fig->Adjust(TRUE, pBat, current_layer,current_staff);
			
			xFigPos+=UL2VU(fig->GetSp2NextFig());
			fig = GetNextF(fig);
			
			// ho fatto questa variazione: invece che impostare l'xFigPos
			// con il distanzaFig uguale per tutti si pesca il numero di UL
			// degli attributi di Figura dati dalla giustificazione, divide per
			// 100 per passare da UL a VUnit
			////////////////////////////////////////////////////////
		} 
		else
			fig = GetNextF(fig);
		
	}
	fig = GetFirstFig();
	while(fig!=NULL)
	{
		pBat->AdjustTremolo(fig,current_layer);
		pBat->AdjustGlissato(fig,current_layer);
		fig=GetNextFig();
	}
	return;
}



void Layer::BeamingAuto(Battuta *pbat,short current_staff, short current_layer) // Questo e'quello del Layer
{  
	Tempo tempo;
	float den, num;
	int lay=current_layer;
	
	RANGE coppie[3];
	
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		msgdebug.sprintf("");
	}
	
	
	float nbeat, beat;
	Figura *pfig;
	
	float dur=0;
	Bool coda; 
	NumCode firstNC=0, lastNC=0;
	
	LegatQuadra *pleg=NULL;
	Figura *plegend=NULL;
	float terz=0, valterz=0;
	
	Bool fineterzina=FALSE;
	Bool abilterzine=FALSE;
	
	
	//in questo metodo dovro' richiamare il checkbatt per verificare se la battuta e'
	//consistente dopo l'inserimento della nuova figura.
	//Se la battuta lo e', allora posso procedere con il beaming automatico.
	
	tempo = pbat->GetTempo();
	num = tempo.GetNumeratore();
	den = tempo.GetDenominatore();
	
	// Posso procedere al beaming automatico ricavando anzitutto il numero di beat 
	// in cui deve essere suddivisa la battuta.
				
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		msgdebug.sprintf("");
	}
	
	
	nbeat = posizionam.PosRuleBeam(pbat,NULL,pbat,TRUE,current_layer,coppie,current_staff); //numero di beat // 
	
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		posizionam.TestTokenCond(NULL, pbat, pbat, TRUE, 1);
		
		WDebugMilla(); 
	}
	
	
	beat = (num/nbeat)/den; //valore del singolo beat
	
	abilterzine = posizionam.PosRuleTer(pbat,NULL,pbat,TRUE,current_layer,current_staff); // 
	
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		posizionam.TestTokenCond(NULL, pbat, pbat, TRUE, 1);
		
		WDebugMilla(); 
	}
	
	
	
	pfig=GetFirstFig();
	
	while(pfig!=NULL)
	{
		while (pfig!=NULL)
		{  
			
			//verifico l'esistenza di un gruppo sovrabbondante e ne valuto l'effettiva durata
			//imposto l'attributo fineterzina a TRUE.
			
			pleg=pfig->GetTerzina();
			if(pleg!=NULL)
			{
				terz=0;
				if(!abilterzine)
				{
					if(firstNC!=0 && lastNC!=0)
					{
						Note2GruppoNote(pbat,lay, AUTOMATICO, firstNC, lastNC);
						firstNC=0; 
						lastNC=0;
					}
					firstNC=pfig->GetNumericCode();
				} 
				dur+=GetDurataTerz(pfig, pleg, den, num);
				plegend=pleg->GetPFigEnd();
				lastNC=plegend->GetNumericCode();
				if(firstNC!=0 && lastNC!=0)
				{
					fineterzina=TRUE;
				}
			} 
			else if(pfig->IsANote())
			{
				coda = ((Nota*)pfig)->GetCoda();
				if(coda && firstNC==0)  firstNC=pfig->GetNumericCode();
				else if(coda && firstNC!=0)  lastNC=pfig->GetNumericCode();
				else if(!coda && firstNC!=0 && lastNC!=0) 
				{
					Note2GruppoNote(pbat,lay, AUTOMATICO, firstNC, lastNC);
					firstNC=0; 
					lastNC=0;
				} 
				dur += pfig->GetDurata();
			} 
			else dur += pfig->GetDurata();
			if (((int)(dur/beat)*beat)==dur)   //controllo che esegue incremento nota fino
			{                                  //al raggiungimento del beat o di un suo
				pfig=GetNextFig();   //multiplo.
				if (fineterzina)
				{
					Note2GruppoNote(pbat,lay, AUTOMATICO, firstNC, lastNC);
					firstNC=0; 
					lastNC=0;
					fineterzina=FALSE;
				}	
				break;
			}
			pfig=GetNextFig();
			if (fineterzina && !abilterzine)
			{
				Note2GruppoNote(pbat,lay, AUTOMATICO, firstNC, lastNC);
				firstNC=0; 
				lastNC=0;
				fineterzina=FALSE;
			}	
		}
		if(!coda && firstNC!=0 && lastNC==0) firstNC=0;
		if(firstNC!=0 && lastNC!=0) 
		{
			Note2GruppoNote(pbat,lay, AUTOMATICO, firstNC, lastNC);
			firstNC=0; 
			lastNC=0;
		}
		dur=0;
		firstNC=0;
		lastNC=0;
	}
}

float Layer::GetTotalDuration(float num, float den)
{
	float time=0.;
	Figura *pfig=GetFirstFig();
	while (pfig!=NULL)
	{
		if(pfig->GetID()!=CL_SPAZIO)
		{
			time+=pfig->GetDurata(den, num);
		}
		pfig=GetNextFig();
	}
	return time;
}

const float Layer::GetAltezzaMedia(){
	float sum = 0.0;
	unsigned int n = 0;
	Figura *pfig=GetFirstFig();
	while (pfig!=NULL)
	{
		if(pfig->GetDurata())
		{
			if (pfig->GetID() == CL_ACCORDO)
			{
				Nota* pN = ((Accordo*)pfig)->GetFirstNote();
				while (pN)
				{
					n++;
					sum += (pN->GetAbsPos()).y;
					pN = ((Accordo*)pfig)->GetNextNote(pN);
				}
			} 
			else
			{
				n++;
				sum += (pfig->GetAbsPos()).y;
			}
		}
		pfig=GetNextFig();
	}

	assert(n > 0); //TODO
	return sum/n;
}

//*BIAN-VERR*
Figura* Layer::GetFigura(NumCode figCode)
{
	Figura *pFig; 
	pFig = GetFirstFig(); 
	Bool found = FALSE;
	while(pFig!=NULL && !found)
	{
		if(pFig->GetNumericCode()==figCode)
			found = TRUE;
		else
		{
			pFig = GetNextFig(); 
		}
	}
	return pFig;
}
//*BIAN-VERR*

//------------------------------------------------//

void Layer::SetColor(const Colore & color){
	Opaque = true;
	CurColor = color;
}

void Layer::UnsetColor(){
	Opaque = false;
}

const Colore & Layer::GetColor(){
	//assert(Opaque);
	return CurColor;
}

bool Layer::isOpaque(){
	return Opaque;
}