/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: listafig.cpp,v 1.1 2005/07/25 10:51:09 bellinpo Exp $ */

#include "bat/listafig.hpp"
#include "fig/gruppono.hpp"
#include "fig/accordo.hpp"
#include "int/legatqua.hpp"

ListaFigure::ListaFigure()
{
	CodeCounter=1;
	this->FlagAcciaccatura=NULL;
	this->NextAcciaccaturaLayer=NULL;
	
}

ListaFigure::~ListaFigure()
{
	
#ifdef DEBUG_FREE
	if (tipoLeggio!=LEGGIO_MASAE)
	{
		FILE *pippo;
		
		pippo = fopen("DEAL_NET.txt","a");
		fprintf(pippo,"\n\n\n ---- non masae   DEALLOCO ListaFigure  ----------------\n\n\n");
		fclose(pippo);
	}
#endif
	
	Free();
}
void ListaFigure::Free()
{
	ClearDeep(); 
}

void ListaFigure::InsertEnd(DrawObject *newobj)
{
	Lista::InsertEnd(newobj);
	UpdateCode((Figura*)newobj);
//	((Figura*)newobj)->SetNumericCode(CodeCounter);
//	CodeCounter++;
}

void ListaFigure::InsertEndSimple(DrawObject *newobj)
{
	Lista::InsertEnd(newobj);
}

void ListaFigure::InsertTop(DrawObject *newobj)
{
	Lista::InsertTop(newobj);
	UpdateCode((Figura*)newobj);
//	((Figura*)newobj)->SetNumericCode(CodeCounter);
//	CodeCounter++;
}


Figura *ListaFigure::ChangeF(Figura *p1, Figura *p2)
{
	UpdateCode(p2);
	//p2->SetNumericCode(CodeCounter++);
	return (Figura*) Change(p1,p2);
}

Figura *ListaFigure::ChangeFSimple(Figura *p1, Figura *p2)
{
	return (Figura*) Change(p1,p2);
}

Figura *ListaFigure::GetFirstF(void)
{
	return (Figura*) GetFirst();
}

Figura *ListaFigure::GetLastF(void) 
{
	return (Figura*) GetLast();
}

Figura *ListaFigure::GetNextF(Figura *pf)
{
	return (Figura*) GetNext(pf);
}

Figura *ListaFigure::GetNextFigura(Figura *pf)
{
	Figura *ptr = GetFirstF();
	Figura *pFGruppo;
	
	while(ptr!=NULL)
	{
		if (ptr==pf) 
		{
			ptr = GetNextF(pf);
			if (ptr==NULL) return NULL;
			
			if (ptr->GetID()==CL_GRUPPONOTE)
				ptr = ((GruppoNote*)ptr)->GetFirstF();
			
			return ptr;
		}
		if (ptr->GetID()==CL_GRUPPONOTE)
		{
			pFGruppo = ((GruppoNote*)ptr)->GetFirstF();
			while(pFGruppo!=NULL)
			{
				if (pFGruppo==pf) 
				{
					pFGruppo = ((GruppoNote*)ptr)->GetNextF(pFGruppo);
					if (pFGruppo==NULL)
					{
						pFGruppo=GetNextF(ptr);
						if(pFGruppo!=NULL && pFGruppo->GetID()==CL_GRUPPONOTE)
							return ((GruppoNote*)pFGruppo)->GetFirstF();
						return pFGruppo;
					}
					else
						return pFGruppo;
				}
				pFGruppo = ((GruppoNote*)ptr)->GetNextF(pFGruppo);
			}
		}
		
		if (ptr->GetID()==CL_ACCORDO)
		{
			pFGruppo = ((Accordo*)ptr)->GetFirstF();
			while(pFGruppo!=NULL)
			{
				if (pFGruppo==pf) 
				{
					return GetNextF(ptr);
				}
				pFGruppo = ((Accordo*)ptr)->GetNextF(pFGruppo);
			}
		}
		
		ptr = GetNextF(ptr);
	}
	
	return NULL;
}

Figura *ListaFigure::GetPrevF(Figura *pf)
{
	return (Figura*) GetPrev(pf);
}

Figura *ListaFigure::DelFirstF(void)
{
	return (Figura*) DelFirst();
}

Figura *ListaFigure::DelLastF(void)
{
	return (Figura*) DelLast();
}

Figura *ListaFigure::AtPosF(unsigned long i)
{
	return (Figura*) AtPos(i);
}

Figura* ListaFigure::GetFig(NumCode nc)
{
	Figura *ptr = GetFirstF();
	
	while(ptr!=NULL)
	{
		if ((ptr->GetNumericCode()) == nc)
			return ptr;
		ptr = GetNextF(ptr);
	}
	
	return NULL;
}

// Cerca anche nei gruppi !
Figura* ListaFigure::GetFigTotal(NumCode nc)
{
	Figura *ptr = GetFirstFig();
	
	while(ptr!=NULL)
	{
		if ((ptr->GetNumericCode()) == nc)
			return ptr;
		ptr = GetNextFig();
	}
	
	return NULL;
}

// Cerca anche negli accordi !
Figura* ListaFigure::GetFigAll(NumCode nc)
{
	Figura *ptr = GetFirstFig();
	Figura *ptr2;
	Accordo * pAccordo;
	
	while(ptr!=NULL)
	{
		// controllo che non sia un accordo !
		if (ptr->GetID()==CL_ACCORDO) {
			pAccordo=(Accordo *)(ptr);
			ptr2=(pAccordo)->GetFig(nc);
			if (ptr2!=NULL) return ptr2;
		} 
		
		if ((ptr->GetNumericCode()) == nc)
			return ptr;
		ptr = GetNextFig();
	}
	
	return NULL;
}

void ListaFigure::InsertAfterF(Figura *pf1,Figura *pf2)
{
	Node *n;
	n=FindNodePtr(pf1);
	if(n!=NULL)
		InsertAfter(n,pf2);

	UpdateCode(pf2);
/*	if(pf2->GetNumericCode()==0)
	{
		pf2->SetNumericCode(CodeCounter);
		CodeCounter++;
	}*/
}

void ListaFigure::InsertAfterFSimple(Figura *pf1,Figura *pf2)
{
	Node *n;
	n=FindNodePtr(pf1);
	if(n!=NULL)
		InsertAfter(n,pf2);
	return;
}

Bool ListaFigure::FigPtr2SymPath(SymPath& symPath, Figura *figptr)
{
	Bool trovato=FALSE;
	
	Figura *ptr=(Figura*)GetFirst();
	
	while((!trovato) && (ptr!=NULL))
	{
		symPath.Array[FIGURA1_PATH_LEVEL]=(ptr->GetNumericCode());  
		symPath.Array[FIGURA2_PATH_LEVEL]=0;  
		symPath.Array[FIGURA3_PATH_LEVEL]=0;  
		
		switch(ptr->GetID())
		{
		case CL_ACCORDO:
			if(ptr==figptr)
                trovato=TRUE;
			else
                trovato = ((Accordo*)ptr)->FigPtr2SymPath(symPath,figptr);
			break;
		case CL_GRUPPONOTE:
			trovato = ptr->FigPtr2SymPath(symPath,figptr);
			break;
		default:
			if(ptr==figptr)
                trovato=TRUE;
		} 
		
		ptr = (Figura*)GetNext((DrawObject*)ptr);
	}      
    
    return trovato;
}

void ListaFigure::SetCodeCounter(NumCode nc)
{
	CodeCounter = nc;
}

NumCode ListaFigure::GetCodeCounter()
{
	return CodeCounter;
}

Bool ListaFigure::FindFig(Figura *pf)
{
	Figura *ptr = GetFirstF();
	Figura *pFGruppo;
	
	while(ptr!=NULL)
	{
		if (ptr==pf) return TRUE;
		if (ptr->GetID()==CL_GRUPPONOTE)
		{
			pFGruppo = ((GruppoNote*)ptr)->GetFirstF();
			while(pFGruppo!=NULL)
			{
				if (pFGruppo==pf) return TRUE;
				pFGruppo = ((GruppoNote*)ptr)->GetNextF(pFGruppo);
			}
		}
		if (ptr->GetID()==CL_ACCORDO)
		{
			pFGruppo = ((Accordo*)ptr)->GetFirstF();
			while(pFGruppo!=NULL)
			{
				if (pFGruppo==pf) return TRUE;
				pFGruppo = ((Accordo*)ptr)->GetNextF(pFGruppo);
			}
		}
		
		ptr = GetNextF(ptr);
	}
	
	return FALSE;
}

Figura *ListaFigure::GetLastFig(void)
{
	Figura* fig = GetLastF();
	if(fig && fig->GetID() == CL_GRUPPONOTE)
		return ((GruppoNote*)fig)->GetLastF();

	return fig;
}

Figura *ListaFigure::GetPrevFig(Figura* pF)
{
	if(!pF) return NULL;


	Figura* fig = pF->GetParentFig();
	if(fig && fig->GetID() == CL_GRUPPONOTE)
	{
		pF = ((GruppoNote*)fig)->GetPrevF(pF);
		if(!pF)
			return GetPrevFig(fig);
		else
			return pF;
	}

	pF = GetPrevF(pF);
	if(pF && pF->GetID() == CL_GRUPPONOTE)
		return ((GruppoNote*)pF)->GetLastF();

	return pF;
}

Figura *ListaFigure::GetNextFig(Figura* pF)
{
	if(!pF) return NULL;


	Figura* fig = pF->GetParentFig();
	if(fig && fig->GetID() == CL_GRUPPONOTE)
	{
		pF = ((GruppoNote*)fig)->GetNextF(pF);
		if(!pF)
			return GetNextFig(fig);
		else
			return pF;
	}

	pF = GetNextF(pF);
	if(pF && pF->GetID() == CL_GRUPPONOTE)
		return ((GruppoNote*)pF)->GetFirstF();

	return pF;
}

Figura *ListaFigure::GetFirstFig(void)
{
	if(first==NULL)
		return NULL;
	
	Figura* fig=(Figura*) first->ObjectPtr;
	lastNode=first;
	
	if(fig!=NULL && fig->GetID()==CL_GRUPPONOTE)
	{
		lastGruppoNote=(GruppoNote*)fig;
		return lastGruppoNote->GetFirstFig();
	}
	lastGruppoNote=NULL;
	return fig;
}

Figura *ListaFigure::GetNextFig(void)
{
	if(lastGruppoNote!=NULL)
	{
		Figura* fig=lastGruppoNote->GetNextFig();
		if(fig!=NULL)
			return fig;
	}
	lastGruppoNote=NULL;
	
	if(lastNode==NULL) return NULL;	  
	lastNode=lastNode->Next;
	if(lastNode==NULL)
		return NULL;
	
	Figura *fig=(Figura*)lastNode->ObjectPtr;
	if(fig==NULL)
		return NULL;
	
	if(fig->GetID()==CL_GRUPPONOTE)
	{
		lastGruppoNote=(GruppoNote*)fig;
		return lastGruppoNote->GetFirstFig();
	}
	return fig;
}

GruppoNote *ListaFigure::GetLastGruppoNote()
{
	return lastGruppoNote;
}

void ListaFigure::DelPtrFig(Figura *pf1, Battuta *pbat, short current_layer, short current_staff)
{
	if(lastGruppoNote!=NULL)
	{
		if(!lastGruppoNote->DelFigura(pf1))
		{
			// se la figura da cancellare e' nel gruppo il gruppo viene sgruppato e cancellata la figura.
			Figura *fig=lastGruppoNote->ExtractFigura(pbat, current_layer);
			while(fig!=NULL)
			{
				InsertAfterF(lastGruppoNote,fig);
				fig=lastGruppoNote->ExtractFigura(pbat, current_layer);
			}
			DelPtr(lastGruppoNote);
		}
		else
		{
			return;
		}
	}
	DelPtr(pf1);
}

void ListaFigure::InsertAfterFig(Figura *pf1,Figura *pf2, Battuta *pbat, short current_layer, short current_staff)
{
	if(lastGruppoNote!=NULL)
	{
		// prova ad inserire la figura (solo lo spazio) nel gruppo
		if(lastGruppoNote->GetLast()==pf1)
		{
			// se la figura deve essere aggiunta dopo l'ultima figura del gruppo viene inserita
			// dopo il gruppo.
			InsertAfterF(lastGruppoNote,pf2);
		}
		else if(lastGruppoNote->AddAfterFigura(pf1,pf2))
		{
			UpdateCode(pf2);
			/* se viene aggiunto lo spazio gli assegna il codice a livello di battuta
			if(pf2->GetNumericCode()==0)
			{
				pf2->SetNumericCode(CodeCounter);
				CodeCounter++;
			}*/
		}
		else
		{
			// se la figura da inserire non e' uno spazio e deve essere inserita all'interno del gruppo
			// il gruppo viene sgruppato e inserita la figura.
			Figura *fig=lastGruppoNote->ExtractFigura(pbat, current_layer);
			while(fig!=NULL)
			{
				InsertAfterF(lastGruppoNote,fig);
				fig=lastGruppoNote->ExtractFigura(pbat, current_layer);
			}
			DelPtr(lastGruppoNote);	  
			InsertAfterF(pf1,pf2);
		}
	}
	else
		InsertAfterF(pf1,pf2);
}

int ListaFigure::GetPosFig(DrawObject *pf)
{
	int pos=0;
	Figura *fig;
	
	fig=GetFirstFig();
	while(fig!=NULL && fig!=pf)
	{
		pos++;
		fig=GetNextFig();
	}
	if(fig==pf)
		return pos;
	return -1;
}

Figura *ListaFigure::AtPosFig(int pos)
{
	Figura *fig;
	int p=0;
	
	fig=GetFirstFig();
	while(fig!=NULL && pos!=p)
	{
		p++;
		fig=GetNextFig();
	}
	if(pos==p)
		return fig;
	return NULL;
}

float ListaFigure::GetDurataTerz(Figura *pfig1, LegatQuadra *pleg, float den, float num)
{
	Figura *plegend=NULL;
	
	int nterz;
	float notaval, terz=0, valterz=0; 
	
	valterz=0;
	plegend=pleg->GetPFigEnd();
	nterz=pleg->GetNumero();
	while (pfig1!=plegend) 
	{
		if(pfig1!=NULL)
		{
			notaval = pfig1->GetDurata(den, num);
			terz = terz+notaval;
			pfig1=GetNextFig();
		}
		else return terz;    
	}
	terz=terz+pfig1->GetDurata(den, num);
	valterz=pleg->GetDurataTerzina(terz, den, num);
	return valterz;
}

UL ListaFigure::GetSpTotale()
{
	// deve restituire l'intero spazio dalla prima nota del gruppo note
	// al punto dove verra' posizionata la figura successiva al gruppo note,
	// al posizionamento corretto del gruppo note ci pensa l'adjust
	
	
	// inizializza var totale spazio che viene ritornata
	UL totale=0;
	// GruppoNote deriva sia da Figura che da ListaFigure, la Figura 
	// precedente al grupponote restituisce la distanza fino alla
	// prima figura del grupponote, mentre questo metodo somma tutte 
	// le distanze dentro al grupponote
	
    
	Node *pnode; 
	Figura *pfig;  
	
	pnode=first;
	while(pnode!=NULL)
	{
		pfig=(Figura*)pnode->ObjectPtr;
		totale+=pfig->GetSp2NextFig();
		
		pnode=pnode->Next;
	}
	return totale;
}

// non autolinebreak e direttoriale
UL ListaFigure::GetSpTotalePar(int autolb, int dirororch)
{
	// deve restituire l'intero spazio dalla prima nota del gruppo note
	// al punto dove verra' posizionata la figura successiva al gruppo note,
	// al posizionamento corretto del gruppo note ci pensa l'adjust
	
	
	// inizializza var totale spazio che viene ritornata
	UL totale=0;
	// GruppoNote deriva sia da Figura che da ListaFigure, la Figura 
	// precedente al grupponote restituisce la distanza fino alla
	// prima figura del grupponote, mentre questo metodo somma tutte 
	// le distanze dentro al grupponote
	
    
	Node *pnode; 
	Figura *pfig;  
	
	pnode=first;
	while(pnode!=NULL)
	{
		pfig=(Figura*)pnode->ObjectPtr;
		totale+=pfig->GetSp2NextFigPar(autolb, dirororch);
		
		pnode=pnode->Next;
	}
	return totale;
}

void ListaFigure::GetMinMaxStaff(short& minStaff, short& maxStaff)
{
	Figura *pFig;
	Node *pnode;
	short figMinStaff,figMaxStaff;
	
	minStaff=4;
	maxStaff=-1;
	
	pnode = first;
	while (pnode!=NULL)
	{
		pFig = (Figura*)pnode->ObjectPtr;
		pFig->GetMinMaxStaff(figMinStaff,figMaxStaff);
		if (figMinStaff<minStaff)
		{
			minStaff=figMinStaff; 
		}
		if(figMaxStaff>maxStaff)
		{
			maxStaff=figMaxStaff; 
		}
		pnode = pnode->Next;
	}  
}

void ListaFigure::SetFlagAcciaccatura(Figura *FPtr){
	
	FlagAcciaccatura=FPtr;
}

Figura* ListaFigure::GetFlagAcciaccatura(){

	return FlagAcciaccatura;
}

void ListaFigure::SetNextAcciaccaturaLayer(ListaFigure *NextAccLayer){

	NextAcciaccaturaLayer=NextAccLayer;
}

ListaFigure* ListaFigure::GetNextAcciaccaturaLayer(){
	
	return NextAcciaccaturaLayer;
}

void ListaFigure::UpdateCode(Figura *newobj)
{
	NumCode ref = newobj->GetNumericCode();
	if (ref > CodeCounter)
		CodeCounter = ref+1;
	else
		newobj->SetNumericCode(CodeCounter++);
}