/* ISO_HEADER_START */

/*********************************************************************

 This software module was originally developed by
 
  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)
  
   in the course of development of the MPEG-4 standard.
   This software module is an implementation of a part of one or more
   MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
   of the MPEG-4 standard free license to this software module or
   modifications thereof for use in hardware or software products
   claiming conformance to MPEG-4. Those intending to use this software
   module in hardware or software products are advised that its use may
   infringe existing patents.  The original developer of this software
   module and his/her company, the subsequent editors and their
   companies, and ISO/IEC have no liability for use of this software
   module or modifications thereof in an implementation.  Copyright is
   not released for non MPEG-4 conforming products. The DISIT-DSI and 
   EXITECH retains full right to use the code for its own purpose,
   assign or donate the code to a third party and to inhibit third
   parties from using the code for non MPEG-4 conforming products.  This
   copyright notice must be included in all copies or derivative
   works. Copyright (c) 2005.
   
***********************************************************************/

/* ISO_HEADER_END */

/* $Id: listafig.hpp,v 1.1 2005/07/25 10:51:09 bellinpo Exp $ */

#ifndef LISTAFIG_HPP
#define LISTAFIG_HPP

#include <stdio.h>

#include "fig/figura.hpp"      
#include "lst/lista.hpp"

class GruppoNote;

class MoodsMidiWrite;

class ListaFigure: public Lista
{
private:
	void UpdateCode(Figura *newobj);	//FFF for undos

protected:
	NumCode CodeCounter;
    // contatore per assegnare il codice numerico ad ogni oggetto in lista
	Node* lastNode;
    // ultimo nodo visitato con GetFirstFig/GetNextFig
	GruppoNote* lastGruppoNote;
    // ultimo gruppo visistato con GetFirstFig/GetNextFig
	Figura* FlagAcciaccatura;
    // diverso da null se la prima nota dell stesso layer nella battuta successiva  una acciaccatura
	ListaFigure* NextAcciaccaturaLayer;
	//memorizza l'indirizzo della lista figure dell'acciaccatura se questa  nella battuta successiva.
    
public:  
	ListaFigure();
    // Costruttore che pone CodeCounter=1.
	~ListaFigure();
    // Distruttore.
	
	Figura* GetFlagAcciaccatura();
	void SetFlagAcciaccatura(Figura* FPtr);
	ListaFigure* GetNextAcciaccaturaLayer();
	void SetNextAcciaccaturaLayer(ListaFigure* NextAccLayer);
	
	
    void Suona(MoodsMidiWrite *);
	void SuonaTremolo(MoodsMidiWrite *mmw,Figura* FPtr, Figura* NextFPtr);
	
	void Free();
    // dealloca la lista.
	
	void InsertEnd(DrawObject *newobj);
    // inserisce un elemento in fondo alla lista
	void InsertEndSimple(DrawObject *newobj);
    // inserisce una figura senza cambiare il codice numerico !
	void InsertTop(DrawObject *newobj);
    // Inserisce un elemento in cima alla lista
	
	Figura *GetFirstF(void) ;
	// restituisce il primo elemento della lista
	Figura *GetLastF(void) ;
	// restituisce l'ultimo elemento della lista
	Figura *GetNextF(Figura *pf);
	// restituisce l'elemento successivo di do
	Figura *GetPrevF(Figura *pf);    
	// restituisce l'elemento precedente di do
	
	void InsertAfterF(Figura *pf1,Figura *pf2);
	// inserisce figura di pf2 dopo figura di pf1.
	void InsertAfterFSimple(Figura *pf1,Figura *pf2);
	// inserisce figura come sopra ma non rienumera i codici numerici
	
	Figura *DelFirstF(void);
	// cancella il primo elemento della lista
	Figura *DelLastF(void);
	// cancella l'ultimo elemento della lista
	Figura *AtPosF(unsigned long i);
	// restituisce l'elemento i-esimo della lista (i>=0 && i<numobject)
	Figura *ChangeF(Figura *p1, Figura *p2);
	// Sostituisce la figura p1 con la figura p2
	Figura *ChangeFSimple(Figura *p1, Figura *p2);
	
	Figura* GetFig(NumCode nc);
	// Ritorna la figura con NumericCode nc
	Figura* GetFigTotal(NumCode nc);
	// Scandisce anche i gruppi di note !
	Figura* GetFigAll(NumCode nc);
	
	Figura *GetNextFigura(Figura *pf);
	// restituisce l'elemento successivo di pf 
	// (cerca anche nei gruppi e negli accordi).
	
	Bool FindFig(Figura *pf);
	// restituisce True se pf e' nella lista di figure (cerca anche nei gruppi di
	// note e negli accordi).
	Bool FigPtr2SymPath(SymPath&, Figura*);
	// ricostruisce il PATH di una figura a partire dal puntatore.
	
	void SetCodeCounter(NumCode);
	NumCode GetCodeCounter();
	
	/////////////////////////////////////////////////////////////////////////
	
	Figura *GetFirstFig(void);
	// restituisce la prima figura della lista entrando anche nei gruppi note
	Figura *GetNextFig(void);
	// restituisce l'elemento successivo 
	Figura *GetPrevFig(Figura* pF);
	Figura *GetNextFig(Figura* pF);
	Figura *GetLastFig(void);

	////////////////////////////////////////////////////////////////////////
	// questi metodi sono utili per la spaziatura perche' permettono di
	// entrare dentro i gruppi note  e ottenere le varie figure contenute.
	// cioe' il grupponote (note beamate) e' un contenitore di figure
	/////////////////////////////////////////////////////////////////////////
	
	GruppoNote* GetLastGruppoNote(void);
	// restituisce il puntatore al gruppo note che eventualmente contiene l'ultima figura
	// estratta con GetFirstFig/GetNextFig. Ritorna NULL se tale gruppo non esiste.
	void InsertAfterFig(Figura *pf1,Figura *pf2, Battuta *pbat, short current_layer,short current_staff);
	// inserisce figura pf2 dopo figura di pf1 nella battuta se non esiste un grupponote.
	void DelPtrFig(Figura *pf1, Battuta *pbat, short current_layer,short current_staff);
	// cancella la figura nella battuta o nell'ultimo gruppo note.
	int GetPosFig(DrawObject* pf);
	// trova la posizione della figura nella lista delle figura estesa con i gruppi note.
	Figura *AtPosFig(int pos);
	// trova la figura in posizione pos nella lista delle figura estesa con i gruppi note.
	float GetDurataTerz(Figura *pfig1, LegatQuadra *pleg, float den, float num);
	
	
	
	UL GetSpTotale();
	
	UL GetSpTotalePar(int aultolb, int dirororch);
	// autolb =0 autolinebreaking automatico =1 disabilitato
	// dirororch =0 direttoriale =1 orchestrale
	
	void GetMinMaxStaff(short& minStaff, short& maxStaff);
	
	int SaveXML(FILE *fp);
};

#endif //LISTAFIG_HPP
