/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: tempo.cpp,v 1.1 2005/07/25 10:51:09 bellinpo Exp $ */


#include "bat/tempo.hpp"
#include <string.h>

Tempo::Tempo()
{
	ID=CL_TEMPO;
	Numeratore.SetFont(nMusicFont);
	Denominatore.SetFont(nMusicFont);
	LetteraC = FALSE;
}

void Tempo::SetVU2Rgt(VUnit vu) 
{TEMPO2Rgt=vu;};  

VUnit Tempo::GetVU2Up() 
{return 4*VU_PER_ROW;};

VUnit Tempo::GetVU2Dwn() 
{return 0;};

VUnit Tempo::GetVU2Lft() 
{return 0;};

VUnit Tempo::GetVU2Rgt() 
{return TEMPO2Rgt;};


void Tempo::SetVU()
{
	if(LetteraC)
	{
		SetVU2Rgt(WDT_TEMPO_C);
	}
	else
	{
		SetVU2Rgt(Numeratore.GetVU2Rgt());
		if(Denominatore.GetVU2Rgt()>GetVU2Rgt()) 
			SetVU2Rgt(Denominatore.GetVU2Rgt());
	}
}


Bool Tempo::HasSameDesc(char tmp[])
{
	char tmpArray[6];
    // Vettore temporaneo per copia stringa
	Bool risult=FALSE;
	
	Bool composta = FALSE;
	for (short k=0;tmp[k] != 0;k++)
		if (tmp[k]=='')
			composta = TRUE;
		if (composta)     // contiene 
		{
			// Rappresentazione esterna diversa da rappr. interna
			wxString espressione;
			for (k=0;tmp[k] != 0;k++)
				espressione += tmp[k];
			//Costruisco il tempo esterno ed interno
			size_t mu = espressione.Index('');
			wxString tmp_est = espressione.Left(mu);
			wxString tmp_int = espressione.Right(espressione.Length() - mu - 1);
			//Li valido entrambi
			if ((Validator(tmp_est,false) * Validator(tmp_int,true)) == 0)
				return true;
			//Costruisco i quattro campi
			size_t primo_slash = tmp_est.Index('/');
			size_t secondo_slash = tmp_int.Index('/');
			char numesterno[20], numinterno[20], denesterno[20], deninterno[20];
			sprintf(numesterno, "%s",tmp_est.Left(primo_slash));
			sprintf(denesterno, "%s",tmp_est.Right(tmp_est.Length() - primo_slash - 1));
			sprintf(numinterno, "%s",tmp_int.Left(secondo_slash));
			sprintf(deninterno, "%s",tmp_int.Right(tmp_int.Length() - secondo_slash - 1));
			
			risult = (strcmp(numesterno,(char*)Numeratore.GetTxt())==0);
			risult = risult && (strcmp(denesterno,(char*)Denominatore.GetTxt()) == 0);
			risult = risult && (atoi(numinterno) == Numeratore.GetNumero());
			risult = risult && (atoi(deninterno) == Numeratore.GetNumero());
			
			return risult;
		}
		
		else 
		{
			//BBBBBBBB Classico
			// Esamina i casi "c" e "C".
			if(tmp[1]=='\0')
			{
				if(*tmp=='c') 				{
					if ( (Numeratore.GetNumero() == 4) && (Denominatore.GetNumero() == 4) &&
						LetteraC )
						return TRUE;
				}
				if(*tmp=='C')
				{
					if ( (Numeratore.GetNumero() == 2) && (Denominatore.GetNumero() == 2) &&
						LetteraC )
						return TRUE;
				}
				return FALSE;
			}
			
			// Esamina le stringhe del tipo "num/den".
			strncpy(tmpArray, tmp, 5);
			tmpArray[5] = '\0';
			if(tmpArray[1]=='/' && (tmpArray[3]=='\0' || tmpArray[4]=='\0'))
				// Numeratore con un carattere e denominatore con 1 o 2.
			{
				tmpArray[1] = '\0';
				
				risult = (strcmp(tmpArray,(char*)Numeratore.GetTxt()) == 0);
				risult = risult && (strcmp(tmpArray+2,(char*)Denominatore.GetTxt()) == 0);
			}
			else if(tmpArray[2]=='/')
			{
				tmpArray[2] = '\0';
				
				risult = (strcmp(tmpArray,(char*)Numeratore.GetTxt()) == 0);
				risult = risult && (strcmp(tmpArray+3,(char*)Denominatore.GetTxt()) == 0);
			}
			
			return risult;
		}
}

Bool Tempo::SetTempo(char tmp[])
{
	char tmpArray[6];
    // Vettore temporaneo per copia stringa
	Bool risult=FALSE;  
	
	Bool composta = FALSE;
	for (short k=0;tmp[k] != 0;k++)
		if (tmp[k]=='')
			composta = TRUE;
		if (composta)     // contiene 
		{
			// Rappresentazione esterna diversa da rappr. interna
			wxString espressione;
			for (k=0;tmp[k] != 0;k++)
				espressione += tmp[k];
			//Costruisco il tempo esterno ed interno
			size_t mu = espressione.Index('');
			wxString tmp_est = espressione.Left(mu);
			wxString tmp_int = espressione.Right(espressione.Length() - mu - 1);
			//Li valido entrambi
			if ((Validator(tmp_est,false) * Validator(tmp_int,true)) == 0)
				return false;
			//Costruisco i quattro campi
			size_t primo_slash = tmp_est.Index('/');
			size_t secondo_slash = tmp_int.Index('/');
			char numesterno[20], numinterno[20], denesterno[20], deninterno[20];
			sprintf(numesterno, "%s",tmp_est.Left(primo_slash));
			sprintf(denesterno, "%s",tmp_est.Right(tmp_est.Length() - primo_slash - 1));
			sprintf(numinterno, "%s",tmp_int.Left(secondo_slash));
			sprintf(deninterno, "%s",tmp_int.Right(tmp_int.Length() - secondo_slash - 1));
			
			risult = Numeratore.SetExternal((unsigned char *)numesterno);
			risult = risult && (Denominatore.SetExternal((unsigned char *)denesterno));
			risult = risult && (Numeratore.SetInternal((unsigned char *)numinterno));
			risult = risult && (Denominatore.SetInternal((unsigned char *)deninterno));

			if (risult)
			{
				((numesterno == "c") || (numesterno == "c")) ? (LetteraC = TRUE) : (LetteraC = FALSE);
			}
			return risult;			
		}
		
		else {
			//Caso classico
			
			// Esamina i casi "c" e "C".
			if(tmp[1]=='\0')
			{
				if(*tmp=='c')
				{
					Numeratore.SetNumero(4);
					Denominatore.SetNumero(4);
					LetteraC = TRUE;
					return TRUE;
				}
				if(*tmp=='C')
				{
					Numeratore.SetNumero(2);
					Denominatore.SetNumero(2);
					LetteraC = TRUE;
					return TRUE;
				}
				return FALSE;
			}
			
			// Esamina le stringhe del tipo "num/den".
			strncpy(tmpArray, tmp, 5);
			tmpArray[5] = '\0';
			if(tmpArray[1]=='/' && (tmpArray[3]=='\0' || tmpArray[4]=='\0'))
				// Numeratore con un carattere e denominatore con 1 o 2.
			{
				tmpArray[1] = '\0';
				risult = Numeratore.SetTxt((unsigned char*)tmpArray)
					&& Denominatore.SetTxt((unsigned char*)(tmpArray+2));
			}
			if(tmpArray[2]=='/')
			{
				tmpArray[2] = '\0';
				risult = Numeratore.SetTxt((unsigned char*)tmpArray) 
					&& Denominatore.SetTxt((unsigned char*)(tmpArray+3));
			}
			// Rifiuta i numeri nulli.
			if(risult)
			{
				if(Numeratore.GetNumero()==0 || Denominatore.GetNumero()==0)
					risult=FALSE;
			}
			if(risult) LetteraC=FALSE;
			
			return risult;
		}
		
}

Bool Tempo::GetTempo(char tmp[])
{
	Bool result = TRUE;
	//TODO come capisco che e' tempo composto?
	
	if (LetteraC)
	{
		if(Numeratore.GetNumero() == 2 && Denominatore.GetNumero() == 2)
			tmp = "C";
		else if(Numeratore.GetNumero() == 4 && Denominatore.GetNumero() == 4)
			tmp = "c";
		else
			result = FALSE;
	}
	else if(strcmp((const char*)(Numeratore.GetExternal()),"")==0)
	{
		sprintf(tmp,"%d/%d",Numeratore.GetNumero(), Denominatore.GetNumero());
	}
	else
		sprintf(tmp,"%s/%s%d/%d",Numeratore.GetExternal(), Denominatore.GetExternal(), Numeratore.GetNumero(), Denominatore.GetNumero());

	return result;
}

void Tempo::SetPos(DrawObject *d,const Point& p)
{
	VUnit dxDenom=0, dxNumer=0;
	
	DrawObject::SetPos(d,p);
	
	if(!LetteraC)
	{
		if(Numeratore.GetVU2Rgt()>Denominatore.GetVU2Rgt())
		{
			dxDenom = (Numeratore.GetVU2Rgt()-Denominatore.GetVU2Rgt())/2;
		}
		else
		{
			dxNumer = (-Numeratore.GetVU2Rgt()+Denominatore.GetVU2Rgt())/2;
		}
		Denominatore.SetPos(this, Point(dxDenom,0));
		Numeratore.SetPos(this, Point(dxNumer,2*VU_PER_ROW));
	}
}  

void Tempo::SetColor(int col)
{
	DrawObject::SetColor((ColorIcon)col);
	Numeratore.SetColor((ColorIcon)col);
	Denominatore.SetColor((ColorIcon)col);
	
	return; 
}


void Tempo::Draw()
{
	if (GetAbsPos()==Point(0,0)) return;
	
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());


	if(LetteraC)
	{
		Scr->TextTo(GetAbsPos() + Point(0,VU_PER_ROW));
		if(Numeratore.GetNumero()==4)
			Scr->OutText((unsigned char *)"c");
		else
			Scr->OutText((unsigned char *)"C");
	}
	else
	{
		/* FFF
		Numeratore.SetColor(GetColor());
		Denominatore.SetColor(GetColor());
		*/
		Numeratore.Draw();
		Denominatore.Draw();
	}  

	Scr->SetColor(oldcol);
}


int Tempo::operator!=(Tempo tempo)
{
	if(Numeratore.GetNumero()!=tempo.Numeratore.GetNumero()) return TRUE;
	if(Denominatore.GetNumero()!=tempo.Denominatore.GetNumero()) return TRUE;
	if(LetteraC!=tempo.LetteraC) return TRUE;
	return FALSE;
}

void Tempo::operator=(Tempo tempo)
{
	ID=CL_TEMPO;
	//SetColor((ColorIcon)NORMAL_COLOR);
	
	Numeratore.SetNumero(tempo.Numeratore.GetNumero());
	Denominatore.SetNumero(tempo.Denominatore.GetNumero());
	LetteraC=tempo.LetteraC;
	Numeratore.SetFont(nMusicFont);
	Denominatore.SetFont(nMusicFont);
	
}

int Tempo::GetNumeratore()
{ return Numeratore.GetNumero(); }

int Tempo::GetDenominatore()
{return Denominatore.GetNumero(); }



int Tempo::Validator(wxString text,bool inner)
// il parametro inner se e' true va a settare il tempo interno
// altrimenti setta il tempo interno
// ritorna 0 se il tempo e' sintatticamente sbagliato
//		   1 se nella forma a/d
//         2 se nella forma a+..+c / d
// Si noti che il simbolo '' significa la presenza del simbolo '+'
{
	
	wxString tempo;
	
	size_t len = text.length();
	//controlla se esiste qualche carattere non consentito
	for (size_t k=0;k<len;k++)
	{
		if (strchr(" 01234567890/cC",text[k])==NULL)
			return 0;
	}
	tempo = text;
	//Controlla se e' 4/4 con lettera
	if (strcmp(tempo,"c")!=0)
	{
		return 1;
	}
	
	//Controlla se e' 2/2 con lettera
	if (strcmp(tempo,"C")!=0) 
	{
		return 1;
	}
	
	// Dai check precedenti sono rimasti solo '+' o '/' o numeri
	//Se non ho un unico '/' e' sicuramente sbagliato
	short slash_count = 0;
	for (k = 0; k < len; k++)
	{
		if (strchr("/",tempo[k])!=NULL)
			slash_count++;
	}
	if (slash_count != 1)
		return 0;
	else 
	{
		// costruisci numeratore e denominatore
		size_t indice = tempo.Index('/');
		wxString numeratore, denominatore;
		numeratore = tempo.Left(indice);
		denominatore = tempo.Right(len - indice - 1);
		// adesso num e den hanno solo numeri e '+'
		// check sul denominatore
		if (!denominatore.IsNumber())
			return 0;
		else
		{
			// check sul numeratore
			// se il numeratore e' un numero
			if (numeratore.IsNumber())
			{
				//caso standard
				return 1;
			}
			else
			{
				//check se siamo in a+..+z 
				//se numeratore inizia o termina con + e' sbagliato
				size_t fine = numeratore.length();
				if ((strchr("",numeratore[unsigned int(0)]) != NULL) || (strchr("",numeratore[fine-1]) != NULL))
					return 0;
				for (k=1;k<fine;k++)
				{
					//se ho due '+' consecutivi e' sbagliato
					if ( (strchr("",numeratore[k]) != NULL) && (strchr("",numeratore[k-1])!=NULL) )
						return 0;
				}
				// forma a+..+z
				return 2;
			}
			
		}
		
	}
}


