/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */
#include <windows.h> //for GetShortPathName

#include "bit/odmap.hpp"
#include "bitstreamdecoder.hpp"
#include "smr/smrmain.hpp"
#include "smr/dlg/settings.hpp"

#include <wx/txtstrm.h>
#include <wx/zstream.h>
#include <wx/wfstream.h>
#include <wx/mstream.h>

#include "lioo.hpp"
extern void Message(char* msg, char* title);
XERCES_CPP_NAMESPACE_USE

class BitInputStream
{
public:
	BitInputStream(wxInputStream& stream)
	{
		_buffer=0;
		_stream=&stream;
		_bitMask=0;
		_bitsRead=0;
	}

	bool isEOF()
	{
		return _stream->Eof();
	}

	unsigned int readBits(int bits)
	{
		unsigned int v=0;
		for(int i=bits-1; i>=0; i--)
			if(readBit())
				v = writeBit(v,i);
		return v;
	}

	unsigned int readVluimsf8()
	{
		bool notLast;
		unsigned int v = 0;
		do
		{
			notLast=readBit();
			for(int i=0; i<7; i++)
			{
				v = v << 1;
				v |= readBit();
			}
		} while(notLast);
		return v;
	}

	int readBit()
	{
		if(_bitMask==0)
		{
			_stream->Read(&_buffer,1);
			_bitMask=0x80;
		}
		int b= (_buffer & _bitMask) >0;
		_bitMask = _bitMask >> 1;
		_bitsRead++;
		return b;
	}

	int bitsRead() { return _bitsRead; }

private:
	unsigned char _buffer;
	wxInputStream *_stream;
	unsigned char _bitMask;
	int _bitsRead;

	unsigned int writeBit(unsigned int v, int i) { return (v | (1 << i)); }

};

class ChunkInputStream : public wxFilterInputStream
{
public:
	ChunkInputStream(wxInputStream& strm, unsigned int length) : wxFilterInputStream(strm), m_length(length)
	{
		m_read=0;
	}

protected:
	size_t OnSysRead(void* buffer, size_t bufsize)
	{
		size_t toRead = m_read+bufsize>m_length ? m_length-m_read : bufsize; 
		size_t readCount=m_parent_i_stream->Read(buffer, toRead).LastRead();
		m_read += readCount;
		return readCount;
	}

private:
	unsigned int m_length;
	size_t m_read;

};

BitStreamDecoder::BitStreamDecoder(wxInputStream* IStream)
{
	_MidiStream = NULL;
	_IStream = IStream;
	_BitIStream = new BitInputStream(*_IStream);

	if (readHeader() == -1)
		Message("Error on Reading Header","BitStreamDecoder Decoder");

	_chunkPartID = 0;
	_chunkLength = 0;
	_BimDecoderInit = NULL;
	_BimDecoderInitLen = 0;
}

BitStreamDecoder::~BitStreamDecoder()
{
	delete _BitIStream;
	delete _BimDecoderInit;
}
void BitStreamDecoder::setIStream(wxInputStream* IStream)
{
	delete _BitIStream;
	_IStream = IStream;
	_BitIStream=new BitInputStream(*_IStream);
}

int BitStreamDecoder::readHeader()
{
	_HeaderCommonData.audioObjectType = _BitIStream->readBits(5);
	if(_HeaderCommonData.audioObjectType == 31)
		_HeaderCommonData.audioObjectTypeExt = _BitIStream->readBits(6);
	_HeaderCommonData.samplingFrequencyIndex = _BitIStream->readBits(4);
	_HeaderCommonData.channelConfiguration = _BitIStream->readBits(4);
	if(_HeaderCommonData.audioObjectType==31 && 
		(_HeaderCommonData.audioObjectTypeExt==40-32 ||
		 _HeaderCommonData.audioObjectTypeExt==41-32))
	{
		_HeaderCommonData.version = _BitIStream->readBits(4);
		_HeaderCommonData.pictureWidth = _BitIStream->readBits(12);
		_HeaderCommonData.pictureHeight = _BitIStream->readBits(12);
		_HeaderCommonData.isScoreMultiwindow = _BitIStream->readBits(1);
		_HeaderCommonData.numberOfParts = _BitIStream->readBits(8);
		_HeaderCommonData.notationFormat = _BitIStream->readBits(3);
		
		unsigned int midiUrlSize = _BitIStream->readVluimsf8();
		for(unsigned int i=0; i<midiUrlSize; i++)
			_MidiUrl += char(_BitIStream->readBits(8));
		
		_HeaderDecInitData.codingType = _BitIStream->readBits(2);
		
		unsigned int bimConfigBitSize = _BitIStream->readVluimsf8();
		if(_HeaderDecInitData.codingType == BIMF)
		{
			_DefaultDecoderInit=_BitIStream->readBits(3);
			if(bimConfigBitSize>0)
			{
				_BimDecoderInitLen = bimConfigBitSize/8+1;
				_BimDecoderInit = new char[_BimDecoderInitLen];
				for(i=0; i<bimConfigBitSize/8; i++)
					_BimDecoderInit[i]=_BitIStream->readBits(8);
				int bitsToRead=bimConfigBitSize%8;
				if(bitsToRead>0)
					_BimDecoderInit[i]=_BitIStream->readBits(bitsToRead);
			}
		}
	}
	else if(_HeaderCommonData.audioObjectType==15 || _HeaderCommonData.audioObjectType==13)
	{
		while(_BitIStream->readBit()) //moredata
		{
			int chunkType=_BitIStream->readBits(3);
			unsigned int length=0,i,j,data;
			unsigned char b=0;
			switch(chunkType)
			{
			case 0: //orc_file
				{
					length = _BitIStream->readBits(16);
					for(i=0; i<length; i++)
					{
						unsigned int token = _BitIStream->readBits(8),slen=0;
						switch(token)
						{
						case 0xf0:
							data = _BitIStream->readBits(16); //symbol
							break;
						case 0xf1:
							data = _BitIStream->readBits(32); //float
							break;
						case 0xf2:
							data = _BitIStream->readBits(32); //int
							break;
						case 0xf3:
							slen = _BitIStream->readBits(8); //string length
							for(j=0; j<slen; j++)
								data = _BitIStream->readBits(8);
							break;
						case 0xf4:
							data = _BitIStream->readBits(8); //char
							break;
						}
					}
					break;
				}
			case 1: //score_file
				{
					length = _BitIStream->readBits(20);
					for(i=0; i<length; i++)
					{
						if(_BitIStream->readBit()) //has_time
						{
							data = _BitIStream->readBit();    //use if late
							data = _BitIStream->readBits(32); //time
						}
						data = _BitIStream->readBit(); //high priority
						unsigned int type = _BitIStream->readBits(3),plen=0;
						switch(type)
						{
						case 0: //instr event
							if(_BitIStream->readBit()) // has label
								data = _BitIStream->readBits(16); //symbol
							data = _BitIStream->readBits(16); //symbol instument name
							data = _BitIStream->readBits(32); //float duration
							plen = _BitIStream->readBits(8);
							for(j=0; j<plen; j++)
								data = _BitIStream->readBits(32); //float pfield
							break;
						case 1: //control event
							if(_BitIStream->readBit()) // has label
								data = _BitIStream->readBits(16); //symbol
							data = _BitIStream->readBits(16); //controller name
							data = _BitIStream->readBits(32); //float new falue
							break;
						case 2: // table event
							data = _BitIStream->readBits(16); //symbol
							if(!_BitIStream->readBit()) // destroy
							{
								unsigned int token = _BitIStream->readBits(8); //token?
								if(_BitIStream->readBit()) // refer to sample
									data = _BitIStream->readBits(16); //symbol
								unsigned int numpf = _BitIStream->readBits(16);
								if(token==0x7d)
								{
									data = _BitIStream->readBits(32); //float size
									for(j=0;j<numpf-1;j++)
										data = _BitIStream->readBits(16); //symbol
								}
								else
									for(j=0;j<numpf;j++)
										data = _BitIStream->readBits(32);

							}
							break;
						case 4: // end event
							break;
						case 5: // tempo event
							data = _BitIStream->readBits(32); //float tempo
							break;
						}
					}
					break;
				}
			case 2: //midi_file
				{
					length = _BitIStream->readBits(32);
					char *buffer=new char[length];
					for(i=0; i<length; i++)
						buffer[i]=_BitIStream->readBits(8);
					_MidiStream = new wxMemoryInputStream(buffer,length);
					break;
				}
				break;
			case 3: //sample
				{
					data = _BitIStream->readBits(16); //symbol
					length = _BitIStream->readBits(24);
					if(_BitIStream->readBit()) // has_srate
						data=_BitIStream->readBits(17);
					if(_BitIStream->readBit()) // has_loop
					{
						data = _BitIStream->readBits(24); //loop start
						data = _BitIStream->readBits(24); //loop end
					}
					if(_BitIStream->readBit()) //has base
						data = _BitIStream->readBits(32);
					if(_BitIStream->readBit()) //float_sample
						for(i=0; i<length; i++)
							data = _BitIStream->readBits(32);
					else
						for(i=0; i<length; i++)
							data = _BitIStream->readBits(16);
					break;
				}
			case 4: //sbf
				{
					length = _BitIStream->readBits(32);
					for(i=0; i<length; i++)
						_BitIStream->readBits(8);
					break;
				}
			case 5: //symtable
				{
					length = _BitIStream->readBits(16); //length
					for(i=0; i<length; i++)
					{
						unsigned int slen = _BitIStream->readBits(4);
						for(j=0; j<slen; j++)
							b=_BitIStream->readBits(8);
					}
					break;
				}
			}
		}
	}

	return 0;
}

unsigned int BitStreamDecoder::getPartsNumber()
{
	return _HeaderCommonData.numberOfParts;
}

bool BitStreamDecoder::readMoreData()
{
	bool more_data=_BitIStream->readBit();
	_BitIStream->readBits(7);
	return more_data;
}

int	BitStreamDecoder::readNextChunk()
{
	_ChunkCommonData.chunkType=_BitIStream->readBits(3);
	_BitIStream->readBits(5); //reserved
	_chunkLength = _BitIStream->readVluimsf8();
	_chunkBytesRead=0;
	_chunkPartID = 0;

	Chunk_Type type = (Chunk_Type)_ChunkCommonData.chunkType;
	if ( type == PART || type == LYRC)
	{
		_chunkPartID = _BitIStream->readBits(8);
	}
	if (type == LYRC)
	{
		_chunkLyricID = _BitIStream->readBits(8);
	}
	return 0;
}

Chunk_Type BitStreamDecoder::getChunkType()
{
	return (Chunk_Type)_ChunkCommonData.chunkType;
}

unsigned int BitStreamDecoder::getChunkId()
{
	return _chunkPartID;
}

InputSource* BitStreamDecoder::getChunkInputSource()
{
	return new IStreamInputSource(*_IStream, _chunkLength);
}

static int writeStr(char* s, int p, const char* x)
{
	s[p]=strlen(x);
	memcpy(&s[p+1],x,s[p]);
	return p+1+s[p];
}

wxInputStream* BitStreamDecoder::getChunkInputStream()
{
	switch(getChunkType())
	{
	case MAIN:
	case PART:
	case LYRC:
	case SMFL:
		if(getCodingType()==GZIP)
		{
			wxString tmpGzipFile=wxString(getenv("TEMP"))+"\\tmp.xml.gz";
			wxString tmpXmlFile=wxString(getenv("TEMP"))+"\\tmp.xml";
			wxInputStream* chunk=new ChunkInputStream(*_IStream, _chunkLength);
			{
			wxFileOutputStream gzip(tmpGzipFile);
			gzip.Write(*chunk);
			}
			if(wxFileExists(tmpXmlFile))
				wxRemoveFile(tmpXmlFile);
			wxString cmd = "\"" + SMRApp::GetApp().settings->GetGZipPath()+"gzip\" -d "+tmpGzipFile;
			wxArrayString out,err;
			wxExecute(cmd, out, err);
			//system(cmd.c_str());
			if(!wxFileExists(tmpXmlFile))
			{
				wxMessageBox("Failed execution of:\n" + cmd);
				return NULL;
			}
			return new wxFileInputStream(tmpXmlFile);
		}
		else if(getCodingType()==BIMF)
		{
			wxString bimFileName, xmlFileName;
			wxString schema;
			wxString schemaPath=wxString(smrgetenv("SMR_ERR_PATH"))+"xsd\\";
			switch(getChunkType())
			{
			case PART:
				schema ="urn:mpeg:mpeg-4:schema:smr:smxf-part:2007 \""+schemaPath+"smxf_part.xsd\"";
				xmlFileName=wxString(getenv("TEMP"))+"\\part.xml";
				break;
			case MAIN:
				schema="urn:mpeg:mpeg-4:schema:smr:smxf-mainscore:2007 \""+schemaPath+"smxf_main.xsd\"";
				xmlFileName=wxString(getenv("TEMP"))+"\\main.xml";
				break;
			case LYRC:
				schema="urn:mpeg:mpeg-4:schema:smr:smxf-lyric:2007 \""+schemaPath+"smxf_lyric.xsd\"";
				xmlFileName=wxString(getenv("TEMP"))+"\\lyric.xml";
				break;
			case SMFL:
				schema="urn:mpeg:mpeg-4:schema:smr:smfl:2007 \""+schemaPath+"smfl.xsd\"";
				xmlFileName=wxString(getenv("TEMP"))+"\\smfl.xml";
				break;
			}
			bimFileName=xmlFileName+".bim";

			wxInputStream* chunk=new ChunkInputStream(*_IStream, _chunkLength);
			{
				wxFileOutputStream bim(bimFileName);
				bim.Write(*chunk);
			}
			if(_DefaultDecoderInit)
			{
				char decoderInit[2048];
/* this should work... but bim decoder application doesn't like it.
				decoderInit[0]=0; //systems profile level indication
				decoderInit[1]=0; //UnitSizeCOde=0, NoAdvancedFeatures=0
				decoderInit[2]=0; //AdvancedFeaturesFlags Len
				decoderInit[3]=0x02; //flags
				decoderInit[4]=5; //number of schemas 

				int p=writeStr(decoderInit,5,"http://www.w3.org/2001/XMLSchema"); //uri schemas
				decoderInit[p++]=0; //location hint len
				decoderInit[p++]=0; //number of type codecs

				p=writeStr(decoderInit,p,"urn:mpeg:mpeg-4:schema:smr:smxf-mainscore:2007"); //uri schemas
				decoderInit[p++]=0; //location hint len
				decoderInit[p++]=0; //number of type codecs
				
				p=writeStr(decoderInit,p,"urn:mpeg:mpeg-4:schema:smr:smxf-part:2007"); //uri schemas
				decoderInit[p++]=0; //location hint len
				decoderInit[p++]=0; //number of type codecs
				
				p=writeStr(decoderInit,p,"urn:mpeg:mpeg-4:schema:smr:smxf-lyric:2007"); //uri schemas
				decoderInit[p++]=0; //location hint len
				decoderInit[p++]=0; //number of type codecs
				
				p=writeStr(decoderInit,p,"urn:mpeg:mpeg-4:schema:smr:smfl:2007"); //uri schemas
				decoderInit[p++]=0; //location hint len
				decoderInit[p++]=0; //number of type codecs
*/
				//the following version works... but it is not conformant with BiM decoder config
				decoderInit[0]=0; //systems profile level indication
				decoderInit[1]=0; //UnitSizeCOde=0, NoAdvancedFeatures=0
				decoderInit[2]=0; //AdvancedFeaturesFlags Len
				decoderInit[3]=0x04;
				decoderInit[4]=0; 
				int p=writeStr(decoderInit,5,"http://www.w3.org/2001/XMLSchema"); //uri schemas
				decoderInit[p++]=0; //location hint len
				decoderInit[p++]=0; 

				decoderInit[p++]=0; 
				p=writeStr(decoderInit,p,"urn:mpeg:mpeg-4:schema:smr:smxf-mainscore:2007"); //uri schemas
				decoderInit[p++]=0; 
				p=writeStr(decoderInit,p,schemaPath+"smxf_main.xsd");
				
				decoderInit[p++]=0; 
				p=writeStr(decoderInit,p,"urn:mpeg:mpeg-4:schema:smr:smxf-part:2007"); //uri schemas
				decoderInit[p++]=0; 
				p=writeStr(decoderInit,p,schemaPath+"smxf_part.xsd");

				decoderInit[p++]=0; 
				p=writeStr(decoderInit,p,"urn:mpeg:mpeg-4:schema:smr:smxf-lyric:2007"); //uri schemas
				decoderInit[p++]=0; 
				p=writeStr(decoderInit,p,schemaPath+"smxf_lyric.xsd");
				
				wxFileOutputStream bimConfig(bimFileName+".decoderConfig");
				bimConfig.Write(decoderInit,p);
			}
			else if(_BimDecoderInit!=NULL)
			{
				wxFileOutputStream bimConfig(bimFileName+".decoderConfig");
				bimConfig.Write(_BimDecoderInit, _BimDecoderInitLen);
			}

			if(wxFileExists(xmlFileName))
				wxRemoveFile(xmlFileName);
/*
			wxString bimApp=SMRApp::GetApp().settings->GetBimPath()+"bimDecode.bat";
			if(wxFileExists(bimApp))
			{
				//convert the path to bim application to short path (without spaces)
				char lpShortFileName[1024];
				GetShortPathName(bimApp.c_str(),lpShortFileName, 1024);
				bimApp=lpShortFileName;
			}
			wxString cmd = bimApp+" "+bimFileName+" "+xmlFileName+" "+schema;
*/
			wxString bimPath=SMRApp::GetApp().settings->GetBimPath();
			if(wxDirExists(bimPath))
			{
				//convert the path to bim application to short path (without spaces)
				char lpShortFileName[1024];
				GetShortPathName(bimPath.c_str(),lpShortFileName, 1024);
				bimPath=lpShortFileName;
			}
			wxString cmd = "java -classpath "+
				bimPath+"lib/bim.jar;"+
				bimPath+"lib/gnu-regexp-1.1.3.jar;"+
				bimPath+"lib/jcraft.jar;"+
				bimPath+"lib/xercesImpl.jar;"+
				bimPath+"lib/xml-apis.jar"
				" com.expway.binarisation.GeneralDecompressor "+bimFileName+" "+xmlFileName+" "+schema;
			//system(cmd);
			wxArrayString out, err;
			wxExecute(cmd, out, err);
			if(!wxFileExists(xmlFileName))
			{
				wxMessageBox("Failed execution of:\n" + cmd);
				return NULL;
			}

			return new wxFileInputStream(xmlFileName);
		}
		break;
	case FONT:
		{
			wxInputStream* font=new ChunkInputStream(*_IStream, _chunkLength);
			BitInputStream fontbits(*font);
			int fontFormat=fontbits.readBits(7);
			int storeFont=fontbits.readBit();

			int fontNameLength=fontbits.readBits(8);
			wxString fontName;
			for(int i=0; i<fontNameLength; i++)
				fontName+=char(fontbits.readBits(8));
			
			int fontSubsetID=fontbits.readBits(7);
			int fontSubsetExtensionFlag=fontbits.readBit();
			int fontSpecInfoLength=fontbits.readBits(8);

			wxString fontFileName=wxString(getenv("TEMP"))+"\\font.ttf";
			wxFileOutputStream fontStream(fontFileName);
			fontStream.Write(*font);

			return new wxFileInputStream(fontFileName);;
		}
		break;
	}
	return new ChunkInputStream(*_IStream, _chunkLength);
}

int	BitStreamDecoder::readChunkData(void* buffer, int bufferLength)
{
	if(_chunkBytesRead>=_chunkLength)
		return 0;

	int bufLength = _chunkBytesRead+bufferLength > _chunkLength ? _chunkLength - _chunkBytesRead : bufferLength;

	if(_IStream->Read(buffer, bufLength).LastRead()!=bufLength)
		return -1;
	_chunkBytesRead += bufLength;
	return bufLength;
}

void BitStreamDecoder::decodeSyncInfo(wxInputStream* in, wxOutputStream* out)
{
	wxTextOutputStream txt(*out);
	BitInputStream bin(*in);
	wxString syncInfoText;

	unsigned int type;
	unsigned int nSyncInfo=0;

	type=bin.readBits(2);
	if(type==1)
	{
		bool more_data=true;
		unsigned int measure=0;
		unsigned int duration=0;

		while(more_data)
		{
			bool jump =  bin.readBit();
			if(jump)
				measure = bin.readBits(16);
			else
				measure++;

			duration=bin.readBits(16);

			syncInfoText << measure << " " << duration << "\n";
			nSyncInfo++;

			more_data=bin.readBit();
		}
		txt << nSyncInfo << endl;
		txt << syncInfoText;
	}
}

ODStreamDecoder::ODStreamDecoder(wxInputStream* is)
{
	_IStream = is;
	_BitIStream = new BitInputStream(*_IStream);
	_lastOID = 0;
}
ODStreamDecoder::~ODStreamDecoder()
{
	delete _BitIStream;
}
void ODStreamDecoder::setIStream(wxInputStream* IStream)
{
	delete _BitIStream;
	_IStream = IStream;
	_BitIStream=new BitInputStream(*_IStream);
}

bool ODStreamDecoder::readODStream(ODMap& ods)
{
	while(_BitIStream->bitsRead()<_IStream->GetSize()*8)
	{
		unsigned int type = _BitIStream->readBits(8);
		unsigned int ODUpdSize = _BitIStream->readVluimsf8();
		unsigned int data = 0;
		if(type==1) //OD Update
		{
			int br=_BitIStream->bitsRead();
			while(_BitIStream->bitsRead()-br<ODUpdSize*8)
			{
				readDescriptor(ods);
			}
		}
		else
		{
			for(int i=0;i<ODUpdSize; i++)
				_BitIStream->readBits(8);
		}
	}

	return true;
}

int ODStreamDecoder::readDescriptor(ODMap& ods)
{
	unsigned int tag=_BitIStream->readBits(8);
	unsigned int size = _BitIStream->readVluimsf8();
	if(tag==1) //OD
	{
		readOD(ods, size);
	}
	else if(tag==2) //IOD
	{
		readIOD(ods, size);
	}
	else if(tag==3) //ES
	{
		readES(ods, size);
	}
	else
		for(int i=0;i<size; i++)
			_BitIStream->readBits(8);
	return tag;
}

bool ODStreamDecoder::readOD(ODMap& ods, int size)
{
	int br=_BitIStream->bitsRead();
	_lastOID = _BitIStream->readBits(10);
	int isUrl = _BitIStream->readBit();
	int data = _BitIStream->readBits(5); // cons 0b11111
	if(isUrl)
	{
		unsigned int urlLength = _BitIStream->readBits(8);
		for(int i=0; i<urlLength; i++)
			data = _BitIStream->readBits(8);
	}
	else
	{
		while(_BitIStream->bitsRead()-br<size*8)
		{
			readDescriptor(ods); //ES
		}
	}
	return true;
}

bool ODStreamDecoder::readIOD(ODMap& ods, int size)
{
	int br=_BitIStream->bitsRead();
	_lastOID = _BitIStream->readBits(10);
	int isUrl = _BitIStream->readBit();
	int includeInlineProfileLevel = _BitIStream->readBit();
	int data = _BitIStream->readBits(4); // const 0b1111
	if(isUrl)
	{
		unsigned int urlLength = _BitIStream->readBits(8);
		for(int i=0; i<urlLength; i++)
			data = _BitIStream->readBits(8);
	}
	else
	{
		int ODProfileLevel = _BitIStream->readBits(8);
		int sceneProfileLevel = _BitIStream->readBits(8);
		int audioProfileLevel = _BitIStream->readBits(8);
		int visualProfileLevel = _BitIStream->readBits(8);
		int graphicsProfileLevel = _BitIStream->readBits(8);
		while(_BitIStream->bitsRead()-br<size*8)
		{
			readDescriptor(ods); //ES
		}
	}
	while(_BitIStream->bitsRead()-br<size*8)
	{
		_BitIStream->readBits(8);
	}
	return true;
}

bool ODStreamDecoder::readES(ODMap& ods, int size)
{
	int br=_BitIStream->bitsRead();
	int esid =_BitIStream->readBits(16);
	int oti = 0, streamType = 0, bufferSize = 0, maxBitRate = 0, avgBitRate = 0, ocrid=0;
	bool streamDependanceFlag=_BitIStream->readBits(1);
	bool URLFlag=_BitIStream->readBits(1);
	bool OCRFlag=_BitIStream->readBits(1);
	int streamPriority=_BitIStream->readBits(5);
	if(streamDependanceFlag)
		_BitIStream->readBits(16);
	if(URLFlag)
	{
		int length=_BitIStream->readBits(8);
		for(int i=0; i<length; i++)
			_BitIStream->readBits(8);
	}
	if(OCRFlag)
		ocrid=_BitIStream->readBits(16);
	
	int tag=_BitIStream->readBits(8);
	int dcSize=_BitIStream->readVluimsf8();
	oti=_BitIStream->readBits(8);
	streamType=_BitIStream->readBits(6);
	bool upStream=_BitIStream->readBits(1);
	bool reserved=_BitIStream->readBits(1);
	bufferSize = _BitIStream->readBits(24);
	maxBitRate = _BitIStream->readBits(32);
	avgBitRate = _BitIStream->readBits(32);
	
	ods.addOD(_lastOID,new ESData(esid, oti, streamType, bufferSize, maxBitRate, avgBitRate));

	//skip all data until the end of descriptor
	int Br=(_BitIStream->bitsRead()-br)/8;
	for(int i=0;i<size-Br; i++)
		_BitIStream->readBits(8);
	return true;
}

BIFSStreamDecoder::BIFSStreamDecoder(wxInputStream* is)
{
	_IStream = is;
	_BitIStream = new BitInputStream(*_IStream);
	readConfig();
}
BIFSStreamDecoder::~BIFSStreamDecoder()
{
	delete _BitIStream;
}
void BIFSStreamDecoder::setIStream(wxInputStream* IStream)
{
	delete _BitIStream;
	_IStream = IStream;
	_BitIStream=new BitInputStream(*_IStream);
}
bool BIFSStreamDecoder::readConfig()
{
	bool use3DMeshCoding = _BitIStream->readBits(1);
	bool usePredictiveMFField = _BitIStream->readBits(1);
	int PROTOIDbits = _BitIStream->readBits(5);
	int nodeIDBits = _BitIStream->readBits(5);
	int routeIDBits = _BitIStream->readBits(5);
	bool isCommandStream = _BitIStream->readBit();
	if(isCommandStream)
	{
		bool isPixelMetric = _BitIStream->readBit();
		bool hasSize = _BitIStream->readBit();
		if(hasSize) {
			unsigned int pixelWidth = _BitIStream->readBits(16);
			unsigned int pixelHeight = _BitIStream->readBits(16);
		}
	}
	return true;
}
bool BIFSStreamDecoder::decode()
{
	return true;
}
