/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#ifndef BITSTREAMDECODER_HPP
#define BITSTREAMDECODER_HPP

#include "istreaminputsource.hpp"

#include <wx/stream.h>
#include <wx/string.h>


#include "def_bit.hpp"

class BitInputStream;

class BitStreamDecoder
{
public:
	BitStreamDecoder(wxInputStream* IStream);
	~BitStreamDecoder();

	void				setIStream(wxInputStream* IStream);
	/*HEADER*/
	unsigned int		getAudioObjectType() { return _HeaderCommonData.audioObjectType==31 ? _HeaderCommonData.audioObjectTypeExt + 32 : _HeaderCommonData.audioObjectType; }

	unsigned int		getVersion()		{return _HeaderCommonData.version;};
	unsigned long		getPictureWidth()	{return _HeaderCommonData.pictureWidth;};
	unsigned long		getPictureHeight()	{return _HeaderCommonData.pictureHeight;};
	NotationFormat_Type getNotationFormat()	{return (NotationFormat_Type)_HeaderCommonData.notationFormat;};
	bool				getMultiwindow()	{return (_HeaderCommonData.isScoreMultiwindow!=0);};
	Coding_Type			getCodingType()		{return (Coding_Type)_HeaderDecInitData.codingType;};
	unsigned int		getPartsNumber();
	wxString			getMidiUrl()		{return _MidiUrl; }

	/*CHUNKS*/
	bool				readMoreData();
	int					readNextChunk();
	Chunk_Type			getChunkType();
	unsigned int		getChunkId();
	XERCES_CPP_NAMESPACE::InputSource*		getChunkInputSource();
	wxInputStream*		getChunkInputStream();
	unsigned int		getChunkLength() { return _chunkLength; }
	int					readChunkData(void* buffer, int bufferLength);

	void				decodeSyncInfo(wxInputStream*, wxOutputStream*);

	//Structured Audio (for MIDI) AOT 15
	wxInputStream*		getMidiInputStream() { return _MidiStream; }

private:
	wxInputStream*		_IStream;
	BitInputStream*		_BitIStream;
	
	/*HEADER*/
	HEADER_COMMON		_HeaderCommonData;
	HEADER_DECODER		_HeaderDecInitData;
	wxString			_MidiUrl;
	wxInputStream*		_MidiStream;
	int					_DefaultDecoderInit;
	char*				_BimDecoderInit;
	int					_BimDecoderInitLen;
	
	/*CHUNKS*/
	CHUNK_COMMON		_ChunkCommonData;
	CHUNK_HASNEXT		_ChunkHasNextData;
	unsigned int		_chunkPartID;
	unsigned int		_chunkLyricID;
	unsigned int		_chunkLength;

	unsigned int		_chunkBytesRead;

	unsigned long		var2fixUint(unsigned char first);

	int readHeader();

	//extra
	unsigned char BitReverse(unsigned char v);
};

class ODMap;

class ODStreamDecoder
{
public:
	ODStreamDecoder(wxInputStream* IStream);
	~ODStreamDecoder();

	void setIStream(wxInputStream* IStream);
	bool readODStream(ODMap& ods);
	int  readDescriptor(ODMap& ods);
	bool readOD(ODMap& ods, int size);
	bool readES(ODMap& ods, int size);
	bool readIOD(ODMap& ods, int size);
private:
	wxInputStream*		_IStream;
	BitInputStream*		_BitIStream;
	int					_lastOID;
};

class BIFSStreamDecoder
{
public:
	BIFSStreamDecoder(wxInputStream* IStream);
	~BIFSStreamDecoder();

	void setIStream(wxInputStream* IStream);
	bool readConfig();
	bool decode();
private:
	wxInputStream*		_IStream;
	BitInputStream*		_BitIStream;
};

#endif