/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include "bit/odmap.hpp"
#include "bitstreamencoder.hpp"

#include <windows.h>
#include "lioo.hpp"
#include "smr/smrmain.hpp"
#include "smr/dlg/settings.hpp"


//
#include <wx/listimpl.cpp>
#include <wx/txtstrm.h>
#include <wx/wfstream.h>
#include <wx/zstream.h>
#include <wx/mstream.h>
#include <wx/utils.h>

WX_DEFINE_LIST(ChunkList);
//

extern void Message(char* msg, char* title);

class BitOutputStream
{
public:
	BitOutputStream(wxOutputStream& stream)
	{
		_buffer=0;
		_stream=&stream;
		_bitMask=0x80;
	}
	~BitOutputStream()
	{
		flush();
	}

	void writeBits(unsigned int v, int bits)
	{
		for(int i=bits-1; i>=0; i--)
			writeBit(bit(v,i));
	}

	void writeVluimsbf8(unsigned int v)
	{
		int nbits;
		for (nbits=31; nbits>=0; nbits--)
		{
			if(bit(v,nbits))
				break;
		}
		if(nbits==-1)
			nbits=0;

		int nbytes = (nbits+1+6)/7;
		for(int i=nbytes*7; i>=1; i--)
		{
			if(i%7==0)
			{
				writeBit( i/7 > 1 );
			}
			writeBit(bit(v,i-1));
		}
	}

	void align() { flush(); }
	void flush()
	{
		if(_bitMask!=0x80)
			_stream->Write(&_buffer, 1);
		_bitMask=0x80;
		_buffer=0;
	}
private:
	unsigned char _buffer;
	wxOutputStream *_stream;
	unsigned char _bitMask;

	bool bit(unsigned int v, int i) { return (v & (1 << i))!=0; }
	void writeBit(bool b)
	{
		_buffer = _buffer | (_bitMask * b);
		_bitMask = _bitMask >> 1;
		if(_bitMask==0)
		{
			_stream->Write(&_buffer,1);
			_bitMask=0x80;
			_buffer=0;
		}
	}

};

BitStreamEncoder::BitStreamEncoder(wxOutputStream* OStream)
{
	_OStream = OStream;
	_BitOStream = new BitOutputStream(*_OStream);

	cList.Clear();

	/*Header Defaults*/	
	_HeaderCommonData.audioObjectType		= 31;
	_HeaderCommonData.audioObjectTypeExt	= 40 - 32;
	_HeaderCommonData.samplingFrequencyIndex = 4;
	_HeaderCommonData.channelConfiguration	= 1;
	_HeaderCommonData.version				= 0;
	_HeaderCommonData.pictureWidth			= 600;
	_HeaderCommonData.pictureHeight			= 400; 
	_HeaderCommonData.isScoreMultiwindow	= 0;	
	_HeaderCommonData.notationFormat		= CWMN;
	_HeaderCommonData.numberOfParts			= 0;

	_HeaderDecInitData.codingType			= XMLF;
	_HeaderHasNextData.more_data			= 0;

}

BitStreamEncoder::~BitStreamEncoder()
{
	delete _BitOStream;
}

void BitStreamEncoder::setOStream(wxOutputStream* OStream)
{
	delete _BitOStream;
	_OStream = OStream;
	_BitOStream = new BitOutputStream(*_OStream);
}

void BitStreamEncoder::AddSource(Chunk_Type Type, wxInputStream* Source, int PartID)
{
	cList.Append(new DataChunk(Type, Source, PartID));
	_HeaderHasNextData.more_data = 1;
}

int BitStreamEncoder::writeDownDecoderConfig(bool onlySMRConfig)
{
	//setting parts number
	if(_HeaderDecInitData.codingType==BIMF)
		_HeaderCommonData.audioObjectTypeExt = 41 - 32;

	int numberOfChunk					= cList.GetCount();

	_HeaderHasNextData.more_data		= (numberOfChunk > 0);

	//header
	if (writeHeader(onlySMRConfig) != 0)
	{
		Message("Error on Writing Header","BitStreamEncoder Encoder");
		return -1;
	}
	
	//chunks
	ChunkList::Node *node = cList.GetFirst();
	while (node)
	{
		if (writeChunk( node->GetData(), node->GetNext() == NULL ) != 0)
		{
			Message("Error on Writing Chunk","BitStreamEncoder Encoder");
			return -1;		
		}
		node = node->GetNext();
	}

	this->_BitOStream->writeBits(0,11); //syncExtensionType TBR

	return 0;
}

int BitStreamEncoder::writeDownAccessUnit()
{	
	//write chunks
	ChunkList::Node *node = cList.GetFirst();
	
	_BitOStream->writeBits(node!=NULL, 1); //more_data = 1
	_BitOStream->writeBits(0, 7); //padding
	while (node)
	{
		if (writeChunk( node->GetData(), node->GetNext() == NULL ) != 0)
		{
			Message("Error on Writing Chunk","BitStreamEncoder Encoder");
			return -1;		
		}
		node = node->GetNext();
	}

	return 0;
}

int BitStreamEncoder::writeChunk( DataChunk* data, bool isLast )
{
	//file size
	unsigned long size = 0;
	wxString gzipFileName, bimFileName;

	if(_HeaderDecInitData.codingType == XMLF || data->GetType() == FONT || data->GetType() == SYNC)
	{
		char buffer[1024];
		while((data->GetSource())->LastError() != wxSTREAM_EOF )
		{		
			size += ((data->GetSource())->Read(&buffer,1024)).LastRead();
		}
		(data->GetSource())->SeekI(0);
	}
	else if(_HeaderDecInitData.codingType == GZIP)
	{
		wxString xmlFileName=wxString(getenv("TEMP"))+"\\output.xml";
		gzipFileName=xmlFileName+".gz";

		{
			wxFileOutputStream xml(xmlFileName);
			xml.Write(*data->GetSource());
		}

		if(wxFileExists(gzipFileName))
			wxRemoveFile(gzipFileName);
		wxString cmd = "\"" + SMRApp::GetApp().settings->GetGZipPath()+"gzip\" "+xmlFileName;
		wxArrayString out, err;
		wxExecute(cmd,out,err);
		if(!wxFileExists(gzipFileName))
		{
			wxMessageBox("Failed execution of:\n"+cmd);
			return -1;
		}

		wxFileInputStream gzip(gzipFileName);

		char buffer[1024];
		while(gzip.LastError() != wxSTREAM_EOF )
		{		
			size += (gzip.Read(&buffer,1024)).LastRead();
		}
	}
	else if(_HeaderDecInitData.codingType == BIMF)
	{
		wxString xmlFileName;
		wxString schema;

		switch(data->GetType())
		{
		case PART:
			//schema ="urn:mpeg:mpeg-4:schema:smr:smxf-part:2007 \""+wxString(smrgetenv("SMR_ERR_PATH"))+"\\xsd\\smxf_part.xsd\"";
			xmlFileName=wxString(getenv("TEMP"))+"\\part.xml";
			break;
		case MAIN:
			//schema="urn:mpeg:mpeg-4:schema:smr:smxf-mainscore:2007 \""+wxString(smrgetenv("SMR_ERR_PATH"))+"\\xsd\\smxf_main.xsd\"";
			xmlFileName=wxString(getenv("TEMP"))+"\\main.xml";
			break;
		case LYRC:
			//schema="urn:mpeg:mpeg-4:schema:smr:smxf-lyric:2007 \""+wxString(smrgetenv("SMR_ERR_PATH"))+"\\xsd\\smxf_lyric.xsd\"";
			xmlFileName=wxString(getenv("TEMP"))+"\\lyric.xml";
			break;
		case SMFL:
			//schema="urn:mpeg:mpeg-4:schema:smr:smfl:2007 \""+wxString(smrgetenv("SMR_ERR_PATH"))+"\\xsd\\smfl.xsd\"";
			xmlFileName=wxString(getenv("TEMP"))+"\\smfl.xml";
			break;
		}
		schema ="urn:mpeg:mpeg-4:schema:smr:smxf-part:2007 \""+wxString(smrgetenv("SMR_ERR_PATH"))+"\\xsd\\smxf_part.xsd\"";
		schema +=" urn:mpeg:mpeg-4:schema:smr:smxf-mainscore:2007 \""+wxString(smrgetenv("SMR_ERR_PATH"))+"\\xsd\\smxf_main.xsd\"";
		schema +=" urn:mpeg:mpeg-4:schema:smr:smxf-lyric:2007 \""+wxString(smrgetenv("SMR_ERR_PATH"))+"\\xsd\\smxf_lyric.xsd\"";
		//schema +=" urn:mpeg:mpeg-4:schema:smr:smfl:2007 \""+wxString(smrgetenv("SMR_ERR_PATH"))+"\\xsd\\smfl.xsd\"";

		bimFileName=xmlFileName+".bim";

		{
			wxFileOutputStream xml(xmlFileName);
			xml.Write(*data->GetSource());
		}

		if(wxFileExists(bimFileName))
			wxRemoveFile(bimFileName);
/*
		wxString bimApp=SMRApp::GetApp().settings->GetBimPath()+"bimEncode.bat";
		if(wxFileExists(bimApp))
		{
			//convert the path to bim application to short path (without spaces)
			char lpShortFileName[1024];
			GetShortPathName(bimApp.c_str(),lpShortFileName, 1024);
			bimApp=lpShortFileName;
		}
		wxString cmd=bimApp+" "+xmlFileName+" "+bimFileName+" "+schema;
		system(cmd); */
		wxString bimPath=SMRApp::GetApp().settings->GetBimPath();
		if(wxDirExists(bimPath))
		{
			//convert the path to bim application to short path (without spaces)
			char lpShortFileName[1024];
			GetShortPathName(bimPath.c_str(),lpShortFileName, 1024);
			bimPath=lpShortFileName;
		}
		wxString cmd = "java -classpath "+
			bimPath+"lib/bim.jar;"+
			bimPath+"lib/gnu-regexp-1.1.3.jar;"+
			bimPath+"lib/jcraft.jar;"+
			bimPath+"lib/xercesImpl.jar;"+
			bimPath+"lib/xml-apis.jar"
			" com.expway.binarisation.GeneralBinaryHandler "+xmlFileName+" "+bimFileName+" "+schema;
		wxArrayString out, err;
		wxExecute(cmd, out, err);

		if(wxFileExists(bimFileName))
		{
			wxFileInputStream bim(bimFileName);

			char buffer[1024];
			while(bim.LastError() != wxSTREAM_EOF )
			{		
				size += (bim.Read(&buffer,1024)).LastRead();
			}
		}
		else
		{
			wxMessageBox("Failed execution of:\n"+cmd);
			return -1;
		}
	}

	_BitOStream->writeBits(data->GetType(), 3);
	_BitOStream->writeBits(0, 5); //reserved
	_BitOStream->writeVluimsbf8(size);

	if (data->GetType() == PART || data->GetType() == LYRC)
	{
		_BitOStream->writeBits(data->GetPartID(), 8);
	}
	if (data->GetType() == LYRC)
	{
		_BitOStream->writeBits(data->GetLyricID(), 8);
	}

	//file body
	if (_HeaderDecInitData.codingType == XMLF || data->GetType() == SYNC || data->GetType() == FONT)
	{
		unsigned int rd;
		unsigned long wt=0;
		char buffer[1024];
		wxInputStream* source = data->GetSource();
		while( source->LastError() != wxSTREAM_EOF )
		{	 	
			rd = (source->Read(&buffer,1024)).LastRead();
			for(unsigned int i=0; i<rd; i++)
				_BitOStream->writeBits(buffer[i],8);
		}
	}
	else if (_HeaderDecInitData.codingType == GZIP)
	{
		//GZIP data encoding
		unsigned int rd;
		char buffer[1024];
		wxFileInputStream gzip(gzipFileName);
		while( gzip.LastError() != wxSTREAM_EOF )
		{	 	
			rd = (gzip.Read(&buffer,1024)).LastRead();
			for(unsigned int i=0; i<rd; i++)
				_BitOStream->writeBits(buffer[i],8);
		}
	}
	else if (_HeaderDecInitData.codingType == BIMF)
	{
		//BIM data encoding
		unsigned int rd;
		char buffer[1024];
		wxFileInputStream bim(bimFileName);
		while( bim.LastError() != wxSTREAM_EOF )
		{	 	
			rd = (bim.Read(&buffer,1024)).LastRead();
			for(unsigned int i=0; i<rd; i++)
				_BitOStream->writeBits(buffer[i],8);
		}
	}
	else
	{
		return -1;
	}

	_BitOStream->writeBits(!isLast, 1);
	_BitOStream->flush();

	return 0;
}

int BitStreamEncoder::writeHeader(bool onlySMRHeader)
{
	if(!onlySMRHeader)
	{
		_BitOStream->writeBits(_HeaderCommonData.audioObjectType, 5);
		_BitOStream->writeBits(_HeaderCommonData.audioObjectTypeExt, 6);
		_BitOStream->writeBits(_HeaderCommonData.samplingFrequencyIndex, 4);
		_BitOStream->writeBits(_HeaderCommonData.channelConfiguration, 4);
	}
	_BitOStream->writeBits(_HeaderCommonData.version, 4);
	_BitOStream->writeBits(_HeaderCommonData.pictureWidth, 12);
	_BitOStream->writeBits(_HeaderCommonData.pictureHeight, 12);
	_BitOStream->writeBits(_HeaderCommonData.isScoreMultiwindow, 1);
	_BitOStream->writeBits(_HeaderCommonData.numberOfParts, 8);
	_BitOStream->writeBits(_HeaderCommonData.notationFormat, 3);
	// MIDI url
	_BitOStream->writeVluimsbf8(_MidiUrl.Length());
	for(unsigned int i=0; i< _MidiUrl.Length(); i++)
		_BitOStream->writeBits(_MidiUrl[i], 8);
	_BitOStream->writeBits(_HeaderDecInitData.codingType, 2);

	_BitOStream->writeVluimsbf8(0); // BiM decoder config length
	if(_HeaderDecInitData.codingType==BIMF)
	{
		_BitOStream->writeBits(1, 3); // 001 default decoderInit
	}

	_BitOStream->writeBits(_HeaderHasNextData.more_data, 1);
	_BitOStream->align();
	return 0;
}

wxInputStream* BitStreamEncoder::encodeSyncInfo(wxInputStream* s)
{
		wxTextInputStream syncStream(*s);
		wxFileOutputStream* outfile=new wxFileOutputStream("sync.bin");

		BitOutputStream out(*outfile);

		wxString line;
		line=syncStream.ReadLine(); //skip the first line

		wxUint32 prevMeasure=0;

		//write 01 
		out.writeBits(1,2);

		while(!s->Eof())
		{
			wxUint32 measure, duration, syncPos;
			char syncType[20];

			line=syncStream.ReadLine();

			int n=sscanf(line.c_str(),"%d %d %d %s", &measure, &duration, &syncPos, syncType);

			if(n>=2 && measure>0)
			{
				
				if(prevMeasure!=0)
				{
					//write more_data=1
					out.writeBits(1,1);
				}
				
				if(measure!=prevMeasure+1)
				{
					//write jump=1
					out.writeBits(1,1);
					
					//write measure number (16bit)
					out.writeBits(measure,16);
				}
				else
				{
					//write jump=0
					out.writeBits(0,1);
				}
				//write duration (16bit)
				out.writeBits(duration,16);
				
				prevMeasure=measure;
			}
		}
		//write more_data=0
		out.writeBits(0,1);
		out.flush();
		delete outfile;
		delete s;

		return new wxFileInputStream("sync.bin");
}

int LoadTrueTypeFont(char* buffer, int size, char* fontName, void** hfont);

wxInputStream* BitStreamEncoder::encodeFont(wxInputStream* s)
{
	wxString tmpFontFile=wxString(getenv("TEMP"))+"\\font.bin";
	wxFileOutputStream* outfile=new wxFileOutputStream(tmpFontFile);
	
	char fontName[255];
	char* buffer;
	int size=s->GetSize();
	
	buffer=new char[size];
	size_t rd=s->Read(buffer, size).LastRead();
	
	LoadTrueTypeFont(buffer, size, fontName, NULL);
	
	//encoding as EnhancedFontAccessUnit 14496-18 clause 5.2.2
	BitOutputStream out(*outfile);
	out.writeBits(1,7); //fontFormat 1=OpenType with true type outline (uncompressed)
	out.writeBits(1,1); //storeFont
	out.writeBits(strlen(fontName),8);
	for (unsigned int i=0; i<strlen(fontName); i++)
		out.writeBits(fontName[i],8);
	out.writeBits(0, 7); //fontSubsetID (0=all font)
	out.writeBits(0,1);  //fontSubsetExtensionFlag 0=it is not an extension of a previous font AU
	out.writeBits(0, 8); //fontSpecInfoLength -- no font spec info
	
	for(i=0; i<rd; i++)
		out.writeBits(buffer[i],8);
	delete[] buffer;
	
	return new wxFileInputStream(tmpFontFile);
}

SABitStreamEncoder::SABitStreamEncoder(wxOutputStream* OStream)
{
	_OStream = OStream;
	_BitOStream = new BitOutputStream(*_OStream);
}
SABitStreamEncoder::~SABitStreamEncoder()
{
	delete _BitOStream;
}
int SABitStreamEncoder::writeDownDecoderConfig(int AOT, wxInputStream* midi)
{
	_BitOStream->writeBits(AOT, 5);
	_BitOStream->writeBits(4, 4); //SamplingFreq. Index
	_BitOStream->writeBits(1, 4); //channel config
	_BitOStream->writeBits(1,1); //more data
	_BitOStream->writeBits(2,3); //type=midifile

	char* buffer;
	int size=midi->GetSize();	
	buffer=new char[size];
	size_t rd=midi->Read(buffer, size).LastRead();

	_BitOStream->writeBits(rd, 32);
	for(int i=0; i<rd; i++)
		_BitOStream->writeBits(buffer[i],8);
	delete[] buffer;

	return 0;
}

ODStreamEncoder::ODStreamEncoder(wxOutputStream* OStream)
{
	_OStream = OStream;
	_BitOStream = new BitOutputStream(*_OStream);
}
ODStreamEncoder::~ODStreamEncoder()
{
	delete _BitOStream;
}

static int vluimsbf8Size(int v)
{
	if(v< (1 << 7))
		return 1;
	if(v< (1 << 14))
		return 2;
	if(v< (1 << 21))
		return 3;
	return 4;
}

int ODStreamEncoder::writeDownStream(ODMap* ods)
{
	std::vector<int> oids=ods->getOIDs();
	std::vector<int> odSize, esSize, decoderConfigSize;
	int odUpdateSize = 0;
	int i;
	
	//precalculate sizes
	for(i=0; i<oids.size(); i++)
	{
		ESData* esData = ods->getESData(oids[i]);
		int size1=1+1+3+4+4+1+vluimsbf8Size(esData->getDecoderInfoSize())+esData->getDecoderInfoSize();
		decoderConfigSize.push_back(size1);
		int size2=2+1+1+vluimsbf8Size(decoderConfigSize[i])+decoderConfigSize[i]+1+1+1;
		esSize.push_back(size2);
		int size3=2+1+vluimsbf8Size(esSize[i])+esSize[i];
		odSize.push_back(size3);

		odUpdateSize += 1+vluimsbf8Size(odSize[i])+odSize[i];
	}

	//write an ODUpdate command
	_BitOStream->writeBits(1, 8); //OD Update command
	_BitOStream->writeVluimsbf8(odUpdateSize);
	for(i=0; i<oids.size(); i++)
	{
		_BitOStream->writeBits(1, 8); //OD
		_BitOStream->writeVluimsbf8(odSize[i]);
		_BitOStream->writeBits(oids[i], 10); //OID
		_BitOStream->writeBits(0, 1); //no URL flag
		_BitOStream->writeBits(31,5); //11111

		_BitOStream->writeBits(3, 8); //ES_Descriptor
		_BitOStream->writeVluimsbf8(esSize[i]);

		ESData* esData = ods->getESData(oids[i]);

		_BitOStream->writeBits(esData->getESID(), 16); //ESID
		_BitOStream->writeBits(0, 1); //no Stream Dependence
		_BitOStream->writeBits(0, 1); //no URL
		_BitOStream->writeBits(0, 1); //no OCR Stream
		_BitOStream->writeBits(16, 5); //stream priority

		_BitOStream->writeBits(4,8); //DecoderConfigDescrTag
		_BitOStream->writeVluimsbf8(decoderConfigSize[i]);

		_BitOStream->writeBits(esData->getOTI(), 8); //ObjectTypeIndication
		_BitOStream->writeBits(esData->getStreamType(), 6); //StreamType
		_BitOStream->writeBits(0, 1); //no upstream
		_BitOStream->writeBits(1, 1); //reserved

		_BitOStream->writeBits(esData->getBufferSizeDB(), 24);
		_BitOStream->writeBits(esData->getMaxBitRate(), 32);
		_BitOStream->writeBits(esData->getAvgBitRate(), 32);

		_BitOStream->writeBits(5, 8); //DecSpecificInfoTag
		_BitOStream->writeVluimsbf8(esData->getDecoderInfoSize());
		for(int j=0; j<esData->getDecoderInfoSize(); j++)
			_BitOStream->writeBits(esData->geDecoderInfo()[j], 8);

		//SLInfo
		_BitOStream->writeBits(6, 8); //SLConfigDescrTag
		_BitOStream->writeVluimsbf8(1);
		_BitOStream->writeBits(2, 8); //predefined = 2
	}

	return 0;
}
