/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#ifndef BITSTREAMENCODER_HPP
#define BITSTREAMENCODER_HPP

#include <wx/stream.h>
#include <wx/string.h>
#include <wx/list.h>

#include "def_bit.hpp"

class DataChunk
{
protected:
	Chunk_Type _type;
	wxInputStream* _source;
	unsigned int _partid;
	unsigned int _lyricid;

public:
	DataChunk(Chunk_Type Type, wxInputStream* Source, unsigned int PartID = 0, unsigned int LyricID = 0)
		{_type = Type; _source = Source; _partid = PartID; _lyricid = LyricID; };
	
	~DataChunk(){delete _source;};

	Chunk_Type		GetType()			{return _type; };
	wxInputStream*	GetSource()			{return _source; };
	unsigned int	GetPartID()			{return _partid; };
	unsigned int	GetLyricID()		{return _lyricid; };
};

WX_DECLARE_LIST(DataChunk, ChunkList);

///////////////////////////////////////

class BitOutputStream;

class BitStreamEncoder
{
public:
	BitStreamEncoder(wxOutputStream* OStream);
	~BitStreamEncoder();

	void setOStream(wxOutputStream* OStream);
	void AddSource(Chunk_Type Type, wxInputStream* Source, int PartID = 0);

	int writeDownDecoderConfig(bool onlySMRConfig = false);
	int writeDownAccessUnit();

	void setVersion(unsigned int ver)					{_HeaderCommonData.version = ver;};
	void setPictureWidth(unsigned long width)			{_HeaderCommonData.pictureWidth = width;};
	void setPictureHeight(unsigned long height)			{_HeaderCommonData.pictureHeight = height;};
	void setMultiwindow(bool isMultiWin)				{_HeaderCommonData.isScoreMultiwindow = isMultiWin;};
	void setNotationFormat(NotationFormat_Type format)	{_HeaderCommonData.notationFormat = format;};
	void setCodingType(Coding_Type coding)				{_HeaderDecInitData.codingType = coding;};
	void setNumberOfParts(int nparts)					{_HeaderCommonData.numberOfParts = nparts;};
	void setMidiUrl(wxString midiUrl)					{_MidiUrl = midiUrl;};

	wxInputStream* encodeSyncInfo(wxInputStream*);
	wxInputStream* encodeFont(wxInputStream*);

private:
	wxOutputStream*		_OStream;
	BitOutputStream*	_BitOStream;
	
	ChunkList			cList;
	
	/*HEADER*/
	HEADER_COMMON		_HeaderCommonData;
	HEADER_DECODER		_HeaderDecInitData;
	CHUNK_HASNEXT		_HeaderHasNextData;
	wxString			_MidiUrl;

	int writeHeader(bool onlySMRHeader);
	int writeChunk( DataChunk* data, bool isLast );
};

class SABitStreamEncoder
{
public:
	SABitStreamEncoder(wxOutputStream* OStream);
	~SABitStreamEncoder();

	void setOStream(wxOutputStream* OStream);

	int writeDownDecoderConfig(int AOT, wxInputStream* midi);

private:
	wxOutputStream*		_OStream;
	BitOutputStream*	_BitOStream;
};

class ODMap;

class ODStreamEncoder
{
public:
	ODStreamEncoder(wxOutputStream* OStream);
	~ODStreamEncoder();

	void setOStream(wxOutputStream* OStream);

	int writeDownStream(ODMap*);
private:
	wxOutputStream*		_OStream;
	BitOutputStream*	_BitOStream;
};
#endif