/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: pargraff.cpp,v 1.1 2005/07/25 11:06:30 bellinpo Exp $ */


#include "bra/pargraff.hpp"

ParGraffa::ParGraffa() 
{
	ID=CL_PARGRAFFA;
	Width = WIDTH_PARENTESI;
	Height = 0;
	BitMap = NULL;
};


ParGraffa::ParGraffa(VUnit h)
{ 
	ID=CL_PARGRAFFA;
	Width = WIDTH_PARENTESI;
	Height = 0;
	BitMap = NULL;
	Init(h);
};


ParGraffa::~ParGraffa() 
{
	if(BitMap!=NULL)
		delete[] BitMap;
}


void ParGraffa::Init(VUnit h)
{  
	VUnit H2, X, YPix;
	float x, y;
	float Cost1, Cost2;
	
	if(h>0 && Height!=h)
	{
		Height = h;
		
		// La funzione x = y^2 - y^3 ha massimo ~.15 nell'intervallo [-1/3, 1].
		// Per il cambiamento di scala si usa .15.
		Cost1 = (float)Width / float(0.15);
		Cost2 = ((float)(Width - 1)) / float(0.15);
		H2 = Height / 2;
		if(BitMap!=NULL)
			delete[] BitMap;
		BitMap = new unsigned char[H2+1];
		if (BitMap==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ParGraffa","Init()");
		
		
		for(YPix = 0; YPix<=H2; YPix++)
		{
			BitMap[YPix] = 0;
			// 4/3 e` l'ampiezza dell'intervallo sul quale e` calcolata la funzione.
			y = (4*(float)YPix/H2 -1)/3;
			x = y*y*(1-y);
			X = Width - 1 - (VUnit)(Cost1*x);
			if(X<8)
				BitMap[YPix] |= (1<<X); 
			// Ispessimento della parentesi nella curva.
			if(y>0)
			{
				X = Width - 1 - (VUnit)(Cost2*x);
				if(X<8)
					BitMap[YPix] |= (1<<X);
			}
		}
	}    
} 


VUnit ParGraffa::GetVU2Up()
{ return Height; };

VUnit ParGraffa::GetVU2Dwn()
{ return 0; };

VUnit ParGraffa::GetVU2Rgt()
{ return Width; };

VUnit ParGraffa::GetVU2Lft()
{ return 0; };


void ParGraffa::Draw()
{
	Point p1, p2;  
	
	if(BitMap==NULL)
		return;
	
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

#ifdef USE_GDIPLUS
	Scr->Brace(GetAbsPos()+Point(Width,Height/2), Height);
#else
	H2 = Height / 2;
	for(YPix = 0; YPix<=H2; YPix++)
	{
		p1.y = GetAbsPos().y + H2 + YPix;
		p2.y = GetAbsPos().y + H2 - YPix;
		for(i=0; i<Width; i++)
		{
			if(((BitMap[YPix]>>i)&1) == 1)
			{
				p1.x = p2.x = GetAbsPos().x + i;
				Scr->PutPixel(p1);
				Scr->PutPixel(p2);
			}
		}
	}
#endif

	Scr->SetColor(oldcol);
} 



void ParGraffa::Draw4single(int d4s_x, int d4s_y)
{
	VUnit YPix, H2;
	Point p1, p2;  
	short i;
	
	if(BitMap==NULL)
		return;
	
	if(isOpaque)
		Scr->SetColor(GetColor());
	if ((d4s_y!=0)&&(d4s_x!=0))
	{
		H2 = Height / 2;
		for(YPix = 0; YPix<=H2; YPix++)
		{
			p1.y = d4s_y - H2 + YPix;
			p2.y = d4s_y - H2 - YPix;
			for(i=0; i<Width; i++)
			{
				if(((BitMap[YPix]>>i)&1) == 1)
				{
					p1.x = p2.x = d4s_x + i - Width - 2;
					Scr->PutPixel(p1);
					Scr->PutPixel(p2);
				}
			}
		}
	}
} 