/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: bezier.cpp,v 1.1 2005/07/25 16:30:28 bellinpo Exp $ */


#include "stdio.h"
#include "stdlib.h"
#include "string.h"

#include "drw/bezier.hpp" 
#include "drw/point.hpp"




/**************************************************************/
Bezier::Bezier(void )
{ 
	int i;
	for (i=0;i<4;i++) xv[i]=yv[i]=0.0;
	res = RESDEFAULT;
	//SetColor((ColorIcon)NORMAL_COLOR);	//FFF
	status = CURVEINVALID;
}

/**************************************************************/
Bezier::Bezier( double x[], double y[])
{ 
	int i,ret;
	for (i=0;i<4;i++) { xv[i]=x[i]; yv[i]=y[i]; }
	ret=ChkCurve();
}

/**************************************************************/
Bezier::Bezier( Point p1, Point p2, Point p3, Point p4, int col)
{ 
	int ret;
	xv[0]=p1.x;  yv[0]=p1.y; 
	xv[1]=p2.x;  yv[1]=p2.y; 
	xv[2]=p3.x;  yv[2]=p3.y; 
	xv[3]=p4.x;  yv[3]=p4.y; 
	res = RESDEFAULT;
	
	ret=ChkCurve();
	
}

/**************************************************************/
int Bezier::ChkCurve(void )
{ 
	double tmp;
	int i;
	if (0.0 <= xv[0] && xv[0]<xv[1] && xv[1]<xv[2] && xv[2]<xv[3]) 
	{ 
		for (i=1;i<4;i++)
		{ 
			xv[i]=xv[i]-xv[0];
			yv[i]=yv[i]-yv[0];
		}
		status = CURVEVALID; return(CURVEVALID); 
	}
	else if (xv[0]>xv[1] && xv[1]>xv[2] && xv[2]>xv[3] && xv[3]>=0.0) 
	{ 
		tmp=xv[0]; xv[0]=xv[3]; xv[3]=tmp;
		tmp=yv[0]; yv[0]=yv[3]; yv[3]=tmp;
		tmp=xv[1]; xv[1]=xv[2]; xv[2]=tmp;
		tmp=yv[1]; yv[1]=yv[2]; yv[2]=tmp;
		for (i=1;i<4;i++)
		{ 
			xv[i]=xv[i]-xv[0];
			yv[i]=yv[i]-yv[0];
		}
		status = CURVEVALID; return(CURVEVALID); 
	}
	else
	{ status = CURVEINVALID;  return(CURVEINVALID);  }
}

/**************************************************************/
int Bezier::SetCurve( double x[], double y[])
{ 
	int i,ret;
	for (i=0;i<4;i++) { xv[i]=x[i]; yv[i]=y[i]; }
	ret=ChkCurve();
	return(status);
}

/**************************************************************/
void Bezier::SetCurveResolution(double r)
{ res = r; }

/**************************************************************/
double Bezier::GetCurveResolution(void )
{ return (res); }


/**************************************************************/
double Bezier::YdiT(double t) /* t varia fra 0 ed 1 */
{
	double val;
	if (t<0 || t>1) return (OUTOFT);
	val= t*t*t*(3.0*yv[1]-3.0*yv[2]+yv[3]);
	val = val + t*t*(-6.0*yv[1]+3*yv[2]) + t*3.0*yv[1];
	return (val);
}

/**************************************************************/
double Bezier::YprimediT(double t) /* t varia fra 0 ed 1 */
{
	double val;
	if (t<0 || t>1) return (OUTOFT);
	val= 3.0*t*t*(3.0*yv[1]-3.0*yv[2]+yv[3]);
	val = val + 2.0*t*(-6.0*yv[1]+3*yv[2]) + 3.0*yv[1];
	return (val);
}

/**************************************************************/
double Bezier::TforYmax(void) /* t varia fra 0 ed 1 */
{
	double val,den,delta;
	den = (3.0*yv[1]-3.0*yv[2]+yv[3]);
	delta = (-2.0*yv[1] + yv[2])*(-2.0*yv[1] + yv[2]);
	delta = delta - (3.0*yv[1]-3.0*yv[2]+yv[3])*yv[1];
	val = (2.0*yv[1]-yv[2]-sqrt(delta) )/den;
	return(val);
}

/**************************************************************/
double Bezier::Ymax(void) /* t varia fra 0 ed 1 */
{ 
	double val;
	val = TforYmax();
	val = YdiT(val) + yv[0];
	return(val);
}

/**************************************************************/
void Bezier::Draw(void)           // disegna spline
{
	double ind;
	int x,y;
	
	res = xv[3]/4.0;
	if(isOpaque)
		Scr->SetColor(GetColor());
	
	Scr->MoveTo((short)xv[0],(short)yv[0]);
	for (ind=0;ind<=1;ind=ind+res)
	{
		x = (short) (ind*xv[3] + xv[0]);
		y = (short) (YdiT(ind) + yv[0]);
		Scr->LineTo((short)x,(short)y,SOLID_LINE);
	}
}
