/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: bezier.hpp,v 1.1 2005/07/25 16:30:28 bellinpo Exp $ */


#ifndef BEZIER_HPP
#define BEZIERHXX

#define PSTART 0
#define PPRIMO  1
#define PSECONDO 2
#define PEND   3

#define RESDEFAULT 0.05

#define CURVEVALID 0
#define CURVEINVALID 1 

#define OUTOFT -99999

#include "lioo.hpp"
#include "drw/drawobje.hpp"

class  Bezier : public DrawObject
{
protected:
	double xv[4], yv[4]; 
	// 0: punto iniziale (alla sinistra)
	// 1: primo punto intermedio
	// 2: secondo punto intermedio
	// 3: punto finale
	// e' stato scelto di farlo in double perche' sara' utile per la stampa
	double res; // risoluzione per il disegno
	// step per la scansione di t da 0 a 1 durante il disegno
	int status; /* puo' essere valido e invalido */
	
public:
	
	Bezier(void);
	Bezier( double xv[], double yv[]);
	Bezier( Point, Point, Point, Point, int col=NORMAL_COLOR);
	
	int ChkCurve(void); /* 1 se non va, 0 se ok */
	/* questa funzione scambia l'ordine se e' opposto */
	/* e dice se la curva puo' andare o no */
	
	int SetCurve(double xv[], double yv[]);
	void SetCurveResolution(double res);
	
	double GetCurveResolution(void);
	
	double YdiT(double t); /* t varia fra 0 ed 1 */
	double YprimediT(double t); /* t varia fra 0 ed 1 */
								/* il ritorno deve essere diverso da OUTOFT
								se negativo e' stato dato un t sbagliato */
	
	double TforYmax(void); /* t varia fra 0 ed 1 */
	double Ymax(void); 

	Point GetFrom() { return Point(xv[0],yv[0]); };
	Point GetTo()   { return Point(xv[3]+xv[0],yv[3]+yv[0]); };
	Point GetCP1()  { return Point(xv[1]+xv[0],yv[1]+yv[0]); };
	Point GetCP2()  { return Point(xv[2]+xv[0],yv[2]+yv[0]); };
	
	void Draw(void);           // disegna spline
};

#endif


