/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: button.cpp,v 1.1 2005/07/25 16:30:28 bellinpo Exp $ */


#include "drw/button.hpp"

#include "drw/scrlist.hpp"

extern ScrList LiooListWidget;
ExtLiooAbil liooAbil;

Button::Button() 
{
	Enabled = TRUE;
	ID=CL_BUTTON;
	text=NULL; 
	font=-1; 
	height=0; 
	width=0; 
	drwFun=NULL; 
	actFun=NULL; 
	BackColor = (ColorIcon)gray;
	SetOn(); 
	
	SetScrWindow(NULL);
	SetScrDrawArea(NULL);
}


Bool Button::GetEnabled()
{
	return Enabled;
}

void Button::SetEnabled(Bool b)
{
	Enabled = b;
}

AbilID Button::GetAbilID()
{
	return myname;
}

void Button::SetAbilID(AbilID nome)
{
	myname=nome;
}

VUnit Button::GetVU2Up() 
{ 
	return height; 
}

VUnit Button::GetVU2Dwn() 
{ 
	return 0; 
}

VUnit Button::GetVU2Lft() 
{ 
	return 0; 
}

VUnit Button::GetVU2Rgt() 
{ 
	return width; 
}

void Button::SetWidth(VUnit w)
{ 
	width=w; 
}

void Button::SetHeight(VUnit h)
{ 
	height=h; 
}

void Button::SetOn()
{
	On=TRUE;
}

void Button::SetOff()
{
	On=FALSE;
}

void Button::Toggle()
{
	On=!On;
}

Bool Button::isOn()
{ 
	return On;
}


void Button::InitText(unsigned char *txt, int fnt, VUnit hgt, VUnit wdt, ActionFunction afun, Bool DrawArr, short bc)
{
	DrawArrow = DrawArr;
	BackColor = (ColorIcon)bc;
	
	text = txt;
	font = fnt;
	actFun = afun;
	
	
	height = hgt;
	width = wdt;
	
	drwFun = NULL;
	SetOn();
}

void Button::InitImage(DrawFunction dfun,VUnit hgt, VUnit wdt,ActionFunction afun, Bool DrawArr, short bc)
{
	DrawArrow = DrawArr;
	BackColor = (ColorIcon)bc;
	text=NULL;
	font=-1;
	
	height=hgt;
	width=wdt;
	
	actFun=afun;
	drwFun=dfun;
}

void Button::ChangeText(unsigned char *txt)
{
	Rectangle r;
	FontID ofont;
	
	if (drwFun!=NULL) 
		EM.Error(MUSICAL_ERR_AREA,"EC_BUTTON_GESTIONE","Button","ChangeText()");
	
	text=txt;
	
	Scr->GetMouse()->Hide();
	
	Scr->SetColor(BackColor);
	r=Rectangle(*this);
	Scr->Rect(r,TRUE);
	
	if (liooAbil.isAbil(myname)) Scr->SetColor(GetColor());
	else Scr->SetColor(Colore((ColorIcon)white)); 
	ofont=Scr->SetFont(font);
	
//	Scr->TextTo(GetAbsPos()+Point((width-Scr->GetTextWidth(text))/2,(height-Scr->GetFontHeight())/2+Scr->GetFontHeight()-Scr->GetFontAscent()));
		Scr->TextTo( GetAbsPos()+ Point( (width-Scr->GetTextWidth(text))/2,
			( (height- (height*Scr->GetFontAscent()) / Scr->GetFontHeight()) ) ) );
	
	Scr->OutText((unsigned char *)text);
	Scr->SetFont(ofont);
	
	Scr->ShadowRect(r,isOn());
	Scr->GetMouse()->Show();
}

void Button::ChangeText(unsigned char *txt, int fnt)
{
	Rectangle r;
	FontID ofont;
	
	font = fnt;
	
	if (drwFun!=NULL) 
		EM.Error(MUSICAL_ERR_AREA,"EC_BUTTON_GESTIONE","Button","ChangeText()");
	
	text=txt;
	
	Scr->GetMouse()->Hide();
	
	Scr->SetColor(BackColor);
	r=Rectangle(*this);
	Scr->Rect(r,TRUE);
	
	if (liooAbil.isAbil(myname)) Scr->SetColor(GetColor());
	else Scr->SetColor(Colore((ColorIcon)white)); 
	ofont=Scr->SetFont(font);
	
//	Scr->TextTo(GetAbsPos()+Point((width-Scr->GetTextWidth(text))/2,(height- (height*Scr->GetFontAscent()) / Scr->GetFontHeight())));
		Scr->TextTo( GetAbsPos()+ Point( (width-Scr->GetTextWidth(text))/2,
			( (height- (height*Scr->GetFontAscent()) / Scr->GetFontHeight()) ) ) );
	
	Scr->OutText((unsigned char *)text);
	Scr->SetFont(ofont);
	
	Scr->ShadowRect(r,isOn());
	Scr->GetMouse()->Show();
}


void Button::ChangeImage(DrawFunction dfun)
{
	Rectangle r;
	
	if (text!=NULL) 
		EM.Error(MUSICAL_ERR_AREA,"EC_BUTTON_GESTIONE","Button","ChangeImage()");
	drwFun=dfun;
	Draw();
}

void Button::Draw()
{                
	Rectangle r;
	Scr->GetMouse()->Hide();
	Scr->SetDrwMode(DefMode);
	Scr->SetColor(BackColor);
	r=Rectangle(*this);
	Scr->Rect(r,TRUE);
	
	if (liooAbil.isAbil(myname)) Scr->SetColor(GetColor());
	else Scr->SetColor(Colore((ColorIcon)white)); 
	
	char code[2]={254,'\0'};
	if (DrawArrow)
	{
		int ofont;
		ofont=Scr->SetFont(nTextFont);
		
		Scr->TextTo(GetAbsPos() + Point(GetWidth()*3/4,5));
		Scr->OutText((unsigned char *)code);
		
		Scr->SetFont(ofont);
	}
	
	
	if(drwFun!=NULL)
		(*drwFun)(r);
	else
	{
		int ofont;
		ofont=Scr->SetFont(font);
		
		/*
		Scr->TextTo(GetAbsPos()+Point((width-Scr->GetTextWidth(text))/2,
			(height-Scr->GetFontHeight())/2+Scr->GetFontHeight()-Scr->GetFontAscent()));
		*/
		Scr->TextTo( GetAbsPos()+ Point( (width-Scr->GetTextWidth(text))/2,
			( (height- (height*Scr->GetFontAscent()) / Scr->GetFontHeight()) ) ) );
		
		Scr->OutText((unsigned char *)text);                               
		Scr->SetFont(ofont);
	}
	
	if (Enabled) Scr->ShadowRect(r,isOn());
	Scr->GetMouse()->Show();
}

EventResult Button::LeftButtonPress(const Point& p)
{
	if (!(liooAbil.isAbil(myname)))
		return ER_NO_HIT;
	
	if (!Enabled)
		return ER_NO_HIT;
	
	if(Hit(p))
	{
		SetOff();
		if (Scr->GetMouse()->IsShowing())
		{
			Scr->GetMouse()->Hide();
			Scr->ShadowRect(Rectangle(*this),isOn());
			Scr->GetMouse()->Show();
		}
		else
			Scr->ShadowRect(Rectangle(*this),isOn());
		return ER_CONTINUE;
	}
	return ER_NO_HIT;
}

EventResult Button::LeftButtonRelease(const Point& p)
{
	if (!(liooAbil.isAbil(myname)))
		return ER_NO_HIT;
	
	if (!Enabled)
		return ER_NO_HIT;
	
	if(!isOn())
	{      
		SetOn();
		if (Scr->GetMouse()->IsShowing())
		{
			Scr->GetMouse()->Hide();
			Scr->ShadowRect(Rectangle(*this),isOn());
			Scr->GetMouse()->Show();
		}
		else
			Scr->ShadowRect(Rectangle(*this),isOn());
	}
	
	if(Hit(p))
	{ 
		ScrObject *scrMenu;
		
		if (GetScrWindow()!=NULL)
		{
			scrMenu = LiooListWidget.GetNodoW(GetScrWindow());
		}  
		if (GetScrWindow()!=NULL && scrMenu!=NULL)
		{ 
			Scr->GSetFocusWindow(GetScrWindow());
		}
		else
		{  
			if (actFun!=NULL)
			{ 
				EventResult r = (*actFun)();
				Scr->GSetMainDrawArea();
				return r;
			}
			else
			{ 
				Scr->GSetMainDrawArea();
				return ER_CONTINUE;
			}
			Scr->GSetMainDrawArea();
			return ER_NO_HIT;
		}
	}
	return ER_NO_HIT;
}

