/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: button.hpp,v 1.1 2005/07/25 16:30:28 bellinpo Exp $ */


#ifndef BUTTON_HPP
#define BUTTON_HPP

#include "drw/scrobjec.hpp"
#include "gui/extabil.hpp"

typedef void (*DrawFunction)(const class Rectangle&);
typedef EventResult (*ActionFunction)();

class Button : public ScrObject
{ 
protected:
	
	AbilID myname;
	// attributo che indica la sua attivazione oppure no
	
    unsigned char *text;
	// stringa del testo da visualizzare nel bottone.
	// se e' NULL il bottone non ha testo.
    FontID font;
	// identificatore del font da utilizzare per scrivere il testo.
    VUnit height,width;
	// altezza e larghezza del bottone.
    VUnit dW;
	// contiene il numero di pixel di cui deve essere aumentata la
	// larghezza del testo per ottenere la larghezza effettiva.
    VUnit dY;
	// indica lo scostamento in y del testo nel bottone.
    DrawFunction drwFun;
	// Puntatore alla funzione per disegnare il bottone.
	// Se e' NULL viene esaminato il testo.
	// Alla funzione viene passato come parametro il rettangolo 
	// che contiene il bottone.
    ActionFunction actFun;
	// Puntatore alla funzione da chiamare quando il bottone viene premuto.
	// Se e' NULL non viene richiamata nessuna funzione.
    Bool On;
	// Se vale TRUE il bottone e' rialzato altrimenti e' abbassato.
	
    Bool DrawArrow;
	// Se vale TRUE il bottone apre un altro menu
	
    Bool Enabled;
	// Se vale TRUE il bottone e' abilitato
	
    Colore BackColor;
	// Colore con cui disegnare l'oggetto
	
public:        
    Button();
	
    VUnit GetVU2Up();
    VUnit GetVU2Dwn();
    VUnit GetVU2Lft();
    VUnit GetVU2Rgt();
	void SetAbilID(AbilID );
	AbilID GetAbilID();
    
	Bool GetEnabled();
	void SetEnabled(Bool);
	
	
    void InitText(unsigned char *txt,FontID fnt,VUnit hgt, VUnit wdt,ActionFunction, Bool DrawArr = FALSE, short bc = gray);
	// Inizializza il bottone come bottone con testo.
    void InitImage(DrawFunction,VUnit hgt, VUnit wdt,ActionFunction, Bool DrawArr = FALSE, short bc = gray);
	// Inizializza il bottone come bottone con funzione propria di disegno.
	
    void ChangeText(unsigned char *txt); // USA IL FONT INIZIALE
	void ChangeText(unsigned char *txt, int fnt); // cambia il font
	// Cambia il testo del bottone e lo ridisegna.
	// Valido se iniziazlizzato con InitText.
    void ChangeImage(DrawFunction);
	// Cambia la funzione per disegnare il bottone e lo ridisegna.
	// Valido se iniziazlizzato con InitImage.
    void SetWidth(VUnit w);  
	// Imposta la larghezza del bottone.
    void SetHeight(VUnit h);  
	// Imposta l'altezza del bottone.
    void SetOn();
	// Imposta il bottone come rialzato
    void SetOff();
	// Imposta il bottone come abbassato.
    void Toggle();
	// Inverte lo stato del bottone.
    Bool isOn();
	// Ritorna TRUE se il bottone e' rialzato.
    void Draw();
	// Disegna il bottone.
    EventResult LeftButtonPress(const Point&);
	// Serve a notificare al bottone che e' stato premuto il bottone del mouse 
	// nel punto indicato. 
	// Ritorna TRUE se il bottone e' stato colpito.
    EventResult LeftButtonRelease(const Point&);
	// Serve a notificare al bottone che e' stato rilasciato il bottone del mouse
	// nel punto indicato.
	// Se il bottone e' abbassato (On==FALSE) e il punto e' interno al
	// bottone allora richiama la funzione di azione e ritorna il valore
	// da essa prodotto. Se la funzione non e' stata definita (actFun==NULL)
	// allora ritorna il valore 1. Se invece il punto non e' interno
	// ritorna il valore 0.
};

#endif // BUTTON_HPP
