/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: drawobje.cpp,v 1.1 2005/07/25 16:30:28 bellinpo Exp $ */



#include "drw/drawobje.hpp"
#include "lst/lista.hpp"

int DrawObject::currentView=0;

DrawObject::DrawObject() 
{ 
	ID=CL_DRAWOBJECT;
	Color = (ColorIcon)NORMAL_COLOR;
	isSelected = FALSE;
	isOpaque = FALSE;
	markType=0;
	markVariant=0;
	VIPUserData=0;
	TraceMilla=FALSE;
	for(int i=0;i<2;i++)
	{
		dxMainScore[i]=0;
		dyMainScore[i]=0;
		dxPart[i]=0;
		dyPart[i]=0;
	}
}

ClassID DrawObject::GetID() 
{ 
	return ID;
}

void DrawObject::SetID(ClassID id) 
{ 
	ID = id;
}

void DrawObject::SetColor(const ColorIcon& col)
{ 
	Color = col;
	isOpaque = TRUE;
}

void DrawObject::SetColor(unsigned char r, unsigned char g, unsigned char b)
{
	Color.Set(r, g, b);
	isOpaque = TRUE;
}

/*
int DrawObject::GetColor()
{
	return Color;
}
*/
/*
void DrawObject::SetOldColor(const ColorIcon& col)
{
	OldColor = col;
	return;
}

int DrawObject::GetOldColor(void)
{
	
	return OldColor;
}
*/
void DrawObject::SetColor(const Colore& col)
{ 
	Color = col;
	isOpaque = TRUE;
}

void DrawObject::UnsetColor(){
	isOpaque = FALSE;
}


const Colore& DrawObject::GetColor()
{
	return Color;
}
/*
void DrawObject::SetOldColor(const Colore& col)
{
	OldColor = col;
	return;
}

const Colore& DrawObject::GetOldColor(void)
{
	return OldColor;
}
*/

VUnit DrawObject::GetWidth()
{
	return GetVU2Lft()+GetVU2Rgt();
}

VUnit DrawObject::GetHeight()
{
	return GetVU2Up()+GetVU2Dwn();
}

void DrawObject::SetPos(DrawObject *d,const Point& p)
{
	if(d!=NULL)
		AbsPos[currentView]=d->GetAbsPos()+p;
	else
		AbsPos[currentView]=p;
}

void DrawObject::Move(const Point& p)
{
	AbsPos[currentView]+=p;
}

void DrawObject::Select()
{
	isSelected = TRUE;
	//SetOldColor(Color);
	//
	//OldOpacity = isOpaque;
	//SetColor(Colore((ColorIcon)SELECTED_COLOR));
	Draw();

}

void DrawObject::DrawContour(VUnit width){

	Scr->SetColor((ColorIcon)SELECTED_COLOR);
	/*
	Scr->BigRect(class Rectangle(GetAbsPos(), 
		GetVU2Lft() + width,
		GetVU2Rgt() + width, 
		GetVU2Up() + width, 
		GetVU2Dwn() + width),
		3);*/
	Scr->TransparentRect(class Rectangle(GetAbsPos(), 
		GetVU2Lft(),
		GetVU2Rgt()-1, 
		GetVU2Up(), 
		GetVU2Dwn()), 190);
}


void DrawObject::Deselect()
{
	isSelected = FALSE;
	//SetColor(OldColor);
	//isOpaque = OldOpacity;
	Draw();
}

Bool DrawObject::Hit(const Point &p)
{
	return Rectangle(*this).hasIn(p-Point(GetDx(),GetDy()));
}

Bool DrawObject::Hit(const Rectangle &rr)
{
	return Rectangle(*this).isIn(rr);
}

Bool DrawObject::Hit(const Point& p, SelObj selObject,DrawObject*& drwObj,
					 SymPath& symPath)
{
	drwObj=NULL;
	
  if(Rectangle(*this).hasIn(p-Point(GetDx(),GetDy())))
	{
		if(selObject==ANY_SEL)
		{  
			drwObj=this;
			symPath.Array[SIMBOLO_PATH_LEVEL]=GetID();
		}
		return TRUE;
	}
	return FALSE;
}

Bool DrawObject::Hit(const Rectangle& rr, SelObj selObject, SymPath& sp1,
					 SymPath& sp2)
{
	if(Rectangle(*this).isIn(rr))
	{
		return TRUE;
	}
	return FALSE;
}

Bool DrawObject::SymbolicHit(SymPath& symPath, SelObj selObject, 
							 DrawObject*& drwObj)
{
	
	drwObj=NULL;
	if(symPath.Array[SIMBOLO_PATH_LEVEL]==GetID())
	{
		if(selObject==ANY_SEL)
			drwObj=this;
		return TRUE;
	}
	return FALSE;
}


CmdResult DrawObject::Command(CmdID,CmdArgV)
{ 
	EM.Error(GENERAL_ERR_AREA,"EC_COMMAND","DrawObject","Command()");
	return CMD_UNKNOWN;
}

CmdResult DrawObject::SymCommand(SymCmd&, Battuta *pBattuta)
{ 
	EM.Error(GENERAL_ERR_AREA,"EC_SYMCOMMAND","DrawObject","SymCommand()");
	return CMD_UNKNOWN;
}

//*BIAN-VERR*
CmdResult DrawObject::GetSymCode(SymCmd&, Battuta *pBattuta)
{ 
	EM.Error(GENERAL_ERR_AREA,"EC_SYMCOMMAND","DrawObject","SymCommand()");
	return CMD_UNKNOWN;
}
//*BIAN-VERR*


DrawObject::operator Rectangle()
{
	return Rectangle(AbsPos[currentView],GetVU2Lft(),GetVU2Rgt(),GetVU2Up(),GetVU2Dwn());
}

Bool DrawObject::DeleteSym(DrawObject* p)
{
	return FALSE;
}

Bool DrawObject::is(ClassID CLId)
{
	return(ID==CLId); 
}

void DrawObject::Edit(Battuta *pBat)
{
	Message("Editing...");
}

Bool DrawObject::GetTraceMilla()
{
	return TraceMilla;
	
}

void DrawObject::SetTraceMilla(Bool t)
{
	TraceMilla=t;
	
}

VUnit DrawObject::GetDx(bool isMainScore, bool isPrintView)
{
	if(isMainScore)
		return VU_PER_ROW*GetDxMainScore(isPrintView);
	else
		return VU_PER_ROW*GetDxPart(isPrintView);
} 

VUnit DrawObject::GetDy(bool isMainScore, bool isPrintView)
{
	if(isMainScore)
		return VU_PER_ROW*GetDyMainScore(isPrintView);
	else
		return VU_PER_ROW*GetDyPart(isPrintView);
} 

VUnit DrawObject::GetDx()
{
	return GetDx(tipoLeggio==LEGGIO_MASAE,currentViewType==1);
}

VUnit DrawObject::GetDy()
{
	return GetDy(tipoLeggio==LEGGIO_MASAE,currentViewType==1);
}