/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: drawobje.hpp,v 1.1 2005/07/25 16:30:29 bellinpo Exp $ */


#ifndef DRAWOBJECT_HPP
#define DRAWOBJECT_HPP

#include "lioo.hpp"
#include "drw/point.hpp"
#include "drw/rectangl.hpp"

#include "drw/colore.hpp"

class Lista;
class Battuta;

#define MAX_VIEW 10

class DrawObject
{ 
private:
	// Colore con cui disegnare l'oggetto
	Colore Color;		//FFF
	//Colore OldColor;	//FFF
	//Bool OldOpacity;	//FFF

protected:
    ClassID ID;
	// Numero che identifica il tipo di oggetto grafico
    //short Color;
	//short OldColor;
	Bool isOpaque;

	Bool isSelected;

	Bool TraceMilla;
	
	Point AbsPos[MAX_VIEW];
	// Posizione assoluta del centro dell'oggetto nello schermo.
    
    static int currentView;

   	float dxMainScore[2],dyMainScore[2],dxPart[2],dyPart[2];

public:  
    DrawObject();
    virtual ~DrawObject() {};
	
    operator class Rectangle();
	// converte il draw object in un rettangolo
	
    ClassID GetID();
	// restituisce l'identificatore della classe.
    void SetID(ClassID id);
	// imposta l'identificatore della classe.
	
    virtual void SetColor(const ColorIcon& col);
	virtual void SetColor(const Colore& col);	//FFF
	virtual void SetColor(unsigned char r, unsigned char g, unsigned char b);	//FFF

	virtual void UnsetColor();

	// imposta il colore.
    //int GetColor();
	virtual const Colore& GetColor();	//FFF
	// ritorna il colore selezionato.
 
	//FFF
	//virtual void SetOldColor(const ColorIcon& col);
 //   virtual void SetOldColor(const Colore& col);	

 //   //virtual int GetOldColor(void);
 //   virtual const Colore& GetOldColor(void);	
	//FFF
	
    virtual VUnit GetVU2Up() = 0 ;
	// restituisce la distanza da AbsPos al punto superiore.
    virtual VUnit GetVU2Dwn() = 0 ;
	// restituisce la distanza da AbsPos al punto inferiore.
    virtual VUnit GetVU2Lft() = 0 ;
	// restituisce la distanza da AbsPos al punto a sinistra.
    virtual VUnit GetVU2Rgt() = 0 ;
	// restituisce la distanza da AbsPos al punto a destra.
	
    VUnit GetWidth();
	// restituisce la larghezza dell'oggetto
    VUnit GetHeight();
	// restituisce l'altezza dell'oggetto.

	float GetDxMainScore(int viewType)  {return dxMainScore[viewType];};
	float GetDyMainScore(int viewType)  {return dyMainScore[viewType];};
	float GetDxPart(int viewType)		{return dxPart[viewType];};
	float GetDyPart(int viewType)		{return dyPart[viewType];};
	VUnit GetDx(bool isMainScore, bool isPrintView);
	VUnit GetDy(bool isMainScore, bool isPrintView);
	VUnit GetDx();
	VUnit GetDy();

	void SetDxMainScore(int viewType, float value)  {dxMainScore[viewType]=value;};
	void SetDyMainScore(int viewType, float value)  {dyMainScore[viewType]=value;};
	void SetDxPart(int viewType, float value)		{dxPart[viewType]=value;};
	void SetDyPart(int viewType, float value)		{dyPart[viewType]=value;};

    virtual void SetPos(DrawObject *d,const Point& p);
	// imposta la posizione assoluta dell'oggetto relativamente
	// al drawobject d. Se d==NULL imposta la posizione assoluta come 'p'.
    virtual void Move(const Point& p);
	// Cambia la posizione dell'oggetto nello schermo      
    Point& GetAbsPos() { return AbsPos[currentView]; };
	// Ritorna la posizione assoluta dell'oggetto grafico
	
	Bool is(ClassID);
    // dice se l'oggetto puntato e' un DrawObject 
	
    virtual void Draw() = 0;
	// disegna l'oggetto in AbsPos

    virtual Bool Hit(const Point& apos);
	// restituisce TRUE se l'oggetto e' stato colpito.
    virtual Bool Hit(const Point&, SelObj, DrawObject*&, SymPath&);
	// altro modo per selezionare un oggetto
	
    virtual Bool Hit(const class Rectangle&);
	// restituisce TRUE se l'oggetto e' contenuto nel rettangolo.
    virtual Bool Hit(const class Rectangle&, SelObj, SymPath&, SymPath&);
	// altro modo per selezionare un oggetto (tramite un rettangolo).
    virtual Bool SymbolicHit(SymPath&, SelObj, DrawObject*&);
	// Decodifica un path.
	
	//void SelectBox();
	//// seleziona l'oggetto impostando il box di selezione e ridisegnando
	//// l'oggetto.
	//void DeselectBox();
	//// deseleziona l'oggetto impostando il box normale e ridisegnando
	//// l'oggetto.

    /*virtual */void Select();
	// seleziona l'oggetto impostando il colore di selezione e ridisegnando
	// l'oggetto.
    /*virtual */void Deselect();
	// deseleziona l'oggetto impostando il colore normale e ridisegnando
	// l'oggetto.
	
    virtual CmdResult Command(CmdID,CmdArgV); 
	// Esegue il comando indicato da CmdID con gli argomenti specificati in CmdArgV
	// ritorna CMD_OK se il comando ha avuto successo, CMD_FAIL se il comando
	// non e' stato eseguito, CMD_UNKNOWN se il comando non lo riguarda.
    virtual CmdResult SymCommand(SymCmd&, Battuta* pBattuta=NULL);
	
	//*BIAN-VERR*
	virtual CmdResult GetSymCode(SymCmd&, Battuta* pBattuta=NULL);
	//*BIAN-VERR*

    virtual Bool DeleteSym(DrawObject*);
	// cancella il simbolo specificato. Ritorna TRUE se e' stato cancellato
	
protected:
    int markType;
    int markVariant;
    int VIPUserData;

	virtual void DrawContour(VUnit width = 1);
	// disegna il bounding box
	
public:
    int GetMarkType() { return markType; };
    void SetMarkType(int v) { markType=v; };
    int GetMarkVariant() { return markVariant; };
    void SetMarkVariant(int v) { markVariant=v; };
    int GetVIPUserData() { return VIPUserData; };
    void SetVIPUserData(int v) { VIPUserData=v; };
    
	
	Bool GetTraceMilla(); 
	void SetTraceMilla(Bool t);
	
    virtual void Edit(Battuta *pBat=NULL);
    
    static void SetCurrentView(int nview) { currentView=nview; };
    static int GetCurrentView() { return currentView; };
};

#endif //DRAWOBJECT_HPP
