/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: gdevice.hpp,v 1.1 2005/07/25 16:30:29 bellinpo Exp $ */


#ifndef GDEVICE_HPP
#define GDEVICE_HPP

#include "drw/colore.hpp"

#include "fon/lioofont.hpp"
#include "drw/point.hpp"
#include "drw/rectangl.hpp"

// Definizione degli ID dei font
#define MUSICFONT_ID    0
#define MUSICFONT2_ID   1
#define MENUFONT_ID		2
#define TEXTFONT_ID     3
#define BIGTEXTFONT_ID  4
#define MUSICFONT3_ID   5

enum DrwMode 
{	
	DefMode = 3,	// Modo Standard 	 
		XORMode = 4, 	// XOR tra il codice colore e quello del punto
		ANDMode = 1, 	// Come sopra ma con AND
		ORMode = 0 		// Come sopra ma con OR
};

class GDevice
{
protected:
    int activeFont;
	// FontID del font attivo.
    Point currentPos;
	// Posizione corrente del "pennello"
    Point activeInitTextPos;
	// Posizione della stringa da stampare
    
	
public:
    GDevice();
	// inizializza.
    ~GDevice();
	// dealloca.
	
    LiooFont *lista_font[6+10];
	// Lista dei font disponibili per questo schermo
	// 5 di base piu' 10 dai gruppi per i font generici esterni
	
    virtual void Clear();
	// Cancella lo schermo
    
    virtual int GetMaxPixX();
	// Restituisce il numero massimo di pixel in ascisse
	
    virtual int GetMaxPixY();
	// Restituisce il numero massimo di pixel in ordinate
	
    virtual int GetNumColors();
	// Restituisce il numero massimo di colori
	
    //virtual void SetColor(int);
	virtual void SetColor(const Colore&)=0;
	// Imposta il colore
	
    //virtual int GetColor();
	virtual Colore GetColor()=0;
	// Restituisce il colore attivo
	
    virtual void SetDrwMode(DrwMode);
	// Imposta la modalita' di disegno
	
    virtual void PutPixel(const Point &p);
	// Disegna un pixel nel punto p e restituisce il colore del punto
	// prima di disegnarlo.
    
    void MoveTo(const Point &p);
    void MoveTo(VUnit x,VUnit y);
	// Imposta la posizione corrente tramite un punto o tramite le coordinate
	
    Point CurrentPos();
	// Ritorna la posizione corrente.
	
    virtual void LineTo(const Point& p);
    virtual void LineTo(VUnit x,VUnit y);
	// Disegna una linea dalla posizione corrente al punto indicato.
	// Usa il tipo di linea impostato (SetLineStyle) e il colore impostato
	// (SetColor).
	
    virtual void Line(const Point& p1, const Point& p2);
	// Disegna una linea tra due punti.
	
    virtual void BigLine(const Point& p1, const Point& p2, VUnit width);
	// Disegna una linea tra due punti dello spessore indicato da width
	
    virtual void Rect(const class Rectangle& r,Bool isfilled=false);
	// Disegna un rettangolo che opzionalmente puo' essere riempito
	
	virtual void BigRect(const class Rectangle& r, VUnit width = 1);
	// Disegna un rettangolo vuoto con spessore

    virtual void ShadowRect(const class Rectangle& r,Bool isOn);
	// Disegna un rettangolo in modo da dare l'effetto dell'ombra
	// isOn=TRUE indica che deve essere rialzato
	// isOn=FALSE indica che deve essere incavato
	
    virtual void Arc(const class Rectangle& r, short Angle1, short Angle2, Bool isfilled=false);
	// Disegna l'arco dell'ellisse inscritta nel rettangolo r, compreso
	// tra gli angoli Angle1 ed Angle2 (indicati in gradi).
	// Opzionalmente puo' essere riempito, 
	
    virtual void Ellipse(const class Rectangle& r,Bool isFilled=false);
	// Disegna l'ellisse limitata dal rettangolo r ed opzionalmente
	// puo' essere riempita.
	
    FontID SetFont(FontID);
	// Seleziona il font indicato dal FontID, serve a ripristinare un font
	
    FontID GetActiveFont();
	// Restituisce il FontID del font attivo attualmente.
	
    int GetFontAscent();
	// Restituisce la distanza della base line del font dalla cima.
	
    int GetFontHeight();
	// Restituisce la altezza del font.
    
    void TextTo(const Point &p);
    void TextTo(VUnit x,VUnit y);
	// Imposta la posizione corrente in modo che la base line del font
	// sia nel punto indicato.
	
    virtual void OutText(unsigned char *);
	// Scrive il testo indicato con il font attualmente selezionato.
	
    int GetTextWidth(unsigned char *);
	// Ritorna la larghezza in pixel del testo specificato.
    int GetTextHeight(unsigned char *txt);
	// Ritorna l'altezza in pixel del testo specificato.
	
    long ImageSize(const class Rectangle&);
	// Ritorna il numero di byte necessari a memorizzare l'immagine contenuta nel
	// rettangolo indicato
	
    virtual void PutImage(const Point&,void *buf,int w,int h,DrwMode);
    // Ripristina a partire dal punto p l'immagine memorizzata del buffer buf.
    // Il ripristino avviene con la modalita' indicata
	
	int Get_VU2UP(int carcode);
	int Get_VU2DWN(int carcode);
	int Get_VU2RG(int carcode);
	int Get_VU2LF(int carcode);
	int Get_VU2DX(int carcode);
	int Get_VU2DY(int carcode);
    // 6 funzioni che ritornano i 6 nuovi parametri della struct 'Carattere'	
};

#endif //GDEVICE_HPP
