/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: gscreen.cpp,v 1.1 2005/07/25 16:30:29 bellinpo Exp $ */

#ifdef USE_GDIPLUS

#include <windows.h>
#ifdef WX226
typedef ULONG *ULONG_PTR;
#else
#undef CreateDialog
#endif

#include <GdiPlus.h>
ULONG_PTR           gdiplusToken=NULL;
static int nGDIPlusInit=0;

void GDIPInitialize()
{
	if(gdiplusToken==NULL)
	{
		Gdiplus::GdiplusStartupInput gdiplusStartupInput;
		// Initialize GDI+.
		Gdiplus::GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, NULL);
	}
	nGDIPlusInit++;
}

void GDIPTerminate()
{
	nGDIPlusInit--;
	if(nGDIPlusInit==0 && gdiplusToken)
		Gdiplus::GdiplusShutdown(gdiplusToken);
}

void GDIPDrawBigLine(HDC hdc,int x1,int y1, int x2, int y2, int w, BYTE red, BYTE green, BYTE blue)
{
	Gdiplus::Graphics g(hdc);
	Gdiplus::Pen  blackPen(Gdiplus::Color(255, red, green, blue),1);
	Gdiplus::SolidBrush blackBrush(Gdiplus::Color(255, red, green, blue));
	g.SetSmoothingMode(Gdiplus::SmoothingModeAntiAlias);
	Gdiplus::GraphicsPath path;
	
	path.AddLine(x1, y1, x2, y2);
	path.AddLine(x1, y1, x1, y1-w);
	path.AddLine(x1, y1-w, x2, y2-w);
	path.AddLine(x2, y2-w, x2, y2);
	path.CloseFigure();
	g.DrawPath(&blackPen, &path);
	g.FillPath(&blackBrush, &path);
}

void GDIPDrawSlur(HDC hdc,int x1,int y1, int xc1, int yc1, int xc2,int yc2, int x2, int y2, int w, bool bowUp, BYTE red, BYTE green, BYTE blue, int style=0)
{
	if(x1==0)
		return;
	Gdiplus::Graphics g(hdc);
	Gdiplus::Pen  blackPen(Gdiplus::Color(255, red, green, blue),1);
	Gdiplus::SolidBrush blackBrush(Gdiplus::Color(255, red, green, blue));
	g.SetSmoothingMode(Gdiplus::SmoothingModeAntiAlias);
	if(style==0)
	{
		Gdiplus::GraphicsPath path;
		
		path.AddBezier(x1, y1, xc1, yc1, xc2, yc2, x2, y2);
		path.AddBezier(x2, y2, xc2, yc2-w, xc1, yc1-w, x1, y1);
		g.DrawPath(&blackPen, &path);
		g.FillPath(&blackBrush, &path);
	}
	else
	{
		float dashVals[2];
		if(style==1) //DASH
		{
			dashVals[0]=float(w)*10;
			dashVals[1]=float(w)*5;
		}
		else //DOT
			dashVals[0]=dashVals[1]=float(w)*2;

		blackPen.SetDashPattern(dashVals, 2);

		g.DrawBezier(&blackPen,x1, y1, xc1, yc1, xc2, yc2, x2, y2);
	}
}

void GDIPDrawBrace(HDC hdc,int xc,int yc, int height, BYTE red, BYTE green, BYTE blue)
{
	Gdiplus::Graphics g(hdc);
	Gdiplus::Pen  blackPen(Gdiplus::Color(255, red, green, blue),1);
	Gdiplus::SolidBrush blackBrush(Gdiplus::Color(255, red, green, blue));
	g.SetSmoothingMode(Gdiplus::SmoothingModeAntiAlias);
	{
		Gdiplus::GraphicsPath path;
		
		int x1=xc;
		int y1=yc-height/2;
		int x2=xc-10;
		int y2=yc;
		int xc1=x1-20;
		int yc1=y1+10;
		int xc2=x2+20;
		int yc2=y2-10;
		
		path.AddBezier(x1, y1, xc1, yc1, xc2, yc2, x2, y2);
		path.AddBezier(x2, y2, xc2, yc2, xc1-2, yc1, x1, y1);
		g.DrawPath(&blackPen, &path);
		g.FillPath(&blackBrush, &path);
	}

	{
		Gdiplus::GraphicsPath path;
		
		int x1=xc-10;
		int y1=yc;
		int x2=xc;
		int y2=yc+height/2;
		int xc1=x1+20;
		int yc1=y1-10;
		int xc2=x2-20;
		int yc2=y2+10;
		
		path.AddBezier(x1, y1, xc1, yc1, xc2, yc2, x2, y2);
		path.AddBezier(x2, y2, xc2-2, yc2, xc1, yc1, x1, y1);
		g.DrawPath(&blackPen, &path);
		g.FillPath(&blackBrush, &path);
	}
}

void GDIPDrawTransparentRect(HDC hdc,int x,int y, int w, int h, BYTE red, BYTE green, BYTE blue, BYTE alpha)
{
	Gdiplus::Graphics g(hdc);
	Gdiplus::Pen  pen(Gdiplus::Color(alpha, red, green, blue),1);
	Gdiplus::SolidBrush brush(Gdiplus::Color(alpha, red, green, blue));
	g.SetSmoothingMode(Gdiplus::SmoothingModeAntiAlias);
	g.FillRectangle(&brush,x,y,w,h);
}

#else
void GDIPInitialize()
{
}

void GDIPTerminate()
{
}
#endif

#include "drw/gscreen.hpp"
#include "gui/lioowin.hpp"
#include "drw/scrlist.hpp"
#include "drw/menu.hpp"
#include "drw/musicedt.hpp"
#include "drw/scorefrm.hpp"
#include "drw/menufrm.hpp"
#include "drw/timer.hpp"
#include "frmanage.hpp"

extern LiooWindow *pMusicWin;
extern ScrList LiooList,LiooListWidget;
extern Widget LiooWidget,WindowActive;

#include <wx/wx.h>


wxMusicEditor *MusicEditor=NULL;
wxDC *currDC=NULL, *currMainDC=NULL;
wxMemoryDC *currMDC=NULL;
wxMemoryDC *currMDC2=NULL;

/////////////////////////////////////////////////////////////////////////////

// Macro per traduzione coordinate.
#define xT(x) (x * GetMagnification())
#define yT(y) ((GetMaxPixY()  - (y)) * GetMagnification())

GScreen::GScreen()
{
	// Registra i font standard di LIOO
	
	lista_font[0] = new LiooFont(MUSICFONT);
	if (!lista_font[0])
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","GScreen","GScreen");
	lista_font[0]->SetID(MUSICFONT_ID);
	
	lista_font[1] = new LiooFont(MUSICFONT2);
	if (!lista_font[1])
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","GScreen","GScreen");
	lista_font[1]->SetID(MUSICFONT2_ID);
	
	lista_font[2] = new LiooFont(MENUFONT);
	if (!lista_font[2])
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","GScreen","GScreen");
	lista_font[2]->SetID(MENUFONT_ID);
	
	lista_font[3] = new LiooFont(TEXTFONT);
	if (!lista_font[3])
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","GScreen","GScreen");  
	lista_font[3]->SetID(TEXTFONT_ID);
	
	lista_font[4] = new LiooFont(BIGTEXTFONT);
	if (!lista_font[4])
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","GScreen","GScreen");  
	lista_font[4]->SetID(BIGTEXTFONT_ID);

	lista_font[5] = new LiooFont(MUSICFONT3);
	if (!lista_font[5])
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","GScreen","GScreen");  
	lista_font[5]->SetID(MUSICFONT3_ID);
	
	// Inizialmente nessun font attivo
	SetFont(-1);

	SetColor((ColorIcon)NORMAL_COLOR);
	
	lastKey = tmpKey = 0;
	Copia = NULL;
	Disegno = NULL;
	Current = NULL;
	currMDC=currMDC2=NULL;

	AutoMagnification = FALSE;
	for(int i=0; i< MAX_VIEW; i++)
		Magnification[i] = 1.0;
	GDIPInitialize();
}


GScreen::~GScreen()
{
    if (pMusicWin!=NULL) pMusicWin->KillLioo();
    
    delete pMusicWin;
    LiooWidget=NULL;
	
	// Le ScrList non devono avere il distruttore altrimenti la deallocazione
	// di LiooList provoca Segmentation Fault (LiooList e' una lista di
	// 3 oggetti statici e uno dinamico, e chiamare la delete su oggetti 
	// statici e' dannoso.
    LiooListWidget.DeleteAll();
	GDIPTerminate();
}


Widget GScreen::GOpenScoreWindow(const char *title, int reader)
{
	//Insieme di variabili globali per l'inizializzazione
	//dell'Editor musicale
	//occorre suddividire quelle che sono globali in senso stretto da
	//quelle che sono dipendenti dalla modalita' di esecuzione dell'editor
		  
	if (MusicEditor==NULL)
		MusicEditor= new wxMusicEditor;
	if (reader==0)	
	{
		tipoLeggio=LEGGIO_MASAE;
		//if (LiooWidget!=NULL) return;
		LiooWidget=MusicEditor->MakeMainDrawArea(WinMasaeWidth-6, WinMasaeHeight-6,"WEDEL Music Editor");
	}
	else
	{ 
		tipoLeggio=LEGGIO_DLIOO;
		//if (LiooWidget!=NULL) return;
		LiooWidget=MusicEditor->MakeMainDrawArea(WinDliooWidth-6, WinDliooHeight-6,"WEDEL Music Editor");
	}
	if(title!=NULL)
		MusicEditor->mainFrame->SetTitle(title); 
	
	
	activeClipRegion = class Rectangle(Point(0, GetMaxPixY()), Point(GetMaxPixX(), 0)); 
	
	
	//*scrolling
#ifndef   SCR_FINESTRE
    // In modalita' pixmap: crea le pixmap Copia e Disegno 
	
	if(Copia==NULL)
	{
		InitPixmaps();
	}
#endif
	//*
	
	
#ifdef WITH_NET
    AddTimeOut(PVMTimeInterval, Call_TimeOut, NULL);
#endif
	
	return LiooWidget;
}

void GScreen::InitPixmaps()
{
	if(Copia==NULL)
	{
		Copia = new LPixmap(WIN_PIXMAP_WIDTH, WIN_PIXMAP_HEIGHT);
		if (!Copia)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","GScreen","GOpen");
		
		Disegno = new LPixmap(WIN_PIXMAP_WIDTH, WIN_PIXMAP_HEIGHT);
		if (!Disegno)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","GScreen","GOpen");   
		
		Current = new LPixmap(WIN_PIXMAP_WIDTH, WIN_PIXMAP_HEIGHT);
		if (!Current)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","GScreen","GOpen");  
	}
}

/**************************************************************************/

Widget GScreen::GNewDrawArea(int x, int y, VUnit height, VUnit width, Widget widget, char* title)
{ 
    // se qualcosa va male (sia per la creazione della DrawArea o per il
    // parametro window passatogli) ritorna NULL
	
    // il controllo sul risultato di MakeDrawArea e' demandato all'utilizzatore
    // di GNewDrawArea
#ifdef __WXMSW__
    return MusicEditor->MakeMenuDrawArea((Menu*)widget, x, y, width+2,height,title);
#elif defined __WXGTK__	
	return MusicEditor->MakeMenuDrawArea((Menu*)widget, x, y, width+3,height+15,title);
#endif	
}


void GScreen::GCloseWindow(Widget w)
{
	if (w==NULL) 
	{
		EM.Error(GENERAL_ERR_AREA,"EC_WINDOW_NOT_PRESENT","GScreen","GCloseWindow()"); 
		GSetMainDrawArea();
		return ;
	}
	
	MusicEditor->CloseMainDrawArea(w);
	
	wxFrame *tmp=(wxFrame*)w;
	tmp->Destroy();
	
	if(FrameManager::currFrame==tmp)
		FrameManager::currFrame=NULL;
	
	if (w==LiooWidget) 
	{
		wxMEScoreFrame *tmp=(wxMEScoreFrame*) w;
		LiooWindow *tmp2=tmp->GetMusicWin();
		tmp->SetMusicWin(NULL);
		LiooList.Delete(tmp2);
		LiooWidget=NULL;
		if(pMusicWin==tmp2)
			pMusicWin=NULL;
		MusicEditor->mainFrame=NULL;
	}
}

void GScreen::GSetFocusWindow(Widget w)
{
	if (w==NULL) 
	{
		EM.Error(GENERAL_ERR_AREA,"EC_WINDOW_NOT_PRESENT","GScreen","GCloseWindow()"); 
		GSetMainDrawArea();
		return ;
	}
	
	((wxFrame*)w)->SetFocus();
}

void GScreen::GSetMainDrawArea()
{
	if (LiooWidget==NULL)
	{
		return ;
	}
	
	GSetDrawArea(LiooWidget);
	WindowActive = NULL;
	
	if(currMainDC!=NULL)
		currDC=currMainDC;
}


void GScreen::GSetDrawArea(Widget w)
{
	FrameManager::currFrame=(wxFrame*)w;
	if(w!=LiooWidget)
		WindowActive=w;
}


void GScreen::GClose() {;}


void GScreen::Clear()
{
	if(currDC==NULL) 
		return;
	currDC->Clear();
}


int GScreen::GetMaxPixX()
{
	int x, y;
	if(FrameManager::currFrame==NULL)
	{
		return 0;
	}
	FrameManager::currFrame->FindWindow(wxMEScoreCanvas_ID)->GetSize(&x,&y);
	
	return x-6;
}


int GScreen::GetMaxPixY()
{
	int x, y;
	if(FrameManager::currFrame==NULL)
	{
		return 0;
	}
	wxScrolledWindow* canvas = (wxScrolledWindow*)FrameManager::currFrame->FindWindow(wxMEScoreCanvas_ID);
	canvas->GetVirtualSize(&x,&y);
	if (y==0)
		FrameManager::currFrame->GetClientSize(&x,&y);
#ifdef WX226
	return y-6;
#else
	return y;
#endif
}


int GScreen::GetNumColors()
{ return 256; }


void GScreen::SetColor(const Colore& col)
{
	if(currDC==NULL) 
	return;

	currDC->SetPen(wxPen(col, 1, wxSOLID));
	currDC->SetBrush(wxBrush(col, wxSOLID));
	currDC->SetTextForeground(col);
}

Colore GScreen::GetColor()
{ 
//	assert(currDC);
	if(currDC==NULL)
		return Colore(0,0,0);
	return currDC->GetPen().GetColour();
}
/*
void GScreen::SetColor(int col)
{ 
	if(currDC==NULL) 
		return;
	
	switch(col)
	{ 
	case NERO:
		currDC->SetPen(*wxBLACK_PEN);
		currDC->SetBrush(*wxBLACK_BRUSH);
		break;
	case BIANCO:
		currDC->SetPen(*wxWHITE_PEN);
		currDC->SetBrush(*wxWHITE_BRUSH);
        break;
	case GRIGIO:
		currDC->SetPen(*wxLIGHT_GREY_PEN);
		currDC->SetBrush(*wxLIGHT_GREY_BRUSH);
        break;
	case GRIGIOS:
		currDC->SetPen(*wxGREY_PEN);
		currDC->SetBrush(*wxGREY_BRUSH);
        break;
	case ROSSO:
		currDC->SetPen(*wxRED_PEN);
		currDC->SetBrush(*wxRED_BRUSH);
        break;
	case ARANCIO:
		currDC->SetPen(wxPen(wxColour(255,128,0),1,wxSOLID));
		currDC->SetBrush(wxBrush(wxColour(255,128,0),wxSOLID));
        break;
	case MARRONE:
		currDC->SetPen(wxPen(wxColour(128,64,0),1,wxSOLID));
		currDC->SetBrush(wxBrush(wxColour(128,64,0),wxSOLID));
        break;
	case MAGENTA:
		currDC->SetPen(wxPen(wxColour(255,0,255),1,wxSOLID));
		currDC->SetBrush(wxBrush(wxColour(255,0,255),wxSOLID));
        break;
	case VIOLA:
		currDC->SetPen(wxPen(wxColour(128,0,255),1,wxSOLID));
		currDC->SetBrush(wxBrush(wxColour(128,0,255),wxSOLID));
        break;
	case VERDEC:
		currDC->SetPen(*wxGREEN_PEN);
		currDC->SetBrush(*wxGREEN_BRUSH);
        break;
	case VERDE:
		currDC->SetPen(wxPen(wxColour(0,206,103),1,wxSOLID));
		currDC->SetBrush(wxBrush(wxColour(0,206,103),wxSOLID));
        break;
	case VERDEA:
		currDC->SetPen(wxPen(wxColour(0,187,187),1,wxSOLID));
		currDC->SetBrush(wxBrush(wxColour(0,187,187),wxSOLID));
        break;
	case BLU:
		currDC->SetPen(wxPen(wxColour(0,0,255),1,wxSOLID));
		currDC->SetBrush(wxBrush(wxColour(0,0,255),wxSOLID));
        break;
	case GIALLO:
		currDC->SetPen(wxPen(wxColour(255,255,0),1,wxSOLID));
		currDC->SetBrush(wxBrush(wxColour(255,255,0),wxSOLID));
        break;
	case GIALLOC:
		currDC->SetPen(wxPen(wxColour(255,255,174),1,wxSOLID));
		currDC->SetBrush(wxBrush(wxColour(255,255,174),wxSOLID));
        break;
	case CELESTE:
		currDC->SetPen(*wxCYAN_PEN);
		currDC->SetBrush(*wxCYAN_BRUSH);
        break;
		
	default:
		if(col>=LIV_GRIGIO)
		{
			int lg=col-LIV_GRIGIO;
			currDC->SetPen(wxPen(wxColour(lg,lg,lg),1,wxSOLID));
			currDC->SetBrush(wxBrush(wxColour(lg,lg,lg),wxSOLID));
		}
		else
		{
			currDC->SetPen(*wxBLACK_PEN);
			currDC->SetBrush(*wxBLACK_BRUSH);
		}
		break;
	}
}


int GScreen::GetColor()
{ 
	return 0;
}
*/

void GScreen::SetDrwMode(DrwMode d)
{ 
	if(currDC==NULL) return;
	
	switch (d)
	{
    case DefMode:
		{
			currDC->SetLogicalFunction(wxCOPY);
			break;
		}
    case XORMode:
		{
			currDC->SetLogicalFunction(wxXOR);
			break;
		}
    case ANDMode:
		{
			currDC->SetLogicalFunction(wxAND);
			break;
		}
    case ORMode:
		{
			currDC->SetLogicalFunction(wxOR);
			break;
		}
    default:
		{
			EM.Error(GENERAL_ERR_AREA,"EC_DRWMODE_NOT_VALID","GScreen","SetDrwMode()"); 
			currDC->SetLogicalFunction(wxCOPY);
			break;
		}
	}
}


class Rectangle GScreen::SetClipRegion(const class Rectangle &r)
{
	class Rectangle tmp = activeClipRegion;
	activeClipRegion = r;
	
	// Ritorna l'area impostata precedentemente
	return tmp;
}

void GScreen::PutPixel(const Point &p)
{ 
	if(currDC==NULL) return;
	
    currDC->DrawPoint(xT(p.x),yT(p.y));
}

void GScreen::PutPixel0Up(const Point &p)
{ 
	if(currDC==NULL) return;
	
    currDC->DrawPoint(p.x,p.y);
	
}


int GScreen::GetPixel(const Point& p)
{ 
	
	return 0;
}


void GScreen::MoveTo(const Point &p)
{ 
	// Si limita a settare la variabile di riferimento.
	currentPos.x = p.x;
	currentPos.y = p.y; 
}


void GScreen::MoveTo(VUnit x, VUnit y)
{
	// Si limita a settare la variabile di riferimento.
	currentPos.x = x;
	currentPos.y = y;
}


Point GScreen::CurrentPos()
{ return currentPos; }


void GScreen::LineTo(const Point &p, LineType tp)
{ 
	LineTo(p.x,p.y,tp);
}


void GScreen::LineTo(VUnit x, VUnit y, LineType tp)
{
	if(currDC==NULL) return;
	
	wxPen p=currDC->GetPen(), oldp=p;
	static wxDash dash[]={10,5};
	
	
	switch(tp)
	{
    case SOLID_LINE:
		p.SetStyle(wxSOLID);
		break;
    case DASHED_LINE:
		p.SetDashes(2,dash);
		p.SetStyle(wxSHORT_DASH);
		break;
    case DOTTED_LINE:
		p.SetStyle(wxDOT);
		break;
	}
	p.SetWidth(p.GetWidth()*GetMagnification());
	p.SetCap(wxCAP_BUTT);
	currDC->SetPen(p);    
	currDC->DrawLine(xT(currentPos.x), yT(currentPos.y), xT(x), yT(y));
	currDC->SetPen(oldp);    
	currentPos.x = x;
	currentPos.y = y; 
}


void GScreen::Line(const Point &p1, const Point &p2,LineType tp)
{
	MoveTo(p1); 
	LineTo(p2,tp);
}


void GScreen::BigLine(const Point &p1, const Point &p2, VUnit width)
{
#ifndef USE_GDIPLUS
	wxPoint Punti[4];
	VUnit w = (width>=0 ? width: width);
	
	if(currDC==NULL) return;
	
	
	Punti[0].x = Punti[3].x = (short)xT(p1.x);
	Punti[0].y = (short)yT(p1.y);
	Punti[1].x = Punti[2].x = (short)xT(p2.x);
	Punti[1].y = (short)yT(p2.y);
	Punti[2].y = (short)yT(p2.y + w);
	Punti[3].y = (short)yT(p1.y + w);
	
	currDC->DrawPolygon(4,Punti);	
#else
	if(currDC)
	{
		wxColour col = currDC->GetPen().GetColour();
		GDIPDrawBigLine((HDC)currDC->GetHDC(),xT(p1.x),yT(p1.y),xT(p2.x),yT(p2.y),width*GetMagnification(), col.Red(), col.Green(), col.Blue());
	}
#endif
}

void GScreen::TransparentRect(const class Rectangle &r, int transparency)
{
#ifdef USE_GDIPLUS
	if(currDC)
	{
		wxColour col = currDC->GetPen().GetColour();
		GDIPDrawTransparentRect((HDC)currDC->GetHDC(),xT(r.PUpLft.x),yT(r.PUpLft.y),r.Width(),r.Height(),
			col.Red(), col.Green(), col.Blue(), 255-transparency);
	}
#endif
}

void GScreen::Slur(const Point &p1, const Point &p2, bool bowUp, LineType lt)
{
#ifdef USE_GDIPLUS
//	GDIPDrawSlur((HDC)currDC->GetHDC(),xT(p1.x),yT(p1.y),xT(p2.x),yT(p2.y),2*GetMagnification(), bowUp, lt);
#endif
}

void GScreen::Slur(const Point &p1, const Point &cp1, const Point &cp2, const Point &p2,bool bowUp, LineType lt)
{
#ifdef USE_GDIPLUS
	if(currDC==NULL)
		return;
	wxColour col = currDC->GetPen().GetColour();
	GDIPDrawSlur((HDC)currDC->GetHDC(),xT(p1.x),yT(p1.y),xT(cp1.x),yT(cp1.y),xT(cp2.x),yT(cp2.y),xT(p2.x),yT(p2.y),2*GetMagnification(), bowUp, col.Red(), col.Green(), col.Blue(), lt);
#endif
}

void GScreen::Brace(const Point& center, VUnit height)
{
#ifdef USE_GDIPLUS
	if(currDC==NULL)
		return;
	wxColour col = currDC->GetPen().GetColour();
	GDIPDrawBrace((HDC)currDC->GetHDC(),xT(center.x),yT(center.y), height*GetMagnification(), col.Red(), col.Green(), col.Blue());
#endif
}

void GScreen::Rect(const class Rectangle &r, Bool isfilled)
{
	wxBrush brush;
	
	if(currDC==NULL) return;
	
	if(!isfilled)
	{
		brush=currDC->GetBrush();
		currDC->SetBrush(*wxTRANSPARENT_BRUSH);
	}
	int x=xT(r.PUpLft.x);
	int y=yT(r.PUpLft.y);
	int w=(r.PDwnRgt.x - r.PUpLft.x) * GetMagnification();
	int h=(r.PUpLft.y - r.PDwnRgt.y) * GetMagnification();
	currDC->DrawRectangle(x,y,w,h);
	if (!isfilled)
		currDC->SetBrush(brush);
}

void GScreen::BigRect(const class Rectangle& r, VUnit width){
	wxPen pen, oldpen; 

	if(currDC==NULL || width < 1) return;

	pen = oldpen =	currDC->GetPen();
	pen.SetWidth(width);
	currDC->SetPen(pen);

	Rect(r);

	currDC->SetPen(oldpen);
}


void GScreen::ShadowRect(const class Rectangle& r, Bool isOn)
{
	SetColor((ColorIcon)(isOn ? white : dgray));
	MoveTo(r.PUpLft.x, r.PDwnRgt.y);
	LineTo(r.PUpLft,SOLID_LINE);
	LineTo(r.PDwnRgt.x, r.PUpLft.y,SOLID_LINE);
	
	SetColor((ColorIcon)(isOn ? dgray : white));
	LineTo(r.PDwnRgt,SOLID_LINE);
	LineTo(r.PUpLft.x, r.PDwnRgt.y,SOLID_LINE);
}


void GScreen::Arc(const class Rectangle& r, short Angle1, short Angle2, Bool isfilled)
{
}


void GScreen::Ellipse(const class Rectangle& r,Bool isfilled)
{
	wxBrush brush;
	
	if(currDC==NULL) return;
	
	if(!isfilled)
	{
		brush=currDC->GetBrush();
		currDC->SetBrush(*wxTRANSPARENT_BRUSH);
	}
	currDC->DrawEllipse(xT(r.PUpLft.x), yT(r.PUpLft.y),
		(r.PDwnRgt.x - r.PUpLft.x) * GetMagnification(), (r.PUpLft.y - r.PDwnRgt.y) * GetMagnification());
	if (!isfilled)
		currDC->SetBrush(brush);
}


FontID GScreen::SetFont(FontID nfnt)
{ 
	int ofont;
	
	// Se nessuna font e' attiva...
	// nfnt = -1 cosa vuol dire in termini di errori ?
	if(nfnt == -1)
	{
		return -1;
	}
	
	// Se la font scelta e' gia' attiva
	if(activeFont == nfnt)
		return nfnt;
	ofont = activeFont; 
	activeFont = nfnt;
	
	// Ritorna il font attivo precedentemente
	return ofont;
}


FontID GScreen::GetActiveFont()
{
	return activeFont;
}


int GScreen::GetFontAscent()
{ 
	return lista_font[activeFont]->get_ascend(32); 
}


int GScreen::GetFontHeight()
{
	return lista_font[activeFont]->get_height(32);
}


void GScreen::TextTo(const Point& p)
{ 
	activeInitTextPos.x = p.x;
	activeInitTextPos.y = p.y; 
}


void GScreen::TextTo(VUnit x,VUnit y)
{  
	activeInitTextPos.x = x;
	activeInitTextPos.y = y; 
}


void GScreen::OutText(unsigned char *txt)
{
	lista_font[activeFont]->print_str(xT(activeInitTextPos.x), yT(activeInitTextPos.y), txt);
}                  


int GScreen::GetTextWidth(unsigned char *txt)
{ 
	return lista_font[activeFont]->GetTextWidth(txt);
}

int GScreen::GetTextHeight(unsigned char *txt)
{ 
	return lista_font[activeFont]->get_height(txt[0]);   
}


int GScreen::WaitKey()
{   
	// NON ASPETTA AFFATTO!
	if(tmpKey!=0)
	{
		lastKey = tmpKey;
		tmpKey = 0;
		KeyPressed = FALSE;
	}
	return lastKey;
}


void GScreen::SetTmpKey(int key)
{  
	tmpKey = key;
	KeyPressed = TRUE;
}


int GScreen::KeyHit()
{ 
	Bool b=KeyPressed;
	
	KeyPressed = FALSE;
	return b;
}


int GScreen::LastKey()
{
	return lastKey;
}


long GScreen::ImageSize(const class Rectangle &r)
{
	// Poiche' X impiega 1 byte per pixel e` sufficiente calcolare l'area
	// del rettangolo.
	return ((r.PDwnRgt.x - r.PUpLft.x) * (r.PUpLft.y - r.PDwnRgt.y));
}


void GScreen::GetImage(const class Rectangle &r, void *buffer)
{
}


void GScreen::PutImage(const Point &p, Pixmap img)
{
	if(currDC==NULL) return;
	wxBitmap *bmp=(wxBitmap*)img;
	
	
	int height=bmp->GetHeight();
	if(bmp->Ok())
		currDC->DrawBitmap(*bmp,xT(p.x),yT(height+p.y));
	else
	{
		SetColor(NERO);
		wxBrush brush=currDC->GetBrush();
		currDC->SetBrush(*wxTRANSPARENT_BRUSH);
		currDC->DrawRectangle(xT(p.x), yT(height+p.y),bmp->GetWidth(), height );
		currDC->DrawLine(xT(p.x), yT(height+p.y),xT(p.x+bmp->GetWidth()), yT(p.y) );
		currDC->DrawLine(xT(p.x), yT(p.y),xT(p.x+bmp->GetWidth()), yT(p.y+height) );
		currDC->SetBrush(brush);
	}
	
}


void GScreen::SetMouse(Mouse *m)
{
	mouse = m;
}



Mouse *GScreen::GetMouse()
{
	return mouse;
}


int GScreen::MouseInit()
{
	// L'inizializzazione non fallisce MAI!
	
	return 0;
}


void GScreen::MouseOn()
{
}


void GScreen::MouseOff()
{
}


void GScreen::SetMousePoint(const Point& p)
{
	MousePoint = p;
}


void GScreen::SetMouseAction(int act)
{
	MouseAction = act;
	ActionDetected = FALSE;
}


int GScreen::MousePos(Point &p)
{ 
	int status=NOPUSH;
	
	// Legge la posizione del mouse
	p.x = MousePoint.x;
	p.y = (GetMaxPixY()) - MousePoint.y;
	
	if (!ActionDetected)
	{
		ActionDetected = TRUE;
		return MouseAction;
	}
	else
	{
		switch (MouseAction)
		{
		case LEFTGODOWN:
			{
				status = LEFTDOWN;
				break;
			}
		case RIGHTGODOWN:
			{
				status = RIGHTDOWN;
				break;
			}
		case CENTRALGODOWN:
			{
				status = CENTRALDOWN;
				break;
			}
		}
	}
	return status;
}

#ifdef SCR_FINESTRE

void GScreen::CopyPixmap(Widget s,int x,int y,int width,int height)
{	
    // Ottiene l'id della finestra sorgente
	
	GSetDrawArea(s);
	Drawable DrwbleSorg = GetCurrentDrawable();
	GSetMainDrawArea();
	
	Display * CurrentDisp = GetCurrentDisplay();
	if (CurrentDisp==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_WIDGET_FAULT","GScreen","CopyPixmap()");
		return;
	}
	Drawable DrwbleDest = GetCurrentDrawable();
	GC CurrentGC = GetCurrentGC();
	if (CurrentGC==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_WIDGET_FAULT","GScreen","CopyPixmap()");
		return;
	}
	
	XCopyArea( CurrentDisp,
		DrwbleSorg,DrwbleDest,
		CurrentGC,
		xT(x),yT(y),width,height,xT(x),yT(y));
	
	//  Realizza la copia 
	//  dalla drawing area della finestra sorgente alla
	//  drawing area della finestra principale
	//  attraverso il Grafic Context della finestra principale,
	//  a partire dalle cordinate (xT(x),yT(y)) per
	//  un'ampiezza pari a Width e un'altezza pari a height. 
}

#else

LPixmap *GScreen::GetLPixmapCopia()
{	
	return Copia; 
}

LPixmap *GScreen::GetLPixmapDisegno()
{	
	return Disegno; 
}

LPixmap *GScreen::GetLPixmapCurrent()
{	
	return Current; 
}

void GScreen::CopyPixmap (LPixmap *lpixmap,int x,int y,int width,int height)
{	
	if(currDC==NULL) return;
	
	if(lpixmap==NULL)
		currDC->Blit(xT(x),yT(y),width*GetMagnification(),height*GetMagnification(),currMDC,xT(x),yT(y));
	else if(lpixmap==Current && currMDC2!=NULL)
		currDC->Blit(xT(x),yT(y),width*GetMagnification(),height*GetMagnification(),currMDC2,xT(x),yT(y));
	else
	{
		wxMemoryDC tmpMDC;
		
		tmpMDC.SelectObject(*((wxBitmap*)lpixmap->GetPixmap()));
		currDC->Blit(xT(x),yT(y),width*GetMagnification(),height*GetMagnification(),&tmpMDC,xT(x),yT(y));
	}
}


void GScreen::SwapPixmap()
{
	// questo e' prfettamente inutile dato che per scrollare si usa direttamente
	// currMDC
	LPixmap *temp=Copia;
	Copia=Disegno;
	Disegno=temp;
	
	// copia la pagina scrollata in Current 
	if(currMDC2==NULL)
	{
		currMDC2=new wxMemoryDC;
		currMDC2->SelectObject(*((wxBitmap*)Current->GetPixmap()));
	}
	if(currMDC!=NULL)
		currMDC2->Blit(0,0,WIN_PIXMAP_WIDTH, WIN_PIXMAP_HEIGHT,currMDC,0,0);
}


void GScreen::GResetPixmap()
{
	GSetMainDrawArea();  
}

void GScreen::GSetPixmap (LPixmap *lpixmap)
{
	if(lpixmap==NULL)
		return;
	
	if(currMDC!=NULL)
	{
		delete currMDC;
	}
	currMDC=new wxMemoryDC;
	currDC=currMDC;
	
	currMDC->SelectObject(*((wxBitmap*)lpixmap->GetPixmap()));
}


#endif


CursorTypes GScreen::GGetCursor()
{
	return CurType;
}


int GScreen::Get_VU2UP(int carcode, int fontID)
{
	if(this==NULL) return 0;
	if (carcode<0) carcode=carcode+256;
	return lista_font[fontID]->get_VU2UP(carcode);
}

int GScreen::Get_VU2DWN(int carcode, int fontID)
{
	if(this==NULL) return 0;
	if (carcode<0) carcode=carcode+256;
	return lista_font[fontID]->get_VU2DWN(carcode);
}

int GScreen::Get_VU2RG(int carcode, int fontID)
{
	if(this==NULL) return 0;
	if (carcode<0) carcode=carcode+256;
	return lista_font[fontID]->get_VU2RG(carcode);
}

int GScreen::Get_VU2LF(int carcode, int fontID)
{
	if(this==NULL) return 0;
	if (carcode<0) carcode=carcode+256;
	return lista_font[fontID]->get_VU2LF(carcode);
}

int GScreen::Get_VU2DX(int carcode, int fontID)
{
	if(this==NULL) return 0;
	if (carcode<0) carcode=carcode+256;
	return lista_font[fontID]->get_DX(carcode);
}

int GScreen::Get_VU2DY(int carcode, int fontID)
{
	if(this==NULL) return 0;
	if (carcode<0) carcode=carcode+256;
	return lista_font[fontID]->get_DY(carcode);
}

void GScreen::GSetCursor(CursorTypes ct, int foregc, int backgc)
{
	wxScrolledWindow* canvas = (wxScrolledWindow*)MusicEditor->mainFrame->FindWindow(wxMEScoreCanvas_ID);
	CurType = ct;
	switch (ct)
	{
	case AVANTI: 
	case DEFAULT:
		canvas->SetCursor(*wxCROSS_CURSOR);
		break;
	case WAIT: 
		canvas->SetCursor(*wxHOURGLASS_CURSOR);
		break;
	case EXECUTION: 
		canvas->SetCursor(*wxCROSS_CURSOR);
		break;
	default : 
		canvas->SetCursor(*wxCROSS_CURSOR);
	}
}

void GScreen::GNewDC(Widget w)
{
	wxScrolledWindow* canvas = (wxScrolledWindow*)((wxFrame*)w)->FindWindow(wxMEScoreCanvas_ID);
	currDC= new wxClientDC(canvas);
}

void GScreen::GDeleteDC()
{
	delete currDC;
	currDC=NULL;
}

void GScreen::SetMainDC(wxDC* dc)
{
	currDC=currMainDC=dc;
}

void GScreen::UpdateFont(FontID FONT_ID)
{
	for (int LID=0;LID<2;LID++)
	{
		if (lista_font[LID]->GetID()==FONT_ID)
			if (!lista_font[LID])
				EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","GScreen","GScreen");
			if (LID==0)
				lista_font[LID]->load_table(MUSICFONT);	
			if (LID==1)
				lista_font[LID]->load_table(MUSICFONT2);
	}
	if (FONT_ID>4)
		lista_font[FONT_ID]->load_table(lista_font[FONT_ID]->GetNomeFont());
}

// prende il font da adentro l'oggetto
void GScreen::ReloadBasicFonts()
{
	
	lista_font[0]->load(MUSICFONT);
	
	lista_font[1]->load(MUSICFONT2);
	
	lista_font[3]->load(TEXTFONT);
	
	lista_font[4]->load(BIGTEXTFONT);

	lista_font[5]->load(MUSICFONT3);

}

LiooFont* GScreen::GetFont(const char* fontname)
{
	wxString name=fontname;
	if(name=="musica")
		return lista_font[MUSICFONT_ID];
	else if(name=="musica2")
		return lista_font[MUSICFONT2_ID];
	else if(name=="musica3")
		return lista_font[MUSICFONT3_ID];
	return NULL;
}

void GScreen::ReplaceFont(const char* fontname, LiooFont* font)
{
	wxString name=fontname;
	FontID fid=-1;
	if(name=="musica")
		fid=MUSICFONT_ID;
	else if(name=="musica2")
		fid=MUSICFONT2_ID;
	else if(name=="musica3")
		fid=MUSICFONT3_ID;
	if(fid>=0)
	{
		delete lista_font[fid];
		lista_font[fid] = font;
	}

}


void GScreen::SetSystemFont(wxFont* font)	//DB
{
	if(currDC!=NULL)
	{
		activeSystemFont=*font;
		currDC->SetFont(*font);
	}
}

wxFont* GScreen::GetSystemFont()
{
	return &activeSystemFont;
}

void GScreen::OutSystemText(char* text)
{
	if(currDC!=NULL)
		currDC->DrawText(text, xT(activeInitTextPos.x) , yT(activeInitTextPos.y));
}

VUnit GScreen::GetSystemTextWidth(char* text)
{
	wxCoord w=0,h;
	if(currDC!=NULL)
		currDC->GetTextExtent(text, &w, &h, NULL, NULL, NULL);
	return w;
}

VUnit GScreen::GetSystemTextHeight(char* text)
{
	wxCoord w,h=0;
	if(currDC!=NULL)
		currDC->GetTextExtent(text, &w, &h, NULL, NULL, NULL);
	return h;
}

VUnit GScreen::GetSystemFontAscent()
{
	wxCoord w,h=0,descent=0;
	if(currDC!=NULL)
		currDC->GetTextExtent("g", &w, &h, &descent, NULL, NULL);
	return h-descent;
}

void GScreen::OutSystemText(unsigned short* text)
{
	if(currDC!=NULL)
	{
		HDC dc=(HDC)currDC->GetHDC();
		int x=xT(activeInitTextPos.x);
		int y=yT(activeInitTextPos.y);
		int len=0;
		for(int i=0; text[i]!=0; i++, len++)
			;

        SetTextColor(dc, currDC->GetTextForeground().GetPixel());
		::TextOutW(dc,x,y,text,len);
	}
}

VUnit GScreen::GetSystemTextWidth(unsigned short* text)
{
	wxCoord w=0;
	if(currDC!=NULL)
	{
	    SIZE sizeRect;
		HDC dc=(HDC)currDC->GetHDC();
		int x=xT(activeInitTextPos.x);
		int y=yT(activeInitTextPos.y);
		int len=0;
		for(int i=0; text[i]!=0; i++, len++)
			;

		GetTextExtentPointW(dc, text, len, &sizeRect);
		w=sizeRect.cx;
	}
	return w;
}

VUnit GScreen::GetSystemTextHeight(unsigned short* text)
{
	wxCoord h=0;
	if(currDC!=NULL)
	{
	    SIZE sizeRect;
		HDC dc=(HDC)currDC->GetHDC();
		int x=xT(activeInitTextPos.x);
		int y=yT(activeInitTextPos.y);
		int len=0;
		for(int i=0; text[i]!=0; i++, len++)
			;

		GetTextExtentPointW(dc, text, len, &sizeRect);
		h=sizeRect.cy;
	}
	return h;
}

float GScreen::GetMagnification()
{
	return Magnification[DrawObject::GetCurrentView()];
}

void GScreen::SetMagnification(float magnification)
{
	Magnification[DrawObject::GetCurrentView()]=magnification;
}

Bool GScreen::IsAutoMagnification()
{
	return AutoMagnification;
}

void GScreen::EnableAutoMagnification(Bool status)
{
	AutoMagnification = status;
}

