/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: gscreen.hpp,v 1.1 2005/07/25 16:30:29 bellinpo Exp $ */

#ifndef GSCREEN_HPP
#define GSCREEN_HPP

#include "lioo.hpp"

#include "fon/lioofont.hpp"
#include "drw/point.hpp"
#include "drw/rectangl.hpp"
#include "drw/mouse.hpp"
#include "drw/lpixmap.hpp"
#include "drw/gdevice.hpp"
#include <wx/font.h>

// Definizioni per lo stato dei bottoni del Mouse.

#define NOPUSH         0

#define LEFTDOWN       4
#define CENTRALDOWN    2
#define RIGHTDOWN      1

#define LEFTGODOWN     32
#define CENTRALGODOWN  16
#define RIGHTGODOWN    8

#define LEFTGOUP       256
#define CENTRALGOUP    128
#define RIGHTGOUP      64

#define MAX_VIEW       10

class Mouse;
class GDevice;
class SMRFile;
class wxImage;
class wxFont;
class wxDC;

typedef void* Widget;

class GScreen : public GDevice
{
private:
    Mouse *mouse;
	// gestore del mouse 
    int activeFont;
	// FontID del font attivo.
    class Rectangle activeClipRegion;
	// regione di clipping attiva
    Bool KeyPressed;
	// indicatore della pressione di un tasto
    int tmpKey;
	// codice di tasto temporaneo, comunicato da Call_KPress()
    int lastKey;
	// codice dell'ultimo tasto ottenuto con WaitKey()
    Point currentPos;
	// Posizione corrente del "pennello"
    Point activeInitTextPos;
	// Posizione della stringa da stampare
    Point MousePoint;
	// Posizione del mouse, comunicata da Call_MMoved()
    int MouseAction;
	// Ultima azione rilevata dal mouse
    Bool ActionDetected;
	// E` TRUE se l'ultima azione del mouse e` stata prelevata con MousePos()
	
    CursorTypes CurType;
	
	
    // Queste variabili riguardano la gestione delle pixmap
#ifndef SCR_FINESTRE
	LPixmap *Copia;
	LPixmap *Disegno;
	LPixmap *Current;
	// Definisce due puntatori a pixmap da usare in modalita' Pixmap
#endif    
	
	wxFont activeSystemFont;	  
	
	float Magnification[MAX_VIEW];  
	Bool AutoMagnification;
	
public:
	GScreen();
		// inizializza.
	~GScreen();
		// dealloca.
	Widget GOpenScoreWindow(const char *title, int reader);
	
	void GClose();
	// Chiude lo schermo grafico 
	
	Widget GNewDrawArea(int x, int y, VUnit height, VUnit width, Widget widget,char *title=NULL);
	// Apre una nuova DrawArea e ne ritorna il relativo Widget
	// Se qualcosa va male ritorna NULL
	
	void GSetMainDrawArea();
	// Imposta come finestra attiva quella principale.
	
	void GSetDrawArea(Widget);
	// Imposta come finestra attiva quella specificata dall'argomento.
	
	void GSetFocusWindow(Widget);
	// attiva la finestra indicata
	
	void GCloseWindow(Widget);
	// Chiude la finestra indicata dall'argomento e 
	// riattiva la finestra principale.
		
	void Clear();
	// Cancella lo schermo
	
	int GetMaxPixX();
	// Restituisce il numero massimo di pixel in ascisse
	
	int GetMaxPixY();
	// Restituisce il numero massimo di pixel in ordinate
	
	int GetNumColors();
	// Restituisce il numero massimo di colori
	
	//void SetColor(int);
	void SetColor(const Colore&);	//FFF
	// Imposta il colore
	
	//int GetColor();
	Colore GetColor();	//FFF
	// Restituisce il colore attivo
	
	void SetDrwMode(DrwMode);
	// Imposta la modalita' di disegno
	
	class Rectangle SetClipRegion(const class Rectangle&);
	// Imposta l'area dello schermo modificabile dalle funzioni grafiche
	// ritorna l'area che era attiva precedentemente.
	
	void PutPixel(const Point &p);
	// Disegna un pixel nel punto p 
	void PutPixel0Up(const Point &p);
	// Disegna un pixel nel punto p con (0,0) in alto a sinistra
	
	int GetPixel(const Point &);
	
	void MoveTo(const Point &p);
	void MoveTo(VUnit x,VUnit y);
	// Imposta la posizione corrente tramite un punto o tramite le coordinate
	
	Point CurrentPos();
	// Ritorna la posizione corrente.
	
	void LineTo(const Point& p,LineType=SOLID_LINE);
	void LineTo(VUnit x,VUnit y,LineType=SOLID_LINE);
	
	// Disegna una linea dalla posizione corrente al punto indicato.
	// Usa il tipo di linea impostato (SetLineStyle) e il colore impostato
	// (SetColor).
	
	void Line(const Point& p1, const Point& p2,LineType lt=SOLID_LINE);
	// Disegna una linea tra due punti.
	
	void BigLine(const Point& p1, const Point& p2, VUnit width);
	// Disegna una linea tra due punti dello spessore indicato da width

	void TransparentRect(const class Rectangle& r, int transparency = 0); //0 = full color, 255=transparent

	void Slur(const Point& p1, const Point& p2, bool up, LineType lt=SOLID_LINE);
	void Slur(const Point& p1, const Point& cp1, const Point& cp2, const Point& p2, bool up, LineType lt=SOLID_LINE);
	void Brace(const Point& center, VUnit height);


	void Rect(const class Rectangle& r,Bool isfilled=FALSE);
	// Disegna un rettangolo che opzionalmente puo' essere riempito

	void BigRect(const class Rectangle& r, VUnit width = 1);
	// Disegna un rettangolo vuoto con spessore

	void ShadowRect(const class Rectangle& r,Bool isOn);
	// Disegna un rettangolo in modo da dare l'effetto dell'ombra
	// isOn=TRUE indica che deve essere rialzato
	// isOn=FALSE indica che deve essere incavato
	
	void Arc(const class Rectangle& r, short Angle1, short Angle2, Bool isfilled=FALSE);
	// Disegna l'arco dell'ellisse inscritta nel rettangolo r, compreso
	// tra gli angoli Angle1 ed Angle2 (indicati in gradi).
	// Opzionalmente puo' essere riempito, 
	
	void Ellipse(const class Rectangle& r,Bool isFilled=FALSE);
	// Disegna l'ellisse limitata dal rettangolo r ed opzionalmente
	// puo' essere riempita.
	
	FontID SetFont(FontID);
	// Seleziona il font indicato dal FontID, serve a ripristinare un font
	
	FontID SetTextFont(const wxFont& font);
	// Seleziona il font indicato dal FontID, serve a ripristinare un font
	
	FontID GetActiveFont();
	// Restituisce il FontID del font attivo attualmente.
	
	int GetFontAscent();
	// Restituisce la distanza della base line del font dalla cima.
	
	int GetFontHeight();
	// Restituisce la altezza del font.

	LiooFont* GetFont(const char* fontname);
	void ReplaceFont(const char* fontname, LiooFont*);
	
	void TextTo(const Point &p);
	void TextTo(VUnit x,VUnit y);
	// Imposta la posizione corrente in modo che la base line del font
	// sia nel punto indicato.
	
	void OutText(unsigned char *);
	// Scrive il testo indicato con il font attualmente selezionato.
	
	int GetTextWidth(unsigned char *);
	// Ritorna la larghezza in pixel del testo specificato.
	int GetTextHeight(unsigned char *txt);
	// Ritorna l'altezza in pixel del testo specificato.
	
	void SetSystemFont(wxFont* font);	//DB
	wxFont* GetSystemFont();
	void OutSystemText(char* text);
	VUnit GetSystemTextWidth(char* text);
	VUnit GetSystemTextHeight(char* text);
	void OutSystemText(unsigned short* text);
	VUnit GetSystemTextWidth(unsigned short* text);
	VUnit GetSystemTextHeight(unsigned short* text);
	VUnit GetSystemFontAscent();
	
	
	int WaitKey();
	// Aspetta la pressione di un tasto e ritorna il tasto premuto.
	
	void SetTmpKey(int key);
	// Imposta tmpKey = key e KeyPressed = TRUE.
	
	int KeyHit();
	// ritorna TRUE se e' stato premuto un tasto che puo' essere prelevato
	// con WaitKey.
	
	int LastKey();
	// ritorna l'ultimo tasto ottenuto con WaitKey().
	
	long ImageSize(const class Rectangle&);
	// Ritorna il numero di byte necessari a memorizzare l'immagine contenuta nel
	// rettangolo indicato
	
	void GetImage(const class Rectangle&,void *buf);
	// Riempie il buffer buf con i dati relativi all'immagine presente nel rettangolo r.
	
	void PutImage(const Point&,Pixmap img);
	// Ripristina a partire dal punto p l'immagine memorizzata del buffer buf.
	// Il ripristino avviene con la modalita' indicata
	
	void GSetCursor (CursorTypes ct, int fgc = NERO, int bgc = BIANCO);
	CursorTypes GGetCursor();
	void SetMouse(Mouse* m);
	// imposta il gestore del mouse.
	
	Mouse *GetMouse();
	// restiuisce il gestore del mouse attivo.
	
	int MouseInit();
	// Inizializza il Mouse.
	// Ritorna -1 se l'inizializzazione fallisce (driver non presente)
	// IN REALTA` NON FA NULLA E RITORNA SEMPRE 0.
	
	void MouseOn();
	// Non fa nulla.
	
	void MouseOff();
	// Non fa nulla.
	
	void SetMousePoint(const Point &p);
	// Imposta MousePoint = p.
	
	void SetMouseAction(int act);
	// Imposta MouseAction = act.
	
	int MousePos(Point& pm);
	// Mette in pm la posizione del mouse e ritorna lo stato dei bottoni
	// del mouse.                   
	
	int Get_VU2UP(int carcode, int fontID=MUSICFONT_ID);
	int Get_VU2DWN(int carcode, int fontID=MUSICFONT_ID);
	int Get_VU2RG(int carcode, int fontID=MUSICFONT_ID);
	int Get_VU2LF(int carcode, int fontID=MUSICFONT_ID);
	int Get_VU2DX(int carcode, int fontID=MUSICFONT_ID);
	int Get_VU2DY(int carcode,  int fontID=MUSICFONT_ID);
	// 6 funzioni che ritornano i 6 nuovi parametri della struct 'Carattere'
	
	
#ifdef SCR_FINESTRE    
	void CopyPixmap (Widget sorg,int x,int y,int width,int height);
	// realizza la copia dalla finestra sorgente  alla finestra attiva 
#else
	LPixmap* GetLPixmapCopia();	
	// ritorna il puntatore alla pixmap di Copia
	LPixmap* GetLPixmapDisegno();
	// ritorna il puntatore alla pixmap di Disegno
	LPixmap* GetLPixmapCurrent();
	// ritorna il puntatore alla pixmap corrente
	
	void InitPixmaps();
	
	void CopyPixmap (LPixmap *pixmap,int x,int y,int width,int height);
	// realizza la copia dalla pixmap sorgente alla finestra attiva
	void SwapPixmap();
	// scambia i puntatori Disegno e Copia
	void GResetPixmap();
	// imposta i precedenti valori relativi a  window, drawgc, widget 
	void GSetPixmap ( LPixmap *lpixmap);
	// salva i valori precedenti di window, drawgc, widget e imposta 
	// i valori relativi alla pixmap
#endif    
	void GNewDC(Widget);
	void GDeleteDC();
	void SetMainDC(wxDC* dc);
	
	// per aggiungere un font personalizzato
	void AddFont(char * FontName);
	
	// per caricare di nuovo la tabella del font, il .tbl 
	void UpdateFont(FontID FONT_ID);
	
	// per caricare di nuovo la tabella del font, il .fon 
	void ReloadBasicFonts();
	
	float GetMagnification();
	void SetMagnification(float magnification);

	Bool IsAutoMagnification();
	void EnableAutoMagnification(Bool status = TRUE);
};
 
#endif //GSCREEN_HPP
