/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: lwindow.cpp,v 1.1 2005/07/25 16:30:30 bellinpo Exp $ */


#include "drw/lwindow.hpp"
#include <string.h>

LWindow::LWindow()
{
	width=0;
	height=0;
	title[0]='\0';
	savedArea=NULL;
}

LWindow::LWindow(VUnit w, VUnit h)
{
	// Misure interne della finestra
	width=w;
	height=h;
	
	title[0]='\0';
	savedArea=NULL;
}

VUnit LWindow::GetVU2Up() { return height; };
VUnit LWindow::GetVU2Dwn() { return 0; };
VUnit LWindow::GetVU2Rgt() { return width; };
VUnit LWindow::GetVU2Lft() { return 0; };

Point LWindow::Center()
{
	return Rectangle(*this).Center();
}

void LWindow::Draw()
{
	Rectangle r = Rectangle(*this);
	Rectangle rall = r.Expand(WIN_LFT_BRD_WDT,
		WIN_RGT_BRD_WDT,
		WIN_UP_BRD_WDT,
		WIN_DWN_BRD_WDT);
	
	Scr->GetMouse()->Hide();
	Scr->SetDrwMode(DefMode);
	Colore oldcol = Scr->GetColor();
	Scr->SetColor((ColorIcon)gray);
	Scr->Rect(rall,TRUE);
	Scr->ShadowRect(r.Expand(1,1),FALSE);
	Scr->ShadowRect(rall,TRUE);
	FontID ofont=Scr->SetFont(nTextFont);
	Scr->TextTo(r.PUpLft+Point(10,4));
	Scr->OutText((unsigned char *)title);
	Scr->SetFont(ofont); 
	Scr->GetMouse()->Show();
	Scr->SetColor(oldcol);
} 

void LWindow::Clear()
{
	Scr->GetMouse()->Hide();
	Colore oldcol = Scr->GetColor();
	Scr->SetColor(GetColor());
	Scr->Clear();
	Scr->SetColor(oldcol);
	Scr->GetMouse()->Show();
}

void LWindow::SetTitle(unsigned char *tit)
{
	strcpy((char *)title,(char *)tit);
}

void LWindow::Open(Bool save)
{
	if(save)
	{
		Rectangle rWin=Rectangle(*this).Expand(WIN_LFT_BRD_WDT,WIN_RGT_BRD_WDT,                
			WIN_UP_BRD_WDT,WIN_DWN_BRD_WDT);
		long size=Scr->ImageSize(rWin);
		savedArea=new char[size];
		if (!savedArea)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","LWindow","Open");
		
		Scr->GetMouse()->Hide();
		if(savedArea!=NULL)
			Scr->GetImage(rWin,savedArea);
		Scr->GetMouse()->Show();
	}
	else
		savedArea=NULL;
	Draw();
}

void LWindow::Close()
{
	Rectangle rWin=Rectangle(*this).Expand(WIN_LFT_BRD_WDT,WIN_RGT_BRD_WDT,
		WIN_UP_BRD_WDT,WIN_DWN_BRD_WDT);
	Scr->GetMouse()->Hide();
    Colore oldcol = Scr->GetColor();
	Scr->SetColor((ColorIcon)gray);
    Scr->Rect(rWin,TRUE);
	Scr->GetMouse()->Show();
	Scr->SetColor(oldcol);
}

void LWindow::BeginPaint()
{
	oldClipArea=Scr->SetClipRegion(Rectangle(*this));
}

void LWindow::EndPaint()
{
	Scr->SetClipRegion(oldClipArea);
}


