/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: memenu.cpp,v 1.1 2005/07/25 16:30:30 bellinpo Exp $ */


#include "drw/memenu.hpp"
#include "drw/scorefrm.hpp"
#include "drw/gscreen.hpp"
#include "gui/lioowin.hpp"
#include "drw/menu.hpp"
#include "smr/smrmain.hpp"  
#include "drw/musicedt.hpp"  
#include "frmanage.hpp"

extern int AutoLineBreak;

extern GScreen *Scr;
extern LiooWindow *pMusicWin;
extern Widget LiooWidget;
extern ScrList LiooListWidget;
extern wxDC *currDC;
extern wxDC *currMainDC;
extern wxMemoryDC *currMDC;

extern void CreateKeySignatureMenu();

extern void CreateNotaMenu();
extern void CreateSmallNotesMenu();
extern void CreateBeamingMenu();
extern void CreateRestsMenu();
extern void CreateAccidentalsMenu();

extern void CreateAugmentationsMenu();
extern void CreateRepeatsMenu();
extern void CreateBarlinesMenu();
extern void CreateClefsMenu();
extern void CreateBracketsMenu();
extern void CreateHorizontalsMenu();
extern void CreateRefrainsMenu();
extern void CreateExpressionsMenu();
extern void CreateGenericExpMenu();
extern void CreateOrnamentsMenu();
extern void CreateKoreanOrnamentsMenu();
extern void CreateFingeringMenu();
extern void CreateMutesMenu();
extern void CreateHarmonicsMenu();
extern void CreateBreathGlassMenu();
extern void CreatePedalsMenu();
extern void CreateStringsMenu();
extern void CreateViolinMenu();

BEGIN_EVENT_TABLE(MEMenuBar, wxMenuBar)
END_EVENT_TABLE()


MEMenuBar::MEMenuBar()
{
	fileMenu = NULL;
	editMenu = NULL;
	miscMenu = NULL;
	symbolsMenu = NULL;
	formatMenu = NULL;
	executionMenu = NULL;
	helpMenu = NULL;
	analysisMenu = NULL;
}


MEMenuBar::~MEMenuBar()
{
}


wxMenu* MEMenuBar::GetfileMenu() 
{ 
	return fileMenu; 
};

wxMenu* MEMenuBar::GeteditMenu() 
{ 
	return editMenu;
};

wxMenu* MEMenuBar::GetviewMenu() 
{ 
	return viewMenu;
};

wxMenu* MEMenuBar::GetlyricMenu() 
{ 
	return lyricMenu;
};

wxMenu* MEMenuBar::GetmiscMenu() 
{ 
	return miscMenu;
};

wxMenu* MEMenuBar::GetsymbolsMenu() 
{
	return symbolsMenu; 
};

wxMenu* MEMenuBar::GetformatMenu() 
{ 
	return formatMenu;
};

wxMenu* MEMenuBar::GetexecutionMenu() 
{ 
	return executionMenu; 
};

wxMenu* MEMenuBar::GethelpMenu() 
{ 
	return helpMenu; 
};

wxMenu* MEMenuBar::GetAnalysisMenu() 
{ 
	return analysisMenu; 
};

void MEMenuBar::SetAnalysisMenu(wxMenu* menu) 
{ 
	analysisMenu=menu; 
};

void MEMenuBar::CreateMenu()
{
	//Creazione menu File
	fileMenu = new wxMenu;

	fileMenu->Append(FILE_IMPORTMIDI_ID, "Import MIDI...");
	fileMenu->Append(FILE_EXPORTMIDI_ID, "Export MIDI...");
	fileMenu->AppendSeparator();
	fileMenu->Append(FILE_CLOSE_ID, "Close");
	
	//Creazione menu Edit
	editMenu = new wxMenu;
	editMenu->Append(EDIT_NEWPART_ID, "New Part...");
	editMenu->Append(EDIT_CUTAPART_ID, "Cut a Part");
	editMenu->Append(EDIT_COPYAPART_ID, "Copy a Part");
	editMenu->Append(EDIT_PASTEAPART_ID, "Paste a Part");
	editMenu->Append(EDIT_MOVEPARTS_ID, "Move Parts");
	
	editMenu->AppendSeparator();
	editMenu->Append(EDIT_TRANSPOSITION_ID, "Transposition...");
	
	editMenu->AppendSeparator();
	editMenu->Append(EDIT_SPLITTINGPART_ID, "Splitting Layer -> Part");
	editMenu->Append(EDIT_MERGINGPARTS_ID, "Merging Parts -> Layers");
	
	editMenu->AppendSeparator();
	editMenu->Append(EDIT_JOININGPARTS_ID, "Joining Parts -> Multistaff");
	editMenu->Append(EDIT_DISJOINPARTS_ID, "Disjoin Multistaff -> Parts");
	
	editMenu->AppendSeparator();
	editMenu->Append(EDIT_MOVEALAYER_ID, "Move a Layer");
	
	editMenu->AppendSeparator();
	editMenu->Append(EDIT_NEWMEASUREBEFORE_ID, "New Measure Column Before");
	editMenu->Append(EDIT_NEWMEASUREAFTER_ID, "New Measure Column After");
	editMenu->Append(EDIT_CUTAMEASURE_ID, "Cut a Measure Column ");
	editMenu->Append(EDIT_COPYAMEASURE_ID, "Copy a Measure Column");
	editMenu->Append(EDIT_PASTEAMEASURE_ID, "Paste a Measure Column After");
	
	editMenu->AppendSeparator();
	editMenu->Append(EDIT_NEWSMEASUREBEFORE_ID, "New Single Measure Before");
	editMenu->Append(EDIT_NEWSMEASUREAFTER_ID, "New Single Measure After");
	editMenu->Append(EDIT_CUTSMEASURE_ID, "Cut Single Measure");
	editMenu->Append(EDIT_COPYSMEASURE_ID, "Copy Single Measure");
	editMenu->Append(EDIT_PASTESMEASURE_ID, "Paste Single Measure Over");
	
	editMenu->AppendSeparator();
	editMenu->Append(EDIT_CUTPAGEBOX_ID, "Cut a Page Box");
	editMenu->Append(EDIT_COPYPAGEBOX_ID, "Copy a Page Box");
	editMenu->Append(EDIT_PASTEPAGEBOX_ID, "Paste a Page Box");
	
	//Creazione menu View
	viewMenu = new wxMenu;
	viewMenu->Append(VIEW_COMPUTERVIEW_ID, "Computer View");
	viewMenu->Append(VIEW_PRINTVIEW_ID, "Print View");
	
	viewMenu->AppendSeparator();
	viewMenu->Append(VIEW_COMPUTERVIEWPARAMETERS_ID, "Computer View Parameters...");
	viewMenu->Append(FILE_PRINTPARAMETERS_ID, "Print Parameters...");
	
	viewMenu->AppendSeparator();
	viewMenu->Append(VIEW_HIDEAPART_ID, "Hide a Part");
	viewMenu->Append(VIEW_SHOWALLPARTS_ID, "Show Hidden Parts");
	viewMenu->Append(VIEW_VIEWLAYER_ID, "View/Identify Layer");
	viewMenu->AppendSeparator();
	viewMenu->Append(VIEW_SEEBARLINES_ID, "See Barlines","",TRUE);
	viewMenu->AppendSeparator();
	viewMenu->Append(VIEW_FIT2WINDOW_ID, "Fit Image to Window","");
	
	//Creazione menu Lyric
	lyricMenu = new wxMenu;
	lyricMenu->Append(LYRIC_ADDLYRICTOAPART_ID, "New Lyric");
	lyricMenu->Append(LYRIC_EDITLYRIC_ID, "Edit Lyric");
	lyricMenu->Append(LYRIC_REMOVELYRIC_ID, "Hide Lyric");
	lyricMenu->Append(LYRIC_SELECTLYRIC_ID, "Select Lyric");
	
	//Creazione menu Misc
	miscMenu = new wxMenu;
	miscMenu->Append(MISC_OPENALL_ID, "Open All Menu Symbols");
	miscMenu->Append(MISC_CLOSEALL_ID, "Close All Menu Symbols");
	
	miscMenu->AppendSeparator();
	miscMenu->Append(MISC_JUMPTOSYMBOLIC_ID, "Jump to Symbolic");
	miscMenu->AppendSeparator();
	miscMenu->Append(MISC_SEELINKS_ID, "See Links","",TRUE);
	miscMenu->Append(MISC_EDITLINK_ID, "Edit Links","",TRUE);
	
	//Creazione menu Symbols
	symbolsMenu = new wxMenu;
	symbolsMenu->Append(SYMB_TIMESIGNATURE_ID, "Time Signature...");
	symbolsMenu->Append(SYMB_KEYSIGNATURE_ID, "Key Signature...", "", TRUE);
	symbolsMenu->Append(SYMB_METRONOME_ID, "Metronome...");
	symbolsMenu->Append(SYMB_DYNAMICS_ID, "Dynamics...");
	symbolsMenu->Append(SYMB_FRETBOARDS_ID, "Fretboards...");
	symbolsMenu->Append(SYMB_ANNOTATIONS_ID, "Annotations...");
	symbolsMenu->Append(SYMB_GENERICTEXT_ID, "Generic Text...");
	symbolsMenu->AppendSeparator();
	symbolsMenu->Append(SYMB_INSERTTEXT_ID, "Insert Page Text");	
	symbolsMenu->Append(SYMB_INSERTIMAGE_ID, "Insert Page Image");	
	symbolsMenu->AppendSeparator();
	
	symbolsMenu->Append(SYMB_NOTES_ID, "Notes", "", TRUE);
	symbolsMenu->Append(SYMB_SMALLNOTES_ID, "Small Notes", "", TRUE);
	symbolsMenu->Append(SYMB_ACCIDENTALS_ID, "Accidentals", "", TRUE);
	symbolsMenu->Append(SYMB_RESTS_ID, "Rests", "", TRUE);
	symbolsMenu->Append(SYMB_AUGMENTATIONS_ID, "Augmentations", "", TRUE);
	symbolsMenu->AppendSeparator();
	
	symbolsMenu->Append(SYMB_EXPRESSIONS_ID, "Expressions", "", TRUE);
	symbolsMenu->Append(SYMB_GENERICEXP_ID, "Generic Symbols", "", TRUE);
	symbolsMenu->Append(SYMB_ORNAMENTS_ID, "Ornaments", "", TRUE);
	symbolsMenu->Append(SYMB_KOREANORNAMENTS_ID, "Korean Ornaments", "", TRUE);
	symbolsMenu->Append(SYMB_MUTES_ID, "Mutes", "", TRUE);
	symbolsMenu->Append(SYMB_HARMONICS_ID, "Harmonics", "", TRUE); 
	symbolsMenu->Append(SYMB_FINGERING_ID, "Fingering", "", TRUE);
	symbolsMenu->Append(SYMB_STRINGS_ID, "Strings", "", TRUE);
	symbolsMenu->Append(SYMB_PEDALS_ID, "Pedals", "", TRUE);
	symbolsMenu->Append(SYMB_VIOLIN_ID, "Violin", "", TRUE);
	symbolsMenu->Append(SYMB_PERCUSSIONS_ID, "Percussions", "", TRUE);
	symbolsMenu->Append(SYMB_BREATHGLASS_ID, "Breath/glass", "", TRUE);
	symbolsMenu->AppendSeparator();
	
	symbolsMenu->Append(SYMB_BEAMING_ID, "Beaming", "", TRUE);
	symbolsMenu->Append(SYMB_REPEATS_ID, "Repeats", "", TRUE);
	symbolsMenu->Append(SYMB_BARLINES_ID, "Barlines", "", TRUE);
	symbolsMenu->Append(SYMB_CLEFS_ID, "Clefs", "", TRUE);
	symbolsMenu->Append(SYMB_BRACKETS_ID, "Brackets", "", TRUE);
	symbolsMenu->AppendSeparator();
	
	symbolsMenu->Append(SYMB_HORIZONTALS_ID, "Horizontals", "", TRUE);
	symbolsMenu->Append(SYMB_REFRAINS_ID, "Refrains", "", TRUE);
	
	
	//Creazione menu Format
	formatMenu = new wxMenu;
	formatMenu->Append(FORM_CHECK_ID, "Checking...");
	formatMenu->Append(FORM_JUSTIFICATION_ID, "Justification...");
	formatMenu->Append(FORM_LINEBREAKING_ID, "Line Breaking", "", TRUE);
	
	formatMenu->Append(FORM_AUTOBEAMING_ID, "Auto Beaming", "", TRUE);
	formatMenu->Check(FORM_AUTOBEAMING_ID,true);
	formatMenu->Append(FORM_BEAMING_ID, "Beaming...");
	formatMenu->Append(FORM_UPDOWN_ID, "Up/Down Stem...");
	formatMenu->AppendSeparator();
	formatMenu->Append(FORM_EDITMILLA_ID, "Edit SMFL...");
	formatMenu->Append(FORM_RELOADMILLA_ID, "ReLoad SMFL");
	
	wxMenu *submenuMillaDebug = new wxMenu;
	submenuMillaDebug->Append(FORM_DEBUGMILLAINSERTION_ID, "Debug SMFL Insertion", "", TRUE);
	submenuMillaDebug->Append(FORM_DEBUGMILLAPOSITIONING_ID, "Debug SMFL Positioning", "", TRUE);
	
	formatMenu->Append(FORM_DEBUGNORMAL_ID, "Debug SMFL", submenuMillaDebug);
	formatMenu->Append(FORM_SEEINVISIBLE_ID, "See Invisible Figures", "", TRUE);
	formatMenu->Append(FORM_SEEANCHOR_ID, "See Anchorages", "", TRUE);
	formatMenu->AppendSeparator();
	
	wxMenu *submenuEditTable = new wxMenu;
	submenuEditTable->Append(FORM_EDITTABLE1_ID, "Music Font 1", "", FALSE);
	submenuEditTable->Append(FORM_EDITTABLE2_ID, "Music Font 2", "", FALSE);
	submenuEditTable->Append(FORM_EDITTABLE3_ID, "Music Font 3", "", FALSE);
	submenuEditTable->Append(FORM_EDITGENERIC1_ID, "Custom Font 1", "", FALSE);
	submenuEditTable->Append(FORM_EDITGENERIC2_ID, "Custom Font 2", "", FALSE);
	submenuEditTable->Append(FORM_EDITGENERIC3_ID, "Custom Font 3", "", FALSE);
	submenuEditTable->Append(FORM_EDITPERCUSSIONS_ID, "Custom Percussions", "", FALSE);
	submenuEditTable->Append(FORM_EDITPEDALS_ID, "Custom Pedals", "", FALSE);
	
	formatMenu->Append(FORM_EDITTABLE_ID, "Edit Font Tables", submenuEditTable);
	formatMenu->AppendSeparator();
	formatMenu->Append(FORM_SCOREEXCERPT_ID, "Score Excerpt");

	//Creazione menu Help
	helpMenu = new wxMenu;
	helpMenu->Append(HELP_GUIDE_ID, "Help");
	helpMenu->AppendSeparator();
	helpMenu->Append(HELP_ABOUT_ID, "About...");
	
	//Inserisci i menu nella barra dei menu
	Append(fileMenu, "File");
	Append(editMenu, "Edit");
	Append(viewMenu, "View");
	Append(lyricMenu, "Lyric");
	Append(miscMenu, "Misc");
	Append(symbolsMenu, "Symbols");
	Append(formatMenu, "Format");
	if(analysisMenu)
		Append(analysisMenu,"Analysis");
	Append(helpMenu, "Help");
	// attiva i check per i menu gi aperti
	ScrObject *scrObj=LiooListWidget.GetFirst();
	while (scrObj)
	{
		Menu* menu = (Menu*)scrObj;
		symbolsMenu->Check(menu->GetmenuItemID(), TRUE);
		scrObj=scrObj->GetNext();
	} 
	
	InitMenu();
	
}


void MEMenuBar::InitMenu()
{
	if ((!SimbImg))
	{
		viewMenu->Enable( VIEW_COMPUTERVIEWPARAMETERS_ID, false);
		
		viewMenu->Enable(VIEW_HIDEAPART_ID, false);
		viewMenu->Enable(VIEW_SHOWALLPARTS_ID, false);
		editMenu->Enable(EDIT_NEWPART_ID, false);
		editMenu->Enable(EDIT_CUTAPART_ID, false);
		editMenu->Enable(EDIT_COPYAPART_ID, false);
		editMenu->Enable(EDIT_PASTEAPART_ID, false);
		editMenu->Enable(EDIT_SPLITTINGPART_ID, false);
		editMenu->Enable(EDIT_MERGINGPARTS_ID, false);
		editMenu->Enable(EDIT_JOININGPARTS_ID, false);
		editMenu->Enable(EDIT_DISJOINPARTS_ID, false);
		editMenu->Enable(EDIT_MOVEPARTS_ID, false);
		editMenu->Enable( EDIT_MOVEALAYER_ID, false);
		editMenu->Enable( EDIT_TRANSPOSITION_ID, false);
		
		viewMenu->Enable( VIEW_VIEWLAYER_ID, false);
		editMenu->Enable( EDIT_NEWMEASUREBEFORE_ID, false);
		editMenu->Enable( EDIT_NEWMEASUREAFTER_ID, false);
		editMenu->Enable( EDIT_NEWSMEASUREBEFORE_ID, false);
		editMenu->Enable( EDIT_NEWSMEASUREAFTER_ID, false);
		editMenu->Enable( EDIT_CUTAMEASURE_ID, false);
		editMenu->Enable( EDIT_CUTSMEASURE_ID, false);
		editMenu->Enable( EDIT_PASTEAMEASURE_ID, false);
		editMenu->Enable( EDIT_PASTESMEASURE_ID, false);
		editMenu->Enable( EDIT_COPYAMEASURE_ID, false);
		editMenu->Enable( EDIT_COPYSMEASURE_ID, false);
		editMenu->Enable( EDIT_CUTPAGEBOX_ID, false);
		editMenu->Enable( EDIT_COPYPAGEBOX_ID, false);
		editMenu->Enable( EDIT_PASTEPAGEBOX_ID, false);
		
		lyricMenu->Enable( LYRIC_ADDLYRICTOAPART_ID, false);
		lyricMenu->Enable( LYRIC_EDITLYRIC_ID, false);
		lyricMenu->Enable( LYRIC_REMOVELYRIC_ID, false);
		lyricMenu->Enable( LYRIC_SELECTLYRIC_ID, false);
		
		miscMenu->Enable( MISC_OPENALL_ID, false);
		miscMenu->Enable( MISC_CLOSEALL_ID, false);
		miscMenu->Enable( MISC_JUMPTOIMAGE_ID, false);
		viewMenu->Enable( VIEW_FIT2WINDOW_ID, true);
		
		symbolsMenu->Enable( SYMB_KEYSIGNATURE_ID, false);
		symbolsMenu->Enable( SYMB_TIMESIGNATURE_ID, false);
		symbolsMenu->Enable( SYMB_DYNAMICS_ID, false);
		symbolsMenu->Enable( SYMB_FRETBOARDS_ID, false);
		symbolsMenu->Enable( SYMB_ANNOTATIONS_ID, false);
		symbolsMenu->Enable( SYMB_GENERICTEXT_ID, false);
		symbolsMenu->Enable( SYMB_NOTES_ID, false);
		symbolsMenu->Enable( SYMB_SMALLNOTES_ID, false);
		symbolsMenu->Enable( SYMB_ACCIDENTALS_ID, false);
		symbolsMenu->Enable( SYMB_RESTS_ID, false);
		symbolsMenu->Enable( SYMB_AUGMENTATIONS_ID, false);
		symbolsMenu->Enable( SYMB_EXPRESSIONS_ID, false);
		symbolsMenu->Enable( SYMB_GENERICEXP_ID, false);
		symbolsMenu->Enable( SYMB_ORNAMENTS_ID, false);
		symbolsMenu->Enable( SYMB_KOREANORNAMENTS_ID, false);
		symbolsMenu->Enable( SYMB_MUTES_ID, false);
		symbolsMenu->Enable( SYMB_HARMONICS_ID, false);
		symbolsMenu->Enable( SYMB_STRINGS_ID, false);
		symbolsMenu->Enable( SYMB_VIOLIN_ID, false);
		symbolsMenu->Enable( SYMB_PEDALS_ID, false);
		symbolsMenu->Enable( SYMB_PERCUSSIONS_ID, false);
		symbolsMenu->Enable( SYMB_BREATHGLASS_ID, false);
		symbolsMenu->Enable( SYMB_VIOLIN_ID, false);
		symbolsMenu->Enable( SYMB_FINGERING_ID, false);
		symbolsMenu->Enable( SYMB_REPEATS_ID, false);
		symbolsMenu->Enable( SYMB_CLEFS_ID, false);
		symbolsMenu->Enable( SYMB_BRACKETS_ID, false);
		symbolsMenu->Enable( SYMB_HORIZONTALS_ID, false);
		symbolsMenu->Enable( SYMB_REFRAINS_ID, false);
		symbolsMenu->Enable( SYMB_BEAMING_ID, false);
		symbolsMenu->Enable( SYMB_BARLINES_ID, false);
		
		symbolsMenu->Enable( SYMB_INSERTTEXT_ID, false);
		symbolsMenu->Enable( SYMB_INSERTIMAGE_ID, false);
		
		formatMenu->Enable( FORM_CHECK_ID, false);
		formatMenu->Enable( FORM_JUSTIFICATION_ID, false);
		formatMenu->Enable( FORM_LINEBREAKING_ID, false);
		formatMenu->Enable( FORM_BEAMING_ID, false);
		formatMenu->Enable( FORM_UPDOWN_ID, false);
		formatMenu->Enable( FORM_EDITMILLA_ID, false);
		formatMenu->Enable( FORM_RELOADMILLA_ID, false);
		formatMenu->Enable( FORM_DEBUGMILLAINSERTION_ID, false);
		formatMenu->Enable( FORM_DEBUGMILLAPOSITIONING_ID, false);
		formatMenu->Enable( FORM_DEBUGNORMAL_ID, false);
		formatMenu->Enable( FORM_SEEINVISIBLE_ID, false);
		formatMenu->Enable( FORM_SEEANCHOR_ID, false);
		formatMenu->Enable( FORM_EDITTABLE_ID, false);

		if (tipoLeggio==LEGGIO_DLIOO)
		{
			symbolsMenu->Enable( SYMB_METRONOME_ID, false);
			executionMenu->Enable( EXEC_SYNCHRONIZATION_ID, false);
		}
		
		return;
	}
	else
	{
		viewMenu->Enable(VIEW_SEEBARLINES_ID, false);
		viewMenu->Enable(VIEW_FIT2WINDOW_ID, false);
		if ((tipoLeggio==LEGGIO_DLIOO))
		{			
			viewMenu->Enable(VIEW_HIDEAPART_ID, false);
			viewMenu->Enable(VIEW_SHOWALLPARTS_ID, false);
			editMenu->Enable(EDIT_NEWPART_ID, false);
			editMenu->Enable(EDIT_CUTAPART_ID, false);
			editMenu->Enable(EDIT_COPYAPART_ID, false);
			editMenu->Enable(EDIT_PASTEAPART_ID, false);
			viewMenu->Enable(VIEW_HIDEAPART_ID, false);
			viewMenu->Enable(VIEW_SHOWALLPARTS_ID, false);
			editMenu->Enable(EDIT_SPLITTINGPART_ID, false);
			editMenu->Enable(EDIT_MERGINGPARTS_ID, false);
			editMenu->Enable(EDIT_JOININGPARTS_ID, false);
			editMenu->Enable(EDIT_DISJOINPARTS_ID, false);
			editMenu->Enable(EDIT_MOVEPARTS_ID, false);
			
			editMenu->Enable( EDIT_NEWMEASUREBEFORE_ID, false);
			editMenu->Enable( EDIT_NEWMEASUREAFTER_ID, false);
			editMenu->Enable( EDIT_NEWSMEASUREBEFORE_ID, false);
			editMenu->Enable( EDIT_NEWSMEASUREAFTER_ID, false);
			editMenu->Enable( EDIT_CUTAMEASURE_ID, false);
			editMenu->Enable( EDIT_PASTEAMEASURE_ID, false);
			editMenu->Enable( EDIT_COPYAMEASURE_ID, false);

		}
	}
	
}

