/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: menu.cpp,v 1.1 2005/07/25 16:30:31 bellinpo Exp $ */


#include "drw/menu.hpp"
#include "drw/scrlist.hpp"

#include <wx/frame.h>

#include "drw/menufrm.hpp"
#include "gui/lioomenu.hpp"
#include "smr/smrmain.hpp"
#include "smr/workspac.hpp"


extern Widget WindowActive;
extern ScrList LiooListWidget;
extern void PrimoTasto(Menu *pm);

#ifndef assert
#define assert(x)
#endif

Menu::Menu()
{
	ID=CL_MENU;
	menuVect=NULL;
	nItem=0;
	lastPress=lastSel=-1;
	menuHeight=menuWidth=0;
	active=TRUE;
	
	SetScrWindow(NULL);
	SetScrDrawArea(NULL);
	Orizzontal = TRUE;
	Close = FALSE;
}

Menu::Menu(wxString menutitle, MenuItem itemID, int nbutton, bool orizzontal)
{
	ID=CL_MENU;
	menuVect=NULL;
	nItem=0;
	lastPress=lastSel=-1;
	menuHeight=menuWidth=0;
	active=TRUE;
	
	SetScrWindow(NULL);
	SetScrDrawArea(NULL);
	Orizzontal = orizzontal;
	Close = FALSE;
	menuItemID = itemID;
	title = menutitle;
	Init(nbutton);
	PrimoTasto(this);
}



Menu::~Menu()
{
	if(menuVect != NULL)
	{
		delete[] menuVect;
		menuVect = NULL;
	}
}

wxPoint Menu::GetPosition()
{ 
	int x, y;
	((wxMEMenuFrame*)GetScrDrawArea())->GetPosition(&x, &y);
	return wxPoint(x,y);
}

void Menu::Enable()
{ 
	active=TRUE;
}

void Menu::Disable() 
{
	active=FALSE;
}

void Menu::SetOrizzontal(Bool b)
{
	Orizzontal = b;
}

Bool Menu::GetOrizzontal()
{
	return Orizzontal;
}

void Menu::SetClose(Bool b)
{
	Close = b;
}

Bool Menu::GetClose()
{
	return Close;
}

Button* Menu::GetButton(int n)
{
	if ((n>=0) && (n<nItem))
		return (menuVect+n);
	return NULL;
}

int Menu::GetButton(Widget w)
{
	int n;
	
	if (w==NULL) 
	{
		return -1;
	}
	
	for(n=0;(n<nItem) && ((menuVect[n].GetScrWindow())!=w);n++) ;
	if ((n>=0) && (n<nItem)) return n;
	else return -1; 
}


void Menu::CloseMenu()
{
	SMRApp::GetApp().workspace->SetMenuConfig(GetmenuItemID(), GetPosition().x, GetPosition().y, GetOrizzontal());
	
	Scr->GCloseWindow(this->GetScrWindow());
	SetClose(TRUE);
	LiooListWidget.Delete(this);
}

int Menu::GetLastSel()
{
	return lastSel;
}

int Menu::GetLastPress()
{
	return lastPress;
}

void Menu::ResetLastSel()
{
	lastSel=-1;
}

VUnit Menu::GetVU2Up(){ return menuHeight; };
VUnit Menu::GetVU2Dwn() { return 0; };
VUnit Menu::GetVU2Lft() { return 0; };
VUnit Menu::GetVU2Rgt() { return menuWidth; };

void Menu::Init(int nit)
{
	if(nit != nItem)
	{
		if(menuVect != NULL)
		{
			delete[] menuVect;
		}
		menuVect = new Button[nit];
		if (!menuVect)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Menu.","Init");
		
		nItem = nit;
	}
	lastPress = -1;
	lastSel = -1;
}

void Menu::CalcolaDimensioni()
{
	if (Orizzontal)
	{
		menuVect[0].SetWidth(BUTTON_WIDTH/2);
		menuVect[0].SetHeight(BUTTON_HEIGHT);
		menuHeight = menuVect[nItem-1].GetHeight() + GAP_BUTTON*2;
		menuWidth = GAP_BUTTON;
		for(int n=0;n<nItem;n++) 
			menuWidth += menuVect[n].GetWidth() + GAP_BUTTON;
	}
	else
	{
		menuVect[0].SetWidth(BUTTON_WIDTH);
		menuVect[0].SetHeight(BUTTON_HEIGHT/2);
		menuWidth = menuVect[nItem-1].GetHeight() + GAP_BUTTON*2;
		menuHeight = GAP_BUTTON;
		for(int n=0;n<nItem;n++) 
			menuHeight += menuVect[n].GetHeight() + GAP_BUTTON;
	}  
}



void Menu::SetTextItem(int n, unsigned char *txt, int fnt, ActionFunction afun,
					   AbilID setAbil, Bool DrawArr, short bc, VUnit Width, VUnit Height)
{
	
	assert(n >= 0 && n < nItem);
	menuVect[n].SetAbilID(setAbil); 
	
	menuVect[n].InitText(txt,fnt,Height,Width,afun,DrawArr,bc);
}

void Menu::SetImageItem(int n, DrawFunction dfun, ActionFunction afun,
						AbilID setAbil, Bool DrawArr, short bc, VUnit Width, VUnit Height)
{
	
	assert(n>=0 && n<nItem);
	menuVect[n].SetAbilID(setAbil); 
	menuVect[n].InitImage(dfun,Height,Width,afun,DrawArr,bc);
}                        


void Menu::ChangeTextItem(int n,unsigned char *txt)
{
	if ((n<0) || (n>=nItem))
	{
		EM.Error(MUSICAL_ERR_AREA,"EC_BUTTON_GESTIONE","Menu","ChangeTextItem()");
		return;
	}
	menuVect[n].ChangeText(txt);
}

void Menu::ChangeTextItem(int n,unsigned char *txt, int fnt)
{
	if ((n<0) || (n>=nItem))
	{
		EM.Error(MUSICAL_ERR_AREA,"EC_BUTTON_GESTIONE","Menu","ChangeTextItem()");
		return;
	}
	menuVect[n].ChangeText(txt, fnt);
}


void Menu::ChangeImageItem(int n,DrawFunction dfun)
{
	assert(n>=0 && n<nItem);
	if ((n<0) || (n>=nItem))
	{
		EM.Error(MUSICAL_ERR_AREA,"EC_BUTTON_GESTIONE","Menu","ChangeImageItem()");
		return;
	}
	menuVect[n].ChangeImage(dfun);
}

void Menu::SetPos(DrawObject* drwobj ,const Point& p)
{ 
	int i;
	VUnit x, y;
	
	DrawObject::SetPos(drwobj,p);
	if (GetScrWindow()==NULL)	// menu' senza finestra
	{
		x=0;
		for(i=0;i<nItem;i++)
		{
			menuVect[i].SetPos(this,Point(x,0));
			x+=short(menuVect[i].GetWidth()+GAP_BUTTON);
		}
	}
	else
	{
		if (Orizzontal)
		{
			
			x=0;
			for(i=0;i<nItem;i++)
			{
				menuVect[i].SetPos(this,Point(x,0));
				x+=short(menuVect[i].GetWidth()+GAP_BUTTON);
			}
		}
		else
		{
			y=0;
			for(i=nItem-1;i>=0;i--)
			{
				menuVect[i].SetPos(this,Point(0,y));
				y+=short(menuVect[i].GetHeight()+GAP_BUTTON);
			}
		}
	}
}

void Menu::Draw()
{
	int i;
	Scr->GetMouse()->Hide();
	
	if (GetScrDrawArea()!=NULL)
		Scr->GSetDrawArea(GetScrDrawArea());
	
	Colore oldcol = Scr->GetColor();
	Scr->SetColor((ColorIcon)gray);
	// Oscura il menu precedente.
	Scr->Rect(class Rectangle(*this),TRUE);
	for(i = 0; i < nItem; i++)
	{
		menuVect[i].Draw();
	}
	Scr->GetMouse()->Show();
	Scr->SetColor(oldcol);
}


void Menu::Realize(int x, int y)
{
	Widget DrawA;  
	DrawA = Scr->GNewDrawArea(x, y, menuHeight, menuWidth, this, (char*)title.c_str());
	
	if (DrawA==NULL)
		EM.Error(GENERAL_ERR_AREA,"EC_DRAWAREA_FAULT","Menu","GNewDrawArea()");
	
	SetScrDrawArea(DrawA);
	SetScrWindow(DrawA);                             
	
	int currV=currentView;
	currentView=0; // menu are always on view 0
	SetPos(NULL,MENU_POS);
	currentView=currV;
	Scr->GSetMainDrawArea();
}


EventResult Menu::LeftButtonPress(const Point& p)
{
	int i;
	
	if(!active)
		return ER_NO_HIT;
	
	if ((WindowActive!=NULL) && (GetScrDrawArea()!=WindowActive))
		return ER_NO_HIT;
    
	for(i=0;i<nItem;i++)
		if(menuVect[i].LeftButtonPress(p)!=ER_NO_HIT)
		{
			lastPress=i;
			return ER_CONTINUE;
		}
		return ER_NO_HIT;
}

EventResult Menu::LeftButtonRelease(const Point& p)
{
	EventResult ret=ER_NO_HIT;
	
	if(!active)
		return ER_NO_HIT;
    
	if ((WindowActive!=NULL) && (GetScrDrawArea()!=WindowActive))
		return ER_NO_HIT;
	
	if(lastPress>=0)
	{
		ret=menuVect[lastPress].LeftButtonRelease(p);
		
		if(ret!=ER_NO_HIT)
			lastSel=lastPress;
		else
			lastSel=-1;
		lastPress=-1;
	}          
	
	return ret;
}

EventResult Menu::KeyPressed(int key)
{
	if(key>=-68 && key<=-59)
	{
		int pos=(-key)-59;
		
		if(pos<nItem)
		{
			Point butCenter=class Rectangle(menuVect[pos]).Center();
			
			LeftButtonPress(butCenter);
			return LeftButtonRelease(butCenter);
		}
	}
	return ER_NO_HIT;
}
