/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: menucan.cpp,v 1.1 2005/07/25 16:30:31 bellinpo Exp $ */


#include "drw/menucan.hpp"
#include "drw/gscreen.hpp"
#include "drw/scrlist.hpp"
#include "drw/musicedt.hpp"
#include "drw/scorecan.hpp"

extern GScreen *Scr;
extern ScrList LiooList,LiooListWidget;
extern Widget LiooWidget;
extern LiooWindow *pMusicWin;

extern wxMusicEditor *MusicEditor;

extern wxDC *currDC;
extern wxDC *currMainDC;
extern wxMemoryDC *currMDC;


IMPLEMENT_DYNAMIC_CLASS(wxMEMenuCanvas, wxScrolledWindow)

BEGIN_EVENT_TABLE(wxMEMenuCanvas, wxScrolledWindow)
EVT_PAINT(wxMEMenuCanvas::OnPaint)
EVT_LEFT_DOWN(wxMEMenuCanvas::OnMouse)
EVT_RIGHT_DOWN(wxMEMenuCanvas::OnMouse)
EVT_LEFT_UP(wxMEMenuCanvas::OnMouse)
EVT_RIGHT_UP(wxMEMenuCanvas::OnMouse)
EVT_MOTION(wxMEMenuCanvas::OnMouse)  
EVT_KEY_DOWN(wxMEMenuCanvas::OnKeyDown)  
END_EVENT_TABLE()

wxMEMenuCanvas::wxMEMenuCanvas( wxWindow *parent, const wxPoint &pos, const wxSize &size )
: wxScrolledWindow( parent, wxMEMenuCanvas_ID, pos, size, wxSUNKEN_BORDER )
{
	SetBackgroundColour(* wxLIGHT_GREY);
}

wxMEMenuCanvas::~wxMEMenuCanvas()
{
}

void wxMEMenuCanvas::OnPaint( wxPaintEvent &event )
{
	wxPaintDC dc(this);
	
#ifdef __WXMSW__
	
	OnDraw(dc);
	
#else
	
	PrepareDC(dc);
	
	int vbX,vbY; 
	// Top left corner of client 
	ViewStart(&vbX,&vbY); 
	int vX,vY,vW,vH; // Dimensions of client area in pixels 
	wxRegionIterator upd(GetUpdateRegion()); 
	
	// get the update rect list 
	while (upd) 
	{ vX = upd.GetX(); 
    vY = upd.GetY(); 
	vW = upd.GetW(); 
	vH = upd.GetH(); 
	
	dc.SetClippingRegion(vX,vY, vW, vH);
	OnDraw(dc);
	upd ++ ; 	
	} 
	
#endif
}


void wxMEMenuCanvas::OnDraw(wxDC& dc)
{
	ScrObject *pp;
	wxDC *prevDC=currDC;
	wxDC *prevMainDC=currMainDC;
	
	currDC=&dc;
	currMainDC=NULL;
	
	DrawObject::SetCurrentView(0);
	float scale=Scr->GetMagnification();
	Scr->SetMagnification(1.0);
	
	pp = LiooListWidget.GetNodoDA(GetParent());
	if (pp==NULL)
    {
		return;
    }
	else
    {
		Scr->Clear();
		pp->Draw();
    }
	
	currDC=prevDC;
	currMainDC=prevMainDC;
	Scr->SetMagnification(scale);
}

void wxMEMenuCanvas::OnMouse(wxMouseEvent &event)
{
	long x,y;
	
	EventInfo evInf;
	int action=0; 
	wxDC *prevDC=currDC;
	
	wxClientDC dc(this);
	DrawObject::SetCurrentView(0);
	float scale=Scr->GetMagnification();
	Scr->SetMagnification(1.0);
		
	currDC=&dc;
	currMainDC=NULL;
	PrepareDC(dc);
	wxPoint point = event.GetLogicalPosition(dc);
	x=point.x;
	y=point.y;
	
	
	Scr->SetMousePoint(Point(x, y));
	
	if(event.LeftDown())
		action = LEFTGODOWN;
	else if(event.MiddleDown())
		action = CENTRALGODOWN;
	else if(event.RightDown())
		action = RIGHTGODOWN;
	else if(event.LeftUp())
		action = LEFTGOUP;
	else if(event.MiddleUp())
		action = CENTRALGOUP;
	else if(event.RightUp())
		action = RIGHTGOUP;
	
	
	if(action!=0)
	{
		Scr->SetMouseAction(action);
		Scr->GSetDrawArea(GetParent());
	}
	
	//  segnala l'evento
	evInf=LiooListWidget.SignalEvent();
	
	if(evInf.eventResult==ER_END) 
	{
		// La deallocazione e' curata dal distruttore di GScreen che viene
		// richiamato automaticamente quando si esce dal programma.
		// Funziona anche se si distrugge la finestra col Quit di XWindow!
		Scr->GCloseWindow(LiooWidget);
	}
	
	currDC=prevDC;
	Scr->SetMagnification(scale);
}

#define NORMAL 0
#define SHIFT 1
#define ALT 2
#define CTRL 3

void wxMEMenuCanvas::OnKeyDown(wxKeyEvent &event)
{
}

