/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: mouse.cpp,v 1.1 2005/07/25 16:30:31 bellinpo Exp $ */


#include "drw/mouse.hpp"

Mouse::Mouse()
{
	hide=0;
	isImage=false;
	height=0;
	width=0;
}

void Mouse::On()
{ 
	Scr->MouseOn();
}

void Mouse::Off()
{
	Scr->MouseOff();
}

Point Mouse::GetPos()
{ 
	return MPos;
}

Bool Mouse::IsMoved()
{
	return MPos!=PrecMPos;
}

Point Mouse::GetPrecPos()
{ 
	return PrecMPos;
}

int Mouse::GetButton()
{
	return MBut;
}

void Mouse::SetRectSel()
{
	rectSelStatus=P1_RECT_SEL;
}

void Mouse::SetImgDragSel(int h,int w)
{
    isImage=true;
	height=h;
	width=w;
	rectSelStatus=D_SEL;
}

void Mouse::SetTxtDragSel()
{
	isImage=false;
	height=10;
	width=10;
	rectSelStatus=D_SEL;
}

void Mouse::NoRectSel()
{
	rectSelStatus=NO_RECT_SEL;
}

Rectangle Mouse::GetRectSel()
{
	rectSel.Adjust();
	return rectSel;
};

void Mouse::Show()
{
	hide--;
	if(!hide)
		Draw();
}

void Mouse::Hide()
{
	if(!hide)
		Draw();
	hide++;
}

Bool Mouse::IsShowing()
{
	return !hide;
}

int Mouse::Check()
{
	Point p;
	Bool rectBegin=false,rectEnd=false;                      
	
	MBut=Scr->MousePos(p);
	rectBegin=(rectSelStatus==P1_RECT_SEL && MBut & LEFTGODOWN);
	rectEnd=(rectSelStatus==P2_RECT_SEL && MBut & LEFTGOUP);
	PrecMPos=MPos;
	if(p!=MPos || rectBegin || rectEnd)
	{
		Draw();
		MPos=p;
		
		if(rectBegin)
		{
			rectSel.PUpLft=p;
			rectSelStatus=P2_RECT_SEL;
		}
		else if(rectEnd)
			rectSelStatus=P1_RECT_SEL;
		
		Draw();
	}
	return MBut;
}

int Mouse::Init()
{
	if(Scr->MouseInit() == -1)
		return -1;
	MBut = Scr->MousePos(MPos);
	PrecMPos = MPos;
	hide = 1; // Mouse inizialmente invisibile
	return 0;
}

void Mouse::Draw()
{
	
	if(rectSelStatus == P2_RECT_SEL)
	{ 
		Colore oldcol = Scr->GetColor();
		Scr->SetDrwMode(XORMode);
		Scr->SetColor((ColorIcon)MOUSECOLOR);
		rectSel.PDwnRgt = MPos;
		Scr->Rect(rectSel);
		Scr->SetDrwMode(DefMode);
		Scr->SetColor(oldcol);
	}
	
	if(rectSelStatus == D_SEL)
	{ 
		if(!isImage)	// Testo
		{	
			Colore oldcol = Scr->GetColor();
			Scr->SetDrwMode(XORMode);
			Scr->SetColor((ColorIcon)MOUSECOLOR);
			Scr->Rect(class Rectangle(Point(MPos.x+width, MPos.y-height),
				Point(MPos.x, MPos.y)), TRUE);
			Scr->SetDrwMode(DefMode);
			Scr->SetColor(oldcol);
		}
		else	// Immagine
		{		
			Colore oldcol = Scr->GetColor();
			Scr->SetDrwMode(XORMode);
			Scr->SetColor((ColorIcon)MOUSECOLOR);
			Scr->Rect(class Rectangle(Point(MPos.x, MPos.y),
				Point(MPos.x+width, MPos.y-height)), FALSE);
			Scr->SetDrwMode(DefMode);	
			Scr->SetColor(oldcol);
		}
	}
	
}             

