/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: mouse.hpp,v 1.1 2005/07/25 16:30:31 bellinpo Exp $ */


#ifndef MOUSE_HPP
#define MOUSE_HPP

#include "drw/rectangl.hpp"

class GScreen;

extern GScreen *Scr;

#include "lioo.hpp"

enum RectSel { 
	NO_RECT_SEL, // non si seleziona un rettangolo
		P1_RECT_SEL, // selezione del primo punto del rettangolo
		P2_RECT_SEL, // selezione del secondo punto del rettangolo
		D_SEL	   // selezione drag	
};

class Mouse
{
protected:
    Point MPos,PrecMPos;
	// posizione attuale del mouse e posizione precedente.
    int MBut;
	// stato dei bottoni
    int hide;
	// se hide>0 allora il mouse e' invisibile
    RectSel rectSelStatus;
	// indica lo stato della selezione tramite rettangolo.
    class Rectangle rectSel;
	// indica il rettangolo selezionato o in fase di selezione.
	
	bool isImage;
	int height;
	int width;
	
	public:         
		Mouse();
		int Init();
		// inizializza il mouse ritorna -1 se non e' presente il driver.
		void On();
		// abilita il mouse.
		void Off();
		// disabilita il mouse.
		void Show();
		// mostra il mouse.
		void Hide();
		// nasconde il mouse.
		Bool IsShowing();
		// ritorna TRUE se il cursore del mouse e' visibile.
		int Check();
		// controlla la posizione e lo stato dei bottoni.
		virtual void Draw();
		// disegna il cursore del mouse.
		Point GetPos();
		// ritorna la posizione del mouse l'ultima volta che e'stato controllato con Check.
		Point GetPrecPos();
		// ritorna la posizione del mouse precedente all'ultima volta che e'
		// stata controllata con Check.
		Bool IsMoved();
		// ritorna TRUE se il mouse e' stato mosso nell'ultima chiamata di Check.
		int GetButton();
		// ritorna lo stato dei bottoni l'ultima volta che sono stati controllati con Check.
		void SetRectSel();
		// imposta la selezione del rettangolo.
		void NoRectSel();
		// toglie la selezione del settangolo.
		void SetImgDragSel(int h,int w);
		void SetTxtDragSel();
		// imposta la selezione drag. 
		bool IsDragging(){return rectSelStatus==D_SEL;}
		bool IsDraggingImg(){return isImage;}
		int GetHeight(){return height;}
		int GetWidth(){return width;}
		class Rectangle GetRectSel();
		
};

#endif //MOUSE_HPP
