/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: musicedt.cpp,v 1.1 2005/07/25 16:30:32 bellinpo Exp $ */


#include <wx/wx.h>
#include "drw/musicedt.hpp"
#include "drw/scorefrm.hpp"
#include "drw/menufrm.hpp"
#include "smr/smrmain.hpp"
#include "smr/workspac.hpp"
#include "smr/smr.hpp"
#include "frmanage.hpp"
#include "prt/prtpara.hpp"

class LiooWindow;

extern wxDC *currDC;
extern wxDC *currMainDC;
extern wxMemoryDC *currMDC;
extern LiooWindow* pMusicWin;
extern ScrList LiooListWidget;

wxMusicEditor::wxMusicEditor()
{
	mainFrame=NULL;
	mainScore=NULL;
	smrf=NULL;
	lastViewID=-1;
	for(int i=0; i<MAX_VIEW;i++)
		scoreFrames[i]=NULL;
	
	extern int VerifKey();
}


Widget wxMusicEditor::MakeMainDrawArea(int w,int h,char* title)
{
	wxMEScoreFrame *newFrame=NULL;
	wxWindow* WedelFrame = SMRApp::GetApp().GetTopWindow();
	int x,y,W,H;
	SMRApp::GetApp().workspace->GetWindowConfig("MusicEditor", x, y, W, H);
	int viewID=GetNewViewID();
	newFrame = smrf->NewWxMEScoreFrame(WedelFrame,title,wxPoint(x,y),wxSize(W,H));
	newFrame->SetViewID(viewID);
	scoreFrames[viewID]=newFrame;
	DrawObject::SetCurrentView(viewID);
	lastViewID=viewID;
	
	newFrame->Show(TRUE);
	mainFrame=newFrame;
	FrameManager::currFrame=newFrame;
	return newFrame;
}


Widget wxMusicEditor::MakeMenuDrawArea(Menu* framemenu, int x, int y, int w,int h,char* title)
{
	wxMEMenuFrame *newFrame=NULL;

	newFrame = new wxMEMenuFrame(framemenu, NULL ,title,wxPoint(x,y),wxSize(w+8,h+5));
	newFrame->Show(TRUE);
	FrameManager::currFrame=newFrame;
	return newFrame;
}

void wxMusicEditor::CloseMainDrawArea(Widget frame)
{
	int i;
	bool closed=false;
	for(i=0; i<MAX_VIEW; i++)
	{
		if(scoreFrames[i]==frame)
		{
			scoreFrames[i]=NULL;
			closed=true;
			break;
		}
	}
	
	if(closed)
	{
		for(i=0; i<MAX_VIEW; i++)
			if(scoreFrames[i]!=NULL)
			{
				return;
			}
			CloseAllMenus();
	}
}

int wxMusicEditor::GetNewViewID()
{
	for(int i=0; i<MAX_VIEW; i++)
		if(scoreFrames[i]==NULL)
			return i;
		return -1;
}

bool wxMusicEditor::NewViewAvailable()
{
	for(int i=0; i<MAX_VIEW; i++)
		if(scoreFrames[i]==NULL)
			return true;
		return false;
}

void wxMusicEditor::SetCommand(CmdID cmdId, SelType stype, SelObj sobj, ArgType arg0,
							   ArgType arg1, ArgType arg2, ArgType arg3, ArgType arg4, 
							   char *txt, ArgType arg5, ArgType arg6, ArgType arg7,ArgType arg8)
{
	int oldSimbImg=SimbImg;
	for(int i=0;i<MAX_VIEW;i++)
	{
		if (scoreFrames[i]!=NULL)
		{
			SimbImg=scoreFrames[i]->GetSimbImg();
			scoreFrames[i]->GetMusicWin()->SetCommand(cmdId, stype, sobj, arg0, arg1, arg2, arg3, arg4, 
				txt, arg5, arg6, arg7, arg8);
		}
	}
	SimbImg=oldSimbImg;
};

void wxMusicEditor::ShowTagli()
{ 
	for(int i=0;i<MAX_VIEW;i++)
	{
		if (scoreFrames[i]!=NULL)
			scoreFrames[i]->GetMusicWin()->ShowTagli();
	}
}

void wxMusicEditor::HideTagli()
{ 
	for(int i=0;i<MAX_VIEW;i++)
	{
		if (scoreFrames[i]!=NULL)
			scoreFrames[i]->GetMusicWin()->HideTagli();
	}
}

void wxMusicEditor::RedrawViews(int modifiedScore,bool forceAll)
{ 
	int currView=DrawObject::GetCurrentView(); //the view modified
	for(int i=0;i<MAX_VIEW;i++)
	{
		if ((forceAll && scoreFrames[i]!=NULL) || (i!=currView && scoreFrames[i]!=NULL && 
			(forceAll || scoreFrames[i]->GetNScore()==modifiedScore || 
			scoreFrames[i]->GetNScore()==0)))
		{
			scoreFrames[i]->GetMusicWin()->ForceReposition();
			scoreFrames[i]->scorecanvas->Refresh(TRUE);
		}
	}
}

void wxMusicEditor::RedrawView(int currview)
{ 
}

void wxMusicEditor::CheckSymbolsMenu(int menuID,int value)
{ 
	for(int i=0;i<MAX_VIEW;i++)
	{
		if (scoreFrames[i]!=NULL)
			scoreFrames[i]->GetsymbolsMenu()->Check(menuID,(bool)value);
	}
}

void wxMusicEditor::Close()
{ 
	for(int i=0;i<MAX_VIEW;i++)
	{
		if (scoreFrames[i]!=NULL)
		{
			if(scoreFrames[i]!=SMRApp::GetApp().GetTopWindow())
			{
				scoreFrames[i]->Close();
				scoreFrames[i]=NULL;
			}
			else
			{
				scoreFrames[i]->GetMusicWin()->PPartit = NULL;
			}
		}
	}
	//CloseAllMenus()
	mainFrame=NULL;
	FrameManager::currFrame=NULL;
	pMusicWin=NULL;
	
}

void wxMusicEditor::CloseAllMenus()
{ 
	ScrObject *scrObj=LiooListWidget.GetFirst();
	while (scrObj)
	{
		Menu* menu = (Menu*)scrObj;
		CheckSymbolsMenu(menu->GetmenuItemID(), FALSE);
		menu->CloseMenu(); 
		scrObj=LiooListWidget.GetFirst();
	} 
}

wxMEScoreFrame *wxMusicEditor::GetLastView()
{
	if(lastViewID==-1)
		return NULL;
	return scoreFrames[lastViewID];
}

wxMEScoreFrame *wxMusicEditor::IsOpen(int currentscore,bool imgOrsymb)
{
	wxMEScoreFrame *tmp;
	for(int i=0;i<MAX_VIEW;i++)
	{
		if (scoreFrames[i]!=NULL)
		{
			tmp=scoreFrames[i];
			if((tmp->GetMusicWin()->PPartit->GetCurrentScore()==currentscore)&&(tmp->GetSimbImg()!=imgOrsymb))
				return tmp;
		}
	}
	return NULL;
	
}

wxString wxMusicEditor::GetWdfObjectPath()
{
	return(smrf->GetPath()+PATH_SEP);
}



wxMEScoreFrame *wxMusicEditor::GetCurrentViewFrame()
{
	return scoreFrames[DrawObject::GetCurrentView()];
}

bool wxMusicEditor::Load(wxString path)
{
	smrf->CloseMusicEditors();
	currDC=NULL;
	if(!smrf->Load(path))
		return false;
	smrf->GetScores(0,mainScore);

	for(int i=0;i<MAX_VIEW;i++)
	{
		if (scoreFrames[i]!=NULL)
		{
			wxMEScoreFrame *tmp=scoreFrames[i];
			tmp->GetMusicWin()->PPartit=mainScore;
		}
	}
	RedrawViews(0,true);
	return true;
}

bool wxMusicEditor::LoadMP4(wxString path)
{
	smrf->CloseMusicEditors();
	if(smrf->LoadFromMP4(path)==0)
	{
		smrf->GetScores(0,mainScore);
		
		for(int i=0;i<MAX_VIEW;i++)
		{
			if (scoreFrames[i]!=NULL)
			{
				wxMEScoreFrame *tmp=scoreFrames[i];
				tmp->GetMusicWin()->PPartit=mainScore;
			}
		}
		RedrawViews(0,true);
		return true;
	}
	return false;
}

void wxMusicEditor::New()
{
	smrf->CloseMusicEditors();
	smrf->New("");
	mainScore=smrf->GetMainScore();

	for(int i=0;i<MAX_VIEW;i++)
	{
		if (scoreFrames[i]!=NULL)
		{
			wxMEScoreFrame *tmp=scoreFrames[i];
			tmp->GetMusicWin()->PPartit=mainScore;
		}
	}
	RedrawViews(0,true);
}

void wxMusicEditor::SetMainScore(Partitura* _mainScore)
{
	smrf->CloseMusicEditors();
	mainScore=_mainScore;

	for(int i=0;i<MAX_VIEW;i++)
	{
		if (scoreFrames[i]!=NULL)
		{
			wxMEScoreFrame *tmp=scoreFrames[i];
			tmp->GetMusicWin()->PPartit=_mainScore;
		}
	}
	RedrawViews(0,true);
}