/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: rectangl.cpp,v 1.1 2005/07/25 16:30:32 bellinpo Exp $ */



#include "lioo.hpp"
#include "drw/rectangl.hpp"

Rectangle::Rectangle()
{
}

Rectangle::Rectangle(const Point& p1, const Point& p2)
{
	PUpLft=p1;
	PDwnRgt=p2;
	Adjust();
}

Rectangle::Rectangle(const Point& p,VUnit lft,VUnit rgt,VUnit up,VUnit dwn)
{
	PUpLft=p+Point(-lft,up);
	PDwnRgt=p+Point(rgt,-dwn);
	Adjust();
}

void Rectangle::Adjust()
{
	VUnit tmp;
	
	if(PUpLft.x>PDwnRgt.x)
	{
		tmp=PUpLft.x;
		PUpLft.x=PDwnRgt.x;
		PDwnRgt.x=tmp;
	}
	if(PUpLft.y<PDwnRgt.y)
	{
		tmp=PUpLft.y;
		PUpLft.y=PDwnRgt.y;
		PDwnRgt.y=tmp;
	}
}

Bool Rectangle::hasIn(const Point& p)
{ 
	   
	return p.x >= PUpLft.x  && p.x <= PDwnRgt.x &&
		p.y >= PDwnRgt.y && p.y <= PUpLft.y ; 
	
}

Bool Rectangle::isIn(const Rectangle& r)
{ 
		  return (  PUpLft.x  >= r.PUpLft.x &&   PUpLft.y  <= r.PUpLft.y &&
			  PDwnRgt.x <= r.PDwnRgt.x &&  PDwnRgt.y >= r.PDwnRgt.y ); 
}

Rectangle Rectangle::Expand(VUnit x,VUnit y) const
{ 
	return Rectangle(PUpLft+Point(-x,y),PDwnRgt+Point(x,-y)); 
}

Rectangle Rectangle::Expand(VUnit lft,VUnit rgt,VUnit up,VUnit dwn) const
{ 
	return Rectangle(PUpLft+Point(-lft,up),PDwnRgt+Point(rgt,-dwn)); 
}

Point Rectangle::Center() const 
{ 
			 return Point(VUnit((PUpLft.x+PDwnRgt.x)/2),VUnit((PUpLft.y+PDwnRgt.y)/2)); 
}

VUnit Rectangle::Width() const 
{ 
	return PDwnRgt.x-PUpLft.x+VUnit(1); 
}

VUnit Rectangle::Height() const 
{ 
	return -PDwnRgt.y+PUpLft.y+VUnit(1);  
}
