/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: scorecan.cpp,v 1.2 2005/07/28 09:52:27 bellinpo Exp $ */


#include <wx/clipbrd.h>
#include "drw/scorecan.hpp"
#include "drw/gscreen.hpp"
#include "drw/scrlist.hpp"
#include "drw/musicedt.hpp"
#include "drw/scorefrm.hpp"
#include "gui/masaewin.hpp"
#include "gui/dlioowin.hpp"
#include "drw/frmanage.hpp"
#include "smr/smr.hpp" 
#include "imp/midiseq.h"

extern int DebugNormalMillaPos;
extern GScreen *Scr;
extern ScrList LiooList,LiooListWidget;
extern Widget LiooWidget;
extern LiooWindow *pMusicWin;

extern wxDC *currDC;
extern wxDC *currMainDC;
extern wxMemoryDC *currMDC;


IMPLEMENT_DYNAMIC_CLASS(wxMEScoreCanvas, wxScrolledWindow)

BEGIN_EVENT_TABLE(wxMEScoreCanvas, wxScrolledWindow)
	EVT_SIZE(wxMEScoreCanvas::OnSize)
	EVT_PAINT(wxMEScoreCanvas::OnPaint)
	EVT_LEFT_DOWN(wxMEScoreCanvas::OnMouse)
	EVT_RIGHT_DOWN(wxMEScoreCanvas::OnMouse)
	EVT_LEFT_UP(wxMEScoreCanvas::OnMouse)
	EVT_RIGHT_UP(wxMEScoreCanvas::OnMouse)
	EVT_MOTION(wxMEScoreCanvas::OnMouse)  
	EVT_KEY_DOWN(wxMEScoreCanvas::OnKeyDown)  
	EVT_SCROLLWIN(wxMEScoreCanvas::OnScroll) 
#ifdef WX226 
 	EVT_CUSTOM(EVT_MIDI, ID_NOTEON, wxMEScoreCanvas::On_MIDI_NOTEON)
	EVT_CUSTOM(EVT_MIDI, ID_NOTEOFF, wxMEScoreCanvas::On_MIDI_NOTEOFF)
	EVT_CUSTOM(EVT_MIDI, ID_STOP, wxMEScoreCanvas::On_MIDI_STOP)
#endif
END_EVENT_TABLE()

void wxMEScoreCanvas::OnScroll(wxScrollWinEvent& event)
{
	if(!GetMusicWin()->GetModEsecuzione())
		event.Skip();
}

wxMEScoreCanvas::wxMEScoreCanvas()
{
	SetBackgroundColour(* wxWHITE);
	viewID=-1;
	autoLineBreak=true;   //false;   // per debug  true;
	debugNormalMillaPos=0;
	currentViewType=0; // COMPUTERVIEW
	viewBars=false; // Barlines Non Visibili;
	musicWin=NULL;
	nScore=0;
	simbImg=true;
}

wxMEScoreCanvas::wxMEScoreCanvas( wxWindow *parent, const wxPoint &pos, const wxSize &size )
: wxScrolledWindow( parent, wxMEScoreCanvas_ID, pos, size, wxSUNKEN_BORDER )
{
	SetBackgroundColour(* wxWHITE);
	viewID=-1;
	autoLineBreak=true;   //false;   // per debug  true;
	debugNormalMillaPos=0;
	currentViewType=0; // COMPUTERVIEW
	viewBars=false; // Barlines Non Visibili;
	musicWin=NULL;
	nScore=0;
	simbImg=true;
}

wxMEScoreCanvas::~wxMEScoreCanvas()
{
	if(GetMusicWin()!=NULL)
		LiooList.Delete(GetMusicWin());
}

void wxMEScoreCanvas::Prepare(wxDC* dc)
{
	if(dc!=NULL)
	{
		currDC=dc;
		currMainDC=dc;
	}
	// reimposta le variabili globali per multiview
	DrawObject::SetCurrentView(GetViewID());
	pMusicWin=GetMusicWin();
	if(pMusicWin!=NULL)
	{
		if(pMusicWin->PPartit!=NULL)
			pMusicWin->PPartit->SetCurrentScore(GetNScore());
		LiooWidget=GetParent();
		MusicEditor->mainFrame=(wxFrame*)GetParent();
		if(GetNScore()==0)
		{
			tipoLeggio=LEGGIO_MASAE;
			Leg=false;
		}
		else
		{
			tipoLeggio=LEGGIO_DLIOO;
			Leg=true;
		}
		SimbImg=simbImg;
		AutoLineBreak=autoLineBreak;
		DebugNormalMillaPos=debugNormalMillaPos;
		currentViewType=currentViewType;
	}
}

void wxMEScoreCanvas::OnPaint( wxPaintEvent &event )
{
	wxPaintDC dc(this);

	OnDraw(dc);
}


void wxMEScoreCanvas::OnDraw(wxDC& dc)
{
	if(GetMusicWin()==NULL)
		return;
	
	wxDC *prevDC=currDC;
	wxDC *prevMainDC=currMainDC;
	LiooWindow *prevMusicWin=pMusicWin;
	
	currDC=&dc;
	currMainDC=NULL;
	
	Prepare();
	PrepareDC(dc);
	
	
	Scr->GSetMainDrawArea();
	Scr->SetClipRegion(class Rectangle(Point(0,Scr->GetMaxPixY()),Point(Scr->GetMaxPixX(),0)));
	
	Scr->Clear();
	
	pMusicWin->Resize(VUnit(Scr->GetMaxPixX()),VUnit(Scr->GetMaxPixY()));
	
	// Disegna la cornice e l'interno di pMusicWin 
	// campiona AutoLineBreak
	int flag=AutoLineBreak;
	
	pMusicWin->Draw();
	
	SetAutoLineBreak(AutoLineBreak);
	
	
	wxMEScoreFrame* scoreFrame=dynamic_cast<wxMEScoreFrame*>(GetParent());
	if(scoreFrame!=NULL)
	{
		if (AutoLineBreak)
		{
			scoreFrame->GetformatMenu()->Check(FORM_LINEBREAKING_ID, TRUE);
		}
		else
		{
			scoreFrame->GetformatMenu()->Check(FORM_LINEBREAKING_ID, FALSE);
		}
	}
	
	
	if (2==DebugNormalMillaPos)
	{
		// ho avuto transizione tra gli stati del milla
		// da posizionamento a inserimento
		SetDebugNormalMillaPos(0);
		DebugNormalMillaPos=FALSE;
		//DebugNormalMilla=TRUE l'ho gia' messo nel metodo chiamante
		// es. Partitura::InsertSel
		// deticco i menu
		if(scoreFrame!=NULL)
		{
			scoreFrame->GetformatMenu()->Check(FORM_DEBUGMILLAINSERTION_ID, TRUE);
			scoreFrame->GetformatMenu()->Check(FORM_DEBUGMILLAPOSITIONING_ID, FALSE);
		}	  
	}
	
	if (flag!=AutoLineBreak)
	{
		pMusicWin->ForceReposition();
		pMusicWin->Draw(); 
	}
	currDC=prevDC;
	currMainDC=prevMainDC;
	pMusicWin=prevMusicWin;
}

void wxMEScoreCanvas::OnMouse(wxMouseEvent &event)
{
	if(GetMusicWin()==NULL)
		return;
	
	long x,y;
	EventInfo evInf;
	int action=0; 
	wxDC *prevDC=currDC;
	wxDC *prevMainDC=currMainDC;
	LiooWindow *prevMusicWin=pMusicWin;
	
	wxClientDC dc(this);
	Prepare(&dc);

	currDC=&dc;
	currMainDC=&dc;
	PrepareDC(dc);

	wxPoint point = event.GetLogicalPosition(dc);
	x=point.x / Scr->GetMagnification();
	y=point.y / Scr->GetMagnification();
	
	Scr->SetMousePoint(Point(x, y));
	
	if(event.LeftDown())
		action = LEFTGODOWN;
	else if(event.MiddleDown())
		action = CENTRALGODOWN;
	else if(event.RightDown())
		action = RIGHTGODOWN;
	else if(event.LeftUp())
		action = LEFTGOUP;
	else if(event.MiddleUp())
		action = CENTRALGOUP;
	else if(event.RightUp())
		action = RIGHTGOUP;
	
	
	if(action!=0)
	{
		Scr->SetMouseAction(action);
		Scr->GSetDrawArea(GetParent());
	}
	
	
	
    //  segnala l'evento
	Scr->GSetDrawArea(GetParent());
	evInf=LiooList.SignalEvent();
	
	// se il mouse viene mosso notifica cio' allo spartito per il disegno
	// dei tagli
	if(evInf.mouseMoved)
	{
		musicWin->MouseMove(Scr->GetMouse()->GetPrecPos(),evInf.mousePos);
	}
	
	if (musicWin->GetModEsecuzione()) 
		Scr->GSetMainDrawArea();
	if(evInf.eventResult==ER_END) 
	{
		// La deallocazione e' curata dal distruttore di GScreen che viene
		// richiamato automaticamente quando si esce dal programma.
		// Funziona anche se si distrugge la finestra col Quit di XWindow!
		Scr->GCloseWindow(GetParent());
	}
	
	//milla debug
	SetDebugNormalMillaPos(DebugNormalMillaPos);
	
	currDC=prevDC;
	currMainDC=prevMainDC;
	pMusicWin=prevMusicWin;
	
	if(event.LeftDown())
	{
		wxActivateEvent evt;
		((wxMEScoreFrame*)GetParent())->OnActivate(evt);
	}
	
}

#define NORMAL 0
#define SHIFT 1
#define ALT 2
#define CTRL 3

extern EventResult pausaOp();
extern EventResult AccOp();
extern EventResult DecOp();

void wxMEScoreCanvas::OnKeyDown(wxKeyEvent &event)
{
	if(GetMusicWin()==NULL)
		return;
	
	EventInfo evInf;
	wxDC *prevDC=currDC;
	wxDC *prevMainDC=currMainDC;
    
	Prepare();
	
	wxClientDC dc(this);
	wxClientDC maindc(this);
	
	currDC=&dc;
	currMainDC=&maindc;
	
	switch (event.KeyCode())
	{
	case WXK_RIGHT:
		AccOp();
		break;
	case WXK_LEFT:
		DecOp();
		break;
	case WXK_UP:	 
		pausaOp();
		break;	  
	default: ;
	}
	
	Scr->SetTmpKey(event.KeyCode());
	evInf=LiooList.SignalEvent();
    
	if (musicWin->GetModEsecuzione()) Scr->GSetMainDrawArea();
	
	currDC=prevDC;
	currMainDC=prevMainDC;
}

void wxMEScoreCanvas::OnSize(wxSizeEvent& event)
{
	if(GetPrintStat()==0 && (GetNScore()!=0 || !GetSimbImg()))
	{
		SetScrollbars(event.GetSize().GetWidth(),event.GetSize().GetHeight(),1,1,0,0,TRUE);
	}
}

void wxMEScoreCanvas::PutSubCanvasInClipboard(int x, int y, int width, int height)
{
	wxString msg;
	wxBitmap image(width, height);
	{
		wxMemoryDC temp_dc;
		temp_dc.SelectObject(image);
		
		// We can now draw into the memory DC...
		// Copy from this DC to another DC.
		wxClientDC currDC(this);
		temp_dc.Blit(0, 0, width, height, &currDC, x, y);
	}
	
	
	if (wxTheClipboard->Open())
	{
		
		// This data objects are held by the clipboard, 
		// so do not delete them in the app.
		wxTheClipboard->SetData( new wxBitmapDataObject(image) );
		wxTheClipboard->Close();
		
		wxMessageBox("The sub-image selected is in the clipboard");
	}
}


void wxMEScoreCanvas::GoTop()
{
	extern EventResult GoTop();
	wxClientDC maindc(this);
	currMainDC=&maindc;
	Prepare(); 
	PrepareDC(maindc);
	Scr->GSetMainDrawArea();
	
	::GoTop();
}

void wxMEScoreCanvas::GoForward()
{
	extern EventResult GoForward();
	wxClientDC maindc(this);
	currMainDC=&maindc;
	
	Prepare(); 
	PrepareDC(maindc);
	Scr->GSetMainDrawArea();
	
	::GoForward();
}

void wxMEScoreCanvas::GoForward5()
{
	extern EventResult NetPiu5();
	wxClientDC maindc(this);
	currMainDC=&maindc;
	
	Prepare(); 
	PrepareDC(maindc);
	Scr->GSetMainDrawArea();
	
	::NetPiu5();
}

void wxMEScoreCanvas::GoBackward()
{
	extern EventResult GoBackward();
	wxClientDC maindc(this);
	currMainDC=&maindc;
	
	Prepare(); 
	PrepareDC(maindc);
	Scr->GSetMainDrawArea();
	
	::GoBackward();
}

void wxMEScoreCanvas::GoBackward5()
{
	extern EventResult NetMeno5();
	wxClientDC maindc(this);
	currMainDC=&maindc;
	
	Prepare(); 
	PrepareDC(maindc);
	Scr->GSetMainDrawArea();
	
	::NetMeno5();
}

void wxMEScoreCanvas::GoBottom()
{
	extern EventResult GoBottom();
	wxClientDC maindc(this);
	currMainDC=&maindc;
	
	Prepare(); 
	PrepareDC(maindc);
	Scr->GSetMainDrawArea();
	
	::GoBottom();
}

void wxMEScoreCanvas::Jump(int nMeasure)
{
	if(GetMusicWin()==NULL)
		return;

	wxClientDC maindc(this);
	currMainDC=&maindc;
	
	Prepare(); 
	PrepareDC(maindc);
	Scr->GSetMainDrawArea();
	
	GetMusicWin()->PPartit->GoBatt(nMeasure);
}

int wxMEScoreCanvas::GetNMeasures()
{
	if(GetMusicWin()==NULL)
		return 0;
	return GetMusicWin()->PPartit->GetNumeroBattute();
}

bool wxMEScoreCanvas::SetMusicScore(SMRFile* smrf, bool symImg, int nscore, int viewid)
{
	Partitura* mainScore;
	
	smrf->GetScores(nscore,mainScore);
	
	Widget scoreFrame=NULL;
	
	MusicEditor->mainScore=mainScore;
	MusicEditor->nScore=nscore;
	MusicEditor->smrf=smrf;
	
	MusicEditor->mainFrame=dynamic_cast<wxFrame*>(GetParent());
	
	// Inizializzazione finestra partitura
	if (nscore==0)
	{
		pMusicWin = new MasaeWindow;
		pMusicWin->SetTitle((unsigned char *)"Main Score");
	}
	else
	{
		pMusicWin = new DliooWindow;
		pMusicWin->SetTitle((unsigned char *)"Part");
	}
	
	if(GetMusicWin()!=NULL)
		LiooList.Delete(GetMusicWin());
	
	SetMusicWin(pMusicWin);
	SetNScore(nscore);
	SetSimbImg(symImg);
	SetViewID(viewid);
	SetPrintStat(false);
	SetAutoLineBreak(true);
	SetDebugNormalMillaPos(false);
	SetViewBarlines(false);
	
	wxWindow *win = this;
	do
	{
		win=win->GetParent();
		FrameManager::currFrame=dynamic_cast<wxFrame*>(win);
	}
	while(FrameManager::currFrame==NULL);
	
	Prepare();
	
	Scr->GNewDC((Widget)GetParent());
	Colore oldcol = Scr->GetColor();
	Scr->SetColor((ColorIcon)white);            
	Scr->Rect(class Rectangle(Point(0, Scr->GetMaxPixY()),
		Point(Scr->GetMaxPixX(), 0)), TRUE);
	Scr->SetColor(oldcol);

	pMusicWin->SetInView(viewid);
	
	pMusicWin->WinInit(Point(0,0),      
		VUnit(Scr->GetMaxPixX()-WIN_SPARTITO_DELTA_X),
		VUnit(Scr->GetMaxPixY()-WIN_SPARTITO_DELTA_Y),
		mainScore);
	
	DimensioneX=Scr->GetMaxPixX()-10;
	DimensioneY=Scr->GetMaxPixY()-31-10; 
	
	
	if(mainScore!=NULL && symImg)
	{
		mainScore->SetCurrentScore(nscore);
	}
	
	// Impostazione del cursore a croce
	Scr->GSetCursor(DEFAULT);
	
	LiooList.Add(pMusicWin);
	
	pMusicWin->GoTop();
	pMusicWin->Resize(VUnit(Scr->GetMaxPixX()-WIN_SPARTITO_DELTA_X+1),
		VUnit(Scr->GetMaxPixY()-WIN_SPARTITO_DELTA_Y));
	
	Scr->GDeleteDC();
	
	pMusicWin->ForceReposition();
	Refresh(TRUE);
	
	ContrScrool=true;    
	
	return true;
}

bool wxMEScoreCanvas::ImportMIDI(const wxString& midiFile)
{
	wxClientDC maindc(this);
	currMainDC=&maindc;
	
	Prepare(); 
	PrepareDC(maindc);
	Scr->GSetMainDrawArea();
	
	bool ret=GetMusicWin()->PPartit->ImportMidi((char*)midiFile.c_str(),0);

	pMusicWin->ForceReposition();
	Refresh(TRUE);
	return ret;
}

void wxMEScoreCanvas::Transpose(int daBat, int aBat,int partnumber,int clef,int translation,int interval,int up, int numofstaff, Bool sharps, Bool adjust)
{
	wxClientDC maindc(this);
	currMainDC=&maindc;
	
	Prepare(); 
	PrepareDC(maindc);
	Scr->GSetMainDrawArea();
	
	GetMusicWin()->PPartit->GetListaSpa()->Transpose(daBat, aBat, partnumber, clef, translation, interval, up, numofstaff, sharps, adjust);

	pMusicWin->ForceReposition();
	Refresh(TRUE);
}

bool wxMEScoreCanvas::InitMIDIExecution()
{
	if(!simbImg || nScore!=0) //only symbolic main score can be played
		return false;

	wxClientDC maindc(this);
	wxDC *prevDC=currDC;
	wxDC *prevMainDC=currMainDC;
	LiooWindow* prevMusicWin=pMusicWin;
		
	Prepare(); 
	Scr->SetMainDC(&maindc);

	extern EventResult InitOp();
	extern wxString ettFile;
	extern wxMEScoreCanvas* ScoreInExecution;

	ScoreInExecution=this;

	Scr->InitPixmaps();

	ettFile="Midi generation";

	InitOp();

	currDC=prevDC;
	currMainDC=prevMainDC;
	pMusicWin=prevMusicWin;
	return true;
}

bool wxMEScoreCanvas::InitExecution(SMRSynchInfoItem* exec) //NULL = ask
{
	if(nScore!=0) //only main score can be played
		return false;

	wxClientDC maindc(this);
	wxDC *prevDC=currDC;
	wxDC *prevMainDC=currMainDC;
	LiooWindow* prevMusicWin=pMusicWin;
		
	Prepare(); 
	Scr->SetMainDC(&maindc);

	extern EventResult InitOp();
	extern wxString ettFile;
	extern wxMEScoreCanvas* ScoreInExecution;

	ScoreInExecution=this;

	Scr->InitPixmaps();

	if(exec!=NULL)
		ettFile=exec->GetFilename();
	else
		ettFile="";

	InitOp();

	currDC=prevDC;
	currMainDC=prevMainDC;
	pMusicWin=prevMusicWin;
	return true;
}

bool wxMEScoreCanvas::Start()
{
	wxClientDC maindc(this);
	wxDC *prevDC=currDC;
	wxDC *prevMainDC=currMainDC;
	LiooWindow* prevMusicWin=pMusicWin;
	
	Scr->SetMainDC(&maindc);
	Prepare(); 
	
	extern EventResult GoOp();
	GoOp();
	
	currDC=prevDC;
	currMainDC=prevMainDC;
	pMusicWin=prevMusicWin;

	return true;
}

void wxMEScoreCanvas::Stop()
{
	wxClientDC maindc(this);
	wxDC *prevDC=currDC;
	wxDC *prevMainDC=currMainDC;
	LiooWindow* prevMusicWin=pMusicWin;
	
	Scr->SetMainDC(&maindc);
	Prepare(); 
	
	extern EventResult StopOp();
	StopOp();
	
	currDC=prevDC;
	currMainDC=prevMainDC;
	pMusicWin=prevMusicWin;
}

void wxMEScoreCanvas::Pause()
{
	wxClientDC maindc(this);
	wxDC *prevDC=currDC;
	wxDC *prevMainDC=currMainDC;
	LiooWindow* prevMusicWin=pMusicWin;
	
	Scr->SetMainDC(&maindc);
	Prepare(); 
	
	extern EventResult pauseOp();
	pausaOp();
	
	currDC=prevDC;
	currMainDC=prevMainDC;
	pMusicWin=prevMusicWin;
}

void wxMEScoreCanvas::On_MIDI_NOTEON(wxCommandEvent& evt)
{
	Nota* p_note=(Nota*)(evt.GetInt());	
		wxClientDC maindc(this);
		PrepareDC(maindc);
		currMainDC=&maindc;
		Scr->GSetMainDrawArea();
		p_note->SetColor((ColorIcon)red);
		note_on_buffer.Add((int)p_note);
		p_note->Draw();
		Switch_on_tied_notes(p_note);
}

void wxMEScoreCanvas::Switch_on_tied_notes(Nota* p_note)
{
	IntRef* p_struct=NULL;
	IntEsteso* p_int=NULL;
	IntEsteso* p_aux_int=NULL;
	BYTE num_refs=0;
	do
	{
		num_refs=p_note->GetNIntRefs();
    	if (!num_refs) return;
		p_aux_int=NULL;
		p_int=NULL;
		for(BYTE i=0;i<num_refs;i++)
		{
			p_struct=p_note->GetIntRef(i);
			p_aux_int=p_struct->intRef;
			if ((p_aux_int->GetID()==CL_LEGATURA_VALORE)&&
				(p_struct->type==INT_START))
			{
				p_note=(Nota*)(p_aux_int->GetPFigEnd());
				note_on_buffer.Add((int)p_note);
				p_note->SetColor((ColorIcon)red);
				p_note->Draw();
				p_int=p_aux_int;
			}
		}
	}
	while(p_int);
}

void wxMEScoreCanvas::On_MIDI_NOTEOFF(wxCommandEvent& evt)
{
	Nota* p_note=(Nota*)(evt.GetInt());

		wxClientDC maindc(this);
		PrepareDC(maindc);
		currMainDC=&maindc;
		Scr->GSetMainDrawArea();
		p_note->SetColor((ColorIcon)black);
		if (note_on_buffer.Index((int)p_note)!=wxNOT_FOUND)
			note_on_buffer.Remove((int)p_note);
		p_note->Draw();
		Switch_off_tied_notes(p_note);
}

void wxMEScoreCanvas::Switch_off_tied_notes(Nota* p_note)
{
	IntRef* p_struct=NULL;
	IntEsteso* p_int=NULL;
	IntEsteso* p_aux_int=NULL;
	BYTE num_refs=0;
	do
	{
		num_refs=p_note->GetNIntRefs();
    	if (!num_refs) return;
		p_aux_int=NULL;
		p_int=NULL;
		for(BYTE i=0;i<num_refs;i++)
		{
			p_struct=p_note->GetIntRef(i);
			if (p_struct) 
			{
				p_aux_int=p_struct->intRef;
				if ((p_aux_int->GetID()==CL_LEGATURA_VALORE)&&
					(p_struct->type==INT_END))
				{
					p_note=(Nota*)(p_aux_int->GetPFigStart());
					if (note_on_buffer.Index((int)p_note)!=wxNOT_FOUND)
						note_on_buffer.Remove((int)p_note);
					p_note->SetColor((ColorIcon)black);
					p_note->Draw();
					p_int=p_aux_int;
				}
			}
		}
	}
	while(p_int);
}

void wxMEScoreCanvas::On_MIDI_STOP(wxCommandEvent& evt)
{
	if (note_on_buffer.IsEmpty()) return;
	wxClientDC maindc(this);
	PrepareDC(maindc);
	currMainDC=&maindc;
	Scr->GSetMainDrawArea();
	Nota* p_note=NULL;
	DWORD note_on=note_on_buffer.GetCount();
	for (DWORD i=0;i<note_on;i++)
	{
	p_note=(Nota*)note_on_buffer[0];
	if (p_note!=NULL)
		p_note->SetColor((ColorIcon)black);
	note_on_buffer.RemoveAt(0);
	
	}
}

wxArrayString wxMEScoreCanvas::GetSelectionIds()
{
	if(GetMusicWin())
		return GetMusicWin()->PPartit->GetSelectionIds();
	return wxArrayString();
}

void wxMEScoreCanvas::Select(wxString id, bool select)
{
	if(GetMusicWin())
	{
		GetMusicWin()->PPartit->Select(id, select);
		Refresh();
	}
}

SMRAnnotation wxMEScoreCanvas::GetSelectionAnnotation(wxString id)
{
	if(GetMusicWin())
		return GetMusicWin()->PPartit->GetSelectionAnnotation(id);
	return SMRAnnotation();
}
