/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: scorecan.hpp,v 1.2 2005/07/28 09:52:27 bellinpo Exp $ */


#ifndef SCORECAN
#define SCORECAN

#include <wx/wx.h>
#include "lioo.hpp"

class LiooWindow;
class SMRFile;
class SMRSynchInfoItem;
class Nota;
class SMRAnnotation;

#define wxMEScoreCanvas_ID 100

class wxMEScoreCanvas: public wxScrolledWindow
{
protected:
	int viewID;
	LiooWindow *musicWin;
	int nScore;
    bool simbImg;
	bool autoLineBreak;
	int debugNormalMillaPos;
	int currentViewType;
	bool viewBars;
	
public:
	wxMEScoreCanvas();
    wxMEScoreCanvas( wxWindow *parent, const wxPoint &pos, const wxSize &size );
    ~wxMEScoreCanvas();
	
	//Events
    void OnPaint( wxPaintEvent &event );
	void OnDraw(wxDC& dc);
	void OnMouse(wxMouseEvent &event);
	void OnKeyDown(wxKeyEvent &event);
	void OnScroll(wxScrollWinEvent& event);
	void OnSize(wxSizeEvent& event);
	
	//sets the score to be viewed
	bool SetMusicScore(SMRFile*, bool symImg=true, int nScore=0, int viewID=0);

	int GetViewID() { return viewID; };
	void SetViewID(int id) { viewID=id; }
	LiooWindow* GetMusicWin() { return musicWin; };
	void SetMusicWin(LiooWindow* win) { musicWin=win; };
	int GetNScore() { return nScore; };
	void SetNScore(int ns) { nScore=ns; }
	bool GetSimbImg() { return simbImg; };
	void SetSimbImg(bool si) { simbImg=si; };
	bool GetAutoLineBreak() { return autoLineBreak; };
	void SetAutoLineBreak(bool lb) { autoLineBreak=lb; };
    int GetDebugNormalMillaPos() { return debugNormalMillaPos; };
	void SetDebugNormalMillaPos(int lb) { debugNormalMillaPos=lb; };
    int GetPrintStat() { return currentViewType; };
	void SetPrintStat(int ps) { currentViewType=ps; };
	void SetViewBarlines(bool view) {viewBars=view;};
	bool GetViewBarlines() {return viewBars;};
	
	//setups the global variables
	void Prepare(wxDC* dc=NULL);
	
	void PutSubCanvasInClipboard(int x, int y, int width, int height);
	
	//Movement
	void GoTop();
	void GoForward();
	void GoForward5();
	void GoBackward();
	void GoBackward5();
	void GoBottom();
	void Jump(int nMeasure);
	

	bool ImportMIDI(const wxString& midiFile);
	void Transpose(int daBat, int aBat,int partnumber,int clef,int translation,int interval,int up, int numofstaff, Bool sharps, Bool adjust);


	int GetNMeasures();

	//Execution
	bool InitMIDIExecution();
	bool InitExecution(SMRSynchInfoItem* synch=NULL); //NULL = ask
	bool Start();
	void Stop();
	void Pause();
	
	void On_MIDI_NOTEON(wxCommandEvent& evt);
	void On_MIDI_NOTEOFF(wxCommandEvent& evt);
	void Switch_on_tied_notes(Nota* p_note);
	void Switch_off_tied_notes(Nota* p_note);
	void On_MIDI_STOP(wxCommandEvent& evt);
	wxArrayInt note_on_buffer;
	wxArrayInt& Get_note_on_buffer()
	{return note_on_buffer;};

	wxArrayString GetSelectionIds();
	SMRAnnotation GetSelectionAnnotation(wxString selectionId);
	void Select(wxString selectionId, bool select=true);
	
    DECLARE_DYNAMIC_CLASS(wxMEScoreCanvas)
		DECLARE_EVENT_TABLE()
};

#endif //SCORECAN
