/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: scorefrm.cpp,v 1.2 2005/07/28 09:52:27 bellinpo Exp $ */


#include <wx/wx.h>
#include <wx/laywin.h>
#include <wx/wxexpr.h>

#include "drw/scorefrm.hpp"
#include "gui/lioowin.hpp"
#include "drw/menu.hpp"
#include "smr/smrmain.hpp"
#include "smr/workspac.hpp"
#include "frmanage.hpp"
#include "liooext.hpp"
#include "drw/musicedt.hpp"
#include "gui/dlg/fonttabledlg.hpp"
#include "gui/dlg/prinpdlg.hpp"
#include "smr/smr.hpp"
#include "smr/dlg/settings.hpp"
#include "fig/nota.hpp"
#include "bat/battuta.hpp"
#include "imp/midiseq.h"
#include "lyr/lyricframe.hpp"
#include "mil/smfl.hpp"

extern Widget LiooWidget;
extern wxMusicEditor *MusicEditor;
extern wxDC *currDC;
extern wxDC *currMainDC;
extern wxMemoryDC *currMDC;

extern LiooWindow *pMusicWin;
extern ScrList LiooListWidget;

extern void InitLoad(SMRFile*);
extern int yyparse(void);

extern int AutoLineBreak;

int currentViewType=0;
// 0 computer view 1 print view
bool visible=false;
//variabile globale per rendere visibili pause e note in stato ghosted e hidden
extern int DebugNormalMilla;
extern int DebugNormalMillaPos;

extern SMRFile *smrf;

extern void CreateNotaMenu();
extern void CreateSmallNotesMenu();
extern void CreateBeamingMenu();
extern void CreateRestsMenu();
extern void CreateAccidentalsMenu();

extern void CreateAugmentationsMenu();
extern void CreateRepeatsMenu();
extern void CreateBarlinesMenu();
extern void CreateClefsMenu();
extern void CreateBracketsMenu();
extern void CreateHorizontalsMenu();
extern void CreateRefrainsMenu();
extern void CreateExpressionsMenu();
extern void CreateGenericExpMenu();
extern void CreateKoreanOrnamentsMenu();
extern void CreateOrnamentsMenu();
extern void CreateFingeringMenu();
extern void CreateMutesMenu();
extern void CreateHarmonicsMenu();
extern void CreateBreathGlassMenu();
extern void CreateStringsMenu();
extern void CreatePedalsMenu();
extern void CreateViolinMenu();

wxMEScoreCanvas *ScoreInExecution=NULL;

BEGIN_EVENT_TABLE(wxMEScoreFrame, wxFrame)
EVT_SIZE(wxMEScoreFrame::OnSize)

EVT_MENU(FILE_IMPORTMIDI_ID, wxMEScoreFrame::OnImportMIDI)
EVT_MENU(FILE_IMPORTOMR_ID, wxMEScoreFrame::OnImportOMR)
EVT_MENU(FILE_IMPORTSCORE_ID, wxMEScoreFrame::OnImportSCORE)
EVT_MENU(FILE_IMPORTMOODS_ID, wxMEScoreFrame::OnImportMOODS)
EVT_MENU(FILE_EXPORTMIDI_ID, wxMEScoreFrame::OnExportMIDI)
EVT_MENU(FILE_EXPORTWDF_ID, wxMEScoreFrame::OnExportWDF)
EVT_MENU(FILE_EXPORTIWF_ID, wxMEScoreFrame::OnExportIWF)
EVT_MENU(FILE_EXPORTMOODS_ID, wxMEScoreFrame::OnExportMOODS)
EVT_MENU(FILE_CHANGESLIST_ID, wxMEScoreFrame::OnChangesList)
EVT_MENU(FILE_LOADCHANGES_ID, wxMEScoreFrame::OnLoadChanges)
EVT_MENU(FILE_SAVECHANGES_ID, wxMEScoreFrame::OnSaveChanges)
EVT_MENU(FILE_SAVEASCHANGES_ID, wxMEScoreFrame::OnSaveAsChanges)
EVT_MENU(VIEW_COMPUTERVIEWPARAMETERS_ID, wxMEScoreFrame::OnCompViewPar)
EVT_MENU(FILE_PRINTPARAMETERS_ID, wxMEScoreFrame::OnPrintParams)
EVT_MENU(FILE_CLOSE_ID, wxMEScoreFrame::OnClose)

EVT_MENU(VIEW_COMPUTERVIEW_ID, wxMEScoreFrame::OnComputerView)
EVT_MENU(VIEW_PRINTVIEW_ID, wxMEScoreFrame::OnPrintView)
EVT_MENU(EDIT_FIRSTPAGEHEADER_ID, wxMEScoreFrame::OnFirstPageHeader)
EVT_MENU(EDIT_IMPORTAPART_ID, wxMEScoreFrame::OnImportAPart)
EVT_MENU(EDIT_NEWPART_ID, wxMEScoreFrame::OnNewPart)
EVT_MENU(VIEW_HIDEAPART_ID, wxMEScoreFrame::OnHideAPart)
EVT_MENU(VIEW_SHOWALLPARTS_ID, wxMEScoreFrame::OnShowAllParts)
EVT_MENU(EDIT_CUTAPART_ID, wxMEScoreFrame::OnCutAPart)
EVT_MENU(EDIT_COPYAPART_ID, wxMEScoreFrame::OnCopyAPart)	
EVT_MENU(EDIT_PASTEAPART_ID, wxMEScoreFrame::OnPasteAPart)
EVT_MENU(EDIT_CUTPAGEBOX_ID, wxMEScoreFrame::OnCutPageBox)
EVT_MENU(EDIT_COPYPAGEBOX_ID, wxMEScoreFrame::OnCopyPageBox)  
EVT_MENU(EDIT_PASTEPAGEBOX_ID, wxMEScoreFrame::OnPastePageBox)
EVT_MENU(EDIT_SPLITTINGPART_ID, wxMEScoreFrame::OnSplittingPart)
EVT_MENU(EDIT_MERGINGPARTS_ID, wxMEScoreFrame::OnMergingParts)
EVT_MENU(EDIT_JOININGPARTS_ID, wxMEScoreFrame::OnJoiningParts)
EVT_MENU(EDIT_DISJOINPARTS_ID, wxMEScoreFrame::OnDisjoinParts)
EVT_MENU(EDIT_MOVEPARTS_ID, wxMEScoreFrame::OnMoveParts)
EVT_MENU(EDIT_MOVEALAYER_ID, wxMEScoreFrame::OnMoveALayer)

EVT_MENU(VIEW_VIEWLAYER_ID, wxMEScoreFrame::OnViewLayer)
EVT_MENU(EDIT_NEWMEASUREBEFORE_ID, wxMEScoreFrame::OnNewMeasureBefore)
EVT_MENU(EDIT_NEWMEASUREAFTER_ID, wxMEScoreFrame::OnNewMeasureAfter)
EVT_MENU(EDIT_NEWSMEASUREBEFORE_ID, wxMEScoreFrame::OnNewSMeasureBefore)
EVT_MENU(EDIT_NEWSMEASUREAFTER_ID, wxMEScoreFrame::OnNewSMeasureAfter)
EVT_MENU(EDIT_CUTAMEASURE_ID, wxMEScoreFrame::OnCutAMeasure)
EVT_MENU(EDIT_CUTSMEASURE_ID, wxMEScoreFrame::OnCutSMeasure)
EVT_MENU(EDIT_PASTEAMEASURE_ID, wxMEScoreFrame::OnPasteAMeasure)
EVT_MENU(EDIT_PASTESMEASURE_ID, wxMEScoreFrame::OnPasteSMeasure)
EVT_MENU(EDIT_COPYAMEASURE_ID, wxMEScoreFrame::OnCopyAMeasure)
EVT_MENU(EDIT_COPYSMEASURE_ID, wxMEScoreFrame::OnCopySMeasure)

EVT_MENU(LYRIC_ADDLYRICTOAPART_ID, wxMEScoreFrame::OnAddLyricToAPart)
EVT_MENU(LYRIC_EDITLYRIC_ID, wxMEScoreFrame::OnEditLyric)
EVT_MENU(LYRIC_REMOVELYRIC_ID, wxMEScoreFrame::OnRemoveLyric)
EVT_MENU(LYRIC_SELECTLYRIC_ID, wxMEScoreFrame::OnSelectLyric)

EVT_MENU(MISC_OPENALL_ID, wxMEScoreFrame::OnOpenAll)
EVT_MENU(MISC_CLOSEALL_ID, wxMEScoreFrame::OnCloseAll)
EVT_MENU(EDIT_TRANSPOSITION_ID, wxMEScoreFrame::OnTransposition)
EVT_MENU(MISC_EDITIMAGES_ID, wxMEScoreFrame::OnEditImages)
EVT_MENU(MISC_IMPORTMAINSCOREIMAGE_ID, wxMEScoreFrame::OnImportMainScoreImage)
EVT_MENU(MISC_IMPORTIMAGESCOREPARTS_ID, wxMEScoreFrame::OnImportImageScoreParts)
EVT_MENU(MISC_SEELINKS_ID, wxMEScoreFrame::OnSeeLinks)
EVT_MENU(MISC_EDITLINK_ID, wxMEScoreFrame::OnEditLink)
EVT_MENU(VIEW_FIT2WINDOW_ID, wxMEScoreFrame::OnFit2Window)

EVT_MENU(SYMB_TIMESIGNATURE_ID, wxMEScoreFrame::OnTimeSignature)
EVT_MENU(SYMB_KEYSIGNATURE_ID, wxMEScoreFrame::OnKeySignature)
EVT_MENU(SYMB_METRONOME_ID, wxMEScoreFrame::OnMetronome)
EVT_MENU(SYMB_DYNAMICS_ID, wxMEScoreFrame::OnDynamics)
EVT_MENU(SYMB_ANNOTATIONS_ID, wxMEScoreFrame::OnAnnotations)
EVT_MENU(SYMB_GENERICTEXT_ID, wxMEScoreFrame::OnGenericText)	
EVT_MENU(SYMB_INSERTTEXT_ID, wxMEScoreFrame::OnInsertText)	
EVT_MENU(SYMB_INSERTIMAGE_ID, wxMEScoreFrame::OnInsertImage)	

EVT_MENU(SYMB_FRETBOARDS_ID, wxMEScoreFrame::OnFretboards)
EVT_MENU(SYMB_NOTES_ID, wxMEScoreFrame::OnNotes)
EVT_MENU(SYMB_NOTEHEADS_ID, wxMEScoreFrame::OnNoteheads)
EVT_MENU(SYMB_SMALLNOTES_ID, wxMEScoreFrame::OnSmallNotes)
EVT_MENU(SYMB_BEAMING_ID, wxMEScoreFrame::OnBeaming)
EVT_MENU(SYMB_RESTS_ID, wxMEScoreFrame::OnRests)
EVT_MENU(SYMB_ACCIDENTALS_ID, wxMEScoreFrame::OnAccidentals)
EVT_MENU(SYMB_AUGMENTATIONS_ID, wxMEScoreFrame::OnAugmentations)
EVT_MENU(SYMB_REPEATS_ID, wxMEScoreFrame::OnRepeats)
EVT_MENU(SYMB_BARLINES_ID, wxMEScoreFrame::OnBarlines)
EVT_MENU(SYMB_CLEFS_ID, wxMEScoreFrame::OnClefs)
EVT_MENU(SYMB_BRACKETS_ID, wxMEScoreFrame::OnBrackets)
EVT_MENU(SYMB_HORIZONTALS_ID, wxMEScoreFrame::OnHorizontals)
EVT_MENU(SYMB_REFRAINS_ID, wxMEScoreFrame::OnRefrains)
EVT_MENU(SYMB_EXPRESSIONS_ID, wxMEScoreFrame::OnExpressions)
EVT_MENU(SYMB_GENERICEXP_ID, wxMEScoreFrame::OnGenericExp)
EVT_MENU(SYMB_ORNAMENTS_ID, wxMEScoreFrame::OnOrnaments)
EVT_MENU(SYMB_KOREANORNAMENTS_ID, wxMEScoreFrame::OnKoreanOrnaments)
EVT_MENU(SYMB_FINGERING_ID, wxMEScoreFrame::OnFingering)
EVT_MENU(SYMB_MUTES_ID, wxMEScoreFrame::OnMutes)
EVT_MENU(SYMB_HARMONICS_ID, wxMEScoreFrame::OnHarmonics)
EVT_MENU(SYMB_BREATHGLASS_ID, wxMEScoreFrame::OnBreathGlass)
EVT_MENU(SYMB_STRINGS_ID, wxMEScoreFrame::OnStrings)
EVT_MENU(SYMB_PEDALS_ID, wxMEScoreFrame::OnPedals)
EVT_MENU(SYMB_VIOLIN_ID, wxMEScoreFrame::OnViolin)
EVT_MENU(SYMB_PERCUSSIONS_ID, wxMEScoreFrame::OnPercussions)

EVT_MENU(FORM_CHECK_ID, wxMEScoreFrame::OnCheck)
EVT_MENU(FORM_JUSTIFICATION_ID, wxMEScoreFrame::OnJustification)

EVT_MENU(FORM_LINEBREAKING_ID, wxMEScoreFrame::OnLineBreaking)
EVT_MENU(FORM_AUTOBEAMING_ID, wxMEScoreFrame::OnAutoBeaming)
//
EVT_MENU(FORM_BEAMING_ID, wxMEScoreFrame::OnFormatBeaming)
EVT_MENU(FORM_UPDOWN_ID, wxMEScoreFrame::OnUpDown)
EVT_MENU(FORM_FORMATTINGSETTINGS_ID, wxMEScoreFrame::OnFormattingSettings)
EVT_MENU(FORM_EDITMILLA_ID, wxMEScoreFrame::OnEditMilla)
EVT_MENU(FORM_LOADMILLA_ID, wxMEScoreFrame::OnLoadMilla)
EVT_MENU(FORM_RELOADMILLA_ID, wxMEScoreFrame::OnReLoadMilla)
EVT_MENU(FORM_DEBUGMILLAINSERTION_ID, wxMEScoreFrame::OnDebugMillaIns)
EVT_MENU(FORM_DEBUGMILLAPOSITIONING_ID, wxMEScoreFrame::OnDebugMillaPos)

EVT_MENU(FORM_SEEINVISIBLE_ID, wxMEScoreFrame::OnSeeInvisible)
EVT_MENU(FORM_SEEANCHOR_ID, wxMEScoreFrame::OnSeeAnchor)


EVT_MENU(FORM_EDITTABLE_ID, wxMEScoreFrame::OnEditTable)
EVT_MENU(FORM_EDITTABLE1_ID, wxMEScoreFrame::OnEditTable1)
EVT_MENU(FORM_EDITTABLE2_ID, wxMEScoreFrame::OnEditTable2)
EVT_MENU(FORM_EDITTABLE3_ID, wxMEScoreFrame::OnEditTable3)
EVT_MENU(FORM_EDITGENERIC1_ID, wxMEScoreFrame::OnEditTableGen1)
EVT_MENU(FORM_EDITGENERIC2_ID, wxMEScoreFrame::OnEditTableGen2)
EVT_MENU(FORM_EDITGENERIC3_ID, wxMEScoreFrame::OnEditTableGen3)
EVT_MENU(FORM_EDITPERCUSSIONS_ID, wxMEScoreFrame::OnEditTablePercussions)
EVT_MENU(FORM_EDITPEDALS_ID, wxMEScoreFrame::OnEditTablePedals)
//
EVT_MENU(FORM_LOADTABLE_ID, wxMEScoreFrame::OnLoadTable)
EVT_MENU(FORM_SCOREEXCERPT_ID, wxMEScoreFrame::OnScoreExcerpt)

EVT_MENU(EXEC_TIME_ID, wxMEScoreFrame::OnTime)
EVT_MENU(EXEC_LABELS_ID, wxMEScoreFrame::OnLabels)
EVT_MENU(EXEC_MARKERS_ID, wxMEScoreFrame::OnMarkers)
EVT_MENU(EXEC_ORCHESTRATION_ID, wxMEScoreFrame::OnOrchestration)
EVT_MENU(EXEC_SYNCHRONIZATION_ID, wxMEScoreFrame::OnSynchronization)

EVT_MENU(HELP_GUIDE_ID, wxMEScoreFrame::OnHelp)
EVT_MENU(HELP_ABOUT_ID, wxMEScoreFrame::OnAbout)

EVT_MENU_RANGE(1000,1100,wxMEScoreFrame::OnAnalysis)
EVT_IDLE(wxMEScoreFrame::OnIdle)

EVT_KEY_DOWN(wxMEScoreFrame::OnKeyDown)

EVT_CLOSE(wxMEScoreFrame::OnClose)

EVT_ACTIVATE(wxMEScoreFrame::OnActivate)
EVT_KEY_UP(wxMEScoreFrame::OnKeyUp)

EVT_CLOSE(wxMEScoreFrame::OnCloseWindow)

// 	EVT_CUSTOM(EVT_MIDI, ID_NOTEON, wxMEScoreFrame::On_MIDI_NOTEON)
//	EVT_CUSTOM(EVT_MIDI, ID_NOTEOFF, wxMEScoreFrame::On_MIDI_NOTEOFF)
//	EVT_CUSTOM(EVT_MIDI, ID_STOP, wxMEScoreFrame::On_MIDI_STOP) 
END_EVENT_TABLE()


void wxMEScoreFrame::OnCloseWindow(wxCloseEvent& event)
{
	if (true)
	{
		
		extern EventResult StopOp();
		
		StopOp();
		SMRProcessPendingEvents(); // SVUOTO GLI EVENTI PRIMA DI CHIUDERE
		
		int ans=wxID_YES;
		
		switch (ans)
		{
		case wxID_YES:      // Save, then destroy, quitting app
			Scr->GCloseWindow((Widget)this);
			break;
		case wxID_NO:       // Don't save; just destroy, quitting app
			Scr->GCloseWindow((Widget)this);
			break;
		case wxID_CANCEL:   // Do nothing - so don't quit app.
		default:
			if (!event.CanVeto()) // Test if we can veto this deletion
			{
				Scr->GCloseWindow((Widget)this);
			}
			else
				event.Veto();     // Notify the calling code that we didn't delete the frame.
			
			break;
		}
	}
}

wxMEScoreFrame::wxMEScoreFrame(wxWindow *parent,const wxString& tit,const wxPoint& pos, const wxSize& siz) 
:wxFrame(parent,-1, tit,pos,siz,wxDEFAULT_FRAME_STYLE) 

{
	menubar = NULL;
	toolbar = NULL;
	scorecanvas=NULL;
	currentMusicDC=NULL;
	int w, h;
	SetIcon(wxIcon("wdfmed"));
	menubar = new MEMenuBar();
	menubar->CreateMenu();
	SetMenuBar(menubar);
    GetformatMenu()->Check(FORM_LINEBREAKING_ID, TRUE);
	
	GetClientSize(&w, &h);
	
	toolbar = new ToolBar("editing", this, SASHLAYOUTWINDOW_ID, 
		wxPoint(0, 0), wxSize(w, 30), wxSIMPLE_BORDER);
	
	scorecanvas = new wxMEScoreCanvas(this, wxPoint(0,30), wxSize(w, h-30));
	
	CreateStatusBar();
	
	lyr_frame=NULL;
	
	progressSlide=NULL;
	
}


wxMEScoreFrame::~wxMEScoreFrame()
{
	delete scorecanvas; 
	if (toolbar)
	{
		delete toolbar;
		toolbar = NULL;
	}
	if(FrameManager::currFrame==this)
		FrameManager::currFrame=NULL;
	
	if(lyr_frame!=NULL)	
		lyr_frame->SetPuntMusEdt(NULL);
	
}

void wxMEScoreFrame::OnSize(wxSizeEvent& event)
{
	extern LiooWindow *pMusicWin;
	int w,h;
	GetClientSize(&w, &h);
	if (scorecanvas)
	{
		scorecanvas->SetSize(0, 30, w, h-30); //il 30 e' per la toolbar
		if(scorecanvas->GetPrintStat()==0 && scorecanvas->GetNScore()!=0)
		{
			scorecanvas->SetScrollbars(w-30, h-30,1,1,0,0,TRUE);
		}
	}
	
	ToolBar* toolbar = Gettoolbar();
	if (toolbar)
	{
		toolbar->SetSize(0, 0, w, 30);
	}
}


void wxMEScoreFrame::SaveinWorkspace()
{
	extern ScrList LiooListWidget;
	
	//salva il frame
	int x,y,w,h;
	GetPosition(&x,&y);
	GetSize(&w,&h);
	SMRApp::GetApp().workspace->SetWindowConfig("MusicEditor", x, y, w, h);
	
	//salva i menu
	ScrObject *scrObj=LiooListWidget.GetFirst();
	while (scrObj)
	{
		Menu* menu = (Menu*)scrObj;
		SMRApp::GetApp().workspace->SetMenuConfig(menu->GetmenuItemID(), menu->GetPosition().x,
			menu->GetPosition().y, menu->GetOrizzontal());
		scrObj=scrObj->GetNext();
	} 
	
	wxWindow *audioSync=FindWindow("AudioSyncEditor");
	if(audioSync!=NULL)
	{
		audioSync->GetPosition(&x,&y);
		audioSync->GetSize(&w,&h);
		SMRApp::GetApp().workspace->SetWindowConfig("AudioSyncEditor", x, y, w, h);
	}
	
	//salva il workspace su file
	SMRApp::GetApp().workspace->SaveWorkspace();
}

void wxMEScoreFrame::OnAnalysis(wxCommandEvent& event)
{
}

void wxMEScoreFrame::OnIdle(wxIdleEvent& event)
{
	wxClientDC maindc(scorecanvas);
	wxDC *prevDC=currDC;
	wxDC *prevMainDC=currMainDC;
	
	currDC=currMainDC=&maindc;
	
	currDC=prevDC;
	currMainDC=prevMainDC;
}

void wxMEScoreFrame::OnImportMIDI(wxCommandEvent& event)
{
 extern EventResult ImportOp();

 extern wxMusicEditor *MusicEditor;
 
 wxClientDC maindc(scorecanvas);
 currMainDC=&maindc;
 scorecanvas->Prepare(); Scr->GSetMainDrawArea();

 ImportOp();
}

void wxMEScoreFrame::OnImportOMR(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "OnImportOMR", wxOK);
}

void wxMEScoreFrame::OnImportSCORE(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "OnImportSCORE", wxOK);
}

void wxMEScoreFrame::OnImportMOODS(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "OnImportMOODS", wxOK);
}

void wxMEScoreFrame::OnExportWDF(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "OnExportWDF", wxOK);
}

void wxMEScoreFrame::OnExportIWF(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "OnExportIWF", wxOK);
}

void wxMEScoreFrame::OnExportMOODS(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "OnExportMOODS", wxOK);
}

void wxMEScoreFrame::OnChangesList(wxCommandEvent& event)
{
	extern EventResult LogMenu();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	LogMenu();
}

void wxMEScoreFrame::OnLoadChanges(wxCommandEvent& event)
{
	extern EventResult LoadACLOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	LoadACLOp();
}

void wxMEScoreFrame::OnSaveChanges(wxCommandEvent& event)
{
	extern EventResult SaveACLOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	SaveACLOp();
	
}

void wxMEScoreFrame::OnSaveAsChanges(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "OnSaveAsChanges", wxOK);
}

void wxMEScoreFrame::OnClose(wxCommandEvent& event)
{
	extern EventResult ExitOp();
	extern EventResult StopOp();
	extern wxMusicEditor *MusicEditor;
	
	if(GetLyricFramePtr()!= NULL)
		GetLyricFramePtr()->OnQuitFromME();
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	StopOp();
	if(ScoreInExecution==scorecanvas)
		ScoreInExecution=NULL;
	
	SMRProcessPendingEvents(); // SVUOTO GLI EVENTI PRIMA DI CHIUDERE
	ExitOp();
}


//Menu Edit
void wxMEScoreFrame::OnComputerView(wxCommandEvent& event)
{
	extern wxMusicEditor *MusicEditor;	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	int w,h;
	GetClientSize(&w, &h);
	if (scorecanvas)
	{
		if(scorecanvas->GetNScore()!=0)
		{
			scorecanvas->SetScrollbars(w, h-30,1,1,0,0,TRUE);
		}
	}
	
	if(SimbImg)
	{
		currentViewType=0;
		scorecanvas->SetPrintStat(0);
		pMusicWin->PPartit->SetPrintingStatus(false);
		pMusicWin->PPartit->Redraw(false);						
	}
	
	MusicEditor->RedrawView(DrawObject::GetCurrentView());
}

void wxMEScoreFrame::OnPrintView(wxCommandEvent& event)
{
	extern wxMusicEditor *MusicEditor;
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	if (SimbImg)
	{
		currentViewType=1;
		scorecanvas->SetPrintStat(1);
		pMusicWin->PPartit->SetPrintingStatus(true);
		pMusicWin->PPartit->PrintView();
	}
}

void wxMEScoreFrame::OnPrintParams(wxCommandEvent& event)
{
	extern wxMusicEditor *MusicEditor;
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	if (SimbImg)
	{
		int nPage=1;
		int nStaff=1;
		if(!EditPrintParams(edit, pMusicWin->PPartit->GetPrintViewParams(), nPage, nStaff, true, this))
		{
			currentViewType=1;
			scorecanvas->SetPrintStat(1);
			pMusicWin->PPartit->SetPrintingStatus(true);
			pMusicWin->PPartit->PrintView();
		}
	}
}

void wxMEScoreFrame::OnFirstPageHeader(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "OnFirstPageHeader", wxOK);
}

void wxMEScoreFrame::OnImportAPart(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "OnImportAPart", wxOK);
}

void wxMEScoreFrame::OnNewPart(wxCommandEvent& event)
{
	extern EventResult CAddSpartitoOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CAddSpartitoOp();
}

void wxMEScoreFrame::OnHideAPart(wxCommandEvent& event)
{
	extern EventResult CHidePartOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CHidePartOp();
	
}

void wxMEScoreFrame::OnShowAllParts(wxCommandEvent& event)
{
	extern EventResult CShowAllPartsOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CShowAllPartsOp();
	
}

void wxMEScoreFrame::OnCutAPart(wxCommandEvent& event)
{
	extern EventResult CDelSpartito();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CDelSpartito();
}

void wxMEScoreFrame::OnCopyAPart(wxCommandEvent& event)
{
	extern EventResult CCopySpartito();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CCopySpartito();	
}

void wxMEScoreFrame::OnPasteAPart(wxCommandEvent& event)	//DB
{
	
	extern EventResult CPasteSpartito();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CPasteSpartito();
}


void wxMEScoreFrame::OnSplittingPart(wxCommandEvent& event)
{
	extern EventResult CSplittingPart();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CSplittingPart();
	
}

void wxMEScoreFrame::OnCutPageBox(wxCommandEvent& event)
{
	extern EventResult CCutPageBox();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CCutPageBox();
}

void wxMEScoreFrame::OnCopyPageBox(wxCommandEvent& event)
{
	extern EventResult CCopyPageBox();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CCopyPageBox();	
}

void wxMEScoreFrame::OnPastePageBox(wxCommandEvent& event)	//DB
{
	
	extern EventResult CPastePageBox();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CPastePageBox();
}


void wxMEScoreFrame::OnMergingParts(wxCommandEvent& event)
{
	extern EventResult CMergingParts();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CMergingParts();
}

void wxMEScoreFrame::OnJoiningParts(wxCommandEvent& event)
{
	extern EventResult CJoiningParts();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CJoiningParts();
}

void wxMEScoreFrame::OnDisjoinParts(wxCommandEvent& event)
{
	extern EventResult CDisjoiningPart();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CDisjoiningPart();
}

void wxMEScoreFrame::OnMoveParts(wxCommandEvent& event)
{
	extern EventResult CMovePartOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CMovePartOp();
}

void wxMEScoreFrame::OnMoveALayer12(wxCommandEvent& event)
{
	extern short mainLay;
	extern EventResult CLayer();
	if (mainLay == 1)
	{
		Gettoolbar()->Getedit_layer()->SetLabel("L2");
		CLayer();
	}
}

void wxMEScoreFrame::OnMoveALayer23(wxCommandEvent& event)
{
	extern short mainLay;
	extern EventResult CLayer();
	if (mainLay == 2)
	{
		Gettoolbar()->Getedit_layer()->SetLabel("L3");
		CLayer();
	}
}

void wxMEScoreFrame::OnMoveALayer34(wxCommandEvent& event)
{
	extern short mainLay;
	extern EventResult CLayer();
	if (mainLay == 3)
	{
		Gettoolbar()->Getedit_layer()->SetLabel("L4");
		CLayer();
	}
}

void wxMEScoreFrame::OnMoveALayer45(wxCommandEvent& event)
{
	extern short mainLay;
	extern EventResult CLayer();
	if (mainLay == 4)
	{
		Gettoolbar()->Getedit_layer()->SetLabel("L5");
		CLayer();
	}
}

void wxMEScoreFrame::OnMoveALayer56(wxCommandEvent& event)
{
	extern short mainLay;
	extern EventResult CLayer();
	if (mainLay == 5)
	{
		Gettoolbar()->Getedit_layer()->SetLabel("L6");
		CLayer();
	}
}

void wxMEScoreFrame::OnMoveALayer67(wxCommandEvent& event)
{
	extern short mainLay;
	extern EventResult CLayer();
	if (mainLay == 6)
	{
		Gettoolbar()->Getedit_layer()->SetLabel("L7");
		CLayer();
	}
}

void wxMEScoreFrame::OnMoveALayer78(wxCommandEvent& event)
{
	extern short mainLay;
	extern EventResult CLayer();
	if (mainLay == 7)
	{
		Gettoolbar()->Getedit_layer()->SetLabel("L8");
		CLayer();
	}
}

void wxMEScoreFrame::OnMoveALayer89(wxCommandEvent& event)
{
	extern short mainLay;
	extern EventResult CLayer();
	if (mainLay == 8)
	{
		Gettoolbar()->Getedit_layer()->SetLabel("L9");
		CLayer();
	}
}

void wxMEScoreFrame::OnMoveALayer910(wxCommandEvent& event)
{
	extern short mainLay;
	extern EventResult CLayer();
	if (mainLay == 9)
	{
		Gettoolbar()->Getedit_layer()->SetLabel("L10");
		CLayer();
	}
}

void wxMEScoreFrame::OnMoveALayer1011(wxCommandEvent& event)
{
	extern short mainLay;
	extern EventResult CLayer();
	if (mainLay == 10)
	{
		Gettoolbar()->Getedit_layer()->SetLabel("L11");
		CLayer();
	}
}

void wxMEScoreFrame::OnMoveALayer1112(wxCommandEvent& event)
{
	extern short mainLay;
	extern EventResult CLayer();
	if (mainLay == 11)
	{
		Gettoolbar()->Getedit_layer()->SetLabel("L12");
		CLayer();
	}
}

void wxMEScoreFrame::OnMoveALayer121(wxCommandEvent& event)
{
	extern short mainLay;
	extern EventResult CLayer();
	if (mainLay == 12)
	{
		Gettoolbar()->Getedit_layer()->SetLabel("L1");
		CLayer();
	}
}

void wxMEScoreFrame::OnMoveALayer14(wxCommandEvent& event)
{
	extern short mainLay;
	extern EventResult CLayer();
	if (mainLay == 1)
	{
		Gettoolbar()->Getedit_layer()->SetLabel("L4");
		CLayer();
	}
}

void wxMEScoreFrame::OnMoveALayer48(wxCommandEvent& event)
{
	extern short mainLay;
	extern EventResult CLayer();
	if (mainLay == 4)
	{
		Gettoolbar()->Getedit_layer()->SetLabel("L8");
		CLayer();
	}
}

void wxMEScoreFrame::OnMoveALayer812(wxCommandEvent& event)
{
	extern short mainLay;
	extern EventResult CLayer();
	if (mainLay == 8)
	{
		Gettoolbar()->Getedit_layer()->SetLabel("L12");
		CLayer();
	}
}

void wxMEScoreFrame::OnNewMeasureBefore(wxCommandEvent& event)
{
	extern EventResult InsBatPrimaOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	InsBatPrimaOp();
}

void wxMEScoreFrame::OnNewMeasureAfter(wxCommandEvent& event)
{
	extern EventResult InsBatDopoOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	InsBatDopoOp();
}

void wxMEScoreFrame::OnNewSMeasureBefore(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "New Measure Before", wxOK);
}

void wxMEScoreFrame::OnNewSMeasureAfter(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "New Measure After", wxOK);
}

void wxMEScoreFrame::OnCutAMeasure(wxCommandEvent& event)
{
	extern EventResult CDelColonnaBatOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CDelColonnaBatOp();
}

void wxMEScoreFrame::OnCutSMeasure(wxCommandEvent& event)
{
	extern EventResult CDelBattutaOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CDelBattutaOp();
}

void wxMEScoreFrame::OnPasteAMeasure(wxCommandEvent& event)
{
	extern EventResult CIncollaBattOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CIncollaBattOp();
}

void wxMEScoreFrame::OnPasteSMeasure(wxCommandEvent& event)
{
	extern EventResult CIncollaSBattOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CIncollaSBattOp();
}

void wxMEScoreFrame::OnCopyAMeasure(wxCommandEvent& event)
{
	extern EventResult CCopiaBattOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CCopiaBattOp();
}

void wxMEScoreFrame::OnCopySMeasure(wxCommandEvent& event)
{
	extern EventResult CCopiaSBattOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CCopiaSBattOp();
}

void wxMEScoreFrame::OnImportMainScoreImage(wxCommandEvent& event)
{
	wxMessageBox("Not implemented yet", "OnImportMainScoreImage", wxOK);
}

void wxMEScoreFrame::OnImportImageScoreParts(wxCommandEvent& event)
{
	wxMessageBox("Not implemented yet", "OnImportImageScoreParts", wxOK);
}


LyricFrame* wxMEScoreFrame::OpenLyricEditor(wxMEScoreFrame *linking)
{
	LyricFrame *lyr_frame;
	lyr_frame = new LyricFrame(linking, -1,
		"Lyric Editor",
		wxPoint(100, 100), wxSize(450, 300),wxDEFAULT_FRAME_STYLE,linking);
	
    lyr_frame->SetWriting();
    lyr_frame->Show(TRUE);
    return lyr_frame;
}

void wxMEScoreFrame::OnAddLyricToAPart(wxCommandEvent& event)
{ 
	
	extern EventResult CNewLyric();
	extern wxMusicEditor *MusicEditor;
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	CNewLyric();
}

void wxMEScoreFrame::OnSelectLyric(wxCommandEvent& event)
{ 
	
	extern EventResult CSelectLyric();
	extern wxMusicEditor *MusicEditor;
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CSelectLyric();
	
}


LyricFrame* wxMEScoreFrame::OpenLyricFrame()
{
    if(lyr_frame!=NULL)
	{
		lyr_frame->SetFocus();
		return lyr_frame;
	}
	
	lyr_frame = OpenLyricEditor(this);
    return lyr_frame;
}

LyricFrame* wxMEScoreFrame::GetLyricFramePtr()
{
	return lyr_frame;
}

void wxMEScoreFrame::SetLyricFramePtr(LyricFrame *ptr)
{
	lyr_frame=ptr;
}

void wxMEScoreFrame::OnEditLyric(wxCommandEvent& event)
{
	extern EventResult SelSpaForLyrEdt();
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CEditLyric();
}

void wxMEScoreFrame::OnRemoveLyric(wxCommandEvent& event)
{
	extern EventResult CSelSpaForRemoveLyr();
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CRemoveLyr();
}

void wxMEScoreFrame::OnOpenAll(wxCommandEvent& event)
{
	if (!GetsymbolsMenu()->IsChecked(SYMB_NOTES_ID))
	{
		CreateNotaMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_NOTES_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_SMALLNOTES_ID))
	{
		CreateSmallNotesMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_SMALLNOTES_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_BEAMING_ID))
	{
		CreateBeamingMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_BEAMING_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_RESTS_ID))
	{
		CreateRestsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_RESTS_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_ACCIDENTALS_ID))
	{
		CreateAccidentalsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_ACCIDENTALS_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_AUGMENTATIONS_ID))
	{
		CreateAugmentationsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_AUGMENTATIONS_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_REPEATS_ID))
	{
		CreateRepeatsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_REPEATS_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_BARLINES_ID))
	{
		CreateBarlinesMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_BARLINES_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_CLEFS_ID))
	{
		CreateClefsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_CLEFS_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_BRACKETS_ID))
	{
		CreateBracketsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_BRACKETS_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_HORIZONTALS_ID))
	{
		CreateHorizontalsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_HORIZONTALS_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_REFRAINS_ID))
	{
		CreateRefrainsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_REFRAINS_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_EXPRESSIONS_ID))
	{
		CreateExpressionsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_EXPRESSIONS_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_GENERICEXP_ID))
	{
		CreateGenericExpMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_GENERICEXP_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_ORNAMENTS_ID))
	{
		CreateOrnamentsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_ORNAMENTS_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_KOREANORNAMENTS_ID))
	{
		CreateKoreanOrnamentsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_KOREANORNAMENTS_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_FINGERING_ID))
	{
		CreateFingeringMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_FINGERING_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_MUTES_ID))
	{
		CreateMutesMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_MUTES_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_HARMONICS_ID))
	{
		CreateHarmonicsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_HARMONICS_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_BREATHGLASS_ID))
	{
		CreateBreathGlassMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_BREATHGLASS_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_PEDALS_ID))
	{
		CreatePedalsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_PEDALS_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_STRINGS_ID))
	{
		CreateStringsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_STRINGS_ID, TRUE);
	}
	if (!GetsymbolsMenu()->IsChecked(SYMB_VIOLIN_ID))
	{
		CreateViolinMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_VIOLIN_ID, TRUE);
	}
}


void wxMEScoreFrame::OnCloseAll(wxCommandEvent& event)
{
	MusicEditor->CloseAllMenus();
}

void wxMEScoreFrame::OnTransposition(wxCommandEvent& event)
{
	extern EventResult CTranspose();
	
	extern wxMusicEditor *MusicEditor;
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); 
	Scr->GSetMainDrawArea();
	CTranspose();
}

void wxMEScoreFrame::OnEditImages(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "Edit Images", wxOK);
}


void wxMEScoreFrame::OnSeeLinks(wxCommandEvent& event)
{
	extern EventResult CSeeLinkOp(bool view);
	extern wxMusicEditor *MusicEditor;
	bool viewlink;
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;

	if(GetmiscMenu()->IsChecked(MISC_SEELINKS_ID))
	{
		viewlink=true;
		GetmiscMenu()->Enable(MISC_EDITLINK_ID,FALSE);
	}
	else
	{
		viewlink=false;
		GetmiscMenu()->Enable(MISC_EDITLINK_ID,TRUE);
	}
	scorecanvas->Prepare(); 
	Scr->GSetMainDrawArea();
	CSeeLinkOp(viewlink);
	GetmiscMenu()->Check(MISC_SEELINKS_ID, viewlink);
	if(!SimbImg)
		MusicEditor->RedrawView(DrawObject::GetCurrentView());
}

void wxMEScoreFrame::OnEditLink(wxCommandEvent& event)
{
	extern EventResult CEditLinkOp(bool view2);
	bool viewlink2;
	extern wxMusicEditor *MusicEditor;

	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;

	if(GetmiscMenu()->IsChecked(MISC_EDITLINK_ID))
	{
		viewlink2=true;
		GetmiscMenu()->Enable(MISC_SEELINKS_ID,FALSE);

	}
	else
	{
		viewlink2=false;
		GetmiscMenu()->Enable(MISC_SEELINKS_ID,TRUE);

	}

	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	CEditLinkOp(viewlink2);
	GetmiscMenu()->Check(MISC_EDITLINK_ID, viewlink2);
	if(!SimbImg)
		MusicEditor->RedrawView(DrawObject::GetCurrentView());
}

//Menu Symbols
void wxMEScoreFrame::OnTimeSignature(wxCommandEvent& event)
{
	extern EventResult InsTempoMenu();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	InsTempoMenu();
}

void wxMEScoreFrame::OnKeySignature(wxCommandEvent& event)
{
	extern EventResult CArmChvOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	CArmChvOp();
}

void wxMEScoreFrame::OnMetronome(wxCommandEvent& event)
{
	extern EventResult CMovimentoOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CMovimentoOp();
}

void wxMEScoreFrame::OnDynamics(wxCommandEvent& event)
{
	extern EventResult CTDinamico();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CTDinamico();
}

void wxMEScoreFrame::OnAnnotations(wxCommandEvent& event)
{
	extern EventResult CAnnotazioneOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CAnnotazioneOp();
}


void wxMEScoreFrame::OnGenericText(wxCommandEvent& event)
{
	extern EventResult CtgenericOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CtgenericOp();
}

void wxMEScoreFrame::OnFretboards(wxCommandEvent& event)
{
	extern EventResult CChitarraOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CChitarraOp();
}


void wxMEScoreFrame::OnNotes(wxCommandEvent& event)
{ 
	if (GetsymbolsMenu()->IsChecked(SYMB_NOTES_ID))
	{
		CreateNotaMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_NOTES_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_NOTES_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_NOTES_ID,FALSE);
	}
}


void wxMEScoreFrame::OnNoteheads(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "OnNoteheads", wxOK);
}

void wxMEScoreFrame::OnPercussions(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "OnPercussions", wxOK);
}


void wxMEScoreFrame::OnSmallNotes(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_SMALLNOTES_ID))
	{
		CreateSmallNotesMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_SMALLNOTES_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_SMALLNOTES_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_SMALLNOTES_ID,FALSE);
	}
}

void wxMEScoreFrame::OnBeaming(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_BEAMING_ID))
	{
		CreateBeamingMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_BEAMING_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_BEAMING_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_BEAMING_ID,FALSE);
	}
}


void wxMEScoreFrame::OnRests(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_RESTS_ID))
	{
		CreateRestsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_RESTS_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_RESTS_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_RESTS_ID,FALSE);
	}
}


void wxMEScoreFrame::OnAccidentals(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_ACCIDENTALS_ID))
	{
		CreateAccidentalsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_ACCIDENTALS_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_ACCIDENTALS_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_ACCIDENTALS_ID,FALSE);
	}
}


void wxMEScoreFrame::OnAugmentations(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_AUGMENTATIONS_ID))
	{
		CreateAugmentationsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_AUGMENTATIONS_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_AUGMENTATIONS_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_AUGMENTATIONS_ID,FALSE);
	}
}


void wxMEScoreFrame::OnRepeats(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_REPEATS_ID))
	{
		CreateRepeatsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_REPEATS_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_REPEATS_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_REPEATS_ID,FALSE);
	}
}

void wxMEScoreFrame::OnBarlines(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_BARLINES_ID))
	{
		CreateBarlinesMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_BARLINES_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_BARLINES_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_BARLINES_ID,FALSE);
	}
}

void wxMEScoreFrame::OnClefs(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_CLEFS_ID))
	{
		CreateClefsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_CLEFS_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_CLEFS_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_CLEFS_ID,FALSE);
	}
}

void wxMEScoreFrame::OnBrackets(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_BRACKETS_ID))
	{
		CreateBracketsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_BRACKETS_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_BRACKETS_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_BRACKETS_ID,FALSE);
	}
}

void wxMEScoreFrame::OnHorizontals(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_HORIZONTALS_ID))
	{
		CreateHorizontalsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_HORIZONTALS_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_HORIZONTALS_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_HORIZONTALS_ID,FALSE);
	}
}

void wxMEScoreFrame::OnRefrains(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_REFRAINS_ID))
	{
		CreateRefrainsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_REFRAINS_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_REFRAINS_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_REFRAINS_ID,FALSE);
	}
}


void wxMEScoreFrame::OnExpressions(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_EXPRESSIONS_ID))
	{
		CreateExpressionsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_EXPRESSIONS_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_EXPRESSIONS_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_EXPRESSIONS_ID,FALSE);
	}
}

void wxMEScoreFrame::OnGenericExp(wxCommandEvent& event)
{ 
	extern Simboli simboli;
	
	if (GetsymbolsMenu()->IsChecked(SYMB_GENERICEXP_ID))
	{
		CreateGenericExpMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_GENERICEXP_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_GENERICEXP_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_GENERICEXP_ID,FALSE);
	}
}

void wxMEScoreFrame::OnOrnaments(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_ORNAMENTS_ID))
	{
		CreateOrnamentsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_ORNAMENTS_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_ORNAMENTS_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_ORNAMENTS_ID,FALSE);
	}
}

void wxMEScoreFrame::OnKoreanOrnaments(wxCommandEvent& event)
{
 if (GetsymbolsMenu()->IsChecked(SYMB_KOREANORNAMENTS_ID))
  {
   CreateKoreanOrnamentsMenu();
   MusicEditor->CheckSymbolsMenu(SYMB_KOREANORNAMENTS_ID,TRUE);
  }
  else
   {
    Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_KOREANORNAMENTS_ID);
    menu->CloseMenu();     
    MusicEditor->CheckSymbolsMenu(SYMB_KOREANORNAMENTS_ID,FALSE);
   }
}

void wxMEScoreFrame::OnFingering(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_FINGERING_ID))
	{
		CreateFingeringMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_FINGERING_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_FINGERING_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_FINGERING_ID,FALSE);
	}
}

void wxMEScoreFrame::OnMutes(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_MUTES_ID))
	{
		CreateMutesMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_MUTES_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_MUTES_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_MUTES_ID,FALSE);
	}
}

void wxMEScoreFrame::OnHarmonics(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_HARMONICS_ID))
	{
		CreateHarmonicsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_HARMONICS_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_HARMONICS_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_HARMONICS_ID,FALSE);
	}
}

void wxMEScoreFrame::OnBreathGlass(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_BREATHGLASS_ID))
	{
		CreateBreathGlassMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_BREATHGLASS_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_BREATHGLASS_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_BREATHGLASS_ID,FALSE);
	}
}

void wxMEScoreFrame::OnStrings(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_STRINGS_ID))
	{
		CreateStringsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_STRINGS_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_STRINGS_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_STRINGS_ID,FALSE);
	}
}

void wxMEScoreFrame::OnPedals(wxCommandEvent& event)
{
	
	if (GetsymbolsMenu()->IsChecked(SYMB_PEDALS_ID))
	{
		CreatePedalsMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_PEDALS_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_PEDALS_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_PEDALS_ID,FALSE);
	}
	
}

void wxMEScoreFrame::OnViolin(wxCommandEvent& event)
{
	if (GetsymbolsMenu()->IsChecked(SYMB_VIOLIN_ID))
	{
		CreateViolinMenu();
		MusicEditor->CheckSymbolsMenu(SYMB_VIOLIN_ID,TRUE);
	}
	else
	{
		Menu* menu = (Menu *)LiooListWidget.GetNodoItemID(SYMB_VIOLIN_ID);
		menu->CloseMenu();     
		MusicEditor->CheckSymbolsMenu(SYMB_VIOLIN_ID,FALSE);
	}
}


//Menu Format

void wxMEScoreFrame::OnFormat(wxCommandEvent& event)
{
	
	
}

void wxMEScoreFrame::OnCheck(wxCommandEvent& event)
{
	extern EventResult CCheckBattOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CCheckBattOp();
}


void wxMEScoreFrame::OnLineBreaking(wxCommandEvent& event)
{
	
	extern EventResult CLineBreakOp();
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	// potrei passargli un puntatore a wxMenu per sticcare il line break in caso
	// di uscita strana
	// gli faccio tornare un valore se e' TRUE e' andato tutto OKI, altrimenti sticca la voce di menu di line break
	// o un parametro passato per reference
	CLineBreakOp();
	
	scorecanvas->SetAutoLineBreak(AutoLineBreak);
	if (AutoLineBreak)
	{
		GetformatMenu()->Check(FORM_LINEBREAKING_ID, TRUE);
	}
	else
	{
		GetformatMenu()->Check(FORM_LINEBREAKING_ID, FALSE);
	} 
	
}





void wxMEScoreFrame::OnJustification(wxCommandEvent& event)
{
	extern EventResult CJustifAllOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CJustifAllOp();
}

void wxMEScoreFrame::OnFormatBeaming(wxCommandEvent& event)
{
	extern EventResult CFormatBeamingOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CFormatBeamingOp();
}

void wxMEScoreFrame::OnUpDown(wxCommandEvent& event)
{
	extern EventResult CUpDownOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CUpDownOp();
}

void wxMEScoreFrame::OnFormattingSettings(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "OnFormattingSettings", wxOK);
}

void wxMEScoreFrame::OnEditMilla(wxCommandEvent& event)
{
	wxString smflFile=SMFLManager::Get()->GetSMFLFilename();

	if (wxFileExists(smflFile))   
		SMRApp::GetApp().settings->Execute(smflFile,APP_MILLAEDITOR);
	else
	{
		wxMessageBox("Error: no SMFL file defined");
	}
}

void wxMEScoreFrame::OnLoadMilla(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "OnLoadMilla", wxOK);
}

void wxMEScoreFrame::OnReLoadMilla(wxCommandEvent& event)
{
	extern wxMusicEditor *MusicEditor;	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	SMFLManager::Get()->Reload();
			
	pMusicWin->Redraw();
}

void wxMEScoreFrame::OnDebugMillaIns(wxCommandEvent& event)
{
    if (GetformatMenu()->IsChecked(FORM_DEBUGMILLAPOSITIONING_ID))
		GetformatMenu()->Check(FORM_DEBUGMILLAPOSITIONING_ID, FALSE);
	
    extern EventResult DebugMillaInsOp();
    extern wxMusicEditor *MusicEditor;
    wxClientDC maindc(scorecanvas);
    currMainDC=&maindc;
    
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
    DebugMillaInsOp();
}

void wxMEScoreFrame::OnDebugMillaPos(wxCommandEvent& event)
{
    if (GetformatMenu()->IsChecked(FORM_DEBUGMILLAINSERTION_ID))
		GetformatMenu()->Check(FORM_DEBUGMILLAINSERTION_ID, FALSE);
	
    extern EventResult DebugMillaPosOp();
    extern wxMusicEditor *MusicEditor;
    wxClientDC maindc(scorecanvas);
    currMainDC=&maindc;
    scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
    DebugMillaPosOp();
	
	scorecanvas->SetDebugNormalMillaPos(DebugNormalMillaPos);
}


void wxMEScoreFrame::OnSeeInvisible(wxCommandEvent& event)
{	
	if (GetformatMenu()->IsChecked(FORM_SEEINVISIBLE_ID))
	{
		GetformatMenu()->Check(FORM_SEEINVISIBLE_ID, TRUE);
		visible=true;
	}
	else
	{
		GetformatMenu()->Check(FORM_SEEINVISIBLE_ID, FALSE);
		visible=false;
	}  
	MusicEditor->RedrawViews(0,TRUE);
}

void wxMEScoreFrame::OnSeeAnchor(wxCommandEvent& event)
{	
	if (GetformatMenu()->IsChecked(FORM_SEEANCHOR_ID))
	{
		GetformatMenu()->Check(FORM_SEEANCHOR_ID, TRUE);
	}
	else
	{
		GetformatMenu()->Check(FORM_SEEANCHOR_ID, FALSE);
	}
		
	GetsymbolsMenu()->IsChecked(SYMB_VIOLIN_ID);
	
	extern EventResult VisSpaziOp();
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	VisSpaziOp();
}


void wxMEScoreFrame::OnCompViewPar(wxCommandEvent& event)
{
	extern EventResult CompViewParOp();	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CompViewParOp();
}


void wxMEScoreFrame::OnEditTable(wxCommandEvent& event)
{
	EditFontTable(0);
}

void wxMEScoreFrame::OnEditTable1(wxCommandEvent& event)
{
	EditFontTable(0);
}

void wxMEScoreFrame::OnEditTable2(wxCommandEvent& event)
{
	EditFontTable(1);
}

void wxMEScoreFrame::OnEditTable3(wxCommandEvent& event)
{
	EditFontTable(2);
}

void wxMEScoreFrame::OnEditTableGen1(wxCommandEvent& event)
{
	EditFontTable(6);
}

void wxMEScoreFrame::OnEditTableGen2(wxCommandEvent& event)
{
	EditFontTable(7);
}

void wxMEScoreFrame::OnEditTableGen3(wxCommandEvent& event)
{
	EditFontTable(8);
}

void wxMEScoreFrame::OnEditTablePercussions(wxCommandEvent& event)
{
	EditFontTable(9);
}

void wxMEScoreFrame::OnEditTablePedals(wxCommandEvent& event)
{
	EditFontTable(10);
}

void wxMEScoreFrame::OnLoadTable(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "Loading Specific Font Tables", wxOK);
}

void wxMEScoreFrame::OnScoreExcerpt(wxCommandEvent& event)
{
	extern EventResult CScoreExcerpt();
	extern wxMusicEditor *MusicEditor;

	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();

	CScoreExcerpt();
}

//Menu Execution
void wxMEScoreFrame::OnTime(wxCommandEvent& event)
{
	extern EventResult InsTempoMenu();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	InsTempoMenu();
}

void wxMEScoreFrame::OnLabels(wxCommandEvent& event)
{
	extern EventResult CLetteraOp();
	
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	CLetteraOp();
}

void wxMEScoreFrame::OnMarkers(wxCommandEvent& event)
{
	wxMessageBox(NOTYETIMPLEMENTED, "OnMarkers", wxOK);
}

void wxMEScoreFrame::OnOrchestration(wxCommandEvent& event)
{
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
}

void wxMEScoreFrame::OnExportMIDI(wxCommandEvent& event)
{	
	extern EventResult MidiOp();
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	MidiOp();
}

/* TBR
static bool audioFilter(WDFTreeItemData *item)
{
	if(item->GetType()==WDF_Audio)
	{
		AudioWF *audioItem=(AudioWF*)item;
		wxString ext=audioItem->GetFileName().Right(4).MakeUpper();
		if(ext==".WAV" || ext==".MP3")
			return true;
	}
	return false;
}
*/

void wxMEScoreFrame::OnSynchronization(wxCommandEvent& event)
{
/* TBR
	extern EventResult SyncOp();
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	
	SMRItemList audioList;
	
	MusicEditor->smrf->FilterItems(audioFilter,audioList);
	int nAudios=audioList.GetCount();
	
	if(nAudios==0)
		Message("No .wav or .mp3 files associated with the WEDELMUSIC object");
	else
	{
		wxString *audios=new wxString[nAudios];
		int i=0;
		
		wxWDFItemListNode *node = audioList.GetFirst();
		while (node)
		{
			AudioWF *audioItem = (AudioWF*)node->GetData();
			
			audios[i++]=audioItem->GetDesc();
			
			node = node->GetNext();
		}
		
		wxSingleChoiceDialog audioChoice(this,"Select the audio to syncronize with:","audio resources",nAudios,audios);
		
		audioChoice.SetSize(500,200);
		audioChoice.Centre();
		
		if(audioChoice.ShowModal()==wxID_OK)
		{
			int sel=audioChoice.GetSelection();
			AudioWF *audio=(AudioWF*) audioList.Nth(sel)->GetData();
			int nMeasures;
			if(SimbImg)
				nMeasures=pMusicWin->PPartit->GetNumeroBattute();

			MusicEditor->smrf->OpenAudioSyncFrame(this, "Audio Synchronization",wxPoint(0,0), audio, nMeasures);
		}
		delete[] audios;		
	}
*/
}

//Menu Help
void wxMEScoreFrame::OnHelp(wxCommandEvent& event)
{
	extern EventResult HelpOp();
	
	HelpOp();
}

void wxMEScoreFrame::OnAbout(wxCommandEvent& event)
{
	extern EventResult AboutOp();
	
	AboutOp();
}

void wxMEScoreFrame::OnViewLayer(wxCommandEvent& event)
{
	extern EventResult CViewLayerOp();
	extern wxMusicEditor *MusicEditor;
	
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	CViewLayerOp();
}

void wxMEScoreFrame::PrepareCanvas()
{
	currentMusicDC = new wxClientDC(scorecanvas);
	currMainDC=currentMusicDC;
	scorecanvas->Prepare();
	Scr->GSetMainDrawArea();	
}

void wxMEScoreFrame::UnprepareCanvas()
{	
	if(currentMusicDC)
		delete currentMusicDC;
	currMainDC=NULL;	
}

void wxMEScoreFrame::DisableResize()
{
	wxSize s=GetSize();
	SetSizeHints(s.GetWidth(),s.GetHeight(),s.GetWidth(),s.GetHeight());
}

void wxMEScoreFrame::EnableResize()
{
	SetSizeHints(-1,-1,-1,-1);
}


void wxMEScoreFrame::OnInsertText(wxCommandEvent& event)
{
	extern EventResult CInsertTextOp();
	
	extern wxMusicEditor *MusicEditor;
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	if(currentViewType==1)
		CInsertTextOp();
	Scr->GSetMainDrawArea();
}

void wxMEScoreFrame::OnInsertImage(wxCommandEvent& event)
{
	extern EventResult CInsertImageOp();
	
	extern wxMusicEditor *MusicEditor;
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	if(currentViewType==1)
		CInsertImageOp();
	Scr->GSetMainDrawArea();
}

void wxMEScoreFrame::OnMoveALayer(wxCommandEvent &event)
{
	extern EventResult CSwitchLayer();
	
	extern wxMusicEditor *MusicEditor;
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	CSwitchLayer();
	Scr->GSetMainDrawArea();
}

void wxMEScoreFrame::OnFit2Window(wxCommandEvent &event)
{
	extern EventResult CFit2Window();
	
	extern wxMusicEditor *MusicEditor;
	wxClientDC maindc(scorecanvas);
	currMainDC=&maindc;
	scorecanvas->Prepare(); Scr->GSetMainDrawArea();
	CFit2Window();
	Scr->GSetMainDrawArea();
}

void wxMEScoreFrame::CreateProgressSlide(wxString info, int maxVal)
{
	progressSlide = new wxGauge(GetStatusBar(), -1, maxVal, wxPoint(100,5), wxSize(200,15), wxGA_PROGRESSBAR);
	GetStatusBar()->SetStatusText(info,0);
}

void wxMEScoreFrame::SetProgressSlide(int value, wxString info)
{
	if(progressSlide!=NULL)
		progressSlide->SetValue(value);
	GetStatusBar()->SetStatusText(info,0);
}

void wxMEScoreFrame::DeleteProgressSlide()
{
	if(progressSlide!=NULL)
	{
		delete progressSlide;
		progressSlide=NULL; 
	}
	GetStatusBar()->SetStatusText("",0);
}

void wxMEScoreFrame::OnOpenEtt(wxString ett)
{
	extern wxString ettFile;

	ettFile = ett;
	wxCommandEvent event;
	Gettoolbar()->OnEditexec(event);
	Gettoolbar()->OnExec_init(event);
	Gettoolbar()->OnExec_start(event);
}

void wxMEScoreFrame::OnAutoBeaming(wxCommandEvent &event)
{
	Battuta::SetAutoBeaming(!Battuta::GetAutoBeaming());
	GetformatMenu()->Check(FORM_AUTOBEAMING_ID, Battuta::GetAutoBeaming());
}	

void wxMEScoreFrame::OnActivate(wxActivateEvent& event)
{
	wxWindow::SetFocus();
}

bool static shifted=false;
extern ScrList LiooList;
void wxMEScoreFrame::OnKeyDown(wxKeyEvent &event)
{
	if(GetMusicWin()==NULL)
		return;
	
	EventInfo evInf;
	wxDC *prevDC=currDC;
	wxDC *prevMainDC=currMainDC;
	scorecanvas->Prepare();
	wxClientDC dc(this);
	wxClientDC maindc(this);
	currDC=&dc;
	currMainDC=&maindc;
	wxCommandEvent evt;
	
	switch (event.KeyCode())
	{
	case WXK_DIVIDE:
		if(shifted)
			CLegaturaValore();
		else
			CAcciaccaturaOp();
		break;
	case 'g':
	case 'G':
		CAcciaccaturaOp();
		break;
	case '=':
		CLegaturaValore();
		break;
	case WXK_MULTIPLY:
	case 'n':
	case 'N':
		CBequadroOp();
		break;
	case WXK_SUBTRACT:
	case 'f':
	case 'F':
		CBemolleOp();
		break;
	case WXK_ADD:
	case 's':
	case 'S':
		CDiesisOp();
		break;
	case WXK_NUMPAD8:
	case '8':
		if(shifted)
			CPBreveOp();
		else
			CNBreveOp();
		break;
	case WXK_NUMPAD9:
	case '9':
		CTerzine();
		break;
	case WXK_NUMPAD7:
	case '7':
		if(shifted)
			CPSemibreveOp();
		else
			CNSemibreveOp();
		break;
	case WXK_NUMPAD6:
	case '6':
		if(shifted)
			CPMinimaOp();
		else
			CNMinimaOp();
		break;
	case WXK_NUMPAD5:
	case '5':
		if(shifted)
			CPSemiminimaOp();
		else
			CNSemiminimaOp();
		break;
	case WXK_NUMPAD4:
	case '4':
		if(shifted)
			CPCromaOp();
		else
			CNCromaOp();
		break;
	case WXK_NUMPAD3:
	case '3':
		if(shifted)
			CPSemicromaOp();
		else
			CNSemicromaOp();
		break;
	case WXK_NUMPAD2:
	case '2':
		if(shifted)
			CPBiscromaOp();
		else
			CNBiscromaOp();
		break;
	case WXK_NUMPAD1:
	case '1':
		if(shifted)
			CPSemibiscromaOp();
		else
			CNSemibiscromaOp();
		break;
	case WXK_NUMPAD0:
	case '0':
		if(shifted)
			CPFusaOp();
		else
			CNFusaOp();
		break;
	case WXK_DECIMAL:
		C1PuntoOp();
		break;
	case 'x':
	case 'X':
		C2DiesisOp();
		break;
	case 'v':
	case 'V':
		C2BemolleOp();
		break;
	case 'b':
	case 'B':
		OnAutoBeaming(evt);
		break;
	case WXK_RETURN:
	case WXK_SHIFT:
		shifted=true;
	default: ;
  }
  
  Scr->SetTmpKey(event.KeyCode());
  evInf=LiooList.SignalEvent();
  if (scorecanvas->GetMusicWin()->GetModEsecuzione()) Scr->GSetMainDrawArea();
  currDC=prevDC;
  currMainDC=prevMainDC;
}

void wxMEScoreFrame::OnKeyUp(wxKeyEvent &event)
{
	switch (event.KeyCode())
	{
	case WXK_RETURN:
	case WXK_SHIFT:
		shifted=false;
		break;
    default: ;
	}
}