/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: scrlist.cpp,v 1.1 2005/07/25 16:30:34 bellinpo Exp $ */


#include "drw/scrlist.hpp"
#include "drw/menu.hpp"

extern Widget WindowActive;

ScrList::ScrList()
{
	first=NULL;
	last=NULL;
}

ScrObject *ScrList::GetFirst()
{
	return first;
}

void ScrList::Add(ScrObject* scrObj)
{
	if(last!=NULL)
		last->SetNext(scrObj);
	else
	{
		first=scrObj;
	}
	last=scrObj;
}

ScrObject *ScrList::DropFirst()
{
	ScrObject *scrObj=first;
	
	if(scrObj!=NULL)
	{
		first=scrObj->GetNext();
		if(first==NULL)
			last=NULL;
		scrObj->SetNext(NULL);
	}
	return scrObj;
}

void ScrList::Delete(ScrObject *s)
{
	ScrObject *scrObj=first;
	ScrObject *scrPrec=first;
	
	if(scrObj!=NULL)
	{
		while ((scrObj!=s) && (scrObj!=NULL))
		{
			scrPrec=scrObj;
			scrObj=scrObj->GetNext();
		}
		if (scrObj!=NULL)
		{
			if (first==scrObj) 
			{
				first=first->GetNext();
				delete scrObj;
				scrObj=NULL; 
			}
			else
			{
				if (last==scrObj)
				{
					scrPrec->SetNext(scrObj->GetNext());
					delete scrObj;
					scrObj=NULL; 
					last=scrPrec;
				}
				else
				{
					scrPrec->SetNext(scrObj->GetNext());
					delete scrObj;
					scrObj=NULL; 
				}
			}
			if(first==NULL)
				last=NULL;
		}
	}
}


void ScrList::DeleteAll()
{
	ScrObject *scrObj=first;
	
	while (scrObj!=NULL)
	{
		first = first->GetNext();
		scrObj = first;
	}
	
	first=NULL;
	last=NULL;
}





ScrObject *ScrList::GetNodoItemID(int itemID)
{
    // se qualcosa non va ritorna NULL;
	
	ScrObject *ps=first, *pp=NULL;
	
	while (ps!=NULL)
	{ 
		if ((((Menu *)ps)->GetmenuItemID())==itemID) pp=ps;
		ps=ps->GetNext();
	} 
	return pp;
}


ScrObject *ScrList::GetNodoW(Widget w)
{
    // se qualcosa non va ritorna NULL;
	
	ScrObject *ps=first, *pp=NULL;
	
	while (ps!=NULL)
	{ 
		if ((ps->GetScrWindow())==w) pp=ps;
		ps=ps->GetNext();
	} 
	return pp;
}


ScrObject *ScrList::GetNodoDA(Widget w)
{
    // se qualcosa non va ritorna NULL;
	
	ScrObject *ps=first, *pp=NULL;
	
	while (ps!=NULL)
	{ 
		if ((ps->GetScrDrawArea())==w) pp=ps;
		ps=ps->GetNext();
	} 
	return pp;
}


ScrObject *ScrList::GetActiveNodo()
{
	ScrObject *ps=first, *pp=NULL;
	
	while (ps!=NULL)
	{
		if ((ps->GetScrDrawArea())==WindowActive) 
			pp=ps;
		ps=ps->GetNext();
	} 
	return pp;
}

ScrObject *ScrList::GetNodoButton(Widget w)
{
	ScrObject *scrObj=first;
	int n;
	
	while(scrObj!=NULL)
	{
		n = ((Menu*)scrObj)->GetButton(w);
		if (n>-1) return scrObj;
		scrObj=scrObj->GetNext();
	}
	
	return NULL;
}


EventInfo ScrList::LeftButtonPress(const Point& mousePos)
{ 
	EventResult res=ER_NO_HIT;
	EventInfo evInf;
	ScrObject *scrObj=first;
	
	evInf.eventType=EI_LEFT_PRESS;
	evInf.eventResult=ER_NO_HIT;
	evInf.scrObjHitted=NULL;
	evInf.mousePos=mousePos;
	while(scrObj!=NULL)
	{
		if(scrObj->isInCurrentView() && (res=scrObj->LeftButtonPress(mousePos)!=ER_NO_HIT))
		{
			evInf.eventResult=res;
			evInf.scrObjHitted=scrObj;
			break;
		}
		scrObj=scrObj->GetNext();
	}  
	return evInf;
}

EventInfo ScrList::LeftButtonRelease(const Point& mousePos)
{ 
	EventResult res=ER_NO_HIT;
	EventInfo evInf;
	ScrObject *scrObj=first;
	
	evInf.eventType=EI_LEFT_RELEASE;
	evInf.eventResult=ER_NO_HIT;
	evInf.scrObjHitted=NULL;
	evInf.mousePos=mousePos;
	while(scrObj!=NULL)
	{
		if((res=scrObj->LeftButtonRelease(mousePos))!=ER_NO_HIT)
		{
			if ((scrObj->GetID()==CL_MENU) && (((Menu*)scrObj)->GetClose()))
				Delete(scrObj);
			evInf.eventResult=res;
			evInf.scrObjHitted=scrObj;
			break;
		}
		scrObj=scrObj->GetNext();
	}  
	return evInf;
}

EventInfo ScrList::RightButtonPress(const Point& mousePos)
{ 
	EventResult res=ER_NO_HIT;
	EventInfo evInf;
	ScrObject *scrObj=first;
	
	evInf.eventType=EI_RIGHT_PRESS;
	evInf.eventResult=ER_NO_HIT;
	evInf.scrObjHitted=NULL;
	evInf.mousePos=mousePos;
	while(scrObj!=NULL)
	{
		if((res=scrObj->RightButtonPress(mousePos))!=ER_NO_HIT)
		{
			evInf.eventResult=res;
			evInf.scrObjHitted=scrObj;
			break;
		}
		scrObj=scrObj->GetNext();
	}  
	return evInf;
}

EventInfo ScrList::RightButtonRelease(const Point& mousePos)
{ 
	EventResult res=ER_NO_HIT;
	EventInfo evInf;
	ScrObject *scrObj=first;
	
	evInf.eventType=EI_RIGHT_RELEASE;
	evInf.eventResult=ER_NO_HIT;
	evInf.scrObjHitted=NULL;
	evInf.mousePos=mousePos;
	while(scrObj!=NULL)
	{
		if((res=scrObj->RightButtonRelease(mousePos))!=ER_NO_HIT)
		{
			evInf.eventResult=res;
			evInf.scrObjHitted=scrObj;
			break;
		}
		scrObj=scrObj->GetNext();
	}  
	return evInf;
}

EventInfo ScrList::KeyPressed(int key)
{ 
	EventResult res=ER_NO_HIT;
	EventInfo evInf;
	ScrObject *scrObj=first;
	
	evInf.eventType=EI_KEY_PRESS;
	evInf.eventResult=ER_NO_HIT;
	evInf.scrObjHitted=NULL;
	evInf.key=key;
	while(scrObj!=NULL)
	{
		if((res=scrObj->KeyPressed(key))!=ER_NO_HIT)
		{
			evInf.eventResult=res;
			evInf.scrObjHitted=scrObj;
			break;
		}
		scrObj=scrObj->GetNext();
	}  
	return evInf;
}


EventInfo ScrList::SignalEvent()
{
	EventInfo evInf;
	Point mousePos;
	int buttonStatus;
	Bool event=FALSE;
	
	if(!event)
	{
		buttonStatus=Scr->GetMouse()->Check();
		mousePos=Scr->GetMouse()->GetPos();
		event=FALSE;
		if(buttonStatus & LEFTGODOWN)
		{ 
			evInf=LeftButtonPress(mousePos);
			event=TRUE;
		}
		else if(buttonStatus & LEFTGOUP)
		{
			evInf=LeftButtonRelease(mousePos);
			event=TRUE;
		}
		else if(buttonStatus & RIGHTGODOWN)
		{
			evInf=RightButtonPress(mousePos);
			event=TRUE;
		}
		else if(buttonStatus & RIGHTGOUP)
		{
			evInf=RightButtonRelease(mousePos);
			event=TRUE;
		}
		if(Scr->KeyHit())
		{
			int key=Scr->WaitKey();
			
			evInf=KeyPressed(key);
			event=TRUE;
		}
		evInf.mouseMoved=FALSE;
		
		// 
		// Controlla se il mouse e' stato mosso
		//
		if(Scr->GetMouse()->IsMoved())
		{ 
			evInf.mouseMoved=TRUE;
			if(!event)
			{
				evInf.eventType=EI_MOUSE_MOVE;
				evInf.eventResult=ER_NO_HIT;
				evInf.mousePos=mousePos;
				evInf.key=0;
				evInf.scrObjHitted=NULL;
				event=TRUE;
			}
		}
	}
	return evInf;
}



EventInfo ScrList::WaitEvent()
{
	EventInfo evInf;
	Point mousePos;
	int buttonStatus;
	Bool event=FALSE;
	
	while(!event)
	{
		buttonStatus=Scr->GetMouse()->Check();
		mousePos=Scr->GetMouse()->GetPos();
		event=FALSE;
		if(buttonStatus & LEFTGODOWN)
		{ 
			evInf=LeftButtonPress(mousePos);
			event=TRUE;
		}
		else if(buttonStatus & LEFTGOUP)
		{
			evInf=LeftButtonRelease(mousePos);
			event=TRUE;
		}
		else if(buttonStatus & RIGHTGODOWN)
		{
			evInf=RightButtonPress(mousePos);
			event=TRUE;
		}
		else if(buttonStatus & RIGHTGOUP)
		{
			evInf=RightButtonRelease(mousePos);
			event=TRUE;
		}
		if(Scr->KeyHit())
		{
			int key=Scr->WaitKey();
			
#ifdef _DEBUG      
			if(key==27)
			{ 
				Scr->GClose();
				exit(1);
			}
#endif
			evInf=KeyPressed(key);
			event=TRUE;
		}
		evInf.mouseMoved=FALSE;
		
		// 
		// Controlla se il mouse e' stato mosso
		//
		if(Scr->GetMouse()->IsMoved())
		{ 
			evInf.mouseMoved=TRUE;
			if(!event)
			{
				evInf.eventType=EI_MOUSE_MOVE;
				evInf.eventResult=ER_NO_HIT;
				evInf.mousePos=mousePos;
				evInf.key=0;
				evInf.scrObjHitted=NULL;
				event=TRUE;
			}
		}
	}
	return evInf;
}

void ScrList::ManageEvents()
{
	
	while(WaitEvent().eventResult!=ER_END)
		;
	
}

void ScrList::DrawAll()
{
	ScrObject *scrObj=first;
	
	while(scrObj!=NULL)
	{ 
		scrObj->Draw();
		scrObj=scrObj->GetNext();
	}
	
	Scr->GSetMainDrawArea();
} 

