/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: scrlist.hpp,v 1.1 2005/07/25 16:30:34 bellinpo Exp $ */


#ifndef SCRLIST_HPP
#define SCRLIST_HPP

#include "drw/scrobjec.hpp"

enum EventType {
	EI_LEFT_PRESS=0,
		EI_LEFT_RELEASE=1,
		EI_RIGHT_PRESS=2,
		EI_RIGHT_RELEASE=3,
		EI_KEY_PRESS=4,
		EI_MOUSE_MOVE=5
};

struct EventInfo
{
	EventType eventType;
	EventResult eventResult;
	ScrObject *scrObjHitted;
	int key;
	Point mousePos;
	Bool mouseMoved;
};


class ScrList 
{
protected:
    ScrObject *first,*last;
public:
    ScrList();
    
    void Add(ScrObject*);
    ScrObject *DropFirst();
	
    void Delete(ScrObject *);
	// toglie il nodo puntato dall'argomento.
    void DeleteAll();
	// dealloca tutta la lista.
	
	ScrObject *GetFirst();
    // ritorna il puntatore al primo elemento
	ScrObject *GetNodoItemID(int itemID);
    // ritorna il puntatore al menu' avente itemID come menuItemID
	// altrimenti ritorna NULL
    ScrObject *GetNodoW(Widget w);
	// ritorna il puntatore al menu' la cui ScrWindow vale w
	// altrimenti ritorna NULL
    ScrObject *GetNodoDA(Widget w);
	// ritorna il puntatore al menu' il cui ScrDrawArea vale w
	// altrimenti ritorna NULL
    ScrObject *GetActiveNodo();
	// ritorna il puntatore al nodo la cui ScrDrawArea e' uguale a
	// WindowActive(che e' una variabile globale) altrimenti ritorna NULL
    ScrObject *GetNodoButton(Widget w);
	// ritorna il puntatore al menu' che contiene il pulsante con ScrWindow
	// uguale a w altrimenti ritorna NULL
	
    EventInfo LeftButtonPress(const Point&);
    EventInfo LeftButtonRelease(const Point&);
    EventInfo RightButtonPress(const Point&);
    EventInfo RightButtonRelease(const Point&);
    EventInfo KeyPressed(int);
	
    EventInfo SignalEvent();
    EventInfo WaitEvent();
	
    void ManageEvents();
    
    void DrawAll();
};

#endif
