/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: slur.cpp,v 1.1 2005/07/25 16:30:34 bellinpo Exp $ */



#include "stdio.h"
#include "stdlib.h"
#include "string.h"

#include "drw/point.hpp"  
#include "drw/slur.hpp"
#include "drw/def_drw.hpp"

/**************************************************************/
void Slur::SetValDefault(void)
{
	/* valor di default per fare BOW UP */
	BowUp			= TRUE;    /* 1 or 0, TRUE or FALSE */    
	MultiVoice		= FALSE;   /* 1 or 0, TRUE or FALSE */
	Voice			= VOICE_UPPER; /* UPPER, LOWER, MID */
	Interm			= FALSE;   /* 1 or 0, TRUE or FALSE */  
	StemFirst		= STEM_UP; 
	StemLast		= STEM_UP; 
	StemIntermed	= STEM_UP; 
	ForceBow		= TRUE;
	res = RESDEFAULT;
	lineType		= SOLID_LINE;
	
	/* Decommentare per avere BOW DOWN come default */
	// StemLast     = STEM_DOWN; 
	deforcustom=SLUR_DEFAULT;
	sDIS1y=sDIS2y=1.0;
	beta=0;
}

/**************************************************************/
Slur::Slur()
{
	SetValDefault();
}

/**************************************************************/
VUnit Slur::GetVU2Up()
{
	return 0;
}

/**************************************************************/
VUnit Slur::GetVU2Dwn()
{
	return 0;
}

/**************************************************************/
VUnit Slur::GetVU2Lft()
{
	return 0;
}

/**************************************************************/
VUnit Slur::GetVU2Rgt()
{
	return 0;
}

/**************************************************************/
Slur::Slur(double x[2], double y[2], int col)
{
	double tmp;
	SetValDefault();
	xv[0]=x[0];  yv[0]=y[0]; 
	xv[3]=y[1];  yv[3]=y[1]; 
	xv[1]=xv[2]=yv[1]=yv[2]=0;
	
	status=CURVEINVALID;
	res = RESDEFAULT;
	
	/* riordino i punti if needed */
	if (xv[0]>xv[3]) 
	{ tmp=xv[0]; xv[0]=xv[3]; xv[3]=tmp; 
    tmp=yv[0]; yv[0]=yv[3]; yv[3]=tmp; 
	}
	lineType		= SOLID_LINE;
}

/**************************************************************/
Slur::Slur( Point p1, Point p4, int col)
{
	double tmp;
	xv[0]=p1.x;  yv[0]=p1.y; 
	xv[3]=p4.x;  yv[3]=p4.y; 
	xv[1]=xv[2]=yv[1]=yv[2]=0;
	
	status=CURVEINVALID;
	res = RESDEFAULT;
	SetColor((ColorIcon)col);
	
	/* riordino i punti */
	if (xv[0]>xv[3]) 
	{ tmp=xv[0]; xv[0]=xv[3]; xv[3]=tmp; 
    tmp=yv[0]; yv[0]=yv[3]; yv[3]=tmp; 
	}
	deforcustom=SLUR_DEFAULT;
	lineType		= SOLID_LINE;
	sDIS1y=sDIS2y=1.0;
}

/**************************************************************/
Slur::Slur( Point p1, Point p4, 
		   double DISp1x, double DISp1y, double DISp2x, double DISp2y, int col)
{
	double tmp;
	xv[0]=p1.x;  yv[0]=p1.y; 
	xv[3]=p4.x;  yv[3]=p4.y; 
	xv[1]=xv[2]=yv[1]=yv[2]=0;
	
	status=CURVEINVALID;
	res = RESDEFAULT;
	SetColor((ColorIcon)col);
	
	/* riordino i punti */
	if (xv[0]>xv[3]) 
	{ tmp=xv[0]; xv[0]=xv[3]; xv[3]=tmp; 
    tmp=yv[0]; yv[0]=yv[3]; yv[3]=tmp; 
	}
	
	ImposeXYDIS(DISp1x, DISp1y, DISp2x, DISp2y);
	lineType		= SOLID_LINE;
}


/**************************************************************/
void Slur::SetSlur( Point p1, Point p4, const Colore& col)
{
	double tmp;
	xv[0]=p1.x;  yv[0]=p1.y; 
	xv[3]=p4.x;  yv[3]=p4.y; 
	xv[1]=xv[2]=yv[1]=yv[2]=0;
	
	lineType = SOLID_LINE;
	status=CURVEINVALID;
	res = RESDEFAULT;
	SetColor(col);
	
	/* riordino i punti */
	if (xv[0]>xv[3]) 
	{ tmp=xv[0]; xv[0]=xv[3]; xv[3]=tmp; 
    tmp=yv[0]; yv[0]=yv[3]; yv[3]=tmp; 
	}
	deforcustom=SLUR_DEFAULT;
}

/**************************************************************/
void Slur::Draw(void)
{
	double ind,bg2;
	double bg=0.1; // begin double resolution of the slur
	int x,y;
	double inc;
	
	if(isOpaque)
		Scr->SetColor(GetColor());
	
	EvalXYDIS();
	
	if (BowUp) EvalP1P2Up();
	else EvalP1P2Down();
	if (status==CURVEINVALID) return;
	
	int step1=0 , step2=1 ;	//indicatori per l'alternanza fra il moveto ed il lineto (RC)
	
	switch (lineType) {
		//scelgo il tipo di linea attraverso la varibile "lineType" (RC)
		
	case DASHED_LINE:
		
		// tratteggiato
		
		bg2=0.2;
		/* cambiati per accelerare NESprofile */
		if (dist>60) { res=0.03 ;}
		if (dist>120) { res=0.02 ;}
		if (dist>240) { res=0.015 ;}
		if (dist>320) { res=0.009; bg=0.05; }
		if (dist>360) { res=0.006; bg=0.05 ;}
		
		Scr->MoveTo((short)xv[0],(short)yv[0]);
		
		for (ind=0;ind<=1;ind=ind+res)
		{
			x = (short) (ind*xv[3] + xv[0]);
			y = (short) (this->YdiT(ind) + yv[0]);
			if (step1<=3 && step2%2) 
			{
				Scr->LineTo((short)x,(short)y,SOLID_LINE);
				step1++ ;
			}
			else
			{ 
				Scr->MoveTo((short)x,(short)y);
				if (step1>3) 
				{
					step2++;
					if (step2%2) step1 = 0; 
				}
				else 
				{
					step1++ ;
				}
			}
		}
		break;
		
		
	case DOTTED_LINE:
		
		// punteggiato
		
		bg2=0.2 ;
		if (dist>0) { res=0.015 ;}
		if (dist>40) { res=0.012;}
		if (dist>60) { res=0.0086;}
		if (dist>75) { res=0.0076;}
		if (dist>90) { res=0.0064;}
		if (dist>120) { res=0.0058;}
		if (dist>140) { res=0.0052;}
		if (dist>180) { res=0.0044;}
		if (dist>280) { res=0.0036;}
		if (dist>320) { res=0.0030; bg=0.05 ; }
		if (dist>340) { res=0.0024; bg=0.05 ; }
		if (dist>360) { res=0.002; bg=0.05 ;}
		if (dist>400) { res=0.0018; bg=0.05 ;}
		
		
		
		Scr->MoveTo((short)xv[0],(short)yv[0]);
		
		for (ind=0;ind<=1;ind=ind+res)
		{
			x = (short) (ind*xv[3] + xv[0]);
			y = (short) (this->YdiT(ind) + yv[0]);
			if (step1<=1) 
			{
				Scr->LineTo((short)x,(short)y,SOLID_LINE);
				step1++ ;
			}
			else
			{ 
				Scr->MoveTo((short)x,(short)y);
				if (step1>7) 
				{
					step1=0 ; 
				}
				else {step1++ ;}
			}
		}
		
		
		break;
		
		
	case SOLID_LINE:
		
		// linea continua
		
		bg2=0.2;
		if (dist>80) { res=0.025; bg=0.05; }
		else if (dist>200) {res=0.0125; bg=0.025; }
		else if (dist>500) {res=0.0085; bg=0.0125; }
		else if (dist>800) {res=0.0055; bg=0.0085; }
		
		
		Scr->MoveTo((short)xv[0],(short)yv[0]);
		for (ind=0;ind<=1;ind=ind+res)
		{
			x = (short) (ind*xv[3] + xv[0]);
			y = (short) (this->YdiT(ind) + yv[0]);
			Scr->LineTo((short)x,(short)y,SOLID_LINE);
		}
		
		Scr->MoveTo((short)(xv[0]+bg*xv[3]),(short)(yv[0]+this->YdiT(bg)));
		for (ind=bg+res;ind<=1-bg;ind=ind+res)
		{
			x = (short) (ind*xv[3] + xv[0]);
			if (BowUp) inc=0.75; else inc = -0.75;
			y = (short) (this->YdiT(ind) + yv[0]+inc);
			Scr->LineTo((short)x,(short)y,SOLID_LINE);
		}
		
		Scr->MoveTo((short)(xv[0]+bg2*xv[3]),(short)(yv[0]+this->YdiT(bg2)));
		for (ind=bg2+res;ind<=1-bg2;ind=ind+res)
		{
			x = (short) (ind*xv[3] + xv[0]);
			if (BowUp) inc=1; else inc =-1;
			y = (short) (this->YdiT(ind) + yv[0]+inc);
			Scr->LineTo((short)x,(short)y,SOLID_LINE);
		}
		break;
  }
  
}

/**************************************************************/
Bool Slur::hit(const Point &p)
{
	double ind,bg2,dst,sogliadst;
	double bg=0.1; // begin double resolution of the slur
	int x,y;
	
	EvalXYDIS();
	
	if (BowUp) EvalP1P2Up();
	else EvalP1P2Down();
	if (status==CURVEINVALID) return CURVEINVALID;
	
	sogliadst=2.5;
	
	switch (lineType) {
		
	case DASHED_LINE:
		bg2=0.2;
		if (dist>60) { res=0.03 ;}
		if (dist>120) { res=0.02 ;}
		if (dist>240) { res=0.015 ;}
		if (dist>320) { res=0.009; bg=0.05; }
		if (dist>360) { res=0.006; bg=0.05 ; sogliadst=4;}
		break;
	case DOTTED_LINE:
		// punteggiato	
		bg2=0.2 ;
		if (dist>0) { res=0.015 ;}
		if (dist>40) { res=0.012;}
		if (dist>60) { res=0.0086;}
		if (dist>75) { res=0.0076;}
		if (dist>90) { res=0.0064;}
		if (dist>120) { res=0.0058;}
		if (dist>140) { res=0.0052;}
		if (dist>180) { res=0.0044;}
		if (dist>280) { res=0.0036;}
		if (dist>320) { res=0.0030; bg=0.05 ; }
		if (dist>340) { res=0.0024; bg=0.05 ; }
		if (dist>360) { res=0.002; bg=0.05 ;}
		if (dist>400) { res=0.0018; bg=0.05 ; sogliadst=4;}
		break;	
	default:
	case SOLID_LINE:
		// linea continua
		bg2=0.2;
		if (dist>80) { res=0.025; bg=0.05; }
		else if (dist>200) {res=0.0125; bg=0.025; }
		else if (dist>500) {res=0.0085; bg=0.0125;  sogliadst=3;}
		else if (dist>800) {res=0.0055; bg=0.0085;  sogliadst=4;}
		break;
	}
	
	for (ind=0;ind<=1;ind=ind+res)
	{
		x = (short) (ind*xv[3] + xv[0]);
		y = (short) (this->YdiT(ind) + yv[0]);
		dst = sqrt(double((p.x - x)*(p.x-x) + (p.y - y)*(p.y-y)));
		if (dst<=sogliadst) return (TRUE);
	}
	
	return(FALSE);
}

/******************************************************************/
void Slur::SetSlurParameter(int XMultiVoice, int XVoice, int XStemFirst, int
							XStemLast, int XStemIntermed, int XIntermed) 
{
	MultiVoice=XMultiVoice;
	Voice=XVoice;
	Interm=XIntermed;
	StemFirst=XStemFirst; 
	StemLast=XStemLast; 
	StemIntermed=XStemIntermed;
}

/**************************************************************/
void Slur::EvalBowOrientation(void)
{
	if (MultiVoice && Voice==VOICE_UPPER)  BowUp = TRUE;
	else if (MultiVoice && Voice==VOICE_LOWER) BowUp = FALSE;
	else if (StemUpAll()) BowUp = FALSE;
	else BowUp = TRUE;
}

/*************************************************************/
void Slur::SetBowUp(int a)
{
	BowUp = a;
}  

/********************************************************************/  
int Slur::GetBowUp()
{
	return BowUp;
}

/**************************************************************/
int Slur::StemUpAll(void)
{
	int val;
	
	if (Interm && StemFirst==STEM_UP && StemLast==STEM_UP && StemIntermed==STEM_UP)
		val=TRUE;
	else if (StemFirst==STEM_UP && StemLast==STEM_UP) val=TRUE;
	else val=FALSE;
	
	return(val);
}

/**************************************************************/
void Slur::ForceBowOrientation(int a)
{
	ForceBow=TRUE; /* orientamento forzato */
	BowOrForced=a; /* BOW_UP or  BOW_DOWN */
}

/**************************************************************/
void Slur::SetAutoOrientation(void)
{
	ForceBow=FALSE; /* orientamento  automatico */
}

/**************************************************************/
void Slur::SetSlurCustom(void)
{
	deforcustom=SLUR_CUSTOM;
}

/**************************************************************/
void Slur::SetSlurDefault(void)
{
	deforcustom=SLUR_DEFAULT;
}

/**************************************************************/
void Slur::EvalXYDISDefault(void)
{
	/* calcolo dell'angolo dello slur */
	if (xv[3]!=xv[0])  beta=atan((yv[3]-yv[0])/(xv[3]-xv[0])); 
	else beta=0.0;
	
	/* distanza fra punto finale ed iniziale */
	dist = sqrt((xv[3]-xv[0])*(xv[3]-xv[0]) + (yv[3]-yv[0])*(yv[3]-yv[0]));
	
	/* calcola XDIS ed YDIS nell'ottica di XC ed YC */ 
	if (dist < 5*PIXELPERSTAFF) YDIS=0.2*dist;
	else if (dist < 25.0*PIXELPERSTAFF) YDIS=((dist/PIXELPERSTAFF+5)/10)*PIXELPERSTAFF;
	//era diviso 10
	else  YDIS=3.0*PIXELPERSTAFF;
	// era per 3
	
	if (dist < 4.0*PIXELPERSTAFF) XDIS=0.3*dist - 0.1*PIXELPERSTAFF;
	else if (dist < 25.0*PIXELPERSTAFF) XDIS=((3.0*dist/PIXELPERSTAFF+15.5)/20)*PIXELPERSTAFF;
	else  XDIS=3.0*PIXELPERSTAFF;
	
	// NESNES
	DIS1[0]=XDIS;
	DIS1[1]=YDIS*sDIS1y;
	DIS2[0]=XDIS;
	DIS2[1]=YDIS*sDIS2y;
	
	// si noti che questi valori dovrebbero cambiare anche con la giustificazione e con
	// il line breaking. Infatti cambiano, al momento  quando  custom che non cambiano
	// andrebbero espressi in percentuale. In effetti nel computo dello slur vengono 
	// considerati solo i valori di YDIS.
}

/**************************************************************/
void Slur::EvalXYDIS(void)
{
	//ora if (deforcustom!=SLUR_DEFAULT) return;
	
	EvalXYDISDefault();
}

/**************************************************************/
void Slur::GetDefaultDISValues
(double * DISp1x, double * DISp1y, double * DISp2x, double * DISp2y)
{
	EvalXYDISDefault();
	*DISp1x=DIS1[0];
	*DISp1y=sDIS1y; //ora DIS1[1];
	*DISp2x=DIS2[0];
	*DISp2y=sDIS2y;  //ora DIS2[1];
}

/**************************************************************/
// NESNES, da utilizzare al posto di EVALXYDIS se la slur  custom
// o meglio se la slur  custom EVALXYDIS non deve essere toccato 
// questo metodo serve in fase di editing ed in fase di load per settare 
// i punti di controllo
void Slur::ImposeXYDIS(double DISp1x, double DISp1y, double DISp2x, double DISp2y)
{
	
	DIS1[0]=DISp1x;
	
	//ora DIS1[1]
	sDIS1y=DISp1y;
	
	DIS2[0]=DISp2x;
	
	//ora DIS2[1]
	sDIS2y=DISp2y;
	
	deforcustom=SLUR_CUSTOM;
}

/**************************************************************/
void Slur::EvalP1P2Up(double ypos) /* \-----/ */
{ int ret;

xv[1]=xv[0] + DIS1[0]; 
yv[1]=yv[0] + DIS1[1] + ypos;

xv[2]=xv[0] + dist - DIS2[0];
yv[2]=yv[0] + DIS2[1] + ypos;

ret=ChkCurve();
}

/**************************************************************/
int Slur::ChkCurve(void )
{ double tmp;
int i; 
for (i=1;i<4;i++)
{ xv[i]=xv[i]-xv[0];
yv[i]=yv[i]-yv[0];
}
if (xv[2]<xv[1]) 
{  tmp=xv[1]; xv[1]=xv[2]; xv[2]=tmp;
tmp=yv[1]; yv[1]=yv[2]; yv[2]=tmp;
}

status = CURVEVALID; 
return(CURVEVALID); 
}


/**************************************************************/
void Slur::EvalP1P2Down(double ypos) /* /--------\  */
{ int ret;

xv[1]=xv[0] + DIS1[0]; 
yv[1]=yv[0] - DIS1[1] + ypos;

xv[2]=xv[0] + dist - DIS2[0];
yv[2]=yv[0] - DIS2[1] + ypos;

ret=ChkCurve();
}

/**************************************************************/
double Slur::Yval(int num)
{
	return 0.;
}

/**************************************************************/
double Slur::YdiT(double t) /* t varia fra 0 ed 1 */
{
	double val0, val1;
	if (t<0 || t>1) return (OUTOFT);
	val0 = t*t*t*(3.0*yv[1]-3.0*yv[2]+yv[3]*0.0);
	val0 = val0 + t*t*(-6.0*yv[1]+3*yv[2]) + t*3.0*yv[1];
	
	// t =   dist*t*cos(beta) - val0*sin(beta);
	val1 =  val0*cos(beta) + dist*t*sin(beta);
	
	return (val1);
}

/**************************************************************/
void Slur::Test(Point p1, Point p2, int col)
{
	double an,tmp, cx, cy;
	
	Draw();
	
	cx=xv[0];
	cy=yv[0];
	for (an=0;an<2*3.14;an=an+0.2)
	{ 
		xv[0]=cx; yv[0]=cy;
		yv[3]=cy+dist*sin(an);
		xv[3]=cx+dist*cos(an);
		
		/* riordino i punti if needed */
		if (xv[0]>xv[3]) 
		{ tmp=xv[0]; xv[0]=xv[3]; xv[3]=tmp; 
		tmp=yv[0]; yv[0]=yv[3]; yv[3]=tmp; 
		}
		Draw();
	}
}

/******************************************************************/
double Slur::GetStartHigh(void)
{
	return fabs(YdiT(7.0/(dist)));
}


/*********************************************************************/
double Slur::GetEndHigh(void) 
{
	double val;
	
	val=YdiT(1-(7.0/dist));
	if (yv[3]!=0.0)      val=val-yv[3];
	return fabs(val);
}    

Point Slur::GetCP1()
{
	//the control poit is rotated of beta
	double alpha=atan(yv[1]/xv[1]);
	double d=sqrt(yv[1]*yv[1]+xv[1]*xv[1]);
	return Point(xv[0]+d*cos(alpha+beta),yv[0]+d*sin(alpha+beta));
}
Point Slur::GetCP2()
{
	double alpha=atan(yv[2]/xv[2]);
	double d=sqrt(yv[2]*yv[2]+xv[2]*xv[2]);
	return Point(xv[0]+d*cos(alpha+beta),yv[0]+d*sin(alpha+beta));
}

/*********************************************************************/



















