/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: slur.hpp,v 1.1 2005/07/25 16:30:35 bellinpo Exp $ */


#ifndef SLUR_HPP
#define SLUR_HPP
#define VOICE_UPPER 1
#define VOICE_LOWER 2
#define VOICE_MID   0

#define STEM_UP   1
#define STEM_DOWN 0

#define INCPIXEL 96.0  /* 96 pixel per ogni pollice */
#define INCPT    72.0  /* 72 punti tipografici per pollice */
#define PIXELPT  INCPIXEL/INCPT  /* circa 1.33  PT per pixel */

/* distanza fra due linee dello staff */
#define PIXELPERSTAFF 6

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#define BOW_UP     1
#define BOW_DOWN   0

#include "lioo.hpp" 
#include "drw/bezier.hpp"

#define SLUR_CUSTOM 1
#define SLUR_DEFAULT 0
 

class Slur : public Bezier
{
  protected:
    LineType lineType;
/* voices*/
    int MultiVoice; /* 1 or 0, TRUE or FALSE */ // presenza di piu' voci/layer attivi */
    int Voice;      /* UPPER, LOWER, MID */    

/* presenza di note fra la first e la last dello slur */
    int Interm;   /* 1 or 0, TRUE or FALSE */  

/* direzione dello stem */
    int StemFirst;  // prima nota /* these can be STEM_UP or STEM_DOWN */
    int StemLast;   // ultima nota
    int StemIntermed; // note incluse se vi sono 

/* note beamate o meno, solo i teminali dello slur */
    int BeamFirst; /* boolean */
    int BeamLast; /* boolean */

    int ForceBow;  /* flag che dice se vi e' stato imposta l'orientazione del BOW */
    int BowOrForced;  /* BOW_UP or BOW_DOWN */

/************ parametri calcolati e se necessario imposti */
    double YDIS; /* calcolo dei parametri per la stima diretta di P1 e P2 */
    double XDIS;
	double DIS1[2], DIS2[2]; /* coordinate punti di controllo delle slur custom NESNES */
	double sDIS1y, sDIS2y; // fattore di scala 
    int BowUp;    /* 1 or 0, TRUE or FALSE */    /* TRUE cioe' la gobba e' su /---\ */
    double dist; // calcolata come delta x
    double beta; /* angolo di inclinazione dell'arco */

	// slur di tipo default con DIS automatici o CUSTOM con DIS da valori imposti
	// e memorizzati su file
	int deforcustom;
public:
    Slur();
    Slur(double xv[2], double yv[2], int col=NORMAL_COLOR);
    Slur(Point, Point, int col=NORMAL_COLOR);
	Slur( Point p1, Point p4, 
		   double DISp1x, double DISp1y, double DISp2x, double DISp2y, int col=NORMAL_COLOR);
    
    VUnit GetVU2Up();
    VUnit GetVU2Dwn();
    VUnit GetVU2Lft();
    VUnit GetVU2Rgt();
    
    void SetValDefault(void);

	void SetSlurCustom(void);
	void SetSlurDefault(void);
    void SetSlur(Point, Point, const Colore&);
    void SetSlurParameter(int MultiVoice, int Voice, int
				StemFirst, int StemLast, int StemIntermed, int Interm); 


    int StemUpAll(void); /* TRUE se tutti hanno lo STEM_UP */
    int StemUpBounds(void); /* TRUE se tutti hanno lo STEM_UP */


    void SetAutoOrientation(void); // pone l'orientatmento in automatico 
    void EvalBowOrientation(void); // calcola l'orientamento dell'arco in base a parametri 
    void ForceBowOrientation(int); /* TRUE per UP */
    
    void SetBowUp(int a);
    Bool GetBowUp(void);

    double Yval(int num);

	// li calcola solo nel caso di slur non custom
    void EvalXYDIS(void);

	void EvalXYDISDefault(void);
    void GetDefaultDISValues (double * DISp1x, double * DISp1y, double * DISp2x, double * DISp2y);
	void ImposeXYDIS(double DISp1x, double DISp1y, double DISp2x, double DISp2y);

	
    void EvalP1P2Up(double ypos=0);
    void EvalP1P2Down(double ypos=0);
    int ChkCurve(void );

    double YdiT(double t); /* t varia fra 0 ed 1 */
    void Draw(void); 
	void print(int type);   
    Bool hit(const Point &p);
    void Test(Point p1, Point p2, int col);
    double GetStartHigh(void );
    double GetEndHigh(void);
    void SetLineType(LineType ltype) { lineType=ltype; };

	Point GetCP1();
	Point GetCP2();

	void PrintMySlur(int type);
};

#endif





