/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: textio.cpp,v 1.1 2005/07/25 16:30:36 bellinpo Exp $ */


#include "drw/textio.hpp"

#include <string.h>
#include <ctype.h>

#define KEY_BS 8


TextIO::TextIO(int mc,VUnit w)
{
	ID=CL_TEXTIO;
	maxChar=mc;
	edit=FALSE;
	minEditPos=0;
	currPos=0;
	txt=new unsigned char[mc];
	if (!txt)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","TextIO","TextIO");
	
	txt[0]='\0';
	wdt=w;
}

TextIO::~TextIO() 
{ 
	if(txt != NULL)
		delete[] txt; 
};

void TextIO::SetFont(FontID nfnt)
{
	VUnit w=wdt;
	
	Text::SetFont(nfnt);
	wdt=w;
}

void TextIO::DrawCursor()
{
	// N. B. Non usare direttamente Text::GetAbsPos().
	
	VUnit x=VUnit(Scr->GetTextWidth(txt)+2);
	Scr->SetDrwMode(XORMode);
	Colore oldcol = Scr->GetColor();
	Scr->SetColor((ColorIcon)XOR_SELECTED_COLOR);
	Scr->Line(Text::GetAbsPos()+Point(x,-2),
		Text::GetAbsPos()+Point(x,VUnit(Scr->GetFontAscent()-2)),SOLID_LINE);
	Scr->SetDrwMode(DefMode);
	Scr->SetColor(oldcol);
}

EventResult TextIO::KeyPressed(int key)
{
	EventResult er=ER_NO_HIT;
	
	if(!edit)
		return ER_NO_HIT;
	
	Scr->GetMouse()->Hide();
	switch(key)
	{
    case KEY_BS:
		if(currPos>minEditPos)
		{       
			FontID ofont;
			VUnit twdt;
			Rectangle r;
			
			txt[--currPos]='\0';
			ofont=Scr->SetFont(font);
			twdt=Scr->GetTextWidth(txt);
			Colore oldcol = Scr->GetColor();
			Scr->SetColor((ColorIcon)white);
			r=Rectangle(*(Text*)this);
			
			Rectangle oclprgn=Scr->SetClipRegion(r);
			
			r.PUpLft.x+=twdt;
			Scr->Rect(r,TRUE);
			DrawCursor();
			
			Scr->SetColor(oldcol);

			Scr->SetClipRegion(oclprgn);
			
			Scr->SetFont(ofont);
			er=ER_CONTINUE;
		}
		break;
    default:
		if(currPos<maxChar-1 && isprint(key))
		{
			FontID ofont;
			
			ofont=Scr->SetFont(font); 
			
			Rectangle oclprgn=Scr->SetClipRegion(Rectangle(*((Text*)this)));
			
			DrawCursor();
			
			txt[currPos++]=char(key);
			txt[currPos]='\0';
			Scr->SetColor(Text::GetColor());
			Scr->TextTo(Text::GetAbsPos());
			Scr->OutText((unsigned char *)txt);
			
			DrawCursor();
			
			Scr->SetClipRegion(oclprgn);
			Scr->SetFont(ofont);
			er=ER_CONTINUE;
		}
	}
	Scr->GetMouse()->Show();
	return er;
}

void TextIO::SetMessage(unsigned char *t)
{
	edit=FALSE;
	strncpy((char *)txt,(char *)t,maxChar);
	txt[maxChar-1]='\0';
	currPos=minEditPos=0;
	
	Scr->GetMouse()->Hide();
	Draw();
	Scr->GetMouse()->Show();
}

void TextIO::EditMessage(unsigned char *t)
{
	edit=TRUE;
	strncpy((char *)txt, (char *)t,maxChar);
	txt[maxChar-1]='\0';
	currPos=strlen((char *)txt);
	minEditPos=currPos;
	
	Scr->GetMouse()->Hide();
	Draw();
	Scr->GetMouse()->Show();
}

void TextIO::Draw()
{
	Rectangle r;
	
	Colore oldcol = Scr->GetColor();
	Scr->SetColor((ColorIcon)white);
	r=Rectangle(*((Text*)this));
	
	Rectangle oclprgn=Scr->SetClipRegion(r);
	
	Scr->Rect(r,TRUE);
	FontID ofont=Scr->SetFont(font);
	Text::Draw();
	if(edit)
		DrawCursor();
	
	Scr->SetClipRegion(oclprgn);
    
	Scr->ShadowRect(r.Expand(1,1),FALSE);
	Scr->ShadowRect(r.Expand(4,4),TRUE);
	Scr->SetFont(ofont);  

	Scr->SetColor(oldcol);
}     

unsigned char *TextIO::GetMessage() 
{ 
	return &txt[minEditPos]; 
};
