/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: toolbar.hpp,v 1.1 2005/07/25 16:30:37 bellinpo Exp $ */


#ifndef TOOLBAR_HPP
#define TOOLBAR_HPP

#include <wx/wx.h>
#include <wx/laywin.h>
#include <wx/bitmap.h>
#include <wx/textctrl.h>
#include <wx/button.h>



#if defined(__WXGTK__) || defined(__WXMOTIF__)
#include "bitmaps/toolbar/top.xpm"   //tool fixed
#include "bitmaps/toolbar/meno1pag.xpm"
#include "bitmaps/toolbar/meno5.xpm"
#include "bitmaps/toolbar/meno1.xpm"
#include "bitmaps/toolbar/piu1.xpm"
#include "bitmaps/toolbar/piu5.xpm"
#include "bitmaps/toolbar/piu1pag.xpm"
#include "bitmaps/toolbar/bottom.xpm"
#include "bitmaps/toolbar/jump.xpm"

#include "bitmaps/toolbar/editing/chordnote/chord.xpm"   //tool  for editing
#include "bitmaps/toolbar/editing/chordnote/note.xpm"
#include "bitmaps/toolbar/editing/noteheads/classic.xpm"
#include "bitmaps/toolbar/editing/noteheads/circlex.xpm"
#include "bitmaps/toolbar/editing/noteheads/croix.xpm"
#include "bitmaps/toolbar/editing/noteheads/x.xpm"
#include "bitmaps/toolbar/editing/noteheads/dsharp.xpm"
#include "bitmaps/toolbar/editing/noteheads/fdiamond.xpm"
#include "bitmaps/toolbar/editing/noteheads/ediamond.xpm"
#include "bitmaps/toolbar/editing/noteheads/f_rythmic.xpm"
#include "bitmaps/toolbar/editing/noteheads/e_rythmic.xpm"
#include "bitmaps/toolbar/editing/noteheads/rythmic.xpm"
#include "bitmaps/toolbar/editing/noteheads/letter_1.xpm"
#include "bitmaps/toolbar/editing/noteheads/letter_2.xpm"
#include "bitmaps/toolbar/editing/noteheads/rnumber1.xpm"
#include "bitmaps/toolbar/editing/noteheads/rnumber2.xpm"
#include "bitmaps/toolbar/editing/noteheads/rnumber3.xpm"
#include "bitmaps/toolbar/editing/linesstyle/continuous.xpm"
#include "bitmaps/toolbar/editing/linesstyle/dashed.xpm"
#include "bitmaps/toolbar/editing/linesstyle/dotted.xpm"
#include "bitmaps/toolbar/editing/menospace.xpm"
#include "bitmaps/toolbar/editing/piuspace.xpm"
#include "bitmaps/toolbar/editing/delete.xpm"

#include "bitmaps/toolbar/execution/compile.xpm"   //tool for execution
#include "bitmaps/toolbar/execution/init.xpm"
#include "bitmaps/toolbar/execution/start.xpm"
#include "bitmaps/toolbar/execution/pause.xpm"
#include "bitmaps/toolbar/execution/stop.xpm"
#include "bitmaps/toolbar/execution/meno3.xpm"
#include "bitmaps/toolbar/execution/piu3.xpm"
#endif 


enum
{
	SASHLAYOUTWINDOW_ID = 1,	//toolbar
		
		TOOL_MESSAGE_ID,	//tool fixed
		TOOL_EDITEXEC_ID,
		TOOL_TOP_ID,
		TOOL_MENO1PAG_ID,
		TOOL_MENO5_ID,
		TOOL_MENO1_ID,
		TOOL_PIU1_ID,
		TOOL_PIU5_ID,
		TOOL_PIU1PAG_ID,
		TOOL_BOTTOM_ID,
		TOOL_JUMP_ID,
		
		SEPARATOR_ID,   //separator
		
		EXEC_TOOL_COMPILE_ID,	//tool for execution
		EXEC_TOOL_INIT_ID,
		EXEC_TOOL_START_ID,
		EXEC_TOOL_PAUSE_ID,
		EXEC_TOOL_STOP_ID,
		EXEC_TOOL_MENO3_ID,
		EXEC_TOOL_STATUS_ID,
		EXEC_TOOL_STATUSPERC_ID,
		EXEC_TOOL_PIU3_ID,
		
		EDIT_TOOL_LAYER_ID,	//tool for editing
		EDIT_TOOL_CHORDNOTE_ID,
		
		EDIT_TOOL_NOTEHEAD_ID,
		EDIT_TOOL_COLOR_ID,
		EDIT_TOOL_LINESSTYLE_ID,
		EDIT_TOOL_AUTO_ID,
		EDIT_TOOL_MENOSPACE_ID,
		EDIT_TOOL_PIUSPACE_ID,
		EDIT_TOOL_EDIT_ID,
		EDIT_TOOL_DELETE_ID
};


enum Notehead_type
{
	classic, circlex, croix, x, dsharp, fdiamond, ediamond, 
		frythmic, erythmic, rythmic, letter1, letter2, rnumber1, rnumber2, rnumber3
};

enum LinesStyle_type
{
	continuous, dashed, dotted
};



class ToolBar : public wxSashLayoutWindow
{
protected:
	wxString type;  //toolbar type: "editing" or "execution"
	
	//tool fissi
	wxTextCtrl *messagedisplay;
	wxButton *editexec;
	wxBitmapButton *top;
	wxBitmapButton *meno1pag;
	wxBitmapButton *meno5;
	wxBitmapButton *meno1;
	wxBitmapButton *piu1;
	wxBitmapButton *piu5;
	wxBitmapButton *piu1pag;
	wxBitmapButton *bottom;
	wxBitmapButton *jump;
	
	wxButton *separator;  //separator between tool fixed and others
	
	//tool per l'editing
	wxButton *edit_layer;
	wxBitmapButton *edit_chordnote;
	wxBitmapButton *edit_notehead;
	wxButton *edit_color;
	wxBitmapButton *edit_linesstyle;
	wxButton *edit_auto;
	wxBitmapButton *edit_menospace;
	wxBitmapButton *edit_piuspace;
	wxButton *edit_edit;
	wxBitmapButton *edit_delete;
	
	//tool per l'executing
	wxBitmapButton *exec_compile;
	wxBitmapButton *exec_init;
	wxBitmapButton *exec_start;
	wxBitmapButton *exec_pause;
	wxBitmapButton *exec_stop;
	wxBitmapButton *exec_meno3;
	wxTextCtrl *exec_status;
	wxTextCtrl *exec_statusperc;
	wxBitmapButton *exec_piu3;
	
	
	Notehead_type current_notehead;
	LinesStyle_type current_linesstyle;
	
	
public:
	ToolBar(const wxString &toolbartype = "editing" ,wxWindow* parent = NULL ,wxWindowID id = -1, 
	       const wxPoint& pos = wxDefaultPosition , const wxSize& size = wxDefaultSize, 
		   long style = wxCLIP_CHILDREN |  wxSW_3D, const wxString& name = "layoutWindow");
	
	~ToolBar();
	
	wxString Gettype(void) { return type;};
	void Settype (wxString str) { type = str;};
	wxTextCtrl* Getmessagedisplay(void);
	wxTextCtrl* Getexec_status(void);
	wxTextCtrl* Getexec_statusperc(void);
	wxButton* Getedit_layer(void) { return edit_layer;};
	
	
	void OnEditexec(wxCommandEvent& event);
	void OnTop(wxCommandEvent& event);
	void OnMeno1pag(wxCommandEvent& event);
	void OnMeno5(wxCommandEvent& event);
	void OnMeno1(wxCommandEvent& event);
	void OnPiu1(wxCommandEvent& event);
	void OnPiu5(wxCommandEvent& event);
	void OnPiu1pag(wxCommandEvent& event);
	void OnBottom(wxCommandEvent& event);
	void OnJump(wxCommandEvent& event);
	
	//methods for editing
	void OnEdit_layer(wxCommandEvent& event);
	void OnEdit_chordnote(wxCommandEvent& event);
	void OnEdit_notehead(wxCommandEvent& event);
	void OnEdit_color(wxCommandEvent& event);
	void OnEdit_linesstyle(wxCommandEvent& event);
	void OnEdit_linesstyle_dashed(wxCommandEvent& event);
	void OnEdit_auto(wxCommandEvent& event);
	void OnEdit_menospace(wxCommandEvent& event);
	void OnEdit_piuspace(wxCommandEvent& event);
	void OnEdit_edit(wxCommandEvent& event);
	void OnEdit_delete(wxCommandEvent& event);
	
	//methods for executing
	void OnExec_compile(wxCommandEvent& WXUNUSED(event));
	void OnExec_init(wxCommandEvent& event);
	void OnExec_start(wxCommandEvent& event);
	void OnExec_pause(wxCommandEvent& event);
	void OnExec_stop(wxCommandEvent& event);
	//  void OnExec_jump(wxCommandEvent& event);
	void OnExec_meno3(wxCommandEvent& event);
	void OnExec_piu3(wxCommandEvent& event);
	LinesStyle_type Getcurrent_linesstyle();
	
	
	DECLARE_EVENT_TABLE();
};


#endif // TOOLBAR_HPP
