/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: accordo.cpp,v 1.1 2005/07/25 16:30:38 bellinpo Exp $ */

#include "liooext.hpp"
#include "fig/accordo.hpp"
#include "spa/spartito.hpp"
#include "fig/ncroma.hpp"

Accordo::Accordo()
{ 
	Init();
}

Accordo::~Accordo()
{ 
	Free();
}

void Accordo::Init()
{
	// Usa l'inizializzazione della classe Nota.
	Nota::Init();
	ID=CL_ACCORDO;                               
	SetDiteggiatoUp(TRUE);
	SetPArpeggio(NULL);
	// Imposta la lista vuota.
	first=NULL;
	last=NULL;
	numobject=0;
	CodeCounter=1;
}


ClassID Accordo::GetNoteID()
{
	return NoteID;
}


void Accordo::SetDiteggiatoUp(Bool dup)
{
	DiteggiatoUp=dup;
}


Bool Accordo::GetDiteggiatoUp()
{
	return DiteggiatoUp;
}


void Accordo::SetPArpeggio(Arpeggio *pa)
{
	PArp = pa;
}

Arpeggio *Accordo::GetPArpeggio()
{
	return PArp;
}

Bool Accordo::AddNota(Nota *pn)
{   
	Node *pnode;        
	Nota *pnota;
	Bool found;
    char alt;
    short nstaff; 
    
	
	pn->SetParentFig(this);
	
	Node *NewNode=new Node(pn);
	if(!NewNode)
        EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Accordo","AddNota()");
	
	if(isEmpty())      
	{   
		// Lista vuota => crea la lista.
		first=NewNode;
		last=NewNode;
		NewNode->Next=NULL;
		NewNode->Prev=NULL;      
		// Imposta GetAbsPos(), Altezza e NoteID in base alla prima nota.
		SetAltezza(pn->GetAltezza());
		NoteID=pn->GetID();
		HeadType=pn->GetHeadType();
		
		SetNStaff(pn->GetNStaff());
		
		switch(NoteID)
		{
		case CL_NBREVE:
			durata=2;
			break;
		case CL_NSEMIBREVE:
			durata=1;
			break;
		case CL_NMINIMA:
			durata=0.5;
			break;
		case CL_NSEMIMINIMA:
			durata=0.25;
			break;
		case CL_NCROMA:
			durata=0.125;
			break;
		case CL_NSEMICROMA:
			durata=0.0625;
			break;
		case CL_NBISCROMA:
			durata=0.03125;
			break;
		case CL_NSEMIBISCROMA:
			durata=0.015625;
			break;
		case CL_NFUSA:
			durata=0.0078125;
			break;
		}
		
		// setta l'SpG dell'accordo, tiene conto del tipo di leggio
		
		
        if ((tipoLeggio==LEGGIO_MASAE) || (tipoLeggio==LEGGIO_MASE))
			if(GetSpG()==0)
				SetSpG(pn->GetSpG());
			
			if ((tipoLeggio==LEGGIO_DLIOO) || (tipoLeggio==LEGGIO_PDLOO)) 
				if(GetSpOr()==0)
					SetSpOr(pn->GetSpOr());
				
	}
	else
	{    
		// Controlla se la nota che si vuol inserire ha lo stesso codice di
		// quelle gia' presenti.
		if(pn->GetID()!=GetNoteID())
		{
			delete NewNode;
			NewNode=NULL; 
			return(FALSE);
		}
		//Forza la nota al tipo di testa dell'accordo
		//pn->SetHeadType(GetHeadType());
		if(HeadType!=HEAD_ALPHANUM && HeadType!=HEAD_ALPHANUM_SQUARE && HeadType!=HEAD_ALPHANUM_REVERSE)
			pn->SetHeadCode(HeadCode);
		pn->SetDxGambo(DxGambo);
		pn->SetDyGambo(DyGambo);
		// Scandisci la lista per inserire la nota in modo da
		// mantenere la lista ordinata in base all'altezza delle note.
		alt=pn->GetAltezza();
		
		nstaff=pn->GetNStaff(); 
		
		//  Verifico che la nota inserita appartenga allo
		// stesso staff dell'accordo, se non appartiene allora 
		// setto lo staff dell'accordo a -1 perche' un accordo 
		// su due o piu' staff non appartiene a nessuno di essi.
		if(nstaff!=GetNStaff()) 
		{
			SetNStaff(-1);
		}
		
		pnode=first;
		found=FALSE;
		while(pnode!=NULL && !found)
		{
			pnota=(Nota*)pnode->ObjectPtr;
			if((pnota->GetAltezza() == alt) && (pnota->GetNStaff() == nstaff)) 
			{
				// Se nell'accordo c'e' gia' una nota dell'altezza della nota che si
				// vuol inserire, ed appartiene allo stesso staff (),questa nota non va aggiunta.
				delete NewNode;
				NewNode=NULL; 
				return(FALSE);
			}
			
			//  In questo modo si ordina la lista delle note sui tre staff sempre in ordine di altezza
			else if( ((pnota->GetAltezza() > alt) && (pnota->GetNStaff() == nstaff)) || (pnota->GetNStaff() < nstaff)  )
			{                                                       
				found=TRUE;
				// Inserisci la nota prima di quella puntata da pnode.
				NewNode->Next=pnode;
				NewNode->Prev=pnode->Prev;
				if(pnode->Prev==NULL)
					first=NewNode;
				else
					(pnode->Prev)->Next=NewNode;
				pnode->Prev=NewNode;
			} 
			pnode=pnode->Next;
		}
		if(!found)
		{
			// Inserisci la nota in coda alla lista.
			NewNode->Next=NULL;
			NewNode->Prev=last;
			last->Next=NewNode;
			last=NewNode;
		}
	}   
	
	pn->SetNotina(GetNotina());
	
	// Riduci al minimo i rettangoli che circondano la nota.
	// Serve affinche' una nota non ne nasconda altre nella selezione.
	
	pn->InitRectF();
	
	if(HeadType==HEAD_ALPHANUM || HeadType==HEAD_ALPHANUM_SQUARE || HeadType==HEAD_ALPHANUM_REVERSE)
	{
		Scr->SetFont(nTextFont);
		SetVU2UpF(TEXTUP(HeadCode));
		SetVU2DwnF(TEXTDW(HeadCode));
		SetVU2LftF(TEXTLF(HeadCode));
		SetVU2RgtF(TEXTRG(HeadCode));  
		Scr->SetFont(nMusicFont);
	}
	else if(HeadCode[0])
	{
		// Imposta il rettangolo usando le informazioni del font...
		SetVU2UpF(Scr->Get_VU2UP(HeadCode[0]));
		SetVU2DwnF(Scr->Get_VU2DWN(HeadCode[0]));
		SetVU2LftF(Scr->Get_VU2LF(HeadCode[0]));
		SetVU2RgtF(Scr->Get_VU2RG(HeadCode[0]));
	}
	else if(GetNotina())
	{
		// Imposta il rettangolo che circonda la notina.
		SetVU2UpF(VU2UPNOTA1);
		SetVU2DwnF(VU2DWNNOTA1);
		SetVU2LftF(VU2LFTNOTA1);
		SetVU2RgtF(VU2RGTNOTA1); 
	}
	else
	{
		// Imposta il rettangolo che circonda la nota.
		SetVU2UpF(VU2UPNOTA);
		SetVU2DwnF(VU2DWNNOTA);
		SetVU2LftF(VU2LFTNOTA);
		SetVU2RgtF(VU2RGTNOTA);
	}
	
	numobject++;
	return(TRUE);
 }
 
 
 void Accordo::SetLGambo(VUnit lg)
 {
	 if(GetNoteID()!=CL_NSEMIBREVE || GetNoteID()!=CL_NBREVE)
	 {
		 LGambo=lg;
	 }
 }
 
 
 void Accordo::SetGamboUp()
 {
	 if (GetNotina())
		 SetLGambo(LGAMBODEF/2);
	 else
		 SetLGambo(LGAMBODEF);
 }
 
 
 void Accordo::SetGamboDwn()
 { 
	 if (GetNotina())
		 SetLGambo(-LGAMBODEF/2);
	 else
		 SetLGambo(-LGAMBODEF);
 }
 
 
 void Accordo::SetCoda(Bool coda)
 {
	 if(GetNoteID()==CL_NBREVE || GetNoteID()==CL_NSEMIBREVE ||   
		 GetNoteID()==CL_NMINIMA || GetNoteID()==CL_NSEMIMINIMA)
	 {
		 Coda=FALSE;
	 }      
	 else
	 {
		 Coda=coda;
	 } 
 }               
 
 
 void Accordo::SetPuntiValore(int np,Bool updateSpG)
 {
	 Node *pnode;
	 Nota *pnota;
	 
	 if(np>=1 && np<=3)
	 { 
		 pnode=first;
		 while(pnode!=NULL)
		 {                   
			 pnota=(Nota*)pnode->ObjectPtr;
			 if(pnota->GetPPuntoValore()!=NULL)
				 delete pnota->GetPPuntoValore();
			 PuntoValore *pp=new PuntoValore;
			 if (pp==NULL)
				 EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Accordo","SetPuntiValore()");
			 pp->SetNumPunti(np);
			 pnota->SetPPuntoValore(pp,updateSpG);
			 
			 pnode=pnode->Next;
		 }
		 PuntoValore *pp=new PuntoValore;
		 if (pp==NULL)
			 EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Accordo","SetPuntiValore()");
		 pp->SetNumPunti(np);
		 SetPPuntoValore(pp,updateSpG);
	 }
 }      
 
 Battuta *pbattuta; 
 void Accordo::SetPos(DrawObject *d,const Point &p)
 {
	 pbattuta=(Battuta*)d;
	 
	 if(GetLGambo()>=0)
	 {
		 VUnit YLinea=GetPBattuta()->GetIntest(((Nota*)last->ObjectPtr)->GetNStaff())->GetAbsPos().y;
		 DrawObject::SetPos(NULL,Point(p.x,YLinea));
		 SetAltezza(((Nota*)(last->ObjectPtr))->GetAltezza());
	 }
	 else
	 {
		 VUnit YLinea=GetPBattuta()->GetIntest(((Nota*)first->ObjectPtr)->GetNStaff())->GetAbsPos().y;
		 DrawObject::SetPos(NULL,Point(p.x,YLinea));
		 SetAltezza(((Nota*)(first->ObjectPtr))->GetAltezza());
	 }
	 GetAbsPos().y+=GetAltezza()*VU_PER_ROW/2;
	 
	 // Aggiustamento lunghezza gambo se altezza nota oltre il secondo taglio addizionale
	 //   if(Altezza<=-4 && GetLGambo()>0)
	 //     SetLGambo((4-Altezza)*VU_PER_ROW/2);
	 //   else if(Altezza>=12 && GetLGambo()<0)
	 //     SetLGambo((4-Altezza)*VU_PER_ROW/2);
	 
	 AdjustF();
	 
 }              
 
 
 void Accordo::AdjustF()
 {
	 
	 VUnit YLinea[3]; 
	 // Altezza fisica delle linee piu' basse di ogni staff (prima linea del pentagramma)
	 Battuta *pBattuta; 
	 
	 pBattuta = this->GetPBattuta();
	 
	 if(pBattuta!=NULL)
	 {
		 switch(pBattuta->GetNumberOfStaff())
		 { 
		 case 1:
			 YLinea[0]= pBattuta->GetIntest(0)->GetAbsPos().y; //- GetAltezza()*VU_PER_ROW/2;
			 break;
		 case 2:
			 YLinea[0]= pBattuta->GetIntest(0)->GetAbsPos().y; //- (GetAltezza()*VU_PER_ROW/2);
			 YLinea[1]= pBattuta->GetIntest(1)->GetAbsPos().y;
			 break;
		 case 3:
			 YLinea[0]= pBattuta->GetIntest(0)->GetAbsPos().y; //- (GetAltezza()*VU_PER_ROW/2);
			 YLinea[1]= pBattuta->GetIntest(1)->GetAbsPos().y;
			 YLinea[2]= pBattuta->GetIntest(2)->GetAbsPos().y;
			 break;
		 }
	 }
	 
	 // PRIMA SCANSIONE DELLA LISTA DI NOTE.
	 // Sistema le note e imposta il rettangolo che circonda l'accordo.     
	 if(!isEmpty())   
	 { 
		 if(GetLGambo()>=0)
		 {
			 PlaceNoteGamboUp(YLinea);
		 }
		 else
		 { 
			 PlaceNoteGamboDwn(YLinea);
		 }  
	 }                       
	 
	 // Riporta i valori ottenuti alle VU2Up, etc.
	 SetVU2Up(GetVU2UpF());
	 SetVU2Dwn(GetVU2DwnF());
	 SetVU2Lft(GetVU2LftF());                     
	 SetVU2Rgt(GetVU2RgtF());
 }
 
 void Accordo::SetPosWA(DrawObject *d,const Point &p)
 {
	 if(GetLGambo()>=0)
	 {
		 VUnit YLinea=GetPBattuta()->GetIntest(((Nota*)last->ObjectPtr)->GetNStaff())->GetAbsPos().y;
		 DrawObject::SetPos(NULL,Point(p.x,YLinea));
		 SetAltezza(((Nota*)(last->ObjectPtr))->GetAltezza());
	 }
	 else
	 {
		 VUnit YLinea=GetPBattuta()->GetIntest(((Nota*)first->ObjectPtr)->GetNStaff())->GetAbsPos().y;
		 DrawObject::SetPos(NULL,Point(p.x,YLinea));
		 SetAltezza(((Nota*)(first->ObjectPtr))->GetAltezza());
	 }
	 GetAbsPos().y+=GetAltezza()*VU_PER_ROW/2;
 } 
 
 
 void Accordo::Adjust(Bool primaleg, Battuta *pBattuta, short current_layer,short current_staff)
 {    	
	 VUnit YLinea[3]; 
	 // Altezza fisica delle linee piu' basse dei pentagrammi in ogni staff
	 
	 if(pBattuta!=NULL)
	 {
		 switch(pBattuta->GetNumberOfStaff())
		 { 
		 case 1:
			 YLinea[0]=pBattuta->GetIntest(0)->GetAbsPos().y;
			 break;
		 case 2:
			 YLinea[0]= pBattuta->GetIntest(0)->GetAbsPos().y;
			 YLinea[1]= pBattuta->GetIntest(1)->GetAbsPos().y;
			 break;
		 case 3:
			 YLinea[0]= pBattuta->GetIntest(0)->GetAbsPos().y;
			 YLinea[1]= pBattuta->GetIntest(1)->GetAbsPos().y;
			 YLinea[2]= pBattuta->GetIntest(2)->GetAbsPos().y;
			 break;
		 }
	 }
	 
	 if(primaleg)
	 {
		 SetVU2Up(GetVU2UpF());
		 SetVU2Dwn(GetVU2DwnF());
		 SetVU2Lft(GetVU2LftF());
		 SetVU2Rgt(GetVU2RgtF());  
		 
		 Nota::Adjust(TRUE, pBattuta, current_layer,current_staff);
		 SetNotesHeadCode();
	 }
	 else
	 {  
		 // PRIMA SCANSIONE DELLA LISTA DI NOTE.
		 // Sistema le note e imposta il rettangolo che circonda l'accordo.     
		 if(!isEmpty())   
		 { 
			 if(GetLGambo()>=0)
			 {
				 PlaceNoteGamboUp(YLinea);
			 }
			 else
			 { 
				 PlaceNoteGamboDwn(YLinea);
			 }  
			 
		 }                       
		 
		 AdjustArpeggio(); 
		 
		 // SECONDA SCANSIONE DELLA LISTA DI NOTE.
		 // E' fatta sempre dall'alto verso il basso. Sistema le alterazioni ed i
		 // punti di valore (per questi ultimi non e' previsto lo spostamento
		 // verticale che viene fatto con le note singole).
		 AdjustAlterazPValore();
		 
		 
		 // SISTEMAZIONE DEI SIMBOLI SOPRA E SOTTO L'ACCORDO. 
		 
		 Nota::Adjust(FALSE, pBattuta, current_layer,current_staff);
		 
		 AdjustDiteggiato(YLinea[0],YLinea[0] + VU_PER_ROW*4 );
	 }
 }    
 
 void Accordo::Draw()
 {
	 Colore oldcol= Scr->GetColor();
	 if(isOpaque)
		 Scr->SetColor(GetColor());

	 if(HastobeDraw())
	 { 
		 Node *pnode;            
		 Nota *pnota;
		 int count=0;
		 
		 // Imposta il colore.
		 if(isOpaque)
			 Scr->SetColor(GetColor());
		 
		 // printf("Disegno la nota in accordo, con colore %d\n",GetColor());
		 
		 GetFirstNote()->DrawTagli();
		 GetLastNote()->DrawTagli();
		 if(GetLGambo()>=0)
		 {
			 pnode=first;
			 while(pnode!=NULL)
			 {
				 pnota=(Nota*)pnode->ObjectPtr;
				 // Non imposta il colore della nota poiche' DrawForAccordo e' predisposto
				 // per disegnare con il colore dell'accordo.
				 pnota->DrawForAccordo();
				 
				 pnode=pnode->Next;
			 }
		 }
		 else
		 {
			 pnode=last;
			 while(pnode!=NULL)
			 {
				 pnota=(Nota*)pnode->ObjectPtr;
				 // Non imposta il colore della nota poiche' DrawForAccordo e' predisposto
				 // per disegnare con il colore dell'accordo.
				 pnota->DrawForAccordo();
				 
				 pnode=pnode->Prev;
			 }
		 }
		 
		 if(GetPEspressComposta()!=NULL)
		 {
			 GetPEspressComposta()->Draw();
		 } 
		 if(GetPAbbellimento()!=NULL)
		 {
			 GetPAbbellimento()->Draw();
		 }
		 if(GetPKoreanOrnament()!=NULL)
		 {
			 GetPKoreanOrnament()->Draw();
		 }
		 if(GetPStrumento()!=NULL)
		 {
			 GetPStrumento()->Draw();
		 }
		 if(GetPCorona()!=NULL)
		 {
			 GetPCorona()->Draw();
		 }  
		 if(GetPOcchiali()!=NULL)
		 {
			 GetPOcchiali()->Draw();
		 }
		 if(GetPTDinamico()!=NULL)
		 {
			 GetPTDinamico()->Draw();
		 }
		 if(GetPTGenerico()!=NULL)
		 {
			 GetPTGenerico()->Draw();                  
		 }
		 if(GetPAnnotazione()!=NULL)
		 {
			 GetPAnnotazione()->Draw();                  
		 }
		 if(GetPSordina()!=NULL)
		 {
			 GetPSordina()->Draw();
		 } 
		 if(GetPArmonici()!=NULL)
		 {
			 GetPArmonici()->Draw();
		 }
		 
		 if(GetPArpeggio()!=NULL)
		 {
			 GetPArpeggio()->SetNumArpeg(GetHeight());
			 GetPArpeggio()->Draw();
		 } 
	 }

	 if(isSelected)	 {
		 DrawContour(SELECTBOX_MARGIN);
	 }

	 Scr->SetColor(oldcol);
}          
/* 
void Accordo::DrawContour(){
	Scr->SetColor((ColorIcon)SELECTED_COLOR);
	Scr->Rect(class Rectangle(GetAbsPos(), 
		GetVU2Lft() + SELECTBOX_MARGIN,
		GetVU2Rgt() + SELECTBOX_MARGIN, 
		GetVU2Up() + SELECTBOX_MARGIN, 
		GetVU2Dwn() + SELECTBOX_MARGIN)
		);

	Scr->SetColor((ColorIcon)GRIGIOS);
	Scr->Rect(class Rectangle(GetAbsPos(), 
		GetVU2Lft() + SELECTBOX_MARGIN -1,
		GetVU2Rgt() + SELECTBOX_MARGIN -1, 
		GetVU2Up() + SELECTBOX_MARGIN +1, 
		GetVU2Dwn() + SELECTBOX_MARGIN +1)
		);

	Scr->SetColor((ColorIcon)GRIGIO);
	Scr->Rect(class Rectangle(GetAbsPos(), 
		GetVU2Lft() + SELECTBOX_MARGIN -2,
		GetVU2Rgt() + SELECTBOX_MARGIN -2, 
		GetVU2Up() + SELECTBOX_MARGIN +2, 
		GetVU2Dwn() + SELECTBOX_MARGIN +2)
		);

}
 */
 Bool Accordo::DeleteSym(DrawObject *sym)
 {
	 Node *pnode, *pnod;            
	 Nota *pnota;
	 Bool pv=FALSE;
	 
	 if(sym==GetPArpeggio())
	 {
		 delete GetPArpeggio();
		 SetPArpeggio(NULL); 
		 return TRUE;
	 }
	 if(Nota::DeleteSym(sym))
		 return TRUE;
	 
	 if(sym->GetID()==CL_PUNTOVALORE) pv=TRUE;
	 
	 pnode=first;
	 while(pnode!=NULL)
	 {
		 pnota=(Nota*)pnode->ObjectPtr;
		 if(pnota==sym)
		 {
			 DelPtr(pnota);
			 return TRUE;
		 }
		 if(pnota->DeleteSym(sym))
		 {
			 if(pv)
			 {
				 pnod=first;
				 while(pnod!=NULL)
				 {
					 pnota=(Nota*)pnod->ObjectPtr;
					 pnota->DeleteSym(pnota->GetPPuntoValore());
					 pnod=pnod->Next;
				 }
				 DeleteSym(GetPPuntoValore());   
			 }
			 return TRUE;
		 }
		 pnode=pnode->Next;
	 }
	 return FALSE;	
 }
 
 
 void Accordo::Free()
 { 	
	 if (PArp!=NULL) delete PArp;
	 
	 Nota::Free();
 }
 
 
 Bool Accordo::Hit(const Point& p, SelObj selObj, DrawObject*& drwObjSel,
	 SymPath& symPath)
 {
	 Bool hit;   
	 Point p1;   
	 Node *pnode;
	 Nota *pnota; 	
	 
	 p1=p-GetAbsPos();               
	 hit = -p1.x <= GetVU2LftF() && p1.x <= GetVU2RgtF() &&
		 -p1.y <= GetVU2DwnF() && p1.y <= GetVU2UpF();
	 
	 drwObjSel=NULL;
	 
	 if(selObj==FIGURA_SEL || selObj==NOTA_SEL)
		 if(hit)
		 {
			 drwObjSel=this;
			 symPath.Array[symPath.Level]=GetNumericCode();
			 return(TRUE);
		 }
		 else
			 return(FALSE); 
		 
		 if(hit && drwObjSel==NULL && (selObj==NOTASING_SEL || selObj==ANY_SEL) )
		 {                 
			 // Scende di un livello nel path.
			 symPath.Level++;
			 pnode=first;
			 while(pnode!=NULL)
			 {   
				 pnota=(Nota*)pnode->ObjectPtr;                 
				 if(pnota->Hit(p,selObj,drwObjSel,symPath))
				 {
					 symPath.Array[--symPath.Level]=GetNumericCode();
					 return(TRUE);
				 }
				 pnode=GetNext(pnode);
			 }
			 // Se non e` stata trovata la nota, ripristina il livello precedente.
			 symPath.Level--;
		 }                
		 
		 
		 if(!hit && drwObjSel==NULL && (selObj==ANY_SEL || selObj==NOTASING_SEL))
		 {
			 
			 if(GetPArpeggio()!=NULL &&
				 GetPArpeggio()->Hit(p,selObj,drwObjSel,symPath))
			 { 
				 symPath.Array[symPath.Level]=GetNumericCode();
				 return(TRUE);
			 }
			 
			 
			 if(GetPAbbellimento()!=NULL &&
				 GetPAbbellimento()->Hit(p,selObj,drwObjSel,symPath))
			 { 
				 symPath.Array[symPath.Level]=GetNumericCode();
				 return(TRUE);
			 }
			 if(GetPKoreanOrnament()!=NULL &&
				 GetPKoreanOrnament()->Hit(p,selObj,drwObjSel,symPath))
			 { 
				 symPath.Array[symPath.Level]=GetNumericCode();
				 return(TRUE);
			 }
			 if(GetPDiteggiato()!=NULL &&
				 GetPDiteggiato()->Hit(p,selObj,drwObjSel,symPath))
			 { 
				 symPath.Array[symPath.Level]=GetNumericCode();
				 return(TRUE);
			 }
			 if(GetPSuddivisione()!=NULL &&
				 GetPSuddivisione()->Hit(p,selObj,drwObjSel,symPath))
			 { 
				 symPath.Array[symPath.Level]=GetNumericCode();
				 return(TRUE);
			 }
			 
			 if(GetPEspressComposta()!=NULL &&
				 GetPEspressComposta()->Hit(p,selObj,drwObjSel,symPath))
			 { 
				 symPath.Array[symPath.Level]=GetNumericCode();
				 return(TRUE);
			 } 
			 if(GetPSordina()!=NULL &&
				 GetPSordina()->Hit(p,selObj,drwObjSel,symPath))
			 { 
				 symPath.Array[symPath.Level]=GetNumericCode();
				 return(TRUE);
			 }
			 if(GetPStrumento()!=NULL &&
				 GetPStrumento()->Hit(p,selObj,drwObjSel,symPath))
			 {
				 symPath.Array[symPath.Level]=GetNumericCode();
				 return(TRUE);
			 }
			 if(GetPCorona()!=NULL &&
				 GetPCorona()->Hit(p,selObj,drwObjSel,symPath))
			 {
				 symPath.Array[symPath.Level]=GetNumericCode();
				 return(TRUE);
			 }
			 if(GetPOcchiali()!=NULL &&
				 GetPOcchiali()->Hit(p,selObj,drwObjSel,symPath))
			 {
				 symPath.Array[symPath.Level]=GetNumericCode();
				 return(TRUE);
			 }
			 
			 
			 if(GetPTDinamico()!=NULL &&
				 GetPTDinamico()->Hit(p,selObj,drwObjSel,symPath))
			 {
				 symPath.Array[symPath.Level]=GetNumericCode();
				 return(TRUE);
			 }
			 if(GetPTGenerico()!=NULL &&
				 GetPTGenerico()->Hit(p,selObj,drwObjSel,symPath))
			 {
				 symPath.Array[symPath.Level]=GetNumericCode();
				 return(TRUE);
			 }
			 if(GetPAnnotazione()!=NULL &&
				 GetPAnnotazione()->Hit(p,selObj,drwObjSel,symPath))
			 {
				 symPath.Array[symPath.Level]=GetNumericCode();
				 return(TRUE);
			 }
			 if(GetPArmonici()!=NULL &&
				 GetPArmonici()->Hit(p,selObj,drwObjSel,symPath))
			 {
				 symPath.Array[symPath.Level]=GetNumericCode();
				 return(TRUE);
			 }
			 
			 // Scende di un livello nel path.
			 symPath.Level++;
			 
			 pnode=first;
			 while(pnode!=NULL)
			 {
				 pnota=(Nota*)pnode->ObjectPtr;
				 if(pnota->GetPAlterazComposta()!=NULL &&
					 pnota->GetPAlterazComposta()->Hit(p,selObj,drwObjSel,symPath))
				 { 
					 symPath.Array[symPath.Level-1]=GetNumericCode();
					 symPath.Array[symPath.Level]=pnota->GetNumericCode();
					 symPath.Level--; 
					 return(TRUE);
				 }
				 if(pnota->GetPPuntoValore()!=NULL &&
					 pnota->GetPPuntoValore()->Hit(p,selObj,drwObjSel,symPath))
				 { 
					 symPath.Array[symPath.Level-1]=GetNumericCode();
					 symPath.Array[symPath.Level]=pnota->GetNumericCode();
					 symPath.Level--; 
					 return(TRUE);
				 }
				 if(pnota->GetPDiteggiato()!=NULL &&
					 pnota->GetPDiteggiato()->Hit(p,selObj,drwObjSel,symPath))
				 { 
					 symPath.Array[symPath.Level-1]=GetNumericCode();
					 symPath.Array[symPath.Level]=pnota->GetNumericCode();
					 symPath.Level--; 
					 return(TRUE);
				 }
				 pnode=pnode->Next;
			 }
			 // Se non e` stata trovato il simbolo ripristina il livello precedente.
			 symPath.Level--;
   }
   return(FALSE);
}



Bool Accordo::SymbolicHit(SymPath& symPath, SelObj selObj, DrawObject*& drwObjSel)
{
	Bool QuestoAccordo = (symPath.Array[symPath.Level]==GetNumericCode());   
	Bool ScendereAncora, EsaminareSimboli;
	Node *pnode;
	Nota *pnota;
	
	drwObjSel=NULL;
	
	if(!QuestoAccordo) 
		return FALSE;
	
	ScendereAncora = FALSE;
	if(symPath.Level < FIGURA3_PATH_LEVEL)
		ScendereAncora = (symPath.Array[symPath.Level+1]!=0);
	EsaminareSimboli = (symPath.Array[SIMBOLO_PATH_LEVEL]!=0);
	
	if(selObj==FIGURA_SEL)
	{
		drwObjSel=this;
		return(TRUE);
	}
	
	if(!ScendereAncora && !EsaminareSimboli &&
		(selObj==NOTA_SEL || selObj==ANY_SEL))
	{
		drwObjSel=this;
		return(TRUE);
	} 
	
	
	if(ScendereAncora && drwObjSel==NULL && selObj==NOTASING_SEL)
	{                 
		// Scende di un livello nel path.
		symPath.Level++;
		pnode=first;
		while(pnode!=NULL)
		{   
			pnota=(Nota*)pnode->ObjectPtr;                 
			if(pnota->SymbolicHit(symPath,selObj,drwObjSel))
			{
				symPath.Level--;
				return(TRUE);
			}
			pnode=GetNext(pnode);
		}
		// Se non e` stata trovata la nota, ripristina il livello precedente.
		symPath.Level--;
	}                
	
	if(EsaminareSimboli && drwObjSel==NULL && selObj==ANY_SEL)
	{
		
		if(GetPArpeggio()!=NULL &&
			GetPArpeggio()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);
		if(GetPAbbellimento()!=NULL &&
			GetPAbbellimento()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);   
		if(GetPKoreanOrnament()!=NULL &&
			GetPKoreanOrnament()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE); 	
 		if(GetPDiteggiato()!=NULL &&
			GetPDiteggiato()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE); 
		if(GetPSuddivisione()!=NULL &&
			GetPSuddivisione()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE); 
		if(GetPSordina()!=NULL &&
			GetPSordina()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);
		if(GetPEspressComposta()!=NULL &&
			GetPEspressComposta()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);  
		if(GetPStrumento()!=NULL &&
			GetPStrumento()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);
		if(GetPCorona()!=NULL &&
			GetPCorona()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE); 
		if(GetPOcchiali()!=NULL &&
			GetPOcchiali()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE); 
		if(GetPPuntoValore()!=NULL &&
			GetPPuntoValore()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);    
		if(GetPTDinamico()!=NULL &&
			GetPTDinamico()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);
		if(GetPTGenerico()!=NULL &&
			GetPTGenerico()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);
		if(GetPAnnotazione()!=NULL &&
			GetPAnnotazione()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);
		if(GetPArmonici()!=NULL &&
			GetPArmonici()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);
		if(ScendereAncora)
		{                 
			// Scende di un livello nel path.
			symPath.Level++;
			pnode=first;
			while(pnode!=NULL)
			{   
				pnota=(Nota*)pnode->ObjectPtr;                 
				
				if (symPath.Array[symPath.Level] == pnota->GetNumericCode()) //AA
				{
					if(pnota->GetPAlterazComposta()!=NULL &&
						pnota->GetPAlterazComposta()->SymbolicHit(symPath,selObj,drwObjSel))
					{
						symPath.Level--;
						return(TRUE);
					}
					if(pnota->GetPDiteggiato()!=NULL &&
						pnota->GetPDiteggiato()->SymbolicHit(symPath,selObj,drwObjSel))
					{
						symPath.Level--;
						return(TRUE);
					}
				}
				
				pnode=GetNext(pnode);
			}
			// Se non e` stata trovata la nota, ripristina il livello precedente.
			symPath.Level--;
		}                
	}
	return(FALSE);
}


void Accordo::Select()
{
	DrawObject::Select();
	//Scr->SetColor((ColorIcon)XOR_SELECTED_COLOR);
	//Scr->SetDrwMode(XORMode);
	//Scr->Rect(class Rectangle(*this));
	//Scr->SetDrwMode(DefMode);
}


void Accordo::Deselect()
{
	DrawObject::Deselect();
}     

void Accordo::Raise(int delta_altezza)
{
	Figura* pF=GetFirstF();
	while(pF)
	{
		pF->Raise(delta_altezza);
		pF = GetNextF(pF);
	}
}

CmdResult Accordo::SymCommand(SymCmd& SCmd, Battuta* pBattuta)
{ 
	
	CmdResult cmdRes;
	
	switch(SCmd.CommandId)
	{
	case CMD_FIGURA_1PUNTO:
		SetPuntiValore(1); 
		return(CMD_OK);
		break;                
	case CMD_FIGURA_2PUNTI:
		SetPuntiValore(2);
		return(CMD_OK);
		break;
	case CMD_FIGURA_3PUNTI:
		SetPuntiValore(3);
		return(CMD_OK);
		break;
	case CMD_ACCORDO_ARPEGGIO:
		return CmdAccordoArpeggio();
		break;
	case CMD_ACCORDO_ARPEGGIO_UP:
		return CmdAccordoArpeggioUp();
		break;
	case CMD_ACCORDO_ARPEGGIO_DOWN:
		return CmdAccordoArpeggioDown();
		break;
	case CMD_NOTA_NOTA2NOTINA:
		return CmdAccordoNota2Notina();
		break;
	case CMD_NOTA_NOTINA2NOTA:
		return CmdAccordoNotina2Nota();
		break;
	case CMD_NOTA_ACCIACCATURA:
		return CmdAccordoNota2Acciaccatura();
		break;
	default:   
		cmdRes=Nota::SymCommand(SCmd, pBattuta);
		if(cmdRes!=CMD_UNKNOWN)
			return cmdRes;
	}    
	return(CMD_UNKNOWN);
}



// METODI PRIVATI        

void Accordo::SetNotesHeadCode()
{
	Node *pnode;
	Nota *pnota;           
	
	pnode=first;
	while(pnode!=NULL)
	{           
		pnota=(Nota*)pnode->ObjectPtr;
		NoteHead_ID ht=pnota->GetHeadType();
		if(ht!=HEAD_ALPHANUM && ht!=HEAD_ALPHANUM_SQUARE && ht!=HEAD_ALPHANUM_REVERSE)
		{
			unsigned char hcNota[2];
			hcNota[0]=posizionam.PosRuleHead(pnota, pnota, GetPBattuta(), 0, 0);
			hcNota[1]='\0';
			pnota->SetHeadCode(hcNota);
		}
		pnota->SetDxGambo(DxGambo);  
		pnota->SetDyGambo(DyGambo);  
		pnode=pnode->Next;
	} 
}

void Accordo::PlaceNoteGamboUp(VUnit *YLinea)
{
	Node *pnode;
	Nota *pnota;           
	VUnit lg,vu;
	char PrevAlt;
	// Altezza della nota appena esaminata.
	char alt;
	// Altezza della nota che si sta esaminando.
	Bool PrevMoved;
	// E' TRUE se la nota precedente e' stata spostata.
	Bool Moved;
	// E' TRUE se almeno una nota e' stata spostata.
	
	pnode=first;
	pnota=(Nota*)pnode->ObjectPtr;
	// Imposta la coda della nota piu' bassa in base alla coda
	// dell'accordo.
	pnota->SetCoda(GetCoda());
	
	// Imposta GetAbsPos().x della nota all'GetAbsPos().x dell'accordo e
	// GetAbsPos().y secondo l'altezza della nota.
	// Non va fatto con SetPos(), poiche' questo chiama anche Adjust().
	pnota->SetPosWA(NULL,Point(GetAbsPos().x,YLinea[pnota->GetNStaff()]));
	
	
	
	// Imposta la lunghezza del gambo della nota piu' bassa.
	lg = GetLGambo() + GetAbsPos().y - pnota->GetAbsPos().y;
	
	
	pnota->SetLGambo(lg);
	
	// Inizializza il rettangolo che circonda l'accordo.
	if(GetLGambo() > VU2UPNOTA)
	{                               
		SetVU2UpF(GetLGambo());
	} 
	else
	{
		SetVU2UpF(VU2UPNOTA);
	}
	// Aggiungi a VU2UpF l'altezza delle code.      
	vu=GetLGambo();
	if(GetCoda())
	{
		switch(GetNoteID())
		{
		case CL_NSEMICROMA:
			vu+=DELTAYCODE;
			break;
		case CL_NBISCROMA:
			vu+=2*DELTAYCODE;
			break;
		case CL_NSEMIBISCROMA:
			vu+=3*DELTAYCODE;
			break;
		case CL_NFUSA:
			vu+=4*DELTAYCODE;
			break;
		}
	}
	else
	{
		vu+=INCRNOTASENZACODA;
	}
	SetVU2UpF(vu);
	SetVU2DwnF(GetAbsPos().y - pnota->GetAbsPos().y + VU2DWNNOTA);               
	SetVU2LftF(pnota->GetVU2LftF());
	SetVU2RgtF(pnota->GetVU2RgtF()); 
	
	PrevAlt=pnota->GetAltezza();
	PrevMoved=FALSE; 
	Moved=FALSE;
	pnode=pnode->Next;
	while(pnode!=NULL)
	{           
		pnota=(Nota*)pnode->ObjectPtr;
		pnota->SetNoGambo();  
		pnota->SetCoda(FALSE);
		
		alt=pnota->GetAltezza();
		
		// Imposta GetAbsPos().x della nota all'GetAbsPos().x dell'accordo e
		// GetAbsPos().y secondo l'altezza della nota.
		// Non va fatto con SetPos(), poiche' questo chiama anche Adjust().
		pnota->SetPosWA(NULL,Point(GetAbsPos().x,YLinea[pnota->GetNStaff()]));
		
		
		if(alt-PrevAlt==1 && !PrevMoved)
		{ 
			// Sposta a destra la nota.
			if (GetNotina())
				pnota->MoveX(SHIFTNOTINARGT);
			else
				pnota->MoveX(SHIFTNOTARGT);
			PrevMoved=TRUE;
			Moved=TRUE;
		}
		else PrevMoved=FALSE;
		PrevAlt=alt;
		pnode=pnode->Next;
	} 
	if(Moved)
	{
		// Se almeno una nota e' stata spostata, aggiorna VU2RgtF.
		if (GetNotina())
			SetVU2RgtF(GetVU2RgtF()+SHIFTNOTINARGT);
		else
			SetVU2RgtF(GetVU2RgtF()+SHIFTNOTARGT);
	}
	else
	{   
		// Se non ci sono note spostate a destra e l'accordo ha coda,
		// VU2RgtF va aumentato (se il gambo e' sotto non importa farlo).
		if(GetCoda())
		{
			SetVU2RgtF(VU2RGTFCODA);
		}
	}
 }
 
 
 void Accordo::PlaceNoteGamboDwn(VUnit *YLinea)
 {
	 Node *pnode;
	 Nota *pnota;           
	 VUnit lg,vu;
	 char PrevAlt;
	 // Altezza della nota appena esaminata.
	 char alt;
	 // Altezza della nota che si sta esaminando.
	 Bool PrevMoved;
	 // E' TRUE se la nota precedente e' stata spostata.
	 Bool Moved;
	 // E' TRUE se almeno una nota e' stata spostata.
	 
	 pnode=last;
	 
	 pnota=(Nota*)pnode->ObjectPtr;
	 
	 // Imposta la coda della nota piu' alta in base alla coda
	 // dell'accordo;
	 pnota->SetCoda(GetCoda()); 
	 
	 
	 // Imposta GetAbsPos().x della nota all'GetAbsPos().x dell'accordo e
	 // GetAbsPos().y secondo l'altezza della nota.
	 // Non va fatto con SetPos(), poiche' questo chiama anche Adjust().
	 pnota->SetPosWA(NULL,Point(GetAbsPos().x,YLinea[pnota->GetNStaff()]));
	 
	 
	 // Imposta la lunghezza del gambo della nota piu' alta.
	 lg = GetLGambo() + GetAbsPos().y - pnota->GetAbsPos().y; 
	 pnota->SetLGambo(lg);
	 
	 // Inizializza il rettangolo che circonda l'accordo.
	 if(-GetLGambo() > VU2DWNNOTA)
	 { 
		 SetVU2DwnF(-GetLGambo());
	 }
	 else
	 {
		 SetVU2DwnF(VU2DWNNOTA);
	 }  
	 // Aggiungi a VU2DwnF l'altezza delle code.      
	 vu=-GetLGambo();
	 if(GetCoda())
	 {
		 switch(GetNoteID())
		 {
		 case CL_NSEMICROMA:
			 vu+=DELTAYCODE;
			 break;
		 case CL_NBISCROMA:
			 vu+=2*DELTAYCODE;
			 break;
		 case CL_NSEMIBISCROMA:
			 vu+=3*DELTAYCODE;
			 break;
		 case CL_NFUSA:
			 vu+=4*DELTAYCODE;
			 break;
		 }
	 } 
	 else
	 {
		 vu+=INCRNOTASENZACODA;
	 }
	 SetVU2DwnF(vu);
	 SetVU2UpF(-GetAbsPos().y + pnota->GetAbsPos().y + VU2UPNOTA);     
	 SetVU2LftF(pnota->GetVU2LftF()); 
	 SetVU2RgtF(pnota->GetVU2RgtF());
	 
	 PrevAlt=pnota->GetAltezza();
	 PrevMoved=FALSE;                
	 Moved=FALSE;
	 pnode=pnode->Prev;
	 while(pnode!=NULL)
	 {           
		 pnota=(Nota*)pnode->ObjectPtr;
		 pnota->SetNoGambo();
		 pnota->SetCoda(FALSE);
		 alt=pnota->GetAltezza();
		 // Imposta GetAbsPos().x della nota all'GetAbsPos().x dell'accordo e
		 // GetAbsPos().y secondo l'altezza della nota.
		 // Non va fatto con SetPos(), poiche' questo chiama anche Adjust().
		 pnota->SetPosWA(NULL,Point(GetAbsPos().x,YLinea[pnota->GetNStaff()]));
		 
		 
		 if(PrevAlt-alt==1 && !PrevMoved)
		 {                                              
			 // Sposta a sinistra la nota.
			 if (GetNotina())
				 pnota->MoveX(SHIFTNOTINALFT);
			 else
				 pnota->MoveX(SHIFTNOTALFT);
			 PrevMoved=TRUE; 
			 Moved=TRUE;
		 }
		 else PrevMoved=FALSE;
		 PrevAlt=alt;
		 pnode=pnode->Prev;
	 }
	 if(Moved)
	 {
		 // Se almeno una nota e' stata spostata, aggiorna VU2LftF.
		 if (GetNotina())
			 SetVU2LftF(GetVU2LftF()-SHIFTNOTINALFT);
		 else
			 SetVU2LftF(GetVU2LftF()-SHIFTNOTALFT);
	 }
}                       

void Accordo::AdjustAlterazPValore()
{
	// Scandisce l'accordo dall'alto verso il basso.
	// Sistema le alterazioni ed i
	// punti di valore (per questi ultimi non e' previsto lo spostamento
	// verticale che viene fatto con le note singole).
	Node *pnode;
	Nota *pnota;           
	VUnit vu;
	
	Alterazione *PAlterazione;
	// Puntatore all'alterazione di *pnota.
	PuntoValore *PPV;                        
	// Puntatore al punto di valore di *pnota.
	
	pnode=last;
	VUnit lasty=0,lastdx=0,dx;
	VUnit lastyp=0;
	while(pnode!=NULL)
	{
		pnota=(Nota*)pnode->ObjectPtr;
		if(pnota->GetPAlterazComposta()!=NULL)      
		{ 
			// Sistema l'alterazione a sinistra della nota.
			
			PAlterazione=pnota->GetPAlterazComposta()->GetPAlterazione(pnota->GetPAlterazComposta()->Getn_alterazioni());            
			// Imposta la grandezza dell'alterazione.
			PAlterazione->SetNotina(GetNotina());
			
			// Sistema l'alterazione a sinistra della nota, distanziata con
			// SPNOTAALTERAZIONE (con la meta' se la nota e' una notina).
			
			if(GetNotina())
				dx=SPNOTAALTERAZIONE/2+PAlterazione->GetVU2Rgt();
			else
				dx=SPNOTAALTERAZIONE+PAlterazione->GetVU2Rgt();
			
			if(lasty!=0 && lasty<pnota->GetAbsPos().y+PAlterazione->GetVU2Up())
				dx+=lastdx;
			
			PAlterazione->SetPos(NULL,Point(GetAbsPos().x-GetVU2LftF()-dx,pnota->GetAbsPos().y));
			SetVU2Lft(GetVU2LftF()+dx+PAlterazione->GetVU2Lft());
			
			lastdx=dx+PAlterazione->GetVU2Lft();
			lasty=pnota->GetAbsPos().y-PAlterazione->GetVU2Dwn();
			
			
			if(pnota->GetPAlterazComposta()->Getn_alterazioni()==2)
				// Sistema l'altra alterazione.
			{    
				PAlterazione=pnota->GetPAlterazComposta()->GetPAlterazione(1);
				
				// Imposta la grandezza dell'alterazione.
				PAlterazione->SetNotina(GetNotina());
				
				// Sistema l'alterazione a sinistra distanziata con SPNOTAALTERAZIONE 
				// (con la met se la nota  una notina).
				
				if(GetNotina())
					dx=SPNOTAALTERAZIONE/2+PAlterazione->GetVU2Rgt();
				else
					dx=SPNOTAALTERAZIONE+PAlterazione->GetVU2Rgt();
				
				PAlterazione->SetPos(NULL,Point(GetAbsPos().x-GetVU2Lft()-dx,pnota->GetAbsPos().y));
				SetVU2Lft(GetVU2Lft()+dx+PAlterazione->GetVU2Lft());
				
				lastdx+=dx+PAlterazione->GetVU2Lft();
				VUnit lasty2=pnota->GetAbsPos().y-PAlterazione->GetVU2Dwn();
				if(lasty2<lasty)
					lasty=lasty2;
				
			}
		} 
		
		// Sistema il punto di valore.            
		PPV=pnota->GetPPuntoValore();
		if(PPV!=NULL)
		{
			VUnit dy=0;
			
			// se il punto e'sul rigo viene spostato sullo spazio sovrastante
			if(pnota->GetAltezza()%2==0)
				dy=VU_PER_ROW/2;
			
			if(lastyp==pnota->GetAbsPos().y+dy)
				if(dy!=0)
					dy=-VU_PER_ROW/2;
				
				PPV->SetPos(NULL, Point(
					GetAbsPos().x + GetVU2RgtF() + SPNOTAPUNTOVALORE + PPV->GetVU2Lft(),
					pnota->GetAbsPos().y+dy)); 
				lastyp=pnota->GetAbsPos().y+dy;
				// Se necessario, aggiorna VU2Rgt.
				vu = PPV->GetAbsPos().x + PPV->GetVU2Rgt() - GetAbsPos().x;
				if(vu>GetVU2Rgt()) SetVU2Rgt(vu);
		}
		pnode=pnode->Prev;
	}
}  


Nota* Accordo::GetFirstNote(void)
{
	return((Nota*)(first->ObjectPtr));
}

Nota * Accordo::GetLastNote(void)
{
	return((Nota*)(last->ObjectPtr));
}

Nota * Accordo::GetPrevNote(Nota * pN)
{
	return (Nota*)GetPrevF(pN);
}

Nota * Accordo::GetNextNote(Nota * pN)
{
	return (Nota*)GetNextF(pN);
}

void Accordo::AdjustEspressComposta(VUnit YLinea1,VUnit YLinea5,Bool primaFase)
{
	VUnit dy;
	Espressione *PEspressione;
	VUnit SpaceY;
	int i;
	
	if(GetPEspressComposta()!=NULL)
		for(i=1; i<=PEspressComposta->Getn_espressioni(); i++)
		{
			PEspressione=PEspressComposta->GetPEspressione(i);
			
			if (PEspressione->GetID()==CL_STACCATO)
				SpaceY = 4*RECTSPACEY;
			else
				SpaceY = 2*RECTSPACEY;
			
			if(PEspressione->GetAboveNota())
			{
				// L'espressione i-esima e' sopra l'accordo.
				PEspressione->SetPos(this,Point(0,GetVU2Up()+ SpaceY +
					PEspressione->GetVU2Dwn()));
				// Accorgimento per evitare di sistemare l'espressione dentro
				// il pentagramma.
				dy = YLinea5 + SpaceY + PEspressione->GetVU2Dwn() -
					PEspressione->GetAbsPos().y;
				if(dy>0) PEspressione->Move(Point(0,dy));
				// Aggiorna le VU2Up.
				SetVU2Up(PEspressione->GetAbsPos().y-GetAbsPos().y+
					PEspressione->GetVU2Up());
			}
			else
			{
				// L'espressione i-esima e' sotto l'accordo.
				PEspressione->SetPos(this,Point(0,-GetVU2Dwn()- SpaceY -
					PEspressione->GetVU2Up()));
				// Accorgimento per evitare di sistemare l'espressione dentro
				// il pentagramma.
				dy = YLinea1 - SpaceY - PEspressione->GetVU2Up() -
					PEspressione->GetAbsPos().y;
				if(dy<0) PEspressione->Move(Point(0,dy));
				// Aggiorna le VU2Dwn.
				SetVU2Dwn(-PEspressione->GetAbsPos().y+GetAbsPos().y+
					PEspressione->GetVU2Dwn());
			}                                                                         
			if(PEspressione->GetVU2Lft()>GetVU2Lft())
				SetVU2Lft(PEspressione->GetVU2Lft());
			if(PEspressione->GetVU2Rgt()>GetVU2Rgt())
				SetVU2Rgt(PEspressione->GetVU2Rgt());
		}
}


void Accordo::AdjustAbbellimento(VUnit YLinea1,VUnit YLinea5)
{
	VUnit dy;
	Alterazione *PAlterazione;
	
	if(GetPAbbellimento()!=NULL)
	{                                                                
		if (GetPAbbellimento()->GetID()==CL_TREMOLO) return;
		if (GetPAbbellimento()->GetID()==CL_GLISSATO) return;
		
		if(GetPAbbellimento()->GetAboveNota())
		{
			// L'abbellimento e' sopra la nota.
			PAlterazione=PAbbellimento->GetPAltSotto();
			if(PAlterazione!=NULL)
			{
				//L'abbellimento ha una alterazione sotto.	
				PAlterazione->SetPos(this,Point(1,GetVU2Up()+2*RECTSPACEY+
					PAlterazione->GetVU2Dwn()));
				// Accorgimento per evitare di sistemare l'alterazione dentro
				// il pentagramma.
				dy = YLinea5 + 2*RECTSPACEY + PAlterazione->GetVU2Dwn() -
					PAlterazione->GetAbsPos().y;
				if(dy>0) PAlterazione->Move(Point(0,dy));
				// Aggiorna le VU2Up.
				SetVU2Up(PAlterazione->GetAbsPos().y-GetAbsPos().y+
					PAlterazione->GetVU2Up());
			}	 
			
			PAbbellimento->SetPos(this,Point(0,GetVU2Up()+2*RECTSPACEY+
				PAbbellimento->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare l'abbellimento dentro
			// il pentagramma.
			dy = YLinea5 + 2*RECTSPACEY + PAbbellimento->GetVU2Dwn() -
				PAbbellimento->GetAbsPos().y;
			if(dy>0) PAbbellimento->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PAbbellimento->GetAbsPos().y-GetAbsPos().y+
				PAbbellimento->GetVU2Up());
			
			PAlterazione=PAbbellimento->GetPAltSopra();
			if(PAlterazione!=NULL)
			{
				//L'abbellimento ha una alterazione sopra.	
				PAlterazione->SetPos(this,Point(1,GetVU2Up()+2*RECTSPACEY+
					PAlterazione->GetVU2Dwn()));
				// Aggiorna le VU2Up.
				SetVU2Up(PAlterazione->GetAbsPos().y-GetAbsPos().y+
					PAlterazione->GetVU2Up());
			} 	  
		}
		else
		{
			// L'abbellimento e' sotto la nota.
			PAlterazione=PAbbellimento->GetPAltSopra();
			if(PAlterazione!=NULL)
			{ 
				// L'abbellimento ha una alterazione sopra.
				PAlterazione->SetPos(this,Point(1,-GetVU2Dwn()-2*RECTSPACEY-
					PAlterazione->GetVU2Up()));
				// Accorgimento per evitare di sistemare l'alterazione dentro
				// il pentagramma.
				dy = YLinea1 - 2*RECTSPACEY - PAlterazione->GetVU2Up() -
					PAlterazione->GetAbsPos().y;
				if(dy<0) PAlterazione->Move(Point(0,dy));
				// Aggiorna le VU2Dwn.
				SetVU2Dwn(-PAlterazione->GetAbsPos().y+GetAbsPos().y+
					PAlterazione->GetVU2Dwn());
			}
			
			PAbbellimento->SetPos(this,Point(0,-GetVU2Dwn()-2*RECTSPACEY-
				PAbbellimento->GetVU2Up()));
			// Accorgimento per evitare di sistemare l'abbellimento dentro
			// il pentagramma.
			dy = YLinea1 - 2*RECTSPACEY - PAbbellimento->GetVU2Up() -
				PAbbellimento->GetAbsPos().y;
			if(dy<0) PAbbellimento->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PAbbellimento->GetAbsPos().y+GetAbsPos().y+
				PAbbellimento->GetVU2Dwn());
			
			PAlterazione=PAbbellimento->GetPAltSotto(); 
			if(PAlterazione!=NULL)
			{ 
				// L'abbellimento ha una alterazione sotto.
				PAlterazione->SetPos(this,Point(1,-GetVU2Dwn()-2*RECTSPACEY-
					PAlterazione->GetVU2Up()));
				// Aggiorna le VU2Dwn.
				SetVU2Dwn(-PAlterazione->GetAbsPos().y+GetAbsPos().y+
					PAlterazione->GetVU2Dwn());
			} 
			
		}
		if(PAbbellimento->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PAbbellimento->GetVU2Lft());
		if(PAbbellimento->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PAbbellimento->GetVU2Rgt());         
	}                                                           
}

void Accordo::AdjustKoreanOrnament(VUnit YLinea1,VUnit YLinea5)
{
	VUnit dy;
	
	if(GetPKoreanOrnament()!=NULL)
	{                                                                 		
		if(GetPKoreanOrnament()->GetAboveNota())
		{			
			PKoreanOrnament->SetPos(this,Point(0,GetVU2Up()+2*RECTSPACEY+
				PKoreanOrnament->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare KoreanOrnament dentro
			// il pentagramma.
			dy = YLinea5 + 2*RECTSPACEY + PKoreanOrnament->GetVU2Dwn() -
				PKoreanOrnament->GetAbsPos().y;
			if(dy>0) PKoreanOrnament->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PKoreanOrnament->GetAbsPos().y-GetAbsPos().y+
				PKoreanOrnament->GetVU2Up());
		}
		else
		{
			PKoreanOrnament->SetPos(this,Point(0,-GetVU2Dwn()-2*RECTSPACEY-
				PKoreanOrnament->GetVU2Up()));
			// Accorgimento per evitare di sistemare KoreanOrnament dentro
			// il pentagramma.
			dy = YLinea1 - 2*RECTSPACEY - PKoreanOrnament->GetVU2Up() -
				PKoreanOrnament->GetAbsPos().y;
			if(dy<0) PKoreanOrnament->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PKoreanOrnament->GetAbsPos().y+GetAbsPos().y+
				PKoreanOrnament->GetVU2Dwn());
			
		}
		if(PKoreanOrnament->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PKoreanOrnament->GetVU2Lft());
		if(PKoreanOrnament->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PKoreanOrnament->GetVU2Rgt());         
	}                                                           
}

void Accordo::AdjustStrumento(VUnit YLinea1,VUnit YLinea5)
{
	VUnit dy;
	
	if(GetPStrumento()!=NULL)
	{
		if(GetPStrumento()->GetAboveNota())
		{
			// Il simbolo dello strumento e' sopra la nota.
			PStrumento->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+
				PStrumento->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare il simbolo dello
			// strumento dentro il pentagramma.
			dy = YLinea5 + RECTSPACEY + PStrumento->GetVU2Dwn() -
				PStrumento->GetAbsPos().y;
			if(dy>0) PStrumento->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PStrumento->GetAbsPos().y-GetAbsPos().y+
				PStrumento->GetVU2Up());
		}
		else
		{
			// Il simbolo dello strumento e' sotto la nota.
			PStrumento->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-
				PStrumento->GetVU2Up()));
			// Accorgimento per evitare di sistemare il simbolo dello
			// strumento dentro il pentagramma.
			dy = YLinea1 - RECTSPACEY - PStrumento->GetVU2Up() -
				PStrumento->GetAbsPos().y;
			if(dy<0) PStrumento->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PStrumento->GetAbsPos().y+GetAbsPos().y+
				PStrumento->GetVU2Dwn());
		}
		if(PStrumento->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PStrumento->GetVU2Lft());
		if(PStrumento->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PStrumento->GetVU2Rgt());
		
		switch (GetPStrumento()->GetID())
		{
		case CL_ARCO:
			break;
		case CL_PIZZICATO:
			break;
		default:
			break;
		}
	} 
}


void Accordo::AdjustDiteggiato(VUnit YLinea1,VUnit YLinea5)
{          
	Node *pnode;
	Nota *pnota;
	Diteggiato *PDit;
	VUnit dy;
	
	if(GetDiteggiatoUp())
	{
		// Il diteggiato e' sopra l'accordo. Scandisci l'accordo dal basso
		// verso l'alto.
		pnode=first;
		while(pnode!=NULL)
		{                   
			pnota=(Nota*)pnode->ObjectPtr;
			PDit=pnota->GetPDiteggiato();
			if(PDit!=NULL)
			{
				PDit->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+PDit->GetVU2Dwn()));
				// Accorgimento per evitare di sistemare il diteggiato dentro
				// il pentagramma.
				dy = YLinea5 + RECTSPACEY + PDit->GetVU2Dwn() - PDit->GetAbsPos().y;
				if(dy>0) PDit->Move(Point(0,dy));
				// Aggiorna VU2Up, VU2Lft, VU2Rgt.
				SetVU2Up(PDit->GetAbsPos().y-GetAbsPos().y+PDit->GetVU2Up());
				if(PDit->GetVU2Lft()>GetVU2Lft())
					SetVU2Lft(PDit->GetVU2Lft());
				if(PDit->GetVU2Rgt()>GetVU2Rgt())
					SetVU2Rgt(PDit->GetVU2Rgt());
			}
			pnode=pnode->Next;
		}
	}
	else
	{
		// Il diteggiato e' sotto l'accordo. Scandisci l'accordo dall'alto
		// verso il basso.
		pnode=last;
		while(pnode!=NULL)
		{
			pnota=(Nota*)pnode->ObjectPtr;
			PDit=pnota->GetPDiteggiato();
			if(PDit!=NULL)
			{
				PDit->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-PDit->GetVU2Up()));
				// Accorgimento per evitare di sistemare il diteggiato dentro
				// il pentagramma.
				dy = YLinea1 - RECTSPACEY - PDit->GetVU2Up() - PDit->GetAbsPos().y;
				if(dy<0) PDit->Move(Point(0,dy));
				// Aggiorna VU2Dwn, VU2Lft, VU2Rgt.
				SetVU2Dwn(-PDit->GetAbsPos().y+GetAbsPos().y+PDit->GetVU2Dwn());     
				if(PDit->GetVU2Lft()>GetVU2Lft())
					SetVU2Lft(PDit->GetVU2Lft());
				if(PDit->GetVU2Rgt()>GetVU2Rgt())
					SetVU2Rgt(PDit->GetVU2Rgt());
			}
			pnode=pnode->Prev;
		} 
	}            
}


void Accordo::AdjustCorona(VUnit YLinea1,VUnit YLinea5)
{
	VUnit dy;
	
	if(GetPCorona()!=NULL)
	{
		if(GetPCorona()->GetAboveNota())
		{
			// La corona e' sopra l'accordo.
			PCorona->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+
				PCorona->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare la corona dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + PCorona->GetVU2Dwn() -
				PCorona->GetAbsPos().y;
			if(dy>0) PCorona->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PCorona->GetAbsPos().y-GetAbsPos().y+
				PCorona->GetVU2Up());
		}
		else
		{
			// La corona e' sotto l'accordo.
			PCorona->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-
				PCorona->GetVU2Up()));
			// Accorgimento per evitare di sistemare la corona dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - PCorona->GetVU2Up() -
				PCorona->GetAbsPos().y;
			if(dy<0) PCorona->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PCorona->GetAbsPos().y+GetAbsPos().y+
				PCorona->GetVU2Dwn());
		}
		if(PCorona->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PCorona->GetVU2Lft());
		if(PCorona->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PCorona->GetVU2Rgt());
	}
}

void Accordo::AdjustOcchiali(VUnit YLinea1,VUnit YLinea5)
{
	VUnit dy;
	
	if(GetPOcchiali()!=NULL)
	{
		if(GetPOcchiali()->GetAboveNota())
		{
			// Gli occhiali sono sopra l'accordo.
			POcchiali->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+
				POcchiali->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare gli occhiali dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + POcchiali->GetVU2Dwn() -
				POcchiali->GetAbsPos().y;
			if(dy>0) POcchiali->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(POcchiali->GetAbsPos().y-GetAbsPos().y+
				POcchiali->GetVU2Up());
		}
		else
		{
			// Gli occhiali sono sotto l'accordo.
			POcchiali->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-
				POcchiali->GetVU2Up()));
			// Accorgimento per evitare di sistemare gli occhiali dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - POcchiali->GetVU2Up() -
				POcchiali->GetAbsPos().y;
			if(dy<0) POcchiali->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-POcchiali->GetAbsPos().y+GetAbsPos().y+
				POcchiali->GetVU2Dwn());
		}
		if(POcchiali->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(POcchiali->GetVU2Lft());
		if(POcchiali->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(POcchiali->GetVU2Rgt());
	}
}

void Accordo::AdjustTDinamico(VUnit YLinea1,VUnit YLinea5)
{
	VUnit dy;
	
	if(GetPTDinamico()!=NULL)
	{
		if(GetPTDinamico()->GetAboveNota())
		{
			// Il sego dinamico e' sopra l'accordo.
			PTDinamico->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+
				PTDinamico->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare il segno dinamico dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + PTDinamico->GetVU2Dwn() -
				PTDinamico->GetAbsPos().y;
			if(dy>0) PTDinamico->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PTDinamico->GetAbsPos().y-GetAbsPos().y+
				PTDinamico->GetVU2Up());
		}
		else
		{
			// Il segno dinamico e' sotto l'accordo.
			PTDinamico->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-
				PTDinamico->GetVU2Up()));
			// Accorgimento per evitare di sistemare il segno dinamico dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - PTDinamico->GetVU2Up() -
				PTDinamico->GetAbsPos().y;
			if(dy<0) PTDinamico->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PTDinamico->GetAbsPos().y+GetAbsPos().y+
				PTDinamico->GetVU2Dwn());
		}
		if(PTDinamico->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PTDinamico->GetVU2Lft());
		if(PTDinamico->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PTDinamico->GetVU2Rgt());
	}                                                     
}                            

void Accordo::AdjustTGenerico(VUnit YLinea1,VUnit YLinea5)                            
{
	VUnit dy;
	
	if(GetPTGenerico()!=NULL)
	{
		if(GetPTGenerico()->GetAboveNota())
		{
			// Il segno Generico e' sopra l'accordo.
			PTGenerico->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+
				PTGenerico->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare il segno Generico dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + PTGenerico->GetVU2Dwn() -
				PTGenerico->GetAbsPos().y;
			if(dy>0) PTGenerico->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PTGenerico->GetAbsPos().y-GetAbsPos().y+
				PTGenerico->GetVU2Up());
		}
		else
		{
			// Il segno Generico e' sotto l'accordo.
			PTGenerico->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-
				PTGenerico->GetVU2Up()));
			// Accorgimento per evitare di sistemare il segno Generico dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - PTGenerico->GetVU2Up() -
				PTGenerico->GetAbsPos().y;
			if(dy<0) PTGenerico->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PTGenerico->GetAbsPos().y+GetAbsPos().y+
				PTGenerico->GetVU2Dwn());
		}
		if(PTGenerico->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PTGenerico->GetVU2Lft());
		if(PTGenerico->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PTGenerico->GetVU2Rgt());
	}                                                      
}   


void Accordo::AdjustAnnotazione(VUnit YLinea1,VUnit YLinea5)                            
{
	VUnit dy;
	
	if(GetPAnnotazione()!=NULL)
	{
		if(GetPAnnotazione()->GetAboveNota())
		{
			// Il segno Generico e' sopra l'accordo.
			PAnnotazione->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+
				PAnnotazione->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare il segno Generico dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + PAnnotazione->GetVU2Dwn() -
				PAnnotazione->GetAbsPos().y;
			if(dy>0) PAnnotazione->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PAnnotazione->GetAbsPos().y-GetAbsPos().y+
				PAnnotazione->GetVU2Up());
		}
		else
		{
			// Il segno Generico e' sotto l'accordo.
			PAnnotazione->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-
				PAnnotazione->GetVU2Up()));
			// Accorgimento per evitare di sistemare il segno Generico dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - PAnnotazione->GetVU2Up() -
				PAnnotazione->GetAbsPos().y;
			if(dy<0) PAnnotazione->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PAnnotazione->GetAbsPos().y+GetAbsPos().y+
				PAnnotazione->GetVU2Dwn());
		}
		if(PAnnotazione->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PAnnotazione->GetVU2Lft());
		if(PAnnotazione->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PAnnotazione->GetVU2Rgt());
	}                                                      
}   


void Accordo::AdjustSordina(VUnit YLinea1,VUnit YLinea5)                            
{
	VUnit dy;
	
	if(GetPSordina()!=NULL)
	{
		if(GetPSordina()->GetAboveNota())
		{
			// La sordina e' sopra l'accordo.
			PSordina->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+
				PSordina->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare la sordina dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + PSordina->GetVU2Dwn() -
				PSordina->GetAbsPos().y;
			if(dy>0) PSordina->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PSordina->GetAbsPos().y-GetAbsPos().y+
				PSordina->GetVU2Up());
		}
		else
		{
			// La sordina e' sotto l'accordo.
			PSordina->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-
				PSordina->GetVU2Up()));
			// Accorgimento per evitare di sistemare la sordina dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - PSordina->GetVU2Up() -
				PSordina->GetAbsPos().y;
			if(dy<0) PSordina->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PSordina->GetAbsPos().y+GetAbsPos().y+
				PSordina->GetVU2Dwn());
		}
		if(PSordina->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PSordina->GetVU2Lft());
		if(PSordina->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PSordina->GetVU2Rgt());
	}                                                      
}    

void Accordo::AdjustArpeggio()
{
	VUnit vu;
	
	if (GetPArpeggio()!=NULL)
	{
		GetPArpeggio()->SetPos(this, 
			Point(- (GetVU2Lft()+GetPArpeggio()->GetVU2Rgt()+2),-GetVU2Dwn()));
		
		// Se necessario, aggiorna VU2Lft.
		vu = GetAbsPos().x - GetPArpeggio()->GetAbsPos().x + GetPArpeggio()->GetVU2Lft();
		if(vu>GetVU2Lft()) SetVU2Lft(vu);
	}
}  

CmdResult Accordo::CmdAccordoArpeggio()
{
	if(GetPArpeggio()!=NULL)
		return(CMD_FAIL);
	
	SetPArpeggio(new Arpeggio);
	if(GetPArpeggio()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Accordo","CmdAccordoArpeggio()");   
		return(CMD_FAIL);
	};
	
	GetPArpeggio()->SetNumArpeg(GetHeight());
	return(CMD_OK); 
}

CmdResult Accordo::CmdAccordoArpeggioUp()
{
    if(GetPArpeggio()!=NULL)
		return(CMD_FAIL);
	
    SetPArpeggio(new Arpeggio(UPWARD));
	if(GetPArpeggio()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Accordo","CmdAccordoArpeggioUp()");   
		return(CMD_FAIL);
	};
	
	GetPArpeggio()->SetNumArpeg(GetHeight());
    return(CMD_OK); 
}

CmdResult Accordo::CmdAccordoArpeggioDown()
{
	if(GetPArpeggio()!=NULL)
		return(CMD_FAIL);
	
	SetPArpeggio(new Arpeggio(DOWNWARD));
	if(GetPArpeggio()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Accordo","CmdAccordoArpeggio()");   
		return(CMD_FAIL);
	};
	
	GetPArpeggio()->SetNumArpeg(GetHeight());
	return(CMD_OK); 
}


CmdResult Accordo::CmdAccordoNota2Notina()
{
	Node *pnode;
	Nota *pnota;
	
	if(GetNotina())
		return(CMD_FAIL); 
	
	pnode=first;
	
	while(pnode!=NULL)
	{
		pnota=(Nota*)pnode->ObjectPtr;
		
		pnota->SetNotina(TRUE);
		// Adatta il gambo.    
		if(pnota->GetLGambo()>0)
			pnota->SetGamboUp();    
		if(pnota->GetLGambo()<0)
			pnota->SetGamboDwn();
		
		pnode=pnode->Next;
		
	}
	
	SetNotina(TRUE);
	if(GetLGambo()>0)
		SetGamboUp();    
	if(GetLGambo()<0)
		SetGamboDwn();
	
	return(CMD_OK);
}


CmdResult Accordo::CmdAccordoNotina2Nota()
{
	Node *pnode;
	Nota *pnota;
	
	if(!GetNotina())
		return(CMD_FAIL); 
	
	pnode=first;
	
	while(pnode!=NULL)
	{
		pnota=(Nota*)pnode->ObjectPtr;
		
		if (pnota->GetID()==CL_NCROMA)
			((NCroma*)pnota)->SetAcciaccatura(FALSE);
		
		pnota->SetNotina(FALSE);
		// Adatta il gambo.    
		if(pnota->GetLGambo()>0)
			pnota->SetGamboUp();    
		if(pnota->GetLGambo()<0)
			pnota->SetGamboDwn();
		
		pnode=pnode->Next;
	}
	
	
	SetNotina(FALSE);
	if(GetLGambo()>0)
		SetGamboUp();    
	if(GetLGambo()<0)
		SetGamboDwn();
	return(CMD_OK);
}

int Accordo::GetAltezzaMedia()
{
	Node *pnode;
	Nota *pnota;
	int altMax=-1000,altMin=1000,alt;
	
	pnode=first;
	
	while(pnode!=NULL)
	{
		pnota=(Nota*)pnode->ObjectPtr;
		alt=pnota->GetAltezza();
		if(alt>altMax)
			altMax=alt;
		if(alt<altMin)
			altMin=alt;
		
		pnode=pnode->Next;
	}
	return (altMax+altMin)/2;
}

/*********************************************/
int Accordo::GetAcciaccatura(void)
{
	NCroma *pncromaf;
	Nota *pnotaf;
	pnotaf = (Nota*)first->ObjectPtr;
	
	if (pnotaf->GetID()==CL_NCROMA)
	{
		if (GetLGambo()>0)
		{
			pncromaf = (NCroma*)pnotaf;
			return(pncromaf->GetAcciaccatura());
		}
	}
	return false;
}


/*********************************************/
CmdResult Accordo::CmdAccordoNota2Acciaccatura()
{
	NCroma *pncromal, *pncromaf;
	Nota *pnotal, *pnotaf;
	Node *pnode;
	
	pnotaf = (Nota*)first->ObjectPtr;
	
	if (pnotaf->GetID()==CL_NCROMA)
	{
		
		// Controllo sulla coda per impedire che questo comando venga
		// eseguito sulle note dentro i gruppi.
		if (GetLGambo()>0)
		{
			pncromaf = (NCroma*)pnotaf;
			if((pncromaf->GetAcciaccatura()) || 
				!(pncromaf->GetCoda()))
				return(CMD_FAIL);   
		}
		else
		{
			pnotal = (Nota*)last->ObjectPtr;
			pncromal = (NCroma*)pnotal;
			if((pncromal->GetAcciaccatura()) || 
				!(pncromal->GetCoda()))
				return(CMD_FAIL);   
		}
		
		CmdAccordoNota2Notina();
		
		pnode = first;
		while (pnode!=NULL)
		{
			((NCroma*)pnode->ObjectPtr)->SetAcciaccatura(TRUE);
			pnode = pnode->Next;
		}
		
		return(CMD_OK);
	}
	else
		return(CMD_FAIL);
	
	
}

DrawObject * Accordo::GetAggiunta(SymCmd * MyCmd) 
{
	DrawObject *Appo;
	Appo=Figura::GetAggiunta(MyCmd);
	if (Appo!=NULL) return Appo;
	
	switch(MyCmd->CommandId)
	{
	case CMD_NOTA_ARCO:
	case CMD_ARCO_ARCOSU:
		
    case CMD_ARCO_ARCOGIU:
	case CMD_FIGURA_PEDALDOWN:
   	case CMD_FIGURA_PEDALUP:
		return(Nota::GetPStrumento()->GetObject(MyCmd)); 
		break;
	case CMD_NOTA_DIESIS: 
	case CMD_NOTA_DIESIS1Q:
	case CMD_NOTA_DIESIS3Q:
	case CMD_NOTA_BEMOLLE: 
	case CMD_NOTA_BEMOLLE1Q:
	case CMD_NOTA_BEMOLLE3Q:
	case CMD_NOTA_BEQUADRO: 
	case CMD_NOTA_2DIESIS:
		
   	case CMD_NOTA_2BEMOLLE:
   	case CMD_NOTA_BEQUADRO1Q: 
   	case CMD_NOTA_BEQUADRO3Q: 
		{
			Nota * pNota;
			// Devo Arrivare alla nota con CN = ArgV[6] !
			if (!SymbolicHit(MyCmd->Path1,NOTASING_SEL,(DrawObject *&)pNota)) 
				return NULL;				   				
			
			if (pNota!=NULL) {
				AlterazComposta *MyAlt=pNota->GetPAlterazComposta(); 
				if (MyAlt!=NULL)  {
					return MyAlt->GetPAlterazione(MyCmd->CommandId);
				}
			}			
			return NULL;
		}
		break;  
   	case CMD_NOTA_DITO_0:  
	case CMD_NOTA_DITO_1:  
	case CMD_NOTA_DITO_2:
	case CMD_NOTA_DITO_3:
	case CMD_NOTA_DITO_4:
	case CMD_NOTA_DITO_5:     
		{
			// printf("Passo di qui\n");
			Nota * pNota;
			// Devo Arrivare alla nota con CN = ArgV[6] !
			if (!SymbolicHit(MyCmd->Path1,NOTASING_SEL,(DrawObject *&)pNota)) 
				return NULL;				   				
			if (pNota!=NULL) 
				return pNota->GetAggiunta(MyCmd);
			
			
			return NULL;
		}
		break;  
		// FN
	case CMD_NOTA_TRILLO:       
	case CMD_NOTA_GRUPPETTO_SUP_1:
	case CMD_NOTA_GRUPPETTO_INF_1: 
	case CMD_NOTA_MORDENTE_SUP:
	case CMD_NOTA_MORDENTE_INF: 
	case CMD_NOTA_MORDENTE2_SUP:
	case CMD_NOTA_MORDENTE2_INF: 
	case CMD_NOTA_TREMOLO:
	case CMD_NOTA_TURN_SLASH:
	case CMD_NOTA_TURN_UP:
	case CMD_NOTA_GLISSONDA:
	case CMD_NOTA_GLISSATO:
		return((DrawObject *)(GetPAbbellimento()));
		break;
		
	case CMD_NOTA_SLIDING_UP:
	case CMD_NOTA_SLIDING_DOWN:
	case CMD_NOTA_SLIDING_DUP:
	case CMD_NOTA_SLIDING_DOWNUP:
	case CMD_NOTA_NONGHYN_INC:
	case CMD_NOTA_NONGHYN_DEC:
	case CMD_NOTA_NONGHYN_ARROW:
	case CMD_NOTA_NONGHYN_WIDE:return((DrawObject *)(GetPKoreanOrnament())); break;
		
	case CMD_ACCORDO_ARPEGGIO:
		if (GetID()==CL_ACCORDO) return ((Accordo *)(this))->GetPArpeggio();
		else return NULL;
		break;
		
	case CMD_NOTA_TENUTO:
	case CMD_NOTA_PUNTOALL:
	case CMD_NOTA_SFORZATO:
	case CMD_NOTA_ACCENTO:
	case CMD_NOTA_MARTELLATO:
	case CMD_NOTA_MARTDOLCE:
	case CMD_NOTA_STACCATO:
		{
			EspressComposta *MyEsp=GetPEspressComposta(); 
			if (MyEsp!=NULL) 
				return MyEsp->GetPEspressione(MyCmd->CommandId);
			else 
				return NULL;
			
		}
		break;     
		
    default:  
		// Proviamo a fare una hit sulle note dell'accordo !
		if (!SymbolicHit(MyCmd->Path1,NOTASING_SEL,(DrawObject *&)Appo))
			return(NULL);
		return Appo;
		break;
	}            
	
	return NULL;
}

Bool Accordo::FigPtr2SymPath(SymPath& symPath, Figura *figptr, Bool inGruppo)
{
	Bool trovato=FALSE;
	
	Figura *ptr=(Figura*)GetFirst();
	
	while((!trovato) && (ptr!=NULL))
	{
		
		if(ptr==figptr)
		{
			symPath.Array[inGruppo ? FIGURA3_PATH_LEVEL : 
		FIGURA2_PATH_LEVEL]=(ptr->GetNumericCode());      
		trovato=TRUE; 
		}                     
		ptr = (Figura*)GetNext((DrawObject*)ptr);
	}
	
	return trovato;
}

int Accordo::GetDiffUpperLower()
{
	int upd, lwd;
	Nota *pn1, *pn2;
	
	pn1 = GetFirstNote();
	pn2 = GetLastNote();
	lwd = pn1->GetAltezza()-4;
	upd = pn2->GetAltezza()-4;
	
	return (upd>0?upd:-upd)-(lwd>0?lwd:-lwd);
}

int Accordo::GetDiffNUpNDown()
{
	Nota *pn;
	Node *pnode;
	
	int alt;
	int up=0;
	int down=0;
	
	pnode = first;
	while (pnode!=NULL)
	{
		pn = (Nota*)pnode->ObjectPtr;
		alt = pn->GetAltezza();
		if (alt>4)
		{
			up = up+1; 
		}
		else if (alt<4)
		{
			down = down+1;
		}
		pnode = pnode->Next;
	}
	
	return up-down;
} 


int Accordo::GetNumUp()
{
	Nota *pn;
	Node *pnode;
	
	int alt;
	int up=0;
	
	pnode = first;
	while (pnode!=NULL)
	{
		pn = (Nota*)pnode->ObjectPtr;
		alt = pn->GetAltezza();
		if (alt>4)
		{
			up = up+1; 
		}
		pnode = pnode->Next;
	}  
	return up;  
} 


int Accordo::GetNumDown()
{
	Nota *pn;
	Node *pnode;
	
	int alt;
	int down=0;
	
	pnode = first;
	while (pnode!=NULL)
	{
		pn = (Nota*)pnode->ObjectPtr;
		alt = pn->GetAltezza();
		if (alt<4)
		{
			down = down+1; 
		}
		pnode = pnode->Next;
	}  
	return down;  
}

void Accordo::SetBigAccordo()
{
	CmdAccordoNotina2Nota();
}

void Accordo::SetSmallAccordo()
{
	CmdAccordoNota2Notina();
}

void Accordo::SetStatus(Status_ID sta)
{
	Node *pnode;
	Nota *pnota;
	pnode=first;
	float dur;
	
	while(pnode!=NULL)
	{
		pnota=(Nota*)pnode->ObjectPtr;
		pnota->SetStatus(sta);
		pnota->SetDurationDuringEdit();
		if (sta==GHOSTED || sta==GRACED)
			pnota->SetDurata(0);
		dur=pnota->GetDurata();
		pnode=pnode->Next;
		
	}
	status=sta;
	if (sta==GHOSTED || sta==GRACED)
		SetDurata(0);
	else
		SetDurata(dur);
}


void Accordo::GetMinMaxStaff(short& minStaff, short& maxStaff)
{
	ListaFigure::GetMinMaxStaff(minStaff,maxStaff);
}
