/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: accordo.hpp,v 1.1 2005/07/25 16:30:38 bellinpo Exp $ */


#ifndef ACCORDO_HPP
#define ACCORDO_HPP

#include "lioo.hpp"

#include "fig/nota.hpp"
#include "bat/listafig.hpp"
#include "bat/battuta.hpp" 
#include "var/arpeggio.hpp"

class Accordo: public Nota, public ListaFigure
{                   
private:
    void SetNotesHeadCode();
    void PlaceNoteGamboUp(VUnit*);
	// Posiziona le note dell'accordo nel caso che il gambo sia verso
	// l'alto. L'accordo viene scandito dal basso verso l'alto e le
	// note vengono disposte a sinistra del gambo. Se due note si
	// sovrappongono parzialmente, la superiore viene spostata a
	// destra del gambo.
	// YLinea e' l'altezza fisica della linea piu' bassa del
	// pentagramma in ogni staff. E' un vettore di 3 elementi da 0 a 2..
    void PlaceNoteGamboDwn(VUnit*); 
	// Posiziona le note dell'accordo nel caso che il gambo sia verso
	// il basso. L'accordo viene scandito dall'alto verso il basso e le
	// note vengono disposte a destra del gambo. Se due note si
	// sovrappongono parzialmente, l'inferiore viene spostata a
	// sinistra del gambo.
	// YLinea e' l'altezza fisica della linea piu' bassa del
	// pentagramma in ogni staff. E' un vettore di 3 elementi da 0 a 2.
    void AdjustAlterazPValore();
	// Sistema le alterazioni ed i punti di valore delle note che
	// compongono l'accordo.
	
	
    void AdjustEspressComposta(VUnit YLinea1,VUnit YLinea5,Bool primaFase);
	// Sistema l'espressione composta dell'accordo.
	// YLinea1 e YLinea5 sono le altezze fisiche della linea piu' bassa
	// e piu' alta del pentagramma.  
    void AdjustAbbellimento(VUnit YLinea1,VUnit YLinea5);
	void AdjustKoreanOrnament(VUnit YLinea1,VUnit YLinea5);
    void AdjustStrumento(VUnit YLinea1,VUnit YLinea5);
    void AdjustDiteggiato(VUnit YLinea1,VUnit YLinea5);
    void AdjustCorona(VUnit YLinea1,VUnit YLinea5);
    void AdjustOcchiali(VUnit YLinea1,VUnit YLinea5);
    void AdjustTDinamico(VUnit YLinea1,VUnit YLinea5);
    void AdjustTGenerico(VUnit YLinea1,VUnit YLinea5);
    void AdjustSordina(VUnit YLinea1,VUnit YLinea5);
    void AdjustAnnotazione(VUnit YLinea1,VUnit YLinea5);
    void AdjustArpeggio();
    CmdResult CmdAccordoArpeggio();
	CmdResult CmdAccordoArpeggioUp();
	CmdResult CmdAccordoArpeggioDown();
    CmdResult CmdAccordoNotina2Nota();
    CmdResult CmdAccordoNota2Notina();
    CmdResult CmdAccordoNota2Acciaccatura();
	
protected:
    ClassID NoteID;
	// Identificatore del tipo di nota che compone l'accordo. Un accordo
	// puo' contenere note di un solo tipo.
    
    Bool DiteggiatoUp;
	// E' TRUE se le diteggiature delle singole note vanno disegnate sopra
	// l'accordo, FALSE se vanno disegnate sotto.
    
    Arpeggio *PArp;
	// Puntatore alla classe Arpeggio.
	
//	void DrawContour();
    
public: 
    Accordo();
    ~Accordo();
	
    void Suona(MoodsMidiWrite *);
	void SuonaWithAppoggiatura(MoodsMidiWrite* mmw,float durata_appoggiatura);

    void Init();
	// Inizializza l'accordo come lista vuota. 
    
    ClassID GetNoteID();
	// Restituisce il tipo di nota che compone l'accordo.
    
    void SetDiteggiatoUp(Bool dup);
	// Imposta DiteggiatoUp;
    Bool GetDiteggiatoUp(); 
	// Restituisce DiteggiatoUp;
    
    void SetPArpeggio(Arpeggio *pa);
	// Imposta Arpeggio;
    Arpeggio *GetPArpeggio(); 
	// Restituisce Arpeggio;
	
	int GetAcciaccatura(void);
	// per sapere se l'accordo  acciccato oppure no
	
    Bool AddNota(Nota *pn);
	// Aggiunge una nota all'accordo. Se la nota non  dello stesso valore
	// di quelle gia' presenti, non viene aggiunta ed il metodo restituisce
	// FALSE. 
	// Se nell'accordo e' gia' presente una nota della stessa altezza di
	// quella che si vuol inserire, la nota non viene aggiunta ed il metodo
	// restituisce FALSE.
	// Questo metodo rifiuta le notine, cioe' le note che hanno Notina = TRUE.
	// La prima nota che viene inserita nell'accordo ne determina NoteID.
	// Non e' necessario che le note siano presentate allineate:
	// sara' Adjust ad allinearle correttamente in base ad AbsPos.x.
	
	void SetBigAccordo();
	void SetSmallAccordo();
	
    void SetLGambo(VUnit lg);
	// Imposta la lunghezza del gambo.
	// Se l'accordo e' composto da semibrevi, non fa niente.
    
    void SetGamboUp();
	// Imposta il gambo sopra la nota, con lunghezza di default.
	// Se l'accordo e' composto da semibrevi, non fa niente.
    
    void SetGamboDwn();
	// Imposta il gambo sotto la nota, con lunghezza di default.
	// Se l'accordo e' composto da semibrevi, non fa niente.
    
	void SetStatus(Status_ID stat);
	
    void SetCoda(Bool coda);
	// Imposta la coda dell'accordo. Se l'accordo e' composto da semibrevi,
	// minime o semiminime, Coda resta sempre FALSE.
	
    void SetPuntiValore(int np,Bool updateSpG=TRUE);
	// Imposta np punti di valore a tutte le note che compongono l'accordo.
	// np deve essere compreso fra 1 e 3, altrimenti il metodo non ha 
	// effetto. 
    
    void SetPos(DrawObject *d,const Point &p);
	// Imposta la posizione dell'accordo, ignorandone l'Altezza, e chiama
	// Adjust().
	// LGambo deve essere gia' stata impostata.                         
	// Imposta anche l'Altezza dell'accordo,
	// presa come l'altezza della nota superiore se il gambo e' sopra o
	// l'altezza della nota inferiore se il gambo e' sotto [questa con-
	// venzione serve affinche' GruppoNote tratti l'accordo come una nota
	// semplice]. Dall'altezza dell'accordo viene determinata anche la sua
	// AbsPos (l'allineamento verra' fatto in base a quest'ultima). 
	// Se si vogliono aggiungere note all'accordo, dopo l'aggiunta occor-
	// re rieseguire questo metodo.
	
    void SetPosWA(DrawObject *d,const Point &p);
	// Uguale a SetPos(), ma non chiama Adjust().
	
    void AdjustF();
	// Aggiusta la posizione orizzontale delle note dell'accordo in base
	// alla direzione del gambo e predispone il disegno del gambo.
	// Va eseguito dopo aver impostato il gambo e l'eventuale coda
	// dell'accordo.
	// Se LGambo=0, questo metodo si comporta come se LGambo fosse
	// maggiore di 0.
    void Adjust(Bool primaleg, Battuta *pBattuta, short current_layer,short current_staff); 
    
    void Draw(); 
    
    void Free();
	// Cancella la lista e dealloca le note.
    
    Bool Hit(const Point&, SelObj, DrawObject*&, SymPath&);
    Bool SymbolicHit(SymPath&, SelObj, DrawObject*&);
    
    Bool FigPtr2SymPath(SymPath&, Figura*, Bool inGruppo=FALSE);
	// ricostruisce il PATH di una figura a partire dal puntatore.
	
    Bool DeleteSym(DrawObject *sym);
	// Cancella il simbolo specificato se e' fra quelli connessi all'accordo.
	// Ritorna TRUE se il simbolo e' stato cancellato
	
    int GetDiffUpperLower();
	// Ritorna un diverso valore a seconda della differenza tra la 
	// distanza della nota superiore di un accordo dalla terza linea
	// del pentagramma e la distanza della nota inferiore dell'accordo
	// dalla stessa linea.
    int GetDiffNUpNDown();
	// Ritorna un diverso valore a seconda della differenza tra il 
	// numero di note di un accordo che stanno sopra la terza linea
	// del pentagramma e il numero di note dello stesso accordo che 
	// stanno sotto.
    int GetNumUp();
    int GetNumDown();
	
    Nota * GetLastNote(void);
	Nota * GetPrevNote(Nota * pN);
	Nota * GetNextNote(Nota * pN);
    Nota * GetFirstNote(void);

    
    void Select();
    void Deselect();
    
	virtual void Raise(int delta_altezza);

    CmdResult SymCommand(SymCmd& SCmd, Battuta* pBattuta);
	virtual DrawObject * GetAggiunta(SymCmd * MyCmd);
	
	int GetAltezzaMedia();
	void GetMinMaxStaff(short& minStaff, short& maxStaff);
	
	int SaveXML(FILE *fp, Figura *f);
};

#endif //ACCORDO_HPP
