/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: def_fig.hpp,v 1.1 2005/07/25 16:30:38 bellinpo Exp $ */


#ifndef DEF_FIG_HPP
#define DEF_FIG_HPP

class Scr;

// Classe Figura
#define TAGLIOLFT -6
// Numero di VUnit (in orizzontale) fra il punto piu' a sinistra del taglio
// addizionale ed il centro della nota.
#define TAGLIORGT 5
// Numero di VUnit (in orizzontale) fra il punto piu' a destra del taglio
// addizionale ed il centro della nota.

#define VU2UPNOTA 3
#define VU2DWNNOTA 3
#define VU2LFTNOTA 4
#define VU2RGTNOTA 3  
#define VU2RGTNSEMIBREVE Scr->Get_VU2RG(119)  

// Questi valori sono riferiti al pallino di una nota (la semibreve ha
// ellisse un po' piu' grande.  

#define VU2UPNOTA1 1
#define VU2DWNNOTA1 2
#define VU2LFTNOTA1 2
#define VU2RGTNOTA1 1  
#define VU2RGTNSEMIBREVE1 2
// Questi valori sono riferiti al pallino di una notina (la semibreve ha
// ellisse un po' piu' grande conb due barre 

#define SPNOTAALTERAZIONE 4
// Spazio orizzontale fra una nota e la sua alterazione.
#define SPNOTAPUNTOVALORE 1
// Spazio orizzontale fra una nota ed il suo punto di valore.
#define RECTSPACEY 1
// Spazio verticale fra il rettangolo che circonda la figura ed i rettan-
// goli dei simboli connessi alla figura.

#define SPACECOLOR gray
// Colore con il quale disegnare (quando necessario) gli spazi.

// Classe Nota
#define DELTAXGAMBOUP 4
// Numero di VUnit (prese in orizzontale) fra il centro di una nota
// ed il suo gambo superiore.            
#define DELTAXGAMBODWN -4
// Numero di VUnit (prese in orizzontale) fra il centro di una nota
// ed il suo gambo inferiore.            
#define LGAMBODEF 20
// Lunghezza di default (in VUnit) del gambo di una nota.           

#define DELTAXNOTA Scr->Get_VU2DX(216)		
#define DELTAYNOTA Scr->Get_VU2DY(216)		
// Correzione da apportare alla posizione fisica dell'ellisse della nota.
// In tal modo il centro dell'ellisse si trovera' sullo schermo nella
// posizione specificata da AbsPos.

#define CODELLISSE 216-256
// Coodice ASCII (216) dell'ellisse per le note piene.
// La sottrazione serve per convertire a char senza che
// il compilatore dia warning.

// Codice ASCII (164) del simbolo di nota Breve.
#define CODBREVE 164-256
#define VU2UPBREVE Scr->Get_VU2UP(164)      
#define VU2DWNBREVE Scr->Get_VU2DWN(164)	
#define VU2RGTBREVE Scr->Get_VU2RG(164)		
#define VU2LFTBREVE Scr->Get_VU2LF(164)		
#define DELTAXBREVE Scr->Get_VU2DX(164)		
#define DELTAYBREVE Scr->Get_VU2DY(164)		

#define CODMINIMA 163-256
// Codice ASCII (163) dell'ellisse per la minima.              
#define CODSEMIBREVE 119
// Codice ASCII (119) dell'ellisse per la semibreve.
#define COD1CODAUP 106
// Codice ASCII (106) della coda piu' bassa della nota con gambo sopra.
#define COD0CODAUP 108
// Codice ASCII (108) del pezzo di coda piu' basso della nota con gambo sopra.
#define CODCODAUP 150-256
// Codice ASCII (150) delle code superiori della nota con gambo sotto.
#define COD0CODADWN 71
// Codice ASCII (71) del pezzo di coda piu' basso della nota con gambo sotto.
#define COD1CODADWN 74
// Codice ASCII (74) della coda piu' alta della nota con gambo sotto.
#define CODCODADWN 208-256
// Codice ASCII (208) delle code inferiori della nota con gambo sotto.

#define DELTAXCODAUP Scr->Get_VU2DX(106)       
// Differenza fra la x del gambo della nota e la x a cui disegnare le code
// superiori.
#define DELTAYCODAUP Scr->Get_VU2DY(106)       
// Differenza fra la y del punto piu' alto del gambo della nota e la y a
// cui disegnare la prima coda superiore.
#define DELTAXCODADWN Scr->Get_VU2DX(208)       
// Differenza fra la x del gambo della nota e la x a cui disegnare le code
// inferiori.
#define DELTAYCODADWN Scr->Get_VU2DY(208)       
// Differenza fra la y del punto piu' basso del gambo della nota e la y a
// cui disegnare la prima coda inferiore.
#define DELTAYCODE 6                       
// Differenza di y fra le code della nota. 

#define VU2RGTFCODA 11
// VU2RgtF delle note che hanno coda a destra.  

#define INCRNOTASENZACODA 5
// Questa quantita' viene aggiunta a VU2UpF o VU2DwnF delle note senza
// coda. Se una nota e' senza coda, e' segno che fa parte di un gruppo.
// Percio' il rettangolo che la circonda va aumentato in modo da non 
// disegnare dentro le barrette i simboli sopra o sotto la nota.

/**********************************************************/

// classi delle ripetizioni

// Codice ASCII (151) del simbolo di ripetizione del tempo (una barra inclinata)
#define CODRIPTEMPO 151-256
#define VU2UPRIPTEMPO Scr->Get_VU2UP(151)      
#define VU2DWNRIPTEMPO Scr->Get_VU2DWN(151)    
#define VU2RGTRIPTEMPO Scr->Get_VU2RG(151)     
#define VU2LFTRIPTEMPO Scr->Get_VU2LF(151)     
#define DELTAXRIPTEMPO Scr->Get_VU2DX(151)     
#define DELTAYRIPTEMPO Scr->Get_VU2DY(151)     

// Codice ASCII (153) ripetizione mezza battuta
#define CODRIPMBAT 153-256
#define DELTAXRIPMEZZABATTUTA Scr->Get_VU2DX(153)      
#define DELTAYRIPMEZZABATTUTA Scr->Get_VU2DY(153)      
#define VU2UPRIPMEZZABATTUTA Scr->Get_VU2UP(153)       
#define VU2DWNRIPMEZZABATTUTA Scr->Get_VU2DWN(153)     
#define VU2RGTRIPMEZZABATTUTA Scr->Get_VU2RG(153)      
#define VU2LFTRIPMEZZABATTUTA Scr->Get_VU2LF(153)      

// Codice ASCII (152) ripetizione della battuta
#define CODRIPBAT 152-256
#define VU2UPRIPBATTUTA Scr->Get_VU2UP(152)        
#define VU2DWNRIPBATTUTA Scr->Get_VU2DWN(152)      
#define VU2RGTRIPBATTUTA Scr->Get_VU2RG(152)       
#define VU2LFTRIPBATTUTA Scr->Get_VU2LF(152)       
#define DELTAXRIPBATTUTA Scr->Get_VU2DX(152)       
#define DELTAYRIPBATTUTA Scr->Get_VU2DY(152)       

/////////////NOTINE////////////////////////////////////////////////////////////

#define CODELLISSE1 89
// Coodice ASCII (89) dell'ellisse per le notine piene.

// Codice ASCII (174) del simbolo per la notina Breve.
#define CODBREVE1 174-256
#define VU2UPBREVE1 Scr->Get_VU2UP(174)     
#define VU2DWNBREVE1 Scr->Get_VU2DWN(174)	
#define VU2RGTBREVE1 Scr->Get_VU2RG(174)		
#define VU2LFTBREVE1 Scr->Get_VU2LF(174)		
#define DELTAXBREVE1 Scr->Get_VU2DX(174)
#define DELTAYBREVE1 Scr->Get_VU2DY(174)	

#define CODMINIMA1 92
// Codice ASCII (92) dell'ellisse per la notina minima.              
#define CODSEMIBREVE1 91
// Codice ASCII (91) dell'ellisse per la notina semibreve.
#define CODCODAUP1 59
// Codice ASCII (59) delle code superiori della notina con gambo sotto.
#define CODCODADWN1 58
// Codice ASCII (58) delle code inferiori della notina con gambo sotto.

#define DELTAXCODAUP1 Scr->Get_VU2DX(58)       
// Differenza fra la x del gambo della notina e la x a cui disegnare le code
// superiori.
#define DELTAYCODAUP1 Scr->Get_VU2DY(58)       
// Differenza fra la y del punto piu' alto del gambo della notina e la y a
// cui disegnare la prima coda superiore.
#define DELTAXCODADWN1 Scr->Get_VU2DX(59)       
// Differenza fra la x del gambo della notina e la x a cui disegnare le code
// inferiori.
#define DELTAYCODADWN1 Scr->Get_VU2DY(59)       
// Differenza fra la y del punto piu' basso del gambo della notina e la y a
// cui disegnare la prima coda inferiore.
#define DELTAYCODE1 4                      
// Differenza di y fra le code della notina.  

#define VU2RGTFCODA1 6
// VU2RgtF delle notine che hannno coda a destra.

#define DELTAXACCIACC 3
#define DELTAYACCIACC 2

// Correzioni da apportare al disegno del taglio obliquo dell'acciaccatura.

/**********************************************************/

// Classi delle Pause

// Codice ASCII (146) della pausa generica
#define CODPGENERICA 146-256
#define DELTAXPGENERIC Scr->Get_VU2DX(146)    		  
#define DELTAYPGENERIC Scr->Get_VU2DY(146)    		  
#define VU2UPPBATTUTAGENERICA Scr->Get_VU2UP(146)     
#define VU2DWNPBATTUTAGENERICA Scr->Get_VU2DWN(146)   
#define VU2RGTPBATTUTAGENERICA Scr->Get_VU2RG(146)    
#define VU2LFTPBATTUTAGENERICA Scr->Get_VU2LF(146)    

// Codice ASCII (149) della pausa di quattro battute
#define CODPQUATTROBATTUTE 149-256
#define DELTAXPQUATTROBATTUTE Scr->Get_VU2DX(149)     
#define DELTAYPQUATTROBATTUTE Scr->Get_VU2DY(149)     
#define VU2UPPQUATTROBATTUTE Scr->Get_VU2UP(149)      
#define VU2DWNPQUATTROBATTUTE Scr->Get_VU2DWN(149)    
#define VU2RGTPQUATTROBATTUTE Scr->Get_VU2RG(149)     
#define VU2LFTPQUATTROBATTUTE Scr->Get_VU2LF(149)     

// Codice ASCII (148) della pausa di due battute
#define CODPDUEBATTUTE 148-256
#define VU2UPPDUEBATTUTE Scr->Get_VU2UP(148)     
#define VU2DWNPDUEBATTUTE Scr->Get_VU2DWN(148)   
#define VU2RGTPDUEBATTUTE Scr->Get_VU2RG(148)    
#define VU2LFTPDUEBATTUTE Scr->Get_VU2LF(148)    
#define DELTAXPDUEBATTUTE Scr->Get_VU2DX(148)    
#define DELTAYPDUEBATTUTE Scr->Get_VU2DY(148)    

// Codice ASCII (140) del rettangolo per le pause di semibreve e minima.
// Queste VUnit servono per disegnare una piccola riga che serve da
// appoggio per le pause di semibreve e minima quando sono disegnate
// fuori dal pentagramma.
#define CODRETTANGOLINO 138-256
#define APPOGGIOPAUSALFT Scr->Get_VU2LF(CODRETTANGOLINO) 
#define APPOGGIOPAUSARGT Scr->Get_VU2RG(CODRETTANGOLINO)

// pausa da 4/4 un'intera misura, semibreve
#define CODE_FITTIZIO_PSEMIBREVE 7
#define DELTAXPSEMIBREVE Scr->Get_VU2DX(CODE_FITTIZIO_PSEMIBREVE) 
#define DELTAYPSEMIBREVE Scr->Get_VU2DY(CODE_FITTIZIO_PSEMIBREVE)
#define VU2UPFPSEMIBREVE Scr->Get_VU2UP(CODE_FITTIZIO_PSEMIBREVE)  		
#define VU2RGTFPSEMIBREVE Scr->Get_VU2RG(CODE_FITTIZIO_PSEMIBREVE)  		
#define VU2DWNFPSEMIBREVE Scr->Get_VU2DWN(CODE_FITTIZIO_PSEMIBREVE)       		
#define VU2LFTFPSEMIBREVE Scr->Get_VU2LF(CODE_FITTIZIO_PSEMIBREVE)       	

// pausa da 2/4, mezza misura, MINIMA
#define CODE_FITTIZIO_PMINIMA 8
#define DELTAXPMINIMA Scr->Get_VU2DX(CODE_FITTIZIO_PMINIMA)
#define DELTAYPMINIMA Scr->Get_VU2DY(CODE_FITTIZIO_PMINIMA)
#define VU2UPFPMINIMA Scr->Get_VU2UP(CODE_FITTIZIO_PMINIMA)  		
#define VU2RGTFPMINIMA Scr->Get_VU2RG(CODE_FITTIZIO_PMINIMA)  		
#define VU2DWNFPMINIMA Scr->Get_VU2DWN(CODE_FITTIZIO_PMINIMA)       		
#define VU2LFTFPMINIMA Scr->Get_VU2LF(CODE_FITTIZIO_PMINIMA)       	

// Codice ASCII (215) della pausa di un quarto.
#define CODPSEMIMINIMA 215-256
#define DELTAXPSEMIMINIMA Scr->Get_VU2DX(215)  
#define DELTAYPSEMIMINIMA Scr->Get_VU2DY(215)  
#define VU2UPFPSEMIMINIMA Scr->Get_VU2UP(215)  		
#define VU2RGTFPSEMIMINIMA Scr->Get_VU2RG(215)  		
#define VU2DWNFPSEMIMINIMA Scr->Get_VU2DWN(215)       		
#define VU2LFTFPSEMIMINIMA Scr->Get_VU2LF(215)       	

// Codice ASCII (132) della pausa di un ottavo.
#define CODPCROMA 132-256
#define DELTAXPCROMA Scr->Get_VU2DX(132)       
#define DELTAYPCROMA Scr->Get_VU2DY(132)       
#define VU2UPFPCROMA Scr->Get_VU2UP(132)       		
#define VU2RGTFPCROMA Scr->Get_VU2RG(132)       	
#define VU2DWNFPCROMA Scr->Get_VU2DWN(132)       		
#define VU2LFTFPCROMA Scr->Get_VU2LF(132)       	

// Codice ASCII (143) della pausa di un sedicesimo.
#define CODPSEMICROMA 101
#define DELTAXPSEMICROMA Scr->Get_VU2DX(CODPSEMICROMA)   
#define DELTAYPSEMICROMA Scr->Get_VU2DY(CODPSEMICROMA)   
#define VU2UPFPSEMICROMA Scr->Get_VU2UP(CODPSEMICROMA)  	    
#define VU2RGTFPSEMICROMA Scr->Get_VU2RG(CODPSEMICROMA)  	     
#define VU2DWNFPSEMICROMA Scr->Get_VU2DWN(CODPSEMICROMA)       		
#define VU2LFTFPSEMICROMA Scr->Get_VU2LF(CODPSEMICROMA)       	

// codice pausa da un 32 esimo
// Codice ASCII (249) della pausa di un trentaduesimo.
#define CODPBISCROMA 90  
#define DELTAXPBISCROMA Scr->Get_VU2DX(CODPBISCROMA)    
#define DELTAYPBISCROMA Scr->Get_VU2DY(CODPBISCROMA)    
#define VU2UPFPBISCROMA Scr->Get_VU2UP(CODPBISCROMA)         
#define VU2RGTFPBISCROMA Scr->Get_VU2RG(CODPBISCROMA)  	     
#define VU2DWNFPBISCROMA Scr->Get_VU2DWN(CODPBISCROMA)       		
#define VU2LFTFPBISCROMA Scr->Get_VU2LF(CODPBISCROMA)       	

// Codice ASCII (147) della pausa di un sessantaquattresimo.
#define CODPSEMIBISCROMA 147-256
#define DELTAXPSEMIBISCROMA Scr->Get_VU2DX(147)  
#define DELTAYPSEMIBISCROMA Scr->Get_VU2DY(147)  
#define VU2UPFPSEMIBISCROMA Scr->Get_VU2UP(147)     
#define VU2RGTFPSEMIBISCROMA Scr->Get_VU2RG(147)  	
#define VU2DWNFPSEMIBISCROMA Scr->Get_VU2DWN(147)       		
#define VU2LFTFPSEMIBISCROMA Scr->Get_VU2LF(147)       	

// Codice ASCII (75) della pausa fusa
#define CODPFUSA 75
#define VU2UPFPFUSA Scr->Get_VU2UP(CODPFUSA)   
#define VU2RGTFPFUSA Scr->Get_VU2RG(CODPFUSA)  
#define DELTAXPFUSA Scr->Get_VU2DX(CODPFUSA)   
#define DELTAYPFUSA Scr->Get_VU2DY(CODPFUSA)   
#define VU2DWNFPFUSA Scr->Get_VU2DWN(CODPFUSA)   
#define VU2LFTFPFUSA Scr->Get_VU2LF(CODPFUSA)  

/*************************************************************************/

// PAUSINE

// Codice ASCII (200) della pausa di quattro battute
#define CODPQUATTROBATTUTES 200
#define DELTAXPQUATTROBATTUTES Scr->Get_VU2DX(CODPQUATTROBATTUTES, MUSICFONT2_ID)     
#define DELTAYPQUATTROBATTUTES Scr->Get_VU2DY(CODPQUATTROBATTUTES, MUSICFONT2_ID)     
#define VU2UPPQUATTROBATTUTES Scr->Get_VU2UP(CODPQUATTROBATTUTES, MUSICFONT2_ID)      
#define VU2DWNPQUATTROBATTUTES Scr->Get_VU2DWN(CODPQUATTROBATTUTES, MUSICFONT2_ID)    
#define VU2RGTPQUATTROBATTUTES Scr->Get_VU2RG(CODPQUATTROBATTUTES, MUSICFONT2_ID)     
#define VU2LFTPQUATTROBATTUTES Scr->Get_VU2LF(CODPQUATTROBATTUTES, MUSICFONT2_ID)     

// Codice ASCII (199) della pausa di due battute
#define CODPDUEBATTUTES 199
#define VU2UPPDUEBATTUTES Scr->Get_VU2UP(CODPDUEBATTUTES, MUSICFONT2_ID)     
#define VU2DWNPDUEBATTUTES Scr->Get_VU2DWN(CODPDUEBATTUTES, MUSICFONT2_ID)   
#define VU2RGTPDUEBATTUTES Scr->Get_VU2RG(CODPDUEBATTUTES, MUSICFONT2_ID)    
#define VU2LFTPDUEBATTUTES Scr->Get_VU2LF(CODPDUEBATTUTES, MUSICFONT2_ID)    
#define DELTAXPDUEBATTUTES Scr->Get_VU2DX(CODPDUEBATTUTES, MUSICFONT2_ID)    
#define DELTAYPDUEBATTUTES Scr->Get_VU2DY(CODPDUEBATTUTES, MUSICFONT2_ID)    

// Codice ASCII (140) del rettangolo per le pause di semibreve e minima.
// Queste VUnit servono per disegnare una piccola riga che serve da
// appoggio per le pause di semibreve e minima quando sono disegnate
// fuori dal pentagramma. viene diseganto anche dentro
#define CODRETTANGOLINOS 207
#define APPOGGIOPAUSALFTS Scr->Get_VU2LF(CODRETTANGOLINOS, MUSICFONT2_ID) 
#define APPOGGIOPAUSARGTS Scr->Get_VU2RG(CODRETTANGOLINOS, MUSICFONT2_ID)

// pausa da 4/4 un'intera misura, semibreve
#define CODE_FITTIZIO_PSEMIBREVES 5
#define DELTAXPSEMIBREVES Scr->Get_VU2DX(CODE_FITTIZIO_PSEMIBREVES) 
#define DELTAYPSEMIBREVES Scr->Get_VU2DY(CODE_FITTIZIO_PSEMIBREVES)
#define VU2UPFPSEMIBREVES Scr->Get_VU2UP(CODE_FITTIZIO_PSEMIBREVES)  		
#define VU2RGTFPSEMIBREVES Scr->Get_VU2RG(CODE_FITTIZIO_PSEMIBREVES)  		
#define VU2DWNFPSEMIBREVES Scr->Get_VU2DWN(CODE_FITTIZIO_PSEMIBREVES)       		
#define VU2LFTFPSEMIBREVES Scr->Get_VU2LF(CODE_FITTIZIO_PSEMIBREVES)       	

// pausa da 2/4, mezza misura, MINIMA
#define CODE_FITTIZIO_PMINIMAS 6
#define DELTAXPMINIMAS Scr->Get_VU2DX(CODE_FITTIZIO_PMINIMAS)
#define DELTAYPMINIMAS Scr->Get_VU2DY(CODE_FITTIZIO_PMINIMAS)
#define VU2UPFPMINIMAS Scr->Get_VU2UP(CODE_FITTIZIO_PMINIMAS)  		
#define VU2RGTFPMINIMAS Scr->Get_VU2RG(CODE_FITTIZIO_PMINIMAS)  		
#define VU2DWNFPMINIMAS Scr->Get_VU2DWN(CODE_FITTIZIO_PMINIMAS)       		
#define VU2LFTFPMINIMAS Scr->Get_VU2LF(CODE_FITTIZIO_PMINIMAS)       	

// Codice ASCII della pausa di un quarto.
#define CODPSEMIMINIMAS 201
#define DELTAXPSEMIMINIMAS Scr->Get_VU2DX(CODPSEMIMINIMAS, MUSICFONT2_ID)  
#define DELTAYPSEMIMINIMAS Scr->Get_VU2DY(CODPSEMIMINIMAS, MUSICFONT2_ID)  
#define VU2UPFPSEMIMINIMAS Scr->Get_VU2UP(CODPSEMIMINIMAS, MUSICFONT2_ID)  		
#define VU2RGTFPSEMIMINIMAS Scr->Get_VU2RG(CODPSEMIMINIMAS, MUSICFONT2_ID)  		
#define VU2DWNFPSEMIMINIMAS Scr->Get_VU2DWN(CODPSEMIMINIMAS, MUSICFONT2_ID)       		
#define VU2LFTFPSEMIMINIMAS Scr->Get_VU2LF(CODPSEMIMINIMAS, MUSICFONT2_ID)       	

// Codice ASCII della pausa di un ottavo.
#define CODPCROMAS 202
#define DELTAXPCROMAS Scr->Get_VU2DX(CODPCROMAS, MUSICFONT2_ID)       
#define DELTAYPCROMAS Scr->Get_VU2DY(CODPCROMAS, MUSICFONT2_ID)       
#define VU2UPFPCROMAS Scr->Get_VU2UP(CODPCROMAS, MUSICFONT2_ID)       		
#define VU2RGTFPCROMAS Scr->Get_VU2RG(CODPCROMAS, MUSICFONT2_ID)       	
#define VU2DWNFPCROMAS Scr->Get_VU2DWN(CODPCROMAS, MUSICFONT2_ID)       		
#define VU2LFTFPCROMAS Scr->Get_VU2LF(CODPCROMAS, MUSICFONT2_ID)       	

// Codice ASCII della pausa di un sedicesimo.
#define CODPSEMICROMAS 203
#define DELTAXPSEMICROMAS Scr->Get_VU2DX(CODPSEMICROMAS, MUSICFONT2_ID)   
#define DELTAYPSEMICROMAS Scr->Get_VU2DY(CODPSEMICROMAS, MUSICFONT2_ID)   
#define VU2UPFPSEMICROMAS Scr->Get_VU2UP(CODPSEMICROMAS, MUSICFONT2_ID)  	    
#define VU2RGTFPSEMICROMAS Scr->Get_VU2RG(CODPSEMICROMAS, MUSICFONT2_ID)  	     
#define VU2DWNFPSEMICROMAS Scr->Get_VU2DWN(CODPSEMICROMAS, MUSICFONT2_ID)       		
#define VU2LFTFPSEMICROMAS Scr->Get_VU2LF(CODPSEMICROMAS, MUSICFONT2_ID)       	

// codice pausa da un 32 esimo
#define CODPBISCROMAS 204
#define DELTAXPBISCROMAS Scr->Get_VU2DX(CODPBISCROMAS, MUSICFONT2_ID)    
#define DELTAYPBISCROMAS Scr->Get_VU2DY(CODPBISCROMAS, MUSICFONT2_ID)    
#define VU2UPFPBISCROMAS Scr->Get_VU2UP(CODPBISCROMAS, MUSICFONT2_ID)         
#define VU2RGTFPBISCROMAS Scr->Get_VU2RG(CODPBISCROMAS, MUSICFONT2_ID)  	     
#define VU2DWNFPBISCROMAS Scr->Get_VU2DWN(CODPBISCROMAS, MUSICFONT2_ID)       		
#define VU2LFTFPBISCROMAS Scr->Get_VU2LF(CODPBISCROMAS, MUSICFONT2_ID)       	

// Codice ASCII (205) della pausa di un sessantaquattresimo.
#define CODPSEMIBISCROMAS 205
#define DELTAXPSEMIBISCROMAS Scr->Get_VU2DX(CODPSEMIBISCROMAS, MUSICFONT2_ID)  
#define DELTAYPSEMIBISCROMAS Scr->Get_VU2DY(CODPSEMIBISCROMAS, MUSICFONT2_ID)  
#define VU2UPFPSEMIBISCROMAS Scr->Get_VU2UP(CODPSEMIBISCROMAS, MUSICFONT2_ID)     
#define VU2RGTFPSEMIBISCROMAS Scr->Get_VU2RG(CODPSEMIBISCROMAS, MUSICFONT2_ID)  	
#define VU2DWNFPSEMIBISCROMAS Scr->Get_VU2DWN(CODPSEMIBISCROMAS, MUSICFONT2_ID)       		
#define VU2LFTFPSEMIBISCROMAS Scr->Get_VU2LF(CODPSEMIBISCROMAS, MUSICFONT2_ID)       	

//DAFARE Codice ASCII (206) della pausa fusa
#define CODPFUSAS 206
#define VU2UPFPFUSAS Scr->Get_VU2UP(CODPFUSAS, MUSICFONT2_ID)   
#define VU2RGTFPFUSAS Scr->Get_VU2RG(CODPFUSAS, MUSICFONT2_ID)  
#define DELTAXPFUSAS Scr->Get_VU2DX(CODPFUSAS, MUSICFONT2_ID)   
#define DELTAYPFUSAS Scr->Get_VU2DY(CODPFUSAS, MUSICFONT2_ID)   
#define VU2DWNFPFUSAS Scr->Get_VU2DWN(CODPFUSAS, MUSICFONT2_ID)   
#define VU2LFTFPFUSAS Scr->Get_VU2LF(CODPFUSAS, MUSICFONT2_ID)  


/****************************************************************/

// Classe Accordo

#define SHIFTNOTARGT 8
#define SHIFTNOTALFT -7
#define SHIFTNOTINARGT 4
#define SHIFTNOTINALFT -4

/***********************************************************/

// Classe GruppoNote

#define BSLOPEMAX 0.5 
// Inclinazione massima delle barrette del gruppo.

#define BSLOPEMIN 0.15 
// Inclinazione minima delle barrette del gruppo.

#define THICKBARRETTA 3
//#define THICKBARRETTA 4   
// Spessore di una barretta del gruppo. 

#define THICKBARRETTA_PRINT 4 

#define DELTAYBARRETTE 6
// Differenza di altezza fra le barrette del gruppo.    

#define LGAMBOMIN 10 
// Lunghezza minima del gambo di una nota. 

#define THICKBARRETTA1 2
// Spessore di una barretta del gruppo di notine. 

#define DELTAYBARRETTE1 4
// Differenza di altezza fra le barrette del gruppo di notine.    

#define SPACEWIDTHDEF 10
// Serve per inizializzare. DA PRENDERE DA ALTRE CLASSI!


#define INC_ALTEZZA 127		
// i valori che puo' prendere Altezza possono essere negativi 
// col significato di figura sotto il pentagramma, e questo da noia per 
// la trasmissione sotto rete: per ovviare a cio'in InsertHit il 
// valore di Altezza viene settato aggiungendoci INC_ALTEZZA (sperando
// che cio' basti a rendere positivo Altezza).
// Il valore originale di Altezza viene Ripristinato in InsPausa e InsNota

enum Status_ID {
	
	NORMAL,
		GRACED,
		HIDDEN,
		GHOSTED
};


#endif //DEF_FIG_HPP