/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include "wx/wx.h"

#include "lioo.hpp"
#include "liooext.hpp"
#include "fig/figura.hpp"
#include "drw/gscreen.hpp"
#include "fig/pausa.hpp"
#include "fig/spazio.hpp"
#include "bat/battuta.hpp"
#include "int/legatqua.hpp"
#include "str/ditafile.hpp"
#include "str/pedaldown.hpp"
#include "str/pedalup.hpp"
#include "drw/musicedt.hpp"
#include "gui/dlg/linkdlg.hpp"
#include "smr/smr.hpp"
#include "smr/smrmain.hpp" 
#include "smr/dlg/settings.hpp"

extern wxMusicEditor *MusicEditor;

extern int DebugNormalMilla;
extern int DebugNormalMillaPos;
extern wxString msgdebug;

extern bool visible;
// per vedere la variabile di stato globale


// VARIABILI DI STATO
// inizializzo la variabile di stato che mi dice di fare il
// line breaking automatico
int AutoLineBreak=FALSE;
// vista del direttoriale (0) o dell'orchestrale (1)
int DirOrOrc=0;
// variabile di stato che mi dice se sono in computer view o in print preview


Figura::Figura()
{  
	durata=0.;
	Init();
	/////////////////////////////////////////////////////////////////////////////
	// nel costruttore di Figura inizializzo i valori di spaziatura con un valore
	// proporzionale alla durata
	// il valore di default (da rivedere) e' pari a 1500UL (15 pixel).
	// Non si tiene conto della spaziatura di blocco
	
	SpG[0] =  SpG[1] = (int)(GetDurata() * SPG_DEF_FACTOR);   // 1/16
	SpOr[0]= SpOr[1] = SpG[0];   // anche se non giustificato inizializza 
	SpDirUt[0]= SpDirUt[1] = 0;  
	SpOrcUt[0]= SpOrcUt[1]=0;
	SpDLine[0]=SpDLine[1]=0; 
	SpOLine[0]=SpOLine[1]=0;
	pBattuta=NULL;
} 

Figura::Figura(double dur)
{  
	pBattuta=NULL; 
	Init();
	/////////////////////////////////////////////////////////////////////////////
	//
	// nel costruttore di Figura inizializzo i valori di spaziatura con un valore
	// proporzionale alla durata
	// il valore di default (da rivedere) e' pari a 1500UL (~15 pixel) per un quarto.
	// Non si tiene conto della spaziatura di blocco
	durata=float(dur);
	
	SpG[0] =  SpG[1] = (int)(GetDurata() * SPG_DEF_FACTOR);   // 1/16
	SpOr[0]= SpOr[1] = SpG[0];   // anche se non giustificato inizializza 
	SpDirUt[0]= SpDirUt[1] = 0;  
	SpOrcUt[0]= SpOrcUt[1]=0;
	SpDLine[0]=SpDLine[1]=0; 
	SpOLine[0]=SpOLine[1]=0;	
} 


Figura::~Figura()
{ 
	Free();
}

void Figura::Init()
{ 
	ID=CL_FIGURA;
	
	SetNStaff(0);
	
	int i=0;
	while(i<4)
	{
		AppSil[i]=0;
		i++;
	};
	
	
	// Presuppone che la figura al momento della sua
	// costruzione non abbia alcuna sillaba,di alcuna
	// strofa,associata.
	SetAltezza(0);
	// Presuppone che la figura occupera' un solo spazio.                
	SetNumSpaces(1);
	SetPCorona(NULL);
	SetPOcchiali(NULL);
	SetPDitaCorde(NULL);
	SetPPuntoValore(NULL);
	SetPTDinamico(NULL);
	SetPTGenerico(NULL);
	SetPAnnotazione(NULL);
	SetPStrumento(NULL);
	SetParentFig(NULL);
	
	// Imposta il rettangolo che circonda la nota.
	SetVU2UpF(VU2UPNOTA);
	SetVU2DwnF(VU2DWNNOTA);
	SetVU2LftF(VU2LFTNOTA);
	SetVU2RgtF(VU2RGTNOTA); 
	// Imposta il rettangolo che circonda la nota completa di simboli.
	SetVU2Up(VU2UPNOTA);
	SetVU2Dwn(VU2DWNNOTA);
	SetVU2Lft(VU2LFTNOTA);
	SetVU2Rgt(VU2RGTNOTA);
	
	// Il numcode 0 indica che il numeric code non e' stato assegnato.
	SetNumericCode(0);
	terz=NULL;
	
	SetNotina(FALSE);
	nIntRefs=0;
	intRefs=NULL;
	// Status normal indica che la figura e' visibile e time-space consuming
	status=NORMAL;
};


/////////////////////////////////////////////////////////////////////////
void Figura::SetSpG(UL spazi)
{
	SpG[currentViewType] = spazi;
};

void Figura::SetSpG(UL spazi, int vorp)
{
	SpG[vorp] = spazi;
};

void Figura::SetSpOr(UL spazi)
{
	SpOr[currentViewType] = spazi;
};

void Figura::SetSpOr(UL spazi, int vorp)
{
	SpOr[vorp] = spazi;
};

void Figura::SetSpDirUt(UL spazi)
{
	SpDirUt[currentViewType] = spazi;
};

void Figura::SetSpDirUt(UL spazi, int vorp)
{
	SpDirUt[vorp] = spazi;
};

void Figura::SetSpOrcUt(UL spazi)
{
	SpOrcUt[currentViewType] = spazi;
};

void Figura::SetSpOrcUt(UL spazi, int vorp)
{
	SpOrcUt[vorp] = spazi;
};

void Figura::SetSpDLine(UL spazi)
{
	SpDLine[currentViewType] = spazi;
};

void Figura::SetSpOLine(UL spazi)
{
	SpOLine[currentViewType] = spazi;
};


UL Figura::GetSpG()
{
	return SpG[currentViewType];
};

UL Figura::GetSpG(int vorp)
{
	return SpG[vorp];
};


UL Figura::GetSpOr()
{
	return SpOr[currentViewType];
};

UL Figura::GetSpOr(int vorp)
{
	return SpOr[vorp];
};


UL Figura::GetSpDirUt()
{
	return SpDirUt[currentViewType];
};

UL Figura::GetSpDirUt(int vorp)
{
	return SpDirUt[vorp];
};


UL Figura::GetSpOrcUt()
{
	return SpOrcUt[currentViewType];
};

UL Figura::GetSpOrcUt(int vorp)
{
	return SpOrcUt[vorp];
};


UL Figura::GetSpDLine()
{
	return SpDLine[currentViewType];
};

UL Figura::GetSpOLine()
{
	return SpOLine[currentViewType];
};




/////////////////////////////

// definizione del metodo da ridefinire in futuro, per ora ci si accontenta
// di SpG. Si mettera' una var globale per capire se lo chiama DLIOO o MASAE
// e si faranno i conti con tutti gli incrementi

UL Figura::GetSp2NextFig()
{
	if ((tipoLeggio==LEGGIO_MASAE) || (tipoLeggio==LEGGIO_MASE)) DirOrOrc=0;
	if ((tipoLeggio==LEGGIO_DLIOO) || (tipoLeggio==LEGGIO_PDLOO)) DirOrOrc=1;
	// non autobreak e direttoriale
	if (AutoLineBreak==0 && DirOrOrc==0) return GetSpG()+GetSpDirUt();
    // autobreak e direttoriale
    else if (AutoLineBreak==1 && DirOrOrc==0) return GetSpDLine()+GetSpDirUt();
	// non autobreak e orchestrale  
	else if (AutoLineBreak==0 && DirOrOrc==1) return GetSpOr()+GetSpOrcUt();
	// autobreak e orchestrale
	else if (AutoLineBreak==1 && DirOrOrc==1) return GetSpOLine()+GetSpOrcUt();
	// default !!NON CI DOVREBBE ARRIVARE, ma lo richiede la sintassi
	else return GetSpG()+GetSpDirUt();
	
}

UL Figura::GetSp2NextFigPar(int autolb, int dirororch)
{
	// non autobreak e direttoriale
	if (autolb==0 && dirororch==0) return GetSpG()+GetSpDirUt();
    // autobreak e direttoriale
    else if (autolb==1 && dirororch==0) return GetSpDLine()+GetSpDirUt();
	// non autobreak e orchestrale  
	else if (autolb==0 && dirororch==1) return GetSpOr()+GetSpOrcUt();
	// autobreak e orchestrale
	else if (autolb==1 && dirororch==1) return GetSpOLine()+GetSpOrcUt();
	// default !!NON CI DOVREBBE ARRIVARE, ma lo richiede la sintassi
	else return GetSpG()+GetSpDirUt();
	
}

////////////////////////
Bool Figura::GetNotina()
{
	return Figurina;
}

void Figura::SetNotina(Bool b)
{
	Figurina=b;
}   

//////////////////////////////////////////////////////////////////////////////


void Figura::SetTerzina(LegatQuadra *pleg)
{
	if(terz==NULL) terz=pleg;
}  

void Figura::DeleteTerzina(LegatQuadra *pleg)
{  
	if(terz==pleg) terz=NULL;
}  


LegatQuadra *Figura::GetTerzina()
{
	return terz;
}    

Bool Figura::IsANote()
{
	return FALSE;
}   

Bool Figura::IsARest()
{
	return FALSE;
}  

void Figura::Raise(int delta_altezza)
{
	Altezza += delta_altezza;
}

void Figura::SetAltezza(int alt)
{
	Altezza=alt;
}  

int Figura::GetAltezza()
{ 
	return Altezza;
}

void Figura::SetSegnoAltezza(bool SA)
{
	Segno=SA;
}

Bool Figura::GetSegnoAltezza()
{
	return Segno;
}

void Figura::SetDurata(float dur)
{
	durata=dur;
};

float Figura::GetDurata()
{
	if (GetPPuntoValore()!=NULL)
	{
		switch(GetPPuntoValore()->GetNumPunti())
		{
		case 1:
			return durata+durata/2;
		case 2:
			return durata+durata/2+durata/4;
		case 3:
			return durata+durata/2+durata/4+durata/8;
		} 
	}    
	return durata;
}


float Figura::GetDurata(float den, float num)
{
	float dur=GetDurata();
	if(GetTerzina()!=NULL)
	{
		return GetTerzina()->GetDurataTerzina(dur,den, num);
	}
	return dur;
}

///////////////////////////////////////////////////////////////////


void Figura::SetVU2UpF(VUnit vu)
{
	VU2UpF=vu;
}


void Figura::SetVU2DwnF(VUnit vu)
{
	VU2DwnF=vu;
}


void Figura::SetVU2LftF(VUnit vu)
{
	VU2LftF=vu;
}


void Figura::SetVU2RgtF(VUnit vu)
{
	VU2RgtF=vu;
}


VUnit Figura::GetVU2UpF()
{
	return VU2UpF;
}


VUnit Figura::GetVU2DwnF()
{
	return VU2DwnF;
}


VUnit Figura::GetVU2LftF()
{
	return VU2LftF;
}


VUnit Figura::GetVU2RgtF()
{
	return VU2RgtF;
}


void Figura::SetVU2Up(VUnit vu)
{
	VU2Up=vu;
}


void Figura::SetVU2Dwn(VUnit vu)
{
	VU2Dwn=vu;
}


void Figura::SetVU2Lft(VUnit vu)
{
	VU2Lft=vu;
}


void Figura::SetVU2Rgt(VUnit vu)
{
	VU2Rgt=vu;
}


VUnit Figura::GetVU2Up()
{
	return VU2Up;
}


VUnit Figura::GetVU2Dwn()
{
	return VU2Dwn;
}


VUnit Figura::GetVU2Lft()
{
	return VU2Lft;
}


VUnit Figura::GetVU2Rgt()
{
	return VU2Rgt;
}


void Figura::SetNumSpaces(int ns)
{
	NumSpaces=ns;
}


int Figura::GetNumSpaces()
{
	return NumSpaces;
}  


void Figura::SetNumericCode(NumCode nc)
{
	NumericCode=nc;
}


NumCode Figura::GetNumericCode()
{
	return NumericCode;
}  


void Figura::SetPCorona(Corona *pc)
{
	PCorona=pc;
}

Corona* Figura::GetPCorona()
{
	return PCorona;
}

void Figura::SetPOcchiali(Occhiali *po)
{
	POcchiali=po;
}

Occhiali *Figura::GetPOcchiali()
{
	return POcchiali;
}

void Figura::SetPDitaCorde(DitaCorde *pd)
{
	PDitaCorde=pd;
}

DitaCorde *Figura::GetPDitaCorde()
{
	return PDitaCorde;
}

void Figura::SetPPuntoValore(PuntoValore *ppv,Bool updateSpG)
{
	PPuntoValore=ppv;
	
	
	if(updateSpG && ppv!=NULL)
		SetSpG((int)(GetDurata() * SPG_DEF_FACTOR));  
	
}


PuntoValore* Figura::GetPPuntoValore()
{
	return PPuntoValore;
}

void Figura::SetPTDinamico(TDinamico *ptd)
{
	PTDinamico=ptd;
}                


TDinamico* Figura::GetPTDinamico()
{
	return PTDinamico;
}         


void Figura::SetPTGenerico(TGenerico *ptg)
{
	PTGenerico=ptg;
}                


TGenerico* Figura::GetPTGenerico()
{
	return PTGenerico;
}         

void Figura::SetPAnnotazione(Annotazione *pa)
{
	PAnnotazione = pa;
}                


Annotazione* Figura::GetPAnnotazione()
{
	return PAnnotazione;
}         

void Figura::SetPStrumento(Strumento *pst)
{
	PStrumento=pst;
}

Strumento* Figura::GetPStrumento()
{
	return PStrumento;
}
//////////////////////////////////////////////////////////////////////
void Figura::SetParentFig(Figura *par)
{
	ParentFig=par;
}

Figura* Figura::GetParentFig()
{
	return ParentFig;
} 
//////////////////////////////////////////////////////////////////////
void Figura::SetStatus(Status_ID stat)
{
	switch (stat)
	{
	default:
	case NORMAL:
		SetDurationDuringEdit();
		break;
	case GRACED:
		SetDurata(0);
		break;
	case GHOSTED:
		SetDurata(0);
		SetColor(ROSSO);
		break;
	case HIDDEN:
        SetDurationDuringEdit();
		SetColor(ROSSO);
		break;
	}
	
	status=stat;
}

bool Figura::HastobeDraw()
{
	if(((GetStatus()==HIDDEN)||(GetStatus()==GHOSTED))&&(visible==FALSE))
	{
		return FALSE;
	}
	else
	{
		return TRUE;
	}
}


Status_ID Figura::GetStatus()
{
	return status;
}

void Figura::SetPos(DrawObject *d,const Point& p)
// Per come e' stato realizzato il calcolo della posizione, *d deve
// essere il pentagramma su cui si vuol disegnare la figura.
{
	DrawObject::SetPos(d,p);
	GetAbsPos().y+=Altezza*VU_PER_ROW/2;
	
	SetVU2Up(GetVU2UpF());
	SetVU2Dwn(GetVU2DwnF());
	SetVU2Lft(GetVU2LftF());
	SetVU2Rgt(GetVU2RgtF());  
}; 


void Figura::SetPosWA(DrawObject *d,const Point& p)
{
	DrawObject::SetPos(d,p);
	GetAbsPos().y+=Altezza*VU_PER_ROW/2;
}; 


void Figura::Adjust(Bool primaleg, Battuta* pBattuta, short current_layer,short current_staff)
{
	VUnit YLinea1=0,YLinea5=0;
    // Altezza fisica delle linee piu' bassa e piu' alta del pentagramma.
	
	
	AdjustPuntoValore();
	if(pBattuta!=NULL)
	{
		int nstaff=GetNStaff();
		if(nstaff==-1) //accordo multistaff
		{
			short minStaff,maxStaff;
			GetMinMaxStaff(minStaff,maxStaff);
			
			VUnit YLineaBaseMax=pBattuta->GetIntest(maxStaff)->GetAbsPos().y;
			YLinea1=-pBattuta->GetParentScore()->GetBaseVU2Dwn(maxStaff)+YLineaBaseMax;
			
			VUnit YLineaBaseMin=pBattuta->GetIntest(minStaff)->GetAbsPos().y;
			YLinea5=pBattuta->GetParentScore()->GetBaseVU2Up(minStaff)+YLineaBaseMin;
		}
		else
		{
			VUnit YLineaBase=pBattuta->GetIntest(nstaff)->GetAbsPos().y;
			YLinea1=-pBattuta->GetParentScore()->GetBaseVU2Dwn(nstaff)+YLineaBase;
			YLinea5=pBattuta->GetParentScore()->GetBaseVU2Up(nstaff)+YLineaBase;
		}
	}
	AdjustCorona(YLinea1,YLinea5);
	AdjustTDinamico(YLinea1,YLinea5);
	AdjustTGenerico(YLinea1,YLinea5);
	AdjustAnnotazione(YLinea1,YLinea5);
	AdjustOcchiali(YLinea1,YLinea5);
	AdjustPedalDown(YLinea1,YLinea5);
	AdjustPedalUp(YLinea1,YLinea5);
	AdjustDitaCorde(YLinea1,YLinea5);
} 


void Figura::Draw()
{ 
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	// Disegno dell'eventuale punto di valore.
	if(GetPPuntoValore()!=NULL)
	{
		GetPPuntoValore()->Draw();
	}
	// Disegno dell'eventuale corona.
	if(GetPCorona()!=NULL)
	{
		GetPCorona()->Draw();
	}
	// Disegno degli eventuali occhiali.
	if(GetPOcchiali()!=NULL)
    {
		GetPOcchiali()->Draw();
    }
	// Disegno delle eventuali dita su corde.
	if(GetPDitaCorde()!=NULL)
    {
		GetPDitaCorde()->Draw();
    }
	// Disegno dell'eventuale segno dinamico.
	if(GetPTDinamico()!=NULL)
	{
		GetPTDinamico()->Draw();
	}
	// Disegno dell'eventuale segno Generico.
	if(GetPTGenerico()!=NULL)
	{
		GetPTGenerico()->Draw();
	}             
	// Disegno dell'eventuale segno Annotazione.
	if(GetPAnnotazione()!=NULL)
	{
		GetPAnnotazione()->Draw();
	}             
	// Disegno dell'eventuale strumento.
	if(GetPStrumento()!=NULL)
	{
		GetPStrumento()->Draw();
	}

	if(isSelected)	{
		DrawContour();
	}

	Scr->SetColor(oldcol);
}


Bool Figura::DeleteSym(DrawObject *sym)
{
	
	if(sym==GetTerzina())
		DeleteTerzina((LegatQuadra*)sym);
	
	if(sym==GetPCorona())
	{
		delete GetPCorona();
		SetPCorona(NULL);
		return TRUE;
	}
	if(sym==GetPOcchiali())
	{
		delete GetPOcchiali();
		SetPOcchiali(NULL);
		return TRUE;
	}
	if(sym==GetPDitaCorde())
	{
		delete GetPDitaCorde();
		SetPDitaCorde(NULL);
		return TRUE;
	}
	if(sym==GetPPuntoValore())
	{
		delete GetPPuntoValore();
		SetPPuntoValore(NULL);
		return TRUE;
	}
	if(sym==GetPTDinamico())
	{
		delete GetPTDinamico();
		SetPTDinamico(NULL);
		return TRUE;
	}
	if(sym==GetPTGenerico())
	{
		delete GetPTGenerico();
		SetPTGenerico(NULL);
		return TRUE;
	} 
	if(sym==GetPAnnotazione())
	{
		delete GetPAnnotazione();
		SetPAnnotazione(NULL);
		return TRUE;
	}  
	
	// casi particolari degli strumenti.
	// e' necessario richiamarli dalle figure interessate.
	//  return DeleteStrumento(sym);
	
	return FALSE;
}


Bool Figura::DeleteStrumento(DrawObject *sym)
{
	// i casi particolari vengono gestiti dalle singole figure: se passa di qua
	// significa che quella figura non ha casi particolari
	return FALSE;
}

void Figura::Free()
{
	
	if(GetPPuntoValore()!=NULL)
		delete GetPPuntoValore();
	if(GetPCorona()!=NULL)
		delete PCorona;
	if(GetPOcchiali()!=NULL)
		delete POcchiali;
	if(GetPDitaCorde()!=NULL)
		delete PDitaCorde;
	if(GetPTDinamico()!=NULL)
		delete GetPTDinamico();
	if(GetPTGenerico()!=NULL)
		delete GetPTGenerico();
	if(GetPAnnotazione()!=NULL)
		delete GetPAnnotazione();
	if(GetPStrumento()!=NULL)
		delete PStrumento;
	Init();
}


Bool Figura::Hit(const Point& p, SelObj selObj, DrawObject*& drwObjSel,
				 SymPath& symPath)
{
	Bool hit;   
	Point p1;
	
	
	p1 = p-GetAbsPos();               
	hit = -p1.x <= GetVU2LftF() && p1.x <= GetVU2RgtF() &&
        -p1.y <= GetVU2DwnF() && p1.y <= GetVU2UpF();
	
	drwObjSel=NULL;
	
	if(selObj==FIGURA_SEL || selObj==ANY_SEL)
		if(hit)
		{
			drwObjSel=this;
			symPath.Array[symPath.Level]=GetNumericCode();
			return(TRUE);
		}
		else if(selObj!=ANY_SEL)
			return(FALSE);
		
		
		if(!hit && drwObjSel==NULL && selObj==ANY_SEL)
		{
			if(GetPCorona()!=NULL &&
				GetPCorona()->Hit(p,selObj,drwObjSel,symPath))
			{ 
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
			}
			if(GetPPuntoValore()!=NULL &&
				GetPPuntoValore()->Hit(p,selObj,drwObjSel,symPath))
			{ 
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
			}
			if(GetPOcchiali()!=NULL &&
				GetPOcchiali()->Hit(p,selObj,drwObjSel,symPath))
			{ 
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
			}
			if(GetPDitaCorde()!=NULL &&
				GetPDitaCorde()->Hit(p,selObj,drwObjSel,symPath))
			{ 
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
			}
			if(GetPTDinamico()!=NULL &&
				GetPTDinamico()->Hit(p,selObj,drwObjSel,symPath))
			{ 
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
			}
			if(GetPTGenerico()!=NULL &&
				GetPTGenerico()->Hit(p,selObj,drwObjSel,symPath))
			{ 
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
			}
			if(GetPAnnotazione()!=NULL &&
				GetPAnnotazione()->Hit(p,selObj,drwObjSel,symPath))
			{ 
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
			}
			if(GetPStrumento()!=NULL &&
				GetPStrumento()->Hit(p,selObj,drwObjSel,symPath))
			{ 
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
			}
		}
		return(FALSE);
}


Bool Figura::Hit(const class Rectangle& r,SelObj selObj,SymPath& sp1,SymPath& sp2)
{
	class Rectangle RPallino = class Rectangle(GetAbsPos(), VU2LFTNOTA, VU2RGTNOTA, VU2UPNOTA,
		VU2DWNNOTA); 
    // Rettangolo attorno al pallino. Si considera questo per note, pause e
    // spazi al fine di facilitare la selezione tramite rettangolo.
	
	if(RPallino.isIn(r) && selObj==ANY_SEL)
	{
		if(sp1.Array[sp1.Level]==0)
			sp1.Array[sp1.Level] = GetNumericCode();
		sp2.Array[sp2.Level] = GetNumericCode();
		return TRUE;
	}
	return FALSE;
}


Bool Figura::SymbolicHit(SymPath& symPath, SelObj selObj, DrawObject*& drwObjSel)
{
	drwObjSel=NULL;
	
	
	
	if(!(symPath.Array[symPath.Level]==GetNumericCode()))
		return FALSE;
	
	if(symPath.Array[SIMBOLO_PATH_LEVEL]==0)
	{
		// Il path si ferma a questa figura.
		if(selObj==FIGURA_SEL || selObj==ANY_SEL)
		{
			drwObjSel=this;
			return(TRUE);
		}
	}    
	else if(drwObjSel==NULL && selObj==ANY_SEL)
	{
		if(GetPStrumento()!=NULL &&
			GetPStrumento()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);
		if(GetPCorona()!=NULL &&
			GetPCorona()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE); 
		if(GetPOcchiali()!=NULL &&
			GetPOcchiali()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);   
		if(GetPDitaCorde()!=NULL &&
			GetPDitaCorde()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);   
		if(GetPPuntoValore()!=NULL &&
			GetPPuntoValore()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);    
		if(GetPTDinamico()!=NULL &&
			GetPTDinamico()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);
		if(GetPTGenerico()!=NULL &&
			GetPTGenerico()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);
		if(GetPAnnotazione()!=NULL &&
			GetPAnnotazione()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);
	}
	return(FALSE);
}


CmdResult Figura::SymCommand(SymCmd& SCmd, Battuta *pBattuta)
{ 
	TipoInserimento tpIns=(TipoInserimento)(Bool)SCmd.ArgV[0];
	Bool aboveNota=FALSE;
	Bool hiddenTxt=(Bool)SCmd.ArgV[1];
	
	switch(tpIns)
	{
    case SOPRA:
		aboveNota=TRUE;
		break;
    case SOTTO:
		aboveNota=FALSE;
		break;
	case AUTOMATICO:
		{
			switch(SCmd.CommandId)
			{
			case CMD_FIGURA_CORONA:
			case CMD_FIGURA_CORONAQUAD:
			case CMD_FIGURA_CORONATRIANG:
			case CMD_FIGURA_OCCHIALI:
			case CMD_FIGURA_DITACORDE:
				aboveNota=TRUE;
				break;  
			case CMD_FIGURA_PPPPPP:
			case CMD_FIGURA_PPPPP:
			case CMD_FIGURA_PPPP:
			case CMD_FIGURA_PPP:
			case CMD_FIGURA_PP:
			case CMD_FIGURA_P:
			case CMD_FIGURA_FFFFFF:
			case CMD_FIGURA_FFFFF:
			case CMD_FIGURA_FFFF:
			case CMD_FIGURA_FFF:
			case CMD_FIGURA_FF:
			case CMD_FIGURA_F:
			case CMD_FIGURA_MP:
			case CMD_FIGURA_MF:
			case CMD_FIGURA_SP:
			case CMD_FIGURA_SF:
			case CMD_FIGURA_FZ:
			case CMD_FIGURA_SFZ:
			case CMD_FIGURA_FP:
			case CMD_FIGURA_RF:
			case CMD_FIGURA_FFP:
			case CMD_FIGURA_FFPP:
			case CMD_FIGURA_FPP:
			case CMD_FIGURA_RFZ:
			case CMD_FIGURA_SFF:
			case CMD_FIGURA_SFFF:
			case CMD_FIGURA_SFFP:
			case CMD_FIGURA_SFFZ:
			case CMD_FIGURA_SFP:
			case CMD_FIGURA_SFPP:
			case CMD_FIGURA_TGENERICO:
			case CMD_FIGURA_ANNOTAZIONE:
			case CMD_FIGURA_PEDALDOWN:
			case CMD_FIGURA_PEDALUP:
			case CMD_FIGURA_SEELINK:
			case CMD_FIGURA_EDITLINK:
			case NO_COMMAND:
				
				
				
				aboveNota=FALSE;
				break;
			}
			break;
		}
	}
	
	switch(SCmd.CommandId)
	{
    case CMD_FIGURA_1PUNTO:
		return(CmdFiguraPunto(1, pBattuta)); 
		break;
    case CMD_FIGURA_2PUNTI:
		return(CmdFiguraPunto(2, pBattuta)); 
		break;
    case CMD_FIGURA_3PUNTI:  
		return(CmdFiguraPunto(3, pBattuta)); 
		break;
    case CMD_FIGURA_CORONA:
		return(CmdFiguraCorona(aboveNota,Classic));
		break;  
    case CMD_FIGURA_CORONAQUAD:
		return(CmdFiguraCorona(aboveNota,Quad));
		break;  
    case CMD_FIGURA_CORONATRIANG:
		return(CmdFiguraCorona(aboveNota,Triang));
		break;  
    case CMD_FIGURA_OCCHIALI:
		return(CmdFiguraOcchiali(aboveNota));
	case CMD_FIGURA_PEDALDOWN:
		return(CmdFiguraPedalDown(aboveNota));
    case CMD_FIGURA_PEDALUP:
		return(CmdFiguraPedalUp(aboveNota));
    case CMD_FIGURA_DITACORDE:
		return(CmdFiguraDitaCorde(aboveNota,SCmd.Text));
		break;
   	case CMD_FIGURA_PPPPPP:
   	case CMD_FIGURA_PPPPP:
    case CMD_FIGURA_PPPP:
    case CMD_FIGURA_PPP:
    case CMD_FIGURA_PP:
    case CMD_FIGURA_P:
   	case CMD_FIGURA_FFFFFF:
    case CMD_FIGURA_FFFFF:
    case CMD_FIGURA_FFFF:
    case CMD_FIGURA_FFF:
    case CMD_FIGURA_FF:
    case CMD_FIGURA_F:
    case CMD_FIGURA_MP:
    case CMD_FIGURA_MF:
    case CMD_FIGURA_SP:
    case CMD_FIGURA_SF:
    case CMD_FIGURA_FZ:
    case CMD_FIGURA_SFZ:
    case CMD_FIGURA_FP:
	case CMD_FIGURA_RF:
	case CMD_FIGURA_FFP:
	case CMD_FIGURA_FFPP:
	case CMD_FIGURA_FPP:
	case CMD_FIGURA_RFZ:
	case CMD_FIGURA_SFF:
	case CMD_FIGURA_SFFF:
	case CMD_FIGURA_SFFP:
	case CMD_FIGURA_SFFZ:
	case CMD_FIGURA_SFP:
	case CMD_FIGURA_SFPP:
		return(CmdFiguraTDinamico(aboveNota,(TDinamicType)SCmd.ArgV[1]));
		break;   
    case CMD_FIGURA_TGENERICO:
		return(CmdFiguraTGenerico(aboveNota,(char *)SCmd.Text,hiddenTxt));
		break;   
    case CMD_FIGURA_ANNOTAZIONE:
		return(CmdFiguraAnnotazione(aboveNota,(char *)SCmd.Text));
		break;   
	case CMD_FIGURA_SEELINK:
		return(CmdFiguraSeeLink(pBattuta)); //Passare ptr a pBattuta
		break;   
	case CMD_FIGURA_EDITLINK:
		return(CmdFiguraEditLink((char *)SCmd.Text));
		break;   
	case NO_COMMAND:
		break;
	}     
	return(CMD_UNKNOWN);
}

// METODI PRIVATI 


void Figura::AdjustPuntoValore()
{
	VUnit dy=0;
	
	if(PPuntoValore!=NULL)
	{ 
		// Calcola lo spostamento verticale del punto per le varie pause.
		if( GetID()==CL_PSEMIMINIMA ||GetID()==CL_PSEMICROMA)
		{
			dy=2*VU_PER_ROW;
		}
		
		if(GetID()==CL_PCROMA)
		{
			dy=VU_PER_ROW;
		}
		
		if(GetID()==CL_PBISCROMA || GetID()==CL_PSEMIBISCROMA)
		{
			dy=3*VU_PER_ROW;
		} 
		// Sistema il punto immediatamente a destra della figura. Se il punto
		// cade sopra un rigo, viene spostato nello spazio immediatamente
		// superiore. 
		if(GetAltezza()%2==0)
		{
			dy+=VU_PER_ROW/2;
		}
		if(GetID()==CL_PSEMIBREVE) dy=-VU_PER_ROW/2;
		PPuntoValore->SetPos(this,Point(GetVU2RgtF()+SPNOTAPUNTOVALORE+
			PPuntoValore->GetVU2Lft(),dy));
		SetVU2Rgt(GetVU2RgtF()+SPNOTAPUNTOVALORE+PPuntoValore->GetWidth());
		// Aggiorna VU2Up e VU2Dwn.
		if(GetVU2Up()<(dy+PPuntoValore->GetVU2Up()))
		{
			SetVU2Up(dy+PPuntoValore->GetVU2Up());
		}  
		if(GetVU2Dwn()<(-dy+PPuntoValore->GetVU2Dwn()))
		{
			SetVU2Dwn(-dy+PPuntoValore->GetVU2Dwn());
		}
	}
}


void Figura::AdjustCorona(VUnit YLinea1,VUnit YLinea5)
{
	VUnit dy;
	
	if(GetPCorona()!=NULL)
	{
		if(GetPCorona()->GetAboveNota())
		{
			// La corona e' sopra la nota.
			PCorona->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+
				PCorona->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare la corona dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + PCorona->GetVU2Dwn() -
				PCorona->GetAbsPos().y;
			if(dy>0) PCorona->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PCorona->GetAbsPos().y-GetAbsPos().y+
				PCorona->GetVU2Up());
		}
		else
		{
			// La corona e' sotto la nota.
			PCorona->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-
				PCorona->GetVU2Up()));
			// Accorgimento per evitare di sistemare la corona dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - PCorona->GetVU2Up() -
				PCorona->GetAbsPos().y;
			if(dy<0) PCorona->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PCorona->GetAbsPos().y+GetAbsPos().y+
				PCorona->GetVU2Dwn());
		}
		if(PCorona->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PCorona->GetVU2Lft());
		if(PCorona->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PCorona->GetVU2Rgt());
	}
}


void Figura::AdjustTDinamico(VUnit YLinea1,VUnit YLinea5)
{
	VUnit dy; 
	
	if(GetPTDinamico()!=NULL)
	{
		if(GetPTDinamico()->GetAboveNota())
		{
			// Il segno dinamico e' sopra la nota.
			PTDinamico->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+
				PTDinamico->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare il segno dinamico dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + PTDinamico->GetVU2Dwn() -
				PTDinamico->GetAbsPos().y;
			if(dy>0) PTDinamico->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PTDinamico->GetAbsPos().y-GetAbsPos().y+
				PTDinamico->GetVU2Up());
		}
		else
		{
			// Il segno dinamico e' sotto la nota.
			PTDinamico->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-
				PTDinamico->GetVU2Up()));
			// Accorgimento per evitare di sistemare il segno dinamico dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - PTDinamico->GetVU2Up() -
				PTDinamico->GetAbsPos().y;
			if(dy<0) PTDinamico->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PTDinamico->GetAbsPos().y+GetAbsPos().y+
				PTDinamico->GetVU2Dwn());
		}
		if(PTDinamico->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PTDinamico->GetVU2Lft());
		if(PTDinamico->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PTDinamico->GetVU2Rgt());
	}        
}


void Figura::AdjustTGenerico(VUnit YLinea1,VUnit YLinea5)
{
	VUnit dy;
	extern bool visible;
	
	if(GetPTGenerico()!=NULL && (!GetPTGenerico()->GetHidden() || visible) )
	{
		if(GetPTGenerico()->GetAboveNota())
		{
			// Il segno Generico e' sopra la nota.
			PTGenerico->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+
				PTGenerico->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare il segno Generico dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + PTGenerico->GetVU2Dwn() -
				PTGenerico->GetAbsPos().y;
			if(dy>0) PTGenerico->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PTGenerico->GetAbsPos().y-GetAbsPos().y+
				PTGenerico->GetVU2Up());
		}
		else
		{
			// Il segno Generico e' sotto la nota.
			PTGenerico->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-
				PTGenerico->GetVU2Up()));
			// Accorgimento per evitare di sistemare il segno Generico dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - PTGenerico->GetVU2Up() -
				PTGenerico->GetAbsPos().y;
			if(dy<0) PTGenerico->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PTGenerico->GetAbsPos().y+GetAbsPos().y+
				PTGenerico->GetVU2Dwn());
		}
		if(PTGenerico->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PTGenerico->GetVU2Lft());
		if(PTGenerico->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PTGenerico->GetVU2Rgt());
	}                                    
}

void Figura::AdjustAnnotazione(VUnit YLinea1,VUnit YLinea5)
{
	VUnit dy;
	
	if(GetPAnnotazione()!=NULL)
	{
		if(GetPAnnotazione()->GetAboveNota())
		{
			// Il segno Generico e' sopra la nota.
			PAnnotazione->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+
				PAnnotazione->GetVU2Dwn()));
			
			// Accorgimento per evitare di sistemare il segno Generico dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + PAnnotazione->GetVU2Dwn() -
				PAnnotazione->GetAbsPos().y;
			
			if(dy>0) PAnnotazione->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PAnnotazione->GetAbsPos().y-GetAbsPos().y+
				PAnnotazione->GetVU2Up());
			
		}
		else
		{
			// Il segno Generico e' sotto la nota.
			PAnnotazione->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-
				PAnnotazione->GetVU2Up()));
			
			// Accorgimento per evitare di sistemare il segno Generico dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - PAnnotazione->GetVU2Up() -
				PAnnotazione->GetAbsPos().y;
			if(dy<0) PAnnotazione->Move(Point(0,dy));
			
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PAnnotazione->GetAbsPos().y+GetAbsPos().y+
				PAnnotazione->GetVU2Dwn());
		}
		if(PAnnotazione->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PAnnotazione->GetVU2Lft());
		if(PAnnotazione->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PAnnotazione->GetVU2Rgt());
	}                                    
}

void Figura::AdjustOcchiali(VUnit YLinea1,VUnit YLinea5)
{
	VUnit dy;
	
	if(GetPOcchiali()!=NULL)
	{
		if(GetPOcchiali()->GetAboveNota())
		{
			// Gli occhiali sono sopra la nota.
			POcchiali->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+
				POcchiali->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare gli occhiali dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + POcchiali->GetVU2Dwn() -
				POcchiali->GetAbsPos().y;
			if(dy>0) POcchiali->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(POcchiali->GetAbsPos().y-GetAbsPos().y+
				POcchiali->GetVU2Up());
		}
		else
		{
			// Gli occhiali sono sotto la nota.
			POcchiali->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-
				POcchiali->GetVU2Up()));
			// Accorgimento per evitare di sistemare gli occhiali dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - POcchiali->GetVU2Up() -
				POcchiali->GetAbsPos().y;
			if(dy<0) POcchiali->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-POcchiali->GetAbsPos().y+GetAbsPos().y+
				POcchiali->GetVU2Dwn());
		}
		if(POcchiali->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(POcchiali->GetVU2Lft());
		if(POcchiali->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(POcchiali->GetVU2Rgt());
	}
}   

void Figura::AdjustPedalDown(VUnit YLinea1,VUnit YLinea5)
{
	VUnit dy;
	
	if(GetPStrumento()!=NULL && GetPStrumento()->GetID()==CL_PEDALDOWN)
	{
		if(GetPStrumento()->GetAboveNota())
		{		
			// Pedaldown e' sopra la nota.
			PStrumento->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+PStrumento->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare Pedaldown dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + PStrumento->GetVU2Dwn() -
				PStrumento->GetAbsPos().y;
			if(dy>0) PStrumento->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PStrumento->GetAbsPos().y-GetAbsPos().y+
				PStrumento->GetVU2Up());
		}
		else
		{
			// Pedaldown e' sotto la nota.
			PStrumento->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY- PStrumento->GetVU2Up()));
			// Accorgimento per evitare di sistemare Pedaldown dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - PStrumento->GetVU2Up() - PStrumento->GetAbsPos().y;
			if(dy<0) PStrumento->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PStrumento->GetAbsPos().y+GetAbsPos().y+PStrumento->GetVU2Dwn());
		}
		
		if(PStrumento->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PStrumento->GetVU2Lft());
		if(PStrumento->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PStrumento->GetVU2Rgt());
	}
}   

void Figura::AdjustPedalUp(VUnit YLinea1,VUnit YLinea5)
{
	VUnit dy;
	
	if(GetPStrumento()!=NULL && GetPStrumento()->GetID()==CL_PEDALUP)
	{
		if(GetPStrumento()->GetAboveNota())
		{
			// Gli occhiali sono sopra la nota.
			PStrumento->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+
				PStrumento->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare gli occhiali dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + PStrumento->GetVU2Dwn() -
				PStrumento->GetAbsPos().y;
			if(dy>0) PStrumento->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PStrumento->GetAbsPos().y-GetAbsPos().y+
				PStrumento->GetVU2Up());
		}
		else
		{
			// Gli occhiali sono sotto la nota.
			PStrumento->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-
				PStrumento->GetVU2Up()));
			// Accorgimento per evitare di sistemare gli occhiali dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - PStrumento->GetVU2Up() -
				PStrumento->GetAbsPos().y;
			if(dy<0) PStrumento->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PStrumento->GetAbsPos().y+GetAbsPos().y+
				PStrumento->GetVU2Dwn());
		}
		if(PStrumento->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PStrumento->GetVU2Lft());
		if(PStrumento->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PStrumento->GetVU2Rgt());
	}
}   

void Figura::AdjustDitaCorde(VUnit YLinea1,VUnit YLinea5)
{
	VUnit dy;
	
	if(GetPDitaCorde()!=NULL)
	{
		if(GetPDitaCorde()->GetAboveNota())
		{
			// Le dita sulle corde sono sopra la nota.
			PDitaCorde->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+
				PDitaCorde->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare le dita sulle corde dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + PDitaCorde->GetVU2Dwn() -
				PDitaCorde->GetAbsPos().y;
			if(dy>0) PDitaCorde->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PDitaCorde->GetAbsPos().y-GetAbsPos().y+
				PDitaCorde->GetVU2Up());
		}
		else
		{
			// i frets sono sotto la nota.
			PDitaCorde->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-
				PDitaCorde->GetVU2Up()));
			// Accorgimento per evitare di sistemare i frets dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - PDitaCorde->GetVU2Up() -
				PDitaCorde->GetAbsPos().y;
			if(dy<0) PDitaCorde->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PDitaCorde->GetAbsPos().y+GetAbsPos().y+
				PDitaCorde->GetVU2Dwn());    
		}  
		if(PDitaCorde->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PDitaCorde->GetVU2Lft());
		if(PDitaCorde->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PDitaCorde->GetVU2Rgt());
	}
}   


CmdResult Figura::CmdFiguraPunto(int NPunti, Battuta *pBattuta)
{
	if(GetPPuntoValore()!=NULL)
		delete GetPPuntoValore();
	PuntoValore *ppv=new PuntoValore;
	if(ppv==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Figura","CmdFiguraPunto()");    
		return(CMD_FAIL);
	}; 
	ppv->SetNumPunti(NPunti);
	SetPPuntoValore(ppv);
	if(pBattuta!=NULL)
	{
		if(Battuta::GetAutoBeaming())
			pBattuta->BeamingAuto();
	}
	return(CMD_OK); 
}


CmdResult Figura::CmdFiguraCorona(Bool AboveF, CoronaType tipocorona)
{                            
	if(GetPCorona()!=NULL)
		delete GetPCorona();
	SetPCorona(new Corona);
	if(GetPCorona()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Figura","CmdFiguraCorona()");  
		return(CMD_FAIL);
	};
	GetPCorona()->SetAboveNota(AboveF);
	GetPCorona()->SetTipo(tipocorona);
	return(CMD_OK);
} 

CmdResult Figura::CmdFiguraOcchiali(Bool AboveF)
{                            
	if(GetPOcchiali()!=NULL)
		delete GetPOcchiali();
	SetPOcchiali(new Occhiali);
	if(GetPOcchiali()==NULL)
	{ 
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Figura","CmdFiguraOcchiali()");  
		return(CMD_FAIL);
	};
	GetPOcchiali()->SetAboveNota(AboveF);
	return(CMD_OK);
}               

CmdResult Figura::CmdFiguraDitaCorde(Bool AboveF,char *fullDescr)
{                          
	DitaCorde *app=NULL;
	
	app=new DitaCorde(fullDescr);
	app->SetAboveNota(AboveF);
	if(GetPDitaCorde()!=NULL)
		delete GetPDitaCorde();
	SetPDitaCorde(app);
	if(GetPDitaCorde()==NULL)
	{ 
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Figura","CmdFiguraDitaCorde()");  
		return(CMD_FAIL);
	}
	return(CMD_OK);
}               

CmdResult Figura::CmdFiguraTDinamico(Bool AboveF, TDinamicType td)
{                            
	if(GetPTDinamico()==NULL)
	{  SetPTDinamico(new TDinamico);
    if(GetPTDinamico()==NULL)
    {      
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Figura","CmdFiguraTDinamico()");
		return(CMD_FAIL);
    };
	}
	else
	{
		if ((GetPTDinamico()->GetTipo() == td) && (GetPTDinamico()->GetAboveNota() == AboveF))
			return(CMD_FAIL);
	}
	
	if(GetPTDinamico()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Figura","CmdFiguraTDinamico()");    
		return(CMD_FAIL);
	};
	GetPTDinamico()->SetAboveNota(AboveF);
	if (!(GetPTDinamico()->SetTipo(td)))
	{
		EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Figura","CmdFiguraTDinamico()");   
		return(CMD_FAIL);
	};
	
	return(CMD_OK);
}


CmdResult Figura::CmdFiguraTGenerico(Bool AboveF, char * txt, Bool hidden)
{
	if(GetPTGenerico()!=NULL)
	{
		if ((GetPTGenerico()->GetAboveNota() == AboveF) && 
			(strcmp((char *)(GetPTGenerico()->GetTxt()),(char *)txt)))
			return(CMD_FAIL);
		else
			delete GetPTGenerico();
	}
	SetPTGenerico(new TGenerico);
	if(GetPTGenerico()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Figura","CmdFiguraTGenerico()");   
		return(CMD_FAIL);
	};
	
	GetPTGenerico()->SetAboveNota(AboveF);
	GetPTGenerico()->SetHidden(hidden);
	
	if (!(GetPTGenerico()->SetTxt(txt)))
	{
		EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Figura","CmdFiguraTGenerico()");   
		return CMD_FAIL;
	}
	return(CMD_OK);
}

CmdResult Figura::CmdFiguraAnnotazione(Bool AboveF, char * txt)
{
	if(GetPAnnotazione()!=NULL)
	{
		if ((GetPAnnotazione()->GetAboveNota() == AboveF) && 
			(strcmp((char *)(GetPAnnotazione()->GetTxt()),(char *)txt)))
			return(CMD_FAIL);
		else
			delete GetPAnnotazione();
	}
	SetPAnnotazione(new Annotazione);
	if(GetPAnnotazione()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Figura","CmdFiguraAnnotazione()");
		return(CMD_FAIL);
	};
	GetPAnnotazione()->SetAboveNota(AboveF);
	if (!(GetPAnnotazione()->SetTxt(txt)))
	{
		EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Figura","CmdFiguraAnnotazione()");   
		return CMD_FAIL;
	}
	return(CMD_OK);
}

CmdResult Figura::CmdFiguraSeeLink(Battuta *pBattuta)
{
	wxString hyperlink;
	long current_score = pMusicWin->PPartit->GetCurrentScore();
	short current_measure = pBattuta->GetNumProgress();
	hyperlink=Get_Link();

	if(!hyperlink.IsEmpty())
		SMRApp::GetApp().settings->Execute(hyperlink,APP_BROWSER);
	SetColor(VIOLA);

	return(CMD_OK);
}


CmdResult Figura::CmdFiguraEditLink(char * txt)
{
	wxString check1="http://";
	wxString check2="www.";
	wxString hyperlink = txt;
	wxString link;
	LinkOption dialog;
	dialog.Centre();		  
	
	if(Get_Link().IsEmpty())
	{
		if(dialog.ShowModal()==wxID_OK)
		{   
			if(dialog.GetChoice())	
			{			
				link.Clear();
				link=dialog.GetLink();	
				if(!link.IsEmpty())	
					Set_Link(link);
			}			
		}
	}
	else
	{
		dialog.SetLink(Get_Link());
		dialog.OnSwitch1();
		
		if(dialog.ShowModal()==wxID_OK)
		{   
			if(dialog.GetChoice())	
			{			
				link.Clear();
				link=dialog.GetLink();	
				if(!link.IsEmpty())	
					Set_Link(link);
			}
		}
	}
	
	pMusicWin->PPartit->View_Link(true);
	Scr->GSetMainDrawArea();
	return(CMD_OK);
}


CmdResult Figura::CmdFiguraPedalDown(Bool AboveF)
{                            
	if(GetPStrumento())
		
		delete GetPStrumento();
	SetPStrumento(new PedalDown);
	
	if(GetPStrumento()==NULL)
	{ 
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Figura","CmdFiguraPedalDown()");  
		return(CMD_FAIL);
	};
	GetPStrumento()->SetAboveNota(AboveF);
	return(CMD_OK);
}               

CmdResult Figura::CmdFiguraPedalUp(Bool AboveF)
{                            
	if(GetPStrumento()!=NULL)
		delete GetPStrumento();
	SetPStrumento(new PedalUp);
	if(GetPStrumento()==NULL)
	{ 
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Figura","CmdFiguraPedalUp()");  
		return(CMD_FAIL);
	};
	GetPStrumento()->SetAboveNota(AboveF);
	return(CMD_OK);
}



Bool Figura::FigPtr2SymPath(SymPath& symPath, Figura *figptr)
{
	return false;
}

DrawObject * Figura::GetAggiunta(SymCmd * MyCmd)
{
	switch(MyCmd->CommandId) {
	case CMD_FIGURA_CORONA:
	case CMD_FIGURA_CORONAQUAD:
	case CMD_FIGURA_CORONATRIANG:return ((DrawObject *)(PCorona)); break;
	case CMD_FIGURA_OCCHIALI: return ((DrawObject *)(POcchiali)); break;
	case CMD_FIGURA_PEDALDOWN: return ((DrawObject *)(PStrumento)); break;
	case CMD_FIGURA_PEDALUP: return ((DrawObject *)(PStrumento)); break;
	case CMD_FIGURA_DITACORDE: return ((DrawObject *)(PDitaCorde)); break;
	case CMD_FIGURA_1PUNTO:
	case CMD_FIGURA_2PUNTI: 
	case CMD_FIGURA_3PUNTI: return ((DrawObject *)(PPuntoValore)); break;
   	case CMD_FIGURA_PPPPPP:
   	case CMD_FIGURA_PPPPP:
	case CMD_FIGURA_PPPP:
	case CMD_FIGURA_PPP:
	case CMD_FIGURA_PP:
	case CMD_FIGURA_P:
   	case CMD_FIGURA_FFFFFF:
   	case CMD_FIGURA_FFFFF:
	case CMD_FIGURA_FFFF:
	case CMD_FIGURA_FFF:
	case CMD_FIGURA_FF:
	case CMD_FIGURA_F:
	case CMD_FIGURA_MP:
	case CMD_FIGURA_MF:
	case CMD_FIGURA_SP:
	case CMD_FIGURA_SF:
	case CMD_FIGURA_FZ:
	case CMD_FIGURA_SFZ:
	case CMD_FIGURA_RF:
	case CMD_FIGURA_FFP:
	case CMD_FIGURA_FFPP:
	case CMD_FIGURA_FPP:
	case CMD_FIGURA_RFZ:
	case CMD_FIGURA_SFF:
	case CMD_FIGURA_SFFF:
	case CMD_FIGURA_SFFP:
	case CMD_FIGURA_SFFZ:
	case CMD_FIGURA_SFP:
	case CMD_FIGURA_SFPP:
	case CMD_FIGURA_FP: return ((DrawObject *)(PTDinamico)); break;
	case CMD_FIGURA_TGENERICO: return((DrawObject *)(PTGenerico)); break;
	case CMD_FIGURA_ANNOTAZIONE: return((DrawObject *)(PAnnotazione)); break;
	case CMD_SPAZIO_PRFIATO: 
		{
			if (GetID()==CL_SPAZIO) {
				return (DrawObject *)(((Spazio *)(this))->GetPPrFiato());
			} else return NULL; 
			break;
		}
	case CMD_PAUSA_TIMPANO:
		switch (GetID()) {
		case CL_PAUSA:
		case CL_PQUATTROBATTUTE:
		case CL_PDUEBATTUTE:
		case CL_PSEMIBREVE:
		case CL_PMINIMA:
		case CL_PSEMIMINIMA:
		case CL_PCROMA:
		case CL_PSEMICROMA:
		case CL_PBISCROMA:
		case CL_PSEMIBISCROMA:
		case CL_PFUSA:
		case CL_PGENERICA:
			if (((Pausa *)(this))->GetPStrumento()==NULL) return NULL;
			else return ((Pausa *)(this))->GetPStrumento()->GetObject(MyCmd); break;
		default: return NULL; break;
		} 
		return NULL;
		break;
		
		default:	return NULL; break;
	}
	// tanto di qui non ci passera' mai !
	return NULL;
}

void Figura::SetupSimboli()
{
}

void Figura::AddIntRef(IntRefType t,IntEsteso* ref)
{
	IntRef* tmp=new IntRef[nIntRefs+1];
	
	for(int i=0; i<nIntRefs; i++)
		tmp[i]=intRefs[i];
	tmp[nIntRefs].type=t;
	tmp[nIntRefs].intRef=ref;
	
	if(intRefs!=NULL)
		delete[] intRefs;
    
	intRefs=tmp;
	nIntRefs++;
}

void Figura::DelIntRefs()
{
	if(intRefs!=NULL)
	{
		delete[] intRefs;
		intRefs=NULL;
		nIntRefs=0;
	}
}

IntRef *Figura::GetIntRef(int n)
{
	if (n>=0 && n<nIntRefs)
		return &intRefs[n];
	return NULL;
}


void Figura::SetNStaff(short numstaff)
{
	NStaff = numstaff;
}

short Figura::GetNStaff()
{
	return NStaff;
}

void Figura::GetMinMaxStaff(short& minStaff, short& maxStaff)
{
	minStaff=NStaff;
	maxStaff=NStaff;
}

void Figura::SetPBattuta(Battuta *pBat) 
{
	pBattuta = pBat;
}

Battuta * Figura::GetPBattuta() 
{
	return pBattuta;
}

Raggruppamento* Figura::GetRaggruppamentoIniziale(){	//FFF

	Raggruppamento* raggr=NULL;
	for(int i=0; i<=nIntRefs; i++){
		if(GetIntRef(i) && (GetIntRef(i)->intRef)){
			if (((GetIntRef(i)->intRef)->GetID()==CL_RAGGRUPPAMENTO)&&(GetIntRef(i)->type==INT_START))
				raggr=(Raggruppamento*)(GetIntRef(i)->intRef);
			
		}
	}
	return raggr;
}


Raggruppamento* Figura::GetRaggruppamento(){			//FFF

	Raggruppamento* raggr=NULL;
	for(int i=0; i<=nIntRefs; i++){
		if(GetIntRef(i) && (GetIntRef(i)->intRef)){
			if ((GetIntRef(i)->intRef)->GetID()==CL_RAGGRUPPAMENTO)
				raggr=(Raggruppamento*)(GetIntRef(i)->intRef);
			
		}
	}
	return raggr;
}