/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: figura.hpp,v 1.1 2005/07/25 16:30:39 bellinpo Exp $ */

#ifndef FIGURA_HPP
#define FIGURA_HPP

#include "lioo.hpp"

#include "wx/wx.h"
#include "wx/string.h"

#include "drw/drawobje.hpp"
#include "var/corona.hpp"
#include "var/occhiali.hpp"                                   
#include "var/puntoval.hpp"
#include "txt/tdinamic.hpp"
#include "txt/tgeneric.hpp"
#include "txt/annotazi.hpp"
#include "str/strument.hpp"  
#include "str/ditacord.hpp" 
#include "str/pedaldown.hpp"
#include "str/pedalup.hpp"
class Raggruppamento;

///////////////////////////////////////////////////////////////////////

typedef int UL;
#define UL2VU(x) ((x+50)/100)
#define VU2UL(x) (x*100)
#define SPG_DEF_FACTOR 24000

//////////////////////////////////////////////////
// in seguito quando si cambiera' il draw si ridefiniranno i VUnit come
// UL. Allora saranno tutti VUnit, per ora faccio cosi', a tutti gli
// effetti le UL sono memorizzate in variabili di tipo int
////////////////////////////////////////////////////////////////////////

class IntEsteso;
enum IntRefType {INT_START,INT_OVER,INT_END};


struct IntRef {
	IntRefType type;
	IntEsteso *intRef;
};

class Chiave;
class LegatQuadra;

class MoodsMidiWrite;

class Figura: public DrawObject
{ 
private:
	
	
    UL SpG[2],SpOr[2],SpDirUt[2],SpOrcUt[2];
	
    
    //UL SpG,SpOr,SpDirUt,SpOrcUt;
    
    // Attributi di spaziatura per la giustificazione:
    // 1) SpG -   Spazi Giustificazione attiva
    //           Sono gli spazi calcolati per la giustificazione, sia essa
    //           lineare o logaritmica, tiene conto anche degli allineamenti 
    //           questo e'per il direttoriale
    // 2) SpOr -  Spazi Orchestrale
    //           sono gli spazi della giustificazione
    //           dellrchestrale, tiene conto degli allineamenti dei due layer
    //           
    // 3) SpDirUt -  Spazi Direttoriale Utente
    //           Delta di spazi rispetto a SG nella partitura (l'archivista
    //           e il direttore vedono la stessa partitura) introdotti
    //           dall'utente stesso
    // 4) SpOrcUt -  Spazi Orchestrale Utente
    //           Delta di spazi da aggiungere a SO nella parte del DLIOO
    //           (orchestrale) perche' introdotti dall'orchestrale stesso
    //           per il tuning personale della giustificazione.
    //           Non devono essere visualizzati dal MASAE
    //           e dall'archivista
    // Riassumendo il direttore vede SG+SDU
    // e l'orchestrale vede SO+SOU
    // I delta sono il numero di unita' logiche di incremento o decremento
    // ovvero hanno segno.
    // Gli spazi e i delta sono espressi in Unita Logiche (UL) cosi' definite:
    //
    // 1 UL = 1/600 staff space (verificare che nel MOODS 1 NHW=1,2 SS)
    //                                                      noteheadwidth
    // nella rappresentazione a monitor standard si ha che uno staff space
    // =6 pixel, ovvero 1 pixel=100 UL
    //
    //
    // rappresentano gli spazi fino alla prossima figura della battuta      
    ////////////
	
    UL SpDLine[2], SpOLine[2];
	// spazi necessari per il line breaking, non vengono serializzati (usati solo in runtime)
	
    
    void AdjustPuntoValore();
    // Sistema il punto di valore a destra della figura. Se il punto cade
    // sopra un rigo del pentagramma o su un taglio addizionale, viene
    // spostato nello spazio immediatamente superiore.
    // Adatta il rettangolo che circonda la figura completa di simboli in modo
    // da fargli contenere anche questo simbolo.                               
	
    void AdjustCorona(VUnit YLinea1,VUnit YLinea5);
    // Sistema la corona sopra (se l'attributo AboveNota della corona
    // e' TRUE) o sotto la figura. I parametri di ingresso
    // YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu'
    // bassa e di quella piu' alta del pentagramma.
    // Adatta il rettangolo che circonda la figura completa di simboli in modo
    // da fargli contenere anche questo simbolo.
    void AdjustOcchiali(VUnit YLinea1,VUnit YLinea5);
    // Sistema gli occhiali sopra (se l'attributo AboveNota degli occhiali
    // e' TRUE) o sotto la figura. I parametri di ingresso
    // YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu'
    // bassa e di quella piu' alta del pentagramma.
    // Adatta il rettangolo che circonda la figura completa di simboli in modo
    // da fargli contenere anche questo simbolo.
    void AdjustDitaCorde(VUnit YLinea1,VUnit YLinea5);
    // Sistema le dita sulle corde sopra 
    // I parametri di ingresso YLinea1 ed YLinea5 sono le altezze fisiche 
    // (in VUnit) della linea pi bassa e di quella piu' alta del pentagramma.
    // Adatta il rettangolo che circonda la figura completa di simboli in modo
    // da fargli contenere anche questo simbolo.
    void AdjustTDinamico(VUnit YLinea1,VUnit YLinea5);
    // Sistema il segno dinamico sopra (se l'attributo AboveNota del segno    //dinamico
    // e' TRUE) o sotto la figura. I parametri di ingresso
    // YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu'
    // bassa e di quella piu' alta del pentagramma.
    // Adatta il rettangolo che circonda la figura completa di simboli in modo
    // da fargli contenere anche questo simbolo.
	
    void AdjustTGenerico(VUnit YLinea1,VUnit YLinea5);
	// Sistema il segno generico sopra (se l'attributo AboveNota del segno //generico
	// e' TRUE) o sotto la figura. I parametri di ingresso
	// YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu'
	// bassa e di quella piu' alta del pentagramma.
	// Adatta il rettangolo che circonda la figura completa di simboli in modo
	//da fargli contenere anche questo simbolo.
    
    void AdjustAnnotazione(VUnit YLinea1,VUnit YLinea5);
	// Sistema il segno Annotazione sopra (se l'attributo AboveNota del segno 
	// Annotazionee' TRUE) o sotto la figura. I parametri di ingresso
	// YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu'
	// bassa e di quella piu' alta del pentagramma.
	// Adatta il rettangolo che circonda la figura completa di simboli in modo
	//da fargli contenere anche questo simbolo.
	
    void AdjustPedalDown(VUnit YLinea1,VUnit YLinea5);
	// Sistema il segno PedalDown sopra (se l'attributo AboveNota del segno 
	// Annotazionee' TRUE) o sotto la figura. I parametri di ingresso
	// YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu'
	// bassa e di quella piu' alta del pentagramma.
	// Adatta il rettangolo che circonda la figura completa di simboli in modo
	//da fargli contenere anche questo simbolo.
	
    void AdjustPedalUp(VUnit YLinea1,VUnit YLinea5);
	// Sistema il segno PedalUp sopra (se l'attributo AboveNota del segno 
	// Annotazionee' TRUE) o sotto la figura. I parametri di ingresso
	// YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu'
	// bassa e di quella piu' alta del pentagramma.
	// Adatta il rettangolo che circonda la figura completa di simboli in modo
	//da fargli contenere anche questo simbolo.
	
	
    CmdResult CmdFiguraCorona(Bool AboveF, CoronaType tipocorona);
	// Imposta la corona alla figura. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL. Occorre specificare AboveF=TRUE per
	// la corona sopra la figura oppure FALSE per la corona sotto la
	// figura. Occorre inoltre specificare il tipo di corona di cui si tratta
	// {Classic, Quad, Triang}.
    CmdResult CmdFiguraOcchiali(Bool AboveF);
	// Imposta gli occhiali alla figura. Se riesce, restituisce CMD_OK,
	// altrimenti CMD_FAIL. Occorre specificare AboveF=TRUE per gli occhiali
	// sopra la figura, oppure FALSE per disegnarli sotto.
    CmdResult CmdFiguraDitaCorde(Bool AboveF,char *fullDescr);
	// Imposta le dita sulle corde alla figura. Se riesce, restituisce CMD_OK,
	// altrimenti CMD_FAIL. Occorre specificare AboveF=TRUE per i frets
	// sopra la figura, oppure FALSE per disegnarli sotto.
    CmdResult CmdFiguraTDinamico(Bool AboveF, TDinamicType td);
	// Assegna alla figura il segno Dinamico (Pianissimo, Piano, Forte, ...).
	// Se riesce, restituisce CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveF=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	CmdResult CmdFiguraPedalDown(Bool AboveF);
	// Imposta il pedaldown alla figura. Se riesce, restituisce CMD_OK,
	// altrimenti CMD_FAIL. Occorre specificare AboveF=TRUE per gli occhiali
	// sopra la figura, oppure FALSE per disegnarli sotto.
	CmdResult CmdFiguraPedalUp(Bool AboveF);
	// Imposta il pedalup alla figura. Se riesce, restituisce CMD_OK,
	// altrimenti CMD_FAIL. Occorre specificare AboveF=TRUE per gli occhiali
	// sopra la figura, oppure FALSE per disegnarli sotto.
    CmdResult CmdFiguraTGenerico(Bool AboveF, char * s, Bool hidden=FALSE);
	// Assegna alla figura il testo Generico s (Rallentato, Accellerato, ...).
	// Se riesce, restituisce CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveF=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
    CmdResult CmdFiguraAnnotazione(Bool AboveF, char * s);
	// Assegna alla figura l'Annotazione
	// Se riesce, restituisce CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveF=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	
	
	short NStaff; 
	Battuta *pBattuta; 
	
	
	CmdResult CmdFiguraSeeLink(Battuta * pBattuta);
	//Visualizza e lancia il link associato alla figura
	
	CmdResult CmdFiguraEditLink(char * s);
	//Inserisce e modifica il link associato alla figura
	
	/*BIAN-VERR*/
	bool Segno;
	/*BIAN-VERR*/

 protected:
	 
	 LegatQuadra *terz;
	 
	 IntRef *intRefs;
	 int nIntRefs;
	 
	 wxString HyperLink; // Link associato alla figura  
	 
	 int Altezza;
	 // Altezza musicale grafica della figura rispetto alla nota che occupa la
	 // riga pi bassa del pentagramma (Mi nella chiave di violino).

	 float durata;  
	 VUnit VU2UpF,VU2DwnF,VU2LftF,VU2RgtF;
	 // Distanze da AbsPos dei punti superiore, inferiore, sinistro e destro
	 // del rettangolo contenente la figura.  
	 VUnit VU2Up,VU2Dwn,VU2Lft,VU2Rgt;
	 // Distanze da AbsPos dei punti superiore, inferiore, sinistro e
	 // destro del rettangolo contenente la figura e tutti i simboli ad
	 // essa connessi.               
	 int NumSpaces;
	 // Numero di unita' di spazio che la figura occupa. E' diverso da 1
	 // solo per i gruppi di note.
	 Corona *PCorona;      
	 // Puntatore alla classe Corona. 
	 Occhiali *POcchiali; 
	 // Puntatore alla classe Occhiali.          
	 DitaCorde *PDitaCorde; 
	 // Puntatore alla classe DitaCorde.          
	 PuntoValore *PPuntoValore;
	 // Puntatore alla classe PuntoValore.
	 TDinamico *PTDinamico;
	 // Puntatore alla classe TDinamico.
	 TGenerico *PTGenerico;
	 // Puntatore alla classe TGenerico.
	 Annotazione *PAnnotazione;
	 // Puntatore alla classe Annotazione.
	 NumCode NumericCode;
	 // Codice numerico.
	 Strumento *PStrumento;
	 // Puntatore ai segni di strumento.
	 Figura *ParentFig;
	 // Puntatore alla classe Figura
	 
	 
	 int  AppSil[4];
	 // Ogni posizione di questo vettore si
	 // riferisce ad una distinta strofa,tra
	 // quattro possibili,in cui alla nota
	 // pu essere associata una sillaba.
	 // Se il contenuto di un elemento del
	 // vettore  1,indica che alla figura
	 //  associata una sillaba(o una sua
	 // estensione) appartenente alla strofa
	 // il cui numero corrisponde alla
	 // posizione stessa.
	 
	 
	 // dice se la figura puntata e' in formato ridotto (notina o pausina)
	 Bool Figurina;
	 
	 Status_ID status;
	 // Indica lo stato della figura: normal, ghosted, hidden, graced.
	 
 public:                                     
	 Figura();
	 Figura(double);
	 ~Figura();
	 
	 ////////////////////////////////////////////////////////////////////
	 // Prototipi delle funzioni che controllano l'accesso agli attributi
	 // di spaziatura
	 void SetSpG(UL spazi);
	 void SetSpOr(UL spazi);
	 void SetSpDirUt(UL spazi);
	 void SetSpOrcUt(UL spazi);
	 
	 void SetSpG(UL spazi, int vorp);
	 void SetSpOr(UL spazi, int vorp);
	 void SetSpDirUt(UL spazi, int vorp);
	 void SetSpOrcUt(UL spazi, int vorp);
	 
	 UL GetSpG();
	 UL GetSpOr();
	 UL GetSpDirUt();
	 UL GetSpOrcUt();
	 
	 UL GetSpG(int );
	 UL GetSpOr(int );
	 UL GetSpDirUt(int );
	 UL GetSpOrcUt(int );
	 
	 UL GetSpDLine();
	 void SetSpDLine(UL spazi);
	 UL GetSpOLine();
	 void SetSpOLine(UL spazi);
	 
	 
	 // Definisco un metodo che mi dice quanto spazio (UL) ci sono fino alla
	 // prossima figura nel pentagramma
	 
	 virtual UL GetSp2NextFig();
	 
	 
	 //gli stessi ma a comando sui parametri di line break e dirett/orchestr
	 virtual UL GetSp2NextFigPar(int autolb, int dirororch);
	 // autolb=0 Non AutoLineBreak =1 AutoLineBreak  
	 // dirororch=0 direttoriale  =1 orchestrale
	 
	 void SetNotina(Bool b);
	 // Seleziona la grandezza della nota o pausa.
	 Bool GetNotina();
	 // Dice se la nota o pausa e' una notina.
	 
	 
	 // Qui si setta il numero di 
	 // pentagramma a cui appartiene la figura
	 // all'interno della battuta "multi-pentagramma"
	 // Gli indici vanno da 1 a 3 compresi.
	 //
	 // Per Intestazione* intest[NUM_MAX_INTEST]
	 // Gli indici vanno da 0 a 2
	 
	 void SetNStaff(short numstaff); 
	 short GetNStaff(); 
	 virtual void GetMinMaxStaff(short& minStaff, short& maxStaff);
	 
	 void SetPBattuta(Battuta *pBat); 
	 Battuta *GetPBattuta(); 

	 virtual void Suona(MoodsMidiWrite *);
	 virtual float SuonaAppoggiatura(MoodsMidiWrite *);
	 virtual void  SuonaWithAppoggiatura(MoodsMidiWrite *,float);
	 virtual void SuonaAsAcciaccatura(MoodsMidiWrite *);
	 virtual void Suonalanota(MoodsMidiWrite * mmw, float durata, int altezza);
	 virtual void SuonaWithMordenteSuperiore(MoodsMidiWrite *mmw);
	 virtual void SuonaWithMordenteInferiore(MoodsMidiWrite *mmw);
	 virtual void SuonaWithGruppettoSuperiore(MoodsMidiWrite *mmw);
	 virtual void SuonaWithGruppettoInferiore(MoodsMidiWrite *mmw);
	 virtual void SuonaWithTremolo(MoodsMidiWrite *mmw);
	 virtual void SuonaWithTrillo(MoodsMidiWrite *mmw);
	 
	 void SetTerzina(LegatQuadra *pleg);
	 void DeleteTerzina(LegatQuadra *pleg);
	 LegatQuadra *GetTerzina(void);
	 
	 CmdResult CmdFiguraPunto(int NPunti, Battuta *pBattuta=NULL);
	 // Imposta NPunti punti di valore alla figura. Se riesce, restituisce
	 // CMD_OK, altrimenti CMD_FAIL.
	 // E' utilizzato da Metronomo, pr questo e' pubblico.
	 
	 
     void SetAppSil(int a,int b){AppSil[a]=b; };
	 // Richiamata in ListaSillabe(Add) viene
	 // settata a 1 o 0 per indicare che alla
	 // figura  associata o meno una sillaba
	 // della strofa specificata dall'intero a.
	 int GetAppSil(int a) { return AppSil[a]; };
	 // Restituisce lo stato di occupazione
	 // della figura in relazione alla strofa
	 // passatagli in ingresso.
	 
	 virtual Bool IsANote();
	 virtual Bool IsARest();
	 
	 
	 /*BIAN-VERR*/ 	 
   	 void SetSegnoAltezza(bool SA);
	 Bool GetSegnoAltezza();
	 /*BIAN-VERR*/ 
	 
	 
	 virtual void Init();
	 // Inizializza la figura, impostando le variabili e ponendo NULL tutti
	 // i puntatori.
	 
	 virtual void Raise(int delta_altezza);

	 void SetAltezza(int alt);  
	 // Imposta l'altezza musicale grafica della figura.

	 int GetAltezza();
	 // Restituisce l'altezza musicale grafica della figura.

	 virtual float GetDurata(); 
	 //restituisce la durata della nota.  
	 
	 float GetDurata(float den, float num);
	 void SetDurata(float dur); 
	 
	 
	 void SetVU2UpF(VUnit vu);  
	 void SetVU2DwnF(VUnit vu);  
	 void SetVU2LftF(VUnit vu);  
	 void SetVU2RgtF(VUnit vu);  
	 
	 VUnit GetVU2UpF();
	 VUnit GetVU2DwnF();
	 VUnit GetVU2LftF();
	 VUnit GetVU2RgtF();
	 
	 void SetVU2Up(VUnit vu);  
	 void SetVU2Dwn(VUnit vu);  
	 void SetVU2Lft(VUnit vu);  
	 void SetVU2Rgt(VUnit vu);  
	 
	 VUnit GetVU2Up();
	 VUnit GetVU2Dwn();
	 VUnit GetVU2Lft();
	 VUnit GetVU2Rgt();
	 
	 void SetNumSpaces(int ns);
	 int GetNumSpaces();      
	 
	 void SetNumericCode(NumCode nc);
	 NumCode GetNumericCode();
	 
	 void SetPCorona(Corona *pc);
	 // Imposta il puntatore alla corona della figura.  
	 Corona *GetPCorona();
	 // Restituisce il puntatore alla corona.
	 void SetPOcchiali(Occhiali *po);
	 // Imposta il puntatore agli occhiali della figura.
	 Occhiali *GetPOcchiali();  
	 // Restituisce il puntatore agli occhiali.
	 void SetPDitaCorde(DitaCorde *pd);
	 // Imposta il puntatore ai frets della figura.
	 DitaCorde *GetPDitaCorde();  
	 // Restituisce il puntatore ai frets.
	 void SetPPuntoValore(PuntoValore *ppv, Bool updateSpG=TRUE);
	 // Imposta il puntatore al punto di valore della figura.  
	 PuntoValore *GetPPuntoValore();
	 // Restituisce il puntatore al punto di valore.
	 void SetPTDinamico(TDinamico *ptd);
	 // Imposta il puntatore al testo dinamico della figura.
	 TDinamico *GetPTDinamico();
	 // Restituisce il puntatore al testo dinamico della figura.
	 void SetPTGenerico(TGenerico *ptg);
	 // Imposta il puntatore al testo Generico della figura.
	 TGenerico *GetPTGenerico();
	 // Restituisce il puntatore al testo Generico della figura. 
	 void SetPAnnotazione(Annotazione *pa);
	 // Imposta il puntatore all'Annotazione della figura.
	 Annotazione *GetPAnnotazione();
	 // Restituisce il puntatore all'Annotazione della figura. 
	 void SetPStrumento(Strumento *pst);
	 Strumento *GetPStrumento();
	 /////////////////////////////////////////////////////////////////
	 void SetParentFig(Figura *par);
	 Figura *GetParentFig();
	 /////////////////////////////////////////////////////////////////
	 virtual	void SetStatus(Status_ID stat);
	 Status_ID GetStatus();
	 //Setta e ritorna lo stato della figura.
	 
	 bool HastobeDraw();
	 //Riferisce se la figura deve essere disegnata o no.
	 
	 
	 void Set_Link(wxString hyperlink){HyperLink=hyperlink;}
	 wxString Get_Link() {return HyperLink;} 
	 
	 virtual void SetDurationDuringEdit(){}
	 // Imposta la durata propria di ogni Figura
	 void SetPos(DrawObject *d,const Point& p);
	 // Imposta la posizione della figura e chiama Adjust().  
	 virtual void SetPosWA(DrawObject *d,const Point& p);
	 // Imposta la posizione della figura (Without Adjusting).
	 
	 virtual void Adjust(Bool primaleg, Battuta* pBattuta, short current_layer,short current_staff);
	 // Sistema al posto giusto tutti i simboli riferiti alla figura.
	 
	 void Draw();                           
	 // Disegna la figura.
	 
	 Bool DeleteSym(DrawObject *sym);
	 // Cancella il simbolo specificato se e' fra quelli connessi a figura.
	 // Ritorna TRUE se il simbolo e' stato cancellato
	 Bool DeleteStrumento(DrawObject *sym);
	 // Utilizzato da DeleteSym: tratta i casi particolari degli strumenti.
	 // per figura ritorna FALSE, deve essere ridefinito nelle figure che hanno
	 // gli strumenti.
	 
	 virtual void Free();
	 // Dealloca tutti i simboli connessi alla figura.
	 // Inizializza la figura. 
	 
	 Bool Hit(const Point&, SelObj, DrawObject*&, SymPath&);
	 // Ritorna TRUE se: - e' stato colpito e e' impostata una selezione FIGURA_SEL oppure ANY_SEL.
	 // 		  - e' stato colpito uno degli attributi "legati" alla figura ed e' stata
	 // 			impostata una selezione ANY_SEL.
	 Bool Hit(const class Rectangle&,SelObj, SymPath&, SymPath&);
	 Bool SymbolicHit(SymPath&, SelObj, DrawObject*&);
	 
	 virtual Bool FigPtr2SymPath(SymPath&, Figura*);
	 // ricostruisce il PATH di una figura a partire dal puntatore.
	 
	 virtual DrawObject * GetAggiunta(SymCmd * MyCmd);
	 
	 CmdResult SymCommand(SymCmd&, Battuta *pBattuta=NULL);
	 
	 void SetupSimboli();
	 
	 
	 
	 
	 void AddIntRef(IntRefType,IntEsteso*);
	 void DelIntRefs();
	 
	 int GetNIntRefs() { return nIntRefs; };
	 IntRef *GetIntRef(int n);
	 
	Raggruppamento* GetRaggruppamento();	//FFF
	//restituisce il puntatore posto sopra alla figura.Se la figura non fa parte di un raggruppamento restituisce NULL
	Raggruppamento* GetRaggruppamentoIniziale();	//FFF
	//restituisce il puntatore del raggruppamento che inizia dalla figura corrente

    virtual int SaveXML(FILE *);
	int SaveXMLGiust(FILE *);

};    

#endif //FIGURA_HPP
