/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: gruppono.cpp,v 1.1 2005/07/25 16:30:39 bellinpo Exp $ */

#include "liooext.hpp"
#include "fig/gruppono.hpp" 
#include "fig/spazio.hpp" 
#include "gui/dialogs.hpp"


extern int DebugNormalMilla;
extern int DebugNormalMillaPos;
extern wxString msgdebug;

#define STAFF_DIST 20 
#define LGAMBOBEAMMIN 15


GruppoNote::GruppoNote()
{
	Init();
}   

void GruppoNote::Init()
{
	Figura::Init();         
	ID=CL_GRUPPONOTE;
	// Imposta la lista vuota.
	first=NULL;
	last=NULL;
	numobject=0;
	beamPos=BEAM_UP;
	// Inizializza gli attributi. 
	SetNumNote(0); 
	SetFirstNota(NULL);
	SetLastNota(NULL);
	SetNotina(FALSE);
	SetSpaceWidth(SPACEWIDTHDEF);
	SetLGambo(0);
	// Inizializza NoteID con CL_GRUPPONOTE come segnale che nel gruppo
	// non ci sono ancora note.
	SetNoteID(CL_GRUPPONOTE);
	NBarre=0;
	GetBPoint1()=Point(0,0);
	GetBPoint2()=Point(0,0);
	BSlope[currentView]=0;
}               

void GruppoNote::SetNumNote(int nn)
{
	NumNote=nn;
}


int GruppoNote::GetNumNote()
{
	return NumNote;
}


void GruppoNote::SetFirstNota(Nota *pn) 
{
	FirstNota=pn;
} 


Nota* GruppoNote::GetFirstNota()
{ 
	return FirstNota;
}    


void GruppoNote::SetLastNota(Nota *pn)
{
	LastNota=pn;
}                


Nota* GruppoNote::GetLastNota()
{ 
	return LastNota;
}    


void GruppoNote::SetNotina(Bool b)
{
	Notina=b;
}


Bool GruppoNote::GetNotina()
{
	return Notina;
}

///////////////////////////////////////////////////////////////

void GruppoNote::SetBeamStatus(Status_ID stat)
{
	Figura *tmp;
	tmp=GetFirstFig();
	if(tmp!=NULL)
	{
		while(tmp!=NULL)
		{
			((Nota *)(tmp))->SetStatus(stat);
			tmp=GetNextF(tmp);
		}
		
	}
	SetStatus(stat);
}
///////////////////////////////////////////////////////////////
void GruppoNote::SetSmallBeam(Bool var)
{
	Figura *tmp;
	tmp=GetFirstFig();
	if(tmp!=NULL)
	{
		while(tmp!=NULL)
		{
			if(tmp->GetID()==CL_ACCORDO)
			{
				if(var)
					((Accordo *)(tmp))->SetSmallAccordo();
				else
					((Accordo *)(tmp))->SetBigAccordo();
			}
			else
				((Nota *)(tmp))->SetNotina(var);
			tmp=GetNextF(tmp);
		}
		
	}
	SetNotina(var);
}

///////////////////////////////////////////////////////////////
void GruppoNote::SetBeamUp()
{
	Figura *tmp;
	tmp=GetFirstFig();
	if(tmp!=NULL)
	{
		while(tmp!=NULL)
		{
			if (tmp->GetID()==CL_NCROMA || tmp->GetID()==CL_NSEMICROMA ||
				tmp->GetID()==CL_NBISCROMA || tmp->GetID()==CL_NSEMIBISCROMA ||
				tmp->GetID()==CL_NFUSA )
				((Nota *)(tmp))->SetGamboUp();
			tmp=GetNextF(tmp);
		}
		
	}
}

///////////////////////////////////////////////////////////////
void GruppoNote::SetBeamDwn()
{
	Figura *tmp;
	tmp=GetFirstFig();
	if(tmp!=NULL)
	{
		while(tmp!=NULL)
		{
			if (tmp->GetID()==CL_NCROMA || tmp->GetID()==CL_NSEMICROMA ||
				tmp->GetID()==CL_NBISCROMA || tmp->GetID()==CL_NSEMIBISCROMA ||
				tmp->GetID()==CL_NFUSA )
				((Nota *)(tmp))->SetGamboDwn();
			tmp=GetNextF(tmp);
		}
	}
}
/////////////////////////////////////////////////////////////// 
void GruppoNote::SetSpaceWidth(VUnit sw) 
{
	SpaceWidth=sw;
}


VUnit GruppoNote::GetSpaceWidth()
{
	return SpaceWidth;
}


void GruppoNote::SetLGambo(VUnit lg)
{  
	if(GetNotina())
	{
        if(abs(lg)<LGAMBOMIN/2)
		{
			LGambo = (lg>=0)? LGAMBOMIN/2: -LGAMBOMIN/2;
		}   
		else        
		{
			LGambo=lg;
		}
	}
	else  
	{
		if(abs(lg)<LGAMBOMIN)
		{
			LGambo = (lg>=0)? LGAMBOMIN: -LGAMBOMIN;
		}   
		else        
		{
			LGambo=lg;
		}
	}
}     


void GruppoNote::SetGamboUp()
{
	beamPos=BEAM_UP;
	if(GetNotina())
	{
		SetLGambo(LGAMBODEF/2);
	}
	else
	{
		SetLGambo(LGAMBODEF);
	}
}


void GruppoNote::SetGamboDwn()
{
	beamPos=BEAM_DWN;
	if(GetNotina())
	{
		SetLGambo(-LGAMBODEF/2);
	}
	else
	{
		SetLGambo(-LGAMBODEF);
	}
}

Bool GruppoNote::AddFigura(Figura *pf)
{    
	// ho eliminato il parametro TipoInserimento tpIns                            
	ClassID nID=pf->GetID();    
	Nota *pn;
	Accordo *pa;
	
	if(nID==CL_SPAZIO ||
		nID==CL_PSEMIBREVE || nID==CL_PMINIMA || nID==CL_PSEMIMINIMA ||
		nID==CL_PCROMA || nID==CL_PSEMICROMA || nID==CL_PBISCROMA ||
		nID==CL_PSEMIBISCROMA || nID==CL_PFUSA) 
	{
        if (first!=NULL) InsertEndSimple(pf);       
		///////////////////////////////////////////////////////////////////////////
		pf->SetParentFig(this);
		///////////////////////////////////////////////////////////////////////////
        SetNumSpaces((int)numobject);
		return TRUE;
	}
	
	if(nID==CL_ACCORDO)
	{            
		// Se nel gruppo si sta inserendo un accordo, prendi come nID il
		// codice delle note dell'accordo.
		pa=(Accordo*)pf;
		nID=pa->GetNoteID();
	}
	// Controllo della classe della figura inserita.
	if(nID!=CL_NCROMA && nID!=CL_NSEMICROMA && nID!=CL_NBISCROMA && nID!=CL_NSEMIBISCROMA && nID!=CL_NFUSA)
		return FALSE;
	
	if(GetNumNote()==0)
	{
		// La nota da inserire e' la prima => le impostazioni di questa nota
		// diventano le impostazioni del gruppo. Percio' questa nota non puo'
		// avere gambo nullo.
		pn=(Nota*)pf;                        
		SetNotina(pn->GetNotina());
		SetFirstNota(pn);
	}
	else
	{
		pn=(Nota*)pf;  
		// Controllo della dimensione della nota.
		if(GetNotina()!=pn->GetNotina())
			return(FALSE);
	}
	
    InsertEndSimple(pn); 
	///////////////////////////////////////////////////////////////////////////
    pn->SetParentFig(this);
    ///////////////////////////////////////////////////////////////////////////
	SetNumNote(GetNumNote()+1);
	SetNumSpaces((int)numobject);
	SetLastNota(pn);
	return(TRUE);
}                  

Bool GruppoNote::AddAfterFigura(Figura *pf1,Figura *pf2)
{                                 
	ClassID nID=pf2->GetID();    
	
	if(nID==CL_SPAZIO || 
		nID==CL_PSEMIBREVE || nID==CL_PMINIMA || nID==CL_PSEMIMINIMA ||
		nID==CL_PCROMA || nID==CL_PSEMICROMA || 
		nID==CL_PBISCROMA || nID==CL_PSEMIBISCROMA || nID==CL_PFUSA) 
	{
		Node *n;
		n=FindNodePtr(pf1);
        if(n!=NULL)
		{
			InsertAfter(n,pf2);
			///////////////////////////////////////////////////////////////////////////
			pf2->SetParentFig(this);
			///////////////////////////////////////////////////////////////////////////
			SetNumSpaces((int)numobject);
			return TRUE;
		}
	}                  
	
	return FALSE; 
}

Bool GruppoNote::DelFigura(Figura *pf1)
{                                 
	ClassID nID=pf1->GetID();    
	
	if(nID==CL_SPAZIO ||
		nID==CL_PSEMIBREVE || nID==CL_PMINIMA || nID==CL_PSEMIMINIMA ||
		nID==CL_PCROMA || nID==CL_PSEMICROMA || 
		nID==CL_PBISCROMA || nID==CL_PSEMIBISCROMA || nID==CL_PFUSA) 
	{
		DelPtr(pf1);
		SetNumSpaces((int)numobject);
		return TRUE;
	}
	return FALSE;
}


Figura* GruppoNote::ExtractFigura(Battuta *pbat, short current_layer)
{ 
	Figura *pfig;
	Nota *pnota;
	ClassID nID;
	Bool gamboupdwn;
	int lgambo;
    if (DebugNormalMilla || DebugNormalMillaPos)
    { 
		msgdebug.sprintf("");
    }
    pfig=DelLastF();
    if(pfig!=NULL)
    {
        nID=pfig->GetID();
		pfig->SetParentFig(NULL);
		SetNumSpaces((int)numobject);     
		// Se si e' estratto una nota...
		if(nID!=CL_SPAZIO && 
			nID!=CL_PSEMIBREVE && nID!=CL_PMINIMA && nID!=CL_PSEMIMINIMA &&
			nID!=CL_PCROMA && nID!=CL_PSEMICROMA && nID!=CL_PBISCROMA &&
			nID!=CL_PSEMIBISCROMA && nID!=CL_PFUSA) 
		{                      
			SetNumNote(GetNumNote()-1);
			SetFirstNota((Nota*)GetFirst());
			// Ripristina la nota estratta.
			pnota=(Nota*)pfig;
			
			
			if (immissione.ImmRule(nID, pnota, pbat, NULL, current_layer))  
				
			{ 
				if (pnota->GetNotina())
				{ 
					pnota->SetGamboUp();
				} 
				else
				{ 
					gamboupdwn = TRUE;
					lgambo = int(posizionam.PosRule((DrawObject*)pnota, pnota, pbat,
						gamboupdwn, current_layer,pfig->GetNStaff()));
					pnota->SetLGambo(lgambo);
				}
			}	  
			else 
			{  
				if (pnota->GetNotina())
				{ 
					pnota->SetGamboDwn();
				} 
				else
				{ 
					gamboupdwn = FALSE;
					lgambo = int(posizionam.PosRule((DrawObject*)pnota, pnota, pbat,
						gamboupdwn, current_layer,pfig->GetNStaff()));  
					pnota->SetLGambo(-lgambo);
				}
			}	 
			pnota->SetCoda(TRUE);
		}
		// NON ESEGUE ADJUST NE' DELLA NOTA ESTRATTA, NE' DEL GRUPPO RIMASTO! 
		
		
		
	}
    
    
	if (DebugNormalMilla || DebugNormalMillaPos)
	{
		
        if (pnota->GetTraceMilla())
			posizionam.TestTokenCond(pnota, (DrawObject*)pnota, pbat, gamboupdwn, current_layer);
		
        
		WDebugMilla(); 
	}
    
    
    
    
	return pfig;
	
}


void GruppoNote::SetPos(DrawObject* d,const Point& p)
{
	
	DrawObject::SetPos(d,p);
}

void GruppoNote::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	if(HastobeDraw())
	{
		Node *pnode;
		Figura *pfig;
				
		// Disegna le note.                                     
		pnode=first;
		while(pnode!=NULL)
		{           
			pfig=(Figura*)pnode->ObjectPtr;
			pfig->Draw();
			pnode=pnode->Next;
		} 
				
		DrawBarre();
	}

	if(isSelected) {
		DrawContour();
	}

	Scr->SetColor(oldcol);
}


void GruppoNote::DrawTratto(Nota *pnota, short nBarra, short Direz,
							VUnit BThick)
{
	short i;
	
	// Disegna le barre dal basso verso l'alto.
	for(i=0;i<nBarra;i++)
	{
		if(Tratto[currentView][i].P1!=Point(0,0))
		{
			if(Tratto[currentView][i].P1==Tratto[currentView][i].P2)
			{
				// Nota "isolata".
				if(Tratto[currentView][i].P1.x==GetBPoint1().x)
				{
					// Nota iniziale del gruppo => mezzo tratto verso destra.
					Tratto[currentView][i].P2.x = Tratto[currentView][i].P1.x + TRATTOMEZZABARRA;
					Tratto[currentView][i].P2.y = Tratto[currentView][i].P1.y + (VUnit)(BSlope[currentView]*TRATTOMEZZABARRA);
				}
				else
				{
					// Mezzo tratto verso sinistra.
					Tratto[currentView][i].P2 = Tratto[currentView][i].P1;
					Tratto[currentView][i].P1.x = Tratto[currentView][i].P2.x - TRATTOMEZZABARRA;
					Tratto[currentView][i].P1.y = Tratto[currentView][i].P2.y - (VUnit)(BSlope[currentView]*TRATTOMEZZABARRA);
				}
			}
			Scr->BigLine(Tratto[currentView][i].P1-Point(0,(Direz*BThick)), Tratto[currentView][i].P2-Point(0,(Direz*BThick)), Direz*BThick);
			// Resetta l'inizio del tratto.
			Tratto[currentView][i].P1 = Point(0,0);
		}
	}
	
	
	// Se siamo al termine, non necessita aggiornamento.
	if(pnota==NULL)
		return;
	// Aggiorna il punto finale dei tratti soprastanti.
	for(i=nBarra;i<NBarre-1;i++)
	{
		// DXGambo deve essere assegnato in base alla direzione ed alla
		// dimensione della nota.
		Tratto[currentView][i].P2.x = pnota->GetXgambo();
		Tratto[currentView][i].P2.y = Tratto[currentView][i].PRef.y +
			VUnit(BSlope[currentView] * (Tratto[currentView][i].P2.x - Tratto[currentView][i].PRef.x));
		
		// Se questo e` l'inizio di un tratto, memorizza il punto iniziale.
		if(Tratto[currentView][i].P1==Point(0,0))
			Tratto[currentView][i].P1 = Tratto[currentView][i].P2;
	}
	
}


void GruppoNote::DrawBarre()
{
	Node *pnode;
	Figura *pfig;
	Nota *pnota;
	short Direz, i;
	
	ClassID nID;
	VUnit BThick, dYB;
	
	if(beamPos==BEAM_12 || beamPos==BEAM_23)
	{
		DrawBarreMiddle();
		return;
	}
	// Disegna le barrette solo se il metodo Adjust ha avuto buon esito.                   
	if(GetBPoint1()==Point(0,0) || GetBPoint1().x==GetBPoint2().x)
		return;            
	
	Direz = (GetLGambo()>0)? 1: -1;
	GetBPoint2().y = GetBPoint1().y + (VUnit)(BSlope[currentView]*(GetBPoint2().x-GetBPoint1().x));
	
	BThick = (GetNotina()?THICKBARRETTA1:THICKBARRETTA);
	dYB = (GetNotina()?DELTAYBARRETTE1:DELTAYBARRETTE);
	
	
	// DISEGNO PER LE NOTE.
	// Inizia dalla barra piu` esterna.
	Scr->BigLine(GetBPoint1()+Point(0, Direz*(NBarre-1)*dYB-(Direz*BThick)),
		GetBPoint2()+Point(0, Direz*(NBarre-1)*dYB-(Direz*BThick)),
		Direz*BThick);
	// Eventualmente prosegui con le barre interne.
	for(i=0;i<NBarre-1;i++)
	{
		Tratto[currentView][i].PRef = GetBPoint1()+Point(0, Direz*(i)*dYB);
		Tratto[currentView][i].P1 = Point(0,0);     
	}
	
	
	pnode = first;
	while(pnode!=NULL)
	{
		pfig = (Figura*)pnode->ObjectPtr;
		nID = pfig->GetID();
		// Salta gli spazi
		if(nID!=CL_SPAZIO)
		{
			pnota = (Nota*)pfig;
			
			if(nID==CL_ACCORDO)
				nID = ((Accordo*)pnota)->GetNoteID();
			
			switch(nID)
			{
			case CL_NCROMA:
				DrawTratto(pnota, NBarre-1, Direz, BThick);
				break;
			case CL_NSEMICROMA:
				DrawTratto(pnota, NBarre-2, Direz, BThick);
				break;
			case CL_NBISCROMA:
				DrawTratto(pnota, NBarre-3, Direz, BThick);
				break;
			case CL_NSEMIBISCROMA:
				DrawTratto(pnota, NBarre-4, Direz, BThick);
				break;
			case CL_NFUSA:
				DrawTratto(pnota, NBarre-5, Direz, BThick);
				break;
			}
		}
		pnode=GetNext(pnode);
		
	}
	// Concludi il disegno dei tratti.
	DrawTratto(NULL, NBarre-1, Direz, BThick);
}                                                  



Bool GruppoNote::Hit(const Point& p, SelObj selObj, DrawObject*& drwObjSel,
					 SymPath& symPath)
{
	Bool hit;   
	Point p1;
	Node *pnode;
	Figura *pfig;
	
	p1=p-GetAbsPos();               
	hit = -p1.x <= GetVU2LftF() && p1.x <= GetVU2RgtF() &&
		-p1.y <= GetVU2DwnF() && p1.y <= GetVU2UpF();
	
	drwObjSel=NULL;
	
	if(selObj==GRUPPONOTE_SEL)
		if(hit)
		{
			drwObjSel=this;
			symPath.Array[symPath.Level]=GetNumericCode();
			return(TRUE);
		}
		
		// Se si seleziona una nota o una figura del gruppo e il punto di hit e' interno al gruppo
		// oppure se si effettua una selezione ANY_SEL (indipendentementa da hit)
		// allora vengono scandite le figura del gruppo per individuare l'oggetto.      
		if((hit && drwObjSel==NULL && 
			(selObj==NOTA_SEL || selObj==FIGURA_SEL || selObj==NOTASING_SEL)) ||
			(selObj==ANY_SEL))
		{            
			// Scende di un livello nel path.
			symPath.Level++;
			pnode=first;
			while(pnode!=NULL)
			{
				pfig=(Figura*)pnode->ObjectPtr; 
				if(pfig->Hit(p,selObj,drwObjSel,symPath))
				{
					symPath.Array[--symPath.Level]=GetNumericCode();
					return(TRUE);
				}
				pnode=GetNext(pnode);
			}  
			// Se non e` stata trovata la nota, ripristina il livello precedente.
			symPath.Level--;
			// se e' stato colpito il gruppo note ma nessuno degli oggetti ad esso collegati 
			// viene individuato il gruppo note stesso.
		}
		return(FALSE);
}


Bool GruppoNote::SymbolicHit(SymPath& symPath, SelObj selObj,
							 DrawObject*& drwObjSel)
{
	Bool QuestoGruppo = (symPath.Array[symPath.Level]==GetNumericCode()); 
	Bool ScendereAncora = FALSE;
	
	Node *pnode;
	Figura *pfig;
	drwObjSel=NULL;
	
	
	if(symPath.Level < FIGURA3_PATH_LEVEL) 
		ScendereAncora = (symPath.Array[symPath.Level+1]!=0);
	if(!ScendereAncora && QuestoGruppo && 
		(selObj==GRUPPONOTE_SEL))
	{
		drwObjSel=this;
		return(TRUE);
	}
	else if(drwObjSel==NULL && QuestoGruppo &&
		(selObj==NOTA_SEL || selObj==FIGURA_SEL || selObj==NOTASING_SEL  || selObj==ANY_SEL))
	{            
		if(!ScendereAncora)
		{
			drwObjSel=this;
			return(TRUE);
		}
		// Scende di un livello nel path.
		symPath.Level++;
		pnode=first;
		while(pnode!=NULL)
		{
			pfig=(Figura*)pnode->ObjectPtr; 
			if(pfig->SymbolicHit(symPath,selObj,drwObjSel))
			{
				symPath.Level--;
				return(TRUE);
			}
			pnode=GetNext(pnode);
		}  
		// Se non e` stata trovata la nota, ripristina il livello precedente.
		symPath.Level--;
	}
	
	// Prova a cercare la figura nel gruppo
	pnode=first;
	while(pnode!=NULL)
	{
		pfig=(Figura*)pnode->ObjectPtr; 
		if(pfig->SymbolicHit(symPath,selObj,drwObjSel))
		{
			return(TRUE);
		}
		pnode=GetNext(pnode);
	}  
	
	return(FALSE);
}


Bool GruppoNote::Hit(const class Rectangle& r,SelObj selObj,SymPath& sp1,SymPath& sp2)
{
	Node *pnode;
	Bool found=FALSE;
	
	// Aggiorna i livelli per scendere alle singole figure.
	sp1.Level++;
	sp2.Level++;
	pnode = first;
	while(pnode!=NULL)
	{
		if(((Figura*)pnode->ObjectPtr)->Hit(r,selObj,sp1,sp2))
		{
			found = TRUE;
		}
		else if(found)
		{
			// In questo caso il rettangolo e` gia` stato ispezionanto al
			// completo, percio` e` inutile proseguire.
			break;
		}
		pnode = GetNext(pnode);
	}
	// Torna al livello del gruppo di note.
	sp1.Level--;
	sp2.Level--;
	if(found)
	{
		if(sp1.Array[sp1.Level]==0)
			sp1.Array[sp1.Level] = GetNumericCode();
		sp2.Array[sp2.Level] = GetNumericCode();
		return TRUE;
	}
	return FALSE;
}


void GruppoNote::Select()
{
	DrawObject::Select();

	//Scr->SetColor((ColorIcon)XOR_SELECTED_COLOR);
	//Scr->SetDrwMode(XORMode);
	//Scr->Rect(class Rectangle(*this));
	//Scr->SetDrwMode(DefMode);
}


void GruppoNote::Deselect()
{
	DrawObject::Deselect();
	//Select();
}

void GruppoNote::Raise(int delta_altezza)
{
	Figura* pF=GetFirstF();
	while(pF)
	{
		pF->Raise(delta_altezza);
		pF = GetNextF(pF);
	}
}

CmdResult GruppoNote::SymCommand(SymCmd& SCmd, Battuta *pBattuta)
{
	switch(SCmd.CommandId)
	{
	case CMD_FIGURA_1PUNTO:
	case CMD_FIGURA_2PUNTI:
	case CMD_FIGURA_3PUNTI:  
	case CMD_FIGURA_CORONA:
    case CMD_FIGURA_CORONAQUAD:
    case CMD_FIGURA_CORONATRIANG:
	case CMD_FIGURA_PPPPPP:
   	case CMD_FIGURA_PPPPP:
	case CMD_FIGURA_PPPP:
	case CMD_FIGURA_PPP:
	case CMD_FIGURA_PP:
	case CMD_FIGURA_P:
   	case CMD_FIGURA_FFFFFF:
   	case CMD_FIGURA_FFFFF:
	case CMD_FIGURA_FFFF:
	case CMD_FIGURA_FFF:
	case CMD_FIGURA_FF:
	case CMD_FIGURA_F:
	case CMD_FIGURA_MP:
	case CMD_FIGURA_MF:
	case CMD_FIGURA_SP:
	case CMD_FIGURA_SF:
	case CMD_FIGURA_FZ:
	case CMD_FIGURA_SFZ:
	case CMD_FIGURA_FP:
   	case CMD_FIGURA_RF:
   	case CMD_FIGURA_FFP:
   	case CMD_FIGURA_FFPP:
   	case CMD_FIGURA_FPP:
   	case CMD_FIGURA_RFZ:
   	case CMD_FIGURA_SFF:
   	case CMD_FIGURA_SFFF:
   	case CMD_FIGURA_SFFP:
   	case CMD_FIGURA_SFFZ:
   	case CMD_FIGURA_SFP:
   	case CMD_FIGURA_SFPP:
		return(CMD_FAIL);
	}
	return(CMD_UNKNOWN);
}

Bool GruppoNote::FigPtr2SymPath(SymPath& symPath, Figura *figptr)
{
	Bool trovato=FALSE;
	
	Figura *ptr=(Figura*)GetFirst();
	
	while((!trovato) && (ptr!=NULL))
	{
		symPath.Array[FIGURA2_PATH_LEVEL]=(ptr->GetNumericCode());
		
		switch(ptr->GetID())
		{
		case CL_ACCORDO:
			if(ptr==figptr)
				trovato=TRUE;
			else
				trovato = ((Accordo*)ptr)->FigPtr2SymPath(symPath,figptr,TRUE);
			break;
		default:
			if(ptr==figptr)
				trovato=TRUE;
		} 
		
		ptr = (Figura*)GetNext((DrawObject*)ptr);
	}      
	
	return trovato;
}

// un gruppo di note con una sola nota
Bool GruppoNote::isOne(void)
{
	return first==last;
}

Bool GruppoNote::DeleteSym(DrawObject* obj)
{
	Node *pnode;
	Figura *pfig;
	
	pnode=first;
	while(pnode!=NULL)
	{
		pfig=(Figura*)pnode->ObjectPtr; 
		if(pfig!=obj && pfig->DeleteSym(obj))
		{
			return(TRUE);
		}
		pnode=GetNext(pnode);
	}  
	return(FALSE);
}

float GruppoNote::GetMedia()
{
	Node *pnode;
	Nota *pn;
	
	float a; 
	float alt=0;
	float num;
	
	num = GetNumNote();
	
	pnode = first;
	
	int staffinbeam=GetStaffInBeam();
	
	
	while (pnode!=NULL)
	{
		pn = (Nota*)pnode->ObjectPtr;
		if(pn->GetID()!=CL_SPAZIO)
		{ 
			if (staffinbeam>1)
			{
				a = pn->GetAltezza()-4-pn->GetNStaff()*STAFF_DIST;
			}
			else
			{
				a = pn->GetAltezza()-4;
			}
			alt+=a;
		}
		pnode = pnode->Next;
	}  
	
	return alt/num; 
}    

float GruppoNote::GetMean()
{
	Node *pnode;
	Nota *pn;
	
	float a; 
	float alt=0;
	float mean=0;
	float num;
	
	num = GetNumNote();
	
	pnode = first;
	
	int staffinbeam=GetStaffInBeam();
	
	while (pnode!=NULL)
	{  
		pn = (Nota*)pnode->ObjectPtr;
		
		if(pn->GetID()!=CL_SPAZIO)
		{ 
			if (staffinbeam>1)
			{
				a = pn->GetAltezza()-4-pn->GetNStaff()*STAFF_DIST;
			}
			else
			{
				a = pn->GetAltezza()-4; //DB: Prova!
			}
			alt+=a;
		}
		pnode = pnode->Next;
	}  
	
	if(num!=0) mean = alt/num; 
	return mean;
}    

float GruppoNote::GetMeanHighLow()
{
	Node *pnode;
	Nota *pn;
	
	float high, low; 
	float mean;
	float alt;
	
	int staffinbeam=GetStaffInBeam();
	
	pnode = first;
	pn = GetFirstNota();
	
	if(staffinbeam>1)
	{
		high = pn->GetAltezza()-4-pn->GetNStaff()*STAFF_DIST;
		low = pn->GetAltezza()-4-pn->GetNStaff()*STAFF_DIST;
	}
	else
	{
		high = pn->GetAltezza()-4;
		low = pn->GetAltezza()-4;
	}
	
	while (pnode!=NULL)
	{
		pn = (Nota*)pnode->ObjectPtr;
		
		if(pn->GetID()!=CL_SPAZIO)
		{
			if(staffinbeam>1)
			{
				alt = pn->GetAltezza()-4-pn->GetNStaff()*STAFF_DIST;
			}
			else
			{
				alt = pn->GetAltezza()-4;
			}
			if (alt>high)
			{
				high = alt;
			}
			if (alt<low)
			{
				low = alt;
			}
		}        
		pnode = pnode->Next;    
	} 
	
	mean = (high+low)/2; 
	return mean;
}     


float GruppoNote::GetBeamDis()
{      
	Nota *pn;       
	Node *pnode;  
	float low, high, alt; 
	
	int staffinbeam=GetStaffInBeam();
	pnode = first;
	pn = GetFirstNota();  
	
	if(staffinbeam>1)
	{
		high = pn->GetAltezza()-4-pn->GetNStaff()*STAFF_DIST;
		low = pn->GetAltezza()-4-pn->GetNStaff()*STAFF_DIST;
	}
	else
	{
		high = pn->GetAltezza()-4;
		low = pn->GetAltezza()-4;
	}
	while (pnode!=NULL)
	{
		pn = (Nota*)pnode->ObjectPtr;
		if(pn->GetID()!=CL_SPAZIO)
		{
			if(staffinbeam>1)
			{
				alt = pn->GetAltezza()-4-pn->GetNStaff()*STAFF_DIST;
			}
			else
			{
				alt = pn->GetAltezza()-4;
			}
			if (alt>high)
			{
				high = alt;
			}
			if (alt<low)
			{
				low = alt;
			} 
		}      
		pnode = pnode->Next;    
	}
	//FFF
	return (abs(high)-abs(low));
}  

float GruppoNote::GetDelta()
{
	float num, alt, high, low;
	Nota *pn;
	Node *pnode;
	
	num = GetNumNote();
	
	int staffinbeam=GetStaffInBeam();
	pnode = first;    
	pn = GetFirstNota(); 
	
	if(staffinbeam>1)
	{
		high = pn->GetAltezza()-4-pn->GetNStaff()*STAFF_DIST;
		low = pn->GetAltezza()-4-pn->GetNStaff()*STAFF_DIST;
	}
	else
	{
		high = pn->GetAltezza()-4;
		low = pn->GetAltezza()-4;
	}
	
	while (pnode!=NULL)
	{
		pn = (Nota*)pnode->ObjectPtr;
		if(pn->GetID()!=CL_SPAZIO)
		{
			if(staffinbeam>1)
			{
				alt = pn->GetAltezza()-4-pn->GetNStaff()*STAFF_DIST;
			}
			else
			{
				alt = pn->GetAltezza()-4;
			}
			if (alt>=high)
			{
				high = alt;
			}
			if (alt<=low)
			{
				low = alt;
			}   
		}   
		pnode = pnode->Next;    
	}
	//FFF
	return (abs(high)-abs(low))/(num+1);
	
}  

void GruppoNote::Adjust(Bool primaleg, Battuta *pBattuta, short current_layer, short current_staff)
{
	Node *pnode; 
	Figura *pfig;
	Nota *pnota;
	Nota *pfn,*pln;
	// Puntatori alla prima e all'ultima nota del gruppo.
	Nota *PNSignif;
	// Puntatore alla NOTA PIU' SIGNIFICATIVA del gruppo, presa come la nota
	// piu' alta del gruppo se LGambo>0 o la piu' bassa se LGambo<0.  
	VUnit x,dist,y,y0,lg,dx,dy;
	VUnit prova[12], anchorpos[12]; 
	VUnit MHL=0;
	VUnit mindistMHL=0;
	Bool CorrectAdj;
	// Segnala se Adjust sta lavorando correttamente.
    short Times, nb;
    int datracciare=0;
    ClassID nID;
	
	for(int i=0; i< NUM_MAX_LAYER;i++)
		prova[i]=0;
	
    
    if (DebugNormalMilla || DebugNormalMillaPos)
    { 
		msgdebug.sprintf("");
    }
    
    
	if(primaleg)
	{
		// Inizializza i BPoint.
		GetBPoint1()=Point(0,0);
		GetBPoint2()=Point(0,0);                  
		
		// SCANSIONE DELLE NOTE PER DETERMINARE LA NOTA PIU' SIGNIFICATIVA
		// E PER SISTEMARE LA POSIZIONE DELLE NOTE E DEGLI SPAZI.
		PNSignif=GetFirstNota();
		x=0;
		NBarre=0;
		pnode=first;
		switch(beamPos)
		{             
		case BEAM_UP:
			while(pnode!=NULL)
			{          
				pfig=(Figura*)pnode->ObjectPtr;
				pfig->SetPBattuta(this->GetPBattuta());
				nID=pfig->GetID(); 
				// Sistema le note e gli spazi. 
				// Se *pfig e' un accordo, prima di fare SetPos, occorre impostare
				// il gambo.
				
				if(nID==CL_SPAZIO) 
				{
					Spazio *figsp=(Spazio *)pfig;
					anchorpos[current_layer]=((x-prova[current_layer])*figsp->Getrelpos())/100;
					anchorpos[current_layer]+=prova[current_layer];
					//pfig->SetPosWA(this,Point(UL2VU(anchorpos[current_layer]),0));
					pfig->SetPosWA(NULL,Point(this->GetAbsPos().x+UL2VU(anchorpos[current_layer]),pBattuta->GetIntest(pfig->GetNStaff())->GetAbsPos().y));
				}
				else
				{
					prova[current_layer]=x; 
					if(nID==CL_ACCORDO) 
						((Accordo*)pfig)->SetLGambo(GetLGambo());
					
					short nstaff=pfig->GetNStaff();
					if(nstaff<0){
						pfig->SetPosWA(NULL,Point(this->GetAbsPos().x+UL2VU(x),pBattuta->GetIntest(0)->GetAbsPos().y));
					}
					else {
						pfig->SetPosWA(NULL,Point(this->GetAbsPos().x+UL2VU(x),pBattuta->GetIntest(pfig->GetNStaff())->GetAbsPos().y));
					}
					
					if(nID!=CL_SPAZIO && 
						nID!=CL_PSEMIBREVE && nID!=CL_PMINIMA && nID!=CL_PSEMIMINIMA &&
						nID!=CL_PCROMA && nID!=CL_PSEMICROMA && nID!=CL_PBISCROMA &&
						nID!=CL_PSEMIBISCROMA && nID!=CL_PFUSA) 
					{    
						pnota=(Nota*)pfig;
						pnota->SetLGambo(GetLGambo());
						if(pnota->GetNStaff()<PNSignif->GetNStaff() ||
							(pnota->GetNStaff()==PNSignif->GetNStaff() &&
							pnota->GetAltezza()>PNSignif->GetAltezza()))
							PNSignif=pnota;
						// Calcola il numero di barrette.
						nID=pnota->GetID();
						if(nID==CL_ACCORDO)
							nID=((Accordo*)pnota)->GetNoteID();
						nb = nID - CL_NCROMA + 1;
						if(nb>NBarre)
							NBarre=nb;
					}
				}
				
				if (pnota->GetTraceMilla())
					datracciare=1;
				pnode=pnode->Next;
				
				x+=pfig->GetSp2NextFig(); //GetSpaceWidth();
			}
			break;
		case BEAM_DWN:
			while(pnode!=NULL)
			{
				pfig=(Figura*)pnode->ObjectPtr;
				nID=pfig->GetID(); 
				pfig->SetPBattuta(this->GetPBattuta());
				// Sistema le note e gli spazi. 
				// Se *pfig e' un accordo, prima di fare SetPos, occorre impostare
				// il gambo.
				if(nID==CL_SPAZIO) 
				{
					Spazio *figsp=(Spazio *)pfig;
					anchorpos[current_layer]=((x-prova[current_layer])*figsp->Getrelpos())/100;
					anchorpos[current_layer]+=prova[current_layer];
					pfig->SetPosWA(NULL,Point(this->GetAbsPos().x+UL2VU(anchorpos[current_layer]),pBattuta->GetIntest(pfig->GetNStaff())->GetAbsPos().y));
				}
				else
				{
					prova[current_layer]=x; 
					if(nID==CL_ACCORDO) 
						((Accordo*)pfig)->SetLGambo(GetLGambo());
					
					short nstaff=pfig->GetNStaff();
					if(nstaff<0){
						pfig->SetPosWA(NULL,Point(this->GetAbsPos().x+UL2VU(x),pBattuta->GetIntest(0)->GetAbsPos().y));
					}
					else {
						pfig->SetPosWA(NULL,Point(this->GetAbsPos().x+UL2VU(x),pBattuta->GetIntest(pfig->GetNStaff())->GetAbsPos().y));
					}
					if(nID!=CL_SPAZIO && 
						nID!=CL_PSEMIBREVE && nID!=CL_PMINIMA && nID!=CL_PSEMIMINIMA &&
						nID!=CL_PCROMA && nID!=CL_PSEMICROMA && nID!=CL_PBISCROMA &&
						nID!=CL_PSEMIBISCROMA && nID!=CL_PFUSA) 
					{    
						pnota=(Nota*)pfig;
						pnota->SetLGambo(GetLGambo());
						if(pnota->GetNStaff()>PNSignif->GetNStaff() ||
							(pnota->GetNStaff()==PNSignif->GetNStaff() &&
							pnota->GetAltezza()<PNSignif->GetAltezza()))
							PNSignif=pnota;
						// Calcola il numero di barrette.
						nID=pnota->GetID();
						if(nID==CL_ACCORDO)
							nID=((Accordo*)pnota)->GetNoteID();
						nb = nID - CL_NCROMA + 1;
						if(nb>NBarre)
							NBarre=nb;
					}  
				}
				if (pnota->GetTraceMilla()) datracciare=1;
				pnode=pnode->Next;
				
				
				x+=pfig->GetSp2NextFig();//GetSpaceWidth();
			}
			break;
		case BEAM_12:
		case BEAM_23:
			{	
				VUnit nMax=0,nMin=0;
				
				while(pnode!=NULL)
				{
					pfig=(Figura*)pnode->ObjectPtr;
					nID=pfig->GetID(); 
					pfig->SetPBattuta(this->GetPBattuta());
					// Sistema le note e gli spazi. 
					// Se *pfig e' un accordo, prima di fare SetPos, occorre impostare
					// il gambo.
					if(nID==CL_SPAZIO) 
					{
						Spazio *figsp=(Spazio *)pfig;
						anchorpos[current_layer]=((x-prova[current_layer])*figsp->Getrelpos())/100;
						anchorpos[current_layer]+=prova[current_layer];
						pfig->SetPosWA(NULL,Point(this->GetAbsPos().x+UL2VU(anchorpos[current_layer]),pBattuta->GetIntest(pfig->GetNStaff())->GetAbsPos().y));
					}
					else
					{
						prova[current_layer]=x; 
						if(nID==CL_ACCORDO) 
							((Accordo*)pfig)->SetLGambo(GetLGambo());
						
						short nstaff=pfig->GetNStaff();
						if(nstaff<0){
							pfig->SetPosWA(NULL,Point(this->GetAbsPos().x+UL2VU(x),pBattuta->GetIntest(0)->GetAbsPos().y));
						}
						else {
							pfig->SetPosWA(NULL,Point(this->GetAbsPos().x+UL2VU(x),pBattuta->GetIntest(pfig->GetNStaff())->GetAbsPos().y));
						}
						if(nID!=CL_SPAZIO && 
							nID!=CL_PSEMIBREVE && nID!=CL_PMINIMA && nID!=CL_PSEMIMINIMA &&
							nID!=CL_PCROMA && nID!=CL_PSEMICROMA && nID!=CL_PBISCROMA &&
							nID!=CL_PSEMIBISCROMA && nID!=CL_PFUSA) 
						{    
							pnota=(Nota*)pfig;
							pnota->SetLGambo(GetLGambo());
							if((beamPos==BEAM_12 && pnota->GetNStaff()!=2)||
								(beamPos==BEAM_23 && pnota->GetNStaff()!=0))
							{
								if(nMax==0 || pnota->GetAbsPos().y > nMax) nMax=pnota->GetAbsPos().y;
								if(nMin==0 || pnota->GetAbsPos().y < nMin) nMin=pnota->GetAbsPos().y;
							}
							// Calcola il numero di barrette.
							nID=pnota->GetID();
							if(nID==CL_ACCORDO)
								nID=((Accordo*)pnota)->GetNoteID();
							nb = nID - CL_NCROMA + 1;
							if(nb>NBarre)
								NBarre=nb;
						}
					}
					
					if (pnota->GetTraceMilla()) datracciare=1;
					pnode=pnode->Next;
					
					
					x+=pfig->GetSp2NextFig();//GetSpaceWidth();
				}
				pnode=first;
				MHL=(nMax+nMin)/2;
				mindistMHL=0;
				while(pnode!=NULL)
				{
					pfig=(Figura*)pnode->ObjectPtr;
					
					if(nID!=CL_SPAZIO && 
						nID!=CL_PSEMIBREVE && nID!=CL_PMINIMA && nID!=CL_PSEMIMINIMA &&
						nID!=CL_PCROMA && nID!=CL_PSEMICROMA && nID!=CL_PBISCROMA &&
						nID!=CL_PSEMIBISCROMA && nID!=CL_PFUSA) 
					{    
						pnota=(Nota*)pfig;
						if(beamPos==BEAM_12 || beamPos==BEAM_23)
						{
							VUnit diff=pnota->GetAbsPos().y-MHL;
							VUnit dist=abs(diff);
							if(mindistMHL==0 || dist < mindistMHL)
							{
								mindistMHL=dist;
								PNSignif=pnota;
							}
							if(diff>0 )
								pnota->SetLGambo(-GetLGambo());
							else
								pnota->SetLGambo(GetLGambo());
						}
					}  
					pnode=pnode->Next;
				}
			}
			break;
		}      
		
		// RAGGRUPPAMENTO DELLE NOTE.       
		
		if(GetNumNote()>1)
		{  
			pfn=GetFirstNota();
			pln=GetLastNota();
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				msgdebug.sprintf("");
			}
			
			
			BSlope[currentView]=posizionam.PosRule(this, (Nota*)this, pBattuta, 0, 0,0); //  4 e 5 zero sono current_layer e current_staff			
			
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (datracciare==1) {
					posizionam.TestTokenCond((Nota*)this, this, pBattuta, 0, 0);
					
					WDebugMilla(); 
					
					datracciare=0;
				}     
			}
			
			
			// Imposta il punto di riferimento per calcolare i gambi. Esso  preso
			// LGambo VUnit sopra (o sotto) la nota piu' significativa. Percio' anche
			// la lunghezza del gambo della nota piu' significativa viene modificata.
			if(beamPos==BEAM_UP || beamPos==BEAM_DWN)
				PNSignif->SetLGambo(GetLGambo());
			else
				PNSignif->SetLGambo(MHL-PNSignif->GetAbsPos().y);
			PNSignif->AdjustF();
			PNSignif->SetVU2Up(PNSignif->GetVU2UpF());
			PNSignif->SetVU2Dwn(PNSignif->GetVU2DwnF());
			PNSignif->SetVU2Lft(PNSignif->GetVU2LftF());
			PNSignif->SetVU2Rgt(PNSignif->GetVU2RgtF());  
			PNSignif->Adjust(TRUE,pBattuta,current_layer,current_staff);
			y0= PNSignif->GetYgambo() + PNSignif->GetLGambo();
			CorrectAdj=FALSE;
			Times=0;
			
			// Di regola il corpo del seguente ciclo verra' eseguito una sola volta.
			// Se pero' qualche gambo si ritrova una lunghezza troppo bassa, il
			// valore di BSlope[currentView] viene cambiato e viene fatta una seconda iterazione.
			// La variabile Times impedisce i loop infiniti.
			while(!CorrectAdj && Times<2)
			{       
				Times++;
				// Suppone che l'impostazione avverra' correttamente.
				CorrectAdj=TRUE;
				// Scandisce la lista per impostare i gambi delle note.
				pnode=first;
				while(pnode!=NULL && CorrectAdj)
				{      
					pfig=(Figura*)pnode->ObjectPtr;
					nID=pfig->GetID();
					// Salta gli spazi.
					if(nID!=CL_SPAZIO&& 
						nID!=CL_PSEMIBREVE && nID!=CL_PMINIMA && nID!=CL_PSEMIMINIMA &&
						nID!=CL_PCROMA && nID!=CL_PSEMICROMA && nID!=CL_PBISCROMA &&
						nID!=CL_PSEMIBISCROMA && nID!=CL_PFUSA) 
					{
						pnota=(Nota*)pfig;
						pnota->Adjust(TRUE,pBattuta,current_layer,current_staff);
						dist= pnota->GetXgambo() - PNSignif->GetXgambo();
						y=(int)(y0+BSlope[currentView]*dist);
						lg=y - pnota->GetYgambo();
						// Se il gambo appena calcolato e' lungo meno di LGAMBOMIN/2 o e'
						// nella direzione opposta a quella desiderata, la scansione del
						// gruppo viene interrotta e va rifatta da capo, con un nuovo
						// valore di BSlope[currentView].
						if((pnota->GetLGambo()>0 && lg<(GetNotina()?LGAMBOBEAMMIN/2:LGAMBOBEAMMIN)) ||
							(pnota->GetLGambo()<0 && lg>-(GetNotina()?LGAMBOBEAMMIN/2:LGAMBOBEAMMIN)))
						{
							CorrectAdj=FALSE;       
							BSlope[currentView]=0;
						}
						else
						{ 
							if(pnota==GetFirstNota())
								GetBPoint1()=Point(pnota->GetXgambo(), y);
							if(pnota==GetLastNota())
								GetBPoint2()=Point(pnota->GetXgambo(), y);
							if(pnota->GetLGambo()>0)
							{
								if(beamPos==BEAM_12 || beamPos==BEAM_23)
								{
									if(GetNotina())
										pnota->SetLGambo(lg);
									else
										pnota->SetLGambo(lg);
								}
								else
								{
									if(GetNotina())
										pnota->SetLGambo(lg+(NBarre-1)*DELTAYBARRETTE1);
									else
										pnota->SetLGambo(lg+(NBarre-1)*DELTAYBARRETTE-(THICKBARRETTA-THICKBARRETTA/3));
								}
							}
							else if(pnota->GetLGambo()<0)
							{
								if(beamPos==BEAM_12 || beamPos==BEAM_23)
								{
									if(GetNotina())
										pnota->SetLGambo(lg);
									else
										pnota->SetLGambo(lg);
								}
								else
								{
									if(GetNotina())
										pnota->SetLGambo(lg-(NBarre-1)*DELTAYBARRETTE1);
									else
										pnota->SetLGambo(lg-(NBarre-1)*DELTAYBARRETTE+(THICKBARRETTA-THICKBARRETTA/3));
								}
							}
						}
						
						pnota->SetCoda(FALSE);
						
						pnota->AdjustF();
						pnota->SetVU2Up(pnota->GetVU2UpF());
						pnota->SetVU2Dwn(pnota->GetVU2DwnF());
						pnota->SetVU2Lft(pnota->GetVU2LftF());
						pnota->SetVU2Rgt(pnota->GetVU2RgtF());  
						
						pnota->Adjust(TRUE, pBattuta, current_layer,current_staff);
					}
					pnode=pnode->Next;
				}  
			}
       }
	 }
     else
     {
		 // CALCOLO DEI RETTANGOLI CHE CIRCONDANO IL GRUPPO.
		 
		//TEST
		//SetVU2UpF(0);
		//SetVU2DwnF(0);
		//SetVU2Up(0);
		//SetVU2Dwn(0);
		//
		 bool isfirst = true;
		 VUnit y_up,y_dn,y_upF,y_dnF;
		 pnode=first;
		 while(pnode!=NULL)
		 {           
			 pfig=(Figura*)pnode->ObjectPtr;
			 nID=pfig->GetID();
			 if(nID!=CL_SPAZIO && 
				 nID!=CL_PSEMIBREVE && nID!=CL_PMINIMA && nID!=CL_PSEMIMINIMA &&
				 nID!=CL_PCROMA && nID!=CL_PSEMICROMA && nID!=CL_PBISCROMA &&
				 nID!=CL_PSEMIBISCROMA && nID!=CL_PFUSA)
			 {
				 pnota=(Nota*)pfig;
				 pnota->Adjust(FALSE, pBattuta, current_layer,current_staff); 
				 //aggiunto l'argomento da Titti..verificare
				 // quando sistemo Adjust del gruppo. 
				 dy=pnota->GetAbsPos().y-GetAbsPos().y;

				 if(isfirst)
				 {
					 isfirst = false;
					 y_up	=pnota->GetVU2Up()	+dy;
					 y_dn	=pnota->GetVU2Dwn()	-dy;
					 y_upF	=pnota->GetVU2UpF()	+dy;
					 y_dnF	=pnota->GetVU2DwnF()-dy;
				 }
				 else
				 {
					 if(pnota->GetVU2UpF()+dy > y_upF)
						 y_upF	=pnota->GetVU2UpF()	+dy;
					 if(pnota->GetVU2DwnF()-dy > y_dnF)
						 y_dnF	=pnota->GetVU2DwnF()-dy;
					 if(pnota->GetVU2Up()+dy > y_up)
						 y_up	=pnota->GetVU2Up()	+dy;
					 if(pnota->GetVU2Dwn()-dy > y_dn)
						 y_dn	=pnota->GetVU2Dwn()	-dy;
				 }

				 //if(pnota->GetVU2UpF()+dy > GetVU2UpF())
					// SetVU2UpF(pnota->GetVU2UpF()+dy);
				 //if(pnota->GetVU2DwnF()-dy > GetVU2DwnF())
					// SetVU2DwnF(pnota->GetVU2DwnF()-dy);
				 //if(pnota->GetVU2Up()+dy > GetVU2Up())
					// SetVU2Up(pnota->GetVU2Up()+dy);
				 //if(pnota->GetVU2Dwn()-dy > GetVU2Dwn())
					// SetVU2Dwn(pnota->GetVU2Dwn()-dy);
			 }
			 pnode=pnode->Next;
		 }
		 
		 SetVU2UpF(y_upF);
		 SetVU2DwnF(y_dnF);
		 SetVU2Up(y_up);
		 SetVU2Dwn(y_dn);

		 // Sistema il rettangolo attorno al gruppo di note.
		 SetVU2LftF(((Figura*)first->ObjectPtr)->GetVU2LftF());   
		 SetVU2RgtF(((Figura*)last->ObjectPtr)->GetVU2RgtF() +
			 ((Figura*)last->ObjectPtr)->GetAbsPos().x -
			 ((Figura*)first->ObjectPtr)->GetAbsPos().x); 
		 if(GetNumNote()>0)
		 { 
			 dx = GetAbsPos().x - GetFirstNota()->GetAbsPos().x +
				 GetFirstNota()->GetVU2Lft();
			 if(GetVU2LftF()<dx)
				 SetVU2Lft(dx);  
			 else
				 SetVU2Lft(GetVU2LftF());
			 dx = GetLastNota()->GetAbsPos().x + GetLastNota()->GetVU2Rgt() -
				 GetAbsPos().x;
			 if(GetVU2RgtF()<dx)
				 SetVU2Rgt(dx);   
			 else
				 SetVU2Rgt(GetVU2RgtF());
		 }
		 else
		 {
			 // Se nel gruppo ci sono solo spazi.
			 SetVU2Lft(GetVU2LftF());
			 SetVU2Rgt(GetVU2RgtF());
		 }  
     }
	 
     return;
}  


UL GruppoNote::GetSp2NextFig()
{
	return GetSpTotale();
}

// Metodi per gestire il Beaming in multistaff	
/************************************************/

int GruppoNote::GetStaffInBeam()
{
	Node *pnode;
	Figura *pn;
	
	int s=0,n=0;
	
	bool staff[3]={false,false,false};
	
	pnode = first;
	
	while (pnode!=NULL)
	{  
		pn = (Figura*)pnode->ObjectPtr;
		
		if(pn->GetID()!=CL_SPAZIO)
		{ 
			s = pn->GetNStaff();
			if(!staff[s])
			{
				n++;
				staff[s]=true;
			}
		}
		pnode = pnode->Next;
	}  
	
	return n;
}


void GruppoNote::SetBeamPosition(BeamPos Bp)
{
	beamPos=Bp;
	if(Bp==BEAM_UP) SetGamboUp();
	else if(Bp==BEAM_DWN) SetGamboDwn();
}


void GruppoNote::DrawBarreMiddle()
{
	Node *pnode_prec,*pnode_corr;
	Figura *pfig_corr,*pfig_prec;
	
	Nota *pnota_prec,*pnota_corr;
 	  // Puntatori alla nota attuale ed alla sua precedente
 	  // Si guarda indietro, poiche e' la prima nota a dominare
 	  // la parte da cui si disegna il tratto in base al suo gambo,
 	  // infatti se sopra o sotto la linea di mezzo.
	
	short Direz;
	
	ClassID nID_prec=CL_NOTA;
	VUnit BThick, dYB;
	
	
	// Disegna le barrette solo se il metodo Adjust ha avuto buon esito.                   
	if(GetBPoint1()==Point(0,0) || GetBPoint1().x==GetBPoint2().x)
		return;            
	
	Direz = (GetLGambo()>0)? 1: -1;
	// GetBPoint2().y viene ricalcolato poiche' precedenti errori di approssimazione
	// potrebbero guastare il parallelismo della barra superiore.
	GetBPoint2().y = GetBPoint1().y + (VUnit)(BSlope[currentView]*(GetBPoint2().x-GetBPoint1().x));
	
	BThick = (GetNotina()?THICKBARRETTA1:THICKBARRETTA);
	dYB = (GetNotina()?DELTAYBARRETTE1:DELTAYBARRETTE);
	
	
	// DISEGNO PER LE NOTE.
	// Inizia dalla barra di mezzo che ora sta solo nel mezzo,
	// e quando si tratta di beam tra gli staff c'e' sempre.
	Scr->BigLine(GetBPoint1(),GetBPoint2(),BThick);
	
	pnode_prec = first;
	pnode_corr = GetNext(pnode_prec);
	while(pnode_prec!=NULL)
	{
		pfig_corr = (Figura*)pnode_corr->ObjectPtr;
		pfig_prec = (Figura*)pnode_prec->ObjectPtr;
		// Salta gli spazi
		if(nID_prec!=CL_SPAZIO)
		{
			pnota_corr = (Nota*)pfig_corr;
			pnota_prec = (Nota*)pfig_prec;
			
			if(nID_prec==CL_ACCORDO)
				nID_prec = ((Accordo*)pnota_prec)->GetNoteID();
			
			DrawTrattoMiddle(pnota_corr,pnota_prec,BThick,dYB);
		}
		pnode_corr=GetNext(pnode_corr);
		if(pnode_corr==NULL)
		{
			pnode_corr=pnode_prec;
			pnode_prec=GetPrev(pnode_prec);
			break;
		}
		pnode_prec=GetNext(pnode_prec);
		
		
	}
} 



void GruppoNote::DrawTrattoMiddle(Nota *pnota_corr,Nota *pnota_prec, VUnit BThick, VUnit dYB)
{
	short i,NBarre_corr=0,NBarre_prec=0,NBarre_Nota_prec=0,NBarre_Nota_corr=0;
	short comune,NMezzo_tratt_prec,NMezzo_tratt_corr;
	ClassID nID_corr,nID_prec; 
	
	
	//  Calcola il numero di barrette della nota corrente
	nID_corr=pnota_corr->GetID();
	if(nID_corr==CL_ACCORDO)
		nID_corr=((Accordo*)pnota_corr)->GetNoteID();
	NBarre_Nota_corr = nID_corr - CL_NCROMA + 1;
	
	//  Calcola il numero di barrette della nota precedente
	nID_prec=pnota_prec->GetID();
	if(nID_prec==CL_ACCORDO)
		nID_prec=((Accordo*)pnota_prec)->GetNoteID();
	NBarre_Nota_prec = nID_prec - CL_NCROMA + 1;
	
	
	//  si scalano di uno per via della barra centrale sempre presente
	NBarre_corr = NBarre_Nota_corr - 1; // numero di barrette scalate di quella centrale
	NBarre_prec = NBarre_Nota_prec - 1; // numero di barrette scalate di quella centrale
	
	//  barre che si possono unire delle due note, naturalmente escludendo quella centrale
	comune = (NBarre_corr > NBarre_prec)? NBarre_prec:NBarre_corr;
	
	NMezzo_tratt_prec = NBarre_Nota_prec - comune - 1;
	
	
	//  caso particolare nel caso la prima delle due sia una CROMA
	if(NBarre_prec == 0 && (NBarre_corr >=0 && (GetLastNota()!=pnota_corr)))
	{
		return;
	}
	//  caso particolare che la penultima sia una CROMA e l'ultima sia di durata minore
	else if(NBarre_prec == 0 && (NBarre_corr >=0 && (GetLastNota()==pnota_corr)))
	{
		NMezzo_tratt_corr = NBarre_Nota_corr - comune - 1;
		if((pnota_corr->GetNStaff() == 0 && beamPos == BEAM_12) || (pnota_corr->GetNStaff() <= 1 && beamPos == BEAM_23) )
		{
			for(i=0;i<NMezzo_tratt_corr;i++)
			{
				Tratto[currentView][i].P1=Point(0,0);
				Tratto[currentView][i].PRef = GetBPoint2()+Point(0, i*dYB);
				// DXGambo deve essere assegnato in base alla direzione ed alla
				// dimensione della nota.
				Tratto[currentView][i].P2.x = pnota_corr->GetXgambo();
				Tratto[currentView][i].P2.y = Tratto[currentView][i].PRef.y +
					VUnit((BSlope[currentView]) * (Tratto[currentView][i].P2.x - Tratto[currentView][i].PRef.x));
				// Se questo e` l'inizio di un tratto, memorizza il punto iniziale.
				if(Tratto[currentView][i].P1==Point(0,0))
					Tratto[currentView][i].P1 = Tratto[currentView][i].P2;
			}
			for(i=0;i<NMezzo_tratt_corr;i++)
			{
				
				
				// Mezzo tratto verso sinistra, al di sopra della midline
				Tratto[currentView][i].P2 = Tratto[currentView][i].P1;
				Tratto[currentView][i].P1.x = Tratto[currentView][i].P2.x - TRATTOMEZZABARRA;
				Tratto[currentView][i].P1.y = Tratto[currentView][i].P2.y + (VUnit)((-BSlope[currentView])*TRATTOMEZZABARRA);
				Scr->BigLine(Tratto[currentView][i].P1+Point(0,BThick+dYB/2), Tratto[currentView][i].P2+Point(0,BThick+dYB/2),BThick);
			}
		}
		else if((pnota_corr->GetNStaff() > 0 && (beamPos == BEAM_12) || (pnota_corr->GetNStaff() > 1 && beamPos == BEAM_23)))
		{
			for(i=0;i<NMezzo_tratt_corr;i++)
			{
				Tratto[currentView][i].P1=Point(0,0);
				Tratto[currentView][i].PRef = GetBPoint2()-Point(0, i*dYB);
				// DXGambo deve essere assegnato in base alla direzione ed alla
				// dimensione della nota.
				Tratto[currentView][i].P2.x = pnota_corr->GetXgambo();
				Tratto[currentView][i].P2.y = Tratto[currentView][i].PRef.y -
					VUnit((BSlope[currentView]) * (Tratto[currentView][i].P2.x - Tratto[currentView][i].PRef.x));
				// Se questo e` l'inizio di un tratto, memorizza il punto iniziale.
				if(Tratto[currentView][i].P1==Point(0,0))
					Tratto[currentView][i].P1 = Tratto[currentView][i].P2;
			}
			for(i=0;i<NMezzo_tratt_corr;i++)
			{
				// DXGambo deve essere assegnato in base alla direzione ed alla
				// dimensione della nota.
				Tratto[currentView][i].P2.x = pnota_corr->GetXgambo();
				Tratto[currentView][i].P2.y = Tratto[currentView][i].PRef.y -
					VUnit((BSlope[currentView]) * (Tratto[currentView][i].P2.x - Tratto[currentView][i].PRef.x));
				
				// Mezzo tratto verso sinistra, al di sotto delle midline
				Tratto[currentView][i].P2 = Tratto[currentView][i].P1;
				Tratto[currentView][i].P1.x = Tratto[currentView][i].P2.x - TRATTOMEZZABARRA;
				Tratto[currentView][i].P1.y = Tratto[currentView][i].P2.y - (VUnit)((BSlope[currentView])*TRATTOMEZZABARRA);
				Scr->BigLine(Tratto[currentView][i].P1-Point(0,BThick+dYB/2), Tratto[currentView][i].P2-Point(0,BThick+dYB/2),BThick);
			}
		}
	} //  caso in cui la prima nota sia isolata da una CROMA che segue
	else if(((NBarre_prec >= 0 && (GetFirstNota() == pnota_prec)) && NBarre_corr == 0) || (NBarre_prec >= 0 && NBarre_corr == 0 ))
	{
		NMezzo_tratt_prec = NBarre_Nota_prec - comune - 1;
		if((pnota_prec->GetNStaff() == 0 && beamPos == BEAM_12) || (pnota_prec->GetNStaff() <= 1 && beamPos == BEAM_23) )
		{
			for(i=0;i<NMezzo_tratt_prec;i++)
			{
				Tratto[currentView][i].P1=Point(0,0);
				Tratto[currentView][i].PRef = GetBPoint1()+Point(0, i*dYB);
				// DXGambo deve essere assegnato in base alla direzione ed alla
				// dimensione della nota.
				Tratto[currentView][i].P2.x = pnota_prec->GetXgambo();
				Tratto[currentView][i].P2.y = Tratto[currentView][i].PRef.y +
					VUnit((-BSlope[currentView]) * (Tratto[currentView][i].P2.x - Tratto[currentView][i].PRef.x));
				// Se questo e` l'inizio di un tratto, memorizza il punto iniziale.
				if(Tratto[currentView][i].P1==Point(0,0))
					Tratto[currentView][i].P1 = Tratto[currentView][i].P2;
			}
			for(i=0;i<NMezzo_tratt_prec;i++)
			{
				
				
				// Mezzo tratto verso destra, al di sopra della midline
				Tratto[currentView][i].P2 = Tratto[currentView][i].P1;
				Tratto[currentView][i].P1.x = Tratto[currentView][i].P2.x + TRATTOMEZZABARRA;
				Tratto[currentView][i].P1.y = Tratto[currentView][i].P2.y + (VUnit)((BSlope[currentView])*TRATTOMEZZABARRA);
				Scr->BigLine(Tratto[currentView][i].P1+Point(0,BThick+dYB/2), Tratto[currentView][i].P2+Point(0,BThick+dYB/2),BThick);
			}
		}
		else if((pnota_prec->GetNStaff() > 0 && (beamPos == BEAM_12) || (pnota_prec->GetNStaff() > 1 && beamPos == BEAM_23)))
		{
			for(i=0;i<NMezzo_tratt_prec;i++)
			{
				Tratto[currentView][i].P1=Point(0,0);
				Tratto[currentView][i].PRef = GetBPoint1()-Point(0, i*dYB);
				// DXGambo deve essere assegnato in base alla direzione ed alla
				// dimensione della nota.
				Tratto[currentView][i].P2.x = pnota_prec->GetXgambo();
				Tratto[currentView][i].P2.y = Tratto[currentView][i].PRef.y -
					VUnit(BSlope[currentView] * (Tratto[currentView][i].P2.x - Tratto[currentView][i].PRef.x));
				// Se questo e` l'inizio di un tratto, memorizza il punto iniziale.
				if(Tratto[currentView][i].P1==Point(0,0))
					Tratto[currentView][i].P1 = Tratto[currentView][i].P2;
			}
			for(i=0;i<NMezzo_tratt_prec;i++)
			{
				// DXGambo deve essere assegnato in base alla direzione ed alla
				// dimensione della nota.
				Tratto[currentView][i].P2.x = pnota_prec->GetXgambo();
				Tratto[currentView][i].P2.y = Tratto[currentView][i].PRef.y -
					VUnit(BSlope[currentView] * (Tratto[currentView][i].P2.x - Tratto[currentView][i].PRef.x));
				
				// Mezzo tratto verso destra, al di sotto delle midline
				Tratto[currentView][i].P2 = Tratto[currentView][i].P1;
				Tratto[currentView][i].P1.x = Tratto[currentView][i].P2.x + TRATTOMEZZABARRA;
				Tratto[currentView][i].P1.y = Tratto[currentView][i].P2.y - (VUnit)((-BSlope[currentView])*TRATTOMEZZABARRA);
				Scr->BigLine(Tratto[currentView][i].P1-Point(0,BThick+dYB/2), Tratto[currentView][i].P2-Point(0,BThick+dYB/2),BThick);
			}
		}
	}
	
	//  disegna le barre prendendo come riferimento (nota dominante)
	// la prima nota che ha una barra in piu rispetto alla centrale e 
	// come verso, quello dalla parte dello staff di appartenenza.
	else 
	{
		NMezzo_tratt_prec = NBarre_Nota_prec - comune - 1;
		NMezzo_tratt_corr = NBarre_Nota_corr - comune - 1;
		if((pnota_prec->GetNStaff() == 0 && beamPos == BEAM_12) || (pnota_prec->GetNStaff() <= 1 && beamPos == BEAM_23) )
		{
			for(i=0;i<=comune;i++)
			{
				Tratto[currentView][i].P1=Point(0,0);
				Tratto[currentView][i].PRef.x = pnota_prec->GetXgambo();
				Tratto[currentView][i].PRef.y = pnota_prec->GetYgambo()+pnota_prec->GetLGambo();
				Tratto[currentView][i].PRef = Tratto[currentView][i].PRef - Point(0,i*dYB);
				// DXGambo deve essere assegnato in base alla direzione ed alla
				// dimensione della nota.
				if(nID_corr==CL_NCROMA)
				{
					Tratto[currentView][i].P2.x = pnota_prec->GetXgambo();
					Tratto[currentView][i].P2.y = pnota_prec->GetYgambo()+pnota_prec->GetLGambo();
				}
				else
				{
					Tratto[currentView][i].P2.x = pnota_corr->GetXgambo();
					Tratto[currentView][i].P2.y = Tratto[currentView][i].PRef.y -
						VUnit((-BSlope[currentView]) * (Tratto[currentView][i].P2.x - Tratto[currentView][i].PRef.x));
				}
				// Se questo e` l'inizio di un tratto, memorizza il punto iniziale.
				//if(Tratto[currentView][i].P1==Point(0,0))
				//	Tratto[currentView][i].P1 = Tratto[currentView][i].P2;
				Scr->BigLine(Tratto[currentView][i].PRef+Point(0,BThick+dYB/2), Tratto[currentView][i].P2+Point(0,BThick+dYB/2),BThick);
				Scr->Line(Tratto[currentView][i-1].PRef,Tratto[currentView][i-1].PRef+Point(0,BThick+dYB/2),SOLID_LINE); // per disegnare i prolongamenti dei gambi
				Scr->Line(Tratto[currentView][i-1].P2,Tratto[currentView][i-1].P2+Point(0,BThick+dYB),SOLID_LINE); // per disegnare i prolongamneti dei gambi
				
			}
			
		}
		else if((pnota_prec->GetNStaff() > 0 && (beamPos == BEAM_12) || (pnota_prec->GetNStaff() > 1 && beamPos == BEAM_23)))
		{
			for(i=0;i<=comune;i++)
			{
				Tratto[currentView][i].P1=Point(0,0);
				Tratto[currentView][i].PRef.x = pnota_prec->GetXgambo();
				Tratto[currentView][i].PRef.y = pnota_prec->GetYgambo()+pnota_prec->GetLGambo();
				Tratto[currentView][i].PRef = Tratto[currentView][i].PRef - Point(0,i*dYB);
				// DXGambo deve essere assegnato in base alla direzione ed alla
				// dimensione della nota.
				if(nID_corr==CL_NCROMA)
				{
					Tratto[currentView][i].P2.x = pnota_prec->GetXgambo();
					Tratto[currentView][i].P2.y = pnota_prec->GetYgambo()+pnota_prec->GetLGambo();
				}
				else
				{
					Tratto[currentView][i].P2.x = pnota_corr->GetXgambo();
					Tratto[currentView][i].P2.y = Tratto[currentView][i].PRef.y -
						VUnit((-BSlope[currentView]) * (Tratto[currentView][i].P2.x - Tratto[currentView][i].PRef.x));
				}
				// Se questo e` l'inizio di un tratto, memorizza il punto iniziale.
				if(Tratto[currentView][i].P1==Point(0,0))
					Tratto[currentView][i].P1 = Tratto[currentView][i].P2;
				Scr->BigLine(Tratto[currentView][i].PRef+Point(0,BThick+dYB/2), Tratto[currentView][i].P2+Point(0,BThick+dYB/2),BThick);
			}
			
		}
	}
}

/*  end metodi per beaming in multistaff
*************************************************************************/

void GruppoNote::GetMinMaxStaff(short& minStaff, short& maxStaff)
{
	ListaFigure::GetMinMaxStaff(minStaff,maxStaff);
}

