/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: ncroma.cpp,v 1.1 2005/07/25 16:30:40 bellinpo Exp $ */


#include "fig/ncroma.hpp" 
#include "drw/gscreen.hpp"    

NCroma::NCroma() : Nota(0.125)
{
	Init();
	//durata = 0.125;
}



void NCroma::Init()
{
	Nota::Init();
	ID=CL_NCROMA;
	Acciaccatura=FALSE;
}


void NCroma::SetAcciaccatura(Bool b)
{
	if(b)
	{
		SetNotina(TRUE);
	}
	Acciaccatura=b;
}


Bool NCroma::GetAcciaccatura()
{
	return Acciaccatura;
}


void NCroma::AdjustF()
{ 
	VUnit lg;
	
	InitRectF();
	lg=ABS(GetLGambo());
	if(GetNotina())
	{
		if(GetCoda()) 
		{ 
			if(GetLGambo()>0)
				SetVU2RgtF(VU2RGTFCODA1);
		}
		else 
		{
			lg+=INCRNOTASENZACODA;
			SetVU2RgtF(VU2RGTNOTA1);
		}
	}
	else
	{
		if(GetCoda())  
		{                          
			if(GetLGambo()>0)
				SetVU2RgtF(VU2RGTFCODA);
		} 
		else 
		{
			lg+=INCRNOTASENZACODA;
			SetVU2RgtF(VU2RGTNOTA);
		} 
	}
	if(GetLGambo()>0)
		SetVU2UpF(lg);
	if(GetLGambo()<0)
		SetVU2DwnF(lg);
}


void NCroma::DrawCode()
{
	unsigned char code[2]={0,0};
	VUnit xcoda,ycoda,xgambo,ygambo;
	VUnit y2;
	
	// Disegno della coda superiore.
	if(GetLGambo()>0)
	{        
		xgambo=GetXgambo();
		ygambo=GetYgambo();
		code[0]=(GetNotina()?CODCODAUP1:COD1CODAUP);
        xcoda=xgambo+(GetNotina()?DELTAXCODAUP1:DELTAXCODAUP);
        ycoda=ygambo+GetLGambo()+(GetNotina()?DELTAYCODAUP1:DELTAYCODAUP);
		Scr->TextTo(xcoda,ycoda);
        Scr->OutText((unsigned char *)code);
        if(GetAcciaccatura())
        {
			// Disegna il taglio obliquo.
			
			y2=ygambo+GetLGambo()-LGAMBOMIN/2;
			Scr->MoveTo(xgambo-DELTAXACCIACC, y2-DELTAYACCIACC);
			Scr->LineTo(GetAbsPos().x+VU2RGTFCODA1, y2+DELTAYACCIACC,SOLID_LINE);
        }
	}
	// Disegno della coda inferiore.
	if(GetLGambo()<0)
	{
		xgambo=GetXgambo();
		ygambo=GetYgambo();
        code[0]=(GetNotina()?CODCODADWN1:COD1CODADWN);
        xcoda=xgambo+(GetNotina()?DELTAXCODADWN1:DELTAXCODADWN);
        ycoda=ygambo+GetLGambo()-(GetNotina()?DELTAYCODADWN1:DELTAYCODADWN);
        Scr->TextTo(xcoda,ycoda);
        Scr->OutText((unsigned char *)code);
        if(GetAcciaccatura())
        {
			// Disegna il taglio obliquo.
			y2=ygambo+GetLGambo()+LGAMBOMIN/2;
			Scr->MoveTo(xgambo-DELTAXACCIACC,
				y2-DELTAYACCIACC);
			Scr->LineTo(xgambo+DELTAXACCIACC,
				y2+DELTAYACCIACC,SOLID_LINE);
        }
	}
}

void NCroma::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	if(HastobeDraw())
	{
		Nota::Draw();
		if(Coda)
		{
			DrawCode();
		}
	}

	Scr->SetColor(oldcol);
};             


void NCroma::DrawForAccordo()
{ 
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	Nota::DrawForAccordo();     
	if(Coda)
		DrawCode();

	Scr->SetColor(oldcol);
};              


CmdResult NCroma::SymCommand(SymCmd& SCmd, Battuta* pBattuta)
{ 
	
	switch(SCmd.CommandId)
	{
    case CMD_NOTA_ACCIACCATURA:
		// Controllo sulla coda per impedire che questo comando venga
		// eseguito sulle note dentro i gruppi.
		if(GetAcciaccatura() || !GetCoda())
			return(CMD_FAIL);   
		SetAcciaccatura(TRUE);
		SetStatus(GRACED);
		// Adatta il gambo.    
		if(GetLGambo()>0)
			SetGamboUp();    
		if(GetLGambo()<0)
			SetGamboDwn();
		return(CMD_OK);
		break;
    case CMD_NOTA_NOTINA2NOTA:
		SetAcciaccatura(FALSE);
		return(Nota::SymCommand(SCmd, pBattuta));
		break;
    default:
		return(Nota::SymCommand(SCmd, pBattuta));
		break;
	}
	return(CMD_UNKNOWN);
}
