/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: nfusa.cpp,v 1.1 2005/07/25 16:30:40 bellinpo Exp $ */


#include "fig/nfusa.hpp"
#include "drw/gscreen.hpp"


NFusa::NFusa() : Nota(0.0078125)
{
	Init();
	//durata = 0.0078125;
}



void NFusa::Init()
{
	Nota::Init();
	ID=CL_NFUSA;
}


void NFusa::AdjustF()
{                   
	VUnit lg; 
	
	InitRectF();
	lg=ABS(GetLGambo());
	if(GetNotina())
	{
		if(GetCoda())  
		{                          
			lg+=4*DELTAYCODE1;
			if(GetLGambo()>0)
				SetVU2RgtF(VU2RGTFCODA1);
		} 
		else 
		{
			lg+=INCRNOTASENZACODA;
			SetVU2RgtF(VU2RGTNOTA1);
		}
	}
	else
	{
		if(GetCoda())  
		{                          
			lg+=4*DELTAYCODE;
			if(GetLGambo()>0)
				SetVU2RgtF(VU2RGTFCODA);
		} 
		else 
		{
			lg+=INCRNOTASENZACODA;
			SetVU2RgtF(VU2RGTNOTA);
		}
	}
	if(GetLGambo()>0)
		SetVU2UpF(lg);
	if(GetLGambo()<0)
		SetVU2DwnF(lg);
}


void NFusa::DrawCode()
{ 
	unsigned char code[2]={0,0};
	VUnit xcoda,ycoda,xgambo,ygambo;
	int i;
	
	// Disegno delle code superiori.
	if(GetLGambo()>0)
	{
		xgambo=GetXgambo();
		ygambo=GetYgambo();
		code[0]=(GetNotina()?CODCODAUP1:CODCODAUP);
        xcoda=xgambo+(GetNotina()?DELTAXCODAUP1:DELTAXCODAUP);
        ycoda=ygambo+GetLGambo()+(GetNotina()?DELTAYCODAUP1:DELTAYCODAUP);
        for(i=0;i<5;i++)
        {
			if(i==3)
				code[0]=(GetNotina()?CODCODAUP1:COD1CODAUP);
			if(i==4)
				code[0]=(GetNotina()?CODCODAUP1:COD0CODAUP);
			Scr->TextTo(xcoda,ycoda);
			Scr->OutText((unsigned char *)code);
			if(GetNotina())
				ycoda+=DELTAYCODE1;
			else
				ycoda-=DELTAYCODE;
        }
	}
	if(GetLGambo()<0)
	{
        // Disegno delle code inferiori.
		xgambo=GetXgambo();
		ygambo=GetYgambo();
        code[0]=(GetNotina()?CODCODADWN1:CODCODADWN);
        xcoda=xgambo+(GetNotina()?DELTAXCODADWN1:DELTAXCODADWN);
        ycoda=ygambo+GetLGambo()+(GetNotina()?DELTAYCODADWN1:DELTAYCODADWN);
        for(i=0;i<5;i++)
        {
			if(i==3)
				code[0]=(GetNotina()?CODCODADWN1:COD1CODADWN);
			if(i==4)
				code[0]=(GetNotina()?CODCODADWN1:COD0CODADWN);
			Scr->TextTo(xcoda,ycoda);
			Scr->OutText((unsigned char *)code);
			if(GetNotina())
				ycoda-=DELTAYCODE1;
			else
				ycoda+=DELTAYCODE;
        }
	}
}

void NFusa::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	if(HastobeDraw())
	{
		unsigned char code[2]={0,0};
		
		
		Nota::Draw();
		if(Coda)
			DrawCode();  
		
	}

	Scr->SetColor(oldcol);
};           

void NFusa::DrawForAccordo()
{ 
	unsigned char code[2]={0,0};
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	Nota::Draw();
	if(Coda)
		DrawCode();  

	Scr->SetColor(oldcol);	
};
