/* ISO_HEADER_START */

/*********************************************************************

 This software module was originally developed by
 
  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)
  
   in the course of development of the MPEG-4 standard.
   This software module is an implementation of a part of one or more
   MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
   of the MPEG-4 standard free license to this software module or
   modifications thereof for use in hardware or software products
   claiming conformance to MPEG-4. Those intending to use this software
   module in hardware or software products are advised that its use may
   infringe existing patents.  The original developer of this software
   module and his/her company, the subsequent editors and their
   companies, and ISO/IEC have no liability for use of this software
   module or modifications thereof in an implementation.  Copyright is
   not released for non MPEG-4 conforming products. The DISIT-DSI and 
   EXITECH retains full right to use the code for its own purpose,
   assign or donate the code to a third party and to inhibit third
   parties from using the code for non MPEG-4 conforming products.  This
   copyright notice must be included in all copies or derivative
   works. Copyright (c) 2005.
   
***********************************************************************/

/* ISO_HEADER_END */

/* $Id: nota.cpp,v 1.1 2005/07/25 16:30:41 bellinpo Exp $ */


extern int DebugNormalMilla;
extern int DebugNormalMillaPos;
#include "wx/wx.h"
extern wxString msgdebug;
extern wxString msgtoken;
#include "gui/dialogs.hpp"
#include "liooext.hpp"
#include "fig/nota.hpp"
#include "mil/priorita.hpp"

#include "abb/ginferio.hpp"
#include "abb/gruppett.hpp"
#include "abb/gsuperio.hpp"
#include "abb/minferio.hpp"
#include "abb/mordente.hpp"
#include "abb/msuperio.hpp"
#include "abb/trillo.hpp"
#include "abb/tremolo.hpp"
#include "abb/glissato.hpp"
#include "abb/gslash.hpp"
#include "abb/gup.hpp"

#include "kor/slidingup.hpp"
#include "kor/slidingdown.hpp"
#include "kor/slidingdup.hpp"
#include "kor/slidingdownup.hpp"
#include "kor/nonghyninc.hpp"
#include "kor/nonghyndec.hpp"
#include "kor/nonghynarrow.hpp"
#include "kor/nonghynwide.hpp"

#include "ind/consord.hpp"
#include "ind/viasord.hpp"
#include "ind/sordina.hpp"
#include "ind/armonici.hpp"

#include "int/modifott.hpp"

#include "str/strument.hpp"
#include "str/violino.hpp"
#include "str/arco.hpp"
#include "str/arcogiu.hpp"
#include "str/arcosu.hpp"
#include "str/pizzicat.hpp"
#include "str/ponticel.hpp"
#include "str/punta.hpp"
#include "str/tallone.hpp"
#include "str/tastiera.hpp"

#include "mil/m.tab.h"
#include "mil/lgiust.h"


#define VUDIMNOTA 7 


Nota::Nota()
{
	Init();
} 

Nota::Nota(double dur) : Figura(dur)
{
	Init();
} 
///////////////////////      
Nota::~Nota()
{
	Free();
}  


void Nota::Init()
{
	
	Figura::Init();
	ID=CL_NOTA;  
	Figura::SetNotina(FALSE);
	SetNoGambo();
	SetCoda(FALSE);
	SetPAlterazComposta(NULL);
	SetPAbbellimento(NULL);     
	SetPKoreanOrnament(NULL);
	SetPDiteggiato(NULL);
	SetPEspressComposta(NULL);
	SetPSordina(NULL);
	SetPArmonici(NULL);
	SetPSuddivisione(NULL);
	SetTie(FALSE); 
	SetTraceMilla(FALSE);
	IsValidPitch = FALSE;
}


void Nota::InitRectF()
{
	if(HeadType==HEAD_ALPHANUM || HeadType==HEAD_ALPHANUM_SQUARE || HeadType==HEAD_ALPHANUM_REVERSE)
	{
		Scr->SetFont(nTextFont);
		SetVU2UpF(TEXTUP(HeadCode));
		SetVU2DwnF(TEXTDW(HeadCode));
		SetVU2LftF(TEXTLF(HeadCode));
		SetVU2RgtF(TEXTRG(HeadCode));  
		Scr->SetFont(nMusicFont);
	}
	else if(HeadCode[0])
	{
		// Imposta il rettangolo usando le informazioni del font...
		SetVU2UpF(Scr->Get_VU2UP(HeadCode[0]));
		SetVU2DwnF(Scr->Get_VU2DWN(HeadCode[0]));
		SetVU2LftF(Scr->Get_VU2LF(HeadCode[0]));
		SetVU2RgtF(Scr->Get_VU2RG(HeadCode[0]));
	}
	else if(GetNotina())
	{
		// Imposta il rettangolo che circonda la notina.
		SetVU2UpF(VU2UPNOTA1);
		SetVU2DwnF(VU2DWNNOTA1);
		SetVU2LftF(VU2LFTNOTA1);
		SetVU2RgtF(VU2RGTNOTA1); 
	}
	else
	{
		// Imposta il rettangolo che circonda la nota.
		SetVU2UpF(VU2UPNOTA);
		SetVU2DwnF(VU2DWNNOTA);
		SetVU2LftF(VU2LFTNOTA);
		SetVU2RgtF(VU2RGTNOTA);
	}
}

Bool Nota::IsANote()
{
	return TRUE;
}  

NoteHead_ID Nota::GetHeadType()
{
	return HeadType;
}

void Nota::SetHeadType(NoteHead_ID headtype)
{
	HeadType=headtype;
}

void Nota::SetDxGambo(int dx)
{
	DxGambo=dx;
}

void Nota::SetDyGambo(int dy)
{
	DyGambo=dy;
}

void Nota::SetHeadCode(unsigned char *code)
{
	strcpy((char*)(HeadCode),(const char*)code);
}

float Nota::GetDurata()
{
	return Figura::GetDurata();  
}   

void Nota::SetLGambo(VUnit lg)
{
	LGambo=lg;
}           


void Nota::SetGamboUp()
{
	if(GetNotina())
	{
		SetLGambo(LGAMBODEF/2);
	}
	else
	{
		SetLGambo(LGAMBODEF);
	}
}


void Nota::SetNoGambo()
{
	LGambo=0;
} 


void Nota::SetGamboDwn()
{
	if(GetNotina())
	{
		SetLGambo(-LGAMBODEF/2);
	}
	else
	{
		SetLGambo(-LGAMBODEF);
	}
}


VUnit Nota::GetLGambo()
{
	return LGambo;
}


void Nota::SetCoda(Bool coda)
{
	Coda=coda;
}

Bool Nota::GetCoda()
{
	return Coda;
}



void Nota::SetPAlterazComposta(AlterazComposta *palt)
{
	PAlterazComposta=palt;
}


AlterazComposta* Nota::GetPAlterazComposta()
{
	return PAlterazComposta;
}

void Nota::SetPAbbellimento(Abbellimento *pabb)
{ 
	PAbbellimento=pabb;
}


Abbellimento* Nota::GetPAbbellimento()
{
	return PAbbellimento;
} 

void Nota::SetPKoreanOrnament(KoreanOrnament *pkor)
{
	PKoreanOrnament = pkor;
}

KoreanOrnament* Nota::GetPKoreanOrnament()
{
	return PKoreanOrnament;
}

void Nota::SetPDiteggiato(Diteggiato *pd)
{ 
	PDiteggiato=pd;
}


Diteggiato* Nota::GetPDiteggiato()
{
	return PDiteggiato;
}

void Nota::SetPSordina(Sordina *ps)
{
	PSordina=ps;
}

Sordina* Nota::GetPSordina()
{
	return PSordina;
}

void Nota::SetPArmonici(Armonici *pa)
{
	PArmonici=pa;
}

Armonici* Nota::GetPArmonici()
{
	return PArmonici;
}


void Nota::SetPEspressComposta(EspressComposta *pe)
{
	PEspressComposta=pe;
}


EspressComposta* Nota::GetPEspressComposta()
{
	return PEspressComposta;
} 

void Nota::SetPSuddivisione(Suddivisione *ps)
{
	PSuddivisione = ps;
}

Suddivisione *Nota::GetPSuddivisione()
{
	return PSuddivisione;
}

void Nota::MoveX(VUnit dx)
{
	GetAbsPos().x+=dx;
}





void Nota::Adjust(Bool primaleg, Battuta *pBattuta, short layer, short current_staff)
{                         
	VUnit YLinea1=0,YLinea5=0;
    // Altezza fisica delle linee piu' bassa e piu' alta del pentagramma.
	int NMax;
    // Numero di simboli contenuti nella lista delle priorit
	ClassID symbol;
    // Identificatore dei simboli contenuti nela lista delle priorit
	int j;
	int LegPos;
	Bool gamboupdown;
	
	
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		msgdebug.sprintf("");
	}
	
	if(GetLGambo()>0)
		gamboupdown = TRUE;
	else  if(GetLGambo()<0)
		gamboupdown = FALSE;
	else if(GetLGambo()==0)
	{
		if (GetAltezza()>=4)
			gamboupdown=FALSE;
		else
			gamboupdown=TRUE;
	}
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{   
		msgdebug.sprintf("");
		
	}
	
    if(HeadType!=HEAD_ALPHANUM && HeadType!=HEAD_ALPHANUM_SQUARE && HeadType!=HEAD_ALPHANUM_REVERSE)
    {
		HeadCode[0]=posizionam.PosRuleHead(this, this, pBattuta, gamboupdown, layer);
		HeadCode[1]='\0';
    }
    posizionam.PosRuleStartGambo(this, this, pBattuta, gamboupdown, layer);
	
	
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		if (this->GetTraceMilla())
		{  
			posizionam.TestTokenCond(this, this, pBattuta, gamboupdown, layer);
			
			WDebugMilla(); 
			
			this->SetTraceMilla(FALSE);
		}
	}
	
	AdjustAlterazComposta();
	
	if(pBattuta!=NULL)
	{
		int nstaff=GetNStaff();
		if(nstaff==-1) //accordo multistaff
		{
			short minStaff,maxStaff;
			GetMinMaxStaff(minStaff,maxStaff);
			
			VUnit YLineaBaseMax=pBattuta->GetIntest(maxStaff)->GetAbsPos().y;
			YLinea1=-pBattuta->GetParentScore()->GetBaseVU2Dwn(maxStaff)+YLineaBaseMax;
			
			VUnit YLineaBaseMin=pBattuta->GetIntest(minStaff)->GetAbsPos().y;
			YLinea5=pBattuta->GetParentScore()->GetBaseVU2Up(minStaff)+YLineaBaseMin;
		}
		else
		{
			VUnit YLineaBase=pBattuta->GetIntest(nstaff)->GetAbsPos().y;
			YLinea1=-pBattuta->GetParentScore()->GetBaseVU2Dwn(nstaff)+YLineaBase;
			YLinea5=pBattuta->GetParentScore()->GetBaseVU2Up(nstaff)+YLineaBase;
		}
	}
	Espressione *PEspressione; 
	int da, a, IDsym; 
	PITEMPOS PosRule;
	Bool onspace=TRUE, onstaff=FALSE;
	
	Point point;
	
	NMax = priorita.GetPriorityUsage();
	priorita.SetLegPos();
	LegPos = priorita.GetLegPos();
	
	
	if(primaleg)
	{ 
		da = 0;
		a = LegPos;
		SetVU2Up(GetVU2UpF());
		SetVU2Dwn(GetVU2DwnF());
		
		// per debug
		int provaleft=GetVU2LftF();
		//
		
		SetVU2Lft(GetVU2LftF());
		SetVU2Rgt(GetVU2RgtF());
	}
	else 
	{
		da = LegPos+1;
		a = NMax;
	}      
	for(j=da; j<a; j++)
	{   
		symbol = (ClassID)priorita.GetSymbolPri(j);
		
		if(symbol>=2000)
		{
			IDsym=symbol-2000;
			symbol=CL_ESPGENERIC;
		}	
		switch (symbol)
		{
		case CL_STACCATO:
		case CL_TENUTO:
		case CL_SFORZATO: 
		case CL_ACCENTO:
		case CL_MARTELLATO:
		case CL_MARTDOLCE:
		case CL_PUNTOALLUNGATO:
		case CL_ESPGENERIC:
			if(GetPEspressComposta()!=NULL)
			{
				PEspressione = PEspressComposta->GetPEspressione(symbol,IDsym); 
				if(PEspressione!=NULL) 
				{
					
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						msgdebug.sprintf("");
					}
					
					
					
					
					PosRule = posizionam.PosRulePunt((DrawObject*)PEspressione, this, pBattuta,
						gamboupdown, layer, 0,0);	
					
					
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (PEspressione->GetTraceMilla())
						{  
							posizionam.TestTokenCond(this, (DrawObject*)PEspressione, pBattuta, gamboupdown, layer);
							
							WDebugMilla(); 
							
							PEspressione->SetTraceMilla(FALSE);
						}
					}
					
					
					
					
					if(PosRule!=NULL)
					{
						if(PosRule->staff==ONSTAFF) 
						{
							onstaff=TRUE;
						}
						else if(PosRule->staff==OUTSIDESTAFF)
						{
							onstaff=FALSE;
						}	  	 
						if(PosRule->cat==ONSPACE) onspace=TRUE;
						
						switch (PosRule->Symbol)
						{
						case STAC:
						case SFOR: 
						case MARTF:
						case MART:
						case ACCE:
						case TEN:
						case TASTIERA:
						case PONTICELLO:
						case PUNTA:
						case TALLONE:
						case PIZZICATO:
						case ARCO:
						case CORDA:
							point = PosRule->GetPoint(GetLGambo());
							break;
						default:
							if (PosRule->Symbol >=1000)
								point = PosRule->GetPoint(GetLGambo());
							break;
						}
					}   										     
					AdjustEspressComposta(YLinea1,YLinea5,onspace,onstaff,point,PEspressione);
					
					
				}
			}
			
			break;
		case CL_PUNTA:   
		case CL_TALLONE:
			if(GetPStrumento()!=NULL && GetPStrumento()->GetID()==CL_ARCO)
			{ 
				if((((Arco*)GetPStrumento())->GetPParteArco())!=NULL &&  
					(((Arco*)GetPStrumento())->GetPParteArco())->GetID()==symbol)
				{
					ParteArco *parc;
					parc = ((Arco*)GetPStrumento())->GetPParteArco();
					if (GetLGambo()>0) gamboupdown = TRUE;
					else if (GetLGambo()<0) gamboupdown = FALSE;  
					else if(GetLGambo()==0)
					{ 
						if(GetAltezza()>=4) gamboupdown = FALSE;
						else gamboupdown = TRUE;
					}  
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						msgdebug.sprintf("");
					}
					
					
					
					
					point = posizionam.PosRuleP((DrawObject*)parc, 
						this, pBattuta,gamboupdown, layer);
					
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (parc->GetTraceMilla())
						{  
							posizionam.TestTokenCond(this, (DrawObject*)parc, pBattuta, gamboupdown, layer);
							
							WDebugMilla(); 
							
							parc->SetTraceMilla(FALSE);
						}
					}
					
					AdjustParteArco(YLinea1,YLinea5,point);
					
					
				}
			}  
			
			
			break; 
		case CL_PONTICELLO:
		case CL_TASTIERA:
			if(GetPStrumento()!=NULL && GetPStrumento()->GetID()==CL_ARCO)
			{
				if((((Arco*)GetPStrumento())->GetPPosizArco())!=NULL && 
					(((Arco*)GetPStrumento())->GetPPosizArco())->GetID()==symbol)
				{
					PosizArco *poarc;
					poarc = ((Arco*)GetPStrumento())->GetPPosizArco();
					if (GetLGambo()>0) gamboupdown = TRUE;
					else if (GetLGambo()<0) gamboupdown = FALSE;
					else if(GetLGambo()==0) 
					{
						if(GetAltezza()>=4) gamboupdown = FALSE;
						else gamboupdown = TRUE;
					}  
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						msgdebug.sprintf("");
					}
					
					
					
					point = posizionam.PosRuleP((DrawObject*)poarc, this, pBattuta,
						gamboupdown, layer);
					
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (poarc->GetTraceMilla())
						{  
							posizionam.TestTokenCond(this, (DrawObject*)poarc, pBattuta, gamboupdown, layer);
							
							WDebugMilla(); 
							
							poarc->SetTraceMilla(FALSE);
						}
					}
					
					AdjustPosizArco(YLinea1,YLinea5,point);
					
					
				}
			}  
			
			
			break; 
		case CL_ARCOSU:
		case CL_ARCOGIU:
			if(GetPStrumento()!=NULL && GetPStrumento()->GetID()==CL_ARCO)
			{
				if((((Arco*)GetPStrumento())->GetPDirezArco())!=NULL && 
					(((Arco*)GetPStrumento())->GetPDirezArco())->GetID()==symbol)
				{
					DirezArco *darc;
					darc = ((Arco*)GetPStrumento())->GetPDirezArco();	    
					if (GetLGambo()>0) gamboupdown = TRUE;
					else if (GetLGambo()<0) gamboupdown = FALSE;
					else if(GetLGambo()==0) 
					{
						if(GetAltezza()>=4) gamboupdown = FALSE;
						else gamboupdown = TRUE;
					}  
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						msgdebug.sprintf("");
					}
					
					
					
					point = posizionam.PosRuleP((DrawObject*)darc, this, pBattuta,
						gamboupdown, layer);
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (darc->GetTraceMilla())
						{  
							posizionam.TestTokenCond(this, (DrawObject*)darc, pBattuta, gamboupdown, layer);
							
							WDebugMilla(); 
							
							darc->SetTraceMilla(FALSE);
						}
					}
					
					AdjustDirezArco(YLinea1,YLinea5,point);
					
					
				}
			}
			
			break; 
		case CL_CORDA:
			if(GetPStrumento()!=NULL && (GetPStrumento()->GetID()==CL_ARCO ||
				GetPStrumento()->GetID()==CL_PIZZICATO))
			{
				if((((Violino*)GetPStrumento())->GetPCorda())!=NULL &&
					(((Violino*)GetPStrumento())->GetPCorda())->GetID()==CL_CORDA)
				{
					Corda *corda;
					corda = ((Violino*)GetPStrumento())->GetPCorda();
					if (GetLGambo()>0) gamboupdown = TRUE;
					else if (GetLGambo()<0) gamboupdown = FALSE;
					else if(GetLGambo()==0) 
					{
						if(GetAltezza()>=4) gamboupdown = FALSE;
						else gamboupdown = TRUE;
					}  
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						msgdebug.sprintf("");
					}
					
					
					
					point = posizionam.PosRuleP((DrawObject*)corda, this, pBattuta,
						gamboupdown, layer);
					
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (corda->GetTraceMilla())
						{  
							posizionam.TestTokenCond(this, (DrawObject*)corda, pBattuta, gamboupdown, layer);
							
							WDebugMilla(); 
							
							corda->SetTraceMilla(FALSE);
						}
					}
					
					AdjustCorda(YLinea1,YLinea5,point);
					
					
				}
			}
			
			
			break; 
		case CL_ARCO:
			if(GetPStrumento()!=NULL)
			{
				if(GetPStrumento()->GetID()==CL_ARCO)
				{
					Arco *arco;
					arco = (Arco*)GetPStrumento();	   
					if (GetLGambo()>0) gamboupdown = TRUE;
					else if (GetLGambo()<0) gamboupdown = FALSE;
					else if(GetLGambo()==0) 
					{
						if(GetAltezza()>=4) gamboupdown = FALSE;
						else gamboupdown = TRUE;
					}  
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						msgdebug.sprintf("");
					}
					
					
					
					point = posizionam.PosRuleP((DrawObject*)arco, this, pBattuta,
						gamboupdown, layer);
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (arco->GetTraceMilla())
						{  
							posizionam.TestTokenCond(this, (DrawObject*)arco, pBattuta, gamboupdown, layer);
							
							WDebugMilla(); 
							
							arco->SetTraceMilla(FALSE);
						}
					}
					
					AdjustStrumento(YLinea1,YLinea5,point);
					
					
					
				}
			}  
			
			break;	  
		case CL_PIZZICATO:
			if(GetPStrumento()!=NULL)
			{
				if(GetPStrumento()->GetID()==CL_PIZZICATO)
				{
					Strumento *pizz;
					pizz = GetPStrumento();	 
					if (GetLGambo()>0) gamboupdown = TRUE;
					else if (GetLGambo()<0) gamboupdown = FALSE;
					else if(GetLGambo()==0) 
					{
						if(GetAltezza()>=4) gamboupdown = FALSE;
						else gamboupdown = TRUE;
					}  
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						msgdebug.sprintf("");
					}
					
					
					
					point = posizionam.PosRuleP((DrawObject*)pizz, this, pBattuta,
						gamboupdown, layer);
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (pizz->GetTraceMilla())
						{  
							posizionam.TestTokenCond(this, (DrawObject*)pizz, pBattuta, gamboupdown, layer);
							
							WDebugMilla(); 
							
							pizz->SetTraceMilla(FALSE);
						}
					}
					
					AdjustStrumento(YLinea1,YLinea5,point);
					
					
				}
			}  
			
			
			
			
			break;	   
		case CL_MORDENTE:
			if(GetPAbbellimento()!=NULL)
			{
				if(GetPAbbellimento()->GetID()==CL_MSUPERIORE ||
					GetPAbbellimento()->GetID()==CL_MINFERIORE)
				{
					Abbellimento *abb;
					abb = GetPAbbellimento();	 
					if (GetLGambo()>0) gamboupdown = TRUE;
					else if (GetLGambo()<0) gamboupdown = FALSE;
					else if(GetLGambo()==0) 
					{
						if(GetAltezza()>=4) gamboupdown = FALSE;
						else gamboupdown = TRUE;
					}  
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						msgdebug.sprintf("");
					}
					
					
					
					point = posizionam.PosRuleP((DrawObject*)abb, this, pBattuta,
						gamboupdown, layer);
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (abb->GetTraceMilla())
						{  
							posizionam.TestTokenCond(this, (DrawObject*)abb, pBattuta, gamboupdown, layer);
							
							WDebugMilla(); 
							
							abb->SetTraceMilla(FALSE);
						}
					}
					
					AdjustAbbellimento(YLinea1,YLinea5,point);
					
					
				}
			}  
			
			
			
			
			break;	   
		case CL_GRUPPETTO:
			if(GetPAbbellimento()!=NULL)
			{
				if(GetPAbbellimento()->GetID()==CL_GSUPERIORE ||
					GetPAbbellimento()->GetID()==CL_GINFERIORE ||
					GetPAbbellimento()->GetID()==CL_GSLASH ||
					GetPAbbellimento()->GetID()==CL_GUP)
				{
					Abbellimento *abb;
					abb = GetPAbbellimento();	 
					if (GetLGambo()>0) gamboupdown = TRUE;
					else if (GetLGambo()<0) gamboupdown = FALSE;
					else if(GetLGambo()==0) 
					{
						if(GetAltezza()>=4) gamboupdown = FALSE;
						else gamboupdown = TRUE;
					}  
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						msgdebug.sprintf("");
					}
					
					
					
					point = posizionam.PosRuleP((DrawObject*)abb, this, pBattuta,
						gamboupdown, layer);
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (abb->GetTraceMilla())
						{  
							posizionam.TestTokenCond(this, (DrawObject*)abb, pBattuta, gamboupdown, layer);
							
							WDebugMilla(); 
							
							abb->SetTraceMilla(FALSE);
						}
					}
					
					AdjustAbbellimento(YLinea1,YLinea5,point);
					
					
					
				}
			}  
			
			
			
			
			break;	   
		case CL_TRILLO:
			if(GetPAbbellimento()!=NULL)
			{
				if(GetPAbbellimento()->GetID()==CL_TRILLO)
				{
					Abbellimento *abb;
					abb = GetPAbbellimento();	 
					if (GetLGambo()>0) gamboupdown = TRUE;
					else if (GetLGambo()<0) gamboupdown = FALSE;
					else if(GetLGambo()==0) 
					{
						if(GetAltezza()>=4) gamboupdown = FALSE;
						else gamboupdown = TRUE;
					}  
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						msgdebug.sprintf("");
					}
					
					
					
					point = posizionam.PosRuleP((DrawObject*)abb, this, pBattuta,
						gamboupdown, layer);
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (abb->GetTraceMilla())
						{  
							posizionam.TestTokenCond(this, (DrawObject*)abb, pBattuta, gamboupdown, layer);
							
							WDebugMilla(); 
							
							abb->SetTraceMilla(FALSE);
						}
					}
					
					AdjustAbbellimento(YLinea1,YLinea5,point);
					
					
				}
			}  
			
			
			
			
			
			break;
		case CL_CONSORDINA:
		case CL_VIASORDINA:
			if(GetPSordina()!=NULL)
			{
				if((symbol==CL_CONSORDINA && 
					(((Indicazione*)GetPSordina())->GetID()==CL_CONSORDINAARCHI ||
					((Indicazione*)GetPSordina())->GetID()==CL_CONSORDINAOTTONI ||
					((Indicazione*)GetPSordina())->GetID()==CL_CONSORDINATESTO)) ||
					(symbol==CL_VIASORDINA && 
					(((Indicazione*)GetPSordina())->GetID()==CL_VIASORDINAARCHI ||
					((Indicazione*)GetPSordina())->GetID()==CL_VIASORDINAOTTONI ||
					((Indicazione*)GetPSordina())->GetID()==CL_VIASORDINATESTO)))
				{
					Sordina *sor;
					sor = GetPSordina();	 
					if (GetLGambo()>0) gamboupdown = TRUE;
					else if (GetLGambo()<0) gamboupdown = FALSE;
					else if(GetLGambo()==0) 
					{
						if(GetAltezza()>=4) gamboupdown = FALSE;
						else gamboupdown = TRUE;
					}  
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						msgdebug.sprintf("");
					}
					
					
					
					point = posizionam.PosRuleP(((DrawObject*)(Indicazione*)sor), this, pBattuta,
						gamboupdown, layer);
					
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (sor->GetTraceMilla())
						{  
							posizionam.TestTokenCond(this, (DrawObject*)(Indicazione*)sor, pBattuta, gamboupdown, layer);
							
							WDebugMilla(); 
							
							sor->SetTraceMilla(FALSE);
						}
					}
					
					
					AdjustSordina(YLinea1,YLinea5,point);
					
					
				}
			}  
			
			
			
			
			break;
		case CL_DITEGGIATO:
			if(GetPDiteggiato()!=NULL)
			{
				Diteggiato *dit;
				dit = GetPDiteggiato();	 
				if (GetLGambo()>0) gamboupdown = TRUE;
				else if (GetLGambo()<0) gamboupdown = FALSE;
				else if(GetLGambo()==0) 
				{
					if(GetAltezza()>=4) gamboupdown = FALSE;
					else gamboupdown = TRUE;
				}  
				
				if (DebugNormalMilla || DebugNormalMillaPos)
				{ 
					msgdebug.sprintf("");
				}
				
				
				
				point = posizionam.PosRuleP((DrawObject*)dit, this, pBattuta,
					gamboupdown, layer);
				
				
				if (DebugNormalMilla || DebugNormalMillaPos)
				{ 
					if (dit->GetTraceMilla())
					{  
						posizionam.TestTokenCond(this, (DrawObject*)dit, pBattuta, gamboupdown, layer);
						
						WDebugMilla(); 
						
						dit->SetTraceMilla(FALSE);
					}
				}
				
				AdjustDiteggiato(YLinea1,YLinea5,point);
				
			}  
			
			
			
			
			break;
		case CL_ARMONICI:
			if(GetPArmonici()!=NULL)
			{
				Armonici *arm;
				arm = GetPArmonici();	 
				if (GetLGambo()>0) gamboupdown = TRUE;
				else if (GetLGambo()<0) gamboupdown = FALSE;
				else if(GetLGambo()==0) 
				{
					if(GetAltezza()>=4) gamboupdown = FALSE;
					else gamboupdown = TRUE;
				}  
				
				if (DebugNormalMilla || DebugNormalMillaPos)
				{ 
					msgdebug.sprintf("");
				}
				
				
				
				point = posizionam.PosRuleP((DrawObject*)arm, this, pBattuta,
					gamboupdown, layer);
				
				
				if (DebugNormalMilla || DebugNormalMillaPos)
				{ 
					if (arm->GetTraceMilla())
					{  
						posizionam.TestTokenCond(this, (DrawObject*)arm, pBattuta, gamboupdown, layer);
						
						WDebugMilla(); 
						
						arm->SetTraceMilla(FALSE);
					}
				}
				
				AdjustArmonici(YLinea1,YLinea5,point);
			}  			
			break;	  
		case CL_KOREANORNAMENT:
			if(GetPKoreanOrnament()!=NULL)
			{
				KoreanOrnament *kor;
				kor = GetPKoreanOrnament();	 
				if (GetLGambo()>0) gamboupdown = TRUE;
				else if (GetLGambo()<0) gamboupdown = FALSE;
				else if(GetLGambo()==0) 
				{
					if(GetAltezza()>=4) gamboupdown = FALSE;
					else gamboupdown = TRUE;
				}  
				point = posizionam.PosRuleP((DrawObject*)kor, this, pBattuta,
					gamboupdown, layer);
				AdjustKoreanOrnament(YLinea1,YLinea5,point);
			}  
			break;
			
      }
  }   
  
  if(!primaleg)
  {
	  AdjustSuddivisione();
	  
	  Figura::Adjust(primaleg, pBattuta, layer,0);
  } 
  
  
} 



void Nota::DrawTesta()
{
	int dx,dy;

	Colore oldcol = Scr->GetColor();

	if(isOpaque)
		Scr->SetColor(GetColor());

	if(HeadType!=HEAD_ALPHANUM && HeadType!=HEAD_ALPHANUM_SQUARE && HeadType!=HEAD_ALPHANUM_REVERSE)
	{
		dx=Scr->Get_VU2DX(HeadCode[0]);
		dy=Scr->Get_VU2DY(HeadCode[0]);
	}
	else
	{
		Scr->SetFont(nTextFont);
		dx=TEXTDX(HeadCode);
		dy=TEXTDY(HeadCode);
	}


	if(HeadType==HEAD_ALPHANUM || HeadType==HEAD_ALPHANUM_SQUARE)
	{
		oldcol = Scr->GetColor();
		Scr->SetColor((ColorIcon)white);
		Scr->Rect(class Rectangle(GetAbsPos()-Point(TEXTLF(HeadCode),TEXTDW(HeadCode)),
			GetAbsPos()+Point(TEXTRG(HeadCode),TEXTUP(HeadCode))),TRUE);
		Scr->SetColor(oldcol);
	}
	else if(HeadType==HEAD_ALPHANUM_REVERSE)
	{  
		Scr->Rect(class Rectangle(GetAbsPos()-Point(TEXTLF(HeadCode),TEXTDW(HeadCode)),
			GetAbsPos()+Point(TEXTRG(HeadCode),TEXTUP(HeadCode))),TRUE);
	}


	if(HeadType==HEAD_ALPHANUM_SQUARE)  
	{  
		Scr->Rect(class Rectangle(GetAbsPos()-Point(TEXTLF(HeadCode),TEXTDW(HeadCode)),
			GetAbsPos()+Point(TEXTRG(HeadCode),TEXTUP(HeadCode))));
	}
	
	if((HeadType==HEAD_ALPHANUM || HeadType==HEAD_ALPHANUM_SQUARE))
	{
		if(isOpaque)
			 Scr->SetColor(GetColor());
		//Scr->TextTo(GetAbsPos()+Point(dx,dy));
		//Scr->OutText((unsigned char *)HeadCode);
	}
	else if(HeadType==HEAD_ALPHANUM_REVERSE)
	{
		Scr->SetColor((ColorIcon)white);
		//Scr->TextTo(GetAbsPos()+Point(dx,dy));
		//Scr->OutText((unsigned char *)HeadCode);

	}

	Scr->TextTo(GetAbsPos()+Point(dx,dy));
	Scr->OutText((unsigned char *)HeadCode);

	Scr->SetColor(oldcol);	

	Scr->SetFont(nMusicFont);
}

VUnit Nota::GetXgambo()
{
	VUnit xgambo;
	
	xgambo=GetAbsPos().x;
	if(DxGambo>0)
	{
        if(HeadType!=HEAD_ALPHANUM && HeadType!=HEAD_ALPHANUM_SQUARE && HeadType!=HEAD_ALPHANUM_REVERSE)
			xgambo+=Scr->Get_VU2RG(HeadCode[0]);
		else
		{
			Scr->SetFont(nTextFont);
			xgambo+=TEXTRG(HeadCode);
			Scr->SetFont(nMusicFont);
		}
	}
	else if(DxGambo<0)
	{
        if(HeadType!=HEAD_ALPHANUM && HeadType!=HEAD_ALPHANUM_SQUARE && HeadType!=HEAD_ALPHANUM_REVERSE)
			xgambo-=Scr->Get_VU2LF(HeadCode[0]);
		else
        {
			Scr->SetFont(nTextFont);
			xgambo+=TEXTLF(HeadCode);
			Scr->SetFont(nMusicFont);
        }
	}
	return xgambo;
}

VUnit Nota::GetYgambo()
{     
	VUnit ygambo;
	
	ygambo=GetAbsPos().y;
	if(DyGambo>0)
	{
        if(HeadType!=HEAD_ALPHANUM && HeadType!=HEAD_ALPHANUM_SQUARE && HeadType!=HEAD_ALPHANUM_REVERSE)
			ygambo+=Scr->Get_VU2UP(HeadCode[0]);
		else
		{
			Scr->SetFont(nTextFont);
			ygambo+=TEXTUP(HeadCode);
			Scr->SetFont(nMusicFont);
		}
	}
	else if(DyGambo<0)
	{
        if(HeadType!=HEAD_ALPHANUM && HeadType!=HEAD_ALPHANUM_SQUARE && HeadType!=HEAD_ALPHANUM_REVERSE)
			ygambo-=Scr->Get_VU2DWN(HeadCode[0]);
		else
		{
			Scr->SetFont(nTextFont);
			ygambo-=TEXTDW(HeadCode);
			Scr->SetFont(nMusicFont);
		}
	}
	return ygambo;
}


void Nota::DrawTagli()
{
	VUnit ytaglio,xtagliolf,xtagliorg;
	VUnit YLinea1, YLinea5;
	
	if(HeadType==HEAD_ALPHANUM || HeadType==HEAD_ALPHANUM_SQUARE || HeadType==HEAD_ALPHANUM_REVERSE)
	{
		Scr->SetFont(nTextFont);
		xtagliolf=GetAbsPos().x-TEXTLF(HeadCode);
		xtagliorg=GetAbsPos().x+TEXTRG(HeadCode);
		Scr->SetFont(nMusicFont);
	}
	else
	{
		xtagliolf=GetAbsPos().x-Scr->Get_VU2LF(HeadCode[0]);
		xtagliorg=GetAbsPos().x+Scr->Get_VU2RG(HeadCode[0]);
	}
	
	// Disegno degli eventuali tagli addizionali.
	if(GetAltezza()<1)
	{ 
        YLinea1=GetAbsPos().y-GetAltezza()*VU_PER_ROW/2;
		for(ytaglio=YLinea1-VU_PER_ROW;ytaglio>=GetAbsPos().y;ytaglio-=VU_PER_ROW)
		{
			Scr->MoveTo(xtagliolf+(GetNotina()?OURTAGLIOLFT/2:OURTAGLIOLFT),ytaglio);
			Scr->LineTo(xtagliorg+(GetNotina()?OURTAGLIORGT/2:OURTAGLIORGT),ytaglio,SOLID_LINE);
		}
	}
	if(GetAltezza()>9)
	{            
		YLinea5=GetAbsPos().y+(8-GetAltezza())*VU_PER_ROW/2;
		for(ytaglio=YLinea5+VU_PER_ROW;ytaglio<=GetAbsPos().y;ytaglio+=VU_PER_ROW)
		{               
			Scr->MoveTo(xtagliolf+(GetNotina()?OURTAGLIOLFT/2:OURTAGLIOLFT),ytaglio);
			Scr->LineTo(xtagliorg+(GetNotina()?OURTAGLIORGT/2:OURTAGLIORGT),ytaglio,SOLID_LINE);
		}
	}
}

void Nota::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	VUnit xgambo,ygambo;
    // Altezza fisica delle linee piu' bassa e piu' alta del pentagramma.
	
	Figura::Draw();
	
	// Disegno dei tagli addizionali.
	DrawTagli();
	
	// Disegno della testa.
	DrawTesta();
	
	// Disegno dell'eventuale gambo.
	if(GetLGambo())
	{
		xgambo=GetXgambo();
		ygambo=GetYgambo();
		Scr->MoveTo(xgambo,ygambo);
		Scr->LineTo(xgambo,ygambo+GetLGambo(),SOLID_LINE);
	} 
	
	// Disegno dell'eventuale alterazione composta.
	if(GetPAlterazComposta()!=NULL) 
	{
		GetPAlterazComposta()->Draw();
	} 
	
	// Disegno dell'eventuale espressione composta.
	if(GetPEspressComposta()!=NULL)
	{
		GetPEspressComposta()->Draw();
	}
	// Disegno dell'eventuale abbellimento.
	if(GetPAbbellimento()!=NULL)
	{
		GetPAbbellimento()->Draw();
	}
	// Disegno dell'eventuale KoreanOrnament.
	if(GetPKoreanOrnament()!=NULL)
	{
		GetPKoreanOrnament()->Draw();
	}
	// Disegno dell'eventuale diteggiato.
	if(GetPDiteggiato()!=NULL)
	{
		GetPDiteggiato()->Draw();
	}
	// Disegno dell'eventuale sordina.
	if(GetPSordina()!=NULL)
	{
		GetPSordina()->Draw();
	}
	// Disegno dell'eventuale Armonici.
	if(GetPArmonici()!=NULL)
	{
		GetPArmonici()->Draw();
	}
	// Disegno dell'eventuale Suddivisione.
	if(GetPSuddivisione()!=NULL)
	{
		GetPSuddivisione()->Draw();
	}

	Scr->SetColor(oldcol);
};                 

void Nota::DrawForAccordo()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());
	
	VUnit xgambo,ygambo;
	// Altezza fisica delle linee piu' bassa e piu' alta del pentagramma.
	
	// Non esegue Scr->SetColor(GetColor()) poiche' il colore dello schermo deve
	// rimanere quello dell'accordo che chiama questo metodo.
	
    DrawTesta();
	
    if(GetLGambo())
    {
		xgambo=GetXgambo();
		ygambo=GetYgambo();
		Scr->MoveTo(xgambo,ygambo);
		Scr->LineTo(xgambo,ygambo+GetLGambo(),SOLID_LINE);
    }
	
	// Disegno dell'eventuale alterazione composta.
	if(GetPAlterazComposta()!=NULL) 
	{
		GetPAlterazComposta()->Draw();
	}

	// Disegno dell'eventuale punto di valore.
	if(PPuntoValore!=NULL)
	{
		PPuntoValore->Draw();
	}
	
	// Disegno dell'eventuale diteggiato.
	if(GetPDiteggiato()!=NULL)
	{
		GetPDiteggiato()->Draw();
	}

	if (isSelected) {
		int dx,dy;

		if(HeadType!=HEAD_ALPHANUM && HeadType!=HEAD_ALPHANUM_SQUARE && HeadType!=HEAD_ALPHANUM_REVERSE){
			dx=Scr->Get_VU2DX(HeadCode[0]);
			dy=Scr->Get_VU2DY(HeadCode[0]);
		}
		else{
			dx=TEXTDX(HeadCode);
			dy=TEXTDY(HeadCode);
		}

		Scr->SetColor((ColorIcon)SELECTED_COLOR);
		Scr->BigRect(class Rectangle(GetAbsPos(), 
			abs(dx) + SELECTBOX_MARGIN,
			abs(dx) + SELECTBOX_MARGIN, 
			abs(dy) + SELECTBOX_MARGIN, 
			abs(dy) + SELECTBOX_MARGIN),
			3);
	}

	Scr->SetColor(oldcol);	
}                 

void Nota::DrawContour(VUnit width){
	DrawObject::DrawContour(SELECTBOX_MARGIN);
}

Bool Nota::DeleteSym(DrawObject *sym)
{
	
	
	if(GetPAlterazComposta()!=NULL)               
		if(GetPAlterazComposta()->DeleteSym(sym))
		{                                              
			if(GetPAlterazComposta()->Getn_alterazioni()==0)
			{                
				delete GetPAlterazComposta();
				SetPAlterazComposta(NULL);
			}           
			return TRUE; 
		}
		
		if(sym==GetPAbbellimento())
		{
			delete GetPAbbellimento();
			SetPAbbellimento(NULL);
			return TRUE;
		}
		if(GetPAbbellimento()!=NULL) 
			if(GetPAbbellimento()->DeleteSym(sym))
				return TRUE;
			
			if(sym==GetPDiteggiato())
			{
				delete GetPDiteggiato();
				SetPDiteggiato(NULL);
				return TRUE;
			}
			
			if(sym==GetPSuddivisione())
			{
				delete GetPSuddivisione();
				SetPSuddivisione(NULL);
				return TRUE;
			}
			
			if(((Indicazione *)sym)==GetPSordina())
			{
				delete GetPSordina();
				SetPSordina(NULL);
				return TRUE;
			}
			
			if(sym==GetPArmonici())
			{
				delete GetPArmonici();
				SetPArmonici(NULL);
				return TRUE;
			}
			
			
			if(GetPEspressComposta()!=NULL)               
				if(GetPEspressComposta()->DeleteSym(sym))
				{                                              
					if(GetPEspressComposta()->Getn_espressioni()==0)
					{                
						delete GetPEspressComposta();
						SetPEspressComposta(NULL);
					}           
					return TRUE; 
				}  
				
				if(sym==GetPOcchiali())
				{
					delete GetPOcchiali();
					SetPOcchiali(NULL);
					return TRUE;
				} 
				
				if(sym==GetPKoreanOrnament())
				{
					delete GetPKoreanOrnament();
					SetPKoreanOrnament(NULL);
					return TRUE;
				} 
				
				if (Figura::DeleteSym(sym))
					return TRUE;
				else
					return DeleteStrumento(sym);
}


Bool Nota::DeleteStrumento(DrawObject *sym)
{
	Strumento *pS = GetPStrumento();
	Bool DelSym;
	
	
	if (pS==NULL)
		return FALSE;
	
	
	switch(pS->GetID())
	{
    case CL_ARCO:
		
		DelSym = FALSE;
		
		if(sym==pS)
		{
			(((Arco*)pS)->SetScrivereArco(FALSE));
			DelSym = TRUE;
		}   
		if(sym==(((Arco*)pS)->GetPParteArco()))
		{
			delete ((Arco*)pS)->GetPParteArco();
			((Arco*)pS)->SetPParteArco(NULL);
			DelSym = TRUE;
		} 
		if(sym==(((Arco*)pS)->GetPDirezArco()))
		{
			delete ((Arco*)pS)->GetPDirezArco();
			((Arco*)pS)->SetPDirezArco(NULL);
			DelSym = TRUE;
		} 
		if(sym==(((Arco*)pS)->GetPPosizArco()))
		{
			delete ((Arco*)pS)->GetPPosizArco();
			((Arco*)pS)->SetPPosizArco(NULL);
			DelSym = TRUE;
		} 
		if(sym==(((Arco*)pS)->GetPCorda()))
		{
			delete ((Arco*)pS)->GetPCorda();
			((Arco*)pS)->SetPCorda(NULL);
			DelSym = TRUE;
		} 
		
        // controllo che Arco non sia sparito completamente
		if ((((Arco*)pS)->GetScrivereArco()==FALSE) && (((Arco*)pS)->GetPPosizArco()==NULL) &&
			(((Arco*)pS)->GetPDirezArco()==NULL) && (((Arco*)pS)->GetPParteArco()==NULL) && 
			(((Arco*)pS)->GetPCorda()==NULL))
		{
			delete GetPStrumento();
			SetPStrumento(NULL);
		}
		
		if (DelSym)
			return TRUE;
		
		break;
		
    case CL_PIZZICATO:
		if(sym==pS)
		{
			delete GetPStrumento();
			SetPStrumento(NULL);
			return TRUE;
		}   
		if(sym==(((Pizzicato*)pS)->GetPCorda()))
		{
			delete ((Pizzicato*)pS)->GetPCorda();
			((Pizzicato*)pS)->SetPCorda(NULL);
			return TRUE;
		} 
		break;
		
    default: 
		if(sym==pS)
		{
			delete GetPStrumento();
			SetPStrumento(NULL);
			return TRUE;
		}   
	}
	
	
	return FALSE;
}



void Nota::Free()
{
	
	if(GetPAlterazComposta()!=NULL) 
		delete PAlterazComposta;
	if(GetPEspressComposta()!=NULL) 
		delete PEspressComposta;  
	if(GetPAbbellimento()!=NULL)
		delete PAbbellimento;
	if(GetPKoreanOrnament()!=NULL)
		delete PKoreanOrnament;	
	if(GetPDiteggiato()!=NULL)
		delete PDiteggiato;
	if(GetPSuddivisione()!=NULL)
		delete PSuddivisione;
	if(GetPSordina()!=NULL)
		delete PSordina;
	if(GetPArmonici()!=NULL)
		delete PArmonici;  
	
	// Il Free() di Figura chiama anche Init().               
	Figura::Free();
}                 


Bool Nota::Hit(const Point& p, SelObj selObj, DrawObject*& drwObjSel,
			   SymPath& symPath)
{
	Bool hit;   
	Point p1;
	
	
	if (Figura::Hit(p, selObj, drwObjSel, symPath))
		return TRUE;
	
	p1=p-GetAbsPos();               
	hit = -p1.x <= GetVU2LftF() && p1.x <= GetVU2RgtF() &&
        -p1.y <= GetVU2DwnF() && p1.y <= GetVU2UpF();
	
	drwObjSel=NULL;
	
	if(selObj==NOTA_SEL || selObj==NOTASING_SEL)
		if(hit)
		{
			drwObjSel=this;
			symPath.Array[symPath.Level]=GetNumericCode();
			return(TRUE);
		}
		
		
		if(selObj!=ANY_SEL)
			return(FALSE);
		
		if(drwObjSel==NULL) 
		{
			
			if(GetPAlterazComposta()!=NULL &&
				GetPAlterazComposta()->Hit(p,selObj,drwObjSel,symPath))
			{ 
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
			}
			
			if(GetPAbbellimento()!=NULL &&
				GetPAbbellimento()->Hit(p,selObj,drwObjSel,symPath))
			{ 
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
			}
			if(GetPKoreanOrnament()!=NULL &&
				GetPKoreanOrnament()->Hit(p,selObj,drwObjSel,symPath))
			{ 
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
			}
			if(GetPDiteggiato()!=NULL &&
				GetPDiteggiato()->Hit(p,selObj,drwObjSel,symPath))
			{ 
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
			}
			if(GetPSuddivisione()!=NULL &&
				GetPSuddivisione()->Hit(p,selObj,drwObjSel,symPath))
			{ 
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
			}
			
			
			if(GetPEspressComposta()!=NULL &&
				GetPEspressComposta()->Hit(p,selObj,drwObjSel,symPath))
			{            
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
			} 
			if(GetPSordina()!=NULL &&
				GetPSordina()->Hit(p,selObj,drwObjSel,symPath))
			{ 
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
			}
			if(GetPArmonici()!=NULL &&
				GetPArmonici()->Hit(p,selObj,drwObjSel,symPath))
			{ 
				symPath.Array[symPath.Level]=GetNumericCode();
				return(TRUE);
			}
		}
		return(FALSE);
}


Bool Nota::SymbolicHit(SymPath& symPath, SelObj selObj, DrawObject*& drwObjSel)
{
	
	
	if(!(symPath.Array[symPath.Level]==GetNumericCode()))
		return FALSE;
	
	if(Figura::SymbolicHit(symPath, selObj, drwObjSel))
		return TRUE;
	
	if(symPath.Array[SIMBOLO_PATH_LEVEL]==0)
	{
		// Il path si ferma a questa nota.
		if(selObj==NOTA_SEL || selObj==NOTASING_SEL)
		{
			drwObjSel=this;
			return(TRUE);
		}
	}    
	else if(drwObjSel==NULL && selObj==ANY_SEL)
	{
		
		if(GetPAlterazComposta()!=NULL &&
			GetPAlterazComposta()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);
		if(GetPAbbellimento()!=NULL &&
			GetPAbbellimento()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);   
		if(GetPKoreanOrnament()!=NULL &&
			GetPKoreanOrnament()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE); 	
		if(GetPDiteggiato()!=NULL &&
			GetPDiteggiato()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE); 
		if(GetPSuddivisione()!=NULL &&
			GetPSuddivisione()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE); 
		if(GetPSordina()!=NULL &&
			GetPSordina()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);
		if(GetPArmonici()!=NULL &&
			GetPArmonici()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);
		
		
		if(GetPEspressComposta()!=NULL &&
			GetPEspressComposta()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE); 
	}
	return(FALSE);
}



CmdResult Nota::SymCommand(SymCmd& SCmd, Battuta *pBattuta)
{
	CmdResult cmdRes;
	TipoInserimento tpIns=(TipoInserimento)SCmd.ArgV[0];
	Bool inMultiVoice=(Bool)SCmd.ArgV[2];
	int layer=SCmd.Path1.Array[3];
	Bool aboveNota=FALSE;
	int IDsym=SCmd.ArgV[1];
	int tipoDito=(TipiFinger)SCmd.ArgV[1];
	short current_staff=SCmd.ArgV[9];//GB
    
	cmdRes=Figura::SymCommand(SCmd, pBattuta);
	if(cmdRes==CMD_OK)
        return(cmdRes);
	
	switch(tpIns)
	{
	case SOPRA:
		aboveNota=TRUE;
		break;
	case SOTTO:
		aboveNota=FALSE;
		break;
	case AUTOMATICO:
		{
			if(inMultiVoice)
			{
				if(layer==1)
					aboveNota=TRUE;
				else
					aboveNota=FALSE;
			}
			else
			{
				if(GetLGambo()>0)
					aboveNota=FALSE;
				else if(GetLGambo()<0)
					aboveNota=TRUE;
				else if(GetAltezza()>=4)
					aboveNota=TRUE;
				else
					aboveNota=FALSE;
			}
			break;
		}
	}
	
	switch(SCmd.CommandId)
	{
    case CMD_NOTA_PIZZICATO:
		return(CmdNotaPizzicato(tpIns, this, pBattuta));
		break;  
    case CMD_NOTA_ARCO:
    case CMD_ARCO_ARCOSU:
    case CMD_ARCO_ARCOGIU:
    case CMD_ARCO_PONTICELLO:
    case CMD_ARCO_TASTIERA:
    case CMD_ARCO_PUNTA:
    case CMD_ARCO_TALLONE:
		return(CmdNotaStrArco(SCmd, tpIns, this, pBattuta));
		break;
    case CMD_NOTA_CORDA_0:
    case CMD_NOTA_CORDA_1:
    case CMD_NOTA_CORDA_2:
    case CMD_NOTA_CORDA_3:
    case CMD_NOTA_CORDA_4:
    case CMD_NOTA_CORDA_5:
    case CMD_NOTA_CORDA_6:
    case CMD_NOTA_CORDA_7:
		return(CmdNotaCorda(SCmd, tpIns, this, pBattuta));
		break;
    case CMD_NOTA_DIESIS:
		return(CmdNotaDiesis());
		break;
    case CMD_NOTA_DIESIS1Q:
		return(CmdNotaDiesis1Q());
		break;
    case CMD_NOTA_DIESIS3Q:
		return(CmdNotaDiesis3Q());
		break;    
    case CMD_NOTA_BEMOLLE:
		return(CmdNotaBemolle());
		break;
    case CMD_NOTA_BEMOLLE1Q:
		return(CmdNotaBemolle1Q());
		break;
    case CMD_NOTA_BEMOLLE3Q:
		return(CmdNotaBemolle3Q());
		break;    
    case CMD_NOTA_BEQUADRO:
		return(CmdNotaBequadro());
		break;  
    case CMD_NOTA_2DIESIS:
		return(CmdNota2Diesis());
		break;  
    case CMD_NOTA_2BEMOLLE:
		return(CmdNota2Bemolle());
		break;  
    case CMD_NOTA_BEQUADRO1Q:
		return(CmdNotaBequadro1Q());
		break;  
    case CMD_NOTA_BEQUADRO3Q:
		return(CmdNotaBequadro3Q());
		break;  
    case CMD_NOTA_TRILLO:
		return(CmdNotaTrillo(tpIns, this, pBattuta,(int)SCmd.ArgV[1]));
		break;  
	case CMD_NOTA_GRUPPETTO_SUP_1:
		return(CmdNotaGruppettoSup1(tpIns, this, pBattuta));
		break;  
    case CMD_NOTA_GRUPPETTO_INF_1:
		return(CmdNotaGruppettoInf1(tpIns, this, pBattuta));
		break;  
	case CMD_NOTA_TURN_SLASH:
		return(CmdNotaTurnSlash(tpIns, this, pBattuta));
		break;  
	case CMD_NOTA_TURN_UP:
		return(CmdNotaTurnUp(tpIns, this, pBattuta));
		break;  
    case CMD_NOTA_MORDENTE_SUP:
		return(CmdNotaMordenteSup(tpIns, this, pBattuta));
		break;
    case CMD_NOTA_MORDENTE2_SUP:
		return(CmdNotaMordente2Sup(tpIns, this, pBattuta));
		break;    
    case CMD_NOTA_MORDENTE_INF:
		return(CmdNotaMordenteInf(tpIns, this, pBattuta));
		break;
    case CMD_NOTA_MORDENTE2_INF:
		return(CmdNotaMordente2Inf(tpIns, this, pBattuta));
		break;   
	case CMD_NOTA_SLIDING_UP:
		return(CmdNotaSlidingUp(tpIns, this, pBattuta));
		break;
	case CMD_NOTA_SLIDING_DOWN:
		return(CmdNotaSlidingDown(tpIns, this, pBattuta));
		break;
	case CMD_NOTA_SLIDING_DUP:
		return(CmdNotaSlidingDUp(tpIns, this, pBattuta));
		break;
	case CMD_NOTA_SLIDING_DOWNUP:
		return(CmdNotaSlidingDownUp(tpIns, this, pBattuta));
		break;
	case CMD_NOTA_NONGHYN_INC:
		return(CmdNotaNonghynInc(tpIns, this, pBattuta));
		break;
	case CMD_NOTA_NONGHYN_DEC:
		return(CmdNotaNonghynDec(tpIns, this, pBattuta));
		break;
	case CMD_NOTA_NONGHYN_ARROW:
		return(CmdNotaNonghynArrow(tpIns, this, pBattuta));
		break;
	case CMD_NOTA_NONGHYN_WIDE:
		return(CmdNotaNonghynWide(tpIns, this, pBattuta));
		break;
    case CMD_NOTA_NOTA2NOTINA:
		return(CmdNotaNota2Notina());
		break;  
    case CMD_NOTA_NOTINA2NOTA:
		return(CmdNotaNotina2Nota(tpIns, this, pBattuta));
		break;  
    case CMD_NOTA_DITO_0:
		return(CmdNotaDito(tpIns, this, pBattuta,0,tipoDito));
		break;  
    case CMD_NOTA_DITO_1:
		return(CmdNotaDito(tpIns, this, pBattuta,1,tipoDito));
		break;  
    case CMD_NOTA_DITO_2:
		return(CmdNotaDito(tpIns, this, pBattuta,2,tipoDito));
		break;  
    case CMD_NOTA_DITO_3:
		return(CmdNotaDito(tpIns, this, pBattuta,3,tipoDito));
		break;  
    case CMD_NOTA_DITO_4:
		return(CmdNotaDito(tpIns, this, pBattuta,4,tipoDito));
		break;  
    case CMD_NOTA_DITO_5:
		return(CmdNotaDito(tpIns, this, pBattuta,5,tipoDito));
		break;  
    case CMD_NOTA_TENUTO:
		return(CmdNotaTenuto(tpIns, this, pBattuta));
		break;
    case CMD_NOTA_PUNTOALL:
		return(CmdNotaPuntoAll(tpIns, this, pBattuta));
		break;
    case CMD_NOTA_SFORZATO:
		return(CmdNotaSforzato(tpIns, this, pBattuta));
		break;  
    case CMD_NOTA_ACCENTO:
		return(CmdNotaAccento(tpIns, this, pBattuta));
		break;
    case CMD_NOTA_MARTELLATO:
		return(CmdNotaMartellato(tpIns, this, pBattuta));
		break;     
    case CMD_NOTA_MARTDOLCE:
		return(CmdNotaMartDolce(tpIns, this, pBattuta));
		break; 
    case CMD_NOTA_STACCATO:
		return(CmdNotaStaccato(tpIns, this, pBattuta));
		break;
	case CMD_NOTA_ESPGENERIC:
		return(CmdNotaEspGeneric(IDsym, tpIns, this, pBattuta));
		break;   
    case CMD_NOTA_CONSORDINAOTTONI:
		return(CmdNotaConSordinaOttoni(tpIns, this, pBattuta));
		break;  
    case CMD_NOTA_CONSORDINAARCHI:
		return(CmdNotaConSordinaArchi(tpIns, this, pBattuta));
		break;
    case CMD_NOTA_CONSORDINATESTO:
		return(CmdNotaConSordinaTesto(tpIns, this, pBattuta));
		break;     
    case CMD_NOTA_VIASORDINAOTTONI:
		return(CmdNotaViaSordinaOttoni(tpIns, this, pBattuta));
		break;
    case CMD_NOTA_VIASORDINAARCHI:
		return(CmdNotaViaSordinaArchi(tpIns, this, pBattuta));
		break;
    case CMD_NOTA_VIASORDINATESTO:
		return(CmdNotaViaSordinaTesto(tpIns, this, pBattuta));
		break;
    case CMD_NOTA_ARMONICIOTTONI:
		return(CmdNotaArmoniciOttoni(tpIns, this, pBattuta));
		break;
    case CMD_NOTA_ARMONICIARCHI:
		return(CmdNotaArmoniciArchi(tpIns, this, pBattuta));
		break;    
    case CMD_NOTA_SUDDIVISIONE:
		return(CmdNotaSuddivisione((short)SCmd.ArgV[0]));
		break;    
    case CMD_NOTA_TREMOLO:
		return CmdNotaTremolo((short)SCmd.ArgV[0]);
		break;
    case CMD_NOTA_GLISSATO:
		return CmdNotaGlissato();
		break;  
    case CMD_NOTA_GLISSONDA:
		return CmdNotaGlissonda();
		break;  
    default:
		cmdRes=Figura::SymCommand(SCmd, pBattuta);
		if(cmdRes!=CMD_UNKNOWN)
            return(cmdRes);
  }            
  return(CMD_UNKNOWN);
}


// METODI PRIVATI


void Nota::AdjustAlterazComposta()
{
	Alterazione *PAlterazione;
	
	if(GetPAlterazComposta()!=NULL)      
	{ 
		// Sistema l'alterazione a sinistra della nota.
		
		PAlterazione=PAlterazComposta->GetPAlterazione(PAlterazComposta->Getn_alterazioni());            
		// Imposta la grandezza dell'alterazione.
		PAlterazione->SetNotina(GetNotina());
		
		// Sistema l'alterazione a sinistra della nota, distanziata con
		// SPNOTAALTERAZIONE (con la meta' se la nota e' una notina).
		if(GetNotina())
		{
			PAlterazione->SetPos(this,Point(-GetVU2LftF()-
				SPNOTAALTERAZIONE/2-PAlterazione->GetVU2Rgt(),0));
			SetVU2Lft(GetVU2LftF()+SPNOTAALTERAZIONE/2+
				PAlterazione->GetWidth());
		}
		else
		{
			PAlterazione->SetPos(this,Point(-GetVU2LftF()-SPNOTAALTERAZIONE-
				PAlterazione->GetVU2Rgt(),0));
			SetVU2Lft(GetVU2LftF()+SPNOTAALTERAZIONE+PAlterazione->GetWidth());
		}
		// Aggiorna VU2Up e VU2Dwn.
		if(GetVU2Up()<PAlterazione->GetVU2Up())
			SetVU2Up(PAlterazione->GetVU2Up());  
		if(GetVU2Dwn()<PAlterazione->GetVU2Dwn())
			SetVU2Dwn(PAlterazione->GetVU2Dwn());  
		
		
		if(PAlterazComposta->Getn_alterazioni()==2)
			// Sistema l'altra alterazione.
		{    
			PAlterazione=PAlterazComposta->GetPAlterazione(1);
			
			// Imposta la grandezza dell'alterazione.
			PAlterazione->SetNotina(GetNotina());
			
			// Sistema l'alterazione a sinistra distanziata con SPNOTAALTERAZIONE 
			// (con la met se la nota  una notina).
			if(GetNotina())
			{
				PAlterazione->SetPos(this, Point(-GetVU2Lft()-SPNOTAALTERAZIONE/2- 
                    PAlterazione->GetVU2Rgt(),0));
				SetVU2Lft(GetVU2Lft()+SPNOTAALTERAZIONE/2+PAlterazione->GetWidth());
			}
			else
			{
				PAlterazione->SetPos(this,Point(-GetVU2Lft()-SPNOTAALTERAZIONE-
                    PAlterazione->GetVU2Rgt(),0));
				SetVU2Lft(GetVU2Lft()+SPNOTAALTERAZIONE+PAlterazione->GetWidth());
			}
			// Aggiorna VU2Up e VU2Dwn.
			if(GetVU2Up()<PAlterazione->GetVU2Up())
				SetVU2Up(PAlterazione->GetVU2Up());  
			if(GetVU2Dwn()<PAlterazione->GetVU2Dwn())
				SetVU2Dwn(PAlterazione->GetVU2Dwn());   	    
		}   
	} 
}



void Nota::AdjustEspressComposta(VUnit YLinea1,VUnit YLinea5,Bool onspace,Bool onstaff,
                                 Point point,Espressione *PEspressione)
{
	VUnit dy;
	
	int num,diff;
	int offset;
	
	num = GetAltezza()/2; //quoziente intero
	diff = GetAltezza() - (num*2); //resto della divisione
	
	if (onstaff)
	{
		if (onspace)
		{
			if (diff==0)  offset = 2; //la nota  su una linea
			else offset = -1;
		}
		else
		{
			if (diff==0) offset = 1; // la nota  su una linea
			else offset = 2;
		}
	} 
	else offset = 0;
	
	if(PEspressione->GetID()==CL_TENUTO)
		offset -= 1;   
	
	if(GetPEspressComposta()!=NULL)
	{		   
		if(PEspressione->GetAboveNota())
		{  
			PEspressione->SetPos(this,Point(0+point.x,(GetVU2Up()+
				PEspressione->GetVU2Dwn())+4+offset+point.y));
			
			// Accorgimento per evitare di sistemare l'espressione dentro
			// il pentagramma per simboli che non siano lo staccato o il tenuto.
			
			if(!onstaff)
			{
				dy = YLinea5+3+PEspressione->GetVU2Dwn()-PEspressione->GetAbsPos().y;
				if(dy>0) PEspressione->Move(Point(0,dy));
			}	 
			
			// Aggiorna le VU2Up.
			SetVU2Up(PEspressione->GetAbsPos().y-GetAbsPos().y+
				PEspressione->GetVU2Up());
		}
		else
		{    
			PEspressione->SetPos(this,Point(0+point.x,(-GetVU2Dwn()-
				PEspressione->GetVU2Up())-4-offset-point.y));
			
			// Accorgimento per evitare di sistemare l'espressione dentro
			// il pentagramma.
			if (!onstaff)
			{
				dy = YLinea1-3-PEspressione->GetVU2Up()-PEspressione->GetAbsPos().y;
				if(dy<0) PEspressione->Move(Point(0,dy));
			}   
			
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PEspressione->GetAbsPos().y+GetAbsPos().y+
				PEspressione->GetVU2Dwn());
		}
		if(PEspressione->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PEspressione->GetVU2Lft());
		if(PEspressione->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PEspressione->GetVU2Rgt());
	}
}

void Nota::AdjustKoreanOrnament(VUnit YLinea1,VUnit YLinea5, Point point)
{
	VUnit dy;
	
	if(GetPKoreanOrnament()!=NULL)
	{
		if(GetPKoreanOrnament()->GetAboveNota())
		{
			// L'KoreanOrnament e' sopra la nota.
			PKoreanOrnament->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+
				PKoreanOrnament->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare l'KoreanOrnament dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + PKoreanOrnament->GetVU2Dwn() -
				PKoreanOrnament->GetAbsPos().y;
			if(dy>0) PKoreanOrnament->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PKoreanOrnament->GetAbsPos().y-GetAbsPos().y+
				PKoreanOrnament->GetVU2Up());
		}
		else
		{
			// L'KoreanOrnament e' sotto la nota.
			PKoreanOrnament->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-
				PKoreanOrnament->GetVU2Up()));
			// Accorgimento per evitare di sistemare l'KoreanOrnament dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - PKoreanOrnament->GetVU2Up() -
				PKoreanOrnament->GetAbsPos().y;
			if(dy<0) PKoreanOrnament->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PKoreanOrnament->GetAbsPos().y+GetAbsPos().y+
				PKoreanOrnament->GetVU2Dwn());
		}
		if(PKoreanOrnament->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PKoreanOrnament->GetVU2Lft());
		if(PKoreanOrnament->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PKoreanOrnament->GetVU2Rgt());
	}
}  


void Nota::AdjustAbbellimento(VUnit YLinea1,VUnit YLinea5, Point p)
{                                                                  
	VUnit dy;
	Alterazione *PAlterazione;
	
	if(GetPAbbellimento()!=NULL)
	{                                                                
		if (GetPAbbellimento()->GetID()==CL_TREMOLO) return;
		if (GetPAbbellimento()->GetID()==CL_GLISSATO) return;
		
		if(GetPAbbellimento()->GetAboveNota())
		{
			// L'abbellimento e' sopra la nota.
			PAlterazione=PAbbellimento->GetPAltSotto();
			if(PAlterazione!=NULL)
			{
				//L'abbellimento ha una alterazione sotto.	
				PAlterazione->SetPos(this,Point(p.x+1,GetVU2Up()+2*RECTSPACEY+
					PAlterazione->GetVU2Dwn()+p.y));
				// Accorgimento per evitare di sistemare l'alterazione dentro
				// il pentagramma.
				dy = YLinea5 + 2*RECTSPACEY + PAlterazione->GetVU2Dwn() -
					PAlterazione->GetAbsPos().y;
				if(dy>0) PAlterazione->Move(Point(0,dy));
				// Aggiorna le VU2Up.
				SetVU2Up(PAlterazione->GetAbsPos().y-GetAbsPos().y+
					PAlterazione->GetVU2Up());
			}	 
			
			PAbbellimento->SetPos(this,Point(p.x,GetVU2Up()+2*RECTSPACEY+
				PAbbellimento->GetVU2Dwn()+p.y));
			// Accorgimento per evitare di sistemare l'abbellimento dentro
			// il pentagramma.
			dy = YLinea5 + 2*RECTSPACEY + PAbbellimento->GetVU2Dwn() -
				PAbbellimento->GetAbsPos().y;
			if(dy>0) PAbbellimento->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PAbbellimento->GetAbsPos().y-GetAbsPos().y+
				PAbbellimento->GetVU2Up());
			
			PAlterazione=PAbbellimento->GetPAltSopra();
			if(PAlterazione!=NULL)
			{
				//L'abbellimento ha una alterazione sopra.	
				PAlterazione->SetPos(this,Point(p.x+1,GetVU2Up()+2*RECTSPACEY+
					PAlterazione->GetVU2Dwn()+p.y));
				// Aggiorna le VU2Up.
				SetVU2Up(PAlterazione->GetAbsPos().y-GetAbsPos().y+
					PAlterazione->GetVU2Up());
			} 	  
		}
		else
		{
			// L'abbellimento e' sotto la nota.
			PAlterazione=PAbbellimento->GetPAltSopra();
			if(PAlterazione!=NULL)
			{ 
				// L'abbellimento ha una alterazione sopra.
				PAlterazione->SetPos(this,Point(p.x+1,-GetVU2Dwn()-2*RECTSPACEY-
					PAlterazione->GetVU2Up()+p.y));
				// Accorgimento per evitare di sistemare l'alterazione dentro
				// il pentagramma.
				dy = YLinea1 - 2*RECTSPACEY - PAlterazione->GetVU2Up() -
					PAlterazione->GetAbsPos().y;
				if(dy<0) PAlterazione->Move(Point(0,dy));
				// Aggiorna le VU2Dwn.
				SetVU2Dwn(-PAlterazione->GetAbsPos().y+GetAbsPos().y+
					PAlterazione->GetVU2Dwn());
			}
			
			PAbbellimento->SetPos(this,Point(p.x,-GetVU2Dwn()-2*RECTSPACEY-
				PAbbellimento->GetVU2Up()+p.y));
			// Accorgimento per evitare di sistemare l'abbellimento dentro
			// il pentagramma.
			dy = YLinea1 - 2*RECTSPACEY - PAbbellimento->GetVU2Up() -
				PAbbellimento->GetAbsPos().y;
			if(dy<0) PAbbellimento->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PAbbellimento->GetAbsPos().y+GetAbsPos().y+
				PAbbellimento->GetVU2Dwn());
			
			PAlterazione=PAbbellimento->GetPAltSotto(); 
			if(PAlterazione!=NULL)
			{ 
				// L'abbellimento ha una alterazione sotto.
				PAlterazione->SetPos(this,Point(p.x+1,-GetVU2Dwn()-2*RECTSPACEY-
					PAlterazione->GetVU2Up()));
				// Aggiorna le VU2Dwn.
				SetVU2Dwn(-PAlterazione->GetAbsPos().y+GetAbsPos().y+
					PAlterazione->GetVU2Dwn()+p.y);
			} 
			
		}
		if(PAbbellimento->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PAbbellimento->GetVU2Lft());
		if(PAbbellimento->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PAbbellimento->GetVU2Rgt());         
	}                                                           
}

void Nota::AdjustDiteggiato(VUnit YLinea1,VUnit YLinea5, Point point)
{
	VUnit dy;
	
	if(GetPDiteggiato()!=NULL)
	{
		if(GetPDiteggiato()->GetAboveNota())
		{
			// Il diteggiato e' sopra la nota.
			PDiteggiato->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+
				PDiteggiato->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare il diteggiato dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + PDiteggiato->GetVU2Dwn() -
				PDiteggiato->GetAbsPos().y;
			if(dy>0) PDiteggiato->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PDiteggiato->GetAbsPos().y-GetAbsPos().y+
				PDiteggiato->GetVU2Up());
		}
		else
		{
			// Il diteggiato e' sotto la nota.
			PDiteggiato->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-
				PDiteggiato->GetVU2Up()));
			// Accorgimento per evitare di sistemare il diteggiato dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - PDiteggiato->GetVU2Up() -
				PDiteggiato->GetAbsPos().y;
			if(dy<0) PDiteggiato->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PDiteggiato->GetAbsPos().y+GetAbsPos().y+
				PDiteggiato->GetVU2Dwn());
		}
		if(PDiteggiato->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PDiteggiato->GetVU2Lft());
		if(PDiteggiato->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PDiteggiato->GetVU2Rgt());
	}       
}  

void Nota::AdjustSordina(VUnit YLinea1,VUnit YLinea5, Point point)
{
	VUnit dy;
	
	if(GetPSordina()!=NULL)
	{
		if(GetPSordina()->GetAboveNota())
		{
			// La sordina e' sopra la nota.
			PSordina->SetPos(this,Point(0,GetVU2Up()+2*RECTSPACEY+
				PSordina->GetVU2Dwn()));
			// Accorgimento per evitare di sistemare la sordina dentro
			// il pentagramma.
			dy = YLinea5 + 2*RECTSPACEY + PSordina->GetVU2Dwn() -
				PSordina->GetAbsPos().y;
			if(dy>0) PSordina->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PSordina->GetAbsPos().y-GetAbsPos().y+
				PSordina->GetVU2Up());
		}
		else
		{
			// La sordina e' sotto la nota.
			PSordina->SetPos(this,Point(0,-GetVU2Dwn()-2*RECTSPACEY-
				PSordina->GetVU2Up()));
			// Accorgimento per evitare di sistemare la sordina dentro
			// il pentagramma.
			dy = YLinea1 - 2*RECTSPACEY - PSordina->GetVU2Up() -
				PSordina->GetAbsPos().y;
			if(dy<0) PSordina->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PSordina->GetAbsPos().y+GetAbsPos().y+
				PSordina->GetVU2Dwn());
		}
		if(PSordina->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PSordina->GetVU2Lft());
		if(PSordina->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PSordina->GetVU2Rgt());
	}       
}  

void Nota::AdjustArmonici(VUnit YLinea1,VUnit YLinea5, Point point)
{
	VUnit dy;
	
	if(GetPArmonici()!=NULL)
	{                                                    
		if(GetPArmonici()->GetAboveNota())
		{                                                    
			// Armonici e' sopra la nota.
			PArmonici->SetPos(this,Point(0,GetVU2Up()+2*RECTSPACEY+
				PArmonici->GetVU2Dwn()));       
			// Accorgimento per evitare di sistemare Armonici dentro
			// il pentagramma.
			dy = YLinea5 + 2*RECTSPACEY + PArmonici->GetVU2Dwn() -
				PArmonici->GetAbsPos().y;                         
			if(dy>0) PArmonici->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(PArmonici->GetAbsPos().y-GetAbsPos().y+
				PArmonici->GetVU2Up());                          
		}
		else
		{                                                  
			// Armonici e' sotto la nota.
			PArmonici->SetPos(this,Point(0,-GetVU2Dwn()-2*RECTSPACEY-
				PArmonici->GetVU2Up()));
			// Accorgimento per evitare di sistemare Armonici dentro
			// il pentagramma.
			dy = YLinea1 - 2*RECTSPACEY - PArmonici->GetVU2Up() -
				PArmonici->GetAbsPos().y;
			if(dy<0) PArmonici->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-PArmonici->GetAbsPos().y+GetAbsPos().y+
				PArmonici->GetVU2Dwn());
		}
		if(PArmonici->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(PArmonici->GetVU2Lft());
		if(PArmonici->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(PArmonici->GetVU2Rgt());
	}
}

void Nota::AdjustSuddivisione()
{
	if (GetPSuddivisione()!=NULL)
	{
		if (GetLGambo()>0) // Gambo sopra la nota.
			GetPSuddivisione()->SetPos(this,Point(DELTAXSUDDIVISIONEUP,DELTAYSUDDIVISIONEUP));
		else
			GetPSuddivisione()->SetPos(this,Point(DELTAXSUDDIVISIONEDOWN,DELTAYSUDDIVISIONEDOWN));
	}
}


void Nota::AdjustStrumento(VUnit YLinea1,VUnit YLinea5,Point point)
{
	VUnit dy;
	
	if(GetPStrumento()!=NULL)
	{ 
		if(GetPStrumento()->GetWidth()>0)
		{
			if(GetPStrumento()->GetAboveNota())
			{
				// I segni di strumento sono sopra la nota.
				PStrumento->SetPos(this,Point(0+point.x,
					(GetVU2Up()+RECTSPACEY+PStrumento->GetVU2Dwn())+point.y));
				
				// Accorgimento per evitare di sistemare i segni di strumento dentro
				// il pentagramma.
				dy = YLinea5 + RECTSPACEY + PStrumento->GetVU2Dwn() - PStrumento->GetAbsPos().y;
				if(dy>0) PStrumento->Move(Point(0,dy));
				// Aggiorna le VU2Up.
				SetVU2Up(PStrumento->GetAbsPos().y-GetAbsPos().y + PStrumento->GetVU2Up());
			}
			else
			{
				// I segni di strumento sono sotto la nota.
				PStrumento->SetPos(this,Point(0+point.x,
					(-GetVU2Dwn()-RECTSPACEY-PStrumento->GetVU2Up())+point.y));
				// Accorgimento per evitare di sistemare i segni di strumento dentro
				// il pentagramma.
				dy = YLinea1 - RECTSPACEY - PStrumento->GetVU2Up() -
					PStrumento->GetAbsPos().y;
				if(dy<0) PStrumento->Move(Point(0,dy));
				// Aggiorna le VU2Dwn.
				SetVU2Dwn(-PStrumento->GetAbsPos().y+GetAbsPos().y+
					PStrumento->GetVU2Dwn());
			}
			if(PStrumento->GetVU2Lft()>GetVU2Lft())
				SetVU2Lft(PStrumento->GetVU2Lft());
			if(PStrumento->GetVU2Rgt()>GetVU2Rgt())
				SetVU2Rgt(PStrumento->GetVU2Rgt());
		}
	}
}


void Nota::AdjustCorda(VUnit YLinea1,VUnit YLinea5,Point point)
{
	Violino* pV = (Violino*)GetPStrumento();  
	VUnit dy;
	
	if(pV->GetPCorda()!=NULL)
	{
		if(pV->GetPCorda()->GetAboveNota())
		{
			// Il segno della corda e' sopra la nota.
			pV->GetPCorda()->SetPos(this,Point(-3+point.x,
				(GetVU2Up()+pV->GetPCorda()->GetVU2Dwn()+RECTSPACEY)+point.y));
			
			// Accorgimento per evitare di sistemare i segni di strumento dentro il pentagramma.
			dy = YLinea5 + RECTSPACEY + pV->GetPCorda()->GetVU2Dwn() - pV->GetPCorda()->GetAbsPos().y;
			if(dy>0) pV->GetPCorda()->Move(Point(0,dy));
			
			// Aggiorna le VU2Up.
			SetVU2Up(pV->GetPCorda()->GetAbsPos().y-GetAbsPos().y+pV->GetPCorda()->GetVU2Up());
		}
		else
		{
			// Il segno della corda e' sotto la nota.
			pV->GetPCorda()->SetPos(this,Point(-3+point.x,
				(-GetVU2Dwn()-pV->GetPCorda()->GetVU2Up()-RECTSPACEY)+point.y));
			
			// Accorgimento per evitare di sistemare i segni di strumento dentro il pentagramma.
			dy = YLinea1 - RECTSPACEY - pV->GetPCorda()->GetVU2Up() - pV->GetPCorda()->GetAbsPos().y;
			if(dy<0) pV->GetPCorda()->Move(Point(0,dy));
			
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-pV->GetPCorda()->GetAbsPos().y+GetAbsPos().y+pV->GetPCorda()->GetVU2Dwn());
		}
		if(pV->GetPCorda()->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(pV->GetPCorda()->GetVU2Lft());
		if(pV->GetPCorda()->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(pV->GetPCorda()->GetVU2Rgt());
	}
}



void Nota::AdjustParteArco(VUnit YLinea1,VUnit YLinea5,Point point)
{
	Arco* pA = (Arco*)GetPStrumento();  
	VUnit dy;
	
	if(pA->GetPParteArco()!=NULL)
	{
		if(pA->GetPParteArco()->GetAboveNota())
		{
			// Il segno del ParteArco e' sopra la nota.
			pA->GetPParteArco()->SetPos(this,Point(0+point.x,
				(GetVU2Up()+pA->GetPParteArco()->GetVU2Dwn()+RECTSPACEY)+point.y));
			
			// Accorgimento per evitare di sistemare i segni di strumento dentro il pentagramma.
			dy = YLinea5 + RECTSPACEY + pA->GetPParteArco()->GetVU2Dwn() - pA->GetPParteArco()->GetAbsPos().y;
			if(dy>0) pA->GetPParteArco()->Move(Point(0,dy));
			
			// Aggiorna le VU2Up.
			SetVU2Up(pA->GetPParteArco()->GetAbsPos().y-GetAbsPos().y+pA->GetPParteArco()->GetVU2Up());
		}
		else
		{
			// Il segno di ParteArco e' sotto la nota.
			pA->GetPParteArco()->SetPos(this,Point(0+point.x,
				(-GetVU2Dwn()-pA->GetPParteArco()->GetVU2Up()-RECTSPACEY)+point.y));
			
			// Accorgimento per evitare di sistemare i segni di strumento dentro il pentagramma.
			dy = YLinea1 - RECTSPACEY - pA->GetPParteArco()->GetVU2Up() - pA->GetPParteArco()->GetAbsPos().y;
			if(dy<0) pA->GetPParteArco()->Move(Point(0,dy));
			
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-pA->GetPParteArco()->GetAbsPos().y+GetAbsPos().y+pA->GetPParteArco()->GetVU2Dwn());
		}
		if(pA->GetPParteArco()->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(pA->GetPParteArco()->GetVU2Lft());
		if(pA->GetPParteArco()->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(pA->GetPParteArco()->GetVU2Rgt());     
	}
}  


void Nota::AdjustPosizArco(VUnit YLinea1,VUnit YLinea5,Point point)
{
	Arco* pA = (Arco*)GetPStrumento();  
	VUnit dy;
	
	if((pA->GetPPosizArco())!=NULL)
	{
		if((pA->GetPPosizArco())->GetAboveNota())
		{
			// Il segno di PosizArco e' sopra. 
			(pA->GetPPosizArco())->SetPos(this,Point(0+point.x,
				(GetVU2Up()+(pA->GetPPosizArco())->GetVU2Dwn()+RECTSPACEY)+point.y));
			
			// Accorgimento per evitare di sistemare i segni di strumento dentro il pentagramma.
			dy = YLinea5 + RECTSPACEY + (pA->GetPPosizArco())->GetVU2Dwn() - (pA->GetPPosizArco())->GetAbsPos().y;
			if(dy>0) (pA->GetPPosizArco())->Move(Point(0,dy));
			
			// Aggiorna le VU2Up.
			SetVU2Up((pA->GetPPosizArco())->GetAbsPos().y-GetAbsPos().y+(pA->GetPPosizArco())->GetVU2Up());
		}
		else
		{
			// Il segno di PosizArco e' sotto.
			(pA->GetPPosizArco())->SetPos(this,Point(0+point.x,
				(-GetVU2Dwn()-(pA->GetPPosizArco())->GetVU2Up()-RECTSPACEY)+point.y));
			
			// Accorgimento per evitare di sistemare i segni di strumento dentro il pentagramma.
			dy = YLinea1 - RECTSPACEY - (pA->GetPPosizArco())->GetVU2Up() - (pA->GetPPosizArco())->GetAbsPos().y;
			if(dy<0) (pA->GetPPosizArco())->Move(Point(0,dy));
			
			// Aggiorna le VU2Dwn.  
			SetVU2Dwn(-(pA->GetPPosizArco())->GetAbsPos().y+GetAbsPos().y+(pA->GetPPosizArco())->GetVU2Dwn());                 
		}
		if((pA->GetPPosizArco())->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft((pA->GetPPosizArco())->GetVU2Lft());
		if((pA->GetPPosizArco())->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt((pA->GetPPosizArco())->GetVU2Rgt());
	}                                      
}



void Nota::AdjustDirezArco(VUnit YLinea1,VUnit YLinea5,Point point)
{
	Arco* pA = (Arco*)GetPStrumento();  
	VUnit dy;
	
	if(pA->GetPDirezArco()!=NULL)
	{
		if(pA->GetPDirezArco()->GetAboveNota())
		{
			// Il segno di DirezArco e' sopra. 
			pA->GetPDirezArco()->SetPos(this,Point(0+point.x,
				(GetVU2Up()+pA->GetPDirezArco()->GetVU2Dwn()+RECTSPACEY)+point.y));
			
			// Accorgimento per evitare di sistemare i segni di strumento dentro il pentagramma.
			dy = YLinea5 + RECTSPACEY + pA->GetPDirezArco()->GetVU2Dwn() - pA->GetPDirezArco()->GetAbsPos().y;
			if(dy>0) pA->GetPDirezArco()->Move(Point(0,dy));
			
			// Aggiorna le VU2Up.
			SetVU2Up(pA->GetPDirezArco()->GetAbsPos().y-GetAbsPos().y+pA->GetPDirezArco()->GetVU2Up());
		}
		else
		{
			// Il segno di DirezArco e' sotto.
			pA->GetPDirezArco()->SetPos(this,Point(0+point.x,
				(-GetVU2Dwn()-pA->GetPDirezArco()->GetVU2Up()-RECTSPACEY)+point.y));
			
			// Accorgimento per evitare di sistemare i segni di strumento dentro il pentagramma.
			dy = YLinea1 - RECTSPACEY - pA->GetPDirezArco()->GetVU2Up() - pA->GetPDirezArco()->GetAbsPos().y;
			if(dy<0) pA->GetPDirezArco()->Move(Point(0,dy));
			
			// Aggiorna le VU2Dwn.  
			SetVU2Dwn(-pA->GetPDirezArco()->GetAbsPos().y+GetAbsPos().y+pA->GetPDirezArco()->GetVU2Dwn());                 
		}
		if(pA->GetPDirezArco()->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(pA->GetPDirezArco()->GetVU2Lft());
		if(pA->GetPDirezArco()->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(pA->GetPDirezArco()->GetVU2Rgt());
	}                                      
}



CmdResult Nota::CmdNotaDiesis()
{                            
	if(GetPAlterazComposta()==NULL)
	{
		SetPAlterazComposta(new AlterazComposta);
		if(GetPAlterazComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaDiesis()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPAlterazComposta()->InserisciAlt(CL_DIESIS))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}


CmdResult Nota::CmdNotaDiesis1Q()
{                            
	if(GetPAlterazComposta()==NULL)
	{
		SetPAlterazComposta(new AlterazComposta);
		if(GetPAlterazComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaDiesis1Q()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPAlterazComposta()->InserisciAlt(CL_DIESIS1Q))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}

CmdResult Nota::CmdNotaDiesis3Q()
{                            
	if(GetPAlterazComposta()==NULL)
	{
		SetPAlterazComposta(new AlterazComposta);
		if(GetPAlterazComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaDiesis3Q()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPAlterazComposta()->InserisciAlt(CL_DIESIS3Q))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}

CmdResult Nota::CmdNotaBemolle()
{                            
	if(GetPAlterazComposta()==NULL)
	{
		SetPAlterazComposta(new AlterazComposta);
		if(GetPAlterazComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaBemolle()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPAlterazComposta()->InserisciAlt(CL_BEMOLLE))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}

CmdResult Nota::CmdNotaBemolle1Q()
{                            
	if(GetPAlterazComposta()==NULL)
	{
		SetPAlterazComposta(new AlterazComposta);
		if(GetPAlterazComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaBemolle1Q()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPAlterazComposta()->InserisciAlt(CL_BEMOLLE1Q))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}

CmdResult Nota::CmdNotaBemolle3Q()
{                            
	if(GetPAlterazComposta()==NULL)
	{
		SetPAlterazComposta(new AlterazComposta);
		if(GetPAlterazComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaBemolle3Q()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPAlterazComposta()->InserisciAlt(CL_BEMOLLE3Q))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}

CmdResult Nota::CmdNotaBequadro()
{                            
	if(GetPAlterazComposta()==NULL)
	{
		SetPAlterazComposta(new AlterazComposta);
		if(GetPAlterazComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaBequadro()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPAlterazComposta()->InserisciAlt(CL_BEQUADRO))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}

CmdResult Nota::CmdNotaBequadro1Q()
{                            
	if(GetPAlterazComposta()==NULL)
	{
		SetPAlterazComposta(new AlterazComposta);
		if(GetPAlterazComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaBequadro1Q()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPAlterazComposta()->InserisciAlt(CL_BEQUADRO1Q))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}

CmdResult Nota::CmdNotaBequadro3Q()
{                            
	if(GetPAlterazComposta()==NULL)
	{
		SetPAlterazComposta(new AlterazComposta);
		if(GetPAlterazComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaBequadro3Q()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPAlterazComposta()->InserisciAlt(CL_BEQUADRO3Q))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}
//


CmdResult Nota::CmdNota2Diesis()
{                            
	if(GetPAlterazComposta()==NULL)
	{
		SetPAlterazComposta(new AlterazComposta);
		if(GetPAlterazComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNota2Diesis()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPAlterazComposta()->InserisciAlt(CL_DOPPIODIESIS))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}

CmdResult Nota::CmdNota2Bemolle()
{                            
	if(GetPAlterazComposta()==NULL)
	{
		SetPAlterazComposta(new AlterazComposta);
		if(GetPAlterazComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNota2Bemolle()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPAlterazComposta()->InserisciAlt(CL_DOPPIOBEMOLLE))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}


CmdResult Nota::CmdNotaTrillo(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta,int NOnde)
{                            
	if(GetPAbbellimento()!=NULL)
		delete GetPAbbellimento();
	SetPAbbellimento(new Trillo);
	if(GetPAbbellimento()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaTrillo()");  
		return(CMD_FAIL);
	};
    GetPAbbellimento()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	((Trillo*)GetPAbbellimento())->SetNumTrilli(NOnde);
	return(CMD_OK);
}               


CmdResult Nota::CmdNotaGruppettoSup1(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPAbbellimento()!=NULL)
		delete GetPAbbellimento();
	SetPAbbellimento(new GSuperiore);
	if(GetPAbbellimento()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaGruppettoSup1()");  
		return(CMD_FAIL);
	};
	GetPAbbellimento()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	return(CMD_OK);
}


CmdResult Nota::CmdNotaTurnSlash(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPAbbellimento()!=NULL)
		delete GetPAbbellimento();
	SetPAbbellimento(new GSlash);
	if(GetPAbbellimento()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaTurnSlash()");  
		return(CMD_FAIL);
	};
	GetPAbbellimento()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	return(CMD_OK);
}

CmdResult Nota::CmdNotaTurnUp(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPAbbellimento()!=NULL)
		delete GetPAbbellimento();
	SetPAbbellimento(new GUp);
	if(GetPAbbellimento()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaTurnUp()");  
		return(CMD_FAIL);
	};
	GetPAbbellimento()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	return(CMD_OK);
}

//

CmdResult Nota::CmdNotaGruppettoInf1(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPAbbellimento()!=NULL)
		delete GetPAbbellimento();
	SetPAbbellimento(new GInferiore);
	if(GetPAbbellimento()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaGruppettoInf1()");  
		return(CMD_FAIL);
	};
	GetPAbbellimento()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	return(CMD_OK);
}                    


CmdResult Nota::CmdNotaMordenteSup(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPAbbellimento()!=NULL)
		delete GetPAbbellimento();
	SetPAbbellimento(new MSuperiore);
	if(GetPAbbellimento()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaMordenteSup()");  
		return(CMD_FAIL);
	};
	
	GetPAbbellimento()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	return(CMD_OK);
}

CmdResult Nota::CmdNotaMordente2Sup(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPAbbellimento()!=NULL)
		delete GetPAbbellimento();
	SetPAbbellimento(new MSuperiore);
	if(GetPAbbellimento()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaMordenteSup()");  
		return(CMD_FAIL);
	};
	
	GetPAbbellimento()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	GetPAbbellimento()->SetDoppio(TRUE);
	return(CMD_OK);
}

CmdResult Nota::CmdNotaMordenteInf(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPAbbellimento()!=NULL)
		delete GetPAbbellimento();
	SetPAbbellimento(new MInferiore);
	if(GetPAbbellimento()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaMordenteInf()");  
		return(CMD_FAIL);
	};
	GetPAbbellimento()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	return(CMD_OK);
}

CmdResult Nota::CmdNotaMordente2Inf(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPAbbellimento()!=NULL)
		delete GetPAbbellimento();
	SetPAbbellimento(new MInferiore);
	if(GetPAbbellimento()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaMordenteInf()");  
		return(CMD_FAIL);
	};
	GetPAbbellimento()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	GetPAbbellimento()->SetDoppio(TRUE);
	return(CMD_OK);
}

CmdResult Nota::CmdNotaSlidingUp(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPKoreanOrnament()!=NULL)
					   delete GetPKoreanOrnament();
	SetPKoreanOrnament(new SlidingUp);
	if(GetPKoreanOrnament()==NULL)
	{
					   EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaSlidingUp()");  
					   return(CMD_FAIL);
	};
	GetPKoreanOrnament()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	return(CMD_OK);
}

CmdResult Nota::CmdNotaSlidingDown(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPKoreanOrnament()!=NULL)
					   delete GetPKoreanOrnament();
	SetPKoreanOrnament(new SlidingDown);
	if(GetPKoreanOrnament()==NULL)
	{
					   EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaSlidingDown()");  
					   return(CMD_FAIL);
	};
	GetPKoreanOrnament()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	return(CMD_OK);
}

CmdResult Nota::CmdNotaSlidingDUp(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPKoreanOrnament()!=NULL)
					   delete GetPKoreanOrnament();
	SetPKoreanOrnament(new SlidingDUp);
	if(GetPKoreanOrnament()==NULL)
	{
					   EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaSlidingDUp()");  
					   return(CMD_FAIL);
	};
	GetPKoreanOrnament()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
		return(CMD_OK);
}

CmdResult Nota::CmdNotaSlidingDownUp(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPKoreanOrnament()!=NULL)
					   delete GetPKoreanOrnament();
	SetPKoreanOrnament(new SlidingDownUp);
	if(GetPKoreanOrnament()==NULL)
	{
					   EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaSlidingDownUp()");  
					   return(CMD_FAIL);
	};
	GetPKoreanOrnament()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	return(CMD_OK);
}

CmdResult Nota::CmdNotaNonghynInc(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPKoreanOrnament()!=NULL)
					   delete GetPKoreanOrnament();
	SetPKoreanOrnament(new NonghynInc);
	if(GetPKoreanOrnament()==NULL)
	{
					   EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaNonghynInc()");  
					   return(CMD_FAIL);
	};
	GetPKoreanOrnament()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	return(CMD_OK);
}

CmdResult Nota::CmdNotaNonghynDec(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPKoreanOrnament()!=NULL)
					   delete GetPKoreanOrnament();
	SetPKoreanOrnament(new NonghynDec);
	if(GetPKoreanOrnament()==NULL)
	{
					   EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaNonghynDec()");  
					   return(CMD_FAIL);
	};
	GetPKoreanOrnament()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	return(CMD_OK);
}

CmdResult Nota::CmdNotaNonghynArrow(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPKoreanOrnament()!=NULL)
					   delete GetPKoreanOrnament();
	SetPKoreanOrnament(new NonghynArrow);
	if(GetPKoreanOrnament()==NULL)
	{
					   EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaNonghynArrow()");  
					   return(CMD_FAIL);
	};
	GetPKoreanOrnament()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	return(CMD_OK);
}

CmdResult Nota::CmdNotaNonghynWide(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPKoreanOrnament()!=NULL)
					   delete GetPKoreanOrnament();
	SetPKoreanOrnament(new NonghynWide);
	if(GetPKoreanOrnament()==NULL)
	{
					   EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaNonghynWide()");  
					   return(CMD_FAIL);
	};
	GetPKoreanOrnament()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	return(CMD_OK);
}

CmdResult Nota::CmdNotaNota2Notina()
{                    
	SetNotina(TRUE);
	//Assegno lo status GRACED.
	SetStatus(GRACED);
	// Adatta il gambo.    
	if(GetLGambo()>0)
		SetGamboUp();    
	if(GetLGambo()<0)
		SetGamboDwn();
	// Ci pensera' Adjust a far diventare piccola l'alterazione.
	return(CMD_OK);
}


CmdResult Nota::CmdNotaNotina2Nota(TipoInserimento tpIns, Nota *pNota, Battuta
								   *pBattuta)
{   
	Bool gamboupdwn;
	int lgambo;
	int layer=0;
	
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		
		msgdebug.sprintf("");
		
	}
	
	
	
	
	
	if(!GetNotina())
		return(CMD_FAIL);
	// Accorgimento per evitare di ingrandire una nota dentro
	// un gruppo.
	if(!GetCoda() && (GetID()==CL_NCROMA || GetID()==CL_NSEMICROMA ||
		GetID()==CL_NBISCROMA || GetID()==CL_NSEMIBISCROMA) || GetID()==CL_NFUSA)
		return(CMD_FAIL);
	SetNotina(FALSE);
	SetStatus(NORMAL);
	
	switch(tpIns)
	{
    case AUTOMATICO:
		
		
		if (immissione.ImmRule(GetID(), this, pBattuta, NULL, layer))  
		{ 
			gamboupdwn = TRUE;
			lgambo = int(posizionam.PosRule((DrawObject*)this, this, pBattuta,
				gamboupdwn, layer));
			SetLGambo(lgambo);
		}	  
		else 
		{  
			gamboupdwn = FALSE;
			lgambo = int(posizionam.PosRule((DrawObject*)this, this, pBattuta,
				gamboupdwn, layer));
			SetLGambo(-lgambo);
		}	 
		
		
		break;  
    case SOPRA:
		gamboupdwn = TRUE;
		lgambo = int(posizionam.PosRule((DrawObject*)this, this, pBattuta,
			gamboupdwn, layer));
		SetLGambo(lgambo);
		break;
    case SOTTO:
		gamboupdwn = FALSE;
		lgambo = int(posizionam.PosRule((DrawObject*)this, this, pBattuta,
			gamboupdwn, layer));
		SetLGambo(-lgambo);	
		break;
	} 
	
	
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		if (this->GetTraceMilla())
			posizionam.TestTokenCond(this, (DrawObject*)this, pBattuta, gamboupdwn, layer);
		
		WDebugMilla(); 
		
	}
	
	
	
	return(CMD_OK);
}               


CmdResult Nota::CmdNotaDito(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta,int NDito, int tipoDito)
{                            
	if(GetPDiteggiato()!=NULL)
		delete GetPDiteggiato();
	SetPDiteggiato(new Diteggiato);
	if(GetPDiteggiato()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaDito()");  
		return(CMD_FAIL);
	};
	
	GetPDiteggiato()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	GetPDiteggiato()->SetDito(NDito);
	GetPDiteggiato()->SetTipoDito(tipoDito);
	return(CMD_OK);
}



CmdResult Nota::CmdNotaTenuto(TipoInserimento tpIns, Nota* pNota, Battuta*
							  pBattuta)
{   
	if(GetPEspressComposta()==NULL)
	{
		SetPEspressComposta(new EspressComposta);
		if(GetPEspressComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaTenuto()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPEspressComposta()->InserisciEsp(CL_TENUTO, tpIns, pNota, pBattuta))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}

CmdResult Nota::CmdNotaPuntoAll(TipoInserimento tpIns, Nota* pNota, Battuta*
								pBattuta)
{                            
	if(GetPEspressComposta()==NULL)
	{
		SetPEspressComposta(new EspressComposta);
		if(GetPEspressComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaPuntoAll()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPEspressComposta()->InserisciEsp(CL_PUNTOALLUNGATO, tpIns, pNota,
		pBattuta))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}

CmdResult Nota::CmdNotaStaccato(TipoInserimento tpIns, Nota* pNota, Battuta*
								pBattuta)
{                            
	if(GetPEspressComposta()==NULL)
	{
		SetPEspressComposta(new EspressComposta);
		if(GetPEspressComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaStaccato()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPEspressComposta()->InserisciEsp(CL_STACCATO, tpIns, pNota, pBattuta))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}

CmdResult Nota::CmdNotaEspGeneric(int IDsym, TipoInserimento tpIns, Nota* pNota, Battuta*
								  pBattuta)
{                            
	if(GetPEspressComposta()==NULL)
	{
		SetPEspressComposta(new EspressComposta);
		if(GetPEspressComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaEspGeneric()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPEspressComposta()->InserisciEsp(CL_ESPGENERIC, tpIns, pNota, pBattuta, IDsym))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}
CmdResult Nota::CmdNotaMartellato(TipoInserimento tpIns, Nota* pNota, Battuta*
								  pBattuta)
{                            
	if(GetPEspressComposta()==NULL)
	{
		SetPEspressComposta(new EspressComposta);
		if(GetPEspressComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaMartellato()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPEspressComposta()->InserisciEsp(CL_MARTELLATO, tpIns, pNota, pBattuta))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}

CmdResult Nota::CmdNotaAccento(TipoInserimento tpIns, Nota* pNota, Battuta*
							   pBattuta)
{                            
	if(GetPEspressComposta()==NULL)
	{
		SetPEspressComposta(new EspressComposta);
		if(GetPEspressComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaAccento()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPEspressComposta()->InserisciEsp(CL_ACCENTO, tpIns, pNota, pBattuta))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}

CmdResult Nota::CmdNotaSforzato(TipoInserimento tpIns, Nota* pNota, Battuta*
								pBattuta)
{                            
	if(GetPEspressComposta()==NULL)
	{
		SetPEspressComposta(new EspressComposta);
		if(GetPEspressComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaSforzato()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPEspressComposta()->InserisciEsp(CL_SFORZATO, tpIns, pNota, pBattuta))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}

CmdResult Nota::CmdNotaMartDolce(TipoInserimento tpIns, Nota* pNota, Battuta*
								 pBattuta)
{                            
	if(GetPEspressComposta()==NULL)
	{
		SetPEspressComposta(new EspressComposta);
		if(GetPEspressComposta()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaMartDolce()");  
			return(CMD_FAIL);
		};
	} 
	if (GetPEspressComposta()->InserisciEsp(CL_MARTDOLCE, tpIns, pNota, pBattuta))
		return(CMD_OK);
	else 
		return (CMD_FAIL); 
}               


CmdResult Nota::CmdNotaConSordinaOttoni(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPSordina()!=NULL)
		delete GetPSordina();
	SetPSordina(new ConSord(CL_CONSORDINAOTTONI));
	if(GetPSordina()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaConSordinaOttoni()");  
		return(CMD_FAIL);
	};
	GetPSordina()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	
	return(CMD_OK);
}

CmdResult Nota::CmdNotaConSordinaArchi(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPSordina()!=NULL)
		delete GetPSordina();
	SetPSordina(new ConSord(CL_CONSORDINAARCHI));
	if(GetPSordina()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaConSordinaArchi()");  
		return(CMD_FAIL);
	};
	GetPSordina()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	
	return(CMD_OK);
}

CmdResult Nota::CmdNotaConSordinaTesto(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPSordina()!=NULL)
		delete GetPSordina();
	SetPSordina(new ConSord(CL_CONSORDINATESTO));
	if(GetPSordina()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaConSordinaTesto()");  
		return(CMD_FAIL);
	};
	GetPSordina()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	return(CMD_OK);
}

CmdResult Nota::CmdNotaViaSordinaArchi(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPSordina()!=NULL)
		delete GetPSordina();
	SetPSordina(new ViaSord(CL_VIASORDINAARCHI));
	if(GetPSordina()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaViaSordinaArchi()");  
		return(CMD_FAIL);
	};
	GetPSordina()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	
	return(CMD_OK);
}

CmdResult Nota::CmdNotaViaSordinaOttoni(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPSordina()!=NULL)
		delete GetPSordina();
	SetPSordina(new ViaSord(CL_VIASORDINAOTTONI));
	if(GetPSordina()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaViaSordinaOttoni()");  
		return(CMD_FAIL);
	};
	GetPSordina()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	
	return(CMD_OK);
}

CmdResult Nota::CmdNotaViaSordinaTesto(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPSordina()!=NULL)
		delete GetPSordina();
	SetPSordina(new ViaSord(CL_VIASORDINATESTO));
	if(GetPSordina()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaViaSordinaTesto()");  
		return(CMD_FAIL);
	};
	GetPSordina()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	
	return(CMD_OK);
}

CmdResult Nota::CmdNotaArmoniciOttoni(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPArmonici()!=NULL)
		delete GetPArmonici();
	SetPArmonici(new Armonici(CL_ARMONICIOTTONI));
	if(GetPArmonici()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaArmoniciOttoni()");  
		return(CMD_FAIL);
	};
	
	GetPArmonici()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	
	return(CMD_OK);
}

CmdResult Nota::CmdNotaArmoniciArchi(TipoInserimento tpIns,Nota* pNota, Battuta* pBattuta)
{                            
	if(GetPArmonici()!=NULL)
		delete GetPArmonici();
	SetPArmonici(new Armonici(CL_ARMONICIARCHI));
	if(GetPArmonici()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaArmoniciArchi()");  
		return(CMD_FAIL);
	};
	GetPArmonici()->SetAboveNotaAuto(tpIns,pNota,pBattuta);
	return(CMD_OK);
}


CmdResult Nota::CmdNotaSuddivisione(short nt)
{    
	
	if(GetPSuddivisione()!=NULL)
	{
		if ((GetPSuddivisione()->GetNumTratti())==nt)
			return(CMD_FAIL);
		else
			delete GetPSuddivisione();
	}
	SetPSuddivisione(new Suddivisione);
	if(GetPSuddivisione()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaSuddivisione()");  
		return(CMD_FAIL);
	};
	GetPSuddivisione()->SetNumTratti(nt);
	return(CMD_OK);
}


CmdResult Nota::CmdNotaPizzicato(TipoInserimento tpIns,
								 Nota* pNota, Battuta* pBattuta)
{
	if(GetPStrumento()!=NULL)
	{
		if ((GetPStrumento()->GetID() == CL_PIZZICATO))    
			return(CMD_FAIL);
		else
			delete GetPStrumento();
	}
	SetPStrumento(new Pizzicato);
	if(GetPStrumento()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaPizzicato()");  
		return(CMD_FAIL);
	};
    switch (tpIns)
	{
	case SOPRA:
		GetPStrumento()->SetAboveNota(TRUE);
		break;
	case SOTTO:
		GetPStrumento()->SetAboveNota(FALSE);
		break;
	case AUTOMATICO:
		GetPStrumento()->SetAboveNotaAuto(CL_PIZZICATO, pNota, pBattuta);
		break;
	}  
	return(CMD_OK);
}

CmdResult Nota::CmdNotaStrArco(SymCmd SCmd, TipoInserimento tpIns, Nota* pNota,
							   Battuta* pBattuta)
{
	if(GetPStrumento()==NULL)
	{
		SetPStrumento(new Arco);
		if(GetPStrumento()==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaStrArco()");  
			return(CMD_FAIL);
		}; 
	}
	else
	{
		if (GetPStrumento()->GetID() != CL_ARCO)
		{
			delete GetPStrumento();
			SetPStrumento(new Arco);
			if(GetPStrumento()==NULL)
			{
				EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaStrArco()");  
				return(CMD_FAIL);
			}; 
		}
	}
	return ((Arco*)PStrumento)->SymCommand(SCmd, tpIns, this, pBattuta);
}    

CmdResult Nota::CmdNotaCorda(SymCmd SCmd, TipoInserimento
							 tpIns, Nota* pNota, Battuta* pBattuta)
{
	if(GetPStrumento()==NULL)
		SetPStrumento(new Arco);
    
	if(GetPStrumento()==NULL) 
	{   
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaCorda()");
		return(CMD_FAIL);
	};   
	
	if((GetPStrumento()->GetID()!=CL_PIZZICATO) &&
		(GetPStrumento()->GetID()!=CL_ARCO))
		return(CMD_FAIL);
	return ((Arco*)PStrumento)->SymCommand(SCmd, tpIns, this, pBattuta);
}


CmdResult Nota::CmdNotaTremolo(short nb)
{
	if(GetPAbbellimento()!=NULL)
	{
		if (GetPAbbellimento()->GetID() == CL_TREMOLO)
			if (((Tremolo*)GetPAbbellimento())->GetNumBarre() == nb)
				return(CMD_FAIL);
			delete GetPAbbellimento();
	}
	SetPAbbellimento(new Tremolo);
	if(GetPAbbellimento()==NULL)
	{   
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaTremolo()");     
		return(CMD_FAIL);
	};
	((Tremolo*)GetPAbbellimento())->SetNumBarre(nb);
	return(CMD_OK);
}


CmdResult Nota::CmdNotaGlissato()
{
	if(GetPAbbellimento()!=NULL)
		delete GetPAbbellimento();
	SetPAbbellimento(new Glissato);
	if(GetPAbbellimento()==NULL)
	{   
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaGlissato()");     
		return(CMD_FAIL);
	};
	return(CMD_OK);
}

CmdResult Nota::CmdNotaGlissonda()
{
	if(GetPAbbellimento()!=NULL)
		delete GetPAbbellimento();
	SetPAbbellimento(new Glissato(TRUE));
	if(GetPAbbellimento()==NULL)
	{   
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Nota","CmdNotaGlissonda()");     
		return(CMD_FAIL);
	};
	return(CMD_OK);
}

DrawObject * Nota::GetAggiunta(SymCmd * MyCmd) 
{
	DrawObject *Appo;
	Appo=Figura::GetAggiunta(MyCmd);
	if (Appo!=NULL) return Appo;
	
	switch(MyCmd->CommandId)
	{
	case CMD_NOTA_PIZZICATO: 
    case CMD_NOTA_ARCO:
    case CMD_ARCO_ARCOSU:
    case CMD_ARCO_ARCOGIU:
    case CMD_ARCO_PONTICELLO:
    case CMD_ARCO_TASTIERA:
    case CMD_ARCO_PUNTA:
    case CMD_ARCO_TALLONE: 
    case CMD_NOTA_CORDA_0:
    case CMD_NOTA_CORDA_1:
    case CMD_NOTA_CORDA_2:
    case CMD_NOTA_CORDA_3:
    case CMD_NOTA_CORDA_4:
    case CMD_NOTA_CORDA_5:
    case CMD_NOTA_CORDA_6:
	case CMD_NOTA_CORDA_7:
		if(GetPStrumento()!=NULL)  //PB
			return(GetPStrumento()->GetObject(MyCmd));
		else
			return NULL;
		break;  
    case CMD_NOTA_DIESIS: 
	case CMD_NOTA_DIESIS1Q:
	case CMD_NOTA_DIESIS3Q:
	case CMD_NOTA_BEMOLLE: 
	case CMD_NOTA_BEMOLLE1Q:
	case CMD_NOTA_BEMOLLE3Q:
	case CMD_NOTA_BEQUADRO: 
	case CMD_NOTA_2DIESIS:
	case CMD_NOTA_2BEMOLLE:
	case CMD_NOTA_BEQUADRO1Q: 
	case CMD_NOTA_BEQUADRO3Q: 
		{
			AlterazComposta *MyAlt=GetPAlterazComposta(); 
			if (MyAlt!=NULL) 
				return MyAlt->GetPAlterazione(MyCmd->CommandId);
			else 
				return NULL;
			
		}
		break;     
    case CMD_NOTA_TRILLO:       
    case CMD_NOTA_GRUPPETTO_SUP_1:
    case CMD_NOTA_GRUPPETTO_INF_1:
    case CMD_NOTA_TURN_SLASH:
    case CMD_NOTA_TURN_UP:
    case CMD_NOTA_MORDENTE_SUP:
    case CMD_NOTA_MORDENTE_INF: 
	case CMD_NOTA_MORDENTE2_SUP:
    case CMD_NOTA_MORDENTE2_INF: 
	case CMD_NOTA_TREMOLO: return((DrawObject *)(GetPAbbellimento())); break;  
	case CMD_NOTA_NOTA2NOTINA: 
    case CMD_NOTA_NOTINA2NOTA: return((DrawObject *)(this)); break;  
		
	case CMD_NOTA_SLIDING_UP:
	case CMD_NOTA_SLIDING_DOWN:
	case CMD_NOTA_SLIDING_DUP:
	case CMD_NOTA_SLIDING_DOWNUP:
	case CMD_NOTA_NONGHYN_INC:
	case CMD_NOTA_NONGHYN_DEC:
	case CMD_NOTA_NONGHYN_ARROW:
	case CMD_NOTA_NONGHYN_WIDE:return((DrawObject *)(GetPKoreanOrnament())); break;
		
    case CMD_NOTA_DITO_0:  
    case CMD_NOTA_DITO_1:  
    case CMD_NOTA_DITO_2:
    case CMD_NOTA_DITO_3:
    case CMD_NOTA_DITO_4:
    case CMD_NOTA_DITO_5:  
		return((DrawObject *)(GetPDiteggiato())); break;  	
		
    case CMD_NOTA_ARMONICIOTTONI:
    case CMD_NOTA_ARMONICIARCHI:
		return (DrawObject *)GetPArmonici();
		break;    
		
	case CMD_NOTA_TENUTO:
    case CMD_NOTA_PUNTOALL:
    case CMD_NOTA_SFORZATO:
    case CMD_NOTA_ACCENTO:
    case CMD_NOTA_MARTELLATO:
    case CMD_NOTA_MARTDOLCE:
    case CMD_NOTA_STACCATO:
	case CMD_NOTA_ESPGENERIC:
		{
			EspressComposta *MyEsp=GetPEspressComposta(); 
			if (MyEsp!=NULL) 
				return MyEsp->GetPEspressione(MyCmd->CommandId);
			else 
				return NULL;
			
		}
		break;     
		
		
    case CMD_NOTA_CONSORDINAOTTONI:
    case CMD_NOTA_CONSORDINAARCHI:
    case CMD_NOTA_VIASORDINAOTTONI:
    case CMD_NOTA_VIASORDINAARCHI:
		
    case CMD_NOTA_VIASORDINATESTO: 
    case CMD_NOTA_CONSORDINATESTO: return((DrawObject *)((Indicazione*)(GetPSordina()))); break;
		
    case CMD_NOTA_SUDDIVISIONE: return (DrawObject *)GetPSuddivisione(); break;  	
		
    default:  return(NULL); break;
	}            
	
	return NULL;
}

void Nota::SetPos(DrawObject *d,const Point& p)
// Per come e' stato realizzato il calcolo della posizione, *d deve
// essere il pentagramma su cui si vuol disegnare la nota.
{
	DrawObject::SetPos(d,p);
	GetAbsPos().y+=Altezza*VU_PER_ROW/2;
	
	
	AdjustF();
	
	
	SetVU2Up(GetVU2UpF());
	SetVU2Dwn(GetVU2DwnF());
	SetVU2Lft(GetVU2LftF());
	SetVU2Rgt(GetVU2RgtF());  
	
}; 

void Nota::Adjust0()
{
}

DrawObject* Nota::UltimoDrwPriority(Bool sopra)
{
	
	EspressComposta *espComp = GetPEspressComposta();
	Strumento *stru = GetPStrumento();
	
	Espressione* esp;
	DrawObject *ultimodrw=NULL;
	
	ParteArco *parc;
	PosizArco *poarc;
	DirezArco *darc;
	
	int j;
	ClassID symbol;
	int LegPos = priorita.GetLegPos();
	for(j=0; j<LegPos; j++)
	{	  
		symbol = (ClassID)priorita.GetSymbolPri(j);
		switch (symbol)
		{
		case CL_STACCATO:
		case CL_TENUTO:
		case CL_SFORZATO: 
		case CL_ACCENTO:
		case CL_MARTELLATO:
		case CL_MARTDOLCE:
		case CL_PUNTOALLUNGATO:
			if(espComp!=NULL)
			{
				esp = espComp->GetPEspressione(symbol); 
				if(esp!=NULL && esp->GetAboveNota()==sopra)
				{
					ultimodrw = esp;
				}
			}
			break;
		case CL_PUNTA:
		case CL_TALLONE:
			if(stru!=NULL)
			{
				parc = ((Arco *)stru)->GetPParteArco();
				if (parc!=NULL && parc->GetAboveNota()==sopra)
				{
					ultimodrw = parc;
				}
			}
			break; 
		case CL_PONTICELLO:
		case CL_TASTIERA:
			if(stru!=NULL)
			{
				poarc = ((Arco *)stru)->GetPPosizArco();
				if (poarc!=NULL && poarc->GetAboveNota()==sopra)
				{
					ultimodrw = poarc;
				}
			}
			break;
		case CL_ARCOSU:
		case CL_ARCOGIU:
			if(stru!=NULL)
			{
				darc = ((Arco *)stru)->GetPDirezArco();
				if (darc!=NULL && darc->GetAboveNota()==sopra)
				{
					ultimodrw = darc;
				}
			}
			break;	  
		}
	}   
	
	return ultimodrw;
}   

void Nota::SetTie(Bool t)
{
	
	Tie=t;
	
}

Bool Nota::GetTie()
{
	
	return Tie;
	
}



//Restituisce il nome della nota nel riferimento chiave di Violino
static int CalculateOffset(ClassID clef)
{
	int offset=0;
	switch(clef)
	{
				case CL_CVIOLINO:
					offset=0;
					break;
				case CL_CBASSO:
					offset=-12;
					break;
				case CL_CBARITONO:
					offset=-10;
					break;
				case CL_CTENORE:
					offset=-8;
					break;
				case CL_CCONTRALTO:
					offset=-6;
					break;
				case CL_CMEZZOSOPRANO:
					offset=-4;
					break;
				case CL_CSOPRANO:
					offset=-2;
					break;
	}
	return offset;
}

static int IsInKeySign(tipoArmaturaChiave keySign, int pitch)
{
	switch(keySign)
	{
	case 0:
		return 0;
		break;
	case 1:     //case1-7 l'alterazione e' un diesis quindi return 1 perche' si deve sommare 1
		if (pitch==1)
		{return 1;}
		break;
	case 2:
		if (pitch==1 || pitch==5)
		{return 1;}
		break;
	case 3:
		if (pitch==1 ||pitch==5 ||pitch==2)
		{return 1;}
		break;
	case 4:
		if (pitch==1 ||pitch==5||pitch==2 || pitch==6)
		{return 1;}
		break;
	case 5:
		if (pitch==1 ||pitch==5 ||pitch==2 || pitch==6 || pitch==3)
		{return 1;}
		break;
	case 6:
		if (pitch==1 ||pitch==5 ||pitch==2 || pitch==6 || pitch==3 || pitch==0)
		{return 1;}
		break;
	case 7:
		if (pitch==1 ||pitch==5 ||pitch==2 || pitch==6 || pitch==3 || pitch==0 || pitch==4)
		{return 1;}
		break;
	case 8:  //case 8-14 l'alterazione e' un bemolle dunque si deve sottrarre 1
		if (pitch==4)
		{return -1;}
		break;
	case 9:
		if (pitch==4 || pitch==0)
		{return -1;}
		break;
	case 10:
		if (pitch==4 ||pitch==0 ||pitch==3)
		{return -1;}
		break;
	case 11:
		if (pitch==4 ||pitch==0 ||pitch==3 || pitch==6)
		{return -1;}
		break;
	case 12:
		if (pitch==4 ||pitch==0 ||pitch==3 || pitch==6 || pitch==2)
		{return -1;}
		break;
	case 13:
		if (pitch==4 ||pitch==0 ||pitch==3 || pitch==6 || pitch==2 || pitch==5)
		{return -1;}
		break;
	case 14:
		if (pitch==4 ||pitch==0 ||pitch==3 || pitch==6 || pitch==2 || pitch==5 || pitch==1)
		{return -1;}
		break;
	}
	return 0;
}


//Restituisce l'ottava nel riferimento chiave di Violino
int Nota::GetMyOctave(ClassID clef)
{
	int altezza=0;
	int offset=0;
	
	switch(clef)
	{
				case CL_CVIOLINO:
					offset=0;
					break;
				case CL_CBASSO:
					offset=-12;
					break;
				case CL_CBARITONO:
					offset=-10;
					break;
				case CL_CTENORE:
					offset=-8;
					break;
				case CL_CCONTRALTO:
					offset=-6;
					break;
				case CL_CMEZZOSOPRANO:
					offset=-4;
					break;
				case CL_CSOPRANO:
					offset=-2;
					break;
	}
	
	
	altezza=GetAltezza()+offset;
	if(altezza<0)
		altezza=altezza-6;
	return(altezza/7);
}

const char* Nota::GetMyName(ClassID clef,tipoArmaturaChiave keySign)
{
	return FifthsVect[3];
}


ClassID Nota::GetAccidentalType(tipoArmaturaChiave keySign,ClassID temp_Accidental[], int pitch)
{
	Alterazione *alterazPtr;
	int flagalterazione=0;
	
	if (GetPAlterazComposta())  //la nota ha un'alterazione associata PREDOMINA QUESTA SU ARMATURA DI KEY
	{
		alterazPtr=PAlterazComposta->GetPAlterazione(1);
		if (alterazPtr)
		{
			switch(alterazPtr->GetID())
			{
			case CL_DOPPIODIESIS:
				temp_Accidental[pitch] = CL_DOPPIODIESIS;
				break;
			case CL_DOPPIOBEMOLLE:
				temp_Accidental[pitch] = CL_DOPPIOBEMOLLE;
				break;
			case CL_DIESIS:
				temp_Accidental[pitch] = CL_DIESIS;
				break;
			case CL_BEMOLLE:
				temp_Accidental[pitch] = CL_BEMOLLE;
				break;
			case CL_BEQUADRO:
				{
					Alterazione *alteraz2Ptr;
					alteraz2Ptr=PAlterazComposta->GetPAlterazione(2);
					if (alteraz2Ptr)
					{
						switch(alteraz2Ptr->GetID())
						{
						case CL_DIESIS:
							temp_Accidental[pitch] = CL_DIESIS;
							break;
						case CL_BEMOLLE:
							temp_Accidental[pitch] = CL_BEMOLLE;
							break;
						default:
							temp_Accidental[pitch] = CL_BEQUADRO; //PROVVISORIA
						}
					}
					else 
						temp_Accidental[pitch] = CL_BEQUADRO;
				}
				break;
			}	
		}
	}
	else //Si controlla il vettore dei delle alterazioni temporanee
	{
		if(temp_Accidental[pitch]== CL_NATURAL)
		{
			flagalterazione=IsInKeySign(keySign,pitch);
			switch(flagalterazione)
			{
			case 0:
				break;
			case 1:
				temp_Accidental[pitch]=CL_DIESIS;
				break;
			case -1:
				temp_Accidental[pitch]= CL_BEMOLLE;
				break;
			}
		}
	}
	
	return temp_Accidental[pitch];
	
}



int Nota::GetMyPitch(ClassID clef)
{
	int offset=0;
	int pitch=0;
	
	offset=CalculateOffset(clef);
	pitch=(GetAltezza()+offset)%7;
	if(pitch<0)
		pitch=pitch+7;
	return pitch;
}

int Nota::GetAccidentalOffset(ClassID alterazione)
{
	int flagalterazione=0;
	
	switch(alterazione)
	{	
				case CL_DOPPIODIESIS:
					flagalterazione=+2; 
					break;
				case CL_DOPPIOBEMOLLE:
					flagalterazione=-2;
					break;
				case CL_DIESIS:
					flagalterazione=1;
					break;
				case CL_BEMOLLE:
					flagalterazione=-1;
				case CL_BEQUADRO:
					flagalterazione=0;
					break;
				case CL_NATURAL:
					flagalterazione=0;
					break;
					
	}
	return flagalterazione;
}	


int Nota::GetMyPosInFifths(ClassID clef,ClassID alterazione)
{
	int pitch=0;
	int flagalterazione=0;
	
	pitch=GetMyPitch(clef);
	
	flagalterazione=GetAccidentalOffset(alterazione);
	
	return Note2Fifths[pitch]+(7*flagalterazione);
}



void Nota::TransposeNoteByInterv(Battuta *measure, int shifting, ClassID temp1_accidental[], tipoArmaturaChiave start_keySign, ClassID temp_Accidental[], int up)
{
	int target_note_fifths=0;
	int my_pitch=0;
	int new_pitch=0;
	int my_octave=0;
	int new_octave=0;
	int type_alteraz=0;
	int my_pos_fifths=0;
	ClassID clef, accidental;
	tipoArmaturaChiave target_keySign;
	
	target_keySign=measure->GetArmatura(GetNStaff());
	clef=measure->GetChiave(GetNStaff())->GetID();
	
	my_pitch=GetMyPitch(clef);
	my_octave=GetMyOctave(clef);
	accidental=GetAccidentalType(start_keySign,temp1_accidental,my_pitch);
	
	my_pos_fifths=GetMyPosInFifths(clef,accidental);
	
	
	
	if ((shifting!=0)&&(shifting!=12)&&(shifting!=-12))
	{
		target_note_fifths=my_pos_fifths+shifting;
		if (target_note_fifths>34)
			target_note_fifths-=12;
		if (target_note_fifths<0)
			target_note_fifths+=12;
		
		
		new_pitch=Fifths2Note[target_note_fifths%7];
		
		if(target_note_fifths<=6)
			target_note_fifths=-target_note_fifths-14;
		else if((target_note_fifths<14)&&(target_note_fifths>6))
			target_note_fifths=-target_note_fifths;
		else
			target_note_fifths=target_note_fifths-14;
		
		type_alteraz=(target_note_fifths)/7;
		
		if(up==1)
		{
			if(my_pitch>new_pitch)
				new_octave=my_octave+1;
			else
				new_octave=my_octave;
		}
		else
		{
			if(my_pitch<new_pitch)
				new_octave=my_octave-1;
			else
				new_octave=my_octave;
		}
		
		
		AlterazComposta *new_alt =NULL;
		switch(type_alteraz)
		{
		case -2: //ci metto il doppio bemolle senza problemi
			if(temp_Accidental[new_pitch]!=CL_DOPPIOBEMOLLE)
			{
				if (GetPAlterazComposta()!=NULL)
				{
					delete GetPAlterazComposta();
					SetPAlterazComposta(NULL);
				}
				
				temp_Accidental[new_pitch]=CL_DOPPIOBEMOLLE;
				new_alt = new AlterazComposta;
				new_alt->InserisciAlt(CL_DOPPIOBEMOLLE);
			}
			else if(temp_Accidental[new_pitch]==CL_DOPPIOBEMOLLE)
			{
				if (GetPAlterazComposta()!=NULL)
				{
					delete GetPAlterazComposta();
					SetPAlterazComposta(NULL);
					temp_Accidental[new_pitch]=CL_DOPPIOBEMOLLE;
					new_alt = new AlterazComposta;
					new_alt->InserisciAlt(CL_DOPPIOBEMOLLE);
				}
			}
			
			break;
		case -1: //se e' nell'armatura non ci metto niente altrimenti ci metto il bemolle
			if((temp_Accidental[new_pitch]==CL_NATURAL)||(temp_Accidental[new_pitch]==CL_BEQUADRO))
			{
				if ((IsInKeySign(target_keySign, new_pitch)==-1)&&(temp_Accidental[new_pitch]!=CL_BEQUADRO))
				{
					if (GetPAlterazComposta()!=NULL)
					{
						delete GetPAlterazComposta();
						SetPAlterazComposta(NULL);
					}
					temp_Accidental[new_pitch]=CL_BEMOLLE;
					
				}
				else
				{
					if (GetPAlterazComposta()!=NULL)
					{
						delete GetPAlterazComposta();
						SetPAlterazComposta(NULL);
					}
					
					temp_Accidental[new_pitch]=CL_BEMOLLE;
					new_alt = new AlterazComposta;
					new_alt->InserisciAlt(CL_BEMOLLE);
				}
				
			}
			else
			{
				if(temp_Accidental[new_pitch]!=CL_BEMOLLE)
				{
					if (GetPAlterazComposta()!=NULL)
					{
						delete GetPAlterazComposta();
						SetPAlterazComposta(NULL);
					}
					
					temp_Accidental[new_pitch]=CL_BEMOLLE;
					new_alt = new AlterazComposta;
					
					if(temp_Accidental[new_pitch]!=CL_DOPPIOBEMOLLE)
						new_alt->InserisciAlt(CL_BEQUADRO);
					
					new_alt->InserisciAlt(CL_BEMOLLE);
				}
				else
				{
					if (GetPAlterazComposta()!=NULL)
					{
						delete GetPAlterazComposta();
						SetPAlterazComposta(NULL);
						temp_Accidental[new_pitch]=CL_BEMOLLE;
						new_alt = new AlterazComposta;
						new_alt->InserisciAlt(CL_BEMOLLE);
					}
				}
			}
			break;
		case 0:  //se e' nell'armatura o diesis o bemolle ci metto il bequadro altrimenti niente (controllo sulla precedente alterazione?)
			
			if((temp_Accidental[new_pitch]!=CL_NATURAL)&&(temp_Accidental[new_pitch]!=CL_BEQUADRO))
			{
				if (GetPAlterazComposta()!=NULL)
				{
					delete GetPAlterazComposta();
					SetPAlterazComposta(NULL);
				}
				
				temp_Accidental[new_pitch]=CL_BEQUADRO;
				new_alt = new AlterazComposta;
				new_alt->InserisciAlt(CL_BEQUADRO);
			}
			else
			{
				if(temp_Accidental[new_pitch]==CL_NATURAL)
				{
					if ((IsInKeySign(target_keySign, new_pitch)==-1) || (IsInKeySign(target_keySign, new_pitch)==1))
					{//settare il bequadro
						if (GetPAlterazComposta()!=NULL)
						{
							delete GetPAlterazComposta();
							SetPAlterazComposta(NULL);
						}
						
						temp_Accidental[new_pitch]=CL_BEQUADRO;
						new_alt = new AlterazComposta;
						new_alt->InserisciAlt(CL_BEQUADRO);
					}
					else
						if (GetPAlterazComposta()!=NULL)
						{
							delete GetPAlterazComposta();
							SetPAlterazComposta(NULL);
						}
				}
				if(temp_Accidental[new_pitch]==CL_BEQUADRO)
				{
					if (GetPAlterazComposta()!=NULL)
					{
						delete GetPAlterazComposta();
						temp_Accidental[new_pitch]=CL_BEQUADRO;
						new_alt = new AlterazComposta;
						new_alt->InserisciAlt(CL_BEQUADRO);
						
					}
				}
				else 
					if (GetPAlterazComposta()!=NULL)
					{
						delete GetPAlterazComposta();
						SetPAlterazComposta(NULL);
					}
			}	
			break;
		case 1: //se e' nell'armatura non ci metto niente altrimenti ci metto il diesis
			//settare il diesis
			if((temp_Accidental[new_pitch]==CL_NATURAL)||(temp_Accidental[new_pitch]==CL_BEQUADRO))
			{
				if ((IsInKeySign(target_keySign, new_pitch)==1)&&(temp_Accidental[new_pitch]!=CL_BEQUADRO))
				{
					if (GetPAlterazComposta()!=NULL)
					{
						delete GetPAlterazComposta();
						SetPAlterazComposta(NULL);
					}
					temp_Accidental[new_pitch]=CL_DIESIS;
					
				}
				else
				{
					if (GetPAlterazComposta()!=NULL)
					{
						delete GetPAlterazComposta();
						SetPAlterazComposta(NULL);
					}
					temp_Accidental[new_pitch]=CL_DIESIS;
					new_alt = new AlterazComposta;
					new_alt->InserisciAlt(CL_DIESIS);
				}
				
			}
			else
			{
				if(temp_Accidental[new_pitch]!=CL_DIESIS)
				{
					if (GetPAlterazComposta()!=NULL)
					{
						delete GetPAlterazComposta();
						SetPAlterazComposta(NULL);
					}
					temp_Accidental[new_pitch]=CL_DIESIS;
					new_alt = new AlterazComposta;
					
					if(temp_Accidental[new_pitch]!=CL_DOPPIODIESIS)
						new_alt->InserisciAlt(CL_BEQUADRO);
					
					new_alt->InserisciAlt(CL_DIESIS);
				}
				else
				{
					if (GetPAlterazComposta()!=NULL)
					{
						delete GetPAlterazComposta();
						SetPAlterazComposta(NULL);
						temp_Accidental[new_pitch]=CL_DIESIS;
						new_alt = new AlterazComposta;
						new_alt->InserisciAlt(CL_DIESIS);
					}
				}
			}
			break;
		case 2: 
			if(temp_Accidental[new_pitch]!=CL_DOPPIODIESIS)
			{
				if (GetPAlterazComposta()!=NULL)
				{
					delete GetPAlterazComposta();
					SetPAlterazComposta(NULL);
				}
				
				temp_Accidental[new_pitch]=CL_DOPPIODIESIS;
				new_alt = new AlterazComposta;
				new_alt->InserisciAlt(CL_DOPPIODIESIS);
			}
			else
			{
				if(temp_Accidental[new_pitch]==CL_DOPPIODIESIS)
				{
					if (GetPAlterazComposta()!=NULL)
					{
						delete GetPAlterazComposta();
						SetPAlterazComposta(NULL);
						
						temp_Accidental[new_pitch]=CL_DOPPIODIESIS;
						new_alt = new AlterazComposta;
						new_alt->InserisciAlt(CL_DOPPIODIESIS);
					}
				}
			}	
			break;//ci metto il doppio diesis senza problemi
		}
		new_pitch=new_pitch-CalculateOffset(clef);
		SetAltezza(new_pitch+(new_octave*7));
		SetPAlterazComposta(new_alt);
	}
	if(shifting==12)
	{
		new_pitch=my_pitch;
		new_octave=my_octave+1;
		new_pitch=new_pitch-CalculateOffset(clef);
		SetAltezza(new_pitch+(new_octave*7));
	}
	if(shifting==-12)
	{
		new_pitch=my_pitch;
		new_octave=my_octave-1;
		new_pitch=new_pitch-CalculateOffset(clef);
		SetAltezza(new_pitch+(new_octave*7));
	}
	
}
/*******************************************/

void Nota::Edit(Battuta *pBat)

{
	if(GetParentFig()!=NULL && GetParentFig()->GetID()==CL_GRUPPONOTE)
	{
		int fac=0;
		GruppoNote *grp=(GruppoNote*)GetParentFig();
		
		wxString tmp;
		tmp.sprintf("MEAN=%f\nMHL=%f\nDELTA=%f\nBEAMDIS=%f\nSTAFFNUMBER=%d",
			grp->GetMean(),
			grp->GetMeanHighLow(),
			grp->GetDelta(),
			grp->GetBeamDis(),
			grp->GetStaffInBeam());
		wxMessageBox(tmp);
	}
	
    EditingNotesDlg(this, pBat);
	
}

void Nota::TransSymb(Nota *ptr)
{
	
	if(ptr->GetPEspressComposta()!=NULL)
	{
		SetPEspressComposta(ptr->GetPEspressComposta());
		ptr->SetPEspressComposta(NULL);
	}
	if(ptr->GetPAbbellimento()!=NULL)
	{
		SetPAbbellimento(ptr->GetPAbbellimento());
		ptr->SetPAbbellimento(NULL);   
	}
	if(ptr->GetPKoreanOrnament()!=NULL)
	{
		SetPKoreanOrnament(ptr->GetPKoreanOrnament());
		ptr->SetPKoreanOrnament(NULL);   
	}
	if(ptr->GetPStrumento()!=NULL)
	{
		SetPStrumento(ptr->GetPStrumento());
		ptr->SetPStrumento(NULL);
	}
	
	if(ptr->GetPCorona()!=NULL)
	{
		SetPCorona(ptr->GetPCorona());
		ptr->SetPCorona(NULL);
	}                        
	if(ptr->GetPTDinamico()!=NULL)
	{
		SetPTDinamico(ptr->GetPTDinamico());
		ptr->SetPTDinamico(NULL);
	}
	if(ptr->GetPTGenerico()!=NULL)
	{
		SetPTGenerico(ptr->GetPTGenerico());
		ptr->SetPTGenerico(NULL);
	}
	if(ptr->GetPAnnotazione()!=NULL)
	{
		SetPAnnotazione(ptr->GetPAnnotazione());
		ptr->SetPAnnotazione(NULL);
	}
	if(ptr->GetPSordina()!=NULL)
	{
		SetPSordina(ptr->GetPSordina());
		ptr->SetPSordina(NULL);
	}
	if(ptr->GetPAlterazComposta()!=NULL)
	{
		SetPAlterazComposta(ptr->GetPAlterazComposta());
		ptr->SetPAlterazComposta(NULL);
	}
	if(!(ptr->Get_Link()).IsEmpty())
	{
		Set_Link(ptr->Get_Link());
	}
	if(ptr->GetPPuntoValore()!=NULL)
	{
		SetPPuntoValore(ptr->GetPPuntoValore());
		ptr->SetPPuntoValore(NULL);
	}
	
	
}


void Nota::UpDownSymb(Battuta *pBattuta)
{
	
	EspressComposta *pExpComp=NULL;
	
	if(GetPEspressComposta()!=NULL)
	{
		pExpComp=GetPEspressComposta();
		for(int i=1;i<=pExpComp->Getn_espressioni(); i++)
			pExpComp->GetPEspressione(i)->SetAboveNotaAuto(AUTOMATICO,this,pBattuta);
	}
	if(GetPAbbellimento()!=NULL)
	{
		GetPAbbellimento()->SetAboveNotaAuto(AUTOMATICO,this,pBattuta);
	}
	
	if(GetPKoreanOrnament()!=NULL)
	{
		GetPKoreanOrnament()->SetAboveNotaAuto(AUTOMATICO,this,pBattuta);
	}
	
	if(GetPSordina()!=NULL)
	{
		GetPSordina()->SetAboveNotaAuto(AUTOMATICO,this,pBattuta);
	}
	
}
//\\//\\//\\//\\//\\//\\//\\//\\
\\//\\//\\//\\//\\//\\//\\//\\//
wxString Nota::CreatePitchStr()
{
	CreatePitch();
	
	short letter = PPitch.Letter;
	short octave = PPitch.Octave;
	char alter =' ';
	if (PPitch.Accidental == 1)
		alter = '#';
	else if (PPitch.Accidental == -1)
		alter = 'b';
	
	return wxString::Format("%c%c%hd",NoteChars[letter],alter,octave);
}

wxString Nota::GetPitchStr()
{
	if (IsValidPitch)
	{
		short letter = PPitch.Letter;
		short octave = PPitch.Octave;
		char alter =' ';
		if (PPitch.Accidental == 1)
			alter = '#';
		else if (PPitch.Accidental == -1)
			alter = 'b';
		
		return wxString::Format("%c%c%hd",NoteChars[letter],alter,octave);
	}
	else return "";
}

TipoPitch* Nota::CreatePitch()
{	
	ClassID	alterazNota = CL_DRAWOBJECT; 
	int altezza, altezzaInSol, numeroOttava, pitchShift;
	
	int staffId					= GetNStaff();
	
	Battuta *pBat				= GetPBattuta();
	Chiave *pChiave				= pBat->GetChiave(staffId);
	ClassID chiaveId			= pChiave->GetID();
	tipoArmaturaChiave keySign	= pBat->GetArmatura(staffId);
	
	altezza						= GetAltezza();				//altezza grafica
	
	//pitch corrente in ch violino DO-SI (0-6)
	altezzaInSol = RiduciAltezzaIn7Note(chiaveId, altezza);
	numeroOttava = 3 +(int)floor( ((float)( altezza + TrasponiInChiaveDiSol(chiaveId)+2 )) / 7 );
	
	
	//calcolo alteraz finale
	alterazNota = EstraiAlterazioneDaNota((Nota*)this);
	
	if (alterazNota == CL_DRAWOBJECT)		//non ha alterazioni proprie
		alterazNota = EstraiAlterazioneDaPrecedenti(pBat, altezzaInSol);
	
	if (alterazNota == CL_DRAWOBJECT)		//non ha alterazioni precedenti
		alterazNota = EstraiAlterazioneDaArmatura(keySign, altezzaInSol);	
	
	//eventuali altri controlli su alterazioni aggiuntive
	
	switch(alterazNota)
	{
	case CL_DOPPIODIESIS:
		altezzaInSol = (altezzaInSol+1)%7;
		if (altezzaInSol == 0 || altezzaInSol == 3)
			pitchShift = 1;
		else
			pitchShift = 0;
		break;
	case CL_DOPPIOBEMOLLE:
		altezzaInSol = (altezzaInSol+6)%7;
		if (altezzaInSol == 2 || altezzaInSol == 6)
			pitchShift = -1;
		else
			pitchShift = 0;
		break;
	case CL_DIESIS:
		pitchShift = 1;
		break;
	case CL_BEMOLLE:
		pitchShift = -1;
		break;
	case CL_BEQUADRO:
		pitchShift = 0;
		break;
	default:
		pitchShift = 0;
		break;
	}
	
	//controllo se presente un intervallo modifOttava
	for(int i=0; i<=nIntRefs; i++)
	{
		if(GetIntRef(i) && (GetIntRef(i)->intRef))
		{
			if ( ( GetIntRef(i)->intRef )->GetID() == CL_MODIFICA_OTTAVA )
			{
				switch( ((ModifOttava*)(GetIntRef(i)->intRef))->tipo8 )
				{
				case VA8:
					numeroOttava+= 1;
					break;
				case BA8:
					numeroOttava+= -1;
					break;
				case MA15:
					numeroOttava+= 2;
					break;
				case BA15:
					numeroOttava+= -2;
					break;
				}
			}			
		}
	}
	
	PPitch.Accidental	= pitchShift;
	PPitch.Octave		= numeroOttava;
	PPitch.Letter		= altezzaInSol;
	
	return &PPitch;
}

TipoPitch* Nota::GetPitch()
{
	if (IsValidPitch)
		return &PPitch;
	else
		return NULL;
}

void Nota::SetPitch(TipoPitch *newPitch)
{
	if (newPitch != NULL)
	{
		PPitch = TipoPitch(*newPitch);
		IsValidPitch = TRUE;
	}
	else
		IsValidPitch = FALSE;
}
/*	CONTROLLO DI CONSISTENZA (SERVIRA' PER CONVERTIRE ALTEZZA IN PITCH)
char newLetter		= newPitch->Letter;
int newOctave		= newPitch->Octave;
int newAccid		= newPitch->Accidental;

  TipoPitch curPitch	= GetPitch();	
  if (curPitch.Letter != newLetter && 
		curPitch.Octave != newOctave && 
		curPitch.Accidental != newAccid)
		{
		int i, altezzaInSol, altezza;
		int *alterazNota;
		wxString pitchStr;
		
		  Battuta *pBat		= GetPBattuta();
		  Chiave *pChiave		= pBat->GetChiave(GetNStaff());
		  ClassID chiaveId	= pChiave->GetID();
		  tipoArmaturaChiave keySign = pBat->GetArmatura(GetNStaff());	
		  
			//coordinate pitch per Box errore
			if (newAccid == 1)
			pitchStr = wxString::Format("on %c#%d\nMeasure %d",newLetter, newOctave, pBat->GetNumProgress() );
			else if (newAccid == -1)
			pitchStr = wxString::Format("on %cb%d\nMeasure %d",newLetter, newOctave, pBat->GetNumProgress() );
			else 
			pitchStr = wxString::Format("on %c %d\nMeasure %d",newLetter, newOctave, pBat->GetNumProgress() );
			
			  //conversione in 'height' rispetto a ch. violino
			  altezzaInSol = ( newOctave-3 )*7 +( newLetter-2 );
			  
				
				  //CONSIDERO SOLO IL PITCH CONTESTUALE, SE NON E' RAPPRESENTABILE
				  //NELLA CHIAVE CORRENTE E CON ALTERAZIONE PROPRIA E PREESISTENTE GENERO UN ERRORE
				  //INOLTRE: IN OGNI BATTUTA A NOTE GRAFICHE UGUALI CORRISPONDONO PITCH UGUALI
				  
					
					  //alterazione nota selez.
					  alterazNota = EstraiAlterazioneDaNota((Nota*)this);
					  
						//alterazioni da note prec.
						if (alterazNota == NULL)	
						alterazNota = EstraiAlterazioneDaPrecedenti(pBat, altezzaInSol);
						
						  //alterazioni in chiave
						  if (alterazNota == NULL)
						  alterazNota = EstraiAlterazioneDaArmatura(keySign, altezzaInSol);
						  
							try{			
							if (alterazNota != NULL) //ci sono accid (compreso bequadro)
							{
							//aggiustamento ##
							if (*alterazNota == 2 && newAccid >= 0)
							{
							altezzaInSol--;
							//newLetter = (newLetter-1)%7;
							if  ( (newLetter == 3 || newLetter == 0) && newAccid == 1)  
							newAccid = 1;
							else if ( !(newLetter == 3 || newLetter == 0) && newAccid == 0 )//OK
							newAccid = 2;
							else
							throw wxString("No DoubleSharp Conversion:\n");
							}
							//aggiustamento bb
							else if (*alterazNota == -2 && newAccid <= 0)
							{
							altezzaInSol++;
							//newLetter = (newLetter+1)%7;
							if ( (newLetter == 2 || newLetter == 6) && newAccid == -1)
							newAccid = -1;
							if ( !(newLetter == 2 || newLetter == 6) && newAccid == 0)
							newAccid = -2;
							else
							throw wxString("No DoubleFlat Conversion ");
							}
							//compatibilit alterazione grafica
							else if (*alterazNota != newAccid)
							{
							if (*alterazNota == 1)
							throw wxString("Incompatible Sharp ");
							else if (*alterazNota == -1)
							throw wxString("Incompatible Flat ");
							else
							throw wxString("Incompatible Natural ");
							}
							}
							else
							if (newAccid != 0)
							throw wxString("Accidental Missing ");
							
							  }catch(wxString msg){
							  wxMessageBox(msg + pitchStr, "PITCH CONTEXT ERROR"); 
							  }
							  
								altezza = altezzaInSol - TrasponiInChiaveDiSol(pChiave->GetID());
								SetAltezzaInPlay(altezza);
	}*/
	
	
	int Nota::RiduciAltezzaIn7Note(const int chiave, const int altezza)
	{
		int i,diff;
		int altsol = altezza + TrasponiInChiaveDiSol(chiave); 
		
		if (altsol>=1000) altsol=0;
		//DIFF PRATICAMENTE TIENE CONTO DI TIPO DI NOTA.....DIFF =0 DO,DIFF=1 RE......
		for (i=-37;i<34;i+=7)
		{
			diff = altsol - i;
			if ( diff<=6  && diff>=0 )
				break;
		}
		return diff;
	}
	
	int Nota::TrasponiInChiaveDiSol(const int chiave) 
	{	
		//altezza grafica del E3 (rispetto al primo rigo)
		switch(chiave)
		{
		case CL_CVIOLINO:	
			return 0;
			break;
		case CL_CVIOLINO8:	
			return 0-7;
			break;
		case CL_CVIOLINO8SOPRA:
			return 0+7;
			break;
		case CL_CBASSO:	
		case CL_CBASSOOLD:
			return -12;
			break;
		case CL_CBASSO8:
			return -12-7;
			break;
		case CL_CBASSO8SOPRA:
			return -12+7;
			break;
		case CL_CBARITONO:
			return -10;
			break;
		case CL_CTENORE:
			return -8;
			break;
		case CL_CTENORE8:
			return -8-7;
			break;
		case CL_CCONTRALTO:
			return -6;
			break;
		case CL_CMEZZOSOPRANO:
			return -4;
			break; 
		case CL_CSOPRANO:
			return -2;
			break; 
		default:
			return 0;
			break;
		}
	}
	
	ClassID Nota::EstraiAlterazioneDaArmatura(tipoArmaturaChiave keySign, const int altezza)
	{
		int key,i;
		key = (keySign > 7 ? 7-keySign : keySign);
		if (key > 0)	//#
		{
			for(i=1; i<=key; i++)
			{
				if (altezza == (4*i-1)%7)
					return CL_DIESIS;
			}	
		}
		if (key < 0)	//b
		{
			for(i=1; i<=-key; i++)
			{
				if (altezza == (3*i+3)%7)
					return CL_BEMOLLE;
			}	
		}
		return CL_DRAWOBJECT;	
	}
	
	ClassID Nota::EstraiAlterazioneDaNota(Nota* pNota)
	{
		AlterazComposta		*APtr;
		Alterazione			*AAPtr;
		
		APtr=(pNota)->GetPAlterazComposta();
		if (APtr) 
		{	   
			AAPtr = APtr->GetPAlterazione(1);     
			if (AAPtr) 
				return AAPtr->GetID();	 		
		}
		return CL_DRAWOBJECT;
	}
	
	
	ClassID	Nota::EstraiAlterazioneDaPrecedenti(Battuta *pBat, int altezzaInSol)
	{
		ClassID alterazNota, alterPrec = CL_DRAWOBJECT;
		int layerId, staffId;
		Layer *pLayer;
		Figura *pFig, *pRefFig;
		staffId = GetNStaff();
		ClassID chiaveId		= pBat->GetChiave(staffId)->GetID();
		
		//reference alla strutt. verticale contenente la nota
		pRefFig = GetParentFig();										//caso nota in accordo
		if( pRefFig == NULL || pRefFig->GetID() == CL_GRUPPONOTE )	//altrimenti (nota singola o beam)
			pRefFig = this;
		
		for (layerId=0;layerId<NUM_MAX_LAYER ;layerId++)
		{
			pLayer  = pBat->GetPLayer( layerId );
			if (pLayer != NULL)
			{
				pFig	= pLayer->GetFirstFig();
				if(  pFig!=NULL && pFig->GetID() == CL_ACCORDO )
					((Accordo*)pFig)->GetFirstNote();
				
				while( pFig!=NULL && pFig->GetNStaff()==staffId && pFig!=pRefFig )
				{
					if(pFig->IsANote() && altezzaInSol == RiduciAltezzaIn7Note(chiaveId, pFig->GetAltezza())) 
					{
						alterazNota = EstraiAlterazioneDaNota((Nota*)pFig);
						if(alterazNota != CL_DRAWOBJECT)
							alterPrec = alterazNota;
					}
					if( pLayer->GetNextFigura(pFig) == NULL && pFig->GetParentFig() != NULL 
						&& pFig->GetParentFig()->GetID() == CL_ACCORDO )
						pFig = pLayer->GetNextFigura(pFig->GetParentFig());
					else
						pFig = pLayer->GetNextFigura(pFig);
					
				}
			}
		}
		return alterPrec;
	}
