/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: nota.hpp,v 1.1 2005/07/25 16:30:42 bellinpo Exp $ */


#ifndef NOTA_HPP
#define NOTA_HPP

#include "lioo.hpp"                       

#include "fig/figura.hpp" 

#include "alt/altecomp.hpp"
#include "abb/abbellim.hpp"
#include "str/violino.hpp"
#include "ind/esprcomp.hpp" 
#include "ind/diteggia.hpp"
#include "ind/armonici.hpp"
#include "ind/sordina.hpp"
#include "var/suddivis.hpp"
#include "kor/koreanorn.hpp"

//define per il box contenente il testo.

#define TEXTDX(HeadCode) -Scr->GetTextWidth(HeadCode)/2
#define TEXTDY(HeadCode) -4
#define TEXTLF(HeadCode)  Scr->GetTextWidth(HeadCode)/2+1
#define TEXTRG(HeadCode)  Scr->GetTextWidth(HeadCode)/2+3
#define TEXTUP(HeadCode)  6
#define TEXTDW(HeadCode)  7


#define OURTAGLIOLFT -2
#define OURTAGLIORGT 2

static const char NoteChars[]={'C','D','E','F','G','A','B'};
typedef struct{
	short Letter;
	short Octave;
	short Accidental;
} TipoPitch;
/*	Rappresentazione del Ciclo delle Quinte		  */


static const char* FifthsVect[]={"Fbb","Cbb","Gbb","Dbb","Abb","Ebb","Bbb","Fb","Cb","Gb","Db","Ab","Eb","Bb","F",
"C","G","D","A","E","B","F#","C#","G#","D#","A#","E#","B#","F##","C##","G##",
"D##","A##","E##","B##"};

//static const char NotevVect[]={'E','F','G','A','B','C','D'};
static const int Note2Fifths[]={19,14,16,18,20,15,17};
static const int Fifths2Note[]={1,5,2,6,3,0,4};
/***********************************************************/


class Battuta;

class MoodsMidiWrite;

class Nota: public Figura
{
private:           
	void AdjustAlterazComposta();
	// Sistema le alterazioni di alterazione composta a sinistra della nota.
	// Adatta il rettangolo che circonda la figura completa di simboli in modo
	// da fargli contenere anche i simboli dell'alterazione composta.
protected:
	void AdjustEspressComposta(VUnit YLinea1,VUnit YLinea5,Bool onspace,Bool
        onstaff,Point point,Espressione *PEspressione=NULL);  
	
	
	Bool Tie; 

	TipoPitch PPitch;
	// Altezza musicale sonora della figura. riferimento Do centrale C3.
	Bool IsValidPitch;	 
	
	void DrawContour(VUnit width = SELECTBOX_MARGIN);

private:
	void AdjustAbbellimento(VUnit YLinea1,VUnit YLinea5,Point);
	// Posiziona l'abbellimento sopra o sotto la nota.
	// YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu'
	// bassa e di quella piu' alta del pentagramma.
	// Adatta il rettangolo che circonda la figura completa di simboli in modo
	// da fargli contenere anche questo simbolo.
	void AdjustKoreanOrnament(VUnit YLinea1,VUnit YLinea5,Point);
	// Posiziona il KoreanOrnament sopra o sotto la nota.
	// YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu'
	// bassa e di quella piu' alta del pentagramma.
	// Adatta il rettangolo che circonda la figura completa di simboli in modo
	// da fargli contenere anche questo simbolo.
	void AdjustDiteggiato(VUnit YLinea1,VUnit YLinea5, Point point);
	// Posiziona il diteggiato sopra o sotto la nota.
	// YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu'
	// bassa e di quella piu' alta del pentagramma.
	// Adatta il rettangolo che circonda la figura completa di simboli in modo
	// da fargli contenere anche questo simbolo.
protected:
	void AdjustArmonici(VUnit YLinea1,VUnit YLinea5, Point point);
	// Posiziona armonici sopra o sotto la nota.
	// YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu'
	// bassa e di quella piu' alta del pentagramma.
	// Adatta il rettangolo che circonda la figura completa di simboli in modo
	// da fargli contenere anche questo simbolo.  
private:	 
	void AdjustSordina(VUnit YLinea1,VUnit YLinea5, Point point);
	// Posiziona la sordina sopra o sotto la nota.
	// YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu' 
	// bassa e di quella piu' alta del pentagramma.
	// Adatta il rettangolo che circonda la figura completa di simboli in modo
	// da fargli contenere anche questo simbolo.
	//  void AdjustStrumento(VUnit YLinea1,VUnit YLinea5);
	void AdjustStrumento(VUnit YLinea1,VUnit YLinea5,Point point);
	// Posiziona il simbolo dello strumento sopra o sotto la nota.
	// YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu' 
	// bassa e di quella piu' alta del pentagramma.
	// Adatta il rettangolo che circonda la figura completa di simboli in modo
	// da fargli contenere anche questo simbolo.
	// Richiama metodi di posizionamento pi\ux specifici per le classi figlie 
	// di strumento.
	void AdjustSuddivisione();
	// Posiziona la suddivisione sulla nota.
protected:
	//  void AdjustParteArco(VUnit YLinea1,VUnit YLinea5);
	void AdjustParteArco(VUnit YLinea1,VUnit YLinea5,Point point);
	// Posiziona la ParteArco sopra o sotto la nota.
	// YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu' 
	// bassa e di quella piu' alta del pentagramma.
	// Adatta il rettangolo che circonda la figura completa di simboli in modo
	// da fargli contenere anche questo simbolo.
	// void AdjustPosizArco(VUnit YLinea1,VUnit YLinea5);
	void AdjustPosizArco(VUnit YLinea1,VUnit YLinea5,Point point);
	// Posiziona la PosizArco sopra o sotto la nota.
	// YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu' 
	// bassa e di quella piu' alta del pentagramma.
	// Adatta il rettangolo che circonda la figura completa di simboli in modo
	// da fargli contenere anche questo simbolo.
	//  void AdjustDirezArco(VUnit YLinea1,VUnit YLinea5);
	void AdjustDirezArco(VUnit YLinea1,VUnit YLinea5,Point point);
	// Posiziona la DirezArco sopra o sotto la nota.
	// YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu' 
	// bassa e di quella piu' alta del pentagramma.
	// Adatta il rettangolo che circonda la figura completa di simboli in modo
	// da fargli contenere anche questo simbolo.
	//  void AdjustCorda(VUnit YLinea1,VUnit YLinea5);
	void AdjustCorda(VUnit YLinea1,VUnit YLinea5,Point point);
	// Posiziona la Corda sopra o sotto la nota.
	// YLinea1 ed YLinea5 sono le altezze fisiche (in VUnit) della linea piu' 
	// bassa e di quella piu' alta del pentagramma.
	// Adatta il rettangolo che circonda la figura completa di simboli in modo
	// da fargli contenere anche questo simbolo.
private:
    
	CmdResult CmdNotaCorda(SymCmd SCmd, TipoInserimento
		tpIns, Nota* pNota, Battuta* pBattuta);
	
	CmdResult CmdNotaStrArco(SymCmd SCmd, TipoInserimento tpIns, Nota* pNota,
		Battuta* pBattuta);
	
	CmdResult CmdNotaDiesis();
	// Assegna alla nota l'alterazione diesis. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	CmdResult CmdNotaDiesis1Q();
	// Assegna alla nota l'alterazione diesis1Q. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	CmdResult CmdNotaDiesis3Q();
	// Assegna alla nota l'alterazione diesis3Q. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL 
	CmdResult CmdNotaBemolle();
	// Assegna alla nota l'alterazione bemolle. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	CmdResult CmdNotaBemolle1Q();
	// Assegna alla nota l'alterazione bemolle1Q. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	CmdResult CmdNotaBemolle3Q();
	// Assegna alla nota l'alterazione bemolle3Q. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.  
	CmdResult CmdNotaBequadro();
	// Assegna alla nota l'alterazione bequadro. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	CmdResult CmdNotaBequadro1Q();
	// Assegna alla nota l'alterazione bequadro1Q. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	CmdResult CmdNotaBequadro3Q();
	// Assegna alla nota l'alterazione bequadro3Q. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	CmdResult CmdNota2Diesis();
	// Assegna alla nota l'alterazione doppio diesis. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	CmdResult CmdNota2Bemolle();
	// Assegna alla nota l'alterazione doppio bemolle. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
    
	CmdResult CmdNotaTrillo(TipoInserimento,Nota*,Battuta*,int NOnde);
	// Assegna alla nota il segno trillo. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura. 
	// Per rappresentare il prolungamento dell'effetto di trillo, al simbolo
	// (tr) si fa seguire un'onda piu' o meno lunga. Il parametro
	// d'ingresso non pu\ox assumere i valori: 0 (nessuna onda), 1, 2, 3, 4
	// (onda con la massima lunghezza).
	CmdResult CmdNotaGruppettoSup1(TipoInserimento,Nota*,Battuta*);
	// Assegna alla nota il segno gruppetto superiore. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	CmdResult CmdNotaTurnSlash(TipoInserimento,Nota*,Battuta*);
    // Assegna alla nota il segno turn slash. Se riesce, restituisce
    // CMD_OK, altrimenti CMD_FAIL.
    // Occorre specificare AboveN=TRUE per disegnare questo
    // segno sopra la figura, oppure FALSE per disegnarlo sotto la
    // figura.
	CmdResult CmdNotaGruppettoInf1(TipoInserimento,Nota*,Battuta*);
	// Assegna alla nota il segno turn slash. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	CmdResult CmdNotaTurnUp(TipoInserimento,Nota*,Battuta*);
	// Assegna alla nota il segno turn up. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	
	CmdResult CmdNotaMordenteSup(TipoInserimento,Nota*,Battuta*);
	// Assegna alla nota il segno mordente superiore. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	CmdResult CmdNotaMordente2Sup(TipoInserimento,Nota*,Battuta*);
	// Assegna alla nota il segno mordente doppio superiore. 
	// Se riesce, restituisce CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura. 
	CmdResult CmdNotaMordenteInf(TipoInserimento,Nota*,Battuta*);
	// Assegna alla nota il segno mordente inferiore. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	CmdResult CmdNotaMordente2Inf(TipoInserimento,Nota*,Battuta*);
	// Assegna alla nota il segno mordente doppio inferiore. 
	// Se riesce, restituisce CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.    
	CmdResult CmdNotaSlidingUp(TipoInserimento,Nota*,Battuta*);
	CmdResult CmdNotaSlidingDown(TipoInserimento,Nota*,Battuta*);
	CmdResult CmdNotaSlidingDUp(TipoInserimento,Nota*,Battuta*);
	CmdResult CmdNotaSlidingDownUp(TipoInserimento,Nota*,Battuta*);
	CmdResult CmdNotaNonghynInc(TipoInserimento,Nota*,Battuta*);
	CmdResult CmdNotaNonghynDec(TipoInserimento,Nota*,Battuta*);
	CmdResult CmdNotaNonghynArrow(TipoInserimento,Nota*,Battuta*);
	CmdResult CmdNotaNonghynWide(TipoInserimento,Nota*,Battuta*);
	// Assegna alla nota il simbolo korean-orrnament. 
	// Se riesce, restituisce CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.    
	CmdResult CmdNotaDito(TipoInserimento, Nota*, Battuta*, int NDito, int tipoDito);
	// Assegna alla nota la diteggiatura. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	// NDito puo' assumere i valori 1, 2, 3, 4 o 5.
	
	
	CmdResult CmdNotaPizzicato(TipoInserimento tpIns,
		Nota* pNota, Battuta* pBattuta);
	// Assegna alla nota il segno pizzicato. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	
	CmdResult CmdNotaTenuto(TipoInserimento tpIns, Nota* pNota, Battuta*
		pBattuta);
	// Assegna alla nota il segno tenuto. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	//  CmdResult CmdNotaPuntoAll(Bool AboveN);
	CmdResult CmdNotaPuntoAll(TipoInserimento tpIns, Nota* pNota, Battuta*
		pBattuta);
	// Assegna alla nota il segno punto allungato. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura. 
	//  CmdResult CmdNotaSforzato(Bool AboveN);
	CmdResult CmdNotaSforzato(TipoInserimento tpIns, Nota* pNota, Battuta*
		pBattuta);
	// Assegna alla nota il segno sforzato. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	//  CmdResult CmdNotaAccento(Bool AboveN);
	CmdResult CmdNotaAccento(TipoInserimento tpIns, Nota* pNota, Battuta*
		pBattuta);
	// Assegna alla nota il segno accento. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	//   CmdResult CmdNotaMartellato(Bool AboveN);
	CmdResult CmdNotaMartellato(TipoInserimento tpIns, Nota* pNota, Battuta*
		pBattuta);
	// Assegna alla nota il segno martellato. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura. 
    //    CmdResult CmdNotaMartDolce(Bool AboveN);
	CmdResult CmdNotaMartDolce(TipoInserimento tpIns, Nota* pNota, Battuta*
		pBattuta);
	// Assegna alla nota il segno martellato dolce. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	//    CmdResult CmdNotaStaccato(Bool AboveN);
	CmdResult CmdNotaStaccato(TipoInserimento tpIns, Nota* pNota, Battuta*
		pBattuta);
	// Assegna alla nota il segno staccato. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	CmdResult CmdNotaEspGeneric(int IDsym, TipoInserimento tpIns, Nota* pNota, Battuta*
		pBattuta); 
	
	CmdResult CmdNotaArmoniciOttoni(TipoInserimento, Nota*, Battuta*);
	// Assegna alla nota il segno Armonici per Ottoni. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	CmdResult CmdNotaArmoniciArchi(TipoInserimento, Nota*, Battuta*);
	// Assegna alla nota il segno Armonici per Archi. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	
	CmdResult CmdNotaViaSordinaOttoni(TipoInserimento, Nota*, Battuta*);
	// Assegna alla nota il segno ViaSordinaOttoni. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	CmdResult CmdNotaViaSordinaArchi(TipoInserimento, Nota*, Battuta*);
	// Assegna alla nota il segno ViaSordinaArchi. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	CmdResult CmdNotaViaSordinaTesto(TipoInserimento, Nota*, Battuta*);
	// Assegna alla nota il segno ViaSordinaTesto. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.  
	CmdResult CmdNotaConSordinaOttoni(TipoInserimento, Nota*, Battuta*);                             
	// Assegna alla nota il segno ConSordinaOttoni. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	CmdResult CmdNotaConSordinaArchi(TipoInserimento, Nota*, Battuta*);                             
	// Assegna alla nota il segno ConSordinaArchi. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	CmdResult CmdNotaConSordinaTesto(TipoInserimento, Nota*, Battuta*);                             
	// Assegna alla nota il segno ConSordinaTesto. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
    //    CmdResult CmdNotaViaSordina(Bool AboveN);
	// Assegna alla nota il segno ViaSordina. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
    //    CmdResult CmdNotaConSordina(Bool AboveN);                             
	// Assegna alla nota il segno ConSordina. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare AboveN=TRUE per disegnare questo
	// segno sopra la figura, oppure FALSE per disegnarlo sotto la
	// figura.
	CmdResult CmdNotaSuddivisione(short nt);
	// Assegna alla nota l'oggetto Suddivisione. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare il numero di tratti che suddividono la nota
	CmdResult CmdNotaTremolo(short nb);
	// Assegna alla nota l'oggetto Tremolo. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL.
	// Occorre specificare il numero di barre di cui e' composto l'oggetto.
	CmdResult CmdNotaGlissato();
	// Assegna alla nota l'oggetto Glissato. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL. 
	CmdResult CmdNotaGlissonda();
	// Assegna alla nota l'oggetto Glissato con l'attributo glissonda. Se riesce, restituisce
	// CMD_OK, altrimenti CMD_FAIL. 
	
    
      protected:
		  NoteHead_ID HeadType;
		  unsigned char HeadCode[3];
		  int DxGambo;
		  int DyGambo;    
		  
		  VUnit LGambo;
		  
		  Bool Coda;  
          // Indica se la nota va disegnata con la coda.              
		  AlterazComposta *PAlterazComposta; 
          // Puntatore all'alterazione composta della nota.
		  Abbellimento *PAbbellimento;
          // Puntatore all'abbellimento della nota.
		  KoreanOrnament *PKoreanOrnament;
		  //Puntatore al Korean Ornament della nota
		  Diteggiato *PDiteggiato;                                         
          // Puntatore alla diteggiatura della nota.                                
		  EspressComposta *PEspressComposta;
          // Puntatore a espressione composta.
		  Armonici *PArmonici;
          // Puntatore al segno Armonici.  
		  Sordina *PSordina;
          // Puntatore al segno di sordina.
		  Suddivisione *PSuddivisione;
          // Puntatore al segno di Suddivisione.
		  
		  
      public:
		  Nota(); 
		  
		  Nota(double); 
		  //////////
		  ~Nota();

		  virtual void Suona(MoodsMidiWrite *);
		  virtual float SuonaAppoggiatura(MoodsMidiWrite *);//silvestre (per il rendering dell'appoggiatura)
		  virtual void SuonaWithAppoggiatura(MoodsMidiWrite *,float);//silvestre (per il rendering dell'appoggiatura)
		  virtual void SuonaAsAcciaccatura(MoodsMidiWrite *);//silvestre (per il rendering dell'acciaccatura)
		  virtual void Suonalanota(MoodsMidiWrite * mmw, float durata, int altezza);
		  virtual void SuonaWithMordenteSuperiore(MoodsMidiWrite *mmw);
		  virtual void SuonaWithMordenteInferiore(MoodsMidiWrite *mmw);
		  virtual void SuonaWithGruppettoSuperiore(MoodsMidiWrite *mmw);
		  virtual void SuonaWithGruppettoInferiore(MoodsMidiWrite *mmw);
		  virtual void SuonaWithTremolo(MoodsMidiWrite *mmw);
		  virtual void SuonaWithTrillo(MoodsMidiWrite *mmw);

		  void SetTie(Bool t);
		  Bool GetTie();
		  
		  void Init();
          // Inizializza la nota, azzerando le variabili e ponendo NULL tutti
          // i puntatori.
		  
		  void InitRectF();
          // Inizializza il rettangolo che circonda la nota ai valori di
          // default per nota e notina.   
		  Bool IsANote();
		  
          
		  float GetDurata(); 
		  
		  NoteHead_ID GetHeadType(); 
		  void SetHeadType(NoteHead_ID); 
		  
		  void SetDxGambo(int dx); 
		  void SetDyGambo(int dy); 
		  
		  VUnit GetXgambo(); 
		  VUnit GetYgambo(); 
		  
		  void SetHeadCode(unsigned char *code); 
		  unsigned char* GetHeadCode() { return HeadCode; };
          
		  virtual void SetLGambo(VUnit lg);           
          // Imposta il gambo della nota. 
		  virtual void SetGamboUp();
          // Imposta il gambo sopra la nota, con lunghezza di default 
          // (meta' lunghezza di default se la nota e' una notina).
		  void SetNoGambo();
          // Imposta la nota senza gambo.
		  virtual void SetGamboDwn();
          // Imposta il gambo sotto la nota, con lunghezza di default
          // (meta' lunghezza di default se la nota e' una notina).
		  VUnit GetLGambo();
          // Restituisce la lunghezza del gambo della nota.
          
		  virtual void SetCoda(Bool coda);
          // Imposta la coda della nota.
		  Bool GetCoda();   
          // Indica se la nota ha coda.   
		  
		  void SetPAlterazComposta(AlterazComposta *palt);
		  AlterazComposta *GetPAlterazComposta();
		  
		  void SetPAbbellimento(Abbellimento *pabb);
		  Abbellimento *GetPAbbellimento();
		  
		  void SetPKoreanOrnament(KoreanOrnament *pabb);
		  KoreanOrnament *GetPKoreanOrnament();

		  void SetPDiteggiato(Diteggiato *pd);
		  Diteggiato *GetPDiteggiato();
		  
		  void SetPEspressComposta(EspressComposta *pe);
		  EspressComposta *GetPEspressComposta();
		  
		  void SetPSordina(Sordina *ps);
		  Sordina *GetPSordina();
		  
		  void SetPArmonici(Armonici *pa);
		  Armonici *GetPArmonici();
		  
		  void SetPSuddivisione(Suddivisione *ps);
		  Suddivisione *GetPSuddivisione();
		  
		  void MoveX(VUnit dx);
          // Sposta la nota con spostamento orizzontale di dx VUnit.
		  
		  void Adjust(Bool primaleg, Battuta *pBattuta, short current_layer, short current_staff);
		  // Sistema al posto giusto tutti i simboli riferiti alla nota. Imposta
          // VU2Up, VU2Dwn, VU2Lft, VU2Rgt in base alla dimensione della nota ed
          // a quella dei simboli connessi alla nota. Se una croma, o una semi-
          // croma, etc. ha Coda=FALSE, significa che fa parte di un accordo o di
          // un gruppo. Ora gli accordi hanno un proprio Adjust, mentre i gruppi
          // fanno riferimento a questo. Percio' alle note di questo tipo viene
          // alzato (con INCRYNOTESENZACODA) il rettangolo che le circonda per 
          // adeguarle alle barrette dei gruppi.
		  
		  void SetPos(DrawObject*,const Point&);
		  
		  void Draw();  
          // Disegna la nota ed i simboli ad essa connessi.
		  
		  void DrawTesta();  
          // Disegna la testa della nota.
		  void printTesta();
		  
		  void DrawTagli();  
          // Disegna i tagli addizionali della nota.
		  void printTagli();
		  
		  virtual void DrawForAccordo();
          // Disegna la nota. Attorno alla nota disegna
          // solo l'alterazione, il punto di valore e la diteggiatura. Serve per
          // disegnare le note negli accordi.
		  
		  Bool DeleteSym(DrawObject *sym);
          // Cancella il simbolo specificato se e' fra quelli connessi alla nota.
          // Ritorna TRUE se il simbolo e' stato cancellato
		  Bool DeleteStrumento(DrawObject *sym);
          // Utilizzato da DeleteSym: tratta i casi particolari degli strumenti.
          
		  void Free();  
          // Dealloca tutti i simboli connessi alla nota.
          // Inizializza la nota.
		  
		  Bool Hit(const Point&, SelObj, DrawObject*&, SymPath&);
		  Bool SymbolicHit(SymPath&, SelObj, DrawObject*&);
		  
		  CmdResult SymCommand(SymCmd&, Battuta* pBattuta=NULL);
		  
		  CmdResult CmdNotaNota2Notina();
		  // Trasforma una nota in notina ponendo Notina=TRUE e restituisce CMD_OK.
		  // Rimpiccolisce il gambo della nota, conservandone la direzione.
		  // Se la nota e' gia' una notina oppure fa parte di un gruppo di note,
		  // il metodo non fa niente e restituisce CMD_FAIL.
		  CmdResult CmdNotaNotina2Nota(TipoInserimento tpIns, Nota *pNota, Battuta*
			  pBattuta);
          // Trasforma una notina in nota ponendo Notina=FALSE e restituisce CMD_OK.
		  // Ingrandisce il gambo della nota, conservandone la direzione.
		  // Se la nota e' gia' a formato normale oppure fa parte di un gruppo di note,
		  // il metodo non fa niente e restituisce CMD_FAIL.
		  
		  virtual DrawObject * GetAggiunta(SymCmd * MyCmd);
		  
		  virtual void AdjustF() {};
		  virtual void Adjust0();
		  
		  DrawObject* UltimoDrwPriority(Bool sopra);
		  
		  //Restituisce la posizione della nota all'interno del Ciclo delle Quinte
		  //nel riferimento chiave di Violino e rispetto all'alterazione presente
		  int GetMyPosInFifths(ClassID clef,ClassID alterazione);
		  //Restituisce il nome della nota nel riferimento chiave di Violino
		  const char* GetMyName(ClassID clef, tipoArmaturaChiave keySign);
		  //Restituisce l'ottava nel riferimento chiave di Violino
		  int GetMyOctave(ClassID clef);
		  //Esegue la trasposizione di intervallo
		  void TransposeNoteByInterv(Battuta *measure, int shifting, 
			  ClassID temp1_accidental[], tipoArmaturaChiave start_keySign, ClassID temp_Accidental[], int up);
		  
		  int GetAccidentalOffset(ClassID alterazione);
		  int GetMyPitch(ClassID clef);
		  ClassID GetAccidentalType(tipoArmaturaChiave keySign,ClassID temp_Accidental[], int pitch);
		  
		  //Trasferimento dei simboli da una nota 
		  void TransSymb(Nota *ptr);
		  
		  //Definizione dei parametri della nota a partire dal nom
		  
		  
		  
		  /*******************************************/
		  
		  void Edit(Battuta *pBat=NULL);
		  
		  
		  void UpDownSymb(Battuta *pBattuta);
		  
		  /*******************************************/
		  
		  void			SetPitch(TipoPitch *pitch);
		  
		  TipoPitch*	GetPitch();
		  wxString		GetPitchStr();
		  //se il pitch non  settato queste 2 funz. ritornano NULL

		  TipoPitch*	CreatePitch();
		  wxString		CreatePitchStr();
		  //creazione del pitch dal valore grafico dell'altezza,

		  int SaveXML(FILE *);
	private:
		//utils per get/set pitch
		  int TrasponiInChiaveDiSol(const int chiave);
		  ClassID EstraiAlterazioneDaNota(Nota* pNota);
		  ClassID EstraiAlterazioneDaArmatura(tipoArmaturaChiave keySign, const int altezza);
		  ClassID EstraiAlterazioneDaPrecedenti(Battuta *pBat, int altezzaInSol);
		  int RiduciAltezzaIn7Note(const int chiave, const int altezza);
    };
	
#endif //NOTA_HPP
	
    
    
	
	
	
	
	
	
	
	
	
	
	
