/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: pausa.cpp,v 1.1 2005/07/25 16:30:44 bellinpo Exp $ */


#include "fig/pausa.hpp"
#include "str/timpano.hpp"
#include "gui/dialogs.hpp"
#include "bat/battuta.hpp"  // GB

Pausa::Pausa()
{
	Init();
}

Pausa::Pausa(double dur) : Figura(dur)
{
	Init();
}
/////////////////

void Pausa::Init()
{
	Figura::Init();
	ID=CL_PAUSA;
	Figura::SetNotina(FALSE);
}


Bool Pausa::IsARest()
{
	return TRUE;
} 

float Pausa::GetDurata()
{
	return Figura::GetDurata();  
}


void Pausa::Adjust(Bool primaleg, Battuta *pBattuta, short current_layer, short curStaff)
{
	VUnit YLinea1,YLinea5;
	// Altezza fisica delle linee piu' bassa e piu' alta del pentagramma.
	
	SetVU2Up(GetVU2UpF());
	SetVU2Dwn(GetVU2DwnF());
	SetVU2Lft(GetVU2LftF());
	SetVU2Rgt(GetVU2RgtF());  
	
	VUnit YLineaBase=pBattuta->GetIntest(GetNStaff())->GetAbsPos().y;
	YLinea1=-pBattuta->GetParentScore()->GetBaseVU2Dwn(GetNStaff())+YLineaBase;
	YLinea5=pBattuta->GetParentScore()->GetBaseVU2Up(GetNStaff())+YLineaBase;
	
	AdjustStrumento(YLinea1,YLinea5);
	
	Figura::Adjust(primaleg, pBattuta, current_layer,this->GetNStaff());
}


void Pausa::Draw()
{
	Figura::Draw();
}


Bool Pausa::DeleteSym(DrawObject *sym)
{
	if(sym==GetPStrumento())
	{
		delete GetPStrumento();
		SetPStrumento(NULL);
		return TRUE;
	}   
	
	return Figura::DeleteSym(sym);
}


CmdResult Pausa::SymCommand(SymCmd& SCmd, Battuta *pBattuta)
{ 
	CmdResult cmdRes;
	TipoInserimento tpIns=(TipoInserimento)(Bool)SCmd.ArgV[0];
	Bool aboveNota=FALSE;
	
	switch(tpIns)
	{
    case SOPRA:
		aboveNota=TRUE;
		break;
    case SOTTO:
		aboveNota=FALSE;
		break;
	case AUTOMATICO:
		{
			if(SCmd.CommandId==CMD_PAUSA_TIMPANO)
				aboveNota=FALSE;
			break;
		}
	}
	
	switch(SCmd.CommandId)
	{
    case CMD_PAUSA_TIMPANO:
		return(CmdPausaTimpano(aboveNota,(Bool)SCmd.ArgV[1],
			(unsigned char*)SCmd.Text));
    case CMD_NOTA_NOTA2NOTINA:
		return(CmdPausaNota2Notina());
		break;  
    case CMD_NOTA_NOTINA2NOTA:
		return(CmdPausaNotina2Nota());
		break;  
    default:
		cmdRes=Figura::SymCommand(SCmd,pBattuta);
		if(cmdRes!=CMD_UNKNOWN)
            return(cmdRes);
	}     
	return(CMD_UNKNOWN);
}

CmdResult Pausa::CmdPausaTimpano(Bool AboveN, Bool ConFreccia, unsigned char *txt)
{
	if(GetPStrumento()!=NULL)
	{
		if ((GetPStrumento()->GetID() == CL_TIMPANO) && (GetPStrumento()->GetAboveNota() == AboveN))
		{
			if  ((strcmp((char *)(((Timpano*)GetPStrumento())->GetTxt()),(char *)txt)) &&
				(((Timpano*)GetPStrumento())->GetFreccia() == ConFreccia))
				return(CMD_FAIL);
		}
		else
			delete GetPStrumento();
	}
	SetPStrumento(new Timpano);
	if(GetPStrumento()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Pausa","CmdPausaTimpano()");  
		return(CMD_FAIL);
	}; 
	GetPStrumento()->SetAboveNota(AboveN);
	if (!(((Timpano*)GetPStrumento())->SetTxt(txt)))
	{
		EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Pausa","CmdPausaTimpano()");  
		return CMD_FAIL;
	}
	((Timpano*)GetPStrumento())->SetFreccia(ConFreccia);
	return(CMD_OK);
}


void Pausa::AdjustStrumento(VUnit YLinea1,VUnit YLinea5)
{
	VUnit dy;
	Strumento *pT;
	
	if(GetPStrumento()!=NULL)
	{ 
		pT = GetPStrumento(); 
		if(pT->GetAboveNota())
		{
			// I segni di strumento sono sopra la nota.
			pT->SetPos(this,Point(0,GetVU2Up()+RECTSPACEY+pT->GetVU2Dwn()));
			
			// Accorgimento per evitare di sistemare i segni di strumento dentro
			// il pentagramma.
			dy = YLinea5 + RECTSPACEY + pT->GetVU2Dwn() - pT->GetAbsPos().y;
			if(dy>0) pT->Move(Point(0,dy));
			// Aggiorna le VU2Up.
			SetVU2Up(pT->GetAbsPos().y-GetAbsPos().y + pT->GetVU2Up());
		}
		else
		{
			// I segni di strumento sono sotto la nota.
			pT->SetPos(this,Point(0,-GetVU2Dwn()-RECTSPACEY-pT->GetVU2Up()));
			// Accorgimento per evitare di sistemare i segni di strumento dentro
			// il pentagramma.
			dy = YLinea1 - RECTSPACEY - pT->GetVU2Up() - pT->GetAbsPos().y;
			if(dy<0) pT->Move(Point(0,dy));
			// Aggiorna le VU2Dwn.
			SetVU2Dwn(-pT->GetAbsPos().y+GetAbsPos().y+pT->GetVU2Dwn());
		}
		if(pT->GetVU2Lft()>GetVU2Lft())
			SetVU2Lft(pT->GetVU2Lft());
		if(pT->GetVU2Rgt()>GetVU2Rgt())
			SetVU2Rgt(pT->GetVU2Rgt());
	}
}


CmdResult Pausa::CmdPausaNota2Notina()
{                    
	if(GetNotina())
		return(CMD_FAIL); 
	
	SetNotina(TRUE);
	return(CMD_OK);
}


CmdResult Pausa::CmdPausaNotina2Nota()
{                            
	if(!GetNotina())
		return(CMD_FAIL);
	
	SetNotina(FALSE);
	return(CMD_OK);
}               

void Pausa::Edit(Battuta *pBat)
{
	
    EditingRestsDlg(this, pBat);
	
}