/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: spazio.cpp,v 1.1 2005/07/25 16:30:50 bellinpo Exp $ */


#include "fig/spazio.hpp"

#include "bat/battuta.hpp" 

bool ViewSpace=false; 

// variabile di stato del sistema non del singolo spazio
void SetViewSpace(Bool vs)
{
	ViewSpace=(bool)vs;
}


Bool GetViewSpace()
{ 
	return ViewSpace;
}


Spazio::Spazio()
{
	Init();
	durata = 0;
}


Spazio::~Spazio()
{
	Free();
} 

void Spazio::Init()
{
	Figura::Init(); 
	ID=CL_SPAZIO;
	
	//SetColor((ColorIcon)SPACECOLOR); //FFF
	
	SetPPrFiato(NULL);
}  

void Spazio::SetPPrFiato(PrFiato *prf)
{
	PPrFiato=prf;
}

PrFiato* Spazio::GetPPrFiato()
{
	return PPrFiato;
}

void Spazio::Draw()
{               
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());
	
	Figura::Draw();   
	
	if(ViewSpace)
	{
		if(isOpaque)
			Scr->SetColor(GetColor());
		
		Point p1, p2;
		p1 = GetAbsPos() + Point(-VU2LFTNOTA,VU2UPNOTA);
		p2 = GetAbsPos()+Point(VU2RGTNOTA,-VU2DWNNOTA);
		Scr->Rect(class Rectangle(p1,p2));
		
	}
	
	//disegno dell'eventuale simbolo di prendere fiato
	
	if(GetPPrFiato()!=NULL)
	{
		GetPPrFiato()->Draw();
	}

	Scr->SetColor(oldcol);
}


void Spazio::Select()
{
	DrawObject::Select();

	//Scr->SetColor((ColorIcon)XOR_SELECTED_COLOR);
	//Scr->SetDrwMode(XORMode);
	//Scr->Rect(class Rectangle(*this));
	//Scr->SetDrwMode(DefMode);
}


void Spazio::Deselect()
{
	DrawObject::Deselect();

//	Select();
}


Bool Spazio::Hit(const Point& p, SelObj selObj, DrawObject*& drwObj,
				 SymPath& symPath)
{
	Bool hit;   
	Point p1;   
	
	p1=p-GetAbsPos();               
	hit = -p1.x <= GetVU2LftF() && p1.x <= GetVU2RgtF() &&
        -p1.y <= GetVU2DwnF() && p1.y <= GetVU2UpF();
	
	drwObj=NULL;
	
	if(hit)
	{
		if(selObj==ANY_SEL || selObj==FIGURA_SEL || selObj==NOTASING_SEL)
		{  
			drwObj=this;
			symPath.Array[symPath.Level]=GetNumericCode();
		}
		return TRUE;
	}
	
	if(drwObj==NULL) // non serve questo if, ma per scrupolo si lascia
	{
		if(GetPPrFiato()!=NULL &&
			GetPPrFiato()->Hit(p,selObj,drwObj,symPath))
		{ 
			symPath.Array[symPath.Level]=GetNumericCode();
			return(TRUE);
		}
	}
	return Figura::Hit(p,selObj,drwObj,symPath);
}

Bool Spazio::SymbolicHit(SymPath& symPath, SelObj selObj, DrawObject*& drwObj)
{
	drwObj=NULL;
	
	Bool EsaminareSimboli = (symPath.Array[SIMBOLO_PATH_LEVEL]!=0);
	
	if(!EsaminareSimboli && symPath.Array[symPath.Level]==GetNumericCode())
	{
		if(selObj==ANY_SEL || selObj==FIGURA_SEL || selObj==NOTASING_SEL)
			drwObj=this;
		return TRUE;
	}
	if(EsaminareSimboli && drwObj==NULL && selObj==ANY_SEL)
	{
		if(GetPPrFiato()!=NULL &&
			GetPPrFiato()->SymbolicHit(symPath,selObj,drwObj))
			return(TRUE);
	}
	return Figura::SymbolicHit(symPath,selObj,drwObj);
}


CmdResult Spazio::SymCommand(SymCmd& SCmd, Battuta *pBattuta)
{
	CmdResult cmdRes;
	
	switch(SCmd.CommandId)
	{
    case CMD_SPAZIO_PRFIATO:
		return CmdSpazioPrFiato();
    case CMD_FIGURA_TGENERICO:
    case CMD_FIGURA_ANNOTAZIONE:
    case CMD_FIGURA_OCCHIALI:
		cmdRes=Figura::SymCommand(SCmd);
		if(cmdRes!=CMD_UNKNOWN)
            return(cmdRes);
		
    default: ;
	}
	return(CMD_UNKNOWN);
}

void Spazio::Free()
{
	
	if(GetPPrFiato()!=NULL)
		delete GetPPrFiato();
	Init();
}

CmdResult Spazio::CmdSpazioPrFiato()
{
	
	if(GetPPrFiato()!=NULL)
		delete GetPPrFiato();
	SetPPrFiato(new PrFiato);
	if(GetPPrFiato()!=NULL)
	{
		PPrFiato->SetPos(this, Point(0,(ALT_PRFIATO-Altezza)*VU_PER_ROW/2));
		return(CMD_OK);
	}
	else
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Spazio","CmdSpazioPrFiato");  
		return(CMD_FAIL);
	}; 
};

void Spazio::Adjust(Bool primaleg, Battuta *pBattuta, short current_layer, short current_staff)
{         
	SetVU2UpF(VU2UPNOTA);
	SetVU2DwnF(VU2DWNNOTA);
	SetVU2LftF(VU2LFTNOTA);
	SetVU2RgtF(VU2RGTNOTA);
	
	SetVU2Up(VU2UPNOTA);
	SetVU2Dwn(VU2DWNNOTA);
	SetVU2Lft(VU2LFTNOTA);
	SetVU2Rgt(VU2RGTNOTA);
	
	if(GetPPrFiato()!=NULL)
		PPrFiato->SetPos(this, Point(0,(ALT_PRFIATO-Altezza)*VU_PER_ROW/2));
	Figura::Adjust(primaleg, pBattuta, current_layer, current_staff);
}

Bool Spazio::DeleteSym(DrawObject* obj)
{
	if(obj==GetPPrFiato())
	{
		delete GetPPrFiato();
		SetPPrFiato(NULL);
		return TRUE;
	}
	return Figura::DeleteSym(obj);
}


void Spazio::Setrelpos(int pos)
{
	relpos=pos;  
}

int Spazio::Getrelpos()
{
	return relpos;
}



