/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: lioofont.cpp,v 1.1 2005/07/25 16:30:51 bellinpo Exp $ */
#include <wx/wx.h>
#include <wx/wxexpr.h>

#include <string.h>
#include "fon/lioofont.hpp"
#include "drw/gscreen.hpp"
#include "drw/musicedt.hpp"
#include "smr/smr.hpp"

extern GScreen *Scr;
int LoadTrueTypeFont(const char* fontFile, char* fontName, void** hfont);
void ReleaseTrueTypeFont(void* hfont);
/////////////////////////////////////////

extern Widget LiooWidget;
extern wxMusicEditor *MusicEditor;
extern wxDC *currDC;
extern wxDC *currMainDC;
extern wxMemoryDC *currMDC;
extern LiooWindow *pMusicWin;
extern SMRFile *smrf;
extern wxMusicEditor *MusicEditor;

/////////////////////////////////////////

//
// Crea l'oggetto e carica il font
//

char LiooFont::defaultTextFont[255]="Times New Roman";

LiooFont::LiooFont(char *filename, SMRFile* smrf, int fontSize)
{
	int xx;
	
	nomeFont=NULL;
	for(xx = 0; xx < 256; xx++)
	{
		item[xx].image=NULL;
		item[xx].ascend = 0;
		item[xx].height = 0;
		item[xx].width=0;
		item[xx].realwidth = 0;
		item[xx].DX=0;
		item[xx].DY=0;
		item[xx].VU2DWN=0;
		item[xx].VU2LF=0;
		item[xx].VU2RG=0;
		item[xx].VU2UP=0;
		item[xx].code=xx;
	}
	isTTF=false;
	hfont=NULL;
	load(filename,smrf,fontSize);
}


//
// Distrugge l'oggetto
//
LiooFont::~LiooFont(void)
{
	int xx;
	
	// Dealloca l'immagine del font
	delete nomeFont;
	for(xx = 0; xx < 256; xx++)
	{
		if (item[xx].image!=NULL)
			delete item[xx].image;
	}
	if(hfont)
		ReleaseTrueTypeFont(hfont);
}

//
// Setta lo ID del font
//
void LiooFont::SetID(FontID i)
{ id = i; }

//
// Restituisce lo ID del font
//
FontID LiooFont::GetID(void)
{ return id; }

//
// Ritorna l'altezza del carattere
//
int LiooFont::get_height(int code)
{
	if(isTTF)
	{
		char c[2] = { 0, 0 };
		c[0] = code;
		wxFont font(defaultSize * Scr->GetMagnification(),wxDEFAULT,wxNORMAL,wxNORMAL, false, nomeFont);
		Scr->SetSystemFont(&font);
		return Scr->GetSystemTextHeight(c);
	}
	if (code<0) code+=256;
	return item[code].height;
}

//
// Ritorna la larghezza del carattere
//
int LiooFont::get_width(int code)
{
	if (code<0) code+=256;
	return item[code].realwidth;
}

//
// Ritorna 'ascend' del carattere
//
int LiooFont::get_ascend(int code)
{
	if(isTTF)
	{
		wxFont font(defaultSize * Scr->GetMagnification(),wxDEFAULT,wxNORMAL,wxNORMAL, false, nomeFont);
		Scr->SetSystemFont(&font);
		return Scr->GetSystemFontAscent();
	}
	if (code<0) code+=256;
	return item[code].ascend;
}

// Restituisce VU2UP del carattere (distanza dal AbsPoint al margine superiore)
int LiooFont::get_VU2UP(int code)
{
	if (code<0) code+=256;
	return item[code].VU2UP;
} 

// Restituisce VU2DWN del carattere (distanza dal AbsPoint al margine inferiore)
int LiooFont::get_VU2DWN(int code)
{
	if (code<0) code+=256;
	return item[code].VU2DWN;
} 

// Restituisce VU2RG del carattere (distanza dal AbsPoint al margine destro)
int LiooFont::get_VU2RG(int code)
{
	if (code<0) code+=256;
	return item[code].VU2RG;
} 

// Restituisce VU2LF del carattere (distanza dal AbsPoint al margine sinistro)
int LiooFont::get_VU2LF(int code)
{
	if (code<0) code+=256;
	return item[code].VU2LF;
} 

// Restituisce DX del carattere (offset orizontale)
int LiooFont::get_DX(int code)
{
	if (code<0) code+=256;
	return item[code].DX;
}

// Restituisce DY del carattere (offset verticale)
int LiooFont::get_DY(int code)
{
	if (code<0) code+=256;
	return item[code].DY;
}

void LiooFont::SetDimension(int code, int up, int dwn, int lft, int rgt, int dx, int dy)
{
	if (code<0) code+=256;
	item[code].code=code;
	item[code].VU2UP=up;
	item[code].VU2DWN=dwn;
	item[code].VU2LF=lft;
	item[code].VU2RG=rgt;
	item[code].DX=dx;
	item[code].DY=dy;
}

//
// Stampa il carattere nella posizione specificata riferita alla baseline
//
void LiooFont::print(int x_pos, int y_pos, int code)
{
	if(isTTF)
	{
		char c[2]= { 0, 0 };
		c[0] = code;

		wxFont font(defaultSize * Scr->GetMagnification(),wxDEFAULT,wxNORMAL,wxNORMAL, false, nomeFont);
		Scr->SetSystemFont(&font);
		VUnit y = (Scr->GetMaxPixY() - y_pos/Scr->GetMagnification()) ;
		VUnit x = x_pos  / Scr->GetMagnification();
		Scr->TextTo(x, y + Scr->GetSystemFontAscent() / Scr->GetMagnification() );
		Scr->OutSystemText(c);

		//Scr->Line(Point(x-10,y),Point(x+10,y));
		//Scr->Line(Point(x,y-10),Point(x,y+10));
	}
	else
	{
		int xx, yy;
		int col, row;
		unsigned char ret;
		int bytes_per_row;
		int bt;
		
		if (item[code].image==NULL)
			return;
		
		col = row = 0;
		// Comincia dal primo pixel il alto a sinistra
		y_pos = y_pos - item[code].ascend + 1; // (LM)
		
		// Stampa il carattere
		bytes_per_row = (int)ceil((double)item[code].width / 8);
		bt = 0;
		for(xx = 0; xx < item[code].nbytes; xx++)
		{
			// Legge il byte
			for(yy = 7; yy >= 0; yy--)
			{
				if((8 * bt + yy) >= item[code].width)
					break;
				ret = ((item[code].image[xx])>>yy)&0x1;
				if(ret == 1)
					Scr->PutPixel0Up(Point(x_pos + col, y_pos + row));
				col++;
			}
			// Passa al byte successivo della riga se c'e'
			if(bt < (bytes_per_row - 1))
				bt++;
			else
			{
				// Passa alla riga successiva
				row++;
				col= 0;
				bt = 0;
			}
		}
	}
}

//
// Stampa una sequenza di caratteri
//
#include <windows.h>

void LiooFont::print_str(int x_pos, int y_pos, unsigned char *string)
{
	if(isTTF)
	{
		bool unicodeText=false;
		unsigned short wstring[255];
		if(id==TEXTFONT_ID)
		{
			MultiByteToWideChar(CP_UTF8,0,(char*)string,-1,wstring,255);         
			//strcpy(nomeFont,"Baekmuk Batang");
			unicodeText=true;
		}
		wxFont font(defaultSize * Scr->GetMagnification(),wxDEFAULT,wxNORMAL,wxNORMAL, false, nomeFont);
		Scr->SetSystemFont(&font);
		VUnit y = (Scr->GetMaxPixY() - y_pos/Scr->GetMagnification()) ;
		VUnit x = x_pos  / Scr->GetMagnification();
		Scr->TextTo(x, y + Scr->GetSystemFontAscent() / Scr->GetMagnification() );
		if(unicodeText)
		{
			Scr->OutSystemText(wstring);
		}
		else
			Scr->OutSystemText((char*)string);

		//Scr->Line(Point(x-10,y),Point(x+10,y));
		//Scr->Line(Point(x,y-10),Point(x,y+10));
	}
	else
	{
		int xx;
		
		for(xx = 0; xx < (int)strlen((char *)string); xx++)
		{ 
			print(x_pos, y_pos, string[xx]);
			x_pos += item[string[xx]].realwidth + INTERSPACE;
		}
	}
}

int LiooFont::GetTextWidth(unsigned char* txt)
{
	if(isTTF)
	{
		wxFont font(defaultSize * Scr->GetMagnification(),wxDEFAULT,wxNORMAL,wxNORMAL, false, nomeFont);
		Scr->SetSystemFont(&font);
		return Scr->GetSystemTextWidth((char*)txt) / Scr->GetMagnification();
	}
	else
	{
		int xx;
		int len = 0;
	
		// Calcola la lunghezza totale della stringa
		for(xx = 0; xx < (int)strlen((char *)txt); xx++)
			len += get_width(txt[xx]) + INTERSPACE;
	
		return len; 
	}
}

static int getInt(FILE* fp)
{
	union {
		int rl;
		unsigned char rc[4];
	} r;
	
	fread(&r,sizeof(r),1,fp);
#ifdef HPPA  
	union {
		int rl;
		unsigned char rc[4];
	}s;
	s.rc[0]=r.rc[3];
	s.rc[1]=r.rc[2];
	s.rc[2]=r.rc[1];
	s.rc[3]=r.rc[0];
	
	return s.rl;
#else
	return r.rl;
#endif
}

void LiooFont::SetDefaultTextFont(const char* fontname)
{
	strcpy(defaultTextFont, fontname);
}

int LiooFont::load(char *filename, SMRFile* smrf, int size)
{
	if(hfont)
		ReleaseTrueTypeFont(hfont);

	wxString fname=filename;
/*
	if(fname == TEXTFONT)
	{
		isTTF=true;
		delete[] nomeFont;
		nomeFont = new char[20];
		strcpy(nomeFont, defaultTextFont);
		if(size==0)
			size=12;
		defaultSize = size;
		return 0;
	}
	else if(fname == BIGTEXTFONT)
	{
		isTTF=true;
		delete[] nomeFont;
		nomeFont = new char[20];
		strcpy(nomeFont, defaultTextFont);
		if(size==0)
			size=30;
		defaultSize = size;
		return 0;
	}
	else
*/
	if(fname == TEXTFONT)
	{
		if(size==0)
			size=12;
	}
	else if(fname == BIGTEXTFONT)
	{
		if(size==0)
			size=30;
		fname=TEXTFONT;
	}
	{
		if(size==0)
			size=50;
		defaultSize=size;
		if(fname.AfterLast('.')=="fon")
			return load_fon((char*)fname.c_str(), smrf);
		if(fname.AfterLast('.')=="ttf")
			return load_ttf((char*)fname.c_str(), smrf);
	}
	return 1;
}
//
//  Carica in memoria il font dal file .FON specificato
//  Ritorna il numero di caratteri letti
//
int LiooFont::load_fon(char *filename, SMRFile* smrf)
{
	
	FILE *filein=NULL;    // Puntatore al file del font
	int xx;      // Contatori generici
	long nbytes;     // Dimensione del bitmap del carattere
	int c_read;      // Contatore dei caratteri letti
	int baseline;	   // Baseline	
	
	
    int caratt;				        // byte letto dal file
	long indice;			        // posizione corrente nel file
	long inizio4BT;	                // posizione dei 4 byte che definiscono l'offset
	int position,placed;
	long startpoint;                // da dove comincia a leggere dal file (byte)
	long startpoint_tmp, flag_startpoint; // per file lunghi
	int rows,columns,maxcolumns;	// quante righe, colonne metto per carattere
	unsigned long dasommare1, dasommare2, dasommare;
	char *car_array=NULL;
	long array_indice;
	int firstchar, lastchar;
	
    caratt=0;
	indice=-1;
	array_indice=0;
	position=0;
	placed=0;
	//indicepos=0;
	startpoint=0;
	startpoint_tmp=0;
	flag_startpoint=0;
	dasommare=0;
	dasommare1=0;
	dasommare2=0;
	inizio4BT=0;
    c_read = 0;
	
	int ret;
	bool removeAfterUse=false;
	wxString fontFile;
	isTTF=false;
	
	
	if(smrf==NULL)
	{
		char dirx[200];
		strcpy (dirx, smrgetenv("SMR_FONT_PATH"));
		strcat(dirx,filename);
		filein = fopen(dirx, "rb");			
    }
	else
	{
		fontFile=smrf->GetFontPath(filename, removeAfterUse);
		if(fontFile!="")
			filein = fopen(fontFile, "rb");			
	}
	
	if(filein != NULL)
    {  
		// Legge la baseline
		
		ret=fseek(filein,538,SEEK_SET);
		if (ret) return(9);
		caratt = getc (filein);
		baseline=caratt;
		
		ret=fseek(filein,559,SEEK_SET);
		if (ret) return(9);
		caratt = getc (filein);
		firstchar=caratt;
		
		ret=fseek(filein,560,SEEK_SET);
		if (ret) return(9);
		caratt = getc (filein);
		lastchar=caratt;
		
		ret=fseek(filein,552,SEEK_SET);
		if (ret) return(9);
		caratt = getc (filein);
		rows=caratt; 
		
		
		ret=fseek(filein,557,SEEK_SET);
		if (ret) return(9);
		caratt = getc (filein);
		maxcolumns=(caratt/8)+1;
		if ((caratt%8)==0) maxcolumns--;
		
		// Ciclo di lettura dei caratteri
		for(xx = firstchar; xx <= lastchar; xx++)
		{
			// Legge le caratteristiche del carattere		
			
			inizio4BT = (586+((xx-firstchar-1)*4));
			
			ret=fseek(filein,inizio4BT,SEEK_SET);
			if (ret) return(9);
			caratt = getc (filein);
			columns = (caratt/8)+1;
			if ((caratt%8)==0) columns--; 
			nbytes=(rows*columns);
			
			ret=fseek(filein,inizio4BT+2,SEEK_SET);
			if (ret) return(9);
			caratt = getc (filein);
			
			dasommare1=caratt;
			
			ret=fseek(filein,inizio4BT+3,SEEK_SET);
			if (ret) return(9);
			caratt = getc (filein);
			dasommare2 = caratt;
			dasommare=dasommare1+(dasommare2<<8);
			startpoint_tmp=(464+dasommare+flag_startpoint);
			if (startpoint_tmp<startpoint) flag_startpoint=66000-464;
			startpoint=(464+dasommare+flag_startpoint);  
			
			ret=fseek(filein,startpoint,SEEK_SET);
			if (ret) return(9);
			array_indice=0;
			car_array = new char[nbytes];
			for (position=0;position<rows*columns;position++)
			{
				caratt = getc (filein);
				car_array[array_indice]=caratt;
				array_indice++;
			};
			
			
			// ora bisogna trasformare l'array mio (a colonne) nel formato a righe 
			
			
			item[xx].code = xx;
			item[xx].height = rows;
			item[xx].width = columns*8;
			item[xx].ascend = baseline;
			item[xx].nbytes = nbytes;
			item[xx].VU2UP = 0;
			item[xx].VU2DWN = 0;
			item[xx].VU2RG = 0;
			item[xx].VU2LF = 0;
			item[xx].DX = 0;
			item[xx].DY = 0;
			
			
			if(item[xx].width != 0)
			{
				// Alloca la memoria per il carattere
				if(nbytes != 0)
				{
					item[xx].image = new unsigned char[nbytes];
					for(position=0; position<(rows*columns); position++)
					{
						item[xx].image[position]=car_array[(position%columns)*rows+position/columns];
					};
					delete car_array;
					car_array=NULL; 
				}
				
				/* per calcolare la larghezza effetiva del carattere */
				/* nel file .FON ho l'ultimo byte -- ci serve l'ultimo bit */
				
				int yy,zz;
				unsigned char ret;
				int bytes_per_row;
				int bt, flagweight;
				
				
				bytes_per_row = (int)ceil((double)item[xx].width / 8);
				bt = 0;
				flagweight = 0;
				for(zz = 0; zz < item[xx].nbytes; zz++)
				{
					// Legge il byte
					for(yy = 7; yy >= 0; yy--)
					{
						if((8 * bt + yy) >= item[xx].width)
							break;
						ret = ((item[xx].image[zz])>>yy)&0x1;
						if ((ret == 1)&&(flagweight<=(8 * bt + 7 - yy)))
							flagweight = (8 * bt + 7 - yy);
					}
					// Passa al byte successivo della riga se c'e'
					if(bt < (bytes_per_row - 1))
						bt++;
					else
					{
						// Passa alla riga successiva
						bt = 0;
					}
				}
				item[xx].realwidth=flagweight+1;
				
				
				
			}	 
			
			c_read++;  // Contatore di caratteri
		}
		fclose(filein);
		if(removeAfterUse)
			wxRemove(fontFile);
    }
	else
	{
		char msg[500];
		sprintf(msg, "Error opening font: \"%s\"\n", filename);
		Message(msg);
		
		if(nomeFont!=NULL)
			delete nomeFont;
		nomeFont=NULL;
		return 0;
	}
	
	
	load_table(filename, smrf);
	
	if(nomeFont!=NULL)
		delete nomeFont;
	nomeFont=new char[strlen(filename)+1];
	strcpy(nomeFont,filename);
	
	return c_read;
	
};	

int LiooFont::load_ttf(char *filename, SMRFile* smrf)
{	
	wxString fontFile;
	bool removeAfterUse=false;

	if(smrf==NULL)
	{
		fontFile = smrgetenv("SMR_FONT_PATH");
		fontFile += filename;
    }
	else
	{
		fontFile=smrf->GetFontPath(filename, removeAfterUse);
	}
	
	if(nomeFont!=NULL)
		delete nomeFont;
	nomeFont=new char[255];
	isTTF=true;
	if(LoadTrueTypeFont(fontFile.c_str(), nomeFont, &hfont)==0)
	{
		load_table(filename, smrf);
		return 0;
	}
	return 1;
}

void LiooFont::load_table(char *filename, SMRFile* smrf)
{
	FILE *tablefile=NULL; // Puntatore al file con le informazioni su vari VU2.. e DX e DY
	char tablename[25];
	
	strcpy(tablename,filename);
	tablename[strlen(filename)-3]='\0'; // toglie estensione fon
	strcat(tablename,"tbl");	  
	
	bool removeAfterUse=false;
	wxString tblFile;
	
	
	if(smrf==NULL)
	{
		char dirx[200],*env;
		env= smrgetenv("SMR_FONT_PATH");
		strcpy (dirx, env);
		strcat(dirx,tablename);
		tablefile = fopen(dirx, "r");
	}
	else
	{
		tblFile=smrf->GetFontPath(tablename, removeAfterUse);
		if(tblFile!="")
			tablefile = fopen(tblFile, "rb");			
	}
	
	if(tablefile != NULL)
    {
		char riga[100];
		int tmpCar, tmpDW, tmpUP, tmpRG, tmpLF, tmpdx, tmpdy, ret;
		while(fgets(riga,100,tablefile)!=NULL)
		{
			ret=sscanf(riga,"%i %i %i %i %i %i %i",&tmpCar,&tmpDW,&tmpUP,&tmpRG,
				&tmpLF,&tmpdx,&tmpdy);

			if (ret==7 && tmpCar>0 && tmpCar<256) 
			{
				item[tmpCar].VU2UP = tmpUP;
				item[tmpCar].VU2DWN = tmpDW;
				item[tmpCar].VU2RG = tmpRG;
				item[tmpCar].VU2LF = tmpLF;
				item[tmpCar].DX = tmpdx;
				item[tmpCar].DY = tmpdy;
			}
			if (ret==7 && tmpCar==0)
			{
				if(tmpDW!=0)
					defaultSize=tmpDW;
			}
		}
		fclose(tablefile);	
		if(removeAfterUse)
			wxRemove(tblFile);
	}
}
