/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: lioofont.hpp,v 1.1 2005/07/25 16:30:52 bellinpo Exp $ */


#ifndef LIOOFON_HPP
#define LIOOFON_HPP

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>

typedef int FontID;

// Definisce il numero di pixel tra un carattere ed un altro nella stampa delle stringhe
#define INTERSPACE 0

// Questa struttura contiene le carateristiche del carattere,
// serve per gestire i font non proporzionali convertiti in bitmap

class SMRFile;


struct Carattere
{
	int code;   // Codice carattere
	int height; // Altezza del carattere
	int width;  // Larghezza del carattere
	int realwidth; // Larghezza effettiva del carattere
	int nbytes; // Occupazione di memoria in bytes
	int ascend; // Distanza dal top alla baseline in pixel
	unsigned char *image;  /* Immagine binaria del carattere codificata dall'alto verso il basso  */
	int VU2UP, VU2DWN, VU2RG, VU2LF, DX, DY;
};


class LiooFont
{	
public:
	LiooFont(char *filename, SMRFile* smrf=NULL, int fontSize=0);  // Crea il font e lo carica da disco o da dentro l'oggetto se esiste
	LiooFont(void);
	~LiooFont(void);
	
	// Loads a .FON or .TTF font,
	int load(char *filename, SMRFile* smrf=NULL, int fontSize=0);
	
	// Setta lo ID del font
	void SetID(FontID id);
	// Legge lo ID del font
	FontID GetID(void);
	// Restituisce l'altezza del carattere
	int get_height(int code);
	// Restituisce la larghezza del carattere
	int get_width(int code);
	// Restituisce 'ascend' del carattere
	int get_ascend(int code);
	// Restituisce VU2UP del carattere (distanza dal AbsPoint al margine superiore)
	int get_VU2UP(int code);
	// Restituisce VU2DWN del carattere (distanza dal AbsPoint al margine inferiore)
	int get_VU2DWN(int code);
	// Restituisce VU2RG del carattere (distanza dal AbsPoint al margine destro)
	int get_VU2RG(int code);
	// Restituisce VU2LF del carattere (distanza dal AbsPoint al margine sinistro)
	int get_VU2LF(int code);
	// Restituisce DX del carattere (offset orizontale)
	int get_DX(int code);
	// Restituisce DY del carattere (offset verticale)
	int get_DY(int code);

	void SetDimension(int code, int up, int dwn, int lft, int rgt, int dx, int dy);

	// Stampa sullo schermo il carattere di codice specificato
	void print(int x_pos, int y_pos, int code);
	// Stampa sullo schermo la sequenza di caratteri specificati
	void print_str(int x_pos, int y_pos, unsigned char *string);

	int GetTextWidth(unsigned char* txt);
	
	// Carica in memoria dalla tabella *.tbl se sono dentro l'oggetto
	void load_table(char *filename, SMRFile* smrf=NULL);
		
	char *GetNomeFont() {return nomeFont;};
	static void SetDefaultTextFont(const char* fontname);
private:
	char *nomeFont;       // Nome del font
	bool isTTF;
	void* hfont;
	int id;             // Identificatore (fornito dalla lista di font)
	int defaultSize;
	struct Carattere item[256];  // Tutti i caratteri del font
	
	int load_fon(char *filename, SMRFile* smrf=NULL);
	int load_ttf(char *filename, SMRFile* smrf=NULL);
	static char defaultTextFont[255];
};

#endif
