/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include "wx/wx.h"
#include "wx/dialog.h"
#include "wx/resource.h"
#include "drw/frmanage.hpp"
#include "lioo.hpp"
#include "par/partitur.hpp"
#include "drw/scorefrm.hpp"


#if defined (__WXGTK__)
#include "../wxr/xselectmeasdlg.wxr"
#include "../wxr/xselectmeasdlg.h"
#endif

#if defined(__WXMSW__)
#include "../wxr/selectmeasdlg.wxr"
#include "../wxr/selectmeasdlg.h"

#endif







class SelectMeasureDlg :public wxDialog
{
protected:
   	wxCheckBox		*checkSingle, *forceUnbeaming;	
	wxTextCtrl		*startMeasure;	
	wxTextCtrl		*endMeasure;
	
	wxButton	*OkButton;
	wxButton	*CancelButton;
	
	wxChoice	*choicepart;
	
	wxStaticText	*infotext;
	
	Partitura	*partitura;
	
	int		currentscore;
	int		num_of_measure;
	
	
	
	
public:	
	SelectMeasureDlg();
	//Costruttore
	
	~SelectMeasureDlg();
	//Distruttore
	
	
    void OnCancel(wxCommandEvent& event);
	//Metodo per la chiusura e la distruzione della finestra
	
	void OnOk(wxCommandEvent& event);
	//Metodo per la gestione dell'evento di Ok 
	
	void OnSingleCheck(wxCommandEvent& event);
	//Metodo per la gestione della selezione "single", azzera il contenuto 
	//textctrl relativo ad endmeasure.
	
	
	void SetPParam(Partitura *pPart);
	//Metodo per il Set up dei parametri relativi alla partitura:
	//si riceve il puntatore a partitura per avere accesso alle informazioni
	//necessarie
	
	bool CheckValues(int type);
	//Metodo per il controllo dell'input: si verifica la consistenza dei valori
	//inseriti in relazione al contesto. Il parametro type definisce la modalita'
	//singola battuta se il suo valore e' unitario, altrimenti la modalita' intervallo
	
	int GetStartMeasure();
	
	int GetEndMeasure();
	
	int GetType();
	
	int GetPart();
	
	bool GetForceUnbeaming();
	
	
private:
    DECLARE_EVENT_TABLE()
};




BEGIN_EVENT_TABLE(SelectMeasureDlg, wxDialog)
EVT_BUTTON(ID_BUTTONOK, SelectMeasureDlg::OnOk)
EVT_BUTTON(ID_BUTTONCANCEL, SelectMeasureDlg::OnCancel)
EVT_CHECKBOX(ID_SINGLECHECK, SelectMeasureDlg::OnSingleCheck)
END_EVENT_TABLE()



SelectMeasureDlg::SelectMeasureDlg() 
{
	
	checkSingle			= NULL;
	startMeasure		= NULL;
	endMeasure			= NULL;
	infotext			= NULL;
	choicepart			= NULL;
	forceUnbeaming		= NULL;
	
	OkButton			= NULL;
	CancelButton			= NULL;
	
	wxResourceParseData(SelectMeasure); 
	if (LoadFromResource(FrameManager::currFrame, "SelectMeasure"))
	{
		
		checkSingle			= (wxCheckBox *)wxFindWindowByName("SingleChoice", this);
		forceUnbeaming		= (wxCheckBox *)wxFindWindowByName("ForceUnbeaming", this);
		
		startMeasure			= (wxTextCtrl *)wxFindWindowByName("startmeasure", this);
		endMeasure			= (wxTextCtrl *)wxFindWindowByName("endmeasure", this);
		infotext				= (wxStaticText *)wxFindWindowByName("infotext", this);
		choicepart			= (wxChoice *)wxFindWindowByName("partselect", this);
		
		
		OkButton				= (wxButton *)wxFindWindowByName("okbutton", this);
		CancelButton			= (wxButton *)wxFindWindowByName("cancelbutton", this);
		
	}
	else
	{
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
	}
	
	
	
	currentscore=-1;
	num_of_measure=0;
	partitura=NULL;
	
	
}

SelectMeasureDlg::~SelectMeasureDlg()
{
	if(checkSingle !=NULL) 
		delete checkSingle;
	if(forceUnbeaming !=NULL) 
		delete forceUnbeaming;
	if(startMeasure !=NULL) 
		delete startMeasure;
	if(endMeasure!=NULL) 
		delete endMeasure;
	
	if(choicepart!=NULL) 
		delete choicepart;
	
	
	if(OkButton !=NULL) 
		delete OkButton;
	if(CancelButton !=NULL) 
		delete CancelButton;
	
	
}

int SelectMeasureDlg::GetStartMeasure()
{
	long result;
	wxString value = startMeasure->GetValue();
	value.ToLong(&result);
	return ((int) result);
}
int SelectMeasureDlg::GetEndMeasure()
{
	long result;
	wxString value = endMeasure->GetValue();
	value.ToLong(&result);
	return ((int) result);
}
int SelectMeasureDlg::GetType()
{
	if(checkSingle->GetValue())
		return 1;
	else
		return 0;
}

void SelectMeasureDlg::OnOk(wxCommandEvent& event)
{
	int type;
	if(checkSingle->GetValue())
		type = 1;
	else
		type=0;
	
	if (CheckValues(type))
		EndModal(wxID_OK);
}

void SelectMeasureDlg::OnCancel(wxCommandEvent& event)
{
	EndModal(wxID_CANCEL);
}



void SelectMeasureDlg::OnSingleCheck(wxCommandEvent& event)
{
	
	if(checkSingle->GetValue())
		endMeasure->Clear();
	else
	{
		wxString tmp;
		tmp.sprintf("%d",num_of_measure);
		endMeasure->SetValue(tmp);
	}
}


void SelectMeasureDlg::SetPParam(Partitura *pPart)
{
	wxString tmp,tmp2;
	Spartito *currSpa=NULL;
	partitura = pPart;
	currentscore=pPart->GetCurrentScore();
	num_of_measure=pPart->GetNumeroBattute();
	tmp.sprintf("%d",num_of_measure);
	tmp2=infotext->GetLabel();
	tmp2.Append(tmp);
	infotext->SetLabel(tmp2);
	startMeasure->SetValue(wxString("1"));
	endMeasure->SetValue(tmp);
	
	choicepart->Append("Main Score");
	tmp.Clear();
	int numParts = pPart->GetNumSpa();
	for(int i=0;i<numParts;i++)
	{
		currSpa=pPart->GetSpartNum(i);
		if(currSpa->IsVisible())
		{
			tmp.sprintf("%s",currSpa->GetNome());
			if(tmp.IsEmpty())
			{
				tmp.Clear();
				tmp.sprintf("Part %d",i+1);
			}
			choicepart->Append(tmp);
		}
	}
	
	int part = pPart->GetCurrentScore();
	choicepart->SetSelection(part);
	if (part!=0) choicepart->Enable(FALSE);
}





bool SelectMeasureDlg::CheckValues(int type)
{
	wxString start, end;
	long first, last;
	wxString message;
	bool checkval=true;
	
	switch (type)
	{
	case 0:
		
		start=startMeasure->GetValue();
		end=endMeasure->GetValue();
		if(start.IsEmpty())
		{
			message.Append("Insert a value for Start Measure\n");
			checkval = false;
		}
		if(end.IsEmpty())
		{
			message.Append("Insert a value for End Measure\n");
			checkval = false;
		}
		
		if(!start.IsNumber())
		{
			message.Append("Start Measure value must be a number\n");
			checkval = false;
		}
		if(!end.IsNumber())
		{
			message.Append("End Measure value must be a number\n");
			checkval = false;
		}
		
		start.ToLong(&first);
		end.ToLong(&last);
		if((first<0)||(last<0))
		{
			message.Append("Values must be positive\n");
			checkval = false;
		}
		if(last>num_of_measure)
		{
			message.Append("End Measure value out of range\n");
			checkval = false;
		}
		
		if(!checkval)
			wxMessageBox(message,"Error!");
		break;
		
		
	case 1:
		
		start=startMeasure->GetValue();
		if(!start.IsNumber())
		{
			message.Append("Start Measure value must be a number");
			checkval = false;
		}
		start.ToLong(&first);
		if(first<0)
		{
			message.Append("Values must be positive\n");
			checkval = false;
		}
		break;
	}
	
	return checkval;
}

int SelectMeasureDlg::GetPart()
{
	
	return (choicepart->GetSelection());
}

bool SelectMeasureDlg::GetForceUnbeaming()
{
	return forceUnbeaming->GetValue();
}


/************Funzione di chiamata alla dialog**************************/



Bool BeamingDlg(Partitura *pPart, int *start, int *end , int *type, int *part, bool *forceUnbeaming)
{
	wxString control;
	
	SelectMeasureDlg *dialog=new SelectMeasureDlg;
	dialog->SetTitle("Beaming");
	dialog->SetPParam(pPart);
	
	dialog->Centre();
	if(dialog->ShowModal()==wxID_OK)
	{
		*start=dialog->GetStartMeasure();
		*end=dialog->GetEndMeasure();
		*type=dialog->GetType();
		*part=dialog->GetPart();
		*forceUnbeaming=dialog->GetForceUnbeaming();
		return TRUE;
	}
	else 
		return FALSE;
	
}

Bool UpDownDlg(Partitura *pPart, int *start, int *end , int *type, int *part)
{
	wxString control;
	
	SelectMeasureDlg *dialog=new SelectMeasureDlg;
	dialog->SetTitle("UpDown Stem");
	dialog->SetPParam(pPart);
	
	dialog->Centre();
	if(dialog->ShowModal()==wxID_OK)
	{
		*start=dialog->GetStartMeasure();
		*end=dialog->GetEndMeasure();
		*type=dialog->GetType();
		*part=dialog->GetPart();
		return TRUE;
	}
	else 
		return FALSE;
	
}