/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */



#include "../wxr/casellaimgdlgdrag.wxr"	
#include "../wxr/casellaimgdlgdrag.h"
#include "gui/dlg/casellaimgdlg.hpp"
#include <wx/file.h>

BEGIN_EVENT_TABLE(CasellaImgDlg, wxDialog)
EVT_BUTTON(ID_BUTTONOK, CasellaImgDlg::OnOk)
EVT_BUTTON(ID_BUTTONCANC, CasellaImgDlg::OnCancel)
EVT_BUTTON(ID_BUTTONSFOGLIA, CasellaImgDlg::OnSfoglia)
EVT_BUTTON(ID_BUTTONDRAG, CasellaImgDlg::OnDrag)
END_EVENT_TABLE()


CasellaImgDlg::CasellaImgDlg()
{	
	wxResourceParseData(casellaimgdlg);
	if (LoadFromResource(FrameManager::currFrame, "casellaimgdlg"))
	{	
		path=(wxTextCtrl *)wxFindWindowByName("path",this);
		x=(wxTextCtrl *)wxFindWindowByName("x",this);
		y=(wxTextCtrl *)wxFindWindowByName("y",this);
		width=(wxTextCtrl *)wxFindWindowByName("width",this);
		height=(wxTextCtrl *)wxFindWindowByName("height",this);
	}
	else
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
}


CasellaImgDlg::~CasellaImgDlg()
{
	if(path!=NULL)
		delete path;
	if(x!=NULL)
		delete x;
	if(y!=NULL)
		delete y;
	if(width!=NULL)
		delete width;
	if(height!=NULL)
		delete height;
	
}



void CasellaImgDlg::OnOk(wxCommandEvent& event)
{
	wxFile *file=NULL;
	bool flag=file->Exists(GetPath());
	
	if ((x->GetValue().IsEmpty())||	(y->GetValue().IsEmpty()) )
	{
		wxMessageBox("Empty Coordinates!", "Message",  wxOK , NULL, -1, -1);
		return ;
	}
	if ((width->GetValue().IsEmpty())||
		(height->GetValue().IsEmpty()))
	{
		wxMessageBox("Empty Dimension!", "Message",  wxOK , NULL, -1, -1);
		return ;
	}
	
	if ( path->GetValue().IsEmpty() || !flag )
	{
		wxMessageBox("Empty File name!", "Message",  wxOK , NULL, -1, -1);
		return ;
	}
	
	if ((!width->GetValue().IsNumber())||
		(!height->GetValue().IsNumber()) || 
		(GetWidth()<5)||
		(GetHeight()<5) )
	{	
		wxMessageBox("Invalid Dimension!", "Message", wxOK |  wxCENTRE, NULL, -1, -1);
		return ;
	}
	
	EndModal(wxID_OK);
}

void CasellaImgDlg::OnCancel(wxCommandEvent& event)
{
	EndModal(wxID_CANCEL);
}

void CasellaImgDlg::OnSfoglia(wxCommandEvent& event)
{
	wxFileDialog *dialog = new wxFileDialog(this, "Choose a file", "", "", "*.*", 0, wxDefaultPosition);
	dialog->SetTitle("Browsing");
	if (dialog->ShowModal()==wxID_OK)
	{
		SetPath(dialog->GetPath());
	}
}

void CasellaImgDlg::OnDrag(wxCommandEvent& event)
{
	Scr->GetMouse()->SetImgDragSel(GetHeight(),GetWidth());
	Show(false);
}

float CasellaImgDlg::GetX()
{
	double n;
	(x->GetValue()).ToDouble(&n);
	return (float)n;
}

void CasellaImgDlg::SetX(wxString xx)
{
	x->SetValue(xx);
}

float CasellaImgDlg::GetY()
{
	double n;
	(y->GetValue()).ToDouble(&n);
	return (float)n;
}

void CasellaImgDlg::SetY(wxString yy)
{
	y->SetValue(yy);
}

int CasellaImgDlg::GetWidth()
{
	long n;
	(width->GetValue()).ToLong(&n);
	return n;
}

void CasellaImgDlg::SetWidth(wxString w)
{
	width->SetValue(w);
}

int CasellaImgDlg::GetHeight()
{
	long n;
	(height->GetValue()).ToLong(&n);
	return n;
}

void CasellaImgDlg::SetHeight(wxString h)
{
	height->SetValue(h);
}


wxString CasellaImgDlg::GetPath()
{
	return path->GetValue();
}

void CasellaImgDlg::SetPath(wxString t)
{
	path->SetValue(t);
}







































