/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include "wx/fontdlg.h"
#include "../wxr/casellatxtdlgdrag.wxr"	
#include "../wxr/casellatxtdlgdrag.h"
#include "gui/dlg/casellatxtdlg.hpp"
#include "drw/scorecan.hpp"
#include "drw/musicedt.hpp"


BEGIN_EVENT_TABLE(CasellaTxtDlg, wxDialog)
EVT_BUTTON(ID_BUTTONOK, CasellaTxtDlg::OnOk)
EVT_BUTTON(ID_BUTTONCANC, CasellaTxtDlg::OnCancel)
EVT_BUTTON(ID_BUTTONFONT, CasellaTxtDlg::OnFont)
EVT_BUTTON(ID_BUTTONDRAG, CasellaTxtDlg::OnDrag)	
END_EVENT_TABLE()


CasellaTxtDlg::CasellaTxtDlg()
{	
	wxResourceParseData(casellatxtdlg);
	if (LoadFromResource(FrameManager::currFrame, "casellatxtdlg"))
	{	
		testo=(wxTextCtrl *)wxFindWindowByName("testo",this);
		x=(wxTextCtrl *)wxFindWindowByName("x",this);
		y=(wxTextCtrl *)wxFindWindowByName("y",this);
		align=(wxChoice *)wxFindWindowByName("choice",this);
		align->SetSelection(0);
		font.SetFamily(wxDEFAULT);
		font.SetPointSize(14);
		font.SetFaceName("Arial");
		testo->SetFont(font);

	}
	else
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
}


CasellaTxtDlg::~CasellaTxtDlg()
{
	if(testo!=NULL)
		delete testo;
	if(x!=NULL)
		delete x;
	if(y!=NULL)
		delete y;	
	if(align!=NULL)
		delete align;	
}



void CasellaTxtDlg::OnOk(wxCommandEvent& event)
{
	if	((testo->GetValue().IsEmpty()) )
	{
		wxMessageBox("Empy Text!", "Message",  wxOK , NULL, -1, -1);
		EndModal(wxID_OK);
		return ;
	}
	if ((x->GetValue().IsEmpty())||	(y->GetValue().IsEmpty()) )
	{
		wxMessageBox("Empty Coordinates!", "Message",  wxOK , NULL, -1, -1);
		return ;
	}
	if ((align->GetSelection()==-1))
	{
		wxMessageBox("Specify the Alignment!", "Message",  wxOK , NULL, -1, -1);
		return ;
	}
	EndModal(wxID_OK);
}

void CasellaTxtDlg::OnCancel(wxCommandEvent& event)
{
	EndModal(wxID_CANCEL);
}

void CasellaTxtDlg::OnFont(wxCommandEvent& event)
{
	wxFontData data;
	data.SetInitialFont(testo->GetFont());
	data.SetAllowSymbols(false);
	wxFontDialog dialog(this, &data);
	dialog.Centre();   
	if (dialog.ShowModal() == wxID_OK)
	{
		wxFontData retData = dialog.GetFontData();
		font = retData.GetChosenFont();
		testo->SetFont(font);
	}
}


void CasellaTxtDlg::OnDrag(wxCommandEvent& event)
{
	Scr->GetMouse()->SetTxtDragSel();
	Show(false);
}

float CasellaTxtDlg::GetX()
{
	double n;
	(x->GetValue()).ToDouble(&n);
	return n;
}

void CasellaTxtDlg::SetX(wxString xx)
{
	x->SetValue(xx);
}

float CasellaTxtDlg::GetY()
{
	double n;
	(y->GetValue()).ToDouble(&n);
	return n;
}

void CasellaTxtDlg::SetY(wxString yy)
{
	y->SetValue(yy);
}

Align CasellaTxtDlg::GetAlign()
{
	switch(align->GetSelection())
	{
	case 0:
		return A_Left;;
		break;
	case 1:
		return A_Right;
		break;
	case 2:
		return A_Centre;
		break;
	}
	return A_Right;
}

void CasellaTxtDlg::SetAlign(int c)
{
	align->SetSelection(c);
}

wxString CasellaTxtDlg::GetTesto()
{
	return testo->GetValue();
}

void CasellaTxtDlg::SetTesto(wxString t)
{
	testo->SetValue(t);
}

wxFont CasellaTxtDlg::GetFont()
{
	return testo->GetFont();
}

void CasellaTxtDlg::SetFont(wxFont f)
{
	testo->SetFont(f);
}






