/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include "wx/wx.h"
#include "wx/dialog.h"
#include "wx/resource.h"
#include "drw/frmanage.hpp"
#include "lioo.hpp"
#include "par/partitur.hpp"
#include "drw/scorefrm.hpp"


#if defined (__WXGTK__)
#include "../wxr/xchecklogdlg.wxr"
#include "../wxr/xchecklogdlg.h"
#endif

#if defined(__WXMSW__)
#include "../wxr/checklogdlg.wxr"
#include "../wxr/checklogdlg.h"

#endif







class CheckDlg :public wxDialog
{
protected:
   	wxCheckBox		*checkSingle;	
	wxTextCtrl		*startMeasure;	
	wxTextCtrl		*endMeasure;
	wxListBox		*log;
	wxStaticText	*infotext;
	
	wxButton	*checkButton;
	wxButton	*closeButton;
	
	Partitura	*partitura;
	
	int		currentscore;
	int		num_of_measure;
	
	
	
	
public:	
	CheckDlg();
	//Costruttore
	
	~CheckDlg();
	//Distruttore
	
	void OnCheck(wxCommandEvent& event);
	//Metodo per la gestione dell'evento di pressione del bottone check
	
    void OnClose(wxCommandEvent& event);
	//Metodo per la chiusura e la distruzione della finestra
	
	void OnChoice(wxCommandEvent& event);
	//Metodo per la gestione dell'evento double click nella finestra di log
	//per la visualizzazione delle battute inconsistenti
	
	void OnSingleCheck(wxCommandEvent& event);
	//Metodo per la gestione della selezione "single", azzera il contenuto 
	//textctrl relativo ad endmeasure.
	
	void SetPParam(Partitura *pPart);
	//Metodo per il Set up dei parametri relativi alla partitura:
	//si riceve il puntatore a partitura per avere accesso alle informazioni
	//necessarie
	
	bool CheckValues(int type);
	//Metodo per il controllo dell'input: si verifica la consistenza dei valori
	//inseriti in relazione al contesto. Il parametro type definisce la modalita'
	//singola battuta se il suo valore e' unitario, altrimenti la modalita' intervallo
	
	bool Check(int type);
	//Metodo per la gestione del check e aggiornamento della finestra per la visualizzazione
	//della lista delle battute incosistenti
	
	void ViewMeasure();
	//Metodo per la visualizzazione della battuta incosistente selezionata nella log window
	//direttamente sul canvas del music editor. E' stato usato usato il gobatt che consente
	//di visualizzare la battuta scelta come prima battuta a sx della pagina del canvas
	
private:
    DECLARE_EVENT_TABLE()
};




BEGIN_EVENT_TABLE(CheckDlg, wxDialog)
EVT_BUTTON(ID_BUTTONCHECK, CheckDlg::OnCheck)
EVT_BUTTON(ID_BUTTONCLOSE, CheckDlg::OnClose)
EVT_LISTBOX_DCLICK(ID_LISTBOX6005, CheckDlg::OnChoice)
EVT_CHECKBOX(ID_CHECKBOX106, CheckDlg::OnSingleCheck)	
END_EVENT_TABLE()



CheckDlg::CheckDlg() 
{
	
	checkSingle			= NULL;
	startMeasure		= NULL;
	endMeasure			= NULL;
	log					= NULL;
	infotext			= NULL;
	
	checkButton			= NULL;
	closeButton			= NULL;
	
	wxResourceParseData(CheckMeasure); 
	if (LoadFromResource(FrameManager::currFrame, "CheckMeasure"))
	{
		
		checkSingle			= (wxCheckBox *)wxFindWindowByName("SingleChoice", this);
		
		startMeasure			= (wxTextCtrl *)wxFindWindowByName("startmeasure", this);
		endMeasure			= (wxTextCtrl *)wxFindWindowByName("endmeasure", this);
		log					= (wxListBox *)wxFindWindowByName("log", this);
		infotext				= (wxStaticText *)wxFindWindowByName("infotext", this);
		
		
		checkButton			= (wxButton *)wxFindWindowByName("checkbutton", this);
		closeButton			= (wxButton *)wxFindWindowByName("closebutton", this);
		
	}
	else
	{
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
	}
	
	
	
	SetModal(FALSE);
	currentscore=-1;
	num_of_measure=0;
	partitura=NULL;
	
	
}

CheckDlg::~CheckDlg()
{
	if(checkSingle !=NULL) 
		delete checkSingle;
	if(startMeasure !=NULL) 
		delete startMeasure;
	if(endMeasure!=NULL) 
		delete endMeasure;
	if(log!=NULL) 
		delete log;
	
	if(checkButton !=NULL) 
		delete checkButton;
	if(closeButton !=NULL) 
		delete closeButton;
	
	
}




void CheckDlg::OnCheck(wxCommandEvent& event)
{
	
	bool choice = checkSingle->GetValue();
	log->Clear();
	switch (choice)
	{
	case true:
		if(CheckValues(1))
			Check(1);
		break;
	case false:
		if(CheckValues(0))
			Check(0);
		break;
	}
	
	
}

void CheckDlg::OnClose(wxCommandEvent& event)
{
	Destroy();
}

void CheckDlg::OnChoice(wxCommandEvent& event)
{
	wxString control;
	control = log->GetStringSelection();
	
	if((control!="Measures are time consistent")&&(control!="Measure is time consistent"))
		ViewMeasure();
}

void CheckDlg::OnSingleCheck(wxCommandEvent& event)
{
	
	if(checkSingle->GetValue())
		endMeasure->Clear();
	else
	{
		wxString tmp;
		tmp.sprintf("%d",num_of_measure);
		endMeasure->SetValue(tmp);
	}
}


void CheckDlg::SetPParam(Partitura *pPart)
{
	wxString tmp,tmp2;
	partitura = pPart;
	currentscore=pPart->GetCurrentScore();
	num_of_measure=pPart->GetNumeroBattute();
	tmp.sprintf("%d",num_of_measure);
	tmp2=infotext->GetLabel();
	tmp2.Append(tmp);
	infotext->SetLabel(tmp2);
	startMeasure->SetValue(wxString("1"));
	endMeasure->SetValue(tmp);
}


void CheckDlg::ViewMeasure()
{
	wxString selection;
	
	int measure=0;
	selection=log->GetStringSelection();
	
	if(currentscore==0)
	{
		wxString tmp = selection.AfterFirst(',');
		sscanf(tmp.c_str()," measure %d",&measure);
	}
	else
		sscanf(selection.c_str(),"Measure %d",&measure);
	((wxMEScoreFrame *)GetParent())->PrepareCanvas();
	partitura->GoBatt(measure);
	((wxMEScoreFrame *)GetParent())->UnprepareCanvas();
}


bool CheckDlg::CheckValues(int type)
{
	wxString start, end;
	long first, last;
	wxString message;
	bool checkval=true;
	
	switch (type)
	{
	case 0:
		
		start=startMeasure->GetValue();
		end=endMeasure->GetValue();
		if(start.IsEmpty())
		{
			message.Append("Insert a value for Start Measure\n");
			checkval = false;
		}
		if(end.IsEmpty())
		{
			message.Append("Insert a value for End Measure\n");
			checkval = false;
		}
		
		if(!start.IsNumber())
		{
			message.Append("Start Measure value must be a number\n");
			checkval = false;
		}
		if(!end.IsNumber())
		{
			message.Append("End Measure value must be a number\n");
			checkval = false;
		}
		
		start.ToLong(&first);
		end.ToLong(&last);
		if((first<0)||(last<0))
		{
			message.Append("Values must be positive\n");
			checkval = false;
		}
		if(last>num_of_measure)
		{
			message.Append("End Measure value out of range\n");
			checkval = false;
		}
		
		if(!checkval)
			wxMessageBox(message,"Error!");
		break;
		
		
	case 1:
		
		start=startMeasure->GetValue();
		if(!start.IsNumber())
		{
			message.Append("Start Measure value must be a number");
			checkval = false;
		}
		start.ToLong(&first);
		if(first<0)
		{
			message.Append("Values must be positive\n");
			checkval = false;
		}
		break;
	}
	
	return checkval;
}

bool CheckDlg::Check(int type)
{
	Spartito	*tmpSpa=NULL;
	wxString message;
	wxString start, end;
	const char *partname;
	long start_bat, end_bat;
	int num_part=0;
	bool *checklayer=NULL;
	float check_value[NUM_MAX_LAYER]; //Numero di Layer
	
	int i;
	
	checklayer = new bool[NUM_MAX_LAYER];
	for(i=0;i<NUM_MAX_LAYER;i++) 
		checklayer[i]=true;
	
	message.Clear();
	switch(type)
	{
	case 1:	//caso relativo al check di battute singole
		start=startMeasure->GetValue();
		start.ToLong(&start_bat);
		if(currentscore==0)
		{
			tmpSpa=partitura->GetFirstSpa();
			num_part=1;
			while(tmpSpa!=NULL)
			{
				if(!tmpSpa->GetBattNum(start_bat-1)->CheckBatt(checklayer, check_value))
				{
					partname=tmpSpa->GetNome();
					wxString name;
					name.sprintf("%s",partname);
					if(!name.IsEmpty())
						message.sprintf("%s, measure %d, ",partname,start_bat);
					else
						message.sprintf("Part %d, measure %d, ",num_part,start_bat);
					wxString msg;
					for(int j=0;j<NUM_MAX_LAYER;j++)
					{
						if(!checklayer[j])
						{
							msg.sprintf("Layer %d, Gap %6.4f",j+1,check_value[j]);
							log->Append(message+msg);
						}
						else
							msg.Clear();
					}
				}
				tmpSpa=partitura->GetNextSpa(tmpSpa);
				num_part++;
			}
			if(log->Number()==0)
			{
				message.Append("Measures are time consistent");
				log->Append(message);
			}
		}
		else
		{
			tmpSpa=partitura->GetSpartNum(currentscore-1);
			if(!tmpSpa->GetBattNum(start_bat-1)->CheckBatt(checklayer, check_value))
			{
				message.sprintf("Measure %d, ",(int)start_bat);
				wxString msg;
				for(int j=0;j<NUM_MAX_LAYER;j++)
				{
					if(!checklayer[j])
					{
						msg.sprintf("Layer %d, Gap %6.4f",j+1,check_value[j]);
						log->Append(message+msg);
					}
					else
						msg.Clear();
				}
			}
			else
			{
				message.Append("Measure is time consistent");
				log->Append(message);
			}
		}
		break;
	case 0: //caso relativo al check di un intervallo di battute
		start=startMeasure->GetValue();
		start.ToLong(&start_bat);
		end=endMeasure->GetValue();
		end.ToLong(&end_bat);
		if(currentscore==0)
		{
			tmpSpa=partitura->GetFirstSpa();
			num_part=1;
			while(tmpSpa!=NULL)
			{
				for(i=(int)start_bat-1;i<(int)end_bat;i++)
				{
					if(!tmpSpa->GetBattNum(i)->CheckBatt(checklayer, check_value))
					{
						partname=tmpSpa->GetNome();
						if(strcmp(partname,"\0")!=0)
							message.sprintf("%s, measure %d, ",partname,i+1);
						else
							message.sprintf("Part %d, measure %d, ",num_part,i+1);
						wxString msg;
						for(int j=0;j<NUM_MAX_LAYER;j++)
						{
							if(!checklayer[j])
							{
								msg.sprintf("Layer %d, Gap %6.4f",j+1,check_value[j]);
								log->Append(message+msg);
								msg.Clear();
							}
							else
								msg.Clear();
						}
						
						
					}
				}
				tmpSpa=partitura->GetNextSpa(tmpSpa);
				num_part++;
			}
			if(log->Number()==0)
			{
				message.Append("Measures are time consistent");
				log->Append(message);
			}
		}
		else
		{
			tmpSpa=partitura->GetSpartNum(currentscore-1);
			
			for(i=(int)start_bat-1;i<(int)end_bat;i++)
			{
				if(!(tmpSpa->GetBattNum(i))->CheckBatt(checklayer, check_value))
				{
					message.sprintf("Measure %d, ",i+1);
					wxString msg;
					for(int j=0;j<NUM_MAX_LAYER;j++)
					{
						if(!checklayer[j])
						{
							msg.sprintf("Layer %d, Gap %6.4f",j+1,check_value[j]);
							log->Append(message+msg);
							msg.Clear();
						}
						else
							msg.Clear();
					}
				}
			}
			
			if(log->Number()==0)
			{
				message.Append("Measures are time consistent");
				log->Append(message);
			}
		}
		break;
	}
	if(checklayer!=NULL)
		delete checklayer;
	return true;
}


/************Funzione di chiamata alla dialog**************************/



Bool CheckLogDlg(Partitura *pPart)
{
	wxString control;
	
	CheckDlg *dialog=new CheckDlg;
	dialog->SetPParam(pPart);
	
	dialog->Centre();
	if(dialog->Show(TRUE))
		return TRUE;
	else 
		return FALSE;
	
}