/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include "wx/wx.h"
#include "wx/dialog.h"
#include "wx/resource.h"

#include "../wxr/editintervals.h"
#include "../wxr/editintervals.wxr"

#include "lioo.hpp"
#include "stdlib.h"
#include "stdio.h"

#include "int/intestes.hpp"
#include "int/forcella.hpp"
#include "drw/frmanage.hpp"

// per il redraw
#include "drw/musicedt.hpp"
extern wxMusicEditor *MusicEditor;

class EditIntervalsDlg :public wxDialog
{
protected:
	wxComboBox	*comboLine;
	
	wxRadioBox  *radioUpDown;
	
	int init_updown;
	int curr_updown;
	int init_line;
	int curr_line;
	
	IntEsteso *pIntEsteso;
	
public:	
	EditIntervalsDlg(IntEsteso *ptr);
	~EditIntervalsDlg();
	
	void UpdateValue();
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
    void OnApply(wxCommandEvent& event);
	
	
private:
    DECLARE_EVENT_TABLE()   
};

/****************************************************/
BEGIN_EVENT_TABLE(EditIntervalsDlg, wxDialog)
EVT_BUTTON(ID_BUTTONOK, EditIntervalsDlg::OnOk)
EVT_BUTTON(ID_BUTTONCANCEL, EditIntervalsDlg::OnCancel)
EVT_BUTTON(ID_BUTTONAPPLY, EditIntervalsDlg::OnApply)
END_EVENT_TABLE()
/****************************************************/

EditIntervalsDlg::EditIntervalsDlg(IntEsteso *ptr)
{
	
	//assegnazione puntatori
	pIntEsteso = ptr;
	
	wxResourceParseData(editinterval);
	
	if (LoadFromResource(FrameManager::currFrame, "editinterval"))
	{
		comboLine = (wxComboBox *)wxFindWindowByName("comboLine", this);
		
		radioUpDown	= (wxRadioBox *)wxFindWindowByName("radioUpDown", this);
		
	}
	
	else
	{
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
	}
	
	SetModal(FALSE);
	
	comboLine->Append("Continuous");
	comboLine->Append("Dashed");
	comboLine->Append("Dotted");
	
	//Inizializzazione di comboLine
	init_line=pIntEsteso->GetLineType();
	comboLine->SetSelection(init_line);
	
	//Inizializzazione di radioUpDown
	init_updown=pIntEsteso->GetSopra();
	if(init_updown)
		radioUpDown->SetSelection(0);
	else
		radioUpDown->SetSelection(1);
	
}


EditIntervalsDlg::~EditIntervalsDlg()
{
	if(comboLine !=NULL) 
		delete comboLine;
	if(radioUpDown !=NULL) 
		delete radioUpDown;
	
}

void EditIntervalsDlg::OnOk(wxCommandEvent& event)
{		
	UpdateValue();
	Destroy();
}

void EditIntervalsDlg::OnApply(wxCommandEvent& event)
{		
	UpdateValue();
}

void EditIntervalsDlg::OnCancel(wxCommandEvent& event)
{
	//Recupera i valori iniziali (init), li impone e chiude la finestra di dialogo
	
	pIntEsteso->SetSopra(init_updown);
	pIntEsteso->SetLineType((LineType)init_line);
	MusicEditor->RedrawViews(0,TRUE);
	Destroy();
}

void EditIntervalsDlg::UpdateValue()
{
	curr_line=comboLine->GetSelection();
	pIntEsteso->SetLineType((LineType)curr_line);
	
	curr_updown=radioUpDown->GetSelection();	  
		  if(curr_updown==0)
			  pIntEsteso->SetSopra(TRUE);
		  else
			  pIntEsteso->SetSopra(FALSE);
		  
		  MusicEditor->RedrawViews(0,TRUE);
		  
}

Bool EditingIntervalsDlg(IntEsteso *pIntEsteso)
{
	
	wxString control;
	
	EditIntervalsDlg *dialog = new EditIntervalsDlg(pIntEsteso);
	dialog->Centre();
	if (dialog->Show(TRUE)) 
	{
		return TRUE;
	}
	else 
		return FALSE;
	
}
