/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include "wx/wx.h"
#include "wx/dialog.h"
#include "lioo.hpp"
#include "stdlib.h"
#include "stdio.h"
#include "drw/frmanage.hpp"
#include "drw/drawobje.hpp"
#include "wx/resource.h"

#include "../wxr/editmarkers.h"
#include "../wxr/editmarkers.wxr"

#include "str/arcogiu.hpp"
#include "str/arcosu.hpp"
#include "str/corda.hpp"
#include "str/pedaldown.hpp"
#include "str/pedalup.hpp"
#include "str/pizzicat.hpp"
#include "str/ponticel.hpp"
#include "str/punta.hpp"
#include "str/tallone.hpp"
#include "str/tastiera.hpp"
#include "str/timpano.hpp"
#include "str/strument.hpp"

#include "kor/koreanorn.hpp"

// per il redraw
#include "drw/musicedt.hpp"
extern wxMusicEditor *MusicEditor;

class EditMarkersDlg :public wxDialog
{
protected:
	
    wxRadioBox  *radioUpDown;
	
	DrawObject *pDrawObject;
	
	int init_updown;
	int curr_updown;
	
public:	
	EditMarkersDlg(DrawObject *ptr);
	~EditMarkersDlg();
	
	void UpdateValue();
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
    void OnApply(wxCommandEvent& event);
	
	void SetAboveNota(Bool b);
	Bool GetAboveNota();
	
private:
    DECLARE_EVENT_TABLE()   
};

/****************************************************/
BEGIN_EVENT_TABLE(EditMarkersDlg, wxDialog)
EVT_BUTTON(ID_BUTTONOK, EditMarkersDlg::OnOk)
EVT_BUTTON(ID_BUTTONCANCEL, EditMarkersDlg::OnCancel)
EVT_BUTTON(ID_BUTTONAPPLY, EditMarkersDlg::OnApply)
END_EVENT_TABLE()
/****************************************************/

EditMarkersDlg::EditMarkersDlg(DrawObject *ptr)
{
	
	//assegnazione puntatori
	
	pDrawObject = ptr;
	
	wxResourceParseData(editmarker);
	
	if (LoadFromResource(FrameManager::currFrame, "editmarker"))
		
	{
		radioUpDown			= (wxRadioBox *)wxFindWindowByName("radioUpDown", this);  
	}
	
	else
	{
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
	}
	
	SetModal(FALSE);
	
	//Inizializzazione di radioUpDown con salvataggio dei valori iniziali.
	
	init_updown=GetAboveNota();
	if(init_updown)
		radioUpDown->SetSelection(0);
	else
		radioUpDown->SetSelection(1);
	
}


EditMarkersDlg::~EditMarkersDlg()
{
	if(radioUpDown !=NULL) 
		delete radioUpDown;
}

void EditMarkersDlg::OnOk(wxCommandEvent& event)

//Impone i valori correnti (curr) e chiude la finestra di dialogo.
{		
	UpdateValue();
	Destroy();
}

void EditMarkersDlg::OnApply(wxCommandEvent& event)

//Impone i valori correnti (curr).
{		
	UpdateValue();
}

void EditMarkersDlg::OnCancel(wxCommandEvent& event)

//Recupera i valori iniziali (init), li impone e chiude la finestra di dialogo.
{
	SetAboveNota(init_updown);
	MusicEditor->RedrawViews(0,TRUE);
	Destroy();
}

void EditMarkersDlg::UpdateValue()

//Impone i valori correnti (curr)
{
	curr_updown=radioUpDown->GetSelection();
	if (curr_updown==0)
		SetAboveNota(TRUE);
	else
		SetAboveNota(FALSE);
	
	MusicEditor->RedrawViews(0,TRUE);
}

void EditMarkersDlg::SetAboveNota(Bool b)
{
	Strumento *pStr=dynamic_cast<Strumento*>(pDrawObject);
	if(pStr!=NULL)
		pStr->SetAboveNota(b);
	else
	{
		IntEsteso *pInt=dynamic_cast<IntEsteso*>(pDrawObject);
		if(pInt!=NULL)
			pInt->SetSopra(b);
		else
		{
			Abbellimento *pAbb=dynamic_cast<Abbellimento*>(pDrawObject);
			if(pAbb!=NULL)
				pAbb->SetAboveNota(b);
			else
			{
				Indicazione *pInd=dynamic_cast<Indicazione*>(pDrawObject);
				if(pInd!=NULL)
					pInd->SetAboveNota(b);
				else
				{
					DirezArco *pDir=dynamic_cast<DirezArco*>(pDrawObject);
					if(pDir!=NULL)
						pDir->SetAboveNota(b);
					else
					{
						ParteArco *pPar=dynamic_cast<ParteArco*>(pDrawObject);
						if(pPar!=NULL)
							pPar->SetAboveNota(b);
						else
						{
							TDinamico *pTdi=dynamic_cast<TDinamico*>(pDrawObject);
							if(pTdi!=NULL)
								pTdi->SetAboveNota(b);
							else
							{
								Occhiali *pOcc=dynamic_cast<Occhiali*>(pDrawObject);
								if(pOcc!=NULL)
									pOcc->SetAboveNota(b);
								else
								{
									Corda *pCor=dynamic_cast<Corda*>(pDrawObject);
									if(pCor!=NULL)
										pCor->SetAboveNota(b);
									else
									{
										PosizArco *pPos=dynamic_cast<PosizArco*>(pDrawObject);
										if(pPos!=NULL)
											pPos->SetAboveNota(b);
										else
										{
											Annotazione *pAnn=dynamic_cast<Annotazione*>(pDrawObject);
											if(pAnn!=NULL)
												pAnn->SetAboveNota(b);
											else
											{
												TGenerico *pTge=dynamic_cast<TGenerico*>(pDrawObject);
												if(pTge!=NULL)
													pTge->SetAboveNota(b);
												else
												{
													KoreanOrnament *pKor=dynamic_cast<KoreanOrnament*>(pDrawObject);
													if(pKor!=NULL)
														pKor->SetAboveNota(b);
												}
											}
										}
									}
								}
							}
						}
					}
				}	
			}
		}
	}
}

Bool EditMarkersDlg::GetAboveNota()
{
	Strumento *pStr=dynamic_cast<Strumento*>(pDrawObject);
	if(pStr!=NULL)
		return pStr->GetAboveNota();
	else
	{
		IntEsteso *pInt=dynamic_cast<IntEsteso*>(pDrawObject);
		if(pInt!=NULL)
			return pInt->GetSopra();
		else
		{
			Abbellimento *pAbb=dynamic_cast<Abbellimento*>(pDrawObject);
			if(pAbb!=NULL)
				return pAbb->GetAboveNota();
			else
			{
				Indicazione *pInd=dynamic_cast<Indicazione*>(pDrawObject);
				if(pInd!=NULL)
					return pInd->GetAboveNota();
				else
				{
					DirezArco *pDir=dynamic_cast<DirezArco*>(pDrawObject);
					if(pDir!=NULL)
						return pDir->GetAboveNota();
					else
					{
						ParteArco *pPar=dynamic_cast<ParteArco*>(pDrawObject);
						if(pPar!=NULL)
							return pPar->GetAboveNota();
						else
						{
							TDinamico *pTdi=dynamic_cast<TDinamico*>(pDrawObject);
							if(pTdi!=NULL)
								return pTdi->GetAboveNota();
							else
							{
								Occhiali *pOcc=dynamic_cast<Occhiali*>(pDrawObject);
								if(pOcc!=NULL)
									return pOcc->GetAboveNota();
								else
								{
									Corda *pCor=dynamic_cast<Corda*>(pDrawObject);
									if(pCor!=NULL)
										return pCor->GetAboveNota();
									else
									{
										PosizArco *pPos=dynamic_cast<PosizArco*>(pDrawObject);
										if(pPos!=NULL)
											return pPos->GetAboveNota();
										else
										{
											Annotazione *pAnn=dynamic_cast<Annotazione*>(pDrawObject);
											if(pAnn!=NULL)
												return pAnn->GetAboveNota();
											else
											{
												TGenerico *pTge=dynamic_cast<TGenerico*>(pDrawObject);
												if(pTge!=NULL)
													pTge->GetAboveNota();
												else
												{
													KoreanOrnament *pKor=dynamic_cast<KoreanOrnament*>(pDrawObject);
													if(pKor!=NULL)
														return pKor->GetAboveNota();
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
			
		}
	}
	return FALSE;
}

Bool EditingMarkersDlg(DrawObject *pDrawObject)
{
	
	wxString control;
	
	EditMarkersDlg *dialog = new EditMarkersDlg(pDrawObject);
	dialog->Centre();
	if (dialog->Show(TRUE)) 
	{
		return TRUE;
	}
	else 
		return FALSE;
	
}
