/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include "../wxr/editmeasuredlg.wxr"	
#include "../wxr/editmeasuredlg.h"
#include "gui/dlg/editmeasuredlg.hpp"
#include "drw/musicedt.hpp"
#include "liooext.hpp"

BEGIN_EVENT_TABLE(EditMeasureDlg, wxDialog)
EVT_BUTTON(ID_BUTTONOK, EditMeasureDlg::OnOk)
EVT_BUTTON(ID_BUTTONCANC, EditMeasureDlg::OnCancel)
END_EVENT_TABLE()


EditMeasureDlg::EditMeasureDlg()
{	
	wxResourceParseData(editmeasuredlg);
	if (LoadFromResource(FrameManager::currFrame, "editmeasuredlg"))
	{	
		staff1=(wxChoice *)wxFindWindowByName("staff1",this);
		staff2=(wxChoice *)wxFindWindowByName("staff2",this);
		staff3=(wxChoice *)wxFindWindowByName("staff3",this);
		mainPrintView=(wxCheckBox *)wxFindWindowByName("cb1",this);
		mainCompView=(wxCheckBox *)wxFindWindowByName("cb2",this);
		partPrintView=(wxCheckBox *)wxFindWindowByName("cb3",this);
		partCompView=(wxCheckBox *)wxFindWindowByName("cb4",this);
		timeSignatureStatus=(wxChoice *)wxFindWindowByName("timesignature",this);

	}
	else
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);

	for(int i=0;i<25;i++)
	{
		staff1->Append(wxString::Format("%d",i+1));
		staff2->Append(wxString::Format("%d",i+1));
		staff3->Append(wxString::Format("%d",i+1));
	}	
	timeSignatureStatus->SetSelection(0);
}


EditMeasureDlg::~EditMeasureDlg()
{
	if(staff1!=NULL)
		delete staff1;
	if(staff2!=NULL)
		delete staff2;
	if(staff3!=NULL)
		delete staff3;	
	
	if(mainPrintView!=NULL)
		delete mainPrintView;
	if(mainCompView!=NULL)
		delete mainCompView;
	if(partPrintView!=NULL)
		delete partPrintView;	
	if(partCompView!=NULL)
		delete partCompView;	
	if(timeSignatureStatus!=NULL)
		delete timeSignatureStatus;	
}


void EditMeasureDlg::OnOk(wxCommandEvent& event)
{
	switch(GetEnabledCheckBox())
	{
	case 1:
		bat->SetMainScoreLineBreak(1,mainPrintView->GetValue());
		break;
	case 2:
		bat->SetMainScoreLineBreak(0,mainCompView->GetValue());
	case 3:
		bat->SetPartLineBreak(1,partPrintView->GetValue());
		break;
	case 4:
		bat->SetPartLineBreak(0,partCompView->GetValue());
		break;
	}
	
	if(GetNStaff1() != -1)
		psp->SetNCorde(0,GetNStaff1()+1);
	if(GetNStaff2() != -1)
		psp->SetNCorde(1,GetNStaff2()+1);
	if(GetNStaff3() != -1)
		psp->SetNCorde(2,GetNStaff3()+1);
	
	for(int i=0;i<bat->GetNumberOfStaff();i++)
		bat->GetIntest(i)->SetTimeSignatureStatus(GetTimeSignatureStatus());
	
	MusicEditor->RedrawViews(0,TRUE);
	Destroy();
	
}

void EditMeasureDlg::OnCancel(wxCommandEvent& event)
{
	Destroy();
}


int EditMeasureDlg::GetNStaff1()
{
	if((staff1->GetSelection()) != -1)
		return staff1->GetSelection();
	return -1;
}

void EditMeasureDlg::SetNStaff1(int nstaff1)
{
	staff1->SetSelection(nstaff1-1);
}

int EditMeasureDlg::GetNStaff2()
{
	if((staff2->GetSelection()) != -1)
		return staff2->GetSelection();
	return -1;
}

void EditMeasureDlg::SetNStaff2(int nstaff2)
{
	staff2->SetSelection(nstaff2-1);
}

int EditMeasureDlg::GetNStaff3()
{
	if((staff3->GetSelection()) != -1)
		return staff3->GetSelection();
	return -1;
}

void EditMeasureDlg::SetNStaff3(int nstaff3)
{
	staff3->SetSelection(nstaff3-1);
}

void EditMeasureDlg::SetTimeSignatureStatus(TimeSignatureStatus status)
{
	switch(status)
	{
	case TimeSignature_NORMAL:
		timeSignatureStatus->SetSelection(0);
		break;
	case TimeSignature_HIDDEN:
		timeSignatureStatus->SetSelection(1);
		break;
	case TimeSignature_ALWAYSVISIBLE:
		timeSignatureStatus->SetSelection(2);
		break;
	}
}

TimeSignatureStatus EditMeasureDlg::GetTimeSignatureStatus()
{
	TimeSignatureStatus status;
	switch(timeSignatureStatus->GetSelection())
	{
	case 1:
		status=TimeSignature_HIDDEN;
		break;
	case 2:
		status=TimeSignature_ALWAYSVISIBLE;
		break;
	default:
		status=TimeSignature_NORMAL;
		break;
	}
	return status;
}

void EditMeasureDlg::DisableStaff(int staffnumber)
{
	if(staffnumber==1)
	{
		staff2->Enable(false);
		staff3->Enable(false);
	}
	if(staffnumber==2)	
		staff3->Enable(false);
}

void EditMeasureDlg::SetPsp(Spartito *pspart)
{
	psp = pspart;
}

void EditMeasureDlg::SetBat(Battuta *pbat)
{
	bat = pbat;
}

void EditMeasureDlg::Apply()
{
	SetupLineBreaking();
	DisableStaff(psp->GetNumberOfStaff());
	switch(psp->GetNumberOfStaff())
	{
	case 1:
		SetNStaff1(psp->GetNCorde(0));
		break;
	case 2:
		SetNStaff1(psp->GetNCorde(0));
		SetNStaff2(psp->GetNCorde(1));
		break;
	case 3:
		SetNStaff1(psp->GetNCorde(0));
		SetNStaff2(psp->GetNCorde(1));
		SetNStaff3(psp->GetNCorde(2));
		break;
	default:
		SetNStaff1(psp->GetNCorde(0));
		break;
		
	}
	SetTimeSignatureStatus(bat->GetIntest()->GetTimeSignatureStatus());
}

void EditMeasureDlg::EnableCheckBox(bool cb1,bool cb2,bool cb3,bool cb4)
{
	mainPrintView->Enable(cb1);
	mainCompView->Enable(cb2);
	partPrintView->Enable(cb3);
	partCompView->Enable(cb4);
}

int EditMeasureDlg::GetEnabledCheckBox()
{
	if(mainPrintView->IsEnabled())
		return 1;
	if(mainCompView->IsEnabled())
		return 2;
	if(partPrintView->IsEnabled())
		return 3;
	if(partCompView->IsEnabled())
		return 4;
	return -1;
}

void EditMeasureDlg::SetupLineBreaking()
{
	bool cb1=false,cb2=false,cb3=false,cb4=false;
	int score=pMusicWin->PPartit->GetCurrentScore();
	
	switch(score)
	{
	case 0:
		if(currentViewType==1)
			cb1=true;
		else
			cb2=true;
		break;
	default:
		if(currentViewType==1)
			cb3=true;
		else
			cb4=true;
	}
	EnableCheckBox(cb1,cb2,cb3,cb4);
	if(bat!=NULL)
	{
		mainPrintView->SetValue(bat->GetMainScoreLineBreak(1));
		mainCompView->SetValue(bat->GetMainScoreLineBreak(0));
		partPrintView->SetValue(bat->GetPartLineBreak(1));
		partCompView->SetValue(bat->GetPartLineBreak(0));
	}
}





