/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include "wx/wx.h"
#include "wx/dialog.h"
#include "wx/resource.h"
#include <wx/spinctrl.h>
#include <wx/colordlg.h>

#include "../wxr/editnotes2.h"
#include "../wxr/editnotes2.wxr"

#include "lioo.hpp"
#include "stdlib.h"
#include "stdio.h"
#include "fig/nota.hpp"
#include "int/raggruppamento.hpp"
#include "fig/ncroma.hpp"
#include "fig/accordo.hpp"
#include "fig/gruppono.hpp"
#include "drw/frmanage.hpp"

//costanti modalit pitch
#define C_AUTOPITCH		" auto "
#define C_MANUALPITCH	"manual"

// per il redraw
#include "drw/musicedt.hpp"
extern wxMusicEditor *MusicEditor;


class EditNotesDlg :public wxDialog
{
protected:
	wxComboBox		*comboDuration;
	wxComboBox		*comboStatus;
	wxComboBox		*comboColor;
	wxComboBox		*comboStaff;
	wxComboBox		*comboNotehead;
	
	wxRadioBox		*radioSize;
	wxRadioBox		*radioStem;
	
	wxCheckBox		*checkAcciaccatura;
	
    wxSpinCtrl		*spinHighness;
	wxSpinButton	*spinPitch;

	wxTextCtrl		*textPitch;
	wxButton		*togglePMode;
	
	wxButton		*buttonPalette;

	Nota			*pNota;
	Battuta			*currBat;
	
	//Variabili di registro con i valori iniziali (init) e correnti (curr)

	int init_duration;
	int init_notehead;
	int init_highness;
	int init_staff;
	int init_gambo;
	Colore init_color;
	int init_size;
	int init_status;
	int init_acciaccatura;
	int init_numpuntival;
    unsigned char *init_headcode;
	TipoPitch init_pitch;
	
	int curr_duration;
	int curr_notehead;
	int curr_highness;
	int curr_staff;
	int curr_gambo;
	Colore curr_color;
	int curr_size;
	int curr_status;
	int curr_acciaccatura;
	TipoPitch curr_pitch;
	
	wxColourData palette;

public:	
	EditNotesDlg(Nota *ptr, Battuta *ptrBat);
	~EditNotesDlg();
	
	void UpdateValue();
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
	void OnApply(wxCommandEvent& event);
	void OnDuration(wxCommandEvent& event);
	void OnSizexx(wxCommandEvent& event);
	void OnStatus(wxCommandEvent& event);
	void OnSpinCtrlChange(wxCommandEvent& event);

	void OnSpinBtnUp(wxCommandEvent& event);
	void OnSpinBtnDown(wxCommandEvent& event);

	void OnToggleBtnClick(wxCommandEvent& event);

	void OnPaletteBtnClick(wxCommandEvent& event);

private:

    DECLARE_EVENT_TABLE()   
};

/********************************************************************/
BEGIN_EVENT_TABLE(EditNotesDlg, wxDialog)
EVT_BUTTON(ID_BUTTONOK, EditNotesDlg::OnOk)
EVT_BUTTON(ID_BUTTONCANCEL, EditNotesDlg::OnCancel)
EVT_BUTTON(ID_BUTTONAPPLY, EditNotesDlg::OnApply)
EVT_COMBOBOX(ID_COMBODURATION, EditNotesDlg::OnDuration)
EVT_COMBOBOX(ID_COMBOSTATUS, EditNotesDlg::OnStatus)
EVT_RADIOBOX(ID_RADIOSIZE, EditNotesDlg::OnSizexx)
EVT_SPINCTRL(ID_SPINCTRL, EditNotesDlg::OnSpinCtrlChange)
EVT_SPIN_UP(ID_SPINBTN, EditNotesDlg::OnSpinBtnUp) 
EVT_SPIN_DOWN(ID_SPINBTN, EditNotesDlg::OnSpinBtnDown) 
EVT_BUTTON(ID_TOGGLEBTN, EditNotesDlg::OnToggleBtnClick)
EVT_BUTTON(ID_PALETTEBTN, EditNotesDlg::OnPaletteBtnClick)
//	EVT_CLOSE (EditNotesDlg::OnCancel)
END_EVENT_TABLE()
/********************************************************************/

EditNotesDlg::EditNotesDlg(Nota *ptr, Battuta *ptr_Bat)
{
	
	//assegnazione puntatori
	
	pNota = ptr;
	currBat=ptr_Bat;
	
	wxResourceParseData(editnote); 
	if (LoadFromResource(FrameManager::currFrame, "editnote"))
	{
		comboDuration		= (wxComboBox *)wxFindWindowByName("comboDuration", this);
		comboStatus			= (wxComboBox *)wxFindWindowByName("comboStatus", this);
		comboColor			= (wxComboBox *)wxFindWindowByName("comboColor", this);
		comboStaff		    = (wxComboBox *)wxFindWindowByName("comboStaff", this);
		comboNotehead		= (wxComboBox *)wxFindWindowByName("comboNotehead", this);
		
		radioSize			= (wxRadioBox *)wxFindWindowByName("radioSize", this);
		radioStem			= (wxRadioBox *)wxFindWindowByName("radioStem", this);
		
		checkAcciaccatura	= (wxCheckBox *)wxFindWindowByName("checkAcciaccatura", this);
		
		spinHighness        = (wxSpinCtrl *)wxFindWindowByName("spinHighness", this);
	
		textPitch			= (wxTextCtrl *)wxFindWindowByName("textPitch", this);
		togglePMode			= (wxButton *)wxFindWindowByName("togglePMode", this);

		buttonPalette		= (wxButton *)wxFindWindowByName("buttonPalette", this);

		textPitch->SetBackgroundColour(*wxLIGHT_GREY);	
	}
	
	else
	{
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
	}
	
	SetModal(FALSE);
	
	int w,h;
	int x,y,y2;
	comboDuration->GetClientSize(&w,&h);
	comboNotehead->GetPosition(&x,&y);
	y2=y;
	comboDuration->GetPosition(&x,&y);
	
	spinHighness = new wxSpinCtrl(this, ID_SPINCTRL, wxEmptyString, wxPoint(x,y2), wxSize(w,h), wxSP_ARROW_KEYS, -24, 24);
	
	comboDuration->Append("2 - Breve");
	comboDuration->Append("1 - Semibreve");
	comboDuration->Append("1/2 - Minima");
	comboDuration->Append("1/4 - Semiminima");
	comboDuration->Append("1/8 - Croma");
	comboDuration->Append("1/16 - Semicroma");
	comboDuration->Append("1/32 - Biscroma");
	comboDuration->Append("1/64 - SemibisCroma");
	comboDuration->Append("1/128 - Fusa");
	
	comboStatus->Append("Normal");
	comboStatus->Append("Graced");
	comboStatus->Append("Hidden");
	comboStatus->Append("Ghosted");
	
	for(int s=1; s<=currBat->GetNumberOfStaff(); s++)
		comboStaff->Append(wxString::Format("%d",s));
	
	comboNotehead->Append("Classic");
	comboNotehead->Append("Alphanum");
	comboNotehead->Append("Alphanum_Square");
	comboNotehead->Append("Alphanum_Reverse");
	comboNotehead->Append("Circlex");
	comboNotehead->Append("Cluster");
	comboNotehead->Append("X");
	comboNotehead->Append("Croix");
	comboNotehead->Append("Ddiesis");
	comboNotehead->Append("Diamond");
	comboNotehead->Append("Triang");
	comboNotehead->Append("Triang_Up");
	comboNotehead->Append("Triang_Down");
	comboNotehead->Append("Triang_Left");
	comboNotehead->Append("Triang_Right");
	comboNotehead->Append("Triang_Round");
	comboNotehead->Append("Moon");
	comboNotehead->Append("Plus");
	comboNotehead->Append("Rhythmic");
	comboNotehead->Append("Square");
	
	comboColor->Append("White");
	comboColor->Append("Black");
	comboColor->Append("Grey");
	comboColor->Append("Dark Grey");
	comboColor->Append("Green");
	comboColor->Append("Light Green");
	comboColor->Append("Water Green");
	comboColor->Append("Blue");
	comboColor->Append("Cyan");
	comboColor->Append("Violet");
	comboColor->Append("Magenta");
	comboColor->Append("Red");
	comboColor->Append("Orange");
	comboColor->Append("Brown");
	comboColor->Append("Yellow");
	comboColor->Append("Light Yellow");
	
	// DEFAULT COLORS
	palette.SetCustomColour(BIANCO,		Colore(BIANCO));
	palette.SetCustomColour(NERO,		Colore(NERO));
	palette.SetCustomColour(GRIGIO,		Colore(GRIGIO));
	palette.SetCustomColour(GRIGIOS,	Colore(GRIGIOS));
	palette.SetCustomColour(VERDE,		Colore(VERDE));
	palette.SetCustomColour(VERDEC,		Colore(VERDEC));
	palette.SetCustomColour(VERDEA,		Colore(VERDEA));
	palette.SetCustomColour(BLU,		Colore(BLU));
	palette.SetCustomColour(CELESTE,	Colore(CELESTE));
	palette.SetCustomColour(VIOLA,		Colore(VIOLA));
	palette.SetCustomColour(MAGENTA,	Colore(MAGENTA));
	palette.SetCustomColour(ROSSO,		Colore(ROSSO));
	palette.SetCustomColour(ARANCIO,	Colore(ARANCIO));
	palette.SetCustomColour(MARRONE,	Colore(MARRONE));
	palette.SetCustomColour(GIALLO,		Colore(GIALLO));
	palette.SetCustomColour(GIALLOC,	Colore(GIALLOC));
	//
	//Inizializzazione delle finestre con salvataggio dei valori iniziali
	//in variabili della classe opportune.
	
	//Inizializzazione di comboDuration
	
	init_duration=pNota->GetID()-CL_NOTA-1;
	comboDuration->SetSelection(init_duration);
	
	//Inizializzazione di comboNotehead
	
	init_notehead=pNota->GetHeadType();
	comboNotehead->SetSelection(init_notehead);
	
	switch(init_notehead)
	{
	case HEAD_ALPHANUM:
	case HEAD_ALPHANUM_SQUARE:
	case HEAD_ALPHANUM_REVERSE:
		{
			init_headcode=pNota->GetHeadCode();
		}
		break;
	}
	
	//Inizializzazione di comboStatus
	
	if(pNota->GetID()==CL_RAGGRUPPAMENTO){

		init_status=(((Raggruppamento *)pNota)->GetFirstFigura())->GetStatus();
		comboStatus->SetSelection(init_status);
	}
	else if(pNota->GetParentFig()!=NULL)
	{
		if(pNota->GetParentFig()->GetID()==CL_ACCORDO && pNota->GetParentFig()->GetParentFig()==NULL)
		{
			init_status=((Accordo *)(pNota->GetParentFig()))->GetStatus();
			comboStatus->SetSelection(init_status);
		}
		else if(((pNota->GetParentFig())->GetID())==CL_GRUPPONOTE)
		{
			init_status=((GruppoNote *)(pNota->GetParentFig()))->GetStatus();
			comboStatus->SetSelection(init_status);
		}
		else if(pNota->GetParentFig()->GetParentFig()->GetID()==CL_GRUPPONOTE)
		{
			init_status=((GruppoNote *)(pNota->GetParentFig()->GetParentFig()))->GetStatus();
			comboStatus->SetSelection(init_status);
		}
	}
	else
	{
		init_status=pNota->GetStatus();
		comboStatus->SetSelection(init_status);
	}
	
	//Inizializzazione di comboColor
	
	if((init_status==0)||(init_status==1))
	{
		init_color=pNota->GetColor();
		comboColor->SetValue(init_color.GetCode());
	}
	else
	{
		//init_color=-1;
		comboColor->SetSelection(-1);//init_color);
		comboColor->Enable(FALSE);
	}
	
	//Inizializzazione di spinHighness
	
	init_highness=pNota->GetAltezza();
	spinHighness->SetValue(init_highness);

	//Inizializzazione di textPitch

	int sbx, sby, sbw, sbh;
	textPitch->GetPosition(&sbx,&sby);
	textPitch->GetSize(&sbw,&sbh);
	spinPitch = new wxSpinButton(this, ID_SPINBTN, wxPoint(1+ sbx+ sbw, sby), wxSize(-1,sbh));
	
	wxString pitchStr;
	if (pNota->GetPitch() == NULL)
	{
		pitchStr = pNota->CreatePitchStr();		
		togglePMode->SetLabel(C_MANUALPITCH);	//auto text hilight
		spinPitch->Enable(FALSE);
		spinPitch->Show(FALSE);
	}
	else
	{
		init_pitch = TipoPitch(*(pNota->GetPitch()));
		curr_pitch = TipoPitch(init_pitch);
		
		pitchStr = pNota->GetPitchStr();
		togglePMode->SetLabel(C_AUTOPITCH);//manual text hilight
		spinPitch->Enable(TRUE);
		spinPitch->Show(TRUE);
	}

	textPitch->SetValue( pitchStr );
	
	//Inizializzazione di comboStaff
	
	init_staff=pNota->GetNStaff();
	comboStaff->SetSelection(init_staff);
	
	//Inizializzazione di radioStem (con distinzione fra note, beam e accordi).
	
	radioStem->Enable(2,false); // BEAM12 normalmente disabilitato
	radioStem->Enable(3,false); // BEAM23
	
	if((pNota->GetID())==CL_RAGGRUPPAMENTO)
			{
			  if(((Raggruppamento *)pNota)->GetFirstBattuta()->GetNumberOfStaff()>=2)
				  radioStem->Enable(2,true); //BEAM12 attivo se ci sono 2/3 staff
			  if(((Raggruppamento *)pNota)->GetFirstBattuta()->GetNumberOfStaff()>=3)
				  radioStem->Enable(3,true); //BEAM23 attivo se ci sono 3 staff
			  init_gambo=((Raggruppamento *)(pNota))->GetBeamPosition();
			  radioStem->SetSelection(init_gambo);

			  init_staff=((Nota*)(((Raggruppamento*)pNota)->GetFirstFigura()))->GetNStaff();
			  comboStaff->SetSelection(init_staff);
			  init_size=((Raggruppamento*)pNota)->GetNotina();
				if(init_size)
					radioSize->SetSelection(1);
				else
					radioSize->SetSelection(0);			  
			}

	else if(pNota->GetParentFig()!=NULL)
	{
		if(((pNota->GetParentFig())->GetID())==CL_ACCORDO && pNota->GetParentFig()->GetParentFig()==NULL)
		{
			init_gambo=((Accordo *)(pNota->GetParentFig()))->GetLGambo();
			if(init_gambo>0)
				radioStem->SetSelection(0);
			if(init_gambo<0)
				radioStem->SetSelection(1);
		}
		else if(((pNota->GetParentFig())->GetID())==CL_GRUPPONOTE)
		{
			if(pNota->GetPBattuta()->GetNumberOfStaff()>=2)
				radioStem->Enable(2,true); //BEAM12 attivo se ci sono 2/3 staff
			if(pNota->GetPBattuta()->GetNumberOfStaff()>=3)
				radioStem->Enable(3,true); //BEAM23 attivo se ci sono 3 staff
			init_gambo=((GruppoNote *)(pNota->GetParentFig()))->GetBeamPosition();
			radioStem->SetSelection(init_gambo);
		}
		else if(pNota->GetParentFig()->GetParentFig()->GetID()==CL_GRUPPONOTE)
		{
			if(pNota->GetPBattuta()->GetNumberOfStaff()>=2)
				radioStem->Enable(2,true); //BEAM12 attivo se ci sono 2/3 staff
			if(pNota->GetPBattuta()->GetNumberOfStaff()>=3)
				radioStem->Enable(3,true); //BEAM23 attivo se ci sono 3 staff
			init_gambo=((GruppoNote *)(pNota->GetParentFig()->GetParentFig()))->GetBeamPosition();
			radioStem->SetSelection(init_gambo);
		}
	}
	else
	{
		
		init_gambo=pNota->GetLGambo();
		if(init_gambo>0)
			radioStem->SetSelection(0);
		if(init_gambo<0)
			radioStem->SetSelection(1);
		if(init_gambo==0)
		{
			radioStem->Enable(0,false);
			radioStem->Enable(1,false);
			radioStem->Enable(2,false);
			radioStem->Enable(3,false);
		}
	}
	
	//Inizializzazione di radioSize
	
	init_size=pNota->GetNotina();
	if(init_size)
		radioSize->SetSelection(1);
	if(init_size==FALSE)
		radioSize->SetSelection(0);
	
	//Inizializzazione di checkAcciaccatura
	
	if((init_size)&&(comboDuration->GetSelection()==4))
	{
		init_acciaccatura=((NCroma *)pNota)->GetAcciaccatura();
		
		checkAcciaccatura->SetValue(init_acciaccatura);
	}
	else
	{
		checkAcciaccatura->Enable(FALSE);      
		checkAcciaccatura->SetValue(FALSE);
		init_acciaccatura=FALSE;
	}
	
}


EditNotesDlg::~EditNotesDlg()
{
	if(comboDuration !=NULL) 
		delete comboDuration;
	if(comboStatus !=NULL) 
		delete comboStatus;
	if(comboColor !=NULL) 
		delete comboColor;
	if(comboStaff !=NULL) 
		delete comboStaff;
	if(comboNotehead !=NULL) 
		delete comboNotehead;
	if(radioSize !=NULL) 
		delete radioSize;
	if(radioStem !=NULL) 
		delete radioStem;
	if(checkAcciaccatura !=NULL) 
		delete checkAcciaccatura;
	if(spinHighness !=NULL) 
		delete spinHighness;

	if(textPitch !=NULL) 
		delete textPitch;
	if(togglePMode !=NULL) 
		delete togglePMode;
	if(spinPitch !=NULL) 
		delete spinPitch;	
	
}

void EditNotesDlg::OnDuration(wxCommandEvent& event)
{
	
	if((comboDuration->GetSelection()==4)&&(radioSize->GetSelection()==1))
	{
		checkAcciaccatura->Enable(TRUE);
	}
	else
	{
		checkAcciaccatura->Enable(FALSE);
	}
	
	if((comboDuration->GetSelection()==0)||(comboDuration->GetSelection()==1))
	{
		radioStem->Enable(0,false);
		radioStem->Enable(1,false);
		radioStem->Enable(2,false);
		radioStem->Enable(3,false);
	}
	else
	{
		radioStem->Enable(0,true);
		radioStem->Enable(1,true);
		if(pNota->GetParentFig()!=NULL)
		{
			if(((pNota->GetParentFig())->GetID())==CL_GRUPPONOTE)
			{
				if(pNota->GetPBattuta()->GetNumberOfStaff()>=2)
					radioStem->Enable(2,true); //BEAM12 attivo se ci sono 2/3 staff
				if(pNota->GetPBattuta()->GetNumberOfStaff()>=3)
					radioStem->Enable(3,true); //BEAM23 attivo se ci sono 3 staff
			}
		}
	}
	
}

void EditNotesDlg::OnSpinCtrlChange(wxCommandEvent& WXUNUSED(event))
{
	//autoPitch
	if (togglePMode->GetLabel() == C_MANUALPITCH)
	{
		pNota->SetAltezza(spinHighness->GetValue());	
		textPitch->SetValue(pNota->CreatePitchStr());
	}
}

void EditNotesDlg::OnToggleBtnClick(wxCommandEvent& event)
{
	wxString pitchStr;
	if (togglePMode->GetLabel() == C_AUTOPITCH)	//switch to auto mode
	{
		pNota->SetPitch(NULL);
		pNota->SetAltezza(spinHighness->GetValue());

		pitchStr = pNota->CreatePitchStr();		
		togglePMode->SetLabel(C_MANUALPITCH);
		spinPitch->Enable(FALSE);
		spinPitch->Show(FALSE);
	}
	else					//switch to manual mode
	{
		curr_pitch = TipoPitch(*(pNota->CreatePitch()));
		pNota->SetPitch(&curr_pitch);

		pitchStr = pNota->GetPitchStr();
		togglePMode->SetLabel(C_AUTOPITCH);
		spinPitch->Enable(TRUE);
		spinPitch->Show(TRUE);
	}

	textPitch->SetValue( pitchStr );
}

void EditNotesDlg::OnPaletteBtnClick(wxCommandEvent& event)
{
	if (comboColor->GetSelection() == -1)
		palette.SetColour(Colore(comboColor->GetValue()));
	else
		palette.SetColour(Colore((ColorIcon)comboColor->GetSelection()));
	
	wxColourDialog dialog(this, &palette);
	dialog.SetTitle("Choose a color...");

	if (dialog.ShowModal() == wxID_OK)
	{
		wxColourData retData = dialog.GetColourData();
		Colore col = retData.GetColour();
		comboColor->SetValue(col.GetCode());
	}
}

void EditNotesDlg::OnSpinBtnUp(wxCommandEvent& event)
{
	short alter		= curr_pitch.Accidental;
	short letter	= curr_pitch.Letter;
	short octave	= curr_pitch.Octave;

	wxString msg;

	if (alter == 1 || letter == 2 || letter == 6)
	{
		if (letter == 6)
			octave++;
		letter = (letter+1)%7;
		alter = 0;
	}
	else
		alter++;

	if (alter == 1)
		msg.Printf("%c#%hd",NoteChars[letter],octave);
	else		
		msg.Printf("%c %hd",NoteChars[letter],octave);

	curr_pitch.Accidental = alter;
	curr_pitch.Letter = letter;
	curr_pitch.Octave = octave;

	textPitch->SetValue(msg);
}

void EditNotesDlg::OnSpinBtnDown(wxCommandEvent& event)
{
	short alter		= curr_pitch.Accidental;
	short letter	= curr_pitch.Letter;
	short octave	= curr_pitch.Octave;

	wxString msg;

	if (alter == -1 || letter == 0 || letter == 3)
	{
		if (letter == 0)
			octave--;
		letter = (letter+6)%7;
		alter = 0;
	}
	else
		alter--;
		
	if (alter == -1)
		msg.Printf("%cb%hd",NoteChars[letter],octave);
	else		
		msg.Printf("%c %hd",NoteChars[letter],octave);

	curr_pitch.Accidental = alter;
	curr_pitch.Letter = letter;
	curr_pitch.Octave = octave;

	textPitch->SetValue(msg);
}


void EditNotesDlg::OnSizexx(wxCommandEvent& WXUNUSED(event))
{
	if((comboDuration->GetSelection()==4)&&(radioSize->GetSelection()==1))
		checkAcciaccatura->Enable(TRUE);
	else
		checkAcciaccatura->Enable(FALSE);
}

void EditNotesDlg::OnStatus(wxCommandEvent& event)
{
	if((comboStatus->GetSelection()==0)||(comboStatus->GetSelection()==1))
		comboColor->Enable(TRUE);
	else
		comboColor->Enable(FALSE);
}

void EditNotesDlg::OnOk(wxCommandEvent& WXUNUSED(event))
{	
	extern wxDC* currDC;
	currDC=NULL;
	UpdateValue();
	Destroy();
}

void EditNotesDlg::OnApply(wxCommandEvent& event)
{
	extern wxDC* currDC;
	currDC=NULL;
	UpdateValue();
}

void EditNotesDlg::OnCancel(wxCommandEvent& event)
{
	extern wxDC* currDC;
	currDC=NULL;
	//Recupera i valori iniziali (init), li impone e chiude la finestra di dialogo
	
	ClassID Target_ID=(ClassID)(init_duration+CL_NOTA+1);
	if(Target_ID!=pNota->GetID())
		pNota=(Nota *)currBat->ChangeFigDuration((Figura *)pNota, Target_ID);
	
	pNota->SetAltezza(init_highness);
	if(pNota->GetPitch() != NULL) 
		pNota->SetPitch(&init_pitch);

	pNota->SetNStaff(init_staff);
	
	pNota->SetColor(init_color);
	
	pNota->SetHeadType((NoteHead_ID)init_notehead);
	switch(init_notehead)
	{
	case HEAD_ALPHANUM:
	case HEAD_ALPHANUM_SQUARE:
	case HEAD_ALPHANUM_REVERSE:
		{
			pNota->SetHeadCode(init_headcode);
		}
		break;
	}
	
	
	
	////////////////////////////////////////////////////////////////////////////////////
	// Distinzione fra note, beam e accordi.
	////////////////////////////////////////////////////////////////////////////////////
	if((pNota->GetParentFig()!=NULL)&&(pNota->GetID()!=CL_RAGGRUPPAMENTO))
	{
		if(((pNota->GetParentFig())->GetID())==CL_ACCORDO && pNota->GetParentFig()->GetParentFig()==NULL)
		{
			ClassID idacc=pNota->GetID();
			if((idacc>=CL_NCROMA) && (idacc<=CL_NFUSA))
				pNota->SetCoda(TRUE);
			
			if(init_size==0)
				((Accordo *)(pNota->GetParentFig()))->SetBigAccordo();
			else if(init_size==1)
				((Accordo *)(pNota->GetParentFig()))->SetSmallAccordo();
			
			if(init_gambo>0)
				((Accordo *)(pNota->GetParentFig()))->SetGamboUp();
			else if(init_gambo<0)
				((Accordo *)(pNota->GetParentFig()))->SetGamboDwn();
			
			if ((pNota->GetID())==CL_NCROMA && pNota->GetNotina() )
				((NCroma *)pNota)->SetAcciaccatura(init_acciaccatura);
			else if ((pNota->GetID())==CL_NCROMA && (pNota->GetNotina()==FALSE))
				((NCroma *)pNota)->SetAcciaccatura(0);
			
			((Accordo *)(pNota->GetParentFig()))->SetStatus((Status_ID)init_status);
		}
		
		else if(pNota->GetParentFig()->GetID()==CL_GRUPPONOTE || pNota->GetParentFig()->GetParentFig()->GetID()==CL_GRUPPONOTE)
		{
			GruppoNote * beam=NULL;
			if(pNota->GetParentFig()->GetID()==CL_GRUPPONOTE)
				beam=(GruppoNote *)pNota->GetParentFig();
			else
				beam=(GruppoNote *)pNota->GetParentFig()->GetParentFig();
			
			beam->SetSmallBeam(init_size);
			
			if(init_gambo==0)
			{
				beam->SetBeamUp();
				beam->SetBeamPosition(GruppoNote::BEAM_UP);
			}
			else if(init_gambo==1)
			{
				beam->SetBeamDwn();
				beam->SetBeamPosition(GruppoNote::BEAM_DWN);
			}
			else if(init_gambo==2)
				beam->SetBeamPosition(GruppoNote::BEAM_12);
			else if(init_gambo==3)
				beam->SetBeamPosition(GruppoNote::BEAM_23);
			
			if ((pNota->GetID())==CL_NCROMA && pNota->GetNotina() )
				((NCroma *)pNota)->SetAcciaccatura(init_acciaccatura);
			if ((pNota->GetID())==CL_NCROMA && (pNota->GetNotina()==FALSE))
				((NCroma *)pNota)->SetAcciaccatura(0);
			
			beam->SetBeamStatus((Status_ID)init_status);
		}
		
	}
	else if (pNota->GetID()==CL_RAGGRUPPAMENTO)
	{
		Raggruppamento * beam=NULL;
		
		beam=(Raggruppamento*)pNota;
		
		beam->SetSmallBeam(init_size);
		
		if(init_gambo==0)
		{
			beam->SetBeamUp();
			beam->SetBeamPosition(Raggruppamento::BEAM_UP);
		}
		else if(init_gambo==1)
		{
			beam->SetBeamDwn();
			beam->SetBeamPosition(Raggruppamento::BEAM_DWN);
		}
		else if(init_gambo==2)
			beam->SetBeamPosition(Raggruppamento::BEAM_12);
		else if(init_gambo==3)
			beam->SetBeamPosition(Raggruppamento::BEAM_23);
		
		beam->SetStatus((Status_ID)init_status);
	}
	else
	{
		ClassID id=pNota->GetID();
		if((id>=CL_NCROMA) && (id<=CL_NFUSA))
			pNota->SetCoda(TRUE);
		
		pNota->SetNotina(init_size);
		
		if(init_gambo>0)
			pNota->SetGamboUp();
		else if(init_gambo<0)
			pNota->SetGamboDwn();
		
		if (id==CL_NCROMA && pNota->GetNotina() )
			((NCroma *)pNota)->SetAcciaccatura(init_acciaccatura);
		/******************************************************************************/
		if ((id==CL_NCROMA) && (pNota->GetNotina()==FALSE))
			((NCroma *)pNota)->SetAcciaccatura(0);
		/******************************************************************************/
		pNota->SetStatus((Status_ID)init_status);
		
	}
	
	currBat->SetupGambi();
    MusicEditor->RedrawViews(0,TRUE);
	Destroy();
	
}


void EditNotesDlg::UpdateValue()
//Legge i valori correnti (curr) e li impone.

{
	
	curr_duration=comboDuration->GetSelection();
	curr_status=comboStatus->GetSelection();
	curr_size=radioSize->GetSelection();
	curr_gambo=radioStem->GetSelection();
	curr_notehead=comboNotehead->GetSelection();
	
	if (comboColor->GetSelection() == -1)
		curr_color=Colore(comboColor->GetValue());
	else
		curr_color=Colore((ColorIcon)comboColor->GetSelection());
	
	curr_highness=spinHighness->GetValue();
	curr_staff=comboStaff->GetSelection();
	curr_acciaccatura=checkAcciaccatura->GetValue();
	
	ClassID currID=(ClassID)(curr_duration+CL_NOTA+1);
	if((currID!=pNota->GetID())&&(pNota->GetID()!=CL_RAGGRUPPAMENTO))
	{
		pNota=(Nota *)currBat->ChangeFigDuration((Figura *)pNota, currID);
	}		
	
	if(pNota!=NULL)
	{
		pNota->SetColor(curr_color);
		
		pNota->SetHeadType((NoteHead_ID)curr_notehead);
		switch(curr_notehead)
		{
		case HEAD_ALPHANUM:
		case HEAD_ALPHANUM_SQUARE:
		case HEAD_ALPHANUM_REVERSE:
			{
				char headVal[10];
				
				headVal[0]=0;
				if(InputString("Head value (max 2 chars):",headVal,10,"Head type"))
				{
					headVal[2]=0;
					pNota->SetHeadCode((unsigned char*)headVal);
				}
			}
			break;
		}
		
		
		pNota->SetAltezza(curr_highness);
		if(pNota->GetPitch() != NULL)
			pNota->SetPitch(&curr_pitch);
		
		pNota->SetNStaff(curr_staff);
		
		////////////////////////////////////////////////////////////////////////////////////
		// Distinzione fra note, beam e accordi.
		////////////////////////////////////////////////////////////////////////////////////
		
		if((pNota->GetID())==CL_RAGGRUPPAMENTO)
		{
			Raggruppamento *beam=NULL;
			
			beam=(Raggruppamento *)pNota;
			
			beam->SetSmallBeam(curr_size);
			
			if(curr_gambo==0)
			{
				beam->SetBeamUp();
				beam->SetBeamPosition(Raggruppamento::BEAM_UP);
			}
			else if(curr_gambo==1)
			{
				beam->SetBeamDwn();
				beam->SetBeamPosition(Raggruppamento::BEAM_DWN);
			}
			else if(curr_gambo==2)
				beam->SetBeamPosition(Raggruppamento::BEAM_12);
			else if(curr_gambo==3)
				beam->SetBeamPosition(Raggruppamento::BEAM_23);
			
			((Raggruppamento *)pNota)->SetStatus((Status_ID)curr_status);
			//momentaneamente commentato
			
		}
		else if(pNota->GetParentFig()!=NULL)
		{
			if(((pNota->GetParentFig())->GetID())==CL_ACCORDO && ((pNota->GetParentFig()->GetParentFig()))==NULL)
			{
				if(curr_size==0)
					((Accordo *)(pNota->GetParentFig()))->SetBigAccordo();
				else if(curr_size==1)
					((Accordo *)(pNota->GetParentFig()))->SetSmallAccordo();
				
				if(curr_gambo==0)
					((Accordo *)(pNota->GetParentFig()))->SetGamboUp();
				else if(curr_gambo==1)
					((Accordo *)(pNota->GetParentFig()))->SetGamboDwn();
				
				ClassID idacc=pNota->GetID();
				if((idacc>=CL_NCROMA) && (idacc<=CL_NFUSA))
					((Accordo *)(pNota->GetParentFig()))->SetCoda(TRUE);
				
				if((curr_size==1)&(curr_duration==4))
				{
					((NCroma *)pNota)->SetAcciaccatura(curr_acciaccatura);
				}
				else
				{
					if (idacc==CL_NCROMA)
						((NCroma *)pNota)->SetAcciaccatura(FALSE);
				}
				
				((Accordo *)(pNota->GetParentFig()))->SetStatus((Status_ID)curr_status);
				
			}
			
			else if(((pNota->GetParentFig())->GetID())==CL_GRUPPONOTE || 
				((pNota->GetParentFig()->GetParentFig())->GetID())==CL_GRUPPONOTE)
			{
				GruppoNote *beam=NULL;
				if(((pNota->GetParentFig())->GetID())==CL_GRUPPONOTE)
					beam=(GruppoNote *)pNota->GetParentFig();
				else
					beam=(GruppoNote *)pNota->GetParentFig()->GetParentFig();
				
				beam->SetSmallBeam(curr_size);
				
				if(curr_gambo==0)
				{
					beam->SetBeamUp();
					beam->SetBeamPosition(GruppoNote::BEAM_UP);
				}
				else if(curr_gambo==1)
				{
					beam->SetBeamDwn();
					beam->SetBeamPosition(GruppoNote::BEAM_DWN);
				}
				else if(curr_gambo==2)
					beam->SetBeamPosition(GruppoNote::BEAM_12);
				else if(curr_gambo==3)
					beam->SetBeamPosition(GruppoNote::BEAM_23);
				
				
				//((GruppoNote *)(pNota->GetParentFig()))->SetBeamStatus((Status_ID)curr_status);
				beam->SetBeamStatus((Status_ID)curr_status);
			}
			
		}
		
		else
		{
			
			ClassID id=pNota->GetID();
			if((id>=CL_NCROMA) && (id<=CL_NFUSA))
				pNota->SetCoda(TRUE);
			
			pNota->SetNotina(curr_size);
			
			if(curr_gambo==0)
				pNota->SetGamboUp();
			if(curr_gambo==1)
				pNota->SetGamboDwn();
			
			if((curr_size==1)&&(curr_duration==4))
			{
				((NCroma *)pNota)->SetAcciaccatura(curr_acciaccatura);
			}
			else
			{
				if (id==CL_NCROMA)
					((NCroma *)pNota)->SetAcciaccatura(FALSE);
			}				  
			
			pNota->SetStatus((Status_ID)curr_status);
			
		}
		
    }
    
	currBat->SetupGambi();
	MusicEditor->RedrawViews(0,TRUE);
	
}

Bool EditingNotesDlg(Nota *pNota, Battuta *currBat)
{
	
	wxString control;
	
	EditNotesDlg *dialog = new EditNotesDlg(pNota, currBat);
	
	dialog->Centre();
	if (dialog->Show(TRUE)) 
	{
		return TRUE;
	}
	else 
		return FALSE;
	
}
