/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include "wx/wx.h"
#include "wx/dialog.h"
#include "wx/resource.h"
#include <wx/spinctrl.h>
#include <wx/colordlg.h>

#include "../wxr/editrests2.h"
#include "../wxr/editrests2.wxr"

#include "lioo.hpp"
#include "stdlib.h"
#include "stdio.h"
#include "fig/pausa.hpp"
#include "drw/frmanage.hpp"

// per il redraw
#include "drw/musicedt.hpp"
extern wxMusicEditor *MusicEditor;

class EditRestsDlg :public wxDialog
{
protected:
	wxComboBox	*comboDuration;
	wxComboBox	*comboStatus;
	wxComboBox	*comboColor;
	wxComboBox	*comboStaff;
	
	wxRadioBox  *radioSize;
	
	wxSpinCtrl	*spinHighness;

	wxButton	*buttonPalette;
	
	Pausa *pPausa;
	Battuta *currBat;
	
	int init_duration;
	int init_status;
	int init_highness;
	int init_staff;
	Colore init_color;
	int init_size;
	
	int curr_duration;
	int curr_status;
	int curr_highness;
	int curr_staff;
	Colore curr_color;
	int curr_size;
	
	wxColourData palette;
	
public:	
	EditRestsDlg(Pausa *ptr, Battuta *ptrBat);
	~EditRestsDlg();
	
	void UpdateValue();
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
	void OnApply(wxCommandEvent& event);
	void OnStatus(wxCommandEvent& event);
	
	void OnPaletteBtnClick(wxCommandEvent& event);

private:
    DECLARE_EVENT_TABLE()   
};

/****************************************************/
BEGIN_EVENT_TABLE(EditRestsDlg, wxDialog)
EVT_BUTTON(ID_BUTTONOK, EditRestsDlg::OnOk)
EVT_BUTTON(ID_BUTTONCANCEL, EditRestsDlg::OnCancel)
EVT_BUTTON(ID_BUTTONAPPLY, EditRestsDlg::OnApply)
EVT_COMBOBOX(ID_COMBOSTATUS, EditRestsDlg::OnStatus)
EVT_BUTTON(ID_PALETTEBTN, EditRestsDlg::OnPaletteBtnClick)
END_EVENT_TABLE()
/****************************************************/

EditRestsDlg::EditRestsDlg(Pausa *ptr, Battuta *ptrBat)
{
	
	//assegnazione puntatori
	
	pPausa = ptr;
	currBat = ptrBat;
	
	wxResourceParseData(editrest); 
	if (LoadFromResource(FrameManager::currFrame, "editrest"))
	{
		comboDuration			= (wxComboBox *)wxFindWindowByName("comboDuration", this);
		comboStatus			= (wxComboBox *)wxFindWindowByName("comboStatus", this);
		comboColor			= (wxComboBox *)wxFindWindowByName("comboColor", this);
		comboStaff		    = (wxComboBox *)wxFindWindowByName("comboStaff", this);
		
		radioSize				= (wxRadioBox *)wxFindWindowByName("radioSize", this);
		
		spinHighness          = (wxSpinCtrl *)wxFindWindowByName("spinHighness", this);

		buttonPalette		= (wxButton *)wxFindWindowByName("buttonPalette", this);
	
	}
	
	else
	{
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
	}
	
	SetModal(FALSE);
	
	int w,h;
	int x,y,y2;
	comboDuration->GetClientSize(&w,&h);
	comboColor->GetPosition(&x,&y);
	y2=y;
	comboDuration->GetPosition(&x,&y);
	
	spinHighness = new wxSpinCtrl(this, -1, wxEmptyString, wxPoint(x,y2), wxSize(w,h), wxSP_ARROW_KEYS, -24, 24);
	
	
	comboDuration->Append("4 measures");
	comboDuration->Append("2 measures");
	comboDuration->Append("1 - Semibreve");
	comboDuration->Append("1/2 - Minima");
	comboDuration->Append("1/4 - Semiminima");
	comboDuration->Append("1/8 - Croma");
	comboDuration->Append("1/16 - Semicroma");
	comboDuration->Append("1/32 - Biscroma");
	comboDuration->Append("1/64 - Semibiscroma");
	comboDuration->Append("1/128 - Fusa");
	comboDuration->Append("Generic rest");
	
	comboStatus->Append("Normal");
	comboStatus->Append("Graced");
	comboStatus->Append("Hidden");
	comboStatus->Append("Ghosted");
	
	for(int s=1; s<=currBat->GetNumberOfStaff(); s++)
		comboStaff->Append(wxString::Format("%d",s));
	
	comboColor->Append("White");
	comboColor->Append("Black");
	comboColor->Append("Grey");
	comboColor->Append("Dark Grey");
	comboColor->Append("Green");
	comboColor->Append("Light Green");
	comboColor->Append("Water Green");
	comboColor->Append("Blue");
	comboColor->Append("Cyan");
	comboColor->Append("Violet");
	comboColor->Append("Magenta");
	comboColor->Append("Red");
	comboColor->Append("Orange");
	comboColor->Append("Brown");
	comboColor->Append("Yellow");
	comboColor->Append("Light Yellow");
	
	// DEFAULT COLORS
	palette.SetCustomColour(BIANCO,		Colore(BIANCO));
	palette.SetCustomColour(NERO,		Colore(NERO));
	palette.SetCustomColour(GRIGIO,		Colore(GRIGIO));
	palette.SetCustomColour(GRIGIOS,	Colore(GRIGIOS));
	palette.SetCustomColour(VERDE,		Colore(VERDE));
	palette.SetCustomColour(VERDEC,		Colore(VERDEC));
	palette.SetCustomColour(VERDEA,		Colore(VERDEA));
	palette.SetCustomColour(BLU,		Colore(BLU));
	palette.SetCustomColour(CELESTE,	Colore(CELESTE));
	palette.SetCustomColour(VIOLA,		Colore(VIOLA));
	palette.SetCustomColour(MAGENTA,	Colore(MAGENTA));
	palette.SetCustomColour(ROSSO,		Colore(ROSSO));
	palette.SetCustomColour(ARANCIO,	Colore(ARANCIO));
	palette.SetCustomColour(MARRONE,	Colore(MARRONE));
	palette.SetCustomColour(GIALLO,		Colore(GIALLO));
	palette.SetCustomColour(GIALLOC,	Colore(GIALLOC));

	//Inizializzazione di radioSize
	
	init_size=pPausa->GetNotina();
	if(init_size)
		radioSize->SetSelection(1);
	if(init_size==FALSE)
		radioSize->SetSelection(0);
	
	//Inizializzazione di comboStatus
	
	init_status=pPausa->GetStatus();
	comboStatus->SetSelection(init_status);
	
	//Inizializzazione di comboColor
	
	if((init_status==0)||(init_status==1))
	{
		init_color=pPausa->GetColor();
		comboColor->SetValue(init_color.GetCode());
	}
	else
	{
		//init_color=-1;
		comboColor->SetSelection(-1);//init_color);
		comboColor->Enable(FALSE);
	}
	
	//Inizializzazione di comboDuration
	
	init_duration=pPausa->GetID()-CL_PAUSA-1;
	comboDuration->SetSelection(init_duration);
	
	//Inizializzazione di spinHighness
	
	init_highness=pPausa->GetAltezza();
	spinHighness->SetValue(init_highness);
	
	init_staff=pPausa->GetNStaff();
	comboStaff->SetSelection(init_staff);
	
}


EditRestsDlg::~EditRestsDlg()
{
	if(comboDuration !=NULL) 
		delete comboDuration;
	if(comboStatus !=NULL) 
		delete comboStatus;
	if(comboColor !=NULL) 
		delete comboColor;
	if(comboStaff !=NULL) 
		delete comboStaff;
	if(radioSize !=NULL) 
		delete radioSize;
	if(spinHighness !=NULL) 
		delete spinHighness;
	
}

void EditRestsDlg::OnStatus(wxCommandEvent& event)
{
	if((comboStatus->GetSelection()==0)||(comboStatus->GetSelection()==1))
		comboColor->Enable(TRUE);
	else
		comboColor->Enable(FALSE);
}

void EditRestsDlg::OnPaletteBtnClick(wxCommandEvent& event)
{
	if (comboColor->GetSelection() == -1)
		palette.SetColour(Colore(comboColor->GetValue()));
	else
		palette.SetColour(Colore((ColorIcon)comboColor->GetSelection()));
	
	wxColourDialog dialog(this, &palette);
	dialog.SetTitle("Choose a color...");

	if (dialog.ShowModal() == wxID_OK)
	{
		wxColourData retData = dialog.GetColourData();
		Colore col = retData.GetColour();
		comboColor->SetValue(col.GetCode());
	}
}


void EditRestsDlg::OnOk(wxCommandEvent& event)
{
	//Impone i valori correnti (curr) e chiude la finestra di dialogo
	
	UpdateValue();
	Destroy();
}

void EditRestsDlg::OnApply(wxCommandEvent& event)
{
	//Impone i valori correnti (curr).
	
	UpdateValue();
}

void EditRestsDlg::OnCancel(wxCommandEvent& event)
{
	//Recupera i valori iniziali (init), li impone e chiude la finestra di dialogo.
	
	ClassID Target_ID=(ClassID)(init_duration+CL_PAUSA+1);
	if(Target_ID!=pPausa->GetID())
		pPausa=(Pausa *)currBat->ChangeFigDuration((Figura *)pPausa, Target_ID);
	pPausa->SetAltezza(init_highness);
	pPausa->SetNStaff(init_staff);
    pPausa->SetNotina(init_size);
	pPausa->SetColor(init_color);
	pPausa->SetStatus((Status_ID)init_status);
    MusicEditor->RedrawViews(0,TRUE);
	Destroy();
}

void EditRestsDlg::UpdateValue()
{
	//Legge i valori correnti (curr) e li impone.
	
	curr_duration=comboDuration->GetSelection();
	ClassID currID=(ClassID)(curr_duration+CL_PAUSA+1);
	if(currID!=pPausa->GetID())
	{
		pPausa=(Pausa *)currBat->ChangeFigDuration((Figura *)pPausa, currID);
	}
	
	if(pPausa!=NULL)
	{
		curr_size=radioSize->GetSelection();
		pPausa->SetNotina(curr_size);
		
		if (comboColor->GetSelection() == -1)
			curr_color=Colore(comboColor->GetValue());
		else
			curr_color=Colore((ColorIcon)comboColor->GetSelection());
		pPausa->SetColor(curr_color);
		
		curr_status=comboStatus->GetSelection();
		pPausa->SetStatus((Status_ID)curr_status);
		
		curr_highness=spinHighness->GetValue();
        pPausa->SetAltezza(curr_highness);
		curr_staff=comboStaff->GetSelection();
        pPausa->SetNStaff(curr_staff);
	}
	
	MusicEditor->RedrawViews(0,TRUE);
}

Bool EditingRestsDlg(Pausa *pPausa, Battuta *currBat)
{
	
	wxString control;
	
	EditRestsDlg *dialog = new EditRestsDlg(pPausa, currBat);
	dialog->Centre();
	if (dialog->Show(TRUE)) 
	{
		return TRUE;
	}
	else 
		return FALSE;
	
}
