/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/////////////////////////////////////////////////////////////////////////////
// Name:        ettmeasuredlg.cpp
// Purpose:     EttMeasureDlg
// Author:      Dario Baronti
// Modified by:
// Created:     
/////////////////////////////////////////////////////////////////////////////

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
  #include "wx/wx.h"
#endif

#if defined (__WXGTK__)
#include "../wxr/ettmeasuredlg.wxr"
#include "../wxr/ettmeasuredlg.h"
#endif


#if defined(__WXMSW__) 
#include "../wxr/ettmeasuredlg.wxr"
#include "../wxr/ettmeasuredlg.h"
#endif

#include "wx/resource.h"
#include "ettmeasuredlg.hpp"

#include "drw/musicedt.hpp"


BEGIN_EVENT_TABLE(EttMeasureDlg, wxDialog)
    EVT_BUTTON(ID_OK, EttMeasureDlg::OnOK)
	EVT_BUTTON(ID_Cancel, EttMeasureDlg::OnOK)
	EVT_BUTTON(114, EttMeasureDlg::OnNew)
	EVT_BUTTON(116, EttMeasureDlg::OnSave)
	EVT_BUTTON(115, EttMeasureDlg::OnDelete)
	EVT_TEXT(120, EttMeasureDlg::OnText)
	EVT_TEXT(121, EttMeasureDlg::OnText)
	EVT_LISTBOX(111, EttMeasureDlg::OnSelect)	

END_EVENT_TABLE()


EttMeasureDlg::EttMeasureDlg(AudioSyncWindow *syncWindow)
{
	CurrentState = INITIAL;
	EttListAux = new wxArrayString();
	
	wxResourceParseData(ettmeasuredlg);
#ifndef WDFEDONLY
	if (LoadFromResource(FrameManager::currFrame, "ettmeasuredlg"))
#else
		if (LoadFromResource(FrameManager::currFrame, "ettmeasuredlg"))
#endif
		{	
			From_tc = (wxTextCtrl *)wxFindWindowByName("from", this);
			To_tc = (wxTextCtrl *)wxFindWindowByName("to", this);
			Item_lb = (wxListBox *)wxFindWindowByName("lb", this);
			New_b = (wxButton *)wxFindWindowByName("new", this);
			Save_b = (wxButton *)wxFindWindowByName("save", this);
			Delete_b = (wxButton *)wxFindWindowByName("delete", this);
			
			Save_b->Enable(FALSE);

			From_tc->Enable(FALSE);
			To_tc->Enable(FALSE);
			Delete_b->Enable(FALSE);

			SyncWindow = syncWindow;
		}
		else
			wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
}


EttMeasureDlg::~EttMeasureDlg()
{
	From_tc->Destroy(); 
	To_tc->Destroy(); 
	Item_lb->Destroy(); 
}


void EttMeasureDlg::SetReadOnly()
{
	From_tc->Enable(FALSE);
	To_tc->Enable(FALSE);
	New_b->Enable(FALSE);
	Delete_b->Enable(FALSE);
	Save_b->Enable(FALSE);
}


void EttMeasureDlg::TransferDataFromObject()
{
	if(EttListAux!=NULL)
		for( int i=0; i<EttListAux->GetCount(); i++)
		{
			Item_lb->Append(EttListAux->Item(i));
		} 
}


bool EttMeasureDlg::InputDataValid()
{
	extern LiooWindow *pMusicWin;
	int maxTo = pMusicWin->PPartit->GetNumeroBattute();
	wxString from="",to="",errorMsg="";
	long f,t;
	bool valid = true;
	from=From_tc->GetValue();
	to=To_tc->GetValue();
	if(from.IsEmpty() && valid)
	{
		valid = false;
		errorMsg="Error: From value is empty!";
	}
	if(to.IsEmpty() && valid)
	{
		valid = false;
		errorMsg="Error: To value is empty!";
	}
	if(from.IsNumber() && to.IsNumber())
	{
		from.ToLong(&f);
		to.ToLong(&t);
	}
	else
	{
		if(valid)
		{
			valid = false;
			errorMsg="Error: To & From value must be numbers!";
		}
	}
	if( (!(f>0) || !(t>0)) && valid)
	{
		valid = false;
		errorMsg="Error: you can't insert negative value!";
	}
	if( f >= t && valid )
	{
		valid = false;
		errorMsg="Error: From value must be lower than To!";
	}
	if(t>maxTo && valid)
	{
		valid = false;
		errorMsg="Error: To value is bigger than number of measures!";
	}
	if(valid==false)
		wxMessageBox(errorMsg);
	return valid;
}

void EttMeasureDlg::OnNew(wxCommandEvent& WXUNUSED(event))
{
	New_b->Enable(FALSE);
	Delete_b->Enable(FALSE);
	
	if (CurrentState == INITIAL)
		Save_b->Enable(TRUE);
	
	CurrentState = CREATENEWITEM;
	
	From_tc->Enable(TRUE);
	To_tc->Enable(TRUE);
	From_tc->SetValue("");
	To_tc->SetValue("");
}

void EttMeasureDlg::OnDelete(wxCommandEvent& WXUNUSED(event))
{
	if (Item_lb->Number() > 0 && Item_lb->GetSelection()!=-1)
	{
		//Delete item
		EttListAux->Remove(Item_lb->GetSelection());
		Item_lb->Delete(Item_lb->GetSelection());

		//clear controls
		From_tc->SetValue("");
		To_tc->SetValue("");
	}
	if (CurrentState == MODIFYITEM)
	{
		Save_b->Enable(FALSE);
		CurrentState = INITIAL;
	}
	if(Item_lb->Number()>0)
	{
		Item_lb->SetSelection(Item_lb->Number()-1);
		wxString selection = Item_lb->GetStringSelection();
		if(selection!="")
		{
			wxString from = selection.BeforeFirst('-');
			wxString to = selection.AfterFirst('-');
			From_tc->SetValue(from);
			To_tc->SetValue(to); 
		}
	}
	else
	{
		Delete_b->Enable(FALSE);
		Save_b->Enable(FALSE);
		From_tc->Enable(FALSE);
		To_tc->Enable(FALSE);
	}
}

void EttMeasureDlg::OnText(wxCommandEvent& event)
{
	if (CurrentState == INITIAL && Item_lb->GetSelection()!=-1)
		CurrentState = MODIFYITEM;
	
	if(From_tc->IsModified() || To_tc->IsModified())
		Save_b->Enable(TRUE);
}

void EttMeasureDlg::OnSelect(wxCommandEvent& event)
{
	From_tc->Enable(TRUE);
	To_tc->Enable(TRUE);
	New_b->Enable(TRUE);
	Delete_b->Enable(TRUE);
	Save_b->Enable(FALSE);
	wxString selection = Item_lb->GetStringSelection();
	if(selection!="")
	{
		wxString from = selection.BeforeFirst('-');
		wxString to = selection.AfterFirst('-');
		From_tc->SetValue(from);
		To_tc->SetValue(to); 
	}
}

void EttMeasureDlg::OnOk(wxCommandEvent& event)
{
	EndModal(ID_OK);
}

void EttMeasureDlg::OnCancel(wxCommandEvent& event)
{
	EndModal(ID_Cancel);
}

void EttMeasureDlg::Block2StreamDataSync()
{
	wxArrayString *data = EttListAux;
	if(data==NULL || data->GetCount()==0)
		return;
	wxArrayString batSeq;
	int from, to;
	long n;
	wxString tmp;
	for( int i=0; i<data->GetCount(); i++)
	{
		tmp = data->Item(i).BeforeFirst('-');
		tmp.ToLong(&n);
		from = (int) n;
		tmp.Clear();
		tmp = data->Item(i).AfterFirst('-');
		tmp.ToLong(&n);
		to = (int) n;
		for(int j=from; j<=to; j++)
		{
			tmp.Clear();
			tmp.sprintf("%d",j);
			batSeq.Add(tmp);
		}
	}
	SyncWindow->SetAudioSyncData(&batSeq);
}

void EttMeasureDlg::StreamDataSync2Block()
{
	wxArrayString *data = SyncWindow->GetAudioSyncData();
	EttListAux->Clear();
	wxString from, to;
	int i=0;
	while(i<data->GetCount())
	{
		long n1,n2;
		from="";
		to="";
		from =  data->Item(i);
		for(int j=i+1;j<data->GetCount();j++)
		{
			data->Item(j).ToLong(&n1);
			data->Item(j-1).ToLong(&n2);
			if(j==(data->GetCount()-1))
			{
				to=data->Item(j);
				break;
			}
			if(n1 != ++n2)
			{
				to = data->Item(j-1);
				break;
			}		
		}
		i=j;
		if(!from.IsEmpty() && !to.IsEmpty())
			EttListAux->Add(from+"-"+to);	
	}	
	TransferDataFromObject();
}
