/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include "wx/wx.h"
#include "wx/dialog.h"
#include "wx/resource.h"
#include "drw/frmanage.hpp"
#include "lioo.hpp"
#include "par/partitur.hpp"
#include "drw/scorefrm.hpp"
#include "drw/musicedt.hpp" 
#include "pag/casellatxt.hpp"



#if defined (__WXGTK__)
#include "../wxr/xgeenrictextdlg.wxr"
#include "../wxr/xgenerictextdlg.h"
#endif

#if defined(__WXMSW__)
#include "../wxr/generictext.wxr"
#include "../wxr/generictext.h"

#endif


enum
{	
	L_BLU,
		L_ROSSO,
		L_GIALLO,
		L_VERDE,
		L_VERDEC,
		L_VERDEA,
		L_CELESTE,
		L_VIOLA,
		L_MAGENTA,
		L_ARANCIO,
		L_MARRONE,
		L_GIALLOC
};




class GenericTextDlg :public wxDialog
{
protected:
   	
	wxTextCtrl		*text_tc;
	wxCheckBox		*hidden_cb;
	wxChoice		*align_c;
	wxRadioBox		*pos_rb;
	wxButton		*buttonfont_b;
	wxFont			font;
	
public:	
	GenericTextDlg(bool canHide);
	//Costruttore
	
	~GenericTextDlg();
	//Distruttore
	
    void OnOk(wxCommandEvent& event);
	//Metodo per la chiusura e la distruzione della finestra
	
	void OnCancel(wxCommandEvent& event);
	//Metodo per la visualizzazione del colore del layer selezionto nella listbox
	
	void SetText(const wxString& txt) { text_tc->SetValue(txt); };
	wxString GetText() { return text_tc->GetValue(); };
	void SetHidden(bool h) { hidden_cb->SetValue(h); };
	bool GetHidden() { return hidden_cb->GetValue(); };
	Align GetAlign();
	void SetAlign(Align align); 
	wxFont GetFont();	
	void SetFont(wxFont f);
	void SetPos(int pos){pos_rb->SetSelection(pos);};
	int GetPos(){return pos_rb->GetSelection();};
	
	bool IsOk() { return text_tc!=NULL && hidden_cb!=NULL && align_c!=NULL && pos_rb!=NULL; };
	
private:
    DECLARE_EVENT_TABLE()
};


BEGIN_EVENT_TABLE(GenericTextDlg, wxDialog)
EVT_BUTTON(wxID_OK, GenericTextDlg::OnOk)
EVT_BUTTON(wxID_CANCEL, GenericTextDlg::OnCancel)
END_EVENT_TABLE()



GenericTextDlg::GenericTextDlg(bool canHide) 
{
	text_tc=NULL;
	hidden_cb=NULL;
	
	wxResourceParseData(generictext); 
	
	if (LoadFromResource(FrameManager::currFrame, "generictext"))
	{
		text_tc = (wxTextCtrl *)wxFindWindowByName("gentext", this);	 
		hidden_cb = (wxCheckBox *)wxFindWindowByName("hidden", this);	 
		align_c = (wxChoice *)wxFindWindowByName("align",this);
		pos_rb = (wxRadioBox *)wxFindWindowByName("pos",this);
		buttonfont_b = (wxButton *)wxFindWindowByName("font",this);
		
		align_c->Enable(false);
		hidden_cb->Enable(canHide);
		buttonfont_b->Enable(false);
		
		//Default 
		SetAlign(A_Left);
		SetFont(wxFont(10,wxDEFAULT,wxNORMAL,wxNORMAL));
		SetPos(1);
		SetHidden(false);
		
	}
	else
	{
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
	}
}

GenericTextDlg::~GenericTextDlg()
{
}

void GenericTextDlg::OnOk(wxCommandEvent& event)
{
	EndModal(wxID_OK);
}

void GenericTextDlg::OnCancel(wxCommandEvent& event)
{
	EndModal(wxID_CANCEL);
}

Align GenericTextDlg::GetAlign()
{
	Align align=A_Left;
	int sel=align_c->GetSelection();
	switch(align)
	{
	case 0:
		align=A_Left;
		break;
	case 1:
		align=A_Right;
		break;
	case 2:
		align=A_Centre;
		break;
	}
	return align;
}

void GenericTextDlg::SetAlign(Align align)
{
	if(align==A_Left)
		align_c->SetSelection(0);
	else if(align==A_Centre)
		align_c->SetSelection(2);
	else if(align==A_Right)
		align_c->SetSelection(1);
}

wxFont GenericTextDlg::GetFont()
{
	return text_tc->GetFont();
}

void GenericTextDlg::SetFont(wxFont f)
{
	text_tc->SetFont(f);
}




/************Funzione di chiamata alla dialog**************************/



Bool InputGenericText(char *txt,int len, bool& hidden, Align& align, wxFont& font, bool& pos, bool canHide)
{
	GenericTextDlg dialog(canHide);
	
	if(!dialog.IsOk())
		return FALSE;
	
	dialog.Centre();
	dialog.SetText(txt);
	dialog.SetHidden(hidden);
	dialog.SetPos(pos);
	if(dialog.ShowModal()==wxID_OK)
	{
		strncpy(txt,dialog.GetText(),len);
		hidden=dialog.GetHidden();
		align=dialog.GetAlign();
		font=dialog.GetFont();
		pos=(bool)dialog.GetPos();
		return TRUE;
	}
	return FALSE;
}