/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include "../wxr/imgscoreprintdlg.wxr"	
#include "../wxr/imgscoreprintdlg.h"
#include "gui/dlg/imgscoreprintdlg.hpp"

BEGIN_EVENT_TABLE(ImgScorePrintDlg, wxDialog)
EVT_BUTTON(ID_BUTTONOK, ImgScorePrintDlg::OnOk)
EVT_BUTTON(ID_BUTTONCANC, ImgScorePrintDlg::OnCancel)
EVT_RADIOBUTTON(ID_BUTTONALL, ImgScorePrintDlg::OnAll)
EVT_RADIOBUTTON(ID_BUTTONCURRENT, ImgScorePrintDlg::OnCurrent)
EVT_RADIOBUTTON(ID_BUTTONSELECTED, ImgScorePrintDlg::OnSelected)
END_EVENT_TABLE()

ImgScorePrintDlg::ImgScorePrintDlg()
{	
	wxResourceParseData(imgscoreprintdlg);
#ifndef WDFEDONLY
	if (LoadFromResource(FrameManager::currFrame, "imgscoreprintdlg"))
#else
		if (LoadFromResource(NULL, "imgscoreprintdlg"))
#endif
		{	
			from=(wxTextCtrl *)wxFindWindowByName("from",this);
			to=(wxTextCtrl *)wxFindWindowByName("to",this);
			all=(wxRadioButton *)wxFindWindowByName("all",this);
			current=(wxRadioButton *)wxFindWindowByName("current",this);
			selected=(wxRadioButton *)wxFindWindowByName("selected",this);
			
			all->SetValue(true);
			from->SetEditable(false);
			to->SetEditable(false);
		}
		else
			wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
		
}


ImgScorePrintDlg::~ImgScorePrintDlg()
{
	if(to!=NULL)
		delete to;
	if(from!=NULL)
		delete from;
	if(all!=NULL)
		delete all;	
	if(current!=NULL)
		delete current;
	if(selected!=NULL)
		delete selected;
}



void ImgScorePrintDlg::OnOk(wxCommandEvent& event)
{
	if(GetModeSelection()==3)
	{
		double t,f;
		GetPrintFrom().ToDouble(&f);
		GetPrintTo().ToDouble(&t);
		if(GetPrintFrom().IsNumber() && GetPrintTo().IsNumber() && (int)t>0 && (int)f>0)
			EndModal(wxID_OK);
		else
			wxMessageBox("Error reading from-to informations!");
	}
	else
		EndModal(wxID_OK);
}

void ImgScorePrintDlg::OnCancel(wxCommandEvent& event)
{
	EndModal(wxID_CANCEL);
}

int ImgScorePrintDlg::GetModeSelection()
{
	if(all->GetValue())
		return 1;
	if(current->GetValue())
		return 2;
	if(selected->GetValue())
		return 3;
	
	return 1;
}

wxString ImgScorePrintDlg::GetPrintFrom()
{
	return from->GetValue();
}

wxString ImgScorePrintDlg::GetPrintTo()
{
	return to->GetValue();
}

void ImgScorePrintDlg::SetModeSelection(int sel)
{
	switch(sel)
	{
	case 1:
		all->SetValue(true);
		from->SetEditable(false);
		to->SetEditable(false);
		break;
	case 2:
		current->SetValue(true);
		from->SetEditable(false);
		to->SetEditable(false);
		break;
	case 3:
		selected->SetValue(true);
		from->SetEditable(true);
		to->SetEditable(true);
		break;
	}
}

void ImgScorePrintDlg::OnAll(wxCommandEvent& event)
{
	from->SetEditable(false);
	to->SetEditable(false);
}

void ImgScorePrintDlg::OnCurrent(wxCommandEvent& event)
{
	from->SetEditable(false);
	to->SetEditable(false);
}

void ImgScorePrintDlg::OnSelected(wxCommandEvent& event)
{
	from->SetEditable(true);
	to->SetEditable(true);
}










