/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include "wx/wx.h"
#include "wx/dialog.h"
#include "wx/resource.h"
#include "drw/frmanage.hpp"
#include "lioo.hpp"
#include "par/partitur.hpp"
#include "drw/scorefrm.hpp"
#include "drw/musicedt.hpp" 



#if defined (__WXGTK__)
#include "../wxr/xmultiplechoicedlg.wxr"
#include "../wxr/xmultiplechoicedlg.h"
#endif

#if defined(__WXMSW__)
#include "../wxr/multiplechoicedlg.wxr"
#include "../wxr/multiplechoicedlg.h"

#endif







class MutipleChoiceDlg :public wxDialog
{
protected:
   	
	wxListBox	*list;
	int			item2select;
	
	
public:	
	MutipleChoiceDlg(wxArrayString *array, wxString title="", int items=1);
	//Costruttore
	
	~MutipleChoiceDlg();
	//Distruttore
	
	void OnOk(wxCommandEvent& event);
	//Metodo per la gestione dell'evento di pressione del bottone ok
	
	void OnCancel(wxCommandEvent& event);
	//Metodo per la gestione dell'evento di pressione del bottone cancel
	
    void OnSelected(wxCommandEvent& event);
	//Metodo per la gestione del controllo delle parti selezionate
	
	int GetSelection(int *array);
	//Metodo per la restituzione dell'elenco delle parti selezionate
	
	
	
	
	
private:
    DECLARE_EVENT_TABLE()
};




BEGIN_EVENT_TABLE(MutipleChoiceDlg, wxDialog)
EVT_BUTTON(ID_BUTTONOK, MutipleChoiceDlg::OnOk)
EVT_BUTTON(ID_BUTTONCANCEL, MutipleChoiceDlg::OnCancel)
EVT_LISTBOX(ID_LISTPART, MutipleChoiceDlg::OnSelected)	
END_EVENT_TABLE()



MutipleChoiceDlg::MutipleChoiceDlg(wxArrayString *array, wxString title, int items) 
{
	
	
	list				= NULL;
	item2select			= items;
	int count			= 0;
	
	
	wxResourceParseData(MChoiceDlg); 
	if (LoadFromResource(FrameManager::currFrame, "MChoiceDlg"))
	{
		
		list = (wxListBox *)wxFindWindowByName("partlist", this);
		
	}
	else
	{
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
	}
	SetTitle(title);
	count = array->GetCount();
	if(count!=0)
		for(int i=0; i<count; i++)
			list->Append(array->Item(i));
		
}


MutipleChoiceDlg::~MutipleChoiceDlg()
{
	
	if(list!=NULL) 
		delete list;
}


void MutipleChoiceDlg::OnSelected(wxCommandEvent& event)
{
	int numofsel, currsel;
	wxArrayInt selections;
	numofsel=list->GetSelections(selections);
	currsel=event.GetSelection();
	if(numofsel>item2select)
		list->Deselect(currsel);
}

void MutipleChoiceDlg::OnOk(wxCommandEvent& event)
{
	EndModal(wxID_OK);
}

void MutipleChoiceDlg::OnCancel(wxCommandEvent& event)
{
	EndModal(wxID_CANCEL);
}

int MutipleChoiceDlg::GetSelection(int *array)
{
	int numofsel;
	wxArrayInt selections;
	numofsel=list->GetSelections(selections);
	if(numofsel>3)
		numofsel=item2select;
	for(int i=0; i<numofsel; i++)
		array[i]=selections[i];
	return numofsel;
}



/************Funzione di chiamata alla dialog**************************/



Bool MutipleChoiceDial(int *parti, Partitura *pPartit, int n_items, wxString title =" ")
{
	int i, j, *vectparti;
	wxArrayString array;
	wxString tmp;
	Spartito *spa;
	int numParts=pPartit->GetNumSpa();
	vectparti = new int[numParts];
	j=0;
	for(i=0;i<numParts;i++)
	{
		spa=pPartit->GetSpartNum(i);
		if(spa->GetNumberOfStaff()==1)
		{
			tmp.sprintf("%s",spa->GetNome());
			if(tmp.IsEmpty())
			{
				tmp.Clear();
				tmp.sprintf("Part %d",i+1);
			}
			array.Add(tmp);
			tmp.Clear();
			vectparti[j]=i;
			j++;
		}
	}
	
	MutipleChoiceDlg dialog(&array,title,n_items);
	int part[3];
	
	
	dialog.Centre();
	if(dialog.ShowModal()== wxID_OK)
	{
		int num=dialog.GetSelection(part);
		switch(num)
		{
		case 2:
		case 3:
			for(i=0; i<num; i++)
				parti[i]=vectparti[part[i]];
			break;
		default:
			wxMessageBox("Invalid choice", "Error");
			return FALSE;
			break;
			
		}
		return TRUE;
	}
	else 
		return FALSE;
	
}