/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "drw/scorefrm.hpp"	
#include "wx/resource.h"
#include "prinpdlg.hpp"
#include "prinpdlg2.hpp"


#if defined (__WXGTK__)
#include "../wxr/ppl1x.wxr"
#include "../wxr/ppl1x.h"
#endif


#if defined(__WXMSW__) 
#include "../wxr/ppl1w.wxr"
#include "../wxr/ppl1w.h"
#endif


BEGIN_EVENT_TABLE(PrintParamsDlg, SMRDialog)
EVT_BUTTON(6024, PrintParamsDlg::OnFing)
EVT_BUTTON(6028, PrintParamsDlg::OnCalculateStaffDistance)
END_EVENT_TABLE()


PrintParamsDlg::PrintParamsDlg(formality_type formality, PrintParams* printp, int page, 
									 int nstaff, bool ismainscore,
                                     wxWindow* parent, const wxWindowID id, 
									 const wxString& title, const wxPoint& pos, const wxSize& size,
                                     const long windowStyle, const wxString& name,
                                     structure_type structure)
									 :SMRDialog(formality, structure, parent, id, title, pos, size, windowStyle)
{
	printParams = printp;
	nPage=page;
	nStaff=nstaff;
	IsMainScore=ismainscore;
	
	
	wxResourceParseData(PP); 
	
	if (panel->LoadFromResource(this, "PP"))
	{
		TopMargin_tc = (wxTextCtrl *)wxFindWindowByName("tc1", panel);
		BottomMargin_tc = (wxTextCtrl *)wxFindWindowByName("tc2", panel);
		LeftMargin_tc = (wxTextCtrl *)wxFindWindowByName("tc3", panel);
		RightMargin_tc = (wxTextCtrl *)wxFindWindowByName("tc4", panel);
		
		PageFormat_c = (wxChoice *)wxFindWindowByName("c2", panel);
		PageFormat_c->Append("A4");
		PageFormat_c->Append("A3");
		PageFormat_c->Append("A4R");
		PageFormat_c->Append("A3R");
		Printer_c = (wxChoice *)wxFindWindowByName("c1", panel);
		Printer_c->Append("300 dpi");
		Printer_c->Append("600 dpi");
		Magnification_c = (wxChoice *)wxFindWindowByName("choice-mag", panel);
		for (int MAidx=0;MAidx<11;MAidx++)
		{
			wxString str = "";
			str.Printf("%.1f",MagnificationArray[MAidx]);
			Magnification_c->Append(str);
		}
		LineLength_tc = (wxTextCtrl *)wxFindWindowByName("tc5", panel); 
		
		NumSystems_tc = (wxTextCtrl *)wxFindWindowByName("textctrl4", panel); 
		Distance_tc = (wxTextCtrl *)wxFindWindowByName("textctrl5", panel); 
		InsPageNum_c = (wxCheckBox *)wxFindWindowByName("checkbox6", panel); 
		
		Calculate_b=(wxButton *)wxFindWindowByName("BCalculate", this);
	}
	else
	{
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
	}
	
	SetSize(-1,-1, PANEL_LEFTMARGIN + panel->GetSize().GetWidth() + PANEL_RIGHTMARGIN, 
		PANEL_TOPMARGIN + panel->GetSize().GetHeight() + PANEL_BOTTOMMARGIN+10);
	
	TransferDataFromObject();
}


PrintParamsDlg::~PrintParamsDlg()
{
	TopMargin_tc->Destroy();
	BottomMargin_tc->Destroy();
	LeftMargin_tc->Destroy();
	RightMargin_tc->Destroy();
	PageFormat_c->Destroy();
	Printer_c->Destroy();
	Magnification_c->Destroy();
	LineLength_tc->Destroy();
	NumSystems_tc->Destroy();
	Distance_tc->Destroy();
	InsPageNum_c->Destroy();
	Calculate_b->Destroy();
}


void PrintParamsDlg::SetReadOnly()
{
	TopMargin_tc->Enable(FALSE);
	BottomMargin_tc->Enable(FALSE);
	LeftMargin_tc->Enable(FALSE);
	RightMargin_tc->Enable(FALSE);
	PageFormat_c->Enable(FALSE);
	Printer_c->Enable(FALSE);
	Magnification_c->Enable(FALSE);
	LineLength_tc->Enable(FALSE);
	NumSystems_tc->Enable(FALSE);
	Distance_tc->Enable(FALSE);
	InsPageNum_c->Enable(FALSE);
	Calculate_b->Enable(FALSE);
}


bool PrintParamsDlg::InputDataValid()
{
	InputErrorList.Clear();
	bool valid = true;
	
	return valid;
}


void PrintParamsDlg::OnFing()
{
	
	bool EditFingerParams(formality_type formality, PrintParams& printp, wxWindow* parent);
	PrintParams wdfppar;
	
	if (ReadOnly)
		EditFingerParams(view, *printParams, this);
	else 
		EditFingerParams(edit, *printParams, this);
}



// copia i parametri nella finestra
void PrintParamsDlg::TransferDataFromObject()
{
	wxString str = "";
	bool sym=((wxMEScoreFrame*)GetParent())->GetSimbImg();
	
	
	str.Printf("%i", printParams->GetTopMargin(nPage));
	TopMargin_tc->SetValue(str);
	
	str.Printf("%i", printParams->GetLeftMargin(nPage));
	LeftMargin_tc->SetValue(str);
	
	str.Printf("%i", printParams->GetRightMargin(nPage));
	RightMargin_tc->SetValue(str);
	
	str.Printf("%i", printParams->GetBottomMargin(nPage));
	BottomMargin_tc->SetValue(str);
	
	
	PageFormat_c->SetSelection(printParams->GetPageFormat());
	
	Printer_c->SetSelection(printParams->GetPrinter());
	
	if(sym)
		Magnification_c->SetSelection(printParams->GetMagnificationIndex(printParams->GetMagnification()));
	else
		Magnification_c->Enable(FALSE);
	
	str.Printf("%i", printParams->GetLineLength());
	LineLength_tc->SetValue(str);
	LineLength_tc->Enable(FALSE);
	
	if(IsMainScore)
		str.Printf("%i", printParams->GetNumSystems(nPage));
	else
		str.Printf("%i", printParams->GetNumStaff(nPage));
	if(sym)
		NumSystems_tc->SetValue(str);
	else
		NumSystems_tc->Enable(FALSE);
	
	str.Printf("%i", printParams->GetStaffDistance(nPage));
	if(sym)
		Distance_tc->SetValue(str);
	else
		Distance_tc->Enable(FALSE);
	
	InsPageNum_c->SetValue(printParams->GetInsPageNum(nPage));
	
	
	if(nStaff==0)
		Calculate_b->Enable(false);
}


void PrintParamsDlg::TransferDataToObject()
{
	
	long top;
	TopMargin_tc->GetValue().ToLong(&top);
	printParams->SetTopMargin(nPage,(int)top);
	
	long left;
	LeftMargin_tc->GetValue().ToLong(&left);
	printParams->SetLeftMargin(nPage,(int)left);
	
	long right;
	RightMargin_tc->GetValue().ToLong(&right);
	printParams->SetRightMargin(nPage,(int)right);
	
	long bottom;
	BottomMargin_tc->GetValue().ToLong(&bottom);
	printParams->SetBottomMargin(nPage,(int)bottom);
	
	printParams->SetPageFormat((PageFormat_type)PageFormat_c->GetSelection());	
	printParams->SetPrinter((Printer_type)Printer_c->GetSelection());
	printParams->SetMagnification(MagnificationArray[Magnification_c->GetSelection()]);
	
	long linelength;
	LineLength_tc->GetValue().ToLong(&linelength);
	printParams->SetLineLength((int)linelength);
	
	long staffdistance;
	Distance_tc->GetValue().ToLong(&staffdistance);;
	printParams->SetStaffDistance(nPage,(int)staffdistance);
	
	long sysnum;
	NumSystems_tc->GetValue().ToLong(&sysnum);
	if(IsMainScore)
		printParams->SetNumSystems(nPage,(int)sysnum);
	else
		printParams->SetNumStaff(nPage,(int)sysnum);
	
	printParams->SetInsPageNum(nPage,InsPageNum_c->GetValue());
	
}

void PrintParamsDlg::OnCalculateStaffDistance()
{
	if(nStaff==0)
		return;
	printParams->CalculateStaffDistance(nPage, nStaff, IsMainScore);
	TransferDataFromObject();
}



bool EditPrintParams(formality_type formality, PrintParams *printp ,int nPage, int nStaff, bool isMainScore, wxWindow* parent)
{ 
	PrintParamsDlg* dialog = new PrintParamsDlg(formality, printp, nPage, nStaff, isMainScore, parent, 1);
	dialog->Centre(wxBOTH);
	if (formality == view)
		dialog->SetReadOnly();
	
	if (dialog->ShowModal() == wxID_OK)
	{
		dialog->TransferDataToObject();
		dialog->Destroy();
		return 0;
	}
	
	dialog->Destroy();
	return 1;
}

