/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/resource.h"
#include "prinpdlg2.hpp"

#if defined (__WXGTK__)
#include "../wxr/ppl2x.wxr"
#include "../wxr/ppl2x.h"
#endif


#if defined(__WXMSW__) 
#include "../wxr/ppl2w.wxr"
#include "../wxr/ppl2w.h"
#endif


BEGIN_EVENT_TABLE(FingerParamsDlg, SMRDialog)
EVT_CHECKBOX(106, FingerParamsDlg::OnAdd)
EVT_BUTTON(6005, FingerParamsDlg::OnAddText)
EVT_BUTTON(107, FingerParamsDlg::OnDel)
END_EVENT_TABLE()


FingerParamsDlg::FingerParamsDlg(formality_type formality, PrintParams* printp, 
									   wxWindow* parent, const wxWindowID id, 
									   const wxString& title, const wxPoint& pos, const wxSize& size,
									   const long windowStyle, const wxString& name,
									   structure_type structure)
									   :SMRDialog(formality, structure, parent, id, title, pos, size, windowStyle)
{
	printParams = printp;
	
	wxResourceParseData(wdfprintr); 
	
	if (panel->LoadFromResource(this, "wdfprintr"))
	{
		Add_b = (wxButton *)wxFindWindowByName("b1", panel);
		AddText_b = (wxButton *)wxFindWindowByName("b-fing", panel);
		Del_b = (wxButton *)wxFindWindowByName("b2", panel);
		Finger_lb = (wxListBox *)wxFindWindowByName("lb1", panel);
		Docs_lb = (wxListBox *)wxFindWindowByName("lb2", panel);
		
	}
	else
	{
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
	}
	
	SetSize(-1,-1, PANEL_LEFTMARGIN + panel->GetSize().GetWidth() + PANEL_RIGHTMARGIN, 
		PANEL_TOPMARGIN + panel->GetSize().GetHeight() + PANEL_BOTTOMMARGIN);
	
	TransferDataFromObject();
}


FingerParamsDlg::~FingerParamsDlg()
{
	Add_b->Destroy();
	AddText_b->Destroy();
	Del_b->Destroy();
	Finger_lb->Destroy();
	Docs_lb->Destroy();
}


void FingerParamsDlg::SetReadOnly()
{
	Add_b->Enable(FALSE);
	AddText_b->Enable(FALSE);
	Del_b->Enable(FALSE);
	Finger_lb->Enable(FALSE);
	Docs_lb->Enable(FALSE);
}


bool FingerParamsDlg::InputDataValid()
{
	InputErrorList.Clear();
	bool valid = true;
	return valid;
}


void FingerParamsDlg::OnAddText()
{
	bool EditFingerTextParams(formality_type formality, PrintParams& printp, wxWindow* parent);
	PrintParams wdfppar;
	
	if (ReadOnly)
		EditFingerTextParams(view, *printParams, this);
	else 
		EditFingerTextParams(edit, *printParams, this);
}

void FingerParamsDlg::OnAdd()
{
	wxMessageBox("Add");
}


void FingerParamsDlg::OnDel()
{
	wxMessageBox("Del");
}



void FingerParamsDlg::TransferDataFromObject()
{
}


void FingerParamsDlg::TransferDataToObject()
{
	wxString str = "";
	
}


bool EditFingerParams(formality_type formality, PrintParams& printp, wxWindow* parent)
{ 
	FingerParamsDlg* dialog = new FingerParamsDlg(formality, &printp, parent, 1);
	dialog->Centre(wxBOTH);
	if (formality == view)
		dialog->SetReadOnly();
	
	if (dialog->ShowModal() == wxID_OK)
	{
		dialog->TransferDataToObject();
		dialog->Destroy();
		return 0;
	}
	
	dialog->Destroy();
	return 1;
}

