/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/resource.h"
#include "prinpdlg3.hpp"

#if defined (__WXGTK__)
#include "../wxr/ppl3x.wxr"
#include "../wxr/ppl3x.h"
#endif


#if defined(__WXMSW__) 
#include "../wxr/ppl3w.wxr"
#include "../wxr/ppl3w.h"
#endif


AddFingerTextDlg::AddFingerTextDlg(formality_type formality, PrintParams* printp, 
										 wxWindow* parent, const wxWindowID id, 
										 const wxString& title, const wxPoint& pos, const wxSize& size,
										 const long windowStyle, const wxString& name,
										 structure_type structure)
										 :SMRDialog(formality, structure, parent, id, title, pos, size, windowStyle)
{
	printParams = printp;
	
	wxResourceParseData(addfingertext); 
	
	if (panel->LoadFromResource(this, "addfingertext"))
	{
		TextF1_tc = (wxTextCtrl *)wxFindWindowByName("textf1", panel);
		TextF2_tc = (wxTextCtrl *)wxFindWindowByName("textf2", panel);	
		TextF3_tc = (wxTextCtrl *)wxFindWindowByName("textf3", panel);
		
	}
	else
	{
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
	}
	
	SetSize(-1,-1, PANEL_LEFTMARGIN + panel->GetSize().GetWidth() + PANEL_RIGHTMARGIN, 
		PANEL_TOPMARGIN + panel->GetSize().GetHeight() + PANEL_BOTTOMMARGIN);
	
	TransferDataFromObject();
}


AddFingerTextDlg::~AddFingerTextDlg()
{
	TextF1_tc->Destroy();
	TextF2_tc->Destroy();
	TextF3_tc->Destroy();
}


void AddFingerTextDlg::SetReadOnly()
{
	TextF1_tc->Enable(FALSE);
	TextF2_tc->Enable(FALSE);
	TextF3_tc->Enable(FALSE);
}


bool AddFingerTextDlg::InputDataValid()
{
	InputErrorList.Clear();
	bool valid = true;
	return valid;
}

void AddFingerTextDlg::TransferDataFromObject()
{
	TextF1_tc->SetValue(printParams->GetFingerPrint1());
	TextF2_tc->SetValue(printParams->GetFingerPrint2());
	TextF3_tc->SetValue(printParams->GetFingerPrint3());
	
}


void AddFingerTextDlg::TransferDataToObject()
{
	printParams->SetFingerPrint1(TextF1_tc->GetValue());
	printParams->SetFingerPrint2(TextF2_tc->GetValue());
	printParams->SetFingerPrint3(TextF3_tc->GetValue());
	
}


bool EditFingerTextParams(formality_type formality, PrintParams& printp, wxWindow* parent)
{ 
	AddFingerTextDlg* dialog = new AddFingerTextDlg(formality, &printp, parent, 1);
	dialog->Centre(wxBOTH);
	if (formality == view)
		dialog->SetReadOnly();
	
	if (dialog->ShowModal() == wxID_OK)
	{
		dialog->TransferDataToObject();
		dialog->Destroy();
		return 0;
	}
	
	dialog->Destroy();
	return 1;
}

