/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/resource.h"
#include "smrdlg.hpp"

BEGIN_EVENT_TABLE(SMRDialog, wxDialog)
EVT_BUTTON(wxID_CANCEL, SMRDialog::OnCancel)
EVT_BUTTON(wxID_OK, SMRDialog::OnOk)
EVT_KEY_UP(SMRDialog::OnKey)
END_EVENT_TABLE()


SMRDialog::SMRDialog(formality_type formality, structure_type structure, 
                     wxWindow* parent, const wxWindowID id, const wxString& title, 
                     const wxPoint& pos, const wxSize& size, const long windowStyle, 
                     const wxString& name)
					 :wxDialog(parent, id, title, pos, size, windowStyle)
{
	
#ifdef WEDEL_PLUGIN_CODE
	Plugin * plugin = Plugin::getPlugin ();
	plugin->disableExplorerWindow ();
#endif
	
	if (formality == edit)
		SetReadOnly(false);
	else if (formality == view)
		SetReadOnly(true);
	
	if (structure == singlePanel)
		SetMultiPanel(false);
	else if (structure == multiPanel)
		SetMultiPanel(true);
	
	if (MultiPanel)
	{
		notebook = new wxNotebook(this, notebook_ID, wxPoint(-1, -1), wxSize(-1, -1)); //, wxTAB_TRAVERSAL);
		panel = NULL;
	}
	else 
	{ 
		panel = new wxPanel();
		notebook = NULL;
	}
	
	ok_b = new wxButton(this, wxID_OK, "Ok", wxPoint(-1, -1), wxSize(-1, -1), 
		0, wxDefaultValidator, "ok_b");
	if (ReadOnly)
		cancel_b = NULL;
	else
		cancel_b = new wxButton(this, wxID_CANCEL, "Cancel", wxPoint(-1, -1), wxSize(-1, -1), 
		0, wxDefaultValidator, "cancel_b");
	
	
	wxLayoutConstraints *c;
	
	//set notebook or panel layout
	c = new wxLayoutConstraints;
	c->top.SameAs(this, wxTop, PANEL_TOPMARGIN);
	c->left.SameAs(this, wxLeft, PANEL_LEFTMARGIN);
	c->right.SameAs(this, wxRight, PANEL_RIGHTMARGIN);
	c->bottom.SameAs(this, wxBottom, PANEL_BOTTOMMARGIN);
	if (MultiPanel)
		notebook->SetConstraints(c);
	else
		panel->SetConstraints(c);
	
	if (ReadOnly)
	{
		//set ok_b layout
		c = new wxLayoutConstraints;
		c->centreX.SameAs(this, wxCentreX);
		c->bottom.SameAs(this, wxBottom, PANEL_BOTTOMMARGIN/2 - ok_b->GetSize().GetHeight()/2 -1);
		c->height.AsIs();
		c->width.AsIs();
		ok_b->SetConstraints(c);
	}
	else 
    {
		//set ok_b layout
		c = new wxLayoutConstraints;
		c->right.SameAs(this, wxCentreX, OKCANCEL_DISTANCE/2);
		c->bottom.SameAs(this, wxBottom, PANEL_BOTTOMMARGIN/2 - ok_b->GetSize().GetHeight()/2 -1);
		c->height.AsIs();
		c->width.AsIs();
		ok_b->SetConstraints(c);
		
		//set cancel_b layout
		c = new wxLayoutConstraints;
		c->left.SameAs(this, wxCentreX, OKCANCEL_DISTANCE/2);
		c->bottom.SameAs(this, wxBottom, PANEL_BOTTOMMARGIN/2 - cancel_b->GetSize().GetHeight()/2 -1);
		c->height.AsIs();
		c->width.AsIs();
		cancel_b->SetConstraints(c);
    }
	
	SetAutoLayout(TRUE);
	Layout();
	
	ErrorMessageDialog = NULL;
	FirstControlError = NULL;
}

SMRDialog::~SMRDialog()
{
	if (MultiPanel)
		notebook->Destroy();
	else
		panel->Destroy();
	ok_b->Destroy();
	if (cancel_b)
		cancel_b->Destroy();
#ifdef WEDEL_PLUGIN_CODE
	Plugin * plugin = Plugin::getPlugin ();
	plugin->enableExplorerWindow ();
#endif
}


void SMRDialog::OnOk(wxCommandEvent& WXUNUSED(event))
{
	if (!ReadOnly)
	{
		if (!InputDataValid()) 
		{
			//show error message
			wxString str = "";
			for (int i=0; i<InputErrorList.Number(); i++)
				str = str + (InputErrorList.Item(i))->GetData() + "\n";
			ErrorMessageDialog = new wxMessageDialog(this, str, "Not valid input",
				wxICON_EXCLAMATION | wxCENTRE);
			ErrorMessageDialog->ShowModal();
			ErrorMessageDialog->Destroy();
			
			
			
			//set focus on first control with incorrect data
			if (MultiPanel)  
			{
				wxPanel *panel = (wxPanel*)Getnotebook()->GetPage(Getnotebook()->GetSelection());
				while (panel != FirstControlError->GetParent())
				{
					Getnotebook()->AdvanceSelection();
					panel = (wxPanel*)Getnotebook()->GetPage(Getnotebook()->GetSelection());
				}
			}
			FirstControlError->SetFocus();
			
			return;
		}  //end InputDataValid
	} //end ReadOnly
	
	EndModal(wxID_OK);
}


void SMRDialog::OnCancel(wxCommandEvent& WXUNUSED(event))
{
    EndModal(wxID_CANCEL);
}

void SMRDialog::OnCloseWindow(wxCloseEvent& WXUNUSED(event))
{
    EndModal(wxID_CANCEL);
}

void SMRDialog::OnKey(wxKeyEvent& event)
{
	switch(event.KeyCode())
	{
	case WXK_PRIOR:
		if(notebook->GetSelection()>0)
			notebook->SetSelection(notebook->GetSelection()-1);
		break;
	case WXK_NEXT:
		if(notebook->GetSelection()<notebook->GetPageCount()-1)
			notebook->SetSelection(notebook->GetSelection()+1);
		break;
	}
}




