/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#ifndef SMRDLG_HPP
#define SMRDLG_HPP

#include "wx/notebook.h"
#include "wx/button.h"
#include "wx/wx.h"


//margins of the panel (or notebook) contents in the dialog
#define PANEL_TOPMARGIN 8
#define PANEL_LEFTMARGIN 8
#define PANEL_RIGHTMARGIN 8
#define PANEL_BOTTOMMARGIN 44  //ok and cancel button are posizionated at middle of
                               //of this margin 
#define OKCANCEL_DISTANCE 30  //horizontal distance between ok and cancel button


enum formality_type {edit, view};
//The dialog can be open in read/write or readonly formality

enum structure_type {singlePanel, multiPanel};
//Dialog structure (with a panel or with a notebook that contents more panels)

class SMRDialog: public wxDialog
{
 protected:
  wxNotebook *notebook;  //created for multiPanel dialog; contents the panels
	wxPanel *panel; //created for singlePanel dialog
	wxButton *ok_b;
	wxButton *cancel_b;  //created if read/write formality

	bool ReadOnly;  //1 if formality is view, 0 if edit
	bool MultiPanel;  //1 if structure is multiPanel, 0 if singlePanel

	wxStringList InputErrorList;  //list with messages relatively to incorrect data   
	wxMessageDialog *ErrorMessageDialog;  //dialog to show InputErrorList
	wxControl *FirstControlError;  //first control of SMRDialog with incorrect data 

 public:
    SMRDialog(formality_type formality, structure_type structure,
              wxWindow* parent, const wxWindowID id, 
		      const wxString& title = "", const wxPoint& pos = wxPoint(-1,-1),
              const wxSize& size = wxSize(-1,-1), 
			  const long windowStyle = wxDEFAULT_DIALOG_STYLE,
              const wxString& name = "dialogBox");
	~SMRDialog();
    
    wxNotebook* Getnotebook() { return notebook;};
	wxPanel* Getpanel() { return panel;};

    bool GetReadOnly() { return ReadOnly; };
	void SetReadOnly(bool readonly) { ReadOnly = readonly; };

    bool GetMultiPanel() { return MultiPanel; };
	void SetMultiPanel(bool MP) { MultiPanel = MP; };

	void OnOk(wxCommandEvent& event);
    //Call 'InputDataValid'; if there are incorrect data show error messages, otherwise close the dialog

	void OnCancel(wxCommandEvent& event);
    //Close the dialog

    void OnCloseWindow(wxCloseEvent& event);
    //Close the dialog

    void OnKey(wxKeyEvent& event);

	void SetReadOnly() { };
	//Disable controls

	void TransferDataFromObject() { };
  	//Show information of the associated object in the dialog

	void TransferDataToObject() { };
	//Save data of the dialog in the associated object

	virtual bool InputDataValid() = 0;
    //Check data insert by user: for each incorrect data add a message in InputErrorList.
	//Return 0 if there are incorrect data, 1 otherwise.

    DECLARE_EVENT_TABLE()
};

#define notebook_ID 1

#endif