/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include "wx/wx.h"
#include "wx/dialog.h"
#include "wx/resource.h"
#include "drw/frmanage.hpp"
#include "lioo.hpp"
#include "stdlib.h"
#include "stdio.h"
#include "wx/spinctrl.h"
#include "wx/layout.h"
#include "par/partitur.hpp"

#if defined (__WXGTK__)
#include "../wxr/xtransposition3.wxr"
#include "../wxr/xtransposition3.h"
#endif

#if defined(__WXMSW__)
#include "../wxr/transposition3.wxr"
#include "../wxr/transposition3.h"
#endif




#define MAX_TRASL 14

static ClassID clefvectorID[]={CL_CVIOLINO,
CL_CBASSO,
CL_CBARITONO,
CL_CTENORE,
CL_CCONTRALTO,
CL_CMEZZOSOPRANO,
CL_CSOPRANO };


class TransposeDlg :public wxDialog
{
protected:
   	wxCheckBox	*checkClef;
	wxCheckBox	*checkInterval;
	wxCheckBox	*checkTransl;
	
	wxCheckBox	*checkSharps;
	wxCheckBox	*checkAdjust;
	
	wxChoice	*choiceClef;
	wxChoice	*choiceInterval;
	wxChoice	*listInstr;
	wxChoice	*staffchoice;
	
	wxTextCtrl *inputPart;
	wxTextCtrl *inputMeasure;
	wxButton   *okButton;
	wxButton   *cancelButton;
	wxRadioBox *up_down_interval;
	wxSpinCtrl *transl_spin;
	int numParts;
	int *measuresPerPart;
	wxSpinCtrl *part_spin;
	int partnum;
	Partitura *ptrPart;
	
	
	
public:	
	TransposeDlg();
	~TransposeDlg();
	
	void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
	void OnChecked(wxCommandEvent& event);
	void OnPartSelected();
	bool Initialize(int,Partitura *pPart,int opt);
	
	int GetClef();
	int GetInterval();
	int GetTranslOffset();
	int GetPart();
	int GetUpDownInterval();
	wxString GetMeasures();
	bool CheckValues();
	
	bool GetSharpPreference();
	bool GetAdjustPreference();
	
	void SetNumStaff(int numPart);
	int GetStaffNum();
	
    
private:
    DECLARE_EVENT_TABLE()
};




BEGIN_EVENT_TABLE(TransposeDlg, wxDialog)
EVT_BUTTON(ID_OK, TransposeDlg::OnOk)
EVT_BUTTON(ID_CANCEL, TransposeDlg::OnCancel)
EVT_CHECKBOX(ID_CHECKCLEF, OnChecked)
EVT_CHECKBOX(ID_CHECKKEYSIGN, OnChecked)
EVT_CHECKBOX(ID_CHECKTRANSL, OnChecked)
EVT_CHOICE(ID_CHOICE6007, TransposeDlg::OnPartSelected)	
END_EVENT_TABLE()



TransposeDlg::TransposeDlg()
{
	
	checkClef			= NULL;
	checkInterval		= NULL;
	checkTransl			= NULL;
	choiceClef			= NULL;
	choiceInterval		= NULL;
	inputMeasure		= NULL;
	okButton			= NULL;
	cancelButton		= NULL;
	up_down_interval	= NULL;
	transl_spin			= NULL;
	
	checkSharps			= NULL;
	checkAdjust			= NULL;
	
	
	listInstr			= NULL;
	staffchoice			= NULL;
	
	
	
}

TransposeDlg::~TransposeDlg()
{
	if(checkClef !=NULL) 
		delete checkClef;
	if(checkInterval !=NULL) 
		delete checkInterval;
	if(checkTransl!=NULL) 
		delete checkTransl;
	if(choiceClef !=NULL) 
		delete choiceClef;
	if(choiceInterval !=NULL) 
		delete choiceInterval;
	if(inputMeasure !=NULL) 
		delete inputMeasure;
	if(okButton !=NULL) 
		delete okButton;
	if(cancelButton !=NULL) 
		delete cancelButton;
	if(up_down_interval !=NULL) 
		delete up_down_interval;
	if(transl_spin !=NULL) 
		delete transl_spin;
	
	if(listInstr != NULL)
		delete listInstr;
	
	if(checkSharps !=NULL) 
		delete checkSharps;
	if(checkAdjust !=NULL) 
		delete checkAdjust;
	
	if(measuresPerPart!=NULL)
		delete measuresPerPart;
	if(staffchoice!=NULL)
		delete staffchoice;
	
}


bool TransposeDlg::Initialize(int part, Partitura *pPart, int opt=1)
{
	wxString set;
	wxString tmp;
	
	int xpos=0, ypos=0;
	
	
	partnum=0;
	numParts=pPart->GetNumSpa();
	ptrPart=pPart;
	
	
	
	if(part!=0)
	{
		measuresPerPart=new int[1];
		measuresPerPart[0]=(int)(pPart->GetSpartNum(part-1)->GetNumeroBattute());
	}
	else
	{  
		measuresPerPart=new int[numParts];
		for(int i=0;i<numParts;i++)
			measuresPerPart[i]=(int)(pPart->GetSpartNum(i)->GetNumeroBattute());
	}
	
	wxResourceParseData(Transposition); 
	if (LoadFromResource(FrameManager::currFrame, "Transposition"))
	{
		
		checkClef				= (wxCheckBox *)wxFindWindowByName("checkclef", this);
		checkInterval			= (wxCheckBox *)wxFindWindowByName("checkkeysign", this);
		checkTransl			= (wxCheckBox *)wxFindWindowByName("checktransl", this);
		choiceClef			= (wxChoice *)wxFindWindowByName("choiceclef", this);
		choiceInterval		= (wxChoice *)wxFindWindowByName("choicekeysign", this);
		inputMeasure			= (wxTextCtrl *)wxFindWindowByName("inputmeasure", this);
		okButton				= (wxButton *)wxFindWindowByName("okbutton", this);
		cancelButton			= (wxButton *)wxFindWindowByName("cancelbutton", this);
		up_down_interval		= (wxRadioBox *)wxFindWindowByName("radioboxInterval", this);
		
		checkSharps			= (wxCheckBox *)wxFindWindowByName("checksharps", this);
		checkAdjust			= (wxCheckBox *)wxFindWindowByName("checkadjust", this);
		
		listInstr				= (wxChoice *)wxFindWindowByName("instrlist", this);
		staffchoice			= (wxChoice *)wxFindWindowByName("staffchoice", this);
	}
	else
	{
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
		return FALSE;
	}
	choiceClef->GetPosition(&xpos,&ypos);
	
#if defined (__WXGTK__)
	transl_spin = new wxSpinCtrl(this,-1,wxEmptyString, wxPoint(xpos,135),wxSize(60,25));
#endif
	
#if defined(__WXMSW__)
	transl_spin = new wxSpinCtrl(this,-1,wxEmptyString, wxPoint(xpos,178),wxSize(60,25));
#endif
	
	transl_spin->SetRange(-MAX_TRASL,MAX_TRASL);
	transl_spin->SetValue(0);
	
	
	listInstr->Append("Main Score");
	for(int i=0;i<numParts;i++)
	{
		tmp.sprintf("%s",pPart->GetSpartNum(i)->GetNome());
		if(tmp.IsEmpty())
		{
			tmp.Clear();
			tmp.sprintf("Part %d",i+1);
		}
		listInstr->Append(tmp);
	}
	
	SetTitle("Transposition");
	
	choiceClef->Append("Treble");
	choiceClef->Append("Bass");
	choiceClef->Append("Baritone");
	choiceClef->Append("Tenor");
	choiceClef->Append("Alto");
	choiceClef->Append("Mezzo Soprano");
	choiceClef->Append("Soprano");
	
	staffchoice->Enable(FALSE);
	
	choiceInterval->Enable(FALSE);
	up_down_interval->Enable(FALSE);
	choiceInterval->Append("0 = prime");
	choiceInterval->Append("1 = Minor second");
	choiceInterval->Append("2 = Major second");
	choiceInterval->Append("3 = Minor third");
	choiceInterval->Append("4 = Major third");
	choiceInterval->Append("5 = Fourth");
	choiceInterval->Append("6 = aug. fourth");
	choiceInterval->Append("7 = Fifth");
	choiceInterval->Append("8 = Minor sixth");
	choiceInterval->Append("9 = Major sixth");
	choiceInterval->Append("10 = Minor seventh");
	choiceInterval->Append("11 = Major seventh");
	choiceInterval->Append("12 = Octave");
	
	transl_spin->Enable(FALSE);
	
	checkSharps->Enable(FALSE);
	checkAdjust->Enable(FALSE);
	checkAdjust->Show(false);
	
	
	listInstr->SetSelection(part);
	if (part!=0) 
	{
		listInstr->Enable(FALSE);
		SetNumStaff(part);
	}
	
	inputMeasure->SetValue("0");
	
	switch(opt)
	{
	case 1:
		checkClef->SetValue(TRUE);
		choiceClef->SetSelection(0);
		break;
	case 2:
		checkInterval->SetValue(TRUE);
		break;
	case 3:
		checkTransl->SetValue(TRUE);
		break;
	}
	
	return TRUE;
}

void TransposeDlg::OnPartSelected()
{
	int numPart=listInstr->GetSelection();
	if(numPart>0)
	{
		SetNumStaff(numPart);
	}
}

void TransposeDlg::SetNumStaff(int numPart)
{
	Spartito *pSpa=ptrPart->GetSpartNum(numPart-1);
	short numstaff=pSpa->GetNumberOfStaff();
	if(numstaff>1)
		staffchoice->Enable(TRUE);
	
	staffchoice->Clear();
	wxString tmp;
	staffchoice->Append("All");
	for(int i=0; i<numstaff; i++)
	{
		tmp.Clear();
		tmp.sprintf("%d",i+1);
		staffchoice->Append(tmp);
	}
	staffchoice->SetSelection(0);
}

int TransposeDlg::GetStaffNum()
{
	if(staffchoice->IsEnabled())
		return staffchoice->GetSelection();
	return (0);
	
}

void TransposeDlg::OnOk(wxCommandEvent& event)
{
	
	if (CheckValues())
		EndModal(wxID_OK);
}

void TransposeDlg::OnCancel(wxCommandEvent& event)
{
	EndModal(wxID_CANCEL);
}


void TransposeDlg::OnChecked(wxCommandEvent& event)
{
	switch(event.GetId())
	{
	case ID_CHECKCLEF:
								if (checkClef->GetValue())
								{
									choiceClef->Enable(TRUE);
									choiceClef->SetSelection(0);
								}
								else
								{
									choiceClef->Enable(FALSE);
									choiceClef->SetSelection(-1);
								}
								break;
	case ID_CHECKKEYSIGN:
								if (checkInterval->GetValue())
								{
									choiceInterval->Enable(TRUE);
									up_down_interval->Enable(TRUE);
									choiceInterval->SetSelection(0);
									
									checkSharps->Enable(TRUE);
									checkAdjust->Enable(TRUE);
								}
								else
								{
									choiceInterval->Enable(FALSE);
									up_down_interval->Enable(FALSE);
									choiceInterval->SetSelection(-1);
									
									checkSharps->Enable(FALSE);
									checkAdjust->Enable(FALSE);
								}
								break;
	case ID_CHECKTRANSL:
								if (checkTransl->GetValue())
								{
									transl_spin->Enable(TRUE);
									
								}
								else
								{
									transl_spin->Enable(FALSE);
									
								}
								break;
	default:
								wxMessageBox("Checking Error!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
								break;
	}
}	

//Restituisce l'ID relativo alla Chiave selezionata. Di default restituisce -1
int TransposeDlg::GetClef()
{
	if(choiceClef->IsEnabled())
		return choiceClef->GetSelection();
	else return -1;
}

//Restituisce la selazione relativo all'intervallo di trasposizione selezionato. Di default restituisce -1
int TransposeDlg::GetInterval()
{
	if(choiceInterval->IsEnabled())
		return choiceInterval->GetSelection();
	else return -1;
}

//Restituisce up=0 o down=1. Di default restituisce -1
int TransposeDlg::GetUpDownInterval()
{
	if(up_down_interval->IsEnabled())
		return up_down_interval->GetSelection();
	else return -1;
}


//rivedere!!!!!!!!!!
int TransposeDlg::GetTranslOffset()
{
	if(transl_spin->IsEnabled())
		return transl_spin->GetValue();
	else return 0;
}

int TransposeDlg::GetPart()
{
	return listInstr->GetSelection();
}

wxString TransposeDlg::GetMeasures()
{
	return inputMeasure->GetValue();
}

bool TransposeDlg::GetSharpPreference()
{
	if(checkSharps->IsEnabled())
		return checkSharps->GetValue();
	else return FALSE;
}

bool TransposeDlg::GetAdjustPreference()
{
	if(checkAdjust->IsEnabled())
		return checkAdjust->GetValue();
	else return FALSE;
}

bool TransposeDlg::CheckValues()
{
	int from;
	int to;
	wxString inputMeasures;
	wxString token;
	
	if(!checkClef->GetValue() && !checkInterval->GetValue() && !checkTransl->GetValue())
	{
		wxMessageBox("No method selected!", "Error", wxOK |  wxCENTRE | wxICON_EXCLAMATION,
			this);
		return FALSE;
		
	}
	inputMeasures = inputMeasure->GetValue();
	
	if(inputMeasures.Find('-')!=-1)
	{	
		token=inputMeasures.BeforeFirst('-');
		if(!token.IsNumber() || token.IsEmpty())			
		{
			wxMessageBox("Invalid input!", "Error", wxOK |  wxCENTRE | wxICON_EXCLAMATION,
				this);
			return FALSE;
			
		}
		else
		{
			from=atoi((const char*) token.c_str());
			
			if (from>measuresPerPart[partnum])
			{
				wxString msg;
				msg.sprintf("Max value allowed = %d",measuresPerPart[partnum]);
				wxMessageBox(msg, "Out of range!", wxOK |  wxCENTRE | wxICON_EXCLAMATION,this);
				return FALSE;
			}
		}
		token=inputMeasures.AfterFirst('-');
		if(!token.IsNumber() || token.IsEmpty())			
		{
			wxMessageBox("Invalid input!", "Error", wxOK |  wxCENTRE | wxICON_EXCLAMATION,
				this);
			return FALSE;
			
		}
		else
		{
			to=atoi((const char*) token.c_str());
			
			if (to>measuresPerPart[partnum])
			{
				wxString msg;
				msg.sprintf("Max value allowed = %d",measuresPerPart[partnum]);
				wxMessageBox(msg, "Out of range!", wxOK |  wxCENTRE | wxICON_EXCLAMATION,this);
				return FALSE;
			}
		}
	}
	else
	{
		token=inputMeasures;
		if (!token.IsNumber())
		{
			wxMessageBox("Invalid input!", "Error", wxOK |  wxCENTRE | wxICON_EXCLAMATION,this);
			return FALSE;
			
		}
		else
		{
			from=atoi((const char*) token.c_str());
			if (from>measuresPerPart[partnum])
			{
				wxString msg;
				msg.sprintf("Max value allowed = %d",measuresPerPart[partnum]);
				wxMessageBox(msg, "Out of range!", wxOK |  wxCENTRE | wxICON_EXCLAMATION,this);
				return FALSE;
			}
		}
	}
	
	return TRUE;
}

/************Funzione di chiamata alla dialog**************************/



Bool TranspositionsDlg(int *clef,wxString *measures, int *translation, int *interval, int *up, int *part, int *staff, Partitura *pPart, Bool *sharps, Bool *adjust)

{
	
	wxString control;
	
	TransposeDlg dialog;
	
	//Il parametro in ingresso puo' essere usato per definire la parte di default
	dialog.Initialize(*part,pPart);
	
	dialog.Centre();
	if (dialog.ShowModal() == wxID_OK) 
	{
		*measures=dialog.GetMeasures();
		*clef=dialog.GetClef();
		*part=dialog.GetPart();
		*up=dialog.GetUpDownInterval();
		*translation=dialog.GetTranslOffset();
		*interval=dialog.GetInterval();
		*staff=dialog.GetStaffNum();
		*sharps=dialog.GetSharpPreference();
		*adjust=dialog.GetAdjustPreference();
		
		return TRUE;
	}
	else 
		return FALSE;
	
}