/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include "wx/wx.h"
#include "wx/dialog.h"
#include "wx/resource.h"
#include "drw/frmanage.hpp"
#include "lioo.hpp"
#include "par/partitur.hpp"
#include "drw/scorefrm.hpp"
#include "drw/musicedt.hpp" 



#if defined (__WXGTK__)
#include "../wxr/xviewlayerdlg.wxr"
#include "../wxr/xviewlayerdlg.h"
#endif

#if defined(__WXMSW__)
#include "../wxr/viewlayerdlg.wxr"
#include "../wxr/viewlayerdlg.h"

#endif


enum
{	
	L_BLU,
		L_ROSSO,
		L_GIALLO,
		L_VERDE,
		L_VERDEC,
		L_VERDEA,
		L_CELESTE,
		L_VIOLA,
		L_MAGENTA,
		L_ARANCIO,
		L_MARRONE,
		L_GIALLOC
};




class ViewLayerDlg :public wxDialog
{
protected:
   	
	wxListBox		*log;
	
	
	wxButton	*viewButton;
	wxButton	*resetButton;
	wxButton	*closeButton;
	
	Partitura   *partitura;
	Spartito	*currSpa;
	wxString	*options;
	wxTextCtrl  *color;
	
	
public:	
	ViewLayerDlg();
	//Costruttore
	
	~ViewLayerDlg();
	//Distruttore
	
	void OnView(wxCommandEvent& event);
	//Metodo per la gestione dell'evento di pressione del bottone view
	
	void OnReset(wxCommandEvent& event);
	//Metodo per la gestione dell'evento di pressione del bottone reset
	
    void OnClose(wxCommandEvent& event);
	//Metodo per la chiusura e la distruzione della finestra
	
	void OnSelect(wxCommandEvent& event);
	//Metodo per la visualizzazione del colore del layer selezionto nella listbox
	
	void SetPParam(Spartito *pSPa);
	
	void ResetColor();
	
private:
    DECLARE_EVENT_TABLE()
};


// GET cOLOR
Colore GetColor(int index);


BEGIN_EVENT_TABLE(ViewLayerDlg, wxDialog)
EVT_BUTTON(ID_BUTTONVIEW, ViewLayerDlg::OnView)
EVT_BUTTON(ID_BUTTONCLOSE, ViewLayerDlg::OnClose)
EVT_BUTTON(ID_BUTTONRESET, ViewLayerDlg::OnReset)
EVT_LISTBOX(ID_CHOICES, ViewLayerDlg::OnSelect)
EVT_LISTBOX_DCLICK(ID_CHOICES, ViewLayerDlg::OnView)
END_EVENT_TABLE()



ViewLayerDlg::ViewLayerDlg() 
{
	
	
	log					= NULL;
	
	viewButton			= NULL;
	closeButton			= NULL;
	resetButton			= NULL;
	
	wxResourceParseData(ViewLayer); 
	if (LoadFromResource(FrameManager::currFrame, "ViewLayer"))
	{
		
		log					= (wxListBox *)wxFindWindowByName("log", this);
		viewButton			= (wxButton *)wxFindWindowByName("viewbutton", this);
		closeButton			= (wxButton *)wxFindWindowByName("closebutton", this);
		resetButton			= (wxButton *)wxFindWindowByName("resetbutton", this);
		color					= (wxTextCtrl *)wxFindWindowByName("color", this);
		
	}
	else
	{
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
	}
	
	
	
	SetModal(FALSE);
	
	partitura=NULL;
	currSpa=NULL;
	options=NULL;
	color->SetCursor(wxNullCursor);
	color->Enable(true);
	
	
	
}

ViewLayerDlg::~ViewLayerDlg()
{
	
	if(log!=NULL) 
		delete log;
	
	if(viewButton !=NULL) 
		delete viewButton;
	if(closeButton !=NULL) 
		delete closeButton;
	if(resetButton !=NULL) 
		delete resetButton;
	if(options !=NULL)
		delete []options;
	
	
}




void ViewLayerDlg::OnView(wxCommandEvent& event)
{
	extern wxMusicEditor *MusicEditor;
	int layer=log->GetSelection();
	if(layer>=0)
	{
		currSpa->View_Layer(true,layer);
		MusicEditor->RedrawViews(0,true);
	}
}

void ViewLayerDlg::OnReset(wxCommandEvent& event)
{
	ResetColor();
}

void ViewLayerDlg::OnClose(wxCommandEvent& event)
{
	ResetColor();
	Destroy();
}

void ViewLayerDlg::ResetColor()
{
	extern wxMusicEditor *MusicEditor;
	for(int i=0;i<NUM_MAX_LAYER;i++)
	{
		currSpa->View_Layer(false,i);
	}
	MusicEditor->RedrawViews(0,true);
}

void ViewLayerDlg::SetPParam(Spartito *pSpa)
{
	log->Clear();
	
	for(int i=0; i<4*pSpa->GetNumberOfStaff(); i++)
	{
		log->Append(wxString::Format("Layer %d",i+1));
	}
	
	currSpa=pSpa;
}


void ViewLayerDlg::OnSelect(wxCommandEvent& event)
{
	int index = event.GetSelection();
	
	color->SetBackgroundColour(GetColor(index));
	color->Refresh();
}

Colore GetColor(int index)
{
	switch(index)
	{ 
	case L_BLU:
		return Colore(*wxBLUE);
        break;
		
	case L_ROSSO:
		return Colore(*wxRED);
        break;
		
	case L_GIALLO:
		return Colore(255,255,0);
		break;
		
	case L_VERDE:
		return Colore(0,206,103);
		break;
		
	case L_VERDEC:
		return Colore(*wxGREEN);
        break;
		
	case L_VERDEA:
		return Colore(0,187,187);
		break;
		
	case L_CELESTE:
		return Colore(*wxCYAN);
        break;
		
	case L_VIOLA:
		return Colore(128,0,255);
		break;
		
	case L_MAGENTA:
		return Colore(255,0,255);
		break;
		
	case L_ARANCIO:
		return Colore(255,128,0);
        break;
		
	case L_MARRONE:
		return Colore(128,64,0);
		break;
		
	case L_GIALLOC:
		return Colore(255,255,174);
		break;
	}
	
	return Colore(*wxRED);
}




/************Funzione di chiamata alla dialog**************************/



Bool ViewLayerDial(Spartito *pSPa)
{
	wxString control;
	
	ViewLayerDlg *dialog=new ViewLayerDlg;
	dialog->SetPParam(pSPa);
	
	dialog->Centre();
	if(dialog->Show(TRUE))
		return TRUE;
	else 
		return FALSE;
	
}