/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: dlioowin.cpp,v 1.1 2005/07/25 16:30:53 bellinpo Exp $ */


#include "gui/dlioowin.hpp"
#include "liooext.hpp"
#include "net/mamass.hpp"
#include "drw/musicedt.hpp" 

extern EventResult VisualizzaMetr(int Metr, int Vel, int rate);	
typedef void (*wxTimerCB)();

void AddTimeOut(int,wxTimerCB,void*);
void Call_Scrolling();
void Call_Scrolling_End();

void DliooWindow::WinInit(const Point &p, VUnit w, VUnit h,Partitura* score)
{
	width = w;
	height = h;
	SetColor((ColorIcon)white);
	SetPos(NULL,p);
	
	extern wxMusicEditor *MusicEditor;
	int PENTNO; 
	
	// imposta la posizione dei pentagrammi nella finestra.
	if(score==NULL)
	{
		PPartit = new LoMas;
		if(!PPartit)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","DliooWindow","WinInit()");
	}
	else
		PPartit=score;
	if (!PPartit->GetPrintingStatus())
		PPartit->SetMusicRect(class Rectangle(*this));
	
	
	
	if (!PPartit->GetPrintingStatus())
	{
		if (WinDliooHeight > 650)
			((ParteL*)PPartit)->SetNPentagrammi(5);
		else
			((ParteL*)PPartit)->SetNPentagrammi(4);
	}
	else
	{
		PENTNO=PPartit->GetPrintViewParams()->GetNumStaff(PPartit->GetCurrentPage());
		((ParteL*)PPartit)->SetNPentagrammi(PENTNO);
	}
	
	// disegna la finestra.
	DrawWindow();
	pino=false;
	
}


void DliooWindow::Resize(VUnit w, VUnit h)
{
	if(w!=width || h!=height)
	{
		LiooWindow::Resize(w,h);
		
		if (!PPartit->GetPrintingStatus())
		{
			if (WinDliooHeight > 650)
				((ParteL*)PPartit)->SetNPentagrammi(5);
			else
				((ParteL*)PPartit)->SetNPentagrammi(4);
		}
		else 
		{
			// quindi siamo in printview
			int PENTNO=PPartit->GetPrintViewParams()->GetNumStaff(PPartit->GetCurrentPage());
			((ParteL*)PPartit)->SetNPentagrammi(PENTNO);
		}
	}
}


void DliooWindow::TerminaImpaginazione()
{
	SetScrollingTime(TIME_SCROLLING_CALL_BACK);
	SetScrollingInc(SCR_INC);
	ModEsecuzione=FALSE;
}

void DliooWindow::StopScrolling()
{
    // Se  attiva la modalit scrolling globale riavvia lo scrolling.
    // Se siamo arrivati all'ultima pagina disattiva lo 
    // Scrollingr globale.
	
	Battuta *pBat;
	Spartito *pSp;
	NumCode np;
	
	
	DrawScrollingIndicator(white);
	ScrollingState=FALSE;
	if (ScrollingAllActive) 
	{
		if (GetImpaginazione())
		{
			pSp = ((LoMas*)PPartit)->GetFirstSpa();
			pBat = pSp->GetfirstBatPage();
			if (pBat!=NULL)
			{
				if (pBat->GetNumPagDLIOO()==0)
				{
					((LoMas*)PPartit)->ImpostaBattPage(pBat->GetNumericCode(), pSp->GetPagCounter());
					pBat->SetNumPagDLIOO(pSp->GetPagCounter());
					pSp->SetPagCounter(pSp->GetPagCounter()+1);
				}
			}
			
			pBat = pSp->GetfirstBatNextPage();
			if (pBat!=NULL)
			{
				if (pBat->GetNumPagDLIOO()==0)
				{
					((LoMas*)PPartit)->ImpostaBattPage(pBat->GetNumericCode(), pSp->GetPagCounter());
					pBat->SetNumPagDLIOO(pSp->GetPagCounter());
					pSp->SetPagCounter(pSp->GetPagCounter()+1);
				}
			}
			
			// richiedere un numero di battute pari a quello presente nella pagina attuale.
			if (PPartit->GetBeginImpag()) 
				((LoMas*)PPartit)->Req_NBat(20);
		}
		else
		{
			np = ((LoMas*)PPartit)->GetNumPagina();
			if (np == 0)
				np = 1;
			else
			{
				
				pSp = ((LoMas*)PPartit)->GetFirstSpa();
				pBat = pSp->GetBattNumProg(np);
				pBat= pSp->GetNextBat(pBat);
				if (pBat!=NULL)
				{
					while (pBat->GetNumPagDLIOO()==0)
					{
						pBat = pSp->GetNextBat(pBat);
						if (pBat==NULL) break;
					}
					if (pBat==NULL)
						np = ((LoMas*)PPartit)->GetNumPagina();
					else
						np = pBat->GetNumProgress();
				}
			}
			
			((LoMas*)PPartit)->SetNumPagina(np);
		}
		
		ScrollingAllActive = ScrollPage();
		if (!ScrollingAllActive)
		{
			TerminaImpaginazione();
		}
	}
}


void DliooWindow::IncScrollingPos()
{
    // in base al tipo di leggio incrementa la posizione di X o di Y
	ScrollingPosY += -ScrollingInc;
}

Bool DliooWindow::CheckScrollingEnd()
{
	
    // in base al tipo di leggio cambia il controllo 
    // per vedere se siamo arrivati alla fine
	return (GetScrollingPosY()>GetScrollingInc());
}


int  DliooWindow::GetScrollingWidth()
{
	return GetVU2Rgt();
}


int  DliooWindow::GetScrollingHeight()
{
	return GetScrollingInc();
}


void DliooWindow::DrawScrollingIndicator(int color)
{
	// disegna l'indicatore di battuta orizzontale o verticale
	// in base al tipo di leggio
	
	int x = GetScrollingPosX()+(GetAbsPos()).x;
	int y = GetScrollingPosY()+(GetAbsPos()).y;
	int step = ScrollingBarraWidth;
	
	
	while ((y-step) < (GetAbsPos()).y) step --;
	Colore oldcol = Scr->GetColor();
	Scr->SetColor((ColorIcon)color);
	Scr->Rect(class Rectangle(Point(x+1,y),Point(x+GetVU2Rgt()-1,y-step)));
	Scr->SetColor((ColorIcon)white);
	Scr->Rect(class Rectangle(Point(x+2,y-1),Point(x+GetVU2Rgt()-2,y-(step-1))),TRUE);  
	Scr->SetColor(oldcol);
}


void DliooWindow::ImpostaBattPage(NumCode nc, NumCode np)
{
	((LoMas*)PPartit)->ImpostaBattPage(nc, np);
}


void DliooWindow::StartScrolling()
{	
	InitScrolling();
	ScrollingState=TRUE;
	
	int ScrT = GetScrollingTime();
	
	SetInitialTime();
	AddTempoIdeale(ScrT);
	
	if (GetModEsecuzione()) AddTimeOut(ScrT, Call_Scrolling, (void *)NULL);
}


Bool DliooWindow::JumpStateEndBatt()
{
	Battuta *pbt;
	Spartito *psp;
	NumCode nb;
	
	
	psp = PPartit->GetFirstSpa();
	
	pbt = psp->GetBattNumProg(countBatt);
	if (pbt==NULL) return FALSE;
	
	return (pbt->GetAbsPos().y + pbt->GetVU2Up() + pbt->GetHeight() + pbt->GetHeight() >= GetScrollingPosY());
	
	
	
	// a pbt faccio puntare l'ultima battuta di 2 righe precedenti
	nb = countBatt - 1;
	pbt = psp->GetBattNumProg(nb);
	if (pbt==NULL) return FALSE;
	
	while (((pbt->GetBattuta()) != INIZIO_PENTAGRAMMA) && ((pbt->GetBattuta()) != INIZIO_PAGINA))
	{
		nb--;
		pbt = psp->GetBattNumProg(nb);
		if (pbt==NULL) return FALSE;
	}
	
	if (pbt->GetBattuta() == INIZIO_PAGINA)
		return TRUE;
	
	nb --;
	while (((pbt->GetBattuta()) != INIZIO_PENTAGRAMMA) && ((pbt->GetBattuta()) != INIZIO_PAGINA))
	{
		nb--;
		pbt = psp->GetBattNumProg(nb);
		if (pbt==NULL) return FALSE;
	}
	
	if (pbt->GetBattuta() == INIZIO_PAGINA)
		return TRUE;
	
	nb --;
	pbt = psp->GetBattNumProg(nb);
	if (pbt==NULL) return FALSE;
	
	return ((pbt->GetAbsPos().y + pbt->GetVU2Up()) >= GetScrollingPosY());
}

Bool DliooWindow::WaitStateEndBatt()
{
	Battuta *pbt;
	Spartito *psp;
	NumCode nb;
	
	psp = PPartit->GetFirstSpa();
	
	// a pbt faccio puntare l'ultima battuta di 1 riga precedente
	nb = countBatt - 1;
	pbt = psp->GetBattNumProg(nb);
	if (pbt==NULL) return FALSE;
	
	while (((pbt->GetBattuta()) != INIZIO_PENTAGRAMMA) && ((pbt->GetBattuta()) != INIZIO_PAGINA))
	{
		nb--;
		pbt = psp->GetBattNumProg(nb);
		if (pbt==NULL) return FALSE;
	}
	
	if (pbt->GetBattuta() == INIZIO_PAGINA)
		return TRUE;
	
	nb --;
	pbt = psp->GetBattNumProg(nb);
	if (pbt==NULL) return FALSE;
	
	
	return ((pbt->GetAbsPos().y + pbt->GetVU2Up()) >= GetScrollingPosY());
}

void DliooWindow::prova()
{
	if (Inizio>2)
	{ 
		// prende i valori assoluti della zona da copiare
		int starty =(GetAbsPos()).y;
		int startx = (GetAbsPos()).x;
#ifdef SCR_FINESTRE
		// realizza la copia con le finestre	
		Scr->CopyPixmap(GetCopiaDrawingArea(),
#else
			// realizza la copia con le pixmap
			Scr->CopyPixmap(Scr->GetLPixmapCopia(),
#endif
			
			// in entrambe i casi gli altri parametri della Copypixmap
			// sono le cordinate di inizio e fine e la posizione
			// inziale: si copia una fetta di ampiezza 
			// ScrollingIncWidth() e ampiezza ScrollingIncHeight()
			GetScrollingPosX()+startx,GetScrollingPosY()+starty,
			GetScrollingWidth(),GetScrollingHeight());
		
		// incrementa ScrollingPosX o ScrollingPosY 
		// che tengono conto della posizione dello scrolling
		IncScrollingPos();
		
		
		// se non ha finito di copiare tutta la pagina 
		// disegna la barra di delimitazione in nero
		// e reimposta il timeout
		// altrimenti azzera lo ScrollingState
		DrawScrollingIndicator();
		
	}
}
void DliooWindow::Draw_Barre_Scrolling(void *data)
{
	
    // Copia della finestra Copia nella pag. attuale
	
	int ScrT = GetScrollingTime();
	
	SetActualTime();
	
	if ((GetEsecuzione()) && (pino==true))
		CheckNpassi1();  
	
	if (CheckScrollingEnd())
	{
		
		if (GetWaitState()==0)
			if (WaitStateEndBatt())
				return;
			
			if (GetJumpState())
				if (JumpStateEndBatt())
					SetScrollingInc(0);
				
				if (GetModEsecuzione()) 
				{
					if ((ScrT + GetDiffTime()) < MIN_CALL_TIMER)
						AddTimeOut(MIN_CALL_TIMER,Call_Scrolling, data);
					else
						AddTimeOut(ScrT + GetDiffTime(),Call_Scrolling, data);
					
				}
	}
	else
	{
		StopScrolling();          
		return;
		
	}
	
	AddTempoIdeale(ScrT);
}

void DliooWindow::InitExec()
{
	fine=false;
	AttivaModEsecuzione();
	pino=true;
	InitExec1();
}

void DliooWindow::InitExec1()
{
	
	int Larghezza;
	Battuta *pbt;
	Bool fl;
	float MetrAtt;
	
	if (countBatt == BattInitPage) // inizio classico
	{
		Inizio=1;
		alt=0;
		rest=0;
		SetScrollingTime(TIME_SCROLLING_CALL_BACK);
		
		InitialBatt = INITIAL_NUM_BATT;
		Waiting = -1;
		
		long te = GetTimeExec(countBatt);
		te = ((te * 1000)/(1000 + VelocEsec));
		PPartit->SetTimeExec(countBatt, te);
		
		
		SetInitialTime();
		
		AttivaModEsecuzione(BattInitPage);
		
		Spartito *psp = PPartit->GetFirstSpa();
		
		pBattutaExec = psp->GetBattNum(countBatt - 1);
		NumHit = 0;
		DrawBarraPosX = 0;
		
		
		
		MetrAtt = float(60.0) * float(1000.0) * pBattutaExec->GetIntest()->GetPtrTemp()->GetNumeratore();
		MetrAtt /= pBattutaExec->GetIntest()->GetPtrTemp()->GetDenominatore();
		MetrAtt /= te;
		
		pbt = pBattutaExec;
		fl = TRUE;
		while (fl)
		{
			fl = (pbt->GetPMovimento()==NULL);
			if (!fl)
				fl = (pbt->GetPMovimento()->GetMetronomo()==NULL);
			if (fl)  
				pbt = PPartit->GetFirstSpa()->GetPrevBat(pbt);
		}
		
		MetrAtt /= pbt->GetPMovimento()->GetMetronomo()->GetValNota();
		Metr = (int)MetrAtt;
		VisualizzaMetr(Metr, GetVelocEsec()/10,0);
		
		
		if (GetModEsecuzione())
		{
			InitialBatt = 1;
			long tt = GetNextTimeExec();	 
		}
	}
	else
	{
		pBattutaExec = PPartit->GetFirstSpa()->GetBattNum(countBatt-1);
		if (pBattutaExec==NULL)
		{
			return;
		}
		NumHit = 0;
		
		SetInitialTime();
		SetActualTime();
		
		InitialBatt = -1;
		AttivaModEsecuzione(BattInitPage);
		
		Npassi = 10;
		NpassiTot = Npassi;
		
		Larghezza = (pBattutaExec->GetAbsPos().x*9/10) / Npassi;
		DrawBarraPosX = 0;
		
		SetScrollingInc(Larghezza);
		IsJump=TRUE;
		ScrollAll();
	}
	
}
void DliooWindow::CheckNpassi1()
{
	Npassi--;
	if ((Npassi<=0) && (fine==false))
		long te=GetNextTimeExec();
	
}
long DliooWindow::GetNextTimeExec()
{
	Battuta *pbt2, *pbt,*pbt3;
	int Larghezza;//, n;
	Bool fl;
	float MetrAtt;
	
	long te = GetTimeExec(countBatt);
	
	pbt2 = PPartit->GetFirstSpa()->GetBattNum(countBatt-2);
	
	
	pBattutaExec = PPartit->GetFirstSpa()->GetBattNum(countBatt-1);
	if (pBattutaExec==NULL) te = -1;
	
	if ((te < 1) && (fine==false)) 
	{
		// esecuzione terminata
		fine=true;   
		DisattivaModEsecuzione();
		((MaMass*)PPartit)->SendEndEsec();
		Message("Execution Completed");
		
		pMusicWin->CursorWait();
		
		PPartit->SetStatoEsecuzione(JUMP_ESECUZIONE);
		
		
		char* s;
		char nome[90];
		char buffer[90];
		strcpy(nome,PPartit->GetFileSaveName());
		s=strstr(nome,".mds");
		if (s!=NULL) (*(s))='\0';  
		s=strrchr(nome,'/');
		if (s!=NULL) (*(s+1))='\0';
		
		
		char filesPath[200];
		
		
		strcpy(filesPath,smrgetenv("SMR_FILES_PATH"));
		strcat(filesPath,PPartit->GetPathSaveName());
		
		if (WSaveETT("Save ETT",filesPath,buffer,nome,200))
		{
			
			strcat(filesPath,nome);
			PPartit->SaveETT(filesPath);
		}
		return te;
	}
	
	NumHit = 0;
	
	
	
	te = ((te * 1000)/(1000 + GetVelocEsec()));
	
	
	MetrAtt = float(60) * float(1000) * pBattutaExec->GetIntest()->GetPtrTemp()->GetNumeratore();
	MetrAtt /= pBattutaExec->GetIntest()->GetPtrTemp()->GetDenominatore();
	MetrAtt /= te;
	
	pbt = pBattutaExec;
	fl = TRUE;
	while (fl)
	{
		fl = (pbt->GetPMovimento()==NULL);
		if (!fl)
			fl = (pbt->GetPMovimento()->GetMetronomo()==NULL);
		if (fl)  
			pbt = PPartit->GetFirstSpa()->GetPrevBat(pbt);
	}
	
	MetrAtt /= pbt->GetPMovimento()->GetMetronomo()->GetValNota();
	Metr = (int)MetrAtt;
	VisualizzaMetr(Metr, GetVelocEsec()/10,0);
	
	
	PPartit->SetTimeExec(countBatt, te);
	
	SetScrollingTime(TIME_SCROLLING_CALL_BACK);
	SetScrollingInc(alt);
	pbt3 = PPartit->GetFirstSpa()->GetBattNum(countBatt);
	
	if (pbt3!=NULL)
	{
		if (pbt3->GetBattuta() == INIZIO_PAGINA)
			InizioPag=countBatt;   
		if (((pbt3->GetBattuta()) == INIZIO_PENTAGRAMMA) || ((pbt3->GetBattuta()) == INIZIO_PAGINA))  
		{
			Inizio++;
			if (Inizio>2)
			{
				int NumBat=1;
				pbt3= PPartit->GetFirstSpa()->GetBattNum(countBatt+NumBat);
				while ((pbt3!=NULL) && (((pbt3->GetBattuta()) != INIZIO_PENTAGRAMMA) &&
					((pbt3->GetBattuta()) != INIZIO_PAGINA)))
				{
					NumBat++;
					pbt3= PPartit->GetFirstSpa()->GetBattNum(countBatt+NumBat);
				}
				alt=pBattutaExec->GetHeight();
				alt =(int) (alt / (NumBat));
				rest=pBattutaExec->GetHeight()-(alt*(NumBat-2));
				SetScrollingInc(alt); 
			}
		}
	}
	if (PPartit->GetETTLenght()-5>countBatt)   
		prova();     
	Npassi = te / TIME_SCROLLING_CALL_BACK;
	
    // calcola la larghezza del prossimo tratto da coprire con lo scrolling
	if (pbt2!=NULL)
	{
		if ((pBattutaExec->GetBattuta()) == INIZIO_PENTAGRAMMA)
			Larghezza = ((GetAbsPos()).x + 
			GetWidth() - GetScrollingPosX());
		else if ((pbt2->GetBattuta()) == INIZIO_PENTAGRAMMA)
		{
			if (GetScrollingPosX() > ((GetAbsPos()).x + GetWidth())/2)
				Larghezza = (pbt2->GetWidth() + pbt2->GetAbsPos().x) +
				((GetAbsPos()).x + GetWidth() - GetScrollingPosX());
			else    
				Larghezza = (pbt2->GetWidth() + pbt2->GetAbsPos().x) - GetScrollingPosX();
		}
		else
			Larghezza = (pBattutaExec->GetAbsPos().x - GetScrollingPosX());
		
		Larghezza -= pbt2->GetWidth()/2;
		if (Larghezza<=0) 
			Larghezza = 0;
		else
		{
			while ( (Larghezza / Npassi) < 1)
				Npassi /= 2;
		}
		
		if (Npassi < 1) Npassi = 1;
		
		Larghezza = Larghezza / Npassi;
		SetScrollingInc(Larghezza);
		
	}
	
	NpassiTot = Npassi;
	
	
	
	SetActualTime();
	
	if (InitialBatt>-1) InitialBatt--;
	
	if (InitialBatt==0)
	{
		// se siamo all'inizio dell'esecuzione deve ancora partire lo scrolling
		
		ScrollAll();
		Scr->GSetMainDrawArea();
	}
	
	
	
	if (GetJumpState())
		IsJump = FALSE;
	
	
    // mando la temporizzazione ai leggii della rete
	PPartit->SincrBatt(countBatt);
	temporealePrec = temporeale;
	
	countBatt++;
	
	return te;
}
void DliooWindow::IncVelocEsec()
{
	
	if (NumHit>=2) return;
	
	if (GetScrollingTime()/2 < MIN_CALL_TIMER)
	{
		SetScrollingInc(GetScrollingInc()*2);
	}
	else
		SetScrollingTime(GetScrollingTime()/2);
    
	SetMetr(GetMetr()*2);
	
	NumHit++;
	VelocEsec += 100;  
	((MaMass*)PPartit)->SendVelocEsec(VelocEsec);
}

void DliooWindow::DecVelocEsec()
{
	
	if (NumHit<=-2) return;
	
	if (VelocEsec > -900)
	{
		NumHit--;
		SetScrollingTime(GetScrollingTime()*2);
		SetMetr(GetMetr()/2);
		VelocEsec -= 100;
		((MaMass*)PPartit)->SendVelocEsec(VelocEsec);
	}
	
}


