/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#if defined (__WXGTK__)
#include "./wxr/editslurxdialog.wxr"
#include "./wxr/editslurxdialog.h"
#endif

#if defined (__WXMSW__)
#include "./wxr/editslurwdialog.wxr"
#include "./wxr/editslurwdialog.h"
#endif

#include "gui/editslurdlg.hpp"
#include "wx/wx.h"
#include "wx/resource.h"
#include "../bat/def_bat.hpp"
#include "../drw/frmanage.hpp"

#include "string.h"

#include "drw/musicedt.hpp"

extern wxMusicEditor *MusicEditor;


/**************************************************************/
BEGIN_EVENT_TABLE(EditSlurDlg, wxDialog)
EVT_BUTTON(ID_OK_SLUR, EditSlurDlg::OnOk)
EVT_BUTTON(ID_CANCE_LSLUR, EditSlurDlg::OnCancel)
EVT_BUTTON(ID_APPLY_SLUR, EditSlurDlg::OnApply)
END_EVENT_TABLE()

/**************************************************************/
EditSlurDlg::EditSlurDlg()
{
	stp1y = NULL;
	stp2y = NULL;
	leg=NULL;
	stbmp=NULL;
}


/**************************************************************/
EditSlurDlg::~EditSlurDlg()
{
	
	if(stp1y !=NULL)
		stp1y->Destroy();
	if(stp2y !=NULL)
		stp2y->Destroy();
	if(stbmp!=NULL)
		stbmp->Destroy();
	
}

/**************************************************************/
void EditSlurDlg::SetLegatura(Legatura *l)
{
	leg = l;	
}

/**************************************************************/
void EditSlurDlg::OnOk(wxCommandEvent& WXUNUSED(event) )
{
	tp1y=atof(stp1y->GetValue());
	tp2y=atof(stp2y->GetValue());
	Destroy();
}


/**************************************************************/
int EditSlurDlg::ShowingValues()
{
	
	wxString buffer;
	double lp1x=0.0, lp1y=0.0, lp2x=0.0, lp2y=0.0;
	
	
	if (leg!=NULL)
	{ 
		leg->GetKnotValue(&lp1x, &lp1y, &lp2x, &lp2y);
		// salvataggio valori iniziali
		initp1x=lp1x;
		initp1y=lp1y;
		initp2x=lp2x;
		initp2y=lp2y;
		
		if (lp1y<1000.0)
		{buffer.sprintf("%10f",lp1y);
        stp1y->SetValue(buffer);
		}
		if (lp2y<1000.0)
		{buffer.sprintf("%10f",lp2y);
        stp2y->SetValue(buffer);
		}
		return FALSE;
	}
	
	return TRUE;
}


/**************************************************************/
void EditSlurDlg::OnCancel(wxCommandEvent& WXUNUSED(event) )
{
	leg->SetKnotValue(initp1x, initp1y, initp2x, initp2y);
	leg->AdjustVUFig();
	MusicEditor->RedrawViews(0,TRUE);
	
	Destroy();
}

/**************************************************************/
void EditSlurDlg::OnApply(wxCommandEvent& WXUNUSED(event) )
{
	tp1y=atof(stp1y->GetValue());
	tp2y=atof(stp2y->GetValue());
	
	if (leg!=NULL)
	{ leg->SetKnotValue(tp1x, tp1y, tp2x, tp2y);
    leg->AdjustVUFig();
	MusicEditor->RedrawViews(0,TRUE);
	}
	else Destroy();
}

/**************************************************************/
bool EditSlurDlg::InitDlg()
{
	
	wxResourceParseData(EditSlurDialog);
	if (LoadFromResource(FrameManager::currFrame, "EditSlurDialog"))
	{
		stp1y = (wxTextCtrl *)wxFindWindowByName("text_p1y", this);
		stp2y = (wxTextCtrl *)wxFindWindowByName("text_p2y", this);
		wxBitmap slurKnots("SLURKNOTS_BMP", wxBITMAP_TYPE_BMP_RESOURCE);
		stbmp=new wxStaticBitmap(this,-1,slurKnots,wxPoint(16,60),wxSize(265,80));
	}
	else
	{
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, FrameManager::currFrame);
		return FALSE;
	}
	
	ShowingValues();
	
	SetModal(FALSE);
	
	// PRENDO I VALORI INIZIALI QUANDO SONO IN EDIT.
	
	
	return TRUE;
}

/**************************************************************/
// MAI UTILIZZATA 
int EditSlurDlg::GetKnotValues(double * p1x, double * p1y, double * p2x, double * p2y)
{ // salvo i valori iniziali
	
	initp1y=*p1y;
	initp2y=*p2y;
	
	if (InitDlg())
	{ 
		*p1y=tp1y;
		*p2y=tp2y;
		
		return TRUE;
	}
	else return FALSE;
}



/**************************************************************/
