/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: errman.cpp,v 1.1 2005/07/25 16:30:54 bellinpo Exp $ */


#ifndef OPD
#include "net/ldmas.hpp"
#endif OPD
#include "gui/errman.hpp"
#include "net/def_net.hpp"

#include "smr/smrmain.hpp"  //per usare SMRApp::GetApp()

#include <string.h>
#include <stdio.h>

extern void Message(char *);
extern char Array[];


// Nomi dei livelli di errore
char *LevelName[] = {
	"Debug",
		"Information",
		"Warning",
		"Error",
		"Fault",
		"Critical"
};

// DEFINIZIONI RELATIVE ALLE AREE

// Nomi delle aree
char *ErrorAreaNames[] = {
	"generale",
		"musicale",
		"load & save",
		"rete",
		"PVM"
};

// Nomi dei file che definiscono gli errori di ogni area
char *ErrorAreaFiles[] = {
	"general.err",
		"musical.err",
		"loadsav.err",
		"net.err",
		"pvm.err"
};


ErrorManager::ErrorManager()
{
	ErrorArea area;
	short i;
	
	
    
	// Inizializzazione flag
	OnFile = FALSE;
	OnScreen = TRUE;
	// Inizializzazione puntatore all'istanza ldmas
	net=NULL; 
	
	// Inizializzazione matrice
	//  Serve per bloccare la scansione nel metodo Error
	for(area=0; area<NUM_AREE; area++)
	{
		for(i=0;i<MAX_ERROR_PER_AREA;i++)
		{
			ErrorMatr[area][i].Code = NULL;
			ErrorMatr[area][i].Descr = NULL;
		}
	}
    
	CurrentLevel = EL_INFO;
	LogFile = NULL;
}


void ErrorManager::ReadErrFiles()
{
	ErrorArea area;
	FILE *pFile;
	char *StrCode, *StrDescr;
	ErrorLevel *pLevel;
	short i, ItemsMatched, EndOfFile;
    
	// Lettura dai file
	area = 0;
	while(area<NUM_AREE)
	{
		pFile = fopen(ErrorAreaFiles[area], "r");
		if(pFile == NULL)
		{
			char *dirx;
			dirx = new char[200];
			if(smrgetenv("SMR_ERR_PATH")!=NULL)
				strcpy (dirx, smrgetenv("SMR_ERR_PATH"));	
			else
				strcpy (dirx, "");	
			char *filenameres;
			filenameres = new char[256];
			filenameres = strcat(dirx,ErrorAreaFiles[area]);
			pFile = fopen(filenameres, "r");
		} 
		if(pFile != NULL)	
		{
			i = 0;
			EndOfFile = 0;   
			while(!EndOfFile)
			{
				StrCode = new char[ERROR_CODE_LENGTH];
				pLevel = new ErrorLevel;
				StrDescr = new char[ERROR_DESCR_LENGTH];
				if(StrCode && pLevel && StrDescr)
				{
					// Allocazione effettuata
					ItemsMatched = fscanf(pFile,"%s %d %s", StrCode, pLevel, StrDescr);
					switch(ItemsMatched)
					{
					case 3: // Linea letta correttamente
						ErrorMatr[area][i].Code = StrCode;
						ErrorMatr[area][i].Level = *pLevel;
						ErrorMatr[area][i].Descr = StrDescr;
						i++;
						break;
					case EOF: // File terminato
						EndOfFile = 1;
						break;              
					default: // Errore nel matching
						exit(0);  
					}
				}
				else
				{
					exit(0);
				}
			}
			fclose(pFile);      
		}
		else     
		{
#ifndef OPD
			if (SMRApp::GetApp().IsInteractive())
				Message("Error files not found\n set SMR_ERR_PATH");
#endif
			exit(0);
		}
		area = area + 1;
	}
}


ErrorLevel ErrorManager::GetLevel(ErrorArea Area, char *ErrCode)
{
	int found, i,ec;
	
	// Ricerca il tipo di errore in base al codice.
	i = 0;
	found = 0;
	while(i<MAX_ERROR_PER_AREA && !found)  
	{
		if(ErrorMatr[Area][i].Code==NULL)
			break;
		
		if(strcmp(ErrorMatr[Area][i].Code, ErrCode)==0)
		{
			ec = i;
			found = 1;
		}
		else
			i++;
	}
	if(!found)
	{
		return EL_DEBUG;
	}
	
	return ErrorMatr[Area][ec].Level;
}

void ErrorManager::SetLevel(ErrorLevel ec)
{
	CurrentLevel = ec;
}

// metodo vecchio
void ErrorManager::Error(ErrorCode ec, char *w, char *c, void *)
{
}


void ErrorManager::Error(ErrorArea Area, char *ErrCode, char *w, char *c, void *)
{
	ErrorLevel el;
	int found, i,ec;
	
	// Ricerca il tipo di errore in base al codice.
	i = 0;
	found = 0;
	
	while(i<MAX_ERROR_PER_AREA && !found)  
	{
		if(ErrorMatr[Area][i].Code==NULL)
		{    	
			break;
        }
		
		if(strcmp(ErrorMatr[Area][i].Code, ErrCode)==0)
		{
			ec = i;
			found = 1;
			
		}
		else
			i++;
	}
	
	if(!found)
	{
#ifndef OPD
		if (net!=NULL)
		{
			// gestione del messaggio in rete
			RetCode infoCode;
			char Codice[DIM_COMMAND_STRING+1];
			char id[2];      
			id[0]=net->Getmy_ID()+1;
			id[1]='\0';
			
			MessageStrCat(id);
			infoCode = net->CodeCommand((short int)ERRORE,Codice);
			MessageStrCat(Codice);
			infoCode = net->CodeCommand((short int)OnFile,Codice);
			MessageStrCat(Codice);
			infoCode = net->CodeCommand((short int)OnScreen,Codice);
			MessageStrCat(Codice);
			
			MessageStrCat("Il tipo di errore specificato non e` stato riscontrato nei file ERR.\n");
			
			
			MessageStrCat("\n");
			MessageStrCat("codice descritto");
			MessageStrCat(": ");
			MessageStrCat(ErrCode);
			MessageStrCat("\nClasse : ");
			MessageStrCat(w);
			MessageStrCat("\nMetodo : ");
			MessageStrCat(c);
			
			Array[0]='\0';
			strcpy(Array,MessageString);
			// invio Errore  
			RetCode sendcode = net->SendMsg_ack(net->Gettid_MASAE());
			switch (sendcode) 
			{
			case OK_SEND:
				break;
			case NEGATIVE_ACK:
				break;
			default:
				break;
			}
			return; 
		}
#endif OPD    
		// adesso siamo su masae
		MessageStrCat("Il tipo di errore specificato non e` stato riscontrato nei file ERR.\n");
		
		MessageStrCat("\n");
		MessageStrCat("codice descritto");
		MessageStrCat(": ");
		MessageStrCat(ErrCode);
		MessageStrCat("\nClasse : ");
		MessageStrCat(w);
		MessageStrCat("\nMetodo : ");
		MessageStrCat(c);
		
#ifndef OPD
		Message(MessageString); 
#else
		//Inserire messaggio di MOOVI
#endif OPD     
		
		
		
		return;
	}
	
	// se quel tipo di errore e' stato trovato
	// vado avanti
	
	el = ErrorMatr[Area][ec].Level;
	if (el >= CurrentLevel)
	{   
		MessageString[0] = '\0';   
#ifndef OPD
		if (net!=NULL) // va inviato in rete
		{
			// Formato messaggio: id - ERROR - OnFile - OnScreen - Message 
			RetCode infoCode;
			char Codice[DIM_COMMAND_STRING+1];
			char id[2];      
			id[0]=net->Getmy_ID()+1;
			id[1]='\0';
			
			MessageStrCat(id);
			infoCode = net->CodeCommand((short int)ERRORE,Codice);
			MessageStrCat(Codice);
			infoCode = net->CodeCommand((short int)OnFile,Codice);
			MessageStrCat(Codice);
			infoCode = net->CodeCommand((short int)OnScreen,Codice);
			MessageStrCat(Codice);
			
			MessageStrCat("Area :");
			MessageStrCat(ErrorAreaNames[Area]);
			MessageStrCat("\n");
			MessageStrCat(LevelName[el]);
			MessageStrCat(": ");
			MessageStrCat(ErrorMatr[Area][ec].Descr);
			MessageStrCat("\nClasse : ");
			MessageStrCat(w);
			MessageStrCat("\nMetodo : ");
			MessageStrCat(c);
			
			Array[0]='\0';
			strcpy(Array,MessageString);
			// invio Errore  
			RetCode sendcode = net->SendMsg_ack(net->Gettid_MASAE());
			switch (sendcode) 
			{
			case OK_SEND:
				break;
			case NEGATIVE_ACK:
				// BO!!!!!
				break;
			default:
				// Per ora niente.
				break;
			}
			
			return;  
		}
#endif OPD
		
		MessageStrCat("Area ");
		MessageStrCat(ErrorAreaNames[Area]);
		MessageStrCat(":\n");
		MessageStrCat(LevelName[el]);
		MessageStrCat(": ");
		MessageStrCat(ErrorMatr[Area][ec].Descr);
		MessageStrCat("\nClasse : ");
		MessageStrCat(w);
		MessageStrCat("\nMetodo : ");
		MessageStrCat(c);
		
		if (OnScreen) 
		{
#ifndef OPD
			Message(MessageString);
#else
			//Inserire messaggio di MOOVI
#endif OPD     
		}
		
		
		if (OnFile)
		{
			if (!(LogFile = fopen(LOGFILE_NAME, "a")))
				Error(EC_OPENFILEWRITE, "ErrorManager::ErrorOnFile","Cannot open log file");
			else 
			{ 
				fprintf(LogFile, "--- MASAE ---\n%s\n",MessageString);      
				fclose(LogFile);
			} 
		} 
	}
	if (el >= EL_FAULT) exit(ec);
} // end Error


void ErrorManager::SetOnFile()
{
	OnFile = TRUE;
}

void ErrorManager::ResetOnFile()
{
	OnFile = FALSE;
}

void ErrorManager::SetOnScreen()
{
	OnScreen = TRUE;
}

void ErrorManager::ResetOnScreen()
{
	OnScreen = FALSE;
}

// Questi sotto per me non hanno ragione di esistere
void ErrorManager::ErrorOnScreen(void)
{
	if (LogFile) {
		fclose(LogFile);
		LogFile = NULL;
	}
}

void ErrorManager::ErrorOnFile(void)
{
	if (!LogFile) {
		if (!(LogFile = fopen(LOGFILE_NAME, "a")))
			Error(EC_OPENFILEWRITE, "ErrorManager::ErrorOnFile",
            "Cannot open log file");
	}
}


void ErrorManager::MessageStrCat(char *str)
{
	if((strlen(MessageString) + strlen(str)) < MESSAGE_STRING_LENGTH)
		strcat(MessageString, str);
}

#ifndef OPD
void ErrorManager::SetLdMas(LdMas* ld)
{
	net = ld;
}  
#endif
