/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: errman.hpp,v 1.1 2005/07/25 16:30:54 bellinpo Exp $ */


#ifndef ERRORMANAGER_HPP
#define ERRORMANAGER_HPP

#include <stdio.h>
#include <stdlib.h>

#define NUM_AREE 5
#define MAX_ERROR_PER_AREA 30
#define ERROR_CODE_LENGTH 50
#define ERROR_DESCR_LENGTH 100

typedef short ErrorArea;

#define GENERAL_ERR_AREA 0
#define MUSICAL_ERR_AREA 1
#define LOADSAVE_ERR_AREA 2
#define NET_ERR_AREA 3
#define PVM_ERR_AREA 4


// N.B. All'interno dei file *.err queste costanti non vengono usate!
enum ErrorLevel {
	EL_DEBUG  = 0,   // livello per i messaggi di debug
		EL_INFO   = 1,   // informazioni generali per l'utente
		EL_WARNING  = 2, // warning
		EL_ERROR  = 3,   // errori recuperabili
		EL_FAULT  = 4,   // errori non recuperabili
		EL_CRITICAL = 5  // livello per i messaggi critici
};

typedef struct{
	char *Code;
	ErrorLevel Level;
	char *Descr;
} ErrorData;


#define MESSAGE_STRING_LENGTH 200

// tenuta per compatibilita con il codice esistente
enum ErrorCode {
	EC_INVALID        = 0,
		EC_MEMORYFAULT    = 1,
		EC_EOF            = 2,
		EC_WARNING        = 3,
		EC_OPENFILEWRITE  = 4,
		EC_OPENFILEREAD   = 5,
		EC_MOUSENOTPRESENT= 6,
		EC_FONTSNOTPRESENT= 7,
		EC_NOTIMPLEMENTED = 8,
		EC_DEBUG          = 9,
};


extern char *LevelName[];
extern ErrorLevel CodeLevel[];
extern char *CodeName[];

#ifndef Bool
#define Bool int
#define TRUE  1
#define FALSE 0
#endif

class LdMas;
class ErrorManager
{
private:
    LdMas* net;
    Bool OnFile; // per default  FALSE
    Bool OnScreen; //per default  TRUE
protected:
    ErrorData ErrorMatr[NUM_AREE][MAX_ERROR_PER_AREA];
	
    ErrorLevel CurrentLevel;
    FILE *LogFile;
    char MessageString[MESSAGE_STRING_LENGTH];
    void MessageStrCat(char *);
	
public:
	
    ErrorManager(void);
    void ReadErrFiles(void);
	
    ErrorLevel GetLevel(ErrorArea Area, char *ErrCode);
    void SetLevel(ErrorLevel);
    void SetOnFile();
    void ResetOnFile();
    void SetOnScreen();
    void ResetOnScreen(); 
    virtual void  Error(ErrorCode, char *, char *, void *wptr=NULL);
	// Tenuta per compatibilita` con il codice esistente
    virtual void  Error(ErrorArea, char *, char *, char *, void *wptr=NULL);
    void SetLdMas(LdMas*); 
    void ErrorOnScreen(void);
    void ErrorOnFile(void);
};

#endif // ERRORMANAGER_HXX

