/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: liooabil.cpp,v 1.1 2005/07/25 16:30:55 bellinpo Exp $ */


#include <string.h>

#include "liooabil.hpp"

static char *abilName[] =
{
	"FILE",
		"MODIFICA",
		"CANCELLA",
		"INTESTAZ",
		"BATTUTA",
		"PARTE",
		"MOVIMENTO",
		"LEGATURA",
		"FORCELLA",
		"OTTAVA", 
		"ONDA",
		"CAMBIORIT",
		"ACCENTI",
		"ABBELLIM",
		"TDINAMICO", 
		"TGENERICO",
		"ANNOTAZDIR",
		"CORONA",
		"SUDDIVIS",
		"DITEGGIATO",
		"SORDINA",
		"PRFIATO",
		"OCCHIALI",
		"VIOLINO",
		"TIMPANO",
		"OBOE",
		"SAX",
		"EDITEXEC"
};

void LiooAbil::setDefault()
{
	// azzera le abilitazioni temporanee
	for(int i=0; i<ABIL_LAST-1; i++)
		abils[i]=ABIL_S;
}

int LiooAbil::LoadFromString(char *str)
{
	char token[20],c;
	int p,tp,state=0,id,i,mayEnd;
	int tmpAbils[ABIL_LAST-1];
	
	// azzera le abilitazioni temporanee
	for(i=0; i<ABIL_TESTI_PRINC-1; i++)
		tmpAbils[i]=ABIL_N;
	
	tp=0;
	for(p=0; (c=str[p])!='\0'; p++)
    {
		// mette in token la stringa fino al prossimo separatore ':'
		// nello stato 0 (AbilID) e ' ' nello stato 1 (SV/SN/NO)
		mayEnd=0;
		if((state==0 && c!=':') || (state==1 && c!=';'))
			token[tp++]=c;
		else
        {
			token[tp]='\0';
			tp=0;
			if(state==0)
            {
				// cerca nella tabella abilName il token, per determinare l'id
				for(id=0;id<ABIL_TESTI_PRINC-1;id++)
					if(strcmp(token,abilName[id])==0)
						break;
					if(id==ABIL_TESTI_PRINC-1)
					{
						// nome non trovato
						return 0;
					}
					state=1; // ora cerca S, N, V
            }
			else
            {
				if(strcmp(token,"V")==0)
					tmpAbils[id]=ABIL_V;
				else if(strcmp(token,"N")==0)
					tmpAbils[id]=ABIL_N;
				else if(strcmp(token,"S")==0)
					tmpAbils[id]=ABIL_S;
				else
                {
					// valore non valido
					return 0;
                }
				state=0; // cerca un altro ID
				mayEnd=1;
            }
        }
    }
	
	if(!mayEnd)
		return 0;
	
	// assegna al vettore le abilitazioni lette dalla stringa
	for(i=0; i<ABIL_LAST-1; i++)
		abils[i]=(AbilVal)tmpAbils[i];
	
	return 1;
}

int LiooAbil::SaveToString(char *buf,int maxLen)
{
	int n=0;
	char *name;
	char *str=buf;
	
	for(int i=0; i<ABIL_TESTI_PRINC-1; i++)
    {
		name=abilName[i];
		while(*name!='\0' && n<maxLen)
        {
			*(str++) = *(name++);
			n++;
        }
		if(n<maxLen)
        {
			*(str++)=':';
			n++;
        }
		if(n<maxLen)
        {
			switch(abils[i])
            {
            case ABIL_N:
				*(str++)='N';
				break;
            case ABIL_S:
				*(str++)='S';
				break;
			case ABIL_V:
				*(str++)='V';
				break;
			} 	 
			n++;
        }
		if(n<maxLen)
        {
			*(str++)=';';
			n++;
        }  
    }
	if(n<maxLen)
    {
		*(str++)='\0';
		return 1;
    }
	return 0;
}

void LiooAbil::SaveToFile(FILE* fp)
{
	for(int i=0;i<ABIL_TESTI_PRINC-1; i++)
    {
		//fprintf(fp,"   abil_%s: %s\n",abilName[i],abils[i]?"SI":"NO");
		switch(abils[i])
		{
		case ABIL_N:
			fprintf(fp,"   abil_%s: %s\n",abilName[i],"NO");
			break;
		case ABIL_S:
			fprintf(fp,"   abil_%s: %s\n",abilName[i],"SI");
			break;
		case ABIL_V:
			fprintf(fp,"   abil_%s: %s\n",abilName[i],"VR");
			break; 
		};  
		
	}
}



