/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: liooabil.hpp,v 1.1 2005/07/25 16:30:55 bellinpo Exp $ */


#ifndef LIOO_ABIL_HPP
#define LIOO_ABIL_HPP

#include <stdio.h>

enum AbilID{
	ABIL = 0,
		ABIL_FILE,
		ABIL_MODIFICA,
		ABIL_CANCELLA,
		ABIL_INTESTAZ,
		ABIL_BATTUTA,
		ABIL_PARTE,
		ABIL_MOVIMENTO,
		ABIL_LEGATURA,
		ABIL_FORCELLA,
		ABIL_OTTAVA, 
		ABIL_ONDA,
		ABIL_CAMBIORIT,
		ABIL_ACCENTI,
		ABIL_ABBELLIM,
		ABIL_TDINAMICO, 
		ABIL_TGENERICO,
		ABIL_ANNOTAZDIR,
		ABIL_CORONA,
		ABIL_SUDDIVIS,
		ABIL_DITEGGIATO,
		ABIL_SORDINA,
		ABIL_PRFIATO,
		ABIL_OCCHIALI,
		ABIL_VIOLINO,
		ABIL_TIMPANO,
		ABIL_PEDALUP,
		ABIL_PEDALDOWN,
		ABIL_OBOE,
		ABIL_SAX,
		ABIL_SILLABA,
		ABIL_EDITEXEC,
		ABIL_KOREANORN,

		//  elementi inseriti per gestire i  "menu' padre" di quelli che si possono abilitare o no 
		ABIL_TESTI_PRINC,
		ABIL_ARTICOL_PRINC,
		ABIL_CHIAVI_PRINC,
		ABIL_STRUM_PRINC,
		ABIL_INTERV_PRINC,
		//
		ABIL_LAST 
} ; 

enum AbilVal 
{
	ABIL_N=0,
		ABIL_V=1,
		ABIL_S=2
};

#define MAX_STR_ABIL_LEN 287

class LiooAbil 
{
private:
	AbilVal abils[ABIL_LAST-1];
public:
	LiooAbil() { setDefault(); };
    // Costruttore che azzera tutte le abilitazioni
	
	AbilVal isAbil(AbilID id) 
    { if(id==ABIL) 
	return ABIL_S;
	else if (id==ABIL_LAST) 
		return ABIL_N;
	else
        return abils[id-1]; };
    // indica se una certa funzione e' abilitata o meno
	
	void setAbil(AbilID id, AbilVal v) 
    { abils[id-1]= v; };
    // imposta il valore della abilitazione di una certa funzione
	
	void setDefault();
    // imposta le abilitazioni di default (tutto disabilitato);
	
	int LoadFromString(char*);
    // carica le abilitazioni da una stringa formata da una sequenza:
    //  <idABIL>:[S|N|V];<idAbil>:[S|N|V];...
    // restituisce 1 se l'operazione ha successo
    // restituisce 0 se l'operazione non ha successo (la stringa non viene
    // riconosciuta), in questo caso le abilitazioni restano immutate.
	
	int SaveToString(char*, int maxLen);
    // memorizza nella stringa le abilitazioni, in modo da poter essere rilette
    // da LoadFromString.
    // Restituisce 1 se il buffer e' abbastanza grande da contenere la stringa.
    // altrimenti restituisce 0, e in questo caso il buffer e' riempito fino
    // al limite massimo, e' comunque presente il terminatore della stringa.
	
	void SaveToFile(FILE*);
    // Salva su file le abilitazioni in modo da poter essere rilette da once
};

#endif
