/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: lioowin.cpp,v 1.1 2005/07/25 16:30:56 bellinpo Exp $ */


#include <sys/types.h>
#include <sys/timeb.h>


#include "gui/lioowin.hpp"
void Call_Scrolling();
typedef void (*wxTimerCB)();
void AddTimeOut(int,wxTimerCB,void*);
void wxStartTimer();
long wxGetElapsedTime(bool);

#include <wx/progdlg.h>
#include <wx/thread.h>
#include "drw/frmanage.hpp"


LiooWindow::LiooWindow()
{
	
	IsJump = FALSE;
	JustResized = FALSE;
	PPartit = NULL;
	IsFormatted = FALSE;
	
	//*scrolling
#ifdef SCR_FINESTRE
	// Le macro SCR_FINESTRE_X e SCR_FINESTRE_Y sono definite
	// in lioo.hpp
	
	Copia = Scr->GNewWindow();
	if (!Copia)
		EM.Error(GENERAL_ERR_AREA,"EC_WINDOW_FAULT","LiooWindow","LiooWindow");
    // Crea la finestra Copia
	
	if ((tipoLeggio == LEGGIO_MASAE)  ||  (tipoLeggio == LEGGIO_MASE))
		CopiaDrawingArea = Scr->GNewDrawArea(-1, -1, WinMasaeHeight,WinMasaeWidth,Copia);
	else
		CopiaDrawingArea = Scr->GNewDrawArea(-1, -1, WinDliooHeight,WinDliooWidth,Copia);
	if (!CopiaDrawingArea)
		EM.Error(GENERAL_ERR_AREA,"EC_DRAWAREA_FAULT","LiooWindow","LiooWindow");
    //Crea la drawing area relativa alla finestra Copia
	
	SetBgColor(CopiaDrawingArea,white);
    //Imposta lo sfondo bianco
	
	ShowDisplay();
    // Visualizza la finestra 
	
	Disegno = Scr->GNewWindow();
	if (!Disegno)
		EM.Error(GENERAL_ERR_AREA,"EC_WINDOW_FAULT","LiooWindow","LiooWindow");
	
	if ((tipoLeggio == LEGGIO_MASAE)  ||  (tipoLeggio == LEGGIO_MASE))
		DisegnoDrawingArea = Scr->GNewDrawArea(-1, -1, WinMasaeHeight,WinMasaeWidth,Disegno);
	else
		DisegnoDrawingArea = Scr->GNewDrawArea(-1, -1, WinDliooHeight,WinDliooWidth,Disegno);
	if (!DisegnoDrawingArea)
		EM.Error(GENERAL_ERR_AREA,"EC_DRAWAREA_FAULT","LiooWindow","LiooWindow");
	
	SetBgColor(DisegnoDrawingArea,white);
	
	Scr->GSetMainDrawArea();
    // Imposta come finestra musicale attiva quella principale
#endif
	
	InitScrolling();
	Waiting = -1;
	
	SetScrollingInc(SCR_INC);
    // Setta l'ampiezza in px della porzione che verra' copiata durante
    // lo scrolling
	
	SetScrollingTime(TIME_SCROLLING_CALL_BACK);
    // Setta a l'intervallo del timer (in ms)
	
	ScrollingAllActive=FALSE;
    // di default lo scrolling avviene pagina per pagina
	
	ScrollingBloc=TRUE;
    // Inizialmente lo scrolling e' bloccato
	
	ModEsecuzione=FALSE;
	
	ScrollingBarraWidth = 20;
	ScrollingDir=FORWARD;
    // all'inizio la direzione di default e' in avanti
	
}


LiooWindow::~LiooWindow()
{
	if(PPartit!=NULL)
		delete PPartit;
}


void LiooWindow::KillLioo()
{
	if (PPartit!=NULL)
		PPartit->KillLioo();
}


void LiooWindow::SetImpaginazione(StateImpag imp)
{
	PPartit->SetImpaginazione(imp);
}


Bool LiooWindow::GetImpaginazione()
{ return PPartit->GetImpaginazione(); }

Bool LiooWindow::GetBeginImpag()
{ return PPartit->GetBeginImpag(); }

Bool LiooWindow::GetFirstEndBattImp()
{ return PPartit->GetFirstEndBattImp(); }

Bool LiooWindow::GetEsecuzione()
{ return PPartit->GetEsecuzione(); }

Bool LiooWindow::GetBeginEsec()
{ return PPartit->GetBeginEsec(); }


char LiooWindow::GetEtichetta(int n)
{
	if (PPartit!=NULL)
		return PPartit->GetEtichetta(n);
	else return -1;
}


void LiooWindow::WinInit(const Point &p, VUnit w, VUnit h,Partitura* score)
{
	// Obsoleto e ridefinito nelle classi figlie!
	
	width = w / Scr->GetMagnification();
	height = h;
	SetColor((ColorIcon)white);
	SetPos(NULL,p);
	
	if(score==NULL)
	{
		PPartit = new Partitura;
		if(!PPartit)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","LiooWindow","WinInit()"); 
	}
	else
		PPartit=score;
	if (!PPartit->GetPrintingStatus())
		PPartit->SetMusicRect(class Rectangle(*this));
}


void LiooWindow::Resize(VUnit w, VUnit h)
{
	w = w/Scr->GetMagnification();
	if((w!=width || h!=height) && PPartit!=NULL)
	{
		width = w;
		height = h; 
		
		// Reimposta la posizione dei pentagrammi nella finestra.
		// funzione del printing status setta l'area di disegno
		if (!PPartit->GetPrintingStatus())
			PPartit->SetMusicRect(class Rectangle(*this));
		
		JustResized = TRUE;
	}
}



void LiooWindow::GoForward()
{	
	if (PPartit->GetPrintingStatus())
	{
		PPartit->GoNextPage();
		PPartit->Redraw();
	}
	else
	{
		// in base alla modalita' di esecuzione avanza di una 
		// pagina oppure esegue lo scrolling in avanti o indietro 
		if (SimbImg)
		{
			if (GetModEsecuzione())
			{
				// se prima avevamo scrollato indietro ora si deve
				// preparare di nuovo le pixmap (o finestre) ausiliarie
				if (ScrollingDir==BACKWARD)
				{
					if (!ScrollingBloc) PPartit->GoNextPage();		
					ScrollingBloc=!(PPartit->GoNextPage());
					// se siamo in fondo alla partitura o in cima
					// si esegue un solo GoNextPage
					
					ScrollingDir=FORWARD;
				}
				ScrollPage();
			}
			else PPartit->GoForward();
		}
	}
}


void LiooWindow::GoBackward()
{
	if (PPartit->GetPrintingStatus())  
	{
		PPartit->GoPrevPage();
		PPartit->Redraw();
	}
	else
	{
		// in base alla modalita' di esecuzione torna indietro 
		// di una pagina oppure esegue lo scrolling in avanti o indietro 
		if (SimbImg)
		{
			if (GetModEsecuzione())
			{
				if (ScrollingDir==FORWARD)
				{	
					if (!ScrollingBloc) PPartit->GoPrevPage();		
					ScrollingBloc=!(PPartit->GoPrevPage());
					ScrollingDir=BACKWARD;
				}
				ScrollPage();
			}
			else PPartit->GoBackward();
		}
	}
}

void LiooWindow::GoTop() 
{
	if(SimbImg && PPartit!=NULL)
		PPartit->GoTop();
};

void LiooWindow::GoBottom() 
{ 
	if(SimbImg && PPartit!=NULL)
		PPartit->GoBottom();
};

void LiooWindow::NetPiu5()
{	
	if (PPartit->GetPrintingStatus())
	{
		PPartit->GoNextPage();
		PPartit->Redraw();
	}
	else
	{
		if (SimbImg) {if (PPartit!=NULL) PPartit->NetPiu5();}
	}
}

void LiooWindow::NetMeno5()
{
	if (PPartit->GetPrintingStatus())
	{
		PPartit->GoPrevPage();
		PPartit->Redraw();
	}
	else
	{
		if (SimbImg) {if (PPartit!=NULL) PPartit->NetMeno5();}
	}
}	 


void LiooWindow::EditingGoBottom()
{
	if (PPartit!=NULL) PPartit->EditingGoBottom();
}     


void LiooWindow::EditingJumpBatt(int nbat)
{
	if (PPartit!=NULL) PPartit->EditingJumpBatt(nbat);
}     


void LiooWindow::EditingGoTop()
{
	if (PPartit!=NULL) PPartit->EditingGoTop();
}     

void LiooWindow::RequestJump()
{
	if (SimbImg && PPartit!=NULL) PPartit->RequestJump();
}

void LiooWindow::GestLabel()
{
	if (PPartit!=NULL) PPartit->GestLabel();
}


char *LiooWindow::GetFileLoadName() 
{ 
	return PPartit->GetFileLoadName();
};

char *LiooWindow::GetPathLoadName() 
{ 
	return PPartit->GetPathLoadName();
};


void LiooWindow::SetCommand(CmdID cmdId, SelType stype, SelObj sobj, ArgType arg0,
							ArgType arg1, ArgType arg2, ArgType arg3, ArgType arg4, 
							char *txt, ArgType arg5, ArgType arg6, ArgType arg7,ArgType arg8)
{
	if (PPartit!=NULL)
	{
		if(SimbImg)
			PPartit->SetCommand(cmdId, stype, sobj, arg0, arg1, arg2, arg3, arg4, 
			txt, arg5, arg6, arg7, arg8);
	}
};


void LiooWindow::ResetCommand() 
{ 
	PPartit->ResetCommand();
};

SymCmd *LiooWindow::GetSymCmd() { return PPartit->GetSymCmd(); };



EventResult LiooWindow::LeftButtonPress(const Point& p)
{
	// se il punto non e' interno alla finestra ritorna NO_HIT
	if(SimbImg)
	{
		if(!Hit(p))
			return ER_NO_HIT;
		
		if (PPartit->DoCommand(p)) 
			return ER_CONTINUE;
		else 
			return ER_NO_HIT;
	}
	return ER_NO_HIT;
}


EventResult LiooWindow::LeftButtonRelease(const Point& p)
{ 
	if(!Hit(p))
		return ER_NO_HIT;
	
	if (SimbImg && PPartit->DoCommandRectSel(p)) 
		return ER_CONTINUE;
	else
		return ER_NO_HIT;
};


EventResult LiooWindow::MouseMove(const Point& from, const Point& to)
{
	if (SimbImg && PPartit && PPartit->GetShowTagli())
	{
		// se il punto p e' interno alla finestra allora disegna i tagli
		if(Hit(from))
			PPartit->DrawTagli(from);
		if(Hit(to))
			PPartit->DrawTagli(to);
	}
	return ER_NO_HIT;
}


void LiooWindow::BeginPaint()
{ 
	PPartit->BeginPaint(); 
}


void LiooWindow::EndPaint()
{ 
	PPartit->EndPaint(); 
}


void LiooWindow::DrawWindow()
{
}


//*scrolling
void LiooWindow::Draw() 
{
    if(JustResized)
    {
		JustResized = FALSE;
		if (PPartit && !PPartit->GetPrintingStatus())
			PPartit->SetMusicRect(class Rectangle(*this));
		Redraw();
		return;
	}
	if (SimbImg && PPartit)
		PPartit->Draw();
};


void LiooWindow::Redraw() 
{ 
	DimensioneX=GetWidth();
	DimensioneY=GetHeight(); 
	if (SimbImg && PPartit!=NULL) 
		PPartit->Redraw();  
};



void LiooWindow::ShowTagli()
{ 
	if (SimbImg && PPartit!=NULL)   
		PPartit->ShowTagli();
	
}

Bool LiooWindow::HideTagli()
{ 
	if (PPartit!=NULL) 
		return PPartit->HideTagli(); 
	else return FALSE;
}


void LiooWindow::ReadFromNetwork()
{
	PPartit->ReadFromNetwork();
}



void LiooWindow::DoCommand() 
{ 
	if(PPartit)
		PPartit->DoCommandImmediate(); 
};


#ifdef SCR_FINESTRE

Widget LiooWindow::GetCopiaDrawingArea()
{	
	return CopiaDrawingArea; 
}


Widget LiooWindow::GetDisegnoDrawingArea()
{	
	return DisegnoDrawingArea; 
}

void LiooWindow::SwapDrawingArea()
{	
	Widget tmp=CopiaDrawingArea;
	CopiaDrawingArea=DisegnoDrawingArea;
	DisegnoDrawingArea=tmp;
	
	tmp=Copia;
	Copia=Disegno;
	Disegno=tmp;
}

#endif //SCR_FINESTRE



void LiooWindow::InitScrolling()
{
	ScrollingState=FALSE;
	ScrollingPosX=0;
	ScrollingPosY=GetVU2Up();
}


Bool LiooWindow::GetScrollingState()
{
	return ScrollingState;
}


int LiooWindow::GetScrollingPosX()
{	
	return ScrollingPosX;
}


void LiooWindow::SetScrollingPosX(int px)
{	
	ScrollingPosX = px;
}



int LiooWindow::GetScrollingPosY()
{	
	return ScrollingPosY;
}

void LiooWindow::SetScrollingPosY(int px)
{	
	ScrollingPosY = px;
}



int LiooWindow::GetScrollingInc()
{
	return ScrollingInc;
}
void  LiooWindow::SetScrollingInc(int i)
{
	ScrollingInc = i;
}

int LiooWindow::GetScrollingTime()
{
	return ScrollingTime;
}


void LiooWindow::SetScrollingTime(int t)
{
	ScrollingTime=t;
}



void LiooWindow::TerminaImpaginazione()
{
}


void LiooWindow::ScrollAll()
{
	ScrollingAllActive=TRUE;
	
#ifdef DEBUG_EXEC
	FILE *pippo;
	pippo = fopen("/home/demo/txtA/DimasExec.txt","a");
	fprintf(pippo,"ScrollAll %d\n",GetJumpState());
	fclose(pippo);
#endif
	ScrollingState=FALSE;
	if (!ScrollPage())
	{
#ifdef DEBUG_EXEC
		pippo = fopen("/home/demo/txtA/DimasExec.txt","a");
		fprintf(pippo,"!ScrollPage \n");
		fclose(pippo);
#endif
		TerminaImpaginazione();
	}
	
}

Bool LiooWindow::ScrollPage()
{
	
#ifdef DEBUG_EXEC
	FILE *pippo;
	pippo = fopen("/home/demo/txtA/DimasExec.txt","a");
	fprintf(pippo,"ScrollPage 1\n");
	fclose(pippo);
#endif
	if ((!PPartit->IsLastPage()) || (GetJumpState()))
	{
		
#ifdef DEBUG_EXEC
		pippo = fopen("/home/demo/txtA/DimasExec.txt","a");
		fprintf(pippo,"ScrollPage 2 %d (TRUE = %d)\n",GetScrollingState(),TRUE);
		fclose(pippo);
#endif
		if (GetScrollingState()==FALSE)
		{
			// Scambia le finestre
#ifdef SCR_FINESTRE
			SwapDrawingArea();	
#else
			Scr->SwapPixmap();
#endif
			StartScrolling();
			
			if (ScrollingDir==FORWARD)
				ScrollingBloc = !(PPartit->GoNextPageInExecution());
			else
				ScrollingBloc = !(PPartit->GoPrevPage());
		}
		
		// Non siamo in fondo oppure  in corso uno scrolling
		return TRUE;  
	}
	
	return FALSE;	
}

void LiooWindow::ScrollAllImg()
{
	ScrollingAllActive=TRUE;
	
	
	
	ScrollingState=FALSE;
	if (!ScrollPageImg())
	{
	}
	
}


Bool LiooWindow::ScrollPageImg()
{
	
	
	
	
    if (GetScrollingState()==FALSE)
    {
		// Scambia le finestre
#ifdef SCR_FINESTRE
		SwapDrawingArea();	
#else
#endif
		StartScrolling();
		
    }
	
	return FALSE;	
}


Bool LiooWindow::GetModEsecuzione()
{   
	return ModEsecuzione; 
}


void LiooWindow::AttivaModEsecuzione(NumCode nb)
{	
	ModEsecuzione=TRUE; 
    // Attiva l'attributo flag
	
	ScrollingAllActive=FALSE;
    // Attiva l'attributo flag per lo scrolling globale
	
	
	ScrollingDir=FORWARD;
    // direzione iniziale in avanti
	
	Scr->GSetMainDrawArea();
	// va alla battuta nb
	if (nb>=0) 
	{	
		if(PPartit->GetBeginImpag())
			PPartit->GoBatt(nb);
		else
			PPartit->GoBattInExecution(nb);
		
		if(PPartit->GetFirstSpa()->IsVisible())
			ScrollingBloc = ((PPartit->GetFirstSpa()->GetfirstBatNextPage()) == NULL);
		else
			ScrollingBloc = ((PPartit->GetListaSpa()->GetFirstVisible()->GetfirstBatNextPage()) == NULL);//IB
		
	}
	
	Draw();
	
}

void LiooWindow::AttivaModEsecuzioneImg(int nb)
{	
	ModEsecuzione=TRUE; 
    // Attiva l'attributo flag
	
	ScrollingAllActive=FALSE;
    // Attiva l'attributo flag per lo scrolling globale
	
	
	ScrollingDir=FORWARD;
    // direzione iniziale in avanti
	
	Scr->GSetMainDrawArea();
	
}

void LiooWindow::DisattivaModEsecuzione()
{	
	ModEsecuzione=FALSE; 
    //Disattiva l'attributo flag ModEsecuzione
	
	ScrollingAllActive=FALSE;
    // Attiva l'attributo flag per lo scrolling globale
	
}




void LiooWindow::ForceFormattingForExecution()
{	
	IsFormatted=FALSE;
}



void LiooWindow::SetTimeExec(Battuta *pbt)
{
	PPartit->SetTimeExec(pbt);
}

long LiooWindow::GetTimeExec(int nc)
{
	return PPartit->GetTimeExec(nc);
}





void LiooWindow::InitEsecuzione(char* name)
{
	countBatt = 1;
	BattInitPage = 1;
	
	InitScrolling();
	
	PPartit->InitEsecuzione(name);
}

void LiooWindow::InitEsecuzioneImg(char* name)
{
	countBatt = 1;
	BattInitPage = 1;
	
	InitScrolling();
	
}

void LiooWindow::JumpToBatt(NumCode nb)
{
}


void LiooWindow::InitImpaginazione()
{
	PPartit->InitImpaginazione();
}







long LiooWindow::GetDiffTime()
{
	return (tempoideale-temporeale);
}


void LiooWindow::SetTempoIdeale(long temp)
{
	tempoideale = temp;
}

void LiooWindow::AddTempoIdeale(long temp)
{
	tempoideale += temp;
}

void LiooWindow::SetActualTime()
{
	temporeale=::wxGetElapsedTime(FALSE);
}


void LiooWindow::SetInitialTime()
{
	::wxStartTimer();
	temporeale = 0;
	tempoideale = 0;
}





void LiooWindow::SetVelocEsec(int v)
{
	VelocEsec = v;
}


int LiooWindow::GetVelocEsec()
{
	return VelocEsec;  
}


Bool LiooWindow::GetWaitState() 
{
	return Waiting;
}

Bool LiooWindow::SetWaitState(Bool b) 
{
	Waiting = b;
	int ScrT;
	if (Waiting==-1)
	{
		SetActualTime();
		
		tempoideale = temporeale;
		
		ScrT = GetScrollingTime();
		if (GetModEsecuzione()) 
			AddTimeOut(ScrT, Call_Scrolling, NULL);
	}
	return true;  
}



Bool LiooWindow::WaitStateEndBatt()
{
	return FALSE;
}


void LiooWindow::SetcountBatt(NumCode c)
{
	countBatt = c;
}


int LiooWindow::GetNumEtichette()
{
	return PPartit->GetNumEtichette();
}

int LiooWindow::GetNumeroBattute()
{
	return PPartit->GetNumeroBattute();
}

NumCode LiooWindow::GetBattEtichetta(char c)
{
	return PPartit->GetBattEtichetta(c);
}


Bool LiooWindow::GetJumpState()
{
	return IsJump;
}

void LiooWindow::SetJumpState(Bool b)
{
	IsJump = b;
}


void LiooWindow::GoBatt(NumCode np)
{
	PPartit->GoBatt(np);
}

void LiooWindow::GoBattIfOut(NumCode np)
{
	PPartit->GoBattIfOut(np);
	
}

Bool LiooWindow::SymbolicHit(SymPath& sp,SelObj sb,DrawObject*& pd)
{
	return PPartit->SymbolicHit(sp,sb,pd);
}

Bool LiooWindow::JumpStateEndBatt()
{
	return FALSE;
}

Bool LiooWindow::OkInitEsec()
{
	return PPartit->OkInitEsec();
}

Bool LiooWindow::OkEsecuzione()
{
	return PPartit->OkEsecuzione();
}

Bool LiooWindow::OkJump()
{
	return PPartit->OkJump();
}

Bool LiooWindow::OkCompila()
{
	return PPartit->OkCompila();
}


void LiooWindow::CursorWait()
{
}

void LiooWindow::CursorEditing()
{
}

void LiooWindow::CursorEsecuzione()
{
}

void LiooWindow::CursorGo()
{
}

StateExecution LiooWindow::GetStatoEsecuzione()
{
	return PPartit->GetStatoEsecuzione();
}

void LiooWindow::Draw_Barre_Scrolling(void *data)
{
}
void LiooWindow::Draw_Barre_ScrollingImg(void *data)
{
}

void LiooWindow::Draw_Barre_Scrolling_End(void *data)
{
}

void LiooWindow::JumpExecImg()
{
}

void LiooWindow::ScrollAllPages()
{
	wxProgressDialog formatting("Formatting...","Page 1",PPartit->GetNumeroBattute());
	int top=1,bottom=1;
	int nPage=1;
	PPartit->GoTop();
	PPartit->GetPageMeasures(top,bottom);
	formatting.Update(bottom,wxString::Format("Page %d",nPage++));
	while(PPartit->GoNextPage())
	{
		PPartit->GetPageMeasures(top,bottom);
		formatting.Update(bottom,wxString::Format("Page %d",nPage++));
	}
	IsFormatted=TRUE;
}
