/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: lioowin.hpp,v 1.1 2005/07/25 16:30:56 bellinpo Exp $ */



#ifndef LIOOWINDOW_HPP
#define LIOOWINDOW_HPP

#include <sys/types.h>
#include <sys/timeb.h>

#include "drw/lwindow.hpp"
#include "par/partitur.hpp"
#include "lioo.hpp"

#include "drw/gscreen.hpp"

// indica la direzione dello scrolling
enum Direzione {FORWARD,BACKWARD};


class LiooWindow : public LWindow
{
protected:
    Bool JustResized;
	// Indicatore di avvenuto resize della finestra.
public:
	Partitura *PPartit;
protected:
	// Puntatore alla partitura
	
	
#ifdef SCR_FINESTRE     
	Widget Copia;			
	// Finestra Copia
	
	Widget CopiaDrawingArea;		
	// Drawing Area della finestra Copia     
	
	Widget Disegno;			
	// Finestra Disegno
	Widget DisegnoDrawingArea;
#endif     
	
    Bool ScrollingState;		
	// Flag che indica se attivo lo scrolling
    int ScrollingPosX;			
	// Posizione dello scrolling X
    int ScrollingPosY;			
	// Posizione dello scrolling Y 
    int ScrollingInc;			
	// Porzione (in pixel) da copiare ogni volta  
    int ScrollingBarraWidth;
	// Porzione (in pixel) da copiare ogni volta  
    int ScrollingTime;			
	// Tempo fra una chiamata e l'altra     				
	
    Bool ScrollingAllActive;   
	// indica se attivo lo scrolling globale
	
    Bool  ModEsecuzione;
	// indica la modalita': editing o execution
	
    Bool ScrollingBloc;     
	// indica se siamo all'inizio o alla fine dello spartito
    Direzione ScrollingDir;
	// indica la direzione dello scrolling    
	
	
	// Attributi per il controllo della temporizzazione dello scrolling  
	
    NumCode countBatt;
    NumCode BattInitPage;
	
	// tempo impiegato realmente dallo scrolling
    long temporeale;
    long temporealePrec;
    
	// tempo ideale necessario allo scrolling
    long tempoideale;
    
	// fattore di correzione dei tempi di esecuzione 
	// delle battute. Indica l'incremento della velocita' 
	// di esecuzione
	// varia da - 1000 a + 1000
	// cioe' da -100.0% a + 100.0%
    int VelocEsec;
    
	// gestione del simbolo di corona in fase di esecuzione
    Bool Waiting;
	
    Bool IsJump;
    int JumpPos;
	
	Bool IsFormatted;
	
	
public:
    LiooWindow();
    ~LiooWindow();
    
	
	virtual void WinInit(const Point&,VUnit w,VUnit h,Partitura* symScore=NULL);
	// Inizializza la finestra ed alloca la partitura.
    virtual void Resize(VUnit w, VUnit h);
	// Ridimensiona la finestra, eventualmente predisponendo il ricalcolo
	// degli oggetti contenuti.
    void PiuPentagrammi();
	// incrementa il numero di pentagrammi visibili contemporaneamente.
    void MenoPentagrammi();
	// diminuisce il numero di pentagramma visibili contemporaneamente.
	
	
    virtual int IncImgVelocEsec( int ) { return 0; };
    virtual int DecImgVelocEsec( int ) { return 0; };
	virtual Bool SetWaitStateImg(Bool b) { return true; };
	
    void GoTop();
	// serve a posizionarsi sulla prima pagina.
    void GoBottom();
	// serve a posizionarsi sull'ultima pagina.
    void GoBatt(NumCode np);
	// serve a posizionarsi sulla battuta indicata
    void GoBattIfOut(NumCode np);
	// Serve a posizionarsi sulla battuta indicata solo se fuori pagina
    void GoForward();
	// scorre di una battuta indietro.
    void GoBackward();
	// scorre di una battuta in avanti.    
	
	void NetPiu5();
	void NetMeno5();
	
	
	void EditingGoBottom();     
	void EditingJumpBatt(int nbat);
	void EditingGoTop();
	void RequestJump();
    void GestLabel();     
    char *GetFileLoadName();
    char *GetPathLoadName();
	// ritorna il nome del file che e' stato caricato l'ultima volta.
	// ritorna "" se non e' stato caricato niente.
    void SetCommand(CmdID cmdId, SelType stype=NO_SEL, SelObj sobj=ANY_SEL, ArgType arg0=0,
		ArgType arg1=0, ArgType arg2=0, ArgType arg3=0, ArgType arg4=0, 
		char *txt=NULL, ArgType arg5=0, ArgType arg6=0, ArgType arg7=0,
		ArgType arg8=0);
	// permette la selezione degli oggetti di tipo SelObj con il tipo di
	// selezione SelType. Se viene premuto Ok nel menu' viene eseguito il 
	// comando CmdId sugli oggetti selezionati.
	// imposta gli argomenti del comando.
    void ResetCommand();
	// resetta lo stato in modo da non eseguire comandi. 
    void DoCommand();
	// esegue l'operazione indicata da cmd sugli oggetti selezionati
	// e' necessario per quei comandi che necessitano di una fase
	// preliminare di selezione degli oggetti su cui agire.
	// una volta eseguito il comando lo stato viene resettato.
    SymCmd *GetSymCmd();
    
    EventResult LeftButtonPress(const Point& p);
	// attivato quando viene premuto il tasto sinistro del mouse.
	// secondo il tipo di selezione impostato con SetCommand esegue
	// differenti operazioni.
	// ritorna TRUE se e' stato colpito qualcosa.
    EventResult LeftButtonRelease(const Point&);
    // attivato quando viene rilasciato il tasto sinistro del mouse.
	// ritorna TRUE se e' stato colpito qualcosa.
    EventResult MouseMove(const Point& from, const Point& to);
	// attivato quando il mouse si muove dalla posizione from a to.
	
    void BeginPaint();
	// toglie i tagli al mouse.
    void EndPaint();
	// rimette i tagli al mouse se erano richiesti.
	
    void DrawWindow();
	// disegna la finestra senza disegnare lo spartito.
    void Draw();
	// disegna la pagina attuale dello spartito.
    void Redraw();
	// ridisegna la pagina attuale dello spartito ricalcolando le posizioni.
	
	
	
    Bool SymbolicHit(SymPath&,SelObj,DrawObject*&);
	// Decodifica un path.
	
	
    void ShowTagli();
	// abilita il disegni dei Tagli delle note.
    Bool HideTagli();
	// disabilita il disegni dei Tagli delle note, ritorna TRUE se i tagli
	// erano attivi.
    
    void ReadFromNetwork();
	// Segnala alla partitura di consultare Network.
	
#ifdef SCR_FINESTRE
	Widget  GetCopiaDrawingArea();
	// ritorna la DrawingArea della finestra Copia
	Widget  GetDisegnoDrawingArea();  
	// ritorna la DrawingArea della finestra Disegno
	void SwapDrawingArea();
	// scambia le finestre Copia e Disegno
#endif
	
    virtual void StartScrolling() {};
	// attiva lo scrolling impostando il timer
    virtual void StopScrolling() {};
	// disattiva lo scrolling
	
    void InitScrolling();
	// inizializza gli attributi riferiti allo scrolling
    Bool GetScrollingState();
	// ritorna il valore di stato dello ScrollingState
    int GetScrollingPosX();
	// ritorna la posizione dell'ascissa dello scrolling
    void SetScrollingPosX(int);
	// imposta la posizione dell'ascissa dello scrolling
    int GetScrollingPosY();
	// ritorna la posizione dell'ordinata dello scrolling
    void SetScrollingPosY(int px);
	// imposta la posizione dell'ordinata dello scrolling
    int GetScrollingInc();
	// ritorna il valore di ScrollingInc
    void SetScrollingInc(int i);
	// imposta il valore di ScrollingInc
    virtual void IncScrollingPos() {};
	// incrementa il valore di ScrollingPos della quantita' ScrollingInc
    int GetScrollingTime();
	// ritorna il valore di ScrollingTime
    void SetScrollingTime(int t);
	// imposta il valore di ScrollingTime
    
    
    virtual Bool CheckScrollingEnd() {return FALSE;};
    
	// indica se siamo alla fine dello spartito
    virtual int GetScrollingWidth() {return 0;};
	// ritorna il valore dell'ampiezza della porzione 
	// che verra' copiata durante lo scrolling
    virtual int GetScrollingHeight() {return 0;};
	// ritorna il valore dell'altezza della porzione che 
	// verra' copiata durante lo scrolling
    
    virtual void DrawScrollingIndicator(int color = black) {};
	// disegna l'indicatore di battuta
    virtual void DrawScrollingIndicatorImg(int color = black) {};      
    void ScrollAll();
	// esegue lo scrolling di tutte le pagine in un passo solo
    Bool ScrollPage();
	// esegue lo scrolling di una pagina
    void ScrollAllPages();
	// executes the scrolling but without any visual effect.
    Bool GetModEsecuzione();
	// ritorna il valore di ModEsecuzione
    void AttivaModEsecuzione(NumCode nb = 0);
	// attiva la modalita' Execution (impostando l'attributo ModEsecution)
    void DisattivaModEsecuzione();
	// attiva la modalita' Editor (impostando l'attributo ModEsecution)
    void ForceFormattingForExecution();
	Bool GetIsFormatted() { return IsFormatted; };
	
	
	char GetEtichetta(int n);
	
	void KillLioo();
    void SetImpaginazione(StateImpag imp);
    Bool GetImpaginazione();
    Bool GetBeginImpag();
	Bool GetFirstEndBattImp();
	
	Bool GetEsecuzione();
	Bool GetBeginEsec();
	// richiamano gli omonimi metodi di Partitura.
	
	
	void SetTimeExec(Battuta *pbt);
	long GetTimeExec(int nc);
	
	virtual void ImpostaBattPage(NumCode nc, NumCode np) {};
	
	void InitEsecuzione(char*);
	void InitEsecuzioneImg(char*);
	virtual void InitImpaginazione();
	virtual void AltScrolling() {};
	virtual void JumpToBatt(NumCode);
	
	virtual long GetNpassi() {return 0;};
	virtual void CheckNpassi() {};
	virtual short GetCountBatt() {return 0;};
	virtual int GetInitialBatt() {return 0;};
	
	virtual long GetNextTimeExec() {return 0;};
	virtual void InitExec() {};
	virtual void InitExecImg() {};
	virtual void StartScrollingImg() {};
	virtual void AltScrollingImg() {};
	long GetDiffTime();
	void SetTempoIdeale(long temp);
	void AddTempoIdeale(long temp);
	// Imposta StartT 
	void SetInitialTime();
	// Imposta StopT e calcola temporeale
	void SetActualTime();
	virtual Bool IncVelocEsec(int) { return TRUE; };
	virtual Bool DecVelocEsec(int) { return TRUE; };
	virtual void SetNumHit( int ) { };
	virtual int  GetNumHit( ) { return 0; };
	
	void SetVelocEsec(int d);
	int GetVelocEsec();
	
	Bool GetWaitState();
	virtual Bool SetWaitState(Bool b);
	virtual Bool WaitStateEndBatt();
	
	void SetcountBatt(NumCode);
	int GetNumEtichette();
	int GetNumeroBattute();
	NumCode GetBattEtichetta(char);
	
	Bool GetJumpState();
	void SetJumpState(Bool);
	virtual int GetJumpPos() {return 0;};
	virtual Bool JumpStateEndBatt();
	
	virtual void TerminaImpaginazione();
	
	Bool OkInitEsec();
	Bool OkEsecuzione();
	Bool OkJump();
	Bool OkCompila();
	void AttivaModEsecuzioneImg(int nb);
	virtual void SetMetr(int n) {};
	virtual int GetMetr() {return 0;};
	virtual void CursorWait();
	virtual void CursorEditing();
	virtual void CursorEsecuzione();
	virtual void CursorGo();
	StateExecution GetStatoEsecuzione();
	virtual void Draw_Barre_Scrolling(void *data);
	virtual void Draw_Barre_ScrollingImg(void *data);
	virtual void Draw_Barre_Scrolling_End(void *data);
	Bool ScrollPageImg();
	void ScrollAllImg();
	virtual long GetNextTimeExecImg(){ return 0; };
	virtual void JumpExecImg();
	
	void ForceReposition() { JustResized=TRUE; };
};

#endif

