/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: masaewin.cpp,v 1.2 2005/07/28 09:52:28 bellinpo Exp $ */

#include "gui/masaewin.hpp"
#include "net/mamass.hpp"
#include "liooext.hpp"	
#include "imp/midiseq.h"
#include "drw/musicedt.hpp"
#include "smr/smr.hpp"

extern ScrList LiooListWidget;
extern EventResult VisualizzaMetr(int Metr, int Vel, int rate );

typedef void (*wxTimerCB)();

void AddTimeOut(int,wxTimerCB,void*);

extern wxMusicEditor *MusicEditor;
 
void Call_Scrolling()
{
	pMusicWin->Draw_Barre_Scrolling(NULL);
}

void Call_Scrolling_End()
{
	pMusicWin->Draw_Barre_Scrolling_End(NULL);
}


void MasaeWindow::WinInit(const Point &p, VUnit w, VUnit h,Partitura* score)
{
	width = w;
	height = h;
	SetColor((ColorIcon)white);
	SetPos(NULL,p);
	pBattutaExec = NULL;
	
    // imposta la posizione dei pentagrammi nella finestra.
	if(score==NULL)
	{
		PPartit = new MaMass;
		
		if(!PPartit)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT"," MasaeWindow","WinInit()");
	}
	else
		PPartit=score;
	
	PPartit->SetMusicRect((class Rectangle)(*this));
	
    // disegna la finestra.
	DrawWindow();
	NumPagina=0;
}


void MasaeWindow::TerminaImpaginazione()
{
	Spartito *psp;
	Battuta *pbat;
	
	SetScrollingTime(TIME_SCROLLING_CALL_BACK);
	SetScrollingInc(SCR_INC);
	ModEsecuzione=FALSE;
	
	if (GetImpaginazione())
	{
		
		psp = PPartit->GetFirstSpa();
		pbat = psp->GetBattNum(0);
		while (pbat != NULL)
		{
			SetTimeExec(pbat);
			pbat = psp->GetNextBat(pbat);
		}
		
		char* s;
		char nome[200];
		char filesPath[200];
		strcpy(nome,PPartit->GetFileSaveName());
		s=strstr(nome,".mds");
		if (s!=NULL)  {
			(*(s))='\0';  
			s=strrchr(nome,'/');
			if (s!=NULL) (*(s+1))='\0';
		} else {
			s=strstr(nome,".acl");
			if (s!=NULL) (*(s))='\0';  
		}
		
		
		strcpy(filesPath,smrgetenv("SMR_FILES_PATH"));
		strcat(filesPath,PPartit->GetPathSaveName());
		strcat(filesPath,nome);
		strcat(filesPath,".ett");
		PPartit->SaveETT(filesPath);
		
		
		((MaMass*)PPartit)->SendEndImp();
		
		GoTop();
		SetImpaginazione(END_IMP);
		DisattivaModEsecuzione();
		
		
		Message("End of Compiling.");
	}
	
}

void MasaeWindow::StopScrolling()
{
    // Se  attiva la mdalit scrolling globale riavvia lo scrolling.
    // Se siamo arrivati all'ultima pagina disattiva lo 
    // Scrolling globale.
    
	DrawScrollingIndicator(white);
	ScrollingState=FALSE;
	if (ScrollingAllActive) 
	{
		ScrollingAllActive=ScrollPage();
		if (!ScrollingAllActive)
		{
			if (GetEsecuzione())
			{
				if (GetModEsecuzione()) 
					AddTimeOut(MIN_CALL_TIMER, Call_Scrolling_End, (void *)NULL);
				return;
			}
			
			TerminaImpaginazione();
		}
	}
}

long MasaeWindow::GetNpassi()
{
	return Npassi;
}

void MasaeWindow::CheckNpassi()
{
	Npassi--;
	Terminata=true;
	
	if (Npassi<=0)
	{
		GetNextTimeExec();	
	}
}

void MasaeWindow::SetMetr(int n)
{
	Metr = n;
}

int MasaeWindow::GetMetr()
{
	return Metr;
}

short MasaeWindow::GetCountBatt()
{
	return countBatt;
}

int MasaeWindow::GetInitialBatt()
{
	return InitialBatt;
}

long MasaeWindow::GetNextTimeExec()
{
	Battuta *pbt2, *pbt;
	int Larghezza;
	Bool fl;
	float MetrAtt;
	Spartito *firstSpa=NULL;
	
	long te = GetTimeExec(countBatt);
	firstSpa = PPartit->GetFirstSpa();
	if(!firstSpa->IsVisible())
		firstSpa = PPartit->GetListaSpa()->GetFirstVisible();
	
	pbt2 = firstSpa->GetBattNum(countBatt-2);
	pBattutaExec = firstSpa->GetBattNum(countBatt-1);
	
	if (pBattutaExec==NULL) te = -1;
	
	if (te < 1) 
	{
		// esecuzione terminata
		DisattivaModEsecuzione();
		((MaMass*)PPartit)->SendEndEsec();
		Message("Execution ended");
		extern void StopExeStatus();
		StopExeStatus();
		
		pMusicWin->CursorWait();
		
		PPartit->SetStatoEsecuzione(JUMP_ESECUZIONE);
		
		if ( PPartit->GetMmExecMode() == MM_SYNC ) 
		{
			char nome[200];
			char buffer[200];
			char filesPath[400];
			
			strcpy(filesPath,smrgetenv("SMR_FILES_PATH"));
			strcat(filesPath,PPartit->GetPathSaveName());
			
			if(SimbImg)
				strcpy(nome,"sym-");
			else
				strcpy(nome,"img-");
			
			strcat(nome,PPartit->GetAudioFile());
			nome[strlen(nome)-4]='-'; // cambio .wav in -wav o .mp3 in -mp3
			
			strcat(nome,".ett");
			
			char *afile=(char*)PPartit->GetAudioFile();
			
			if (WSaveETT("Save ETT",afile,buffer,nome,199))
			{
				strcat(filesPath,nome);		  
				PPartit->SaveETT(filesPath);
				MusicEditor->smrf->AddSynchInfo(true,nome,afile); //true = simbolico
			}
		}
		return te;  
	}
	NumHit = 0;
	
	te = ((te * 1000)/(1000 + GetVelocEsec()));
	
	MetrAtt = float(60) * float(1000) * pBattutaExec->GetIntest()->GetPtrTemp()->GetNumeratore();
	MetrAtt /= pBattutaExec->GetIntest()->GetPtrTemp()->GetDenominatore();
	MetrAtt /= te;
	
	pbt = pBattutaExec;
	fl = TRUE;
	while (fl)
	{
		fl = (pbt->GetPMovimento()==NULL);
		if (!fl)
			fl = (pbt->GetPMovimento()->GetMetronomo()==NULL);
		if (fl)  
			pbt = firstSpa->GetPrevBat(pbt);
	}
	
	MetrAtt /= pbt->GetPMovimento()->GetMetronomo()->GetValNota();
	Metr = (int)MetrAtt;
	VisualizzaMetr(Metr, GetVelocEsec()/10, PPartit->GetAudioRate() );
	
	if (PPartit->Get_p_sequencer()!=NULL && (PPartit->Get_p_sequencer()->start_bat>1)&&
		(pBattutaExec->GetBattuta()==INIZIO_PENTAGRAMMA)&&
		(PPartit->Get_p_sequencer()->restarted==false))
		PPartit->Get_p_sequencer()->start_bat=1;
	
	Spartito* p_spart=firstSpa;
	DWORD prev_bat_start=0;
	
	if (countBatt>1) prev_bat_start=p_spart->
		GetBattNum(countBatt-2)->Get_start_ticks();
	
	if (PPartit->GetMmExecMode()==MM_MIDI)
	{
		DWORD ticks=pBattutaExec->Get_start_ticks()-prev_bat_start;
		PPartit->New_Midi_bat(ticks, te, Metr);
	}
	else 
		PPartit->SetTimeExec(countBatt, te);
	
	SetScrollingTime(TIME_SCROLLING_CALL_BACK);
	Npassi = te / TIME_SCROLLING_CALL_BACK;
	
    // calcola la larghezza del prossimo tratto da coprire con lo scrolling
	if (pbt2!=NULL)
	{
		if ((pBattutaExec->GetBattuta()) == INIZIO_PENTAGRAMMA)
			Larghezza = ((GetAbsPos()).x + 
			GetWidth() - GetScrollingPosX());
		else if ((pbt2->GetBattuta()) == INIZIO_PENTAGRAMMA)
		{
			if (GetScrollingPosX() > ((GetAbsPos()).x + GetWidth())/2)
				Larghezza = (pbt2->GetWidth() + pbt2->GetAbsPos().x) +
				((GetAbsPos()).x + GetWidth() - GetScrollingPosX());
			else    
				Larghezza = (pbt2->GetWidth() + pbt2->GetAbsPos().x) - GetScrollingPosX();
		}
		else
			Larghezza = (pBattutaExec->GetAbsPos().x - GetScrollingPosX());
		
		Larghezza -= pbt2->GetWidth()/2;
		if (Larghezza<=0) 
			Larghezza = 0;
		else if(Npassi>0)
		{
			while ( (Larghezza / Npassi) < 1)
				Npassi /= 2;
		}
		
		if (Npassi < 1) Npassi = 1;
		
		Larghezza = Larghezza / Npassi;
		SetScrollingInc(Larghezza);
		
	}
	if (Npassi < 1) Npassi = 1;
	
	NpassiTot = Npassi;
	
	SetActualTime();
	
	if (InitialBatt>-1) InitialBatt--;
	
	if (InitialBatt==0)
	{
		// se siamo all'inizio dell'esecuzione deve ancora partire lo scrolling
		
		ScrollAll();
		Scr->GSetMainDrawArea();
	}
	
	if (GetJumpState())
		IsJump = FALSE;
	
    // mando la temporizzazione ai leggii della rete
	PPartit->SincrBatt(countBatt);
	
	temporealePrec = temporeale;
	
	countBatt++;
	
	return te;
}


/*****************************************************************/
void MasaeWindow::InitExec()
{
	int Larghezza;//, n;
	Battuta *pbt;
	Bool fl;
	float MetrAtt;
	
	Spartito *firstSpa=NULL;
	firstSpa = PPartit->GetFirstSpa();
	if(!firstSpa->IsVisible())
		firstSpa = PPartit->GetListaSpa()->GetFirstVisible();
	
	if (countBatt == BattInitPage) // inizio classico
	{
		
		SetScrollingTime(TIME_SCROLLING_CALL_BACK);
		
		InitialBatt = INITIAL_NUM_BATT;
		Waiting = -1;
		
		long te = GetTimeExec(countBatt);
		te = ((te * 1000)/(1000 + VelocEsec));
		PPartit->SetTimeExec(countBatt, te);
		
		SetInitialTime();
		
		AttivaModEsecuzione(BattInitPage);
		
		Spartito *psp = firstSpa;
		
		pBattutaExec = psp->GetBattNum(countBatt - 1);
		NumHit = 0;
		DrawBarraPosX = 0;
		
		MetrAtt = float(60) * float(1000) * pBattutaExec->GetIntest()->GetPtrTemp()->GetNumeratore();
		MetrAtt /= pBattutaExec->GetIntest()->GetPtrTemp()->GetDenominatore();
		MetrAtt /= te;
		
		pbt = pBattutaExec;
		fl = TRUE;
		while (fl)
		{
			fl = (pbt->GetPMovimento()==NULL);
			if (!fl)
				fl = (pbt->GetPMovimento()->GetMetronomo()==NULL);
			if (fl)  
				pbt = firstSpa->GetPrevBat(pbt);
		}
		
		MetrAtt /= pbt->GetPMovimento()->GetMetronomo()->GetValNota();
		Metr = (int)MetrAtt;
		VisualizzaMetr(Metr, GetVelocEsec()/10, PPartit->GetAudioRate());
		
		if (GetModEsecuzione())
		{
			if (pBattutaExec->GetNumPagMASAE()!=0)
			{
				InitialBatt = 1;
				long tt = GetNextTimeExec();
			}
		}
	}
	else
	{
		pBattutaExec = firstSpa->GetBattNum(countBatt-1);
		if (pBattutaExec==NULL)
		{
			return;
		}
		NumHit = 0;
		
		SetInitialTime();
		SetActualTime();
		
		InitialBatt = -1;
		AttivaModEsecuzione(BattInitPage);
		
		Npassi = 10;
		NpassiTot = Npassi;
		
		Larghezza = (pBattutaExec->GetAbsPos().x*9/10) / Npassi;
		DrawBarraPosX = 0;
		
		SetScrollingInc(Larghezza);
		IsJump=TRUE;
		ScrollAll();
	}
	
}

/*****************************************************************/
void MasaeWindow::IncScrollingPos()
{
    // in base al tipo di leggio incrementa la posizione di X o di Y
	ScrollingPosX += ScrollingInc;
}

/*****************************************************************/
Bool MasaeWindow::CheckScrollingEnd()
{
    // in base al tipo di leggio cambia il controllo 
    // per vedere se siamo arrivati alla fine
	return (GetScrollingPosX()<(GetVU2Rgt()-GetScrollingInc()));
}

/*****************************************************************/
int MasaeWindow::GetScrollingWidth()
{
	return GetScrollingInc();
}

/*****************************************************************/
int MasaeWindow::GetScrollingHeight()
{
	return GetVU2Up();
}

/*****************************************************************/
void MasaeWindow::DrawScrollingIndicator(int color)
{
	int x, y, step;
	
	x = GetScrollingPosX()+(GetAbsPos()).x;
	y = GetScrollingPosY()+(GetAbsPos()).y;
	step = ScrollingBarraWidth;
	
	
	EXEC_MODE part_mode=PPartit->GetMmExecMode();
	
	// disegna l'indicatore di battuta orizzontale o verticale
	// in base al tipo di leggio
	if (InitialBatt < 0)
	{
		while ((x+step) > ((GetAbsPos()).x + GetWidth())) step --;
		Colore oldcol = Scr->GetColor();
		Scr->SetColor((ColorIcon)white);
		Scr->Rect(class Rectangle(Point(x,y-1),Point(x+step,y-GetVU2Up()+1)),TRUE);
		Scr->SetColor((ColorIcon)color);
		Scr->Line(Point(x+step/2,y-2),Point(x+step/2,y-GetVU2Up()+2),SOLID_LINE);
		Scr->SetColor(oldcol);
	}
		
	// disegna il punto di esecuzione
	if ( (pBattutaExec!=NULL) && (!IsJump) && (!GetImpaginazione()))
	{
		Colore oldcol = Scr->GetColor();
		Scr->SetColor((ColorIcon)15); //XOR_SELECTED_COLOR);
		Scr->SetDrwMode(XORMode);
		
		/*************MIDI**************************
		* Per rendere visibile la barra di scroll anche *
		* durante l'esecuzione midi  sufficiente _____ *
		* rimuovere la condizione part_mode!=MM_MIDI___ *
		* dai 2 if che seguono_________________________ *
		************************************************/
		
		if ((DrawBarraPosX > 0)&&(part_mode!=MM_MIDI)) 
		{
			Scr->Line(Point(DrawBarraPosX + 30,y-1),Point(DrawBarraPosX + 30,y-GetVU2Up()+1),SOLID_LINE);
			Scr->Rect(class Rectangle(Point(DrawBarraPosX,y-1),Point(DrawBarraPosX - 3,y-GetVU2Up()+1)),TRUE);
			Scr->Line(Point(DrawBarraPosX - 33,y-1),Point(DrawBarraPosX - 33,y-GetVU2Up()+1),SOLID_LINE);
		} 
		
		DrawBarraPosX = pBattutaExec->GetAbsPos().x + (NpassiTot - Npassi) * pBattutaExec->GetWidth() / NpassiTot;
		
		if(!PPartit->Get_p_sequencer())
		{
			Scr->Line(Point(DrawBarraPosX + 30,y-1),Point(DrawBarraPosX + 30,y-GetVU2Up()+1),SOLID_LINE);
			Scr->Rect(class Rectangle(Point(DrawBarraPosX,y-1),Point(DrawBarraPosX - 3,y-GetVU2Up()+1)),TRUE);
			Scr->Line(Point(DrawBarraPosX - 33,y-1),Point(DrawBarraPosX - 33,y-GetVU2Up()+1),SOLID_LINE);
		}
		
		Scr->SetDrwMode(DefMode);
		Scr->SetColor(oldcol);
	}  
}

/*****************************************************************/
void MasaeWindow::AltScrolling()
{
	DisattivaModEsecuzione();
	
	ScrollingPosX = GetVU2Rgt()-GetScrollingInc()+1;
	
	if (GetEsecuzione()) ((MaMass*)PPartit)->Send_AltExec();
	
	SetImpaginazione(END_IMP);
}


/*****************************************************************/
void MasaeWindow::StartScrolling()
{	
	InitScrolling();
	ScrollingState=TRUE;
	
	int ScrT = GetScrollingTime();
	
	if (!GetEsecuzione())
	{
		SetInitialTime();
	}
	
	AddTempoIdeale(ScrT);
	
	if (GetModEsecuzione()) Call_Scrolling();
}


/*****************************************************************/
int MasaeWindow::IncVelocEsec(int perc)
{
	EXEC_MODE exec_status=PPartit->GetMmExecMode();
	
	switch (exec_status)
	{
	case MM_SYNC :
		if (VelocEsec<900)  
		{   
			VelocEsec+=perc*10;// quel per dieci serve perche' in realta' albino ha usato il per mille e non il percento  
		}
		break;
		
	case MM_USER :
		VelocEsec+=perc*10;  // selmi : nella versione di Albino era 100 cioe' il 10%, ora e' 1%
		break;
		
	case MM_MIDI :
		VelocEsec+=perc;	
		break;			
	}
	return 0;
}


/*****************************************************************/
Bool MasaeWindow::DecVelocEsec(int perc)
{
	EXEC_MODE exec_status=PPartit->GetMmExecMode();
	
	switch (exec_status)
	{
	case MM_SYNC :
		if (VelocEsec<900)  
		{   
			VelocEsec-=perc*10;// quel per dieci serve perche' in realta' si era usato il per mille e non il percento  
		}
		break;
		
	case MM_USER :
		VelocEsec-=perc*10;  
		break;
		
	case MM_MIDI :
		if (1000+VelocEsec-perc>0) VelocEsec-=perc;	
		break;
	}
	
	return 0;
}


/*****************************************************************/
Bool MasaeWindow::WaitStateEndBatt()
{
	return (Npassi <= 5);
}

/*****************************************************************/
Bool MasaeWindow::SetWaitState(Bool b) 
{
	Waiting = b;
	int ScrT;
	if (Waiting==-1)
	{
		SetActualTime();
		
		if (temporeale - tempoideale < 0)
			PPartit->SetTimeExec(countBatt-1, 0);
		else
			PPartit->SetTimeExec(countBatt-1, temporeale - tempoideale);
		
		tempoideale = temporeale;
		
		ScrT = GetScrollingTime();
		if (GetModEsecuzione()) 
			AddTimeOut(ScrT, Call_Scrolling, (void *)NULL);
	}
	
	((MaMass*)PPartit)->SendWaitState(b);
	return TRUE;
}

/*****************************************************************/
void MasaeWindow::JumpToBatt(NumCode nb)
{
	countBatt = nb;
	PPartit->JumpToBatt(nb);
	BattInitPage = nb;
	
	
	InitScrolling();
	
	
	if (PPartit->GetFirstSpa()->GetBattNum(countBatt - 1)->GetNumPagMASAE()==0)
		PPartit->GoBattInExecution(nb);
	
	((MaMass*)PPartit)->SendVelocEsec(VelocEsec);
}

/*****************************************************************/
int MasaeWindow::GetJumpPos()
{
	return JumpPos;
}

/*****************************************************************/
void MasaeWindow::CursorWait()
{
	Scr->GSetCursor(WAIT, ROSSO);
    // mando in rete
	((MaMass*)PPartit)->SendTypeCursor(WAIT,ROSSO);
}

/*****************************************************************/
void MasaeWindow::CursorEditing()
{
	Scr->GSetCursor(DEFAULT, NERO);
    // mando in rete
	((MaMass*)PPartit)->SendTypeCursor(DEFAULT,NERO);
}

/*****************************************************************/
void MasaeWindow::CursorEsecuzione()
{
	Scr->GSetCursor(EXECUTION, BLU);
    // mando in rete
	((MaMass*)PPartit)->SendTypeCursor(EXECUTION, BLU);
}

/*****************************************************************/
void MasaeWindow::CursorGo()
{
	Scr->GSetCursor(AVANTI, NERO);
    // mando in rete
	((MaMass*)PPartit)->SendTypeCursor(AVANTI,NERO);
}


/*****************************************************************/
void MasaeWindow::Draw_Barre_Scrolling(void *data)
{
	int ScrT = GetScrollingTime();
	
	SetActualTime();
	
	if (InitialBatt < 0)
	{
		// Sostituisce la vecchia pagina con una nuova fetta della pagina successiva.
		// ******************************************************************************
		// ******************************************************************************
		
		// prende i valori assoluti della zona da copiare
		int starty =(GetAbsPos()).y;
		int startx = (GetAbsPos()).x;
		
		
#ifdef SCR_FINESTRE
		// realizza la copia con le finestre	
		Scr->CopyPixmap(GetCopiaDrawingArea(),
#else
			// realizza la copia con le pixmap
			Scr->CopyPixmap(NULL, //Scr->GetLPixmapCopia(),
#endif
			
			// in entrambe i casi gli altri parametri della Copypixmap
			// sono le cordinate di inizio e fine e la posizione
			// inziale: si copia una fetta di ampiezza 
			// ScrollingIncWidth() e ampiezza ScrollingIncHeight()
			GetScrollingPosX()+startx,GetScrollingPosY()+starty,
			GetScrollingWidth(),GetScrollingHeight());
		
		// incrementa ScrollingPosX o ScrollingPosY 
		// che tengono conto della posizione dello scrolling
		IncScrollingPos();
		// ******************************************************************************
		// ******************************************************************************
	}
	
	
	// se non ha finito di copiare tutta la pagina 
	// disegna la barra di delimitazione in nero
    // e reimposta il timeout
    // altrimenti azzera lo ScrollingState
	
	
	if (CheckScrollingEnd())
	{
		
		if (GetWaitState()==0)
			if (WaitStateEndBatt())
				return;
		if (GetEsecuzione())
			CheckNpassi();
		
		DrawScrollingIndicator();

		if (GetModEsecuzione()) 
		{
			if ((ScrT + GetDiffTime()) < MIN_CALL_TIMER)
				AddTimeOut(MIN_CALL_TIMER,Call_Scrolling, data);
			else
				AddTimeOut(ScrT + GetDiffTime(),Call_Scrolling, data);
		}
	}
	else
	{
		StopScrolling();
		return;
	}
	
	AddTempoIdeale(ScrT);
}

/*****************************************************************/
void MasaeWindow::Draw_Barre_Scrolling_End(void *data)
{
	int ScrT = GetScrollingTime();
	
	SetActualTime();
	
    if (GetWaitState()==0)
		if (WaitStateEndBatt())
			return;
		
	if (GetEsecuzione())
		CheckNpassi();
		
	DrawScrollingIndicator();
		
		
	if (GetModEsecuzione()) 
	{
		if ((ScrT + GetDiffTime()) < MIN_CALL_TIMER)
			AddTimeOut(MIN_CALL_TIMER,Call_Scrolling_End, data);
		else
			AddTimeOut(ScrT + GetDiffTime(),Call_Scrolling_End, data);
	}
		
	AddTempoIdeale(ScrT);
}

/*****************************************************************/
void MasaeWindow::InitImpaginazione()
{
	InitialBatt = -1;
	ScrollingPosX = 0;
	DrawBarraPosX = 0;
	PPartit->InitImpaginazione();
	
	Spartito* psp;
	Battuta* pbat;
	
	psp = PPartit->GetFirstSpa();
	pbat = psp->GetBattNum(0);
	while (pbat != NULL)
	{
		SetTimeExec(pbat);
		pbat = psp->GetNextBat(pbat);
	}
	
	char* s;
	char nome[200];
	char filesPath[200];
	strcpy(nome,PPartit->GetFileSaveName());
	s=strstr(nome,".mds");
	if (s!=NULL)  {
		(*(s))='\0';  
		s=strrchr(nome,'/');
		if (s!=NULL) (*(s+1))='\0';
	} else {
		s=strstr(nome,".acl");
		if (s!=NULL) (*(s))='\0';  
	}
	
	strcpy(filesPath,MusicEditor->smrf->GetPath()+PATH_SEP);
	strcat(filesPath,nome);
	strcat(filesPath,".ett");
	PPartit->SaveETT(filesPath);
	
}

/*****************************************************************/
void MasaeWindow::Fine_Esecuzione()
{
    DisattivaModEsecuzione();
    ((MaMass*)PPartit)->SendEndEsec();
    Message("Execution Comleted");
    pMusicWin->CursorWait();
    PPartit->SetStatoEsecuzione(JUMP_ESECUZIONE);
	extern void StopExeStatus();
	StopExeStatus();
}

/*****************************************************************/
int MasaeWindow::GetNumHit()
{
	return NumHit;
}

/*****************************************************************/
void MasaeWindow::SetNumHit( int number )
{
    NumHit = number;
}

/*****************************************************************/

void MasaeWindow::Draw()
{
	if (GetModEsecuzione())
	{
		if(JustResized)
		{
			JustResized = FALSE;
#ifdef SCR_FINESTRE
			Scr->GSetDrawArea(GetDisegnoDrawingArea());  
			Redraw();
			Scr->GSetMainDrawArea();
#else
			Scr->GSetPixmap(Scr->GetLPixmapDisegno());
			Redraw();
			Scr->GResetPixmap();
#endif			
			// ridisegna la finestra Disegno
			
		}
		else
		{
			int x, y, step;
			
			x = GetScrollingPosX()+(GetAbsPos()).x;
			y = GetScrollingPosY()+(GetAbsPos()).y;
			step = ScrollingBarraWidth;
			
			// Copia parte della finestra Copia nella parte sinistra della finestra princ.
			Scr->CopyPixmap(NULL,
				GetAbsPos().x,GetAbsPos().y+GetVU2Up(),
				GetScrollingPosX(),GetVU2Up());
			// Copia parte della finestra Current nella parte destra della finestra princ.
			Scr->CopyPixmap(Scr->GetLPixmapCurrent(),
				x,GetAbsPos().y+GetVU2Up(),
				GetVU2Rgt()-GetScrollingPosX(),GetVU2Up());
			
			
			// disegna l'indicatore di battuta verticale
			if (InitialBatt < 0)
			{
				while ((x+step) > ((GetAbsPos()).x + GetWidth()))
					step --;
				Colore oldcol = Scr->GetColor();
				Scr->SetColor((ColorIcon)white);
				Scr->Rect(class Rectangle(Point(x,y-1),Point(x+step,y-GetVU2Up()+1)),TRUE);
				Scr->SetColor((ColorIcon)black);
				Scr->Line(class Point(x+step/2,y-2),Point(x+step/2,y-GetVU2Up()+2),SOLID_LINE);
				Scr->SetColor(oldcol);
			}
			if ((DrawBarraPosX > 0)&&(PPartit->GetMmExecMode()!=MM_MIDI)) 
			{
				Colore oldcol = Scr->GetColor();
				Scr->SetColor((ColorIcon)15); //XOR_SELECTED_COLOR);
				Scr->SetDrwMode(XORMode);
				
				Scr->Line(Point(DrawBarraPosX + 30,y-1),Point(DrawBarraPosX + 30,y-GetVU2Up()+1),SOLID_LINE);
				Scr->Rect(class Rectangle(Point(DrawBarraPosX,y-1),Point(DrawBarraPosX - 3,y-GetVU2Up()+1)),TRUE);
				Scr->Line(Point(DrawBarraPosX - 33,y-1),Point(DrawBarraPosX - 33,y-GetVU2Up()+1),SOLID_LINE);
				Scr->SetDrwMode(DefMode);
				Scr->SetColor(oldcol);
			} 
			
		}
	}	
	else
		LiooWindow::Draw();
}