/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: masewin.cpp,v 1.1 2005/07/25 16:30:57 bellinpo Exp $ */


#include "gui/masewin.hpp"
#include "liooext.hpp"

typedef void (*wxTimerCB)();

void AddTimeOut(int,wxTimerCB,void*);
void Call_Scrolling();
void Call_Scrolling_End();

void Call_TimeExecution()
{
	long ScrT = pMusicWin->GetNextTimeExec();
	
	if (ScrT < 0)  // battute dello spartito terminate
		return;
	
	pMusicWin->AddTempoIdeale(ScrT);
	
	if (pMusicWin->GetInitialBatt()==0)   // fine della prima parte di scrolling
		return;
	
	if (pMusicWin->GetModEsecuzione()) 
		AddTimeOut(ScrT, Call_TimeExecution, (void *)NULL);
}

void MaseWindow::WinInit(const Point &p, VUnit w, VUnit h,Partitura* score)
{
	width = w;
	height = h;
	SetColor((ColorIcon)white);
	SetPos(NULL,p);
	
	// imposta la posizione dei pentagrammi nella finestra.
	if(score==NULL)
	{
		PPartit = new DiMas;
		if(!PPartit)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT"," MasaeWindow","WinInit()");
	}
	else
		PPartit=score;
	PPartit->SetMusicRect(class Rectangle(*this));
	
	// disegna la finestra.
	DrawWindow();
}


void MaseWindow::TerminaImpaginazione()
{
	if (GetImpaginazione())
	{
		((DiMas*)PPartit)->SetPage();
		((DiMas*)PPartit)->RemoveListaPag();
		((DiMas*)PPartit)->SendEndImp();
	}
	
	SetScrollingTime(TIME_SCROLLING_CALL_BACK);
	SetScrollingInc(SCR_INC);
	ModEsecuzione=FALSE;
	
}

void MaseWindow::StopScrolling()
{
	
    // Se  attiva la mdalit scrolling globale riavvia lo scrolling.
    // Se siamo arrivati all'ultima pagina disattiva lo 
    // Scrolling globale.
	Spartito *pSp;    
	Battuta* pBat;
	NumCode np;
	
	DrawScrollingIndicator(white);
	ScrollingState=FALSE;
	if (ScrollingAllActive) 
	{
		if (GetImpaginazione())
		{
			pSp = ((DiMas*)PPartit)->GetFirstSpa();
			pBat = pSp->GetfirstBatPage();
			if (pBat->GetNumPagMASE()==0)
			{
				((DiMas*)PPartit)->ImpostaBattPage(pBat->GetNumericCode(), pSp->GetPagCounter());
				
				
				pBat->SetNumPagMASE(pSp->GetPagCounter());
				pSp->SetPagCounter(pSp->GetPagCounter()+1);
			}
			
			// richiedere un numero di battute pari a quello presente nella pagina attuale.
			if (PPartit->GetBeginImpag()) 
				((DiMas*)PPartit)->Req_NBat(20);
			int i=pBat->GetNumProgress();
		}
		else
		{
			np = ((DiMas*)PPartit)->GetNumPagina();
			
			if (np == 0)
				np = 1;
			else
			{
				np++;
				
				pSp = ((DiMas*)PPartit)->GetFirstSpa();
				pBat = pSp->GetBattNumProg(np);
				if (pBat!=NULL)
				{
					while (pBat->GetNumPagMASE()==0)
					{
						np++;
						pBat = pSp->GetNextBat(pBat);
						if (pBat==NULL) break;
					}
				}
				if (pBat==NULL) np = ((DiMas*)PPartit)->GetNumPagina();
			}
			
			((DiMas*)PPartit)->SetNumPagina(np);
		}
		
		ScrollingAllActive = ScrollPage();
		if (!ScrollingAllActive) 
		{
			if (GetEsecuzione())
			{
				if (GetModEsecuzione()) AddTimeOut(GetNextTimeExec(), Call_TimeExecution, (void *)NULL);
				return;
			}
			
			TerminaImpaginazione();
		}
	}
}


void MaseWindow::IncScrollingPos()
{
    // in base al tipo di leggio incrementa la posizione di X o di Y
	
	ScrollingPosX += ScrollingInc;
}

Bool MaseWindow::CheckScrollingEnd()
{
    // in base al tipo di leggio cambia il controllo 
    // per vedere se siamo arrivati alla fine
	return (GetScrollingPosX()<(GetVU2Rgt()-GetScrollingInc()));
}


int MaseWindow::GetScrollingWidth()
{
	return GetScrollingInc();
}


int MaseWindow::GetScrollingHeight()
{
	return GetVU2Up();
}


void MaseWindow::DrawScrollingIndicator(int color)
{
	// disegna l'indicatore di battuta orizzontale o verticale
	// in base al tipo di leggio
	
	int x = GetScrollingPosX()+(GetAbsPos()).x;
	int y = GetScrollingPosY()+(GetAbsPos()).y;
	int step = ScrollingBarraWidth;
	
	while ((x+step) > ((GetAbsPos()).x + GetWidth())) step --;
	Colore oldcol = Scr->GetColor();
	Scr->SetColor((ColorIcon)color);
	Scr->Rect(class Rectangle(Point(x,y-1),Point(x+step,y-GetVU2Up()+1)));
	Scr->SetColor((ColorIcon)white);
	Scr->Rect(class Rectangle(Point(x+1,y-2),Point(x+step-1,y-GetVU2Up()+2)),TRUE);  
	Scr->SetColor(oldcol);
}

void MaseWindow::StartScrolling()
{	
#ifdef DEBUG_EXEC
	FILE *pippo;
	pippo = fopen("/home/demo/txtA/DimasExec.txt","a");
	fprintf(pippo,"StartScrolling 1\n");
	fclose(pippo);
#endif
	
	
	InitScrolling();
	ScrollingState=TRUE;
	
	int ScrT = GetScrollingTime();
	
	if (!GetEsecuzione())
	{
		SetInitialTime();
	}
	
	AddTempoIdeale(ScrT);
	
	if (GetModEsecuzione()) Call_Scrolling();
	
	
}


Bool MaseWindow::WaitStateEndBatt()
{
	Battuta *pbt, pbtF;
	
#ifdef DEBUG_EXEC
	FILE *pippo;
	pippo = fopen("/home/demo/txtA/DimasExec.txt","a");
	fprintf(pippo,"WaitStateEndBatt: countBatt %d\n",countBatt);
	fclose(pippo);
#endif
	
	pbt = PPartit->GetFirstSpa()->GetBattNumProg(countBatt);
	
	if (pbt==NULL)
		return TRUE;
	else
	{
		if ((pbt->GetBattuta()) == INIZIO_PENTAGRAMMA)
			return (GetWidth() <= (GetScrollingPosX() + ScrollingBarraWidth*2));
		else
			return (pbt->GetAbsPos().x <= (GetScrollingPosX() + ScrollingBarraWidth*2));
	}
}

Bool MaseWindow::JumpStateEndBatt()
{
	return WaitStateEndBatt();
}

void MaseWindow::Draw_Barre_Scrolling(void *data)
{
    // Copia della finestra Copia nella pag. attuale
	
	int ScrT = GetScrollingTime();
	
	SetActualTime();
	
    // prende i valori assoluti della zona da copiare
	int starty =(GetAbsPos()).y;
	int startx = (GetAbsPos()).x;
	
	
#ifdef SCR_FINESTRE
	// realizza la copia con le finestre	
	Scr->CopyPixmap(GetCopiaDrawingArea(),
#else
		// realizza la copia con le pixmap
		Scr->CopyPixmap(NULL, //Scr->GetLPixmapCopia(),
#endif
		
        // in entrambe i casi gli altri parametri della Copypixmap
        // sono le cordinate di inizio e fine e la posizione
        // inziale: si copia una fetta di ampiezza 
        // ScrollingIncWidth() e ampiezza ScrollingIncHeight()
		GetScrollingPosX()+startx,GetScrollingPosY()+starty,
		GetScrollingWidth(),GetScrollingHeight());
	
	// incrementa ScrollingPosX o ScrollingPosY 
	// che tengono conto della posizione dello scrolling
	IncScrollingPos();
	
	
	// se non ha finito di copiare tutta la pagina 
	// disegna la barra di delimitazione in nero
	// e reimposta il timeout
	// altrimenti azzera lo ScrollingState
	
	if (CheckScrollingEnd())
	{
		if (GetWaitState()==0)
			if (WaitStateEndBatt())
				return;
			
			if (GetJumpState())
				if (JumpStateEndBatt())
					SetScrollingInc(0);
				
				DrawScrollingIndicator();
				
				if (GetModEsecuzione()) 
				{
					if ((ScrT + GetDiffTime()) < MIN_CALL_TIMER)
						AddTimeOut(MIN_CALL_TIMER,Call_Scrolling, data);
					else
						AddTimeOut(ScrT + GetDiffTime(),Call_Scrolling, data);
				}
	}
	else
	{
		StopScrolling();
		return;
	}
	
	AddTempoIdeale(ScrT);
}

void MaseWindow::ImpostaBattPage(NumCode nc, NumCode np)
{
	((DiMas*)PPartit)->ImpostaBattPage(nc, np);
}
