/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/*
 *********************************************************************
 *	TimeSignDlg.cpp													 *
 *	Finestra con pulsanti, textcontrol e checkbox per selezionare le *
 *  Time Signature ESTERNA ed INTERNA								 *
 *																	 *
 *********************************************************************
*/

#if defined (__WXGTK__)
#include "./wxr/timesignaturexdialog.wxr"
#include "./wxr/timesignaturexdialog.h"
#endif

#if defined (__WXMSW__)
#include "./wxr/timesignaturewdialog.wxr"
#include "./wxr/timesignaturewdialog.h"
#endif

#include "gui/timesigndlg.hpp"
#include "wx/wx.h"
#include "wx/resource.h"
#include "../bat/def_bat.hpp"
#include "../drw/frmanage.hpp"

#include "string.h"
#include "bat/tempo.hpp"

BEGIN_EVENT_TABLE(TimeSignDlg, wxDialog)
  	EVT_BUTTON(ID_BUTTON103, TimeSignDlg::OnOk)
	EVT_BUTTON(ID_BUTTON112, TimeSignDlg::OnCancel)
END_EVENT_TABLE()

TimeSignDlg::TimeSignDlg()
{
}


TimeSignDlg::~TimeSignDlg()
{
    tempo_interno->Destroy(); 
	tempo_esterno->Destroy();
	check_interno->Destroy();
}

void TimeSignDlg::OnOk(wxCommandEvent& WXUNUSED(event) )
{
	EndModal(ID_BUTTON103);
}

void TimeSignDlg::OnCancel(wxCommandEvent& WXUNUSED(event) )
{
	EndModal(ID_BUTTON112);
}

wxString TimeSignDlg::GetTimeSignature()
{
	if (InitDlg()==true)
	{
		return TimeString;
	}
	else
	{
		return "\0";
	}
}




bool TimeSignDlg::InitDlg()
{
	
	
	bool selected = true;
	char *buffer = '\0';
	
	wxResourceParseData(TimeSignatureDialog);
	if (LoadFromResource(FrameManager::currFrame, "TimeSignatureDialog"))
	{
		tempo_esterno = (wxTextCtrl *)wxFindWindowByName("textctrl_ext", this);
		tempo_interno = (wxTextCtrl *)wxFindWindowByName("textctrl_int", this);
		check_interno = (wxCheckBox *)wxFindWindowByName("checkbox_int", this);
	}
	else
	{
		wxMessageBox("Error loading resources!","Error!", wxOK | wxICON_EXCLAMATION, FrameManager::currFrame);
		tempo_esterno = NULL;
		tempo_interno = NULL;
		check_interno = NULL;
		return false;
	}
  Centre(wxBOTH);
  if (ShowModal() == ID_BUTTON103) //bottone di Ok
  {
      wxString Esterno, Interno;
	  int e=0, c=0, i=0;
	  if (tempo_esterno->IsModified()) {
		  Tempo temporary;
		  // elimino tutti gli spazi nela stringa e converto i '+' in '' 
		  wxString p = tempo_esterno->GetValue();
		  for (size_t x=0; x<p.length(); x++)
		  {
			  if (strchr(" ",p[x])==NULL)
				  Esterno += p[x];
		  }
		  Esterno.Replace("+","",true);
		  if (Esterno.Matches("C"))
			  Esterno = "c";
		  if (Esterno.Matches("c/") || Esterno.Matches("C/"))
			  Esterno = "C";
		  e = temporary.Validator(Esterno, false);
	  }
	  if (check_interno->GetValue())
	  {
		  c = 1;
		  if (tempo_interno->IsModified())
		  {
			  Tempo temporary;
			  // Elimino tutti gli spazi
			  wxString p = tempo_interno->GetValue();
			  for (size_t x=0; x<p.length(); x++)
			  {
				  if (strchr(" ",p[x])==NULL)
					  Interno += p[x];
			  }
			  
			  i = temporary.Validator(Interno, true);
		  }
	  }
	  // Se l'utente ha immesso almeno un tempo non valido
	  if ((e == 0) || ((e!=0) && (c==1) && (i==0)))
	  {
		  wxMessageBox("Please insert correct data","Syntax Error", wxOK);
		  selected = false;
	  }
	  else
	  {
		  //CHECK 1-6
		  if (e==1) //Tempo Esterno a/d
		  {
			  if (c==0)//!CheckBox
			  {
				  TimeString = Esterno;//tempo_esterno->GetValue();
			  }
			  else // (CheckBox)
			  {
				  if (i==1) //Tempo Interno a/d
				  {
					  TimeString = Esterno;//tempo_esterno->GetValue();
					  TimeString += "";
					  TimeString += Interno; //tempo_interno->GetValue();
				  }
				  else //(Tempo Interno a+b+c/d)
				  {
					  wxMessageBox("The internal time signature must have format num/den!!","Syntax Error",wxOK);
				  }
			  }
		  }
		  else // (Tempo Esterno a+b+c/d)
		  {
			  if (c==0) //!CheckBox
				  wxMessageBox("Please insert the internal time signature.","MISSING DATA",wxOK);
			  else // (CheckBox)
			  {
				  if (i==1) //Tempo Interno a/d
				  {
					  TimeString = Esterno;//tempo_esterno->GetValue();
					  TimeString += "";
					  TimeString += Interno; //tempo_interno->GetValue();
				  }
				  else //Tempo Interno a+b+c/d
					  wxMessageBox("The internal time signature must have format num/den!!","Syntax Error",wxOK);
			  }
		  }   
	  }
  }
  else selected = false;
  
  return selected;

}
