/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */




#include "./wxr/welcomew.wxr"
#include "./wxr/welcomew.h"

#include "gui/welcomedialog.hpp"
#include "wx/wx.h"
#include "wx/resource.h"
#include "../drw/frmanage.hpp"

#include "string.h"

#include "drw/musicedt.hpp"
#include "version.hpp"
#include <windows.h>

extern wxMusicEditor *MusicEditor;


/**************************************************************/
BEGIN_EVENT_TABLE(WelcomeDlg, wxDialog)
END_EVENT_TABLE()

/**************************************************************/
WelcomeDlg::WelcomeDlg()
{
stbmp=NULL;
stversion=NULL;
}


/**************************************************************/
WelcomeDlg::~WelcomeDlg()
{
if (stbmp!=NULL)
	stbmp->Destroy();
if (stversion!=NULL)
	stversion->Destroy();
}

/**************************************************************/
bool WelcomeDlg::InitDlg(wxWindow *parent)
{

	if(!wxResourceParseData(welcome))
		wxMessageBox("Error parsing welcome resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);

	if (LoadFromResource(parent, "welcome"))
	{
		wxBitmap welcomeslide("WELCOME_BMP", wxBITMAP_TYPE_BMP_RESOURCE);
		stbmp=new wxStaticBitmap(this,-1,welcomeslide,wxPoint(5,5),wxSize(424,290));
	}
	else
	{
		wxMessageBox("Error loading welcome resources!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
		return FALSE;
	}
	wxString label = wxString(GetVersione())+", "+wxString(GetDataVersione());
	stversion=new wxStaticText(this,-1,"",wxPoint(20,250),wxSize(400,18),wxALIGN_CENTRE	| wxST_NO_AUTORESIZE);
	stversion->SetBackgroundColour(*wxWHITE);
	stversion->SetFont(wxFont(12, wxROMAN, wxNORMAL, wxBOLD));
	stversion->SetLabel(label);

  SetModal(FALSE);

return TRUE;
}


WelcomeDlg *welcomedlg;

class WelcomeTimer : public wxTimer
{
public:
	void Notify()
	{
		if(welcomedlg!=NULL)
		{
			welcomedlg->Destroy();
			welcomedlg=NULL;
		}
	}
};

/**************************************************************/
int WelcomeOpen(int duration, wxWindow *parent) // in seconds
{
	int ret=TRUE;

	welcomedlg=new WelcomeDlg;
	if (welcomedlg==NULL) return ER_CONTINUE;
	WelcomeTimer *timer=new WelcomeTimer;

	if(duration!=0)
		timer->Start(duration*1000,true);

	ret= welcomedlg->InitDlg(parent);
 
	welcomedlg->Centre();
	if(welcomedlg->Show(TRUE)) 
		ret = TRUE;
	else 
		ret = FALSE;

	wxSafeYield();
	return (ret);
}


/**************************************************************/
int WelcomeClose(int duration) 
{
	Sleep(duration*1000);
	if (welcomedlg!=NULL)
	{
		welcomedlg->Destroy();
		welcomedlg=NULL;
	}

	return (TRUE);
}

