/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include <string.h>
#include <wx/wx.h>
#include <wx/image.h>

#include "img/image.hpp"
#include "drw/musicedt.hpp"
#include "drw/scorefrm.hpp"	
#include "smr/smr.hpp"

#define FATT 2.8346021  

#undef LoadImage

Image::Image()
{
	imap=NULL;
	w=0;
	h=0;
	name="";
	loaded=false;
	isImageScore=false;
}

Image::Image(int width, int height)
{
	imap=(Pixmap)new wxBitmap(width,height);
	w=width;
	h=height;
	name="";
	loaded=false;
	isImageScore=false;

}

void Image::SetScrDimension( int h, int w)
{
	scrhigh=h;
	scrwide=w;
}


Image::~Image()
{
  if(imap!=NULL)
  {
	delete ((wxBitmap*)imap);
  }
}

void Image::Clear()
{
	UnLoad();
	name="";
}  

void Image::UnLoad()
{
	if(imap!=NULL)
	{
		delete ((wxBitmap*)imap);
		imap=NULL;
	}
	w=0;
	h=0;
}  

void Image::Setup(char *fname)
{
	Clear();
	name = wxString(fname);
}

void Image::LoadImage(char *fname)
{
	name = wxString(fname);
	Load();
}

void Image::Load()
{
	
	wxImage image;
	
	UnLoad();

	SMRFile *smrf=MusicEditor->smrf;

	if (!image.LoadFile(smrf->GetPath()+PATH_SEP+name))
	{
			wxMessageBox("Error loading image file "+wxString(name));
	}
	
	loaded=true;
	
	if (!image.Ok()){
		wxMessageBox("error 2 loading image file "+wxString(name));
	}
	
	
	w=image.GetWidth();
	h=image.GetHeight();
	
	
	float dimY=(float) DimensioneY;  
	float dimX=(float) DimensioneX;
	
	float dim=(dimX/(float) w);
	float dim1=(dimY/(float) h);
	float FatScale;
	if (dim<dim1)
		FatScale=dim;
	else
		FatScale=dim1;
	image.Rescale((int) (w*FatScale),(int) (h*FatScale));
	
	
	w=image.GetWidth();
	h=image.GetHeight(); 
	
	imap=(Pixmap)new wxBitmap;
	*((wxBitmap*)imap)=image.ConvertToBitmap();
	if(!((wxBitmap*)imap)->Ok())
	{
		Message("ERROR creating bitmap- low memory?");
	}
	
}

 
void Image::LoadImage(char *fname, int Height, int Width)	//DB
{
	name = wxString(fname);

	wxImage image;

	UnLoad();

	if (!image.LoadFile(name)){
		wxMessageBox("error 4 loading image file "+wxString(name));
	}


	loaded=true;

	if (!image.Ok()){
		wxMessageBox("error 5 loading image file "+wxString(name));
	}


	w=image.GetWidth();
	h=image.GetHeight();



	if(Height!=-1 && Width!=-1)
	{
		float dimY=(float) Height;  
		float dimX=(float) Width;

		float dim=(dimX/(float) w);
		float dim1=(dimY/(float) h);
		float FatScale;
		if (dim<dim1)
			FatScale=dim;
		else
			FatScale=dim1;

		image.Rescale((int) (w*FatScale),(int) (h*FatScale));
	}

	w=image.GetWidth();
	h=image.GetHeight(); 

	imap=(Pixmap)new wxBitmap;
	*((wxBitmap*)imap)=image.ConvertToBitmap();
	if(!((wxBitmap*)imap)->Ok())
	{
		Message("ERROR creating bitmap- low memory?");
	}
}


void Image::Draw()
{
	if(imap!=NULL)
		Scr->PutImage(GetAbsPos(),imap);
	
}

char* Image::GetName()
{
  return((char*)name.c_str());
}

void Image::BlitImage(wxBitmap *pic, int x, int y)
{
	wxMemoryDC tmp1DC,tmp2DC;
	tmp1DC.SelectObject(*pic);
	tmp2DC.SelectObject(*((wxBitmap*)imap));
	tmp2DC.Blit(x,y,pic->GetWidth(),pic->GetHeight(),&tmp1DC,0,0);
}

void Image::RebuildPage()
{	
	wxImage *img;
	wxBitmap *bmp;
	bool removeFile=false;

	double fatt=0.2;

	wxString str,str2,path,path2;
					
	int x=0,y=0; 
	bmp=new wxBitmap(w,h);

	imap=Pixmap(bmp);

	path = MusicEditor->GetWdfObjectPath()+PATH_SEP+"Image"+PATH_SEP;

	loaded=true;
	int N = imageVect.GetCount();
	MusicEditor->GetCurrentViewFrame()->CreateProgressSlide("Building Page..",N+2);	
	for(int i=0; i<N; i++)
	{		
		str=imageVect[i];
		str2=str.Upper();
		if(str2.IsSameAs("NO_L_MARGIN"))
			continue;
		if(str2.Contains("L_MARGIN"))
		{
			path2=path+str;
			img = new wxImage;

			img->LoadFile(path2);
			if(removeFile)
				wxRemoveFile(path2);

			BlitImage(&img->ConvertToBitmap(),0,0);
			x=x+img->GetWidth();
			if(img!=NULL)
			{
				delete img;
				img=NULL; 
			}

		}
		
		if(str2.IsSameAs("NO_R_MARGIN"))
			continue;
		if(str2.Contains("R_MARGIN"))
		{
			path2=path+str;
			img = new wxImage;

			img->LoadFile(path2);
			if(removeFile)
				wxRemoveFile(path2);

			BlitImage(&img->ConvertToBitmap(),w-img->GetWidth(),0);
			if(img!=NULL)
			{
				delete img;
				img=NULL; 
			}
		}
		
		if(str2.IsSameAs("NO_HEADER"))
			continue;
		if(str2.IsSameAs("NO_FOOTER"))
			continue;


		if(!str2.Contains("MARGIN"))
		{
			path2=path+str;
			img = new wxImage;

			img->LoadFile(path2);
			if(removeFile)
				wxRemoveFile(path2);

			BlitImage(&img->ConvertToBitmap(),x,y);
			y+=img->GetHeight();
			if(img!=NULL)
			{
				delete img;
				img=NULL; 
			}
		}
		MusicEditor->GetCurrentViewFrame()->SetProgressSlide(i+1,"Building Page..");					
	}

	wxImage tmp(*bmp);
	tmp.Scale((int) (w*fatt), (int) (fatt*h));
	delete bmp;
	bmp=NULL; 
	MusicEditor->GetCurrentViewFrame()->SetProgressSlide(N+1,"Building Page..");	
	imap=(Pixmap) new wxBitmap(tmp.ConvertToBitmap());
	MusicEditor->GetCurrentViewFrame()->SetProgressSlide(N+2,"Building Page..");	
	MusicEditor->GetCurrentViewFrame()->DeleteProgressSlide();	
}

wxBitmap* Image::Scale(wxBitmap* bitmap, int dh,int dw)
{
	wxImage tmp (*bitmap);
	wxBitmap* newBitmap;
	newBitmap = new wxBitmap(tmp.Scale((int) dw, (int) dh).ConvertToBitmap());
	return newBitmap;
}

void Image::ScaleImg(int dh, int dw)
{
	imap =(Pixmap) Scale((wxBitmap*)imap, dh, dw);
	Image::h=(int)dh;
	Image::w=(int)dw;
}
