/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/////////////////////////////////////////////////

#include "midibuffer.h"

#include "wx/stream.h"

/////////////////////////////////////////////////

void Midi_buffer::Set_buf(UNSIGNED_CHAR_ARRAY* buffer)
{
	buf.Clear();
	WX_APPEND_ARRAY(buf, (*buffer));
}

void Midi_buffer::Load_file(wxString filename)
{
	int read_bytes=0;
	BYTE* apbuf;
	
	if (filename.IsEmpty()) throw wxString("Invalid path !");
	wxFile in(filename, wxFile::read);
	if (buf.GetCount()) buf.Clear();
	apbuf=new BYTE[in.Length()];
	in.Read(apbuf, in.Length()); 
	for (int i=0;i<in.Length();i++) buf.Add(apbuf[i]);
	delete []apbuf;
	filepath=filename;
}

BYTE Midi_buffer::Get_format()
{
	BYTE format_=CharList_to_Int(8, 2);
	if (format_>1) throw wxString("Format not supported !");
	return format_;
}

void Midi_buffer::Save_to_file(wxString filename)
{
	if(filename.IsEmpty()) throw wxString("Incorrect destination path ! ");
	DWORD buf_len=GetCount();
	if (!buf_len) throw wxString("Empty out buffer, can not save !");
	wxFile out(filename, wxFile::write);
	BYTE* apbuf;
	apbuf=new BYTE[buf_len];
	for (DWORD i=0;i<buf_len;i++) apbuf[i]=buf[i];
	out.Write(apbuf, buf_len);
	delete []apbuf;
	apbuf=NULL;
}

DWORD Midi_buffer::CharList_to_Int(DWORD start, DWORD length)
{
	DWORD out=0;
	DWORD di=0;
	BYTE dc=0;
	
	if (GetCount()<start+length) throw wxString("Reached the end of the buffer! ");
	for (DWORD i=0;i<length;i++)
	{
		dc=buf[start+i];
		di=dc;
		di=di<<((length-i-1)*8);
		out+=di;
	}
	
	return out;
}

void Midi_buffer::Add(BYTE item)
{
	buf.Add(item);
}

void Midi_buffer::Append(UNSIGNED_CHAR_ARRAY& array)
{
	WX_APPEND_ARRAY(buf, array);
}

DWORD Midi_buffer::GetCount()
{
	return buf.GetCount();
}

void Midi_buffer::Insert(BYTE item, DWORD start)
{
	buf.Insert(item, start);
}

void Midi_buffer::Clear()
{
	buf.Clear();
	Used_midi_channels=0;
	filepath.Clear();
}

Midi_buffer::~Midi_buffer()
{
	Used_midi_channels=0;
}

wxString Midi_buffer::Get_filename()
{
	if (filepath=="") throw ("No File loaded !");
	wxString filename=filepath;
	filename.Remove(0, filename.Find(92, true)+1);
	return filename;
}

/////////////////////////////////////////////////

void Midi_buffer_IO::Load_file(wxString filename, const bool  only_header)
{
	if (iobuf)
	{
		delete []iobuf;
		iobuf=NULL;
	}
	if (filename.IsEmpty()) throw wxString("Invalid path !");
	wxFile in(filename, wxFile::read);
	if (only_header) buf_len=14;
	else (buf_len=in.Length());
	iobuf=new BYTE[buf_len];
	in.Read(iobuf, buf_len); 
	source=FILE;
	filepath=filename;
}

namespace
{
	struct TempBuffer
	{
		BYTE buffer[10000];
		TempBuffer * next;
		TempBuffer() : next(0) {}
	};
}

void Midi_buffer_IO::readFileInBuffer(wxInputStream * inputStream)
{
	TempBuffer * const first = new TempBuffer;
	TempBuffer * actual = first;
	unsigned int readBytes = 0;
	unsigned int totalReadBytes = 0;
	do
	{
		inputStream->Read(actual->buffer,10000);
		readBytes = inputStream->LastRead();
		totalReadBytes += readBytes;
		actual->next = new TempBuffer;
		if(readBytes<10000)
		{
			delete actual->next;
			actual->next = 0;
		}
		actual = actual->next;
	}
	while(readBytes==10000);
	iobuf = new BYTE [totalReadBytes];
	buf_len = totalReadBytes;
	actual = first;
	unsigned int writtenBytes = 0;
	while(actual)
	{
		unsigned int size = (writtenBytes + 10000 > totalReadBytes ? totalReadBytes - writtenBytes : 10000 );
		if(size>0)
		{
			memcpy(&iobuf[writtenBytes],actual->buffer,size);
			writtenBytes += 10000;
		}
		TempBuffer * tmp = actual;
		actual = actual->next;
		delete tmp;
	}
}

void Midi_buffer_IO::Load_file(wxInputStream * inputStream, const bool  only_header)
{
	if (iobuf)
	{
		delete [] iobuf;
		iobuf = NULL;
	}
	//if (filename.IsEmpty()) 
	if (!inputStream->IsOk()) 
		throw wxString("Invalid path !");
	//wxFile in(filename, wxFile::read);
	if (only_header) 
	{
		buf_len=14;
		iobuf=new BYTE[buf_len];
		inputStream->Read(iobuf, buf_len); 
	}
	else 
	{
		readFileInBuffer(inputStream);
	}
	source=FILE;
	filepath="";
}

Midi_buffer_IO::Midi_buffer_IO()
{
	iobuf=NULL;
	buf_len=0;
	source=NONE;
}

DWORD Midi_buffer_IO::GetCount()
{
	if (source!=FILE) throw wxString("Can't get buf_len ");
	return buf_len;
}

DWORD Midi_buffer_IO::CharList_to_Int(DWORD start, DWORD length)
{
	DWORD out=0;
	DWORD di=0;
	BYTE dc=0;

	if (GetCount()<start+length) 
		throw wxString("Reached the end of the buffer! ");
	for (DWORD i=0;i<length;i++)
	{
		dc=iobuf[start+i];
		di=dc;
		di=di<<((length-i-1)*8);
		out+=di;
	}
	
	return out;
}

Midi_buffer_IO::~Midi_buffer_IO()
{
	Clear();
	source=NONE;
}

void Midi_buffer_IO::Set_iobuf(BYTE* pbuffer)
{
	iobuf=pbuffer;
	source=MEMORY;
}

void Midi_buffer_IO::Clear()
{
	if (iobuf)
	{
		delete []iobuf;
		iobuf=NULL;
	}
	buf_len=0;
	source=NONE;
	filepath.Clear();
}

void Midi_buffer_IO::Save_to_file(wxString filename)
{
	if(filename.IsEmpty()) throw wxString("Destination file name is incorrect ! ");
	if (!buf_len) throw wxString("Empty out buffer, can not save ! ");
	wxFile out(filename, wxFile::write);
	out.Write(iobuf, buf_len);
}

/////////////////////////////////////////////////





