/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include "wx/wx.h"
#include <wx/file.h>
#include <wx/filedlg.h>
#include <wx/string.h>

#ifndef ___MBUFWINTYPES___
typedef unsigned long DWORD;
typedef unsigned char BYTE;
#endif

#ifndef __WXMSW__
typedef unsigned long DWORD;
typedef unsigned char BYTE;
#endif

#ifdef WX226
WX_DEFINE_ARRAY(BYTE,UNSIGNED_CHAR_ARRAY);
#else
typedef unsigned char* UNSIGNED_CHAR_ARRAY; //dummy def
#endif

#define CATCHWXSTRING  \
catch(wxString msg)     \
{						\
	wxMessageBox(msg, "ERROR"); \
}

#define DELARRAY(pointer) \
if (pointer) delete []pointer; \
pointer=NULL; \

#define DELITEM(pointer) \
if (pointer) delete pointer; \
pointer=NULL; \

enum Midi_format
{
	SMF0,
	SMF1
};

//////////MIDI_BUFFER////////////////////////////

class Midi_buffer
{

protected :

// attributes
	
	UNSIGNED_CHAR_ARRAY buf;
	BYTE Used_midi_channels;
	wxString filepath;

public :

// public interface
	
	virtual ~Midi_buffer();
	virtual BYTE Get_Num_Midi_tracks() { return CharList_to_Int(10, 2); }
	virtual BYTE Get_format();
	virtual BYTE Get_used_midi_channels() { return Used_midi_channels; }
	virtual DWORD Get_PPQN() { return CharList_to_Int(12, 2); }
	virtual DWORD CharList_to_Int(DWORD start, DWORD lenght);
	virtual DWORD GetCount();
	virtual UNSIGNED_CHAR_ARRAY& Get_buf(){ return buf; }
	virtual void Load_file(wxString filename);
	virtual void Clear();
	virtual void Add(BYTE item);
	virtual void Append(UNSIGNED_CHAR_ARRAY& array);
	virtual void Insert(BYTE item, DWORD start);
	virtual void Set_Used_midi_channels(BYTE num_chan) { Used_midi_channels=num_chan; }
	virtual void Set_buf(UNSIGNED_CHAR_ARRAY* buffer);	
	virtual void Save_to_file(wxString filename);	
	wxString Get_filename();
	wxString Get_filepath() { return filepath; }
	
// operators overloading

	BYTE& operator[](DWORD index)
	{
		return buf[index];
	}
	
	void operator+=(BYTE item)
	{
#ifdef WX226
		buf.Add(item);
#endif
	}

	void operator+=(UNSIGNED_CHAR_ARRAY* pArray)
	{
#ifdef WX226
		WX_APPEND_ARRAY(buf, (*pArray));
#endif
	}
};

/////////////////////////////////////////////////

const bool ONLY_HEADER=true;

class Midi_buffer_IO : public Midi_buffer
{

protected :

// attributes
	
	enum Source_device
	{
		FILE,
		MEMORY,
		NONE
	};
	
	BYTE* iobuf;
	DWORD buf_len;
	Source_device source;

private :
	void readFileInBuffer(wxInputStream *);

// overriding not available functions

	UNSIGNED_CHAR_ARRAY& Get_buf() { return buf; }
	void Set_buf(UNSIGNED_CHAR_ARRAY* buffer){ }
	void Append(UNSIGNED_CHAR_ARRAY& array){ }
	void Add(BYTE item){ }
	void Insert(BYTE item, DWORD start){ }
	int operator+=(BYTE item){ }
	int operator+=(UNSIGNED_CHAR_ARRAY* pArray){ }

public :

// public interface

	Midi_buffer_IO();
	~Midi_buffer_IO();
	BYTE* Get_iobuf() { return iobuf; }
	void Set_iobuf(BYTE* pbuf);
	
	// overriding functions
	
	DWORD GetCount();
	DWORD CharList_to_Int(DWORD start, DWORD lenght);
	void Load_file(wxString filename, const bool only_header=FALSE);
	void Load_file(wxInputStream *, const bool only_header=FALSE);
	void Save_to_file(wxString filename);
	void Clear();
	
	// operators overloading

	BYTE& operator[](DWORD index)
	{
		return iobuf[index];
	}
};

/////////////////////////////////////////////////






