/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


typedef unsigned long DWORD;
typedef unsigned char BYTE;
typedef float FLOAT;

#include "midiconv.h"

#include "math.h"

#ifndef WDFEDONLY	
#include "bat/battuta.hpp"
#endif

//////////GLOBAL CONSTANTS///////////////////////

extern wxString keyM[], keym[];

//////////MIDI_EVENT IMPLEMENTATION//////////////

Midi_event::Midi_event(DWORD _time, BYTE _status, 
	BYTE _channel, BYTE _source_track)
{
	time=_time;
	status=_status;
	channel=_channel;
	source_track=_source_track;
#ifndef WDFEDONLY
	memset(&moodsinfo, 0, sizeof(MEVT_EXTRA_INFO));
#endif
}

DWORD Midi_event::get_time() 
{ 
	return time; 
}

BYTE Midi_event::get_status() 
{ 
	return status; 
}

BYTE Midi_event::get_channel() 
{ 
	return channel; 
}

BYTE Midi_event::get_source_track() 
{ 
	return source_track; 
}

UNSIGNED_CHAR_ARRAY& Midi_event::get_data() 
{ 
	return data; 
}

void Midi_event::set_time(DWORD _time) 
{ 
	time=_time; 
}

void Midi_event::set_status(BYTE _status)
{ 
	status=_status; 
}

void Midi_event::set_channel(BYTE _channel) 
{ 
	channel=_channel; 
}

void Midi_event::set_source_track(BYTE _source_track) 
{ 
	source_track=_source_track; 
}

void Midi_event::set_data(UNSIGNED_CHAR_ARRAY _data) 
{ 
	data=_data;
}

void Midi_event::append_data(BYTE new_char) 
{ 
	data.Add(new_char); 
}

#ifndef WDFEDONLY
MEVT_EXTRA_INFO* Midi_event::get_p_moodsinfo()
{
	return &moodsinfo;
}

void Midi_event::set_moodsinfo(MEVT_EXTRA_INFO* info)
{
	moodsinfo=*info;
}
#endif

//////////MIDI_STRUCT IMPLEMENTATION/////////////

WX_DEFINE_OBJARRAY(Midi_track);
WX_DEFINE_OBJARRAY(Midi_tracks_set);

Midi_struct::Midi_struct()
{
	struct_mode=TRACK_STRUCT;
}

Midi_struct::~Midi_struct()
{

}

void Midi_struct::Set_struct_mode(Struct_type strmode)
{
	struct_mode=strmode;
}

Struct_type Midi_struct::Get_struct_mode()
{
	return struct_mode;
}

Midi_tracks_set& Midi_struct::Get_Set() 
{ 
	return Set; 
}

DWORD Midi_struct::Get_Num_tracks()
{
	return Set.GetCount();
}

void Midi_struct::Add(Midi_track& newtrack)
{
	Set.Add(newtrack);
}

void Midi_struct::Clear()
{
	Set.Clear();
}

void Midi_struct::Merge_tracks_by_time(Midi_track* p_dest_track)
{
	bool* terminated;
	terminated=new bool[Get_Num_tracks()];
	bool *enter_track;
	enter_track=new bool[Get_Num_tracks()];
	DWORD* act_ind=new DWORD[Get_Num_tracks()];

	for (DWORD i=0;i<Get_Num_tracks();i++)
	{
		act_ind[i]=0;
		terminated[i]=TRUE;
		enter_track[i]=TRUE;
		if (Set[i].GetCount()) terminated[i]=FALSE;
	}

	DWORD min_time=0xFFFFFFFF;
	Midi_event cur_event;
	bool end_merging=FALSE;
	
	while (end_merging==FALSE)
	{
		min_time=0xFFFFFFFF;
		for (i=0;i<Get_Num_tracks();i++)
		{
			if (!terminated[i])
			{
				cur_event=Set[i].Item(act_ind[i]);
				if (cur_event.get_time()<min_time) min_time=cur_event.get_time();
			}
		}
		for (i=0;i<Get_Num_tracks();i++)
		{
			if (!terminated[i])
			{	
				enter_track[i]=TRUE;
				while (enter_track[i])
				{
					cur_event=Set[i].Item(act_ind[i]);
					if ((cur_event.get_time())==(min_time))
					{
						(*p_dest_track).Add(cur_event);
						act_ind[i]=act_ind[i]+1;
						if ((DWORD)act_ind[i]>=Set[i].GetCount()) 
						{
							terminated[i]=TRUE;
							enter_track[i]=FALSE;
						}
					}
					else 
						if (cur_event.get_time()>min_time) enter_track[i]=FALSE;
						else throw wxString("Corrupted track found !");
				}
			}
		}
		
		bool temp_bool=FALSE;
		bool end=TRUE;
	
		for (i=0;i<Get_Num_tracks();i++)
		{
			temp_bool=terminated[i];
			end=end&&temp_bool;
		}
		end_merging=end;
	}
	delete []terminated;
	terminated=NULL; //NESIDELETE;
	delete []enter_track;
	enter_track=NULL; //NESIDELETE
	delete []act_ind;
	act_ind=NULL; //NESIDELETE
}

void Midi_struct::Search_event(Midi_event& evt, Midi_track& sequence)
{
	BYTE status=evt.get_status();
	BYTE channel=evt.get_channel();
	UNSIGNED_CHAR_ARRAY data=evt.get_data();
	DWORD index=0;
	DWORD track_len=0;
	DWORD num_tracks=Set.GetCount();
	Midi_event* p_test_evt=NULL;
	UNSIGNED_CHAR_ARRAY* p_evt_data=NULL; 
	bool found=true;
	for (DWORD track=0;track<num_tracks;track++)
	{
		index=0;
		track_len=Set[track].GetCount();
		for (index;index<track_len;index++)
		{
			p_test_evt=&(Set[track][index]);
			p_evt_data=&(p_test_evt->get_data());
			if ((p_test_evt->get_status()==status)&&
				(p_test_evt->get_channel()==channel)&&
				(p_evt_data->GetCount()>=data.GetCount()))
			{
				for (DWORD dataind=0;dataind<data.GetCount();dataind++)
					if (data[dataind]!=(*p_evt_data)[dataind]) found=false;
				if (found)
				{
					Midi_event add_evt=(*p_test_evt);
					add_evt.set_source_track(track);
					add_evt.set_time(index);
					sequence.Add(add_evt);
				}
				found=true;
			}
		}
	}
}

wxString Midi_struct::Get_key()
{
	Midi_track key_sequence;
	Midi_event key_event(0,15,15);
	UNSIGNED_CHAR_ARRAY data;
	data.Add(0x59);
	data.Add(0x2);
	key_event.set_data(data);
	Search_event(key_event, key_sequence);
	if (key_sequence.GetCount()==0) 
		return wxString("MIDI key not coded"); // tonalit non codificata
		//return keyM[7]; //Do major valore di default
	DWORD track=key_sequence[0].get_source_track();
	DWORD index=key_sequence[0].get_time();
	key_event=Set[track][index];
	data=key_event.get_data();
	if (data[3]==1)
		return (keym[(char)data[2]+7]);
	else
		return (keyM[(char)data[2]+7]);
}

//////////MIDI_DECODER IMPLEMENTATION////////////

Midi_decoder::Midi_decoder()
{
	p_channel_struct=NULL;
	p_track_struct=NULL;
}

void Midi_decoder::Set_channel_struct(Midi_struct* mdstr) 
{ 
	p_channel_struct=mdstr; 
}

void Midi_decoder::Set_track_struct(Midi_struct* mdstr) 
{ 
	p_track_struct=mdstr;
}

Midi_struct* Midi_decoder::Get_channel_struct() 
{ 
	return p_channel_struct; 
}

Midi_struct* Midi_decoder::Get_track_struct() 
{ 
	return p_track_struct; 
}

void Midi_decoder::SMF_decode(enum Struct_type dest)
{
	DWORD act_pos=0;
	DWORD track_len=0;
	DWORD track_end=0;
	Midi_event* Last_event; 
	Midi_struct& channel_struct=(*p_channel_struct);
	Midi_struct& track_struct=(*p_track_struct);

	BYTE Num_In_tracks=Get_Num_Midi_tracks();
	if (!((iobuf[0]=='M')&&(iobuf[1]=='T')&&(iobuf[2]=='h')&&(iobuf[3]=='d')))
		throw wxString("Unknown file header !");
	if (GetCount()<25) throw wxString("Not a SMF source !");
	if (!Num_In_tracks) throw wxString("No Midi tracks to process !"); 

	if ((dest==TRACK_STRUCT)||(dest==CHANNEL_AND_TRACK)) 
	{
		for (DWORD i=0;i<Num_In_tracks;i++)
			track_struct.Add(Midi_track());
	}

	if ((dest==CHANNEL_STRUCT)||(dest==CHANNEL_AND_TRACK)) 
	{   
		for (DWORD j=0;j<17;j++)
			channel_struct.Add(Midi_track());
	}

	if (!Get_PPQN()) throw wxString("PPQN is 0 !"); 
	Last_event=new Midi_event[Num_In_tracks];
	act_pos=14; // skip file header
	track_end=14;
	for (DWORD current_track=0;current_track<Num_In_tracks;current_track++)
	{   
		if (GetCount()<act_pos+8) 
			throw wxString("Problems in file decoding !"); 
		track_len=CharList_to_Int(act_pos+4, 4);
		if (!track_len) 
			throw wxString("Incorrect Midi track size"); 
		act_pos+=8; // skip track header
		track_end+=(8+track_len);
		while (act_pos<track_end) 
			Event_store(act_pos, current_track, Last_event, dest);
	}
    BYTE used_channels=0;
	if (dest!=TRACK_STRUCT)
	{
		for (BYTE channel=0;channel<16;channel++) 
			if (channel_struct[channel].GetCount()) used_channels++;
		Set_Used_midi_channels(used_channels);
	}
	delete []Last_event;
	Last_event=NULL;
}

void Midi_decoder::Event_store(DWORD& act_pos,DWORD current_track, Midi_event* Last_event, Struct_type dest)
{
	Midi_event event;
	BYTE event_dest_channel;
		
	event.set_source_track(current_track);
	event.set_time(Last_event[current_track].get_time()+DTimeDecode(act_pos));
	event.set_status(iobuf[act_pos]>>4);
	if (event.get_status()<8) // Running status
	{
		event.set_status(Last_event[current_track].get_status());
		event.set_channel(Last_event[current_track].get_channel());
		if ((event.get_status()==9)&&(!iobuf[act_pos+1])) event.set_status(8);
		if ((event.get_status()==8)&&(iobuf[act_pos+1])) event.set_status(9);
	}
	else
	{	
		event.set_channel(0x0f & iobuf[act_pos]);
		if ((event.get_status()==9)&&(!iobuf[act_pos+2])) event.set_status(8);
		act_pos++;
	}
	switch(event.get_status())
	{ 
	case  9  : Note_on_handler(act_pos, event); break;
	case  8  : Note_off_handler(act_pos, event); break;
	case 10  : Poly_aft_handler(act_pos, event); break;
	case 11  : Control_chg_handler(act_pos, event); break;
	case 12  : Program_chg_handler(act_pos, event); break;
	case 13  : Channel_aft_handler(act_pos, event); break;
	case 14  : Pitch_wheel_handler(act_pos, event); break;
	case 15  : // Global event 
		switch (event.get_channel()) 
		{
		case  0:  SysEx_Handler(act_pos, event); break;
		case  2:  SongPos_Handler(act_pos, event); break;
		case  3:  SongSelect_Handler(act_pos, event); break;
		case  7:  SysEx_Handler(act_pos, event); break;
		case 15:  Meta_Handler(act_pos, event); break;
		}
		event_dest_channel=16;
	break;
	}
	Last_event[current_track]=event;
	if (event.get_status()!=15) event_dest_channel=event.get_channel();
	if ((dest==CHANNEL_STRUCT)||(dest==CHANNEL_AND_TRACK)) (*p_channel_struct)[event_dest_channel].Add(event);
	if ((dest==TRACK_STRUCT)||(dest==CHANNEL_AND_TRACK)) (*p_track_struct)[event.get_source_track()].Add(event);
} 

DWORD Midi_decoder::DTimeDecode(DWORD& act_pos)
{      
    DWORD deltaT=0;
	BYTE num_bytes=1;
	
				
	if (!(iobuf[act_pos] & 0x80))  
	{
		deltaT=(DWORD)iobuf[act_pos]; 
		act_pos++;
	}
	else
	{
		deltaT=iobuf[act_pos] & 0x7f; 
		do
		{	
			act_pos++;
			deltaT=(deltaT<<7)+(iobuf[act_pos] & 0x7f); 
			num_bytes++;
			if (num_bytes>4) 
				throw wxString("Delta Time decoding error ");
		}
		while (iobuf[act_pos] & 0x80); 
		act_pos++;
	}
	return deltaT;
}  

void Midi_decoder::Note_off_handler(DWORD& act_pos, Midi_event& event)
{
	event.append_data(iobuf[act_pos]);
	event.append_data(iobuf[act_pos+1]);
	act_pos+=2;
}

void Midi_decoder::Note_on_handler(DWORD& act_pos, Midi_event& event)
{
	event.append_data(iobuf[act_pos]);
	event.append_data(iobuf[act_pos+1]);
	act_pos+=2;
}

void Midi_decoder::Poly_aft_handler(DWORD& act_pos, Midi_event& event)
{
	event.append_data(iobuf[act_pos]);
	event.append_data(iobuf[act_pos+1]);
	act_pos+=2;
}

void Midi_decoder::Control_chg_handler(DWORD& act_pos, Midi_event& event)
{
	event.append_data(iobuf[act_pos]);
	event.append_data(iobuf[act_pos+1]);
	act_pos+=2;
}

void Midi_decoder::Program_chg_handler(DWORD& act_pos, Midi_event& event)
{
	event.append_data(iobuf[act_pos]);
	act_pos+=1;
}

void Midi_decoder::Channel_aft_handler(DWORD& act_pos, Midi_event& event)
{
	event.append_data(iobuf[act_pos]);
	act_pos+=1;
}

void Midi_decoder::Pitch_wheel_handler(DWORD& act_pos, Midi_event& event)
{
	event.append_data(iobuf[act_pos]);
	event.append_data(iobuf[act_pos+1]);
	act_pos+=2;
}

void Midi_decoder::SysEx_Handler(DWORD& act_pos, Midi_event& event)
{
	DWORD event_data_length=DTimeDecode(act_pos);

	event.append_data(event_data_length);
	for (DWORD i=0;i<event_data_length;i++) event.append_data(iobuf[act_pos+i]);
	act_pos+=event_data_length;
}

void Midi_decoder::SongPos_Handler(DWORD& act_pos, Midi_event& event)
{
	event.append_data(iobuf[act_pos]);
	event.append_data(iobuf[act_pos+1]);
	act_pos+=2;
}

void Midi_decoder::SongSelect_Handler(DWORD& act_pos, Midi_event& event)
{
	event.append_data(iobuf[act_pos]);
	act_pos+=1;
}

void Midi_decoder::Meta_Handler(DWORD& act_pos, Midi_event& event)
{
	DWORD Meta_event_data_length;
	DWORD delta_pos;
	
	event.append_data(iobuf[act_pos]);
	act_pos++;
	delta_pos=act_pos;
	event.append_data(iobuf[act_pos]);
	Meta_event_data_length=DTimeDecode(act_pos);
	for (DWORD j=1;j<(act_pos-delta_pos);j++) 
		event.append_data(iobuf[delta_pos+j]);
	for (DWORD i=0;i<Meta_event_data_length;i++) event.append_data(iobuf[act_pos+i]);
	act_pos+=Meta_event_data_length;
}

//////////MIDI_ENCODER IMPLEMENTATION////////////

Midi_encoder::Midi_encoder()
{
	p_channel_struct=NULL;
	p_track_struct=NULL;
}

Midi_struct* Midi_encoder::Get_channel_struct() 
{ 
	return p_channel_struct; 
}

Midi_struct* Midi_encoder::Get_track_struct() 
{ 
	return p_track_struct;
}

void Midi_encoder::Build_file_header(BYTE format, BYTE Num_tracks, DWORD PPQN)
{
	Add('M');
	Add('T');
	Add('h');
	Add('d');
	Add(0);
	Add(0);
	Add(0);
	Add(6);
	Add(0);
	Add(format);
	Add(0);
	Add(Num_tracks);
	Add(PPQN>>8);
	Add(PPQN & 0xff);
}

void Midi_encoder::Build_track(Midi_track* pTrack)
{
	DWORD tk_header_pos=GetCount();
	
	Add('M');
	Add('T');
	Add('r');
	Add('k');
		
	Midi_event cur_event;
	UNSIGNED_CHAR_ARRAY deltaT;
	DWORD previous_time=0;
	DWORD track_size;

	for (DWORD act_pos=0;act_pos<(*pTrack).GetCount();act_pos++)
	{
		cur_event=(*pTrack).Item(act_pos);
		deltaT=DTimeEncode(previous_time, cur_event.get_time()); //questo rende deltaT un reference a DTE 
		Append(deltaT);
		Add((cur_event.get_status()<<4)+cur_event.get_channel());
		Append(cur_event.get_data());
		previous_time=cur_event.get_time();
	}
	
	track_size=GetCount()-tk_header_pos;
	Insert(track_size>>24, tk_header_pos+4);
	Insert((track_size & 0xff0000)>>16, tk_header_pos+5);
	Insert((track_size & 0xff00)>>8, tk_header_pos+6);
	Insert(track_size & 0xff, tk_header_pos+7);
}	

void Midi_encoder::Add_TrackEnd()
{
	Add(0);
	Add(0xff);
    Add(0x2f);
	Add(0x00);
}

void Midi_encoder::Set_p_track_struct(Midi_struct* p_struct)
{ 
	p_track_struct=p_struct; 
}

UNSIGNED_CHAR_ARRAY Midi_encoder::DTimeEncode(DWORD time1, DWORD time2)
{
	DWORD delta=time2-time1;
	UNSIGNED_CHAR_ARRAY deltaT_buf;
	BYTE buf_char;
			
	if (delta<0) throw wxString("Negative Delta Time !"); 
	for (int i=3;i>=0;i--)
	{
		buf_char=((delta>>(i*7)) & 0x7f);
		deltaT_buf.Add(buf_char);
	}
	if (deltaT_buf[2]||deltaT_buf[0]||deltaT_buf[1]) deltaT_buf[2]+=0x80;
	if (deltaT_buf[1]||deltaT_buf[0]) deltaT_buf[1]+=0x80;
	if (deltaT_buf[0]) deltaT_buf[0]+=0x80;
	
	UNSIGNED_CHAR_ARRAY deltaT;
	bool found=0;

	for (int t=0;t<4;t++)
	{
		if ((deltaT_buf[t])||(t==3)||(found)) 
		{
			deltaT.Add(deltaT_buf[t]);
			found=1;
		}
	}

	return deltaT;
}

//////////MIDI_CONVERTER IMPLEMENTATION//////////

Midi_converter::Midi_converter()
{
	dec.Set_track_struct(&track_struct);
	dec.Set_channel_struct(&channel_struct);
}	

Midi_encoder& Midi_converter::Get_out_buf() { return enc; }
Midi_decoder& Midi_converter::Get_in_buf() { return dec; }

void Midi_converter::SMF0_to_SMF1()
{
	Midi_track* pBuf_track=&channel_struct[16];
	(*pBuf_track).RemoveAt((*pBuf_track).GetCount()-1);
    BYTE Num_tracks=dec.Get_used_midi_channels()+1;
	enc.Clear();
	enc.Build_file_header(1, Num_tracks, dec.Get_PPQN());
	enc.Build_track(pBuf_track);
	enc.Add_TrackEnd();
	for (BYTE track=0;track<16;track++)
	{
		pBuf_track=&channel_struct[track];
		if ((*pBuf_track).GetCount())
		{
			pBuf_track=&channel_struct[track];
			enc.Build_track(pBuf_track);
			enc.Add_TrackEnd();
		}
	}
}

void Midi_converter::SMF1_to_SMF0()
{
	Midi_track aux_track;
	enc.Clear();
	for (DWORD i=0;i<track_struct.Get_Num_tracks();i++)
		track_struct[i].RemoveAt(track_struct[i].GetCount()-1);
	track_struct.Merge_tracks_by_time(&aux_track);
	enc.Build_file_header(0, 1, dec.Get_PPQN()); 
	enc.Build_track(&aux_track);
	enc.Add_TrackEnd();
}

void Midi_converter::MOODS_SMF_encode(Midi_format dest_format)
{
	if (!track_struct.Get_Num_tracks()) throw wxString("Incorrect destination path !");
	Midi_track* pBuf_track=&track_struct[0];
	for (DWORD i=0;i<track_struct.Get_Num_tracks();i++)
	track_struct[i].RemoveAt(track_struct[i].GetCount()-1);
	enc.Clear();
	if (dest_format==SMF0)
	{
		Midi_track aux_track;
		track_struct.Merge_tracks_by_time(&aux_track);
		enc.Build_file_header(0, 1, MOODS_PPQN); 
		enc.Build_track(&aux_track);
		enc.Add_TrackEnd();
		return;
	}
	if (dest_format!=SMF1) throw wxString("Unknown Midi Format !");
    BYTE Num_tracks=track_struct.Get_Num_tracks();
    enc.Build_file_header(1, Num_tracks, MOODS_PPQN);
	enc.Build_track(pBuf_track);
	enc.Add_TrackEnd();
	for (BYTE track=0;track<Num_tracks;track++)
	{
		pBuf_track=&track_struct[track];
		if ((*pBuf_track).GetCount())
		{
			pBuf_track=&track_struct[track];
			enc.Build_track(pBuf_track);
			enc.Add_TrackEnd();
		}
	}
}

Midi_buffer& Midi_converter::Convert_format(Midi_format dest_format, wxString filename, BYTE* midifile)
{
	if (filename) 
	{
		
		dec.Load_file(filename, ONLY_HEADER);
		if (dec.Get_format()==dest_format) throw wxString("Dest format=Source format !");
		dec.Clear();
		dec.Load_file(filename);
	}
	else 
		if (midifile) 
		{
			dec.Set_iobuf(midifile);
			if (dec.Get_format()==dest_format) throw wxString("Dest format=Source format ");
		}
		else
			throw wxString("No source to load !");

	switch (dest_format)
	{
	case 0 : 
		dec.Set_track_struct(&track_struct);
		dec.SMF_decode(TRACK_STRUCT);
		SMF1_to_SMF0();break;
	case 1 :
		dec.Set_channel_struct(&channel_struct);
		dec.SMF_decode(CHANNEL_STRUCT);
		SMF0_to_SMF1();break;
	default :
		throw wxString("Incorrect Dest format");
	}

	return enc;
}

void Midi_converter::Clear_struct(Struct_type clearsel)
{
	if ((clearsel==IN_BUFFER)||(clearsel==ALL)) dec.Clear();
	if ((clearsel==OUT_BUFFER)||(clearsel==ALL)) enc.Clear();
	if ((clearsel==CHANNEL_STRUCT)||(clearsel==ALL)||(clearsel==CHANNEL_AND_TRACK)) 
		channel_struct.Clear();
	if ((clearsel==TRACK_STRUCT)||(clearsel==ALL)||(clearsel==CHANNEL_AND_TRACK)) 
		track_struct.Clear();
}

Midi_struct& Midi_converter::Get_track_struct()
{
	return track_struct;
}

Midi_struct& Midi_converter::Get_channel_struct()
{
	return channel_struct;
}

//////////MIDI_POSITION_MAPPER IMPLEMENTATION////

Midi_position_mapper::Midi_position_mapper()
{
	p_map=NULL;	
	p_struct=NULL;
}

Midi_position_mapper::~Midi_position_mapper()
{
	DELITEM(p_map);
}

void Midi_position_mapper::Build_measure_map(Midi_struct& m_struct)
{
#ifndef WDFEDONLY	
	p_struct=&(m_struct);
	DWORD num_tracks=m_struct.Get_Num_tracks();
	DWORD search_bat_num=0;
	DWORD map_offset=0;
	DWORD max_bat=Get_max_bat();
	Battuta* p_cur_bat=NULL;
	Midi_event* p_cur_evt=NULL;
	p_map=new DWORD[max_bat*num_tracks];
	memset(p_map, 0xFF, max_bat*num_tracks*sizeof(DWORD));
	for (DWORD track=0;track<num_tracks;track++)
	{
		search_bat_num=1;
		for (DWORD index=0;index<m_struct[track].GetCount();index++)
		{
			p_cur_evt=&(m_struct[track][index]);
			p_cur_bat=p_cur_evt->get_p_moodsinfo()->p_measure;
			if ((p_cur_bat)&&((DWORD)(p_cur_bat->GetNumProgress())>=search_bat_num))
			{
				map_offset=(search_bat_num-1)*num_tracks+track;
				p_map[map_offset]=index;
				search_bat_num=p_cur_bat->GetNumProgress()+1;
			}
		}
	}
#endif
}

DWORD Midi_position_mapper::Get_max_bat()
{
	Midi_struct& m_struct=*p_struct;
	DWORD num_tracks=m_struct.Get_Num_tracks();
	DWORD max_bat=0;
	DWORD cur_bat=0;
	DWORD cur_index=0;
	Midi_event* p_evt=NULL;
#ifndef WDFEDONLY
	for (DWORD track=0;track<num_tracks;track++)
	{
		cur_index=m_struct[track].GetCount()-1;
		if (cur_index) p_evt=&(m_struct[track].Last());
		while((cur_index)&&(!m_struct[track][cur_index].
			get_p_moodsinfo()->p_measure)) cur_index--;
		if (cur_index)
		{
			p_evt=&(m_struct[track][cur_index]);
			cur_bat=p_evt->get_p_moodsinfo()->p_measure->
				GetNumProgress();
			if (cur_bat>max_bat) max_bat=cur_bat;
		}
	}
#endif
	return max_bat;
}

DWORD Midi_position_mapper::Get_start_offset(DWORD bat_num)
{
#ifndef WDFEDONLY
	DWORD num_tracks=p_struct->Get_Num_tracks();
	DWORD* p_array=new DWORD[num_tracks];
	Battuta* p_bat=NULL;
	Midi_event* p_evt=NULL;
	DWORD cur_time=0;
	DWORD max_time=0;
	Get_bat_start(bat_num, p_array);
	for (DWORD track=0;track<num_tracks;track++)
	{
		if (p_array[track]!=0xFFFFFFFF)
		{
			p_evt=&((*p_struct)[track][p_array[track]]);
			cur_time=p_evt->get_time();
			if (cur_time>max_time) max_time=cur_time;
		}
	}
	p_bat=p_evt->get_p_moodsinfo()->p_measure;
	return (max_time-p_bat->Get_start_ticks());
#endif
	return 0;
}

DWORD Midi_position_mapper::Get_bat_start(DWORD bat_num, DWORD* p_start_array)
{
#ifndef WDFEDONLY
	DWORD num_tracks=p_struct->Get_Num_tracks();
	DWORD map_offset=0;
	Midi_event* p_evt=NULL;
	Battuta* p_bat=NULL;
	DWORD bat_start_ticks=0;
	bool found=false;
	for (DWORD track=0;track<num_tracks;track++)
	{
		map_offset=(bat_num-1)*num_tracks+track;
		p_start_array[track]=p_map[map_offset];
		if ((!found)&&(p_start_array[track]!=0xFFFFFFFF))
		{
			p_evt=&((*p_struct)[track][p_start_array[track]]);
			p_bat=p_evt->get_p_moodsinfo()->p_measure;
			if (p_bat) 
			{
				bat_start_ticks=p_bat->Get_start_ticks();
				found=true;
			}
		}
	}
	return bat_start_ticks;
#endif
	return 0;
}

void Midi_position_mapper::Clear()
{
	DELITEM(p_map);
}

void Midi_position_mapper::Get_start_array(DWORD start_ticks, DWORD* p_start_array, Midi_struct& _struct)
{
	DWORD num_tracks=_struct.Get_Num_tracks();
	Midi_track* p_track=NULL;
	Midi_event* p_evt=NULL;
	DWORD index=0;
	bool found=false;
	for (DWORD track=0;track<num_tracks;track++)
	{
		p_track=&(_struct[track]);
		while ((index<p_track->GetCount())&&(!found))
		{
			p_evt=&((*p_track)[index]);
			if (p_evt->get_time()>=start_ticks) found=true;
			else index++;
		}
		p_start_array[track]=index;
		found=false;
		index=0;
	}
}

//////MIDI_SYMBOLIC_INTERFACE IMPLEMENTATION/////

void Midi_symbolic_interface::head(BYTE format, DWORD _num_tracks, DWORD ppqn)
{
	p_struct->Clear();
	for (DWORD i=0;i<_num_tracks;i++) p_struct->Add(Midi_track());
	PPQN=ppqn;
}

void Midi_symbolic_interface::Set_p_struct(Midi_struct* _p_struct)
{
	p_struct=_p_struct;
}

Midi_symbolic_interface::Midi_symbolic_interface()
{
	act_track=0;
	act_ticks=0;
	first_track=true;
}

void Midi_symbolic_interface::track()
{
	if (act_track>=p_struct->Get_Num_tracks()) throw ("Can't add new track !");
	if (!first_track) act_track++;
	else first_track=false;
	act_ticks=0;
}
	
void Midi_symbolic_interface::endtrack()
{
	Midi_track& cur_track=(*p_struct)[act_track];
	Midi_event event(act_ticks, 15, 15, act_track);
	UNSIGNED_CHAR_ARRAY event_data;
	event_data.Add(0x2f);
	event_data.Add(0);
	event.set_data(event_data);
	cur_track.Add(event);
}

void Midi_symbolic_interface::time(DWORD ticks)
{
	act_ticks+=ticks;
}

void Midi_symbolic_interface::program(BYTE channel, BYTE patch)
{
	Midi_track& cur_track=(*p_struct)[act_track];
	Midi_event event(act_ticks, 12, channel, act_track);
	UNSIGNED_CHAR_ARRAY event_data;
	event_data.Add(patch);
	event.set_data(event_data);
	cur_track.Add(event);
}
 
void Midi_symbolic_interface::volume(BYTE channel, BYTE volume)
{
	Midi_track& cur_track=(*p_struct)[act_track];
	Midi_event event(act_ticks, 11, channel, act_track);
	UNSIGNED_CHAR_ARRAY event_data;
	event_data.Add(7);
	event_data.Add(volume);
	event.set_data(event_data);
	cur_track.Add(event);
}

void Midi_symbolic_interface::noteon(BYTE channel, BYTE pitch, BYTE velocity)
{
	Midi_track& cur_track=(*p_struct)[act_track];
	Midi_event event(act_ticks, 9, channel, act_track);
	UNSIGNED_CHAR_ARRAY event_data;
	event_data.Add(pitch);
	event_data.Add(velocity);
	event.set_data(event_data);
	cur_track.Add(event);
}

void Midi_symbolic_interface::noteoff(BYTE channel, BYTE pitch, BYTE velocity)
{
	Midi_track& cur_track=(*p_struct)[act_track];
	Midi_event event(act_ticks, 8, channel, act_track);
	UNSIGNED_CHAR_ARRAY event_data;
	event_data.Add(pitch);
	event_data.Add(velocity);
	event.set_data(event_data);
	cur_track.Add(event);
}

void Midi_symbolic_interface::tempo(DWORD _tempo)
{
	Midi_track& cur_track=(*p_struct)[act_track];
	Midi_event event(act_ticks, 15, 15, act_track);
	UNSIGNED_CHAR_ARRAY event_data;
	event_data.Add(0x51);
	event_data.Add(3);
	event_data.Add((_tempo>>16)&0xff);
	event_data.Add((_tempo>>8)&0xff);
	event_data.Add(_tempo&0xff);
	event.set_data(event_data);
	cur_track.Add(event);
}

void Midi_symbolic_interface::tact(BYTE num, BYTE den, BYTE clocks, BYTE notes)
{
	den=(FLOAT)(log((FLOAT)den)/log(2));
	den=(BYTE)ceil(den);
	Midi_track& cur_track=(*p_struct)[act_track];
	Midi_event event(act_ticks, 15, 15, act_track);
	UNSIGNED_CHAR_ARRAY event_data;
	event_data.Add(0x58);
	event_data.Add(4);
	event_data.Add(num);
	event_data.Add(den);
	event_data.Add(clocks);
	event_data.Add(notes);
	event.set_data(event_data);
	cur_track.Add(event);
}

void Midi_symbolic_interface::key(BYTE signature, bool minor)
{
	Midi_track& cur_track=(*p_struct)[act_track];
	Midi_event event(act_ticks, 15, 15, act_track);
	UNSIGNED_CHAR_ARRAY event_data;
	event_data.Add(0x59);
	event_data.Add(2);
	event_data.Add(signature);
	event_data.Add(minor);
	event.set_data(event_data);
	cur_track.Add(event);
}

DWORD Midi_symbolic_interface::Get_act_ticks()
{
	return act_ticks;
}

/////////////////////////////////////////////////
		


