/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include "midibuffer.h"
#include <wx/thread.h>

#ifndef WDFEDONLY	
class Battuta;


typedef struct
{
	void* p_item;
	Battuta* p_measure;
	bool first_in_measure;
	DWORD dwUser;
}   MEVT_EXTRA_INFO;
#endif

//////////CONSTANTS//////////////////////////////

const DWORD MOODS_PPQN=480;

//////////MIDI_EVENT/////////////////////////////

const DWORD EVT_MIDI=13000;

enum 
{
	NOTE_OFF=8,
	NOTE_ON,
	POLY_AFTERTOUCH,
	CONTROL,
	PROGRAM,
	CHANNEL_AFTERTOUCH,
	PITCH_WHEEL,
	GLOBAL_EVT
};

enum CONTROL_CHANGE
{
	CHANNEL_VOLUME=7,
	PAN=10
};

class Midi_event
{

protected :

// attributes

	DWORD time;
	BYTE status;
	BYTE channel;
	BYTE source_track;
	UNSIGNED_CHAR_ARRAY data;
#ifndef WDFEDONLY
	MEVT_EXTRA_INFO moodsinfo; 
#endif

public :
    
// public interface

	DWORD get_time();
	BYTE get_status();
	BYTE get_channel(); 
	BYTE get_source_track();
    UNSIGNED_CHAR_ARRAY& get_data();
#ifndef WDFEDONLY
	MEVT_EXTRA_INFO* get_p_moodsinfo();
#endif
	void set_time(DWORD _time);
	void set_status(BYTE _status);
	void set_channel(BYTE _channel);
	void append_data(BYTE new_char);
	void set_data(UNSIGNED_CHAR_ARRAY _data);
	void set_source_track(BYTE _source_track);
#ifndef WDFEDONLY
	void set_moodsinfo(MEVT_EXTRA_INFO* info);
#endif
	Midi_event(DWORD _time=0, BYTE _status=0, BYTE _channel=0xff, BYTE _source_track=0xff);
};

//////////MIDI_STRUCT////////////////////////////

WX_DECLARE_OBJARRAY(Midi_event, Midi_track);
WX_DECLARE_OBJARRAY(Midi_track, Midi_tracks_set);
#include <wx/arrimpl.cpp> 

enum Filter_mode
{
	STATUS,
	CHANNEL,
	SPECIFIC
};

enum Struct_type 
{
	CHANNEL_STRUCT,
	TRACK_STRUCT,
	CHANNEL_AND_TRACK,
	IN_BUFFER,
	OUT_BUFFER,
	ALL
};

class Midi_struct
{

protected :

// attributes

	Midi_tracks_set Set;
	Struct_type struct_mode;

// private functions

public :
	
// public interface

	Midi_struct();
	~Midi_struct();
	Midi_tracks_set& Get_Set();
	DWORD Get_Num_tracks();
	void Add(Midi_track& newtrack);
	void Merge_tracks_by_time(Midi_track* p_dest_track);
	void Clear();
	void Set_struct_mode(Struct_type strmode);
	Struct_type Get_struct_mode();
	
	void Search_event(Midi_event& evt, Midi_track& sequence);
	wxString Get_key();
	
	// operators overloading
	
	Midi_track& operator[](DWORD index)
	{
		return Set[index];
	}
};

//////////MIDI_DECODER///////////////////////////

class Midi_decoder : public Midi_buffer_IO
{

protected :

// attributes

	Midi_struct* p_channel_struct;
	Midi_struct* p_track_struct;

// private functions
	
	DWORD DTimeDecode(DWORD& act_pos);
	void Event_store(DWORD& act_pos,DWORD current_track, Midi_event* Last_event, Struct_type dest);
    void Note_off_handler(DWORD& act_pos, Midi_event& event);
	void Note_on_handler(DWORD& act_pos, Midi_event& event);
	void Poly_aft_handler(DWORD& act_pos, Midi_event& event);
	void Control_chg_handler(DWORD& act_pos, Midi_event& event);
	void Program_chg_handler(DWORD& act_pos, Midi_event& event);
	void Channel_aft_handler(DWORD& act_pos, Midi_event& event);
	void Pitch_wheel_handler(DWORD& act_pos, Midi_event& event);
	void SysEx_Handler(DWORD& act_pos, Midi_event& event);
    void SongPos_Handler(DWORD& act_pos, Midi_event& event);
	void SongSelect_Handler(DWORD& act_pos, Midi_event& event);
	void Meta_Handler(DWORD& act_pos, Midi_event& event);

public :

// public interface

	void SMF_decode(Struct_type dest);
	void Set_channel_struct(Midi_struct* mdstr);
	void Set_track_struct(Midi_struct* mdstr);
	Midi_struct* Get_channel_struct();
	Midi_struct* Get_track_struct();
	Midi_decoder();
};

//////////MIDI_ENCODER///////////////////////////

class Midi_encoder : public Midi_buffer
{

protected :	

// attributes

	Midi_struct* p_channel_struct;
	Midi_struct* p_track_struct;

// private functions

	UNSIGNED_CHAR_ARRAY DTimeEncode(DWORD time1, DWORD time2);

public :

// public interface

	Midi_encoder();	
	void Build_file_header(BYTE format, BYTE Num_tracks, DWORD PPQN);
	void Build_track(Midi_track* pTrack);
	void Add_TrackEnd();
	void Set_p_track_struct(Midi_struct* p_struct);
	Midi_struct* Get_channel_struct();
	Midi_struct* Get_track_struct();
};
	
//////////MIDI_CONVERTER/////////////////////////

class Midi_converter 
{

protected :

// attributes

	Midi_struct channel_struct;
	Midi_struct track_struct;
	Midi_decoder dec;
	Midi_encoder enc;

// private functions
	
	// format converter implementation

	void SMF0_to_SMF1();
	void SMF1_to_SMF0();
	
public :

// public interface

	Midi_converter();
	void MOODS_SMF_encode(Midi_format dest_format);
	void Clear_struct(Struct_type clearsel);
	Midi_buffer& Convert_format(Midi_format dest_format, wxString filename="", BYTE* midifile=NULL);
	Midi_struct& Get_track_struct();
	Midi_struct& Get_channel_struct();
	Midi_decoder& Get_in_buf();
	Midi_encoder& Get_out_buf();
};

//////////MIDI_POSITION_MAPPER///////////////////

class Midi_position_mapper
{

protected :

// attributes

	DWORD* p_map;
	Midi_struct* p_struct;

// private functions

	DWORD Get_max_bat();

public :

// public interface

	void Clear();
	void Build_measure_map(Midi_struct& m_struct);
	void Get_start_array(DWORD start_ticks, DWORD* p_start_array, Midi_struct& _struct);
	DWORD Get_bat_start(DWORD bat_num, DWORD* p_start_array);
	DWORD Get_start_offset(DWORD bat_num);
	~Midi_position_mapper();
	Midi_position_mapper();
};

//////////MIDI_SYMBOLIC_INTERFACE////////////////

class Midi_symbolic_interface
{

protected :

// attributes

	Midi_struct* p_struct;
	DWORD PPQN;
	DWORD act_track;
	DWORD act_ticks;
	bool first_track;

public :
	
// public interface

	DWORD Get_act_ticks();
	Midi_symbolic_interface();
	void Set_p_struct(Midi_struct* _p_struct);
	void head(BYTE format, DWORD _num_tracks, DWORD ppqn);
	void tact(BYTE num, BYTE den, BYTE clocks, BYTE notes);
	void key(BYTE signature, bool minor);
	void track();
	void endtrack();
	void time(DWORD ticks);
	void program(BYTE channel, BYTE patch);
	void volume(BYTE channel, BYTE volume);
	virtual void noteon(BYTE channel, BYTE pitch, BYTE velocity);
	virtual void noteoff(BYTE channel, BYTE pitch, BYTE velocity);
	virtual void tempo(DWORD _tempo);
};

/////////////////////////////////////////////////





