/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */




/* $Id: midiimport.cpp,v 1.1 2005/07/25 16:31:16 bellinpo Exp $ */

/*=============================================================
File...................: MidiImport.cpp
Modulo............: IMP
Descrizione.....: Classe " figlia " di Import per la gestione dei file midi.
                    Proviene in realta'  da codice C ed e'  stata adattata a C++.
							
Autore.............: Gambineri Luca (GL)
Sviluppato.....: Ottobre-Novembre 98.
Ultima Revis.: --/--/----
=============================================================*/

// Modificare i puts e printf piu importanti in messaggi MOODS
//Eliminare gli exit(1)  e  controllare le allocazioni.

// METTERE come register alcune var (ex: trova ) !!!
#include <wx/progdlg.h>


#include "imp/midiimport.hpp"
#include "fig/accordo.hpp"

#include "bat/cviolino.hpp"
#include "fig/nbreve.hpp"
#include "fig/nsemibre.hpp"
#include "fig/nminima.hpp"
#include "fig/nsemimin.hpp"
#include "fig/ncroma.hpp"
#include "fig/nsemicro.hpp"
#include "fig/nbiscrom.hpp"
#include "fig/nsemibis.hpp"
#include "fig/nfusa.hpp"
#include "fig/psemibre.hpp"
#include "fig/pminima.hpp"
#include "fig/psemimin.hpp"
#include "fig/pcroma.hpp"
#include "fig/psemicro.hpp"
#include "fig/pbiscrom.hpp"
#include "fig/psemibis.hpp"
#include "fig/pfusa.hpp"
#include "int/legatval.hpp"

#include "liooext.hpp" //PB per regole di immissione

#include <math.h>
#define PI 3.14

void Trace(const char *format, ...)
{
	va_list argptr;
	va_start(argptr,format);
	FILE *fp=fopen("trace.log","a");
	if(fp!=NULL)
	{
		vfprintf(fp,format,argptr);
		fclose(fp);
	}
}

//#define WTRACE() Trace("executing line %d\n",__LINE__)
#define WTRACE()

const char * MidiImport::METALABEL[]={"Sequence Number","Text","(C)opyright",
								"Sequence or Track Name", "Instrument Name","Lyric",
								"Marker", "CUE Point", "Channel Prefix", "End Of Track",
								"SetTempo","SMPTE offset", "Time Signature","Key Signature",
								"Sequencer Specific"};

const unsigned char MidiImport::METAMAP[]={0x0,0x1,0x2,0x3,0x4,0x5,0x6,0x7,0x20,0x2f,
													0x51,0x54,0x58,0x59,0x7f};

const char MidiImport::level8va[]={-37,-30, -23, -16, -9, -2, 5, 12, 19, 26, 33};
const char MidiImport::offset[]={ 0, 0, 1, 1, 2, 3, 3, 4, 4, 5, 5, 6}; 
                                                        /* C,C#,D,D#,E,F,F#,G,G#,A,A#,B */ 
                                                    // oppure   0 , 1, 1, 2, 3, 4 4 5 5 6 6 
const char MidiImport::isAlter[]={0,1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0};  
                                                    // opuure   0,-1,0,-1, 0...
// Servono nel metodo CheckMidiAlt(..)
     // -1 significa che alla nota NON puo corrisp. una alterazione
     // vedi il modello: C, C#, D, D#,E, F... in AddNota() (qui sopra).
const char MidiImport::scalaBase[]={-1,0,-1,0,-1,-1,0,-1,0,-1,0,-1};

    // armDies[x]==valore che deve avere abs(armkey) affinche
    // la nota x sia influenzata dall'armatura.
const char MidiImport::armDies[]={2,2,4,4,6,1,1,3,3,5,5,7}; /* C  la seconda alterazione 
		   nell'ordine di comparsa in armatura , D  la quarta ... */
const char MidiImport::armBem[]={6,4,4,2,2,7,5,5,3,3,1,1};   // Stesso ragionamento...

const unsigned char MidiImport::mask[]={ 0x80, 0x90, 0xa0, 0xb0, 0xc0, 0xd0, 0xe0, 0xf0 };
Spartito  *PSpartito;

    wxString MidiImport::keyM[]={"Cb major","Gb major","Db major",
	"Ab major","Eb major","Bb major","F major",
	"C major","G major","D major","A major","E major",
	"B major","F# major","C# major"};
    wxString MidiImport::keym[]={"Ab minor","Eb minor","Bb minor",
	"F minor","C minor","G minor","D minor",
	"A minor","E minor","B minor","F# minor","C# minor",
	"G# minor","D# minor","A# minor"};

#define  NEXTDELPARSEJUMP  prev->next=parse->next,delete parse,parse=prev, jump=1;

#define  PARSELASTACCORDOFIRST parse->inizio=lastStart,isAccordo=1,parse->pausa=0;


BYTE original_key=100;

MidiImport::MidiImport(char *fileN): Import(fileN)
{
	add_MOODS_key=false;
	MOODS_key=original_key;
	MOODS_minor_key=false;
	microSecXquarto=DEFMICROSEC;
	ppqn=MYPPQN;
	tempoNum=TEMPONUM;
	tempoDen=TEMPODEN, 
	clock4Met=TICKSXMETR;
	num32sime=BISCROMEIN24TICKS;
	ticksXBattuta=(unsigned long)( (4.0 / tempoDen)*tempoNum*ppqn);
	minimaAlt=1000;
	massimaAlt=-1000;
	chanPref=0; 
	startTrk1=0;
	oldMicro=curByte=trackLen=0;
	ticksXBattuta=totTicks=0;
	globalTicks=0;
	formato=traccie=seqNum=0;
	noteInAcc=0;
	armKey=tonalKey=0;
	topOfList=NULL;
	endOfList=NULL;
//	topThisMark=NULL;
	primaBatDiLegato=NULL;
	generalText=NULL;
	seqName=NULL;
	strumName=NULL;
	copyRight=NULL;
	maxbatt=0;

	ibuf=NULL;
	obuf=NULL;
	pConv=NULL;
	End_buffer=0;

	source_buf=0;
	curByte=0;

	int i;
	for (i=0; i < MAXinACC; i++) noteLeg[i]=NULL; /* MAXinACC = 10 (H40) ; noteLEG  UN ARRAY
												      DI PUNTATORI A NOTA (H135) */
	for (i=0; i<NUMNOTE; i++)	centralTick[i]=0; /* NUMNOTE = 14 ; centralTick  un 
												      vettore che rappresenta i ticks 
													  centrali per le 14 note gestibili */											  
	for (i=0; i<NUMLOGN; i++)	{ 	 logNStart[i]=0; muVect[i]=0.0; } /* NUMLOGN  il numero
																	      di distribuzioni
																		  lognormali */
	for (i=0; i<NUMGAUSS; i++) {	gaussCenter[i]=0;varVect[i]=0.0;}
	
	if ( ! (topZeroMark = NewMarker() )) {
//APRINTF	     puts("\n\n **** Unable to allocate Memory.... QUIT FORCED !!! ****");
	     exit(1); /* topZeroMark  di tipo ptrMARKER ; NewMarker  una funzione definita
				      in C3044 e restituisce TARGET di tipo puntatore a MARKER inizializzando
					  inoltre il MARKER a cui punta (TYPEDEF ... MARKER si trova a H91) */
	}
}


MidiImport::~MidiImport()
{
	ptrMarker  appoggio;
	
	//fileName=NULL;
	//srcFile=NULL; /* srcFile  un puntatore a File ed  definito in IMPORT.h */
	
	// Distruggo la lista di eventi Midi associati alla traccia Zero
	appoggio=topZeroMark;
	while( topZeroMark ) /* questo ciclo while procede nel cancellare elementi della
						     lista fino a che non trova topZeroMark = NULL */
	{
	   topZeroMark=topZeroMark->next;
	   delete appoggio;
	   appoggio=topZeroMark;
	}

	if (pConv)
	{
		delete pConv;
		pConv=NULL;
	}
}


/********************************************************  
    			  METODI PRIVATI DELLA CLASSE 	
********************************************************/

  //=============================================\\
 // 	         Metodi privati che elaborano i dati letti                            \\
//===============================================\\

//
inline float MidiImport::Gauss(unsigned int med, float var, unsigned int x)
//{     return (float)(2/(sqrt(2*PI)*var))*exp(-(((x-med)*(x-med))/(2*var*var))) ;}
{     return (float)exp(-(((x-med)*(x-med))/(2*var*var))) ;} /* in realt per avere la
															  gaussiana bisognerebbe
														      dividere per sqrt(2*pi)*var ? */

inline float MidiImport::LogNorm(unsigned int da, float mu, float s,unsigned int x)  
{ return (float)(da/(2*(x-da)*sqrt(2*PI)*s))*exp(-((log(x-da)-mu)*(log(x-da)-mu))/(2*s*s)); }


 //---Inizializza le mappe delle note ---
char MidiImport::InitMusicParser() 
{
				//  coeff[] determinano i ticks centrali delle varie note.
	float	    coeff[]={.03125, .0625, .125, .25, .375, .5, .75, 1, 1.5, 1.75, 2, 3, 3.5, 4},
	               pesi[]={.7F, .80F, 1.0F, .54F, 1.0F, .52F, .9F , .9F };
	float 		toller, sigma2;
	int 		 i=0; 
    char numeri[5];
	 FILE * pf;

    pf=fopen("Mapping.cfg","rb");
//APRINTF    if ( ! pf )
//APRINTF    { 
//APRINTF		puts("\n\tFile di Configurazione della Mappatura Note"); 
//APRINTF	   puts("\t<NON Trovato>.  Utilizzati valori di default... ");
//APRINTF	 }
//APRINTF     else 
	if (pf!=NULL)
	 {
 	     while (fgets(numeri, 5, pf) && i<8)  
		 {
		   pesi[i]=(atof(numeri)/100);
		   i++;
		   strcpy(numeri, "0xxxx");
         }
		 fclose(pf); /* d2 END */
	} 
	
	toller=( (float)TOLERANCE*ppqn)/ (100.0*MYPPQN); /* TOLERANCE=18(35h) ; MYPPQN=640(53h) ;
													     ; ppqn=MYPPQN per default */
	
//APRINTF	if (ppqn <= 240) 
//APRINTF	{
//APRINTF		printf("\n WARNING: Few Ticks per Quarter Note (PPQN=%u)\n",ppqn );
//APRINTF		 puts("	 Identification Method could be affected...\n\n"); 
//APRINTF	}
	
	for (i=0; i<NUMNOTE; i++)	centralTick[i]=(int)(coeff[i]*ppqn);
		
	gaussCenter[0]=centralTick[POS16p]; /* POS16p=4(59h) */
	varVect[0]=(pesi[0]*toller * centralTick[POS16p]) /2; /* varVect  un vettore di float 
														      di lunghezza NUMGAUSS (h168) ;
															  ; pesi  definito in c154 */
    gaussCenter[1]=centralTick[POS8p];
	varVect[1]=(pesi[1]*toller * centralTick[POS8p]) /2;
	gaussCenter[2]=centralTick[POS4p];
	varVect[2]=(pesi[2]*toller * centralTick[POS4p]) /2;
	gaussCenter[3]=centralTick[POS4pp];
	varVect[3]=(pesi[3]*toller *centralTick[POS4pp]) /2;
	gaussCenter[4]=centralTick[POS2_4p];
	varVect[4]=(pesi[4]*toller*centralTick[POS2_4p]) /2;
	gaussCenter[5]=centralTick[POS2_4pp];
	varVect[5]=(pesi[5]*toller *centralTick[POS2_4pp]) /2;
	
    sigma2=(float)SIGMA*SIGMA;
	
	logNStart[0]=centralTick[POS32]+centralTick[POS64]; /* si pone punto di partenza della 
														    lognormale che rappresenta 1/16
															il tick centrale del 32-esimo 
															puntato ? */
	muVect[0]=fabs(log(centralTick[POS16]-logNStart[0])+(pesi[6]*sigma2) ); /* muVect si 
	                      ricava esplicitando mu nella relazione della moda in tg107 ? */
	logNStart[1]=centralTick[POS16p];
	muVect[1]=fabs(log(centralTick[POS8]-logNStart[1])+(pesi[7]*sigma2) );
	logNStart[2]=centralTick[POS8p];
	muVect[2]=fabs(log(centralTick[POS4]-logNStart[2])+sigma2);
	logNStart[3]=centralTick[POS4pp];
	muVect[3]=fabs(log(centralTick[POS2_4]-logNStart[3])+sigma2);
	logNStart[4]=centralTick[POS2_4pp];
	muVect[4]=fabs(log(centralTick[POS4_4]-logNStart[4])+sigma2);

	CHORDLEAP=centralTick[POS64]+centralTick[POS128];// Passo per l'accordo !
	return 0; //!
}

//------- Assegna nuovi ticks ai dati Midi attivi -------------
inline char  MidiImport::UpgradeTicks(unsigned long delta,  char chan) 
{
	ptrNotaChunk  who=topOfList; /* ptrNotaChunk  un puntatore a NotaChunk (descrittore
    di nota , ha gli stessi parametri riportati in tg104 ) ; topOfList=NULL per default */  
	globalTicks+=delta;  // Questi non vengono azzerati fino a fine traccia
	totTicks+=delta;     /* globalTicks e totTicks =0 per default */
	while (who)  
	{	if ((who->type==ATTIVA)&&(who->chan==chan))	
	        who->durata +=delta;
		who=who->next;
//		Message("MIDI-l2");

	}
	if (totTicks > ticksXBattuta && topOfList ) /* controlla che siano verificate le
												    condizioni di (6.2)tg106 ?*/
	{
	   who=topOfList;
	   while(who && who->type != ATTIVA) 
		{
			who=who->next;
//			Message("MIDI-l3");
		}
	  // OLD  if (who==NULL ||  who->attacco > ticksXBattuta ) 
	   if ( who!=NULL && who->attacco > ticksXBattuta ) 
	        return 1; 
	   else return 0; /* 0 dovrebbe voler dire che la prima nota ancora attiva nella lista
			 iniziata prima della fine della battuta analizzata e quindi servir una legatura
			perch tale nota si verr a trovare a cavallo di 2 battute , bisogner poi tenere 
			conto che invece di una nota singola si potrebbe avere a che fare con 
			un accordo */
	}
	return 1;
}

// Controlla che non ci siano note (in procinto di analisi) che siano ancora attive.
// Molto importante nel caso di accordi a fine battuta. Solo una nota dell'accordo
// e' sicuramente FINITA le altre no. Per questo prima si fanno finire....
inline char MidiImport::TestState() /* controlla le condizioni di tg106(6.2) 
                                        e ritorna 1 se sono verificate */
{
	char state=0;
	ptrNotaChunk  who=topOfList;
	
	if (totTicks > ticksXBattuta && topOfList ) 
	{
	   while(who && who->type != ATTIVA) 
	        who=who->next;
	 // OLD  if (who==NULL ||  who->attacco > ticksXBattuta ) state=1;
	   if  ( who !=NULL && who->attacco > ticksXBattuta ) state=1;
	}
	return state;
}

//--------- Marca come Finita una nota dalla Lista --------
//  Ritorna 1 se errore e 0  altrimenti. risulta como fare cosi in ScnaTrx().
inline char  MidiImport::MarkNota(unsigned char freq, unsigned char chan, unsigned char timbro)
{
	ptrNotaChunk   parse=topOfList;

	while ( parse && (parse->type!=ATTIVA  || parse->freq != freq || parse->chan !=chan))
				parse=parse->next; /* si esce dal ciclo quando si arriva alla fine
                della lista (parse = NULL) oppure quando viene trovata una nota ATTIVA
				che abbia canale e timbro uguali a quelli passati come parametri alla 
				funzione */
	if (parse) /* se la lista  finita senza che si sia trovata una nota con gli attributi 
			       cercati la funzione ritorna 1 ed emette un messaggio di errore ,
				   altrimenti l'attributo type della nota trovata viene cambiato da 
				   ATTIVA a FINITA e la funzione ritorna 0 */
	{  parse->rilascio=timbro;
		parse->type=FINITA;
		return 0;
	} 
	else 
	{ 
//APRINTF		puts("<NoteOFF> NOT match any <NoteON>"); // ...(freq=%d chan=%d)\n",freq,chan); 
		 		return 1;
		 }
}

//------------- Inizializza una nota. Equivalrebbe ad un Costruttore se...

MidiImport::ptrNotaChunk MidiImport::AddNota(unsigned char freq,unsigned char chan,unsigned char timbro, char accoda) 
{
	ptrNotaChunk ready;

	ready=(ptrNotaChunk)new NotaChunk;
	if (ready) 
	{	ready->freq=freq;
		ready->chan=chan;
		ready->timbro=timbro;
		ready->ottava=(freq /12); /* ma ottava  un char : perch gli si assegna un valore
								      numerico che pu essere oltretutto non intero ? */
		
		// Assegna l' atezza  sul pentagramma IN CHIAVE DI VIOLINO !!
		// Al primo rigo (Mi) assegna altezza zero.
		// Al Do centrale MIDI ( freq. 60)  assegna altezza "-2".
		
		ready->altezza=level8va[ready->ottava]+offset[(freq %12)];
		
		// Si basa sul vettore offset[] che rispecchia la sequenza di note:
		// C, C#, D, D#, E, F... = 0, 0, 1, 1, 2 3... perch un  C# e' sempre un C
		// ma con alterazione 1....
		// I modelli " oppure "  sono basati sui bemolle: C, Db, D, Eb, F...
        // che per scelta, NON vengono utilizzati.

		ready->alteraz=isAlter[freq%12];
		ready->type=ATTIVA;
		ready->durata=0;
		ready->valore=0;
		ready->attendibile=0.0;
		ready->rilascio=0;
		ready->pausa=0;
		ready->inAcc=0;
		ready->pauPos=0; 
		ready->notaPos=-1; /* cosa rappresentano pauPos e notaPos rispetto alla tabella
						       in tg104 ? */
		ready->attacco=totTicks;
		ready->inizio=totTicks;
		ready->next=NULL;
		if (!topOfList) topOfList=endOfList=ready;
		else 
		{  if (accoda) { 
		       endOfList->next=ready;
			   endOfList=ready;
		    }
		}
	}
//APRINTF	else puts("\nMemory Allocation Foult!");
	return ready;
}

// --- Dice a Moods quale armatura deve raffigurare--- 
tipoArmaturaChiave MidiImport::GetMidiArm(char chiave, char tonalita) /* tipoArmaturaChiave
										 un tipo dichiarato in source\bat\def_bat.hpp */
{	
	//  if (tonalita==0)         Tonalita' maggiore
	//  else                     relative CHIAVI MINORI...
	//  che pero' << Moods NON GESTISCE >>
	
	//  Qui considero sempre tonalita maggiori.
	 if (chiave >=0) 
	    return (tipoArmaturaChiave)chiave;
	 else 
	    return (tipoArmaturaChiave)(7-chiave); /* i valori di chiave<0 danno tonalit in b */
								       
}


// ---- Gestisce le alterazioni delle note nella battuta....-----
// Sia in funzione dell'armatura che delle alterazioni precedenti.

void MidiImport::CheckMidiAlter(ptrNotaChunk check)
{
   unsigned char i ;
   char skip=1, frq, nota; 
   char tester[128];
   
   // Per i Significati dei vettori vedere all'inizio del file.
   
   for (i=0; i<128; i++)  tester[i]=scalaBase[ (i %12)];

	while (check && check->type !=ATTIVA && (check->inizio < ticksXBattuta)) 
	{
       if (check->type != FITTIZIA) /* che cosa si intende per FITTIZIA ? */
	   {
	       frq=check->freq;
		   nota=(frq % 12);
		   
		   // Armatura di # che influenza la nota attuale
		   if (armKey  > 0 && armDies[nota]<=armKey) /* controlla se ci sono abbastanza
	          		 	  alterazioni in chiave per influenzare la nota attuale ;
					      dato il modo in cui  stato definito armDies[] , 
					      armDies[nota] non fa distinzione ad esempio 
					  	  tra un C e un C# , nonostante in nota questa informazione ci sia */
		   {  char ckn=0 ;
		      
			  check->alteraz=0;
			  skip=0;
			  if (tester[frq] ==-1) ckn=1; // nota che vuole il beq.dro(Non ancora trovata)
			  /* controlla se la nota attuale  naturale e quindi , avendo gi stabilito
			      con il precedente if che essa pu essere influenzata dalle
			      alterazioni in armatura , ci vorr un bequadro affinch la nota venga
			      effettivamente interpretata naturale e non alterata */	
			  if (tester[frq]==0 && tester[frq-1]==1)  // Nota # per la quale ho gia trovato un beq.dro
			      ckn=-1;
			  check->alteraz=-ckn;
			  tester[frq]=ckn;
		   }/* nota norm alterata : ckn = 0 alter = 0 ; nota bequadro : ckn = 1 alter = -1 
                tester[frq] = 1 ; gi bequadro : ckn = -1 alter = 1 tester[frq] = -1 */
                  
		   // Armatura di b che influenza la nota attuale
		   if (armKey < 0 &&armBem[nota]<=(-armKey) )
		   {
		      check->alteraz=0;
			  skip=0;
			  
			  if (tester[frq]==0)   // non e' da bequadro
			       check->altezza +=1; /* fa in  modo che se siamo in presenza di armatura in 
				    bemolle ad esempio una nota Bb/A# venga interpretata come Bb e quindi
					posizionata sul terzo rigo invece che nel secondo spazio (viene 
			         aumentata l'altezza non la frequenza) */
			  else if (tester[frq]==-1) // da beq NON ancora trovata
			          {  check->alteraz=-1; /* quindi sembrerebbe che alterazione abbia il
                      significato : inserisci #/b se = 1 , inserisci bequadro se = -1 , non 
					  inserire niente se = 0 ? */ 
					  tester[frq]=1;
					  }
		   }
		   if (armKey==0 || skip) // Nesuna armatura o nota NON influenzata dall'armatura
			   /* skip viene inizializzato = 1 in c374 */
		   {  
			  if (tester[frq]==-1 && tester[frq+1]==1) 
				  // nota naturale per la quale ho trovato una #
				  /* probabilmente il fatto di aver trovato un'alterazione temporanea
				  viene contraddistinto dal mettere tester[frq]=1 , in modo che se 
				  si trover ancora nella stessa battuta la nota [frq] non si aggiunga
				  di nuovo # o b perch deve essere sottinteso */
			  {
			    check->alteraz=-1;  // mettila bequadro
				tester[frq+1]=0;         // annulla l'effetto del preced. #
			  }
			  if (check->alteraz==1 && tester[frq]==1) // nota # gia trovata
			      check->alteraz=0;                                // non mostrare l'alterazione #
			  if (tester[frq]==0) tester[frq]=1;                 // nota # mai trovata, ricordati che ora c'e'
	          /* se nella battuta viene trovata per la prima volta un'alterazione temporanea
			      pone tester[frq]=1 , vedi nota a c433 */
		   } 
	   }                 /* d11 END */
	   check=check->next;
	}   
}

void MidiImport::Sdoppia (ptrNotaChunk nota1,ptrNotaChunk nota2,char v1,char v2,char pAcc)
{
    ptrNotaChunk   hide1, hide2;
 
 // nota1 coincide con la precedente e nota2 con la nota corrente
 // v1, v2 sono le posizioni dei valori quantizzati da assegnare a nota1 e nota2
 // pAcc e' la posizione dell'accordo che si viene a creare.
 
	if ( ! nota1->inAcc)  // non ho la prec in accordo !!
	{ if (pAcc==-1)   /* significato di pAcc=-1 ? */
	  {
	    hide1=AddNota( nota2->freq, nota2->chan, nota2->timbro,0);
		if (hide1)
		{   nota1->valore=centralTick[v1]; /* centralTick  stato inizializzato in c194 */
		     if (nota2->type !=LEGATA) nota2->type=PRIMALEGATA;
			 hide1->type=LEGATA;
			 hide1->attacco=hide1->inizio=nota1->inizio+nota1->valore;
			 hide1->valore=centralTick[v2];
			 nota2->inizio=nota1->inizio;
			 nota2->valore=nota1->valore;
			 nota1->inAcc=nota2->inAcc=1;
			 hide1->next=nota2->next;
			 nota2->next=hide1;
		} 
		else 
		{
//APRINTF			puts("Memory foult!! "); 
			exit(1); 
		}
	  }
	  else
	  {
		hide1=AddNota( nota1->freq, nota1->chan, nota1->timbro,0); // OCCHIO NOTE FITTIZIE !
		if (hide1)
		{    nota1->valore=centralTick[v1];
			 if (nota1->type !=LEGATA) nota1->type=PRIMALEGATA;
			 hide1->type=LEGATA;
			 hide1->attacco=hide1->inizio=nota1->inizio+nota1->valore;
			 hide1->valore=hide1->durata=centralTick[v2];
			 if (pAcc==1)
			 {	nota2->inizio=nota1->inizio;
				 nota2->valore=nota1->valore;
				 nota1->inAcc=nota2->inAcc=1;
				 hide1->next=nota2->next;
				 nota2->next=hide1;
			 }
			 if (pAcc==2)
			 {   nota2->inizio=hide1->inizio;
				 nota2->valore=hide1->valore;
				 hide1->inAcc=nota2->inAcc=1;
				 nota1->next=hide1;
				 hide1->next=nota2;
			 }
			 if (pAcc==0)
			 {  nota2->inizio=nota1->inizio;
				 nota2->valore=nota1->valore;
				 nota1->inAcc=nota2->inAcc=1;
				 nota2->type=PRIMALEGATA;
				 hide2=AddNota(nota2->freq, nota2->chan, nota2->timbro,0);
				 if (hide2)
				 {  hide2->inizio=hide1->inizio;
					hide2->valore=hide1->valore;
					hide2->inAcc=hide1->inAcc=1;
					hide2->type=LEGATA;
					hide2->next=nota1->next;
					hide1->next=hide2;
					nota1->next=hide1;
				 }else 
				 {
//APRINTF					 puts("NO MORE MEMORY !! "); 
					 exit(1);}
			 }
		}
		else 
		{
//APRINTF			puts("NO MORE MEMORY !! "); 
			exit(1);
		}
	  } // di pacc<0	
	}
}

void MidiImport::Triplica(ptrNotaChunk nota1,ptrNotaChunk nota2,char v1,char v2,char v3)
{
    ptrNotaChunk   hide1, hide2;
	
    if (! nota1->inAcc)  // ALLA FINE QUESTO E' DA LEVARE !!
	{	hide1=AddNota(nota1->freq, nota1->chan, nota1->timbro,0); // OCCHIO NOTE FITTIZIE !
		hide2=AddNota(nota2->freq, nota2->chan, nota2->timbro,0);

		if (hide1 && hide2)
		{
			nota1->valore=centralTick[v1];
			if (nota1->type !=LEGATA) nota1->type=PRIMALEGATA;
			
			hide1->valore=centralTick[v2];
			hide1->inizio=hide1->attacco=nota1->inizio+nota1->valore;
			hide1->type=LEGATA;
			hide1->inAcc=hide2->inAcc=1;
			hide2->inizio=hide2->attacco=hide1->inizio;
			hide2->valore=hide1->valore;
			hide2->type=PRIMALEGATA;
			nota2->valore=centralTick[v3];
			nota2->inizio=hide2->inizio+hide2->valore;
			nota2->type=LEGATA; // sempre ??
			nota1->next=hide1;
			hide1->next=hide2;
			hide2->next=nota2;
		}
		else 
		{
//APRINTF			puts("NO MORE MEMORY !! "); 
			exit(1);
		}
	}	
}


//  ---- Assegna i valori della Note in base alle loro durate in ticks -----
unsigned long MidiImport::Quantize(unsigned long tic, char ix) /* a cosa serve ix ? */
{
	register unsigned char cercoT;
	unsigned long value;
    int 	                cercoLog=0,cercoG=0, gauss2=0;
	float		           maxG1=0.0,  maxG2=0.0, maxL=0.0;
	char                  G1[]={0,0,1,1,2,2,3,4,4,5}, 
	                          LNor[]={0,1,1,2,2,-1,3,3,-1,4}, /* i "-1" corrispondono a 
							                                      1/4pp ed a 1/2pp */ 
							  G2[]={-1,-1,-1,-1,-1,3,-1,-1,5,-1},
							  TOP[]={3, 5, 7,10,13}; /* vettore delle posizioni delle note
													 mappate con la lognormale */
											
	value=centralTick[POS128];
	for(cercoT=0; cercoT<NUMNOTE && tic>centralTick[cercoT]; cercoT++);
	
	if ( (cercoT > POS16) && cercoT < NUMNOTE) /* POS16 = 4 */
	{
		cercoG=G1[cercoT-POS16p];
		gauss2=G2[cercoT-POS16p];
		cercoLog=LNor[cercoT-POS16p];
		
		if (cercoLog >=0) /* esclude il caso di cercoLog 1/4pp e 1/2pp */
		{     
			 maxL=LogNorm(logNStart[cercoLog],muVect[cercoLog],(float)SIGMA,centralTick[TOP[cercoLog]]);
		     // Normalzzazione di scala 
			 /* logNStart[] ha dimensione 5 */
		     maxL=( LogNorm(logNStart[cercoLog],muVect[cercoLog],(float)SIGMA,tic) ) / maxL;
		}

		maxG1=(Gauss(gaussCenter[cercoG],varVect[cercoG], tic) );
		if (gauss2>=0) /* cio se cercoT rappresenta 1/4pp o 1/2pp */
		    maxG2=(Gauss(gaussCenter[gauss2],varVect[gauss2],tic) );
		
		if (MAX( maxG1, maxL))
		{	if (MAX( maxG2, maxL))  
		    { value=centralTick[TOP[cercoLog]];
			    //xxx->attendibile=maxL;
			}
			else
			{ 
			  value=gaussCenter[gauss2]; //app4;
			  //xxx->attendibile=maxG2;
			}
		} 
		else {
			if (MAX(maxG1, maxG2)) 
			 {value=gaussCenter[gauss2];//app4;
			  //xxx->attendibile=maxG2;
			 }
			else 
			  {value=gaussCenter[cercoG];//app2;
			   //xxx->attendibile=maxG1;
			  }
		}
	}
	else {
	 	if (cercoT <= POS16 && cercoT)   //  Fino ad  1/16
  	    {	cercoT-=(1-NEAR(tic-centralTick[cercoT-1],centralTick[cercoT]-tic)); /* NEAR 
			 definito in h29 ed  1 se a>b ? */
		     /* notare che a note con durata <= 1/16 si assegna il valore mediante semplice 
		         quantizzazione */
 		    value=centralTick[cercoT];  /* problema quando  cercoT=0 perch si potrebbe avere
										cercoT=-1 con relativi problemi quando si pone value=
										centralTick[cercoT] , in realt il problema non si
			                            pone perch in c609  specificata la condizione
		                                cercoT !=0 */
		}
	    else if (cercoT==NUMNOTE )
	    {	
		    if (tic <= ticksXBattuta+centralTick[POS32])
			    value=centralTick[NUMNOTE-1]; /* ricordarsi che centralTick  un vettore
				 di N elementi il cui indice varia quindi da 0 a N-1 */
//APRINTF			else {	
//APRINTF              printf("  %d  \n",ix);
//APRINTF		      printf("ERRORE DI IDENTIFICAZIONE: Tics=%lu -- value=%lu\n", tic,value);
//APRINTF		      getchar();
//APRINTF			}
	    }
	}
	
	return value;
}


Nota * MidiImport::NewNotaMidi(char pos, int& numSpazi)
{
	Nota              *PFigura=NULL;
	PuntoValore * PVal=NULL;
	
	switch(pos) 
	{
	   case 0: PFigura=new NFusa();
	   		   PFigura->SetCoda(TRUE); /* SetCoda  un metodo di Nota e agisce sul bool Coda
									       che indica se la nota va disegnata con la coda */
			   //numSpazi=0;  
			   break;
	   case 1: PFigura=new NSemibiscroma();
	   		   PFigura->SetCoda(TRUE);
			   //numSpazi=0; 
			   break;
	   case 2: PFigura=new NBiscroma();
	   		   PFigura->SetCoda(TRUE);
			   //numSpazi=0; 
			   break;
	   case 3: PFigura=new NSemicroma();
	   		   PFigura->SetCoda(TRUE);
			   //numSpazi=1; 
			   break;
	   case 4: PFigura=new NSemicroma();
			   PVal=new PuntoValore;
			   PVal->SetNumPunti(1);
	   		   PFigura->SetCoda(TRUE);
			   PFigura->SetPPuntoValore(PVal);  /* la funzione SetPPuntoValore appartiene
			   									alla classe figura */
			   //numSpazi=2;
			   break;
	   case 5: PFigura=new NCroma();
	   		   PFigura->SetCoda(TRUE);
			   //numSpazi=3; 
			   break;
	   case 6: PFigura=new NCroma();
			   PVal=new PuntoValore;
			   PVal->SetNumPunti(1);
			   PFigura->SetPPuntoValore(PVal);
	   		   PFigura->SetCoda(TRUE);
			   //numSpazi=5;
			   break;
	   case 7: PFigura=new NSemiminima();
			   //numSpazi=7; 
			   break;
	   case 8: PFigura=new NSemiminima();
			   PVal=new PuntoValore;
			   PVal->SetNumPunti(1);
			   PFigura->SetPPuntoValore(PVal);
			   //numSpazi=11; 
			   break;
// Questo case 9 e'  tolto, in quanto le note con 2 punti vengono
// suddivise in piu note...
       case 9: PFigura=new NSemiminima();
			   PVal=new PuntoValore;
			   PVal->SetNumPunti(2);
			   PFigura->SetPPuntoValore(PVal);
			   //numSpazi=11; 
			   break;
	   case 10: PFigura=new NMinima();
			   //numSpazi=15; 
			   break;
	   case 11: PFigura=new NMinima();
			   PVal=new PuntoValore;
			   PVal->SetNumPunti(1);
			   PFigura->SetPPuntoValore(PVal);
			   //numSpazi=23;
			   break;
       case 12: PFigura=new NMinima();
			   PVal=new PuntoValore;
			   PVal->SetNumPunti(2);
			   PFigura->SetPPuntoValore(PVal);
			   //numSpazi=23;
			   break;
// Questo case 12 viene tolto, in quanto le note con 2 punti vengono
// suddivise in piu note...
	   case 13:PFigura=new NSemibreve();
			   //numSpazi=31;
			   break;
	}
	numSpazi=0;
	PFigura->SetHeadType(HEAD_CLASSIC);
	return PFigura;
}	


Pausa * MidiImport::PausaMidi(char pos, int& numSpazi)
{
	Pausa          * PFigura=NULL;
	PuntoValore * PVal=NULL;
	
	switch(pos)
	{
	   case 0: PFigura=new PFusa();
	   		   PFigura->SetAltezza(0);
			   //numSpazi=0; 
			   break;
	   case 1: PFigura=new PSemibiscroma();
	   		   PFigura->SetAltezza(0);
			   //numSpazi=0; 
			   break;
	   case 2: PFigura=new PBiscroma();
	   		   PFigura->SetAltezza(0);
			   //numSpazi=0; 
			   break;
	   case 3: PFigura=new PSemicroma();
	   		   PFigura->SetAltezza(0);
			   //numSpazi=1; 
			   break;
	   case 4: PFigura=new PSemicroma();
			   PVal=new PuntoValore;
			   PVal->SetNumPunti(1);
			   PFigura->SetPPuntoValore(PVal);
	   		   PFigura->SetAltezza(0);
			   //numSpazi=2; 
			   break;
	   case 5: PFigura=new PCroma();
	   		   PFigura->SetAltezza(2);
			   //numSpazi=3; 
			   break;
	   case 6: PFigura=new PCroma();
			   PVal=new PuntoValore;
			   PVal->SetNumPunti(1);
			   PFigura->SetPPuntoValore(PVal);
	   		   PFigura->SetAltezza(2);
			   //numSpazi=5; 
			   break;
	   case 7: PFigura=new PSemiminima();
	   		   PFigura->SetAltezza(1);
			   //numSpazi=7; 
			   break;
	   case 8: PFigura=new PSemiminima();
			   PVal=new PuntoValore;
			   PVal->SetNumPunti(1);
			   PFigura->SetPPuntoValore(PVal);
	   		   PFigura->SetAltezza(1);
			   //numSpazi=11; 
			   break;
       //	case 9 vuoto perche' le 1/4pp sono state scomposte
       case 9: PFigura=new PSemiminima();
			   PVal=new PuntoValore;
			   PVal->SetNumPunti(2);
			   PFigura->SetPPuntoValore(PVal);
	   		   PFigura->SetAltezza(1);
			   //numSpazi=11; 
			   break;
	   case 10: PFigura=new PMinima();
	   		   PFigura->SetAltezza(4);
			   //numSpazi=15; 
			   break;
       case 11: PFigura=new PMinima();
			   PVal=new PuntoValore;
			   PVal->SetNumPunti(1);
			   PFigura->SetPPuntoValore(PVal);
	   		   PFigura->SetAltezza(4);   
			   //numSpazi=23; 
			   break;
	   //	case 12 vuoto perche' le 2/4pp sono state scomposte
	   case 12: PFigura=new PMinima();
			   PVal=new PuntoValore;
			   PVal->SetNumPunti(2);
			   PFigura->SetPPuntoValore(PVal);
	   		   PFigura->SetAltezza(4);   
			   //numSpazi=23; 
			   break;
	   case 13:PFigura=new PSemibreve();
	   		   PFigura->SetAltezza(6);     /* rivedere altezza nei casi 11 e 13 */
			   //numSpazi=31; 
			   break;
	}
	numSpazi=0;
	return PFigura;
}	


// --- Scompone le note (pause) doppio puntate in tre elementi distinti ----

void MidiImport::Split ( ptrNotaChunk *prev, ptrNotaChunk current , char isNota)
{
	ptrNotaChunk hideNote;


	if (isNota) hideNote=AddNota(current->freq, current->chan, current->timbro,0);
	else hideNote=AddNota(0,0,0,0); 
	if (hideNote)
	{	if (( *prev) )  (*prev)->next=hideNote;
		else              topOfList=hideNote;
		hideNote->next=current;
		hideNote->attacco=current->attacco;
		hideNote->inizio=current->inizio;
        (*prev) =hideNote;

        if (isNota) hideNote=AddNota(current->freq, current->chan, current->timbro,0);
	    else hideNote=AddNota(0,0,0,0); 
		if (hideNote)
		{     (*prev)->next=hideNote;   // qui di sicuro prev esiste
			   hideNote->next=current;
	   
			   if (isNota)
			   {   
					(*prev)->type=PRIMALEGATA;   
					if (current->type==LEGATA) // INIZIO nota legata
   			               (*prev)->type=LEGATA;
			        hideNote->type=LEGATA; 
					current->type=LEGATA;  // sono divenute come note legate.
					(*prev)->alteraz=current->alteraz;
					hideNote->alteraz=current->alteraz;
					
					hideNote->altezza=current->altezza;
					(*prev)->altezza=current->altezza;
					
			       if ( current->notaPos==POS16p + 5) 		//1/4pp
				   {    (*prev)->valore=centralTick[POS4];
					    (*prev)->notaPos=POS4;
						hideNote->valore=centralTick[POS8];
						hideNote->notaPos=POS8;
						current->valore=centralTick[POS16];
						current->notaPos=POS16;
						
					}
					else 	//2/4pp
					{  (*prev)->valore=centralTick[POS2_4];
					   (*prev)->notaPos=POS2_4;
					   hideNote->valore=centralTick[POS4];
					   hideNote->notaPos=POS4;
					   current->valore=centralTick[POS8];
					   current->notaPos=POS8;
					}
					
		            (*prev)->pausa=current->pausa;
					(*prev)->pauPos=current->pauPos;
					current->pausa=0;
		            current->pauPos=0;
					
					hideNote->inizio=(*prev)->inizio+(*prev)->valore;
					hideNote->attacco=hideNote->inizio;
					current->inizio=hideNote->inizio+hideNote->valore;
					current->attacco=current->inizio;  //SPERIAMO BENE DI NON SFORARE !!!
					
					(*prev)=hideNote;
			   }
			   else 
			   {   (*prev)->type=FITTIZIA;
			       hideNote->type=FITTIZIA;
			       hideNote->attacco=current->attacco;
		           hideNote->inizio=current->inizio;

				   if (  current->pauPos==POS16p + 5) 		//1/4pp
				   {	(*prev)->pausa=centralTick[POS4];
				        (*prev)->pauPos=POS4;
						hideNote->pausa=centralTick[POS8];
						hideNote->pauPos=POS8;
						current->pausa=centralTick[POS16];
						current->pauPos=POS16;
					}
					else //2/4pp
					{  (*prev)->pausa=centralTick[POS2_4];
					   (*prev)->pauPos=POS2_4;
					   hideNote->pausa=centralTick[POS4];
					   hideNote->pauPos=POS4;
					   current->pausa=centralTick[POS8];
					   current->pauPos=POS8;
					}
					(*prev)=hideNote;
				}	
		}		
	}	
}

//  ------------ Ritorna a ScanTrx() il  Numero pesato di errori riscontrati  -----------

char MidiImport::CreateBatt(Spartito *PSpa, unsigned int bat,unsigned int more)// anche qui c'era chan.
{
	register unsigned char trova;
	char                      Errori=0;        
	unsigned long     somQtz=0, tempTic=0,lastStart=0 , surplus=0;
	unsigned char	  primo=1, isAccordo=0, isLunga=0, checkDelay=0, 
		stop=0, jump=0; //, trova;
	static ptrMarker mark4ZeroTrk; //, mark4ThisTrk;
	
	ptrNotaChunk   	    parse, prev,succ=NULL,
	hideNote=NULL, firstNotaAcc=NULL,lastNotaAcc=NULL ;
	Battuta                   * PBattuta=NULL;
	
	
    // se questa e' l'ultima battuta aggiungi una pausa finealla . Questo perche'
	// se la traccia finisse con pause, non avendo piu NoteON non potrei metterle.
	if (!more)
	{
		hideNote=AddNota(0,0,0,1); //1 finale-> accoda la nota
		if (hideNote) 
		{	
			hideNote->type=FITTIZIA;
			hideNote->attacco=ticksXBattuta-1;
			hideNote->inizio=ticksXBattuta-1;
		}		
	}
	
	if (bat==1)  
		mark4ZeroTrk=topZeroMark;  /* dove viene definito il valore di topZeroMark ? */
	
	   // impostazione di TEMPO BATTUTA
	   
	if (mark4ZeroTrk && mark4ZeroTrk->batNum==bat)
	{
		if ( mark4ZeroTrk->ev58Num || mark4ZeroTrk->ev58Den)
		{
			// SOLO se DIVERSO  dal  TEMPO ATTUALE di battuta...
			if ((mark4ZeroTrk->ev58Num !=tempoNum) || (mark4ZeroTrk->ev58Den !=tempoDen))
			{
				if (tempoNum != 0) 
				{
//APRINTF					puts("***** Variazione TEMPO (0) !! ****\n");
					tempoNum=mark4ZeroTrk->ev58Num;
					tempoDen=mark4ZeroTrk->ev58Den;
					ticksXBattuta=(unsigned long)( (4.0 / tempoDen)*tempoNum*ppqn);
//APRINTF					printf("N=%d  D=%d t=%lu\n" , tempoNum, tempoDen, ticksXBattuta);
				} 
			}
		}
	}	
	prev=NULL;
	parse=topOfList; 
	

	while (parse && !stop) 
	{	
		WTRACE();
		succ=parse->next;
		jump=0;
		
		if ( parse->attacco < (ticksXBattuta-(centralTick[POS64]+centralTick[POS128])) )
		{  
			//printf(" Dentro 1  ");
			if (parse->durata >=centralTick[POS64]+centralTick[POS128]) 
			{ 
				if ( parse->durata<=ticksXBattuta+centralTick[POS32])
					parse->valore=Quantize(parse->durata,1);
				else 
					isLunga=1;
			}
			else    // (parse->durata < centralTick[POS64]+centralTick[POS128]) 
			{   
				parse->valore=centralTick[POS64];
				if (parse->type==LEGATA) 
					parse->valore=centralTick[POS32];
			}
			if ((succ)&&((succ->attacco-parse->attacco)<CHORDLEAP)) parse->inAcc=1; /* aggiunta */
			
			if ( parse->attacco >(lastStart+CHORDLEAP ) )//centralTick[POS64]
			{
				//  printf("Dentro 2 ");
				lastNotaAcc=NULL;
				firstNotaAcc=NULL;
				
				tempTic=labs(parse->attacco-somQtz); /* definizione originale di temptic */
				if (tempTic <= CHORDLEAP) // qui chordleap non significa in accordo
				{     
					parse->inizio=somQtz;
					lastStart=parse->inizio;
				}	   
				else 
				{
					if ( parse->attacco >= somQtz) 
					{
						parse->pausa=tempTic; 
						/* aggiungi una pausa in coda ai ticks gi quantizzati */
						if (tempTic<logNStart[0]) /* la lognormale 0  quella che parte dal
												  trentaduesimo puntato e rappresenta il
												  sedicesimo */
						{  
							if (parse==topOfList) /* se la nota attuale  la prima allora non inserire pausa */
								parse->pausa=0;
							else  
							{
								//if (prev && prev->valore==centralTick[POS16p] && parse->valore!=centralTick[POS32])
								if (prev->valore==centralTick[POS16p] && parse->valore!=centralTick[POS32])
								{ 
									if (! prev->inAcc) 
									{
										prev->valore=centralTick[POS8];
										somQtz +=centralTick[POS32];  // aggiungo il 32simo
										parse->pausa=0;
										parse->inizio=somQtz; 
										/* in pratica si assegna alla precedente , che valeva 16p , 
										un valore 8 e non si introducono pause */
									} 
									else 
									{  
										ptrNotaChunk  appo=firstNotaAcc;
										
										if (appo)
										{
											while (appo && appo != parse) 
											{
												WTRACE();
												appo->valore=centralTick[POS8]; 
												/* stesso discorso di c981 ma ora per tutte le note dell'accordo di prev */
												appo=appo->next;
											}
											somQtz+=centralTick[POS32];
											parse->pausa=0;
											parse->inizio=somQtz;
//APRINTF											printf("allungato l'accordo di 16simi_p somQ:%lu\n",somQtz);
										}
									}
								} 
								//if ( prev && prev->valore<centralTick[POS32] && !prev->inAcc)
								if ( prev->valore<centralTick[POS32] && !prev->inAcc)
								{ 
									somQtz -=prev->valore;
									prev->valore=centralTick[POS32];
									somQtz+=prev->valore;
									parse->pausa=0;
									parse->inizio=somQtz; 
								}
								// se pausa!=0 non sono entrato in valore <central..[pos32])
								//if (prev && parse->pausa && prev->valore ==centralTick[POS32] && !prev->inAcc)
								/* NOTARE CHE  ANCORA VALIDA LA CONDIZIONE DI c966 */
								if (parse->pausa && prev->valore ==centralTick[POS32] && !prev->inAcc)
								{  
									if ( parse->valore != centralTick[POS32])
									{
										prev->valore=centralTick[POS16];
										somQtz +=centralTick[POS32]; // aggiungo un 32simo
										parse->pausa=0;
										parse->inizio=somQtz;
									}
									else if (labs(succ->attacco-parse->inizio) <=centralTick[POS64] || 
										succ->attacco > (somQtz+parse->pausa+parse->valore) )
									{
										prev->valore=centralTick[POS16];
										somQtz +=centralTick[POS32]; // aggiungo un 32simo
										parse->pausa=0;
										parse->inizio=somQtz;
									} 
								}
							} // questa e' di if parse==topOfList
							
							if (parse->pausa) 
								checkDelay=1;
							else 
							{ 
								parse->inizio=somQtz;
								lastStart=parse->inizio;
								//if ((somQtz+parse->valore) < ticksXBattuta-??)
								//{	somQtz+=parse->valore;
								//     jump=1;
								//}
							}
						} // di if tempTic<logNStart[0]
					}
					//--------------------------------------------------------------//
					/* prima era solo < !!!!?*/
					/* indica il caso parse->attacco<somQtz , vedi c963 */
					else  if ((somQtz < ticksXBattuta)||((somQtz >= ticksXBattuta) && (parse->inizio < ticksXBattuta)))  // e' una aggiunta !!!
					{  
						if ( prev->type==FITTIZIA || prev->valore<= centralTick[POS32] ) // ||  prev->inAcc )
							parse->inizio=somQtz;
						else 
						{  
							char   cerca, inCoda=0, parseInAcc=0,overleap=0;  // varra'  0 , 16 o 32.
							long   gap2Next=0;
							ptrNotaChunk  test,last=succ,app=succ;
							
//APRINTF							puts(" AREA di  BOOM !! "); 
							
//APRINTF							if (bat==25 || bat==26)
//APRINTF							{
								/* succedeva macello alla battuta 25 e 26 ? */
//APRINTF								printf("[Prev:(%d)  %d  Parse:(%d)  %d ]\n",prev->freq, prev->valore, parse->freq,
//APRINTF									parse->valore); 
//APRINTF								if (succ) 
//APRINTF									printf("SUCC=	 (%d)  %d\n", succ->freq, succ->durata);
//APRINTF								getchar();
//APRINTF							} 
							
							/* non dovrebbe esserci un - al posto di un / , ora che senso ha  ?? */
							if ( ((float)parse->attacco)>=(float(prev->inizio)+float((prev->valore)/2))) 
							{  
								inCoda=1;
								if ( ((prev->inizio+prev->valore)-parse->attacco) <centralTick[POS16p])
									overleap=16;
								if (((prev->inizio+prev->valore)-parse->attacco)<=(centralTick[POS32]+centralTick[POS64]))
									overleap=32;
							}
							
							//  se parseINacc==1  la nota attuale e la successiva sono probabilm.
							//  in accordo. Quindi devo ripercuotere le scente di parse anche su succ.
							if (succ && (succ->attacco-parse->attacco) <= CHORDLEAP)
								parseInAcc=1;
							
							// Skip delle note che sono "in accordo" 
							while( app && (app->attacco-parse->attacco)<=CHORDLEAP) 
							{
								WTRACE();
								app=app->next;
							}
							
							test=app; // test: prima nota NON "in accordo".
							
							// Elimino le note in sovrapposizione. 
							// Forse sarebbe meglio: ...< somQtz-CHORDLEAP ma me ne frego.
							while( app && (app->attacco<somQtz) && (app->attacco<(parse->attacco+parse->valore)))
							/* infatti bisogna verificare di essere ancora nella 
							condizione descritta in c1046 */
							{
								WTRACE();
								succ=app->next; /* succ  una variabile di appoggio */
								delete app; // ???
								app=succ;
							}
							last=app;            // memorizzo l'ultima delle note analizzate.
							if (last != test)
								test->next=last; // ricollego la lista di note.
							/* qui non bisognerebbe aggiornare parse->next = last ?? */
							parse->next = last;
							succ=parse->next;  //  resetto succ.
							
							// Analizzo l'attacco della prima nota che...
							// NotaBene:  gap2Next puo' essere anche NEGATIVO!
							if (last && last->attacco < ticksXBattuta) 
								gap2Next=last->attacco-somQtz;
							else 
								gap2Next=ticksXBattuta-somQtz; 
							
							for (trova=0; trova<NUMNOTE && prev->valore!=centralTick[trova]; trova++);
							for (cerca=0;cerca<NUMNOTE && parse->valore!=centralTick[cerca]; cerca++);
							
//APRINTF							if (bat==25 || bat==26) 
//APRINTF							{
//APRINTF								printf("[Prev:(%d)  %d  Parse:(%d)  %d ]\n",prev->freq, prev->valore, parse->freq,
//APRINTF									parse->valore); 
//APRINTF								if (succ) 
//APRINTF									printf("SUCC=	 (%d)  %d\n", succ->freq, succ->durata);
//APRINTF								getchar();
//APRINTF							}	   
							
							WTRACE();
							switch (trova)
							{
							case POS16 :   // prev = 1/16
								/* isLunga indica una nota pi lunga di una battuta */
								// parse >1/4 o lunga
								/* pone parse->inizio alla fine di prev */
								if (parse->valore > centralTick[POS4] || isLunga ) 
									parse->inizio=somQtz; 
								else
								{ 
									switch(cerca)
									{ 
									case POS32: // parse==32simo
										if (inCoda)  
										{  
											parse->inizio=somQtz;
											if ((somQtz+parse->valore) <= ticksXBattuta) 
											{   
												somQtz+=parse->valore;
												jump=1;
											} 
											else 
											{
												//puts("ERRORE MADORNALE !!!!"); 
												//exit(1);
												// somQtz=ticksxbattuta;......
												Errori=12; jump=1; stop=1;
											}
										}
										else
										{  
											NEXTDELPARSEJUMP;
										}
										break;
									case POS16:	// parse==16simo
										if ( inCoda ) 
										{  
											parse->inizio=somQtz;
											if ((somQtz+parse->valore) <=(ticksXBattuta-centralTick[POS32]) )
											/* perch da qui in poi ticksxbatt diventa 
											ticksxbatt-1/32 ? */
											{  
												somQtz+=parse->valore;
												jump=1;
											} 
										}
										else
										{    // metto parse e prev in accordo 
											PARSELASTACCORDOFIRST;
											if (firstNotaAcc == NULL) firstNotaAcc=prev;
										}
										break;
									case POS16p: // parse==1/16p
										if (inCoda)
										{ 
											parse->inizio=somQtz;
											if ( (somQtz+parse->valore) > ticksXBattuta )
												parse->valore=centralTick[POS16];
											if ((somQtz+parse->valore) <=(ticksXBattuta-centralTick[POS32]))
											{  
												somQtz += parse->valore;
												jump=1;
											}
											// se parseinAcc alla fine....
										}
										else 
										{ /* if not in coda */
											parse->valore=centralTick[POS16]; // accorcio parse
											PARSELASTACCORDOFIRST;
											if (firstNotaAcc == NULL) firstNotaAcc=prev;
										}
										break;
									case POS8: // parse==1/8
										if (inCoda)
										{ 
											parse->inizio=somQtz;
											if ((somQtz+parse->valore) <=(ticksXBattuta-centralTick[POS32]))
											{  
												somQtz += parse->valore;
												jump=1;
											}
										}
										else
										{
											Sdoppia(prev,parse,POS16,POS16, -1);
											if ( (somQtz+centralTick[POS16]) <= (ticksXBattuta-centralTick[POS32]))
											{ 
												somQtz +=centralTick[POS16];
												jump=1;
											}
										}
										break;
									case POS8p: // parse==1/8p
												/* non garantisce che 8p sia interno alla battuta , ma tanto
										non si aggiorna somQtz ? */
										if (gap2Next >= centralTick[POS8])
											parse->inizio=somQtz;
										else 
										{
											if (!prev->inAcc)
											{
												Sdoppia(prev,parse,POS16,POS8, -1);
												if ( (somQtz+centralTick[POS8]) <= (ticksXBattuta-centralTick[POS32]))
												{  
													somQtz +=centralTick[POS8];
													jump=1;
												}
											}
											else
											{   
												NEXTDELPARSEJUMP;
											}
											
										}
										// if parseinAcc.....
										break;
									case POS4: // parse==1/4
										parse->inizio=somQtz;
										if ((somQtz+parse->valore) <=(ticksXBattuta-centralTick[POS32]))
										{  
											somQtz+=parse->valore;
											jump=1;
										}
										// if parseinAcc.....
										
										break;
									default:   // parse e' 64simo  /* default ? */
										NEXTDELPARSEJUMP;
										break;
									} // DELLO SWITCH
							} // DELL'ELSE
							break;
							  case POS16p : // prev = 1/16p
								  if (parse->valore > centralTick[POS4] || isLunga ) // parse >1/4 o lunga
									  parse->inizio=somQtz;
								  else   
								  {
									  switch (cerca)
									  {   
									  case  POS32 :  //  parse e' 1/32
										  if (inCoda)
										  {  
											  if (gap2Next< 0)
											  {  
												  prev->next=parse->next;
												  /* qui ci saranno problemi perch parse->next non 
												  corretto , vedi c1098 */
												  delete parse; // oppure delete (NotaChunk)parse;
												  parse=prev; // cosi esamino succ al prossimo passaggio;
												  jump=1;
											  }
											  else 
											  {
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {  
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
										  }
										  else 
										  {
											  NEXTDELPARSEJUMP;
										  }
										  // if parseinAcc....
										  break;
									  case  POS16 :  //  parse e' 1/16
										  if (inCoda)
										  { 
											  // <=> "<=32simo" quantz.
											  if (gap2Next< logNStart[0] && ! prev->inAcc)   /* lognStart[0] = 32p  
												  e la sua lognormale rappresenta il sedicesimo */
											  { 
												  prev->valore=centralTick[POS16];
												  somQtz-=centralTick[POS32];
											  }
											  parse->inizio=somQtz;
											  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
											  {   
												  somQtz+=parse->valore;
												  jump=1;
											  }
										  }
										  else 
										  {
											  // allungo parse.
											  parse->valore=centralTick[POS16p];
											  PARSELASTACCORDOFIRST;
											  if (firstNotaAcc == NULL) firstNotaAcc=prev;
										  }
										  break;
									  case POS16p:  // parse e' 1/16p
										  if (inCoda)
										  {  
											  parse->inizio=somQtz;
											  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
											  {   
												  somQtz+=parse->valore;
												  jump=1;
											  }
										  }
										  else 
										  {
											  PARSELASTACCORDOFIRST;
											  if (firstNotaAcc == NULL) firstNotaAcc=prev;
										  }
										  // if parseinAcc........
										  break;
									  case POS8:    //  parse e' 1/8
										  if (inCoda)
										  {  
											  /* overleap indica di quanto all'incirca sono sovrapposte parse e prev */
											  if (! prev->inAcc && overleap==16) 
											  {
												  prev->valore=centralTick[POS16];
												  somQtz -=centralTick[POS32];
											  }
											  parse->inizio=somQtz;
											  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
											  {   
												  somQtz+=parse->valore;
												  jump=1;
											  }
										  }
										  else 
										  {
											  if (!prev->inAcc)
											  { 
												  prev->valore=centralTick[POS8]; // allungo prev
												  somQtz+=centralTick[POS32];
											  }
											  else 
												  parse->valore=centralTick[POS16p]; // accorcio parse
											  
											  PARSELASTACCORDOFIRST;
											  if (firstNotaAcc == NULL) firstNotaAcc=prev;
											  
										  }
										  // if parseinAcc........
										  break;
									  case POS8p:   //  parse e' 1/8p
										  if (gap2Next < centralTick[POS16p] && ! prev->inAcc)
											  prev->valore=centralTick[POS16];
										  
										  parse->inizio=somQtz;
										  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
										  {   
											  somQtz+=parse->valore;
											  jump=1;
										  }
										  // if parseinAcc........
										  break;
									  case POS4:    //  parse e' 1/4
										  if (gap2Next < centralTick[POS16p] && ! prev->inAcc)
											  prev->valore=centralTick[POS16];
										  
										  parse->inizio=somQtz;
										  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
										  {   
											  somQtz+=parse->valore;
											  jump=1;
										  }
										  break;
										  // 											  case centralTick[POS4p]:   //  parse e' 1/4p
										  // 														break;
										  // 											  case centralTick[POS4pp]:   //  parse e' 1/4pp
										  // 														break;
										  // 											  case centralTick[POS2_4]:   //  parse e' 2/4
										  // 														break;
										  
										  // se parse->valore <= XXsimo
									  default : 
										  NEXTDELPARSEJUMP;
										  break;
										  
										   }
									   }   // dell' else...
									   break;
							  case POS8: // prev e' 1/8
								  if ( parse->valore > centralTick[POS4p] || isLunga)
									  parse->inizio=somQtz;
								  else
								  { 
									  switch (cerca)
									  {   
									  case  POS32:  //  parse e' 1/32
										  if (inCoda)
										  { 
											  if (overleap==16 || overleap==0)
											  {  
												  NEXTDELPARSEJUMP;
											  }
											  else 
											  {  /* in che situazione pu essere vero questo else
												 se parse  32 e quindi non si pu sovrapporre pi
												  di 32 ? */
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {  
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
										  }
										  else
										  {
											  NEXTDELPARSEJUMP;
										  }
										  break;
									  case POS16:  // parse e' 1/16
										  if (inCoda)
										  {  
											  if ( overleap==16)  
											  {
												  if (!prev->inAcc)
												  {
													  
													  //sdoppia controlla che prev NON sia in acc. 
													  Sdoppia(prev,parse, POS16, POS16, 2);
													  jump=1;
												  }
												  else
												  {   
													  NEXTDELPARSEJUMP;
												  }
											  }
											  else 
											  {
											  parse->inizio=somQtz; /* in questo caso la sovrapposizione  < 16
											  e quindi si accoda parse a prev */
											  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
											  {   
												  somQtz+=parse->valore;
												  jump=1;
											  }
											  }
										  }
										  else
										  {
											  if (prev->inAcc)  
											  {
												  NEXTDELPARSEJUMP;
											  }
											  else 
											  {
												  //sdoppia controlla che prev NON sia in acc.
												  Sdoppia(prev,parse,POS16,POS16,1);
												  somQtz-=centralTick[POS16]; /**/
												  jump=1;
											  }
										  }
										  break;
									  case POS16p:  // parse e' 1/16p
										  if (inCoda)
										  {  
											  if ( overleap==16)  
											  {
												  if (!prev->inAcc)
												  {
													  //sdoppia controlla che prev NON sia in acc. 
													  Sdoppia(prev,parse, POS16, POS16, 2);
													  jump=1;
												  }
												  else
												  {   
													  NEXTDELPARSEJUMP;
												  }
											  }
											  else 
											  {
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {  
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
										  }
										  else 
										  {
											  //i if NON in accordo ??
											  parse->valore=centralTick[POS8];
											  PARSELASTACCORDOFIRST;
											  if (firstNotaAcc == NULL) firstNotaAcc=prev;
										  }
										  break;
									  case POS8:    //  parse e' 1/8
										  if (inCoda)
										  {  
											  if ( overleap==16)  
											  {    
												  if (!prev->inAcc)
												  {
													  //triplica controlla che prev NON sia in acc. 
													  Triplica(prev,parse, POS16, POS16, POS16);
													  if ((somQtz+centralTick[POS16])<=(ticksXBattuta-centralTick[POS32]))
													  {  
														  somQtz +=centralTick[POS16];
														  jump=1;
													  }  
												  }
												  else
												  {   
													  NEXTDELPARSEJUMP;
												  }
											  }
											  else 
											  {
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  { 
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
										  }
										  else 
										  {
											  PARSELASTACCORDOFIRST;
											  if (firstNotaAcc == NULL) firstNotaAcc=prev;
										  }
										  break;
									  case POS8p:   //  parse e' 1/8p
										  if (inCoda)
										  { 
											  if (overleap==32)
											  {
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {   
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  if (overleap==16)
											  {    
												  if (!prev->inAcc)
												  {
													  //triplica controlla che prev NON sia in acc. 
													  Triplica(prev,parse, POS16, POS16, POS8);
													  if ((somQtz+centralTick[POS8])<=(ticksXBattuta-centralTick[POS32]))
													  {  
														  somQtz +=centralTick[POS8];
														  jump=1;
													  }
												  }
												  else
												  {   
													  NEXTDELPARSEJUMP;
												  }
											  }
										  }
										  else 
										  {
											  NEXTDELPARSEJUMP;
										  }
										  break;
									  case POS4:    //  parse e' 1/4
										  if (inCoda)
										  { 
											  if (overleap==32)
											  { 
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {   
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  if (overleap==16)
											  {    
												  if (!prev->inAcc)
												  {
													  //triplica controlla che prev NON sia in acc. 
													  Triplica(prev,parse, POS16, POS16, POS8p);
													  if ((somQtz+centralTick[POS8p])<=(ticksXBattuta-centralTick[POS32]))
													  {  
														  somQtz +=centralTick[POS8p];
														  jump=1;
													  }  
												  }
												  else
												  {   
													  NEXTDELPARSEJUMP;
												  }
											  }
										  }
										  else 
										  {
											  if (!prev->inAcc)
											  {
												  Sdoppia(prev,parse,POS8,POS8, -1);
												  if ( (somQtz+centralTick[POS8]) <= (ticksXBattuta-centralTick[POS32]))
												  { 
													  somQtz +=centralTick[POS8];
													  jump=1;
												  }
											  }
											  else
											  { 
												  NEXTDELPARSEJUMP;
											  }
										  }
										  break;
									  case POS4p:   //  parse e' 1/4p
										  if (inCoda)
										  {  
											  if (overleap==32)
											  {  
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {  
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  else 
											  {
												  NEXTDELPARSEJUMP;
											  }
										  } 
										  else 
										  { 
											  if (!prev->inAcc)
											  {
												  Sdoppia(prev,parse,POS8,POS4, -1);
												  if ( (somQtz+centralTick[POS4]) <= (ticksXBattuta-centralTick[POS32]))
												  { 
													  somQtz +=centralTick[POS4];
													  jump=1;
												  }
											  }
											  else
											  {  
												  NEXTDELPARSEJUMP;
											  }
										  }
										  break;
										  //											  case centralTick[POS4pp]:   //  parse e' 1/4pp
										  //														break;
										  //											  case centralTick[POS2_4]:   //  parse e' 2/4
										  //														break;
										  
										  // se parse->valore <= simo
									  default : 
										  NEXTDELPARSEJUMP;
										  break;
										   }
									   }   // dell' else...
									   break;
							  case POS8p:  // prev e' 1/8p
								  if (parse->valore > centralTick[POS4p] || isLunga)
									  parse->inizio=somQtz;
								  else
								  { 
									  switch (cerca)
									  {   
									  case  POS32 :  //  parse e' 1/32
										  if (inCoda)
										  { 
											  if (overleap==16 || overleap==0)
											  /* come fa ad essere ==16 se la nota
											   lunga 1/32 ? ; se overlap = 0 la sovrapposizione  molto
											  evidente e quindi overlap non viene settato */
											  { 
												  NEXTDELPARSEJUMP;
											  }
											  else
											  {
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  { 
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
										  }
										  else
										  {
											  NEXTDELPARSEJUMP;
										  }
										  break;
									  case  POS16 :  //  parse e' 1/16
										  if (inCoda)
										  {  
											  if ( overleap==32) 
											  {  
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {  
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  else
											  {
												  if (!prev->inAcc)
												  {
													  //sdoppia controlla che prev NON sia in acc. 
													  Sdoppia(prev,parse, POS8, POS16, 2);
													  jump=1;
												  }
												  else
												  {  
													  NEXTDELPARSEJUMP;
												  }
											  }
										  }
										  else 
										  {
											  if (prev->inAcc)  
											  {
												  NEXTDELPARSEJUMP;
											  }
											  else
											  {
												  //sdoppia controlla che prev NON sia in acc.
												  Sdoppia(prev,parse,POS16,POS8,1);
												  somQtz-=centralTick[POS8]; /**/
												  jump=1;
											  }
										  }
										  break;
									  case POS16p:  // parse e' 1/16p
										  if (inCoda)
										  {  
											  if ( overleap==32) 
											  {   
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {  
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  else
											  {
												  if (!prev->inAcc)
												  {
													  //sdoppia controlla che prev NON sia in acc. 
													  Sdoppia(prev,parse, POS8, POS16, 2);
													  jump=1;
													  
												  }
												  else
												  {  
													  NEXTDELPARSEJUMP;
												  }
											  }
										  }
										  else 
										  { /* notare che quando incoda=0 e prev  inacc
											  parse sparisce indipendentemente dal suo valore */
											  if (prev->inAcc)  
											  {
												  NEXTDELPARSEJUMP;
											  }
											  else 
											  {
												  //sdoppia controlla che prev NON sia in acc.
												  Sdoppia(prev,parse,POS16,POS8,1);
												  /* al posto di un 2 non ci vorrebbe un 1
												  dato che incoda=0 e quindi parse
												  andrebbe all'inizio , non alla fine ?? */
												  somQtz-=centralTick[POS8]; /**/
												  jump=1;
											  }
										  }
										  break;
									  case POS8:    //  parse e' 1/8
										  if (inCoda)
										  {  
											  if ( overleap==32) 
											  {  
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {   
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  else
											  {	//sdoppia controlla che prev NON sia in acc. 
												  Sdoppia(prev,parse, POS16, POS8, 2);
												  if (prev->inAcc)  
												  {
													  NEXTDELPARSEJUMP;
												  }
												  jump=1;
											  }
										  }
										  else
										  {
											  if (prev->inAcc)  
											  {
												  NEXTDELPARSEJUMP;
											  }
											  else 
											  {
												  //sdoppia controlla che prev NON sia in acc.
												  Sdoppia(prev,parse,POS8,POS16,1);
												  somQtz-=centralTick[POS16];
												  jump=1;
											  }
										  }
										  break;
									  case POS8p:   //  parse e' 1/8p
										  if (inCoda)
										  {   
											  if (overleap==32) 
											  { 
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {  
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  else
											  {   
												  if(!prev->inAcc)
												  {
													  //triplica controlla che prev NON sia in acc. 
													  Triplica(prev,parse, POS8, POS16, POS8);
													  if ((somQtz+centralTick[POS8])<=(ticksXBattuta-centralTick[POS32]))
													  {  
														  somQtz +=centralTick[POS8];
														  jump=1;
													  }  
												  }
												  else
												  {   
													  NEXTDELPARSEJUMP;
												  }
											  }
										  }
										  else 
										  {
											  //i if NON in accordo ??
											  parse->inizio=lastStart; // cioe' prev->inizio;
											  isAccordo=1;
											  parse->pausa=0;
											  if (firstNotaAcc == NULL) 
												  firstNotaAcc=prev;
										  }
										  break;
									  case POS4:    //  parse e' 1/4
										  if (inCoda)
										  { 
											  if (overleap==32) // o 16 ???
											  { 
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {  
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  else 
											  { /* io avrei messo triplica(
												  prev,parse,pos8,pos16,pos8pp) */
												  NEXTDELPARSEJUMP;
											  }
										  }
										  else 
										  {
											  if (! prev->inAcc)
											  {	  
												  prev->valore=centralTick[POS4];
												  PARSELASTACCORDOFIRST;
												  if (firstNotaAcc == NULL) firstNotaAcc=prev;
											  }
											  else
											  {
												  NEXTDELPARSEJUMP;
											  }
										  }
										  break;
									  case POS4p:   //  parse e' 1/4p
										  if (! inCoda)
										  {  	
											  if (!prev->inAcc)
											  {
												  Sdoppia(prev,parse,POS8,POS4, -1);
												  /* perch pos8 e non pos8p ? */
												  if ( (somQtz+centralTick[POS4]) <= (ticksXBattuta-centralTick[POS32]))
												  {
													  somQtz +=centralTick[POS4];
													  jump=1;
												  }
											  }
											  else
											  {   
												  NEXTDELPARSEJUMP;
											  }
										  }
										  else 
										  {
											  parse->inizio=somQtz;
											  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
											  { 
												  somQtz+=parse->valore;
												  jump=1;
											  }
										  }
										  
										  break;
										  
										  //  case centralTick[POS4pp]:   //  parse e' 1/4pp
										  // break;
										  // se parse->valore <= 64simo
									  default : 
										  NEXTDELPARSEJUMP;
										  break;
										   }
									   }   // dell' else prima di switch
									   break;
							  case POS4:  //prev e' 1/4
								  if (isLunga) 
									  parse->inizio=somQtz;
								  else
								  { 
									  switch (cerca)
									  { 
									  case  POS32 :  //  parse e' 1/32
//APRINTF										  puts("EHI EHI ..4. ");
//APRINTF										  printf("coda %d  leap %d cerca %d",inCoda,overleap,cerca);
										  
										  if (inCoda)
										  { 
											  if (overleap==16 || overleap==0)
											  {  
												  prev->next=parse->next;
												  
//APRINTF												  printf("Elaboro: parFre=%d preF=%d",parse->freq, prev->freq);
												  
												  delete parse; // oppure delete (NotaChunk)parse;
												  parse=prev; // cosi esamino succ al prossimo passaggio;
												  jump=1;
												  
											  }
											  else
											  {
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {   
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
										  }
										  else 
										  {
											  NEXTDELPARSEJUMP;
										  }
										  break;
									  case POS16:  //  parse e' 1/16
										  if (inCoda)
										  { 
											  if ( overleap==32) 
											  {  
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {  
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  if (overleap==16)
											  {	
												  if (!prev->inAcc)
												  {
													  //sdoppia controlla che prev NON sia in acc. 
													  Sdoppia(prev,parse, POS8p, POS16, 2);
													  jump=1;
												  }
												  else
												  {  
													  NEXTDELPARSEJUMP;
												  }
											  }
											  if (overleap==0) // cioe' over NON settato->molto sovrapposto...
											  {
												  NEXTDELPARSEJUMP;
											  }
										  }
										  else 
										  {
											  if (prev->inAcc)  
											  {
												  NEXTDELPARSEJUMP;
											  }
											  else 
											  {
												  //sdoppia controlla che prev NON sia in acc.
												  Sdoppia(prev,parse,POS16,POS8p,1);
												  somQtz-=centralTick[POS8p]; /**/
												  jump=1;
											  }
										  }
										  break;
									  case POS16p:  //  parse e' 1/16p
										  if (inCoda)
										  {  
											  if ( overleap==32) 
											  {  
												  parse->inizio=somQtz;
												  if (gap2Next <= centralTick[POS16])
													  parse->valore=centralTick[POS16];
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {   
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  if (overleap==16)
											  {	
												  if (!prev->inAcc)
												  {
													  //sdoppia controlla che prev NON sia in acc. 
													  Sdoppia(prev,parse, POS8p, POS16, 2);
													  jump=1;
												  }
												  else
												  {   
													  NEXTDELPARSEJUMP;
												  }
											  }
											  if (overleap==0) // cioe' over NON settato->molto sovrapposto...
												  /* NOTA BENE */
											  {
												  NEXTDELPARSEJUMP;
											  }
										  }
										  else
										  {
											  if (prev->inAcc)  
											  {
												  NEXTDELPARSEJUMP;
											  }
											  else 
											  {
												  //sdoppia controlla che prev NON sia in acc.
												  Sdoppia(prev,parse,POS16,POS8p,1);
												  somQtz-=centralTick[POS8p]; /**/
												  jump=1;
											  }
										  }
										  break;
									  case POS8: //  parse e' 1/8
										  if (inCoda)
										  {
											  if (overleap >=16 ) // anche 32
											  {
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  { 
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  else
											  {   
												  if (prev->inAcc)  
												  {  
													  NEXTDELPARSEJUMP;
												  }
												  else 
												  {
													  //sdoppia controlla che prev NON sia in acc.
													  Sdoppia(prev,parse,POS8,POS8,2);
													  jump=1;
												  }
											  }
										  } 
										  else 
										  {   
											  if (prev->inAcc)  
											  {
												  NEXTDELPARSEJUMP;
											  }
											  else
											  {
												  //sdoppia controlla che prev NON sia in acc.
												  Sdoppia(prev,parse,POS8,POS8,1);
												  somQtz-=centralTick[POS8]; /**/
												  jump=1;
											  }
										  }
										  break;
									  case POS8p:  //  parse e' 1/8p
										  if (inCoda)
										  {
											  if (overleap >=16)
											  { 
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  { 
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  else 
											  { 
												  if (! prev->inAcc)
												  {
													  Triplica(prev,parse,POS8,POS8,POS16);
													  if ((somQtz+centralTick[POS16])<=(ticksXBattuta-centralTick[POS32]))
													  { 
														  somQtz +=centralTick[POS16];
														  jump=1;
													  }  
												  }
												  else
												  {  
													  NEXTDELPARSEJUMP;
												  }
											  }
										  }
										  else 
										  {  
											  if (! prev->inAcc)
											  {  
												  Sdoppia(prev,parse,POS8p,POS16,1);
												  somQtz-=centralTick[POS16]; /**/
												  jump=1;
											  }
											  else
											  {  
												  NEXTDELPARSEJUMP;
											  }
										  }
										  break;
									  case POS4:  //  parse e' 1/4
										  if (inCoda)
										  { 
											  if (overleap >=16)
											  { 
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  { 
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  else 
											  {  
												  if (! prev->inAcc)
												  {
													  Triplica(prev,parse,POS8,POS8,POS8);
													  if ((somQtz+centralTick[POS8])<=(ticksXBattuta-centralTick[POS32]))
													  {   
														  somQtz +=centralTick[POS8];
														  jump=1;
													  }  
												  }
												  else
												  {   
													  NEXTDELPARSEJUMP;
												  }
											  }
										  }
										  else 
										  { 
											  if (! prev->inAcc)
											  {  
												  Triplica(prev,parse,POS8,POS8,POS8);
												  if ( (somQtz+centralTick[POS8])<=(ticksXBattuta-centralTick[POS32]))
												  {  
													  somQtz +=centralTick[POS8];
													  jump=1;
												  }  
											  }
											  else 
											  {   
												  NEXTDELPARSEJUMP;
											  }
										  }
										  break;
									  case POS4p:  //  parse e' 1/4p
										  if (inCoda)
										  { 
											  if (overleap >=16)  // cioe' sovrap. 1/16 o 1/32
											  {  
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  { 
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  else 
											  {   // overleap==0 =>sovrap. > 1/16
												  if (! prev->inAcc)
												  {  
													  Triplica(prev,parse,POS8,POS8,POS4);
													  if ( (somQtz+centralTick[POS4])<=(ticksXBattuta-centralTick[POS32]))
													  {  
														  somQtz +=centralTick[POS4];
														  jump=1;
													  }  
												  }
												  else 
												  {
												  /* notare chein questo caso
												  prev era in Acc per parse viene
													  solo accorciato , non eliminato */
													  parse->valore=centralTick[POS4]; // accorcio parse
													  parse->inizio=somQtz;
													  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
													  {  
														  somQtz+=parse->valore;
														  jump=1;
													  }
												  }
											  }
										  }
										  else {  // Non in coda...
											  if (! prev->inAcc)
											  {  
												  Triplica(prev,parse,POS8,POS8,POS4);
												  if ( (somQtz+centralTick[POS4])<=(ticksXBattuta-centralTick[POS32]))
												  { 
													  somQtz +=centralTick[POS4];
													  jump=1;
												  }  
											  }
											  else
											  {
												  NEXTDELPARSEJUMP;
											  }
										  }
										  break;
									  case POS4pp:  //  parse e' 1/4pp
										  if (inCoda)
										  {  
											  if (overleap >=16)  // cioe' sovrap. 1/16 o 1/32
											  {  
												  //SUDDIVIDERE  PARSE IN 2
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {   
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  else 
											  {   // overleap==0 =>sovrap. > 1/16
												  NEXTDELPARSEJUMP;
											  }
										  }
										  else
										  {
											  NEXTDELPARSEJUMP;
										  }
										  break;
									  case POS2_4:  //  parse e' 2/4
										  if (inCoda)
										  {  
											  if (overleap >=16)  // cioe' sovrap. 1/16 o 1/32
											  {  
												  parse->inizio=somQtz;
												  /* perch non viene aggiornato somqtz
												  come veniva fatto in casi simili prima ? */
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {   
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  else
											  {   // overleap==0 =>sovrap. > 1/16
												  if (! prev->inAcc)
												  {  
													  Triplica(prev,parse,POS8,POS8,POS4p);
													  if ( (somQtz+centralTick[POS4p])<=(ticksXBattuta-centralTick[POS32]))
													  {  
														  somQtz +=centralTick[POS4p];
														  jump=1;
													  }  
												  }
												  else 
												  {
													  NEXTDELPARSEJUMP;
												  }
											  }
										  }
										  else {
											  if (! prev->inAcc)
											  {	
												  Sdoppia(prev,parse,POS4,POS4, -1);
												  if ( (somQtz+centralTick[POS4]) <= (ticksXBattuta-centralTick[POS32]))
												  {  
													  somQtz +=centralTick[POS4];
													  jump=1;
												  }
											  }
											  else
											  {
												  NEXTDELPARSEJUMP;
											  }
										  }
										  break;
										  
										  // 										   case POS2_4p:  //  parse e' 2/4p
										  // 										            break;
										  // 										   case POS2_4pp:  //  parse e' 2/4pp
										  // 										            break; 
										  // 										   case POS4_4:  //  parse e' 4/4
										  // 										            break;
										  
									  default:
										  NEXTDELPARSEJUMP;
										  break;
										  }
									   }
									   break;
							  case POS2_4:  //prev e' 2/4
								  if (isLunga) 
									  parse->inizio=somQtz;
								  else
								  {  
									  switch (cerca)
									  { 
									  case POS32:
									  case POS16:
									  case POS16p:
										  if (inCoda)
										  { 
											  if (overleap==32)
											  { 
												  parse->inizio=somQtz;
												  if ((somQtz+parse->valore)<=(ticksXBattuta-centralTick[POS32]))
												  {   
													  somQtz+=parse->valore;
													  jump=1;
												  }
											  }
											  else 
											  {
												  NEXTDELPARSEJUMP;
											  }
										  }
										  else
										  {
											  NEXTDELPARSEJUMP;
										  }
										  break;
										  // 											case POS8: // parse e' 1/8
										  // 													 break;
										  // 											case POS8p: // parse e' 1/8p
										  // 													 break;
										  // 											case POS4:  // parse e' 1/4
										  // 													 break;
									  case POS2_4:  // parse e' 2/4
										  if (inCoda)
										  { 
											  if (overleap >=16)
												  parse->inizio=somQtz;
											  else 
											  {
												  if (! prev->inAcc)
												  { 
													  Triplica(prev,parse,POS4,POS4,POS4);
													  if ( (somQtz+centralTick[POS4])<=(ticksXBattuta-centralTick[POS32]))
													  {  
														  somQtz +=centralTick[POS4];
														  jump=1;
													  }  
												  }
												  else 
												  {
													  NEXTDELPARSEJUMP;
												  }
											  }
										  }
										  else 
										  {  // e' uguale a qui sopra caso non inacc
											  if (! prev->inAcc)
											  { 
												  Triplica(prev,parse,POS4,POS4,POS4);
												  if ( (somQtz+centralTick[POS4])<=(ticksXBattuta-centralTick[POS32]))
												  {   
													  somQtz +=centralTick[POS4];
													  jump=1;
												  }  
											  }
											  else 
											  {
												  NEXTDELPARSEJUMP;
											  }
										  }
										  break;
									  default:
										  NEXTDELPARSEJUMP;
										  break;
									  }
								  }  
								  break;
								  
								  // 							  case POS2_4p:  //prev e' 2/4p
								  // 									   if (isLunga) parse->inizio=somQtz;
								  // 									   else
								  // 									   {  switch (cerca)
								  // 									      { 
								  // 										  }
								  // 									   }  
								  // 									   break;
								  
							  default : 
								  NEXTDELPARSEJUMP;
								  break; //****************
						   }
						   lastStart=parse->inizio;
						}
					}
					
					/*************************************************************/					
				}
			}
			else  	// la nota inizia molto vicino all'inizio della prec.
			{	
				parse->inizio=lastStart;
				if (parse != topOfList)  // per escludere la prima nota della battuta.
				{  
					// Le note devono essere diverse. Anche se e' ovvio che lo siano
					// capita che si trovi nel file (per errori di editing) accordi
					// composti con le stesse note.
					// Cosi garantisco che almeno 2 note siano diverse.
					// Cio'  non significa che poi riesca ad inserire bene la terza...
					// ma dal punto di vista del file MDS funziona bene.
					
					if (prev->freq != parse->freq) 
					{	
						isAccordo=1;
						parse->pausa=0;
						if (firstNotaAcc == NULL) 
							firstNotaAcc=prev;
						//printf("::: pre(%p) parse(%p) firts(%p)\n",prev, parse, firstNotaAcc);
					}	
					else
					{
						NEXTDELPARSEJUMP;
					}
				}
			}
			if (! jump && (somQtz < ticksXBattuta || isAccordo) )
			{	
				//printf("Dentro Jump ");
				if (parse->pausa)  
				{	 
					parse->pausa=Quantize(parse->pausa,2); 
					/* a cosa serve specificare 2 se poi in quantize non viene usato ? */
					
					if (checkDelay)
					{  
						float  rp1=GAP+1, rp2=GAP+1;    /* GAP definito in h35 */ 
						
						checkDelay=0;
						if (prev)     
							rp1=(float)(prev->valore)/ parse->pausa; 
						rp2=parse->durata/parse->pausa;
						if (! isLunga) 
						{
							rp2=(float)(parse->valore)/ parse->pausa;
						}
						
						if ( (rp1 > GAP) &&  (rp2 > GAP) ) /* cio la pausa si trova tra due
							note molto pi lunghe di lei */
						{	   //somQtz -=parse->pausa;
							surplus =(surplus+parse->pausa) / 2;  // circa media degli errori
//APRINTF							printf(" Eliminata pausa molto breve. SURP=%d\n",surplus);
							//parse->inizio=somQtz;
							parse->pausa=0;
						}
					}	
					somQtz +=parse->pausa;
				}
				
				tempTic=0; /* era stato definito in precedenza */
				succ=parse->next;
				
				if (parse->pausa)
				{	  
					
					//---------PER PROVA  VEDERE QUANTO VALE-----
					if (parse->pausa==centralTick[POS16p]) 
					{ 
						if (succ && labs(succ->attacco-parse->inizio) > CHORDLEAP)
						{  
							parse->pausa=centralTick[POS16];
							parse->pauPos=POS16;
							somQtz -=centralTick[POS32];
							parse->inizio=somQtz;
							lastStart=somQtz;
							// pausa non viene messa ad un ottavo perche parse-inizio potrebbe
							// traslare dopo succ-attacco !  E CIO' NON E' BELLO.
						}
					}
					//---------------------------------------------
					
					if (prev && !prev->inAcc)
					{ 
						if ( prev->durata > prev->valore ) // Quant. per difetto
							tempTic=Quantize( (prev->durata-prev->valore), 3); /* il 3  inutile ? */
					}
					/*else if (prev)
					{    ptrNotaChunk  appo=firstNotaAcc;
					
					  - mettere un flag che indica che sono passato di qui !
					  - cercare tra le note quelle che possono essere allungate a dovere.
					  - allungare tutte le note ??
					  tempTic=Quantize( (prev->durata-prev->valore)); //PER ORA !!!
				}*/
					if ( parse->pausa==tempTic  &&  somQtz<=ticksXBattuta) 
					{  	
						hideNote=AddNota(prev->freq, prev->chan, 0, 0);// NON so il TIMBRO !!
						if (hideNote)
						{	
//APRINTF							printf("Sostituisco pausa con nota equivalente...");
							
							hideNote->inizio=prev->inizio+prev->valore;
							hideNote->attacco=hideNote->inizio;
							hideNote->valore=tempTic;
							parse->pausa=0;
							hideNote->next=parse;	 /*la fittizia all'attuale */
							if (prev->type !=LEGATA) 
								prev->type=PRIMALEGATA;
							hideNote->type=LEGATA;
							prev->next=hideNote;	/* La nota prec. punta a quella fittizia*/
							prev=hideNote;
						} 
						else 
						{
//APRINTF							puts("Can't allocate new note..."); 
							stop=Errori=12;
						}
					}
					if (parse->attacco >= somQtz+logNStart[0] ) /*Commento #1*/
					{	
//APRINTF						printf("Pausa di %lu NON adeguata. Correggo...\n",parse->pausa);
//APRINTF						printf("SomQ=%lu   parseattacco=%lu\n", somQtz, parse->attacco);
						
						hideNote=AddNota(0,0,0,0); 
						if (hideNote)
						{	
							hideNote->inizio=somQtz;
							hideNote->attacco=hideNote->inizio;
							hideNote->type=FITTIZIA;  			/* Nota non creata da evento Midi...*/
							hideNote->pausa=parse->pausa; 
							parse->pausa=Quantize(parse->attacco-somQtz,4); /* ancora 
							Quantize con un parametro inutile ? */
							hideNote->next=parse;	 /*la fittizia all'attuale */
							if (prev) 
								prev->next=hideNote;	/* La nota prec. punta a quella fittizia*/
							else 
								topOfList=hideNote;
							prev=hideNote;
							somQtz +=parse->pausa; /* nuova pausa da conteggiare*/
						}
					}
				} 		/* di if pausa...*/
				
				if (! isAccordo)
				{	
					//  se sforo ==> devo rientrare nei ticks di battuta...
					// METTERE UN WHILE (som>ticks..)
					
					if (somQtz > ticksXBattuta && parse->pausa) 
					{   somQtz-=parse->pausa;
					for (trova=0; trova<NUMNOTE && centralTick[trova] !=parse->pausa;trova++);
					if (trova) 
						trova--;
					parse->pausa=centralTick[trova]; // prendo la pausa prima (bastera' ?)
					somQtz += parse->pausa;
//APRINTF					puts(" Cirillo   "); /* ? */
					}
					
					if (somQtz==ticksXBattuta && parse->pausa) 
					{	hideNote=AddNota(0,0,0,0); 
					if (hideNote)
					{	
						hideNote->inizio=parse->inizio-1;
						hideNote->attacco=parse->attacco-1;
						hideNote->type=FITTIZIA;  			/* Nota non creata da evento Midi...*/
						hideNote->pausa=parse->pausa;
						parse->pausa=0; // PROVA
						hideNote->next=parse;	 /*la fittizia all'attuale */
						if (prev)
							prev->next=hideNote;	/* La nota prec. punta a quella fittizia*/
						else
							topOfList=hideNote;
						prev=hideNote;
//APRINTF						puts(" Cirillo 2  "); /* ? */
					}
					}
					parse->inizio=somQtz;
					lastStart=parse->inizio;
					
					if (somQtz < ticksXBattuta)
					{
						unsigned long tmp;
						
						succ=parse->next;     
						tempTic=ticksXBattuta+surplus;
						tmp=parse->attacco+parse->durata+10;
						
						if (succ&&(parse->inizio+parse->valore<ticksXBattuta))
						{ 
							while (succ && labs(succ->attacco-lastStart)<=CHORDLEAP) 
								succ=succ->next; 
							if (succ) 
								tmp=succ->attacco;
							// n note in accordo.
							succ=parse->next;       
							/* non dovrebbe essere parse->next =succ
							in modo che parse venga collegato alla prima nota non 
							in accordo ?? */
						}	
						
						// se isLunga allora valore==0 quindi sbaglierei a valutarlo.
						
						
						if ((somQtz+parse->valore <= tempTic) && ! isLunga)
						{	
							somQtz+=parse->valore; 	// Modifico somQtz  !!! 
							
							tempTic=parse->attacco+parse->durata-surplus;
							if (parse->durata < parse->valore) 
								tempTic=parse->attacco+parse->valore-surplus;
							
							if (tempTic>=(ticksXBattuta+centralTick[POS32]) && (tmp>(parse->attacco+parse->durata)))
							{	
//APRINTF								puts("Legatura (NON rilevata in quantizz.) a fine battuta...");
								
								hideNote=AddNota(parse->freq, parse->chan,parse->timbro,0);
								if (hideNote)
								{
									char ok=1;
									
									if (somQtz < ticksXBattuta) // < stretto cioe' escludo il caso ==
									{	
										tempTic=ticksXBattuta-somQtz; // SOLITO PROBLEMA !!!
										somQtz=ticksXBattuta;
										hideNote->pausa=tempTic; /* ...aggiungo una pausa, prima*/
										hideNote->type=FITTIZIA;
										hideNote->inizio=parse->inizio;
										hideNote->freq=0;
										hideNote->next=parse;
										if (prev)
											prev->next=hideNote;
										else 
											topOfList->next=hideNote;
										prev=hideNote;
										parse->inizio +=tempTic;// tempTic e' quantizz.
										lastStart=parse->inizio;
										firstNotaAcc=parse;
										hideNote=AddNota(parse->freq, parse->chan,parse->timbro,0);
										if (  hideNote == NULL)  
											ok=0;
									}
									if (ok) 
									{
										tempTic=Quantize(labs(parse->durata-parse->valore),5);
										hideNote->valore=tempTic;
										hideNote->durata=tempTic;
										hideNote->attacco=ticksXBattuta;
										hideNote->inizio=ticksXBattuta;
										hideNote->rilascio=parse->rilascio;
										hideNote->next=parse->next;
										parse->next=hideNote;
										if (parse->type!=LEGATA) 
											parse->type=PRIMALEGATA;
										hideNote->type=LEGATA;
										parse=hideNote;
									}
								}
								else { 
//APRINTF									puts("Can't Allocate Note");
									stop=Errori=12;
								}
							}
						}
						else  // cioe'  possibile  una legatura finale...
						{       // NON TOGLIRE l'  uguale  
							if ( (tmp >= (parse->attacco+parse->durata))  || isLunga)
							{
//APRINTF								printf("Legatura a fine battuta...somQtz=%lu\n",somQtz);
								
								hideNote=AddNota(parse->freq, parse->chan,parse->timbro,0);
								if (hideNote)
								{
									if ((succ)&&(succ->attacco>=ticksXBattuta)) 
									{
										hideNote->next=parse;
									}
									else 
									{   
										ptrNotaChunk dum=parse;
										hideNote->next=succ;
										while ((dum->next)&&(((dum->next)->attacco)<ticksXBattuta)) 
											dum=dum->next;
										parse->next=dum->next;
										dum->next=parse;
									}
									if (prev)   
										prev->next=hideNote;
									else         
										topOfList=hideNote;
									hideNote->pausa=parse->pausa;
									
									//hideNote->alteraz=parse->alteraz;
									parse->pausa=0;
									
									if (parse->type==LEGATA) 
										hideNote->type=LEGATA;
									else 
										hideNote->type=PRIMALEGATA;
									parse->type=LEGATA;
									
									hideNote->durata=ticksXBattuta-somQtz; // sono quantizzati.
									
									// hide->durata e' quantizz. , ma NON sono certo che sia una quant.
									//	che conosco ( per ex: 1/4+1/16 )
									for (trova=0;trova<NUMNOTE && centralTick[trova] !=hideNote->durata;trova++);
									if (trova==NUMNOTE)
									{   
										hideNote->durata = Quantize(hideNote->durata,6);
										for (trova=0;trova<NUMNOTE && centralTick[trova] !=hideNote->durata;trova++);
										while( (somQtz+centralTick[trova]) > ticksXBattuta  &&  trova)
											trova--;
										hideNote->durata=centralTick[trova];
									}
									parse->durata-=hideNote->durata;
									parse->valore=parse->durata;
									
									hideNote->valore=hideNote->durata;
									hideNote->attacco=somQtz;
									hideNote->inizio=somQtz;
									
									somQtz+=hideNote->valore;
									parse->attacco =somQtz;
									parse->inizio =parse->attacco;
									
									// serve per gli accordi legati a fine battuta: La legatura della prima nota legata
									//  della' accordo puo' essere scomposta in piu di 2 note (per motivi di quantiz).
									// (vedi for qui sopra).  Si deve far ripartire l' accordo dalla prima di tuttte.
									if (somQtz==ticksXBattuta) 
									{    
										lastStart=somQtz-hideNote->valore;
										firstNotaAcc=hideNote;
										// def=0;   /* aggiunta */
									}
									parse=hideNote;
									while (succ&&(succ->attacco<ticksXBattuta)&&(succ->attacco-parse->inizio>CHORDLEAP)) /* aggiunta*/
									{
										WTRACE();
										parse->next=succ->next;
										delete succ;
										succ=parse->next; //succ=NULL; questo nella vecchia versione
									}
									
								}
								
							}  
							else  // del if tmp >...: la legatura rilevata in quantizz., NON c'e' !
							{ 
								//NON DOVREBBE MAI APPARIRE
								if (isLunga) 
								{
//APRINTF									puts("...Ho raggiunto l'infinito !!");
//APRINTF									getchar();
									stop=1;
								}
								
								for (trova=0;trova<NUMNOTE && centralTick[trova]!=parse->valore;trova++);
								while( trova && parse->inizio+centralTick[trova]>ticksXBattuta) 
									trova--;
								if (trova)
								{  
									parse->valore=centralTick[trova];
									somQtz +=parse->valore;
								}
							}
						}
					}	  // di if somQtz<=ticksXBattuta
				}
				else 
				{   	// Piu note suonate insieme ==> suppongo sempre un accordo.
					// DA DECIDERE COSA FARE SE LE LUNGHEZZE NON CORRISP.
					ptrNotaChunk   curs;
					
					isAccordo=0;
					succ=parse->next;
					if (lastNotaAcc==NULL) 
						lastNotaAcc=firstNotaAcc;
					
					//printf("Dentro 11 F(%p) L(%p)", firstNotaAcc, lastNotaAcc);	    
					
					curs=lastNotaAcc->next;
					tempTic=lastNotaAcc->valore;
					
					if (parse->type==ATTIVA)
					{  
//APRINTF						puts("\t <ERROR !!> Working on a note that seem ACTIVE!!");
						/* infatti se parse  in accordo con la precedente non pu essere
						gi attiva ? non si capisce bene */ 
//APRINTF						puts("\t Identification Problems could occurs:");
//APRINTF						puts("\t Expect something  like: <NoteOff > message for Unknown NoteOn");
//APRINTF						puts("\t Forced <NoteOFF> Message to DEACTIVATE ....");
						
						parse->rilascio=60;
						parse->type=FINITA;
					}
					lastNotaAcc->inAcc=1;
					parse->inAcc=1;
					
					//		printf("ACCORDO:  1) %d	   2) %d  lst=%lu\n", lastNotaAcc->freq,parse->freq, lastStart);	
					
					/*	if (lastNotaAcc->valore<=centralTick[POS16] && parse->valore<=centralTick[POS16])
					{  if ( succ && (succ->attacco-somQtz)> CHORDLEAP) /* cio se succ non fa 
					parte dell'accordo in cui sono parse e lastNotaAcc */
					/*		 {
					somQtz-=lastNotaAcc->valore;
					if ((somQtz+centralTick[POS16])<ticksXBattuta) lastNotaAcc->valore=centralTick[POS16];
					somQtz+=lastNotaAcc->valore;
					}
				}	 */
					// NON VA BENE NEL CASO DI FALSI ACCORDI !
					if (((prev->type)==PRIMALEGATA)&&((prev->inizio)+(prev->valore)==ticksXBattuta))  // aggiunta
					{
                        ptrNotaChunk app=parse;
						while ((app->next)&&(((app->next)->attacco)<ticksXBattuta)) 
							app=app->next;
						while ((app->next)&&((app->next)->inAcc=1)&&((app->next)->type==LEGATA)) 
							app=app->next;
						hideNote=AddNota(parse->freq, parse->chan, parse->timbro,0);
						hideNote->next=app->next;
						app->next=hideNote;
						hideNote->inizio=hideNote->attacco=ticksXBattuta;
						hideNote->durata=hideNote->valore=app->valore;
						hideNote->inAcc=1;
						app->inAcc=1;
						hideNote->type=LEGATA;
					}
					
					
					parse->valore=parse->durata=lastNotaAcc->valore;
					
					if (lastNotaAcc->type == PRIMALEGATA)
						parse->type=PRIMALEGATA;
					
					if ( curs && curs != parse)
					{
						hideNote=AddNota(parse->freq, parse->chan, parse->timbro,0);// TIMBRO ??
						if (hideNote)
						{
//APRINTF							puts("Copio !!  ");
							hideNote->type=LEGATA;
							hideNote->inizio=curs->inizio;
							hideNote->attacco=curs->attacco; 
							hideNote->durata=curs->durata;
							hideNote->valore=curs->valore;
							hideNote->inAcc=curs->inAcc=1; // Le metto in accordo
							hideNote->next=parse->next;
							lastNotaAcc->next=parse; 
							parse->next=curs;
							while (curs->next !=parse) 
								curs=curs->next;
							curs->next=hideNote;
						}
					}
					lastNotaAcc=parse;
				}
			} /* di jump , somQtz e accordo*/
			else 
			{
				if (! jump )  // se vero somQtz==ticksXBatt ma ho una nota INTERNA alla batt.!!
							  /* la condizione di nota interna era infatti testata in c942 ; se !jump
							  allora affinch !(if (! jump && (somQtz < ticksXBattuta || isAccordo) ))
							  condizione trovata in precedenza deve essere !somqtz<ticksxbatt e ! isAccordo */
				{  
					parse->attacco=ticksXBattuta+1;
					parse->inizio=ticksXBattuta+1;	 /* parse viene shiftato alla battuta seguente */
//APRINTF					printf("Dentro 12 ");
				}
			}	
		} 
		else 	/* di attacco <  ..*/
			/*  c 942 , caso di parse molto vicino alla fine della battuta */
		{	    // Sono sempre sicuro che temptic > somQtz ???????
				/* cosa c'entra > somqtz con la definizione di temptic ? , la definizione originale di 
			temptic  a */
			
			tempTic=0;
			
			(somQtz <= ticksXBattuta) ? (tempTic=ticksXBattuta-somQtz) : Errori=12;
			
			if (tempTic > 0)  /* Per Pause a fine battuta...*/  //  modifica Pascolini : eliminato lo scope 
				
			{
				tempTic=Quantize(tempTic,7); /* ancora un parametro inutile ? */
											 /*	if (tempTic <= centralTick[POS32] )
											 {  // NON deve essere FITTIZIA, LEGATA ne PRIMALEG.
											 if (prev->type==FINITA && ! prev->inAcc)
											 {   if (prev->valore==centralTick[POS32] || (prev->valore==centralTick[POS16p]))
											 { prev->valore +=centralTick[POS32]; /* si estende prev fino
				a fine battuta */
				/*			      somQtz=ticksXBattuta;
				tempTic=0;
				}
				}
				if (prev->type==LEGATA) //QUANDO E' IN ACCORDO ?? SBAGLIA !!
				{  puts("\n\n Questa battuta e' certamente scorretta.");
				puts(" Ci sono problemi che si ripercuotono alla fine. !\n");
				puts(" Anche se senza senso (legatura..) aggiungo una pausa !!\n");
				
				  //Mettere LA BATTUTA IN ROSSO
				  }
			} */
			}	
			if (tempTic > 0)  /* Per Pause a fine battuta...*/
			{	
				hideNote=AddNota(0,0,0,0); /* creo nota freq=0, canale=0 e timbro=0*/
				if (hideNote)
				{	   
					//	printf("Dentro 13 Tmptic=%lu e  som=%lu\n",tempTic,somQtz);	
					
//APRINTF					if (tempTic <= centralTick[POS32]) 
//APRINTF						printf("\n! Battuta # %lu CRITICA. Pausa/e di riempimento!\n",bat);
					
					hideNote->type=FITTIZIA;
					hideNote->next=parse;
					
					if (prev)
						prev->next=hideNote;
					else 
						topOfList=hideNote;
					prev=hideNote;
					hideNote->attacco=somQtz;
					hideNote->inizio=somQtz;
					
					if (somQtz+tempTic<=ticksXBattuta)
					{
						hideNote->pausa=tempTic;
					}
					else  /* cio se con l'aggiunta di temptic somqtz supera ticksxbatt */
					{	
						for (trova=0; trova<NUMNOTE && centralTick[trova] !=tempTic; trova++);
						
						if (trova) 
							trova--;   //Prendo la pausa precedente.BASTERA ?
						hideNote->pausa=centralTick[trova];
					}
					somQtz+=hideNote->pausa;
					if (somQtz<ticksXBattuta) 
						parse=hideNote; // Ricontrollare !!
					/* che significato ha ? */
				}
				else 
				{
//APRINTF					puts("Errore nell' allocazione di memoria. IMPOSSIBILE continuare.");
//APRINTF					printf("Problema nell' identificazione della Battuta %lu\n",bat);
					stop=Errori=12;
				}
			}
			else /* cio se !(tempTic > 0) */
				if (parse->type !=LEGATA  || Errori > 8 )  stop=1;
				// else  // TOGLIERE !
				// {if (tempTic ==0) printf("=");
				// }
				
				// La nota non e'  suff. interna alla battuta ma interna si => spostala alla fine!
				if ( (parse->attacco< ticksXBattuta) && (parse->type !=FITTIZIA) )
				{     
//APRINTF					puts(" SPOSTO "); 
					parse->attacco=ticksXBattuta;
					parse->inizio=ticksXBattuta;
					lastStart=parse->inizio;
					
				}	  
		}
		prev=parse;
		parse=parse->next;
		
	}    /*del while(parse && !stop) molto prima */
	
//APRINTF    if (Errori) puts("ERRORE MA PER ADESSO ME NE FREGO !!!!");
	
	
	/* Si controllano e riadattano le alterazioni delle note  affinche' siano concordi con
	quelle di battuta e l'eventuale armatura iniziale. Si scompongono le  PAUSE  e le
	NOTE con due alterazioni di durata ( 1/4pp , 2/4pp ) in piu' pause o note.
    Ho scelto di non usare i metodi delle classi di Moods perche' avrei perso la corri_
    spondenza tra le note Midi della struttuta NotaChunk e le note rappresentate dal
    sistema grafico. ( mantengo una mappatura 1 a 1. )
	
	Viene rivista la sequenza dei dati e scomposte le note (puntate e non) per favorire.*/
	
	
    // Controllo l' alterazione delle note della battuta.
	CheckMidiAlter(topOfList); 
	
	
	/*
	//---------------------- TOGLIERE---------------------------------
	if (!more)
	{
	parse=topOfList;
	while (parse && parse->type!=ATTIVA && (parse->inizio < ticksXBattuta))
	{
	printf("F)Pau: %lu Valore: %lu  Durata:%lu  Attacco: %d -Inizio: %d  -Type:%d -Altezza:(%2d) Alter:(%2d)-Acc:%d Nota : %d\n", 
	parse->pausa, parse->valore,parse->durata, parse->attacco, parse->inizio, parse->type, 
	parse->altezza, parse->alteraz,parse->inAcc,parse->freq);
	
	  parse=parse->next;		 
	  }
	  
		puts("--- queste sotto sono in attesa ---");
		while(parse)
		{
        printf("F)Attacco: %lu  Durata:%lu Altezza:(%2d) Alter:(%2d)- Tipo: %d   Freq: %d\n", 
		parse->attacco, parse->durata,parse->altezza, parse->alteraz, 
		parse->type,parse->freq);
		
		  parse=parse->next;
		  }
		  getchar();
		  }
		  //-----------------------------------------------------------
	*/
	
	parse=topOfList;
	prev=NULL;
	
    //  Per tutte le note FINITE e INTERNE alla battuta....
	while (parse && parse->type !=ATTIVA && (parse->inizio < ticksXBattuta)) 
	{  
		// succ=parse->next;
		if (parse->pausa) 
		{ 	
			for (trova=0;trova<NUMNOTE && centralTick[trova]!=parse->pausa;trova++);
			parse->pauPos=trova;
			// Split delle pause "pp" se necessario.
			if (trova==POS4pp || trova==POS2_4pp) 
				Split(&prev, parse, FALSE);
				/* &prev  l'indirizzo del puntatore alla nota precedente ; FALSE indica che
			si tratta di una pausa e non di una nota */
			
			if (trova == POS4p  || trova==POS2_4p) // scompongo la pausa di 1/4p o 2/4p
			{    
				hideNote=AddNota(0,0,0,0);
				if (hideNote)
				{     
					if (prev) 
						prev->next=hideNote;
					else 
					topOfList=hideNote; /* cio non c' prev e quindi 
					siamo all'inizio della lista */
					hideNote->next=parse;
					hideNote->type=FITTIZIA;
					hideNote->attacco=parse->attacco; /* noi vogliamo che 
													  le pause di hidenote e di parse si sommino , perch 
													  allora hidenote e parse vengono fatte iniziare insieme 
					e non in successione ? */
					hideNote->inizio=parse->inizio;
					if (trova==POS4p)
					{  
						parse->pauPos=POS8;
						hideNote->pauPos=POS4;
						parse->pausa=centralTick[POS8];
						hideNote->pausa=centralTick[POS4];
					} 
					else
					{
						parse->pauPos=POS4;
						hideNote->pauPos=POS2_4;
						parse->pausa=centralTick[POS4];
						hideNote->pausa=centralTick[POS2_4];
					}	
					prev=hideNote;
				}
			}
		}
		
		if (parse->type !=FITTIZIA)  
		{    
			for (trova=0;trova<NUMNOTE && centralTick[trova]!=parse->valore;trova++);
			parse->notaPos=trova; // Per le fittizie notaPos NON ha significato.
			// Split delle note "pp" se necessario.
			
			//			 if (trova==POS4pp || trova==POS2_4pp) Split(&prev, parse, TRUE);
			
		}	 
		
		prev=parse; 
		parse=parse->next; /* aggiornamento */
	} 
    
	unsigned long def=(ticksXBattuta-somQtz);
	
	
	//------------------------------------------------------------------\\
    //------ Inizio dell' INTERFACCIAMENTO con il sistema MOODS ----------\\
	
    
	char          tempo[8];
	int           numSpazi=1, altezzaNota=0;
	unsigned long  TOA=ticksXBattuta+30;  
	// TOA = T_ick(iniziale) O_f A_ccordo impostato FUORI battuta.
	
	Nota       * PNota=NULL;
	Accordo    * PAccordo=NULL;
	Movimento  * PMovim=NULL;
	
	
	PBattuta= new Battuta;
	PBattuta->SetChiave(0,new CViolino);   // SEMPRE battute in chiave di violino.
	PBattuta->SetArmatura(0,GetMidiArm(armKey,tonalKey)); // Setto armatura di chiave.
	PBattuta->SetNumericCode(bat);
	PBattuta->SetNumProgress(bat);
	strcpy(tempo,"4/4");
	sprintf(tempo,"%d/%d",tempoNum,tempoDen);
	PBattuta->SetTempo(tempo);
	
	
	// Elaboro gli eventi descritti nella lista che si riferisce alla traccia Zero
	// del file midi. Questi potrebbero richiedere modifiche dei parametri rispetto
	// a quelli specificati nella traccia corrente. 
	
    while (mark4ZeroTrk && mark4ZeroTrk->batNum == bat)
	{  
		if (mark4ZeroTrk->ev51) // impostazione di METRONOMO
		{ 
			// questo if perche' potrei avere piu' eventi midi 'setMetronomo'
			// nella stessa battuta (quindi tentare di creare piu' puntatore) !! 
			// In futuro cio' potrebbe corrispondere ad un Accelerando/Rallentando
			// Per adesso mostro solo il primo evento51 e by-passo gli altri. In Pratica
			// setto il metronomo della batt. corrente al 1mo setMetronomo che ci trovo.
			
			if (PBattuta->GetPMovimento() == NULL)
			{
				PMovim=new Movimento;
				
				PMovim->SetMetronomo();
				PMovim->MovSetTxt((unsigned char *)"",CL_NSEMIMINIMA,FALSE,mark4ZeroTrk->ev51);
				PBattuta->SetPMovimento(PMovim);
			}  
			
		}
		
		// Per l'impostazione dell' armatura di chiave dovrei controllare anche la
		// tonalita ( se > oppure < ) ma MOODS NON LA GESTISCE .
		// Quando (e se ) lo fara' andra' attivato l' IF dentro  GetMidiArm(..)
		// rimpostazione ARMATURA DI CHIAVE
		
		if (mark4ZeroTrk->ev59Arm) /* metaev 59 : key signature */
			PBattuta->SetArmatura(0,GetMidiArm(mark4ZeroTrk->ev59Arm,mark4ZeroTrk->ev59Ton)); 
		
		mark4ZeroTrk=mark4ZeroTrk->next;
		
	}  // Fine del While sugli eventi traccia Zero.
	
	parse=topOfList;
	prev=NULL;
	
	while (parse && parse->type !=ATTIVA && (parse->inizio < ticksXBattuta)) 
	{   
		succ=parse->next;
		if (parse->pausa) //se la pausa esiste...
		{		
			if ((PAccordo)&&(PAccordo->GetNumObj()<=1))
			{
				Nota* old_note=(Nota*)(PAccordo->GetFirstF());
				PBattuta->GetPLayer(0)->DelLast();
				PBattuta->AddFig(old_note,0);
				delete PAccordo;
				PAccordo=NULL;
			}
			
			// Se questo primo if e' vero, molto probab. ho 2 pause a fine battuta da swappare.
			if (succ && succ->type==FITTIZIA && parse->type==FITTIZIA )
			{
				ptrNotaChunk  succ2=succ->next;  
				
				// determino se sono a fine battuta
				if (succ2 && succ2->attacco >= ticksXBattuta && (!succ2->pausa)) //succ->pausa < parse->pausa) 
				{  
					unsigned long val;
					char temp;
					
					temp=parse->pauPos;
					val=parse->pausa;
					parse->pauPos=succ->pauPos;
					parse->pausa=succ->pausa;
					succ->pauPos=temp;
					succ->pausa=val;  /* in questo scope parse e succ si scambiano i valori
					di paupos e pausa , ma a che scopo ? */
				}
			}  
			// ...aggiungila alla battuta ( sul layer1) e dimmi quanti 
			//spazi sono associati alla pausa stessa.(numSpazi e' passato per indirizzo!!!)			
			PBattuta->AddFig(PausaMidi(parse->pauPos, numSpazi),0); /* numSpazi inizializzato
			a 1 in c2711 */
			//PBattuta->AddFig(new Spazio,2); /* spazio puntatore a figura */
			//for (; numSpazi; numSpazi--) 
			//{
			//		PBattuta->AddFig(new Spazio,1);
			//		PBattuta->AddFig(new Spazio,2);
			//}	
		}
		if (parse->type !=FITTIZIA )
		{	
			// sostituisco alla configurazione: 8vo, pausa 16mo, 16mo la struttura
			// 8vo puntato e 16mo.  VERIFICARE SE E' GIUSTO FARLO ADESSO
			// O METTERLO NEL FILE DELLE REGOLE !!!!!!
			// La variazione infatti potrebbe NON essere sempre giusta nel contesto
			// della battuta corrente.
			/*if (parse->notaPos==POS8 && succ && succ->pauPos==POS16)
			{ if (succ->inizio < ticksXBattuta && succ->notaPos==POS16)
			{
			parse->notaPos=POS8p;
			parse->valore=centralTick[POS8p];
			succ->pausa=0;
			succ->pauPos=0;
			}
			} modifica Pascolini , ho tolto la gestione di questo caso particolare perch in certe situazioni
			dava problemi (Birdland.mid,battuta 200)*/ 
			PNota=NewNotaMidi(parse->notaPos, numSpazi);
			PNota->SetAltezza(parse->altezza);
			
			//if (parse->altezza <= 4) PNota->SetGamboUp();
			//else PNota->SetGamboDwn();
			if (immissione.ImmRule(PNota->GetID(), PNota, PBattuta, NULL, 1)) //PB
				PNota->SetGamboUp();
			else 
				PNota->SetGamboDwn();
			
			
			
			if (((parse->inAcc)==1)&&(TOA != parse->inizio))
			{
				if (!(parse->next)) 
					parse->inAcc=0;
				else
				{
					if (((parse->next)->inizio)!=(parse->inizio)) 
						parse->inAcc=0;
				}
			}  
			
			if ((parse->inAcc)==1)
			{	
				if ( TOA != (parse->inizio)) 
				{  
					if (PAccordo != NULL)
					{
						if (immissione.ImmRule(PAccordo->GetID(), PAccordo, PBattuta, NULL, 1)) //PB 
							PAccordo->SetGamboUp();
						else PAccordo->SetGamboDwn();
						
						if ((PAccordo->GetNumObj())<=1)
						{
							Nota* old_note=(Nota*)(PAccordo->GetFirstF());
							PBattuta->GetPLayer(0)->DelLast();
							PBattuta->AddFig(old_note,0);
							delete PAccordo;
							PAccordo=NULL;
						} 
					}
					
					PAccordo= new Accordo;
					PBattuta->AddFig(PAccordo,0);
					TOA=parse->inizio; //porre TOA in questo modo fa si che nell'analisi della 
					//prossima nota componente questo accordo non venga creato un nuovo accordo
					int NumPunti;
					if (PNota->GetPPuntoValore()) 
					{
						NumPunti=(PNota->GetPPuntoValore())->GetNumPunti();
						PAccordo->SetPuntiValore(NumPunti);
					}
					
				}			
				//  RICONTROLLARE QUESTO  CODICE !!!!!
				Layer *pLayer=PBattuta->GetPLayer(0);//GB, parametrizzato, messo lo 0. 
				
				PNota->SetNumericCode(pLayer->GetCodeCounter());
				pLayer->SetCodeCounter(pLayer->GetCodeCounter()+1);
				PNota->SetPBattuta(PBattuta);
				if (PAccordo->AddNota(PNota))
				{
					if (PNota->GetCoda()) PAccordo->SetCoda(TRUE);
				}
				else
				{
					delete PNota;
					PNota=NULL;
					parse->type=FINITA;
					parse->alteraz=0;
				}
			}
			else
			{   
				if ((PAccordo)&&(PAccordo->GetNumObj()<=1))
				{
					Nota* old_note=(Nota*)(PAccordo->GetFirstF());
					PBattuta->GetPLayer(0)->DelLast();
					PBattuta->AddFig(old_note,0);
					delete PAccordo;
					PAccordo=NULL;
				}
				
				PBattuta->AddFig(PNota,0);
				/*
				def=Quantize(def,3);
				register unsigned char trova;
				for (trova=0;trova<NUMNOTE && centralTick[trova]!=def;trova++);
				if (def>0) PBattuta->AddFig(PausaMidi(trova, numSpazi),1);
				def=0; */
                
				//	PBattuta->AddFig(new Spazio,2);
				
				//	for (; numSpazi; numSpazi--) 
				//	{
				//		PBattuta->AddFig(new Spazio,1);
				//		PBattuta->AddFig(new Spazio,2);
				//	}	
			}
			// Assegnazioni alterazioni al Moods
			if (parse->alteraz)
			{	
				AlterazComposta * PAlt=new AlterazComposta;
				
				if (parse->alteraz >0 ) 
					PAlt->InserisciAlt(CL_DIESIS);
				if (parse->alteraz <0 ) 
					PAlt->InserisciAlt(CL_BEQUADRO);
				
				PNota->SetPAlterazComposta(PAlt);
			}
			// Controllo delle Legature di Valore tra le note...
			if (parse->type==PRIMALEGATA || parse->type==LEGATA)
			{	
				if (parse->type==PRIMALEGATA)
				{   
					if (prev==NULL || (prev && prev->type !=PRIMALEGATA))
					{
						for (trova=0; trova < noteInAcc; trova++)
							noteLeg[trova]=NULL;
						primaBatDiLegato=NULL;
						noteInAcc=0;
						minimaAlt=1000;
						massimaAlt=-1000;
					}
					altezzaNota=parse->altezza;
					if ( minimaAlt > altezzaNota) 
						minimaAlt=altezzaNota;
					if (massimaAlt < altezzaNota) 
						massimaAlt=altezzaNota;
					
					noteLeg[noteInAcc]=PNota;
					primaBatDiLegato=PBattuta;
					noteInAcc++;
				}
				else if (noteInAcc>0) /* cio parse  una nota legata ad una 
					precedente */
				{
					char    found=0;
					ArgType nc=0;
					LegaturaValore * leg= new LegaturaValore;
					
					
					for (trova=0; !found && trova<noteInAcc ; trova++)
					{
						if  (noteLeg[trova]->GetAltezza()==parse->altezza) 
							found=1;
					}
					trova--;
					if (found)
					{
						altezzaNota=noteLeg[trova]->GetAltezza();
						
						leg->SetSopra(altezzaNota > 4  ? TRUE : FALSE);
						
						if (noteInAcc >1)
						{
							if (altezzaNota==minimaAlt) leg->SetSopra(FALSE);
							if (altezzaNota==massimaAlt) leg->SetSopra(TRUE);
						}
						leg->SetPFigStart(noteLeg[trova]); /* noteleg array di puntatori 
						a nota , vedi hpp */
						leg->SetPFigEnd(PNota); /* pNota inizializzata in precedenza */
						leg->SetBattuta(1,primaBatDiLegato);
						leg->SetBattuta(2,PBattuta);
						
						noteLeg[trova]=PNota;          //  puo proseguire
						if (trova==noteInAcc-1) 
							primaBatDiLegato=PBattuta;
						
						PSpa->AddIntEsteso(leg,nc);
					} 
				}   
				else 
					Errori=10;  
			}
		} // di type != FITTIZIA
		
		if (parse->inAcc) 
		{
			int r=6;
		}
		prev=parse;
		parse=parse->next;
	}  // del while...
	
	if (def>0) 
		Complete_measure(&def,PBattuta);
	def=0;
	if (PAccordo != NULL)
	{
		if (immissione.ImmRule(PAccordo->GetID(), PAccordo, PBattuta, NULL, 1)) //PB
			PAccordo->SetGamboUp();
		else
			PAccordo->SetGamboDwn();
	}
	
	if ( more ) 
		PBattuta->SetBarra(SINGOLA);
	else 
		PBattuta->SetBarra(FINALE);	
	
	if(Battuta::GetAutoBeaming())
		PBattuta->BeamingAuto(); //PB
	
	PSpa->AddBattuta(PBattuta);
	
	
	// -------------  FINE INTERFACCIAMENTO CON MOODS  ------------------- \\
	//-----------------------------------------------------------------------\\
	
	// Eliminazione delle note.....!!
	
	parse=topOfList;
	primo=1;
	
	while (parse)													
	{	
		prev=parse;											
		while (prev &&  (prev->inizio < ticksXBattuta) ) // && prev->type!=ATTIVA)
		{	
			parse=prev->next;
			delete prev;
			prev=parse;
		}
		if (primo) 
		{
			primo=0;
			topOfList=parse; /* in modo topoflist sia aggiornato correttamente 
							 per quando si andr a cancellare le note della
			battuta seguente chiamando ancora il precedente ciclo */
			
			if (parse && parse->attacco >=(ticksXBattuta+surplus)) 
				totTicks-=(ticksXBattuta +surplus); 
			else 
				totTicks-=ticksXBattuta;
			
//APRINTF			if (more && somQtz < ticksXBattuta) 
//APRINTF			{	
//APRINTF				puts("\nATTENZIONE: Identificazione Battuta corrente POCO attendibile!!.\n");
//APRINTF				printf("Ticks Quantizzati Assegnati  [%lu]  su %lu\n\n", somQtz, ticksXBattuta);
//APRINTF			}
		}
		while (parse) // qui era  if (parse)  Vedere per note ATTIVE.
					  /* questo  gi un parse che verr mappato nella battuta successiva e perci
					  il suo attacco andr posto a 0 oppure ad un altro valore prossimo all'inizio
					  della battuta successiva , per come si fa a fermare il ciclo dato che questo 
					  richiederebbe parse = NULL ? */
					  
		{	
			if (parse->attacco < ticksXBattuta+surplus)	
				parse->attacco=0;
			else  
				parse->attacco -=(ticksXBattuta+surplus); 
			
			parse->inizio=parse->attacco;
			parse=parse->next;
		}	
	}
	
	if (topOfList==NULL) 
		totTicks=0;
	surplus=0;
	return Errori;
}


// ------ Elimina la lista di note senza alcun controllo. -----

inline void MidiImport::DestroyList()
{
	ptrNotaChunk parse=NULL;
	
	while (topOfList)
	{
		//		Message("MIDI-L6");
		parse=topOfList->next;
		delete topOfList;
		topOfList=parse;
	}
	totTicks=0;
	globalTicks=0;
}

inline MidiImport::ptrMarker  MidiImport::NewMarker() /* qui non si potrebbe fare a meno 
								dell'operatore di risoluzione dello scope ? ; newmarker
								dichiarato in hpp */
{
   ptrMarker  target;
   
   target=(ptrMarker) new Marker;
   
   if (target)
   {	target->batNum=1;
		target->ev51=0;
		target->ev58Num=0;
		target->ev58Den=0;
		target->ev59Arm=0; // arm va da -7..7
		target->ev59Ton=0; // -1 o 0
		target->next=NULL;
   }
   
   return target;
}
			//=================================\\
		   // 		Metodi privati che operano sul File Midi.    
		  //===================================\\
//
unsigned char  MidiImport::IsTrkHeader()
{
	unsigned long 	trackInfo=0L; /* ? */
	unsigned char   ok=0;
//	unsigned char info[4];
	int 					  letti=0;

	// Attenzione: Si suppone che curByte punti al byte iniziale del TrackHeader !!
	// Inoltre il metodo ne modifica il valore in modo opportuno, cosi che il metodo
	// ScanTrx(..) utilizzi SUBITO curByte. (Cioe' anch'esso suppone che curByte...)
	
	if (source_buf==0) 
		trackLen=((unsigned int)ibuf[curByte+4]<<24)+((unsigned int)ibuf[curByte+5]<<16)+
		((unsigned int)ibuf[curByte+6]<<8)+((unsigned int)ibuf[curByte+7]);
	
	if (source_buf==1) 
		trackLen=pConv->Get_out_buf().CharList_to_Int(curByte+4, 4);	
	curByte+=8; 		  // Sono avanzato di 8 bytes...
	ok=1;
	return ok;
}

char MidiImport::UsaMetaEv(unsigned char quale, unsigned long  bytes, ptrMarker *pMark)
{
	//PASCOLINI
	if ((*pMark)&&(add_MOODS_key))
		//se la tonalit non  codificata nel file MIDI la aggiunge
	{
	 armKey=MOODS_key;
	 tonalKey=MOODS_minor_key;
	 add_MOODS_key=false;
	 if (ticksXBattuta)  
		 (*pMark)->batNum=(unsigned int)(1+globalTicks/ticksXBattuta);
	 (*pMark)->ev59Arm=MOODS_key;
	 (*pMark)->ev59Ton=MOODS_minor_key;
	}	  
	//
	float   rapporto=0.0;
	char   stato=0;
	
	switch (METAMAP[quale]) /* definito in cpp */
	{
		case 0x0: seqNum=getc_buf();
				 seqNum=(seqNum << 8) + getc_buf();
//APRINTF				 printf("\tSequence Number: %d. Skipped...\n",seqNum);
				break;
		case 0x1:
		case 0x2: 
		case 0x3: 		
		case 0x4:
		case 0x5: 
		case 0x6: 		
		case 0x7:// Da case 0x1 case 0x7 sono tutti eventi testuali generici. 
				// Di questi mi interessano: 0x2, 0x3,0x4
						
						generalText=new char[bytes+1];
						if (generalText) 
						{	unsigned long   step;
						
							for (step=0; step<bytes; step++) 
							{
								generalText[step]=getc_buf();
								if (generalText[step]=='"') generalText[step]=' ';
							}
							generalText[step]='\0'; /* si aggiunge uno 0 alla fine del
							metaevento testo quando questo viene memorizzato nel vettore
							generaltext */
							bytes=step;
							
							if (METAMAP[quale]==0x2)  
							{  
							   if (copyRight) delete copyRight;
							   copyRight=new char[bytes+100];  
							   if (copyRight) strcpy(copyRight,generalText); 
							   /* copia generaltext in copyRight */
							}
							if (METAMAP[quale]==0x3) 
							{  if (seqName) delete seqName;
							   seqName=new char[bytes+100];
							   if (seqName) strcpy(seqName ,generalText);
							  // printf("<%s > %s\n",METALABEL[quale],seqName);
							}
							if (METAMAP[quale]==0x4)
							{  if (strumName) delete strumName;
							   strumName=new char[bytes+100];
							   if (strumName) strcpy(strumName,generalText);
							   // printf("<%s > %s\n",METALABEL[quale],seqName);
							}
							delete []generalText;
							bytes=0;
						} 
						else 
						{   stato=1;
						    puts ("Memory Allocation FAIL....");
						}
						break;
		case 0x20:	//  da'  un canale Midi a eventi che non lo hanno. INUTILIZZATO
							chanPref =getc_buf();
							//printf("<%s > %d\n",METALABEL[quale],chanPref);
						break;
		case 0x51:	microSecXquarto= getc_buf();
							microSecXquarto= (microSecXquarto<< 8) + getc_buf(); 
							/* da rivedere ? */
							microSecXquarto= (microSecXquarto << 8) + getc_buf();
							rapporto=100*(fabs(((float)oldMicro-microSecXquarto)/microSecXquarto));
							if (rapporto >= 20)
							{	
							  //printf("<%s>\t %3d  metronome beats/min\n",METALABEL[quale],(unsigned int)(60/(microSecXquarto /1e6)));
								oldMicro=microSecXquarto;

								if (*pMark) {
								    if (ticksXBattuta)  
										  (*pMark)->batNum=(unsigned int)(1+globalTicks/ticksXBattuta);
								    (*pMark)->ev51=(unsigned int)(60/(microSecXquarto /1e6));
									
									(*pMark)->next=NewMarker();
									if ( (*pMark)->next ) (*pMark)=(*pMark)->next;
								}	
								
							}else microSecXquarto=oldMicro; // lo riporto al suo prec. valore
							
						break;
		case 0x54:  /***********************************
							Se all'inizio mi accorgo di una  temporizzaz. SMPTE
							esco da programma. Dovrei avere 2 nuove Var e fare:
								  
							smpteHi=getc(srcFile);									// Ore
							smpteHi= (smpteHi << 8) + getc(srcFile); 	 // Minuti
							smpteHi= (smpteHi << 8) + getc(srcFile);	 //  Secondi
							smpteLo=getc(srcFile);								  
							smpteLo=(smpteLo << 8) + getc(srcFile);
							***********************************/
							// Anche se non e' plausibile trovare questo metaEvento in
							// un file con temporizzaz. PPQN adesso lo gestisco cosi:
							
							seek_buf(5); /*  Skip bytes...*/
						break;
		case 0x58:	tempoNum=getc_buf();
							tempoDen=getc_buf();
							tempoDen=(unsigned char)pow(2, tempoDen);
							//ticksXBattuta=(unsigned long)( (4.0 / tempoDen)*tempoNum*ppqn);

							if (*pMark) { /* pMark puntatore al marker */
								 if (ticksXBattuta)  
									     (*pMark)->batNum=(unsigned int)(1+globalTicks/ticksXBattuta);
								 /* aggiunge il numero di battuta al marker del metaevento
								  , notare che tale marker non  presente nello SMF ma viene aggiunto
								  per contraddistinguere la presenza del metaevento nel Moods ? */
							     else (*pMark)->batNum=1;
									 
								 (*pMark)->ev58Num=tempoNum;
								 (*pMark)->ev58Den=tempoDen;
								 
//APRINTF								 printf("Tx=%lu, B=%d\n", ticksXBattuta, (*pMark)->batNum);
								 
								 (*pMark)->next=NewMarker();
								  if ( (*pMark)->next ) (*pMark)=(*pMark)->next;
							}	  
							ticksXBattuta=(unsigned long)( (4.0 / tempoDen)*tempoNum*ppqn);
				  			clock4Met=getc_buf();
					  		num32sime=getc_buf();
							
							//printf("[%s ]\n",METALABEL[quale]);
							//printf("\t\tTempo: %d/%d\n",tempoNum,tempoDen);
							//printf("\n\t\tMidi ticks each metronome beat: %d",clock4Met);
							//printf("\n\t\tNum. 32nd in 24 Midi ticks: %d\n",num32sime);
						break;
		case 0x59:  
			armKey=getc_buf();
			tonalKey=getc_buf();
			if (MOODS_key!=original_key)
			{
				armKey=MOODS_key;
				tonalKey=MOODS_minor_key;
			}
							if (*pMark) {
								 if (ticksXBattuta)  
									 (*pMark)->batNum=(unsigned int)(1+globalTicks/ticksXBattuta);
								 (*pMark)->ev59Arm=armKey;
								 (*pMark)->ev59Ton=tonalKey;
									
								 (*pMark)->next=NewMarker();
								  if ( (*pMark)->next ) (*pMark)=(*pMark)->next;
								}	  
							
							//printf("<%s >\n",METALABEL[quale]);
							//printf("\t\tSF=%d e MI=%d\n", armKey, tonalKey);
						break;
		case 0x7f :  
		default		:  
//APRINTF			printf("<%s>  MetaEvent...  skipped.\n",METALABEL[quale]);
					   seek_buf(bytes);  //  Skip bytes...
					   break;
	
	}
	return stato; /* stato = 1 se ci sono stati errori di allocazione */
}

/*
 	Legge la traccia Midi.  Deve essere nel Formato 1, sia Standard che in modalita'  
    Running Status.  Per il formato 0  si presume una conversione a Formato 1
*/

unsigned char MidiImport::ScanTrX(unsigned int trc)
{
	unsigned char   dato=0,nota=0,timbro=0,RunningEvent,evento,fine=0,cerca=0,nibbleHi,nibbleLow
		            ,isFinished=0,inRunStat=0,oldChan=0,primoOn=0,channel=0; 
	unsigned int 	  battuta=0, len;
	unsigned long  	deltaT=0,bytes=0,app=0;
	int             erroriAmmessi=MAXERRAMMESSI;
	bool            RunningStatus ;
	ptrMarker            zeroMrk,nullMrk=NULL;
	
//APRINTF	printf("\n\t<<>> Decoding  [ TRACK # %lu ] <<>>\n\tTotal bytes: %lu\n\n",trc,trackLen);

	len=trackLen;  // len == la lungh. in byte della traccia.(trackLen: var di classe)
	zeroMrk=topZeroMark;

	//topThisMark= NewMarker();  // se NULL non posso gestire i metaeventi di traccia
	//currMrk=topThisMark;

	globalTicks=0; // Riazzero i ticks globali di traccia.
	
	if (trc>0)	
	{	
		PSpartito=new Spartito;
		PSpartito->Free();
	}	
	dato=getc_buf();
	len--; /* decrementa il numero di bytes da analizzare nella traccia */

//APRINTF	printf("\n WORK IN PROGRESS...\n");

    // se len==0 senza che sia fine==1 ==> condizione di errore
	while (! End_buf()  && (! fine) && (len > 0))
	{ 	
		deltaT=DTimeDecode(dato,&len);
	    dato=getc_buf();
	    len--; 
	
		if ( ! End_buf()) 		// Falso solo se ho errori nel file
		{
			evento=0;
			channel=0;
			RunningStatus=0;
			
			if (dato >= 0x80) 
			{   
				oldChan=StatusCheck(&nibbleHi,&nibbleLow,&evento,dato,&channel);
			    /* i valori sotto 80h indicano bytes di dati */
			    RunningEvent=evento;
			}
			else 
			{	
			    evento=RunningEvent;
				RunningStatus=1;
			}
			if  ( evento <8 )   // Ho trovato un (meta)evento noto
			{
				// Upgrade torna 1 se supero  ticksXBattuta
				/* non  corretto dire questo , infatti upgrade ritorna 1 se non ci sono note
				legate a fine battuta e si sono superati i tiksxbatt , oppure nel caso che non 
				si siano superati i ticksxbatt ?*/
//				char buf[20];
//				sprintf(buf,"MIDI-l1: %d",evento);
//				Message(buf);

				isFinished=UpgradeTicks(deltaT, oldChan);

//				Message("MIDI-L1--CLOSURE");

				switch (evento)
				{ 
				  case 0:    NoteOff_Handler(RunningStatus,dato,&len,oldChan,&erroriAmmessi,&isFinished,PSpartito,&battuta); break;
				  case 1: 	 NoteOn_Handler(RunningStatus,&len,oldChan,dato,&erroriAmmessi,PSpartito,&battuta,&isFinished); break;
				  case 2:    KeyTouch_Handler(&len,RunningStatus); break; 
				  case 3:    ControlChange_Handler(&len,RunningStatus); break; 
				  case 4:    ProgramChange_Handler(&len,RunningStatus,oldChan); break;
				  case 5:    ChannelTouch_Handler(&len,RunningStatus); break;
				  case 6:    PitchBend_Handler(&len,RunningStatus); break;
				  case 7:    GlobalEvent_Handler(&len,oldChan,trc,&battuta,PSpartito,&erroriAmmessi,&fine
								                  ,zeroMrk,nullMrk); break;
				} 

			//	Message("MIDI-L1--CLOSURE-totol");

				if (! fine) dato=getc_buf();
				if (len) len--;  // ???????
			} 
//APRINTF			else printf("Error(11): Midi event : %x(hex) is Unknown.\n",dato);
		} 
//APRINTF   else puts ("Error (10): Unexpected End of File .");
		
		
		if ( erroriAmmessi <=0)  len=0;
	}
//	Message("MIDI-L1-OUT");

//APRINTF	if (fine && trc) printf("** TRACK #%u Analysed....\n\n",trc);
//APRINTF	if (! fine)  	printf ("Error(12):  Decoding Error(s) in  Track #%u.\n\n",trc);
	
    if (battuta>maxbatt) maxbatt=battuta ;
	if ((erroriAmmessi <=0)&&(PSpartito->GetLastBat()!=NULL))
	         (PSpartito->GetLastBat())->SetBarra(FINALE);	
			 
	erroriAmmessi=MAXERRAMMESSI;
	
	// Distruggo la lista delle Note. Se tutto e' andato bene, questa NON fa nulla.
	// Per sicurezza....
	
	DestroyList(); 

	if (trc > 0)
	{
	    PSpartito->SetCodeCounterBat(battuta+1);
		
		if ( strumName ) 
		{
			PSpartito->SetStrumEsec(strumName);
			delete strumName;
			strumName=NULL;
		}
		else 
			if ( seqName ) 
			{
				PSpartito->SetStrumEsec(seqName);
				delete seqName;
				seqName=NULL;
			}
		
		pPart->AddSpartito(PSpartito);  // Aggiungo una parte (spartito) alla partitura.
		
	}
	return fine;
}	


//=================================================
//  -----  METODI PUBBLICI DELLA CLASSE ------     
//=================================================

/*char MidiImport::UseFile(char *fileN)
{		
	conv.File_to_buf(fileN);
	if (conv.Get_in_buf()[9]);
    return ok;
}*/

// Fornisce il formato del file (0, 1, 2)
unsigned int MidiImport::GetFormat() {return formato;}

// Fornisce il numero di traccie Audio. Quelle  totali  sono GetNumTrx+1 !
unsigned int MidiImport::GetNumAudioTrx() {return (traccie-1);}

// Fornisce i  Pulse Per Quarter Note. (ticks associati alla semiminima).
unsigned int MidiImport::GetPpqn() {return ppqn;}


unsigned char  MidiImport::Start(char soloInfo)
{
	int divisioni=0;
	unsigned char prosegui=0;
	unsigned long 	  headLen=0,headType=0,total=0;
//	unsigned long pos;
	
//	ibuf=pConv->Get_in_buf()->Get_iobuf();
	formato=ibuf[9];
	if (formato) traccie=(ibuf[10]<<8)+ibuf[11];					
	divisioni=(ibuf[12]<<8)+ibuf[13];
    ppqn=divisioni;
	curByte=14;
	if (!soloInfo && IsTrkHeader() )
	{					
		startTrk1=curByte+(trackLen*formato); // 1mo byte TrkHeader della 1ma traccia DATI
		/* inizio della prima traccia si trova dopo i bytes dell'MTHD
		   e quelli di dati della traccia 0 */
		if( ScanTrX(0))
		{			
			InitMusicParser ();
			prosegui=1;
		}
	} 
	else if (soloInfo) prosegui=1; 
  	return prosegui;
}

// Ritorna 1 se tutto Ok, 0  altrimenti. /* non  il contrario ? */
unsigned int  MidiImport::ReadAllTrx()
{	
	unsigned int tracks=traccie-1;
	unsigned int tr=traccie; // tracks sempre >=0
	maxbatt=0;
	/*curByte=startTrk1; /* quindi si legge solo le tracce dalla 1 in poi , non la 0, qui
	curByte viune posto all'inizio del track header */
	wxProgressDialog MidiImp("Importing...","",tracks);
	while (tracks && IsTrkHeader() )
	{	
//		Message("MIDI-Lx1");
		MidiImp.Update(traccie-tracks,wxString::Format("Track %d",traccie-tracks));
		int ret=ScanTrX(traccie-tracks);
//APRINTF		if (! ret) 
//APRINTF		       printf("Errors Reading Track #%lu. ...Try next (if there's any...)\n",traccie-tracks);
		
		//curByte+=trackLen; // NON MUOVERE DA QUI !!
		tracks--;
		
	}
//	Message("MIDI-LxOUT");
	Allinea_spartiti(tr);
	if (traccie-tracks >0)  return 0 ; /* tracce dove viene modificato ? */
	else return 1;
}	 

// Legge la traccia richiesta. Torna 1 se tutto Ok, 0 altrimenti.
unsigned int  MidiImport::ReadTrk(unsigned int trk)
{	
	unsigned int targ, ok=0;
	
	if (trk && trk <= GetNumAudioTrx() )
	{	targ=trk;
		curByte=startTrk1;
		
		while (IsTrkHeader() && --targ) curByte+=trackLen;
		if (targ==0) 	ok=ScanTrX(trk);

	}	
	return ok;
}		


//  Decodifica del Deltatime 
unsigned long  MidiImport::DTimeDecode(unsigned char dato, unsigned int *len)
{      
     	unsigned long deltaT=0;
		
		if ( ! (dato & 0x80))  deltaT=(unsigned long)dato; 
		else
		{
			deltaT=dato & 0x7f; 
			do
			{
//				Message("MIDI-L5");

				dato=getc_buf();
				(*len)--;
				deltaT=(deltaT << 7)  + (dato & 0x7f ); 
			}
			while ( dato & 0x80); /* cio cicla se il delta time  composto 
							da altri bytes ancora da leggere , indicato da un 1 nel MSb */
		}
        return deltaT;
}  
		
unsigned char  MidiImport::StatusCheck(unsigned char *nibbleHi,unsigned char *nibbleLow,unsigned char *evento,
										unsigned char dato,unsigned char *channel)
{    
	 unsigned char  oldChan;
     (*nibbleHi)=(dato) & 0xF0;
	 (*nibbleLow)=(dato) & 0x0F;
	 while ( ((*nibbleHi) ^ mask[(*evento)]) && ((*evento) < sizeof(mask)) ) 
	 {
		 (*evento)++;
//		 Message("MIDI-L3");
	 }
	 while ( (*nibbleLow) ^ ((*channel)++));   
	 {
	 (*channel)--; /* infatti dato che nel ciclo precedente  presente l'operatore
				di postincremento ++ quando si esce dal ciclo si  trovato il canale 
				successivo a quello reale */
//		 Message("MIDI-L4");
	 }
	 oldChan=*channel;
	 return oldChan;
}
			
void  MidiImport::NoteOff_Handler(bool RunningStatus,unsigned char dato,unsigned int *len,
				  unsigned char oldChan,int *erroriAmmessi,unsigned char *isFinished,Spartito *PSpartito,unsigned int *battuta
				  )
{  
    unsigned char nota,timbro,channel;
	channel=oldChan;

	if (RunningStatus)		// essendo  in RunStat,  ' dato' NON contiene un evento ma la nota...
	{ 			
		nota=dato;
		timbro=getc_buf(); /* timbro intesocome velocity */
		(*len)--;
		channel=oldChan;
	}
	else	// ...' dato' contiene un realmente evento...
	{ 			
		nota=getc_buf();   	//  freq. della nota
	 	timbro=getc_buf();   //  circa = velocita di rilascio
	 	(*len)-=2;
	}
							  
	(*erroriAmmessi)-=(MarkNota(nota, channel, timbro)); /*
	                marknota spegato in c279 , se marknota d 1 in questo caso
					significa che  stato trovato un noteoff per una nota che al
					momento non era attiva e quindi si  verificato un errore
					e andr diminuito di 1 il numero degli errori che si possono 
					ancora commettere*/
    while ((*isFinished) && (totTicks > ticksXBattuta) && ((*erroriAmmessi)>0))
	{   
//				Message("MIDI-NOFF1");
	    	    (*battuta)++;
				// CreateBatt  modifica totTicks quindi il while
				(*erroriAmmessi)-= CreateBatt(PSpartito,*battuta,1); /*
				CreateBatt dovrebbe anche azzerare totTicks o comunque modificarlo */
//				Message("MIDI-NOFF1-mid");
				*isFinished=TestState(); /* 1 se non ci sono note 
					a cavallo delle 2 battute , ma non  in pi ? */	
	}
//				Message("MIDI-NOFF1-out");
	return ;
}								
								
unsigned char  MidiImport::NoteOn_Handler(bool RunningStatus,unsigned int *len,unsigned char oldChan,unsigned char dato,
										  int *erroriAmmessi,Spartito *PSpartito,unsigned int *battuta,unsigned char *isFinished)	
{   
	unsigned char nota,timbro,channel;
	unsigned char  evento=1;
	channel=oldChan;
	if (! RunningStatus)
	{	
		nota=getc_buf();   // freq. della nota
		timbro=getc_buf();  // Piu o meno forte
		(*len)-=2;
        if ((timbro)&&(! AddNota(nota,channel,timbro,1))) 
			{
			      puts("Error(13): Memory Allocation FAIL. Unable to add new Note.");
			      (*len)=0;			//Esci con errore dal ciclo
			}
		else
			if (!timbro)
			{
				(*erroriAmmessi)-=(MarkNota(nota, channel, timbro)); 
				while ((*isFinished)&& (totTicks > ticksXBattuta) && (*erroriAmmessi>0) ) 
				{
					(*battuta)++;
					// CreateBatt  modifica totTicks quindi il while
					(*erroriAmmessi)-= CreateBatt(PSpartito,*battuta,1);
					/*if (totTicks>ticksXBattuta) 
					{
						(*battuta)++;
						// CreateBatt  modifica totTicks quindi il while
						(*erroriAmmessi)-= CreateBatt(PSpartito,*battuta,1);
					}   era una mia aggiunta ma sembra che non serva !!! */  
					(*isFinished)=TestState();
				}
			}  
	}
	else
	{   
		nota=dato;
		timbro=getc_buf();
		(*len)--;
		if ((timbro) && ( ! AddNota(nota,channel,timbro,1))) 
			{
//APRINTF			      puts("Error(13): Memory Allocation FAIL. Unable to add new Note.");
			      *len=0;			//Esci con errore dal ciclo
			}
		else if (!timbro)
		{
			(*erroriAmmessi)-=(MarkNota(nota, channel, timbro)); 
		    while ((*isFinished)&& (totTicks > ticksXBattuta) && (*erroriAmmessi>0) ) 
			{
				(*battuta)++;
		        // CreateBatt  modifica totTicks quindi il while
		        (*erroriAmmessi)-= CreateBatt(PSpartito,*battuta,1);
				/*if (totTicks>ticksXBattuta) 
				{
					(*battuta)++;
		            // CreateBatt  modifica totTicks quindi il while
		            (*erroriAmmessi)-= CreateBatt(PSpartito,*battuta,1);
				}   era una mia aggiunta ma sembra che non serva !!! */  
				(*isFinished)=TestState();
			}
		}
	}
	return evento ;
}

void MidiImport::KeyTouch_Handler(unsigned int *len,bool RunningStatus)
{
	unsigned char nota,dato;
	if (! RunningStatus)
	{
		nota=getc_buf();
		(*len)--;
	}
	dato=getc_buf();
	(*len)--;
	return;
}

void MidiImport::ControlChange_Handler(unsigned int *len,bool RunningStatus)
{
	unsigned char nota,dato;
	nota = 0;
	if (! RunningStatus)
	{
		nota=getc_buf();
		(*len)--;
	}
	dato=getc_buf();
	(*len)--;
	switch(nota)
	{
		case 7:
			if(!PSpartito->GetMidiAttrib()->IsVolumeSet())
				PSpartito->GetMidiAttrib()->Set_Volume((int) dato);
		break;
		default:
		break;
	}


	return;
}

void MidiImport::ProgramChange_Handler(unsigned int *len,bool RunningStatus, unsigned char channel)
{
	unsigned char dato;
	if (! RunningStatus)
	{
		dato=getc_buf();
		(*len)--;
		if(!PSpartito->GetMidiAttrib()->IsReady())
		{
			PSpartito->GetMidiAttrib()->Set_MidiCode((int) dato);
			PSpartito->GetMidiAttrib()->Set_Channel((int) channel);
		}
	}
	return;
}

void MidiImport::ChannelTouch_Handler(unsigned int *len,bool RunningStatus)
{
	unsigned char dato;
	if (! RunningStatus) 
	{
		dato=getc_buf();
	    (*len)--;
	}
	return;
}

void MidiImport::PitchBend_Handler(unsigned int *len,bool RunningStatus)
{
	unsigned char dato,nota;
	if (! RunningStatus)
	{
		nota=getc_buf();
		(*len)--;
	}
	dato=getc_buf();
	(*len)--;
	return;
}

void MidiImport::GlobalEvent_Handler(unsigned int *len, unsigned char oldChan,unsigned int trc,unsigned int *battuta,Spartito *PSpartito,
                                       int *erroriAmmessi,unsigned char *fine,ptrMarker zeroMrk,ptrMarker nullMrk)
{
	switch (oldChan) 
	{	
	   case  0:  SysEx_Handler(len); break;
	   case  2:  SongPos_Handler(len); break;
	   case  3:  SongSelect_Handler(len); break;
	   case  7:  SysEx_Handler(len); break;
	   case 15:  Meta_Handler(len,trc,battuta,PSpartito,erroriAmmessi,fine,zeroMrk,nullMrk); break;
	}
	return;
}

void MidiImport::SysEx_Handler(unsigned int *len)
{
	unsigned char bytes;
	bytes=getc_buf();
	(*len)--;
    bytes=DTimeDecode(bytes,len);
	seek_buf(bytes);
	(*len)-=bytes;
	return;
}

void MidiImport::SongPos_Handler(unsigned int *len)
{
	unsigned char nota,dato;
	nota=getc_buf();
	dato=getc_buf();
	(*len)-=2;
	return;
}

void MidiImport::SongSelect_Handler(unsigned int *len)
{
	unsigned char dato;
	dato=getc_buf();
	(*len)--;
	return;
}

void MidiImport::Meta_Handler(unsigned int *len,unsigned int trc,unsigned int *battuta,Spartito *PSpartito,int *erroriAmmessi,
							  unsigned char *fine,ptrMarker zeroMrk,ptrMarker nullMrk)
{
	unsigned char dato,bytes,cerca;

	dato=getc_buf();     	// Tipo di MetaEvento 
	bytes=getc_buf(); 
	(*len)-=2;
	bytes=DTimeDecode(bytes,len);
	cerca=0;
	while((cerca< MAXEV) && (dato !=METAMAP[cerca])) cerca++; /* maxev = 15
															, definito in hpp */
												
	// Se ho trovato un Metaevento Standard....
	if ( cerca <MAXEV)
	{ 
		if (dato ==0x2f && ! bytes)  // Fine Traccia
		{ 
			if (trc >0)
			{   
				while ( (totTicks > ticksXBattuta) && (*erroriAmmessi>0))
				{
					(*battuta)++;
					// Create Batt modifica totTicks quindi...
					(*erroriAmmessi)-=CreateBatt(PSpartito,*battuta,1);
				}  
                (*battuta)++;
				if (*erroriAmmessi >0) CreateBatt(PSpartito,*battuta,0);
				   /* crea la battuta finale , more=0 */
			}
			if (! bytes) *fine=1;  else *len=0;
		} 
		else  
		{
			//printf("1) Z0=(%p) Z0N=%p Zero=(%p)\n",
			//  topZeroMark,topZeroMark->next,zeroMrk);
			//  UsaMetaEv alloca nuovi  marker !!
			
			(trc) ? ((*erroriAmmessi)-=UsaMetaEv(cerca,bytes,&nullMrk) ) : ((*erroriAmmessi)-=UsaMetaEv(cerca,bytes,&zeroMrk));
			/* usametaev ritorna 1 se c' un memory fault */
			/* nullmark e zeromark inizializzati a 0 all'inizio di scantrx */
		}	
	}
	else  
	{	
//APRINTF		printf("\nMetaEvent %x(hex) NOT supported...skipped.\n",dato);
		seek_buf(bytes);
		(*len)-=bytes;
		
	}
	if ((*len)>=bytes) (*len)-=bytes;
	else (*len)=0;			// qui andrebbe rivisto....
	return;
}

void MidiImport::Allinea_spartiti(int tr)
{   
	Spartito* pSp=NULL;
	unsigned int extrabat;
	int NumSpazi=1;
	Battuta* pBat;
	unsigned long bat;
	char tempo[20];

	for (int actr=1; actr<=pPart->GetNumSpa(); actr++)
	{
		pSp=(pPart->GetSpart(actr));
        bat=pSp->GetNumeroBattute();
		extrabat=maxbatt-bat;
		if ((extrabat!=0)&&(pSp->GetLastBat()!=NULL)) pSp->GetLastBat()->SetBarra(SINGOLA);
		for (unsigned int acbat=1; acbat<=extrabat; acbat++)
		{
			pBat = new Battuta();
			pBat->SetChiave(0,new CViolino);   // SEMPRE battute in chiave di violino.
			pBat->SetArmatura(0,GetMidiArm(armKey,tonalKey)); // Setto armatura di chiave.
			pBat->SetNumericCode(bat+acbat);
			pBat->SetNumProgress(bat+acbat);
			strcpy(tempo,"4/4");
			sprintf(tempo,"%d/%d",tempoNum,tempoDen);
			pBat->SetTempo(tempo);

			pBat->AddFig(PausaMidi(13,NumSpazi),0);
			if (acbat==extrabat)
				pBat->SetBarra(FINALE);
			else
				pBat->SetBarra(SINGOLA);
			pSp->AddBattuta(pBat);
		}
	}
	return; 
}

// aggiunge pause per completare le battute in difetto 
void MidiImport::Complete_measure(unsigned long* def,Battuta* PBattuta)
{
	unsigned char trova;
    int numSpazi=1;
	//(*def)=Quantize(*def,3);
    for (trova=0;(trova<NUMNOTE) && (centralTick[trova]<(*def));trova++);
	if (trova>=NUMNOTE) return;
	if ((centralTick[trova]-(*def)>0)) trova--;
	PBattuta->AddFig(PausaMidi(trova, numSpazi),0);
	(*def)-=centralTick[trova];
    if ((*def)>0) Complete_measure(def,PBattuta);
}

Midi_converter* MidiImport:: get_pConv()
{
	return pConv;
}

unsigned char MidiImport::getc_buf()
{
	unsigned char data;

	switch (source_buf)
	{
	case 0 : data=ibuf[curByte];
		break;
	case 1 : data=(*obuf)[curByte];
		break;
	}
	curByte++;
	
	return data;
}

void MidiImport::seek_buf(int off)
{
	curByte+=off;
}

bool MidiImport::End_buf()
{
	bool eob=FALSE;

	if (curByte>End_buffer) 
	{
		eob=TRUE;
	}

	return eob;
}

void MidiImport::Set_source_buf(const char source)
{
	source_buf=source;
	if (pConv) obuf=&(pConv->Get_out_buf().Get_buf());
}

void  MidiImport::Set_tracks_num(unsigned int num)
{
	traccie=num;
}

Midi_converter* MidiImport::Midi_conv_create()
{
	pConv=new Midi_converter();
	return pConv;
}

void MidiImport::Set_ibuf(unsigned char* pbuf)
{
	ibuf=pbuf;
}

void MidiImport::Set_End_buf(unsigned int end)
{
	End_buffer=end;
}

void MidiImport::Set_MOODS_key(wxString new_key, bool add_key)
{
	if (add_key) add_MOODS_key=true;
	MOODS_key=Key_encode(new_key);
}

BYTE MidiImport::Key_encode(wxString key)
{
	MOODS_minor_key=key.Contains("minor");
	for (char i=0;i<15;i++)
		if ((keym[i]==key)||(keyM[i]==key)) 
			return (i-7);
	return 0;
}

		 
