/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: midiimport.hpp,v 1.1 2005/07/25 16:31:16 bellinpo Exp $ */
/*---------------------------------------------------------------------
	File: MidiImport.hpp
	Modulo: IMP
	Descrizione: Classe " figlia " di Import per la gestione dei file midi.
					I metodi e le strutture della classe provengono da codice
					C e sono state  modificate per compatibilita'  con il C++. 
					Questo aspetto NON conferisce al modulo una filosofia
					completamente " ad oggetti " che avrebbe richiesto una
					riprogettazione di molte parti di codice.

	Autore:		Gambineri Luca (GL)
	Periodo:	Novembre 98.
----------------------------------------------------------------------*/

#ifndef MIDIIMPORT_HPP
#define MIDIIMPORT_HPP

#include "imp/import.hpp"
#include "bat/battuta.hpp"
#include "fig/nota.hpp"
#include "fig/pausa.hpp"
#include "imp/midiseq.h" //


#ifdef M_PI                 //  Per compatibilita'  con alcune versioni...
	#define PI     M_PI
#endif

#define NEAR(a,b)  (a < b) ? 0 : 1	//se a=b NEAR DEVE dare 1 perche' prendo la nota piu lunga
#define MAX(a,b)   (a>b) ? 0 : 1	/*se a=b MAX deve dare 1 perche (vedi  in CreaBatt() ) cosi
										prendo le note intere (associate alla LogNorm) e non quelle
										puntate (che sono meno comuni! )*/  

#define   SIGMA					  .6		  // param. per la LogNorm.  */
#define   TOLERANCE     	18		  // Toller. sui ticks (basato su ppqn=512)*/ 
#define   GAP					      2		  // Max Rapporto tollerato tra valore e pausa associata*/

#define	  TRACKCHUNK         0x4d54726b
#define   HEADCHUNK 		  0x4d546864
#define   MAXinACC              100              // Max num. di note in accordo(le dita dei piedi NON valgono!)
#define   MAXERRAMMESSI 12             // Max Num errori tollerati nella decodifica di UNA traccia 

#define   DEFMICROSEC 		 500000 	//	Pari a 120 colpi metron./min
#define   TEMPONUM 			   	 	    4	   //	Numer.  Time Signature
#define   TEMPODEN  		    	     2	   //	Denom. Time Sign. =2^TempoDen
#define   TICKSXMETR  		    	  24	 //  Relazione tra clock e metron. 
#define   BISCROMEIN24TICKS  	 8	  //  32sime in 24 tics di clock
#define   DEFHEADERLEN    			  6	  //  bytes per l'header (+8 iniziali) 
#define   MAXEV					            15	 //  Num. Max di MetaEventi.
#define   NUMNOTE				   	     14	  // Num. note rappresentate. 
#define   NUMLOGN					       5   // Num. delle distr. Log-normali
#define   NUMGAUSS				         6   //  Num. delle distr. Gaussiane >2
#define   MYPPQN 				   	     640   //  Impulsi (ticks) per quarter note

#define   POS128           0
#define   POS64             1
#define   POS32             2
#define   POS16             3
#define   POS16p           4	  //  Posiz. della nota 1/16p 
#define   POS8               5
#define   POS8p             6
#define   POS4               7
#define   POS4p             8
#define   POS4pp           9
#define   POS2_4         10
#define   POS2_4p       11
#define   POS2_4pp     12
#define   POS4_4         13


enum NotaState {ATTIVA, FINITA, FITTIZIA, PRIMALEGATA, LEGATA};//QUI

class MidiImport : public Import
{
	private:

		// Struttura (dal codice C) che memorizza i dati della notaMidi.
		
		typedef struct  _NotaChunk
		{	
			char				 ottava, altezza,notaPos,
								 pauPos,alteraz;
			unsigned char 		  freq, chan,inAcc, 
								 timbro,rilascio;
			unsigned long 		  durata,valore,attacco,inizio,pausa;
			float                attendibile;
		    enum NotaState       type;
			struct _NotaChunk    *next;
		} NotaChunk;

		typedef NotaChunk * ptrNotaChunk;
		
		typedef struct _Marker
		{
		   unsigned int      batNum, ev51;
		   char                    ev58Num,ev58Den,
		                               ev59Arm, ev59Ton;
		   struct _Marker   *next;
		
		} Marker;

		typedef  Marker * ptrMarker;

		inline float Gauss(unsigned int med, float var, unsigned int x);
		inline float LogNorm(unsigned int da, float mu, float s,unsigned int x);
		inline void DestroyList();
		inline char  UpgradeTicks(unsigned long delta, char chan);
		inline char  TestState();
        inline ptrMarker  NewMarker();
		inline char  MarkNota(unsigned char freq, unsigned char chan, unsigned char timbro);
		

		unsigned char IsTrkHeader();						// Controlla l' header della traccia
		unsigned char ScanTrX(unsigned int trk);	// Esamina la traccia e chiama altri metodi
		char InitMusicParser();

		char UsaMetaEv(unsigned char quale, unsigned long bytes, ptrMarker *pMrk);
		void Split ( ptrNotaChunk *prev, ptrNotaChunk current , char isNota);
		//void Split (ptrNotaChunk prev, ptrNotaChunk current , char isNota);
		void CheckMidiAlter(ptrNotaChunk check);
		void Sdoppia(ptrNotaChunk nota1,ptrNotaChunk nota2,char v1,char v2,char pAcc);
		void Triplica(ptrNotaChunk nota1,ptrNotaChunk nota2,char v1,char v2,char v3);
		
		char CreateBatt(Spartito * PSpa, unsigned int nBatt, unsigned int more);
		//unsigned long Quantize(unsigned long tic, unsigned long *other);
		unsigned long Quantize(unsigned long tic, char ix) ;
		ptrNotaChunk AddNota(unsigned char freq,unsigned char chan,unsigned char timbro, char accoda);

		Nota * NewNotaMidi(char pos, int& numSpazi);
		Pausa * PausaMidi(char pos, int& numspazi);
		tipoArmaturaChiave GetMidiArm(char chiave, char tonalita);

	protected:
				
		Nota *noteLeg[MAXinACC];    // Memorizza  le note di partenza di una legatura di valore.
		Battuta *primaBatDiLegato;
		int     minimaAlt, massimaAlt;

		static const char    *METALABEL[];	    //  Descrizione Metaeventi
		static const unsigned char METAMAP[];	// Maschere Metaeventi

        // queste tre servono in checkMidiAlt(..)
		static const char scalaBase[];
        static const char armDies[];
        static const char armBem[];   // Stesso ragionamento...
		
		static const char level8va[]; // altezza dell' 8va i-sima
		// Per questi due vettori vedi il metodo AddNota
        static const char offset[]; 
        static const char isAlter[];
		
		unsigned int  CHORDLEAP;															  
		unsigned long 	microSecXquarto, oldMicro, curByte, startTrk1,
                                    trackLen, ticksXBattuta, totTicks, globalTicks;
		unsigned int   	  formato, traccie, seqNum, ppqn;
//		register unsigned int	centralTick[NUMNOTE];
		unsigned int    centralTick[NUMNOTE] ,logNStart[NUMLOGN],  gaussCenter[NUMGAUSS];

		unsigned char	tempoNum, tempoDen, clock4Met,num32sime, 
                                    noteInAcc, chanPref ; 
						
		char			*generalText,  *seqName, *strumName, *copyRight;
		char 			armKey, tonalKey;      // arm e tonalKey: Per L'armatura di chiave.
		
		ptrNotaChunk  	topOfList, endOfList;              //  Variabili Globali di Inizio / Fine lista.
		ptrMarker          topZeroMark; // topThisMark

// Modifiche Pascolini
		float		 	muVect[NUMLOGN] , varVect[NUMGAUSS];
        static const unsigned char   mask[];
        unsigned char   StatusCheck(unsigned char *nibbleHi,unsigned char *nibbleLow,unsigned char *evento,unsigned char *dato) ;
		
	public : 
		MidiImport(char * = NULL);
		~MidiImport();
		
		
//		char UseFile(char *fileN);
		unsigned char Start(char soloInfo);
		unsigned int GetFormat();
		unsigned int GetNumAudioTrx();  // N.B: Le traccie totali sono GetNumTrx()+1 !!
		unsigned int GetPpqn();
		unsigned int ReadAllTrx();
		unsigned int ReadTrk(unsigned int trk);
// Modifiche Pascolini
		unsigned long   DTimeDecode( unsigned char dato, unsigned int *len);
        unsigned char   StatusCheck(unsigned char *nibbleHi,unsigned char *nibbleLow,
			                         unsigned char *evento,unsigned char dato,unsigned char *channel);
        void   NoteOff_Handler(bool RunningStatus,unsigned char dato,unsigned int *len,
				               unsigned char oldChan,int *erroriAmmessi,unsigned char *isFinished,Spartito *PSpartito,unsigned int *battuta);
        unsigned char  NoteOn_Handler(bool RunningStatus,unsigned int *len,unsigned char oldChan,unsigned char dato,
					                              int *erroriAmmessi,Spartito *PSpartito,unsigned int *battuta,unsigned char *isFinished);
        void KeyTouch_Handler(unsigned int *len,bool RunningStatus);
        void ControlChange_Handler(unsigned int *len,bool RunningStatus);
        void ProgramChange_Handler(unsigned int *len,bool RunningStatus, unsigned char channel);
        void ChannelTouch_Handler(unsigned int *len,bool RunningStatus);
        void PitchBend_Handler(unsigned int *len,bool RunningStatus);
        void GlobalEvent_Handler(unsigned int *len, unsigned char oldChan,unsigned int trc,unsigned int *battuta,Spartito *PSpartito,
                                       int *erroriAmmessi,unsigned char *fine,ptrMarker zeroMrk,ptrMarker nullMrk);
        void SysEx_Handler(unsigned int *len);
        void SongPos_Handler(unsigned int *len);
        void SongSelect_Handler(unsigned int *len);
        void MidiImport::Meta_Handler(unsigned int *len,unsigned int trc,unsigned int *battuta,Spartito *PSpartito,int *erroriAmmessi,
							  unsigned char *fine,ptrMarker zeroMrk,ptrMarker nullMrk);
        Spartito  *PSpartito;
		unsigned int maxbatt;
        void Allinea_spartiti(int tr);
        void Complete_measure(unsigned long* def,Battuta* PBattuta);
		Midi_converter* pConv;
		unsigned char* ibuf;
		UNSIGNED_CHAR_ARRAY*obuf;
		Midi_converter* get_pConv();
		unsigned char source_buf;
		unsigned char getc_buf();
		void seek_buf(int off);
		bool End_buf();
		void Set_source_buf(const char source);
		void Set_tracks_num(unsigned int num);
		Midi_converter* Midi_conv_create();
		void Set_ibuf(unsigned char* pbuf);
		unsigned int End_buffer;
		void Set_End_buf(unsigned int end);

		void Set_MOODS_key(wxString new_key, bool add_key);
		BYTE MOODS_key;
		bool MOODS_minor_key;
		BYTE Key_encode(wxString key);
		static wxString keyM[15];
		static wxString keym[15];
		bool add_MOODS_key;
};
#endif  //MIDIIMPORT_HPP
