/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#define ___MBUFWINTYPES___
#include "windows.h"
#include "midiseq.h"
//#include "midiconv.h"
#include "math.h"

#include "fig/figura.hpp"
#include "bat/battuta.hpp"
#include "drw/scorefrm.hpp"

#include "wx/stream.h"

struct Midi_attributes
{
	MIDIHDR* p_playing_head;
	MIDIHDR* p_old_head;
	HMIDISTRM outst;
	MIDIPROPTEMPO tempo_struct;
	MIDIPROPTIMEDIV timediv_struct;  
};

//////////GLOBAL CONSTANTS///////////////////////

UINT MIDIOUTDEV=MIDI_MAPPER;
const DWORD RESULT_OK=0;
const DWORD uSec_in_mSec=1000;

//////////MIDI_PLAYER CONSTANTS//////////////////

// lunghezza in microsecondi del buffer 
// di esecuzione, valori troppo bassi 
// possono causare malfunzionamenti
const DWORD STR_BUF_TIME_LEN=500000;

//////////MIDI_SEQUENCER CONSTANTS///////////////

	//////////VIRTUAL_PLAYER/////////////////////

    // regola l'accuratezza della temporizzazione 
    // nel draw delle note attive, compreso tra 0 e 1, 
    // valori elevati possono provocare errori di 
    // timing rilevabili abilitando TIMING_CHECK
	const FLOAT SLEEP_FACTOR=(FLOAT)0.5; 
	const DWORD SLEEP_TIME_THRESHOLD=50;

	// se TRUE abilita il controllo sulla corretta 
	// temporizzazione del virtual player thread
	const BOOL TIMING_CHECK=FALSE;

	//////////MOODS_SYNCHRONIZATION//////////////

	const FLOAT SYNC_MEASURE_TOLERANCE=(FLOAT)0; 
	const FLOAT SYNC_ERROR_TOLERANCE=(FLOAT)3;
	// imposta l'allineamento in battute del play 
	// rispetto alla barra di esecuzione
	const FLOAT TRACKING_ZERO_OFFSET=(FLOAT)(-0.2);

//////////GLOBAL VARIABLES///////////////////////

DWORD result=0;
DWORD interv, maxint;

//////////GLOBAL FUNCTIONS///////////////////////

inline void CHECKRES(DWORD position=0, wxString msg="")
{
	wxString str;
	if (position) str<<"Bad result from function "<<
		position<<" \n(result="<<result<<")";
	else str<<msg;
	if (result) throw wxString(str); 
}

void STARTT()
{
	interv=timeGetTime();
}

void STOPT()
{
	if (interv)
	{
		interv=timeGetTime()-interv;
		if (interv>maxint) 
			maxint=interv;
	}
}

inline void CHECK_MAXINT(DWORD t_max, Midi_sequencer* p_seq)
{
	if (maxint>t_max)
	{
		wxString error="Virtual Player Timing error";
		error<<" (Time="<<maxint<<"ms)";
		error<<"\nTry to change refresh settings.";
		wxMessageBox(error);
		p_seq->Stop();
	}
}

bool Is_tempo(Midi_event evt)
{
	BYTE status=evt.get_status();
	BYTE channel=evt.get_channel();
	UNSIGNED_CHAR_ARRAY data=evt.get_data();
	if ((data.GetCount()==5)&&
		(status==15)&&
		(channel==15)&&
		(data[0]==0x51)&&
		(data[1]==3)) return true;
	return false;
}

void CALLBACK MidiCallProc(HMIDIOUT hmo, UINT uMsg, DWORD dwInstance, 
		    				DWORD dwParam1, DWORD dwParam2)  
{
	Midi_player* p_player=(Midi_player*)dwInstance;
	
	switch (uMsg)
    {
		case MOM_DONE:
			p_player->On_MOM_DONE();
			break;
		
		case MOM_POSITIONCB:
		    break;

        case MOM_OPEN:
			break;
        
		case MOM_CLOSE:
			break;
    }
}

//////////MIDI_TEMPO_MAPPER IMPLEMENTATION///////

Midi_tempo_mapper::Midi_tempo_mapper()
{
	tempo_track.Clear();
}

void Midi_tempo_mapper::Build_map(Midi_struct& _struct)
{
	Midi_track* p_track=NULL;
	Struct_type _type=_struct.Get_struct_mode();

	if (_type==CHANNEL_STRUCT) p_track=&(_struct[16]);
	else 
		if (_type==TRACK_STRUCT) p_track=&(_struct[0]);
		else 
			throw wxString("Unknown Midi_struct type !");

	Midi_track& track=*p_track;
	DWORD ind=0;
	bool no_tempo=true;

	while (ind<track.GetCount())
	{
		if (Is_tempo(track[ind])) 
		{
			tempo_track.Add(track[ind]);
			no_tempo=false;
		}
		ind++;
	}

	if (no_tempo) throw wxString("Tempo event not found !");
}

DWORD Midi_tempo_mapper::Get_tempo(DWORD _ticks)
{
	DWORD index=0;
	while ((index<tempo_track.GetCount())&&
		(tempo_track[index].get_time()<=_ticks)) index++;
	UNSIGNED_CHAR_ARRAY& data=tempo_track[index-1].get_data();
	DWORD tempo=((data[2]<<16)+(data[3]<<8)+data[4]);
	return tempo;
}

//////////MIDI_PLAYER IMPLEMENTATION/////////////

Midi_player::Midi_player()
{
	p_midi_attr=new Midi_attributes;
	p_midi_attr->timediv_struct.cbStruct=sizeof(MIDIPROPTIMEDIV);
	p_midi_attr->tempo_struct.cbStruct=sizeof(MIDIPROPTEMPO);
	p_midi_attr->p_playing_head=NULL;
	p_midi_attr->p_old_head=NULL;
	Set_act_tempo(400000);
	start_tempo=400000;
	PPQN=0;
	buf_struct_act_pos=NULL;
	buf_struct_start_pos=NULL;
	preparing_strbuf=NULL;
	playing_strbuf=NULL;
	num_tracks=0;
	act_ticks=0;
	memset(status_filter, 1, 16);
	status_filter[15]=0;
	result=0;
	Set_state(STOPPED);
	Set_p_playing_track(NULL);
	p_preparing_track=NULL;
	p_frame=NULL;
	next_tempo=start_tempo;
	external_tempo_change=false;
	send_programs=false;
	endMeasure=-1;
}

Midi_player::~Midi_player()
{
	DELITEM(p_midi_attr)
}

Midi_struct& Midi_player::Get_working_struct()
{
	return working_struct;
}

DWORD Midi_player::Get_act_tempo() 
{ 
	DWORD tempo;
	act_tempo_access.Enter();
	tempo=act_tempo;
	act_tempo_access.Leave();
	return tempo; 
}

void Midi_player::Set_state(PLAYER_STATE _state)
{
	state_access.Enter();
	state=_state;
	state_access.Leave();
}

inline DWORD Midi_player::Get_num_tracks() 
{ 
	return num_tracks; 
}

inline DWORD Midi_player::Get_PPQN() 
{ 
	return PPQN; 
}

inline Midi_attributes* Midi_player::Get_p_midi_attr() 
{ 
	return p_midi_attr; 
}

void Midi_player::Set_PPQN(DWORD ppqn) 
{ 
	PPQN=ppqn; 
}

void Midi_player::Set_act_tempo(DWORD tempo) 
{ 
	act_tempo_access.Enter();
	act_tempo=tempo; 
	act_tempo_access.Leave();
}

void Midi_player::Set_p_playing_track(Midi_track* p_track)
{
	p_playing_track_access.Enter();
	p_playing_track=p_track;
	p_playing_track_access.Leave();
}

wxWindow* Midi_player::Get_p_frame()
{
	return p_frame;
}

void Midi_player::Restore()
{
	result=midiStreamStop(p_midi_attr->outst); CHECKRES(13);
	result=midiOutReset((HMIDIOUT)p_midi_attr->outst); CHECKRES(6);
	Set_act_tempo(start_tempo);
	act_ticks=0;
	buf_struct.Clear();
	DELARRAY(buf_struct_act_pos)
	DELARRAY(buf_struct_start_pos)
	if (p_midi_attr->p_old_head)
	{
		result=midiOutUnprepareHeader((HMIDIOUT)(p_midi_attr->outst), 
			p_midi_attr->p_old_head, sizeof(MIDIHDR)); CHECKRES(7);
		DELITEM(p_midi_attr->p_old_head)
	}
	if (p_midi_attr->p_playing_head)
	{
		result=midiOutUnprepareHeader((HMIDIOUT)(p_midi_attr->outst), 
			p_midi_attr->p_playing_head, sizeof(MIDIHDR)); CHECKRES(8);
		DELITEM(p_midi_attr->p_playing_head)
	}
	DELARRAY(preparing_strbuf)
	DELARRAY(playing_strbuf)
	result=midiStreamClose(p_midi_attr->outst); CHECKRES(9);
	DELITEM(p_midi_attr);
	p_midi_attr=new Midi_attributes;
	p_midi_attr->timediv_struct.cbStruct=sizeof(MIDIPROPTIMEDIV);
	p_midi_attr->tempo_struct.cbStruct=sizeof(MIDIPROPTEMPO);
	p_midi_attr->p_playing_head=NULL;
	p_midi_attr->p_old_head=NULL;
}

bool Midi_player::Buf_struct_extract()
{
	buf_struct.Clear();
	Midi_event* p_cur_event=NULL;
	Midi_track* p_cur_track=NULL;
	memcpy(buf_struct_act_pos, buf_struct_start_pos, num_tracks*sizeof(DWORD));
	bool more_data=true;
	bool more_events=false;
	bool evt_in_cur_buf;
	DWORD buf_len=str_buf_ticks_len();
	for (DWORD track=0;track<num_tracks;track++)
	{
		p_cur_track=&(working_struct[track]);
		buf_struct.Add(Midi_track());
		more_data=true;
		Midi_event next_event;
		if (buf_struct_start_pos[track]<p_cur_track->GetCount())
		{
			more_events=true;
			next_event=(*p_cur_track)[buf_struct_act_pos[track]];
			evt_in_cur_buf=((more_data)&&(next_event.get_time()<act_ticks+buf_len));
			while ((more_data)&&(next_event.get_time()<act_ticks+buf_len))
			{
				p_cur_event=&next_event;
				p_cur_event->set_time(p_cur_event->get_time()-act_ticks);
				buf_struct[track].Add(*p_cur_event);
				buf_struct_act_pos[track]=buf_struct_act_pos[track]+1;
				if (buf_struct_act_pos[track]<p_cur_track->GetCount())
				{
					more_data=true;
					next_event=(*p_cur_track)[buf_struct_act_pos[track]];
				}
				else 
					more_data=false;
				buf_struct_start_pos[track]=buf_struct_act_pos[track];
			}
		}
	}
	act_ticks+=buf_len;
	return more_events;
}

inline DWORD Midi_player::str_buf_ticks_len()
{
	return (DWORD)(STR_BUF_TIME_LEN*PPQN/Get_act_tempo());
}

void Midi_player::Str_buf_prepare()
{
	if (playing_strbuf) 
	{
		midiOutUnprepareHeader ((HMIDIOUT)(p_midi_attr->outst), 
			p_midi_attr->p_old_head, sizeof(MIDIHDR)); CHECKRES(10);
		DELARRAY(playing_strbuf)
		DELARRAY(p_midi_attr->p_old_head)
	} 
	playing_strbuf=preparing_strbuf;
	p_midi_attr->p_old_head=p_midi_attr->p_playing_head;
	STR_BUF_LEN=(p_preparing_track->GetCount()+10)*12;
	preparing_strbuf=new char[STR_BUF_LEN];
	memset(preparing_strbuf, 0, STR_BUF_LEN);
	p_midi_attr->p_playing_head=new MIDIHDR;
	memset(p_midi_attr->p_playing_head, 0, sizeof(MIDIHDR));
	Midi_event start_evt, end_evt;
	p_preparing_track->Insert(start_evt, 0);
	if (external_tempo_change) External_tempo_change();
	end_evt.set_time(str_buf_ticks_len());
	p_preparing_track->Add(end_evt);
	Midi_event cur_event;
	DWORD prev_time=0;
	char* aux_p=preparing_strbuf;
	for (DWORD i=0;i<p_preparing_track->GetCount();i++)
	{
		cur_event=(*p_preparing_track)[i];
		if ((i==0)||(i==(p_preparing_track->GetCount()-1)))
			aux_p=Event_to_stream(cur_event, prev_time, aux_p, true);
		else
			aux_p=Event_to_stream(cur_event, prev_time, aux_p);
		prev_time=cur_event.get_time();
	}
	p_midi_attr->p_playing_head->lpData=preparing_strbuf;
	p_midi_attr->p_playing_head->dwBufferLength=STR_BUF_LEN;
	p_midi_attr->p_playing_head->dwBytesRecorded=12*(p_preparing_track->GetCount());
	result=midiOutPrepareHeader ((HMIDIOUT)(p_midi_attr->outst), 
			p_midi_attr->p_playing_head, sizeof(MIDIHDR)); CHECKRES(11);
	result=midiStreamOut(p_midi_attr->outst, 
		p_midi_attr->p_playing_head, sizeof(MIDIHDR)); CHECKRES(12);
}

void Midi_player::Stop()
{
	Set_state(STOPPED);
	if(p_player_thread!=NULL) //PB
	{
		p_player_thread->Wait();
		DELITEM(p_player_thread);
	}
	//p_player_thread=NULL;
}

char* Midi_player::Event_to_stream(Midi_event& evt, DWORD prev_time, 
								char* aux_p, bool no_op, bool tempo)
{
	BYTE status=evt.get_status();
	UNSIGNED_CHAR_ARRAY data=evt.get_data();
	BYTE channel=evt.get_channel();
	DWORD time=evt.get_time();
	memset(aux_p, 0, 12);
	DWORD deltat=time-prev_time;
	memcpy(aux_p, &(deltat), 4);
	aux_p+=4;
	memset(aux_p, 0, 4);
	aux_p+=4;
	if (Is_tempo(evt))
	{
		(*aux_p)=data[4];
		aux_p++;
		(*aux_p)=data[3];
		aux_p++;
		(*aux_p)=data[2];
		aux_p++;
		(*aux_p)=1;
		DWORD new_time=(data[2]<<16)+(data[3]<<8)+data[4];
		Set_act_tempo(new_time);
		Set_next_tempo(new_time);
	}
	else
	{
		(*aux_p)=(status<<4)+channel;
		aux_p+=1;
		if ((!no_op)&&(data.GetCount())) (*aux_p)=data[0];
		aux_p+=1;
		if ((!no_op)&&(data.GetCount()>1)) (*aux_p)=data[1];
		aux_p+=1;
		if (no_op) (*aux_p)=2;
	}
	aux_p+=1;
	return aux_p;
}

void Midi_player::Filter_track(Midi_track& track, Filter_mode mode, 
			BYTE status, BYTE channel, UNSIGNED_CHAR_ARRAY* p_data)
{
	Midi_event* p_evt;
	bool data_ok;
	bool evt_ok;
	DWORD i=0;
	while (i<track.GetCount())
	{
		evt_ok=false;
		p_evt=&(track[i]);
		switch(mode)
		{
		case SPECIFIC :
			if (p_data)
			{
				data_ok=true;
				for (DWORD y=0;y<p_data->GetCount();y++)
					if ((*p_data)[y]!=p_evt->get_data()[y]) 
						data_ok=false;
			}
			if ((status==p_evt->get_status())&&
				((channel==0xff)||(channel==p_evt->get_channel()))&&
				data_ok) evt_ok=false;
			break;
		case STATUS :
			if (status_filter[p_evt->get_status()]) evt_ok=true;
			if (Is_tempo(*p_evt)) evt_ok=true;
			break;
		case CHANNEL :
			if (channel_filter[p_evt->get_channel()]) evt_ok=true;
			break;
		}

///// PB
		int s=p_evt->get_status();

		if(endMeasure!=-1 && p_frame==NULL && p_evt->get_p_moodsinfo()->p_item!=NULL && ((s==NOTE_ON)||(s==NOTE_OFF)))
		{
			Battuta* pBat=((Figura*)p_evt->get_p_moodsinfo()->p_item)->GetPBattuta();
			if(pBat!=NULL && pBat->GetNumProgress()>endMeasure)
			{
				Set_state(STOPPED);
				break;
			}
		}
///// PB

		if (!evt_ok) track.RemoveAt(i);
		else i++;
	}
}

PLAYER_STATE Midi_player::Get_state()
{
	PLAYER_STATE _state;
	state_access.Enter();
	_state=state;
	state_access.Leave();
	return _state;
}
	
Midi_track* Midi_player::Get_p_playing_track()
{
	wxCriticalSectionLocker locker(p_playing_track_access);
	return p_playing_track;
}

Midi_event& Midi_player::Search_tempo(Midi_track& track, DWORD skip)
{
	DWORD ind=0;
	while (ind<track.GetCount())
	{
		if (Is_tempo(track[ind])) return track[ind];
		ind++;
	}
	throw wxString("Tempo event not found !");
}

void Midi_player::Set_next_tempo(DWORD tempo)
{
	external_tempo_change_access.Enter();
	next_tempo=tempo;
	external_tempo_change_access.Leave();
}

DWORD Midi_player::Get_next_tempo()
{
	wxCriticalSectionLocker locker(external_tempo_change_access);
	return next_tempo;
}

void Midi_player::External_tempo_change()
{
	DWORD tempo;
	external_tempo_change_access.Enter();
	tempo=next_tempo;
	external_tempo_change=false;
	external_tempo_change_access.Leave();
	Midi_event tempo_evt(0, GLOBAL_EVT, 15, 0);
	UNSIGNED_CHAR_ARRAY data;
	data.Add(0x51);
	data.Add(3);
	data.Add((tempo>>16)&(0xFF));
	data.Add((tempo>>8)&(0xFF));
	data.Add((tempo)&(0xFF));
	tempo_evt.set_data(data);
	p_preparing_track->Insert(tempo_evt, 1);
}

void Midi_player::Set_new_time(DWORD time)
{
	wxCriticalSectionLocker locker(external_tempo_change_access);
	if (time==next_tempo) return;
	external_tempo_change=true;
	next_tempo=time;
}

void Midi_player::Program_changes_insert(DWORD ticks)
{
	send_programs=false;
	DWORD index=0;
	Midi_event* p_evt=NULL;
	Midi_track* p_track=NULL;
	BYTE status;
	UNSIGNED_CHAR_ARRAY* p_data=NULL;
	for (DWORD track=0;track<working_struct.Get_Num_tracks();track++)
	{
		p_track=&(working_struct[track]);
		if (p_track->GetCount())
		{
			p_evt=&((*p_track)[0]);
			while ((index<p_track->GetCount()-1)&&(p_evt->get_time()<=ticks))
			{
				status=p_evt->get_status();
				p_data=&(p_evt->get_data());
				if ((status==PROGRAM)||
					((status==CONTROL)&&
					 ((p_data->Item(0)==CHANNEL_VOLUME)||
					  (p_data->Item(0)==PAN))))
				{
					Midi_event evt;
					evt=*p_evt;
					evt.set_time(p_preparing_track->Item(0).get_time());
					p_preparing_track->Insert(evt, 0);
				}
				index++;
				p_evt=&((*p_track)[index]);
			}
		}
		index=0;
	}
}

//////////MIDI_SEQUENCER IMPLEMENTATION//////////

Midi_sequencer::Midi_sequencer(wxWindow* _p_frame)
{
	p_frame=_p_frame;
	p_virtual_player_thread=new Midi_virtual_player_thread(this);
	p_virtual_player_thread->Create();
	p_player_thread=new Midi_player_thread(this);
	p_player_thread->Create();
	start_bat=1;
	ticks_from_bat=0;
	PPQN=MOODS_PPQN;
	restarted=false;
}

void Midi_sequencer::On_Play()
{	
	Play_prepare();
	bool first_buf=true;
	bool stopped=false;
	while ((!stopped)&&(Buf_struct_extract()))
	{
		p_preparing_track=new Midi_track;
		buf_struct.Merge_tracks_by_time(p_preparing_track);
		if (send_programs) Program_changes_insert(act_ticks);
		Filter_track(*p_preparing_track, STATUS);
		if (Get_state()==STOPPED) break; //PB
		Str_buf_prepare(); 
		if (!first_buf) 
		{
			end_play.Wait();
			//PB
			if (Get_state()==PAUSED) 
			{
				midiStreamPause(p_midi_attr->outst);
				restart.Wait();
				midiStreamRestart(p_midi_attr->outst); 
			}
			//PB
		}
		else
		{
			Set_p_playing_track(p_preparing_track);
			result=midiStreamRestart(p_midi_attr->outst); CHECKRES(5);
			p_virtual_player_thread->Set_p_track(p_playing_track);
			refresh.Signal();
			first_buf=false;
		}
		if (Get_state()==STOPPED) stopped=true;
	}
	Set_state(STOPPED);
	end_play.Wait();
	Restore();
}

void Midi_sequencer::Play_prepare()
{
	num_tracks=working_struct.Get_Num_tracks();
	p_midi_attr->tempo_struct.dwTempo=400000;
	p_midi_attr->timediv_struct.dwTimeDiv=PPQN;
	result=midiStreamOpen(&(p_midi_attr->outst), &MIDIOUTDEV, DWORD(1), 
		(DWORD)MidiCallProc, DWORD(this), CALLBACK_FUNCTION); 
	CHECKRES(0,"Midi Mapper device \n""used by another \napplication !");
	result=midiStreamProperty(p_midi_attr->outst, 
		(LPBYTE)&(p_midi_attr->timediv_struct), 
		MIDIPROP_SET | MIDIPROP_TIMEDIV ); CHECKRES(3);
	if (!num_tracks) throw wxString("No events to play !");
	buf_struct_act_pos=new DWORD[num_tracks];
	buf_struct_start_pos=new DWORD[num_tracks];
	memset(buf_struct_act_pos, 0, num_tracks*sizeof(DWORD));
	memset(buf_struct_start_pos, 0, num_tracks*sizeof(DWORD));
	if (start_bat>1) Play_from_measure_prepare();
	Set_state(PLAYING);
	p_virtual_player_thread->Run();
}

void Midi_sequencer::Play_from_measure_prepare()
{
	send_programs=true;
	Midi_position_mapper mapper;
	mapper.Build_measure_map(working_struct);
	DWORD start_offset=
		mapper.Get_bat_start(start_bat, buf_struct_start_pos);
	act_ticks=start_offset;
	tempo_mapper.Build_map(working_struct);
	act_tempo=tempo_mapper.Get_tempo(act_ticks);
	p_midi_attr->tempo_struct.dwTempo=act_tempo;
	result=midiStreamProperty(p_midi_attr->outst, 
	(LPBYTE)&(p_midi_attr->tempo_struct), 
	MIDIPROP_SET | MIDIPROP_TEMPO); CHECKRES(24);
}

void Midi_sequencer::Play(DWORD _start_bat)
{
	start_bat=_start_bat;
	p_player_thread->Run();
}

void Midi_sequencer::On_MOM_DONE()
{
	p_playing_track_access.Enter();
//	DELITEM(p_playing_track)
	p_playing_track=p_preparing_track;
	p_playing_track_access.Leave();
	end_play.Signal(); 
	if ((TIMING_CHECK)&&
		(Get_state()==PLAYING)&&
		(!p_virtual_player_thread->Get_end_play()))
		wxMessageBox("Timing Error!");
	p_virtual_player_thread->Set_p_track(p_playing_track);
	refresh.Signal();
}

Midi_sequencer::~Midi_sequencer()
{

}

MIDI_SYNC_RESULT Midi_sequencer::New_bat(DWORD prev_division, DWORD* p_var)
{
	DWORD TOLERANCE=(SYNC_MEASURE_TOLERANCE*prev_division)/2;
	DWORD tracking_zero=
		prev_division-TOLERANCE+TRACKING_ZERO_OFFSET*prev_division;
	INT _ticks_from_bat=Get_ticks_from_bat();
	INT error=_ticks_from_bat-tracking_zero;
	Set_ticks_from_bat(_ticks_from_bat-prev_division);
	DWORD vel_inc=(abs(error)*1000)/prev_division;
	(*p_var)=(vel_inc);
	if ((DWORD)abs(error)<TOLERANCE) return MIDI_SYNC_OK;
	else
		if ((DWORD)abs(error)>SYNC_ERROR_TOLERANCE*prev_division) 
		{
			return MIDI_SYNC_ERROR;
			wxMessageBox("ERROR");
		}
		else
			if (error>0) return MIDI_SYNC_INC;
			else
				if (vel_inc>=1000) return MIDI_SYNC_ERROR;
				else
					return MIDI_SYNC_DEC; 

	return MIDI_SYNC_ERROR;
}

void Midi_sequencer::Set_ticks_from_bat(INT ticks)
{
	ticks_from_bat_access.Enter();
	ticks_from_bat=ticks;
	ticks_from_bat_access.Leave();
}

INT Midi_sequencer::Get_ticks_from_bat()
{
	wxCriticalSectionLocker locker(ticks_from_bat_access);
	return ticks_from_bat;
}

void Midi_sequencer::Ticks_from_bat_update(DWORD ticks)
{
	ticks_from_bat_access.Enter();
	ticks_from_bat+=ticks;
	ticks_from_bat_access.Leave();
}

void Midi_sequencer::Set_virtual_player_priority(BYTE priority)
{
	if (!p_virtual_player_thread) return;
	if (p_virtual_player_thread->IsAlive()) return;
	if (p_virtual_player_thread->IsPaused()) return;
	p_virtual_player_thread->SetPriority(priority);
}

void Midi_sequencer::Set_working_struct(Midi_struct& midi_struct)
{
	working_struct.Clear();
	working_struct=midi_struct;
}

////// PB
void Midi_sequencer::Pause()
{
	switch (Get_state())
	{
	case PLAYING : 
		Set_state(PAUSED); 
		break;
	case PAUSED : 
		Set_state(PLAYING); 
		restart.Signal();
		break;
	}
}

void Midi_sequencer::Stop()
{
	if(p_frame==NULL && Get_state()==PAUSED)
		Pause(); // restarts

	Midi_player::Stop();
}

void Midi_sequencer::Set_end_measure(int nMeasure)
{
	endMeasure=nMeasure;
};

///// PB

/////MIDI_INTERNAL_PLAYER IMPLEMENTATION/////////

Midi_internal_player::Midi_internal_player(wxString filename)
{
	Open_file(filename);
	p_player_thread=new Midi_player_thread(this);
	p_player_thread->Create();
	position=0;
	start_ticks=0;
	speed_rate=0;
}

Midi_internal_player::Midi_internal_player(wxInputStream * inputStream)
{
	wxASSERT(inputStream!=0);
	inputStream->SeekI(0);
	Open_file(inputStream);
	p_player_thread=new Midi_player_thread(this);
	p_player_thread->Create();
	position=0;
	start_ticks=0;
	speed_rate=0;
}

void Midi_internal_player::On_MOM_DONE()
{
	p_playing_track_access.Enter();
	DELITEM(p_playing_track)
	p_playing_track=p_preparing_track;
	p_playing_track_access.Leave();
	end_play.Signal(); 
}

void Midi_internal_player::On_Play()
{
	Play_prepare();
	bool first_buf=true;
	bool stopped=false;
	while ((!stopped)&&(Buf_struct_extract()))
	{
		Set_position(act_ticks);
		p_preparing_track=new Midi_track;
		buf_struct.Merge_tracks_by_time(p_preparing_track);
		if (send_programs) Program_changes_insert(act_ticks);
		Filter_track(*p_preparing_track, STATUS);
		Str_buf_prepare(); 
		if (!first_buf) 
		{
			end_play.Wait();
			if (Get_state()==PAUSED) 
			{
				midiStreamPause(p_midi_attr->outst);
				restart.Wait();
				midiStreamRestart(p_midi_attr->outst); 
			}
		}
		else
		{
			Set_p_playing_track(p_preparing_track);
			result=midiStreamRestart(p_midi_attr->outst); CHECKRES(5);
			first_buf=false;
		}
		if (Get_state()==STOPPED) stopped=true;
	}
	Set_state(STOPPED);
	end_play.Wait();
	Restore();
}

void Midi_internal_player::Play_prepare()
{
	num_tracks=working_struct.Get_Num_tracks();
	p_midi_attr->tempo_struct.dwTempo=400000;
	p_midi_attr->timediv_struct.dwTimeDiv=PPQN;
	result=midiStreamOpen(&(p_midi_attr->outst), &MIDIOUTDEV, DWORD(1), 
		(DWORD)MidiCallProc, DWORD(this), CALLBACK_FUNCTION); 
	CHECKRES(0,"Midi Mapper device \n""used by another \napplication !");
	result=midiStreamProperty(p_midi_attr->outst, 
		(LPBYTE)&(p_midi_attr->timediv_struct), 
		MIDIPROP_SET | MIDIPROP_TIMEDIV ); CHECKRES(3);
	if (!num_tracks) throw wxString("No events to play !");
	buf_struct_act_pos=new DWORD[num_tracks];
	buf_struct_start_pos=new DWORD[num_tracks];
	memset(buf_struct_act_pos, 0, num_tracks*sizeof(DWORD));
	memset(buf_struct_start_pos, 0, num_tracks*sizeof(DWORD));
	if (start_ticks) Play_from_ticks_prepare();
	Set_state(PLAYING);
}

const BYTE TEMPO_STEP_VARIATION=5;

INT Midi_internal_player::Slower()
{
	DOUBLE original_tempo=tempo_mapper.Get_tempo(GetPosition());
	DWORD tempo_variation=(DWORD)ceil((original_tempo*TEMPO_STEP_VARIATION)/100);
	DWORD actual_tempo=Get_act_tempo();
	DOUBLE new_tempo=actual_tempo+tempo_variation;
	speed_rate+=TEMPO_STEP_VARIATION;
	Set_new_time(new_tempo);
	return speed_rate;
}

INT Midi_internal_player::Faster()
{
	DOUBLE original_tempo=tempo_mapper.Get_tempo(GetPosition());
	DWORD tempo_variation=(DWORD)ceil((original_tempo*TEMPO_STEP_VARIATION)/100);
	DWORD actual_tempo=Get_act_tempo();
	DOUBLE new_tempo=actual_tempo-tempo_variation;
	speed_rate-=TEMPO_STEP_VARIATION;
	Set_new_time(new_tempo);
	return speed_rate;
}

FLOAT Midi_internal_player::GetSpeed()
{
	return speed_rate;
}

void Midi_internal_player::NormalSpeed()
{
	Set_new_time(tempo_mapper.Get_tempo(GetPosition()));
	speed_rate=0;
}

INT Midi_internal_player::GetDuration()
{
	DWORD num_tks=working_struct.Get_Num_tracks();
	Midi_event* p_evt=NULL;
	Midi_track* p_track=NULL;
	DWORD max_time=0;
	DWORD cur_time=0;
	for (DWORD track=0;track<num_tks;track++)
	{
		p_track=&(working_struct[track]);
		if (!p_track->IsEmpty())
		{
			p_evt=&(p_track->Last());
			cur_time=p_evt->get_time();
			if (cur_time>max_time) max_time=cur_time;
		}
	}
	return max_time;
}

void Midi_internal_player::Open_file(wxString filename)
{
    Midi_decoder dec;
	working_struct.Clear();
	dec.Set_channel_struct(&working_struct);
	dec.Set_track_struct(&working_struct);
	dec.Load_file(filename);
	Midi_format format=(Midi_format)dec.Get_format();
	
	Midi_event tempo_event;
	switch (format)
	{
	case SMF0 :
		dec.SMF_decode(CHANNEL_STRUCT);
		working_struct.Set_struct_mode(CHANNEL_STRUCT);
		break;
	case SMF1 :
		dec.SMF_decode(TRACK_STRUCT);
		working_struct.Set_struct_mode(TRACK_STRUCT);
		break;
	} // non serve specificare il caso di default perch se 
	  // il formato  >1 dec lancia un'eccezione

	tempo_mapper.Build_map(working_struct);
	start_tempo=tempo_mapper.Get_tempo(0);
	Set_act_tempo(start_tempo);
	PPQN=dec.Get_PPQN();
}

void Midi_internal_player::Open_file(wxInputStream * inputStream)
{
    Midi_decoder dec;
	working_struct.Clear();
	dec.Set_channel_struct(&working_struct);
	dec.Set_track_struct(&working_struct);
	dec.Load_file(inputStream);
	Midi_format format=(Midi_format)dec.Get_format();
	
	Midi_event tempo_event;
	switch (format)
	{
	case SMF0 :
		dec.SMF_decode(CHANNEL_STRUCT);
		working_struct.Set_struct_mode(CHANNEL_STRUCT);
		break;
	case SMF1 :
		dec.SMF_decode(TRACK_STRUCT);
		working_struct.Set_struct_mode(TRACK_STRUCT);
		break;
	} // non serve specificare il caso di default perch se 
	  // il formato  >1 dec lancia un'eccezione

	tempo_mapper.Build_map(working_struct);
	start_tempo=tempo_mapper.Get_tempo(0);
	Set_act_tempo(start_tempo);
	PPQN=dec.Get_PPQN();
}

void Midi_internal_player::Run()
{
	p_player_thread->Run();	
}

void Midi_internal_player::Pause()
{
	switch (Get_state())
	{
	case PLAYING : 
		Set_state(PAUSED); 
		break;
	case PAUSED : 
		Set_state(PLAYING); 
		restart.Signal();
		break;
	}
}

void Midi_internal_player::Pause(PLAYER_STATE playerState)
{
	Set_state(playerState);
}

void Midi_internal_player::Play_from_ticks_prepare()
{
	send_programs=true;
	Midi_position_mapper mapper;
	mapper.Get_start_array(start_ticks, buf_struct_start_pos, working_struct);
	act_ticks=start_ticks;
	tempo_mapper.Build_map(working_struct);
	tempo_mapper.Get_tempo(act_ticks);
	p_midi_attr->tempo_struct.dwTempo=act_tempo;
	result=midiStreamProperty(p_midi_attr->outst, 
	(LPBYTE)&(p_midi_attr->tempo_struct), 
	MIDIPROP_SET | MIDIPROP_TEMPO); CHECKRES(48);
}

INT Midi_internal_player::GetPosition()
{ 
	wxCriticalSectionLocker locker(position_access);
	return (position);
}

void Midi_internal_player::Set_position(INT pos)
{
	position_access.Enter();
	position=pos;
	position_access.Leave();
}

void Midi_internal_player::JumpTo(wxInputStream * inputStream,INT pos)
{
	if(!p_player_thread)
	{
		midiStreamStop(p_midi_attr->outst);
		midiOutClose((HMIDIOUT)p_midi_attr->outst);
		return;
	}
	/*p_midi_player->Stop();
	p_midi_player=new Midi_internal_player(this->inputStream);
	p_midi_player->Set_start_ticks(time*MIDI_SLIDER_RESOLUTION_SCALE);
	p_midi_player->Run();*/
	/*position_access.Enter();
	position=pos;
	position_access.Leave();*/
	/*	
	Open_file(inputStream);
	p_player_thread=new Midi_player_thread(this);
	p_player_thread->Create();
	position=0;
	start_ticks=0;
	speed_rate=0;*/
	//Set_position(pos);
	//midiStreamPause(p_midi_attr->outst);
	midiStreamRestart(p_midi_attr->outst);
	//Restore();
	Pause(PAUSED);
	p_player_thread->Wait();
	Set_start_ticks(0);
	Play_from_ticks_prepare();
	Pause(PLAYING);
	p_player_thread->Wait();
	//midiStreamStop(p_midi_attr->outst);
	midiStreamPause(p_midi_attr->outst);
	midiOutReset((HMIDIOUT)p_midi_attr->outst);
	midiStreamRestart(p_midi_attr->outst);
	Pause(PAUSED);
	p_player_thread->Wait();
	Set_start_ticks(pos);
	Play_from_ticks_prepare();
	//Run();
	Pause(PLAYING);
	p_player_thread->Wait();
	//midiStreamRestart(p_midi_attr->outst);
}

wxString Midi_internal_player::GetInformation()
{
	return wxString("");
}

void Midi_internal_player::Set_start_ticks(DWORD ticks)
{
	start_ticks=ticks;
}

void Midi_internal_player::Stop()
{
	Set_state(STOPPED);
	Sleep(1000);
	DELITEM(p_player_thread);
}
/////MIDI_PLAYER_THREAD IMPLEMENTATION///////////

Midi_player_thread::Midi_player_thread(Midi_player* _p_player) : 
	wxThread(wxTHREAD_JOINABLE)
{
	p_player=_p_player;
}

void* Midi_player_thread::Entry()
{
    try
	{
		p_player->On_Play();
	}
	CATCHWXSTRING
	return NULL;
}

//MIDI_VIRTUAL_PLAYER_THREAD IMPLEMENTATION//////

Midi_virtual_player_thread::Midi_virtual_player_thread(Midi_sequencer* _p_sequencer) 
{
	p_sequencer=_p_sequencer;
	p_frame=p_sequencer->Get_p_frame();
	cur_time=0;
	p_track=NULL;
}

void* Midi_virtual_player_thread::Entry()
{
 	Midi_event* p_evt;
	DWORD sleep_time=0;
	DWORD deltat=0;
	DWORD status;
	wxCommandEvent refresh_evt(EVT_MIDI, ID_NOTE);
	while (p_sequencer->Get_state()==PLAYING || (p_frame!=NULL && p_sequencer->Get_state()==PAUSED)) //PB
	{
		Set_end_play(true);
		p_sequencer->refresh.Wait(); 
		Set_end_play(false);
		Midi_track track=(*p_track);
		cur_time=0;
		for (DWORD index=0;index<track.GetCount();index++)
		{
    		tempo=p_sequencer->Get_act_tempo();
			p_evt=&(track[index]);
			deltat=p_evt->get_time()-cur_time;
			if (deltat) p_sequencer->Ticks_from_bat_update(deltat);
			status=p_evt->get_status();
		    sleep_time=((tempo/MOODS_PPQN)*deltat)/uSec_in_mSec;
			cur_time=p_evt->get_time();
			if (sleep_time>SLEEP_TIME_THRESHOLD) 
				Sleep(sleep_time*SLEEP_FACTOR);
        	if (dynamic_cast<wxMEScoreCanvas*>(p_frame->FindWindow(wxMEScoreCanvas_ID))!=NULL && p_evt->get_p_moodsinfo()->p_item!=NULL && ((status==NOTE_ON)||(status==NOTE_OFF)))
			{
				if (status==NOTE_ON) refresh_evt.SetId(ID_NOTEON);
				if (status==NOTE_OFF) refresh_evt.SetId(ID_NOTEOFF);
#ifndef WDFEDONLY
				refresh_evt.SetInt((DWORD)(p_evt->
					get_p_moodsinfo()->p_item));
#endif
				p_frame->FindWindow(wxMEScoreCanvas_ID)->AddPendingEvent(refresh_evt);
			}
		} 
	}
	return NULL;
}

bool Midi_virtual_player_thread::Get_end_play()
{
	wxCriticalSectionLocker locker(end_play_access);
	return end_play;
}

void Midi_virtual_player_thread::Set_end_play(bool _end)
{
	end_play_access.Enter();
	end_play=_end;
	end_play_access.Leave();
}
/////////////////////////////////////////////////
