/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#ifndef ___MIDISEQUENCER___
#define ___MIDISEQUENCER___

#include "midiconv.h"
#include "mididef.hpp"

class Battuta;
class Midi_player_thread;
struct Midi_attributes;
typedef float FLOAT;
typedef int INT;
class wxInputStream;


//////////MIDI_TEMPO_MAPPER//////////////////////

class Midi_tempo_mapper
{

// attributes

	Midi_track tempo_track;

public :

// public interface

	Midi_tempo_mapper();
	DWORD Get_tempo(DWORD _ticks);
	void Build_map(Midi_struct& _struct);
};

//////////MIDI_PLAYER////////////////////////////

enum PLAYER_STATE
{
	PLAYING,
	STOPPED,
	PAUSED,
	RESTARTED
};

class Midi_player 
{

protected :

	friend class Midi_player_thread;
	
// attributes

	wxWindow* p_frame;
	bool status_filter[16];
	bool channel_filter[16];
	PLAYER_STATE state;
	DWORD STR_BUF_LEN;
	DWORD start_tempo;
	DWORD next_tempo;
	DWORD act_tempo;
	DWORD PPQN;
	DWORD act_ticks;
	DWORD num_tracks;
	Midi_attributes* p_midi_attr;
	Midi_struct working_struct;
	Midi_player_thread* p_player_thread;
	
// hidden attributes

	char* playing_strbuf;
	char* preparing_strbuf;
	bool external_tempo_change;
	DWORD* buf_struct_act_pos;
	DWORD* buf_struct_start_pos;
	Midi_track* p_preparing_track;
	Midi_track* p_playing_track;
	Midi_struct buf_struct;
	wxCondition end_play;
	wxCondition	sync;
    wxCriticalSection state_access;
	wxCriticalSection act_tempo_access;
	wxCriticalSection p_playing_track_access;
	wxCriticalSection external_tempo_change_access;
	Midi_tempo_mapper tempo_mapper;
	bool send_programs;

	int endMeasure; //PB
	
//  private functions

	void Restore();
	void Str_buf_prepare();
	void Set_state(PLAYER_STATE _state);
	void Set_p_playing_track(Midi_track* p_track);
	void Set_next_tempo(DWORD tempo);
	void External_tempo_change();
	bool Buf_struct_extract();	
	virtual void On_Play()=0;
	DWORD Get_next_tempo();
	DWORD str_buf_ticks_len();
	Midi_track* Get_p_playing_track();
	Midi_event& Search_tempo(Midi_track& track, DWORD skip=0);
	void Filter_track(Midi_track& track, Filter_mode mode, 
		 BYTE status=0, BYTE channel=0xff, 
		 UNSIGNED_CHAR_ARRAY* p_data=NULL);
	char* Event_to_stream(Midi_event& evt, DWORD prev_time, 
          char* aux_p, bool no_op=false, bool tempo=false);

	void Program_changes_insert(DWORD ticks);
	
public :

// public interface

	Midi_attributes* Get_p_midi_attr();
	Midi_struct& Get_working_struct();
	PLAYER_STATE Get_state();
	wxWindow* Get_p_frame();
	DWORD Get_num_tracks();
	DWORD Get_act_tempo();
	DWORD Get_PPQN();
	Midi_player();
	virtual void Stop();
	void Set_PPQN(DWORD ppqn);
	void Set_act_tempo(DWORD tempo);
	void Set_new_time(DWORD time);
	virtual ~Midi_player();
	virtual void On_MOM_DONE()=0; 
};

//////////MIDI_PLAYER_THREAD/////////////////////

class Midi_player_thread : public wxThread 
{

// attributes

	Midi_player* p_player;

public :

// public interface

	Midi_player_thread(Midi_player* _p_player);
	void* Entry();
};

//////////MIDI_SEQUENCER/////////////////////////

enum 
{
	ID_NOTE,
	ID_NOTEON,
	ID_NOTEOFF,
	ID_SLOWER,
	ID_FASTER,
	ID_TICKS_UPDATE,
	ID_STOP
};

class Midi_sequencer : public Midi_player
{
	friend class Partitura;
	friend class MasaeWindow;
	friend class Midi_virtual_player_thread;

// attributes

	Midi_virtual_player_thread* p_virtual_player_thread;
	wxCondition refresh;
	wxCondition restart;
	INT ticks_from_bat;
	DWORD start_bat;
	wxCriticalSection ticks_from_bat_access;
	bool restarted;

// private functions

	void On_Play();
	void Play_prepare();
	void Play_from_measure_prepare();

public :
	
// public interface

	void On_MOM_DONE();
	void Play(DWORD _start_bat);
	void Set_ticks_from_bat(INT ticks);
	void Ticks_from_bat_update(DWORD ticks);
	MIDI_SYNC_RESULT New_bat(DWORD prev_division, DWORD* p_var);
	Midi_sequencer(wxWindow* _p_frame);
	INT Get_ticks_from_bat();
	~Midi_sequencer();
	void Set_virtual_player_priority(BYTE priority);
	void Set_working_struct(Midi_struct& midi_struct);

	void Pause();
	void Stop();
	void Set_end_measure(int nMeasure);
};

//////////MIDI_INTERNAL_PLAYER///////////////////

class Midi_internal_player : public Midi_player
{

// attributes

	DWORD position;
	DWORD start_ticks;
	wxCondition restart;
	wxCriticalSection position_access;
	FLOAT speed_rate;

// private functions

	void On_Play();
	void Set_position(INT pos);
	void Open_file(wxString filename);
	void Open_file(wxInputStream *);
	void Play_from_ticks_prepare();
	void Play_prepare();
	
public :

// public interface

	void Run();
	void Stop();
	void Pause();
	void Pause(PLAYER_STATE);
	void On_MOM_DONE();
	void NormalSpeed();
	void JumpTo(wxInputStream *,INT time);
	void Set_start_ticks(DWORD ticks);
	Midi_internal_player(wxString filename);
	Midi_internal_player(wxInputStream *);
    wxString GetInformation();
	INT GetPosition();
	INT GetDuration();
	FLOAT GetSpeed();
	INT Slower();
	INT Faster();
};

//////////MIDI_VIRTUAL_PLAYER_THREAD/////////////

class Midi_virtual_player_thread : public wxThread
{

// attributes

	DWORD tempo;
	DWORD cur_time;
	Midi_sequencer* p_sequencer;
	wxWindow* p_frame;
	Midi_track* p_track;
	bool end_play;
	wxCriticalSection end_play_access;
	void Set_end_play(bool _end);

public :

// public interface

	Midi_virtual_player_thread(Midi_sequencer* _p_sequencer);
	void Set_p_track(Midi_track* _p_track){p_track=_p_track;}
	void* Entry();
	bool Get_end_play();
};

/////////////////////////////////////////////////

#endif
