/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: diteggia.cpp,v 1.1 2005/07/25 16:31:18 bellinpo Exp $ */


#include "ind/diteggia.hpp"
#include "drw/gscreen.hpp"
#include "gui/dialogs.hpp"
#include "gui/lioomenu.hpp"


Diteggiato::Diteggiato()
{ ID=CL_DITEGGIATO; dito=1; }

char Diteggiato::GetCode()
{
	char code[2];
	
	switch (tipoDito){
	case 1:					//ROUNDED1:
		{
			switch (dito){
			case 0:
				code[0]=65;
				break;
			case 1:
				code[0]=66;
				break;
			case 2:
				code[0]=67;
				break;
			case 3:
				code[0]=68;
				break;
			case 4:
				code[0]=69;
				break;
			case 5:
				code[0]=70;
				break;
			default:
				code[0]=66;
				break;
			}
			
		};
		break;
	case 2:					//ROUNDED2:
		{
			switch (dito){
			case 0:
				code[0]=71;
				break;
			case 1:
				code[0]=72;
				break;
			case 2:
				code[0]=73;
				break;
			case 3:
				code[0]=74;
				break;
			case 4:
				code[0]=75;
				break;
			case 5:
				code[0]=76;
				break;
			default:
				code[0]=72;
				break;
			}
		};
		break;
	default:
		{
			switch (dito){
			case 0:
				code[0]='0';
				break;
			case 1:
				code[0]='1';
				break;
			case 2:
				code[0]='2';
				break;
			case 3:
				code[0]='3';
				break;
			case 4:
				code[0]='4';
				break;
			case 5:
				code[0]='5';
				break;
			default:
				code[0]='1';
				break;
			}
		};
		break;
	}
	
	return (code[0]);
}

VUnit Diteggiato::GetVU2Up()
{
	
	return Scr->Get_VU2UP(GetCode(),nMusicFont2);
};

VUnit Diteggiato::GetVU2Dwn()
{
	
	return Scr->Get_VU2DWN(GetCode(),nMusicFont2);
};

VUnit Diteggiato::GetVU2Lft()
{
	
	
	return Scr->Get_VU2LF(GetCode(),nMusicFont2);
};

VUnit Diteggiato::GetVU2Rgt()
{
	
	return Scr->Get_VU2RG(GetCode(),nMusicFont2);
};

void Diteggiato::SetDito(char dit) { dito=dit;};

void Diteggiato::Draw()
{
	int oldFont;
	char code[2];
	
	int deltax, deltay;
	
	switch (tipoDito){
	case 1:					//ROUNDED1:
		{
			switch (dito){
			case 0:
				code[0]=65;
				deltax=DELTAXDITEGGIATO1R0;
				deltay=DELTAYDITEGGIATO1R0;
				break;
			case 1:
				code[0]=66;
				deltax=DELTAXDITEGGIATO1R1;
				deltay=DELTAYDITEGGIATO1R1;
				break;
			case 2:
				code[0]=67;
				deltax=DELTAXDITEGGIATO1R2;
				deltay=DELTAYDITEGGIATO1R2;
				break;
			case 3:
				code[0]=68;
				deltax=DELTAXDITEGGIATO1R3;
				deltay=DELTAYDITEGGIATO1R3;
				break;
			case 4:
				code[0]=69;
				deltax=DELTAXDITEGGIATO1R4;
				deltay=DELTAYDITEGGIATO1R4;
				break;
			case 5:
				code[0]=70;
				deltax=DELTAXDITEGGIATO1R5;
				deltay=DELTAYDITEGGIATO1R5;
				break;
			default:
				code[0]=66;
				deltax=DELTAXDITEGGIATO1R1;
				deltay=DELTAYDITEGGIATO1R1;
				break;
			}
			
		};
		break;
	case 2:					//ROUNDED2:
		{
			switch (dito){
			case 0:
				code[0]=71;
				deltax=DELTAXDITEGGIATO2R0;
				deltay=DELTAYDITEGGIATO2R0;
				break;
			case 1:
				code[0]=72;
				deltax=DELTAXDITEGGIATO2R1;
				deltay=DELTAYDITEGGIATO2R1;
				break;
			case 2:
				code[0]=73;
				deltax=DELTAXDITEGGIATO2R2;
				deltay=DELTAYDITEGGIATO2R2;
				break;
			case 3:
				code[0]=74;
				deltax=DELTAXDITEGGIATO2R3;
				deltay=DELTAYDITEGGIATO2R3;
				break;
			case 4:
				code[0]=75;
				deltax=DELTAXDITEGGIATO2R4;
				deltay=DELTAYDITEGGIATO2R4;
				break;
			case 5:
				code[0]=76;
				deltax=DELTAXDITEGGIATO2R5;
				deltay=DELTAYDITEGGIATO2R5;
				break;
			default:
				code[0]=72;
				deltax=DELTAXDITEGGIATO2R1;
				deltay=DELTAYDITEGGIATO2R1;
				break;
			}
		};
		break;
	default:
		{
			switch (dito){
			case 0:
				code[0]='0';
				deltax=DELTAXDITEGGIATON0;
				deltay=DELTAYDITEGGIATON0;
				break;
			case 1:
				code[0]='1';
				deltax=DELTAXDITEGGIATON1;
				deltay=DELTAYDITEGGIATON1;
				break;
			case 2:
				code[0]='2';
				deltax=DELTAXDITEGGIATON2;
				deltay=DELTAYDITEGGIATON2;
				break;
			case 3:
				code[0]='3';
				deltax=DELTAXDITEGGIATON3;
				deltay=DELTAYDITEGGIATON3;
				break;
			case 4:
				code[0]='4';
				deltax=DELTAXDITEGGIATON4;
				deltay=DELTAYDITEGGIATON4;
				break;
			case 5:
				code[0]='5';
				deltax=DELTAXDITEGGIATON5;
				deltay=DELTAYDITEGGIATON5;
				break;
			default:
				code[0]='1';
				deltax=DELTAXDITEGGIATON1;
				deltay=DELTAYDITEGGIATON1;
				break;
			}
		};
		break;
	}
	
	code[1]='\0';
	
	oldFont=Scr->SetFont(nMusicFont2);
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	Scr->TextTo(GetAbsPos()+Point(deltax,deltay)+Point( GetDx(),GetDy() ));
	Scr->OutText((unsigned char *)code);
	Scr->SetFont(oldFont);

	Scr->SetColor(oldcol);
};


void Diteggiato::Edit(Battuta *pBat)

{
	
    EditingMarkersDlg(this);
	
}
