/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: esprcomp.cpp,v 1.1 2005/07/25 16:31:19 bellinpo Exp $ */


#include "ind/esprcomp.hpp"

#include "ind/staccato.hpp"
#include "ind/tenuto.hpp"
#include "ind/martella.hpp"
#include "ind/accento.hpp"
#include "ind/sforzato.hpp"
#include "ind/martdolc.hpp"
#include "ind/puntoall.hpp"
#include "ind/espgenerica.hpp"
#include "mil/simboli.hpp"
#include "liooext.hpp"
#include "gui/dialogs.hpp" 

extern int DebugNormalMilla;
extern int DebugNormalMillaPos;
extern wxString msgdebug;



EspressComposta::EspressComposta()
{
  int i;
  n_espressioni=0;
  for(i=1; i<=N_ESP_MAX; i++)
	espressioni[i-1]=NULL;
}


EspressComposta::~EspressComposta()
{
  int i;
  for(i=1; i<=N_ESP_MAX; i++)
	delete espressioni[i-1];
}


int EspressComposta::Getn_espressioni()
{
 return n_espressioni;
}

Espressione *EspressComposta::GetPEspressione(CmdID MyId)
{
  ClassID MyEsp;
 
  int i;
  switch (MyId) {
    case CMD_NOTA_TENUTO: MyEsp=CL_TENUTO; break;
    case CMD_NOTA_PUNTOALL: MyEsp=CL_PUNTOALLUNGATO; break;
    case CMD_NOTA_SFORZATO: MyEsp=CL_SFORZATO; break;
    case CMD_NOTA_ACCENTO: MyEsp=CL_ACCENTO; break;
    case CMD_NOTA_MARTELLATO: MyEsp=CL_MARTELLATO; break;
    case CMD_NOTA_MARTDOLCE: MyEsp=CL_MARTDOLCE; break;
    case CMD_NOTA_STACCATO: MyEsp=CL_STACCATO; break;
	case CMD_NOTA_ESPGENERIC: MyEsp=CL_ESPGENERIC; break;
	
	default: return NULL;
  }
  
 for(i=1; i<=n_espressioni; i++) 
 	if (NULL!=espressioni[i-1]) 
	   if (espressioni[i-1]->GetID()==MyEsp) return espressioni[i-1];
 
 return NULL;
}




Espressione *EspressComposta::GetPEspressione(ClassID symbol, int IDsym)
{
  int i;
  
  for (i=1; i<=n_espressioni; i++) 
  {
    if (espressioni[i-1]->GetID()==symbol)
	{
	  if(symbol==CL_ESPGENERIC)
	  {
	    if(((EspGenerica*)espressioni[i-1])->GetIDsym()==IDsym) 
		  return espressioni[i-1];
	  }
	  else return espressioni[i-1];
	}
  }
  return NULL;
}


Bool EspressComposta::InserisciEsp(ClassID CL_ESP, TipoInserimento tpIns, Nota*
pNota, Battuta* pBattuta, int IDsym)
{
  int i;


  if (DebugNormalMilla || DebugNormalMillaPos)
 { 

   msgdebug.sprintf("");

 }






  if (n_espressioni<N_ESP_MAX)       
   {
	//controllo se l'espressione da inserire c'e' gia'
	for (i=1; i<=n_espressioni; i++)
      if (espressioni[i-1]->GetID()==CL_ESP)
	  {
	    if(CL_ESP!=CL_ESPGENERIC)
		  break;
		else if(((EspGenerica*)espressioni[i-1])->GetIDsym()==IDsym)
		  break;
	  }  
	if (i<=n_espressioni)  //L'espressione c'era gia'.
       return FALSE;
     else 
      {
       for(i=n_espressioni-1; i>=0; i--)
    	if(espressioni[i]->GetID()>CL_ESP) 
          espressioni[i+1]=espressioni[i];
	 else break;
	 switch(tpIns)
          {
            case SOPRA:
	     aboveNota=TRUE;
	    break;
            case SOTTO:
	     aboveNota=FALSE;
	    break;
	    case AUTOMATICO:
		  if(CL_ESP==CL_ESPGENERIC)
		    aboveNota = immissione.ImmRuleGen(IDsym, pNota, pBattuta, NULL, 0);
		  else
            aboveNota = immissione.ImmRule(CL_ESP, pNota, pBattuta, NULL, 0); 
	    break;
	  }
       switch (CL_ESP)
    	{
         case CL_STACCATO:
	   espressioni[i+1]=(new Staccato);
	   break;
	 case CL_TENUTO:
	   espressioni[i+1]=(new Tenuto);
	   break;
	 case CL_MARTELLATO:
	   espressioni[i+1]=(new Martellato);
	   break;
	 case CL_ACCENTO:
	   espressioni[i+1]=(new Accento);
	   break;  
	 case CL_SFORZATO:
	   espressioni[i+1]=(new Sforzato);
	   break;  
	 case CL_MARTDOLCE:
	   espressioni[i+1]=(new MartDolce);
	   break;  
	 case CL_PUNTOALLUNGATO:
	   espressioni[i+1]=(new PuntoAllung);
	   break; 
	 case CL_ESPGENERIC:
	   espressioni[i+1]=(new EspGenerica(IDsym));
	   break;   
    	}
       n_espressioni++;
       espressioni[i+1]->SetAboveNota(aboveNota);



			
 if (DebugNormalMilla || DebugNormalMillaPos)
 { 
   posizionam.TestTokenCond(pNota, NULL, pBattuta, (int)NULL, 0);

   WDebugMilla(); 

 }





       return TRUE;
      }
	}
   else return FALSE;
}        	  

Bool EspressComposta::InserisciEsp(ClassID CL_ESP, Bool aboveNota, int IDsym)
{
  int i;
  if (n_espressioni<N_ESP_MAX)       
   {
	//controllo se l'espressione da inserire c'e' gia'
	for (i=1; i<=n_espressioni; i++)
      if (espressioni[i-1]->GetID()==CL_ESP) 	  
	  {
	    if(CL_ESP!=CL_ESPGENERIC)
		  break;
		else if(((EspGenerica*)espressioni[i-1])->GetIDsym()==IDsym)
		  break;
	  }  
	if (i<=n_espressioni)  //L'espressione c'era gia'.
       return FALSE;
     else 
      {
       for(i=n_espressioni-1; i>=0; i--)
    	if(espressioni[i]->GetID()>CL_ESP) 
          espressioni[i+1]=espressioni[i];
	 else break;
        switch (CL_ESP)
    	{
         case CL_STACCATO:
	   espressioni[i+1]=(new Staccato);
	   break;
	 case CL_TENUTO:
	   espressioni[i+1]=(new Tenuto);
	   break;
	 case CL_MARTELLATO:
	   espressioni[i+1]=(new Martellato);
	   break;
	 case CL_ACCENTO:
	   espressioni[i+1]=(new Accento);
	   break;  
	 case CL_SFORZATO:
	   espressioni[i+1]=(new Sforzato);
	   break;  
	 case CL_MARTDOLCE:
	   espressioni[i+1]=(new MartDolce);
	   break;  
	 case CL_PUNTOALLUNGATO:
	   espressioni[i+1]=(new PuntoAllung);
	   break;  
	 case CL_ESPGENERIC:
	   espressioni[i+1]=(new EspGenerica(IDsym));
	   break;  
    	}
       n_espressioni++;
       espressioni[i+1]->SetAboveNota(aboveNota);
       return TRUE;
      }
	}
   else return FALSE;
} 
       	  

Espressione *EspressComposta::GetPEspressione(int i)
{
 return espressioni[i-1];
};


Bool EspressComposta::DeleteSym(DrawObject *sym)
{        
  int i;
 for(i=1; i<=n_espressioni; i++)
  if(sym==espressioni[i-1])
   {
    delete espressioni[i-1];
    while (i<n_espressioni)
     {
      espressioni[i-1]=espressioni[i];
      i++;
     }
    espressioni[i-1]=NULL; 
    n_espressioni--;
    return TRUE;
   }
 return FALSE;
}      


Bool EspressComposta::Hit(const Point& p, SelObj selObj, DrawObject*& drwObjSel,
	       SymPath& symPath)
{
  int i;
 for(i=1; i<=n_espressioni; i++)
  if(espressioni[i-1]->Hit(p,selObj,drwObjSel,symPath))
    return TRUE;
 return FALSE;
}

      	       
Bool EspressComposta::SymbolicHit(SymPath& symPath, SelObj selObj, 
            DrawObject*& drwObjSel)
{
  int i;
 for(i=1; i<=n_espressioni; i++)
   if (espressioni[i-1]->SymbolicHit(symPath, selObj, drwObjSel))
     return TRUE;
 return FALSE;
}


void EspressComposta::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

  int i;
 for(i=1; i<=n_espressioni; i++)
   espressioni[i-1]->Draw();

 Scr->SetColor(oldcol);
}        

   
VUnit EspressComposta::GetVU2Up()
{  return 0;
};

VUnit EspressComposta::GetVU2Dwn()
{  return 0;
};

VUnit EspressComposta::GetVU2Lft()
{  return 0;
};

VUnit EspressComposta::GetVU2Rgt()
{  return 0;
}; 
