/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: viasord.cpp,v 1.1 2005/07/25 16:31:23 bellinpo Exp $ */


#include "ind/viasord.hpp"


ViaSord::ViaSord(ClassID CL_TIPOSORD)
{ 
	switch (CL_TIPOSORD)
	{ 
	case CL_VIASORDINAOTTONI:
		Indicazione::SetID(CL_VIASORDINAOTTONI);
		TipoSordina='o';
		break;
	case CL_VIASORDINAARCHI:
		Indicazione::SetID(CL_VIASORDINAARCHI);
		TipoSordina='a';
		break;
	case CL_VIASORDINATESTO:
		Text::SetID(CL_VIASORDINATESTO);
		Indicazione::SetID(CL_VIASORDINATESTO);
		SetFont(nTextFont);
		SetTxt((unsigned char*)"Via sord.");
		TipoSordina='t'; 
	}  
};

VUnit ViaSord::GetVU2Up()
{ 
	switch (TipoSordina)
	{
	case 'a':
	case 'o':
		switch (Indicazione::GetID())
		{
		case CL_VIASORDINAOTTONI:
			return VIASORDINAOTTONI2UP;
			break;
		case CL_VIASORDINAARCHI:
			return VIASORDINAARCHI2UP;
			break;
		}
		case 't':
			if (Text::GetID()==CL_VIASORDINATESTO)
				return Text::GetVU2Up()-3;
	}
	return 0;
};   


VUnit ViaSord::GetVU2Dwn()
{ 
	switch (TipoSordina)
	{
	case 'a':
	case 'o':
		switch (Indicazione::GetID())
		{
		case CL_VIASORDINAOTTONI:
			return VIASORDINAOTTONI2DWN;
			break;
		case CL_VIASORDINAARCHI:
			return VIASORDINAARCHI2DWN;
			break;
		}
		case 't':
			if (Text::GetID()==CL_VIASORDINATESTO)
				return Text::GetVU2Dwn()-2;
	}   
	return 0;
}; 

VUnit ViaSord::GetVU2Rgt()
{ 
	switch (TipoSordina)
	{
	case 'a':
	case 'o':
		switch (Indicazione::GetID())
		{
		case CL_VIASORDINAOTTONI:
			return VIASORDINAOTTONI2RGT;
			break;
		case CL_VIASORDINAARCHI:
			return VIASORDINAARCHI2RGT;
			break;
		}
		case 't':
			if (Text::GetID()==CL_VIASORDINATESTO)
				return Text::GetVU2Rgt()-8;
	}    
	return 0;
}; 

VUnit ViaSord::GetVU2Lft()
{ 
	switch (TipoSordina)
	{
	case 'a':
	case 'o':
		switch (Indicazione::GetID())
		{
		case CL_VIASORDINAOTTONI:
			return VIASORDINAOTTONI2LFT;
			break;
		case CL_VIASORDINAARCHI:
			return VIASORDINAARCHI2LFT;
			break;
		}
		case 't':
			if (Text::GetID()==CL_VIASORDINATESTO)
				return Text::GetVU2Lft()+6;
	}     
	return 0;
}; 


void ViaSord::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(Indicazione::isOpaque || Text::isOpaque)
		Scr->SetColor(GetColor());

	switch (TipoSordina)
	{
	case 'a':
	case 'o':
		switch (Indicazione::GetID())
		{
		case CL_VIASORDINAOTTONI:
			{
				char code[2]={0,0};
				code[0]=CODE_VIASORDINAOTTONI;
				Scr->TextTo(GetAbsPos() + Point(DELTAXVIASORDINAOTTONI,DELTAYVIASORDINAOTTONI)+Point( Indicazione::GetDx(),Indicazione::GetDy() ));
				Scr->OutText((unsigned char *)code);
				break;
			} 
		case CL_VIASORDINAARCHI:
			{         
				char code[2]={0,0};
				code[0]=(char)CODE_VIASORDINAARCHI;
				Scr->TextTo(GetAbsPos() + Point(DELTAXVIASORDINAARCHI,DELTAYVIASORDINAARCHI)+Point( Indicazione::GetDx(),Indicazione::GetDy() ));
				Scr->OutText((unsigned char *)code);
				break;
			} 
		}
		case 't':
			if (Text::GetID()==CL_VIASORDINATESTO)
			{
				FontID ofont;
				ofont=Scr->SetFont(font);
				Scr->TextTo(GetAbsPos() + Point(DELTAXVIASORDINATESTO,DELTAYVIASORDINATESTO)+Point( Indicazione::GetDx(),Indicazione::GetDy() ));
				if(txt!=NULL) Scr->OutText((unsigned char *)txt);
				Scr->SetFont(ofont);
			}
			
	}

	Scr->SetColor(oldcol);
	
};  

Bool ViaSord::Hit(const Point& p, SelObj selObj, DrawObject*& drwObjSel,
				  SymPath& symPath)
{
	
	switch (TipoSordina)
	{
	case 'a':
	case 'o':
		return Indicazione::Hit(p, selObj, drwObjSel, symPath);
	case 't':
		if (Text::Hit(p, selObj, drwObjSel, symPath))
		{
			drwObjSel = (Indicazione*)this;
			return TRUE;
		}
		else
			return FALSE;
	default: return FALSE;
	}
}
