/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: bend.cpp,v 1.1 2005/07/25 16:31:24 bellinpo Exp $ */


#include "int/bend.hpp"
#include "gui/dialogs.hpp"

Bend::Bend()
{
	//ID = CL_BEND;
	ID = CL_BEND; 
	blineType=SOLID_LINE ; 
	SetHeight(STANGHETTA + YSHIFT);
}

Bend::~Bend()
{
	
} 

int Bend::GetHeight(void)
{
	SetHeight(STANGHETTA + YSHIFT);
	// dimensione della stanghetta piu' la distanza del numero
	return (DrawObject::GetHeight());
}

void Bend::AdjustVUFig()
{
	Figura *pF1, *pF2;
	IntEstSegment *p1 = PFirstSegm[currentView];
	
	if (p1==NULL) return;
	
	pF1 = GetPFigStart();
	if (pF1==NULL) return;
	
	pF2 = GetPFigEnd();
	if (pF2==NULL) return;
	
	if(GetSopra())
	{
		VUnit d1=p1->AbsP.y-pF1->GetAbsPos().y+GetHeight();
		if(d1>pF1->GetVU2Up())
			pF1->SetVU2Up(d1);
		
		VUnit d2=p1->AbsP.y-pF2->GetAbsPos().y+GetHeight();
		if(d2>pF2->GetVU2Up())
			pF2->SetVU2Up(d2);
	}
	else
	{
		VUnit d1=-p1->AbsP.y+pF1->GetAbsPos().y+GetHeight();
		if(d1>pF1->GetVU2Dwn())
			pF1->SetVU2Dwn(d1);
		
		VUnit d2=-p1->AbsP.y+pF2->GetAbsPos().y+GetHeight();
		if(d2>pF2->GetVU2Dwn())
			pF2->SetVU2Dwn(d2);
	}
	
	return;
}

void Bend::SetDim(VUnit w)
{
	SetWidth(w);
	SetHeight(STANGHETTA + YSHIFT );
}

void Bend::SetLineType(LineType tp)
{ blineType = tp; }

LineType Bend::GetLineType()
{ return blineType; }



void Bend::SetColor(int col)
{
	DrawObject::SetColor((ColorIcon)col);
	return;
}


void Bend::DrawSegm(IntEstSegment *p1)
{ 
	Point p;
	Point from, to;
	
	if(isOpaque)
		Scr->SetColor(GetColor());
	if(Sopra)
	{ 
		p = Point(0+DELTAXBEND,0+DELTAYBEND);
		
		from = p1->AbsP + p +Point( GetDx(),GetDy() );
		to = from + Point(0,STANGHETTA); 
		Scr->Line(from, to, blineType);
		
		
		from = to;
		to = p1->AbsP + p + Point(GetVU2Rgt()+GetDxR()-GetDx()-2,STANGHETTA);
		Scr->Line(from, to, blineType);
		
		
		from = to;
		to = to + Point(0,-STANGHETTA);
		Scr->Line(from, to, blineType);
		
	}
	
	else
	{
		p = Point(0+DELTAXBEND,YSHIFT-DELTAYBEND);
		
		to = p1->AbsP + p +Point( GetDx(),GetDy() );
		from = to + Point(0,STANGHETTA); 
		Scr->Line(from, to, blineType);
		
		
		from = to;
		to = p1->AbsP + p + Point(GetVU2Rgt()+GetDxR()-GetDx()-2,0);
		Scr->Line(from, to, blineType);
		
		
		from = to;
		to = to + Point(0,STANGHETTA);
		Scr->Line(from, to, blineType);
		
	}
	
}

void Bend::ValutaLegature()
{
	Bool sopra=FALSE;
	Bool stemUpAll=FALSE;
	
	if (Interm && StemFirst==1 && StemLast==1 && StemInt==1)
		stemUpAll=TRUE;
	else if (StemFirst==1 && StemLast==1) 
		stemUpAll=TRUE;
	else stemUpAll=FALSE;
	
	if (Multiv && voice==1)  
		sopra = TRUE;
	else if (Multiv && voice==2)
		sopra = FALSE;
	else if (stemUpAll) 
		sopra = TRUE;
	else
		sopra = FALSE;
	
	
	SetSopra(sopra);
	
}     

void Bend::Edit(Battuta *pBat)

{
	
    EditingIntervalsDlg(this);
	
}
