/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: intestes.cpp,v 1.1 2005/07/25 16:31:27 bellinpo Exp $ */


#include "int/intestes.hpp"
#include "drw/slur.hpp"
#include "fig/nota.hpp"
#include "gui/dialogs.hpp"

IntEsteso::IntEsteso()
{
	for(int i=0;i<2;i++)
	{
		dxRMainScore[i]=0;
		dyRMainScore[i]=0;
		dxRPart[i]=0;
		dyRPart[i]=0;
	}
	
	PFigStart = PFigEnd = NULL;
	for(i=0; i<MAX_VIEW; i++)
	{
		PActSegm[i] = NULL;
		PFirstSegm[i] = NULL;
		FirstSegm[i].pNext = NULL;
		FirstSegm[i].Wdt = 0;
		FirstSegm[i].Hgt = 0;
		FirstSegm[i].AbsP = Point(0,0);
		FirstSegm[i].nGamboUp = 0;
		FirstSegm[i].nGamboDwn = 0;
		FirstSegm[i].nFigure = 0;
	}
	Sopra = TRUE;
}

IntEsteso::~IntEsteso()
{
	DeallocaSegm();
}

bool IntEsteso::IsLegOrBend()
{
	return ID==CL_LEGATURA || ID==CL_BEND || ID==CL_LEGATURA_QUADRA; 
}

IntEstSegment* IntEsteso::GetPFirstSegm()
{
	return PFirstSegm[currentView];
}

void IntEsteso::ResetPosSegm()
{
	PActSegm[currentView] = PFirstSegm[currentView];
}

void IntEsteso::GoNextActiveSegm()
{
	if (PActSegm[currentView]!=NULL)
		PActSegm[currentView] = PActSegm[currentView]->pNext;
	else
		EM.Error(MUSICAL_ERR_AREA,"EC_SEGM_NOTFOUND","IntEsteso","GoNextActiveSegm()");
}

void IntEsteso::SetPFigStart(Figura *pF)
{ 
	PFigStart = pF; 
}

void IntEsteso::SetPFigEnd(Figura *pF)
{ 
	PFigEnd = pF; 
}

Figura *IntEsteso::GetPFigStart()
{ 
	return PFigStart; 
}

Figura *IntEsteso::GetPFigEnd()
{ 
	return PFigEnd; 
}


void IntEsteso::SetSopra(Bool b)
{ 
	Sopra = b; 
}

Bool IntEsteso::GetSopra()
{ 
	return Sopra;
}

Bool IntEsteso::GetInizio()
{ return Inizio[currentView]; }

void IntEsteso::SetInizio(Bool b)
{ Inizio[currentView] = b; }

Bool IntEsteso::GetFine()
{ return Fine[currentView]; }

void IntEsteso::SetFine(Bool b)
{ Fine[currentView] = b; }

void IntEsteso::SetVoice(int a)
{ 
	voice = a; 
}

int IntEsteso::GetVoice()
{
	return voice;
}

void IntEsteso::SetMultivoice(Bool a)
{ 
	Multiv = a;
}

Bool IntEsteso::GetMultivoice()
{
	return Multiv;
}

void IntEsteso::SetStemInt(int a)
{ 
	StemInt = a; 
}

Bool IntEsteso::GetStemInt()
{
	return StemInt;
}

void IntEsteso::SetAuto(int a)
{ 
	Auto = a; 
}

Bool IntEsteso::GetAuto()
{
	return Auto;
}

void IntEsteso::SetStemFirst(int a)
{
	StemFirst = a;
}

int IntEsteso::GetStemFirst()
{
	return StemFirst;
}    

void IntEsteso::SetStemLast(int a)
{
	StemLast = a;
}

int IntEsteso::GetStemLast()
{
	return StemLast;
}

void IntEsteso::SetIntermed(int a)
{
	Interm = a;
}

Bool IntEsteso::GetIntermed()
{
	return Interm;
}    

Bool IntEsteso::TestAndSet(Figura *pFStart, Figura *pFEnd, Bool sopra)
{
	Bool result = TRUE;
	
	// Rifiuta i puntatori se coincidono.
	if(pFStart==pFEnd)
		return FALSE;
	
	// Questa classe accetta tutti i tipi di figura eccetto il GruppoNote ed
	// i segni di ripetizione.
	if (pFStart != NULL)
	{
		switch (pFStart->GetID())
		{
		case CL_RIPTEMPO:
		case CL_RIPBATTUTA:
		case CL_RIPMEZZABATTUTA:
		case CL_GRUPPONOTE:
			result = FALSE;
		}
	}
	
	if (pFEnd != NULL)
	{
		switch (pFEnd->GetID())
		{
		case CL_RIPTEMPO:
		case CL_RIPBATTUTA:
		case CL_RIPMEZZABATTUTA:
		case CL_GRUPPONOTE:
			result = FALSE;
		}
	}
	
	if(result)
	{
		// Imposta gli attributi.
		PFigStart = pFStart;
		PFigEnd = pFEnd;
		Sopra = sopra;
	}
	return result;
}   

void IntEsteso::SetAbsPosX(VUnit x)
{ 
	if (PActSegm[currentView]!=NULL)
		PActSegm[currentView]->AbsP.x = x; 
	else
		EM.Error(MUSICAL_ERR_AREA,"EC_SEGM_NOTFOUND","IntEsteso","SetAbsPosX()");
}

void IntEsteso::SetAbsPosY(VUnit y)
{ 
	if (PActSegm[currentView]!=NULL)
		PActSegm[currentView]->AbsP.y = y; 
	else
		EM.Error(MUSICAL_ERR_AREA,"EC_SEGM_NOTFOUND","IntEsteso","SetAbsPosY()");
}

void IntEsteso::SetWidth(VUnit w)
{
	if (PActSegm[currentView]!=NULL)
		PActSegm[currentView]->Wdt = w; 
	else
		EM.Error(MUSICAL_ERR_AREA,"EC_SEGM_NOTFOUND","IntEsteso","SetWidth()");
}

void IntEsteso::SetHeight(VUnit h)
{
	if (PActSegm[currentView]!=NULL)
	{
		if(h<INTERV_MIN_HEIGHT)
			PActSegm[currentView]->Hgt = INTERV_MIN_HEIGHT;
		else if(h>INTERV_MAX_HEIGHT)
			PActSegm[currentView]->Hgt = INTERV_MAX_HEIGHT;
		else
			PActSegm[currentView]->Hgt = h;
	}
	else
		EM.Error(MUSICAL_ERR_AREA,"EC_SEGM_NOTFOUND","IntEsteso","SetHeight()");
}


void IntEsteso::SetDim(VUnit w)
{
}


VUnit IntEsteso::GetVU2Up()
{ 
	if (PActSegm!=NULL)
		return PActSegm[currentView]->Hgt;
	else
		EM.Error(MUSICAL_ERR_AREA,"EC_SEGM_NOTFOUND","IntEsteso","GetVU2Up()");
	return 0;
}

VUnit IntEsteso::GetVU2Dwn()
{ 
	return 0; 
}

VUnit IntEsteso::GetVU2Lft()
{ 
	return 0; 
}

VUnit IntEsteso::GetVU2Rgt()
{ 
	if (PActSegm!=NULL)
		return PActSegm[currentView]->Wdt;  
	else
		EM.Error(MUSICAL_ERR_AREA,"EC_SEGM_NOTFOUND","IntEsteso","GetVU2Rgt()");
	return 0;
}



Bool IntEsteso::Hit(const Point& p, SelObj selObject,DrawObject*& drwObj,
					SymPath& symPath)
{
	// Non controlla se il tipo di oggetto e` ANY_SEL in quanto questo
	// controllo deve essere svolto da Spartito prima di andare a
	// cercare nella lista degli intervalli. 
	
	Bool found=FALSE;
	IntEstSegment *p1 = PFirstSegm[currentView];
	
    // Controlla solo i simboli attualmente posizionati.
	if (PFirstSegm[currentView]!=NULL)
		while ( (p1!=NULL)  && (!(found)) )
		{
		    if (class Rectangle(p1->AbsP+Point(GetDx(),GetDy()),0,p1->Wdt+GetDx()+GetDxR(),p1->Hgt,0).hasIn(p))
				found = TRUE;
			p1 = p1->pNext;
		}
		
		if (found)
		{
			drwObj=this;
			symPath.Array[SIMBOLO_PATH_LEVEL]=GetNumericCode();
			return TRUE;
		}
		else
			return FALSE;
}

Bool IntEsteso::SymbolicHit(SymPath& symPath, SelObj sobj, DrawObject *&  
                            objsel)
{
	objsel=NULL;
	if(symPath.Array[SIMBOLO_PATH_LEVEL]==GetNumericCode())
	{
		if(sobj==ANY_SEL)
			objsel=this;
		return TRUE;
	}
	return FALSE;
}

void IntEsteso::SetNumericCode(NumCode nc)
{ 
	NumericCode=nc; 
}

NumCode IntEsteso::GetNumericCode()
{ 
	return NumericCode; 
}


// aggiunge un segmento in coda alla lista degli IntEstSegment.
void IntEsteso::AddSegm()
{
	IntEstSegment *p1;
	
	
	if (PFirstSegm[currentView]==NULL)
	{
		PFirstSegm[currentView] = &FirstSegm[currentView];
		PActSegm[currentView] = PFirstSegm[currentView];
	}
	else
	{
		p1 = new IntEstSegment;
		if (p1==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","IntEsteso","AddSegm()");
		PActSegm[currentView]->pNext = p1;
		p1->pNext=NULL;
		PActSegm[currentView] = p1;
		PActSegm[currentView]->nGamboUp=0;
		PActSegm[currentView]->nGamboDwn=0;
		PActSegm[currentView]->nFigure=0;
		if(p1!=NULL)
		{
			p1->FirstFig=NULL;
			p1->LastFig=NULL;
		}
	}
	
}


// dealloca tutta la lista dei segmenti (tranne il primo, ovviamente).
void IntEsteso::DeallocaSegm()
{
	if(currentView<0)
		return;

	IntEstSegment *p1 = FirstSegm[currentView].pNext;
	
	PActSegm[currentView] = p1;
	while (p1!=NULL)
	{
		PActSegm[currentView] = PActSegm[currentView]->pNext;
		delete p1;
		p1 = PActSegm[currentView];
	}
	PActSegm[currentView] = NULL;
	FirstSegm[currentView].pNext = NULL;
	FirstSegm[currentView].Wdt = 0;
	FirstSegm[currentView].Hgt = 0;
	FirstSegm[currentView].AbsP = Point(0,0);
	FirstSegm[currentView].nGamboUp = 0;
	FirstSegm[currentView].nGamboDwn = 0;
	FirstSegm[currentView].nFigure = 0;
	PFirstSegm[currentView] = NULL;
}


Point IntEsteso::GetAbsPos()
{
	if (PActSegm[currentView] != NULL)
		return PActSegm[currentView]->AbsP;
	else 
		EM.Error(MUSICAL_ERR_AREA,"EC_SEGM_NOTFOUND","IntEsteso","GetAbsPos()");
	return Point();
}


void IntEsteso::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	PActSegm[currentView] = PFirstSegm[currentView];
		
	while (PActSegm[currentView]!=NULL)
	{
		DrawSegm(PActSegm[currentView]);
		PActSegm[currentView] = PActSegm[currentView]->pNext;
	}
	PActSegm[currentView] = NULL;
	
	Scr->SetColor(oldcol);
}

void IntEsteso::DrawSegm(IntEstSegment *)
{
}


void IntEsteso::SetBattuta(short n, Battuta *pb)
{
	if (n==1) pbat1 = pb;
	else pbat2 = pb;
}

Battuta *IntEsteso::GetBattuta(short n)
{
	if (n==1) return pbat1;
	else return pbat2;
}


void IntEsteso::UpdateGambiInterni(Figura *pF)
{ 
	if (PActSegm[currentView]!=NULL && pF!=PFigStart && pF!=PFigEnd)
	{
		PActSegm[currentView]->nFigure++;
		switch(pF->GetID())
		{
		case CL_ACCORDO:
		case CL_NMINIMA:
		case CL_NSEMIMINIMA:
		case CL_NCROMA:
		case CL_NSEMICROMA:
		case CL_NBISCROMA:
		case CL_NSEMIBISCROMA:
		case CL_NFUSA:
			Nota *pNota=(Nota*)pF;
			if(pNota->GetLGambo()>0)
				PActSegm[currentView]->nGamboUp++;
			else if(pNota->GetLGambo()<0)
				PActSegm[currentView]->nGamboDwn++;
		}
	}
	else
		EM.Error(MUSICAL_ERR_AREA,"EC_SEGM_NOTFOUND","IntEsteso","UpdateGambiInterni()");
}

void IntEsteso::Edit(Battuta *pBat)
{
    EditingMarkersDlg(this);
} 

VUnit IntEsteso::GetDxR(bool isMainScore, bool isPrintView)
{
	if(isMainScore)
		return VU_PER_ROW*GetDxRMainScore(isPrintView);
	else
		return VU_PER_ROW*GetDxRPart(isPrintView);
} 

VUnit IntEsteso::GetDyR(bool isMainScore, bool isPrintView)
{
	if(isMainScore)
		return VU_PER_ROW*GetDyRMainScore(isPrintView);
	else
		return VU_PER_ROW*GetDyRPart(isPrintView);
} 

VUnit IntEsteso::GetDxR()
{
	return GetDxR(tipoLeggio==LEGGIO_MASAE,currentViewType==1);
}

VUnit IntEsteso::GetDyR()
{
	return GetDyR(tipoLeggio==LEGGIO_MASAE,currentViewType==1);
}
