/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: intestes.hpp,v 1.1 2005/07/25 16:31:27 bellinpo Exp $ */


#ifndef INTESTESO_HPP
#define INTESTESO_HPP

#define DISTLEG 2 

#include "lioo.hpp"
#include "drw/drawobje.hpp"
#include "fig/figura.hpp"


typedef struct Segment {
	Point AbsP;
	VUnit Wdt, Hgt;
	Segment *pNext;
	int nGamboUp;
	int nGamboDwn;
	int nFigure;
	short numSegment;//aggiunto per gestire i segmenti del raggruppamento
	Figura *FirstFig;
	Figura *LastFig;
} IntEstSegment;


class IntEsteso: public DrawObject
{
protected:
    Figura *PFigStart, *PFigEnd;
	
    Battuta *pbat1, *pbat2;
    Bool Sopra;
    Bool Remove;
	// Utilizzato per gestire le liste Started di ListaIntEstesi 
	// con priorita' dettata dall'ordinamento.
	
    Bool Inizio[MAX_VIEW];
	// indica se l'intervallo ha inizio nella pagina
	
    Bool Fine[MAX_VIEW];
	// indica se l'intervallo ha fine nella pagina
	
	int voice;
    Bool Multiv;
    Bool StemInt; 
    Bool Auto; 
    int StemFirst;
    int StemLast;
    Bool Interm;  
	
    IntEstSegment FirstSegm[MAX_VIEW];
	// La cima della lista
    IntEstSegment* PFirstSegm[MAX_VIEW];
	// viene inizializzato a NULL (indica non posizionato), successivamente
	// indica la cima della lista.
    IntEstSegment* PActSegm[MAX_VIEW]; 
	// indica il segmento attuale che stiamo posizionando.
	
    NumCode NumericCode;
	// codice numerico dell' intervallo

   	float dxRMainScore[2],dyRMainScore[2],dxRPart[2],dyRPart[2];

public:
    IntEsteso();
    ~IntEsteso();
	
    void SetSopra(Bool b);
    Bool GetSopra();
	
    void SetRemove(Bool b) { Remove = b; };
    Bool GetRemove() { return Remove; };
	
    void SetNumericCode(NumCode nc);
    NumCode GetNumericCode();
	
    void SetNumBat(short,NumCode);
    NumCode GetNumBat(short);
	
	void SetBattuta(short n, Battuta *pb);
	Battuta *GetBattuta(short n);
	
    Bool GetInizio();
    void SetInizio(Bool);
	
    Bool GetFine();
    void SetFine(Bool);
	
	void SetVoice(int a);
    int GetVoice();
    
    void SetMultivoice(Bool a);
    Bool GetMultivoice();
    
    void SetStemInt(int a);
    Bool GetStemInt();
    
    void SetAuto(int a);
    Bool GetAuto();
    
    void SetStemFirst(int a);
    int GetStemFirst();
    
    void SetStemLast(int a);
    int GetStemLast();
    
    void SetIntermed(int a);
    Bool GetIntermed();
	
    void SetPFigStart(Figura *pF);
    void SetPFigEnd(Figura *pF);
    Figura *GetPFigStart();
    Figura *GetPFigEnd();
	
	
    void AddSegm();
	// aggiunge un segmento in coda alla lista degli IntEstSegment.
    void DeallocaSegm();
	// dealloca tutta la lista dei segmenti.
	
    IntEstSegment* GetPFirstSegm();
    void ResetPosSegm();    
	// posiziona PActSegm in cima alla lista.
    void GoNextActiveSegm();
	// Avanza PActSegm di una posizione.
	
    
	
    virtual Bool TestAndSet(Figura *pFStart, Figura *pFEnd, Bool sopra);
	// Controlla se i tipi di figura sono corretti per il tipo di
	// simbolo e, in caso affermativo, imposta gli attributi.
    
	
    Point GetAbsPos();
	// Ritorna la posizione assoluta del segmento attualmente attivo,
	
    void SetAbsPosX(VUnit x);
    void SetAbsPosY(VUnit y);
	// Impostano la posizione al segmento attivo.
	
    
    void SetWidth(VUnit w);
    void SetHeight(VUnit h);
    virtual void SetDim(VUnit w);
	// Imposta la larghezza e altezza al segmento attivo.
	
    VUnit GetVU2Up();
	// Ritorna l'altezza del segmento attualmente attivo,
    VUnit GetVU2Dwn();
	// Ritorna 0
    VUnit GetVU2Lft();
	// Ritorna 0
    VUnit GetVU2Rgt();
	// Ritorna la larghezza del segmento attualmente attivo,

   	float GetDxRMainScore(int viewType)  {return dxRMainScore[viewType];};
	float GetDyRMainScore(int viewType)  {return dyRMainScore[viewType];};
	float GetDxRPart(int viewType)		{return dxRPart[viewType];};
	float GetDyRPart(int viewType)		{return dyRPart[viewType];};
	VUnit GetDxR(bool isMainScore, bool isPrintView);
	VUnit GetDyR(bool isMainScore, bool isPrintView);
	VUnit GetDxR();
	VUnit GetDyR();

	void SetDxRMainScore(int viewType, float value)  {dxRMainScore[viewType]=value;};
	void SetDyRMainScore(int viewType, float value)  {dyRMainScore[viewType]=value;};
	void SetDxRPart(int viewType, float value)		{dxRPart[viewType]=value;};
	void SetDyRPart(int viewType, float value)		{dyRPart[viewType]=value;};

    void Draw();
	// Richiama il DrawSegm per ogni segmento.
	
    virtual void DrawSegm(IntEstSegment *);
	
	
    virtual void AdjustVUFig() {} ; 
    
    Bool Hit(const Point&, SelObj, DrawObject*&, SymPath&);
    Bool SymbolicHit(SymPath&, SelObj, DrawObject*&);
	
	virtual void ValutaLegature() {};
	
    void UpdateGambiInterni(Figura *pF);
	bool IsLegOrBend();
	
	virtual void SetLineType(LineType tp) {};
    virtual LineType GetLineType() {return SOLID_LINE;};
	virtual void Edit(Battuta *pBat);
	
	virtual int SaveXML(FILE *)=0;	
	int SaveXML(FILE *, char* type);

};

#endif //INTESTESO_HPP
