/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: leg.cpp,v 1.1 2005/07/25 16:31:27 bellinpo Exp $ */

#include "int/legatura.hpp"
#include "drw/slur.hpp"
#include "fig/nota.hpp"
#include "fig/accordo.hpp"

void Legatura::SetDim(VUnit w)
{
	SetWidth(w);
	SetHeight(w/8);
}

Bool Legatura::Hit(const Point& p, SelObj selObject,DrawObject*& drwObj,
				   SymPath& symPath)
{
	Slur sl;
	Point px;
	Figura *pF1, *pF2;
	Bool found=FALSE;
	IntEstSegment *p1 = PFirstSegm;
	
	if(p1==NULL)
		return FALSE;
    
	pF1 = GetPFigStart();
	pF2 = GetPFigEnd();
	
	px.x = (p1->AbsP).x+p1->Wdt;
	
	if (Sopra)
	{
		if (pF2->GetID()==CL_ACCORDO) 
			pF2 = ((Accordo *)pF2)->GetLastNote();
		if (pF1->GetID()==CL_ACCORDO) 
			pF1 = ((Accordo *)pF1)->GetLastNote();
		px.x = (p1->AbsP).x+p1->Wdt;
		
		if ((p1->pNext==NULL) && (Fine)) 
			px.y = ((pF2->GetAbsPos()).y)+pF2->GetVU2UpF()+DISTLEG;
		else px.y = (p1->AbsP).y;
		
		if (p1==PFirstSegm)	
			(p1->AbsP).y = ((pF1->GetAbsPos()).y)+pF1->GetVU2UpF()+DISTLEG;
		
		sl.SetBowUp(BOW_UP);
	}
	else
	{
		if (pF2->GetID()==CL_ACCORDO)
			pF2 = ((Accordo *)pF2)->GetFirstNote();
		if (pF1->GetID()==CL_ACCORDO)
			pF1 = ((Accordo *)pF1)->GetFirstNote();
		
		px.x = (p1->AbsP).x+p1->Wdt;
		
		if ((p1->pNext==NULL) && (Fine))
			px.y = ((pF2->GetAbsPos()).y)-pF2->GetVU2DwnF()-DISTLEG;
		else
			px.y = (p1->AbsP).y;
		
		if (p1==PFirstSegm)	
			(p1->AbsP).y = ((pF1->GetAbsPos()).y)-pF1->GetVU2DwnF()-DISTLEG;
		
		sl.SetBowUp(BOW_DOWN);
	}
	
	sl.SetSlur(p1->AbsP, px, Color);
	
	found=sl.hit(p);
	
	if (found)
	{
		drwObj=this;
		symPath.Array[SIMBOLO_PATH_LEVEL]=GetNumericCode();
		return TRUE;
	}
	else
		return FALSE;
}


void Legatura::DrawSegmOld(IntEstSegment *p1)
{
	Rectangle r;
	
	Scr->SetColor(GetColor());
	if(Sopra)
	{
		// Disegna la meta` superiore dell'ellisse. 
		
		
		r = Rectangle(p1->AbsP,0,p1->Wdt,p1->Hgt,
			0).Expand(0,0, 0, GetHeight());
		Scr->Arc(r, 0, 180);
		r = r.Expand(0, 0, -1, -1);
		Scr->Arc(r, 0, 180);
	}
	else
	{
		// Disegna la meta` inferiore dell'ellisse.
		r = Rectangle(p1->AbsP,0,p1->Wdt,p1->Hgt,
			0).Expand(0, 0, GetHeight(), 0);
		Scr->Arc(r, 0, -180);      
		r = r.Expand(0, 0, -1, -1);
		Scr->Arc(r, 0, -180);
	}
	
	
}

void Legatura::ValutaLegature()
{
	Slur slu;
	int bow;
	
	slu.SetAutoOrientation();
	slu.SetSlurParameter(Multiv, voice, StemFirst, StemLast, StemInt, Interm);
	slu.EvalBowOrientation();
	if (slu.GetBowUp() == TRUE) SetSopra(TRUE);
	else SetSopra(FALSE);
	
}     


void Legatura::DrawSegm(IntEstSegment *p1)
{
	Slur sl;
	Point px;
	Figura *pF1, *pF2;
	
	pF1 = GetPFigStart();
	if (pF1==NULL) return;
	
	pF2 = GetPFigEnd();
	if (pF2==NULL) return;
	
	
	if (Sopra)
	{
		if (pF2->GetID()==CL_ACCORDO) 
			pF2 = ((Accordo *)pF2)->GetLastNote();
		if (pF1->GetID()==CL_ACCORDO) 
			pF1 = ((Accordo *)pF1)->GetLastNote();
		px.x = (p1->AbsP).x+p1->Wdt;
		
		if ((p1->pNext==NULL) && (Fine)) 
			px.y = ((pF2->GetAbsPos()).y)+pF2->GetVU2UpF()+DISTLEG;
		else px.y = (p1->AbsP).y;
		
		if (p1==PFirstSegm)	
			(p1->AbsP).y = ((pF1->GetAbsPos()).y)+pF1->GetVU2UpF()+DISTLEG;
		
		sl.SetBowUp(BOW_UP);
	}
	else
	{
		if (pF2->GetID()==CL_ACCORDO)
			pF2 = ((Accordo *)pF2)->GetFirstNote();
		if (pF1->GetID()==CL_ACCORDO)
			pF1 = ((Accordo *)pF1)->GetFirstNote();
		
		px.x = (p1->AbsP).x+p1->Wdt;
		
		if ((p1->pNext==NULL) && (Fine))
			px.y = ((pF2->GetAbsPos()).y)-pF2->GetVU2DwnF()-DISTLEG;
		else
			px.y = (p1->AbsP).y;
		
		if (p1==PFirstSegm)	
			(p1->AbsP).y = ((pF1->GetAbsPos()).y)-pF1->GetVU2DwnF()-DISTLEG;
		
		sl.SetBowUp(BOW_DOWN);
	}
	
	sl.SetSlur(p1->AbsP, px, Color);
	
	sl.Draw();
	
}
