/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: legatqua.cpp,v 1.1 2005/07/25 16:31:27 bellinpo Exp $ */


#include "int/legatqua.hpp"

LegatQuadra::LegatQuadra()
{
	ID = CL_LEGATURA_QUADRA;
	SetHeight(STANGHETTA + YSHIFT);
}

LegatQuadra::~LegatQuadra()
{
	Figura *pfig1,*pfig2;
	
	pfig1 = GetPFigStart();
	pfig2 = GetPFigEnd();
	if(pfig1!=NULL)
		pfig1->DeleteTerzina(this);
	if(pfig2!=NULL)
		pfig2->DeleteTerzina(this);
	
} 

float LegatQuadra::GetDurataTerzina(float terz, float den, float num)
{
	float valterz=terz;
	int nterz=GetNumero();
	
	
	switch(nterz)
	{
    case 0:
    case 1:
		break;
    case 2:
		valterz=(terz/nterz)*(nterz+1);
		break;
    case 3:
		valterz=(terz/nterz)*(nterz-1);
		break;
    case 4:
		if (((int)num) % 3 == 0) switch((int)den)
		{
	case 4:
		valterz=(terz/nterz)*(nterz-1);
		break;
	case 8:
		valterz=(terz/nterz/2)*(nterz+2);
		break;
	case 16:
		valterz=(terz/nterz/2)*(nterz+1);
		break;
		}
		else valterz=(terz/nterz)*(nterz+2); //solo den==8?
		break;  
    case 5:
		if (((int)num) % 3 == 0) switch((int)den)
		{
	case 4:
		valterz=(terz/nterz)*(nterz-1);
		break;
	case 8:
		valterz=(terz/nterz/2)*(nterz+1);
		break;
	case 16:
		valterz=(terz/nterz/2)*(nterz+1);
		break;
		}
		else switch((int)den)
		{
	case 4:
		valterz=(terz/nterz)*(nterz-1);
		break;
	case 8:
		valterz=(terz/nterz)*(nterz+1);
		break;
		}
		break;      
    case 6:
		valterz=(terz/nterz)*(nterz-2);
		break;
    case 7:
		if (((int)num) % 3 == 0) switch((int)den)
		{
	case 4:
		valterz=(terz/nterz)*(nterz-1);
		break;
	case 8:
		valterz=(terz/nterz)*(nterz-1);
		break;
	case 16:
		valterz=(terz/nterz)*(nterz-1);
		break;
		}
		else
			valterz=(terz/nterz)*(nterz-3); 
		break;
    case 8:
		valterz=(terz/nterz)*(nterz+4); 
		break;
    default:  
		valterz=(terz/nterz)*(nterz-1);
		break;
	}
	return valterz;   
	
} 

void LegatQuadra::SetDim(VUnit w)
{
	SetWidth(w);
	SetHeight(STANGHETTA + YSHIFT );
}


void LegatQuadra::AdjustVUFig()
{
	Figura *pF1, *pF2;
	IntEstSegment *p1 = PFirstSegm[currentView];
	
	if (p1==NULL) return;
	
	if(GetSenzaLinea())
		return;
	
	pF1 = GetPFigStart();
	if (pF1==NULL) return;
	
	pF2 = GetPFigEnd();
	if (pF2==NULL) return;
	
	if(GetSopra())
	{
		VUnit d1=p1->AbsP.y-pF1->GetAbsPos().y+GetHeight();
		if(d1>pF1->GetVU2Up())
			pF1->SetVU2Up(d1);
		
		VUnit d2=p1->AbsP.y-pF2->GetAbsPos().y+GetHeight();
		if(d2>pF2->GetVU2Up())
			pF2->SetVU2Up(d2);
	}
	else
	{
		VUnit d1=-p1->AbsP.y+pF1->GetAbsPos().y+GetHeight();
		if(d1>pF1->GetVU2Dwn())
			pF1->SetVU2Dwn(d1);
		
		VUnit d2=-p1->AbsP.y+pF2->GetAbsPos().y+GetHeight();
		if(d2>pF2->GetVU2Dwn())
			pF2->SetVU2Dwn(d2);
	}
	
	return;
}


Bool LegatQuadra::SetNumero(int n)
{
	TNum.SetFont(nTextFont);
	return  TNum.SetNumero(n);
}

int LegatQuadra::GetNumero()
{
	return  TNum.GetNumero();
}

void LegatQuadra::SetColor(int col)
{
	DrawObject::SetColor((ColorIcon)col);
	return;
}

void LegatQuadra::SetSenzaLinea(Bool s)
{
	SenzaLinea = s;
}

int LegatQuadra::GetHeight(void)
{
	SetHeight(STANGHETTA + YSHIFT);
	// dimensione della stanghetta piu' la distanza del numero
	return (DrawObject::GetHeight());
}

Bool LegatQuadra::GetSenzaLinea()
{
	return SenzaLinea;
}

void LegatQuadra::DrawSegm(IntEstSegment *p1)
{ 
	Point p;
	Point from, to;
	
	VUnit startLftF=PFigStart->GetVU2LftF();
	VUnit endRgtF=PFigEnd->GetVU2RgtF();
	
	if(isOpaque)
		Scr->SetColor(GetColor());
	if(Sopra)
	{ 
		p = Point(0,0);
		TNum.SetPos(NULL, p1->AbsP + Point((VUnit)(GetWidth()+GetDxR()-GetDx()-TNum.GetWidth())/2+GetDx(),
			+ YSHIFT));
		TNum.Draw();
		
		if (!SenzaLinea) 
		{
			from = p1->AbsP + Point(-startLftF,0)+Point(GetDx(),GetDy());
			to = from + Point(0,STANGHETTA); 
			Scr->Line(from, to,SOLID_LINE);
			
			from = to;
			to = from + Point(startLftF+(GetVU2Rgt()+GetDxR()-GetDx() - 2*TNum.GetWidth())/2,0);
			Scr->Line(from, to);
			
			from = to + Point(2*TNum.GetWidth(),0);
			to = p1->AbsP + Point(GetVU2Rgt()+GetDxR() + endRgtF,STANGHETTA+GetDy());
			Scr->Line(from, to);
			
			
			from = to;
			to = to + Point(0,-STANGHETTA);
			Scr->Line(from, to,SOLID_LINE);
		}
		
	}
	else
	{
		p = Point(0,YSHIFT);
		TNum.SetPos(NULL,p1->AbsP + Point((VUnit)(GetWidth()+GetDxR()-GetDx()-TNum.GetWidth())/2+GetDx(),
			0));
		TNum.Draw();
		
		if (!SenzaLinea) 
		{
			to = p1->AbsP + p + Point(-startLftF,0)+Point(GetDx(),GetDy());
			from = to + Point(0,STANGHETTA); 
			Scr->Line(from, to,SOLID_LINE);
			
			from = to;
			to = from + Point(startLftF + (GetVU2Rgt()+GetDxR()-GetDx() - 2*TNum.GetWidth())/2,0);
			Scr->Line(from, to);
			
			from = to + Point(2*TNum.GetWidth(),0);
			to = p1->AbsP + p + Point(GetVU2Rgt()+GetDxR() + endRgtF,GetDy());
			Scr->Line(from, to);              
			
			from = to;
			to = to + Point(0,STANGHETTA);
			Scr->Line(from, to,SOLID_LINE);
		}
		
	}
	
}

void LegatQuadra::ValutaLegature()
{
	Bool sopra=FALSE;
	Bool stemUpAll=FALSE;
	
	if (Interm && StemFirst==1 && StemLast==1 && StemInt==1)
		stemUpAll=TRUE;
	else if (StemFirst==1 && StemLast==1) 
		stemUpAll=TRUE;
	else stemUpAll=FALSE;
	
	if (Multiv && voice==1)  
		sopra = TRUE;
	else if (Multiv && voice==2)
		sopra = FALSE;
	else if (stemUpAll) 
		sopra = TRUE;
	else
		sopra = FALSE;
	
	
	SetSopra(sopra);
	
}     

