/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: legatura.cpp,v 1.1 2005/07/25 16:31:28 bellinpo Exp $ */


#define max1(x,y) ((x)>(y) ? (x) : (y))
#define min1(x,y) ((x)<(y) ? (x) : (y))


#include "liooext.hpp"
#include "int/legatura.hpp"
#include "drw/slur.hpp"
#include "fig/nota.hpp"
#include "fig/accordo.hpp"
#include "str/arco.hpp"
#include "gui/dialogs.hpp"

#define DISTLEG1    3
#define DISTLEG2    7
#define DISTLEGVAL2 2
#define LEGSOSP     10

/**************************************************************/
void Legatura::SetDim(VUnit w)
{
	SetWidth(w);
	SetHeight(w/8);
}

/**************************************************************/
Bool Legatura::InitSlur(Slur& sl,IntEstSegment *p1)
{
	Point px;
	Figura *pF1, *pF2,*pFF1,*pFF2;
	VUnit distLeg1=DISTLEG2;
	VUnit distLeg2=DISTLEG2;
	VUnit yPent0;
	VUnit dx1=0,dx2=0;
	DrawObject *ultimodrw;
	
	pF1 = GetPFigStart();
	if(pF1==NULL)
		return FALSE;
	pF2 = GetPFigEnd();
	if(pF2==NULL)
		return FALSE;
	
	if(GetID()==CL_LEGATURA_VALORE)
	{
		if(Sopra)
		{
			px.x = (p1->AbsP).x+p1->Wdt;
			
			px.y = ((pF2->GetAbsPos()).y) + DISTLEGVAL2;
			p1->AbsP.y = ((pF1->GetAbsPos()).y)+ DISTLEGVAL2;
			
			if (!(p1->pNext==NULL && GetFine())) // legatura sospesa alla fine
			{
				px.y = p1->AbsP.y;
			}
			
			if (!(p1==PFirstSegm[currentView] && GetInizio())) // legatura sospesa all'inizio
			{
				p1->AbsP.y = px.y;
			}
			
			sl.SetBowUp(BOW_UP);
		}
		else // SOTTO, LEgatura di valore
		{
			px.x = (p1->AbsP).x+p1->Wdt;
			
			px.y = ((pF2->GetAbsPos()).y) - DISTLEGVAL2;
			p1->AbsP.y = ((pF1->GetAbsPos()).y)- DISTLEGVAL2;
			
			if (!(p1->pNext==NULL && GetFine())) // legatura sospesa alla fine
			{
				px.y = p1->AbsP.y;
			}
			
			if (!(p1==PFirstSegm[currentView] && GetInizio())) // legatura sospesa all'inizio
			{
				p1->AbsP.y = px.y;
			}
			
			sl.SetBowUp(BOW_DOWN);
		}
		sl.SetSlur(p1->AbsP+Point(GetDx(),GetDy()), px+Point(GetDxR(),GetDyR()), GetColor());
		
		return TRUE;
	} // fine per legatura di valore
	
	
	// legatura normale, di frase
	
	if(p1!=PFirstSegm[currentView] && p1->pNext!=NULL)
	{
		// caso pezzo interno di un legatura spezzata in 3 o piu' parti	
		if(Sopra)
			sl.SetBowUp(BOW_UP);
		else
			sl.SetBowUp(BOW_DOWN);
		
		sl.SetSlur(p1->AbsP, p1->AbsP + Point(p1->Wdt,0) , GetColor());
		return TRUE; 
	}
	
	// legatura normale sopra
	if (Sopra)
	{
		if (pF2->GetID()==CL_ACCORDO && pF2->GetParentFig()==NULL) 
			pFF2 = ((Accordo *)pF2)->GetLastNote();
		else
			pFF2 = pF2;
		if (pF1->GetID()==CL_ACCORDO && pF1->GetParentFig()==NULL) 
			pFF1 = ((Accordo *)pF1)->GetLastNote();
		else
			pFF1 = pF1;
		
		px.x = (p1->AbsP).x+p1->Wdt;
		
		if (pF1->GetID() != CL_SPAZIO)
		{ 
			if (p1y>=0.0) // valore normale 
			{ 
				distLeg1=pFF1->GetVU2UpF()+ DISTLEG1;
				ultimodrw = ((Nota*)pF1)->UltimoDrwPriority(Sopra); 
				if(ultimodrw!=NULL)
					distLeg1+=ultimodrw->GetAbsPos().y-pFF1->GetAbsPos().y+
						ultimodrw->GetVU2Up()-pFF1->GetVU2UpF(); 	  
				if (p1->nGamboUp==0 && ultimodrw==NULL && ((Nota*)pF1)->GetLGambo()>=0 (pF1->GetParentFig()==NULL || pF1->GetParentFig()->GetID()==CL_GRUPPONOTE))
				{  
					distLeg1=DISTLEG2;
				}
				p1->AbsP.y = ((pFF1->GetAbsPos()).y)+ distLeg1;
			}
			else // valore con correzione opposta 
			{ 
				distLeg1=pFF1->GetVU2DwnF()+ DISTLEG1;
				ultimodrw = ((Nota*)pF1)->UltimoDrwPriority(FALSE); // deve essere sotto 
				if(ultimodrw!=NULL)
					distLeg1+=-ultimodrw->GetAbsPos().y+pFF1->GetAbsPos().y+
						ultimodrw->GetVU2Dwn()-pFF1->GetVU2DwnF(); 	  
				if (p1->nGamboDwn==0 && ultimodrw==NULL && ((Nota*)pF1)->GetLGambo()<=0 && (pF1->GetParentFig()==NULL || pF1->GetParentFig()->GetID()==CL_GRUPPONOTE))
				{  
					distLeg1=DISTLEG2;
				}
				p1->AbsP.y = ((pFF1->GetAbsPos()).y)- distLeg1;
			}
		}
		else
		{ 
			if (p1y>=0.0) 
				p1->AbsP.y = ((pFF1->GetAbsPos()).y)+ DISTLEG2;
			else
				p1->AbsP.y = ((pFF1->GetAbsPos()).y)- DISTLEG2;
		}
		
		if (pF2->GetID() != CL_SPAZIO)
		{ 
			if (p2y>=0.0) // valore normale
			{ 
				distLeg2=pFF2->GetVU2UpF()+ DISTLEG1;
				ultimodrw = ((Nota*)pF2)->UltimoDrwPriority(Sopra);    
				if(ultimodrw!=NULL)
					distLeg2+=ultimodrw->GetAbsPos().y-pFF2->GetAbsPos().y+
						ultimodrw->GetVU2Up()-pFF2->GetVU2UpF(); 
				if (p1->nGamboUp==0  && ultimodrw==NULL && ((Nota*)pF2)->GetLGambo()>=0 && (pF2->GetParentFig()==NULL || pF2->GetParentFig()->GetID()==CL_GRUPPONOTE))
				{
					distLeg2=DISTLEG2;
				}
				px.y = ((pFF2->GetAbsPos()).y)+ distLeg2;
			}
			else // valore con correzione opposta
			{ 
				distLeg2=pFF2->GetVU2DwnF()+ DISTLEG1;
				ultimodrw = ((Nota*)pF2)->UltimoDrwPriority(FALSE);    // deve essere sotto 
				if(ultimodrw!=NULL)
					distLeg2+=-ultimodrw->GetAbsPos().y+pFF2->GetAbsPos().y+
						ultimodrw->GetVU2Dwn()-pFF2->GetVU2DwnF(); 
				if (p1->nGamboDwn==0  && ultimodrw==NULL && ((Nota*)pF2)->GetLGambo()<=0 && (pF2->GetParentFig()==NULL || pF2->GetParentFig()->GetID()==CL_GRUPPONOTE))
				{
					distLeg2=DISTLEG2;
				}
				px.y = ((pFF2->GetAbsPos()).y)- distLeg2;
			}
		}
		else
		{ 
			if (p2y>=0.0) 
				px.y = ((pFF2->GetAbsPos()).y)+ DISTLEG2;
			else
				px.y = ((pFF2->GetAbsPos()).y)- DISTLEG2;
		}
		
		
		if (!(p1->pNext==NULL && GetFine())) // legatura sospesa alla fine
		{
			int alt1=pFF1->GetAltezza();
			int alt2=pFF2->GetAltezza();
			
			yPent0=pFF1->GetAbsPos().y - alt1 * VU_PER_ROW/2;
			px.y = yPent0+ max1(alt1,alt2) * VU_PER_ROW/2 + DISTLEG2 + LEGSOSP;
		}
		
		if (!(p1==PFirstSegm[currentView] && GetInizio())) // legatura sospesa all'inizio
		{
			int alt1=pFF1->GetAltezza();
			int alt2=pFF2->GetAltezza();
			
			yPent0=pFF2->GetAbsPos().y - alt2 * VU_PER_ROW/2;
			p1->AbsP.y = yPent0 + max1(alt1,alt2) * VU_PER_ROW/2 + DISTLEG2 + LEGSOSP;
		}
		
		sl.SetBowUp(BOW_UP);
	}
	else // legatura normale sotto
	{
		if (pF2->GetID()==CL_ACCORDO && pF2->GetParentFig()==NULL) 
			pFF2 = ((Accordo *)pF2)->GetFirstNote();
		else
			pFF2 = pF2;
		if (pF1->GetID()==CL_ACCORDO && pF1->GetParentFig()==NULL) 
			pFF1 = ((Accordo *)pF1)->GetFirstNote();
		else
			pFF1 = pF1;
		
		px.x = (p1->AbsP).x+p1->Wdt;
		
		if (pF1->GetID() != CL_SPAZIO)
		{ 
			if (p1y<0.0) // valore con correzione opposta 
			{ 
				distLeg1=pFF1->GetVU2UpF()+ DISTLEG1;
				ultimodrw = ((Nota*)pF1)->UltimoDrwPriority(TRUE); // deve essere sopra
				if(ultimodrw!=NULL)
					distLeg1+=+ultimodrw->GetAbsPos().y-pFF1->GetAbsPos().y+
						ultimodrw->GetVU2Up()-pFF1->GetVU2UpF(); 	  
				if (p1->nGamboUp==0 && ultimodrw==NULL && ((Nota*)pF1)->GetLGambo()>=0 && (pF1->GetParentFig()==NULL || pF1->GetParentFig()->GetID()==CL_GRUPPONOTE))
				{  
					distLeg1=DISTLEG2;
				}
				p1->AbsP.y = ((pFF1->GetAbsPos()).y)+ distLeg1;
			}
			else   // valore normale
			{ 
				distLeg1=pFF1->GetVU2DwnF()+ DISTLEG1;
				ultimodrw = ((Nota*)pF1)->UltimoDrwPriority(Sopra); 
				if(ultimodrw!=NULL)
					distLeg1+=-ultimodrw->GetAbsPos().y+pFF1->GetAbsPos().y+
						ultimodrw->GetVU2Dwn()-pFF1->GetVU2DwnF(); 	  
				if (p1->nGamboDwn==0 && ultimodrw==NULL && ((Nota*)pF1)->GetLGambo()<=0 && (pF1->GetParentFig()==NULL || pF1->GetParentFig()->GetID()==CL_GRUPPONOTE))
				{  
					distLeg1=DISTLEG2;
				}
				p1->AbsP.y = ((pFF1->GetAbsPos()).y)- distLeg1;
			}
		}
		else
		{ 
			if (p1y<0.0) 
				p1->AbsP.y = ((pFF1->GetAbsPos()).y)+ DISTLEG2;
			else
				p1->AbsP.y = ((pFF1->GetAbsPos()).y)- DISTLEG2;
		}
		
		if (pF2->GetID() != CL_SPAZIO)
		{ 
			if (p2y<0.0) // valore con correzione opposta 
			{ 
				distLeg2=pFF2->GetVU2UpF()+ DISTLEG1;
				ultimodrw = ((Nota*)pF2)->UltimoDrwPriority(TRUE); // deve essere sopra  
				if(ultimodrw!=NULL)
					distLeg2+=ultimodrw->GetAbsPos().y-pFF2->GetAbsPos().y+
						ultimodrw->GetVU2Up()-pFF2->GetVU2UpF(); 
				if (p1->nGamboUp==0  && ultimodrw==NULL && ((Nota*)pF2)->GetLGambo()>=0 && (pF2->GetParentFig()==NULL || pF2->GetParentFig()->GetID()==CL_GRUPPONOTE))
				{
					distLeg2=DISTLEG2;
				}
				px.y = ((pFF2->GetAbsPos()).y)+ distLeg2;
			}
			else // valore normale
			{ 
				distLeg2=pFF2->GetVU2DwnF()+ DISTLEG1;
				ultimodrw = ((Nota*)pF2)->UltimoDrwPriority(Sopra);    
				if(ultimodrw!=NULL)
					distLeg2+=-ultimodrw->GetAbsPos().y+pFF2->GetAbsPos().y+
						ultimodrw->GetVU2Dwn()-pFF2->GetVU2DwnF(); 
				if (p1->nGamboDwn==0  && ultimodrw==NULL && ((Nota*)pF2)->GetLGambo()<=0 && (pF2->GetParentFig()==NULL || pF2->GetParentFig()->GetID()==CL_GRUPPONOTE))
				{
					distLeg2=DISTLEG2;
				}
				px.y = ((pFF2->GetAbsPos()).y)- distLeg2;
			}
		}
		else
		{ 
			if (p2y<0.0) 
				px.y = ((pFF2->GetAbsPos()).y)+ DISTLEG2;
			else
				px.y = ((pFF2->GetAbsPos()).y)- DISTLEG2;
		}
		
		if (!(p1->pNext==NULL && GetFine())) // legatura sospesa alla fine
		{
			int alt1=pFF1->GetAltezza();
			int alt2=pFF1->GetAltezza();
			
			yPent0=pFF1->GetAbsPos().y - alt1 * VU_PER_ROW/2;
			px.y = yPent0 + min1(alt1,alt2) * VU_PER_ROW/2 - DISTLEG2 - LEGSOSP;
		}
		
		if (!(p1==PFirstSegm[currentView] && GetInizio())) // legatura sospesa all'inizio
		{
			int alt1=pFF1->GetAltezza();
			int alt2=pFF1->GetAltezza();
			
			yPent0=pFF2->GetAbsPos().y - alt2 * VU_PER_ROW/2;
			p1->AbsP.y = yPent0 + min1(alt1,alt2) * VU_PER_ROW/2 - DISTLEG2 - LEGSOSP;
		}
		
		sl.SetBowUp(BOW_DOWN);
	}  
	
	sl.SetSlur(p1->AbsP+Point(GetDx(),GetDy()), px+Point(GetDxR(),GetDyR()), GetColor());
	
	return TRUE;
}

/**************************************************************/
void Legatura::UpdateDefaultCustom(Slur *sl)
{
	if (deforcustom==LEGATURA_DEFAULT) 
	{ sl->SetSlurDefault();
	sl->GetDefaultDISValues(&p1x, &p1y, &p2x, &p2y);
	}
	else 
	{ sl->SetSlurCustom();
	sl->ImposeXYDIS(p1x, p1y, p2x, p2y);
	}
}

/**************************************************************/
Legatura::Legatura()
{ 
	p1x=1.0;
	p1y=1.0;
	p2x=1.0;
	p2y=1.0;
	ID = CL_LEGATURA; 
	lineType=SOLID_LINE; 
	deforcustom=LEGATURA_DEFAULT; 
};


/**************************************************************/
Bool Legatura::Hit(const Point& p, SelObj selObject,DrawObject*& drwObj,
				   SymPath& symPath)
{
	Slur sl;
	Bool found=FALSE;
	
	IntEstSegment *p1 = PFirstSegm[currentView];
	
	if (p1!=NULL)
		while ( (p1!=NULL)  && (!(found)) )
		{
			if(!InitSlur(sl,p1))
				return FALSE;
			
			UpdateDefaultCustom(&sl);
			
			found=sl.hit(p);
			
			p1 = p1->pNext;
		}
		
		if (found)
		{
			drwObj=this;
			symPath.Array[SIMBOLO_PATH_LEVEL]=GetNumericCode();
			return TRUE;
		}
		else
			return FALSE;
}

/**************************************************************/
void Legatura::ValutaLegature()
{
	Slur slu;
	
	slu.SetAutoOrientation();
	slu.SetSlurParameter(Multiv, voice, StemFirst, StemLast, StemInt, Interm);
	slu.EvalBowOrientation();
	if (slu.GetBowUp()) 
		SetSopra(TRUE);
	else 
		SetSopra(FALSE);
	
}     

/**************************************************************/
void Legatura::SetLegaturaCustom(void)
{
	deforcustom=LEGATURA_CUSTOM;
}

/**************************************************************/
void Legatura::SetLegaturaDefault(void)
{
	deforcustom=LEGATURA_DEFAULT;
}

/**************************************************************/
void Legatura::GetKnotValue(double * DISp1x, double * DISp1y, double * DISp2x, double * DISp2y)
{ // prende i valori correnti non quelli di default dello slur
	*DISp1x=p1x;
	*DISp1y=p1y;
	*DISp2x=p2x;
	*DISp2y=p2y;
}

/**************************************************************/
void Legatura::SetKnotValue(double DISp1x, double DISp1y, double DISp2x, double DISp2y)
{
	p1x=DISp1x;
	p1y=DISp1y;
	p2x=DISp2x;
	p2y=DISp2y;
	SetLegaturaCustom();
}

/**************************************************************/
void Legatura::DrawSegm(IntEstSegment *p1)
{
	Slur sl;
	
	if(!InitSlur(sl,p1))
		return;
	
	sl.SetLineType(lineType); 
	
	UpdateDefaultCustom(&sl);

#ifdef USE_GDIPLUS
	//sl.Draw();
	sl.EvalXYDIS();
	if(sl.GetBowUp())
		sl.EvalP1P2Up();
	else
		sl.EvalP1P2Down();
/*
	Point knot1=sl.GetCP1();
	Point knot2=sl.GetCP2();
	Scr->Line(knot1+Point(-3,0),knot1+Point(3,0));
	Scr->Line(knot1+Point(0,-3),knot1+Point(0,3));
	Scr->Line(knot2+Point(-3,0),knot2+Point(3,0));
	Scr->Line(knot2+Point(0,-3),knot2+Point(0,3));
*/
	if(isOpaque)
		Scr->SetColor(GetColor());
	Scr->Slur(sl.GetFrom(),sl.GetCP1(),sl.GetCP2(),sl.GetTo(), sl.GetBowUp(),lineType);
#else	
	sl.Draw();
#endif
}

/**************************************************************/
void Legatura::AdjustVUFig()
{
	double shi, ehi;
	Slur sl;
	Point px;
	Figura *pF1, *pF2;
	Bool found=FALSE;
	IntEstSegment *p1 = PFirstSegm[currentView];
	
	if (p1==NULL) return;
	
	pF1 = GetPFigStart();
	if (pF1==NULL) return;
	
	pF2 = GetPFigEnd();
	if (pF2==NULL) return;
	
	px.x = (p1->AbsP).x+p1->Wdt;
	
	UpdateDefaultCustom(&sl);
	
	if (Sopra) // sopra
	{ 
		if (p2y<0.0) // valore con correzione opposta 
			px.y = ((pF2->GetAbsPos()).y)-DISTLEG;
		else 
			px.y = ((pF2->GetAbsPos()).y)+DISTLEG;
		if (p1y<0.0)// valore con correzione opposta 
			(p1->AbsP).y = ((pF1->GetAbsPos()).y)-DISTLEG;
		else 
			(p1->AbsP).y = ((pF1->GetAbsPos()).y)+DISTLEG;
		sl.SetBowUp(BOW_UP);     
		sl.SetSlur(p1->AbsP, px, GetColor());
		sl.EvalXYDIS();
		sl.EvalP1P2Up();
		shi=sl.GetStartHigh();
		ehi=sl.GetEndHigh();
		if (p1y<0.0)// valore con correzione opposta 
			pF1->SetVU2Dwn(pF1->GetVU2Dwn()+(VUnit)shi+4);//GetVU2DwnF
		else 
			pF1->SetVU2Up(pF1->GetVU2Up()+(VUnit)shi+4);//GetVU2UpF
		if (p1y<0.0)// valore con correzione opposta 
			pF2->SetVU2Dwn(pF2->GetVU2Dwn()+(VUnit)ehi+4);//GetVU2DwnF
		else 
			pF2->SetVU2Up(pF2->GetVU2Up()+(VUnit)ehi+4);//GetVU2UpF
	}
	else // sotto 
	{
		if (p2y<0.0)// valore con correzione opposta 
			px.y = ((pF2->GetAbsPos()).y)+DISTLEG;
		else 
			px.y = ((pF2->GetAbsPos()).y)-DISTLEG;
		if (p1y<0.0)// valore con correzione opposta 
			(p1->AbsP).y = ((pF1->GetAbsPos()).y)+DISTLEG;
		else 
			(p1->AbsP).y = ((pF1->GetAbsPos()).y)-DISTLEG;;
		sl.SetBowUp(BOW_DOWN);
		sl.SetSlur(p1->AbsP, px, GetColor());
		sl.EvalXYDIS();
		sl.EvalP1P2Down();
		shi=sl.GetStartHigh();
		ehi=sl.GetEndHigh();
		if (p1y<0.0)// valore con correzione opposta 
			pF1->SetVU2Up(pF1->GetVU2Up()+(VUnit)shi+4);//GetVU2UpF
		else 
			pF1->SetVU2Dwn(pF1->GetVU2Dwn()+(VUnit)shi+4);//GetVU2DwnF
		if (p1y<0.0)// valore con correzione opposta 
			pF2->SetVU2Up(pF2->GetVU2Up()+(VUnit)ehi+4);//GetVU2UpF
		else 
			pF2->SetVU2Dwn(pF2->GetVU2Dwn()+(VUnit)ehi+4);//GetVU2DwnF
	}  
}

/**************************************************************/
void Legatura::Edit(Battuta *pBat)
{
	
	extern void testeditlegature(IntEsteso *pInt);
	testeditlegature(this);
	
	EditingIntervalsDlg(this);
}

/**************************************************************/