/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: legatura.hpp,v 1.1 2005/07/25 16:31:28 bellinpo Exp $ */


#ifndef LEGATURA_HPP
#define LEGATURA_HPP

#include "lioo.hpp"
#include "int/interval.hpp"

class Slur;



#define LEGATURA_CUSTOM 1
#define LEGATURA_DEFAULT 0

class Legatura: public Intervallo
{
private:
	LineType lineType;
	
	// parametri per le slur con knot di tipo custom
    double p1x, p1y, p2x, p2y;
	
    Bool InitSlur(Slur& slur,IntEstSegment *p1);
	// inizializza la slur per il segmento di intervallo indicato.
	
	// legatura di tipo default con Knot automatici o CUSTOM con knot da valori imposti
	// e memorizzati su file
	int deforcustom;
	// aggiorna i valori della legatura, i knot
	void UpdateDefaultCustom(Slur *sl);
	
public:
    Legatura();
	
    void SetDim(VUnit w);
	// imposta la larghezza passata come parametro e l'altezza calcolata in
	// base al tipo di simbolo musicale.
	
	Bool Hit(const Point& p, SelObj selObject,DrawObject*& drwObj,
		SymPath& symPath);
	
	void ValutaLegature();
    void DrawSegm(IntEstSegment *);
	
	void SetLegaturaCustom(void);
	void SetLegaturaDefault(void);
    void GetKnotValue(double * DISp1x, double * DISp1y, double * DISp2x, double * DISp2y);
	void SetKnotValue(double DISp1x, double DISp1y, double DISp2x, double DISp2y);
    
    void AdjustVUFig(); 
	
	
	void SetLineType(LineType ltype) { lineType=ltype; };
	LineType GetLineType() { return lineType; };
	
	void Edit(Battuta *pBat=NULL);

	int SaveXML(FILE *);
};


#endif //LEGATURA_HPP
