/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: modifott.cpp,v 1.1 2005/07/25 16:31:28 bellinpo Exp $ */


#include "int/modifott.hpp"

void ModifOttava::SetDim(VUnit w)
{
	SetWidth(w);
	VUnit hott;
	switch (GetTipo8())
	{
	case VA8:
		hott=OTTAVA2UP+OTTAVA2DWN;
		break;
	case BA8:
		hott=OTTAVABA2UP+OTTAVABA2DWN;
		break;
	case BA15:
		hott=QBA2UP+QBA2DWN;
		break;
	case MA15:
		hott=QMA2UP+QMA2DWN;
		break;
	}
	SetHeight(hott);
}


/*******************************************************************/
void ModifOttava::DrawSegm(IntEstSegment *p1)
{
	char code[2]={0,0};
	int i, NTratti;
	Point p;
	VUnit wott;
	VUnit hott; 
	VUnit deltax, deltay;
	
	if(isOpaque)
		Scr->SetColor(GetColor());
	int oldFont;
	oldFont=Scr->SetFont(nMusicFont);
	
	
	switch (GetTipo8())
	{
	case VA8:
		oldFont=Scr->SetFont(nMusicFont);
		code[0]=(char)CODE_OTTAVA;
		deltax=DELTAXOTTAVA;
		deltay=DELTAYOTTAVA;
		wott=OTTAVA2RGT+OTTAVA2LFT;
		hott=OTTAVA2UP+OTTAVA2DWN;
		break;
	case BA8:
		oldFont=Scr->SetFont(nMusicFont2);
		code[0]=(char)CODE_OTTAVABA;
		deltax=DELTAXOTTAVABA;
		deltay=DELTAYOTTAVABA;
		wott=OTTAVABA2RGT+OTTAVABA2LFT;
		hott=OTTAVABA2UP+OTTAVABA2DWN;
		break;
	case BA15:
		oldFont=Scr->SetFont(nMusicFont2);
		code[0]=(char)CODE_QBA;
		deltax=DELTAXQBA;
		deltay=DELTAYQBA;
		wott=QBA2RGT+QBA2LFT;
		hott=QBA2UP+QBA2DWN;
		break;
	case MA15:
		oldFont=Scr->SetFont(nMusicFont2);
		code[0]=(char)CODE_QMA;
		deltax=DELTAXQMA;
		deltay=DELTAYQMA;
		wott=QMA2RGT+QMA2LFT;
		hott=QMA2UP+QMA2DWN;
		break;
	}
	deltax+=GetDx();

	NTratti=(p1->Wdt-wott+GetDxR()-GetDx())/(LUNGTRATTO+LUN_SPAZIO);
	
	p = p1->AbsP;
	if(Sopra)   // quindi ottava o 15ma, per default ma anche le altre se forzate
	{
		
		Scr->TextTo(p1->AbsP+Point(deltax+wott/2,deltay));
		Scr->OutText((unsigned char *)code);
		
		p += Point(wott,hott) + Point(DELTAXTRATTOUP,DELTAYTRATTOUP)+Point(GetDx(),GetDy());
		for(i=1;i<=NTratti;i++) 
		{ 
			Scr->Line(p,p + Point(LUNGTRATTO,0),SOLID_LINE);
			p += Point(LUNGTRATTO+LUN_SPAZIO,0);
		}
		p = p1->AbsP + Point(GetDx(),GetDy()) + Point(GetWidth()+GetDxR()-GetDx(),hott) + Point(DELTAXTRATTOUP,DELTAYTRATTOUP);
		Scr->Line(p,p - Point(0,LUNGTRATTO+LUN_SPAZIO) ,SOLID_LINE);
		Scr->Line(p,p - Point(LUNGTRATTO+LUN_SPAZIO,0) ,SOLID_LINE);
	}
	else  // quindi ottavaba o 15ba, per default ma anche le altre se forzate
	{
		Scr->TextTo(p1->AbsP+Point(deltax+wott/2,deltay));
		Scr->OutText((unsigned char *)code);
		
		p += Point(wott,-hott) + Point(DELTAXTRATTODWN,DELTAYTRATTODWN)+Point(GetDx(),GetDy());
		for(i=1;i<=NTratti;i++) 
		{ 
			Scr->Line(p,p + Point(LUNGTRATTO,0),SOLID_LINE);
			p += Point(LUNGTRATTO+LUN_SPAZIO,0);
		}
		p = p1->AbsP + Point(GetDx(),GetDy()) + Point(GetWidth()+GetDxR()-GetDx(),-hott) + Point(DELTAXTRATTODWN,DELTAYTRATTODWN);
		Scr->Line(p,p + Point(0,LUNGTRATTO+LUN_SPAZIO) ,SOLID_LINE);
		Scr->Line(p,p - Point(LUNGTRATTO+LUN_SPAZIO,0) ,SOLID_LINE);
	}
	
	Scr->SetFont(oldFont);
}  

/*******************************************************************/
Bool ModifOttava::TestAndSet(Figura *pFStart, Figura *pFEnd, Bool sopra)
{
	Bool result = TRUE;
	
	// Rifiuta i puntatori se coincidono.
	if(pFStart==pFEnd)
		return FALSE;
	
	// Gli estremi di un simbolo di intervallo possono essere solo note
	// ed accordi.
	
	if (pFStart != NULL)
	{
		switch(pFStart->GetID())
		{
		case CL_GRUPPONOTE:
			result = FALSE;
		}
	}
	
	if (pFEnd != NULL)
	{
		switch(pFEnd->GetID())
		{
		case CL_GRUPPONOTE:
			result = FALSE;
		}
	}
	
	if(result)
	{
		// Imposta gli attributi.
		PFigStart = pFStart;
		PFigEnd = pFEnd;
		Sopra = sopra;
	}
	return result;
}

/*******************************************************************/
