/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#ifndef RAGGRUPPAMENTO_HPP
#define RAGGRUPPAMENTO_HPP

#include "drw/def_drw.hpp"
#include "str/def_str.hpp"
#include "lioo.hpp"
#include "int/intestes.hpp"
#include "spa/spartito.hpp"

#define MAX_NUM_SEGMENT  10

struct TipoTratti{
  Point PRef; // Punto di riferimento
  Point  P1; // Punto iniziale
  Point P2; // Punto finale
};

class Raggruppamento: public IntEsteso
{
	public: 

			enum BeamPos{ BEAM_UP,BEAM_DWN,BEAM_12,BEAM_23};

	protected:
		int NumNote;
		// Numero di note del gruppo.
		int NumLayer;
		// il numero del layer del raggruppamento
		Figura *FirstRealFig, *LastRealFig;
		Nota *FirstNota,*LastNota;
		// Puntatori alla prima ed all'ultima nota del gruppo.
			
		Bool Notina;
		// E' TRUE se il gruppo e' composto da notine, FALSE altrimenti. Viene
		// inizializzato a FALSE.

		//    Bool SmallBeam;
		// E' TRUE se il gruppo e' composto da notine, FALSE altrimenti. Viene
		// inizializzato a FALSE.
	
		VUnit SpaceWidth;
		// Larghezza dello spazio.

		VUnit LGambo[MAX_NUM_SEGMENT];
		// Lunghezza del gambo della nota piu' significativa del gruppo, calco-
		// lata dall'altezza fisica della nota all'altezza della prima barretta.
		// Per nota "piu' significativa" si intende la piu' alta del gruppo se
		// le note del gruppo hanno il gambo sopra, la piu' bassa se le note 
		// hanno il gambo sotto.
      
		ClassID NoteID;
		// Identificatore del tipo di nota che compone il gruppo.
    
		short NBarre[MAX_NUM_SEGMENT];
		// Numero di barrette del gruppo.
                                            
		Point BPoint1[MAX_VIEW][MAX_NUM_SEGMENT],BPoint2[MAX_VIEW][MAX_NUM_SEGMENT];
		// Punti estremi della prima barretta del gruppo. Se BPoint1=(0,0), le
		// barrette non verranno disegnate.
    
		float BSlope[MAX_VIEW][MAX_NUM_SEGMENT];
		// Coeff. angolare delle barrette.
    
		TipoTratti Tratto[MAX_VIEW][MAX_NUM_SEGMENT][4];
		// Vettore conentente tratti di barra paralleli.

		Point& GetBPoint1(short numSegment) { return BPoint1[currentView][numSegment]; };
		Point& GetBPoint2(short numSegment) { return BPoint2[currentView][numSegment]; };

		Status_ID status;

	

		BeamPos beamPos;

		 VUnit VU2UpF,VU2DwnF,VU2LftF,VU2RgtF;
		// Distanze da AbsPos dei punti superiore, inferiore, sinistro e destro
		// del rettangolo contenente la figura.  
		VUnit VU2Up,VU2Dwn,VU2Lft,VU2Rgt;
		// Distanze da AbsPos dei punti superiore, inferiore, sinistro e
		// destro del rettangolo contenente la figura e tutti i simboli ad
		// essa connessi.

  
	public:
		Raggruppamento(); //{ ID = CL_RAGGRUPPAMENTO; };
    
	
		/*void SetDim(VUnit w);
		// imposta la larghezza passata come parametro e l'altezza calcolata in
		// base al tipo di simbolo musicale.

		void DrawSegm(IntEstSegment *);
		void printSegm(IntEstSegment *);

    
		char *Describe(Context);
		char *DescribeNet(Context, NumCode);*/
		void printSegm(IntEstSegment *);
		void DrawSegm(IntEstSegment *);
		void SetDim(VUnit w);
		   
    
		/////////////////////////////////////////////////////////////
		//  UL GetSp2NextFig();
		////////////////////////////////////////////////////////////

		void Init();
		// Inizializza gruppo note come lista vuota.   
      
		void SetNumNote(int nn);
		int GetNumNote(); 
		void SetNumNote();
    
		void SetFirstNota(Nota *pn);
		Nota *GetFirstNota();    
      
		void SetLastNota(Nota *pn);
		Nota *GetLastNota(); 
	
		void SetFirstFigura(Figura *pfig);
		Figura* GetFirstFigura();

		void SetLastFigura(Figura *pfig);
		Figura* GetLastFigura();

		Figura* GetNextFigura(Figura *pfig);
		//scorre solo le figure visibili del raggruppamento 
		Figura* GetNextFig(Figura *pfig);
		//scorre tutte le figure del raggruppamento 
		Figura* GetPrevFigura(Figura *pfig);

		void SetFirstBattuta(Battuta *pbat);
		Battuta* GetFirstBattuta();

		void SetLastBattuta(Battuta *pbat);
		Battuta* GetLastBattuta();

		void SetNumLayer();
		int GetNumLayer();
      
		void SetNotina(Bool b);
		// Seleziona la dimensione del gruppo.
		Bool GetNotina();
		// Dice se il gruppo e' di notine.
		//////////////////////////////////////////////////////////////////////////////////
    	void SetSmallBeam(Bool var);
		// Seleziona la dimensione del gruppo.
		//////////////////////////////////////////////////////////////////////////////////
		void SetBeamUp();
		// Seleziona la direzione del gambo del gruppo verso l'alto
		void SetBeamDwn();
		// Seleziona la direzione del gambo del gruppo verso il basso
		//////////////////////////////////////////////////////////////////////////////////
		void SetBeamStatus(Status_ID stat);
		//////////////////////////////////////////////////////////////////////////////////
		void SetSpaceWidth(VUnit sw);
		VUnit GetSpaceWidth();
    
		void SetLGambo(VUnit lg, short numSegment);  
		// Imposta il gambo della nota piu' significativa. Se il valore 
		// assoluto di lg e' minore di LGAMBOMIN, LGambo prende il valore
		// LGAMBOMIN se lg>=0 o -LGAMBOMIN se lg<0 (se la dimensione del
		// gruppo e' piccola, i confronti vengono fatti con LGAMBOMIN/2).
		// Va eseguito dopo aver impostato la dimensione del gruppo.
      
		void SetGamboUp();
		// Imposta il gambo sopra il gruppo, con lunghezza di default 
		// (meta' lunghezza di default se la nota e' una notina).

		void SetGamboDwn();
		// Imposta il gambo sotto il gruppo, con lunghezza di default
		// (meta' lunghezza di default se la nota e' una notina).
 
		VUnit GetLGambo(short numSegment) {return LGambo[numSegment];};    
		// Restituisce la lunghezza del gambo.
      
		void SetNoteID(ClassID id) {NoteID=id;};
		ClassID GetNoteID() {return NoteID;};  
            
		Bool AddFigura(Figura *pf); // TipoInserimento tpIns=_LOAD);
		// Aggiunge una figura al gruppo.
		// Sono ammessi a formare gruppi le note (e gli accordi) con valore 
		// minore o uguale della croma e gli spazi. Se si vuol inserire nel
		// gruppo una figura che non e' di questi tipi, il metodo restituisce
		// FALSE.
		// Le figure vanno inserite nel gruppo ordinate per posizione oriz-
		// zontale, cioe' da sinistra verso destra. Gli spazi possono essere
		// anche all'inizio ed alla fine del gruppo.
		// La prima nota immessa nel gruppo determina la direzione del gambo
		// del gruppo e la dimensione delle note del gruppo. Se in
		// seguito si vuol aggiungere una nota con diversa
		// dimensione, tale nota non viene aggiunta ed il metodo ritorna FALSE.
		// Negli altri casi il metodo ritorna TRUE.
		// Non e' necessario che le note inserite abbiano il gambo tutte nella
		// stessa direzione.
    	Bool AddAfterFigura(Figura *pf1,Figura* pf2);
		// Aggiunge la figura pf2 al gruppo dopo la figura pf1. Accetta 
		// gli spazi e le pause "normali"
		Bool DelFigura(Figura *pf);
		// Cancella una figura dal gruppo, si puo' cancellare uno spazio o una pausa
		// "normale".
	  
		Figura* ExtractFigura(Battuta *pbat, short layer);
		// Estrae dal gruppo la figura (nota o spazio) piu' a sinistra.
		// Se il gruppo e' vuoto, ritorna NULL.
		// Dopo aver eseguito questo metodo, occorre eseguire Adjust sia della
		// nota estratta (se si e' estratto uno spazio non e' necessario),
		// sia del gruppo rimanente. Questo non e' necessario
		// se la nota ed il gruppo vengono riposizionati con SetPos, in quanto
		// SetPos chiama Adjust.
    
		void SetPos(DrawObject* d,const Point& p);
		// Imposta la posizione del gruppo e chiama Adjust(). 
    
		//   void Adjust0(); 
		//    void Adjust(); 
		// Sistema le note e gli spazi del gruppo in base all'AbsPos del
		// gruppo ed all'altezza delle singole note.                         
		// Imposta le lunghezze dei gambi delle note e cancella la coda alle
      // note, cosi' da poterle disegnare raggruppate.
      // Calcola BPoint1 e BPoint2 e chiama Adjust delle note del gruppo.
      // Se le barrette non vanno disegnate, il metodo produce BPoint1=
      // =BPoint2=(0,0).
    void Adjust(Bool primaleg, Battuta *pBattuta, short current_layer, short current_staff,short numSegment,Figura *FirstFig, Figura *LastFig);  
	
	void AdjustVUFig();
       
    //void Draw();
	//void printSegm(IntEstSegment *);
      // Disegna il gruppo di note

    void DrawTratto(Nota *pnota, short nBarra, short Direz,VUnit BThick,short numSegment);
//	void printTratto(Nota *pnota, short nBarra, short Direz,VUnit BThick,short numSegment);
      // Disegna un tratto di barretta.

	/* 
	void DrawTrattoMiddle(Nota *pnota, short nBarra, short DirezNota, VUnit BThick);
	  // GB Disegna un tratto di barretta per il beaming tra gli staff
	void printTrattoMiddle(Nota *pnota, short nBarra, short Direz, VUnit BThick);
      // GB Stampa un tratto di barretta, per il beaming tra gli staff
	*/

	void DrawTrattoMiddle(Nota *pnota_corr,Nota *pnota_prec, VUnit BThick, VUnit dYB,short numSegment);
	  // GB Disegna un tratto di barretta per il beaming tra gli staff
//	void printTrattoMiddle(Nota *pnota_corr,Nota *pnota_prec, VUnit BThick, VUnit dYB,short numSegment);
      // GB Stampa un tratto di barretta, per il beaming tra gli staff

    void DrawBarre(short numSegment);
	  // Disegna le barrette.
//	void printBarre(short numSegment);
      // Stampa le barrette.

	void DrawBarreMiddle(short numSegment);
	  //GB Disegna la barra centrale
//	void printBarreMiddle(short numSegment);
	  //GB Stampa la barra centrale.
    
//    void Free();
      // Cancella la lista e dealloca gli spazi, le note e gli accordi
      // in lista.    
   
    //Bool DeleteSym(DrawObject *sym);
      // Cancella il simbolo specificato se e' fra quelli connessi al gruppo.
      // Ritorna TRUE se il simbolo e' stato cancellato.
    
   Bool Hit(const Point&, SelObj, DrawObject*&, SymPath&);
   // Bool SymbolicHit(SymPath&, SelObj, DrawObject*&);
   // Bool Hit(const class Rectangle&, SelObj, SymPath&, SymPath&);
    Bool FigPtr2SymPath(SymPath&, Figura*);
      // ricostruisce il PATH di una figura a partire dal puntatore.
    
	
	BeamPos GetBeamPosition(){return beamPos;};//GB

	void SetBeamPosition(BeamPos Bp);//GB

    float GetMedia();
      // Restituisce il parametro media di un gruppo di note.
      // Media = somma delle altezze delle note / numero di note 
    float GetBeamDis();
      // Restituisce un parametro per la determinazione della
      // direzione del gambo nei beam.
    float GetDelta();
      // Restituisce il parametro delta che serve per determinare
      // l'inclinazione dei beam 
    float GetMeanHighLow();  
    float GetMean(); 
	int GetStaffInBeam();
	void DeleteRaggruppamento();
	bool IsInRaggruppamento(Figura* pfig);
	Figura* GetFineSegmento(Figura* figuraIniziale);
	  // GB. Restituisce il numero di staff coinvolti nel beam

   // void Select();
   // void Deselect(); 
    
//    CmdResult Command(CmdID cmd,CmdArgV argV);
    //CmdResult SymCommand(SymCmd& CSmd, Battuta *pb=NULL);

   // char* Describe(Context);
	
	void GetMinMaxStaff(short& minStaff, short& maxStaff);
// ritorna TRUE se c'e' solo una nota
// oppure se e' vuoto
	Bool isOne(void);
	void ChangeFSimple(Figura *pFig1, Figura *pFig2);

	void SetVU2UpF(VUnit vu);  
    void SetVU2DwnF(VUnit vu);  
    void SetVU2LftF(VUnit vu);  
    void SetVU2RgtF(VUnit vu);  
    
    VUnit GetVU2UpF();
    VUnit GetVU2DwnF();
    VUnit GetVU2LftF();
    VUnit GetVU2RgtF();
      
    void SetVU2Up(VUnit vu);  
    void SetVU2Dwn(VUnit vu);  
    void SetVU2Lft(VUnit vu);  
    void SetVU2Rgt(VUnit vu);  
    
    VUnit GetVU2Up();
    VUnit GetVU2Dwn();
    VUnit GetVU2Lft();
    VUnit GetVU2Rgt();

	void Edit(Battuta *pBat);
	IntEstSegment* GetPSegment(short numSegment);
	Status_ID GetStatus();
	void SetStatus(Status_ID s);

	int SaveXML(FILE *);
};

#endif
