/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: lista.cpp,v 1.1 2005/07/25 16:31:31 bellinpo Exp $ */


#include "lst/lista.hpp"

Node::Node(void) {Next=NULL; Prev=NULL; ObjectPtr=NULL;}
Node::Node(DrawObject *p) {Next=NULL; Prev=NULL; ObjectPtr=p;}

Lista::Lista(void)
{
	first = NULL;
	last = NULL;
	lastRef=NULL;
	numobject = 0;
}

Lista::~Lista(void)
{ 
	Clear();
};

Bool Lista::FindPtr(DrawObject *ptr)
{
	return FindNodePtr(ptr)!=NULL;
};

void Lista::Clear(void)
{
	Node *dummy=first;
	Node *dummy1=dummy;
	while(dummy)
	{
		dummy=dummy->Next;
		delete dummy1;
		dummy1=dummy;
	}
	first=NULL;
	last=NULL;
	numobject = 0;
}

unsigned long Lista::GetNumObj (void)
{
	return numobject;
}

DrawObject *Lista::GetFirst(void)
{
	
	lastRef=first;
	return (first ? first->ObjectPtr : NULL);
}

DrawObject *Lista::GetLast(void)
{
	lastRef=last;
	return (last ? last->ObjectPtr : NULL);
}

void Lista::InsertEnd(DrawObject *newobj)
{
	Node *NewNode=new Node(newobj);
	if(!NewNode)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Lista","InsertEnd()");
	
	if(last==NULL)
	{
		// lista vuota
		first = NewNode;
		last = NewNode;
		NewNode -> Next=NULL;
		NewNode -> Prev=NULL;
	}
	else
	{
		// lista non vuota
		last->Next=NewNode;
		NewNode->Next=NULL;
		NewNode->Prev=last;
		last=NewNode;
	}
	numobject ++;
}


// il metodo DelPtr cancella dalla lista l'oggetto individuato dal puntatore

void Lista::DelPtr(DrawObject *ptr)
{
	Node *pre,*post,*dummy;
	
	if(ptr!=NULL)
	{
		dummy=first;
		while(dummy)
		{
			pre=dummy->Prev;
			post=dummy->Next;
			if(dummy->ObjectPtr==ptr)
			{ if (dummy==lastRef) 
			lastRef=lastRef->Prev;
			
			if(pre==NULL && post==NULL)
			{
				first=NULL;
				last=NULL;
			}
			else
			{
				if(pre==NULL)
				{
					first=post;
					post->Prev=NULL;
				}
				else if(post==NULL)
				{
					pre->Next=NULL;
					last=pre;
				}
				else
				{
					post->Prev=pre;
					pre->Next=post;
				}
			}
			delete dummy;
			dummy=NULL;
			numobject--;
			}
			else
			{
				dummy=post;
			}
		}
	}
}

// il metodo DelPtrNode cancella dalla lista l'oggetto pn e 
// l'oggetto puntato pn->ObjectPtr

void Lista::DelPtrNode(Node *pn)
{
	Node *pre,*post;  
	
	if (pn!=NULL)
	{
		delete pn->ObjectPtr;  //cancella l'oggetto puntato da pn
		
		pre=pn->Prev;
		post=pn->Next;
		if (lastRef==pn)
			lastRef=lastRef->Prev;
		
		if(pre==NULL && post==NULL)
		{
			first=NULL;
			last=NULL;
		}
		else if(pre==NULL)
		{
			first=post;
			post->Prev=NULL;
		}
		else if(post==NULL)
		{
			pre->Next=NULL;
			last=pre;
		}
		else
		{
			post->Prev=pre;
			pre->Next=post;
		}
		delete pn;
		pn=NULL;
		numobject--;
		pn=post;
	}
	
}

DrawObject *Lista::GetNext(DrawObject *r)
{
	Node *temp=first;
	
	if (lastRef != NULL)
	{ 
		if (lastRef->ObjectPtr==r)
		{  
			lastRef=lastRef->Next;
			
			if (lastRef) 
				return lastRef->ObjectPtr;
			else return NULL;
		}  
	}
	while(temp)
	{
		if(temp->ObjectPtr==r)
		{
			if(temp->Next)
				return temp->Next->ObjectPtr;
			else
				return NULL;
		}
		temp=temp->Next;
	}
	
	return NULL;
}

DrawObject *Lista::GetPrev(DrawObject *r)
{
	Node *temp=first;
	
	if (lastRef != NULL)
	{ if (lastRef->ObjectPtr==r)
	{  lastRef=lastRef->Prev;
	
	if (lastRef) 
		return lastRef->ObjectPtr;
	   else return NULL;
	}  
	}
	
	while(temp)
	{
		if(temp->ObjectPtr==r)
		{
			if(temp->Prev)
				return temp->Prev->ObjectPtr;
			else
				return NULL;
		}
		temp=temp->Next;
	}
	return NULL;
}

Node *Lista::GetNext(Node *n)
{
	if(n)
		return n->Next;
	else
		return NULL;
}

Node *Lista::GetPrev(Node *n)
{
	if(n)
		return n->Prev;
	else
		return NULL;
}

long Lista::GetPos(DrawObject *pric)
{
	Node *ptr=first;
	long Position=0;
	
	while(ptr)
	{
		if(pric==ptr->ObjectPtr)
			return Position;
		ptr=ptr->Next;
		Position++;
	}
	return -1;
}

DrawObject *Lista::AtPos(unsigned long pos)
{
	Node *ptr=first;
	unsigned long Position=0;
	
	if((pos>=0) && (pos<numobject))
		while(ptr)
		{
			if(Position==pos)
				return ptr->ObjectPtr;
			ptr=ptr->Next;
			Position++;
		}
		return NULL;
}

Node *Lista::FindNodePtr(DrawObject *ptr)
{
	Node *nodeptr=first;
	
	if (lastRef && lastRef->ObjectPtr==ptr)
		return lastRef;
	
	while ( nodeptr != NULL )
	{
		if (nodeptr->ObjectPtr == ptr)
		{ 
			lastRef=nodeptr;
			return nodeptr;
		}  
		nodeptr=nodeptr->Next;
	}
	return NULL;
}


void Lista::Swap(DrawObject *p1, DrawObject *p2)
{
	Node *n1 = FindNodePtr(p1),
		*n2 = FindNodePtr(p2);
	
	if (n1 && n2) {
		n1->ObjectPtr = p2;
		n2->ObjectPtr = p1;
	}
}

DrawObject *Lista::Change(DrawObject *p1, DrawObject *p2)
{
	Node *n1 = FindNodePtr(p1);
	
	if (n1) {
		n1->ObjectPtr = p2;		
		return p1;
	} else
		return NULL;
}

void Lista::InsertTop(DrawObject *newobj)
{
	Node *NewNode = new Node(newobj);
	if(!NewNode)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Lista","InsertTop()");
	
	if (first == NULL) {
		// Empty list
		first = NewNode;
		last =NewNode;
		NewNode->Next = NULL;
	} else {
		first->Prev = NewNode;
		NewNode->Next = first;
		first = NewNode;
	}
	NewNode->Prev = NULL;
	numobject++;
}


Bool Lista::InsertAfter(DrawObject *newobj, DrawObject *refobj)
{
	if (refobj==NULL)
	{
		InsertTop(newobj);
		return TRUE;
	}
	
	Node *n1 = FindNodePtr(refobj);
	
	if (n1==NULL) 
	{
		InsertTop(newobj);
		return FALSE;
	}
	
	Node *NewNode = new Node(newobj);
	if(!NewNode)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Lista","InsertAfter()");
	
	
	NewNode->Next = (n1->Next);
	n1->Next = NewNode;
	NewNode->Prev = n1;
	
	if ((NewNode->Next) !=NULL)
		(NewNode->Next)->Prev = NewNode;
	else	// inserimento in fondo alla lista
		last = NewNode;
	
	numobject ++;
	return TRUE;
}

DrawObject *Lista::DelFirst(void)
{
	Node *pnode = first;
	DrawObject *tmp;
	
	
	if (pnode) {
		first = pnode->Next;
		if (lastRef==pnode) 
			lastRef=first;
		
		if(first==NULL)
			last = NULL;
		else
			first->Prev = NULL;
		numobject--;
		tmp = pnode->ObjectPtr;
		delete pnode;
		pnode=NULL; 
		return tmp;
	}
	
	return NULL;
}

DrawObject *Lista::DelLast(void)
{
	Node *pnode = last;
	DrawObject *tmp;
	
	if (pnode) {
		last = pnode->Prev;
		if (lastRef==pnode) 
			lastRef=last;
		
		if(last==NULL)
			first = NULL;
		else
			last->Next = NULL;
		numobject--;
		tmp = pnode->ObjectPtr;
		delete pnode;
		pnode=NULL; 
		return tmp;
	}
	
	return NULL;
}

Bool Lista::isEmpty(void)
{
	return first==NULL;
}

void Lista::InsertAfter(Node *n, DrawObject *newobj)
{
	Node *NewNode = new Node(newobj);
	if(!NewNode)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Lista","InsertAfter()");
	
	NewNode->Next = n->Next;
	NewNode->Prev = n;
	n->Next = NewNode;
	if(n==last)
		last=NewNode;
	else
		NewNode->Next->Prev=NewNode;
    
	numobject++;
}

/*
*  Metodo per cancellare tutti i contenuti della lista deallocandoli
*
*  usare solo per una lista di cui si e' proprietari.
*/

void Lista::ClearDeep(void)
{
	Node *dummy=first;
	Node *dummy1=dummy;
	while(dummy)
	{
		dummy=dummy->Next;
		if (dummy1->ObjectPtr)
			delete dummy1->ObjectPtr;
		if (dummy1)
			delete dummy1;
		dummy1=dummy;
	}
	first=NULL;
	last=NULL;
	numobject = 0;
}

