/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: lista.hpp,v 1.1 2005/07/25 16:31:31 bellinpo Exp $ */


#ifndef LISTA_HPP
#define LISTA_HPP

#include "drw/drawobje.hpp"

#include "lioo.hpp"

struct Node
{
	DrawObject *ObjectPtr;
	Node *Next;
	Node *Prev;
	Node(void);
	Node(DrawObject *p);
};

/*
* La struttura Node non puo' essere messa protected a causa della funzione
* membro FindNodeName che restituisce un puntatore a Node
*/

class Lista
{
protected:
	Node *first,*last,*lastRef;
    // puntatore al primo e all'ultimo elemento della lista
	unsigned long numobject;
    // contatore di oggetti
	
	Node *GetNext(Node *);
    // restituisce il nodo successivo se presente, altrimenti ritorna NULL
	Node *GetPrev(Node *);
    // restituisce il nodo preccedente se presente, altrimenti ritorna NULL
	Node *FindNodePtr(DrawObject *ptr);
    // ricerca un nodo con associato l'oggetto indicato
	void InsertAfter(Node *, DrawObject *);
    // inserisce un nuovo nodo dopo il nodo indicato.
	
public:
	Lista(void);
	~Lista(void);
	
	unsigned long GetNumObj(void) ;
    // restituisce il numero degli elementi
	Bool isEmpty(void);
    // restituisce 1 se la lista e' vuota       
    
	DrawObject *GetFirst(void) ;
    // restituisce il primo elemento della lista
	DrawObject *GetLast(void) ;
    // restituisce l'ultimo elemento della lista
	DrawObject *GetNext(DrawObject *pdo);
    // restituisce l'elemento successivo di do
	DrawObject *GetPrev(DrawObject *pdo);    
    // restituisce l'elemento precedente di do
	
	virtual void InsertEnd(DrawObject *newobj);
    // inserisce un elemento in fondo alla lista
	virtual void InsertTop(DrawObject *newobj);
    // Inserisce un elemento in cima alla lista
	virtual Bool InsertAfter(DrawObject *newobj, DrawObject *refobj);
    // Inserisce un nuovo elemento dopo l'elemento passato per riferimento:
    // 
    //		Se viene passato NULL come riferimento, inserisce in testa
    //			(ritorna TRUE)
    // 		se l'oggetto di riferimento non viene trovato, ritorna FALSE e
    //			inserisce in testa.
    
    
	void DelPtr(DrawObject *ptr);
    // cancella l'oggetto che punta a ptr
	void DelPtrNode(Node *pn);
    // cancella pn e l'oggetto puntato da pn
	
	DrawObject *DelFirst(void);
    // cancella il primo elemento della lista
	DrawObject *DelLast(void);
    // cancella l'ultimo elemento della lista
	
	Bool FindPtr(DrawObject *ptr);
    // cerca ptr nella lista
    
	long GetPos(DrawObject *ptr);
    // restituisce la posizione dell'elemento nella lista
	DrawObject *AtPos(unsigned long i);
    // restituisce l'elemento i-esimo della lista (i>=0 && i<numobject)
    
	void Swap(DrawObject *p1, DrawObject *p2);
    // Scambia p1 con p2 se sono in lista
	DrawObject *Change(DrawObject *p1, DrawObject *p2);
    // Sostituisce p1 con p2
	
	void Clear(void);
    // dealloca la lista
	void ClearDeep(void);
    // dealloca tutto il contenuto.
	
};

#endif //LISTA_HPP
