/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/xmlunidefs.hpp>
#include <xercesc/dom/deprecated/DOM.hpp>
#include <xercesc/dom/deprecated/DOMParser.hpp>
#include <xercesc/dom/deprecated/DOM_Document.hpp>
#include <xercesc/dom/deprecated/DOM_Node.hpp>
#include <xercesc/dom/deprecated/DOM_NamedNodeMap.hpp>
#include <xercesc/dom/deprecated/DOM_NodeList.hpp>
#include "xml/loa/wdfdomparser.hpp"

#include <wx/file.h>
#include "lyr/listasillabe.hpp"
#include <string.h>
#include "smr/smr.hpp"

#define LINES_DIST 30
#define MIN_DIST_FIG 15

#ifdef _LOAD_XML_	

#ifdef WX226
#include <iostream.h>
#include <fstream.h>
#else
#include <iostream>
#include <fstream>
using namespace std;
#endif

// ---------------------------------------------------------------------------
//  Local data
//
//  doEscapes
//      Indicates whether special chars should be escaped in the output.
//      Defaults to doing escapes, -NoEscape overrides.
// ---------------------------------------------------------------------------
static bool     doEscapes       = true;


// ---------------------------------------------------------------------------
//  Forward references
// ---------------------------------------------------------------------------
void          outputContent(ostream& target, const DOMString &s);
ostream& operator<<(ostream& target, const DOMString& toWrite);
ostream& operator<<(ostream& target, DOM_Node& toWrite);
#endif

// COSTRUTTORE //
//  La costruzione di questa classe si appoggia //
//  ad una classe gi esistente che si occupa   //
//  della gestione delle liste.Nel costruttore  //
//  vengono anche inizializzati tre valori:     //
//  1) Layer.                                   //
//  2) InterLine.                               //
//  3) DistPent.                                //
//  Il primo specifica il Layer della battuta a //
//  cui le sillabe della lista si riferiscono.  //
//  Il secondo la strofa di LIRICA a cui si     //
//  riferiscono.Il terzo la distanza dall'ultimo//
//  rigo di pentagramma.	                //


ListaSillabe::ListaSillabe(int line,short layer,VUnit DISTPENT)
{
	
	CodeCounter=1;
	// Codice numerico assegnato ad ogni
	// elemento della  lista.
	InterLine=line;
	// Indica la strofa a cui si riferisce.	
	Layer=layer;
	// Indica il layer a cui appartengono le
	// note a cui la lirica si riferisce.
	BaseLineFactor= 1.;
	LinesDistFactor=1.;
	DistPent=DISTPENT;
	// Questo valore viene passato per permetterne
	// l'adattamento in caso di figure che escono
    // dal pentagramma.
	listaBat=NULL;
	
	for(int i=0;i<MAX_LYRIC_LINES;i++)
	{
		Started[i]=NULL;
	}
};



// COSTRUTTORE DI DEFAULT //

ListaSillabe::ListaSillabe()
{
	CodeCounter=1;
	InterLine=1;
	Layer=1; 
	BaseLineFactor =1.;
	LinesDistFactor=1.;
	DistPent=LINES_DIST;
	listaBat=NULL;
	for(int i=0;i<MAX_LYRIC_LINES;i++)
	{
		Started[i]=NULL;
	}
	
};


// SETTAGGIO LAYER DI RIFERIMENTO //
// STROFA DELLA LIRICA E MINIMA   //
//   DISTANZA DAL PENTAGRAMMA.    //


void ListaSillabe::SetLine(int line)
{
	InterLine=line;
};

void ListaSillabe::SetLayer(short layer)
{
	Layer=layer;
};

void ListaSillabe::SetDistPent(VUnit distpent)
{
	DistPent=distpent;
};


// DISTRUTTORE //


ListaSillabe::~ListaSillabe()
{
	
	Free();
	
};


void ListaSillabe::Free()
{
	// In questo caso non  stato possibile
	// utilizzare il metodo gi esistente a
	// causa dei problemi(di manipolazione
	// dei puntatori) dovuti al doppio legame
	// di TEXT con la classe DRAWOBJECT.
	
	Node *pn=first;
	Sillaba *p1;
	Node *pn1=pn;
	Figura *pFStart,*pFEnd;
	Battuta *pBatCor;
	
	while(pn!=NULL)
	{               
		pn=pn->Next;
		p1=(Sillaba*)((void*)(pn1->ObjectPtr));
		
		if(listaBat!=NULL)
		{
			// La deallocazione delle sillabe deve
			// preoccuparsi anche di resettare lo
			// attributo AppSil di ogni figura.
			pFStart=p1->GetPFigStart();
			// Ritorna la figura iniziale a cui
			//  associata la sillaba.
			pBatCor=listaBat->GetBattWithFig(pFStart);
			// Permette di ottenere la battuta
			// corrente.
			pFEnd=p1->GetPFigEnd();
			// Ritorna la figura finale a cui
			//  associata la sillaba.
			InterLine=(p1->GetLine()); 
			
			if (pBatCor != NULL)
			{
				const int MIN_INDEX_OF_LAYER = 0; 
				int Index_Max_Of_Layer = pBatCor->GetNumberOfStaff()*4;
				if(pBatCor != NULL)
				{
					for(int indlay =MIN_INDEX_OF_LAYER; indlay<=Index_Max_Of_Layer; indlay++) 
					{ 
						p1->SetReset(pFStart,pFEnd,indlay,pBatCor,InterLine,0,FALSE);
					}
				}
			}
			// Resetta l'attributo AppSil si ogni
			// figura associata alla sillaba corrente.
		}
		
		pn1->ObjectPtr=p1;
		// Permette di evitare i problemi
		// sui puntatori dovuti al doppio
		// legame di Text con DrawObject.
		delete pn1;
		pn1=pn;				 		
		
	};
	first=NULL;
	last=NULL;
	numobject=0;
};


// INIZIALIZZATORE DI LISTA //


void ListaSillabe::Init()
{
	first=NULL;
	last=NULL;
	numobject=0;
};



// FUNZIONI PER IL RILEVAMENTO //
//  DI UNA SILLABA ASSOCIATA   // 
//      AD UNA FIGURA.         //

// Scandisce la lista per determinare un nodo
// che abbia un'eventuale sillaba associata
// che inizia su essa.
// La sillaba pu essere di qualunque natura
// estesa,singola.
// Se lo trova restituisce un puntatore a
// quel nodo,altrimenti ritorna NULL.


Node *ListaSillabe::GetSilF(Figura *pF, Node* pn)
{
	
	
	
	Sillaba *pInt;       
	
	
	if (pn==NULL)
		return NULL;
	
	pInt=(Sillaba*)((void*)(pn->ObjectPtr));                        
	
	while((pn!=NULL) && (pInt->GetPFigStart()!=pF))
	{
		pn=pn->Next;
		if(pn!=NULL)
			pInt=(Sillaba*)((void*)(pn->ObjectPtr));	
		
	}
    
	return pn;
	
};


// Scandisce la lista per determinare un nodo
// che abbia un eventuale prolungamento 
// che inizia su pF.
// Restituisce NULL se non trova niente.
// Setta inoltre comunque la coordinata della
// sillaba associata alla figura,in base al
// valore calcolato in Spartito.


Node* ListaSillabe::GetIntStartF(Figura *pF,VUnit y,Node *pn)
{	
	Sillaba *pInt;
	
	if((pn=GetSilF(pF,pn))==NULL)
		return NULL;
	   
	pInt=(Sillaba*)(void *)(pn->ObjectPtr);
	   
	if ((pn=GetSilF(pF,pn))!=NULL)
	{
		pInt->SetYSil(SetPos(pF,pInt,pInt->GetLine()));
		// In questo caso viene inoltre settata
		// la coordinata y della sillaba,in base
		// al valore passatogli da Spartito,per
		// il suo successivo posizionamento
		// grafico.
		return pn;	
	};
	return NULL;
};


// Scandisce la lista per deterrminare un
// nodo che abbia un eventuale prolungamento
// che termina su pF.
// Restituisce NULL se non trova niente.


Node *ListaSillabe::GetIntEndF(Figura *pF, Node *pn)
{
	
	Sillaba *pInt;
	
	if(pn==NULL)
		return NULL;
	
	pInt=(Sillaba*)(void *)(pn->ObjectPtr);
	
	while((pn!=NULL) && (pInt->GetPFigEnd()!=pF))
	{
		pn=pn->Next;
		if(pn!=NULL)
			pInt=(Sillaba*)(void *)(pn->ObjectPtr);
		
	};
	return pn;
	
	if ((pn!=NULL) && (pInt->GetProl()))
		return pn;
	else
		return NULL;
	
};


// FUNZIONE PER OTTENERE //
//  LA LISTA DI FIGURE   //
//      CORRENTE.        //



ListaFigure *ListaSillabe::GetPLayer(Battuta *pBatCor)
{
	// Questa funzione sulla base del Layer di
	// riferimento e della battuta corrente,
	// richiamando metodi gi esistenti nella
	// classe BATTUTA,permette di ricavare lo
	// indirizzo della lista di figure che si
	// sta considerando.
	
	if(pBatCor==NULL)
		return NULL;
	
	if (Layer==1)
		return(pBatCor->GetPLayer(0));
	else
		return(pBatCor->GetPLayer(1));
	
};


// FUNZIONI DI CONTROLLO //
// FIGURA GIA` OCCUPATA  //



Figura *ListaSillabe::VeriFig(Figura *pCor,Figura *pFEnd,ListaFigure *pLayer,int ns)
{
	
	// Verifica a partire dalla figura succesiva
	// a quella corrente(gi controllata nel metodo
	// che richiama questa) se estiste almeno una
	// figura che ha gi una sillaba assegnata.
	// Se non la trova prima procede fino a che
	// o non trova la figura finale a cui la sillaba
	//  associata o non giunge al termine della
	// battuta.
    Figura *ptr=NULL;
	int Occupato;
	Occupato=0;
	if(pLayer != NULL)
		ptr=pLayer->GetFirstFig();
	else
		ptr = NULL;
	
	while((pCor!=ptr)&&(ptr!= NULL))
		ptr=pLayer->GetNextFig();//vedere  
	
	
	// Questo ciclo e' necessario
	// affinche' il metodo sopra
	// funzioni correttamente.			
	
	while((pCor!=NULL)&&(pCor!=pFEnd)&&(Occupato!=1))
	{				
		pCor=(pLayer->GetNextFig());	
		// Scorre avanti di una figura
		// sia che sia isolata,sia che
		// sia un accordo che un gruppo
		// di note.	
		if(pCor!=NULL)
		{
			Occupato=pCor->GetAppSil(ns-1);
		}
		// Verifica se a quella figura 
		// gi associata una Sillaba.
	};
	
	
	return pCor;	// Restituisce l'ultima figura
	// controllata.
	
	
};



Bool ListaSillabe::VerifOccup(Figura *pF1,Figura *pF2,Battuta *pBatCor,int ns,int IndexLayer)
{
	Figura *punt1;
	Battuta *pBatSuc;
	ListaFigure *pLayer;
	
    // Si vanno a considerare solo le figure
	// del Layer a cui  associata la LIRICA,
	// richiamando la funzione che ritorna
	// il puntatore alla lista corrente.
	   
	pLayer = pBatCor->GetPLayer(IndexLayer);
	
	
	// Se la sillaba estesa si estende a cavallo
	// tra una o pi battute il controllo della
	// variabile che indica che la figura  occupata
	// deve proseguire anche nelle battute successive
	// a quella in cui ha inizio.Il controllo termina
	// quando la figura esaminata coincide con la
	// figura finale a cui  associata la sillaba
	// estesa oppure si trova una figura occupata.
	
	pBatSuc=pBatCor;
	punt1=pF1; 
	
	while((pBatSuc!=NULL) && (punt1!=pF2) && (punt1->GetAppSil(ns-1)!=1))
	{	
		punt1=VeriFig(punt1,pF2,pLayer,ns);
		// Nel caso fossimo a fine battuta va considerata
		// la prima figura della battuta successiva.			
		
		if (punt1==NULL)
		{
			pBatSuc=(Battuta*)(listaBat->GetNext((DrawObject*)(void*)pBatSuc));
			// Si scorre avanti di una battuta.
			
			pLayer = pBatSuc->GetPLayer(IndexLayer);
			if(pLayer!=NULL)
				punt1 = pLayer->GetFirstFig(); 
		};	
	};
	
	if (punt1->GetAppSil(ns-1)==1)		   
		// Il -1  dovuto al fatto che il
		// vettore AppSil, che identifica
		// come occupata,come tale ha come
		// prima posizione lo zero.
		return TRUE;
	else
		return FALSE;
};



// FUNZIONI DI INSERIMENTO //


///////////////////                    
void ListaSillabe::Add(const char *s, Figura *pF1, Figura *pF2, Battuta *pb1,Battuta *pb2 ,
					   int numtoken,Bool Trat,Bool TraSil,char Token,int NumSep,int strofa,
					   char gs ,int NumLayer,bool Associazione, double shf, double Bl)
{
	
	// Metodo richiamato da spartito per //
	// l'inserimento di una sillaba da   //
	// tastiera.
	
	
	Sillaba *pInt;
	NumCode nc;
	
	
	// Se si tenta di inserire una sillaba
	// sopra un'altra,il tentativo deve fallire. 
	if(Associazione && listaBat!=NULL && pF1!=NULL && pb1!=NULL)
	{
		if (VerifOccup(pF1,pF2,pb1,strofa,NumLayer))
			return;
	}
	
	// Se il risultato del metodo richiamato
	// sopra  FALSE significa che alcuna
	// sillaba  gi associata alle figure
	// in questione e si pu quindi procedere
	// all'inserimento.				
	SetBaseLineFactor(Bl);
	SetLinesDistFactor(shf);
	pInt=new Sillaba(s);
	pInt->SetToken('N');
	pInt->SetNumtoken(0);
	// Crea una nuova sillaba con testo s.
	if(Associazione)
		pInt->SetListaBat(listaBat);
	// Il metodo richiamato passa l,indirizzo
	// della lista di battute corrente,affinch
	// sia possibile l'aggiornamento dell'attributo
	// AppSil di ogni figura.
	
	if(Associazione)
	{
		if (pInt==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaSillabe","Add()");
		// MESSAGGUIO DI ERRORE.
	}
	
	pInt->SetLine(strofa);
	pInt->SetShift(shf);
	pInt->SetAss(Associazione);
	
	
	if(Associazione)
		pInt->SetPFig(pF1, pF2,NumLayer,numtoken,pb1,InterLine,Trat,TraSil);
	
	// Si procede al settaggio degli attributi
	// del nuovo oggetto Sillaba della lista.
	
	InsertEnd((DrawObject*)(void*)pInt);
	// Inserisce la sillaba in fondo alla
	// lista.
	
	
#ifdef WITH_NET
	if (tipoLeggio==LEGGIO_MASAE)
		nc = CodeCounter++;
#else
	nc = CodeCounter++;
#endif
	
	pInt->SetNumCode(nc);
	pInt->SetNumBat(1,pb1);
	pInt->SetNumBat(2,pb2);
	
	pInt->SetLine(strofa);
	pInt->SetNLayer(NumLayer);
	pInt->SetShift(shf);
	pInt->Setgs(gs);
	if(Token != 'N')
	{
		SetAttrSep(NumSep,Token,pInt);
	}
	
	/////////////////////////////////////	
};

void ListaSillabe::InserSillaba(Figura *Fig1,Figura *Fig2,const char *testo,Battuta *pBat1,
								Battuta *pBat2,Bool TRAT,Bool TRASIL)
{
	
	
	// Metodo richiamato da SPARTITO  per //
	//  inserire in una lista una lirica  //
	//   gi presente e salvata su un     //
	//        documento XML.              //
	
	
	Sillaba *pInt;
	// Puntatore alla sillaba da inserire.
	NumCode nc;
	// Codice numerico che contraddistingue
	// la sillaba.
	
	pInt=new Sillaba(testo);	
	// Crea una nuova sillaba con testo s.
	
	
	if (pInt==NULL)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaSillabe","Add()");
	
	// SETTAGGIO ATTRIBUTI DELLA SILLABA //
	
	pInt->SetPFigStart(Fig1);
	// Setta la figura iniziale  a cui  associata
	// la sillaba.
	pInt->SetPFigEnd(Fig2);
	// Setta la figura finale a cui  associata
	// la sillaba.
	
	// Settaggio degli attributi che determinano //
	// la natura del prolungamento e calcolo del //
	// numero di figure che sono associate ad    //
	//          ogni sillaba estesa.             //
	
	int NumFig=0;;
	// Variabile temporanea settata inizialmente
	// a zero.
	
	if (Fig1==Fig2)
        // Se la sillaba non  estesa....
		pInt->SetProl(FALSE);
	// In questo caso il ovviamente NumFig
	// rimane nullo.
	else
	{
		pInt->SetProl(TRUE);
		if (!TRASIL)
		{
			// Se non siamo nel caso di sillaba prolungata ma
			// separata dalla successiva da semplice trattino.....
			
			ListaFigure *pLayer=GetPLayer(pBat1);
			// Variabile temporanea contenente
			// il puntatore alla lista di figure
			// corrente.
			Battuta *pBatSuc=pBat1;
			// Puntatore alla battuta successiva.
			Figura *ptr=pLayer->GetFirstFig();
			// Puntaore necessario alla scansione
			// all'interno delle battuta.
			Figura *punt=Fig1;
			
			while(punt!=ptr)
				//  necessaria questa scansione
				// per poter utilizzare correttamente
				// il metodo GetNextFig.
				ptr=pLayer->GetNextFig();
			// Prende la figura successiva.
			
			while((pBatSuc!=NULL) && (punt!=Fig2))
			{
				// Mentre non sono terminate le battute e non
				// si  giunti alla figura finale.
				
				punt=(pLayer->GetNextFig());	
				// Passa alla successiva.Poich potrebbe
				// anche essere uno spazio...
				
				if (punt!=NULL)
				{
					if(punt->GetID()!=CL_SPAZIO)
						NumFig++;
					// Viene incrementato solo se
					// la figura corrente non  uno
					// spazio.
				}
				else
				{
					// Se siamo giunti alla fine della battuta....
					pBatSuc=(Battuta*)(listaBat->GetNext((DrawObject*)(void*)pBatSuc));
					//.. si scorre avanti di una battuta.
					pLayer=GetPLayer(pBatSuc);
					punt=pLayer->GetFirstFig();
					
				};	
				
			};
			
			
			if (!TRAT)
				// Se la sillaba  estesa con prolungamento continuo
				NumFig++;        	
		};
	};	
	
	
	pInt->SetProlTrat(TRAT);
	// Setta l'attributo che specifica il
	// tipo di prolungamento della sillaba
	// qualora sia estesa.
	pInt->SetSegTraSil(TRASIL);
	// Setta l'attributo che specifica che
	// la sillaba  divisa dalla successiva
	// tramite separatore.
	
	pInt->SetNumtoken(NumFig);
	// Setta l'attributo che tiene conto
	// del numero di figure a cui  associato
	// il prolungamento di una sillaba estesa.
	
	InsertEnd((DrawObject*)(void*)pInt);
	// Inserisce in fondo alla lista la
	// nuova sillaba.
	
#ifdef WITH_NET
	if (tipoLeggio==LEGGIO_MASAE)
		nc = CodeCounter++;
#else
	nc = CodeCounter++;
#endif
	
	pInt->SetNumCode(nc);
	pInt->SetNumBat(1,pBat1);
	pInt->SetNumBat(2,pBat2);
	
	
};



// METODI PER LA GESTIONE DEI //
//  PROLUNGAMENTI DURANTE LA  //
// FASE DI COSTRUZIONE DELLO  //
//        SPARTITO.           //



// Memorizzazione,nella variabile STARTED,
// della sillaba il cui prolungamento
// inizia durante la scansione della
// battuta.

void ListaSillabe::InsStarted(Sillaba *pInt)//vecchio
{
	Started[pInt->GetLine()-1]=pInt;
};

// RESTITUZIONE STARTED //


Sillaba *ListaSillabe::GetStarted()
{
	Sillaba *St =NULL;
	for(int i=0;i<MAX_LYRIC_LINES;i++)
	{
		if(Started[i]!= NULL)
		{
            St = Started[i];
		}
	}
	return St;
	
    
};

Sillaba *ListaSillabe::GetStarted(int ns)
{
	return Started[ns];
}

// INIZIALIZZA LO STARTED.
// Utilizzato nei metodi che
// riscontrano l'estensione
// del prolungamento tra due
// battute diverse,permette
// l'interruzione della ricerca.

Sillaba *ListaSillabe::InitStarted()
{
	for(int i=0;i<MAX_LYRIC_LINES;i++)
	{
		Started[i]=NULL;
	}
	
	return NULL;
};


// Rimuove la sillaba dallo STARTED
// ponendolo uguale ad NULL, nel 
// caso il prolungamento abbia inizio e
// fine nella stessa battuta.

void ListaSillabe::RemoveStarted(Sillaba *pInt)//vecchio
{
	pInt->SetRemove(TRUE);
	
	Started[pInt->GetLine()-1]=NULL; 
};

// Imposta la coordinata x della
// linea nello STARTED che costituisce
// il prolungamento di una sillaba
// estesa iniziata in una battuta
// precedente.

void ListaSillabe::PosStarted(const Point& p )
{
	Sillaba *pInt;
	
	for(int i=0;i<MAX_LYRIC_LINES;i++)
	{
		pInt=Started[i];
		
		// Calcolo della coordinata x di partenza della
		// linea costituente il prolungamento.
		
        if (pInt!=NULL)
		{
			
			if (!pInt->GetRemove())
			{
				pInt->SetDim(p.x - pInt->GetAbsPos().x);
				pInt->SetFine(TRUE);
			};
        };    	
	};	
};

void ListaSillabe::PosStarted(Figura *pF)//vecchio
{
	Point p;
	
	p=pF->GetAbsPos(); 
	PosStarted(p);
};

// Dealloca le linee costituenti
// il prolungamento di tutte le
// sillabe estese.

void ListaSillabe::ResetPos()
{
	Node *pn = first;
	Node *pNext=NULL;
	Sillaba *pInt;
	
	while(pn!=NULL)
	{
		pInt = (Sillaba*)(void *)(pn->ObjectPtr);
		
		if (pInt->GetProl())
			pInt->DeallocaLin();
		// Si dealloca i prolungamenti(eccetto
		// il primo) della sillaba in questione.
		// Operazione che si rende necessaria a
		// fine rigo o pagina.
		pInt->SetRemove(FALSE);
		pInt->SetInizio(FALSE);
		pInt->SetYSil(-1);
		// Si inizializza la coordinata y in modo
		// da indicare che la sillaba non  stata
		// ancora definitivamente posizionata.
		pn=pn->Next;
	};
};


// Pone nello STARTED il simbolo
// della pagina precedente che 
// termina con pF.


void ListaSillabe::FindPrevStartedFig(Figura *pF)
{
	Sillaba *pInt;
	Node *pn;
	VUnit y=0;
	
	// Trova la sillaba il cui prolungamento
	// inizia con pF e gli alloca una linea.
	
	
	pn = GetIntStartF(pF,y,first); // Restituisce il puntatore ad un nodo
	// che ha un prolungamento che inizia su
	// pF.	
	while(pn!=NULL)
	{
		pInt = (Sillaba*)(void *)(pn->ObjectPtr);
		if((pInt->GetPFirstLin())==NULL)
			pInt->AddLin();
		// Questa linea non contiene
		// ancora informazioni su
		// larghezza o posizione.
		pn = GetIntStartF(pF,y,pn->Next);
		
    };
	
	// Trova il prolungamento che termina con pF.
	// Poich ogni volta che Spartito si appresta
	// a comporre un nuovo rigo di PENTAGRAMMA
	// dealloca tutti i segmenti componenti le
	// sillabe(non crea problemi perch quelle
	// appartenenti ai righi passati sono gi state
	// disegnate) se ha PFirstLin == NULL,significa
	// che il prolungamento  iniziato nella pagina
	// precedente. Quindi deve essere messo
	// nello STARTED.
	
	pn = GetIntEndF(pF,first);
	while(pn!=NULL)
	{
		pInt = (Sillaba*)(void *)(pn->ObjectPtr);
		
		if((pInt->GetPFirstLin())==NULL && pInt->GetProl())
		{
			// pInt viene dalle pagine precedenti
			// => va posto negli Started.
			pInt->AddLin();
			InsStarted(pInt);
		};
		pn = GetIntEndF(pF,pn->Next);
	};
	
};

// Pone nello STARTED la sillaba estesa
// il cui prolungamento  iniziato nelle
// pagine precedenti termina nella battuta.
// In pratica esegue il metodo precedente
// per ogni figura della battuta.Poich
// ogni sillaba di una strofa pu essere
// associata solo ad un Layer,la scansione
// viene eseguita solo sulle figure della
// battuta del layer a cui la strofa si riferisce.


void ListaSillabe::FindPrevStartedBat(Battuta *pBat)
{
	ListaFigure *pLayer;
	Figura *pF;
	
	InitStarted();
	
	int Index_Max_Of_Layer = pBat->GetNumberOfStaff()*4;
	for(int indlay=0; indlay<=Index_Max_Of_Layer; indlay++) 
	{ 		
		// Sulla base del Layer a cui le sillabe della
		// strofa  si riferiscono,viene inizializzato
		// il puntatore della battuta corrente.
		
		pLayer=pBat->GetPLayer(indlay);
		
        // Assegna a pLayer il puntatore alla lista
		// di figure relative alla battuta pBat.
		if(pLayer==NULL)
			continue;
		
		pF = pLayer->GetFirstFig();
		// Assegna a pF la prima figura della battuta.
		
		// Inizializza lo Started.
		
		while((pF!=NULL) && (GetStarted()==NULL))//vecchio
        {       // Il ciclo ha termine quando le figure che
			// compongono la battuta hanno termine oppure
			// la figura in esame  associata ad una
			// sillaba estesa il cui prolungamento ha
			// termine sulla figura stessa.
			// Poich ad ogna figura pu essere associato
			// un solo prolungamento non ha senso procedere
			// oltre.
			FindPrevStartedFig(pF);
			// Verifica l'esistenza di una sillaba
			// estesa associata alla figura in esame.
			pF=pLayer->GetNextFig();
			// Passa alla figura successiva sia essa
			// un accordo,un gruppo di note oppure
			// una singola nota.
		};
	}
};


// Esamina la figura pF controllando se il
// prolungamento della sillaba associata  
// termina o inizia in pF.
// Se termina in pF viene tolta dallo STARTED 
//	altrimenti vi viene messa.


void ListaSillabe::ExamineFig(Figura *pF,VUnit YPent)
{
	Sillaba *pInt;
	Node *pn;
	VUnit Y=0;
	
	// Se nello STARTED  presente una linea
	// viene calcolata la coordinata x di
	// partenza.
    
	PosStarted(pF);//vecchio
	
	
	// Sulla base della coordinata y della
	// linea finale del pentagramma viene
	// calcolata la coordinata y,che la
	// sillaba deve avere nello Spartito.
	
	
	// Toglie dallo STARTED la sillaba il
	// cui prolungamento termina con *pF.		
	
	pn = GetIntEndF(pF,first);
	while (pn!=NULL) // Se la sillaba  estesa e termina
		      // in pF,viene tolta dallo STARTED.
	{
		//selsilrggiusto
		pInt = (Sillaba*)(void *)(pn->ObjectPtr);
		if(pInt->GetProl())
			RemoveStarted(pInt);
		
		if(pInt->GetPFirstLin()==NULL)
			pInt->AddLin();
		
		Y=YPent-(int)(DistPent*GetBaseLineFactor())-(pInt->GetLine()-1)*(LINES_DIST*GetLinesDistFactor());
		
		pInt->SetAbsPosY(Y);
		
		pn=GetIntEndF(pF,pn->Next); 
	};
	
	// Verifica che la sillaba associata a pF
	// sia estesa e in tal caso la pone in
	// STARTED.
	// Aggiorna le coordinate x ed y della
	// sillaba. 
	
	pn = GetIntStartF(pF,Y,first);
	while (pn!=NULL) // Se la Sillaba  estesa e inizia in
		      // pF viene messa nello STARTED.
	{
		pInt = (Sillaba*)(void *)(pn->ObjectPtr);				  
		InsStarted(pInt);
		
		if(pInt->GetPFirstLin()==NULL)
			pInt->AddLin();
		
		pInt->SetAbsPosX(pF->GetAbsPos().x );
		pInt->SetInizio(TRUE);
		// Indica che il tratto di linea
		// inizia in questa pagina.
		
		Y=YPent-(int)(DistPent*GetBaseLineFactor())-(pInt->GetLine()-1)*(LINES_DIST*GetLinesDistFactor());
		
		pInt->SetAbsPosY(Y);
		pn=GetIntStartF(pF,Y,pn->Next);
	};  
};

// Nel che la battuta in esame sia di 
// inizio pagina o pentagaramma viene 
// posizionata la x di una sillaba nello
// STARTED in cima a sinistra.


void ListaSillabe::ExamineInizioPentagr(Battuta *pBat,int ns)
{
	Sillaba *pInt;
	tipoBattuta tpBat = pBat->GetBattuta();
	Figura *pF;
	VUnit x,Y,YPent;
	
	if((tpBat==INIZIO_PAGINA) || (tpBat==INIZIO_PENTAGRAMMA)
		|| (tpBat==INIZIO_BRANO))
		
	{       // Calcola l'ascissa di partenza del prolungamento
		// all'inizio della battuta.
		
		pF = FindFirstFigOnLayerNotNull(pBat,0); 
		
		// In pF viene cos memorizzato lo
		// indirizzo della prima figura della
		// battuta corrente.
		if(pF!=NULL)
			x = pF->GetAbsPos().x - WIDTH_RIPRESA_SILLABA;
		else
			x = pBat->GetAbsPos().x + pBat->GetIntest()->GetWidth();
		
		// Riprendi il contenuto dello STARTED e
		// lo aggiorna.
		
		for(int i=0; i<4; i++)
		{
			pInt = GetStarted(i);//vecchio pb
			
			if (pInt!=NULL)
			{
				pInt->SetAbsPosX(x);
				// Setta la coordinata x di partenza.
				YPent=pBat->GetIntest(0)->GetAbsPos().y;
				// Rappresenta la coordinata della
				// ultima linea del pentagramma della
				// battuta corrente.
				Y=YPent-(int)(DistPent*GetBaseLineFactor())-(InterLine-1)*(LINES_DIST*GetLinesDistFactor());
				pInt->SetAbsPosY(Y);
			}
		}       // Setta la coordinata y di partenza.
	}
};      						

// Esamina la battuta e per ogni figura pF 
// controlla se la sillaba  estesa e quindi 
// se il prolungamento termina o inizia in pF.
// Nel primo caso la sillaba viene tolta dallo 
// STARTED,nel secondo vi viene messa.


void ListaSillabe::ExamineBat(Battuta *pBat)
{
	ListaFigure *pLayer;
	Figura *pF=NULL;
	VUnit YPent;
	
	YPent=pBat->GetAbsPos().y;
	
	// Poich le battute nel caso DILOO possano
	// seguire su righi consecutivi diversi,per
	// ogni nuova battuta si rende necessario il
	// calcolo dell'ultima linea del pentagramma
	// a cui appartiene.
	
	//  se siamo alla prima passata,
	//  e se (tpBat==INIZIO_PAGINA || tpBat==INIZIO_PENTAGRAMMA)
	//  posiziona la x dello Started in cima a sinistra.
	
	ExamineInizioPentagr(pBat,0);
	
	// Anche in questo caso poich ogni sillaba
	// di una strofa pu riferirsi alle figure
	// di un solo Layer alla volta,determinato
	// quello corrente si determina l'indirizzo
	// della prima figura della battuta.
	;
	
	if(pBat->GetNumberOfStaff()<=1)
		pLayer=pBat->GetPLayer(0);
	else
		pLayer=pBat->GetPLayer(0);
	
    // Indirizzo della lista di figure corrente.
	if(pLayer!=NULL)
        pF=pLayer->GetFirstFig();
	
	// Punta alla prima figura della battuta
	// qualunque sia la sua natura(accordo,
	// gruppo di note o singola figura).
	
	while(pF!=NULL)
		// Fin quando la battuta non  terminata...
	{
		
       	ExamineFig(pF,YPent);
		// Esamina la figura in questione.
       	pF = pLayer->GetNextFig();
		// Passa alla figura successiva.
	};
};


void ListaSillabe::EndPage(Battuta *pB)
{
	Sillaba *pInt;
	Node *pn = first; 
	
	// Richiamata da Spartito ,una volta
	// terminata la scansione di un rigo
	// di pentagramma,conclude,se presente,
	// il prolungamento della sillaba estesa
	// non ancora terninata(il cui indirizzo
	//  in STARTED)sulla barra destra
	// della battuta.
	
	PosStarted(pB->GetBarra()->GetAbsPos() - Point(2,0)); //vecchio
	
	// Cancella il contenuto dello STARTED.
	// Ormai lo started  stato  posizionato,
	// quindi si pu cancellare.Il prolungamento
	// che inizia nella pagina sar riposizionato
	// lungo x nella successiva passata.
	
	for(int i=0; i<MAX_LYRIC_LINES; i++)
		if(GetStarted(i)!=NULL)
			RemoveStarted(GetStarted(i));
		
		pn=first;
		
		// Fatto ci questo metodo si preoccupa
        // di inizializzare il puntatore del segmento
		// attivo da posizionare per procedere quindi
		// al disegno di tutte le sillabe gi prese
		// in considerazione(associate cio alle note
		// gi esaminate nel rigo corrente).
		
		while(pn!=NULL)
		{
			pInt = (Sillaba*)(void *)(pn->ObjectPtr);
			if (pInt->GetPFirstLin()!=NULL) pInt->ResetPosLin();  
			
			// GetPFirstLin()==NULL se la sillaba in esame
			// non  gi stata considerata dal metodo di
			// SPARTITO che si preoccupa del posizionamento
			// delle note e le sillabe a loro associate per
			// ogni rigo di PENTAGRAMMA.
			
			pn = pn->Next;
		}
		
};


// SOLO PER DLIOO:

void ListaSillabe::EndRow(Battuta *pB)
{
	Sillaba *pInt;
	
	// Conclude lo Started sulla barra destra
	// della battuta.
	
	PosStarted(pB->GetBarra()->GetAbsPos());
    
	
	// Alloca le linee per proseguire il
	// prolungamento sui successivi righi.
	
	for(int i=0; i<MAX_LYRIC_LINES; i++)
	{
		pInt =GetStarted(i);
		
		if(pInt!=NULL)
		{
			if (pInt->GetRemove()) // Se  gi stato posizionato..
				InitStarted();
			else
				pInt->AddLin();
			
			// Significa che continua nel rigo
			// sotto,quindi deve essere allocato
			// alla Sillaba una nuova linea.
		}
	}
};

/////////////////////////////////////
// FUNZIONI PER CANCELLARE SILLABE //
// ASSOCIATE A FIGURE O BATTUTE DA //
//            ELIMINARE.           //
/////////////////////////////////////


Figura *ListaSillabe::FindFigPrev(Figura *pFEnd,Figura *pFStart)
{
	// A partire dalla figura finale di una //
	// sillaba trova la prima che la precede//
	// che non sia uno spazio .             //
	
	Battuta  *pBatCor;
	
	if (pFStart==NULL)
	{
		// Nel caso in cui non si abbia la figura iniziale
		// da cui partire per la scansione.....
		
		pBatCor=(Battuta*)(listaBat->GetFirst());
		// Contiene la battuta corrente della lista
		// di figure che sin va a scansionare  la
		// prima della lista.
		pFStart=(GetPLayer(pBatCor))->GetFirstFig();
		while(pFStart==NULL && pBatCor!=NULL)
		{
			pBatCor=(Battuta*)(listaBat->GetNext(pBatCor));
			pFStart=(GetPLayer(pBatCor))->GetFirstFig();
		}
		// Prima figura della prima battuta della
		// lista.
	}
	else
		// Nel caso si abbia la figura iniziale da cui
		// far partire la scansione......
		
		pBatCor=listaBat->GetBattWithFig(pFStart);
	// Contiene la battuta a cui appartiene la
	// figura iniziale da cui incominciare la
	// la scansione .
	
	
	
	Battuta *pBatSuc=pBatCor;
	// Variabile di supporto che
	// memorizza la battuta successiva
	// a quella corrente.
	ListaFigure *pLayer;
	// Variabile di supporto che
	// memorizza la lista di figure
	// corrente.     				
	pLayer=GetPLayer(pBatCor);
	// Settaggio dell'idirizzo della
	// lista di figure sulla base della
	// battuta corrente.
	Figura *ptr=pLayer->GetFirstFig();
	// Prende la prima figura della lista
	// corrente.
	
	
	
	while ((pFStart!=ptr) && (pBatSuc!=NULL))
	{
		ptr=pLayer->GetNextFig();
		// Prende la figura successiva a quella
		// corrente.
		if (ptr==NULL)
		{
			// Se la battuta  terminata la verifica deve continuare
			// nella battuta successiva.
			pBatSuc=(Battuta*)(listaBat->GetNext((DrawObject*)(void*)pBatSuc));
			// Settaggio della battuta successiva.
			pLayer=GetPLayer(pBatSuc);
			ptr=pLayer->GetFirstFig();	
		};
	};
	// Se il ciclo sopra  terminato con successo
	// ptr si trova nella lista delle figure correnti
	// in corrispondenza della figura iniziale a
	// cui  associata la sillaba.
	
	
	if (pBatSuc==NULL)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaSillabe","FindFigPrev()");
	// Nel caso non fosse presente la battuta successiva,
	// si ritorna il messaggio di errore sopra.
	
	
	Figura *prec=NULL;
	// Variabile di supporto.
	
	// Determinazione della figura precedente
	// a quella da cancellare.
	
	while((ptr!=pFEnd) && (pBatSuc!=NULL))
	{
		if (ptr->GetID()!=CL_SPAZIO) 	
			prec=ptr;
		// Aggiornamento avviene solo se la figura
		// precedente  non  uno spazio.
		
		ptr=pLayer->GetNextFig();
		if (ptr==NULL)
		{
			// Se la battuta  terminata la verifica deve continuare
			// nella battuta successiva.				
			pBatSuc=(Battuta*)(listaBat->GetNext((DrawObject*)(void*)pBatSuc));
			// Settaggio della battuta successiva.
			pLayer=GetPLayer(pBatSuc);
			ptr=pLayer->GetFirstFig();
			
		};
		
	};
	
	
	if (pBatSuc==NULL)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaSillabe","FindFigPrev()");
	// Nel caso non fosse presente la battuta successiva,
	// si ritorna il messaggio di errore sopra.
	
	
	return prec;	
	
};

void ListaSillabe::DelNodo(Sillaba *pIntcorr)
{
	
	Node *pn=first;
	Node *pre=NULL;
	Node *post=NULL;
	Node *pn1=pn;
	
	Sillaba *pts;
	
	if(pn==NULL)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaSillabe","DelNodo()");
	
	pts=(Sillaba*)(void *)(pn->ObjectPtr);
	
	while((pn!=NULL) && (pts!=pIntcorr))
		
	{
		pn=pn->Next;
		pts=(Sillaba*)((void *)(pn->ObjectPtr));
	};
	
	pn1=pn;
	if ((pts==pIntcorr) && (pn!=NULL))
	{
		
		
		Node *pre=pn->Prev;;
		// Variabile temporanea contenente
		// il nodo precedente a quello corrente.
		Node *post=pn->Next;
		// Variabile temporanea contenente
		// il nodo successivo a quello corrente.
		
		
		// Aggiornamento puntori della lista.
		if(pre==NULL && post==NULL)
		{
			// Se era l'unico nodo nella lista....
			first=NULL;
			last=NULL;
			numobject=0;
		};
		
		if((pre==NULL)&&(post!=NULL))
		{
			// Se era il primo nodo nella lista...
			first=post;
			post->Prev=NULL;
			numobject--;
		}
		
		if((post==NULL) && (pre!=NULL))
		{
			// Se era l'ultimo nodo nella lista....
			pre->Next=NULL;
			last=pre;
			numobject--;
		};
		
		if ((pre!=NULL) && (post!=NULL))
		{
			// Altrimenti se intermedio.....
			post->Prev=pre;
			pre->Next=post;
			numobject--;	
		};	
		
		pn1->ObjectPtr=pIntcorr;
		delete pn1;
		pn1=NULL; 
	}
	
	else
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaSillabe","DelNodo()");
};



void ListaSillabe::CancProlStart(Node *pncorr,Figura *pFStart,Figura *pFEnd,
								 Battuta *pBatCor)
{
	// Questo metodo viene richiamato nel //
	// caso in cui la figura da eliminare //
	// ha associata una sillaba che inizia//
	// su essa,di tipo estesa.            //
	// Permette di non perdere la sillaba //
	// aggiornando il puntatore alla fig. //
	// iniziale su quello successivo.     //
	
	
	Sillaba *pInt = (Sillaba*)(void*)pncorr->ObjectPtr;
	// Contiene la sillaba corrente associata
	// alla figura da eliminare.
	
	Battuta *pBatSuc=pBatCor;
	// Variabile di supporto che
	// memorizza la battuta successiva
	// a quella corrente.
	ListaFigure *pLayer;
	// Variabile di supporto che
	// memorizza la lista di figure
	// corrente.     				
	pLayer=GetPLayer(pBatCor);
	// Settaggio dell'idirizzo della
	// lista di figure sulla base della
	// battuta corrente.
	Figura *ptr=pLayer->GetFirstFig();
	
	
	while ((pFStart!=ptr) && (pBatSuc!=NULL))
	{
		ptr=pLayer->GetNextFig();
		if (ptr==NULL)
		{
			// Se la battuta  terminata la verifica deve continuare
			// nella battuta successiva.
			pBatSuc=(Battuta*)(listaBat->GetNext((DrawObject*)(void*)pBatSuc));
			// Settaggio della battuta successiva.
			pLayer=GetPLayer(pBatSuc);
			ptr=pLayer->GetFirstFig();	
		};
	};
	// Se il ciclo sopra  terminato con successo
	// ptr si trova nella lista delle figure correnti
	// in corrispondenza della figura iniziale a
	// cui  associata la sillaba.
	
	
	if (pBatSuc==NULL)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaSillabe","CancProlStart()");
	
	// Nel caso non fosse presente la battuta successiva,
	// si ritorna il messaggio di errore sopra.
	
	Bool Trovato=FALSE;
	// Variabile temporanea.
	
   	while((Trovato==FALSE) &&( pBatSuc!=NULL) && (ptr!=pFEnd))
	{
		
		ptr=pLayer->GetNextFig();
		if (ptr==NULL)
		{
			// Se la battuta  terminata la verifica deve continuare
			// nella battuta successiva.				
			pBatSuc=(Battuta*)(listaBat->GetNext((DrawObject*)(void*)pBatSuc));
			// Settaggio della battuta successiva.
			pLayer=GetPLayer(pBatSuc);
			ptr=pLayer->GetFirstFig();
			
		};
		if (ptr->GetID()!=CL_SPAZIO)	
			Trovato=TRUE;
		
	};
	
	if (pBatSuc==NULL)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaSillabe","CancProlStart()");
	// Nel caso non fosse presente la battuta successiva,
	// si ritorna il messaggio di errore sopra.
	// Altrimenti in ptr  memorizzata la figura
	// successiva.
	
	
	if(ptr==pFEnd)
	{	
		// Se puntatore aggiornato coincide  con
		// quello finale....
		
		
		if ( (!pInt->GetProlTrat())  ||  (pInt->GetProlTrat())  ) 
		{
			// Se la sillaba  prolungata con estensione
			// continua oppure  tratteggiata ma senza la
			// figura finale.....
			
			if (!pInt->GetProlTrat())
				// se la sillaba corrente ha prolungamento
				// continuo
				
				pInt->LinkTogetherTxt((char*)pInt->GetTxt(),pInt->GetNumtoken(),'_');
			
			else
	              	// ..altrimenti se tratteggiata senza sillaba
					// finale
					
					pInt->LinkTogetherTxt((char*)pInt->GetTxt(),pInt->GetNumtoken(),'-');
			// Aggiorna il testo aggiungendogli in
			// coda i token che la separavano dalla
			// figura finale.
			
			pInt->SetPFigStart(ptr);					
			// Setta il puntatore iniziale in base
			// a quello aggiornato.		
			pInt->ResetFirstLin();
			// Non essendoci il prolungamento non
			// ha ragione di esistere la lista dei
			// segmenti che lo compongono.
			pInt->SetProl(FALSE);
			// Poich la sillaba non  pi estesa.
			// l'attributo che memorizza ci deve
			// essere posto a False...
			pInt->SetProlTrat(FALSE);
			// ..come sopra per l'attributi che
			// specificano il tipo di tratteggio.
			// Singolo...
			pInt->SetSegTraSil(FALSE);	
			//...o meno.
			pInt->SetNumtoken(0);
			// Azzera il Numtoken
		};
		
		if ((pInt->GetProlTrat()) && (GetSilF(ptr,first/*,0*/)))
		{
			// Se ha estensione tratteggiata deve essere
			// fatta una fusione con la sillaba finale.
			
			Node *pnsucc=GetSilF(ptr,first/*,0*/);
			// Recupera la sillaba successiva  con cui
			// va unita quella corrente.
			Sillaba *pIntsucc = (Sillaba*)(void*)pnsucc->ObjectPtr;
			// Sillaba successiva a quella corrente,che
			// inizia sulla figura finale.
			
			// Aggiornamento testo sillaba successiva.
			pInt->LinkTogetherTxt((char*)pInt->GetTxt(),pInt->GetNumtoken() ,'-');
			// Aggiunge al testo della sillaba corrente i
			// token che la separavano dalla successiva.
			pIntsucc->LinkTogetherTxt((char*)pInt->GetTxt(),(char*)pIntsucc->GetTxt() ,' ');
			// Aggiunge al testo della sillaba successiva
			// quello della sillaba corrente modificato
			// dalmetodo sopra.
			
			//Eliminazione nodo dalla lista.
			DelNodo(pInt);
		}
	}
	else	
		
		pInt->SetPFigStart(ptr);					
	// Setta il puntatore iniziale in base
	// a quello aggiornato.		
	
};		



void ListaSillabe::CancProlEnd(Node *pncorr,Node* pnsucc,Figura *pFStart,Figura *pFEnd)

{
	
	Sillaba *pIntsucc ;
	// Variabile di supporto.	
	Sillaba *pInt = (Sillaba*)(void*)pncorr->ObjectPtr;
	// Contiene la sillaba il cui prolungamento
	// termina sulla figura da cancellare.
	
	if (pnsucc!=NULL)
		pIntsucc = (Sillaba*)(void*)pnsucc->ObjectPtr;
	// Contiene la sillaba successiva che inizia
	// eventualmente sulla figura da cancellare.
	
	Figura *prec=FindFigPrev(pFEnd,pFStart);
	// Prende la figura precedente a quella
	// finale della sillaba estesa.
	
	pInt->SetPFigEnd(prec);
	// Settaggio del nuovo puntatore
	// alla figura finale.
	
				
	if((prec!=pFStart) && (pInt->GetProlTrat()) && (pnsucc!=NULL))
	{		
		// Se la sillaba corrente  tratteggiata ,ha un'altra
		// sillaba associata alla figura finale,ed esiste almeno
		// un'altra figura tra quella finale e quella iniziale....
		
		// In ogni caso qualunque sia la natura della
		// sillaba successiva..
		pIntsucc->SetPFigStart(prec);
		// Associa alla sillaba  successiva singola
		// l'indirizzo della figura precedente a quella
		// cancellare.
		
		if((!pIntsucc->GetProl()))
			
			// Se la sillaba successiva a uella corrente,
			// che inizia sulla figura da cancellare  singola...
			pIntsucc->SetPFigEnd(prec);
		// ..poich singola anche l'indirizzo della figura
		// finale deve essere settato secondo il valore
		// della figura precedente.
		
	};
	
	
	if (prec==pFStart)
	{
		
		// Nel caso la sillaba estesa fosse
		// associata a solo due figure la
		// cancellazione di una determina
		// la trasformazione da sillaba
		// estesa a sillaba singola.
		
		
		if  (pnsucc!=NULL)
		{
			// Caso di sillaba con prolungamento
			// tratteggiato e sillaba finale.......
			
			// Aggiornamento testo sillaba successiva.
			pInt->LinkTogetherTxt((char*)pInt->GetTxt(),pInt->GetNumtoken() ,'-');
			// Aggiunge al testo della sillaba corrente i
			// token che la separavano dalla successiva.
			pInt->LinkTogetherTxt((char*)pInt->GetTxt(),(char*)pIntsucc->GetTxt() ,' ');
			// Aggiunge al testo della sillaba successiva
			// quello della sillaba corrente modificato
			// dalmetodo sopra.
			
			if (pIntsucc->GetProl())
			{
				// Se la sillaba successiva  estesa...
				
				pInt->SetPFigEnd(pIntsucc->GetPFigEnd());
				// La nuova sillaba data dall'unione
				// sar essa stessa singola per i
				// puntatori alla figura iniziale e
				// finale coincidono.
				pInt->SetProl(pIntsucc->GetProl());
				// Poich la sillaba non  pi estesa.
				// l'attributo che memorizza ci deve
				// essere posto a False...
				pInt->SetProlTrat(pIntsucc->GetProlTrat());
				// ..come sopra per l'attributi che
				// specificano il tipo di tratteggio.
				// Singolo...
				pInt->SetSegTraSil(pIntsucc->GetSegTraSil());	
				//...o meno.
				pInt->SetNumtoken(pIntsucc->GetNumtoken());
				// Setta il numero di token in base a quello
				// della variabile successiva
			}
			else
			{
				
				pInt->SetPFigEnd(pInt->GetPFigStart());
				// La sillaba da estesa deve diventare singola
				// con figura iniziale e finale coincidente.
				//pInt->ResetFirstLin();
				// Non essendoci il prolungamento non
				// ha ragione di esistere la lista dei
				// segmenti che lo compongono.
				pInt->SetProl(FALSE);
				// Poich la sillaba non  pi estesa.
				// l'attributo che memorizza ci deve
				// essere posto a False...
				pInt->SetProlTrat(FALSE);
				// ..come sopra per l'attributi che
				// specificano il tipo di tratteggio.
				// Singolo...
				pInt->SetSegTraSil(FALSE);	
				//...o meno.
				pInt->SetNumtoken(0);	
			};
			
			DelNodo(pIntsucc);	
		}
		else
		{
			// Caso di sillaba con prolungamento
			// continuo o tratteggiato ma senza
			// sillaba finale....
			
			// Aggiornamento testo della sillaba
			// corrente.
			pInt->LinkTogetherTxt((char*)pInt->GetTxt(),pInt->GetNumtoken(),'_');
			
			pInt->SetProl(FALSE);
			// Poich la sillaba non  pi estesa.
			// l'attributo che memorizza ci deve
			// essere posto a False...
			pInt->SetProlTrat(FALSE);
			// ..come sopra per l'attributi che
			// specificano il tipo di tratteggio.
			// Singolo...
			pInt->SetSegTraSil(FALSE);	
			//...o meno.
			pInt->SetNumtoken(0);
			
		};
	};					
	
};



void ListaSillabe::CancSingTrat(Sillaba *pcorr,Sillaba* psucc,int PrimaLettera)
{
	// Permette la creazione di una nuova   //
	// sillaba ottenuta dall'unione di due  //
	// appartenenti ad una stessa parola.   //
	// Viene utilizzata quando la figura da //
	// cancellare  associata ad una sillaba//
	// facente parte di una parola quindi   //
	// estesa con prolungamento costituito  //
	// da unico tratto.                     //
	
	if (PrimaLettera==0)
	{
		// Se non  la prima lettera di una parola....
		
		// Aggiornamento testo sillaba precedente.
		pcorr->LinkTogetherTxt((char*)pcorr->GetTxt(),(char*)psucc->GetTxt() ,'/');
		// Aggiunge al testo della sillaba successiva
		// quello della sillaba corrente modificato
		// dalmetodo sopra.
		
		if (psucc->GetProl())
		{
			// Se la sillaba successiva non  singola......
			pcorr->SetNumtoken(psucc->GetNumtoken());
			pcorr->SetPFigEnd(psucc->GetPFigEnd());
			// In questo caso la sillaba risultante
			// deve avere le stesse caratteristiche
			// della sillaba sottostante la figura
			// da cancellare,quindi il puntatore
			// alla sillaba finale deve coincidere
			// con il puntatore finale della sillaba
			// associata alla figura da cancellare.
		}
		else
			// ..altrimenti se  singola.........
			pcorr->SetPFigEnd(pcorr->GetPFigStart());
		// La nuova sillaba data dall'unione
		// sar essa stessa singola per i
		// puntatori alla figura iniziale e
		// finale coincidono.
		
		pcorr->SetProlTrat(psucc->GetProlTrat());
		// La nuova sillaba unione deve avere			
		// in termini di prolungamento le stesse
		// caratteristiche della sillaba che va
		// unita alla precedente.
		pcorr->SetSegTraSil(psucc->GetSegTraSil());	
		//...per lo stesso motivo di sopra.	
		pcorr->SetProl(psucc->GetProl());	
		//...per lo stesso motivo di sopra.
		
		// Eliminazione sillaba successiva.
		DelNodo(psucc);
	}
	else
	{
		// ..altrimenti deve essere settato il testo della
		// sillaba successiva a cui quella corrente viene
		// attaccata...
		
		// Aggiornamento testo sillaba successiva.
		psucc->LinkTogetherTxt((char*)pcorr->GetTxt(),(char*)psucc->GetTxt() ,'/');
		// Aggiunge al testo della sillaba successiva
		// quello della sillaba corrente modificato
		// dalmetodo sopra.
		
		// ..mentre gli attributi non devono
		// essere aggiornati,rimangono validi quelli
		// della sillaba successiva a cui quella
		// corrente  stata attaccata...
		
		// Eliminazione nodo.
		DelNodo(pcorr);			
		// ..e la sillaba da cancellare 
		// proprio quella corrente.	
	};
};



void ListaSillabe::CancSilFig(Figura *pF)
{
	Sillaba *pInt;
	Node *pn1,*pn2,*pncorr;
	Figura *pFEnd,*pFStart;
	Battuta *pBatCor;
	
	
	pn1 = GetIntEndF(pF,first);
	// Verifica se esiste una sillaba la cui
	// estensione termina sulla figura pF.
	pn2 = GetSilF(pF,first/*,0*/);
	// Verifica se esiste una sillaba
	// associata alla figura pF,nel
	// senso che la prima figura ad
	// essa associata  proprio pF.
	
	
	if((pn1!=NULL) || (pn2!=NULL))
	{
		// Se esiste una sillaba estesa o meno
		// associata alla figura pF,si procede
		// alla sua cancellazione.
		
		if (pn1!=NULL)
			
			pncorr = pn1;
		
		else
			pncorr = pn2;
		
		// In pncorr  cos memorizzato il nodo relativo
		// alla sillabe che o termina(pn1<>null) o
		// inizia(pn2<>null) sulla figura pF.
		
		pInt = (Sillaba*)(void*)pncorr->ObjectPtr;
		// In pInt viene memorizzato l'indirizzo
		// della sillaba corrispondente al nodo
		// pn sopra individuato.
		
		
		pFStart=pInt->GetPFigStart();
		// Ritorna la figura iniziale a cui
		//  associata la sillaba.
		
		pBatCor=listaBat->GetBattWithFig(pFStart);
		// Permette di ottenere la battuta
		// corrente.
		
		pFEnd=pInt->GetPFigEnd();
		// Ritorna la figura finale a cui
		//  associata la sillaba.		
		
		if (pn1!=NULL) 			
		{
			// Se vale la condizione sopra significa che
			// la figura da cancellare  associata ad una
			// sillaba estesa con prolungamento generico
			// che finisce su essa.
			
			if  (GetSilF(pFEnd,first/*,0*/))
				// Se anche questa condizione  verificata
				// significa,inoltre, che associata alla
				// figura pF oltre alla sillaba pInt 
				// associata un'altra sillaba che ha come
				// figura iniziale proprio pF.
				// (La sillaba corrente pInt  estesa dunque
				// del tipo tratteggiato  oppure fa parte di
				// una parola,singola lineetta).
			{
				
				
				Node * pnsucc=pncorr->Next;
				// Contiene il nodo della lista
				// associato alla sillaba successiva
				// a quella corrente che terminava
				// il prolungamento sulla figura
				// in esame.
				Sillaba *pIntSucc=(Sillaba*)(void*)pnsucc->ObjectPtr;
				// Sillaba relativa al nodo successivo
				// a quello corrente.
				if (pInt->GetSegTraSil())
				{
					// La sillaba corrente  del tipo
					// estesa con prolungamento fatto
					// da un unico tratto.
					CancSingTrat(pInt,pIntSucc,0);
					//return;
				}			
				else
				{
					// In questo caso la sillaba corrente 
					// estesa con prolungamento tratteggiato.
					CancProlEnd(pncorr,pnsucc,pFStart,pFEnd);		
					//return;       				     				
					// Ritorna al chiamante.
				};						
			}
			
			else
				// E' il caso di sillaba estesa con prolungamento
				// continuo che termina sulla figura da cancellare,
				// oppure sillaba con prolungamento tratteggiato ma
				// senza la sillaba finale.
				CancProlEnd(pncorr,NULL,pFStart,pFEnd);
		}
		
		
		
		else
		{
			// ..altrimenti significa che associata alla
			// alla figura da cancellare c' una sillaba
			// singola o estesa il cui prolungamento inizia
			// su essa.
			
			
			Node * pnprec=pncorr->Prev;
			// Memorizza l'eventuale sillaba
			// precedente a quella corrente.
			Node * pnsucc=pncorr->Next;
			// Memorizza l'eventuale sillaban
			// successiva a quella corrente.
			Node *pntemp=NULL;
			
			if (pnprec!=NULL)
				pntemp=pnprec;
			else
				pntemp=pnsucc;
			// Se esiste una sillaba precedente
			// o successiva a quella in esame nella
			// variabile temporanea pntemp si mette
			// il suo indirizzo.
			
			Sillaba *pIntemp;
			// Puntatore alla sillaba precedente o successiva
			// a quella corrente nella lista.
			
			
			if (pntemp!=NULL)   	
			{
				// Nel caso la sillaba non  sia l'unica
				// nella lista  in pntemp  contenuto
				// l'indirizzo della sillaba precedente
				// o successiva
				
				
				pIntemp=(Sillaba*)(void*)pntemp->ObjectPtr;	
				// Sillaba temporanea precedente o successiva
				// a quella corrente.
			};
			Figura *prev=NULL;
			// Variabile di supporto contenente la
			// figura precedente a quella da eliminare.
			prev=FindFigPrev(pInt->GetPFigStart(),NULL);
			// Poich la figura da eliminare  la prima
			// associata alla sillaba la scansione deve
			// iniziare dalla prima battuta della lista
			// per cui viene passato NULL.
			if (prev!=NULL)
			{
				// Se  presente una figura precedente..
				
				if (((pnprec!=NULL) && ((pIntemp->GetPFigEnd())!= prev)) || (pnprec==NULL))
				{
					// Se nella lista  presente un'altra sillaba che
					// precede quella corrente ma la cui figura finale
					// non coincide con quella precedente appena calcolata,
					// oppure non  presente alcuna sillaba precedente...
					
					pInt->SetPFigStart(prev);	
					// Si sposta la sillaba associata
					// alla figura da cancellare sulla
					// figura a lei precedente.
					prev->SetAppSil((InterLine-1),1); 
					// Marcatura della figura precedente
					// come occupata.
					if  (!pInt->GetProl())
						// Se singola, poiche anche se traslata
						// deve mantenere le stesse caratteristiche
						// tipologiche.....
						pInt->SetPFigEnd(prev);
					// Si aggiorna anche il puntatore
					// finale in base alla figura precedente.		
					return;		
					
				};
				if ((pnprec!=NULL) && ((pIntemp->GetPFigEnd())== prev))
				{
					// Se  invece presente una sillaba precedente
					// la cui figura finale coincide con quella
					// appena calcolata...
					
					if(!pIntemp->GetProl())
					{
						// Si unisce il testo della sillaba da cancellare
						// a quello della sillaba precedente .....
						
						// Aggiornamento testo sillaba precedente.
						pIntemp->LinkTogetherTxt((char *)pIntemp->GetTxt(),(char *)pInt->GetTxt() ,' ');
						
						// Aggiornamento attributi tipologici.
						if (pInt->GetProl())
							// Se la sillaba da eliminare  estesa...
							pIntemp->SetPFigEnd(pInt->GetPFigEnd());
						//... setta il puntatore alla figura finale.
						// della silaba precedente in base al suo.
						
						pIntemp->SetProl(pInt->GetProl());
						// Aggiornamento prolungamento generale.
						pIntemp->SetProlTrat(pInt->GetProlTrat());
						// Aggiornamento tratto del prolungamento.
						pIntemp->SetSegTraSil(pInt->GetSegTraSil());
						// Aggiornamento caso di semplice separatore
						// tra sillabe.					
						pIntemp->SetNumtoken(pInt->GetNumtoken());
						
						//Cancellazione.
						DelNodo(pInt);
						return;
						// Ritorna al chiamante.
					}
					
					else
					{
						// Se la sillaba precedente non  singola
						// la sua estensione va spostata alla
						// figura precedente.
						
						CancProlEnd(pnprec,NULL,pIntemp->GetPFigStart(),pIntemp->GetPFigEnd());
						// Si accorcia il prolungamento sulla
						// figura precedente.
						
						// Si sposta la sillaba corrente sotto la figura
						// finale della sillaba precedente.
						pInt->SetPFigStart(prev);	
						// Si sposta la sillaba associata
						// alla figura da cancellare sulla
						// figura a lei precedente.
						if  (!pInt->GetProl())
							// Se singola, poiche anche se traslata
							// deve mantenere le stesse caratteristiche
							// tipologiche.....
							pInt->SetPFigEnd(prev);
						// Si aggiorna anche il puntatore
						// finale in base alla figura precedente.      		        		
					};					
					
					
				};
				
			}
			else
			{
				// Se la figura precente a quella da eliminare
				//  nulla.......
				
				if (!pInt->GetProl())
				{
					// Se  singola si valuta l'esistenza di una
					// figura successiva a quella da eliminare.......
					
					Figura *succ=NULL;
					// Variabile temporanea contenete la figura
					// successiva a quella da eliminare.
					
					Battuta *pBatCor=listaBat->GetBattWithFig(pFStart);
					// Contiene la battuta a cui appartiene la
					// figura iniziale da cui incominciare la
					// la scansione .
					Battuta *pBatSuc=pBatCor;
					// Variabile di supporto che
					// memorizza la battuta successiva
					// a quella corrente.
					ListaFigure *pLayer;
					// Variabile di supporto che
					// memorizza la lista di figure
					// corrente.     				
					pLayer=GetPLayer(pBatCor);
					// Settaggio dell'idirizzo della
					// lista di figure sulla base della
					// battuta corrente.
					Figura *ptr=pLayer->GetFirstFig();
					
					
					while ((pFStart!=ptr) && (pBatSuc!=NULL))
					{
						ptr=pLayer->GetNextFig();
						if (ptr==NULL)
						{
							// Se la battuta  terminata la verifica deve continuare
							// nella battuta successiva.
							pBatSuc=(Battuta*)(listaBat->GetNext((DrawObject*)(void*)pBatSuc));
							// Settaggio della battuta successiva.
							if(pBatSuc!=NULL)
							{
								pLayer=GetPLayer(pBatSuc);
								ptr=pLayer->GetFirstFig();	
							}
						};
					};
					// Se il ciclo sopra  terminato con successo
					// ptr si trova nella lista delle figure correnti
					// in corrispondenza della figura iniziale a
					// cui  associata la sillaba.
					
					
					if (pBatSuc==NULL)
						// Nel caso non fosse presente la battuta successiva..
						EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaSillabe","CancSilFig()");
					//  ..si ritorna il messaggio di errore sopra.
					
					Bool Trovato=FALSE;
					// Variabile temporanea.
					
					while((Trovato==FALSE) &&( pBatSuc!=NULL))
					{
						
						ptr=pLayer->GetNextFig();
						if (ptr==NULL)
						{
							// Se la battuta  terminata la verifica deve continuare
							// nella battuta successiva.				
							pBatSuc=(Battuta*)(listaBat->GetNext((DrawObject*)(void*)pBatSuc));
							// Settaggio della battuta successiva.
							if(pBatSuc!=NULL)
							{
								pLayer=GetPLayer(pBatSuc);
								ptr=pLayer->GetFirstFig();
							}
							
						};
						if (ptr!=NULL && ptr->GetID()!=CL_SPAZIO)	
						{
							Trovato=TRUE;
							succ=ptr;
						};			
						
					};
					
					
					if (succ!=NULL)
					{
						// Se  presente una figura successiva...
						
						if ((pnsucc==NULL) || ((pnsucc!=NULL) && (pIntemp->GetPFigStart()!=succ)))
						{
							// Se  presente una figura successiva e a questa
							// non  associata alcuna sillaba....
							
							pInt->SetPFigStart(succ);	
							// Si sposta la sillaba associata
							// alla figura da cancellare sulla
							// figura a lei precedente.
							succ->SetAppSil((InterLine-1),1);//inutilizzato
							// Marcatura della figura precedente
							// come occupata.
							pInt->SetPFigEnd(succ);
							// Si aggiorna anche il puntatore
							// finale in base alla figura precedente.
							return;	
						};
						if ((pnsucc!=NULL) && (pIntemp->GetPFigStart()==succ))
						{
							// Se alla figura successiva  associta una sillaba...
							
							pIntemp->LinkTogetherTxt((char *)pInt->GetTxt(),(char *)pIntemp->GetTxt() ,' ');
							// Si congiunge il testo con la successiva....
							DelNodo(pInt);
							// Si elimina dalla lista.
							return;
						};		
					};
					if((prev==NULL) && (succ==NULL))
						// Se non  presente n una figura precedente
						// n una successiva...
						DelNodo(pInt);
					// Si elimina dalla lista.
					return;
					// Ritorna al chiamante.       		     		
					
					
				};
				
				if(pInt->GetSegTraSil())
				{
					// Caso in cui la siilaba associata alla figura da
					// cancellare sia la prima di una parola sillabata
					// tramite separatori(unico trattino).
					CancSingTrat(pInt,pIntemp,1);
				                return;
								// Ritorna al chiamante.
				};		
				
				if  ( (pInt->GetProl())
					|| ( (pInt->GetProlTrat())
					&&(!pInt->GetSegTraSil())))
					// Nel caso in cui sia prolungata con prolungamento
					// sia continuo che tratteggiato,si richiama ......			
					CancProlStart(pncorr,pFStart,pFEnd,pBatCor);	
				
				
			  };			
			  
		};
		
	};
	
};


void ListaSillabe::CancSilBat(Battuta *pB)
{
	Sillaba *pInt;
	Node *pNext;
	Node *pn = first;
	Figura *pFStart,*pFEnd;
	
	
	while(pn!=NULL)
	{
		pNext = pn->Next;
		pInt = (Sillaba*)(void *)pn->ObjectPtr;
		// La variabile pInt prende l'indirizzo
		// della sillaba associata al nodo pn
		// corrente.
		if ((pInt->GetBattuta(1)==pB) || (pInt->GetBattuta(2)==pB))
			
		{       // Se esiste una sillaba  associatata
			// ad una nota della battuta da eliminare,
			//  necessario aggiornarla se possibile.
			
			pFStart=pInt->GetPFigStart();
			// Ritorna la figura iniziale a cui
			//  associata la sillaba.
			pFEnd=pInt->GetPFigEnd();
			// Ritorna la figura finale a cui
			//  associata la sillaba.
			
			
			if ((pInt->GetBattuta(2)==pB) &&  (pInt->GetBattuta(1)!=pB))
				
				// Se vale la condizione sopra significa che
				// la figura da cancellare  associata ad una
				// sillaba estesa che termina nella battuta
				// corrente.
				CancSilFig(pFEnd);
			
			else
				// Se vale la condizione sopra significa che
				// la figura da cancellare  associata ad una
				// sillaba estesa o singola che inizia nella
				//  battuta corrente.
				CancSilFig(pFStart);
			
			
		};
		pn = pNext;
	};
	
}; 	





// SETTAGGIO //
// Queste funzioni permettono di //
// aggiornare il valore numerico //
// che individua ogni sillaba in //
// LISTASILLABE.                 //


void ListaSillabe::SetCodeCounter(NumCode nc)
{
	CodeCounter=nc;
	
};

NumCode ListaSillabe::GetCodeCounter(void)
{
	return CodeCounter;
};


// Questo metodo dato il codice  //
// numerico ritorna l'indirizzo  //
// della sillaba corrispondente. //


Sillaba *ListaSillabe::GetNumCodeSil(NumCode nc)
{
	Sillaba *pInt;
	Node *pn = first;
	
	while(pn!=NULL)
	{
		pInt = (Sillaba*)(void*)(pn->ObjectPtr);
		if ((pInt->GetNumCode())==nc)
			return pInt;
		pn = pn->Next;
	};
	
	return NULL;
};
// Permette di disegnare le
// linee che compongono il
// il prolungamento.


void ListaSillabe::Draw()
{
	Node *pn = first;
	Node *pn1;
	VUnit a;
	Sillaba *pInt = NULL;
	
	while(pn!=NULL)
	{
		pInt=((Sillaba*)(void *)pn->ObjectPtr);
		int ns=pInt->GetLine();
		
		// Se si riferisce ad una sillaba gi esaminata
		// da Spartito per la costruzione del rigo allora
		// pu essere disegnata.
		
		if (!pInt->GetProl())
		{
			// Se la sillaba  non estesa si disegna
			// solo il suo testo sullo schermo.
			if(pInt->GetYSil()>0)
				pInt->DrawText(ns);
		}
		else if(pInt->GetPFirstLin()!=NULL)
			// Altrimenti se la sillaba  estesa...
		{						
			pn1=pn->Next;
			// Puntatore alla sillaba successiva.
			if ((pn1!=NULL) && (GetSilF(pInt->GetPFigEnd(),first)))
				// Se la sillaba successiva non  nulla
				// ed  effettivamente quella alla fine
				// del prolungamento,associata alla stessa
				// figura finale di quella corrente....
				
				a=(((Sillaba*)(void *)pn1->ObjectPtr))->GetLeft();
			// In questo caso il discostamento per
			// calcolare la larghezza tra le due
			// sillabe, dato dalla distanza tra
			// il punto centrale del testo e il
			// suo estremo sinistro.
			
			else
			{     	
				// Altrimenti il discostamento coincide
				// con la distanza tra il punto centrale
				// del rettangolo contenente la figura
				// e il suo estremo sinistro meno la
				// minima distanza tra due figure.
				
				Figura *pf=pInt->GetPFigEnd();
				// Figura finale nuova a cui 
				// associata la sillaba.
				a=-(pf->GetVU2Rgt()); 
				
			};
			
			pInt->DrawSilProl(a,ns-1);
			// Disegna la sillaba.
		};
		pn = pn->Next;
		// Passa alla sillaba successiva da
		// disegnare.
	};
};

//////////////////////////////////////////////////////////////////////////////////////////////////
//				       FINE          						//
//////////////////////////////////////////////////////////////////////////////////////////////////

extern wxString XMLFormatString(const wxString& s);


int ListaSillabe::SalvaLyr(SMRLyricsItem *lwf,char *FileName)
{
        // Variabili di supporto che permettono la
	// scansione della lista.
#ifdef _LOAD_XML_
	Node *pn = first;
	Sillaba *pSyl;
	
	if (first==NULL)
	{
	// In questo caso la lista  vuota cio alcuna
	// lirica  associata allo spartito per cui
	// non viene salvato niente...
			printf("NON E' PRESENTE ALCUNA LIRICA \n");
			return 0;
			//..e ritorna
	};

	wxString lyrName;

	if(FileName==NULL)
	{
		if(lwf->GetFilename()!="")
			lyrName=lwf->GetSMRFile()->GetPath()+PATH_SEP+lwf->GetFilename();
		else
		{
			int index =0;
			wxString basename=lwf->GetSMRFile()->GetPath()+PATH_SEP+wxString::Format("%d.lyric",lwf->GetPartID());
			do
			{
				index++; 
				lyrName=basename+wxString::Format(".%d.xml",index );	
			}
			while(wxFileExists(lyrName));
			lwf->SetFilename(lyrName.AfterLast(PATH_SEP[0]));
		}
	}
	else
		lyrName=FileName;
	
    ofstream xml(lyrName, ios::out);
	xml << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
#ifdef SAVE_DTD
	xml << "<!DOCTYPE LWF SYSTEM \"" DTD_PATH "lwf.dtd\">" << endl;
#endif
	xml << "<SMXF_Lyric xmlns=\"urn:mpeg:mpeg-4:schema:smr:smxf-lyric:2007\" SCOREID=\"" << lwf->GetPartID() <<"\"";
	if(BaseLineFactor!=1.)
		xml<< " BASELINE=\"" << BaseLineFactor <<"\"";
	if(LinesDistFactor!=1.)
		xml<< " LINESDIST=\"" << LinesDistFactor <<"\"";
	xml << ">"<< endl;
	xml << "\t<ID>" << lwf->GetID() <<"</ID>" << endl;
	xml << "\t<language>"<< lwf->GetLanguage() << "</language>" << endl;

	pn=first;
    while(pn!=NULL)
	{
		pSyl=((Sillaba*)(void *)pn->ObjectPtr);
		
		if(pSyl->GetPFigStart()==NULL && pSyl->GetPFigEnd()==NULL)
		{
			xml << "\t<text>" << XMLFormatString(pSyl->GetTxt()) << "</text>\n";
		}
		else if(pSyl->GetPFigStart()!=NULL && (!pSyl->GetProl() || pSyl->GetPFigEnd()!=NULL))
		{
			xml << "\t<syllable ";
			
			xml << "LINE=\"" << pSyl->GetLine() << "\" ";
			if(pSyl->Getgs()!=' ')
				xml << "REFRAIN=\"" << pSyl->Getgs() << "\" ";

			xml << "SEP=\"";
			int ns=pSyl->GetNumSep();
			if(ns<=0) ns=1;
			for(int i=0; i< ns; i++)
			{
				switch(pSyl->GetToken())
				{
				case '\n':
					xml << 'n';
					break;
				case '\t':
					xml << 't';
					break;
				case 'e':
					break;
				default:
					xml << pSyl->GetToken();
					break;
				}
			}
			xml << "\">\n";

			xml << "\t\t<text>" << XMLFormatString(pSyl->GetTxt()) << "</text>\n";
						
			SymPath sp1,sp2;	
			// Variabile di supporto per la memorizzazione
			// del path che specifica l'indirizzo logico
			// di ogni nota.
			
			// Attraverso l'istruzione successiva si preleva,
			// se presente,l'indirizzo simbolico della prima
			// figura a cui  associata la sillaba.
			
			
			Battuta *pb1=listaBat->GetBattWithFig(pSyl->GetPFigStart());
			// Battuta a cui appartiene la prima figura a
			// cui  associata la SILLABA.
			
			if(!pb1->FigPtr2SymPath(sp1,pSyl->GetPFigStart()))
			{
				printf("Errore in SalvaLyr,mancanza prima figura\n");
				return 1;
			};
			
			xml << "\t\t<start ";
			xml << "MEASURE=\"" << sp1.Array[2] << "\" ";
			xml << "LAYER=\"" << sp1.Array[3] << "\" ";
			xml << "FIGURE=\"" << sp1.Array[4] << "\"";
			if(sp1.Array[5]!=0)
			{
				xml << " CHORD.OR.BEAM=\"" << sp1.Array[5] << "\"";
				if(sp1.Array[6]!=0)
					xml << " CHORD.IN.BEAM=\"" << sp1.Array[6] << "\"";
			}
			xml << "/>\n";
			
			if (pSyl->GetProl())
				// Se la sillaba  estesa gli sono associate due
				// figure e di conseguenza due indirizzi,per cui
				// un 'altro attributo ADDRESS deve essere aggiunto.			
			{
				// Attraverso l'istruzione successiva si preleva,
				// se presente,l'indirizzo simbolico della seconda
				// figura a cui  associata la sillaba
				
				
				Battuta *pb2=listaBat->GetBattWithFig(pSyl->GetPFigEnd());
				// Battuta a cui appartiene la seconda figura a
				// cui  associata la SILLABA.
				
				if(!pb2->FigPtr2SymPath(sp2,pSyl->GetPFigEnd()))
				{
					printf("Errore in SalvaLyr,mancanza seconda figura\n");
					return 1;
				}
				
				xml << "\t\t<end ";
				xml << "MEASURE=\"" << sp2.Array[2] << "\" ";
				xml << "LAYER=\"" << sp2.Array[3] << "\" ";
				xml << "FIGURE=\"" << sp2.Array[4] << "\" ";
				if(sp2.Array[5]!=0)
				{
					xml << " CHORD.OR.BEAM=\"" << sp2.Array[5] << "\"";
					if(sp2.Array[6]!=0)
						xml << " CHORD.IN.BEAM=\"" << sp2.Array[6] << "\"";
				}
				xml << "/>\n";
			}
			xml << "\t</syllable>\n";
		}
       	pn = pn->Next;
	}
	xml << "</SMXF_Lyric>" << endl;
	xml.close();
	lwf->SetIsModified(false);
#endif
    	return 0;
};

static wxString getNodeValue(const DOM_Node& node)
{
	if(node.isNull())
		return "";
	return node.getNodeValue().transcode();
}

int ListaSillabe::LoadLyr(char *Name, InputSource *src, int validScoreId)
{
	
	//    Questo metodo si preoccupa di verificare     //
	//     l'esistenza di una lirica associata allo    //
	//  spartito corrente ed eventualmente richiamare  //
	//   il metodo di ListaSillabe adatto allo scopo.  //
	
#ifdef _LOAD_XML_
	
	WDFDOMParser parser;
	// Si crea un oggetto di tipo Parser.
	
	
	// Fare il parser del file significa leggere il
	// file XML presente in memoria,interpretarlo e
	// creare la struttura DOM,ad albero,da cui poter
	// prelevare le informazioni di interesse.
	
	
	
	if(Name!=NULL)
		parser.setXMLFile(Name);
	else if(src!=NULL)
		parser.setInputSource(src);
	else
		return 0;

	parser.setValidationSchema("urn:mpeg:mpeg-4:schema:smr:smxf-lyric:2007 smxf_lyric.xsd");

    // Verifica se  possibile effettuareil PARSE.
	try
	{
		parser.Parse();
		// Effettua il parsing,eventualmente dando
		// errore.
	}		
	catch (const XMLException&)
	{
        return 0;			
		// Se il parse  stato eseguito non correttamente
		// viene stampato il messaggio sopra e si ritorna
		// al chiamant
	}
	
	
	extern wxDC *currDC;
	currDC=NULL;
	
	Init();
	// Inizializzazione,della lista di
	// sillabe che conterra quelle relative
	// allo spartito corrente.
	
	DOM_Document doc;
	// Questa variabile di tipo nodo permette
	// la navigazione all'interno della struttura
	// ad albero nella quale  memorizzato il docu=
	// mento.
	
	doc = parser.GetWDFParser()->getDocument ();
	// Il metodo getDocument ritorna uno
	// oggetto DOM_Document.
	
	DOM_Node wdfheader=doc.getFirstChild();
	if(wdfheader.isNull())
		return 0;
	DOM_Node child = wdfheader.getFirstChild();
	if(child==0) // Se c'e' il DOCTYPE lo si salta
	{
		wdfheader=wdfheader.getNextSibling();
		child = wdfheader.getFirstChild();
	}
	
	DOM_NamedNodeMap lwfattr=wdfheader.getAttributes();
	DOM_Node scoreId=lwfattr.getNamedItem("SCOREID");
	int id=0;
	if(scoreId!=0)
	{
		id=atoi(scoreId.getNodeValue().transcode());
		if(validScoreId!=0 && id!=validScoreId)
		{
			wxMessageBox("ERROR: Lyric not for the score");
			return 0;
		}
	}
	DOM_Node baseLine=lwfattr.getNamedItem("BASELINE");
	double bl=1.,ld=1.;
	if(baseLine!=0)
	{
		bl=atof(baseLine.getNodeValue().transcode());
		SetBaseLineFactor(bl);
	}
	DOM_Node linesDist=lwfattr.getNamedItem("LINESDIST");
	if(linesDist!=0)
	{
		ld=atof(linesDist.getNodeValue().transcode());
		SetLinesDistFactor(ld);
	}
	
	while( child != 0)
	{
		if(child.getLocalName().equals("ID"))
		{
		}
		else if(child.getLocalName().equals("WDFID"))
		{
		}
		else if(child.getLocalName().equals("Dimension"))
		{
		}
		else if(child.getLocalName().equals("Checksum"))
		{
		}
		else if(child.getLocalName().equals("language") || child.getLocalName().equals("Language"))
		{
		}
		if(child.getLocalName().equals("Element"))
		{
			DOM_NamedNodeMap attr=child.getAttributes();
			DOM_Node line=attr.getNamedItem("SCORE.LINE");
			DOM_Node token=attr.getNamedItem("TOKEN");
			DOM_Node refrain=attr.getNamedItem("REFRAIN");
			
			int iline=atoi(getNodeValue(line));
			wxString tk=getNodeValue(token);
			char *rf=" ";
			if(!refrain.isNull())
				rf=refrain.getNodeValue().transcode();
			char *syl=NULL;
			int addr=0;
			Figura *pf[2]={NULL,NULL};
			Battuta *pb[2]={NULL,NULL};
			int sylLayer=0;
			bool start=false;
			
			DOM_Node gchild=child.getFirstChild();
			while(gchild!=0)
			{
				if(gchild.getLocalName().equals("Syllable") && gchild.getFirstChild()!=0)
				{
					syl=gchild.getFirstChild().getNodeValue().transcode();
				}
				else if(gchild.getLocalName().equals("Address") && addr<2)
				{
					DOM_NamedNodeMap attr=gchild.getAttributes();
					int measure=atoi(getNodeValue(attr.getNamedItem("MEASURE")));
					int layer=atoi(getNodeValue(attr.getNamedItem("LAYER")));
					int figure=atoi(getNodeValue(attr.getNamedItem("FIGURE")));
					int chordOrBeam=atoi(getNodeValue(attr.getNamedItem("CHORD.OR.BEAM")));
					int chordInBeam=atoi(getNodeValue(attr.getNamedItem("CHORD.IN.BEAM")));
					
					SymPath sp;
					sp.Array[2]=measure;
					sp.Array[3]=layer;
					sp.Array[4]=figure;
					sp.Array[5]=chordOrBeam;
					sp.Array[6]=chordInBeam;
					if(listaBat!=NULL)
					{
						pf[addr]=listaBat->Symbolic(sp);
						pb[addr]=listaBat->GetBattWithFig(pf[addr]);
					}
					if(addr==0)
						start=true;
					addr++;
					
					sylLayer=layer-1;
				}
				gchild = gchild.getNextSibling();
			}
			char t=tk[(unsigned int)0];
			if(start && syl!=NULL)
			{
				Bool Trat=FALSE,TraSil=FALSE;
				// Variabili di supporto per l'identificazione
				// della tipologia della sillaba.
				
				if(t=='\0')
					tk="e";
				
				if (t=='/')
					// Caso di sillabe separate da semplice trattino.
				{
					Trat=TRUE;
					TraSil=TRUE;
				}
				
				if (t=='-' || t=='@')
				{
					// Caso di sillaba estesa con prolungamento di tipo
					// tratteggiato.
					
					Trat=TRUE;
					TraSil=FALSE;
				};
				
				if ((t=='_') || (t=='0') || (t==' ') || (t=='e'))
				{
					// Caso di sillaba estesa con prolungamento a tratto
					// continuo o sillaba singola.
					
					Trat=FALSE;
					TraSil=FALSE;
				};	
				
				if(pf[1]==NULL)
				{
					pf[1]=pf[0];
					pb[1]=pb[0];
				}
				
				SetLayer(1);
				// Setta il layer,sulla base della sola figura
				// iniziale presupponendo che nel caso di sillaba
				// estesa il layer delle due figure coincidono.
				//InsertSil(pf[0],pf[1],syl,tk[0],Trat,TraSil);	
				int nsep=strlen(tk);
				int dur=nsep;
				if(t=='_')
					dur--;
				else if(t=='@')
					dur++;
				
				Add(syl,pf[0],pf[1],pb[0],pb[1],dur,Trat,TraSil,t,nsep,iline,rf[(unsigned int)0],sylLayer,TRUE,ld,bl);
			}
			else
				Add("",NULL,NULL,NULL,NULL,0,FALSE,FALSE,t,strlen(tk),iline,rf[(unsigned int)0],sylLayer,FALSE,ld,bl);
			
		}
		if(child.getLocalName().equals("syllable"))
		{
			DOM_NamedNodeMap attr=child.getAttributes();
			DOM_Node line=attr.getNamedItem("LINE");
			DOM_Node sep=attr.getNamedItem("SEP");
			DOM_Node refrain=attr.getNamedItem("REFRAIN");
			
			int iline=atoi(getNodeValue(line));
			wxString sp=getNodeValue(sep);
			char *rf=" ";
			if(!refrain.isNull())
				rf=refrain.getNodeValue().transcode();
			wxString syl;
			int addr=0;
			Figura *pf[2]={NULL,NULL};
			Battuta *pb[2]={NULL,NULL};
			int sylLayer=0;
			bool start=false;
			bool end=false;
			
			DOM_Node gchild=child.getFirstChild();
			while(gchild!=0)
			{
				if(gchild.getLocalName().equals("text") && gchild.getFirstChild()!=0)
				{
					DOMString s=gchild.getFirstChild().getNodeValue();
					StrX xsyl(s.rawBuffer(),s.length());

					syl=xsyl.localForm();
				}
				else if(gchild.getLocalName().equals("start") || gchild.getLocalName().equals("end"))
				{
					DOM_NamedNodeMap attr=gchild.getAttributes();
					int measure=atoi(getNodeValue(attr.getNamedItem("MEASURE")));
					int layer=atoi(getNodeValue(attr.getNamedItem("LAYER")));
					int figure=atoi(getNodeValue(attr.getNamedItem("FIGURE")));
					int chordOrBeam=atoi(getNodeValue(attr.getNamedItem("CHORD.OR.BEAM")));
					int chordInBeam=atoi(getNodeValue(attr.getNamedItem("CHORD.IN.BEAM")));
					
					SymPath sp;
					sp.Array[2]=measure;
					sp.Array[3]=layer;
					sp.Array[4]=figure;
					sp.Array[5]=chordOrBeam;
					sp.Array[6]=chordInBeam;
					if(gchild.getLocalName().equals("start"))
					{
						addr=0;
						start=true;
					}
					else
					{
						addr=1;
						end=true;
					}
					if(listaBat!=NULL)
					{
						pf[addr]=listaBat->Symbolic(sp);
						pb[addr]=listaBat->GetBattWithFig(pf[addr]);
					}
					
					sylLayer=layer-1;
				}
				gchild = gchild.getNextSibling();
			}
			if(start && syl!="")
			{
				Bool Trat=FALSE,TraSil=FALSE;
				// Variabili di supporto per l'identificazione
				// della tipologia della sillaba.
				
				if(sp=="")
					sp="e";
				
				switch(sp[(unsigned int)0])
				{
				case '/':
					// Caso di sillabe separate da semplice trattino.
					Trat=TRUE;
					TraSil=TRUE;
					break;
				case '-':
				case '@':
					// Caso di sillaba estesa con prolungamento di tipo
					// tratteggiato.					
					Trat=TRUE;
					TraSil=FALSE;
					break;
				case 'n':
					sp.SetChar(0,'\n');
					break;
				case 't':
					sp.SetChar(0,'\t');
					break;
				}
				
				if(pf[1]==NULL)
				{
					pf[1]=pf[0];
					pb[1]=pb[0];
				}
				
				SetLayer(1);
				// Setta il layer,sulla base della sola figura
				// iniziale presupponendo che nel caso di sillaba
				// estesa il layer delle due figure coincidono.
				//InsertSil(pf[0],pf[1],syl,tk[0],Trat,TraSil);	
				int nsep=sp.Length();
				int dur=nsep;
				if(sp[unsigned int(0)]=='_')
					dur--;
				else if(sp[unsigned int(0)]=='@')
					dur++;
				
				Add(syl,pf[0],pf[1],pb[0],pb[1],dur,Trat,TraSil,sp[unsigned int(0)],nsep,iline,rf[unsigned int(0)],sylLayer,TRUE,ld,bl);
			}
			else
				Add("",NULL,NULL,NULL,NULL,0,FALSE,FALSE,sp[unsigned int(0)],strlen(sp),iline,rf[unsigned int(0)],sylLayer,FALSE,ld,bl);
			
		}
		else if(child.getLocalName().equals("text") && child.getFirstChild()!=0)
		{
			char *syl=child.getFirstChild().getNodeValue().transcode();
			Add(syl,NULL,NULL,NULL,NULL,0,FALSE,FALSE,'\0',0,0,' ',0,FALSE,ld,bl);
		}
		child = child.getNextSibling();
	}
#endif	
	return 1;
};

wxString ListaSillabe::ExtractLyr()
{	
	wxString Buffer;
	Sillaba *pSil = NULL;
	DrawObject *obj =GetFirst();
	Buffer = "";
	
	while(obj!=NULL)
	{
		pSil=(Sillaba *)((Node*)obj);
		if(pSil->Getgs() == '2')
			Buffer += '%';
		
		if(((pSil->Getgs()) == '[') || ((pSil->Getgs()) == '%'))
		{
			Buffer += (pSil->Getgs());
		}
		
		Buffer += pSil->GetTxt();
		
		if((pSil->Getgs()) == '2')
			Buffer += ']';
		
		if((pSil->GetToken() != '-') && (pSil->GetToken() != '_'))
		{
			if((pSil->Getgs()) == ']')
			{
				Buffer += (pSil->Getgs());
			}
		}
		if(pSil->GetToken()!='N' && pSil->GetToken() != '0' && pSil->GetToken()!='e')
		{
			wxString Buffer1(pSil->GetTxt());
			for(int n=1;n <=(pSil->GetNumSep()); n++)
			{
				if(!Buffer1.IsSameAs('$'))
				{
					Buffer += (pSil->GetToken());
				}
				else
				{
				}
			}
		}
		
		if((pSil->GetToken() == '-') && (pSil->GetToken() == '_'))
		{
			if((pSil->Getgs()) == ']')
			{
				Buffer += (pSil->Getgs());
			}
		}
		obj = GetNext(obj);
	}
	return Buffer;
}

wxString RemoveLyricSeparators(wxString lyrText, bool hideHTML)
{
	wxString text;
	bool copy=true;
	int braceCount=0;
	bool htmlTag=false;
	
	for(unsigned int i=0; i<lyrText.Length(); i++)
	{
		char c=lyrText[i];
		switch(c)
		{
		case'/':
		case '-':
		case '_':
		case '@':
		case '[':
		case '%':
		case ']':
			break;
		case '+':
			text+=' ';
			break;
		case '{':
			braceCount=1;
			i++;
			htmlTag=(lyrText[i]=='<');
			for(;i<lyrText.Length(); i++)
			{
				char c=lyrText[i];
				if(c=='{')
					braceCount++;
				else if(c=='}')
				{
					braceCount--;
					if(braceCount<=0)
						break;
				}
				if(!htmlTag || !hideHTML) 
					text+=c;
			}
			break;
		default:
			text+=c;
		}
	}
	
	return text;
}

void ListaSillabe::ExportLyricToHTML(wxString htmlFile)
{
	wxString text=RemoveLyricSeparators(ExtractLyr(),false);
	
	if(text.Find("<")==-1) // non ci sono tag
		text.Replace("\n","<br>\n");
	
	wxFile html(htmlFile, wxFile::write);
	
	if(html.IsOpened())
	{
		html.Write("<html>\n<body>\n");
		html.Write(text);
		html.Write("</body>\n</html>\n");
	}
}

void ListaSillabe::SetAttrSep(int ns, char tk,Sillaba* pInt1)
{
	pInt1->SetToken(tk);
	pInt1->SetNumSep(ns);
}

VUnit ListaSillabe::SetPos(Figura *pF,Sillaba *pInt,int n)
{
	VUnit YPent,Y;
	int i=0;
	Battuta *pb = (pInt->GetBattuta(1));
	
	YPent=pb->GetIntest(0)->GetAbsPos().y;
	
	Y= YPent-(DistPent*GetBaseLineFactor())-(n-1)*(LINES_DIST*GetLinesDistFactor());
    return Y;
}

int ListaSillabe::GetNumLayerWithPFigSpecified(Battuta *pBat,Figura *pFig)
{
	const short INDEX_MIN_OF_LAYER = 0;//una volta gli indici partivano da 1, se venissero ripristinati cosi' basta cambiare questo
	int Index_Max_Of_Layer;
	bool Trovato = FALSE;
	int indlay = -1;
	class Layer *tmp;
	
	if(pBat != NULL)
		Index_Max_Of_Layer = pBat->GetNumberOfStaff()*4;
	else
		return indlay;
	
	for(int i=INDEX_MIN_OF_LAYER; (i< Index_Max_Of_Layer)&&(Trovato == FALSE); i++)
	{
		tmp=pBat->GetPLayer(i);  
		if(tmp!=NULL)
        {
			if(tmp->FindFig(pFig))
			{
				indlay = i;
				Trovato = TRUE;
			}
		}
	}
	return indlay;
}

Figura* ListaSillabe::FindFirstFigOnLayerNotNull(Battuta *pBat,short CurrentLayer)
{
	const short INDEX_MIN_OF_LAYER = 0;
	int Index_Max_Of_Layer = /*pBat->GetNUsedLayers()*/11;
	Figura *pFig;
	
	if((CurrentLayer < INDEX_MIN_OF_LAYER)||(CurrentLayer > Index_Max_Of_Layer))
	{
		//il layer non e`stato settato
		return NULL;
	}
	else
	{
		ListaFigure *pL = pBat->GetPLayer(CurrentLayer);
		if(pL != NULL)
		{
			pFig = pL->GetFirstFig();
			if(pFig == NULL)
			{ 
				bool Trovato = FALSE;
				for(short i =0; (i<= Index_Max_Of_Layer)&&(Trovato == FALSE); i++)
				{
					ListaFigure *pL = pBat->GetPLayer(CurrentLayer);
					pFig = pL->GetFirstFig();
					if(pFig != NULL)
						Trovato = TRUE;
				}	       
			}
		}
		return  pFig;	 
	}
}

Sillaba *ListaSillabe::FindpSilPrecToTheActual(Node *pThisNode,Battuta *pBat,int NToLyr)
{
	Node *pn = NULL;
	Sillaba *pSilExaminated = NULL;
	
	if(pThisNode != NULL)
		pn = pThisNode->Prev;
	
	while((pn!=NULL)&&(pn!=first))
	{
		pSilExaminated=((Sillaba*)(void *)pn->ObjectPtr);
		if(pSilExaminated==NULL)
		{
			return NULL;
		}
		else
		{
			if((pBat == (pSilExaminated->GetPFigEnd()->GetPBattuta()))&&(NToLyr == (pSilExaminated->GetLine())) )
			{
				if(pSilExaminated->GetYSil() == -1)
				{
					return pSilExaminated;
				}
				
			}
		}
		pn = pn->Prev;	
    };
    return pSilExaminated;
}

void ListaSillabe::SetBaseLineFactor(double bl)
{
	BaseLineFactor = bl;
}

double ListaSillabe::GetBaseLineFactor()
{
	return BaseLineFactor;
}

void ListaSillabe::SetLinesDistFactor(double ld)
{
	LinesDistFactor = ld;
}

double ListaSillabe::GetLinesDistFactor()
{
	return LinesDistFactor;
}