/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#ifndef LISTASILLABE_HPP
#define LISTASILLABE_HPP

#define DELTAX_FIGURA_SILLABA 2
#define WIDTH_RIPRESA_SILLABA 2

#include <wx/msgdlg.h>
#include <lioo.hpp>
#include "lyr/sillaba.hpp"
#include "lst/lista.hpp"
#include "bat/battuta.hpp"
#include "spa/listabat.hpp"

class LyricFrame;
class SMRLyricsItem;
namespace xercesc_2_6
{
	class InputSource;
}

#define MAX_LYRIC_LINES 4

class ListaSillabe:public Lista
{
	
protected:
	NumCode CodeCounter;
	// Contatore per assegnare il codice
	// numerico ad ogni oggetto della lista.
	int InterLine;
	// Valore numerico che indica la strofa
	// di Lirica a cui si riferiscono le
	// sillabe.Le strofe possibili sono 4.
	short Layer;
	// Valore numerico che indica a quale
	// layer della battuta le sillabe si
	// riferiscono.
	
	double LinesDistFactor;
	double BaseLineFactor;
    Sillaba *Started[MAX_LYRIC_LINES];
	
	VUnit DistPent;
	// Valore numerico che indica la distanza
	// dall'ultima riga del pentagramma alla
	// quale deve essere posta la prima strofa 
	// della lirica.  
	//Sillaba *Started; 
	// Contiene l'indirizzo della sillaba
	// estesa che  iniziata durante la
	// scansione della battuta.
	
	Node *GetSilF(Figura *pF,Node* );
	// Scandisce la lista per verificare
	// l'esistenza di una sillaba associata
	// alla pF passata come riferimento.
	// Restituisce NULL se non ha trovato
	// niente.
	// Viene utilizzata nella costruzione
	// dello SPARTITO.
	Node *GetIntStartF(Figura *pF,VUnit,Node* );
	// Scandisce la lista per trovare,se
	// presente,una sillaba estesa il cui
	// prolungamento ha inizio sulla figura
	// pF.Se non trova niente resituisce NULL.
	// Durante la seconda scansione delle
	// figure che costituiscono il rigo di
	// pentagramma,imposta anche la coordinata
	// grafica y della sillaba.
	Node *GetIntEndF(Figura *pF,Node*);
	// Scandisce la lista per trovare,se
	// presente,una sillaba estesa il cui
	// prolunganento ha termine sulla figura
	// pF.Se non trova niente ritorna NULL.
	
	ListaBattute *listaBat;
	// Puntatore alla lista di battute
	// corrente. 
	
public:
	ListaSillabe();
	// Costruttore di default.
    ListaSillabe(int line,short layer,VUnit DISTPENT);	
	// Costruttore.
	~ListaSillabe();
	// Distruttore.
	
	void SetLayer(short layer);
	// Setta il layer a cui la strofa di 
	// lirica si riferisce.
	void SetLine(int line);
	// Setta la variabile che indica il
	// numero della strofa.
	void SetDistPent(VUnit distpent);
	// Setta la distanza minima dalla
	// ultima riga del pentagramma in
	// modo che le sillabe non vadano
	// a sovrapporsi alle note che ne
	// fuoriescono.
	
	
	void Free();
	// Dealloca la lista.
	void DelNodo(Sillaba *pncorr);
	void Init();
	// Inizializza la lista.
	
	
	void SetListaBattute(ListaBattute *a){ listaBat=a;};
	// Setta il parametro che memorizza la
	// lista di battutte che contiene le
	// figure a cui va associata la LIRICA.
    ListaFigure *GetPLayer(Battuta *battutacorrente);
	   // Sulla base della battuta corrente e
	   // del Layer di riferimento determina
	   // l'indirizzo della lista di figure che
	   // si sta considerando.	
	Figura *VeriFig(Figura *pCor,Figura *pFEnd,ListaFigure *pLayer,int ns);
	// Per ogni figura della battuta corrente
	// verifica che non vi sia gia` una sillaba
	// associata.Restituisce il puntatore alla
	// successiva figura.
	Bool VerifOccup(Figura *pF1,Figura *pF2,Battuta *pCor,int ns,int IndexLayer);
	// Richiama il metodo precedente piu` volte
	// nel caso la sillaba si estenda a cavallo 
	// tra due o piu` battute.Restituisce FALSE
	// se la sillaba puo` essere aggiunta(figure
	// libere) e TRUE altrimenti.
	
	
	void Add(const char *s,Figura *pF1,Figura *pF2,Battuta *pb1,
		Battuta *pb2,int numtoken,Bool Trat,Bool TraSil,char Token,int NumSep,int strofa,char gs,int NumLayer,bool Associazione, double shf, double Bl);///////////poggiolini
	// Aggiunge una nuova sillaba allocandola
	// e settandone testo e attributi.
	void InserSillaba(Figura *Fig1,Figura *Fig2,const char *testo,Battuta *pBat1,
		Battuta *pBat2,Bool TRAT,Bool TRASIL);
	// Richiamata da Spartito quando deve essere
	// l'eventuale lirica associata,inserisce la
	// sillaba,passatagli, nella corrispondente
	// lista.
	
	void InsStarted(Sillaba *pInt);
	// Inserisce nello Started la sillaba
	// estesa il cui prolungamento  iniziato
	// nella battuta presa in esame.
	Sillaba *GetStarted();
	// Resttuisce il contenuto di STARTED.
	Sillaba *InitStarted();
	// Pone in Started NULL,inizializzandolo.
	// Utilizzato nei metodi che riscontrano
	// i'estensione della sillaba tra due battute
	// diverse.	
	void RemoveStarted(Sillaba *pInt);
	// Rimuove l 'indirizzo della sillaba
	// dallo STARTED qualora la sillaba estesa
	// abbia inizio e fine nella stessa battuta.
	
    void PosStarted(Figura *pF);
	void PosStarted(const Point& p);
	// Sulla base della figura a cui  associata
	// la sillaba,impostano la coordinata x di
 	  // partenza del prolungamento memorizzato
	// nello STARTED.
	
	
	Figura *FindFigPrev(Figura *pFEnd,Figura *pFStart);
	// Data una figura in ingresso permette
	// di determinare la figura prima precedente
	// che non sia uno spazio.
	// La scansione inizia dalla prima figura o
	// della battuta contenente la figura iniziale o
	// nel caso questa sia nulla, della prima battuta
	// della lista.
	void CancProlStart(Node *pn,Figura *pFStart,Figura *pFEnd,Battuta *pBatCor);
	// Viene impiegato dal metodo che si preoccupa,
	// nel caso di cancellazione di una figura, di
	// cancellare l'eventuale sillaba associata.
	// Viene richiamato quando la sillaba inizia
	// sulla figura in questione ed  estesa,con
	// Prolungamento continuo o tratteggiato. Si
	// preoccupare di aggiornare il puntatore
	// iniziale alla figura successiva a quella
	// da cancellare.
	
	void CancProlEnd(Node *pncorr,Node *pnsucc,Figura *pFStart,Figura *pFEnd);
	// Viene impiegato dal metodo che si preoccupa,
	// nel caso di cancellazione di una figura, di
	// cancellare l'eventuale sillaba associata.
	// Viene richiamato quando la sillaba inizia
	// l dove termina una estesa tratteggiata
	// oppure quando la sillaba  estesa,senza la
	// figura finale e termina su quella da cancellare.
	
	void CancSingTrat(Sillaba *pcorr,Sillaba *psucc,int PrimaLettera);
	// Viene impiegata nel metodo CancSilFig quando
	// la figura da cancellare ha associata una
	// sillaba facente parte di una parola sillabata
	// e quindi le cui sillabe sono separate da un
	// unico trattino(prolungate del tipo SegTraSil).
	// Si preoccupa di fondere la sillaba in questione
	// a quella successiva o a quella precedente,nello
	// ultimo caso aggiornando sulla base di quella
	// attuale la tipologia.
	// Prende in ingresso la sillaba legata alla
	// figura da cancellare,quella q cui fonderla
	// e un intero che indica se  la prima o meno
	// di una parola(va in tal caso fusa con la
	// successiva(Restituisce il nodo da eliminare.
	
	void CancSilFig(Figura *pF);
	// Trova l'eventuale sillaba estesa il cui
	// prolungamento termina o inizia sulla
	// figura e la cancella.
    void CancSilBat(Battuta *pbat);
	// Per ogni figura della battuta verifica
	// se vi  associata una sillaba e richiama
	// il metodo precedente.
	
    void ResetPos();
	// Dealloca le linee che compongano il
	// prolungamento della sillaba estesa.
	
    void FindPrevStartedFig(Figura *pF);
	// Pone nello STARTED la sillaba  estesa
	// che iniziata nella pagina precedente
	// termina con la figura *pF.
    void FindPrevStartedBat(Battuta *pB);
	// Pone nello STARTED la sillaba estesa
	// il cui prolungamento iniziato nelle pagine
	// precedentitermina nella battuta corrente.
	// Esegue il metodo precedente per ogni
	// figura che compone la battuta.
	
	
    void ExamineBat(Battuta *pB);
	// Esamina la battuta e per ogni figura pF
	// controlla se esiste una sillaba estesa
	// che termina o inizia in pF:
	// Quella che termina in pF viene tolta dallo
	// STARTED;quella che  inizia in pF vi viene messa.
	// Viene impiegata in SPARTITO per la disposizione
	// della lirica.
    void ExamineFig(Figura *pF, VUnit YPent);
	// Esamina la figura pF  e controlla se vi 
	// associata una sillaba estesa che inizia o
	// termina in pF.Se termina in pF viene tolta
	// dallo STARTED;se inizia in pF vi viene messa.
    void ExamineInizioPentagr(Battuta *pB,int ns);
	// inizio pagina o pentagramma viene impostata
	// la x della Sillaba in Started,in cima a sinistra.
	
	void EndPage(Battuta *pB);
	// Conclude il disegno del prolungamento con
	// la battuta assegnata (cancella lo STARTED).
    void EndRow(Battuta *pB);
	// Conclude il disegno del prolungamento con
	// la battuta assegnata;aggiunge un segmento
	// alla Sillaba nello Started in quanto per
	// DLIOO sono presenti pi righi consecutivi.
	
    void Draw();
	// Disegna i simboli di intervallo per i
	// quali YSil>=0(Sillaba non posizionata).
	
	
	
	void SetCodeCounter(NumCode);
	// Setta il codice numerico associato
	// ad ogni sillaba.
	NumCode GetCodeCounter(void);
	// Restituisce il NumCode della sillaba.
	Sillaba *GetNumCodeSil(NumCode);
	// Restituisce l'indirizzo della sillaba
	// corrispondente al NumCode.
	
    int  SalvaLyr(SMRLyricsItem* lwf,char *FileName);
	// Crea un documento di tipo XML(DOMDocument)
	// e ne salva il contenuto sul file il cui nome
	//   identificato dal codice numerico dello
	// spartito a cui si riferisce.Tale codice gli
	// viene passato dal metodo che richiama
	// SalvaLyr.
	int LoadLyr(char *Name, xercesc_2_6::InputSource *src=NULL, int validScoreId=0);
	
	void SetAttrSep(int ns, char tk,Sillaba* pInt1);
	
	VUnit SetPos(Figura *pF,Sillaba *pInt,int n);
	
	wxString ExtractLyr();
	void ExportLyricToHTML(wxString file);
	
	Sillaba *GetStarted(int ns);
	
	int GetNumLayerWithPFigSpecified(Battuta *pBat,Figura *pFig);
	Figura *FindFirstFigOnLayerNotNull(Battuta *pBat,short CurrentLayer);
	Sillaba *FindpSilPrecToTheActual(Node *pThisNode,Battuta *pBat,int NToLyr);
	
	void PrintSil();
	
	void SetBaseLineFactor(double bl);
    double GetBaseLineFactor();
	
	void SetLinesDistFactor(double bl);
    double GetLinesDistFactor();
	
};
#endif LISTASILLABE_HPP
