/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include <lyr/LyricFrame.hpp>
#include "drw/musicedt.hpp" 
#include <gui/lioowin.hpp>
#include <gui/lioomenu.hpp>
#include "drw/scorefrm.hpp"

#include "lyr/lyrdlgint.hpp"

extern short mainLay;


// ----------------------------------------------------------------------------
// event tables
// ----------------------------------------------------------------------------



BEGIN_EVENT_TABLE(LyricFrame, wxFrame)
EVT_MENU(wxID_EXIT, LyricFrame::OnQuit)
EVT_CLOSE(LyricFrame::OnQuit)
EVT_MENU(wxID_NEW, LyricFrame::OnNew)
EVT_MENU(wxID_OPEN, LyricFrame::OnOpen)
EVT_MENU(50, LyricFrame::OnViewHTML)

EVT_MENU(IDM_TOOLBAR_ADD, LyricFrame::Add)
EVT_MENU(IDM_TOOLBAR_VIEW, LyricFrame::View)
EVT_MENU(IDM_TOOLBAR_JUST, LyricFrame::OnSetShift)

EVT_MENU(wxID_COPY, LyricFrame::CopyT)
EVT_MENU(wxID_CUT, LyricFrame::CutT)
EVT_MENU(wxID_PASTE, LyricFrame::PasteT)
EVT_TEXT(100,LyricFrame::OnWriting)
END_EVENT_TABLE()


// ----------------------------------------------------------------------------
// LyricFrame
// ----------------------------------------------------------------------------

//costruttore
LyricFrame::LyricFrame(wxFrame* parent,
					   wxWindowID id,
					   const wxString& title,
					   const wxPoint& pos,
					   const wxSize& size,
					   long style,wxMEScoreFrame* framecaller)
					   : wxFrame(parent, id, title, pos, size, style | wxFRAME_FLOAT_ON_PARENT	)
{
    m_textWindow = new wxTextCtrl(this, 100, "", wxPoint(0, 0), wxSize(-1, -1), wxTE_MULTILINE);
	
	PercorsoDefault = "Z:\\prove\\";
	BufferS = "";
	BufferS1 = "";
	Strada = "";
	PathOpen ="",
		CompletePath = "";
	PartialPath = "";
	Modified = FALSE;
    Editable = FALSE;
    m_smallToolbar = TRUE;
    m_horzToolbar = TRUE;
    m_rows = 1;
    m_textWindow->SetEditable(FALSE);
	pSpa = NULL;
	
	LinkMusEdit=framecaller;
	
    CreateStatusBar(2);
	
    SetIcon(wxICON(LyrIcon));
	
    wxMenu *fileMenu = new wxMenu;
	fileMenu->Append(wxID_NEW, "&New Lyric\tCtrl-N", "Crea un nuovo file" );
	fileMenu->Append(wxID_OPEN, "&Open Lyric\tCtrl-O", "Apre file esistente" );
	fileMenu->Append(wxID_SAVE, "&Save Lyric\tCtrl-S", "Salva il file" );
	fileMenu->Append(50, "&Export HTML\tCtrl-E", "Salva il file" );
	fileMenu->AppendSeparator();
	fileMenu->Append(wxID_EXIT, "E&xit", "Quit " );
	
	wxMenu *editMenu = new wxMenu;
	editMenu->Append(wxID_CUT, "&Cut\tCtrl-X", "Taglia" );
	editMenu->Append(wxID_COPY, "C&opy\tCtrl-C", "Copia" );
	editMenu->Append(wxID_PASTE, "&Paste\tCtrl-V", "Incolla" );
	editMenu->Append(IDM_TOOLBAR_JUST, "Set&tings\tCtrl-I", "Shift" );
	
	wxMenu *commandMenu = new wxMenu;	
	commandMenu->Append(IDM_TOOLBAR_VIEW, "&View tokens\tCtrl-K", "view");
	commandMenu->Append(IDM_TOOLBAR_ADD, "L&ink\tCtrl-J", "Link");
	
    wxMenu *helpMenu = new wxMenu;
	helpMenu->Append(wxID_HELP, "&About\tF1", "About LyricEditor");
	
    wxMenuBar* menuBar = new wxMenuBar( wxMB_DOCKABLE );
	menuBar->Append(fileMenu, "&File");
	menuBar->Append(editMenu, "&Edit");
	menuBar->Append(commandMenu, "&Command");
	menuBar->Append(helpMenu, "&Help");
	
    SetMenuBar(menuBar);
	
    RecreateToolbar();
	GetToolBar()->EnableTool(IDM_TOOLBAR_VIEW,FALSE);
	ResetTool();
}

void LyricFrame::RecreateToolbar()
{
    // delete and recreate the toolbar
    wxToolBarBase *toolBar = GetToolBar();
    delete toolBar;
	
    SetToolBar(NULL);
	
    long style = wxNO_BORDER | wxTB_FLAT | wxTB_DOCKABLE;
    style |= m_horzToolbar ? wxTB_HORIZONTAL : wxTB_VERTICAL;
	
    toolBar = CreateToolBar(style, ID_TOOLBAR);
    toolBar->SetMargins( 4, 4 );
	
    // Set up toolbar
    wxBitmap toolBarBitmaps[11]; //10
	
    toolBarBitmaps[0] = wxBITMAP(NEW);
    toolBarBitmaps[1] = wxBITMAP(OPEN);
    toolBarBitmaps[2] = wxBITMAP(SAVE);
    toolBarBitmaps[3] = wxBITMAP(COPY);
    toolBarBitmaps[4] = wxBITMAP(CUT);
    toolBarBitmaps[5] = wxBITMAP(PASTE);
    toolBarBitmaps[6] = wxBITMAP(PRINT);
    toolBarBitmaps[7] = wxBITMAP(HELP);
	toolBarBitmaps[8] = wxBITMAP(ADD);
	toolBarBitmaps[9] = wxBITMAP(VIEW);
	
    if ( !m_smallToolbar )
    {
        int w = 2*toolBarBitmaps[0].GetWidth(),
            h = 2*toolBarBitmaps[0].GetHeight();
        for ( size_t n = 0; n < WXSIZEOF(toolBarBitmaps); n++ )
        {
            toolBarBitmaps[n] =
                wxImage(toolBarBitmaps[n]).Scale(w, h).ConvertToBitmap();
        }
		
        toolBar->SetToolBitmapSize(wxSize(w, h));
    }
	
#ifdef __WXMSW__
    int width = 24;
#else
    int width = 16;
#endif
	
    int currentX = 5;
	
    toolBar->AddTool(wxID_NEW, toolBarBitmaps[0], wxNullBitmap, FALSE, currentX, -1, (wxObject *) NULL, "New file");
    currentX += width + 5;
    toolBar->AddTool(wxID_OPEN, toolBarBitmaps[1], wxNullBitmap, FALSE, currentX, -1, (wxObject *) NULL, "Open file");
	
    currentX += width + 5;
    toolBar->AddTool(wxID_SAVE, toolBarBitmaps[2], wxNullBitmap, FALSE, currentX, -1, (wxObject *) NULL, "Save File");
    currentX += width + 5;
    
	
    currentX += width + 7;
	toolBar->AddSeparator();
    toolBar->AddTool(wxID_CUT, toolBarBitmaps[4], wxNullBitmap, FALSE, currentX, -1, (wxObject *) NULL, "Cut");
    currentX += width + 5;
	toolBar->AddTool(wxID_COPY, toolBarBitmaps[3], wxNullBitmap, FALSE, currentX, -1, (wxObject *) NULL, "Copy");
    currentX += width + 5;
    toolBar->AddTool(wxID_PASTE, toolBarBitmaps[5], wxNullBitmap, FALSE, currentX, -1, (wxObject *) NULL, "Paste");
	
    currentX += width + 7;
	toolBar->AddSeparator();
	toolBar->AddTool(IDM_TOOLBAR_VIEW, toolBarBitmaps[9], wxNullBitmap, TRUE, currentX, -1, (wxObject *) NULL, "View");
    currentX += width + 5;
	toolBar->AddTool(IDM_TOOLBAR_ADD, toolBarBitmaps[8], wxNullBitmap, FALSE, currentX, -1, (wxObject *) NULL, "Link");
    currentX += width + 5;
    toolBar->AddSeparator();
    toolBar->AddTool(wxID_HELP, toolBarBitmaps[7], wxNullBitmap, TRUE, currentX, -1, (wxObject *) NULL, "Help button");
	
	
    toolBar->Realize();
	
    toolBar->SetRows(m_horzToolbar ? m_rows : 10 / m_rows);
}

#if USE_GENERIC_TBAR

wxToolBar* LyricFrame::OnCreateToolBar(long style,
									   wxWindowID id,
									   const wxString& name)
{
    return (wxToolBar *)new wxToolBarSimple(this, id,
		wxDefaultPosition, wxDefaultSize,
		style, name);
}

#endif // USE_GENERIC_TBAR

void LyricFrame::OnNew(wxCommandEvent& event)
{   
	SaveRequest();
	m_textWindow->Clear();
	BufferS="";
	BufferS1="";
	PathOpen ="",
		CompletePath = "";
	PartialPath = "";
	GetToolBar()->EnableTool(IDM_TOOLBAR_VIEW,TRUE);
	indsh = 1;
	bash = 1;
}

wxString RemoveLyricSeparators(wxString lyrText, bool hideHTML);

void LyricFrame::OnViewHTML(wxCommandEvent& event)
{
	wxFileDialog SaveWindow(this,"Save","","","HTML Files (*.htm)|*.htm",wxSAVE | wxOVERWRITE_PROMPT);
	if(SaveWindow.ShowModal()!=wxID_OK)//
		return;
	if (SaveWindow.GetPath() != "")
	{
		wxString text=RemoveLyricSeparators(m_textWindow->GetValue(),false);
		
		if(text.Find("<")==-1) // non ci sono tag
			text.Replace("\n","<br>\n");
		
		wxFile html(SaveWindow.GetPath(), wxFile::write);
		
		html.Write("<html>\n<body>\n");
		html.Write(text);
		html.Write("</body>\n</html>\n");
	}
} 


void LyricFrame::OnOpen(wxCommandEvent& event)
{
	SaveRequest();
	GetToolBar()->EnableTool(IDM_TOOLBAR_VIEW,TRUE);
	wxFileDialog OpenWindow(this,"Open","","","TEXT Files (*.txt)|*.txt| LWF Files (*.lwf)|*.lwf",wxOPEN);
	OpenWindow.ShowModal();
	
	if (OpenWindow.GetPath() != "")
	{
		PathOpen = OpenWindow.GetPath();
		Trasferimento(OpenWindow.GetPath());
	}	
}

void LyricFrame::OnQuitFromME()
{
	wxCommandEvent Evt;
	OnQuit(Evt);
}


void LyricFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{
	SaveRequest();
	
	if(LinkMusEdit!=NULL)
		LinkMusEdit->SetLyricFramePtr(NULL);
	Destroy();
}

void LyricFrame::View(wxCommandEvent& event)
{
	View();
} 

void LyricFrame::View()
{
	Editable =!Editable;                          
	GetToolBar()->ToggleTool(IDM_TOOLBAR_VIEW,Editable);
	m_textWindow->SetEditable(Editable);
	
	if(Editable)
	{ 
		SetTool();
		m_textWindow->Clear();
		m_textWindow->AppendText(BufferS);  
	}
	else
	{ 
		ResetTool();
		BufferS = m_textWindow->GetValue(); 
		
		Clear(BufferS);
	}	
}

void LyricFrame::Add(wxCommandEvent& event)
{
	AddNewLyric(BufferS,pSpa);
}

void LyricFrame::CutT(wxCommandEvent& event)
{
	m_textWindow->Cut();   
}

void LyricFrame::CopyT(wxCommandEvent& event)
{
	m_textWindow->Copy();
	
}

void LyricFrame::PasteT(wxCommandEvent& event)
{
	m_textWindow->Paste();
}

void LyricFrame::Trasferimento(wxString Percorso)
{	
	wxTextFile FileT;
	wxFile BufFile;
    unsigned int nLineT;
	wxString copia;
	
    BufferS = "";
	FileT.Open(Percorso);
	
	for(nLineT = 0; nLineT < FileT.GetLineCount(); nLineT++)
	{
		BufferS+=FileT.GetLine(nLineT);
		BufferS+="\n"; //aggiunta
	}
	
	BufferS1 =BufferS;
	if(Editable)
	{
		m_textWindow->Clear();
		m_textWindow->AppendText(BufferS);
	}
	else
		Clear(BufferS);
}

void LyricFrame::Clear(wxString lyrText)
{
	m_textWindow->Clear();
	m_textWindow->AppendText(RemoveLyricSeparators(lyrText,true));
	m_textWindow->SetEditable(FALSE);	
}

void LyricFrame::OnWriting(wxCommandEvent& event)
{
	Modified=TRUE;
}

void LyricFrame::LoadStringWithText()
{
	BufferS=m_textWindow->GetValue();
}

void LyricFrame::LoadTextWithString()
{
	m_textWindow->Clear();
	m_textWindow->AppendText(BufferS);
}

void LyricFrame::Avviso()
{
}

void LyricFrame::SaveRequest()
{
	wxString ConfCont;
	
	if(Editable)
	{		      
		ConfCont += m_textWindow->GetValue(); 
		      
        if(ConfCont != BufferS)
		{
			Avviso();
		}
		else
		{
			if(BufferS != BufferS1)
			{
				Avviso();
			}
		}
	}
	else
	{
		if(BufferS != BufferS1)
		{
			LoadTextWithString();
			Avviso();
		}
	}
}

void LyricFrame::SetTool()
{
	GetToolBar()->EnableTool(wxID_CUT,TRUE);
	GetToolBar()->EnableTool(wxID_COPY,TRUE);
	GetToolBar()->EnableTool(wxID_PASTE,TRUE);
	GetToolBar()->EnableTool(wxID_SAVE,TRUE);	
}

void LyricFrame::ResetTool()
{
	GetToolBar()->EnableTool(wxID_CUT,FALSE);
	GetToolBar()->EnableTool(wxID_COPY,FALSE);
	GetToolBar()->EnableTool(wxID_PASTE,FALSE);
	GetToolBar()->EnableTool(wxID_SAVE,FALSE);
}

void LyricFrame::GetCompPath(wxString CompPath)
{ 
	CompletePath = CompPath;	
}

wxString LyricFrame::GetPartPath()
{	
	return CompletePath;
}

void LyricFrame::SetWriting()
{
	wxCommandEvent Evt;
	OnNew(Evt);
	View(Evt);
}

void LyricFrame::SetPuntMusEdt(wxMEScoreFrame* Ptr)
{
	LinkMusEdit = Ptr;
}

void LyricFrame::WriteLyric(wxString text,Spartito* PS)
{
	SaveRequest();
	if(!text.IsEmpty())
	{
		m_textWindow->Clear();
		m_textWindow->AppendText(text);
		LoadStringWithText();
	}
	pSpa = PS;
	if(pSpa!=NULL)
	{
		ListaSillabe *listaSil=pSpa->GetListaSillabe();
		SetShift(listaSil->GetLinesDistFactor());
		SetBllf(listaSil->GetBaseLineFactor());
	}
}

wxString LyricFrame::InsBlkToString(wxString str)//
{
	return str;
}

wxString LyricFrame::StringManage(wxString str)
{
	wxString str1 = "";
	for(unsigned int i=0;i<= str.Length();i++)
	{
		switch (str.GetChar(i))
        {
		case '%':
			if((str.GetChar(i-1) != '-')
				&&(str.GetChar(i-1) != '_'))
			{
				str1 += " %";
			}
			else
				str1 += "%";			
			break;
		case ']':
			if((str.GetChar(i-1) != '-')
				&&(str.GetChar(i-1) != '_'))
			{
				str1 += "] ";
			}
			else
				str1 += "]";//???
			break;
		default:
			str1 += str.GetChar(i);
			break;
		}
	}
	return str1;	
}

wxString LyricFrame::ManagerAt(wxString Str)
{
	return Str;
	
}

wxString LyricFrame::AddNewLyric(wxString Name,Spartito *pSpartito)
{
	wxString BufferMoods;
	pSpa = pSpartito; 
	
	if(LinkMusEdit!= NULL) //Trova il puntatore = a NULL
	{ 
		if(Editable)
		{ 
			LoadStringWithText();
			if(!SyntaxCheck(BufferS))
				return "";
			
			if(BufferS != "")
			{    
				
				LinkMusEdit->SetFocus();
				
				if(pSpartito != NULL)
				{  
					extern wxDC *currDC;
					currDC=NULL;
					pSpartito->AssignLyric(BufferS, GetShift(), GetBllf());
				}
				
				extern wxMusicEditor *MusicEditor;
				MusicEditor->RedrawViews(0,true);    
			}
		}
	}
	else
	{
		wxMessageBox("MusicEditor already closed!","ERROR!");
	}	
	return BufferMoods;
}

bool LyricFrame::SyntaxCheck(wxString Testo)
{
	char Prec = ' ';
	char Succ = ' ';
	char Curr = ' ';
	
    int NumOfLyrLine =0;
	for(unsigned int index =0; index<= Testo.Length();index++)
	{
		if(index > 0)
			Prec=Testo.GetChar(index-1);
		else
			Prec=Testo.GetChar(0);
		
		Curr=Testo.GetChar(index);
		
		if(index < Testo.Length())
			Succ=Testo.GetChar(index+1);
		
		if(DetectSymbolLyr(Curr))  //e' un simbolo di separazione sillaba.
		{	
            if((index == 0)&&(Curr != '@')) //solo gli @ possono andare all'inizio.
			{
				SyntaxError("Syllable separators cannot not stay in this position",index);
				return false;
			}
            ///////////////////// casi tra i simboli di lirica
			if(DetectSymbolLyr(Succ))  //simboli diversi accostati a sx es _- .
			{
				//caso permesso es @@[ per iniziare 2 note dopo.
				if(Curr!=Succ)
				{
                    SyntaxError("Two adjacent syllable separators must be equal",index);
					return false;
				}
			}
			
			////////////////////casi tra simboli lyr e simboli refrain.
			if(DetectSymbolRefrain(Succ))  //simboli diversi accostati a sx es _- .
			{
				if((Succ != '[')&&(Curr != '@')) //caso permesso es @@[ per iniziare 2 note dopo.
				{
					SyntaxError("Refrain symbols cannot be in this position",index+1);
					return false;
				}
			}
			
			if(DetectSymbolRefrain(Prec))   //simboli diversi accostati a dxes _- .
			{
				SyntaxError("This symbols do not be in this position",index);
				return false;
			}
		}
		
		if(DetectSymbolLyrPar(Curr))
		{
            ///////////////////// casi tra i simboli di lirica
			if(DetectSymbolLyr(Succ))  //simboli diversi accostati a sx es _- .
				if(Curr!=Succ)
				{
					SyntaxError("Two adjacent syllable separators must be equal",index+1);
					return false;
				}
		}
		
		if(DetectSymbolRefrain(Curr))
		{
			if(DetectSymbolLyr(Succ)) 
			{
				SyntaxError("The lyric symbol must be started with syllabes",index);
				return false;
			}
			if(DetectSymbolLyr(Prec) && Prec != '@')
			{
				SyntaxError("The lyric symbol must be ended with syllabes",index);
				return false;
			}	
			if((Curr == '[')||(Curr == ']'))  
				if(Curr==Succ)
				{
					SyntaxError("Two symbols adjacent do not be equals",index);
					return false;
				}
		}
		
		if(Curr == '%')
		{
			NumOfLyrLine++;
			if(NumOfLyrLine>3)
			{
				SyntaxError("You have only four line of strofes",index);
				return false;
			}
		}
		if(Curr == ']')
		{
			NumOfLyrLine =0;
		}
	}
	SetStatusText("",1);
	return true;
}

void LyricFrame::SyntaxError(wxString msg,int pos)
{
	m_textWindow->SetInsertionPoint(pos+1);
	m_textWindow->SetSelection(pos+1,pos+2);
	m_textWindow->SetFocus();
	SetStatusText(msg,1);
}

bool LyricFrame::DetectSymbolLyr(char s)
{
	if(s == '-' || s =='_' || s =='/' || s =='@' || s =='+')
		return TRUE;
	else
		return FALSE;
}

bool LyricFrame::DetectSymbolRefrain(char s)
{
	if(s=='[' || s =='%' || s ==']')
		return TRUE;
	else
		return FALSE;
}

bool LyricFrame::DetectSymbolLyrPar(char s)
{
	if(s ==' ' || s =='0' || s =='|')  
		return TRUE;		
	else
		return FALSE;
}

wxString LyricFrame::InsertSpaceAmongSymbols(wxString imm)
{
	wxString testo = "";
	char s =' ';
    
    for(unsigned int i=0;i<= imm.Length();i++)
	{		
		testo += imm.GetChar(i);
		if(imm.GetChar(i)=='-'||imm.GetChar(i)=='_')
		{
			s = imm.GetChar(i+1);
			if((!DetectSymbolLyrPar(s))&&(!DetectSymbolLyr(s)))
			{
				testo += " ";
				testo += imm.GetChar(i);
			}
			else
				testo += imm.GetChar(i);
		}
    }
	return testo;
}

wxString LyricFrame::DelView(wxString imm)
{
	wxString testo = "";
	
    for(unsigned int i=0;i<= imm.Length();i++)
	{
		if(imm.GetChar(i)=='>')
			testo += imm.GetChar(i+1);
		else
			testo += imm.GetChar(i);			
    }
	return testo;
}

wxString LyricFrame::SpaceForNoView(wxString imm)
{
	wxString testo = "";
    
    for(unsigned int i=0;i<= imm.Length();i++)
	{
		if(imm.GetChar(i)=='>')
			testo += ' ';
		else
			testo += imm.GetChar(i);		
    }
	return testo;	
}

wxString LyricFrame::detectspcForNV(wxString imm)
{	
    for(unsigned int i=0;i<= imm.Length();i++)
	{        
		if(imm.GetChar(i)=='-' || imm.GetChar(i)=='_')
		{ 
			if(imm.GetChar(i+1)==' ')  
				imm.SetChar(i+1,'>');
		}
    }
	
	wxString testo;
    for(unsigned int j=0;j<= imm.Length();j++)
	{		
		if(imm.GetChar(j)=='>')
		{ 
			testo += imm.GetChar(j);
			testo += ' ';			
		}
		else
			testo += imm.GetChar(j);
    }
	return testo;
}

void LyricFrame::OnSetShift(wxCommandEvent& event)
{
	LyrDlgInt dialog;
	if(pSpa!=NULL)
	{
		ListaSillabe *listaSil=pSpa->GetListaSillabe();
		dialog.Initialize(listaSil->GetBaseLineFactor(),listaSil->GetLinesDistFactor());
	}
	else
		dialog.Initialize();
	
	dialog.Centre();
	
	if(dialog.ShowModal()==wxID_OK)
	{
		indsh = dialog.GetLinesDist();
		bash = dialog.GetBaseLine();
	}
}

double LyricFrame::GetShift()
{
	return indsh;
}

double LyricFrame::GetBllf()
{
	return bash;
}

void LyricFrame::SetShift(double s)
{
	indsh = s;
}

void LyricFrame::SetBllf(double b)
{
	bash = b;
}

