/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#ifndef SILLABA_HPP
#define SILLABA_HPP

#include "lioo.hpp"
#include "drw/text.hpp"
#include "drw/gscreen.hpp"
#include "drw/drawobje.hpp"
#include "fig/figura.hpp"
#include "bat/battuta.hpp"
#include "spa/listabat.hpp"

// La classe sillaba incapsula la singola sillaba    //  
// di una parola; ogni sillaba pu essere associata  //
// ad una o pi figure adiacenti .                   //
// La sillaba si limita a memorizzare la figura di   //
// inizio e di fine della sequenza di figure a cui   //
//  associata. Incapsula inoltre un metodo che si   // 
// preoccupa di trasformare la sillaba in un oggetto //
// grafico.                                          //


class ProlLine
{
public:
	Point AbsP;
	// Posizione assoluta del punto
	// centrale della linea.
	VUnit Wdt,Hgt;
	// Larghezza e altezza della linea.				
	ProlLine *pNext;
	// Puntatore alla linea successiva
	// presente nella pagina successiva
	// nel caso di sillaba estesa a
	// cavallo tra due pagine.			
	ProlLine();
	// Costruttore.
	~ProlLine();
	// Distruttore.		
};


class Sillaba :public Text
{
protected:
	
	Figura *PFigStart, *PFigEnd;
	// Puntatori alle figure a cui la sillaba
	// si riferisce.
	Bool Prol;
	// Settata a TRUE indica che la sillaba
	//  estesa e il suo prolungamento 
	// costituito da un tratto continuo.
	Bool ProlTrat;
	// Settata a TRUE indica che la sillaba 
	// estesa ma il prolungamento in questione
	// non  continuo ma tratteggiato.
	Bool SegTraSil;
	// (CASO DI INSERIMENTO SILLABA DI UNA
	// PAROLA ASSOCIATA TEORICAMENTE AD UNA
	// SOLA NOTA).
	// Settata a TRUE indica che il testo
	// inserito   una sillaba estesa il cui
	// prolungamento  costituito da un solo
	// trattino.Sia in questo caso che in
	// quello pecedente si pu verificare
	// l'esistenza di due sillabe associate
	// ad una stessa figura(per una la finale
	// per l'altra l'iniziale).
	int Numtoken;
	// Indica il numero di token,inserito
	// dal'utente nel caso di sillaba estesa
	// continua o tratteggiata.Permette nel
	// caso di cancellazione di non perdere
	// informazioni sulla natura iniziale
	// della sillaba.E' nullo per sillabe
	// singole o divise da semplice trattino.
	
	char gs;
	char Token;
	int  NumSep;
	int LineStrf;
	//VUnit Yp,Dp;
	int NLayer;
	bool Is_Assoceable;
	double Shift;
    ////////////////////////////////////////////////////
	
	
	
	Battuta *pbat1,*pbat2;
	
	ListaBattute *listaBat;
	// Variabile che permette il settaggio dello
	// attributo AppSil della figura per una
	// sillaba estesa il cui prolungamento 
	// cavallo tra una o p battute.
	// Memorizza la lista di battute in cui
	// si trova la figura di interesse.	
	
	Bool Inizio[MAX_VIEW],Fine[MAX_VIEW];
	// Indicano se l'eventuale prolungamento della
	// sillaba ha inizio(fine) nella pagina o nella
	// battuta.
	Bool Remove;
	// Viene utilizzato per gestire  lo STARTED di
	// ListaSillabe
	
	VUnit YSil[MAX_VIEW];
	// Rappresenta la coordinata y in cui deve
	// essere posizionato il testo della sillaba
	// sullo spartito.Nel caso assuma valore -1
	// significa testo non posizionato.	
	
	
	ProlLine *FirstLin[MAX_VIEW];
	// Indica la cima della lista dei segmenti
	// che costituiscono il prolungamento.
	ProlLine *PFirstLin[MAX_VIEW];
	// Inizializzato a NULL(non posizionato) in 
	// seguito indica la cima della lista dei
	// segmenti.
	ProlLine *PActLin[MAX_VIEW];
	// Indica il segmento attuale da posizionare.
	
	NumCode NumericCode;
	// Numero che identifica ogni sillaba dentro
	// la lista che la contiene.
	
	
public:
	Sillaba(const char *s);
	// Costruttore.
	~Sillaba();
	// Distruttore.
	
	void SetRemove(Bool b){Remove=b; };
	Bool GetRemove(void) {return Remove; };
	// Indicano se la sillaba estesa  stata
	// rimossa o meno dallo STARTED.Questo
	// utilizzato in ListaSillabe contiene
	// l'indirizzo della variabile estesa,
	// presente nella battuta(pagina) corrente
	// il cui prolungamento non  ancora
	// terminato.
	
	void SetListaBat(ListaBattute *a) {listaBat=a;};
	// Richiamata da ListaSillabe ogni volta
	// debba essere aggiunta una nuova sillaba,
	// permette di identificare la lista di
	// battute nella quale  contenuta la
	// figura associata alla sillaba.
	Figura *SetSilOcc(Figura *PCor,Figura *pFEnd,ListaFigure *pLayer,
		int InterLine,int a);
	// Per ogni figura della battuta corrente
	// a cui  associata la sillaba estesa setta
	// l'attributo che indica che la figura 
	// gi occupata.
	// Ritorna il puntatore alla successiva
	// figura oppure NULL se la battuta  terminata.
	
	void SetReset(Figura *pFStart,Figura *pFEnd,int Layer,
		Battuta *pBatCor,int InterLine,int a,Bool TipoProl);
	// Permette di settare ad a l'attributo
	   // AppSil delle figure associate ad una
	   // sillaba estesa anche nel caso di
	   // estensione a cavallo tra pi battute.
	   // A tal proposito per ogni battuta	
	   // richiama SetSilOcc.
	void SetPFig(Figura *pfStart, Figura *pfEnd,int Layer,int numtoken,
		Battuta *pBatCor,int InterLine,Bool Trat,Bool TraSil);
	// Associa alla sillaba i puntatori che
	// indicano le figura a cui si riferisce.
	// Sulla base dei puntatori(uguali o non)
	// e della sillaba di ingresso viene
	// settato Prol,ProlTrat,TraSil.
	// Inoltre ad ogni nota associata alla
	// sillaba setta l'attributo AppSil per
	// indicare che a quella figura  gi
	// associata una sillaba o il suo prolungamento.
	  	
	
	
	void SetPFigStart(Figura *pF);
	void SetPFigEnd(Figura *pF);
	// Settano i puntatori alle figure a cui
	//  associata la sillaba. 	
	Figura *GetPFigStart();
	Figura *GetPFigEnd();
	// Restituiscono i puntatori alle figure
	// di fine e inizio a cui  associata la
	// sillaba.
	
	
	ProlLine *GetPFirstLin();
	// Restituisce il puntatore al primo segmento
	// costituente il prolungamento da posizionare.  	
    
	Bool GetProl();
	// Restituisce una variabile booleana
	// che indica se la sillaba si estende
	// anche sulle note successive con un
	// prolungamento costituito da un tratto
	// continuo.
	Bool GetProlTrat();
	// Restituisce una variabile booleana
	// che indica se la sillaba si estende
	// anche sulle note successive con un
	// prolungamento costituito da un tratto
	// tratteggiato.
	Bool GetSegTraSil();
	// Restituisce una variabile booleana
	// che indica se la sillaba si estende
	// anche sulle note successive con un
	// prolungamento costituito da un  solo
	// trattino.
	int GetNumtoken();
	// Restituisce il numero di token che
	// prima di una cancellazione individuavano
	// il numero di note che assegnate al
	// prolungamento una di sillaba estesa.
	
	
	void SetProl(Bool a);
	// Setta l'attributo Prol booleano che
	// indica se la sillaba si estende o no
	// anche sulle note successive con un
	// prolungamento costituito da un tratto
	// continuo.
	void SetProlTrat(Bool a);
	// Setta l'attributo ProlTrat booleano
	// che indica se la sillaba si estende
	// o meno anche sulle note successive
	// con un prolungamento costituito da
	// un tratto tratteggiato.
	void SetSegTraSil(Bool a);
	// Setta l'attributo SegTraSil booleano
	// che indica se la sillaba si estende
	// o meno anche sulle note successive
	// con un prolungamento costituito da
	// un solo trattino.
	void SetNumtoken(int num);
	// Setta l'attributo Numtoken secondo
	// il numero passatogli dalla funzione
	// richiamante.
	
	char Getgs();
	void Setgs(char cr);
	int GetNumSep();
	char GetToken();
	void SetNumSep(int i);
	void SetToken(char tk);
	void SetText(unsigned char *txt);
	VUnit SetYStrofe(int ns,VUnit yy);
	void SetLine(int ns);
	int GetLine();
	void SetNLayer(int NL);
	int GetNLayer();
	Figura* ManagerLayer(Battuta *pBat,short CurrentLayer);
	void ManagerPlus(unsigned char *testo);
	VUnit DrawSilWithSlur(int ns);
	VUnit PrintText(); 
	VUnit PrintSilWithSlur(char D);
	void PrintSilProl(VUnit a,int ns); 
	VUnit PrintLin(ProlLine *PAct,VUnit x, int ns);
	void DrawSilWithMoreSlur(wxString tmp,Point Ini);
	void PrintSilWithMoreSlur(wxString tmp,Point Ini,VUnit Wd,char D);
	VUnit PrintSlur();
	bool GetAss();
	void SetAss(bool Is_Ass);
	double GetShift();
    void SetShift(double shft);
	
	
	
	
	
	
	////////////////////////////////////
	
	
	
    
	void ResetPosLin();
	// Posiziona PActLin in cima alla lista.
	void ResetFirstLin();
	
	void SetNumCode(NumCode nc);
	// Imposta il Codice Numerico della sillaba
	// che permette di individuarla come simbolo
	// grafico.
	NumCode GetNumCode();
	// Ritorna il numcode dell'oggetto in questione.
	
	void SetNumBat(short,Battuta *pb);
	Battuta *GetBattuta(short n);
	
	Bool GetInizio();
	void SetInizio(Bool);
	Bool GetFine();
	void SetFine(Bool);
	// Vengono utilizzate nei metodi delle classi
	// atte a gestire il posizionamento della
	// sillaba e dell'eventuale prolungamento.
	// Indicano se la sillaba estesa ha inizio
	// (termine) nella battuta o pagina corrente.
	
	void AddLin();
	// Aggiunge un segmento in coda alla lista dei
	// segmenti di cui  costituito il prolungamento.
	void DeallocaLin();
	// Dealloca tutta la lista dei segmenti che
	// costituiscono il prolungamento,tranne il
	// primo.
	
	Point GetAbsPos();
	// Ritorna la posizione assoluta del segmento
	// attivo.
	void SetAbsPosX(VUnit x);
	void SetAbsPosY(VUnit y);
	   // Impostano le coordinate del segmento.
	
	void SetWidth(VUnit w);
	void SetHeight(VUnit hgt);
	void SetDim(VUnit w);
	// Imposta la larghezza e altezza al segmento
	// attivo.
	void SetYSil(VUnit y);
	// Imposta la coordinata y che deve assumere
	// il testo della sillaba sullo spartito.
	VUnit GetYSil();
	// Ritorna la coordinata y che deve assumere
	// il testo della sillaba sullo spartito.
	VUnit GetVu2Rgt();
	// Ritorna la larghezza del segmento attivo.
	VUnit GetLeft();
	// Ritorna l'estremit sinistra del rettangolo
	// contenente il testo della sillaba.
	
	VUnit DrawText(int ns); 
	// Permette di disegnare sullo scherno il
	// testo di una sillaba.
	// Viene richiamata dal metodo DRAW della
	// classe ListaSillabe per disegnare una
	// non estesa.Viene richiamata dai metodi
	// successivi,invece,per disegnare il
	// testo di una sillaba con prolungamento.
	// A tal fine restituisce la larghezza del
	// testo della sillaba.
	void DrawSilProl(VUnit a,int ns);
	// Richiamata da DRAW di ListaSillabe disegna
	// una sillaba estesa con prolungamento a
	// a tratto continuo,tratteggiato oppure
	// costituito da un solo trattino.
	// Prende in ingresso il discostamento
	// dalla sillaba successiva.
	VUnit DrawLin(ProlLine *PAct,VUnit x, int ns); 
	// Restituisce la coordinata iniziale
	// del prolungamento in base alla
	// situazione corrente dello scorrimento
	// del pentagramma(stessa battuta,rigo..).
	
   	void LinkTogetherTxt(char *s,int Numtoken ,char Token);
	// Prende in ingresso un testo  di una
	// sillaba un numero di caratteri e un
	// token.Accoda al testo tanti caratteri
	// quanti specificati da NumToken.
	void  LinkTogetherTxt(char *s1,char *s2 ,char Token);
	// Prende in ingresso due testi ed un
	// separatore.
	// Crea un nuovo testo come unine dei
	// due separati dal token.Aggiorbna poi
	// il testo della sillaba.
};
#endif //SILLABA_HPP








