/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include "mdo/moodsmidiwrite.hpp"
#include "lioo.hpp"

MoodsMidiWrite::MoodsMidiWrite(): Midi_symbolic_interface()
{
	memset(&evt_info, 0, sizeof(MEVT_EXTRA_INFO));
};

void MoodsMidiWrite::orchestrazione(Partitura *pPart)
{	
	tprecdinamico=-1;
	velocitadiattacco=127;
		
	tempodinamico[0]=10;
	tempodinamico[1]=10;
	tempodinamico[2]=10;
	tempodinamico[3]=20;
	tempodinamico[4]=30;
	tempodinamico[5]=40;
	tempodinamico[6]=50;
	tempodinamico[7]=60;
	tempodinamico[8]=70;
	tempodinamico[9]=80;
	tempodinamico[10]=90;
	tempodinamico[11]=100;
	tempodinamico[12]=100;
	tempodinamico[13]=100;
	tempodinamico[14]=100;
	tempodinamico[15]=100;
	tempodinamico[16]=100;
	tempodinamico[17]=100;
	tempodinamico[18]=100;
	tempodinamico[19]=100;
	
	
	for(int i=0; i<33; i++)
	{
		orchestra[i][0] = 0;
		orchestra[i][1] = 127;
		orchestra[i][2] = i % 16;
	}
	
	firs1 = new listaindici;
	firs2 =firs1;   
	
	
	//Inserire il riempimento dei dati sugli strumenti da usare e il numero delle voci
	Spartito *tmp = pPart->GetFirstSpa();
	i = 0;
	int Midicode;
	while(tmp!=NULL)
	{
		if(tmp->IsVisible())
		{
			Midicode=tmp->GetMidiAttrib()->Get_MidiCode();
			orchestra[i][0] = Midicode;
			firs1->valore = i;
			firs1->next = new listaindici;
			firs1 = firs1->next;
			firs1->next = NULL;
			if(tmp->GetMidiAttrib()->Get_Muting())
				orchestra[i][1] = 0;
			else
				orchestra[i][1] = tmp->GetMidiAttrib()->Get_Volume();
			orchestra[i][2] = tmp->GetMidiAttrib()->Get_Channel();
			i++;
		}
		tmp = pPart->GetNextSpa(tmp);
		
	}
	
};

void MoodsMidiWrite::SetFlagPrevKeySign(tipoArmaturaChiave Archiave)
{
	
	int i=0;
	switch(Archiave)
	{
	case 0:
		break;
	case 1:     //case1-7 l'alterazione e' un bemolle quindi flagalterazione=1 perche' si deve sommare 1
		flagprecalter[3]=1;
		break;
	case 2:
		flagprecalter[3]=flagprecalter[0]=1;
		break;
	case 3:
		flagprecalter[3]=flagprecalter[0]=flagprecalter[4]=1;
		break;
	case 4:
		flagprecalter[3]=flagprecalter[0]=flagprecalter[4]=flagprecalter[1]=1;
		break;
	case 5:
		flagprecalter[3]=flagprecalter[0]=flagprecalter[4]=flagprecalter[1]=flagprecalter[5]=1;
		break;
	case 6:
		for(i=0;i<6;i++)
			flagprecalter[i]=1;
		break;
	case 7:
		for(i=0;i<7;i++)
			flagprecalter[i]=1;
		
		break;
	case 8:  //case 8-14 l'alterazione e' un diesis dunque si deve sottrarre 1
		flagprecalter[6]=-1;
		break;
	case 9:
		flagprecalter[6]=flagprecalter[2]=-1;
		break;
	case 10:
		flagprecalter[6]=flagprecalter[2]=flagprecalter[5]=-1;
		break;
	case 11:
		flagprecalter[6]=flagprecalter[2]=flagprecalter[5]=flagprecalter[1]=-1;
		break;
	case 12:
		flagprecalter[6]=flagprecalter[2]=flagprecalter[5]=flagprecalter[1]=flagprecalter[4]=-1;
		break;
	case 13:
		flagprecalter[6]=flagprecalter[2]=flagprecalter[5]=flagprecalter[1]=flagprecalter[4]=flagprecalter[0]=-1;
		break;
	case 14:
		for(i=0;i<7;i++)
			flagprecalter[i]=-1;
		break;
	}
}


/////////////////////////////////////////////////

void MoodsSequencerWrite::tempo(DWORD _tempo)
{
	if (!found_tempo) 
	{
		found_tempo=true;
		p_seq->Set_act_tempo(_tempo);
	}
	
	MoodsMidiWrite::tempo(_tempo);
}

MoodsSequencerWrite::MoodsSequencerWrite(Midi_sequencer* _p_seq)
{
	found_tempo=false;
	p_seq=_p_seq;
	Set_p_struct(&p_seq->Get_working_struct());
	p_seq->Set_PPQN(480);
}

void MoodsSequencerWrite::noteon(BYTE channel, BYTE pitch, BYTE velocity)
{
	Midi_symbolic_interface::noteon(channel, pitch, velocity);
	Midi_event* p_evt=&((*p_struct)[act_track].Last());
	p_evt->set_moodsinfo(&evt_info);
}

void MoodsSequencerWrite::noteoff(BYTE channel, BYTE pitch, BYTE velocity)
{
	Midi_symbolic_interface::noteoff(channel, pitch, velocity);
	Midi_event* p_evt=&((*p_struct)[act_track].Last());
	p_evt->set_moodsinfo(&evt_info);
}

/////////////////////////////////////////////////