/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */




#include "mil/immissione.hpp"
#include "mil/rulemanager.hpp"
#include "mil/m.tab.h"
#include "liooext.hpp"


extern int DebugNormalMilla;
extern int DebugNormalMillaPos;
#include "wx/wx.h"
extern wxString msgdebug;
extern wxString msgcond;

int dumpcontrolloimm=0;
////////////////


ITEMIMM Immissione::ItemListImm[MAXNUMITEM];
CONDITIONIMM Immissione::ConditionListImm[MAXNUMCONDITION];
int Immissione::CurrentConditionImmUsage = 0;
int Immissione::CurrentItemImmUsage = 0;


void Errore(char * comm, char * nom, int num);
RuleManager ruleManager;

Immissione::Immissione()
{
	CurrentConditionImmUsage = 0;
	CurrentItemImmUsage = 0;
}

/*****************************************************************************/

/* Compare functions for the binary searchs on Keywords or Identifiers */
int  CompareItemImm(const void *  Element1, const void *  Element2 )
{ 
	return (strcmp( ((PITEMIMM) Element1)->ItemName,
		((PITEMIMM) Element2)->ItemName)); 
}
/*****************************************************************************/

int CompareSymbolImm(const void *  Element1, const void *  Element2 )
{ 
	return (((PITEMIMM)Element1)->Symbol - ((PITEMIMM)Element2)->Symbol);
}

/***************************************************************************/
/* Funzione che, se trova due condizioni facenti riferimento allo 
stesso simbolo, confronta il numero di condizioni. Altrimenti
confronta le condizioni */

int CompareConditionImm(const void *  Element1, const void *  Element2 )
{ 
	int conf;
	conf = (((PCONDITIONIMM) Element1)->Symbol - 
		((PCONDITIONIMM) Element2)->Symbol); 
	if (conf ==0)
	{
		return (((PCONDITIONIMM) Element1)->numCond - 
            ((PCONDITIONIMM) Element2)->numCond);
	}
	else return conf;	   	   
}

/******************************************************************/

int CompareConditionCondImm(const void *  Element1, const void *  Element2 )
{ 
	return (((PCONDITIONIMM) Element1)->numCond - 
		((PCONDITIONIMM) Element2)->numCond);	   	   
}

/*************************************************************************/

int CompareConditionSyImm(const void *  Element1, const void *  Element2 )
{ 
	
	return (((PCONDITIONIMM) Element1)->Symbol - 
		((PCONDITIONIMM) Element2)->Symbol);    	   
}


/*******************************************************************************/

void Immissione::Reset()
{
	CurrentConditionImmUsage = 0;
	CurrentItemImmUsage = 0;
}

void Immissione::InitRuleImm()
{
	int i;
	PCONDITIONIMM ConditionImm;
	
	for (i=0; i<CurrentConditionImmUsage; i++)
	{
		ConditionImm = &ConditionListImm[i];
		ConditionImm->Rule = searchRuleImm(ConditionImm->RuleName);
	 if(ConditionImm->Rule!=NULL)
		ConditionImm->Symbol = (ConditionImm->Rule)->Symbol;
	 else
		ConditionImm->Symbol = 99;
	}   
}


/*****************************************************************************/

PITEMIMM Immissione::AddItemImm(char * Item, int ItemType)
{
	
	ITEMIMM CurrentItemImm;
	
	ItemListImm[CurrentItemImmUsage].ItemName = (char *)malloc(strlen(Item) + 1);
	if (ItemListImm[CurrentItemImmUsage].ItemName==NULL)
		Errore("allocation","ItemName",0);
	strcpy(ItemListImm[CurrentItemImmUsage].ItemName, Item);
	
	ItemListImm[CurrentItemImmUsage].ItemType = ItemType;
	ItemListImm[CurrentItemImmUsage].Type = 99;
	ItemListImm[CurrentItemImmUsage].Symbol = 99;
	ItemListImm[CurrentItemImmUsage].rel = 99;
	ItemListImm[CurrentItemImmUsage].cat = 99;
	
	
	CurrentItemImmUsage++;
	
	/* Sort the list for future search */
	qsort((char *)&ItemListImm[0], CurrentItemImmUsage, sizeof(ITEMIMM),
        CompareItemImm);
		/* Adesso cerco dove e' stata posizionata la mia regola dopo il qsort
	altrimenti non so dove mi mette gli altri campi della regola*/
	CurrentItemImm.ItemName = Item;
	return (ITEMIMM *) bsearch(&CurrentItemImm, &ItemListImm[0], 
		CurrentItemImmUsage, sizeof(ITEMIMM), CompareItemImm);
	
}

/**********************************************************************/

PCONDITIONIMM Immissione::AddConditionImm()
{
	for(int i=0;i<NUM_MAX_CONDIZIONI;i++)
	{
		ConditionListImm[CurrentConditionImmUsage].condizioni[i].conditionrel.f1=99;
		ConditionListImm[CurrentConditionImmUsage].condizioni[i].conditionrel.Op=99;
		ConditionListImm[CurrentConditionImmUsage].condizioni[i].conditionrel.f2=99;
		ConditionListImm[CurrentConditionImmUsage].condizioni[i].conditionrel.res=99;
		ConditionListImm[CurrentConditionImmUsage].condizioni[i].conditionrel.num=99;   
		ConditionListImm[CurrentConditionImmUsage].condizioni[i].range.max=99;
		ConditionListImm[CurrentConditionImmUsage].condizioni[i].range.min=-99;
		ConditionListImm[CurrentConditionImmUsage].condizioni[i].cond=99;
	}
	
	ConditionListImm[CurrentConditionImmUsage].Symbol=99;
	ConditionListImm[CurrentConditionImmUsage].numCond=0;
	CurrentConditionImmUsage++;
	
	return &ConditionListImm[CurrentConditionImmUsage-1];
}

/******************************************************************/    

Bool Immissione::ImmRuleGen(int IDsym, Nota* pNota, Battuta* pBattuta,
							DrawObject* drw, short current_layer)
{
	PITEMIMM RuleImm;
	int IDgroup;
	
	char *groupname;
	
	groupname=simboli.GetGroup(IDsym);
	IDgroup=simboli.GetGroupPos(groupname);
	
	RuleImm = searchConditionImm(IDgroup+1000, pNota, pBattuta, drw, current_layer,0);
	
	
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		wxString tmp;
		if (RuleImm!=NULL)
		{
			RuleImm = searchConditionImm(IDgroup+1000, pNota, pBattuta, drw, current_layer,1);
			
			tmp.sprintf("Applied Rule: %s\n",
				RuleImm->ItemName);
			msgdebug.Append(tmp);
		}
	}
	
	if(RuleImm!=NULL)
	{
		if (RuleImm->cat==STEMUP) return TRUE;
		if (RuleImm->cat==STEMDOWN) return FALSE;
		if (RuleImm->cat==OPPOSITETOSTEM && pNota->GetLGambo()>0) return FALSE;
		if (RuleImm->cat==OPPOSITETOSTEM && pNota->GetLGambo()<0) return TRUE;
		
		if (RuleImm->cat==OPPOSITETOSTEM && pNota->GetLGambo()==0 &&
			pNota->GetAltezza()>=4) return TRUE;
		if (RuleImm->cat==OPPOSITETOSTEM && pNota->GetLGambo()==0 &&
			pNota->GetAltezza()<4) return FALSE;
		if (RuleImm->cat==ABOVE) return TRUE;
		
		if (RuleImm->cat==ONSTEM && pNota->GetLGambo()>0) return TRUE;  
		if (RuleImm->cat==ONSTEM && pNota->GetLGambo()<0) return FALSE;
	}
	return FALSE;
}    

/******************************************************************/    

Bool Immissione::ImmRule(ClassID ID, Nota* pNota, Battuta* pBattuta,
						 DrawObject* drw, short current_layer)
{
	
	PITEMIMM RuleImm;
    
	RuleImm = searchConditionImm(tab[ID], pNota, pBattuta, drw, current_layer,0); 
	
	
	
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		wxString tmp;
		if (RuleImm!=NULL)
		{
			RuleImm = searchConditionImm(tab[ID], pNota, pBattuta, drw, current_layer, 1);
			
			tmp.sprintf("Applied Rule: %s\n",
				RuleImm->ItemName);
			msgdebug.Append(tmp);
		}
	}
	
	
	
	if(RuleImm!=NULL)
	{
		if (RuleImm->cat==STEMUP) return TRUE;
		if (RuleImm->cat==STEMDOWN) return FALSE;
		if (RuleImm->cat==BEAM12) return 2;
		if (RuleImm->cat==BEAM23) return 3;
		if (RuleImm->cat==OPPOSITETOSTEM && pNota->GetLGambo()>0) return FALSE;
		if (RuleImm->cat==OPPOSITETOSTEM && pNota->GetLGambo()<0) return TRUE;
		
		if (RuleImm->cat==OPPOSITETOSTEM && pNota->GetLGambo()==0 &&
			pNota->GetAltezza()>=4) return TRUE;
		if (RuleImm->cat==OPPOSITETOSTEM && pNota->GetLGambo()==0 &&
			pNota->GetAltezza()<4) return FALSE;
		if (RuleImm->cat==ABOVE) return TRUE;
		
		if (RuleImm->cat==ONSTEM && pNota->GetLGambo()>0) return TRUE;  
		if (RuleImm->cat==ONSTEM && pNota->GetLGambo()<0) return FALSE;
	}  
	return FALSE;
}    


/***************************************************************************/

PITEMIMM Immissione::searchConditionImm(int a, Nota* pNota, Battuta* pBattuta,
										DrawObject* drw, short current_layer, int traceconditionsmilla)
{
	
	
	Bool result=FALSE; 
	msgcond.sprintf("");	
	/////////////////////////////////
	
	
	PCONDITIONIMM SearchConditionImm;
	
	CONDITIONIMM CurrentConditionImm;
	CurrentConditionImm.Symbol = a;
	
	SearchConditionImm= (CONDITIONIMM *) bsearch(&CurrentConditionImm, 
		&ConditionListImm[0], CurrentConditionImmUsage, sizeof(CONDITIONIMM), 
		CompareConditionSyImm);
	
	if (SearchConditionImm!=NULL)
	{   
		while (SearchConditionImm-ConditionListImm<CurrentConditionImmUsage &&
			SearchConditionImm->Symbol==a)
		{
			SearchConditionImm++;
		}
	}  
	
	if (SearchConditionImm!=NULL)
	{   
		SearchConditionImm--;
		
		while (SearchConditionImm-ConditionListImm>=0 && 
			SearchConditionImm->Symbol==a)
		{
			result=FALSE; 
			int j;  
			for(j=0; j<NUM_MAX_CONDIZIONI; j++)
			{ 
				if (SearchConditionImm->condizioni[j].cond!=99)
				{
					result = ValutaCondizioni(&SearchConditionImm->condizioni[j], pNota,
						pBattuta, drw, current_layer,TRUE, traceconditionsmilla);
//FFF					if(traceconditionsmilla)
//FFF						msgDebugMILLA.Append(msgCond);					
					if(!result) break;
				}
				else 
					break;
			}  
			
/*FFF			if(traceconditionsmilla)
			{
				if(result)
					msgDebugMILLA.Append(wxString::Format("Applied Rule: %s", SearchConditionImm->Rule->ItemName ));
				else
					msgDebugMILLA.Append("No rule found!");
				
				wxString msgToken=ValutaCondizioniDebug(pNota, pBattuta, drw, current_layer, TRUE);
				
				WDebugMilla(title,msgDebugMILLA,msgToken);
			} FFF*/
			if (result)
			{
				msgdebug.Append(msgcond);	
				// simbolo per milla debug
				wxString pip;
				msgdebug.Append(pip);
				/////////////////////////////////
				
				return SearchConditionImm->Rule;
			}  
			
			SearchConditionImm--;     
			msgcond.sprintf("");	
			/////////////////////////////////
			
		}
	}
	return NULL;
	
}

/*************************************************************************/

PITEMIMM Immissione::searchRuleImm(char* ruleName)
{
	
	ITEMIMM CurrentItemImm;
	CurrentItemImm.ItemName = ruleName;
	
	return (ITEMIMM *) bsearch(&CurrentItemImm, &ItemListImm[0], 
		CurrentItemImmUsage, sizeof(ITEMIMM), CompareItemImm); 
	
} 

/*****************************************************************************/

PITEMIMM Immissione::SearchNameImm(char *name)
{ 
	ITEMIMM item;
	item.ItemName = name;
	return (ITEMIMM *) bsearch(&name, &ItemListImm[0],
		CurrentItemImmUsage, sizeof(ITEMIMM), CompareItemImm);
}  

/*********************************************************************/

void Immissione::sortConditionImm(void)
{
	qsort((char *)&ConditionListImm[0], CurrentConditionImmUsage, 
		sizeof(CONDITIONIMM), CompareConditionImm);
}  

/*****************************************************************************
* show the list of Item di immissione
*****************************************************************************/
void Immissione::seeItemListImm(void )
{ 
	int i;
	for (i=0;i<CurrentItemImmUsage;i++)
    {  
		fprintf(stdout,"Rulename %s, RuleType %d, Type %d ",
			ItemListImm[i].ItemName,
			ItemListImm[i].ItemType,
			ItemListImm[i].Type);       
		if(ItemListImm[i].Symbol!=99)
		{
			fprintf(stdout, "Symbol %d ",ItemListImm[i].Symbol);
		}
		if(ItemListImm[i].rel!=99)
		{
			fprintf(stdout, "rel %d ",ItemListImm[i].rel);
		}	 
		if(ItemListImm[i].cat!=99)
		{
			fprintf(stdout, "cat %d \n",ItemListImm[i].cat);
		}    
    }
}  


/*****************************************************************************
* show the list of ConditionImm
*****************************************************************************/
void Immissione::seeConditionListImm(void )
{ 
	int i;
	for (i=0;i<CurrentConditionImmUsage;i++)
    { 
		int j; 
		for (j=0; j<NUM_MAX_CONDIZIONI; j++)
		{
			if(ConditionListImm[i].condizioni[j].cond!=99)
				fprintf(stdout, "C:%d ",ConditionListImm[i].condizioni[j].cond);
			
			if(ConditionListImm[i].condizioni[j].range.min!=-99)
				fprintf(stdout, "X:%g ",ConditionListImm[i].condizioni[j].range.min);
			
			if(ConditionListImm[i].condizioni[j].range.max!=99)
				fprintf(stdout, "Y:%g ",ConditionListImm[i].condizioni[j].range.max);
			
			if(ConditionListImm[i].condizioni[j].conditionrel.f1!=99)
				fprintf(stdout, "f1:%d ",ConditionListImm[i].condizioni[j].conditionrel.f1);
			
			if(ConditionListImm[i].condizioni[j].conditionrel.Op!=99)
				fprintf(stdout, "Op:%d ",ConditionListImm[i].condizioni[j].conditionrel.Op); 
			
			if(ConditionListImm[i].condizioni[j].conditionrel.f2!=99)
				fprintf(stdout, "f2:%d ",ConditionListImm[i].condizioni[j].conditionrel.f2);
			
			if(ConditionListImm[i].condizioni[j].conditionrel.res!=99)
				fprintf(stdout, "res:%d ",ConditionListImm[i].condizioni[j].conditionrel.res); 
			
			if(ConditionListImm[i].condizioni[j].conditionrel.num!=99)
				fprintf(stdout, "num:%lf ",ConditionListImm[i].condizioni[j].conditionrel.num); 	  
		}
		fprintf(stdout," RuleName: %s,"
			" Simbolo: %d,"
			" NumCondiz.: %d\n",
			ConditionListImm[i].RuleName,
			ConditionListImm[i].Symbol,   
			ConditionListImm[i].numCond); 
    }
	
}

/*************************************************************************/   

void Immissione::sortSymbolImm(void)
{
	qsort((char *)&ItemListImm[0], CurrentItemImmUsage, sizeof(ITEMIMM), 
		CompareSymbolImm);  
}    

/**************************************************************************/

void Immissione::sortRuleImm(void)
{
	qsort((char *)&ItemListImm[0], CurrentItemImmUsage, sizeof(ITEMIMM), 
		CompareItemImm);  
}    


void Immissione::searchImmGen(int a)
{
}


void Immissione::dump(void)
{
	
	int j=0, k=0;
	int controllo=0;
	
	FILE *pFileDati;
	if ((pFileDati=fopen ("C://Debug/dumpmillaimmissione.dat","w"))==NULL)
		printf ("Errore nell'apertura del file dumpmillaimmissione.dat\n");
	else {
		fprintf(pFileDati,"\n\nREGOLE DI IMMISSIONE\n");
		fprintf(pFileDati,"\nItemName ItemType Type Symbol rel cat ");
		
		for (j=0; j<MAXNUMITEM; j++)
		{
			
			if (ItemListImm[j].ItemName!=NULL)
				fprintf(pFileDati,"\n[ %s,  %d,  %d,  %d, %d, %d ] ",ItemListImm[j].ItemName, ItemListImm[j].ItemType, ItemListImm[j].Type, ItemListImm[j].Symbol, ItemListImm[j].rel, ItemListImm[j].cat );
		}
		fprintf(pFileDati,"\n\nCONDIZIONI DI IMMISSIONE\n");
		
		for (j=0; j<MAXNUMCONDITION; j++)
		{
			if (ConditionListImm[j].RuleName!=NULL)
			{	
				
				fprintf(pFileDati,"\n\n RuleName, Symbol,  numCond");
				fprintf(pFileDati,"\n[ %s,  %d,  %d ] ", ConditionListImm[j].RuleName, ConditionListImm[j].Symbol, ConditionListImm[j].numCond );
				fprintf(pFileDati,"\nSingoli Token verificati");
				fprintf(pFileDati,"\ncondizioni[%d].cond, condizioni[%d].RANGEmin, condizioni[%d].RANGEmax condizioni[%d].conditionrel.f1, condizioni[%d].conditionrel.Op, condizioni[%d].conditionrel.f2, condizioni[%d].conditionrel.res, condizioni[%d].conditionrel.num", k, k, k, k, k, k, k, k, k);
				
				for (k=0; k<ConditionListImm[j].numCond; k++)
				{				
					fprintf(pFileDati,"\n[ %d, %f, %f, %d, %d, %d, %d, %f ] ",ConditionListImm[j].condizioni[k].cond, ConditionListImm[j].condizioni[k].range.min, ConditionListImm[j].condizioni[k].range.max, ConditionListImm[j].condizioni[k].conditionrel.f1, ConditionListImm[j].condizioni[k].conditionrel.Op, ConditionListImm[j].condizioni[k].conditionrel.f2, ConditionListImm[j].condizioni[k].conditionrel.res, ConditionListImm[j].condizioni[k].conditionrel.num);
				}
			}
		}
	}
	fflush(pFileDati);

	controllo=fclose (pFileDati);
}  
