/************************************************************************/
/*  Progetto......: MOODS                                               */
/*  Modulo........: lioo loader                                         */
/*  File..........: lgiust.l                                            */
/*  Autore........: Timna Panfietti Monaco Riccardo Della Santa         */
/*  Data revisione: 09/09/98                                            */
/*  Descrizione:    Analizzatore lessicale                              */ 
/************************************************************************/
 

%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "mil/lgiust.h"
#include "mil/immissione.hpp"
#include "mil/rulemanager.hpp"
#include "mil/posiz.hpp"
#include "mil/priorita.hpp"
#include "mil/simboli.hpp"

#include "mil/m.tab.h" 

extern "C" 
{
 int milwrap()
 {
   return 1;
 }
} 
/* #define LEXONLY */

/* Questa roba serve per il debugging */
#ifdef LEXONLY
  #define RETURNIT(a,b) {printf("%s :\"%s\"\n",(b), miltext);}
#else
  #define RETURNIT(a,b) {return((a));}
#endif

/*********************************************************************/
/* function prototypes */
int MilCompareId(const void * Element1, const void * Element2);
void Errore(char * comm, char * nom, int num);

void action_keyword(void);
void AddToken(void);
int new_item(int * ident, char * comm);
int new_condition(int * ident, char * comm);

/*********************************************************************/

static KEYWORDMIL  CurrentKeyword;
static PKEYWORDMIL PKeyword;

KEYWORDMIL MilKeywords[] = {  
  {RELNOTA, "RELNOTA"},
  {RELNOTA, "RELNOTE"},
  {RELPAUSA, "RELPAUSA"},
  {RELPAUSA, "RELREST"},

  {RELSTAFF, "RELSTAFF"},
  {RELNOTES, "RELNOTES"},
  {DEFAULTX, "DEFAULT"},
  {ALTERNATIVE, "ALTERNATIVE"},
  {ONLEFT, "ONLEFT"},
  {ONRIGHT, "ONRIGHT"},
  {BELOW, "BELOW"},
  {ABOVE, "ABOVE"},
  {STEMUP, "STEMUP"},
  {STEMDOWN, "STEMDOWN"},
  {BEAM12, "BEAM12"},
  {BEAM23, "BEAM23"},
  {INMULTISTAFF, "INMULTISTAFF"},
  {STAFFNUMBER, "STAFFNUMBER"},
  {ONSTAFFNUMBER, "ONSTAFFNUMBER"},
  {STAFFLINES, "STAFFLINES"},
  {SAMESTEMONLEFT, "SAMESTEMONLEFT"},
  {SAMESTEMONRIGHT, "SAMESTEMONRIGHT"},
  {AVOIDINSTAFF, "AVOIDINSTAFF"},
  {OUTSIDESTAFF, "OUTSIDESTAFF"},
  {ANYWHERE, "ANYWHERE"},
  {ONSPACE, "ONSPACE"},
  {ONLINE, "ONLINE"},
  {ONSTAFF, "ONSTAFF"},
  {DOWNWARDS, "DOWNWARDS"},
  {UPWARDS, "UPWARDS"},
  {PAUSA, "PAUSA"},
  {PAUSA, "REST"},

  {NEARNOTA, "NEARNOTA"},
  {NEARNOTA, "NEARNOTE"},

  {PSEMIBREVE, "PSEMIBREVE"},
  {PSEMIBREVE, "REST1"},

  {PMINIMA, "PMINIMA"},
  {PMINIMA, "REST1_2"},

  {THEN, "THEN"},
  {immIF, "immIF"},
  {immIF, "insIF"},

  {posIF, "posIF"},
  {INMULTIVOICEUPPER, "INMULTIVOICEUPPER"},
  {INMULTIVOICELOWER, "INMULTIVOICELOWER"},
  {INENDBATTUTA, "INENDBATTUTA"},
  {INENDBATTUTA, "INENDMEASURE"},

  {INBEAM, "INBEAM"},
  {INTUPLET, "INTUPLET"},
  {INPUNTOVALORE, "INPUNTOVALORE"},
  {INPUNTOVALORE, "INAUGDOT"},

  {INCHORDSUSPENDED, "INCHORDSUSPENDED"},
  {INSTACCATO, "INSTACCATO"},
  {INLEGVALORE, "INLEGVALORE"},
  {INLEGVALORE, "INTIE"},

  {CORONA, "CORONA"},
  {CORONA, "FERMATA"},

  {NOT, "NOT"},
  {NOTE, "NOTE"},
  {NEARNOTA, "NEARNOTA"},
  {NEARNOTA, "NEARNOTE"},
 
  {NEARNOTES, "NEARNOTES"},
  {INCHORD, "INCHORD"},
  {INMULTIVOICE, "INMULTIVOICE"},
  {OPPOSITETOSTEM, "OPPOSITETOSTEM"},
  {ONSTEM, "ONSTEM"},
  {SUSPENDED, "SUSPENDED"},
  {UPPER, "UPPER"},
  {LOWER, "LOWER"},
  {INTERMEDIATE, "INTERMEDIATE"},
  {CAMBIOCHIAVE, "CAMBIOCHIAVE"},
  {CAMBIOCHIAVE, "CLEFCHANGE"},


  {AND, "AND"},
  {OR, "OR"},
  {XOR, "XOR"},
  {APP, "APP"},
  {ACC, "ACC"},
  {ACC, "ACCIACC"},
  
  {GRUP, "GRUP"},
  {GRUP, "TURN"},

  {MORD, "MORD"},
  {MORD, "MORDENT"},
  
  {TRILLO, "TRILLO"},
  {TRILLO, "TRILL"},

  {TREM, "TREM"},
  {TREM, "TREMOLO"},

  {ARP, "ARP"},
  {ARP, "ARPEGGIO"},

  {GLISS, "GLISS"},
  {GLISS, "GLISSANDO"},

  {DIESIS, "DIESIS"},
  {DIESIS, "SHARP"},

  {BEMOL, "BEMOL"},
  {BEMOL, "FLAT"},

  {DOPDIES, "DOPDIES"},
  {DOPDIES, "DSHARP"},

  {DOPBEMOL, "DOPBEMOL"},
  {DOPBEMOL, "DFLAT"},

  {MONES, "MONES"},
  {MONES, "SHARP1Q"},

  {TRIES, "TRIES"},
  {TRIES, "SHARP3Q"},

  {LEG, "LEG"},
  {LEG, "SLUR"},

  {STAC, "STAC"},
  {STAC, "STACCATO"},

  {SFOR, "SFOR"},
  {SFOR, "SFORZATO"},

  {ACCE, "ACCE"},
  {ACCE, "ACCENTO"},

  {MARTF, "MARTF"},
  {MARTF, "MARTELLATO"},

  {TEN, "TEN"},
  {TEN, "TENUTO"},

  {RULEIMM, "RULEIMM"},
  {RULEIMM, "RULEINS"},
  
  {RULEPOS, "RULEPOS"},
  {MART, "MART"},
  {MART, "MARTDOLCE"},

  {PUNTOALL, "PUNTOALL"},
  {PUNTOALL, "PUNTOALLUNGATO"},
  
  {LEGVAL, "LEGVAL"},
  {LEGVAL, "SLUR"},

  {PUNTOVAL, "PUNTOVAL"},
  {PUNTOVAL, "AUGDOT"},
  
  {SCALE, "SCALE"},
  {ANGLE, "ANGLE"},
  {STEM, "STEM"},
  {PSEMIMINIMA, "PSEMIMINIMA"},
  {PSEMIMINIMA, "REST1_4"},
  
  {PCROMA, "PCROMA"},
  {PCROMA, "REST1_8"},

  {PSEMICROMA, "PSEMICROMA"},
  {PSEMICROMA, "REST1_16"},

  {PBISCROMA, "PBISCROMA"},
  {PBISCROMA, "REST1_32"},

  {PSEMIBISCROMA, "PSEMIBISCROMA"},
  {PSEMIBISCROMA, "REST1_64"},

  {DELTAX, "DX"},
  {DELTAY, "DY"},
  {NUMDOWN, "NUMDOWN"},
  {NUMUP, "NUMUP"},
  {VAR, "VAR"},
  {LENGHT, "LENGHT"},
  {LENGHT, "LENGTH"},
  
  {WITH, "WITH"},
  {BEAM, "BEAM"},
  {INCHORDTRE, "INCHORDTRE"},
  {INCHORDTRE, "INCHORDTHREE"},
 
  {ALTERAZIONE, "ALTERAZIONE"},
  {ALTERAZIONE, "ACCIDENTAL"},

  {INCHORDSETTIMA, "INCHORDSETTIMA"},
  {INCHORDSETTIMA, "INCHORDSEVENTH"},

  {INCHORDminSETTIMA, "INCHORDminSETTIMA"},
  {INCHORDminSETTIMA, "INCHORDminSEVENTH"},

  {INCHORDSUSPINTERM, "INCHORDSUSPINTERM"},
  {INCHORDSUSPUPPER, "INCHORDSUSPUPPER"},
  {INCHORDSUSPLOWER, "INCHORDSUSPLOWER"},
  {ARCATA, "ARCATA"},
  
  {PIZZICATO, "PIZZICATO"},
  {CORDA, "CORDA"},
  {CORDA, "STRING"},

  {ARCATASU, "ARCATASU"},
  {ARCATASU, "BOWUP"},
 
  {ARCATAGIU, "ARCATAGIU"},
  {ARCATAGIU, "BOWDOWN"},

  {PUNTA, "PUNTA"},
  {TALLONE, "TALLONE"},
  {PONTICELLO, "PONTICELLO"},
  {TASTIERA, "TASTIERA"},
  {ARCO, "ARCO"},
  {OTTAVA, "OTTAVA"},
  {OTTAVA, "OCTAVE"},
  
  {INOTTAVASOPRA, "INOTTAVASOPRA"},
  {INOTTAVASOPRA, "IN8VA"},
 
  {INOTTAVASOTTO, "INOTTAVASOTTO"},
  {INOTTAVASOTTO, "IN8BA"},
  
  {DITEG, "DITEG"},
  {DITEG, "FINGER"},

  {INTWOSTAFF, "INTWOSTAFF"},
  {INSINGLELAYER, "INSINGLELAYER"},
  {NOTAWITH, "NOTAWITH"},
  {NOTAWITH, "NOTEWITH"},
 
  {NOTES, "NOTES"},
  {BISCROMA, "BISCROMA"},
  {BISCROMA, "NOTE1_32"},

  {FUSA, "FUSA"},
  {FUSA, "NOTE1_128"},

  {SEMIBISCROMA, "SEMIBISCROMA"},
  {SEMIBISCROMA, "NOTE1_64"},
  
  {RULEPRI, "RULEPRI"},
  {STEMHEIGHT, "STEMHEIGHT"},
  {CROMA, "CROMA"},
  {CROMA, "NOTE1_8"},
  
  {SEMIMINIMA, "SEMIMINIMA"},
  {SEMIMINIMA, "NOTE1_4"},

  {MINIMA, "MINIMA"},
  {MINIMA, "NOTE1_2"},

  {SEMIBREVE, "SEMIBREVE"},
  {SEMIBREVE, "NOTE1"},

  {BREVE, "BREVE"},
  {BREVE, "NOTE2"},

  {NOTINA, "NOTINA"},
  {NOTINA, "SMALLNOTE"},
  
  {SEMICROMA, "SEMICROMA"},
  {SEMICROMA, "NOTE1_16"},

  {MOBEMOL, "MOBEMOL"},
  {MOBEMOL, "FLAT1Q"},

  {TRIBEMOL, "TRIBEMOL"},
  {TRIBEMOL, "FLAT3Q"},

  {FIRSTUPPER, "FIRSTUPPER"},
  {FIRSTUPPER, "FIRSTUPEST"},

  {FIRSTLOWER, "FIRSTLOWER"},
  {FIRSTLOWER, "FIRSTLOWEST"},

  {PRECEED, "PRECEED"},
  {NBEAT, "NBEAT"},
  {TIME, "TIME"},
  {BATTUTA, "BATTUTA"},
  {BATTUTA, "MEASURE"},
 
  {COPPIE, "COPPIE"},
  {COPPIE, "COUPLES"},

  {OFF, "OFF"},
  {ON, "ON"},
  {GTinBEAT, "GTinBEAT"},
  {GREATER, "GREATER"},
  {LOWer, "LOWer"},
  {LOWer, "LOWER"},

  {GREATERe, "GREATERe"},
  {GREATERe, "GREATEREQ"},

  {LOWERe, "LOWERe"},
  {LOWERe, "LOWEREQ"},

  {UPPERD, "UPPERD"},
  {UPPERD, "UPESTD"},
   
  {LOWERD, "LOWERD"},
  {LOWERD, "LOWESTD"},

  {NUMUP, "NUMUP"},
  {NUMDOWN, "NUMDOWN"},
  {MEAN, "MEAN"},
  {DELTA, "DELTA"},
  {BEAMDIS, "BEAMDIS"},
  {MHL, "MHL"},
  {PLUS, "PLUS"},
  {MIN, "MIN"},
  {DEFGROUP, "DEFGROUP"},
  {DEFSYM, "DEFSYM"},
  {CODE, "CODE"},
  {DIM, "DIM"},
  {GROUP, "GROUP"},
  {CONSORD, "CONSORD"},
  {CONSORD, "WITHMUTE"},

  {VIASORD, "VIASORD"},
  {VIASORD, "WITHOUTMUTE"},

  {ARM, "ARM"},
  {ARM, "HARM"},

  {HEAD,"HEAD"}, 
  {CLASSIC, "CLASSIC"},
  {ALPHANUM, "ALPHANUM"},
  {ALPHANUM_SQUARE, "ALPHANUM_SQUARE"},
  {ALPHANUM_REVERSE, "ALPHANUM_REVERSE"},
  {CIRCLEX, "CIRCLEX"},
  {CLUSTER, "CLUSTER"},
  {X_HEAD, "X_HEAD"},
  {CROIX, "CROIX"},
  {DDIESIS, "DDIESIS"},
  {DDIESIS, "HSHARP"},
 
  {DIAMOND, "DIAMOND"},
  {TRIANG, "TRIANG"},
  {TRIANG_UP, "TRIANG_UP"},
  {TRIANG_DOWN, "TRIANG_DOWN"},
  {TRIANG_LEFT, "TRIANG_LEFT"},
  {TRIANG_RIGHT, "TRIANG_RIGHT"},
  {TRIANG_ROUND, "TRIANG_DOWN"},
  {MOON, "MOON"},
  {PLUS_HEAD, "PLUS_HEAD"},
  {RHYTHMIC, "RHYTHMIC"},
  {SQUARE, "SQUARE"},
  
  {STEMSTART,"STEMSTART"}
};

ITEMPOS CurrentItemPos;
PITEMPOS PItemPos;
PITEMPOS CurrentRulePos=NULL;


ITEMIMM CurrentItemImm;
PITEMIMM PItemImm;
PITEMIMM CurrentRuleImm=NULL;


RANGE CurrentRange;

CONDITIONIMM CurrentConditionImm;
PCONDITIONIMM PConditionImm;
PCONDITIONIMM CurrentCondImm;

CONDITIONPOS CurrentConditionPos;
PCONDITIONPOS PConditionPos;
PCONDITIONPOS CurrentCondPos;

extern Immissione* pimm;
extern Posiz* ppos;
extern Priorita* Ppri;
extern Simboli* psym;

PNEWSYMBOL pnewsym;


int CurrentColImm=0;
int CurrentColPos=0;

/* Numero delle parole chiave */
int MilKeywordNumber = sizeof(MilKeywords)/sizeof(KEYWORDMIL);

/*****************************************************************************/
/* variabili per la gestione della semantica*/
int linetoken=0;
char TokenBuffer[MAXNUMITEM][NAMELENG];

/*******************************************************************/
/* variabili di stato */
int inrulepos=0;
int inruleimm=0;
int incondimm=0;
int incondpos=0;


/*******************************************************************/
extern int millineno;

/*******************************************************************/

char Comment[NAMELENG];
int identif;
int ret=0;
int var=0;
int lasttoken;
int token;
unsigned line_num = 1;

/*******************************************************************/
%}

DIGIT [0-9]
STR [a-zA-Z0-9]*
STRI \"[^\"\n]*[\"\n]
ID   [_a-zA-Z][_a-zA-Z0-9]*
NL  \n

%x comment


%%



{ID}  {

/*Ricerca nella lista delle keyword*/
  CurrentKeyword.IdName = miltext;    
  PKeyword = (KEYWORDMIL *)bsearch(&CurrentKeyword,
    MilKeywords, MilKeywordNumber, sizeof(KEYWORDMIL), MilCompareId);
  if(PKeyword != NULL)  /* E' UNA KEYWORD */
   { 
     
    action_keyword();

    // Message(miltext);
    lasttoken=  PKeyword->IdValue;
   // printf("KEYWORD %s (%d)\n",miltext,lasttoken);
    /*in questo modo scrive sempre il numero associato alla keyword*/
    /*io devo farlo scrivere solo se inrule=2*/
    RETURNIT(PKeyword->IdValue,"Keyword");
    
   }
    
  
  else
    {
  /*Ricerca nella lista delle item (regole) perch non  una keyword*/
  
    PItemImm = pimm->SearchNameImm(miltext);
	      
    PItemPos = ppos->SearchNamePos(miltext);
	      
    if (PItemImm!=NULL)  /* E' UN ITEM (REGOLA) DI IMMISSIONE */
      {
         /*  action_item(); */
       //  printf("DEF RULE %s \n",miltext);
         millval.string = miltext;
         RETURNIT(PItemImm->ItemType, "E' una regola gi definita");
      }
    else if (PItemPos!=NULL)  /* E' UN ITEM (REGOLA) DI POSIZIONAMENTO */
      {
        // printf("DEF RULE %s \n",miltext);
         millval.string = miltext;
         RETURNIT(PItemPos->ItemType, "E' una regola gi definita");
      }   
    else /* e' un nuovo item (nuova regola)...qui sia PItemImm che 
            PItemPos sono NULL*/
      { 
     
      ret=new_item(&identif,Comment); 

      if (ret==0) { 
 //   printf("NEW RULE %s \n",miltext);
      RETURNIT(identif,Comment); }
      else 
        {
  //  printf("UNDEF RULE %s \n",miltext);
        RETURNIT(IDENTIFIER,"Identifier");
        }
      }
    } 
  }

">" RETURNIT(GREATER, miltext);
"<" RETURNIT(LOWer, miltext);
"=" RETURNIT(EQUAL, miltext);
"," RETURNIT(COMMA, miltext);
";" RETURNIT(SEMICOLON, miltext);
"(" RETURNIT(LEFTPAR, miltext);
")" RETURNIT(RIGHTPAR, miltext);
"[" RETURNIT(LEFTPARQ, miltext);
"]" RETURNIT(RIGHTPARQ, miltext);
">=" RETURNIT(GREATERe, miltext);
"<=" RETURNIT(LOWERe, miltext);
"+" RETURNIT(PLUS, miltext);
"-" RETURNIT(MIN, miltext);


    

[-]?{DIGIT}+  { 
	  sscanf(miltext,"%d", &millval.integer);
	  RETURNIT(INTEGER, "Integer");
	}
	
[-]?{DIGIT}+("."{DIGIT}+)?  {
          sscanf(miltext,"%lf", &millval.real);
	  RETURNIT(REAL, "Real");
	 }	

{STRI} {
//      if (inrule) AddRule(millval.string);
//      sscanf(miltext,"%s",millval.string); 
       millval.string = miltext;
       RETURNIT(STRING, "String");
      }
	  
{NL}  { extern int millineno; millineno++; }


^#.*"\n"        {/* Eat remaining preprocessor directives after preprocessing
                  * #line,#pragma
                  */
                (void)fprintf(milout,"%s", miltext);
                }
"/*"            {//(void)fprintf(milout,"\n%s", miltext);
                BEGIN(comment);
                }
" "             { //(void)fprintf(milout," ");  
                }
		
"//"[^\n]*            { /* a Line comment */
                     
                      }		
		
<comment>[^*\n]*   {/* eat anything that's not a '*' */
                   //(void)fprintf(milout,"%s", miltext);
                   }
<comment>"*"+[^*/\n]* {/* eat up '*'s not followed by '/'s */
                   //(void)fprintf(milout,"%s", miltext);
                   }
<comment>\n        {
                  // (void)fprintf(milout,"%s", miltext);
                   ++line_num;
                   }
<comment>"*"+"/"   {//(void)fprintf(milout,"%s", miltext); 
                   BEGIN(INITIAL);
                   }

%.* ;
[\t]+ ;
%%


/******************************************************************/

void action_keyword(void)
{
//char buffe[30];
  switch(PKeyword->IdValue)
    {
    case immIF:
         incondimm=1;
	 CurrentCondImm = pimm->AddConditionImm();
	 break;
    case posIF:
         incondpos=1;
	 CurrentCondPos = ppos->AddConditionPos();
	 break;	 
    case RULEIMM:
         inruleimm=1;
//	 printf("Qui inruleimm=1\n");  
         break;
    case RULEPOS:
         inrulepos=1;
//	 printf("Qui inrulepos=1\n"); 
         break; 
    case RULEPRI:
//         printf("Regola di priorita'\n");
	 break;
    }
}

 

/*****************************************************************************/
int new_item(int * ident, char comm[])
{

if (inruleimm==1) 
  { 
    CurrentRuleImm = pimm->AddItemImm(miltext,RULE_NAME);
    AddToken();
    *ident=RULE_NAME;
    strcpy(comm,"Rule Name");
    inruleimm=2; 
    /*pongo qui inruleimm=2 perche' primo lo poneva quando riutilizzavo*/
    /*una regola e non quando la definivo*/
 //   printf("Qui inruleimm=2\n");
  }
else if (inrulepos==1)
 {
    CurrentRulePos = ppos->AddItemPos(miltext,RULE_NAME);
    AddToken();
    *ident=RULE_NAME;
    strcpy(comm,"Rule Name");
    inrulepos=2; 
    /*pongo qui inrulepos=2 perche' primo lo poneva quando riutilizzavo*/
    /*una regola e non quando la definivo*/
 //   printf("Qui inrulepos=2\n");
  }  
else return(1);  /* non e' un nuovo item */

return(0);
}

/*****************************************************************************/
void AddToken(void )
{ 
// if (indebug==1)  fprintf(stderr,"ADD Token %s \n",miltext);
//strcpy(&(TokenBuffer[linetoken][0]),miltext); 
//linetoken++; 
}


/*****************************************************************************/
void Errore(char * comm, char * nom, int num)
{
  fprintf(stderr,"\nError at line %d ---> %s %s %d\n",millineno,comm,nom,num);
  exit(1);
}

/*******************************************************************************/

int MilCompareId(const void * Element1, const void * Element2)
{
  return (strcmp(((PKEYWORDMIL)Element1)->IdName,
		 ((PKEYWORDMIL)Element2)->IdName));
}
