/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/parsers/saxparser.hpp>
#include <xercesc/sax/documenthandler.hpp>
#include <xercesc/sax/errorhandler.hpp>
#include <xercesc/sax/saxparseexception.hpp>
#include <xercesc/sax/entityresolver.hpp>
#include <xercesc/framework/localfileinputsource.hpp>

#include "xml/loa/xmlcommonclasses.hpp"



#include "mil/m.tab.h"
#include "mil/lgiust.h"

#include "mil/immissione.hpp"
#include "mil/posiz.hpp"
#include "mil/priorita.hpp"
#include "mil/simboli.hpp"


int MilCompareId(const void * Element1, const void * Element2);
extern KEYWORDMIL MilKeywords[];

extern int MilKeywordNumber;
extern ITEMPOS CurrentItemPos;
extern ITEMIMM CurrentItemImm;
extern CONDITIONIMM CurrentConditionImm;
extern CONDITIONPOS CurrentConditionPos;
extern ITEMPOS ItemListPos[];
extern ITEMPOS ItemListPos[];
extern CONDITIONIMM ConditionListImm[];
extern CONDITIONPOS ConditionListPos[];


PITEMPOS CurrentRulePos=NULL;
PITEMIMM CurrentRuleImm=NULL;
PCONDITIONIMM CurrentCondImm=NULL;
PCONDITIONPOS CurrentCondPos=NULL;

extern RANGE CurrentRange;
static KEYWORDMIL  CurrentKeyword;
static PKEYWORDMIL PKeyword;


class XMLMillaHandler : public DocumentHandler, public ErrorHandler
{
private:
	int			CurrentColImm;
	int			CurrentColPos;

	StrX*		curGroupDefName;
	StrX*		curFontMapDefName;
	LiooFont*	curFont;

	StrX*		curRuleType;
	StrX*		curTargetRule;

	StrX*		curSymbolName;

	int*		curCode;
	int*		curDim;
	//dimension sub tags
	int			curTop;
	int			curBottom;
	int			curLeft;
	int			curRight;
	int			curDx;
	int			curDy;

	double*		curDelta;
	double*		curLength;
	
	int*		curDur;
	int*		curHead;

	char*		curText;

	double		curHeight;
	double*		curHeightMax;
	double*		curHeightMin;
	
	int			curCondType;
	int*		curCondWithCode;

	char*		curExpression;
	char*		curOperator;
	char*		curFirstFactor;
	char*		curSecondFactor;
	double*		curRefValue;

	double		curTime;
	double*		curTimeNum;
	double*		curTimeDen;

	int			curFirstSym;
	int			curSecondSym;
	int*		curPrecSym; 
	
	int*		curOrderSym;

	int*		curAlignx;
	int*		curAligny;

	double*		curSlope;

	bool		curTupleInBeam;

	int*		curTextCode;
	bool		isSet;

	
protected:
	Immissione*		pimm;
	Posiz*			ppos;
	Priorita*		ppri;
	Simboli*		psym;
	
	int errCount;
	
public:
	XMLMillaHandler(Immissione* pruleimm, Posiz* prulepos,
		Priorita* prulepri, Simboli* psymbol);
	
	void  setDocumentLocator (const Locator* const locator) {};
	void  resetDocument () { };
	
	void  startDocument ();
	void  startElement ( const XMLCh* const name , AttributeList& attrs );
	void  characters ( const XMLCh* const chars , const unsigned int length );
	void  endElement (const XMLCh* const name);
	void  endDocument ();
	
	void  ignorableWhitespace ( const XMLCh* const chars , const unsigned int length );
	void  processingInstruction ( const XMLCh* const target , const XMLCh* const data );
	
	int getErrCount() { return errCount; };
	
	void  resetErrors () { errCount=0;};
	void  warning (const SAXParseException& exception) { 
		StrX msg=exception.getMessage();
		StrX filename=exception.getSystemId();
		wxLogMessage("WARNING %s(%d:%d): %s",  filename.localForm(), exception.getLineNumber(), exception.getColumnNumber() , msg.localForm());
		errCount++; 
	};
	void  error (const SAXParseException& exception) { 
		StrX msg=exception.getMessage();
		StrX filename=exception.getSystemId();
		wxLogMessage("ERROR %s(%d:%d): %s",  filename.localForm(), exception.getLineNumber(), exception.getColumnNumber() , msg.localForm());
		errCount+=2; 
	};
	void  fatalError (const SAXParseException& exception) 
	{ 
		StrX msg=exception.getMessage();
		StrX filename=exception.getSystemId();
		wxLogError("FATALERROR %s(%d:%d): %s",  filename.localForm(), exception.getLineNumber(), exception.getColumnNumber() , msg.localForm());
		errCount+=10;
	};

	void notImplemented(const char* element, const char*attribute=NULL, const char* value=NULL);
	
	int GetKeyCode(char* miltext);
	
	//--- LEVEL 1 ---//
	void startFontMapDef(AttributeList& attrs);
	void endFontMapDef();
	void startGroupDef(AttributeList& attrs);
	void endGroupDef();

	void startSymbolPositionRule(AttributeList& attrs);
	void startSymbolDirectionRule(AttributeList& attrs);
	void startSymbolOrderRule(AttributeList& attrs);
	
	void startHeadRule(AttributeList& attrs);
	void startStemLengthRule(AttributeList& attrs);
	void startStemStartRule(AttributeList& attrs);
	void startStemDirectionRule(AttributeList& attrs);	
	void startBeamSlopeRule(AttributeList& attrs);
	void startBeamDirectionRule(AttributeList& attrs);
	void startAutoBeamRule(AttributeList& attrs);
	
	void endRule();

	void startApplyRule(AttributeList& attrs);
	void endApplyRule();
	//--- END LEVEL 1 ---//

	//--- GROUP DEF --- //
	void startSymbolDef(AttributeList& attrs);
	void endSymbolDef();
	void startSymbolDefCode(AttributeList& attrs);
	void endSymbolDefCode();
	void startSymbolDefDimension(AttributeList& attrs);
	void endSymbolDefDimension();
	//--- END GROUP DEF --- //

	//--- START RULES ---//
	void startSymbolOrder(AttributeList& attrs);
	void endSymbolOrder();

	void startCode(AttributeList& attrs);
	void endCode();

	void startLength(AttributeList& attrs);
	void endLength();
	void endNoteHeight();

	void startAlignx(AttributeList& attrs);
	void endAlignx();
	void startAligny(AttributeList& attrs);
	void endAligny();

	void startSlope(AttributeList& attrs);
	void endSlope();
	void endSlopeMax();
	void endSlopeMin();


	void startHeight(int min_max);
	void endHeight();
	void endHeightMax();
	void endHeightMin();

	void startSymbol(AttributeList& attrs);
	void endSymbol();

	void startDuration(AttributeList& attrs);
	void endDuration();

	void startDelta(AttributeList& attrs);
	void endDx();
	void endDy();
	
	//--- END RULES ---/

	//-- CONDITIONS --//
	void startNote(AttributeList& attrs);
	void endNote();
	void startRest(AttributeList& attrs);
	void endRest();
	void startChord(AttributeList& attrs);
	void endChord();
	void startMeasure(AttributeList& attrs);
	void endMeasure();
	void startBeam(AttributeList& attrs);
	void endBeam();
	void startPreceed(AttributeList& attrs);
	void endPreceed();
	void startExpression(char* value);
	void endExpression();

	void startRange(AttributeList& attrs);
	void endRange(int token, double min, double max);
	
	void startCondWith(AttributeList& attrs);
	void endCondWith();

	void startHead(AttributeList& attrs);
	void endHead();

	void startTimeNum(AttributeList& attrs);
	void startTimeDen(AttributeList& attrs);
	void endTimeNum();
	void endTimeDen();


	void startOperator(char* value);
	void endOperator();
	void startFactor(char* value);
	void startRefValue(AttributeList& attrs);

	void startSymbolPrec(AttributeList& attrs);
	void endSymbolPrec();

	//--- start/end TOKENS ---//
	void setCat(int lasttoken);
	void setSymbol(int lasttoken);
	void setCol(int lasttoken);
	void setStaff(int lasttoken);
	void setCondizione(int token, int lasttoken);

};

/*
class XMLMillaEntityResolver : public EntityResolver
{
public:
	InputSource* resolveEntity( const XMLCh* const publicId, const XMLCh* const systemId )
	{
		return new LocalFileInputSource(L"C:\\moods\\dtd\\", systemId);
	}
};
*/

XMLMillaHandler::XMLMillaHandler(Immissione* pruleimm, Posiz* prulepos,
								 Priorita* prulepri, Simboli* psymbol)
{
	pimm = pruleimm;
	ppos = prulepos;
	ppri = prulepri;
	psym = psymbol;  
	
	qsort(MilKeywords, MilKeywordNumber, sizeof(KEYWORDMIL), MilCompareId);
	
	errCount=0;
};

void  XMLMillaHandler::startDocument ()
{
	
	CurrentColImm=0;
	CurrentColPos=0;
	pimm->Reset();
	ppos->Reset();
	ppri->Reset();
	psym->Reset();
	
	curGroupDefName	= NULL;
	curFontMapDefName = NULL;
	curFont			= NULL;
	curRuleType		= NULL;
	curTargetRule	= NULL;

	curSymbolName	= NULL;
	curCode			= NULL;
	curDim			= NULL;
	curDelta		= NULL;
	curLength		= NULL;

	curDur			= NULL;
	curHead			= NULL;
	curHeight		= NULL;
	curHeightMax	= NULL;
	curHeightMin	= NULL;;

	curCondType		= NULL;
	curCondWithCode = NULL;
	curExpression	= NULL;
	curOperator		= NULL;
	curFirstFactor	= NULL;
	curSecondFactor = NULL;
	curRefValue		= NULL;

	curTime			= NULL;
	curTimeNum		= NULL;
	curTimeDen		= NULL;

	curFirstSym		= NULL;
	curSecondSym	= NULL;
	curPrecSym		= NULL;

	curOrderSym		= NULL;
	
	curText			= NULL;

	curAlignx		= NULL;
	curAligny		= NULL;

	curSlope		= NULL;

	curTupleInBeam	= false;

	curTextCode		= NULL;
}

void  XMLMillaHandler::startElement (const XMLCh* const name , AttributeList& attrs )
{
	StrX lname(name);

	isSet = false;

	//-- LEVEL 1 --//
	if (lname.equals("groupDef") && curTargetRule == NULL && curRuleType == NULL)
		startGroupDef(attrs);
	if (lname.equals("fontMapDef") && curTargetRule == NULL && curRuleType == NULL)
		startFontMapDef(attrs);
	if (curGroupDefName == NULL && curTargetRule == NULL)
	{
		if (lname.equals("symbolDirectionRule"))
			startSymbolDirectionRule(attrs);
		if (lname.equals("symbolPositionRule"))
			startSymbolPositionRule(attrs);
		if (lname.equals("symbolOrderRule"))
			startSymbolOrderRule(attrs);
		
		if (lname.equals("headRule"))
			startHeadRule(attrs);
		if (lname.equals("stemLengthRule"))
			startStemLengthRule(attrs);
		if (lname.equals("stemStartRule"))
			startStemStartRule(attrs);
		if (lname.equals("stemDirectionRule"))
			startStemDirectionRule(attrs);
		if (lname.equals("beamSlopeRule"))
			startBeamSlopeRule(attrs);
		if (lname.equals("beamDirectionRule"))
			startBeamDirectionRule(attrs);
		if (lname.equals("autoBeamRule"))
			startAutoBeamRule(attrs);
	}
	if (lname.equals("applyRule") && curRuleType == NULL && curGroupDefName == NULL)
		startApplyRule(attrs);
	//-- END LEVEL 1 --//

	//-- GROUP DEF --//
	if (curGroupDefName !=NULL || curFontMapDefName != NULL)
	{
		if (lname.equals("symbolDef"))
			startSymbolDef(attrs);
		if (lname.equals("code"))
			startSymbolDefCode(attrs);
		if (lname.equals("dimension"))
			startSymbolDefDimension(attrs);
	}
	//-- END GROUP DEF --//

	//-- START APPLY RULE
	if(curTargetRule != NULL)
	{
		if (lname.equals("note"))
			startNote(attrs); 
		if (lname.equals("rest"))
			startRest(attrs);
		if (lname.equals("chord"))
			startChord(attrs);
		if (lname.equals("symbol") && curCondType != PRECEED)
			startNote(attrs);
		if (lname.equals("measure"))
			startMeasure(attrs);
		if (lname.equals("beam"))
			startBeam(attrs);
		if (lname.equals("preceed"))
			startPreceed(attrs);
		//--- conditions : expressions ---//
		if	(lname.equals("gt"))
			startExpression("GREATER");
		if	(lname.equals("lt"))
			startExpression("LOWer");
		if	(lname.equals("ge"))
			startExpression("GREATERe");
		if	(lname.equals("le"))
			startExpression("LOWERe");
		if	(lname.equals("eq"))
			startExpression("EQUAL");

		if (curExpression != NULL)
		{
			if (lname.equals("minus"))
				startOperator("MIN");
			if (lname.equals("plus"))
				startOperator("PLUS");

			if (lname.equals("staffNumber"))
				startFactor("STAFFNUMBER");
			if (lname.equals("staffLines"))
				startFactor("STAFFLINES");
			if (lname.equals("beam.delta"))
				startFactor("DELTA");
			if (lname.equals("beam.mean"))
				startFactor("MEAN");
			if (lname.equals("beam.mhl"))
				startFactor("MHL");
			if (lname.equals("beam.dis"))
				startFactor("BEAMDIS");

			if (lname.equals("chord.upperd"))
				startFactor("UPPERD");
			if (lname.equals("chord.lowerd"))
				startFactor("LOWERD");
			if (lname.equals("chord.numUp"))
				startFactor("NUMUP");
			if (lname.equals("chord.numDown"))
				startFactor("NUMDOWN");
			
			if (lname.equals("value"))
				startRefValue(attrs);
		}
		//-- --//
		if(curCondType == NOTE || curCondType == PAUSA)// || curCondType == INCHORD)
		{
			if	(lname.equals("isSmall"))
				setCondizione(NOTE, NOTINA);
			if	(lname.equals("isFirstUpper"))
				setCondizione(NOTE, FIRSTUPPER);
			if	(lname.equals("isFirstLower"))
				setCondizione(NOTE, FIRSTLOWER);
			if	(lname.equals("inChord"))
				setCondizione(NOTE, INCHORD);
			if	(lname.equals("inBeam"))
				setCondizione(NOTE, INBEAM);
			if	(lname.equals("inSingleVoice"))
				setCondizione(NOTE, INSINGLELAYER);
			if	(lname.equals("inMultivoice"))
				setCondizione(NOTE, INMULTIVOICE);
			if	(lname.equals("inMultivoiceUpper"))
				setCondizione(NOTE, INMULTIVOICEUPPER);
			if	(lname.equals("inMultivoiceLower"))
				setCondizione(NOTE, INMULTIVOICELOWER);
			if	(lname.equals("duration"))
				startDuration(attrs);

			if	(lname.equals("height"))
				startHeight(0);
			if	(lname.equals("heightGE"))
				startHeight(-1);
			if	(lname.equals("heightLE"))
				startHeight(1);
			if	(lname.equals("heightAny"))
				endRange(curCondType, VAR, VAR);

			if	(lname.equals("stemDown"))
				setCondizione(curCondType, STEMDOWN);
			if	(lname.equals("stemUp"))
				setCondizione(curCondType, STEMUP);
			if	(lname.equals("with"))
				startCondWith(attrs);
			if	(lname.equals("head"))
				startHead(attrs);

			if	(lname.equals("onStem"))
				setCol(ONSTEM);
			if	(lname.equals("oppositeToStem"))
				setCol(OPPOSITETOSTEM);
		}

		if(lname.equals("symbol") && curCondType == PRECEED)
			startSymbolPrec(attrs);

		if (curCondType == TIME)
		{
			if(lname.equals("timeNum")) 
				startTimeNum(attrs);
			if(lname.equals("timeDen")) 
				startTimeDen(attrs);
		}

		if(curCondType == BEAM)
		{
			if(lname.equals("inMultiStaff"))		
				setCol(INMULTISTAFF);
			if(lname.equals("inMultivoiceUpper"))			
				setCondizione(NOTES, INMULTIVOICEUPPER);
			if(lname.equals("inMultivoiceLower"))		
				setCondizione(NOTES, INMULTIVOICELOWER);
			if(lname.equals("inSingleVoice"))			
				setCondizione(NOTES, INSINGLELAYER);
		}
	}
	//-- END APPLY RULE

	//-- START RULES --//
	if (curRuleType != NULL)
	{
		if (curRuleType->equals("SymbolOrder")){
			if(lname.equals("symbol"))
				startSymbolOrder(attrs);
		}
		
		if (curRuleType->equals("Head")){
			if(lname.equals("code"))
				startCode(attrs);
		}

		if (curRuleType->equals("StemLength")){
			if (lname.equals("length"))
				startLength(attrs);
		}

		if (curRuleType->equals("StemStart")){
			if (lname.equals("alignx"))
				startAlignx(attrs);
			if (lname.equals("aligny"))
				startAligny(attrs);
		}

		if (curRuleType->equals("BeamSlope")){
			if (lname.equals("slope"))
				startSlope(attrs);
			if (lname.equals("slopeMax"))
				startSlope(attrs);
			if (lname.equals("slopeMin"))
				startSlope(attrs);
		}

		if (curRuleType->equals("AutoBeam")){
			if (lname.equals("nbeat"))
				startLength(attrs);
			if (lname.equals("tupleInBeam"))
				curTupleInBeam = true;
		}

		if (curRuleType->equals("SymbolDirection") || curRuleType->equals("SymbolPosition"))
		{
			if (lname.equals("symbol") || lname.equals("group"))
				startSymbol(attrs);
		}

		//--- cat:stem ---//
		if	(lname.equals("stemUp"))
			setCat(STEMUP);
		if	(lname.equals("stemDown"))
			setCat(STEMDOWN);
		if	(lname.equals("beam12"))			//	<-------------------------------------
			setCat(BEAM12);
		if	(lname.equals("beam23"))			//	<-------------------------------------
			setCat(BEAM23);
		
		//--- cat:wheresimbolo ---//
		if (lname.equals("onStem"))
			setCat(ONSTEM);
		if	(lname.equals("oppositeToStem"))
			setCat(OPPOSITETOSTEM);
		
		//--- cat:belwab ---//
		if (lname.equals("above"))
			setCat(ABOVE);
		if	(lname.equals("below"))
			setCat(BELOW);	
		
		//--- cat:spaceline ---//
		if	(lname.equals("onLine"))
			setCat(ONLINE);
		if	(lname.equals("onSpace"))
			setCat(ONSPACE);
		
		//--- cat: staff ---//
		if	(lname.equals("outsideStaff"))
			setStaff(OUTSIDESTAFF);
		if	(lname.equals("onStaff"))
			setStaff(ONSTAFF);
		if	(lname.equals("avoidStaff"))
			setStaff(AVOIDINSTAFF);
		if	(lname.equals("anywhere"))		//commentato nel vecchio milla
			setStaff(ANYWHERE);	
		
		//--- coord ---//
		if	( lname.equals("dx") || lname.equals("dy") )
			startDelta(attrs);					
	}
		//-- END RULES --//

}

void  XMLMillaHandler::characters ( const XMLCh* const chars , const unsigned int length )
{
	StrX text(chars);
	curText = (char*)(const char*)text;
	
	if (curText != NULL && !isSet)
	{
		if (curCode != NULL && curDim == NULL)		
			*curCode = atoi(curText);
		
		if (curDim != NULL)
			*curDim = atoi(curText);

		if (curTextCode != NULL)
		{
			*curTextCode = GetKeyCode(curText);
			if (*curTextCode == NULL)
			{
				int position = psym->GetGroupPos(curText);
				if (position != -1)
					setSymbol(1000 + position);
			}
		}
		
		if (curDelta != NULL)
			*curDelta = atof(curText);
		
		if (curLength != NULL)
			*curLength = atof(curText);
		
		if (curHeightMax != NULL || curHeightMin != NULL)
			curHeight = atof(curText);

		if (curDur != NULL)
		{
			/*if (text.equals("D4M"))
				curText = "";
			if (text.equals("D2M"))
				curText = "";*/
			if (text.equals("D2"))
				curText = "BREVE";
			else if (text.equals("D1"))
				curText = "SEMIBREVE";
			else if (text.equals("D1_2"))
				curText = "MINIMA";
			else if (text.equals("D1_4"))
				curText = "SEMIMINIMA";
			else if (text.equals("D1_8"))
				curText = "CROMA";
			else if (text.equals("D1_16"))
				curText = "SEMICROMA";
			else if (text.equals("D1_32"))
				curText = "BISCROMA";
			else if (text.equals("D1_64"))
				curText = "SEMIBISCROMA";
			else if (text.equals("D1_128"))
				curText = "FUSA";
			/*if (text.equals("DGENERIC"))
				curText = "";*/
			else return;

			if (curCondType == PAUSA)
			{
				char* str_p = "P";
				strcat(str_p, curText);
				*curDur = GetKeyCode(str_p);
			}
			else *curDur = GetKeyCode(curText);
		}
		
		if (curCondWithCode != NULL)
			*curCondWithCode = GetKeyCode(curText);

		if (curHead != NULL)
			*curHead = GetKeyCode(curText);
		
		if (curRefValue != NULL)
			*curRefValue = atof(curText);
		
		if (curTimeNum != NULL || curTimeDen != NULL)
			curTime = atof(curText);

		if (curAlignx != NULL)
		{
			if (text.equals("LEFT"))
				*curAlignx = -1;
			if (text.equals("CENTER"))
				*curAlignx = 0;
			if (text.equals("RIGHT"))
				*curAlignx = 1;
		}
		if (curAligny != NULL)
		{
			if (text.equals("TOP"))
				*curAligny = 1;
			if (text.equals("CENTER"))
				*curAligny = 0;
			if (text.equals("BOTTOM"))
				*curAligny = -1;
		}
		
		if (curOrderSym != NULL)
		{
			*curOrderSym = GetKeyCode(curText);
			if(*curOrderSym == NULL)
			{
				int position = psym->GetSymbolPos(curText);		
				if (position != -1)
					ppri->AddPriorityGen(2000+position);
			}
		}
		
		if (curPrecSym != NULL)
			*curPrecSym = GetKeyCode(curText);
		
		if (curSlope != NULL)
			*curSlope = atof(curText);

		if (curTupleInBeam)
		{
			if (curText == "true")
				CurrentRulePos->abilterzine = ON;
			else if (curText == "false")
				CurrentRulePos->abilterzine = OFF;
		}
	isSet = true;
	}
}

void  XMLMillaHandler::endElement (const XMLCh* const name)
{
	StrX lname(name);
	
	if (lname.equals("groupDef") && curTargetRule == NULL && curRuleType == NULL)
		endGroupDef();	
	if (lname.equals("fontMapDef") && curTargetRule == NULL && curRuleType == NULL)
		endFontMapDef();	
	if(curTargetRule == NULL && curGroupDefName == NULL)
	{
		if (lname.equals("symbolDirectionRule"))
			endRule();
		if (lname.equals("symbolPositionRule"))
			endRule();
		if (lname.equals("symbolOrderRule"))
			endRule();
		
		if (lname.equals("headRule"))
			endRule();
		if (lname.equals("stemLengthRule"))
			endRule();
		if (lname.equals("stemStartRule"))
			endRule();
		if (lname.equals("stemDirectionRule"))
			endRule();
		if (lname.equals("beamSlopeRule"))
			endRule();
		if (lname.equals("beamDirectionRule"))
			endRule();
		if (lname.equals("autoBeamRule"))
			endRule();
	}
	if (lname.equals("applyRule") && curGroupDefName == NULL && curRuleType == NULL)
		endApplyRule();
	
	//-- GROUP DEF --//
	if (curGroupDefName !=NULL || curFontMapDefName != NULL)
	{
		if (lname.equals("symbolDef"))
			endSymbolDef();
		if (lname.equals("code"))
			endSymbolDefCode();
		if (lname.equals("dimension"))
			endSymbolDefDimension();
		
		//---dimension sub tags start
		if (lname.equals("toTop"))
			curTop = *curDim;
		if (lname.equals("toBottom"))
			curBottom = *curDim;
		if (lname.equals("toLeft"))
			curLeft = *curDim;
		if (lname.equals("toRight"))
			curRight = *curDim;
		if (lname.equals("dx"))
			curDx = *curDim;
		if (lname.equals("dy"))
			curDy = *curDim;
	}
	//-- END GROUP DEF --//
	
	//-- START APPLY RULE
	if(curTargetRule != NULL)
	{
		if (lname.equals("note"))
			endNote(); 
		if (lname.equals("symbol") && curCondType != PRECEED)
			endNote(); 
		if (lname.equals("rest"))
			endRest();
		if (lname.equals("chord"))
			endChord();
		if (lname.equals("measure"))
			endMeasure();
		if (lname.equals("beam"))
			endBeam();
		if (lname.equals("preceed"))
			endPreceed();
		//--- expressions ---//
		if	(lname.equals("gt"))
			endExpression();
		if	(lname.equals("lt"))
			endExpression();
		if	(lname.equals("ge"))
			endExpression();
		if	(lname.equals("le"))
			endExpression();
		if	(lname.equals("eq"))
			endExpression();


		if (curCondType == NOTE || curCondType == PAUSA)
		{
			if (lname.equals("height"))
				endHeight();
			if (lname.equals("heightGE"))
				endHeightMin();
			if (lname.equals("heightLE"))
				endHeightMax();

			if (lname.equals("with"))
				endCondWith(); 
			if (lname.equals("duration"))
				endDuration();
			if	(lname.equals("head"))
				endHead();
		}

		if(lname.equals("symbol") && curCondType == PRECEED)
			endSymbolPrec();

		if (curCondType == TIME)
		{
			if(lname.equals("timeNum")) 
				endTimeNum();
			if(lname.equals("timeDen")) 
				endTimeDen();
		}

	}
	//-- END APPLY RULE

	//-- START RULES --//
	if (curRuleType != NULL)
	{
		if (curRuleType->equals("SymbolOrder")){
			if(lname.equals("symbol"))
				endSymbolOrder();
		}
		
		if (curRuleType->equals("Head")){
			if(lname.equals("code"))
				endCode();
		}
		
		if (curRuleType->equals("StemLength")){
			if (lname.equals("length"))
				endLength();
			if (lname.equals("noteHeight"))
				endNoteHeight();	
		}
		
		if (curRuleType->equals("StemStart")){
			if (lname.equals("alignx"))
				endAlignx();
			if (lname.equals("aligny"))
				endAligny();
		}
		
		if (curRuleType->equals("BeamSlope")){
			if (lname.equals("slope"))
				endSlope();
			if (lname.equals("slopeMax"))
				endSlopeMax();
			if (lname.equals("slopeMin"))
				endSlopeMin();
		}
		
		if (curRuleType->equals("AutoBeam")){
			if (lname.equals("nbeat"))
				endLength();
			if (lname.equals("tupleInBeam"))
				curTupleInBeam = false;
		}

		if (curRuleType->equals("SymbolDirection") || curRuleType->equals("SymbolPosition"))
		{
			if (lname.equals("symbol") || lname.equals("group"))
				endSymbol();
			if (lname.equals("dx"))
				endDx();
			if (lname.equals("dy"))
				endDy();		
		}

	}
	//-- END RULES --//
}

void  XMLMillaHandler::endDocument ()
{
}

void  XMLMillaHandler::ignorableWhitespace ( const XMLCh* const chars , const unsigned int length )
{
}

void  XMLMillaHandler::processingInstruction ( const XMLCh* const target , const XMLCh* const data )
{
}


void  XMLMillaHandler::notImplemented(const char* element, const char* attribute, const char* value)
{
	char s[200];
	
	if(attribute==NULL)
		sprintf(s,"not implemented\nelement: %s",element);
	else
		sprintf(s,"not implemented\nelement: %s\nattribute: %s=%s",element,attribute,value);
	
	Message(s);
}

//-------------------- SAX PARSER CALL ---------------------------------//

int InitLoad(char* filename, Immissione* pruleimm, Posiz* prulepos, Priorita* prulepri, Simboli* psymbol)		//InitLoad
{
	try
	{
		XMLPlatformUtils::Initialize();
	}
	catch (const XMLException&)
	{
		Message("Errors on init XML","Load SMFL");
		return 1;
	}
	SAXParser milParser;
	XMLMillaHandler *milHandler =new XMLMillaHandler(pruleimm, prulepos, prulepri, psymbol);
	//XMLMillaEntityResolver * milEntRes=new XMLMillaEntityResolver;
	
    milParser.setValidationScheme(SAXParser::Val_Always);
    milParser.setDoNamespaces(true);
	milParser.setDoSchema(true);
    milParser.setValidationSchemaFullChecking(true);
	wxString schemaPath=smrgetenv("SMR_ERR_PATH");
	schemaPath.Replace(" ", "%20");

	milParser.setExternalSchemaLocation("urn:mpeg:mpeg-4:schema:smr:smfl:2007 "+schemaPath+"xsd\\smfl.xsd");

	milParser.setDocumentHandler(milHandler);
	milParser.setErrorHandler(milHandler);
	//milParser.setEntityResolver(milEntRes);
	
	milParser.setDoValidation(true);
	
	milParser.parse(filename);
	
	int errCnt = milHandler->getErrCount();
	
	milParser.setDocumentHandler(NULL);
	milParser.setErrorHandler(NULL);
	milParser.setEntityResolver(NULL);
	delete milHandler;
	//delete milEntRes;
	
	return errCnt;
}



//////////////////////////////////////////////////////
int XMLMillaHandler::GetKeyCode(char* miltext)
{
	CurrentKeyword.IdName = miltext;    
	PKeyword = (KEYWORDMIL *)bsearch(&CurrentKeyword,
		MilKeywords, MilKeywordNumber, sizeof(KEYWORDMIL), MilCompareId);
	if(PKeyword != NULL)
		return PKeyword->IdValue;
	else 
		return NULL;
}

/*****************************************************************************/
void Errore(char * comm, char * nom, int num)
{
	fprintf(stderr,"\nError at line ---> %s %s %d\n",comm,nom,num);
	exit(1);
}

/*******************************************************************************/

int MilCompareId(const void * Element1, const void * Element2)
{
	return (strcmp(((PKEYWORDMIL)Element1)->IdName,
		((PKEYWORDMIL)Element2)->IdName));
}

//-------------------------------------------------------//
//-----------------------TAG CALLS-----------------------//
//-------------------------------------------------------//

void XMLMillaHandler::startGroupDef(AttributeList& attrs)
{
	curGroupDefName = new StrX(attrs.getValue("name"));
	StrX groupDefFont = attrs.getValue("font");
	StrX xmlFontSize=attrs.getValue("fontSize");
	int fontSize=atoi(xmlFontSize);
	if(fontSize<=0)
		fontSize=50;

	psym->AddGroup((char*)curGroupDefName->localForm(),(char*)groupDefFont.localForm(), fontSize);
}

void XMLMillaHandler::endGroupDef()
{
	/*if (lasttoken != -1)
		setSymbol(1000+token);*/
	delete curGroupDefName;
	curGroupDefName = NULL;
}

void XMLMillaHandler::startFontMapDef(AttributeList& attrs)
{
	curFontMapDefName = new StrX(attrs.getValue("name"));
	StrX fontMapDefFont = attrs.getValue("font");
	StrX xmlFontSize=attrs.getValue("fontSize");
	int fontSize=atoi(xmlFontSize);
	if(fontSize<=0)
		fontSize=50;

	curFont = new LiooFont((char*)fontMapDefFont.localForm(),psym->GetSMRFile(),fontSize);
}

void XMLMillaHandler::endFontMapDef()
{
	Scr->ReplaceFont(curFontMapDefName->localForm(),curFont);
	delete curFontMapDefName;
	curFontMapDefName = NULL;
}

void XMLMillaHandler::startSymbolDirectionRule(AttributeList& attrs)
{
	curRuleType =			new StrX(L"SymbolDirection");

	StrX curRuleId =		attrs.getValue("ruleId");
	CurrentRuleImm =		pimm->AddItemImm((char*)(const char*)curRuleId,RULE_NAME);
	CurrentRuleImm->rel=	RELNOTA;//RELNOTA
}

void XMLMillaHandler::startSymbolPositionRule(AttributeList& attrs)
{
	curRuleType =			new StrX(L"SymbolPosition");

	StrX curRuleId =		attrs.getValue("ruleId");
	CurrentRulePos =		ppos->AddItemPos((char*)(const char*)curRuleId,RULE_NAME);
	CurrentRulePos->rel=	RELNOTA;//RELNOTA
}

void XMLMillaHandler::startSymbolOrderRule(AttributeList& attrs)
{
	curRuleType =			new StrX(L"SymbolOrder");
}

void XMLMillaHandler::startSymbolOrder(AttributeList& attrs)
{
	curOrderSym = new int();
}

void XMLMillaHandler::endSymbolOrder()
{
	if (*curOrderSym != NULL)		
		ppri->AddPriority(*curOrderSym);


	delete curOrderSym;
	curOrderSym = NULL;
}

void XMLMillaHandler::startHeadRule(AttributeList& attrs)
{
	curRuleType =			new StrX(L"Head");

	StrX curRuleId =		attrs.getValue("ruleId");
	CurrentRulePos =		ppos->AddItemPos((char*)(const char*)curRuleId,RULE_NAME);
	CurrentRulePos->rel=	RELNOTA;//RELNOTA

	setSymbol(HEAD);
}

void XMLMillaHandler::startStemLengthRule(AttributeList& attrs)
{
	curRuleType =			new StrX(L"StemLength");

	StrX curRuleId =		attrs.getValue("ruleId");
	CurrentRulePos =		ppos->AddItemPos((char*)(const char*)curRuleId,RULE_NAME);
	CurrentRulePos->rel=	RELNOTA;//RELNOTA
	
	setSymbol(STEM);
}

void XMLMillaHandler::startStemStartRule(AttributeList& attrs)
{
	curRuleType =			new StrX(L"StemStart");

	StrX curRuleId =		attrs.getValue("ruleId");
	CurrentRulePos =		ppos->AddItemPos((char*)(const char*)curRuleId,RULE_NAME);
	CurrentRulePos->rel=	RELNOTA;//RELNOTA
	
	setSymbol(STEMSTART);
}

void XMLMillaHandler::startStemDirectionRule(AttributeList& attrs)
{
	curRuleType =			new StrX(L"StemDirection");

	StrX curRuleId =		attrs.getValue("ruleId");
	CurrentRuleImm =		pimm->AddItemImm((char*)(const char*)curRuleId,RULE_NAME);
	CurrentRuleImm->rel=	RELNOTA;//RELNOTA

	setSymbol(STEM);
}
	
void XMLMillaHandler::startBeamSlopeRule(AttributeList& attrs)
{
	curRuleType =			new StrX(L"BeamSlope");

	StrX curRuleId =		attrs.getValue("ruleId");
	CurrentRulePos =		ppos->AddItemPos((char*)(const char*)curRuleId,RULE_NAME);
	CurrentRulePos->rel=	RELNOTES;//RELNOTA
	
	setSymbol(BEAM);
}

void XMLMillaHandler::startBeamDirectionRule(AttributeList& attrs)
{
	curRuleType =			new StrX(L"BeamDirection");

	StrX curRuleId =		attrs.getValue("ruleId");
	CurrentRuleImm =		pimm->AddItemImm((char*)(const char*)curRuleId,RULE_NAME);
	CurrentRuleImm->rel=	RELNOTES;//RELNOTA
	
	setSymbol(BEAM);
}

void XMLMillaHandler::startAutoBeamRule(AttributeList& attrs)
{
	curRuleType =			new StrX(L"AutoBeam");

	StrX curRuleId =		attrs.getValue("ruleId");
	CurrentRulePos =		ppos->AddItemPos((char*)(const char*)curRuleId,RULE_NAME);
	CurrentRulePos->rel=	RELSTAFF;//RELNOTA
	
	setSymbol(BATTUTA);
}

void XMLMillaHandler::endRule()
{
	delete curRuleType;
	curRuleType = NULL;
	
	CurrentRuleImm = NULL;
	CurrentRulePos = NULL;
}

void XMLMillaHandler::startApplyRule(AttributeList& attrs)
{
	curTargetRule = new StrX(attrs.getValue("rule"));
	char* curRuleName_str =	(char*)curTargetRule->localForm();

	if (pimm->searchRuleImm(curRuleName_str))
	{
		//e' di immissione
		CurrentCondImm = pimm->AddConditionImm();
		CurrentCondImm->RuleName = (char *)malloc(strlen(curRuleName_str) + 1);
//        if (CurrentCondImm->RuleName==NULL)
//			debug_msg("allocation",curRuleName_str);
		strcpy(CurrentCondImm->RuleName, curRuleName_str);
	}
	else if (ppos->searchRulePos(curRuleName_str))
	{
		//e' di posizionamento
		CurrentCondPos = ppos->AddConditionPos();
		CurrentCondPos->RuleName = (char *)malloc(strlen(curRuleName_str) + 1);
//		if (CurrentCondPos->RuleName==NULL)
//			debug_msg("allocation",curRuleName_str);
		strcpy(CurrentCondPos->RuleName, curRuleName_str);
	}
}

void XMLMillaHandler::endApplyRule()
{
	delete curTargetRule;
	curTargetRule = NULL;

	CurrentColImm=0; 
    CurrentCondImm = NULL;

	CurrentColPos=0; 
    CurrentCondPos = NULL;
}

//--- GROUP DEF --- //
void XMLMillaHandler::startSymbolDef(AttributeList& attrs){
	curSymbolName = new StrX(attrs.getValue("name"));

	if(curGroupDefName!=NULL)
	{	
		psym->AddSymbolName((char*)curSymbolName->localForm());
		psym->AddGroupName((char*)curGroupDefName->localForm());
	}
}

void XMLMillaHandler::endSymbolDef(){
	if(curGroupDefName!=NULL)
		psym->Incrementa();
	
	delete curSymbolName;
	curSymbolName = NULL;
}

void XMLMillaHandler::startSymbolDefCode(AttributeList& attrs){
	curCode = new int();
}

void XMLMillaHandler::endSymbolDefCode(){
	if(curGroupDefName!=NULL)
		psym->AddCode(*curCode);	
}

void XMLMillaHandler::startSymbolDefDimension(AttributeList& attrs){
	curDim = new int();
}

void XMLMillaHandler::endSymbolDefDimension(){
	if(curGroupDefName!=NULL)
		psym->AddDim(curTop, curBottom, curLeft, curRight, curDx, curDy);
	if(curFontMapDefName!=NULL && curCode!=NULL)
	{
		curFont->SetDimension(*curCode,curTop,curBottom,curLeft,curRight,curDx,curDy); 
	}
	delete curCode;
	curCode = NULL;
	delete curDim;
	curDim = NULL;
}
//--- END GROUP DEF --- //



void XMLMillaHandler::startSymbol(AttributeList& attrs)
{
	curTextCode = new int();
}

void XMLMillaHandler::endSymbol()
{
	if (*curTextCode != NULL)
		setSymbol(*curTextCode);

	delete curTextCode;
	curTextCode = NULL;
}

void XMLMillaHandler::startCode(AttributeList& attrs)
{
	curCode = new int();
}

void XMLMillaHandler::endCode()
{
	CurrentRulePos->code = *curCode;
	
	delete curCode;
	curCode = NULL;
}


void XMLMillaHandler::startDuration(AttributeList& attrs)
{
	curDur = new int();
}

void XMLMillaHandler::endDuration()
{
	setCondizione(curCondType, *curDur);
	
	delete curDur;
	curDur = NULL;
}

void XMLMillaHandler::startLength(AttributeList& attrs)
{
	curLength = new double();
}

void XMLMillaHandler::endLength()
{
	if (CurrentRulePos != NULL)
		CurrentRulePos->lenght = *curLength;
	
	delete curLength;
	curLength = NULL;
}

void XMLMillaHandler::endNoteHeight()
{
	if (CurrentRulePos != NULL)
		CurrentRulePos->lenght = STEMHEIGHT;
}

void XMLMillaHandler::startAlignx(AttributeList& attrs){curAlignx = new int();}
void XMLMillaHandler::endAlignx()
{
	if (CurrentRulePos != NULL)
		CurrentRulePos->dx = *curAlignx;
	
	delete curAlignx;
	curAlignx = NULL;
}
void XMLMillaHandler::startAligny(AttributeList& attrs){curAligny = new int();}
void XMLMillaHandler::endAligny()
{
	if (CurrentRulePos != NULL)
		CurrentRulePos->dy = *curAligny;
	
	delete curAligny;
	curAligny = NULL;
}

void XMLMillaHandler::startSlope(AttributeList& attrs){curSlope = new double();}
void XMLMillaHandler::endSlope()
{
	CurrentRulePos->angle = *curSlope;

	delete curSlope;
	curSlope = NULL;
}
void XMLMillaHandler::endSlopeMax()
{	
	CurrentRulePos->rangeangle.max = *curSlope;
	CurrentRulePos->angle = BSLOPE;

	delete curSlope;
	curSlope = NULL;
}
void XMLMillaHandler::endSlopeMin()
{	
	CurrentRulePos->rangeangle.min = *curSlope;
	CurrentRulePos->angle = BSLOPE;

	delete curSlope;
	curSlope = NULL;
}

void XMLMillaHandler::startHeight(int min_max)
{ 
	if (min_max >-1) curHeightMax = new double();
	if (min_max < 1) curHeightMin = new double();
}
void XMLMillaHandler::endHeight()
{
	*curHeightMax = curHeight;
	*curHeightMin = curHeight;
}
void XMLMillaHandler::endHeightMax()
{
	*curHeightMax = curHeight;
}
void XMLMillaHandler::endHeightMin()
{
	*curHeightMin = curHeight;
}

void XMLMillaHandler::startDelta(AttributeList& attrs)
{
	curDelta = new double();
}

void XMLMillaHandler::endDx()
{
	if (CurrentRulePos != NULL)
		CurrentRulePos->dx = *curDelta;
	
	delete curDelta;
	curDelta = NULL;
}

void XMLMillaHandler::endDy()
{
	if (CurrentRulePos != NULL)
		CurrentRulePos->dy = *curDelta;
		
	delete curDelta;
	curDelta = NULL;
}



void XMLMillaHandler::startNote(AttributeList& attrs)
{
	curCondType = NOTE;
}

void XMLMillaHandler::endNote()
{
	/*
	if (curHeightMin == NULL && curHeightMax == NULL)
		endRange(curCondType, VAR, VAR);
	*/
	if (curHeightMin != NULL && curHeightMax == NULL)
		endRange(curCondType, *curHeightMin, VAR);
	if (curHeightMin == NULL && curHeightMax != NULL)
		endRange(curCondType, VAR, *curHeightMax);
	if (curHeightMin != NULL && curHeightMax != NULL)
		endRange(curCondType, *curHeightMin , *curHeightMax);
	
	curCondType = NULL;
	delete curHeightMin;
	delete curHeightMax;
	curHeightMin = NULL;
	curHeightMax = NULL;
}

void XMLMillaHandler::startRest(AttributeList& attrs){curCondType = PAUSA;}
void XMLMillaHandler::endRest()
{
	if (curHeightMin != NULL && curHeightMax == NULL)
		endRange(curCondType, *curHeightMin, VAR);
	if (curHeightMin == NULL && curHeightMax != NULL)
		endRange(curCondType, VAR, *curHeightMax);
	if (curHeightMin != NULL && curHeightMax != NULL)
		endRange(curCondType, *curHeightMin , *curHeightMax);
	
	curCondType = NULL;
	delete curHeightMin;
	delete curHeightMax;
	curHeightMin = NULL;
	curHeightMax = NULL;
}
void XMLMillaHandler::startChord(AttributeList& attrs)
{
	curCondType = NOTE;//"INCHORD";
	setCol(INCHORD);
}
void XMLMillaHandler::endChord(){curCondType = NULL;}
void XMLMillaHandler::startMeasure(AttributeList& attrs)
{
	curCondType = TIME;
}

void XMLMillaHandler::endMeasure()
{
	endRange(curCondType, *curTimeNum, *curTimeDen);

	curCondType = NULL;

	delete curTimeNum;
	delete curTimeDen;
	curTimeNum = NULL;
	curTimeDen = NULL;
}
void XMLMillaHandler::startBeam(AttributeList& attrs){curCondType = BEAM;}
void XMLMillaHandler::endBeam(){curCondType = NULL;}
void XMLMillaHandler::startPreceed(AttributeList& attrs)
{
	curCondType = PRECEED;
	curFirstSym = NULL;
	curSecondSym = NULL;
}
void XMLMillaHandler::endPreceed()
{
	setCondizione(curFirstSym, curSecondSym);
	

	curCondType = NULL;
}

void XMLMillaHandler::startSymbolPrec(AttributeList& attrs)
{
	curPrecSym = new int();
}

void XMLMillaHandler::endSymbolPrec()
{
	if (curFirstSym == NULL)
		curFirstSym = *curPrecSym;
	else
		curSecondSym = *curPrecSym;
	
	delete curPrecSym;
	curPrecSym = NULL;
}


void XMLMillaHandler::endRange(int token, double min, double max)
{
	if (CurrentColImm<20)
	{
		if (CurrentCondImm!=NULL)
		{
			CurrentCondImm->condizioni[CurrentColImm].cond=token;
			CurrentCondImm->condizioni[CurrentColImm].range.min=min;
			CurrentCondImm->condizioni[CurrentColImm++].range.max=max;
			CurrentCondImm->numCond++;
		}
	}
	if (CurrentColPos<20)
	{
		if (CurrentCondPos!=NULL)
		{
			CurrentCondPos->condizioni[CurrentColPos].cond=token;
			CurrentCondPos->condizioni[CurrentColPos].range.min=min;
			CurrentCondPos->condizioni[CurrentColPos++].range.max=max;
			CurrentCondPos->numCond++;
		}
	} 	
	curHeight = NULL;
}

void XMLMillaHandler::startCondWith(AttributeList& attrs)
{
	curCondWithCode = new int();
}

void XMLMillaHandler::endCondWith()
{
	setCondizione(NOTAWITH, *curCondWithCode);
	delete curCondWithCode;
	curCondWithCode = NULL;
}

void XMLMillaHandler::startHead(AttributeList& attrs)
{
	curHead = new int();
}

void XMLMillaHandler::endHead()
{
	setCondizione(HEAD, *curHead);
	delete curHead;
	curHead = NULL;
}

void XMLMillaHandler::startExpression(char* value)
{
	curExpression = value;
}

void XMLMillaHandler::endExpression()
{
	if (CurrentColImm<20)
	{
		if (CurrentCondImm!=NULL)
		{
			CurrentCondImm->condizioni[CurrentColImm].cond=RELATION;
			CurrentCondImm->condizioni[CurrentColImm].conditionrel.f1=GetKeyCode(curFirstFactor);
			if (curOperator != NULL){
				CurrentCondImm->condizioni[CurrentColImm].conditionrel.Op=GetKeyCode(curOperator);
				CurrentCondImm->condizioni[CurrentColImm].conditionrel.f2=GetKeyCode(curSecondFactor);
			}else{
				CurrentCondImm->condizioni[CurrentColImm].conditionrel.Op=99;
				CurrentCondImm->condizioni[CurrentColImm].conditionrel.f2=99;
			}
			CurrentCondImm->condizioni[CurrentColImm].conditionrel.res=GetKeyCode(curExpression);
			CurrentCondImm->condizioni[CurrentColImm++].conditionrel.num=*curRefValue;
			CurrentCondImm->numCond++;
		}
	} 
	if (CurrentColPos<20)
	{
		if (CurrentCondPos!=NULL)
		{
			CurrentCondPos->condizioni[CurrentColPos].cond=RELATION;
			CurrentCondPos->condizioni[CurrentColPos].conditionrel.f1=GetKeyCode(curFirstFactor);
			if (curOperator != NULL){
				CurrentCondPos->condizioni[CurrentColPos].conditionrel.Op=GetKeyCode(curOperator);
				CurrentCondPos->condizioni[CurrentColPos].conditionrel.f2=GetKeyCode(curSecondFactor);
			}else{
				CurrentCondPos->condizioni[CurrentColPos].conditionrel.Op=99;
				CurrentCondPos->condizioni[CurrentColPos].conditionrel.f2=99;
			}
			CurrentCondPos->condizioni[CurrentColPos].conditionrel.res=GetKeyCode(curExpression);
			CurrentCondPos->condizioni[CurrentColPos++].conditionrel.num=*curRefValue;
			CurrentCondPos->numCond++;
		}
	}
			 
	curExpression = NULL;
	curFirstFactor = NULL;
	curSecondFactor = NULL;
	curOperator = NULL;

	delete curRefValue;
	curRefValue = NULL;
}

void XMLMillaHandler::startOperator(char* value)
{
	curOperator = value;
}


void XMLMillaHandler::startFactor(char* value)
{
	if (curFirstFactor == NULL)
		curFirstFactor = value;
	else
		curSecondFactor = value;
}

void XMLMillaHandler::startRefValue(AttributeList& attrs)
{
	curRefValue = new double();
}


void XMLMillaHandler::startTimeNum(AttributeList& attrs){ curTimeNum = new double(); }
void XMLMillaHandler::startTimeDen(AttributeList& attrs){ curTimeDen = new double(); }

void XMLMillaHandler::endTimeNum(){ *curTimeNum = curTime; }
void XMLMillaHandler::endTimeDen(){ *curTimeDen = curTime; }

//--- start/end TOKENS ---//

void XMLMillaHandler::setCat(int lasttoken)
{
	if (CurrentRuleImm!=NULL) 
		CurrentRuleImm->cat=lasttoken;
	
	if (CurrentRulePos!=NULL) 
		CurrentRulePos->cat=lasttoken;	
}

void XMLMillaHandler::setStaff(int lasttoken)
{
	if (CurrentRulePos!=NULL) 
	{
		CurrentRulePos->staff=lasttoken;
	}	
}

void XMLMillaHandler::setSymbol(int lasttoken)
{
	if (CurrentRuleImm!=NULL) 
	{
		CurrentRuleImm->Symbol=lasttoken;
	}
	
	if (CurrentRulePos!=NULL) 
	{
		CurrentRulePos->Symbol=lasttoken;
	}
} 

void XMLMillaHandler::setCol(int lasttoken)
{ 
	if (CurrentColImm<20)
	{
		if (CurrentCondImm!=NULL)
		{
			CurrentCondImm->condizioni[CurrentColImm++].cond=lasttoken;
			CurrentCondImm->numCond++;
		}
	}
	if (CurrentColPos<20)
	{
		if (CurrentCondPos!=NULL)
		{
			CurrentCondPos->condizioni[CurrentColPos++].cond=lasttoken;
			CurrentCondPos->numCond++;
		}
	} 
}

void XMLMillaHandler::setCondizione(int token, int lasttoken)
{
	if (CurrentColImm<20)
	{
		if (CurrentCondImm!=NULL)
		{
			CurrentCondImm->condizioni[CurrentColImm++].cond=1000*token+lasttoken;
			CurrentCondImm->numCond++;
		}
	}
	if (CurrentColPos<20)
	{
		if (CurrentCondPos!=NULL)
		{
			CurrentCondPos->condizioni[CurrentColPos++].cond=1000*token+lasttoken;
			CurrentCondPos->numCond++;	
		}
	} 
} 

