/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */



#include "mil/posiz.hpp"
#include "mil/rulemanager.hpp"
#include "mil/m.tab.h"
#include "ind/espgenerica.hpp"
#include "int/raggruppamento.hpp"

#define VUDIMNOTA 7

ITEMPOS Posiz::ItemListPos[MAXNUMITEM];
CONDITIONPOS Posiz::ConditionListPos[MAXNUMCONDITION];
int Posiz::CurrentConditionPosUsage = 0;
int Posiz::CurrentItemPosUsage = 0;

extern int DebugNormalMilla;
extern int DebugNormalMillaPos;
#include "wx/wx.h"
extern wxString msgdebug;
wxString msgcond;
wxString msgtoken;

int dumpcontrollo=0;
////////////////


Point ITEMPOS::GetPoint(int lgambo)
{
	
	Point point;
	if (lgambo>0)
	{
		point.x = int(dx*(VUDIMNOTA-1.5));
		point.y = int(dy*VU_PER_ROW);
	} 
	else if (lgambo<0)
	{
		point.x = int(dx*(VUDIMNOTA+2.5));
		point.y = int(dy*VU_PER_ROW);
	}  
	else if (lgambo==0)
	{
		point.x = int(dx*VUDIMNOTA);  
		point.y = int(dy*VU_PER_ROW);
	} 
	return point;
}



void Errore(char * comm, char * nom, int num);

Posiz::Posiz()
{
	CurrentConditionPosUsage = 0;
	CurrentItemPosUsage = 0;
	
	
} 

void Posiz::Reset()
{
	CurrentConditionPosUsage = 0;
	CurrentItemPosUsage = 0;
} 

/*****************************************************************************/

/* Compare functions for the binary searchs on Keywords or Identifiers */
int  CompareItemPos(const void *  Element1, const void *  Element2 )
{ 
	return (strcmp( ((PITEMPOS) Element1)->ItemName,
		((PITEMPOS) Element2)->ItemName)); 
}

/*****************************************************************************/

int CompareSymbolPos(const void *  Element1, const void *  Element2 )
{ 
	return (((PITEMPOS)Element1)->Symbol - ((PITEMPOS)Element2)->Symbol);
}

/*****************************************************************************/

int CompareConditionPos(const void *  Element1, const void *  Element2 )
{ 
	int conf;
	conf = (((PCONDITIONPOS) Element1)->Symbol - 
		((PCONDITIONPOS) Element2)->Symbol); 
	if (conf ==0)
	{
		return (((PCONDITIONPOS) Element1)->numCond - 
			((PCONDITIONPOS) Element2)->numCond);
	}
	else return conf; 	   
}

/*************************************************************************/

int CompareConditionSyPos(const void *	Element1, const void *	Element2 )
{ 
	
	return (((PCONDITIONPOS) Element1)->Symbol - 
		((PCONDITIONPOS) Element2)->Symbol);		   
}

/****************************************************************************/

void Posiz::InitRulePos()
{  
	int i;
	PCONDITIONPOS ConditionPos;
	
	for (i=0; i<CurrentConditionPosUsage; i++)
	{
		ConditionPos = &ConditionListPos[i];
		ConditionPos->Rule = searchRulePos(ConditionPos->RuleName);
		ConditionPos->Symbol = (ConditionPos->Rule)->Symbol;  
	} 
}  

/**************************************************************************/

PITEMPOS Posiz::AddItemPos(char * Item, int ItemType)
{
	
	ITEMPOS CurrentItemPos;
	
	ItemListPos[CurrentItemPosUsage].ItemName = (char *)malloc(strlen(Item) + 1);
	if (ItemListPos[CurrentItemPosUsage].ItemName==NULL)
							 Errore("allocation","ItemName",0);
	strcpy(ItemListPos[CurrentItemPosUsage].ItemName, Item);
	
	ItemListPos[CurrentItemPosUsage].ItemType = ItemType;
	ItemListPos[CurrentItemPosUsage].Type = 99;
	ItemListPos[CurrentItemPosUsage].Symbol = 99;
	ItemListPos[CurrentItemPosUsage].rel = 99;
	ItemListPos[CurrentItemPosUsage].cat = 99;
	ItemListPos[CurrentItemPosUsage].staff = 99;
	ItemListPos[CurrentItemPosUsage].lenght = 99;
	ItemListPos[CurrentItemPosUsage].dx = 99;
	ItemListPos[CurrentItemPosUsage].dy = 99;
	ItemListPos[CurrentItemPosUsage].rangelenght.min = -99;
	ItemListPos[CurrentItemPosUsage].rangelenght.max = 99;
	ItemListPos[CurrentItemPosUsage].rangedx.min = -99;
	ItemListPos[CurrentItemPosUsage].rangedx.max = 99;
	ItemListPos[CurrentItemPosUsage].rangedy.min = -99;
	ItemListPos[CurrentItemPosUsage].rangedy.max = 99;
	ItemListPos[CurrentItemPosUsage].scale = 99;
	ItemListPos[CurrentItemPosUsage].angle = 99;
	ItemListPos[CurrentItemPosUsage].shape = 99;
	CurrentItemPosUsage++;
	
	/* Sort the list for future search */
	qsort((char *)&ItemListPos[0], CurrentItemPosUsage, sizeof(ITEMPOS),
		CompareItemPos);
		/* Adesso cerco dove e' stata posizionata la mia regola dopo il qsort
	altrimenti non so dove mi mette gli altri campi della regola*/
	CurrentItemPos.ItemName = Item;
	return (ITEMPOS *) bsearch(&CurrentItemPos, &ItemListPos[0], 
		CurrentItemPosUsage, sizeof(ITEMPOS), CompareItemPos);
	
}


/**********************************************************************/

PCONDITIONPOS Posiz::AddConditionPos()
{
	for(int i=0;i<NUM_MAX_CONDIZIONI;i++)
	{
		ConditionListPos[CurrentConditionPosUsage].condizioni[i].conditionrel.f1=99;
		ConditionListPos[CurrentConditionPosUsage].condizioni[i].conditionrel.Op=99;
		ConditionListPos[CurrentConditionPosUsage].condizioni[i].conditionrel.f2=99;
		ConditionListPos[CurrentConditionPosUsage].condizioni[i].conditionrel.res=99;
		ConditionListPos[CurrentConditionPosUsage].condizioni[i].conditionrel.num=99;
		ConditionListPos[CurrentConditionPosUsage].condizioni[i].range.max=99;
		ConditionListPos[CurrentConditionPosUsage].condizioni[i].range.min=-99;
		ConditionListPos[CurrentConditionPosUsage].condizioni[i].cond=99;
	}
	
	ConditionListPos[CurrentConditionPosUsage].Symbol=99;
	ConditionListPos[CurrentConditionPosUsage].numCond=0;
	CurrentConditionPosUsage++;
	
	return &ConditionListPos[CurrentConditionPosUsage-1];
}

/******************************************************************/	

float Posiz::PosRule(DrawObject* drw, Nota* pNota, Battuta*
					 pBattuta, Bool gamboupdwn, short current_layer, short current_staff)
{
	
	PITEMPOS RulePos;
	if(drw->GetID()==CL_RAGGRUPPAMENTO) //FFF
	RulePos = searchConditionPos(tab[drw->GetID()], (Nota*)(((Raggruppamento*)drw)->GetFirstFigura()), pBattuta,
								drw, gamboupdwn, current_layer,current_staff,1);

	else RulePos = searchConditionPos(tab[drw->GetID()], pNota, pBattuta,
								drw, gamboupdwn, current_layer,current_staff,1);
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		wxString tmp;
		if (RulePos!=NULL)
		{
			//traccia lo stato
			RulePos = searchConditionPos(tab[drw->GetID()], pNota, pBattuta,
				drw, gamboupdwn, current_layer, current_staff, 1);
			tmp.sprintf("Applied Rule: %s \n",
				RulePos->ItemName );
			
			msgdebug.Append(tmp);
		}
	}
	
	
	
	if (RulePos!=NULL)
	{
		if (RulePos->Symbol==STEM)
		{
			if (RulePos->lenght==STEMHEIGHT) 
				if (CL_ACCORDO!=pNota->GetID())
					return float(abs((pNota->GetAltezza()-4)*VU_PER_ROW)/2);
				else {
					// per gestire la lunghezza dei gambi degli accordi
					Accordo* pacc=(Accordo *)pNota;	
					if(pacc->GetFirstNote()->GetNStaff()==pacc->GetLastNote()->GetNStaff())
					{
						int d1=(abs((pacc->GetFirstNote()->GetAltezza()-4)*VU_PER_ROW)/2);
						int d2=(abs((pacc->GetLastNote()->GetAltezza()-4)*VU_PER_ROW)/2);
						if (d1<d2)  return d1;
						else return d2;
					}
					else
					{
						Message("MILLA: STEMHEIGHT rule cannot be used for multistaff chords.\n"
							"using 3.5 stem instead");
						return 3.5*VU_PER_ROW;
					}
				}
				else 		   
				{
					return float((RulePos->lenght)*VU_PER_ROW);
				}
		}	
		if ((RulePos->Symbol==BEAM)&&(drw->GetID()==CL_GRUPPONOTE))
		{
			if (RulePos->angle==BSLOPE)
			{
				VUnit dx,dy;
				float BSlope;
				
				dy=(((GruppoNote*)pNota)->GetLastNota())->GetAbsPos().y-
					(((GruppoNote*)pNota)->GetFirstNota())->GetAbsPos().y;
				dx=(((GruppoNote*)pNota)->GetLastNota())->GetAbsPos().x-
					(((GruppoNote*)pNota)->GetFirstNota())->GetAbsPos().x;
				
				BSlope = (float)dy / dx;
				if(BSlope>0)
				{
					if(BSlope < RulePos->rangeangle.min) BSlope = RulePos->rangeangle.min;
					if(BSlope > RulePos->rangeangle.max) BSlope = RulePos->rangeangle.max;
				}
				else
				{
					if(BSlope > -RulePos->rangeangle.min) BSlope = -RulePos->rangeangle.min;
					if(BSlope < -RulePos->rangeangle.max) BSlope = -RulePos->rangeangle.max;
				}
				return BSlope;
			}
			else return float(RulePos->angle);	
		}
		
		if ((RulePos->Symbol==BEAM) && (drw->GetID()==CL_RAGGRUPPAMENTO))	//FFF
		{
			//if (RulePos->angle==BSLOPE)
			//{
			VUnit dx,dy;
			float BSlope;
			
			Figura *firstfig =((Raggruppamento*)drw)->GetFirstFigura();	//FFF
			Figura *lastfig =((Raggruppamento*)drw)->GetLastFigura();
			
			if((lastfig!=NULL)&&(firstfig!=NULL)){
				dy=lastfig->GetAbsPos().y-firstfig->GetAbsPos().y;
				dx=lastfig->GetAbsPos().x-firstfig->GetAbsPos().x;
			}
			
			BSlope = (float)dy / dx;
			if(BSlope>0)
			{
				if(BSlope < RulePos->rangeangle.min) BSlope = RulePos->rangeangle.min;
				if(BSlope > RulePos->rangeangle.max) BSlope = RulePos->rangeangle.max;
			}
			else
			{
				if(BSlope > -RulePos->rangeangle.min) BSlope = -RulePos->rangeangle.min;
				if(BSlope < -RulePos->rangeangle.max) BSlope = -RulePos->rangeangle.max;
			}
			return BSlope;
			//}
			//else return float(RulePos->angle);	 
			
		}
		if (RulePos->Symbol==BATTUTA)
			return float(RulePos->lenght);  
	}	 
	return 0.;
} 

/******************************************************************/	

float Posiz::PosRuleBeam(DrawObject* drw, Nota* pNota, Battuta*
						 pBattuta, Bool gamboupdwn, short current_layer, RANGE* coppie,short current_staff)
{
	
	PITEMPOS RulePos;
	RulePos = searchConditionPos(tab[drw->GetID()], pNota, pBattuta,
		drw, gamboupdwn, current_layer,current_staff,0);
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		wxString tmp;
		if (RulePos!=NULL)
		{
			// traccia cond milla
			RulePos = searchConditionPos(tab[drw->GetID()], pNota, pBattuta,
				drw, gamboupdwn, current_layer, current_staff, 1);
			tmp.sprintf("Applied Rule: %s \n",
				RulePos->ItemName );
			
			msgdebug.Append(tmp);
		}
	}
	
	
	if (RulePos!=NULL)
	{
		if (RulePos->Symbol==BATTUTA)
		{
			
			return float(RulePos->lenght);  
		} 
	}	  
	return 0.;
}

/******************************************************************/	



Bool Posiz::PosRuleTer(DrawObject* drw, Nota* pNota, Battuta*
					   pBattuta, Bool gamboupdwn, short current_layer,short current_staff)
{
	
	PITEMPOS RulePos;
	RulePos = searchConditionPos(tab[drw->GetID()], pNota, pBattuta,
		drw, gamboupdwn, current_layer,current_staff,0);
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		wxString tmp;
		if (RulePos!=NULL)
		{
			// traccia cond milla
			RulePos = searchConditionPos(tab[drw->GetID()], pNota, pBattuta,
				drw, gamboupdwn, current_layer, current_staff, 1);
			
			tmp.sprintf("Applied Rule: %s \n",
				RulePos->ItemName );
			
			msgdebug.Append(tmp);
		}
	}
	
	
	if (RulePos!=NULL)
	{
		if (RulePos->Symbol==BATTUTA)
		{
			if(RulePos->abilterzine==ON) return TRUE;
			if(RulePos->abilterzine==OFF) return FALSE;  
		} 
	}      
	return 0;
}


/************************************************************/

PITEMPOS Posiz::PosRulePunt(DrawObject* drw, Nota* pNota, Battuta*
							pBattuta, Bool gamboupdwn, short current_layer,short current_staff, int traceconditionsmilla)
{
	
	PITEMPOS RulePos;
	if(drw->GetID()==CL_ESPGENERIC)
	{ 
		RulePos=searchConditionPos(((EspGenerica*)drw)->GetIDgroup()+1000,pNota, pBattuta,
			drw, gamboupdwn, current_layer,current_staff,0);
	}
	else
	{							  
		RulePos = searchConditionPos(tab[drw->GetID()], pNota, pBattuta,
			drw, gamboupdwn, current_layer,current_staff,0);
	} 						 
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		
		wxString tmp;
		if (RulePos!=NULL)
		{
			if(drw->GetID()==CL_ESPGENERIC)
			{ 
				RulePos=searchConditionPos(((EspGenerica*)drw)->GetIDgroup()+1000,pNota, pBattuta,
					drw, gamboupdwn, current_layer, current_staff, 0);
			}
			else
			{ 						  
				RulePos = searchConditionPos(tab[drw->GetID()], pNota, pBattuta,
					drw, gamboupdwn, current_layer,current_staff, 0);
			} 						 
			
			
			tmp.sprintf("Applied Rule: %s \n",
				RulePos->ItemName );
			
			msgdebug.Append(tmp);
		}
	}
	
	
	return RulePos;
}  


/******************************************************************/	

Point Posiz::PosRuleP(DrawObject* drw, Nota* pNota, Battuta*
					  pBattuta, Bool gamboupdwn, short current_layer,short current_staff)
{
	Point p;
	
	PITEMPOS RulePos;
	
	RulePos = PosRulePunt(drw, pNota, pBattuta, gamboupdwn, current_layer,current_staff,0); 
	
	
	if (DebugNormalMilla || DebugNormalMillaPos)
	{ 
		wxString tmp;
		if (RulePos!=NULL)
		{
			RulePos = PosRulePunt(drw, pNota, pBattuta, gamboupdwn, current_layer, current_staff,0); 
			tmp.sprintf("Applied Rule: %s \n",
				RulePos->ItemName );
			
			msgdebug.Append(tmp);
		}
	}
	
	
	
	
	
	if (RulePos!=NULL)
	{
		p = RulePos->GetPoint(pNota->GetLGambo());
		return p;
	}
	else return Point(0,0);	   
}	 


/***************************************************************************/

int Posiz::PosRuleHead(DrawObject* drw, Nota* pNota, Battuta*
					   pBattuta, Bool gamboupdwn, short current_layer, short current_staff)
{
	
	PITEMPOS RulePos;
	RulePos = searchConditionPos(HEAD, pNota, pBattuta,
								drw, gamboupdwn, current_layer, current_staff,0);
	if(RulePos!=NULL)
	{
		if (DebugNormalMilla || DebugNormalMillaPos)
		{ 
			wxString tmp;
			if (RulePos!=NULL)
			{
				RulePos = searchConditionPos(HEAD, pNota, pBattuta,
					drw, gamboupdwn, current_layer, current_staff,0);
				
				tmp.sprintf("Applied Rule: %s \n",
					RulePos->ItemName );
				
				msgdebug.Append(tmp);
			}
		}
		
		
		
		
		
		return RulePos->code;
	}
	return 0;  
}

/******************************************************************/	

void Posiz::PosRuleStartGambo(DrawObject* drw, Nota* pNota, Battuta*
							  pBattuta, Bool gamboupdwn, short current_layer, short current_staff)
{
	
	PITEMPOS RulePos;
	RulePos = searchConditionPos(STEMSTART, pNota, pBattuta,
		drw, gamboupdwn, current_layer, current_staff,0);
	
	if(RulePos!=NULL)
	{
		
		
		if (DebugNormalMilla || DebugNormalMillaPos)
		{ 
			wxString tmp;
			if (RulePos!=NULL)
			{
				RulePos = searchConditionPos(STEMSTART, pNota, pBattuta,
					drw, gamboupdwn, current_layer, current_staff, 0);
				
				tmp.sprintf("Applied Rule: %s \n",
					RulePos->ItemName );
				
				msgdebug.Append(tmp);
			}
		}
		
		
		
		
		
		pNota->SetDxGambo(int(RulePos->dx));
		pNota->SetDyGambo(int(RulePos->dy));
	}
}

/******************************************************************/	

PITEMPOS Posiz::searchConditionPos(int a, Nota* pNota, Battuta* pBattuta,
								   DrawObject* drw, Bool gamboupdwn, short current_layer,short current_staff, int traceconditionsmilla)
								   
{
	Bool result=FALSE;   
	wxString pip;
	
	PCONDITIONPOS SearchConditionPos=NULL;
	
	CONDITIONPOS CurrentConditionPos;
	CurrentConditionPos.Symbol = a;
	
	SearchConditionPos = (CONDITIONPOS *) bsearch(&CurrentConditionPos, 
		&ConditionListPos[0], CurrentConditionPosUsage, sizeof(CONDITIONPOS), 
		CompareConditionSyPos);
	
	if (SearchConditionPos!=NULL)
	{   
		while (SearchConditionPos-ConditionListPos<CurrentConditionPosUsage &&
			SearchConditionPos->Symbol==a)
		{
			SearchConditionPos++;
		}
	} 
	
	if (SearchConditionPos!=NULL)
	{
		SearchConditionPos--;
		
		while (SearchConditionPos-ConditionListPos>=0 && SearchConditionPos->Symbol==a)
		{
			
			int j;
			result=FALSE; 
			
			for(j=0; j<NUM_MAX_CONDIZIONI; j++)
			{
				if (SearchConditionPos->condizioni[j].cond!=99)
				{
					result = ValutaCondizioni(&SearchConditionPos->condizioni[j], pNota,
						pBattuta, drw, current_layer, gamboupdwn, traceconditionsmilla);
					if(!result) 
						break;
				}
				else
					break;
			}				 
/*****/			
			if (result)
				break;
			
			SearchConditionPos--;	
		}
	}
	
	if (DebugNormalMilla || DebugNormalMillaPos) 
	{ 
		if (result)
		{
			msgdebug.Append(msgcond);	   
			msgdebug.Append(pip);
		}
		else
		{
			msgdebug.Append("No rule found!");
		}
	} 
	
	if (result)
	{
		return SearchConditionPos->Rule;
	}
	if ((drw->GetID()==CL_RAGGRUPPAMENTO)&&(SearchConditionPos->Rule!=NULL))
	{
		SearchConditionPos++;
		return SearchConditionPos->Rule;
	}
	return NULL;

/******************************			
			if (result)
			{
				if (DebugNormalMilla || DebugNormalMillaPos) //nesopt
				{ msgdebug.Append(msgcond);	   
				msgdebug.Append(pip);
				} 
				return SearchConditionPos->Rule;
			} 	
										  
			if ((drw->GetID()==CL_RAGGRUPPAMENTO)&&(SearchConditionPos->Rule!=NULL))	//FFF
			{
				return SearchConditionPos->Rule;
			}
			
			SearchConditionPos--;
			
			if (DebugNormalMilla || DebugNormalMillaPos) //nesopt
				msgcond.sprintf("");	
			
		}
	}
	return NULL;
*******************************/	
}

/*************************************************************************/

PITEMPOS Posiz::searchRulePos(char* ruleName)
{
	
	ITEMPOS CurrentItemPos;
	CurrentItemPos.ItemName = ruleName;
	
	return (ITEMPOS *) bsearch(&CurrentItemPos, &ItemListPos[0], 
		CurrentItemPosUsage, sizeof(ITEMPOS), CompareItemPos); 
	
} 

/*****************************************************************************/

PITEMPOS Posiz::SearchNamePos(char *name)
{ 
	ITEMPOS item;
	item.ItemName = name;
	return (ITEMPOS *) bsearch(&item, &ItemListPos[0],
		CurrentItemPosUsage, sizeof(ITEMPOS), CompareItemPos);
}

/*********************************************************************/

void Posiz::sortConditionPos(void)
{
	qsort((char *)&ConditionListPos[0], CurrentConditionPosUsage, 
		sizeof(CONDITIONPOS), CompareConditionPos);
}  

/*****************************************************************************
* show the list of Item di posizionamento
*****************************************************************************/
void Posiz::seeItemListPos(void )
{ 
	int i;
	for (i=0;i<CurrentItemPosUsage;i++)
	{  
		fprintf(stdout,"Rulename %s, RuleType %d, Type %d ",
			ItemListPos[i].ItemName,
			ItemListPos[i].ItemType,
			ItemListPos[i].Type);
		if(ItemListPos[i].Symbol!=99)
		{
			fprintf(stdout, "Symbol %d ",ItemListPos[i].Symbol);
		}
		if(ItemListPos[i].rel!=99)
		{
			fprintf(stdout, "rel %d ",ItemListPos[i].rel);
		}	 
		if(ItemListPos[i].cat!=99)
		{
			fprintf(stdout, "cat %d ",ItemListPos[i].cat);
		}	
		if(ItemListPos[i].staff!=99)
		{
			fprintf(stdout, "staff %d ",ItemListPos[i].staff);
		}	 
		if(ItemListPos[i].lenght!=99)
		{
			fprintf(stdout, "lenght %g ",ItemListPos[i].lenght);
		}
		if(ItemListPos[i].dx!=99)
		{
			fprintf(stdout, "DX: %g ",ItemListPos[i].dx);
		}
		if(ItemListPos[i].dy!=99)
		{
			fprintf(stdout, "DY: %g ",ItemListPos[i].dy);
		}
		if(ItemListPos[i].rangelenght.min!=-99)
		{
			fprintf(stdout, "lenghtmin: %g ",ItemListPos[i].rangelenght.min);
		}
		if(ItemListPos[i].rangelenght.max!=99)
		{
			fprintf(stdout, "lenghtmax: %g ",ItemListPos[i].rangelenght.max);
		}
		if(ItemListPos[i].rangedx.min!=-99)
		{
			fprintf(stdout, "dxmin: %g ",ItemListPos[i].rangedx.min);
		}
		if(ItemListPos[i].rangedx.max!=99)
		{
			fprintf(stdout, "dxmax: %g ",ItemListPos[i].rangedx.max);
		}
		if(ItemListPos[i].rangedy.min!=-99)
		{
			fprintf(stdout, "dymin: %g ",ItemListPos[i].rangedy.min);
		}
		if(ItemListPos[i].rangedy.max!=99)
		{
			fprintf(stdout, "dymax: %g ",ItemListPos[i].rangedy.max);
		}	 
		if(ItemListPos[i].scale!=99)
		{
			fprintf(stdout, "scale %g ",ItemListPos[i].scale);
		}
		if(ItemListPos[i].angle!=99)
		{
			fprintf(stdout, "angle %g ",ItemListPos[i].angle);
		}
		if(ItemListPos[i].shape!=99)
		{
			fprintf(stdout, "shape %d ",ItemListPos[i].shape);
		}
	}
}

/*****************************************************************************
* show the list of Conditionpos
*****************************************************************************/
void Posiz::seeConditionListPos(void )
{ 
	int i;
	for (i=0;i<CurrentConditionPosUsage;i++)
	{  
		int j;
		for (j=0; j<NUM_MAX_CONDIZIONI; j++)
		{
			if(ConditionListPos[i].condizioni[j].cond!=99)
				fprintf(stdout, "C:%d ",ConditionListPos[i].condizioni[j].cond);
			
			if(ConditionListPos[i].condizioni[j].range.min!=-99)
				fprintf(stdout, "X:%g ",ConditionListPos[i].condizioni[j].range.min);
			
			if(ConditionListPos[i].condizioni[j].range.max!=99)
				fprintf(stdout, "Y:%g ",ConditionListPos[i].condizioni[j].range.max);
			
			if(ConditionListPos[i].condizioni[j].conditionrel.f1!=99)
				fprintf(stdout, "f1:%d ",ConditionListPos[i].condizioni[j].conditionrel.f1);
			
			if(ConditionListPos[i].condizioni[j].conditionrel.Op!=99)
				fprintf(stdout, "Op:%d ",ConditionListPos[i].condizioni[j].conditionrel.Op); 
			
			if(ConditionListPos[i].condizioni[j].conditionrel.f2!=99)
				fprintf(stdout, "f2:%d ",ConditionListPos[i].condizioni[j].conditionrel.f2);
			
			if(ConditionListPos[i].condizioni[j].conditionrel.res!=99)
				fprintf(stdout, "res:%d ",ConditionListPos[i].condizioni[j].conditionrel.res); 
			
			if(ConditionListPos[i].condizioni[j].conditionrel.num!=99)
				fprintf(stdout, "num:%lf ",ConditionListPos[i].condizioni[j].conditionrel.num);	  
		}
		fprintf(stdout," RuleName: %s,"
			" Simbolo: %d,"
			" NumCondiz.: %d\n",
			ConditionListPos[i].RuleName,
			ConditionListPos[i].Symbol,   
			ConditionListPos[i].numCond);	
	}
}

/*************************************************************************/   

void Posiz::sortSymbolPos(void)
{
	qsort((char *)&ItemListPos[0], CurrentItemPosUsage, sizeof(ITEMPOS), 
		CompareSymbolPos);  
}	

/*************************************************************************/   

void Posiz::sortRulePos(void)
{
	qsort((char *)&ItemListPos[0], CurrentItemPosUsage, sizeof(ITEMPOS), 
		CompareItemPos);  
}  


void Posiz::dump(void)
{
	
	int j=0, k=0;
	int controllo=0;
	
	FILE *pFileDati;
	if ((pFileDati=fopen ("C://Debug/dumpmillaposizionamento.dat","w"))==NULL)
		printf ("Errore nell'apertura del file dumpmillaposizionamento.dat\n");
	else {
		fprintf(pFileDati,"\n\nREGOLE DI POSIZIONAMENTO\n");
		fprintf(pFileDati,"\nItemName ItemType Type Symbol rel cat staff lenght code dx dy regola ranglenght.min ranglenght.max rangedx.min rangedx.max rangedy.min rangedy.max rangeangle.min rangeangle.max scale angle shape coppie[0] abilterzine ");
		
		for (j=0; j<MAXNUMITEM; j++)
		{
			
			if (ItemListPos[j].ItemName!=NULL)
				fprintf(pFileDati,"\n[ %s,  %d,  %d,  %d, %d, %d, %d, %f, %d, %f, %f,  %s, %f, %f, %f, %f, %f, %f, %f, %f, %f, %f, %d, %f, %f, %d  ] ",ItemListPos[j].ItemName, ItemListPos[j].ItemType, ItemListPos[j].Type, ItemListPos[j].Symbol, ItemListPos[j].rel, ItemListPos[j].cat, ItemListPos[j].staff, ItemListPos[j].lenght, ItemListPos[j].code, ItemListPos[j].dx, ItemListPos[j].dy, ItemListPos[j].regola, ItemListPos[j].rangelenght.min, ItemListPos[j].rangelenght.max, ItemListPos[j].rangedx.min, ItemListPos[j].rangedx.max, ItemListPos[j].rangedy.min, ItemListPos[j].rangedy.max, ItemListPos[j].rangeangle.min, ItemListPos[j].rangeangle.max, ItemListPos[j].scale, ItemListPos[j].angle, ItemListPos[j].shape, ItemListPos[j].coppie[0].min, ItemListPos[j].abilterzine );
		}
		fprintf(pFileDati,"\n\nCONDIZIONI DI POSIZIONAMENTO\n");
		
		for (j=0; j<MAXNUMCONDITION; j++)
		{
			if (ConditionListPos[j].RuleName!=NULL)
			{	
				
				fprintf(pFileDati,"\n\n RuleName, Symbol,  numCond");
				fprintf(pFileDati,"\n[ %s,  %d,  %d ] ", ConditionListPos[j].RuleName, ConditionListPos[j].Symbol, ConditionListPos[j].numCond );
				fprintf(pFileDati,"\nSingoli Token verificati");
				fprintf(pFileDati,"\ncondizioni[%d].cond, condizioni[%d].RANGEmin, condizioni[%d].RANGEmax condizioni[%d].conditionrel.f1, condizioni[%d].conditionrel.Op, condizioni[%d].conditionrel.f2, condizioni[%d].conditionrel.res, condizioni[%d].conditionrel.num", k, k, k, k, k, k, k, k, k);
				
				for (k=0; k<ConditionListPos[j].numCond; k++)
				{
					
					fprintf(pFileDati,"\n[ %d, %f, %f, %d, %d, %d, %d, %f ] ",ConditionListPos[j].condizioni[k].cond, ConditionListPos[j].condizioni[k].range.min, ConditionListPos[j].condizioni[k].range.max, ConditionListPos[j].condizioni[k].conditionrel.f1, ConditionListPos[j].condizioni[k].conditionrel.Op, ConditionListPos[j].condizioni[k].conditionrel.f2, ConditionListPos[j].condizioni[k].conditionrel.res, ConditionListPos[j].condizioni[k].conditionrel.num);
				}
			}
		}
	}
	fflush(pFileDati);

	controllo=fclose (pFileDati);
}  

// traccia tutti i token relativi a un simbolo dicendo se sono
// verificati oppure no
void Posiz::TestTokenCond(Nota* pNota, DrawObject* drw, Battuta* pBattuta, Bool gamboupdown, short layer)
{
	
	msgtoken.sprintf("");
	msgtoken.sprintf("State verified     Token \n"); 
	
	
	ValutaCondizioniDebug(pNota, pBattuta, drw, layer, gamboupdown);
	
}