/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */



#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "liooext.hpp" 
#include "mil/rulemanager.hpp"
#include "mil/m.tab.h"
#include "lioo.hpp"

extern int DebugNormalMilla;
extern int DebugNormalMillaPos;
#include "wx/wx.h"
extern wxString msgdebug;  // messaggio stampato
extern wxString msgcond;   // token verificati che fanno parte della condizione applicata
extern wxString msgtoken;   // lista dei token, o verificati o non verificati

////////////////


extern void InitLoad();
extern int milparse(void);
extern FILE *milin;
extern int millineno;
extern int milloadError;
int RuleManager::tab[NUMEROID];  

int Errore(char * comm, char * nom, int num);

RuleManager::RuleManager()
{ 
	tab[CL_STACCATO]=STAC;
	tab[CL_TENUTO]=TEN;
	tab[CL_ACCENTO]=ACCE;
	tab[CL_SFORZATO]=SFOR;
	tab[CL_MARTELLATO] = MARTF;
	tab[CL_MARTDOLCE] = MART;
	tab[CL_PUNTOALLUNGATO] = PUNTOALL;
	
	tab[CL_GRUPPONOTE] = BEAM;
	tab[CL_RAGGRUPPAMENTO] = BEAM;	//FFF
	
	tab[CL_TASTIERA] = TASTIERA;
	tab[CL_PUNTA] = PUNTA;
	tab[CL_TALLONE] = TALLONE;
	tab[CL_PONTICELLO] = PONTICELLO; 
	tab[CL_ARCOGIU] = ARCATAGIU;
	tab[CL_ARCOSU] = ARCATASU;
	tab[CL_PIZZICATO] = PIZZICATO;
	tab[CL_ARCO] = ARCO;
	tab[CL_CORDA] = CORDA;
	
	tab[CL_NSEMIBREVE] = STEM;
	tab[CL_NBREVE] = STEM;
	tab[CL_NMINIMA] = STEM;
	tab[CL_NSEMIMINIMA] = STEM;
	tab[CL_NCROMA] = STEM;
	tab[CL_NSEMICROMA] = STEM;
	tab[CL_NBISCROMA] = STEM;
	tab[CL_NSEMIBISCROMA] = STEM;
	tab[CL_NFUSA] = STEM;
	tab[CL_ACCORDO] = STEM;
	
	tab[CL_BATTUTA] = BATTUTA;
	
	tab[CL_MINFERIORE] = MORD;
	tab[CL_MSUPERIORE] = MORD;
	tab[CL_GINFERIORE] = GRUP;
	tab[CL_GSUPERIORE] = GRUP;
	tab[CL_GUP] = GRUP;
	tab[CL_GSLASH] = GRUP;
	tab[CL_TRILLO] = TRILLO;
	
	tab[CL_CONSORDINAOTTONI] = CONSORD;
	tab[CL_CONSORDINATESTO] = CONSORD;
	tab[CL_CONSORDINAARCHI] = CONSORD;
	tab[CL_VIASORDINAOTTONI] = VIASORD;
	tab[CL_VIASORDINATESTO] = VIASORD;
	tab[CL_VIASORDINAARCHI] = VIASORD;
	tab[CL_ARMONICIARCHI] = ARM;
	tab[CL_ARMONICIOTTONI] = ARM;
	tab[CL_DITEGGIATO] = DITEG;
	
	tab[CL_SLIDINGUP]		= KORN;
	tab[CL_SLIDINGDOWN]	= KORN;
	tab[CL_SLIDINGDUP]	= KORN;
	tab[CL_SLIDINGDOWNUP]	= KORN;
	tab[CL_NONGHYNINC]	= KORN;
	tab[CL_NONGHYNDEC]	= KORN;
	tab[CL_NONGHYNARROW]	= KORN;
	tab[CL_NONGHYNWIDE]	= KORN;
	
} 


/*********************************************************************/
float RuleManager::ValutaFactor(int fact, Nota *pNota, DrawObject *drw)
{
	Nota *pn1, *pn2;
	float fac=0;
	if(pNota->GetID()==CL_ACCORDO)
	{
        pn2 = ((Accordo*)pNota)->GetLastNote();
        pn1 = ((Accordo*)pNota)->GetFirstNote();
        switch(fact) 
        {
        case UPPERD:
			fac=pn2->GetAltezza()-4;
			fac=fac>0?fac:-fac;
			break;	
        case LOWERD:
			fac=pn1->GetAltezza()-4;
			fac=fac>0?fac:-fac;
			break;	
        case NUMUP:
			fac=((Accordo*)pNota)->GetNumUp();
			break; 
        case NUMDOWN:
			fac=((Accordo*)pNota)->GetNumDown();
			break;
        }
	} 	
	if(pNota->GetID()==CL_GRUPPONOTE)
	{
        switch(fact)
        {
        case MEAN:
			fac=((GruppoNote*)pNota)->GetMean();		
			break;
        case MHL:
			fac=((GruppoNote*)pNota)->GetMeanHighLow();
			break;
        case DELTA: 
			fac=((GruppoNote*)pNota)->GetDelta();
			break;
        case BEAMDIS:
			fac=((GruppoNote*)pNota)->GetBeamDis();
			break;
		case STAFFNUMBER:
			fac=((GruppoNote*)pNota)->GetStaffInBeam();
			break;
        }	 
	}
	if((drw!=NULL)&&(drw->GetID()==CL_RAGGRUPPAMENTO))//FFF
	{
        switch(fact)
        {
        case MEAN:
			fac=((Raggruppamento*)drw)->GetMean();
			break;
        case MHL:
			fac=((Raggruppamento*)drw)->GetMeanHighLow();
			break;
        case DELTA: 
			fac=((Raggruppamento*)drw)->GetDelta();
			break;
        case BEAMDIS:
			fac=((Raggruppamento*)drw)->GetBeamDis();
			break;
		case STAFFNUMBER:
			fac=((Raggruppamento*)drw)->GetStaffInBeam();
			break;
        }	 
	}
	if(fact==ONSTAFFNUMBER)
		fac=pNota->GetNStaff();
	if(fact==STAFFLINES)
	{
		Battuta *pBat=pNota->GetPBattuta();
		fac=5;
		if(pBat!=NULL)
		{
			Spartito *pSpa=pBat->GetParentScore();
			if(pSpa!=NULL)
			{
				short minStaff,maxStaff;
				if((drw!=NULL)&&(drw->GetID()!=CL_RAGGRUPPAMENTO)) //FFF
				pNota->GetMinMaxStaff(minStaff,maxStaff);
				else if((drw!=NULL)&&(drw->GetID()==CL_RAGGRUPPAMENTO)){
				((Raggruppamento*)drw)->GetMinMaxStaff(minStaff,maxStaff);
				}
				fac=pSpa->GetNCorde(maxStaff);
			}
		}
	}
	if(fact==99) fac=0;
	return fac;  
}     


/*******************************************************************/


Bool RuleManager::ValutaCondizioni(Condizione *pcond, Nota* pNota, Battuta* pBattuta, 
                                   DrawObject* drw, short layer, Bool gamboupdwn, int traceconditionsmilla)
{
	Bool above;
	RANGE NotaRange=pcond->range;
	
	
	switch (pcond->cond)
	{
	case NOTEINSINGLELAYER:
		if(pBattuta!=NULL)
		{
			if (!pBattuta->GetMultiVoice()) {
				if (DebugNormalMilla || DebugNormalMillaPos)
				{ 
					if (traceconditionsmilla==1)
					{
						wxString tmp;
						tmp.sprintf("State verified: NOTEINSINGLELAYER \n");
						msgcond.Append(tmp);	  
					}					
				}
				return TRUE;
			}
		}	 
		break;
	case NOTEINMULTIVOICE:
		if(pBattuta!=NULL)
		{      
			if (pBattuta->GetMultiVoice()) {
				{
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTEINMULTIVOICE \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
			}  
		}	 
		break;    
	case INCHORD:
		if (pNota->GetID()==CL_ACCORDO) {
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: INCHORD \n");
					msgcond.Append(tmp);	  
				}
			}
			return TRUE;
		}
		break; 
	case OPPOSITETOSTEM:
		switch (drw->GetID())
		{  
		case CL_TENUTO:
		case CL_ACCENTO:
		case CL_MARTELLATO:
		case CL_MARTDOLCE:
		case CL_STACCATO:
		case CL_SFORZATO:
			above = ((Espressione*)drw)->GetAboveNota();
			break;
		}	 
		if ((pNota->GetLGambo())>0!=above) {
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: OPPOSITETOSTEM \n");
					msgcond.Append(tmp);	  
				}
			}
			return TRUE;
		}
		break;
		case ONSTEM:
			switch (drw->GetID())
			{        
			case CL_TENUTO:
			case CL_ACCENTO:
			case CL_MARTELLATO:
			case CL_MARTDOLCE:
			case CL_STACCATO:
			case CL_SFORZATO:
				above = ((Espressione*)drw)->GetAboveNota();
				break;
			}	 
			if ((pNota->GetLGambo()>0)==above) {
				
				if (DebugNormalMilla || DebugNormalMillaPos)
				{ 
					if (traceconditionsmilla==1)
					{
						wxString tmp;
						tmp.sprintf("State verified: ONSTEM \n");
						msgcond.Append(tmp);	  
					}
				}
				return TRUE;
			}
			break;    
			case NOTAWITHSTAC: {
				EspressComposta* pespr=NULL;
				Espressione *pstac=NULL;
				if ((pespr = pNota->GetPEspressComposta())!=NULL)
				{
					pstac = pespr->GetPEspressione(CMD_NOTA_STACCATO);
				}
				if (pstac!=NULL) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTAWITHSTAC \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
							   }
				break;
			case NOTAWITHACCE: 	{
				EspressComposta* pespr=NULL;
				Espressione *pacce=NULL;
				if ((pespr = pNota->GetPEspressComposta())!=NULL)
				{
					pacce = pespr->GetPEspressione(CMD_NOTA_ACCENTO);
				}
				if (pacce!=NULL) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTAWITHACCE \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
								}
				break;
			case NOTAWITHTEN: 	{
				EspressComposta* pespr=NULL;
				Espressione *pten=NULL;
				if ((pespr = pNota->GetPEspressComposta())!=NULL)
				{
					pten = pespr->GetPEspressione(CMD_NOTA_TENUTO);
				}
				if (pten!=NULL) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTAWITHTEN \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
								}
				break;
			case NOTE: {
				int alt;
				if (pNota->GetID()==CL_ACCORDO) return FALSE;
				alt = pNota->GetAltezza()-4;
				if (NotaRange.min == VAR)  // range minimo = VAR
					NotaRange.min = -100;
				if (NotaRange.max == VAR)  // range massimo = VAR
					NotaRange.max = 100; 
				if (alt<=NotaRange.max && alt>=NotaRange.min) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTE ");
							msgcond.Append(tmp);	  
							
							if (NotaRange.min == -100)  {  // range minimo = VAR
								tmp.sprintf("[VAR;");
								msgcond.Append(tmp);	  
							}
							else  {
								tmp.sprintf("[%d;",(int)NotaRange.min);
								msgcond.Append(tmp);	  
							}  
							if (NotaRange.max == 100)  {  // range massimo = VAR
								tmp.sprintf("VAR]\n");
								msgcond.Append(tmp);	  
							}
							else  {
								tmp.sprintf("%d]\n",(int)NotaRange.max);
								msgcond.Append(tmp);	  
							}  
						}
					}
					return TRUE;
				}
					   }
				break;
			case NOTEINMULTIVOICEUPPER: 
				if(pBattuta!=NULL)
				{
					if (pBattuta->GetMultiVoice() && layer%2==0 ) {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								wxString tmp;
								tmp.sprintf("State verified: NOTEINMULTIVOICELOWER \n");
								msgcond.Append(tmp);	  
							}
						}
						return TRUE;
					}
				}		 
				break;
			case NOTEINMULTIVOICELOWER: 
				if(pBattuta!=NULL)
				{      
					if (pBattuta->GetMultiVoice() && layer%2==1) {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								wxString tmp;
								tmp.sprintf("State verified: NOTEINMULTIVOICELOWER \n");
								msgcond.Append(tmp);	  
							}
						}            
						return TRUE;
					}
				}	 
				break;    
			case INMULTISTAFF: 
				if(pBattuta!=NULL)
				{
					if (pBattuta->GetNumberOfStaff()>1) 
					{ 
						
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: INMULTISTAFF \n");
							msgcond.Append(tmp);	  
						}
						return TRUE;
					}
				}
				break;
				
				
			case BEAM12: 
				if(pBattuta!=NULL)
				{
					if(pNota!=NULL)
						if (pNota->GetID()==CL_GRUPPONOTE) 
							
						{
							if (GruppoNote::BEAM_12==((GruppoNote*)pNota)->GetBeamPosition()) 
							{ 
								
								if (traceconditionsmilla==1)
								{
									wxString tmp;
									tmp.sprintf("State verified: BEAM12 \n");
									msgcond.Append(tmp);	  
								}
								return TRUE;
							}
						}
					if(drw!=NULL)	//FFF
						if (drw->GetID()==CL_RAGGRUPPAMENTO) 					
						{
							if (Raggruppamento::BEAM_12==((Raggruppamento*)drw)->GetBeamPosition()) 
							{ 
								if (traceconditionsmilla==1)
								{
									wxString tmp;
									tmp.sprintf("State verified: BEAM12 \n");
									msgcond.Append(tmp);	  
								}
								return TRUE;
							}
						}

				}
				break;
				////////////////////////////////////////////////////////////
				
			case BEAM23: 
				if(pBattuta!=NULL)
				{
					if(pNota!=NULL)
						if (pNota->GetID()==CL_GRUPPONOTE) 
							
						{
							if (GruppoNote::BEAM_23==((GruppoNote*)pNota)->GetBeamPosition()) 
							{ 
								
								if (traceconditionsmilla==1)
								{
									wxString tmp;
									tmp.sprintf("State verified: BEAM23 \n");
									msgcond.Append(tmp);	  
								}
								return TRUE;
							}
						}
					if(drw!=NULL)	//FFF
						if (drw->GetID()==CL_RAGGRUPPAMENTO) 					
						{
							if (Raggruppamento::BEAM_23==((Raggruppamento*)drw)->GetBeamPosition()) 
							{ 
								if (traceconditionsmilla==1)
								{
									wxString tmp;
									tmp.sprintf("State verified: BEAM23 \n");
									msgcond.Append(tmp);	  
								}
								return TRUE;
							}
						}
				}
				break;
				////////////////////////////////////////////////////////////
				
				
				
				
			case NOTESTEMUP:
				if (gamboupdwn) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTESTEMUP \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break;
			case NOTESTEMDOWN:
				if (!gamboupdwn) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTESTEMDOWN \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break;
			case NOTEONSPACE:
				if (pNota->GetAltezza()%2!=0) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTEONSPACE \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;   
				}
				break;
			case NOTEONLINE:
				if (pNota->GetAltezza()%2==0) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTEONLINE \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;   
				}
				break;
			case NOTEINPUNTOVALORE:
				if (pNota->GetPPuntoValore()!=NULL) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTEINPUNTOVALORE \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break;
			case NOTESUSPENDED:
				
				break;
			case NOTESINSINGLELAYER:
				if(pBattuta!=NULL)
				{
					if (!pBattuta->GetMultiVoice()) {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								wxString tmp;
								tmp.sprintf("State verified: NOTESINSINGLELAYER \n");
								msgcond.Append(tmp);	  
							}
						}
						return TRUE;
					}
				}	 
				break;
			case NOTESINMULTIVOICEUPPER:
				if(pBattuta!=NULL)
				{   
					if (pBattuta->GetMultiVoice() && layer%2==0) {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								wxString tmp;
								tmp.sprintf("State verified: NOTESINMULTIVOICEUPPER \n");
								msgcond.Append(tmp);	  
							}
						}
						return TRUE;
					}
				}	 
				break;
			case NOTESINMULTIVOICELOWER:
				if(pBattuta!=NULL)
				{      
					if (pBattuta->GetMultiVoice() && layer%2==1) {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								wxString tmp;
								tmp.sprintf("State verified: NOTESINMULTIVOICELOWER \n");
								msgcond.Append(tmp);	  
							}
						}
						return TRUE;
					}
				}	 
				break;
			case NOTESSTEMUP: {
				Nota *pn1=NULL;
				if(drw->GetID()==CL_GRUPPONOTE)	//FFF
					pn1 = ((GruppoNote*)pNota)->GetFirstNota();
				if(drw->GetID()==CL_RAGGRUPPAMENTO)
					pn1 = (Nota*)(((Raggruppamento*)drw)->GetFirstFigura());
				if (pn1->GetLGambo()>0)  {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTESSTEMUP \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
							  }
				break;
			case NOTESSTEMDOWN: {
				Nota *pn1=NULL;
				if(drw->GetID()==CL_GRUPPONOTE)	//FFF
					pn1 = ((GruppoNote*)pNota)->GetFirstNota();
				if(drw->GetID()==CL_RAGGRUPPAMENTO)
					pn1 = (Nota*)(((Raggruppamento*)drw)->GetFirstFigura());

				if (pn1->GetLGambo()<0)  {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTESSTEMDOWN \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
								}
				break;
			case NOTEBISCROMA:
				if (pNota->GetID()==CL_ACCORDO)
				{  
					Nota *pn=NULL;
					pn = ((Accordo*)pNota)->GetFirstNote();
					if (pn->GetID()==CL_NBISCROMA) {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								wxString tmp;
								tmp.sprintf("State verified: NOTEBISCROMA \n");
								msgcond.Append(tmp);	  
							}
						}
						return TRUE;
					}
				}
				if (pNota->GetID()==CL_NBISCROMA)  {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							
							wxString tmp;
							tmp.sprintf("State verified: NOTEBISCROMA \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break;
			case NOTESEMIBISCROMA:
				if (pNota->GetID()==CL_ACCORDO)
				{  
					Nota *pn;
					pn = ((Accordo*)pNota)->GetFirstNote();
					if (pn->GetID()==CL_NSEMIBISCROMA) {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								wxString tmp;
								tmp.sprintf("State verified: NOTESEMIBISCROMA \n");
								msgcond.Append(tmp);	  
							}
						}
						return TRUE;
					}
				}
				if (pNota->GetID()==CL_NSEMIBISCROMA) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTESEMIBISCROMA \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break;
			case NOTEFUSA:   
				if (pNota->GetID()==CL_ACCORDO)
				{  
					Nota *pn;
					pn = ((Accordo*)pNota)->GetFirstNote();
					if (pn->GetID()==CL_NFUSA) {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								wxString tmp;
								tmp.sprintf("State verified: NOTEFUSA \n");
								msgcond.Append(tmp);	  
							}
						}
						return TRUE;
					}
				}
				if (pNota->GetID()==CL_NFUSA)  {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTEFUSA \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break; 
			case NOTECROMA:
				if (pNota->GetID()==CL_ACCORDO)
				{  
					Nota *pn;
					pn = ((Accordo*)pNota)->GetFirstNote();
					if (pn->GetID()==CL_NCROMA) {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								wxString tmp;
								tmp.sprintf("State verified: NOTECROMA \n");
								msgcond.Append(tmp);	  
							}
						}
						return TRUE;
					}
					
				}
				if (pNota->GetID()==CL_NCROMA) { 
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTECROMA \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break;
				
			case NOTESEMIMINIMA:
				if (pNota->GetID()==CL_ACCORDO)
				{  
					Nota *pn;
					pn = ((Accordo*)pNota)->GetFirstNote();
					if (pn->GetID()==CL_NSEMIMINIMA) {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								wxString tmp;
								tmp.sprintf("State verified: NOTESEMIMINIMA \n");
								msgcond.Append(tmp);	  
							}
						}
						return TRUE;
					}
				}
				if (pNota->GetID()==CL_NSEMIMINIMA) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTESEMIMINIMA \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break;
				
			case NOTEMINIMA:
				if (pNota->GetID()==CL_ACCORDO)
				{  
					Nota *pn;
					pn = ((Accordo*)pNota)->GetFirstNote();
					if (pn->GetID()==CL_NMINIMA) {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								wxString tmp;
								tmp.sprintf("State verified: NOTEMINIMA \n");
								msgcond.Append(tmp);	  
							}
						}
						return TRUE;
					}
				}
				if (pNota->GetID()==CL_NMINIMA) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTEMINIMA \n");
							msgcond.Append(tmp);	  
						}
						
					}
					return TRUE;
				}
				break;
			case NOTESEMIBREVE:
				if (pNota->GetID()==CL_ACCORDO)
				{  
					Nota *pn;
					pn = ((Accordo*)pNota)->GetFirstNote();
					if (pn->GetID()==CL_NSEMIBREVE) {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								wxString tmp;
								tmp.sprintf("State verified: NOTESEMIBREVE \n");
								msgcond.Append(tmp);	  
							}
						}
						return TRUE;
					}
				}
				if (pNota->GetID()==CL_NSEMIBREVE) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTESEMIBREVE \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break;
			case NOTEBREVE:
				if (pNota->GetID()==CL_ACCORDO)
				{  
					Nota *pn;
					pn = ((Accordo*)pNota)->GetFirstNote();
					if (pn->GetID()==CL_NBREVE)  {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								wxString tmp;
								tmp.sprintf("State verified: NOTEBREVE \n");
								msgcond.Append(tmp);	  
							}
						}
						return TRUE;
					}
				}
				if (pNota->GetID()==CL_NBREVE) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTEBREVE \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break;
				
			case NOTENOTINA:
				if (pNota->GetNotina())  {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTENOTINA \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break;
				
			case NOTESEMICROMA: 
				if (pNota->GetID()==CL_ACCORDO)
				{  
					Nota *pn;
					pn = ((Accordo*)pNota)->GetFirstNote();
					if (pn->GetID()==CL_NSEMICROMA)  {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								wxString tmp;
								tmp.sprintf("State verified: NOTESEMICROMA \n");
								msgcond.Append(tmp);	  
							}
							
						}
						return TRUE;
					}
				}
				if (pNota->GetID()==CL_NSEMICROMA)  {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTESEMICROMA \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break; 
			case NOTEFIRSTUPPER:
				Nota *pfn,*pln;
				if(drw->GetID()==CL_GRUPPONOTE){	//FFF
					pfn=((GruppoNote*)pNota)->GetFirstNota(); 
					pln=((GruppoNote*)pNota)->GetLastNota();
				}
				else if(drw->GetID()==CL_RAGGRUPPAMENTO){
					pfn=(Nota*)((Raggruppamento*)drw)->GetFirstFigura();
					pln=(Nota*)((Raggruppamento*)drw)->GetLastFigura();
				}
				
				if ((pfn->GetAltezza())>(pln->GetAltezza())) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTEFIRSTUPPER \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break; 
			case NOTEFIRSTLOWER: 
				if(drw->GetID()==CL_GRUPPONOTE){	//FFF
					pfn=((GruppoNote*)pNota)->GetFirstNota(); 
					pln=((GruppoNote*)pNota)->GetLastNota();
				}
				else if(drw->GetID()==CL_RAGGRUPPAMENTO){
					pfn=(Nota*)((Raggruppamento*)drw)->GetFirstFigura();
					pln=(Nota*)((Raggruppamento*)drw)->GetLastFigura();
				}
				
				if ((pln->GetAltezza())>(pfn->GetAltezza())) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: NOTEFIRSTLOWER \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break; 
			case STACPRECEEDTEN:
				int stac, ten;
				stac=priorita.GetPosizione(CL_STACCATO);
				ten=priorita.GetPosizione(CL_TENUTO);
				if (stac<ten) {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: STACPRECEEDTEN \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break;
			case TENPRECEEDSTAC:
				stac=priorita.GetPosizione(CL_STACCATO);
				ten=priorita.GetPosizione(CL_TENUTO);
				if (ten<stac)  {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: TENPRECEEDSTAC \n");
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break;   
			case TIME:{
				Tempo tempo;
				float num, den;
				tempo=pBattuta->GetTempo();
				num=float(tempo.GetNumeratore());
				den=float(tempo.GetDenominatore());
				
				if (num==NotaRange.min && den==NotaRange.max)  {
					
					if (DebugNormalMilla || DebugNormalMillaPos)
					{ 
						if (traceconditionsmilla==1)
						{
							wxString tmp;
							tmp.sprintf("State verified: TIME=%d;%d\n",NotaRange.min,NotaRange.max);
							msgcond.Append(tmp);	  
						}
					}
					return TRUE;
				}
				break;
					  }
			case RELATION:{
				int f1=pcond->conditionrel.f1;
				int f2=pcond->conditionrel.f2;
				int Op=pcond->conditionrel.Op;
				int res=pcond->conditionrel.res;
				double num=pcond->conditionrel.num;
				
				wxString fact1="";
				wxString fact2="";
				wxString Oper="";
				wxString Conf="";
				wxString Numfin="";
				
				wxString tmp="";
				
				
				switch(f1) 
				{
				case UPPERD:
					fact1.sprintf("State verified: UPPERD");
					break;	
				case LOWERD:
					fact1.sprintf("State verified: LOWERD");
					break;	
				case NUMUP:
					fact1.sprintf("State verified: NUMUP");
					break; 
				case NUMDOWN:
					fact1.sprintf("State verified: NUMDOWN");
					break;
				case MEAN:
					fact1.sprintf("State verified: MEAN");
					break;
				case MHL:
					fact1.sprintf("State verified: MHL");
					break;
				case DELTA: 
					fact1.sprintf("State verified: DELTA");
					break;
				case BEAMDIS:
					fact1.sprintf("State verified: BEAMDIS");
					break;
				case STAFFNUMBER:
					fact1.sprintf("State verified: STAFFNUMBER");
					break;
				case ONSTAFFNUMBER:
					fact1.sprintf("State verified: ONSTAFFNUMBER");
					break;
				case STAFFLINES:
					fact1.sprintf("State verified: STAFFLINES");
					break;	
				}
				
				// operazione
				if(Op==PLUS) Oper.sprintf("+");
				if(Op==MIN) Oper.sprintf("-");
				
				if(Op!=99)  { 
					// secondo fattore
					switch(f2) 
					{
					case UPPERD:
						fact2.sprintf("UPPERD");
						break;	
					case LOWERD:
						fact2.sprintf("LOWERD");
						break;	
					case NUMUP:
						fact2.sprintf("NUMUP");
						break; 
					case NUMDOWN:
						fact2.sprintf("NUMDOWN");
						break;
					case MEAN:
						fact2.sprintf("MEAN");
						break;
					case MHL:
						fact2.sprintf("MHL");
						break;
					case DELTA: 
						fact2.sprintf("DELTA");
						break;
					case BEAMDIS:
						fact2.sprintf("BEAMDIS");
						break;
					case STAFFNUMBER:
						fact2.sprintf("STAFFNUMBER");
						break;
					case ONSTAFFNUMBER:
						fact2.sprintf("ONSTAFFNUMBER");
						break;
					case STAFFLINES:
						fact2.sprintf("STAFFLINES");
						break;	
					}
				}
				
				// operatore di confronto
				if(res==GREATER) 
					Conf.sprintf(">");        
				if(res==LOWer) 
					Conf.sprintf("<");        
				if(res==GREATERe) 
					Conf.sprintf(">=");        
				if(res==LOWERe) 
					Conf.sprintf("<=");        
				if(res==EQUAL) 
					Conf.sprintf("==");        
				
				// numero finale
				Numfin.sprintf("%f\n", pcond->conditionrel.num);
				
				// riassume l'espressione nella stringa tmp mettendo anche il numero di risultato
				tmp.Append(fact1);	  
				tmp.Append(Oper);	  
				tmp.Append(fact2);	  
				tmp.Append(Conf);	  
				tmp.Append(Numfin);	  
				
				////////////////
				float fat1=ValutaFactor(f1, pNota ,drw);
				float fat2=ValutaFactor(f2, pNota ,drw);
				
				float somma; 
				if(Op==PLUS) somma=fat1+fat2;
				if(Op==MIN) somma=fat1-fat2;
				if(Op==99) somma=fat1;
				
				if(res==GREATER) 
				{
					if(somma>num)  {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								msgcond.Append(tmp);	  
							}
						}
						return TRUE;
					}
				}
				if(res==LOWer) 
				{
					if(somma<num)  {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								msgcond.Append(tmp);	  
							}
						}
						return TRUE; 
					}
				}
				if(res==GREATERe) 
				{
					if(somma>=num) {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								msgcond.Append(tmp);	  
							}
						}
						return TRUE;
					}
				}
				if(res==LOWERe) 
				{
					if(somma<=num)  {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								msgcond.Append(tmp);	  
							}
						}
						return TRUE;
					}
				}
				if(res==EQUAL) 
				{
					if(somma==num)  {
						
						if (DebugNormalMilla || DebugNormalMillaPos)
						{ 
							if (traceconditionsmilla==1)
							{
								msgcond.Append(tmp);	  
							}
						}
						return TRUE;
					}
				}
    }  
    break; 
    case HEADCLASSIC: {
		if(pNota->GetHeadType()==HEAD_CLASSIC) {
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADCLASSIC \n");
					msgcond.Append(tmp);	  
				}
				
			}
			return(TRUE);
		}
                      }
		break;
    case HEADALPHANUM: {
		if(pNota->GetHeadType()==HEAD_ALPHANUM)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADALPHANUM \n");
					msgcond.Append(tmp);	  
				}
			}
			return(TRUE);
		}
                       }
		break;
    case HEADALPHANUM_SQUARE: {
		if(pNota->GetHeadType()==HEAD_ALPHANUM_SQUARE)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADALPHANUM_SQUARE \n");
					msgcond.Append(tmp);	  
				}
			}
			return(TRUE);
		}
                              }
		break;
    case HEADALPHANUM_REVERSE: {
		if(pNota->GetHeadType()==HEAD_ALPHANUM_REVERSE)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADALPHANUM_REVERSE \n");
					msgcond.Append(tmp);	  
				}
			}
			return(TRUE);
		}
                               }
		break;
    case HEADCIRCLEX: {
		if(pNota->GetHeadType()==HEAD_CIRCLEX)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADCIRCLEX \n");
					msgcond.Append(tmp);
				}
			}
			return(TRUE);
		}
                      }
		break;
    case HEADCLUSTER: {
		if(pNota->GetHeadType()==HEAD_CLUSTER)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADCLUSTER \n");
					msgcond.Append(tmp);
				}
			}
			return(TRUE);
		}
                      }
		break;
    case HEADX: {
		if(pNota->GetHeadType()==HEAD_X)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADX \n");
					msgcond.Append(tmp);
					
				}
			}
			
			
			
			return(TRUE);
		}
                }
		break;
    case HEADCROIX: {
		if(pNota->GetHeadType()==HEAD_CROIX)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADCROIX \n");
					msgcond.Append(tmp);
					
				}
			}
			
			
			
			return(TRUE);
		}
                    }
		break;
    case HEADDDIESIS: {
		if(pNota->GetHeadType()==HEAD_DDIESIS)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADDDIESIS \n");
					msgcond.Append(tmp);
					
				}
			}
			
			
			
			return(TRUE);
		}
                      }
		break;
    case HEADDIAMOND: {
		if(pNota->GetHeadType()==HEAD_DIAMOND)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADDIAMOND \n");
					msgcond.Append(tmp);
					
				}
			}
			
			
			
			return(TRUE);
		}
                      }
		break;
    case HEADTRIANG: {
		if(pNota->GetHeadType()==HEAD_TRIANG)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADTRIANG \n");
					msgcond.Append(tmp);
					
				}
			}
			
			
			
			return(TRUE);
		}
                     }
		break;
    case HEADTRIANG_UP: {
		if(pNota->GetHeadType()==HEAD_TRIANG_UP)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADTRIANG_UP \n");
					msgcond.Append(tmp);
					
				}
			}
			
			
			
			return(TRUE);
		}
                        }
		break;
    case HEADTRIANG_DOWN: {
		if(pNota->GetHeadType()==HEAD_TRIANG_DOWN)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADTRIANG_DOWN \n");
					msgcond.Append(tmp);
					
				}
			}
			
			
			
			return(TRUE);
		}
                          }
		break;
    case HEADTRIANG_LEFT: {
		if(pNota->GetHeadType()==HEAD_TRIANG_LEFT)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADTRIANG_LEFT \n");
					msgcond.Append(tmp);
					
				}
			}
			
			
			
			return(TRUE);
		}
                          }
		break;
    case HEADTRIANG_RIGHT: {
		if(pNota->GetHeadType()==HEAD_TRIANG_RIGHT)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADTRIANG_RIGHT \n");
					msgcond.Append(tmp);
					
				}
			}
			
			
			
			return(TRUE);
		}
                           }
		break;
    case HEADTRIANG_ROUND: {
		if(pNota->GetHeadType()==HEAD_TRIANG_ROUND)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADTRIANG_ROUND \n");
					msgcond.Append(tmp);
					
				}
			}
			
			
			
			return(TRUE);
		}
                           }
		break;
    case HEADMOON: {
		if(pNota->GetHeadType()==HEAD_MOON)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADMOON \n");
					msgcond.Append(tmp);
					
				}
			}
			
			
			
			return(TRUE);
		}
                   }
		break;
    case HEADPLUS: {
		if(pNota->GetHeadType()==HEAD_PLUS)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADPLUS \n");
					msgcond.Append(tmp);
					
				}
			}
			
			
			
			return(TRUE);
		}
                   }
		break;
    case HEADRHYTHMIC: {
		if(pNota->GetHeadType()==HEAD_RHYTHMIC)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADRHYTHMIC \n");
					msgcond.Append(tmp);
					
				}
			}
			
			
			
			return(TRUE);
		}
                       }
		break;
    case HEADSQUARE: {
		if(pNota->GetHeadType()==HEAD_SQUARE)
		{
			
			if (DebugNormalMilla || DebugNormalMillaPos)
			{ 
				if (traceconditionsmilla==1)
				{
					wxString tmp;
					tmp.sprintf("State verified: HEADSQUARE \n");
					msgcond.Append(tmp);
					
				}
			}
			
			
			
			return(TRUE);
		}
                     }
		break;
  }
  return FALSE;
}





//////////////////////////////////////////////////////////////////////////

void RuleManager::ValutaCondizioniDebug( Nota* pNota, Battuta* pBattuta, 
										DrawObject* drw,short layer, Bool gamboupdown)
{
	Bool above;
	Bool gamboupdwn = gamboupdown;  
	wxString tmp;
	
	
	///////////////////////////////////////////
	//Dunque: i parametri passati spono
	// Nota* pNota, Battuta* pBattuta, 
	// DrawObject* drw,short layer, Bool gamboupdwn)
	// guardare se non sono nulli e fare i confronti del caso
	
	
	if(pBattuta!=NULL)
	{
		if (!pBattuta->GetMultiVoice()) {
			tmp.sprintf("YES     NOTEINSINGLELAYER \n");
			
		}
		else 
		{
			tmp.sprintf("NO      NOTEINSINGLELAYER \n");
			
		}
		msgtoken.Append(tmp);	  
	}	 
	//////////////////////////////////////////////////////////
	if(pBattuta!=NULL)
	{      
		if (pBattuta->GetMultiVoice()) {
			tmp.sprintf("YES     NOTEINMULTIVOICE \n");
		}       
		else
		{
			tmp.sprintf("NO      NOTEINMULTIVOICE \n");
			
		}   
		msgtoken.Append(tmp);	 
	}	 
	
	///////////////////////////////////////////////////////////  
	if(pBattuta!=NULL)
	{
		if (pBattuta->GetNumberOfStaff()>1) 
		{ 
			tmp.sprintf("YES	    INMULTISTAFF \n");
		}
		else
		{ 
			tmp.sprintf("NO       INMULTISTAFF \n");
		}
		msgtoken.Append(tmp);	  
	}
	
	///////////////////////////////////////////////////////////  
	if (pNota!=NULL) 
	{
		if (pNota->GetID()==CL_ACCORDO) 
		{
			tmp.sprintf("YES     INCHORD \n");
		}
		else
		{
			tmp.sprintf("NO      INCHORD \n");
			
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	if (drw==NULL) return;
	switch(drw->GetID()) 
	{
    case CL_INDICAZIONE:
    case CL_ESPRESSIONE:
	case CL_STACCATO:	
    case CL_TENUTO:
    case CL_MARTELLATO:
    case CL_ACCENTO:
    case CL_SFORZATO:
    case CL_MARTDOLCE:
    case CL_PUNTOALLUNGATO:
    case CL_ESPGENERIC:
		
		
		if ((Espressione*)drw!=NULL) 
		{
			above = ((Espressione*)drw)->GetAboveNota();
			if ((pNota->GetLGambo())>0!=above) {
				tmp.sprintf("YES     OPPOSITETOSTEM \n");
			}
			else
			{
				tmp.sprintf("NO      OPPOSITETOSTEM \n");
				
			} 	
			msgtoken.Append(tmp);  
		}
		break;
		// volendo lo potrei anche cancellare
	case DEFAULT:
		tmp.sprintf("NA      OPPOSITETOSTEM \n");
		msgtoken.Append(tmp);  
		
		
	}
	///////////////////////////////////////////////////////////  
	switch(drw->GetID()) 
	{
    case CL_INDICAZIONE:
    case CL_ESPRESSIONE:
	case CL_STACCATO:	
    case CL_TENUTO:
    case CL_MARTELLATO:
    case CL_ACCENTO:
    case CL_SFORZATO:
    case CL_MARTDOLCE:
    case CL_PUNTOALLUNGATO:
    case CL_ESPGENERIC:
		
		
		
		if ((Espressione*)drw!=NULL) 
		{
			above = ((Espressione*)drw)->GetAboveNota();
			if (pNota!=NULL) 
			{
				if ((pNota->GetLGambo()>0)==above) {
					tmp.sprintf("YES     ONSTEM \n");
				}
				else
				{
					tmp.sprintf("NO      ONSTEM \n");
					
				}   
				msgtoken.Append(tmp);
			}
		}
		break;
		
		// volendo lo potrei anche cancellare
	case DEFAULT:
		
		tmp.sprintf("NA      ONSTEM \n");
		msgtoken.Append(tmp);  
		
		
	}
	
	///////////////////////////////////////////////////////////  
	EspressComposta* pespr;
	Espressione *pstac=NULL;
	if (pNota!=NULL && pNota->GetID()!=CL_GRUPPONOTE) 
	{
		if ((pespr = pNota->GetPEspressComposta())!=NULL)
		{
			pstac = pespr->GetPEspressione(CMD_NOTA_STACCATO);
		}
		if (pstac!=NULL) {
			tmp.sprintf("YES     NOTAWITHSTAC \n");
			
		}
		else
		{
			tmp.sprintf("NO      NOTAWITHSTAC \n");
			
		}   
		msgtoken.Append(tmp);
	} 
	///////////////////////////////////////////////////////////  
	
	
	Espressione *pacce=NULL;
	if (pNota!=NULL && pNota->GetID()!=CL_GRUPPONOTE) 
	{
		
		if ((pespr = pNota->GetPEspressComposta())!=NULL)
		{
			pacce = pespr->GetPEspressione(CMD_NOTA_ACCENTO);
		}
		if (pacce!=NULL) {
			tmp.sprintf("YES     NOTAWITHACCE \n");
		}
		else
		{
			tmp.sprintf("NO      NOTAWITHACCE \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	
	Espressione *pten=NULL;
	if (pNota!=NULL && pNota->GetID()!=CL_GRUPPONOTE) 
	{
		
		if ((pespr = pNota->GetPEspressComposta())!=NULL)
		{
			pten = pespr->GetPEspressione(CMD_NOTA_TENUTO);
		}
		if (pten!=NULL) {
			tmp.sprintf("YES     NOTAWITHTEN \n");
		}
		else
		{
			tmp.sprintf("NO      NOTAWITHTEN \n");
		}   
		msgtoken.Append(tmp);	 
	}  
	///////////////////////////////////////////////////////////  
	
	
	int alt;
	if (pNota!=NULL) 
	{
		
//FFF		if (pNota->GetID()!=CL_ACCORDO && pNota->GetID()!=CL_GRUPPONOTE) 
		if (pNota!=NULL && pNota->GetID()!=CL_GRUPPONOTE &&  drw->GetID()!=CL_RAGGRUPPAMENTO)
		{
			alt = pNota->GetAltezza()-4;
			
			tmp.sprintf("      NOTE [%d]\n",alt);
			msgtoken.Append(tmp);	  
		}
	}
	///////////////////////////////////////////////////////////  
	
	if(pBattuta!=NULL)
	{
		if (pBattuta->GetMultiVoice() && layer==0) 
		{
			tmp.sprintf("YES     NOTEINMULTIVOICEUPPER \n");
		}
		else
		{
			tmp.sprintf("NO      NOTEINMULTIVOICEUPPER \n");
		}   
		msgtoken.Append(tmp);	 
	}
	
	///////////////////////////////////////////////////////////  
	
	if(pBattuta!=NULL)
	{      
		if (pBattuta->GetMultiVoice() && layer==1)
		{
			tmp.sprintf("YES     NOTEINMULTIVOICELOWER \n");
		}
		else
		{
			tmp.sprintf("NO      NOTEINMULTIVOICELOWER \n");
		}   
		msgtoken.Append(tmp);	   
	}	 
	
	///////////////////////////////////////////////////////////  
	
	if (gamboupdwn) {
		tmp.sprintf("YES     NOTESTEMUP \n");
	}
	else
	{
		tmp.sprintf("NO      NOTESTEMUP \n");
	}   
	msgtoken.Append(tmp);	 
	///////////////////////////////////////////////////////////  
	
	
	if (!gamboupdwn) {
		tmp.sprintf("YES     NOTESTEMDOWN \n");
	}
	else
	{
		tmp.sprintf("NO      NOTESTEMDOWN \n");
	}   
	msgtoken.Append(tmp);	 
	
	///////////////////////////////////////////////////////////  
	
	
	
	if (pNota!=NULL && pNota->GetID()!=CL_GRUPPONOTE) 
	{
		
		if (pNota->GetAltezza()%2!=0) {
			tmp.sprintf("YES     NOTEONSPACE \n");
		}
		else
		{
			tmp.sprintf("NO      NOTEONSPACE \n");
		}   
		msgtoken.Append(tmp);	 
	}  
	///////////////////////////////////////////////////////////  
	
	
	
	if (pNota!=NULL && pNota->GetID()!=CL_GRUPPONOTE) 
	{
		
		if (pNota->GetAltezza()%2==0) {
			tmp.sprintf("YES     NOTEONLINE \n");
		}
		else
		{
			tmp.sprintf("NO      NOTEONLINE \n");
		}   
		msgtoken.Append(tmp);	 
	}  
	///////////////////////////////////////////////////////////  
	
	if (pNota!=NULL && pNota->GetID()!=CL_GRUPPONOTE) 
	{
		
		if (pNota->GetPPuntoValore()!=NULL) {
			tmp.sprintf("YES     NOTEINPUNTOVALORE \n");
		}
		else
		{
			tmp.sprintf("NO      NOTEINPUNTOVALORE \n");
		}   
		msgtoken.Append(tmp);	 
	}  
	///////////////////////////////////////////////////////////  
	
	
	
	if(pBattuta!=NULL)
	{
		if (!pBattuta->GetMultiVoice()) {
			tmp.sprintf("YES     NOTESINSINGLELAYER \n");
		}
		else
		{
			tmp.sprintf("NO      NOTESINSINGLELAYER \n");
		}   
		msgtoken.Append(tmp);	 
	}	 
	
	///////////////////////////////////////////////////////////  
	
	
	if(pBattuta!=NULL)
	{      
		if (pBattuta->GetMultiVoice() && layer==0) {
			tmp.sprintf("YES     NOTESINMULTIVOICEUPPER \n");
		}
		else
		{
			tmp.sprintf("NO      NOTESINMULTIVOICEUPPER \n");
		}   
		msgtoken.Append(tmp);	 
		
	}
	///////////////////////////////////////////////////////////  
	
	if(pBattuta!=NULL)
	{      
		if (pBattuta->GetMultiVoice() && layer==1) {
			tmp.sprintf("YES     NOTESINMULTIVOICELOWER \n");
		}
		else
		{
			tmp.sprintf("NO      NOTESINMULTIVOICELOWER \n");
		}   
		msgtoken.Append(tmp);	 
	}	 
	
	///////////////////////////////////////////////////////////  
	
	
	Nota *pn1;
	if (pNota!=NULL) 
	{
		if (pNota->GetID()==CL_GRUPPONOTE)
		{
			
			pn1 = ((GruppoNote*)pNota)->GetFirstNota();
			if (pn1!=NULL) 
			{
				
				if (pn1->GetLGambo()>0)  {
					tmp.sprintf("YES     NOTESSTEMUP \n");
				}
				else
				{
					tmp.sprintf("NO      NOTESSTEMUP \n");
				}   
				msgtoken.Append(tmp);	 
			}  
		}
	}
	///////////////////////////////////////////////////////////  
	
	
	
	Nota *pn;
	if (pNota!=NULL) 
	{
		if (pNota->GetID()==CL_GRUPPONOTE)
		{
			pn = ((GruppoNote*)pNota)->GetFirstNota();
			if (pn!=NULL) 
			{
				
				if (pn->GetLGambo()<0)  {
					tmp.sprintf("YES     NOTESSTEMDOWN \n");
				}
				else
				{
					tmp.sprintf("NO      NOTESSTEMDOWN \n");
				}   
				msgtoken.Append(tmp);	 
			}
		}
	}
	///////////////////////////////////////////////////////////  
	
	
	
	if (pNota!=NULL) 
	{
		
		if (pNota->GetID()==CL_ACCORDO)
		{  
			Nota *pn;
			pn = ((Accordo*)pNota)->GetFirstNote();
			if (pn!=NULL) 
			{
				
				if (pn->GetID()==CL_NBISCROMA) {
					tmp.sprintf("YES     NOTEBISCROMA \n");
				}
				else
				{
					tmp.sprintf("NO      NOTEBISCROMA \n");
				}   
				msgtoken.Append(tmp);	 
			}
		}
		
		else {
			if (pNota->GetID()==CL_NBISCROMA)  {
				tmp.sprintf("YES     NOTEBISCROMA \n");
			}
			else
			{
				tmp.sprintf("NO      NOTEBISCROMA \n");
			}   
			msgtoken.Append(tmp);	 
		}
	}
	///////////////////////////////////////////////////////////  
	
	
	if (pNota!=NULL) 
	{
		
		if (pNota->GetID()==CL_ACCORDO)
		{  
			Nota *pn;
			pn = ((Accordo*)pNota)->GetFirstNote();
			if (pn!=NULL) 
			{
				
				if (pn->GetID()==CL_NSEMIBISCROMA) {
					tmp.sprintf("YES     NOTESEMIBISCROMA \n");
				}
				else
				{
					tmp.sprintf("NO      NOTESEMIBISCROMA \n");
				}   
				msgtoken.Append(tmp);	 
			}
		}
		else {
			if (pNota->GetID()==CL_NSEMIBISCROMA) {
				tmp.sprintf("YES     NOTESEMIBISCROMA \n");
			}
			else
			{
				tmp.sprintf("NO      NOTESEMIBISCROMA \n");
			}   
			msgtoken.Append(tmp);	 
		}
	}
	///////////////////////////////////////////////////////////  
	if (pNota!=NULL) 
	{
		
		if (pNota->GetID()==CL_ACCORDO)
		{  
			Nota *pn;
			pn = ((Accordo*)pNota)->GetFirstNote();
			if (pn!=NULL) 
			{
				
				if (pn->GetID()==CL_NFUSA) {
					tmp.sprintf("YES     NOTEFUSA \n");
				}
				else
				{
					tmp.sprintf("NO      NOTEFUSA \n");
				}   
				msgtoken.Append(tmp);	 
			}
		}
		else {
			if (pNota->GetID()==CL_NFUSA)  {
				tmp.sprintf("YES     NOTEFUSA \n");
			}
			else
			{
				tmp.sprintf("NO      NOTEFUSA \n");
			}   
			msgtoken.Append(tmp);	 
		}
	}
	///////////////////////////////////////////////////////////  
	
	
	if (pNota!=NULL) 
	{
		
		if (pNota->GetID()==CL_ACCORDO)
		{  
			Nota *pn;
			pn = ((Accordo*)pNota)->GetFirstNote();
			if (pn!=NULL) 
			{
				
				if (pn->GetID()==CL_NCROMA) {
					tmp.sprintf("YES     NOTECROMA \n");
				}
				else
				{
					tmp.sprintf("NO      NOTECROMA \n");
				}   
				msgtoken.Append(tmp);	 
			}
		}
		else {
			if (pNota->GetID()==CL_NCROMA) { 
				tmp.sprintf("YES     NOTECROMA \n");
			}
			else
			{
				tmp.sprintf("NO      NOTECROMA \n");
			}   
			msgtoken.Append(tmp);	 
		}
	}
	///////////////////////////////////////////////////////////  
	
	if (pNota!=NULL) 
	{
		
		if (pNota->GetID()==CL_ACCORDO)
		{  
			Nota *pn;
			pn = ((Accordo*)pNota)->GetFirstNote();
			if (pn!=NULL) 
			{
				
				if (pn->GetID()==CL_NSEMIMINIMA) {
					tmp.sprintf("YES     NOTESEMIMINIMA \n");
				}
				else
				{
					tmp.sprintf("NO      NOTESEMIMINIMA \n");
				}   
				msgtoken.Append(tmp);	 
			}
		}
		else {
			if (pNota->GetID()==CL_NSEMIMINIMA) {
				tmp.sprintf("YES     NOTESEMIMINIMA \n");
			}
			else
			{
				tmp.sprintf("NO      NOTESEMIMINIMA \n");
			}   
			msgtoken.Append(tmp);	 
		} 
	}
	///////////////////////////////////////////////////////////  
	
	if (pNota!=NULL) 
	{
		
		if (pNota->GetID()==CL_ACCORDO)
		{  
			Nota *pn;
			pn = ((Accordo*)pNota)->GetFirstNote();
			if (pn!=NULL) 
			{
				
				if (pn->GetID()==CL_NMINIMA) {
					tmp.sprintf("YES     NOTEMINIMA \n");
				}
				else
				{
					tmp.sprintf("NO      NOTEMINIMA \n");
				}   
				msgtoken.Append(tmp);	 
			}
		}
		else {
			if (pNota->GetID()==CL_NMINIMA) 
				tmp.sprintf("YES     NOTEMINIMA \n");
			else
				tmp.sprintf("NO      NOTEMINIMA \n");
			msgtoken.Append(tmp);	 
		}
	}
	///////////////////////////////////////////////////////////  
	
	
	if (pNota!=NULL) 
	{
		
		if (pNota->GetID()==CL_ACCORDO)
		{  
			Nota *pn;
			pn = ((Accordo*)pNota)->GetFirstNote();
			if (pn!=NULL) 
			{
				
				if (pn->GetID()==CL_NSEMIBREVE) {
					tmp.sprintf("YES     NOTESEMIBREVE \n");
				}
				else
				{
					tmp.sprintf("NO      NOTESEMIBREVE \n");
				}   
				msgtoken.Append(tmp);	 
			}
			
		}
		else {
			if (pNota->GetID()==CL_NSEMIBREVE) {
				tmp.sprintf("YES     NOTESEMIBREVE \n");
			}
			else
			{
				tmp.sprintf("NO      NOTESEMIBREVE \n");
			}   
			msgtoken.Append(tmp);	 
			
		}
	}
	///////////////////////////////////////////////////////////  
	
	if (pNota!=NULL) 
	{
		
		if (pNota->GetID()==CL_ACCORDO)
		{  
			Nota *pn;
			pn = ((Accordo*)pNota)->GetFirstNote();
			if (pn!=NULL) 
			{
				
				if (pn->GetID()==CL_NBREVE)  {
					tmp.sprintf("YES     NOTEBREVE \n");
				}
				else
				{
					tmp.sprintf("NO      NOTEBREVE \n");
				}   
				msgtoken.Append(tmp);	 
			}
		}
		else {
			if (pNota->GetID()==CL_NBREVE) {
				tmp.sprintf("YES     NOTEBREVE \n");
			}
			else
			{
				tmp.sprintf("NO      NOTEBREVE \n");
			}   
			msgtoken.Append(tmp);	 
			
		}
	}
	///////////////////////////////////////////////////////////  
	
	if (pNota!=NULL) 
	{
		
		if (pNota->GetNotina())  {
			tmp.sprintf("YES     NOTENOTINA \n");
		}
		else
		{
			tmp.sprintf("NO      NOTENOTINA \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	
	if (pNota!=NULL) 
	{
		
		if (pNota->GetID()==CL_ACCORDO)
		{  
			Nota *pn;
			pn = ((Accordo*)pNota)->GetFirstNote();
			if (pn!=NULL) 
			{
				
				if (pn->GetID()==CL_NSEMICROMA)  {
					tmp.sprintf("YES     NOTESEMICROMA \n");
				}
				else
				{
					tmp.sprintf("NO      NOTESEMICROMA \n");
				}   
				msgtoken.Append(tmp);	 
				
			}
		}
		else {
			if (pNota->GetID()==CL_NSEMICROMA)  {
				tmp.sprintf("YES     NOTESEMICROMA \n");
			}
			else
			{
				tmp.sprintf("NO      NOTESEMICROMA \n");
			}   
			msgtoken.Append(tmp);	 
		}
	}
	///////////////////////////////////////////////////////////  
	
	
	
	Nota *pfn,*pln;
	if (pNota!=NULL && pNota->GetID()==CL_GRUPPONOTE)
	{
		if ((GruppoNote*)pNota!=NULL) 
			pfn=((GruppoNote*)pNota)->GetFirstNota(); 
		if ((GruppoNote*)pNota!=NULL) 
			pln=((GruppoNote*)pNota)->GetLastNota();
		
		if (pfn!=NULL) 
			if (pln!=NULL) 
				if ((pfn->GetAltezza())>(pln->GetAltezza())) {
					tmp.sprintf("YES     NOTEFIRSTUPPER \n");
				}
				else
				{
					tmp.sprintf("NO      NOTEFIRSTUPPER \n");
				}   
				msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	
	if (pNota!=NULL && pNota->GetID()==CL_GRUPPONOTE)
	{
		if ((GruppoNote*)pNota!=NULL) 
			pfn=((GruppoNote*)pNota)->GetFirstNota(); 
		if ((GruppoNote*)pNota!=NULL) 
			pln=((GruppoNote*)pNota)->GetLastNota();
		
		if (pfn!=NULL) 
			if (pln!=NULL) 
				if ((pln->GetAltezza())>(pfn->GetAltezza())) {
					tmp.sprintf("YES     NOTEFIRSTLOWER \n");
				}
				else
				{
					tmp.sprintf("NO      NOTEFIRSTLOWER \n");
				}   
				msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	
	
	int stac, ten;
	stac=priorita.GetPosizione(CL_STACCATO);
	ten=priorita.GetPosizione(CL_TENUTO);
	
	
	if (stac<ten) {
		tmp.sprintf("YES     STACPRECEEDTEN \n");
	}
	else
	{
		tmp.sprintf("NO      STACPRECEEDTEN \n");
	}   
	msgtoken.Append(tmp);	 
	///////////////////////////////////////////////////////////  
	
	
	stac=priorita.GetPosizione(CL_STACCATO);
	ten=priorita.GetPosizione(CL_TENUTO);
	if (ten<stac)  {
		tmp.sprintf("YES     TENPRECEEDSTAC \n");
	}
	else
	{
		tmp.sprintf("NO      TENPRECEEDSTAC \n");
	}   
	msgtoken.Append(tmp);	 
	///////////////////////////////////////////////////////////  
	
	Tempo tempo;
	float num, den;
	if (pBattuta!=NULL) 
	{
		tempo=pBattuta->GetTempo();
		num=float(tempo.GetNumeratore());
		den=float(tempo.GetDenominatore());
		
		tmp.sprintf("         TIME=%d;%d\n",(int)num,(int)den);
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	
	tmp="";
	int fac=0;
	
	//////////////////////
	
	if (pNota!=NULL) 
	{
		
		if(pNota->GetID()==CL_ACCORDO)
		{
			Nota *pn1, *pn2;
			
			if ((Accordo*)pNota!=NULL) 
			{
				
				
				pn2 = ((Accordo*)pNota)->GetLastNote();
				pn1 = ((Accordo*)pNota)->GetFirstNote();
			}
			if (pn2!=NULL) 
			{
				if (pn1!=NULL) 
				{
					
					
					// UPPERD
					fac=pn2->GetAltezza()-4;
					fac=fac>0?fac:-fac;
					if(fac==99) fac=0;
					tmp.sprintf("State verified: UPPERD=%d\n", fac);
					msgtoken.Append(tmp);	 
					// LOWERD
					fac=pn1->GetAltezza()-4;
					fac=fac>0?fac:-fac;
					if(fac==99) fac=0;
					tmp.sprintf("State verified: LOWERD=%d\n", fac);
					msgtoken.Append(tmp);
					// NUMUP
					fac=((Accordo*)pNota)->GetNumUp();
					if(fac==99) fac=0;
					tmp.sprintf("State verified: NUMUP=%d\n", fac);
					msgtoken.Append(tmp);
					// NUMDOWN  
					fac=((Accordo*)pNota)->GetNumDown();
					if(fac==99) fac=0;
					tmp.sprintf("State verified: NUMDOWN=%d\n", fac);
					msgtoken.Append(tmp);
				}
			}
			
		}
	}
	///////////////////////////////////////////////////////////  
	if (pNota!=NULL) 
	{
		//FFF
		float val;
		if(pNota->GetID()==CL_GRUPPONOTE)
		{
			// MEAN
			val=((GruppoNote*)pNota)->GetMean();
			tmp.sprintf("State verified: MEAN=%f\n", val);
			msgtoken.Append(tmp);
			// MHL:
			val=((GruppoNote*)pNota)->GetMeanHighLow();
			tmp.sprintf("State verified: MHL=%f\n", val);
			msgtoken.Append(tmp);
			// DELTA: 
			val=((GruppoNote*)pNota)->GetDelta();
			tmp.sprintf("State verified: DELTA=%f\n", val);
			msgtoken.Append(tmp);
			// BEAMDIS:
			val=((GruppoNote*)pNota)->GetBeamDis();
			tmp.sprintf("State verified: BEAMDIS=%f\n", val);
			msgtoken.Append(tmp);
		}
	} //FFF
	
	///////////////////////////////////////////////////////////  
	if(pBattuta!=NULL)
	{
		if(pNota!=NULL)
			if (pNota->GetID()==CL_GRUPPONOTE) 
				
			{
				if (GruppoNote::BEAM_12==((GruppoNote*)pNota)->GetBeamPosition()) 
					tmp.sprintf("YES	    BEAM12 \n");
				else
					tmp.sprintf("NO       BEAM12 \n");
				msgtoken.Append(tmp);
			}
			
	}
	
	///////////////////////////////////////////////////////////  
	///////////////////////////////////////////////////////////  
	if(pBattuta!=NULL)
	{
		if(pNota!=NULL)
			if (pNota->GetID()==CL_GRUPPONOTE) 
				
			{
				if (GruppoNote::BEAM_23==((GruppoNote*)pNota)->GetBeamPosition()) 
					
					
					tmp.sprintf("YES	    BEAM23 \n");
				
				else
					
					tmp.sprintf("NO       BEAM23 \n");
				
				msgtoken.Append(tmp);	  
			}
	}
	///////////////////////////////////////////////////////////  
	
	
	
	///////////////////////////////////////////////////////////  
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_CLASSIC) {
			tmp.sprintf("YES     HEADCLASSIC \n");
		}
		else
		{
			tmp.sprintf("NO      HEADCLASSIC \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_ALPHANUM)
		{
			tmp.sprintf("YES     HEADALPHANUM \n");
		}
		else
		{
			tmp.sprintf("NO      HEADALPHANUM \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_ALPHANUM_SQUARE)
		{
			tmp.sprintf("YES     HEADALPHANUM_SQUARE \n");
		}
		else
		{
			tmp.sprintf("NO      HEADALPHANUM_SQUARE \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_ALPHANUM_REVERSE)
		{
			tmp.sprintf("YES     HEADALPHANUM_REVERSE \n");
		}
		else
		{
			tmp.sprintf("NO      HEADALPHANUM_REVERSE \n");
		}   
		msgtoken.Append(tmp);	 
	} 
	///////////////////////////////////////////////////////////  
	
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_CIRCLEX)
		{
			tmp.sprintf("YES     HEADCIRCLEX \n");
			
		}
		else
		{
			tmp.sprintf("NO      HEADCIRCLEX \n");
		}   
		msgtoken.Append(tmp);	 
	} 
	///////////////////////////////////////////////////////////  
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_CLUSTER)
		{
			tmp.sprintf("YES     HEADCLUSTER \n");
			
		}
		else
		{
			tmp.sprintf("NO      HEADCLUSTER \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	
	
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_X)
		{
			tmp.sprintf("YES     HEADX \n");
			
		}
		else
		{
			tmp.sprintf("NO      HEADX \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_CROIX)
		{
			tmp.sprintf("YES     HEADCROIX \n");
			
		}
		else
		{
			tmp.sprintf("NO      HEADCROIX \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_DDIESIS)
		{
			tmp.sprintf("YES     HEADDDIESIS \n");
			
		}
		else
		{
			tmp.sprintf("NO      HEADDDIESIS \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_DIAMOND)
		{
			tmp.sprintf("YES     HEADDIAMOND \n");
			
		}
		else
		{
			tmp.sprintf("NO      HEADDIAMOND \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_TRIANG)
		{
			tmp.sprintf("YES     HEADTRIANG \n");
			
		}
		else
		{
			tmp.sprintf("NO      HEADTRIANG \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_TRIANG_UP)
		{
			tmp.sprintf("YES     HEADTRIANG_UP \n");
		}
		else
		{
			tmp.sprintf("NO      HEADTRIANG_UP \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_TRIANG_DOWN)
		{
			tmp.sprintf("YES     HEADTRIANG_DOWN \n");
			
		}
		else
		{
			tmp.sprintf("NO      HEADTRIANG_DOWN \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_TRIANG_LEFT)
		{
			tmp.sprintf("YES     HEADTRIANG_LEFT \n");
			
		}
		else
		{
			tmp.sprintf("NO      HEADTRIANG_LEFT \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_TRIANG_RIGHT)
		{
			tmp.sprintf("YES     HEADTRIANG_RIGHT \n");
			
		}
		else
		{
			tmp.sprintf("NO      HEADTRIANG_RIGHT \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_TRIANG_ROUND)
		{
			tmp.sprintf("YES     HEADTRIANG_ROUND \n");
			
		}
		else
		{
			tmp.sprintf("NO      HEADTRIANG_ROUND \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	
	
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_MOON)
		{
			tmp.sprintf("YES     HEADMOON \n");
			
		}
		else
		{
			tmp.sprintf("NO      HEADMOON \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_PLUS)
		{
			tmp.sprintf("YES     HEADPLUS \n");
			
		}
		else
		{
			tmp.sprintf("NO      HEADPLUS \n");
		}   
		msgtoken.Append(tmp);	 
	} 
	///////////////////////////////////////////////////////////  
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_RHYTHMIC)
		{
			tmp.sprintf("YES     HEADRHYTHMIC \n");
			
		}
		else
		{
			tmp.sprintf("NO      HEADRHYTHMIC \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	if (pNota!=NULL) 
	{
		
		if(pNota->GetHeadType()==HEAD_SQUARE)
		{
			tmp.sprintf("YES     HEADSQUARE \n");
			
		}
		else
		{
			tmp.sprintf("NO      HEADSQUARE \n");
		}   
		msgtoken.Append(tmp);	 
	}
	///////////////////////////////////////////////////////////  
	
	
	
	
	
	return;
}


