/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */




#include "mil/m.tab.h"
#include "lioo.hpp"
#include "mil/simboli.hpp"

#include "gui/lioowin.hpp"

#include "drw/musicedt.hpp"

extern GScreen *Scr;
extern LiooWindow *pMusicWin;

extern wxMusicEditor* MusicEditor; 

int Simboli::CurrentGroupUsage = 0;
int Simboli::CurrentSymbolUsage = 0;
GROUPS Simboli::Groups[MAXNUMGROUPS];
NEWSYMBOL Simboli::Symbol[MAXNUMSYM];

Simboli::Simboli()
{
	CurrentGroupUsage = 0;  
	CurrentSymbolUsage = 0;
	smrf=NULL;
}

void Simboli::Reset()
{
	CurrentGroupUsage = 0;  
	CurrentSymbolUsage = 0;
}

void Simboli::AddGroup(char *groupname, char *filename, int fontSize)
{   
	char FileName[100];

	Groups[CurrentGroupUsage].GroupName=(char *)malloc(strlen(groupname) + 1);
	strcpy(Groups[CurrentGroupUsage].GroupName, groupname);
	
	if (strcmp(filename, "") == 0){
		strcpy(FileName,groupname);
		strcat(FileName,".fon");
	}else
		strcpy(FileName,filename);

	Scr->lista_font[CurrentGroupUsage+6] = new LiooFont(FileName, smrf, fontSize);

	if (!Scr->lista_font[CurrentGroupUsage+6])
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","GScreen","GScreen");
    Scr->lista_font[CurrentGroupUsage+6]->SetID(CurrentGroupUsage+6);
	

	CurrentGroupUsage++;
}

void Simboli::AddSymbolName(char *symbolname)
{   
	Symbol[CurrentSymbolUsage].SymbolName=(char *)malloc(strlen(symbolname) + 1);
	strcpy(Symbol[CurrentSymbolUsage].SymbolName, symbolname);
}

void Simboli::AddGroupName(char *groupname)
{        
	Symbol[CurrentSymbolUsage].GroupName=(char *)malloc(strlen(groupname) + 1);
	strcpy(Symbol[CurrentSymbolUsage].GroupName, groupname);
}

void Simboli::AddCode(int code)
{        
	Symbol[CurrentSymbolUsage].Code=code;
	Symbol[CurrentSymbolUsage].fromXml=false;
}

void Simboli::AddDim(int dimup, int dimdown, int dimsx, int dimdx, int deltax, int deltay)
{       
	Symbol[CurrentSymbolUsage].fromXml=true;
	Symbol[CurrentSymbolUsage].DimUp=dimup;  
	Symbol[CurrentSymbolUsage].DimDown=dimdown; 
	Symbol[CurrentSymbolUsage].DimSx=dimsx; 
	Symbol[CurrentSymbolUsage].DimDx=dimdx;
	Symbol[CurrentSymbolUsage].DimDeltaX=deltax;
	Symbol[CurrentSymbolUsage].DimDeltaY=deltay;
}

void Simboli::SetDim(int id_font, int id_symb)
{
	if(!Symbol[id_symb].fromXml)
	{
		int cod_char=GetCode(id_symb);
		Symbol[id_symb].DimDeltaX=Scr->Get_VU2DX(cod_char,id_font);
		Symbol[id_symb].DimDeltaY=Scr->Get_VU2DY(cod_char,id_font);
		Symbol[id_symb].DimUp=Scr->Get_VU2UP(cod_char,id_font); 
		Symbol[id_symb].DimDown=Scr->Get_VU2DWN(cod_char,id_font);       
		Symbol[id_symb].DimSx=Scr->Get_VU2LF(cod_char,id_font);       
		Symbol[id_symb].DimDx=Scr->Get_VU2RG(cod_char,id_font);
	}
}

void Simboli::Incrementa()
{  
	CurrentSymbolUsage++;
}

int Simboli::GetSymbolUsage(void)
{
	return CurrentSymbolUsage;
}

int Simboli::GetGroupUsage(void)
{
	return CurrentGroupUsage;
}

int Simboli::GetGroupPos(char *groupname)
{
	if (!groupname)
		return -1;
		
	int i;
	for(i=0; i<MAXNUMGROUPS; i++)
	{
		// devo fare un confronto tra due stringhe
		if(Groups[i].GroupName!=NULL)
		{
			if(strcmp(Groups[i].GroupName,groupname)==0)
				return i;
		}	 
	}
	return -1;
}  

int Simboli::GetSymbolPos(char *symbolname)
{
	int i;
	for(i=0; i<MAXNUMSYM; i++)
	{
		// devo fare un confronto tra due stringhe
		if(Symbol[i].SymbolName!=NULL)
		{
			if(strcmp(Symbol[i].SymbolName,symbolname)==0)
				return i;
		}	
	}
	return -1;
}  


void Simboli::seeStruct()
{
	
	int i;
	for (i=0;i<CurrentSymbolUsage;i++)
    {  
		fprintf(stdout,"SymbolName %s, GroupName %s, Code %d, DimUp %d"
			" DimDown %d, DimSx %d, DimDx %d, Deltax %d, DeltaY %d \n",
			Symbol[i].SymbolName,
			Symbol[i].GroupName,
			Symbol[i].Code,
			Symbol[i].DimUp,
			Symbol[i].DimDown,
			Symbol[i].DimSx,
			Symbol[i].DimDx,
			Symbol[i].DimDeltaX,
			Symbol[i].DimDeltaY);		       
    }
	
}

char *Simboli::GetGroup(int IDsym)
{
	return Symbol[IDsym].GroupName;
}

VUnit Simboli::GetDim(int IDsym, int a)
{
	if(a==1) return Symbol[IDsym].DimUp; 
	if(a==2) return Symbol[IDsym].DimDown; 
	if(a==3) return Symbol[IDsym].DimSx; 
	if(a==4) return Symbol[IDsym].DimDx;
	if(a==5) return Symbol[IDsym].DimDeltaX; 
	if(a==6) return Symbol[IDsym].DimDeltaY;
	return 0; 
}

int Simboli::GetCode(int IDsym)
{
	return Symbol[IDsym].Code; 
}

char *Simboli::GetSymName(int ID)
{
	return Symbol[ID].SymbolName; 
}












