/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include <locale.h>

#include <wx/wx.h>

#include "smr/smrmain.hpp"
#include "smr/workspac.hpp"

#include "lioo.hpp"
#include "drw/gscreen.hpp"
#include "drw/scrlist.hpp"
#include "liooext.hpp"
#include "mil/posiz.hpp"
#include "mil/immissione.hpp"
#include "mil/priorita.hpp"
#include "mil/simboli.hpp"
#include "drw/musicedt.hpp"
#include "drw/scorefrm.hpp"
#include "smr/smr.hpp"
#include "gui/masaewin.hpp"
#include "gui/dlioowin.hpp"
#include "musiceditor.hpp"



class SMRFile;

static SMRFile *smrf=NULL;
static Partitura *mainScore=NULL;
static int nPart;

Mouse mouse;

extern void LoadDialogs();
extern Widget LiooWidget;
extern ScrList LiooListWidget;
extern ScrList LiooList;
extern wxMusicEditor *MusicEditor;

int VerifKey();

static bool initialized[]={false,false,false};

#ifndef WDFEDONLY

static void InitWDFMusicEditor(int where)
{
  assert(where>=0 && where<=2);
  if(initialized[where])
	  return;
  if(where==INITME_EXITAPP) // on startup
  {
	  smrf=NULL;

	  initialized[where]=true;
	  return;
  }

  if(where==INITME_STARTAPP) // on startup
  {
    setlocale(LC_ALL,"C");

    EM.SetLevel(EL_ERROR);
  }
   		
  if(where==INITME_BEFORE_LOAD || where==INITME_STARTAPP)
  {
    if (Scr==NULL)
    {
      Scr=new GScreen;
    }
    
    WinDliooWidth = 1000;
    WinDliooHeight = WIN_DLIOO_HEIGHT;
    WinMasaeWidth = 1000;
    WinMasaeHeight = WIN_DLIOO_HEIGHT;
  
    //inizializzazione dei colori
    black = NERO;
    // Produce il blu da impiegarsi in modo XOR.
    blue_xor = BLU; //BLUE^WHITE;
    dgray  = GRIGIOS;
    gray  = GRIGIO;
    red = ROSSO;
    white = BIANCO;

    // Inizializzazione identificatori font.
    Scr->SetFont(TEXTFONT_ID);
    nTextFont = Scr->SetFont(BIGTEXTFONT_ID);
    nBigTextFont = Scr->SetFont(MUSICFONT_ID);
    nMusicFont = Scr->SetFont(MUSICFONT2_ID);
    nMusicFont2 = Scr->SetFont(MUSICFONT3_ID);
    nMusicFont3 = Scr->SetFont(MENUFONT_ID);
	nMenuFont = Scr->SetFont(MUSICFONT_ID);

	Scr->SetMouse(&mouse); 
    int x,y,W,H;
    SMRApp::GetApp().workspace->GetWindowConfig("MusicEditor", x, y, W, H);
    DimensioneX=W-10;
    DimensioneY=H-10; 
    Leg=true;

    if (MusicEditor==NULL)
      MusicEditor= new wxMusicEditor;
  }
  initialized[where]=true;
}

void InitMusicEditor(int where)
{
	InitWDFMusicEditor(where);
}

void InitMusicEditors(int where)
{
	InitExtraMusicEditor(where); //for VIP
	InitWDFMusicEditor(where); //for image
}

int CloseMusicEditor()
{
	if(MusicEditor!=NULL)
		MusicEditor->Close();
	
	smrf=NULL;
	mainScore=NULL;
	return 1;
}

void WDFPartAdded(int partNum, char *name)
{
  Message("WDFPartAdded");
}

void WDFPartDeleted(int partNum)
{
  wxString msg;

  msg.sprintf("WDFPartDeleted: %d",partNum);
  wxMessageBox(msg);
}

void SetMusicEditorSMRF(SMRFile* psmrf)
{
	if(MusicEditor!=NULL)
		MusicEditor->smrf=psmrf;
	smrf=psmrf;
}

void MusicEditorPartAdded(int partNum, char *name)
{
	if(smrf!=NULL)
	{
		smrf->PartAdded(partNum,name);
	}
}

void MusicEditorPartDeleted(int partNum)
{
	if(smrf!=NULL)
	{		
		smrf->PartDeleted(partNum);
	}
}

static int StartMusicEditor(SMRFile *psmrf,Partitura* pmainScore, int nScore)
{
  Widget scoreFrame=NULL;
  smrf=psmrf;
  mainScore=pmainScore;
  nPart=nScore;

  if(VerifKey()!=0)
	  exit(1);
  
  if(!MusicEditor->NewViewAvailable())
  {
	  Message("No more views available\nclose a window and retry");
	  return 0;
  }
  MusicEditor->mainScore=pmainScore;
  MusicEditor->nScore=nScore;
  MusicEditor->smrf=psmrf;

  wxString title, tx=" - WEDEL Music Editor";

  scoreFrame=Scr->GOpenScoreWindow(title,nScore);
  
  Scr->GNewDC(LiooWidget);

  Colore oldcol = Scr->GetColor();
  Scr->SetColor((ColorIcon)white);            
  Scr->Rect(class Rectangle(Point(0, Scr->GetMaxPixY()),
                      Point(Scr->GetMaxPixX(), 0)), TRUE);
  Scr->SetColor(oldcol);
  // Inizializzazione finestra partitura
  if (nScore==0)
  {
    pMusicWin = new MasaeWindow;
    if(!pMusicWin) 
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","lioo.cpp","Main()");
    pMusicWin->SetTitle((unsigned char *)"Archivist");
  }
  else
  {
    pMusicWin = new DliooWindow;
    if(!pMusicWin)	
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","lioo.cpp","Main()");
	pMusicWin->SetTitle((unsigned char *)"Active");
  }
  ((wxMEScoreFrame*)scoreFrame)->SetMusicWin(pMusicWin);
  ((wxMEScoreFrame*)scoreFrame)->SetNScore(nScore);
  ((wxMEScoreFrame*)scoreFrame)->SetSimbImg(SimbImg);
  

  {
	  ((wxMEScoreFrame*)scoreFrame)->GetmiscMenu()->Enable(MISC_JUMPTOSYMBOLIC_ID,FALSE);
	  ((wxMEScoreFrame*)scoreFrame)->GetviewMenu()->Enable(VIEW_FIT2WINDOW_ID,FALSE);
  }


  pMusicWin->SetInView(((wxMEScoreFrame*)scoreFrame)->GetViewID());

  pMusicWin->WinInit(Point(0,0),      
                      VUnit(Scr->GetMaxPixX()-WIN_SPARTITO_DELTA_X),
                      VUnit(Scr->GetMaxPixY()-WIN_SPARTITO_DELTA_Y),
		      mainScore);
  DimensioneX=Scr->GetMaxPixX()-10;
  DimensioneY=Scr->GetMaxPixY()-31-10; 

  if(pmainScore!=NULL)
  {
  	  pmainScore->SetCurrentScore(nScore);
  }

  LoadDialogs();

  mouse.Show();

  // Impostazione del cursore a croce
  Scr->GSetCursor(DEFAULT);

  LiooList.Add(pMusicWin);
  
  pMusicWin->GoTop();
  pMusicWin->Resize(VUnit(Scr->GetMaxPixX()-WIN_SPARTITO_DELTA_X+1),
   		    VUnit(Scr->GetMaxPixY()-WIN_SPARTITO_DELTA_Y));
  

  Scr->GDeleteDC();  

  ContrScrool=true;    

  return 1;
}

int OpenMusicEditor(SMRFile *psmrf,Partitura* pmainScore,int nScore)
{
  if(nScore>pmainScore->GetNumSpa())
  {
    Message("Symbolic Score not present");
    return 0;
  }

  SimbImg=true; //symbolic score
  
  return StartMusicEditor(psmrf,pmainScore,nScore);
}

void InitExtraMusicEditor(int where) // 0 = STARTUP, 1=BEFORE LOAD 2=EXIT
{
}

int OpenExtraMusicEditor(SMRFile*,Partitura*,int nScore)
{
	return 0;
}

int CloseExtraMusicEditor()
{
	return 1;
}

char* GetExtraMusicEditorName()
{
	return 0; //NULL = not available
}

#endif