/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: def_net.hpp,v 1.1 2005/07/25 16:31:39 bellinpo Exp $ */


#ifndef NETDEF
#define NETDEF

#define DIRETTORE "MASE"
#define LOGFILE_NAME "lioo.log"

#ifdef OPD
#define LENGTH_COMMANDID sizeof(short int)
#define LENGTH_SELECTIONTYPE sizeof(short int)
#define LENGTH_SELOBJTYPE sizeof(short int)
#define DIM_FIELD_SymCmd sizeof(short)

// Massimo numero di nodi per gruppo
#define MAX_NODES_PER_GROUP 100
// Massimo numero di HOST
#define MAXHOSTS 256
#endif OPD

// Definizione intervallo consultazione PVM (in millisecondi).
#define PVMTimeInterval 10
// numero di battute inviate all'inizio e con i salti in editing
#define NUM_BAT 50

// Codice di ritorno generico ( < 0 indica una condizione di errore)
typedef int RetCode;


// ---------------------------------------------------------------------------
// ------------    Enum per gli stati di avanzamento delle battute  ----------

enum StateBat {
	NO_FORWARD,
		NO_BACKWARD,
		NO_MOVE,
		OK_MOVE
};

// -------------------------------------------------------------------------
//   -------------------   Tipi di messaggio     ---------------------------

// Tipo di messaggio : messaggio effettivo (usato nella pvm_send e pvm_receive)
#define MSG_EFFETTIVO 2

// Tipo di messaggio : messaggio multicast (usato nella pvm_send e pvm_receive)
#define MSG_MULTICAST 3


enum TipoMsg {
    MSG_ACK,         // Tipo di messaggio : messaggio di ACK (usato nella pvm_send e pvm_receive)
		MSG_ACK_MULTI   // Tipo di messaggio : messaggio multicast (usato nella pvm_send e pvm_receive)
};

// Parametro per la ricezione : ricezione da qualsiasi host
#define ANY_SENDER -1          

// Parametro per la ricezione : ricezione di qualsiasi messaggio
#define ANY_MSG -1          

//-----------------------------------------------------------------------------
//------------    define per la lunghezza dei campi nella configurazione ------

//
#define NODE_NAME_LEN 20

//
#define NODE_PLAYER_LEN 40

//
#define NODE_UIF_LEN 40

//
#define NODE_ABIL_LEN 29

//
#define HOST_IP_LEN 16

//
#define HOST_NAME_LEN 80

//
#define HOST_DISPLAY_LEN 20

//
#define HOST_DEVICE_LEN 20

//
#define GROUP_PARTE_LEN 20
//

// massimo numero di gruppi presenti
#define MAXGROUPS 100


// Massimo numero di nodi 
#define MAXNODES 255

// --------------------------------------------------------------------------
// -----------     define per gestione dei messaggi   ------------------------ 


// Ok .Messaggio ricevuto e presente nel buffer
#define OK_RECEIVE_MSG 0

// ok receive.Messaggio ricevuto
#define OK_RECEIVE 0

// Messaggio inviato correttamente
#define OK_SEND 0

// Messaggio di spacchettamento corretto
#define OK_UNPACKET 0


// Messaggio di Ack inviato correttamente
#define OK_SEND_ACK 0

// Messaggio di Ack inviato non correttamente
#define NO_SEND_ACK -15 


#define NO_PACKET -1
#define OK_PACKET 0

// Nessun messaggio presente nel buffer
#define NO_MSG -8



// No receive
#define NO_RECEIVE -1



#define NO_UNPACKET -10
#define NO_RECEIVE_ACK -1
#define NO_UNPACK_ACK -2
#define NO_BUF_SEND -3
#define NO_PACKET_SEND -4
#define NO_SEND_SEND -5
#define NO_SEND -6
#define NEGATIVE_ACK -7
#define ERR_REC -9




// 
//----------------------------------------------------------------------------
//  ----------------  Dimensione dei campi del messaggio  ---------------------


// Dimensione del campo TID in byte
#define DIM_TID 6

// Dimensione in Byte del campo ID di ogni processo
#define DIM_ID sizeof(char)

// Dimensione in Byte della stringa che rappresenta il tipo di comando
#define DIM_COMMAND_STRING (sizeof(short int)*2)  // OSS:  uguale a DIM_ACK

// Dimensione in Byte della codifica in stringa del comando
#define DIM_COMMAND_CODE (sizeof(short int)*2)

// Dimensione in Byte del campo di flag per ack positivo o negativo
#define DIM_FLAG sizeof(char)


// Dimensione in Byte del messaggio di Ack
#define DIM_ACK (DIM_ID+DIM_ID+DIM_COMMAND_STRING+DIM_FLAG)


// Dimensione in Byte del time della battuta nel .ett
#define DIM_TIME_ETT (sizeof(short int)*2)

// numero di battute a sinistra rispetto alla pagina corrente mantenute in memoria
#define LEFT_MARGIN_BATT 30

// numero di battute a destra rispetto alla pagina corrente mantenute in memoria
#define RIGHT_MARGIN_BATT 40


// numero di battute a sinistra rispetto alla pagina corrente 
// dopo le quali posso richiedere battute a MASAE
#define LEFT_DIST_BATT 10

// numero di battute a destra rispetto alla pagina corrente 
// dopo le quali posso richiedere battute a MASAE
#define RIGHT_DIST_BATT 20



enum StatoTrasmissioneBattute {
	BATTUTE_COMPLETATE = 1,
		BATTUTE_TERMINATE,
		BATTUTE_CONT
};                      


// ----------------------------------------------------------------------------
// ---------------         Tipi Di Messaggi        ----------------------------

enum MessageType {
	
	INIZIALIZZAZIONE = 1,         // Identificatore del comando di inizializzazione ID
		REQUEST_BATTUTA_FORWARD, 	// Identificatore del comando di richiesta battuta successiva
		REQUEST_BATTUTA_BACKWARD, 	// Identificatore del comando di richiesta battuta precedente
		ESEGUI_BATT, 			// Identificatore del comando di esecuzione battuta
		INIT_BATT, 			// Identificatore del comando di inizializzazione della battuta
		END_INIT_PARTE, 		// Identificatore del comando di fine invio della parte assegnata
		ESEGUI_COMMAND, 		// Identificatore del comando di esecuzione comando
		ESEGUI_QUIT, 			// Identificatore del comando di quit
		ESEGUI_NEW, 			// Identificatore del comando di new spartito
		REQUEST_EXEC_COMMAND, 		// richiesta di esecuziuone comando  dal dlioo
		SET_SPART_NUMCODE, 		// impostazione del NumericCode dello Spartito del DLIOO
		INIT_IMPAGINA, 			// identificatore di inizio impaginazione
		INIT_PARTE, 			// Identificatore del comando di inizio invio della parte assegnata
		REQUEST_NBAT, 			// identificatore di richiesta N battute
		SET_PAGE, 			// messaggio di settaggio battuta come inizio pagina
		FINE_IMPAGINA, 			// identificatore di fine impaginazione
		ESEGUI_BATT_IMP, 		// identificatore per eseguire la battuta nel caso di impaginazione
		IMPAGINA_ENDBATTUTE, 		// identificatore di fine battute per l'impaginazione
		ESEGUI_BATT_LOAD, 		// identificatore di fine inizializzazione della parte
		SINCR_BATT,                     // identificatore di sincronizzazione (perl'esecuzione)
		END_BATTUTE_FORWARD, 		// identificatore di fine battute avanti
		END_BATTUTE_BACKWARD, 		// identificatore di fine battute indietro
		LOAD_ETT,                        // messaggio di caricamento ett delle battute
		EXEC_PAGE_FORWARD,               // pagina successiva per l'esecuzione
		EXEC_ENDBATTUTE,                 // terminate le battute per l'esecuzione  
		FINE_EXEC,                       // fine fase di esecuzione
		INIT_EXEC,                       // inizializzazione della fase di esecuzione   
		ESEGUI_BATT_EXEC,                // esecuzione battuta in fase di execution
		ALT_EXEC,                        // alt all'esecuzione
		ERRORE,                           // messaggio di errore
		JUMP_TO_LABEL,                   // salto ad etichetta
		JUMP_TO_PAGE,                    // salto a pagina numero
		JUMP_TO_BATT,                     // salto a battuta numero                 
		KILL,
		DELTA_TIME,                      // messaggio di variazione tempo durata battuta
		WAIT_TIME,                       // messaggio di wait tempo durata battuta:ferma lo scrolling   
		DELETE_BATTUTE,
		NETGOTOP,
		NETGOBOTTOM,
		NETGOPIUCINQUE,                  // 40
		NETGOMENOCINQUE,
		NETGOLABEL,
		REQUESTJUMP,
		NETGOBATTUTA,
		NETGOBATTUTA_N,
		JUMPEDITING,                       // 45
		CAMBIA_ABIL,               // messaggio per cambiamento abilitazioni nodo (OPD -> MaMass)
		AGG_ABIL,                  // messaggio per cambiamento abilitazioni nodo (MaMass -> LDMAS)
		TYPE_CURSOR,
		GEST_LABEL,
		OFF_WAIT_MOVE              // messaggio per ripristinare il movimento in editing sui leggii
};                      



#endif
