/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: ldmas.cpp,v 1.1 2005/07/25 16:31:40 bellinpo Exp $ */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "net/ldmas.hpp"
#include "net/def_net.hpp"
#include "liooext.hpp"
#include "gui/extabil.hpp"
#include "drw/scrobjec.hpp"

extern ExtLiooAbil liooAbil;
extern ScrList LiooListWidget;


LdMas::LdMas(Partitura * Entry)
{
	miaPartitura = Entry;
	waitEditing = 0;
	ListaPag = NULL;
	
#ifdef WITH_NET    
	RetCode info_ack, reccode;
	char Ack[DIM_ACK+1];  // Vedi struttura Ack in Network.
	char CommandStringa[DIM_COMMAND_STRING+1];
	short int CommandCode;
	
	// inizializzazione puntatote a ldmas per l'errormanager
	EM.SetLdMas(this);
	
	// inizializzazione flag di wait per l'impaginazione
	wait = FALSE;
	
	// inizializzazione flag per richiesta battute avanti e indietro
	RichBatt = NO_MOVE;
	
	// Inizializzazione del tid_MASAE 
	tid_MASAE = pvm_parent();
	if (tid_MASAE<0)
	{
		EM.Error(PVM_ERR_AREA, "EC_PVM_NOPARENT", "LdMas", "LdMas()" );
	}
	
	// Attesa ID : interrogo la rete aspettando il messaggio
	reccode = NO_MSG;
	while (reccode==NO_MSG) 
		reccode=Receive();   
	
	// decodifica tipo di messaggio ricevuto
	strncpy(CommandStringa,Array+DIM_ID,DIM_COMMAND_STRING);
	CommandStringa[DIM_COMMAND_STRING]='\0'; 
	CommandCode = DecodeCommand(CommandStringa);
	
	// controllo che il comando ricevuto sia di INIZIALIZZAZIONE
	if (CommandCode != INIZIALIZZAZIONE)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","LdMas","LdMas()");
		//return;
	}
	
	// inizializzazione dell' attributo ID 
	my_ID = Array[DIM_ID+DIM_COMMAND_STRING];
	ID_MASAE = Array[0]-1;  
	if (ID_MASAE!=0)
	{
		ID_MASAE=0;
	}
	
	// inizializzazione menu
	liooAbil.LoadFromString(&Array[DIM_ID+DIM_COMMAND_STRING+1]);    
	
	// formazione ed invio dell ACK : ricevuta stringa di inizializzazione
	FormaAck(my_ID,Array,'p',Ack);
	info_ack = SendAck(tid_MASAE,Ack,MSG_ACK); 
	if (info_ack!=OK_SEND_ACK) 
	{
		EM.Error(NET_ERR_AREA,"EC_NO_ACK","DiMas","LdMas()");
		//return;  //problemi con l'ack 
	}
	// pulisco l'Array
	Array[0]='\0';
	
	// inizializzazione attributi per l'impaginazione 
	Amp = 0;
	ind=0;
	ListaPag = NULL;
#else
#endif 
}


LdMas::~LdMas()
{  
#ifdef WITH_NET   
	
	if (ListaPag!=NULL) delete[] ListaPag;
#ifdef DEBUG_FREE
	FILE *pippo; 
	pippo = fopen("/home/demo/integrazione/src/DEAL_NET.txt","a");
	fprintf(pippo,"\n\n\n ------------   DEALLOCO LDMASS ----------------\n\n\n");
	fclose(pippo);
#else
#endif 
#else
#endif 
	
}   


void LdMas::SetNumPagina(NumCode np)
{ NumPagina = np; }

NumCode LdMas::GetNumPagina()
{ return NumPagina;}

void LdMas::MenuJump(int NumBatt,int NumEtich,char *arrayLabel,int* arrayNumProg)
{
#ifdef WITH_NET   
	int nbat;
	char buffer[200];
	
	if (EDWidgetJump("Jump in Score",buffer,200,NumEtich ,NumBatt,arrayLabel,arrayNumProg))
	{ 
		
		if (!(strcmp(buffer,"TOP")))
		{
			LdMas::EditingGoTop();
		}  
		else if (!(strcmp(buffer,"BOT."))) 
		{
			LdMas::EditingGoBottom();
		} 
		else {   
			nbat = atoi(buffer);  // trova la battuta in forma numerica (NumericCode)
			LdMas::EditingJumpBatt(nbat);
		} 
	}		
#else
#endif 
}


void LdMas::RequestJump()
{
#ifdef WITH_NET   
	
	char Codice[DIM_COMMAND_STRING+1];
	RetCode infoCode, sendcode;
	
	// invio richiesta di jump al top  
	Array[0] = my_ID+1 ;   // +1 per evitare problemi quando l' ID e zero
	Array[1] ='\0';
	
	// codifica comando richiesto: da short int --> in 4 BYTE
	infoCode = CodeCommand((short int)REQUESTJUMP,Codice);
	strcat(Array,Codice);
	
	sendcode =SendMsg_ack(tid_MASAE);
	switch (sendcode) 
	{
    case OK_SEND:
		break;
    case NEGATIVE_ACK:
		EM.Error(NET_ERR_AREA,"EC_DEADLOCK","LdMas","RequestJump()");
		break;
    default:
		EM.Error(NET_ERR_AREA,"EC_NO_SEND","LdMas","RequestJump()");
		// Per ora niente.
		break;
	}
	Array[0]='\0'; 
#else
#endif   
}

void LdMas::GestLabel()
{
#ifdef WITH_NET   
	
	char Codice[DIM_COMMAND_STRING+1];
	RetCode infoCode, sendcode;
	
	// invio richiesta di jump al top  
	Array[0] = my_ID+1 ;   // +1 per evitare problemi quando l' ID e zero
	Array[1] ='\0';
	
	// codifica comando richiesto: da short int --> in 4 BYTE
	infoCode = CodeCommand((short int)GEST_LABEL,Codice);
	strcat(Array,Codice);
	
	sendcode =SendMsg_ack(tid_MASAE);
	switch (sendcode) 
	{
    case OK_SEND:
		break;
    case NEGATIVE_ACK:
		EM.Error(NET_ERR_AREA,"EC_DEADLOCK","LdMas","GestLabel()");
		break;
    default:
		EM.Error(NET_ERR_AREA,"EC_NO_SEND","LdMas","GestLabel()");
		// Per ora niente.
		break;
	}
	Array[0]='\0'; 
#else
#endif   
}




void LdMas::EditingGoTop()
{
#ifdef WITH_NET   
	char Codice[DIM_COMMAND_STRING+1];
	char CommandStringa[DIM_COMMAND_STRING+1];
	RetCode infoCode, sendcode; 
	short int CommandCode;
	
	waitEditing = NUM_BAT;
	
	Array[0] ='\0';
	
	// invio richiesta di jump al top  
	Array[0] = my_ID+1 ;   // +1 per evitare problemi quando l' ID e zero
	Array[1] ='\0';
	
	// codifica comando richiesto: da short int --> in 4 BYTE
	infoCode = CodeCommand((short int)JUMPEDITING,Codice);
	strcat(Array,Codice);
	
	infoCode = CodeCommand((short int)NETGOTOP,Codice);
	strcat(Array,Codice);
	
	strncpy(CommandStringa,Array+DIM_ID,DIM_COMMAND_STRING);
	CommandStringa[DIM_COMMAND_STRING]='\0'; 
	CommandCode = DecodeCommand(CommandStringa);
	
	sendcode =SendMsg_ack(tid_MASAE);
	switch (sendcode) 
	{
    case OK_SEND:
		RichBatt = OK_MOVE;
		break;
    case NEGATIVE_ACK:
		EM.Error(NET_ERR_AREA,"EC_DEADLOCK","LdMas","EditingGoTop()");
		break;
    default:
		EM.Error(NET_ERR_AREA,"EC_NO_SEND","LdMas","EditingGoTop()");
		// Per ora niente.
		break;
	}
	Array[0]='\0';
	
#else 
#endif 
}  


void LdMas::EditingGoBottom()
{
#ifdef WITH_NET            
	char Codice[DIM_COMMAND_STRING+1];
	RetCode infoCode, sendcode;
	
	waitEditing = NUM_BAT;
	
	// invio richiesta di jump al bottom  
	Array[0] = my_ID+1 ;   // +1 per evitare problemi quando l' ID e zero
	Array[1] ='\0';
	
	// codifica comando richiesto: da short int --> in 4 BYTE
	infoCode = CodeCommand((short int)JUMPEDITING,Codice);
	strcat(Array,Codice);
	
	infoCode = CodeCommand((short int)NETGOBOTTOM,Codice);
	strcat(Array,Codice);
	
	sendcode =SendMsg_ack(tid_MASAE);
	switch (sendcode) 
	{
    case OK_SEND:
		RichBatt = OK_MOVE;
		break;
    case NEGATIVE_ACK:
		EM.Error(NET_ERR_AREA,"EC_DEADLOCK","LdMas","EditingGoBottom()");
		break;
    default:
		EM.Error(NET_ERR_AREA,"EC_NO_SEND","LdMas","EditingGoBottom()");
		// Per ora niente.
		break;
	}
	
	Array[0]='\0';
#else 
#endif 
}  


void LdMas::EditingJumpBatt(int N)
{
#ifdef WITH_NET            
	char Codice[DIM_COMMAND_STRING+1];
	RetCode infoCode, sendcode;
	
	waitEditing = NUM_BAT;
	
	// invio richiesta di jump ad N  
	Array[0] = my_ID+1 ;   // +1 per evitare problemi quando l' ID e zero
	Array[1] ='\0';
	
	// codifica comando richiesto: da short int --> in 4 BYTE
	infoCode = CodeCommand((short int)JUMPEDITING,Codice);
	strcat(Array,Codice);
	
	infoCode = CodeCommand((short int)NETGOBATTUTA,Codice);
	strcat(Array,Codice);
	
	infoCode = CodeCommand((short int)N,Codice);
	strcat(Array,Codice);
	
	sendcode =SendMsg_ack(tid_MASAE);
	switch (sendcode) 
	{
    case OK_SEND:
		RichBatt = OK_MOVE;
		break;
    case NEGATIVE_ACK:
		EM.Error(NET_ERR_AREA,"EC_DEADLOCK","LdMas","EditingJumpBatt()");
		break;
    default:
		EM.Error(NET_ERR_AREA,"EC_NO_SEND","LdMas","EditingJumpBatt()");
		// Per ora niente.
		break;
	}
	Array[0]='\0';
#else 
#endif 
}  


int LdMas::Gettid_MASAE()
{
	return(tid_MASAE);
}



// metodo per andare indietro di nb battute
void LdMas::RichBattBackward(short nb)
{               
#ifdef WITH_NET              
	RetCode infoCode, sendcode;
	Battuta *pbat, *pbat1;
	Spartito *psp;
	char p, r;
	char Codice[DIM_COMMAND_STRING+1];
	int nbatdel; //numero di battute da deallocare (TMP)
	
	// per sapere se si deve deallocare qualcosa
	pbat1 = miaPartitura->GetFirstSpa()->GetLastBat();
	pbat = miaPartitura->GetFirstSpa()->GetfirstBatNextPage();
	
	p = 1;
	if (pbat!=NULL)
	{
		if (pbat1!=NULL)
		{
			// numero di battute da deallocare in cima.
			p = (pbat1->GetNumProgress() - pbat->GetNumProgress() + 1) - 10;
			
			
			if (p < 0) p = 0;
			// per permettere la trasmissione in rete
			
			int nl=0;
			int nltot=0;
			psp = miaPartitura->GetFirstSpa();
			int i=psp->GetNumeroBattute() -1;
			nbatdel=0;
			while(nltot<p)
			{
				psp = miaPartitura->GetFirstSpa();
				while (psp != NULL) 
				{
					pbat = psp->GetBattNum(i);
					nl = pbat->GetNumMultiRest();
					if(nl==0) nl=1; //se nl  zero vuol dire che  una battuta normale
					nltot+=nl;
					i--;
					nbatdel++;
					psp = miaPartitura->GetNextSpa(psp);
				} 
			}
			if(nltot>p)
			{
				nbatdel--;
				p=nltot-nl;
			}
			
			p++;   
			
			
			
			
			
			
			
		}
		else {  
			EM.Error(NET_ERR_AREA,"????????","LdMas","RichBattBackward()");
			return; 
		}
	}
	
	r = 1;
	if ((RichBatt!=NO_BACKWARD) && (RichBatt!=NO_MOVE))
	{
		pbat = miaPartitura->GetFirstSpa()->GetBattNum(0);
		pbat1 = miaPartitura->GetFirstSpa()->GetfirstBatPage();   
		
		
		if ((pbat!=NULL) && (pbat1!=NULL))
		{
			r = 10 - (pbat1->GetNumProgress() - pbat->GetNumProgress());
			if (r < 0) r = 0;
			r++;
		}
		else {  
			EM.Error(NET_ERR_AREA,"????????","LdMas","RichBattBackward()");
			return; 
		}
	}
	
	// trasmetto una informazione (richiesta battute oppure aggiornamento margini)
	if ((p > 1) || (r > 1))
	{
		// inserimento dell' ID del mittente 
		Array[0] = my_ID+1 ;   // +1 per evitare problemi quando l' ID e zero
		Array[1] ='\0';
		
		// codifica comando richiesto: da short int --> in 4 BYTE
		infoCode = CodeCommand((short int)REQUEST_BATTUTA_BACKWARD,Codice);
		strcat(Array,Codice);
		
		Array[5] = p;
		Array[6] = r;
		Array[7] ='\0';
		
		//
		// inserire controllo che la stringa non ecceda la dimensione del buffer 
		//
		
		// invio richiesta battuta  
		sendcode = SendMsg_ack(tid_MASAE);
		switch (sendcode) 
		{
		case OK_SEND:
			//  dealloco 
			
			if(nbatdel > 0)
			{
				if (RichBatt==NO_FORWARD) RichBatt=OK_MOVE;
				else if (RichBatt==NO_MOVE) RichBatt=NO_BACKWARD;
			}
			for (; nbatdel > 0; nbatdel--)
			{
				// cancellero' tutte le prime battute degli spartiti.
				psp = miaPartitura->GetFirstSpa();
				while (psp != NULL) 
				{
					pbat = psp->GetLastBat();
					psp->DeleteBat(pbat);
					psp = miaPartitura->GetNextSpa(psp);
				} 
			}
			
			{     
				Spartito *psp;
				Battuta *pbat,*pbat1;
				psp=miaPartitura->GetFirstSpa();
				pbat=psp->GetBattNum(0);
				pbat1=psp->GetBattNum(psp->GetNumeroBattute()-1);
				
			}
			
			waitEditing = r - 1;
			
			break;
		case NEGATIVE_ACK:
			EM.Error(NET_ERR_AREA,"EC_DEADLOCK","LdMas","RichBattBackward()");
			break;
		default:
			EM.Error(NET_ERR_AREA,"EC_NO_SEND","LdMas","RichBattBackward()");
			// Per ora niente.
			break;
		}
		return;
    }
#else 
#endif 
}



// metodo per andare avanti di nb battute
void LdMas::RichBattForward(short nb)
{   
	
#ifdef WITH_NET   
	RetCode infoCode, sendcode;
	Battuta *pbat, *pbat1;
	Spartito *psp;
	char p, r;
	char Codice[DIM_COMMAND_STRING+1];
	int nbatdel;
	
	// per sapere se si deve deallocare qualcosa
	pbat1 = miaPartitura->GetFirstSpa()->GetLastBat();
	pbat = miaPartitura->GetFirstSpa()->GetBattNum(0);
	
	pbat1 = miaPartitura->GetFirstSpa()->GetfirstBatPage();
	
	p = 1;
	if ((pbat!=NULL) && (pbat1!=NULL))
	{
		// numero di battute da deallocare in cima.
		p = pbat1->GetNumProgress() - pbat->GetNumProgress() - 10;
		if (p < 0) p = 0;    // per permettere la trasmissione in rete
		
		int nl=0;
		int nltot=0;
		int i=0;
		while(nltot<p)
		{
			psp = miaPartitura->GetFirstSpa();
			while (psp != NULL) 
			{
				pbat = psp->GetBattNum(i);
				nl = pbat->GetNumMultiRest();
				if(nl==0) nl=1;
				nltot+=nl;
				i++;
				psp = miaPartitura->GetNextSpa(psp);
			} 
		}
		if(nltot>p)
		{
			i--;
			p=nltot-nl;
		}
		nbatdel=i;
		
		p++;
		
		
		
	}
	else {  
		EM.Error(NET_ERR_AREA,"????????","LdMas","RichBattForward()");
		return; 
	}
	
	r = 1;
	if ((RichBatt!=NO_FORWARD) && (RichBatt!=NO_MOVE))
	{
		pbat1 = miaPartitura->GetFirstSpa()->GetLastBat();
		pbat = miaPartitura->GetFirstSpa()->GetfirstBatNextPage();
		
		if ((pbat!=NULL) && (pbat1!=NULL))
		{
			r = 10 - (pbat1->GetNumProgress() - pbat->GetNumProgress() + 1);
			if (r < 0) r = 0;
			r++;
		}
		else
		{
			if (pbat==NULL)
				r = 11;
			else {  
				EM.Error(NET_ERR_AREA,"????????","LdMas","RichBattForward()");
				return; 
            }
		}
	}
	
	
	// trasmetto una informazione (richiesta battute oppure aggiornamento margini)
	if ((p > 1) || (r > 1))
	{
		// inserimento dell' ID del mittente 
		Array[0] = my_ID+1 ;   // +1 per evitare problemi quando l' ID e zero
		Array[1] ='\0';
		
		// codifica comando richiesto: da short int --> in 4 BYTE
		infoCode = CodeCommand((short int)REQUEST_BATTUTA_FORWARD,Codice);
		strcat(Array,Codice);
		
		Array[5] = p;
		Array[6] = r;
		Array[7] ='\0';
		
		//
		// inserire controllo che la stringa non ecceda la dimensione del buffer 
		//
		
		// invio richiesta battuta  
		sendcode = SendMsg_ack(tid_MASAE);
		switch (sendcode) 
		{
		case OK_SEND:
			//  dealloco 
			if (nbatdel > 0)
			{
				if (RichBatt==NO_BACKWARD) RichBatt=OK_MOVE;
				else if (RichBatt==NO_MOVE) RichBatt=NO_FORWARD;
			}
			
			for (; nbatdel > 0; nbatdel--)
			{
				// cancellero' tutte le prime battute degli spartiti.
				psp = miaPartitura->GetFirstSpa();
				while (psp != NULL) 
				{
					pbat = psp->GetBattNum(0);
					psp->DeleteBat(pbat);
					psp = miaPartitura->GetNextSpa(psp);
				} 
			}
			
			{     
				Spartito *psp;
				Battuta *pbat,*pbat1;
				psp=miaPartitura->GetFirstSpa();
				pbat=psp->GetBattNum(0);
				pbat1=psp->GetBattNum(psp->GetNumeroBattute()-1);
				
			}
			
			
			waitEditing = r - 1;
			
			break;
		case NEGATIVE_ACK:
			EM.Error(NET_ERR_AREA,"EC_DEADLOCK","LdMas","RichBattForward()");
			break;
		default:
			EM.Error(NET_ERR_AREA,"EC_NO_SEND","LdMas","RichBattForward()");
			// Per ora niente.
			break;
		}
		return;
	}   
#else 
#endif 
}




//Richiesta di inserimento comando
void LdMas::InvioComandosuRete(SymCmd SC)
{
	
#ifdef WITH_NET   
	
	RetCode infoCode,sendcode;
	char Codice[DIM_COMMAND_STRING+1];
	
	Array[0]=my_ID+1;
	Array[1]='\0';
	infoCode = CodeCommand((short int) REQUEST_EXEC_COMMAND,Codice);
	strcat(Array,Codice);
	
	Sym2String(&Array[DIM_ID+DIM_COMMAND_STRING],SC);
	
	sendcode =SendMsg_ack(tid_MASAE);
	switch (sendcode)
	{
    case OK_SEND:
		break;
    case NEGATIVE_ACK:
		EM.Error(NET_ERR_AREA,"EC_DEADLOCK","LdMas","InvioComandosuRete()");
		break;
    default:
		EM.Error(NET_ERR_AREA,"EC_NO_SEND","LdMas","InvioComandosuRete()");
		// Per ora niente.
		break;
	}
#else
	
#endif 
	
}  



// Esegue il comando che  in Array, che e' stato inviato da Mamass
// Inteso con questo anche l'aggiornamento della battuta interessata ecc.
// 
void LdMas::EseguiCommand(SymCmd &SC)
{
	
#ifdef WITH_NET   
	// conversione da stringa a comando simbolico dell'informazione arrivata
	String2Sym(&Array[DIM_ID+DIM_COMMAND_STRING],SC); 
	
	// Esecuzione del comando ricevuto
	miaPartitura->DoNetSymCmd();
#else
#endif 
}

// settaggio pagina
void LdMas::AggiungiPagina(NumCode nc, NumCode np)
{
	int i;
	static NumCode Nump = 0;
	
	
	if (Amp==0)
	{
		ListaPag = new NumCode[21];
		if (ListaPag==NULL) 
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","LdMas","AggiungiPagina()");
		
		Amp = 20;
		
		ListaPag[0] = (miaPartitura->GetFirstSpa()->GetBattNum(0))->GetNumericCode();
		ListaPag[1] = 1;
		ind = 2;
		Nump = 0;
		return;
	}
	
	if (Nump == np) return;
	
	ListaPag[ind++] = nc;
	ListaPag[ind++] = np;
	Nump = np;
	
	if (ind >= Amp)
	{
		NumCode *list, *p;
		list = new NumCode[Amp+21];
		
		for (i=0; i<ind; i++) list[i] = ListaPag[i];
		Amp = Amp + 20;
		
		p = ListaPag;
		ListaPag = list;
		list = NULL;
		delete[] p;
		p=NULL; 
	}
}


//
void LdMas::RemoveListaPag()
{
	if(ListaPag!=NULL) delete[] ListaPag;
	ListaPag = NULL;
	Amp = 0;
	ind = 0;
}


// settaggio pagina
void LdMas::SetPage()
{
	RetCode infoCode,sendcode;
	char Codice[DIM_COMMAND_STRING+1];
	char i; 
	
	if (ind==0) return;
	
	Array[0]=my_ID+1;
	Array[1]='\0';
	infoCode = CodeCommand((short int) SET_PAGE,Codice);
	strcat(Array,Codice);
	
	infoCode = CodeCommand((short int) (ind/2),Codice);
	strcat(Array,Codice);
	
	
	for (i=0;i<ind;i++)
	{
		
		infoCode = CodeCommand((short int) ListaPag[i],Codice);
		strcat(Array,Codice);
	};
	
	ind = 0;
	
	sendcode =SendMsg_ack(tid_MASAE);
	switch (sendcode) 
	{
	case OK_SEND:
		break;
	case NEGATIVE_ACK:
		if (!(pMusicWin->GetImpaginazione()))
			EM.Error(NET_ERR_AREA,"EC_DEADLOCK","LdMas","SetPage()");
		break;
	default:
		EM.Error(NET_ERR_AREA,"EC_NO_SEND","LdMas","SetPage()");
		// Per ora niente.
		break;
	} 
};



// Messaggio di richiesta di N battute a MaMass 
void LdMas::Req_NBat(short int nbat)
{
	RetCode infoCode,sendcode;
	char Codice[DIM_COMMAND_STRING+1];
	
	if (wait) return;
	
	SetPage();
	
	Array[0]=my_ID+1;
	Array[1]='\0';
	infoCode = CodeCommand((short int) REQUEST_NBAT,Codice);
	strcat(Array,Codice);
	
	infoCode = CodeCommand((short int)nbat,Codice);
	strcat(Array,Codice);
	
	sendcode =SendMsg_ack(tid_MASAE);
	switch (sendcode) 
	{
	case OK_SEND:
		break;
	case NEGATIVE_ACK:
		EM.Error(NET_ERR_AREA,"EC_DEADLOCK","LdMas","Req_NBat()");
		break;
	default:
		EM.Error(NET_ERR_AREA,"EC_NO_SEND","LdMas","Req_NBat()");
		// Per ora niente.
		break;
	}
	
	wait = TRUE;  
	
};

// settaggio della lista che contiene le informazioni 
// della impaginazione  effettuata fino al momento della 
// chiamata di questo metodo
void LdMas::ImpostaBattPage(NumCode nc, NumCode np)
{
	AggiungiPagina(nc,np);
}


void LdMas::SendEndImp()
{ 
	RetCode infoCode,sendcode;
	char Codice[DIM_COMMAND_STRING+1];
	
	Array[0]=my_ID+1;
	Array[1]='\0';
	infoCode = CodeCommand((short int) FINE_IMPAGINA,Codice);
	strcat(Array,Codice);
	
	sendcode =SendMsg_ack(tid_MASAE);
	switch (sendcode) 
	{
	case OK_SEND:
		break;
	case NEGATIVE_ACK:
		if (!(pMusicWin->GetImpaginazione()))
			EM.Error(NET_ERR_AREA,"EC_DEADLOCK","LdMas","SendEndImp()");
		break;
	default:
		EM.Error(NET_ERR_AREA,"EC_NO_SEND","LdMas","SendEndImp()");
		// Per ora niente.
		break;
	}
};



void LdMas::WaitState()
{
#ifdef WITH_NET 
	Bool State;
	char CommandStringa[DIM_COMMAND_STRING+1];
	short int CommandCode;
	char *ack_sincr;
	RetCode info_ack;
	
	
	// invio ack + proprio id di riconoscimento
	ack_sincr=new char[DIM_ACK+1];   
	ack_sincr=FormaAck(my_ID,Array,'n',ack_sincr);
	
    
	info_ack=SendAck(tid_MASAE, ack_sincr,MSG_ACK_MULTI);
	if (info_ack!=OK_SEND_ACK) 
	{
		EM.Error(NET_ERR_AREA,"EC_NO_ACK","DiMas","SincrBatt()");
	}
	delete[] ack_sincr;
	
	
	// gestione riferimento battuta attuale di esecuzione
	strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
	CommandStringa[DIM_COMMAND_STRING] = '\0';
	CommandCode = DecodeCommand(CommandStringa);
	// in commandCode c'e il numero della battuta in formato 1 ...
	State = CommandCode;
	
	pMusicWin->SetWaitState(State);
#else
#endif 
}

void LdMas::TypeCursor()
{
#ifdef WITH_NET 
	char *ack_sincr;
	RetCode info_ack;
	int ct, fgc;
	char CommandStringa[DIM_COMMAND_STRING+1];
	
	strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
	ct = DecodeCommand(CommandStringa);
	
	strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
	fgc = DecodeCommand(CommandStringa);
	
	Scr->GSetMainDrawArea();
	Scr->GSetCursor((CursorTypes)ct,fgc);
	
	// invio ack + proprio id di riconoscimento
	ack_sincr=new char[DIM_ACK+1];   
	ack_sincr=FormaAck(my_ID,Array,'p',ack_sincr);
	info_ack=SendAck(tid_MASAE, ack_sincr,MSG_ACK_MULTI);
	if (info_ack!=OK_SEND_ACK) 
	{
		EM.Error(NET_ERR_AREA,"EC_NO_ACK","DiMas","SincrBatt()");
		//return;  //problemi con l'ack 
	}
	delete[] ack_sincr;
	
#else
#endif 
}

void LdMas::ReadFromNetwork()
{ 
#ifdef WITH_NET 
	int indice,i,j, ct, fgc, NumEtich, veloc;
	short int CommandCode;
	int* arrayNumProg; 
	float fat;
	long te;
	
	char Ack[DIM_ACK+1];  // Vedi struttura Ack in Network.
	char CommandStringa[DIM_COMMAND_STRING+1];
	char* arrayLabel;
	
	RetCode info_ack, reccode = NO_MSG;   
	Spartito *psp1;
	Battuta *pbat, *pbat1;
	NumCode nc, NumBatt;
	
	FILE *pippo;
	
	// formazione Ack   
	FormaAck(my_ID,Array,'p',Ack);
	
	strncpy(CommandStringa,Array+DIM_ID,DIM_COMMAND_STRING);
	
	CommandStringa[DIM_COMMAND_STRING] = '\0';
	CommandCode = DecodeCommand(CommandStringa);   
	
	//
	//  ------------  gestione messaggio ricevuto 
	//
    
	
	
	switch (CommandCode)
	{
    case (OFF_WAIT_MOVE):
		
		// gestione del flag di wait
		
		waitEditing = 0;
		
		
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LDMas","CommandProcess()");
		break;	      
		
    case (TYPE_CURSOR):
		TypeCursor();
		break;	      
		
    case (NETGOBATTUTA_N):
		strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
		NumBatt = DecodeCommand(CommandStringa);
		
		miaPartitura->GoBatt(NumBatt);
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LDMas","CommandProcess()");
		break;
		
    case (GEST_LABEL):
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LDMas","CommandProcess()");
		
		
		strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
		NumEtich = DecodeCommand(CommandStringa);
		arrayLabel=new char [NumEtich+1];
		arrayNumProg=new int [NumEtich+1];
		
		indice=(int)(DIM_ID+(2*DIM_COMMAND_STRING));
		
		// creazione vettore delle label
		for (i=0;i<NumEtich;i++)
		{
			arrayLabel[i]= Array[indice+i];
		}         
		char label[2];
		label[1]='\0';
		if (WReadEtich("Labels",label,200, NumEtich,arrayLabel)) 
		{
			label[1]='\0';
			pMusicWin->SetCommand(CMD_BATTUTA_LETTERA,IMMEDIATE_SEL,BATTUTA_SEL,0,0,0,0,0,label);
			
		}
		else
			Scr->GSetMainDrawArea();
		
		pMusicWin->HideTagli();
		
		delete[] arrayLabel;
		delete[] arrayNumProg; 		 
		
		break;
		
		
    case (REQUESTJUMP):
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LDMas","CommandProcess()");
		
		
		strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
		NumBatt = DecodeCommand(CommandStringa);
		
		strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
		NumEtich = DecodeCommand(CommandStringa);
		arrayLabel=new char [NumEtich+1];
		arrayNumProg=new int [NumEtich+1];
		
		indice=(int)(DIM_ID+(3*DIM_COMMAND_STRING));
		
		// creazione vettore delle label
		for (i=0;i<NumEtich;i++)
		{
			arrayLabel[i]= Array[indice+i];
			strncpy(CommandStringa,Array+indice+i+1,DIM_COMMAND_STRING);
			arrayNumProg[i] = DecodeCommand(CommandStringa);
			
			indice += DIM_COMMAND_STRING;
		}         
		
		MenuJump(NumBatt,NumEtich,arrayLabel,arrayNumProg); 
		
		delete[] arrayLabel;
		delete[] arrayNumProg; 		 
		
		break;
		
	case (NETGOTOP):
		miaPartitura->GoTop();
		// invio dell ACK 
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LDMas","CommandProcess()");
		break;
		
	case (NETGOBOTTOM):
		miaPartitura->GoBottom();
		// invio dell ACK 
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LDMas","CommandProcess()");
		break;
		
		
	case (KILL): // caso particolare, non si reinvia l'ack perche' utile cosi'
		exit(1);
		break;
		
    case (END_BATTUTE_FORWARD):
		// aggiornamento stato di scrolling
		if (RichBatt==NO_BACKWARD) RichBatt=NO_MOVE;
		else RichBatt=NO_FORWARD;
		waitEditing = 0;
		
		// invio dell ACK       
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK); 
		if (info_ack!=OK_SEND_ACK) 
		{
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LDMas","CommandProcess()");
			//return;  //problemi con l'ack 
		}
		{     
			Spartito *psp;
			Battuta *pbat,*pbat1;
			psp=miaPartitura->GetFirstSpa();
			pbat=psp->GetBattNum(0);
			pbat1=psp->GetBattNum(psp->GetNumeroBattute()-1);
			
		}
		break;
		
    case (END_BATTUTE_BACKWARD):
		// aggiornamento stato di scrolling         
		if (RichBatt==NO_FORWARD) RichBatt=NO_MOVE;
		else RichBatt=NO_BACKWARD;
		waitEditing = 0;
		
		// invio dell ACK       
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
		{
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LDMas","CommandProcess()");
			//return;  //problemi con l'ack 
		}
		{ 
			Spartito *psp;
			Battuta *pbat,*pbat1;
			psp=miaPartitura->GetFirstSpa();
			pbat=psp->GetBattNum(0);
			pbat1=psp->GetBattNum(psp->GetNumeroBattute()-1);
			
		}
		break;
		
    case (ESEGUI_BATT):           
		miaPartitura->LoadS(DIM_ID + DIM_COMMAND_STRING);
		
		{ 
			Spartito *psp;
			Battuta *pbat,*pbat1;
			psp=miaPartitura->GetFirstSpa();
			pbat=psp->GetBattNum(0);
			pbat1=psp->GetBattNum(psp->GetNumeroBattute()-1);
			
		}
		
		waitEditing --;
		
		// Usato l'Array, lo pulisco 
		Array[0] = '\0'; 
		// invio dell ACK       
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LDMas","CommandProcess()");
		break; 
		
	case (ESEGUI_QUIT):
		// Usato l'Array, lo pulisco            
		Array[0] = '\0';
		
		// invio dell ACK       
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
		{
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LDMas","CommandProcess()");
			//return;  //problemi con l'ack 
		}
		exit(1); 
		break;
		
	case (DELTA_TIME):
		info_ack=SendAck(tid_MASAE, Ack,MSG_ACK_MULTI);
		if (info_ack!=OK_SEND_ACK) 
		{
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LDMas","CommandProcess()");
			//return;  //problemi con l'ack 
		}
		
		strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
		veloc = DecodeCommand(CommandStringa);
		
		if (veloc > pMusicWin->GetVelocEsec()) // aumento della velocita'
		{
			if (pMusicWin->GetScrollingTime()/2 < MIN_CALL_TIMER)
			{
				pMusicWin->SetScrollingInc(pMusicWin->GetScrollingInc()*2);
			}
			else
				pMusicWin->SetScrollingTime(pMusicWin->GetScrollingTime()/2);
		}
		else	// diminuzione della velocita'
			pMusicWin->SetScrollingTime(pMusicWin->GetScrollingTime()*2);
		
		pMusicWin->SetVelocEsec(veloc);
		break;
		
	case (WAIT_TIME):
		
		WaitState();	 
		break;
		
	case (AGG_ABIL):   // Per aggiornamento abilitazioni
		
		// invio dell ACK       
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
		{
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LDMas","CommandProcess()");
		}
		
		// inizializzazione menu
		liooAbil.LoadFromString(&Array[DIM_ID+DIM_COMMAND_STRING]);
		LiooListWidget.DrawAll();
		Scr->GSetMainDrawArea();		
		menurapido.Draw();
		break;	 
		
	default: 
		EM.Error(NET_ERR_AREA,"EC_NO_ACK","LDMas","CommandProcess() cod");
		// Usato l'Array, lo pulisco 
        Array[0] = '\0';
		// invio dell ACK       
        info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
        if (info_ack!=OK_SEND_ACK) 
		{
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LDMas","CommandProcess()");
			//return;  //problemi con l'ack 
		}
		break;          
    }  
#else
#endif 
}



//
//------------------------ funzioni di utilita'  ----------------------------------------------------
//

void LdStampaSym(FILE *pippo,SymCmd sc)
{ int j;
fprintf(pippo,"%d %d %d ",sc.CommandId,sc.SelectionType,sc.SelObjType);
for (j=0;j<PATH_LENGTH;j++){fprintf(pippo,"%d ",sc.Path1.Array[j]);};
fprintf(pippo,"%d ",sc.Path1.Level);for (j=0;j<PATH_LENGTH;j++){\
fprintf(pippo,"%d ",sc.Path2.Array[j]);};
fprintf(pippo,"%d ",sc.Path1.Level);
for (j=0;j<SYM_MAXARG;j++){fprintf(pippo,"%d ",sc.ArgV[j]);};
fprintf(pippo,sc.Text);
fprintf(pippo,"\n");}

void LdMas::StampaStato(void)
{
	FILE *pippo;
	pippo=fopen("/home/demo/moodsMic/txt/STATO.TXT","a");
	
	fprintf(pippo,"STATO ATTUALE : \n");
	fprintf(pippo," Codice Numerico Prima Battuta  : %d\n",miaPartitura->GetFirstSpa()->GetBattNum(0)->GetNumericCode());
	fprintf(pippo," Numero progres. Prima Battuta  : %u\n",miaPartitura->GetFirstSpa()->GetBattNum(0)->GetNumProgress());
	fprintf(pippo," Codice Numerico Ultima Battuta : %d\n",miaPartitura->GetFirstSpa()->GetLastBat()->GetNumericCode());
	fprintf(pippo," Numero progres. Ultima Battuta : %u\n",miaPartitura->GetFirstSpa()->GetLastBat()->GetNumProgress());
	fclose(pippo);
}

void LdMas::BEEP(char * Stringa)
{
	return;
}

void LdMas::BEEP(unsigned int Appo)
{
	
	return;
}



