/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: logfile.cpp,v 1.1 2005/07/25 16:31:40 bellinpo Exp $ */


#include "lioo.hpp"
#include "net/logfile.hpp"
#include "gui/lioowin.hpp"
#include "net/mamass.hpp"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#undef LoadString

extern LiooWindow *pMusicWin;

/*-------------------------------------------------------------------------------------*/
/*                                       Classe LogListEntry                           */
/*-------------------------------------------------------------------------------------*/

LogListEntry::LogListEntry(void) {
	// Costruttore di base
	Next=NULL;
	Prev=NULL;
	return;
};

LogListEntry::~LogListEntry(void) 
{
	
	return;
};



LogListEntry::LogListEntry(int TID,char *NomeHost,char *NomePlayer,char *NomeSpartito, SymCmd *CmdStrt,
						   signed int b1,signed int f1,signed int b2,signed int f2,
						   Bool IsDir,Bool MustVer)
{
	// Inizializzo i dati principali dell'oggetto LogListEntry
	SetData(TID,NomeHost,NomePlayer,NomeSpartito,
		b1,f1,b2,f2,IsDir,MustVer);	
	
	// Copio il SymCmd nella mia struttura
	CopiaCommandStruct(&MyCmd,CmdStrt);	
	
	// Inizilizzo i campi "TipoComando" e "DescrizioneComando"
	PreparaComando();
	
	Next=NULL;
	Prev=NULL;
	return;
};


void LogListEntry::SetData (int Tid,char *NomeHo,char *NomePl,char *NomeSp, 
							signed int b1,signed int f1,signed int b2,signed int f2,
							Bool IsDir,Bool MustVer)
{
	TID=Tid;
	strncpy(NomeHost,NomeHo,MaxListEntryString-1);
	strncpy(NomePlayer,NomePl,MaxListEntryString-1);
	strncpy(NomeSpartito,NomeSp,MaxListEntryString-1);
	
	nbat=b1;
	nfig=f1;
	nbat1=b2;
	nfig1=f2;
	
	FromDirector=IsDir;
	MustBeConvalidated=MustVer;
	
	return;
};

void LogListEntry::Convalida(void)
{
	MustBeConvalidated=FALSE;
	return;
}

void LogListEntry::Invalida(void)
{
	MustBeConvalidated=TRUE;
	return;
}

Bool LogListEntry::IsInFilter(LogFilter * CurrentFilter)
{
	
	if (CurrentFilter->CheckFilter(FILTER_NUM_BAT)) {
		// Se vedo che esce da questa catgoria posso uscire immediatamente con FALSE
		if ((nbat !=0) && (CurrentFilter->GetNumBat()!=nbat)) return FALSE;			
	}
	
	if (CurrentFilter->CheckFilter(FILTER_PARTE)) {
		// Se vedo che esce da questa catgoria posso uscire immediatamente con FALSE
		// Faccio la symbolic hit del comando per vedere quale spartito ha colpito !
		Spartito *pSpa;
		if (pMusicWin->SymbolicHit(MyCmd.Path1,SPARTITO_SEL,(DrawObject *&)pSpa)) {
			if (pSpa!=NULL) {			
				Spartito* Entry=pMusicWin->PPartit->GetSpartNum(CurrentFilter->GetNumPar());
				if (Entry!=NULL) {
					if (Entry!=pSpa) return FALSE;
					
				}			
			}
		}	
	}
	
	if (CurrentFilter->CheckFilter(FILTER_MUSICIST)) {
        // Se vedo che esce da questa catgoria posso uscire immediatamente con FALSE	
		char Nome[200];
		char *Temp;
		Temp=((MaMass *)(pMusicWin->PPartit))->GetMusicista(CurrentFilter->GetNumMus()+1);
		if (Temp!=NULL) { 
			strcpy(Nome,Temp);			
			if (strcmp(Nome,NomePlayer)) return FALSE;
		}
		
		
	}
	
	return TRUE;
}

void LogListEntry::PreparaComando(void) 
{
	
	switch(MyCmd.CommandId) {
	case CMD_DEL_SYMBOL:
		TipoComando=ComandoCancellazione;
		strcpy(DescrizioneComando,"Simbol");	
		break;
	case CMD_DEL_COL_FIGURE:
        TipoComando=ComandoCancellazione;
		strcpy(DescrizioneComando,"Figure Column");
		break;
	case CMD_PARTITURA_ADD_GRAFFA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Curly Bracket");
		break;
	case CMD_PARTITURA_ADD_SPARTITO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Part");
		break;
	case CMD_PARTITURA_DEL_SPARTITO:
		TipoComando=ComandoCancellazione;
		strcpy(DescrizioneComando,"Part");
		break;
	case CMD_PARTITURA_NEW:
		TipoComando=ComandoAzione;
		strcpy(DescrizioneComando,"New Part");
        break;
	case CMD_PARTITURA_LOAD:
		TipoComando=ComandoAzione;
		strcpy(DescrizioneComando,"Load Part");
        break;
	case CMD_PARTITURA_SAVEAS:
		TipoComando=ComandoAzione;
		strcpy(DescrizioneComando,"Save as Part");
        break;
	case CMD_PARTITURA_SAVE:
		TipoComando=ComandoAzione;
		strcpy(DescrizioneComando,"Save Part");
        break;
	case CMD_PARTITURA_END:
		TipoComando=ComandoAzione;
		strcpy(DescrizioneComando,"Close Part");
        break;
	case CMD_PARTITURA_ADD_BATTUTA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Bar");
		break;
	case CMD_PARTITURA_DEL_BATTUTA:
		TipoComando=ComandoCancellazione;
		strcpy(DescrizioneComando,"Bar");
		break;
	case CMD_SPARTITO_ADD_LEGATURA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Slur");
		break;
	case CMD_SPARTITO_ADD_LEGATURA_VALORE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Tie");
		break;
	case CMD_SPARTITO_ADD_FORC_CRESC:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Wedge Inc.");
		break;
	case CMD_SPARTITO_ADD_FORC_DIMIN:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Wedge Dec.");
		break;
	case CMD_SPARTITO_ADD_FRECCIA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Arrow");
		break;
	case CMD_SPARTITO_ADD_ONDA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Wave");
		break;
	case CMD_SPARTITO_ADD_LEG_QUADRA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Bracket");
		break;
	case CMD_SPARTITO_ADD_CAMBIO_RIT:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Ending");
		break;
	case CMD_SPARTITO_ADD_MODIF8:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Octave sign");
		break;
	case CMD_SPARTITO_ADD_MODIF15MA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Quindicesima sign");
		break;
	case CMD_SPARTITO_ADD_MODIF8BA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Octave sign");
		break;
	case CMD_SPARTITO_ADD_MODIF15BA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Quindicesima sign");
		break;
	case CMD_SPARTITO_SET_NAME:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Part Name");
		break;
	case CMD_BATTUTA_NOTE2GRUPPONOTE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Notes->Beam");
        break;
	case CMD_BATTUTA_GRUPPONOTE2NOTE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Beam->Notes");
        break;
	case CMD_BATTUTA_ELIMINAGRUPPI:
		TipoComando=ComandoAzione;
		strcpy(DescrizioneComando,"Debuilt Beams");
        break;
	case CMD_BATTUTA_ERASE:
		TipoComando=ComandoCancellazione;
		strcpy(DescrizioneComando,"Bar");
		break;
	case CMD_BATTUTA_BSINGOLA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Single Barline");
		break;
	case CMD_BATTUTA_BSINGOLA_RIP2B:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Repeat Double Measure Barline");
		break;
	case CMD_BATTUTA_BDOPPIA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Double Barline");
		break;
	case CMD_BATTUTA_BINIZIORIT:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Left Repeat Barline");
		break;
	case CMD_BATTUTA_BFINERIT:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Right Repeat Barline");
		break;
	case CMD_BATTUTA_BINIZIOFINERIT:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Double Repeat Barline");
		break;
	case CMD_BATTUTA_BFINALE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Heavvy double Barline");
		break;
	case CMD_BATTUTA_BINVISIBILE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Hidden Barline");
		break;
		
	case CMD_BATTUTA_NUGUALE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"NUGUALE??");
		break;
	case CMD_BATTUTA_TSALTO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Jump Text");
		break;
	case CMD_BATTUTA_RIFERIMENTO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Refrain Reference Symbol");
		break;
		//
	case CMD_BATTUTA_LETTERA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Bookmark");
		break;
	case CMD_BATTUTA_SCANSIONE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Scansione???");
		break;
	case CMD_BATTUTA_MOVIMENTO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Tempo mark");
		break;
	case CMD_BATTUTA_INS_NBREVE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Double whole note");
		break;
	case CMD_BATTUTA_INS_NSEMIBREVE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Whole note");
		break;
	case CMD_BATTUTA_INS_NMINIMA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Half note");
		break;
	case CMD_BATTUTA_INS_NSEMIMINIMA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Quarter note");
		break;
	case CMD_BATTUTA_INS_NCROMA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Eight note");
		break;
	case CMD_BATTUTA_INS_NSEMICROMA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Sixteenth note");
		break;
	case CMD_BATTUTA_INS_NBISCROMA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Thirty-second note");
		break;
	case CMD_BATTUTA_INS_NSEMIBISCROMA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Sixty-fourth note");
		break;
	case CMD_BATTUTA_INS_NFUSA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"1/128 note");
		break;
	case CMD_BATTUTA_INS_PQUABATT:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Four measure rest");
		break;
	case CMD_BATTUTA_INS_PDUEBATT:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Two mesasure rest");
		break;
	case CMD_BATTUTA_INS_PSEMIBREVE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Whole rest");
		break;
	case CMD_BATTUTA_INS_PMINIMA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Half rest");
		break;
	case CMD_BATTUTA_INS_PSEMIMINIMA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Quarter rest");
		break;
	case CMD_BATTUTA_INS_PCROMA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Eight rest");
		break;
	case CMD_BATTUTA_INS_PSEMICROMA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Sixteenth rest");
		break;
	case CMD_BATTUTA_INS_PBISCROMA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Thirty-second rest");
		break;
	case CMD_BATTUTA_INS_PSEMIBISCROMA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Sixty-fourth rest");
		break;
	case CMD_BATTUTA_INS_PFUSA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"1/128 rest");
		break;
	case CMD_BATTUTA_INS_PGENERICA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Generic rest");
		break;
	case CMD_BATTUTA_INS_SPAZIO:
		if (MyCmd.ArgV[2]==0) {
			TipoComando=ComandoInserimento;
			strcpy(DescrizioneComando,"Empty");
		} else switch (MyCmd.ArgV[2]) {
				case CMD_BATTUTA_CVIOLINO:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"G Clef");
					break;
				case CMD_BATTUTA_CVIOLINO8:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"G8vb Clef");
					break;
				case CMD_BATTUTA_CVIOLINO8SOPRA:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"G8va Clef");
					break;
				case CMD_BATTUTA_CBASSO:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"F Clef");
					break;
				case CMD_BATTUTA_CBASSO8:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"F8vb Clef");
					break;
				case CMD_BATTUTA_CBASSO8SOPRA:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"F8va Clef");
					break;
				case CMD_BATTUTA_CBASSOOLD:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"OldC Clef");
					break;
				case CMD_BATTUTA_CBARITONO:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"Baritone clef");
					break;
				case CMD_BATTUTA_CTENORE:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"Tenor clef");
					break;
				case CMD_BATTUTA_CTENORE8:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"Tenor clef 8 VB");
					break;
				case CMD_BATTUTA_CCONTRALTO:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"Alto clef");
					break;
				case CMD_BATTUTA_CMEZZOSOPRANO:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"Mezzo-soprano clef");
					break;
				case CMD_BATTUTA_CSOPRANO:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"Soprano clef");
					break;
				case CMD_BATTUTA_CPERCUSBOX:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"Percus Box clef");
					break;
				case CMD_BATTUTA_CPERCUS2LINES:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"Percus 2 Lines clef");
					break;
				case CMD_BATTUTA_CTAB:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"Tab clef");
					break;
				case CMD_BATTUTA_CVUOTA:
					TipoComando=ComandoInserimento;
					strcpy(DescrizioneComando,"Empty clef");
					break;
		}
		break;
	case CMD_BATTUTA_INS_RIPTEMPO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Beat repeat");
		break;
	case CMD_BATTUTA_INS_RIPBATTUTA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Measure repeat");
		break;
	case CMD_BATTUTA_INS_RIPMEZZABATTUTA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Half measure repeat");
		break;
		
	case CMD_BATTUTA_CVIOLINO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"G Clef");
		break;
	case CMD_BATTUTA_CVIOLINO8:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"G8vb Clef");
		break;
	case CMD_BATTUTA_CVIOLINO8SOPRA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"G8va Clef");
		break;
	case CMD_BATTUTA_CBASSO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"F Clef");
		break;
	case CMD_BATTUTA_CBASSO8:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"F8vb Clef");
		break;
	case CMD_BATTUTA_CBASSO8SOPRA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"F8va Clef");
		break;
	case CMD_BATTUTA_CBASSOOLD:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Old C Clef");
		break;
	case CMD_BATTUTA_CBARITONO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Baritone clef");
		break;
	case CMD_BATTUTA_CTENORE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Tenor clef");
		break;
	case CMD_BATTUTA_CTENORE8:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Tenor clef 8 VB");
		break;
	case CMD_BATTUTA_CCONTRALTO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Alto clef");
		break;
	case CMD_BATTUTA_CMEZZOSOPRANO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Mezzo-soprano clef");
		break;
	case CMD_BATTUTA_CSOPRANO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Soprano clef");
		break;
	case CMD_BATTUTA_CPERCUSBOX:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Percus Box clef");
		break;
	case CMD_BATTUTA_CPERCUS2LINES:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Percus 2 Lines clef");
		break;
	case CMD_BATTUTA_CTAB:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Tab clef");
		break;
	case CMD_BATTUTA_CVUOTA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Empty clef");
		break;
	case CMD_BATTUTA_ARMCHV_0D:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Key = 0 Sharp");
		break;
	case CMD_BATTUTA_ARMCHV_1D:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Key = 1 Sharp");
		break;
	case CMD_BATTUTA_ARMCHV_2D:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Key = 2 Sharp");
		break;
	case CMD_BATTUTA_ARMCHV_3D:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Key = 3 Sharp");
		break;
	case CMD_BATTUTA_ARMCHV_4D:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Key = 4 Sharp");
		break;
	case CMD_BATTUTA_ARMCHV_5D:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Key = 5 Sharp");
		break;
	case CMD_BATTUTA_ARMCHV_6D:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Key = 6 Sharp");
		break;
	case CMD_BATTUTA_ARMCHV_7D:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Key = 7 Sharp");
		break;
	case CMD_BATTUTA_ARMCHV_1B:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Key = 1 Flat");
		break;
	case CMD_BATTUTA_ARMCHV_2B:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Key = 2 Flat");
		break;
	case CMD_BATTUTA_ARMCHV_3B:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Key = 3 Flat");
		break;
	case CMD_BATTUTA_ARMCHV_4B:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Key = 4 Flat");
		break;
	case CMD_BATTUTA_ARMCHV_5B:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Key = 5 Flat");
		break;
	case CMD_BATTUTA_ARMCHV_6B:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Key = 6 Flat");
		break;
	case CMD_BATTUTA_ARMCHV_7B:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Key = 7 Flat");
		break;
	case CMD_BATTUTA_TEMPO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Time");
		break;
	case CMD_FIGURA_1PUNTO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"1 point");
		break;
	case CMD_FIGURA_2PUNTI:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"2 point");
		break;
	case CMD_FIGURA_3PUNTI:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"3 point");
		break;
	case CMD_FIGURA_CORONA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Fermata");
		break;
	case CMD_FIGURA_CORONAQUAD:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Fermata");
		break;
	case CMD_FIGURA_CORONATRIANG:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Fermata");
		break;
	case CMD_FIGURA_OCCHIALI:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Glasses");
		break;
	case CMD_FIGURA_PPPPPP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = PPPPPP");
		break;
	case CMD_FIGURA_PPPPP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = PPPPP");
		break;
	case CMD_FIGURA_PPPP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = PPPP");
		break;
	case CMD_FIGURA_PPP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = PPP");
		break;
	case CMD_FIGURA_PP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = PP");
		break;
	case CMD_FIGURA_P:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = P");
		break;
	case CMD_FIGURA_FFFFFF:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = FFFFFF");
		break;
	case CMD_FIGURA_FFFFF:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = FFFFF");
		break;
	case CMD_FIGURA_FFFF:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = FFFF");
		break;
	case CMD_FIGURA_FFF:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = FFF");
		break;
	case CMD_FIGURA_FF:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = FF");
		break;
	case CMD_FIGURA_F:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = F");
		break;
	case CMD_FIGURA_MP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = MP");
		break;
	case CMD_FIGURA_MF:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = MF");
		break;
	case CMD_FIGURA_SP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = SP");
		break;
	case CMD_FIGURA_SF:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = SF");
		break;
	case CMD_FIGURA_FZ:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = FZ");
		break;
	case CMD_FIGURA_SFZ:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = SFZ");
		break;
	case CMD_FIGURA_FP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = FP");
		break;
	case CMD_FIGURA_RF:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = RF");
		break;
	case CMD_FIGURA_FFP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = FFP");
		break;
	case CMD_FIGURA_FFPP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = FFPP");
		break;
	case CMD_FIGURA_FPP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = FPP");
		break;
	case CMD_FIGURA_RFZ:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = RFZ");
		break;
	case CMD_FIGURA_SFF:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = SFF");
		break;
	case CMD_FIGURA_SFFF:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = SFFF");
		break;
	case CMD_FIGURA_SFFP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = SFFP");
		break;
	case CMD_FIGURA_SFFZ:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = SFFZ");
		break;
	case CMD_FIGURA_SFP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = SFP");
		break;
	case CMD_FIGURA_SFPP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Dynamic = SFPP");
		break;
	case CMD_FIGURA_TGENERICO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Generic Text");
		break;
	case CMD_FIGURA_ANNOTAZIONE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Mark");
		break;
	case CMD_NOTA_CORDA_0:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"String 0");
		break;
	case CMD_NOTA_CORDA_1:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"String 1");
		break;
	case CMD_NOTA_CORDA_2:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"String 2");
		break;
	case CMD_NOTA_CORDA_3:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"String 3");
		break;
	case CMD_NOTA_CORDA_4:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"String 4");
		break;
	case CMD_NOTA_CORDA_5:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"String 5");
		break;
	case CMD_NOTA_CORDA_6:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"String 6");
		break;
	case CMD_NOTA_CORDA_7:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"String 7");
		break;
	case CMD_NOTA_DIESIS:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Sharp");
		break;
	case CMD_NOTA_BEMOLLE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Flat");
		break;
	case CMD_NOTA_BEQUADRO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Natural");
		break;
	case CMD_NOTA_BEQUADRO1Q:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Natural Up");
		break;
	case CMD_NOTA_BEQUADRO3Q:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Natural Down");
		break;
	case CMD_NOTA_2DIESIS:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"2 Sharp");
		break;
	case CMD_NOTA_2BEMOLLE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"2 Flat");
		break;
	case CMD_NOTA_DIESIS1Q:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"1/4 Sharp");
		break;
	case CMD_NOTA_DIESIS3Q:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"3/4 Sharp");
		break;
	case CMD_NOTA_BEMOLLE1Q:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"1/4 Flat");
		break;
	case CMD_NOTA_BEMOLLE3Q:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"3/4 Flat");
		break;
	case CMD_NOTA_TREMOLO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Tremolo");
		break;
	case CMD_NOTA_TRILLO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Trill");
		break;
	case CMD_NOTA_GRUPPETTO_SUP_1:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Turn 1");
		break;
	case CMD_NOTA_GRUPPETTO_SUP_2:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Turn 2");
		break;
	case CMD_NOTA_TURN_SLASH:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Turn Slash");
		break;
	case CMD_NOTA_TURN_UP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Turn Up");
		break;
	case CMD_NOTA_GRUPPETTO_INF_1:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"BackTurn 1");
		break;
	case CMD_NOTA_GRUPPETTO_INF_2:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"BackTurn 2");
		break;
	case CMD_NOTA_MORDENTE_SUP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Mordent Sup.");
		break;
	case CMD_NOTA_MORDENTE_INF:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Mordent Inf.");
		break;
	case CMD_NOTA_MORDENTE2_SUP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"2 Mordent Sup.");
		break;
	case CMD_NOTA_MORDENTE2_INF:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"2 Mordent Inf.");
		break;
	case CMD_NOTA_SLIDING_UP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Sliding Up");
		break;
	case CMD_NOTA_SLIDING_DOWN:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Sliding Down");
		break;
	case CMD_NOTA_SLIDING_DUP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Sliding DUp");
		break;
	case CMD_NOTA_SLIDING_DOWNUP:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Sliding DownUp");
		break;
	case CMD_NOTA_NONGHYN_INC:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Nonghyn Inc");
		break;
	case CMD_NOTA_NONGHYN_DEC:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Nonghyn Dec");
		break;
	case CMD_NOTA_NONGHYN_ARROW:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Nonghyn Arrow");
		break;
	case CMD_NOTA_NONGHYN_WIDE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Nonghyn Wide");
		break;
	case CMD_NOTA_ARMONICIARCHI:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"String Armonic");
		break;	
	case CMD_NOTA_ARMONICIOTTONI:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Brass Armonic");
		break;
	case CMD_NOTA_NOTA2NOTINA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Little Note");
		break;
	case CMD_NOTA_NOTINA2NOTA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Normal Note");
		break;
	case CMD_NOTA_ACCIACCATURA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Grace Note");
		break;
	case CMD_NOTA_SUDDIVISIONE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Suddivisione");
		break;
	case CMD_NOTA_DITO_0:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Finger 0");
		break;
	case CMD_NOTA_DITO_1:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Finger 1");
		break;
	case CMD_NOTA_DITO_2:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Finger 2");
		break;
	case CMD_NOTA_DITO_3:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Finger 3");
		break;
	case CMD_NOTA_DITO_4:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Finger 4");
		break;
	case CMD_NOTA_DITO_5:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Finger 5");
		break;
	case CMD_NOTA_SFORZATO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Sforzato");
		break;
	case CMD_NOTA_ACCENTO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Accent");
		break;
	case CMD_NOTA_TENUTO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Tenuto");
		break;
	case CMD_NOTA_PUNTOALL:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Punto Allungato");
		break;
	case CMD_NOTA_MARTELLATO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Martellato");
		break;
	case CMD_NOTA_MARTDOLCE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Martellato dolce");
		break;
	case CMD_NOTA_STACCATO: 
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Staccato dot");
		break;
	case CMD_NOTA_ESPGENERIC: 
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Espressione Generica");
		break;
	case CMD_NOTA_VIASORDINAOTTONI:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Togli SordinaOttoni");
		break;
	case CMD_NOTA_VIASORDINAARCHI:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Togli SordinaArchi");
		break;
	case CMD_NOTA_VIASORDINATESTO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Togli SordinaTesto");
		break;
	case CMD_NOTA_CONSORDINAOTTONI:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Metti SordinaOttoni");
		break;
	case CMD_NOTA_CONSORDINAARCHI:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Metti SordinaArchi");
		break;
	case CMD_NOTA_CONSORDINATESTO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Metti SordinaTesto");
		break;
	case CMD_NOTA_ARCO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Arco");
		break;
	case CMD_NOTA_PIZZICATO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Pizzicato");
		break;
	case CMD_PAUSA_TIMPANO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Timpano");
		break;
	case CMD_SPAZIO_PRFIATO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Prendi fiato");
		break;
	case CMD_ACCORDO_ARPEGGIO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Arpeggio");
		break;
	case CMD_ARCO_PUNTA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Arco punta");
		break;
	case CMD_ARCO_TALLONE:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Arco tallone");
		break;
	case CMD_ARCO_TASTIERA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Arco tastiera");
		break;
	case CMD_ARCO_PONTICELLO:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Arco ponticello");
		break;
	case CMD_ARCO_ARCOSU:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Arco su");
		break;
	case CMD_ARCO_ARCOGIU:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Arco giu");
		break;
	case CMD_ARCO_CORDA:
		TipoComando=ComandoInserimento;
		strcpy(DescrizioneComando,"Arco corona");
		break;
	default:
		TipoComando=ComandoNonConosciuto;
		strcpy(DescrizioneComando,"Sconosciuto");
		break;
 };
}



// Questa funzione prepara una stringa che descrive il comando
// La stringa sara' del tipo di quelle visualizzate nella list-box della finestra

void LogListEntry::MakeString(char *Buffer)
{
	// preparazione stringa con sprintf()
	char Appo[100];
	char Appo1[100];
	
	Buffer[0]=' ';
	if (MustBeConvalidated) {
		Buffer[1]='X';
		Buffer[2]='-';
	}
	else {
		Buffer[1]='-';
		Buffer[2]='O';
	}
	
	if (FromDirector) {
		Buffer[3]='D';
		Buffer[4]='-';
	}
	else {
		Buffer[3]='-';
		Buffer[4]='L';
	}
	
	switch (TipoComando) {
	case ComandoInserimento: 
		Buffer[5]='I'; 
		Buffer[6]='-';
		Buffer[7]='-';
		break;
	case ComandoAzione: 
		Buffer[5]='-'; 
		Buffer[6]='A';
		Buffer[7]='-';
		break;
	case ComandoCancellazione: 
		Buffer[5]='-'; 
		Buffer[6]='-';
		Buffer[7]='C';
		break;			
	default : 
		Buffer[5]='-'; 
		Buffer[6]='-';
		Buffer[7]='-';
		break;			
	}
	Buffer[8]=0;
	
    
	if (TID!=0) sprintf(Appo," %6d",TID);
	else strcpy(Appo,"  ???  ");
	strcat(Buffer,Appo);
	sprintf(Appo," %-20.20s - %-20.20s - %-11.11s | ",NomeHost,NomePlayer,NomeSpartito);
	strcat(Buffer,Appo);
	sprintf(Appo," (%3d) ",MyCmd.CommandId);
	strcat(Buffer,Appo);
	
	switch (TipoComando) {
	case ComandoInserimento:   strcpy(Appo," Insertion ");
		break;
	case ComandoCancellazione: strcpy(Appo," Delete ");
		break;
	case ComandoAzione:	     strcpy(Appo," Action ");
		break;
	default:                   strcpy(Appo," Unknown ");
		break;
	};
	
	strcat(Appo,DescrizioneComando); // In Appo  pronta la stringa
	sprintf(Appo1,"%-30.30s",Appo);   // In Appo1  stata formattata
	strcat(Buffer,Appo1);            // In Buffer  stata copiata
	
	if (nbat==-1) strcat(Buffer," B/F: -/");
	else {
		sprintf(Appo," B/F: %d/",nbat);
		strcat(Buffer,Appo);     
	}
	
	
	if (nfig==-1) strcat(Buffer,"-");
	else {
		sprintf(Appo,"%d",nfig);
		strcat(Buffer,Appo);     
	}
	
	if (nbat1==-1) strcat(Buffer," B/F: -/");
	else {
		sprintf(Appo,"     B/F: %d/",nbat1);
		strcat(Buffer,Appo);     
	}
	
	
	if (nfig1==-1) strcat(Buffer,"-");
	else {
		sprintf(Appo,"%d",nfig1);
		
		strcat(Buffer,Appo);     
	}
	
	
	return;     
};




// Prepara una stringa che descrive il comando simbolico
// La stringa sara' del tipo di quella visualizzata nel controllo di edit nella finestra

void LogListEntry::MakeDump(char *Buffer)   // crea una stringa per il dump di MyCmd
{
	sprintf(Buffer," CommandId : %d\n SelectionType : %d\n SelObjType : %d\n",MyCmd.CommandId,MyCmd.SelectionType,MyCmd.SelObjType);
	char Appo[255];
	
	// Primo Path
	sprintf(Appo," Path 1 : \n  - Level : %d\n  - Array :",MyCmd.Path1.Level);
	strcat(Buffer,Appo);
	int i;
	for (i=0;i<PATH_LENGTH;i++) {
		sprintf(Appo," %d",MyCmd.Path1.Array[i]);
		strcat(Buffer,Appo);
	}
	strcat(Buffer,"\n");
	
	// Secondo Path 
	sprintf(Appo," Path 2 : \n  - Level : %d\n  - Array :",MyCmd.Path2.Level);
	strcat(Buffer,Appo);
	
	for (i=0;i<PATH_LENGTH;i++) {
		sprintf(Appo," %d",MyCmd.Path2.Array[i]);
		strcat(Buffer,Appo);
	}
	strcat(Buffer,"\n ArgV :");
	
	for (i=0;i<SYM_MAXARG;i++) {
		sprintf(Appo," %d",MyCmd.ArgV[i]);
		strcat(Buffer,Appo);
	}
	
	sprintf(Appo,"\n Text : %s",MyCmd.Text);
	strcat(Buffer,Appo);
	
	return;
	
}

// Questa funzione Serializza (ovvero scrive/legge su/da uno stream) l'oggetto LogListEntry
// se Store == True scrive, altrimenti legge.

void LogListEntry::Serialize(Bool Store,FILE *fp)
{
	if (Store) { 
		
		if (NomeHost[0]=='*') SaveString(NomeHost+1,fp);
		else SaveString(NomeHost,fp);
		if (NomePlayer[0]=='*') SaveString(NomePlayer+1,fp);
		else SaveString(NomePlayer,fp);
		if (NomeSpartito[0]=='*') SaveString(NomeSpartito+1,fp);
		else SaveString(NomeSpartito,fp);
		// SaveString(DescrizioneComando,fp);
		
		fwrite(&TID,sizeof(TID),1,fp);
		fwrite(&nbat,sizeof(nbat),1,fp);
		fwrite(&nfig,sizeof(nfig),1,fp);
		fwrite(&nbat1,sizeof(nbat1),1,fp);
		fwrite(&nfig1,sizeof(nfig1),1,fp);
		
		fwrite(&FromDirector,sizeof(FromDirector),1,fp);
		fwrite(&MustBeConvalidated,sizeof(MustBeConvalidated),1,fp);
		
		SalvaCommandStruct(&MyCmd,fp);
		
		// Ricavo i campi Descrizione comando e TipoComando
		
		
		
	} else {
		
		NomeHost[0]='*';
		NomePlayer[0]='*';
		NomeSpartito[0]='*';
		
		LoadString(NomeHost+1,fp);
		
		LoadString(NomePlayer+1,fp);
		LoadString(NomeSpartito+1,fp);
		// LoadString(DescrizioneComando,fp);
        
		fread(&TID,sizeof(TID),1,fp);
		TID=0; // potrei anche non salvarlo !
		fread(&nbat,sizeof(nbat),1,fp);
		fread(&nfig,sizeof(nfig),1,fp);
		fread(&nbat1,sizeof(nbat1),1,fp);
		fread(&nfig1,sizeof(nfig1),1,fp);
		
		fread(&FromDirector,sizeof(FromDirector),1,fp);
		
		fread(&MustBeConvalidated,sizeof(MustBeConvalidated),1,fp);
		
		CaricaCommandStruct(&MyCmd,fp);
		
		// Ricavo i campi Descrizione comando e TipoComando
		
		PreparaComando();
	}
}

// Questa funzione Serializza un elemento classificandolo "Flushed"

void LogListEntry::Flush(FILE *fp)
{
	// devo salvare l'elemento in fp marchiato come "Flushed"
	fwrite(FLUSH_TAG,strlen(FLUSH_TAG),1,fp);
	Serialize(True,fp);
};

// Questa funzione Serializza un elemento classificandolo "Stored"

void LogListEntry::Salva(FILE *fp)
{
	// devo salvare l'elemento in fp marchiato come "non Flushed"
	
	fwrite(STORE_TAG,strlen(STORE_TAG),1,fp);
	
	Serialize(True,fp);
	
	
};

// Questa funzione ricarica un elemento da uno stream
// ritorna True se l'elemento era stato salvato come "Flushed", altrimenti ritorna False.
Bool LogListEntry::Carica(FILE *fp)
{
	char *TAG;
	TAG=new char[strlen(STORE_TAG)+1];
	fread(TAG,strlen(STORE_TAG),1,fp);
	TAG[strlen(STORE_TAG)]=0;
	
	if (strcmp(TAG,STORE_TAG)==0) {
		// E' un elemento Store (non "Flushed")
		
		Serialize(False,fp);
		delete TAG;
		return False;
	} else {
		// E' un elemento "Flushed"
		
		Serialize(False,fp);
		delete TAG;
		return True;
	};
};


void LogListEntry::Execute(void)
{ 
	pMusicWin->PPartit->SetNetSymCmd(MyCmd);
	pMusicWin->PPartit->DoNetSymCmd();
	
	return;
}


/*----------------------------------------------------------------------------------------------*/
/*                                        Classe LogFile                                        */
/*----------------------------------------------------------------------------------------------*/



LogFile::LogFile(void) {
	// Cancellazione precedente file di log
	if (DUMP_FILE) {
		FILE *fp;
		fp = fopen(NomeFileDebug,"w");
		if (fp!=NULL) {
			// Scrittura Header del file di log
			fprintf(fp,"                    Progetto MOODS\n");
			fprintf(fp,"          LogFile di comandi in arrivo al MASAE.\n");
			fprintf(fp,"    -------------------------------------------------\n\n");
			fclose(fp);
		}
	}
	// Preparazione della lista
	HPList=NULL;
	LastEntry=NULL;
	active=FALSE;
	ArrayStringhe=NULL;
	CurrentSelected=-1;
	
	CurrentFilter.ResetFilter();
		  
	return;
};

LogFile::~LogFile(void) 
{
	EliminaLista();
	return;
};

// Determina, dato un certo ID di comando, se si tratta di inserimento di una figura.

Bool LogFile::IsFigureInsertion(CmdID CommandId) 
{
	switch(CommandId)
	{
    case CMD_BATTUTA_INS_NBREVE:
    case CMD_BATTUTA_INS_NSEMIBREVE:
    case CMD_BATTUTA_INS_NMINIMA:
    case CMD_BATTUTA_INS_NSEMIMINIMA:
    case CMD_BATTUTA_INS_NCROMA:
    case CMD_BATTUTA_INS_NSEMICROMA:
    case CMD_BATTUTA_INS_NBISCROMA:
    case CMD_BATTUTA_INS_NSEMIBISCROMA:
    case CMD_BATTUTA_INS_NFUSA:
    case CMD_BATTUTA_INS_PQUABATT:
    case CMD_BATTUTA_INS_PDUEBATT:
    case CMD_BATTUTA_INS_PSEMIBREVE:
    case CMD_BATTUTA_INS_PMINIMA:
    case CMD_BATTUTA_INS_PSEMIMINIMA:
    case CMD_BATTUTA_INS_PCROMA:
    case CMD_BATTUTA_INS_PSEMICROMA:
    case CMD_BATTUTA_INS_PBISCROMA:
    case CMD_BATTUTA_INS_PSEMIBISCROMA:
    case CMD_BATTUTA_INS_PFUSA:
    case CMD_BATTUTA_INS_PGENERICA:
    case CMD_BATTUTA_INS_SPAZIO:
    case CMD_BATTUTA_INS_RIPTEMPO:
    case CMD_BATTUTA_INS_RIPBATTUTA:
    case CMD_BATTUTA_INS_RIPMEZZABATTUTA:
		return TRUE;
		break;
    default: return FALSE; break;
	}
}

Bool LogFile::IsSpartitoCommand(CmdID CommandId) 
{
	switch(CommandId)
	{
	case CMD_SPARTITO_ADD_LEGATURA:
	case CMD_SPARTITO_ADD_LEGATURA_VALORE:
	case CMD_SPARTITO_ADD_FORC_CRESC:
	case CMD_SPARTITO_ADD_FORC_DIMIN:
	case CMD_SPARTITO_ADD_FRECCIA:
	case CMD_SPARTITO_ADD_ONDA:
	case CMD_SPARTITO_ADD_LEG_QUADRA:
	case CMD_SPARTITO_ADD_CAMBIO_RIT:
	case CMD_SPARTITO_ADD_MODIF8:
	case CMD_SPARTITO_ADD_MODIF15MA:
	case CMD_SPARTITO_ADD_MODIF8BA:
	case CMD_SPARTITO_ADD_MODIF15BA:
	case CMD_SPARTITO_NPENT:
	case CMD_SPARTITO_SET_NAME: return TRUE;
	default: return FALSE;
	}
	return FALSE;
} 


Bool LogFile::IsAggiunta(CmdID CommandId) 
{
	switch(CommandId)
	{
	case CMD_FIGURA_1PUNTO:
	case CMD_FIGURA_2PUNTI:
	case CMD_FIGURA_3PUNTI:
	case CMD_FIGURA_CORONA:
	case CMD_FIGURA_CORONAQUAD:
	case CMD_FIGURA_CORONATRIANG:
	case CMD_FIGURA_OCCHIALI:
	case CMD_FIGURA_PPPPPP:
	case CMD_FIGURA_PPPPP:
	case CMD_FIGURA_PPPP:
	case CMD_FIGURA_PPP:
	case CMD_FIGURA_PP:
	case CMD_FIGURA_P:
	case CMD_FIGURA_FFFFFF:
	case CMD_FIGURA_FFFFF:
	case CMD_FIGURA_FFFF:
	case CMD_FIGURA_FFF:
	case CMD_FIGURA_FF:
	case CMD_FIGURA_F:
	case CMD_FIGURA_MP:
	case CMD_FIGURA_MF:
	case CMD_FIGURA_SP:
	case CMD_FIGURA_SF:
	case CMD_FIGURA_FZ:
	case CMD_FIGURA_SFZ:
	case CMD_FIGURA_FP:
	case CMD_FIGURA_RF:
	case CMD_FIGURA_FFP:
	case CMD_FIGURA_FFPP:
	case CMD_FIGURA_FPP:
	case CMD_FIGURA_RFZ:
	case CMD_FIGURA_SFF:
	case CMD_FIGURA_SFFF:
	case CMD_FIGURA_SFFP:
	case CMD_FIGURA_SFFZ:
	case CMD_FIGURA_SFP:
	case CMD_FIGURA_SFPP:
	case CMD_FIGURA_TGENERICO:
	case CMD_FIGURA_ANNOTAZIONE:
		
	case CMD_NOTA_CORDA_0:
	case CMD_NOTA_CORDA_1:
	case CMD_NOTA_CORDA_2:
	case CMD_NOTA_CORDA_3:
	case CMD_NOTA_CORDA_4:
	case CMD_NOTA_CORDA_5:
	case CMD_NOTA_CORDA_6:
	case CMD_NOTA_CORDA_7:
	case CMD_NOTA_DIESIS: 
	case CMD_NOTA_DIESIS1Q:
	case CMD_NOTA_DIESIS3Q:
	case CMD_NOTA_BEMOLLE: 
	case CMD_NOTA_BEMOLLE1Q:
	case CMD_NOTA_BEMOLLE3Q:
	case CMD_NOTA_BEQUADRO: 
	case CMD_NOTA_2DIESIS:
	case CMD_NOTA_2BEMOLLE:
	case CMD_NOTA_BEQUADRO1Q: 
	case CMD_NOTA_BEQUADRO3Q: 
		
	case CMD_NOTA_TREMOLO:
	case CMD_NOTA_TRILLO:
	case CMD_NOTA_GRUPPETTO_SUP_1:
	case CMD_NOTA_GRUPPETTO_SUP_2:
    case CMD_NOTA_TURN_SLASH:
	case CMD_NOTA_TURN_UP:
	case CMD_NOTA_GRUPPETTO_INF_1:
	case CMD_NOTA_GRUPPETTO_INF_2:
	case CMD_NOTA_MORDENTE_SUP:
	case CMD_NOTA_MORDENTE_INF:
	case CMD_NOTA_MORDENTE2_SUP:
	case CMD_NOTA_MORDENTE2_INF:
	case CMD_NOTA_NOTA2NOTINA:
	case CMD_NOTA_NOTINA2NOTA:
	case CMD_NOTA_SLIDING_UP:
	case CMD_NOTA_SLIDING_DOWN:
	case CMD_NOTA_SLIDING_DUP:
	case CMD_NOTA_SLIDING_DOWNUP:
	case CMD_NOTA_NONGHYN_INC:
	case CMD_NOTA_NONGHYN_DEC:
	case CMD_NOTA_NONGHYN_ARROW:
	case CMD_NOTA_NONGHYN_WIDE:
	case CMD_NOTA_ACCIACCATURA:
	case CMD_NOTA_SUDDIVISIONE:
	case CMD_NOTA_DITO_0:
	case CMD_NOTA_DITO_1:
	case CMD_NOTA_DITO_2:
	case CMD_NOTA_DITO_3:
	case CMD_NOTA_DITO_4:
	case CMD_NOTA_DITO_5:
	case CMD_NOTA_TENUTO:
	case CMD_NOTA_PUNTOALL:
	case CMD_NOTA_SFORZATO:
	case CMD_NOTA_ACCENTO:
	case CMD_NOTA_MARTELLATO:
	case CMD_NOTA_MARTDOLCE:
	case CMD_NOTA_STACCATO:
	case CMD_NOTA_ESPGENERIC: 
	case CMD_NOTA_CONSORDINAOTTONI:
	case CMD_NOTA_CONSORDINAARCHI:
	case CMD_NOTA_VIASORDINAOTTONI:
	case CMD_NOTA_VIASORDINAARCHI:
	case CMD_NOTA_VIASORDINATESTO: 
	case CMD_NOTA_CONSORDINATESTO:
	case CMD_NOTA_ARMONICIARCHI:
    case CMD_NOTA_ARMONICIOTTONI:
		
	case CMD_NOTA_ARCO:
	case CMD_NOTA_PIZZICATO:
	case CMD_PAUSA_TIMPANO:
	case CMD_SPAZIO_PRFIATO:
	case CMD_ACCORDO_ARPEGGIO:
		
		
	case CMD_ARCO_PUNTA:
	case CMD_ARCO_TALLONE:
	case CMD_ARCO_TASTIERA:
	case CMD_ARCO_PONTICELLO:
	case CMD_ARCO_ARCOSU:
	case CMD_ARCO_ARCOGIU:
	case CMD_ARCO_CORDA:
		
		
		
		return TRUE;
		break;
    default: return FALSE; break;
   }
}

Bool LogFile::IsBattutaAggiunta(CmdID CommandId) 
{
	switch(CommandId)
	{
	case CMD_BATTUTA_CVIOLINO:
	case CMD_BATTUTA_CVIOLINO8:
	case CMD_BATTUTA_CVIOLINO8SOPRA:
	case CMD_BATTUTA_CBASSO:
	case CMD_BATTUTA_CBASSO8:
	case CMD_BATTUTA_CBASSO8SOPRA:
	case CMD_BATTUTA_CBASSOOLD:
	case CMD_BATTUTA_CBARITONO:
	case CMD_BATTUTA_CTENORE:
	case CMD_BATTUTA_CTENORE8:
	case CMD_BATTUTA_CCONTRALTO:
	case CMD_BATTUTA_CMEZZOSOPRANO:
	case CMD_BATTUTA_CSOPRANO:
	case CMD_BATTUTA_CPERCUSBOX:
	case CMD_BATTUTA_CPERCUS2LINES:
	case CMD_BATTUTA_CTAB:
	case CMD_BATTUTA_CVUOTA:
		
	case CMD_BATTUTA_ARMCHV_0D:
	case CMD_BATTUTA_ARMCHV_1D:
	case CMD_BATTUTA_ARMCHV_2D:
	case CMD_BATTUTA_ARMCHV_3D:
	case CMD_BATTUTA_ARMCHV_4D:
	case CMD_BATTUTA_ARMCHV_5D:
	case CMD_BATTUTA_ARMCHV_6D:
	case CMD_BATTUTA_ARMCHV_7D:
	case CMD_BATTUTA_ARMCHV_1B:
	case CMD_BATTUTA_ARMCHV_2B:
	case CMD_BATTUTA_ARMCHV_3B:
	case CMD_BATTUTA_ARMCHV_4B:
	case CMD_BATTUTA_ARMCHV_5B:
	case CMD_BATTUTA_ARMCHV_6B:
	case CMD_BATTUTA_ARMCHV_7B:
		
	case CMD_BATTUTA_ARMCHV_0Dm:
	case CMD_BATTUTA_ARMCHV_1Dm:
	case CMD_BATTUTA_ARMCHV_2Dm:
	case CMD_BATTUTA_ARMCHV_3Dm:
	case CMD_BATTUTA_ARMCHV_4Dm:
	case CMD_BATTUTA_ARMCHV_5Dm:
	case CMD_BATTUTA_ARMCHV_6Dm:
	case CMD_BATTUTA_ARMCHV_7Dm:
	case CMD_BATTUTA_ARMCHV_1Bm:
	case CMD_BATTUTA_ARMCHV_2Bm:
	case CMD_BATTUTA_ARMCHV_3Bm:
	case CMD_BATTUTA_ARMCHV_4Bm:
	case CMD_BATTUTA_ARMCHV_5Bm:
	case CMD_BATTUTA_ARMCHV_6Bm:
	case CMD_BATTUTA_ARMCHV_7Bm:
		//
	case CMD_BATTUTA_TEMPO:
	case CMD_BATTUTA_BSINGOLA: 
	case CMD_BATTUTA_BSINGOLA_RIP2B: 
	case CMD_BATTUTA_BDOPPIA: 
	case CMD_BATTUTA_BFINALE:
	case CMD_BATTUTA_BINIZIORIT:
	case CMD_BATTUTA_BFINERIT:
	case CMD_BATTUTA_BINIZIOFINERIT: 
	case CMD_BATTUTA_BINVISIBILE: 
	case CMD_BATTUTA_INS_RIPTEMPO:
	case CMD_BATTUTA_INS_RIPBATTUTA:
	case CMD_BATTUTA_INS_RIPMEZZABATTUTA: 
	case CMD_BATTUTA_NOTE2GRUPPONOTE:
	case CMD_BATTUTA_GRUPPONOTE2NOTE:
	case CMD_BATTUTA_NUGUALE:
	case CMD_BATTUTA_SCANSIONE:		
    case CMD_BATTUTA_TSALTO:			
	case CMD_BATTUTA_RIFERIMENTO:			
		//
	case CMD_BATTUTA_LETTERA:
	case CMD_BATTUTA_MOVIMENTO:		
		return TRUE;
		break;
    default: return FALSE; break;
	}
	return FALSE;
}



// Elimina la lista di LogListEntry

void LogFile::EliminaLista(void)
{
	LogListEntry *Runner;
	
	while (HPList!=NULL) {
		Runner=HPList;
		HPList=HPList->GetNext();
		delete Runner;
	}
	LastEntry=NULL;
	
	if (active) ModifyWindow();
	return;
}

// Aggiunge l'LogListEntry passatogli in fondo alla lista.
// L'oggetto LogListEntry deve essere allocato esternamente alla funzione

void LogFile::AddCommand(LogListEntry *Elemento)
{
	// L'elemento deve essere allocato esternamente alla classe !
	
	// Inserimento in coda alla lista
	if (LastEntry!=NULL) 
	{
		LastEntry->SetNext(Elemento);
		Elemento->SetPrev(LastEntry);
	}
	else 
	{
		HPList=Elemento;
		Elemento->SetPrev(NULL);
	}
	
	LastEntry=Elemento;
	if (DUMP_FILE) DumpOnFile(Elemento);
}

// Ritorna il numero di LogListEntry attualmente presenti nella lista

int LogFile::GetNumCommand(void)
{
	LogListEntry *Runner;
	int  Num=0;
	Runner=HPList;
	
	while (Runner!=NULL) { 	
		Num++;
		Runner=Runner->GetNext();			
	}
	return Num;
};

int LogFile::GetNumFilteredCommand(void)
{
	LogListEntry *Runner;
	int  Num=0;
	Runner=HPList;
	
	while (Runner!=NULL) { 	
		if (Runner->IsInFilter(&CurrentFilter)) Num++;
		Runner=Runner->GetNext();			
	}
	return Num;
};



// Aggiunge un LogListEntry creato in base ai parametri passatigli

void LogFile::AddCommand(int TID,char *NomeHost,char *NomePlayer,char *NomeSpartito, SymCmd *CmdStrt,
						 signed int b1,signed int f1,signed int b2,signed int f2,                        
						 Bool IsDir, Bool MustVer)
						 
{
	
	LogListEntry *NewEntry;
	
	NewEntry = new LogListEntry(TID,NomeHost,NomePlayer,NomeSpartito,CmdStrt,
		b1,f1,b2,f2,IsDir,MustVer);
	
	
	AddCommand(NewEntry);
	
	if (active) 
		ModifyWindow(True);
	
	return;
};



// Modifica la finestra di log 

void LogFile::ModifyWindow(Bool ManteinSelection)
{ 
	int N=GetNumFilteredCommand();
	char **ArrayStringheTemp;
	
	ArrayStringheTemp=new char*[N+1]; 
	int i;
	for (i=0;i<N;i++) {
		ArrayStringheTemp[i]=new char[MAX_STRINGH_LENGTH];
		PrepareCommandString(i,ArrayStringheTemp[i],TRUE);  // TRUE perch devo filtrare l'insieme   
	}
	
	ArrayStringheTemp[N]=NULL;
	
	
	if (!ManteinSelection) WUpdateCommandList(ArrayStringheTemp);
	else WUpdateCommandList(ArrayStringheTemp,CurrentSelected);
	
	// cancello la lista precedente
	if (ArrayStringhe!=NULL) {
		i=0;
		while (ArrayStringhe[i]!=NULL) {
			delete ArrayStringhe[i];
			ArrayStringhe[i]=NULL;
			i++;
		}
		delete[] ArrayStringhe;
	}
	
	ArrayStringhe=ArrayStringheTemp;
	return;
}


// Crea, apre e visualizza la finestra di log

Bool LogFile::OpenWindow(char *title)
{ 
	active=TRUE;
	int N=GetNumFilteredCommand();
	
	ArrayStringhe=new char*[N+1]; 
	int i;
	for (i=0;i<N;i++) {
		ArrayStringhe[i]=new char[MAX_STRINGH_LENGTH];
		PrepareCommandString(i,ArrayStringhe[i],TRUE);    // TRUE perch devo filtrare l'insieme
	}
	
	ArrayStringhe[N]=NULL;
	WCommandList(title,ArrayStringhe);  
	return FALSE;
}


// Prepara una riga di testo della scroll list nella finestra principale.

void LogFile::PrepareCommandString(int Quale,char *Stringa,Bool Filter)
{
	LogListEntry * Runner;
	Runner=GetEntry(Quale,Filter);
	
	if (Runner!=NULL) 
		Runner->MakeString(Stringa);
	else strcpy(Stringa,"No Entry");
	
	return;
}

// Chiude e distrugge la finestra

void LogFile::CloseWindow(void) 
{
	active=FALSE;  
	WCloseCommandList();
	int i=0;
	
	CurrentSelected=-1;
	return;
}


// Apre / Chiude la finestra

void LogFile::Switch(void) 
{
	if (active) {CloseWindow();}
	else OpenWindow("Additional Command Window");
}

// Ritorna l'elemento da una certa posizione nella lista

LogListEntry * LogFile::GetEntry(int index,Bool Filter)
{
	LogListEntry * Runner;
	Runner=HPList;
	index+=1;
	
	while ((Runner!=NULL) && (index>0))
	{
		if (Filter) {
			if (Runner->IsInFilter(&CurrentFilter)) index--;
		}
		else index--;
		if (index>0) Runner=Runner->GetNext();	
	}
	return Runner;
};


void LogFile::SetCurrentSelected(int index)
{
	CurrentSelected=index;
	return;
}

int LogFile::GetCurrentSelected(void)
{
	return CurrentSelected;
}


// prepara la stringa di Dump del SymCmd

void LogFile::MakeDump(int index,char *Buffer)
{
	LogListEntry * Runner;
	Runner=GetEntry(index);
	if (Runner!=NULL) Runner->MakeDump(Buffer);
	else strcpy(Buffer,"NULL");
	
	return;
}


// Convalida un dato elemento della lista

void LogFile::Convalida(int index)
{
	// se  il valore -1 (non  stato passato) prendo quello correntemente selezionato
	
	Battuta *pBat;
	DrawObject *pObj;
	
	if (index==-1) index=CurrentSelected;
	
	// se ancora non  selezionato nessuno esco
	if (index==-1) return;
	// Vado all'elemento dalla lista
	LogListEntry * Runner;
	Runner=GetEntry(index,TRUE);   // Applico il filtro
	if (Runner!=NULL) 
	{
		Runner->Convalida();
		
		Bool InPage;
		if (CatchFigure(&(Runner->MyCmd),(Battuta *&)pBat,(DrawObject *&)pObj)) {
			if (pObj!=NULL) {
				int col=GetColor(&(Runner->MyCmd),Runner->TID,
					Runner->FromDirector,Runner->MustBeConvalidated);
				pObj->SetColor((ColorIcon)col);		
				//  Verifico se e' nella pagina corrente !
				Spartito *pSp = (Spartito *)pMusicWin->PPartit->GetFirstSpa();
				if (pSp!=NULL) {
					Battuta *pBat1=pSp->GetfirstBatPage();
					Battuta *pBat2=pSp->GetfirstBatNextPage();
					if ((pBat1==NULL) || (pBat2==NULL)) InPage=True;
					else 
						if ((pBat->GetNumProgress()>=pBat1->GetNumProgress()) && (pBat->GetNumProgress()<pBat2->GetNumProgress())) InPage=True;
						else InPage=False;
				} else InPage=False;
				
				if (InPage) pObj->Draw();  						
			}
		}
		
		
		
		if (active)   // Se la finestra  attiva la ridisegno !
			ModifyWindow(TRUE); 
		
	}
	
	return;
}

// Invalida un dato elemento della lista

void LogFile::Invalida(int index)
{
	// se  il valore -1 (non  stato passato) prendo quello correntemente selezionato
	
	Battuta *pBat;
	DrawObject *pObj;
	
	if (index==-1) index=CurrentSelected;
	// se ancora non  selezionato nessuno esco
	if (index==-1) return;
	// Vado all'elemento dalla lista
	LogListEntry * Runner;
	Runner=GetEntry(index,TRUE);  // Applico il filtro
	if (Runner!=NULL) 
	{
		Runner->Invalida();
		
		Bool InPage;
		if (CatchFigure(&(Runner->MyCmd),(Battuta *&)pBat,(DrawObject *&)pObj)) {
			if (pObj!=NULL) {
				int col=GetColor(&(Runner->MyCmd),Runner->TID,
					Runner->FromDirector,Runner->MustBeConvalidated);
				pObj->SetColor((ColorIcon)col);		
				// Lo disegno solo se la battuta e' visualizzata !
				Spartito *pSp = (Spartito *)pMusicWin->PPartit->GetFirstSpa();
				if (pSp!=NULL) {
					Battuta *pBat1=pSp->GetfirstBatPage();
					Battuta *pBat2=pSp->GetfirstBatNextPage();
					if ((pBat1==NULL) || (pBat2==NULL)) InPage=True;
					else 
						if ((pBat->GetNumProgress()>=pBat1->GetNumProgress()) && (pBat->GetNumProgress()<pBat2->GetNumProgress())) InPage=True;
						else InPage=False;
				} else InPage=False;
				
				if (InPage) pObj->Draw();
			}
		}
		
		if (active)   // Se la finestra  attiva la ridisegno !
			ModifyWindow(TRUE);
	}
	
	return;
}

void LogFile::ConvalidaAll(void)
{ 
	Battuta *pBat;
	DrawObject *pObj;
	
	
	LogListEntry * Runner;
	Runner=HPList;
	int col;
	Bool Fatto=FALSE;
	Bool InPage;
	
	while (Runner!=NULL) 
	{
		if ((Runner->MustBeConvalidated) && (Runner->IsInFilter(&CurrentFilter))) {
			Runner->Convalida();
			
			
			if (CatchFigure(&(Runner->MyCmd),(Battuta *&)pBat,(DrawObject *&)pObj)) {
				if (pObj!=NULL) {
					col=GetColor(&(Runner->MyCmd),Runner->TID,
						Runner->FromDirector,Runner->MustBeConvalidated);
					pObj->SetColor((ColorIcon)col);		
					
					// Verifico se e' nella pagina corrente !
					Spartito *pSp = (Spartito *)pMusicWin->PPartit->GetFirstSpa();
					if (pSp!=NULL) {
						Battuta *pBat1=pSp->GetfirstBatPage();
						Battuta *pBat2=pSp->GetfirstBatNextPage();
						if ((pBat1==NULL) || (pBat2==NULL)) InPage=True;
						else 
							if ((pBat->GetNumProgress()>=pBat1->GetNumProgress()) && (pBat->GetNumProgress()<pBat2->GetNumProgress())) InPage=True;
							else InPage=False;
					} else InPage=False;
					
					if (InPage) pObj->Draw();
				}
			}
			Fatto=TRUE;
		}
		Runner=Runner->GetNext();
	}
	
	if ((active)  && (Fatto)) // Se la finestra  attiva e ho modificato almeno un elemento la ridisegno !
		ModifyWindow(TRUE); 
	
	return;
	
}


Bool LogFile::CatchFigure(SymCmd * MyCmd,Battuta *& pBat,DrawObject *& pObj)
{ 
	
	if (!pMusicWin->SymbolicHit(MyCmd->Path1,BATTUTA_SEL,(DrawObject *&)pBat)) {
		pObj=NULL;
		return FALSE;  
	}
	if (pBat==NULL) {				
		pObj=NULL;
		return TRUE;
	}
	// ho trovato la battuta 
	// Verifico alcuni comandi particolari di partitura
	if (MyCmd->CommandId==CMD_PARTITURA_ADD_BATTUTA) {
		pObj=NULL;
		DrawObject *pd;
		if (!pMusicWin->SymbolicHit(MyCmd->Path1,SPARTITO_SEL,pd)) 
			return FALSE;
		
		if(pd==NULL) 
			return TRUE;
		
		
		Spartito *pSp;
		pSp = (Spartito*)pd; 
		Battuta *Appo=pSp->GetBatt(MyCmd->ArgV[6]);
		if (Appo!=NULL) pBat=(Battuta *&)Appo;
		return TRUE;
	}
	
	Figura *pAccordo,*pFig;
	
	if (IsFigureInsertion(MyCmd->CommandId)) {
		pAccordo=(Figura *)(pBat->GetFigAll(MyCmd->ArgV[1],MyCmd->ArgV[6]));
		
		if (pAccordo==NULL) {
			
			pObj=NULL;
			return FALSE;
		}
		
		pObj=(DrawObject *&)pAccordo;
		return TRUE;
	} else {
		// Non siamo nel caso di inserimento di una figura
		// L'informazione sull'oggetto colpito risiede nel path ....
		
		if (!pMusicWin->SymbolicHit(MyCmd->Path1,FIGURA_SEL,(DrawObject *&)pFig)) 
			// Non trovo la figura ma solo la battuta
			pFig=NULL;	
	}
	// Controllo che non sia un inserimento come aggiunta alla nota
	// pFig Sarebbe la figura in cui e' stato inserito
	
	if (IsAggiunta(MyCmd->CommandId)) {
		if (pFig==NULL) {
			// posso non averla trovata perche' e' stata inserita su un abbellimento
			if (!pMusicWin->SymbolicHit(MyCmd->Path1,ANY_SEL,(DrawObject *&)pFig)) {
				pObj=NULL;
				return FALSE;  
			}
			if (pFig==NULL) {				
				pObj=NULL;
				return TRUE;
			}
			pObj=((Abbellimento *)(pFig))->GetAlterazione(MyCmd);
			return TRUE;		  
		} 
		
		
		DrawObject *Appo=pFig->GetAggiunta(MyCmd);
		if (Appo!=NULL) pObj=(DrawObject *&)Appo;    
		else pObj=(DrawObject *&)pFig;
		
		
		
		return TRUE;
	}
	if (IsSpartitoCommand(MyCmd->CommandId)) {
		
		DrawObject *pd;
		if (!pMusicWin->SymbolicHit(MyCmd->Path1,SPARTITO_SEL,pd)) {
			pObj=NULL;
			return FALSE;
		}
		if(pd==NULL) {
			pObj=NULL;
			return TRUE;
		}
		
		
		Spartito *pSp;
		pSp = (Spartito*)pd;
		// pSp e' lo spartito
		DrawObject * Appo=pSp->GetObject(MyCmd);
		
		if (Appo!=NULL) pObj=Appo;
		else pObj=NULL;
		return TRUE;
		
	}
	if (IsBattutaAggiunta(MyCmd->CommandId)) {
		
		if (pBat==NULL) {
			pObj=NULL;
			return TRUE;
		}
		DrawObject *Appo=pBat->GetObject(MyCmd,0); 
		
		if (Appo!=NULL) pObj=(DrawObject *&)Appo;    
		else pObj=(DrawObject *&)pFig;
		
		return TRUE;	  
	}
	
	
	pObj=(DrawObject *&)pFig;
	
	return TRUE;
}



int LogFile::GetColor(SymCmd * CmdStrt,int TID,Bool IsDir,Bool MustVer)
{
	if (!MustVer) return black;
	
	LogListEntry NewEntry(TID,"","","",CmdStrt,0,0,0,0,IsDir,MustVer);
	
	
	if (IsDir) {
		// Colori direttori
		if (NewEntry.TipoComando==ComandoInserimento) return COL_DIR_INS;	
		return COL_DIR;
	}
	if (TID==0) {
		// Colori archivista
		if (NewEntry.TipoComando==ComandoInserimento) return COL_ARC_INS;	
		return COL_ARC;
	}
	// Colori musicisti
	if (NewEntry.TipoComando==ComandoInserimento) return COL_MUS_INS;	
	return COL_MUS;
	
}




// Elimina dalla lista (e salva su NomeFileTemp) i primi elementi Validati della lista

void LogFile::FlushFirstConvalidated(void)
{
	// appendo al file Temporaneo
	FILE *fp;
	fp = fopen(NomeFileTemp,"a");
	if (fp!=NULL) {
		LogListEntry *Runner;
		Runner=HPList;
		while ((HPList!=NULL) && (HPList->IsConvalidated())) {
			Runner=HPList;
			HPList=HPList->GetNext();
			// Serializzo come "flushed"
			Runner->Flush(fp);
			// Stacco e cancello
			Runner->SetNext(NULL);
			HPList->SetPrev(NULL);
			delete Runner;
		}
		fclose(fp);
	}
	if (active)   // Se la finestra  attiva la ridisegno !
		ModifyWindow();
	return;
}


// Indica se uno stream e' arrivato alla fine del file

Bool LogFile::Myeof(FILE *fp)
{
	int ch;
	ch=fgetc(fp);
	if (ch==EOF) return True;
	ungetc(ch,fp);
	return FALSE; 
};


// Scrive tutta la lista su NomeFileTemp o Legge una lista precedentemente salvata su fp

void LogFile::SerializeAll(Bool Store,FILE *fp,Bool Execute)
{
	// appendo al file Temporaneo
	FILE *fp1;
	Battuta *pBat;
	DrawObject *pObj;
	
	if (Store) {
		if (fp!=NULL) {
			LogListEntry *Runner;
			Runner=HPList;
			while (Runner!=NULL) {
				// Serializzo come "Non flushed"
				Runner->Salva(fp);
				Runner=Runner->GetNext();		   
			}
			
			
		} 
	} else {
		// Cancello la lista attuale (eventualmente)
		
		EliminaLista();
		
		
		fp1 = fopen(NomeFileTemp,"w");
		
		if (fp!=NULL) {
			LogListEntry *Runner;		   
			
			while (Myeof(fp)==0) {
				Runner=new LogListEntry;
				
				if (!Runner->Carica(fp))
				{
					
					// non e' un elemento "Flushed", va nella lista
					AddCommand(Runner);	    
					if (Execute) {
						Runner->Execute();
						if (CatchFigure(&(Runner->MyCmd),(Battuta *&)pBat,(DrawObject *&)pObj)) {
							if (pObj!=NULL) {
								int col=GetColor(&(Runner->MyCmd),Runner->TID,
									Runner->FromDirector,Runner->MustBeConvalidated);
								pObj->SetColor((ColorIcon)col);		
							}
						}
					}
				} else {
					// e' Flushed, va nel file se si e' riussciti ad aprirlo
					if (fp1!=NULL) {
						Runner->Flush(fp1);
						if (Execute) {
							Runner->Execute();
							if (CatchFigure(&(Runner->MyCmd),(Battuta *&)pBat,(DrawObject *&)pObj)) {
								if (pObj!=NULL) {
									int col=GetColor(&(Runner->MyCmd),Runner->TID,
										Runner->FromDirector,Runner->MustBeConvalidated);
									pObj->SetColor((ColorIcon)col);		
								}
							}
						}
						delete Runner;
					} else // Altrimenti lo riimmetto lo stesso nella lista
						AddCommand(Runner);	     
				}	
			}
			fclose(fp);
			fclose(fp1);   
		} else 
			EM.Error(LOADSAVE_ERR_AREA,"EC_OPENFILEREAD","LogFile","SerializeAll()");
		
		
	}
	
	return;
}

// Scrive tutta la lista su NomeFileTemp o Legge una lista precedentemente salvata su fp
//   Ritorna la posizione nel file ACL dove iniziano i comandi non flushed
long LogFile::LoadExecuteFlushed(FILE *fp,Bool Save)
{
	// appendo al file Temporaneo
	FILE *fp1;
    
	EliminaLista(); 
	
	fp1=NULL;
	if (Save) fp1 = fopen(NomeFileTemp,"w");
	
	LogListEntry *Runner;	
	DrawObject *pObj;
	Battuta *pBat;	   
	long pos=ftell(fp);
	
	DeleteTempFile();
	
	while (Myeof(fp)==0) {
		Runner=new LogListEntry;
		
		if (!Runner->Carica(fp))
		{
			// Sono finiti i Flushed ! devo ritornale la posizione del file salvata
			return pos;		
		} else {
			// se e' stato salvataggio disgiunto fa nel file temporaneo
			if (fp1!=NULL) 
				Runner->Flush(fp1);		
			
			// lo eseguo
			Runner->Execute();
			if (CatchFigure(&(Runner->MyCmd),(Battuta *&)pBat,(DrawObject *&)pObj)) {
				if (pObj!=NULL) {
					int col=GetColor(&(Runner->MyCmd),Runner->TID,
						Runner->FromDirector,Runner->MustBeConvalidated);
					pObj->SetColor((ColorIcon)col);		
				}
			}
			// lo cancello
			delete Runner;
			pos=ftell(fp);
		}
	}
	
	if (fp1!=NULL) fclose(fp1);   
	
	return pos;
}

Bool LogFile::LoadExecuteNonFlushed(FILE *fp)
{
	EliminaLista(); 
	
	LogListEntry *Runner;	
	DrawObject *pObj;
	Battuta *pBat;	   
	
	while (Myeof(fp)==0) {
		Runner=new LogListEntry;
		
		if (Runner->Carica(fp))
		{
			// E' Flushed e non deve esserlo !!!!!!
			return FALSE;		
		} else {
			
			// lo eseguo
			Runner->Execute();
			// lo inserisco !
			AddCommand(Runner);	    
			if (CatchFigure(&(Runner->MyCmd),(Battuta *&)pBat,(DrawObject *&)pObj)) {
				if (pObj!=NULL) {
					int col=GetColor(&(Runner->MyCmd),Runner->TID,
						Runner->FromDirector,Runner->MustBeConvalidated);
					pObj->SetColor((ColorIcon)col);		
				}
			}
		}
	}
	
	return TRUE;
}


void LogFile::CopyTemp2File(FILE *fp)
{
	FILE *fpsorg;
	char Buffer[512];
	int Quanti;
	fpsorg=fopen(NomeFileTemp,"r");
	if (fpsorg==NULL) return;
	while (!Myeof(fpsorg)) {
		Quanti=fread(Buffer,sizeof(char),512,fpsorg);
		fwrite(Buffer,sizeof(char),Quanti,fp);
	}
	fclose(fpsorg);
	return;
}

// Procedura per scrivere su uno stream l'header
void LogFile::ScriviHeader(FILE *fp,char *Stringa,char *Stringa1,Bool Tipo)
{
	SaveString(START_HEADER,fp);  
	SaveString(Stringa,fp);
	SaveString(Stringa1,fp);
	fwrite(&Tipo,sizeof(Tipo),1,fp);
	SaveString(END_HEADER,fp);
	return;
}

Bool LogFile::LoadHeader(FILE *fp,char *Buffer,char *Buffer1,Bool *Tipo)
{
	char Appo[20];
	LoadString(Appo,fp);
	if (strcmp(Appo,START_HEADER)!=0) {
		// Errore di versione o file errato
		Message("INIT TAG Sbagliato");
		return False;
	}
	LoadString(Buffer,fp);
	LoadString(Buffer1,fp);
	fread(Tipo,sizeof(*Tipo),1,fp);
	LoadString(Appo,fp);
	if (strcmp(Appo,END_HEADER)!=0) {
		// Errore di versione o file errato
		Message("CLOSE TAG Sbagliato");
		return False;
	} 
	return TRUE;
}


// Procedura generale di salvataggio; parametri
//     - NomeFileACL = Nome del file che conerra' la ACL
//     - NomeFilePar = Nome del file (o descrizione dell'insieme) della partitura
//     - Insieme = Informazione sul tipo di salvataggio da porre nell'header
Bool LogFile::Salva(char *NomeFileACL,char *NomeFilePar,char *NomePathPar,Bool Insieme)
{
	// procedura generale di salvataggio !
	FILE *fp;
	// non verifico l'esistenza del file !
	fp=fopen(NomeFileACL,"w");
	if (fp==NULL) {
		Message("Errore nella creazione del file\n");
		return False;
	}	
	if (HPList==NULL) {
		Message("LogList vuota !\n");
		return False;
	}	
	// Scrittura dell'header
	ScriviHeader(fp,NomeFilePar,NomePathPar,Insieme); 
	
	// Copio i primi comandi che erano stati flushati
	CopyTemp2File(fp);
	
	// Svuoto tutta la lista sul file temporaneo !
	SerializeAll(True,fp);
	
	
	// chiuso
	fclose(fp);
	
	return True;
}

Bool LogFile::LoadAndExecute(char *NomeFileACL)
{
	FILE *fp;
	char Buffer[255];
	char Buffer1[255];
	Bool Tipo;
	
	fp=fopen(NomeFileACL,"r");
	if (fp==NULL) return False;
	// Caricamento Header!
	if (!LoadHeader(fp,Buffer,Buffer1,&Tipo)) 
		return False;
	
	// non carica nella lista i Flushed
	
	SerializeAll(False,fp);
	
	fclose(fp);
	ExecuteACL();
	return True; 
}


void LogFile::ExecuteACL(void)
{
	LogListEntry * Runner;
	Runner=HPList;
	Battuta *pBat;
	DrawObject * pObj;
	
	while (Runner!=NULL) {
		Runner->Execute();
		if (CatchFigure(&(Runner->MyCmd),(Battuta *&)pBat,(DrawObject *&)pObj)) {
			if (pObj!=NULL) {
				int col=GetColor(&(Runner->MyCmd),Runner->TID,
					Runner->FromDirector,Runner->MustBeConvalidated);
				pObj->SetColor((ColorIcon)col);		
			}
		}
		Runner=Runner->GetNext();	 
	}
	
	
	
	if (active)   // Se la finestra  attiva la ridisegno !
		ModifyWindow();
	
	pMusicWin->Redraw();
	
	Message("Commands Executed");
	
	return;
}


void LogFile::DeleteTempFile(void)
{
	remove(NomeFileTemp);
	return;
}





// Procedura per la verifica della sicurezza di un comando 
//  Ritorna True se il comando  insicuro
Bool LogFile::CheckAbilCommand(SymCmd * MyCmd,ExtLiooAbil * Abilitazioni)
{
	Bool Abil=True;
	switch(MyCmd->CommandId) {
	case CMD_DEL_SYMBOL:
		if ( (Abilitazioni->isAbil(ABIL_MODIFICA)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_INTESTAZ)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_BATTUTA)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_PARTE)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_MOVIMENTO)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_LEGATURA)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_FORCELLA)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_OTTAVA)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_ONDA)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_CAMBIORIT)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_ACCENTI)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_ABBELLIM)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_KOREANORN)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_TDINAMICO)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_TGENERICO)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_ANNOTAZDIR)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_CORONA)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_SUDDIVIS)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_DITEGGIATO)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_SORDINA)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_PRFIATO)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_OCCHIALI)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_VIOLINO)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_TIMPANO)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_OBOE)==ABIL_V) ||
			(Abilitazioni->isAbil(ABIL_SAX)==ABIL_V) ) return True;
		
		return False;
		
		break;
	case CMD_DEL_COL_FIGURE:
		// Di qui non passera mai perch lo puo' fare solo il MASAE, i cui comandi sono sempre sicuri e non vengono analizzati !
        Abil=False;
		break;
	case CMD_PARTITURA_ADD_GRAFFA:
		Abil=(Abilitazioni->isAbil(ABIL_PARTE)==ABIL_V);
		break;
	case CMD_PARTITURA_ADD_SPARTITO:
		Abil=(Abilitazioni->isAbil(ABIL_PARTE)==ABIL_V);
		break;
	case CMD_PARTITURA_DEL_SPARTITO:
		Abil=(Abilitazioni->isAbil(ABIL_PARTE)==ABIL_V);
		break;
	case CMD_PARTITURA_NEW:  	
		Abil=(Abilitazioni->isAbil(ABIL_FILE)==ABIL_V);
        break;
	case CMD_PARTITURA_LOAD:
		Abil=(Abilitazioni->isAbil(ABIL_FILE)==ABIL_V);
        break;
	case CMD_PARTITURA_SAVEAS:
		Abil=(Abilitazioni->isAbil(ABIL_FILE)==ABIL_V);
        break;
	case CMD_PARTITURA_SAVE:
		Abil=(Abilitazioni->isAbil(ABIL_FILE)==ABIL_V);
        break;
	case CMD_PARTITURA_END:
		Abil=(Abilitazioni->isAbil(ABIL_FILE)==ABIL_V);
        break;
	case CMD_PARTITURA_ADD_BATTUTA: 	
		Abil=(Abilitazioni->isAbil(ABIL_BATTUTA)==ABIL_V);
		break;
	case CMD_PARTITURA_DEL_BATTUTA:
		Abil=(Abilitazioni->isAbil(ABIL_BATTUTA)==ABIL_V);
		break;
	case CMD_SPARTITO_ADD_LEGATURA:
		Abil=(Abilitazioni->isAbil(ABIL_LEGATURA)==ABIL_V);
		break;
	case CMD_SPARTITO_ADD_LEGATURA_VALORE:
		Abil=(Abilitazioni->isAbil(ABIL_LEGATURA)==ABIL_V);
		break;
	case CMD_SPARTITO_ADD_FORC_CRESC:
		Abil=(Abilitazioni->isAbil(ABIL_FORCELLA)==ABIL_V);
		break;
	case CMD_SPARTITO_ADD_FORC_DIMIN:
		Abil=(Abilitazioni->isAbil(ABIL_FORCELLA)==ABIL_V);
		break;
	case CMD_SPARTITO_ADD_FRECCIA:
		Abil=(Abilitazioni->isAbil(ABIL_ONDA)==ABIL_V);
		break;
	case CMD_SPARTITO_ADD_ONDA:
		Abil=(Abilitazioni->isAbil(ABIL_ONDA)==ABIL_V);
		break;
	case CMD_SPARTITO_ADD_LEG_QUADRA:
		Abil=(Abilitazioni->isAbil(ABIL_LEGATURA)==ABIL_V);
		break;
	case CMD_SPARTITO_ADD_CAMBIO_RIT: 	
		Abil=(Abilitazioni->isAbil(ABIL_CAMBIORIT)==ABIL_V);
		break;
	case CMD_SPARTITO_ADD_MODIF8:
	case CMD_SPARTITO_ADD_MODIF15MA:
	case CMD_SPARTITO_ADD_MODIF8BA:
	case CMD_SPARTITO_ADD_MODIF15BA:
		Abil=(Abilitazioni->isAbil(ABIL_OTTAVA)==ABIL_V);
		break;
	case CMD_SPARTITO_SET_NAME:
		Abil=(Abilitazioni->isAbil(ABIL_PARTE)==ABIL_V);
		break;
	case CMD_BATTUTA_NOTE2GRUPPONOTE:
		Abil=(Abilitazioni->isAbil(ABIL_MODIFICA)==ABIL_V);
        break;
	case CMD_BATTUTA_GRUPPONOTE2NOTE:
		Abil=(Abilitazioni->isAbil(ABIL_MODIFICA)==ABIL_V);
        break;
	case CMD_BATTUTA_ELIMINAGRUPPI:
		Abil=(Abilitazioni->isAbil(ABIL_MODIFICA)==ABIL_V);
        break;
	case CMD_BATTUTA_ERASE:
		Abil=(Abilitazioni->isAbil(ABIL_BATTUTA)==ABIL_V);
		break;
	case CMD_BATTUTA_BSINGOLA:
		Abil=(Abilitazioni->isAbil(ABIL_BATTUTA)==ABIL_V);
		break;
	case CMD_BATTUTA_BSINGOLA_RIP2B:
		Abil=(Abilitazioni->isAbil(ABIL_BATTUTA)==ABIL_V);
		break;
	case CMD_BATTUTA_BDOPPIA:
		Abil=(Abilitazioni->isAbil(ABIL_BATTUTA)==ABIL_V);
		break;
	case CMD_BATTUTA_BINIZIORIT:
		Abil=(Abilitazioni->isAbil(ABIL_BATTUTA)==ABIL_V);
		break;
	case CMD_BATTUTA_BFINERIT:
		Abil=(Abilitazioni->isAbil(ABIL_BATTUTA)==ABIL_V);
		break;
	case CMD_BATTUTA_BINIZIOFINERIT:
		Abil=(Abilitazioni->isAbil(ABIL_BATTUTA)==ABIL_V);
		break;
	case CMD_BATTUTA_BFINALE:
		Abil=(Abilitazioni->isAbil(ABIL_BATTUTA)==ABIL_V);
		break;
	case CMD_BATTUTA_BINVISIBILE:
		Abil=(Abilitazioni->isAbil(ABIL_BATTUTA)==ABIL_V);
		break;
	case CMD_BATTUTA_NUGUALE:
		Abil = True;
		break;
	case CMD_BATTUTA_TSALTO:
		Abil=(Abilitazioni->isAbil(ABIL_MOVIMENTO)==ABIL_V);
		break;
	case CMD_BATTUTA_RIFERIMENTO:
		Abil=(Abilitazioni->isAbil(ABIL_MOVIMENTO)==ABIL_V);
		break;
		//
	case CMD_BATTUTA_LETTERA:
		Abil=(Abilitazioni->isAbil(ABIL_MOVIMENTO)==ABIL_V);
		break;
	case CMD_BATTUTA_SCANSIONE:
		Abil=(Abilitazioni->isAbil(ABIL_MOVIMENTO)==ABIL_V);
		break;
	case CMD_BATTUTA_MOVIMENTO:
		Abil=(Abilitazioni->isAbil(ABIL_MOVIMENTO)==ABIL_V);
		break;
	case CMD_BATTUTA_INS_NBREVE:
	case CMD_BATTUTA_INS_NSEMIBREVE:
	case CMD_BATTUTA_INS_NMINIMA:
	case CMD_BATTUTA_INS_NSEMIMINIMA:
	case CMD_BATTUTA_INS_NCROMA:
	case CMD_BATTUTA_INS_NSEMICROMA:
	case CMD_BATTUTA_INS_NBISCROMA:
	case CMD_BATTUTA_INS_NSEMIBISCROMA:
	case CMD_BATTUTA_INS_NFUSA:
	case CMD_BATTUTA_INS_PQUABATT:
	case CMD_BATTUTA_INS_PDUEBATT:
	case CMD_BATTUTA_INS_PSEMIBREVE:
	case CMD_BATTUTA_INS_PMINIMA:
	case CMD_BATTUTA_INS_PSEMIMINIMA:
	case CMD_BATTUTA_INS_PCROMA:
	case CMD_BATTUTA_INS_PSEMICROMA:
	case CMD_BATTUTA_INS_PBISCROMA:
	case CMD_BATTUTA_INS_PSEMIBISCROMA:
	case CMD_BATTUTA_INS_PFUSA:
	case CMD_BATTUTA_INS_PGENERICA:
		Abil=(Abilitazioni->isAbil(ABIL_MODIFICA)==ABIL_V);
		break;
	case CMD_BATTUTA_INS_SPAZIO:
		if (MyCmd->ArgV[2]==0) {
			Abil=(Abilitazioni->isAbil(ABIL_MODIFICA)==ABIL_V);
		} else switch (MyCmd->ArgV[2]) {
	case CMD_BATTUTA_CVIOLINO:
	case CMD_BATTUTA_CVIOLINO8:
	case CMD_BATTUTA_CVIOLINO8SOPRA:
	case CMD_BATTUTA_CBASSO:
	case CMD_BATTUTA_CBASSO8:
	case CMD_BATTUTA_CBASSO8SOPRA:
	case CMD_BATTUTA_CBASSOOLD:
	case CMD_BATTUTA_CBARITONO:
	case CMD_BATTUTA_CTENORE:
	case CMD_BATTUTA_CTENORE8:
	case CMD_BATTUTA_CCONTRALTO:
	case CMD_BATTUTA_CMEZZOSOPRANO:
	case CMD_BATTUTA_CSOPRANO:
	case CMD_BATTUTA_CPERCUSBOX:
	case CMD_BATTUTA_CPERCUS2LINES:
	case CMD_BATTUTA_CTAB:
	case CMD_BATTUTA_CVUOTA:
 				   Abil=(Abilitazioni->isAbil(ABIL_INTESTAZ)==ABIL_V);
				   break;
		}
		break;
	case CMD_BATTUTA_INS_RIPTEMPO:
	case CMD_BATTUTA_INS_RIPBATTUTA:
	case CMD_BATTUTA_INS_RIPMEZZABATTUTA:
		Abil=(Abilitazioni->isAbil(ABIL_MODIFICA)==ABIL_V);
		break;
	case CMD_BATTUTA_CVIOLINO:
	case CMD_BATTUTA_CVIOLINO8:
	case CMD_BATTUTA_CVIOLINO8SOPRA:
	case CMD_BATTUTA_CBASSO:
	case CMD_BATTUTA_CBASSO8:
	case CMD_BATTUTA_CBASSO8SOPRA:
	case CMD_BATTUTA_CBASSOOLD:
	case CMD_BATTUTA_CBARITONO:
	case CMD_BATTUTA_CTENORE:
	case CMD_BATTUTA_CTENORE8:
	case CMD_BATTUTA_CCONTRALTO:
	case CMD_BATTUTA_CMEZZOSOPRANO:
	case CMD_BATTUTA_CSOPRANO:
	case CMD_BATTUTA_CPERCUSBOX:
	case CMD_BATTUTA_CPERCUS2LINES:
	case CMD_BATTUTA_CTAB:
	case CMD_BATTUTA_CVUOTA:
	case CMD_BATTUTA_ARMCHV_0D:
	case CMD_BATTUTA_ARMCHV_1D:
	case CMD_BATTUTA_ARMCHV_2D:
	case CMD_BATTUTA_ARMCHV_3D:
	case CMD_BATTUTA_ARMCHV_4D:
	case CMD_BATTUTA_ARMCHV_5D:
	case CMD_BATTUTA_ARMCHV_6D:
	case CMD_BATTUTA_ARMCHV_7D:
	case CMD_BATTUTA_ARMCHV_1B:
	case CMD_BATTUTA_ARMCHV_2B:
	case CMD_BATTUTA_ARMCHV_3B:
	case CMD_BATTUTA_ARMCHV_4B:
	case CMD_BATTUTA_ARMCHV_5B:
	case CMD_BATTUTA_ARMCHV_6B:
	case CMD_BATTUTA_ARMCHV_7B:
		
	case CMD_BATTUTA_ARMCHV_0Dm:
	case CMD_BATTUTA_ARMCHV_1Dm:
	case CMD_BATTUTA_ARMCHV_2Dm:
	case CMD_BATTUTA_ARMCHV_3Dm:
	case CMD_BATTUTA_ARMCHV_4Dm:
	case CMD_BATTUTA_ARMCHV_5Dm:
	case CMD_BATTUTA_ARMCHV_6Dm:
	case CMD_BATTUTA_ARMCHV_7Dm:
	case CMD_BATTUTA_ARMCHV_1Bm:
	case CMD_BATTUTA_ARMCHV_2Bm:
	case CMD_BATTUTA_ARMCHV_3Bm:
	case CMD_BATTUTA_ARMCHV_4Bm:
	case CMD_BATTUTA_ARMCHV_5Bm:
	case CMD_BATTUTA_ARMCHV_6Bm:
	case CMD_BATTUTA_ARMCHV_7Bm:
		//
	case CMD_BATTUTA_TEMPO:
		Abil=(Abilitazioni->isAbil(ABIL_INTESTAZ)==ABIL_V);
		break;
	case CMD_FIGURA_1PUNTO:
	case CMD_FIGURA_2PUNTI:
	case CMD_FIGURA_3PUNTI:
		Abil=(Abilitazioni->isAbil(ABIL_MODIFICA)==ABIL_V);
		break;
	case CMD_FIGURA_CORONA:
	case CMD_FIGURA_CORONAQUAD:
	case CMD_FIGURA_CORONATRIANG:
		Abil=(Abilitazioni->isAbil(ABIL_CORONA)==ABIL_V);
		break;
	case CMD_FIGURA_OCCHIALI:
		Abil=(Abilitazioni->isAbil(ABIL_OCCHIALI)==ABIL_V);
		break;
	case CMD_FIGURA_PPPPPP:
	case CMD_FIGURA_PPPPP:
	case CMD_FIGURA_PPPP:
	case CMD_FIGURA_PPP:
	case CMD_FIGURA_PP:
	case CMD_FIGURA_P:
	case CMD_FIGURA_FFFFFF:
	case CMD_FIGURA_FFFFF:
	case CMD_FIGURA_FFFF:
	case CMD_FIGURA_FFF:
	case CMD_FIGURA_FF:
	case CMD_FIGURA_F:
	case CMD_FIGURA_MP:
	case CMD_FIGURA_MF:
	case CMD_FIGURA_SP:
	case CMD_FIGURA_SF:
	case CMD_FIGURA_FZ:
	case CMD_FIGURA_SFZ:
	case CMD_FIGURA_RF:
	case CMD_FIGURA_FFP:
	case CMD_FIGURA_FFPP:
	case CMD_FIGURA_FPP:
	case CMD_FIGURA_RFZ:
	case CMD_FIGURA_SFF:
	case CMD_FIGURA_SFFF:
	case CMD_FIGURA_SFFP:
	case CMD_FIGURA_SFFZ:
	case CMD_FIGURA_SFP:
	case CMD_FIGURA_SFPP:
	case CMD_FIGURA_FP:
		Abil=(Abilitazioni->isAbil(ABIL_TDINAMICO)==ABIL_V);
		break;
	case CMD_FIGURA_TGENERICO:
		Abil=(Abilitazioni->isAbil(ABIL_TGENERICO)==ABIL_V);
		break;
	case CMD_FIGURA_ANNOTAZIONE:
		Abil=(Abilitazioni->isAbil(ABIL_ANNOTAZDIR)==ABIL_V);
		break;
	case CMD_NOTA_CORDA_0:
	case CMD_NOTA_CORDA_1:
	case CMD_NOTA_CORDA_2:
	case CMD_NOTA_CORDA_3:
	case CMD_NOTA_CORDA_4:
	case CMD_NOTA_CORDA_5:
	case CMD_NOTA_CORDA_6:
	case CMD_NOTA_CORDA_7:
		Abil=(Abilitazioni->isAbil(ABIL_VIOLINO)==ABIL_V); 
		break;
	case CMD_NOTA_DIESIS:
	case CMD_NOTA_BEMOLLE:
	case CMD_NOTA_BEQUADRO:
	case CMD_NOTA_BEQUADRO1Q:
	case CMD_NOTA_BEQUADRO3Q:
	case CMD_NOTA_2DIESIS:
	case CMD_NOTA_2BEMOLLE:
	case CMD_NOTA_DIESIS1Q:
	case CMD_NOTA_DIESIS3Q:
	case CMD_NOTA_BEMOLLE1Q:
	case CMD_NOTA_BEMOLLE3Q:
		Abil=(Abilitazioni->isAbil(ABIL_MODIFICA)==ABIL_V);
		break;
	case CMD_NOTA_TREMOLO:
	case CMD_NOTA_TRILLO:
	case CMD_NOTA_GRUPPETTO_SUP_1:
	case CMD_NOTA_GRUPPETTO_SUP_2:
	case CMD_NOTA_TURN_SLASH:
	case CMD_NOTA_TURN_UP:
	case CMD_NOTA_GRUPPETTO_INF_1:
	case CMD_NOTA_GRUPPETTO_INF_2:
	case CMD_NOTA_MORDENTE_SUP:
	case CMD_NOTA_MORDENTE_INF:
	case CMD_NOTA_MORDENTE2_SUP:
	case CMD_NOTA_MORDENTE2_INF:
		Abil=(Abilitazioni->isAbil(ABIL_ABBELLIM)==ABIL_V);
		break;
	case CMD_NOTA_SLIDING_UP:
	case CMD_NOTA_SLIDING_DOWN:
	case CMD_NOTA_SLIDING_DUP:
	case CMD_NOTA_SLIDING_DOWNUP:
	case CMD_NOTA_NONGHYN_INC:
	case CMD_NOTA_NONGHYN_DEC:
	case CMD_NOTA_NONGHYN_ARROW:
	case CMD_NOTA_NONGHYN_WIDE:
		Abil=(Abilitazioni->isAbil(ABIL_KOREANORN)==ABIL_V);
		break;
	case CMD_NOTA_ARMONICIARCHI:
	case CMD_NOTA_ARMONICIOTTONI:
        // Vedi Lioomenu.cpp, funzione MenuStrumento
		
		Abil=(Abilitazioni->isAbil(ABIL_SORDINA)==ABIL_V);
		break;
	case CMD_NOTA_NOTA2NOTINA:
	case CMD_NOTA_NOTINA2NOTA:
	case CMD_NOTA_ACCIACCATURA:
		Abil=(Abilitazioni->isAbil(ABIL_MODIFICA)==ABIL_V);
		break;
	case CMD_NOTA_SUDDIVISIONE:
		Abil=(Abilitazioni->isAbil(ABIL_SUDDIVIS)==ABIL_V);
		break;
	case CMD_NOTA_DITO_0:
	case CMD_NOTA_DITO_1:
	case CMD_NOTA_DITO_2:
	case CMD_NOTA_DITO_3:
	case CMD_NOTA_DITO_4:
	case CMD_NOTA_DITO_5:
		Abil=(Abilitazioni->isAbil(ABIL_DITEGGIATO)==ABIL_V);
		break;
	case CMD_NOTA_SFORZATO:
	case CMD_NOTA_ACCENTO:
	case CMD_NOTA_TENUTO:
	case CMD_NOTA_PUNTOALL:
	case CMD_NOTA_MARTELLATO:
	case CMD_NOTA_MARTDOLCE:
	case CMD_NOTA_STACCATO:
	case CMD_NOTA_ESPGENERIC: 
		Abil=(Abilitazioni->isAbil(ABIL_ACCENTI)==ABIL_V);
		break;
	case CMD_NOTA_VIASORDINAOTTONI:
	case CMD_NOTA_VIASORDINAARCHI:
	case CMD_NOTA_VIASORDINATESTO:
	case CMD_NOTA_CONSORDINAOTTONI:
	case CMD_NOTA_CONSORDINAARCHI:
	case CMD_NOTA_CONSORDINATESTO:
		Abil=(Abilitazioni->isAbil(ABIL_SORDINA)==ABIL_V);
		break;
	case CMD_NOTA_ARCO:
	case CMD_NOTA_PIZZICATO:
		Abil=(Abilitazioni->isAbil(ABIL_VIOLINO)==ABIL_V);
		break;
	case CMD_PAUSA_TIMPANO:
		Abil=(Abilitazioni->isAbil(ABIL_TIMPANO)==ABIL_V);
		break;
	case CMD_SPAZIO_PRFIATO:
		Abil=(Abilitazioni->isAbil(ABIL_PRFIATO)==ABIL_V);
		break;
	case CMD_ACCORDO_ARPEGGIO:
		Abil=(Abilitazioni->isAbil(ABIL_ABBELLIM)==ABIL_V);	
		break;
	case CMD_ARCO_PUNTA:
	case CMD_ARCO_TALLONE:
	case CMD_ARCO_TASTIERA:
	case CMD_ARCO_PONTICELLO:
	case CMD_ARCO_ARCOSU:
	case CMD_ARCO_ARCOGIU: 	
	case CMD_ARCO_CORDA:
		Abil=(Abilitazioni->isAbil(ABIL_VIOLINO)==ABIL_V);
		break;
	default:
		Abil = True;
 };
 return Abil;
 
}


// Procedure per il DEBUG !
//----------------------------------------------------------
//----------------------------------------------------------

// Basta settare la define DUMP_FILE a False per evitare che si salvi su disco ogni comando
// in forma testuale.


void LogFile::DumpOnFile(LogListEntry *Entry) 
{
	FILE *fp;
	fp = fopen(NomeFileDebug,"a");
	if (fp!=NULL) {
		fprintf(fp,"Comando ricevuto dalla rete :\n");
		fprintf(fp,"---- INFO  MITTENTE ----\n");
		fprintf(fp," TID = %u\n",Entry->TID);
		fprintf(fp," Nome dell'host : ");
		fprintf(fp,"%s",Entry->NomeHost);fprintf(fp,"\n");
		fprintf(fp," Nome del musicista : ");
		fprintf(fp,"%s",Entry->NomePlayer);fprintf(fp,"\n");
		fprintf(fp," Nome dello spartito : ");
		fprintf(fp,"%s",Entry->NomeSpartito);fprintf(fp,"\n");
		fprintf(fp,"---- INFO  COMANDO ----\n");
		fprintf(fp," ID del comando : %u\n",Entry->MyCmd.CommandId);
		fprintf(fp," Tipo di comando : ");
		switch (Entry->TipoComando) {
		case ComandoInserimento: fprintf(fp,"Inserimento\n");
			break;
		case ComandoCancellazione: fprintf(fp,"Cancellazione\n");
			break;
		case ComandoAzione:	fprintf(fp,"Azione\n");
			break;
		default: fprintf(fp,"Sconosciuto\n");
			break;
		};
		fprintf(fp," Descrizione : %s\n",Entry->DescrizioneComando);
		
		fprintf(fp," Battuta 1 interessata : ");
		if (Entry->nbat1==(-1)) 
			fprintf(fp,"Nessuna");
		else fprintf(fp,"%u",Entry->nbat);
		fprintf(fp,"\n");
		fprintf(fp," Figura 1 interessata : ");
		if (Entry->nfig1==(-1)) 
			fprintf(fp,"Nessuna");
		else fprintf(fp,"%d",Entry->nfig);
		fprintf(fp,"\n");
		fprintf(fp," Battuta 2 interessata : ");
		if (Entry->nbat1==(-1)) 
			fprintf(fp,"Nessuna");
		else fprintf(fp,"%u",Entry->nbat);
		fprintf(fp,"\n");
		fprintf(fp," Figura 2 interessata : ");
		if (Entry->nfig1==(-1)) 
			fprintf(fp,"Nessuna");
		else fprintf(fp,"%d",Entry->nfig);
		
		
		fprintf(fp,"\n\n");
		fclose(fp);	
	}
	return;
};



