/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: logfile.hpp,v 1.1 2005/07/25 16:31:40 bellinpo Exp $ */


#ifndef LOGFILE_HPP
#define LOGFILE_HPP

// Classe che gestisce il file di log.


// Nomi del file per un possibile Dump testuale
#define NomeFileDebug "/home/demo/integrazione/src/net/LOGFILE.LOG"
// Nome del file temporaneo su cui vengono salvati i comandi flushed
#define NomeFileTemp "/home/demo/integrazione/TEMP.ACL"


// TAG di delimitazione dell'header del file ACL
#define START_HEADER 	"START-ACL-HEADER"
#define END_HEADER 	"END-ACL-HEADER"

// Massima lunghezza delle stringhe in  un LogListEntry
#define MaxListEntryString	30

// Tipi di comando possibili (per una schematizzazione testuale
#define ComandoInserimento	0
#define ComandoCancellazione	1
#define ComandoAzione		2
#define ComandoNonConosciuto    10

#define MAX_STRINGH_LENGTH	255

// Porre a True questa define per avere un DUMP testuale sul file NomeFileDebug
#define DUMP_FILE	False

// Tag che definiscono che tipo di LogListEntry viene serializzato : 
//  VERY IMPORTANT : devono essere due stringhe qualunque (diverse) purche' della stessa lunghezza
#define FLUSH_TAG	"F"
#define STORE_TAG	"S"

#define COL_DIR_INS   BLU
#define COL_DIR       ROSSO
#define COL_ARC_INS   GRIGIOS //black
#define COL_ARC       GRIGIOS //black
#define COL_MUS_INS   VERDE
#define COL_MUS       GIALLO


#include "drw/scrobjec.hpp"
#include "gui/errman.hpp"
#include "gui/extabil.hpp"

#define FILTER_NONE	0
#define FILTER_NUM_BAT	1
#define FILTER_PARTE	2
#define FILTER_MUSICIST 4

class LogFilter {
private:
	int FilteringMode;
	int NumBat;
	int NumParte;
	int Musicista;
	
public:
	void SetFilteringMode(int Filter) {
		if (Filter!=FILTER_NONE) FilteringMode|=Filter; 
		else FilteringMode=FILTER_NONE; 
		return;
	};
	void ClearFilteringMode(int Filter) {if (CheckFilter(Filter)) FilteringMode-=Filter; return;};
	void ResetFilter(void) {SetFilteringMode(FILTER_NONE); return; };
	Bool CheckFilter(int Filter) {
		if (Filter==FILTER_NONE) return FilteringMode==FILTER_NONE;
		else return ((FilteringMode & Filter)==Filter); 
	};			
	
	int GetNumBat(void) { return NumBat;};
	void SetNumBat(int Value) { NumBat=Value; return; };
	
	void SetNumPar(int Value) { NumParte=Value; return; };
	int GetNumPar(void) { return NumParte;};
	
	void SetNumMus(int Value) {Musicista=Value; return;};
	int GetNumMus(void) {return Musicista; };
	
	void ToggleFilter(int Filter) {
		if (CheckFilter(Filter)) FilteringMode-=Filter;
		else FilteringMode += Filter;
	}
	
};


class LogListEntry {
	
private:
	LogListEntry *Next;
	LogListEntry *Prev;	
public:
	// Dati di un elemento di log
	int TID;
	char NomeHost[MaxListEntryString];
	char NomePlayer[MaxListEntryString];
	char NomeSpartito[MaxListEntryString];
	char DescrizioneComando[MaxListEntryString];
	char TipoComando;
	
	signed int nbat,nfig,nbat1,nfig1;
	// Queste informazioni probabilmente verranno contenute dentro la struttura SymCmd,
	// per ora devono essere passate al momento dell'inserimento del comando !
	Bool	FromDirector;
	Bool	MustBeConvalidated;
	
	
	SymCmd MyCmd;
	
	// Funzioni Membro private
private:
	void PreparaComando(void);	
	void SetData (int Tid,char *NomeHo,char *NomePl,char *NomeSp, 
		signed int b1,signed int f1,signed int b2,signed int f2,
		Bool IsDir,Bool MustVer);
	// Questa funzione riempie tutte le variabili interne dell'oggetto
	
public:
	LogListEntry (void);	
	LogListEntry(int TID,char *NomeHost,char *NomePlayer,char *NomeSpartito, SymCmd *CmdStrt,
		signed int b1,signed int f1,signed int b2,signed int f2,
		Bool IsDir,Bool MustVer);
	~LogListEntry(void);
	
	
	void Convalida(void);	// Convalida questo elemento
	void Invalida(void);	// Invalida questo elemento
	Bool IsConvalidated(void) { return !MustBeConvalidated; };
	
	Bool IsInFilter(LogFilter * CurrentFilter);
	
	void SetNext (LogListEntry *Entry) { Next=Entry; return; };
	LogListEntry * GetNext(void) { return Next; };
	
	void SetPrev (LogListEntry *Entry) { Prev=Entry; return; };
	LogListEntry * GetPrev(void) { return Prev; };
	
	
	
	void MakeString(char *Buffer); // crea una stringa che spiega il comando
	void MakeDump(char *Buffer);   // crea una stringa per il dump di MyCmd
	
	void Serialize(Bool Store,FILE *fp); 	// salva/carica su/da uno stream i dati 	
	void Salva(FILE *fp); 		// salva sullo stream come "Non Flushed"
	void Flush(FILE *fp);		// salva sullo stream come "Flushed"
	Bool Carica(FILE *fp);		// Ritorna TRUE se "Flushed"
	
	void Execute(void);
};

class LogFile {
private:
	LogListEntry *HPList;
	// Puntatore al primo elemento della lista
	LogListEntry *LastEntry;
	// Puntatore all'ultimo elemento della lista
	// (Usato per velocizzare l'inserimento in coda)
	
	
	Bool active;	      // Indica se la finestra e' correntemente attiva (Visualizzata)
	int CurrentSelected;  // Contiene il numero dell'Entry nella lista correntemente
	// Selezionato (quando la finestra e' attiva) o -1
	// void AvviaFinestra(void);
	void PrepareCommandString(int Quale,char *Stringa,Bool Filter=FALSE);
	// Prepara ogni stringa da inserire in ArrayStringhe
	// Se Filter e' TRUE allora  l'elemento Quale dell'insieme filtrato
	int GetNumCommand(void);
    int GetNumFilteredCommand(void);
	// Ritorna il numero di LogListEntry nella lista 
	
	char **ArrayStringhe;  // Puntatore all'array di puntatori a stringhe usato per
	// contenere (non inizializzare) i dati della Widget listbox
public:
	LogFilter	CurrentFilter;
	Bool Myeof(FILE *fp);
	
	
	LogFile (void);
	~LogFile(void);
	
	void EliminaLista(void);
	// Cancella tutta la lista
	
	
	Bool IsActive(void) { return active; };
	void AddCommand(int TID,char *NomeHost,char *NomePlayer,char *NomeSpartito, SymCmd *CmdStrt,
		signed int b1,signed int f1,signed int b2,signed int f2,                        
		Bool IsDir, Bool MustVer);
	//  Aggiunge un LogListEntry in coda alla lista con i dati passati
	
	void AddCommand(LogListEntry *Elemento);
				// Aggiunge il LogListEntry passato in coda alla lista
	Bool CatchFigure(SymCmd * MyCmd,Battuta *& pBat,DrawObject *& pObj);
				// Cerca di ottenere il puntatore alla battuta e figura interessata da un SymCmd
	int GetColor(SymCmd * CmdStrt,int TID,Bool IsDir,Bool MustVer);
				// Ritorna il colore di un comando
	
	void DumpOnFile(LogListEntry *Entry);
				// Per ora fa un Dump su file in modo Testo descritto
	
	
	Bool IsFigureInsertion(CmdID CommandId);
				// Determina a partire dall'ID del comando se si tratta di un
				// Inserimento di una figura musicale
	Bool IsAggiunta(CmdID CommandId);
				// Determina a partire dall'ID del comando se si tratta di un 
				// inserimento di un simbolo ad una figura musicale (ins. Corona, etc.)
	Bool IsSpartitoCommand(CmdID CommandId);
				// Determina se e' un'operazione del tipo CMD_SPARTITO_...
	Bool IsBattutaAggiunta(CmdID CommandId);
				// Determina se e' un'operazione del tipo Armature, chiavi, etc.
	
	Bool OpenWindow(char *title);
				// Apre la finestra con un titolo passato
	void ModifyWindow(Bool ManteinSelection=FALSE);
				// Modifica i dati contenuti nella finestra 
	void CloseWindow(void);
				// Chiude la finestra
	
	void Switch(void);
				// Apre / Chiude la finestra
	LogListEntry * GetEntry(int index,Bool Filter=FALSE);
				// Ritorna un dato LogListEntry della lista (0 based index)
				// Se Filter  TRUE applica il filtro
	void MakeDump(int index,char *Buffer);
				// Genera il Dump su una stringa di un dato LogListEntry
				// E' usato per la finestra che visualizza il SymCmd.
	void SetCurrentSelected(int index);
				// Memorizza la corrente selezione della listbox
	int GetCurrentSelected(void);
				// Ritorna il numero dell'elemento correntemente selezionato !
	void Convalida(int index);
				//  Convalida un LogListEntry (-1 per quello selezionato)
	void Invalida(int index);
				//  Invalida un LogListEntry (-1 per quello selezionato)
	void ConvalidaAll(void);
	// Convalida tutti i comandi nella lista
	void FlushFirstConvalidated(void);
				// Esegue il flush dei primi comandi convalidati !
	
	void SerializeAll(Bool Store,FILE *fp=NULL,Bool Execute=FALSE);
				// salva sul file temporaneo o carica la lista da un file
				// con il nome specificato
	void CopyTemp2File(FILE *fp);
public:
	Bool Salva(char *NomeFileACL,char *NomeFilePar,char *NomePathPar,Bool Insieme);
	Bool LoadAndExecute(char *NomeFileACL);
	long LoadExecuteFlushed(FILE *fp,Bool Save);
	Bool LoadExecuteNonFlushed(FILE *fp);
	void ExecuteACL(void);
	void DeleteTempFile(void);
	
	
	void ScriviHeader(FILE *fp,char *Stringa,char *Stringa1,Bool Tipo);
	Bool LoadHeader(FILE *fp,char *Buffer,char *Buffer1,Bool *Tipo);
	
public:	
	// Verifica se un comando  sicuro o no
	Bool CheckAbilCommand(SymCmd * MyCmd,ExtLiooAbil * Abilitazioni);
};



// Funzioni esterne per l'interfaccia grafica (sorgente in /lioomsg.cpp
extern void WCloseCommandList(void);
extern Bool WCommandList(char *title,char** Stringhe);
extern void WUpdateCommandList(char** Stringhe,int cursel=0);


#endif // LOGFILE_HPP
