/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: lomas.cpp,v 1.1 2005/07/25 16:31:41 bellinpo Exp $ */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "net/lomas.hpp"
#include "net/def_net.hpp"
#include "liooext.hpp"

LoMas::LoMas(bool empty) : LdMas(this), ParteL(empty) 
{
}; 

LoMas::~LoMas()
{  
#ifdef WITH_NET   
#ifdef DEBUG_FREE
	FILE *pippo;
	
	pippo = fopen("/home/demo/integrazione/src/DEAL_NET.txt","a");
	fprintf(pippo,"\n\n\n ------------   DEALLOCO LOMASS ----------------\n\n\n");
	fclose(pippo);
#else
#endif 
#else
#endif 
}   


void LoMas::EditingGoBottom()
{
	LdMas::EditingGoBottom();
};     

void LoMas::EditingJumpBatt(int nbat)
{
	LdMas::EditingJumpBatt(nbat);
};

void LoMas::EditingGoTop()
{
	LdMas::EditingGoTop();
};

void LoMas::RequestJump()
{
#ifdef WITH_NET
	LdMas::RequestJump();
#else
	Partitura::RequestJump();
#endif
}

void LoMas::GestLabel()
{
#ifdef WITH_NET
	LdMas::GestLabel();
#else
	Partitura::GestLabel();
#endif
}

// metodo per andare indietro di nb battuta 
void LoMas::RichBattBackward(short nb)
{               
	LdMas::RichBattBackward(nb);
};


// metodo per andare avanti di nb battuta 
void LoMas::RichBattForward(short nb)
{   
	LdMas::RichBattForward(nb);
}


//Richiesta di inserimento comando
void LoMas::NetInvioComando(SymCmd SC)
{
	InvioComandosuRete(SC);
}  


void LoMas::SincrBatt()
{
#ifdef WITH_NET   
	int Altezza, nbat, tsc;
	short int CommandCode;
	long te, tee, NPassi;
	char CommandStringa[DIM_COMMAND_STRING+1];
	char *ack_sincr;
	
	RetCode info_ack;
	NumCode NumBatt, nb, NumFirstBat;
	Battuta *pbt, *pbt2;
	Spartito *psp;
	
	int MRWait;
	
	FILE *pippo;
	
#ifdef DEBUG_EXEC
	pippo = fopen("/home/demo/txtA/DliooExec.txt","a");
	fprintf(pippo,"SincrBatt()\n");
	fclose(pippo);
#endif
	
	
	// calcolo del numero progressivo della battuta attualmente in esecuzione
	strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
	CommandStringa[DIM_COMMAND_STRING] = '\0';
	CommandCode = DecodeCommand(CommandStringa);
	// in commandCode c'e il numero della battuta in formato 1 ... 
	NumBatt = CommandCode;
	
	// invio ack + proprio id di riconoscimento
	ack_sincr=new char[DIM_ACK+1];
	
    ack_sincr=FormaAck(my_ID,Array,'n',ack_sincr);
	
    
	info_ack=SendAck(tid_MASAE, ack_sincr,MSG_ACK_MULTI);
	if (info_ack!=OK_SEND_ACK) 
	{
		EM.Error(NET_ERR_AREA,"EC_NO_ACK","DiMas","SincrBatt()");
		
	}
	delete[] ack_sincr;
	
	
	// ricavo la battuta attualmente in esecuzione
	psp = GetFirstSpa();
	if (psp==NULL)
	{
		EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","LoMas1","SincrBatt()");
		return;	
	}
	
	
	
	pbt = psp->GetBattNumProg(NumBatt);
	if (pbt==NULL)
	{
		
#ifdef DEBUG_EXEC
		pippo = fopen("/home/demo/txtA/DliooExec.txt","a");
		fprintf(pippo,"\nNon trova la battuta %d \n",NumBatt);
		fclose(pippo);
#endif
		
		int BattNumR=NumBatt;
		BattNumR--;
		while(BattNumR>0 && (pbt=psp->GetBattNumProg(BattNumR))==NULL)
			BattNumR--;
		
		if(BattNumR==0)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_BAT_NOTFOUND","LoMas2","SincrBatt()");
			return;	
		}
		
		nb = pbt->GetNumMultiRest();
		MRWait=BattNumR+nb-1-NumBatt;
		NumBatt=BattNumR;
		
	}
	else 
	{
		if(pbt->GetIsMultiRest()) 
		{ 
			nb = pbt->GetNumMultiRest();
			MRWait=nb-1;   
		} 
		else   MRWait=0;	 
	} 
	
	NumPentagrammi = pbt->GetAbsPos().y/pbt->GetHeight();
	NumPentagrammi = psp->GetNPentagrammi() - NumPentagrammi;
	
#ifdef DEBUG_EXEC
	pippo = fopen("/home/demo/txtA/DliooExec.txt","a");
	fprintf(pippo,"\nNumero di Battuta=%d (NumPentagrammi=%d)\n",NumBatt,NumPentagrammi);
	fclose(pippo);
#endif
	
	if (pMusicWin->GetJumpState())
	{
		
		if ((NumPentagrammi>2) && (!ScrollingAvviato))
		{
			pMusicWin->SetScrollingInc(SCR_INC*10);
			pMusicWin->SetScrollingTime(MIN_CALL_TIMER);
			
			pMusicWin->SetcountBatt(NumBatt);
			
			ScrollingAvviato=TRUE;
			
			pMusicWin->SetInitialTime();
			pMusicWin->SetActualTime();
			pMusicWin->AttivaModEsecuzione(psp->GetfirstBatPage()->GetNumProgress());
			pMusicWin->ScrollAll();
			return;
		}
		else if ((NumPentagrammi>=2) || (ScrollingAvviato))
		{
#ifdef DEBUG_EXEC
			pippo = fopen("/home/demo/txtA/DliooLiooCall.txt","a");
			fprintf(pippo,"\n\nSTACCO\n\n");
			fclose(pippo);
			
			pippo = fopen("/home/demo/txtA/DliooExec.txt","a");
			fprintf(pippo,"NumPentagrammi>=2\n");
			fclose(pippo);
#endif
			
			pMusicWin->SetInitialTime();
			pMusicWin->SetActualTime();
			pMusicWin->SetJumpState(FALSE);
		}
		else
		{
			pMusicWin->SetJumpState(FALSE);
			return;
		}
		
	}
	
#ifdef DEBUG_EXEC
	pippo = fopen("/home/demo/txtA/DliooExec.txt","a");
	fprintf(pippo,"Exec 1\n");
	fclose(pippo);
#endif
	
	pMusicWin->SetcountBatt(NumBatt);
	   
	// calcolo, in te,il tempo necessario per eseguire l'attuale riga musicale
	te = GetTimeExec(NumBatt);	// ?? indice ??
	te = ((te * 1000)/(1000 + pMusicWin->GetVelocEsec()));
	
	
	if (te < 1) 
	{
		EM.Error(MUSICAL_ERR_AREA,"EC_TIME_NOTFOUND","LoMas3","SincrBatt()"); 
		return;	
	}
	
	
	te=te+te*MRWait;
	
	
	
	pbt=psp->GetNextBat(pbt);
	if(pbt!=NULL)  nb=pbt->GetNumProgress();
	int i;
	int nbb;
	while (pbt!=NULL)
	{
		if (((pbt->GetBattuta()) != INIZIO_PENTAGRAMMA) && ((pbt->GetBattuta()) != INIZIO_PAGINA) )
		{
			nbb=1;
			tee = GetTimeExec(nb);     
			if(pbt->GetIsMultiRest())
			{
				nbb=pbt->GetNumMultiRest();
				tee*=nbb;
			}   
			tee = ((tee * 1000)/(1000 + pMusicWin->GetVelocEsec()));
			
			if (tee < 1) 
			{
#ifdef DEBUG_EXEC
				pippo = fopen("/tmp/time.log","a");
				fprintf(pippo,"Time=%d\n",tee);
				fclose(pippo);
#endif
				EM.Error(MUSICAL_ERR_AREA,"EC_TIME_NOTFOUND","LoMas4","SincrBatt()"); 
				return;	
			}
			te += tee;
			nb+=nbb;
			pbt = psp->GetBattNumProg(nb);
		}
		
		else
			pbt=NULL;
	}
	
    nb = NumBatt;
    if (nb < 1) return;
    pbt = psp->GetBattNumProg(nb);
	
    while (((pbt->GetBattuta()) != INIZIO_PENTAGRAMMA) && ((pbt->GetBattuta()) != INIZIO_PAGINA) )
    {
		pbt = psp->GetPrevBat(pbt);
		if (pbt==NULL) 
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_BAT_NOTFOUND","LoMas6","SincrBatt()"); 
			return;	
		}
	}  
	
    pbt = psp->GetPrevBat(pbt);
	
    if (pbt==NULL) 
    {
		EM.Error(MUSICAL_ERR_AREA,"EC_BAT_NOTFOUND","LoMas5","SincrBatt()"); 
		return;	
    } 
	
    while (((pbt->GetBattuta()) != INIZIO_PENTAGRAMMA) && ((pbt->GetBattuta()) != INIZIO_PAGINA) )
    {
		pbt = psp->GetPrevBat(pbt);
		if (pbt==NULL) 
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_BAT_NOTFOUND","LoMas6","SincrBatt()"); 
			return;	
		}
	}
	
    pbt = psp->GetPrevBat(pbt);
	
    if (pbt==NULL) 
    {
		EM.Error(MUSICAL_ERR_AREA,"EC_BAT_NOTFOUND","LoMas5","SincrBatt()"); 
		return;	
    } 
	
	
	
	Battuta *pbt3;
	
	
	if (NumPagina == 0)
	{
		pbt3 = psp->GetBattNum(0);
		pbt3 = psp->GetNextBat(pbt3);
		while (pbt3->GetNumPagDLIOO()==0)
		{
			pbt3 = psp->GetNextBat(pbt3);
			if (pbt3==NULL) break;
		}
		
		if (pbt3==NULL) return; // puo' verificarsi solo se non e' ancora partito lo scrolling, oppure c'e' una sola pagina.
		else NumPagina = pbt3->GetNumProgress();
	}  
	
	NumFirstBat = NumPagina;
	
	if (pbt->GetNumProgress() >= NumFirstBat)
	{
		// trovo la precedente battuta di inizio pagina (prima di pbt): se e' diversa da NumFirstBat, allora lo scrolling
		// e' rimasto indietro
		pbt3 = pbt;
		while (pbt3->GetNumPagDLIOO()==0)
		{
			pbt3 = psp->GetPrevBat(pbt3);
			if (pbt3==NULL) break;
		}
		
		if (pbt3==NULL) Altezza = 0;
		else if (pbt3->GetNumProgress() != NumFirstBat) // lo scrolling e' indietro, e scrolla tutta una pagina intera
		{
			Altezza = pMusicWin->GetHeight();
		}		
		else
		{
			Altezza = pbt->GetHeight() + pMusicWin->GetScrollingPosY(); 
		}		
	}
	else
	{
		// trovo la successiva battuta di inizio pagina (dopo pbt): se e' diversa da NumFirstBat, allora lo scrolling
		// e' scappato
		pbt3 = pbt;
		while (pbt3->GetNumPagDLIOO()==0)
		{
			pbt3 = psp->GetNextBat(pbt3);
			if (pbt3==NULL) break;
		}
		
		if (pbt3==NULL) Altezza = 0;
		else if (pbt3->GetNumProgress() != NumFirstBat) // lo scrolling e' scappato, allora si ferma ed attende
		{
			pippo = fopen("/tmp/nump.log","a");
			fprintf(pippo,"scrolling scappato..NUM3=%d\n",pbt3->GetNumProgress());
			fclose(pippo);
			Altezza = 0;
		}		
		else
		{
			Altezza = pMusicWin->GetScrollingPosY() - (pbt->GetAbsPos().y - pbt->GetVU2Dwn());  
		}		
	}
	
	if (Altezza < 0) Altezza = 0;
	
	pMusicWin->SetScrollingInc(SCR_INC);
	NPassi = Altezza/(pMusicWin->GetScrollingInc());
	if (NPassi <= 0) 
	{
		pMusicWin->SetScrollingInc(0);
		tsc = TIME_SCROLLING_CALL_BACK;
	}
	else 
		tsc = te / NPassi;
	
	pMusicWin->SetScrollingTime(tsc);
	
	// dopo un certo numero di righi musicali avvio lo scrolling
	if ((!ScrollingAvviato) && (NumPentagrammi>1))
	{
#ifdef DEBUG_EXEC
		pippo = fopen("/home/demo/txtA/DliooExec.txt","a");
		fprintf(pippo,"Avvio Scrolling %d\n",psp->GetfirstBatPage()->GetNumProgress());
		
		for (i=0;i<31;i++)
			fprintf(pippo,"Battuta %d, Tempo %d\n",i,GetTimeExec(i));
		
		fclose(pippo);
#endif
		
		ScrollingAvviato = TRUE;
		pMusicWin->AttivaModEsecuzione(psp->GetfirstBatPage()->GetNumProgress());
		pMusicWin->ScrollAll();
	}
#else
#endif 
}


int LoMas::GetWaitEditing()
{
	return waitEditing;
}


static Bool InizializzazioneExec = FALSE;
static Bool InizializzazioneImp = FALSE;

void LoMas::ReadFromNetwork()
{
#ifdef WITH_NET   
	int nbat, tsc, veloc, Altezza, indice, i, flag=0;
	unsigned short int time;   
	short int CommandCode;
	float fat, fat2;
	long te, tee, NPassi;
	
	char *ack_sincr;
	char Ack[DIM_ACK+1]; 
	char CommandStringa[DIM_COMMAND_STRING+1];
	
	NumCode InitDel, EndDel;
	
	RetCode info_ack, reccode = NO_MSG;
	Spartito *psp, *psp1;
	Battuta *pbat, *pbat1, *pbt, *pbt2, *pbt3;
	NumCode nc, NumBatt, nb;
	
	FILE *pippo;
	
	// interrogazione dalla rete
	reccode=Receive();
	if (reccode==NO_MSG) return;
	
	// controllo ID ricevuto: pu essere solo quello del MASAE  
	if ((Array[0]-1)!=ID_MASAE) 
	{
		// gestione errore : arrivato un messaggio strano
		EM.Error(NET_ERR_AREA, "EC_INVALID_MASAE", "LoMas", "ReadFromNetwork()" );
		// Usato l'Array, lo pulisco 
		Array[0] = '\0'; 
		return;
	}
	
	// formazione Ack  : Vedi struttura Ack in Network.
	FormaAck(my_ID,Array,'p',Ack);
	
	// decodifica tipo di messaggio ricevuto
	strncpy(CommandStringa,Array+DIM_ID,DIM_COMMAND_STRING); 
	CommandStringa[DIM_COMMAND_STRING] = '\0';
	CommandCode = DecodeCommand(CommandStringa);
	
#ifdef DEBUG_EXEC
	pippo = fopen("/home/demo/txtA/DliooExec.txt","a");
	fprintf(pippo,"ReadFromNetwork() - CommandCode = %d\n",CommandCode);
	fclose(pippo);
#endif
	
	//
	//  ------------  gestione messaggio ricevuto ------------------------
	//
	switch (CommandCode)
	{
		
    case (JUMP_TO_BATT):
		
		pMusicWin->SetJumpState(TRUE);
		strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
		NumBatt = DecodeCommand(CommandStringa);
		
		
		miaPartitura->GoBatt(NumBatt);
		
		psp = GetFirstSpa();
		pbt3 = psp->GetBattNumProg(NumBatt);
		pbt3 = psp->GetNextBat(pbt3);
		
		if (pbt3!=NULL) 
		{ 
			while (pbt3->GetNumPagDLIOO()==0)
			{
				pbt3 = psp->GetNextBat(pbt3);
				if (pbt3==NULL) break;
			}
		}
		
		if (pbt3==NULL) 
		{   
			NumPagina = NumBatt; 
		}
		else NumPagina = pbt3->GetNumProgress();
		
		// invio dell ACK       
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LoMas","CommandProcess()");
		break;
		
	case (DELETE_BATTUTE):
		indice=DIM_ID+DIM_COMMAND_STRING;
		strncpy(CommandStringa,Array+indice,DIM_COMMAND_STRING); 
		CommandStringa[DIM_COMMAND_STRING] = '\0';
		InitDel = DecodeCommand(CommandStringa);   
		indice+=DIM_COMMAND_STRING;
		
		strncpy(CommandStringa,Array+indice,DIM_COMMAND_STRING); 
		CommandStringa[DIM_COMMAND_STRING] = '\0';
		EndDel = DecodeCommand(CommandStringa);
		psp=GetFirstSpa();
		
		
		// controllo per eliminare la battuta vuota quando cancello tutte le battute presenti
		NumBatt = psp->GetNumeroBattuteVere();
		
		// il caso > non dovrebbe esistere (ma a volte si verifica)
		if ((EndDel-InitDel+1)>0)
		{
			if ((EndDel-InitDel+1)>=NumBatt) 
				flag=1; 
			else 
				flag=0;
		}
		else 
			flag=0;
		
		
		
		for (i=InitDel;i<=EndDel;i++)
		{	  
			pbat= psp->GetBattNumProg(i);
			if (pbat!=NULL) psp->DeleteBat(pbat);
		} 
		
		if (flag==1) GetFirstSpa()->Free();
		//chiedere al Biagini perche' questa riga e' cosi' importante!!!!!!!
		
		// invio dell ACK       
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
		{
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LoMas","ReadFromNetwork()");
		}
		break; 
		
    case (EXEC_ENDBATTUTE):
		//  settaggio flag per controllo impaginazione 
		SetImpaginazione(END_BATT_ESEC);
		
		// invio dell ACK       
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
		{
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LoMas","ReadFromNetwork()");
		}
		break;
		
	case (ALT_EXEC):
		NumPentagrammi = 0;
		ScrollingAvviato = FALSE;
		
		// aggiornamento stato di scrolling          
		RichBatt=NO_BACKWARD;
		
		// settaggio stato di impaginazione
		SetImpaginazione(END_IMP);
		
		pMusicWin->SetScrollingPosY(pMusicWin->GetScrollingInc() - 1);
		
		
		pMusicWin->DisattivaModEsecuzione();
		// invio dell ACK       
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
		{
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LoMas","ReadFromNetwork()");
		}
		break;
		
    case (FINE_EXEC):
		// aggiornamento stato di scrolling          
		RichBatt=NO_BACKWARD;
		
		// aggiornamento stato di impaginazione
		SetImpaginazione(END_IMP);
		
		// aggirnamento flag di stato di esecuzione
		pMusicWin->DisattivaModEsecuzione();
		
		// invio dell ACK       
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
		{
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LoMas","ReadFromNetwork()");
		}
		break;
		
    case (INIT_EXEC):
		pMusicWin->SetJumpState(FALSE);
		NumPagina = 0;
		
		New();      
		// determinazione numero di battute presenti nell'opera
		strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
		nbat = DecodeCommand(CommandStringa);
		// costruzione vettore ett 
		BuildETT(nbat);
		
		// settaggio stato di impaginazione  
		SetImpaginazione(BEGIN_ESEC);
		
		GoTop();
		GetFirstSpa()->Free();
		pMusicWin->SetVelocEsec(0);
		
		// aggiornamento flag di stato di esecuzione
		InizializzazioneExec = TRUE;
		NumPentagrammi = 0;
		ScrollingAvviato = FALSE;
		
		// invio dell ACK       
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
		{
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LoMas","ReadFromNetwork()");
		}
		break;
		
    case (ESEGUI_BATT_EXEC):    
		struct timeval timevI, timevF;
		struct timezone timezI, timezF;
		double timeD;
		short int time1,time2;
		
		// aggiornamento stato di attesa per impaginazione  
		if ((Array[DIM_ID+DIM_COMMAND_STRING])!=BATTUTE_CONT)
			wait = FALSE;
		else
			wait = TRUE;
		
		// lettura durata della battuta
		strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING+1,DIM_COMMAND_STRING);
		time1 = DecodeCommand(CommandStringa);
		strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING+DIM_COMMAND_STRING+1,DIM_COMMAND_STRING);
		time2 = DecodeCommand(CommandStringa);
		time=time2+(time1*256);
		
		// determinazione della battuta in questione 
		strncpy(CommandStringa,Array + DIM_ID + 3*DIM_COMMAND_STRING + 1, DIM_COMMAND_STRING);
		nbat = DecodeCommand(CommandStringa);
		
		// settaggio nel vettore ett della durata della battuta
		SetTimeExec(nbat + 1, time);
		
		// caricamento della battuta
		LoadS(DIM_ID+(3*DIM_COMMAND_STRING)+1);
		
		// Usato l'Array, lo pulisco 
		Array[0] = '\0'; 
		
		// invio dell ACK       
		info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
		if (info_ack!=OK_SEND_ACK) 
		{
			EM.Error(NET_ERR_AREA,"EC_NO_ACK","LoMas","ReadFromNetwork()");
		}
		
		// controllo possibilita di impaginazione
		if ((!(wait)) && (InizializzazioneExec))
		{
			InizializzazioneExec = FALSE;
			GoTop();
			Draw();
		}
		
		{
			
			Battuta *pbat=GetFirstSpa()->GetBattNumProg(nbat+1);
			if(pbat!=NULL && pbat->GetIsMultiRest())
			{
				FILE *pippo=fopen("/tmp/mb.log","a");
				fprintf(pippo,"  BATTUTA MULTI REST: n. %d pag. %d \n",pbat->GetNumMultiRest(),pbat->GetNumPagDLIOO());
				fclose(pippo);
			} 
		} 
		break;
		
		case (SINCR_BATT):
			SincrBatt();   
			break; 
			
		case (IMPAGINA_ENDBATTUTE):
			// settaggio stato di impaginazione  
			SetImpaginazione(END_BATT_IMP_FIRST);
			
			// invio dell ACK       
			info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
			if (info_ack!=OK_SEND_ACK) 
			{
				EM.Error(NET_ERR_AREA,"EC_NO_ACK","LoMas","ReadFromNetwork()");
			}
			break;
			
		case (FINE_IMPAGINA):
			// aggiornamento stato di scrolling          
			RichBatt=NO_BACKWARD;
			
			// settaggio stato di impaginazione
			SetImpaginazione(END_IMP);
			
			// settaggio stato di esecuzione in Dlioowin
			pMusicWin->DisattivaModEsecuzione();
			
			// invio dell ACK       
			info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
			if (info_ack!=OK_SEND_ACK) 
			{
				EM.Error(NET_ERR_AREA,"EC_NO_ACK","LoMas","ReadFromNetwork()");
				
			}
			break;
			
		case (INIT_IMPAGINA):
			//  Setto lo stato di inizio impaginazione :  Impaginazione=BEGIN_IMP 
			pMusicWin->SetJumpState(FALSE);
			SetImpaginazione(BEGIN_IMP);
			
			// Setto il tempo fra una chiamata e l'altra e la Porzione (in pixel) da copiare 
			// ogni volta  (per lo scrolling)  
			pMusicWin->SetScrollingTime(30);
			pMusicWin->SetScrollingInc(50);
			// inizializzo il flag di stato per l'impaginazione 
			InizializzazioneImp = TRUE;
			
			New();      
			GoTop();
			GetFirstSpa()->Free();
			pMusicWin->AttivaModEsecuzione();
			
			// invio dell ACK       
			info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
			if (info_ack!=OK_SEND_ACK) 
				EM.Error(NET_ERR_AREA,"EC_NO_ACK","LoMas","ReadFromNetwork()");
			break;
			
		case (INIT_PARTE):
			
			// aggiornamento stato di scrolling 
			RichBatt=NO_BACKWARD;
			
			// lettura numericcode
			strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING,DIM_COMMAND_STRING); 
			CommandStringa[DIM_COMMAND_STRING] = '\0';
			CommandCode = DecodeCommand(CommandStringa);
			
			psp1 = GetFirstSpa();
			psp1->SetNumericCode(CommandCode);
			
			psp1=GetFirstSpa();
			GetFirstSpa()->Free();   
			
			// invio dell ACK       
			info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
			if (info_ack!=OK_SEND_ACK) 
			{
				EM.Error(NET_ERR_AREA,"EC_NO_ACK","LoMas","ReadFromNetwork()");
				
			}
			break; 
			
		case (END_INIT_PARTE) :
			Redraw();          
			
			// Usato l'Array, lo pulisco 
			Array[0] = '\0';
			
			// invio dell ACK       
			info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
			if (info_ack!=OK_SEND_ACK) 
			{
				EM.Error(NET_ERR_AREA,"EC_NO_ACK","LoMas","ReadFromNetwork()");
				
			}
			break;  
			
		case (ESEGUI_BATT_IMP):
			// aggiornamento stato di attesa per impaginazione  
			if ((Array[DIM_ID+DIM_COMMAND_STRING])==BATTUTE_COMPLETATE)
				wait = FALSE;
			else if ((Array[DIM_ID+DIM_COMMAND_STRING])==BATTUTE_TERMINATE)
			{
				wait = FALSE;
				SetImpaginazione(END_BATT_IMP_FIRST);
			}
			else
				wait = TRUE;
			
			LoadS(DIM_ID+DIM_COMMAND_STRING+1);  
			
			// Usato l'Array, lo pulisco 
			Array[0] = '\0'; 
			
			// invio dell ACK       
			info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
			if (info_ack!=OK_SEND_ACK) 
			{
				EM.Error(NET_ERR_AREA,"EC_NO_ACK","LoMas","ReadFromNetwork()");
				
			}
			
			// se e' terminata la trasmissione delle N battute(wait) e 
			// siamo ancora in impaginazione (InizializzazioneImp)
			if ((!(wait)) && (InizializzazioneImp))
			{
				InizializzazioneImp = FALSE;
				pMusicWin->AttivaModEsecuzione();
				pMusicWin->ScrollAll();
			}
			break;
			
		case (ESEGUI_BATT_LOAD):
			LoadS(DIM_ID+DIM_COMMAND_STRING);
			
			// Usato l'Array, lo pulisco 
			Array[0] = '\0'; 
			// invio dell ACK       
			info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
			if (info_ack!=OK_SEND_ACK) 
			{
				EM.Error(NET_ERR_AREA,"EC_NO_ACK","LoMas","ReadFromNetwork()");
				
			}
			break;
			
		case (ESEGUI_NEW) :      
			// Usato l'Array, lo pulisco 
			Array[0] = '\0';
			
			// invio dell ACK       
			info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
			if (info_ack!=OK_SEND_ACK) 
			{
				EM.Error(NET_ERR_AREA,"EC_NO_ACK","LoMas","ReadFromNetwork()");
				
			}
			New();      
			GoTop();
			break;
			
		case (ESEGUI_COMMAND): 
			EseguiCommand(NetSymCmd); 
			
			// Usato l'Array, lo pulisco 
			Array[0] = '\0';
			
			// invio dell ACK       
			info_ack = SendAck(tid_MASAE,Ack,MSG_ACK);
			Redraw();
			if (info_ack!=OK_SEND_ACK) 
			{
				EM.Error(NET_ERR_AREA,"EC_NO_ACK","LoMas","ReadFromNetwork()");
				
			}
			
			break;     
			
		case (TYPE_CURSOR):
			LdMas::ReadFromNetwork();
			Draw();
			break;
			
		default: 
			LdMas::ReadFromNetwork();
			break;
  }  
#else
#endif 
}


//
//  -----------------------------  funzioni di utilita'  -------------------------------------
//
void LoMas::EseguiDebug(void *arg1, void *arg2, void *arg3) 
{
	FILE *pippo;
	
	Spartito  *psp;
	Battuta *pbt;
	
	int nb;
	
	switch(UsrSymCmd.CommandId)
	{
    case CMD_DEBUG_UNO:
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Lomas","ReadFromNetwork()");
		break;
		
		
    case CMD_DEBUG_DUE:
		
		break;
		
		
    case CMD_DEBUG_TRE:
		break;
		
    case CMD_DEBUG_QUATTRO:
		break;        
    case CMD_DEBUG_CINQUE:
		
		
		break;        
    case CMD_DEBUG_SEI:
		
		pippo = fopen("/home/demo/txt/finestrDlioo.txt","w");
		fprintf(pippo,"\n ------------   Finestra DLIOO ----------------\n\n\n");
		fclose(pippo);
		
		nb=0;
		psp=GetFirstSpa();
		pippo = fopen("/home/demo/txt/finestrDlioo.txt","a");
		fprintf(pippo,"1\n");
		fclose(pippo);
		pbt = psp->GetBattNum(nb);
		pippo = fopen("/home/demo/txt/finestrDlioo.txt","a");
		fprintf(pippo,"2\n");
		fclose(pippo);
		
		pippo = fopen("/home/demo/txt/finestrDlioo.txt","a");
		while(pbt!=NULL)
		{
			psp=GetFirstSpa();
			fprintf(pippo,"Battuta %d:",nb);
			while(psp!=NULL)
			{
				pbt = psp->GetBattNum(nb);
				fprintf(pippo,"\t%d\t%d\t%d",pbt->GetNumProgress(),pbt->GetNumericCode(),GetTimeExec(pbt->GetNumProgress()));
				psp=GetNextSpa(psp);
			}
			
			fprintf(pippo,"\n");
			nb++;
			psp=GetFirstSpa();
			pbt = psp->GetBattNum(nb);
		}
        
		fclose(pippo);
		break;        
		
        
		break;        
    case CMD_DEBUG_SETTE:
		
        break;        
		
    case CMD_DEBUG_OTTO:
        break;
		
    case CMD_DEBUG_NOVE:
        break;
		
	}
	
}  
void LoMas::InitEsecuzione(char *nome)
{ 
	// leggo il file ett associato a quella opera
	InitTimeExec();
	if(nome!=NULL)
		ReadETT(nome);
	
	if ((ETTLenght-1) != GetFirstSpa()->GetNumeroBattute())
	{
		Message("Attenzione ETT non conforme al brano in memoria. Inizializzazione fallita.");
		return;
	}
	// posiziono all'inizio della partitura 
	GoTop();
	// setto il flag di stato per l'impaginazione
	SetImpaginazione(BEGIN_ESEC);  
	
}
void LoMas::InitImpaginazione()
{
	//  Posiziona all'inizio della partitura.
	GoTop();
	
	//  Costruisco il vettore dell'ETT  della dimensione dello spartito
	BuildETT(GetFirstSpa()->GetLastBat()->GetNumProgress() + 1);
	
	//  Resetto gli attributi del'impaginazione di ogni battuta
	Spartito *psp1=GetFirstSpa();
	while (psp1!=NULL)
	{
		psp1->InitImpagina();
		psp1 = GetNextSpa(psp1);
	}
	
	//  Setto lo stato di inizio impaginazione :  Impaginazione=BEGIN_IMP 
	SetImpaginazione(BEGIN_IMP);
	
	// Setto il tempo fra una chiamata e l'altra e la Porzione (in pixel) da copiare 
	// ogni volta  (per lo scrolling) e la modalita di esecuzione
	pMusicWin->SetScrollingTime(1);
	pMusicWin->SetScrollingInc(200);
	pMusicWin->AttivaModEsecuzione();
	pMusicWin->ScrollAll();
	
} 


