/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: mamass.cpp,v 1.1 2005/07/25 16:31:41 bellinpo Exp $ */



// Parte di codice inserito per fare un test sulla velocita' di invio di un
//                              comando
#include <sys/types.h>
#include <sys/timeb.h>

#include <string.h>
#include "net/mamass.hpp"
#include "net/def_net.hpp"
#include "liooext.hpp"

extern LogFile MyLog;
extern ExtLiooAbil liooAbil;

#define NOME_OPD "opd"

MaMass::MaMass()
{   
	
#ifdef WITH_NET    
	char menu[MAX_STR_ABIL_LEN];
	char *taskname = NOME_OPD;
	int tids[4];
	int infospawnopd;
	char Codice[DIM_COMMAND_STRING+1];  
	RetCode infoCode, sendCode;
	
	my_ID = 0;   
	
	// Inizializzazione della tabella della configurazione
	netONCM.InitTabella(); 
	
	// carico la tabella della configurazione
	netONCM.LoadTabella(); 
	
	// Inizializzazione della tabella delle parti 
	netONCM.InitPindex();
	
	// Avvio dei processi DLIOO sui vari Host della rete secondo la configurazione
	// del file NetStart.cfg creato da ONCM 
	StartNodes(); 
	
	// costruzione del vettore dei tid dei LIOO per il multicast  
	netONCM.BuildArrayOfTid(); 
	
#ifndef TINYMOODS
	// lancia l'OPD su mamass solo se ci sono dei leggi attivi
	if(netONCM.NActiveLIOO>1)
	{
		infospawnopd = pvm_spawn( taskname, 0, PvmTaskHost, ".", 1, &tids[0]);
		OPD_TID=tids[0];
		OPD_ID=netONCM.NActiveLIOO;
		
		// per l'invio del OPD_ID all'OPD
		Array[0]=my_ID+1;
		Array[1]='\0';
		infoCode = CodeCommand((short int)INIZIALIZZAZIONE,Codice);
		strcat(Array,Codice);  
		Array[DIM_ID+DIM_COMMAND_STRING]=OPD_ID;
		Array[DIM_ID+DIM_COMMAND_STRING+1]='\0';	 
		sendCode = SendMsg_ack_MaM(OPD_TID);
		if (sendCode!=OK_SEND) 
		{
			// problemi di invio messaggio
			EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","MaMass()");
		}
	}
#endif TINYMOODS
#endif WITH_NET
}


MaMass::~MaMass()
{
#ifdef DEBUG_FREE
#endif 
	
}

char * MaMass::GetMusicista(int Numseq) 
{ 
	if (Numseq<=(netONCM.NActiveLIOO-1)) return netONCM.ListofNodes[Numseq].nome_player;
	else return NULL;
}


void MaMass::RequestJump()
{
	Partitura::RequestJump();
}

void MaMass::GestLabel()
{
	
	char label[2];
	int i;
	int NumEtich=GetNumEtichette();
	char* arrayLabel;
	
	arrayLabel=new char [NumEtich+1];
	
	// creazione vettore delle label
	for (i=0;i<NumEtich;i++)
	{
		arrayLabel[i]=GetEtichetta(i);
	} 
	
	label[1]='\0';
	if (WReadEtich("Labels",label,200, NumEtich,arrayLabel)) 
	{
		label[1]='\0';
		
		pMusicWin->SetCommand(CMD_BATTUTA_LETTERA,IMMEDIATE_SEL,BATTUTA_SEL,0,0,0,0,0,label);
		
	}
	else
		Scr->GSetMainDrawArea();
	
	delete arrayLabel;
	
}



void MaMass::MenuJump(int NumBatt,int NumEtich,char *arrayLabel,int* arrayNumProg)
{
	int nbat;
	char buffer[200];
	if (EDWidgetJump("Jump in Score",buffer,200,NumEtich ,NumBatt,arrayLabel,arrayNumProg))
	{ 
		if (!(strcmp(buffer,"TOP")))
		{
			GoTop();
		}  
		else if (!(strcmp(buffer,"BOT."))) 
        {
			GoBottom();
		} 
		else {   
			nbat = atoi(buffer);  // trova la battuta in forma numerica (NumericCode)
			GoBatt(nbat);
		} 
	}		
#ifdef WITH_NET   
#else
#endif 
}

int MaMass::ControlloLimiti(unsigned char id_mitt,int limite)
{ 
	Battuta *pbat, *pcbat;
	Spartito *psp;
	int nb;
	
	if (netONCM.CheckDirector(id_mitt))
	{
		return limite;
	}
	
	psp=psp = GetSpartNum(netONCM.ListofNodes[id_mitt].IDP-1);
	pbat=psp->GetBattNumProg(limite);
	
	if(pbat!=NULL)
	{ 
		if(pbat->GetIsMultiRest())
		{ 
			// una battuta che fa parte di una multirest
			if(pbat->GetPMultiRest()!=NULL)
			{
				// la prima battuta della serie
				pcbat=pbat->GetPMultiRest();
				if(pcbat!=NULL)
				{
					nb=pbat->GetNumMultiRest();
					return pbat->GetNumProgress()+nb;   
				} 
			}
			else
			{
				//non  la prima battuta perci devo tornare indietro
				while(pbat->GetPMultiRest()==NULL)
				{
					pbat=psp->GetPrevBat(pbat);
				}
				
				pcbat=pbat->GetPMultiRest();
				if(pcbat!=NULL)
				{
					nb=pcbat->GetNumMultiRest();
					return pbat->GetNumProgress()+nb;
				} 
			}
		}
		else
		{
			return limite;
		}
	}
	return 0; //!
}

void MaMass::JumpEditing(int N,unsigned char id_mitt)
{
#ifdef WITH_NET   
	RetCode infoCode, sendCode;
	char Codice[DIM_COMMAND_STRING+1];
	Spartito* psp;
	Battuta* PBattuta;
	int i,tid_mittente;
	unsigned long MAX;
	int x,y,SX,DX;
	int StartInvio, EndInvio, StartDel, EndDel;
	int nb;
	Battuta *pcbat;
	
	//    Sul Masae determino x ed y sapendo che voglio la N:
	//    x=nuovo margine di visualizzazione sinix che voglio avere
	//    y=nuovo margine di visualizazione destro che voglio avere
	
	psp=GetFirstSpa();
	
	MAX=psp->GetNumeroBattute();
	
	x = N - 10;
	if (x < 1) x = 1;
	
	y = N + NUM_BAT;
	if (y > MAX) y = MAX;
	
	// confronto x ed y con i margini di visualizzazione attuali   
	
	SX=netONCM.GetSxLimit(id_mitt)+1;  // vero margine di visuaizzazione Sx	 	     
	DX=netONCM.GetDxLimit(id_mitt);    // vero margine di visualizzazione Dx	
	
	
	// Controllo che battute devo cancellare sul leggio chiamante
	// e quali devo inviare dal MASAE
	
	// StartInvio    Prima battuta da inviare;
	// EndInvio      Ultima battuta da inviare  
	// StartDel      Prima battuta che il leggio deve cancellare
	// EndDel        Ultima battuta che il leggio deve cancellare
	
	// Quando StartDel ed EndDel sono 0 significa che non devono essere
	// cancellate battute           
	
	
	
	// Aggiorno i valori di x e y verificando se le battute ad esse associate
	// sono di pausa e fanno parte di una battuta multirest.
	
	x=ControlloLimiti(id_mitt,x);
	y=ControlloLimiti(id_mitt,y);
	
	// devo settare i margini di visualizzazione: da x ad y
	
	netONCM.SetSxLimit(id_mitt,x-1);
	netONCM.SetDxLimit(id_mitt,y);
	
	if ((DX < x) && (x < y))
	{
		
		StartInvio = x;
		EndInvio   = y;
		StartDel   = SX;
		EndDel = ControlloLimiti(id_mitt,DX)-1;
	}	
	else if ((x == DX) && (x < y))
	{
		StartInvio = ControlloLimiti(id_mitt,x)+1;
		EndInvio   = y;
		StartDel   = SX;
		EndDel     = ControlloLimiti(id_mitt,DX)-1;
	}	
	else if ((SX < x) && (x < DX) && (DX < y))
	{
		StartInvio = DX+1;
		EndInvio   = y;
		StartDel   = SX;
		EndDel     = x-1;
	}	
	else if ((SX < x) && (x < DX) && (DX == y))
	{
		StartInvio = 0;        //////////////////
		EndInvio   = 0;        ////////////////// 
		StartDel   = SX;
		EndDel     = x-1;
	}	
	else if ((SX < x) && (x < y) && (y < DX))
	{
		StartInvio = 0;    ////////////////
		EndInvio   = 0;    ////////////////
		StartDel   = SX;
		EndDel     = x-1;
		
		netONCM.SetDxLimit(id_mitt,DX);
	}	
	else if ((SX == x) && (x < y) && (y < DX))
	{
		StartInvio = 0;   /////////////////
		EndInvio   = 0;   ////////////////
		StartDel   = y+1;
		EndDel     = DX;
	}	
	else if ((SX == x) && (x < y) && (y == DX))
	{
		StartInvio = 0;   /////////////////
		EndInvio   = 0;   ////////////////
		StartDel   = 0;
		EndDel     = 0;
	}	
	else if ((x < SX) && (SX < y) && (y < DX))  
	{
		StartInvio = x;
		EndInvio   = SX-1;
		StartDel   = y+1;
		EndDel     = DX;
	}	
	else if ((x < SX) && (x < y) && (y == SX))
	{
		StartInvio = x;
		EndInvio   = SX-1;
		StartDel   = SX+1;
		EndDel     = DX;
	}	
	else if ((y < SX) && (x < y))
	{
		StartInvio = x;
		EndInvio   = y;
		StartDel   = SX;
		EndDel     = DX;
	}	
	else if ((x < SX) && (DX < y))
	{
		StartInvio = x;
		EndInvio   = SX-1;
		StartDel   = 0;
		EndDel     = 0;
		netONCM.SetDxLimit(id_mitt,DX);
	}	
	else if ((x == SX) && (DX < y))
	{
		StartInvio = DX+1;
		EndInvio   = y;
		StartDel   = 0;
		EndDel     = 0;
	}	
	else if ((x < SX) && (DX == y))
	{
		StartInvio = x;
		EndInvio   = SX-1;
		StartDel   = 0;
		EndDel     = 0;
	}	
	
	// Acquisizione  TID del mittente    
	tid_mittente = netONCM.FindPTID(id_mitt);
	
	
	// CANCELLAZIONE BATTUTE
	if ((StartDel!=0) && (EndDel!=0))
	{
		// devo creare il messaggio per dire che battute cancellare   
		Array[0]=(char)(my_ID+1);
		Array[1]='\0';
		
		infoCode = CodeCommand((short int)DELETE_BATTUTE,Codice);
		strcat(Array,Codice);
		
		infoCode = CodeCommand((short int)StartDel,Codice);
		strcat(Array,Codice);
		
		infoCode = CodeCommand((short int)EndDel,Codice);
		strcat(Array,Codice);
		
		// invio comando di cancellazione battute          
		sendCode = SendMsg_ack_MaM(tid_mittente);
		if (sendCode!=OK_SEND) 
		{
			// problemi di invio messaggio
			EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","JumpEditing()");
		}  
	} 
	
	
	
	if ((StartInvio!=0) && (EndInvio!=0))
	{
		// formazione header del comando da inviare
		char header_command[2*DIM_COMMAND_STRING+DIM_ID+2];
		infoCode = CodeCommand((short int)ESEGUI_BATT,Codice);
		header_command[0]=(char)(my_ID+1);
		header_command[1]='\0';
		strcat(header_command,Codice);
		
		if (netONCM.CheckDirector(id_mitt))
		{
			// E' un direttore ! vuole tutta la colonna di spartiti ....        
			for ( i=StartInvio;i<=EndInvio;i++)
			{  
				Array[0]='\0';
				strcat(Array,header_command);
				
				psp=GetSpartNum(0);
				while (psp!=NULL)
				{
					PBattuta=psp->GetBattNumProg(i);
					if (PBattuta!=NULL) 
					{
						strcat(Array,psp->Describe(PBattuta));					   	
					}
					psp=GetNextSpa(psp);
				}
				
				sendCode = SendMsg_ack_MaM(tid_mittente);
				if (sendCode!=OK_SEND) 
				{
					// problemi di invio messaggio
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","JumpEditing()");
				}
			}
		}
		else 
		{
			// non e' un direttore
			psp = GetSpartNum(netONCM.ListofNodes[id_mitt].IDP-1);
			
			if (psp==NULL)
			{
				EM.Error(MUSICAL_ERR_AREA, "EC_SPA_NOTFOUND","MaMass","JumpEditing()");
				// gestione errore non trovato lo spartito
				return;
			}
			
			for ( i=StartInvio;i<=EndInvio;i++)
			{
				// MaMass cerca la battuta  nello spartito  selezionato
				PBattuta=psp->GetBattNumProg(i);      
				Array[0]='\0';
				strcat(Array,header_command);
				
				nb=1;
				if (PBattuta!=NULL) // se la battuta esiste 
				{         
					if(PBattuta->GetIsMultiRest())
					{
						//  una battuta che fa parte di una multirest...
						// invio allora direttamente
						// la battuta multirest che mi sono creata al caricamento.
						
						pcbat=PBattuta->GetPMultiRest();
						if(pcbat!=NULL)
						{
							nb=pcbat->GetNumMultiRest();
							strcat(Array,psp->Describe(pcbat)); 
							i+=nb;
						}
					} 
					else
					{
						strcat(Array,psp->Describe(PBattuta)); 
					} 
					
					
					// invio battuta
					sendCode = SendMsg_ack_MaM(tid_mittente);
					if (sendCode!=OK_SEND) 
					{
						// problemi di invio messaggio
						EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","JumpEditing()");
						//return FALSE;  
					};
				}	
			} 
		} 
		
	} 
	
	
	// adesso invio un messaggio per sbloccare il wait che controlla la possibilita' di spostartsi all'interno del brano
	// sui diversi leggii
	
	Array[0]=(char)(my_ID+1);
	Array[1]='\0';
	infoCode = CodeCommand((short int)OFF_WAIT_MOVE,Codice);
	strcat(Array,Codice);
	
	// invio messaggio
	
	sendCode = SendMsg_ack_MaM(tid_mittente);
	if (sendCode!=OK_SEND) 
	{
		// problemi di invio messaggio
		EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","JumpEditing()");
	};
	
#else
#endif 
}


void MaMass::New(void)
{
	Partitura::New();
	MyLog.EliminaLista();
	return;
}


Bool MaMass::DoCommand(const Point& p)
{
	SymCmd TmpCmd;
	CopiaCommandStruct(&TmpCmd,&UsrSymCmd);
	
	
	if (Partitura::DoCommand(p)) 
	{
		
		InserisciComandoACLMaMass(&UsrSymCmd);
		
		// provo a colorare
		DrawObject * CurObj;
		Bool InPage;
		
		if (UsrSymCmd.CommandId!=CMD_FIGURA_SEELINK && CatchFigure(&UsrSymCmd,(DrawObject *&)CurObj,&InPage)) 		
			if (CurObj!=NULL) 
			{
				
				CurObj->SetColor((ColorIcon)MyLog.GetColor(&UsrSymCmd,0,False,True));
				if (InPage) CurObj->Draw();
				// Forse e' preferibile ridisegnare tutta la schermata qui
				// anziche' all'esecuzione del comando !
			} 
			ResetPath(UsrSymCmd);
			CopiaCommandStruct(&UsrSymCmd,&TmpCmd);
			
			return TRUE;
	}
	
	CopiaCommandStruct(&UsrSymCmd,&TmpCmd);
	return FALSE;
}


Bool MaMass::DoCommandRectSel(const Point& p)
{
	if (Partitura::DoCommandRectSel(p))
	{
		InserisciComandoACLMaMass(&UsrSymCmd);
		// provo a colorare
		DrawObject * CurObj;
		Bool InPage;
		
		if (CatchFigure(&UsrSymCmd,(DrawObject *&)CurObj,&InPage))
			if (CurObj!=NULL)
			{
				CurObj->SetColor((ColorIcon)MyLog.GetColor(&UsrSymCmd,0,False,True));
				if (InPage) CurObj->Draw();
			}
			ResetPath(UsrSymCmd);
			return TRUE;
	}
	return FALSE;
}


Bool MaMass::CatchFigure(SymCmd * MyCmd,DrawObject *& RetObj,Bool * InPage)
{
	Battuta *pBat;
	DrawObject *pObj;
	Bool Appo;
	
	Appo=MyLog.CatchFigure(MyCmd,(Battuta *&)pBat,(DrawObject *&)pObj);
	if (pObj!=NULL) RetObj=pObj;
	else RetObj=(DrawObject *&)pBat;
	
	if(RetObj==NULL)
		return Appo;
	
	
	//  Verifico se e' nella pagina corrente !
	Spartito *pSp = (Spartito *)pMusicWin->PPartit->GetFirstSpa();
	if (pSp!=NULL) {
		Battuta *pBat1=pSp->GetfirstBatPage();
		Battuta *pBat2=pSp->GetfirstBatNextPage();
		if ((pBat1==NULL) || (pBat2==NULL)) *InPage=True;
		else 
			if ((pBat->GetNumProgress()>=pBat1->GetNumProgress()) && (pBat->GetNumProgress()<pBat2->GetNumProgress())) 
			             *InPage=True;
			else 
				*InPage=False;
	} else *InPage=False;
	
	return Appo;
}


void MaMass::KillLioo()
{
#ifdef WITH_NET   
	RetCode infoCode;
	char Codice[DIM_COMMAND_STRING+1];
	
	// costruzione ed invio messaggio
	Array[0]=my_ID+1;
	Array[1]='\0';
	infoCode = CodeCommand((short int)KILL,Codice);
	strcat(Array,Codice);  
	
	infoCode=SendMulticast(netONCM.ArrayOfTid,(netONCM.NActiveLIOO-1));
	if (infoCode!=OK_SEND) 
	{
		// problemi di invio messaggio
		EM.Error(NET_ERR_AREA,"EC_NO_MULTICAST","MaMass","KillLioo()");
	};  
#else
#endif 
}


// metodo per sincronizzare lo scorrimento delle battute in fase di esecuzione  
// usando l'invio multicast , dopo il quale ci mettiamo in attesa degli ack   
// messaggio del tipo
//                 IDMASAE - SINCR_BATT - battuta attuale
void MaMass::SincrBatt(NumCode num_bat)
{
#ifdef WITH_NET   
	RetCode infoCode;
	char Codice[DIM_COMMAND_STRING+1];
	char* Ack_atteso=new char[DIM_ACK+1];
	Battuta *pbt;
	RetCode reccode;
	unsigned char id_ack_ricevuto;
	
	if ((Ack_atteso)==NULL)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","MaMass","SincrBatt()");
	
	// costruzione ed invio messaggio
	Array[0]=my_ID+1;
	Array[1]='\0';
	infoCode = CodeCommand((short int)SINCR_BATT,Codice);
	strcat(Array,Codice);  
	infoCode = CodeCommand(num_bat,Codice);  // valore della battuta
	strcat(Array,Codice);
	strncpy(Ack_atteso,Array,DIM_ACK);
	
	// reset dei flag di ack per i LIOO che attendono il messaggio  
	netONCM.ResetAllE_Ack();
	
	// invio battuta
	infoCode=SendMulticast(netONCM.ArrayOfTid,(netONCM.NActiveLIOO-1));
	if (infoCode!=OK_SEND) 
	{
		// problemi di invio messaggio
		EM.Error(NET_ERR_AREA,"EC_NO_MULTICAST","MaMass","SincrBatt()");
	};  
	
	// blocco per la ricezione degli ack del multicast  
	reccode = NO_MSG;
	while (netONCM.CheckE_Ack()==FALSE)
	{ 
		while (reccode==NO_MSG) 
			reccode=ReceiveAckMulticast(Ack_atteso,id_ack_ricevuto); 
		reccode = NO_MSG;
		netONCM.SetE_Ack(id_ack_ricevuto);
	}
	delete[] Ack_atteso;
	
	//
	// Controllo a quali leggii deve essere inviata una nuova pagina rispetto alla battuta corrente   
	//
	Spartito* psp;
	char* nome_parte;
	unsigned char id,idleader,indice_parte;
	
	struct timeval timevI, timevF;
	struct timezone timezI, timezF;
	double time;
	
	//----------   controllo per i direttori ---------------------
	
	
	
	
	
	psp=GetFirstSpa();
	pbt = psp->GetBattNum(num_bat);
	if (pbt==NULL)
	{
		return ;
	}
	
	if (pbt->GetNumPagMASE())
	{ // invio ai direttori la pagina successiva
		// ciclo che scorre tutti i direttori 
		for (id=1;id<=netONCM.NActiveLIOO;id++)
			if (netONCM.CheckDirector(id))
				Send_NextPag(id,1);
	}
	
	//-----------  controllo per i dlioo  ------------------------	 
	psp=GetFirstSpa();
	while (psp!=NULL)
	{
		// per i Dlioo associati	
		pbt = psp->GetBattNum(num_bat);
		if (pbt==NULL)
		{
			return ;
		}
		
		
		if (pbt->GetNumPagDLIOO())
		{
			
			
			// cerco il leader della parte
			nome_parte = psp->GetStrumEsec();      
			indice_parte=netONCM.FindIDP(nome_parte);
			idleader=netONCM.Pindex[indice_parte].IdLeader;       
			
			// controllo che quella parte sia eseguita da qualcuno      
			if (idleader!=0)
			{ 
				Send_NextPag(idleader,1);
				// per i Dlioo associati	
                // invio ai DLIOO la pagina successiva
				id=netONCM.ListofNodes[idleader].Kids_id;
				while (id!=0)
				{
					// L'ultimo DLIOO al quale e' associata quella parte ha Kids_id 0 che e' la 
					// condizione di fine ciclo 	    
					Send_NextPag(id,1);
					id=netONCM.ListofNodes[id].Kids_id;
				}
			}
		}
		// passa alla parte successiva
		psp=GetNextSpa(psp);
	}
#else
#endif 
}   



// Avvio dei processi DLIOO sulla rete secondo la configurazione
// del file NetStart.cfg creato da ONCM
void MaMass::StartNodes()
{ 
#ifdef WITH_NET   
	RetCode infoCode, sendCode; 
	int tid_LIOO_lanciato;
	unsigned char index_nodes ;
	char Codice[DIM_COMMAND_STRING+1];
	char header_command[DIM_COMMAND_STRING+DIM_ID+2];
	
	// Numero di DLIOO attualmente presenti : secondo l'attuale implementazione e' uno(il MASAE)
	index_nodes=netONCM.NActiveLIOO ;   
	
	// formazione header del comando da inviare
	infoCode = CodeCommand((short int)INIZIALIZZAZIONE,Codice);
	header_command[0]=(char)(my_ID+1);
	header_command[1]='\0';
	strcat(header_command,Codice);
	
	// ciclo per avviare i vari LIOO  
	while (!(netONCM.ListofNodes[index_nodes].Host==NULL)) 
	{
		// avvio del processo
		netONCM.StartNode(index_nodes);
		
		// ricerca del tid del DLIOO lanciato
		tid_LIOO_lanciato = netONCM.FindPTID(index_nodes);
		
		// inserisco header del comando
		Array[0]='\0';
		strcat(Array,header_command);
		
		// inserimento dell'ID del LIOO lanciato 
		Array[DIM_ID+DIM_COMMAND_STRING] = (char)index_nodes;
		Array[DIM_ID+DIM_COMMAND_STRING+1] = '\0';
		
		// inserimento delle abilitazioni dei menu
		netONCM.MakeImpostMenu(&Array[DIM_ID+DIM_COMMAND_STRING+1],index_nodes);
		
		// invio messaggio al LIOO lanciato 
		sendCode = SendMsg_ack_MaM(tid_LIOO_lanciato);
		if (sendCode!=OK_SEND) 
		{
			// problemi di invio messaggio
			EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","StartNodes()");
			
		}
		
		// incremento indice dei DLIOO avviati
		index_nodes++;
	}  // fine ciclo avvio DLIOO
#else
#endif 
}




// aggiorna le abilitazioni se arriva un messaggio dal OPD
void MaMass::Send_Abil(unsigned char node_to_update[20], char NEW_ABIL[300])
{
#ifdef WITH_NET   
	
	RetCode infoCode, sendCode;
	char Codice[DIM_COMMAND_STRING+1];  
	
	// trova l'ID del LIOO da aggiornare
	int LIOO_TID=netONCM.Find_TID_from_NODENAME(node_to_update);
	
	
	// costruzione ed invio messaggio
	Array[0]=my_ID+1;
	Array[1]='\0';
	infoCode = CodeCommand((short int)AGG_ABIL,Codice);
	strcat(Array,Codice);  
	
	// inserimento delle abilitazioni dei menu
	strcat (Array,NEW_ABIL);
	sendCode = SendMsg_ack_MaM(LIOO_TID);
	if (sendCode!=OK_SEND) 
	{
		// problemi di invio messaggio
		EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","Send_Abil()**");
	}
#else
#endif 
}



// Carica Indice delle parti leggendo i vari Spartiti nella tabella delle parti
//  
void MaMass::LoadPindex()
{ 
	
#ifdef WITH_NET   
	
	char *nome_parte;	
	unsigned char idnp=0;
	unsigned char idleader;
	unsigned char idnH=1;
	unsigned char id_parte;
	Spartito *psp1;  	  
	
	// Refresh delle Tabelle : si azzerano i margini di visibilita' e Pindex
	netONCM.InitPindex();
	netONCM.RefreshTabella();
	
	// ciclo per scorrere la lista delle parti presenti 
	psp1=GetFirstSpa();
	while (psp1!=NULL)
	{		   
		idnp++;
		// definisco il nome della parte da inserire in tabella
		nome_parte = psp1->GetStrumEsec();
		
		// controllo che quella parte sia eseguita da qualcuno
		idleader = netONCM.FindIDL(nome_parte);  
		if (idleader!=0)
		{ // controllo che il leader non sia in realt un direttore
			//    if (netONCM.CheckDirector(idleader))
			
			// 1)carica nome della parte 
			// 2)carica rispettivo indice (nella tabella di configurazione) del leader
			//   della parte caricata 
			netONCM.InsertPindex(idnp,strlen(psp1->GetStrumEsec()),nome_parte,idleader);
		}
		
		// passa alla parte successiva
		psp1=GetNextSpa(psp1);
		
	}   //  fine ciclo lettura lista delle parti  
	
	// setto la variabile che tiene conto del numero delle parti presenti nello spartito
	// che non corrisponde al numero delle parti presenti nella tabella.Nella tabella NActiveParte 
	// corrisponde al numero di parti presenti nello ""SPARTITO""
    netONCM.NActiveParte=idnp;
	
	
	// Aggiunge alla tabella degli host l'identificativo della parte associata
	// saltando il MASAE   
	
    while (netONCM.ListofNodes[idnH].Host!=NULL)
    {            
		// cerco l'IDP : se e' zero significa che quell'host non viene usato
		// per l'opera che e' stata caricata
		id_parte = netONCM.FindIDP(netONCM.ListofNodes[idnH].Parte);
		
		netONCM.ListofNodes[idnH].IDP = id_parte;
		idnH++;
    }	
#else
#endif 
}


void MaMass::ComprimiBattute()
{
	Spartito *psp;
	
	// questo metodo deve analizzare tutte le battute di una parte:
	// 1) imposta a TRUE un attributo se la battuta fa parte di una battuta
	//    multirest.
	// 2) per la prima di queste battute devo creare un riferimento ad una nuova
	//    battuta multirest.
	// Creo dunque eventuali battute multirest in fase di caricamento della parte
	// e non all'invio della stessa.
	
	// Con la nuova interfaccia per la compressione adesso questo metodo deve:
	// 1) analizzare l'attributo Abil_Compress di ogni parte per vedere se deve
	//    essere inviata al leggio la parte compressa oppure no.
	// 2) se l'attributo  TRUE, allora questo metodo deve essere eseguito cosi'
	//    come  adesso.
	// 3) se l'attributo  FALSE, questo metodo dovr analizzare tutte le battute
	//    del brano, e se ne trova qulcuna scompressa, la deve scomprimere!!
	
	int i;
	
	for(i=0;i<MAXNODES;i++)
	{ 
		psp=GetSpartNum(netONCM.ListofNodes[i].IDP-1);
		if(psp!=NULL)
			psp->ComprimiBattute(netONCM.ListofNodes[i].Abil_Compress);
	} 
	
	
}


// viene richiamata appena si fa un caricamento da file
// Invia nstartbatt pagine a tutti i nodi (escluso se stesso id=0) che sono
// usati in quella opera dopo aver inviato il comando di inizializzazione della
// parte
void MaMass::InitParte()  
{
#ifdef WITH_NET   
	char Codice[DIM_COMMAND_STRING+1];
	int tid_destinatario;
	RetCode  sendCode,infoCode ;
	int nstartpages=1;
	int num_spa;
	unsigned char id;
	Spartito *psp1;
	int INVIO_START_PAGES;
	int nstartbatt=NUM_BAT;
	RetCode reccode = NO_MSG;
	unsigned char id_ack_ricevuto;
	char header_command[DIM_COMMAND_STRING+DIM_ID+2];
	char Ack_atteso[DIM_ACK+1];
	short int num_par,fineparentesi,inizioparentesi,tipoparentesi,numcodepar;
	Parentesi *pPar;
	
	
	ComprimiBattute();
	
	
	// formazione del comando da inviare
	Array[0]=(char)(my_ID+1);
	Array[1]='\0';
	infoCode = CodeCommand((short int)ESEGUI_NEW,Codice);
	strcat(Array,Codice);  
	
	// invio a tutti i DLIOO del comando di new scorrendo la tabella      
	for(id =1;((id<MAXNODES)&&(netONCM.ListofNodes[id].Host!=NULL));id++)
	{  
		// Acquisizione  TID del destinatario della parte    
		tid_destinatario = netONCM.FindPTID(id);
		
		// invio comando di inizializzazione della parte  
		sendCode = SendMsg_ack_MaM(tid_destinatario);
		if (sendCode!=OK_SEND) 
		{
			// problemi di invio messaggio
			EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","InitParte()");
		}  
	}; // end ciclo for di id
	
	
	// formazione header del comando da inviare per l'inizializzazione della parte
	infoCode = CodeCommand((short int)INIT_PARTE,Codice);
	header_command[0]=(char)(my_ID+1);
	header_command[1]='\0';
	strcat(header_command,Codice);
	
	for(id =1;((id<MAXNODES)&&(netONCM.ListofNodes[id].Host!=NULL));id++)
	{
		INVIO_START_PAGES=0;
		
		if (netONCM.CheckDirector(id)) 
		{// E' un direttore, lo devo inizializzare con tutte le parti     
			
			// Ricavo il tid del direttore
			tid_destinatario = netONCM.FindPTID(id);
			
			// inserisco header del comando
			Array[0]='\0';
			strcat(Array,header_command);
			
			// calcolo il numero degli spartiti
			num_spa=0;
			psp1=GetFirstSpa();
			while (psp1!=NULL)
			{  
				num_spa++;
				psp1 = GetNextSpa(psp1);
			}
			infoCode = CodeCommand((short int)num_spa,Codice);
			strcat(Array,Codice); 
			
			// scandisco tutta la lista delle parti per inviare i numeric code    
			psp1=GetFirstSpa();
			while (psp1!=NULL)
			{                          
				infoCode = CodeCommand((short int)psp1->GetNumericCode(),Codice);
				strcat(Array,Codice); 
				psp1 = GetNextSpa(psp1);
			}
			
			
			float margine;
			
			margine=GetLeftMargin();
			infoCode = CodeCommand((short int)margine,Codice);
			strcat(Array,Codice); 
			
			margine=GetTopMargin();
			infoCode = CodeCommand((short int)margine,Codice);
			strcat(Array,Codice); 
			
			margine=GetBottomMargin();
			infoCode = CodeCommand((short int)margine,Codice);
			strcat(Array,Codice); 
			
			//---------------------- trattamento lista delle parentesi
			num_par=listaPar.GetNumObj();
			infoCode = CodeCommand((short int)num_par,Codice);
			strcat(Array,Codice); 
			
			pPar = listaPar.GetFirstPar();   
			while (pPar!=NULL)
			{
				tipoparentesi = pPar->GetID();
				inizioparentesi = pPar->GetPSpaStart()->GetNumericCode();
				fineparentesi = pPar->GetPSpaEnd()->GetNumericCode();
				numcodepar = pPar->GetNumericCode();
				
				infoCode = CodeCommand((short int)tipoparentesi,Codice);
				strcat(Array,Codice); 
				
				infoCode = CodeCommand((short int)inizioparentesi,Codice);
				strcat(Array,Codice); 
				
				infoCode = CodeCommand((short int)fineparentesi,Codice);
				strcat(Array,Codice);   
				
				infoCode = CodeCommand((short int)numcodepar,Codice);
				strcat(Array,Codice);   
				
				pPar = listaPar.GetNextPar(pPar);
			}
			
			//------------------ aggiungo i nomi degli spartiti
			psp1=GetFirstSpa();
			while (psp1!=NULL)
			{                                
				strcat(Array,psp1->GetStrumEsec());
				strcat(Array,"\n"); 
				psp1 = GetNextSpa(psp1);
			}
			
			
			// invio comando di inizializzazione della parte  
			sendCode = SendMsg_ack_MaM(tid_destinatario);
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","InitParte()");
			}
			INVIO_START_PAGES=1;
		}
		else if (netONCM.ListofNodes[id].IDP!=0)
		{// E' uno strumento, lo devo inizializzare solo con la sua parte       
			
			// Acquisizione  TID del destinatario della parte    
			tid_destinatario = netONCM.FindPTID(id);  
			
			//
			psp1=GetFirstSpa();
			while (psp1!=NULL) 
			{
				if (!(strcmp(netONCM.ListofNodes[id].Parte,psp1->GetStrumEsec())))
				{
					// inserisco header del comando
					Array[0]='\0';
					strcat(Array,header_command);
					
					// inserisco numcode  
					infoCode = CodeCommand((short int)psp1->GetNumericCode(),Codice);
					strcat(Array,Codice);
					
					// invio comando di inizializzazione della parte  
					sendCode = SendMsg_ack_MaM(tid_destinatario);
					if (sendCode!=OK_SEND) 
					{   
						// problemi di invio messaggio
						EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","InitParte()");
						
					}
					psp1 = NULL;
					INVIO_START_PAGES=1;
				}
				else psp1 = GetNextSpa(psp1);
			}
		} 
		
		
		if (INVIO_START_PAGES==1)
		{// E' stato precedentemente inizializzato un Host Valido
			
			// pulisco l'Array
			Array[0]='\0';
			
			BattutaForward(id,ESEGUI_BATT_LOAD,TRUE,nstartbatt);
			
			// posiziono a 0 il limite sx della visualizzazione 
			netONCM.SetSxLimit(id,0);
			
			// formazione del comando di fine inizializzazione parte  
			Array[0]=(char)(my_ID+1);
			Array[1]='\0';
			
			infoCode = CodeCommand((short int)END_INIT_PARTE,Codice);
			strcat(Array,Codice);
			
			// invio comando di fine inizializzazione della parte          
			sendCode = SendMsg_ack_MaM(tid_destinatario);
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","InitParte()");
				
			}  
		} 
 } 
 
#else
#endif 
}


// metodo parzialmente implementato perche' si deve ancora definire cosa deve
// fare in particolare
void MaMass::BattutaNum(unsigned char id_mitt,int Nump)
{
#ifdef WITH_NET   
#else
#endif 
}



void MaMass::BattutaBackward(unsigned char id_mitt, Bool fl, short numbat)
{
#ifdef WITH_NET   
	RetCode infoCode;
	char Codice[DIM_COMMAND_STRING+1]; 
	int tid_mittente;
	short indice;
	RetCode info_ack;
	Battuta* PBattuta;
	RetCode  sendCode;
	Spartito *psp; 
	int nb;  
	Battuta *pcbat;
	Battuta *ptempbat=NULL; 	  
	
	for (indice=1;indice<=numbat;indice++)
	{
		// pulisco l'Array
		Array[0]='\0'; 
		// firmo la stringa e inserisco il comando ESEGUI_BATT
		Array[0]=(char)(my_ID+1);
		Array[1]='\0';
		infoCode = CodeCommand((short int)ESEGUI_BATT,Codice);
		
		// Acquisizione  TID del mittente    
		tid_mittente = netONCM.FindPTID(id_mitt);
		
		if (netONCM.CheckDirector(id_mitt)) 
		{// E' un direttore ! vuole tutta la colonna di spartiti ....        
			int flag=0;
			
			psp=GetSpartNum(0);
			while (psp!=NULL) 
			{
				PBattuta=psp->GetBattNum(netONCM.GetSxLimit(id_mitt)-1);
				if (PBattuta!=NULL) 
				{
					if (flag==0)
					{
						strcat(Array,Codice);  
						flag=1;
					}
					strcat(Array,psp->Describe(PBattuta));			
					psp=GetNextSpa(psp);	
				} 
				else 
				{
					infoCode = CodeCommand((short int)END_BATTUTE_BACKWARD,Codice);
					strcat(Array,Codice);	  	
					psp=NULL;
					indice=numbat;
				}	            
			}
			
			sendCode = SendMsg_ack_MaM(tid_mittente);
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","BattutaBackward()");
			}
			if (flag==1)
			{
				netONCM.SetSxLimit(id_mitt,netONCM.GetSxLimit(id_mitt)-1);
				if (fl) netONCM.SetDxLimit(id_mitt,netONCM.GetDxLimit(id_mitt)-1);
			}     
		}
		else 
		{// non e' un direttore
			
			// 1--MaMass identifica lo spartito 
			psp = GetSpartNum(netONCM.ListofNodes[id_mitt].IDP-1);
			if (psp==NULL)
			{
				EM.Error(MUSICAL_ERR_AREA, "EC_SPA_NOTFOUND","MaMass","BattutaBackward()");
				// gestione errore non trovato lo spartito
				return;
			}
			
			// 2--MaMass cerca la battuta assegnata nello spartito  selezionato
			if (netONCM.GetSxLimit(id_mitt)>0)
				PBattuta=psp->GetBattNum(netONCM.GetSxLimit(id_mitt)-1);
			//    PBattuta=psp->GetBattNumProg(netONCM.GetSxLimit(id_mitt)-1);      
			else 
				PBattuta=NULL;
			
			
			nb=1;
			if (PBattuta!=NULL) // se la battuta esiste 
			{
				if(PBattuta->GetIsMultiRest())
				{
					
					//  una battuta che fa parte di una multirest...invio allora direttamente
					// la battuta multirest che mi sono creata al caricamento.
					// In questo caso pero' devo risalire alla prima di queste battute
					// perche' solo lei ha il puntatore alla battuta multirest.
					
					while(PBattuta->GetIsMultiRest() && PBattuta->GetPMultiRest()==NULL)
					{
						ptempbat=PBattuta;
						PBattuta=psp->GetPrevBat(PBattuta);
						if(PBattuta==NULL) 
						{
							PBattuta=ptempbat;
							break;
						}	      
					}
					
					pcbat=PBattuta->GetPMultiRest();
					if(pcbat!=NULL)
					{
						nb=pcbat->GetNumMultiRest();
						strcat(Array,Codice);  
						strcat(Array,psp->Describe(pcbat)); 
					}
				} 
				else
				{
					strcat(Array,Codice);
					strcat(Array,psp->Describe(PBattuta)); 
				} 
				
				// 2.1--MaMass Descrive e Manda la battuta
				//strcat(Array,Codice);  
				//strcat(Array,psp->Describe(PBattuta));
				
				// invio battuta
				sendCode = SendMsg_ack_MaM(tid_mittente);
				if (sendCode!=OK_SEND) 
				{
					// problemi di invio messaggio
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","BattutaBackward()");
					//return FALSE;  //perch false
					
				};
				// setto i nuovi eventuali limiti dx e sx di visualizzazione  
				netONCM.SetSxLimit(id_mitt,netONCM.GetSxLimit(id_mitt)-nb);
				if (fl) netONCM.SetDxLimit(id_mitt,netONCM.GetDxLimit(id_mitt)-1);//boh!
			} 
			else 
			{// --MaMass invia comando di fine battute
				
				// formazione del comando di fine battute 
				Array[0]=(char)(my_ID+1);
				Array[1]='\0';
				indice=numbat;
				infoCode = CodeCommand((short int)END_BATTUTE_BACKWARD,Codice);
				strcat(Array,Codice);
				
				// invio comando di fine battuta  
				sendCode = SendMsg_ack_MaM(tid_mittente);
				if (sendCode!=OK_SEND) 
				{
					// problemi di invio messaggio
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","BattutaBackward()");
					// return FALSE;  //perch false
				}  
			}
		}
   } // for(indice)
   printf("SXb:%d, DXb:%d\n",netONCM.GetSxLimit(id_mitt), netONCM.GetDxLimit(id_mitt));
#else
#endif 
}



// Invio della battuta successiva a quella presente sul DLIOO interessato
void MaMass::BattutaForward(unsigned char id_mitt, short int CommandCode, Bool fl, short numbat)
{
#ifdef WITH_NET   
	char Codice[DIM_COMMAND_STRING+1];   	
	int tid_mittente;
	short indice;
	RetCode info_ack, infoCode, sendCode;
	Battuta* PBattuta;  	
	Spartito *psp;
	int nb;
	Battuta *pcbat;
	
	
	
	for(indice=1;indice<=numbat;indice++)
	{
		
		// pulisco l'Array
		Array[0]=(char)(my_ID+1);
		Array[1]='\0';
		infoCode = CodeCommand((short int)CommandCode,Codice);
		
		// Acquisizione  TID del mittente    
		tid_mittente = netONCM.FindPTID(id_mitt);
		
		if (netONCM.CheckDirector(id_mitt))
		{
			// E' un direttore ! vuole tutta la colonna di spartiti ....        
			int flag=0;
			
#ifdef DEBUG_TIME
			struct timeval timevI, timevF;
			struct timezone timezI, timezF;
			double time;
			gettimeofday(&timevI,&timezI);
#endif
			
			psp=GetSpartNum(0);
			while (psp!=NULL)
			{
				PBattuta=psp->GetBattNum(netONCM.GetDxLimit(id_mitt));
				if (PBattuta!=NULL) 
				{
					if (flag==0) 
					{
						strcat(Array,Codice);  
						flag=1;
					}
					strcat(Array,psp->Describe(PBattuta));			
					psp=GetNextSpa(psp);	
				} 
				else 
				{
					infoCode = CodeCommand((short int)END_BATTUTE_FORWARD,Codice);
					strcat(Array,Codice);	  	
					psp=NULL;
					indice = numbat;
				}
			}	
			
#ifdef DEBUG_TIME
			if (flag==1) 
			{
				gettimeofday(&timevF,&timezF);
				time = (double)((double)((timevF.tv_sec-timevI.tv_sec)*1000000 + timevF.tv_usec - timevI.tv_usec)/1000);
			}
#endif 
			
			
			sendCode = SendMsg_ack_MaM(tid_mittente);
			if (sendCode!=OK_SEND) 
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","BattutaForward()");
			
			if (flag==1) 
			{
				netONCM.SetDxLimit(id_mitt,netONCM.GetDxLimit(id_mitt)+1);
				if (fl) netONCM.SetSxLimit(id_mitt,netONCM.GetSxLimit(id_mitt)+1);
			}
			
		} 
		else
		{// non e' un direttore
			
			// 1--MaMass identifica lo spartito
			psp = GetSpartNum(netONCM.ListofNodes[id_mitt].IDP-1);
			if (psp==NULL)
			{
				EM.Error(MUSICAL_ERR_AREA, "EC_SPA_NOTFOUND","MaMass","BattutaForward()");
				return;
			}
			// 2--MaMass cerca la battuta assegnata nello spartito  selezionato
			PBattuta=psp->GetBattNum(netONCM.GetDxLimit(id_mitt));      
#ifdef DEBUG_TIME
			gettimeofday(&timevI,&timezI);
#endif
			
			nb=1;
			if (PBattuta!=NULL) // se la battuta esiste 
			{
				if(PBattuta->GetIsMultiRest())
				{
					//  una battuta che fa parte di una multirest...invio allora direttamente
					// la battuta multirest che mi sono creata al caricamento.
					
					pcbat=PBattuta->GetPMultiRest();
					if(pcbat!=NULL)
					{
						nb=pcbat->GetNumMultiRest();
						strcat(Array,Codice);  
						strcat(Array,psp->Describe(pcbat)); 
					}
				} 
				else
				{
					strcat(Array,Codice);
					strcat(Array,psp->Describe(PBattuta)); 
				} 
				
				
				
				// 2.1--MaMass Descrive e Manda la battuta
				
				
				
#ifdef DEBUG_TIME
				gettimeofday(&timevF,&timezF);
				time = (double)((double)((timevF.tv_sec-timevI.tv_sec)*1000000 + timevF.tv_usec - timevI.tv_usec)/1000);
#endif 
				
				// invio battuta
				sendCode = SendMsg_ack_MaM(tid_mittente);
				if (sendCode!=OK_SEND) 
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","BattutaForward()");
				
				netONCM.SetDxLimit(id_mitt,netONCM.GetDxLimit(id_mitt)+nb);
				if (fl) netONCM.SetSxLimit(id_mitt,netONCM.GetSxLimit(id_mitt)+1);
			}
			else 
			{
				infoCode = CodeCommand((short int)END_BATTUTE_FORWARD,Codice);
				strcat(Array,Codice);
				
				// invio comando di fine battuta  
				sendCode = SendMsg_ack_MaM(tid_mittente);
				if (sendCode!=OK_SEND) 
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","BattutaForward()");
				indice = numbat;
			}
		}  
  } /
	  printf("SXf:%d, DXf:%d\n",netONCM.GetSxLimit(id_mitt), netONCM.GetDxLimit(id_mitt));                		
#else
#endif 
}


void MaMass::PageBackward(unsigned char id)
{
#ifdef WITH_NET   
#else
#endif 
}


// Invia NUM_BAT battute al DLIOO in questione per la fase di editing
void MaMass::PageForward(unsigned char id)
{
#ifdef WITH_NET   
	BattutaForward(id,ESEGUI_BATT,TRUE,NUM_BAT);
#else
#endif 
}



// Invio del comando eseguito ai DLIOO interessati
void MaMass::NetInvioComando(SymCmd SC)
{
#ifdef WITH_NET   
	char Codice[DIM_COMMAND_STRING+1];
	RetCode infoCode, sendCode;
	int tid_destinatario;
	unsigned char id, indice_parte;
	
	// inserita per test di rete 
	struct timeb StartT, StopT;
	static int i=1; 
	
	switch(SC.CommandId)
	{
    case CMD_PARTITURA_END:                
		// formazione del comando da inviare
        Array[0]=(char)(my_ID+1);
        Array[1]='\0';
        infoCode = CodeCommand((short int)ESEGUI_QUIT,Codice);
        strcat(Array,Codice);  
		
		// invio a tutti i DLIOO del comando di quit scorrendo la tabella      
        for(id =1;((id<MAXNODES)&&(netONCM.ListofNodes[id].Host!=NULL));id++)
        {
			// Acquisizione  TID del destinatario della parte    
			tid_destinatario = netONCM.ListofNodes[id].PTID;
			
			// invio comando di quit 
			sendCode = SendMsg_ack_MaM(tid_destinatario);
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","NetInvioComando()");
				
			}  
        } // end ciclo for di id
#ifndef TINYMOODS
		if(netONCM.NActiveLIOO > 1)
			pvm_kill(OPD_TID); // termina OPD
#endif  
		return;
		
    case CMD_PARTITURA_NEW:               
		// formazione del comando da inviare
        Array[0]=(char)(my_ID+1);
        Array[1]='\0';
        infoCode = CodeCommand((short int)ESEGUI_NEW,Codice);
        strcat(Array,Codice);  
		
		// invio a tutti i DLIOO del comando di new scorrendo la tabella      
        for(id =1;((id<MAXNODES)&&(netONCM.ListofNodes[id].Host!=NULL));id++)
        {  
			// Acquisizione  TID del destinatario della parte    
			tid_destinatario = netONCM.FindPTID(id);
			
			// invio comando di inizializzazione della parte  
			sendCode = SendMsg_ack_MaM(tid_destinatario);
			
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","NetInvioComando()");
				
			}  
			
        } // end ciclo for di id
		
		return;
		
    case CMD_PARTITURA_LOAD:
    case CMD_PARTITURA_SAVEAS:
    case CMD_PARTITURA_SAVE:
    case CMD_SPARTITO_NPENT:
    case CMD_DEBUG_UNO:
    case CMD_DEBUG_DUE:
    case CMD_DEBUG_TRE:
    case CMD_DEBUG_QUATTRO:
    case CMD_DEBUG_CINQUE:
    case CMD_DEBUG_SEI:
    case CMD_DEBUG_SETTE:
    case CMD_DEBUG_OTTO:
    case CMD_DEBUG_NOVE:
		return ;
	}
	
	
	// formazione del comando da inviare
	Array[0]=(char)(my_ID+1);
	Array[1]='\0';
	infoCode = CodeCommand((short int)ESEGUI_COMMAND,Codice);
	strcat(Array,Codice);  
	
	Sym2String(&Array[DIM_ID+DIM_COMMAND_STRING],SC);
	//   printf("valore dell'array appena formato =>%s \n",Array);
	//------------------------------- NOTIFICAZIONE AL DIRETTORE 
	indice_parte=netONCM.FindIDPDirector();
	if (indice_parte!=0)
	{  
		id=netONCM.ListofNodes[indice_parte].Group_leader;
		// ricerca tid di tutti i MASE a cui inviare il comando partendo dal leader dei direttori .
		while (id!=0)
		{
			tid_destinatario = netONCM.FindPTID(id);
			
			// invio comando a questo direttore
			sendCode = SendMsg_ack_MaM(tid_destinatario);
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","NetInvioComando()");
				
			}
			
			// L'ultimo MASE della lista e' quello con Kids_id=0
			id=netONCM.ListofNodes[id].Kids_id;
		}     //ciclo while id
	} // endif
	
	//--------------------------------- NOTIFICAZIONE AI DLIOO 
	// ricerca della parte modificata
	char *ModificaSuParte;
	ModificaSuParte = (GetSpart(SC.Path1.Array[SPARTITO_PATH_LEVEL]))->GetStrumEsec();
	
	// ricerca dell'indice della parte modificata nella tabella delle parti
	indice_parte=netONCM.FindIDP(ModificaSuParte);
	
	
	// se indice_parte e' non nullo significa che e' tutto ok, altrimenti significa che quella parte
	// non la esegue nessuno e quindi esco perche' non  devo inviare il comando a nessuno
	if (indice_parte==0)
	{
		return;
	}  
	id=netONCM.Pindex[indice_parte].IdLeader;
	
	// ricerca tid dei DLIOO interessati alla modifica a cui inviare il comando
	// partendo dal leader della parte. 
	while (id!=0)
	{
		tid_destinatario = netONCM.FindPTID(id);
		
		// invio comando     
		sendCode = SendMsg_ack_MaM(tid_destinatario);
		if (sendCode!=OK_SEND) 
		{
			// problemi di invio messaggio
			EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","NetInvioComando()");
			
		}
		
		// L'ultimo DLIOO al quale e' associata quella parte ha Kids_id==0 che e' la 
		// condizione di fine ciclo 	    
		id=netONCM.ListofNodes[id].Kids_id;
	}     
	// pulisco l'Array        
	Array[0] = '\0';
	
#else
	NetSymCmd = UsrSymCmd;
#endif 
}



void MaMass::InitImpaginazione()
{
	
	//  Costruisco il vettore dell'ETT  della dimensione dello spartito
	BuildETT(GetFirstSpa()->GetLastBat()->GetNumProgress() + 1);
	
	if(!pMusicWin->GetIsFormatted())
	{
		//  Posiziona all'inizio della partitura.
		GoTop();
		
		//  Resetto gli attributi del'impaginazione di ogni battuta
		Spartito *psp1=GetFirstSpa();
		while (psp1!=NULL)
		{
			psp1->InitImpagina();
			psp1 = GetNextSpa(psp1);
		}
		
		//  Setto lo stato di inizio impaginazione :  Impaginazione=BEGIN_IMP 
		SetImpaginazione(BEGIN_IMP);
		
		// Setto il tempo fra una chiamata e l'altra e la Porzione (in pixel) da copiare 
		// ogni volta  (per lo scrolling) e la modalita di esecuzione
		pMusicWin->SetScrollingTime(1);
		pMusicWin->SetScrollingInc(200);
		pMusicWin->AttivaModEsecuzione();
		
		
		
		ComprimiBattute();
		
		
#ifdef WITH_NET   
		RetCode infoCode;
		RetCode sendCode;
		char Codice[DIM_COMMAND_STRING+1];
		char header_command[DIM_COMMAND_STRING+DIM_ID+2];
		int tid_LIOO_leader,tid_PDLOO, tid_direttore;
		unsigned char indiceparte, idleader, id_PDLOO, id_kid;
		
		
		// formazione header del comando da inviare
		infoCode = CodeCommand((short int)INIT_IMPAGINA,Codice);
		header_command[0]=(char)(my_ID+1);
		header_command[1]='\0';
		strcat(header_command,Codice);
		
		// ciclo per gestire l'invio del comando di inizio impaginazione a tutti i dlioo
		// e le prime NUM_BAT battute da impaginare ai LiooLeader  
		indiceparte=1; 
		NDliooInImp=0; 
		while (indiceparte <= (netONCM.NActiveParte)) 
		{
			if (netONCM.Pindex[indiceparte].Parte!=NULL) 
			{
				NDliooInImp++;
				idleader=netONCM.Pindex[indiceparte].IdLeader;
				
				// ricerca del tid del leader
				tid_LIOO_leader = netONCM.FindPTID(idleader);
				
				// inserisco header del comando
				Array[0]='\0';
				strcat(Array,header_command);
				
				
				// invio messaggio di initimpaginazione al LIOO leader
				sendCode = SendMsg_ack_MaM(tid_LIOO_leader);
				
				if (sendCode!=OK_SEND) 
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","InitImpaginazione()");
				
				netONCM.SetDxLimit(idleader,0);
				// invio delle prime NUM_BAT battute per effettuare l'impaginazione ad ogni leader 
				Send_NBat(NUM_BAT,idleader);
				
				// invio ai nodi PDLOO il comando di inizio impaginazione 
				id_PDLOO=netONCM.ListofNodes[netONCM.Pindex[indiceparte].IdLeader].Kids_id;
				while (id_PDLOO!=0)
				{       
					tid_PDLOO= netONCM.FindPTID(id_PDLOO);
					
					// inserisco header del comando
					Array[0]='\0';
					strcat(Array,header_command);
					
					// invio comando
					sendCode = SendMsg_ack_MaM(tid_PDLOO);
					if (sendCode!=OK_SEND) 
					{
						// problemi di invio messaggio
						EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","InitImpaginazione()");
						
					}
					
					// L'ultimo DLIOO al quale e' associata quella parte ha Kids_id 0 che e' la 
					// condizione di fine ciclo 	    
					id_PDLOO=netONCM.ListofNodes[id_PDLOO].Kids_id;
				} 
			};  
			indiceparte++; 
		};
		
		// ------------------    ciclo per la gestione del direttore  ----------------
		for(unsigned char id =1;((id<MAXNODES)&&(netONCM.ListofNodes[id].Host!=NULL));id++)
		{
			if (netONCM.CheckDirector(id))
			{
				// ricerca del tid del leader
				tid_direttore = netONCM.FindPTID(id);
				
				// inserisco header del comando
				Array[0]='\0';
				strcat(Array,header_command);
				
				// invio comando
				sendCode = SendMsg_ack_MaM(tid_direttore);
				if (sendCode!=OK_SEND) 
				{
					// problemi di invio messaggio
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","InitImpaginazione()");
					
				}
				NDliooInImp++;     
				netONCM.SetDxLimit(id,0);
				// invio delle prime NUM_BAT battute per effettuare l'impaginazione ad ogni leader 
				Send_NBat(NUM_BAT,id);
				
				// invio ai nodi non leader direttoriali il comando di inizio esecuzione 
				id_kid=netONCM.ListofNodes[id].Kids_id;
				while (id_kid!=0)
				{
					tid_direttore = netONCM.FindPTID(id_kid);
					
					// inserisco header del comando
					Array[0]='\0';
					strcat(Array,header_command);
					
					// invio comando
					sendCode = SendMsg_ack_MaM(tid_direttore);
					if (sendCode!=OK_SEND) 
					{
						// problemi di invio messaggio
						EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","InitImpaginazione()");
						
					}
					
					id_kid=netONCM.ListofNodes[id_PDLOO].Kids_id;
					
				} 
			};
		}; 
		
		// se nessuno DLIOO e presente  l'archivista puo'fare l'impaginazione 
		if (NDliooInImp==0)
			pMusicWin->ScrollAll();
		
#else
		pMusicWin->ScrollAllPages();
#endif 
  }
}



void MaMass::InitEsecuzione(char *nome)
{ 
	InitTimeExec();

	// leggo il file ett associato a quella opera
	if(nome!=NULL)
		ReadETT(nome);
	
	if ((ETTLenght-1) != GetFirstSpa()->GetNumeroBattute())
	{
		Message("Wrong ETT! Initialization failed.", "Attention");
		return;
	}
    
	
	// posiziono all'inizio della partitura 
	GoBattInExecution(1);
	
	
	// setto il flag di stato per l'impaginazione
	SetImpaginazione(BEGIN_ESEC);
	
#ifdef WITH_NET   
	RetCode infoCode;
	RetCode sendCode;
	char Codice[DIM_COMMAND_STRING+1];
	char CodiceNumBat[DIM_COMMAND_STRING+1];
	char header_command[2*DIM_COMMAND_STRING+DIM_ID+2];
	int tid_LIOO_leader,tid_PDLOO, tid_direttore;
	unsigned int NumeroBattute;
	unsigned char indiceparte ,idleader,id_PDLOO, id_kid;
	
	// Refresh della Tabella: si azzerano i margini di visibilita'
	netONCM.RefreshTabella();
	
	// calcolo il numero delle battute per quella opera 
	NumeroBattute = (unsigned int )(GetFirstSpa()->GetNumeroBattute() + 1);
	
	// formazione header del comando da inviare
	infoCode = CodeCommand((short int)INIT_EXEC,Codice);
	header_command[0]=(char)(my_ID+1);
	header_command[1]='\0';
	strcat(header_command,Codice);
	
	
	// codifica in stringa del numero di battute del brano  
	infoCode = CodeCommand((short int)NumeroBattute,CodiceNumBat);     
	strcat(header_command,CodiceNumBat);
	
	// ciclo per inviare (a tutti i lioo che hanno una parte da eseguire) il comando
	// di inizio esecuzione
	indiceparte=1; 
	NDliooInImp=0;
	while (indiceparte <= (netONCM.NActiveParte))
	{ 
		if (netONCM.Pindex[indiceparte].Parte!=NULL) 
		{   
			NDliooInImp++;
			idleader=netONCM.Pindex[indiceparte].IdLeader;
			
			// ricerca del tid del leader
			tid_LIOO_leader = netONCM.FindPTID(idleader);
			
			// inserisco header del comando
			Array[0]='\0';
			strcat(Array,header_command);
			
			// invio messaggio di initimpaginazione al LIOO leader
			sendCode = SendMsg_ack_MaM(tid_LIOO_leader);
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","InitEsecuzione()");
			};
			
			// manda le prime sei pagine 
			Send_NextPag(idleader,6);
			
			// invio ai nodi PDLOO il comando di inizio esecuzione 
			//  id_PDLOO=netONCM.Pindex[indiceparte].IdLeader;
			id_PDLOO=netONCM.ListofNodes[netONCM.Pindex[indiceparte].IdLeader].Kids_id;
			while (id_PDLOO!=0)
			{
				tid_PDLOO= netONCM.FindPTID(id_PDLOO);
				
				// inserisco header del comando
				Array[0]='\0';
				strcat(Array,header_command);
				
				// invio comando
				sendCode = SendMsg_ack_MaM(tid_PDLOO);
				
				if (sendCode!=OK_SEND) 
				{
					// problemi di invio messaggio
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","InitEsecuzione()");	   
					
				}
				
				// manda le prime sei pagine 	
				Send_NextPag(id_PDLOO,6);
				
				// L'ultimo DLIOO al quale e' associata quella parte ha Kids_id 0 che e' la 
				// condizione di fine ciclo 	    
				id_PDLOO=netONCM.ListofNodes[id_PDLOO].Kids_id;
			} 
			
		}; 
		indiceparte++; 
	};
	
	
	// ----------      ciclo per la gestione del direttore   --------------------------------------
	for(unsigned char id =1;((id<MAXNODES)&&(netONCM.ListofNodes[id].Host!=NULL));id++)
	{
		if (netONCM.CheckDirector(id))
		{
			// ricerca del tid del leader
			tid_direttore = netONCM.FindPTID(id);
			
			// inserisco header del comando
			Array[0]='\0';
			strcat(Array,header_command);
			
			// calcolo il numero degli spartiti
			int num_spa=0;
			psp1=GetFirstSpa();
			while (psp1!=NULL)
			{  
				num_spa++;
				psp1 = GetNextSpa(psp1);
			}
			infoCode = CodeCommand((short int)num_spa,Codice);
			strcat(Array,Codice); 
			
			// scandisco tutta la lista delle parti per inviare i numeric code    
			psp1=GetFirstSpa();
			while (psp1!=NULL)
			{                          
				infoCode = CodeCommand((short int)psp1->GetNumericCode(),Codice);
				strcat(Array,Codice); 
				psp1 = GetNextSpa(psp1);
			}
			
			// ----------------------------- trattamento lista delle parentesi
			short int num_par,fineparentesi,inizioparentesi,tipoparentesi,numcodepar;
			Parentesi *pPar;
			
			num_par=listaPar.GetNumObj();
			infoCode = CodeCommand((short int)num_par,Codice);
			strcat(Array,Codice); 
			
			pPar = listaPar.GetFirstPar();   
			while (pPar!=NULL)
			{
				tipoparentesi = pPar->GetID();
				inizioparentesi = pPar->GetPSpaStart()->GetNumericCode();
				fineparentesi = pPar->GetPSpaEnd()->GetNumericCode();
				numcodepar = pPar->GetNumericCode();
				
				infoCode = CodeCommand((short int)tipoparentesi,Codice);
				strcat(Array,Codice); 
				
				infoCode = CodeCommand((short int)inizioparentesi,Codice);
				strcat(Array,Codice); 
				
				infoCode = CodeCommand((short int)fineparentesi,Codice);
				strcat(Array,Codice);   
				
				infoCode = CodeCommand((short int)numcodepar,Codice);
				strcat(Array,Codice);   
				
				pPar = listaPar.GetNextPar(pPar);
			}
			
			// aggiungo i nomi degli spartiti
			psp1=GetFirstSpa();
			while (psp1!=NULL)
			{                                
				strcat(Array,psp1->GetStrumEsec());
				strcat(Array,"\n"); 
				psp1 = GetNextSpa(psp1);
			}
			
			// invio messaggio di initimpaginazione al direttore
			sendCode = SendMsg_ack_MaM(tid_direttore);
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","InitEsecuzione()");	 
				
			};
			// manda le prime sei pagine al direttore
			Send_NextPag(id,6);
			
			// invio ai nodi non leader direttoriali il comando di inizio esecuzione 
			id_kid=netONCM.ListofNodes[id].Kids_id;
			while (id_kid!=0)
			{
				tid_direttore = netONCM.FindPTID(id_kid);
				// inserisco header del comando
				Array[0]='\0';
				strcat(Array,header_command);
				
				// invio comando
				sendCode = SendMsg_ack_MaM(tid_direttore);
				if (sendCode!=OK_SEND) 
				{
					// problemi di invio messaggio
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","InitEsecuzione()");
					
				}
				
				// manda le prime sei pagine ai direttori non leader	
				Send_NextPag(id_kid,6);
				id_kid=netONCM.ListofNodes[id_PDLOO].Kids_id;
			} 
    }; 
 };
 
#endif 
 StatoEsecuzione =   ESECUZIONE;
 //Message("End of Initialization.");
}




void MaMass::JumpToBatt(NumCode& nb)
{
	Spartito *psp;
	Battuta *pbat;
	NumCode  NumBat;
	
	//  GoTop();
	SetImpaginazione(BEGIN_ESEC);
	
#ifdef WITH_NET   
	RetCode infoCode, sendCode;
	char Codice[DIM_COMMAND_STRING+1], CodiceNumBat[DIM_COMMAND_STRING+1], 
		CodiceBatJump[DIM_COMMAND_STRING+1],
		header_command[2*DIM_COMMAND_STRING+DIM_ID+2],
		header_jump[2*DIM_COMMAND_STRING+DIM_ID+2];
	unsigned char indiceparte ,idleader, id_PDLOO, id_kid, id;
	int tid_LIOO_leader, tid_PDLOO, tid_direttore, num_spa;
	unsigned int NumeroBattute;
	short int num_par,fineparentesi,inizioparentesi,tipoparentesi,numcodepar;
	Parentesi *pPar;
	
	// faccio un reset della visualizzazione
	netONCM.RefreshTabella();
	
	// formazione dell'header del primo comando per il riavvio della esecuzione 
	NumeroBattute = (unsigned int )(GetFirstSpa()->GetNumeroBattute() + 1); 
	
	indiceparte = 1;
	NDliooInImp = 0;
	while (indiceparte <= (netONCM.NActiveParte))
	{ 
		if (netONCM.Pindex[indiceparte].Parte!=NULL) 
		{   
			idleader=netONCM.Pindex[indiceparte].IdLeader;
			psp = GetSpartNum(netONCM.ListofNodes[idleader].IDP-1);
			pbat = psp->GetBattNum(nb-1);
			while (pbat->GetNumPagDLIOO()==0)
			{
				pbat = psp->GetPrevBat(pbat);
				if (pbat==NULL) break;
			}
			
			if(pbat!=NULL) 
			{
				NumBat = pbat->GetNumProgress();
				pbat = psp->GetPrevBat(pbat);
				if (pbat!=NULL) 
				{
					while (pbat->GetNumPagDLIOO()==0)
					{
						pbat = psp->GetPrevBat(pbat);
						if (pbat==NULL) break;
					}
				}
				if (pbat==NULL) pbat = psp->GetBattNum(0);
			}
			else 
			{
				pbat = psp->GetBattNum(0);
				NumBat = pbat->GetNumProgress();
			}
			NumLim = pbat->GetNumProgress();
			netONCM.SetSxLimit(idleader,NumLim-1);
			netONCM.SetDxLimit(idleader,NumLim-1);
			
			tid_LIOO_leader = netONCM.FindPTID(idleader);
			
			infoCode = CodeCommand((short int)INIT_EXEC,Codice);
			header_command[0]=(char)(my_ID+1);
			header_command[1]='\0';
			strcat(header_command,Codice); 
			infoCode = CodeCommand((short int)NumeroBattute,CodiceNumBat);
			strcat(header_command,CodiceNumBat);
			
			Array[0]='\0';
			//strcat(Array,header_command);
			strcpy(Array,header_command);
			
			// invio ai nodi DLIOO il comando di inizio esecuzione  e sei pagine
			sendCode = SendMsg_ack_MaM(tid_LIOO_leader);
			if (sendCode!=OK_SEND) 
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","JumpToBatt()");
			
			Send_NextPag(idleader,6);
			
			infoCode = CodeCommand((short int)JUMP_TO_BATT,Codice);
			header_jump[0]=(char)(my_ID+1);
			header_jump[1]='\0';
			strcat(header_jump,Codice);
			
			infoCode = CodeCommand((short int)NumBat,CodiceBatJump);
			strcat(header_jump,CodiceBatJump);
			
			Array[0]='\0';
			// strcat(Array,header_jump);
			strcpy(Array,header_jump);
			
			sendCode = SendMsg_ack_MaM(tid_LIOO_leader);
			if (sendCode!=OK_SEND) 
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","JumpToBatt()");
			
			id_PDLOO = netONCM.ListofNodes[netONCM.Pindex[indiceparte].IdLeader].Kids_id;
			while (id_PDLOO!=0)
			{
				netONCM.SetDxLimit(id_PDLOO,NumLim-1);
				netONCM.SetSxLimit(id_PDLOO,NumLim-1);
				
				tid_PDLOO= netONCM.FindPTID(id_PDLOO);
				
				infoCode = CodeCommand((short int)INIT_EXEC,Codice);
				header_command[0]=(char)(my_ID+1);
				header_command[1]='\0';
				strcat(header_command,Codice); 
				infoCode = CodeCommand((short int)NumeroBattute,CodiceNumBat);
				strcat(header_command,CodiceNumBat);
				
				
				Array[0]='\0';
				//strcat(Array,header_command);
				strcpy(Array,header_command);
				
				// invio ai nodi PDLOO il comando di inizio esecuzione  e sei pagine
				sendCode = SendMsg_ack_MaM(tid_PDLOO);
				if (sendCode!=OK_SEND) 
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","JumpToBatt()");	   
				Send_NextPag(id_PDLOO,6);
				
				infoCode = CodeCommand((short int)JUMP_TO_BATT,Codice);
				header_jump[0]=(char)(my_ID+1);
				header_jump[1]='\0';
				strcat(header_jump,Codice);
				
				infoCode = CodeCommand((short int)NumBat,CodiceBatJump);
				strcat(header_jump,CodiceBatJump);
				
				Array[0]='\0';
				//strcat(Array,header_jump);
				strcat(Array,header_jump);
				
				sendCode = SendMsg_ack_MaM(tid_PDLOO);
				if (sendCode!=OK_SEND) 
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","JumpToBatt()");
				
				id_PDLOO = netONCM.ListofNodes[id_PDLOO].Kids_id;
			}
    }
	indiceparte++; 
  }
  
  // --------------------------- ciclo per la gestione del direttore
  
  psp = GetFirstSpa();
  pbat = psp->GetBattNum(nb-1);
  while (pbat->GetNumPagMASE()==0)
  {
	  pbat = psp->GetPrevBat(pbat);
	  if (pbat==NULL) break;
  }
  
  if (pbat!=NULL) 
  {
	  NumBat = pbat->GetNumProgress();
	  pbat = psp->GetPrevBat(pbat);
	  if (pbat!=NULL) 
	  {
		  while (pbat->GetNumPagMASE()==0)
		  {
			  pbat = psp->GetPrevBat(pbat);
			  if (pbat==NULL) break;
		  }
	  }
	  if (pbat==NULL) pbat = psp->GetBattNum(0);
  }
  else 
  {
	  pbat = psp->GetBattNum(0);
	  NumBat = pbat->GetNumProgress();
  }
  
  NumLim = pbat->GetNumProgress();
  
  for(id =1;((id<MAXNODES)&&(netONCM.ListofNodes[id].Host!=NULL));id++)
  {
	  if (netONCM.CheckDirector(id))
	  {
		  tid_direttore = netONCM.FindPTID(id);
		  
		  netONCM.SetDxLimit(id,NumLim-1);
		  netONCM.SetSxLimit(id,NumLim-1);
		  
		  infoCode = CodeCommand((short int)INIT_EXEC,Codice);
		  header_command[0]=(char)(my_ID+1);
		  header_command[1]='\0';
		  strcat(header_command,Codice);
		  infoCode = CodeCommand((short int)NumeroBattute,CodiceNumBat);
		  strcat(header_command,CodiceNumBat);
		  
		  Array[0]='\0';
		  //strcat(Array,header_command);
		  strcpy(Array,header_command);
		  
		  num_spa = 0;
		  psp = GetFirstSpa();
		  while (psp!=NULL)
		  {  
			  num_spa++;
			  psp = GetNextSpa(psp);
		  }
		  
		  // trasmetto il numero di spartiti
		  infoCode = CodeCommand((short int)num_spa,Codice);
		  strcat(Array,Codice); 
		  
		  // aggiungo i numeric code    
		  psp=GetFirstSpa();
		  while (psp!=NULL)
		  {                          
			  infoCode = CodeCommand((short int)psp->GetNumericCode(),Codice);
			  strcat(Array,Codice); 
			  psp = GetNextSpa(psp);
		  }
		  
		  //-------------- trattamento lista delle parentesi
		  num_par=listaPar.GetNumObj();
		  infoCode = CodeCommand((short int)num_par,Codice);
		  strcat(Array,Codice); 
		  
		  pPar = listaPar.GetFirstPar();   
		  while (pPar!=NULL)
		  {
			  tipoparentesi = pPar->GetID();
			  inizioparentesi = pPar->GetPSpaStart()->GetNumericCode();
			  fineparentesi = pPar->GetPSpaEnd()->GetNumericCode();
			  numcodepar = pPar->GetNumericCode();
			  
			  infoCode = CodeCommand((short int)tipoparentesi,Codice);
			  strcat(Array,Codice); 
			  
			  infoCode = CodeCommand((short int)inizioparentesi,Codice);
			  strcat(Array,Codice); 
			  
			  infoCode = CodeCommand((short int)fineparentesi,Codice);
			  strcat(Array,Codice);   
			  
			  infoCode = CodeCommand((short int)numcodepar,Codice);
			  strcat(Array,Codice);   
			  
			  pPar = listaPar.GetNextPar(pPar);
		  }
		  
		  //--------------- aggiungo i nomi degli spartiti
		  psp=GetFirstSpa();
		  while (psp!=NULL)
		  {                                
			  strcat(Array,psp->GetStrumEsec());
			  strcat(Array,"\n"); 
			  psp = GetNextSpa(psp);
		  }
		  
		  sendCode = SendMsg_ack_MaM(tid_direttore);
		  if (sendCode!=OK_SEND) 
			  EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","JumpToBatt()");	 
		  
		  Send_NextPag(id,6);
		  
		  infoCode = CodeCommand((short int)JUMP_TO_BATT,Codice);
		  header_jump[0]=(char)(my_ID+1);
		  header_jump[1]='\0';
		  strcat(header_jump,Codice);
		  
		  infoCode = CodeCommand((short int)NumBat,CodiceBatJump);
		  strcat(header_jump,CodiceBatJump);
		  
		  Array[0]='\0';
		  //strcat(Array,header_jump);
		  strcpy(Array,header_jump);
		  
		  sendCode = SendMsg_ack_MaM(tid_direttore);
		  if (sendCode!=OK_SEND) 
			  EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","JumpToBatt()");
		  
		  // invio ai nodi non leader direttoriali il comando di inizio esecuzione 
		  id_kid=netONCM.ListofNodes[id].Kids_id;
		  while (id_kid!=0)
		  {
			  netONCM.SetDxLimit(id_kid,NumLim-1);
			  netONCM.SetSxLimit(id_kid,NumLim-1);
			  
			  tid_direttore = netONCM.FindPTID(id_kid);
			  
			  infoCode = CodeCommand((short int)INIT_EXEC,Codice);
			  header_command[0]=(char)(my_ID+1);
			  header_command[1]='\0';
			  strcat(header_command,Codice); 
			  infoCode = CodeCommand((short int)NumeroBattute,CodiceNumBat);
			  strcat(header_command,CodiceNumBat);
			  
			  Array[0]='\0';
			  //strcat(Array,header_command);
			  strcpy(Array,header_command);
			  
			  sendCode = SendMsg_ack_MaM(tid_direttore);
			  if (sendCode!=OK_SEND) 
				  EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","JumpToBatt()");
			  
			  Send_NextPag(id_kid,6);
			  
			  infoCode = CodeCommand((short int)JUMP_TO_BATT,Codice);
			  header_jump[0]=(char)(my_ID+1);
			  header_jump[1]='\0';
			  strcat(header_jump,Codice);
			  
			  infoCode = CodeCommand((short int)NumBat,CodiceBatJump);
			  strcat(header_jump,CodiceBatJump);
			  
			  Array[0]='\0';
			  //strcat(Array,header_jump);
			  strcpy(Array,header_jump);
			  
			  sendCode = SendMsg_ack_MaM(tid_direttore);
			  if (sendCode!=OK_SEND) 
				  EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","JumpToBatt()");
			  
			  id_kid = netONCM.ListofNodes[id_PDLOO].Kids_id;
			  
		  }
      }
   }
#else
#endif 
   
   
   psp = GetFirstSpa();
   pbat = psp->GetBattNum(nb-1);
   while (pbat->GetNumPagMASAE()==0)
   {
	   pbat = psp->GetPrevBat(pbat);
	   if (pbat==NULL) break;
   }
   
   if (pbat==NULL) 
	   pbat = psp->GetBattNum(0);
   
   NumBat = pbat->GetNumProgress();
   nb = NumBat;
   
   Scr->GSetMainDrawArea();
   GoBattInExecution(nb);
   DrawInExecution();
   //Redraw();
}





void MaMass::SendEndImp()
{
	
	// alla fine di ogni impaginazione devo salvare il nuovo file ett perche puo
	// essere cambiato qualcosa nella partitura
	
	char *s;
	char nome[FILE_NAME_LEN];
	
	strcpy(nome,GetFileSaveName());
	// Il nome puo' avere estensione .mds e .acl, quindi provo
	// entrambi i casi 
	s=strstr(nome,".mds");
	if (s!=NULL) (*(s+1))='\0';
	else {
		s=strstr(nome,".acl");
		if (s!=NULL) (*(s+1))='\0';
	}
	strcat(nome,".ett");
	
#ifdef WITH_NET   
	RetCode infoCode,sendCode;
	char Codice[DIM_COMMAND_STRING+1];
	char header_command[DIM_COMMAND_STRING+DIM_ID+2];
	int tid_LIOO_leader, tid_PDLOO;
	unsigned char indiceparte ,idleader,id_PDLOO;
	int tid_direttore;
	unsigned char id_kid;
	
	netONCM.RefreshTabella();
	
	// formazione header del comando da inviare
	infoCode = CodeCommand((short int)FINE_IMPAGINA,Codice);
	header_command[0]=(char)(my_ID+1);
	header_command[1]='\0';
	strcat(header_command,Codice);
	
	
	// ciclo per inviare a tutti i lioo che hanno una parte del comando di fine
	// processo di impaginazione
	indiceparte=1; 
	while (indiceparte <= (netONCM.NActiveParte))
	{
		if (netONCM.Pindex[indiceparte].Parte!=NULL) 
		{
			idleader=netONCM.Pindex[indiceparte].IdLeader;
			
			// ricerca del tid del leader
			tid_LIOO_leader = netONCM.FindPTID(idleader);
			
			// inserisco header del comando
			Array[0]='\0';
			strcat(Array,header_command);
			
			// invio messaggio di FINE al LIOO leader
			sendCode = SendMsg_ack_MaM(tid_LIOO_leader);
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","SendEndImp()");
				
			};
			
			// invio ai nodi PDLOO il comando di fine impaginazione 
			id_PDLOO=netONCM.ListofNodes[netONCM.Pindex[indiceparte].IdLeader].Kids_id;
			
			// ciclo per inviare la prima pagina anche ai LIOO non leader 
			while (id_PDLOO!=0)
			{
				tid_PDLOO= netONCM.FindPTID(id_PDLOO);
				
				// inserisco header del comando
				Array[0]='\0';
				strcat(Array,header_command);
				
				// invio comando
				sendCode = SendMsg_ack_MaM(tid_PDLOO);
				if (sendCode!=OK_SEND) 
				{
					// problemi di invio messaggio
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","SendEndImp()");
					
				}
				// L'ultimo DLIOO al quale e' associata quella parte ha Kids_id 0 che e' la 
				// condizione di fine ciclo 	    
				id_PDLOO=netONCM.ListofNodes[id_PDLOO].Kids_id;
				
			}
		}; 
		indiceparte++; 
	};
	
	
	// ------------------    ciclo per la gestione del direttore  ----------------
	for(unsigned char id =1;((id<MAXNODES)&&(netONCM.ListofNodes[id].Host!=NULL));id++)
	{
		if (netONCM.CheckDirector(id))
		{
			// ricerca del tid del leader
			tid_direttore = netONCM.FindPTID(id);
			
			// inserisco header del comando
			Array[0]='\0';
			strcat(Array,header_command);
			
			// invio comando
			sendCode = SendMsg_ack_MaM(tid_direttore);
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","SendEndImp()");
				
			}
			
			// invio ai nodi non leader direttoriali il comando di inizio esecuzione 
			id_kid=netONCM.ListofNodes[id].Kids_id;
			while (id_kid!=0)
			{
				tid_direttore = netONCM.FindPTID(id_kid);
				// inserisco header del comando
				Array[0]='\0';
				strcat(Array,header_command);
				
				// invio comando
				sendCode = SendMsg_ack_MaM(tid_direttore);
				if (sendCode!=OK_SEND) 
				{
					// problemi di invio messaggio
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","SendEndImp()");
					
				}
				
				id_kid=netONCM.ListofNodes[id_PDLOO].Kids_id;
			} 
		};
	}; 
	
	InitParte();  
	
#else
#endif 
}



void MaMass::SendEndEsec()
{
#ifdef WITH_NET   
	RetCode infoCode;
	RetCode sendCode;
	char Codice[DIM_COMMAND_STRING+1];  
	char nome[FILE_NAME_LEN];
	char header_command[DIM_COMMAND_STRING+DIM_ID+2];
	int tid_LIOO_leader, tid_PDLOO;
	unsigned char indiceparte ,idleader,id_PDLOO;
	int tid_direttore;
	unsigned char id_kid;
	
	
	// formazione header del comando da inviare
	infoCode = CodeCommand((short int)FINE_EXEC,Codice);
	header_command[0]=(char)(my_ID+1);
	header_command[1]='\0';
	strcat(header_command,Codice);
	
	// ciclo per inviare comando di fine esecuzione
	indiceparte=1; 
	while (indiceparte <= (netONCM.NActiveParte))
	{
		if (netONCM.Pindex[indiceparte].Parte!=NULL) 
		{
			idleader=netONCM.Pindex[indiceparte].IdLeader;
			
			// ricerca del tid del leader
			tid_LIOO_leader = netONCM.FindPTID(idleader);
			
			// inserisco header del comando
			Array[0]='\0';
			strcat(Array,header_command);
			
			// invio messaggio di FINE al LIOO leader
			sendCode = SendMsg_ack_MaM(tid_LIOO_leader);
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","SendEndEsec()");
			};
			
			// invio ai nodi PDLOO il comando di fine impaginazione 
			id_PDLOO=netONCM.ListofNodes[netONCM.Pindex[indiceparte].IdLeader].Kids_id;
			
			while (id_PDLOO!=0)
			{
				tid_PDLOO= netONCM.FindPTID(id_PDLOO);
				
				// inserisco header del comando
				Array[0]='\0';
				strcat(Array,header_command);
				
				// invio comando
				sendCode = SendMsg_ack_MaM(tid_PDLOO);
				if (sendCode!=OK_SEND) 
				{
					// problemi di invio messaggio
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","SendEndEsec()");
					
				}
				
				// L'ultimo DLIOO al quale e' associata quella parte ha Kids_id 0 che e' la 
				// condizione di fine ciclo 	    
				id_PDLOO=netONCM.ListofNodes[id_PDLOO].Kids_id;
				
			}
		}; 
		indiceparte++; 
	};
	
	// ------------------    ciclo per la gestione del direttore  ----------------
	for(unsigned char id =1;((id<MAXNODES)&&(netONCM.ListofNodes[id].Host!=NULL));id++)
	{
		if (netONCM.CheckDirector(id))
		{
			// ricerca del tid del leader
			tid_direttore = netONCM.FindPTID(id);
			
			// inserisco header del comando
			Array[0]='\0';
			strcat(Array,header_command);
			
			// invio comando
			sendCode = SendMsg_ack_MaM(tid_direttore);
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","SendEndEsec()");
				
			}
			
			// invio ai nodi non leader direttoriali il comando di inizio esecuzione 
			id_kid=netONCM.ListofNodes[id].Kids_id;
			while (id_kid!=0)
			{
				tid_direttore = netONCM.FindPTID(id_kid);
				
				// inserisco header del comando
				Array[0]='\0';
				strcat(Array,header_command);
				
				// invio comando
				sendCode = SendMsg_ack_MaM(tid_direttore);
				if (sendCode!=OK_SEND) 
				{
					// problemi di invio messaggio
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","SendEndEsec()");
					
				}
				id_kid=netONCM.ListofNodes[id_PDLOO].Kids_id;
				
			} 
		};
	}; 
	
#else
#endif 
} 



// Ok per inviare le battute dell'impaginazione ai LIOO
// da modificare per i direttori
void MaMass::Send_NBat(int nbat,unsigned char id_mitt)  
{   
#ifdef WITH_NET   
	RetCode infoCode;
	char Codice[DIM_COMMAND_STRING+1];   	
	char header_command[DIM_COMMAND_STRING+DIM_ID+2];
	int tid_mittente;
	RetCode info_ack;
	Battuta* PBattuta, *pbt;
	RetCode  sendCode;
	Spartito *psp;
	Bool Direttore;
	char finebatt;
	char* descr_bat;
	int flag;
	int nb; Battuta *pcbat;
	
	// controllo se ho un direttore oppure no
	Direttore=netONCM.CheckDirector(id_mitt);
	
	// formazione header del comando da inviare
	infoCode = CodeCommand((short int)ESEGUI_BATT_IMP,Codice);
	header_command[0]=(char)(my_ID+1);
	header_command[1]='\0';
	strcat(header_command,Codice);
	
	
	// invio delle prime battute per l'impaginazione  
	int nbatsent=1;
	while (nbatsent<=nbat)  
	{	
		// Acquisizione  TID del mittente    
		tid_mittente = netONCM.FindPTID(id_mitt);
		
		// 1--MaMass identifica lo spartito
		if (Direttore)
			psp = GetSpartNum(0); 
		else
			psp = GetSpartNum(netONCM.ListofNodes[id_mitt].IDP-1); 
		if (psp==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA, "EC_SPA_NOTFOUND","MaMass","Send_Nbat()");
			// gestione errore non trovato lo spartito
			return;
		} 
		
		// 2--MaMass cerca la battuta assegnata nello spartito  selezionato
		PBattuta = psp->GetBattNum(netONCM.GetDxLimit(id_mitt));
		
		// finebatt viene settato a 1 (che equivale a wait=FALSE nel DLIOO) quando 
		// si e' raggiunto il numero di pagine che si vuole inviare oppure
		// quando si e' raggiunta la fine della partitura
		if (PBattuta==NULL) // se la battuta esiste 
		{
			if (GetImpaginazione())
			{
				Array[0]=(char)(my_ID+1);
				Array[1]='\0';
				infoCode = CodeCommand((short int)IMPAGINA_ENDBATTUTE,Codice);
				strcat(Array,Codice);
				
				// invio dell'informazione
				sendCode = SendMsg_ack_MaM(tid_mittente);
				if (sendCode!=OK_SEND)
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","Send_NBat()");
			}
			break; // cosi esco subito dal while
		}
		
		finebatt=1;
		
		//  controllo se la battuta successiva e' quella finale(NULL) o l'ultima che
		//  devo inviare
		pbt = psp->GetNextBat(PBattuta);
		if (nbat==nbatsent) finebatt = BATTUTE_COMPLETATE;
		else if (pbt==NULL)
		{
			finebatt = BATTUTE_TERMINATE;
			nbatsent = nbat + 1; // cosi esco subito dal while
		}
		else  finebatt = BATTUTE_CONT;
		
		Array[0]='\0';
		strcat(Array,header_command);
		Array[DIM_ID+DIM_COMMAND_STRING] = finebatt;
		Array[DIM_ID+DIM_COMMAND_STRING+1] = '\0';
		
		
		nb=1;
		// determino la descrizione della battuta
		if (!Direttore)
		{
			if (PBattuta!=NULL) // se la battuta esiste 
			{
				if(PBattuta->GetIsMultiRest())
				{
					//  una battuta che fa parte di una multirest...invio allora direttamente
					// la battuta multirest che mi sono creata al caricamento.
					
					pcbat=PBattuta->GetPMultiRest();
					if(pcbat!=NULL)
					{
						nb=pcbat->GetNumMultiRest(); 
						strcat(Array,psp->Describe(pcbat)); 
						pcbat->GetNumMultiRest(),pcbat->GetNumPagDLIOO());
					}
				} 
				else
				{
					descr_bat = psp->Describe(PBattuta);
					strcat(Array,descr_bat);
					PBattuta->GetNumPagDLIOO());
				}
			}   
		}
		else
		{
			while (psp!=NULL)
			{
				PBattuta=psp->GetBattNum(netONCM.GetDxLimit(id_mitt));
				if (flag==0) 
				{
					strcat(Array,Codice);  
					flag=1;
				}
				strcat(Array,psp->Describe(PBattuta));			
				psp=GetNextSpa(psp);	
			}
		} 
		
		// invio della battuta
		sendCode = SendMsg_ack_MaM(tid_mittente);
		if (sendCode!=OK_SEND)
			EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","Send_NBat()");
		
		netONCM.SetDxLimit(id_mitt,netONCM.GetDxLimit(id_mitt)+nb);
		nbatsent++;
    }; 
#else
#endif 
};  





//
// Invia "npag" pagine al DLIOO (se ci sono "npag" pagine).
//  Usata in fase di esecuzione
//
void MaMass::Send_NextPag(unsigned char id_mitt, int npag)  
{   
#ifdef WITH_NET   
	char Codice[DIM_COMMAND_STRING+1];   	  
	char header_command[DIM_COMMAND_STRING+DIM_ID+2];
	char finebatt;
	char* descr_bat;
	unsigned short int time;
	int tid_mittente;
	int NumPagSend=0;
	int nbatsent;
	Bool IsDirector;
	int nb;
	Battuta *pcbat;
	
	RetCode infoCode, info_ack, sendCode;
	Battuta* PBattuta;  	
	Spartito *psp;
	
	// controllo se e un direttore
	IsDirector = netONCM.CheckDirector(id_mitt);
	
	// 1--MaMass identifica lo spartito
	if (IsDirector)
		psp=GetSpartNum(0);
	else
		psp = GetSpartNum(netONCM.ListofNodes[id_mitt].IDP-1); 
	
	if (psp==NULL)
	{
		EM.Error(MUSICAL_ERR_AREA, "EC_SPA_NOTFOUND","MaMass","Send_NextPage()");
		// gestione errore non trovato lo spartito
		return;
	} 
	
	nbatsent = netONCM.GetDxLimit(id_mitt);
	PBattuta = psp->GetBattNum(nbatsent);
	
	// formazione header del comando  
	infoCode = CodeCommand((short int)ESEGUI_BATT_EXEC,Codice);
	header_command[0]=(char)(my_ID+1);
	header_command[1]='\0';
	strcat(header_command,Codice);
	
	// Acquisizione  TID del mittente    
	tid_mittente = netONCM.FindPTID(id_mitt);
	
	do 
	{
		finebatt=1;
		
		// pulisco l'Array 
		Array[0]='\0';
		
		if (PBattuta!=NULL) // se la battuta esiste 
		{
			// formazione della battuta da inviare
			strcat(Array,header_command);
			
			// finebatt viene settato a 1 (che equivale a wait=FALSE nel DLIOO) quando 
			// si e' raggiunto il numero di pagine che si vuole inviare oppure
			// quando si e' raggiunta la fine della partitura
			if ((psp->GetBattNum(nbatsent+1))==NULL)
				finebatt = BATTUTE_TERMINATE;
			else 
			{
				if (IsDirector)
				{
					if (psp->GetBattNum(nbatsent+1)->GetNumPagMASE()) 
						if (NumPagSend == (npag-1)) finebatt = BATTUTE_COMPLETATE;
						else finebatt = BATTUTE_CONT;
						else
							finebatt = BATTUTE_CONT;
				}
				else
				{
					if (psp->GetBattNum(nbatsent+1)->GetNumPagDLIOO())
						if (NumPagSend == (npag-1)) finebatt = BATTUTE_COMPLETATE;
						else finebatt = BATTUTE_CONT;
						else 
							finebatt = BATTUTE_CONT;
				}
			}
			Array[DIM_ID+DIM_COMMAND_STRING] = finebatt;
			Array[DIM_ID+DIM_COMMAND_STRING+1] = '\0';
			
			// inserimento time di esecuzione della battuta che devo inviare
			
			time=GetTimeExec(nbatsent+1);
			
			short int time1,time2;
			time1= (short int) ((time & 0xff00) / 256);
			time2= (short int) (time & 0x00ff);
			infoCode = CodeCommand(time1,Codice);
			strcat(Array,Codice);
			infoCode = CodeCommand(time2,Codice);
			strcat(Array,Codice);
			
			
			// inserisco il numero della battuta che invio 
			infoCode = CodeCommand(nbatsent,Codice);
			strcat(Array,Codice);
			
			// descrizione della battuta 
			
#ifdef DEBUG_TIME
			struct timeval timevI, timevF;
			struct timezone timezI, timezF;
			double time;
#endif		   
			
			nb=1;
			if (IsDirector)
			{
#ifdef DEBUG_TIME
				gettimeofday(&timevI,&timezI);
#endif		   
				while (psp!=NULL)
				{
					PBattuta=psp->GetBattNum(netONCM.GetDxLimit(id_mitt));
					strcat(Array,psp->Describe(PBattuta));			
					psp=GetNextSpa(psp);	
				}
				
#ifdef DEBUG_TIME
				gettimeofday(&timevF,&timezF);
				time = (double)((double)((timevF.tv_sec-timevI.tv_sec)*1000000 + timevF.tv_usec - timevI.tv_usec)/1000);
#endif		   
				
				psp=GetSpartNum(0);
				if (psp==NULL)
				{
					EM.Error(MUSICAL_ERR_AREA, "EC_SPA_NOTFOUND","MaMass","Send_NextPage()");
					// gestione errore non trovato lo spartito
					return;
				} 
				
			}
			else
			{
#ifdef DEBUG_TIME
				gettimeofday(&timevI,&timezI);
#endif		   
				if (PBattuta!=NULL) // se la battuta esiste 
				{
					if(PBattuta->GetIsMultiRest())
					{
						//  una battuta che fa parte di una multirest...invio allora direttamente
						// la battuta multirest che mi sono creata al caricamento.
						
						pcbat=PBattuta->GetPMultiRest();
						if(pcbat!=NULL)
						{
							nb=pcbat->GetNumMultiRest();  
							strcat(Array,psp->Describe(pcbat)); 
							pcbat->GetNumMultiRest(),pcbat->GetNumPagDLIOO());
							
						}
					}
					else
					{ 
						strcat(Array,psp->Describe(PBattuta));
						PBattuta->GetNumPagDLIOO());
					} 
				}
#ifdef DEBUG_TIME
				gettimeofday(&timevF,&timezF);
				time = (double)((double)((timevF.tv_sec-timevI.tv_sec)*1000000 + timevF.tv_usec - timevI.tv_usec)/1000);
#endif
				
			}
			
			// invio della battuta
			sendCode = SendMsg_ack_MaM(tid_mittente);
			
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","SendNextPage()");
				
			}  
			netONCM.SetDxLimit(id_mitt,netONCM.GetDxLimit(id_mitt)+nb);   
     }
	 
     nbatsent+=nb;
	 
	 // controllo del numero di pagine inviate e controllo per quando siamo 
	 // all'ultima pagina che puo' anche non essere completa        
     PBattuta = psp->GetBattNum(nbatsent);
     if (PBattuta==NULL) // se la battuta successiva a quella inviata 
     {                    // non esiste 
		 if (GetEsecuzione())
		 {
			 Array[0]=(char)(my_ID+1);
			 Array[1]='\0';
			 infoCode = CodeCommand((short int)EXEC_ENDBATTUTE,Codice);
			 strcat(Array,Codice);
			 
			 // invio dell'informazione
			 sendCode = SendMsg_ack_MaM(tid_mittente);
			 if (sendCode!=OK_SEND) 
			 {
				 EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","Send_NextPag()");
				 
			 }
		 }
		 NumPagSend = npag;  // cosi' esco dal ciclo
	 }
	 else
	 {
		 if (IsDirector)
		 {
			 if (psp->GetBattNum(nbatsent)->GetNumPagMASE())
				 NumPagSend++;
		 }
		 else
			 if (psp->GetBattNum(nbatsent)->GetNumPagDLIOO())
				 NumPagSend++;
	 }
	 
  } while (NumPagSend < npag);
  
#else
#endif 
}  




void MaMass::Send_AltExec()
{
#ifdef WITH_NET   
	RetCode infoCode;
	RetCode sendCode;
	char Codice[DIM_COMMAND_STRING+1], nome[FILE_NAME_LEN];
	char header_command[DIM_COMMAND_STRING+DIM_ID+2];
	int tid_LIOO_leader, tid_PDLOO;
	unsigned char indiceparte ,idleader,id_PDLOO;
	
	// formazione header del comando da inviare
	infoCode = CodeCommand((short int)ALT_EXEC,Codice);
	header_command[0]=(char)(my_ID+1);
	header_command[1]='\0';
	strcat(header_command,Codice);
	
	indiceparte=1; 
	while (indiceparte <= (netONCM.NActiveParte))
	{
		if (netONCM.Pindex[indiceparte].Parte!=NULL) 
		{
			idleader=netONCM.Pindex[indiceparte].IdLeader;
			
			// ricerca del tid del leader
			tid_LIOO_leader = netONCM.FindPTID(idleader);
			
			
			// inserisco header del comando
			Array[0]='\0';
			strcat(Array,header_command);
			
			
			// invio messaggio di ALT al LIOO leader
			sendCode = SendMsg_ack_MaM(tid_LIOO_leader);
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","Send_AltExec()");         
				//return ;  
			};
			
			// invio ai nodi PDLOO il comando di ALT
			id_PDLOO=netONCM.ListofNodes[netONCM.Pindex[indiceparte].IdLeader].Kids_id;
			while (id_PDLOO!=0)
			{  
				tid_PDLOO= netONCM.FindPTID(id_PDLOO);
				
				// inserisco header del comando
				Array[0]='\0';
				strcat(Array,header_command);
				
				// invio comando
				sendCode = SendMsg_ack_MaM(tid_PDLOO);
				if (sendCode!=OK_SEND) 
				{
					// problemi di invio messaggio
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","Send_AltExec()");	     
					
				}
				
				// L'ultimo DLIOO al quale e' associata quella parte ha Kids_id 0 che e' la 
				// condizione di fine ciclo 	    
				id_PDLOO=netONCM.ListofNodes[id_PDLOO].Kids_id;
				
			}
			
		}; 
		indiceparte++; 
	};
	
	// ------------------    ciclo per la gestione del direttore  ----------------
	int tid_direttore;
	unsigned char id_kid;
	for(unsigned char id =1;((id<MAXNODES)&&(netONCM.ListofNodes[id].Host!=NULL));id++)
	{
		if (netONCM.CheckDirector(id))
		{
			// ricerca del tid del leader
			tid_direttore = netONCM.FindPTID(id);
			
			// inserisco header del comando
			Array[0]='\0';
			strcat(Array,header_command);
			
			// invio comando
			sendCode = SendMsg_ack_MaM(tid_direttore);
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","Send_AltExec()");
				
			}
			
			// invio ai nodi non leader direttoriali il comando di inizio esecuzione 
			id_kid=netONCM.ListofNodes[id].Kids_id;
			while (id_kid!=0)
			{
				tid_direttore = netONCM.FindPTID(id_kid);
				
				// inserisco header del comando
				Array[0]='\0';
				strcat(Array,header_command);
				
				// invio comando
				sendCode = SendMsg_ack_MaM(tid_direttore);
				if (sendCode!=OK_SEND) 
				{
					// problemi di invio messaggio
					EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","Send_AltExec()");
					
				}
				
				id_kid=netONCM.ListofNodes[id_PDLOO].Kids_id;
				
			} 
			
		};
	}; 
	
#else
	
#endif 
	
}


// lettura dalla rete .Interrogazione del buffer dei messaggi della pvm
// Viene chiamata in modo asincrono ogni PvmDatatime
void MaMass::ReadFromNetwork()
{
	
#ifdef WITH_NET   
	
	
	RetCode infoCode, info_ack, sendCode;
	
	char Codice[DIM_COMMAND_STRING+1], Ack[DIM_ACK+1], r, p;
	int tid_mittente, Numbat, j, numPag, nbat, nfig;
	unsigned char id_mitt, ID;
	
	Battuta *pBat;
	Spartito *psp;
	Figura *pfig;
	
	
	int nb;
	Battuta *pbt;
	
	//interrogazione dalla rete per vedere se ci sono messaggi presenti
	RetCode reccode = NO_MSG;
	reccode = Receive(); 
	if (reccode == NO_MSG) return;
	
	// decodifica tipo di messaggio ricevuto
	short int CommandCode, CommandCode1;
	char CommandStringa[DIM_COMMAND_STRING+1];
	strncpy(CommandStringa,Array+DIM_ID,DIM_COMMAND_STRING);
	CommandStringa[DIM_COMMAND_STRING]='\0'; 
	CommandCode = DecodeCommand(CommandStringa);
	
	// Acquisizione ID mittente ultimo messaggio
	id_mitt=(Array[0])-1;
	
	
	// Acquisizione  TID del mittente    
    if (id_mitt==OPD_ID)
		tid_mittente = OPD_TID;
    else
		tid_mittente = netONCM.FindPTID(id_mitt);
	
	
	FormaAck(my_ID,Array,'p',Ack);
	
	
	// invio ACK
	info_ack = SendAck(tid_mittente,Ack,MSG_ACK); 
	if (info_ack!=OK_SEND_ACK) 
	{
		EM.Error(NET_ERR_AREA,"EC_NO_OK","MaMass","ReadFromNetwork()");
		//problemi con l'ack         
	}
	
	
	int N=0;
	int MAX,ETICH;
	int indice;
	
	
	switch (CommandCode)
	{
	case (REQUESTJUMP):
		
		psp=GetFirstSpa();
		MAX=psp->GetNumeroBattute();
		ETICH=GetNumEtichette();
		
		Array[0] = my_ID+1 ;   // +1 per evitare problemi quando l' ID e zero
		Array[1] ='\0';
		
		// codifica comando richiesto: da short int --> in 4 BYTE
		infoCode = CodeCommand((short int)REQUESTJUMP,Codice);
		strcat(Array,Codice);
		
		infoCode = CodeCommand((short int)MAX,Codice);
		strcat(Array,Codice);
		
		infoCode = CodeCommand((short int)ETICH,Codice);
		strcat(Array,Codice);
		
		int a;
		indice=DIM_ID+(3*DIM_COMMAND_STRING);
		for (j=0;j<ETICH;j++)
		{
			Array[indice+j]=(char)GetEtichetta(j);
			Array[indice+j+1]='\0';
			a=GetBattEtichetta(Array[indice+j]);
			infoCode = CodeCommand((short int)a,Codice);
			strcat(Array,Codice);
			indice += DIM_COMMAND_STRING;
		}
		Array[indice+j]='\0';
		
		sendCode = SendMsg_ack_MaM(tid_mittente);
		if (sendCode!=OK_SEND) 
		{
			// problemi di invio messaggio
			EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","StartNodes()");
			
		}
		
		
		break;
		
	case (GEST_LABEL):
		psp=GetFirstSpa();
		ETICH=GetNumEtichette();
		
		Array[0] = my_ID+1 ;   // +1 per evitare problemi quando l' ID e zero
		Array[1] ='\0';
		
		// codifica comando richiesto: da short int --> in 4 BYTE
		infoCode = CodeCommand((short int)GEST_LABEL,Codice);
		strcat(Array,Codice);
		
		infoCode = CodeCommand((short int)ETICH,Codice);
		strcat(Array,Codice);
		
		indice=DIM_ID+(2*DIM_COMMAND_STRING);
		for (j=0;j<ETICH;j++)
		{
			Array[indice+j]=(char)GetEtichetta(j);
			Array[indice+j+1]='\0';
		}
		Array[indice+j]='\0';
		
		sendCode = SendMsg_ack_MaM(tid_mittente);
		if (sendCode!=OK_SEND) 
		{
			// problemi di invio messaggio
			EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","StartNodes()");
			
		}
		
		
		break;
		
		
		
	case (JUMPEDITING):
		strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
		CommandStringa[DIM_COMMAND_STRING]='\0'; 
		CommandCode1 = DecodeCommand(CommandStringa);
		psp=GetFirstSpa();
		MAX=psp->GetNumeroBattute();
		
		switch (CommandCode1)
		{
		case (NETGOTOP):
			
			N=1;
			JumpEditing(N,id_mitt);
			//invio messaggio per effettuare il Gotop() sul leggio
			Array[0] = my_ID+1 ;   // +1 per evitare problemi quando l' ID e zero
			Array[1] ='\0';
			
			// codifica comando richiesto: da short int --> in 4 BYTE
			
			infoCode = CodeCommand((short int)NETGOTOP,Codice);
			strcat(Array,Codice);
			sendCode = SendMsg_ack_MaM(tid_mittente);
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","StartNodes()");
				
			}
			
			
			break;
			
		case (NETGOBOTTOM):
			N=MAX;
			JumpEditing(N,id_mitt);	    
			Array[0] = my_ID+1 ;   // +1 per evitare problemi quando l' ID e zero
			Array[1] ='\0';
			
			// codifica comando richiesto: da short int --> in 4 BYTE
			
			infoCode = CodeCommand((short int)NETGOBOTTOM,Codice);
			strcat(Array,Codice);
			sendCode = SendMsg_ack_MaM(tid_mittente);
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","StartNodes()");
				
			}
			
			
			break;
			
		case (NETGOPIUCINQUE):
			strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
			CommandStringa[DIM_COMMAND_STRING]='\0'; 
			N = DecodeCommand(CommandStringa);
			if (N>MAX) N=MAX;
            JumpEditing(N,id_mitt);
			break;
			
		case (NETGOMENOCINQUE):
			strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
			CommandStringa[DIM_COMMAND_STRING]='\0'; 
			N = DecodeCommand(CommandStringa);
			if (N<1) N=1;
            JumpEditing(N,id_mitt);
			break;
			
		case (NETGOLABEL):
			break;
		case (NETGOBATTUTA):
			strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
			CommandStringa[DIM_COMMAND_STRING]='\0'; 
			N = DecodeCommand(CommandStringa);
			if (N<1) N=1;
			else if (N>MAX) N=MAX;
			
			JumpEditing(N,id_mitt);
			// invio comando di riposizionarsi alla battuta N
			Array[0] = my_ID+1 ;   // +1 per evitare problemi quando l' ID e zero
			Array[1] ='\0';
			
			// codifica comando richiesto: da short int --> in 4 BYTE
			
			infoCode = CodeCommand((short int)NETGOBATTUTA_N,Codice);
			strcat(Array,Codice);
			
			infoCode = CodeCommand((short int)N,Codice);
			strcat(Array,Codice);
			
			sendCode = SendMsg_ack_MaM(tid_mittente);
			if (sendCode!=OK_SEND) 
			{
				// problemi di invio messaggio
				EM.Error(NET_ERR_AREA,"EC_NO_SEND","MaMass","StartNodes()");
				
			}
			
			
			break;
			
		default:
			break; 
		};
		break;
		
		case (ERROR):
			// gestione messaggio di errore dai DLIOO
			// Formato messaggio: id - ERROR - OnFile - OnScreen - Message 
			
			Bool OnFile,OnScreen; 
			FILE* LogFile;
			
			strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
			CommandStringa[DIM_COMMAND_STRING]='\0'; 
			OnFile = DecodeCommand(CommandStringa);    
			
			strncpy(CommandStringa,Array+DIM_ID+(2*DIM_COMMAND_STRING),DIM_COMMAND_STRING);
			CommandStringa[DIM_COMMAND_STRING]='\0'; 
			OnScreen = DecodeCommand(CommandStringa);    
			
			strcat(Array,"\n\nNODO : ");
			strcat(Array,netONCM.ListofNodes[id_mitt].nome_nodo);
			strcat(Array,"\nPLAYER : ");
			strcat(Array,netONCM.ListofNodes[id_mitt].nome_player);
			strcat(Array,"\nPARTE : ");
			strcat(Array,netONCM.ListofNodes[id_mitt].Parte);
			strcat(Array,"\n");
			
			
			OnFile=TRUE;
			
			
			if (OnScreen) Message(Array+DIM_ID+(3*DIM_COMMAND_STRING));
			if (OnFile)
			{
				char buffer[256];
				sprintf(buffer, "%s/%s", smrgetenv("LIOO_LOG_PATH"), LOGFILE_NAME);
				
				
				if (!(LogFile = fopen(buffer, "a")))
					EM.Error(EC_OPENFILEWRITE, "ErrorManager::ErrorOnFile","Cannot open log file");
				else 
				{ 
					fprintf(LogFile, "Errore -> %s\n",Array+DIM_ID+(3*DIM_COMMAND_STRING));      
					fclose(LogFile);
				} 
			}
			break;
			
		case (FINE_IMPAGINA):
			// decremento il flag che indica quanri liooleader devono ancora terminare l'impaginazione 
			NDliooInImp--;
			// quando tuuti hanno effettuato l'impaginazione, anche l'archivista puo'fare l'impaginazione 
			if (NDliooInImp==0)
				pMusicWin->ScrollAll();
			break;
			
		case (SET_PAGE):
			// determino il numero di pagine(numPag) che devo settare 
			strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
			CommandStringa[DIM_COMMAND_STRING]='\0'; 
			numPag = DecodeCommand(CommandStringa);    
			
			if (netONCM.CheckDirector(id_mitt))
			{
				// determino lo spartito per il quale setto le pagine
				psp = GetFirstSpa();
				if (psp==NULL)
				{
					EM.Error(MUSICAL_ERR_AREA, "EC_SPA_NOTFOUND","MaMass","ReadFromNetwork()");
					return;
				} 
				// ciclo per marcare le battute delle pagine 
				for (j=0;j<numPag;j++)
				{
					// determino in quale battuta devo settare la marcatura della pagina
					strncpy(CommandStringa,Array+DIM_ID+(2+2*j)*DIM_COMMAND_STRING,DIM_COMMAND_STRING);
					CommandStringa[DIM_COMMAND_STRING]='\0'; 
					pBat = psp->GetBatt(DecodeCommand(CommandStringa));
					
					if (pBat!=NULL)
					{
						// determino che valore di pagina devo settare per quella battuta
						strncpy(CommandStringa,Array+DIM_ID+(2+2*j+1)*DIM_COMMAND_STRING,DIM_COMMAND_STRING);
						CommandStringa[DIM_COMMAND_STRING]='\0'; 
						
						pBat->SetNumPagMASE(DecodeCommand(CommandStringa));
					}
					else
						EM.Error(MUSICAL_ERR_AREA, "EC_BAT_NOTFOUND","MaMass","ReadFromNetwork()");
				}  
				
			}
			else
			{
				// determino lo spartito per il quale setto le pagine
				psp = GetSpartNum(netONCM.ListofNodes[id_mitt].IDP-1); 
				if (psp==NULL)
				{
					EM.Error(MUSICAL_ERR_AREA, "EC_SPA_NOTFOUND","MaMass","ReadFromNetwork()");
					// gestione errore non trovato lo spartito
					return;
				} 
				
				// ciclo per marcare le battute delle pagine 
				for (j=0;j<numPag;j++)
				{
					// determino in quale battuta devo settare la marcatura della pagina
					strncpy(CommandStringa,Array+DIM_ID+(2+2*j)*DIM_COMMAND_STRING,DIM_COMMAND_STRING);
					CommandStringa[DIM_COMMAND_STRING]='\0'; 
					pBat = psp->GetBatt(DecodeCommand(CommandStringa));
					
					if (pBat!=NULL)
					{
						// determino che valore di pagina devo settare per quella battuta
						strncpy(CommandStringa,Array+DIM_ID+(2+2*j+1)*DIM_COMMAND_STRING,DIM_COMMAND_STRING);
						CommandStringa[DIM_COMMAND_STRING]='\0'; 
						
						if(pBat->GetIsMultiRest())
						{
							Battuta *pcbat=pBat->GetPMultiRest();
							if(pcbat!=NULL)
							{
								pcbat->SetNumPagDLIOO(DecodeCommand(CommandStringa));
							}
						}
						pBat->SetNumPagDLIOO(DecodeCommand(CommandStringa));
					}
					else
						EM.Error(MUSICAL_ERR_AREA, "EC_BAT_NOTFOUND","MaMass","ReadFromNetwork()");
				}  
			}
			break;
			
		case (REQUEST_NBAT):             
			// decodifica numero di battute richieste in avanti (per l'impaginazione)
			strncpy(CommandStringa,Array+DIM_ID+DIM_COMMAND_STRING,DIM_COMMAND_STRING);
			CommandStringa[DIM_COMMAND_STRING]='\0'; 
			Numbat = DecodeCommand(CommandStringa);
			
			// invio dedlle battute richieste (per l'impaginazione) 
			Send_NBat(Numbat,id_mitt);
			break;
			
		case (EXEC_PAGE_FORWARD):
			// invio della pagina successiva (in fase di esecuzione)
			Send_NextPag(id_mitt);
			
		case (REQUEST_BATTUTA_BACKWARD): 
			p = Array[5] - 1;
			r = Array[6] - 1;
			
			BattutaBackward(id_mitt,FALSE,r);
			
			if (p > 0)
				netONCM.SetDxLimit(id_mitt,netONCM.GetDxLimit(id_mitt)-p);
			
			break;
			
		case (REQUEST_BATTUTA_FORWARD): 
			p = Array[5] - 1;
			r = Array[6] - 1;
			
			BattutaForward(id_mitt,ESEGUI_BATT,FALSE,r);
			
			if (p > 0)
				netONCM.SetSxLimit(id_mitt,netONCM.GetSxLimit(id_mitt)+p);
			
			break;
			
		case (REQUEST_EXEC_COMMAND):
			String2Sym(&Array[DIM_ID+DIM_COMMAND_STRING],NetSymCmd);
			
			
			if (DoNetSymCmd()) {
				Bool Verify=InserisciComandoACL(tid_mittente);
				DrawObject * CurObj;
				Bool InPage;
				
				if (CatchFigure(&NetSymCmd,(DrawObject *&)CurObj,&InPage)) 
					if (CurObj!=NULL) {
						CurObj->SetColor(MyLog.GetColor(&NetSymCmd,tid_mittente,netONCM.CheckDirector(id_mitt),Verify));
						// Cerco di capire se l'oggetto e' nella pagina
						
					}
					
					
					Redraw();
			}
			
			
			break;
			
			
		case (CAMBIA_ABIL):         // aggiunto per receive dal OPD
			
			unsigned char NODENAME[20];
			
			char NEWABILS[300];
			
			sscanf(&Array[DIM_ID+DIM_COMMAND_STRING],"%[^&]&%s",NODENAME,NEWABILS);
			
			Send_Abil(NODENAME, NEWABILS);
			netONCM.UpdateAbil(NODENAME,NEWABILS);
			
			
			break;
			
			
			
		default: 
			//gestione errore : messaggio non contemplato
			break;
    } 
	
	// pulisco l'Array        
	Array[0] = '\0';
	
#else
#endif 
	
}


Bool MaMass::InserisciComandoACL(int tid_mittente)
{
	signed int nbat=-1;
	signed int nfig=-1;
	signed int nbat2=-1;
	signed int nfig2=-1;
	
	Battuta *pBat;
	Spartito *psp;
	Figura *pfig;
	
	unsigned char ID;
	ID=netONCM.FindID(tid_mittente);     
	
	psp = GetSpart(NetSymCmd.Path1.Array[SPARTITO_PATH_LEVEL]);
	
	
	if (psp!=NULL)
	{
		pBat = psp->GetBatt(NetSymCmd.Path1.Array[BATTUTA_PATH_LEVEL]);
		if (pBat!=NULL)
		{
			nbat = pBat->GetNumProgress();      
			
			// controllo che l'inserimento sia una figura ...
			if (MyLog.IsFigureInsertion(NetSymCmd.CommandId)) 
			{
				// controllo che l'inserimento non sia in testa			
				if ((Bool)NetSymCmd.ArgV[5]) 
					nfig=0;
				else 
					nfig=(NumCode)NetSymCmd.ArgV[4];
			} else {
				pfig = pBat->GetFig(NetSymCmd.Path1.Array[LAYER_PATH_LEVEL], NetSymCmd.Path1.Array[FIGURA1_PATH_LEVEL]);
				if (pBat!=NULL) {
					nfig = pBat->GetPos(NetSymCmd.Path1.Array[LAYER_PATH_LEVEL], pfig);	  
				} 
			}
		}
		// Vediamo se trovo anche il secondo oggetto !
		psp = GetSpart(NetSymCmd.Path2.Array[SPARTITO_PATH_LEVEL]);   
		if (psp!=NULL) {
			pBat = psp->GetBatt(NetSymCmd.Path2.Array[BATTUTA_PATH_LEVEL]);
			if (pBat!=NULL) {
				nbat2 = pBat->GetNumProgress();
				if (!MyLog.IsFigureInsertion(NetSymCmd.CommandId)) 
				{		 
					// Se fosse inserimento figura il secondo oggetto non esisterebbe !
					pfig = pBat->GetFig(NetSymCmd.Path2.Array[LAYER_PATH_LEVEL], NetSymCmd.Path2.Array[FIGURA1_PATH_LEVEL]);
					if (pBat!=NULL) 
						nfig2 = pBat->GetPos(NetSymCmd.Path2.Array[LAYER_PATH_LEVEL], pfig);	  		         		  
				}
			}
		}
		
		
		Bool Conferma;
		Conferma = CheckTrustedCommand(ID,&NetSymCmd);
		MyLog.AddCommand(tid_mittente,
			netONCM.ListofNodes[ID].Host,
			netONCM.ListofNodes[ID].nome_player,
			netONCM.ListofNodes[ID].Parte,
			&NetSymCmd,
			nbat,nfig,nbat2,nfig2,
			netONCM.CheckDirector(ID),// Se  un Direttore  TRUE, altrimenti  FALSE
			Conferma // Se  un Comando con conferma  TRUE, altrimenti  FALSE
			);
		return Conferma;
	}     
	
	return False;
}

Bool MaMass::CheckTrustedCommand(int ID,SymCmd * MyCmd)
{
	// Ottengo la stringa delle abilitazioni !	
	char menu[MAX_STR_ABIL_LEN];
	netONCM.MakeImpostMenu(menu,ID);
	// Lo analizzo !
	ExtLiooAbil Abilitazioni;
	Abilitazioni.LoadFromString(menu);
	return MyLog.CheckAbilCommand(MyCmd,&Abilitazioni);  
}

void MaMass::InserisciComandoACLMaMass(SymCmd* SrcCmd)
{
	signed int nbat=-1;
	signed int nfig=-1;
	signed int nbat2=-1;
	signed int nfig2=-1;
	
	Battuta *pBat;
	Spartito *psp;
	Figura *pfig;
	
	unsigned char ID;
	ID=netONCM.FindID(0);     
	psp = GetSpart(UsrSymCmd.Path1.Array[SPARTITO_PATH_LEVEL]);
	
	
	if (psp!=NULL)
	{
		pBat = psp->GetBatt(UsrSymCmd.Path1.Array[BATTUTA_PATH_LEVEL]);
		if (pBat!=NULL)
		{
			
			nbat = pBat->GetNumProgress();
			// se inserimento di figura allora il codice numerico per pfig e'
			// (NumCode)UsrSymCmd.ArgV[4]
			// pero' se inserimento in testa ( (Bool)UsrSymCmd.ArgV[5] )
			// pfig non ha senso (cioe' la nuuova figura e' la prima )!
			
			
			// controllo che l'inserimento sia una figura ...
			if (MyLog.IsFigureInsertion(UsrSymCmd.CommandId)) 
			{
				// controllo che l'inserimento non sia in testa			
				if ((Bool)UsrSymCmd.ArgV[5]) 
					nfig=0;
				else 
					nfig=(NumCode)UsrSymCmd.ArgV[4];
			} else {
				pfig = pBat->GetFig(UsrSymCmd.Path1.Array[LAYER_PATH_LEVEL]-1, UsrSymCmd.Path1.Array[FIGURA1_PATH_LEVEL]);
				if (pfig!=NULL) {
					nfig = pBat->GetPos(UsrSymCmd.Path1.Array[LAYER_PATH_LEVEL]-1, pfig);	  
				} 
			}
		}
		// Vediamo se trovo anche il secondo oggetto !
		psp = GetSpart(UsrSymCmd.Path2.Array[SPARTITO_PATH_LEVEL]);   
		if (psp!=NULL) {
			pBat = psp->GetBatt(UsrSymCmd.Path2.Array[BATTUTA_PATH_LEVEL]);
			if (pBat!=NULL) {
				nbat2 = pBat->GetNumProgress();
				if (!MyLog.IsFigureInsertion(UsrSymCmd.CommandId)) 
				{		 
					// Se fosse inserimento figura il secondo oggetto non esisterebbe !
					pfig = pBat->GetFig(UsrSymCmd.Path2.Array[LAYER_PATH_LEVEL]-1, UsrSymCmd.Path2.Array[FIGURA1_PATH_LEVEL]);
					if (pBat!=NULL) 
						nfig2 = pBat->GetPos(UsrSymCmd.Path2.Array[LAYER_PATH_LEVEL]-1, pfig);	  		         		  
				}
			}
		}
		
		
		
		MyLog.AddCommand(0,
			"Local",
			"Archivist",
			"None",
			SrcCmd,
			nbat,nfig,nbat2,nfig2,
			FALSE,//netONCM.CheckDirector(ID),// Se  un Direttore  TRUE, altrimenti  FALSE
			True// Se  un Comando con conferma  TRUE, altrimenti  FALSE
			);
	}     
	
	return;
}




// metodo per inviare i cambiamenti della durata delle battute in esecuzione
void MaMass::SendVelocEsec(int d_time)
{
	
#ifdef WITH_NET   
	RetCode infoCode;
	char Codice[DIM_COMMAND_STRING+1];
	char Ack_atteso[DIM_ACK+1];
	
	// costruzione ed invio messaggio
	Array[0]=my_ID+1;
	Array[1]='\0';
	infoCode = CodeCommand((short int)DELTA_TIME,Codice);
	strcat(Array,Codice);  
	
	infoCode = CodeCommand((short int)d_time,Codice);
	strcat(Array,Codice);  
	
	// formazione dell'ack atteso   
	strncpy(Ack_atteso,Array,DIM_ACK);
	
	// reset dei flag di ack per i LIOO che attendono il messaggio  
	netONCM.ResetAllE_Ack();
	
	
	infoCode=SendMulticast(netONCM.ArrayOfTid,(netONCM.NActiveLIOO-1));
	if (infoCode!=OK_SEND) 
	{
		// problemi di invio messaggio
		EM.Error(NET_ERR_AREA,"EC_NO_MULTICAST","MaMass","DeltaTime()");
		
		
	};  
	
	// blocco per la ricezione degli ack del multicast  
	RetCode reccode = NO_MSG;
	unsigned char id_ack_ricevuto;
	
	while (netONCM.CheckE_Ack()==FALSE)
	{ 
		while (reccode==NO_MSG) 
			reccode=ReceiveAckMulticast(Ack_atteso,id_ack_ricevuto); 
		reccode = NO_MSG;
		netONCM.SetE_Ack(id_ack_ricevuto);
	}
	
	
#else
#endif 
	
}


// metodo per inviare il tipo di cursore ai leggii
void MaMass::SendTypeCursor(CursorTypes ct, int fc)
{
	
#ifdef WITH_NET   
	RetCode infoCode;
	char Codice[DIM_COMMAND_STRING+1];
	char Ack_atteso[DIM_ACK+1];
	
	// costruzione ed invio messaggio
	Array[0]=my_ID+1;
	Array[1]='\0';
	infoCode = CodeCommand((short int)TYPE_CURSOR,Codice);
	strcat(Array,Codice);  
	
	infoCode = CodeCommand((short int)ct,Codice);
	strcat(Array,Codice);  
	
	infoCode = CodeCommand((short int)fc,Codice);
	strcat(Array,Codice);  
	
	// formazione dell'ack atteso   
	strncpy(Ack_atteso,Array,DIM_ACK);
	
	// reset dei flag di ack per i LIOO che attendono il messaggio  
	netONCM.ResetAllE_Ack();
	
	infoCode=SendMulticast(netONCM.ArrayOfTid,(netONCM.NActiveLIOO-1));
	if (infoCode!=OK_SEND) 
	{
		// problemi di invio messaggio
		EM.Error(NET_ERR_AREA,"EC_NO_MULTICAST","MaMass","DeltaTime()");
		
		
	};  
	
	// blocco per la ricezione degli ack del multicast  
	RetCode reccode = NO_MSG;
	unsigned char id_ack_ricevuto;
	
	while (netONCM.CheckE_Ack()==FALSE)
	{ 
		while (reccode==NO_MSG) 
			reccode=ReceiveAckMulticast(Ack_atteso,id_ack_ricevuto); 
		
		reccode = NO_MSG;
		netONCM.SetE_Ack(id_ack_ricevuto);
	}
	
	
#else
#endif 
	
}


void MaMass::SendWaitState(Bool ws)
{
	
#ifdef WITH_NET   
	RetCode infoCode;
	char Codice[DIM_COMMAND_STRING+1];
	char* Ack_atteso=new char[DIM_ACK+1];
	if ((Ack_atteso)==NULL)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","MaMass","SendWaitState()");
	
	// costruzione ed invio messaggio
	Array[0]=my_ID+1;
	Array[1]='\0';
	infoCode = CodeCommand((short int)WAIT_TIME,Codice);
	strcat(Array,Codice);  
	infoCode = CodeCommand((short int)ws,Codice);
	strcat(Array,Codice);  
	
	// formazione dell'ack atteso   
    strncpy(Ack_atteso,Array,DIM_ACK);
	
	// reset dei flag di ack per i LIOO che attendono il messaggio  
	netONCM.ResetAllE_Ack();
	
	
	infoCode=SendMulticast(netONCM.ArrayOfTid,(netONCM.NActiveLIOO-1));
	if (infoCode!=OK_SEND) 
	{
		// problemi di invio messaggio
		EM.Error(NET_ERR_AREA,"EC_NO_MULTICAST","MaMass","DeltaTime()");
		
        
	};  
	
	// blocco per la ricezione degli ack del multicast  
    RetCode reccode = NO_MSG;
    unsigned char id_ack_ricevuto;
	
    while (netONCM.CheckE_Ack()==FALSE)
	{ 
		while (reccode==NO_MSG) 
			reccode=ReceiveAckMulticast(Ack_atteso,id_ack_ricevuto); 
		reccode = NO_MSG;
		netONCM.SetE_Ack(id_ack_ricevuto);
	}
	delete[] Ack_atteso;
	
	
#else
#endif 
	
}





//
//  ------------------------   funzioni di utilita'    ----------------------------
//


void MfStampaSym(FILE *pippo,SymCmd sc)
{ 
}

int conv(char *s)
{int i,n=0;
for(i=0; s[i]; i++)
n = n*10 + (s[i] - '0');
return n;
}







//  bottone debug.


void MaMass::EseguiDebug(void *arg1, void *arg2, void *arg3) 
{
	char ss[100];
	FILE *pippo;
	
	Spartito  *psp;
	Battuta *pbt;
	Colore oldcol;

	int j, nb;
	
	switch(UsrSymCmd.CommandId)
	{
    case CMD_DEBUG_UNO:
		//      legge ed esegue un comando da file!
		
		pippo = fopen("/home/antonio/command.txt","r");
		fscanf(pippo,"%s",ss);
		NetSymCmd.CommandId = (CmdID)conv(ss);
		fscanf(pippo,"%s",ss);
		NetSymCmd.SelectionType = (SelType)conv(ss);
		fscanf(pippo,"%s",ss);
		NetSymCmd.SelObjType = (SelObj)conv(ss);
		for (j=0;j<PATH_LENGTH;j++)
		{
			fscanf(pippo,"%s",ss);
			NetSymCmd.Path1.Array[j] = conv(ss);
		}
	       fscanf(pippo,"%s",ss);
		   NetSymCmd.Path1.Level = conv(ss);
		   
		   for (j=0;j<PATH_LENGTH;j++)
		   {
			   fscanf(pippo,"%s",ss);
			   NetSymCmd.Path2.Array[j] = conv(ss);
		   };
		   fscanf(pippo,"%s",ss);
		   NetSymCmd.Path1.Level = conv(ss);
		   
		   for (j=0;j<SYM_MAXARG;j++)
		   {
			   fscanf(pippo,"%s",ss);
			   NetSymCmd.ArgV[j] = conv(ss);
		   };
		   
		   fclose(pippo);
		   
		   DoNetSymCmd();
		   
		   break;
		   
		   
    case CMD_DEBUG_DUE:
		//      scrive un comando su file!
		
		pippo = fopen("/home/antonio/command.txt","w");
		MfStampaSym(pippo,NetSymCmd);
		
		fclose(pippo);
		
		break;
		
		
    case CMD_DEBUG_TRE:
		break;
		
    case CMD_DEBUG_QUATTRO:
		
		oldcol = Scr->GetColor();
		Scr->SetColor((ColorIcon)black);
		Scr->Rect(class Rectangle(Point(pMusicWin->GetAbsPos().x,500),
			Point(pMusicWin->GetAbsPos().x + pMusicWin->GetWidth()+1,550)));
		Scr->SetColor(oldcol);
		break;        
    case CMD_DEBUG_CINQUE:
		
		break;        
    case CMD_DEBUG_SEI:
		
		nb=0;
		psp=GetFirstSpa();
		pbt = psp->GetBattNum(nb);
		
		while(pbt!=NULL)
		{
			psp=GetFirstSpa();
			while(psp!=NULL)
			{
				pbt = psp->GetBattNum(nb);
				psp=GetNextSpa(psp);
			}
			
			nb++;
			psp=GetFirstSpa();
			pbt = psp->GetBattNum(nb);
		}
        
		break;        
    case CMD_DEBUG_SETTE:
		
		nb=0;
		psp=GetFirstSpa();
		pbt = psp->GetBattNum(nb);
		
		while(pbt!=NULL)
		{
			psp=GetFirstSpa();
			while(psp!=NULL)
			{
				pbt = psp->GetBattNum(nb);
				psp=GetNextSpa(psp);
			}
			
			nb++;
			psp=GetFirstSpa();
			pbt = psp->GetBattNum(nb);
		}
        
		break;        
		
    case CMD_DEBUG_OTTO:
		
        break;
		
    case CMD_DEBUG_NOVE:
        Scr->GSetMainDrawArea();
		pMusicWin->InitExec();
        break;
		
  }
  
}  

int MaMass::GetIDleader(char* parte)
{ 
	
	return netONCM.FindIDL(parte);
	
}

void MaMass::SetAbilCompress(char* parte, Bool abil)
{ 
	int i;
	for(i=0;i<MAXNODES;i++)
	{ 
		if(netONCM.ListofNodes[i].Parte!=NULL)
		{ 
			if(strcmp(parte,netONCM.ListofNodes[i].Parte)==0)
				netONCM.ListofNodes[i].Abil_Compress=abil;
		}
	}
}

void MaMass::SetAbilCompressAll(Bool abil)
{ 
	int i;
	
	for(i=0;i<MAXNODES;i++)
	{
		netONCM.ListofNodes[i].Abil_Compress=abil;
	}	 
}

Bool MaMass::GetAbil_Compress(int i)
{
	
	return netONCM.GetAbil_Compress(i);
	
}


