/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: netoncm.cpp,v 1.1 2005/07/25 16:31:42 bellinpo Exp $ */




#include "netoncm.hpp"
#include "lioo.hpp"
#include "liooext.hpp"

extern char *NomeEseguibile;
extern char *NomeNetStart;

#ifdef TINYMOODS
#define NOME_ESEGUIBILE_DLIOO "tinyMoods"
#else
#define NOME_ESEGUIBILE_DLIOO "Moods"
#endif

#define NOME_NETSTART "NetStart.cfg"


NetONCM::NetONCM()
{
#ifdef WITH_NET  
	
	ArrayOfTid=NULL;
	int i;
	for (i=0;i<MAXGROUPS;i++)
	{
		Pindex[i].Parte=NULL;
		Pindex[i].IdLeader=0; 
	}
	
	for (i=0;i<MAXNODES;i++)
		ListofNodes[i].StrAbil=NULL;
	
	
#else
#endif 
	
}

NetONCM::~NetONCM()
{
	
	
#ifdef DEBUG_FREE
	printf("\n\n\n ------------   DEALLOCO NETONCM  ----------------\n\n\n");
#endif 
	
#ifdef WITH_NET  
	unsigned char i;
	if (ArrayOfTid!=NULL) delete[] ArrayOfTid;
	
	for (i=0;i<NActiveLIOO;i++)
	{
		if  (ListofNodes[i].Host!=NULL) delete[] ListofNodes[i].Host ;
		if (ListofNodes[i].Ip!=NULL) delete[] ListofNodes[i].Ip ;
		if (ListofNodes[i].Type!=NULL) delete[] ListofNodes[i].Type ;
		if (ListofNodes[i].Display!=NULL) delete[] ListofNodes[i].Display ;
		if (ListofNodes[i].Parte!=NULL) delete[] ListofNodes[i].Parte ;
		if (ListofNodes[i].Device!=NULL) delete[] ListofNodes[i].Device;	
		if (ListofNodes[i].nome_nodo!=NULL) delete[] ListofNodes[i].nome_nodo;
		if (ListofNodes[i].nome_player!=NULL) delete[] ListofNodes[i].nome_player;
		if (ListofNodes[i].StrAbil!=NULL) delete[] ListofNodes[i].StrAbil;
	};
	
	
	for (i=0;i<=NActiveParte;i++)
		if (Pindex[i].Parte!=NULL) delete[] Pindex[i].Parte;
		
		
#else
#endif 
		
}


void NetONCM::InsertPindex(unsigned char idnp,int dim_parte,char* nome_parte,
                           unsigned char idleader)
{
	
#ifdef WITH_NET   
	
	// carica nome della parte   	
    Pindex[idnp].Parte=new char[dim_parte+1];
    if ((Pindex[idnp].Parte)==NULL)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","InsertPindex()");
	
	
    strcpy((Pindex[idnp].Parte),nome_parte);
	
	// carica rispettivo indice (nella tabella di configurazione) del leader
	// della parte caricata 
    Pindex[idnp].IdLeader=idleader;    
	
#endif 
	
} 

// restituisce l'indice della tabella che contiene la parte Pname	
unsigned char NetONCM::FindIDPDirector()
{
	
#ifdef WITH_NET   
	
	int k_index;
	int to_return;
	to_return=0;
	for (k_index=1;k_index<=NActiveLIOO;k_index++)
	{
		if (CheckDirector(k_index)) {
			to_return = k_index;
			break; 
		}      
	}
	// significa che non e' stata trovata e quindi restituisco il valore 0
	// che non e' significativo
    return ((unsigned char)to_return);
	
#else
	return 0;
#endif 
	
}


// Inizializza a TRUE il campo E_Ack per ogni entrata della tabella
void NetONCM::InitE_Ack()
{
#ifdef WITH_NET  
	int i;
	for (i=0;i<=MAXNODES;i++)
		ListofNodes[i].E_Ack=TRUE;
#else
#endif 
	
}


// Porta il campo E_Ack al valore FALSE,indicando che si deve ricevere l'ack
// del messaggio inviato con il multicast
void NetONCM::ResetAllE_Ack()
{
#ifdef WITH_NET  
	int i;
	
	for (i=1;i<NActiveLIOO;i++)
		ListofNodes[i].E_Ack=FALSE;
#else
#endif 
	
}


// Setta al valore TRUE il campo E_Ack indicando che per il leggio "id" e'
// stato ricevuto l'ack del multicast 
void NetONCM::SetE_Ack(unsigned char id)
{
#ifdef WITH_NET  
    ListofNodes[id].E_Ack=TRUE;
#else
#endif 
}


// controlla se tutti i leggii hanno ricevuto l'ack del messaggio multicast
// restituisce : TRUE se gli ack sono tutti arrivati
//               FALSE se almeno un ack non e' arrivato
Bool NetONCM::CheckE_Ack()
{
#ifdef WITH_NET  
	
	int i;
	i=1;
	while (((ListofNodes[i].E_Ack)) && (i<NActiveLIOO))    
		i++;
	
	if (i==NActiveLIOO) return(TRUE);
	else return (FALSE); 
	
#else
#endif
	return FALSE;
}


// Crea il vettore ArrayOfTid che contiene i tid dei processi LIOO lanciati
void NetONCM::BuildArrayOfTid()
{ 
#ifdef WITH_NET  
	
	int i;
	if (ArrayOfTid!=NULL) delete[] ArrayOfTid;
	ArrayOfTid=new int[NActiveLIOO-1]; 
	for (i=1;i<NActiveLIOO;i++)
		ArrayOfTid[i-1]=ListofNodes[i].PTID;
	
#else
#endif 
}  




void NetONCM::MakeImpostMenu(char * menu,unsigned char index_nodes)
{
#ifdef WITH_NET   
	
	unsigned char i=1;
	FILE *cfgfile;
	char s[200];
	char s1[HOST_NAME_LEN],s2[HOST_IP_LEN],s3[30],s4[HOST_DISPLAY_LEN];
	char s5[HOST_DEVICE_LEN],s6[GROUP_PARTE_LEN],s7[NODE_NAME_LEN],s8[NODE_PLAYER_LEN];
	int n0,n1,n2,n3;
	char *nmnetst;
	char tmp[100];
	if (NomeNetStart!=NULL) nmnetst=NomeNetStart;
	else nmnetst=NOME_NETSTART;
	
	if (ListofNodes[index_nodes].StrAbil!=NULL) {
		strncpy(menu,ListofNodes[index_nodes].StrAbil,MAX_STR_ABIL_LEN);
		return;
	}
	
	
	strcpy(tmp,"./");
	strcat(tmp,nmnetst);
	
	cfgfile=fopen(tmp,"r");
	if (cfgfile==NULL) 
	{ 
		char *env=getenv("NETSTART_PATH");
		if(env!=NULL)
			strcpy(tmp,env);
		else
			strcpy(tmp,"./");
		strcat(tmp,nmnetst);
		cfgfile=fopen(tmp,"r");
		if (cfgfile==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_OPENFILEREAD","NetONCM","LoadTabella");
			// gestione errore file
		}
	}
	
	char c;  
	fscanf(cfgfile,"%s\n",s);
	while (!(feof(cfgfile))&&(i<=index_nodes ))
	{
		if (fscanf(cfgfile,"%[^#]#%s %s %s %[^#]#%s %s %s",s1,s2,s3,s4,s5,s6,s7,s8)!=8)
			EM.Error(GENERAL_ERR_AREA,"EC_INFILE","NetONCM","LoadTabella");
		if (fscanf(cfgfile,"%d %d %d %d",&n0,&n1,&n2,&n3)!=4)
			EM.Error(GENERAL_ERR_AREA,"EC_INFILE","NetONCM","LoadTabella");
		if (fscanf(cfgfile,"%s ",menu)!=1)
			EM.Error(GENERAL_ERR_AREA,"EC_INFILE","NetONCM","LoadTabella");
		
		i++;  
	}	
	fclose(cfgfile);
	
	ListofNodes[index_nodes].StrAbil = new char[MAX_STR_ABIL_LEN];
	strcpy(ListofNodes[index_nodes].StrAbil,menu);
	
#else
#endif 
	
}



// lancia un lioo secondo le opzioni impostate
// NOTA:  Il processo deve essere presente nella $HOME/pvm3/bin/LINUX/
// vedi pg 149 del manuale PVM.
RetCode NetONCM::StartNode(unsigned char id)
{
#ifdef WITH_NET   
	
	int infospawn,ntask;
	unsigned short j;                               
	int tids[4];
	char* LineArgV[7];
	char *taskname;
	if (NomeEseguibile!=NULL) taskname = NomeEseguibile;
	else taskname = NOME_ESEGUIBILE_DLIOO;
	
	char *hostname,*moodstype;
	char stringa[NODE_PLAYER_LEN+NODE_NAME_LEN+GROUP_PARTE_LEN+1];
	char stringa2[NODE_PLAYER_LEN+NODE_NAME_LEN+GROUP_PARTE_LEN+1];
	
	// determinazione numero di processi da lanciare su ogni macchina
	ntask=1;
	
	// controllo ed impostazione del tipo di leggio(processo) che si deve lanciare
	if (CheckDirector(id)) 
	{
		LineArgV[0] = "-m"; 
		moodstype="MASE";
	}
	else if (CheckLeader(id))
	{
		LineArgV[0] = "-d";
		moodstype="DLIOO";
	}
	else
	{
		LineArgV[0] = "-p"; 
		moodstype="PDLOO";
	}
	
	// impostazione dei parametri per la caratterizzazione del leggio
	LineArgV[1]="-n";
	strcpy(stringa,ListofNodes[id].Parte);
	strcat(stringa,"    -    node: '");
	strcat(stringa,ListofNodes[id].nome_nodo);
	strcat(stringa,"'    -    player: '");
	strcat(stringa,ListofNodes[id].nome_player);
	strcat(stringa,"'");
	LineArgV[2] = stringa;
	
	
	strcpy(stringa2,"-x");
	strcat(stringa2,ListofNodes[id].Display);
	LineArgV[3] = stringa2;
	
	LineArgV[4] = NULL;
	
	
	
	// determinazione dell'host sul quale deve essere lanciato il processo
	hostname = ListofNodes[id].Host; 
	// creazione dei processi secondo i parametri determinati
	infospawn = pvm_spawn( taskname, LineArgV, PvmTaskHost, hostname, ntask, &tids[0]);
	if (infospawn==1)
	{
		// Nodes ToIns;
		//     ToIns.PTID = tids[0];
		//     ToIns.Host = hostname;
		
		SetTid(NActiveLIOO,tids[0]);
		NActiveLIOO++;
	}
	else
	{
		switch (tids[0])
		{
		case -6:
			EM.Error(PVM_ERR_AREA, "EC_PVM_NOHOST", "NetONCM", "StartNode");
			return(-6);
		case -7:
			EM.Error(PVM_ERR_AREA,"EC_PVM_NOFILE","NetONCM","StartNode");
			return(-7);
		case -10:
			EM.Error(PVM_ERR_AREA,"EC_PVM_NOMEMg","NetONCM","StartNode");
			return(-10);
		case -14:
			EM.Error(PVM_ERR_AREA,"EC_PVM_SYSERROR","NetONCM","StartNode");
			return(-14);
		case -27:
			EM.Error(PVM_ERR_AREA,"EC_PVM_NORES","NetONCM","StartNode");
			return(-27);
		default:
			EM.Error(PVM_ERR_AREA,"EC_PVM_UNDEF","NetONCM","StartNode");
			return(-40);
		} 
	}
	
	
#else
    return 0;
#endif 
	
}



void NetONCM::InitTabella()
{
	
#ifdef WITH_NET   
	
	int i;
	for (i=0;i<=MAXNODES;i++)
	{
		// 
		ListofNodes[i].Host = NULL;
		ListofNodes[i].Ip = NULL;
		ListofNodes[i].Type = NULL;
		
		
		ListofNodes[i].Display =NULL;
		ListofNodes[i].Device = 0;	//default 0=Mouse
		
		ListofNodes[i].Parte = NULL;   
		ListofNodes[i].nome_nodo= NULL;
		ListofNodes[i].nome_player= NULL;
		
		ListofNodes[i].Leader = 0;
		ListofNodes[i].Group_leader = 0;     
		ListofNodes[i].Kids_id=0;
		ListofNodes[i].IDP = 0;
		ListofNodes[i].PTID = 0;  
		ListofNodes[i].Ns=0;
		ListofNodes[i].Ne=0;
		ListofNodes[i].E_Ack=TRUE;
		ListofNodes[i].Abil_Compress=TRUE; 
	}
	NActiveLIOO = 1 ;	//perche' 1 e non 0	
	
	
#else
#endif 
	
}
// Utilizzato quando si carica un file per riordinare gli indici di visualizzazione
void NetONCM::RefreshTabella()
{
	
#ifdef WITH_NET   
	
	for (int i=0;i<=MAXNODES;i++)
	{
		ListofNodes[i].Ns=0;
		ListofNodes[i].Ne=0;
	}  
	
#else
#endif 
	
}

void NetONCM::SetAbil_Compress(unsigned char id, Bool abil)
{
#ifdef WITH_NET 
	
	ListofNodes[id].Abil_Compress=abil;
	
#endif 
}

Bool NetONCM::GetAbil_Compress(unsigned char id)
{
#ifdef WITH_NET 
	
	return ListofNodes[id].Abil_Compress;
	
#else
	return FALSE;
	
#endif 
}



// si carica la tabella leggendo il file NetStart
void NetONCM::LoadTabella()
{
	
#ifdef WITH_NET   
#ifdef TINYMOODS
	int mase_id=0;
	int dlioo_id=0;
	int id=1;
	
	char *mase_hst=getenv("TINYMOODS_MASE_HOSTNAME");
	if(mase_hst!=NULL)
		mase_id=id++;
	char *dlioo_hst=getenv("TINYMOODS_DLIOO_HOSTNAME");
	if(dlioo_hst!=NULL)
		dlioo_id=id++;
	
	if(mase_id!=0)
	{
		ListofNodes[mase_id].Host=new char[HOST_NAME_LEN];
		if ((ListofNodes[mase_id].Host)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		ListofNodes[mase_id].Ip=new char[HOST_IP_LEN];
		if ((ListofNodes[mase_id].Ip)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		ListofNodes[mase_id].Type=new char[30];
		if ((ListofNodes[mase_id].Type)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		ListofNodes[mase_id].Display=new char[HOST_DISPLAY_LEN];
		if ((ListofNodes[mase_id].Display)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		ListofNodes[mase_id].Device=new char[HOST_DEVICE_LEN];		 
		if ((ListofNodes[mase_id].Device)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		ListofNodes[mase_id].Parte=new char[GROUP_PARTE_LEN];		 
		if ((ListofNodes[mase_id].Device)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		ListofNodes[mase_id].nome_nodo=new char[NODE_NAME_LEN];
		if ((ListofNodes[mase_id].nome_nodo)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		ListofNodes[mase_id].nome_player=new char[NODE_PLAYER_LEN];
		if ((ListofNodes[mase_id].nome_player)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		ListofNodes[mase_id].StrAbil=new char[MAX_STR_ABIL_LEN];
		if ((ListofNodes[mase_id].StrAbil)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		char *mase_dsp=getenv("TINYMOODS_MASE_DISPLAY");
		if(mase_dsp==NULL)
			mase_dsp="1024x768";
		
		strcpy(ListofNodes[mase_id].Host,mase_hst);
		strcpy(ListofNodes[mase_id].Ip,"0.0.0.0");
		strcpy(ListofNodes[mase_id].Type,"MASE");
		strcpy(ListofNodes[mase_id].Display,mase_dsp);
		strcpy(ListofNodes[mase_id].Device,"mouse");
		strcpy(ListofNodes[mase_id].Parte,"Directors");
		strcpy(ListofNodes[mase_id].nome_nodo,"Director");
		strcpy(ListofNodes[mase_id].nome_player,"Someone");
		strcpy(ListofNodes[mase_id].StrAbil,"FILE:V;MODIFICA:V;CANCELLA:V;INTESTAZ:V;BATTUTA:V;PARTE:V;MOVIMENTO:V;LEGATURA:V;FORCELLA:V;OTTAVA:V;ONDA:V;CAMBIORIT:V;ACCENTI:V;ABBELLIM:V;TDINAMICO:V;TGENERICO:V;ANNOTAZDIR:V;CORONA:V;SUDDIVIS:V;DITEGGIATO:V;SORDINA:V;PRFIATO:V;OCCHIALI:V;VIOLINO:V;TIMPANO:V;OBOE:V;SAX:V;EDITEXEC:V;");
		ListofNodes[mase_id].Leader=(unsigned char) 1;
		ListofNodes[mase_id].Group_leader=(unsigned char) mase_id;
		ListofNodes[mase_id].Kids_id=(unsigned char) 0;
	}
	
	if(dlioo_id!=0)
	{  
		ListofNodes[dlioo_id].Host=new char[HOST_NAME_LEN];
		if ((ListofNodes[dlioo_id].Host)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		ListofNodes[dlioo_id].Ip=new char[HOST_IP_LEN];
		if ((ListofNodes[dlioo_id].Ip)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		ListofNodes[dlioo_id].Type=new char[30];
		if ((ListofNodes[dlioo_id].Type)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		ListofNodes[dlioo_id].Display=new char[HOST_DISPLAY_LEN];
		if ((ListofNodes[dlioo_id].Display)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		ListofNodes[dlioo_id].Device=new char[HOST_DEVICE_LEN];		 
		if ((ListofNodes[dlioo_id].Device)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		ListofNodes[dlioo_id].Parte=new char[GROUP_PARTE_LEN];		 
		if ((ListofNodes[dlioo_id].Device)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		ListofNodes[dlioo_id].nome_nodo=new char[NODE_NAME_LEN];
		if ((ListofNodes[dlioo_id].nome_nodo)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		ListofNodes[dlioo_id].nome_player=new char[NODE_PLAYER_LEN];
		if ((ListofNodes[dlioo_id].nome_player)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		ListofNodes[dlioo_id].StrAbil=new char[MAX_STR_ABIL_LEN];
		if ((ListofNodes[dlioo_id].StrAbil)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		char *dlioo_dsp=getenv("TINYMOODS_DLIOO_DISPLAY");
		if(dlioo_dsp==NULL)
			dlioo_dsp="800x600";
		
		char *dlioo_parte=getenv("TINYMOODS_DLIOO_PARTE");
		if(dlioo_parte==NULL)
			dlioo_parte="Vln.I";
		
		strcpy(ListofNodes[dlioo_id].Host,dlioo_hst);
		strcpy(ListofNodes[dlioo_id].Ip,"0.0.0.1");
		strcpy(ListofNodes[dlioo_id].Type,"DLIOO");
		strcpy(ListofNodes[dlioo_id].Display,dlioo_dsp);
		strcpy(ListofNodes[dlioo_id].Device,"mouse");
		strcpy(ListofNodes[dlioo_id].Parte,dlioo_parte);
		strcpy(ListofNodes[dlioo_id].nome_nodo,"Node1");
		strcpy(ListofNodes[dlioo_id].nome_player,"Someone");
		strcpy(ListofNodes[dlioo_id].StrAbil,"FILE:N;MODIFICA:N;CANCELLA:N;INTESTAZ:N;BATTUTA:N;PARTE:N;MOVIMENTO:N;LEGATURA:V;FORCELLA:N;OTTAVA:N;ONDA:V;CAMBIORIT:N;ACCENTI:N;ABBELLIM:N;TDINAMICO:N;TGENERICO:N;ANNOTAZDIR:N;CORONA:N;SUDDIVIS:N;DITEGGIATO:S;SORDINA:S;PRFIATO:S;OCCHIALI:S;VIOLINO:S;TIMPANO:N;OBOE:N;SAX:N;EDITEXEC:N;");
		ListofNodes[dlioo_id].Leader=(unsigned char) 1;
		ListofNodes[dlioo_id].Group_leader=(unsigned char) dlioo_id;
		ListofNodes[dlioo_id].Kids_id=(unsigned char) 0;
	} 
#else
	unsigned char i=1;
	FILE *cfgfile;
	char s[200];
	char *s1,*s2,*s3,*s4,*s5,*s6,*s7,*s8;
	int n0,n1,n2,n3;
	char menu[MAX_STR_ABIL_LEN];
	char * nmnetst;
	char tmp[100];
	if (NomeNetStart!=NULL) nmnetst=NomeNetStart;
	else nmnetst=NOME_NETSTART;
	
	
	strcpy(tmp,"./");
	strcat(tmp,nmnetst);
	
	cfgfile=fopen(tmp,"r");
	if (cfgfile==NULL) 
	{ 
		char *env=getenv("NETSTART_PATH");
		if(env!=NULL)
			strcpy(tmp,env);
		else
			strcpy(tmp,"./");
		strcat(tmp,nmnetst);
		cfgfile=fopen(tmp,"r");
		if (cfgfile==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_OPENFILEREAD","NetONCM","LoadTabella");
			// gestione errore file
		}
	}
	
	char c;
	fscanf(cfgfile,"%s\n",s);
	while (!(feof(cfgfile)))
	{
		s1=ListofNodes[i].Host=new char[HOST_NAME_LEN];
		if ((ListofNodes[i].Host)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		
		s2=ListofNodes[i].Ip=new char[HOST_IP_LEN];
		if ((ListofNodes[i].Ip)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		s3=ListofNodes[i].Type=new char[30];
		if ((ListofNodes[i].Type)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		s4=ListofNodes[i].Display=new char[HOST_DISPLAY_LEN];
		if ((ListofNodes[i].Display)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		s5=ListofNodes[i].Device=new char[HOST_DEVICE_LEN];		 
		if ((ListofNodes[i].Device)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		s6=ListofNodes[i].Parte=new char[GROUP_PARTE_LEN];		 
		if ((ListofNodes[i].Device)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		
		s7=ListofNodes[i].nome_nodo=new char[NODE_NAME_LEN];
		if ((ListofNodes[i].nome_nodo)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		s8=ListofNodes[i].nome_player=new char[NODE_PLAYER_LEN];
		if ((ListofNodes[i].nome_player)==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","LoadTabella()");
		
		
		if (fscanf(cfgfile,"%[^#]#%s %s %s %[^#]#%s %s %s",s1,s2,s3,s4,s5,s6,s7,s8)!=8)
			EM.Error(GENERAL_ERR_AREA,"EC_INFILE","NetONCM","LoadTabella");
		
		
		if (fscanf(cfgfile,"%d %d %d %d ",&n0,&n1,&n2,&n3)!=4)
			EM.Error(GENERAL_ERR_AREA,"EC_INFILE","NetONCM","LoadTabella");
		
		if (fscanf(cfgfile,"%s ",menu)!=1)
			EM.Error(GENERAL_ERR_AREA,"EC_INFILE","NetONCM","LoadTabella");
		
		
		ListofNodes[i].Leader=(unsigned char) n1;
		ListofNodes[i].Group_leader=(unsigned char) n2;
		ListofNodes[i].Kids_id=(unsigned char) n3;
		ListofNodes[i].Abil_Compress=TRUE;
		i++;
		
		
	}	
	fclose(cfgfile);
#endif TINYMOODS
#endif WITH_NET
	
}


void NetONCM::SetDxLimit(unsigned char id,NumCode nbat)
{
	
#ifdef WITH_NET   
	
	ListofNodes[id].Ne=nbat;
	
#else
#endif 
	
}


NumCode NetONCM::GetDxLimit(unsigned char id)
{
	
#ifdef WITH_NET   
	
	return(ListofNodes[id].Ne);
	
#else
	return 0;
#endif 
	
}


void NetONCM::SetSxLimit(unsigned char id,NumCode nbat)
{
	
#ifdef WITH_NET   
	
	ListofNodes[id].Ns=nbat;
	
#else
#endif 
	
}


NumCode NetONCM::GetSxLimit(unsigned char id)
{
	
#ifdef WITH_NET   
	
	return(ListofNodes[id].Ns);
#else
	return 0; 
#endif 
	
}

RetCode NetONCM::SetTid(unsigned char id,int tid)
{
	
#ifdef WITH_NET   
	
	ListofNodes[id].PTID=tid;
	
#else
	return 0;
#endif 
	
}


// inizializzazione tabella delle parti
void NetONCM::InitPindex()
{	
	
#ifdef WITH_NET   
	
	
	// La tabella sara' significativa solo da 1 in poi perche' cosi gestisco
	// meglio l'eventualita' di un Host non impiegato in una opera e perche'
	// mantengo l'idea che i DLIOO vengono numerati da 1	
    int i;
    for (i=0;i<MAXGROUPS;i++)
    {
		if ((Pindex[i].Parte)!=NULL) delete[] (Pindex[i].Parte);
		Pindex[i].Parte=NULL;
		Pindex[i].IdLeader=0; 
    }
	
#else
#endif 
	
}

// restituisce l'indice della tabella Pindex che contiene la parte Pname	
unsigned char NetONCM::FindIDP(char *Pname)
{
	
#ifdef WITH_NET   
	
	int k_index;
	int to_return;
	to_return=0;
	for (k_index=1;k_index <= NActiveParte;)
	{
		// se non verifica l'if significa che quell'host ha una parte
		// che non viene usata in quell'opera 
		if ((Pindex[k_index].Parte)!=NULL)
		{	
			if (!(strcmp(Pname,(Pindex[k_index].Parte))))
			{
				to_return = k_index;
				break;
			}
		}
		k_index++;
	}
	// significa che non e' stata trovata e quindi restituisco il valore 0
	// che non e' significativo
    return ((unsigned char)to_return);
	
#else
	return 0;
#endif 
	
}	

// restituisce il leader della parte Pname
unsigned char NetONCM::FindIDL(char *Pname)
{
	
#ifdef WITH_NET   
	
	int k_index;
	int to_return=0;
	for (k_index=1;k_index<NActiveLIOO;k_index++)
	{
		if (!(strcmp(Pname,ListofNodes[k_index].Parte)))
		{
			to_return = (unsigned char) k_index;
			break;
		}
	}
	
	// significa che non e' stata trovata e quindi restituisco il valore 0
	// che non e' significativo	
    return (unsigned char)to_return;
	
#else
	return 0;
#endif 
	
}

// Ritorna una righa della tabella ListofNodes  
Nodes NetONCM::ReadTab(unsigned char Elem)
{
	
#ifdef WITH_NET   
	
	Nodes Readed;
	Readed = ListofNodes[Elem];
	return Readed;
	
#else
	return Nodes();
#endif 
	
}

void NetONCM::UpdateAbil(unsigned char * node_to_update, char * NEW_ABIL)
{
	// Passo da Nome del nodo a TID a ID
	int id=FindID(Find_TID_from_NODENAME(node_to_update));
	if (ListofNodes[id].StrAbil==NULL) 
		ListofNodes[id].StrAbil = new char[MAX_STR_ABIL_LEN];
	
	strncpy(ListofNodes[id].StrAbil,NEW_ABIL,MAX_STR_ABIL_LEN);
	return;  
}


//Trova il TID partendo dal ID 
int NetONCM::FindPTID(unsigned char idin)
{
	
#ifdef WITH_NET   
	
	unsigned char k_index;
	Nodes Var1;
	int to_return;
	for (k_index=0;k_index<=NActiveLIOO;k_index++)
	{
		Var1 = ReadTab(k_index);
		if (k_index == idin)
		{
			to_return = Var1.PTID;
			break;
		}
		else
			to_return = 0;
	}
	return(to_return);
	
#else
	return 0;
#endif 
	
}




//Trova il TID partendo dal ID 
int NetONCM::Find_TID_from_NODENAME(unsigned char NODENAME[])
{
	
#ifdef WITH_NET   
	
	unsigned char k_index;
	Nodes Var3;
	int to_return;
	for (k_index=1;k_index<NActiveLIOO;k_index++)
	{
		Var3 = ReadTab(k_index);
		char NODE_VAR[20];
		strcpy(NODE_VAR,Var3.nome_nodo);
		if (strcmp(NODE_VAR,(char*)NODENAME)==0)
		{
			to_return = Var3.PTID;
			break;
		}
		else
			to_return = 0;
	}
	return(to_return);
	
#else
	return 0;
#endif 
	
}






//Dovrebbe trovare l'ID partendo dal TID  
unsigned char NetONCM::FindID(int tidin) 
{
	
#ifdef WITH_NET   
	
	unsigned char k_index;
	unsigned char to_return;
	Nodes Var2;
	for (k_index=0;k_index<NActiveLIOO;k_index++)
	{
		Var2 = ReadTab(k_index);
		if (Var2.PTID == tidin)
		{	
			to_return = k_index; 
			break;
		}	
		else
			to_return = 0;
	}
	return(to_return);
	
#else
	return 0;
#endif 
	
}	


//Trova il nome del host partendo dal ID 
char *NetONCM::FindHOST(unsigned char idin)                   
{
	
#ifdef WITH_NET   
	
	unsigned char k_index;
	Nodes Var1;
	char *to_return;
	
	for (k_index=0;k_index<=NActiveLIOO;k_index++)
	{
		Var1 = ReadTab(k_index);
		if (k_index==idin)
		{
			to_return = new char [strlen(Var1.Host)];
			if(!to_return)
				EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","NetONCM","FindHOST()");               
			strcpy(to_return, Var1.Host);
			break;
		}	
		else to_return = NULL;
	}
	
	return (to_return);	
	
#else
	return NULL;
#endif 
	
}       

// Ritorna l'IP del host partendo dal ID 
char *NetONCM::FindHOSTIP(unsigned char idin) 
{
	
#ifdef WITH_NET   
	
	unsigned char k_index;
	Nodes Var1;
	char *to_return;
	for (k_index=0;k_index<NActiveLIOO;k_index++)
	{
		Var1 = ReadTab(k_index);
		if (k_index == idin)
			return(Var1.Ip);
		else
			to_return = NULL;
	}
	return(to_return);
	
#else
	return NULL;
#endif 
	
}           


// Ritorna l'ID del host partendo dal nome 
unsigned char NetONCM::FindIDfromHOST(char *namehost)
{
	
#ifdef WITH_NET   
	
	unsigned char k_index;
	Nodes Var1;
	unsigned char to_return;
	for (k_index=0;k_index<=NActiveLIOO;k_index++)
	{
		Var1 = ReadTab(k_index);
		if (!strcmp(Var1.Host, namehost))
		{
			to_return = k_index;
			break;
		}
		else
			to_return = 255;
	}
	return(to_return);
	
	
#else
	return 0;
#endif 
	
}        


// controlla se e' un direttore
Bool NetONCM::CheckDirector(char id_LIOO)
{
	
#ifdef WITH_NET   
	if (ListofNodes[id_LIOO].Type==NULL) return FALSE;
	if (!(strcmp(ListofNodes[id_LIOO].Type,DIRETTORE))) return TRUE;
	else
		return FALSE;
#else
	return 0;
#endif 
	
}


// controlla se e' il leader della parte 
Bool NetONCM::CheckLeader(char id_LIOO)
{
	
#ifdef WITH_NET   
	
	if ((ListofNodes[id_LIOO].Leader==1)) return TRUE;
	else return FALSE;
	
#else
	return 0;
#endif 
	
}




RetCode NetONCM::AddHost(char *name)
{ 
	return 0;
}

RetCode NetONCM::DelHost(char *nome)
{
	return 0;
}
RetCode NetONCM::CheckHost(char *hostname)
{
	
#ifdef WITH_NET   
	
	RetCode infocheck;
	cout<<"LOOKING FOR "<<hostname<<" ... ";
	infocheck=pvm_mstat(hostname);
	cout<<"*********"<<infocheck;
	switch (infocheck)
	   {
	   case 0:cout<<"Host OK \n";
		   break;
	   }
	return (infocheck);	 
	cout<<"\n +++++++++++++";	
	
#else
	return 0;
#endif 
}


int NetONCM::EndNode(int tid)
{
	
#ifdef WITH_NET   
	
    switch (pvm_kill(tid))
	{
	case 0:cout<<"Node closed\n";
		return(0);
	case -2:cout<<"Tid value not valid\n";
		return(-2); 
	case -14:cout<<"Warning pvmd not responding \n";
		return(-14);
	}
	
	
#else
    return 0;
#endif 
}
int NetONCM::CheckNode(char *hostname)
{
	return 0;
}

int NetONCM::GetActiveNodeList()
{
	return 0;
}




