/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: netoncm.hpp,v 1.1 2005/07/25 16:31:42 bellinpo Exp $ */




#ifndef NETONCM_HPP
#define NETONCM_HPP


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gui/liooabil.hpp"

#ifdef WITH_NET   

#include "/home/pvm/pvm3/include/pvm3.h"

#endif

#include "def_net.hpp"
#include "lioo.hpp"


#define MAX_SIZE_NOME_MACCHINA 30
#define SIZE_IP 15
#define SIZE_TIPO 6




// Struttura che crea il collegamento tid - host - identifier - gruppo etc
struct Nodes
{
	
	char *Host;
    // Informazioni provenienti direttamente dal file di configurazione
    // Nome host
	char *Ip;
    // indirizzo IP
	char *Type;
    // tipo  (MASAE,MASE,DLIOO)
	char *Display; 
    // Dimensioni Display in pixel separate da x (es. 800x600)
	char *Device; 
    // Dispositivo di puntamento (Mouse,Trackball,ecc)
	
	char *Parte; 
	// Nome della parte musicale (nome tabellato)
    char* nome_nodo;
    // nome del nodo
	char* nome_player;
    // nome del 'proprietario' del leggio   
	
	
	
	// Altre informazioni da inserire all'avvio
	unsigned char Leader;
    // 0 o 1 se  un leader di gruppo
	unsigned char Group_leader; 
    // id del leader di gruppo
	unsigned char Kids_id;  
    // id puntatore al successivo appartenete al gruppo
	
	
    
	
	unsigned char IDP;
    // ID della Parte (indice dell' array Pindex)
	int PTID;
    // tid associato al nodo dalla pvm
	NumCode Ns;
    // Nbattuta Start finestratura	
	NumCode Ne;
    // Nbattuta End finestratura
	Bool E_Ack; 
    // flag per controllo ack nel multicast
	char * StrAbil;
	
	Bool Abil_Compress;
    // booleano che indica se la parte ha o no abilitata la compressione
	// (ci si riferisce alle pause multirest)
	
};

struct SPindex {
	char * Parte;
	unsigned char IdLeader;
};

class NetONCM
{
public:
    Nodes ListofNodes[MAXNODES+1];
    SPindex Pindex[MAXGROUPS+1];
    unsigned char NActiveLIOO;
    unsigned char NActiveParte;
    int * ArrayOfTid;         
	
	
    NetONCM();	
	// Costruttore
    ~NetONCM();    
	// Distruttore 
	
    void InsertPindex(unsigned char,int,char*,unsigned char);
    unsigned char FindIDPDirector();
    int AddHost(char *name);
    void BuildArrayOfTid();
	// costruisce l'array contenente tutti i tid dei processi lanciati,usato
	// poi per il multicast
    int DelHost(char *name);
    Bool CheckE_Ack();
	// controlla se tutti i leggii hanno ricevuto l'ack del messaggio
	// multicast
	// restituisce : TRUE se gli ack sono tutti arrivati
	//               FALSE se almeno un ack non e' arrivato
	
    int CheckHost(char *name);
    int StartNode(unsigned char);	
    int EndNode(int tid); 
    int CheckNode(char *hostname);
    int GetActiveNodeList();
	
    void InitE_Ack();
	// Inizializza a TRUE il campo E_Ack per ogni entrata della tabella
    
    void MakeImpostMenu(char * menu,unsigned char );
	void UpdateAbil(unsigned char * node_to_update, char * NEW_ABIL);
    void InitPindex();
    void RefreshTabella();
    void InitTabella();
    void LoadTabella();
    
	
    void ResetAllE_Ack();
	// Porta il campo E_Ack al valore FALSE,indicando che si deve ricevere l'ack
	// del messaggio inviato con il multicast   
	
    void SetDxLimit(unsigned char,NumCode);	 
    NumCode GetDxLimit(unsigned char);
    
    void SetE_Ack(unsigned char id);
	// Setta al valore TRUE il campo E_Ack indicando che per il leggio "id" e'
	// stato ricevuto l'ack del multicast 
    void SetSxLimit(unsigned char,NumCode);	 
    NumCode GetSxLimit(unsigned char);
    unsigned char FindIDP(char *);
    RetCode SetTid(unsigned char,int);
    Nodes ReadTab(unsigned char);
    unsigned char FindID(int);                    
	// trova ID dato PTID
    int Find_TID_from_NODENAME(unsigned char NODENAME[]);
	//  trova il TID dal nome del nodo
	unsigned char FindIDfromHOST(char *);
    int FindPTID(unsigned char);		      
	// trova PTID dato ID
    char *FindHOST(unsigned char);                
	// trova nome HOST dato ID      
    char *FindHOSTIP(unsigned char);              
	// trova IP dato ID
    unsigned char FindIDL(char *);
    Bool CheckLeader(char);
    Bool CheckDirector(char id_LIOO);	
	
	void SetAbil_Compress(unsigned char id, Bool abil);
	Bool GetAbil_Compress(unsigned char id);
};

#endif

