/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: oncm.cpp,v 1.1 2005/07/25 16:31:42 bellinpo Exp $ */


#ifndef ONCM_DF
#define ONCM_DF


#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <string.h>
#include "oncm.hpp"
#include "./../loa/y.tab.cpp"

ONCM::~ONCM(){};


ONCM::ONCM()
{
	Orchestra.Groups_intersection=no;
	InitNodes();
	InitHosts();
	InitGroups();
}

void ONCM::InitNodes()
{
	unsigned short i;
	for (i=0;i<MAXNODES;i++)
	{
		ListofNodes[i].Name =NULL;
		ListofNodes[i].Host =NULL;
		ListofNodes[i].Category=NULL;
		ListofNodes[i].Type =NULL;
		ListofNodes[i].Turn_page=NULL;
		ListofNodes[i].Player=NULL;
		ListofNodes[i].Uifile=NULL;
		
	}
}

void ONCM::InitHosts()
{
	unsigned short i;
	for (i=0;i<MAXHOSTS;i++)
	{
		ListofHosts[i].Ip = NULL;
		ListofHosts[i].Name = NULL;
		ListofHosts[i].Display =NULL;
		ListofHosts[i].Device =NULL;
		
	}
}
void ONCM::InitGroups()
{
	unsigned short i,h;
	for (i=0;i<MAXGROUPS;i++)
	{
		ListofGroups[i].Desc=NULL;
		ListofGroups[i].Parte_name=NULL;
		for (h=0;h<MAX_NODES_PER_GROUP;h++)
		{
			ListofGroups[i].Node_name[h]=NULL;
			
		}
	}
}

void ONCM::InitCfg()
{
	unsigned short i;
	for (i=0;i<MAXNODES;i++)
	{
		Cfgimage[i].Host=NULL;
		Cfgimage[i].Ip=NULL;
		Cfgimage[i].Parte_name=NULL;
		Cfgimage[i].Type=NULL;
		Cfgimage[i].Display=NULL;
		Cfgimage[i].Device=NULL;
		Cfgimage[i].Uifile=NULL;
		Cfgimage[i].Leader=0;
		Cfgimage[i].Group_leader=0;
		Cfgimage[i].Kids_id=0;
		
		
	}
}

void ONCM::Loadcfg(ONCM *p)
{	
	//int lineno;
	yyin = fopen("netboot.cfg", "r");
	if(yyin == NULL)
	{
		//APRINT		printf("Source file not found\n");
		exit(1);
	}
	
	
	
	yy_init = 1;
	lineno = 0;
	Initload(p);
	yyparse(); // lancia il parser
	
	fclose(yyin);
	
}

void ONCM::Create_cfg_image()
{
	int nID;
	InitCfg();
	// Associa i nodi agli hosts
	for (nID=0;!(ListofNodes[nID].Host==NULL);++nID)
	{
		
		Cfgimage[nID].Ip=ListofHosts[Findid(ListofNodes[nID].Host)].Ip;	
		
		Cfgimage[nID].Host=ListofNodes[nID].Host;
		
		Cfgimage[nID].Type=ListofNodes[nID].Type;	
		
		Cfgimage[nID].Display=ListofHosts[Findid(ListofNodes[nID].Host)].Display;
		Cfgimage[nID].Device=ListofHosts[Findid(ListofNodes[nID].Host)].Device;	
		Cfgimage[nID].Uifile=ListofNodes[nID].Uifile;
		//if (strcpy(ListofNodes[nID].Category,"Master")==0)
		//	Cfgimage[nID].Leader=1;
		// il leader deve essere anche il primo nodo del gruppo
		
		
	}
	
	
	// Per ogni gruppo crea la lista dei nodi usando i campi ID
	// Presenti nella tabella 
	int ng=-1;
	while (!(ListofGroups[++ng].Desc==NULL))
	{
		int ngn=0;
		int lastid,currentid;
		int idlead;	// ID leader di questo gruppo
		idlead=Findidfnn(ListofGroups[ng].Node_name[1]);
		Cfgimage[idlead].Leader=1;
		Cfgimage[idlead].Parte_name=ListofGroups[ng].Parte_name;
		//APRINT		if ((Cfgimage[idlead].Group_leader>0)&(!(Orchestra.Groups_intersection)))
		//APRINT			{
		//APRINT				printf("Nodo %s gia' appartenente ad un gruppo",Cfgimage[currentid].Host);
		//APRINT			}
		Cfgimage[idlead].Group_leader=idlead;
		// si presuppone che il primo sia il leader
		lastid=idlead;
		while (!(ListofGroups[ng].Node_name[++ngn]==NULL))
		{
			currentid=Findidfnn(ListofGroups[ng].Node_name[ngn]);
			Cfgimage[lastid].Kids_id=currentid;
			Cfgimage[currentid].Parte_name=ListofGroups[ng].Parte_name;
			//APRINT			if ((Cfgimage[currentid].Group_leader>0)&(!(Orchestra.Groups_intersection)))
			//APRINT			{
			//APRINT				printf("Nodo %s gia' appartenente ad un gruppo",Cfgimage[currentid].Host);
			//APRINT			}
			Cfgimage[currentid].Group_leader=idlead;
			lastid=currentid;
			
		}
		
	}	
	
}


void ONCM::Savecfg()
{
	
	int i=-1;
	FILE *cfgfile;
	cfgfile=fopen("NetStart.cfg","w+");
	
	while (!(Cfgimage[++i].Host==NULL))
	{
		
		fprintf(cfgfile,"%s#%s %s ",Cfgimage[i].Host,Cfgimage[i].Ip,Cfgimage[i].Type);
		fprintf(cfgfile,"%s %s %s#",Cfgimage[i].Display,Cfgimage[i].Device,Cfgimage[i].Uifile);
		fprintf(cfgfile,"%s ",Cfgimage[i].Parte_name);
		fprintf(cfgfile,"%d %d %d",Cfgimage[i].Leader,Cfgimage[i].Group_leader,Cfgimage[i].Kids_id);
		
	}	
	fclose(cfgfile);
}



int ONCM::Findid(char *Hostname)
{
	unsigned int i=0; // ID MASAE=0
	while ((!(strcmp(ListofHosts[i].Name,Hostname)==0))&(i<MAXHOSTS)) i++;
	//printf("i=%d Nome1=%s Nome2=%s \n",i,ListofHosts[1].Name,ListofHosts[2].Name);
	return(i);	// ID per mamass i-1
	
}

int ONCM::Findidfnn(char *Nodename)
{
	unsigned int i=0;	// ID MASAE=0
	while ((!(strcmp(ListofNodes[i].Name,Nodename)==0))&(i<MAXNODES)) i++;
	
	
	return(i);
	
}





// dichirazioni globali
int lineno;
ONCM *pONCM;

int yyerror(char *s)
{ 
	//APRINTF    printf("Configuration error in Netboot.cfg line %d\n ",lineno);
}

void main() {
	
	// carica netboot.cfg
	pONCM=new ONCM;
	pONCM->Loadcfg(pONCM);
	pONCM->Create_cfg_image();		
	pONCM->Savecfg();
	
}


#endif
