/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include "drw/musicedt.hpp"
#include "smr/smr.hpp"
#include <wx/font.h>
#include "pag/casellatxt.hpp"
#include "gui/dlg/casellatxtdlg.hpp"


CasellaTxt::CasellaTxt()
{
	Allineamento=A_Centre;
	font= wxFont(20, wxSWISS, wxNORMAL, wxNORMAL);
}

CasellaTxt::~CasellaTxt()
{}

void CasellaTxt::Allinea(Align b)
{
	int w,h,cont=0;
	float x,y;
	wxString testo=Testo,stampa;
	h=Scr->GetSystemTextHeight((char*)Testo.c_str());
	w=Scr->GetSystemTextWidth((char*)Testo.c_str());
	float magn = Scr->GetMagnification();
	switch(b)
	{
	case A_Left:
		{	
			x=GetX()/magn;
			SetAllineamento(A_Left);
			if(Testo.Find('\n')==-1)	
			{	
				y=GetY()/magn;
				Scr->TextTo(x,y-GAP);
				Scr->OutSystemText((char*)Testo.c_str());
			}
			
			else
			{	
				while(!testo.IsEmpty())
				{	
					stampa=testo.BeforeFirst('\n');
					w=Scr->GetSystemTextWidth((char*)stampa.c_str());
					y=GetY()/magn-h*cont;
					Scr->TextTo(x,y-GAP);
					Scr->OutSystemText((char*)stampa.c_str());
					testo=testo.AfterFirst('\n');	
					cont++;
				}
			}
			
		}
		break;
	case A_Right:
		{
			SetAllineamento(A_Right);
			if(!Testo.Find('\n'))	
			{	
				x=GetX()/magn+Width-w;
				y=GetY()/magn;
				Scr->TextTo(x,y-GAP);
				Scr->OutSystemText((char*)Testo.c_str());
			}
			
			else
			{	
				while(!testo.IsEmpty())
				{	
					stampa=testo.BeforeFirst('\n');
					w=Scr->GetSystemTextWidth((char*)stampa.c_str());
					x=GetX()/magn+Width-w;
					y=GetY()/magn-h*cont;
					Scr->TextTo(x,y-GAP);
					Scr->OutSystemText((char*)stampa.c_str());
					testo=testo.AfterFirst('\n');	
					cont++;
				}
			}
			
		}
		
		break;
	case A_Centre:
		{	
			SetAllineamento(A_Centre);
			if(!Testo.Find('\n'))	
			{	
				x=GetX()/magn+(Width-w)/2;
				y=GetY()/magn;
				Scr->TextTo(x,y-GAP);
				Scr->OutSystemText((char*)Testo.c_str());
			}
			
			else
			{	
				while(!testo.IsEmpty())
				{	
					stampa=testo.BeforeFirst('\n');
					w=Scr->GetSystemTextWidth((char*)stampa.c_str());
					x=GetX()/magn+(Width-w)/2;
					y=GetY()/magn-h*cont;
					Scr->TextTo(x,y-GAP);
					Scr->OutSystemText((char*)stampa.c_str());
					testo=testo.AfterFirst('\n');	
					cont++;
				}
			}
		}
		break;
		
	}
	
}


void CasellaTxt::Draw()
{
	float magn = Scr->GetMagnification();
	Scr->SetSystemFont(&font);
	
	SetPos(NULL,Point(X/magn,Y/magn-Heigh));
	Scr->Rect(class Rectangle(Point(X/magn+Width, Y/magn-Heigh),
		Point(X/magn, Y/magn)), TRUE);
	Allinea(Allineamento);	
}

void CasellaTxt::Edit(Battuta *pBat)
{
	extern wxMusicEditor *MusicEditor;
	extern LiooWindow *pMusicWin;
	wxString x,y;
	CasellaTxtDlg* dlg = new CasellaTxtDlg();
	pMusicWin->PPartit->SetCasellaPtr(this);
	dlg->Centre();
	dlg->SetX(wxString::Format("%f",X));
	dlg->SetY(wxString::Format("%f",Y));
	dlg->SetFont(font);
	dlg->SetTesto(Testo);
	dlg->SetAlign(Allineamento);
	if (dlg->ShowModal() == wxID_OK)
	{
		SetHeigh(dlg->GetHeight());
		SetWidth(dlg->GetWidth());
		SetX(dlg->GetX());
		SetY(dlg->GetY());
		SetTesto(dlg->GetTesto());
		SetAllineamento(dlg->GetAlign());
		SetFont(dlg->GetFont());
		//MusicEditor->smrf->GetPartitura()->Draw();
		pMusicWin->PPartit->Draw();
	}
	
}

Casella* CasellaTxt::Clone()
{
	CasellaTxt *cas = new CasellaTxt();
	cas->SetAllineamento(Allineamento);
	cas->SetFont(font);
	cas->SetTesto(Testo);
	return cas;
}








