/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include "xml/loa/xmlcommonclasses.hpp"

#include "gui/dlg/casellaimgdlg.hpp"
#include "pag/listapagine.hpp"
#include "drw/musicedt.hpp"
#include "smr/smr.hpp"
#include "gui/dlg/casellatxtdlg.hpp"	
#include <wx/textfile.h>


ListaPagine::ListaPagine()
{}

ListaPagine::~ListaPagine()
{
	ClearDeep(); 
}

Pagina* ListaPagine::GetPage(int n,bool create)
{
	Node *ptr=first;
	while(ptr)
	{
		if(((Pagina*)ptr->ObjectPtr)->GetNPagina()==n)
			return (Pagina*)ptr->ObjectPtr;
		ptr=ptr->Next;
	}
	if(create) 
	{
		Pagina *NewPage=new Pagina();
		this->InsertEnd(NewPage);
		NewPage->SetNPagina(n);
		return NewPage;
	}
	return NULL;
}

void ListaPagine::DrawPage(int n)
{
	Pagina* ptr;
	ptr=GetPage(n,false);
	if(ptr!=NULL)
		ptr->Draw();
}

Bool ListaPagine::Hit(const Point& p, int Npagina, DrawObject *& objsel)
{
	Pagina* ptr;
	ptr=GetPage(Npagina,false);
	if(ptr!=NULL)
		if(ptr->Hit(p,Npagina,objsel))
			return TRUE;
		
		return FALSE;
}

Bool ListaPagine::DeleteSym(DrawObject* p, int Npage)
{
	Pagina* ptr;
	ptr=GetPage(Npage,false);
	if(ptr!=NULL)
	{
		if(ptr->DeleteSym(p))
			return TRUE;
	}
	return FALSE;
}

CmdResult ListaPagine::CmdInsertText(const Point& p, int Npagina)
{
	extern wxMusicEditor *MusicEditor;
	CasellaTxtDlg* dlg = new CasellaTxtDlg();
	CasellaTxt* casella = new CasellaTxt();
	dlg->Centre();
	float magn = Scr->GetMagnification();
	float x1 = (p.x)*magn;
	float y1 = (p.y)*magn;
	dlg->SetX(wxString::Format("%f",x1));
	dlg->SetY(wxString::Format("%f",y1));
	if (dlg->ShowModal() == wxID_OK)
	{
		casella->SetHeigh(dlg->GetHeight());
		casella->SetWidth(dlg->GetWidth());
		casella->SetX(dlg->GetX());
		casella->SetY(dlg->GetY());
		casella->SetTesto(dlg->GetTesto());
		casella->SetAllineamento(dlg->GetAlign());
		casella->SetFont(dlg->GetFont());
		MusicEditor->mainScore->AddCasella(casella,Npagina);
		MusicEditor->RedrawViews(0,TRUE);
		return(CMD_OK);;
	}
	
	return(CMD_FAIL);;
}



CmdResult ListaPagine::CmdInsertImage(class Rectangle r, int Npagina)
{
	if((r.Height()>5)&&(r.Width()>5))
	{
		extern wxMusicEditor *MusicEditor;
		CasellaImgDlg* dlg = new CasellaImgDlg();
		CasellaImg* casella = new CasellaImg();
		dlg->Centre();
		float magn = Scr->GetMagnification();
		float x1 = (r.PUpLft.x)*magn;
		float y1 = (r.PDwnRgt.y)*magn;
		dlg->SetX(wxString::Format("%f",x1));
		dlg->SetY(wxString::Format("%f",y1));
		dlg->SetHeight(wxString::Format("%d",r.Height()));
		dlg->SetWidth(wxString::Format("%d",r.Width()));
		if (dlg->ShowModal() == wxID_OK)
		{	
			casella->SetHeigh(dlg->GetHeight());
			casella->SetWidth(dlg->GetWidth());
			casella->SetX(dlg->GetX());
			casella->SetY(dlg->GetY());
			MusicEditor->smrf->AddImage(dlg->GetPath());	
			wxString d = wxFileNameFromPath(dlg->GetPath());
			casella->SetFileImg(d);
			MusicEditor->mainScore->AddCasella(casella,Npagina);
			MusicEditor->RedrawViews(0,TRUE);
			return(CMD_OK);	
		}
	}
	else
		Message("Please drag mouse for defining image size!");
	return(CMD_FAIL);	
}

void ListaPagine::Free()
{
	ClearDeep();
}


bool ListaPagine::Create(wxString strumName)
{
	extern wxMusicEditor *MusicEditor;
	wxString index;
	if(strumName=="Main Score")
		index = "main_index.txt";
	else
		index = strumName+"_index.txt";
	
	wxString str,path, str2, str3, w, h;
	Pagina *pag=NULL;
	CasellaImg *casella = NULL;
	class Image *img=NULL;
	wxArrayString *tmp=NULL;
	wxTextFile file;
	bool removeFile=false;

	path=MusicEditor->GetWdfObjectPath()+PATH_SEP+"Image"+PATH_SEP;
	index=path+index;
	if(!file.Open(index))
		return false;
	int num_pag=1;
	double width, heigth;
	wxString npag=wxString::Format("%d",num_pag);
	int N = file.GetLineCount();
	for (int i=0; i<N; i++) 
	{
		str=wxFileNameFromPath(file.GetLine(i));
		str3=str;
		if(str3.Truncate(1)=="#")
		{
			str2=str.AfterFirst(' ');
			w=str2.BeforeFirst(' ');
			w=w.AfterFirst('=');
			h=str2.AfterFirst(' ');
			h=h.AfterFirst('=');
			w.ToDouble(&width);
			h.ToDouble(&heigth);
			pag=GetPage(num_pag,true);
			casella=new CasellaImg(); 
			img=new class Image;
			img->SetDimension(width, heigth);
			casella->SetImage(img);
			pag->InsertEnd(casella);	
			tmp=img->GetImageVect();
			if(strumName=="Main Score")
				img->SetName(path+"main"+npag+".png");
			else
				img->SetName(path+strumName+npag+".png");
			npag=wxString::Format("%d",++num_pag);
		}
		else
		{
			tmp->Add(str);
		}	
	}
	if(removeFile)
		wxRemoveFile(path);
	return true;
}
