/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: listapar.cpp,v 1.1 2005/07/25 16:31:44 bellinpo Exp $ */


#include "par/listapar.hpp"
#include "bra/pargraff.hpp"
#include "bra/parquad.hpp"
#include "bat/battuta.hpp" 

ListaParentesi::ListaParentesi()
{
	
	Init();
	
}

ListaParentesi::~ListaParentesi()
{
	Free();
}

void ListaParentesi::Init()
{
	
	// Inizializza la lista.
	first=NULL;
	last=NULL;
	numobject=0;
	CodeCounter = 1;
	
}  

void ListaParentesi::Free()
{
	ClearDeep();
}


Bool ListaParentesi::Add(CmdID cmd, Spartito *pSS, Spartito *pSE, ArgType& nc)
{
	Parentesi *pPar;
	
	switch(cmd)
	{
    case CMD_PARTITURA_ADD_QUADRA:
		pPar = new ParQuadra;
		if (pPar==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaParentesi","Add()");
		break;
    case CMD_PARTITURA_ADD_GRAFFA:
		pPar = new ParGraffa;
		if (pPar==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaParentesi","Add()");
		break;
	}
	pPar->SetPSpaStart(pSS);
	pPar->SetPSpaEnd(pSE); 
	
	InsertEnd(pPar);
	nc=CodeCounter++;
	pPar->SetNumericCode(nc); 
	return TRUE;
}

void ListaParentesi::SetCodeCounter(NumCode nc)
{
	CodeCounter = nc;
}

NumCode ListaParentesi::GetCodeCounter()
{
	return CodeCounter;
}

void ListaParentesi::Redraw(VUnit width,bool printing, int nsys)
{
	Node *pn=first;
	Parentesi *pPar;
	Spartito *psp1, *psp2;
	VUnit h, x;
	int liv;
	
	while(pn!=NULL)
	{
		pPar = ((Parentesi*)pn->ObjectPtr);
		
		psp1 = pPar->GetPSpaStart();
		psp2 = pPar->GetPSpaEnd();
		if(psp1!=NULL && psp2!=NULL && psp1->IsVisible() && psp2->IsVisible() && psp1->GetIsUsed(nsys))
		{
			liv = pPar->GetLivello();
			h = psp1->GetAbsPos(nsys).y - psp2->GetAbsPos(nsys).y + psp2->GetBaseVU2Dwn() + psp1->GetBaseVU2Up() + 6*liv;
			
			if (h<0) 
				h = -h;
			pPar->Init(h);
			x = psp2->GetAbsPos(nsys).x - liv*(pPar->GetWidth() + 2);
			
			pPar->SetPos(NULL,Point(x , psp2->GetAbsPos(nsys).y - psp2->GetBaseVU2Dwn() - 3*liv));//DB
			pPar->Draw();			
		}
		pn=GetNext(pn);
	}
}

void ListaParentesi::Draw()
{
	Node *pn=first;
	Spartito *psp1, *psp2;
	Parentesi *pPar;
	
	while(pn!=NULL)
	{
		pPar = ((Parentesi*)pn->ObjectPtr);
		
		psp1 = pPar->GetPSpaStart();
		psp2 = pPar->GetPSpaEnd();
		if(psp1!=NULL && psp2!=NULL	&& psp1->IsVisible() && psp2->IsVisible())
			pPar->Draw();
		
		pn=GetNext(pn);
	}
}

VUnit ListaParentesi::GetWidth()
{
	VUnit w = 0;
	Node *pn=first;
	
	
	while(pn!=NULL)
	{
		if (((Parentesi*)pn->ObjectPtr)->GetLivello() > w)
			w = ((Parentesi*)pn->ObjectPtr)->GetLivello();
		
		pn=GetNext(pn);
	}
	
	w *= (WIDTH_PARENTESI + 2);
	return w;
}

Parentesi *ListaParentesi::GetFirstPar()
{
	if( first!=NULL)
	{
		return (Parentesi *)(first->ObjectPtr);
	}
	else return NULL;
}

Parentesi *ListaParentesi::GetNextPar(Parentesi *r)
{
	return (Parentesi *)GetNext(r);
}  


Bool ListaParentesi::DeleteSym(DrawObject* sym)
{
	Node *pn=first;
	
	ClassID ID;
	
	ID=sym->GetID();
	if((ID==CL_PARGRAFFA)   ||
		(ID==CL_PARQUADRA))  
	{     	
		while(pn!=NULL)
		{
			if(pn->ObjectPtr==sym)
			{
				DelPtr(sym);   
				return TRUE;
			}	
			pn=GetNext(pn);
		}
	} 
	return FALSE;
} 


Bool ListaParentesi::Hit(const Point& p, SelObj sobj, DrawObject *& objsel,
						 SymPath& symPath)
{
	Node *pn=first;
	
	objsel=NULL;
	if (sobj==ANY_SEL)
	{
		while(pn!=NULL)
		{
			if(pn->ObjectPtr->Hit(p,sobj,objsel,symPath))
			{
				symPath.Array[LISTA_PATH_LEVEL]= 3;
				return TRUE;
			}
			pn=GetNext(pn);
		}
	}   
	
	return FALSE;
	
} 

Bool ListaParentesi::SymbolicHit(SymPath& symPath, SelObj sobj,
								 DrawObject *& objsel)
{
	
	Node *pn=first;
	
	objsel=NULL;
	
	while(pn!=NULL)
	{
		if(((Parentesi*)pn->ObjectPtr)->SymbolicHit(symPath,sobj,objsel))
			return TRUE;
		pn=GetNext(pn);
	}
	
	return FALSE;
}



void ListaParentesi::CancParentesiSpa(Spartito *psp)
{
	Node *pn=first;
	Parentesi *par;
	
	while(pn!=NULL)
	{
		par = ((Parentesi*)pn->ObjectPtr);
		if ((par->GetPSpaStart()==psp) || (par->GetPSpaEnd()==psp))
		{
			DelPtrNode(pn);
			pn = first;
		}
		else
			pn=GetNext(pn);
	}
}
