/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: listaspa.cpp,v 1.2 2005/07/28 09:52:30 bellinpo Exp $ */


#include "par/listaspa.hpp"
#include "spa/listabat.hpp"
#include "fig/spazio.hpp"
#include "fig/figura.hpp"
#include "drw/scorecan.hpp"
#include "musiceditor.hpp"
#include "par/listapar.hpp"
#include "drw/musicedt.hpp"
#include "bat/giustif.hpp"

#include <wx/progdlg.h>

ListaSpartiti::ListaSpartiti()
{
	Init();
}


ListaSpartiti::~ListaSpartiti()
{
	Free();
}


void ListaSpartiti::Init()
{
	Spartito *pSp = new Spartito;
	if(!pSp)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaSpartiti","Init()");
	
	// Inizializza la lista.
	first=NULL;
	last=NULL;
	numobject=0;
	CodeCounter = 1;
	currentScore=0;
	PtrSpa4Midi=NULL;
	Add(pSp,false);
	
	for(int i=0;i<MAX_VIEW;i++)
	{
		currentPage[i]=1;
	}
	
	TmpPrintViewParams=NULL;
	PrintViewParams=NULL;
	ComputerViewParams=NULL;
	
	cut_copy=false;	
	
	
	
	for(i=0; i<10; i++)
		MidiDinamicIndValue[i]=10*(i+1);
	
}


void ListaSpartiti::Free()
{
	int nParts=GetNumObj();
	ClearDeep();
	CodeCounter = 1; 
	
	
	if(PrintViewParams!=NULL)
	{
		delete PrintViewParams;
		PrintViewParams=NULL;
	}
	if(ComputerViewParams!=NULL)
	{
		delete ComputerViewParams;
		ComputerViewParams=NULL;
	}
}


void ListaSpartiti::SetMusicRect(const class Rectangle& r)
{
	MusicRect[DrawObject::GetCurrentView()].PUpLft = r.PUpLft;
	MusicRect[DrawObject::GetCurrentView()].PDwnRgt = r.PDwnRgt;
}


class Rectangle ListaSpartiti::GetMusicRect()
{ return MusicRect[DrawObject::GetCurrentView()]; }

void ListaSpartiti::SetPrintRect(const class Rectangle& r)
{
	PrintRect.PUpLft = r.PUpLft;
	PrintRect.PDwnRgt = r.PDwnRgt;
}


class Rectangle ListaSpartiti::GetPrintRect()
{ return PrintRect; }



Spartito *ListaSpartiti::GetSpart(NumCode nc)
{
	Spartito *ptr=(Spartito*)GetFirst();
	
	while(ptr!=NULL && ptr->GetNumericCode()!=nc)
		ptr = (Spartito*)GetNext(ptr);
    
	if (ptr!=NULL && ptr->GetNumericCode()==nc)
		return ptr;
	else
		return NULL;
}


void ListaSpartiti::PlaceSpartiti(int LeftMargin, int TopMargin, int RightMargin, int BottomMargin, int StaffDistance, VUnit& width,bool printing)
{
	Node *pn=first;
	Spartito *psp=NULL, *pspNext=NULL;
	int WidthInit, maxWidth;
	VUnit pwdt,spc,x,y;
	extern wxMusicEditor *MusicEditor;
	int xx, yy, wsh,wsw;  
	int NStaff=1;
	
	if(printing)
	{
		int SystemsNo = GetPrintViewParams()->GetNumSystems(GetCurrentPage());
		pwdt=PrintRect.Width()-LeftMargin-RightMargin;
		spc=StaffDistance;
		x=PrintRect.PUpLft.x + LeftMargin;
		y=PrintRect.PUpLft.y-spc-VUnit(((Spartito*)first->ObjectPtr)->GetBaseVU2Up()) - TopMargin;
				
		// imposta le scroll bar
		wxScrolledWindow* canvas = (wxScrolledWindow*)MusicEditor->mainFrame->FindWindow(wxMEScoreCanvas_ID);
		int vw,vh;
		canvas->GetVirtualSize(&vw,&vh);
		if(vw/10 != GetPrintRect().Width()/10+2 || vh/10 != GetPrintRect().Height()/10+2)
			canvas->SetScrollbars(10, 10,(GetPrintRect().Width()*Scr->GetMagnification())/10+2,
			(GetPrintRect().Height()/*Scr->GetMagnification()*/)/10+2);
		
		FontID ofont=Scr->SetFont(nTextFont);
		
		maxWidth = 0;
		while(pn!=NULL)
		{ // inserisce lo spazio a sinistra del sistema di parti in base 
			// alle dimensioni del nome della parte
			psp = (Spartito*)pn->ObjectPtr;
			if(psp->IsVisible())
			{
				WidthInit = Scr->GetTextWidth((unsigned char *)(psp->GetStrumEsec()));
				if (maxWidth < WidthInit) maxWidth = WidthInit;
			}
			pn = GetNext(pn);
		}
		
		maxWidth += width+3;
		
		Scr->SetFont(ofont); 
		
		for (int NrSys = 0; NrSys<SystemsNo; NrSys++)
		{
			pn = first;
			while(pn!=NULL)
			{
				psp = (Spartito*)pn->ObjectPtr;
				
				if(psp->IsVisible())
				{
					if (NrSys == 0)
					{
						psp->SetMaxWidth(pwdt-maxWidth);
						psp->SetWidth(pwdt-maxWidth); // Precauzione...
						psp->SetSpaceUpDwn(spc);
						psp->SetSpaceInit_FirstSystem(maxWidth);
					}
					else
					{
						psp->SetMaxWidth(pwdt-width,NrSys);
						psp->SetWidth(pwdt-width,NrSys);
						psp->SetSpaceUpDwn(spc,NrSys);
						psp->SetSpaceInit(width);
					}
					psp->SetPos(NULL,Point(x,y),NrSys);
					psp->SetNRighi(SystemsNo);
					
					
					pn=GetNext(pn);
					while(pn!=NULL)
					{
						pspNext = (Spartito*)pn->ObjectPtr;
						if(pspNext->IsVisible())
						{
							y -= (2*spc+VUnit(pspNext->GetBaseVU2Up())+psp->GetBaseVU2Dwn()); //PB&GB
							break;
						}
						pn=GetNext(pn);
					}
				}
				else
					pn = GetNext(pn);
				
			}
			
			if(psp!=NULL && psp->IsVisible()) 
				y -= (2*spc+psp->GetBaseVU2Dwn()); 
			y-=spc*2;
		}
		
		width = maxWidth;
		
	}
	else
	{
		wxScrolledWindow* canvas = NULL;
		if(MusicEditor->mainFrame)
			canvas=(wxScrolledWindow*)MusicEditor->mainFrame->FindWindow(wxMEScoreCanvas_ID);
		
		VUnit height=TopMargin+BottomMargin;
		pn = first;
		while(pn!=NULL)
		{
			psp = (Spartito*)pn->ObjectPtr;
			if(psp->IsVisible())
			{
				psp->SetSpaceUpDwn(StaffDistance);
				height+=psp->GetHeight();
			}
			pn = GetNext(pn);	   
		}
		if(Scr->IsAutoMagnification())
		{
			wsh=Scr->GetMaxPixY();
			Scr->SetMagnification(wsh/float(height));
		}
		height*=Scr->GetMagnification();
		
		if(canvas)
		{
			canvas->GetSize(&wsw,&wsh);
			if(wsh>height)
				height=wsh;
		
			canvas->GetVirtualSize(&xx, &yy);
			if (yy!=height)
				canvas->SetScrollbars(10, 10, 50, height/10,0,0,TRUE);
		}
		else
		{
			wsw=Scr->GetMaxPixX();
			wsh=Scr->GetMaxPixY();
			/*if(wsh>height)
				height=wsh;*/
			height=wsh;
		}
				
		pwdt=GetMusicRect().Width()-LeftMargin-RightMargin;
		MusicRect[DrawObject::GetCurrentView()].PUpLft.y = height;
		
		spc=StaffDistance;
		
		x=GetMusicRect().PUpLft.x + LeftMargin;
		y=GetMusicRect().PUpLft.y - spc - VUnit(((Spartito*)first->ObjectPtr)->GetBaseVU2Up()) - TopMargin; 
		
		
		FontID ofont=Scr->SetFont(nTextFont);
		pn=first;
		
		maxWidth = 0;
		while(pn!=NULL)
		{ // inserisce lo spazio a sinistra del sistema di parti in base 
			// alle dimensioni del nome della parte
			psp = (Spartito*)pn->ObjectPtr;
			if(psp->IsVisible()){	
				WidthInit = Scr->GetTextWidth((unsigned char *)(psp->GetStrumEsec()));
				if (maxWidth < WidthInit) maxWidth = WidthInit;
			}
			pn = GetNext(pn);
		}
		
		maxWidth += width+3;
		
		Scr->SetFont(ofont); 
		
		
		pn = first;
		while(pn!=NULL)
		{
			psp = (Spartito*)pn->ObjectPtr;
			if(psp->IsVisible()){	//IB
				psp->SetMaxWidth(pwdt-maxWidth);
				psp->SetWidth(pwdt-maxWidth); // Precauzione...
				psp->SetSpaceUpDwn(spc);
				psp->SetSpaceInit(maxWidth);
				psp->SetSpaceInit_FirstSystem(maxWidth);
				psp->SetPos(NULL,Point(x,y));
				psp->SetNRighi(1);
				
				pn=GetNext(pn);
				while(pn!=NULL)
				{
					pspNext = (Spartito*)pn->ObjectPtr;
					if(pspNext->IsVisible())
					{
						y -= (2*spc+VUnit(pspNext->GetBaseVU2Up())+psp->GetBaseVU2Dwn()); //PB&GB
						break;
					}
					pn=GetNext(pn);
				}
			}
			else
				pn = GetNext(pn);
			width = maxWidth;
		}
	}
}

void ListaSpartiti::PlaceSimboli(int ncol)
{
	Spartito *psp;
	Node *pn = first;
	int i;
	
	while (pn!=NULL)
	{
		psp = (Spartito*)(pn->ObjectPtr);
		psp->ResetCurrent();
		pn = pn->Next;
	}   
	
	for(i=0;i<ncol;i++)
	{
		pn = first;
		while(pn!=NULL)
		{
			psp = (Spartito*)pn->ObjectPtr;
			psp->SetupCurrentSimboli();
			pn = GetNext(pn);
		}
	}
}   

// giustifica la colonna di battute se e' 
// la prima volta che viene visualizzata
void ListaSpartiti::JustifIfFirst()
{
	// dichiarazioni
	Node *pn;
	Spartito *psp;
	VUnit PentWidth = 0;
	int j=0, l=0, maigiustificata=0;	
	TipoGiust tp;
	double kGiust;
	int nParti=GetNumObj();
	Battuta * vbat[50];
	
	Bool continua = TRUE, FineSpartiti = FALSE;  
	int ncol=0;  
	
	pn = first;
	j=-1;
	while(pn!=NULL)
	{
		psp = (Spartito*)pn->ObjectPtr;
 		     j++;
			 vbat[j]=NULL;
			 if (NULL!=psp->GetCurrentBatLst())
			 {
				 vbat[j]=psp->GetCurrentBatLst();
				 vbat[j]->SetPos(NULL, Point(0,0));
				 vbat[j]->SetupSimboli();		
			 }
			 
			 pn = GetNext(pn);
	}
	
	
	for (l=0;l<nParti;l++)	{
		if (vbat[l]==NULL) 
			// non ci deve essere un puntatore nullo
		{ 
			return;
		}
		
		//verifica di giustificazione per la prima volta
		maigiustificata=0;
		
		if (NO==vbat[l]->GetEverJustified(DIR, currentViewType))
		{	maigiustificata=1;
		break;
		}
	}
	
	// qui effettua la giustificazione se almeno una battuta della
	// colonna non e' mai stata giustificata
	
	//if (1==maigiustificata)
	{ 
		
		//giustifica la colonna e aggiorna il valore di everjustified
		tp=vbat[0]->Gettipogiust();
		kGiust=vbat[0]->Getkgiustificazione();	 
		if (G_NONE==tp)
		{
			tp=G_LOG;
			kGiust=2;
		}
		
		GiustificaBattute(vbat, tp, kGiust);
		
		for (l=0;l<nParti;l++)	{
			
			vbat[l]->SetEverJustified(DIR, currentViewType, YES);
			
		}		
	}
	
	
} 




int ListaSpartiti::AlignBattute(int system)
{
	Node *pn;
	Spartito *psp;
	VUnit x = 0, xPrec = 0, PentWidth = 0;
	VUnit MaxIntWdt, MaxWdt, IntWdt, Wdt;
	Bool continua = TRUE, FineSpartiti = FALSE;  
	int ncol=0;  
	bool preclb=false;
	bool currlb=false;
	
	pn = first;
	if(pn==NULL)
		return 0;
	
	PentWidth = ((Spartito*)pn->ObjectPtr)->GetMaxWidth();
	
	
	// Posizionamento ad inizio pagina. del primo sistema
	
	while(pn!=NULL)
	{
		if (system==0)
			((Spartito*)pn->ObjectPtr)->ResetCurrent();
		((Spartito*)pn->ObjectPtr)->SaveCurrentPosition();
		pn=GetNext(pn);
	}
	
	
	// Sistemazione della pagina attuale.
	while(continua)
	{
		// Esamina una colonna di battute.
		MaxIntWdt = MaxWdt = 0;
		
		JustifIfFirst();
		//
		
		currlb=false;
		pn = first;
		while(pn!=NULL && !FineSpartiti)
		{
			psp = (Spartito*)pn->ObjectPtr;
			
			if(psp->GetCurrentBatLst()!=NULL)
				currlb=currlb || psp->GetCurrentBatLst()->GetMainScoreLineBreak(currentViewType);
			
			if(psp->CheckCurrent(IntWdt,Wdt))
			{
				if(Wdt>MaxWdt)
				{
					// Aggiorna i valori massimi.
					MaxWdt = Wdt;
					MaxIntWdt = IntWdt;
				}
				pn = GetNext(pn);
			}
			else
				FineSpartiti = TRUE;
		}
		if(FineSpartiti)
			continua = FALSE;
		else
		{
			// Controlla se la colonna attuale entra nella pagina.
			xPrec = x;
			x += MaxWdt;
			if(x<=PentWidth && !preclb) // La colonna entra nella pagina.
			{
				pn = first;
				while(pn!=NULL)
				{
					// Questa funzione produce anche l'avanzamento nelle liste
					// di battute.
					psp = (Spartito*)pn->ObjectPtr;
					psp->SetupCurrent(xPrec, MaxIntWdt, MaxWdt, system);
					psp->SetWidth(x,system);
					pn = GetNext(pn);
				}
				ncol++;
			}
			else // La colonna non entra nella pagina.
			{
				continua = FALSE;
			}
			preclb=currlb;
		}
	}   
	return ncol;   
}



//////////////////////////////////
// versione che gestisce il line-breaking
int ListaSpartiti::AlignBattuteLineBreak(int system)
{
	Node *pn;
	int nParti=GetNumObj(), totale=0;
	Bool Consistent=TRUE;  // mi dice se le battute sono consistenti
	Spartito *psp;
	VUnit x = 0, xPrec = 0, PentWidth = 0;
	VUnit MaxIntWdt, MaxWdt, IntWdt, Wdt;
	Bool continua = TRUE, FineSpartiti = FALSE;  
	int ncol=0, i=0, j=0, l=0;  
	Battuta * vbat[50];
	bool preclb=false;
	bool currlb=false;
	
	struct { VUnit MaxWdt,MaxIntWdt,xPrec, target; } Cols[30];
	
	struct { VUnit MaxWdt,MaxIntWdt,xPrec, target; } Colslineb[30];
	
	
	// non basta piu' ncol, ci vuole un vettore dinamico
	// con il Maxwidth e maxintwdt per ogni colonna, serve per
	// sapere quanto e'larga una colonna e di quanto la devo stretchare
	// per  arrivare a marginare a dx
	//////////////////////
	VUnit avanzo=0, parziale=0;
	//////////////////////  
	
	
	if (system==0)
	{
		pn=first;
		if(pn==NULL)
			return 0;
		// larghezza del margine dx
		PentWidth = ((Spartito*)pn->ObjectPtr)->GetMaxWidth(system);
		// Posizionamento ad inizio pagina: col cursore
		while(pn!=NULL)
		{
			((Spartito*)pn->ObjectPtr)->ResetCurrent();
			((Spartito*)pn->ObjectPtr)->SaveCurrentPosition();
			((Spartito*)pn->ObjectPtr)->ResetfirstBatNextPage();
			pn=GetNext(pn);
		}
	}
	else
	{
		pn=first;
		if(pn==NULL)
			return 0;
		// larghezza del margine dx
		PentWidth = ((Spartito*)pn->ObjectPtr)->GetMaxWidth(system);
		// Posizionamento ad inizio pagina: col cursore
		while(pn!=NULL)
		{
			((Spartito*)pn->ObjectPtr)->SaveCurrentPosition();
			((Spartito*)pn->ObjectPtr)->ResetfirstBatNextPage();
			pn=GetNext(pn);
		}
	}
	
	
	// Sistemazione della pagina attuale.
	// ciclo di check
	i=0;
	int old;
	Battuta *pBat;
	while(continua)
	{
		// Esamina una colonna di battute.
		MaxIntWdt = MaxWdt = 0;
		
		JustifIfFirst();
		currlb=false;
		
		pn=first;
		while(pn!=NULL && !FineSpartiti)
		{
			psp = (Spartito*)pn->ObjectPtr;
			
			if(psp->GetCurrentBatLst()!=NULL)
				currlb=currlb || psp->GetCurrentBatLst()->GetMainScoreLineBreak(currentViewType);
			
			pBat=psp->GetCurrentBat();
			
			old=AutoLineBreak;  
			AutoLineBreak=FALSE;
			Bool check=psp->CheckCurrent(IntWdt,Wdt);
			AutoLineBreak=old;
			
			if(check)
			{
				if(Wdt>MaxWdt)
				{
					// Aggiorna i valori massimi.
					MaxWdt = Wdt;
					MaxIntWdt = IntWdt;
				}	
				pn = GetNext(pn);			
				psp->MoveCurrentBatForw();
			}
			else
				FineSpartiti = TRUE;
		}
		if(FineSpartiti)
		{ 
			xPrec=x;	
			continua = FALSE;
		}
		else
		{
			// Controlla se la colonna attuale entra nella pagina.
			xPrec = x;
			x += MaxWdt;
			
			if(x<=PentWidth && !preclb) // La colonna entra nella pagina.
			{
				// rds
				i++;
				Cols[i].MaxWdt=MaxWdt; 
				Cols[i].MaxIntWdt=MaxIntWdt; 
				Cols[i].xPrec=xPrec; // senza intestazione
				ncol++;
			}
			else // La colonna non entra nella pagina.
			{
				continua = FALSE;
			}
			preclb=currlb;
		}    // fine else controlla se entra nella pagina
	}   // fine while continua
	avanzo=PentWidth-xPrec; 
	parziale=xPrec;
	if (ncol==0) 
		return ncol;
	
	
	// ciclo di setup: ho un vettore con le larghezze delle colonne e l'avanzo
	// ripercorro le varie colonne di battute facendo lo stretch e il setup
	
	
	pn=first;
	while(pn!=NULL)
	{
		((Spartito*)pn->ObjectPtr)->ResetCurrent(pn);
		pn=GetNext(pn);
	}
	
	// calcolo dei target, assegnazione alla battute e redistribuzione dellventuale avanzo
	totale=0;
	for (i=1;i<=ncol;i++)
	{
		if (parziale!=0)
			Cols[i].target=(int)( Cols[i].MaxWdt+Cols[i].MaxWdt*(float)avanzo/(float)parziale);
		else Cols[i].target=Cols[i].MaxWdt+avanzo;
		totale+=Cols[i].target;
	}
	totale=(parziale+avanzo)-totale; 
	while (totale>0)
	{
		for (i=1;i<=ncol;i++)
			if (totale>0) 
			{
				totale--;
				Cols[i].target++;
			}
	}
	
	
	// costruisco vbat: puntatori alle battute della colonna
	for(i=1;i<=ncol;i++)
	{
		pn=first;
		j=-1;
		while(pn!=NULL)
		{
			// Questa funzione produce anche l'avanzamento nelle liste
			// di battute.
			psp = (Spartito*)pn->ObjectPtr;
			// crea il vettore vbat con i puntatori alle battute della colonna corrente da stretchare
			j++; 	
			vbat[j]=psp->GetCurrentBat();
			
			
			if (vbat[j]==NULL) vbat[j]=psp->GetfirstBatNextPage();
			
			
			psp->MoveCurrentBatForw();
			pn = GetNext(pn);
		}
		
		//verifica di consistenza
		Consistent=TRUE;
		
		for (l=0;l<nParti;l++)	{
			if (vbat[l]==NULL) 
				// non ci deve essere un puntatore nullo
			{ Message ("Null pointer: line break disabled");
			AutoLineBreak=FALSE;   
			return -1;
			}
			
			Consistent=vbat[l]->CheckBatt();
			
			if (Consistent==FALSE) break;  
		}
		
		
		// qui effettua lo stretch della battuta modificando 
		// i parametri di line breaking 
		int controllino;
		if ((controllino=StretchBattute(vbat, Cols[i].target,0 ))==-1) /// SOSTITUIRE 0 con 1 per far funzionare la spaziatura di note senza durata
		{
			AutoLineBreak=FALSE;
			return -1;
		}
		
		
		// se sono uscito perche' e' fallita la convergenza del meccanismo
		// di line break
		//gli do un'altra possibilita', questa volta sincronizzo la struttura 
		if (controllino==-3)
		{
			if ((controllino=StretchBattute(vbat, Cols[i].target, 1 ))==-1)
			{
				AutoLineBreak=FALSE;
				return -1;
			}
			
			if (controllino==-3)
			{
				AutoLineBreak=FALSE;
				return -1;
				
			}
			
			
		}
		
		
	}
	
	
	// Ciclo di setup
	// Posizionamento ad inizio pagina.
	
	pn=first;
	while(pn!=NULL)
	{
		((Spartito*)pn->ObjectPtr)->ResetCurrent(pn);
		pn=GetNext(pn);
	}
	
	
	
	xPrec=0;
	for(i=1;i<=ncol;i++)
	{
		pn=first;
		MaxWdt = 0;
		while(pn!=NULL)
		{
			// Questa funzione produce anche l'avanzamento nelle liste
			// di battute.
			psp = (Spartito*)pn->ObjectPtr;
			Bool check = psp->CheckCurrent(IntWdt,Wdt);
			if(check)
			{
				if(Wdt>MaxWdt)
				{
					// Aggiorna i valori massimi.
					MaxWdt = Wdt;
					// Sulla fiducia: si presuppone che la battuta piu` lunga sia
					// quella con intestazione piu` lunga.
					MaxIntWdt = IntWdt;
				}
				
			}
			pn = GetNext(pn);
		}
		
		pn=first;
		while(pn!=NULL)
		{
			// Questa funzione produce anche l'avanzamento nelle liste
			// di battute.
			psp = (Spartito*)pn->ObjectPtr;
			// fai lo stretch 
			// crea il vettore vbat con i puntatori alle battute della colonna corrente da stretchare
			psp->SetupCurrent(xPrec, MaxIntWdt, MaxWdt, system);
			
			psp->SetWidth(xPrec+MaxWdt, system);
			pn = GetNext(pn);
		}
		
		xPrec+=MaxWdt;
		
		Colslineb[i].MaxWdt=MaxWdt; 
		Colslineb[i].MaxIntWdt=MaxIntWdt; 
		Colslineb[i].xPrec=xPrec; 
		
		
		
		
	}
	
	return ncol;   
}

//  metodo che formatta una colonna di battute della dimensione voluta  
// facendo anche la sincronizzazione
int ListaSpartiti::StretchBattute(Battuta* vbat[], UL totale, int sincr)  
{  
	// questo metodo fa un ciclo di giustificazione iterata sulla colonna
	// di battute e si ferma quando lo spazio della colonna  spacecol
	// che gli passo come parametro
	// giustifica colonna di battute
	
	// esegui il metodo dicotomico per trovare il parametro che fa si' che la battuta sia larga 
	// la sua lunghezza piu' surplus.
	// i nuovi parametri di spaziatura delle figure si chiamano DLine per direttoriale e OLine
	// per l' orchestrale
	Bool trovato=FALSE;
	double kmin, kmax, inc=1, kold, k0;
	//  double kprova;
	UL totmax,totprova,totmin, totold;
	TipoGiust tp;
	// leggo il k di giustificazione della prima battuta della colonna, prendi il puntatore a battuta
	//kmin=vbat[0]->kgiustificazione;
	
	k0=kmin=vbat[0]->Getkgiustificazione();
	tp=vbat[0]->Gettipogiust();
	if (tp==G_NONE) {
		Message("Please Justify score");
		return -1;
	}
	
	switch (tp)
	{
	case G_LOG: 
		inc=0.2;
		break;
	case G_LIN: inc=100;
		break;
		
	}
	
	
	// controllo: se la battuta e' vuota assegna totale allo spazio prima della 
	// prima figura (attributo di battuta sul quale gioco)
	// questo e' per far funzionare il linebreak per battute vuote
	Layer *pLayer; // GB
	int nonvuoto=0;
	int nParti=GetNumObj(); 
	Figura *figtmp;
	for(int l=0;l<nParti; l++)
	{
		// GB
		for(int i=0; i<NUM_MAX_LAYER; i++)
		{	
			pLayer=vbat[l]->GetPLayer(i);
			if(pLayer!=NULL)
			{
				figtmp=pLayer->GetFirstFig();
				if (figtmp!=NULL) 
					nonvuoto=1;
			}
		}
	}      
	if (nonvuoto==0)
	{     
		for(l=0;l<nParti; l++)
		{
			for(int i=0; i<NUM_MAX_LAYER; i++)
			{	
				vbat[l]->SetDirSp2FirstFig(i,totale);
			}
			
		}
		return 1;
	}
	
	
	// crea gli oggetti della giustificazione
	int maxFig=0,i,maxLay=0;
	Giustificazione gd;
	// calcola maxFig
	for(i=0;i<nParti;i++)
	{
		vbat[i]->SetupNumeroFig();
		maxFig+=vbat[i]->GetNumeroFig()*2;
		maxLay+=vbat[i]->GetNUsedLayers();
	}
	gd.Costruisci(maxFig,maxLay);
	for(i=0;i<nParti;i++)
	{
		gd.Inserimento(vbat[i]);
	}
	tp=vbat[0]->Gettipogiust();
	
	gd.Sincronizza();
	
	gd.CleanTable();
	
	
	
	switch(tp)
	{
    case G_LOG:
		gd.GiustificaLog(kmin);
		break;
    case G_LIN:
		gd.GiustificaLin(kmin);
		break;  
	}
	gd.SetUpSpDLine();
	int Massimo=0;
	int larghezza=0;
	for(i=0;i<nParti;i++)
	{
		vbat[i]->SetVU(0);
		larghezza=vbat[i]->GetWidth();
		if (larghezza>Massimo)
			Massimo=larghezza;
	}
	
	totmax=Massimo;
	totmin=Massimo;
	
	
	kmax=kmin+inc;
	int uscita=0;
	while (totmax<totale) {
		
		
		if (sincr==1)
		{
			gd.Costruisci(maxFig,maxLay);
			for(i=0;i<nParti;i++)
			{
				gd.Inserimento(vbat[i]);
			}
			gd.Sincronizza();
		}  
		else gd.CleanTable();
		
		
		
		
		
		switch(tp)
		{
		case G_LOG:
			gd.GiustificaLog(kmax);
			break;
		case G_LIN:
			gd.GiustificaLin(kmax);
			break;  
		}
		gd.SetUpSpDLine();
		int Massimo=0;
		int larghezza=0;
		for(i=0;i<nParti;i++)
		{
			vbat[i]->SetVU(0);
			larghezza=vbat[i]->GetWidth();
			if (larghezza>Massimo)
				Massimo=larghezza;
		}
		
		totmax=Massimo;
		
		kmax=kmax+inc;
		if(++uscita>1000)
		{
			//Message("Warning: A) Too Iterations for getting Line Breaking");
			AutoLineBreak=FALSE;
			return -1;
		}
		
	}
	kmax=kmax-inc;
	
	if (totmax==totale) return 1;
	
	
	// ricerca kopt
	trovato=FALSE;
	uscita=0;
	while (trovato==FALSE) {
		
		if (sincr==1)
		{
			gd.Costruisci(maxFig,maxLay);
			for(i=0;i<nParti;i++)
			{
				gd.Inserimento(vbat[i]);
			}
			gd.Sincronizza();
		}  
		else gd.CleanTable();
		
		
		
		switch(tp)
		{
		case G_LOG:
			gd.GiustificaLog((kmax+kmin)/2);
			
			
			break;
		case G_LIN:
			gd.GiustificaLin((kmax+kmin)/2);
			break;  
		}
		
		
		
		gd.SetUpSpDLine();
		
		
		
		int Massimo=0;
		int larghezza=0;
		for(i=0;i<nParti;i++)
		{
			vbat[i]->SetVU(0);
			larghezza=vbat[i]->GetWidth();
			if (larghezza>Massimo)
				Massimo=larghezza;
		}
		
		totprova=Massimo;
		
		
		uscita++;
		
		if (totprova<totale) 
		{ kmin=(kmax+kmin)/2;
		totmin=totprova;
		}
		else if (totprova>totale) {
			kmax=(kmax+kmin)/2;
			totmax=totprova;
		}
		
		if (totmax<totmin) {
			kold=kmin;
			kmin=kmax;
			kmax=kold;
			totold=totmin;
			totmin=totmax;
			totmax=totold;
			
			// per debug da levare
			//Message("Inversione di monotonicita");
			
		}
		
		if (abs(totprova-totale)<1) { 
			trovato=TRUE;
			
		}	
		if ( fabs(kmin-kmax)<0.001 )
		{ 
			return -3;
			
			break;
		}
		if(++uscita>1000)
		{
			//Message("Warning: B) Too Iterations for getting Line Breaking");
			AutoLineBreak=FALSE;
			return -1;
		}
		
		
	}	
	
	
	return 1;
	
}  


void ListaSpartiti::PlaceIntervalli(Bool onlyLegature)
{
	Node *pn=first;
	
	while(pn!=NULL)
	{
		((Spartito*)pn->ObjectPtr)->PlaceIntervalli(onlyLegature);
		pn = pn->Next;
	}
}

/* FUNZIONI GESTIONE LIRICA */

void ListaSpartiti::PlaceSillabe()
{
	// Dispone le sillabe presenti nella lista
	// associate allo spartito in questione.
	
	Node *pn=first;
	
	while(pn!=NULL)
	{
		((Spartito*)pn->ObjectPtr)->PlaceSillabe();
		pn = pn->Next;
	};
}

void ListaSpartiti:: LoadLyr(char *Dir)
{
	// Questo metodo viene richiamato da PARTITURA
	// per caricare la lirica associata ad uno
	// spartito da caricare.
	
	Node *pn=first;
	// In pn viene posto il puntatore
	// al primo spartito della lista.
	int ID;
	// Numero di identificazione dello
	// spartito all'interno della lista. 
	char *filenameres;
	// Variabile di supporto contenente
	// il nome del file da cercare per
	// verificare la presenza di una
	// lirica associata allo spaertito
	// corrente.
   	filenameres = new char[256];
   	
	Bool LiricaPresente=FALSE;
	
	while(pn!=NULL)
	{
		// Inizializzazione della stringa contenente
		// il file da cercare.
		for(int i=0; i<256; i++)
  	        	filenameres[i]=0;
		
		ID=((Spartito*)pn->ObjectPtr)->GetNumericCode();
		// Viene posto in ID il codice numerico che
		// permette ,all'interno della partitura
		// corrente di individuarlo in modo univoco.
		char buffer[256];
		// Variabile temporanea di deposito.
		sprintf(buffer,"%d",ID);
		// Pone in buffer l'identificativo dello
		// spartito trasformato in stringa.
		
        filenameres=strcat(filenameres,Dir);
		filenameres=strcat(filenameres,buffer);
		filenameres = strcat(filenameres,".lyric.xml");
		// Dopo queste operazioni in filenameres  contenuto
		// l'eventuale file xml di lirica associato allo
		// spartito corrente.
		
		((Spartito*)pn->ObjectPtr)->LoadLyr(filenameres);
		// Richiama il metodo che permette di
		// caricare la lirica associata allo
		// spartito corrente,se presente.
		pn = pn->Next;
	};
	
};




void ListaSpartiti::GoBatt(NumCode nb)
{
	Node *pn=first;
	
	while(pn!=NULL)
	{
		((Spartito*)pn->ObjectPtr)->GoBatt(nb);
		pn=GetNext(pn);
	}
}

void ListaSpartiti::GoBattInExecution(NumCode nb)
{
	Node *pn=first;
	
	while(pn!=NULL)
	{
		((Spartito*)pn->ObjectPtr)->GoBattInExecution(nb);
		pn=GetNext(pn);
	}
}


Bool ListaSpartiti::GoForward()
{
	Node *pn=first;
	Bool result = FALSE;
    
	while(pn!=NULL)
	{
		if(((Spartito*)pn->ObjectPtr)->GoForward())
			result = TRUE; 
		pn=GetNext(pn);
	}
	return result;
}


Bool ListaSpartiti::GoBackward()
{
	Node *pn=first;
	Bool result = FALSE;
    
	while(pn!=NULL)
	{
		if(((Spartito*)pn->ObjectPtr)->GoBackward())
			result = TRUE; 
		pn=GetNext(pn);
	}
	return result;
}


Bool ListaSpartiti::GoNextPage()
{	
	Node *pn=first;
	Bool result = FALSE;
	
    //  Scorre la lista e avanza per ogni spartito    
	while(pn!=NULL)
	{
		if(((Spartito*)pn->ObjectPtr)->GoNextPage())
			result = TRUE; 
		pn=GetNext(pn);
	}
	if(result)
		currentPage[DrawObject::GetCurrentView()]++;	
	return result;
}

Bool ListaSpartiti::GoNextPageInExecution()
{	
	Node *pn=first;
	Bool result = FALSE;
	
    //  Scorre la lista e avanza per ogni spartito    
	while(pn!=NULL)
	{
		if(((Spartito*)pn->ObjectPtr)->GoNextPageInExecution())
			result = TRUE; 
		pn=GetNext(pn);
	}
	return result;
}

Bool ListaSpartiti::GoPrevPage()
{
	Node *pn=first;
	Bool result = FALSE;
	
    //  Scorre la lista e indietreggia per ogni spartito    
	while(pn!=NULL)
	{
		if(((Spartito*)pn->ObjectPtr)->GoPrevPage())
			result = TRUE; 
		pn=GetNext(pn);
	}
	
	return result;
}


void ListaSpartiti::DrawPage(bool printing)
{
	Node *pn=first;
	int idx_system=1;
	// Disegna le parti.
	while(pn!=NULL)
	{
		if(((Spartito*)pn->ObjectPtr)->IsVisible())
			((Spartito*)pn->ObjectPtr)->Draw();	
		pn=GetNext(pn);
	}
	
	if (printing)
	{		
		// disegna le linee per la pagina ed i margini (MS)
		int nPage=GetCurrentPage();
		int left = GetPrintViewParams()->GetLeftMargin(nPage);
		int bottom = GetPrintViewParams()->GetBottomMargin(nPage);
		int right = GetPrintViewParams()->GetRightMargin(nPage);
		int top = GetPrintViewParams()->GetTopMargin(nPage);
		int pr_width = GetPrintRect().Width();
		int pr_height = GetPrintRect().Height();

		Colore oldcol = Scr->GetColor();
		Scr->SetColor((ColorIcon)SELECTED_COLOR);
		Scr->SetDrwMode(XORMode);
		// segnala pagina in celeste - la pagina = PrintRect
		Scr->Rect(class Rectangle(Point(0,0), Point(pr_width,pr_height)));
		Scr->SetDrwMode(DefMode);
		// segnala rettangolo di stampa (rosso) = pagina - margini
		Scr->Rect(class Rectangle(Point(left,bottom), Point(pr_width-right+1,pr_height-top)));

		Scr->SetColor(oldcol);
		//Scr->SetColor((ColorIcon)NORMAL_COLOR);
		Scr->SetDrwMode(DefMode);

		// Disegna la linea di inizio rigo.
		if (first != NULL)
		{ 
			for(int idx=0;idx<GetPrintViewParams()->GetNumSystems(nPage);idx++) 
			{	  
				Spartito *spStart, *spEnd;
				spStart=GetFirstVisible();
				spEnd=GetLastVisible();
				if(spStart!=NULL && spEnd!=NULL && spStart->GetIsUsed(idx))
					Scr->Line(spStart->GetAbsPos(idx) + Point(0,(spStart->GetBaseVU2Up())), 
					spEnd->GetAbsPos(idx)+Point(0,-(spEnd->GetBaseVU2Dwn())),SOLID_LINE);
			}
		}
	}
	else
		
		
		if (first != NULL)
		{ 
			Spartito *spStart, *spEnd;
			spStart=GetFirstVisible();
			spEnd=GetLastVisible();

			//if(spStart && spEnd && spStart != spEnd) {

			//	Node *pNext, *pCur = first;
			//	Spartito *curObj, *nextObj;

			//	Point pStart, pEnd;

			//	while(pCur != NULL){
			//		curObj = (Spartito*)pCur->ObjectPtr;
			//		if(curObj->IsVisible()) {
			//			pNext = GetNext(pCur);
			//			while(pNext != NULL) {
			//				nextObj = (Spartito*)pNext->ObjectPtr;
			//				if(nextObj->IsVisible()) {
			//					pStart = curObj->GetAbsPos();
			//					pEnd = nextObj->GetAbsPos();
			//					Scr->Line( 
			//						pStart + Point(0,  curObj->GetBaseVU2Dwn()), 
			//						pEnd + Point(0, -nextObj->GetBaseVU2Up()),
			//						SOLID_LINE
			//						);

			//					pCur = GetPrev(pNext);
			//					break;
			//				}
			//				pNext = GetNext(pNext);
			//			}
			//		}
			//		pCur = GetNext(pCur);
			//	}
			//}
			if(spStart!=NULL && spEnd!=NULL && spStart != spEnd)
				Scr->Line((spStart->GetAbsPos()) + Point(0,(spStart->GetBaseVU2Up())), 
				spEnd->GetAbsPos()+Point(0,-(spEnd->GetBaseVU2Dwn())),SOLID_LINE);
		}
		
}

void ListaSpartiti::DrawPageInExecution()
{
	Node *pn=first;
	int idx_system=1;
	// Disegna le parti.
	while(pn!=NULL)
	{
		if(((Spartito*)pn->ObjectPtr)->IsVisible())
			((Spartito*)pn->ObjectPtr)->DrawInExecution();	
		pn=GetNext(pn);
	}
	
	if (first != NULL)
	{ 
		//Colore oldcol = Scr->GetColor();
		//Scr->SetColor((ColorIcon)black); 
		Spartito *spStart, *spEnd;
		spStart=GetFirstVisible();
		spEnd=GetLastVisible();
		if(spStart!=NULL && spEnd!=NULL)
			Scr->Line((spStart->GetAbsPos()) + Point(0,(spStart->GetBaseVU2Up())), 
			spEnd->GetAbsPos()+Point(0,-(spEnd->GetBaseVU2Dwn())),SOLID_LINE);
		//Scr->SetColor(oldcol);
	}
	
}

Spartito *ListaSpartiti::GetFirstVisible()
{
	Node *pn=first;
	while(pn!=NULL)
	{
		if(((Spartito*)pn->ObjectPtr)->IsVisible())
			return ((Spartito*)pn->ObjectPtr);
		pn=GetNext(pn);
	}
	return NULL;
}

Spartito *ListaSpartiti::GetLastVisible()
{
	Node *pn=last;
	while(pn!=NULL)
	{
		if(((Spartito*)pn->ObjectPtr)->IsVisible())
			return ((Spartito*)pn->ObjectPtr);
		pn=GetPrev(pn);
	}
	return NULL;
}
void ListaSpartiti::RedrawPage(int LeftMargin, int TopMargin, int RightMargin, int BottomMargin, int StaffDistance, VUnit& width,bool printing)
{
	
	extern wxMusicEditor *MusicEditor;	
	int SystemsNo = GetPrintViewParams()->GetNumSystems(GetCurrentPage());

	if(GetNumObj()==0) //no scores!
		return;

	PlaceSpartiti(LeftMargin, TopMargin, RightMargin, BottomMargin, StaffDistance, width, printing);
	int ncol;
	int ncols=0; //mi serve per sapere per contare il numero complessivo di colone per pagina  (MS)
	if(AutoLineBreak==1)  
	{  // line breaking automatico
		if (printing)
		{
			for (int i=0;i<SystemsNo;i++)
			{
				ncol= AlignBattuteLineBreak(i);
				if (ncol==-1)  
					ncol= AlignBattute(i);
				ncols = ncols + ncol;
			}
		}
		else
		{
			ncol= AlignBattuteLineBreak();
			if (ncol==-1)  ncol= AlignBattute();
			
		}
		PlaceIntervalli(TRUE); // solo legature
		if (printing)
			PlaceSimboli(ncols);
		else
			PlaceSimboli(ncol);
		PlaceIntervalli(FALSE); // altri simboli intervallo
		PlaceSillabe();
	}    
	else   // line breaking disabilitato
	{
		if (printing)
			for (int i=0;i<SystemsNo;i++)
			{
				ncol= AlignBattute(i);
				ncols = ncols + ncol;
			}
			else
			{
				ncol= AlignBattute();	
			}
			PlaceIntervalli(TRUE); // solo legature
			if (printing)
				PlaceSimboli(ncols);
			else
				PlaceSimboli(ncol);
			PlaceIntervalli(FALSE); // altri simboli intervallo
			PlaceSillabe();
	}
	
	DrawPage(printing); 
}


void ListaSpartiti::DrawTagli(const Point& p)
{
	Node *pn=first;
	
	while(pn!=NULL)
	{
		((Spartito*)pn->ObjectPtr)->DrawTagli(p);
		pn=GetNext(pn);
	}
}


Bool ListaSpartiti::Hit(const Point& p)
{
	Node *pn=first;
	
	
	while(pn!=NULL)
	{
		if(pn->ObjectPtr->Hit(p))
			return TRUE;
		pn=GetNext(pn);
	}
	
	return FALSE;
}


Bool ListaSpartiti::Hit(const Point& p, SelObj sobj, DrawObject *& objsel,
						SymPath& symPath)
{
	Node *pn=first;
	
	objsel=NULL;
	
	while(pn!=NULL)
	{
		if((pn->ObjectPtr->Hit(p,sobj,objsel,symPath))&&(((Spartito*)pn->ObjectPtr)->IsVisible()))
			return TRUE;
		pn=GetNext(pn);
	}
	
	return FALSE;
}


Bool ListaSpartiti::Hit(const class Rectangle& r,SelObj selObj,short layer1,
						SymPath& sp1,SymPath& sp2)
{  
	Node *pn=first;
	
	while(pn!=NULL)
	{
		if((((Spartito*)pn->ObjectPtr)->Hit(r,selObj,layer1,sp1,sp2))&&(((Spartito*)pn->ObjectPtr)->IsVisible()))
			return TRUE;
		pn=GetNext(pn);
	}
	
	return FALSE;
}


Bool ListaSpartiti::SymbolicHit(SymPath& symPath, SelObj sobj,
								DrawObject *& objsel)
{
	
	Node *pn=first;
	
	objsel=NULL;
	
	while(pn!=NULL)
	{
		if(((Spartito*)pn->ObjectPtr)->SymbolicHit(symPath,sobj,objsel))
			return TRUE;
		pn=GetNext(pn);
	}
	
	return FALSE;
}


void ListaSpartiti::Add(Spartito *pSp,bool addInWDFTree)
{
	//  tipoSpartito ts;
	
	
	// Ricava il tipo del nuovo spartito.
	if(first==NULL)
		pSp->SetTipo(SPART_UNICO);
	else
	{
		pSp->SetTipo(SPART_INFERIORE);
		if(first==last)
		{
			// Imposta superiore l'unico spartito attualmente presente.
			((Spartito*)GetFirst())->SetTipo(SPART_SUPERIORE);
		}
		else  
		{
			// Imposta come generico lo spartito attualmente inferiore.
			((Spartito*)GetLast())->SetTipo(SPART_GENERICO);
		}
	}
	
	InsertEnd(pSp);
	if(pSp->GetNumericCode()==0)
	{
		pSp->SetNumericCode(CodeCounter++);
	}
	else
	{
		NumCode nc=pSp->GetNumericCode();
		if(CodeCounter<=nc)
			CodeCounter=nc+1;
	}
	
	if(addInWDFTree)
		MusicEditorPartAdded(GetNumObj(),"new part");
}


void ListaSpartiti::AddAfter(Spartito *pSp, Spartito *pSpRef,bool modifyTree)
{
	Spartito *pSp1;
	
	InsertAfter(pSp, pSpRef);
	
    // Imposta il tipo del nuovo spartito.
	if(first==last)
		pSp->SetTipo(SPART_UNICO);
	else
	{
		pSp1 = (Spartito*)GetNext(pSp);
		if (pSp1==NULL)
		{
			pSp->SetTipo(SPART_INFERIORE);
			pSp1 = (Spartito*)GetPrev(pSp);
			if (pSp1!=NULL) 
			{
				if (pSp1 == (first->ObjectPtr))
					pSp1->SetTipo(SPART_SUPERIORE);
				else
					pSp1->SetTipo(SPART_GENERICO);
			}
		}
		else 
		{
			pSp1 = (Spartito*)GetPrev(pSp);
			if (pSp1==NULL)
			{
				pSp->SetTipo(SPART_SUPERIORE);
				pSp1 = (Spartito*)GetNext(pSp);
				if (pSp1!=NULL) 
				{
					if (pSp1 == (last->ObjectPtr))
						pSp1->SetTipo(SPART_INFERIORE);
					else
						pSp1->SetTipo(SPART_GENERICO);
				}
			}
			else
				pSp->SetTipo(SPART_GENERICO);
		}
	}
	
	pSp->SetNumericCode(CodeCounter++);
	if(modifyTree)
		MusicEditorPartAdded(GetPos(pSp)+1,"new part");
}


void ListaSpartiti::Del(Spartito *pSp,bool modifyTree)
{
	int pNum=GetPos(pSp)+1;
	DelPtr(pSp);
	
	if (GetFirst()==GetLast())
	{
		((Spartito*)GetFirst())->SetTipo(SPART_UNICO);
	}
	else
	{
		((Spartito*)GetFirst())->SetTipo(SPART_SUPERIORE);
		((Spartito*)GetLast())->SetTipo(SPART_INFERIORE);
	}
	
	if(modifyTree) 
		MusicEditorPartDeleted(pNum);
}



Bool ListaSpartiti::DeleteSym(DrawObject* sym)
{
	Node *pn=first;
	
	while(pn!=NULL)
	{
		if(pn->ObjectPtr->DeleteSym(sym))
			return TRUE;
		pn=GetNext(pn);
	}
	return FALSE;
}

void ListaSpartiti::GiustificaBattute(Battuta** vbat,TipoGiust tp,double kGiust, bool clearSpUt)
{
	// giustifica colonna di battute
	
	int nParti=GetNumObj(),maxFig=0,i,maxLay=0;
	if (currentScore!=0) nParti=1;
	Bool Consistent=TRUE;
	Giustificazione gd;
	
	// calcola maxFig
	for(i=0;i<nParti;i++)
	{
		vbat[i]->SetupNumeroFig();
		maxFig+=vbat[i]->GetNumeroFig()*2; //NUM_MAX_LAYER;
		maxLay+=vbat[i]->GetNUsedLayers();
		// setto i parametri di giustificazione di battuta
		if ((tipoLeggio==LEGGIO_MASAE) || (tipoLeggio==LEGGIO_MASE))
		{
			vbat[i]->Settipogiust(tp);
			vbat[i]->Setkgiustificazione(kGiust);	 
		}
		if ((tipoLeggio==LEGGIO_DLIOO) || (tipoLeggio==LEGGIO_PDLOO))
		{
			vbat[i]->Settipogiustparte(tp);
			vbat[i]->Setkgiustificazioneparte(kGiust);	 
		}		
	}
	
	//verifica di consistenza
	Consistent=TRUE;
	for (i=0;i<nParti;i++)	{
		Consistent=vbat[i]->CheckBatt();
		if (Consistent==FALSE) break;  
	}
	
	gd.Costruisci(maxFig,maxLay);
	
	for(i=0;i<nParti;i++)
	{
		gd.Inserimento(vbat[i]);
	}
	
	switch(tp)
	{
    case G_LOG:
		gd.Sincronizza();	
		gd.GiustificaLog(kGiust);
		break;
    case G_LIN:
		gd.Sincronizza();
		gd.GiustificaLin(kGiust);
		break;  
	}
	
	if ((tipoLeggio==LEGGIO_MASAE) || (tipoLeggio==LEGGIO_MASE)) gd.SetUpSpG(clearSpUt);
	if ((tipoLeggio==LEGGIO_DLIOO) || (tipoLeggio==LEGGIO_PDLOO)) gd.SetUpSpOr(clearSpUt);
}

void ListaSpartiti::GiustificaDaA(int daBat, int aBat,TipoGiust tp,double kGiust)
{
	int nParti=GetNumObj(),i,curBat=0;
	Battuta **vbat=NULL;
	Spartito* pSpa=NULL;
	
	// per just all breakpoint per verificare questo num=num tot battute?
	if(aBat==0)
		aBat=((Spartito*)GetFirst())->GetNumeroBattute();
    
	if(currentScore!=0)
		nParti=1;
    
	vbat=new Battuta*[nParti];
	
	if(currentScore==0)
	{
		pSpa=(Spartito*)GetFirst();
		for(i=0;i<nParti;i++)
		{
			vbat[i]=pSpa->GetBattNum(daBat-1);
			pSpa=(Spartito*)GetNext(pSpa);
		}
	}
	else
	{
		pSpa=(Spartito*)AtPos(currentScore-1);
		vbat[0]=pSpa->GetBattNum(daBat-1);
	}
	
	wxProgressDialog justifying("Justifying...","Measure 0",aBat-daBat+1);
	curBat=daBat;
	while(curBat<=aBat)
	{
		GiustificaBattute(vbat,tp,kGiust, true);
		curBat++;
		
		if(currentScore==0)
		{
			pSpa=(Spartito*)GetFirst();
			for(i=0;i<nParti;i++)
			{
				vbat[i]=pSpa->GetNextBat(vbat[i]);
				pSpa=(Spartito*)GetNext(pSpa);
			}
		}
		else
			vbat[0]=pSpa->GetNextBat(vbat[0]);
		
		justifying.Update(curBat-daBat,wxString::Format("Measure %d",curBat));
	}  
}


void ListaSpartiti::Transpose(int daBat, int aBat,int partnumber,int clef,int translation,int interval,int up, int numofstaff, Bool sharps, Bool adjust)
{
	int nParti,i,curBat=0;
	Battuta **vbat=NULL;
	Spartito* pSpa=NULL;
	
	
	//controllare se aBat e' dentro il numero di battute!!
	
	if(aBat==0)
		aBat=((Spartito*)GetFirst())->GetNumeroBattute();
    
	if(currentScore!=0)
	{
		nParti=1;
		pSpa=(Spartito*)AtPos(currentScore-1);
		
		vbat=new Battuta*[nParti];
		
		vbat[0]=pSpa->GetBattNum(daBat-1);
	}
	else
	{  
		if(partnumber==0)
		{
			nParti=GetNumObj();
			pSpa=(Spartito*)GetFirst();
			
			vbat=new Battuta*[nParti];
			
			for(i=0;i<nParti;i++)
			{
				vbat[i]=pSpa->GetBattNum(daBat-1);
				pSpa=(Spartito*)GetNext(pSpa);
			}
		}
		else
		{
			nParti=1;
			pSpa=(Spartito*)AtPos(partnumber-1);
			
			vbat=new Battuta*[nParti];
			
			vbat[0]=pSpa->GetBattNum(daBat-1);
		}
	}
	
	
	//	lavora per righe
	Battuta *curMeas=NULL;
	if(currentScore==0 && partnumber==0)
		pSpa=(Spartito*)GetFirst();
	for(i=0;i<nParti;i++)
	{
		curBat=daBat;
		curMeas=vbat[i];
		while(curBat<=aBat)
		{
			if(!curMeas->Transpose(clef,translation,interval,up, numofstaff, sharps, adjust)) 
				break;
			curBat++;
			curMeas=pSpa->GetNextBat(curMeas);
		}
		if(currentScore==0 && partnumber==0)
			pSpa=(Spartito*)GetNext(pSpa);
	}
}				

/************************************************/


int ListaSpartiti::GetNumeroBattute()
{
	return ((Spartito*)GetFirst())->GetNumeroBattute();
}

Bool ListaSpartiti::IsLastPage()
{
	Node *pn=first;
	
	while(pn!=NULL)
	{
		
		if(((Spartito*)pn->ObjectPtr)->GetfirstBatNextPage()==NULL)
			return TRUE;
		pn=GetNext(pn);  
	}
	return FALSE;
}

void ListaSpartiti::SetupIntRefs()
{
	Node *pn=first;
	
	while(pn!=NULL)
	{
		((Spartito*)pn->ObjectPtr)->SetupIntRefs();
		pn=GetNext(pn);
	}
}

void ListaSpartiti::SetCurrentPage(int Npagina)
{
	currentPage[DrawObject::GetCurrentView()]=Npagina;
}

PrintParams *ListaSpartiti::GetPrintViewParams()
{
	if(TmpPrintViewParams!=NULL)
		return TmpPrintViewParams;
	if(PrintViewParams==NULL)
		PrintViewParams=new PrintParams;
	return PrintViewParams;
}

PrintParams *ListaSpartiti::GetComputerViewParams()
{
	if(ComputerViewParams==NULL)
	{
		ComputerViewParams=new PrintParams;
		ComputerViewParams->SetScaleFactor(1.0);
	}
	return ComputerViewParams;
}

void ListaSpartiti::SetComputerViewParams(int tm, int bm, int lm, int rm, int sd, float magnification)
{
	PrintParams *pars=GetComputerViewParams();
	pars->SetTopMargin(1,tm);
	pars->SetBottomMargin(1,bm);
	pars->SetLeftMargin(1,lm);
	pars->SetRightMargin(1,rm);
	pars->SetStaffDistance(1,sd);
	pars->SetMagnification(magnification);
}

void ListaSpartiti::SetTmpPrintViewParams(PrintParams* tmpPars)
{
	TmpPrintViewParams=tmpPars;
}



void ListaSpartiti::SetMidiDinamicIndValue(int index, int val)
{
	if(index<=10 && index>=0)
		MidiDinamicIndValue[index]=val;
}

int ListaSpartiti::GetMidiDinamicIndValue(int index)
{
	if(index<=10 && index>=0)
		return MidiDinamicIndValue[index];
	else
		return (100);
}


void ListaSpartiti::GetPageMeasures(int& top, int& bottom)
{
	((Spartito*)GetFirst())->GetPageMeasures(top,bottom);
}

Spartito * ListaSpartiti::GetPtrSpa4Midi()
{
	return PtrSpa4Midi;
}

void ListaSpartiti::SetPtrSpa4Midi(Spartito *spa)
{
	PtrSpa4Midi=spa;
}

void ListaSpartiti::BalanceSpartiti()
{
	int diff=0;
	Battuta* refBat;
	Node *pn=first;
	int maxNBatt=0;
	
	while(pn!=NULL)	//Calcola numero battute da aggiungere
	{
		if(((Spartito*)pn->ObjectPtr)->GetNumeroBattute()>maxNBatt)
			maxNBatt=((Spartito*)pn->ObjectPtr)->GetNumeroBattute();
		pn=GetNext(pn);
	}
	
	pn=first;
	while(pn!=NULL)	//Aggiunge le battute
	{
		if(((Spartito*)pn->ObjectPtr)->GetNumeroBattute()<maxNBatt)
		{
			diff=maxNBatt-((Spartito*)pn->ObjectPtr)->GetNumeroBattute();
			for(int i=0;i<diff;i++)
			{
				refBat=((Spartito*)pn->ObjectPtr)->GetLastBat();
				((Spartito*)pn->ObjectPtr)->AddBattutaDopo(refBat);
				
			}
		}
		pn=GetNext(pn);
		
	}
}