/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: listaspa.hpp,v 1.1 2005/07/25 16:31:45 bellinpo Exp $ */


#ifndef LISTASPARTITI_HPP
#define LISTASPARTITI_HPP

#include "lioo.hpp"
#include "lst/lista.hpp"
#include "spa/spartito.hpp"
#include "bat/giustif.hpp"

class Giustificazione;
class PrintParams;

class MoodsMidiWrite;

class ListaSpartiti : public Lista
{
protected:
    NumCode CodeCounter;
    class Rectangle MusicRect[MAX_VIEW];
	// Rettangolo contenente la partitura vera e propria.
    class Rectangle PrintRect;
	// Rettangolo contenente la partitura vera e propria per l'anteprima di stampa
    int currentScore; 
	//the current Score viewed in single score view.
	// 0 = main score
	// 1 = 1st score
	// 2 = 2nd score
	// ...
	// WARNING: this attribute is temporary with multi view it may be removed 
	//          or changed.
	int currentPage[MAX_VIEW];	
	
	
	int MidiDinamicIndValue[10]; 
	
	
	bool cut_copy;				
	
	Spartito *PtrSpa4Midi;
	
	PrintParams *ComputerViewParams;
	PrintParams *PrintViewParams;
	PrintParams *TmpPrintViewParams;
	
	public:
		ListaSpartiti();
		~ListaSpartiti();
		
		bool Suona(MoodsMidiWrite *);
		
		void Init();
		void Free();
		
		
		int GetCurrentPage(){return currentPage[DrawObject::GetCurrentView()];}
		void SetCurrentPage(int Npagina);
		
		bool GetCutCopy() {return cut_copy;}			
		void SetCutCopy(bool cc) {cut_copy=cc;}			
		
		void BalanceSpartiti();
		
		void SetMusicRect(const class Rectangle& r);
		class Rectangle GetMusicRect();
		
		void SetPrintRect(const class Rectangle& r);
		class Rectangle GetPrintRect();
		
		void JustifIfFirst();
		
		Spartito *GetSpart(NumCode nc);
		
		void ListaSpartiti::PlaceSpartiti
			(int LeftMargin, int TopMargin, int RightMargin, int BottomMargin, int StaffDistance, 
			VUnit& width,bool printing);
		// Posiziona gli spartiti nella pagina attuale.
		void PlaceSimboli(int ncol);  
		int AlignBattute(int system=0);
		// Allinea le battute della pagina attuale.
		void PlaceIntervalli(Bool onlyLegature=TRUE);
		// Posiziona i simboli di intervallo nella pagina attuale.
		void PlaceSillabe();
		// Posiziona le sillabe nella pagina attuale.
		void LoadLyr(char *Dir);
		// Permette di caricare,se presente,la lirica
		// associata allo spartito.
		
		void GoBatt(NumCode nb);
		// Porta tutti gli spartiti alla battuta nb
		void GoBattInExecution(NumCode nb);
		Bool GoForward();
		// Avanza di una battuta.
		Bool GoBackward();
		// Indietreggia di una battuta.
		
		Bool GoNextPage();
		// Avanza di una pagina.
		Bool GoNextPageInExecution();
		
		Bool GoPrevPage();
		// Indietreggia di una pagina.
		
		void DrawPage(bool printing=false);
		// disegna la pagina.
		void DrawPageInExecution();
		void printPage(char codeWM_1[5],  char codeWM_2[5],  char codeWM_3[5],  char codeWM_4[7]);
		
		void ListaSpartiti::RedrawPage(int LeftMargin, int TopMargin, int RightMargin, int BottomMargin, int StaffDistance, VUnit& width,bool printing);
		// ricalcola e ridisegna la pagina.
		
		void DrawTagli(const Point&);
		// disegna i tagli necessari per arrivare al punto indicato.
		
		Bool Hit(const Point& p);
		Bool Hit(const Point&,SelObj,DrawObject*&,SymPath&);
		// identifica un oggetto.
		Bool Hit(const class Rectangle&,SelObj,short layer,SymPath&,SymPath&);
		// aggiunge alla lista gli oggetti nel rettangolo del tipo indicato.
		Bool SymbolicHit(SymPath&,SelObj,DrawObject*&);
		// Decodifica un path.
		
		void Add(Spartito *pSp,bool addInWDFTree=true);
		// Aggiunge un nuovo spartito in fondo alla lista.
		void AddAfter(Spartito *pSp, Spartito *pSpRef,bool modifyTree=true);
		// Aggiunge un nuovo spartito dopo quello passato come riferimento.
		
		
		Spartito* GetFirstVisible();
		Spartito* GetLastVisible();
		
		void Del(Spartito *pSp,bool modifyTree=true);
		// Cancella lo spartito dalla lista.
		
		Bool DeleteSym(DrawObject*);
		// cancella il simbolo indicato.
		
		void GiustificaBattute(Battuta**,TipoGiust,double kGiust, bool clearSpUt=false);	
		void GiustificaDaA(int daBat, int aBat,TipoGiust,double kGiust);
		int GetNumeroBattute();
		
		void Transpose(int daBat, int aBat,int partnumber,int clef,int translation,int interval,int up, int numofstaff, Bool sharps, Bool adjust);
		void SetMidiDinamicIndValue(int index, int val);
		int GetMidiDinamicIndValue(int index);
		Spartito * GetPtrSpa4Midi();
		void SetPtrSpa4Midi(Spartito *spa);
		/**********************/
		
		int AlignBattuteLineBreak(int system=0);
		// se deve anche sincronizzare per giustificare
		int StretchBattute(Battuta** vbat, UL spacecol, int sincr);  
		
		
		Bool IsLastPage();
		
		void SetupIntRefs();
		void SetCurrentScore(int curS) { currentScore=curS; };
		int GetCurrentScore() { return currentScore; };
		
		PrintParams *GetPrintViewParams();
		PrintParams *GetComputerViewParams();	
		void SetComputerViewParams(int tm, int bm, int lm, int rm, int sd, float magnification = 1.0);
		void SetTmpPrintViewParams(PrintParams*);
		
		void GetPageMeasures(int& top, int& bottom);

		int SaveXML(FILE *fp,char *fileName,char *Path);
};

#endif //LISTASPARTITI_HPP
