/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: partitur.hpp,v 1.1 2005/07/25 16:31:47 bellinpo Exp $ */


#ifndef PARTITURA_HPP
#define PARTITURA_HPP

#include "pag/casella.hpp"	

#include "pag/listapagine.hpp"
#include "par/listaspa.hpp"
#include "par/listapar.hpp"
#include "net/def_net.hpp"
#include "prt/prtpara.hpp"
#include "imp/mididef.hpp"

enum EXEC_MODE
{
	MM_SYNC,
		MM_USER,
		MM_MIDI
};

typedef unsigned long DWORD;
typedef unsigned char BYTE;
typedef long LONG;
const BYTE MAX_SYNC_ERROR_MESURES=1;

# define DECLICK_ON 1
# define DECLICK_OFF 0 
# define VIDEO_STEP 1 //percentuale di incremento o decremento della velocita' dello scrolling

typedef struct LEtic {
	char etic;
	NumCode nc;
	LEtic *pNext;
} ListaEtichette;


class Battuta;
class Spartito;
class AudioThread;
class MoodsMidiWrite;  
class Midi_sequencer;
class Midi_player_thread;

class SMRPartItem;
class SMRMainScoreItem;
class SMRLyricsItem;
class SMRSynchInfoItem;
class PrintParams;

class SMRAnnotation
{
public:
	wxArrayString shortText;
	wxArrayString text;
	wxArrayString url;
};

class Partitura
{
protected:
	bool printing_status;  
    bool SetImg;    
    
    bool Suona(MoodsMidiWrite *); 
	// crea file midi;
	
    SaveType SaveMethod;
	void Save();

	bool owner;
	// the object is the owner of the listaSpa, listaPar, listaPag
	// it has to delete them when destroing the object

    ListaSpartiti* listaSpa;
	// lista contenente le parti della partitura.
	
    ListaParentesi* listaPar;
	// lista contenente le parentesi della partitura.
	ListaPagine* listaPag;
	
	static DrawObject *pageBox;	
	
	Casella *casellaPtr;	
	
    SymCmd UsrSymCmd, NetSymCmd, UsrSymCmdTwoSel, AntiComando, Comando;
	// Comandi simbolici.
	
    wxString fileName;
	
    Lista listaSel;
	// lista dei simboli selezionati.
	
    Bool showTagli;
	// se TRUE visualizza i tagli delle note.
	
    StateImpag Impaginazione;
	// Indica lo stato dell'impaginazione.
	
    void DeselectAll();
	// deseleziona tutti gli oggetti che sono stati selezionati.
    void SelectAll();
	// Seleziona tutti gli oggetti che sono stati raccolti.
    void ResetPath(SymCmd& sc);
	// Azzera l'attributo Path di sc.
	
    wxString fileLoadName;
	wxString fileLoadPath;
	wxString fileSaveName;
	wxString fileSavePath;
	
	
	Bool FirstSel;
	// utilizzato per gestire i comandi in TwoSel
	
	
	long* ETT;
	NumCode ETTLenght;
	char ETTFileName[FILE_NAME_LEN];
	
	static ListaEtichette *listaEti;
	
	StateExecution StatoEsecuzione;
	// variabile di stato che controlla in quale fase di esecuzione siamo
	// impaginazione, init exec, esecuzione
	
	AudioThread *pAudioThread;
	int audioRate;
    char audioFile[FILE_NAME_LEN];
	int maxNumHit;
	EXEC_MODE mmExecMode;
	int declickMode;
	int* measureSync;
	int beginBlock;
	/////////////////////////////////////////////////

	void *selections;

public:	
	//*BIAN-VERR*
	bool CreateNetCommand(const Point&, char* cmd, char* anti);
	Bool DoNetCommand(SymCmd SC);
	//*BIAN-VERR*	

private:
	//
	//*BIAN-VERR*
	void ResetGenericCommand(SymCmd SC); 

	Bool CreateNetInsert(const Point& p);
	Bool CreateNetImmediate(const Point& p);

	Bool DoNetInsert(SymCmd SC);
	Bool DoNetImmediate(SymCmd SC);
	//*BIAN-VERR*

	//FFF
	//Bool GetCommandInsert(const Point& p);
	//Bool GetCommandSelect(const Point& p);
	//FFF
	//
    Bool SingleSel(const Point& p);
	// singola selezione: utilizzato solo da GruppoNote2Note
	// Viene eseguito quando si e' scelto un oggetto GruppoNote
    virtual Bool ImmediateSel(const Point& p);
	// selezione per esecuzione immediata del comando
	// viene eseguito quando si e' scelto
	// l'oggetto su cui eseguire il comando.
    Bool TwoSel(Point p);
	// selezione doppia: per inserimento intevalli
	Bool DragSel(const Point& p);	//DB
	// selezione tramite drag.
	
    Bool DoCommandDelBat(const Point& p);
	// eseguono su tutti gli spartiti, il comando impostato.
    Bool DoCommandBarra(const Point& p);
	// esegue su tutta una colonna di battute, il comando impostato.
    virtual Bool DoCommandGruppo();
	// esegue i due possibili comandi relativi al Gruppo di Note:
	//	GruppoNote2Note e Note2GruppoNote.
    Bool DoCommandDelColFig(const Point& p);
	
	Midi_sequencer* p_sequencer;
	bool Play_symbolic(int startMeasure=-1, int endMeasure=-1);
	void Midi_sequencer_pause();
	wxString MIDI_coded_key;
	friend class DMidi;
	
public :
	
	Midi_sequencer* Get_p_sequencer(){return p_sequencer;}
	MIDI_SYNC_RESULT New_Midi_bat(DWORD prev_division, LONG& te, DWORD metr_tempo);
	
protected:
	Bool InsertSel(const Point& p);
	// selezione per inserimento: gestisce 
	// l'inserimento di nuove figure. Viene eseguito quando
	// si e' scelto il punto in cui inserire
	
public:
	
    Partitura(bool empty=false);
    virtual ~Partitura();
	
	bool GetPrintingStatus(){return printing_status;};
	void SetPrintingStatus(bool ps){ printing_status = ps;};
	
	int GetLeftMargin();
	int GetRightMargin();
	int GetTopMargin();
	int GetBottomMargin();
	
	void SetLeftMargin(int);
	void SetRightMargin(int);
	void SetTopMargin(int);
	void SetBottomMargin(int);
	
	void SetDimSpazio(float );
	void SetDimFig(float );
	void SetScala(float );
	
    void CompViewParametersOp();
	
	void AddCasella(Casella *casella, int Npage);	
	Casella* GetCasellaPtr() {return casellaPtr;}
	void SetCasellaPtr(Casella* casellaptr) {casellaPtr=casellaptr;}
	static DrawObject* GetPageBox() {return pageBox;}
	static void SetPageBox(DrawObject* ptr) {pageBox=ptr;}
	
	ListaSpartiti* GetListaSpa();
	
	float GetDimSpazio();
	float GetDimFig();
	float GetScala();
	
    virtual void KillLioo();
    void SetImpaginazione(StateImpag);
	// imposta lo stato dell'impaginazione
    Bool GetImpaginazione();
	// ritorna lo stato dell'impaginazione: fine impaginazione oppure no
    Bool GetBeginImpag();
	// ritorna lo stato dell'impaginazione: inizio impaginazione oppure no
    Bool GetFirstEndBattImp();
	// ritorna lo stato dell'impaginazione: end batt imp prima volta
	
	Bool GetEsecuzione();
	Bool GetBeginEsec();
	Bool DoCommandAddBattuta(const Point& p);
	
	
	StateExecution GetStatoEsecuzione();
	void SetStatoEsecuzione(StateExecution );
	
    void BeginPaint();
	// toglie i tagli al mouse.
    void EndPaint();
	// rimette i tagli al mouse se erano richiesti.
	
    void ShowTagli();
	// abilita il disegni dei Tagli delle note.
    Bool HideTagli();
	// disabilita il disegni dei Tagli delle note, ritorna TRUE se i tagli
	// erano attivi.
    Bool GetShowTagli();
	// ritorna il valore di showTagli.
    virtual void DrawTagli(const Point&);
	// disegna i tagli necessari per arrivare al punto indicato.
	
	
    virtual void New();
	// inizia una nuova partitura. 
    virtual void Free();
	// not deleting parts and brackets
	void FullFree(bool removeFromWDFTree=true);
	// deletes all
	
    virtual void Init();
	
	
	
    void SetMusicRect(const class Rectangle& r);
	// Imposta il rettangolo che contiene la parte musicale.
	
	void SetPrintRect(const class Rectangle& r);
	//  Imposta il rettangolo che contiene la parte musicale per l'anteprima di stampa
	
    virtual void GoForward();
	// Avanza di una battuta.
    virtual void GoBackward();
	// Indietreggia di una battuta.
    virtual Bool GoNextPage(); 
	// avanza di una pagina  
    Bool GoNextPageInExecution();
	
    virtual Bool GoPrevPage();      
	// torna indietro di una battuta
    
    virtual void Draw();
	// disegna la pagina attuale della partitura.
	void DrawInExecution();
    virtual void Redraw(bool printing);
	virtual void Redraw(void);
	// ridisegna la pagina attuale ricalcolando le posizioni.
    virtual void GoTop();
	// Posiziona all'inizio della partitura.
    virtual void GoBatt(NumCode np);
    void GoBattInExecution(NumCode np);
    void GoBattIfOut(NumCode nb);
	
	
    virtual void AddSpartito(Spartito *pSp,bool addInWDFTree=true);
	// aggiunge lo spartito in basso.
    void AddNewSpartito(Spartito *pSpRef, Bool sopra,int nStaff=1, char* name=NULL);
	// Crea uno spartito e lo aggiunge sopra o sotto lo spartito passato per
	// riferimento.
	
    int GetNumSpa(void) { return listaSpa->GetNumObj(); };
	Spartito *GetFirstSpa();
	// Restituisce il puntatore al primo spartito.
	Spartito *GetLastSpa();
	// Restituisce il puntatore all'ultimo spartito.
    Spartito *GetNextSpa(Spartito *psp);
	// Restituisce il puntatore allo spartito successivo a psp.
    Spartito* GetPrevSpa(Spartito *psp);
	// Restituisce il puntatore allo spartito precedente a psp.
    Spartito* GetSpartNum(int n);
	// Ritorna lo spartito con numero progressivo n.
    Spartito* GetSpart(NumCode nc);
	// Ritorna lo spartito con NumericCode nc.
	
	void SetSaveMethod(SaveType NewSaveMethod);
	SaveType GetSaveMethod(void);
	
	void ResetACL(void);
	
	Bool CopiaFileMDS2HID(wxString SourcePath, wxString SourceName, wxString DestPath, wxString DestName);
	
	
	//
	// Impostazione dei comandi
	//
    void SetCommand(CmdID cmdId, SelType stype=NO_SEL, SelObj sobj=ANY_SEL, ArgType arg0=0,
		ArgType arg1=0, ArgType arg2=0, ArgType arg3=0, ArgType arg4=0, 
		char *txt=NULL, ArgType arg5=0, ArgType arg6=0, ArgType arg7=0,
		ArgType arg8=0);
	// permette la selezione degli oggetti di tipo SelObj con il tipo di
	// selezione SelType. Se viene premuto Ok nel menu' viene eseguito il 
	// comando CmdId sugli oggetti selezionati.
	// imposta gli argomenti del comando.
    void ResetCommand();
	// resetta il comando UsrSymCmd.
    SymCmd *GetSymCmd();
	// Restituisce il puntatore a UsrSymCmd.
    void SetUsrSymCmd(SymCmd sc);
	// Imposta UsrSymCmd.
	void SetNetSymCmd(SymCmd sc);
	// Imposta UsrSymCmd.
	
	//
	// SELEZIONE di oggetti
	//
    Bool Hit(const Point& p);
	// restituisce TRUE se l'oggetto e' stato colpito.
    Bool Hit(const Point&,SelObj,DrawObject*&,SymPath&);
	// identifica un oggetto.
    Bool Hit(const class Rectangle&,SelObj,short layer,SymPath&,SymPath&);
	// altro modo per selezionare un oggetto (tramite un rettangolo).
    Bool SymbolicHit(SymPath&,SelObj,DrawObject*&);
	// Decodifica un path.
	
	//
	// Esecuzione COMANDI
	//
	
	// quando si deve eseguire un comando ci sono 3 metodi che lo gestiscono:
	//
	//	- DoCommandImmediate: quando non occorre la una selezione (col 
	//		mouse) di un oggetto di partitura. 
	//
	//	- DoCommandRectSel: per i comandi per cui e' stata impostata una 
	//		selezione di tipo rettangolo; viene eseguito in corrispondenza
	//		del rilascio del tasto del mouse sinistro.
	//
	//	- DoCommand: per la maggior parte dei comandi. Viene eseguito in
	//		corrispondenza della pressione del tasto sinistro del
	//		mouse.
	//
    void DoCommandImmediate();
	// Esecuzione dei comandi che non necessitano di una selezione col 
	// mouse di un oggetto di partitura. 
	
    virtual Bool DoCommandRectSel(const Point& p);
	// Utilizzato per i comandi Note2GruppoNote e per l'inserimento degli Intervalli.
	
    Bool DoCommandInterv(SymCmd& sc);
	// Metodo per inserire un nuovo Intervallo.
	
    Bool DoCommand(const Point& p);
	// Esegue il comando: a seconda della selezione impostata, richiama
	//  il metodo opportuno: InsertSel, SingleSel, ImmediateSel, ...

    virtual Bool DoNetSymCmd();
	// Esegue il comando che gli viene passato attraverso la rete
	
    virtual CmdResult SymCommand(SymCmd&, Battuta *pb=NULL);
	// esegue comandi che riguardano la partitura.
	
    Bool DeleteSym(DrawObject *s);
	// cancella il simbolo puntato da s.
	
    Bool CmdAddLettera(SymCmd & SC);
	
	//
	// Scambio messaggi con RETE
	//
    virtual void NetInvioComando(SymCmd sc);
	// MaMass richiama questo metodo quando ha gi eseguito il comando e lo vuole fare eseguire al DLIOO.
	// LoMas lo richiama senza eseguire il comando.
	
    virtual void ReadFromNetwork();
	// Lettura (periodica) della rete.
    virtual void LoadPindex();
    virtual void InitParte();
	// Utilizzati in concomitanza di una lettura da file, per inizializzare
	// i DLIOO.
	
	virtual int GetIDleader(char* parte); 
	virtual void SetAbilCompress(char* parte, Bool abil); 
    virtual void SetAbilCompressAll(Bool abil); 
	virtual Bool GetAbil_Compress(int i);		
	
	//
	// Lettura e Salvataggio FILE
	//
    void SetFileLoadName(char* fName);
    char* GetFileLoadName();
	void SetFileSaveName(char* fName);
    char* GetFileSaveName();
	
	void SetPathSaveName(char* fName);
    char* GetPathSaveName();
	void SetPathLoadName(char* fName);
    char* GetPathLoadName();
	
    wxString GetfileLoadPath();
	
    virtual Bool Load(char *fName);
	// Carica da file.
    Bool LoadS(char);
	// Carica da stringa passata via rete.
    Bool ImportMidi(const char *fName,int trc, int keySignCode=-1); 
      // Carica da file midi.
    Bool ExportMidi(char *fName,int trc); 
      // Genera file midi.

	void HidePart(Spartito *currSpa);
	void ShowAllParts();
	void SplittingPart(Spartito *currSpa);
	void DisjoiningPart(Spartito *currSpa);
	bool JoiningParts(int part1, int part2, int part3);
	bool MergingParts(int part1, int part2);
	bool AreCompatible4MergeParts(Spartito *spa1,Spartito *spa2);
	
	void ViewLayerInPart(Spartito *currSpa);
	void BeamingInPart();
	void UpDownInPart();

	Bool DelSpartito(SymCmd SCmd);
	void InitTimeExec();
	void SetTimeExec(Battuta *pbt);
	// passando la posizione della battuta inserisce la sua durata
	// in esecuzione
	// id vale 1 ... NumeroBattute
	void SetTimeExec(int pos, long time);
	
	long GetTimeExec(int nc);
	// passando la posizione della battuta restituisce la sua durata in
	//esecuzione
	// nc vale 1 ... NumeroBattute
	void ReadETT(char *filename);    
	void ReadETT(char *filename, char *audioname);   
	// Legge il file .ETT (con estensione) ed inserisce i dati nel vettore ETT dopo averlo
	// creato.Cancella un eventuale vettore ETT precedente.
	void SaveETT(char* filename);
	// per adesso occorre l'estensione .quando bennati ha fatto per bene il loa e
	// sav rivedere
	
	void BuildETT(NumCode num_batt);
	// costruisce il vettore ETT di dimensione num_batt
	
	virtual void SincrBatt(NumCode);
	// momentaneo
    virtual void EseguiDebug(void*,void*,void*);
	
	virtual void InitEsecuzione(char*);
	virtual void JumpToBatt(NumCode&) {};
	virtual void InitImpaginazione();
	virtual void RichPageForward() {};
	
	void AddEtichetta(char,NumCode);
	Bool DelEtichetta(char c);
	
	char GetEtichetta(int n);
	// restituisce l'etichetta di posizione n-esima
	Bool FindEtichetta(char);
	
	NumCode GetBattEtichetta(char);
	// restituisce il numero progressivo della battuta a cui e' associata l'etichetta passata come parametro
	
	int GetNumEtichette();
	
	int GetNumeroBattute();
    
	int CalcLivello(Spartito *ps1, Spartito *ps2);
	
	int GetETTLenght();
	virtual void EditingGoBottom();     
	virtual void EditingJumpBatt(int nbat);
	virtual void EditingGoTop();
	virtual void RequestJump();
	virtual void GestLabel();
		
	virtual void NetPiu5();
	virtual void NetMeno5();
	
	virtual void GoBottom();
	// Posiziona alla fine della partitura.
	
	Bool OkInitEsec();
	Bool OkEsecuzione();
	Bool OkJump();
	Bool OkCompila();
	
	void AddParentesiGraffa(NumCode ns, NumCode ne, NumCode nc);
	void AddParentesiQuadra(NumCode ns, NumCode ne, NumCode nc);
	void CalcLivelli();
	void CalcProlBarre();
	
	void ScoreJustify();
	void SetModo(bool contr);
	Bool GetModo();
	void prova();
	
	bool InitMmExec(SMRSynchInfoItem* ett=NULL);
	void StartMmExec(int startMeasure=-1, int endMeasure=-1 );
	void StopMmExec( );
	void PauseMmExec( );
	void SlowerMmExec( );
	void FasterMmExec( );
	EXEC_MODE GetMmExecMode( );
	void SetMmExecMode(EXEC_MODE mode); 
	
	int GetDeclickMode( );
	void SetDeclickMode( int );
	
	int  GetAudioRate( );
	/////////////////////////////////////////////////////
	
	Bool IsLastPage();
	void SetFrom(Partitura*);
	
	void PrintView();
	// crea l'anteprima di stampa 
    void SetupIntRefs();
    
    void SetCurrentScore(int curS) { listaSpa->SetCurrentScore(curS); };
    int GetCurrentScore() { return listaSpa->GetCurrentScore(); };
	
	Bool LoadPartXML(const char *fName, SMRPartItem*);
	Bool LoadMainScoreXML(const char *fName, SMRMainScoreItem*);
	int SavePartXML(const char *fName, int pos); 
	int SaveMainScoreXML(const char *fName);

	void View_Link(bool);
	
	void SetAudioFile(const char *);
	const char *GetAudioFile();
	
	PrintParams *GetPrintViewParams();
	PrintParams *GetComputerViewParams();
	void SetComputerViewParams(int tm, int bm, int lm, int rm, int sd, float magnification = 1.0);
	
	void SetTmpPrintViewParams(PrintParams *ppars);
	int GetCurrentPage();
	bool CopyToClipboard(SymCmd SCmd);	
	bool PasteFromClipboard(Spartito *refPsp, bool sopra);	
	
	void CopyMeasure(int nProgressive);
	Bool PasteMeasureAfter(int nProgressive);
	void CopySingleMeasure(NumCode scoreId, NumCode measureId);
	Bool PasteSingleMeasureIn(NumCode scoreId, NumCode measureId);
	
	bool CutPageBox(DrawObject *obj);
	bool CopyPageBox(DrawObject *obj);	
	bool PastePageBox(const Point& p);
    bool FindPageBox(DrawObject *obj, bool del); 
	
	void AddSpartitoAt(Spartito *pSp,Spartito *refpSp, bool posUp);
	void GetPageMeasures(int& top, int& bottom);
	void RemovedLyr(SMRLyricsItem*);
	
	bool AlignScores(int maxMeasures=0);
	int GetNStaff();

	void SetSelections(void* n) { selections=n; }
	void* GetSelections() {  return selections; }

	wxArrayString GetSelectionIds();
	SMRAnnotation GetSelectionAnnotation(wxString selectionId);
	void Select(wxString selectionId, bool select=true);

private:
	const int CMDtoCL(const SymCmd&);
	const int CLtoCMD(const ClassID&);
	const int CLtoCMD(const tipoArmaturaChiave&);
	const int CLtoGROUP(const ClassID&);

};



#endif //PARTITURA_HPP
