/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include <xercesc/dom/deprecated/dom.hpp>
#include "xml/loa/XMLCommonClasses.hpp"
#include "par/partitur.hpp"

wxArrayString Partitura::GetSelectionIds()
{
	wxArrayString ids;
	DOM_Node* domSels= (DOM_Node*) selections;
	DOM_Node n;

	if(domSels)
	{
		n=domSels->getFirstChild();
		while(!n.isNull())
		{
			if(n.getNodeName().equals("selections"))
				break;
			n=n.getNextSibling();
		}
		if(!n.isNull())
			n=n.getFirstChild();

		while(!n.isNull())
		{
			const char* name=n.getNodeName().transcode();
			if(n.getNodeName().equals("selection"))
			{
				DOM_NamedNodeMap attrs=n.getAttributes();
				DOM_Node selectionId=attrs.getNamedItem("id");
				wxString id=selectionId.getNodeValue().transcode();
				ids.Add(id);
			}
			n=n.getNextSibling();
		}
	}
	return ids;
}

SMRAnnotation Partitura::GetSelectionAnnotation(wxString id)
{
	SMRAnnotation sel;
	DOM_Node* domSels= (DOM_Node*) selections;
	DOM_Node n;

	if(domSels)
	{
		n=domSels->getFirstChild();
		while(!n.isNull())
		{
			if(n.getNodeName().equals("selections"))
				break;
			n=n.getNextSibling();
		}
		if(!n.isNull())
			n=n.getFirstChild();
		
		while(!n.isNull())
		{
			if(n.getNodeName().equals("selection"))
			{
				DOM_NamedNodeMap attrs=n.getAttributes();
				DOM_Node selectionId=attrs.getNamedItem("id");
				wxString sid=selectionId.getNodeValue().transcode();
				if(sid==id)
				{
					break;
				}
			}
			n=n.getNextSibling();
		}
		
		if(n.isNull())
			return SMRAnnotation(); //selection not found
		
		n=n.getFirstChild();
		while(!n.isNull())
		{
			if(n.getNodeName().equals("annotation"))
			{
				DOM_Node a=n.getFirstChild();
				while(!a.isNull())
				{
					if(a.getNodeName().equals("short_text"))
						sel.shortText.Add(a.getFirstChild().getNodeValue().transcode());
					if(a.getNodeName().equals("text"))
						sel.text.Add(a.getFirstChild().getNodeValue().transcode());
					if(a.getNodeName().equals("url"))
						sel.url.Add(a.getFirstChild().getNodeValue().transcode());
					a=a.getNextSibling();
				}

			}
			n=n.getNextSibling();
		}
	}
	return sel;
}

void Partitura::Select(wxString id, bool select)
{
	DOM_Node* domSels= (DOM_Node*) selections;
	DOM_Node n;

	if(domSels)
	{
		n=domSels->getFirstChild();
		while(!n.isNull())
		{
			if(n.getNodeName().equals("selections"))
				break;
			n=n.getNextSibling();
		}
		if(!n.isNull())
			n=n.getFirstChild();
		
		while(!n.isNull())
		{
			if(n.getNodeName().equals("selection"))
			{
				DOM_NamedNodeMap attrs=n.getAttributes();
				DOM_Node selectionId=attrs.getNamedItem("id");
				wxString sid=selectionId.getNodeValue().transcode();
				if(sid==id)
				{
					break;
				}
			}
			n=n.getNextSibling();
		}
		
		if(n.isNull())
			return; //selection not found
		
		n=n.getFirstChild();
		while(!n.isNull())
		{
			if(n.getNodeName().equals("extaddress"))
			{
				DOM_NamedNodeMap attrs=n.getAttributes();
				DOM_Node partNode=attrs.getNamedItem("PART");
				int partID=partNode.isNull() ? 0 : atoi(partNode.getNodeValue().transcode());
				DOM_Node measureNode=attrs.getNamedItem("MEASURE");
				int measureID=measureNode.isNull() ? 0 : atoi(measureNode.getNodeValue().transcode());
				DOM_Node layerNode=attrs.getNamedItem("LAYER");
				int layerID=layerNode.isNull() ? 0 : atoi(layerNode.getNodeValue().transcode());
				DOM_Node figureNode=attrs.getNamedItem("FIGURE");
				int figureID=figureNode.isNull() ? 0 : atoi(figureNode.getNodeValue().transcode());
				
				SymPath path;
				path.Array[SPARTITO_PATH_LEVEL]=partID;
				path.Level=SPARTITO_PATH_LEVEL;
				path.Array[LISTA_PATH_LEVEL]=1;
				path.Array[BATTUTA_PATH_LEVEL]=measureID;
				if(measureID!=0)
					path.Level=BATTUTA_PATH_LEVEL;
				path.Array[LAYER_PATH_LEVEL]=layerID;
				if(layerID!=0)
					path.Level=LAYER_PATH_LEVEL;
				path.Array[FIGURA1_PATH_LEVEL]=figureID;
				if(figureID!=0)
					path.Level=FIGURA1_PATH_LEVEL;
				path.Array[SIMBOLO_PATH_LEVEL]=0;
				
				DrawObject* obj=NULL;
				if(SymbolicHit(path,ANY_SEL,obj) && obj)
				{
					if(select)
						obj->Select();
					else
						obj->Deselect();
				}
			}
			n=n.getNextSibling();
		}
	}
}
