/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "prtpara.hpp"
#include "lioo.hpp"

float MagnificationArray[11] = {0.5F, 0.6F, 0.7F, 0.8F, 0.9F, 1.0F, 1.1F, 1.2F, 1.3F, 1.4F, 1.5F};

PrintPageParams::PrintPageParams()
{
	TopMargin = 20;
	BottomMargin = 30;
	LeftMargin = 30;
	RightMargin = 40;
	StaffDistance = 50;
	NumStaff=9;
	NumSystems=2;
}

PrintParams::PrintParams()
{
	PageFormat = pfA4;
	
	LineLength = 100;
	Magnification = MagnificationArray[3];
	Printer = dpi600;
}

PrintParams::~PrintParams()
{
	
}

PrintPageParams& PrintParams::GetPageParams(int nPage)
{
	if(nPage==1)
		return pageParams[0];
	return pageParams[1];
}

wxString PrintParams::GetPageFormatAsString()
{
	switch(GetPageFormat())
	{
	case pfA4:
		return "A4";
	case pfA3:
		return "A3";
	case pfA4R:
		return "A4R";
	case pfA3R:
		return "A3R";
	}
	return "A4";
}

wxString PrintParams::GetPrinterAsString()
{
	switch(GetPrinter())
	{
	case dpi300:
		return "300";
	case dpi600:
		return "600";
	} 
	return "300";
}

void PrintParams::CalculateStaffDistance(int page, int numStaff, bool isMainScore)
{
	int pageHeight, pageTopMargin, pageBottomMargin, pageStaff, spaceY;
	float Magnif = GetMagnification();
	switch(GetPageFormat()) 
	{
	case pfA4:
		pageHeight = (int)(838/Magnif); 
		break;
	case pfA3:
		pageHeight = (int)(1186/Magnif);
		break;
	case pfA4R:
		pageHeight = (int)(593/Magnif); 
		break;
	case pfA3R:
		pageHeight = (int)(838/Magnif);
		break;
	}
	
	pageTopMargin = GetPageParams(page).GetTopMargin();
	pageBottomMargin = GetPageParams(page).GetBottomMargin();
	
	if(isMainScore)
	{
		pageStaff=GetNumSystems(page);
		spaceY = (pageHeight-pageTopMargin-pageBottomMargin-pageStaff*numStaff*VU_PER_ROW*4)/(2*numStaff*pageStaff+pageStaff-1);
	}
	else
	{
		pageStaff=GetNumStaff(page);
		spaceY = (pageHeight-pageTopMargin-pageBottomMargin)/(numStaff*pageStaff);
		spaceY=(spaceY-VU_PER_ROW*4 )/2;
	}
	
	GetPageParams(page).SetStaffDistance(spaceY);
}


