/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#ifndef PPARA_HPP
#define PPARA_HPP

enum PageFormat_type { pfA4, pfA3, pfA4R, pfA3R};

enum Printer_type { dpi300, dpi600 };

extern float MagnificationArray[11];


class PrintPageParams
{
protected:
	int TopMargin;    //in mm (in print view), in pixel in Computer view
	int BottomMargin; //in mm
	int LeftMargin;   //in mm
	int RightMargin;  //in mm
	int NumStaff;
	int NumSystems;
	int StaffDistance;
	bool InsPageNum;
	bool InsTitle;
public:
    PrintPageParams();
	
    int GetTopMargin() { return TopMargin; };
	void SetTopMargin(int TM) { TopMargin = TM; };
	
    int GetBottomMargin() { return BottomMargin; };
	void SetBottomMargin(int BM) { BottomMargin = BM; };
	
    int GetLeftMargin() { return LeftMargin; };
	void SetLeftMargin(int LM) { LeftMargin = LM; };
	
    int GetRightMargin() { return RightMargin; };
	void SetRightMargin(int RM) { RightMargin = RM; };
	
    int GetNumStaff() { return NumStaff; };
	void SetNumStaff(int NS) { NumStaff = NS; };
	
	int GetNumSystems() { return NumSystems; };
	void SetNumSystems(int NSys) { NumSystems = NSys; };
	
	int GetStaffDistance() { return StaffDistance; };
	void SetStaffDistance(int Di) { StaffDistance = Di; };
	
	bool GetInsPageNum() { return InsPageNum; };
	void SetInsPageNum(bool IPN) { InsPageNum = IPN; };
	
	bool GetInsTitle() { return InsTitle; };
	void SetInsTitle(bool IT) { InsTitle = IT; };
};


class PrintParams
{
protected:
	
	
    wxString FingerPrint1;
	wxString FingerPrint2;
	wxString FingerPrint3;
	
	PageFormat_type PageFormat;
	Printer_type Printer;
	float Magnification;
    int LineLength;   //in mm
	
	bool Braille;
	
	PrintPageParams pageParams[2];
	
public:
    PrintParams();
    ~PrintParams();
	
	
	PrintPageParams& GetPageParams(int nPage);
	
    int GetTopMargin(int nPage=1) { return GetPageParams(nPage).GetTopMargin(); };
	void SetTopMargin(int nPage, int TM) { GetPageParams(nPage).SetTopMargin(TM); };
	
    int GetBottomMargin(int nPage=1) { return GetPageParams(nPage).GetBottomMargin(); };
	void SetBottomMargin(int nPage,int BM) { GetPageParams(nPage).SetBottomMargin(BM); };
	
    int GetLeftMargin(int nPage=1) { return GetPageParams(nPage).GetLeftMargin(); };
	void SetLeftMargin(int nPage,int LM) { GetPageParams(nPage).SetLeftMargin(LM); };
	
    int GetRightMargin(int nPage=1) { return GetPageParams(nPage).GetRightMargin(); };
	void SetRightMargin(int nPage,int RM) { GetPageParams(nPage).SetRightMargin(RM); };
	
    int GetNumStaff(int nPage=1) { return GetPageParams(nPage).GetNumStaff(); };
	void SetNumStaff(int nPage,int NS) { GetPageParams(nPage).SetNumStaff(NS); };
	
	int GetNumSystems(int nPage=1) { return GetPageParams(nPage).GetNumSystems(); };
	void SetNumSystems(int nPage,int NSys) { GetPageParams(nPage).SetNumSystems(NSys); };
	
	int GetStaffDistance(int nPage=1) { return GetPageParams(nPage).GetStaffDistance(); };
	void SetStaffDistance(int nPage,int Di) { GetPageParams(nPage).SetStaffDistance(Di); };
	
	bool GetInsPageNum(int nPage=1) { return GetPageParams(nPage).GetInsPageNum(); };
	void SetInsPageNum(int nPage,bool IPN) { GetPageParams(nPage).SetInsPageNum(IPN); };
	
	bool GetInsTitle(int nPage=1) { return GetPageParams(nPage).GetInsTitle(); };
	void SetInsTitle(int nPage,bool IT) { GetPageParams(nPage).SetInsTitle(IT); };
	
    PageFormat_type GetPageFormat() { return PageFormat; };
    wxString GetPageFormatAsString();
	void SetPageFormat(PageFormat_type PF) { PageFormat = PF; };
	
	Printer_type GetPrinter() { return Printer; };
	wxString GetPrinterAsString();
	void SetPrinter(Printer_type Pt) { Printer = Pt; };
	
	float GetMagnification() { return Magnification; };
	int GetMagnificationIndex(float SM) 
	{
		for (int idx=0; idx<11; idx++)
			if (MagnificationArray[idx]==SM)
				return idx;
			return -1;
	};
	void SetMagnification(float SM) { Magnification = SM; };
	
    int GetLineLength() { return LineLength; };
	void SetLineLength(int LL) { LineLength = LL; };
	
	
	bool GetBraille() { return Braille; };
	void SetBraille(bool Br) { Braille = Br; };
	
    wxString GetFingerPrint1() { return FingerPrint1; };
	void SetFingerPrint1(const wxString & FP1) { FingerPrint1 = FP1; };
	
    wxString GetFingerPrint2() { return FingerPrint2; };
	void SetFingerPrint2(const wxString & FP2) { FingerPrint2 = FP2; };
	
    wxString GetFingerPrint3() { return FingerPrint3; };
	void SetFingerPrint3(const wxString & FP3) { FingerPrint3 = FP3; };
	
    float GetScaleFactor() { return GetMagnification(); };
	void SetScaleFactor(float SF) { SetMagnification(SF); };
	
	
	int SaveComputerViewXML(FILE*);
	int SavePrintViewXML(FILE*,bool isMainScore);
	
	void CalculateStaffDistance(int page, int numStaff, bool isMainScore);
};

#endif
