/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: metronom.cpp,v 1.1 2005/07/25 16:31:48 bellinpo Exp $ */


#include "sca/metronom.hpp"

#include "sca/def_sca.hpp"

#include "fig/nbreve.hpp"  
#include "fig/nsemibre.hpp"  
#include "fig/nminima.hpp"
#include "fig/nsemimin.hpp"
#include "fig/ncroma.hpp"
#include "fig/nsemicro.hpp"
#include "fig/nbiscrom.hpp"
#include "fig/nsemibis.hpp"
#include "fig/nfusa.hpp"


Metronomo :: Metronomo()
{
	
	ID = CL_METRONOMO;
	MetroTNum.SetFont(nTextFont);
	TParAperta.SetFont(nTextFont);
	TParChiusa.SetFont(nTextFont);
	TUguale.SetFont(nTextFont);
	ptrNota = NULL;
	TParAperta.SetTxt((unsigned char*)"(");
	TParChiusa.SetTxt((unsigned char*)")");
	TUguale.SetTxt((unsigned char*)"=");
};

Metronomo :: Metronomo(Metronomo* mt)
{
	
	ID = CL_METRONOMO;
	MetroTNum.SetFont(nTextFont);
	TParAperta.SetFont(nTextFont);
	TParChiusa.SetFont(nTextFont);
	TUguale.SetFont(nTextFont);
	ptrNota = NULL;
	TParAperta.SetTxt((unsigned char*)"(");
	TParChiusa.SetTxt((unsigned char*)")");
	TUguale.SetTxt((unsigned char*)"=");
	
	SetNota(mt->ptrNota->GetID(), mt->ptrNota->GetPPuntoValore()!=NULL);
	SetTNNumero(mt->MetroTNum.GetNumero());  
};


TNumerico* Metronomo :: GetTNumerico()
{
	return &MetroTNum;
}

Bool Metronomo :: SetTNNumero(int s)
{
	return MetroTNum.SetNumero(s);
};

Bool Metronomo :: SetNota(ClassID ident ,Bool punt)
{
	if (ptrNota != NULL) delete(ptrNota);
	switch(ident)
	{
    case CL_NBREVE: 
		ptrNota=new NBreve;
		break;
    case CL_NSEMIBREVE: 
		ptrNota=new NSemibreve;
		break;
    case CL_NMINIMA:
		ptrNota=new NMinima;
		break;
    case CL_NSEMIMINIMA:                
		ptrNota=new NSemiminima;
		break;	
    case CL_NCROMA:                
		ptrNota=new NCroma;
		break;    
    case CL_NSEMICROMA:                
		ptrNota=new NSemicroma;
		break;
    case CL_NBISCROMA:
		ptrNota=new NBiscroma;
		break;
    case CL_NSEMIBISCROMA:                
		ptrNota=new NSemibiscroma;
		break;
    case CL_NFUSA:
		ptrNota=new NFusa;
		break;
    default: 
		return FALSE;
	}
	if (ptrNota==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT"," Metronomo","SetNota()");
		return FALSE;
	};
	ptrNota->SetNotina(TRUE);
	ptrNota->SetGamboUp();
	ptrNota->SetCoda(TRUE);
	ptrNota->SetAltezza(0);
	ptrNota->SetHeadType(HEAD_CLASSIC);
	if (punt) ptrNota->CmdFiguraPunto(METRONPUNTI);
	return TRUE;
};


void Metronomo :: SetPos(DrawObject *drwobj,const Point& posizione)
{
	VUnit a,b,c;
	Point p,dp(GetDx(),GetDy());
	
	TParAperta.SetFont(nTextFont);
	TUguale.SetFont(nTextFont);
	MetroTNum.SetFont(nTextFont);
	TParChiusa.SetFont(nTextFont);
	
	DrawObject::SetPos(drwobj,posizione);
	TParAperta.SetPos(NULL,GetAbsPos());
	c=(TParAperta.GetVU2Rgt()+METROSPAZIO);
	ptrNota->SetPos(NULL,GetAbsPos()+Point(c,0));
	ptrNota->Adjust(FALSE, NULL, 0, 0); //aggiunto titti...verificare
	
	c=c+ptrNota->GetVU2Rgt()+METROSPAZIO;
	TUguale.SetPos(NULL,GetAbsPos()+Point(c,0));
	c=c+TUguale.GetVU2Rgt()+METROSPAZIO;								
	MetroTNum.SetPos(NULL,GetAbsPos()+Point(c,0));							
	c=c+MetroTNum.GetVU2Rgt();
	TParChiusa.SetPos(NULL,GetAbsPos()+Point(c,0));                                                                                                          	                	     		                                    		
	VU2Rgt=TParAperta.GetVU2Rgt()+ptrNota->GetVU2Rgt()+	
		TUguale.GetVU2Rgt()+MetroTNum.GetVU2Rgt()+
		TParChiusa.GetVU2Rgt()+METRONUMSPAZIO*METROSPAZIO;
	a=MetroTNum.GetVU2Up();
	b=ptrNota->GetVU2Up();
	if(a>b) 
		VU2Up=a;
	else 
		VU2Up=b;
	
	a=MetroTNum.GetVU2Dwn();
	b=ptrNota->GetVU2Dwn();
	if(a>b) 
		VU2Dwn=a+1;
	else 
		VU2Dwn=b+1;
};

VUnit Metronomo :: GetVU2Up()
{
	return VU2Up;
};

VUnit Metronomo :: GetVU2Rgt()
{
	return VU2Rgt;
};
VUnit Metronomo :: GetVU2Dwn()
{
	return VU2Dwn;
};

VUnit Metronomo :: GetVU2Lft()
{
	return 0;
};


void  Metronomo :: Draw()
{ 
	TParAperta.Draw();
	if (ptrNota!=NULL) 
	{
		ptrNota->Draw();
	}
	TUguale.Draw();
	MetroTNum.Draw();
	TParChiusa.Draw();
};

float Metronomo :: GetTimeExec()
{
	float ValNota = 0.0;
	
	ValNota = 60 / GetValNota();
	ValNota *= 1000;
	ValNota /= MetroTNum.GetNumero();
	
	return ValNota;  
	
}

float Metronomo :: GetValNota()
{
	float ValNota=0.0, rif;
	int i;
	
	switch(ptrNota->GetID())
	{
    case CL_NBREVE: 
		ValNota = 2.0;
		break;
    case CL_NSEMIBREVE: 
		ValNota = 1.0;
		break;
    case CL_NMINIMA:
		ValNota = 0.5;
		break;
    case CL_NSEMIMINIMA:                
		ValNota = 0.25;
		break;	
    case CL_NCROMA:                
		ValNota = 0.125;
		break;    
    case CL_NSEMICROMA:                
		ValNota = 0.0625;
		break;
    case CL_NBISCROMA:
		ValNota = 0.03125;
		break;
    case CL_NSEMIBISCROMA:                
		ValNota = 0.015625;
		break;
    case CL_NFUSA:
		ValNota = 0.0078125;
		break;
    default: 
		return 0;
	}
	
	if ((ptrNota->GetPPuntoValore())!=NULL)
	{
		rif=ValNota;
		for(i=1; i<=(ptrNota->GetPPuntoValore())->GetNumPunti(); i++)
		{
			ValNota += rif/2;
			rif /= 2;
		}
	}
    
	return ValNota;  
	
}

void Metronomo::SetColor(int col)
{
	DrawObject::SetColor((ColorIcon)col);
	TParAperta.SetColor(GetColor());
	if (ptrNota!=NULL) ptrNota->SetColor(GetColor());
	TUguale.SetColor(GetColor());
	MetroTNum.SetColor(GetColor());
	TParChiusa.SetColor(GetColor());
	return;
}

