/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: moviment.cpp,v 1.1 2005/07/25 16:31:48 bellinpo Exp $ */


#include "sca/moviment.hpp"
#include "sca/def_sca.hpp"



Movimento::Movimento()
{
	ID = CL_MOVIMENTO;	
	TMov.SetFont(nTextFont);
	ptrMetr = NULL;
};

Movimento::Movimento(Movimento* m)
{
	ID = CL_MOVIMENTO;	
	TMov.SetFont(nTextFont);
	TMov.SetTxt(m->TMov.GetTxt());
	if(m->ptrMetr!=NULL)
		ptrMetr = new Metronomo(m->ptrMetr);
	else
		ptrMetr=NULL;
};

Movimento::~Movimento() 
{
	if (ptrMetr!=NULL) delete ptrMetr;
}

Metronomo *Movimento::GetMetronomo()
{
	return ptrMetr;
}

Bool Movimento :: SetMetronomo()
{
	if (ptrMetr!=NULL)
		delete ptrMetr;
	ptrMetr=new (Metronomo);
	if (!ptrMetr) 
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Movimento"," SetMetronomo()");
	return TRUE;
};

void Movimento :: DelMetronomo()
{
	if (ptrMetr!=NULL)
	{
		delete ptrMetr;
		ptrMetr=NULL; 
	}
};

void Movimento :: SetPos(DrawObject *d,const Point &p)
{
	DrawObject :: SetPos(d,p);
	TMov.SetPos(NULL,GetAbsPos());
	Point dp(GetDx(),GetDy());
	
	TMov.SetFont(nTextFont);
	VU2Rgt = TMov.GetVU2Rgt() + MOVSPAZIO;
	VU2Up = TMov.GetVU2Up();
	VU2Dwn = TMov.GetVU2Dwn();
	
	if (ptrMetr!=NULL) 
	{
		ptrMetr->SetPos(NULL,GetAbsPos()+Point(VU2Rgt,0));
		VU2Rgt += ptrMetr->GetVU2Rgt();
		if (ptrMetr->GetVU2Up() > VU2Up) 
			VU2Up = ptrMetr->GetVU2Up();
		if (ptrMetr->GetVU2Dwn() > VU2Dwn)
			VU2Dwn = ptrMetr->GetVU2Dwn();
	};
};	

void Movimento :: SetColor(int col)
{
	TMov.SetColor((ColorIcon)col);
	if (ptrMetr!=NULL) 
		ptrMetr->SetColor(col);
	return;
}

void Movimento :: Draw()
{
	TMov.Draw();
	if (ptrMetr!=NULL) 
		ptrMetr->Draw();
	
};

VUnit Movimento::GetVU2Up()
{
	return VU2Up;
};  	

VUnit Movimento::GetVU2Rgt()
{
	return VU2Rgt;
};  	

VUnit Movimento::GetVU2Dwn()
{
	return VU2Dwn;
};  	

VUnit Movimento::GetVU2Lft()
{
	return 0;
};  	

Bool Movimento::MovSetTxt(unsigned char *p, ClassID ident, Bool punt, int s)
{ 
	if (ptrMetr!=NULL)
	{
		if (!(ptrMetr->SetTNNumero(s))) return FALSE;
		if (!(ptrMetr->SetNota(ident, punt))) return FALSE;
	}
	if (!(TMov.SetTxt(p))) return FALSE;
	return TRUE;
};  

float Movimento::GetTimeExec()
{
	
	if (ptrMetr!=NULL) 
		return ptrMetr->GetTimeExec();
	else
		return 0;
}

void Movimento::Edit(Battuta *pBat)
{
    unsigned char* num=ptrMetr->GetTNumerico()->GetTxt();
	unsigned char* mov=TMov.GetTxt();
	Nota* nota=ptrMetr->GetNota();
	int id = nota->GetID()-CL_NBREVE;
	bool punt=false;
	if(nota->GetPPuntoValore()!=NULL)
		punt=true;
	
	char buffer[200];
	strcpy(buffer,(const char*)mov);
	strcat(buffer,"*");
    
	char c[2];
    
	c[0]=id+'0';
	
    c[1]='\0';
    strcat(buffer,c);
	
	if (punt)
		strcat(buffer,"1");
	else strcat(buffer,"0");
	strcat(buffer,(const char*)num);
	
	if (WMetronomo("Metronome",buffer,TGENERICO_LEN))
	{  
		ClassID idNota;
		long sel;
		bool punto;
		wxString buf = buffer;
		wxString mov = buf.BeforeFirst('*');
		buf=buf.AfterFirst('*');
		wxString fract = buf.GetChar(0);
		wxString point = buf.GetChar(1);
		buf=buf.Remove(0,2);
		wxString numfig = buf;
		fract.ToLong(&sel);
		
		switch ((int)sel)
		{
		case 0:
			idNota = CL_NBREVE;
			break;
		case 1:
			idNota = CL_NSEMIBREVE;
			break;
		case 2:
			idNota = CL_NMINIMA;
			break;
		case 3:
			idNota = CL_NSEMIMINIMA;
			break;
		case 4:
			idNota = CL_NCROMA;
			break;
		case 5:
			idNota = CL_NSEMICROMA;
			break;
		case 6:
			idNota = CL_NBISCROMA;
			break;
		case 7:
			idNota = CL_NSEMIBISCROMA;
			break;
		case 8:
			idNota = CL_NFUSA;
			break;
		default:
			idNota = CL_NSEMIMINIMA;
			break;
		}
		
		ptrMetr->GetTNumerico()->SetTxt((unsigned char*)numfig.c_str());
		TMov.SetTxt((unsigned char*)mov.c_str());
		
		if(point == "1")
			punto=true;
		else
			punto=false;
		ptrMetr->SetNota(idNota,punto);
	}
}
