/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include "wx/wx.h"
#include "wx/string.h"
#include "wx/config.h"
#include "wx/mimetype.h"
#include "wx/process.h"

#ifdef __WXMSW__
#define PATH_SEP "\\"
#else
#define PATH_SEP "/"
#endif

#include "smr/dlg/settings.hpp"
#include "smr/smrmain.hpp"

#include "../wxr/envsettings.wxr"	
#include "../wxr/envsettings.h"

/*************************************************************/
BEGIN_EVENT_TABLE(SMREnvSettingsDlg, wxDialog)
EVT_BUTTON(ID_ONOK, SMREnvSettingsDlg::OnOk)
EVT_BUTTON(ID_ONCANC, SMREnvSettingsDlg::OnCancel)
END_EVENT_TABLE()

SMREnvSettingsDlg::SMREnvSettingsDlg(SMRSettings* ws)
{
	wsettings=ws;	
}

SMREnvSettingsDlg::~SMREnvSettingsDlg()
{
	int i=0;
	
	for(i=0;i<3;i++)
	{
		if (envpath[i]!=NULL)
			delete envpath[i];
	}
}



void SMREnvSettingsDlg::OnOk(wxCommandEvent& event)
{
	EndModal(wxID_OK);
}

void SMREnvSettingsDlg::OnCancel(wxCommandEvent& event)
{
	EndModal(wxID_CANCEL);
}


void SMREnvSettingsDlg::ShowDlg()
{
	wxString temp;
	wxResourceParseData(envsettings); 
	
	if (LoadFromResource(NULL, "envsettings"))
	{
		
		for(int i=0;i<4;i++)
		{
			temp.sprintf("path%d",i);
			envpath[i]=(wxTextCtrl *)wxFindWindowByName(temp, this);
		}
		
	}
	else
		wxMessageBox("Error loading resources y!","Error!", wxOK | wxICON_EXCLAMATION, NULL);
	
	SetSettings();
	Centre();
	if (ShowModal() == wxID_OK) 
	{
		UpdateDlg();
		wsettings->SaveSettings();	
	}
	
}

void SMREnvSettingsDlg::UpdateDlg()
{		
	
	wxString temp = PATH_SEP;
	if  (envpath[0]->GetValue().Last() != (char)*temp.c_str())	
		wsettings->SetFilesPath(envpath[0]->GetValue()+PATH_SEP);
	else 
		wsettings->SetFilesPath(envpath[0]->GetValue());
	if  (envpath[1]->GetValue().Last() != (char)*temp.c_str())	
		wsettings->SetErrPath(envpath[1]->GetValue()+PATH_SEP);
	else 
		wsettings->SetErrPath(envpath[1]->GetValue());
	if  (envpath[2]->GetValue().Last() != (char)*temp.c_str())	
		wsettings->SetMidiPath(envpath[2]->GetValue()+PATH_SEP);
	else 
		wsettings->SetMidiPath(envpath[2]->GetValue());
	if  (envpath[3]->GetValue().Last() != (char)*temp.c_str())	
		wsettings->SetFontPath(envpath[3]->GetValue()+PATH_SEP);
	else 
		wsettings->SetFontPath(envpath[3]->GetValue());
}

void SMREnvSettingsDlg::SetSettings()
{
	
	envpath[0]->SetValue(wsettings->GetFilesPath());
	envpath[1]->SetValue(wsettings->GetErrPath());
	envpath[2]->SetValue(wsettings->GetMidiPath());
	envpath[3]->SetValue(wsettings->GetFontPath()); 	
}

/*************************************************************/

SMRSettings::SMRSettings()
{
}


SMRSettings::~SMRSettings()
{
}

wxString SMRSettings::GetBimPath()
{
	if(GetAppsPath()=="" || GetAppsPath()=="\\")
		return SMRApp::GetApp().GetAppPath();
	return GetAppsPath();
}

wxString SMRSettings::GetGZipPath()
{
	if(GetAppsPath()=="" || GetAppsPath()=="\\")
		return SMRApp::GetApp().GetAppPath();
	return GetAppsPath();
}

bool SMRSettings::LoadSettings()
{
	wxConfigBase *pConfig=wxConfigBase::Get();

	if(!pConfig->Exists("/paths"))
		return false;

	pConfig->Read("/paths/smrpath", &Files);
	if(Files=="")
	{
		pConfig->Read("/paths/wdfpath", &Files);	
	}

	Files+=PATH_SEP;
	pConfig->Read("/paths/errpath", &Err);	
	Err+=PATH_SEP;
	pConfig->Read("/paths/midipath", &Midi);
	Midi+=PATH_SEP;		
	pConfig->Read("/paths/fontpath", &Font);
	Font+=PATH_SEP;
	pConfig->Read("/paths/appspath", &Apps);
	Apps+=PATH_SEP;

	return true;
}

void SMRSettings::SaveSettings()
{
	wxConfigBase *pConfig=wxConfigBase::Get();
	char* sep=PATH_SEP;
	
	
	wxString files=GetFilesPath();
	if (files.Last() == sep[0])	
		files=files.RemoveLast();
	pConfig->Write("/paths/smrpath", files);
	
	wxString err=GetErrPath();
	if (err.Last() == sep[0])	
		err=err.RemoveLast();
	pConfig->Write("/paths/errpath", err);
	
	wxString font=GetFontPath();
	if (font.Last() == sep[0])	
		font=font.RemoveLast();
	pConfig->Write("/paths/fontpath", font);
	
	wxString midi=GetMidiPath();
	if (midi.Last() == sep[0])	
		midi=midi.RemoveLast();
	pConfig->Write("/paths/midipath", midi);

	wxString apps=GetAppsPath();
	if (apps.Last() == sep[0])	
		apps=apps.RemoveLast();
	if(apps=="")
	{
		apps=SMRApp::GetApp().GetAppPath();
		apps=apps.RemoveLast();
	}

	pConfig->Write("/paths/appspath", apps);

	pConfig->Flush();	
}

/***********************************************************************/
class CleanAfterProcessTermination : public wxProcess
{
protected:
	wxString fileToBeRemoved;
public:
	CleanAfterProcessTermination(wxString file) { fileToBeRemoved=file; };
	void OnTerminate(int pid, int status) const;
};

/***********************************************************************/
void CleanAfterProcessTermination::OnTerminate(int pid, int status) const
{
	wxRemoveFile(fileToBeRemoved);
}

/***********************************************************************/
wxString CleanCommand(wxString open)
{
	wxString opx;
	char buf[250];
	unsigned int i,l=open.Length();
	int stat=0,k=0;
	
	if (open.Contains("\" \""))
	{ 
		open.Replace("\" \"","", TRUE);
	}
	
	
	if (open.Contains("WX_DDE#"))
	{  
		open.Replace("WX_DDE#","", TRUE);
		open.Replace("/dde#Flash##[open()]","", TRUE);
		l=open.Length();
		for (i=0;i<l;i++)
		{ 
			if (open[i]=='\"') 
			{ 
				if (stat==0) stat=1;
				else stat=2;
			}
			if (stat!=0) 
			{ 
				buf[k]=open[i]; 
				k++; 
			}
			if (stat==2) break;
		}
		buf[k]=0;	
		opx=buf;
	}
	else
		opx=open;
	
	if (stat==0)
		opx=open;
	
	return opx;
}

void SMRSettings::Execute(wxString parameters, ApplicationId ID_prg, bool remove)
{
	wxString command;

	wxString ext=parameters.AfterLast('.');
	if (ext!="")
	{ 
		wxFileType *filetype = wxTheMimeTypesManager->GetFileTypeFromExtension(ext);
		if (filetype!=NULL)
		{ 
			wxString open, type;
			wxFileType::MessageParameters params(" ", " ");
			if (filetype->GetOpenCommand(&open, params))
			{
				wxString cmd=CleanCommand(open); 
				if (cmd[cmd.Length()-1]=='\\' && cmd[cmd.Length()-2]=='\\')
					cmd[cmd.Length()-1]=0;
				command=cmd+" \""+parameters+"\"";

				wxProcess* doOnTermination=NULL;
				if(remove)
					doOnTermination=new CleanAfterProcessTermination(parameters);

				if (wxExecute(command,FALSE,doOnTermination)==0)
					wxMessageBox("Impossible to Execute!\n"+open, "Error", wxOK |  wxCENTRE | wxICON_EXCLAMATION,NULL); 
			}
			else 
				wxMessageBox("No program is correctly associated with the extension:\n"+ext, "Error", wxOK |  wxCENTRE | wxICON_EXCLAMATION,NULL); 
		}
		else
			wxMessageBox("No program associated with the extension:\n"+ext, "Error", wxOK |  wxCENTRE | wxICON_EXCLAMATION,NULL); 
	}
}	
