/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */
#include <xercesc/dom/deprecated/DOMParser.hpp> //nedded otherwise gives error the following inclusion on VS6.0
#include "xercesc/framework/localfileinputsource.hpp"
#include "bit/odmap.hpp"
#include "bit/bitstreamdecoder.hpp"

#include <wx/wx.h>
#include <wx/mstream.h>
#include <wx/wfstream.h>
#include <wx/zstream.h>
#include <wx/listimpl.cpp>
#include <wx/dir.h>
#include <wx/progdlg.h>

#ifdef WX226
#include <fstream.h>
#else
#include <fstream>
using namespace std;
#endif

#include "smr/smr.hpp"
#include "smr/smrmain.hpp"

#include "par/partitur.hpp"
#include "bit/bitstreamencoder.hpp"
#include "drw/scorefrm.hpp"
#include "musiceditor.hpp"
#include "net/mamass.hpp"
#include "net/lomas.hpp"
#include "mil/smfl.hpp"
#include "xml/loa/wdfdomparser.hpp"
#include "xml/loa/wdfcomponentparser.hpp"
#include "xml/sav/saveheaderxml.hpp"

#include "ISOMovies.h"
#include "MP4Atoms.h"

WX_DEFINE_LIST(SMRItemList);

#define DEFAULT_EXT ".xsm"

SMRFile::SMRFile()
{
	m_mainScore = new Partitura();
	Clear();
	m_codingType=XMLF;
	m_destroyMS=true;
	m_pictureHeight=0;
	m_pictureWidth=0;
	m_version=0;
	m_musicEditorIsOpen=false;
	m_lastLyrID=0;
	m_fromMIDI=false;
}

SMRFile::~SMRFile()
{
	if(m_destroyMS)
		delete m_mainScore;
}

Partitura* SMRFile::GetMainScore()
{
	return m_mainScore;
}

Spartito* SMRFile::GetPart(int partNumber)
{
	return m_mainScore->GetSpartNum(partNumber-1);
}

int SMRFile::GetPartCount()
{
	return m_mainScore->GetNumSpa();
}

const char* SMRFile::GetPartName(int partNumber)
{
	return m_mainScore->GetSpartNum(partNumber-1)->GetStrumEsec();
}

void SMRFile::Clear()
{
	//TBR
	m_mainScore->FullFree();
}

int SMRFile::GetMeasureCount()
{
	return m_mainScore->GetNumeroBattute();
}

int SMRFile::New(const wxString& title)
{
	m_itemsList.Clear();
	
	m_filename = title+DEFAULT_EXT;
	m_filePath = (wxString)(smrgetenv("SMR_FILES_PATH")) + PATH_SEP + m_filename;
	m_title = title;
	m_fromMIDI = false;
	if(m_mainScore!=NULL && m_destroyMS)
	{
		m_mainScore->FullFree();
		delete m_mainScore;
		m_mainScore=NULL;
	}
	
	// load SMFL
	SMFLManager::Get()->LoadDefault();
	
	m_mainScore=new MaMass;
	
	wxString Path=GetFilename();
	
	m_mainScore->New();
	m_mainScore->GoTop(); 
	
	SMRMainScoreItem* ms=new SMRMainScoreItem(this,"Main score", GetTitle()+".xml");
	m_itemsList.Append(ms);

	SMRPartItem *ps=new SMRPartItem(this,"Single Part", "1."+GetTitle()+".xml");
	m_itemsList.Append(ps);
	
	return 1;
}

int SMRFile::Load(const wxString &FullPath)
{
   	wxString ext;
	wxString xml;
	wxString objPath=FullPath;
	
	if(wxDir::Exists(objPath))
	{
		//FullPath is a directory i.e. c:\smr\test.xsm
		m_filePath=objPath;
		ext="*" DEFAULT_EXT;
		
		wxDir dir(objPath);
		if(!dir.IsOpened())
			return FALSE;
		if(!dir.GetFirst(&m_filename,ext))
		{
			if(!dir.GetFirst(&m_filename,"*.wdf"))
				return FALSE;
		}
		m_title=m_filename.BeforeLast('.');
		
		//  to clean files produced from sibelius plug in
		//SibeliusSource(FilePath+ "\\"+ Filename);
		
		wxString dummy;
		if(dir.GetNext(&dummy))
		{
			wxMessageBox(wxString::Format("WARNING! more than one .xsm/.wdf file in %s\n"
				"opening %s",FullPath.c_str(),m_filename.c_str()));
			return FALSE;
		}
	}
	else if(wxFileExists(objPath))
	{
		//FullPath is a file i.e. c:\smr\test.xsm\test.xsm
		wxSplitPath(objPath,&m_filePath,&m_filename,&ext);
		m_title=m_filename;
		ext.MakeLower();
		if (ext!="xsm" && ext!="wdf")
			return FALSE;
		m_filename+="."+ext;
	}
	else
		return FALSE;

	CloseMusicEditor();
	if(m_mainScore!=NULL)
	{
		delete m_mainScore;
		m_mainScore=NULL;
	}
	m_itemsList.Clear();
	
	xml=m_filePath + PATH_SEP + m_filename;
		
	WDFHeaderParser HeaderParse((char *)xml.c_str(), this);

	if(!HeaderParse.ReadHeader())
	{
		wxMessageBox("WDFHeader "+xml+" not found or not valid");
		return FALSE;		
	}

	LoadScore();

	SetMusicEditorSMRF(this);
	
	return 1;
}
int SMRFile::Save()
{
	::Save save;

	save.Naviga(this);

	SaveSMRStructure(GetPath()+"\\"+GetTitle()+"-index.xml");

	if (m_title!="" && SMRApp::GetApp().IsInteractive())
	{
		wxMessageBox("SMR \""+m_title+"\" saved in:\n"+GetPath());
	}
	return 1;
}
int SMRFile::SaveAs(const wxString& name, const wxString& path)
{
	wxString srcPath,dstPath=path;
	srcPath = GetPath()+PATH_SEP;
	if(dstPath=="")
		dstPath = (wxString)smrgetenv("SMR_FILES_PATH")+name+DEFAULT_EXT;

	::Save save;

	m_title=name;
	m_filename=name+DEFAULT_EXT;
	m_filePath=dstPath;

	dstPath+=PATH_SEP;

	// change filenames of mainscore, parts and lyrics 
	wxSMRItemListNode *node = this->m_itemsList.GetFirst();
    while (node)
	{
		SMRItem *item = node->GetData();
		if (item->GetType()==SMR_MainScore)
		{
			wxString newFile=name+".xml";
			item->SetFilename(newFile);
		}
		else if(item->GetType()==SMR_Part)
		{
			wxString file=item->GetFilename(),newFile;
			newFile=file.BeforeFirst('.')+'.'+name+".xml";
			item->SetFilename(newFile);
		}
		else if (item->GetType()==SMR_Lyrics)
		{
			wxString file=item->GetFilename(),newFile;
			newFile=file.BeforeLast('.')+".xml";
			item->SetFilename(newFile);
		}
		node = node->GetNext();
	}

	save.Naviga(this);
	SaveSMRStructure(dstPath+name+"-index.xml");

	// copy files
	node = m_itemsList.GetFirst();
    while (node)
	{
		SMRItem *item = node->GetData();
		wxString file;
		switch(item->GetType())
		{
		case SMR_SMFL:
		case SMR_Font:
		case SMR_Lyrics:
		case SMR_SynchInfo:
		case SMR_Audio:
			file=item->GetFilename();
			wxCopyFile(srcPath+file,dstPath+file);
			break;
		}
		node = node->GetNext();
	}

	if (SMRApp::GetApp().IsInteractive())
		wxMessageBox("SMR \""+m_title+"\" saved in:\n"+GetPath());

	return 1;
}
int SMRFile::EncodeSMXF(const wxString FullPath, Coding_Type codingType)
{
	wxFileOutputStream* smrFile = new wxFileOutputStream(FullPath);
	wxFileOutputStream* configFile = new wxFileOutputStream(FullPath+".config");
	if (!smrFile->Ok() || !configFile->Ok())
	{	
		delete configFile;
		delete smrFile;
		return -1;
	}

	int err=EncodeSMXF(configFile, smrFile, codingType);

	delete configFile;
	delete smrFile;
	return err;
}

int SMRFile::EncodeSMXF(wxOutputStream* configFile, wxOutputStream* smrFile, Coding_Type codingType, SMRAudioItem* midiItem)
{
	BitStreamEncoder encoder(configFile);

	encoder.setNumberOfParts(m_mainScore->GetNumSpa());
	encoder.setCodingType(codingType);
	encoder.setPictureWidth(m_pictureWidth);
	encoder.setPictureHeight(m_pictureHeight);
	encoder.setVersion(0);
	if(midiItem!=NULL)
		encoder.setMidiUrl(midiItem->GetID());

	encoder.writeDownDecoderConfig();

	encoder.setOStream(smrFile);

	wxSMRItemListNode *node = m_itemsList.GetFirst();
    while (node)
	{
		SMRItem *item = node->GetData();
		wxString FileName = item->GetFilename();
		int partID = atoi((FileName.BeforeFirst('.')).c_str());


		switch (item->GetType())
		{
		case SMR_MainScore:
			if(!m_fromMIDI)
				encoder.AddSource(MAIN, new wxFileInputStream(GetPath()+"\\"+FileName));
			break;
		case SMR_Part:
			if(!m_fromMIDI)
				encoder.AddSource(PART, new wxFileInputStream(GetPath()+"\\"+FileName), partID);
			break;
		case SMR_Lyrics:
			if(!m_fromMIDI)
				encoder.AddSource(LYRC, new wxFileInputStream(GetPath()+"\\"+FileName), partID);
			break;
		case SMR_SynchInfo:
			encoder.AddSource(SYNC, encoder.encodeSyncInfo(new wxFileInputStream(GetPath()+"\\"+FileName)));	
			break;
		case SMR_SMFL:
			encoder.AddSource(SMFL, new wxFileInputStream(GetPath()+"\\"+FileName));
			break;
		case SMR_Font:
			encoder.AddSource(FONT, encoder.encodeFont(new wxFileInputStream(GetPath()+"\\"+FileName)));
			break;
		} 

		node = node->GetNext();
	}

	//write all the chunks in an Access Unit
	if (encoder.writeDownAccessUnit() != 0)
	{
		return -1;
	}

	return 0;
}

int SMRFile::EncodeMIDI(wxOutputStream* configFile, SMRAudioItem* midiItem)
{
	SABitStreamEncoder encoder(configFile);

	return encoder.writeDownDecoderConfig(15, new wxFileInputStream(GetPath()+"\\"+midiItem->GetFilename()));
}


static ISOErr setDecoderSpecificInfo( ISOHandle decoderSpecificInfoH, char* smrDecoderSpecificInfo, unsigned int length )
{
	ISOErr err;
	u8 *p;
	u32 i;
	u8 l[4],nl=0;

	err = ISONoErr;

	if(length<128)
	{
		l[0]=length;
		nl=1;
	}
	else if(length< (1<<14) )
	{
		l[0]=(length & 0x3f80) >> 7 | 0x80;
		l[1]=(length & 0x7f);
		nl=2;
	}
	else
	{
		err = ISOBadParamErr; goto bail;
	}
	
	err = ISOSetHandleSize( decoderSpecificInfoH, length+1+nl ); if (err) goto bail;
	p = (u8*) *decoderSpecificInfoH;
	*p++ = 0x05; /* DecSpecificInfoTag, 14496-1, Sec 8.2.2.2, Table 1*/
	for(i=0;i<nl;i++)
		*p++ = l[i]; /* sizeOfInstance, 14496-1, Sec 12.3.3 */
	for (i=0; i<length; i++)
		*p++ = smrDecoderSpecificInfo[i];
bail:
	return err;
}

static ISOErr createODTrack(ISOMovie& moov, char* odAccessUnit, int odAccessUnitSize)
{
	ISOTrack odTrak;
	ISOMedia odMedia;
	ISOErr err;
	u32 timeScale = 10; //to be verified
	u64 mediaDuration;
	u8* p;

	//create the OD track
	err = ISONewMovieTrack( moov, ISONewTrackIsAudio, &odTrak ); if (err) goto bail;
	err = MP4AddTrackToMovieIOD( odTrak ); if (err) goto bail;
	err = ISONewTrackMedia( odTrak, &odMedia, MP4ObjectDescriptorHandlerType, timeScale, NULL ); if (err) goto bail;
	err = ISOBeginMediaEdits( odMedia ); if (err) goto bail;
	
	ISOHandle odEntryH;
	ISOHandle odDataH;
	ISOHandle odDurationH;
	ISOHandle odSizeH;
	ISOHandle odDecoderSpecificInfoH;
	
	err = ISOSetMediaLanguage( odMedia, "und" ); if (err) goto bail;

	err = ISONewHandle( 0, &odDecoderSpecificInfoH ); if (err) goto bail;
	err = setDecoderSpecificInfo( odDecoderSpecificInfoH, 0, 0 ); if (err) goto bail;

	err = ISONewHandle( 0, &odEntryH ); if (err) goto bail;
	err = MP4NewSampleDescription( odTrak, odEntryH,
		1, //dataReferenceIndex
		0x02, //systems v2
		0x01, //OD stream type
		odAccessUnitSize,
		128000,
		128000,
		odDecoderSpecificInfoH ); if (err) goto bail;

	/////////////////
	err = ISONewHandle( sizeof(u32), &odDurationH ); if (err) goto bail;
	*((u32*) *odDurationH) = 0;

	err = ISONewHandle( 0, &odDataH ); if (err) goto bail;
	err = ISONewHandle( sizeof(u32), &odSizeH ); if (err) goto bail;
	
	err = MP4SetHandleSize( odDataH, odAccessUnitSize ); if (err) goto bail;
	p = (u8*) *odDataH;
	memcpy ( p, odAccessUnit, odAccessUnitSize);

	err = ISOGetHandleSize( odDataH, (u32*) *odSizeH ); if (err) goto bail;
	err = ISOAddMediaSamples( odMedia, odDataH, 1, odDurationH, odSizeH, odEntryH, NULL, NULL ); if (err) goto bail;
	if ( odEntryH )
	{
		err = ISODisposeHandle( odEntryH ); if (err) goto bail;
		odEntryH = NULL;
	}	
	
	err = ISOEndMediaEdits( odMedia ); if (err) goto bail;
	err = ISOGetMediaDuration( odMedia, &mediaDuration ); if (err) goto bail;
	err = ISOInsertMediaIntoTrack( odTrak, 
		0, //trackStartTime
		0, //mediaStartTime
		mediaDuration,
		1 //mediaRate
		); if (err) goto bail;
bail:
	return err;
}

static ISOErr createSMRTrack(ISOMovie& moov, ESData* smrES, ESData* midiES, char* smrAccessUnit, int smrAccessUnitSize)
{
	ISOErr err;
	ISOTrack smrTrak;
	ISOMedia smrMedia;
	u32 timeScale = 10; //to be verified
	u64 mediaDuration;
	u8* p;

	//create the SMR track
	err = ISONewMovieTrackWithID( moov, ISONewTrackIsAudio, smrES->getESID(), &smrTrak ); if (err) goto bail;

	if(midiES)
	{
		u32 outReferenceIndex;
		// adds a reference to the ES containing the MIDI (ESID=101)
		err = ISOAddTrackReferenceWithID( smrTrak, midiES->getESID(), MP4ODTrackReferenceAtomType, &outReferenceIndex); if (err) goto bail;
	}
	
	err = ISONewTrackMedia( smrTrak, &smrMedia, ISOAudioHandlerType, timeScale, NULL ); if (err) goto bail;
	err = ISOBeginMediaEdits( smrMedia ); if (err) goto bail;
	
	ISOHandle smrEntryH;
	ISOHandle smrDataH;
	ISOHandle smrDurationH;
	ISOHandle smrSizeH;
	ISOHandle smrDecoderSpecificInfoH;
	
	err = ISOSetMediaLanguage( smrMedia, "und" ); if (err) goto bail;

	err = ISONewHandle( 0, &smrDecoderSpecificInfoH ); if (err) goto bail;
	err = setDecoderSpecificInfo( smrDecoderSpecificInfoH, smrES->geDecoderInfo(), smrES->getDecoderInfoSize() ); if (err) goto bail;

	err = ISONewHandle( 0, &smrEntryH ); if (err) goto bail;
	err = MP4NewSampleDescription( smrTrak, smrEntryH,
		1, //dataReferenceIndex
		smrES->getOTI(),
		smrES->getStreamType(),
		smrES->getBufferSizeDB(),
		smrES->getMaxBitRate(),
		smrES->getAvgBitRate(),
		smrDecoderSpecificInfoH ); if (err) goto bail;

	/////////////////
	err = ISONewHandle( sizeof(u32), &smrDurationH ); if (err) goto bail;
	*((u32*) *smrDurationH) = 0;

	err = ISONewHandle( 0, &smrDataH ); if (err) goto bail;
	err = ISONewHandle( sizeof(u32), &smrSizeH ); if (err) goto bail;
	
	err = MP4SetHandleSize( smrDataH, smrAccessUnitSize ); if (err) goto bail;
	p = (u8*) *smrDataH;
	memcpy ( p, smrAccessUnit, smrAccessUnitSize);

	err = ISOGetHandleSize( smrDataH, (u32*) *smrSizeH ); if (err) goto bail;
	err = ISOAddMediaSamples( smrMedia, smrDataH, 1, smrDurationH, smrSizeH, smrEntryH, NULL, NULL ); if (err) goto bail;
	if ( smrEntryH )
	{
		err = ISODisposeHandle( smrEntryH ); if (err) goto bail;
		smrEntryH = NULL;
	}	
	
	err = ISOEndMediaEdits( smrMedia ); if (err) goto bail;
	err = ISOGetMediaDuration( smrMedia, &mediaDuration ); if (err) goto bail;
	err = ISOInsertMediaIntoTrack( smrTrak, 
		0, //trackStartTime
		0, //mediaStartTime
		mediaDuration,
		1 //mediaRate
		); if (err) goto bail;
bail:
	return err;
}

static ISOErr createMIDITrack(ISOMovie& moov, ESData* midiES)
{
	ISOErr err;
	u32 timeScale = 10; //to be verified
	u64 mediaDuration;
	ISOTrack midiTrak;
	ISOMedia midiMedia;
	err = ISONewMovieTrackWithID( moov, ISONewTrackIsAudio, midiES->getESID(), &midiTrak ); if (err) goto bail;

	err = ISONewTrackMedia( midiTrak, &midiMedia, ISOAudioHandlerType, timeScale, NULL ); if (err) goto bail;
	err = ISOBeginMediaEdits( midiMedia ); if (err) goto bail;
	
	ISOHandle midiEntryH;
	ISOHandle midiDecoderSpecificInfoH;
	
	err = ISOSetMediaLanguage( midiMedia, "und" ); if (err) goto bail;
	
	err = ISONewHandle( 0, &midiDecoderSpecificInfoH ); if (err) goto bail;
	err = setDecoderSpecificInfo( midiDecoderSpecificInfoH, midiES->geDecoderInfo(), midiES->getDecoderInfoSize() ); if (err) goto bail;
	
	err = ISONewHandle( 0, &midiEntryH ); if (err) goto bail;
	err = MP4NewSampleDescription( midiTrak, midiEntryH,
		1,
		midiES->getOTI(),
		midiES->getStreamType(),
		midiES->getBufferSizeDB(),
		midiES->getMaxBitRate(),
		midiES->getAvgBitRate(),
		midiDecoderSpecificInfoH ); if (err) goto bail;
	/////////////////
	ISOHandle midiDataH;
	ISOHandle midiDurationH;
	ISOHandle midiSizeH;
	
	err = ISONewHandle( sizeof(u32), &midiDurationH ); if (err) goto bail;
	*((u32*) *midiDurationH) = 0;
	
	err = ISONewHandle( 0, &midiDataH ); if (err) goto bail;
	err = ISONewHandle( sizeof(u32), &midiSizeH ); if (err) goto bail;
	err = MP4SetHandleSize( midiDataH, 0 ); if (err) goto bail;
	
	err = ISOGetHandleSize( midiDataH, (u32*) *midiSizeH ); if (err) goto bail;
	err = ISOAddMediaSamples( midiMedia, midiDataH, 1, midiDurationH, midiSizeH, midiEntryH, NULL, NULL ); if (err) goto bail;
	if ( midiEntryH )
	{
		err = ISODisposeHandle( midiEntryH ); if (err) goto bail;
		midiEntryH = NULL;
	}
	
	err = ISOEndMediaEdits( midiMedia ); if (err) goto bail;
	err = ISOGetMediaDuration( midiMedia, &mediaDuration ); if (err) goto bail;
	err = ISOInsertMediaIntoTrack( midiTrak, 0, 0, mediaDuration, 1 ); if (err) goto bail;

bail:
	return err;
}

static ISOErr createBIFSTrack(ISOMovie& moov, unsigned char* bifsAccessUnit, int bifsAccessUnitSize)
{
	ISOErr err;
	u32 timeScale = 10; //to be verified
	u64 mediaDuration;
	ISOTrack bifsTrak;
	ISOMedia bifsMedia;
	u8* p;
	unsigned char bifsConfig[] = { 0x14, 0xA5, 0x60 };
	int bifsConfigSize = 3;
	u32 outReferenceIndex;

	err = ISONewMovieTrackWithID( moov, ISONewTrackIsAudio, 2, &bifsTrak ); if (err) goto bail;
	err = MP4AddTrackToMovieIOD( bifsTrak ); if (err) goto bail;
	err = ISOAddTrackReferenceWithID( bifsTrak, 10, MP4ODTrackReferenceAtomType, &outReferenceIndex); if (err) goto bail;
	
	err = ISONewTrackMedia( bifsTrak, &bifsMedia, ISOAudioHandlerType, timeScale, NULL ); if (err) goto bail;
	err = ISOBeginMediaEdits( bifsMedia ); if (err) goto bail;
	
	ISOHandle bifsEntryH;
	ISOHandle bifsDecoderSpecificInfoH;
	
	err = ISOSetMediaLanguage( bifsMedia, "und" ); if (err) goto bail;
	
	err = ISONewHandle( 0, &bifsDecoderSpecificInfoH ); if (err) goto bail;
	err = setDecoderSpecificInfo( bifsDecoderSpecificInfoH, (char*)bifsConfig , bifsConfigSize ); if (err) goto bail;
	
	err = ISONewHandle( 0, &bifsEntryH ); if (err) goto bail;
	err = MP4NewSampleDescription( bifsTrak, bifsEntryH,
		1,
		2, // OTI systems v2
		3, // ST BIFS
		1000,
		128000,
		128000,
		bifsDecoderSpecificInfoH ); if (err) goto bail;
	/////////////////
	ISOHandle bifsDataH;
	ISOHandle bifsDurationH;
	ISOHandle bifsSizeH;
	
	err = ISONewHandle( sizeof(u32), &bifsDurationH ); if (err) goto bail;
	*((u32*) *bifsDurationH) = 0;
	
	err = ISONewHandle( 0, &bifsDataH ); if (err) goto bail;
	err = ISONewHandle( sizeof(u32), &bifsSizeH ); if (err) goto bail;
	
	err = MP4SetHandleSize( bifsDataH, bifsAccessUnitSize ); if (err) goto bail;
	p = (u8*) *bifsDataH;
	memcpy ( p, bifsAccessUnit, bifsAccessUnitSize);

	err = ISOGetHandleSize( bifsDataH, (u32*) *bifsSizeH ); if (err) goto bail;
	err = ISOAddMediaSamples( bifsMedia, bifsDataH, 1, bifsDurationH, bifsSizeH, bifsEntryH, NULL, NULL ); if (err) goto bail;
	if ( bifsEntryH )
	{
		err = ISODisposeHandle( bifsEntryH ); if (err) goto bail;
		bifsEntryH = NULL;
	}
	
	err = ISOEndMediaEdits( bifsMedia ); if (err) goto bail;
	err = ISOGetMediaDuration( bifsMedia, &mediaDuration ); if (err) goto bail;
	err = ISOInsertMediaIntoTrack( bifsTrak, 0, 0, mediaDuration, 1 ); if (err) goto bail;

bail:
	return err;
}

int SMRFile::SaveToMP4(const wxString& FullPath, Coding_Type codingType)
{
	ISOErr err;
	ISOMovie moov;
	u8 OD_profileAndLevel;
	u8 scene_profileAndLevel;
	u8 audio_profileAndLevel;
	u8 visual_profileAndLevel;
	u8 graphics_profileAndLevel;
	u32 initialObjectDescriptorID;
	ODMap ods;

	err = ISONoErr;
	
	initialObjectDescriptorID = 1;
	OD_profileAndLevel = 0x1; /* Core */
	scene_profileAndLevel = 0xfe; /* none specified*/
	audio_profileAndLevel = 0xfe; /* none specified */
	visual_profileAndLevel = 0xff; /* none required */
	graphics_profileAndLevel = 0xff; /* none required */
	
	SMRAudioItem* midiItem=GetMIDIItem();
	if(midiItem!=NULL)
	{
		midiItem->SetID("od:20");
	}

	//////////////////////////////////////////////////////////////////////////
	// prepare DecoderInfo and access unit
	wxMemoryOutputStream *decoderConfigMemStream=new wxMemoryOutputStream();
	wxMemoryOutputStream *accessUnitMemStream=new wxMemoryOutputStream();

	EncodeSMXF(decoderConfigMemStream, accessUnitMemStream, codingType, midiItem);

	int smrDecoderInfoSize=decoderConfigMemStream->GetSize();
	char *smrDecoderInfo=new char[smrDecoderInfoSize];
	int len=decoderConfigMemStream->CopyTo(smrDecoderInfo,smrDecoderInfoSize);

	int smrAccessUnitSize=accessUnitMemStream->GetSize();
	char *smrAccessUnit=new char[smrAccessUnitSize];
	accessUnitMemStream->CopyTo(smrAccessUnit,smrAccessUnitSize);
	delete accessUnitMemStream;
	delete decoderConfigMemStream;

	ESData* smrES=new ESData(100, //ESID
		0x40, //OTI
		0x5,  //stream type
		smrAccessUnitSize, //bufferSize
		128000, //maxBitRate
		128000, //avgBitRate
		smrDecoderInfo,
		smrDecoderInfoSize);
	ods.addOD(10, smrES);
	/////////////////////////////////////////////////////////////////////////
    // if the MIDI file is present prepares the Decoder info for SA
	int midiDecoderInfoSize=0;
	char *midiDecoderInfo=NULL;
	ESData* midiES=NULL;
	if(midiItem)
	{
		wxMemoryOutputStream *decoderConfigMemStream=new wxMemoryOutputStream();
		EncodeMIDI(decoderConfigMemStream, midiItem);
		midiDecoderInfoSize=decoderConfigMemStream->GetSize();
		midiDecoderInfo=new char[midiDecoderInfoSize];
		int len=decoderConfigMemStream->CopyTo(midiDecoderInfo,midiDecoderInfoSize);
		
		midiES = new ESData(101, //ESID
			0x40, //OTI mpeg-4 audio
			0x5,  //stream type audio
			0, //bufferSize
			128000, //maxBitRate
			128000, //avgBitRate
			midiDecoderInfo,
			midiDecoderInfoSize);
		ods.addOD(20, midiES);
	}
	///////////////////////////////////////////////////////////////////////
	wxMemoryOutputStream *odMemStream=new wxMemoryOutputStream();
	ODStreamEncoder encoder(odMemStream);
	encoder.writeDownStream(&ods);

	int odAccessUnitSize=odMemStream->GetSize();
	char *odAccessUnit=new char[odAccessUnitSize];
	odMemStream->CopyTo(odAccessUnit,odAccessUnitSize);
	delete odMemStream;
	//////////////////////////////////////////////////////////////////////
	unsigned char bifsAccessUnit[] = { 0xc0,0x10,0x12,0x60,0x40,0x80,0x7e,0x99,0x99,0x9a,0x7e,0x99,0x99,0x9a,0x7e,0x99,0x99,0x9a,0x7a,0x78,0x44,0x48,0x00,0x00,0x44,0x16,0x00,0x00,0xc9,0x81,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0xea,0x20,0x45,0x82,0x63,0x7b,0x0b,0x23,0x4b,0x73,0x39,0x03,0x9b,0x1b,0x7b,0x93,0x29,0x71,0x71,0x75,0x20,0x10,0xea,0x0b,0x93,0x4b,0x0b,0x64,0x48,0x79,0x35,0x25,0x11,0x11,0x31,0x16,0x54,0x1a,0x00,0x00,0x0e,0x00,0x00,0x00,0x20,0x60,0x01,0xaa,0x05,0x50,0x88,0x8b,0x00,0x00,0x88,0x27,0x00,0x00,0xab,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0xdf,0xf8,0x00,0x00,0x00,0x00,0x00,0x00,0x10,0x35,0x04,0xf0 };
	int bifsAccessUnitSize = sizeof(bifsAccessUnit);

	//create MP4Movie
	err = MP4NewMovie( &moov,
		initialObjectDescriptorID, 
		OD_profileAndLevel,
		scene_profileAndLevel,
		audio_profileAndLevel,
		visual_profileAndLevel,
		graphics_profileAndLevel ); if (err) goto bail;

	err = createODTrack(moov, odAccessUnit, odAccessUnitSize); if (err) goto bail;
	err = createBIFSTrack(moov, bifsAccessUnit, bifsAccessUnitSize); if (err) goto bail;
	err = createSMRTrack(moov, smrES, midiES, smrAccessUnit, smrAccessUnitSize); if(err) goto bail;
	if(midiItem)
	{
		err = createMIDITrack(moov, midiES); if(err) goto bail;
	}
	err = ISOWriteMovieToFile( moov, FullPath.c_str() ); if (err) goto bail;
	err = MP4DisposeMovie( moov ); if (err) goto bail;
bail:
	delete smrAccessUnit;
	delete smrDecoderInfo;
	delete midiDecoderInfo;
	delete odAccessUnit;
	delete smrES;
	delete midiES;
	return err;
}

int SMRFile::DecodeBITS(const wxString srcFullPath)
{
	wxFileInputStream* sourceFile = new wxFileInputStream(srcFullPath);
	
	if (!sourceFile->Ok())
	{	
		delete sourceFile;
		return -1;
	}

	//creazione folder .xsm
	unsigned int dirCount = 1;
	wxString path = srcFullPath.BeforeLast('.');
	while (::wxDirExists(path + DEFAULT_EXT))
		path = srcFullPath.BeforeLast('.') + wxString::Format( "%u",dirCount++ );
	//

	int ret = DecodeBITS(NULL, sourceFile, path+DEFAULT_EXT);

	delete sourceFile;

	return ret;
}

int SMRFile::DecodeBITS(wxInputStream *decoderConfig, wxInputStream* source, wxString dstPath)
{
	unsigned int partsNumber;

	if(! ::wxDirExists(dstPath))
		::wxMkdir(dstPath);

	BitStreamDecoder decoder(decoderConfig ? decoderConfig : source);

	partsNumber = decoder.getPartsNumber();
	if (!partsNumber)
	{	
		return -1;
	}

	if(decoder.readMoreData())
		DecodeBITS(decoder, NULL, dstPath);

	if(decoderConfig)
	{
		decoder.setIStream(source);

		return DecodeBITS(decoder, NULL, dstPath);
	}
	return 0;
}

int SMRFile::DecodeBITS(BitStreamDecoder& decoder, wxInputStream* midiStream, wxString dstPath)
{
	wxString fullFileName, fullFileNameTxt;
	wxString fname = dstPath.AfterLast('\\');
	wxString part_str = "",part_code = "";
	wxString wdfSymItem="", wdfLyrItem="", wdfEttItem="";
	wxString wdfSymMillaItem, wdfSymFontItem, wdfAudioItem;

	wxFileOutputStream*		outFile;
	unsigned int			lyrId[256];

	for (int i=0; i<256; i++)
		lyrId[i] = 0;

	Chunk_Type				chunkType;
	unsigned int			chunkId=0;
	int						err=0;
	unsigned int			syncCount=0;
	unsigned int			fontCount=0;

	while(decoder.readMoreData())
	{
		err = decoder.readNextChunk();
		if(err == -1)
			return -1;

		chunkId = decoder.getChunkId();
		if (chunkId)
			part_str = wxString::Format( "%u",chunkId );

		chunkType = decoder.getChunkType();
		switch (chunkType)
		{
		case MAIN:
			fullFileName = fname + ".xml";
			wdfSymItem += "<Wdfitem><Filename>"+ fullFileName + "</Filename><WDFCID>000000000000</WDFCID><Textual_description>Main Score</Textual_description></Wdfitem>\r\n";
			break;
		case PART:
			fullFileName = fname + ".xml";
			part_code = wxString::Format( "%012u", chunkId );
			wdfSymItem += "<Wdfitem><Filename>" + part_str + "."  + fullFileName + "</Filename><WDFCID>"+part_code+"</WDFCID><Textual_description>part "+part_str+"</Textual_description></Wdfitem>\r\n";
			part_str += ".";
			break;
		case LYRC:
			fullFileName = "lyric." + wxString::Format( "%u",++(lyrId[chunkId]) )  + ".xml";
			part_code = wxString::Format( "%04u%08u", chunkId, lyrId[chunkId]);
			part_str += ".";
			break;
		case SYNC:
			part_str = wxString::Format( "%u",++(syncCount) );
			part_code = wxString::Format( "%012u",syncCount );
			fullFileName = "performance" +part_str +".bin";
			fullFileNameTxt = "performance" +part_str +".ett";
			wdfEttItem += "<Wdfitem><Filename>"+ fullFileNameTxt + 
				"</Filename><WDFCID>"+part_code+"</WDFCID><Textual_description>performance "+part_str+"</Textual_description></Wdfitem>\r\n";
			part_str ="";
			break;
		case SMFL:
			part_str="";
			fullFileName="smfl.xml";
			wdfSymMillaItem +="<Wdfitem><Filename>"+fullFileName+"</Filename><WDFCID>000008000008</WDFCID><Textual_description>SMFL</Textual_description></Wdfitem>\r\n";
			break;
		case FONT:
			part_str="";
			fontCount++;
			fullFileName=wxString::Format("font%d.ttf", fontCount);
			wdfSymFontItem +="<Wdfitem><Filename>"+fullFileName+"</Filename><WDFCID>000006000008</WDFCID><Textual_description>"+fullFileName+"</Textual_description></Wdfitem>\r\n";
			break;
		}
		
		// 
		//BinIStream = (decoder.getChunkInputSource())->makeStream(); //this crashes in IM1...
		wxString destFileName = dstPath + "\\" + part_str + fullFileName;
		
		outFile = new wxFileOutputStream(destFileName);

		wxInputStream *source=decoder.getChunkInputStream();
		if(source==NULL)
			return -1;
		
		outFile->Write(*source);
		//while (count = decoder.readChunkData(buffer, 10*1024))
		//	outFile->Write(&buffer, count);

		delete outFile;
		delete source;

		if(chunkType==SYNC)
		{
			wxFileInputStream binSyncInfo(destFileName);
			wxFileOutputStream txtSyncInfo(dstPath + "\\" + part_str + fullFileNameTxt);
			decoder.decodeSyncInfo(&binSyncInfo, &txtSyncInfo);
		}
		else if(chunkType==LYRC)
		{
			//read ID and Language from the XML file.
			WDFDOMParser parser;

			parser.setValidationSchema("urn:mpeg:mpeg-4:schema:smr:smxf-lyric:2007 smxf_lyric.xsd");	
			parser.setXMLFile((char*)destFileName.c_str());
			parser.Parse();
			
			DOM_Document doc;
			
			doc = parser.GetWDFParser()->getDocument ();
			
			DOM_Node header=doc.getFirstChild();
			if(!header.isNull())
			{
				DOM_Node child = header.getFirstChild();
				if(child==0) // Se c'e' il DOCTYPE lo si salta
				{
					header=header.getNextSibling();
					child = header.getFirstChild();
				}
				wxString id, language=wxString::Format("lg%d",lyrId[chunkId]);
				
				while( child != 0)
				{
					if(child.getLocalName().equals("ID"))
					{
						DOM_Node IDNode = child.getFirstChild();
						if(!IDNode.isNull())
							id=IDNode.getNodeValue().transcode();
					}
					else if(child.getLocalName().equals("language"))
					{
						DOM_Node languageNode = child.getFirstChild();
						if(!languageNode.isNull())
							language=languageNode.getNodeValue().transcode();
					}
					child = child.getNextSibling();
				}
				wdfLyrItem += "<Wdfitem><Filename>" + part_str + fullFileName + "</Filename>"
					"<WDFCID>"+id+"</WDFCID>"
					"<Textual_description>"+language+"</Textual_description>"
					"</Wdfitem>\r\n";
			}
		}
	}
	if(midiStream)
	{
		fullFileName="midi.mid";
		wxString destFileName = dstPath + "\\" + fullFileName;
		outFile = new wxFileOutputStream(destFileName);
		outFile->Write(*midiStream);
		delete outFile;
		wdfAudioItem +="<Wdfitem><Filename>"+fullFileName+"</Filename><WDFCID>000007000008</WDFCID><Textual_description>"+fullFileName+"</Textual_description></Wdfitem>\r\n";
	}
	
	//WDF WRITING
	wxString wdf="";

	wdf+=	"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n";
	wdf+=	"<wdfheader WDF_CONDITION=\"FULL\">\r\n";
	wdf+=	"<Identification/>\r\n<Classification/>\r\n<Protection/>\r\n";
	wdf+=	"<Symbolic>\r\n"+wdfSymItem+"</Symbolic>\r\n";
	wdf+=	"<Symbolic_Milla>\r\n"+wdfSymMillaItem+"</Symbolic_Milla>\r\n";
	wdf+=	"<Symbolic_Fonts>\r\n"+wdfSymFontItem+"</Symbolic_Fonts>\r\n";
	wdf+=	"<Lyric>\r\n"+wdfLyrItem+"</Lyric>\r\n";
	wdf+=	"<Audio>\r\n"+wdfAudioItem+"</Audio>\r\n";
	wdf+=	"<Performance>\r\n"+wdfEttItem+"</Performance>\r\n";
	wdf+=	"</wdfheader>";
	

	wxFileOutputStream xsmFile(dstPath + "\\" + fname + DEFAULT_EXT);
	xsmFile.Write(wdf.c_str(), wdf.length());

	if (xsmFile.LastWrite() != wdf.length())
	{	
		return -1;
	}
	//
	
	if (err < 0)
	{	
		return -1;
	}

	return 0;
}

int SMRFile::LoadFromMP4(wxString mp4file)
{
	char *p;
	MP4Err err;
	MP4Movie moov;
	MP4Track trak;
	MP4Media media;
	MP4TrackReader reader;
	MP4Handle initialObjectDescriptorH;
	MP4Handle decoderConfigH;
	MP4Handle decoderSpecificInfoH;
	MP4Handle sampleH;
	u32 handlerType;
	u32 handleSize;
	u32 oidSize;
 	u32 objectTypeIndication;
	u32 streamType;
	u32 decoderBufferSize;
	u32 trackCount=0;
	u32 ntrack;

	BitStreamDecoder* smrDecoder = NULL;
	int nparts=-1;
	int width=-1;
	int height=-1;
	Coding_Type ct;
	wxInputStream* midiStream=NULL;
	wxMemoryInputStream* decoderInfoMStream=NULL;
	wxMemoryInputStream* accessUnitMStream=NULL;
	char* smrAccessUnit=NULL;
	wxString wdfPath = wxString(getenv("TEMP"))+"\\"+mp4file.AfterLast('\\');
	bool smrFound=false;
	ODMap odMap;

	err = MP4NoErr;
	err = MP4OpenMovieFile( &moov, mp4file, MP4OpenMovieDebug ); if (err) goto bail;

 	err = MP4NewHandle( 0, &initialObjectDescriptorH ); if (err) goto bail;
	err = MP4GetMovieInitialObjectDescriptor( moov, initialObjectDescriptorH ); if (err) goto bail;
	err = MP4GetHandleSize( initialObjectDescriptorH, &oidSize ); if (err) goto bail;
	p = *initialObjectDescriptorH;
	accessUnitMStream=new wxMemoryInputStream(p,oidSize);
	{
		ODStreamDecoder ods(accessUnitMStream);
		ods.readDescriptor(odMap);
	}
	err = MP4DisposeHandle( initialObjectDescriptorH);
	delete accessUnitMStream;

	err = MP4GetMovieTrackCount( moov, &trackCount); if (err) goto bail;
	for(ntrack=1; ntrack<=trackCount; ntrack++)
	{
		u32 refCount=0;
		err = MP4GetMovieIndTrack( moov, ntrack, &trak ); if (err) goto bail;

		err = MP4GetTrackMedia( trak, &media ); if (err) goto bail;
		err = MP4GetMediaHandlerDescription( media, &handlerType, NULL ); if (err) goto bail;
		err = MP4CreateTrackReader( trak, &reader ); if (err) goto bail;
		err = MP4NewHandle( 0, &decoderConfigH ); if (err) goto bail;
		err = MP4TrackReaderGetCurrentDecoderConfig( reader, decoderConfigH ); if (err) goto bail;
		err = MP4NewHandle( 0, &decoderSpecificInfoH ); if (err) goto bail;
		err = MP4GetMediaDecoderType( media, 1, 
			&objectTypeIndication, 
			&streamType, 
			&decoderBufferSize, 
			decoderSpecificInfoH ); if (err) goto bail;

		if(streamType == 5 && ( objectTypeIndication == 0x40 || objectTypeIndication == 0xc5))
		{			
			err = MP4GetHandleSize( decoderSpecificInfoH, &handleSize ); if (err) goto bail;
			
			p = *decoderSpecificInfoH;						
			decoderInfoMStream = new wxMemoryInputStream(p,handleSize);
			
			smrDecoder=new BitStreamDecoder(decoderInfoMStream);
			if(!smrFound && (smrDecoder->getAudioObjectType()==40 || 
				smrDecoder->getAudioObjectType()==41))
			{
				if(! ::wxDirExists(wdfPath))
					::wxMkdir(wdfPath);
				nparts=smrDecoder->getPartsNumber();
				width=smrDecoder->getPictureWidth();
				height=smrDecoder->getPictureHeight();
				ct=smrDecoder->getCodingType();
				wxString midiUrl=smrDecoder->getMidiUrl();

				if(midiUrl!="")
				{
					//from the object id get the ESID (=track ID)
				}
				
				if(smrDecoder->readMoreData())
					DecodeBITS(*smrDecoder, NULL, wdfPath);
				
				delete decoderInfoMStream;
				
				err = MP4NewHandle( 0, &sampleH ); if (err) goto bail;	
				/* play every frame */
				for (;;)
				{
					u32 unitSize;
					u32 cts;
					u32 dts;
					u32 sampleFlags;
					
					err = MP4TrackReaderGetNextAccessUnit( reader, sampleH, &unitSize,&sampleFlags, &cts, &dts );
					if ( err )
					{
						if ( err == MP4EOF )
							err = MP4NoErr;
						break;
					}
					p = *sampleH;
					
					accessUnitMStream=new wxMemoryInputStream(p,unitSize);
					break;
				}
				smrFound=true;
			}
			else if(smrDecoder->getAudioObjectType()==13 ||
				smrDecoder->getAudioObjectType()==15)
			{
				midiStream=smrDecoder->getMidiInputStream();
			}
			else
				delete smrDecoder;
		}
		else if(streamType == 1 && ( objectTypeIndication == 0x01 || objectTypeIndication == 0x00 || objectTypeIndication == 0x02)) //OD stream
		{
				err = MP4NewHandle( 0, &sampleH ); if (err) goto bail;	
				/* play every frame */
				for (;;)
				{
					u32 unitSize;
					u32 cts;
					u32 dts;
					u32 sampleFlags;
					
					err = MP4TrackReaderGetNextAccessUnit( reader, sampleH, &unitSize,&sampleFlags, &cts, &dts );
					if ( err )
					{
						if ( err == MP4EOF )
							err = MP4NoErr;
						break;
					}
					p = *sampleH;
					
					wxMemoryInputStream* accessUnitMStream=new wxMemoryInputStream(p,unitSize);
					ODStreamDecoder ods(accessUnitMStream);
					ods.readODStream(odMap);
				}
		}
		else if(streamType == 3 && ( objectTypeIndication == 0x02))
		{			
			err = MP4GetHandleSize( decoderSpecificInfoH, &handleSize ); if (err) goto bail;
			
			p = *decoderSpecificInfoH;						
			decoderInfoMStream = new wxMemoryInputStream(p,handleSize);
			
			BIFSStreamDecoder BIFS(decoderInfoMStream);
			delete decoderInfoMStream;
				
			err = MP4NewHandle( 0, &sampleH ); if (err) goto bail;	
			/* play every frame */
			for (;;)
			{
				u32 unitSize;
				u32 cts;
				u32 dts;
				u32 sampleFlags;
				
				err = MP4TrackReaderGetNextAccessUnit( reader, sampleH, &unitSize,&sampleFlags, &cts, &dts );
				if ( err )
				{
					if ( err == MP4EOF )
						err = MP4NoErr;
					break;
				}
				p = *sampleH;

				accessUnitMStream=new wxMemoryInputStream(p,unitSize);
				BIFS.setIStream(accessUnitMStream);
				BIFS.decode();
				delete accessUnitMStream;
				break;
			}
		}
	}
	if(smrFound)
	{
		smrDecoder->setIStream(accessUnitMStream);
		DecodeBITS(*smrDecoder, midiStream, wdfPath);
		
		Load(wdfPath);
		delete accessUnitMStream;
		delete smrDecoder;
	}
	else
		wxMessageBox("No SMR stream found in mp4 file!");
bail:
	return err;
}

void SMRFile::SaveSMRStructure(wxString xmlfile)
{
	ofstream xml(xmlfile.c_str());

	xml << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
		   "<smr-structure>\n";
	wxSMRItemListNode *node = m_itemsList.GetFirst();

    while (node)
	{
		SMRItem *item = node->GetData();
		wxString FileName = item->GetFilename();
		int partNum = atoi((FileName.BeforeFirst('.')).c_str());

		switch (item->GetType())
		{
		case SMR_MainScore:
			if(!m_fromMIDI)
				xml << "  <mainscore>" << FileName.c_str() << "</mainscore>\n";
			break;
		case SMR_Part:
			if(!m_fromMIDI)
				xml << "  <part>" << FileName.c_str() << "</part>\n";
			break;
		case SMR_Lyrics:
			if(!m_fromMIDI)
				xml << "  <lyrics ID=\"" << item->GetID().c_str() << "\">" << FileName.c_str() << "</lyrics>\n";
			break;
		case SMR_SynchInfo:
			xml << "  <sync-info>" << FileName.c_str() << "</sync-info>\n";
			break;
		case SMR_SMFL:
			xml << "  <smfl>" << FileName.c_str() << "</smfl>\n";
			break;
		case SMR_Font:
			xml << "  <font>" << FileName.c_str() << "</font>\n";
			break;
		case SMR_Audio:
			xml << "  <audio>" << FileName.c_str() << "</audio>\n";
			break;
		} 

		node = node->GetNext();
	}
	
	xml << "</smr-structure>\n";
}

void SMRFile::GetItems(SMR_ItemType type, SMRItemList& list)
{
	wxSMRItemListNode *node = m_itemsList.GetFirst();
	
	list.Clear();
    while (node)
	{
		SMRItem *item = node->GetData();
		if (item->GetType()==type || (type==SMR_Symbolic && (item->GetType()==SMR_MainScore || item->GetType()==SMR_Part)))
			list.Append(item);
		node = node->GetNext();
	}
}
int SMRFile::CountItems(SMR_ItemType type)
{
	wxSMRItemListNode *node = m_itemsList.GetFirst();
	int count=0;
	
    while (node)
	{
		SMRItem *item = node->GetData();
		if (item->GetType()==type || (type==SMR_Symbolic && (item->GetType()==SMR_MainScore || item->GetType()==SMR_Part)))
			count++;
		node = node->GetNext();
	}
	return count;
}
SMRItem *SMRFile::GetSymItem(int nScore)
{
	wxSMRItemListNode *node = m_itemsList.GetFirst();
	int count=0;
	
    while (node)
	{
		SMRItem *item = node->GetData();
		if (item->GetType()==SMR_MainScore && nScore==0)
			return item;
		else if(item->GetType()==SMR_Part)
		{
			count++;
			if(count==nScore)
				return item;
		}
		node = node->GetNext();
	}
	return NULL;
}
SMRItem* SMRFile::GetItemByID(const wxString& id)
{
	wxSMRItemListNode *node = m_itemsList.GetFirst();

	while (node)
	{
		SMRItem *item = node->GetData();
		if (item->GetID()==id)
			return item;
		node = node->GetNext();
	}
	return NULL;
}
SMRItem* SMRFile::GetItemByFilename(const wxString& filename)
{
	wxSMRItemListNode *node = m_itemsList.GetFirst();

	while (node)
	{
		SMRItem *item = node->GetData();
		if (item->GetFilename()==filename)
			return item;
		node = node->GetNext();
	}
	return NULL;
}

SMRAudioItem* SMRFile::GetMIDIItem()
{
	wxSMRItemListNode *node = m_itemsList.GetFirst();

	while (node)
	{
		SMRItem *item = node->GetData();
		if (item->GetType()==SMR_Audio && item->GetFilename().AfterLast('.').IsSameAs("mid",FALSE))
			return (SMRAudioItem*)item;
		node = node->GetNext();
	}
	return NULL;
}

void SMRFile::GetScores(int nScore,Partitura*& score)
{
	score=NULL;
	if (m_mainScore==NULL)
	{
		MaMass *mm=new MaMass;
		m_mainScore=mm;
		
		m_mainScore->Free();
		m_mainScore->Init();
	}
	
	if(nScore==0)
	{
		MaMass *mm=new MaMass;
		score=mm;
	}
	else
	{
		LoMas *lm=new LoMas(true); //empty score
		score=lm;
	}
	
	score->SetFrom(m_mainScore);
}
bool SMRFile::OpenSymbolicScore(int score,short current_measure)
{
	InitMusicEditors(INITME_BEFORE_LOAD);	
	Partitura *mainScore=NULL;
	bool ok;
	
	GetScores(score,mainScore);
	SetMusicEditorSMRF(this);
	
	ok=OpenMusicEditor(this,mainScore,score);
	
	if(ok)
	{   
		m_musicEditorIsOpen=true;
		if(current_measure!=0) mainScore->GoBatt(current_measure);
		return true;
	}
	return false;
}
void SMRFile::CloseMusicEditors()
{
	if(m_musicEditorIsOpen)
	{
		CloseMusicEditor();
		
		m_musicEditorIsOpen=false;
	}
}
void SMRFile::ExchangeParts(int pos, int posRef)
{
}
void SMRFile::HidePart(int pos)
{
}
void SMRFile::ShowPart(int pos)
{
}
SMRPartItem* SMRFile::PartAdded(int pos, const wxString& name)
{
	int count=0;
	wxString partFile;
	int partID=0;
	wxSMRItemListNode *node = m_itemsList.GetFirst();

	if(m_mainScore==NULL)
		return NULL;
	
	partID=m_mainScore->GetSpartNum(pos-1)->GetNumericCode();
	partFile.sprintf("%d.%s.xml",partID,m_title.c_str());
	SMRPartItem *part = new SMRPartItem(this,name,partFile);
	//item->SetPartitura(m_mainScore);
	
    while (node)
	{
		SMRItem *item = node->GetData();
		if (item->GetType()==SMR_Part)
		{
			count++;
			if(count==pos)
			{
				m_itemsList.Insert(node, part);
				return part;
			}
		}
		
		node = node->GetNext();
	}	
	m_itemsList.Append(part);
	return part;
}
bool SMRFile::PartDeleted(int pos)
{
	wxSMRItemListNode *node = m_itemsList.GetFirst();
	int count=0;
    while (node)
	{
		SMRItem *item = node->GetData();
		if (item->GetType()==SMR_Part)
		{
			count++;
			if(count==pos)
			{
				delete item;
				m_itemsList.DeleteNode(node);
				return true;
			}
		}
		
		node = node->GetNext();
	}
	return false;
}
SMRSynchInfoItem* SMRFile::AddSynchInfo(bool symbolic, char* synchFile, char* audioFile)
{
	// looking for the synch file if already present
	bool found=false;
	wxSMRItemListNode *node = m_itemsList.GetFirst();
    while (node)
	{
		SMRItem *item = node->GetData();
		
		if(item->GetType()==SMR_SynchInfo && item->GetFilename()==synchFile)
		{
			found=true;
			break;
		}
		node = node->GetNext();
	}
	// se non c'e' lo aggiungo, e lo metto nel tree.
	if(!found)
	{
		SMRSynchInfoItem *synchInfo=new SMRSynchInfoItem(this,"Synch. Info", synchFile);
		//synchInfo->SetAudioFile(audioFile);
				
		m_itemsList.Append(synchInfo);
		return synchInfo;
	}
	return NULL;
}
SMRLyricsItem* SMRFile::AddLyrics( const wxString& lyrFile,int partID, const wxString& language, const wxString& desc)
{
	SMRLyricsItem *lyr=new SMRLyricsItem(this,language,desc,lyrFile);
	
	//ID
	lyr->SetID(wxString::Format("LYR-%02d",++m_lastLyrID));
	lyr->SetPartID(partID);
	
	m_itemsList.Append(lyr);
	return lyr;
}
bool SMRFile::AddSMFL(wxString fullFilename)
{
	if(GetTitle()=="")
	{
		wxMessageBox("ERROR: save the object before adding a SMFL file");
		return false;
	}
	wxString file=fullFilename.AfterLast('\\');
	SMRItem *item=NULL;
	// we see if it is already present
	bool found=false;
	wxSMRItemListNode *node = m_itemsList.GetFirst();
    while (node)
	{
		item = node->GetData();
		if(item->GetFilename()==file)
		{
			found=true;
			break;
		}
		node = node->GetNext();
	}
	// if not present we add it
	if(!found)
	{
		SMRSMFLItem *item=new SMRSMFLItem(this,"SMFL file",file);
				
		m_itemsList.Append(item);

	}
	else
		item->SetFilename(file);
	wxString destFileName=GetPath()+"\\"+file;
	wxCopyFile(fullFilename,destFileName);
	return true;
}
bool SMRFile::AddFont(wxString fullFilename)
{
	if(GetTitle()=="")
	{
		wxMessageBox("ERROR: save the object before adding a font file");
		return false;
	}
	wxString file=fullFilename.AfterLast('\\');

	// we see if it is already present
	bool found=false;
	wxSMRItemListNode *node = m_itemsList.GetFirst();
    while (node)
	{
		SMRItem *item = node->GetData();
		if(item->GetFilename()==file)
		{
			found=true;
			break;
		}
		node = node->GetNext();
	}
	// if not present we add it
	if(!found)
	{
		SMRFontItem *item=new SMRFontItem(this,"Font file",file);
				
		m_itemsList.Append(item);

		wxString destFileName=GetPath()+"\\"+file;

		wxCopyFile(fullFilename,destFileName);
	}
	else
		return false;
	return true;
}
bool SMRFile::AddImage(wxString description)
{
	return false;
}
bool SMRFile::AddAudio(wxString fullFilename)
{
	if(GetTitle()=="")
	{
		wxMessageBox("ERROR: save the object before adding an audio file");
		return false;
	}
	wxString file=fullFilename.AfterLast('\\');

	// we see if it is already present
	bool found=false;
	wxSMRItemListNode *node = m_itemsList.GetFirst();
    while (node)
	{
		SMRItem *item = node->GetData();
		if(item->GetFilename()==file)
		{
			found=true;
			break;
		}
		node = node->GetNext();
	}
	// if not present we add it
	if(!found)
	{
		SMRAudioItem *item=new SMRAudioItem(this,"Audio",file);
				
		m_itemsList.Append(item);

		wxString destFileName=GetPath()+"\\"+file;

		wxCopyFile(fullFilename,destFileName);
	}
	else
		return false;
	return true;
}
bool SMRFile::AddMIDI(wxString fullFilename)
{
	if(GetTitle()=="")
	{
		wxMessageBox("ERROR: save the object before adding a midi file");
		return false;
	}
	wxString file=fullFilename.AfterLast('\\');
	SMRItem *item=NULL;
	// we see if it is already present
	bool found=false;
	wxSMRItemListNode *node = m_itemsList.GetFirst();
    while (node)
	{
		item = node->GetData();
		if(item->GetFilename()==file)
		{
			found=true;
			break;
		}
		node = node->GetNext();
	}
	// if not present we add it
	if(!found)
	{
		SMRAudioItem *item=new SMRAudioItem(this,"MIDI",file);
				
		m_itemsList.Append(item);
	}
	else
		item->SetFilename(file);

	wxString destFileName=GetPath()+"\\"+file;
	wxCopyFile(fullFilename,destFileName);
	m_mainScore->ImportMidi(fullFilename.c_str(),0);
	m_fromMIDI = true;
	return true;
}
wxString SMRFile::GetFontPath(const char* fontName,bool& removeAfterUse)
{
	wxString fontFile;
	removeAfterUse=false;

	fontFile=GetPath()+PATH_SEP+fontName;
	if(GetItemByFilename(fontName)!=NULL && wxFileExists(fontFile))
		return fontFile;
		
	fontFile=wxString(smrgetenv("SMR_FONT_PATH"))+fontName;
	if(wxFileExists(fontFile))
		return fontFile;
	return "";
}
wxString SMRFile::GetTmpPath()
{
	return "";
}
void SMRFile::CleanTmpPath()
{
}

wxMEScoreFrame* SMRFile::NewWxMEScoreFrame(wxWindow* parent,const wxString& tit,const wxPoint& pos, const wxSize& siz)
{
	return new wxMEScoreFrame(parent, tit, pos, siz);
}

void SMRFile::LoadScore()
{	
	wxString midiFile;
	UpdateSMFL();

	if(m_mainScore==NULL)
		m_mainScore=new MaMass;

	int k=0;
	SMRMainScoreItem* ms=NULL;

	wxProgressDialog *loadSym=NULL;
	if(SMRApp::GetApp().IsInteractive())
	{
		loadSym =new wxProgressDialog("Symbolic","Loading part 1",CountItems(SMR_Symbolic));
	}

	wxSMRItemListNode *node = m_itemsList.GetFirst();
	while (node)
	{
		SMRItem *item = node->GetData();
		if(item->GetType()==SMR_MainScore)
		{
			ms=(SMRMainScoreItem*)item;
			m_mainScore->FullFree(false); 
		}
		else if(item->GetType()==SMR_Part)
		{
			SMRPartItem* part = (SMRPartItem*)item;
			k++;
			part->LoadXML();
			if(loadSym)
				loadSym->Update(k,wxString::Format("Loading part %d",k+1));
		}
		else if(item->GetType()==SMR_Lyrics)
		{
			wxString lyrFile=GetPath()+PATH_SEP+item->GetFilename();
			WDFComponentParser lyr((char*)lyrFile.c_str());
			lyr.ReadLyrics((SMRLyricsItem*)item);
		}
		else if(item->GetType()==SMR_Audio && item->GetFilename().AfterLast('.')=="mid") //is MIDI
		{
			midiFile=GetPath()+PATH_SEP+item->GetFilename();
		}
		node = node->GetNext();
	}
	if(ms!=NULL)
	{
		if(loadSym)
			loadSym->Update(k+1,"Loading main score");
		ms->LoadXML();
	}

	if(loadSym)
		loadSym->Destroy();

	if(midiFile!="")
	{
		m_mainScore->ImportMidi(midiFile,0);
		int nSpa=m_mainScore->GetNumSpa();
		m_itemsList.Append(new SMRMainScoreItem(this));
		for(int i=0; i<nSpa; i++)
		{
			m_itemsList.Append(new SMRPartItem(this));
		}

		m_fromMIDI = true;
	}
	// carica i font dall'oggetto se ci sono 
	//Scr->UpdateFontFon(this);
	m_mainScore->GoTop();
}
void SMRFile::UpdateSMFL()
{
	SMRItemList smfl;
	GetItems(SMR_SMFL,smfl);

	if(smfl.GetCount()>0)
	{
		wxString smflFile=smfl[0]->GetFilename();
		SMFLManager::Get()->LoadCustom(GetPath()+PATH_SEP+smflFile,this);
	}
	else
		SMFLManager::Get()->LoadDefault();
}
InputSource* SMRFile::CreateFileInputSource(const wxString& file)
{
	wchar_t wfile[255];
	wxConvUTF8.MB2WC(wfile,file,255);
	return new LocalFileInputSource(wfile);
}