/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#ifndef SMR_HPP
#define SMR_HPP

#include "wx/string.h"

#include "smr/smritem.hpp"
#include "bit/def_bit.hpp"

class Partitura;
class Spartito;
class BitStreamDecoder;

class wxMEScoreFrame;
class AudioSyncFrame;
class wxWindow;
class wxPoint;
class wxSize;
class wxOutputStream;
class wxInputStream;

namespace xercesc_2_6
{
	class InputSource;
}

class SMRFile
{
public:
	SMRFile();
	~SMRFile();

	int New(const wxString& title);

	// removes all the content
	void Clear();

	int Load(const wxString &FullPath);
	int Save();
	int SaveAs(const wxString& title, const wxString& dstPath="");
	
	int LoadFromMP4(wxString mp4file);
	int DecodeBITS(const wxString srcFullPath);
	int DecodeBITS(wxInputStream *decoderConfig, wxInputStream* source, wxString dstPath);
	int DecodeBITS(BitStreamDecoder& decoder, wxInputStream* midiStream, wxString dstPath);

	int SaveToMP4(const wxString& FullPath, Coding_Type codingType);
	int EncodeSMXF(const wxString FullPath, Coding_Type codingType);
	int EncodeSMXF(wxOutputStream* configFile, wxOutputStream* smrFile, Coding_Type codingType, SMRAudioItem* midiItem=NULL);
	int EncodeMIDI(wxOutputStream* configFile, SMRAudioItem* midiItem);

	void SaveSMRStructure(wxString xmlfile);

	Partitura* GetMainScore();
//	Partitura* GetPartitura() { return getMainScore(); }
	
	void SetDestroyMS(bool destroyMS);

	int GetPartCount();

	Spartito* GetPart(int partNumber);

	const char* GetPartName(int partNumber);

	int GetMeasureCount();

	int GetVersion() { return m_version; };

	int GetPictureWidth() { return m_pictureWidth; }
	void GetPictureWidth(int width) { m_pictureWidth=width; }

	int GetPictureHeight() { return m_pictureHeight; }
	void GetPictureHeight(int height) { m_pictureHeight=height; }

	Coding_Type GetCodingType() { return m_codingType; }
	void SetCodingType(Coding_Type ct) { m_codingType=ct; }

	void GetItems(SMR_ItemType tipo, SMRItemList& list);	
	int CountItems(SMR_ItemType);
	SMRItem *GetSymItem(int nScore);
	SMRItem* GetItemByID(const wxString& id);
	SMRItem* GetItemByFilename(const wxString& filename);
	SMRAudioItem *GetMIDIItem();

	SMRItemList& GetItemsList() {return m_itemsList; };
	void GetScores(int nScore,Partitura*&);

	virtual bool OpenSymbolicScore(int score,short current_measure=0);
	void CloseMusicEditors();
	virtual void ExchangeParts(int pos, int posRef);
	virtual void HidePart(int pos);
	virtual void ShowPart(int pos);
	
	virtual SMRPartItem* PartAdded(int pos, const wxString& name);
	virtual bool PartDeleted(int pos);
	
	virtual SMRSynchInfoItem* AddSynchInfo(bool symbolic, char* ettFile, char* audioFile);
	virtual SMRLyricsItem* AddLyrics( const wxString& lwfFile,int partID, const wxString& language, const wxString& desc);
	virtual bool AddImage(wxString description);
	virtual bool AddSMFL(wxString file);
	virtual bool AddFont(wxString file);
	virtual bool AddAudio(wxString file);
	virtual bool AddMIDI(wxString file);

	wxString GetFilename() { return m_filename; };
	wxString GetPath() { return m_filePath; };
	wxString GetTitle() { return m_title; };
	bool IsFromMIDI() { return m_fromMIDI; };

	wxString GetFontPath(const char* fontName,bool& removeAfterUse);

	static wxString GetTmpPath();
	static void CleanTmpPath();

	virtual wxMEScoreFrame* NewWxMEScoreFrame(wxWindow* parent,const wxString& tit,const wxPoint& pos, const wxSize& siz);
	virtual AudioSyncFrame* OpenAudioSyncFrame(wxMEScoreFrame* parent, const wxString& title, const wxPoint& pos, SMRAudioItem * audio, int nMeasures) { return NULL; };
	virtual xercesc_2_6::InputSource* CreateFileInputSource(const wxString& file);

private:
	void LoadScore();
	void UpdateSMFL();


	wxString m_title;
	wxString m_filePath;
	wxString m_filename;
	SMRItemList m_itemsList;
	Partitura* m_mainScore;
	bool m_destroyMS;

	int m_version;
	int m_pictureWidth;
	int m_pictureHeight;
	Coding_Type m_codingType;
	bool m_musicEditorIsOpen;
	int m_lastLyrID;
	bool m_fromMIDI;
};

#endif