/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#ifndef SMRITEM_HPP
#define SMRITEM_HPP

#include <wx/string.h>
#include <wx/list.h>

enum SMR_ItemType
{
	SMR_Symbolic,
	  SMR_MainScore,
	  SMR_Part,
	SMR_Font,
	SMR_Lyrics,
	SMR_SynchInfo,
	SMR_SMFL,
	SMR_Audio,
	SMR_Image
};

class SMRFile;
class Partitura;

class SMRItem
{
public:
	SMRItem(SMR_ItemType type, SMRFile* smrf, wxString descr="", wxString filename="")
	{
		m_type=type;
		m_description=descr;
		m_filename=filename;
		m_parent=smrf;
	};
	
	SMR_ItemType GetType() { return m_type; }
	SMRFile* GetSMRFile() { return m_parent; }
	
	wxString GetItemDescription() { return m_description; }
	void SetDescription(wxString d) { m_description=d; } 
	
	wxString GetFilename() { return m_filename; }
	void SetFilename(wxString f) { m_filename=f; }

	wxString GetID() { return m_id; }
	void SetID(wxString id) { m_id=id; }
	
private:
	SMR_ItemType m_type;
	wxString m_description;
	wxString m_filename;
	wxString m_id;
	SMRFile* m_parent;
};


class SMRMainScoreItem : public SMRItem
{
public:
	SMRMainScoreItem(SMRFile* smrf,wxString descr="", wxString filename="") : SMRItem(SMR_MainScore,smrf,descr,filename) { }
	void LoadXML();
	void SaveXML();
private:
	Partitura* m_mainScore;
};

class SMRPartItem : public SMRItem
{
public:
	SMRPartItem(SMRFile* smrf, wxString descr="", wxString filename="") : SMRItem(SMR_Part,smrf,descr,filename) { }
	void LoadXML();
	void SaveXML(int nscore);
};

class SMRSMFLItem : public SMRItem
{
public:
	SMRSMFLItem(SMRFile* smrf,wxString descr="", wxString filename="") : SMRItem(SMR_SMFL,smrf,descr,filename) {}
};

class SMRFontItem : public SMRItem
{
public:
	SMRFontItem(SMRFile* smrf, wxString descr="", wxString filename="") : SMRItem(SMR_Font,smrf,descr,filename) {}
};

class SMRLyricsItem : public SMRItem
{
public:
	SMRLyricsItem(SMRFile* smrf, wxString lang, wxString descr="", wxString filename="") : SMRItem(SMR_Lyrics,smrf,descr,filename)
	{
		m_partID=0;
		m_modified=false;
	}

	bool GetIsModified() { return m_modified; }
	void SetIsModified(bool m) { m_modified=m; }

	int GetPartID() { return m_partID; }
	void SetPartID(int id) { m_partID=id; }

	wxString GetLanguage() { return m_language; }
	void SetLanguage(wxString l) { m_language=l; }
private:
	wxString m_language;
	int m_partID;
	bool m_modified;
};

class SMRSynchInfoItem : public SMRItem
{
public:
	SMRSynchInfoItem(SMRFile* smrf,wxString descr="", wxString filename="") : SMRItem(SMR_SynchInfo,smrf,descr,filename) {}
};

class SMRAudioItem : public SMRItem
{
public:
	SMRAudioItem(SMRFile* smrf,wxString descr="", wxString filename="") : SMRItem(SMR_Audio,smrf,descr,filename) {}
};

class SMRImageItem : public SMRItem
{
public:
	SMRImageItem(SMRFile* smrf, wxString descr="", wxString filename="") : SMRItem(SMR_Image,smrf,descr,filename) {}
};

WX_DECLARE_LIST(SMRItem, SMRItemList);

#endif