/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include "wx/wx.h"
#include "wx/image.h"

#include "smr/smrmain.hpp"
#include "smr/workspac.hpp"
//#include "smr/idmanager.hpp"
#include "smr/dlg/settings.hpp"
//#include "smr/smr.hpp"

#include "musiceditor.hpp"

int WelcomeOpen(int duration,wxWindow *parent); // in seconds

SMRApp* SMRApp::smrApp=NULL;

SMRApp::SMRApp()
{
	if(smrApp==NULL)
		smrApp=this;
	workspace=NULL;
	idmanager=NULL;
	settings=NULL;
}

bool SMRApp::OnInit()
{
	FrameManager::currFrame=NULL;
	
	//if(IsInteractive())
	//	wxMessageBox(argv[0],argv[1],wxOK | wxCENTRE | wxICON_EXCLAMATION) ; 

	wxSplitPath(argv[0],&appPath,NULL,NULL);
	appPath+=PATH_SEP;
	
	SetVendorName("SMR");
	SetAppName("");
	
	wxImage::AddHandler(new wxGIFHandler);

	settings = new SMRSettings; 

	char* fileToLoad = NULL;

	switch(argc)
    {
	case 2:
		fileToLoad = argv[1];
	case 3:
		if (strcmp(argv[1],"-i")==0)
		{
			// metti a posto le variabili di ambiente
			wxString mainpath=argv[2];
			settings->SetFilesPath(mainpath+"\\SMR\\") ;
			settings->SetMidiPath(mainpath+"\\midi\\"); 
			settings->SetErrPath(mainpath+"\\errmsg\\"); 
			settings->SetFontPath(mainpath+"\\font\\");
			settings->SaveSettings();
		}
		break;
	default:
		break;
	}
	
	if(!settings->LoadSettings())
	{
		SMREnvSettingsDlg dlg(settings);
		dlg.ShowDlg();
	}
		
	workspace = new WorkSpace;
	workspace->LoadWorkspace();

/*
	idmanager = new IDManager;	
	idmanager->Load();
*/

	EM.ReadErrFiles();
	InitMusicEditors(INITME_STARTAPP);
	
	OpenTopWindow(fileToLoad);
	
//	SMRFile::CleanTmpPath();
	
	return TRUE;
}

int SMRApp::OnExit()
{
	delete workspace;
	delete settings;
	delete idmanager;
	
	InitMusicEditors(INITME_EXITAPP);
	return 0;
}

void SMRProcessPendingEvents()
{
	SMRApp::GetApp().ProcessPendingEvents();
}

/********************************************************************************/
char * smrgetenv(char* var)
{
	wxString v=var;
	
	if(v=="SMR_FILES_PATH")
	{
		return (char*)SMRApp::GetApp().settings->GetFilesPath().c_str();
	}
	else if(v=="SMR_MIDI_PATH")
	{
		return (char*)SMRApp::GetApp().settings->GetMidiPath().c_str();
	}
	else if(v=="SMR_ERR_PATH")
	{
		return (char*)SMRApp::GetApp().settings->GetErrPath().c_str();
	}
	else if(v=="SMR_FONT_PATH")
	{
		return (char*)SMRApp::GetApp().settings->GetFontPath().c_str();
	}
	else if(v=="IMG_SCORE_PATH")
	{
		return (char*)SMRApp::GetApp().settings->GetImgScorePath().c_str();
	}
	
	return NULL;
}
