/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include "workspac.hpp"
#include "smr/smrmain.hpp"

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "wx/config.h"


/*
implementation of class MenuConfig
*/
MenuConfig::MenuConfig(MenuItem ItemID, int X, int Y, bool Orizzontal)
{
	itemID = ItemID;
	orizzontal = Orizzontal;
	x = X;
	y = Y;
}

MenuConfig::~MenuConfig()
{
	
}

WX_DEFINE_LIST(MenuConfigList);


/*
implementation of class WindowConfig
*/
WindowConfig::WindowConfig(wxString Name, int X, int Y, int W, int H)
{
	name = Name;
	x = X;
	y = Y;
	w = W;
	h = H;
}

WindowConfig::~WindowConfig()
{
	
}

WX_DEFINE_LIST(WindowConfigList);




/*
implementation of class WorkSpace
*/
WorkSpace::WorkSpace()
{
	menuConfigList = new MenuConfigList(wxKEY_INTEGER);
	windowConfigList = new WindowConfigList(wxKEY_STRING);
}


WorkSpace::~WorkSpace()
{
	menuConfigList->DeleteContents(TRUE);
	delete menuConfigList;
	
	windowConfigList->DeleteContents(TRUE);
	delete windowConfigList;
}


void WorkSpace::SaveWorkspace()
{
	wxConfigBase *pConfig = wxConfigBase::Get();
	
	//save menu
	MenuConfigList::Node *m_node = menuConfigList->GetFirst();
	while (m_node)
	{
		MenuConfig *menuConfig = m_node->GetData();
		
		wxString path=wxString::Format("/menus/%d/",menuConfig->GetItemID());
		pConfig->SetPath(path);
		pConfig->Write("x",(long)menuConfig->GetX());
		pConfig->Write("y",(long)menuConfig->GetY());
		pConfig->Write("horiz",(long)menuConfig->GetOrizzontal());
		
		m_node = m_node->GetNext();
	}
	//save window
	WindowConfigList::Node *w_node = windowConfigList->GetFirst();
	while (w_node)
	{
		WindowConfig *windowConfig = w_node->GetData();
		
		wxString path=wxString::Format("/windows/%s",windowConfig->GetName());
		pConfig->SetPath(path);
		pConfig->Write("x",(long)windowConfig->GetX());
		pConfig->Write("y",(long)windowConfig->GetY());
		pConfig->Write("w",(long)windowConfig->GetW());
		pConfig->Write("h",(long)windowConfig->GetH());
		w_node = w_node->GetNext();
	}
	
}


void WorkSpace::LoadWorkspace()
{
	wxConfigBase *pConfig = wxConfigBase::Get();
	if(!pConfig->Exists("/menus"))
	{

	}
	if(!pConfig->Exists("/windows/WDFEditor"))
	{
		//Set default values for window
		WindowConfig* WDFEditorconfig = new WindowConfig("WDFEditor", 50, 50, 250, 700);
		windowConfigList->Append("WDFEditor", WDFEditorconfig);
	}
	if(!pConfig->Exists("/windows/MusicEditor"))
	{
		WindowConfig* MusicEditorconfig = new WindowConfig("MusicEditor", -1, -1, 1000, 800);
		windowConfigList->Append("MusicEditor", MusicEditorconfig);	
	}
	if(!pConfig->Exists("/windows/AudioSyncEditor"))
	{
		WindowConfig* AudioSyncEditorconfig = new WindowConfig("AudioSyncEditor", -1, -1, 1000, 100);
		windowConfigList->Append("AudioSyncEditor", AudioSyncEditorconfig);	
	}

	pConfig->SetPath("/menus");

	for (int item = SYMB_NOTES_ID; item <= SYMB_VIOLIN_ID; item++)
	{
		long menu_x=-1, menu_y=-1;
		long menu_horiz=1;

		pConfig->Read(wxString::Format("%d/x",item),&menu_x);
		pConfig->Read(wxString::Format("%d/y",item),&menu_y);
		pConfig->Read(wxString::Format("%d/horiz",item),&menu_horiz);

		MenuConfig *menuconfig = new MenuConfig((MenuItem)item, menu_x, menu_y, (bool)menu_horiz);
		menuConfigList->Append(item, menuconfig);
	}

	pConfig->SetPath("/windows");

	wxString winName;
	bool cont;
	long dummy;

	cont=pConfig->GetFirstGroup(winName,dummy);
	while(cont)
	{
		long win_x=-1, win_y=-1;
		long win_h=-1, win_w=-1;

		pConfig->Read(wxString::Format("%s/x",winName.c_str()),&win_x);
		pConfig->Read(wxString::Format("%s/y",winName.c_str()),&win_y);
		pConfig->Read(wxString::Format("%s/h",winName.c_str()),&win_h);
		pConfig->Read(wxString::Format("%s/w",winName.c_str()),&win_w);

		WindowConfig *windowconfig = new WindowConfig(winName, win_x, win_y, win_w, win_h);
		windowConfigList->Append(winName, windowconfig);

		cont=pConfig->GetNextGroup(winName,dummy);
	}
}

void WorkSpace::GetMenuConfig(MenuItem ItemID, int &X, int &Y, bool &Orizzontal)
{
	int scrW,scrH;
	MenuConfigList::Node *node = menuConfigList->Find(ItemID);
	if (node==NULL) return;
	MenuConfig *menuConfig = node->GetData();
	X = menuConfig->GetX();
	Y = menuConfig->GetY();
	wxDisplaySize(&scrW,&scrH);
	if(X<-1 || X>scrW-1)
		X=100;
	if(Y<-1 || Y>scrW-1)
		Y=100;
	Orizzontal = menuConfig->GetOrizzontal();
}


void WorkSpace::SetMenuConfig(MenuItem ItemID, int X, int Y, bool Orizzontal)
{
	MenuConfigList::Node *node = menuConfigList->Find(ItemID);
	if (node==NULL) return;
	MenuConfig *menuConfig = node->GetData();
	menuConfig->SetX(X);
	menuConfig->SetY(Y);
	menuConfig->SetOrizzontal(Orizzontal);
}

void WorkSpace::GetWindowConfig(wxString name, int &X, int &Y, int &W, int &H)
{
	WindowConfigList::Node *node = windowConfigList->Find(name);
	if (node==NULL) return;
	WindowConfig *windowConfig = node->GetData();
	X = windowConfig->GetX();
	Y = windowConfig->GetY();
	W = windowConfig->GetW();
	H = windowConfig->GetH();
}


void WorkSpace::SetWindowConfig(wxString name, int X, int Y, int W, int H)
{
	WindowConfigList::Node *node = windowConfigList->Find(name);
	if (node==NULL) return;
	WindowConfig *windowConfig = node->GetData();
	windowConfig->SetX(X);
	windowConfig->SetY(Y);
	windowConfig->SetW(W);
	windowConfig->SetH(H);
}
