/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#ifndef WORKSPAC_HPP
#define WORKSPAC_HPP

#include "drw/memenu.hpp"
#include <wx/wxexpr.h>
#include <wx/list.h>
#include <wx/listimpl.cpp>


/*
This class contents menu parameters that must be save in the workspace
*/
class MenuConfig : public wxObject
{
 //DECLARE_DYNAMIC_CLASS(MenuConfig)
 protected:
   MenuItem itemID;  //identification 
   bool orizzontal;  //orientation
   int x, y;  //position

 public:
   MenuConfig(MenuItem ItemID, int X = -1, int Y = -1, bool Orizzontal = TRUE);
   ~MenuConfig();

   MenuItem GetItemID() { return itemID; };
   bool GetOrizzontal() { return orizzontal; };
   int GetX() { return x; };
   int GetY() { return y; };
   
   void SetItemID(MenuItem item) { itemID = item; };
   void SetOrizzontal(bool Orizzontal) { orizzontal = Orizzontal; };
   void SetX(int X) { x = X; };
   void SetY(int Y) { y = Y; };
};


WX_DECLARE_LIST(MenuConfig, MenuConfigList);

/*
This class contents window parameters that must be save in the workspace
*/
class WindowConfig : public wxObject
{
 protected:
   wxString name;  //name 
   int x, y;  //position
   int w, h;  //dimension

 public:
   WindowConfig(wxString Name, int X = -1, int Y = -1, int W = 100, int H = 100);
   ~WindowConfig();

   wxString GetName() { return name; };
   int GetX() { return x; };
   int GetY() { return y; };
   int GetW() { return w; };
   int GetH() { return h; };
   
   void SetTitle(wxString Name) { name = Name; };
   void SetX(int X) { x = X; };
   void SetY(int Y) { y = Y; };
   void SetW(int W) { w = W; };
   void SetH(int H) { h = H; };
};

WX_DECLARE_LIST(WindowConfig, WindowConfigList);

/*
This class contents the workspace: position and dimension of menu and window
of the application.
*/
class WorkSpace : public wxObject
{
 //DECLARE_DYNAMIC_CLASS(WorkSpace)
 protected:
	MenuConfigList *menuConfigList;  //list of MenuConfig
	WindowConfigList* windowConfigList;  //list of WindowConfig

 public:
    WorkSpace();
    ~WorkSpace();

    MenuConfigList* GetMenuConfigList() { return menuConfigList; };
    WindowConfigList* GetWindowConfigList() { return windowConfigList; };

	void SaveWorkspace();
	//Save workspace in the file WORKSPACE_FILE.
	//Call from da 'WDFFrame::OnQuit(wxCommandEvent& WXUNUSED(event))', from 'Menu::CloseMenu()' 
	//and from 'WDFFrame::OnSaveWorkspace'

	void LoadWorkspace();
	//Read workspace from file WORKSPACE_FILE. If this file don't exist create a 
	//workspace with default parameters.
	//Call from 'WDFApp::OnInit()'
	
	void GetMenuConfig(MenuItem ItemID, int &X, int &Y, bool &Orizzontal);
	//Return the configuration of menu identificated by ItemID. 
	//It's call before creation of new menu to give it position and 
	//orientation registred in the workspace.
	
	void SetMenuConfig(MenuItem ItemID, int X, int Y, bool Orizzontal);
	//Save the current configuration of the menu identificated by ItemID in the workspace.
	//Call when workspace is saved with 'WDFFrame::OnSaveWorkspace' and when
	//a menu is closed.

	void GetWindowConfig(wxString name, int &X, int &Y, int &W, int &H);
	//Return the configuration of the window identificated by name. 
	//Call before creation of a new window to give it position and
	//dimension registred in the workspace.

	void SetWindowConfig(wxString name, int X, int Y, int W, int H);
	//Save the current configuration of the window identificated by name in the workspace.

};

#endif