/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


/* $Id: listabat.cpp,v 1.1 2005/07/25 09:35:30 bellinpo Exp $ */


#include "spa/listabat.hpp"
#include "bat/cviolino.hpp"
#include "liooext.hpp"
#include "bat/giustif.hpp"

ListaBattute::ListaBattute(Spartito *psp)
{
//	Init(psp);
	nNullNodes=0;
}

ListaBattute::~ListaBattute()
{

#ifdef DEBUG_FREE
  if (tipoLeggio!=LEGGIO_MASAE)
  {
    FILE *pippo;

    pippo = fopen("DEAL_NET.txt","a");
    fprintf(pippo,"\n\n\n ---- non masae   DEALLOCO LISTABAT  ----------------\n\n\n");
    fclose(pippo);
  }
#endif


  Free();
}

void ListaBattute::Init(Spartito* psp)	
{
  // Inizializza la lista.
  first=NULL;
  last=NULL;
  numobject=0;
  CodeCounter = 1;
  PagCounter = 1;
  
  // Inizializza lo spartito con una battuta in chiave di violino.
  Battuta *bat=NULL;
  bat=new Battuta(psp->GetNCorde());  //prova DB!
  if (bat==NULL)
     EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaBattute","Init()");
 
  if(bat!=NULL)
   {
    bat->SetChiave(0,new CViolino);
    if (bat->GetChiave(0)==NULL)
       EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaBattute","Init()");

    bat->SetArmatura(0,DO_maggiore);
    bat->SetTempo("4/4");
    bat->SetBarra(FINALE);
    bat->SetNumProgress(1);
    bat->SetNumericCode(CodeCounter++);
	bat->SetNumberOfStaff(psp->GetNumberOfStaff()); //GB
    InsertTop(bat);
	bat->SetParentScore(psp);
	parentScore=psp;


  }  
  
  for(int i=0; i<MAX_VIEW; i++)
  {
	  firstBatPage[i]=NULL;
	  firstBatNextPage[i]=NULL;
	  CurrentNode[i]=NULL;
  }
}

/*
Battuta* ListaBattute::GetLastBatHit()
{
  return lastBatHit;
}
*/

void ListaBattute::Free()
{
//   //inizializza i puntatori.
//   first=NULL;
//   last=NULL;
//   numobject = 0;
  

#ifdef DEBUG_FREE
//APRINTF  printf("-----   per masae....  DEALLOCO LISTABAT  ----------------\n");
#endif

  ClearDeep(); 

  GetFirstBatPageNode()=GetFirstBatNextPageNode()=NULL;  
}




void ListaBattute::SetPagCounter(NumCode p)
{
  PagCounter = p;
}

NumCode ListaBattute::GetPagCounter()
{
  return PagCounter;
}


void ListaBattute::SetBattPage(Battuta *pbat)
{
  pbat->SetNumPagDLIOO(PagCounter++);
}


void ListaBattute::SetfirstBatPage(Battuta *pbat)
{ 
  GetFirstBatPageNode() = FindNodePtr(pbat);
}


Battuta *ListaBattute::GetfirstBatPage()
{ 
  if (GetFirstBatPageNode()==NULL) return NULL;
  return (Battuta*)(GetFirstBatPageNode()->ObjectPtr);
}

Battuta *ListaBattute::GetfirstBatNextPage()
{ 
  if (GetFirstBatNextPageNode()==NULL) return NULL;
  return (Battuta*)(GetFirstBatNextPageNode()->ObjectPtr); 
}

// rds giustifica se e' la prima volta che 
// la battuta viene visualizzata
void ListaBattute::JustifIfFirst(Battuta *bat)
{
   // dichiarazioni
   TipoGiust tp;
   double kGiust;

   // prendi i parametri
	// leggi se e'la prima volta che viene giustificata
	kGiust=bat->Getkgiustificazioneparte(currentViewType);
	tp=bat->Gettipogiustparte(currentViewType);
    
	if (G_NONE==tp)
		   {
			   tp=G_LOG;
			   kGiust=2;
		   }

    // per ora lo faccio sempre
    if (NO==bat->GetEverJustified(ORC, currentViewType))
	{
    	// rds
		bat->Setup(0);
		bat->SetPos(NULL, Point(0,0));
		bat->SetupSimboli();
		//bat->SetPos(NULL, Point(0,0));
	//	bat->SetVU(0);
		
		bat->GiustificaBattuta(tp, kGiust);
		
		//bat->GiustificaBattuta( tp, kGiust);
        bat->SetEverJustified(ORC, currentViewType, YES);

	}


   
}

Node* ListaBattute::CalcRigo(Node* frst,Pentagramma* pentagr,int nRig, Bool FirstColonna)
{ 
	Node *n = frst;
	Node *nPrec = NULL;
	Node *ret = NULL;
	Node *lst = NULL;
	VUnit x = 0, xPrec = 0, distFig = 0, pentWidth = 0;
	int nTotFig = 0, nTotFigPrec = 0;
	Battuta *bat;
	Intestazione **prevIntest = NULL;
	bool preclb=false;
	
	// preleva l'intestazione della battuta precedente se possibile.
	nPrec = GetPrevMeasure(n); 
	if(nPrec != NULL)
		prevIntest = GetMeasure(nPrec)->GetAllIntestEnd();
    
	nPrec = NULL;
	
	// determina la larghezza del pentagramma su cui sistemare le battute.
	pentWidth = pentagr->GetWidth();

	// segue la lista delle battute iniziando da quella specificata. 
	// Ogni battuta viene inizializzata con la distanza minima tra le figure
	// poi viene calcolato lo spazio occupato dalla battuta. Questo viene
	// fatto fino a che non si supera la larghezza del pentagramma.
	while(x <= pentWidth && n != NULL && !preclb)
	{
		bat=GetMeasure(n);
		JustifIfFirst(bat);
		bat->SetFirsColonna(FirstColonna);
		
		// imposta il tipo di battuta.
		if ((GetPrevMeasure(n) == NULL) && (bat->GetNumProgress()==1))
			bat->SetBattuta(INIZIO_BRANO);
		else if(x == 0 && nRig == 0)
			bat->SetBattuta(INIZIO_PAGINA, prevIntest);
		else if(x == 0)
			bat->SetBattuta(INIZIO_PENTAGRAMMA, prevIntest);
		else
			bat->SetBattuta(GENERICA, prevIntest);
		
		if (GetNextMeasure(n) == NULL) 
		{}
		
		// prepara prevIntest per la prossima battuta.
		prevIntest=bat->GetAllIntestEnd();
		
		// imposta la battuta con la minima distanza tra le figure.
		bat->SetDistanzaFig(MIN_DIST_FIG);
		bat->SetxBarra(0);
		bat->Setup(0);
		
		// memorizza lo spazio totale occupato fino a questo momento.
		xPrec=x;
		// incrementa lo spazio occupato con quello della battuta.
		x+=bat->GetWidth();
		
		// DB: se la battuta ha il line break si ferma
		if(nPrec!=NULL)
			preclb=GetMeasure(nPrec)->GetPartLineBreak(currentViewType);
		
		// memorizza il nodo attuale.
		nPrec=n;
		n=GetNextMeasure(n);
	}
	if(x>pentWidth && xPrec==0)
	{
		// se lo spazio occupato dalla prima battuta supera la larghezza del pentagramma
		// ritorna la battuta successiva dato che una battuta non puo' essere spezzata.
		return n;
	}
	// determina lo spazio effettivamente occupato dalle battute che riescono
	// ad entrare nel pentagramma.
	if(x<=pentWidth && n==NULL)
	{
		// se lo spazio totale occupato dalle battute esaminate e' inferiore allo
		// spazio disponibile e non ci sono piu' battute da esaminare allora
		// x e nTotFig contengono i valori effettivi delle battute da considerare.
		ret=NULL;
		lst=nPrec;
	}
	else
	{
		// se l'ultima battuta considerata ha fatto superare il limite allora
		// si deve riportare x e nTotFig allo stato precedente.
		x=xPrec;
		nTotFig=nTotFigPrec;
		ret=nPrec;
		lst=GetPrevMeasure(nPrec);
	}
	// RDS questo non esiste pi (dist_fig), ora c' la giustificazione
	// calcola la distanza tra le figure per il rigo, in modo da distribuire lo
	// spazio che avanza tra le figure delle battute.  
	if(nTotFig!=0)
		distFig=VUnit(MIN_DIST_FIG+(pentWidth-x)/nTotFig);
	else
		distFig=MIN_DIST_FIG;
	
	// ripercorre le battute fino all'ultima considerata 
	// (quella che ha fatto fallire il test) per posizionare le battute sul
	// pentagramma e impostare il valore della distanza tra figure trovata.
	n=frst;
	x=0;
	while(n!=ret)
	{
		bat=GetMeasure(n);
		// imposta la distanza tra le figure e lo spazio sopra e sotto la battuta.
		bat->SetDistanzaFig(distFig);
		bat->SetVU2Up(pentagr->GetVU2Up());    
		bat->SetVU2Dwn(GetParentScore()->GetVU2Dwn()); 
		bat->SetVU();
		bat->GetBarra()->Setup(0,0);   
		// se la battuta e' l'ultima, aggiusta la distanza della barra in modo da
		// occupare tutto il pentagramma.
		if(n==lst)
		{
			bat->SetxBarra(pentWidth-(x+bat->GetWidth()));
			bat->SetVU();
		}
		// posiziona la battuta sul pentagramma.
		bat->SetPos(pentagr,Point(x,0));
		x+=bat->GetWidth();
		n=GetNextMeasure(n);
	}
	// ritorna il nodo associato alla prossima battuta da posizionare, NULL
	// se non ci sono piu' battute.
	return ret;
}


Node* ListaBattute::CalcRigoLineBreak(Node* frst,Pentagramma* pentagr,int nRig, Bool FirstColonna)
{ 
	Node *n = frst;
	Node *nPrec = NULL;
	Node *ret = NULL;
	Node *lst = NULL;
	VUnit x = 0, xPrec = 0, distFig = 0, pentWidth = 0, wdt=0;
	// non basta pi n, ci vuole la larghezza e il target
	struct { VUnit wdt, xPrec, target; }  Cols[100]; 
	int nTotFig = 0, nTotFigPrec = 0, ncol=0, i=0, totale=0;
	Battuta *bat;
	Intestazione **prevIntest = NULL;
	Bool Consistent=TRUE, old=FALSE;
	VUnit avanzo=0, parziale=0;
	bool preclb=false;
	
	// RDS spezzo in due cicli, uno fa il check di quante battute
	// entrano nel rigo, l'altro fa lo stretch delle battute per il
	// line breaking e chiama il setup
	// Sistema la pagina attuale
	// ciclo di check
	// preleva l'intestazione della battuta precedente se possibile.
	nPrec = GetPrevMeasure(n);
	if(nPrec != NULL)
		prevIntest = GetMeasure(nPrec)->GetAllIntestEnd(); //GB
    
	nPrec = NULL;
	
	// determina la larghezza del pentagramma su cui sistemare le battute.
	pentWidth = pentagr->GetWidth();
	
	// segue la lista delle battute iniziando da quella specificata. 
	// Ogni battuta viene considerata per la sua larghezza tra le figure.
	// Questo viene
	// fatto fino a che non si supera la larghezza del pentagramma.
	
	while(x <= pentWidth && n != NULL && !preclb)
	{
		bat = GetMeasure(n);
		JustifIfFirst(bat);
		bat->SetFirsColonna(FirstColonna);
		
		// imposta il tipo di battuta.
		if ((GetPrev(n) == NULL) && (bat->GetNumProgress()==1))
			bat->SetBattuta(INIZIO_BRANO);
		else if(x == 0 && nRig == 0)
			bat->SetBattuta(INIZIO_PAGINA, prevIntest);
		else if(x == 0)
			bat->SetBattuta(INIZIO_PENTAGRAMMA, prevIntest);
		else
			bat->SetBattuta(GENERICA, prevIntest);
		
		if (GetNextMeasure(n) == NULL) 
		{}
		// prepara prevIntest per la prossima battuta.
		prevIntest=bat->GetAllIntestEnd(); 
		
		// imposta la battuta con la minima distanza tra le figure.
		bat->SetxBarra(0);
		old=AutoLineBreak;
		AutoLineBreak=FALSE;
		bat->Setup(0);
		AutoLineBreak=old;  
		// memorizza lo spazio totale occupato fino a questo momento.
		xPrec=x;
		// incrementa lo spazio occupato con quello della battuta.
		wdt=bat->GetWidth();
		x+=wdt;
		// memorizza il nodo attuale.
		
		// DB: se la battuta ha il line break si ferma
		if(nPrec!=NULL)
			preclb=GetMeasure(nPrec)->GetPartLineBreak(currentViewType);
		
		nPrec=n;
		n=GetNextMeasure(n);
		
		// determina lo spazio effettivamente occupato dalle battute che riescono
		// ad entrare nel pentagramma.
		if(x<=pentWidth && !preclb)
		{
			Cols[ncol].wdt=wdt;
			Cols[ncol].xPrec=xPrec;
			ncol++;
		}
		
	}
	if(x>pentWidth && xPrec==0)
	{
		// se lo spazio occupato dalla prima battuta supera la larghezza del pentagramma
		// ritorna la battuta successiva dato che una battuta non puo' essere spezzata.
		return n;
	}
	
	// determina lo spazio effettivamente occupato dalle battute che riescono
	// ad entrare nel pentagramma.
	if(x<=pentWidth && n==NULL)
	{
		// se lo spazio totale occupato dalle battute esaminate e' inferiore allo
		// spazio disponibile e non ci sono piu' battute da esaminare allora
		// x e nTotFig contengono i valori effettivi delle battute da considerare.
		xPrec=x;
		ret=NULL;
		lst=nPrec;
	}
	else
	{
		// se l'ultima battuta considerata ha fatto superare il limite allora
		// si deve riportare x e nTotFig allo stato precedente.
		x=xPrec;
		ret=nPrec;
		lst=GetPrevMeasure(n);
	}
	// calcola la distanza tra le figure per il rigo, in modo da distribuire lo
	// spazio che avanza tra le figure delle battute.  
	avanzo=pentWidth-xPrec;
	parziale=xPrec;
	// RDS ciclo di determinazione dei target
	totale=0;
	for (i=0; i<ncol; i++)
	{
		if (parziale!=0)
			Cols[i].target=(int) (Cols[i].wdt+Cols[i].wdt*(float)avanzo/(float)parziale);
		else Cols[i].target=Cols[i].wdt+avanzo;
		totale+=Cols[i].target;
	}
	totale=(parziale+avanzo)-totale;
	while (totale>0)
	{
		for (i=0; i<ncol; i++)
			if (totale>0)
			{
				totale--;
				Cols[i].target++;
			}
	}
	// ripercorre le battute fino all'ultima considerata 
	// (quella che ha fatto fallire il test) per posizionare le battute sul
	// pentagramma e impostare il valore della distanza tra figure trovata.
	n=frst;
	ncol=0;
	x=0;
	while(n!=ret)
	{
		bat=GetMeasure(n);
		// stretcha le battute a seconda delle necessit
		// verifica di consistenza
		Consistent=TRUE;
		Consistent=bat->CheckBatt();
		// inserire questi metodi
		int controllino=0;
		if((controllino=StretchBattuta(bat, Cols[ncol].target))==-1)
		{
			return 0;
		}
		ncol++;
		// imposta la distanza tra le figure e lo spazio sopra e sotto la battuta.
		bat->SetVU2Up(pentagr->GetVU2Up());    
		bat->SetVU2Dwn(GetParentScore()->GetVU2Dwn()); 
		
		// RDS dai un'occhiata
		bat->SetVU();
		bat->GetBarra()->Setup(0,0);   
		
		// se la battuta e' l'ultima, aggiusta la distanza della barra in modo da
		// occupare tutto il pentagramma.
		if(n==lst)
		{
			bat->SetxBarra(pentWidth-(x+bat->GetWidth()));
			bat->SetVU();
		}
		// posiziona la battuta sul pentagramma.
		// dai un'occhiata
		bat->SetPos(pentagr,Point(x,0));
		x+=bat->GetWidth();
		n=GetNextMeasure(n);
	}
	// ritorna il nodo associato alla prossima battuta da posizionare, NULL
	// se non ci sono piu' battute.
	return ret;
}

// RDS metodo che formatta una colonna di battute della dimensione voluta  
int ListaBattute::StretchBattuta(Battuta* vbat, UL totale)  
{  
  // questo metodo fa un ciclo di giustificazione iterata sulla colonna
  // di battute e si ferma quando lo spazio della colonna  spacecol
  // che gli passo come parametro
  // giustifica colonna di battute
  
  // esegui il metodo dicotomico per trovare il parametro che fa si' che la battuta sia larga 
  // la sua lunghezza piu' surplus.
  // i nuovi parametri di spaziatura delle figure si chiamano DLine per direttoriale e OLine
  // per l' orchestrale
  Bool trovato=FALSE;
  double kmin, kmax,inc=1, kold;
  UL totmax,totprova,totmin, totold;
  TipoGiust tp;
  // leggo il k di giustificazione della prima battuta della colonna, prendi il puntatore a battuta
  //kmin=vbat[0]->kgiustificazione;
  
  
  kmin=vbat->Getkgiustificazioneparte();
  tp=vbat->Gettipogiustparte();
  
  switch (tp)
  {
  case G_LOG: inc=0.2;
    break;
  case G_LIN: inc=100;
    break;
    
  }
  
  // controllo: se la battuta e' vuota assegna totale allo spazio prima della 
  // prima figura (attributo di battuta sul quale gioco)
  // questo e' per far funzionare il linebreak per battute vuote
  ListaFigure *pLayer;
  int nonvuoto=0;
  Figura *figtmp;
  //GB
  for(int i=0; i<NUM_MAX_LAYER; i++)
  {
	pLayer=vbat->GetPLayer(i);
	if(pLayer!=NULL)
	{
		figtmp=pLayer->GetFirstFig();
		if (figtmp!=NULL) 
			nonvuoto=1;
	}
  }
  
  if (nonvuoto==0)
  {
    vbat->SetDirSp2FirstFig(0,totale);
    
    return 1;
  }
  
  
  // imposto gli estremi del parametro 
  // chiama per la colonna justforlineb e guarda e aumenta finch lo spazio non  maggiore di spacecol 
  totmax=totmin=JustVerify(vbat,kmin);
  /* while (totmin>totale ) {
  kmin=kmin - inc;
  totmin=JustVerify(vbat,kmin);
}  */
  if (totmin==totale) return 1;
  kmax=kmin+inc;
  int uscita=0;
  while (totmax<totale) {
    //?    kmin=kmax;
    totmax=JustVerify(vbat,kmax);
    if (totmax<totale) kmax=kmax+inc;
    uscita++;
    if(uscita>10000)
    {
      //Message("Warning: Line Breaking Disabled");
      AutoLineBreak=FALSE;
      return -1;
    }
  }
  
  if (totmax==totale) return 1;
  
  //FILE *gt;
  //gt=fopen("prova.k", "a+");
  //fprintf(gt, "inizio %d, min %d %f, max %d %f \n", totale, totmin, kmin, totmax, kmax);
  
  // ricerca kopt
  trovato=FALSE;
  uscita=0;
  while (trovato==FALSE) {
    totprova=JustVerify(vbat,(kmax+kmin)/2);
    uscita++;
    
    //fprintf(gt, " %d %f %f %f \n", totprova, kmin, kmax, (kmax+kmin)/2);
    
    if (totprova<totale) 
    { kmin=(kmax+kmin)/2;
    totmin=totprova;
    }
    else if (totprova>totale) {
      kmax=(kmax+kmin)/2;
      totmax=totprova;
    }
	   
    if (totmax<totmin) {
      kold=kmin;
      kmin=kmax;
      kmax=kold;
      totold=totmin;
      totmin=totmax;
      totmax=totold;
      
    }
    
    
    if (abs(totprova-totale)<1) { 
      //kopt=(kmax+kmin)/2;
      trovato=TRUE;
    }	
    if ( fabs(kmin-kmax)<0.0001 )
      break;
    
    if (uscita>3000)
    {
      AutoLineBreak=FALSE;
      return -1;
    }
    
    
    if (abs(totprova-totale)<1) { 
      //kopt=(kmax+kmin)/2;
      trovato=TRUE;
    }	
    if ( fabs(kmin-kmax)<0.0001 )
      break;
    if(uscita>3000)
    {
      Message("Warning: Timeout!!, Line Break Disabled");
      AutoLineBreak=FALSE;
      return -1;
    }
    
  }	
  
  
  return 1;
  //fclose(gt);
  
}  

// RDS metodo che restituisce la larghezza di una battuta con
// certi parametri in ingresso  
UL ListaBattute::JustVerify(Battuta* vbat, double kGiust)  
{  
  TipoGiust tp;
  int maxFig=0;//,i;
  Giustificazione gd;
  // calcola maxFig
  vbat->SetupNumeroFig();
  maxFig+=vbat->GetNumeroFig()*2;

  gd.Costruisci(maxFig,vbat->GetNUsedLayers());//GB
  gd.Inserimento(vbat);
  // da cancellare in futuro
  //FILE *pFileDati;
  //int controllo;
  // Tabella= durata (-1 se null); beat corrente; spazio fino alla prossima; fi
  //if ((pFileDati=fopen ("tabsim.dat","a"))==NULL)
  //     printf ("Errore nell'apertura del file tabsim.dat\n");
  //else fprintf(pFileDati,"\n\ndopo Inserimento Giust.Battute ListaSpa\n");
  //if (( controllo=fclose (pFileDati))!=0)
  //     printf ("Errore nella chiusura del file tabsim.dat\n");
  //  gd.dump();
  tp=vbat->Gettipogiustparte();
  switch(tp)
  {
    case G_LOG:
      gd.Sincronizza();
	  //if ((pFileDati=fopen ("tabsim.dat","a"))==NULL)
      // printf ("Errore nell'apertura del file tabsim.dat\n");
  //else fprintf(pFileDati,"\n\ndopo Sincronizza Giust.Battute ListaSpa\n");
  //if (( controllo=fclose (pFileDati))!=0)
    //   printf ("Errore nella chiusura del file tabsim.dat\n");
    //	  gd.dump();
      gd.GiustificaLog(kGiust);

/*	  if ((pFileDati=fopen ("tabsim.dat","a"))==NULL)
       printf ("Errore nell'apertura del file tabsim.dat\n");
  else fprintf(pFileDati,"\n\ndopo Giustifica Giust.Battute ListaSpa\n");
  if (( controllo=fclose (pFileDati))!=0)
       printf ("Errore nella chiusura del file tabsim.dat\n");
*/
//	  gd.dump();
      break;
    case G_LIN:
      gd.Sincronizza();
      gd.GiustificaLin(kGiust);
      break;  
  }
//  gd.dump();

  // dovr settare qualcosa di diverso da spg, gli spazi di line breaking
  gd.SetUpSpOLine();
  
  vbat->SetVU(0);
  return vbat->GetWidth(); //"spazio della colonna di battute, memorizzato nei parametri di line breaking" getsptotaleline
}  
  



void ListaBattute::PageSetUp(Pentagramma pentagr[],int nRighi, Bool FirstColonna)
{
	Node *n = GetFirstBatPageNode();
    int rigo;
	int nStaff=((Battuta*)GetFirst())->GetNumberOfStaff();
    
	if(AutoLineBreak==1) // line breaking automatico 
	{  
        for(rigo = 0; rigo < nRighi && n != NULL; rigo++)
			n = CalcRigoLineBreak(n, &pentagr[rigo*nStaff], rigo, FirstColonna);
		if (n==0)  n = CalcRigo(n, &pentagr[rigo*nStaff], rigo, FirstColonna);
	}    
	else   // line breaking disabilitato
        for(rigo = 0; rigo < nRighi && n != NULL; rigo++)
			n = CalcRigo(n, &pentagr[rigo*nStaff], rigo, FirstColonna);
		
	GetFirstBatNextPageNode() = n;		
}


void ListaBattute::SaveCurrentPosition()
{
	SavedPosition = GetCurrentNode();
}

void ListaBattute::ResetCurrent(Node* SavedPos)
{
  Node *nPrec; 
    
  if (SavedPos==NULL)
		GetCurrentNode() = GetFirstBatPageNode();  
  else
		GetCurrentNode() = SavedPosition;

    // Preleva l'intestazione della battuta precedente (se possibile).
  nPrec = GetPrev(GetCurrentNode());
  if(nPrec != NULL)
	  PrevIntest[DrawObject::GetCurrentView()] = ((Battuta*)nPrec->ObjectPtr)->GetIntestEnd(); //IB
}

Battuta* ListaBattute::GetCurrentBatLst()
{
  Battuta *pB;
  
  if(GetCurrentNode()==NULL)
    return NULL;
  else
  {
    pB = (Battuta*)GetCurrentNode()->ObjectPtr;
    return pB;
  }
}


Battuta* ListaBattute::GetCurrentBat()
{
  Battuta *pB;
  
  if(GetCurrentNode()==NULL || GetCurrentNode()==GetFirstBatNextPageNode())
    return NULL;
  else
  {
    pB = (Battuta*)GetCurrentNode()->ObjectPtr;
//    GetCurrentNode() = GetCurrentNode()->Next;
    return pB;
  }
}

Bool ListaBattute::MoveCurrentBatForw()
{
 // Battuta *pB;
  
  if(GetCurrentNode()==NULL || GetCurrentNode()==GetFirstBatNextPageNode())
    return FALSE;
  else
  {
    GetCurrentNode() = GetCurrentNode()->Next;
    return TRUE;
  }
}

  
Bool ListaBattute::CheckCurrent(tipoSpartito ts, VUnit& IntWdt,
				VUnit& Wdt, Bool FirstColonna)
{
  Battuta *pbat, *pbat2;
  Intestazione **pIntest=NULL;


  if(GetCurrentNode()==NULL)
    return FALSE;

  
  pbat = (Battuta*)(GetCurrentNode()->ObjectPtr);

  pbat->SetFirsColonna(FirstColonna);

    // imposta il tipo di battuta.
  if(GetPrev(GetCurrentNode()) == NULL)
  {
    pbat->SetBattuta(INIZIO_BRANO, NULL, ts);
  }

    // INIZIO_PAGINA nongobat previsto per la partitura!
  else
  {
	  //pIntest=((Battuta*)GetPrev(GetCurrentNode())->ObjectPtr)->GetIntestEnd(); //PB
  	  pIntest=((Battuta*)GetPrev(GetCurrentNode())->ObjectPtr)->GetAllIntestEnd(); //PB
	  if(GetCurrentNode() == GetFirstBatPageNode() || (GetCurrentNode() == SavedPosition))
	  {
        pbat->SetBattuta(INIZIO_PENTAGRAMMA, pIntest, ts);
	  }
      else
	  {
        pbat->SetBattuta(GENERICA, pIntest, ts);
	  }
  }
    
  tipoBattuta tb;
  
  

  if ((GetNext(GetCurrentNode()) == NULL) && (pMusicWin->GetImpaginazione()) && (tipoLeggio==LEGGIO_MASAE))
  {
    pbat2 = (Battuta*)GetFirst();
    while (pbat2 != NULL)
    {
      tb = pbat2->GetBattuta();
      
      if ((tb == INIZIO_PENTAGRAMMA) || (tb == INIZIO_BRANO) ||
         (tb == INIZIO_PAGINA))
         pbat2->SetNumPagMASAE(PagCounter++);
      pbat2 = (Battuta*)GetNext(pbat2);
    }
  }

    // Imposta la battuta con la minima distanza tra le figure.
  pbat->SetDistanzaFig(MIN_DIST_FIG);
  pbat->SetxBarra(0);

  pbat->Setup(0);

  IntWdt = pbat->GetIntest(0)->GetWidth(); //PB&GB
  Wdt = pbat->GetWidth();
  
  return TRUE;
}


void ListaBattute::SetupCurrent(Pentagramma *pentagr, VUnit PosX, 
				VUnit IntWdt, VUnit MaxWdt, VUnit Batt2Up, VUnit Batt2Dwn,
				tipoSpartito tsp, Bool ProlBarUp, Bool ProlBarDown, int sistema)
{
  Node *pnext;
  Battuta *pbat;
  Barra *pbar;
  VUnit ProlUp=0, ProlDwn=0;
  
  if(GetCurrentNode()==NULL)
    return;
  
  pbat = (Battuta*)(GetCurrentNode()->ObjectPtr);

  pbat->Setup(IntWdt,MaxWdt);
/*  
  // Imposta le dimensioni verticali della battuta.
  if(ProlUp==0)
    // Nel caso spartito superiore o unico non si prolunga la
    // barra, ma la battuta va "prolungata" ugualmente.
    prol = ProlDwn+1;
  else
    prol = ProlUp;
  pbat->SetVU2Up(PENTHEIGHT+prol);
  pbat->SetVU2Dwn(prol-1);
  
  // Se richiesto, imposta i prolungamenti della barra.
  if((ProlBarra) && ((pbar = pbat->GetBarra()) != NULL))
    pbar->Setup(ProlUp, ProlDwn);
*/ 
  pbat->SetVU2Up(Batt2Up);
  pbat->SetVU2Dwn(Batt2Dwn);
  // Imposta gli eventuali prolungamenti della barra.
  if((pbar = pbat->GetBarra()) != NULL)
  {
    switch(tsp)
    {
      case SPART_SUPERIORE:
	    if (ProlBarDown)
          ProlDwn = pentagr->GetSpaceUpDwn(); //Batt2Dwn;
        break;
      case SPART_GENERICO:
	    if (ProlBarDown)
		  ProlDwn = pentagr->GetSpaceUpDwn(); //Batt2Dwn;
	    if (ProlBarUp)
		  ProlUp = pentagr->GetSpaceUpDwn(); //Batt2Up - pentagr->GetBaseVU2Up() +1;
		break;
      case SPART_INFERIORE:
	    if (ProlBarUp)  
		  ProlUp = pentagr->GetSpaceUpDwn(); //Batt2Up - pentagr->GetBaseVU2Up() +1;
		  break;
      // Per spartito unico nessun prolungamento.
    }
    pbar->Setup(ProlUp, ProlDwn);
  }
  
  
  // posiziona la battuta sul pentagramma.
  pbat->SetPos(pentagr,Point(PosX,0));
  //pbat->SetPos(pentagr,Point(PosX,0-sistema*300));
    
  PrevIntest[DrawObject::GetCurrentView()] = pbat->GetIntestEnd();//IB
  pnext = GetNext(GetCurrentNode());
  // Aggiorna il segnale di fine pagina.
  GetFirstBatNextPageNode() = pnext;
  GetCurrentNode() = pnext;

}


void ListaBattute::CurrentInNextPage()
{ GetFirstBatNextPageNode() = GetCurrentNode(); }


void ListaBattute::ResetfirstBatNextPage()
{
	GetFirstBatNextPageNode() = NULL;
}
  
/*
void ListaBattute::PageSetUpPart(Pentagramma* pentagr)
{ 
  Node *n = GetFirstBatPageNode();
  Node *nPrec = NULL;
  Node *ret = NULL;
  Node *lst = NULL;
  VUnit x = 0, xPrec = 0, distFig = 0, pentWidth = 0;
  int nTotFig = 0, nTotFigPrec = 0;
  Battuta *bat;
  Intestazione *prevIntest = NULL;

  // preleva l'intestazione della battuta precedente se possibile.
  nPrec = GetPrev(n);
  if(nPrec != NULL)
    prevIntest = ((Battuta*)nPrec->ObjectPtr)->GetIntest();
    
  nPrec = NULL;

  // determina la larghezza del pentagramma su cui sistemare le battute.
  pentWidth = pentagr->GetMaxWidth();
  
  // segue la lista delle battute iniziando dalla prima della pagina. 
  // Ogni battuta viene inizializzata con la distanza minima tra le figure
  // poi viene calcolato lo spazio occupato dalla battuta. Questo viene
  // fatto fino a che non si supera la larghezza del pentagramma.
  while(x <= pentWidth && n != NULL)
  {
    bat = (Battuta*)n->ObjectPtr;
    
    // imposta il tipo di battuta.
    if(GetPrev(n) == NULL)
    {
      bat->SetBattuta(INIZIO_BRANO, NULL);
    }
    // INIZIO_PAGINA non previsto per la partitura!
    else if(x == 0)
    {
      bat->SetBattuta(INIZIO_PENTAGRAMMA, prevIntest);
    }
    else
    {
      bat->SetBattuta(GENERICA, prevIntest);
    }
    
    // prepara prevIntest per la prossima battuta.
    prevIntest=bat->GetIntest();

    // imposta la battuta con la minima distanza tra le figure.
    bat->SetDistanzaFig(MIN_DIST_FIG);
    bat->SetxBarra(0);
    bat->Setup(0);
      
    // memorizza il numero totale di figure fino ad ora trovate.
    nTotFigPrec=nTotFig; 
    // incrementa il numero totale di figure esaminate con quelle della battuta attuale.
    nTotFig+=bat->GetNumeroFig()+1;

    // memorizza lo spazio totale occupato fino a questo momento.
    xPrec=x;
    // incrementa lo spazio occupato con quello della battuta.
    x+=bat->GetWidth();
    
    // memorizza il nodo attuale.
    nPrec=n;
    n=GetNext(n);
  }

  // determina lo spazio effettivamente occupato dalle battute che riescono
  // ad entrare nel pentagramma.
  if(x<=pentWidth && n==NULL)
  {
    // se lo spazio totale occupato dalle battute esaminate e' inferiore allo
    // spazio disponibile e non ci sono piu' battute da esaminare allora
    // x e nTotFig contengono i valori effettivi delle battute da considerare.
    ret=NULL;
    lst=nPrec;
  }
  else
  {
    // se l'ultima battuta considerata ha fatto superare il limite, allora
    // si deve riportare x e nTotFig allo stato precedente.
    x=xPrec;
    nTotFig=nTotFigPrec;
    ret=nPrec;
    lst=GetPrev(nPrec);
  }
  GetFirstBatNextPageNode() = ret;
  
  // Imposta la lunghezza effettiva del pentagramma.
  pentagr->SetWidth(x);
  
  // ripercorre le battute fino all'ultima considerata 
  // (quella che ha fatto fallire il test) per posizionare le battute sul
  // pentagramma.
  n=GetFirstBatPageNode();
  x=0;
  while(n!=ret)
  {
    bat=(Battuta*)n->ObjectPtr;
    
    // imposta lo spazio sopra e sotto la battuta.
//    bat->SetDistanzaFig(distFig);
//    bat->SetVU2UpDwn(pentagr->GetSpaceUpDwn());    
    bat->SetVU();

    // posiziona la battuta sul pentagramma.
    bat->SetPos(pentagr,Point(x,0));
    x+=bat->GetWidth();
    
    n=GetNext(n);
  }
}
*/


void ListaBattute::GoBatt(NumCode nb)
{
	nNullNodes=0;
	GetFirstBatPageNode() = first;
	if (GetFirstBatPageNode()==NULL) return;
	
	Battuta *pbt = (Battuta*)(GetFirstBatPageNode()->ObjectPtr);
	
	while (pbt->GetNumProgress() < nb) 
		if ((GetFirstBatPageNode()->Next) !=NULL) 
		{
			if (tipoLeggio==LEGGIO_DLIOO)
			{
				if(pbt->GetNumProgress()+pbt->GetNumMultiRest()>nb)
					break;
				GetFirstBatPageNode() = GetNextMeasure(GetFirstBatPageNode());
				pbt = GetMeasure(GetFirstBatPageNode());
			}
			else
			{
				GetFirstBatPageNode() = GetNext(GetFirstBatPageNode());
				pbt = (Battuta*)(GetFirstBatPageNode()->ObjectPtr);
			}
		}
		else
		{
			nNullNodes=nb-pbt->GetNumProgress();
			GetFirstBatPageNode()=NULL;
			return;
		}
}

void ListaBattute::GoBattInExecution(NumCode nb)
{
	// DB: Attenzione funziona solo per il main score!

	GetFirstBatPageNode() = first;
	if (GetFirstBatPageNode()==NULL)
		return;
	
	Battuta *pbt = (Battuta*)(GetFirstBatPageNode()->ObjectPtr);
	
	while (pbt->GetNumProgress() < nb) 
	{
		if ((GetFirstBatPageNode()->Next) !=NULL) 
		{
			GetFirstBatPageNode() = GetNext(GetFirstBatPageNode());
			pbt = (Battuta*)(GetFirstBatPageNode()->ObjectPtr);
		}
		else
			return;
	}
		
	Node* pn=GetFirstBatPageNode();
	pn=pn->Next;
	while(pn!=NULL &&
		((Battuta*)pn->ObjectPtr)->GetBattuta()!=INIZIO_PENTAGRAMMA &&
		((Battuta*)pn->ObjectPtr)->GetBattuta()!=INIZIO_PAGINA &&
		((Battuta*)pn->ObjectPtr)->GetBattuta()!=INIZIO_BRANO)
	{
		pn=pn->Next;
	}
	GetFirstBatNextPageNode()=pn;
}


void ListaBattute::GoTop(Pentagramma pentagr[],int nRighi)
{
  GetFirstBatPageNode()=first;
  nNullNodes=0;
//  PageSetUp(pentagr,nRighi);
}
    

Bool ListaBattute::GoForward()
{
	// avanza di una battuta
	Node *pn=NULL;
	if (tipoLeggio==LEGGIO_DLIOO)
		pn = GetNextMeasure(GetFirstBatPageNode());
	else
		pn = GetNext(GetFirstBatPageNode());
	GetFirstBatPageNode() = pn;
	GetFirstBatNextPageNode()=NULL; //PB
	if(pn==NULL)
	{
		nNullNodes++;
		return 0;
	}
	return 1;
}


Bool ListaBattute::GoBackward()
{
	// indietregia di una battuta
	Node *pn=NULL;
	if (tipoLeggio==LEGGIO_DLIOO)
		pn= GetPrevMeasure(GetFirstBatPageNode());
	else if(GetFirstBatPageNode()!=NULL)
		pn= GetPrev(GetFirstBatPageNode());
	else
	{
		nNullNodes--;
		if(nNullNodes==0)
			pn=last;
	}
	if(pn!=NULL)
	{
		GetFirstBatPageNode() = pn;
		GetFirstBatNextPageNode()=NULL; //PB
		return 1;
	}
	else
		return 0;
}


Bool ListaBattute::GoNextPage()
{
  Node *pn = GetFirstBatNextPageNode();
 // Battuta *pBat;
  if (tipoLeggio==LEGGIO_MASE)
  {
#ifdef WITH_NET    
        // dealloca la pagina attuale
        // che diverra' la pagina precedente
        // poiche' stiamo scrollando.
    if (pMusicWin->GetImpaginazione())
    {
	  if (GetFirstBatPageNode()!=NULL)
	  {
    	pBat = (Battuta*)(GetFirstBatPageNode()->ObjectPtr);
    	if (pBat->GetNumPagMASE()==0)
    	{
	      pMusicWin->ImpostaBattPage(pBat->GetNumericCode(), PagCounter);
          pBat->SetNumPagMASE(PagCounter);
          PagCounter++;
    	}
	  }

//       for(n=GetFirstBatPageNode();(n!=GetFirstBatNextPageNode()) && (n!=NULL);)
//       {
// 	pn = n;
// 	n = GetNext(n);
// 	obj = pn->ObjectPtr;
// 	DelPtr(obj);
// 	delete (Battuta*)obj;
//       }
    }
#endif
  }

    // Se GetFirstBatNextPageNode() e' diverso da NULL scambia i puntatori:
    // GetFirstBatPageNode() = GetFirstBatNextPageNode()
  if(pn==NULL)
    return FALSE;
  else
  {
    GetFirstBatPageNode() = pn;
	GetFirstBatNextPageNode()=NULL; //PB
    return TRUE;
  }

}

Bool ListaBattute::GoNextPageInExecution()
{
	Node *pn = GetFirstBatNextPageNode();
	if(pn==NULL)
		return FALSE;
	else
	{
		GetFirstBatPageNode() = pn;
		pn=pn->Next;
		while(pn!=NULL &&
			((Battuta*)pn->ObjectPtr)->GetBattuta()!=INIZIO_PENTAGRAMMA &&
			((Battuta*)pn->ObjectPtr)->GetBattuta()!=INIZIO_PAGINA &&
			((Battuta*)pn->ObjectPtr)->GetBattuta()!=INIZIO_BRANO)
		{
			pn=pn->Next;
		}
		GetFirstBatNextPageNode()=pn;
		//GetCurrentNode()=pn;
		return TRUE;
	}
}

Bool ListaBattute::GoPrevPage()
{
   Node *pn = first;
   Node *pPrev = NULL;
   
     // scorre la lista delle battute e cerca l'ultima battuta che era 
     // di inizio pagina!
	int n = GetCodeCounter();
 //impaginazione

   while (pn!=GetFirstBatPageNode())
   {
     if ((((Battuta*)pn->ObjectPtr)->GetBattuta()==INIZIO_PAGINA) ||
     (((Battuta*)pn->ObjectPtr)->GetBattuta()==INIZIO_PENTAGRAMMA) ||
     (((Battuta*)pn->ObjectPtr)->GetBattuta()==INIZIO_BRANO))
        pPrev = pn;
     pn=GetNext(pn);
   }
 
 
   if(pPrev==NULL)
     return FALSE;
   else
   {	
     GetFirstBatPageNode() = pPrev;
 	 GetFirstBatNextPageNode()=NULL; //PB
     return TRUE;
   }
}


Bool ListaBattute::PageUp(Pentagramma pentagr[],int nRighi, Bool FirstColonna)
{
  Node *n=GetFirstBatPageNode();

    // se e' la prima battuta dello spartito non si puo' tornare indietro.
  if(GetPrev(n)==NULL)
    return FALSE;
    
    // torna indietro fino alla prima battuta di inizio pagina registrata.
    // imposta questa battuta come prima battuta della pagina e inizializza
    // la pagina.
  do
    n=GetPrev(n);
  while(((Battuta*)n->ObjectPtr)->GetBattuta()!=INIZIO_PAGINA &&
        ((Battuta*)n->ObjectPtr)->GetBattuta()!=INIZIO_BRANO);
        
  GetFirstBatPageNode()=n; 
  
  PageSetUp(pentagr,nRighi,FirstColonna);
  return TRUE;
}

static Bool PrimaVolta = TRUE;

Bool ListaBattute::PageDwn(Pentagramma pentagr[],int nRighi, Bool FirstColonna)
{
 // Node *n,*pn;
//  DrawObject *obj;
//  Battuta *pBat;


  if(GetFirstBatNextPageNode()!=NULL)
  {

#ifdef WITH_NET    
        // dealloca la pagina attuale
        // che diverra' la pagina precedente
        // poiche' stiamo scrollando.
    if (pMusicWin->GetImpaginazione())
    {
      pBat = (Battuta*)(GetFirstBatPageNode()->ObjectPtr);
      if (pBat!=NULL)
	  {
    	if (pBat->GetNumPagDLIOO()==0)
    	{
	      pMusicWin->ImpostaBattPage(pBat->GetNumericCode(), PagCounter);
          pBat->SetNumPagDLIOO(PagCounter);
          PagCounter++;
    	}
	  }

      pBat = (Battuta*)(GetFirstBatNextPageNode()->ObjectPtr);
      if (pBat!=NULL)
	  {
    	if (pBat->GetNumPagDLIOO()==0)
    	{
	      pMusicWin->ImpostaBattPage(pBat->GetNumericCode(), PagCounter);
          pBat->SetNumPagDLIOO(PagCounter);
          PagCounter++;
    	}
	  }

//       for(n=GetFirstBatPageNode();(n!=GetFirstBatNextPageNode()) && (n!=NULL);)
//       {
// 	pn = n;
// 	n = GetNext(n);
// 	obj = pn->ObjectPtr;
// 	DelPtr(obj);
// 	delete (Battuta*)obj;
//       }
    }
#endif
    
    GetFirstBatPageNode()=GetFirstBatNextPageNode();
	GetFirstBatNextPageNode()=NULL; //PB
    PageSetUp(pentagr,nRighi,FirstColonna);
    return TRUE;
  }
  



  if (pMusicWin->GetBeginImpag())
  {
    PageSetUp(pentagr,nRighi,FirstColonna);
    return TRUE;
  }
  else 
  {
    if (pMusicWin->GetFirstEndBattImp())
	{
      PageSetUp(pentagr,nRighi,FirstColonna);
      pMusicWin->SetImpaginazione(END_BATT_IMP);
	  return TRUE;
	}
  }
	

  return FALSE;
}




void ListaBattute::DrawPageMainScore()
{
  Node *n=GetFirstBatPageNode();

  while(n!=GetFirstBatNextPageNode() && n!=NULL)
  {
    ((Battuta*)n->ObjectPtr)->Draw();
    n=GetNext(n);
  }
}

void ListaBattute::DrawPagePart()
{
  Node *n=GetFirstBatPageNode();

  while(n!=GetFirstBatNextPageNode() && n!=NULL)
  {
    GetMeasure(n)->Draw();
    n=GetNextMeasure(n);
  }
}


void ListaBattute::RedrawPage(Pentagramma pentagr[], int nRighi, Bool FirstColonna)
{
  PageSetUp(pentagr,nRighi, FirstColonna);
  //DrawPage();
}


Bool ListaBattute::Hit(const Point& p, SelObj sobj, DrawObject *& objsel,
		       SymPath& symPath)
{
  Node *n;

  objsel=NULL;
  // lastBatHit=NULL;


//  for( n=first; n!=NULL; n=GetNext(n))
  for( n=GetFirstBatPageNode(); n!=NULL && n!=GetFirstBatNextPageNode(); n=GetNext(n))
    if(n->ObjectPtr->Hit(p,sobj,objsel,symPath))
    { 
      // if(objsel!=NULL)
      //  lastBatHit=(Battuta*)n->ObjectPtr;
      return TRUE;
    }
    
  return FALSE;
}


Bool ListaBattute::Hit(const class Rectangle& r,SelObj selObj,short layer1,
		       SymPath& sp1,SymPath& sp2)
{
  Node *n=GetFirstBatPageNode();
  Bool found=FALSE;
  
  while(n!=GetFirstBatNextPageNode())
  {
    if(((Battuta*)n->ObjectPtr)->Hit(r,selObj,layer1,sp1,sp2))
    {
      found = TRUE;
    }
    else if(found)
      break;
    n=GetNext(n);
  }
  if(found)
    return TRUE;
  
  return FALSE;
}

Bool ListaBattute::SymbolicHit(SymPath& symPath, SelObj sobj,
			       DrawObject *& objsel)
{
  Node *n;

  objsel=NULL;
  // lastBatHit=NULL;
  
//  while(n!=GetFirstBatNextPageNode())
  for( n=first; n!=NULL; n=GetNext(n))
    if(((Battuta*)n->ObjectPtr)->SymbolicHit(symPath,sobj,objsel))
    { 
      // if(objsel!=NULL)
      //   lastBatHit=(Battuta*)n->ObjectPtr;
      return TRUE;
    }

  return FALSE;
}


void ListaBattute::AddAfter(Battuta* bat)
{
	DrawObject *curObj;
	Bool fl,eq;
	
	
	fl = TRUE;
	eq = FALSE;
	curObj = GetFirst();
	while (curObj!=NULL && fl)
	{
		fl = (bat->GetNumProgress() > ((Battuta*)curObj)->GetNumProgress());
		eq = (bat->GetNumProgress() == ((Battuta*)curObj)->GetNumProgress());
		curObj = GetNext(curObj);
	}
	
	if(curObj!=NULL && eq)
	{
		Battuta *pBattuta=(Battuta*)GetPrev(curObj);
		while (pBattuta!=NULL)
		{
			pBattuta->SetNumProgress(pBattuta->GetNumProgress()+1);
			pBattuta = (Battuta*)GetNext(pBattuta);
		}
	}
	
	if (curObj!=NULL || !fl)
	{
		if (curObj==NULL) 
			curObj = GetLast();
		else
			curObj = GetPrev(curObj);
		
		if (curObj==GetFirst())
		{
			InsertTop(bat);
			GetFirstBatPageNode()=first;
		}
		else
		{
			curObj = GetPrev(curObj);
			InsertAfter(bat,curObj); 
		}
	}
	else
	{
		if (GetLast()!=NULL)
			InsertEnd(bat); 
		else
		{
			InsertTop(bat);
			GetFirstBatPageNode()=first;
		}
	}
}

void ListaBattute::Add(Battuta* bat)
{
  int np = 0;
  
  if (GetLast()!=NULL)
  { 
    np = ((Battuta *)GetLast())->GetNumProgress();
    InsertEnd(bat); 
  }
  else
  {
    InsertTop(bat);
    GetFirstBatPageNode()=first;
  }    

#ifdef WITH_NET
  if (tipoLeggio==LEGGIO_MASAE)
  {
    bat->SetNumProgress(np +1);
    bat->SetNumericCode(CodeCounter++);
  }
#else
  bat->SetNumProgress(np +1);
  bat->SetNumericCode(CodeCounter++);
#endif
  
  // Possibile alternativa (da valutare): 
  //CmdAddBattuta((Battuta*)GetLast(), TRUE,0);
}


Bool ListaBattute::CmdAddBattuta(Battuta *bat, Bool addAfter,VUnit spcUD, ArgType& nc, ArgType& np, Spartito* psp)
{
  Node *prevNode, *pn;
  Battuta *newBat,*preBat;
  int nump;
  
  
  if(bat==NULL)
    return FALSE;

  newBat=new Battuta(psp->GetNCorde());
  if(newBat==NULL)
  { 
    EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaBattute","CmdAddBattuta()");
    return FALSE;
  };
  newBat->SetNStaffLine(psp->GetNCorde());
  newBat->SetNumberOfStaff(psp->GetNumberOfStaff());
  newBat->SetParentScore(psp);	//DB
  parentScore=psp;

// Istruzione tolta: togliere anche spcUD ?
//  newBat->SetVU2UpDwn(spcUD);

  if (tipoLeggio==LEGGIO_MASAE)
  {
    nc = (ArgType)CodeCounter;
    newBat->SetNumericCode(CodeCounter++);
  }
  else
#ifdef WITH_NET   
    newBat->SetNumericCode((NumCode)nc);
#else
    newBat->SetNumericCode(CodeCounter++);
#endif

  
  if(addAfter)
    prevNode=FindNodePtr(bat);
  else
    prevNode=GetPrev(FindNodePtr(bat));

  if(prevNode!=NULL)
  {
    preBat=(Battuta*) prevNode->ObjectPtr;
    
	newBat->CopyAllIntest(preBat->GetAllIntestEnd());
    //!!! newBat->CopyIntest(preBat->GetIntestEnd()); //IB

    // se la battuta precedente e' l'ultima allora se la battuta precedente
    // ha la barra finale (potrebbe avere la barra di fine ritornello) si mette
    // la barra singola, e comunque si mette la barra finale alla nuova battuta.
    if(prevNode->Next==NULL)
    {
      if(preBat->GetBarra()->GetID()==CL_BFINALE)
        preBat->SetBarra(SINGOLA);
      newBat->SetBarra(FINALE);
    }
    else
      newBat->SetBarra(SINGOLA);

    InsertAfter(prevNode,newBat);

    // Imposta il numero progressivo alla nuova battuta ed alle seguenti.
    nump = preBat->GetNumProgress();
    nump++;
    if (tipoLeggio==LEGGIO_MASAE)
      np = (ArgType)nump;
    newBat->SetNumProgress(nump);
    nump++;
    pn = GetNext(GetNext(prevNode));
    while(pn!=NULL)
    {
      ((Battuta*)pn->ObjectPtr)->SetNumProgress(nump);
      nump++;
      pn = GetNext(pn);
    }
    //	Poiche' non e' la prima battuta della lista, sia DLIOO che MASAE
    //	possono rinumerarsi da soli partendo dal nump della battuta
    //	precedente. In piu' MASAE deve passare a DLIOO il suo nump attraverso
    //	np, perche' potrebbe essere la prima battuta della lista di DLIOO.
    // (DLLIO infatti ha un pezzo della lista di MASAE e non tutta la lista).
    
  }
  else
  {
    newBat->SetChiave(0,new CViolino);
    if (newBat->GetChiave(0)==NULL)
       EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaBattute","CmdAddBattuta()");

    newBat->SetArmatura(0,DO_maggiore);
    newBat->SetTempo("4/4");
    newBat->SetBarra(SINGOLA);
    pn = first;
    InsertTop(newBat);

    // Imposta il numero progressivo alla nuova battuta ed alle seguenti.
    if (tipoLeggio==LEGGIO_MASAE)
    {
      nump=1;
      np=1;
    }
    else
#ifdef WITH_NET   
      nump = (int)np;
#else
      nump=1;
#endif
      
    // 	In questo caso la battuta e' la prima della lista, quindi si deve 
    //	settare in maniera opportuna il nump: 1 per MASAE, per DLIOO si prende
    // il valore passato da MASAE.
    //
    newBat->SetNumProgress(nump);
    nump++;
    while(pn!=NULL)
    {
      ((Battuta*)pn->ObjectPtr)->SetNumProgress(nump);
      nump++;
      pn = GetNext(pn);
    }
 
    GetFirstBatPageNode()=first;
    // se e' stata aggiunta una battuta prima della prima battuta allora
    // bisogna cambiare la prima battuta.
  }
  newBat->SetNumberOfStaff(bat->GetNumberOfStaff());
  return TRUE;
}

Bool ListaBattute::CmdAddBattuta(Battuta *bat, Bool addAfter,VUnit spcUD, Spartito* psp)
{
  ArgType nc, np;
  nc=0;
  np=0;

  return CmdAddBattuta(bat, addAfter, spcUD, nc, np, psp);
}

Bool ListaBattute::CmdDelBattuta(Battuta *bat)
{ 
  Node *node, *n;
  Bool del=FALSE, flag;
  int np;

  if(bat==NULL)
    return FALSE;
  
  node=FindNodePtr(bat);

    // se si cancella l'ultima battuta si deve spostare la barra finale alla
    // battuta precedente;
  if (tipoLeggio==LEGGIO_MASAE)
    if(GetNext(node)==NULL && GetPrev(node)!=NULL)
      ((Battuta*)GetPrev(node)->ObjectPtr)->SetBarra(FINALE);

    // se e' la prima battuta della pagina sposta la prima battuta alla successiva
  if(GetFirstBatPageNode()==node)
  {
    GetFirstBatPageNode()=GetNext(node);
      // se non ci sono battute dopo, si posiziona sulla pagina precedente.
    if(GetFirstBatPageNode()==NULL)
    {
      n=node;
      // se e' la prima battuta dello spartito non si puo' tornare indietro.
      if(GetPrev(n)!=NULL)
      {
        // torna indietro fino alla prima battuta di inizio pagina registrata.
        // imposta questa battuta come prima battuta della pagina e inizializza
        // la pagina.
		flag = TRUE;
        while (flag)
		{
          n=GetPrev(n);
		  flag = (n!=NULL);
		  if (flag) 
		      flag = (((Battuta*)n->ObjectPtr)->GetBattuta()!=INIZIO_PAGINA &&
              ((Battuta*)n->ObjectPtr)->GetBattuta()!=INIZIO_BRANO);
		}

        GetFirstBatPageNode()=n;
      }
    }
  }

    // Riorganizza i numeri progressivi delle battute seguenti.
#ifdef WITH_NET
  if(tipoLeggio==LEGGIO_MASAE)
  {
    np = bat->GetNumProgress();
    n = GetNext(node);
    while(n!=NULL)
    {
      ((Battuta*)n->ObjectPtr)->SetNumProgress(np);
      np++;
      n = GetNext(n);
    }
  }
#else
    np = bat->GetNumProgress();
    n = GetNext(node);
    while(n!=NULL)
    {
      ((Battuta*)n->ObjectPtr)->SetNumProgress(np);
      np++;
      n = GetNext(n);
    }
#endif  

//     np = bat->GetNumProgress();
//     n = GetNext(node);
//     while(n!=NULL)
//     {
//       ((Battuta*)n->ObjectPtr)->SetNumProgress(np);
//       np++;
//       n = GetNext(n);
//     }
// 

    // cancella il nodo dalla lista.
  if(node==first)
  {
    // Se c'e` un solo nodo...
    if(node==last)
    {
      first=NULL;
      last=NULL;
      del=TRUE;
    }
    else
    {
      node->Next->Prev=NULL;
      first=node->Next;
      del=TRUE;
    }
  }
  else if(node==last)
  {
    node->Prev->Next=NULL;
    last=node->Prev;
    del=TRUE;
  }
  if(!del)
  {
    node->Prev->Next=node->Next;
    node->Next->Prev=node->Prev;
  }
  numobject--;
  delete node; // =NULL; //NESIDELETE
  delete bat;  // =NULL; //NESIDELETE
  lastRef=NULL;

    // se non ci sono piu' battute aggiunge una battuta.
  if(isEmpty())
  {
    Init();
    GetFirstBatPageNode()=first;
  }
  return TRUE;
}


Bool ListaBattute::DeleteSym(DrawObject* sym)
{
//   Node *n=GetFirstBatPageNode();
//   
//   while(n!=GetFirstBatNextPageNode())
//   {
//     if(n->ObjectPtr->DeleteSym(sym))
//        return TRUE;
//     n=GetNext(n);
//   }
// 
//   return FALSE;

  Node *n=first;
  
  while(n!=NULL)
  {
    if(n->ObjectPtr->DeleteSym(sym))
       return TRUE;
    n=GetNext(n);
  }

  return FALSE;
}


Battuta* ListaBattute::GetBattWithFig(Figura* fig)
{
	Battuta *ptr=(Battuta*)GetFirst();
	
	while (ptr!=NULL)
	{
		for(int layer=0;layer<NUM_MAX_LAYER;layer++)
		{
			Layer *pLayer=ptr->GetPLayer(layer);
			if (pLayer!=NULL && pLayer->FindFig(fig))
				return ptr;
		}
		ptr = (Battuta*)GetNext((DrawObject*)ptr);	
    }
	return NULL;
}

Battuta* ListaBattute::GetBattNumProg(NumCode np)
{
  Battuta *ptr=(Battuta*)GetFirst();

  while(((ptr->GetNumProgress())!=np) && (GetNext(((DrawObject*)ptr))!=NULL))
    ptr = (Battuta*)GetNext((DrawObject*)ptr);
    
  if ((ptr->GetNumProgress())==np)
    return ptr;
  else
    return NULL;
}


Battuta* ListaBattute::GetBatt(NumCode nc)
{
  Battuta *ptr=(Battuta*)GetFirst();

  while(((ptr->GetNumericCode())!=nc) && (GetNext(((DrawObject*)ptr))!=NULL))
    ptr = (Battuta*)GetNext((DrawObject*)ptr);
    
  if ((ptr->GetNumericCode())==nc)
    return ptr;
  else
    return NULL;
}

Bool ListaBattute::FigPtr2SymPath(SymPath& symPath, Figura *figptr)
{
  Bool trovato;
  
  Battuta *ptr=(Battuta*)GetFirst();
  if(ptr!=NULL)
     {
      trovato = ptr->FigPtr2SymPath(symPath,figptr);
     }
  else
     {
      return FALSE;
     }
     
  while((!trovato) && (GetNext(((DrawObject*)ptr))!=NULL))
     {
      ptr = (Battuta*)GetNext((DrawObject*)ptr);
      trovato = ptr->FigPtr2SymPath(symPath,figptr);
     }      
    
    return trovato;
}

void ListaBattute::SetCodeCounter(NumCode nc)
{
 CodeCounter = nc;
}

NumCode ListaBattute::GetCodeCounter()
{
 return CodeCounter;
}

void ListaBattute::SetupCurrentSimboli()
{
  Battuta *pbat;
  if(GetCurrentNode()==NULL)
    return;
  
  pbat = (Battuta*)(GetCurrentNode()->ObjectPtr);
  pbat->SetupSimboli();
  GetCurrentNode() = GetNext(GetCurrentNode());
}


void ListaBattute::SetupSimboli(Bool beforeLegature)
{
  Node *n=GetFirstBatPageNode();

  while(n!=GetFirstBatNextPageNode() && n!=NULL)
  {
    ((Battuta*)n->ObjectPtr)->SetupSimboli(beforeLegature);
    n=GetNext(n);
  }
}
 
// DrawObject *ListaBattute::PosComp(unsigned long pos, Bool forward, int &nb)
// {
//   Node *ptr=first;
//   Battuta *pbat=NULL;
//   Battuta *pfbat=NULL, *plbat=NULL;
//   Battuta *pcbat=NULL;
//   unsigned long Position=0;
//   int nbat=0;
// 
//   if((pos>=0) && (pos<numobject))
//     while(ptr)
//     {
//       if(Position==pos) //ho trovato la battuta situata all'n-esima posizione
//       {
// 	pbat=(Battuta*)ptr->ObjectPtr;
// 	if(pbat->CheckBattPause(NULL))
// 	{
// 	  nbat=1;
// 	  pfbat=pbat;
// 	   // adesso devo vedere se la successiva o la precedente
// 	   //  sempre una battuta di sole pause
// 	  if(forward) 
// 	    pbat=(Battuta*)GetNext(pbat);
// 	  else 
// 	    pbat=(Battuta*)GetPrev(pbat); 
//           while(pbat!=NULL && pbat->CheckBattPause(pfbat))
// 	  {
// 	   nbat++;
// 	   plbat=pbat;
// 	   if(forward) 
// 	     pbat=(Battuta*)GetNext(pbat);
// 	   else 
// 	     pbat=(Battuta*)GetPrev(pbat); 
// 	  }
// 	  nb=nbat;
// 	  if(nbat!=1) return pcbat=CompressBatt(pfbat,plbat,nb);
// 	  else return ptr->ObjectPtr;  
// 	}
// 	else 
// 	{  
// 	  nb=1;
// 	  return ptr->ObjectPtr;
// 	}
//       }
//       nb=0;
//       ptr=ptr->Next;
//       Position++;
//     }
//   return NULL;
// }


void ListaBattute::ComprimiBattute(Bool abil_compress)
{
	Node *ptr=first;
	Battuta *pbat=NULL;
	Battuta *pfbat=NULL, *plbat=NULL;
	Battuta *pcbat=NULL;
	Bool skipnext;
	int check;
	
	int nbat=0;
	
	if(abil_compress)
	{		
		while(ptr!=NULL)
		{
			pbat=(Battuta*)ptr->ObjectPtr;
			skipnext=FALSE;
			if(pbat->CheckBattPause(NULL)==1) // il metodo Check.. ritorna 1 se la battuta  di sole pause
			{
				nbat=1;
				pfbat=pbat; // prima battuta a verificare il check
				
				// adesso devo vedere se la successiva battuta
				//  sempre una battuta di sole pause e con lo stesso tempo della prima
				
				ptr=ptr->Next;
				if(ptr!=NULL) pbat=(Battuta*)ptr->ObjectPtr;
				else pbat=NULL; 
				
				while(pbat!=NULL && (check=pbat->CheckBattPause(pfbat))==1)
				{
					nbat++;
					plbat=pbat; // ultima battuta a verificare il check
					if(plbat->GetPMultiRest()!=NULL)
					{
						delete plbat->GetPMultiRest();
						plbat->SetPMultiRest(NULL);
						plbat->SetIsMultiRest(FALSE);
						plbat->SetNumMultiRest(0);
					} 
					plbat->SetIsMultiRest(); 
					pfbat->SetIsMultiRest();
					
					ptr=ptr->Next;
					if(ptr!=NULL) pbat=(Battuta*)ptr->ObjectPtr;
					else pbat=NULL;
				}
				if(pbat!=NULL && pbat->GetIsMultiRest())
				{
					if(pbat->GetPMultiRest()!=NULL)	  
						delete pbat->GetPMultiRest();
					pbat->SetPMultiRest(NULL);
					pbat->SetIsMultiRest(FALSE); 
					pbat->SetNumMultiRest(0);
				}  
				
				if(pbat!=NULL && check==2)
				{	    	    
					skipnext=TRUE;		
				}
				if(pbat!=NULL && check==3)
				{	    	    
					plbat=pbat;	
					pfbat->SetIsMultiRest();
					plbat->SetIsMultiRest(); 
					nbat++;	
					//		skipnext=TRUE;
				}
				if(nbat!=1) CompressBatt(pfbat,plbat,nbat);
			}
			else 
			{ 
				if(pbat->GetPMultiRest()!=NULL)
				{
					delete pbat->GetPMultiRest();
					pbat->SetPMultiRest(NULL);
					pbat->SetIsMultiRest(FALSE); 
					pbat->SetNumMultiRest(0);
				}  
			}
			if(ptr!=NULL && !skipnext) ptr=ptr->Next;
		}
	} //fine if dell'abil_compress
	else
	{
		// la parte non ha la compressione abilitata dunque devo analizzare
		// tutte le battute del brano e se ve ne sono compresse le devo scomprimere!
		ptr=first;		
		while(ptr!=NULL)
		{
			pbat=(Battuta*)ptr->ObjectPtr;
			if(pbat->GetIsMultiRest())
			{
				//  una battuta compressa........la devo scomprimere
				pcbat=pbat->GetPMultiRest();
				if(pcbat!=NULL) 
				{  
					delete pcbat;
					pbat->SetPMultiRest(NULL);
				}
				pbat->SetIsMultiRest(FALSE);
			}
			ptr=ptr->Next;	
		}
	} 
}


void ListaBattute::CompressBatt(Battuta* pbat1, Battuta *pbat2, int nb)
{
	//devo costruirmi una battuta fittizzia!!
	Battuta *pcbat=pbat1->GetPMultiRest();
	
	if(pcbat!=NULL)
	{
		if(pcbat->GetNumMultiRest()==nb && pbat1->GetNumProgress()==pcbat->GetNumProgress())
			return;
		else delete pcbat; 
	}
	
	pcbat=new Battuta;  
	if(pcbat==NULL)
	{ 
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaBattute","CompressBatt()");
	};
	
	pcbat->SetParentScore(pbat1->GetParentScore());
	pcbat->MakeBattMultiRest(pbat1,pbat2,nb);
	pbat1->SetPMultiRest(pcbat);
}


int ListaBattute::GetNumeroBattuteVere()
{
  Node *pn=first;
  Battuta *pbat;
  int nb;
  int numb=0;

  while(pn!=NULL)
  {
    pbat=(Battuta*)pn->ObjectPtr;
    if(pbat->GetIsMultiRest())
    {
      //  una battuta che fa parte di una multirest
      // in tal caso  sempre la prima
     
      nb=pbat->GetNumMultiRest();
      numb+=nb;
    }
    else numb++;
    pn=pn->Next;
  } 
  return numb;   
}

/**		  MODIFICA NECESSARIA PER LIRICA               **/

Figura * ListaBattute::Symbolic(SymPath& symPath)
{


	Node *pn=first;
        // Puntatore ai nodi della lista necessario
        // per la scansione.
        Figura *fig=NULL;
        // Variabile temporanea.
  	
        // Scansione della lista di figure.
        while((pn!=NULL) && (fig==NULL))
        {
        // Fino a quando non  terminata la lista delle
        // figure  oppure quella corrente corrisponde a
        // quella cercata....
        	//cout << " ARRIVO IN LISTA BAT---->"<<   (((Battuta*)pn->ObjectPtr)->GetNumericCode())<< endl;
        	fig=(((Battuta*)pn->ObjectPtr)->Symbolic(symPath));
  	        // Verifica se la figura appartiene alla battuta
  	        // corrente e in tal caso restituisce l'indirizzo
  	        // fisico attuale.
      		pn=pn->Next;
      		// Passa alla successiva.	
    	};
      	return fig;
        // Se la figura non appartiene alla lista
        // ritorna NULL altrimenti l'indirizzo
        // fisico attuale.
};


/**			FINE                         **/

void ListaBattute::SetNStaffLine(int nstaffline)	//DB
{
	Battuta *ptr=(Battuta*)GetFirst();
	while (ptr!=NULL)
	{
	   ptr->SetNStaffLine(nstaffline);
	   ptr = (Battuta*)GetNext(ptr);
	}
}

void ListaBattute::GetPageMeasures(int& top, int& bottom)
{
	top=GetfirstBatPage()->GetNumProgress();
	if(GetfirstBatNextPage()!=NULL)
		bottom=GetfirstBatNextPage()->GetNumProgress()-1;
	else
		bottom=((Battuta*)GetLast())->GetNumProgress();
}

Node* ListaBattute::GetNextMeasure(Node *node)
{
	if(node==NULL)
		return NULL;
	node=node->Next;
	if(node!=NULL && ((Battuta*)node->ObjectPtr)->GetIsMultiRest())
		while(node!=NULL && ((Battuta*)node->ObjectPtr)->GetIsMultiRest() && ((Battuta*)node->ObjectPtr)->GetPMultiRest()==NULL)
			node=node->Next;
	return node;
}

Node* ListaBattute::GetPrevMeasure(Node *node)
{
	if(node==NULL)
		return NULL;
	node=node->Prev;
	if(node!=NULL && ((Battuta*)node->ObjectPtr)->GetIsMultiRest())
		while(node!=NULL && ((Battuta*)node->ObjectPtr)->GetIsMultiRest() && ((Battuta*)node->ObjectPtr)->GetPMultiRest()==NULL)
			node=node->Prev;
	return node;
}

Battuta* ListaBattute::GetNextMeasure(Battuta* bat)
{
	Node *temp=first;
	
	if (lastRef != NULL)
	{ 
		if (lastRef->ObjectPtr==bat)
		{  
			temp=lastRef;
		}  
	}
	while(temp)
	{
		if(temp->ObjectPtr==bat)
		{
			break;
		}
		temp=temp->Next;
	}
	
	Node* n=GetNextMeasure(temp);
	lastRef=n;
	if(n!=NULL)
		return (Battuta*)n->ObjectPtr;
	return NULL;
}

Battuta* ListaBattute::GetPrevMeasure(Battuta* bat)
{
	Node *temp=first;
	
	if (lastRef != NULL)
	{ 
		if (lastRef->ObjectPtr==bat)
		{  
			temp=lastRef;
		}  
	}
	while(temp)
	{
		if(temp->ObjectPtr==bat)
		{
			break;
		}
		temp=temp->Prev;
	}
	
	Node* n=GetPrevMeasure(temp);
	lastRef=n;
	if(n!=NULL)
		return (Battuta*)n->ObjectPtr;
	return NULL;
}

Battuta* ListaBattute::GetMeasure(Node *node)
{
	if(((Battuta*)node->ObjectPtr)->GetPMultiRest()!=NULL)
		return ((Battuta*)node->ObjectPtr)->GetPMultiRest();
	return (Battuta*)node->ObjectPtr;
}
	
