/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: listabat.hpp,v 1.1 2005/07/25 09:35:31 bellinpo Exp $ */


#ifndef LISTABATTUTE_HPP
#define LISTABATTUTE_HPP

#include <stdio.h>

#include "lioo.hpp"
#include "lst/lista.hpp"
#include "bat/battuta.hpp"
#include "spa/pentagra.hpp"

class MoodsMidiWrite;

class ListaBattute : public Lista
{
  protected:
    NumCode CodeCounter;
      // contatore per dotare ogni battuta di codice numerico
    NumCode PagCounter;
      // Contatore per numerare in maniera progressiva ogni pagina del brano.

    Node *firstBatPage[MAX_VIEW],*firstBatNextPage[MAX_VIEW];
      // puntatori alla prima battuta della pagina attuale e della pagina seguente.
      // firstBatNextPage e' NULL se e' l'ultima pagina

    Node* CurrentNode[MAX_VIEW];
    Intestazione* PrevIntest[MAX_VIEW];
    Node* SavedPosition;  // (MS)

	int nNullNodes;
    
    Node* CalcRigo(Node*,Pentagramma*,int rigo, Bool);
      // calcola le posizioni degli elementi sul rigo di pentagramma indicato
      // partendo dal nodo fornito e procedendo in avanti fino a che non 
      // termina lo spazio disponibile.
      // ritorna il puntatore alla prima battuta del rigo successivo se
      // non ci sono piu' battute ritorna NULL.
   

	Node* CalcRigoLineBreak(Node* frst,Pentagramma* pentagr,int nRig, Bool FirstColonna);
	// come sopra per margina a destra lo spartito del musicista singolo


    int StretchBattuta(Battuta* vbat, UL totale);
	// stira la la battuta rigiustificandola finche' la sua larghezza non 
	// uguale al target passato come parametro

    UL JustVerify(Battuta* vbat, double kGiust);
	// verifica giustificando quanto viene la larghezza di una battuta
	// con un certo parametro di giustificazione in ingresso

	Spartito *parentScore;	

	Node* GetNextMeasure(Node* node);
	Node* GetPrevMeasure(Node* node); 
	Battuta* GetMeasure(Node* node);

  private:
	  Node*& GetCurrentNode() { return CurrentNode[DrawObject::GetCurrentView()]; };
	  Node*& GetFirstBatPageNode() { return firstBatPage[DrawObject::GetCurrentView()]; };
	  Node*& GetFirstBatNextPageNode() { return firstBatNextPage[DrawObject::GetCurrentView()]; };
  public:
    ListaBattute(Spartito* psp=NULL);
    ~ListaBattute();
  
	void SetNStaffLine(int nstaffline);
	int GetNStaffLine();

    void JustifIfFirst(Battuta *bat);

    void Suona(MoodsMidiWrite *);

    void SetPagCounter(NumCode);
      // imposta $PagCounter$ al valore passato come parametro.
    NumCode GetPagCounter();
      // restituisce il valore $PagCounter$.
    void SetBattPage(Battuta*);
      // imposta la battuta passata come parametro come di 
      // inizio pagina con il numero di pagina appropriato.
    void SetfirstBatPage(Battuta *);
      // imposta il valore $firstBatPage$.
    Battuta *GetfirstBatPage();
      // restituisce il valore $firstBatPage$.
    Battuta *GetfirstBatNextPage();
      // restituisce il valore $firstBatNextPage$.

    void Init(Spartito* psp =NULL);
      // inizializza la lista battute con una battuta con chiave di violino,
      // in do maggiore e tempo 4/4.
    void Free();
      // disalloca la lista delle battute.

    void GoBatt(NumCode nb);
    void GoBattInExecution(NumCode nb);
      // si posiziona sulla battuta nb
    void GoTop(Pentagramma pent[],int nPent);
      // si posiziona sulla prima pagina.
    Bool GoForward();
      // scorre di una battuta in avanti.
    Bool GoBackward();
      // scorre di una battuta indietro.    

    Bool GoNextPage();
      // scorre di una pagina in avanti.
    Bool GoNextPageInExecution();
      // scorre di una pagina in avanti.
    Bool GoPrevPage();
      // scorre di una pagina in dietro.

    Bool PageUp(Pentagramma pent[],int nPent,Bool);
      // torna indietro di una pagina, ritorna FALSE se cio' non e' possibile.
    Bool PageDwn(Pentagramma pent[],int nPent,Bool);
      // avanza di una pagina, ritorna FALSE se cio' non e' possibile.

    void ResetCurrent(Node* SavedPos = NULL);

	void SaveCurrentPosition();

    Battuta* GetCurrentBat();
    // senza il controllo del next node posto a null
	Battuta* GetCurrentBatLst();
    Bool MoveCurrentBatForw();
    Bool CheckCurrent(tipoSpartito ts, VUnit& IntWdt, VUnit& Wdt,Bool firstColonna);
	void SetupCurrent(Pentagramma *pentagr, VUnit PosX,
		      VUnit IntWdt, VUnit MaxWdt,VUnit Batt2Up, VUnit Batt2Dwn,
		      tipoSpartito tsp, Bool ProlBarUp, Bool ProlBarDown, int sistema=0);
    void CurrentInNextPage();
//    void PageSetUpPart(Pentagramma *pentagr);
      // Imposta il rigo della pagina attuale della partitura.

    void DrawPageMainScore();
	void DrawPagePart();
      // disegna la pagina.
    void RedrawPage(Pentagramma pent[],int nPent,Bool);
      // ricalcola e ridisegna la pagina.
      
    Bool Hit(const Point&,SelObj,DrawObject*&,SymPath&);
      // identifica un oggetto.
    Bool Hit(const class Rectangle&,SelObj,short layer1,SymPath&,SymPath&);
      // aggiunge alla lista gli oggetti nel rettangolo del tipo indicato.
    Bool SymbolicHit(SymPath&,SelObj,DrawObject*&);
      // Decodifica un path.
    Figura *Symbolic(SymPath&);
      // Sulla base dell'indirizzo logico di una
      // figura verifica se questa appartiene ad
      // una battuta della lista restituendo in
      // tal caso il relativo(attuale) indirizzo
      // fisico della figura.Altrimenti ritorna
      // NULL:


    void SetupCurrentSimboli();  
    void SetupSimboli(Bool beforeLegature=FALSE);  
	
    void PageSetUp(Pentagramma pent[],int nPent,Bool);
      // imposta una pagina partendo da firstBatPage.
 
    Bool CmdAddBattuta(Battuta*,Bool insAfter,VUnit spc, Spartito* psp);
    Bool CmdAddBattuta(Battuta*,Bool insAfter,VUnit spc, ArgType& nc, ArgType& np, Spartito* psp);	//DB
      // aggiunge una nuova battuta dopo o prima della battuta indicata
      // secondo il valore di insAfter (TRUE=>Dopo). La intestazione della nuova battuta
      // viene inizializzata in modo da essere uguale a quella della battuta
      // precedente. 
      // Se il tipo di leggio e' MASAE vengono riportati i valori di NumericCode(nc) e 
      // NumProgress(np) della nuova battuta, altrimenti gli argomenti vengono utilizzati
      // per assegnare i nuovi valori di NumericCode e NumProgress alla battuta.
    Bool CmdDelBattuta(Battuta*);
      // cancella una battuta dalla lista delle battute.

    void Add(Battuta* bat);
      // Aggiunge la battuta passata come parametro in fondo alla lista.
    void AddAfter(Battuta* bat);
      // Aggiunge la battuta passata come parametro nella posizione 
      // specificata da NumProgress.
    
//    int Save(FILE *,Context);
      // registra la lista delle battute su file in formato MusicTEX.

    Bool DeleteSym(DrawObject*);
      // cancella il simbolo indicato dalla pagina attuale.
    Battuta *GetBatt(NumCode nc);
      // Ritorna la Battuta con NumericCode nc.
    Battuta* GetBattNumProg(NumCode np);
      // Ritorna la Battuta con numero progressivo np.
    Battuta *GetBattWithFig(Figura*);
      // Ritorna la Battuta che contiene la figura. Ritorna NULL se non la
      // trova.
    Bool FigPtr2SymPath(SymPath&, Figura*);
      // ricostruisce il PATH di una figura a partire dal puntatore.
    void SetCodeCounter(NumCode);
    NumCode GetCodeCounter();
    
    //DrawObject* PosComp(unsigned long pos, Bool forward, int &nb);
    void ComprimiBattute(Bool abil_compress);
    void CompressBatt(Battuta* pbat1, Battuta *pbat2, int nb);
    int GetNumeroBattuteVere();
	
	Bool printPage(Pentagramma[],int);
	void printPage();
	void ResetfirstBatNextPage();

	Spartito* GetParentScore() {return parentScore;}
	void SetParentScore(Spartito *parentscore) {parentScore=parentscore;}

	void GetPageMeasures(int& top, int& bottom);

	Battuta* GetNextMeasure(Battuta* bat);
	Battuta* GetPrevMeasure(Battuta* bat); 
	
    int SaveXML(FILE *);
};

#endif //LISTABATTUTE_HPP
