/* ISO_HEADER_START */

/*********************************************************************

 This software module was originally developed by
 
  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)
  
   in the course of development of the MPEG-4 standard.
   This software module is an implementation of a part of one or more
   MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
   of the MPEG-4 standard free license to this software module or
   modifications thereof for use in hardware or software products
   claiming conformance to MPEG-4. Those intending to use this software
   module in hardware or software products are advised that its use may
   infringe existing patents.  The original developer of this software
   module and his/her company, the subsequent editors and their
   companies, and ISO/IEC have no liability for use of this software
   module or modifications thereof in an implementation.  Copyright is
   not released for non MPEG-4 conforming products. The DISIT-DSI and 
   EXITECH retains full right to use the code for its own purpose,
   assign or donate the code to a third party and to inhibit third
   parties from using the code for non MPEG-4 conforming products.  This
   copyright notice must be included in all copies or derivative
   works. Copyright (c) 2005.
   
***********************************************************************/

/* ISO_HEADER_END */

/* $Id: listaint.cpp,v 1.1 2005/07/25 09:35:31 bellinpo Exp $ */


#include "spa/listaint.hpp"

#include "int/legatura.hpp"
#include "int/legatval.hpp"
#include "int/forcella.hpp"
#include "int/freccia.hpp"
#include "int/onda.hpp"
#include "int/trilloonda.hpp"
#include "int/pedalefinoa.hpp"
#include "int/raggruppamento.hpp"
#include "int/bend.hpp"
#include "int/legatqua.hpp"
#include "int/modifott.hpp"
#include "int/cambiori.hpp"
#include "fig/accordo.hpp"
#include "fig/nota.hpp"
#include "spa/spartito.hpp"
#include "drw/musicedt.hpp"

#define DELTAYINTERVALLI 2
#define DELTAX_FIGURA_INTERVALLO 2
#define WIDTH_RIPRESA_INTERV 20
#include "mil/immissione.hpp"
extern Immissione immissione;

ListaIntEst::ListaIntEst()
: LStartedSopra(), LStartedSotto()
{
	
	// Inizializza la lista.
	first=NULL;
	last=NULL;
	numobject=0;
	onlyLegature=TRUE;
	CodeCounter = 1;
	spartito=NULL;
	
}


ListaIntEst::~ListaIntEst()
{
	Free();
}

void ListaIntEst::Init()
{
	// Inizializza la lista.
	first=NULL;
	last=NULL;
	numobject=0;
}

void ListaIntEst::Free()
{
	ClearDeep(); 
}

void ListaIntEst::SetRevisionY(Bool ry)
{ RevisionY = ry; }


Node* ListaIntEst::GetIntStartF(Figura *pF, Node *pN, Bool Accordo)
{
	Node *pn = pN;
	Point p;
	IntEsteso *pInt;
	
	while(pn!=NULL)
	{
		pInt=(IntEsteso*)(pn->ObjectPtr);
		if (pInt->GetPFigStart()==pF)
		{
			return pn;
		}     
		pn = pn->Next;
	}
	return NULL;
}

Node* ListaIntEst::GetIntEndF(Figura *pF, Node *pN)
{
	Node *pn = pN;
	while(pn!=NULL)
	{
		if(((IntEsteso*)pn->ObjectPtr)->GetPFigEnd()==pF)
			return pn;
		pn = pn->Next;
	}
	return NULL;
}

void ListaIntEst::ValutaLegature()
{
	Node *pn = first;
	while(pn!=NULL)
	{
		(((Legatura*)pn->ObjectPtr)->ValutaLegature());
		pn = pn->Next;
	}
}

Bool ListaIntEst::AddLegVal(CmdID cmd, Figura *pF1, Figura *pF2, Battuta *pb1, Battuta *pb2,
							TipoInserimento SSA,
							ArgType Arg, ArgType& nc, int voice, Bool Multiv, int StemFirst, int StemLast)
{
	LegaturaValore *pInt=NULL;
	Figura *pf1, *pf2, *pf;
	Accordo *pacc;
	Bool flag;
	
	Bool Sopra=FALSE,autoSopra=FALSE;
	
	if (cmd!=CMD_SPARTITO_ADD_LEGATURA_VALORE)
		return FALSE;
	
	switch(SSA)
	{
    case SOPRA:
		Sopra=TRUE;
		break;
    case SOTTO:
		Sopra=FALSE;
		break;
    case AUTOMATICO:
		Sopra=autoSopra;
		break;
	}
	
	if (pF1->GetID()!=CL_ACCORDO && pF2->GetID()!=CL_ACCORDO)
	{ 
		pInt = new LegaturaValore;
		if (pInt!=NULL)
		{
			if((pF1->GetAltezza()==pF2->GetAltezza() && pF1->GetNStaff()==pF2->GetNStaff()) && (pInt->TestAndSet(pF1, pF2,
				Sopra)))
			{
				InsertEnd(pInt);				
#ifdef WITH_NET
				if (tipoLeggio==LEGGIO_MASAE)
					nc = CodeCounter++;
#else
				nc = CodeCounter++;
#endif
				
				pInt->SetNumericCode(nc);
				pInt->SetBattuta(1,pb1);
				pInt->SetBattuta(2,pb2);
				if(SSA==AUTOMATICO)
				{
					pInt->SetVoice(voice);
					pInt->SetMultivoice(Multiv);
					pInt->SetStemFirst(StemFirst);
					pInt->SetStemLast(StemLast);
					pInt->SetIntermed(FALSE); 
					pInt->SetStemInt(FALSE);
					((Legatura*)pInt)->ValutaLegature();
				}   
			}
			else
			{
				EM.Error(MUSICAL_ERR_AREA,"EC_FIG_NOTFOUND","ListaIntEst","Add()");
				delete pInt;
				return FALSE;
			}
		}
		else 
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
			return FALSE;
		}  	
	}
	else if ((pF1->GetID()==CL_ACCORDO) && (pF2->GetID()==CL_ACCORDO))
	{
		pf1 = ((Accordo *)pF1)->GetFirstNote();
		pf2 = ((Accordo *)pF2)->GetFirstNote();
		while (pf1!=NULL && pf2!=NULL)
		{
			if (pf1->GetAltezza() != pf2->GetAltezza() && pf1->GetNStaff()==pf2->GetNStaff())
			{
				EM.Error(MUSICAL_ERR_AREA,"EC_FIG_NOTFOUND","ListaIntEst","Add()");
				delete pInt;
				return FALSE;
			}	
			pf1 = ((Accordo *)pF1)->GetNextF(pf1);
			pf2 = ((Accordo *)pF2)->GetNextF(pf2);
		} 
		pf1 = ((Accordo *)pF1)->GetFirstNote();
		pf2 = ((Accordo *)pF2)->GetFirstNote();
		
		flag = TRUE;
		while (pf1!=NULL && pf2!=NULL)
		{
			pInt = new LegaturaValore;
			if (pInt!=NULL)
			{
				if(pInt->TestAndSet(pf1, pf2,Sopra))
				{
					InsertEnd(pInt);
					
#ifdef WITH_NET
					if (tipoLeggio==LEGGIO_MASAE)
						nc = CodeCounter++;
#else
					nc = CodeCounter++;
#endif
					
					pInt->SetNumericCode(nc);
					pInt->SetBattuta(1,pb1);
					pInt->SetBattuta(2,pb2);
					
					if (flag)
					{
						pInt->SetSopra(FALSE);
					}
					else    
					{
						if ( pf1->GetAltezza()>=4)
						{ 
							pInt->SetSopra(TRUE);
						}
						else pInt->SetSopra(FALSE);
					}   		
					
				}
			}
			else
			{
				EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
				return FALSE;
			} 	  
			pf1 = ((Accordo *)pF1)->GetNextF(pf1);
			pf2 = ((Accordo *)pF2)->GetNextF(pf2);
			flag = FALSE;
		}
		if (pInt!=NULL)
		{
			pInt->SetSopra(TRUE);
		} 
		else
		{
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
			return FALSE;
		}          
	} 
	else 
	{
		if (pF1->GetID()==CL_ACCORDO && pF2->GetID()!=CL_ACCORDO)
		{
			pacc=(Accordo *)pF1;
			pf=pF2;
		}
		else 
		{
			pacc=(Accordo *)pF2;
			pf=pF1;
		} 
		pf1 = pacc->GetFirstNote();
		while (pf1!=NULL)
		{
			if (pf1->GetAltezza() == pf->GetAltezza() && pf1->GetNStaff()==pf->GetNStaff())
			{
				pInt = new LegaturaValore;
				if (pInt!=NULL)
				{
					if(pInt->TestAndSet(pf1, pf,Sopra))
					{
						InsertEnd(pInt);
						
#ifdef WITH_NET
						if (tipoLeggio==LEGGIO_MASAE)
							nc = CodeCounter++;
#else
						nc = CodeCounter++;
#endif
						
						pInt->SetNumericCode(nc);
						pInt->SetBattuta(1,pb1);
						pInt->SetBattuta(2,pb2);
						if (SSA == AUTOMATICO)
						{
							if ( pf->GetAltezza()>=4)
							{ 
								pInt->SetSopra(TRUE);
							}
							else pInt->SetSopra(FALSE);
						}	 
					}
				}
				else
				{
					EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
					return FALSE;
				}  
			}	  	
			pf1 = pacc->GetNextF(pf1);
		}
	}
	return TRUE;
}




Bool ListaIntEst::Add(CmdID cmd, Figura *pF1, Figura *pF2, Battuta *pb1, Battuta *pb2,
					  TipoInserimento SSA, 
					  ArgType Arg, ArgType& nc, int voice, Bool Multiv, Bool StemInt, 
					  int StemFirst, int StemLast, Bool Interm, Bool DrawInt,LineType ltype)
{
	IntEsteso *pInt;
	Bool Sopra=FALSE,autoSopra=FALSE;
	bool Inserire=TRUE;
	int NumLayer;
	
	switch(cmd)
	{
	case CMD_SPARTITO_ADD_BEAM:	
		{
			
			DrawObject *CurrentInt = GetFirst();
			bool IsIn = FALSE;//controlla se  presente un raggruppamento a sovrapposto al nuovo
			
			Raggruppamento *raggr1, *raggr2;
			
			///////calcola il numero di layer
			
			class Layer *tmp;
			Bool Trovato=FALSE;
			short i;
			
			for(i=0; (i< NUM_MAX_LAYER); i++)
			{
				tmp = (pb1)->GetPLayer(i);  
				if(tmp!=NULL && tmp->FindFig(pF1))
				{
					NumLayer = i;
					break;
				}
			}
			
			//CONTROLLO SE I LAYERS SONO UGUALI else CMD_FAIL
			for(i=0; (i< NUM_MAX_LAYER); i++)
			{
				tmp = (pb2)->GetPLayer(i);  
				if(tmp!=NULL && tmp->FindFig(pF2))
				{
					if (i != NumLayer)
						return CMD_FAIL;
					break;
				}
			}
			//////////////////////////////////////////////
			raggr1 =NULL;
			raggr2 =NULL;
			while ((CurrentInt!=NULL)/*&&(IsIn!=TRUE)*/){
				if(CurrentInt->GetID()==CL_RAGGRUPPAMENTO){
					if(((Raggruppamento*)CurrentInt)->IsInRaggruppamento(pF1)){
						//((Raggruppamento*)CurrentInt)->SetLastFigura(pF2);
						pF1=((Raggruppamento*)CurrentInt)->GetPFigStart();
						pb1=pF1->GetPBattuta();
						IsIn=TRUE;
						//pInt=((Raggruppamento*)Cu rrentInt);
						raggr1=((Raggruppamento*)CurrentInt);
						Inserire=FALSE;
					}
					if (((Raggruppamento*)CurrentInt)->IsInRaggruppamento(pF2)){
						//((Raggruppamento*)CurrentInt)->SetFirstFigura(pF1);
						pF2=((Raggruppamento*)CurrentInt)->GetPFigEnd();
						pb2=pF2->GetPBattuta();
						IsIn=TRUE;
						//pInt=((Raggruppamento*)CurrentInt);
						raggr2=((Raggruppamento*)CurrentInt);
						Inserire=FALSE;
					}
				}
				CurrentInt=this->GetNext(CurrentInt);
			}
			//Figura *nextfig;
			
			
			
			if (IsIn==FALSE)
			{
				pInt = new Raggruppamento(); 
				((Raggruppamento*)pInt)->SetFirstFigura(pF1);
				((Raggruppamento*)pInt)->SetLastFigura(pF2);
				((Raggruppamento*)pInt)->SetFirstBattuta(pb1);
				((Raggruppamento*)pInt)->SetLastBattuta(pb2);
			}
			else if ((raggr1!=NULL)&&(raggr2==NULL)){
				pInt=raggr1;
				((Raggruppamento*)pInt)->SetLastFigura(pF2);
			}
			else if ((raggr1==NULL)&&(raggr2!=NULL)){
				pInt=raggr2;
				((Raggruppamento*)pInt)->SetFirstFigura(pF1);
			}
			else if ((raggr1!=NULL)&&(raggr2!=NULL)&&(raggr1!=raggr2)){
				pInt=raggr1;
				//raggr2->DeleteRaggruppamento();
				DeleteSym((DrawObject*)raggr2);
			}
			else
				pInt=raggr1;
			
			if (pInt==NULL)
				EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
			else{
				switch(SSA)
				{
				case AUTOMATICO: 	 //ImmRule rende un bool !!! come funziona lo switch ??? da errore in compilazione
					switch (immissione.ImmRule(CL_RAGGRUPPAMENTO,(Nota*)pF1, pb1, (DrawObject*)pInt, NumLayer))//mettere il current layer
					{
					case TRUE:
						((Raggruppamento*)pInt)->SetBeamPosition(Raggruppamento::BEAM_UP);//GB
						break;
					case FALSE:
						((Raggruppamento*)pInt)->SetBeamPosition(Raggruppamento::BEAM_DWN);//GB
						break;
					case 2:
						((Raggruppamento*)pInt)->SetBeamPosition(Raggruppamento::BEAM_12);//GB
						break;
					case 3:
						((Raggruppamento*)pInt)->SetBeamPosition(Raggruppamento::BEAM_23);//GB
						break;
					default:
						((Raggruppamento*)pInt)->SetGamboUp();//GB di comodo
					}   
					break; 
					case SOPRA:
						((Raggruppamento*)pInt)->SetGamboUp();
						break;
					case SOTTO:
						((Raggruppamento*)pInt)->SetGamboDwn();
						break; 
						
				} 
			}
			autoSopra=FALSE;
			
	}
    break; 
	case CMD_SPARTITO_ADD_LEGATURA:
		pInt = new Legatura;
		if (pInt==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
		((Legatura*)pInt)->SetLineType(ltype);
		break;	  
	case CMD_SPARTITO_ADD_FORC_CRESC:  
		pInt = new Forcella;
		if (pInt==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
		((Forcella*)pInt)->SetCrescendo(TRUE);
		autoSopra=FALSE;
		((Forcella*)pInt)->SetLineType(ltype);
		break;
	case CMD_SPARTITO_ADD_FORC_DIMIN:  
		pInt = new Forcella;
		if (pInt==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
		((Forcella*)pInt)->SetCrescendo(FALSE);
		((Forcella*)pInt)->SetLineType(ltype);
		autoSopra=FALSE;
		break;
	case CMD_SPARTITO_ADD_FRECCIA:  
		pInt = new Freccia;
		if (pInt==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
		autoSopra=TRUE;
		break;
	case CMD_SPARTITO_ADD_ONDA:  
		pInt = new Onda;
		if (pInt==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
		autoSopra=TRUE;
		break;            
	case CMD_SPARTITO_ADD_LEG_QUADRA:  
		pInt = new LegatQuadra;
		if (pInt==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
		((LegatQuadra *)pInt)->SetNumero((int)Arg);
		((LegatQuadra *)pInt)->SetSenzaLinea(!DrawInt);
		pF1->SetTerzina((LegatQuadra *)pInt);
		pF2->SetTerzina((LegatQuadra *)pInt);
		
		autoSopra=TRUE;
		break;
	case CMD_SPARTITO_ADD_CAMBIO_RIT:  
		pInt = new CambioRit;
		if (pInt==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
		((CambioRit *)pInt)->SetText((unsigned char)Arg);
		autoSopra=TRUE;
		break;
	case CMD_SPARTITO_ADD_MODIF8:  // ottava alta
		pInt = new ModifOttava;
		if (pInt==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
		autoSopra=TRUE; 
		((ModifOttava*)pInt)->SetTipo8(VA8);
		break;
	case CMD_SPARTITO_ADD_MODIF8BA: // ottava bassa
		pInt = new ModifOttava;
		if (pInt==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
		autoSopra=FALSE; 
		((ModifOttava*)pInt)->SetTipo8(BA8);
		break;
	case CMD_SPARTITO_ADD_MODIF15MA: //15 esima alta
		pInt = new ModifOttava;
		if (pInt==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
		autoSopra=TRUE; 
		((ModifOttava*)pInt)->SetTipo8(MA15);
		break;    
	case CMD_SPARTITO_ADD_MODIF15BA: // 15 bassa
		pInt = new ModifOttava;
		if (pInt==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
		autoSopra=FALSE; 
		((ModifOttava*)pInt)->SetTipo8(BA15);
		break;
	case CMD_SPARTITO_ADD_TRILLOONDA:  
		pInt = new TrilloOnda;
		if (pInt==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
		autoSopra=TRUE;
		break;
	case CMD_SPARTITO_ADD_BEND:  
		pInt = new Bend;
		if (pInt==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
		autoSopra=TRUE;
		((Bend*)pInt)->SetLineType(ltype);
		break;
	case CMD_SPARTITO_ADD_PEDALE:  
		pInt = new PedaleFinoA;
		if (pInt==NULL)
			EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","ListaIntEst","Add()");
		autoSopra=FALSE;
		break; 
	}
	switch(SSA)
	{
    case SOPRA:
		Sopra=TRUE;
		break;
	case SOTTO:
		Sopra=FALSE;
		break;
	case AUTOMATICO:
		Sopra=autoSopra;
		break;
	}
	if(pInt->TestAndSet(pF1, pF2, Sopra))
	{
		if(Inserire)
			InsertEnd(pInt);
		
#ifdef WITH_NET
		if (tipoLeggio==LEGGIO_MASAE)
			nc = CodeCounter++;
#else
		nc = CodeCounter++;
#endif
		
		pInt->SetNumericCode(nc);
		pInt->SetBattuta(1,pb1);
		pInt->SetBattuta(2,pb2);
		if(SSA==AUTOMATICO && 
			(pInt->GetID()==CL_LEGATURA || pInt->GetID()==CL_LEGATURA_QUADRA))
		{
			pInt->SetVoice(voice);
			pInt->SetMultivoice(Multiv);
			pInt->SetStemInt(StemInt);
			pInt->SetStemFirst(StemFirst);
			pInt->SetStemLast(StemLast);
			pInt->SetIntermed(Interm);
			pInt->ValutaLegature();
			if(cmd==CMD_SPARTITO_ADD_LEG_QUADRA)
			{
				if(Battuta::GetAutoBeaming())
					pb1->BeamingAuto();
			}
		}
		return TRUE;
	}
	else
	{
		EM.Error(MUSICAL_ERR_AREA,"EC_FIG_NOTFOUND","ListaIntEst","Add()");
		delete pInt;
		return FALSE;
	}
}

void ListaIntEst::Add(IntEsteso *pInt, ArgType& nc)
{
	Figura *pF1,*pF2;
	Bool Sopra;
	
	Sopra = pInt->GetSopra();
	pF1 = pInt->GetPFigStart();
	pF2 = pInt->GetPFigEnd();
	
	if(pInt->TestAndSet(pF1, pF2, Sopra))
	{
		InsertEnd(pInt);
#ifdef WITH_NET
		if (tipoLeggio==LEGGIO_MASAE)
			nc = CodeCounter++;
#else
		nc = CodeCounter++;
#endif
		
		pInt->SetNumericCode(nc);
		if(pInt->GetID()==CL_LEGATURA_QUADRA)
		{
			pF1->SetTerzina((LegatQuadra*)pInt);
			pF2->SetTerzina((LegatQuadra*)pInt);
		}  
	}
	else
	{
		EM.Error(MUSICAL_ERR_AREA,"EC_FIG_NOTFOUND","ListaIntEst","Add()");
		delete pInt;
	} 
}


void ListaIntEst::Add(IntEsteso *pInt)
{
	ArgType nc = 1;
    // questa funzione deve essere utilizzata solo da metodi
    // che si occupano di risettare in maniera opportuna il
    // numeric code ed i numeri di battuta
    
    // al momento viene utilizzata solo dal
    // motore di caricamento da file e da stringa da rete
	
	Add(pInt, nc);
	
}


void ListaIntEst::AddStartedSopra(IntEsteso *pInt)
{
	
	IntEsteso *pIntPrec, *pIntRef;
	
	pIntPrec = NULL;
	pIntRef = (IntEsteso*)LStartedSopra.GetFirst();
	while(pIntRef!=NULL)
	{
		if ((pInt->GetID()) <= (pIntRef->GetID()))
			pIntRef = NULL;
		else
		{
			pIntPrec = pIntRef;
			pIntRef = (IntEsteso*)LStartedSopra.GetNext(pIntRef);
		}
	}
	
	LStartedSopra.InsertAfter(pInt, pIntPrec);
}

void ListaIntEst::AddStartedSotto(IntEsteso *pInt)
{
	
	IntEsteso *pIntPrec, *pIntRef;
	
	pIntPrec = NULL;
	pIntRef = (IntEsteso*)LStartedSotto.GetFirst();
	while(pIntRef!=NULL)
	{
		if ((pInt->GetID()) <= (pIntRef->GetID()))
			pIntRef = NULL;
		else
		{
			pIntPrec = pIntRef;
			pIntRef = (IntEsteso*)LStartedSotto.GetNext(pIntRef);
		}
	}
	
	LStartedSotto.InsertAfter(pInt, pIntPrec);
}

void ListaIntEst::AddStarted(IntEsteso *pInt)
{
	
	if((onlyLegature && !pInt->IsLegOrBend())||
		(!onlyLegature && pInt->IsLegOrBend()))
		return;
	
	
	if(pInt->GetSopra())
		AddStartedSopra(pInt);
	else
		AddStartedSotto(pInt);
}


// ridispone tutti gli intervalli uno sopra l'altro assicurandosi che siano
// tutti sopra P.y.
// Se e' la prima passata:
// Aggiorna la larghezza degli intervalli  (e implicitamente l'altezza, in 
// quanto l'altezza e' calcolata nei metodi SetDim degli intervalli) in base al
// valore P.x.
void ListaIntEst::UpdateStartedSopra(const Point& Plv, const Point& P, 
									 VUnit YPent, Bool fine, Figura* pF, Bool inAccordo)
{
	
	IntEsteso *pInt;
	VUnit y, y1, ylv;
	
	y = YPent + spartito->GetBaseVU2Up();
	
	y1 = P.y;
	if(y < y1) y = y1;
	y += DELTAYINTERVALLI;
	
	ylv = YPent + spartito->GetBaseVU2Up();
	
	y1 = Plv.y;
	if(ylv < y1) ylv = y1;
	ylv += DELTAYINTERVALLI;
	
	// Riposiziona i simboli sopra la figura, nell'ordine della lista.
	pInt = (IntEsteso*)LStartedSopra.GetFirst();
	while(pInt!=NULL)
	{
		if((onlyLegature && pInt->IsLegOrBend()) ||
			(!onlyLegature && !pInt->IsLegOrBend()))
		{
			if (pInt->GetID()==CL_LEGATURA_VALORE)
			{
				if(pInt->GetAbsPos().y < ylv) pInt->SetAbsPosY(ylv);
				else ylv = pInt->GetAbsPos().y;
				// Calcola l'altezza (provvisoria) di *pInt.
				// pInt->SetHeight(4);
				if ((!RevisionY) && (!pInt->GetRemove()))
				{
					pInt->SetDim(Plv.x - pInt->GetAbsPos().x);
					pInt->SetFine(fine);
				}
				
				
				ylv += pInt->GetHeight() + DELTAYINTERVALLI;
				if (y < ylv) y = ylv;
			}
			else
			{
				if(pInt->GetAbsPos().y < y) pInt->SetAbsPosY(y);
				else y = pInt->GetAbsPos().y;
				
				if ((!RevisionY) && (!pInt->GetRemove()))
				{
					pInt->SetDim(P.x - pInt->GetAbsPos().x);
					pInt->SetFine(fine);
				}
				
				if(pInt->GetID()!=CL_LEGATURA)
					y += pInt->GetHeight() + DELTAYINTERVALLI;
				else
					y += DELTAYINTERVALLI;
			}
			if(pF!=NULL && !inAccordo && (!RevisionY) && (!pInt->GetRemove()))
				pInt->UpdateGambiInterni(pF);
		}
		
		pInt = (IntEsteso*)LStartedSopra.GetNext(pInt);
	}
}

void ListaIntEst::UpdateStartedSopra(Figura *pF, VUnit YPent, Bool inAccordo, Bool fine)
{
	Point p, pLegVal;
	
	p = pF->GetAbsPos() + Point(pF->GetWidth()/2 - DELTAX_FIGURA_INTERVALLO,
		pF->GetVU2Up());
	pLegVal = pF->GetAbsPos() + Point(- VU2LFTNOTA - DELTAX_FIGURA_INTERVALLO, 
		-VU2DWNNOTA);
	
	UpdateStartedSopra(pLegVal, p, YPent, fine, pF, inAccordo);
	
}

void ListaIntEst::UpdateStartedSotto(const Point& Plv, const Point& P, 
									 VUnit YPent, Bool fine, Figura* pF, Bool inAccordo)
{
	IntEsteso *pInt;
	VUnit y, y1, ylv;
	
	y = YPent - spartito->GetBaseVU2Dwn();
	y1 = P.y;
	if(y > y1) y = y1;
	
	ylv = YPent - spartito->GetBaseVU2Dwn();
	
	y1 = Plv.y;
	if(ylv > y1) ylv = y1;
	
	// Riposiziona i simboli sotto la figura, nell'ordine della lista.
	pInt = (IntEsteso*)LStartedSotto.GetFirst();
	while(pInt!=NULL)
	{
		if((onlyLegature && pInt->IsLegOrBend()) ||
			(!onlyLegature && !pInt->IsLegOrBend()))
		{
			if (pInt->GetID()==CL_LEGATURA_VALORE)
			{
				if ((!RevisionY) && (!pInt->GetRemove()))
				{
					pInt->SetDim(Plv.x - pInt->GetAbsPos().x);
					pInt->SetFine(fine);
				}
				
				ylv -= pInt->GetHeight() + DELTAYINTERVALLI;
				if(pInt->GetAbsPos().y > ylv) pInt->SetAbsPosY(ylv);
				else ylv = pInt->GetAbsPos().y;
				if (y > ylv) y = ylv;
			}
			else
			{
				if ((!RevisionY) && (!pInt->GetRemove()))
				{
					pInt->SetDim(P.x - pInt->GetAbsPos().x);
					pInt->SetFine(fine);
				}
				
				if(pInt->GetID()!=CL_LEGATURA)
					y -= pInt->GetHeight() + DELTAYINTERVALLI;
				else
					y -= DELTAYINTERVALLI;
				if(pInt->GetAbsPos().y > y) pInt->SetAbsPosY(y);
				else y = pInt->GetAbsPos().y;
			}
			
			if(pF!=NULL && !inAccordo && (!RevisionY) && (!pInt->GetRemove()))
				pInt->UpdateGambiInterni(pF);
			
		}
		
		pInt = (IntEsteso*)LStartedSotto.GetNext(pInt);
	}
}

void ListaIntEst::UpdateStartedSotto(Figura *pF, VUnit YPent, Bool inAccordo, Bool fine)
{
	Point p, pLegVal;
	
	p = pF->GetAbsPos() + 
		Point(pF->GetWidth()/2 - DELTAX_FIGURA_INTERVALLO, -(pF->GetVU2Dwn()));
	
	pLegVal = pF->GetAbsPos() + Point(- VU2LFTNOTA - DELTAX_FIGURA_INTERVALLO, 
		+ VU2DWNNOTA);
	
	UpdateStartedSotto(pLegVal, p, YPent, fine, pF, inAccordo);
	
}

void ListaIntEst::UpdateStarted(Figura *pF, VUnit YPent, Bool inAccordo, Bool fine)
{
	UpdateStartedSopra(pF, YPent, inAccordo, fine);
	UpdateStartedSotto(pF, YPent, inAccordo, fine);
}


void ListaIntEst::RemoveStarted(IntEsteso *pInt)
{
	IntEsteso *p_Int, *p_Int2;
	
	if((onlyLegature && !pInt->IsLegOrBend())||
		(!onlyLegature && pInt->IsLegOrBend()))
		return;
	
	pInt->SetRemove(TRUE);
	
	if(pInt->GetSopra())
	{
		p_Int = (IntEsteso*)LStartedSopra.GetFirst();
		while(p_Int!=NULL)
		{
			if (p_Int->GetRemove())
			{
				p_Int2 = (IntEsteso*)LStartedSopra.GetNext(p_Int);
				LStartedSopra.DelPtr(p_Int);
				p_Int = p_Int2;
			}
			else
				p_Int = NULL;
		}
	}
	else
	{
		p_Int = (IntEsteso*)LStartedSotto.GetFirst();
		while(p_Int!=NULL)
		{
			if (p_Int->GetRemove())
			{
				p_Int2 = (IntEsteso*)LStartedSotto.GetNext(p_Int);
				LStartedSotto.DelPtr(p_Int);
				p_Int = p_Int2;
			}
			else
				p_Int = NULL;
		}
	}
    
}

void ListaIntEst::ClearStarted()
{
	LStartedSopra.Clear();
	LStartedSotto.Clear();
}


void ListaIntEst::ResetPos()
{
	Node *pn = first; 
	IntEsteso *pInt;
	
	while(pn!=NULL)
	{
		pInt = (IntEsteso*)(pn->ObjectPtr);
		pInt->DeallocaSegm();
		pInt->SetRemove(FALSE);
		pInt->SetInizio(FALSE);
		pn = pn->Next;
	}
}


void ListaIntEst::FindPrevStartedFig(Figura *pF)
{
	IntEsteso *pInt;
	Node *pn;
	
    // Trova gli intervalli che iniziano con *pF.
    // gli alloca Segm e non li mette negli Started
	pn = GetIntStartF(pF,first,FALSE);
	while(pn!=NULL)
	{
		pInt = (IntEsteso*)pn->ObjectPtr;
		
		if((onlyLegature && pInt->IsLegOrBend()) ||
			(!onlyLegature && !pInt->IsLegOrBend()))
		{
			if((pInt->GetPFirstSegm())==NULL)
				pInt->AddSegm();
		}
		pn = GetIntStartF(pF, pn->Next,FALSE);
	}
	
    // Trova gli intervalli che terminano con *pF.
    // se hanno PFirstSegm == NULL allora li metto negli Started
	pn = GetIntEndF(pF,first);
	while(pn!=NULL)
	{
		pInt = (IntEsteso*)pn->ObjectPtr;
		
		if((onlyLegature && pInt->IsLegOrBend()) ||
			(!onlyLegature && !pInt->IsLegOrBend()))
		{
			if((pInt->GetPFirstSegm())==NULL)
			{
				pInt->AddSegm();
				AddStarted(pInt);
			}
		}
		pn = GetIntEndF(pF, pn->Next);
	}
}


// trova gli intervalli che iniziano nelle pagine precedenti e che terminano in
// questa pagina e li mette negli Started
void ListaIntEst::FindPrevStartedBat(Battuta *pBat)
{
	ListaFigure *pLayer;
	Figura *pF, *pFNotaGruppo, *pFNotaAcc;
	short i;
	
	// Scandisce i layer
	for(i=0; i<NUM_MAX_LAYER; i++)
	{
		pLayer = pBat->GetPLayer(i);
		if(pLayer!=NULL)
		{
			pF = pLayer->GetFirstF();
			while(pF!=NULL)
			{
				switch(pF->GetID())
				{
				case CL_GRUPPONOTE:
					pFNotaGruppo = ((GruppoNote*)pF)->GetFirstF();
					while(pFNotaGruppo!=NULL)
					{
						if(pFNotaGruppo->GetID()==CL_ACCORDO)
						{
							pFNotaAcc = ((Accordo*)pFNotaGruppo)->GetFirstF();
							while(pFNotaAcc!=NULL)
							{
								FindPrevStartedFig(pFNotaAcc);
								pFNotaAcc = ((Accordo*)pFNotaGruppo)->GetNextF(pFNotaAcc);
							}
						}
						FindPrevStartedFig(pFNotaGruppo);
						pFNotaGruppo = ((GruppoNote*)pF)->GetNextF(pFNotaGruppo);
					}
					break;
					
				case CL_ACCORDO:
					pFNotaAcc = ((Accordo*)pF)->GetFirstF();
					while(pFNotaAcc!=NULL)
					{
						FindPrevStartedFig(pFNotaAcc);
						pFNotaAcc = ((Accordo*)pF)->GetNextF(pFNotaAcc);
					}
					FindPrevStartedFig(pF);
					break;
				default:
					FindPrevStartedFig(pF);
				}
				pF = pLayer->GetNextF(pF);
			}
		}
	}
}


//
// Esamina la figura pF controllando gli intervalli che terminano o iniziano
// in pF:
//	quelli che terminano in pF, vengono tolti dagli Started;
//	quelli che iniziano in pF, vengono messi negli Started (fino ad ora non
// 		c'erano negli Started).
//
void ListaIntEst::ExamineFig(Figura *pF, VUnit YPent,Bool Accordo, Bool inAccordo)
{
	IntEsteso *pInt;
	Node *pn;
	Bool SecondUpdate;
	
	
    // Aggiorna i simboli started in modo da sovrastare o sottostare
    // questa figura. 
    // A questo punto del ciclo, tutti gli intervalli negli Started, "coprono"
    // questa figura, per cui si deve aggiornare la posizione y degli Started 
    // in modo da non intersecare la figura.
    // Nella prima passata, inoltre, calcola anche altezza e larghezza
    // dell'intervallo.
	UpdateStarted(pF, YPent,inAccordo);
	
    // Toglie dagli Started gli intervalli che terminano con *pF.
	pn = GetIntEndF(pF,first);
	while(pn!=NULL)
	{
		pInt = (IntEsteso*)pn->ObjectPtr;
		if((onlyLegature  && (pInt->IsLegOrBend())) ||
			(!onlyLegature && (!pInt->IsLegOrBend())))
		{
			RemoveStarted(pInt);
		}
		pn = GetIntEndF(pF, pn->Next);
	}
	
    // Trova gli intervalli che iniziano con *pF e li aggiunge negli
    // Started. 
    // Nella prima passata: Aggiorna il valore x degli intervalli con quello 
    // di pF
	pn = GetIntStartF(pF,first,Accordo);
	SecondUpdate = pn!=NULL;
	while(pn!=NULL)
	{
		pInt = (IntEsteso*)pn->ObjectPtr;
		
		if((onlyLegature && pInt->IsLegOrBend())||
			(!onlyLegature && !pInt->IsLegOrBend()))
		{   
			AddStarted(pInt);
			if(!RevisionY)
			{
				if (pInt->GetID()==CL_LEGATURA_VALORE)
					pInt->SetAbsPosX(pF->GetAbsPos().x + pF->GetVU2Rgt()+DELTAX_FIGURA_INTERVALLO);
				else
					pInt->SetAbsPosX(pF->GetAbsPos().x );
				
				pInt->SetInizio(TRUE);
				
				// I simboli StartedSotto vengono posizionati dall'alto verso il
				// basso, percio` l'inizializzazione a 0 della ordinata va aggiornata
				// con inizializzazione a YPent.
				if(!pInt->GetSopra())
					pInt->SetAbsPosY(YPent);
			}
		}
		pn = GetIntStartF(pF, pn->Next,Accordo);
	}
	
    // Se esiste un intervallo che inizia su questa figura occorre risistemare
    // tutti gli started.
	if(SecondUpdate)
		UpdateStarted(pF, YPent,inAccordo);
}


void ListaIntEst::ExamineInizioPentagr(Battuta *pBat)
{
	IntEsteso *pInt;
	tipoBattuta tpBat = pBat->GetBattuta();
	Figura *pF;
	VUnit x;
	
	if((tpBat==INIZIO_PAGINA) || (tpBat==INIZIO_PENTAGRAMMA) || 
		(tpBat==INIZIO_BRANO))
	{
		// Calcola l'ascissa di partenza del simbolo di intervallo (ad
		// inizio battuta).
		pF = pBat->GetPLayer(0)->GetFirstF();
		if(pF!=NULL)
			x = pF->GetAbsPos().x - WIDTH_RIPRESA_INTERV;
		else
			x = pBat->GetAbsPos().x + pBat->GetIntest()->GetWidth();
		
		// Riprendi gli started.
		pInt = (IntEsteso*)LStartedSopra.GetFirst();
		while(pInt!=NULL)
		{
			if((onlyLegature && pInt->IsLegOrBend()) ||
				(!onlyLegature && !pInt->IsLegOrBend()))
			{
				pInt->SetAbsPosX(x);
			}
			pInt = (IntEsteso*)LStartedSopra.GetNext(pInt);
		}
		pInt = (IntEsteso*)LStartedSotto.GetFirst();
		while(pInt!=NULL)
		{
			if((onlyLegature && pInt->IsLegOrBend()) ||
				(!onlyLegature && !pInt->IsLegOrBend()))
			{
				pInt->SetAbsPosX(x);
				// Poiche' gli StartedSotto vengono posizionati verso il basso, la
				// loro Y va reinizializzata.
				pInt->SetAbsPosY(pBat->GetAbsPos().y);
			}
			pInt = (IntEsteso*)LStartedSotto.GetNext(pInt);
		}
	}
}


//
// Esamina la battuta e per ogni figura pF controlla gli intervalli che 
// terminano o iniziano in pF:
//	quelli che terminano in pF, vengono tolti dagli Started;
//	quelli che iniziano in pF, vengono messi negli Started (fino ad ora non
// 		c'erano negli Started).
//
void ListaIntEst::ExamineBat(Battuta *pBat)
{
	ListaFigure *pLayer;
	Figura *pF, *pFNotaGruppo, *pFNotaAcc;
	VUnit YPentagr = pBat->GetAbsPos().y;
	short i;
	
    //  se siamo ala prima passata, 
    //  e se (tpBat==INIZIO_PAGINA || tpBat==INIZIO_PENTAGRAMMA)
    //  posiziona la x degli Started in cima a sinistra
    //  Alla seconda passata gli Started sono stati cancellati (non ci sono
    // piu')!
	if(!RevisionY)
		ExamineInizioPentagr(pBat);
    
    // ExamineFig per tutte le figure della battuta:
	for(i=0; i<NUM_MAX_LAYER; i++)
	{
		pLayer = pBat->GetPLayer(i);
		if(pLayer!=NULL)
		{
			pF = pLayer->GetFirstF();
			while(pF!=NULL)
			{
				switch(pF->GetID())
				{
				case CL_GRUPPONOTE:
					pFNotaGruppo = ((GruppoNote*)pF)->GetFirstF();
					while(pFNotaGruppo!=NULL)
					{
						if(pFNotaGruppo->GetID()==CL_ACCORDO)
						{
							Figura *pFNotaAcc = ((Accordo*)pFNotaGruppo)->GetFirstF();
							while(pFNotaAcc!=NULL)
							{
								ExamineFig(pFNotaAcc, YPentagr,TRUE,TRUE);
								pFNotaAcc = ((Accordo*)pFNotaGruppo)->GetNextF(pFNotaAcc);
							}
							ExamineFig(pFNotaGruppo, YPentagr,FALSE,FALSE);
						}
						else
							ExamineFig(pFNotaGruppo, YPentagr,TRUE,FALSE);
						pFNotaGruppo = ((GruppoNote*)pF)->GetNextF(pFNotaGruppo);
					}
					break;
					
				case CL_ACCORDO:
					pFNotaAcc = ((Accordo*)pF)->GetFirstF();
					while(pFNotaAcc!=NULL)
					{
						ExamineFig(pFNotaAcc, YPentagr,TRUE,TRUE);
						pFNotaAcc = ((Accordo*)pF)->GetNextF(pFNotaAcc);
					}
					ExamineFig(pF, YPentagr,FALSE,FALSE);
					break;
					
				default:
					ExamineFig(pF, YPentagr,FALSE,FALSE);
				}
				pF = pLayer->GetNextF(pF);
			}
		}
	}
	
}


void ListaIntEst::EndPage(Battuta *pB)
{
	IntEsteso* pInt;
	Node *pn = first; 
	
    // Concludi gli Started sulla barra destra della battuta.
    // calcola la larghezza e altezza.
	UpdateStartedSopra(pB->GetBarra()->GetAbsPos() - Point(2,0), 
		pB->GetBarra()->GetAbsPos()- Point(2,0), 
		pB->GetAbsPos().y, FALSE);
	UpdateStartedSotto(pB->GetBarra()->GetAbsPos() - Point(2,0), 
		pB->GetBarra()->GetAbsPos() - Point(2,0), 
		pB->GetAbsPos().y, FALSE);
	
    // cancella gli Started. 
    // Ormai gli started sono stati tutti posizionati, quindi si possono
    // cancellare. Gli Intervalli che iniziano nella pagina saranno
    // riposizionati lungo y nella successiva passata.
    // Quelli che erano iniziati nella pagina precedente non vengono
    // considerati.
	ClearStarted();
	
	pn=first;
	
    // Riposiziona i segmenti di tutti gli intervalli da disegnare.
	while(pn!=NULL)
	{
		pInt = (IntEsteso*)(pn->ObjectPtr);
		if((onlyLegature && pInt->IsLegOrBend()) ||
			(!onlyLegature && !pInt->IsLegOrBend()))
		{
			if (pInt->GetPFirstSegm()!=NULL) pInt->ResetPosSegm();
		}
		pn = pn->Next;
	}
	
}

// Solo per DLIOO: 
void ListaIntEst::EndRow(Battuta *pB)
{
	IntEsteso* pInt, *pInt2;
	
    // Concludi gli Started sulla barra destra della battuta.
    // calcola la larghezza e altezza.
	UpdateStartedSopra(pB->GetBarra()->GetAbsPos(), pB->GetBarra()->GetAbsPos(), 
		pB->GetAbsPos().y);
	UpdateStartedSotto(pB->GetBarra()->GetAbsPos(), pB->GetBarra()->GetAbsPos(), 
		pB->GetAbsPos().y);
	
    // Alloca i segmenti per proseguire gli intervalli sui successivi
    // righi.
	pInt = (IntEsteso*)LStartedSotto.GetFirst();
	while(pInt!=NULL)
	{
		pInt2 = (IntEsteso*)LStartedSotto.GetNext(pInt);
		if((onlyLegature && pInt->IsLegOrBend()) ||
			(!onlyLegature && !pInt->IsLegOrBend()))
		{
			
			if (pInt->GetRemove())
				LStartedSotto.DelPtr(pInt);
			else
			{
				if(RevisionY) pInt->GoNextActiveSegm();
				else pInt->AddSegm();
			}
		}
		pInt = pInt2;
	}
	
	pInt = (IntEsteso*)LStartedSopra.GetFirst();
	while(pInt!=NULL)
	{
		pInt2 = (IntEsteso*)LStartedSopra.GetNext(pInt);
		if((onlyLegature && pInt->IsLegOrBend()) ||
			(!onlyLegature && !pInt->IsLegOrBend()))
		{
			
			if (pInt->GetRemove())
				LStartedSopra.DelPtr(pInt);
			else
			{
				if(RevisionY) pInt->GoNextActiveSegm();
				else pInt->AddSegm();
			}
		}
		pInt = pInt2;
	}
}

void ListaIntEst::Draw()
{
	Node *pn = first;
	while(pn!=NULL)
	{
		if (((IntEsteso*)pn->ObjectPtr)->GetPFirstSegm()!=NULL)
			((IntEsteso*)pn->ObjectPtr)->Draw();
		pn = pn->Next;
	}
}


Bool ListaIntEst::Hit(const Point& p, SelObj sobj, DrawObject *& objsel,
					  SymPath& symPath)
{
	Node *pn = first;
	
	objsel=NULL;
	
	while(pn!=NULL)
	{
		if(pn->ObjectPtr->Hit(p,sobj,objsel,symPath))
			return TRUE;
		else
			pn=GetNext(pn);
	}
	
	return FALSE;
}

Bool ListaIntEst::SymbolicHit(SymPath& symPath, SelObj sobj, DrawObject *& objsel)
{
	Node *n=first;
	
	objsel=NULL;
	while(n!=NULL)
	{
		if((IntEsteso*)n->ObjectPtr->SymbolicHit(symPath,sobj,objsel))
			return TRUE;
		n = GetNext(n);
	}
	return FALSE;
}

Bool ListaIntEst::DeleteSym(DrawObject *pD)
{
	Node *pn = first;
	
	
	while(pn!=NULL)
	{
		if(pn->ObjectPtr==pD)
		{
			if(pD->GetID()==CL_LEGATURA_QUADRA)
			{
				int a=0;
			}
			if(pD->GetID()==CL_RAGGRUPPAMENTO)
			{
				((Raggruppamento*)pD)->DeleteRaggruppamento();
				DelPtr(pD);
				return TRUE;
			}
			
			DelPtr(pD);
			delete pD;
			
			return TRUE;
		}
		else
			pn = pn->Next;
	}
	return FALSE;
}


void ListaIntEst::SetCodeCounter(NumCode nc)
{
	CodeCounter = nc;
}

NumCode ListaIntEst::GetCodeCounter()
{
	return CodeCounter;
}

void ListaIntEst::ReplaceIntEstFig(Figura *pOldF, Figura *pNewF)
{
	Node *pn,*pNext;
	IntEsteso* pInt;
	
	pn = GetIntEndF(pOldF,first);
	while(pn!=NULL)
	{
		pInt = (IntEsteso*)pn->ObjectPtr;
		pNext=pn->Next;
		pInt->SetPFigEnd(pNewF);
		pn = GetIntEndF(pOldF,pNext);
	}
	pn = GetIntStartF(pOldF,first,FALSE);
	while(pn!=NULL)
	{
		pInt = (IntEsteso*)pn->ObjectPtr;
		pNext=pn->Next;
		pInt->SetPFigStart(pNewF);
		pn = GetIntStartF(pOldF,pNext,FALSE);
	}
}

void ListaIntEst::CancIntEstFig(Figura *pF)
{
	IntEsteso *pInt;
	Node *pn,*pNext;
	Figura *pFigStart, *pFigEnd;
	Battuta *pBat;//Start, *pBattEnd;
	int NumLayer;
	bool remove;
		  
		  //trova gli eventuali simboli che terminano con *pF.
		  pn = GetIntEndF(pF,first);
		  while(pn!=NULL)
		  {
			  remove = TRUE;
			  pInt = (IntEsteso*)pn->ObjectPtr;
			  pNext=pn->Next;
			  if(pInt->GetID()==CL_LEGATURA_QUADRA)
			  {
				  pInt->GetBattuta(1)->DeleteSym(pInt);
				  pInt->GetBattuta(2)->DeleteSym(pInt);
			  }
			  if(pInt->GetID()==CL_RAGGRUPPAMENTO)
			  {
				  ((Raggruppamento*)pInt)->SetNumLayer();
				  NumLayer = ((Raggruppamento*)pInt)->GetNumLayer();
				  
				  pBat = pF->GetPBattuta();
				  pFigStart = ((Raggruppamento*)pInt)->GetFirstFigura();
				  pFigEnd = (pBat->GetPLayer(NumLayer))->GetPrevF(pF);
				  while (pFigEnd == NULL)
				  {
					  pBat = (pBat->GetParentScore())->GetPrevBat(pBat);
					  pFigEnd = (pBat->GetPLayer(NumLayer))->GetLastF();
					  
					  while (pFigEnd != NULL && !pFigEnd->IsANote())
					  {
						  pFigEnd = (pBat->GetPLayer(NumLayer))->GetPrevF(pFigEnd); 
					  }
				  }
				  
				  if(pFigEnd == pFigStart)
					  ((Raggruppamento*)pInt)->DeleteRaggruppamento();
				  else
				  {
					  remove = FALSE;						
					  ((Raggruppamento*)pInt)->SetLastFigura(pFigEnd);
					  ((Raggruppamento*)pInt)->SetLastBattuta(pFigEnd->GetPBattuta());  
					  
					  switch (immissione.ImmRule(CL_RAGGRUPPAMENTO,(Nota*)pFigStart, pFigStart->GetPBattuta(), (DrawObject*)pInt, NumLayer))
					  {
					  case TRUE:
						  ((Raggruppamento*)pInt)->SetBeamPosition(Raggruppamento::BEAM_UP);
						  break;
					  case FALSE:
						  ((Raggruppamento*)pInt)->SetBeamPosition(Raggruppamento::BEAM_DWN);
						  break;
					  case 2:
						  ((Raggruppamento*)pInt)->SetBeamPosition(Raggruppamento::BEAM_12);
						  break;
					  case 3:
						  ((Raggruppamento*)pInt)->SetBeamPosition(Raggruppamento::BEAM_23);
						  break;
					  default:
						  ((Raggruppamento*)pInt)->SetGamboUp();
					  }   				  
				  }				  
			  }
			  
			  if (remove){
				  DelPtrNode(pn);
			  }
			  pn = GetIntEndF(pF,pNext);
		  }
		  
		  //trova gli eventuali simboli che iniziano con *pF.
		  pn = GetIntStartF(pF,first,FALSE);
		  while(pn!=NULL)
		  {
			  remove = TRUE;
			  pInt = (IntEsteso*)pn->ObjectPtr;
			  pNext=pn->Next;
			  if(pInt->GetID()==CL_LEGATURA_QUADRA)
			  {
				  pInt->GetBattuta(1)->DeleteSym(pInt);
				  pInt->GetBattuta(2)->DeleteSym(pInt);
			  }
			  if(pInt->GetID()==CL_RAGGRUPPAMENTO)
			  {
				  ((Raggruppamento*)pInt)->SetNumLayer();
				  NumLayer = ((Raggruppamento*)pInt)->GetNumLayer();
				  
				  pBat = pF->GetPBattuta();
				  pFigEnd = ((Raggruppamento*)pInt)->GetLastFigura();
				  pFigStart = (pBat->GetPLayer(NumLayer))->GetNextF(pF);
				  while (pFigStart == NULL)
				  {
					  pBat = (pBat->GetParentScore())->GetNextBat(pBat);
					  pFigStart = (pBat->GetPLayer(NumLayer))->GetFirstF();
					  
					  while (pFigStart != NULL && !pFigStart->IsANote())
					  {
						  pFigStart = (pBat->GetPLayer(NumLayer))->GetNextF(pFigEnd); 
					  }
				  }
				  
				  if(pFigEnd == pFigStart)
					  ((Raggruppamento*)pInt)->DeleteRaggruppamento();
				  else
				  {
					  remove = FALSE;		
					  ((Raggruppamento*)pInt)->SetFirstFigura(pFigStart);
					  ((Raggruppamento*)pInt)->SetFirstBattuta(pFigStart->GetPBattuta());  
					  
					  switch (immissione.ImmRule(CL_RAGGRUPPAMENTO,(Nota*)pFigStart, pFigStart->GetPBattuta(), (DrawObject*)pInt, NumLayer))
					  {
					  case TRUE:
						  ((Raggruppamento*)pInt)->SetBeamPosition(Raggruppamento::BEAM_UP);
						  break;
					  case FALSE:
						  ((Raggruppamento*)pInt)->SetBeamPosition(Raggruppamento::BEAM_DWN);
						  break;
					  case 2:
						  ((Raggruppamento*)pInt)->SetBeamPosition(Raggruppamento::BEAM_12);
						  break;
					  case 3:
						  ((Raggruppamento*)pInt)->SetBeamPosition(Raggruppamento::BEAM_23);
						  break;
					  default:
						  ((Raggruppamento*)pInt)->SetGamboUp();
					  }   
				  }	
			  }
			  if (remove){
				  DelPtrNode(pn);
			  }
			  pn = GetIntStartF(pF,pNext,FALSE);
		  }
	  }
	  
	  
	  void ListaIntEst::CancIntEstBat(Battuta *pB)
	  {
		  IntEsteso *pInt;
		  Node *pNext;
		  Node *pn = first;
		  while(pn!=NULL)
		  {
			  pNext = pn->Next;
			  pInt = (IntEsteso*)pn->ObjectPtr;
			  if ((pInt->GetBattuta(1)==pB) || (pInt->GetBattuta(2)==pB))
			  {
				  DelPtrNode(pn);
			  }
			  pn = pNext;
		  }
	  }
	  
	  IntEsteso *ListaIntEst::GetNumCodeInt(NumCode nc)
	  {
		  IntEsteso *pInt;
		  Node *pn = first;
		  
		  while(pn!=NULL)
		  {
			  pInt = (IntEsteso*)(pn->ObjectPtr);
			  if ((pInt->GetNumericCode())==nc)
				  return pInt;
			  pn = pn->Next;
		  }
		  
		  return NULL; 
	  }
	  
	  void ListaIntEst::AdjustVUFig()
	  {
		  Node *pn = first;
		  
		  while (pn!=NULL)
		  {
			  if(pn->ObjectPtr->GetID()==CL_LEGATURA || pn->ObjectPtr->GetID()==CL_LEGATURA_QUADRA || 
				  pn->ObjectPtr->GetID()==CL_BEND || pn->ObjectPtr->GetID()==CL_RAGGRUPPAMENTO)
				  (((IntEsteso*)pn->ObjectPtr)->AdjustVUFig());
			  
			  pn = pn->Next;
		  }   
	  }
	  
	  void ListaIntEst::SetOnlyLegature(Bool onlyLeg)
	  {
		  onlyLegature=onlyLeg;
	  }
	  
	  void ListaIntEst::SetupIntRefs(Battuta *pBat,Lista activeInts[])
	  {
		  ListaFigure *pLayer;
		  Figura *pFig;
		  
		  for(int lay=0;lay<NUM_MAX_LAYER;lay++)
		  {
			  pLayer=pBat->GetPLayer(lay);
			  if(pLayer!=NULL)
			  {
				  pFig=pLayer->GetFirstFig();
				  while(pFig!=NULL)
				  {
					  Node *n=first;
					  while ((n=GetIntEndF(pFig,n))!=NULL)
					  {
						  pFig->AddIntRef(INT_END,(IntEsteso*)n->ObjectPtr);
						  activeInts[lay].DelPtr(n->ObjectPtr);
						  n=GetNext(n);
					  }
					  
					  IntEsteso *ref=(IntEsteso*)activeInts[lay].GetFirst();
					  while(ref!=NULL)
					  {
						  pFig->AddIntRef(INT_OVER,ref);
						  ref=(IntEsteso*)activeInts[lay].GetNext(ref);
					  }
					  
					  n=first;
					  while ((n=GetIntStartF(pFig,n,FALSE))!=NULL)
					  {
						  pFig->AddIntRef(INT_START,(IntEsteso*)n->ObjectPtr);
						  activeInts[lay].InsertEnd(n->ObjectPtr);      
						  n=GetNext(n);
					  }
					  pFig=pLayer->GetNextFig();
				  }
			  } 
		  }
	  }
