/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: listaint.hpp,v 1.1 2005/07/25 09:35:31 bellinpo Exp $ */


#ifndef LISTAINTEST_HPP
#define LISTAINTEST_HPP

#include "lioo.hpp"
#include "lst/lista.hpp"
#include "bat/battuta.hpp"
#include "int/intestes.hpp"

class ListaIntEst : public Lista
{
protected:
	
    Spartito *spartito;	
	
    NumCode CodeCounter;
	// contatore per dotare ogni intervallo di codice numerico
    Lista LStartedSopra, LStartedSotto;
	// Queste liste contengono i simboli di intervallo che sono iniziati
	// rispettivamente sopra e sotto il rigo durante la scansione delle
	// battute.
	
    Bool RevisionY;
	// Va impostata TRUE prima di effettuare la scansione della lista
	// per rivedere solo il posizionamento verticale.
	
    Node* GetIntStartF(Figura *pF, Node *pN, Bool Accordo);
	// Scandisce la lista a partire dal nodo pN per determinare il prossimo
	// nodo corrispondente ad un intervallo che inizia sulla figura pF.
	// Se restituisce NULL, non ha trovato nulla.
    Node* GetIntEndF(Figura *pF, Node *pN);
	// Scandisce la lista a partire dal nodo pN  per determinare il prossimo
	// nodo corrispondente ad un intervallo che termina sulla figura pF.
	// Se restituisce NULL, non ha trovato nulla.
	
    Bool onlyLegature;
		  
public:
    ListaIntEst();
    ~ListaIntEst();
	
    void Init();
    void Free();
	
    void SetCodeCounter(NumCode);
    NumCode GetCodeCounter();
	IntEsteso *GetNumCodeInt(NumCode);
	
	Spartito* GetSpartito(){return spartito;}	  
	void SetSpartito(Spartito *sp){spartito=sp;}  
    
	void SetRevisionY(Bool ry);  
    void ValutaLegature();
	void SetOnlyLegature(Bool onlyLeg=TRUE);
	
    Bool AddLegVal(CmdID cmd, Figura *pF1, Figura *pF2, Battuta *pb1, Battuta *pb2, 
		TipoInserimento, ArgType Arg, ArgType& nc, int voice, Bool Multiv, 
		int StemFirst, int StemLast);
    Bool Add(CmdID cmd, Figura *pF1, Figura *pF2, Battuta *pb1, Battuta *pb2, 
		TipoInserimento, ArgType Arg, ArgType& nc, int voice, Bool Multiv, 
		Bool StemInt, int StemFirst, int StemLast, Bool Interm, Bool DrawInt,LineType ltype);
    void Add(IntEsteso *pInt, ArgType& nc);
    void Add(IntEsteso *pInt);
    void AddStartedSopra(IntEsteso *pInt);
    void AddStartedSotto(IntEsteso *pInt);
    void AddStarted(IntEsteso *pInt);
	
    void UpdateStartedSopra(const Point& Plv, const Point& P, VUnit YPent, Bool fine = TRUE,Figura* pF=NULL,Bool inAccordo=FALSE);
    void UpdateStartedSopra(Figura *pF, VUnit YPent, Bool inAccordo, Bool fine = TRUE);
    void UpdateStartedSotto(const Point& Plv, const Point& P, VUnit YPent, Bool fine = TRUE,Figura* pF=NULL,Bool inAccordo=FALSE);
    void UpdateStartedSotto(Figura *pF, VUnit YPent, Bool inAccordo, Bool fine = TRUE);
    void UpdateStarted(Figura *pF, VUnit YPent,Bool inAccordo, Bool fine = TRUE);
	// Aggiorna le posizioni dei simboli iniziati, quelli nelle liste
	// Started, in modo da sovrastare
	// o sottostare la figura *pF (sempre fuori dal pentagramma).
    void RemoveStarted(IntEsteso *pInt);
    void ClearStarted();
	// cancella l'oggetto individuato dal puntatore.
	// disalloca inoltre sia l'oggetto che il puntatore.
	void ReplaceIntEstFig(Figura *pOldF, Figura *pNewF);
	// trova gli eventuali simboli di intervallo esteso che iniziano
	// o terminano sulla figura pOldF e aggiorna a pNewF.
    void CancIntEstFig(Figura *pF);
	// trova gli eventuali simboli di intervallo esteso che iniziano
	// o terminano sulla figura e li cancella.
    void CancIntEstBat(Battuta *pbat);
	
    void ResetPos();
	// Dealloca i Segmenti di tutti gli intervalli in lista.
    void FindPrevStartedFig(Figura *pF);
	// Pone fra gli started i simboli nelle pagine precedenti che
	// terminano con la figura *pF.
    void FindPrevStartedBat(Battuta *pB);
	// Pone fra gli started i simboli iniziati nelle pagine precedenti che
	// terminano nella battuta.
	
	
    void ExamineBat(Battuta *pB);
	// Esamina la battuta e per ogni figura pF controlla gli intervalli che 
	// terminano o iniziano in pF:
	//	quelli che terminano in pF, vengono tolti dagli Started;
	//	quelli che iniziano in pF, vengono messi negli Started 
    void ExamineFig(Figura *pF, VUnit YPent, Bool Accordo,Bool inAccordo);
	// Esamina la figura pF controllando gli intervalli che terminano o 
	// iniziano in pF:
	//	quelli che terminano in pF, vengono tolti dagli Started;
	//	quelli che iniziano in pF, vengono messi negli Started 
    void ExamineInizioPentagr(Battuta *pB);
	//  se siamo ala prima passata, 
	//  e se (tpBat==INIZIO_PAGINA || tpBat==INIZIO_PENTAGRAMMA)
	//  posiziona la x degli Started in cima a sinistra
	// Se la battuta inizia un pentagramma, imposta l'inizio di ogni
	// simbolo Started.
	
    void EndPage(Battuta *pB);
	// Conclude il disegno dei simboli di intervallo con la battuta 
	// assegnata (cancella gli Started).
    void EndRow(Battuta *pB);
	// Conclude il disegno dei simboli di intervallo con la battuta 
	// assegnata aggiunge un segmento agli intervalli nello Started..
	
    void Draw();
	
	// Disegna i simboli di intervallo per i quali PFirstSegm!=NULL
	
    void AdjustVUFig();
	
    Bool Hit(const Point& p, SelObj sobj, DrawObject *& objsel, SymPath&
		symPath);
    Bool SymbolicHit(SymPath&,SelObj,DrawObject*&);
	
    Bool DeleteSym(DrawObject *pD);
	// Elimina dalla lista il simbolo puntato da pD e lo disalloca.
	
    void SetupIntRefs(Battuta*,Lista[]);

	int SaveXML(FILE *);
};

#endif //LISTAINTEST_HPP
