/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: pentagra.cpp,v 1.1 2005/07/25 09:35:31 bellinpo Exp $ */


#include "spa/pentagra.hpp"

Pentagramma::Pentagramma()
{
	ID=CL_PENTAGRAMMA;
	for(int i=0; i< MAX_VIEW; i++)
	{
		spaceUpDwn[i]=0;
		width[i]=0;
		MaxWidth[i]=0;
	}
	NRighi=5;  //Numero Righi Pentagramma
	isUsed=false;
}

Pentagramma::Pentagramma(VUnit w)
{
	ID=CL_PENTAGRAMMA;
	for(int i=0; i< MAX_VIEW; i++)
	{
		spaceUpDwn[i]=0;
		width[i]=0;
		MaxWidth[i]=0;
	}
	SetWidth(w);
	SetMaxWidth(w);
	isUsed=false;
};

void Pentagramma::SetWidth(VUnit w)
{ width[currentView]=w; }

void Pentagramma::SetMaxWidth(VUnit w)
{ MaxWidth[currentView]=w; }

VUnit Pentagramma::GetMaxWidth()
{ return MaxWidth[currentView]; };

void Pentagramma::SetSpaceUpDwn(VUnit spc)
{ spaceUpDwn[currentView]=spc; };

VUnit Pentagramma::GetSpaceUpDwn()
{ return spaceUpDwn[currentView]; };

VUnit Pentagramma::GetVU2Rgt()
{ return width[currentView]; };

VUnit Pentagramma::GetVU2Up()
{ return GetPentHeight()+spaceUpDwn[currentView]; };


VUnit Pentagramma::GetVU2Lft()
{ return 0; };

VUnit Pentagramma::GetVU2Dwn()
{ 
	// Il -1 serve a non far toccare i rettangoli che delimitano due
	// pentagrammi sovrapposti.
	return spaceUpDwn[currentView]-1;
};


VUnit Pentagramma::GetBaseVU2Up()
{
	int gap;
	switch( NRighi )
	{
	case 1: gap=4;
		break;
	case 2: 
	case 3: gap=NRighi;
		break;
	case 4: gap=3;
		break;
	default:
		gap=4+(NRighi-5)/2;
		break;
	}
	return gap*VU_PER_ROW;	
}

VUnit Pentagramma::GetBaseVU2Dwn()
{ 
	return GetPentHeight()-GetBaseVU2Up();
}

VUnit Pentagramma::GetPentHeight()
{
	if(NRighi==1)
		return VU_PER_ROW*4;
	
	return VU_PER_ROW*(NRighi-1);
}


void Pentagramma::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	int i;              
	if((NRighi %2)==0)
	{
		Scr->MoveTo(GetAbsPos().x,GetAbsPos().y+2*VU_PER_ROW);
		Scr->LineTo(GetAbsPos().x+GetWidth(),GetAbsPos().y+2*VU_PER_ROW);
		for(i=1;i<NRighi/2;i++)	
		{
			Scr->MoveTo(GetAbsPos().x,GetAbsPos().y+2*VU_PER_ROW+i*VU_PER_ROW);
			Scr->LineTo(GetAbsPos().x+GetWidth(),GetAbsPos().y+2*VU_PER_ROW+i*VU_PER_ROW);
		}
		for(i=1;i<(NRighi/2)+1;i++)	
		{
			Scr->MoveTo(GetAbsPos().x,GetAbsPos().y+2*VU_PER_ROW-i*VU_PER_ROW);
			Scr->LineTo(GetAbsPos().x+GetWidth(),GetAbsPos().y+2*VU_PER_ROW-i*VU_PER_ROW);
			
		}
		
	}
	else	
	{	  
		Scr->MoveTo(GetAbsPos().x,GetAbsPos().y+2*VU_PER_ROW);
		Scr->LineTo(GetAbsPos().x+GetWidth(),GetAbsPos().y+2*VU_PER_ROW);
		if(NRighi!=1)
			for(i=1;i<=NRighi/2;i++)
			{	
				Scr->MoveTo(GetAbsPos().x,GetAbsPos().y+2*VU_PER_ROW+i*VU_PER_ROW);
				Scr->LineTo(GetAbsPos().x+GetWidth(),GetAbsPos().y+2*VU_PER_ROW+i*VU_PER_ROW);
				
				Scr->MoveTo(GetAbsPos().x,GetAbsPos().y+2*VU_PER_ROW-i*VU_PER_ROW);
				Scr->LineTo(GetAbsPos().x+GetWidth(),GetAbsPos().y+2*VU_PER_ROW-i*VU_PER_ROW);
				
			}	  
	}

	Scr->SetColor(oldcol);
}

void Pentagramma::DrawTagli(const Point& p)
{
	int alt=((p.y-GetAbsPos().y)*2)/VU_PER_ROW;
	
	// Disegno degli eventuali tagli addizionali.
	Colore oldcol = Scr->GetColor();
	Scr->SetDrwMode(XORMode);
	Scr->SetColor((ColorIcon)MOUSECOLOR);
	if(alt<1)
	{ 
		VUnit YLinea1=GetAbsPos().y;
		for(VUnit ytaglio=YLinea1-VU_PER_ROW;ytaglio>=YLinea1+alt*VU_PER_ROW/2;ytaglio-=VU_PER_ROW)
		{
			Scr->MoveTo(p.x+TAGLIOLFT,ytaglio);
			Scr->LineTo(p.x+TAGLIORGT,ytaglio,SOLID_LINE);
		}
	}
	else if(alt>9)
	{             
		VUnit YLinea5=GetAbsPos().y+4*VU_PER_ROW;
		for(VUnit ytaglio=YLinea5+VU_PER_ROW;ytaglio<=GetAbsPos().y+alt*VU_PER_ROW/2;ytaglio+=VU_PER_ROW)
		{               
			Scr->MoveTo(p.x+TAGLIOLFT,ytaglio);
			Scr->LineTo(p.x+TAGLIORGT,ytaglio,SOLID_LINE);
		}
	}
	Scr->SetDrwMode(DefMode);
	Scr->SetColor(oldcol);
}


