/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: spartito.cpp,v 1.2 2005/07/28 12:38:22 bellinpo Exp $ */


#include <xercesc/dom/deprecated/DOMParser.hpp>

#include "xml/loa/xmlcommonclasses.hpp"
#include "xml/loa/wdfdomparser.hpp"

#include <string.h>
#include <stdlib.h>
#include "spa/spartito.hpp"
#include "fig/spazio.hpp"
#include "drw/scorefrm.hpp"
//#include "par/partitur.hpp"
#include "drw/frmanage.hpp"
#include "spa/spartito.hpp"
#include "fig/spazio.hpp"
#include "drw/musicedt.hpp"
#include "lyr/lyricframe.hpp"


Spartito::Spartito(short nStaff) 
{
	NumericCode=0;
	strcpy(strumes,"");
	strcpy(shortstrumes,"");
	SpaceInit = 0;
	SpaceInit_FirstSystem = 0;
	tipoSpart = SPART_GENERICO;
	
	for(int r=0; r<MAX_VIEW; r++)
		nRighi[r]=1;
	
	listaSil.SetListaBattute(&listaBat);
	listaInt.SetSpartito(this);
	// Questa operazione permette di passare
	// alla lista delle sillabe la lista di
	// battute corrente che spartito sta
	// elaborando.
	// PtrParGraffa = NULL;
	NCorde[0]=5;	// Numero Righi Pentagramma
	NCorde[1]=5;
	NCorde[2]=5;
	NumberOfStaff = nStaff; 
	
	listaBat.Init(this);
	for(int i=0; i<MAXRIGHI; i++)
		pentagr[i].SetNRighi(NCorde[i%nStaff]);
	visible=true;
	
	for(int j=0;j<MAX_VIEW;j++)
	{
		currentPage[j]=1;
	}
	
	TmpPrintViewParams=NULL;
	PrintViewParams=NULL;
	ComputerViewParams=NULL;
	currentLyrItem=NULL;
	instrumentType=0;
	smallSize=false;
	transpose=0;
}

Spartito::~Spartito()
{
	
#ifdef DEBUG_FREE
	
	if (tipoLeggio!=LEGGIO_MASAE)
	{
		FILE *pippo;
		
		pippo = fopen("DEAL_NET.txt","a");
		fprintf(pippo,"\n\n\n ---- non masae   DEALLOCO SPARTITO  ----------------\n\n\n");
		fclose(pippo);
	}
	
#endif
	
	Free();
}



void Spartito::Free()
{
	listaInt.Free();
	listaBat.Free();
	
	listaSil.Free();
	// Dealloca la lista di sillabe.
	
	if(PrintViewParams!=NULL)
	{
		delete PrintViewParams;
		PrintViewParams=NULL;
	}
	if(ComputerViewParams!=NULL)
	{
		delete ComputerViewParams;
		ComputerViewParams=NULL;
	}
}

void Spartito::Init()
{
	
	listaBat.Init(this);
	listaInt.Init();
	
	listaSil.Init();
	
}

void Spartito::New()
{
	listaInt.Free();
	listaBat.Free();
	
	listaSil.Init();
	listaSil.Free();
	
	
	listaBat.Init(this);
	listaInt.Init();
	
	GoBatt(0);
	// Va alla battuta iniziale.
	
};


void Spartito::SetTipo(tipoSpartito ts)
{ tipoSpart = ts; }

tipoSpartito Spartito::GetTipo()
{ return tipoSpart; }

// SETTA LE DIMENSIONI DEL PENTAGRAMMA //

void Spartito::SetWidth(VUnit w,int system)
{ 
	for(int k=0; k<GetNumberOfStaff(); k++) 
		pentagr[k+system*GetNumberOfStaff()].SetWidth(w);
}


void Spartito::SetMaxWidth(VUnit w,int system)
{ 
	for(int k=0; k<GetNumberOfStaff(); k++) 
		pentagr[k+system*GetNumberOfStaff()].SetMaxWidth(w);
}

void Spartito::SetSpaceUpDwn(VUnit sp, int nsys)
{ 
	int base=nsys*GetNumberOfStaff();
	for(int k=0; k<GetNumberOfStaff(); k++) 
		pentagr[base+k].SetSpaceUpDwn(sp); 
}

void Spartito::SetNCorde(int nstaff, int ncorde)
{
	NCorde[nstaff]=ncorde;
	if(nstaff==0)
		listaBat.SetNStaffLine(ncorde);  
	for(int i=nstaff; i<MAXRIGHI; i+=NumberOfStaff) 
		pentagr[i].SetNRighi(ncorde);
	
}

VUnit Spartito::GetBaseVU2Up()
{ return pentagr[0].GetBaseVU2Up(); }

VUnit Spartito::GetBaseVU2Dwn()
{ 
	VUnit d = pentagr[0].GetBaseVU2Dwn();
	for(int i=1;i<NumberOfStaff;i++)
		d+=(2*pentagr[i].GetSpaceUpDwn()+pentagr[i].GetPentHeight());
	return d;
}

VUnit Spartito::GetBaseVU2Up(int staff)
{ return pentagr[staff].GetBaseVU2Up(); }

VUnit Spartito::GetBaseVU2Dwn(int staff)
{ 
	return pentagr[staff].GetBaseVU2Dwn(); 
}

VUnit Spartito::GetVU2Up()
{ return pentagr[0].GetVU2Up(); }

VUnit Spartito::GetVU2Dwn() 
{ 
	VUnit maxwidth = 0;
	for(int k = 0;k<GetNumberOfStaff();k++) maxwidth += pentagr[k].GetHeight();
	return maxwidth - pentagr[0].GetVU2Up();
}

VUnit Spartito::GetVU2Lft()
{ return pentagr[0].GetVU2Lft(); }

VUnit Spartito::GetVU2Rgt()
{ return pentagr[0].GetVU2Rgt(); }

VUnit Spartito::GetMaxWidth(int system) 
{ return pentagr[system*GetNumberOfStaff()].GetMaxWidth(); }

void Spartito::SetProlBarraUp(Bool spbu)
{ ProlBarraUp = spbu; }

void Spartito::SetProlBarraDown(Bool spbd)
{ ProlBarraDown = spbd; }

Bool Spartito::GetProlBarraUp()
{ return ProlBarraUp;}

Bool Spartito::GetProlBarraDown()
{ return ProlBarraDown;}


void Spartito::SetNumericCode(NumCode nc)
{ NumericCode = nc; }

NumCode Spartito::GetNumericCode()
{ return NumericCode; }

void Spartito::SetSpaceInit(int sp)
{ SpaceInit = sp; }

int Spartito::GetSpaceInit()
{ return SpaceInit; }

void Spartito::SetSpaceInit_FirstSystem(int sp)
{ SpaceInit_FirstSystem = sp; }

int Spartito::GetSpaceInit_FirstSystem()
{ return SpaceInit_FirstSystem; }

unsigned long Spartito::GetNumeroBattute()
{
	return listaBat.GetNumObj();
}

unsigned long Spartito::GetNumeroBattuteVere()
{
	return listaBat.GetNumeroBattuteVere();
}


void Spartito::DeleteBat(Battuta *bat)
{
	
	listaInt.CancIntEstBat(bat);
	
	listaSil.CancSilBat(bat);
	// Cancella le sillabe associate alle
	// figure della battuta in questione.
	
	listaBat.CmdDelBattuta(bat);
}


Bool Spartito::DeleteSym(DrawObject *sym)
{
	
	Figura *pFig;
	
	
	
	ID=sym->GetID();
	if((ID==CL_NBREVE)        	||
		(ID==CL_NSEMIBREVE)    	||
		(ID==CL_NMINIMA)       	||
		(ID==CL_NSEMIMINIMA)   	||
		(ID==CL_NCROMA)        	||
		(ID==CL_NSEMICROMA)    	||
		(ID==CL_NBISCROMA)     	||
		(ID==CL_NSEMIBISCROMA) 	||
		(ID==CL_NFUSA)         	||
		(ID==CL_GRUPPONOTE)    	||
		(ID==CL_ACCORDO)       	||
		(ID==CL_PQUATTROBATTUTE)   ||
		(ID==CL_PDUEBATTUTE)       ||
		(ID==CL_PSEMIBREVE)       	||
		(ID==CL_PMINIMA)       	||
		(ID==CL_PSEMIMINIMA)       ||
		(ID==CL_PCROMA)       	||
		(ID==CL_PSEMICROMA)       	||
		(ID==CL_PBISCROMA)       	||
		(ID==CL_PSEMIBISCROMA)     ||
		(ID==CL_PFUSA)       	||
		(ID==CL_PGENERICA)       	||
		(ID==CL_SPAZIO)       	||
		(ID==CL_RIPTEMPO)       	||
		(ID==CL_RIPBATTUTA)       	||
		(ID==CL_RIPMEZZABATTUTA))
	{
		pFig = (Figura *)sym;
		listaInt.CancIntEstFig(pFig);
		
		listaSil.CancSilFig(pFig);
		// Cancella la sillaba associata
		// alla figura,sia che inizii in essa
		// sia che vi si estenda.
		
	};
	
	
	if(listaPag.DeleteSym(sym,GetCurrentPage()))	
		return TRUE; 
	
	if(listaBat.DeleteSym(sym))
		return TRUE;
	
	
	
	if(listaInt.DeleteSym(sym))
	{
		SetupIntRefs();
		return TRUE;
	}
	
	
	return FALSE;
};

// Setta le coordinate del PENTAGRAMMA //

void Spartito::SetNPentagrammi(int n5,const class Rectangle& r)
{
	SetNPentagrammi(n5, r, false);
}


void Spartito::SetNPentagrammi(int n5,const class Rectangle& r, bool printing)
{
	if(n5<1)
		n5=MAXRIGHI;
	else if(n5>MAXRIGHI)
		n5=1;
	
	
	
	extern wxMusicEditor *MusicEditor;
	int top, bottom, left, right, staffdist;
	VUnit pwdt, x, y;
	VUnit spc;
	
	if  (!printing)
	{
		top=GetComputerViewParams()->GetTopMargin();
		bottom=GetComputerViewParams()->GetBottomMargin();
		right=GetComputerViewParams()->GetRightMargin();
		left=GetComputerViewParams()->GetLeftMargin();
		staffdist=GetComputerViewParams()->GetStaffDistance();
		pwdt=r.Width()-VUnit(2*DELTAPENT)-left-right;
		
		n5=r.Height()/((VUnit(GetPentHeight())+2*staffdist)*GetNumberOfStaff());
		
		if(n5==0) n5=1;
	}
	else
	{
		int nPage=GetCurrentPage();
		top=GetPrintViewParams()->GetTopMargin(nPage);
		bottom=GetPrintViewParams()->GetBottomMargin(nPage);
		right=GetPrintViewParams()->GetRightMargin(nPage);
		left=GetPrintViewParams()->GetLeftMargin(nPage);
		staffdist=GetPrintViewParams()->GetStaffDistance(nPage);
		
		pwdt=r.Width()-left-right;
		
		n5 = GetPrintViewParams()->GetNumStaff(nPage);
		if(n5*GetNumberOfStaff()>MAXRIGHI)
			n5 = MAXRIGHI/GetNumberOfStaff();
		
		
	}
	
	spc=staffdist;
	x=r.PUpLft.x+left;
	y=r.PUpLft.y-spc-VUnit(GetPentHeight())-top;	
	
	
	SetNRighi(n5);
	int k=0;
	
	for(int i=0;i<n5*GetNumberOfStaff();i++)
	{
		pentagr[i].SetWidth(pwdt);
		pentagr[i].SetMaxWidth(pwdt);
		pentagr[i].SetSpaceUpDwn(spc);
		pentagr[i].SetPos(NULL,Point(x,y));
		pentagr[i].SetIsUsed(false);
		
		if(i<n5*GetNumberOfStaff()-1)
			y-=2*pentagr[i].GetSpaceUpDwn()+pentagr[i].GetBaseVU2Dwn()+pentagr[i+1].GetBaseVU2Up();
		
	}
	
}

int Spartito::GetNPentagrammi()
{
	return nRighi[GetCurrentView()];
}

void Spartito::GoBatt(NumCode nb)
{
	listaBat.GoBatt(nb);
};

void Spartito::GoBattInExecution(NumCode nb)
{
	listaBat.GoBattInExecution(nb);
};

Bool Spartito::GoForward()
{
	if(listaBat.GoForward())
	{
		listaBat.SetupSimboli();
		return TRUE;
	}
	return FALSE;
}

Bool Spartito::GoBackward()
{
	if(listaBat.GoBackward())
	{
		listaBat.SetupSimboli();
		return TRUE;
	}
	return FALSE;
}


Bool Spartito::GoNextPage()
{
	return listaBat.GoNextPage();
}

Bool Spartito::GoNextPageInExecution()
{
	return listaBat.GoNextPageInExecution();
}


Bool Spartito::GoPrevPage()
{	
	return listaBat.GoPrevPage();
}


Bool Spartito::PageUp()
{
	if(currentPage[DrawObject::GetCurrentView()]>1)
		currentPage[DrawObject::GetCurrentView()]--;	
	return listaBat.PageUp(pentagr,GetNRighi(),((tipoSpart==SPART_SUPERIORE) || (tipoSpart==SPART_UNICO)));
}


Bool Spartito::PageDwn()
{
	Bool flag=FALSE;
	flag=listaBat.PageDwn(pentagr,GetNRighi(),((tipoSpart==SPART_SUPERIORE) || (tipoSpart==SPART_UNICO)));
	if(flag) 
		currentPage[DrawObject::GetCurrentView()]++;	
	return flag;
	
}


Battuta* Spartito::GetBattNum(int n)
{
	return (Battuta*)(listaBat.AtPos(n));
}

void Spartito::ComprimiBattute(Bool abil_compress)
{
	listaBat.ComprimiBattute(abil_compress);
}

//devo mettere i due casi forward e backward..parametri nel metodo!
Battuta* Spartito::GetBattCompNum(int n, Bool forward, int &nb)
{
	return NULL;
}

Battuta* Spartito::GetBatt(NumCode nc)
{
	return (Battuta*)(listaBat.GetBatt(nc));
}

Battuta* Spartito::GetBattNumProg(NumCode np)
{
	return (Battuta*)(listaBat.GetBattNumProg(np));
}

Battuta* Spartito::GetFirstBat()
{
	return (Battuta*)(listaBat.GetFirst());
}

Battuta* Spartito::GetNextBat(Battuta* pbt)
{
	return (Battuta*)(listaBat.GetNext(pbt));
};

Battuta* Spartito::GetPrevBat(Battuta* pbt)
{
	return (Battuta*)(listaBat.GetPrev(pbt));
};

Battuta* Spartito::GetLastBat()
{
	return (Battuta*)(listaBat.GetLast());
};


void Spartito::SetPos(DrawObject *d,const Point& p, int System)
{
	VUnit y=0;
	
	if (System==0)
		DrawObject::SetPos(d, p + Point(SpaceInit_FirstSystem,0));
	else
		DrawObject::SetPos(d, p + Point(SpaceInit,0));
	for(int k=0;k<GetNumberOfStaff();k++) 
	{
		pentagr[k+System*GetNumberOfStaff()].SetPos(NULL, GetAbsPos()+Point(0,y));
		pentagr[k+System*GetNumberOfStaff()].SetIsUsed(false);
		if(k<GetNumberOfStaff()-1)
			y-=2*pentagr[k].GetSpaceUpDwn()+pentagr[k].GetBaseVU2Dwn()+pentagr[k+1].GetBaseVU2Up();
	}
	
}

Point Spartito::GetAbsPos(int System)
{
	if (System==-1)
		return DrawObject::GetAbsPos();
	else
		return pentagr[System*GetNumberOfStaff()].GetAbsPos();
}

bool Spartito::GetIsUsed(int System)
{
	if(System==-1)
		return pentagr[0].GetIsUsed();
	return pentagr[System*GetNumberOfStaff()].GetIsUsed();
}

void Spartito::ResetCurrent(Node* SavedPos)
{
	listaBat.ResetCurrent(SavedPos);
}

Bool Spartito::CheckCurrent(VUnit& IntWdt, VUnit& Wdt)
{
	return listaBat.CheckCurrent(tipoSpart,IntWdt,Wdt,((tipoSpart==SPART_SUPERIORE) || (tipoSpart==SPART_UNICO)));
}

void Spartito::SetupCurrent(VUnit PosX, VUnit IntWdt, VUnit MaxWdt, int sistema)
{
	VUnit ProlUp=0, ProlDwn=0;
	Bool ProlBarra;
	ProlDwn = GetVU2Dwn();
	ProlUp = GetVU2Up();
	ProlBarra = tipoSpart != SPART_UNICO;
	listaBat.SetupCurrent(&pentagr[sistema*GetNumberOfStaff()], PosX, IntWdt, MaxWdt, ProlUp, ProlDwn, tipoSpart, ProlBarraUp,
		ProlBarraDown, sistema);
}

void Spartito::CurrentInNextPage()
{
	listaBat.CurrentInNextPage();
};


// FUNZIONI PER LA GESTIONE DELLA //
//           LIRICA.              //



void Spartito::PlaceSillabe(void)
{
	Battuta *pBat,*pPrevBat;
	Sillaba *pSilPrec=NULL;
	
	listaSil.ResetPos(); 
	// Resetta la posizione provvisoria
	// delle linee costituenti tutti i
	// prolungamenti presenti.
	
	
	// SCANSIONE BATTUTE CONTENUTE NELLE PAGINA ATTUALE.. 
	// PRIMA SCANSIONE: 
	// determina la sillaba estesa il cui prolungamento  
	// iniziato  nella pagina precedente,termina in una
	// battuta qualsiasi di quella attuale,aggiornando
	// lo started.
	
	listaBat.ResetCurrent();
	pBat = listaBat.GetCurrentBat();
	
	// Poich una sola sillaba estesa,il cui
	// prolungamento  iniziato nella pagina
	// precedente,si pu estendere in quella
	// attuale,il ciclo termina quando la
	// ricerca ha avuto successo oppure non
	// non esiste alcuna sillaba estesa,iniziata
	// nella pagina precedente,che si estende
	// in quella corrente.
	
	listaSil.InitStarted();
	while((pBat!=NULL) && (listaSil.GetStarted(/*ns*/)==NULL)) 
	{
		listaSil.FindPrevStartedBat(pBat/*,ns*/);
		listaBat.MoveCurrentBatForw();
		pBat = listaBat.GetCurrentBat();
    };
	
	// SECONDA SCANSIONE: posiziona le sillabe.
	// Per ogni battuta si eseguono le seguenti operazioni:
	// Esamina la battuta e per ogni figura pF si controlla 
	// se la sillaba associata  estesa e in tal caso se il 
	// prolungamento termina o inizia in pF.Nel primo caso 
	// la sillaba viene tolta da Started,nel secondo vi viene 
	// posta.
	// Inoltre si posizionano le sillabe lungo x e y.
	
	
	listaBat.ResetCurrent();
	pBat = pPrevBat = listaBat.GetCurrentBat();
	
	
	while(pBat!=NULL)
	{
      		listaSil.ExamineBat(pBat/*,ns*/);
			pPrevBat = pBat;
			listaBat.MoveCurrentBatForw();
            pBat = listaBat.GetCurrentBat();
			
			// Solo per DLIOO: 
			// Conclude gli eventuali prolungamenti sulla 
			// barra destra della battuta.
			
			if (pBat!=NULL)
			{
      	         if ((pBat->GetBattuta())==INIZIO_PENTAGRAMMA)
					 listaSil.EndRow(pPrevBat);
			};
			
	};
	
	// Conclude gli Started sulla barra destra della 
	// battuta.Annulla la lista degli Started:
	
	if(pPrevBat!=NULL)
		listaSil.EndPage(pPrevBat);
	
	
};

void Spartito::InsertSil( Figura *Fig1,Figura *Fig2,char *TESTO,char TOKEN,
						 Bool TRAT,Bool TRASIL)
{
	
	Battuta *pBat1,*pBat2;
	// Variabili temporanee contenenti rispettivamente
	// la battuta contenente la prima figura e quella
	// contenente la seconda.
	
	pBat1 = listaBat.GetBattWithFig(Fig1);
	// Battuta in cui  contenuta la prima
	// figura.
	
	pBat2 = listaBat.GetBattWithFig(Fig2);
	// Battuta in cui  contenuta la seconda
	// figura.
	
	
	if ((pBat2==NULL) || (pBat1==NULL) || (Fig1==NULL) || (Fig2==NULL))
		// SE uno dei puntatori sopra non  presente....
		return ;
	// ..non fa niente e ritorna.
	Ordina(Fig1, Fig2, pBat1, pBat2);
	
	listaSil.InserSillaba(Fig1,Fig2,TESTO,pBat1,pBat2,TRAT,TRASIL);
	// Procede  all'inserimento delle sillabe memorizzate
	// nel documento,nella lista di memoria attuale.
	
	return;	  		
};


void Spartito::LoadLyr(char *Name,InputSource *src)
{
	listaSil.LoadLyr(Name,src,GetNumericCode());
};


//  FINE  //

void Spartito::ResetfirstBatNextPage()
{ 
	listaBat.ResetfirstBatNextPage();
}

Battuta* Spartito::GetCurrentBat()
{ 
	return listaBat.GetCurrentBat();
}

Battuta* Spartito::GetCurrentBatLst()
{ 
	return listaBat.GetCurrentBatLst();
}


Bool Spartito::MoveCurrentBatForw()
{ 
	return listaBat.MoveCurrentBatForw();
}

void Spartito::PlaceIntervalli(Bool onlyLegature)
{
	Battuta *pBat, *pPrevBat;
	short i;
	
	IntEsteso *pIntPrec = NULL;
	listaInt.SetOnlyLegature(onlyLegature); // Imposta la variabile solo
	// legature a vero per il posi=
	// zionamento. 
	
	if(onlyLegature)
		listaInt.ResetPos();  // Dealloca i segmenti di tutti gli intervalli
	// in lista in modo che risultino non posizionati.
	
	// scansione delle battute contenute nella pagina attuale.
	
	// Prima scansione:
	// determina i simboli di intervallo iniziati nelle
	// pagine precedenti che terminano in questa pagina.
	// e li mette negli Started
	// Alloca un segmento per tutti gli intervalli che iniziano e/o
	// terminano in questa pagina
	
	listaInt.SetRevisionY(FALSE);// Imposta a FALSE la variabile RevisionY
	// in modo da vedere il posizionamento
	// anche orizzontale.	
	
	listaBat.ResetCurrent();
	pBat = listaBat.GetCurrentBat();
	while(pBat!=NULL)
	{  ////modifica da confermare per gestire int. estesi dei quali non 
		////visualizzato n il punto di inizio e n quello di fine
		listaInt.FindPrevStartedBat(pBat);
		listaBat.MoveCurrentBatForw();
		//pBat = listaBat.GetCurrentBat();
		pBat = listaBat.GetNextMeasure(pBat);
	}    
	for(i=0;i<2;i++)
	{
		listaBat.ResetCurrent();
		pBat = pPrevBat = listaBat.GetCurrentBat();
		while(pBat!=NULL)
		{
			
			listaInt.ExamineBat(pBat);
			pPrevBat = pBat;
			listaBat.MoveCurrentBatForw();
			pBat = listaBat.GetCurrentBat();
			
			// Solo per DLIOO:
			// Conclude gli Started sulla barra destra della battuta.
			// ne calcola la larghezza e altezza.
			// Alloca i segmenti per proseguire gli intervalli sui successivi
			// righi.
			
			if (pBat!=NULL)
			{
				if ((pBat->GetBattuta())==INIZIO_PENTAGRAMMA)// Se la prossima battuta
					//  di inizio
					// pentagramma.. 	
					
					listaInt.EndRow(pPrevBat);// Conclude il disegno dei simboli di
				// intervallo col la battuta assegnata
				//  ed aggiunge un segmento agli
				// intervalli nello Started.
			}
			
		}
		
		// Conclude gli Started sulla barra destra della battuta.
		// ne calcola la larghezza e altezza.
		// Annulla la lista degli Started:
		
		if(pPrevBat!=NULL)
			listaInt.EndPage(pPrevBat);// Conclude il disegno dei simboli
		// di intervallo con la battuta
		// assegnata e cancella gli started. 
		
		// Terza scansione:
		// e' necessaria un'altra passata per posizionare le y degli intervalli;
		// l'altezza degli intervalli, infatti, non si puo' conoscere fino a che
		// la seconda passata non si e' conclusa!
		// Questa scansione non ricalcola Width e Height dei simboli, ma
		// controlla semplicemente che i simboli non si sovrappongano.
		
		listaInt.SetRevisionY(TRUE);
	}
	listaInt.AdjustVUFig();
}


void Spartito::Draw()
{ 
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	//barra verticale a sx
	Scr->Line(GetAbsPos() + Point(0,GetBaseVU2Up()), 
		GetAbsPos()+Point(0,-GetBaseVU2Dwn()),
		SOLID_LINE);

	DrawPentagrammi();

	if (tipoLeggio!=LEGGIO_DLIOO)
		listaBat.DrawPageMainScore();
	else
		listaBat.DrawPagePart();

	listaInt.Draw();
	
	listaSil.Draw();
	// Disegna la Sillabe.Se estese anche il prolungamento.
	if (tipoLeggio!=LEGGIO_DLIOO)
	{
		FontID ofont=Scr->SetFont(nTextFont);
		VUnit x = pentagr[0].GetAbsPos().x;
		VUnit y = (pentagr[0].GetAbsPos().y+pentagr[NumberOfStaff-1].GetAbsPos().y)/2;
		y+=VU_PER_ROW;
		Scr->TextTo(Point( x - SpaceInit_FirstSystem, y));
		Scr->OutText((unsigned char *)GetStrumEsec());
		Scr->SetFont(ofont); 
	}

	if(isSelected) {
		DrawContour(SELECTBOX_MARGIN);
	}

	Scr->SetColor(oldcol);
}
/*
void Spartito::DrawContour(){
	Scr->SetColor((ColorIcon)SELECTED_COLOR);
	Scr->Rect(class Rectangle(GetAbsPos(), 
		GetVU2Lft() + SELECTBOX_MARGIN,
		GetVU2Rgt() + SELECTBOX_MARGIN, 
		GetVU2Up() + SELECTBOX_MARGIN, 
		GetVU2Dwn() + SELECTBOX_MARGIN)
		);

	Scr->SetColor((ColorIcon)GRIGIOS);
	Scr->Rect(class Rectangle(GetAbsPos(), 
		GetVU2Lft() + SELECTBOX_MARGIN -1,
		GetVU2Rgt() + SELECTBOX_MARGIN -1, 
		GetVU2Up() + SELECTBOX_MARGIN +1, 
		GetVU2Dwn() + SELECTBOX_MARGIN +1)
		);

	Scr->SetColor((ColorIcon)GRIGIO);
	Scr->Rect(class Rectangle(GetAbsPos(), 
		GetVU2Lft() + SELECTBOX_MARGIN -2,
		GetVU2Rgt() + SELECTBOX_MARGIN -2, 
		GetVU2Up() + SELECTBOX_MARGIN +2, 
		GetVU2Dwn() + SELECTBOX_MARGIN +2)
		);
	
}
*/
void Spartito::DrawInExecution()
{ 
	DrawPentagrammi();
	listaBat.DrawPageMainScore();
	
	listaBat.SetupSimboli(TRUE);
	PlaceIntervalli(TRUE); // solo legature
	PlaceSillabe();
	listaBat.SetupSimboli();
	PlaceIntervalli(FALSE);
	
	listaInt.Draw();
	listaSil.Draw();
	// Disegna la Sillabe.Se estese anche il prolungamento.
	
	if (tipoLeggio!=LEGGIO_DLIOO)
	{
		FontID ofont=Scr->SetFont(nTextFont);
		Scr->TextTo(pentagr[0].GetAbsPos() + Point( - SpaceInit_FirstSystem, 0));
		Scr->OutText((unsigned char *)GetStrumEsec());
		// Scrive il testo indicato con il font attualmente selezionato.  
		Scr->SetFont(ofont); 
	}	
}

void Spartito::Redraw()
{ 
	listaBat.RedrawPage(pentagr,GetNRighi(),((tipoSpart==SPART_SUPERIORE) || (tipoSpart==SPART_UNICO)));
	
	// Questo va fatto dopo poiche' l'istruzione precedente puo`
	// modificare la lunghezza del pentagramma.
	
	PlaceIntervalli(TRUE); // solo legature
	
	
	PlaceSillabe();
	
	
	listaBat.SetupSimboli();
	
	PlaceIntervalli(FALSE);
	// altri simboli di intervallo
	
	listaBat.DrawPagePart();
	DrawPentagrammi();
	
	listaInt.Draw();
	
	listaSil.Draw();
	
	
	
}


void Spartito::DrawPentagrammi()
{
	for(int i=0;i<GetNRighi()*GetNumberOfStaff();i++)
		if(pentagr[i].GetIsUsed()) 
			pentagr[i].Draw();
}


void Spartito::DrawTagli(const Point& p)
{
	for(int i=0;i<GetNRighi()*GetNumberOfStaff();i++)
	{
		if(pentagr[i].Hit(p))
		{
			pentagr[i].DrawTagli(p);
			break;
		}
	}
}


Bool Spartito::Hit(const Point& p, SelObj sobj, DrawObject *& objsel,
				   SymPath& symPath)
{
	
	if(sobj==SPARTITO_SEL)
	{
		if(class Rectangle(*this).hasIn(p))
		{
			objsel=this;
			symPath.Array[SPARTITO_PATH_LEVEL]=GetNumericCode();
			return TRUE;
		}
		return FALSE;
	}
	
	
	
	if(sobj==ANY_SEL)
	{	
		
		if(listaPag.Hit(p,GetCurrentPage(),objsel))
			return TRUE;
		
		// La lista degli intervalli risponde solo ad ANY_SEL.
		if(listaInt.Hit(p,sobj,objsel,symPath))
		{
			symPath.Array[SPARTITO_PATH_LEVEL] = GetNumericCode();
			symPath.Array[LISTA_PATH_LEVEL] = 2;
			return TRUE;
		};
		
	};
	
	
	// La lista di battute risponde da sola a diversi tipi di oggetto
	// selezionato.
	if(listaBat.Hit(p,sobj,objsel,symPath))
	{
		symPath.Array[SPARTITO_PATH_LEVEL] = GetNumericCode();
		symPath.Array[LISTA_PATH_LEVEL] = 1;
		return TRUE;
	};
	
	
	return FALSE;
};

Bool Spartito::SymbolicHit(SymPath& symPath, SelObj sobj, DrawObject *& objsel)
{
	if(symPath.Array[SPARTITO_PATH_LEVEL]!=GetNumericCode())
		return FALSE;
	
	if(sobj==SPARTITO_SEL)
	{
		objsel = this;
		return TRUE;
	}
	
	if(symPath.Array[LISTA_PATH_LEVEL]==1)
		return listaBat.SymbolicHit(symPath,sobj,objsel);
    // Questo cerca nella lista degli intervalli
	if(symPath.Array[LISTA_PATH_LEVEL]==2)
	{
		if (symPath.Array[FIGURA1_PATH_LEVEL]>0)
			return listaBat.SymbolicHit(symPath,sobj,objsel); //caso raggruppamento
		else
			return listaInt.SymbolicHit(symPath,sobj,objsel);
	}
		return FALSE;
	
};


Bool Spartito::Hit(const class Rectangle& r,SelObj selObj,short layer,
				   SymPath& sp1, SymPath& sp2)
{
	if(listaBat.Hit(r,selObj,layer, sp1,sp2))
	{
		sp1.Array[SPARTITO_PATH_LEVEL] = GetNumericCode();
		sp1.Array[LISTA_PATH_LEVEL] = 1;
		sp2.Array[SPARTITO_PATH_LEVEL] = GetNumericCode();
		sp2.Array[LISTA_PATH_LEVEL] = 1;
		return TRUE;
	}
	// Non controlla la lista degli intervalli.
	return FALSE;
}


CmdResult Spartito::SymCommand(SymCmd& SCmd, Battuta *pBattuta)
{
	DrawObject *pobj, *pobj1;
	Figura *pf1, *pf2;
	Battuta *pBat1, *pBat2;
	Battuta *pBat, *pbat;
	Bool fl;
	ArgType nc,np;
	NumCode nb1, nb2;
	int voice=0;
	Bool Multiv=false;
	Bool StemInt=false;
	int StemFirst=0;
	int StemLast=0;
	Bool Interm=false;
	ClassID ChvID;
	tipoArmaturaChiave ArmCh;
	int Num, Den;
	
	
	switch(SCmd.CommandId)
	{
    case CMD_DEL_COL_FIGURE:
		if (SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj))
		{
			pBat = (Battuta*) pobj;
			DeleteFigure(pBat,(NumCode)SCmd.ArgV[0], (short)SCmd.ArgV[1]);
			return CMD_OK;
		}
		return CMD_FAIL;
		
    case CMD_BATTUTA_ARMCHV_0D:
    case CMD_BATTUTA_ARMCHV_1D:
    case CMD_BATTUTA_ARMCHV_2D:
    case CMD_BATTUTA_ARMCHV_3D:
    case CMD_BATTUTA_ARMCHV_4D:
    case CMD_BATTUTA_ARMCHV_5D:
    case CMD_BATTUTA_ARMCHV_6D:
    case CMD_BATTUTA_ARMCHV_7D:
    case CMD_BATTUTA_ARMCHV_1B:
    case CMD_BATTUTA_ARMCHV_2B:
    case CMD_BATTUTA_ARMCHV_3B:
    case CMD_BATTUTA_ARMCHV_4B:
    case CMD_BATTUTA_ARMCHV_5B:
    case CMD_BATTUTA_ARMCHV_6B:
    case CMD_BATTUTA_ARMCHV_7B:
    case CMD_BATTUTA_ARMCHV_0Dm:
    case CMD_BATTUTA_ARMCHV_1Dm:
    case CMD_BATTUTA_ARMCHV_2Dm:
    case CMD_BATTUTA_ARMCHV_3Dm:
    case CMD_BATTUTA_ARMCHV_4Dm:
    case CMD_BATTUTA_ARMCHV_5Dm:
    case CMD_BATTUTA_ARMCHV_6Dm:
    case CMD_BATTUTA_ARMCHV_7Dm:
    case CMD_BATTUTA_ARMCHV_1Bm:
    case CMD_BATTUTA_ARMCHV_2Bm:
    case CMD_BATTUTA_ARMCHV_3Bm:
    case CMD_BATTUTA_ARMCHV_4Bm:
    case CMD_BATTUTA_ARMCHV_5Bm:
    case CMD_BATTUTA_ARMCHV_6Bm:
    case CMD_BATTUTA_ARMCHV_7Bm:
		if (SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj))
		{
			pbat = (Battuta*) pobj;
			ArmCh = pbat->GetArmatura(0);
			ChvID = pbat->GetChiave(0)->GetID();
			while (pbat!=NULL)
			{
				if (pbat->SymCommand(SCmd) == CMD_FAIL)
					return CMD_FAIL;
				pbat = GetNextBat(pbat);
				if (pbat!=NULL)
					if ((pbat->GetChiave(0)->GetID() != ChvID) || (pbat->GetArmatura(0) != ArmCh))
						pbat = NULL;
			}
			return CMD_OK;
		}
		return CMD_FAIL;
		
	case CMD_BATTUTA_CVIOLINO:
	case CMD_BATTUTA_CVIOLINO8:
	case CMD_BATTUTA_CVIOLINO8SOPRA:
	case CMD_BATTUTA_CBASSO:
	case CMD_BATTUTA_CBASSO8:
	case CMD_BATTUTA_CBASSO8SOPRA:
	case CMD_BATTUTA_CBASSOOLD:
	case CMD_BATTUTA_CBARITONO:
	case CMD_BATTUTA_CTENORE:
	case CMD_BATTUTA_CTENORE8:
	case CMD_BATTUTA_CCONTRALTO:
	case CMD_BATTUTA_CMEZZOSOPRANO:
	case CMD_BATTUTA_CSOPRANO:
	case CMD_BATTUTA_CPERCUSBOX:
	case CMD_BATTUTA_CPERCUS2LINES:
	case CMD_BATTUTA_CTAB:
	case CMD_BATTUTA_CVUOTA:
		if (SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj))
		{
			pbat = (Battuta*) pobj;
			ChvID = pbat->GetChiave(0)->GetID();
			while (pbat!=NULL)
			{
				if (pbat->SymCommand(SCmd) == CMD_FAIL)
					return CMD_FAIL;
				pbat = GetNextBat(pbat);
				if (pbat!=NULL)
					if (pbat->GetChiave(0)->GetID() != ChvID)
						pbat = NULL;
			}
			return CMD_OK;
		}
		return CMD_FAIL;
		
    case CMD_BATTUTA_TEMPO:
		if (SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj))
		{
			pbat = (Battuta*) pobj;
			pbat->EliminaGruppi();
			Num = pbat->GetIntest(0)->GetPtrTemp()->GetNumeratore();
			Den = pbat->GetIntest(0)->GetPtrTemp()->GetDenominatore();
			while (pbat!=NULL)
			{
				if (pbat->SymCommand(SCmd) == CMD_FAIL)
					return CMD_FAIL;
				pbat = GetNextBat(pbat);
				if (pbat!=NULL)
					if ((pbat->GetIntest(0)->GetPtrTemp()->GetNumeratore() != Num) ||
						(pbat->GetIntest(0)->GetPtrTemp()->GetDenominatore() != Den) )
						pbat = NULL;
			}
			return CMD_OK;
		}
		return CMD_FAIL;
		
		
    case CMD_PARTITURA_ADD_BATTUTA:
		if (SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj))
		{
			nc = SCmd.ArgV[6];
			np = SCmd.ArgV[7];
			
			fl = listaBat.CmdAddBattuta((Battuta*)pobj,(Bool)SCmd.ArgV[1],
				pentagr[0].GetSpaceUpDwn(),nc,np,this);	
			
			if(fl)
			{
				SCmd.ArgV[6] = nc;
				SCmd.ArgV[7] = np;
				return CMD_OK;
			}
		}
		return CMD_FAIL;
    case CMD_PARTITURA_NEW:
		New();
		return CMD_OK;
    case CMD_PARTITURA_SAVE:
		
		
		return CMD_OK;
    case CMD_PARTITURA_DEL_BATTUTA:
		if (SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj))
		{
			DeleteBat((Battuta*)pobj);
			return CMD_OK;
		}
		return CMD_FAIL;
		
    case CMD_SPARTITO_ADD_LEGATURA_VALORE:
		if (SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj))
		{
			pobj1 = GetNextFigura(pobj);
			
			if (pobj1!=NULL)
			{
				if (SymbolicHit(SCmd.Path1,BATTUTA_SEL,(DrawObject*&)pBat))
					nb1 = ((Battuta*)pBat)->GetNumericCode();
				else
					return CMD_FAIL;
				
				
				if ( (pBat->GetPLayer(0)->GetPosFig((Figura*)pobj1) > -1) ||
					(pBat->GetPLayer(1)->GetPosFig((Figura*)pobj1) > -1) )
				{
					nb2 = pBat->GetNumericCode();
					pBat1=pBat;
				}
				else
				{
					pBat1 = GetNextBat(pBat);
					if ( pBat1 != NULL &&
						((pBat1->GetPLayer(0)->GetPosFig((Figura*)pobj1) > -1) ||
						(pBat1->GetPLayer(1)->GetPosFig((Figura*)pobj1) > -1) ))
						nb2 = pBat1->GetNumericCode();
					else
						return CMD_FAIL;
				}
				
				voice = pBat->GetVoice((Figura*)pobj);
				Multiv = pBat->GetMultiVoice();
				
				if ( ((Nota *)pobj)->GetLGambo()>0 )
					StemFirst = 1;
				else if ( ((Nota *)pobj)->GetLGambo()<0 )
					StemFirst = 0;
				
				if ( ((Nota *)pobj1)->GetLGambo()>0 )
					StemLast = 1;
				else if ( ((Nota *)pobj1)->GetLGambo()<0 )
					StemLast = 0;
				
				nc = SCmd.ArgV[6];
				if(listaInt.AddLegVal(SCmd.CommandId, (Figura*)pobj, (Figura*)pobj1, pBat, pBat1,
					(TipoInserimento)SCmd.ArgV[3], SCmd.ArgV[1],nc, voice, Multiv,
					StemFirst, StemLast))
				{
					SCmd.ArgV[6] = nc;
					return CMD_OK;
				}
			}
		}
		return CMD_FAIL;
		
    case CMD_SPARTITO_ADD_LEG_QUADRA:
    case CMD_SPARTITO_ADD_LEGATURA:
		if (SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj))
		{
			if (SymbolicHit(SCmd.Path2,SCmd.SelObjType,pobj1))
			{
				pf1 = (Figura *)pobj;
				pf2 = (Figura *)pobj1;
				
				pBat1 = listaBat.GetBattWithFig(pf1);
				pBat2 = listaBat.GetBattWithFig(pf2);
				
				if ((pBat2==NULL) || (pBat1==NULL) || (pf1==NULL) || (pf2==NULL))
					return CMD_FAIL;
				
				Ordina(pf1, pf2, pBat1, pBat2);
				
				nb1 = pBat1->GetNumericCode();
				nb2 = pBat2->GetNumericCode();
				
				voice = pBat1->GetVoice(pf1);
				Multiv = pBat1->GetMultiVoice();
				StemInt = GetStem(nb1,pf1,nb2,pf2);
				
				if ( ((Nota *)pf1)->GetLGambo()>0 )
				{
					StemFirst = 1;
				}
				else if ( ((Nota *)pf1)->GetLGambo()<0 )
				{
					StemFirst = 0;
				}
				
				if ( ((Nota *)pf2)->GetLGambo()>0 )
				{
					StemLast = 1;
				}
				else if ( ((Nota *)pf2)->GetLGambo()<0 )
				{
					StemLast = 0;
				}
				Interm = Intermed(nb1,pf1,nb2,pf2);
				nc = SCmd.ArgV[6];
				if(listaInt.Add(SCmd.CommandId, pf1, pf2, pBat1, pBat2,
					(TipoInserimento)SCmd.ArgV[3], SCmd.ArgV[1],nc, voice,
					Multiv, StemInt, StemFirst, StemLast, Interm, SCmd.ArgV[4],(LineType)SCmd.ArgV[7]))
				{
					SCmd.ArgV[6] = nc;
					return CMD_OK;
				}
			}
		}
		return CMD_FAIL;
		// INSERIMENTO SILLABE //
	case CMD_SPARTITO_ADD_SILLABA:
		{
			return CMD_FAIL;
		}
    case CMD_SPARTITO_EDIT_LYRIC:
		{ 
			EditLyric();
			return CMD_OK;
		}		
    case CMD_SPARTITO_REMOVE_LYRIC:
		{
			
			listaSil.Free();
			return CMD_OK;
		}		
    case CMD_SPARTITO_ADD_FORC_CRESC:
    case CMD_SPARTITO_ADD_FORC_DIMIN:
    case CMD_SPARTITO_ADD_FRECCIA:
    case CMD_SPARTITO_ADD_ONDA:
    case CMD_SPARTITO_ADD_CAMBIO_RIT:
    case CMD_SPARTITO_ADD_MODIF8:
	case CMD_SPARTITO_ADD_MODIF15MA:
	case CMD_SPARTITO_ADD_MODIF8BA:
	case CMD_SPARTITO_ADD_MODIF15BA:
    case CMD_SPARTITO_ADD_TRILLOONDA:
    case CMD_SPARTITO_ADD_BEND:
    case CMD_SPARTITO_ADD_PEDALE:
		if (SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj))
		{
			if (SymbolicHit(SCmd.Path2,SCmd.SelObjType,pobj1))
			{
				pf1 = (Figura *)pobj;
				pf2 = (Figura *)pobj1;
				
				pBat1 = listaBat.GetBattWithFig(pf1);
				pBat2 = listaBat.GetBattWithFig(pf2);
				
				if ((pBat2==NULL) || (pBat1==NULL) || (pf1==NULL) || (pf2==NULL))
					return CMD_FAIL;
				
				Ordina(pf1, pf2, pBat1, pBat2);
				
				nb1 = pBat1->GetNumericCode();
				nb2 = pBat2->GetNumericCode();
				
				nc = SCmd.ArgV[6];
				
				if(listaInt.Add(SCmd.CommandId, pf1, pf2, pBat1, pBat2,
					(TipoInserimento)SCmd.ArgV[3], SCmd.ArgV[1],nc, voice,
					Multiv, StemInt, StemFirst, StemLast, Interm, SCmd.ArgV[4],(LineType)SCmd.ArgV[7]))
					
					
				{
					SCmd.ArgV[6] = nc;
					SetupIntRefs();
					return CMD_OK;
				}
			}
		}
		return CMD_FAIL;
	case CMD_SPARTITO_ADD_BEAM:
		if (SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj))
		{
			if (SymbolicHit(SCmd.Path2,SCmd.SelObjType,pobj1))
			{
				pf1 = (Figura *)pobj;
				pf2 = (Figura *)pobj1;
				
				pBat1 = listaBat.GetBattWithFig(pf1);
				pBat2 = listaBat.GetBattWithFig(pf2);
				
				SetupIntRefs();//inizializza i simboli orizzontali per le figure
				
				Raggruppamento *raggrFinale=pf2->GetRaggruppamento();	////
				Raggruppamento *raggrCorrente=NULL;
				
								
				if ((pBat2==NULL) || (pBat1==NULL) || (pf1==NULL) || (pf2==NULL))
					return CMD_FAIL;
				
				Ordina(pf1, pf2, pBat1, pBat2);
				
				nb1 = pBat1->GetNumericCode();
				nb2 = pBat2->GetNumericCode();
				
				nc = SCmd.ArgV[6];
				
				Figura *parentFig=NULL;
				ClassID nID;
				bool FigureCorrette=TRUE, Finito=FALSE;
				//TEST PRIMA FIGURA
				nID=pf1->GetID();
				parentFig=pf1->GetParentFig();
				if(parentFig && parentFig->GetID()==CL_GRUPPONOTE)
					FigureCorrette=FALSE;
				if(nID==CL_ACCORDO){
					Accordo *pa=(Accordo*)pf1;
					nID=pa->GetNoteID();
				}
				if(nID!=CL_NCROMA && nID!=CL_NSEMICROMA && nID!=CL_NBISCROMA && nID!=CL_NSEMIBISCROMA && nID!=CL_NFUSA)
					FigureCorrette=FALSE;				
				if(pf2->GetNotina())
					FigureCorrette=FALSE;
				//END TEST
				if (pf1!=pf2)
				{
					//TEST ULTIMA FIGURA
					nID=pf2->GetID();
					parentFig=pf2->GetParentFig();
					if(parentFig && parentFig->GetID()==CL_GRUPPONOTE)
						FigureCorrette=FALSE;
					if(nID==CL_ACCORDO){
						Accordo *pa=(Accordo*)pf2;
						nID=pa->GetNoteID();
					}
					if(nID!=CL_NCROMA && nID!=CL_NSEMICROMA && nID!=CL_NBISCROMA && nID!=CL_NSEMIBISCROMA && nID!=CL_NFUSA)
						FigureCorrette=FALSE;				
					if(pf2->GetNotina())
						FigureCorrette=FALSE;
					//END TEST
				}
				//TEST FIGURE INTERMEDIE
				Figura *currentFig=GetNextFigura((DrawObject*)pf1);
				while((currentFig!=NULL)&&(currentFig!=pf2)){
					nID=currentFig->GetID();
					parentFig=currentFig->GetParentFig();
					if(parentFig && parentFig->GetID()==CL_GRUPPONOTE)
						FigureCorrette=FALSE;
					if(nID==CL_ACCORDO){
						Accordo *pa=(Accordo*)currentFig;
						nID=pa->GetNoteID();
					}
					if( (nID!=CL_NCROMA && nID!=CL_NSEMICROMA && nID!=CL_NBISCROMA && nID!=CL_NSEMIBISCROMA && nID!=CL_NFUSA) &&
						(nID!=CL_PCROMA && nID!=CL_PSEMICROMA && nID!=CL_PBISCROMA && nID!=CL_PSEMIBISCROMA && nID!=CL_PFUSA)    )
						FigureCorrette=FALSE;
					if(currentFig->GetNotina())
						FigureCorrette=FALSE;
					currentFig=this->GetNextFigura((DrawObject*)currentFig);
				}
				//END TEST
				currentFig=GetNextFigura((DrawObject*)pf1);

				while((currentFig!=NULL)&&(currentFig!=pf2)){
					
					raggrCorrente=currentFig->GetRaggruppamentoIniziale();
					if(raggrCorrente!=NULL && raggrCorrente!=raggrFinale)
						FigureCorrette=FALSE;
					currentFig=GetNextFigura((DrawObject*)currentFig);
					
				}
				if(FigureCorrette){
					if(listaInt.Add(SCmd.CommandId, pf1, pf2, pBat1, pBat2,
						(TipoInserimento)SCmd.ArgV[3], SCmd.ArgV[1],nc, voice,
						Multiv, StemInt, StemFirst, StemLast, Interm, SCmd.ArgV[4],(LineType)SCmd.ArgV[7])) // FN
					{
						SCmd.ArgV[6] = nc;
						return CMD_OK;
					}
				}
			}
		}
		return CMD_FAIL;
    case CMD_BATTUTA_ERASE:
		
		if(SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj))
        {
			EraseBattuta((Battuta*)pobj);
			return CMD_OK;
        }
		return CMD_FAIL;
    case CMD_DEL_SYMBOL:
		if (SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj))
		{
			DeleteSym((DrawObject*)pobj);
			return CMD_OK;
		}
		return CMD_FAIL;
  }
  return CMD_UNKNOWN;
};

void Spartito::AddBattuta(Battuta *bat)
{
	bat->SetParentScore(this);	
	listaBat.AddAfter(bat);
}

void Spartito::AddBattutaDopo(Battuta *bat)
{
	listaBat.CmdAddBattuta(bat,TRUE,0,this);	
}

void Spartito::SetStrumEsec(const char* StEsec)
{
	strcpy(strumes, StEsec);
}

const char *Spartito::GetStrumEsec()
{
	return strumes;
}

void Spartito::SetShortInstrName(const char* sn)
{
	strcpy(shortstrumes, sn);
}

void Spartito::AddIntEsteso(IntEsteso *intesteso)
{
	listaInt.Add(intesteso);
}

void Spartito::AddIntEsteso(IntEsteso *intesteso, ArgType& nc)
{
	listaInt.Add(intesteso,nc);
}

Bool Spartito::FigPtr2SymPath(SymPath& symPath, Figura *figptr)
{
	return FALSE;
	symPath.Array[SPARTITO_PATH_LEVEL]=GetNumericCode();
	return listaBat.FigPtr2SymPath(symPath,figptr);
}

void Spartito::SetCodeCounterInt(NumCode nc)
{
	listaInt.SetCodeCounter(nc);
}

NumCode Spartito::GetCodeCounterInt()
{
	return listaInt.GetCodeCounter();
}



void Spartito::SetCodeCounterSil(NumCode nc)
{
	listaSil.SetCodeCounter(nc);
};
NumCode Spartito::GetCodeCounterSil()
{
	return listaSil.GetCodeCounter();
};



void Spartito::SetCodeCounterBat(NumCode nc)
{
	listaBat.SetCodeCounter(nc);
}

NumCode Spartito::GetCodeCounterBat()
{
	return listaBat.GetCodeCounter();
}

void Spartito::EraseBattuta(Battuta *bat)
{
	ListaFigure *plF;
	Figura *pFig;
	short nLayer=0;
	
	while(nLayer<NUM_MAX_LAYER)
	{
		plF = bat->GetPLayer(nLayer);
		if(plF!=NULL)
		{
			pFig = plF->GetFirstF(); //serve a partire dal primo elemento della
			//lista di figure all'interno di quella  battuta
			while(pFig!=NULL)
			{
				listaInt.CancIntEstFig(pFig);
				
				listaSil.CancSilFig(pFig);
				
				pFig = plF->GetNextF(pFig);
			} 
		}
		nLayer++;
	}
	bat->EraseBattuta();
}

Bool Spartito::DeleteFigure(Battuta *bat, NumCode FigCode, short layer)
{
	ListaFigure *plF1,*plF2;
	Figura *pFig1, *pFig2;
	
	Spazio *spaziol=NULL;
	
	plF1 = bat->GetPLayer(0);
	pFig1 = plF1->GetFirstFig();
	
	plF2 = bat->GetPLayer(1);
	pFig2 = plF2->GetFirstFig();
	
	while(pFig1!=NULL && pFig2!=NULL)
	{
		if(pFig1->GetNumericCode()==FigCode)
		{
			listaInt.CancIntEstFig(pFig1);
			listaSil.CancSilFig(pFig1);
			
			plF1->DelPtrFig(pFig1, bat, layer,0);
			delete pFig1;
			
			listaInt.CancIntEstFig(pFig2);
			listaSil.CancSilFig(pFig2);
			
			Redraw();
			
			plF2->DelPtrFig(pFig2, bat, layer,0);
			delete pFig2;
			if(Battuta::GetAutoBeaming())
				bat->BeamingAuto();
			return TRUE;
		}
		pFig1 = plF1->GetNextFig();
		pFig2 = plF2->GetNextFig();
	}
	return FALSE;
}

// ritorna la figura successiva (che non sia uno spazio)
// cercando anche nella battuta successiva.

Figura *Spartito::GetNextFigura(DrawObject *pobj)
{
	Figura *pobj1;
	DrawObject *pBat, *pBat1;
	Layer *LFig=NULL;
	int layer;
	
	pBat = listaBat.GetBattWithFig((Figura*)pobj);
	if (pBat==NULL) return NULL;
	
	for(layer=0;layer<NUM_MAX_LAYER;layer++)
	{
		LFig = ((Battuta*)pBat)->GetPLayer(layer);
		if (LFig!=NULL && LFig->FindFig((Figura*)pobj))
		{       
			break;
		}
	}
	
	pobj1 = LFig->GetNextFigura((Figura*)pobj);
	
	while ((pobj1!=NULL) && (pobj1->GetID()==CL_SPAZIO))
		pobj1 = LFig->GetNextFigura((Figura*)pobj1);
	
	if (pobj1==NULL)
	{
		pBat1 = (Battuta*)(listaBat.GetNext(pBat));
		if (pBat1==NULL) return NULL;
		
		LFig = ((Battuta*)pBat1)->GetPLayer(layer);
		
		pobj1 = LFig->GetFirstF();
		
		if ((pobj1!=NULL) && pobj1->GetID()==CL_GRUPPONOTE)
			pobj1 = ((GruppoNote*)pobj1)->GetFirstF();
		
		while ((pobj1!=NULL) && (pobj1->GetID()==CL_SPAZIO))
			pobj1 = LFig->GetNextFigura((Figura*)pobj1);
	}
	
	return pobj1;
}


Bool Spartito::Ordina(Figura *& pf1, Figura *& pf2, Battuta *& pBat1, Battuta *& pBat2)
{
	Battuta *pBat;
	Figura *pf;
	int np1, np2;
	Bool fl;
	
	np1 = pBat1->GetNumProgress();
	np2 = pBat2->GetNumProgress();
	
	if (np1 > np2)
		fl = TRUE;
	else if (np1 == np2)
		fl = (pf1->GetAbsPos().x > pf2->GetAbsPos().x);
	else
		fl = FALSE;
	
	
	
	if (fl)
	{
		pf = pf1;
		pf1 = pf2;
		pf2 = pf;
		
		pBat = pBat1;
		pBat1 = pBat2;
		pBat2 = pBat;
		
	}
	
	return fl;
}



void Spartito::SetBattPage(Battuta *pbat)
{
	listaBat.SetBattPage(pbat);
}


void Spartito::SetfirstBatPage(Battuta *pbat)
{
	listaBat.SetfirstBatPage(pbat);
}

Battuta *Spartito::GetfirstBatPage()
{
	return listaBat.GetfirstBatPage();
}

Battuta *Spartito::GetfirstBatNextPage()
{
	return listaBat.GetfirstBatNextPage();
}

NumCode Spartito::GetPagCounter()
{
	return listaBat.GetPagCounter();
}

void Spartito::SetPagCounter(NumCode nc)
{
	listaBat.SetPagCounter(nc);
}

IntEsteso *Spartito::GetNumCodeInt(NumCode nc)
{
	return listaInt.GetNumCodeInt(nc);
}


Sillaba *Spartito::GetNumCodeSil(NumCode nc)
{
	return listaSil.GetNumCodeSil(nc);
};


void Spartito::InitImpagina()
{
	Battuta *pbt;
	
	pbt = GetBattNum(0);
	while (pbt!=NULL)
	{
		pbt->SetNumPagMASAE(0);
		pbt->SetNumPagMASE(0);
		pbt->SetNumPagDLIOO(0);
		if(pbt->GetPMultiRest()!=NULL)
		{
			pbt->GetPMultiRest()->SetNumPagDLIOO(0);
		}
		pbt = GetNextBat(pbt);
	}
}

DrawObject * Spartito::GetObject(SymCmd * MyCmd)
{
	switch(MyCmd->CommandId)
	{
	case CMD_SPARTITO_ADD_LEGATURA:
	case CMD_SPARTITO_ADD_LEGATURA_VALORE:
	case CMD_SPARTITO_ADD_FORC_CRESC:
	case CMD_SPARTITO_ADD_FORC_DIMIN:
	case CMD_SPARTITO_ADD_FRECCIA:
	case CMD_SPARTITO_ADD_ONDA:
	case CMD_SPARTITO_ADD_LEG_QUADRA:
	case CMD_SPARTITO_ADD_CAMBIO_RIT:
	case CMD_SPARTITO_ADD_MODIF8:
	case CMD_SPARTITO_ADD_MODIF15MA:
	case CMD_SPARTITO_ADD_MODIF8BA:
	case CMD_SPARTITO_ADD_MODIF15BA:
	case CMD_SPARTITO_NPENT:
		return ((DrawObject *)(GetNumCodeInt(MyCmd->ArgV[6])));
		break;
	case CMD_SPARTITO_ADD_SILLABA:
		return ((DrawObject *)(GetNumCodeSil(MyCmd->ArgV[6])));
		break;
		
		
	case CMD_SPARTITO_SET_NAME: return NULL; break;
	default: return NULL; break;
	}
	return NULL;
}


Bool Spartito::GetStem(NumCode nb1, Figura *pF1, NumCode nb2, Figura *pF2, Bool ForAll)
{
	Battuta *pBt1, *pBt2;
	Figura *pfig;
	int lay, lay1, lay2;
	int pos1, pos2, pos;
	ClassID Iden;
	Bool flUp = TRUE;
	int nb;
	ListaFigure *pLayer;
	
	
	if (nb1 == nb2)
	{
		pBt1 = GetBatt(nb1);
		
		lay1 = pBt1->GetVoice(pF1);
		pos1 = pBt1->GetPos(lay1, pF1);
		
		lay2 = pBt1->GetVoice(pF2);
		pos2 = pBt1->GetPos(lay2, pF2);
		
		for (lay=0; lay<NUM_MAX_LAYER; lay++)
			
			
			
		{
			pLayer = pBt1->GetPLayer(lay);
			if(pLayer!=NULL)
			{
				
				for(pos=pos1+1; pos<pos2; pos++)
				{
					pfig = pLayer->AtPosF(pos);
					if(pfig==NULL)
						break;
					Iden = pfig->GetID();
					if ((Iden == CL_NSEMIMINIMA) ||
						(Iden == CL_NCROMA) || (Iden == CL_NSEMICROMA) ||
						(Iden == CL_NBISCROMA) || (Iden == CL_NSEMIBISCROMA) ||
						(Iden == CL_NFUSA) || (Iden == CL_ACCORDO))
					{
						if (ForAll)
							if (((Nota*)pfig)->GetLGambo() < 0) return FALSE;
							else
								if (((Nota*)pfig)->GetLGambo() >= 0) return TRUE;
					}
				}
			}
		}
	}
	else if (nb2 > nb1)
	{
		pBt1 = GetBatt(nb1);
		
		lay1 = pBt1->GetVoice(pF1);
		pos1 = pBt1->GetPos(lay1, pF1);
		
		for (lay=0; lay<NUM_MAX_LAYER; lay++)
		{
			pLayer = pBt1->GetPLayer(lay);
			
			if(pLayer!=NULL)
			{
				pfig = pLayer->AtPosF(pos1+1);
				while(pfig!=NULL)
				{
					Iden = pfig->GetID();
					if ((Iden == CL_NSEMIMINIMA) ||
						(Iden == CL_NCROMA) || (Iden == CL_NSEMICROMA) ||
						(Iden == CL_NBISCROMA) || (Iden == CL_NSEMIBISCROMA) ||
						(Iden == CL_NFUSA) || (Iden == CL_ACCORDO))
					{
						if (ForAll)
							if (((Nota*)pfig)->GetLGambo() < 0) return FALSE;
							else
								if (((Nota*)pfig)->GetLGambo() >= 0) return TRUE;
					}
					pfig = pLayer->GetNextF(pfig);
				}
			}
		}
		
		
		
		for(nb=nb1+1;nb<nb2;nb++)
		{
			pBt1 = GetBatt(nb);
			
			for (lay=0; lay<NUM_MAX_LAYER; lay++)
			{
				pLayer = pBt1->GetPLayer(lay);
				
				if(pLayer!=NULL)
				{
					pfig = pLayer->GetFirstF();
					while(pfig!=NULL)
					{
						Iden = pfig->GetID();
						if ((Iden == CL_NSEMIMINIMA) ||
							
							(Iden == CL_NCROMA) || (Iden == CL_NSEMICROMA) ||
							
							(Iden == CL_NBISCROMA) || (Iden == CL_NSEMIBISCROMA) ||
							(Iden == CL_NFUSA) || (Iden == CL_ACCORDO))
						{
							if (ForAll)
								if (((Nota*)pfig)->GetLGambo() < 0) return FALSE;
								else
									if (((Nota*)pfig)->GetLGambo() >= 0) return TRUE;
						}
						pfig = pLayer->GetNextF(pfig);
					}
				}
			}
		}
		
		pBt2 = GetBatt(nb2);
		
		lay2 = pBt2->GetVoice(pF2);
		pos2 = pBt1->GetPos(lay2, pF2);
		
		for (lay=0; lay<NUM_MAX_LAYER; lay++)
		{
			pLayer = pBt2->GetPLayer(lay);
			
			if(pLayer!=NULL)
			{
				pfig = pLayer->AtPosF(1);
				while(pfig!=NULL)
				{
					Iden = pfig->GetID();
					if ((Iden == CL_NSEMIMINIMA) ||
						(Iden == CL_NCROMA) || (Iden == CL_NSEMICROMA) ||
						(Iden == CL_NBISCROMA) || (Iden == CL_NSEMIBISCROMA) ||
						(Iden == CL_NFUSA) || (Iden == CL_ACCORDO))
					{
						if (ForAll)
							if (((Nota*)pfig)->GetLGambo() < 0) return FALSE;
							else
								if (((Nota*)pfig)->GetLGambo() >= 0) return TRUE;
					}
					pfig = pLayer->GetNextF(pfig);
				}
			}
		}
		
	}
	
	if (ForAll)
		return TRUE;
	else
		return FALSE;
	
}


Bool Spartito::Intermed(NumCode nb1, Figura *pF1, NumCode nb2, Figura *pF2)
{
	Battuta *pBt1;
	Figura *pfig1;
	int lay1, lay2;
	int pos1, pos2, pos3;
	ClassID Iden;
	int nb3;
	Layer *pLayer1; 
	
	Bool flag = FALSE;
	
	pBt1 = GetBatt(nb1);
	lay1 = pBt1->GetVoice(pF1);
	pos1 = pBt1->GetPos(lay1, pF1);
	pLayer1 = pBt1->GetPLayer(lay1);
	pfig1 = pLayer1->AtPosF(pos1);
	nb3=nb1;
	
	pfig1 = pLayer1->GetNextF(pfig1);
	while ( pfig1 != NULL )
	{
		Iden = pfig1->GetID();
		if (  (Iden == CL_NBREVE) || (Iden == CL_NSEMIBREVE) ||
			(Iden == CL_NMINIMA) || (Iden == CL_NSEMIMINIMA) ||
			(Iden == CL_NCROMA) || (Iden == CL_NSEMICROMA) ||
			(Iden == CL_NBISCROMA) || (Iden == CL_NSEMIBISCROMA) ||
			(Iden == CL_NFUSA) || (Iden == CL_ACCORDO)              )
		{
			pos3 = pBt1->GetPos(lay1, pfig1);
			pfig1 = NULL;
			flag = TRUE;
		}
		else
		{
			pfig1 = pLayer1->GetNextF(pfig1);
			if (pfig1==NULL)
			{
				nb3++;
				pBt1 = GetBatt(nb3);
				if (pBt1!=NULL)
				{
					pLayer1 = pBt1->GetPLayer(lay1);
					pfig1 = pLayer1->AtPosF(1);
				}
			}
		}
	}
	
	if (flag)
	{
		pBt1 = GetBatt(nb2);
		lay2 = pBt1->GetVoice(pF2);
		pos2 = pBt1->GetPos(lay2, pF2);
		
		if (nb2>nb3)
			return TRUE;
		else if ((nb2==nb3) && (pos2>pos3))
			return TRUE;
		else
			return FALSE;
	}
	else
		return FALSE;
	
}

void Spartito::SetupCurrentSimboli()
{
	listaBat.SetupCurrentSimboli();
};


void Spartito::SetupIntRefs()
{
	Battuta *pBat;
	Lista activeInt[NUM_MAX_LAYER];
	
	pBat=(Battuta*)listaBat.GetFirst();
	while(pBat!=NULL)
	{
		pBat->ResetIntRefs();
		
		listaInt.SetupIntRefs(pBat,activeInt);
		
		pBat=(Battuta*)listaBat.GetNext(pBat);
	}
}

void Spartito::SaveCurrentPosition()
{
	listaBat.SaveCurrentPosition();
}

void Spartito::SetNumberOfStaff(short i) 
{
	if (i<1 || i >3) NumberOfStaff = 1;
	else
		NumberOfStaff = i;
}

short Spartito::GetNumberOfStaff() 
{
	return NumberOfStaff;
}


void Spartito::View_Link(bool view)
{
	Battuta *currBat;
	currBat=GetBattNum(0);
	while (currBat != NULL)
	{
		currBat->View_Link(view);
		currBat=GetNextBat(currBat);
	}
}

void Spartito::View_Layer(bool view, int num)
{
	Battuta *currBat;
	currBat=GetBattNum(0);
	while (currBat != NULL)
	{
		currBat->View_Layer(view,num);
		currBat=GetNextBat(currBat);
	}
}

void Spartito::Transfer_Layer(Spartito *fromSpa, int layerFrom)
{
	Battuta *currBat, *currBatfromSpa=NULL;
	
	currBat=GetBattNum(0);
	currBatfromSpa=fromSpa->GetBattNum(0);
	Movimento *movimento = new Movimento(currBatfromSpa->GetPMovimento());
	currBat->SetPMovimento(movimento);
	
	while ((currBat != NULL)&&(currBatfromSpa!=NULL))
	{
		currBat->CopyAllIntest(currBatfromSpa->GetAllIntest());
		currBat->Transfer_Layer(currBatfromSpa,layerFrom);
		
		currBatfromSpa=fromSpa->GetNextBat(currBatfromSpa);
		currBat=GetNextBat(currBat);
	}
	currBat=GetBattNum(0);
	while (currBat != NULL)
	{
		Trasfer_Interv(currBat,fromSpa);
		currBat=GetNextBat(currBat); 
	}
}

void Spartito::Transfer_Layer_MergeParts(Spartito *fromSpa)
{
	
	Battuta *currBat, *currBatfromSpa=NULL;
	
	currBat=GetBattNum(0);
	currBatfromSpa=fromSpa->GetBattNum(0);
	
	while ((currBat != NULL)&&(currBatfromSpa!=NULL))
	{
		currBat->Transfer_Layer_MergeParts(currBatfromSpa);
		currBatfromSpa=fromSpa->GetNextBat(currBatfromSpa);
		currBat=GetNextBat(currBat);
	}
	currBat=GetBattNum(0);
	while (currBat != NULL)
	{
		Trasfer_Interv(currBat,fromSpa);
		currBat=GetNextBat(currBat); 
	}
	
}


void Spartito::Multi2Single_Transfer_Layer(Spartito *fromSpa, int staffnum)
{
	Battuta *currBat, *currBatfromSpa=NULL;
	
	currBat=GetBattNum(0);
	currBatfromSpa=fromSpa->GetBattNum(0);
	if (currBatfromSpa->GetPMovimento()!=NULL)
	{
		Movimento *movimento = new Movimento(currBatfromSpa->GetPMovimento());
		currBat->SetPMovimento(movimento);
	}
	while ((currBat != NULL)&&(currBatfromSpa!=NULL))
	{
		currBat->CopyIntest(0,currBatfromSpa->GetIntest(staffnum));
		currBat->Multi2Single_Transfer_Layer(currBatfromSpa,staffnum);
		
		currBatfromSpa=fromSpa->GetNextBat(currBatfromSpa);
		currBat=GetNextBat(currBat); 
	}
	currBat=GetBattNum(0);
	while (currBat != NULL)
	{
		Trasfer_Interv(currBat,fromSpa);
		currBat=GetNextBat(currBat); 
	}
}

void Spartito::Single2Multi_Transfer_Layer(Spartito *fromSpa, int staffnum)
{
	Battuta *currBat, *currBatfromSpa=NULL;
	
	currBat=GetBattNum(0);
	currBatfromSpa=fromSpa->GetBattNum(0);
	if(currBat->GetPMovimento()==NULL)
	{
		if (currBatfromSpa->GetPMovimento()!=NULL)
		{
			Movimento *movimento = new Movimento(currBatfromSpa->GetPMovimento());
			currBat->SetPMovimento(movimento);
		}
	}
	
	while ((currBat != NULL)&&(currBatfromSpa!=NULL))
	{
		currBat->CopyIntest(staffnum,currBatfromSpa->GetIntest(0));
		currBat->Single2Multi_Transfer_Layer(currBatfromSpa,staffnum);
		
		currBatfromSpa=fromSpa->GetNextBat(currBatfromSpa);
		currBat=GetNextBat(currBat); 
	}
	currBat=GetBattNum(0);
	while (currBat != NULL)
	{
		Trasfer_Interv(currBat,fromSpa);
		currBat=GetNextBat(currBat); 
	}
	
}


void Spartito::Trasfer_Interv(Battuta *currBat,Spartito *fromSpa)
{
	ListaFigure *currLayer;
	ListaIntEst *listaIntfromSpa=fromSpa->GetListaIntervalli();
	Figura *fig=NULL;
	DrawObject *tmp=NULL;
	IntEsteso *interv=NULL;
	
	for(int lay=0; lay<NUM_MAX_LAYER; lay++)
	{
		currLayer=currBat->GetPLayer(lay);
		if(currLayer!=NULL)
		{
			fig=currLayer->GetFirstFig();
			while(fig!=NULL)
			{
				if(fig->GetID()!=CL_ACCORDO)
				{
					tmp=listaIntfromSpa->GetFirst();
					while(tmp!=NULL)
					{
						interv=(IntEsteso *)tmp;
						if(interv->GetPFigStart()==fig)
						{
							interv->SetBattuta(1,fig->GetPBattuta());
							interv->SetBattuta(2,interv->GetPFigEnd()->GetPBattuta());
							listaInt.Add(interv);
							tmp=listaIntfromSpa->GetNext(tmp);
							listaIntfromSpa->DelPtr(interv);
						}
						else
							tmp=listaIntfromSpa->GetNext(tmp);
						
					}
				}
				else
				{
					Figura *tmpfig = ((Accordo *) fig)->GetFirstF();
					tmp=listaIntfromSpa->GetFirst();
					while(tmpfig!=NULL)
					{
						while(tmp!=NULL)
						{
							interv=(IntEsteso *)tmp;
							if(interv->GetPFigStart()==tmpfig)
							{
								interv->SetBattuta(1,tmpfig->GetPBattuta());
								interv->SetBattuta(2,interv->GetPFigEnd()->GetPBattuta());
								listaInt.Add(interv);
								tmp=listaIntfromSpa->GetNext(tmp);
								listaIntfromSpa->DelPtr(interv);
							}
							else
								tmp=listaIntfromSpa->GetNext(tmp);
							
						}
						tmpfig = ((Accordo *) fig)->GetNextF(tmpfig);
					}
				}
				
				fig=currLayer->GetNextFig();
			}
		}
	}
}


void Spartito::Delete_Layer(int layer)
{
	
	Battuta *currBat=NULL;
	
	currBat=GetBattNum(0);
	
	while (currBat != NULL)
	{
		currBat->Delete_Layer(layer);
		currBat=GetNextBat(currBat);
	}
	
}

bool Spartito::IsSplittable()
{
	Battuta *currBat=NULL;
	currBat=GetBattNum(0);
	
	while (currBat != NULL)
	{
		if(currBat->GetNUsedLayers()>1)
			return true;
		currBat=GetNextBat(currBat);
	}
	return false;
}

bool Spartito::IsDisjoinable()
{
	Battuta *currBat=NULL;
	currBat=GetBattNum(0);
	
	while (currBat != NULL)
	{
		if(!currBat->IsDisjoinable())
			return false;
		currBat=GetNextBat(currBat);
	}
	return true;
}


void Spartito::SetMidiAttrib(Midi_ParamVoice *attrib)
{
	midiAttrib.Set_Instrument(attrib->Get_Instrument());
	midiAttrib.Set_MidiCode(attrib->Get_MidiCode());
	midiAttrib.Set_Volume(attrib->Get_Volume());
	midiAttrib.Set_Channel(attrib->Get_Channel());
	midiAttrib.Set_Muting(attrib->Get_Muting());
	midiAttrib.Set_Solo(attrib->Get_Solo());
	midiAttrib.Set_TrackNumber(attrib->Get_TrackNumber());
}

/****************************************************************/


void Spartito::AddCasella(Casella *casella, int Npage)
{
	Pagina* pag =listaPag.GetPage(Npage,true);
	pag->InsertEnd(casella);
}

void Spartito::DrawPage()
{
	Draw();
	listaPag.DrawPage(GetCurrentPage()); 
}

void Spartito::SetCurrentPage(int Npagina)
{
	currentPage[DrawObject::GetCurrentView()]=Npagina;
}

PrintParams *Spartito::GetPrintViewParams()
{
	if(TmpPrintViewParams!=NULL)
		return TmpPrintViewParams;
	if(PrintViewParams==NULL)
		PrintViewParams=new PrintParams;
	return PrintViewParams;
}

PrintParams *Spartito::GetComputerViewParams()
{
	if(ComputerViewParams==NULL)
	{
		ComputerViewParams=new PrintParams;
		ComputerViewParams->SetScaleFactor(1.0);
	}
	return ComputerViewParams;
}

void Spartito::SetComputerViewParams(int tm, int bm, int lm, int rm, int sd, float magnification)
{
	PrintParams *pars=GetComputerViewParams();
	pars->SetTopMargin(1,tm);
	pars->SetBottomMargin(1,bm);
	pars->SetLeftMargin(1,lm);
	pars->SetRightMargin(1,rm);
	pars->SetStaffDistance(1,sd);
	pars->SetMagnification(magnification);
}

void Spartito::SetTmpPrintViewParams(PrintParams* tmpPars)
{
	TmpPrintViewParams=tmpPars;
}

void Spartito::CopyToClipboard()
{
	
	FILE *fp;
	fp = fopen(SMRFile::GetTmpPath()+ PATH_SEP "clipboard.swf", "w");
	fprintf(fp,"<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
	fprintf(fp,"<SMXF_Part>");
	fclose(fp);	
}

void Spartito::GetPageMeasures(int& top, int& bottom)
{
	listaBat.GetPageMeasures(top,bottom);
}

bool Spartito::CutPageBox(DrawObject *obj)
{
	if(FindPageBox(obj,true))
	{
		Partitura::SetPageBox(obj);
		return TRUE;
	}
	else
		return FALSE;
}

bool Spartito::CopyPageBox(DrawObject *obj)
{
	if(FindPageBox(obj,false))
	{
		DrawObject *pagebox = Partitura::GetPageBox();
		if(pagebox!=NULL)
			delete pagebox;
		Partitura::SetPageBox(((Casella*)obj)->Clone());
		return TRUE;
	}
	else
		return FALSE;
}

bool Spartito::PastePageBox(const Point& p, int NPage)
{
	Pagina *pag;
	float h=0;
	float magn = Scr->GetMagnification();
	if(Partitura::GetPageBox()!=NULL)
	{
		DrawObject* clone = Partitura::GetPageBox();
		Casella *cas = ((Casella*)clone)->Clone();
		if((cas->GetHeight()!=10)&&(cas->GetWidth()!=10))
			h = cas->GetHeigh();
		cas->SetX(p.x*magn);
		cas->SetY((p.y-h)*magn);
		pag=listaPag.GetPage(NPage,false);
		if(pag!=NULL)
			pag->InsertEnd(cas);
		else
		{
			pag=listaPag.GetPage(NPage,true);
			pag->InsertEnd(cas);
		}
		return TRUE;
	}
	else 
		return FALSE;
}

bool Spartito::FindPageBox(DrawObject *obj, bool del)
{
	int i;
	for(i=1;i<MAX_VIEW+1;i++)
	{
		Pagina *pag=listaPag.GetPage(i,false);
		if(pag!=NULL)
		{
			if(pag->FindPtr(obj))
			{
				if(del)
					pag->DelPtr(obj);
				return TRUE;
				
			}
		}
	}
	return FALSE;
}

void Spartito::SwitchLayer(int from, int to)
{
	if(from==to)
		return;
	
	Battuta *currBatSpa=NULL;
	currBatSpa=GetBattNum(0);
	while (currBatSpa != NULL)
	{
		currBatSpa->SwitchLayer(from, to);
		currBatSpa=GetNextBat(currBatSpa);
	}
}

void Spartito::RemoveLyr()
{
	if(currentLyrItem!=NULL && currentLyrItem->GetIsModified())
	{
		wxMessageBox("saving lyric:"+currentLyrItem->GetFilename(),"",wxOK);
	}
	listaSil.Free();
	currentLyrItem=NULL;
}

void Spartito::EditLyric()
{
	extern wxMusicEditor *MusicEditor;
	
	if(currentLyrItem==NULL)
	{
		SelectLyric();
	}
	if(currentLyrItem!=NULL)
	{
		int pos = DrawObject::GetCurrentView();
		wxMEScoreFrame *scoreframe=MusicEditor->GetScoreFrame(pos);
		LyricFrame *lyr_frame=scoreframe->OpenLyricFrame();
		lyr_frame->WriteLyric(ExtractLyr(),this);
		lyr_frame->SetBllf(listaSil.GetBaseLineFactor());
		lyr_frame->SetFocus();
	}
}

#include <wx/textdlg.h>


bool Spartito::NewLyric()
{
	extern wxMusicEditor *MusicEditor;
	wxString language, description;
	
	int pos = DrawObject::GetCurrentView();
	wxMEScoreFrame *scoreframe=MusicEditor->GetScoreFrame(pos);
	if(!listaSil.isEmpty())
	{
		RemoveLyr();
	}
	
	wxTextEntryDialog dialog(scoreframe,  "Please enter the language","Settings lyric language");
	if(dialog.ShowModal()==wxID_OK)
		language = dialog.GetValue();
	else
		return false;
	
	wxString name=GetStrumEsec();
	if(name!="")
		description = name+":"+language;
	else
		description = language;
	
	LyricFrame *lyr_frame=scoreframe->OpenLyricFrame();
	
	lyr_frame->AddNewLyric("",this);
	lyr_frame->SetFocus();
	
	currentLyrItem = MusicEditor->smrf->AddLyrics("",GetNumericCode(), language, description);
	currentLyrItem->SetIsModified(true);
	
    return true;
}

const int MAX_DEEP = 5;

class LyricRefrainManager
{
private:
	int currentLine,startLine[MAX_DEEP],maxLine[MAX_DEEP];
	int maxAssignedNotes[MAX_DEEP],currentAssignedNotes[MAX_DEEP];
	Figura *currentFigure ,*endFigure[MAX_DEEP],*startFigure[MAX_DEEP];
	Battuta *currentMeasure, *endMeasure[MAX_DEEP],*startMeasure[MAX_DEEP];
	char refrainChr;
	int n;
	
	void SetCurrentFigure(Figura *pFig,Battuta *pBat)
	{  
		currentFigure=pFig; //puntatori a figura e battuta di start per lirica
		currentMeasure=pBat; 
	}
	
	bool PushFigure(Figura *pFig,Battuta *pBat, int line)
	{
		if(n>=MAX_DEEP)
			return false;
		
		startFigure[n]=pFig;
		startMeasure[n]=pBat;
		startLine[n]=line;
		maxLine[n]=line;
		n++;
		return true;
	}
	
	void PopFigure()
	{
		if(n>0)
			n--;
	}
	
	Figura *GetStartFigure() { return startFigure[n-1]; }
	Battuta *GetStartMeasure() { return startMeasure[n-1]; }
	int GetStartLine() { return startLine[n-1]; }
	Figura *GetEndFigure() { return endFigure[n-1]; }
	Battuta *GetEndMeasure() { return endMeasure[n-1]; }
	
	void SetEndFigure(Figura *pFig,Battuta *pBat)
	{
		endFigure[n-1]=pFig;
		endMeasure[n-1]=pBat;
	}
	
	int GetMaxLine() { return maxLine[n-1]; }
	void SetMaxLine(int line) 
	{
		for(int i=0; i<n; i++)
			if(line>maxLine[i])
				maxLine[i]=line;
	}
	
	int& MaxAssignedNotes() { return maxAssignedNotes[n-1]; }
	int& CurrentAssignedNotes() { return currentAssignedNotes[n-1]; }
	
public:
	
	LyricRefrainManager(Figura *pF,Battuta *pB)
	{
		SetCurrentFigure(pF,pB);
		currentLine=1;
		n=0;
		refrainChr=' ';
		for(unsigned int i=0;i <MAX_DEEP;i++)
		{  	
			startFigure[i]=endFigure[i]=NULL;
			startMeasure[i]=endMeasure[i]=NULL;
		}		
	}
	
	int GetCurrentLine() { return currentLine; }
	
	Figura *GetCurrentFigure() { return currentFigure; }
	Battuta *GetCurrentMeasure() { return currentMeasure; }
	
	char GetRefrainChr() { return refrainChr; }
	
	bool ScanRefrainChr(wxString &syl)
	{
		refrainChr =' ';
		char s=syl[(unsigned int) 0];
		
		if(s=='[')
		{
			if(!PushFigure(GetCurrentFigure(),GetCurrentMeasure(),currentLine))
			{
				wxMessageBox("ERROR: too many nested refrain operators");
				return false;
			}
			refrainChr ='[';
			syl.Remove(0,1);
			CurrentAssignedNotes()=1;
			MaxAssignedNotes()=0;
		}
		else if(s=='%')
		{
			if(CurrentAssignedNotes()>MaxAssignedNotes())
			{
				SetEndFigure(GetCurrentFigure(),GetCurrentMeasure());
				MaxAssignedNotes()=CurrentAssignedNotes();
				CurrentAssignedNotes()=1;
			}
			SetCurrentFigure(GetStartFigure(),GetStartMeasure());
			currentLine=GetMaxLine()+1;
			if(currentLine>MAX_LYRIC_LINES)
			{
				wxMessageBox("ERROR: too many lyric lines");
				return false;
			}
			SetMaxLine(currentLine);
			if(syl.Last()==']')
				refrainChr ='2';
			else
				refrainChr ='%';
			syl.Remove(0,1);
		}
		else
		{
			if(n>0)
				CurrentAssignedNotes()++;
		}
		if(!syl.IsEmpty() && syl.Last()==']')
		{
			syl.RemoveLast();
			if(refrainChr!='2')
				refrainChr=']';
		}
		return true;
	}
	
	void UpdateCurrentFigure(Figura *pF,Battuta *pB)
	{
		if(refrainChr==']' || refrainChr=='2')
		{
			int maxNotes=0;
			if(CurrentAssignedNotes()>MaxAssignedNotes())
			{
				SetCurrentFigure(pF,pB);
				maxNotes=CurrentAssignedNotes();
			}
			else
			{
				SetCurrentFigure(GetEndFigure(),GetEndMeasure());
				maxNotes=MaxAssignedNotes();
			}
			currentLine=GetStartLine();
			PopFigure();
			if(n>0)
				CurrentAssignedNotes()+=maxNotes;
		}
		else
			SetCurrentFigure(pF,pB);
	}
}; 

wxString Spartito::ExtractLyr()
{	
	return listaSil.ExtractLyr();
}



int Spartito::AssignLyric(wxString Testo, double is, double baselinea)
{      
	Battuta *pBatPrec = NULL;
	Figura *pFigPrec = NULL;
	Figura *pf1 = NULL;
	Battuta *pBat1 = NULL;
	NumCode nc,nb1,nb2;
	ListaFigure *LFig;
	
	listaSil.Free();
	if(currentLyrItem!=NULL)
		currentLyrItem->SetIsModified(true);
	
	pBat1 = GetBattNum(0);
	while(pBat1 != NULL)
	{
		LFig=pBat1->GetPLayer(0);
		pf1=LFig->GetFirstFig();
		while( pf1 != NULL &&  (!pf1->IsANote() || (pf1->GetNotina() && pf1->GetStatus()==GRACED)) )
		{
			pf1 = LFig->GetNextFig();
		}
		if(pf1 != NULL)
			break;
		pBat1 = GetNextBat(pBat1);
	}
	if(pBat1 == NULL)
		return FALSE;
	if(pf1 != NULL && pf1->GetNStaff()>0)
	{
		wxMessageBox("Only first staff can be linked with Lyric","Error");
		return false;
	}
	
	Battuta *pBatSuc=pBat1;
	Battuta *pBatCor=pBat1;                 	
	Figura *pfCor=pf1;
	
	LyricRefrainManager lyrMgr(pfCor,pBatCor);
	
	bool Assegnable = true; 	
	int DUR=0;	
	int LayerIndex =0;
	nc = 0;	
	
	if( Testo.Length()>1 && pBat1 != NULL)	
	{
		WordExploder inputText(Testo);
		wxString curSyl;
		
		while (inputText.GetSyllable(curSyl)!=SYL_END)
		{
			Assegnable = inputText.GetCurrentSyllableType()!=SYL_TEXT;
			
			if(Assegnable)
			{			
				if(!lyrMgr.ScanRefrainChr(curSyl))
				{
					listaSil.Free();
					return FALSE;
				}
				
				pfCor =lyrMgr.GetCurrentFigure();
				pBatCor=lyrMgr.GetCurrentMeasure();
				pFigPrec=pfCor;
				pBatPrec=pBatCor;
				
				Figura *pf2 = NULL;
				Battuta *pBat2 = NULL;
				
				Figura *prec=NULL;
				Figura *ptr=NULL;
				pf1=pf2=pfCor;	
				pBat1=pBat2=pBatCor;
				
				if(inputText.GetCurrentSeparatorType()!='\0')
				{	
					if(inputText.GetCurrentSeparatorType()=='/' || inputText.GetCurrentSeparatorType()=='e' ||
						isBlankSeparator(inputText.GetCurrentSeparatorType()))
						DUR=1;
					else if(inputText.GetCurrentSeparatorType()=='@')
						DUR=inputText.GetCurrentSeparatorCount();
					else
						DUR=inputText.GetCurrentSeparatorCount()+1;
					
					LayerIndex =  listaSil.GetNumLayerWithPFigSpecified(pBat1,pf1);
					LFig = pBat1->GetPLayer(LayerIndex);
					ptr = LFig->GetFirstFig();;
					while(pf1!=ptr)
						ptr=LFig->GetNextFig();
					
					for( int i=0; i<DUR; i++)	
					{
						prec=ptr;
						ptr=LFig->GetNextFig();
						Bool Spazio=TRUE;
						while ((Spazio))
						{
							if(ptr!=NULL)
							{
								if( !ptr->IsANote() || (ptr->GetNotina() && ptr->GetStatus()==GRACED) )
								{
									Spazio=TRUE;
									ptr=LFig->GetNextFig();
								}
								else
									Spazio=FALSE;
							}
							else
							{
								pBatSuc=GetNextBat(pBatCor);
								
								if (pBatSuc==NULL)
								{
									WordExploder tmpExp=inputText;
									wxString nextSyl;
									
									while(tmpExp.GetSyllable(nextSyl)==SYL_TEXT)
										;
									
									if(tmpExp.GetCurrentSyllableType()==SYL_END || 
										nextSyl[(unsigned int)0]=='%' || (inputText.GetCurrentSeparatorType()=='_' && i==DUR-1))
									{
										pBat2 = pBatCor;
										ptr=prec;
										Spazio=FALSE;
									}
									else
									{
										wxMessageBox("Not enought notes for the lyric", "Warning");
										listaSil.Free();
										return FALSE;
									}
								}
								else
								{
									pBatCor=pBatSuc;
									pBat2 = pBatSuc;
									LFig=pBat2->GetPLayer(LayerIndex);
									ptr=LFig->GetFirstFig();
								}
							}
						}
					};					
					
					if(!isBlankSeparator(inputText.GetCurrentSeparatorType()) && inputText.GetCurrentSeparatorType()!='e')
						pf2=ptr;
					else
					{
						pf2=pf1;
						pBat2=pBat1;
					};
					pfCor=ptr;
				}
				else        
				{
					pf2=pf1;
					pBat2=pBat1;
				};
				
				if ((pBat2==NULL) || (pBat1==NULL) || (pf1==NULL) || (pf2==NULL))
					return FALSE;
				
				Ordina(pf1, pf2, pBat1, pBat2);		
				nb1 = pBat1->GetNumericCode();
				nb2 = pBat2->GetNumericCode();
				nc = 0;
				
				const char *syl=curSyl.c_str();
				char tm ='N';
				int lt = 0;
				
				switch(inputText.GetCurrentSeparatorType())
				{
				case '/' :
					tm ='/';
					lt = 1;
					listaSil.Add(syl, pf1, pf2, pBat1, pBat2,0,TRUE,TRUE,tm,lt,lyrMgr.GetCurrentLine(),
						lyrMgr.GetRefrainChr(),LayerIndex,Assegnable, is, baselinea);
					break;  
				case'-': 
					tm ='-';
					lt = DUR-1;
					listaSil.Add(syl, pf1, pf2, pBat1, pBat2,DUR-1,TRUE,FALSE,tm,lt,lyrMgr.GetCurrentLine(),
						lyrMgr.GetRefrainChr(),LayerIndex,Assegnable, is, baselinea); 
					break; 
				case'@': 
					tm ='@';
					lt = DUR;
					listaSil.Add(syl, pf1, pf2, pBat1, pBat2,DUR,TRUE,FALSE,tm,lt,lyrMgr.GetCurrentLine(),
						lyrMgr.GetRefrainChr(),LayerIndex,Assegnable, is, baselinea); 
					break;          
				case'_': 
					tm ='_';
					lt = DUR-1;
					listaSil.Add(syl, pf1, prec,pBat1, pBat2,DUR-1,FALSE,FALSE,tm,lt,lyrMgr.GetCurrentLine(),
						lyrMgr.GetRefrainChr(),LayerIndex,Assegnable, is, baselinea); 
					break;
				case '\0':
					tm ='0';
					lt = 1;
					listaSil.Add(syl, pf1, pf2, pBat1, pBat2,0,FALSE,FALSE,tm,lt,lyrMgr.GetCurrentLine(),
						lyrMgr.GetRefrainChr(),LayerIndex,Assegnable, is, baselinea); 
					break;
				case ' ':
				case '\n':
				case '\t':
				case 'e': //empty
					tm =inputText.GetCurrentSeparatorType();
					lt = 1;
					listaSil.Add(syl, pf1, pf2, pBat1, pBat2,0,FALSE,FALSE,tm,lt,lyrMgr.GetCurrentLine(),
						lyrMgr.GetRefrainChr(),LayerIndex,Assegnable, is, baselinea); 
					break; 
				}
				
				lyrMgr.UpdateCurrentFigure(pfCor,pBatCor);
			}	
			else
			{   
				const char *syl=curSyl.c_str();
				
				listaSil.Add(syl, NULL, NULL, NULL, NULL,0,FALSE,FALSE,'\0',0,1,' ',0,Assegnable,is,baselinea);
			}
		}
	};
	return TRUE;
}

void Spartito::SelectLyric()
{
	extern wxMusicEditor *MusicEditor;
	SMRItemList LyrList;
    wxString Descrizione = "";
	wxString nomefile;
	wxString lingua;
	
	MusicEditor->smrf->GetItems(SMR_Lyrics,LyrList);
	
	int nLyr=LyrList.GetCount();
	if(nLyr==0)
	{
		Message("No Lyric files present");
		return;
	}
	
	wxString *Lyrs=new wxString[nLyr+1];
	SMRLyricsItem **Lwf=new SMRLyricsItem*[nLyr+1];
    SMRLyricsItem *lwf;
	int n=1;
	Lyrs[0]="No Lyric";
	Lwf[0]=NULL;
	
	wxSMRItemListNode *node = LyrList.GetFirst();
	while (node)
	{
		lwf= (SMRLyricsItem*)node->GetData();
		nomefile =lwf->GetFilename();
		lingua = lwf->GetLanguage();
		
		if(lwf->GetPartID()==GetNumericCode())
		{
			Lyrs[n]=lwf->GetItemDescription();
			Lwf[n]=lwf;
			n++;
		}
		
		node = node->GetNext();
	}
	
	if(n==1)
	{
		delete[] Lyrs;
		delete[] Lwf;
		wxMessageBox(wxString::Format("No Lyric files present for %s","nomeFile"));
		return;
	}
	
	wxSingleChoiceDialog LyrChoice(FrameManager::currFrame,"Select the lyric","lyric resources",n,Lyrs);
	
	LyrChoice.SetSize(300,200);
	LyrChoice.Centre();
	
	if(LyrChoice.ShowModal()==wxID_OK)
	{
		RemoveLyr();
		
		lwf = Lwf[LyrChoice.GetSelection()];
		
		if(lwf!=NULL)
		{
			wxString lyrFile;
			currentLyrItem=lwf;
			SMRFile *smrf=MusicEditor->smrf;
			lyrFile = smrf->GetPath()+PATH_SEP+lwf->GetFilename();
			LoadLyr((char *)lyrFile.c_str());
		}
	}
	
	delete[] Lyrs;
	delete[] Lwf;
}


bool Spartito::AlignTo(int measures)
{
	int measuresToAdd=measures-GetNumeroBattute();
	
	if(measuresToAdd<0)
		return false;
	
	for(int i=1; i<=measuresToAdd; i++)
	{
		AddBattutaDopo(GetLastBat());
	}
	return true;
}