/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: spartito.hpp,v 1.1 2005/07/25 09:35:32 bellinpo Exp $ */



#ifndef SPARTITO_HPP
#define SPARTITO_HPP

#include "lioo.hpp"
#include "drw/drawobje.hpp"
#include "spa/listabat.hpp"
#include "spa/listaint.hpp"
#include "pag/listapagine.hpp"
#include "pag/casella.hpp"
#include "mdo/midiparamvoice.hpp"

#include "lyr/listasillabe.hpp"
#include "lyr/wordexploder.hpp"

#include<wx/string.h>
#include<wx/choicdlg.h>


#include "spa/pentagra.hpp"

class MoodsMidiWrite;

class PrintParams;
class SMRLyricsItem;

namespace xercesc_2_6
{
	class InputSource;
}

class Spartito: public DrawObject
{
protected:
	
    tipoSpartito tipoSpart;
	// Tipo di spartito
    int nRighi[MAX_VIEW];
	// numero di righi di pentagramma.
    Pentagramma pentagr[MAXRIGHI];
	// pentagrammi visibili dello spartito.
    ListaBattute listaBat;
	// lista contenente tutte le battute dello spartito.
    ListaIntEst listaInt;
	// lista di tutti i simboli di intervallo.
	
	int NCorde[3];	
	
	int currentPage[MAX_VIEW];	
	
    ListaSillabe listaSil;
	// lista di tutte le sillabe della lirica.
	ListaPagine listaPag;
	
	bool visible;
	Midi_ParamVoice midiAttrib;
	
    NumCode NumericCode;
	// Codice numerico dello spartito.
	
    int SpaceInit;
	int SpaceInit_FirstSystem;
    char strumes[STRUM_NAME_LEN];
	
    void DrawPentagrammi();
	// disegna i pentagrammi.
	Bool ProlBarraUp, ProlBarraDown;
	short NumberOfStaff; 
	// numero di pentagrammi di cui e' composta la battuta (default 1)
	PrintParams *ComputerViewParams;
	PrintParams *PrintViewParams;
	PrintParams *TmpPrintViewParams;
	SMRLyricsItem *currentLyrItem;
	int instrumentType;

	int transpose;
	bool smallSize;
    char shortstrumes[STRUM_NAME_LEN];

//	void DrawContour();
	
private:
    Figura *GetNextFigura(DrawObject *pobj);
	// ritorna la figura successiva (che non sia uno spazio)
	// cercando anche nella battuta successiva.
	
    Bool Ordina(Figura *& pf1, Figura *& pf2, Battuta *& pBat1, Battuta *& pBat2);
	// controlla che f1 preceda f2, altrimenti scambia i puntatori.
	// Restituisce FALSE se le figure non appartengono allo stesso layer.
	
	
public:
    Spartito(short nStaff=1);
	// inizializza i dati
    ~Spartito();
	// richiama Free()
	
    void Suona(MoodsMidiWrite *);    
	
	int GetNCorde(int nstaff=0){return NCorde[nstaff];}
	void SetNCorde(int nstaff, int ncorde);
	VUnit GetBaseVU2Up();
	VUnit GetBaseVU2Dwn();
	VUnit GetBaseVU2Up(int staff);
	VUnit GetBaseVU2Dwn(int staff);
	
	VUnit GetPentHeight(){return pentagr[0].GetPentHeight();};
	
    void AddCasella(Casella *casella, int Npage);
	void DrawPage();
	
	int GetCurrentPage(){return currentPage[DrawObject::GetCurrentView()];}
	void SetCurrentPage(int Npagina);
	
	
    void Free();
	// Svuota la lista delle battute,quella degli intervalli,
	// e quella delle sillabe.
	
    void New();
	// inizia un nuovo spartito
    void Init();
    void SetTipo(tipoSpartito ts);
	// Imposta il tipo.
    tipoSpartito GetTipo();
	// Ritorna il tipo impostato.
    
    void SetWidth(VUnit w,int system=0);
    void SetMaxWidth(VUnit w,int system=0);
    void SetSpaceUpDwn(VUnit sp,int system=0);
	VUnit GetSpaceUpDwn(); 
	
    VUnit GetVU2Up();
    VUnit GetVU2Dwn();
    VUnit GetVU2Lft();
    VUnit GetVU2Rgt();
    VUnit GetMaxWidth(int system=0);
	
	void SetProlBarraUp(Bool);
	void SetProlBarraDown(Bool);
	Bool GetProlBarraUp();
	Bool GetProlBarraDown();
	
	void SetSpaceInit(int);
	int GetSpaceInit();
	
	void SetSpaceInit_FirstSystem(int);
	int GetSpaceInit_FirstSystem();
	
	unsigned long GetNumeroBattute();
	unsigned long GetNumeroBattuteVere();
	void InitImpagina();
	
    void SetNumericCode(NumCode nc);
    NumCode GetNumericCode();
	
    NumCode GetPagCounter();
	// Restituisce il valore dell'attributo PagCounter di ListaBattute.
    void SetPagCounter(NumCode);
	// Imposta il valore dell'attributo PagCounter di ListaBattute.
    void SetBattPage(Battuta*);
	// imposta la battuta passata come parametro come di
	// inizio pagina con il numero di pagina appropriato.
    void SetfirstBatPage(Battuta *);
	// imposta il valore $firstBatPage$ di ListaBattute.
    Battuta *GetfirstBatPage();
	// restituisce il valore $firstBatPage$ di ListaBattute.
    Battuta *GetfirstBatNextPage();
	// restituisce il valore $firstBatNextPage$ di ListaBattute.
	
	void SaveCurrentPosition();
	
    void InsertSil( Figura *Fig1,Figura *Fig2,char *TESTO,char TOKEN,
		Bool TRAT,Bool TRASIL);
	// Permette di inserire in una lista corrente
	// la lirica associata allo spartito corrente
	// e memorizzata in un file xml.Viene richiamata
	// da LOADLYR:
    void LoadLyr(char *Name,xercesc_2_6::InputSource* src=NULL);
	// Richiamato da ListaSpartiti carica,se
	// presente,la lirica associata allo
	// spartito corrente.
	
    void DeleteBat(Battuta *bat);
	// cancella la battuta.
    Bool DeleteSym(DrawObject *s);
	// cancella il simbolo puntato da s.
	
    void SetNPentagrammi(int, const class Rectangle&);
	// imposta il numero di pentagrammi e li posiziona nel rettangolo indicato.
	void SetNPentagrammi(int, const class Rectangle&, bool printing);
	// imposta il numero di pentagrammi e li posiziona nel rettangolo indicato 
	// inviando allo spartito	il valore del flag di stampa
    int GetNPentagrammi();
	// restituisce il numero di pentagrammi settati.
	
    void GoBatt(NumCode nb);
    void GoBattInExecution(NumCode nb);
	// Porta lo spartito alla battuta nb
    Bool GoForward();
	// scorre di una battuta indietro.
    Bool GoBackward();
	// scorre di una Battuta in avanti.
	
    Bool GoNextPage();
	// scorre di una pagina avanti
    Bool GoNextPageInExecution();
	
    Bool GoPrevPage();
	// scorre di una pagina dietro
    Bool PageUp();
	// scorre di una pagina indietro.
    Bool PageDwn();
	// scorre di una pagina in avanti.
	
    Battuta* GetBattNum(int n);
	// Restituisce il puntatore all'n-esima battuta della lista.
    Battuta* GetBattCompNum(int n, Bool forward, int &nb);
	// Restituisce il puntatore all'n-esima battuta della lista
	// in modalita' compressione multirest.
    void ComprimiBattute(Bool abil_compress);
	
    Battuta* GetBatt(NumCode nc);
	// Restituisce il puntatore della battuta che ha codice numerico uguale a
	// nc
    Battuta* GetBattNumProg(NumCode np);
	// Restituisce il puntatore della battuta che ha numero progressivo uguale a
	// np
	Battuta* GetFirstBat();
	// Restituisce il puntatore alla prima battuta.
	Battuta* GetNextBat(Battuta* pbt);
	// Restituisce il puntatore alla battuta successiva.
    Battuta* GetPrevBat(Battuta* pbt);
	// Restituisce il puntatore alla battuta precedente.
    Battuta* GetLastBat();
	// Restituisce il puntatore all'ultima battuta.
	
    void SetStrumEsec(const char*);
	// imposta il nome dello strumento di esecuzione
    const char *GetStrumEsec();
	// ritorna il nome dello strumento che e' stato caricato.
	// ritorna "" se non e' stato caricato niente.
	
    void SetPos(DrawObject *d,const Point& p, int System=0);
	// Imposta la posizione dello spartito e del pentagramma (modo master).
	
	Point GetAbsPos(int System=-1);
	bool GetIsUsed(int System);
	
    void ResetCurrent(Node* SavedPos = NULL);
    Bool CheckCurrent(VUnit& IntWdt, VUnit& Wdt);
    
    // aggiunto sistema 
    void SetupCurrent(VUnit PosX, VUnit IntWdt, VUnit MaxWdt, int System=0);
    
    Battuta* GetCurrentBat();
	// quello della lista senza controlli null
	Battuta* GetCurrentBatLst();
	
	Bool MoveCurrentBatForw();
	
	
	
	void CurrentInNextPage();
	
    void PlaceIntervalli(Bool onlyLegature=TRUE);
	// Dispone i simboli di intervallo.
	
	
    void PlaceSillabe(void);
	// Dispone le sillabe.	
	
	
    void Draw();
	// disegna la pagina attuale dello spartito.
    void DrawInExecution();
    void Redraw() ;
	// ridisegna la pagina attuale dello spartito ricalcolando le posizioni.
	
    void DrawTagli(const Point&);
	// disegna i tagli necessari per arrivare al punto indicato.
	
    Bool Hit(const Point&,SelObj,DrawObject*&,SymPath&);
	// identifica un oggetto.
    Bool Hit(const class Rectangle&,SelObj,short layer,SymPath&,SymPath&);
    Bool SymbolicHit(SymPath&,SelObj,DrawObject*&);
	// Decodifica un path.
	
	
    void AddBattuta(Battuta *bat);
	// aggiunge una battuta alla fine dello spartito
    void AddBattutaDopo(Battuta *bat);
	// aggiunge una battuta dopo bat (utilizzata da listaspartiti).
	
    void AddIntEsteso(IntEsteso *intesteso);
	// aggiunge un intervallo esteso nella lista intervalli estesi di spartito.
	// senza impostare il numeric code (gestito a mano)
    void AddIntEsteso(IntEsteso *intesteso,ArgType& nc);
	// aggiunge un intervallo esteso nella lista intervalli estesi di spartito.
	// impostando il numeric code (ritorna in nc).
	
	int AssignLyric(wxString Testo, double is, double baselinea);
	wxString ExtractLyr();
	
	void RemoveLyr();
	void EditLyric();
	bool NewLyric();
	void SelectLyric();
	
	ListaSillabe* GetListaSillabe() { return &listaSil; }
	
	void SetVisible(bool value){visible=value;};
	
	bool IsVisible(){return visible;};
	void Transfer_Layer(Spartito *fromSpa, int layerFrom);
	void Delete_Layer(int layer);
	bool IsSplittable();
	bool IsDisjoinable();
	void Trasfer_Interv(Battuta *currBat,Spartito *fromSpa);
	void Multi2Single_Transfer_Layer(Spartito *fromSpa, int staffnum);
	void Single2Multi_Transfer_Layer(Spartito *fromSpa, int staffnum);
	Midi_ParamVoice *GetMidiAttrib() {return &midiAttrib;};
	void SetMidiAttrib(Midi_ParamVoice *attrib);

	void Transfer_Layer_MergeParts(Spartito *fromSpa);
	void SwitchLayer(int from, int to);
	
	ListaIntEst *GetListaIntervalli() {return &listaInt;};
	
	
    Bool FigPtr2SymPath(SymPath&, Figura*);
	// determina il path a partire da un puntatore.
	
    void SetCodeCounterBat(NumCode);
    NumCode GetCodeCounterBat();
    
    void SetCodeCounterSil(NumCode);
    NumCode GetCodeCounterSil();
	
    void SetCodeCounterInt(NumCode);
    NumCode GetCodeCounterInt();
	
    IntEsteso *GetNumCodeInt(NumCode nc);
	
	
    Sillaba *GetNumCodeSil(NumCode nc);
	
	
    void SetupCurrentSimboli();
	
    void EraseBattuta(Battuta *bat);
	//sostituisce la battuta selezionata con degli spazi
	//cancella inoltre eventuali simboli di intervallo esteso
	
    Bool DeleteFigure(Battuta *bat,NumCode FigCode, short layer);
	// Cancella la figura del layer 1 con codice FigCode e la
	// sottostante nel layer 2. Cancella anche gli intervalli collegati
	// e le sillabe,con eventuali prolungamenti,presenti.
	
    Bool GetStem(NumCode nb1, Figura *pF1, NumCode nb2, Figura *pF2, Bool
		ForAll=TRUE);
    Bool Intermed(NumCode nb1, Figura *pF1, NumCode nb2, Figura *pF2);
	
	//    CmdResult Command(CmdID,CmdArgV);
    CmdResult SymCommand(SymCmd&, Battuta *pb=NULL);
	// esegue comandi sullo spartito.
	// il parametro viene passato per riferimento, perche' per mantenere
	// consistenza tra i dati, e' necessario che i numeric code delle nuove battute
	// e dei nuovi intervalli e i numeri progressivi delle nuove battute
	// vengano impostati da MASAE e trasmessi in rete agli altri leggii
	// attraverso gli argomenti ArgV[6] per il numeric code e ArgV[7]
	// per il numero progressivo
	
    char *Describe(Battuta *);
	
    DrawObject * GetObject(SymCmd * MyCmd);
	// dato un SymCmd con un comando di spartito restituisce l'oggetto colpito

	const char * GetNome(void) { return strumes; };
	const char * GetShortInstrName(void) { return shortstrumes; };
	void SetShortInstrName(const char* sn);

	int GetTranspose() { return transpose; }
	void SetTranspose(int t) { transpose=t; }

	bool IsSmallSize() { return smallSize; }
	void SetSmallSize(bool s) { smallSize=s; }

	// metodo per la stampa
	void print(char codeWM_1[5],  char codeWM_2[5],  char codeWM_3[5],  char codeWM_4[7]);
	void printdir(char codeWM_1[5],  char codeWM_2[5],  char codeWM_3[5],  char codeWM_4[7]);
	
	void ResetfirstBatNextPage();
	// mette a NULL firstBatNextPage per permettere di stampare i sistemi successivi al primo
	
	
	
	// Qui si setta il numero di
	// pentagrammi perogni battuta
	// Gli incici vanno da 1 a 3 compresi.
	//
	// Per Intestazione* intest[NUM_MAX_INTEST]
	// Gli indici vanno da 0 a 2,
	
	void SetNumberOfStaff(short i); 
   	short GetNumberOfStaff(); 
	
    void SetupIntRefs();
    void SetNRighi(int nr) { nRighi[GetCurrentView()]=nr; };
    int GetNRighi() { return nRighi[GetCurrentView()]; };
    void View_Link(bool);
    void View_Layer(bool view, int num);
	
	PrintParams *GetPrintViewParams();
	PrintParams *GetComputerViewParams();
	void SetComputerViewParams(int tm, int bm, int lm, int rm, int sd, float magnification=1.0);
	void SetTmpPrintViewParams(PrintParams*);
	void CopyToClipboard();	
	bool CutPageBox(DrawObject *obj);  
	bool CopyPageBox(DrawObject *obj); 
	bool PastePageBox(const Point& p,int NPage); 
	bool FindPageBox(DrawObject *obj,bool del);
	void GetPageMeasures(int& top, int& bottom);
	
	void SetCurrentLyrWF(SMRLyricsItem* lyr) { currentLyrItem=lyr; };
	SMRLyricsItem *GetCurrentLyrWF() { return currentLyrItem; };
	
	void SetInstrumentType(int itype) { instrumentType=itype; };
	int GetInstrumentType() { return instrumentType; };
	
	bool AlignTo(int measures);

	int Save(FILE *, char*);
    int SaveXML(FILE *, char *lyrFileName=NULL);
};

#endif //SPARTITO_HPP
