/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: arco.cpp,v 1.1 2005/07/25 09:35:32 bellinpo Exp $ */




#include "str/arco.hpp"
#include "str/ponticel.hpp"
#include "str/tastiera.hpp"
#include "str/punta.hpp"
#include "str/tallone.hpp"
#include "str/arcosu.hpp"
#include "str/arcogiu.hpp"

Arco::Arco()
{
	Init();
}

Arco::~Arco()
{
	Free();
}

void Arco::Init()
{
	ID=CL_ARCO;
	ScrivereArco=FALSE;	 
	
	Violino::Init();
	SetPDirezArco(NULL);
	SetPPosizArco(NULL);
	SetPParteArco(NULL);
	
	SetVU2Up(0);
	SetVU2Dwn(0);
	SetVU2Lft(0);
	SetVU2Rgt(0);
	
}


void Arco::SetPDirezArco(DirezArco *pd)
{
	PDirezArco=pd;
}

DirezArco* Arco::GetPDirezArco()
{
	return PDirezArco;
}

void Arco::SetPPosizArco(PosizArco *pp)
{
	PPosizArco=pp;
}

PosizArco* Arco::GetPPosizArco()
{
	return PPosizArco;
}

void Arco::SetPParteArco(ParteArco *pa)
{
	PParteArco=pa;
}

ParteArco* Arco::GetPParteArco()
{
	return PParteArco;
}


void Arco::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	if(ScrivereArco)
	{
		int oldFont;
		char code[5]=CODE_ARCO;
		
		oldFont=Scr->SetFont(nTextFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXARCO,DELTAYARCO));
		Scr->OutText((unsigned char *)code);
		Scr->SetFont(oldFont);
		
	}
	
	//Disegno dell'eventuale simbolo di direzione dell'arco 
	if(GetPDirezArco()!=NULL)
	{
		GetPDirezArco()->Draw();
	}
	
	//Disegno dell'eventuale simbolo di posizione dell'arco 
	if(GetPPosizArco()!=NULL)
	{
		GetPPosizArco()->Draw();
	}
	//Disegno dell'eventuale simbolo di parte dell'arco 
	if(GetPParteArco()!=NULL)
	{
		GetPParteArco()->Draw();
	}
	
	// Draw della Corda.   
	Violino::Draw();  
	
	Scr->SetColor(oldcol);
}



void Arco::Free()
{
	if(GetPDirezArco()!=NULL)
		delete GetPDirezArco();
	if(GetPPosizArco()!=NULL)
		delete PPosizArco;
	if(GetPParteArco()!=NULL)
		delete GetPParteArco(); 
	Init();
}


CmdResult Arco::SymCommand(SymCmd& SCmd, TipoInserimento tpIns, Nota* pNota,
						   Battuta* pBattuta)
{
	
	CmdResult cmdRes;
	
	switch(SCmd.CommandId)
	{
    case CMD_NOTA_CORDA_0:
    case CMD_NOTA_CORDA_1:
    case CMD_NOTA_CORDA_2:
    case CMD_NOTA_CORDA_3:
    case CMD_NOTA_CORDA_4:
    case CMD_NOTA_CORDA_5:
	case CMD_NOTA_CORDA_6:
	case CMD_NOTA_CORDA_7:
		cmdRes=Violino::SymCommand(SCmd, tpIns, pNota, pBattuta);
		if(cmdRes!=CMD_UNKNOWN)
			return(cmdRes);
		break;
    case CMD_ARCO_PONTICELLO:
		return(CmdArcoPonticello(tpIns, pNota, pBattuta));
		break;
    case CMD_ARCO_TASTIERA:
		return(CmdArcoTastiera(tpIns, pNota, pBattuta));
		break;
    case CMD_ARCO_PUNTA:  
		return(CmdArcoPunta(tpIns, pNota, pBattuta));
		break;
    case CMD_ARCO_TALLONE:
		return(CmdArcoTallone(tpIns, pNota, pBattuta));
		break;  
    case CMD_ARCO_ARCOSU:
		return(CmdArcoArcoSu(tpIns, pNota, pBattuta));
		break;                 
    case CMD_ARCO_ARCOGIU:
		return(CmdArcoArcoGiu(tpIns, pNota, pBattuta));
		break;   
    case CMD_NOTA_ARCO:
		return(CmdArco(tpIns, pNota, pBattuta));
		break;   
		
	}            
	return(CMD_UNKNOWN);
}


// METODI PRIVATI



Bool Arco::GetScrivereArco() 
{ 
	return ScrivereArco; 
};

void Arco::SetScrivereArco(Bool b) 
{ 
	ScrivereArco = b; 
	if (ScrivereArco)
	{
		SetVU2Up(ARCO2UP);
		SetVU2Dwn(ARCO2DWN);
		SetVU2Lft(ARCO2LFT);
		SetVU2Rgt(ARCO2RGT);
	}
	else
	{
		SetVU2Up(0);
		SetVU2Dwn(0);
		SetVU2Lft(0);
		SetVU2Rgt(0);
	}
};


CmdResult Arco::CmdArco(TipoInserimento tpIns, Nota* pNota,
						Battuta* pBattuta)
{      
	switch (tpIns)
	{
	case SOPRA:
		SetAboveNota(TRUE);
		break;
	case SOTTO:
		SetAboveNota(FALSE);
		break;
	case AUTOMATICO:
		SetAboveNotaAuto(CL_ARCO, pNota, pBattuta);
		break;
	}                         
	SetScrivereArco(TRUE);
	
	return(CMD_OK);
}


CmdResult Arco::CmdArcoArcoSu(TipoInserimento tpIns, Nota* pNota,
							  Battuta* pBattuta)
{                            
	if(GetPDirezArco()!=NULL)
	{
		if ((GetPDirezArco()->GetID() == CL_ARCOSU) && (GetPDirezArco()->GetAboveNota()) == aboveNota)
			return(CMD_FAIL);
		delete GetPDirezArco();
	}
	SetPDirezArco(new ArcoSu);
	if(GetPDirezArco()==NULL)
	{ 
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Arco","CmdArcoArcoSu()");
		return(CMD_FAIL);
	};
	switch (tpIns)
	{
	case SOPRA:
		GetPDirezArco()->SetAboveNota(TRUE);
		break;
	case SOTTO:
		GetPDirezArco()->SetAboveNota(FALSE);
		break;
	case AUTOMATICO:
		GetPDirezArco()->SetAboveNotaAuto(CL_ARCOSU, pNota, pBattuta);
		break;
	}  
	return(CMD_OK);
}             


CmdResult Arco::CmdArcoArcoGiu(TipoInserimento tpIns, Nota* pNota,
							   Battuta* pBattuta)
{                            
	if(GetPDirezArco()!=NULL)
	{
		if ((GetPDirezArco()->GetID() == CL_ARCOGIU) && (GetPDirezArco()->GetAboveNota()) == aboveNota)
			return(CMD_FAIL);
		delete GetPDirezArco();
	}
	SetPDirezArco(new ArcoGiu);
	if(GetPDirezArco()==NULL)
	{ 
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Arco","CmdArcoArcoGiu()");
		return(CMD_FAIL);
	};
	switch (tpIns)
	{
	case SOPRA:
		GetPDirezArco()->SetAboveNota(TRUE);
		break;
	case SOTTO:
		GetPDirezArco()->SetAboveNota(FALSE);
		break;
	case AUTOMATICO:
		GetPDirezArco()->SetAboveNotaAuto(CL_ARCOGIU, pNota, pBattuta);
		break;
	}  
	return(CMD_OK);
}             


CmdResult Arco::CmdArcoPonticello(TipoInserimento tpIns, Nota* pNota,
								  Battuta* pBattuta)
{                            
	if(GetPPosizArco()!=NULL)
	{
		if ((GetPPosizArco()->GetID() == CL_PONTICELLO) && (GetPPosizArco()->GetAboveNota()) == aboveNota)
			return(CMD_FAIL);
		delete GetPPosizArco();
	}
	SetPPosizArco(new Ponticello);
	if(GetPPosizArco()==NULL)
	{ 
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Arco","CmdArcoTastiera()");
		return(CMD_FAIL);
	};
	switch (tpIns)
	{
	case SOPRA:
		GetPPosizArco()->SetAboveNota(TRUE);
		break;
	case SOTTO:
		GetPPosizArco()->SetAboveNota(FALSE);
		break;
	case AUTOMATICO:
		GetPPosizArco()->SetAboveNotaAuto(CL_PONTICELLO, pNota, pBattuta);
		break;
	}  
	return(CMD_OK);
} 


CmdResult Arco::CmdArcoTastiera(TipoInserimento tpIns, Nota* pNota,
								Battuta* pBattuta)
{                            
	if(GetPPosizArco()!=NULL)
	{
		if ((GetPPosizArco()->GetID() == CL_TASTIERA) && (GetPPosizArco()->GetAboveNota()) == aboveNota)
			return(CMD_FAIL);
		delete GetPPosizArco();
	}
	SetPPosizArco(new Tastiera);
	if(GetPPosizArco()==NULL)
	{ 
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Arco","CmdArcoTastiera()");
		return(CMD_FAIL);
	};
	switch (tpIns)
	{
	case SOPRA:
		GetPPosizArco()->SetAboveNota(TRUE);
		break;
	case SOTTO:
		GetPPosizArco()->SetAboveNota(FALSE);
		break;
	case AUTOMATICO:
		GetPPosizArco()->SetAboveNotaAuto(CL_TASTIERA, pNota, pBattuta);
		break;
	}  
	return(CMD_OK);
}


CmdResult Arco::CmdArcoPunta(TipoInserimento tpIns, Nota* pNota,
							 Battuta* pBattuta)
{                            
	if(GetPParteArco()!=NULL)
	{
		if ((GetPParteArco()->GetID() == CL_PUNTA) && (GetPParteArco()->GetAboveNota()) == aboveNota)
			return(CMD_FAIL);
		delete GetPParteArco();
	}
	SetPParteArco(new Punta);
	if(GetPParteArco()==NULL)
	{ 
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Arco","CmdArcoPunta()");
		return(CMD_FAIL);
	};  
	switch (tpIns)
	{
	case SOPRA:
		GetPParteArco()->SetAboveNota(TRUE);
		break;
	case SOTTO:
		GetPParteArco()->SetAboveNota(FALSE);
		break;
	case AUTOMATICO:
		GetPParteArco()->SetAboveNotaAuto(CL_PUNTA, pNota, pBattuta);
		break;
	}  
	return(CMD_OK);
} 


CmdResult Arco::CmdArcoTallone(TipoInserimento tpIns, Nota* pNota,
							   Battuta* pBattuta)
{                            
	if(GetPParteArco()!=NULL)
	{
		if ((GetPParteArco()->GetID() == CL_TALLONE) && (GetPParteArco()->GetAboveNota()) == aboveNota)
			return(CMD_FAIL);
		delete GetPParteArco();
	}
	SetPParteArco(new Tallone);
	if(GetPParteArco()==NULL)
	{ 
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Arco","CmdArcoTallone()");
		return(CMD_FAIL);
	};  
	switch (tpIns)
	{
	case SOPRA:
		GetPParteArco()->SetAboveNota(TRUE);
		break;
	case SOTTO:
		GetPParteArco()->SetAboveNota(FALSE);
		break;
	case AUTOMATICO:
		GetPParteArco()->SetAboveNotaAuto(CL_TALLONE, pNota, pBattuta);
		break;
	}  
	return(CMD_OK);
}        

Bool Arco::Hit(const Point& p, SelObj selObj, DrawObject*& drwObjSel,
			   SymPath& symPath)
{
	if (Violino::Hit(p, selObj, drwObjSel, symPath))
		return TRUE;
	
	if(selObj!=ANY_SEL)
		return(FALSE);
	
	if(GetPDirezArco()!=NULL &&
		GetPDirezArco()->Hit(p,selObj,drwObjSel,symPath))
		return(TRUE);
	if(GetPParteArco()!=NULL &&
		GetPParteArco()->Hit(p,selObj,drwObjSel,symPath))
		return(TRUE);
	if(GetPPosizArco()!=NULL &&
		GetPPosizArco()->Hit(p,selObj,drwObjSel,symPath))
		return(TRUE);
	return(FALSE);
}

Bool Arco::SymbolicHit(SymPath& symPath, SelObj selObj, DrawObject*& drwObjSel)
{
	
	if(Violino::SymbolicHit(symPath, selObj, drwObjSel))
		return TRUE;
	
	if(drwObjSel==NULL && selObj==ANY_SEL)
	{
		
		if(GetPDirezArco()!=NULL &&
			GetPDirezArco()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);
		
		if(GetPParteArco()!=NULL &&
			GetPParteArco()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);   
		
		if(GetPPosizArco()!=NULL &&
			GetPPosizArco()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE); 
		
	}
	return(FALSE);
}

Bool Arco::DeleteSym(DrawObject *sym)
{
	
	if(sym==GetPPosizArco())
	{
		delete GetPPosizArco();
		SetPPosizArco(NULL);
		return TRUE;
	}
	if(sym==GetPParteArco())
	{
		delete GetPParteArco();
		SetPParteArco(NULL);
		return TRUE;
	}
	if(sym==GetPDirezArco())
	{
		delete GetPDirezArco();
		SetPDirezArco(NULL);
		return TRUE;
	}
	
	
	return Violino::DeleteSym(sym);
}


DrawObject * Arco::GetObject(SymCmd * SCmd)
{
    
	switch(SCmd->CommandId)
	{
    case CMD_ARCO_PONTICELLO:
		if(GetPPosizArco()!=NULL)
		{
			if (GetPPosizArco()->GetID() == CL_PONTICELLO)
				return(DrawObject *)GetPPosizArco();
			
		}
		return NULL;
		break;
    case CMD_ARCO_TASTIERA:
		if(GetPPosizArco()!=NULL)
		{
			if (GetPPosizArco()->GetID() == CL_TASTIERA)
				return (DrawObject *)GetPPosizArco();
			
		}
		return NULL;
		break;
    case CMD_ARCO_PUNTA:  
		if(GetPParteArco()!=NULL)
		{
			if (GetPParteArco()->GetID() == CL_PUNTA)
				return (DrawObject *)GetPParteArco();
			
		}
		return NULL;
		break;
    case CMD_ARCO_TALLONE:
		if(GetPParteArco()!=NULL)
		{
			if (GetPParteArco()->GetID() == CL_TALLONE)
				return (DrawObject *)GetPParteArco();
			
		}
		return NULL;
		break;  
    case CMD_ARCO_ARCOSU:
		if(GetPDirezArco()!=NULL)
		{
			if (GetPDirezArco()->GetID() == CL_ARCOSU)
				return (DrawObject *)GetPDirezArco();
			
		}
		return NULL;
		break;                 
    case CMD_ARCO_ARCOGIU:
		if(GetPDirezArco()!=NULL)
		{
			if (GetPDirezArco()->GetID() == CL_ARCOGIU)
				return (DrawObject *)GetPDirezArco();
			
		}
		return NULL;
		break;   
    case CMD_NOTA_ARCO:
		return (DrawObject *)(this);
		break; 
	default:  return Violino::GetObject(SCmd);
		
	}            
	return NULL;
}


