/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: ditacord.cpp,v 1.1 2005/07/25 09:35:34 bellinpo Exp $ */


#include "str/ditacord.hpp"
#include "drw/gscreen.hpp"

DitaCorde::DitaCorde()
{ 
	ID=CL_DITACORDE;
	SetVU2Up(0);
	SetVU2Dwn(0);
	SetVU2Rgt(0);
	SetVU2Lft(0);
	nstrings=0;
	name=new char[1];
	strcpy(name,"");		
	frets=new char[1];
	strcpy(frets,"");	
	strcpy(barre,"");	
	head=0;
	fingers=new char[1];
	strcpy(fingers,"");
	SetAboveNota(TRUE);
}

DitaCorde::DitaCorde(const char *INname,int INnstrings,const char *INfrets,const char *INbarre,int INhead,const char* INfingers)
{ 
	ID=CL_DITACORDE;
	nstrings=INnstrings;
	name=new char[strlen(INname)+1];
	strcpy(name,INname);		
	frets=new char[nstrings+1];
	strcpy(frets,INfrets);
	frets[nstrings]='\0';
	strcpy(barre,INbarre);
	head=INhead;			
	fingers=new char[nstrings+1];
	strcpy(fingers,INfingers);
	fingers[nstrings]='\0';	
	SetAboveNota(TRUE);
	SetVU2Up((NTASTI-TASTOCENTRO)*DELTATASTO+UPNOME+DELTACAPO+DELTASIMB+DELTANOME);
	SetVU2Dwn(TASTOCENTRO*DELTATASTO+DWNUM+DELTANUM+DWSPACE);
	SetVU2Rgt((nstrings-1-CORDACENTRO)*DELTACORDA/* +RGTASTO+DELTARGTASTO*/ );
	SetVU2Lft(CORDACENTRO*DELTACORDA);
}


DitaCorde::DitaCorde(const char *INfulldesc)
{ 
	ID=CL_DITACORDE;
	name=new char[50];
	frets=new char[10];
	fingers=new char[10];
	
	sscanf(INfulldesc,"%s %s %s %d %s", name, frets, barre, &head, fingers);
	
	nstrings=strlen(frets);
	
	SetAboveNota(TRUE);
	SetVU2Up((NTASTI-TASTOCENTRO)*DELTATASTO+UPNOME+DELTACAPO+DELTASIMB+DELTANOME);
	SetVU2Dwn(TASTOCENTRO*DELTATASTO+DWNUM+DELTANUM+DWSPACE);
	SetVU2Rgt((nstrings-1-CORDACENTRO)*DELTACORDA/*+RGTASTO+DELTARGTASTO*/ );
	SetVU2Lft(CORDACENTRO*DELTACORDA);
}

DitaCorde::DitaCorde(const DitaCorde &INChord)
{ 
	ID=CL_DITACORDE;
	nstrings=INChord.nstrings;
	name=new char[strlen(INChord.name)+1];
	strcpy(name,INChord.name);		
	frets=new char[nstrings+1];
	strcpy(frets,INChord.frets);
	frets[nstrings]='\0';
	strcpy(barre,INChord.barre);
	head=INChord.head;			
	fingers=new char[nstrings+1];
	strcpy(fingers,INChord.fingers);
	fingers[nstrings]='\0';	
	SetAboveNota(TRUE);
	
	SetVU2Up((NTASTI-TASTOCENTRO)*DELTATASTO+UPNOME+DELTACAPO+DELTASIMB+DELTANOME);
	SetVU2Dwn(TASTOCENTRO*DELTATASTO+DWNUM+DELTANUM+DWSPACE);
	SetVU2Rgt((nstrings-1-CORDACENTRO)*DELTACORDA/* + RGTASTO +DELTARGTASTO */);
	SetVU2Lft(CORDACENTRO*DELTACORDA);
}

DitaCorde::~DitaCorde()
{ 
	delete name;
	delete frets;
	delete fingers;
}

char *DitaCorde::GetNote()
{
	char *chnote;
	
	chnote=new char[3];  
	strcpy(chnote,name);
	chnote[2]='\0';
	if(chnote[1]!='b' && chnote[1]!='#')
		chnote[1]='\0';
	return chnote;
}

int DitaCorde::GetNStrings()
{
	return nstrings;
}

char *DitaCorde::GetName()
{
	return name;
}

void DitaCorde::GetFullDescription(char *dsc,int len)
{
	sprintf(dsc,"%s %s %s %d %s", name, frets, barre, head, fingers);
}

void DitaCorde::Draw()
{     
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	int i,corda,tasto,cordaini,cordafin;
	Rectangle *AppDito;
	Slur *AppBarre;
	char appsimb[2]={0,0},apphead[6];
	Point abspos=GetAbsPos()+Point(GetDx(),GetDy());

	Scr->SetFont(nTextFont);
	for(i=0;i<nstrings;i++)
	{
		Scr->Line(abspos+Point((-CORDACENTRO+i)*DELTACORDA,(NTASTI-TASTOCENTRO)*DELTATASTO),
			abspos+Point((-CORDACENTRO+i)*DELTACORDA,-TASTOCENTRO*DELTATASTO));
	}
	for(i=1;i<NTASTI+1;i++)
	{  
		Scr->Line(abspos+Point(-CORDACENTRO*DELTACORDA,(NTASTI-TASTOCENTRO-i)*DELTATASTO),
			abspos+Point((nstrings-1-CORDACENTRO)*DELTACORDA,(NTASTI-TASTOCENTRO-i)*DELTATASTO));
	}
	for(i=0;i<nstrings;i++)
	{
		corda=nstrings-1-i;
		if(frets[i]=='x' || frets[i]=='o')
		{    
			appsimb[0]=frets[i];
			Scr->TextTo(abspos+Point((nstrings-1-CORDACENTRO-corda)*DELTACORDA-Scr->GetTextWidth((unsigned char *)appsimb)/2,
				(NTASTI-TASTOCENTRO)*DELTATASTO+DELTACAPO+UPSIMB));
			Scr->OutText((unsigned char *)appsimb);
		}      
		else
		{
			if(fingers[i]!='9')
				appsimb[0]=fingers[i];
			else
				appsimb[0]='T';          
			Scr->TextTo(abspos+Point((nstrings-1-CORDACENTRO-corda)*DELTACORDA-Scr->GetTextWidth((unsigned char *)appsimb)/2-1,
				-TASTOCENTRO*DELTATASTO-DWNUM-DELTANUM));
			Scr->OutText((unsigned char *)appsimb);
			tasto=frets[i]-'1';
			AppDito=new Rectangle(abspos+Point((nstrings-1-CORDACENTRO-corda)*DELTACORDA,(NTASTI-TASTOCENTRO-0.5-tasto)*DELTATASTO),
				DELTADITOX/2,DELTADITOX/2,DELTADITOY/2,DELTADITOY/2);
			Scr->Ellipse(*AppDito,TRUE);
			delete AppDito;
		}    
	}
	if(!head)
		Scr->BigLine(abspos+Point(-CORDACENTRO*DELTACORDA,(NTASTI-TASTOCENTRO)*DELTATASTO),
		abspos+Point((nstrings-1-CORDACENTRO)*DELTACORDA,(NTASTI-TASTOCENTRO)*DELTATASTO),DELTACAPO);
	else
	{
		Scr->Line(abspos+Point(-CORDACENTRO*DELTACORDA,(NTASTI-TASTOCENTRO)*DELTATASTO),
			abspos+Point((nstrings-1-CORDACENTRO)*DELTACORDA,(NTASTI-TASTOCENTRO)*DELTATASTO));
		sprintf(apphead,"%d fr",head);
		Scr->TextTo(abspos+Point((nstrings-1-CORDACENTRO)*DELTACORDA+RGTASTO,
			(NTASTI-TASTOCENTRO-1)*DELTATASTO));    
		Scr->OutText((unsigned char *)apphead);
	}
	if(strcmp(barre,"00"))
	{
		cordaini='0'+nstrings-barre[0];
		cordafin='0'+nstrings-barre[1];
		tasto=frets[barre[0]-'1']-'1';    
		AppBarre=new Slur(abspos+Point((nstrings-1-CORDACENTRO-cordaini)*DELTACORDA,(NTASTI-TASTOCENTRO-tasto)*DELTATASTO+(tasto?0:DELTACAPO)+1),
			abspos+Point((nstrings-1-CORDACENTRO-cordafin)*DELTACORDA,(NTASTI-TASTOCENTRO-tasto)*DELTATASTO+(tasto?0:DELTACAPO)+2));
		AppBarre->SetBowUp(TRUE);
		AppBarre->Draw();
		delete AppBarre;
	}
	Scr->TextTo(abspos+Point(-Scr->GetTextWidth((unsigned char*)name)/2,(NTASTI-TASTOCENTRO)*DELTATASTO+UPNOME+DELTACAPO));   
	Scr->OutText((unsigned char *)name);
	Scr->SetFont(nMusicFont);
	

	Scr->SetColor(oldcol);
	
};

