/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: ditafile.cpp,v 1.1 2005/07/25 09:35:35 bellinpo Exp $ */


#include "str/ditafile.hpp"
#include "str/ditacord.hpp"

DitaFile::DitaFile(const char *FilePath)
{
    Open(FilePath);
}

DitaFile::~DitaFile()
{
    Close();
}

int DitaFile::Open(const char *FilePath)
{
    if((fp=fopen(FilePath,"rb+"))==NULL)
        return(-1);
    return(0);
}

int DitaFile::IsOpen(void)
{
    if(fp==NULL)
        return(0);
    return(1);
}

int DitaFile::IsOnEOF(void)
{
    long pos;
    char line[40];
    if(fp==NULL)
        return(-1);
    if(feof(fp))
        return(1);
    pos=ftell(fp);
    if(fgets(line,40,fp)==NULL || !strcmp(line,""))
        return(1);
    fseek(fp,pos,SEEK_SET);
    return(0);
}

void DitaFile::Close(void)
{
    if(fp!=NULL)
        fclose(fp);
}

DitaCorde *DitaFile::GetChord(int *OUTvar,char *OUTcomment)
{
    char toParse[100];
    DitaCorde *toGet;
    
    char app_barre[6];
    char app_nstrings[4];
    char app_var[4];
    char app_name[30];
    char name[30];
    char frets[MAXCORDE+1];
    char barre[3];
    char fingers[MAXCORDE+1];
    int nstrings,var,head;
    int i=0,j=0;
	
    if(fp==NULL || feof(fp))
        return NULL;
    if(fgets(toParse,100,fp)==NULL)
        return NULL;
    toParse[strlen(toParse)-1]='\0';
    if(!strcmp(toParse,DELETED_STRING))
    {
        toGet=new DitaCorde(DELETED_NAME,1,"","",0,"");
        return(toGet);
    }
    if(toParse==NULL || toParse[0]=='\0')
        return NULL;
    sscanf(toParse,"%s%s%s%s%s%s",app_name,app_nstrings,app_var,frets,app_barre,fingers);
    app_name[strlen(app_name)-1]='\0';
    strcpy(name,SymTrad(app_name));
    app_nstrings[strlen(app_nstrings)-1]='\0';
    sscanf(app_nstrings,"%d",&nstrings);
    app_var[strlen(app_var)-1]='\0';
    sscanf(app_var,"%d",&var);
    frets[strlen(frets)-1]='\0';
    app_barre[strlen(app_barre)-1]='\0';
    if(strcmp(app_barre,"0"))
    {
        sscanf(app_barre+2,"%d",&head);
        app_barre[2]='\0';
        strcpy(barre,app_barre);
    }
    else
    {
        head=0;
        strcpy(barre,"00");
    }
    fingers[strlen(fingers)-1]='\0';
    if(OUTvar!=NULL)
		*OUTvar=var;
    if(OUTcomment!=NULL)
    {
        while(toParse[i]!='"' && toParse[i]!='\0')
			i++;
		if(toParse[i]=='\0')
			OUTcomment[0]='\0';
		else
		{
			i++;
			while(toParse[i]!='"' && toParse[i]!='\0')
				OUTcomment[j++]=toParse[i++];
			OUTcomment[j]='\0';
		}
    }
    toGet=new DitaCorde(name,nstrings,frets,barre,head,fingers);
    return(toGet);
}

int DitaFile::GoToPos(int pos)
{
    DitaCorde *app;
    long prev;
    int i=0;
	
    if(fp==NULL)
        return(-1);
    rewind(fp);
    while(!IsOnEOF() && i<pos+1)
    {
        prev=ftell(fp);
        app=GetChord();
        if(strcmp(app->GetName(),DELETED_NAME))
            i++;
		delete app;
    }
    if(IsOnEOF())
        return(-1);
    fseek(fp,prev,SEEK_SET);
    return(0);
}

int DitaFile::Search(int nstrings,const char* ChName)
{
    return Search(nstrings,ChName,1);
}

int DitaFile::Search(int nstrings,const char* ChName, int var)
{
    DitaCorde *app;
    long filepos;
    int OUTvar;
    char find=0;
	
    if(fp==NULL)
        return(-1);
    GoToPos(0);
    filepos=ftell(fp);
    while(!IsOnEOF() && !find)
    {
        app=GetChord(&OUTvar);
        if(app->GetNStrings()!=nstrings || strcmp(app->GetName(),ChName) || OUTvar!=var)
            filepos=ftell(fp);
        else
            find=1;
        delete app;
    }
    if(!find)
        return(0);
    fseek(fp,filepos,SEEK_SET);
    return(1);
}

int DitaFile::NotesCount(int nstrings)
{
    char appList[12][3];
    int count=0,i;
    DitaCorde *app;
	
    if(nstrings==0)
        return(-1);
    GoToPos(0);
    while(!IsOnEOF())
    {
        app=GetChord();
        if(strcmp(app->GetName(),DELETED_NAME) && app->GetNStrings()==nstrings)
        {
            i=count;
            while(i>0 && strcmp(app->GetNote(),appList[i-1]))
                i--;
            if(i==0)
                strcpy(appList[count++],app->GetNote());
        }
        delete app;
    }
    return count;
}

int DitaFile::NotesList(int nstrings, char **myList)
{
    int count=0,i;
    DitaCorde *app;
	
	
    if(nstrings==0)
        return(-1);
    GoToPos(0);
    while(!IsOnEOF())
    {
        app=GetChord();
        if(strcmp(app->GetName(),DELETED_NAME) && app->GetNStrings()==nstrings)
        {
            i=count;
            while(i>0 && strcmp(app->GetNote(),myList[i-1]))
                i--;
            if(i==0)
                strcpy(myList[count++],app->GetNote());
        }
        delete app;
    }
    return count;
}

int DitaFile::NamesCount(int nstrings, const char *Note)
{
    int count=0;
    DitaCorde *app;
    int OUTvar;
	
    if(nstrings==0 || Note==NULL || Note[0]=='\0')
        return(-1);
    GoToPos(0);
    while(!IsOnEOF())
    {
        app=GetChord(&OUTvar);
        if(!strcmp(app->GetNote(),Note) && app->GetNStrings()==nstrings && OUTvar==1)
            count++;
        delete app;
    }
    return count;
}

int DitaFile::NamesList(int nstrings, const char *Note,char **myList)
{
    int count=0;
    DitaCorde *app;
    int OUTvar;
	
    if(nstrings==0 || Note==NULL || Note[0]=='\0' || myList==NULL)
        return(-1);
    GoToPos(0);
    while(!IsOnEOF())
    {
        app=GetChord(&OUTvar);
        if(!strcmp(app->GetNote(),Note) && app->GetNStrings()==nstrings && OUTvar==1)
		{
            strcpy(myList[count++],app->GetName());	    
        }
		delete app;
    }
    return count;
}

int DitaFile::VarsCount(int nstrings, const char *Name)
{
    int count=0;
    DitaCorde *app;
	
    if(nstrings==0 || Name==NULL || Name[0]=='\0')
        return(-1);
    GoToPos(0);
    while(!IsOnEOF())
    {
        app=GetChord();
        if(!strcmp(app->GetName(),Name) && app->GetNStrings()==nstrings)
            count++;
        delete app;
    }
    return count;
}

int DitaFile::VarsList(int nstrings, const char *Name, char **comments, int *vars, DitaCorde **Chords)
{
    int count=0;
    DitaCorde *app;
    int OUTvar;
    char OUTcomment[25];
	
    if(nstrings==0 || Name==NULL || Name[0]=='\0' || comments==NULL || vars==NULL)
        return(-1);
    GoToPos(0);
    while(!IsOnEOF())
    {
        app=GetChord(&OUTvar,OUTcomment);
        if(!strcmp(app->GetName(),Name) && app->GetNStrings()==nstrings)
        {
            strcpy(comments[count],OUTcomment);
			vars[count]=OUTvar;
			Chords[count++]=app;
        }
    }
    return count;
}

char *DitaFile::SymTrad(char *toTrad)
{
    char *trad;
    unsigned int i=0,j=0;
	
    trad=new char[20];
    while(toTrad[i]!='\0')
    {
        if(toTrad[i]!='?')
            trad[j]=toTrad[i];
        else
        {
            switch(toTrad[++i])
            {
			case '0':
				trad[j]='';
                break;
			case '1':
				trad[j]='b';
                break;
			case '2':
				trad[j]='#';
            }
        }
        i++;
        j++;
    }
    trad[j]='\0';
    return trad;
}
