/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: violino.cpp,v 1.1 2005/07/25 09:35:39 bellinpo Exp $ */



#include "str/violino.hpp"
#include "str/corda.hpp"
#include "gui/lioomenu.hpp"

Violino::Violino()
{
	Init();
}

void Violino::Init()
{   
	SetVU2Up(0);
	SetVU2Dwn(0);
	SetVU2Lft(0);
	SetVU2Rgt(0);
	
	SetPCorda(NULL); 
};

Violino::~Violino()
{
	Free();
}

void Violino::SetPCorda(Corda *pc)
{
	PCorda=pc;
};

Corda* Violino::GetPCorda()
{
	return PCorda;
}

void Violino::SetPos(DrawObject *d,const Point& p)
{
	DrawObject::SetPos(d,p);
	
}



void Violino::Draw()
{
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	if (GetPCorda()!=NULL) 
		GetPCorda()->Draw();

	Scr->SetColor(oldcol);
}

void Violino::Free()
{
	if(GetPCorda()!=NULL)
		delete GetPCorda();
	
	Init();
}


CmdResult Violino::SymCommand(SymCmd& SCmd, TipoInserimento tpIns, 
							  Nota* pNota, Battuta* pBattuta)
{ 
	switch(SCmd.CommandId)
	{
    case CMD_NOTA_CORDA_0:
		return(CmdViolinoCorda(tpIns,0, pNota, pBattuta));
		break;
    case CMD_NOTA_CORDA_1:
		return(CmdViolinoCorda(tpIns,1, pNota, pBattuta)); 
		break;
    case CMD_NOTA_CORDA_2:
		return(CmdViolinoCorda(tpIns,2, pNota, pBattuta));
		break;
    case CMD_NOTA_CORDA_3:
		return(CmdViolinoCorda(tpIns,3, pNota, pBattuta)); 
		break;
    case CMD_NOTA_CORDA_4:
		return(CmdViolinoCorda(tpIns,4, pNota, pBattuta));
		break;
    case CMD_NOTA_CORDA_5:
		return(CmdViolinoCorda(tpIns,5, pNota, pBattuta)); 
		break;
    case CMD_NOTA_CORDA_6:
		return(CmdViolinoCorda(tpIns,6, pNota, pBattuta)); 
		break;
    case CMD_NOTA_CORDA_7:
		return(CmdViolinoCorda(tpIns,7, pNota, pBattuta)); 
		break;
	}            
	return(CMD_UNKNOWN);
}

// METODI PRIVATI




CmdResult Violino::CmdViolinoCorda(TipoInserimento tpIns,
								   short numc, Nota* pNota, Battuta* pBattuta)
{
	if(GetPCorda()!=NULL)
	{
		extern Bool GetStringsRoman();
		extern TipiStrings GetTipoStrings();
		if ((GetPCorda()->GetNumCorda() == numc) && (GetPCorda()->GetIsRoman()==GetStringsRoman()) && GetPCorda()->GetTipoCorda()==GetTipoStrings())
			return(CMD_FAIL);
		delete GetPCorda();
	}
	SetPCorda(new Corda);
	if(GetPCorda()==NULL)
	{
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Violino","CmdViolinoCorda()");
		return CMD_FAIL;
	};
	switch (tpIns)
	{
	case SOPRA:
		GetPCorda()->SetAboveNota(TRUE);
		break;
	case SOTTO:
		GetPCorda()->SetAboveNota(FALSE);
		break;
	case AUTOMATICO:
		GetPCorda()->SetAboveNotaAuto(CL_CORDA, pNota, pBattuta);
		break;
	}  
	GetPCorda()->SetNumCorda(numc);
	return(CMD_OK);
}                 

Bool Violino::Hit(const Point& p, SelObj selObj, DrawObject*& drwObjSel,
				  SymPath& symPath)
{
	Point p1;
	
	if (DrawObject::Hit(p, selObj, drwObjSel, symPath))
		return TRUE;
	
	if(selObj!=ANY_SEL)
		return(FALSE);
	
	if(GetPCorda()!=NULL &&
		GetPCorda()->Hit(p,selObj,drwObjSel,symPath))
		return(TRUE);
	
	return(FALSE);
}


Bool Violino::SymbolicHit(SymPath& symPath, SelObj selObj, DrawObject*& drwObjSel)
{
	
	
	
	if (DrawObject::SymbolicHit(symPath, selObj, drwObjSel))
		return TRUE;
	
	if(drwObjSel==NULL && selObj==ANY_SEL)
	{
		if(GetPCorda()!=NULL &&
			GetPCorda()->SymbolicHit(symPath,selObj,drwObjSel))
			return(TRUE);
	}
	return(FALSE);
}

Bool Violino::DeleteSym(DrawObject *sym)
{
	if(sym==GetPCorda())
	{
		delete GetPCorda();
		SetPCorda(NULL);
		return TRUE;
	}
	return FALSE;
}



DrawObject * Violino::GetObject(SymCmd * SCmd)
{
	
	switch(SCmd->CommandId)
	{
    case CMD_NOTA_CORDA_0:
    case CMD_NOTA_CORDA_1:
    case CMD_NOTA_CORDA_2:
    case CMD_NOTA_CORDA_3:
    case CMD_NOTA_CORDA_4:
    case CMD_NOTA_CORDA_5:
	case CMD_NOTA_CORDA_6:
	case CMD_NOTA_CORDA_7:
		return ((DrawObject *)(GetPCorda())); break;
	default: return NULL; break;
    case CMD_NOTA_PIZZICATO:
		if (GetID()==CL_PIZZICATO) return (DrawObject *)(this);
		else return NULL;
		
	}            
	return NULL;
}


Bool Violino::is(ClassID CLId)
{
	if (CLId==CL_VIOLINO)
		return TRUE;
	return Strumento::is(CLId);
}

