/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: tdinamic.cpp,v 1.1 2005/07/25 09:35:41 bellinpo Exp $ */


#include "txt/tdinamic.hpp"
#include "drw/gscreen.hpp"
#include "gui/dialogs.hpp"

TDinamico::TDinamico()
{ ID=CL_TDINAMICO;}

Bool TDinamico::GetAboveNota()
{ return aboveNota; };

void TDinamico::SetAboveNota(Bool above)
{ aboveNota=above;};

Bool TDinamico::SetTipo(TDinamicType tp)
{
	switch(tp)
	{
	case TD_PPPPPP:
        CodeForDraw = CODE_PPPPPP;
        Tipo=tp;
        break;
	case TD_PPPPP:
        CodeForDraw = CODE_PPPPP;
        Tipo=tp;
        break;
    case TD_PPPP:
		CodeForDraw = CODE_PPPP;
		Tipo=tp;
		break;
    case TD_PPP:
		CodeForDraw = CODE_PPP;
		Tipo=tp;
		break;
    case TD_PP:
		CodeForDraw = CODE_PP;
		Tipo=tp;
		break;
    case TD_P:
		CodeForDraw = CODE_P;
		Tipo=tp;
		break;
    case TD_MP:
		CodeForDraw = CODE_MP;
		Tipo=tp;
		break;
    case TD_MF:
		CodeForDraw = CODE_MF;
		Tipo=tp;
		break;
    case TD_F:
		CodeForDraw = CODE_F;
		Tipo=tp;
		break;
    case TD_FF:
		CodeForDraw = CODE_FF;
		Tipo=tp;
		break;
    case TD_FFF:
		CodeForDraw = CODE_FFF;
		Tipo=tp;
		break;
    case TD_FFFF:
		CodeForDraw = CODE_FFFF;
		Tipo=tp;
		break;
	case TD_FFFFF:
        CodeForDraw = CODE_FFFFF;
        Tipo=tp;
        break;
	case TD_FFFFFF:
        CodeForDraw = CODE_FFFFFF;
        Tipo=tp;
        break;
    case TD_SF:
		CodeForDraw = CODE_SF;
		Tipo=tp;
		break;
    case TD_SP:
		CodeForDraw = CODE_SP;
		Tipo=tp;
		break;
    case TD_SFZ:
		CodeForDraw = CODE_SFZ;
		Tipo=tp;
		break;
    case TD_FZ:
		CodeForDraw = CODE_FZ;
		Tipo=tp;
		break;
    case TD_FP:
		CodeForDraw = CODE_FP;
		Tipo=tp;
		break;
	case TD_RF:
		CodeForDraw = CODE_RF;
		Tipo=tp;
		break;
	case TD_FFP:
		CodeForDraw = CODE_FFP;
		Tipo=tp;
		break;
	case TD_FFPP:
		CodeForDraw = CODE_FFPP;
		Tipo=tp;
		break;
	case TD_FPP:
		CodeForDraw = CODE_FPP;
		Tipo=tp;
		break;
	case TD_RFZ:
		CodeForDraw = CODE_RFZ;
		Tipo=tp;
		break;
	case TD_SFF:
		CodeForDraw = CODE_SFF;
		Tipo=tp;
		break;
	case TD_SFFF:
		CodeForDraw = CODE_SFFF;
		Tipo=tp;
		break;
	case TD_SFFP:
		CodeForDraw = CODE_SFFP;
		Tipo=tp;
		break;
	case TD_SFFZ:
		CodeForDraw = CODE_SFFZ;
		Tipo=tp;
		break;
	case TD_SFP:
		CodeForDraw = CODE_SFP;
		Tipo=tp;
		break;
	case TD_SFPP:
		CodeForDraw = CODE_SFPP;
		Tipo=tp;
		break;
    default:
		return FALSE;
    }
    return TRUE;
}

TDinamicType TDinamico::GetTipo() { return Tipo; }


VUnit TDinamico::GetVU2Up()
{
	switch(Tipo)
	{
	case TD_PPPPPP:
		return PPPPPP2UP;
		break;
	case TD_PPPPP:
		return PPPPP2UP;
		break;
	case TD_PPPP:
		return PPPP2UP;
		break;
	case TD_PPP:
		return PPP2UP;
		break; 
	case TD_PP:
		return PP2UP;
		break;
	case TD_P:
		return P2UP;
		break;
	case TD_MP:
		return MP2UP;
		break;
	case TD_MF:
		return MF2UP;
		break; 
	case TD_F:
		return F2UP;
		break;
	case TD_FF:
		return FF2UP;
		break;
	case TD_FFF:
		return FFF2UP;
		break;
	case TD_FFFF:
		return FFFF2UP;
		break; 
	case TD_FFFFF:
		return FFFFF2UP;
		break; 
	case TD_FFFFFF:
		return FFFFFF2UP;
		break; 
	case TD_SP:
		return SP2UP;
		break;
	case TD_SF:
		return SF2UP;
		break;
	case TD_SFZ:
		return SFZ2UP;
		break;
	case TD_FZ:
		return FZ2UP;
		break; 
	case TD_FP:
		return FP2UP;
		break;
	case TD_RF:
		return RF2UP;
		break;
	case TD_FFP:
		return FFP2UP;
		break;
	case TD_FFPP:
		return FFPP2UP;
		break;
	case TD_FPP:
		return FPP2UP;
		break;
	case TD_RFZ:
		return RFZ2UP;
		break;
	case TD_SFF:
		return SFF2UP;
		break;
	case TD_SFFF:
		return SFFF2UP;
		break;
	case TD_SFFP:
		return SFFP2UP;
		break;
	case TD_SFFZ:
		return SFFZ2UP;
		break;
	case TD_SFP:
		return SFP2UP;
		break;
	case TD_SFPP:
		return SFPP2UP;
		break;
    default:
		return 0;
		
	}
};

VUnit TDinamico::GetVU2Dwn()
{
	switch(Tipo)
	{
	case TD_PPPPPP:
		return PPPPPP2DWN;
		break;
	case TD_PPPPP:
		return PPPPP2DWN;
		break;
	case TD_PPPP:
		return PPPP2DWN;
		break;
	case TD_PPP:
		return PPP2DWN;
		break; 
	case TD_PP:
		return PP2DWN;
		break;
	case TD_P:
		return P2DWN;
		break;
	case TD_MP:
		return MP2DWN;
		break;
	case TD_MF:
		return MF2DWN;
		break; 
	case TD_F:
		return F2DWN;
		break;
	case TD_FF:
		return FF2DWN;
		break;
	case TD_FFF:
		return FFF2DWN;
		break;
	case TD_FFFF:
		return FFFF2DWN;
		break; 
	case TD_FFFFF:
		return FFFFF2DWN;
		break; 
	case TD_FFFFFF:
		return FFFFFF2DWN;
		break; 
	case TD_SP:
		return SP2DWN;
		break;
	case TD_SF:
		return SF2DWN;
		break;
	case TD_SFZ:
		return SFZ2DWN;
		break;
	case TD_FZ:
		return FZ2DWN;
		break; 
	case TD_FP:
		return FP2DWN;
		break;
	case TD_RF:
		return RF2DWN;
		break;
	case TD_FFP:
		return FFP2DWN;
		break;
	case TD_FFPP:
		return FFPP2DWN;
		break;
	case TD_FPP:
		return FPP2DWN;
		break;
	case TD_RFZ:
		return RFZ2DWN;
		break;
	case TD_SFF:
		return SFF2DWN;
		break;
	case TD_SFFF:
		return SFFF2DWN;
		break;
	case TD_SFFP:
		return SFFP2DWN;
		break;
	case TD_SFFZ:
		return SFFZ2DWN;
		break;
	case TD_SFP:
		return SFP2DWN;
		break;
	case TD_SFPP:
		return SFPP2DWN;
		break;
	default:
		return 0;
		
	}
}

VUnit TDinamico::GetVU2Lft()
{ 
	switch(Tipo)
	{
	case TD_PPPPPP:
		return PPPPPP2LFT;
		break;
	case TD_PPPPP:
		return PPPPP2LFT;
		break;
	case TD_PPPP:
		return PPPP2LFT;
		break;
	case TD_PPP:
		return PPP2LFT;
		break; 
	case TD_PP:
		return PP2LFT;
		break;
	case TD_P:
		return P2LFT;
		break;
	case TD_MP:
		return MP2LFT;
		break;
	case TD_MF:
		return MF2LFT;
		break; 
	case TD_F:
		return F2LFT;
		break;
	case TD_FF:
		return FF2LFT;
		break;
	case TD_FFF:
		return FFF2LFT;
		break;
	case TD_FFFF:
		return FFFF2LFT;
		break; 
	case TD_FFFFF:
		return FFFFF2LFT;
		break; 
	case TD_FFFFFF:
		return FFFFFF2LFT;
		break; 
	case TD_SP:
		return SP2LFT;
		break;
	case TD_SF:
		return SF2LFT;
		break;
	case TD_SFZ:
		return SFZ2LFT;
		break;
	case TD_FZ:
		return FZ2LFT;
		break; 
	case TD_FP:
		return FP2LFT;
		break;
	case TD_RF:
		return RF2LFT;
		break;
	case TD_FFP:
		return FFP2LFT;
		break;
	case TD_FFPP:
		return FFPP2LFT;
		break;
	case TD_FPP:
		return FPP2LFT;
		break;
	case TD_RFZ:
		return RFZ2LFT;
		break;
	case TD_SFF:
		return SFF2LFT;
		break;
	case TD_SFFF:
		return SFFF2LFT;
		break;
	case TD_SFFP:
		return SFFP2LFT;
		break;
	case TD_SFFZ:
		return SFFZ2LFT;
		break;
	case TD_SFP:
		return SFP2LFT;
		break;
	case TD_SFPP:
		return SFPP2LFT;
		break;
	default:
		return 0;
	}
}

VUnit TDinamico::GetVU2Rgt()
{
	switch(Tipo)
	{
	case TD_PPPPPP:
		return PPPPPP2RGT;
		break;
	case TD_PPPPP:
		return PPPPP2RGT;
		break;
	case TD_PPPP:
		return PPPP2RGT;
		break;
	case TD_PPP:
		return PPP2RGT;
		break; 
	case TD_PP:
		return PP2RGT;
		break;
	case TD_P:
		return P2RGT;
		break;
	case TD_MP:
		return MP2RGT;
		break;
	case TD_MF:
		return MF2RGT;
		break; 
	case TD_F:
		return F2RGT;
		break;
	case TD_FF:
		return FF2RGT;
		break;
	case TD_FFF:
		return FFF2RGT;
		break;
	case TD_FFFF:
		return FFFF2RGT;
		break; 
	case TD_FFFFF:
		return FFFFF2RGT;
		break; 
	case TD_FFFFFF:
		return FFFFFF2RGT;
		break; 
	case TD_SP:
		return SP2RGT;
		break;
	case TD_SF:
		return SF2RGT;
		break;
	case TD_SFZ:
		return SFZ2RGT;
		break;
	case TD_FZ:
		return FZ2RGT;
		break; 
	case TD_FP:
		return FP2RGT;
		break;
	case TD_RF:
		return RF2RGT;
		break;
	case TD_FFP:
		return FFP2RGT;
		break;
	case TD_FFPP:
		return FFPP2RGT;
		break;
	case TD_FPP:
		return FPP2RGT;
		break;
	case TD_RFZ:
		return RFZ2RGT;
		break;
	case TD_SFF:
		return SFF2RGT;
		break;
	case TD_SFFF:
		return SFFF2RGT;
		break;
	case TD_SFFP:
		return SFFP2RGT;
		break;
	case TD_SFFZ:
		return SFFZ2RGT;
		break;
	case TD_SFP:
		return SFP2RGT;
		break;
	case TD_SFPP:
		return SFPP2RGT;
		break;
	default:
		return 0;
	}
}

void TDinamico::Draw()
{
	char code[2]={CodeForDraw,0};
	FontID ofont=Scr->SetFont(nMusicFont);
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());
	
	switch(Tipo)  
	{
	case TD_PPPPPP:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXPPPPPP,DELTAYPPPPPP)+Point(GetDx(),GetDy()));
		break;
	case TD_PPPPP:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXPPPPP,DELTAYPPPPP)+Point(GetDx(),GetDy()));
		break;
	case TD_PPPP:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXPPPP,DELTAYPPPP)+Point(GetDx(),GetDy()));
		break;
	case TD_PPP:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXPPP,DELTAYPPP)+Point(GetDx(),GetDy()));
		break;
	case TD_PP:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXPP,DELTAYPP)+Point(GetDx(),GetDy()));
		break;
	case TD_P:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXP,DELTAYP)+Point(GetDx(),GetDy()));
		break;
	case TD_MP:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXMP,DELTAYMP)+Point(GetDx(),GetDy()));
		break;
	case TD_MF:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXMF,DELTAYMF)+Point(GetDx(),GetDy()));
		break;
	case TD_F:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXF,DELTAYF)+Point(GetDx(),GetDy()));
		break;
	case TD_FF:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXFF,DELTAYFF)+Point(GetDx(),GetDy()));
		break;
	case TD_FFF:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXFFF,DELTAYFFF)+Point(GetDx(),GetDy()));
		break;
	case TD_FFFF:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXFFFF,DELTAYFFFF)+Point(GetDx(),GetDy()));
		break;
	case TD_FFFFF:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXFFFFF,DELTAYFFFFF)+Point(GetDx(),GetDy()));
		break;
	case TD_FFFFFF:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXFFFFFF,DELTAYFFFFFF)+Point(GetDx(),GetDy()));
		break;
	case TD_SF:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXSF,DELTAYSF)+Point(GetDx(),GetDy()));
		break;
	case TD_SP:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXSP,DELTAYSP)+Point(GetDx(),GetDy()));
		break;
	case TD_SFZ:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXSFZ,DELTAYSFZ)+Point(GetDx(),GetDy()));
		break;
	case TD_FZ:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXFZ,DELTAYFZ)+Point(GetDx(),GetDy()));
		break;
	case TD_FP:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXFP,DELTAYFP)+Point(GetDx(),GetDy()));
		break;
	case TD_RF:                          
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXRF,DELTAYRF)+Point(GetDx(),GetDy()));
		break;
	case TD_FFP:                        
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXFFP,DELTAYFFP)+Point(GetDx(),GetDy()));
		break;
	case TD_FFPP:                       
		ofont  = Scr->SetFont(nMusicFont);
		Scr->TextTo(GetAbsPos()+Point(DELTAXFFPP,DELTAYFFPP)+Point(GetDx(),GetDy()));
		break;
	case TD_FPP:                        
		ofont  = Scr->SetFont(nMusicFont2);
		Scr->TextTo(GetAbsPos()+Point(DELTAXFPP,DELTAYFPP)+Point(GetDx(),GetDy()));
		break;
	case TD_RFZ:                        
		ofont  = Scr->SetFont(nMusicFont2);
		Scr->TextTo(GetAbsPos()+Point(DELTAXRFZ,DELTAYRFZ)+Point(GetDx(),GetDy()));
		break;
	case TD_SFF:                        
		ofont  = Scr->SetFont(nMusicFont2);
		Scr->TextTo(GetAbsPos()+Point(DELTAXSFF,DELTAYSFF)+Point(GetDx(),GetDy()));
		break;
	case TD_SFFF:                      
		ofont  = Scr->SetFont(nMusicFont2);
		Scr->TextTo(GetAbsPos()+Point(DELTAXSFFF,DELTAYSFFF)+Point(GetDx(),GetDy()));
		break;
	case TD_SFFP:                      
		ofont  = Scr->SetFont(nMusicFont2);
		Scr->TextTo(GetAbsPos()+Point(DELTAXSFFP,DELTAYSFFP)+Point(GetDx(),GetDy()));
		break;
	case TD_SFFZ:                       
		ofont  = Scr->SetFont(nMusicFont2);
		Scr->TextTo(GetAbsPos()+Point(DELTAXSFFZ,DELTAYSFFZ)+Point(GetDx(),GetDy()));
		break;
	case TD_SFP:                        
		ofont  = Scr->SetFont(nMusicFont2);
		Scr->TextTo(GetAbsPos()+Point(DELTAXSFP,DELTAYSFP)+Point(GetDx(),GetDy()));
		break;
	case TD_SFPP:                       
		ofont  = Scr->SetFont(nMusicFont2);
		Scr->TextTo(GetAbsPos()+Point(DELTAXSFPP,DELTAYSFPP)+Point(GetDx(),GetDy()));
		break;
    default:
		return ;
     }
	 
	 Scr->OutText((unsigned char *)code);
	 Scr->SetFont(ofont);

	 Scr->SetColor(oldcol);

};

void TDinamico::Edit(Battuta *pBat)
{
    EditingMarkersDlg(this);
}  
