/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: arpeggio.cpp,v 1.1 2005/07/25 09:35:42 bellinpo Exp $ */


#include "var/arpeggio.hpp"
#include "drw/gscreen.hpp"

Arpeggio::Arpeggio()
{
	ID=CL_ARPEGGIO;
	NArpeg=0;
	SetDirezioneArpeggio(NO_DIRECTION);
}

Arpeggio::Arpeggio(TipiDirezioneArpeggio dir)
{
	ID=CL_ARPEGGIO;
	NArpeg=0;
	SetDirezioneArpeggio(dir);
}

TipiDirezioneArpeggio Arpeggio::GetDirezioneArpeggio()
{
	return DirezioneArpeggio;
}

void Arpeggio::SetDirezioneArpeggio(TipiDirezioneArpeggio dir)
{
	DirezioneArpeggio = dir;
}

void Arpeggio::SetNumArpeg(VUnit h)
{
	NArpeg=h/ALTEZZAARP;
}


VUnit Arpeggio::GetVU2Up()
{
	return (NArpeg)*ALTEZZAARP + 4;
}

VUnit Arpeggio::GetVU2Dwn()
{
	return -4;
}

VUnit Arpeggio::GetVU2Rgt()
{
	return ARPEGGIO2RGT;
}

VUnit Arpeggio::GetVU2Lft()
{
	return ARPEGGIO2LFT;
}


void Arpeggio::Draw()
{ 
	char code[2] = {CODE_ARPEGGIO, '\0'};
	int i;
	
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());


	
	switch (GetDirezioneArpeggio())
	{
	case NO_DIRECTION:
		{
			for(i=0;i<NArpeg;i++)
			{
				Scr->TextTo(GetAbsPos() + Point(0,ALTEZZAARP*(i-1)) + 
					Point(DELTAXARPEGGIO,DELTAYARPEGGIO));
				code[0] = CODE_ARPEGGIO;
				Scr->OutText((unsigned char *)code);
			}
		};
		break;
	case UPWARD:
		{
			for(i=0;i<NArpeg;i++)
			{
				Scr->TextTo(GetAbsPos() + Point(0,ALTEZZAARP*(i-1)) + 
					Point(DELTAXARPEGGIO,DELTAYARPEGGIO));
				if (i==NArpeg-1) code[0] = CODE_ARPEGGIO_UP;
				else code[0] = CODE_ARPEGGIO;
				Scr->OutText((unsigned char *)code);
			}
		};
		break;
	case DOWNWARD:
		{
			for(i=0;i<NArpeg;i++)
			{
				Scr->TextTo(GetAbsPos() + Point(0,ALTEZZAARP*(i-1)) + 
					Point(DELTAXARPEGGIO,DELTAYARPEGGIO));
				if (i==0) code[0] = CODE_ARPEGGIO_DOWN;
				else code[0] = CODE_ARPEGGIO;
				Scr->OutText((unsigned char *)code);
			}
		};
		break;
	}
	
	Scr->SetColor(oldcol);
	
}  


