/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: corona.cpp,v 1.1 2005/07/25 09:35:42 bellinpo Exp $ */


#include "var/corona.hpp"
#include "drw/gscreen.hpp"

Corona::Corona()
{ ID=CL_CORONA; abovenota=TRUE; tipo=Classic;}

VUnit Corona::GetVU2Up()
{ 
	if(abovenota)
	{
		switch(tipo)
		{
		case (Classic):
			return UPCORONA2UP;
			break;
		case (Quad):
			return UPCORONAQUAD2UP;
			break;
		case (Triang):
			return UPCORONATRIANG2UP;
			break;
		}
	}
	
	else
	{
		switch(tipo)
		{
		case (Classic):
			return DWNCORONA2UP;
			break;
		case (Quad):
			return DWNCORONAQUAD2UP;
			break;
		case (Triang):
			return DWNCORONATRIANG2UP;
			break;
		default:
			return DWNCORONA2UP;
			break;
		}
	}
	return DWNCORONA2UP;
};

VUnit Corona::GetVU2Dwn()
{ 
	if(abovenota)
	{
		switch(tipo)
		{
		case (Classic):
			return UPCORONA2DWN;
			break;
		case (Quad):
			return UPCORONAQUAD2DWN;
			break;
		case (Triang):
			return UPCORONATRIANG2DWN;
			break;
		}
	}
	
	else
	{
		switch(tipo)
		{
		case (Classic):
			return DWNCORONA2DWN;
			break;
		case (Quad):
			return DWNCORONAQUAD2DWN;
			break;
		case (Triang):
			return DWNCORONATRIANG2DWN;
			break;
		default:
			return DWNCORONA2DWN;
			break;
		}
	}
	return DWNCORONA2DWN;
}; 

VUnit Corona::GetVU2Rgt()
{ 
	if(abovenota)
	{
		switch(tipo)
		{
		case (Classic):
			return UPCORONA2RGT;
			break;
		case (Quad):
			return UPCORONAQUAD2RGT;
			break;
		case (Triang):
			return UPCORONATRIANG2RGT;
			break;
		}
	}
	
	else
	{
		switch(tipo)
		{
		case (Classic):
			return DWNCORONA2RGT;
			break;
		case (Quad):
			return DWNCORONAQUAD2RGT;
			break;
		case (Triang):
			return DWNCORONATRIANG2RGT;
			break;
		default:
			return DWNCORONA2RGT;
			break;
			
		}
	}
	return DWNCORONA2RGT;
}; 

VUnit Corona::GetVU2Lft()
{ 
	if(abovenota)
	{
		switch(tipo)
		{
		case (Classic):
			return UPCORONA2LFT;
			break;
		case (Quad):
			return UPCORONAQUAD2LFT;
			break;
		case (Triang):
			return UPCORONATRIANG2LFT;
			break;
		}
	}
	else
	{
		switch(tipo)
		{
		case (Classic):
			return DWNCORONA2LFT;
			break;
		case (Quad):
			return DWNCORONAQUAD2LFT;
			break;
		case (Triang):
			return DWNCORONATRIANG2LFT;
			break;
		default:
			return DWNCORONA2LFT;
			break;
		}
	}
	return DWNCORONA2LFT;
}; 

void Corona::SetAboveNota(Bool corpos) {abovenota=corpos;};

void Corona::SetTipo(CoronaType tipocorona) {tipo = tipocorona;};

Bool Corona::GetAboveNota() {return abovenota;};

void Corona::Draw()
{
	FontID oldfont;
	char code[2]={0,0};
	
	oldfont = nMusicFont;
	
	if (abovenota)
	{
		switch(tipo)
		{
		case (Classic):
			code[0]=(char)CODE_CORONAUP;
			break;
		case (Quad):
			code[0]=(char)CODE_CORONAQUADUP;
			break;
		case (Triang):
			code[0]=(char)CODE_CORONATRIANGUP;
			break;
		}
	}
	else
	{
		switch(tipo)
		{
		case (Classic):
			code[0]=CODE_CORONADWN;
			break;
		case (Quad):
			Scr->SetFont(nMusicFont2);
			code[0]=CODE_CORONAQUADDWN;
			break;
		case (Triang):
			Scr->SetFont(nMusicFont2);
			code[0]=CODE_CORONATRIANGDWN;
			break;
		default:
			code[0]=CODE_CORONADWN;
			break;
		};
	}
	
	Colore oldcol= Scr->GetColor();
	if(isOpaque)
		Scr->SetColor(GetColor());

	
	if (abovenota)
	{
		switch(tipo)
		{
		case (Classic):
			Scr->TextTo(GetAbsPos()+Point(UPDELTAXCORONA,UPDELTAYCORONA)+Point(GetDx(),GetDy()));
			break;
		case (Quad):
			Scr->TextTo(GetAbsPos()+Point(UPDELTAXCORONAQUAD,UPDELTAYCORONAQUAD)+Point(GetDx(),GetDy()));
			break;
		case (Triang):
			Scr->TextTo(GetAbsPos()+Point(UPDELTAXCORONATRIANG,UPDELTAYCORONATRIANG)+Point(GetDx(),GetDy()));
			break;
		}
	}
	else
	{
		switch(tipo)
		{
		case (Classic):
			Scr->TextTo(GetAbsPos()+Point(DWNDELTAXCORONA,DWNDELTAYCORONA)+Point(GetDx(),GetDy()));
			break;
		case (Quad):
			Scr->TextTo(GetAbsPos()+Point(DWNDELTAXCORONAQUAD,DWNDELTAYCORONAQUAD)+Point(GetDx(),GetDy()));
			break;
		case (Triang):
			Scr->TextTo(GetAbsPos()+Point(DWNDELTAXCORONATRIANG,DWNDELTAYCORONATRIANG)+Point(GetDx(),GetDy()));
			break;
		default:
			Scr->TextTo(GetAbsPos()+Point(DWNDELTAXCORONA,DWNDELTAYCORONA)+Point(GetDx(),GetDy()));
			break;
			
		}
	}
	
	Scr->OutText((unsigned char *)code);
	Scr->SetFont(oldfont);

	Scr->SetColor(oldcol);
};  
