/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */



#include <xercesc/parsers/saxparser.hpp>
#include <xercesc/sax/documenthandler.hpp>
#include <xercesc/sax/errorhandler.hpp>
#include <xercesc/sax/saxparseexception.hpp>
#include <xercesc/sax/entityresolver.hpp>
#include <xercesc/framework/localfileinputsource.hpp>
#include <xercesc/dom/deprecated/dom.hpp>
#include <xercesc/dom/deprecated/DOMParser.hpp>
#include "XMLCommonClasses.hpp"
#include "smr/smr.hpp"
#include "xml/loa/wdfcomponentparser.hpp"
#include "xml/all.hpp"	
#include "bat/bsingola.hpp"
#include "pag/casellatxt.hpp"
#include "pag/casellaimg.hpp"

#define COMPUTER_VIEW 0
#define PRINT_VIEW 1

#ifndef STRREPLACE
#define STRREPLACE

int strreplace(char* str, char a, char b)
{
	int nrep=0;
	while(*str!='\0')
	{
		if(*str==a)
		{
			nrep++;
			*str=b;
		}
		str++;
	}
	return nrep;
}
#endif

class XMLSymbolicHandler : public DocumentHandler, public ErrorHandler
{
protected:
	Partitura* mainScore;
	Locator* const loc;
	SMRFile* smrf;
	
	Spartito* curScore;
	Battuta* curMeasure;
	int curLayer;
	int curStaff;
	Figura* curFigure;
	Nota* curNote;
	TipoPitch curPitch;
	Pausa* curRest;
	Accordo* curChord;
	Nota * curChordNote;
	bool curChordAcciaccatura;
	GruppoNote* curBeam;
	Lettera *curLabel;
	TSalto *curJump;
	TGenerico *curText;
	Annotazione *curAnnotation;
	IntEsteso *curHoriz;
	int curAddress;
	int curPage;
	CasellaTxt* curTextBox;
	
	int tabSize[3];	
	
	StrX *chordDur;
	
	int errCount;
	
	bool domMode;
	DOM_Document domDoc;
	DOM_Node parent;
	bool loadingMainScore;
	bool clipboard;	
	Spartito* refSpa;
	bool posUp;
	int progressive;
	
public:
	XMLSymbolicHandler(Partitura* ms,SMRFile*,bool lms,bool cp=false);
	
	void  setRefSpa(Spartito *refspa) {refSpa=refspa;}
	void  setPosUp(bool mode) {posUp=mode;}
	
	Spartito* GetCurrentScore() { return curScore; };
	
	void  setDocumentLocator (const Locator* const locator) {};
	void  resetDocument () { };
	
	void  startDocument ();
	void  startElement ( const XMLCh* const name , AttributeList& attrs );
	void  characters ( const XMLCh* const chars , const unsigned int length );
	void  endElement (const XMLCh* const name);
	void  endDocument ();
	
	void  startDom ( const XMLCh* const name , AttributeList& attrs );
	void  domCharacters ( const XMLCh* const chars , const unsigned int length );
	void  endDom (const XMLCh* const name);
	DOM_Document getDomDocument() { return domDoc; };
	
	void  ignorableWhitespace ( const XMLCh* const chars , const unsigned int length );
	void  processingInstruction ( const XMLCh* const target , const XMLCh* const data );
	
	int getErrCount() { return errCount; };
	
	void  resetErrors () { errCount=0;};
	void  warning (const SAXParseException& exception) { 
		StrX msg=exception.getMessage();
		StrX filename=exception.getSystemId();
		wxLogMessage("WARNING %s(%d:%d): %s",  (const char*)filename, exception.getLineNumber(), exception.getColumnNumber() , (const char*)msg);
		errCount++; 
	};
	void  error (const SAXParseException& exception) { 
		StrX msg=exception.getMessage();
		StrX filename=exception.getSystemId();
		wxLogMessage("ERROR %s(%d:%d): %s",  (const char*)filename, exception.getLineNumber(), exception.getColumnNumber() , (const char*)msg);
		errCount+=2; 
	};
	void  fatalError (const SAXParseException& exception) 
	{ 
		StrX msg=exception.getMessage();
		StrX filename=exception.getSystemId();
		wxLogError("FATALERROR %s(%d:%d): %s",  (const char*)filename, exception.getLineNumber(), exception.getColumnNumber() , (const char*)msg);
		errCount+=10;
	};
	
	void notImplemented(const char* element, const char*attribute=NULL, const char* value=NULL);
	
	Chiave* parseClefType(const StrX& cleftype);
	tipoArmaturaChiave parseKeySignatureType(const StrX& kstype);
	Alterazione* parseAccidentalType(const StrX& kstype);
	
	NoteHead_ID parseNoteHeadType(const StrX& type);
	
	void startBracket(AttributeList& attrs);
	void startScore(AttributeList& attrs);
	void endScore();
	void startMeasure(AttributeList& attrs);
	void endMeasure();
	void startJustification(AttributeList& attrs);
	void startLabel(AttributeList& attrs);
	void endLabel();
	void startJump(AttributeList& attrs);
	void endJump();
	void startHeader(AttributeList& attrs);
	void startClef(AttributeList& attrs);
	void endClef();
	void startKeySignature(AttributeList& attrs);
	void endKeySignature();
	void startTimeSignature(AttributeList& attrs);
	void endTimeSignature();
	void startBeatScan(AttributeList& attrs);
	void startMetronome(AttributeList& attrs);
	void startLayer(AttributeList& attrs);
	void endLayer();
	void startBarline(AttributeList& attrs);
	void startNote(AttributeList& attrs);
	void endNote();
	void startPitch(AttributeList& attrs);
	void startRest(AttributeList& attrs);
	void endRest();
	void startBeam(AttributeList& attrs);
	void endBeam();
	void startChord(AttributeList& attrs);
	void endChord();
	void startChordNote(AttributeList& attrs);
	void endChordNote();
	void startChangeClef(AttributeList& attrs);
	void endFigure();
	void startChangeKeySignature(AttributeList& attrs);
	void startAnchorage(AttributeList& attrs);
	void startRepetition(AttributeList& attrs);
	void startHorizontal(AttributeList& attrs);
	void startAddress(AttributeList& attrs);
	void endHorizontal();
	
	void startAccidental(AttributeList& attrs);
	void startAugmentation(AttributeList& attrs);
	void startUserSpacing(AttributeList& attrs);
	
	void startOrnament(AttributeList& attrs);
	void startKoreanOrnament(AttributeList& attrs);
	void startArpeggio(AttributeList& attrs);
	
	void startText(AttributeList& attrs);
	void endText();
	void startAnnotation(AttributeList& attrs);
	void endAnnotation();
	void startFingering(AttributeList& attrs);
	void startStrings(AttributeList& attrs);
	void startMute(AttributeList& attrs);
	void startHarmonic(AttributeList& attrs);
	void startFretboard(AttributeList& attrs);
	void startFermata(AttributeList& attrs);
	void startDynamicText(AttributeList& attrs);
	void startGlasses(AttributeList& attrs);
	void startBreath(AttributeList& attrs);
	void startPiano(AttributeList& attrs);
	void startDrum(AttributeList& attrs);
	void endDrum();
	void startMarker(AttributeList& attrs);
	void startPrintPages(AttributeList& attrs);
	void startPage(AttributeList& attrs);	
	void startTextBox(AttributeList& attrs);
	void endTextBox();
	void startImageBox(AttributeList& attrs);
	void startFormatPageComputerView(AttributeList& attrs);
	void startFormatPagePrintView(AttributeList& attrs);
	void startFormatPage(AttributeList& attrs);
	
	
};

class XMLSymbolicEntityResolver : public EntityResolver
{
public:
	InputSource* resolveEntity( const XMLCh* const publicId, const XMLCh* const systemId );
};

static void ReadDxDyAttr(AttributeList& attrs, DrawObject *ptrDrw)
{
	if(ptrDrw==NULL)
		return;

	StrX maindx=attrs.getValue("MAINDX");
	StrX maindy=attrs.getValue("MAINDY");

	StrX partdx=attrs.getValue("PARTDX");
	StrX partdy=attrs.getValue("PARTDY");

	StrX pmaindx=attrs.getValue("PMAINDX");
	StrX pmaindy=attrs.getValue("PMAINDY");

	StrX ppartdx=attrs.getValue("PPARTDX");
	StrX ppartdy=attrs.getValue("PPARTDY");

	ptrDrw->SetDxMainScore(COMPUTER_VIEW,atof(maindx));
	ptrDrw->SetDyMainScore(COMPUTER_VIEW,atof(maindy));
	ptrDrw->SetDxPart(COMPUTER_VIEW,atof(partdx));
	ptrDrw->SetDyPart(COMPUTER_VIEW,atof(partdy));
	ptrDrw->SetDxMainScore(PRINT_VIEW,atof(pmaindx));
	ptrDrw->SetDyMainScore(PRINT_VIEW,atof(pmaindy));
	ptrDrw->SetDxPart(PRINT_VIEW,atof(ppartdx));
	ptrDrw->SetDyPart(PRINT_VIEW,atof(ppartdx));
}

static void ReadDxDyRAttr(AttributeList& attrs, IntEsteso *ptrIntEst)
{
	if(ptrIntEst==NULL)
		return;

	StrX maindx=attrs.getValue("MAINDXR");
	StrX maindy=attrs.getValue("MAINDYR");

	StrX partdx=attrs.getValue("PARTDXR");
	StrX partdy=attrs.getValue("PARTDYR");

	StrX pmaindx=attrs.getValue("PMAINDXR");
	StrX pmaindy=attrs.getValue("PMAINDYR");

	StrX ppartdx=attrs.getValue("PPARTDXR");
	StrX ppartdy=attrs.getValue("PPARTDYR");

	ptrIntEst->SetDxRMainScore(COMPUTER_VIEW,atof(maindx));
	ptrIntEst->SetDyRMainScore(COMPUTER_VIEW,atof(maindy));
	ptrIntEst->SetDxRPart(COMPUTER_VIEW,atof(partdx));
	ptrIntEst->SetDyRPart(COMPUTER_VIEW,atof(partdy));
	ptrIntEst->SetDxRMainScore(PRINT_VIEW,atof(pmaindx));
	ptrIntEst->SetDyRMainScore(PRINT_VIEW,atof(pmaindy));
	ptrIntEst->SetDxRPart(PRINT_VIEW,atof(ppartdx));
	ptrIntEst->SetDyRPart(PRINT_VIEW,atof(ppartdx));
}

InputSource* XMLSymbolicEntityResolver::resolveEntity( const XMLCh* const publicId, const XMLCh* const systemId )
{
	return new LocalFileInputSource(L"Z:\\moods\\dtd\\", systemId);
}

XMLSymbolicHandler::XMLSymbolicHandler(Partitura* ms, SMRFile *psmr, bool lms, bool cp) : loc(0) 
{
	mainScore=ms; 
	errCount=0;
	curScore=NULL;
	curMeasure=NULL;
	curFigure=NULL;
	curNote=NULL;
	curRest=NULL;
	curChord=NULL;
	curChordNote=NULL;
	chordDur=NULL;
	curBeam=NULL;
	curLayer=1;
	curLabel=NULL;
	curJump=NULL;
	curText=NULL;
	curAnnotation=NULL;
	domMode=false;
	curPage=0;
	curTextBox=NULL;
	loadingMainScore=lms;
	clipboard=cp;
	smrf=psmr;
		
   	// Crea il documento.
	DOM_DOMImplementation impl;
	
	domDoc = impl.createDocument
		(
		0,                 // namespace
		"TMP",    	       // il nome dell'elemento root
		DOM_DocumentType() // DTD xxx.
		);
	parent=domDoc.getDocumentElement();
};

void  XMLSymbolicHandler::startDocument ()
{
	curScore=NULL;
	curMeasure=NULL;
	curFigure=NULL;
	curNote=NULL;
	curRest=NULL;
	curChord=NULL;
	curChordNote=NULL;
	chordDur=NULL;
	curBeam=NULL;
	curLabel=NULL;
	curJump=NULL;
	curText=NULL;
	curAnnotation=NULL;
	curLayer=1;
	curChordAcciaccatura=false;
	domMode=false;
	curPage=0;
	curTextBox=NULL;
}

bool isHeaderElement(const StrX& lname)
{
	return lname.equals("Identification") || lname.equals("Classification") || lname.equals("selections");
}

void  XMLSymbolicHandler::startElement ( const XMLCh* const qname , AttributeList& attrs )
{
	//remove the possible namespace
	const XMLCh* name=qname;
	for(int i=0; qname[i]!=0; i++)
		if(qname[i]==':')
		{
			name=&name[i+1];
			break;
		}

	StrX lname(name);
	
	if(isHeaderElement(lname))	//FFF WaterMark va tolto, Identification & Classification diventano optional
		domMode=true;
	
	if(domMode)
	{
		startDom(name,attrs);
		return;
	}
								//FFF Preferences da aggiungere
	if(lname.equals("bracket"))
		startBracket(attrs);
	else if(lname.equals("score"))
		startScore(attrs);
	else if(lname.equals("measure"))
		startMeasure(attrs);
	else if(lname.equals("barline"))
		startBarline(attrs);
	else if(lname.equals("justification"))
		startJustification(attrs);
	else if(lname.equals("label"))
		startLabel(attrs);
	else if(lname.equals("jump"))
		startJump(attrs);
	else if(lname.equals("header"))
		startHeader(attrs);
	else if(lname.equals("clef"))
		startClef(attrs);
	else if(lname.equals("keysignature"))
		startKeySignature(attrs);
	else if(lname.equals("timesignature"))
		startTimeSignature(attrs);
	else if(lname.equals("beatscan"))
		startBeatScan(attrs);
	else if(lname.equals("metronome"))
		startMetronome(attrs);
	else if(lname.equals("layer"))
		startLayer(attrs);
	else if(lname.equals("note"))
		startNote(attrs);
	else if(lname.equals("pitch"))
		startPitch(attrs);
	else if(lname.equals("rest"))
		startRest(attrs);
	else if(lname.equals("beam"))
		startBeam(attrs);
	else if(lname.equals("chord"))
		startChord(attrs);
	else if(lname.equals("chordnote"))
		startChordNote(attrs);
	else if(lname.equals("changeclef"))
		startChangeClef(attrs);
	else if(lname.equals("changekeysignature"))
		startChangeKeySignature(attrs);
	else if(lname.equals("anchorage"))
		startAnchorage(attrs);
	else if(lname.equals("repetition"))
		startRepetition(attrs);
	else if(lname.equals("horizontal"))
		startHorizontal(attrs);
	else if(lname.equals("address"))
		startAddress(attrs);
	else if(lname.equals("accidental"))
		startAccidental(attrs);
	else if(lname.equals("augmentation"))
		startAugmentation(attrs);
	else if(lname.equals("userspacing"))
		startUserSpacing(attrs);
	else if(lname.equals("ornament"))
		startOrnament(attrs);
	else if(lname.equals("korean-ornament"))
		startKoreanOrnament(attrs);
	else if(lname.equals("arpeggio"))
		startArpeggio(attrs);
	else if(lname.equals("text"))
		startText(attrs);
	else if(lname.equals("annotation"))
		startAnnotation(attrs);
	else if(lname.equals("fingering"))
		startFingering(attrs);
	else if(lname.equals("strings"))
		startStrings(attrs);
	else if(lname.equals("mute"))
		startMute(attrs);
	else if(lname.equals("harmonic"))
		startHarmonic(attrs);
	else if(lname.equals("fretboard"))
		startFretboard(attrs);
	else if(lname.equals("fermata"))
		startFermata(attrs);
	else if(lname.equals("dynamictext"))
		startDynamicText(attrs);
	else if(lname.equals("glasses"))
		startGlasses(attrs);
	else if(lname.equals("breath"))
		startBreath(attrs);
	else if(lname.equals("piano"))
		startPiano(attrs);
	else if(lname.equals("drum"))
		startDrum(attrs);
	else if(lname.equals("marker"))
		startMarker(attrs);
	else if(lname.equals("printpages"))		
		startPrintPages(attrs);
	else if(lname.equals("page"))	
		startPage(attrs);
	else if(lname.equals("textbox"))	
		startTextBox(attrs);
	else if(lname.equals("imagebox"))	
		startImageBox(attrs);
	else if(lname.equals("formatpageprintview"))	
		startFormatPagePrintView(attrs);
	else if(lname.equals("formatpagecomputerview"))	
		startFormatPageComputerView(attrs);
	else if(lname.equals("formatpage"))	
		startFormatPage(attrs);
}

void  XMLSymbolicHandler::characters ( const XMLCh* const chars , const unsigned int length )
{
	
	if(domMode)
	{
		domCharacters(chars,length);
		return;
	}
	
	if(curLabel!=NULL)
	{
		StrX label(chars);
		if(curLabel->GetTxt()==NULL)
			curLabel->SetTxt((unsigned char*)(const char*)label);
		else
		{
			wxString txt=curLabel->GetTxt();
			txt+=label;
			curLabel->SetTxt((unsigned char*)txt.c_str());
		}
	}
	else if(curJump!=NULL)
	{
		StrX jump(chars);
		if(curJump->GetTxt()==NULL)
			curJump->SetTxt((unsigned char*)(const char*)jump);
		else
		{
			wxString txt=curJump->GetTxt();
			txt+=jump;
			curJump->SetTxt((unsigned char*)txt.c_str());
		}
	}
	else if(curText!=NULL)
	{
		StrX text(chars);
		if(curText->GetTxt()==NULL)
			curText->SetTxt((char*)(const char*)text);
		else
		{
			wxString txt=curText->GetTxt();
			txt+=text;
			curText->SetTxt((char*)txt.c_str());
		}
	}
	else if(curAnnotation!=NULL)
	{
		StrX annotation(chars);
		if(curAnnotation->GetTxt()==NULL)
			curAnnotation->SetTxt((char*)(const char*)annotation);
		else
		{
			wxString txt=curAnnotation->GetTxt();
			txt+=annotation;
			curAnnotation->SetTxt((char*)txt.c_str());
		}
	}
	else if(curTextBox!=NULL)
	{
		StrX text(chars);
		wxString txt=curTextBox->GetTesto()+text;
		curTextBox->SetTesto(txt);
	}
}

void  XMLSymbolicHandler::endElement (const XMLCh* const qname)
{
	//remove the possible namespace
	const XMLCh* name=qname;
	for(int i=0; qname[i]!=0; i++)
		if(qname[i]==':')
		{
			name=&name[i+1];
			break;
		}

	StrX lname(name);
	
	if(domMode)
	{
		endDom(name);
		if(isHeaderElement(lname))
			domMode=false;
	}
	
	if(lname.equals("selections"))
	{
		DOM_Node* selections=new DOM_Node(parent.cloneNode(true));
		mainScore->SetSelections(selections);
	}
	else if(lname.equals("score"))
		endScore();
	else if(lname.equals("measure"))
		endMeasure();
	else if(lname.equals("label"))
		endLabel();
	else if(lname.equals("jump"))
		endJump();
	else if(lname.equals("note"))
		endNote();
	else if(lname.equals("rest"))
		endRest();
	else if(lname.equals("beam"))
		endBeam();
	else if(lname.equals("chord"))
		endChord();
	else if(lname.equals("chordnote"))
		endChordNote();
	else if(lname.equals("changeclef"))
		endFigure();
	else if(lname.equals("changekeysignature"))
		endFigure();
	else if(lname.equals("anchorage"))
		endFigure();
	else if(lname.equals("repetition"))
		endFigure();
	else if(lname.equals("text"))
		endText();
	else if(lname.equals("annotation"))
		endAnnotation();
	else if(lname.equals("horizontal"))
		endHorizontal();
	else if(lname.equals("textbox"))	
		endTextBox();
}

void  XMLSymbolicHandler::endDocument ()
{
}

void  XMLSymbolicHandler::ignorableWhitespace ( const XMLCh* const chars , const unsigned int length )
{
}

void  XMLSymbolicHandler::processingInstruction ( const XMLCh* const target , const XMLCh* const data )
{
}

void  XMLSymbolicHandler::startScore(AttributeList& attrs)
{
	curScore=new Spartito;
	curScore->Free();
	
	StrX instrument(attrs.getValue("INSTRUMENT"));
	
	if(!instrument.equals(""))
	{
		curScore->SetStrumEsec((char*)(const char*)instrument);
	}
	
	StrX id=attrs.getValue("ID");
	int scoreID=atoi(id);
	
	if(scoreID!=0)
		curScore->SetNumericCode(scoreID);
	
	StrX nstaffs=attrs.getValue("NUMBEROFSTAFFS");
	int ns=atoi(nstaffs);
	if(ns>0)
		curScore->SetNumberOfStaff(ns);
	else
		curScore->SetNumberOfStaff(1);
	
	for(int i=0;i<curScore->GetNumberOfStaff();i++)
	{
		StrX tablatureSize=attrs.getValue(wxString::Format("TABLATURESIZE%d",i+1));
		tabSize[i]=atoi(tablatureSize);
	}
	
	StrX lyric=attrs.getValue("LYRIC");
	if(smrf!=NULL)
	{
		SMRItem *lyr=smrf->GetItemByID(wxString(lyric));
		if(lyr!=NULL && lyr->GetType()==SMR_Lyrics)
			curScore->SetCurrentLyrWF((SMRLyricsItem*)lyr);
	}

	StrX transpose=attrs.getValue("TRANSPOSE");
	curScore->SetTranspose(atoi(transpose));

	StrX size=attrs.getValue("SIZE");
	if(size.equals("SMALL"))
		curScore->SetSmallSize(true);

	StrX shortInstrName=attrs.getValue("SHORTINSTRNAME");
	curScore->SetShortInstrName(shortInstrName);
	
	progressive=0;
}

void  XMLSymbolicHandler::endScore()
{
	if(curScore!=NULL)
	{
		NumCode nc=curScore->GetNumericCode();
		
		
		for(int i=0;i<curScore->GetNumberOfStaff();i++)
		{
			if(tabSize[i]>0)
				curScore->SetNCorde(i,tabSize[i]); 
			else  
				curScore->SetNCorde(i,5);   
		}
		
		if(!clipboard)
		{
			mainScore->AddSpartito(curScore,false); //non va aggiunto al smrtree perche' c'e' gia'
			if(curScore->GetCurrentLyrWF()!=NULL)
			{
				wxString lyrFileName=smrf->GetPath()+PATH_SEP+curScore->GetCurrentLyrWF()->GetFilename();
				curScore->LoadLyr((char*)lyrFileName.c_str());
			}
		}
		else
		{
			mainScore->AddSpartitoAt(curScore,refSpa,posUp);
			mainScore->GetListaSpa()->BalanceSpartiti();
			mainScore->GoTop();
		}
	}
}

void  XMLSymbolicHandler::startMeasure(AttributeList& attrs)
{
	char empty[1]={0};
	curMeasure=new Battuta;
	StrX id=attrs.getValue("ID");
	NumCode nc=atoi(id);
	curMeasure->SetNumericCode(nc);
	
	StrX barline=attrs.getValue("BARLINE");
	if(barline.equals("SINGLE"))
		curMeasure->SetBarra(SINGOLA);
	else if(barline.equals("END"))
		curMeasure->SetBarra(FINALE);
	else if(barline.equals("DOUBLE"))
		curMeasure->SetBarra(DOPPIA);
	else if(barline.equals("START_REFRAIN"))
		curMeasure->SetBarra(INIZIORIT);
	else if(barline.equals("END_REFRAIN"))
		curMeasure->SetBarra(FINERIT);
	else if(barline.equals("START_END_REFRAIN"))
		curMeasure->SetBarra(INIZIOFINERIT);
	else if(barline.equals("INVISIBLE"))
		curMeasure->SetBarra(INVISIBILE);
	else if(barline.equals(empty))
		;
	else 
		notImplemented("measure","BARLINE",barline);
	
	progressive++;
	StrX progress=attrs.getValue("PROGRESSIVE");
	int prog=atoi(progress);
	curMeasure->SetNumProgress(progressive);
	
	StrX nstaffs=attrs.getValue("NUMBEROFSTAFFS");
	int ns=atoi(nstaffs);
	if(ns>0)
		curMeasure->SetNumberOfStaff(ns);
	else
		curMeasure->SetNumberOfStaff(curScore->GetNumberOfStaff());
	curStaff=-1;
}

void  XMLSymbolicHandler::startBarline(AttributeList& attrs)
{
	StrX type=attrs.getValue("TYPE");
	StrX repeat=attrs.getValue("REPEAT");
	StrX ltype=attrs.getValue("LINETYPE");
	
	if(type.equals("SINGLE"))
	{
		curMeasure->SetBarra(SINGOLA);
		BSingola *bsingola=(BSingola*)curMeasure->GetBarra();
		
		if(repeat.equals("TRUE") || repeat.equals("true"))
			bsingola->SetRepeatDoubleMeasure(true);
		
		if(ltype.equals("SOLID"))
			bsingola->SetLineType(SOLID_LINE);
		else if(ltype.equals("DOT"))
			bsingola->SetLineType(DOTTED_LINE);
		else if(ltype.equals("DASH"))
			bsingola->SetLineType(DASHED_LINE);
		else
			bsingola->SetLineType(SOLID_LINE);
	}
	else if(type.equals("END"))
		curMeasure->SetBarra(FINALE);
	else if(type.equals("DOUBLE"))
		curMeasure->SetBarra(DOPPIA);
	else if(type.equals("START_REFRAIN"))
		curMeasure->SetBarra(INIZIORIT);
	else if(type.equals("END_REFRAIN"))
		curMeasure->SetBarra(FINERIT);
	else if(type.equals("START_END_REFRAIN"))
		curMeasure->SetBarra(INIZIOFINERIT);
	else if(type.equals("INVISIBLE"))
		curMeasure->SetBarra(INVISIBILE);
	else 
		notImplemented("barline","TYPE",type);
	
}

void  XMLSymbolicHandler::endMeasure()
{
	curMeasure->SetupGambi();
	curScore->AddBattuta(curMeasure);
	if(curScore!=NULL)
	{
		if(curScore->GetCodeCounterBat()<=curMeasure->GetNumericCode())
			curScore->SetCodeCounterBat(curMeasure->GetNumericCode()+1);
	}
	curMeasure=NULL;
}

void  XMLSymbolicHandler::startJustification(AttributeList& attrs)
{
	StrX maintype=attrs.getValue("MAINTYPE");
	StrX mainjust=attrs.getValue("MAINJUST");
	StrX parttype=attrs.getValue("PARTTYPE");
	StrX partjust=attrs.getValue("PARTJUST");
	StrX pmaintype=attrs.getValue("PMAINTYPE");
	StrX pmainjust=attrs.getValue("PMAINJUST");
	StrX pparttype=attrs.getValue("PPARTTYPE");
	StrX ppartjust=attrs.getValue("PPARTJUST");
	
	StrX pmainlinebreak=attrs.getValue("PMAINLINEBREAK");
	StrX mainlinebreak=attrs.getValue("MAINLINEBREAK");
	StrX ppartlinebreak=attrs.getValue("PPARTLINEBREAK");
	StrX partlinebreak=attrs.getValue("PARTLINEBREAK");
	
	if(maintype.equals("LINEAR"))
		curMeasure->Settipogiust(G_LIN,COMPUTER_VIEW);
	else if(maintype.equals("LOG"))
		curMeasure->Settipogiust(G_LOG,COMPUTER_VIEW);
	else
		curMeasure->Settipogiust(G_NONE,COMPUTER_VIEW);
	curMeasure->Setkgiustificazione(atof(mainjust),COMPUTER_VIEW);
	
	if(parttype.equals("LINEAR"))
		curMeasure->Settipogiustparte(G_LIN,COMPUTER_VIEW);
	else if(parttype.equals("LOG"))
		curMeasure->Settipogiustparte(G_LOG,COMPUTER_VIEW);
	else
		curMeasure->Settipogiustparte(G_NONE,COMPUTER_VIEW);
	curMeasure->Setkgiustificazioneparte(atof(partjust),COMPUTER_VIEW);
	
	if(pmaintype.equals("LINEAR"))
		curMeasure->Settipogiust(G_LIN,PRINT_VIEW);
	else if(pmaintype.equals("LOG"))
		curMeasure->Settipogiust(G_LOG,PRINT_VIEW);
	else
		curMeasure->Settipogiust(G_NONE,PRINT_VIEW);
	curMeasure->Setkgiustificazione(atof(pmainjust),PRINT_VIEW);
	
	if(pparttype.equals("LINEAR"))
		curMeasure->Settipogiustparte(G_LIN,PRINT_VIEW);
	else if(pparttype.equals("LOG"))
		curMeasure->Settipogiustparte(G_LOG,PRINT_VIEW);
	else
		curMeasure->Settipogiustparte(G_NONE,PRINT_VIEW);
	curMeasure->Setkgiustificazioneparte(atof(ppartjust),PRINT_VIEW);
	
	if(pmainlinebreak.equals("TRUE") || pmainlinebreak.equals("true"))
		curMeasure->SetMainScoreLineBreak(PRINT_VIEW,true);
	if(mainlinebreak.equals("TRUE") || mainlinebreak.equals("true"))
		curMeasure->SetMainScoreLineBreak(COMPUTER_VIEW,true);
	if(ppartlinebreak.equals("TRUE") || ppartlinebreak.equals("true"))
		curMeasure->SetPartLineBreak(PRINT_VIEW,true);
	if(partlinebreak.equals("TRUE") || partlinebreak.equals("true"))
		curMeasure->SetPartLineBreak(COMPUTER_VIEW,true);
}

void  XMLSymbolicHandler::startLabel(AttributeList& attrs)
{
	DrawObject *pDraw=NULL;
	StrX type=attrs.getValue("TYPE");
	if (type.equals("CODA"))
	{
		Coda *pCoda=new Coda;
		curMeasure->SetPRiferimento(pCoda);
		pDraw=pCoda;
	}	
	else if(type.equals("SEGNO"))
	{
		Segno *pSegno=new Segno;
		curMeasure->SetPRiferimento(pSegno);
		pDraw=pSegno;
	}
	else if(type.equals("TEXT"))
	{
		pDraw=curLabel=new Lettera;
	}
	ReadDxDyAttr(attrs,pDraw);
}

void  XMLSymbolicHandler::endLabel()
{
	if(curLabel!=NULL)
	{
		curMeasure->SetPLettera(curLabel);
		mainScore->AddEtichetta(curLabel->GetTxt()[0],curMeasure->GetNumericCode());
		curLabel=NULL;
	}
}

void  XMLSymbolicHandler::startJump(AttributeList& attrs)
{
	StrX type=attrs.getValue("TYPE");
	if (type.equals("DC"))
		notImplemented("jump","TYPE","DC");
	else if(type.equals("DS"))
		notImplemented("jump","TYPE","DS");
	else if(type.equals("DCAS"))
		notImplemented("jump","TYPE","DCAS");
	else if(type.equals("TEXT"))
	{
		curJump=new TSalto;
		ReadDxDyAttr(attrs,curJump);
	}
}

void  XMLSymbolicHandler::endJump()
{
	curMeasure->SetPTSalto(curJump);
	curJump=NULL;
}

void  XMLSymbolicHandler::startHeader(AttributeList& attrs)
{
	curStaff++;
}

Chiave*  XMLSymbolicHandler::parseClefType(const StrX& type)
{
	Chiave *curClef=NULL;
	
	if(type.equals("BARITONE"))
		curClef=new CBaritono;
	else if(type.equals("BASS"))
		curClef=new CBasso;
	else if(type.equals("ALTO"))
		curClef=new CContralto;
	else if(type.equals("MEZZOSOPRANO"))
		curClef=new CMezzoSoprano;
	else if(type.equals("SOPRANO"))
		curClef=new CSoprano;
	else if(type.equals("TENOR"))
		curClef=new CTenore;
	else if(type.equals("TENOR8"))
		curClef=new CTenore8;
	else if(type.equals("TREBLE"))
		curClef=new CViolino;
	else if(type.equals("TREBLE8"))
		curClef=new CViolino8;
	else if(type.equals("8TREBLE"))
		curClef=new CViolino8Sopra;
	else if(type.equals("BASS8"))
		curClef=new CBasso8;
	else if(type.equals("8BASS"))
		curClef=new CBasso8Sopra;
	else if(type.equals("BASSOLD"))
		curClef=new CBassoOld;
	else if(type.equals("EMPTY"))
		curClef=new CVuota;
	else if(type.equals("TAB"))
		curClef=new CTab;
	else if(type.equals("PERCUSBOX"))
		curClef=new CPercusBox;
	else if(type.equals("PERCUS2LINES"))
		curClef=new CPercus2Lines;
	else
		notImplemented("clef","TYPE",type);
	
	return curClef;
}

void  XMLSymbolicHandler::startClef(AttributeList& attrs)
{
	StrX type=attrs.getValue("TYPE");
	curMeasure->SetChiave(curStaff,parseClefType(type));
}

void  XMLSymbolicHandler::endClef()
{
}

tipoArmaturaChiave XMLSymbolicHandler::parseKeySignatureType(const StrX& type)
{
	tipoArmaturaChiave curKeySignature;
	
	if(type.equals("DObM"))
		curKeySignature=DOb_maggiore;
	else if(type.equals("DOdM"))
		curKeySignature=DOd_maggiore;
	else if(type.equals("DOM"))
		curKeySignature=DO_maggiore;
	else if(type.equals("FAdM"))
		curKeySignature=FAd_maggiore;
	else if(type.equals("FAM"))
		curKeySignature=FA_maggiore;
	else if(type.equals("LAbM"))
		curKeySignature=LAb_maggiore;
	else if(type.equals("LAM"))
		curKeySignature=LA_maggiore;
	else if(type.equals("MIbM"))
		curKeySignature=MIb_maggiore;
	else if(type.equals("MIM"))
		curKeySignature=MI_maggiore;
	else if(type.equals("REbM"))
		curKeySignature=REb_maggiore;
	else if(type.equals("REM"))
		curKeySignature=RE_maggiore;
	else if(type.equals("SIbM"))
		curKeySignature=SIb_maggiore;
	else if(type.equals("SIM"))
		curKeySignature=SI_maggiore;
	else if(type.equals("SOLbM"))
		curKeySignature=SOLb_maggiore;
	else if(type.equals("SOLM"))
		curKeySignature=SOL_maggiore;
	
	else if(type.equals("DOdm"))
		curKeySignature=DOd_minore;
	else if(type.equals("DOm"))
		curKeySignature=DO_minore;
	else if(type.equals("FAdm"))
		curKeySignature=FAd_minore;
	else if(type.equals("FAm"))
		curKeySignature=FA_minore;
	else if(type.equals("LAdm"))
		curKeySignature=LAd_minore;
	else if(type.equals("LAm"))
		curKeySignature=LA_minore;
	else if(type.equals("MIbm"))
		curKeySignature=MIb_minore;
	else if(type.equals("MIm"))
		curKeySignature=MI_minore;
	else if(type.equals("REdm"))
		curKeySignature=REd_minore;
	else if(type.equals("REm"))
		curKeySignature=RE_minore;
	else if(type.equals("SIbm"))
		curKeySignature=SIb_minore;
	else if(type.equals("SIm"))
		curKeySignature=SI_minore;
	else if(type.equals("SOLdm"))
		curKeySignature=SOLd_minore;
	else if(type.equals("SOLm"))
		curKeySignature=SOL_minore;
	else if(type.equals("LAbm"))
		curKeySignature=LAb_minore;
	else
		notImplemented("keysignature","type",type);
	
	return curKeySignature;
}

void  XMLSymbolicHandler::startKeySignature(AttributeList& attrs)
{
	StrX type=attrs.getValue("TYPE");
	curMeasure->SetArmatura(curStaff,parseKeySignatureType(type));
}

void  XMLSymbolicHandler::endKeySignature()
{
}

void  XMLSymbolicHandler::startTimeSignature(AttributeList& attrs)
{
	StrX status=attrs.getValue("STATUS");
	StrX type=attrs.getValue("TYPE");
	
	if(type.equals("C"))
		curMeasure->SetTempo("c");
	else if(type.equals("CSLASH"))
		curMeasure->SetTempo("C");
	else if(type.equals("FRACTION"))
	{
		char time[200];
		char empty[1]={0};
		StrX num=attrs.getValue("NUMERATOR");
		StrX den=attrs.getValue("DENOMINATOR");
		StrX tnum=attrs.getValue("TIMENUMERATOR");
		StrX tden=attrs.getValue("TIMEDENOMINATOR");
		if(tden.equals(empty) || tnum.equals(empty))
			sprintf(time,"%s/%s",(const char*)num,(const char*)den);
		else
		{
			extern int strreplace(char*,char,char);
			
			strreplace((char*)(const char*)num,'+','');
			strreplace((char*)(const char*)den,'+','');
			sprintf(time,"%s/%s%s/%s",(const char*)num,(const char*)den,(const char*)tnum,(const char*)tden);
		}
		curMeasure->SetTempo(time);
	}
	else
		notImplemented("timesignature","type",type);
	TimeSignatureStatus stat=TimeSignature_NORMAL;
	if(status.equals("HIDDEN"))
		stat=TimeSignature_HIDDEN;
	else if(status.equals("ALWAYSVISIBLE"))
		stat=TimeSignature_ALWAYSVISIBLE;
	for(int i=0;i<curMeasure->GetNumberOfStaff();i++)
		curMeasure->GetIntest(i)->SetTimeSignatureStatus(stat);
}

void  XMLSymbolicHandler::endTimeSignature()
{
}

void  XMLSymbolicHandler::startBeatScan(AttributeList& attrs)
{
	StrX nbeats=attrs.getValue("NBEATS");
	Scansione* curBeatScan=new Scansione;
	curBeatScan->SetNumScan(atoi(nbeats));
	curMeasure->SetPScansione(curBeatScan);
}

void  XMLSymbolicHandler::startMetronome(AttributeList& attrs)
{
	StrX movementtext=attrs.getValue("MOVEMENTTEXT");
	StrX dur=attrs.getValue("NOTETYPE");
	StrX dots=attrs.getValue("DOTS");
	StrX value=attrs.getValue("VALUE");
	
	Bool dot=atoi(dots)!=0;
	ClassID idnote;
	
	if(dur.equals("D2"))
		idnote=CL_NBREVE;
	else if(dur.equals("D1"))
		idnote=CL_NSEMIBREVE;
	else if(dur.equals("D1_2"))
		idnote=CL_NMINIMA;
	else if(dur.equals("D1_4"))
		idnote=CL_NSEMIMINIMA;
	else if(dur.equals("D1_8"))
		idnote=CL_NCROMA;
	else if(dur.equals("D1_16"))
		idnote=CL_NSEMICROMA;
	else if(dur.equals("D1_32"))
		idnote=CL_NBISCROMA;
	else if(dur.equals("D1_64"))
		idnote=CL_NSEMIBISCROMA;
	else if(dur.equals("D1_128"))
		idnote=CL_NFUSA;
	else if(dur.equals(""))
		idnote=(ClassID)0;
	else
		notImplemented("metronome","NOTETYPE",dur);
	
	Movimento *curMetronome=new Movimento;
	if(idnote!=0)
		curMetronome->SetMetronomo();
	curMetronome->MovSetTxt((unsigned char*)(const char*)movementtext,idnote,dot,atoi(value));
	curMeasure->SetPMovimento(curMetronome);
	ReadDxDyAttr(attrs,curMetronome);
}

void  XMLSymbolicHandler::startLayer(AttributeList& attrs)
{
	StrX id=attrs.getValue("NUMBER");
	curLayer=atoi(id);
}

void  XMLSymbolicHandler::endLayer()
{
}

NoteHead_ID XMLSymbolicHandler::parseNoteHeadType(const StrX& type)
{
	NoteHead_ID head=HEAD_CLASSIC;
	
	if(type.equals("CLASSIC"))
		head=HEAD_CLASSIC;
	else if(type.equals("CIRCLEX"))
		head=HEAD_CIRCLEX;
	else if(type.equals("ALPHANUM"))
		head=HEAD_ALPHANUM;
	else if(type.equals("ALPHANUM_SQUARE"))
		head=HEAD_ALPHANUM_SQUARE;
	else if(type.equals("ALPHANUM_REVERSE"))
		head=HEAD_ALPHANUM_REVERSE;
	else if(type.equals("X"))
		head=HEAD_X;
	else if(type.equals("CROIX"))
		head=HEAD_CROIX;
	else if(type.equals("DSHARP"))
		head=HEAD_DDIESIS;
	else if(type.equals("DIAMOND"))
		head=HEAD_DIAMOND;
	else if(type.equals("TRIANG"))
		head=HEAD_TRIANG;
	else if(type.equals("UPTRIANG"))
		head=HEAD_TRIANG_UP;
	else if(type.equals("DOWNTRIANG"))
		head=HEAD_TRIANG_DOWN;
	else if(type.equals("LEFTTRIANG"))
		head=HEAD_TRIANG_LEFT;
	else if(type.equals("RIGHTTRIANG"))
		head=HEAD_TRIANG_RIGHT;
	else if(type.equals("TRIANGROUND"))
		head=HEAD_TRIANG_ROUND;
	else if(type.equals("MOON"))
		head=HEAD_MOON;
	else if(type.equals("RHYTHMIC"))
		head=HEAD_RHYTHMIC;
	else if(type.equals("SQUARE"))
		head=HEAD_SQUARE;
	else if(type.equals("PLUS"))
		head=HEAD_PLUS;
	
	return head;
}

Alterazione* XMLSymbolicHandler::parseAccidentalType(const StrX& type)
{
	Alterazione *acc=NULL;
	
	if(type.equals("SHARP"))
		acc=new Diesis;
	else if(type.equals("FLAT"))
		acc=new Bemolle;
	else if(type.equals("NATURAL"))
		acc=new Bequadro;
	else if(type.equals("DSHARP"))
		acc=new DoppioDiesis;
	else if(type.equals("DFLAT"))
		acc=new DoppioBemolle;
	else if(type.equals("SHARP1Q"))
		acc=new Diesis1Q;
	else if(type.equals("SHARP3Q"))
		acc=new Diesis3Q;
	else if(type.equals("FLAT1Q"))
		acc=new Bemolle1Q;
	else if(type.equals("FLAT3Q"))
		acc=new Bemolle3Q;
	else if(type.equals("NATURALUP"))
		acc=new Bequadro1Q;
	else if(type.equals("NATURALDOWN"))
		acc=new Bequadro3Q;
	
	return acc;
}

void  XMLSymbolicHandler::startNote(AttributeList& attrs)
{
	char empty[1]={0};
	StrX dur=attrs.getValue("DURATION");
	StrX id=attrs.getValue("ID");
	StrX stem=attrs.getValue("STEM");
	StrX height=attrs.getValue("HEIGHT");
	StrX size=attrs.getValue("SIZE");
	StrX head=attrs.getValue("HEAD");
	StrX acciaccatura=attrs.getValue("ACCIACCATURA");
	StrX link=attrs.getValue("LINK");
	StrX nstaff=attrs.getValue("STAFF");
	StrX sstatus=attrs.getValue("STATUS");
	StrX ncolor=attrs.getValue("COLOR");
	
	if(dur.equals("D2"))
		curFigure=curNote=new NBreve;
	else if(dur.equals("D1"))
		curFigure=curNote=new NSemibreve;
	else if(dur.equals("D1_2"))
		curFigure=curNote=new NMinima;
	else if(dur.equals("D1_4"))
		curFigure=curNote=new NSemiminima;
	else if(dur.equals("D1_8"))
		curFigure=curNote=new NCroma;
	else if(dur.equals("D1_16"))
		curFigure=curNote=new NSemicroma;
	else if(dur.equals("D1_32"))
		curFigure=curNote=new NBiscroma;
	else if(dur.equals("D1_64"))
		curFigure=curNote=new NSemibiscroma;
	else if(dur.equals("D1_128"))
		curFigure=curNote=new NFusa;
	else
		notImplemented("note","DURATION",dur);
	
	if(curNote!=NULL)
	{
		curNote->SetNumericCode(atoi(id));
		curNote->SetCoda(TRUE);
		NoteHead_ID nhead=parseNoteHeadType(head);
		curNote->SetHeadType(nhead);
		switch(nhead)
		{
		case HEAD_ALPHANUM:
		case HEAD_ALPHANUM_SQUARE:
		case HEAD_ALPHANUM_REVERSE:
			{
				StrX headCode=attrs.getValue("HEAD_ALPHANUM");
				curNote->SetHeadCode((unsigned char*)(const char*)headCode);
			}
			break;
		default:
			break;
		}
		
		if(size.equals("SMALL"))
			curNote->SetNotina(TRUE);
		else if(size.equals("NORMAL"))
			curNote->SetNotina(FALSE);
		else
			curNote->SetNotina(FALSE);

		if(stem.equals("UP"))
			curNote->SetGamboUp();
		else if(stem.equals("DOWN"))
			curNote->SetGamboDwn();
		
		if(curNote->GetID()==CL_NCROMA)
		{
			if(acciaccatura.equals("TRUE") || acciaccatura.equals("true"))
				((NCroma*)curNote)->SetAcciaccatura(TRUE);
			else
				((NCroma*)curNote)->SetAcciaccatura(FALSE);
		}	
		curNote->SetAltezza(atoi(height));
		curNote->SetNStaff(atoi(nstaff));
			
		if(!link.equals(""))
			curNote->Set_Link((const char*)link);
		
		if(sstatus.equals("NORMAL"))
			curNote->SetStatus(NORMAL);
		else if(sstatus.equals("GRACED"))
			curNote->SetStatus(GRACED);
		else if(sstatus.equals("HIDDEN"))
			curNote->SetStatus(HIDDEN);
		else if(sstatus.equals("GHOSTED"))
			curNote->SetStatus(GHOSTED);
		else
			curNote->SetStatus(NORMAL);
		
		if(!ncolor.equals("")) curNote->SetColor(Colore(wxString((const char*)ncolor)));

	}
}

void  XMLSymbolicHandler::endNote()
{
	if(curNote!=NULL)
	{
		if(curBeam==NULL)
		{
			NumCode id=curNote->GetNumericCode();
			curMeasure->AddFig(curNote, curLayer-1);
			curNote->SetNumericCode(id);
			if(curMeasure->GetCodeCounterFig(curLayer-1)<=id)
				curMeasure->SetCodeCounterFig(curLayer-1,id+1);
		}
		else
		{
			curNote->SetPBattuta(curMeasure);
			curBeam->AddFigura(curNote);
		}
	}
	curFigure=curNote=NULL;
}

void  XMLSymbolicHandler::startPitch(AttributeList& attrs)
{
	if(curNote!=NULL || curChordNote!= NULL)
	{
		StrX base			= attrs.getValue("BASE");
		StrX shift			= attrs.getValue("SHIFT");
		StrX octave			= attrs.getValue("OCTAVE");
		
		if (shift.equals("#"))
			curPitch.Accidental = 1;
		else if (shift.equals("b"))
			curPitch.Accidental = -1;
		else
			curPitch.Accidental = 0;
		
		curPitch.Letter	= ( ((short)*((const char*)base)) +3 )%7;
		curPitch.Octave	= atoi(octave);

		if(curNote)			curNote->SetPitch(&curPitch);
		if(curChordNote)	curChordNote->SetPitch(&curPitch);
	}
}

void  XMLSymbolicHandler::startRest(AttributeList& attrs)
{
	char empty[1]={0};
	StrX dur=attrs.getValue("DURATION");
	StrX id=attrs.getValue("ID");
	StrX height=attrs.getValue("HEIGHT");
	StrX size=attrs.getValue("SIZE");
	StrX link=attrs.getValue("LINK");
	StrX nstaff=attrs.getValue("STAFF");
	StrX sstatus=attrs.getValue("STATUS");
	StrX ncolor=attrs.getValue("COLOR");
	
	if(dur.equals("D4M"))
		curFigure=curRest=new PQuattroBattute;
	else if(dur.equals("D2M") || dur.equals("D2"))
		curFigure=curRest=new PDueBattute;
	else if(dur.equals("D1"))
		curFigure=curRest=new PSemibreve;
	else if(dur.equals("D1_2"))
		curFigure=curRest=new PMinima;
	else if(dur.equals("D1_4"))
		curFigure=curRest=new PSemiminima;
	else if(dur.equals("D1_8"))
		curFigure=curRest=new PCroma;
	else if(dur.equals("D1_16"))
		curFigure=curRest=new PSemicroma;
	else if(dur.equals("D1_32"))
		curFigure=curRest=new PBiscroma;
	else if(dur.equals("D1_64"))
		curFigure=curRest=new PSemibiscroma;
	else if(dur.equals("D1_128"))
		curFigure=curRest=new PFusa;
	else if(dur.equals("DGENERIC"))
	{
		StrX measures=attrs.getValue("MEASURES");
		int nmeasures=atoi(measures);
		curFigure=curRest=new PGenerica(curMeasure);
		NumPausa *curNumRest = new NumPausa;
		curNumRest->SetNumero(nmeasures);
		curMeasure->SetPNumGrande(curNumRest);
		curMeasure->SetNumMultiRest(nmeasures);
		curMeasure->SetIsMultiRest();
	}
	else
		notImplemented("rest","DURATION",dur);
	
	if(curRest!=NULL)
	{
		curRest->SetNumericCode(atoi(id));
		curRest->SetAltezza(atoi(height));
		curRest->SetNStaff(atoi(nstaff));
		
		if(size.equals("SMALL"))
			curRest->SetNotina(TRUE);
		else if(size.equals("NORMAL"))
			curRest->SetNotina(FALSE);
		
		if(!link.equals(""))
			curRest->Set_Link((const char*)link);
		
		if(sstatus.equals("NORMAL"))
			curRest->SetStatus(NORMAL);
		else if(sstatus.equals("GRACED"))
			curRest->SetStatus(GRACED);
		else if(sstatus.equals("HIDDEN"))
			curRest->SetStatus(HIDDEN);
		else if(sstatus.equals("GHOSTED"))
			curRest->SetStatus(GHOSTED);
		else
			curRest->SetStatus(NORMAL);
		
		if(!ncolor.equals("")) curRest->SetColor(Colore(wxString((const char*)ncolor)));

	}
}

void  XMLSymbolicHandler::endRest()
{
	if(curRest!=NULL)
	{
		if(curBeam==NULL)
		{
			NumCode id=curRest->GetNumericCode();
			curMeasure->AddFig(curRest, curLayer-1);
			curRest->SetNumericCode(id);
			if(curMeasure->GetCodeCounterFig(curLayer-1)<=id)
				curMeasure->SetCodeCounterFig(curLayer-1,id+1);
		}
		else
		{
			curRest->SetPBattuta(curMeasure);
			curBeam->AddFigura(curRest);
		}
	}
	curFigure=curRest=NULL;
}

void  XMLSymbolicHandler::startBeam(AttributeList& attrs)
{
	StrX id=attrs.getValue("ID");
	StrX stems=attrs.getValue("STEMS");
	StrX size=attrs.getValue("SIZE");
	StrX sstatus=attrs.getValue("STATUS");
	
	curBeam=new GruppoNote;
	curBeam->SetNumericCode(atoi(id));
	if(size.equals("SMALL"))
		curBeam->SetNotina(TRUE);
	else if(size.equals("NORMAL"))
		curBeam->SetNotina(FALSE);
	else
		curBeam->SetNotina(FALSE);
	
	if(stems.equals("UP"))
		curBeam->SetGamboUp();
	else if(stems.equals("DOWN"))
		curBeam->SetGamboDwn();
	else if(stems.equals("STAFF12"))
		curBeam->SetBeamPosition(GruppoNote::BEAM_12);
	else if(stems.equals("STAFF23"))
		curBeam->SetBeamPosition(GruppoNote::BEAM_23);
	
	if(sstatus.equals("NORMAL"))
		curBeam->SetBeamStatus(NORMAL);
	else if(sstatus.equals("GRACED"))
		curBeam->SetBeamStatus(GRACED);
	else if(sstatus.equals("HIDDEN"))
		curBeam->SetBeamStatus(HIDDEN);
	else if(sstatus.equals("GHOSTED"))
		curBeam->SetBeamStatus(GHOSTED);
	else
		curBeam->SetBeamStatus(NORMAL);
}

void  XMLSymbolicHandler::endBeam()
{
	NumCode id=curBeam->GetNumericCode();
	if(curBeam->GetBeamPosition()==GruppoNote::BEAM_UP)
		curBeam->SetGamboUp();
	else if(curBeam->GetBeamPosition()==GruppoNote::BEAM_DWN)
		curBeam->SetGamboDwn();
	curMeasure->AddFig(curBeam, curLayer-1);
	curBeam->SetNumericCode(id);
	if(curMeasure->GetCodeCounterFig(curLayer-1)<=id)
		curMeasure->SetCodeCounterFig(curLayer-1,id+1);
	curBeam=NULL;
}

void  XMLSymbolicHandler::startChord(AttributeList& attrs)
{
	StrX id=attrs.getValue("ID");
	StrX stem=attrs.getValue("STEM");
	StrX size=attrs.getValue("SIZE");
	StrX acciaccatura=attrs.getValue("ACCIACCATURA");
	StrX link=attrs.getValue("LINK");
	StrX sstatus=attrs.getValue("STATUS");
	
	chordDur=new StrX(attrs.getValue("DURATION"));
	
	curFigure=curNote=curChord=new Accordo;
	curChord->SetNumericCode(atoi(id));
	curChord->SetCoda(TRUE);
	if(size.equals("SMALL"))
		curNote->SetNotina(TRUE);
	else if(size.equals("NORMAL"))
		curNote->SetNotina(FALSE);
	
	if(stem.equals("UP"))
		curChord->SetGamboUp();
	else if(stem.equals("DOWN"))
		curChord->SetGamboDwn();
	
	if(acciaccatura.equals("TRUE") || acciaccatura.equals("true"))
		curChordAcciaccatura=true;
	else
		curChordAcciaccatura=false;
	
	if(!link.equals(""))
		curChord->Set_Link((const char*)link);
	
	if(sstatus.equals("NORMAL"))
		curChord->SetStatus(NORMAL);
	else if(sstatus.equals("GRACED"))
		curChord->SetStatus(GRACED);
	else if(sstatus.equals("HIDDEN"))
		curChord->SetStatus(HIDDEN);
	else if(sstatus.equals("GHOSTED"))
		curChord->SetStatus(GHOSTED);
	else
		curChord->SetStatus(NORMAL);
}

void  XMLSymbolicHandler::endChord()
{
	if(curChord!=NULL)
	{
		curChord->SetCoda(TRUE);
		if(curBeam==NULL)
		{
			NumCode id=curChord->GetNumericCode();
			curMeasure->AddFig(curChord, curLayer-1);
			curChord->SetNumericCode(id);
			if(curMeasure->GetCodeCounterFig(curLayer-1)<=id)
				curMeasure->SetCodeCounterFig(curLayer-1,id+1);
		}
		else
		{
			curChord->SetPBattuta(curMeasure);
			curBeam->AddFigura(curChord);
		}
		if(chordDur!=NULL)
		{
			delete chordDur;
			chordDur=NULL;
		}
	}
	curFigure=curNote=curChord=NULL;
}

void  XMLSymbolicHandler::startChordNote(AttributeList& attrs)
{
	char empty[1]={0};
	StrX id=attrs.getValue("ID");
	StrX height=attrs.getValue("HEIGHT");
	StrX head=attrs.getValue("HEAD");
	StrX nstaff=attrs.getValue("STAFF");
	StrX ncolor=attrs.getValue("COLOR");
	
	if(chordDur->equals("D2"))
		curChordNote=new NBreve;
	else if(chordDur->equals("D1"))
		curChordNote=new NSemibreve;
	else if(chordDur->equals("D1_2"))
		curChordNote=new NMinima;
	else if(chordDur->equals("D1_4"))
		curChordNote=new NSemiminima;
	else if(chordDur->equals("D1_8"))
	{
		curChordNote=new NCroma;
		if(curChordAcciaccatura)
			((NCroma*)curChordNote)->SetAcciaccatura(TRUE);
	}
	else if(chordDur->equals("D1_16"))
		curChordNote=new NSemicroma;
	else if(chordDur->equals("D1_32"))
		curChordNote=new NBiscroma;
	else if(chordDur->equals("D1_64"))
		curChordNote=new NSemibiscroma;
	else if(chordDur->equals("D1_128"))
		curChordNote=new NFusa;
	else
		notImplemented("chordnote","DURATION",*chordDur);
	
	if(curChordNote!=NULL)
	{
		curChordNote->SetNumericCode(atoi(id));
		curChordNote->SetCoda(FALSE);
		NoteHead_ID nhead=parseNoteHeadType(head);
		curChordNote->SetHeadType(nhead);
		switch(nhead)
		{
		case HEAD_ALPHANUM:
		case HEAD_ALPHANUM_SQUARE:
		case HEAD_ALPHANUM_REVERSE:
			{
				StrX headCode=attrs.getValue("HEAD_ALPHANUM");
				curChordNote->SetHeadCode((unsigned char*)(const char*)headCode);
			}
			break;
		default:
			break;
		}
		curChordNote->SetAltezza(atoi(height));
		curChordNote->SetNStaff(atoi(nstaff));
	
		if(!ncolor.equals("")) curChordNote->SetColor(Colore(wxString((const char*)ncolor)));		

	}
}

void  XMLSymbolicHandler::endChordNote()
{
	if(curChordNote!=NULL)
	{
		curChordNote->SetPBattuta(curMeasure);
		curChord->AddNota(curChordNote);
	}
	curChordNote=NULL;
}

void  XMLSymbolicHandler::startChangeClef(AttributeList& attrs)
{
	StrX type=attrs.getValue("TYPE");
	StrX id=attrs.getValue("ID");
	StrX size=attrs.getValue("SIZE");
	StrX nstaff=attrs.getValue("STAFF");
	
	Chiave *clef=parseClefType(type);
	if(size.equals("NORMAL"))
		clef->SetChiavina(FALSE);
	else
		clef->SetChiavina(TRUE);
	curFigure=clef;
	curFigure->SetNumericCode(atoi(id));
	curFigure->SetNStaff(atoi(nstaff));
}

void XMLSymbolicHandler::endFigure()
{
	if(curFigure!=NULL)
	{
		if(curBeam==NULL)
		{
			NumCode id=curFigure->GetNumericCode();
			curMeasure->AddFig(curFigure, curLayer-1);
			curFigure->SetNumericCode(id);
			if(curMeasure->GetCodeCounterFig(curLayer-1)<=id)
				curMeasure->SetCodeCounterFig(curLayer-1,id+1);
		}
		else
		{
			curFigure->SetPBattuta(curMeasure);
			curBeam->AddFigura(curFigure);
		}
	}
	curFigure=NULL;
}

void  XMLSymbolicHandler::startChangeKeySignature(AttributeList& attrs)
{
	StrX type=attrs.getValue("TYPE");
	StrX id=attrs.getValue("ID");
	StrX nstaff=attrs.getValue("STAFF");
	
	tipoArmaturaChiave tpKs=parseKeySignatureType(type);
	ArmaturaChiave *ks=new ArmaturaChiave;
	ks->SetArmatura(tpKs);
	ks->SetVU(0);
	curFigure=ks;
	curFigure->SetNumericCode(atoi(id));
	curFigure->SetNStaff(atoi(nstaff));
}

void  XMLSymbolicHandler::startAnchorage(AttributeList& attrs)
{
	StrX id=attrs.getValue("ID");
	StrX size=attrs.getValue("SIZE");
	StrX nstaff=attrs.getValue("STAFF");
	
	Spazio *anc=new Spazio;
	
	anc->Setrelpos(atoi(size));
	curFigure=anc;
	curFigure->SetNumericCode(atoi(id));
	curFigure->SetNStaff(atoi(nstaff));
}

void  XMLSymbolicHandler::startRepetition(AttributeList& attrs)
{
	StrX type=attrs.getValue("TYPE");
	StrX id=attrs.getValue("ID");
	StrX nstaff=attrs.getValue("STAFF");
	
	if(type.equals("MEASURE"))
	{
		StrX measures=attrs.getValue("MEASURES");
		curFigure=new RipBattuta;
		if(atoi(measures)!=0)
		{
			NumGrande *pNG=new NumGrande;
			pNG->SetNumero(atoi(measures));
			
			curMeasure->SetPNumGrande(pNG);
		}
	}
	else if(type.equals("TWOMEASURE"))
		notImplemented("repetition","TYPE",type);
	else if(type.equals("BEAT"))
		curFigure=new RipTempo;
	else if(type.equals("TWOBEAT"))
		curFigure=new RipMezzaBattuta;
	else
		notImplemented("repetition","TYPE",type);
	curFigure->SetNumericCode(atoi(id));
	curFigure->SetNStaff(atoi(nstaff));
}

void  XMLSymbolicHandler::startHorizontal(AttributeList& attrs)
{
	StrX type=attrs.getValue("TYPE");
	StrX id=attrs.getValue("ID");
	StrX knot1=attrs.getValue("KNOT1");
	StrX knot2=attrs.getValue("KNOT2");
	StrX stems=attrs.getValue("BEAM.STEMS");
	StrX sstatus=attrs.getValue("BEAM.STATUS");
	StrX color=attrs.getValue("COLOR");
	StrX size=attrs.getValue("BEAM.SIZE");

	double kv1,kv2;
	bool custom=false;
	char empty[]={ 0 };
	
	if(!knot1.equals(empty) && !knot2.equals(empty))
	{
		sscanf(knot1,"%lf",&kv1);
		sscanf(knot2,"%lf",&kv2);
		custom=true;
	}
	
	curHoriz=NULL;
	
	if(type.equals("WAVE"))
		curHoriz=new Onda;
	else if(type.equals("ARROW"))
		curHoriz=new Freccia;
	else if(type.equals("SLUR"))
	{
		Legatura *leg=new Legatura;
		
		if(custom)
			leg->SetKnotValue(0.,kv1,0.,kv2);
		
		curHoriz=leg;
	}
	else if(type.equals("SLURDOT"))
	{
		Legatura *leg=new Legatura;
		leg->SetLineType(DOTTED_LINE);
		
		if(custom)
			leg->SetKnotValue(0.,kv1,0.,kv2);
		
		curHoriz=leg;
	}
	else if(type.equals("SLURDASH"))
	{
		Legatura *leg=new Legatura;
		leg->SetLineType(DASHED_LINE);
		
		if(custom)
			leg->SetKnotValue(0.,kv1,0.,kv2);
		
		curHoriz=leg;
	}
	else if(type.equals("TIE"))
		curHoriz=new LegaturaValore;
	else if(type.equals("OCTVA"))
		curHoriz=new ModifOttava;
	else if(type.equals("OCTBA"))
	{
		ModifOttava *modifOct=new ModifOttava;
		modifOct->SetTipo8(BA8);
		curHoriz=modifOct;
	}
	else if(type.equals("QUINDMA"))
	{
		ModifOttava *modifOct=new ModifOttava;
		modifOct->SetTipo8(MA15);
		curHoriz=modifOct;
	}
	else if(type.equals("QUINDBA"))
	{
		ModifOttava *modifOct=new ModifOttava;
		modifOct->SetTipo8(BA15);
		curHoriz=modifOct;
	}
	else if(type.equals("BEND"))
		curHoriz=new Bend;
	else if(type.equals("BENDDOT"))
	{
		Bend *bend=new Bend;
		bend->SetLineType(DOTTED_LINE);
		curHoriz=bend;
	}
	else if(type.equals("BENDDASH"))
	{
		Bend *bend=new Bend;
		bend->SetLineType(DASHED_LINE);
		curHoriz=bend;
	}
	else if(type.equals("DIMINUENDO"))
	{
		Forcella* dim=new Forcella;
		dim->SetCrescendo(FALSE);
		curHoriz=dim;
	}
	else if(type.equals("DIMDOT"))
	{
		Forcella* dim=new Forcella;
		dim->SetCrescendo(FALSE);
		dim->SetLineType(DOTTED_LINE);
		curHoriz=dim;
	}
	else if(type.equals("DIMDASH"))
	{
		Forcella* dim=new Forcella;
		dim->SetCrescendo(FALSE);
		dim->SetLineType(DASHED_LINE);
		curHoriz=dim;
	}
	else if(type.equals("CRESCENDO"))
	{
		Forcella* cre=new Forcella;
		cre->SetCrescendo(TRUE);
		curHoriz=cre;
	}
	else if(type.equals("CREDOT"))
	{
		Forcella* cre=new Forcella;
		cre->SetCrescendo(TRUE);
		cre->SetLineType(DOTTED_LINE);
		curHoriz=cre;
	}
	else if(type.equals("CREDASH"))
	{
		Forcella* cre=new Forcella;
		cre->SetCrescendo(TRUE);
		cre->SetLineType(DASHED_LINE);
		curHoriz=cre;
	}
	else if(type.equals("CHANGEREF"))
	{
		CambioRit *chref=new CambioRit;
		StrX refnumber=attrs.getValue("REFNUMBER");
		chref->SetText(((const char*)refnumber)[0]);
		curHoriz=chref;
	}
	else if(type.equals("TRILL"))
		curHoriz=new TrilloOnda;
	else if(type.equals("TUPLE"))
	{
		LegatQuadra *tuple=new LegatQuadra;
		
		StrX line=attrs.getValue("TUPLELINE");
		if(line.equals("FALSE") || line.equals("false"))
			tuple->SetSenzaLinea(TRUE);
		else
			tuple->SetSenzaLinea(FALSE);
		
		StrX num=attrs.getValue("TUPLENUMBER");
		tuple->SetNumero(atoi(num));
		
		curHoriz=tuple;
	}
	else if(type.equals("PIANOPEDAL"))
		curHoriz=new PedaleFinoA;
	else if(type.equals("BEAM"))
	{
		curHoriz=new Raggruppamento;	
		if(stems.equals("UP"))
			((Raggruppamento*)curHoriz)->SetGamboUp();
		else if(stems.equals("DOWN"))
			((Raggruppamento*)curHoriz)->SetGamboDwn();
		else if(stems.equals("STAFF12"))
			((Raggruppamento*)curHoriz)->SetBeamPosition(Raggruppamento::BEAM_12);
		else if(stems.equals("STAFF23"))
			((Raggruppamento*)curHoriz)->SetBeamPosition(Raggruppamento::BEAM_23);
		
		if(sstatus.equals("NORMAL"))
			((Raggruppamento*)curHoriz)->SetStatus(NORMAL);
		else if(sstatus.equals("GRACED"))
			((Raggruppamento*)curHoriz)->SetStatus(GRACED);
		else if(sstatus.equals("HIDDEN"))
			((Raggruppamento*)curHoriz)->SetStatus(HIDDEN);
		else if(sstatus.equals("GHOSTED"))
			((Raggruppamento*)curHoriz)->SetStatus(GHOSTED);
		else
			((Raggruppamento*)curHoriz)->SetStatus(NORMAL);
		
		if(size.equals("SMALL"))
			((Raggruppamento*)curHoriz)->SetNotina(true);
		else
			((Raggruppamento*)curHoriz)->SetNotina(false);
	}		
	else
		notImplemented("horizontal","TYPE",type);
	
	if(curHoriz!=NULL)
	{
		StrX updown=attrs.getValue("UPDOWN");
		if(updown.equals("UP"))
			curHoriz->SetSopra(TRUE);
		else if(updown.equals("DOWN"))
			curHoriz->SetSopra(FALSE);
		else
			notImplemented("horizontal","UPDOWN",updown);
		
		curHoriz->SetNumericCode(atoi(id));

		if(!color.equals("")) curHoriz->SetColor(Colore(wxString((const char*)color)));
	}
	
	curAddress=1;
	ReadDxDyAttr(attrs,curHoriz);
	ReadDxDyRAttr(attrs,curHoriz);
}

void  XMLSymbolicHandler::startAddress(AttributeList& attrs)
{
	StrX s_measure=attrs.getValue("MEASURE");
	StrX s_layer=attrs.getValue("LAYER");
	StrX s_figure=attrs.getValue("FIGURE");
	StrX s_chordOrBeam=attrs.getValue("CHORD.OR.BEAM");
	StrX s_chordInBeam=attrs.getValue("CHORD.IN.BEAM");
	Figura * fig=NULL;
	
	NumCode idMeasure=atoi(s_measure);
	int layer=atoi(s_layer)-1;
	NumCode figure=atoi(s_figure);
	NumCode chordOrBeam=atoi(s_chordOrBeam);
	NumCode chordInBeam=atoi(s_chordInBeam);
	
	Battuta *measure = curScore->GetBatt(idMeasure);
	if(measure != NULL)
	{
		Figura* fig1 = measure->GetFig(layer,figure);
		Figura* fig2=NULL;
		Figura* fig3=NULL;
		
		if(fig1 != NULL)
		{
			if(chordOrBeam == 0)
				fig = fig1;
			else
			{
				if(fig1->GetID() == CL_ACCORDO)
					fig2 = ((Accordo*)fig1)->GetFig(chordOrBeam);
				else
					fig2 = ((GruppoNote*)fig1)->GetFig(chordOrBeam);
				if(fig2 != NULL)
				{
					if(chordInBeam == 0)
						fig = fig2;
					else
					{
						fig3 = ((Accordo*)fig2)->GetFig(chordInBeam);
						if(fig3 != NULL)
							fig = fig3;
					}
				}
			} 
		}
	}
	if(fig!=NULL && curHoriz!=NULL)
	{
		if(curAddress==1)
			curHoriz->SetPFigStart(fig);
		else
			curHoriz->SetPFigEnd(fig);
		curHoriz->SetBattuta(curAddress,measure);
	}
	curAddress++;
}

void XMLSymbolicHandler::endHorizontal()
{
	if(curHoriz!=NULL)
	{
		NumCode nc=curHoriz->GetNumericCode();
		IntEsteso *horiz2 = curScore->GetNumCodeInt(nc);
		if (horiz2!=NULL)
			curScore->DeleteSym(horiz2);
		curScore->AddIntEsteso(curHoriz);
		curHoriz->SetNumericCode(nc);
		if(curScore->GetCodeCounterInt()<=nc)
			curScore->SetCodeCounterInt(nc+1);
	}
}

void  XMLSymbolicHandler::startAccidental(AttributeList& attrs)
{
	StrX type=attrs.getValue("TYPE");
	StrX roundbracket=attrs.getValue("ROUNDBRACKET");
	int parentesi=FALSE;
	
	Alterazione *acc=parseAccidentalType(type);
	if(acc!=NULL)
	{
		Nota *pNote;
		AlterazComposta *multiAcc=NULL;
		
		if(curChordNote!=NULL)
			pNote=curChordNote;
		else
			pNote=curNote;
		
		if((multiAcc=pNote->GetPAlterazComposta())==NULL)
		{
			multiAcc=new AlterazComposta;
			pNote->SetPAlterazComposta(multiAcc);
		}
		
		if(roundbracket.equals("YES"))
			parentesi=TRUE;
		else
			parentesi=FALSE;
		
		multiAcc->InserisciAlt(acc->GetID(),parentesi);
		ReadDxDyAttr(attrs,multiAcc->GetPAlterazione(acc->GetID()));
		delete acc;
	}
}

void  XMLSymbolicHandler::startAugmentation(AttributeList& attrs)
{
	StrX dots=attrs.getValue("DOTS");
	
	if(curChord==NULL)
	{
		PuntoValore *aug=new PuntoValore;
		aug->SetNumPunti(atoi(dots));
		curFigure->SetPPuntoValore(aug,FALSE);
		ReadDxDyAttr(attrs,aug);
	}
	else
		curChord->SetPuntiValore(atoi(dots),FALSE);
}

void  XMLSymbolicHandler::startUserSpacing(AttributeList& attrs)
{
	StrX s_spacemain=attrs.getValue("SPACEMAIN");
	StrX s_spacepart=attrs.getValue("SPACEPART");
	StrX s_spacepmain=attrs.getValue("SPACEPMAIN");
	StrX s_spaceppart=attrs.getValue("SPACEPPART");
	
	if(curFigure!=NULL)
	{
		curFigure->SetSpDirUt(atoi(s_spacemain),COMPUTER_VIEW);
		curFigure->SetSpOrcUt(atoi(s_spacepart),COMPUTER_VIEW);
		curFigure->SetSpDirUt(atoi(s_spacepmain),PRINT_VIEW);
		curFigure->SetSpOrcUt(atoi(s_spaceppart),PRINT_VIEW);
	}
	else
	{
		
		curMeasure->SetDirSp2FirstFigUt(curLayer-1,atoi(s_spacemain),COMPUTER_VIEW);
		curMeasure->SetOrcSp2FirstFigUt(curLayer-1,atoi(s_spacepart),COMPUTER_VIEW);
		curMeasure->SetDirSp2FirstFigUt(curLayer-1,atoi(s_spacepmain),PRINT_VIEW);
		curMeasure->SetOrcSp2FirstFigUt(curLayer-1,atoi(s_spaceppart),PRINT_VIEW);
		
	}
}

void  XMLSymbolicHandler::startOrnament(AttributeList& attrs)
{
	StrX type=attrs.getValue("TYPE");
	StrX updown=attrs.getValue("UPDOWN");
	StrX accsup=attrs.getValue("ACCSUP");
	StrX accinf=attrs.getValue("ACCINF");
	StrX number=attrs.getValue("NUMBER");
	Abbellimento *curOrnament=NULL;
	
	if(type.equals("TRILL"))
	{
		curOrnament=new Trillo;
	}
	else if(type.equals("TURN"))
	{
		curOrnament=new GInferiore;
	}
	else if(type.equals("TURNBACK"))
	{
		curOrnament=new GSuperiore;
	}
	else if(type.equals("TURNUP"))
	{
		curOrnament=new GUp;
	}
	else if(type.equals("TURNSLASH"))
	{
		curOrnament=new GSlash;
	}
	else if(type.equals("MORDENT"))
	{
		curOrnament=new MInferiore;
	}
	else if(type.equals("MORDENTSUP"))
	{
		curOrnament=new MSuperiore;
	}
	else if(type.equals("TREMOLO"))
	{
		curOrnament=new Tremolo;
		((Tremolo*)curOrnament)->SetNumBarre(atoi(number));
	}
	else if(type.equals("DMORDENT"))
	{
		curOrnament=new MInferiore;
		curOrnament->SetDoppio(TRUE);
	}
	else if(type.equals("DMORDENTSUP"))
	{
		curOrnament=new MSuperiore;
		curOrnament->SetDoppio(TRUE);
	}
	else if(type.equals("GLISSWAVE"))
	{
		curOrnament=new Glissato(TRUE);
	}
	else if(type.equals("GLISSLINE"))
	{
		curOrnament=new Glissato;
	}
	else if(type.equals("STEMTREMOLO"))
	{
		Suddivisione *pSuddivisione=new Suddivisione;
		pSuddivisione->SetNumTratti(atoi(number));
		curNote->SetPSuddivisione(pSuddivisione);
		return;
	}
	
	if(curOrnament!=NULL)
	{
		if(updown.equals("UP"))
			curOrnament->SetAboveNota(TRUE);
		else if(updown.equals("DOWN"))
			curOrnament->SetAboveNota(FALSE);
		else if(updown.equals("AUTO"))
			curOrnament->SetAboveNotaAuto(AUTOMATICO,curNote,curMeasure);
		
		Alterazione* accSup=parseAccidentalType(accsup);
		if(accSup!=NULL)
			curOrnament->SetPAltSopra(accSup);
		Alterazione* accInf=parseAccidentalType(accinf);
		if(accInf!=NULL)
			curOrnament->SetPAltSotto(accInf);
		
		curNote->SetPAbbellimento(curOrnament);
		ReadDxDyAttr(attrs,curOrnament);
	}
}

void  XMLSymbolicHandler::startKoreanOrnament(AttributeList& attrs)
{
	StrX type=attrs.getValue("TYPE");
	StrX updown=attrs.getValue("UPDOWN");
	KoreanOrnament *curKoreanOrnament=NULL;

	if(type.equals("SLIDINGUP")){
		curKoreanOrnament=new SlidingUp;
	}
	else if(type.equals("SLIDINGDOWN")){
		curKoreanOrnament=new SlidingDown;
	}
	else if(type.equals("SLIDINGDUP")){
		curKoreanOrnament=new SlidingDUp;
	}
	else if(type.equals("SLIDINGDOWNUP")){
		curKoreanOrnament=new SlidingDownUp;
	}
	else if(type.equals("NONGHYNINC")){
		curKoreanOrnament=new NonghynInc;
	}
	else if(type.equals("NONGHYNDEC")){
		curKoreanOrnament=new NonghynDec;
	}
	else if(type.equals("NONGHYNARROW")){
		curKoreanOrnament=new NonghynArrow;
	}
	else if(type.equals("NONGHYNWIDE")){
		curKoreanOrnament=new NonghynWide;
	}

	if(curKoreanOrnament!=NULL)
	{
		if(updown.equals("UP"))
			curKoreanOrnament->SetAboveNota(TRUE);
		else if(updown.equals("DOWN"))
			curKoreanOrnament->SetAboveNota(FALSE);
		else if(updown.equals("AUTO"))
			curKoreanOrnament->SetAboveNotaAuto(AUTOMATICO,curNote,curMeasure);


		curNote->SetPKoreanOrnament(curKoreanOrnament);
	}

	ReadDxDyAttr(attrs,curKoreanOrnament);
}

void  XMLSymbolicHandler::startArpeggio(AttributeList& attrs)
{
	StrX direction=attrs.getValue("DIRECTION");
	
	Arpeggio *pArpeggio=new Arpeggio;
	
	if(direction.equals("UP"))
		pArpeggio->SetDirezioneArpeggio(UPWARD);
	else if(direction.equals("DOWN"))
		pArpeggio->SetDirezioneArpeggio(DOWNWARD);
	else
		pArpeggio->SetDirezioneArpeggio(NO_DIRECTION);
	
	curChord->SetPArpeggio(pArpeggio);
	ReadDxDyAttr(attrs,pArpeggio);
}

void  XMLSymbolicHandler::startText(AttributeList& attrs)
{
	StrX updown=attrs.getValue("UPDOWN");
	StrX hidden=attrs.getValue("HIDDEN");
	curText=new TGenerico;
	if(updown.equals("UP"))
		curText->SetAboveNota(TRUE);
	else if(updown.equals("DOWN"))
		curText->SetAboveNota(FALSE);
	else
		notImplemented("text","UPDOWN",updown);
	
	if(hidden.equals("TRUE") || hidden.equals("true"))
		curText->SetHidden(TRUE);
	else
		curText->SetHidden(FALSE);
	ReadDxDyAttr(attrs,curText);
}

void  XMLSymbolicHandler::endText()
{
	curFigure->SetPTGenerico(curText);
	curText=NULL;
}

void  XMLSymbolicHandler::startAnnotation(AttributeList& attrs)
{
	if(curFigure)
	{
		StrX updown=attrs.getValue("UPDOWN");
		curAnnotation=new Annotazione;
		if(updown.equals("UP"))
			curAnnotation->SetAboveNota(TRUE);
		else if(updown.equals("DOWN"))
			curAnnotation->SetAboveNota(FALSE);
		else
			notImplemented("annotation","UPDOWN",updown);
		ReadDxDyAttr(attrs,curAnnotation);
	}
}

void  XMLSymbolicHandler::endAnnotation()
{
	if(curFigure)
	{
		curFigure->SetPAnnotazione(curAnnotation);
		curAnnotation=NULL;
	}
}

void  XMLSymbolicHandler::startFingering(AttributeList& attrs)
{
	StrX finger=attrs.getValue("FINGER");
	StrX updown=attrs.getValue("UPDOWN");
	StrX type=attrs.getValue("TYPE");
	
	Diteggiato *curFinger=new Diteggiato;
	curFinger->SetDito(atoi(finger));
	if(updown.equals("UP"))
		curFinger->SetAboveNota(TRUE);
	else if(updown.equals("DOWN"))
		curFinger->SetAboveNota(FALSE);
	else
		notImplemented("fingering","UPDOWN",updown);
	
	if(type.equals("CIRCLED1"))
		curFinger->SetTipoDito(1);
	else if(type.equals("CIRCLED2"))
		curFinger->SetTipoDito(2);
	else
		curFinger->SetTipoDito(0); //NORMAL
	
	if(curChordNote!=NULL)
		curChordNote->SetPDiteggiato(curFinger);
	else
		curNote->SetPDiteggiato(curFinger);
	ReadDxDyAttr(attrs,curFinger);
}

void  XMLSymbolicHandler::startStrings(AttributeList& attrs)
{
	StrX string=attrs.getValue("STRING");
	StrX updown=attrs.getValue("UPDOWN");
	StrX type=attrs.getValue("TYPE");
	StrX roman=attrs.getValue("ROMAN");
	
	Corda *curStrings=new Corda;
	
	if(updown.equals("UP"))
		curStrings->SetAboveNota(TRUE);
	else if(updown.equals("DOWN"))
		curStrings->SetAboveNota(FALSE);
	else
		notImplemented("strings","UPDOWN",updown);
	
	if(roman.equals("TRUE") || roman.equals("true"))
		curStrings->SetIsRoman(TRUE);
	else if(roman.equals("FALSE") || roman.equals("false"))
		curStrings->SetIsRoman(FALSE);
	else
		notImplemented("strings","ROMAN",roman);
	
	if(type.equals("NORMAL"))
		curStrings->SetTipoCorda(NOTCIRCLED);
	else if(type.equals("BRACKETS"))
		curStrings->SetTipoCorda(BRACKETS);
	else if(type.equals("CIRCLED1"))
		curStrings->SetTipoCorda(CIRCLED1);
	else if(type.equals("CIRCLED2"))
		curStrings->SetTipoCorda(CIRCLED2);
	else
		notImplemented("strings","TYPE",type);
	
	curStrings->SetNumCorda(atoi(string));
	ReadDxDyAttr(attrs,curStrings);
	
	Strumento *pStrumento=curNote->GetPStrumento();
	Arco *pArco=NULL;
	
	if(pStrumento==NULL)
	{
		pArco=new Arco;
		curNote->SetPStrumento(pArco);
		pArco->SetScrivereArco(FALSE);
	}
	else
	{
		if(pStrumento->GetID()==CL_ARCO)
			pArco=(Arco*)pStrumento;
	}
	if(pArco!=NULL)
		pArco->SetPCorda(curStrings);
}

void  XMLSymbolicHandler::startMute(AttributeList& attrs)
{
	StrX type=attrs.getValue("TYPE");
	StrX updown=attrs.getValue("UPDOWN");
	
	Sordina* pSordina=NULL;
	
	if(type.equals("PLUS"))
	{
		pSordina=new ConSord(CL_CONSORDINAOTTONI);
	}
	else if(type.equals("ON"))
	{
		pSordina=new ConSord(CL_CONSORDINAARCHI);
	}
	else if(type.equals("IN"))
	{
		pSordina=new ConSord(CL_CONSORDINATESTO);
	}
	else if(type.equals("MINUS"))
	{
		pSordina=new ViaSord(CL_VIASORDINAOTTONI);
	}
	else if(type.equals("OFF"))
	{
		pSordina=new ViaSord(CL_VIASORDINAARCHI);
	}
	else if(type.equals("VIA"))
	{
		pSordina=new ViaSord(CL_VIASORDINATESTO);
	}
	
	if(pSordina!=NULL)
	{
		if(updown.equals("UP"))
			pSordina->SetAboveNota(TRUE);
		else if(updown.equals("DOWN"))
			pSordina->SetAboveNota(FALSE);
		else
			notImplemented("mute","UPDOWN",updown);
		curNote->SetPSordina(pSordina);
		ReadDxDyAttr(attrs,(Indicazione*)pSordina);
	}
}

void  XMLSymbolicHandler::startHarmonic(AttributeList& attrs)
{
	StrX type=attrs.getValue("TYPE");
	StrX updown=attrs.getValue("UPDOWN");
	
	Armonici* pArmonici=NULL;
	
	if(type.equals("STRINGS"))
	{
		pArmonici=new Armonici(CL_ARMONICIARCHI);
	}
	else if(type.equals("FLUTES"))
	{
		pArmonici=new Armonici(CL_ARMONICIOTTONI);
	}
	
	if(pArmonici!=NULL)
	{
		if(updown.equals("UP"))
			pArmonici->SetAboveNota(TRUE);
		else if(updown.equals("DOWN"))
			pArmonici->SetAboveNota(FALSE);
		else
			notImplemented("harmonic","UPDOWN",updown);
		curNote->SetPArmonici(pArmonici);
		ReadDxDyAttr(attrs,pArmonici);
	}
}

void  XMLSymbolicHandler::startFretboard(AttributeList& attrs)
{
	StrX updown=attrs.getValue("UPDOWN");
	StrX number=attrs.getValue("NUMBER");
	StrX name=attrs.getValue("NAME");
	StrX frets=attrs.getValue("FRETS");
	StrX fingers=attrs.getValue("FINGERS");
	StrX barre=attrs.getValue("BARRE");
	StrX head=attrs.getValue("HEAD");
	
	DitaCorde *pDitaCorde=new DitaCorde(name,atoi(number),frets,barre,atoi(head),fingers);
	
	if(updown.equals("UP"))
		pDitaCorde->SetAboveNota(TRUE);
	else if(updown.equals("DOWN"))
		pDitaCorde->SetAboveNota(FALSE);
	else
		notImplemented("fretboard","UPDOWN",updown);
	
	curFigure->SetPDitaCorde(pDitaCorde);
	ReadDxDyAttr(attrs,pDitaCorde);
}

void  XMLSymbolicHandler::startFermata(AttributeList& attrs)
{
	StrX type=attrs.getValue("TYPE");
	StrX updown=attrs.getValue("UPDOWN");
	
	Corona *pCorona=new Corona;
	
	if(updown.equals("UP"))
		pCorona->SetAboveNota(TRUE);
	else if(updown.equals("DOWN"))
		pCorona->SetAboveNota(FALSE);
	else
		notImplemented("fermata","UPDOWN",updown);
	
	if(type.equals("SHORT"))
		pCorona->SetTipo(Triang);
	else if(type.equals("LONG"))
		pCorona->SetTipo(Quad);
	else
		pCorona->SetTipo(Classic);
	
	
	curFigure->SetPCorona(pCorona);
	ReadDxDyAttr(attrs,pCorona);
}

void  XMLSymbolicHandler::startDynamicText(AttributeList& attrs)
{
	StrX dynamic=attrs.getValue("DYNAMIC");
	StrX updown=attrs.getValue("UPDOWN");
	
	TDinamico *pTDinamico=new TDinamico;
	
	if(dynamic.equals("P"))
		pTDinamico->SetTipo(TD_P);
	else if(dynamic.equals("PP"))
		pTDinamico->SetTipo(TD_PP);
	else if(dynamic.equals("PPP"))
		pTDinamico->SetTipo(TD_PPP);
	else if(dynamic.equals("PPPP"))
		pTDinamico->SetTipo(TD_PPPP);
	else if(dynamic.equals("PPPPP"))
		pTDinamico->SetTipo(TD_PPPPP);
	else if(dynamic.equals("PPPPPP"))
		pTDinamico->SetTipo(TD_PPPPPP);
	else if(dynamic.equals("MP"))
		pTDinamico->SetTipo(TD_MP);
	else if(dynamic.equals("MF"))
		pTDinamico->SetTipo(TD_MF);
	else if(dynamic.equals("F"))
		pTDinamico->SetTipo(TD_F);
	else if(dynamic.equals("FF"))
		pTDinamico->SetTipo(TD_FF);
	else if(dynamic.equals("FFF"))
		pTDinamico->SetTipo(TD_FFF);
	else if(dynamic.equals("FFFF"))
		pTDinamico->SetTipo(TD_FFFF);
	else if(dynamic.equals("FFFFF"))
		pTDinamico->SetTipo(TD_FFFFF);
	else if(dynamic.equals("FFFFFF"))
		pTDinamico->SetTipo(TD_FFFFFF);
	else if(dynamic.equals("FFPP"))
		pTDinamico->SetTipo(TD_FFPP);
	else if(dynamic.equals("FFP"))
		pTDinamico->SetTipo(TD_FFP);
	else if(dynamic.equals("FPP"))
		pTDinamico->SetTipo(TD_FPP);
	else if(dynamic.equals("RFZ"))
		pTDinamico->SetTipo(TD_RFZ);
	else if(dynamic.equals("SFF"))
		pTDinamico->SetTipo(TD_SFF);
	else if(dynamic.equals("SFFF"))
		pTDinamico->SetTipo(TD_SFFF);
	else if(dynamic.equals("SFFP"))
		pTDinamico->SetTipo(TD_SFFP);
	else if(dynamic.equals("SFFZ"))
		pTDinamico->SetTipo(TD_SFFZ);
	else if(dynamic.equals("SFP"))
		pTDinamico->SetTipo(TD_SFP);
	else if(dynamic.equals("SFPP"))
		pTDinamico->SetTipo(TD_SFPP);
	else if(dynamic.equals("SP"))
		pTDinamico->SetTipo(TD_SP);
	else if(dynamic.equals("SF"))
		pTDinamico->SetTipo(TD_SF);
	else if(dynamic.equals("SFZ"))
		pTDinamico->SetTipo(TD_SFZ);
	else if(dynamic.equals("FZ"))
		pTDinamico->SetTipo(TD_FZ);
	else if(dynamic.equals("FP"))
		pTDinamico->SetTipo(TD_FP);
	else if(dynamic.equals("RF"))
		pTDinamico->SetTipo(TD_RF);
	else
		notImplemented("dynamictext","DYNAMIC",dynamic);
	
	if(updown.equals("UP"))
		pTDinamico->SetAboveNota(TRUE);
	else if(updown.equals("DOWN"))
		pTDinamico->SetAboveNota(FALSE);
	else
		notImplemented("dynamictext","UPDOWN",updown);
	
	curFigure->SetPTDinamico(pTDinamico);
	ReadDxDyAttr(attrs,pTDinamico);
}

void  XMLSymbolicHandler::startGlasses(AttributeList& attrs)
{
	StrX updown=attrs.getValue("UPDOWN");
	
	Occhiali *pOcchiali=new Occhiali;
	
	if(updown.equals("UP"))
		pOcchiali->SetAboveNota(TRUE);
	else if(updown.equals("DOWN"))
		pOcchiali->SetAboveNota(FALSE);
	else
		notImplemented("glasses","UPDOWN",updown);
	
	curFigure->SetPOcchiali(pOcchiali);
	ReadDxDyAttr(attrs,pOcchiali);
}

void  XMLSymbolicHandler::startBreath(AttributeList& attrs)
{
	StrX updown=attrs.getValue("UPDOWN");
	
	PrFiato *pPrFiato=new PrFiato;
	
	if(curFigure->GetID()==CL_SPAZIO)
	{
		((Spazio*)curFigure)->SetPPrFiato(pPrFiato);
		ReadDxDyAttr(attrs,pPrFiato);
	}
}

void  XMLSymbolicHandler::startPiano(AttributeList& attrs)
{
	StrX type=attrs.getValue("TYPE");
	StrX updown=attrs.getValue("UPDOWN");
	
	Pedal* pPedal=NULL;
	
	if(type.equals("PEDALUP"))
	{
		pPedal=new PedalUp;
	}
	else if(type.equals("PEDALDOWN"))
	{
		pPedal=new PedalDown;
	}
	else
		notImplemented("piano","TYPE",type);
	
	if(pPedal!=NULL)
	{
		if(updown.equals("UP"))
			pPedal->SetAboveNota(TRUE);
		else if(updown.equals("DOWN"))
			pPedal->SetAboveNota(FALSE);
		else
			notImplemented("piano","UPDOWN",updown);
		curFigure->SetPStrumento(pPedal);
		ReadDxDyAttr(attrs,pPedal);
	}
}

void  XMLSymbolicHandler::startDrum(AttributeList& attrs)
{
}

void XMLSymbolicHandler::endDrum()
{
}

void  XMLSymbolicHandler::startMarker(AttributeList& attrs)
{
	StrX type=attrs.getValue("TYPE");
	StrX updown=attrs.getValue("UPDOWN");

	ClassID espID=CL_DRAWOBJECT;
	ClassID symID=CL_DRAWOBJECT;

	DrawObject *pDrw=NULL;

	if(type.equals("PORTATO") || type.equals("TENUTO"))
		espID=CL_TENUTO;
	else if(type.equals("SFORZATO"))
		espID=CL_SFORZATO;
	else if(type.equals("ACCENTO") || type.equals("ACCENTOFORTE"))
		espID=CL_ACCENTO;
	else if(type.equals("PUNTOSOPRA") || type.equals("STACCATO"))
		espID=CL_STACCATO;
	else if(type.equals("MARTDOLCE"))
		espID=CL_MARTDOLCE;
	else if(type.equals("PUNTOALLUNGATO"))
		espID=CL_PUNTOALLUNGATO;
	else if(type.equals("MARTELLATO"))
		espID=CL_MARTELLATO;
	else if(type.equals("GENERIC"))
		espID=CL_ESPGENERIC;
	else if(type.equals("ARCO"))
		symID=CL_ARCO;
	else if(type.equals("PONTICELLO"))
		symID=CL_PONTICELLO;
	else if(type.equals("TASTIERA"))
		symID=CL_TASTIERA;
	else if(type.equals("PUNTA"))
		symID=CL_PUNTA;
	else if(type.equals("TALLONE"))
		symID=CL_TALLONE;
	else if(type.equals("BOWUP"))
		symID=CL_ARCOSU;
	else if(type.equals("BOWDOWN"))
		symID=CL_ARCOGIU;
	else if(type.equals("PIZZ"))
		symID=CL_PIZZICATO;
	else
		notImplemented("piano","TYPE",type);

	if(espID!=CL_DRAWOBJECT)
	{
		TipoInserimento upDownAuto=AUTOMATICO;
		if(updown.equals("UP"))
			upDownAuto=SOPRA;
		else if(updown.equals("DOWN"))
			upDownAuto=SOTTO;
		else if(updown.equals("AUTO"))
			upDownAuto=AUTOMATICO;
		else
			notImplemented("marker","UPDOWN",updown);

		EspressComposta *pEspComposta=curNote->GetPEspressComposta();
		if(pEspComposta==NULL)
		{
			pEspComposta=new EspressComposta;
			curNote->SetPEspressComposta(pEspComposta);
		}

		if(espID!=CL_ESPGENERIC)
			pEspComposta->InserisciEsp(espID,upDownAuto,curNote,curMeasure);
		else
		{
			StrX name=attrs.getValue("NAME");
			//char s[200];

			//sprintf(s,"\"%s\"",(char*)(const char*)name);
			int idSym=simboli.GetSymbolPos((char*)(const char*)name);
			pEspComposta->InserisciEsp(espID,upDownAuto,curNote,curMeasure,idSym);
			pDrw=pEspComposta->GetPEspressione(espID,idSym);
		}
	}
	if(symID!=CL_DRAWOBJECT)
	{
		TipoInserimento upDownAuto=AUTOMATICO;
		if(updown.equals("UP"))
			upDownAuto=SOPRA;
		else if(updown.equals("DOWN"))
			upDownAuto=SOTTO;
		else if(updown.equals("AUTO"))
			upDownAuto=AUTOMATICO;
		else
			notImplemented("marker","UPDOWN",updown);

		if(symID==CL_PIZZICATO)
		{
			Pizzicato *pPizz=new Pizzicato;
			pDrw=pPizz;
			switch(upDownAuto)
			{
			case SOPRA:
				pPizz->SetAboveNota(TRUE);
				break;
			case SOTTO:
				pPizz->SetAboveNota(FALSE);
				break;
			case AUTOMATICO:
				pPizz->SetAboveNotaAuto(symID,curNote,curMeasure);
				break;
			}

			curNote->SetPStrumento(pPizz);
		}
		else
		{
			Strumento *pStrumento=curNote->GetPStrumento();
			Arco *pArco=NULL;

			if(pStrumento==NULL)
			{
				pArco=new Arco;
				pDrw=pArco;
				curNote->SetPStrumento(pArco);
				pArco->SetScrivereArco(FALSE);
			}
			else
			{
				if(pStrumento->GetID()==CL_ARCO)
					pArco=(Arco*)pStrumento;
			}

			switch(symID)
			{
			case CL_ARCO:
				pArco->SetScrivereArco(TRUE);
				break;
			case CL_ARCOSU:
			case CL_ARCOGIU:
				{
					DirezArco *pDirArco=NULL;

					if(symID==CL_ARCOSU)
						pDirArco=new ArcoSu;
					else
						pDirArco=new ArcoGiu;
					pDrw=pDirArco;

					switch(upDownAuto)
					{
					case SOPRA:
						pDirArco->SetAboveNota(TRUE);
						break;
					case SOTTO:
						pDirArco->SetAboveNota(FALSE);
						break;
					case AUTOMATICO:
						pDirArco->SetAboveNotaAuto(symID,curNote,curMeasure);
						break;
					}

					pArco->SetPDirezArco(pDirArco);
				}
				break;
			case CL_PUNTA:
			case CL_TALLONE:
				{
					ParteArco *pParteArco=NULL;

					if(symID==CL_PUNTA)
						pParteArco=new Punta;
					else
						pParteArco=new Tallone;

					pDrw=pParteArco;
					switch(upDownAuto)
					{
					case SOPRA:
						pParteArco->SetAboveNota(TRUE);
						break;
					case SOTTO:
						pParteArco->SetAboveNota(FALSE);
						break;
					case AUTOMATICO:
						pParteArco->SetAboveNotaAuto(symID,curNote,curMeasure);
						break;
					}

					pArco->SetPParteArco(pParteArco);
				}
				break;
			case CL_PONTICELLO:
			case CL_TASTIERA:
				{
					PosizArco *pPosizArco=NULL;

					if(symID==CL_PONTICELLO)
						pPosizArco=new Ponticello;
					else
						pPosizArco=new Tastiera;
					pDrw=pPosizArco;

					switch(upDownAuto)
					{
					case SOPRA:
						pPosizArco->SetAboveNota(TRUE);
						break;
					case SOTTO:
						pPosizArco->SetAboveNota(FALSE);
						break;
					case AUTOMATICO:
						pPosizArco->SetAboveNotaAuto(symID,curNote,curMeasure);
						break;
					}

					pArco->SetPPosizArco(pPosizArco);
				}
				break;
			}
		}
	}
	if(pDrw!=NULL)
		ReadDxDyAttr(attrs,pDrw);
}

void  XMLSymbolicHandler::startBracket(AttributeList& attrs)
{
	StrX startpartid=attrs.getValue("STARTPARTID");
	StrX endpartid=attrs.getValue("ENDPARTID");
	StrX type=attrs.getValue("TYPE");
	StrX id=attrs.getValue("ID");
	
	if(type.equals("BRACKET"))
		mainScore->AddParentesiQuadra(atoi(startpartid),atoi(endpartid),atoi(id));
	else if(type.equals("BRACE"))
		mainScore->AddParentesiGraffa(atoi(startpartid),atoi(endpartid),atoi(id));
	else
		notImplemented("bracket","TYPE",type);
}


void  XMLSymbolicHandler::notImplemented(const char* element, const char* attribute, const char* value)
{
	char s[200];
	
	if(attribute==NULL)
		sprintf(s,"not implemented\nelement: %s",element);
	else
		sprintf(s,"not implemented\nelement: %s\nattribute: %s=%s",element,attribute,value);
	
	Message(s);
}

Bool Partitura::LoadPartXML(const char* file,SMRPartItem* swf)
{
	SAXParser symParser;
	SMRFile *smrf=swf ? swf->GetSMRFile() : NULL;
	XMLSymbolicHandler *symHandler=new XMLSymbolicHandler(this,smrf,false);
	XMLSymbolicEntityResolver * symEntRes=new XMLSymbolicEntityResolver;
	
	//symParser.setEntityResolver(symEntRes);
	extern wxDC *currDC;
	currDC=NULL;
	
    symParser.setValidationScheme(SAXParser::Val_Always);	//Rimettere
    symParser.setDoNamespaces(true);
	symParser.setDoSchema(true);
    symParser.setValidationSchemaFullChecking(true);
	wxString schemaPath=smrgetenv("SMR_ERR_PATH");
	schemaPath.Replace(" ","%20");

	//symParser.setExternalNoNamespaceSchemaLocation(schemaPath+"xsd\\smxf_part.xsd");
	symParser.setExternalSchemaLocation("urn:mpeg:mpeg-4:schema:smr:smxf-part:2007 "+schemaPath+"xsd\\smxf_part.xsd");

	symParser.setDocumentHandler(symHandler);
	symParser.setErrorHandler(symHandler);

	symParser.parse(file);
	
/*	if(symHandler->getErrCount()>0)	//Rimettere
	{
		wxMessageBox("Errors loading XML file:"+wxString(file),"Load XML");
	}*/ 
	
	symParser.setDocumentHandler(NULL);
	symParser.setErrorHandler(NULL);
	symParser.setEntityResolver(NULL);
	delete symHandler;
	delete symEntRes;
	
	return TRUE;
}

Bool Partitura::LoadMainScoreXML(const char* file,SMRMainScoreItem* swf)
{
	SAXParser symParser;
	SMRFile *smrf=swf ? swf->GetSMRFile() : NULL;
	XMLSymbolicHandler *symHandler=new XMLSymbolicHandler(this,smrf,true);
	XMLSymbolicEntityResolver * symEntRes=new XMLSymbolicEntityResolver;
	
	symParser.setDocumentHandler(symHandler);
	symParser.setErrorHandler(symHandler);
	//symParser.setEntityResolver(symEntRes);
	
    symParser.setValidationScheme(SAXParser::Val_Always);
    symParser.setDoNamespaces(true);
	symParser.setDoSchema(true);
    symParser.setValidationSchemaFullChecking(true);
	wxString schemaPath=smrgetenv("SMR_ERR_PATH");
	schemaPath.Replace(" ","%20");

	symParser.setExternalSchemaLocation("urn:mpeg:mpeg-4:schema:smr:smxf-mainscore:2007 "+schemaPath+"xsd\\smxf_main.xsd");

	symParser.parse(file);
	
/*	if(symHandler->getErrCount()>0)	//Rimettere
	{
		Message("Errors loading XML file","Load XML");
	}*/
	
	symParser.setDocumentHandler(NULL);
	symParser.setErrorHandler(NULL);
	symParser.setEntityResolver(NULL);
	delete symHandler;
	delete symEntRes;
	
	AlignScores();
	
	return TRUE;
}


bool Partitura::PasteFromClipboard(Spartito *refPsp, bool sopra)
{
	
	XMLPlatformUtils::Initialize();
	SAXParser symParser;
	
	XMLSymbolicHandler *symHandler=new XMLSymbolicHandler(this,NULL,false,true);
	XMLSymbolicEntityResolver * symEntRes=new XMLSymbolicEntityResolver;
	symHandler->setRefSpa(refPsp);
	symHandler->setPosUp(sopra);
	
	
	symParser.setDocumentHandler(symHandler);
	symParser.setErrorHandler(symHandler);
	//symParser.setEntityResolver(symEntRes);
	
	symParser.setDoValidation(false);
	
	symParser.parse(SMRFile::GetTmpPath()+PATH_SEP "clipboard.swf");
	
	if(symHandler->getErrCount()>0)
	{
		wxMessageBox("Errors loading Clipboard");
		return false;
	}
	
	symParser.setDocumentHandler(NULL);
	symParser.setErrorHandler(NULL);
	symParser.setEntityResolver(NULL);
	
	
	delete symHandler;
	delete symEntRes;
	
	return true;
}



/******** DOM ********/

void  XMLSymbolicHandler::startDom( const XMLCh* const name , AttributeList& attrs )
{
	DOM_Element elem=domDoc.createElement(name);
	
	for(int i=0; i<(int)attrs.getLength(); i++)
		elem.setAttribute(attrs.getName(i),attrs.getValue(i));
	
	parent.appendChild(elem);
	parent=elem;
}

void  XMLSymbolicHandler::domCharacters ( const XMLCh* const chars , const unsigned int length )
{
	parent.appendChild(domDoc.createTextNode(chars));
	parent.normalize();
	return;
}

void  XMLSymbolicHandler::endDom (const XMLCh* const name)
{
	parent=parent.getParentNode();
}


void XMLSymbolicHandler::startPrintPages(AttributeList& attrs)
{}

void XMLSymbolicHandler::startPage(AttributeList& attrs)
{
	StrX num=attrs.getValue("NUMBER");
	curPage=atoi(num);
	
	
}

void XMLSymbolicHandler::startTextBox(AttributeList& attrs)
{
	curTextBox=new CasellaTxt;
	wxFont font;
	
	StrX x=attrs.getValue("X");
	wxString xx(x);
	if(xx.Find('.')==-1)
		curTextBox->SetX(atoi(x)*0.8);
	else
		curTextBox->SetX(atof(x));
	
	StrX y=attrs.getValue("Y");
	wxString yy(y);
	if(yy.Find('.')==-1)
		curTextBox->SetY(atoi(y)*0.8);
	else
		curTextBox->SetY(atof(y));
	
	StrX align=attrs.getValue("ALIGN");
	if(align.equals("LEFT")) curTextBox->SetAllineamento(A_Left);
	if(align.equals("RIGHT")) curTextBox->SetAllineamento(A_Right);
	if(align.equals("CENTRE") || align.equals("CENTER")) curTextBox->SetAllineamento(A_Centre);	
	StrX fonttype=attrs.getValue("FONTTYPE");
	StrX fontsize=attrs.getValue("FONTSIZE");
	StrX fontstyle=attrs.getValue("FONTSTYLE");
	font.SetFaceName((const char *)fonttype);
	font.SetPointSize(atoi(fontsize));
	
	if(fontstyle.equals("NORMAL_NORMAL"))
	{
		font.SetStyle(wxNORMAL);
		font.SetWeight(wxNORMAL);
	}
	if(fontstyle.equals("NORMAL_LIGHT"))
	{
		font.SetStyle(wxNORMAL);
		font.SetWeight(wxLIGHT);
	}
	if(fontstyle.equals("NORMAL_BOLD"))
	{
		font.SetStyle(wxNORMAL);
		font.SetWeight(wxBOLD);
	}	
	if(fontstyle.equals("SLANT_NORMAL"))
	{
		font.SetStyle(wxSLANT);
		font.SetWeight(wxNORMAL);
	}
	if(fontstyle.equals("SLANT_LIGHT"))
	{
		font.SetStyle(wxSLANT);
		font.SetWeight(wxLIGHT);
	}	
	if(fontstyle.equals("SLANT_BOLD"))
	{
		font.SetStyle(wxSLANT);
		font.SetWeight(wxBOLD);
	}	
	if(fontstyle.equals("ITALIC_NORMAL"))
	{
		font.SetStyle(wxITALIC);
		font.SetWeight(wxNORMAL);
	}	
	if(fontstyle.equals("ITALIC_LIGHT"))
	{
		font.SetStyle(wxITALIC);
		font.SetWeight(wxLIGHT);
	}	
	if(fontstyle.equals("ITALIC_BOLD"))
	{
		font.SetStyle(wxITALIC);
		font.SetWeight(wxBOLD);
	}
	
	curTextBox->SetFont(font);
	
}

void XMLSymbolicHandler::endTextBox()
{
	if(loadingMainScore)
		mainScore->SetCurrentScore(0);
	else 
		mainScore->SetCurrentScore(mainScore->GetNumSpa());
	mainScore->AddCasella(curTextBox,curPage);
	curTextBox=NULL;
}

void XMLSymbolicHandler::startImageBox(AttributeList& attrs)
{
	CasellaImg *curImageBox=new CasellaImg;
	
	StrX x=attrs.getValue("X");
	wxString xx(x);
	if(xx.Find('.')==-1)
		curImageBox->SetX(atoi(x)*0.8);
	else
		curImageBox->SetX(atof(x));
	
	StrX y=attrs.getValue("Y");
	wxString yy(y);
	if(yy.Find('.')==-1)
		curImageBox->SetY(atoi(y)*0.8);
	else
		curImageBox->SetY(atof(y));
	
	StrX height=attrs.getValue("H");
	curImageBox->SetHeigh(atoi(height));
	StrX width=attrs.getValue("W");
	curImageBox->SetWidth(atoi(width));
	
	StrX imagefile=attrs.getValue("IMAGEFILE");
	curImageBox->SetFileImg((const char *)imagefile);
	if(loadingMainScore)
		mainScore->SetCurrentScore(0);
	else 
		mainScore->SetCurrentScore(mainScore->GetNumSpa());
	
	mainScore->AddCasella(curImageBox,curPage);
}

void XMLSymbolicHandler::startFormatPageComputerView(AttributeList& attrs)
{
	StrX top=attrs.getValue("TOPMARGIN");
	StrX bottom=attrs.getValue("BOTTOMMARGIN");
	StrX left=attrs.getValue("LEFTMARGIN");
	StrX right=attrs.getValue("RIGHTMARGIN");
	StrX dist=attrs.getValue("STAFFDISTANCE");
	StrX scale=attrs.getValue("SCALE");
	PrintParams *cv;
	
	if(loadingMainScore)
	{
		mainScore->SetCurrentScore(0);
		cv=mainScore->GetComputerViewParams();
	}
	else 
		cv=curScore->GetComputerViewParams();
	
	cv->SetTopMargin(1,atoi(top));
	cv->SetBottomMargin(1,atoi(bottom));
	cv->SetRightMargin(1,atoi(right));
	cv->SetLeftMargin(1,atoi(left));
	cv->SetStaffDistance(1,atoi(dist));
	float s=atof(scale);
	cv->SetScaleFactor( s>0.0 ? s : 1.0);
}

void XMLSymbolicHandler::startFormatPagePrintView(AttributeList& attrs)
{
	StrX pagef=attrs.getValue("PAGEFORMAT");
	StrX scale=attrs.getValue("SCALE");
	StrX res=attrs.getValue("RESOLUTION");
	StrX fing1=attrs.getValue("FING1");
	StrX fing2=attrs.getValue("FING2");
	StrX fing3=attrs.getValue("FING3");
	
	PrintParams *pv;
	
	if(loadingMainScore)
	{
		mainScore->SetCurrentScore(0);
		pv=mainScore->GetPrintViewParams();
	}
	else 
		pv=curScore->GetPrintViewParams();
	
	if(pagef.equals("A4"))
		pv->SetPageFormat(pfA4);
	else if(pagef.equals("A3"))
		pv->SetPageFormat(pfA3);
	else if(pagef.equals("A4R"))
		pv->SetPageFormat(pfA4R);
	else if(pagef.equals("A3R"))
		pv->SetPageFormat(pfA3R);
	else
		pv->SetPageFormat(pfA4);
	
	pv->SetScaleFactor(atof(scale));
	if(res.equals("300"))
		pv->SetPrinter(dpi300);
	else if(res.equals("600"))
		pv->SetPrinter(dpi600);
	else
		pv->SetPrinter(dpi300);
	
	pv->SetFingerPrint1((const char*)fing1);
	pv->SetFingerPrint2((const char*)fing2);
	pv->SetFingerPrint3((const char*)fing3);
}

void XMLSymbolicHandler::startFormatPage(AttributeList& attrs)
{
	StrX top=attrs.getValue("TOPMARGIN");
	StrX bottom=attrs.getValue("BOTTOMMARGIN");
	StrX left=attrs.getValue("LEFTMARGIN");
	StrX right=attrs.getValue("RIGHTMARGIN");
	StrX dist=attrs.getValue("STAFFDISTANCE");
	StrX nsys=attrs.getValue("NSYSTEMS");
	StrX nstaff=attrs.getValue("NSTAFFS");
	StrX insnpage=attrs.getValue("INSPAGENUM");
	StrX instit=attrs.getValue("INSTITLE");
	StrX page=attrs.getValue("PAGE");
	PrintParams *pv;
	int nPage;
	
	if(loadingMainScore)
	{
		mainScore->SetCurrentScore(0);
		pv=mainScore->GetPrintViewParams();
	}
	else 
		pv=curScore->GetPrintViewParams();
	
	nPage=atoi(page);
	
	pv->SetTopMargin(nPage,atoi(top));
	pv->SetBottomMargin(nPage,atoi(bottom));
	pv->SetRightMargin(nPage,atoi(right));
	pv->SetLeftMargin(nPage,atoi(left));
	pv->SetStaffDistance(nPage,atoi(dist));
	
	if(loadingMainScore)
		pv->SetNumSystems(nPage,atoi(nsys));
	else
		pv->SetNumStaff(nPage,atoi(nstaff));
	
	pv->SetInsPageNum(nPage,insnpage.equals("TRUE") || insnpage.equals("true"));
	pv->SetInsTitle(nPage,instit.equals("TRUE")  || instit.equals("true"));
}