/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include <xercesc/dom/deprecated/DOMParser.hpp>
#include <xercesc/dom/deprecated/DOM.hpp>

#include "wdfdomparser.hpp"
#include "wdfcomponentparser.hpp"

#include <wx/msgdlg.h>
#include <wx/progdlg.h>

#include "smr/smrmain.hpp"

char* smrgetenv(char*);

#ifdef __WXMSW__
#define PATH_SEP "\\"
#else
#define PATH_SEP "/"
#endif

#undef SetPrinter

static wxString getNodeValue(const DOM_Node& node)
{
	if(node.isNull())
		return "";
	return node.getNodeValue().transcode();
}

WDFDOMParser::WDFDOMParser()
{
	XMLFile=NULL;
	isource=NULL;
	
	try
	{
		XMLPlatformUtils::Initialize();
	}
	catch (const XMLException&)
	{
		return;
	}
	WDFParser=new DOMParser();
	WDFParser->setDoValidation(false);
	ErrorHandler *errReporter = new DOMTreeErrorReporter();
	WDFParser->setErrorHandler(errReporter);
}

WDFDOMParser::WDFDOMParser(char *XMLtoparse)
{
	XMLFile=new char[strlen(XMLtoparse)+1];
	strcpy(XMLFile, XMLtoparse);
	isource=NULL;
	try
	{
		XMLPlatformUtils::Initialize();
	}
	catch (const XMLException&)
	{
		return;
	}
	WDFParser=new DOMParser();
	WDFParser->setDoValidation(false);
	ErrorHandler *errReporter = new DOMTreeErrorReporter();
	WDFParser->setErrorHandler(errReporter);
}

WDFDOMParser::WDFDOMParser(InputSource *src)
{
	XMLFile=NULL;
	isource=src;
	
	try
   	{
       	XMLPlatformUtils::Initialize();
   	}
   	catch (const XMLException&)
   	{
       	return;
   	}
	WDFParser=new DOMParser();
	WDFParser->setDoValidation(false);
   	ErrorHandler *errReporter = new DOMTreeErrorReporter();
   	WDFParser->setErrorHandler(errReporter);
}

bool WDFDOMParser::Parse()
{
    bool errorsOccured = false;
    FILE *fp;
	if(XMLFile!=NULL)
	{
		if((fp=fopen(XMLFile, "r"))==NULL)
			return true;
		else
			fclose(fp);
	}
	
	if(validationSchema!="")
	{
		WDFParser->setValidationScheme(DOMParser::Val_Always);
		WDFParser->setDoNamespaces(true);
		WDFParser->setDoSchema(true);
		WDFParser->setValidationSchemaFullChecking(true);
		wxString schemaPath=smrgetenv("SMR_ERR_PATH");
		schemaPath.Replace(" ", "%20");

		//WDFParser->setExternalNoNamespaceSchemaLocation(schemaPath+"xsd\\"+validationSchema);
		WDFParser->setExternalSchemaLocation(validationSchema.BeforeFirst(' ')+" "+schemaPath+"xsd\\"+validationSchema.AfterFirst(' '));
	}

    try
    {
		if(XMLFile!=NULL)
			WDFParser->parse(XMLFile);
		else if(isource!=NULL)
			WDFParser->parse(*isource);
    }
    catch (const XMLException&)
    {
        errorsOccured = true;
    }
	
	return errorsOccured;
}


WDFDOMParser::~WDFDOMParser()
{
	delete WDFParser;
	if(XMLFile!=NULL)
		delete[] XMLFile;
	if(isource!=NULL)
		delete isource;
}

void WDFDOMParser::setXMLFile(char *XMLtoparse)
{
	if(XMLFile)
		delete[] XMLFile;
	XMLFile=new char[strlen(XMLtoparse)+1];
	strcpy(XMLFile, XMLtoparse);
}

WDFHeaderParser::WDFHeaderParser() : WDFDOMParser()
{
	nSymbolic=0;
	isClearHeader=false;
}

WDFHeaderParser::WDFHeaderParser(char *s, SMRFile *w) : WDFDOMParser()
{
	nSymbolic=0;
	smrf=w;
	Items=&w->GetItemsList();
	isClearHeader=false;
	
	isource=smrf->CreateFileInputSource(s);
}

bool WDFHeaderParser::ReadHeader()
{
	bool parseinit;

	//setValidationSchema("wdfheader.xsd");

	parseinit=Parse();
	if(parseinit)
	{
		return false;
	}
	DOM_Document doc = WDFParser->getDocument();
	DOM_Node wdfheader=doc.getFirstChild();
	//  Adesso sono sull'elemento wdfheader
	if(wdfheader.isNull())
		return false;
	DOM_Node child = wdfheader.getFirstChild();
	if(child==0) // Se c'e' il DOCTYPE lo si salta
	{
		wdfheader=wdfheader.getNextSibling();
		child = wdfheader.getFirstChild();
	}
	
	DOM_Node condition= wdfheader.getAttributes().getNamedItem("WDF_CONDITION");
	if(condition.getNodeValue().equals("HEADER"))
		isClearHeader=true;
	else
		isClearHeader=false;
		
    while( child != 0)
    {
		if((child.getNodeName()).equals("Symbolic"))
		{
			ParseSymbolic(child);
		}
		if((child.getNodeName()).equals("Symbolic_Milla"))
		{
			ParseSymbolicMilla(child);
		}
		if((child.getNodeName()).equals("Symbolic_Fonts"))
		{
			ParseSymbolicFonts(child);
		}
		if((child.getNodeName()).equals("Performance"))
		{
			ParsePerformance(child);
		}
		if((child.getNodeName()).equals("Lyric"))
		{
			ParseLyric(child);
		}
		if((child.getNodeName()).equals("Audio"))
		{
			ParseAudio(child);
		}
        child = child.getNextSibling();
	}
	return true;	
}

void WDFHeaderParser::ParseWDFItem(DOM_Node &start, SMRItem *item)
{
	wxString filename,wdfcid,textdescr;
	
	DOM_NamedNodeMap attr=start.getAttributes();
	DOM_Node hidden= attr.getNamedItem("HIDDEN");
	
	DOM_Node child = start.getFirstChild();
    while( child != 0)
    {
        if(!isClearHeader && child.getNodeName().equals("Filename"))
			filename=getNodeValue(child.getFirstChild());
		else if(child.getNodeName().equals("WDFCID"))
			wdfcid=getNodeValue(child.getFirstChild());
		else if(child.getNodeName().equals("Textual_description"))
			textdescr=getNodeValue(child.getFirstChild());
		
		child = child.getNextSibling();
	}
	
	item->SetFilename(filename);
	item->SetID(wdfcid);
	item->SetDescription(textdescr);
}

bool WDFHeaderParser::ParseSymbolic(DOM_Node &rootnode) 
{
	DOM_Node child = rootnode.getFirstChild();
    while( child != 0)
    {
        if(child.getNodeName().equals("Wdfitem") || child.getNodeName().equals("WdfitemRestricted"))
		{
			if(nSymbolic==0)
			{
				SMRItem *sym= new SMRMainScoreItem(smrf,"");
				ParseWDFItem(child, sym);
				Items->Append(sym);
			}
			else
			{
				SMRItem *sym= new SMRPartItem(smrf,"","");
				ParseWDFItem(child, sym);
				Items->Append(sym);
			}
			nSymbolic++;
		}
		child = child.getNextSibling();
	}
	return true;
}

bool WDFHeaderParser::ParseSymbolicMilla(DOM_Node &rootnode) 
{
	DOM_Node child = rootnode.getFirstChild();
    while( child != 0)
    {
        if(child.getNodeName().equals("Wdfitem") || child.getNodeName().equals("WdfitemRestricted"))
		{
			SMRItem *sym= new SMRSMFLItem(smrf);
			ParseWDFItem(child, sym);
			Items->Append(sym);
		}
		child = child.getNextSibling();
	}
	return true;
}

bool WDFHeaderParser::ParseSymbolicFonts(DOM_Node &rootnode) 
{
	DOM_Node child = rootnode.getFirstChild();
    while( child != 0)
    {
        if(child.getNodeName().equals("Wdfitem") || child.getNodeName().equals("WdfitemRestricted"))
		{
			SMRItem *sym= new SMRFontItem(smrf);
			ParseWDFItem(child, sym);
			Items->Append(sym);
		}
		child = child.getNextSibling();
	}
	return true;
}

bool WDFHeaderParser::ParsePerformance(DOM_Node &rootnode) 
{
	DOM_Node child = rootnode.getFirstChild();
    while( child != 0)
    {
        if(child.getNodeName().equals("Wdfitem") || child.getNodeName().equals("WdfitemRestricted"))
		{
			SMRItem *item= new SMRSynchInfoItem(smrf);
			ParseWDFItem(child, item);
			Items->Append(item);
		}
		child = child.getNextSibling();
	}
	return true;
}

bool WDFHeaderParser::ParseLyric(DOM_Node &rootnode) 
{
	DOM_Node child = rootnode.getFirstChild();
    while( child != 0)
    {
        if(child.getNodeName().equals("Wdfitem") || child.getNodeName().equals("WdfitemRestricted"))
		{
			SMRItem *item= new SMRLyricsItem(smrf, "");
			ParseWDFItem(child, item);
			Items->Append(item);
		}
		child = child.getNextSibling();
	}
	return true;
}

bool WDFHeaderParser::ParseAudio(DOM_Node &rootnode) 
{
	DOM_Node child = rootnode.getFirstChild();
    while( child != 0)
    {
        if(child.getNodeName().equals("Wdfitem") || child.getNodeName().equals("WdfitemRestricted"))
		{
			SMRItem *sym= new SMRAudioItem(smrf);
			ParseWDFItem(child, sym);
			Items->Append(sym);
		}
		child = child.getNextSibling();
	}
	return true;
}
