/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#ifndef _WDFDOMParser_
#define _WDFDOMParser_
#include "XMLCommonClasses.hpp"
#include <xercesc/dom/deprecated/DOMParser.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include "smr/smr.hpp"
#include "wx/date.h"

class SymWFExtra;

class WDFDOMParser
{
	protected:
		DOMParser *WDFParser;
		char *XMLFile;
		InputSource *isource;
		ErrorHandler *errReporter;
		wxString validationSchema;
	public:
		WDFDOMParser();
		WDFDOMParser(char *);
		WDFDOMParser(InputSource *);
		bool Parse();
		virtual ~WDFDOMParser();
		void setXMLFile(char *s);
		void setInputSource(InputSource *s) { isource=s; };
		void setValidationSchema(wxString s) { validationSchema=s; }

		DOMParser* GetWDFParser() { return WDFParser; };
};


class WDFHeaderParser : public WDFDOMParser
{
	protected:
		bool ParseSymbolic(DOM_Node &child);
		bool ParseSymbolicMilla(DOM_Node &child);
		bool ParseSymbolicFonts(DOM_Node &child);
		bool ParsePerformance(DOM_Node &child);
		bool ParseLyric(DOM_Node &child);
		bool ParseScore(DOM_Node &child);
		bool ParseAudio(DOM_Node &child);

		SMRFile *smrf;
		SMRItemList *Items;

		int nSymbolic;
		bool isClearHeader;

		void ParseWDFItem(DOM_Node &start, SMRItem *item);
	public:
		WDFHeaderParser();
		WDFHeaderParser(char *, SMRFile *);
		virtual ~WDFHeaderParser() {}
		bool ReadHeader();
		bool IsClearHeader() { return isClearHeader; };
};
#endif _WDFDOMParser_
