/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */


#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/dom/deprecated/DOMString.hpp>
#include <xercesc/util/bininputstream.hpp>

#include "XMLCommonClasses.hpp"

#include "drw\scorefrm.hpp"


#include <wx/wx.h>
#include <wx/progdlg.h>
#include <wx/wfstream.h>
#include <wx/file.h>

// Global streaming operator for DOMString is defined in DOMPrint.cpp
extern ostream& operator<<(ostream& target, const DOMString& s);


void DOMTreeErrorReporter::warning(const SAXParseException& exception)
{
	StrX msg=exception.getMessage();
	StrX filename=exception.getSystemId();
	wxLogMessage("WARNING %s(%d:%d): %s",  (const char*)filename, exception.getLineNumber(), exception.getColumnNumber() , (const char*)msg);
}

void DOMTreeErrorReporter::error(const SAXParseException& exception)
{
	StrX msg=exception.getMessage();
	StrX filename=exception.getSystemId();
	wxLogMessage("ERROR %s(%d:%d): %s",  (const char*)filename, exception.getLineNumber(), exception.getColumnNumber() , (const char*)msg);
}

void DOMTreeErrorReporter::fatalError(const SAXParseException& exception)
{
	StrX msg=exception.getMessage();
	StrX filename=exception.getSystemId();
	wxLogError("FATALERROR %s(%d:%d): %s",  (const char*)filename, exception.getLineNumber(), exception.getColumnNumber() , (const char*)msg);
}

void DOMTreeErrorReporter::resetErrors()
{
    // No-op in this case
}



class StringFormatTarget : public XMLFormatTarget
{
public:
	wxString str;
	
	StringFormatTarget() {};
    void writeChars
    (
          const XMLByte* const      toWrite
        , const unsigned int        count
        ,       XMLFormatter* const formatter
    )
	{
		str+=toWrite;
	}
    
	/*void writeChars( const   XMLByte* const  toWrite)
	{
		str+=toWrite;
	};*/
};

// ---------------------------------------------------------------------------
//  StrX: Private helper methods
// ---------------------------------------------------------------------------
void StrX::transcode(const XMLCh* const toTranscode,
					 XMLFormatter::EscapeFlags escapeFlags,
					 XMLFormatter::UnRepFlags unrepFlags )
{
    // Short circuit if its a null pointer or empty string
    if (!toTranscode || (!toTranscode[0]))
    {
        fLocalForm = new char[1];
        fLocalForm[0] = 0;
        return;
	}
	
	StringFormatTarget target;
	
	XMLFormatter formatter(fEncoding,&target,escapeFlags,unrepFlags);
	
	formatter << toTranscode;
	
	fLocalForm=new char[strlen(target.str.c_str())+1];
	strcpy(fLocalForm,target.str.c_str());
}


bool SaveToFile(InputSource* source, const wxString& file)
{
	XMLByte buffer[40*1024];
	unsigned int read;
	FILE *fp=fopen(file,"wb");
	if(fp==NULL)
		return false;
	
	BinInputStream *s=source->makeStream();
	
	do
	{
		read=s->readBytes(buffer,40*1024);
		fwrite(buffer,1,read,fp);
	}
	while(read==40*1024);
	
	fclose(fp);
	delete s;
	
	return true;
}

bool SaveToFile(wxInputStream* source, const wxString& file, bool quiet)
{
	XMLByte buffer[40*1024];
	unsigned int read;
	FILE *fp=fopen(file,"wb");
	if(fp==NULL)
		return false;
	if(source==NULL)
		return false;
	
	int count=source->GetSize()/(40*1024)+1;
	int v=0;
	
	wxProgressDialog *cnt=NULL;
	if(count>5 && !quiet)
		cnt=new wxProgressDialog("Component extraction","Extracting...",count);
	
	do
	{
		source->Read(buffer,40*1024);
		read=source->LastRead();
		
		fwrite(buffer,1,read,fp);
		if(cnt!=NULL && ++v<=count)
			cnt->Update(v);
	}
	while(read==40*1024);
	
	fclose(fp);
	
	if(cnt!=NULL)
		delete cnt;
	return true;
}

bool SaveToFile(wxInputStream* source, const wxString& file, wxMEScoreFrame* scoreFrame)
{
	XMLByte buffer[40*1024];
	unsigned int read;
	FILE *fp=fopen(file,"wb");
	if(fp==NULL)
		return false;
	if(source==NULL)
		return false;
	
	int count=source->GetSize()/(40*1024)+1;
	int v=0;
	
	if(count>5 && scoreFrame!=NULL)
		scoreFrame->CreateProgressSlide("Component extraction",count);
	
	do
	{
		source->Read(buffer,40*1024);
		read=source->LastRead();
		fwrite(buffer,1,read,fp);
		if(++v<=count && scoreFrame!=NULL)
			scoreFrame->SetProgressSlide(v);
	}
	while(read==40*1024);
	fclose(fp);
	if(scoreFrame!=NULL)
		scoreFrame->DeleteProgressSlide();
	return true;
}

#define BUF_SIZE 100*1024

off_t wxZlibInputStreamEx::OnSysSeek(off_t pos, wxSeekMode mode)
{
	
	return wxInvalidOffset;
	
}
