/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#ifndef _XMLCOMMONCLASSES_
#define _XMLCOMMONCLASSES_

#include <xercesc/sax/ErrorHandler.hpp>
#include <xercesc/sax/inputsource.hpp>
#include <xercesc/framework/XMLFormatter.hpp>
#ifdef WX226
#include <iostream.h>
#else
#include <iostream>
using std::ostream;
#endif

//#include <wx/wfstream.h>
//#include <wx/zipstrm.h>
#include <wx/zstream.h>
#include <wx/string.h>

class wxMEScoreFrame;

XERCES_CPP_NAMESPACE_USE

class DOMTreeErrorReporter : public ErrorHandler
{
public:
    // -----------------------------------------------------------------------
    //  Constructors and Destructor
    // -----------------------------------------------------------------------
    DOMTreeErrorReporter()
    {
    }

    ~DOMTreeErrorReporter()
    {
    } 

    // -----------------------------------------------------------------------
    //  Implementation of the error handler interface
    // -----------------------------------------------------------------------
    void warning(const SAXParseException& toCatch);
    void error(const SAXParseException& toCatch);
    void fatalError(const SAXParseException& toCatch);
    void resetErrors();
};

// ---------------------------------------------------------------------------
//  This is a simple class that lets us do easy (though not terribly efficient)
//  trancoding of XMLCh data to local code page for display.
// ---------------------------------------------------------------------------

class StrX
{
public :
    // -----------------------------------------------------------------------
    //  Constructors and Destructor
    // -----------------------------------------------------------------------
	StrX(const XMLCh* const toTranscode,
		 const char* encoding = "UTF-8",
		 XMLFormatter::EscapeFlags escapeFlags = XMLFormatter::NoEscapes,
         XMLFormatter::UnRepFlags unrepFlags = XMLFormatter::UnRep_Fail ) : fLocalForm(0), fEncoding(encoding)
    {
        // Call the private transcoding method
        transcode(toTranscode,escapeFlags,unrepFlags);
    }

	StrX(const XMLCh* const toTranscode,
		 int length,
		 const char* encoding = "UTF-8",
		 XMLFormatter::EscapeFlags escapeFlags = XMLFormatter::NoEscapes,
         XMLFormatter::UnRepFlags unrepFlags = XMLFormatter::UnRep_Fail ) : fLocalForm(0), fEncoding(encoding)
    {
		XMLCh* tmp=new XMLCh[length+1];
		memcpy(tmp,toTranscode,length*sizeof(XMLCh)+1);
		tmp[length]=0;
        transcode(tmp,escapeFlags,unrepFlags);
		delete tmp;
    }

    ~StrX()
    {
        delete [] fLocalForm;
    }


    // -----------------------------------------------------------------------
    //  Getter methods
    // -----------------------------------------------------------------------
    const char* localForm() const
    {
        return fLocalForm;
    }

	operator const char*() const
	{
		return fLocalForm;
	}

	bool equals(const char* str) const
	{
		return strcmp(fLocalForm,str)==0;
	}

private :
    // -----------------------------------------------------------------------
    //  Private helper methods
    // -----------------------------------------------------------------------
	void transcode (const XMLCh* const toTranscode,
		 XMLFormatter::EscapeFlags escapeFlags = XMLFormatter::NoEscapes,
         XMLFormatter::UnRepFlags unrepFlags = XMLFormatter::UnRep_Fail );

    // -----------------------------------------------------------------------
    //  Private data members
    //
    //  fLocalForm
    //      This is the local code page form of the string.
    // -----------------------------------------------------------------------
    char*   fLocalForm;
	const char*	fEncoding;
};

inline ostream& operator<<(ostream& target, const StrX& toDump)
{
    target << toDump.localForm();
    return target;
}

bool SaveToFile(InputSource* source, const wxString& file);
bool SaveToFile(wxInputStream* source, const wxString& file, bool quiet=false);
bool SaveToFile(wxInputStream* source, const wxString& file, wxMEScoreFrame* scoreFrame);
int AddWMOFile(const wxString& wmo,const wxString& file, const wxString& path);
int ExistWMOFile(const char* wmo,const char* file);

const unsigned int ZSTREAM_BUFFER_SIZE = 1024;

class wxZlibInputStreamEx : public wxZlibInputStream
{
public:
	wxZlibInputStreamEx(wxInputStream& stream) : wxZlibInputStream(stream) {};
protected:
	off_t OnSysSeek(off_t pos, wxSeekMode mode);
	off_t OnSysTell() const
	{
		return 0;
	}
};

#endif _XMLCOMMONCLASSES_
