/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2008.

***********************************************************************/

/* ISO_HEADER_END */

//////////////////////////////////////////////////////////////////////////////
//
// File:  MusicScoreProxy.cpp
//
// Author: Pierfrancesco Bellini, pbellini@dsi.unifi.it
//
// Modified:
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the MusicScore node.
//
//
///////////////////////////////////////////////////////////////////////////


#include "Transform2DProxy.h"
#include "defs.h"
#include "Nodes-CSELT.h"
#include "MusicScoreProxy.h"
#include "ScoreShapeProxy.h"	
#include "Stream.h"
#include "trace.h"
#include "../SMRDecoder/SMRDecoder.h"

void MusicScore::OnUpdate()
{
	float so=scoreOffset;
	so=so+0;
};

void MusicScore::OnmousePosition()
{
}

void MusicScore::OnexecuteCommand()
{
	SMRDecoder* smrDecoder=((SMRDecoder*)GetDecoder());
	if(smrDecoder!=NULL && (Bool)executeCommand)
	{
		Vec3f position=mousePosition;
		if((Bool)hyperlinkEnable)
		{
			//activatedLink.Remove();
			char* link=smrDecoder->Link(position.x,position.y);
			if(link!=NULL)
				activatedLink=ZString(link);
		}
		else
		{
			LPCSTR cmd=commandOnExecute;
			const char **argv=NULL;
			int argc=argumentsOnExecute.GetLength();

			if(argc>0)
			{
				argv=new const char*[argc];
				for(int i=0; i<argc; i++)
					argv[i]=argumentsOnExecute[i];
			}

			smrDecoder->DoCommand((char*)cmd,position.x,position.y,argc,argv);
			delete[] argv;
		}
	}
};

void MusicScore::OngotoLabel()
{
	ZString label=gotoLabel;

	if(label!="")
		((SMRDecoder*)GetDecoder())->JumpToLabel(label);
};

void MusicScore::OngotoMeasure()
{
	int measure=gotoMeasure;

	if(measure>0)
		((SMRDecoder*)GetDecoder())->JumpToMeasure(measure);
};

void MusicScore::OnhighlightTimePosition()
{
	float timePosition=highlightTimePosition;

	if(timePosition>0)
		((SMRDecoder*)GetDecoder())->JumpToTime(timePosition*1000);
};

// =====================================================================

//
// Presenter::InitMovieTexture
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

 MusicScoreProxy * Presenter::InitMusicScore(MusicScore* pObj)
 {
	return new MusicScoreProxy(pObj);
 }


// =====================================================================

//
// MusicScoreProxy::MusicScoreProxy
//
// Class constructor.
//

MusicScoreProxy::MusicScoreProxy(MediaObject* pMO) : DrawableImage(pMO)
{
	m_firstTimeAtStartTime = TRUE;
	m_firstTimeAtStopTime  = TRUE;
	m_endOfTimeReached = FALSE;
	m_startTime = 0;
	m_timesStarted = 0;
	m_OpenStreamIsBeingUsed=FALSE;
	tempTime = 0;
	currentTime = 0;
}

MusicScoreProxy::~MusicScoreProxy()
{
}	
	

void MusicScoreProxy::DeleteObject()
{
//?	GetPresenter()->RemoveOpenStream(this);
}


// =====================================================================

//
// MovieTextureProxy::PreRender
//
//

void MusicScoreProxy::PreRender(Effects & effects,DrawableContext * pContext)
{
	if (!pContext->IsSwitchedOn())
	{
        return;
   }
	
	currentTime = (double)((Presenter*)GetPresenter())->GetTime()/1000; 
	if (m_firstTimeAtStartTime)
	{
			
		MusicScore* pMusicScore = GetMusicScore();
		
		Vec2f size = pMusicScore->size;
		if(pMusicScore->GetDecoder())
		{
			((SMRDecoder*)pMusicScore->GetDecoder())->SetSize(size.x,size.y);
			((SMRDecoder*)pMusicScore->GetDecoder())->ShowParts(pMusicScore->partsShown.GetArray().GetBuffer(),
									  pMusicScore->partsShown.GetLength());
		}

		m_firstTimeAtStartTime = FALSE;
		m_firstTimeAtStopTime = TRUE;
		//pMusicScore->isActive = TRUE;
		m_timesStarted++;


		if (m_timesStarted > 1)
		{

			//? m_startTime = (double)pMovieTexture->startTime;
			pMusicScore->OpenStream();
			TRACE_PRINT1("MusicScore: Starting at %g\n", m_startTime);
			return;                       
		}
			// we also need to start the stream
		pMusicScore->OpenStream();
		//GetPresenter()->AddToOpenStreams(this);
        TRACE_PRINT1("MusicScore: Starting at %g\n", m_startTime);

	}

	DrawableImage::PreRender(effects,pContext);
	if(GetMusicScore()->GetDecoder())
	{
		MusicScore* pMusicScore = GetMusicScore();
		SMRDecoder *smrDecoder=(SMRDecoder*)pMusicScore->GetDecoder();
		
		if(smrDecoder->IsReady())
		{
					
		/*-------------------------------------------------------------
		startTime / stopTime fields need to be offset by current time on first access
		-------------------------------------------------------------*/
			GetMusicScore() -> startTime.ApplyOffset (pContext ->GetScoreShapeProxy()->GetTimeContext());
			GetMusicScore() -> stopTime.ApplyOffset (pContext ->GetScoreShapeProxy () ->GetTimeContext ());
			int nMeasures=smrDecoder->GetMeasureCount();
			if(pMusicScore->numMeasures != nMeasures)
				pMusicScore->numMeasures=nMeasures;

			if(pMusicScore->availableCommands.GetLength()==0)
			{
				char** commands=smrDecoder->GetAvailableCommands();
				if(commands!=NULL)
				{
					while (*commands!=NULL)
					{
						pMusicScore->availableCommands.Append(*commands);
						commands++;
					}
				}
			}
			
			if(pMusicScore->availableViewTypes.GetLength()==0)
			{
				char** views=smrDecoder->GetAvailableViewTypes();
				if(views!=NULL)
				{
					while (*views!=NULL)
					{
						pMusicScore->availableViewTypes.Append(*views);
						views++;
					}
				}
			}

			if(pMusicScore->partNames.GetLength()==0)
			{
				int n=smrDecoder->GetPartCount();
				for(int i=0; i<n; i++)
				{
					char* name=smrDecoder->GetPartName(i);
					pMusicScore->partNames.Append(name);
				}
			}
			
			if(pMusicScore->availableLabels.GetLength()==0)
			{
				int n=smrDecoder->GetLabelCount();
				for(int i=0; i<n; i++)
				{
					char* name=smrDecoder->GetLabelName(i);
					pMusicScore->availableLabels.Append(name);
				}
			}

			if(pMusicScore->availableLyricLanguages.GetLength()==0)
			{
				char** lyrics=smrDecoder->GetAvailableLyrics();
				if(lyrics!=NULL)
				{
					while (*lyrics!=NULL)
					{
						pMusicScore->availableLyricLanguages.Append(*lyrics);
						lyrics++;
					}
				}
			}

			if(pMusicScore->partsShown.WasFieldUpdated())
			{
				smrDecoder->ShowParts(pMusicScore->partsShown.GetArray().GetBuffer(),
									  pMusicScore->partsShown.GetLength());
				pMusicScore->partsShown.ResetFieldUpdate();
			}

			if(pMusicScore->partsLyrics.WasFieldUpdated())
			{
				smrDecoder->SetPartLyrics(pMusicScore->partsLyrics.GetArray().GetBuffer(),
											pMusicScore->partsLyrics.GetLength());
				pMusicScore->partsLyrics.ResetFieldUpdate();
			}

			int firstVisMeasure=smrDecoder->GetFirstVisibleMeasure();
			if(firstVisMeasure!=pMusicScore->firstVisibleMeasure)
				pMusicScore->firstVisibleMeasure=firstVisMeasure;

			int lastVisMeasure=smrDecoder->GetLastVisibleMeasure();
			if(lastVisMeasure!=pMusicScore->lastVisibleMeasure)
				pMusicScore->lastVisibleMeasure=lastVisMeasure;
				
			if(pMusicScore->startTime.GetValue()!=-1 && (currentTime)>=pMusicScore->startTime.GetValue() && (pMusicScore->stopTime.GetValue() ==-1 || (currentTime)<=pMusicScore->stopTime.GetValue()))
			{
				int x=0,y=0;
				double startTime=pMusicScore->startTime.GetValue();
				float speed=pMusicScore->speed.GetValue();
				bool reachedEnd=false;
				smrDecoder->JumpToTime((((currentTime-tempTime)-startTime) * speed * 1000), x, y, reachedEnd);
				Vec2f size = pMusicScore->size;
				pMusicScore->highlightPosition=Vec3f(x + effects.transform.m_data[0][2]-size.x/2, size.y/2 - y + effects.transform.m_data[1][2],0);

				if(reachedEnd)
				{
					if((Bool)GetMusicScore()->loop)
					{
						tempTime = currentTime-pMusicScore->startTime.GetValue();				
						smrDecoder->JumpToTime(1);
					}
				}
			}
			else
			{
				int x=0, y=0;
				smrDecoder->GetDefaultHighlightPos(x,y);
				Vec2f size = pMusicScore->size;
				pMusicScore->highlightPosition=Vec3f(x + effects.transform.m_data[0][2]-size.x/2,size.y/2 - y + effects.transform.m_data[1][2],0);
				tempTime=0;
			}


		}
	}

/*		if(pContext->m_hasStreamFinished==TRUE)
		{
	
			if((Bool)GetMusicScore()->loop)
			{
				//GetMusicScore()->CloseStream();
				//GetMusicScore()->OpenStream();
				tempTime = currentTime;				
			}
			pContext->m_hasStreamFinished=FALSE;
		} */
		m_OpenStreamIsBeingUsed=TRUE;
		
} /* MovieTextureProxy::PreRender */

void MusicScoreProxy::OnEndOfStream()
{
	// Call the base class's method
	DrawableImage::OnEndOfStream();

	// Set flags and events which are required at the end of a stream.
	m_firstTimeAtStopTime = FALSE;
	m_firstTimeAtStartTime = TRUE;
	m_endOfTimeReached = TRUE;
//?	GetMusicScore()->isActive = FALSE;
} /* MovieTextureProxy::OnEndOfStream */



// =====================================================================
