/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2008.

***********************************************************************/

//
// File:  ScoreShapeProxy.cpp
//
// Authors: Pierfrancesco Bellini, pbellini@dsi.unifi.it
//
// Modified:
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the ScoreShape node.
//
// Created: February, 2006.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "ScoreShapeProxy.h"
#include "GeometryNode.h"
#include "trace.h"


// =====================================================================

//
// Presenter::InitShape
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

ScoreShapeProxy* Presenter::InitScoreShape(ScoreShape* pObj)
{
		return new  ScoreShapeProxy(pObj);
}


// =====================================================================

//
// ScoreShapeProxy::PreRender
//
// Add the transform parameters of this node to the given effects, and
// then apply the PreRender method to all of the children of this node.
//

void ScoreShapeProxy::PreRender(Effects & effects)
{
	GeometryNode* pGeometryProxy;
	
	MediaObject* pMediaObject=GetGeometry();
	
	if(pMediaObject)
	{
		pGeometryProxy=(GeometryNode*)(pMediaObject)->GetRenderingObject();
		//effects.m_pAppearance=(Appearance*)(MediaObject*) GetShape()->appearance;
		effects.m_pScoreShapeProxy=this;
		effects.m_pScore=GetScore();
		if (pGeometryProxy) pGeometryProxy->PreRender(effects);
	}

} /* ShapeProxy::PreRender */
