/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2008.

***********************************************************************/
//
// File:  ScoreShapeProxy.h
//
// Authors: Pierfrancesco Bellini, pbellini@dsi.unifi.it
//
// Modified:
//
// Description:
//   The definition of the proxy object used by the presenter
//   to render the ScoreShape node.
//
// Created: February, 2006.
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef SCORESHAPE_PROXY_H
#define SCORESHAPE_PROXY_H


#include "Traversable.h"


class ScoreShapeProxy : public Traversable
{
public:
	ScoreShapeProxy(MediaObject* pMO) : Traversable(pMO) {}

	// Return a pointer to the ScoreShape node that this node is the proxy for
	ScoreShape* GetScoreShape() const { return (ScoreShape*)GetObject(); }

	// Return a pointer to the Appearance object belonging to the
	// Shape node that this node is the proxy for.
	MusicScore* GetScore() const
	{ return (MusicScore*)(MediaObject*)(GetScoreShape()->score); }

	// Return a pointer to the geometry object belonging to the
	// Shape node that this node is the proxy for.
	MediaObject* GetGeometry() const
	{ return (MediaObject*)(GetScoreShape()->geometry); }

	// Functions inherited from the base class
	void PreRender(Effects & effects);
	BOOL IsFinished() const { return TRUE; }
};


#endif // SHAPE_PROXY_H
