/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       PCMStream.cpp
AUTHOR:     Bellini
OVERVIEW:   Declaration of PCMStream, the class that handles PCM streams
-------------------------------------------------------------------------*/
#include "defs.h"
#include "PCMStream.h"
#include "odMux.h"

/*-----------------------------------------------------------------
G723Stream constructor reads first character
-----------------------------------------------------------------*/
PCMStream :: PCMStream (FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor)
{
	m_cNext = getc ();
	m_lTime = 0;

	// The stream data specifies the data rate of the audio, either
	// 5.3 or 6.3 Kbits/sec.  The data rate determines the frame size.
	m_nFrameSize = 2646; // 30ms @ 44100 Hz 16bit
	if (pDescriptor -> streamData != (LPCSTR) NULL &&
		pDescriptor -> streamData [0] != 0
	)
	{
		double dRate = atof (pDescriptor -> streamData);
		if (dRate == 6.3)
			m_nFrameSize = 24;
		else if (dRate != 5.3)
			throw ExceptionInvalidData (
				"G723 data rate must be either 5.3 or 6.3.\n"
			);
	}
}

/*-----------------------------------------------------------------
Time stamps are increments of 30 ms
-----------------------------------------------------------------*/
long PCMStream :: GetDTS (long)
{
	if (m_cNext == EOF)
		return -1;
	m_lDTS = m_lTime;
	m_lTime += 30;
	m_nCount = 0;
	return m_lDTS;
}

/*-------------------------------------------------------------
Although every G723 unit is RAP, this needs to be changed in the broadcast
case so that only AUs which have CTS will be RAP
-------------------------------------------------------------*/
bool PCMStream :: IsRAP ()
{
	return true;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one char ahead.  The number
of characters in an AU is determined by the data rate, which is
given in the szStreamData field.
-----------------------------------------------------------------*/
int PCMStream :: GetChar ()
{
	if (m_nCount >= m_nFrameSize)
		return -1;
	++m_nCount;
	int c = m_cNext;
	m_cNext = getc ();
//	assert (c != EOF);
	return c;
}

static StreamFactory<PCMStream> pcmFactory ("PCM");