// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
	// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "MovieTextureProxy.h"
#include "opengl/base/OpenGLPresenter.h"
#include "opengl/base/OpenGLVisualRenderer.h"

#include "Stream.h"
//#include "Executiv.h"
#include "Presenter.h"

#ifdef _SAIT_3DTG_MP4VIDEO_
#include "../../2DPlayer-CSELT/clipping.h"
#include <fstream.h>

FILE *Log;

void ConvertYUVtripleToRGB(int y, int u, int v, int& r, int& g, int& b)
{
  int tempY = y;
  tempY <<= 16;

  r = Clip((tempY              +  91881 * v - 11760828) >> 16);
  g = Clip((tempY -  22554 * u -  46802 * v +  8877496) >> 16);
  b = Clip((tempY + 116130 * u              - 14864613) >> 16);
}

void ConvertYUV2RGB(int nWidth, int nHeight, unsigned char* pYUVFrame, unsigned char* pRGBFrame)
{
	int x = 0;
	int y = 0;
	unsigned char* pYFrame = pYUVFrame;
	unsigned char* pUFrame = pYUVFrame + nWidth * nHeight;
	unsigned char* pVFrame = pYUVFrame + nWidth * nHeight * 5 / 4;

	for(y = 0; y < nHeight; y+=2)
	{
		for(x = 0; x < nWidth; x+=2)
		{
			int Y1 = pYFrame[nWidth*y+x];
			int Y2 = pYFrame[nWidth*y+x+1];
			int Y3 = pYFrame[nWidth*(y+1)+x];
			int Y4 = pYFrame[nWidth*(y+1)+x+1];

			int U = pUFrame[nWidth*y/4+x/2]; //Width of U,Y = nWidth /2
			int V = pVFrame[nWidth*y/4+x/2];

			int red = 0;
			int green = 0;
			int blue = 0;

			ConvertYUVtripleToRGB(Y1, U, V, red, green, blue);
			pRGBFrame[y*nWidth*3+x*3] = red;
			pRGBFrame[y*nWidth*3+x*3+1] = green;
			pRGBFrame[y*nWidth*3+x*3+2] =  blue;

			ConvertYUVtripleToRGB(Y2, U, V, red, green, blue);
			pRGBFrame[y*nWidth*3+x*3+3] = red;
			pRGBFrame[y*nWidth*3+x*3+4] =  green;
			pRGBFrame[y*nWidth*3+x*3+5] =  blue;

			ConvertYUVtripleToRGB(Y3, U, V, red, green, blue);
			pRGBFrame[(y+1)*nWidth*3+x*3] = red;
			pRGBFrame[(y+1)*nWidth*3+x*3+1] =  green;
			pRGBFrame[(y+1)*nWidth*3+x*3+2] =  blue;

			ConvertYUVtripleToRGB(Y4, U, V, red, green, blue);
			pRGBFrame[(y+1)*nWidth*3+x*3+3] = red;
			pRGBFrame[(y+1)*nWidth*3+x*3+4] =  green;
			pRGBFrame[(y+1)*nWidth*3+x*3+5] =  blue;
		}
	}
}

void ConvertYUV2RGB2(int nWidth, int nHeight, unsigned char* pYUVFrame, unsigned char* pRGBFrame)
{
	int x = 0;
	int y = 0;
	unsigned char* pYFrame = pYUVFrame;
	unsigned char* pUFrame = pYUVFrame + nWidth * nHeight;
	unsigned char* pVFrame = pYUVFrame + nWidth * nHeight * 5 / 4;
	unsigned char* pRFrame = pRGBFrame;
	unsigned char* pGFrame = pRGBFrame + nWidth * nHeight;
	unsigned char* pBFrame = pRGBFrame + nWidth * nHeight * 2;

	for(y = 0; y < nHeight; y+=2)
	{
		for(x = 0; x < nWidth; x+=2)
		{
			int Y1 = pYFrame[nWidth*y+x];
			int Y2 = pYFrame[nWidth*y+x+1];
			int Y3 = pYFrame[nWidth*(y+1)+x];
			int Y4 = pYFrame[nWidth*(y+1)+x+1];

			int U = pUFrame[nWidth*y/4+x/2];
			int V = pVFrame[nWidth*y/4+x/2];

			int red = 0;
			int green = 0;
			int blue = 0;

			ConvertYUVtripleToRGB(Y1, U, V, red, green, blue);
			pRFrame[y*nWidth+x] = red;
			pGFrame[y*nWidth+x] = green;
			pBFrame[y*nWidth+x] = blue;

			ConvertYUVtripleToRGB(Y2, U, V, red, green, blue);
			pRFrame[y*nWidth+x+1] = red;
			pGFrame[y*nWidth+x+1] = green;
			pBFrame[y*nWidth+x+1] = blue;

			ConvertYUVtripleToRGB(Y3, U, V, red, green, blue);
			pRFrame[(y+1)*nWidth+x] = red;
			pGFrame[(y+1)*nWidth+x] = green;
			pBFrame[(y+1)*nWidth+x] = blue;

			ConvertYUVtripleToRGB(Y4, U, V, red, green, blue);
			pRFrame[(y+1)*nWidth+x+1] = red;
			pGFrame[(y+1)*nWidth+x+1] = green;
			pBFrame[(y+1)*nWidth+x+1] = blue;
		}
	}
}
#endif _SAIT_3DTG_MP4VIDEO_

// Render object description

//MovieTextureProxy::MovieTextureProxy(MovieTexture* pMovieTexture):TextureNodeProxy(pMovieTexture)
//{
//}


/* virtual */ void
MovieTextureProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  RenderDescriptionBox(pRenderer, "MovieTexture");
}

/* virtual */ BOOL
MovieTextureProxy::Enable(
    OpenGLVisualRenderer * pRenderer, 
    DWORD /* dwTime */)
{
  MovieTexture* pTexture = (MovieTexture*) GetMediaObject();
  MediaStream* pStream = pTexture->GetStream();

  if (pStream)
  {
	TestActive();
	if (GetActive())
	{


		  int nSize;
		  DWORD dwTime;
		  BOOL bEOU;
		  BOOL bWait;

		  long currentTime;
		  long SleepTime = 0;
		  int OffsetTime = -460;  //-700; -360 represent the delay between the time renderer gets the frame and 
		  //the time the frame becomes visible


       unsigned char *  pData=NULL ;
		// return value
		BOOL bRet = false;

		pData = (unsigned char *)pStream->FetchNext(
			nSize, dwTime, bEOU, bWait);


		if((pData == END_OF_STREAM) || (pData== NULL))
			 return TextureNodeProxy::Enable(pRenderer);


		// header information
		m_nNumberOfFrame = ((int*)pData)[0];

		
		m_nWidth = ((int*)pData)[1];
		m_nHeight = ((int*)pData)[2];
		m_nNumberOfChannels = ((int*)pData)[3];
		m_nBytePerPixel = ((int*)pData)[4];

		fprintf(Log,"Size %d x %d TimeStamp %d\n",m_nWidth,m_nHeight,dwTime);
		fflush(Log);

		// frame information
		m_nRGBFrameSize = m_nWidth * m_nHeight * 3;
		m_nYUVFrameSize = m_nWidth * m_nHeight * 3 / 2;

		// video buffer
		m_pVideo = NULL;
		

		// video buffer
		m_pVideo = (unsigned char*)realloc((void*)m_pVideo, m_nYUVFrameSize);
		if(!m_pVideo)
		{
			fprintf(Log,"!m_pVideo\n");
			fflush(Log);

			pStream->Release(TRUE);
			return false;
		}


		memcpy(m_pVideo ,
				pData+sizeof(int)*5 ,m_nYUVFrameSize);


		// convert yuv to rgb
		unsigned char* pRGBFrame = (unsigned char*)malloc(m_nRGBFrameSize);
		ConvertYUV2RGB(m_nWidth, m_nHeight, m_pVideo , pRGBFrame);


		currentTime  = GetMediaObject()->GetRoot()->GetTime();
		if(InitTimeStamp==0)
			InitTimeStamp = currentTime;


		SleepTime =  dwTime  - (currentTime  - InitTimeStamp) + OffsetTime;


		fprintf(Log,"New frame :%d  Cur %d  Jitter %d TimeStamp %d\n",m_nNumberOfFrame,currentTime  - InitTimeStamp,SleepTime,dwTime);
		if(SleepTime>0)
		{
			  Sleep(SleepTime);
				currentTime  = GetMediaObject()->GetRoot()->GetTime();
				SleepTime = dwTime  - (currentTime  - InitTimeStamp) ;

				fprintf(Log,"Compensated Jitter %d \n",SleepTime);

		}
		else// if(SleepTime< -4) // Skip
		{
			pStream->Release(TRUE);
			SkipFrame++;
			fprintf(Log,"Skip frame %d\n",SkipFrame);
			fflush(Log);


			free(pRGBFrame);

			pStream->Release(TRUE);
			return TRUE;

		}


		// enable
		fprintf(Log,"Rendering frame %d\n",m_nNumberOfFrame);
		fflush(Log);
		bRet = TextureNodeProxy::Enable(pRenderer,
			pTexture->GetStreamFormat() == /* FORMAT_RGBA */ MediaFormat::YUV ? RGB : RGBA,
			m_nWidth, m_nHeight, pRGBFrame);
		// free
		free(pRGBFrame);

		pStream->Release(TRUE);

        return bRet;

    }

    return TextureNodeProxy::Enable(pRenderer);
  }
  
  return TRUE;
}


// Stefano Battista
// moved from MovieTexture to Proxy

// Test whether movie should be activated or deactivated

void MovieTextureProxy::TestActive()
{
	MovieTexture * poMovieTexture = GetMovieTexture();
	Time currentTime = poMovieTexture->GetRoot()->GetTime() / 1000.0;
//  Time currentTime = GetMediaObject()->GetRoot()->GetTime() ;


    if (((Time) poMovieTexture->startTime <= currentTime) && 
		((poMovieTexture->stopTime < poMovieTexture->startTime)|| (poMovieTexture->stopTime > currentTime )) )
	{
//		if(!(Bool)poMovieTexture->isActive) //First active
			Start();
	}
	else if ( ((Time) poMovieTexture->startTime > currentTime) || 
			(((Time) poMovieTexture->stopTime <= currentTime) && (poMovieTexture->stopTime > poMovieTexture->startTime)) )
	{
		if((Bool)poMovieTexture->isActive) //First active
			Stop();

	}

}


// Start movie

void MovieTextureProxy::Start()
{
#ifndef BIFSENC
  MovieTexture * poMovieTexture = GetMovieTexture();
  poMovieTexture->isActive = TRUE;
  printf("Start is called for Video\n");
  if (poMovieTexture->GetStream())
  {
    OpenGLPresenter * poPresenter;
    poPresenter = (OpenGLPresenter *) poMovieTexture->GetExecutive()->GetPresenter();
    poPresenter->GetVisualRenderer()->RegisterStreamConsumer(poMovieTexture);

	//JLF 04/00
	poMovieTexture->GetExecutive()->RequestChannels();
	TRACE("[MovieTextureProxy::Enable] StreamConsumer started \n");
  
  }
#endif

}


// Stop movie

void MovieTextureProxy::Stop()
{
#ifndef BIFSENC
  MovieTexture * poMovieTexture = GetMovieTexture();
  poMovieTexture->isActive = FALSE;
  printf("Stop is called for Video\n");

 /* {
    OpenGLPresenter * poPresenter;
    poPresenter = (OpenGLPresenter *) poMovieTexture->GetExecutive()->GetPresenter();
    poPresenter->GetVisualRenderer()->RegisterStreamConsumer(poMovieTexture);
  }*/
	InitTimeStamp=0;
	fprintf(Log,"MovieTextureProxy::Stop is called \n");
	fflush(Log);

  poMovieTexture->CloseStream();
  // PauseStream();
#endif
}

