/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#include "SMRDecoder.h"
#include "SMRDecoderLib.h"

#include "../core/ODImp.h"			// MPEGJ MERGE - RLS
#include "../core/Inline.h"			// MPEGJ MERGE - RLS
#include "../mpegj-player/MPDecoderInterface.h"	// MPEGJ MERGE - RLS
#include "../core/stream.h"

#define HEADER_WDF "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<wdfheader WDF_CONDITION=\"FULL\">\n<Protection PROTECTION_STATUS=\"\" ENCRYPTED=\"\"></Protection>\n<Symbolic>\n"

void *CreateInstance ()
{
	return (void *) new SMRDecoder;
}

BOOL SMRDecoder :: SetFormat (Capabilities &formatOptions,DecoderParams &decoderParams)
{
	decoderParams.m_nDBSize = -1;		//In questa maniera si setta come dinamico
	decoderParams.m_nOptimalAUSize = 0; 
	decoderParams.m_nCBSize = -1;		//In questa maniera si setta come dinamico
	decoderParams.m_bRequiresFullAUs = TRUE;

	if (formatOptions.GetFormatCapabilityLength () == 0) 
		return FALSE; 
	decoderParams.m_nStreamNumber = 0; 
	decoderParams.m_format = formatOptions.GetFormatCapability (0); 
	return TRUE;
}

int SMRDecoder :: GetOptimalOutputSize ()
{
	return 0; //GetAttributes(FRAME_WIDTH,NULL)*GetAttributes(FRAME_HEIGHT,NULL)*3;	//In questo modo si istanzia il buffer di uscita in maniera dinamica
}

int AUCount=0;

BOOL SMRDecoder :: Decode (LPBYTE pInput, int nInputLength,	LPBYTE pOutput,	int &rnOutputLength)
{
	if(!m_isPCMData)
	{
		int optimalOut = GetAttributes(FRAME_WIDTH,NULL)*GetAttributes(FRAME_HEIGHT,NULL)*3;
		if (rnOutputLength<optimalOut)
		{
			rnOutputLength=optimalOut;
			return FALSE;
		}
		else
		{
			if (rnOutputLength < nInputLength)
			{	
				rnOutputLength = nInputLength;
				return FALSE;
			}
			
			m_smrDecoder=new SMRInternalDecoder(m_smrDecoderInfo, m_smrDecoderInfoSize, (char*)pInput, nInputLength,
				0,0,GetAttributes(FRAME_WIDTH,NULL),GetAttributes(FRAME_HEIGHT,NULL));
			m_smrDecoder->ShowParts(m_parts,m_nparts);
			m_smrDecoder->Decode((LPVOID)pOutput,GetAttributes(FRAME_WIDTH,NULL),GetAttributes(FRAME_HEIGHT,NULL));
			//	GetOutputStream () -> Dispatch (nOutSize, dwTime);
			m_ready=TRUE;
			return TRUE;
		}
	}
	else
	{
		if (rnOutputLength < nInputLength)
		{
			rnOutputLength = nInputLength;
			return FALSE;
		}
		
		memcpy (pOutput, pInput, nInputLength);
		AUCount++;
		return TRUE;
	}
}

BOOL SMRDecoder::Setup (
		const BYTE *pSpecificInfo,
		int nSpecificInfoLength,
		int	nESNumber
)
{
	m_smrDecoderInfoSize = nSpecificInfoLength;
	m_smrDecoderInfo = new char[nSpecificInfoLength];
	for(int i=0; i<nSpecificInfoLength; i++)
	{
		m_smrDecoderInfo[i] = pSpecificInfo[i];
	}
	if(m_smrDecoderInfo[0]==0x02 && m_smrDecoderInfo[1]==0x10)
		m_isPCMData=true;
	return TRUE;
}


void SMRDecoder :: Run ()
{
/*	if(m_isPCMData)
	{
		DecoderImp::Run();
		return;
	}
*/	LPBYTE pInData, pOutData;
	int nInSize, nOutSize;
	DWORD dwTime, prevTime;
	BOOL bEOU;

	SetThreadPriority (GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL);

	#ifdef MPGJ
	// MPEGJ MERGE_3 - RLS
	// to get the decoder OD ID to throw the event on to the corresponding decoder on java side - PA
    InlineImp* pInline;
	Executive *myExecutive;
	Decoder* dec;
	myExecutive = GetExecutive();
	pInline = myExecutive->GetInline();
	int len = pInline->m_streamAssociation.GetLength();
	//*** YT START (118)
	int dec_found = 0;

	// associate the decoder with ESID also, not only OD
	for( int i=0; i < len; i++ ) {
		for(int j = pInline->m_streamAssociation[i].m_pOD->esDescr.GetLength()-1;j>=0;j--) {
			dec =  ((ES_DescriptorImp *)(pInline->m_streamAssociation[i].m_pOD->esDescr[j]))->GetDecoder();
			if( this == dec ) {
				ESID = pInline->m_streamAssociation[i].m_pOD->esDescr[j]->ES_ID;
				ODID = pInline->m_streamAssociation[i].m_pOD->ObjectDescriptorID;
				dec_found = 1;
				break;
			}
		}
		if( dec_found )
		{
			break;
		}
	}
	//*** YT END (118)

     /**** get the instance of the function pointer table. - PA ****/
    di = MPJGetTerminalDispatchTable();
    // set the status variable
   	if( di != NULL ) {
    	//STREAM_START
	    // go till end of list and add the new event there
	    decoderQueue* temp;
		temp = &(di->dEQ);
		if( *temp == NULL )
			//*** YT START (119)
			// (*temp) = new decoderEventQueue( 4, ODID);
			(*temp) = new decoderEventQueue( 4, ODID, ESID);
			//*** YT END (119)
		else {
			while ( (*temp)->next != NULL )
				*temp = (*temp)->next;
			//*** YT START (119)
			// (*temp) = new decoderEventQueue( 4, ODID);
			(*temp)->next = new decoderEventQueue( 4, ODID, ESID);
			//*** YT END (119)
		}

	    if( di->isStartedDispatching == 1) {
		   // signal the thread to throw the event
// SOLARIS PORTING
#ifdef WIN32
		   	count = ResumeThread(di->hThrow);
#else
			count = thr_continue( *(thread_t *)(di->hThrow) );
#endif
		}
	}
   	/****** end of thread resume code ****/
	//END MPEGJ MERGE_3 - RLS
	#endif

	prevTime = 0;
	while (IsActive()) {
		pInData = (LPBYTE) m_pInStream -> FetchNext (nInSize, dwTime, bEOU, TRUE);
		assert(pInData);
		if (pInData == END_OF_STREAM) break;

		if (IsActive()) {
			nOutSize = GetOptimalOutputSize ();


			pOutData = (LPBYTE) m_pOutStream -> Allocate (nOutSize, TRUE, FALSE);
			if (pOutData == NULL) {
				throw;
			}
			/*output has been closed while we were waiting for data, just return*/			
			if (pOutData == END_OF_STREAM) {
				return;
			}
			while (stop == 1 || !Decode (pInData, nInSize, pOutData, nOutSize))
			{	pOutData = (LPBYTE) m_pOutStream -> Reallocate (nOutSize);

				if (pOutData == NULL) {
					throw;
				}
				if (pOutData == END_OF_STREAM)
					return;
// WIN - SOL Release - Stop the CPU consumption
#ifdef WIN32
				if(stop == 1)
					Sleep( 100 );
#endif
			}
			/*if we have the same time we're in a seek phase (cf file reader), don't dispatch
			note this shall be modified for scalable codecs
			*/
			if (dwTime==prevTime) 
				nOutSize = 0;
			/*also try to drop frames depending on clock speed / renderer framerate so that we don't fill 
			the composition buffer for nothing*/
			else if (prevTime && CanSkipOutput()) {
				if (m_pInStream->CompareTime(dwTime, prevTime) <= 0) {
					nOutSize = 0;
				}
			}
			if (IsActive()) 
				m_pOutStream -> Dispatch (nOutSize, dwTime);
			if (nOutSize) prevTime = dwTime;
		}
		m_pInStream -> Release (TRUE);
	}
	//m_pOutStream ->Close();
	#ifdef MPGJ
// MPEGJ MERGE_7 - RLS
	/***** THROW STREAM END EVENT - PA */
	// set the status varible
	di = MPJGetTerminalDispatchTable();
	// go till end of list and add the new event there
	decoderQueue* temp;
	temp = &(di->dEQ);
	if( *temp == NULL )
		//*** YT START (119)
		//(*temp) = new decoderEventQueue( 5, ODID);
		(*temp) = new decoderEventQueue( 5, ODID, ESID);
		//*** YT END (119)
	else {
		while ( (*temp)->next != NULL )
			*temp = (*temp)->next;
		//*** YT START (119)
		//(*temp)->next = new decoderEventQueue( 5, ODID);
		(*temp)->next = new decoderEventQueue( 5, ODID, ESID);
		//*** YT END (119)
	}
	if( di->isStartedDispatching == 1) {
		// signal the thread to throw the event
		if ( di->isJVMStarted == 1 ) {
			count = 0;
// SOLARIS PORTING
#ifdef WIN32
			count = ResumeThread(di->hThrow);
#else
			count = thr_continue( *(thread_t *)(di->hThrow) );
#endif
		}
	}

	/***** end of thread resume code  */
	//END  MPEGJ MERGE_7 - RLS
	#endif //MPGJ

	((MediaStreamImp*)m_pOutStream)->SetInitialBufferingOff();


	if(!m_isPCMData)
	{
		m_mutex=new ZMutex();
		//m_smrDecoder->EnableAutoMagnification();
		while(IsActive())
		{
			if(m_smrDecoder->IsUpdated())
			{
				DWORD dwTime=0;
				int nOutSize=GetAttributes(FRAME_WIDTH,NULL)*GetAttributes(FRAME_HEIGHT,NULL)*3;
				m_rendering=TRUE;
				m_mutex->P();
				m_smrDecoder->Decode((LPVOID)pOutData,GetAttributes(FRAME_WIDTH,NULL),GetAttributes(FRAME_HEIGHT,NULL));
				m_mutex->V();
				/*
				int cnt=m_width*m_height*3;
				int c=255-(m_smrDecoder->GetFirstVisibleMeasure()-1)*3;
				if (c<0)
				c=0;
				for(int i=0; i<cnt; i++)
				pOutData[i]=c;
				*/
				m_rendering=FALSE;
				GetOutputStream () -> Dispatch (nOutSize, dwTime);
			}
			
			Sleep(500);
		}
		delete m_mutex;
	}
	GetOutputStream () -> Close ();
}

DWORD SMRDecoder :: GetAttributes (
	ATTRIBUTE_CODE code,	const LPBYTE
)
{
	switch (code) {
	case FRAME_WIDTH:		
		return m_width;
	case FRAME_HEIGHT:
		return m_height;
	case PITCH:
		return 3*m_width;
	case OFFSETX:
		return 0;
	case OFFSETY:
		return 0;
	case AUDIO_FREQUENCY:
		return 44100;
	case BITS_PER_SAMPLE :
		return 16;
	case NUM_CHANNELS:
		return 2;
	default:
		return 0;
	}
}

void SMRDecoder::JumpToMeasure(int measure)
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		m_smrDecoder->JumpToMeasure(measure);
		m_mutex->V();
	}
}

void SMRDecoder::JumpToLabel(const char* label)
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		m_smrDecoder->JumpToLabel((char*)label);
		m_mutex->V();
	}
}

void SMRDecoder::JumpToTime(int time, int& x, int& y, bool& reachedEnd)
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		m_smrDecoder->JumpToTime(time, x, y, reachedEnd);
		m_mutex->V();
	}

}

void SMRDecoder::JumpToTime(int time)
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		m_smrDecoder->JumpToTime(time);
		m_mutex->V();
	}

}

void SMRDecoder::GetDefaultHighlightPos(int& x, int& y)
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		m_smrDecoder->GetDefaultHighlightPos(x, y);
		m_mutex->V();
	}

}

int SMRDecoder::GetMeasureCount()
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		int measureCount = m_smrDecoder->GetMeasureCount();
		m_mutex->V();
		return measureCount;
	}
	return 0;
}

char** SMRDecoder::GetAvailableCommands()
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		char** cmds = m_smrDecoder->GetAvailableCommands();
		m_mutex->V();
		return cmds;
	}
	return 0;
}

char** SMRDecoder::GetAvailableViewTypes()
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		char** views = m_smrDecoder->GetAvailableViewTypes();
		m_mutex->V();
		return views;
	}
	return 0;
}

char** SMRDecoder::GetAvailableLyrics()
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		char** views = m_smrDecoder->GetAvailableLyrics();
		m_mutex->V();
		return views;
	}
	return 0;
}

int SMRDecoder::GetPartCount()
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		int partCount = m_smrDecoder->GetPartCount();
		m_mutex->V();
		return partCount;
	}
	return 0;
}

char* SMRDecoder::GetPartName(int part)
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		char *partName = m_smrDecoder->GetPartName(part);
		m_mutex->V();
		return partName;
	}
	return "";
}

int SMRDecoder::GetLabelCount()
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		int labelCount= m_smrDecoder->GetLabelCount();
		m_mutex->V();
		return labelCount;
	}
	return 0;
}

char* SMRDecoder::GetLabelName(int n)
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		char* label= m_smrDecoder->GetLabelName(n);
		m_mutex->V();
		return label;
	}
	return "";
}

int SMRDecoder::GetFirstVisibleMeasure()
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		int measure=m_smrDecoder->GetFirstVisibleMeasure();
		m_mutex->V();
		return measure;
	}
	return 0;
}

int SMRDecoder::GetLastVisibleMeasure()
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		int measure=m_smrDecoder->GetLastVisibleMeasure();
		m_mutex->V();
		return measure;
	}
	return 0;
}


void SMRDecoder::DoCommand(char* cmd, int x, int y, int argc, const char** argv)
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		m_smrDecoder->DoCommand(cmd,x+m_width/2,m_height/2-y,argc,argv);
		m_mutex->V();
	}
}

BOOL SMRDecoder::IsReady()
{
	return m_ready;
}

char* SMRDecoder::Link(int x,int y)
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		char *link = m_smrDecoder->Link(x+m_width/2,y+m_height/2);
		m_mutex->V();
		return link;
	}
	else
		return 0;
}

void SMRDecoder::ShowParts(int parts[], int n)
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		m_smrDecoder->ShowParts(parts,n);
		m_mutex->V();
	}
	else
	{
		m_parts=new int[n];
		m_nparts=n;
		for(int i=0; i<n; i++)
			m_parts[i]=parts[i];
	}
}

void SMRDecoder::SetPartLyrics(ZString* lyr, int n)
{
	if(m_smrDecoder && m_mutex)
	{
		m_mutex->P();
		char** l=new char*[n+1];
		for(int i=0; i<n;i++)
			l[i]=(char*)(unsigned char*)lyr[i];
		l[n]=NULL;
		m_smrDecoder->SetPartLyrics(l);
		delete[] l;
		m_mutex->V();
	}
}