/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#ifndef __SMRDECODER_H__
#define __SMRDECODER_H__

#include "defs.h"
#include "DecoderImp.h"
#include <direct.h>

class SMRInternalDecoder;

class SMRDecoder : public DecoderImp
{

public:

	LPBYTE GetOutput()
	{
		return m_pOutdata;
	}

	SMRDecoder():DecoderImp()
	{
		m_isPCMData=false;
		m_pOutdata=NULL;
		m_smrDecoder=NULL;
		m_smrDecoderInfo = NULL;
		m_smrDecoderInfoSize = 0;
		m_width = 640;
		m_height = 480;
		m_ready=FALSE;
		m_rendering=FALSE;
		m_mutex=NULL;
		m_parts=NULL;
		m_nparts=0;

		m_end=FALSE;
		m_pOutdata=NULL;
	};

	virtual int GetOptimalOutputSize ();

	virtual BOOL SetFormat (Capabilities &formatOptions,DecoderParams &decoderParams);

	DWORD GetAttributes (ATTRIBUTE_CODE code, const LPBYTE);

	virtual void Run ();

  	virtual BOOL Decode (LPBYTE pInput,	int nInputLength, LPBYTE pOutput, int& rnOutputLength);

	void SetSize(int width, int height) { m_width = width; m_height=height; };

	void JumpToMeasure(int measure);
	void JumpToLabel(const char* label);
	void JumpToTime(int time);
	void JumpToTime(int time, int& x, int& y, bool& reachedEnd);
	void GetDefaultHighlightPos(int& x, int& y);
	
	int GetMeasureCount();
	int GetFirstVisibleMeasure();
	int GetLastVisibleMeasure();

	int GetPartCount();
	char* GetPartName(int part);

	int GetLabelCount();
	char* GetLabelName(int n);

	char** GetAvailableCommands();
	char** GetAvailableViewTypes();
	char** GetAvailableLyrics();

	void ShowParts(int part[], int n);
	void SetViewType(char* viewType);
	void SetHyperlinkEnable(bool enable);

	void SetPartLyrics(ZString* lyr,int n);

	void DoCommand(char* command, int x, int y, int argCount, const char** arguments);

	void Transpose(float value);

	long GetDurationMeasure(int measure);
	void GoNextPage();

	BOOL IsReady();
	BOOL IsRendering() { return m_rendering; }

	char* Link(int x,int y);

private:

	BOOL m_isPCMData;
	BOOL m_bActive;
	LPBYTE m_pOutdata;
	BOOL m_end;
	BOOL m_ready;
	BOOL m_rendering;
	SMRInternalDecoder *m_smrDecoder;
	int m_width, m_height;
	int *m_parts;
	int m_nparts;
	char* m_smrDecoderInfo;
	unsigned int m_smrDecoderInfoSize;
	ZMutex* m_mutex;

	virtual BOOL Setup (
		const BYTE *pSpecificInfo,
		int nSpecificInfoLength,
		int	nESNumber
	);

};

#endif 
