/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence) 
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

#ifndef SMRINTERNALDECODER_H
#define SMRINTERNALDECODER_H

#ifdef SMRDECODERLIB_EXPORTS
  #define DLL_DECL __declspec(dllexport)
#else
  #define DLL_DECL __declspec(dllimport)
#endif

class SMRFile;
class Partitura;
class SMRcursor;

struct SynchInfo
{
	int measure;
	int durate;
	int startTime;
};

typedef unsigned int treecursor[5];

typedef enum treecurpos{
	POS_BEGIN,
	POS_END,
	POS_LEFT,
	POS_RIGHT,
	POS_UP,
	POS_DOWN,
	POS_IN,
	POS_OUT,
	POS_NEXT,
	POS_PREV
}treecursorpos;


class DLL_DECL SMRInternalDecoder
{
public:
	SMRInternalDecoder(int width,int height, int nscore, char* title = NULL);
	SMRInternalDecoder(char* wedelfile, int nscore, int measure, int w, int h);  //nscore: 0 multi, 1 single
	SMRInternalDecoder(char* decoderInfo, int decoderInfoSize, char* buffer, int size, int nscore, int measure, int w, int h);  //nscore: 0 multi, 1 single
	~SMRInternalDecoder();

	void Decode(void* videobuffer,int w, int h);
//	unsigned long GetHBITMAP(int width,int height);			// getting GDI BITMAP handle to the internal bitmap

	bool IsUpdated();// { bool upd=updated; updated=false; return upd; };

	char** GetAvailableCommands();
	char** GetAvailableViewTypes();
	char** GetAvailableLyrics();

	int GetPartCount();
	char* GetPartName(int part); // 0-based position

	int GetLabelCount();
	char* GetLabelName(int pos); // 0-base position

	char** GetPartLyrics();
	void SetPartLyrics(char** lyr);

	int GetMeasureCount();
	int GetFirstVisibleMeasure();
	int GetLastVisibleMeasure();

	void JumpToTime(int time);
	void JumpToTime(int time, int& highlightX, int& highlightY, bool& reachedEnd);
	void JumpToMeasure(int measure);
	void JumpToLabel(char* labelName);
	void GetDefaultHighlightPos(int& highlightX, int& highlightY);

	void ShowParts(int part[], int n, bool useMulti = false);
	void SetViewType(char* viewType);
	void SetHyperlinkEnable(bool enable);
	char* Link(int x, int y);

	void	DoCommand(char* command, int x, int y, int argCount, const char** arguments);
//	char*	GetCommand(char* command, int x, int y, int argCount, const char** arguments);

	void Transpose(float value);
	void LoadSynch(char* path);

	int NewSMR(char* title, int score);
	int LoadSMR(char* path, int nscore);
	int SaveSMR();
	int SaveAsSMR(char* title);

	void GoTop();
	void GoBottom();
	void GoBackward();
	void GoForward();

	//-- SCORE FOLLOWER SPECIFIC COMMANDS --//
	int SetCurrentPart(int part);
	int SetCurrentStaff(int staff);
	int SetCurrentMeasure(int measure);
	int SetCurrentVoice(int voice);
	int SetCurrentEvent(int event);
	int SetChordNote(int event);


	char* GetPartName();
	int GetStaffCount();
	int GetVoiceCount();
	int GetEventCount();
	int GetMeasCount();
	int GetEventType();
	int GetChordNoteCount();

	int GetBarType();
	int GetMetroBPM();
	int GetMetroRef();
	char* GetLabel();
	int GetTimeNum();
	int GetTimeDen();

	void InitDrag(int x, int y);
	void Drag(int dx, int dy);
	void EndDrag(int dx, int dy);

	int GetHeight();
	void SetHeight(int);

	char* GetPitch();
	int GetDuration();
	int SetDuration(int value);//NON FUNZIONA
	int isTied();

	int GetClef(bool isInternal);
	int GetKey(bool isInternal);

	const char* GetSMRDir();

	//void HilightOn();
	//void HilightOff();
	void Highlight(bool val = true);

	void ShowOneBar(bool value = true);
	void SetMagnification(float value = 1.0);
	float GetMagnification();
	void EnableAutoMagnification(bool status = true);

	// identifica un oggetto.
    //Bool SymbolicHit(SymPath&,SelObj,DrawObject*&);
	void SetHitLevel(const unsigned int & level);
	const unsigned int GetHitLevel();

	bool Hit(int x, int y);
	void GetTreeCursor(treecursor & cur);
	void SetTreeCursor(treecursor & cur);
	void MoveTreeCursor(const treecursorpos & tcp);

	// Decodifica un path.

	//*BIAN-VERR*
	bool DoNetCommand(char* stringa);

	bool CreateNetCommand(char* command, int x, int y, int argCount, const char** arguments, char* cmd, char* anti);
	//*BIAN-VERR*

	bool CreateNetCommandFromCursor(char* command, int argCount, const char** arguments, char* cmd, char* anti);

	void SetColor(const unsigned char red, const unsigned char green, const unsigned char blue);

private:
	int scoreWidth;
	int scoreHeight;
	SMRFile* smrf;
	Partitura* partitura;
	void *pixmap;
	bool updated;

	SynchInfo *synchInfo;
	int synchInfoCount;
	int currentMeasure;

	int TimeToSynchPos(int time, bool& reachedEnd);

	//	SMR MODEL ACCESS for Score follow

//	Figura		*pFigure;
//	int			curEvtType;

//	ScoreNavStatus navStatus;
	SMRcursor		*smrCursor;
	unsigned int	hitLevel;
//	Colore			*oldColor;
//	DrawObject		*oldObj;
	//MainSel			*pSel;

	int dragTemp;

	char** lyrics;

	bool			highlighted;
	bool			onebarshown;

	bool SetCommand(char* command, int argCount, const char** arguments);

	void extractLyrics();

	const unsigned int getCurrentDepth();

	void selectCursorAtDepth(const unsigned int depth);

};
#endif