/* ISO_HEADER_START */

/*********************************************************************

This software module was originally developed by

  Pierfrancesco Bellini (DISIT-DSI, University of Florence)
  Paolo Nesi (DISIT-DSI, University of Florence)
  Maurizio Campanai (EXITECH)

in the course of development of the MPEG-4 standard.
This software module is an implementation of a part of one or more
MPEG-4 tools as specified by the MPEG-4 standard. ISO/IEC gives users
of the MPEG-4 standard free license to this software module or
modifications thereof for use in hardware or software products
claiming conformance to MPEG-4. Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents.  The original developer of this software
module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation.  Copyright is
not released for non MPEG-4 conforming products. The DISIT-DSI and 
EXITECH retains full right to use the code for its own purpose,
assign or donate the code to a third party and to inhibit third
parties from using the code for non MPEG-4 conforming products.  This
copyright notice must be included in all copies or derivative
works. Copyright (c) 2005.

***********************************************************************/

/* ISO_HEADER_END */

/* $Id: partitur.cpp,v 1.2 2005/07/28 09:52:30 bellinpo Exp $ */


#include <sys/stat.h>
#include <sys/types.h>
#include <sys/timeb.h>
#include <time.h>
#include <fcntl.h>

#include "gui/dialogs.hpp"
#include "wx/choicdlg.h"
#include "pag/casellaimg.hpp"	

#include "fig/spazio.hpp"
#include "par/partitur.hpp"
#include "net/logfile.hpp"
#include "gui/extabil.hpp"
#include "liooext.hpp"
#include "lioo.hpp"
#include "smr/smrmain.hpp"
#include "smr/smr.hpp"
#include "imp/midiimport.hpp"
#include "mdo/moodsmidiwrite.hpp"
#include "aud/audiothread.hpp"
#include "drw/scorefrm.hpp"
#include "drw/scorecan.hpp"  
#include "drw/musicedt.hpp"  
#include "musiceditor.hpp"

#include <wx/progdlg.h>

const BYTE MIDI_SPEED_VAR_STEP=30; // (30=3%)
const FLOAT INC_TRACKING_RATE=(FLOAT)1;
const FLOAT DEC_TRACKING_RATE=(FLOAT)1;

enum GroupType
{
	NOTE_GROUP,
	REST_GROUP,
	EXPRESSION_GROUP,
	ACCIDENTAL_GROUP,
	LABEL_GROUP,
	CLEF_GROUP,
	TIMESIGN_GROUP,
	KEYSIGN_GROUP,
};

#include "drw/frmanage.hpp"

#include "wx/string.h"

#ifdef __WXMSW__
#define COPY_CMD "copy "
#else
#define COPY_CMD "cp "
#endif

extern int Mkdir(char*,int);
extern bool DirExists(char*);

int loadError;

extern LogFile MyLog;
extern wxMusicEditor *MusicEditor;

extern Bool ComputerViewParWindow(int* topparameter, int* bottomparameter, int* leftparameter, int* rightparameter, int* staffdistparameter, float* scale);  


extern char ArrayBuff[];
static NumCode MeasureNumCode_CutPaste=0;
static NumCode ScoreNumCode_CutPaste=0;

extern int DebugNormalMilla;  //inserimento
extern int DebugNormalMillaPos;  //posizionamento

							 /*
							 *
							 *  Questi metodi sono stati usati per debug, e sono tutti in fondo al file 
							 *
*/
void fStampaSym(FILE *pippo,SymCmd sc);
void StampaSym(SymCmd sc);

extern ExtLiooAbil liooAbil;

void BeginWait();
void EndWait();

ListaEtichette *Partitura::listaEti=NULL;
DrawObject *Partitura::pageBox=NULL;

Partitura::Partitura(bool empty)
{
	if(!empty)
	{
		listaSpa=new ListaSpartiti;
		listaPar=new ListaParentesi;
		listaPag=new ListaPagine;
		owner=true;
	}
	else
	{
		listaSpa=NULL;
		listaPar=NULL;
		listaPag=NULL;
		owner=false;
	}

	p_sequencer=NULL;
	selections=NULL;
	
	ETT=NULL;
	ETTLenght=0;
	Impaginazione = END_IMP;
	StatoEsecuzione = IMPAGINAZIONE;
	printing_status = false;
	
	SetFileLoadName("");
	SetFileSaveName("");
	SetPathLoadName("");
	SetPathSaveName("");
	
	SaveMethod=MDS_SAVE;
	SetImg=FALSE;
	
	casellaPtr=NULL;	
	
	audioRate = 0;
	pAudioThread = NULL;
	maxNumHit = 10;
	mmExecMode = MM_USER;
	declickMode = DECLICK_ON;
	measureSync = NULL;
	beginBlock = 0;
	SetAudioFile("noaudio");
	/////////////////////////////////////////////
}


Partitura::~Partitura()
{
	
#ifdef DEBUG_FREE
	if (tipoLeggio!=LEGGIO_MASAE)
	{
		FILE *pippo;
		
		pippo = fopen("/home/demo/integrazione/src/DEAL_NET.txt","a");
		fprintf(pippo,"\n\n\n ------------   DEALLOCO PARTITUR ----------------\n\n\n");
		fclose(pippo);
	}
#endif
	
	Free();
	
}


void Partitura::Free()
{
	if (ETT!=NULL) delete[] ETT;
	ETT = NULL;
	
	// not deleting scores and brackets
	if(owner)
	{
		delete listaSpa;
		delete listaPar;
		delete listaPag;
		listaSpa=NULL;
		listaPar=NULL;
		listaPag=NULL;
	}
}

void Partitura::FullFree(bool removeFromWDFTree)
{
	if (ETT!=NULL) delete[] ETT;
	ETT = NULL;
	int nScore=GetNumSpa();

	if(listaSpa)
		listaSpa->Free();
	if(listaPar)
		listaPar->Free();
	if(listaPag)
		listaPag->Free();
	
	if(removeFromWDFTree)
		for(int i=0;i<nScore; i++)
			MusicEditorPartDeleted(1);

	ListaEtichette *prec = listaEti;
	while (prec != NULL)
	{
		listaEti = listaEti->pNext;
		delete prec;
		prec = listaEti;
	}
	listaEti = NULL;
	if(listaSpa)
		listaSpa->SetCutCopy(false);	
}

void Partitura::Init()
{
	if(listaSpa==NULL)
		listaSpa=new ListaSpartiti;
	if(listaPar==NULL)
		listaPar=new ListaParentesi;
	if(listaPag==NULL)
		listaPag=new ListaPagine;
	listaSpa->Init();
	listaPar->Init();
	StatoEsecuzione = IMPAGINAZIONE;
}


void Partitura::New()
{
	FullFree();
	Init();
	showTagli=FALSE;
	SetFileLoadName("");
	SetFileSaveName("");
	SetPathLoadName("");
	SetPathSaveName("");
	
	MyLog.DeleteTempFile();
	
	ResetCommand();
}


int Partitura::GetLeftMargin()
{
	return GetPrintViewParams()->GetLeftMargin();
	
}

int Partitura::GetRightMargin()
{
	return GetPrintViewParams()->GetLeftMargin();
}

int Partitura::GetTopMargin()
{
	return GetPrintViewParams()->GetTopMargin();
}

int Partitura::GetBottomMargin()
{
	return GetPrintViewParams()->GetBottomMargin();
}




void Partitura::SetSaveMethod(SaveType NewSaveMethod)
{
	SaveMethod=NewSaveMethod;
	return;
}

SaveType Partitura::GetSaveMethod(void)
{
	return SaveMethod;
}

void Partitura::ResetACL(void)
{ 
	MyLog.EliminaLista(); 
	return; 
};

void Partitura::KillLioo()
{
	return;
}


StateExecution Partitura::GetStatoEsecuzione()
{
	return StatoEsecuzione;
}

void Partitura::SetStatoEsecuzione(StateExecution st)
{
	StatoEsecuzione = st;
}

void Partitura::SetImpaginazione(StateImpag imp)
{
	if (imp == BEGIN_IMP) GetFirstSpa()->SetPagCounter(1);
	if ((imp==END_IMP) && (StatoEsecuzione == IMPAGINAZIONE)) StatoEsecuzione = INIT_ESECUZIONE;
	Impaginazione = imp;
}

Bool Partitura::GetImpaginazione()
{ 
	return ((Impaginazione == BEGIN_IMP) || (Impaginazione == END_BATT_IMP) || (Impaginazione == END_BATT_IMP_FIRST));
}

Bool Partitura::GetBeginImpag()
{ 
	return (Impaginazione == BEGIN_IMP);
}

Bool Partitura::GetFirstEndBattImp()
{ 
	return (Impaginazione == END_BATT_IMP_FIRST);
}

Bool Partitura::GetEsecuzione()
{ 
	return ((Impaginazione == BEGIN_ESEC) || (Impaginazione == END_BATT_ESEC));
}

Bool Partitura::GetBeginEsec()
{
	return (Impaginazione == BEGIN_ESEC);
}


void Partitura::DeselectAll()
{
	DrawObject *drw;
	
	BeginPaint();
	// richiama il metodo Deselect di tutti gli oggetti in listaSel
	drw=listaSel.GetFirst();
	while(drw!=NULL)
	{
		drw->Deselect();
		drw=listaSel.GetNext(drw);
	}
	EndPaint();
}


void Partitura::SelectAll()
{
	DrawObject *drw;
	
	BeginPaint();
	
	// richiama il metodo Select di tutti gli oggetti in listaSel
	drw=listaSel.GetFirst();
	while(drw!=NULL)
	{
		drw->Select();
		drw=listaSel.GetNext(drw);
	}
	EndPaint();
}


void Partitura::BeginPaint()
{
	if(Scr==NULL || pMusicWin==NULL)
		return;
	
	if(showTagli)
	{
		Point mousePos=Scr->GetMouse()->GetPos();
		if(Hit(mousePos))
			DrawTagli(mousePos);
	}
	Scr->GetMouse()->Hide();
	
	((LWindow*)pMusicWin)->BeginPaint(); 
}


void Partitura::EndPaint()
{
	if(Scr==NULL || pMusicWin==NULL)
		return;
	((LWindow*)pMusicWin)->EndPaint(); 
	
	Scr->GetMouse()->Show();
	
	if(showTagli)
	{
		Point mousePos=Scr->GetMouse()->GetPos();
		if(Hit(mousePos))
			DrawTagli(mousePos);
	}
}


void Partitura::ShowTagli()
{
	if(!showTagli)
	{
		Point mousePos=Scr->GetMouse()->GetPos();
		if(Hit(mousePos))
			DrawTagli(mousePos);
		showTagli=TRUE;
	}
}


Bool Partitura::HideTagli()
{
	Bool showingTagli=showTagli;
	if(showTagli)
	{
		Point mousePos=Scr->GetMouse()->GetPos();
		if(Hit(mousePos))
			DrawTagli(mousePos);
		showTagli=FALSE;
	}
	return showingTagli;
}


Bool Partitura::GetShowTagli()
{ return showTagli; }


void Partitura::DrawTagli(const Point& p)
{
	listaSpa->DrawTagli(p);
}


void Partitura::SetMusicRect(const class Rectangle& r)
{
	listaSpa->SetMusicRect(r); 
}

void Partitura::SetPrintRect(const class Rectangle& r)
{
	listaSpa->SetPrintRect(r); 
}

void Partitura::GoForward()
{ 
	if (listaSpa->GoForward())
	{
		//deseleziona gli oggetti
		DeselectAll();
		listaSel.Clear();
		Redraw();  
	}  
}


void Partitura::GoBackward()
{ 
	if (listaSpa->GoBackward())
	{
		//deseleziona gli oggetti
		DeselectAll();
		listaSel.Clear();
		Redraw();
	}
}


Bool Partitura::GoNextPage()
{ 
    // va alla pagina successiva e la disegna in Disegno (finestra o pixmap)
	
	if (listaSpa->GoNextPage())
	{
		
		{
			
#ifdef SCR_FINESTRE
			// Con le finestre imposta la finestra di Disegno e ci scrive
			Scr->GSetDrawArea(pMusicWin->GetDisegnoDrawingArea());  
			Redraw();
			Scr->GSetMainDrawArea();
#else
			// Con le pixmap imposta la Pixmap puntata da Disegno e ci scrive
			Scr->GSetPixmap(Scr->GetLPixmapDisegno());
			Redraw();
			Scr->GResetPixmap();
#endif			
		}
		return TRUE;
	}  
	return FALSE;
}

Bool Partitura::GoNextPageInExecution()
{ 
    // va alla pagina successiva e la disegna in Disegno (finestra o pixmap)
	
	if (listaSpa->GoNextPageInExecution())
	{
#ifdef SCR_FINESTRE
		// Con le finestre imposta la finestra di Disegno e ci scrive
		Scr->GSetDrawArea(pMusicWin->GetDisegnoDrawingArea());  
		Draw();
		Scr->GSetMainDrawArea();
#else
		// Con le pixmap imposta la Pixmap puntata da Disegno e ci scrive
		Scr->GSetPixmap(Scr->GetLPixmapDisegno());
		DrawInExecution();
		Scr->GResetPixmap();
#endif			
		return TRUE;
	}  
	return FALSE;
}


Bool Partitura::GoPrevPage()
{ 
	extern wxMusicEditor *MusicEditor;
	int SystemsNo = GetPrintViewParams()->GetNumSystems(listaSpa->GetCurrentPage());
	
    // va alla battuta precedente e la disegna in Disegno (finestra o pixmap)	
	
	if(GetPrintingStatus())
	{	bool flag=false;
	for(int i=0;i<SystemsNo;i++)	
		flag=listaSpa->GoPrevPage();
	if(listaSpa->GetCurrentPage()>1)
		listaSpa->SetCurrentPage(listaSpa->GetCurrentPage()-1);
	
	return flag;
	}
	else
	{
		if (listaSpa->GoPrevPage())
		{
			//deseleziona gli oggetti
			DeselectAll();
			listaSel.Clear();
			
#ifdef SCR_FINESTRE
			// Con le finestre imposta la finestra di Disegno e ci scrive
			Scr->GSetDrawArea(pMusicWin->GetDisegnoDrawingArea());  
			Redraw();
			Scr->GSetMainDrawArea();
#else
			// Con le pixmap imposta la Pixmap puntata da Disegno e ci scrive
			Scr->GSetPixmap(Scr->GetLPixmapDisegno());
			Redraw();
			Scr->GResetPixmap();
#endif			
			
			return TRUE;
		}  
		return FALSE;
	}
}

void Partitura::EditingGoBottom()
{;};     
void Partitura::EditingJumpBatt(int nbat)
{;};
void Partitura::EditingGoTop()
{;};


void Partitura::NetPiu5()
{
	int j=0;
	int i;
	
	for (i=1;i<=5;i++)
		if (listaSpa->GoForward()) j++;
		if (j>0)
		{
			DeselectAll();
			listaSel.Clear();
			Redraw();
		}
		
}


void Partitura::NetMeno5()
{
	int j=0;
	int i;
	
	for (i=1;i<=5;i++)
		if (listaSpa->GoBackward()) j++;
		
		if (j>0)
		{
			DeselectAll();
			listaSel.Clear();
			Redraw();
		}
		
}




void Partitura::GoBottom()
// Posiziona alla fine della partitura.
{
	Spartito* psp;
	Battuta* bat;
	
	psp=GetFirstSpa();
	bat=psp->GetLastBat();
	
	GoBatt((bat->GetNumProgress()-1));
	
} 

void Partitura::RequestJump()
{
	int i;
	int NumEtich=GetNumEtichette();
	int NumBatt=GetNumeroBattute();
	char* arrayLabel;
	int* arrayNumProg;
	int nbat;
	char buffer[200];
	
	arrayLabel=new char [NumEtich+1];
	arrayNumProg=new int [NumEtich+1];
	
	// creazione vettore delle label
	for (i=0;i<NumEtich;i++)
	{
		arrayLabel[i]=GetEtichetta(i);
		arrayNumProg[i]=GetBattEtichetta(arrayLabel[i]);
	} 
	
	if (EDWidgetJump("Jump in Score",buffer,200,NumEtich ,NumBatt,arrayLabel,arrayNumProg))
	{ 
		if (!(strcmp(buffer,"TOP")))
		{
			GoTop();
		}  
		else if (!(strcmp(buffer,"BOT."))) 
        {
			GoBottom();
		} 
		else {   
			nbat = atoi(buffer);  // trova la battuta in forma numerica (NumericCode)
			GoBatt(nbat);
		} 
	}
}

void Partitura::GestLabel()
{
	char label[2];
	int i;
	int NumEtich=GetNumEtichette();
	char* arrayLabel;
	
	arrayLabel=new char [NumEtich+1];
	
	// creazione vettore delle label
	for (i=0;i<NumEtich;i++)
	{
		arrayLabel[i]=GetEtichetta(i);
	} 
	
	label[1]='\0';
	if (WReadEtich("Labels",label,200, NumEtich,arrayLabel)) 
	{
		label[1]='\0';
		
		pMusicWin->SetCommand(CMD_BATTUTA_LETTERA,IMMEDIATE_SEL,BATTUTA_SEL,0,0,0,0,0,label);
		
	}
	else
		Scr->GSetMainDrawArea();
	
	delete arrayLabel;
	
}	

void Partitura::CalcLivelli()
{ 
	VUnit start, end, startp, endp;
	int liv, livp;
	
	Parentesi *par;
	Parentesi *parL = listaPar->GetFirstPar();
	
	while(parL!=NULL)
	{
		start = listaSpa->GetPos(parL->GetPSpaStart());
		end = listaSpa->GetPos(parL->GetPSpaEnd());
		liv = 0;
		
		par = listaPar->GetFirstPar();
		
		while(par!=parL)
		{
			livp = par->GetLivello();
			startp = listaSpa->GetPos(par->GetPSpaStart());
			endp = listaSpa->GetPos(par->GetPSpaEnd());
			
			if ((( start >= startp && start <= endp )||( end >= startp && end <= endp )||
				( startp <= end && startp >= start)||(endp <= end && endp >= start))
				&& livp > liv)
			{
				liv = livp;
			}  
			par = listaPar->GetNextPar(par);
			
		}
		parL->SetLivello(liv+1);
		
		parL = listaPar->GetNextPar(parL);
	}
	
}


void Partitura::CalcProlBarre()
{
	int startspa, endspa, livp;
	Parentesi *parL = listaPar->GetFirstPar();
	Spartito *startSpaPtr, *endSpaPtr;
	
	// setta tutti i prolungamenti a FALSE  
	for (int i=0;i<listaSpa->GetNumObj();i++)
	{
		((Spartito*)listaSpa->AtPos(i))->SetProlBarraUp(FALSE);
		((Spartito*)listaSpa->AtPos(i))->SetProlBarraDown(FALSE); 
	}
	
	while(parL!=NULL)
	{
		livp = parL->GetLivello();
		startspa = listaSpa->GetPos(parL->GetPSpaStart());
		startSpaPtr=((Spartito*)listaSpa->AtPos(startspa));
		endspa = listaSpa->GetPos(parL->GetPSpaEnd());
		endSpaPtr=((Spartito*)listaSpa->AtPos(endspa));
		if(startSpaPtr!=NULL && endSpaPtr!=NULL && 
			startSpaPtr->IsVisible() && endSpaPtr->IsVisible() && startSpaPtr!=endSpaPtr)
		{
			startSpaPtr->SetProlBarraDown(TRUE);
			endSpaPtr->SetProlBarraUp(TRUE);
			for (int level=startspa+1;level<endspa;level++)
			{
				((Spartito*)listaSpa->AtPos(level))->SetProlBarraDown(TRUE);
				((Spartito*)listaSpa->AtPos(level))->SetProlBarraUp(TRUE);
			}
		}
		
		parL = listaPar->GetNextPar(parL);
	}
}



// disegna senza posizionare
void Partitura::Draw()
{ 	
	if(Scr==NULL || pMusicWin==NULL)
		return;
	
	// abilita il disegno solo all'interno della finestra in modo da non
	// sporcare fuori.
	BeginPaint();
	
	
	// cancella l'area interna
	pMusicWin->Clear();
	//FFF Colore Base ---------------------------//
	Scr->SetColor((ColorIcon)NORMAL_COLOR);

	listaSpa->DrawPage(printing_status);
	if (printing_status)
	{
		for(int nsys=0;nsys<GetPrintViewParams()->GetNumSystems(listaSpa->GetCurrentPage());nsys++)
			listaPar->Redraw(0, printing_status, nsys);
		listaPag->DrawPage(listaSpa->GetCurrentPage());
	}
	else
		listaPar->Draw();
	
	// imposta la regione di disegno precedente.
	EndPaint();
}

void Partitura::DrawInExecution()
{ 
	// abilita il disegno solo all'interno della finestra in modo da non
	// sporcare fuori.
	BeginPaint();
	
	// cancella l'area interna
	pMusicWin->Clear();
	//FFF Colore Base ---------------------------//
	Scr->SetColor((ColorIcon)NORMAL_COLOR);

	listaSpa->DrawPageInExecution();
	listaPar->Draw();
	
	// imposta la regione di disegno precedente.
	EndPaint();
}
void Partitura::Redraw(void)
{
	Redraw(printing_status);
}

// posiziona gli oggetti e li disegna
void Partitura::Redraw(bool printing)
{ 
	extern wxMusicEditor *MusicEditor;
	printing_status=printing;
	
	if(Scr==NULL || pMusicWin==NULL)
		return;
	// abilita il disegno solo all'interno della finestra in modo da non
	// sporcare fuori.
	
	BeginPaint();
	
	if(!printing)
		Scr->SetMagnification(GetComputerViewParams()->GetMagnification());
	else
		Scr->SetMagnification(GetPrintViewParams()->GetMagnification());
	
	// cancella l'area interna
	pMusicWin->Clear();
	//FFF Colore Base ---------------------------//
	Scr->SetColor((ColorIcon)NORMAL_COLOR);

	CalcLivelli();
	CalcProlBarre();
	VUnit width = listaPar->GetWidth();
	if (!printing)
	{
        listaSpa->RedrawPage((int)(GetComputerViewParams()->GetLeftMargin()), 
			(int)(GetComputerViewParams()->GetTopMargin()), 
			(int)(GetComputerViewParams()->GetRightMargin()), 
			(int)(GetComputerViewParams()->GetBottomMargin()), 
			(int)(GetComputerViewParams()->GetStaffDistance()), width, printing_status);
		listaPar->Redraw(width, printing_status, -1);
	}
	else 
	{
		int nPage=listaSpa->GetCurrentPage();
		listaSpa->RedrawPage(GetPrintViewParams()->GetLeftMargin(nPage),
			GetPrintViewParams()->GetTopMargin(nPage),
			GetPrintViewParams()->GetRightMargin(nPage),
			GetPrintViewParams()->GetBottomMargin(nPage),
			GetPrintViewParams()->GetStaffDistance(nPage),
			width,printing_status);
		for(int nsys=0;nsys<GetPrintViewParams()->GetNumSystems(nPage);nsys++)
			listaPar->Redraw(width, printing_status, nsys);
		listaPag->DrawPage(listaSpa->GetCurrentPage());
	}	
	
	
	// imposta la regione di disegno precedente.
	EndPaint();
}



void Partitura::GoTop()
{
	if(GetPrintingStatus())
	{	
		if(GetCurrentScore()==0)
			listaSpa->SetCurrentPage(1);
		else
			GetSpartNum(GetCurrentScore()-1)->SetCurrentPage(1);
	}
	GoBatt(1);
	
}

void Partitura::GoBatt(NumCode nb)
{ 
	//FFF Colore Base ---------------------------//
	Scr->SetColor((ColorIcon)NORMAL_COLOR);

	listaSpa->GoBatt(nb);
	if (pMusicWin!=NULL && pMusicWin->GetModEsecuzione())
	{
#ifdef SCR_FINESTRE
		// Con le finestre imposta la finestra di disegno e ci scrive
		Scr->GSetDrawArea(pMusicWin->GetDisegnoDrawingArea());  
		Redraw();
		Scr->GSetMainDrawArea();
		// scambia da Disegno a Copia perche' questa sara' la prossima pagina
		pMusicWin->SwapDrawingArea();	
#else
		// Con le pixmap imposta la pixmap di disegno e ci scrive
		Scr->GSetPixmap(Scr->GetLPixmapDisegno());
		Redraw();
		Scr->GResetPixmap();
		// scambia da Disegno a Copia perche' questa sara' la prossima pagina
		Scr->SwapPixmap();
#endif
		
		// va alla pagina successiva per disegnare la terza pagina in Disegno	
		//    GoNextPage();	
		
	}
	else if(pMusicWin!=NULL)
		Redraw();
}

void Partitura::GoBattInExecution(NumCode nb)
{ 
	
	listaSpa->GoBattInExecution(nb);
	if (pMusicWin!=NULL && pMusicWin->GetModEsecuzione())
	{
#ifdef SCR_FINESTRE
		// Con le finestre imposta la finestra di disegno e ci scrive
		Scr->GSetDrawArea(pMusicWin->GetDisegnoDrawingArea());  
		DrawInExecution();
		Scr->GSetMainDrawArea();
		// scambia da Disegno a Copia perche' questa sara' la prossima pagina
		pMusicWin->SwapDrawingArea();	
#else
		// Con le pixmap imposta la pixmap di disegno e ci scrive
		Scr->GSetPixmap(Scr->GetLPixmapDisegno());
		DrawInExecution();
		Scr->GResetPixmap();
		// scambia da Disegno a Copia perche' questa sara' la prossima pagina
		Scr->SwapPixmap();
#endif
		
		// va alla pagina successiva per disegnare la terza pagina in Disegno	
		//    GoNextPage();	
		
	}
	else if(pMusicWin!=NULL)
		DrawInExecution();
}

void Partitura::GoBattIfOut(NumCode nb)
{
	Spartito *pSp = (Spartito *)listaSpa->GetFirst();
	if (pSp==NULL) return;
	Battuta *pBat1=pSp->GetfirstBatPage();
	Battuta *pBat2=pSp->GetfirstBatNextPage();
	
	if ((pBat1==NULL) || (pBat2==NULL)) GoBatt(nb);
	else 
		if ((nb<pBat1->GetNumProgress()) || (nb>=pBat2->GetNumProgress()))
			GoBatt(nb);
		
		
}




void Partitura::AddSpartito(Spartito *pSp, bool addInWDFTree)
{
	listaSpa->Add(pSp,addInWDFTree);
}


void Partitura::AddNewSpartito(Spartito *pSpRef, Bool sopra, int nStaff, char *name)
{
	Battuta *pBatt, *pBatt1;
	unsigned long cb;//, cf;
	Spartito *pSp1;
	
	Spartito *pSp = new Spartito(nStaff);
	
	if(!pSp)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Partitura","AddNewSpartito()");
	if(nStaff>1)	
	{
		NumCode nc;
		listaPar->Add(CMD_PARTITURA_ADD_GRAFFA,pSp,pSp, nc);
	}
	if(name!=NULL)
		pSp->SetStrumEsec(name);
	
	if (GetFirstSpa()!=NULL)
	{    
		pSp1 = GetFirstSpa();
		
		cb=0;        
		
		pBatt = pSp->GetBattNum(cb);
		
		//
		// aggiunge gli spazi necessari per riallineare le battute con quelle
		// degli spartiti presenti.
		//
		
		do
		{
			pBatt1 = pSp1->GetBattNum(cb);
			
			// imposta lo stesso numeric code
			pBatt->SetNumericCode(pBatt1->GetNumericCode());
			
			cb++;
			
			pBatt1=pSp1->GetBattNum(cb);
			
			if (pBatt1!=NULL)
			{
				pSp->AddBattutaDopo(pBatt);
				pBatt=pSp->GetBattNum(cb);
			}
		} while(pBatt1!=NULL);
		
		
		
		// imposta la stessa barra alle battute della stessa colonna.
		
		cb=0;        
		
		pBatt = pSp->GetBattNum(cb);
		pBatt1 = pSp1->GetBattNum(cb);
		
		do
		{
			pBatt->SetBarra((pBatt1->GetBarra())->GetTipoBarra());
			cb++;
			
			pBatt = pSp->GetBattNum(cb);
			pBatt1 = pSp1->GetBattNum(cb);
			
		} while(pBatt1!=NULL);
		
		
	}
    
	if (sopra)
	{
		if (pSpRef == GetFirstSpa()) // inserisce in testa alla lista
			listaSpa->AddAfter(pSp,NULL);
		else
		{
			pSpRef = GetPrevSpa(pSpRef);
			listaSpa->AddAfter(pSp,pSpRef);
		}
	}
	else
		listaSpa->AddAfter(pSp,pSpRef);
	
}



Spartito* Partitura::GetFirstSpa()
{
	return (Spartito*)listaSpa->GetFirst();
}  

Spartito* Partitura::GetLastSpa()
{
	return (Spartito*)listaSpa->GetLast();
}  

Spartito* Partitura::GetNextSpa(Spartito *psp)
{
	return (Spartito*)listaSpa->GetNext(psp);
}  


Spartito* Partitura::GetPrevSpa(Spartito *psp)
{
	return (Spartito*)listaSpa->GetPrev(psp);
}  

Spartito* Partitura::GetSpartNum(int n)
{
	return (Spartito*)(listaSpa->AtPos(n));
}


Spartito* Partitura::GetSpart(NumCode nc)
{
	return (Spartito*)(listaSpa->GetSpart(nc));
}


void Partitura::SetCommand(CmdID cmdId, SelType stype, SelObj sobj, ArgType arg0,
						   ArgType arg1, ArgType arg2, ArgType arg3, ArgType arg4,
						   char *txt, ArgType arg5, ArgType arg6, ArgType arg7,ArgType arg8)
{
	
	// inizializzazione variabili per la selezione
	ResetCommand(); //TODO FFF CONTROLLARE!!!
	
	UsrSymCmd.CommandId = cmdId;
	UsrSymCmd.SelectionType = stype;
	UsrSymCmd.SelObjType = sobj;
	
	UsrSymCmd.ArgV[0] = arg0;
	UsrSymCmd.ArgV[1] = arg1;
	UsrSymCmd.ArgV[2] = arg2;
	UsrSymCmd.ArgV[3] = arg3;
	UsrSymCmd.ArgV[4] = arg4;
	UsrSymCmd.ArgV[5] = arg5;
	UsrSymCmd.ArgV[6] = arg6;
	UsrSymCmd.ArgV[7] = arg7;
	UsrSymCmd.ArgV[8] = arg8;
	
	ResetPath(UsrSymCmd);
	
	if (txt!=NULL) 
		strncpy(UsrSymCmd.Text,txt,SYM_MAX_TEXT);
	else
		memset(UsrSymCmd.Text, 0, SYM_MAX_TEXT);	//FFF debug
	
	// attiva la selezione con il rettangolo se necessario
	if(stype==RECT_SEL)
		Scr->GetMouse()->SetRectSel();
	else
		Scr->GetMouse()->NoRectSel();
	FirstSel = TRUE;
};


void Partitura::ResetCommand() 
{ 
	short i;
	
	DeselectAll();
	// termina la selezione  
	Scr->GetMouse()->NoRectSel();
	listaSel.Clear();
	
	UsrSymCmd.SelectionType = NO_SEL;
	UsrSymCmd.SelObjType = ANY_SEL;
	ResetPath(UsrSymCmd);
	for(i=0;i<SYM_MAXARG;i++)
	{
		UsrSymCmd.ArgV[i] = 0;
	}
	UsrSymCmd.Text[0] = '\0';  
};


void Partitura::ResetPath(SymCmd& sc)
{
	short i;
	
	for(i=0;i<PATH_LENGTH;i++)
	{
		sc.Path1.Array[i] = 0;
		sc.Path2.Array[i] = 0;
	}
	sc.Path1.Level = sc.Path2.Level = 0;
}


SymCmd *Partitura::GetSymCmd() 
{ 
	return &UsrSymCmd;   
}


void Partitura::SetUsrSymCmd(SymCmd sc) 
{ 
	UsrSymCmd = sc; 
}

void Partitura::SetNetSymCmd(SymCmd sc)
{ 
	NetSymCmd = sc; 
}




Bool Partitura::Hit(const Point& p)
{
	return listaSpa->Hit(p);
}


Bool Partitura::Hit(const Point& p, SelObj sobj, DrawObject *& objsel,
					SymPath& symPath)
{
	if(GetPrintingStatus())
	{
		if(GetCurrentScore()==0)
		{
			if(listaPag->Hit(p,listaSpa->GetCurrentPage(),objsel))
				return TRUE;
		}
		else
			if(GetSpartNum(GetCurrentScore()-1)->Hit(p,sobj,objsel,symPath))
				return TRUE;
	}
	
	if(GetCurrentScore()==0)
	{
		if(listaSpa->Hit(p,sobj,objsel,symPath))
			return TRUE;
		else 
			return listaPar->Hit(p,sobj,objsel,symPath);
	}
	else
	{
		if(GetSpartNum(GetCurrentScore()-1)->Hit(p,sobj,objsel,symPath))
			return TRUE;
	}
	return FALSE;
}


Bool Partitura::Hit(const class Rectangle& r,SelObj selObj,short layer,
					SymPath& sp1, SymPath& sp2)
{
	return listaSpa->Hit(r,selObj,layer, sp1,sp2);
}


Bool Partitura::SymbolicHit(SymPath& symPath, SelObj sobj, DrawObject *& objsel)
{
	if(symPath.Array[LISTA_PATH_LEVEL]==3)
		return listaPar->SymbolicHit(symPath,sobj,objsel);
	else return listaSpa->SymbolicHit(symPath,sobj,objsel);
}



Bool Partitura::CopiaFileMDS2HID( wxString SourcePath, wxString SourceName, wxString DestPath, wxString DestName)
{
	// Devo copiare source in Dest, con i relativi sotto file. devo aggiungere al dest la data.
	// Copio il file originale :
	char Source[200];
	char Dest[200];
	
    char date[20];
    int val_max=20;
	
    static char* format="%d_%m_%y-%H_%M";
    struct tm* ttime;
    time_t t;
    t=time(NULL);
    ttime=localtime(&t);
	
    strftime(date,val_max,format,ttime);
    
	
	
	
	
	strcpy(Source,smrgetenv("SMR_FILES_PATH"));
	strcat(Source,SourcePath);
	strcat(Source,SourceName);
	strcpy(Dest,smrgetenv("SMR_FILES_PATH"));
	strcat(Dest,DestPath);
	strcat(Dest,DestName);
	strcat(Dest,".");
	strcat(Dest,date);
	
	FILE *fpsource=fopen(Source,"rt");
	FILE *fpdest=fopen(Dest,"wt");
	
	if ((fpsource==NULL) || (fpdest==NULL))
		return False;
	
	char Line[200],fname[100];
	int err;
	char *Sep;
	
	char Comando[400];
	char Dest1[200];
	
	
	
	
	
	while (!MyLog.Myeof(fpsource)) {
		fgets(Line,200,fpsource);
		err=sscanf(Line,"SRC %s\n",fname);
		if (err==1) {	
			
			strcpy(Dest1,fname);		
			Sep=strrchr(Dest1,'.');
			(*Sep)=0;
			strcat(Dest1,".hid.");
			strcat(Dest1,date);
			// preparo il comando
			strcpy(Comando,COPY_CMD);
			strcat(Comando,smrgetenv("SMR_FILES_PATH"));
			strcat(Comando,SourcePath);
			strcat(Comando,fname);
			strcat(Comando," ");
			strcat(Comando,smrgetenv("SMR_FILES_PATH"));
			strcat(Comando,DestPath);
			strcat(Comando,Dest1);
			// Eseguo il comando per copiare il sottofile
			system(Comando);
			
			// Scrivo il file
			fprintf(fpdest,"SRC %s\n",Dest1);		
		} else {
			
			fprintf(fpdest,"%s",Line);
		}
		
	}
	
	fclose(fpsource);
	fclose(fpdest);
	
	
	DestName+=".";
	DestName+=date;
	return True;  
}


Bool Partitura::DoCommandRectSel(const Point& p)
{ 
	class Rectangle rectSel=Scr->GetMouse()->GetRectSel();
	
	switch(UsrSymCmd.SelectionType)
	{
    case RECT_SEL:
		switch(UsrSymCmd.CommandId)
		{
			
		case CMD_PARTITURA_INSERTIMAGE:	
			if(GetPrintingStatus())
			{
				if(GetCurrentScore()==0)
					listaPag->CmdInsertImage(rectSel,listaSpa->GetCurrentPage());
				else 
				{
					int n=GetSpartNum(GetCurrentScore()-1)->GetCurrentPage();
					listaPag->CmdInsertImage(rectSel,n);
				}
				
				return TRUE;
			}
			break;  
			
		case CMD_BATTUTA_NOTE2GRUPPONOTE:
			ResetPath(UsrSymCmd);
			Hit(rectSel,UsrSymCmd.SelObjType,UsrSymCmd.ArgV[0]-1,UsrSymCmd.Path1,UsrSymCmd.Path2);
			// Esegue immediatamente il comando impostato!
			DeselectAll();
			// per la gestione dei comandi per DLIOO vedi DoCommandGruppo()
			if(DoCommandGruppo())
				Redraw();
			else
			{
				EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","DoCommandRectSel()");
				return FALSE;
			}
			return TRUE;
			
			
		case CMD_SPARTITO_ADD_BEAM:
			ResetPath(UsrSymCmd);
			Hit(rectSel,UsrSymCmd.SelObjType,UsrSymCmd.ArgV[0]-1,UsrSymCmd.Path1,UsrSymCmd.Path2);
			// Esegue immediatamente il comando impostato!
			DeselectAll();
			// per la gestione dei comandi per DLIOO vedi DoCommandGruppo()
			if(DoCommandGruppo())
				Redraw();
			else
			{
				EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","DoCommandRectSel()");
				return FALSE;
			}
			return TRUE;	
			
		case CMD_PARTITURA_SCORE_EXCERPT:
			(MusicEditor->GetCurrentViewFrame())->scorecanvas->PutSubCanvasInClipboard(rectSel.PUpLft.x,Scr->GetMaxPixY()-rectSel.PUpLft.y,
				rectSel.Width()-1,rectSel.Height()-1);
			break;
			
		default: ;
			
		}  // end switch interno
		
		break;
		
		default:
			// non deve fare nulla!
			return FALSE;
	}
	return FALSE;
	
}


void Partitura::DoCommandImmediate()
{
	const unsigned int BUFFER_LENGTH = 400;
	char* s;
	char nome[BUFFER_LENGTH],buffer[BUFFER_LENGTH],filesPath[200];;
	
	char *firstItem=NULL;
	char *afile=NULL;
	
	
	switch(UsrSymCmd.CommandId)
	{
    case CMD_DEBUG_UNO:
    case CMD_DEBUG_DUE:
    case CMD_DEBUG_TRE:
    case CMD_DEBUG_QUATTRO:
    case CMD_DEBUG_CINQUE:
    case CMD_DEBUG_SEI:
    case CMD_DEBUG_SETTE:
    case CMD_DEBUG_OTTO:
    case CMD_DEBUG_NOVE:
		EseguiDebug(NULL,NULL,NULL);
		break;
		
		// MASAE: una volta eseguito il comando tarsmette ai DLIOO la loro parte.
		// DLIOO: non e' abilitato ad eseguire il coamndo.
		//
    case CMD_PARTITURA_LOAD:
		Scr->GSetMainDrawArea();
		if (strlen(UsrSymCmd.Text)==0) 
		{
			EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","DoCommandImmediate()");
			return ;
		}
		
		if (Load(UsrSymCmd.Text)) 
		{
			SetFileLoadName(UsrSymCmd.Text);
			SetFileSaveName(UsrSymCmd.Text);
			SetPathSaveName(GetPathLoadName());
			LoadPindex();
			InitParte();
			GoTop();
			ResetACL();
		}
		else
		{
			EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","DoCommandImmediate()");
			return ;
		}
		
		break;
		
		// MASAE: fa eseguire il comando prima ai DLIOO (attraverso una trasmissione via rete con
		//	codici speciali).
		// DLIOO: non e' abilitato ad eseguire il coamndo.
		//
    case CMD_PARTITURA_END:
		// solo MASAE
		NetInvioComando(UsrSymCmd);
		return;
		
    case CMD_PARTITURA_NEW:
		NetInvioComando(UsrSymCmd);
		Scr->GSetMainDrawArea();
		New();
		GoTop();
		break;
		
	case CMD_PARTITURA_IMPORT_MIDI:
      Scr->GSetMainDrawArea();
      if (strlen(UsrSymCmd.Text)==0) 
      {
        EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","DoCommandImmediate()");
        return ;
      }

	  if (ImportMidi(UsrSymCmd.Text , UsrSymCmd.ArgV[0],UsrSymCmd.ArgV[1]))
      {
        LoadPindex();
        InitParte();
        GoTop();
		ResetACL(); 
      }
      else
      {
        EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","DoCommandImmediate()");
        return ;
      }

      break;
	case CMD_PARTITURA_EXPORT_MIDI:
      Scr->GSetMainDrawArea();
      if (strlen(UsrSymCmd.Text)==0) 
      {
        EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","DoCommandImmediate()");
        return ;
      }


      if (!ExportMidi(UsrSymCmd.Text , UsrSymCmd.ArgV[0]))
      {
        EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","DoCommandImmediate()");
        return ;
      }

      break;

    case CMD_PARTITURA_IMPAGINA:
		if (GetFirstSpa()->GetBattNum(0)->GetMetronomoTimeExec()<=0)
		{
			Message("Metronome not present: impossible to continue!","Error!");
			return;
		}
		pMusicWin->InitImpaginazione();
		break;

    case CMD_PARTITURA_INTERROMPI:

	  	  StopMmExec( );
	    
      pMusicWin->AltScrolling();
      break;
      	
    case CMD_PARTITURA_JUMP:
		if ( (UsrSymCmd.ArgV[0] > GetFirstSpa()->GetNumeroBattute())
			|| (UsrSymCmd.ArgV[0] < 1) )
			Message("Measure not present!");
		else
		{
			ReadETT(NULL);
			pMusicWin->JumpToBatt((int)UsrSymCmd.ArgV[0]);
			SetStatoEsecuzione(ESECUZIONE);
			
			if(UsrSymCmd.ArgV[0]==1)
				beginBlock=0;
			else
				beginBlock = measureSync[ UsrSymCmd.ArgV[0]-1 ];//selmi
		}
		
		break;
		
    case CMD_PARTITURA_INITESECUZIONE:
	  if (GetFirstSpa()->GetBattNum(0)->GetMetronomoTimeExec()<=0)
	  {
	    Message("Metronome not present: impossible to continue!","Error!");
    	return;
	  }

	  // SCROLLING
      pMusicWin->InitImpaginazione();

/*
	  strcpy(filesPath,smrgetenv("SMR_FILES_PATH"));

      s=strstr(nome,".mds");
       
      if (s!=NULL) (*(s))='\0';
      
	  strcat(filesPath,GetPathSaveName());
	  strcat(nome,".ett");

	  if(mmExecMode==MM_SYNC)
		  firstItem="Default from metronome indication";
	  else
		  firstItem="Midi generation";

	  afile="";
	  if(mmExecMode==MM_SYNC)
		  afile=audioFile;
      if (WReadETT(firstItem,afile,buffer,nome,BUFFER_LENGTH))
	  {
			strcpy(nome,MusicEditor->smrf->GetPath()+PATH_SEP);

			strcat(nome,buffer);

      }
*/	  
	  pMusicWin->InitEsecuzione(NULL);
	  pMusicWin->CursorEsecuzione();
      break;

    case CMD_PARTITURA_AVVIOESECUZIONE:
      StartMmExec( );
      break;

    case CMD_PARTITURA_PAUSA:
	  PauseMmExec();
      break;

    case CMD_PARTITURA_PAGINAZIONE:
      pMusicWin->AttivaModEsecuzione();    
      break;

    case CMD_PARTITURA_DECELLERA:
		SlowerMmExec();
      break;
    
    case CMD_PARTITURA_ACCELLERA:
		FasterMmExec();
      break;
	
    case CMD_PARTITURA_JUSTIFY:
		int daBattuta;
		int aBattuta;
		TipoGiust tipogiustifparameter;
		double kGiustparameter;
		// chiama la dialog 
		
		if(WJustify(listaSpa->GetNumeroBattute(),&daBattuta, &aBattuta, &tipogiustifparameter, &kGiustparameter))
		{                     
			
			listaSpa->GiustificaDaA(daBattuta,aBattuta,tipogiustifparameter,kGiustparameter);
			Redraw(); 
		}
		break;
		
		
	case CMD_PARTITURA_JOINING_PART:
		JoiningParts(UsrSymCmd.ArgV[0],UsrSymCmd.ArgV[1],UsrSymCmd.ArgV[2]);
		break;
		
	case CMD_PARTITURA_MERGING_PART:
		MergingParts(UsrSymCmd.ArgV[0],UsrSymCmd.ArgV[1]);
		break;
		
		
	case CMD_PARTITURA_TRANSPOSE:
		{
			int clef=-1;				//intero
			int partnumber=0;			//intero
			wxString measures;			//stringa
			int	translation=0;			//intero
			int interval=0;
			int up=0;
			int current_Score;
			int numofstaff=0;
			Bool sharps=FALSE;
			Bool adjust=FALSE;
			
			current_Score=listaSpa->GetCurrentScore();
			if(TranspositionsDlg(&clef, &measures, &translation, &interval, &up, &current_Score, &numofstaff, this, &sharps, &adjust))
			{
				int daBat, aBat;
				if(measures.Find('-')==-1)			
				{
					daBat=aBat=atoi((const char*)measures.c_str());
					if(daBat==0)
						daBat=1;
				}
				else
				{
					daBat	=atoi((const char*)measures.BeforeFirst('-').c_str());
					aBat	=atoi((const char*)measures.AfterFirst('-').c_str());
				}
				listaSpa->Transpose(daBat,aBat,current_Score,clef,translation,interval, up, numofstaff, sharps, adjust);
				pMusicWin->Redraw();
			}
		}
		break;
		
		
    default: 
		EM.Error(GENERAL_ERR_AREA,"EC_CMD_GESTIONE_ERRATA","Partitura","DoCommandImmediate()");
		break;
  }
  
  Scr->GetMouse()->NoRectSel();
  listaSel.Clear();
  UsrSymCmd.SelectionType = NO_SEL;
  
  
}

//*BIAN-VERR*

void Partitura::ResetGenericCommand(SymCmd SC) 
{ 
	short i;
	
	DeselectAll();
	// termina la selezione  
	Scr->GetMouse()->NoRectSel();
	listaSel.Clear();
	
	SC.SelectionType = NO_SEL;
	SC.SelObjType = ANY_SEL;
	ResetPath(SC);
	for(i=0;i<SYM_MAXARG;i++)
	{
		SC.ArgV[i] = 0;
	}
	for(i=0;i<PATH_LENGTH;i++)
	{
		SC.Path1.Array[i] = 0;
	}
	SC.Text[0] = '\0';  
};

Bool Partitura::CreateNetInsert(const Point& p)
{
	Spartito *psp;
	DrawObject *obj;
	Bool QuestaBat, sgnAlt;
	short staff;
	
	if (GetCurrentScore()==0)
		psp = GetFirstSpa();
	else
		psp = GetSpartNum(GetCurrentScore()-1);
	
	do
	{
		obj=NULL;
		// chiede allo spartito se in p ci sono oggetti da selezionare
		ResetPath(UsrSymCmd);
		psp->Hit(p,UsrSymCmd.SelObjType,obj,UsrSymCmd.Path1);
		if(obj==NULL)
		{
			psp = GetNextSpa(psp);    
			continue;
		}
		
		
		// : LA FUNZIONE RICHIAMATA ELIMINA I GRUPPI NELLA
		// BATTUTA. FARE IN MODO CHE LO STESSO ACCADA ANCHE NELLO SLAVE.
		((Battuta*)obj)->InsertHit(p,(short)UsrSymCmd.ArgV[1]-1,(Bool)UsrSymCmd.ArgV[3],QuestaBat,UsrSymCmd.ArgV[0],
			sgnAlt,UsrSymCmd.ArgV[4],UsrSymCmd.ArgV[5],staff);
		
		UsrSymCmd.ArgV[7] = sgnAlt;
		UsrSymCmd.ArgV[9] = staff;
		
		
		
	
		if(QuestaBat) // L'unica battuta effettivamente colpita.
		{		

			if ((obj->GetSymCode(UsrSymCmd)==CMD_OK)) 
			{
				CopiaCommandStruct(&Comando,&UsrSymCmd);
				/*
				UsrSymCmd.SelectionType	=	IMMEDIATE_SEL;
				UsrSymCmd.CommandId		=	CMD_DEL_SYMBOL;
				UsrSymCmd.SelObjType	=	ANY_SEL;
				UsrSymCmd.Path1.Array[3]=	UsrSymCmd.ArgV[1];
				UsrSymCmd.Path1.Array[4]=	UsrSymCmd.ArgV[6];
				*/
				CopiaCommandStruct(&AntiComando,&UsrSymCmd);
				
				AntiComando.SelectionType	=	IMMEDIATE_SEL;
				AntiComando.CommandId		=	CMD_DEL_SYMBOL;
				AntiComando.SelObjType		=	ANY_SEL;
				AntiComando.Path1.Array[3]	=	UsrSymCmd.ArgV[1];
				AntiComando.Path1.Array[4]	=	UsrSymCmd.ArgV[6];

				return TRUE;
			}
			else
				return FALSE;
		}

	
		psp = GetNextSpa(psp);    
	} while(psp!=NULL && GetCurrentScore()==0);  //perch cicla?
	

	return TRUE;

}

Bool Partitura::CreateNetImmediate(const Point& p)
{
	DrawObject *obj, *spobj;
	
	Battuta *PBattuta;
	Spartito *psp;
	
	// chiede allo spartito se in p ci sono oggetti da selezionare
	switch (UsrSymCmd.CommandId) //BATTUTA_SEL
	{
		case CMD_BATTUTA_CVIOLINO:
		case CMD_BATTUTA_CVIOLINO8:
		case CMD_BATTUTA_CVIOLINO8SOPRA:
		case CMD_BATTUTA_CBASSO:
		case CMD_BATTUTA_CBASSO8:
		case CMD_BATTUTA_CBASSO8SOPRA:
		case CMD_BATTUTA_CBASSOOLD:
		case CMD_BATTUTA_CBARITONO:
		case CMD_BATTUTA_CTENORE:
		case CMD_BATTUTA_CTENORE8:
		case CMD_BATTUTA_CCONTRALTO:
		case CMD_BATTUTA_CMEZZOSOPRANO:
		case CMD_BATTUTA_CSOPRANO:
		case CMD_BATTUTA_CPERCUSBOX:
		case CMD_BATTUTA_CPERCUS2LINES:
		case CMD_BATTUTA_CTAB:
		case CMD_BATTUTA_CVUOTA:
		{	
			ResetPath(UsrSymCmd);
			Hit(p,BATTUTA_SEL,spobj,UsrSymCmd.Path1);
			if(spobj==NULL)
			{
				return FALSE;
			}
			PBattuta = ((Battuta*)spobj);
			
			SymbolicHit(UsrSymCmd.Path1,SPARTITO_SEL,spobj);
			psp =  (Spartito*)spobj;
			if(psp==NULL)
			{
				return FALSE;
			}
					
			obj = PBattuta->GetPLayer(0)->GetFirstF();
			if (obj != NULL)
			{
				if (p.x > obj->GetAbsPos().x) 
				{	
					//SymCmd temp;
					//CopiaCommandStruct(&temp,&UsrSymCmd);
					
					SetCommand(CMD_BATTUTA_INS_SPAZIO, INSERT_SEL, BATPART_SEL,
						0,
						UsrSymCmd.ArgV[1],
						UsrSymCmd.CommandId);//???
					
					CreateNetInsert(p);			

					return TRUE;
				}
			}
			else
			{
				CopiaCommandStruct(&Comando,&UsrSymCmd);
				CopiaCommandStruct(&AntiComando,&UsrSymCmd);
				Chiave* Old = PBattuta->GetChiave(UsrSymCmd.Path1.Array[STAFF_PATH_LEVEL]);
				AntiComando.CommandId = (CmdID)CLtoCMD(Old->GetID());

				return TRUE;
			}

			break;
		}

		case CMD_BATTUTA_ARMCHV_0D:
		case CMD_BATTUTA_ARMCHV_1D:
		case CMD_BATTUTA_ARMCHV_2D:
		case CMD_BATTUTA_ARMCHV_3D:
		case CMD_BATTUTA_ARMCHV_4D:
		case CMD_BATTUTA_ARMCHV_5D:
		case CMD_BATTUTA_ARMCHV_6D:
		case CMD_BATTUTA_ARMCHV_7D:
		case CMD_BATTUTA_ARMCHV_1B:
		case CMD_BATTUTA_ARMCHV_2B:
		case CMD_BATTUTA_ARMCHV_3B:
		case CMD_BATTUTA_ARMCHV_4B:
		case CMD_BATTUTA_ARMCHV_5B:
		case CMD_BATTUTA_ARMCHV_6B:
		case CMD_BATTUTA_ARMCHV_7B:
		case CMD_BATTUTA_ARMCHV_0Dm:
		case CMD_BATTUTA_ARMCHV_1Dm:
		case CMD_BATTUTA_ARMCHV_2Dm:
		case CMD_BATTUTA_ARMCHV_3Dm:
		case CMD_BATTUTA_ARMCHV_4Dm:
		case CMD_BATTUTA_ARMCHV_5Dm:
		case CMD_BATTUTA_ARMCHV_6Dm:
		case CMD_BATTUTA_ARMCHV_7Dm:
		case CMD_BATTUTA_ARMCHV_1Bm:
		case CMD_BATTUTA_ARMCHV_2Bm:
		case CMD_BATTUTA_ARMCHV_3Bm:
		case CMD_BATTUTA_ARMCHV_4Bm:
		case CMD_BATTUTA_ARMCHV_5Bm:
		case CMD_BATTUTA_ARMCHV_6Bm:
		case CMD_BATTUTA_ARMCHV_7Bm:
		{
			ResetPath(UsrSymCmd);
			Hit(p,BATTUTA_SEL,spobj,UsrSymCmd.Path1);
			if(spobj==NULL)
			{
				return FALSE;
			}
			PBattuta = ((Battuta*)spobj);
			
			SymbolicHit(UsrSymCmd.Path1,SPARTITO_SEL,spobj);
			psp =  (Spartito*)spobj;
			if(psp==NULL)
			{
				return FALSE;
			}
			
			
			obj = PBattuta->GetPLayer(0)->GetFirstF();
			if (obj != NULL)
			{
				if (p.x > obj->GetAbsPos().x) 
				{					
					SetCommand(CMD_BATTUTA_INS_SPAZIO,INSERT_SEL,BATPART_SEL,0,1,UsrSymCmd.CommandId);
					
					CreateNetInsert(p);			

					return TRUE;
				}
			}
			else
			{
				CopiaCommandStruct(&Comando,&UsrSymCmd);
				CopiaCommandStruct(&AntiComando,&UsrSymCmd);
				tipoArmaturaChiave Old = PBattuta->GetArmatura(UsrSymCmd.Path1.Array[STAFF_PATH_LEVEL]);
				AntiComando.CommandId = (CmdID)CLtoCMD(Old);

				return TRUE;
			}

			break;
		}
		case CMD_BATTUTA_TEMPO:
		{
			ResetPath(UsrSymCmd);
			Hit(p,BATTUTA_SEL,spobj,UsrSymCmd.Path1);
			if(spobj==NULL)
			{
				return FALSE;
			}
			PBattuta = ((Battuta*)spobj);
			
			SymbolicHit(UsrSymCmd.Path1,SPARTITO_SEL,spobj);
			psp =  (Spartito*)spobj;
			if(psp==NULL)
			{
				return FALSE;
			}
			CopiaCommandStruct(&Comando,&UsrSymCmd);
			CopiaCommandStruct(&AntiComando,&UsrSymCmd);
			Tempo time = PBattuta->GetTempo();
			if(!time.GetTempo(AntiComando.Text))
				return FALSE;

			return TRUE;
			//break;
		}

		case CMD_NOTA_PIZZICATO:
		case CMD_NOTA_ARCO:
		case CMD_ARCO_ARCOSU:
		case CMD_ARCO_ARCOGIU:
		case CMD_ARCO_PONTICELLO:
		case CMD_ARCO_TASTIERA:
		case CMD_ARCO_PUNTA:
		case CMD_ARCO_TALLONE:
		case CMD_NOTA_CORDA_0:
		case CMD_NOTA_CORDA_1:
		case CMD_NOTA_CORDA_2:
		case CMD_NOTA_CORDA_3:
		case CMD_NOTA_CORDA_4:
		case CMD_NOTA_CORDA_5:
		case CMD_NOTA_CORDA_6:
		case CMD_NOTA_CORDA_7:
		case CMD_NOTA_TRILLO:
		case CMD_NOTA_GRUPPETTO_SUP_1:
		case CMD_NOTA_GRUPPETTO_INF_1:
		case CMD_NOTA_TURN_SLASH:
		case CMD_NOTA_TURN_UP:
		case CMD_NOTA_MORDENTE_SUP:
		case CMD_NOTA_MORDENTE2_SUP:
		case CMD_NOTA_MORDENTE_INF:
		case CMD_NOTA_MORDENTE2_INF:
		case CMD_NOTA_SLIDING_UP:
		case CMD_NOTA_SLIDING_DOWN:
		case CMD_NOTA_SLIDING_DUP:
		case CMD_NOTA_SLIDING_DOWNUP:
		case CMD_NOTA_NONGHYN_INC:
		case CMD_NOTA_NONGHYN_DEC:
		case CMD_NOTA_NONGHYN_ARROW:
		case CMD_NOTA_NONGHYN_WIDE:
		case CMD_NOTA_DITO_0:
		case CMD_NOTA_DITO_1:
		case CMD_NOTA_DITO_2:
		case CMD_NOTA_DITO_3:
		case CMD_NOTA_DITO_4:
		case CMD_NOTA_DITO_5:
		case CMD_NOTA_TENUTO:
		case CMD_NOTA_PUNTOALL:
		case CMD_NOTA_SFORZATO:
		case CMD_NOTA_ACCENTO:
		case CMD_NOTA_MARTELLATO:
		case CMD_NOTA_MARTDOLCE:
		case CMD_NOTA_STACCATO:
		case CMD_NOTA_ESPGENERIC:
		case CMD_NOTA_CONSORDINAOTTONI:
		case CMD_NOTA_CONSORDINAARCHI:
		case CMD_NOTA_CONSORDINATESTO:
		case CMD_NOTA_VIASORDINAOTTONI:
		case CMD_NOTA_VIASORDINAARCHI:
		case CMD_NOTA_VIASORDINATESTO:
		case CMD_NOTA_ARMONICIOTTONI:
		case CMD_NOTA_ARMONICIARCHI:
		case CMD_FIGURA_1PUNTO:
		case CMD_FIGURA_2PUNTI:
		case CMD_FIGURA_3PUNTI:    
		{
			  // chiede allo spartito se in p ci sono oggetti da selezionare
			ResetPath(UsrSymCmd);
			Hit(p,UsrSymCmd.SelObjType,obj,UsrSymCmd.Path1);

			Battuta* pBat=NULL;
			
			// viene determinata la battuta dell'oggetto per sapere se si e' in multivoice
			SymbolicHit(UsrSymCmd.Path1,BATTUTA_SEL,(DrawObject*&)pBat);
			if(pBat!=NULL)
			{
				UsrSymCmd.ArgV[2]=pBat->GetMultiVoice(); // verificare cosa mettere al posto di 0 (layer)
				CopiaCommandStruct(&Comando,&UsrSymCmd);
				CopiaCommandStruct(&AntiComando,&UsrSymCmd);

				AntiComando.CommandId		=	CMD_DEL_SYMBOL;
				AntiComando.SelObjType		=	ANY_SEL;

				AntiComando.Path1.Array[7] = CMDtoCL(UsrSymCmd);
				if(AntiComando.Path1.Array[7] == -1)
					return FALSE;

				return TRUE;
			}
			
			return FALSE;
		}

	//(like default)
		case CMD_BATTUTA_SEGNO:
		case CMD_BATTUTA_CODA:
		case CMD_BATTUTA_TSALTO:
		{
			ResetPath(UsrSymCmd);
			Hit(p,BATTUTA_SEL,spobj,UsrSymCmd.Path1);

			CopiaCommandStruct(&Comando,&UsrSymCmd);
			CopiaCommandStruct(&AntiComando,&UsrSymCmd);

			AntiComando.CommandId		=	CMD_DEL_SYMBOL;
			AntiComando.SelObjType		=	ANY_SEL;

			AntiComando.Path1.Array[7] = CMDtoCL(UsrSymCmd);
			if(AntiComando.Path1.Array[7] == -1)
				return FALSE;

			return TRUE;
		}
	
		case CMD_NOTA_DIESIS:
		case CMD_NOTA_DIESIS1Q:
		case CMD_NOTA_DIESIS3Q:
		case CMD_NOTA_BEMOLLE:			
		case CMD_NOTA_BEMOLLE1Q:
		case CMD_NOTA_BEMOLLE3Q:
		case CMD_NOTA_BEQUADRO:			
		case CMD_NOTA_BEQUADRO1Q:
		case CMD_NOTA_BEQUADRO3Q:
		case CMD_NOTA_2DIESIS:			
		case CMD_NOTA_2BEMOLLE:				
		{
			ResetPath(UsrSymCmd);
			Hit(p,ANY_SEL,obj,UsrSymCmd.Path1);		

			CopiaCommandStruct(&Comando,&UsrSymCmd);
			CopiaCommandStruct(&AntiComando,&UsrSymCmd);

			//AntiComando.SelectionType	=	IMMEDIATE_SEL;
			AntiComando.CommandId		=	CMD_DEL_SYMBOL;
			AntiComando.SelObjType		=	ANY_SEL;
			
			AntiComando.Path1.Array[7] = CMDtoCL(UsrSymCmd);
			if(AntiComando.Path1.Array[7] == -1)
				return FALSE;

			return TRUE;
		}

		case CMD_DEL_SYMBOL:
		{
			ResetPath(UsrSymCmd);
			Hit(p,UsrSymCmd.SelObjType,obj,UsrSymCmd.Path1);

			if(obj == NULL)	return FALSE;

			ResetGenericCommand(Comando);
			ResetGenericCommand(AntiComando);

			//int i=0;
			ClassID CLid = obj->GetID();

			int id_group = CLtoGROUP(CLid);
			if (id_group == -1)
				return FALSE;

			int id_cmd;
			if (CLid == CL_ARMATURACHIAVE)
				id_cmd = CLtoCMD(((ArmaturaChiave*)obj)->GetArmatura()) ;
			else
				id_cmd = CLtoCMD(CLid);

			if (id_cmd == -1)
				return FALSE;


			switch (id_group)
			{
				case NOTE_GROUP:
				{
					CopiaCommandStruct(&Comando,&UsrSymCmd);
					Nota* pNota = (Nota*)obj;

					SetCommand((CmdID)id_cmd, INSERT_SEL, BATTUTA_SEL,
						abs(pNota->GetAltezza()),//0
						Comando.Path1.Array[LAYER_PATH_LEVEL],	//1 layer
						AUTOMATICO,		//2 GamboUp
						FALSE,			//3 inAccordo
						0,				//4
						"",
						1,				//5 insInTesta
						0,				//6
						(pNota->GetAltezza() < 0 ? 0 : 1),//7
						pNota->GetHeadType());//8 ???

					UsrSymCmd.Path1 = Comando.Path1;

					Layer *pLay = pNota->GetPBattuta()->GetPLayer(Comando.Path1.Array[LAYER_PATH_LEVEL]-1);
					Figura	*pFigPrev, *pFig = pLay->GetFirstFig();
					while (pFig != NULL)
					{
						pFigPrev = pFig;
						pFig = pLay->GetNextFig();
						if(pFig == ((Figura*)obj))
						{
							UsrSymCmd.ArgV[4] = pFigPrev->GetNumericCode();
							UsrSymCmd.ArgV[6] = pFig->GetNumericCode();
							UsrSymCmd.ArgV[5] = 0;

							pFig = NULL;
						}
					}

					CopiaCommandStruct(&AntiComando,&UsrSymCmd);

					return TRUE;
				}
				case REST_GROUP:
				{
					CopiaCommandStruct(&Comando,&UsrSymCmd);
					Pausa* pPausa = (Pausa*)obj;

					SetCommand((CmdID)id_cmd, INSERT_SEL, BATTUTA_SEL,
						abs(pPausa->GetAltezza()),
						Comando.Path1.Array[LAYER_PATH_LEVEL],	//layer
						AUTOMATICO,		//GamboUp
						FALSE,			//inAccordo
						0,		
						"",
						1,				//insInTesta
						0,
						(pPausa->GetAltezza() < 0 ? 0 : 1));//???

					UsrSymCmd.Path1 = Comando.Path1;

					Layer *pLay = pPausa->GetPBattuta()->GetPLayer(Comando.Path1.Array[LAYER_PATH_LEVEL]-1);
					Figura	*pFigPrev, *pFig = pLay->GetFirstFig();
					while (pFig != NULL)
					{
						pFigPrev = pFig;
						pFig = pLay->GetNextFig();
						if(pFig == ((Figura*)obj))
						{
							UsrSymCmd.ArgV[4] = pFigPrev->GetNumericCode();
							UsrSymCmd.ArgV[6] = pFig->GetNumericCode();
							UsrSymCmd.ArgV[5] = 0;

							pFig = NULL;
						}
					}

					CopiaCommandStruct(&AntiComando,&UsrSymCmd);

					return TRUE;
				}

				case CLEF_GROUP:
				case KEYSIGN_GROUP:
				{
					if (UsrSymCmd.Path1.Array[LAYER_PATH_LEVEL] > 0)
					{
						CopiaCommandStruct(&Comando,&UsrSymCmd);
						Spazio* pSpazio = (Spazio*)obj;

						SetCommand(CMD_BATTUTA_INS_SPAZIO,INSERT_SEL,BATPART_SEL,0,1,(CmdID)id_cmd);

						UsrSymCmd.Path1 = Comando.Path1;

						Layer *pLay = pSpazio->GetPBattuta()->GetPLayer(Comando.Path1.Array[LAYER_PATH_LEVEL]-1);
						Figura	*pFigPrev, *pFig = pLay->GetFirstFig();
						while (pFig != NULL)
						{
							pFigPrev = pFig;
							pFig = pLay->GetNextFig();
							if(pFig == ((Figura*)obj))
							{
								UsrSymCmd.ArgV[4] = pFigPrev->GetNumericCode();
								UsrSymCmd.ArgV[6] = pFig->GetNumericCode();

								pFig = NULL;
							}
						}

						CopiaCommandStruct(&AntiComando,&UsrSymCmd);

						return TRUE;
					}
				}
				default:
					CopiaCommandStruct(&Comando, &UsrSymCmd);
					CopiaCommandStruct(&AntiComando,&UsrSymCmd);

					AntiComando.CommandId		=	(CmdID)id_cmd;

					AntiComando.Path1.Array[7]			=	0;

					return TRUE;
			}	
		}

		//------------------------------------------------------------------------------------------
		case CMD_PARTITURA_ADD_BATTUTA :
		{
			ResetPath(UsrSymCmd);
			Hit(p,UsrSymCmd.SelObjType,spobj,UsrSymCmd.Path1);

			if (!spobj)
				return FALSE;

			Hit(p,SPARTITO_SEL,(DrawObject*&)psp,UsrSymCmd.Path1);

			CopiaCommandStruct(&Comando,&UsrSymCmd);
			CopiaCommandStruct(&AntiComando,&UsrSymCmd);

			Comando.ArgV[2] = psp->GetCodeCounterBat()/* + 1*/;

			AntiComando.CommandId		=	CMD_PARTITURA_DEL_BATTUTA;

			AntiComando.Path1.Array[2] = Comando.ArgV[2];

			return TRUE;
		}

		case CMD_PARTITURA_DEL_BATTUTA :
		{
			ResetPath(UsrSymCmd);
			Hit(p,UsrSymCmd.SelObjType,spobj,UsrSymCmd.Path1);

			if (!spobj)
				return FALSE;

			Hit(p,SPARTITO_SEL,(DrawObject*&)psp,UsrSymCmd.Path1);

			CopiaCommandStruct(&Comando,&UsrSymCmd);
			CopiaCommandStruct(&AntiComando,&UsrSymCmd);

			AntiComando.CommandId		=	CMD_PARTITURA_ADD_BATTUTA;
		
			AntiComando.ArgV[1] = 1;	//AFTER
			AntiComando.ArgV[1] = Comando.Path1.Array[2];
			
			Battuta* pBat = psp->GetPrevBat((Battuta*)spobj);

			if (!pBat)
				return FALSE;

			AntiComando.Path1.Array[2] = pBat->GetNumericCode(); // Misura precedente
			

			//AntiComando.SelObjType		=	ANY_SEL;

			//AntiComando.Path1.Array[7] = CMDtoCL(UsrSymCmd);
			//if(AntiComando.Path1.Array[7] == -1)
			//	return FALSE;
			
			return TRUE;
		}
		//------------------------------------------------------------------------------------------

		default:
		{
			if(UsrSymCmd.Path1.Array[0]!=0)
				return TRUE;
			else 
				return FALSE;
		}
	}
	return FALSE;
}

bool Partitura::CreateNetCommand(const Point& p, char* cmd, char* anti)
{
	bool goon = false;
	Network rete;

	
	switch(UsrSymCmd.SelectionType)
	{
		case INSERT_SEL:
			if (CreateNetInsert(p))
			{	
				goon = true;
			}
			break;

		case IMMEDIATE_SEL:
			if (CreateNetImmediate(p))
			{	
//				if (anticmd_nota!=true)
//					CopiaCommandStruct(&Comando,&UsrSymCmd);
				
				goon = true;
			}
				
			break;

		default:
			break;
	}

	if (!goon) return false;

	if (!rete.Sym2String(cmd,Comando))
		return false;
	if (!rete.Sym2String(anti,AntiComando))
		return false;

	return true;
}

Bool Partitura::DoNetInsert(SymCmd SC)
{
	DrawObject *pObj;

	SymbolicHit(SC.Path1, SC.SelObjType, pObj);
	if(pObj==NULL)
	{
		EM.Error(MUSICAL_ERR_AREA,"EC_OGG_NOTFOUND","Partitur","NetInsert()");
		return FALSE;
	}
	
	if (((Battuta*)pObj)->SymCommand(SC)==CMD_OK)
	{
		SetStatoEsecuzione(IMPAGINAZIONE);      
		return TRUE;
	}
	else
	{
		EM.Error(GENERAL_ERR_AREA,"EC_CMDNET_FAULT","Partitur","NetInsert()");
		return FALSE;
	}
		
}

Bool Partitura::DoNetImmediate(SymCmd SC)
{
	DrawObject *obj;
	Battuta *pBattuta;
	Spartito *pSpartito;
	int staff;
	ClassID ChvID;
	tipoArmaturaChiave ArmCh;
	
	switch(SC.CommandId)
	{
		case CMD_BATTUTA_CVIOLINO:
		case CMD_BATTUTA_CVIOLINO8:
		case CMD_BATTUTA_CVIOLINO8SOPRA:
		case CMD_BATTUTA_CBASSO:
		case CMD_BATTUTA_CBASSO8:
		case CMD_BATTUTA_CBASSO8SOPRA:
		case CMD_BATTUTA_CBASSOOLD:
		case CMD_BATTUTA_CBARITONO:
		case CMD_BATTUTA_CTENORE:
		case CMD_BATTUTA_CTENORE8:
		case CMD_BATTUTA_CCONTRALTO:
		case CMD_BATTUTA_CMEZZOSOPRANO:
		case CMD_BATTUTA_CSOPRANO:
		case CMD_BATTUTA_CPERCUSBOX:
		case CMD_BATTUTA_CPERCUS2LINES:
		case CMD_BATTUTA_CTAB:
		case CMD_BATTUTA_CVUOTA:
		{	
			SymbolicHit(SC.Path1,BATTUTA_SEL,(DrawObject*&)pBattuta);
			SymbolicHit(SC.Path1,SPARTITO_SEL,(DrawObject*&)pSpartito);
			
			if(pSpartito==NULL)
			{
				//EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","ImmediateSel()");
				return FALSE;
			}

			//FFF
			staff = SC.Path1.Array[STAFF_PATH_LEVEL];

			if (pBattuta!=NULL)
			{
				ChvID = pBattuta->GetChiave(staff)->GetID();
				while (pBattuta!=NULL)
				{
					pBattuta->SymCommand(SC);
					pBattuta = pSpartito->GetNextBat(pBattuta);
					if (pBattuta!=NULL)
						if (pBattuta->GetChiave(staff)->GetID() != ChvID)
							pBattuta = NULL;
				}
			}
			return TRUE;
			//FFF

			break;
		}
		
		case CMD_BATTUTA_ARMCHV_0D:
		case CMD_BATTUTA_ARMCHV_1D:
		case CMD_BATTUTA_ARMCHV_2D:
		case CMD_BATTUTA_ARMCHV_3D:
		case CMD_BATTUTA_ARMCHV_4D:
		case CMD_BATTUTA_ARMCHV_5D:
		case CMD_BATTUTA_ARMCHV_6D:
		case CMD_BATTUTA_ARMCHV_7D:
		case CMD_BATTUTA_ARMCHV_1B:
		case CMD_BATTUTA_ARMCHV_2B:
		case CMD_BATTUTA_ARMCHV_3B:
		case CMD_BATTUTA_ARMCHV_4B:
		case CMD_BATTUTA_ARMCHV_5B:
		case CMD_BATTUTA_ARMCHV_6B:
		case CMD_BATTUTA_ARMCHV_7B:
		case CMD_BATTUTA_ARMCHV_0Dm:
		case CMD_BATTUTA_ARMCHV_1Dm:
		case CMD_BATTUTA_ARMCHV_2Dm:
		case CMD_BATTUTA_ARMCHV_3Dm:
		case CMD_BATTUTA_ARMCHV_4Dm:
		case CMD_BATTUTA_ARMCHV_5Dm:
		case CMD_BATTUTA_ARMCHV_6Dm:
		case CMD_BATTUTA_ARMCHV_7Dm:
		case CMD_BATTUTA_ARMCHV_1Bm:
		case CMD_BATTUTA_ARMCHV_2Bm:
		case CMD_BATTUTA_ARMCHV_3Bm:
		case CMD_BATTUTA_ARMCHV_4Bm:
		case CMD_BATTUTA_ARMCHV_5Bm:
		case CMD_BATTUTA_ARMCHV_6Bm:
		case CMD_BATTUTA_ARMCHV_7Bm:
			{
				SymbolicHit(SC.Path1,BATTUTA_SEL,(DrawObject*&)pBattuta);
				SymbolicHit(SC.Path1,SPARTITO_SEL,(DrawObject*&)pSpartito);
				
				if(pSpartito==NULL)
				{
					//EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","ImmediateSel()");
					return FALSE;
				}

				//FFF
				if (pBattuta==NULL)
					return FALSE;
				
				//if(pBattuta->SymCommand(SC)!=CMD_OK) 
				//	return FALSE;

				staff = SC.Path1.Array[STAFF_PATH_LEVEL];
				
				//pBattuta = pSpartito->GetNextBat(pBattuta);
				if (pBattuta!=NULL)
				{
					ArmCh = pBattuta->GetArmatura(staff);
					ChvID = pBattuta->GetChiave(staff)->GetID();
					while (pBattuta!=NULL)
					{
						pBattuta->SymCommand(SC);
						pBattuta = pSpartito->GetNextBat(pBattuta);
						if (pBattuta!=NULL)
							if ((pBattuta->GetChiave(staff)->GetID() != ChvID) || (pBattuta->GetArmatura(staff) != ArmCh))
								pBattuta = NULL;
					}
				}
				return TRUE;
				//FFF


				/* CONTROLLARE
				staff=0; 
				
				obj = pBattuta->GetPLayer(0)->GetFirstF();
				if (obj != NULL)
				{
					SymbolicHit(SC.Path2,FIGURA_SEL,spobj);
					if(spobj==NULL)
					{
						EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","ImmediateSel()");
						return FALSE;
					}
					
					if (!(pBattuta->GetChiaveFig(staff,(Figura*)spobj)))
					{
						pBattuta = pSpartito->GetNextBat(pBattuta);
						if (pBattuta!=NULL)
						{
							ArmCh = pBattuta->GetArmatura(staff);
							ChvID = pBattuta->GetChiave(staff)->GetID();
							while (pBattuta!=NULL)
							{
								pBattuta->SymCommand(SC);
								pBattuta = pSpartito->GetNextBat(pBattuta);
								if (pBattuta!=NULL)
									if ((pBattuta->GetChiave(staff)->GetID() != ChvID) || 
										(pBattuta->GetArmatura(staff) != ArmCh))
										pBattuta = NULL;
							}
						}
					}
					return TRUE;
				}
				obj=NULL;
				
				// viene determinata la battuta dell'oggetto per sapere se si e' in multivoice
				SymbolicHit(SC.Path1,SPARTITO_SEL,obj);
				if(obj!=NULL)
					if (((Spartito *)obj)->SymCommand(SC)==CMD_OK) 
						return TRUE;
	
				return FALSE;*/
				break;
			}

		case CMD_NOTA_PIZZICATO:
		case CMD_NOTA_ARCO:
		case CMD_ARCO_ARCOSU:
		case CMD_ARCO_ARCOGIU:
		case CMD_ARCO_PONTICELLO:
		case CMD_ARCO_TASTIERA:
		case CMD_ARCO_PUNTA:
		case CMD_ARCO_TALLONE:
		case CMD_NOTA_CORDA_0:
		case CMD_NOTA_CORDA_1:
		case CMD_NOTA_CORDA_2:
		case CMD_NOTA_CORDA_3:
		case CMD_NOTA_CORDA_4:
		case CMD_NOTA_CORDA_5:
		case CMD_NOTA_CORDA_6:
		case CMD_NOTA_CORDA_7:
		case CMD_NOTA_TRILLO:
		case CMD_NOTA_GRUPPETTO_SUP_1:
		case CMD_NOTA_GRUPPETTO_INF_1:
		case CMD_NOTA_TURN_SLASH:
		case CMD_NOTA_TURN_UP:
		case CMD_NOTA_MORDENTE_SUP:
		case CMD_NOTA_MORDENTE2_SUP:
		case CMD_NOTA_MORDENTE_INF:
		case CMD_NOTA_MORDENTE2_INF:
		case CMD_NOTA_SLIDING_UP:
		case CMD_NOTA_SLIDING_DOWN:
		case CMD_NOTA_SLIDING_DUP:
		case CMD_NOTA_SLIDING_DOWNUP:
		case CMD_NOTA_NONGHYN_INC:
		case CMD_NOTA_NONGHYN_DEC:
		case CMD_NOTA_NONGHYN_ARROW:
		case CMD_NOTA_NONGHYN_WIDE:
		case CMD_NOTA_DITO_0:
		case CMD_NOTA_DITO_1:
		case CMD_NOTA_DITO_2:
		case CMD_NOTA_DITO_3:
		case CMD_NOTA_DITO_4:
		case CMD_NOTA_DITO_5:
		case CMD_NOTA_TENUTO:
		case CMD_NOTA_PUNTOALL:
		case CMD_NOTA_SFORZATO:
		case CMD_NOTA_ACCENTO:
		case CMD_NOTA_MARTELLATO:
		case CMD_NOTA_MARTDOLCE:
		case CMD_NOTA_STACCATO:
		case CMD_NOTA_ESPGENERIC:
		case CMD_NOTA_CONSORDINAOTTONI:
		case CMD_NOTA_CONSORDINAARCHI:
		case CMD_NOTA_CONSORDINATESTO:
		case CMD_NOTA_VIASORDINAOTTONI:
		case CMD_NOTA_VIASORDINAARCHI:
		case CMD_NOTA_VIASORDINATESTO:
		case CMD_NOTA_ARMONICIOTTONI:
		case CMD_NOTA_ARMONICIARCHI:
		case CMD_FIGURA_1PUNTO:
		case CMD_FIGURA_2PUNTI:
		case CMD_FIGURA_3PUNTI:   
		{		
			// viene determinata la battuta dell'oggetto per sapere se si e' in multivoice
			SymbolicHit(SC.Path1,BATTUTA_SEL,(DrawObject*&)pBattuta);
			if(pBattuta!=NULL)
			{
				SC.ArgV[2]=pBattuta->GetMultiVoice(); 
/*
				obj=pBattuta->GetFiguraInLayer(SC.Path1.Array[3]-1,SC.Path1.Array[4]);					
*/			
				SymbolicHit(SC.Path1,NOTA_SEL,(DrawObject*&)obj);

				if (obj!=NULL)
				{
					if ( ((Nota*)obj)->SymCommand(SC, pBattuta)==CMD_OK ) 
						//questo obj e' una nota.
					{
						return TRUE;
					}
				}
				else
					return FALSE;
			}

			return FALSE;
		}

		case CMD_NOTA_DIESIS:
		case CMD_NOTA_DIESIS1Q:
		case CMD_NOTA_DIESIS3Q:
		case CMD_NOTA_BEMOLLE:
		case CMD_NOTA_BEMOLLE1Q:
		case CMD_NOTA_BEMOLLE3Q:
		case CMD_NOTA_BEQUADRO:
		case CMD_NOTA_BEQUADRO1Q:
		case CMD_NOTA_BEQUADRO3Q:
		case CMD_NOTA_2DIESIS:
		case CMD_NOTA_2BEMOLLE:	
		{
/*			SymbolicHit(SC.Path1,BATTUTA_SEL,(DrawObject*&)pBattuta);

			if(pBattuta == NULL)
				return FALSE;
*/
			SymbolicHit(SC.Path1,SC.SelObjType,(DrawObject*&)obj);

			// se obj e' NULL allora non c'e' stata selezione
			if(obj==NULL)
			{
			  //EM.Error(MUSICAL_ERR_AREA,"EC_OGG_NOTFOUND","Partitura","ImmediateSel()");
			  return FALSE;
			}

			if (obj->SymCommand(SC)==CMD_OK)
			{
				return TRUE;
			}
			else return FALSE;

			break;	
		}
		case CMD_DEL_SYMBOL:
		{ 
			obj=NULL;
			SymbolicHit(SC.Path1,SC.SelObjType,obj);
			if(obj!=NULL)
			{   
				if (DeleteSym((DrawObject*) obj))
					return TRUE;
				else
					return FALSE;
			}
			else 
			{
				EM.Error(MUSICAL_ERR_AREA,"EC_OGG_NOTFOUND","Partitura","NetImmediate()");
				return FALSE;
			}
			
			break;
		}
		case CMD_BATTUTA_TEMPO:
			{
				obj=NULL;
				
				// viene determinata la battuta dell'oggetto per sapere se si e' in multivoice
				SymbolicHit(SC.Path1,SPARTITO_SEL,obj);
				if(obj!=NULL)
					if (((Spartito *)obj)->SymCommand(SC)==CMD_OK) 
						return TRUE;
	
				return FALSE;
			}
			
			//--------------------------------------------------------------------
		case CMD_PARTITURA_ADD_BATTUTA :
		case CMD_PARTITURA_DEL_BATTUTA :
			{
				pSpartito = GetFirstSpa();
				do 
				{
					//ResetPath(UsrSymCmd);
					//pSpartito->Hit(p,UsrSymCmd.SelObjType,pobj,UsrSymCmd.Path1);
					if(SC.CommandId == CMD_PARTITURA_ADD_BATTUTA) {
						if(pSpartito->GetCodeCounterBat() <= SC.ArgV[2]/* - 1*/)
							pSpartito->SetCodeCounterBat(SC.ArgV[2]/* - 1*/); //TODO: da controllare
						else
							return FALSE;
					}

					pSpartito->SymbolicHit(SC.Path1,SC.SelObjType,obj);

					if(obj==NULL)
					{
						EM.Error(MUSICAL_ERR_AREA,"EC_OGG_NOTFOUND","Partitura","DoCommandAddBattuta()");
						return FALSE; 
					}

					if (tipoLeggio==LEGGIO_MASAE)
					{
						if (pSpartito->SymCommand(SC)==CMD_OK)
						{
							//SetStatoEsecuzione(IMPAGINAZIONE);
							//NetInvioComando(UsrSymCmd);
						}
						else
							return FALSE;
					}

					pSpartito = GetNextSpa(pSpartito);
				} while(pSpartito!=NULL);

				return TRUE;
			}

		//--------------------------------------------------------------------

		default:
			{
				Battuta* pBat=NULL;
				
				// viene determinata la battuta dell'oggetto per sapere se si e' in multivoice
				SymbolicHit(SC.Path1,BATTUTA_SEL,(DrawObject*&)pBat);
				if(pBat!=NULL)
					if (pBat->SymCommand(SC)==CMD_OK) 
						return TRUE;
	
				return FALSE;
			}
	  }
}

Bool Partitura::DoNetCommand(SymCmd SC)
{	
	if (SC.CommandId!=CMD_DEBUGMILLA_SEL_SYMBOL) 
		if (1==DebugNormalMillaPos)
		{
			DebugNormalMilla=TRUE;
			DebugNormalMillaPos=2;
		}
	
	switch(SC.SelectionType)
	{
	case NO_SEL:
		return FALSE;
	case INSERT_SEL:
		if (DoNetInsert(SC))
		{
			Redraw();
		}
		else
		{
			EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","DoCommand()");
			return FALSE;
		}
		break;
	case IMMEDIATE_SEL:
		if (DoNetImmediate(SC)) 
		{
			Redraw();
		}
		else
		{
			EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","DoCommand()");
			return FALSE;
		}
		break;

	default: 
		return FALSE;
	}
	
		
	int nScore=GetCurrentScore();
	if(nScore==0) // per il main score determina in qualse spartito  stata fatta la modifica
	{
		Spartito *psp = GetFirstSpa();
		int ns=1;
		while (psp!=NULL) 
		{
			if (psp->GetNumericCode()==SC.Path1.Array[SPARTITO_PATH_LEVEL])
			{
				nScore=ns;
				break;
			}
			ns++;
			psp=GetNextSpa(psp);	
		}
	}
	
	MusicEditor->RedrawViews(nScore);
	return TRUE;
}

//*BIAN-VERR*


Bool Partitura::DoCommand(const Point& p)
{
	if (UsrSymCmd.CommandId!=CMD_DEBUGMILLA_SEL_SYMBOL) 
		if (1==DebugNormalMillaPos)
		{
			DebugNormalMilla=TRUE;
			DebugNormalMillaPos=2;
		}
		
		if(Scr->GetMouse()->IsDragging())
			return DragSel(p);
		
		switch(UsrSymCmd.SelectionType)
		{
		case NO_SEL:
			return FALSE;
		case INSERT_SEL:
			if (InsertSel(p)) 
			{
				Redraw();
			}
			else
			{
				EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","DoCommand()");
				return FALSE;
			}
			break;
		case IMMEDIATE_SEL:
			if (ImmediateSel(p)) 
			{
				Redraw();
			}
			else
			{
				EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","DoCommand()");
				return FALSE;
			}
			
			break;
			
		case MULTI_SEL:
			EM.Error(GENERAL_ERR_AREA,"EC_CMD_GESTIONE_ERRATA","Partitura","DoCommand()");
			return FALSE;
			break;
			
		case SINGLE_SEL:
			EM.Error(GENERAL_ERR_AREA,"EC_CMD_GESTIONE_ERRATA","Partitura","DoCommand()");
			return TRUE;
			break;
			
		case TWO_SEL:
			if (!TwoSel(p))
				return FALSE;
			
			break;
			
			
		case RECT_SEL:  
			break;
			
		case DRAG_SEL:
			if (DragSel(p)) 
			{
				Redraw();
			}
			else
			{
				EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","DoCommand()");
				return FALSE;
			}
			
			break;
			
			
		default: 
			return FALSE;
		}
		
		int nScore=GetCurrentScore();
		if(nScore==0) // per il main score determina in qualse spartito  stata fatta la modifica
		{
			Spartito *psp = GetFirstSpa();
			int ns=1;
			while (psp!=NULL) 
			{
				if (psp->GetNumericCode()==UsrSymCmd.Path1.Array[SPARTITO_PATH_LEVEL])
				{
					nScore=ns;
					break;
				}
				ns++;
				psp=GetNextSpa(psp);	
			}
			
		}
		
		MusicEditor->RedrawViews(nScore);
		return TRUE;
}
/*FFF
Bool Partitura::GetCommandInsert(const Point& p)
{
	
	Spartito *psp;
	DrawObject *obj;
	Bool QuestaBat, sgnAlt;
	SymCmd ACLTmpCmd;
	short staff;
	
	if (GetCurrentScore()==0)
		psp = GetFirstSpa();
	else
		psp = GetSpartNum(GetCurrentScore()-1);
	
	do
	{
		obj=NULL;
		// chiede allo spartito se in p ci sono oggetti da selezionare
		ResetPath(UsrSymCmd);
		psp->Hit(p,UsrSymCmd.SelObjType,obj,UsrSymCmd.Path1);
		if(obj==NULL)
		{
			psp = GetNextSpa(psp);    
			continue;
		}
			

//Bool InsertHit(const Point& posiz, short current_layer, Bool inAccordo,
//		  Bool& questaBat, NumCode& Altezza, Bool& SegnoAltezza, 
//		  NumCode& FigCode, NumCode& InsInTesta, short& staff);


		// : LA FUNZIONE RICHIAMATA ELIMINA I GRUPPI NELLA
		// BATTUTA. FARE IN MODO CHE LO STESSO ACCADA ANCHE NELLO SLAVE. ???
		((Battuta*)obj)->InsertHit(p,(short)UsrSymCmd.ArgV[1]-1,(Bool)UsrSymCmd.ArgV[3],QuestaBat,UsrSymCmd.ArgV[0],
			sgnAlt,UsrSymCmd.ArgV[4],UsrSymCmd.ArgV[5],staff);
		
		UsrSymCmd.ArgV[7] = sgnAlt;
		UsrSymCmd.ArgV[9] = staff;
		
		if(QuestaBat) // L'unica battuta effettivamente colpita.
		{

				// A Questo punto devo salvare il comando, per poterlo ripristinare alla fine
				// in modo tale da passarlo correttamente (le informazioni corrette) alla AddCommand
				// della lista ACL

//TRASCURO I CONTROLLI DI CONSISTENZA
				if (!obj->SymCommand(UsrSymCmd)==CMD_OK) 
				{
//					SetStatoEsecuzione(IMPAGINAZIONE);      
//					NetInvioComando(UsrSymCmd);
//					CopiaCommandStruct(&ACLTmpCmd,&UsrSymCmd);
//				}
//				else
//					return FALSE;
			short cur_layer = UsrSymCmd.Path1.Array[LAYER_PATH_LEVEL];
			Layer* pLayer = ((Battuta*)obj)->GetPLayer(cur_layer);
			if ( pLayer == NULL)
				UsrSymCmd.ArgV[6] = 1;
			else
				UsrSymCmd.ArgV[6] = pLayer->GetCodeCounter() + 1;
		
			CopiaCommandStruct(&ACLTmpCmd,&UsrSymCmd);
		}
		
		psp = GetNextSpa(psp);    
	} while(psp!=NULL && GetCurrentScore()==0);
	
	// A Questo punto devo ripristinare il comando, per poterlo passare 
	// correttamente (le informazioni corrette) alla AddCommand
	// della lista ACL
//	if (tipoLeggio==LEGGIO_MASAE)
		CopiaCommandStruct(&UsrSymCmd,&ACLTmpCmd);
	
	return TRUE;
}

Bool Partitura::GetCommandSelect(const Point& p)
{
	// chiede allo spartito se in p ci sono oggetti da selezionare
	DrawObject *obj;
	ResetPath(UsrSymCmd);
	Hit(p,UsrSymCmd.SelObjType,obj,UsrSymCmd.Path1);
  
  
	// se obj e' NULL allora non c'e' stata selezione
	if(obj==NULL)
		return FALSE;	
	  
	return TRUE;
}
//FFF*/
//-----------------------------------------------------------------------------------------------------------


//	Inserimento di Intervalli:
Bool Partitura::DoCommandInterv(SymCmd& SC)
{
	DrawObject *pd;
	Spartito *pSp;
	
	SymbolicHit(SC.Path1,SPARTITO_SEL,pd);
	if(pd==NULL)
	{
		EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","DoCommandInterv()");
		return FALSE;
	}
	
	pSp = (Spartito*)pd;
	
	
	
	return (pSp->SymCommand(SC)==CMD_OK);
}


//	Inserimento di Figure:
//		lo esegueono sia MASAE che DLIOO.
Bool Partitura::InsertSel(const Point& p)
{
	
	Spartito *psp;
	DrawObject *obj;
	Bool QuestaBat, sgnAlt;
	SymCmd ACLTmpCmd;
	short staff;
	
	if (GetCurrentScore()==0)
		psp = GetFirstSpa();
	else
		psp = GetSpartNum(GetCurrentScore()-1);
	
	do
	{
		obj=NULL;
		// chiede allo spartito se in p ci sono oggetti da selezionare
		ResetPath(UsrSymCmd);
		psp->Hit(p,UsrSymCmd.SelObjType,obj,UsrSymCmd.Path1);
		if(obj==NULL)
		{
			psp = GetNextSpa(psp);    
			continue;
		}
		
		
		// : LA FUNZIONE RICHIAMATA ELIMINA I GRUPPI NELLA
		// BATTUTA. FARE IN MODO CHE LO STESSO ACCADA ANCHE NELLO SLAVE.
		((Battuta*)obj)->InsertHit(p,(short)UsrSymCmd.ArgV[1]-1,(Bool)UsrSymCmd.ArgV[3],QuestaBat,UsrSymCmd.ArgV[0],
			sgnAlt,UsrSymCmd.ArgV[4],UsrSymCmd.ArgV[5],staff);
		
		UsrSymCmd.ArgV[7] = sgnAlt;
		UsrSymCmd.ArgV[9] = staff;
		
		
		
		
		if(QuestaBat) // L'unica battuta effettivamente colpita.
		{
			if (tipoLeggio==LEGGIO_MASAE)
			{
				// A Questo punto devo salvare il comando, per poterlo ripristinare alla fine
				// in modo tale da passarlo correttamente (le informazioni corrette) alla AddCommand
				// della lista ACL
				
				if (obj->SymCommand(UsrSymCmd)==CMD_OK) {
					SetStatoEsecuzione(IMPAGINAZIONE);      
					NetInvioComando(UsrSymCmd);
					CopiaCommandStruct(&ACLTmpCmd,&UsrSymCmd);  
				}
				else
					return FALSE;
			}
			else
			{
#ifdef WITH_NET   
				NetInvioComando(UsrSymCmd);
#else
				if (obj->SymCommand(UsrSymCmd)!=CMD_OK)
					return FALSE;
#endif
			}
		}
		
		psp = GetNextSpa(psp);    
	} while(psp!=NULL && GetCurrentScore()==0);
	
	// A Questo punto devo ripristinare il comando, per poterlo passare 
	// correttamente (le informazioni corrette) alla AddCommand
	// della lista ACL
	if (tipoLeggio==LEGGIO_MASAE)
		CopiaCommandStruct(&UsrSymCmd,&ACLTmpCmd);
	
	return TRUE;
}

Bool Partitura::DragSel(const Point& p)
{	
	Casella *cas = casellaPtr;
	float magn = Scr->GetMagnification();
	float x,y;
	if(Scr->GetMouse()->IsDraggingImg())
	{
		x = (Scr->GetMouse()->GetPos().x)*magn;
		y = (Scr->GetMouse()->GetPos().y-Scr->GetMouse()->GetHeight())*magn;
	}
	else
	{
		x = (Scr->GetMouse()->GetPos().x)*magn;
		y = (Scr->GetMouse()->GetPos().y)*magn;
	}
	if(cas!=NULL)
	{
		cas->SetX(x);
		cas->SetY(y);
		MusicEditor->RedrawView(DrawObject::GetCurrentView());
		
	}
	Scr->GetMouse()->NoRectSel();
	return true;
}



Bool Partitura::ImmediateSel(const Point& p)
{
	DrawObject *obj, *spobj;
	Bool fl;
	NumCode np;
	
	Battuta *PBattuta, *pbat;
	Spartito *psp;
	
	ClassID ChvID;
	tipoArmaturaChiave ArmCh;
	SymCmd SC;
	int staff;
	
    // Comandi che coinvolgono tutta la partitura.
	switch(UsrSymCmd.CommandId)
	{
	case CMD_BATTUTA_CVIOLINO:
	case CMD_BATTUTA_CVIOLINO8:
	case CMD_BATTUTA_CVIOLINO8SOPRA:
	case CMD_BATTUTA_CBASSO:
	case CMD_BATTUTA_CBASSO8:
	case CMD_BATTUTA_CBASSO8SOPRA:
	case CMD_BATTUTA_CBASSOOLD:
	case CMD_BATTUTA_CBARITONO:
	case CMD_BATTUTA_CTENORE:
	case CMD_BATTUTA_CTENORE8:
	case CMD_BATTUTA_CCONTRALTO:
	case CMD_BATTUTA_CMEZZOSOPRANO:
	case CMD_BATTUTA_CSOPRANO:
	case CMD_BATTUTA_CPERCUSBOX:
	case CMD_BATTUTA_CPERCUS2LINES:
	case CMD_BATTUTA_CTAB:
	case CMD_BATTUTA_CVUOTA:
		
		ResetPath(UsrSymCmd);
		Hit(p,BATTUTA_SEL,spobj,UsrSymCmd.Path1);
		if(spobj==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_BAT_NOTFOUND","Partitura","ImmediateSel()");
			return FALSE;
		}
		PBattuta = ((Battuta*)spobj);
		
		SymbolicHit(UsrSymCmd.Path1,SPARTITO_SEL,spobj);
		psp =  (Spartito*)spobj;
		if(psp==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","ImmediateSel()");
			return FALSE;
		}
		
		staff=0; 
		
		
		obj = PBattuta->GetPLayer(0)->GetFirstF();
		if (obj != NULL)
		{
			if (p.x > obj->GetAbsPos().x) 
			{
				CopiaCommandStruct(&SC,&UsrSymCmd);
				
				SetCommand(CMD_BATTUTA_INS_SPAZIO,INSERT_SEL,BATPART_SEL,0,UsrSymCmd.ArgV[1],UsrSymCmd.CommandId);
				
				fl = InsertSel(p);
				
				UsrSymCmd.Path1.Array[FIGURA1_PATH_LEVEL] = (NumCode)UsrSymCmd.ArgV[6];
				UsrSymCmd.Path1.Array[LAYER_PATH_LEVEL] = (NumCode)UsrSymCmd.ArgV[1];
				
				SymbolicHit(UsrSymCmd.Path1,FIGURA_SEL,spobj);
				if(spobj==NULL)
				{
					EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","ImmediateSel()");
					return FALSE;
				}
				
				if (!(PBattuta->GetChiaveFig(staff,(Figura*)spobj)))
				{
					PBattuta = psp->GetNextBat(PBattuta);
					if (PBattuta!=NULL)
					{
						ChvID = PBattuta->GetChiave(staff)->GetID();
						while (PBattuta!=NULL)
						{
							PBattuta->SymCommand(SC);
							PBattuta = psp->GetNextBat(PBattuta);
							if (PBattuta!=NULL)
								if (PBattuta->GetChiave(staff)->GetID() != ChvID)
									PBattuta = NULL;
						}
					}
				}
				
				return fl;
			}
		}
		break;
		
    case CMD_BATTUTA_ARMCHV_0D:
    case CMD_BATTUTA_ARMCHV_1D:
    case CMD_BATTUTA_ARMCHV_2D:
    case CMD_BATTUTA_ARMCHV_3D:
    case CMD_BATTUTA_ARMCHV_4D:
    case CMD_BATTUTA_ARMCHV_5D:
    case CMD_BATTUTA_ARMCHV_6D:
    case CMD_BATTUTA_ARMCHV_7D:
    case CMD_BATTUTA_ARMCHV_1B:
    case CMD_BATTUTA_ARMCHV_2B:
    case CMD_BATTUTA_ARMCHV_3B:
    case CMD_BATTUTA_ARMCHV_4B:
    case CMD_BATTUTA_ARMCHV_5B:
    case CMD_BATTUTA_ARMCHV_6B:
    case CMD_BATTUTA_ARMCHV_7B:
    case CMD_BATTUTA_ARMCHV_0Dm:
    case CMD_BATTUTA_ARMCHV_1Dm:
    case CMD_BATTUTA_ARMCHV_2Dm:
    case CMD_BATTUTA_ARMCHV_3Dm:
    case CMD_BATTUTA_ARMCHV_4Dm:
    case CMD_BATTUTA_ARMCHV_5Dm:
    case CMD_BATTUTA_ARMCHV_6Dm:
    case CMD_BATTUTA_ARMCHV_7Dm:
    case CMD_BATTUTA_ARMCHV_1Bm:
    case CMD_BATTUTA_ARMCHV_2Bm:
    case CMD_BATTUTA_ARMCHV_3Bm:
    case CMD_BATTUTA_ARMCHV_4Bm:
    case CMD_BATTUTA_ARMCHV_5Bm:
    case CMD_BATTUTA_ARMCHV_6Bm:
    case CMD_BATTUTA_ARMCHV_7Bm:
		ResetPath(UsrSymCmd);
		Hit(p,BATTUTA_SEL,spobj,UsrSymCmd.Path1);
		if(spobj==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_BAT_NOTFOUND","Partitura","ImmediateSel()");
			return FALSE;
		}
		PBattuta = ((Battuta*)spobj);
		
		SymbolicHit(UsrSymCmd.Path1,SPARTITO_SEL,spobj);
		psp =  (Spartito*)spobj;
		if(psp==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","ImmediateSel()");
			return FALSE;
		}
		
		staff=0; 
		
		obj = PBattuta->GetPLayer(0)->GetFirstF();
		if (obj != NULL)
		{
			if (p.x > obj->GetAbsPos().x) 
			{
				CopiaCommandStruct(&SC,&UsrSymCmd);
				
				SetCommand(CMD_BATTUTA_INS_SPAZIO,INSERT_SEL,BATPART_SEL,0,1,UsrSymCmd.CommandId);
				
				fl = InsertSel(p);
				
				UsrSymCmd.Path1.Array[FIGURA1_PATH_LEVEL] = (NumCode)UsrSymCmd.ArgV[6];
				UsrSymCmd.Path1.Array[LAYER_PATH_LEVEL] = (NumCode)UsrSymCmd.ArgV[1];
				
				SymbolicHit(UsrSymCmd.Path1,FIGURA_SEL,spobj);
				if(spobj==NULL)
				{
					EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","ImmediateSel()");
					return FALSE;
				}
				
				if (!(PBattuta->GetChiaveFig(staff,(Figura*)spobj)))
				{
					PBattuta = psp->GetNextBat(PBattuta);
					if (PBattuta!=NULL)
					{
						ArmCh = PBattuta->GetArmatura(staff);
						ChvID = PBattuta->GetChiave(staff)->GetID();
						while (PBattuta!=NULL)
						{
							PBattuta->SymCommand(SC);
							PBattuta = psp->GetNextBat(PBattuta);
							if (PBattuta!=NULL)
								if ((PBattuta->GetChiave(staff)->GetID() != ChvID) || 
									(PBattuta->GetArmatura(staff) != ArmCh))
									PBattuta = NULL;
						}
					}
				}
				
				return fl;
			}
		}
		break;
		
    case CMD_PARTITURA_MOVE_LAYER:
		{
			int from, to, numLay;
			ResetPath(UsrSymCmd);
			Hit(p,SPARTITO_SEL,spobj,UsrSymCmd.Path1);
			if(spobj==NULL)
			{
				EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","ImmediateSel()");
				return FALSE;
			}
			numLay=((Spartito *)spobj)->GetNumberOfStaff()*4;
			if(MoveLayerDial(from, to, numLay))
				((Spartito *)spobj)->SwitchLayer(from,to);
			return TRUE;
		}
		
		
		
    case CMD_PARTITURA_ADD_SPARTITO:
		ResetPath(UsrSymCmd);
		Hit(p,SPARTITO_SEL,spobj,UsrSymCmd.Path1);
		if(spobj==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","ImmediateSel()");
			return FALSE;
		}
		UsrSymCmd.ArgV[0] = ( p.y > spobj->GetAbsPos().y );
        // se e' vero inserisce sotto lo spartito selezionato
        // altrimenti inserisce sopra.
		
		Scr->GSetMainDrawArea();
		if(spobj!=NULL)
		{
			AddNewSpartito((Spartito*)spobj, (Bool)UsrSymCmd.ArgV[0],(int)UsrSymCmd.ArgV[1],UsrSymCmd.Text);
			
			if (tipoLeggio==LEGGIO_MASAE)
				NetInvioComando(UsrSymCmd);
			
			GoTop();
			
			
			return TRUE;
		}
		else
		{
			EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","DoCommandImmediate()");
			return FALSE;
		}
		
    case CMD_PARTITURA_ADD_BATTUTA:
		return DoCommandAddBattuta(p);
		
    case CMD_BATTUTA_BSINGOLA:
    case CMD_BATTUTA_BSINGOLA_RIP2B:
    case CMD_BATTUTA_BDOPPIA:
    case CMD_BATTUTA_BINIZIORIT:
    case CMD_BATTUTA_BFINERIT:
    case CMD_BATTUTA_BFINALE:
    case CMD_BATTUTA_BINIZIOFINERIT:
	case CMD_BATTUTA_BINVISIBILE:
		return DoCommandBarra(p);
		
    case CMD_DEL_COL_FIGURE:
		return DoCommandDelColFig(p);
		
    case CMD_PARTITURA_DEL_BATTUTA:
		return DoCommandDelBat(p);
		
		// Cancella una battuta e sostituisce con spazi
    case CMD_BATTUTA_ERASE:
		ResetPath(UsrSymCmd);
		Hit(p,SPARTITO_SEL,spobj,UsrSymCmd.Path1);
		Hit(p,UsrSymCmd.SelObjType,obj,UsrSymCmd.Path1);
		if(spobj==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","ImmediateSel()");
			return FALSE;
		}
		
		if (liooAbil.GetDelAbil(obj->GetID())==0) 
		{
			Message("Utente non abilitato a questa cancellazione");
			return FALSE;
		} 
		
		if (((Spartito*)spobj)->SymCommand(UsrSymCmd)==CMD_OK)
		{
			NetInvioComando(UsrSymCmd);
			return TRUE;
		}
		else
			return FALSE;
		
    case CMD_BATTUTA_MOVIMENTO:
		ResetPath(UsrSymCmd);
		Hit(p,BATTUTA_SEL,spobj,UsrSymCmd.Path1);
		if(spobj==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_BAT_NOTFOUND","Partitura","ImmediateSel()");
			return FALSE;
		}
		
		np = ((Battuta*)spobj)->GetNumProgress();
		
		psp = (Spartito*)listaSpa->GetLast();
		
		while (psp!=NULL)
		{
			PBattuta = psp->GetBattNum(np-1);
			if (PBattuta->SymCommand(UsrSymCmd)==CMD_OK) 
			{
				UsrSymCmd.Path1.Array[0] = psp->GetNumericCode();
				UsrSymCmd.Path1.Array[2] = PBattuta->GetNumericCode();
				NetInvioComando(UsrSymCmd);
			}
			else
				return FALSE;
			
			psp = GetPrevSpa(psp);
		}
		
		if (GetStatoEsecuzione() > IMPAGINAZIONE)
		{
			psp = GetFirstSpa();
			pbat = psp->GetBattNum(0);
			while (pbat != NULL)
			{
				SetTimeExec(pbat);
				pbat = psp->GetNextBat(pbat);
			}
			
			char* s;
			char nome[200];
			char filesPath[200];
			strcpy(nome,GetFileSaveName());
			s=strstr(nome,".mds");
			if (s!=NULL) (*(s))='\0';  
			s=strrchr(nome,'/');
			if (s!=NULL) (*(s+1))='\0';
			
			strcpy(filesPath,smrgetenv("SMR_FILES_PATH"));
			strcat(filesPath,GetPathSaveName());
			strcat(filesPath,nome);
			strcat(filesPath,".ett");
			SaveETT(filesPath);
			
			SetStatoEsecuzione(JUMP_ESECUZIONE);
		}
		return TRUE;
		
		
    case CMD_BATTUTA_LETTERA:
		ResetPath(UsrSymCmd);
		Hit(p,BATTUTA_SEL,spobj,UsrSymCmd.Path1);
		if(spobj==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_BAT_NOTFOUND","Partitura","ImmediateSel()");
			return FALSE;
		}
		
		return CmdAddLettera(UsrSymCmd);
		
		
    default: ;
  }
  
  
  // chiede allo spartito se in p ci sono oggetti da selezionare
  ResetPath(UsrSymCmd);
  Hit(p,UsrSymCmd.SelObjType,obj,UsrSymCmd.Path1);
  
  
  // se obj e' NULL allora non c'e' stata selezione
  if(obj==NULL)
  {
	  switch(UsrSymCmd.CommandId)
	  {
		  
	  case CMD_PARTITURA_INSERTTEXT:	
		  if(GetPrintingStatus())
		  {
			  listaPag->CmdInsertText(p,listaSpa->GetCurrentPage());
			  return TRUE;
		  }
		  break; 
		  
	  case CMD_PARTITURA_PASTE_PAGEBOX:		
		  {
			  if(GetPrintingStatus())
				  return PastePageBox(p);
		  }
		  
	  case CMD_NOTA_DIESIS:
      case CMD_NOTA_DIESIS1Q:
      case CMD_NOTA_DIESIS3Q:
      case CMD_NOTA_BEMOLLE:
      case CMD_NOTA_BEMOLLE1Q:
      case CMD_NOTA_BEMOLLE3Q:
      case CMD_NOTA_BEQUADRO:
      case CMD_NOTA_BEQUADRO1Q:
      case CMD_NOTA_BEQUADRO3Q:
      case CMD_NOTA_2DIESIS:
      case CMD_NOTA_2BEMOLLE:
		  ResetPath(UsrSymCmd);
		  Hit(p,ANY_SEL,obj,UsrSymCmd.Path1);
		  
		  // se obj e' NULL allora non c'e' stata selezione
		  if(obj==NULL)
		  {
			  EM.Error(MUSICAL_ERR_AREA,"EC_OGG_NOTFOUND","Partitura","ImmediateSel()");
			  return FALSE;
		  }
		  
		  if (obj->SymCommand(UsrSymCmd)==CMD_OK)
		  {
			  NetInvioComando(UsrSymCmd);
			  return TRUE;
		  }
		  else
			  return FALSE;
		  
		  break;
		  
	  default:
		  EM.Error(MUSICAL_ERR_AREA,"EC_OGG_NOTFOUND","Partitura","ImmediateSel()");
		  return FALSE;
		  
	  }
	  
  }
  
  
  
  switch(UsrSymCmd.CommandId)
  {
	  
  case CMD_PARTITURA_INSERTTEXT:	
	  
	  if(GetPrintingStatus())
	  {
		  listaPag->CmdInsertText(p,listaSpa->GetCurrentPage());
		  return TRUE;
	  }
	  break; 
	  
  case CMD_BATTUTA_TEMPO:
  case CMD_BATTUTA_ARMCHV_0D:
  case CMD_BATTUTA_ARMCHV_1D:
  case CMD_BATTUTA_ARMCHV_2D:
  case CMD_BATTUTA_ARMCHV_3D:
  case CMD_BATTUTA_ARMCHV_4D:
  case CMD_BATTUTA_ARMCHV_5D:
  case CMD_BATTUTA_ARMCHV_6D:
  case CMD_BATTUTA_ARMCHV_7D:
  case CMD_BATTUTA_ARMCHV_1B:
  case CMD_BATTUTA_ARMCHV_2B:
  case CMD_BATTUTA_ARMCHV_3B:
  case CMD_BATTUTA_ARMCHV_4B:
  case CMD_BATTUTA_ARMCHV_5B:
  case CMD_BATTUTA_ARMCHV_6B:
  case CMD_BATTUTA_ARMCHV_7B:
  case CMD_BATTUTA_ARMCHV_0Dm:
  case CMD_BATTUTA_ARMCHV_1Dm:
  case CMD_BATTUTA_ARMCHV_2Dm:
  case CMD_BATTUTA_ARMCHV_3Dm:
  case CMD_BATTUTA_ARMCHV_4Dm:
  case CMD_BATTUTA_ARMCHV_5Dm:
  case CMD_BATTUTA_ARMCHV_6Dm:
  case CMD_BATTUTA_ARMCHV_7Dm:
  case CMD_BATTUTA_ARMCHV_1Bm:
  case CMD_BATTUTA_ARMCHV_2Bm:
  case CMD_BATTUTA_ARMCHV_3Bm:
  case CMD_BATTUTA_ARMCHV_4Bm:
  case CMD_BATTUTA_ARMCHV_5Bm:
  case CMD_BATTUTA_ARMCHV_6Bm:
  case CMD_BATTUTA_ARMCHV_7Bm:
	  
  case CMD_BATTUTA_CVIOLINO:
  case CMD_BATTUTA_CVIOLINO8:
  case CMD_BATTUTA_CVIOLINO8SOPRA:
  case CMD_BATTUTA_CBASSO:
  case CMD_BATTUTA_CBASSO8:
  case CMD_BATTUTA_CBASSO8SOPRA:
  case CMD_BATTUTA_CBASSOOLD:
  case CMD_BATTUTA_CBARITONO:
  case CMD_BATTUTA_CTENORE:
  case CMD_BATTUTA_CTENORE8:
  case CMD_BATTUTA_CCONTRALTO:
  case CMD_BATTUTA_CMEZZOSOPRANO:
  case CMD_BATTUTA_CSOPRANO:
  case CMD_BATTUTA_CPERCUSBOX:
  case CMD_BATTUTA_CPERCUS2LINES:
  case CMD_BATTUTA_CTAB:
  case CMD_BATTUTA_CVUOTA:
  case CMD_SPARTITO_ADD_BEAM:
		SymbolicHit(UsrSymCmd.Path1,SPARTITO_SEL,spobj);
	  
      if(spobj==NULL)
      {
		  EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","ImmediateSel()");
		  return FALSE;
      }
	  
      if (((Spartito *)spobj)->SymCommand(UsrSymCmd)==CMD_OK) 
      {
		  SetStatoEsecuzione(IMPAGINAZIONE);      
		  NetInvioComando(UsrSymCmd);
		  return TRUE;
      }
      else
		  return FALSE;
	  
	  
  case CMD_PARTITURA_COPIA_BATTUTA:
	  CopyMeasure(((Battuta*)obj)->GetNumProgress());
	  return TRUE;
	  
  case CMD_PARTITURA_INCOLLA_BATTUTA:
  	   return PasteMeasureAfter(((Battuta*)obj)->GetNumProgress());
	   
  case CMD_PARTITURA_COPIA_SINGOLABATTUTA:
	  CopySingleMeasure(UsrSymCmd.Path1.Array[SPARTITO_PATH_LEVEL],UsrSymCmd.Path1.Array[BATTUTA_PATH_LEVEL]);
	  return TRUE;       
	  
  case CMD_PARTITURA_INCOLLA_SINGOLABATTUTA:
  	   return PasteSingleMeasureIn(UsrSymCmd.Path1.Array[SPARTITO_PATH_LEVEL],UsrSymCmd.Path1.Array[BATTUTA_PATH_LEVEL]);
	   
  case CMD_PARTITURA_CHECK_BATTUTA:  
      PBattuta = ((Battuta*)obj);
      if(PBattuta->CheckBatt())
		  Message("Measure is time consistent");
	  else
		  Message("Measure is NOT time consistent");
      return TRUE; 
      
  case CMD_SPARTITO_ADD_LEGATURA_VALORE:
      SymbolicHit(UsrSymCmd.Path1,SPARTITO_SEL,spobj);
	  
      if(spobj==NULL)
      {
		  EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","ImmediateSel()");
		  return FALSE;
      }
      if (((Spartito *)spobj)->SymCommand(UsrSymCmd)==CMD_OK) 
      {
		  NetInvioComando(UsrSymCmd);
		  return TRUE;
      }
      else
		  return FALSE;
      
	  
  case CMD_DEL_SYMBOL:
	  
	  
	  if ((obj->GetID()==CL_LETTERA) || (obj->GetID()==CL_MOVIMENTO))
	  {
		  char c;
		  SymbolicHit(UsrSymCmd.Path1,BATTUTA_SEL,spobj);
		  if(spobj==NULL)
		  {
			  EM.Error(MUSICAL_ERR_AREA,"EC_BAT_NOTFOUND","Partitura","ImmediateSel()");
			  return FALSE;
		  }
		  
		  np = ((Battuta*)spobj)->GetNumProgress();
		  
		  psp = (Spartito*)listaSpa->GetLast();
		  
		  while (psp!=NULL)
		  {
			  PBattuta = psp->GetBattNum(np-1);
			  
			  if (obj->GetID()==CL_LETTERA)
			  {
				  c = *(PBattuta->GetPLettera()->GetTxt());
				  delete PBattuta->GetPLettera();
				  PBattuta->SetPLettera(NULL);
			  }
			  else if (obj->GetID()==CL_MOVIMENTO)
			  {
				  delete PBattuta->GetPMovimento();
				  PBattuta->SetPMovimento(NULL);
			  }
			  
			  UsrSymCmd.Path1.Array[0] = psp->GetNumericCode();
			  UsrSymCmd.Path1.Array[2] = PBattuta->GetNumericCode();
			  NetInvioComando(UsrSymCmd);
			  
			  psp = GetPrevSpa(psp);
		  }
		  DelEtichetta(c);
		  
		  return TRUE;
	  }
	  else
	  {
		  int id=obj->GetID();
		  if(id==CL_LEGATURA_QUADRA)
		  {
			  pbat = ((IntEsteso*)obj)->GetBattuta(1);
		  }
		  
		  if (DeleteSym((DrawObject*) obj))
		  {
			  NetInvioComando(UsrSymCmd);
			  if(id==CL_LEGATURA_QUADRA)
			  {
				  if(Battuta::GetAutoBeaming())      
					  pbat->BeamingAuto();
			  }
			  return TRUE;
		  }
		  else
			  return FALSE;
	  }
	  
	  // per il trace dell'oggetto
  case CMD_DEBUGMILLA_SEL_SYMBOL:
	  
	  // ho fatto lo hit e ho il puntatore a drawobject (obj)
	  // pero per chiamare Immrule e posrule mi servono il puntatore a nota e 
	  // alla battuta per cui devo chiamare lo opportuno symbolichit
	  // che dal path simbolico che ho mi restituisce questi valori
	  //	    SymbolicHit(UsrSymCmd.Path1,BATTUTA_SEL,spobj); questo ad esempio cerca il puntatore a spartito
	  // 
	  
	  obj->SetTraceMilla(TRUE);
	  
	  
	  
	  return TRUE;
	  /////////////////////////////
	  
	  
	  
	  
	  
  case CMD_EDIT_SYMBOL:
	  {
		  Battuta* pBat=NULL;
		  // viene determinata la battuta dell'oggetto per sapere se si e' in multivoice
		  SymbolicHit(UsrSymCmd.Path1,BATTUTA_SEL,(DrawObject*&)pBat);
		  {
			  obj->Edit(pBat);	  
		  }
		  
	  }
	  return TRUE;
	  
	  
  case CMD_PARTITURA_DEL_SPARTITO: 
	  {
		  if(CopyToClipboard(UsrSymCmd))
			  listaSpa->SetCutCopy(true);
		  if (DelSpartito(UsrSymCmd))
		  {
			  NetInvioComando(UsrSymCmd);
			  return TRUE;
		  }
		  else
			  return FALSE;
	  }
	  
  case CMD_PARTITURA_COPY_SPARTITO: 
	  {
		  if(CopyToClipboard(UsrSymCmd))
		  {	
			  listaSpa->SetCutCopy(true);
			  NetInvioComando(UsrSymCmd);
			  return TRUE;
		  }
		  else
			  return FALSE;
	  }
	  
  case CMD_PARTITURA_PASTE_SPARTITO: 
	  {
		  ResetPath(UsrSymCmd);
		  Hit(p,SPARTITO_SEL,spobj,UsrSymCmd.Path1);
		  if(spobj==NULL)
		  {
			  EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","ImmediateSel()");
			  return FALSE;
		  }
		  UsrSymCmd.ArgV[0] = ( p.y > spobj->GetAbsPos().y );
		  // se e' vero inserisce sotto lo spartito selezionato
		  // altrimenti inserisce sopra.
		  
		  Scr->GSetMainDrawArea();
		  if(spobj!=NULL)
		  {
			  if(PasteFromClipboard((Spartito*)spobj, (Bool)UsrSymCmd.ArgV[0]))	
			  {
				  Scr->GSetMainDrawArea();
				  NetInvioComando(UsrSymCmd);
				  return TRUE;
			  }
			  else
				  return FALSE;
			  GoTop();
			  
			  
			  return TRUE;
		  }
		  else
		  {
			  EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","DoCommandImmediate()");
			  return FALSE;
		  }
		  
	  }
  case CMD_PARTITURA_CUT_PAGEBOX: 
	  {		
		  if(GetPrintingStatus())
			  return CutPageBox(obj);			
	  }
	  
  case CMD_PARTITURA_COPY_PAGEBOX: 
	  {
		  if(GetPrintingStatus())
			  return CopyPageBox(obj);
	  }
	  
  case CMD_PARTITURA_PASTE_PAGEBOX: 
	  {
		  if(GetPrintingStatus())
			  return PastePageBox(p);
	  }
	  
	  
  case CMD_SPARTITO_SET_NAME:
	  if (SymCommand(UsrSymCmd)==CMD_OK)
	  {
		  NetInvioComando(UsrSymCmd);
		  return TRUE;
	  }
	  else
		  return FALSE;
	  
  case CMD_PARTITURA_NEW_LYRIC:
	  if (SymCommand(UsrSymCmd)==CMD_OK)
	  {
		  NetInvioComando(UsrSymCmd);
		  return TRUE;
	  }
	  else
		  return FALSE;
	  
  case CMD_PARTITURA_EDIT_LYRIC:
	  if (SymCommand(UsrSymCmd)==CMD_OK)
	  {
		  NetInvioComando(UsrSymCmd);
		  return TRUE;
	  }
	  else
		  return FALSE;
	  
  case CMD_PARTITURA_REMOVE_LYRIC:
	  if (SymCommand(UsrSymCmd)==CMD_OK)
	  {
		  NetInvioComando(UsrSymCmd);
		  return TRUE;
	  }
	  else
		  return FALSE;
	  
  case CMD_PARTITURA_SELECT_LYRIC:
	  if (SymCommand(UsrSymCmd)==CMD_OK)
	  {
		  NetInvioComando(UsrSymCmd);
		  return TRUE;
	  }
	  else
		  return FALSE;
	  ////////////////////////////////////////////
	  
	  
	  
	  
	  
  case CMD_PARTITURA_HIDE_PART:
	  if (SymCommand(UsrSymCmd)==CMD_OK)
	  {
		  NetInvioComando(UsrSymCmd);
		  return TRUE;
	  }
	  else
		  return FALSE;
	  
 	case CMD_PARTITURA_SPLITTING_PART:
		if (SymCommand(UsrSymCmd)==CMD_OK)
		{
			NetInvioComando(UsrSymCmd);
			return TRUE;
		}
		else
			return FALSE;
		
		
	case CMD_PARTITURA_DISJOINING_PART:
		if (SymCommand(UsrSymCmd)==CMD_OK)
		{
			NetInvioComando(UsrSymCmd);
			return TRUE;
		}
		else
			return FALSE;
		
	case CMD_PARTITURA_VIEW_LAYER:
		if(SymCommand(UsrSymCmd)==CMD_OK)
		{
			NetInvioComando(UsrSymCmd);
			return TRUE;
		}
		else
			return FALSE;
		
    case CMD_BATTUTA_GRUPPONOTE2NOTE:
		if(DoCommandGruppo())
		{
			Redraw();
			return TRUE;
		}
		else
			return FALSE;
		
		
		
    case CMD_NOTA_PIZZICATO:
    case CMD_NOTA_ARCO:
    case CMD_ARCO_ARCOSU:
    case CMD_ARCO_ARCOGIU:
    case CMD_ARCO_PONTICELLO:
    case CMD_ARCO_TASTIERA:
    case CMD_ARCO_PUNTA:
    case CMD_ARCO_TALLONE:
    case CMD_NOTA_CORDA_0:
    case CMD_NOTA_CORDA_1:
    case CMD_NOTA_CORDA_2:
    case CMD_NOTA_CORDA_3:
    case CMD_NOTA_CORDA_4:
    case CMD_NOTA_CORDA_5:
	case CMD_NOTA_CORDA_6:
	case CMD_NOTA_CORDA_7:
    case CMD_NOTA_TRILLO:
    case CMD_NOTA_GRUPPETTO_SUP_1:
    case CMD_NOTA_GRUPPETTO_INF_1:
	case CMD_NOTA_TURN_SLASH:
	case CMD_NOTA_TURN_UP:
    case CMD_NOTA_MORDENTE_SUP:
    case CMD_NOTA_MORDENTE2_SUP:
    case CMD_NOTA_MORDENTE_INF:
    case CMD_NOTA_MORDENTE2_INF:
	case CMD_NOTA_SLIDING_UP:
	case CMD_NOTA_SLIDING_DOWN:
	case CMD_NOTA_SLIDING_DUP:
	case CMD_NOTA_SLIDING_DOWNUP:
	case CMD_NOTA_NONGHYN_INC:
	case CMD_NOTA_NONGHYN_DEC:
	case CMD_NOTA_NONGHYN_ARROW:
	case CMD_NOTA_NONGHYN_WIDE:
	case CMD_NOTA_DITO_0:
	case CMD_NOTA_DITO_1:
	case CMD_NOTA_DITO_2:
	case CMD_NOTA_DITO_3:
	case CMD_NOTA_DITO_4:
	case CMD_NOTA_DITO_5:
    case CMD_NOTA_TENUTO:
    case CMD_NOTA_PUNTOALL:
    case CMD_NOTA_SFORZATO:
    case CMD_NOTA_ACCENTO:
    case CMD_NOTA_MARTELLATO:
    case CMD_NOTA_MARTDOLCE:
    case CMD_NOTA_STACCATO:
	case CMD_NOTA_ESPGENERIC:
    case CMD_NOTA_CONSORDINAOTTONI:
    case CMD_NOTA_CONSORDINAARCHI:
    case CMD_NOTA_CONSORDINATESTO:
    case CMD_NOTA_VIASORDINAOTTONI:
    case CMD_NOTA_VIASORDINAARCHI:
    case CMD_NOTA_VIASORDINATESTO:
    case CMD_NOTA_ARMONICIOTTONI:
    case CMD_NOTA_ARMONICIARCHI:
    case CMD_FIGURA_1PUNTO:
    case CMD_FIGURA_2PUNTI:
    case CMD_FIGURA_3PUNTI:    
		{
			Battuta* pBat=NULL;
			
			// viene determinata la battuta dell'oggetto per sapere se si e' in multivoice
			SymbolicHit(UsrSymCmd.Path1,BATTUTA_SEL,(DrawObject*&)pBat);
			if(pBat!=NULL)
			{
				UsrSymCmd.ArgV[2]=pBat->GetMultiVoice(); // verificare cosa mettere al posto di 0 (layer)
			}
			
			if(pBat!=NULL)
			{
				if (obj->SymCommand(UsrSymCmd, pBat)==CMD_OK) 
					//questo obj e' una nota.
				{
					NetInvioComando(UsrSymCmd);
					return TRUE;
				}
			}   
			
			
			return FALSE;
		}
		
    case CMD_FIGURA_SEELINK:
		{
			Battuta* pBat=NULL;
			SymbolicHit(UsrSymCmd.Path1,BATTUTA_SEL,(DrawObject*&)pBat);
			if(pBat!=NULL)
			{
				if (obj->SymCommand(UsrSymCmd, pBat)==CMD_OK) 
					//questo obj e' una nota.
				{
					NetInvioComando(UsrSymCmd);
					return TRUE;
				}
			}   
			return FALSE;
		}
	default:
		if (obj->SymCommand(UsrSymCmd)==CMD_OK)
		{
			NetInvioComando(UsrSymCmd);
			return TRUE;
		}
		else
			return FALSE;
  }
  return FALSE;
}


Bool Partitura::CmdAddLettera(SymCmd & SC)
{
	DrawObject *spobj;
	NumCode np;
	
	Spartito *psp;
	Battuta *PBattuta;
	
	if (FindEtichetta(SC.Text[0]))
	{
		Message("Label already present!","Error!");
		return TRUE;
	}
	else
	{	
		SymbolicHit(SC.Path1,BATTUTA_SEL,spobj);
		if(spobj==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_BAT_NOTFOUND","Partitura","CmdAddLettera()");
			return FALSE;
		}
		
		np = ((Battuta*)spobj)->GetNumProgress();
		
		psp = (Spartito*)listaSpa->GetLast();
		
		while (psp!=NULL)
		{
			PBattuta = psp->GetBattNum(np-1);
			if (PBattuta->SymCommand(SC)==CMD_OK) 
			{
				SC.Path1.Array[0] = psp->GetNumericCode();
				SC.Path1.Array[2] = PBattuta->GetNumericCode();
				NetInvioComando(SC);
			}
			else
				return FALSE;
			
			psp = GetPrevSpa(psp);
		}
		
		AddEtichetta(SC.Text[0], SC.Path1.Array[BATTUTA_PATH_LEVEL]);
		return TRUE;
	}
}


Bool Partitura::TwoSel(Point p)
{
	//
	// utilizzato solo per Intervalli
	//
	
	DrawObject *obj;
	Bool flag;
	
	
	// chiede se in p ci sono oggetti da selezionare
	if (FirstSel)
	{
		Hit(p,UsrSymCmd.SelObjType,obj,UsrSymCmd.Path1);
		if(obj==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_OGG_NOTFOUND","Partitura","TwoSel()");
			ResetPath(UsrSymCmd);
			return FALSE;
		}
		FirstSel = FALSE;
		CopiaCommandStruct(&UsrSymCmdTwoSel,&UsrSymCmd);
		
		if(UsrSymCmdTwoSel.CommandId!=CMD_SPARTITO_ADD_SILLABA)
			return FALSE;
		if (DoCommandInterv(UsrSymCmdTwoSel))
		{
			NetInvioComando(UsrSymCmdTwoSel);
			Redraw();
			CopiaCommandStruct(&UsrSymCmd,&UsrSymCmdTwoSel);
			ResetPath(UsrSymCmdTwoSel);
			FirstSel = TRUE;
			return TRUE;
		}
		else
		{
			EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","TwoSel()");
			ResetPath(UsrSymCmd);
			ResetPath(UsrSymCmdTwoSel);
			CopiaCommandStruct(&UsrSymCmd,&UsrSymCmdTwoSel);
			return FALSE;
		}
		
	}
	
	
	Hit(p,UsrSymCmdTwoSel.SelObjType,obj,UsrSymCmdTwoSel.Path2);
	
	if(obj==NULL)
	{
		EM.Error(MUSICAL_ERR_AREA,"EC_OGG_NOTFOUND","Partitura","TwoSel()");
		return FALSE;
	}
	
	
	flag=FALSE;
	
	
	switch(UsrSymCmdTwoSel.CommandId)
	{
		
    case CMD_PARTITURA_ADD_GRAFFA:
    case CMD_PARTITURA_ADD_QUADRA:
	case CMD_PARTITURA_MOVE_PART:
		if (SymCommand(UsrSymCmdTwoSel)== CMD_OK)
			flag = TRUE;
		
		break;
		
    default:	
		
		if (tipoLeggio==LEGGIO_MASAE)
		{
			// Esegue immediatamente il comando impostato!
			
			if (DoCommandInterv(UsrSymCmdTwoSel))
			{
				NetInvioComando(UsrSymCmdTwoSel);
				flag=TRUE;
			}
			else
				flag=FALSE;
		}
		else
		{
#ifdef WITH_NET   
			NetInvioComando(UsrSymCmdTwoSel);
			flag=TRUE;
#else
			// Esegue immediatamente il comando impostato!
			if (DoCommandInterv(UsrSymCmdTwoSel)) 
			{
				NetInvioComando(UsrSymCmdTwoSel); 
				flag=TRUE;
			}
			else
				flag=FALSE;
#endif
		}
	}
	
	
	if (flag)
	{
		Redraw();
		CopiaCommandStruct(&UsrSymCmd,&UsrSymCmdTwoSel);
		ResetPath(UsrSymCmdTwoSel);
		FirstSel = TRUE;
		return TRUE;
	}
	else
	{
		EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","TwoSel()");
		ResetPath(UsrSymCmd);
		ResetPath(UsrSymCmdTwoSel);
		CopiaCommandStruct(&UsrSymCmd,&UsrSymCmdTwoSel);
		return FALSE;
	}
	
}


Bool Partitura::SingleSel(const Point& p)
{
	
	DrawObject *obj;
	
	// chiede se in p ci sono oggetti da selezionare
	ResetPath(UsrSymCmd);
	Hit(p,UsrSymCmd.SelObjType,obj,UsrSymCmd.Path1);
	
	// se obj e' NULL allora non c'e' stata selezione
	
	if(obj==NULL)
	{
		EM.Error(MUSICAL_ERR_AREA,"EC_OGG_NOTFOUND","Partitura","SingleSel()");
		return FALSE;
	}
	
	
	if(listaSel.FindPtr(obj))
	{
		// se l'oggetto e' gia' stato selezionato allora deve essere deselezionato
		listaSel.DelPtr(obj);
		obj->Deselect();
	}
	else
	{    
		// se e' gia' stato selezionato un oggetto va deselezionato.  
		if(listaSel.GetNumObj()!=0)
		{
			DrawObject *lst=listaSel.DelLast();
			lst->Deselect(); 
			//   se l' oggetto e' in una pagina non visibile ha 
			// dei problemi
		}
		// inserisce il nuovo oggetto e lo seleziona.
		listaSel.InsertEnd(obj);
		obj->Select();
	}
	
	
	
	
	DeselectAll();
	
	if(DoCommandGruppo())
	{
		Redraw();
		return TRUE;
	}
	else
		return FALSE;
}



// MASAE: esegue e manda a DLIOO.
// DLIOO: non abilitato.
//
Bool Partitura::DoCommandAddBattuta(const Point& p)
{
	DrawObject *pobj;
	Spartito *psp;
	
	psp = GetFirstSpa();
	do 
	{
		ResetPath(UsrSymCmd);
		psp->Hit(p,UsrSymCmd.SelObjType,pobj,UsrSymCmd.Path1);

		if(pobj==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_OGG_NOTFOUND","Partitura","DoCommandAddBattuta()");
			return FALSE; 
		}

		if (tipoLeggio==LEGGIO_MASAE)
		{
			if (psp->SymCommand(UsrSymCmd)==CMD_OK)
			{
				SetStatoEsecuzione(IMPAGINAZIONE);
				NetInvioComando(UsrSymCmd);
			}
			else
				return FALSE;
		}
		else
		{
#ifndef WITH_NET   
			if (psp->SymCommand(UsrSymCmd)!=CMD_OK)
				return FALSE;
#endif
		}

		psp = GetNextSpa(psp);
	} while(psp!=NULL);

	return TRUE; // Sulla fiducia!
}


// 
// Elimina una colonna di battute
//
// MASAE: esegue e manda a DLIOO.
// DLIOO: non e' abilitato
//
Bool Partitura::DoCommandDelBat(const Point& p)
{
	DrawObject *pobj=NULL;
	Spartito *psp;
	
	psp = GetFirstSpa();
	
	
	do
	{
		ResetPath(UsrSymCmd);
		psp->Hit(p,UsrSymCmd.SelObjType,pobj,UsrSymCmd.Path1);
		if(pobj==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_OGG_NOTFOUND","Partitura","DoCommandDelBat()");
			return FALSE; 
		}
		
		
		if (tipoLeggio==LEGGIO_MASAE)
		{
			if (psp->SymCommand(UsrSymCmd)==CMD_OK)
			{
				SetStatoEsecuzione(IMPAGINAZIONE);      
				NetInvioComando(UsrSymCmd);
			}
			else
				return FALSE;
		}
		else
		{
			
#ifndef WITH_NET   
			if (psp->SymCommand(UsrSymCmd)!=CMD_OK)
				return FALSE;
#endif
		}
		
		psp = GetNextSpa(psp);
	}  while(psp!=NULL);
	
	return TRUE; 
}


// MASAE: esegue e manda a DLIOO.
// DLIOO: non e' abilitato
//
Bool Partitura::DoCommandBarra(const Point& p)
{
	DrawObject *pobj;
	Spartito *psp;
	
	psp = GetFirstSpa();
	
	do
	{
		ResetPath(UsrSymCmd);
		psp->Hit(p,UsrSymCmd.SelObjType,pobj,UsrSymCmd.Path1);
		if(pobj==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_OGG_NOTFOUND","Partitura","DoCommandBarra()");
			return FALSE; 
		}
		
		if (tipoLeggio==LEGGIO_MASAE)
		{
			if (pobj->SymCommand(UsrSymCmd)==CMD_OK)
			{
				SetStatoEsecuzione(IMPAGINAZIONE);      
				NetInvioComando(UsrSymCmd);
			}
			else
				return FALSE;
		}
		else
		{
#ifndef WITH_NET   
			if (pobj->SymCommand(UsrSymCmd)!=CMD_OK)
				return FALSE;
#endif
		}
		
		psp = GetNextSpa(psp);
	}  while(psp!=NULL);
	
	return TRUE; 
}


Bool Partitura::DoCommandGruppo()
{
	short layer;
	DrawObject *pObj;
	
	switch (UsrSymCmd.CommandId)
	{
		
		// MASAE: esegue e manda a DLIOO.
		// DLIOO: comunica a MASAE
		//
    case CMD_BATTUTA_NOTE2GRUPPONOTE:
		// Controllo preliminare: le figure selezionate appartengono alla stessa
		// battuta ed allo stesso layer.
	case CMD_SPARTITO_ADD_BEAM:
		if(UsrSymCmd.Path1.Array[BATTUTA_PATH_LEVEL]==0 ||
			UsrSymCmd.Path1.Array[BATTUTA_PATH_LEVEL]!=
			UsrSymCmd.Path2.Array[BATTUTA_PATH_LEVEL])
		{
			EM.Error(GENERAL_ERR_AREA,"EC_SELEZIONE_ERRATA","Partitura","DoCommandGruppo()");
			return FALSE;
		}
		layer = UsrSymCmd.Path1.Array[LAYER_PATH_LEVEL]-1;//GB
		if(layer==-1 || layer!=UsrSymCmd.Path2.Array[LAYER_PATH_LEVEL]-1)//GB
		{
			EM.Error(GENERAL_ERR_AREA,"EC_SELEZIONE_ERRATA","Partitura","DoCommandGruppo()");
			return FALSE;
		}
		
		// Rintraccia la battuta.
		SymbolicHit(UsrSymCmd.Path1,BATTUTA_SEL,pObj);
		if(pObj!=NULL)
		{     
			//	Imposta gli argomenti simbolici
			UsrSymCmd.ArgV[0] = (ArgType)layer+1;
			UsrSymCmd.ArgV[1] = (ArgType)(UsrSymCmd.Path1.Array[FIGURA1_PATH_LEVEL]);
			UsrSymCmd.ArgV[2] = (ArgType)(UsrSymCmd.Path2.Array[FIGURA1_PATH_LEVEL]);
			
			if (((Battuta*)pObj)->SymCommand(UsrSymCmd)==CMD_OK) 
			{
				NetInvioComando(UsrSymCmd);
				return TRUE;
			}
			else
				return FALSE;  
		}
		
        EM.Error(MUSICAL_ERR_AREA,"EC_BAT_NOTFOUND","Partitura","DoCommandGruppo()");
		return FALSE;
		
		break;
		
		// MASAE: esegue e manda a DLIOO.
		// DLIOO: comunica a MASAE
		//
    case CMD_BATTUTA_GRUPPONOTE2NOTE:
		// Rintraccia la battuta.
		SymbolicHit(UsrSymCmd.Path1,BATTUTA_SEL,pObj);
		if(pObj!=NULL)
		{
			// L'argomento e` gia` stato impostato
			if (((Battuta*)pObj)->SymCommand(UsrSymCmd)==CMD_OK)
			{
				NetInvioComando(UsrSymCmd);
				return TRUE;
			}
			else
				return FALSE;  
		}
        EM.Error(MUSICAL_ERR_AREA,"EC_BAT_NOTFOUND","Partitura","DoCommandGruppo()");
		return FALSE;
		
		break;
    default: ;
	}
    
	return FALSE;
}


Bool Partitura::DoCommandDelColFig(const Point& p)
{
	Spartito *psp;
	DrawObject *obj;
	
	psp = GetFirstSpa();
	
	do 
	{
		// chiede allo spartito se in p ci sono oggetti da selezionare
		ResetPath(UsrSymCmd);
		psp->Hit(p,UsrSymCmd.SelObjType,obj,UsrSymCmd.Path1);
		if(obj==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_OGG_NOTFOUND","Partitura","DoCommandDelColFig()");
			return FALSE; 
		}    
		
		// : LA FUNZIONE RICHIAMATA ELIMINA I GRUPPI NELLA
		// BATTUTA. FARE IN MODO CHE LO STESSO ACCADA ANCHE NELLO SLAVE.
		((Battuta*)obj)->DeleteHit(p,UsrSymCmd.ArgV[0],UsrSymCmd.ArgV[5]);
		
		
		
		if (psp->SymCommand(UsrSymCmd)==CMD_OK)
		{
			SetStatoEsecuzione(IMPAGINAZIONE);      
			NetInvioComando(UsrSymCmd);
		}
		
		psp = GetNextSpa(psp);    
	} while(psp!=NULL);
	
	return TRUE;
}




CmdResult Partitura::SymCommand(SymCmd& SCmd, Battuta *pBattuta)
{ 
	DrawObject *pobj, *pobj1;
	Spartito *ps1, *ps2, *ps,*nextps1, *prevps1;
	Parentesi *ptrBracket;
	NumCode nc;
	
	switch(SCmd.CommandId)
	{
    case CMD_SPARTITO_SET_NAME:
        // Rintraccia lo spartito
		SymbolicHit(SCmd.Path1,SPARTITO_SEL,pobj);
		if(pobj!=NULL)
		{
			((Spartito*)pobj)->SetStrumEsec(SCmd.Text);
			return CMD_OK;
		}
		else
			return CMD_FAIL;
		
	case CMD_PARTITURA_HIDE_PART:
		SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj);
		if(pobj!=NULL)
		{
			HidePart((Spartito*)pobj); 
			return CMD_OK;
		}
		else
			return CMD_FAIL;
		break;
		
	case CMD_PARTITURA_SPLITTING_PART:
		SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj);
		if(pobj!=NULL)
		{
			SplittingPart((Spartito*)pobj); 
			return CMD_OK;
		}
		else
			return CMD_FAIL;
		break;
		
	case CMD_PARTITURA_DISJOINING_PART:
		SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj);
		if(pobj!=NULL)
		{
			DisjoiningPart((Spartito*)pobj); 
			return CMD_OK;
		}
		else
			return CMD_FAIL;
		break;
		
    case CMD_PARTITURA_NEW_LYRIC:
		SymbolicHit(SCmd.Path1,SPARTITO_SEL,pobj);
		if(pobj!=NULL)
		{
			((Spartito*)pobj)->NewLyric();
			return CMD_OK;
		}
		else
			return CMD_FAIL;
		break;
		
	case CMD_PARTITURA_EDIT_LYRIC:
		SymbolicHit(SCmd.Path1,SPARTITO_SEL,pobj);
		if(pobj!=NULL)
		{
			((Spartito*)pobj)->EditLyric();
			return CMD_OK;
		}
		else
			return CMD_FAIL;
		break;
		
	case CMD_PARTITURA_REMOVE_LYRIC:
		SymbolicHit(SCmd.Path1,SPARTITO_SEL,pobj);
		if(pobj!=NULL)
		{
			((Spartito*)pobj)->RemoveLyr();
			((Spartito*)pobj)->SetCurrentLyrWF(NULL);
			return CMD_OK;
		}
		else
			return CMD_FAIL;
		break;
		
	case CMD_PARTITURA_SELECT_LYRIC:
		SymbolicHit(SCmd.Path1,SPARTITO_SEL,pobj);
		if(pobj!=NULL)
		{
			((Spartito*)pobj)->SelectLyric();
			return CMD_OK;
		}
		else
			return CMD_FAIL;
		break;
		
		
		
		//////////////////////////////////////////////////////
		
		
	case CMD_PARTITURA_VIEW_LAYER:
        // Rintraccia lo spartito
		SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj);
		if(pobj!=NULL)
		{
			ViewLayerInPart((Spartito*)pobj); 
			return CMD_OK;
		}
		else
			return CMD_FAIL;
		break;
		
		
	case CMD_PARTITURA_ADD_GRAFFA:
    case CMD_PARTITURA_ADD_QUADRA:
		SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj);
		if (pobj==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","SymCommand()");
			return CMD_FAIL;
		}
		
		SymbolicHit(SCmd.Path2,SCmd.SelObjType,pobj1);
		if (pobj1==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","SymCommand()");
			return CMD_FAIL;
		}
		
		ps1 = (Spartito *)pobj;
		ps2 = (Spartito *)pobj1;
		
		if (listaSpa->GetPos(ps1) > listaSpa->GetPos(ps2))
		{
			ps = ps1;
			ps1 = ps2;
			ps2 = ps;
		}
		
		if(listaPar->Add(SCmd.CommandId, ps1, ps2, nc))
		{
			SCmd.ArgV[6] = nc;
			return CMD_OK;
		}
		else
			return CMD_FAIL;
		
	case CMD_PARTITURA_MOVE_PART:  
		SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj);
		if (pobj==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","SymCommand()");
			return CMD_FAIL;
		}
		
		SymbolicHit(SCmd.Path2,SCmd.SelObjType,pobj1);
		if (pobj1==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_SPA_NOTFOUND","Partitura","SymCommand()");
			return CMD_FAIL;
		}
		
		ps1 = (Spartito *)pobj; //Primo Spartito selezionato
		ps2 = (Spartito *)pobj1; //Spartito dopo il quale inserire
		
		nextps1=(Spartito*) listaSpa->GetNext((DrawObject*)ps1);
		prevps1=(Spartito*) listaSpa->GetPrev((DrawObject*)ps1);
		
		if(ps1!=ps2)
		{		
			MusicEditor->smrf->ExchangeParts(listaSpa->GetPos(ps1)+1,listaSpa->GetPos(ps2)+1);
			listaSpa->Del(ps1,false);
			listaSpa->AddAfter(ps1,ps2,false);
			
			
			ptrBracket=listaPar->GetFirstPar();
			while(ptrBracket!=NULL)
			{
				if(ps1==ptrBracket->GetPSpaStart())
				{
					Spartito *tmp=(Spartito*) listaSpa->GetNext((DrawObject*)ps1);
					bool ok=false;
					while(tmp!=NULL)
					{
						if(tmp==ptrBracket->GetPSpaEnd())
						{
							ok=true;
							break;
						}
						tmp=(Spartito*) listaSpa->GetNext((DrawObject*)tmp);
					}
					if(!ok)
					{
						if(nextps1!=NULL)
							ptrBracket->SetPSpaStart(nextps1);
						else 
							ptrBracket->SetPSpaEnd(ptrBracket->GetPSpaStart());
						
					}
				}
				else if(ps1==ptrBracket->GetPSpaEnd())
				{
					Spartito *tmp=(Spartito*) listaSpa->GetPrev((DrawObject*)ps1);
					bool ok=false;
					while(tmp!=NULL)
					{
						if(tmp==ptrBracket->GetPSpaStart())
						{
							ok=true;
							break;
						}
						tmp=(Spartito*) listaSpa->GetPrev((DrawObject*)tmp);
					}
					if(!ok)
					{
						if(prevps1!=NULL)
							ptrBracket->SetPSpaEnd(prevps1);
						else
							ptrBracket->SetPSpaStart(ptrBracket->GetPSpaEnd());
						
					}
					
				}
				
				
				//Eliminazione della parentesi in caso di degenerazione a singolo pentagramma con parentesi
				//Legge di evoluzione del ciclo while
				if(ptrBracket->GetPSpaEnd()==ptrBracket->GetPSpaStart())
				{
					Parentesi *tmpBracket=ptrBracket;
					ptrBracket=listaPar->GetNextPar(ptrBracket);
					listaPar->DelPtr((DrawObject*)tmpBracket);
					if(tmpBracket!=NULL)
						delete tmpBracket;
				}
				else
					ptrBracket=listaPar->GetNextPar(ptrBracket);
				
			}
			
			
		}
		
		
		
		return CMD_OK;
		
		
    default:
		EM.Error(GENERAL_ERR_AREA,"EC_CMD_GESTIONE_ERRATA","Partitura","SymCommand()");
  }
  
  return CMD_UNKNOWN;
}


Bool Partitura::DeleteSym(DrawObject *sym)
{
	
	if(GetPrintingStatus())
	{
		if(GetCurrentScore()==0)
		{
			if(listaPag->DeleteSym(sym,listaSpa->GetCurrentPage()))	//DB
				return TRUE;
		}
		else
			if(GetSpartNum(GetCurrentScore()-1)->DeleteSym(sym))
				return TRUE;
	}
	
	if(listaSpa->DeleteSym(sym))
		return TRUE;
	else if (listaPar->DeleteSym(sym))
		return TRUE;
	else
	{
		EM.Error(GENERAL_ERR_AREA,"EC_CMD_FAULT","Partitura","DeleteSym()");
		return FALSE;
	}
}


Bool Partitura::DelSpartito(SymCmd SCmd)
{
	DrawObject *pobj;
	
	SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj);
	
	
	if (pobj!=NULL)
	{
		
		if (GetNextSpa(GetFirstSpa())==NULL)
		{
			
			// se c'e' un solo spartito non lo elimina, ma fa una New();
			Scr->GSetMainDrawArea();
			New();
			GoTop();
			
			return TRUE;
		}
		
		listaPar->CancParentesiSpa((Spartito *)pobj);
		listaSpa->Del((Spartito *)pobj);
		
		return TRUE;
		
	}
	
	return FALSE;
}


void Partitura::NetInvioComando(SymCmd SC)
{
}


void Partitura::ReadFromNetwork()
{
}

Bool Partitura::DoNetSymCmd()
{
	return FALSE;
}

void Partitura::LoadPindex() 
{
}


void Partitura::InitParte() 
{
}



void Partitura :: SetFileLoadName(char* fName)
{
	fileLoadName = wxString(fName);
}


char* Partitura :: GetFileLoadName()
{
	return ((char*)fileLoadName.c_str());
}

void Partitura :: SetPathLoadName(char* fName)
{
	fileLoadPath = wxString(fName);
}


char* Partitura :: GetPathLoadName()
{
	return ((char*)fileLoadPath.c_str());
}

wxString Partitura :: GetfileLoadPath()
{
	return fileLoadPath;
}


void Partitura :: SetFileSaveName(char* fName)
{
	fileSaveName = wxString(fName);
}


char* Partitura :: GetFileSaveName()
{
	return (char*)fileSaveName.c_str();
}

void Partitura :: SetPathSaveName(char* fName)
{
	fileSavePath = wxString(fName);
}


char* Partitura :: GetPathSaveName()
{
	return (char*)fileSavePath.c_str();
}


Bool Partitura::Load(char *fName)
{ 
	wxString dirx="";
	
	FullFree();
	StatoEsecuzione = IMPAGINAZIONE;
	
	
#ifdef DEBUG_TIME
	struct timeval timevI, timevF;
	struct timezone timezI, timezF;
	double time;
	
	gettimeofday(&timevI,&timezI);
	
	
	gettimeofday(&timevF,&timezF);
	time = (double)((double)((timevF.tv_sec-timevI.tv_sec)*1000000 + timevF.tv_usec - timevI.tv_usec)/1000);
	
	gettimeofday(&timevI,&timezI);
	
	listaSpa->LoadLyr(dirx);
	
	gettimeofday(&timevF,&timezF);
	time = (double)((double)((timevF.tv_sec-timevI.tv_sec)*1000000 + timevF.tv_usec - timevI.tv_usec)/1000);
	
#else
	
	
	
	listaSpa->LoadLyr((char*)dirx.c_str());
	
	
#endif
	
	
	char *str;
	str = new char[256];
	if(!str)
		EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Partitura","Load()");
	
	strcpy(str,"Archivist");
	
	// se non ci sono spartiti inizializza con lo spartito standard.
	if(listaSpa->GetNumObj()==0)
	{
		Init();
		if(pMusicWin)
			pMusicWin->SetTitle((unsigned char *)str);
		delete[] str;
		return TRUE;
	}
	
	if(tipoLeggio==LEGGIO_MASAE)
	{
		strcat(str," - ");
		strcat(str,fName);
		if(pMusicWin)
			pMusicWin->SetTitle((unsigned char *)str);
	}
	
	
	
	SetupIntRefs();
	delete[] str;
	return TRUE;
	
}

Bool Partitura::LoadS(char offset)
{       
	return TRUE;
}

void Partitura::InitTimeExec()
{
	if(ETTLenght<GetNumeroBattute())
		BuildETT(GetNumeroBattute()+1);
	Spartito* psp = GetFirstSpa();
	Battuta* pbat = psp->GetBattNum(0);
	while (pbat != NULL)
	{
		SetTimeExec(pbat);
		pbat = psp->GetNextBat(pbat);
	}
}

void Partitura::SetTimeExec(Battuta *pbt)
{
	Battuta *pbt2;
	int id = pbt->GetNumProgress();
	float time, timeTempo;
	
	if (ETT==NULL)
		EM.Error(GENERAL_ERR_AREA,"EC_NO_VETTORE_ETT","Partitura","SetTimeExec");
	else  
	{
		if ((id<ETTLenght) && (id>0))
		{
			
			time = pbt->GetMetronomoTimeExec();
			if (time<=0) 
			{
				time = (float) ETT[id-1];
				
				pbt2 = GetFirstSpa()->GetPrevBat(pbt);
				if (pbt2!=NULL)
				{
					timeTempo = pbt->GetTempoTimeExec();
					if (timeTempo != pbt2->GetTempoTimeExec())
					{
						timeTempo /= pbt2->GetTempoTimeExec();
						time *= timeTempo;
					}
				}
			}
			else
			{
				time *= pbt->GetTempoTimeExec();
			}
			
			ETT[id] = (long) time;
			
		}
	}
}


long Partitura::GetTimeExec(int id)
{
	
	if (ETT==NULL)
	{  EM.Error(GENERAL_ERR_AREA,"EC_NO_VETTORE_ETT","Partitura","GetTimeExec");
	return 0;
	};       
	
	if (id<ETTLenght)
		return(ETT[id]);
	else
		return -1;
}     



void Partitura::SaveETT(char* filename)
{
	
	FILE *ettfile;
	int control_file;
	NumCode ind_batt;
	
	if (ETT==NULL)
	{  
		EM.Error(GENERAL_ERR_AREA,"EC_NO_VETTORE_ETT","Partitura","SaveETT");
		return;
	};  
	
	ettfile=fopen(filename,"w");
	if (ettfile==NULL) 
	{ 
		ettfile=fopen(MusicEditor->smrf->GetTmpPath()+PATH_SEP+wxFileNameFromPath(filename),"w");
		if(ettfile==NULL)
			return;
		// gestione errore file
	}
	
	
	// 1) scrivo il primo campo del file dove c'e' il nome e il numero delle
	//  battute
	
	control_file = fprintf(ettfile,"SYNC sym \"%s\" %d\n", audioFile, ETTLenght);
	if ( control_file <= 0)
		EM.Error(GENERAL_ERR_AREA,"EC_ERR_IN_FILE","Partitura","SaveETT"); 
	
	// 2) scrivo le informazioni come :   indice_battuta : durata batt 
	
    ind_batt=0;  
    while (ind_batt < ETTLenght)
    {
		if ( measureSync != NULL )  control_file = fprintf(ettfile,"%d %ld %d F\n",ind_batt,ETT[ind_batt],measureSync[ind_batt] );//selmi
		else  control_file = fprintf(ettfile,"%d %ld %d\n",ind_batt,ETT[ind_batt],0 );
		if ( control_file <= 0)
			EM.Error(GENERAL_ERR_AREA,"EC_ERR_IN_FILE","Partitura","SaveETT");        
		ind_batt++;   
    };
	
	fclose(ettfile);
	
}


///////
#include <time.h>

void Partitura::SetTimeExec(int ind_batt, long time)
{
	static long timer=0;
	int currentBlock = 0;
	
    if (ETTLenght>0)
		if ((ind_batt>=0) && (ind_batt<ETTLenght))
		{
			{
				long ora=clock();
				if(ind_batt>1)
				{
					time=ora-timer;
					ind_batt--;
				}
				timer=ora;
				
				ETT[ind_batt] = time; 
			}
			
			// sincronizzazione delle battute con i blocchi audio : 
			if ( pAudioThread != NULL )
			{
				if(measureSync==NULL) measureSync=new int[ETTLenght+1];
				currentBlock = pAudioThread->GetCurrentBlock();//per ridurre il numero delle sezioni critiche
				if ( mmExecMode == MM_SYNC )  measureSync[ind_batt] = currentBlock;
				else if( measureSync[ind_batt] > 0 )
				{
					if ( currentBlock < measureSync[ind_batt] - 2)
						pMusicWin->DecVelocEsec(VIDEO_STEP*((measureSync[ind_batt]-currentBlock)/3));// quel 2 (quindi anche il 3) serve per evitare l'innescamento di oscillazioni, e' del tutto empirico
					else if ( currentBlock > measureSync[ind_batt] + 2 )
						pMusicWin->IncVelocEsec(VIDEO_STEP*((currentBlock-measureSync[ind_batt])/3)   );// 1 e'la pwercentuale di incremento della velocita' video
				}																										 
			}
		}
}

void Partitura::ReadETT(char *filename, char *audioname)
{
	ReadETT(filename);
	strcpy(audioname,audioFile);
	return;
}


void Partitura::ReadETT(char *filename)
{
	
	FILE *ettfile;  
	int ind_batt;
	short control_file;
	short i,EndOfFile,ItemsMatched;
	long prova;
	int block;
	char line[100];
	char symImg[20];
	
	
	if(filename!=NULL)
		strcpy(ETTFileName,filename);
	
	ettfile=fopen(ETTFileName,"r");
	if (ettfile==NULL) 
	{ 
		ettfile=fopen(MusicEditor->smrf->GetTmpPath()+PATH_SEP+wxFileNameFromPath(filename),"r");
		if(ettfile==NULL)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_OPENFILEREAD","Partitura","ReadETT");
			return;
		}
		// gestione errore file
	}
	
	// 1) leggo il primo campo del file dove c'e' il numero delle battute
	fgets(line,100,ettfile);
    control_file = sscanf (line,"SYNC %s \"%[^\"]\" %d", symImg, audioFile,&ind_batt);
    if ( control_file != 3)
	{
		control_file = sscanf (line,"%s %d", audioFile,&ind_batt);
		if ( control_file != 2)
		{
			EM.Error(GENERAL_ERR_AREA,"EC_ERR_IN_FILE","Partitura","ReadETT"); 
		}
	}
	
	ETTLenght = (NumCode)ind_batt;
    if (ETT!=NULL) delete[] ETT;
    ETT=new long[ETTLenght];
    if(measureSync!=NULL) delete[] measureSync;
	measureSync=new int[ETTLenght];
	
	
	// 2) leggo le informazioni come :   num batt%durata batt 
	
	
	ind_batt=0;     
	i=0;
	EndOfFile=0;
	
	
	///////////////////////////////////////////////////////////////////////////////selmi :
	while (1)
	{  
		char dummy[10];
		
		if(fgets(line,100,ettfile)==NULL)
			break;
		
		ItemsMatched = sscanf (line,"%d %ld %d %s",&ind_batt,&prova,&block,dummy);
		
		
		switch(ItemsMatched)
		{
		case 2:// Linea letta correttamente,  ETT di vecchio tipo              
			ETT[ind_batt] = prova;
			measureSync[ind_batt] = 0;
			i++;
			break;
		case 3:// Linea letta correttamente,  ETT ultimo modello       
		case 4:// Linea letta correttamente,  ETT ultimo modello       
			ETT[ind_batt] = prova;
			measureSync[ind_batt] = block;
			i++;
			break;
		default: // Errore nel matching
			EM.Error(GENERAL_ERR_AREA,"EC_ERR_IN_FILE","Partitura","ReadETT"); 
			exit(0);  
		}      
    };
	///////////////////////////////////////////////////////////////////////////////////////
	
	
	fclose(ettfile);
}   


void Partitura::BuildETT(NumCode num_batt)
{
	
	int i;
	
	if(num_batt>0)
	{
		ETTLenght = num_batt;
		if (ETT!=NULL) delete[] ETT;
		ETT=new long[ETTLenght];
		
		for(i=0; i < ETTLenght ; i++)
			ETT[i] = 0;
	}
	
}

void Partitura::SincrBatt(NumCode nc)
{}





void Partitura::InitEsecuzione(char* name)
{
}


void Partitura::InitImpaginazione()
{
	
}

void Partitura::AddEtichetta(char c, NumCode n)
{
	ListaEtichette *le, *prec;
	
	
	if (FindEtichetta(c)) return;
	
	if (listaEti == NULL)
	{
		listaEti = new ListaEtichette;
		le = listaEti;
		prec = NULL;
	}
	else
	{
		le = new ListaEtichette;
		prec = listaEti;
		while (prec->pNext != NULL)
			prec = prec->pNext;
	}
    
	le->etic = c;
	le->nc = n;
	
	le->pNext = NULL;
	
	if (prec != NULL)
		prec->pNext = le;
	
}


// restituisce l'etichetta di posizione n-esima
char Partitura::GetEtichetta(int n)
{
	ListaEtichette *prec = listaEti;
	int i;
	
	for (i=0;i<n;i++) if (prec != NULL) prec = prec->pNext;
	
	if (prec != NULL) return prec->etic;
	else return -1;
}


Bool Partitura::FindEtichetta(char c)
{
	ListaEtichette *prec = listaEti;
	
	while (prec != NULL)
	{
		if (prec->etic == c) return TRUE;
		prec = prec->pNext;
	}
	
	return FALSE;
}

Bool Partitura::DelEtichetta(char c)
{
	ListaEtichette *prec = listaEti, *tmp;
	tmp = prec;
	
	if(tmp==NULL)
		return FALSE;
	
	if (tmp->etic == c) //E' la prima della lista
	{
		listaEti = tmp->pNext;
		delete tmp;
		return TRUE;
	}
	
	while (tmp != NULL) //E' dentro la lista
	{
		if (tmp->etic == c) 
		{
			prec->pNext = tmp->pNext;
			delete tmp;
			return TRUE;
		}
		
		prec = tmp;
		tmp = tmp->pNext;
	}
	
	return FALSE;
}

int Partitura::GetNumEtichette()
{
	ListaEtichette *prec = listaEti;
	int i;
	
	for (i=0; (prec != NULL); i++) prec = prec->pNext;
	
	return i;
}



int Partitura::GetNumeroBattute()
{
	return GetFirstSpa()->GetNumeroBattute();
}


NumCode Partitura::GetBattEtichetta(char c)
{
	ListaEtichette *prec = listaEti;
	NumCode nnumc = 0;
	Battuta* pbat;
	
	while (prec != NULL)
	{
		if (prec->etic == c) 
		{
			nnumc = prec->nc;
			break;
		}
		
		prec = prec->pNext;
	}
	
	if (nnumc > 0)
	{
		pbat = GetFirstSpa()->GetBatt(nnumc);
		if (pbat==NULL)
		{
			EM.Error(MUSICAL_ERR_AREA,"EC_BAT_NOTFOUND","Partitura","GetBattEtichetta()");
			return FALSE; 
		}
		return pbat->GetNumProgress();
	}
    
	else
		return 0;
}



Bool Partitura::OkCompila()
{
	char msg[100] = "Score doesn't need to be compiled. Compile anyway?";
	char msg2[100] = "Compile?";
	
	if (GetStatoEsecuzione() > IMPAGINAZIONE)
	{
		if (YNMessage(msg," Compiling "))
			return TRUE;
		else
			return FALSE;
	}
	
	if ( YNMessage(msg2," Compiling "))
		return TRUE;  
	else
		return FALSE;  
}


Bool Partitura::OkInitEsec()
{
	if (GetStatoEsecuzione() < INIT_ESECUZIONE)
	{
		Message("Score not compiled!","Error!");
		return FALSE;
	}
	return TRUE;  
}

Bool Partitura::OkEsecuzione()
{
	if (GetStatoEsecuzione() < INIT_ESECUZIONE)
	{
		Message("Score not compiled!","Error!");
		return FALSE;
	}
	if (GetStatoEsecuzione() <= INIT_ESECUZIONE)
	{
		Message("Score not initialized!","Error!");
		return FALSE;
	}
	if (GetStatoEsecuzione() < ESECUZIONE)
	{
		Message("Score must be initialized again (or use Jump)!","Error!");
		return FALSE;
	}
	return TRUE;  
}


Bool Partitura::OkJump()
{
	if (GetStatoEsecuzione() < INIT_ESECUZIONE)
	{
		Message("Score not compiled!","Error!");
		return FALSE;
	}
	if (GetStatoEsecuzione() <= INIT_ESECUZIONE)
	{
		Message("Score not initialized!","Error!");
		return FALSE;
	}
	return TRUE;  
}


void Partitura::AddParentesiGraffa(NumCode ns, NumCode ne, NumCode nc)
{
	listaPar->Add(CMD_PARTITURA_ADD_GRAFFA,GetSpart(ns), GetSpart(ne), nc);
	
}

void Partitura::AddParentesiQuadra(NumCode ns, NumCode ne, NumCode nc)
{
	listaPar->Add(CMD_PARTITURA_ADD_QUADRA,GetSpart(ns), GetSpart(ne), nc);
}

// Import da file

Bool Partitura::ImportMidi(const char *fName, int trc, int keySignCode)
{
#ifdef WX226
  MidiImport   midi;
  Midi_buffer_IO m_buf;
  Midi_converter* pConv=NULL;
  char *str;
  const char *nomeF;
  extern wxString keyM[], keym[];

  try
  {  
	  wxString new_key;
	  if(keySignCode>=0 && keySignCode < 30)
	  {
		  if(keySignCode<15)
			  new_key = keym[keySignCode];
		  else
			  new_key = keyM[keySignCode-15];
	  }
	  if (new_key!="")
	  {
		  if (MIDI_coded_key!="MIDI key not coded")
			midi.Set_MOODS_key(new_key, false);
		  else
			midi.Set_MOODS_key(new_key, true);
	  }
	  FullFree();
	  StatoEsecuzione = IMPAGINAZIONE;

	  nomeF=fName;
	  m_buf.Load_file(nomeF);
	  
	  midi.Set_ibuf(m_buf.Get_iobuf());
	  if (!m_buf.Get_format())
	  {
		  pConv=midi.Midi_conv_create();
		  pConv->Convert_format(SMF1, nomeF);
		  pConv->Clear_struct(CHANNEL_AND_TRACK);
		  midi.Set_source_buf(1);
		  midi.Set_tracks_num(pConv->Get_out_buf().Get_Num_Midi_tracks());
		  midi.Set_End_buf(pConv->Get_out_buf().GetCount());
	  }
	  else 
	  {
		  midi.Set_source_buf(0);
		  midi.Set_End_buf(m_buf.GetCount());
	  }
  
	  if (midi.Start(0))  
	  {
		  if (!m_buf.Get_format()) pConv->Clear_struct(IN_BUFFER);
		  midi.SetPartitura(this);
		  if (trc == 0) 
			  midi.ReadAllTrx();
		  else
		      midi.ReadTrk(trc);
		  if (trc) ScoreJustify(); 
		      loadError=0;
	  }
	  else loadError=1;
	  m_buf.Clear();
  }

  CATCHWXSTRING

  str = new char[256];
  if(!str)
    EM.Error(GENERAL_ERR_AREA,"EC_MEMORYFAULT","Partitura","Load()");
  
  strcpy(str,"Archivist");

  if(loadError)
  {
    New();        // Ripulisce la Partitura
	if(pMusicWin!=NULL)
		pMusicWin->SetTitle((unsigned char *)str);
	delete[] str;
    return TRUE;
  }
  
    // se non ci sono spartiti inizializza con lo spartito standard.
  if(listaSpa->GetNumObj()==0)
  {
    Init();
	if(pMusicWin!=NULL)
		pMusicWin->SetTitle((unsigned char *)str);
	delete[] str;
    return TRUE;
  }

  if(tipoLeggio==LEGGIO_MASAE)
  {
	strcat(str," - ");
	strcat(str, fName); 
	if(pMusicWin!=NULL)
		pMusicWin->SetTitle((unsigned char *)str);
  }
    
  delete[] str;
#endif
  return TRUE;
}

Bool Partitura::ExportMidi(char* fName,int n)
{
#ifdef WX226
		wxString filepath=wxFileSelector("Save MIDI","","",
  			"","Midi files (*.mid)|*.mid", wxSAVE);
		if (filepath=="") 
			return false ;
			
		MoodsMidiWrite momiwr;
		Midi_converter midiconv;
		momiwr.Set_p_struct(&midiconv.Get_track_struct());
		Suona(&momiwr);
		wxString available_formats[2], message, caption;
		available_formats[0]="SMF0";
		available_formats[1]="SMF1";
		message="Please choose destination format";
		caption="Midi Format";
		wxString format_result=wxGetSingleChoice(message, caption, 2, available_formats);
		Midi_format dest_format=SMF1;
		if (format_result=="SMF0") dest_format=SMF0;
        midiconv.MOODS_SMF_encode(dest_format);
		midiconv.Get_out_buf().Save_to_file(filepath);
		wxMessageBox("Export completed !");  
#endif
    return true;
}

bool Partitura::Play_symbolic(int startMeasure, int endMeasure)
{
#ifdef WX226
	p_sequencer=new Midi_sequencer(FrameManager::currFrame);
	p_sequencer->Set_virtual_player_priority(WXTHREAD_MAX_PRIORITY);
	MoodsSequencerWrite mosew(p_sequencer);
	if(Suona(&mosew))
	{
		int fromBat=1; 
		if(startMeasure>1)
			fromBat=startMeasure;
		else if(pMusicWin!=NULL)
			fromBat=pMusicWin->GetCountBatt();

		p_sequencer->Set_end_measure(endMeasure); 

		p_sequencer->Play(fromBat);
		return true;
	}
	else
#endif
		return false;	
}

MIDI_SYNC_RESULT Partitura::New_Midi_bat(DWORD prev_division, LONG& te,
										 DWORD metr_tempo)
{
#ifdef WX226
	DWORD tempo=((1000000*60)/metr_tempo);
	p_sequencer->Set_new_time(tempo);
	DWORD var;
	if (p_sequencer->start_bat>1)
	{
		p_sequencer->start_bat=1;
		return MIDI_SYNC_OK;
	} 
    MIDI_SYNC_RESULT result=p_sequencer->New_bat(prev_division, &var);
	switch (result)
	{
	case MIDI_SYNC_OK : 
		break;
	case MIDI_SYNC_INC : 
		te=(1000*te)/(1000+var*INC_TRACKING_RATE);
		break;
	case MIDI_SYNC_DEC : 
		te=(1000*te)/(1000-var*DEC_TRACKING_RATE);
		break;
	case MIDI_SYNC_ERROR : 
		break;
	}
	return result;
#else
	return MIDI_SYNC_OK;
#endif
}


void Partitura::ScoreJustify()
{
}

/*
*
*	Questi sono metodi per il Debug
*
*
*/


void Partitura::EseguiDebug(void *arg1, void *arg2, void *arg3) 
{
	FILE *pippo;
	int i;
	int c;
	
	
	switch(UsrSymCmd.CommandId)
	{
    case CMD_DEBUG_UNO:
		
		break;
		
		
    case CMD_DEBUG_DUE:
		// leggo da file una battuta per DLIOO
		
		
		GetFirstSpa()->Free();
		
		
		
		
		pippo = fopen("/home/albino/dial.txt","r");
		i=0;
		while ((c=getc(pippo))!=EOF)
		{
			Array[i]=c;
			i++;
		}
		LoadS(0);
		Redraw();
		Array[0] = '\0'; 
		fclose(pippo);
		break;
		
		
    case CMD_DEBUG_TRE:
		
		break;
		
		
    case CMD_DEBUG_QUATTRO:
    case CMD_DEBUG_CINQUE:
    case CMD_DEBUG_SEI:
    case CMD_DEBUG_SETTE:
    case CMD_DEBUG_OTTO:
    case CMD_DEBUG_NOVE:
		;
	}
	
	
	return;
};

void fStampaSym(FILE *pippo,SymCmd sc)
{ int j;
fprintf(pippo,"%d %d %d ",sc.CommandId,sc.SelectionType,sc.SelObjType);

for (j=0;j<PATH_LENGTH;j++)
{
    fprintf(pippo,"%d ",sc.Path1.Array[j]);
};
fprintf(pippo,"%d ",sc.Path1.Level);

for (j=0;j<PATH_LENGTH;j++)
{
    fprintf(pippo,"%d ",sc.Path2.Array[j]);
};
fprintf(pippo,"%d ",sc.Path1.Level);

for (j=0;j<SYM_MAXARG;j++)
{
    fprintf(pippo,"%d ",sc.ArgV[j]);
};
fprintf(pippo,sc.Text);
}

void StampaSym(SymCmd sc)
{ int j;
printf(""
	   "CommandId = %d\n"
	   "SelectionType = %d\n"
	   "SelObjType = %d\n",sc.CommandId,sc.SelectionType,sc.SelObjType);

for (j=0;j<PATH_LENGTH;j++)
{
    printf("Path1: %d\n",sc.Path1.Array[j]);
};
printf("Path1.Level: %d\n",sc.Path1.Level);

for (j=0;j<PATH_LENGTH;j++)
{
    printf("Path2: %d\n",sc.Path2.Array[j]);
};
printf("Path2.Level: %d\n",sc.Path2.Level);

for (j=0;j<SYM_MAXARG;j++)
{
    printf("Argv(%d): %d\n",j,sc.ArgV[j]);
};
printf("Text: %s\n",sc.Text);

}


int Partitura::GetIDleader(char* parte)
{ 
	return 0;
}


void Partitura::SetAbilCompress(char* parte, Bool abil)
{ 
}

void Partitura::SetAbilCompressAll(Bool abil)
{ 
}

Bool Partitura::GetAbil_Compress(int i)
{ 
	return FALSE;
}

void Partitura::SetModo(bool contr)
{
	SetImg=contr;
}

Bool Partitura::GetModo()
{
	return SetImg;
}


//////////////////////////////////////////////////////////////////selmi:
bool Partitura::InitMmExec(SMRSynchInfoItem* ett)
{ 	
	bool canStart=false;
	wxString ettFile;

	if(ett!=NULL)
	{
		ettFile=MusicEditor->smrf->GetPath()+PATH_SEP+ett->GetFilename();
		ReadETT((char*)ettFile.c_str());
		canStart=true;
	}
	else
	{
		SetAudioFile("noaudio");
		canStart=true;
	}

	return canStart;
}

 
void Partitura::StartMmExec(int startMeasure, int endMeasure)
{ 	

#ifdef __WXGTK__
	Message("Audio not supported on Linux");
	pMusicWin->InitExec();// video
#else
	
	audioRate = 0;
	if ( mmExecMode == MM_USER )  maxNumHit = 5;
	else  maxNumHit = 10;
	
	if(strcmp(audioFile,"noaudio")==0)
	{
#ifdef WX226
		if (p_sequencer) 
			delete p_sequencer;
	    if(Play_symbolic(startMeasure,endMeasure))
		{
			if(pMusicWin!=NULL)
				pMusicWin->InitExec();
			mmExecMode=MM_MIDI;
			return;
		}
#endif
	}
	else
	{
/* TBR
		if(pAudioThread!=NULL)
		{
			pAudioThread->Stop();
		}
		pAudioThread = 0;

		wxString filename;

		filename=MusicEditor->smrf->GetPath()+PATH_SEP+(wxString)audioFile;
		if(startMeasure>=1)
		{
			if(startMeasure==1)
				beginBlock=0;
			else
				beginBlock = measureSync[ startMeasure-1 ];
		}

		int endBlock=-1;
		if(endMeasure>=1 && endMeasure>=startMeasure)
		{
			endBlock = measureSync[ endMeasure ];
		}

		wxInputStream * audioStream = AudioWF::GetWxInputStream(MusicEditor->smrf,filename);
		wxASSERT(audioStream!=0);
		if(filename.MakeLower().Contains(".wav"))
			pAudioThread = AudioManager::GetAudioManager()->NewWaveThread(audioStream, beginBlock, endBlock, FROM_BEGIN, TO_END, true );
		else if(filename.MakeLower().Contains(".mp3"))
			pAudioThread = AudioManager::GetAudioManager()->NewMpegThread(audioStream, beginBlock, endBlock, FROM_BEGIN, TO_END, true );
		if(pAudioThread!=NULL)
		{
			pAudioThread->Run();
		}
		if(pMusicWin!=NULL)
			pMusicWin->InitExec(); //video
*/
    }	
#endif	
}

void Partitura::StopMmExec( )
{ 
	if ((mmExecMode==MM_MIDI)&&(p_sequencer)) 
	{
#ifdef WX226	
		p_sequencer->Stop();
		DELITEM(p_sequencer)
		mmExecMode=MM_USER;
		wxFrame* act_frame=FrameManager::currFrame;
		if(dynamic_cast<wxMEScoreCanvas*>(act_frame->FindWindow(wxMEScoreCanvas_ID))!=NULL)
			act_frame->FindWindow(wxMEScoreCanvas_ID)->AddPendingEvent(wxCommandEvent(EVT_MIDI, ID_STOP));
#endif
	}
/*
	else
		if (pAudioThread!=NULL && pAudioThread->IsAlive())
		{
			pAudioThread->Stop();
			pAudioThread = NULL;
		}
*/

	if (pMusicWin!=NULL && pMusicWin->GetModEsecuzione()) //PB
	   SetStatoEsecuzione(JUMP_ESECUZIONE);
	if(mmExecMode!=MM_SYNC)
		SetAudioFile("noaudio");
	beginBlock = 0;
}

void Partitura::PauseMmExec( )
{
 	if ((mmExecMode==MM_MIDI)&&(p_sequencer)) 
	{
#ifdef WX226
		Midi_sequencer_pause();
#endif
		return;
	}
	if(pMusicWin!=NULL)
	{
		if (pMusicWin->GetWaitState()!=-1)  pMusicWin->SetWaitState(-1);
		else  pMusicWin->SetWaitState(0);
	}

	if ( mmExecMode == MM_USER && pAudioThread!=NULL && pAudioThread->IsAlive() )  pAudioThread->Pause();

}

void Partitura::Midi_sequencer_pause()
{
#ifdef WX226
	if (p_sequencer->Get_state()==PAUSED)
	{
		if(pMusicWin!=NULL)
		{
			pMusicWin->SetVelocEsec(0);
			Midi_struct aux_struct;
			aux_struct=p_sequencer->Get_working_struct();
			DELITEM(p_sequencer)
			p_sequencer=new Midi_sequencer(FrameManager::currFrame);
			p_sequencer->Set_working_struct(aux_struct);
			p_sequencer->restarted=true;
			p_sequencer->Play(pMusicWin->GetCountBatt());
			
			if (pMusicWin->GetWaitState()!=-1)  pMusicWin->SetWaitState(-1);
			else  pMusicWin->SetWaitState(0);
		}
		else
		{
			p_sequencer->Pause();
		}
		return;
	}
	if(pMusicWin!=NULL)
	{
		if (pMusicWin->GetWaitState()!=-1) pMusicWin->SetWaitState(-1);
		else  pMusicWin->SetWaitState(0);
		p_sequencer->Stop();
		p_sequencer->Set_state(PAUSED);
		wxFrame* act_frame=FrameManager::currFrame;
		if(dynamic_cast<wxMEScoreCanvas*>(act_frame->FindWindow(wxMEScoreCanvas_ID))!=NULL)
			act_frame->FindWindow(wxMEScoreCanvas_ID)->AddPendingEvent(wxCommandEvent(EVT_MIDI, ID_STOP));
	}
	else
	{
		p_sequencer->Pause();
	}
#endif
}

void Partitura::SlowerMmExec( )
{
	if ((mmExecMode==MM_MIDI)&&(p_sequencer)) 
	{
		if (pMusicWin!=NULL) pMusicWin->DecVelocEsec(MIDI_SPEED_VAR_STEP);
		return;
	}

	/////////////////////////////////////////////////

	if ( pMusicWin!=NULL && pMusicWin->GetNumHit() <= -maxNumHit )
		return;
	else
	{
        if ( mmExecMode == MM_USER 	&& pAudioThread!=NULL && pAudioThread->IsAlive())
		{	 
			if ( pAudioThread->Slower() )  
			{	 
				audioRate = audioRate + 1;
				if (pMusicWin!=NULL) pMusicWin->DecVelocEsec(VIDEO_STEP);// 1 e'la percentuale di decremento della velocita'video
			}
		}
		else  
			if (pMusicWin!=NULL) pMusicWin->DecVelocEsec(VIDEO_STEP);//MM_SYNC
		if (pMusicWin!=NULL) pMusicWin->SetNumHit( pMusicWin->GetNumHit() - 1 );
	}
}

void Partitura::FasterMmExec( )
{
	if ((mmExecMode==MM_MIDI)&&(p_sequencer)) 
	{
		if(pMusicWin!=NULL)
			pMusicWin->IncVelocEsec(MIDI_SPEED_VAR_STEP);
		return;
	}

    if ( pMusicWin!=NULL && pMusicWin->GetNumHit() >= maxNumHit ) return;
	else
    {
        if ( mmExecMode == MM_USER 	&& pAudioThread!=NULL && pAudioThread->IsAlive())
		{	 
            if ( pAudioThread->Faster() )  
			{	 
			    audioRate = audioRate - 1;
				if(pMusicWin!=NULL)
					pMusicWin->IncVelocEsec(VIDEO_STEP);//e' la percentuale di incremento sulla velocita'
			}
		}
	    else if(pMusicWin!=NULL) pMusicWin->IncVelocEsec(VIDEO_STEP);//MM_SYNC
		if(pMusicWin!=NULL) pMusicWin->SetNumHit( pMusicWin->GetNumHit() + 1 );
    }
}

EXEC_MODE Partitura::GetMmExecMode()
{
    return  mmExecMode;
}

void Partitura::SetMmExecMode(EXEC_MODE mode )
{
    mmExecMode = mode;
}


int Partitura::GetDeclickMode()
{
    return  declickMode;
}

void Partitura::SetDeclickMode( int mode )
{
    declickMode = mode;
	
#ifndef __WXGTK__
	if (pAudioThread != NULL)
	{
		if ( declickMode == DECLICK_ON )  pAudioThread->DeclickON();
		else pAudioThread->DeclickOFF();
	}	
#endif
	
}

int Partitura::GetAudioRate()
{
    return  audioRate;
}

/////////////////////////////////////////////////////////////////////////////////////////////////

void Partitura::prova()
{
	BeginPaint();
	pMusicWin->Clear();
	EndPaint();
}

int Partitura::GetETTLenght()
{
	return ETTLenght;
} 

Bool Partitura::IsLastPage()
{
	return listaSpa->IsLastPage();
}

void Partitura::SetFrom(Partitura* pPart)
{
	if(owner)
	{
		delete listaSpa;
		delete listaPar;
		delete listaPag;
	}
	listaSpa = pPart->listaSpa;
	listaPar = pPart->listaPar;
	listaPag = pPart->listaPag;
	selections = pPart->selections;
	owner=false;

	SetFileLoadName(pPart->GetFileLoadName());
	SetFileSaveName(pPart->GetFileSaveName());
	SetPathLoadName(pPart->GetPathLoadName());
	SetPathSaveName(pPart->GetPathSaveName());
}



void Partitura::CompViewParametersOp()
{
	
	extern wxMusicEditor *MusicEditor;
	int top, bottom, left, right, staffdist;
	float scale;
    
	top=GetComputerViewParams()->GetTopMargin();
	bottom=GetComputerViewParams()->GetBottomMargin();
	right=GetComputerViewParams()->GetRightMargin();
	left=GetComputerViewParams()->GetLeftMargin();
	staffdist=GetComputerViewParams()->GetStaffDistance();
	scale=GetComputerViewParams()->GetMagnification();
	// qui istanziare la dialog iobo
	
	if (ComputerViewParWindow(&top, &bottom, &left, &right, &staffdist, &scale)) 
	{
		//settaggio parametri della computer view
		SetComputerViewParams(top, bottom, left, right, staffdist, scale);
		//
		if ((tipoLeggio==LEGGIO_DLIOO) || (tipoLeggio==LEGGIO_PDLOO)) 
			((ParteL*)this)->SetNPentagrammi(5);
		Scr->SetMagnification(scale);
		pMusicWin->Resize(VUnit(Scr->GetMaxPixX()),VUnit(Scr->GetMaxPixY()));
		Redraw();
	}
}

void Partitura::PrintView()
{	
	// qui si dovrebbe impostare il canvas ed il PrintRect a seconda del valore 
	// di Magnification (impostato nella Print Params)
	
	// poi si passa a Redraw(true) e basta
	extern wxMusicEditor *MusicEditor;
	class Rectangle r=listaSpa->GetMusicRect();
	
	pMusicWin->Clear(); 
	
	float Magnif = GetPrintViewParams()->GetMagnification();
	int PENTNO = GetPrintViewParams()->GetNumStaff(GetCurrentPage());
	
	Scr->SetMagnification(Magnif);	
	
	
	int PR_width=0, PR_height=0;
	
	int tmpCurrentScore = GetCurrentScore();
	if (tmpCurrentScore>0) 
		tmpCurrentScore--;
	
	switch(GetPrintViewParams()->GetPageFormat()) 
	{
	case pfA4:
		PR_height = (int)(838/Magnif); 
		PR_width = (int)(593/Magnif);
		break;
	case pfA3:
		PR_height = (int)(1186/Magnif);
		PR_width = (int)(838/Magnif);
		break;
	case pfA4R:
		PR_height = (int)(593/Magnif); 
		PR_width = (int)(838/Magnif);
		break;
	case pfA3R:
		PR_height = (int)(838/Magnif);
		PR_width = (int)(1186/Magnif);
		break;
	}
	SetPrintRect(class Rectangle (Point (0,0), Point(PR_width,PR_height)));
	((Spartito*)listaSpa->AtPos(tmpCurrentScore))->SetNPentagrammi(PENTNO,class Rectangle (Point (0,0), Point(PR_width,PR_height)),true);
	
	/* ********************************
	I conti si fanno partendo dal PRN->GetMax ... e dividendo per MAGNIFICATION
	X e' 580 e Y e 780 che sono le dimensioni della pagina A4 ----------------
	
	  QUINDI ORA IL RETTANGOLO DI STAMPA E' UGUALE ALLA PAGINA A4
	************************************* */
	
	GoTop(); //Mi riporto nella prima pagina
	Redraw(true);  // is printing
	
}

void Partitura::SetupIntRefs()
{
	listaSpa->SetupIntRefs();
}


void Partitura::View_Link(bool view)
{
	Spartito *currSpa;
	currSpa=GetFirstSpa();
	while (currSpa != NULL)
	{
		currSpa->View_Link(view);
		currSpa=GetNextSpa(currSpa);
	}
	Draw();
}

void Partitura::HidePart(Spartito *currSpa)
{
	currSpa->SetVisible(false);
	Spartito *tmpSpa, *firstVis=NULL, *lastVis=NULL;
	tmpSpa=GetFirstSpa();
	while (tmpSpa != NULL)
	{
		if(tmpSpa->IsVisible())
			tmpSpa->SetTipo(SPART_GENERICO);
		tmpSpa=GetNextSpa(tmpSpa);
	}
	firstVis=listaSpa->GetFirstVisible();
	lastVis=listaSpa->GetLastVisible();
	if(firstVis && lastVis)
	{
		if(firstVis==lastVis)
			firstVis->SetTipo(SPART_UNICO);
		else
		{
			firstVis->SetTipo(SPART_SUPERIORE);
			lastVis->SetTipo(SPART_INFERIORE);
		}
	}
	
	MusicEditor->smrf->HidePart(listaSpa->GetPos(currSpa)+1);
}

void Partitura::ShowAllParts()
{
	Spartito *currSpa,*firstSpa=NULL, *lastSpa=NULL;
	currSpa=GetFirstSpa();
	while (currSpa != NULL)
	{
		currSpa->SetVisible(true);
		currSpa->SetTipo(SPART_GENERICO);
		MusicEditor->smrf->ShowPart(listaSpa->GetPos(currSpa)+1);
		currSpa=GetNextSpa(currSpa);
	}
	firstSpa=(Spartito*)listaSpa->GetFirst();
	lastSpa=(Spartito*)listaSpa->GetLast();
	if(firstSpa && lastSpa)
	{
		if(firstSpa==lastSpa)
			firstSpa->SetTipo(SPART_UNICO);
		else
		{
			firstSpa->SetTipo(SPART_SUPERIORE);
			lastSpa->SetTipo(SPART_INFERIORE);
		}
	}
	
	Redraw();
}

void Partitura::SplittingPart(Spartito *currSpa)
{
	
	Spartito *newSpa;
	wxString options[NUM_MAX_LAYER];
	int nlayers=4*currSpa->GetNumberOfStaff();
	
	for(int l=1; l<=nlayers; l++)
		options[l-1]=wxString::Format("Layer %d",l);
	
    if(currSpa->IsSplittable())
	{
		wxSingleChoiceDialog *dialog = new wxSingleChoiceDialog(NULL,"Select Layer", "Splitting Part", nlayers, options);
		if(dialog->ShowModal()==wxID_OK)
		{
			int layer=dialog->GetSelection();
			Scr->GSetMainDrawArea();
			AddNewSpartito((Spartito*)listaSpa->GetLast(),FALSE,currSpa->GetNumberOfStaff());
			GoTop();
			newSpa=(Spartito*)listaSpa->GetLast();
			newSpa->Transfer_Layer(currSpa,(int)layer);
		}
		delete dialog;
	}
	else
		wxMessageBox("Impossible to split\nonly one layer is present","Information");
	
}

void Partitura::DisjoiningPart(Spartito *currSpa)
{
	
	Spartito *newSpa;
	wxString options[NUM_MAX_LAYER];
	int nlayers=4*currSpa->GetNumberOfStaff();
	
	if(currSpa->GetNumberOfStaff()>1)
	{
		if(currSpa->IsDisjoinable())
		{	
			Scr->GSetMainDrawArea();
			
			for(int i=0; i<currSpa->GetNumberOfStaff(); i++)
			{
				AddNewSpartito((Spartito*)listaSpa->GetLast(),FALSE);
				newSpa=(Spartito*)listaSpa->GetLast();
				newSpa->Multi2Single_Transfer_Layer(currSpa,i);
				newSpa->SetNCorde(0,currSpa->GetNCorde(i));
			}
			
			listaPar->CancParentesiSpa(currSpa);
			listaSpa->Del(currSpa);
			GoTop();
		}
		else
			wxMessageBox("Impossible to disjoin\nlayers through staves","Information");
	}
	else
		wxMessageBox("Impossible to disjoin\nonly one staff is present","Information");
}

bool Partitura::JoiningParts(int part1, int part2, int part3)
{
	
	int numofstaff=0;
	Spartito *spa[3], *newSpa;
	if(part1!=-1)
	{
		spa[0]=GetSpartNum(part1);
		numofstaff++;
	}
	if(part2!=-1)
	{
		spa[1]=GetSpartNum(part2);
		numofstaff++;
	}
	if(part3!=-1)
	{
		spa[2]=GetSpartNum(part3);
		numofstaff++;
	}
	if(numofstaff==0)
		return false;
	
	AddNewSpartito((Spartito*)listaSpa->GetLast(),FALSE,numofstaff);
	newSpa=(Spartito*)listaSpa->GetLast();
	for(int i=0; i<numofstaff; i++)
	{
		newSpa->Single2Multi_Transfer_Layer(spa[i],i);
		newSpa->SetNCorde(i,spa[i]->GetNCorde());
		listaPar->CancParentesiSpa(spa[i]);
		listaSpa->Del(spa[i]);
	}
	
	GoTop();
	
	return true;
}

bool Partitura::MergingParts(int part1, int part2)
{
	
	int numofstaff=0;
	Spartito *spa[2];
	if(part1==-1 || part2==-1)
		return false;
	
	spa[0]=GetSpartNum(part1);
	spa[1]=GetSpartNum(part2);
	
	
	if(!AreCompatible4MergeParts(spa[0],spa[1]))
		return false;
	spa[0]->CopyToClipboard();
	PasteFromClipboard((Spartito*)listaSpa->GetLast(),FALSE);
	spa[0]=(Spartito*)listaSpa->GetLast();
	spa[1]->CopyToClipboard();
	PasteFromClipboard((Spartito*)listaSpa->GetLast(),FALSE);
	spa[1]=(Spartito*)listaSpa->GetLast();
	spa[1]->SetVisible(false);
	spa[0]->Transfer_Layer_MergeParts(spa[1]);
	listaSpa->Del(spa[1]);
	GoTop();
	
	return true;
}

bool Partitura::AreCompatible4MergeParts(Spartito *spa1,Spartito *spa2)
{
	if(spa1->GetNumberOfStaff()!=1 || spa2->GetNumberOfStaff()!=1)
		return false;
	
	Battuta *currBatSpa1=NULL, *currBatSpa2=NULL;
	
	currBatSpa1=spa1->GetBattNum(0);
	currBatSpa2=spa2->GetBattNum(0);
	while ((currBatSpa1 != NULL)&&(currBatSpa2!=NULL))
	{
		if(currBatSpa1->GetNUsedLayers()>2 || currBatSpa2->GetNUsedLayers()>2)
			return false;
		if(!currBatSpa1->CompareIntest(currBatSpa2))
			return false;
		currBatSpa1=spa1->GetNextBat(currBatSpa1);
		currBatSpa2=spa2->GetNextBat(currBatSpa2);
	}
	if (currBatSpa1!=NULL || currBatSpa2!=NULL)
		return false;
	
	return true;
}

void Partitura::ViewLayerInPart(Spartito *currSpa)
{
    
	ViewLayerDial(currSpa);
}

void Partitura::BeamingInPart()
{
    bool forceUnbeaming;
	int start, end, type, part;
	Battuta *tmp;
	Spartito *currSpa=NULL;
	
	if(BeamingDlg(this, &start, &end, &type, &part, &forceUnbeaming))
	{
		if(part!=0)
		{
			currSpa=GetSpartNum(part-1);
			if(type==0)
			{
				
				wxProgressDialog beaming("Beaming...","Measure 0",end-start+1);
				
				tmp=currSpa->GetBattNum(start-1);
				for(int i=start; i<=end; i++)
				{
					if(forceUnbeaming)
						tmp->EliminaGruppi();
					else
						tmp->BeamingAuto();
					tmp=currSpa->GetNextBat(tmp);
					if (i%10==0)
						beaming.Update(i-start+1,wxString::Format("Measure %d",i));
				}
			}
			else
			{
				if(forceUnbeaming)
					(currSpa->GetBattNum(start-1))->EliminaGruppi();
				else
					(currSpa->GetBattNum(start-1))->BeamingAuto();
			}
		}
		else
		{
			wxProgressDialog beaming("Beaming...","Part 0",GetNumSpa());
			currSpa=GetFirstSpa();
			int px=0;
			while (currSpa!=NULL)
			{
				if(type==0)
				{
					tmp=currSpa->GetBattNum(start-1);
					for(int i=start; i<=end; i++)
					{
						if(forceUnbeaming)
							tmp->EliminaGruppi();
						else
							tmp->BeamingAuto();
						tmp=currSpa->GetNextBat(tmp);
					}
				}
				else
				{
					if(forceUnbeaming)
						(currSpa->GetBattNum(start-1))->EliminaGruppi();
					else
						(currSpa->GetBattNum(start-1))->BeamingAuto();
				}
				currSpa=GetNextSpa(currSpa);
				beaming.Update(px,wxString::Format("Part %d",px++));
			}
		}
		
	}
}

void Partitura::UpDownInPart()
{
    int start, end, type, part;
	Battuta *tmp;
	Spartito *currSpa=NULL;
	
	if(UpDownDlg(this, &start, &end, &type, &part))
	{
		
		if(part!=0)
		{
			currSpa=GetSpartNum(part-1);
			if(type==0)
			{
				wxProgressDialog updating("Deciding Up/Down...","Measure 0",end-start+1);
				tmp=currSpa->GetBattNum(start-1);
				for(int i=start; i<=end; i++)
				{
					tmp->UpDownAuto();
					tmp=currSpa->GetNextBat(tmp);
					if (i%10==0)
						updating.Update(i-start+1,wxString::Format("Measure %d",i));
				}
			}
			else
				(currSpa->GetBattNum(start-1))->UpDownAuto();
		}
		else
		{
			wxProgressDialog updating("Deciding Up/Down...","Part 0",GetNumSpa());
			currSpa=GetFirstSpa();
			int px=0;
			while (currSpa!=NULL)
			{
				if(type==0)
				{
					tmp=currSpa->GetBattNum(start-1);
					for(int i=start; i<=end; i++)
					{
						tmp->UpDownAuto();
						tmp=currSpa->GetNextBat(tmp);
					}
				}
				else
					(currSpa->GetBattNum(start-1))->UpDownAuto();
				currSpa=GetNextSpa(currSpa);
				updating.Update(px,wxString::Format("Part %d",px++));
			}
		}
	}
}

void Partitura::SetAudioFile(const char *fname)
{
	strcpy(audioFile,fname);
}

const char* Partitura::GetAudioFile()
{
	return audioFile;
}

void Partitura::AddCasella(Casella *casella, int Npage)
{
	if(GetCurrentScore()==0)
	{	
		Pagina *pag=listaPag->GetPage(Npage,true);
		pag->InsertEnd(casella);
	}
	else
		GetSpartNum(GetCurrentScore()-1)->AddCasella(casella,Npage);	
	
}

PrintParams *Partitura::GetPrintViewParams()
{
	if(GetCurrentScore()==0)
		return listaSpa->GetPrintViewParams();
	else
	{
		Spartito *pSpa=GetSpartNum(GetCurrentScore()-1);
		if(pSpa!=NULL)
			return pSpa->GetPrintViewParams();
		return NULL;
	}
}

PrintParams *Partitura::GetComputerViewParams()
{
	if(GetCurrentScore()==0)
		return listaSpa->GetComputerViewParams(); 
	else
	{
		Spartito *pSpa=GetSpartNum(GetCurrentScore()-1);
		if(pSpa!=NULL)
			return pSpa->GetComputerViewParams();
		return NULL;
	}
}

void Partitura::SetComputerViewParams(int tm, int bm, int lm, int rm, int sd, float magnification)
{
	if(GetCurrentScore()==0)
		listaSpa->SetComputerViewParams(tm,bm,lm,rm,sd, magnification); 
	else
		GetSpartNum(GetCurrentScore()-1)->SetComputerViewParams(tm,bm,lm,rm,sd,magnification);
}

void Partitura::SetTmpPrintViewParams(PrintParams *ppars)
{
	if(GetCurrentScore()==0)
		listaSpa->SetTmpPrintViewParams(ppars); 
	else
		GetSpartNum(GetCurrentScore()-1)->SetTmpPrintViewParams(ppars); 
};

int Partitura::GetCurrentPage()
{
	if(GetCurrentScore()==0)
		return listaSpa->GetCurrentPage(); 
	else
		return GetSpartNum(GetCurrentScore()-1)->GetCurrentPage(); 
};


ListaSpartiti *Partitura::GetListaSpa()
{
	return listaSpa;
}


bool Partitura::CopyToClipboard(SymCmd SCmd)
{	
	DrawObject *pobj;
	
	SymbolicHit(SCmd.Path1,SCmd.SelObjType,pobj);
	
	if (pobj!=NULL)
	{
		((Spartito *)pobj)->CopyToClipboard();
		return true;
	}
	return false;
	
}

void Partitura::AddSpartitoAt(Spartito *pSp,Spartito *refpSp, bool posUp)
{	
	if (posUp == TRUE)
	{
		if (refpSp == GetFirstSpa())
			listaSpa->AddAfter(pSp,NULL);
		else
		{
			refpSp = GetPrevSpa(refpSp);
			listaSpa->AddAfter(pSp,refpSp);
		}
	}
	else
		listaSpa->AddAfter(pSp,refpSp);
};


void Partitura::GetPageMeasures(int& top, int& bottom)
{
	if(GetCurrentScore()==0)
		listaSpa->GetPageMeasures(top,bottom); 
	else
		GetSpartNum(GetCurrentScore()-1)->GetPageMeasures(top,bottom);
};

bool Partitura::CutPageBox(DrawObject *obj)
{
	if(FindPageBox(obj,true))
	{
		pageBox=obj;
		return TRUE;
	}
	else
		return FALSE;
}

bool Partitura::CopyPageBox(DrawObject *obj)
{
	if(FindPageBox(obj,false))
	{
		if(pageBox!=NULL)
			delete pageBox;
		pageBox=((Casella*)obj)->Clone();
		return TRUE;
	}
	else
		return FALSE;	
}

bool Partitura::PastePageBox(const Point& p)
{
	float magn = Scr->GetMagnification();
	float h=0;
	if(pageBox!=NULL)
	{
		Casella *cas = ((Casella*)pageBox)->Clone();
		if((cas->GetHeight()!=10)&&(cas->GetWidth()!=10))
			h = cas->GetHeigh();
		cas->SetX(p.x*magn);
		cas->SetY((p.y-h)*magn);
		Pagina *pag=listaPag->GetPage(GetCurrentPage(),false);
		if(pag!=NULL)
			pag->InsertEnd(cas);
		else
		{
			pag=listaPag->GetPage(GetCurrentPage(),true);
			pag->InsertEnd(cas);
		}
		return TRUE;
	}
	else 
		return FALSE;
}

bool Partitura::FindPageBox(DrawObject *obj, bool del)
{
	int i;
	for(i=1;i<MAX_VIEW+1;i++)
	{
		Pagina *pag=listaPag->GetPage(i,false);
		if(pag!=NULL)
		{
			if(pag->FindPtr(obj))
			{
				if(del)
					pag->DelPtr(obj);
				return TRUE;
			}
		}
	}
	return FALSE;
}

void Partitura::CopyMeasure(int nProgressive)
{
	MeasureNumCode_CutPaste=0;
	ScoreNumCode_CutPaste=0;
	Spartito *pSp = GetFirstSpa();
	if(pSp!=NULL)
	{
		Battuta *pBattuta=pSp->GetBattNum(nProgressive-1);
		if(pBattuta!=NULL)
		{
			MeasureNumCode_CutPaste = pBattuta->GetNumericCode();
			ScoreNumCode_CutPaste=0;
			
			pSp = GetFirstSpa();
			while (pSp!=NULL) 
			{
				pBattuta=pSp->GetBattNum(nProgressive-1);
				if (pBattuta!=NULL && pBattuta->GetNumberOfStaff()!=1)
				{
					Message("Sorry, I cannot copy multistaff measures");
					MeasureNumCode_CutPaste=0;
					ScoreNumCode_CutPaste=0;
					return;
				}
				pSp=GetNextSpa(pSp);	
			}
		}
	}
}

Bool Partitura::PasteMeasureAfter(int nProgressive)
{
	if(MeasureNumCode_CutPaste<=0 || ScoreNumCode_CutPaste!=0) // non ha fatto copia o ha fatto copy single measure
	{
		Message("Warning! Use Copy Measure Column first");
		return FALSE;
	}
	
	Spartito *psp = GetFirstSpa();
	Battuta *PBattuta = psp->GetBatt(MeasureNumCode_CutPaste);
	int np = PBattuta->GetNumProgress()-1;
	int np_new = nProgressive;
	Array[0]='\0';
	
	psp = GetFirstSpa();
	while (psp!=NULL) 
	{
		PBattuta=psp->GetBattNum(np);
		if (PBattuta!=NULL)
		{
			int ncc = PBattuta->GetNumericCode();
			int npp = PBattuta->GetNumProgress();
			
			PBattuta->SetNumericCode(psp->GetCodeCounterBat());
			psp->SetCodeCounterBat(psp->GetNumericCode()+1);
			
			PBattuta->SetNumProgress(np_new+1);
			   			   
			PBattuta->SetNumericCode(ncc);
			PBattuta->SetNumProgress(npp);
			
			PBattuta=psp->GetBattNum(np_new);
			while (PBattuta!=NULL)
			{
				PBattuta->SetNumProgress(PBattuta->GetNumProgress()+1);
				PBattuta = psp->GetNextBat(PBattuta);
			}
		}
		else
		{
        	   EM.Error(MUSICAL_ERR_AREA,"EC_BAT_NOTFOUND","Partitura","ImmediateSel()");
			   return FALSE;
		}
		psp=GetNextSpa(psp);	
	}
	
	
	LoadS(0);
	Array[0] = '\0';
	return TRUE;
}

void Partitura::CopySingleMeasure(NumCode scoreID, NumCode measureID)
{
	ScoreNumCode_CutPaste=scoreID;
	MeasureNumCode_CutPaste = measureID;
	Spartito *pSpSrc = GetSpart(ScoreNumCode_CutPaste);
	if(pSpSrc!=NULL)
	{
		Battuta *pBatSrc = pSpSrc->GetBatt(MeasureNumCode_CutPaste);
		if(pBatSrc!=NULL)
			if(pBatSrc->GetNumberOfStaff()==1)
				return;
			else
				Message("Sorry, I cannot copy a multistaff measure");
	}
	ScoreNumCode_CutPaste = 0;
	MeasureNumCode_CutPaste = 0;
}

Bool Partitura::PasteSingleMeasureIn(NumCode scoreID, NumCode measureID)
{
	if(MeasureNumCode_CutPaste<=0 || ScoreNumCode_CutPaste==0) // non ha fatto copia single measure
	{
		Message("Warning! Use Copy Single Measure first");
		return FALSE;
	}
	
	Spartito *pSpSrc = GetSpart(ScoreNumCode_CutPaste);
	Battuta *pBatSrc = pSpSrc->GetBatt(MeasureNumCode_CutPaste);
	Spartito *pSpTrg = GetSpart(scoreID);
	Battuta *pBatTrg = pSpTrg->GetBatt(measureID);
	
	Array[0]='\0';
	
	int ncc = pBatSrc->GetNumericCode();
	int npp = pBatSrc->GetNumProgress();
	
	pBatSrc->SetNumericCode(pBatTrg->GetNumericCode());
	pBatSrc->SetNumProgress(pBatTrg->GetNumProgress());
	
	
	pBatSrc->SetNumericCode(ncc);
	pBatSrc->SetNumProgress(npp);
	
	pSpTrg->DeleteBat(pBatTrg);
	
	LoadS(0);
	Array[0] = '\0';
	return TRUE;
}

void Partitura::RemovedLyr(SMRLyricsItem* lyr)
{
	Spartito* psp = GetFirstSpa();
	while (psp!=NULL) 
	{
		if(psp->GetCurrentLyrWF()==lyr)
			psp->SetCurrentLyrWF(NULL);
		psp=GetNextSpa(psp);	
	}
}

bool Partitura::AlignScores(int maxMeasures)
{
	listaSpa->BalanceSpartiti();
	return true;
}

int Partitura::GetNStaff()
{
	if(GetCurrentScore()==0)
	{
		int space=0;
		Spartito* psp = GetFirstSpa();
		while (psp!=NULL) 
		{
			space+=psp->GetNumberOfStaff();
			psp=GetNextSpa(psp);	
		}
		return space;
	}
	else
		return GetSpartNum(GetCurrentScore()-1)->GetNumberOfStaff();
}


//** FFF
const int Partitura::CMDtoCL(const SymCmd& com)
{
	switch(com.CommandId)
	{
		case CMD_BATTUTA_SEGNO:			return CL_SEGNO;
		case CMD_BATTUTA_CODA:			return CL_CODA;
		case CMD_BATTUTA_TSALTO:		return CL_TSALTO;

		case CMD_BATTUTA_ARMCHV_0D:		return DO_maggiore;
		case CMD_BATTUTA_ARMCHV_1D:		return SOL_maggiore;
		case CMD_BATTUTA_ARMCHV_2D:		return RE_maggiore;
		case CMD_BATTUTA_ARMCHV_3D:		return LA_maggiore;
		case CMD_BATTUTA_ARMCHV_4D:		return MI_maggiore;
		case CMD_BATTUTA_ARMCHV_5D:		return SI_maggiore;
		case CMD_BATTUTA_ARMCHV_6D:		return FAd_maggiore;
		case CMD_BATTUTA_ARMCHV_7D:		return DOd_maggiore;
		case CMD_BATTUTA_ARMCHV_1B:		return FA_maggiore;
		case CMD_BATTUTA_ARMCHV_2B:		return SIb_maggiore;
		case CMD_BATTUTA_ARMCHV_3B:		return MIb_maggiore;
		case CMD_BATTUTA_ARMCHV_4B:		return LAb_maggiore;
		case CMD_BATTUTA_ARMCHV_5B:		return REb_maggiore;
		case CMD_BATTUTA_ARMCHV_6B:		return SOLb_maggiore;
		case CMD_BATTUTA_ARMCHV_7B:		return DOb_maggiore;
		case CMD_BATTUTA_ARMCHV_0Dm:	return LA_minore;
		case CMD_BATTUTA_ARMCHV_1Dm:	return MI_minore;
		case CMD_BATTUTA_ARMCHV_2Dm:	return SI_minore;
		case CMD_BATTUTA_ARMCHV_3Dm:	return FAd_minore;		
		case CMD_BATTUTA_ARMCHV_4Dm:	return DOd_minore;
		case CMD_BATTUTA_ARMCHV_5Dm:	return SOLd_minore;
		case CMD_BATTUTA_ARMCHV_6Dm:	return REd_minore;
		case CMD_BATTUTA_ARMCHV_7Dm:	return LAd_minore;
		case CMD_BATTUTA_ARMCHV_1Bm:	return RE_minore;
		case CMD_BATTUTA_ARMCHV_2Bm:	return SOL_minore;
		case CMD_BATTUTA_ARMCHV_3Bm:	return DO_minore;
		case CMD_BATTUTA_ARMCHV_4Bm:	return FA_minore;
		case CMD_BATTUTA_ARMCHV_5Bm:	return SIb_minore;
		case CMD_BATTUTA_ARMCHV_6Bm:	return MIb_minore;
		case CMD_BATTUTA_ARMCHV_7Bm:	return LAb_minore;

		case CMD_BATTUTA_CVIOLINO:		return CL_CVIOLINO;
		case CMD_BATTUTA_CBASSO:		return CL_CBASSO;
		case CMD_BATTUTA_CBARITONO:		return CL_CBARITONO;
		case CMD_BATTUTA_CCONTRALTO:	return CL_CCONTRALTO;
		case CMD_BATTUTA_CSOPRANO:		return CL_CSOPRANO;
		case CMD_BATTUTA_CPERCUSBOX:	return CL_CPERCUSBOX;
		case CMD_BATTUTA_CMEZZOSOPRANO:	return CL_CMEZZOSOPRANO;
		case CMD_BATTUTA_CTENORE:		return CL_CTENORE;
		case CMD_BATTUTA_CVUOTA:		return CL_CVUOTA;

		case CMD_NOTA_DIESIS:			return CL_DIESIS;
		case CMD_NOTA_DIESIS1Q:			return CL_DIESIS1Q;
		case CMD_NOTA_DIESIS3Q:			return CL_DIESIS3Q;
		case CMD_NOTA_BEMOLLE:			return CL_BEMOLLE;
		case CMD_NOTA_BEMOLLE1Q:		return CL_BEMOLLE1Q;
		case CMD_NOTA_BEMOLLE3Q:		return CL_BEMOLLE3Q;
		case CMD_NOTA_BEQUADRO:			return CL_BEQUADRO;
		case CMD_NOTA_BEQUADRO1Q:		return CL_BEQUADRO1Q;	
		case CMD_NOTA_BEQUADRO3Q:		return CL_BEQUADRO3Q;
		case CMD_NOTA_2DIESIS:			return CL_DOPPIODIESIS;
		case CMD_NOTA_2BEMOLLE:			return CL_DOPPIOBEMOLLE;

		case CMD_NOTA_TENUTO:			return CL_TENUTO;
		case CMD_NOTA_SFORZATO:			return CL_SFORZATO;
		case CMD_NOTA_ACCENTO:			return CL_ACCENTO;
		case CMD_NOTA_STACCATO:			return CL_STACCATO;
		case CMD_NOTA_MARTDOLCE:		return CL_MARTDOLCE;
		case CMD_NOTA_PUNTOALL:			return CL_PUNTOALLUNGATO;
		case CMD_NOTA_MARTELLATO:		return CL_MARTELLATO;

		case CMD_NOTA_ARCO:				return CL_ARCO;
		case CMD_ARCO_PONTICELLO:		return CL_PONTICELLO;
		case CMD_ARCO_TASTIERA:			return CL_TASTIERA;
		case CMD_ARCO_PUNTA:			return CL_PUNTA;
		case CMD_ARCO_TALLONE:			return CL_TALLONE;
		case CMD_ARCO_ARCOSU:			return CL_ARCOSU;
		case CMD_ARCO_ARCOGIU:			return CL_ARCOGIU;
		case CMD_ARCO_CORDA:			return CL_PIZZICATO;

	}
	return -1;
}

const int Partitura::CLtoCMD(const ClassID& id)
{
	switch(id)
	{
		case CL_SEGNO:			return CMD_BATTUTA_SEGNO;
		case CL_CODA:			return CMD_BATTUTA_CODA;
		case CL_TSALTO:			return CMD_BATTUTA_TSALTO;
		
		case CL_CVIOLINO:		return CMD_BATTUTA_CVIOLINO;
		case CL_CBASSO:			return CMD_BATTUTA_CBASSO;
		case CL_CBARITONO:		return CMD_BATTUTA_CBARITONO;
		case CL_CCONTRALTO:		return CMD_BATTUTA_CCONTRALTO;
		case CL_CSOPRANO:		return CMD_BATTUTA_CSOPRANO;
		case CL_CPERCUSBOX:		return CMD_BATTUTA_CPERCUSBOX;
		case CL_CMEZZOSOPRANO:	return CMD_BATTUTA_CMEZZOSOPRANO;
		case CL_CTENORE:		return CMD_BATTUTA_CTENORE;
		case CL_CVUOTA:			return CMD_BATTUTA_CVUOTA;

		case CL_NBREVE:			return CMD_BATTUTA_INS_NBREVE;
		case CL_NSEMIBREVE:		return CMD_BATTUTA_INS_NSEMIBREVE;
		case CL_NMINIMA:		return CMD_BATTUTA_INS_NMINIMA;
		case CL_NSEMIMINIMA:	return CMD_BATTUTA_INS_NSEMIMINIMA;
		case CL_NCROMA:			return CMD_BATTUTA_INS_NCROMA;	
		case CL_NSEMICROMA:		return CMD_BATTUTA_INS_NSEMICROMA;	
		case CL_NBISCROMA:		return CMD_BATTUTA_INS_NBISCROMA;		
		case CL_NSEMIBISCROMA:	return CMD_BATTUTA_INS_NSEMIBISCROMA; 
		case CL_NFUSA:			return CMD_BATTUTA_INS_NFUSA;			

		case CL_PQUATTROBATTUTE:return CMD_BATTUTA_INS_PQUABATT;		
		case CL_PDUEBATTUTE:	return CMD_BATTUTA_INS_PDUEBATT;		
		case CL_PSEMIBREVE:		return CMD_BATTUTA_INS_PSEMIBREVE;	
		case CL_PMINIMA:		return CMD_BATTUTA_INS_PMINIMA;		
		case CL_PSEMIMINIMA:	return CMD_BATTUTA_INS_PSEMIMINIMA;	
		case CL_PCROMA:			return CMD_BATTUTA_INS_PCROMA;		
		case CL_PSEMICROMA:		return CMD_BATTUTA_INS_PSEMICROMA;	
		case CL_PBISCROMA:		return CMD_BATTUTA_INS_PBISCROMA;		
		case CL_PSEMIBISCROMA:	return CMD_BATTUTA_INS_PSEMIBISCROMA; 
		case CL_PFUSA:			return CMD_BATTUTA_INS_PFUSA;			

		case CL_DOPPIODIESIS:	return CMD_NOTA_2DIESIS;				
		case CL_DOPPIOBEMOLLE:	return CMD_NOTA_2BEMOLLE;				
		case CL_DIESIS:			return CMD_NOTA_DIESIS;			
		case CL_DIESIS1Q:		return CMD_NOTA_DIESIS1Q;
		case CL_DIESIS3Q:		return CMD_NOTA_DIESIS3Q;	
		case CL_BEMOLLE:		return CMD_NOTA_BEMOLLE;	
		case CL_BEMOLLE1Q:		return CMD_NOTA_BEMOLLE1Q;
		case CL_BEMOLLE3Q:		return CMD_NOTA_BEMOLLE3Q;
		case CL_BEQUADRO:		return CMD_NOTA_BEQUADRO;			
		case CL_BEQUADRO1Q:		return CMD_NOTA_BEQUADRO1Q;	
		case CL_BEQUADRO3Q:		return CMD_NOTA_BEQUADRO3Q;

		case CL_TENUTO:			return CMD_NOTA_TENUTO;
		case CL_SFORZATO:		return CMD_NOTA_SFORZATO;
		case CL_ACCENTO:		return CMD_NOTA_ACCENTO;
		case CL_STACCATO:		return CMD_NOTA_STACCATO;
		case CL_MARTDOLCE:		return CMD_NOTA_MARTDOLCE;
		case CL_PUNTOALLUNGATO:	return CMD_NOTA_PUNTOALL;
		case CL_MARTELLATO:		return CMD_NOTA_MARTELLATO;
	}
	return -1;
}

const int Partitura::CLtoCMD(const tipoArmaturaChiave& id)
{
	switch(id)
	{
		case DO_maggiore:		return CMD_BATTUTA_ARMCHV_0D;
		case SOL_maggiore:		return CMD_BATTUTA_ARMCHV_1D;
		case RE_maggiore:		return CMD_BATTUTA_ARMCHV_2D;
		case LA_maggiore:		return CMD_BATTUTA_ARMCHV_3D;
		case MI_maggiore:		return CMD_BATTUTA_ARMCHV_4D;
		case SI_maggiore:		return CMD_BATTUTA_ARMCHV_5D;
		case FAd_maggiore:		return CMD_BATTUTA_ARMCHV_6D;
		case DOd_maggiore:		return CMD_BATTUTA_ARMCHV_7D;
		case FA_maggiore:		return CMD_BATTUTA_ARMCHV_1B;
		case SIb_maggiore:		return CMD_BATTUTA_ARMCHV_2B;
		case MIb_maggiore:		return CMD_BATTUTA_ARMCHV_3B;
		case LAb_maggiore:		return CMD_BATTUTA_ARMCHV_4B;
		case REb_maggiore:		return CMD_BATTUTA_ARMCHV_5B;
		case SOLb_maggiore:		return CMD_BATTUTA_ARMCHV_6B;
		case DOb_maggiore:		return CMD_BATTUTA_ARMCHV_7B;

		case LA_minore:			return CMD_BATTUTA_ARMCHV_0Dm;
		case MI_minore:			return CMD_BATTUTA_ARMCHV_1Dm;
		case SI_minore:			return CMD_BATTUTA_ARMCHV_2Dm;
		case FAd_minore:		return CMD_BATTUTA_ARMCHV_3Dm;
		case DOd_minore:		return CMD_BATTUTA_ARMCHV_4Dm;
		case SOLd_minore:		return CMD_BATTUTA_ARMCHV_5Dm;
		case REd_minore:		return CMD_BATTUTA_ARMCHV_6Dm;
		case LAd_minore:		return CMD_BATTUTA_ARMCHV_7Dm;
		case RE_minore:			return CMD_BATTUTA_ARMCHV_1Bm;
		case SOL_minore:		return CMD_BATTUTA_ARMCHV_2Bm;
		case DO_minore:			return CMD_BATTUTA_ARMCHV_3Bm;
		case FA_minore:			return CMD_BATTUTA_ARMCHV_4Bm;
		case SIb_minore:		return CMD_BATTUTA_ARMCHV_5Bm;
		case MIb_minore:		return CMD_BATTUTA_ARMCHV_6Bm;
		case LAb_minore:		return CMD_BATTUTA_ARMCHV_7Bm;
	}
	return -1;
}

const int Partitura::CLtoGROUP(const ClassID& id)
{
/*	NOTE_GROUP,
	REST_GROUP,
	EXPRESSION_GROUP,
	ACCIDENTAL_GROUP,
	LABEL_GROUP,
	CLEF_GROUP,
	TIMESIGN_GROUP,
	KEYSIGN_GROUP,	
*/	
	switch(id)
	{
		case CL_SEGNO:
		case CL_CODA:
		case CL_TSALTO:
			return LABEL_GROUP;

		case CL_ARMATURACHIAVE:
			return KEYSIGN_GROUP;

		case CL_CVIOLINO:
		case CL_CBASSO:	
		case CL_CBARITONO:
		case CL_CCONTRALTO:
		case CL_CSOPRANO:
		case CL_CPERCUSBOX:
		case CL_CMEZZOSOPRANO:
		case CL_CTENORE:	
		case CL_CVUOTA:		
			return CLEF_GROUP;

		case CL_NBREVE:			
		case CL_NSEMIBREVE:		
		case CL_NMINIMA:		
		case CL_NSEMIMINIMA:	
		case CL_NCROMA:			
		case CL_NSEMICROMA:			
		case CL_NBISCROMA:				
		case CL_NSEMIBISCROMA:	 
		case CL_NFUSA:			
			return NOTE_GROUP;

		case CL_PQUATTROBATTUTE:		
		case CL_PDUEBATTUTE:			
		case CL_PSEMIBREVE:		 	
		case CL_PMINIMA:			
		case CL_PSEMIMINIMA:		
		case CL_PCROMA:					
		case CL_PSEMICROMA:			
		case CL_PBISCROMA:				
		case CL_PSEMIBISCROMA:	 
		case CL_PFUSA:						
			return REST_GROUP;

		case CL_DOPPIODIESIS:				
		case CL_DOPPIOBEMOLLE:					
		case CL_DIESIS:						
		case CL_DIESIS1Q:		
		case CL_DIESIS3Q:			
		case CL_BEMOLLE:			
		case CL_BEMOLLE1Q:		
		case CL_BEMOLLE3Q:		
		case CL_BEQUADRO:					
		case CL_BEQUADRO1Q:			
		case CL_BEQUADRO3Q:		
			return ACCIDENTAL_GROUP;

		case CL_TENUTO:
		case CL_SFORZATO:
		case CL_ACCENTO:
		case CL_STACCATO:
		case CL_MARTDOLCE:
		case CL_PUNTOALLUNGATO:
		case CL_MARTELLATO:		
			return EXPRESSION_GROUP;

	}
	return -1;
}
//** FFF
